package airports

// Airport contains information about an airport
type Airport struct {
	ICAO      string  `json:"icao_code,omitempty"`
	IATA      string  `json:"iata_code,omitempty"`
	Name      string  `json:"name,omitempty"`
	City      string  `json:"city,omitempty"`
	State     string  `json:"state,omitempty"`
	Country   string  `json:"country,omitempty"`
	Elevation int     `json:"elevation,omitempty"`
	Latitude  float64 `json:"latitude,omitempty"`
	Longitude float64 `json:"longitude,omitempty"`
	Timezone  string  `json:"timezone,omitempty"`
}

// Airports contains ICAO airport codes mapped to information about them.
// Source: https://github.com/mwgg/Airports.
// Automatically converted to Go using github.com/alecthomas/repr.
var Airports = map[string]Airport{
	"00AK": {
		ICAO:      "00AK",
		Name:      "Lowell Field",
		City:      "Anchor Point",
		State:     "Alaska",
		Country:   "US",
		Elevation: 450,
		Latitude:  59.94919968,
		Longitude: -151.695999146,
		Timezone:  "America/Anchorage",
	},
	"00AL": {
		ICAO:      "00AL",
		Name:      "Epps Airpark",
		City:      "Harvest",
		State:     "Alabama",
		Country:   "US",
		Elevation: 820,
		Latitude:  34.8647994995,
		Longitude: -86.7703018188,
		Timezone:  "America/Chicago",
	},
	"00AZ": {
		ICAO:      "00AZ",
		Name:      "Cordes Airport",
		City:      "Cordes",
		State:     "Arizona",
		Country:   "US",
		Elevation: 3810,
		Latitude:  34.3055992126,
		Longitude: -112.1650009155,
		Timezone:  "America/Phoenix",
	},
	"00CA": {
		ICAO:      "00CA",
		Name:      "Goldstone /Gts/ Airport",
		City:      "Barstow",
		State:     "California",
		Country:   "US",
		Elevation: 3038,
		Latitude:  35.3504981995,
		Longitude: -116.888000488,
		Timezone:  "America/Los_Angeles",
	},
	"00CO": {
		ICAO:      "00CO",
		Name:      "Cass Field",
		City:      "Briggsdale",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4830,
		Latitude:  40.6222000122,
		Longitude: -104.34400177,
		Timezone:  "America/Denver",
	},
	"00FA": {
		ICAO:      "00FA",
		Name:      "Grass Patch Airport",
		City:      "Bushnell",
		State:     "Florida",
		Country:   "US",
		Elevation: 53,
		Latitude:  28.6455001831,
		Longitude: -82.21900177,
		Timezone:  "America/New_York",
	},
	"00FL": {
		ICAO:      "00FL",
		Name:      "River Oak Airport",
		City:      "Okeechobee",
		State:     "Florida",
		Country:   "US",
		Elevation: 35,
		Latitude:  27.2308998108,
		Longitude: -80.9692001343,
		Timezone:  "America/New_York",
	},
	"00GA": {
		ICAO:      "00GA",
		Name:      "Lt World Airport",
		City:      "Lithonia",
		State:     "Georgia",
		Country:   "US",
		Elevation: 700,
		Latitude:  33.7675018311,
		Longitude: -84.0682983398,
		Timezone:  "America/New_York",
	},
	"00ID": {
		ICAO:      "00ID",
		Name:      "Delta Shores Airport",
		City:      "Clark Fork",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2064,
		Latitude:  48.1453018188,
		Longitude: -116.2139968872,
		Timezone:  "America/Los_Angeles",
	},
	"00IL": {
		ICAO:      "00IL",
		Name:      "Hammer Airport",
		City:      "Polo",
		State:     "Illinois",
		Country:   "US",
		Elevation: 840,
		Latitude:  41.9784011841,
		Longitude: -89.5604019165,
		Timezone:  "America/Chicago",
	},
	"00IS": {
		ICAO:      "00IS",
		Name:      "Hayenga's Cant Find Farms Airport",
		City:      "Kings",
		State:     "Illinois",
		Country:   "US",
		Elevation: 820,
		Latitude:  40.0256004333,
		Longitude: -89.1229019165,
		Timezone:  "America/Chicago",
	},
	"00KS": {
		ICAO:      "00KS",
		Name:      "Hayden Farm Airport",
		City:      "Gardner",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1100,
		Latitude:  38.7277984619,
		Longitude: -94.9304962158,
		Timezone:  "America/Chicago",
	},
	"00KY": {
		ICAO:      "00KY",
		Name:      "Robbins Roost Airport",
		City:      "Stanford",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 1265,
		Latitude:  37.4094009399,
		Longitude: -84.6196975708,
		Timezone:  "America/New_York",
	},
	"00LS": {
		ICAO:      "00LS",
		Name:      "Lejeune Airport",
		City:      "Esterwood",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 12,
		Latitude:  30.1362991333,
		Longitude: -92.429397583,
		Timezone:  "America/Chicago",
	},
	"00MD": {
		ICAO:      "00MD",
		Name:      "Slater Field",
		City:      "Federalsburg",
		State:     "Maryland",
		Country:   "US",
		Elevation: 45,
		Latitude:  38.7570991516,
		Longitude: -75.7537994385,
		Timezone:  "America/New_York",
	},
	"00MN": {
		ICAO:      "00MN",
		Name:      "Battle Lake Municipal Airport",
		City:      "Battle Lake",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1365,
		Latitude:  46.2999992371,
		Longitude: -95.700302124,
		Timezone:  "America/Chicago",
	},
	"00MO": {
		ICAO:      "00MO",
		Name:      "Cooper Flying Service Airport",
		City:      "Alba",
		State:     "Missouri",
		Country:   "US",
		Elevation: 970,
		Latitude:  37.2028007507,
		Longitude: -94.412399292,
		Timezone:  "America/Chicago",
	},
	"00MT": {
		ICAO:      "00MT",
		Name:      "Sands Ranch Airport",
		City:      "Havre",
		State:     "Montana",
		Country:   "US",
		Elevation: 2600,
		Latitude:  48.5374984741,
		Longitude: -109.7050018311,
		Timezone:  "America/Denver",
	},
	"00NC": {
		ICAO:      "00NC",
		Name:      "North Raleigh Airport",
		City:      "Louisburg",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 348,
		Latitude:  36.0852012634,
		Longitude: -78.3713989258,
		Timezone:  "America/New_York",
	},
	"00NY": {
		ICAO:      "00NY",
		Name:      "Weiss Airfield",
		City:      "West Bloomfield",
		State:     "New York",
		Country:   "US",
		Elevation: 1000,
		Latitude:  42.900100708,
		Longitude: -77.4997024536,
		Timezone:  "America/New_York",
	},
	"00OH": {
		ICAO:      "00OH",
		Name:      "Exit 3 Airport",
		City:      "Wauseon",
		State:     "Ohio",
		Country:   "US",
		Elevation: 785,
		Latitude:  41.5909004211,
		Longitude: -84.1421966553,
		Timezone:  "America/New_York",
	},
	"00PN": {
		ICAO:      "00PN",
		Name:      "Ferrell Field",
		City:      "Mercer",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1301,
		Latitude:  41.2995,
		Longitude: -80.211111,
		Timezone:  "America/New_York",
	},
	"00PS": {
		ICAO:      "00PS",
		Name:      "Thomas Field",
		City:      "Loysville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 815,
		Latitude:  40.3777999878,
		Longitude: -77.3653030396,
		Timezone:  "America/New_York",
	},
	"00SC": {
		ICAO:      "00SC",
		Name:      "Flying O Airport",
		City:      "Sumter",
		State:     "South Carolina",
		Country:   "US",
		Elevation: 150,
		Latitude:  34.0093994141,
		Longitude: -80.2671966553,
		Timezone:  "America/New_York",
	},
	"00TN": {
		ICAO:      "00TN",
		Name:      "Ragsdale Road Airport",
		City:      "Manchester",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1100,
		Latitude:  35.5158996582,
		Longitude: -85.9535980225,
		Timezone:  "America/Chicago",
	},
	"00TS": {
		ICAO:      "00TS",
		Name:      "Alpine Range Airport",
		City:      "Everman",
		State:     "Texas",
		Country:   "US",
		Elevation: 670,
		Latitude:  32.6076011658,
		Longitude: -97.2419967651,
		Timezone:  "America/Chicago",
	},
	"00VA": {
		ICAO:      "00VA",
		Name:      "Vaughan Airport",
		City:      "Alton",
		State:     "Virginia",
		Country:   "US",
		Elevation: 551,
		Latitude:  36.57600021,
		Longitude: -78.9991684,
		Timezone:  "America/New_York",
	},
	"00VI": {
		ICAO:      "00VI",
		Name:      "Groundhog Mountain Airport",
		City:      "Hillsville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 2680,
		Latitude:  36.6632995605,
		Longitude: -80.49949646,
		Timezone:  "America/New_York",
	},
	"00WA": {
		ICAO:      "00WA",
		Name:      "Howell Airport",
		City:      "Longbranch",
		State:     "Washington",
		Country:   "US",
		Elevation: 150,
		Latitude:  47.1783981323,
		Longitude: -122.7720031738,
		Timezone:  "America/Los_Angeles",
	},
	"00WI": {
		ICAO:      "00WI",
		Name:      "Northern Lite Airport",
		City:      "Waupaca",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 860,
		Latitude:  44.3042984009,
		Longitude: -89.0501022339,
		Timezone:  "America/Chicago",
	},
	"00WN": {
		ICAO:      "00WN",
		Name:      "Hawks Run Airport",
		City:      "Asotin",
		State:     "Washington",
		Country:   "US",
		Elevation: 2900,
		Latitude:  46.25,
		Longitude: -117.2490005493,
		Timezone:  "America/Los_Angeles",
	},
	"00WV": {
		ICAO:      "00WV",
		Name:      "Lazy J. Aerodrome",
		City:      "Beverly",
		State:     "West Virginia",
		Country:   "US",
		Elevation: 2060,
		Latitude:  38.8288993835,
		Longitude: -79.8660964966,
		Timezone:  "America/New_York",
	},
	"00XS": {
		ICAO:      "00XS",
		Name:      "L P Askew Farms Airport",
		City:      "O'Donnell",
		State:     "Texas",
		Country:   "US",
		Elevation: 3110,
		Latitude:  33.0334014893,
		Longitude: -101.9339981079,
		Timezone:  "America/Chicago",
	},
	"01AL": {
		ICAO:      "01AL",
		Name:      "Ware Island Airport",
		City:      "Clanton",
		State:     "Alabama",
		Country:   "US",
		Elevation: 344,
		Latitude:  32.9459991455,
		Longitude: -86.5139007568,
		Timezone:  "America/Chicago",
	},
	"01CL": {
		ICAO:      "01CL",
		Name:      "Swansboro Country Airport",
		City:      "Placerville",
		State:     "California",
		Country:   "US",
		Elevation: 2594,
		Latitude:  38.7999000549,
		Longitude: -120.7340011597,
		Timezone:  "America/Los_Angeles",
	},
	"01FA": {
		ICAO:      "01FA",
		Name:      "Rybolt Ranch Airport",
		City:      "Orlando",
		State:     "Florida",
		Country:   "US",
		Elevation: 55,
		Latitude:  28.5893993378,
		Longitude: -81.144203186,
		Timezone:  "America/New_York",
	},
	"01FL": {
		ICAO:      "01FL",
		Name:      "Cedar Knoll Flying Ranch Airport",
		City:      "Geneva",
		State:     "Florida",
		Country:   "US",
		Elevation: 19,
		Latitude:  28.7819004059,
		Longitude: -81.1592025757,
		Timezone:  "America/New_York",
	},
	"01GE": {
		ICAO:      "01GE",
		Name:      "The Farm Airport",
		City:      "Wrightsville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 375,
		Latitude:  32.6744003296,
		Longitude: -82.7711029053,
		Timezone:  "America/New_York",
	},
	"01IA": {
		ICAO:      "01IA",
		Name:      "Stender Airport",
		City:      "Maysville",
		State:     "Iowa",
		Country:   "US",
		Elevation: 725,
		Latitude:  41.6610984802,
		Longitude: -90.7413024902,
		Timezone:  "America/Chicago",
	},
	"01ID": {
		ICAO:      "01ID",
		Name:      "Lava Hot Springs Airport",
		City:      "Lava Hot Springs",
		State:     "Idaho",
		Country:   "US",
		Elevation: 5268,
		Latitude:  42.6082000732,
		Longitude: -112.0319976807,
		Timezone:  "America/Boise",
	},
	"01II": {
		ICAO:      "01II",
		Name:      "Myers Field",
		City:      "Lizton",
		State:     "Indiana",
		Country:   "US",
		Elevation: 950,
		Latitude:  39.8849983215,
		Longitude: -86.5066986084,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"01IS": {
		ICAO:      "01IS",
		Name:      "William E. Koenig Airport",
		City:      "Dow",
		State:     "Illinois",
		Country:   "US",
		Elevation: 670,
		Latitude:  39.0162010193,
		Longitude: -90.3181991577,
		Timezone:  "America/Chicago",
	},
	"01KS": {
		ICAO:      "01KS",
		Name:      "Flying N Ranch Airport",
		City:      "Lost Springs",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1485,
		Latitude:  38.540599823,
		Longitude: -97.0033035278,
		Timezone:  "America/Chicago",
	},
	"01LA": {
		ICAO:      "01LA",
		Name:      "Barham Airport",
		City:      "Oak Ridge",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 90,
		Latitude:  32.638999939,
		Longitude: -91.7736968994,
		Timezone:  "America/Chicago",
	},
	"01LL": {
		ICAO:      "01LL",
		Name:      "Schumaier Restricted Landing Area",
		City:      "Pinckneyville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 555,
		Latitude:  38.1258010864,
		Longitude: -89.4638977051,
		Timezone:  "America/Chicago",
	},
	"01LS": {
		ICAO:      "01LS",
		Name:      "Country Breeze Airport",
		City:      "Slaughter",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 125,
		Latitude:  30.7059993744,
		Longitude: -91.0774002075,
		Timezone:  "America/Chicago",
	},
	"01MT": {
		ICAO:      "01MT",
		Name:      "Crystal Lakes Resort Airport",
		City:      "Fortine",
		State:     "Montana",
		Country:   "US",
		Elevation: 3141,
		Latitude:  48.789100647,
		Longitude: -114.8799972534,
		Timezone:  "America/Denver",
	},
	"01NC": {
		ICAO:      "01NC",
		Name:      "Topsail Airpark",
		City:      "Holly Ridge",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 65,
		Latitude:  34.4752998352,
		Longitude: -77.5813980103,
		Timezone:  "America/New_York",
	},
	"01NE": {
		ICAO:      "01NE",
		Name:      "Detour Airport",
		City:      "Wellfleet",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3000,
		Latitude:  40.8436012268,
		Longitude: -100.6529998779,
		Timezone:  "America/Chicago",
	},
	"01NH": {
		ICAO:      "01NH",
		Name:      "Moore Airfield",
		City:      "Enfield",
		State:     "New Hampshire",
		Country:   "US",
		Elevation: 835,
		Latitude:  43.6445007324,
		Longitude: -72.0869979858,
		Timezone:  "America/New_York",
	},
	"01NM": {
		ICAO:      "01NM",
		Name:      "Champion Ranch Airport",
		City:      "Artesia",
		State:     "New Mexico",
		Country:   "US",
		Elevation: 3630,
		Latitude:  33.008611,
		Longitude: -104.540278,
		Timezone:  "America/Denver",
	},
	"01NV": {
		ICAO:      "01NV",
		Name:      "Lantana Ranch Airport",
		City:      "Yerington",
		State:     "Nevada",
		Country:   "US",
		Elevation: 4600,
		Latitude:  38.7639007568,
		Longitude: -119.0270004272,
		Timezone:  "America/Los_Angeles",
	},
	"01OK": {
		ICAO:      "01OK",
		Name:      "Lawrence Airport",
		City:      "Eakly",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1525,
		Latitude:  35.2944984436,
		Longitude: -98.6364974976,
		Timezone:  "America/Chicago",
	},
	"01OR": {
		ICAO:      "01OR",
		Name:      "Red & White Flying Service Airport",
		City:      "Silver Lake",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4346,
		Latitude:  43.1193008423,
		Longitude: -121.0449981689,
		Timezone:  "America/Los_Angeles",
	},
	"01PN": {
		ICAO:      "01PN",
		Name:      "Bierly(Personal Use) Airport",
		City:      "Bellefonte",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 960,
		Latitude:  40.9305992126,
		Longitude: -77.738899231,
		Timezone:  "America/New_York",
	},
	"01PS": {
		ICAO:      "01PS",
		Name:      "Nort's Resort Airport",
		City:      "Meshoppen",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1040,
		Latitude:  41.5959014893,
		Longitude: -76.0296020508,
		Timezone:  "America/New_York",
	},
	"01SC": {
		ICAO:      "01SC",
		Name:      "York Airport",
		City:      "York",
		State:     "South Carolina",
		Country:   "US",
		Elevation: 779,
		Latitude:  35.0321006775,
		Longitude: -81.2527999878,
		Timezone:  "America/New_York",
	},
	"01TE": {
		ICAO:      "01TE",
		Name:      "Smith Field",
		City:      "Forney",
		State:     "Texas",
		Country:   "US",
		Elevation: 505,
		Latitude:  32.7375984192,
		Longitude: -96.4280014038,
		Timezone:  "America/Chicago",
	},
	"01TN": {
		ICAO:      "01TN",
		Name:      "Colonial Air Park",
		City:      "Collierville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 370,
		Latitude:  34.9958992004,
		Longitude: -89.7305984497,
		Timezone:  "America/Chicago",
	},
	"01TX": {
		ICAO:      "01TX",
		Name:      "Mims Farm Ultralightport",
		City:      "Waxahachie",
		State:     "Texas",
		Country:   "US",
		Elevation: 610,
		Latitude:  32.3875999451,
		Longitude: -96.8775024414,
		Timezone:  "America/Chicago",
	},
	"01UT": {
		ICAO:      "01UT",
		Name:      "La Sal Junction Airport",
		City:      "La Sal",
		State:     "Utah",
		Country:   "US",
		Elevation: 6000,
		Latitude:  38.3083000183,
		Longitude: -109.3960037231,
		Timezone:  "America/Denver",
	},
	"01VA": {
		ICAO:      "01VA",
		Name:      "Pickles Airport",
		City:      "Berryville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 500,
		Latitude:  39.125,
		Longitude: -77.9250030518,
		Timezone:  "America/New_York",
	},
	"01WI": {
		ICAO:      "01WI",
		Name:      "Prehn Cranberry Company Airport",
		City:      "Tomah",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 930,
		Latitude:  44.0099983215,
		Longitude: -90.3891983032,
		Timezone:  "America/Chicago",
	},
	"01WY": {
		ICAO:      "01WY",
		Name:      "Keyhole Airport",
		City:      "Moorcroft",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4250,
		Latitude:  44.3471984863,
		Longitude: -104.8109970093,
		Timezone:  "America/Denver",
	},
	"02AK": {
		ICAO:      "02AK",
		Name:      "Rustic Wilderness Airport",
		City:      "Willow",
		State:     "Alaska",
		Country:   "US",
		Elevation: 190,
		Latitude:  61.8768997192,
		Longitude: -150.0980072021,
		Timezone:  "America/Anchorage",
	},
	"02AL": {
		ICAO:      "02AL",
		Name:      "Bass Field",
		City:      "Foley",
		State:     "Alabama",
		Country:   "US",
		Elevation: 61,
		Latitude:  30.3715000153,
		Longitude: -87.7643966675,
		Timezone:  "America/Chicago",
	},
	"02AR": {
		ICAO:      "02AR",
		Name:      "Three Rivers Airport",
		City:      "Little Rock",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 264,
		Latitude:  34.8224449158,
		Longitude: -92.4444198608,
		Timezone:  "America/Chicago",
	},
	"02AZ": {
		ICAO:      "02AZ",
		Name:      "Winchester Farm Airstrip",
		City:      "Willcox",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4200,
		Latitude:  32.3763999939,
		Longitude: -109.93699646,
		Timezone:  "America/Phoenix",
	},
	"02CD": {
		ICAO:      "02CD",
		Name:      "Shannon Field",
		City:      "Clarksdale",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 165,
		Latitude:  34.1296005249,
		Longitude: -90.5240020752,
		Timezone:  "America/Chicago",
	},
	"02CL": {
		ICAO:      "02CL",
		Name:      "Conover Air Lodge Airport",
		City:      "Frazier Park",
		State:     "California",
		Country:   "US",
		Elevation: 5160,
		Latitude:  34.761100769,
		Longitude: -119.0589981079,
		Timezone:  "America/Los_Angeles",
	},
	"02CO": {
		ICAO:      "02CO",
		Name:      "Mc Cullough Airport",
		City:      "Monte Vista",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7615,
		Latitude:  37.6432991028,
		Longitude: -106.04699707,
		Timezone:  "America/Denver",
	},
	"02FA": {
		ICAO:      "02FA",
		Name:      "Osborn Airfield",
		City:      "Groveland",
		State:     "Florida",
		Country:   "US",
		Elevation: 121,
		Latitude:  28.5266990662,
		Longitude: -81.8748016357,
		Timezone:  "America/New_York",
	},
	"02FL": {
		ICAO:      "02FL",
		Name:      "Cuchens Airport",
		City:      "Defuniak Springs",
		State:     "Florida",
		Country:   "US",
		Elevation: 215,
		Latitude:  30.642778,
		Longitude: -86.123889,
		Timezone:  "America/Chicago",
	},
	"02GA": {
		ICAO:      "02GA",
		Name:      "Doug Bolton Field",
		City:      "Commerce",
		State:     "Georgia",
		Country:   "US",
		Elevation: 884,
		Latitude:  34.2025985718,
		Longitude: -83.4290008545,
		Timezone:  "America/New_York",
	},
	"02GE": {
		ICAO:      "02GE",
		Name:      "Etowah Fields Airport",
		City:      "Euharlee",
		State:     "Georgia",
		Country:   "US",
		Elevation: 710,
		Latitude:  34.1753005981,
		Longitude: -84.9244003296,
		Timezone:  "America/New_York",
	},
	"02ID": {
		ICAO:      "02ID",
		Name:      "Morgan Ranch Airport",
		City:      "Cascade",
		State:     "Idaho",
		Country:   "US",
		Elevation: 5634,
		Latitude:  44.5555000305,
		Longitude: -115.3050003052,
		Timezone:  "America/Boise",
	},
	"02II": {
		ICAO:      "02II",
		Name:      "King Ultralightport",
		City:      "Westfield",
		State:     "Indiana",
		Country:   "US",
		Elevation: 925,
		Latitude:  40.0623016357,
		Longitude: -86.2105026245,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"02IN": {
		ICAO:      "02IN",
		Name:      "Diamond P. Field",
		City:      "Muncie",
		State:     "Indiana",
		Country:   "US",
		Elevation: 904,
		Latitude:  40.2089004517,
		Longitude: -85.540802002,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"02KS": {
		ICAO:      "02KS",
		Name:      "Jmj Landing Airport",
		City:      "St Marys",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1170,
		Latitude:  39.2221984863,
		Longitude: -96.0552978516,
		Timezone:  "America/Chicago",
	},
	"02ME": {
		ICAO:      "02ME",
		Name:      "Old Acton Airfield",
		City:      "Acton",
		State:     "Maine",
		Country:   "US",
		Elevation: 700,
		Latitude:  43.5373001099,
		Longitude: -70.929397583,
		Timezone:  "America/New_York",
	},
	"02MI": {
		ICAO:      "02MI",
		Name:      "Fairplains Airpark",
		City:      "Greenville",
		State:     "Michigan",
		Country:   "US",
		Elevation: 850,
		Latitude:  43.1571006775,
		Longitude: -85.1484985352,
		Timezone:  "America/Detroit",
	},
	"02MN": {
		ICAO:      "02MN",
		Name:      "Greenbush Municipal Airport",
		City:      "Greenbush",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1070,
		Latitude:  48.6833000183,
		Longitude: -96.1920013428,
		Timezone:  "America/Chicago",
	},
	"02MO": {
		ICAO:      "02MO",
		Name:      "Troy Airpark",
		City:      "Troy",
		State:     "Missouri",
		Country:   "US",
		Elevation: 650,
		Latitude:  39.0499992371,
		Longitude: -91.0335006714,
		Timezone:  "America/Chicago",
	},
	"02MS": {
		ICAO:      "02MS",
		Name:      "Watts Field",
		City:      "Rochdale",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 153,
		Latitude:  34.1001014709,
		Longitude: -90.8468017578,
		Timezone:  "America/Chicago",
	},
	"02MT": {
		ICAO:      "02MT",
		Name:      "Barrett Field",
		City:      "Cascade",
		State:     "Montana",
		Country:   "US",
		Elevation: 3350,
		Latitude:  47.2374000549,
		Longitude: -111.7419967651,
		Timezone:  "America/Denver",
	},
	"02MU": {
		ICAO:      "02MU",
		Name:      "Timber Line Airpark",
		City:      "Cassville",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1550,
		Latitude:  36.6542015076,
		Longitude: -93.7974014282,
		Timezone:  "America/Chicago",
	},
	"02NE": {
		ICAO:      "02NE",
		Name:      "Benes Service Airport",
		City:      "Valparaiso",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1550,
		Latitude:  41.0745010376,
		Longitude: -96.9045028687,
		Timezone:  "America/Chicago",
	},
	"02NV": {
		ICAO:      "02NV",
		Name:      "Paiute Meadows Airport",
		City:      "Winnemucca",
		State:     "Nevada",
		Country:   "US",
		Elevation: 4443,
		Latitude:  41.3032989502,
		Longitude: -118.5979995728,
		Timezone:  "America/Los_Angeles",
	},
	"02OH": {
		ICAO:      "02OH",
		Name:      "Zimmerman Airport",
		City:      "Fremont",
		State:     "Ohio",
		Country:   "US",
		Elevation: 614,
		Latitude:  41.3763999939,
		Longitude: -83.0832977295,
		Timezone:  "America/New_York",
	},
	"02OI": {
		ICAO:      "02OI",
		Name:      "Murtha Airport",
		City:      "Conneaut",
		State:     "Ohio",
		Country:   "US",
		Elevation: 950,
		Latitude:  41.8019981384,
		Longitude: -80.5653991699,
		Timezone:  "America/New_York",
	},
	"02OR": {
		ICAO:      "02OR",
		Name:      "Rowena Dell Airport",
		City:      "The Dalles",
		State:     "Oregon",
		Country:   "US",
		Elevation: 705,
		Latitude:  45.6814994812,
		Longitude: -121.3160018921,
		Timezone:  "America/Los_Angeles",
	},
	"02PR": {
		ICAO:      "02PR",
		Name:      "Cuylers Airport",
		City:      "Vega Baja",
		State:     "Vega Baja",
		Country:   "PR",
		Elevation: 15,
		Latitude:  18.4533004761,
		Longitude: -66.366897583,
		Timezone:  "America/Puerto_Rico",
	},
	"02PS": {
		ICAO:      "02PS",
		Name:      "Hughes Ultralightport",
		City:      "Tioga",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1700,
		Latitude:  41.9006004333,
		Longitude: -77.2303009033,
		Timezone:  "America/New_York",
	},
	"02SC": {
		ICAO:      "02SC",
		Name:      "Harpers Airport",
		City:      "Estill",
		State:     "South Carolina",
		Country:   "US",
		Elevation: 111,
		Latitude:  32.7584991455,
		Longitude: -81.2248001099,
		Timezone:  "America/New_York",
	},
	"02TN": {
		ICAO:      "02TN",
		Name:      "Ellis Field",
		City:      "Murfreesboro",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 840,
		Latitude:  35.7806015015,
		Longitude: -86.584197998,
		Timezone:  "America/Chicago",
	},
	"02TX": {
		ICAO:      "02TX",
		Name:      "The Palms At Kitty Hawk Airport",
		City:      "New Home",
		State:     "Texas",
		Country:   "US",
		Elevation: 3235,
		Latitude:  33.3704032898,
		Longitude: -101.92288208,
		Timezone:  "America/Chicago",
	},
	"02UT": {
		ICAO:      "02UT",
		Name:      "Lucin Airport",
		City:      "Lucin",
		State:     "Utah",
		Country:   "US",
		Elevation: 4412,
		Latitude:  41.3662986755,
		Longitude: -113.8430023193,
		Timezone:  "America/Denver",
	},
	"02VA": {
		ICAO:      "02VA",
		Name:      "The Greenhouse Airport",
		City:      "Culpeper",
		State:     "Virginia",
		Country:   "US",
		Elevation: 320,
		Latitude:  38.4356994629,
		Longitude: -77.8572006226,
		Timezone:  "America/New_York",
	},
	"02WA": {
		ICAO:      "02WA",
		Name:      "Cawleys South Prairie Airport",
		City:      "South Prairie",
		State:     "Washington",
		Country:   "US",
		Elevation: 690,
		Latitude:  47.1523017883,
		Longitude: -122.09400177,
		Timezone:  "America/Los_Angeles",
	},
	"02WI": {
		ICAO:      "02WI",
		Name:      "Beer Airport",
		City:      "Hudson",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 920,
		Latitude:  45.0318984985,
		Longitude: -92.6557998657,
		Timezone:  "America/Chicago",
	},
	"02WN": {
		ICAO:      "02WN",
		Name:      "Fowler Field",
		City:      "Tonasket",
		State:     "Washington",
		Country:   "US",
		Elevation: 2150,
		Latitude:  48.7458000183,
		Longitude: -119.3190002441,
		Timezone:  "America/Los_Angeles",
	},
	"02XS": {
		ICAO:      "02XS",
		Name:      "Seidel Ranch Airport",
		City:      "Elroy",
		State:     "Texas",
		Country:   "US",
		Elevation: 510,
		Latitude:  30.1476993561,
		Longitude: -97.6628036499,
		Timezone:  "America/Chicago",
	},
	"03AZ": {
		ICAO:      "03AZ",
		Name:      "Thompson International Aviation Airport",
		City:      "Hereford",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4275,
		Latitude:  31.4333992004,
		Longitude: -110.0839996338,
		Timezone:  "America/Phoenix",
	},
	"03CO": {
		ICAO:      "03CO",
		Name:      "Kugel-Strong Airport",
		City:      "Platteville",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4950,
		Latitude:  40.2125015259,
		Longitude: -104.744003296,
		Timezone:  "America/Denver",
	},
	"03FA": {
		ICAO:      "03FA",
		Name:      "Lake Persimmon Airstrip",
		City:      "Lake Placid",
		State:     "Florida",
		Country:   "US",
		Elevation: 70,
		Latitude:  27.353099823,
		Longitude: -81.4080963135,
		Timezone:  "America/New_York",
	},
	"03FD": {
		ICAO:      "03FD",
		Name:      "Tharpe Airport",
		City:      "Bonifay",
		State:     "Florida",
		Country:   "US",
		Elevation: 115,
		Latitude:  30.8288002014,
		Longitude: -85.7310028076,
		Timezone:  "America/Chicago",
	},
	"03IA": {
		ICAO:      "03IA",
		Name:      "East Field",
		City:      "Montezuma",
		State:     "Iowa",
		Country:   "US",
		Elevation: 954,
		Latitude:  41.5819015503,
		Longitude: -92.4613037109,
		Timezone:  "America/Chicago",
	},
	"03ID": {
		ICAO:      "03ID",
		Name:      "Flying Y Ranch Airport",
		City:      "Council",
		State:     "Idaho",
		Country:   "US",
		Elevation: 3180,
		Latitude:  44.7943000793,
		Longitude: -116.5329971313,
		Timezone:  "America/Boise",
	},
	"03II": {
		ICAO:      "03II",
		Name:      "Davis Field Ultralightport",
		City:      "Mount Vernon",
		State:     "Indiana",
		Country:   "US",
		Elevation: 465,
		Latitude:  37.9620018005,
		Longitude: -87.7789001465,
		Timezone:  "America/Chicago",
	},
	"03IL": {
		ICAO:      "03IL",
		Name:      "Wix Airport",
		City:      "Monee",
		State:     "Illinois",
		Country:   "US",
		Elevation: 750,
		Latitude:  41.4023017883,
		Longitude: -87.8167037964,
		Timezone:  "America/Chicago",
	},
	"03IN": {
		ICAO:      "03IN",
		Name:      "Heinzman Airport",
		City:      "Arcadia",
		State:     "Indiana",
		Country:   "US",
		Elevation: 850,
		Latitude:  40.1800003052,
		Longitude: -86.0124969482,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"03KY": {
		ICAO:      "03KY",
		Name:      "Flying H Farms Airport",
		City:      "Henderson",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 385,
		Latitude:  37.7916984558,
		Longitude: -87.5413970947,
		Timezone:  "America/Chicago",
	},
	"03MA": {
		ICAO:      "03MA",
		Name:      "Hadley Airport",
		City:      "Hadley",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 150,
		Latitude:  42.3931007385,
		Longitude: -72.5520019531,
		Timezone:  "America/New_York",
	},
	"03ME": {
		ICAO:      "03ME",
		Name:      "Maple Ridge Airport",
		City:      "Harrison",
		State:     "Maine",
		Country:   "US",
		Elevation: 556,
		Latitude:  44.0842018127,
		Longitude: -70.6272964478,
		Timezone:  "America/New_York",
	},
	"03MN": {
		ICAO:      "03MN",
		Name:      "Nauerth Land Ranch Airport",
		City:      "Lakefield",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1435,
		Latitude:  43.6251983643,
		Longitude: -95.2247009277,
		Timezone:  "America/Chicago",
	},
	"03MO": {
		ICAO:      "03MO",
		Name:      "Cahoochie Airport",
		City:      "Urbana",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1010,
		Latitude:  37.8844985962,
		Longitude: -93.1315994263,
		Timezone:  "America/Chicago",
	},
	"03MT": {
		ICAO:      "03MT",
		Name:      "Cascade Airport",
		City:      "Cascade",
		State:     "Montana",
		Country:   "US",
		Elevation: 3580,
		Latitude:  47.2708015442,
		Longitude: -111.7220001221,
		Timezone:  "America/Denver",
	},
	"03MU": {
		ICAO:      "03MU",
		Name:      "Mc Donnell Airport",
		City:      "Archie",
		State:     "Missouri",
		Country:   "US",
		Elevation: 874,
		Latitude:  38.4925003052,
		Longitude: -94.4124984741,
		Timezone:  "America/Chicago",
	},
	"03NC": {
		ICAO:      "03NC",
		Name:      "Pilots Ridge Airport",
		City:      "Carolina Beach",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 35,
		Latitude:  34.1043014526,
		Longitude: -77.9041976929,
		Timezone:  "America/New_York",
	},
	"03ND": {
		ICAO:      "03ND",
		Name:      "Olafson Brothers Airport",
		City:      "Edinburg",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1045,
		Latitude:  48.6263999939,
		Longitude: -97.8290023804,
		Timezone:  "America/Chicago",
	},
	"03NE": {
		ICAO:      "03NE",
		Name:      "Hyde Ranch Airport",
		City:      "Comstock",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2430,
		Latitude:  41.5463981628,
		Longitude: -99.331199646,
		Timezone:  "America/Chicago",
	},
	"03NV": {
		ICAO:      "03NV",
		Name:      "Llama Ranch Airport",
		City:      "Ruby Valley",
		State:     "Nevada",
		Country:   "US",
		Elevation: 6120,
		Latitude:  40.584400177,
		Longitude: -115.297996521,
		Timezone:  "America/Los_Angeles",
	},
	"03NY": {
		ICAO:      "03NY",
		Name:      "Talmage Field",
		City:      "Riverhead",
		State:     "New York",
		Country:   "US",
		Elevation: 95,
		Latitude:  40.9584007263,
		Longitude: -72.716796875,
		Timezone:  "America/New_York",
	},
	"03OH": {
		ICAO:      "03OH",
		Name:      "Gibbs Field",
		City:      "Fremont",
		State:     "Ohio",
		Country:   "US",
		Elevation: 580,
		Latitude:  41.4183998108,
		Longitude: -83.0174026489,
		Timezone:  "America/New_York",
	},
	"03OK": {
		ICAO:      "03OK",
		Name:      "Sahoma Lake Airport",
		City:      "Sapulpa",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 890,
		Latitude:  36.0411987305,
		Longitude: -96.1613998413,
		Timezone:  "America/Chicago",
	},
	"03OR": {
		ICAO:      "03OR",
		Name:      "Powwatka Ridge Airport",
		City:      "Troy",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3340,
		Latitude:  45.8554000854,
		Longitude: -117.4840011597,
		Timezone:  "America/Los_Angeles",
	},
	"03PS": {
		ICAO:      "03PS",
		Name:      "Ziggy's Field",
		City:      "Bellefonte",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1050,
		Latitude:  40.8499984741,
		Longitude: -77.9056015015,
		Timezone:  "America/New_York",
	},
	"03TA": {
		ICAO:      "03TA",
		Name:      "Gay Hill Farm Airport",
		City:      "Gay Hill",
		State:     "Texas",
		Country:   "US",
		Elevation: 505,
		Latitude:  30.2626991272,
		Longitude: -96.5001983643,
		Timezone:  "America/Chicago",
	},
	"03TE": {
		ICAO:      "03TE",
		Name:      "Barronena Ranch Airport",
		City:      "Hebbronville",
		State:     "Texas",
		Country:   "US",
		Elevation: 600,
		Latitude:  27.4911003113,
		Longitude: -98.6695022583,
		Timezone:  "America/Chicago",
	},
	"03UT": {
		ICAO:      "03UT",
		Name:      "A Z Minerals Corporation Airport",
		City:      "Mexican Hat",
		State:     "Utah",
		Country:   "US",
		Elevation: 5315,
		Latitude:  37.1208000183,
		Longitude: -109.986000061,
		Timezone:  "America/Denver",
	},
	"03WA": {
		ICAO:      "03WA",
		Name:      "Spangle Field",
		City:      "Spangle",
		State:     "Washington",
		Country:   "US",
		Elevation: 2440,
		Latitude:  47.4081993103,
		Longitude: -117.3720016479,
		Timezone:  "America/Los_Angeles",
	},
	"03WI": {
		ICAO:      "03WI",
		Name:      "Zink Airport",
		City:      "Berlin",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 880,
		Latitude:  44.0275001526,
		Longitude: -88.880897522,
		Timezone:  "America/Chicago",
	},
	"03WN": {
		ICAO:      "03WN",
		Name:      "Aerostone Ranch Airport",
		City:      "Goldendale",
		State:     "Washington",
		Country:   "US",
		Elevation: 2320,
		Latitude:  45.875,
		Longitude: -120.6699981689,
		Timezone:  "America/Los_Angeles",
	},
	"03XS": {
		ICAO:      "03XS",
		Name:      "Creekside Airport",
		City:      "Mertzon",
		State:     "Texas",
		Country:   "US",
		Elevation: 2100,
		Latitude:  31.3180999756,
		Longitude: -100.7539978027,
		Timezone:  "America/Chicago",
	},
	"04AZ": {
		ICAO:      "04AZ",
		Name:      "Chinle Airport",
		City:      "Chinle",
		State:     "Arizona",
		Country:   "US",
		Elevation: 5515,
		Latitude:  36.1542015076,
		Longitude: -109.5510025024,
		Timezone:  "America/Denver",
	},
	"04CA": {
		ICAO:      "04CA",
		Name:      "Gray Butte Field",
		City:      "Palmdale",
		State:     "California",
		Country:   "US",
		Elevation: 3020,
		Latitude:  34.5638999939,
		Longitude: -117.6750030518,
		Timezone:  "America/Los_Angeles",
	},
	"04CL": {
		ICAO:      "04CL",
		Name:      "Hunt's Sky Ranch Airport",
		City:      "Julian",
		State:     "California",
		Country:   "US",
		Elevation: 2310,
		Latitude:  33.0816993713,
		Longitude: -116.4410018921,
		Timezone:  "America/Los_Angeles",
	},
	"04FA": {
		ICAO:      "04FA",
		Name:      "Richards Field",
		City:      "Homestead",
		State:     "Florida",
		Country:   "US",
		Elevation: 9,
		Latitude:  25.5587005615,
		Longitude: -80.5150985718,
		Timezone:  "America/New_York",
	},
	"04FL": {
		ICAO:      "04FL",
		Name:      "Cross Creek Farms Airport",
		City:      "Ormond",
		State:     "Florida",
		Country:   "US",
		Elevation: 30,
		Latitude:  29.2394008636,
		Longitude: -81.2220001221,
		Timezone:  "America/New_York",
	},
	"04IA": {
		ICAO:      "04IA",
		Name:      "Middlekoop Airport",
		City:      "Packwood",
		State:     "Iowa",
		Country:   "US",
		Elevation: 801,
		Latitude:  41.0882987976,
		Longitude: -92.0546035767,
		Timezone:  "America/Chicago",
	},
	"04ID": {
		ICAO:      "04ID",
		Name:      "Lanham Field",
		City:      "Emmett",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2343,
		Latitude:  43.8777008057,
		Longitude: -116.5370025635,
		Timezone:  "America/Boise",
	},
	"04II": {
		ICAO:      "04II",
		Name:      "Turkey Run Airport",
		City:      "New Haven",
		State:     "Indiana",
		Country:   "US",
		Elevation: 765,
		Latitude:  41.1305999756,
		Longitude: -84.994102478,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"04IL": {
		ICAO:      "04IL",
		Name:      "Schertz Aerial Service - Hudson Airport",
		City:      "Hudson",
		State:     "Illinois",
		Country:   "US",
		Elevation: 755,
		Latitude:  40.6375007629,
		Longitude: -89.0070037842,
		Timezone:  "America/Chicago",
	},
	"04IS": {
		ICAO:      "04IS",
		Name:      "Van Gorder Airport",
		City:      "Mansfield",
		State:     "Illinois",
		Country:   "US",
		Elevation: 728,
		Latitude:  40.1786003113,
		Longitude: -88.5690002441,
		Timezone:  "America/Chicago",
	},
	"04KY": {
		ICAO:      "04KY",
		Name:      "Natchez Trace Farm Airport",
		City:      "Hiseville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 710,
		Latitude:  37.1351013184,
		Longitude: -85.7893981934,
		Timezone:  "America/Chicago",
	},
	"04LL": {
		ICAO:      "04LL",
		Name:      "Brunner Airport",
		City:      "West Dundee",
		State:     "Illinois",
		Country:   "US",
		Elevation: 750,
		Latitude:  42.136100769,
		Longitude: -88.2875976563,
		Timezone:  "America/Chicago",
	},
	"04ME": {
		ICAO:      "04ME",
		Name:      "Beech Hill Airport",
		City:      "Mercer",
		State:     "Maine",
		Country:   "US",
		Elevation: 625,
		Latitude:  44.6666984558,
		Longitude: -69.8938980103,
		Timezone:  "America/New_York",
	},
	"04MI": {
		ICAO:      "04MI",
		Name:      "Rapids Airway Airport",
		City:      "Eaton Rapids",
		State:     "Michigan",
		Country:   "US",
		Elevation: 880,
		Latitude:  42.5345001221,
		Longitude: -84.6283035278,
		Timezone:  "America/Detroit",
	},
	"04MN": {
		ICAO:      "04MN",
		Name:      "Helblad Airport",
		City:      "Kelliher",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1195,
		Latitude:  47.9858016968,
		Longitude: -94.6868972778,
		Timezone:  "America/Chicago",
	},
	"04MO": {
		ICAO:      "04MO",
		Name:      "Airpark Private Airport",
		City:      "Vandalia",
		State:     "Missouri",
		Country:   "US",
		Elevation: 767,
		Latitude:  39.3012008667,
		Longitude: -91.451499939,
		Timezone:  "America/Chicago",
	},
	"04MS": {
		ICAO:      "04MS",
		Name:      "Nick's Flying Service Inc Airport",
		City:      "Rolling Fork",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 100,
		Latitude:  32.9396018982,
		Longitude: -90.8373031616,
		Timezone:  "America/Chicago",
	},
	"04MT": {
		ICAO:      "04MT",
		Name:      "Pluhar Airport",
		City:      "Cohagen",
		State:     "Montana",
		Country:   "US",
		Elevation: 2715,
		Latitude:  47.1374977778,
		Longitude: -106.488095,
		Timezone:  "America/Denver",
	},
	"04NE": {
		ICAO:      "04NE",
		Name:      "Mc Connell Field Airport",
		City:      "Sarben",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3035,
		Latitude:  41.1599998474,
		Longitude: -101.299003601,
		Timezone:  "America/Denver",
	},
	"04NJ": {
		ICAO:      "04NJ",
		Name:      "Emmanuel Airport",
		City:      "Elmer",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 155,
		Latitude:  39.5968017578,
		Longitude: -75.2334976196,
		Timezone:  "America/New_York",
	},
	"04NV": {
		ICAO:      "04NV",
		Name:      "Kingston Ranch Airport",
		City:      "Sandy Valley",
		State:     "California",
		Country:   "US",
		Elevation: 2620,
		Latitude:  35.7555007935,
		Longitude: -115.665000916,
		Timezone:  "America/Los_Angeles",
	},
	"04NY": {
		ICAO:      "04NY",
		Name:      "Klaverack Airport",
		City:      "Hudson",
		State:     "New York",
		Country:   "US",
		Elevation: 250,
		Latitude:  42.262298584,
		Longitude: -73.6961975098,
		Timezone:  "America/New_York",
	},
	"04OH": {
		ICAO:      "04OH",
		Name:      "Bossow Airport",
		City:      "Garrettsville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1080,
		Latitude:  41.2476997375,
		Longitude: -81.1007003784,
		Timezone:  "America/New_York",
	},
	"04OR": {
		ICAO:      "04OR",
		Name:      "Collins Landing Strip",
		City:      "Mitchell",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1500,
		Latitude:  44.7499008179,
		Longitude: -120.200996399,
		Timezone:  "America/Los_Angeles",
	},
	"04TN": {
		ICAO:      "04TN",
		Name:      "Hensley Airpark",
		City:      "Chuckey",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1430,
		Latitude:  36.1833000183,
		Longitude: -82.6789016724,
		Timezone:  "America/New_York",
	},
	"04TX": {
		ICAO:      "04TX",
		Name:      "Pocock Airport",
		City:      "China Spring",
		State:     "Texas",
		Country:   "US",
		Elevation: 565,
		Latitude:  31.7322006226,
		Longitude: -97.3688964844,
		Timezone:  "America/Chicago",
	},
	"04UT": {
		ICAO:      "04UT",
		Name:      "Navajo Mountain Airport",
		City:      "Navajo Mountain Trdng Post",
		State:     "Utah",
		Country:   "US",
		Elevation: 6160,
		Latitude:  37.0083007812,
		Longitude: -110.794998169,
		Timezone:  "America/Denver",
	},
	"04VG": {
		ICAO:      "04VG",
		Name:      "Manquin Flight Park Ultralightport",
		City:      "Manquin",
		State:     "Virginia",
		Country:   "US",
		Elevation: 50,
		Latitude:  37.7144012451,
		Longitude: -77.2117996216,
		Timezone:  "America/New_York",
	},
	"04VT": {
		ICAO:      "04VT",
		Name:      "Lightning Bolt Field Airport",
		City:      "Cabot",
		State:     "Vermont",
		Country:   "US",
		Elevation: 2156,
		Latitude:  44.4011001587,
		Longitude: -72.2636032104,
		Timezone:  "America/New_York",
	},
	"04WA": {
		ICAO:      "04WA",
		Name:      "Ox Meadows Airport",
		City:      "Spokane",
		State:     "Washington",
		Country:   "US",
		Elevation: 2345,
		Latitude:  47.5817985535,
		Longitude: -117.43699646,
		Timezone:  "America/Los_Angeles",
	},
	"04WI": {
		ICAO:      "04WI",
		Name:      "Dutch Gap Airstrip",
		City:      "Bristol",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 760,
		Latitude:  42.523399353,
		Longitude: -88.0167007446,
		Timezone:  "America/Chicago",
	},
	"04WN": {
		ICAO:      "04WN",
		Name:      "Stillwater Creek Airport",
		City:      "Royal City",
		State:     "Washington",
		Country:   "US",
		Elevation: 1510,
		Latitude:  46.963889,
		Longitude: -119.641667,
		Timezone:  "America/Los_Angeles",
	},
	"05AK": {
		ICAO:      "05AK",
		Name:      "Wasilla Creek Airpark",
		City:      "Palmer",
		State:     "Alaska",
		Country:   "US",
		Elevation: 620,
		Latitude:  61.66830063,
		Longitude: -149.1880035,
		Timezone:  "America/Anchorage",
	},
	"05AR": {
		ICAO:      "05AR",
		Name:      "Ozark Skies Airpark",
		City:      "Kingston",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1900,
		Latitude:  36.000633,
		Longitude: -93.57025,
		Timezone:  "America/Chicago",
	},
	"05CL": {
		ICAO:      "05CL",
		Name:      "Pope Valley Airport",
		City:      "Pope Valley",
		State:     "California",
		Country:   "US",
		Elevation: 618,
		Latitude:  38.6106987,
		Longitude: -122.39099884,
		Timezone:  "America/Los_Angeles",
	},
	"05CO": {
		ICAO:      "05CO",
		Name:      "Rancho De Aereo Airport",
		City:      "Mead",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4978,
		Latitude:  40.2149839,
		Longitude: -104.9844228,
		Timezone:  "America/Denver",
	},
	"05FA": {
		ICAO:      "05FA",
		Name:      "Melanie's Airport",
		City:      "Florala",
		State:     "Florida",
		Country:   "US",
		Elevation: 295,
		Latitude:  30.9188995361,
		Longitude: -86.2182998657,
		Timezone:  "America/Chicago",
	},
	"05FL": {
		ICAO:      "05FL",
		Name:      "Charlton Strip",
		City:      "La Belle",
		State:     "Florida",
		Country:   "US",
		Elevation: 36,
		Latitude:  26.5230998993,
		Longitude: -81.4083023071,
		Timezone:  "America/New_York",
	},
	"05GA": {
		ICAO:      "05GA",
		Name:      "Raju Airport",
		City:      "Richland",
		State:     "Georgia",
		Country:   "US",
		Elevation: 625,
		Latitude:  32.0713996887,
		Longitude: -84.7144012451,
		Timezone:  "America/New_York",
	},
	"05IA": {
		ICAO:      "05IA",
		Name:      "Spotts Field",
		City:      "Nora Springs",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1155,
		Latitude:  43.1375007629,
		Longitude: -93.0682983398,
		Timezone:  "America/Chicago",
	},
	"05ID": {
		ICAO:      "05ID",
		Name:      "Running Creek Ranch Airport",
		City:      "Grangeville",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2969,
		Latitude:  45.914100647,
		Longitude: -114.8359985352,
		Timezone:  "America/Los_Angeles",
	},
	"05II": {
		ICAO:      "05II",
		Name:      "Reichhart Airport",
		City:      "New Haven",
		State:     "Indiana",
		Country:   "US",
		Elevation: 795,
		Latitude:  41.0287017822,
		Longitude: -84.9972000122,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"05IL": {
		ICAO:      "05IL",
		Name:      "Classic Landings Airport",
		City:      "Bonfield",
		State:     "Illinois",
		Country:   "US",
		Elevation: 635,
		Latitude:  41.1567001343,
		Longitude: -88.0167007446,
		Timezone:  "America/Chicago",
	},
	"05IN": {
		ICAO:      "05IN",
		Name:      "Cooper Airport",
		City:      "Belleville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 860,
		Latitude:  39.641998291,
		Longitude: -86.5042037964,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"05IS": {
		ICAO:      "05IS",
		Name:      "Hardy Airport",
		City:      "Mount Auburn",
		State:     "Illinois",
		Country:   "US",
		Elevation: 605,
		Latitude:  39.7258987427,
		Longitude: -89.263999939,
		Timezone:  "America/Chicago",
	},
	"05KS": {
		ICAO:      "05KS",
		Name:      "Darbro Field",
		City:      "Elk City",
		State:     "Kansas",
		Country:   "US",
		Elevation: 945,
		Latitude:  37.2662010193,
		Longitude: -95.959197998,
		Timezone:  "America/Chicago",
	},
	"05KY": {
		ICAO:      "05KY",
		Name:      "Cartersville Airport",
		City:      "Berea",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 1070,
		Latitude:  37.53689957,
		Longitude: -84.40720367,
		Timezone:  "America/New_York",
	},
	"05LA": {
		ICAO:      "05LA",
		Name:      "Greg's Flying Service Airport",
		City:      "Baskin",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 76,
		Latitude:  32.2653007507,
		Longitude: -91.6977996826,
		Timezone:  "America/Chicago",
	},
	"05LS": {
		ICAO:      "05LS",
		Name:      "Grass Roots Airport",
		City:      "Alexandria",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 83,
		Latitude:  31.252199173,
		Longitude: -92.4757995605,
		Timezone:  "America/Chicago",
	},
	"05MD": {
		ICAO:      "05MD",
		Name:      "Breezecroft Airport",
		City:      "Chestertown",
		State:     "Maryland",
		Country:   "US",
		Elevation: 20,
		Latitude:  39.2500991821,
		Longitude: -76.2052001953,
		Timezone:  "America/New_York",
	},
	"05ME": {
		ICAO:      "05ME",
		Name:      "Drisko Airport",
		City:      "Jonesboro",
		State:     "Maine",
		Country:   "US",
		Elevation: 70,
		Latitude:  44.6570014954,
		Longitude: -67.5591964722,
		Timezone:  "America/New_York",
	},
	"05NE": {
		ICAO:      "05NE",
		Name:      "Mc Kay Airport",
		City:      "Spalding",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2010,
		Latitude:  41.7221984863,
		Longitude: -98.3089981079,
		Timezone:  "America/Chicago",
	},
	"05OI": {
		ICAO:      "05OI",
		Name:      "Dorlon Airpark",
		City:      "Columbia Station",
		State:     "Ohio",
		Country:   "US",
		Elevation: 830,
		Latitude:  41.2883987427,
		Longitude: -81.962600708,
		Timezone:  "America/New_York",
	},
	"05OK": {
		ICAO:      "05OK",
		Name:      "Hawk Haven Airport",
		City:      "Crescent",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1075,
		Latitude:  35.8534011841,
		Longitude: -97.5674972534,
		Timezone:  "America/Chicago",
	},
	"05OR": {
		ICAO:      "05OR",
		Name:      "Peacock Ranch Airport",
		City:      "Enterprise",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3500,
		Latitude:  45.4415016174,
		Longitude: -117.3509979248,
		Timezone:  "America/Los_Angeles",
	},
	"05PS": {
		ICAO:      "05PS",
		Name:      "Mills Brothers Airport",
		City:      "Burnt Cabins",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1210,
		Latitude:  40.1427993774,
		Longitude: -77.8619003296,
		Timezone:  "America/New_York",
	},
	"05TA": {
		ICAO:      "05TA",
		Name:      "Brandes Air Field",
		City:      "Sealy",
		State:     "Texas",
		Country:   "US",
		Elevation: 276,
		Latitude:  29.8090991974,
		Longitude: -96.2650985718,
		Timezone:  "America/Chicago",
	},
	"05TE": {
		ICAO:      "05TE",
		Name:      "Hilde-Griff Field",
		City:      "Georgetown",
		State:     "Texas",
		Country:   "US",
		Elevation: 950,
		Latitude:  30.7220993042,
		Longitude: -97.7913970947,
		Timezone:  "America/Chicago",
	},
	"05TS": {
		ICAO:      "05TS",
		Name:      "Dew Drop Airport",
		City:      "Justin",
		State:     "Texas",
		Country:   "US",
		Elevation: 750,
		Latitude:  33.1292991638,
		Longitude: -97.3585968018,
		Timezone:  "America/Chicago",
	},
	"05TX": {
		ICAO:      "05TX",
		Name:      "Circle 'A' Ranch Airport",
		City:      "Blum",
		State:     "Texas",
		Country:   "US",
		Elevation: 608,
		Latitude:  32.1291999817,
		Longitude: -97.4041976929,
		Timezone:  "America/Chicago",
	},
	"05UT": {
		ICAO:      "05UT",
		Name:      "Oljato Airport",
		City:      "Oljato",
		State:     "Utah",
		Country:   "US",
		Elevation: 4838,
		Latitude:  37.0332984924,
		Longitude: -110.3170013428,
		Timezone:  "America/Denver",
	},
	"05VA": {
		ICAO:      "05VA",
		Name:      "Providence Airport",
		City:      "Farmville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 445,
		Latitude:  37.2242012024,
		Longitude: -78.4186019897,
		Timezone:  "America/New_York",
	},
	"05WI": {
		ICAO:      "05WI",
		Name:      "Ames Private Airport",
		City:      "Darien",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 958,
		Latitude:  42.6511001587,
		Longitude: -88.7459030151,
		Timezone:  "America/Chicago",
	},
	"05WN": {
		ICAO:      "05WN",
		Name:      "Flat Creek Field",
		City:      "Kettle Falls",
		State:     "Washington",
		Country:   "US",
		Elevation: 1570,
		Latitude:  48.702222,
		Longitude: -118.0475,
		Timezone:  "America/Los_Angeles",
	},
	"05XS": {
		ICAO:      "05XS",
		Name:      "Johnson Memorial Airport",
		City:      "Wilmot",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 105,
		Latitude:  33.0792999268,
		Longitude: -91.5431976318,
		Timezone:  "America/Chicago",
	},
	"06AK": {
		ICAO:      "06AK",
		Name:      "June Lake Airpark",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 425,
		Latitude:  61.627619,
		Longitude: -149.575331,
		Timezone:  "America/Anchorage",
	},
	"06AR": {
		ICAO:      "06AR",
		Name:      "Bondair Airport",
		City:      "Higden",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 499,
		Latitude:  35.5444984436,
		Longitude: -92.1848983765,
		Timezone:  "America/Chicago",
	},
	"06CO": {
		ICAO:      "06CO",
		Name:      "Jecan Airport",
		City:      "Branson",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5100,
		Latitude:  37.3875007629,
		Longitude: -103.6910018921,
		Timezone:  "America/Denver",
	},
	"06FA": {
		ICAO:      "06FA",
		Name:      "William P Gwinn Airport",
		City:      "Jupiter",
		State:     "Florida",
		Country:   "US",
		Elevation: 28,
		Latitude:  26.9083995819,
		Longitude: -80.3289031982,
		Timezone:  "America/New_York",
	},
	"06FD": {
		ICAO:      "06FD",
		Name:      "Grass Roots Airpark",
		City:      "Mascotte",
		State:     "Florida",
		Country:   "US",
		Elevation: 105,
		Latitude:  28.6417007446,
		Longitude: -81.8850021362,
		Timezone:  "America/New_York",
	},
	"06IA": {
		ICAO:      "06IA",
		Name:      "Rich Field",
		City:      "Walford",
		State:     "Iowa",
		Country:   "US",
		Elevation: 880,
		Latitude:  41.8414001465,
		Longitude: -91.8342971802,
		Timezone:  "America/Chicago",
	},
	"06ID": {
		ICAO:      "06ID",
		Name:      "Larkin Airport",
		City:      "Kuna",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2750,
		Latitude:  43.4626998901,
		Longitude: -116.3590011597,
		Timezone:  "America/Boise",
	},
	"06IL": {
		ICAO:      "06IL",
		Name:      "Humm Airport",
		City:      "Earlville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 708,
		Latitude:  41.5994987488,
		Longitude: -88.8699035645,
		Timezone:  "America/Chicago",
	},
	"06IN": {
		ICAO:      "06IN",
		Name:      "Ellis Fly-In Airport",
		City:      "Blackhawk",
		State:     "Indiana",
		Country:   "US",
		Elevation: 575,
		Latitude:  39.2827987671,
		Longitude: -87.3035964966,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"06IS": {
		ICAO:      "06IS",
		Name:      "Sinele Strip",
		City:      "Nauvoo",
		State:     "Illinois",
		Country:   "US",
		Elevation: 695,
		Latitude:  40.5463981628,
		Longitude: -91.2742996216,
		Timezone:  "America/Chicago",
	},
	"06MN": {
		ICAO:      "06MN",
		Name:      "Pike Field Airport",
		City:      "Virginia",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1463,
		Latitude:  47.6519012451,
		Longitude: -92.4167022705,
		Timezone:  "America/Chicago",
	},
	"06MO": {
		ICAO:      "06MO",
		Name:      "Noahs Ark Airport",
		City:      "Waldron",
		State:     "Missouri",
		Country:   "US",
		Elevation: 755,
		Latitude:  39.2305984497,
		Longitude: -94.804397583,
		Timezone:  "America/Chicago",
	},
	"06MT": {
		ICAO:      "06MT",
		Name:      "Torgerson Airport",
		City:      "Ethridge",
		State:     "Montana",
		Country:   "US",
		Elevation: 3541,
		Latitude:  48.5583000183,
		Longitude: -112.1090011597,
		Timezone:  "America/Denver",
	},
	"06NC": {
		ICAO:      "06NC",
		Name:      "Tailwinds Airport",
		City:      "St. Pauls",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 163,
		Latitude:  34.8274993896,
		Longitude: -78.9302978516,
		Timezone:  "America/New_York",
	},
	"06ND": {
		ICAO:      "06ND",
		Name:      "Fitterer's Strip",
		City:      "Glen Ullin",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 2180,
		Latitude:  46.7489013672,
		Longitude: -101.6689987183,
		Timezone:  "America/North_Dakota/New_Salem",
	},
	"06NH": {
		ICAO:      "06NH",
		Name:      "Cole Farm Airport",
		City:      "Kensington",
		State:     "New Hampshire",
		Country:   "US",
		Elevation: 160,
		Latitude:  42.9283981323,
		Longitude: -70.9730987549,
		Timezone:  "America/New_York",
	},
	"06NY": {
		ICAO:      "06NY",
		Name:      "Murphy Field",
		City:      "Auburn",
		State:     "New York",
		Country:   "US",
		Elevation: 1100,
		Latitude:  42.8428001404,
		Longitude: -76.4452972412,
		Timezone:  "America/New_York",
	},
	"06OH": {
		ICAO:      "06OH",
		Name:      "Hal Joy Airfield",
		City:      "Williamsfield",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1160,
		Latitude:  41.5097999573,
		Longitude: -80.5912017822,
		Timezone:  "America/New_York",
	},
	"06OI": {
		ICAO:      "06OI",
		Name:      "Green Acres Airport",
		City:      "New Lebanon",
		State:     "Ohio",
		Country:   "US",
		Elevation: 915,
		Latitude:  39.7347984314,
		Longitude: -84.3822021484,
		Timezone:  "America/New_York",
	},
	"06OR": {
		ICAO:      "06OR",
		Name:      "Hayden Mountain Airport",
		City:      "Forest Grove",
		State:     "Oregon",
		Country:   "US",
		Elevation: 850,
		Latitude:  45.4650993347,
		Longitude: -123.0800018311,
		Timezone:  "America/Los_Angeles",
	},
	"06PA": {
		ICAO:      "06PA",
		Name:      "Freefall Oz Airport",
		City:      "Ceres",
		State:     "New York",
		Country:   "US",
		Elevation: 1440,
		Latitude:  42.0083999634,
		Longitude: -78.258102417,
		Timezone:  "America/New_York",
	},
	"06R": {
		ICAO:      "06R",
		Name:      "Grawunder Field Airport",
		City:      "Bellville",
		State:     "Texas",
		Country:   "US",
		Elevation: 89,
		Latitude:  29.9409722222,
		Longitude: -96.24525,
		Timezone:  "America/Chicago",
	},
	"06SN": {
		ICAO:      "06SN",
		Name:      "Beyer Farm Airport",
		City:      "Derby",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1340,
		Latitude:  37.5839004517,
		Longitude: -97.2530975342,
		Timezone:  "America/Chicago",
	},
	"06TA": {
		ICAO:      "06TA",
		Name:      "Glaser Field",
		City:      "Buckholts",
		State:     "Texas",
		Country:   "US",
		Elevation: 480,
		Latitude:  30.9085006714,
		Longitude: -97.116897583,
		Timezone:  "America/Chicago",
	},
	"06TE": {
		ICAO:      "06TE",
		Name:      "Ainsworth Airport",
		City:      "Cleveland",
		State:     "Texas",
		Country:   "US",
		Elevation: 129,
		Latitude:  30.3127002716,
		Longitude: -95.0269012451,
		Timezone:  "America/Chicago",
	},
	"06TN": {
		ICAO:      "06TN",
		Name:      "Hunter STOLport",
		City:      "Columbia",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 640,
		Latitude:  35.6972999573,
		Longitude: -86.9569015503,
		Timezone:  "America/Chicago",
	},
	"06TS": {
		ICAO:      "06TS",
		Name:      "Roscoe Mc Connico Airport",
		City:      "Kerens",
		State:     "Texas",
		Country:   "US",
		Elevation: 350,
		Latitude:  32.1207008362,
		Longitude: -96.1791000366,
		Timezone:  "America/Chicago",
	},
	"06TX": {
		ICAO:      "06TX",
		Name:      "Diamond N Ranch Airport",
		City:      "Hockley",
		State:     "Texas",
		Country:   "US",
		Elevation: 215,
		Latitude:  30.0576000214,
		Longitude: -95.8128967285,
		Timezone:  "America/Chicago",
	},
	"06VA": {
		ICAO:      "06VA",
		Name:      "Mount Horeb Field",
		City:      "Grottoes",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1160,
		Latitude:  38.2490005493,
		Longitude: -78.8553009033,
		Timezone:  "America/New_York",
	},
	"06WI": {
		ICAO:      "06WI",
		Name:      "Young Private Airport",
		City:      "Frederic",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1075,
		Latitude:  45.6627006531,
		Longitude: -92.4502029419,
		Timezone:  "America/Chicago",
	},
	"06WN": {
		ICAO:      "06WN",
		Name:      "Western Airpark",
		City:      "Yelm",
		State:     "Washington",
		Country:   "US",
		Elevation: 394,
		Latitude:  46.9249992371,
		Longitude: -122.5530014038,
		Timezone:  "America/Los_Angeles",
	},
	"06XS": {
		ICAO:      "06XS",
		Name:      "Campbell Field",
		City:      "Aubrey",
		State:     "Texas",
		Country:   "US",
		Elevation: 546,
		Latitude:  33.3246994019,
		Longitude: -97.0441970825,
		Timezone:  "America/Chicago",
	},
	"07AR": {
		ICAO:      "07AR",
		Name:      "Morrilton Airport",
		City:      "Morrilton",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 300,
		Latitude:  35.1944999695,
		Longitude: -92.800201416,
		Timezone:  "America/Chicago",
	},
	"07CL": {
		ICAO:      "07CL",
		Name:      "Richvale Airport",
		City:      "Richvale",
		State:     "California",
		Country:   "US",
		Elevation: 100,
		Latitude:  39.4976997375,
		Longitude: -121.7710037231,
		Timezone:  "America/Los_Angeles",
	},
	"07CO": {
		ICAO:      "07CO",
		Name:      "Comanche Creek Airport",
		City:      "Kiowa",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6620,
		Latitude:  39.2635993958,
		Longitude: -104.4270019531,
		Timezone:  "America/Denver",
	},
	"07FA": {
		ICAO:      "07FA",
		Name:      "Ocean Reef Club Airport",
		City:      "Key Largo",
		State:     "Florida",
		Country:   "US",
		Elevation: 8,
		Latitude:  25.3253993988,
		Longitude: -80.2748031616,
		Timezone:  "America/New_York",
	},
	"07FD": {
		ICAO:      "07FD",
		Name:      "Flying G Ranch STOLport",
		City:      "Lee",
		State:     "Florida",
		Country:   "US",
		Elevation: 90,
		Latitude:  30.4179992676,
		Longitude: -83.2837982178,
		Timezone:  "America/New_York",
	},
	"07GA": {
		ICAO:      "07GA",
		Name:      "Alcovy Airport",
		City:      "Covington",
		State:     "Georgia",
		Country:   "US",
		Elevation: 750,
		Latitude:  33.6749992371,
		Longitude: -83.7630996704,
		Timezone:  "America/New_York",
	},
	"07ID": {
		ICAO:      "07ID",
		Name:      "Interstate Airport",
		City:      "Malta",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4580,
		Latitude:  42.2781982422,
		Longitude: -113.3050003052,
		Timezone:  "America/Boise",
	},
	"07IN": {
		ICAO:      "07IN",
		Name:      "Gray Airport",
		City:      "Boggstown",
		State:     "Indiana",
		Country:   "US",
		Elevation: 750,
		Latitude:  39.5736999512,
		Longitude: -85.9139022827,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"07KS": {
		ICAO:      "07KS",
		Name:      "Heape Field",
		City:      "Augusta",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1321,
		Latitude:  37.7341995239,
		Longitude: -96.9366989136,
		Timezone:  "America/Chicago",
	},
	"07KY": {
		ICAO:      "07KY",
		Name:      "Blue Lick Airport",
		City:      "Louisville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 532,
		Latitude:  38.0806007385,
		Longitude: -85.6932983398,
		Timezone:  "America/Kentucky/Louisville",
	},
	"07MD": {
		ICAO:      "07MD",
		Name:      "Baugher's Orchard Airport",
		City:      "Westminster",
		State:     "Maryland",
		Country:   "US",
		Elevation: 700,
		Latitude:  39.6082992554,
		Longitude: -77.051902771,
		Timezone:  "America/New_York",
	},
	"07MI": {
		ICAO:      "07MI",
		Name:      "Scott Airstrip",
		City:      "Lake",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1065,
		Latitude:  43.864200592,
		Longitude: -85.0239028931,
		Timezone:  "America/Detroit",
	},
	"07MN": {
		ICAO:      "07MN",
		Name:      "Dairyview Airport",
		City:      "Hadley",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1680,
		Latitude:  44.0550003052,
		Longitude: -95.9095001221,
		Timezone:  "America/Chicago",
	},
	"07MO": {
		ICAO:      "07MO",
		Name:      "Lake Sexton Airport",
		City:      "Warrensburg",
		State:     "Missouri",
		Country:   "US",
		Elevation: 880,
		Latitude:  38.6500015259,
		Longitude: -93.616897583,
		Timezone:  "America/Chicago",
	},
	"07MT": {
		ICAO:      "07MT",
		Name:      "Glasgow Industrial Airport",
		City:      "Glasgow",
		State:     "Montana",
		Country:   "US",
		Elevation: 2762,
		Latitude:  48.4211006165,
		Longitude: -106.5279998779,
		Timezone:  "America/Denver",
	},
	"07NC": {
		ICAO:      "07NC",
		Name:      "Hawks Meadow Airport",
		City:      "Waxhaw",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 625,
		Latitude:  34.8331985474,
		Longitude: -80.7511978149,
		Timezone:  "America/New_York",
	},
	"07OK": {
		ICAO:      "07OK",
		Name:      "Pleasant Valley Airport",
		City:      "Snyder",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1322,
		Latitude:  34.6333999634,
		Longitude: -99.0336990356,
		Timezone:  "America/Chicago",
	},
	"07OR": {
		ICAO:      "07OR",
		Name:      "Bruce's Airport",
		City:      "Milwaukie",
		State:     "Oregon",
		Country:   "US",
		Elevation: 250,
		Latitude:  45.4217987061,
		Longitude: -122.6200027466,
		Timezone:  "America/Los_Angeles",
	},
	"07PN": {
		ICAO:      "07PN",
		Name:      "Folmar Airport",
		City:      "Grassflat",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1690,
		Latitude:  40.998889,
		Longitude: -78.125278,
		Timezone:  "America/New_York",
	},
	"07SN": {
		ICAO:      "07SN",
		Name:      "Churchill Airport",
		City:      "Phillipsburg",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1965,
		Latitude:  39.7728004456,
		Longitude: -99.3600997925,
		Timezone:  "America/Chicago",
	},
	"07TA": {
		ICAO:      "07TA",
		Name:      "Salaika Aviation Airport",
		City:      "Danbury",
		State:     "Texas",
		Country:   "US",
		Elevation: 30,
		Latitude:  29.2404994965,
		Longitude: -95.3447036743,
		Timezone:  "America/Chicago",
	},
	"07TE": {
		ICAO:      "07TE",
		Name:      "Cuddihy Field",
		City:      "Corpus Christi",
		State:     "Texas",
		Country:   "US",
		Elevation: 39,
		Latitude:  27.7210998535,
		Longitude: -97.512802124,
		Timezone:  "America/Chicago",
	},
	"07TN": {
		ICAO:      "07TN",
		Name:      "Corntassel Airport",
		City:      "Vonore",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 970,
		Latitude:  35.5685997009,
		Longitude: -84.2410964966,
		Timezone:  "America/New_York",
	},
	"07TS": {
		ICAO:      "07TS",
		Name:      "Cross-Country Estates Airport",
		City:      "Georgetown",
		State:     "Texas",
		Country:   "US",
		Elevation: 690,
		Latitude:  30.625,
		Longitude: -97.5727996826,
		Timezone:  "America/Chicago",
	},
	"07VA": {
		ICAO:      "07VA",
		Name:      "Alpha Hotel Airport",
		City:      "Clover",
		State:     "Virginia",
		Country:   "US",
		Elevation: 350,
		Latitude:  36.8195991516,
		Longitude: -78.7830963135,
		Timezone:  "America/New_York",
	},
	"07VT": {
		ICAO:      "07VT",
		Name:      "Meadow STOLport",
		City:      "Jericho",
		State:     "Vermont",
		Country:   "US",
		Elevation: 801,
		Latitude:  44.4472007751,
		Longitude: -72.9147033691,
		Timezone:  "America/New_York",
	},
	"07WI": {
		ICAO:      "07WI",
		Name:      "Bartell Strip",
		City:      "Genessee Depot",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 990,
		Latitude:  42.9653015137,
		Longitude: -88.3915023804,
		Timezone:  "America/Chicago",
	},
	"08AK": {
		ICAO:      "08AK",
		Name:      "Fisher Airport",
		City:      "Big Lake",
		State:     "Alaska",
		Country:   "US",
		Elevation: 200,
		Latitude:  61.569639,
		Longitude: -149.724439,
		Timezone:  "America/Anchorage",
	},
	"08CL": {
		ICAO:      "08CL",
		Name:      "Kistler Ranch Airport",
		City:      "Jamestown",
		State:     "California",
		Country:   "US",
		Elevation: 1025,
		Latitude:  37.8596000671,
		Longitude: -120.5540008545,
		Timezone:  "America/Los_Angeles",
	},
	"08CO": {
		ICAO:      "08CO",
		Name:      "Terra Firma Airport",
		City:      "Rush",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5600,
		Latitude:  38.7324981689,
		Longitude: -104.0410003662,
		Timezone:  "America/Denver",
	},
	"08FA": {
		ICAO:      "08FA",
		Name:      "Duda Airstrip",
		City:      "La Belle",
		State:     "Florida",
		Country:   "US",
		Elevation: 35,
		Latitude:  26.5797996521,
		Longitude: -81.4837036133,
		Timezone:  "America/New_York",
	},
	"08FD": {
		ICAO:      "08FD",
		Name:      "Sunniland Ranch Airport",
		City:      "Okeechobee",
		State:     "Florida",
		Country:   "US",
		Elevation: 65,
		Latitude:  27.3631000519,
		Longitude: -80.7727966309,
		Timezone:  "America/New_York",
	},
	"08FL": {
		ICAO:      "08FL",
		Name:      "J. H. Hendrie Farms Airport",
		City:      "Lake Placid",
		State:     "Florida",
		Country:   "US",
		Elevation: 103,
		Latitude:  27.0844993591,
		Longitude: -81.3286972046,
		Timezone:  "America/New_York",
	},
	"08GA": {
		ICAO:      "08GA",
		Name:      "Sapelo Island Airport",
		City:      "Darien",
		State:     "Georgia",
		Country:   "US",
		Elevation: 12,
		Latitude:  31.4258003235,
		Longitude: -81.2858963013,
		Timezone:  "America/New_York",
	},
	"08ID": {
		ICAO:      "08ID",
		Name:      "Symms Airport",
		City:      "Marsing",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2680,
		Latitude:  43.5693016052,
		Longitude: -116.7770004272,
		Timezone:  "America/Boise",
	},
	"08IL": {
		ICAO:      "08IL",
		Name:      "Harms Airstrip",
		City:      "Tampico",
		State:     "Illinois",
		Country:   "US",
		Elevation: 643,
		Latitude:  41.5769996643,
		Longitude: -89.7939987183,
		Timezone:  "America/Chicago",
	},
	"08IS": {
		ICAO:      "08IS",
		Name:      "Hemmer RLA Restricted Landing Area",
		City:      "Du Quoin",
		State:     "Illinois",
		Country:   "US",
		Elevation: 400,
		Latitude:  37.9777984619,
		Longitude: -89.2029037476,
		Timezone:  "America/Chicago",
	},
	"08KS": {
		ICAO:      "08KS",
		Name:      "Alderson Airport",
		City:      "Bennington",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1250,
		Latitude:  39.0346984863,
		Longitude: -97.5800018311,
		Timezone:  "America/Chicago",
	},
	"08KY": {
		ICAO:      "08KY",
		Name:      "Boss Airport",
		City:      "Burnside",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 820,
		Latitude:  36.9725990295,
		Longitude: -84.6260986328,
		Timezone:  "America/New_York",
	},
	"08LL": {
		ICAO:      "08LL",
		Name:      "Sauer Airport",
		City:      "Dwight",
		State:     "Illinois",
		Country:   "US",
		Elevation: 665,
		Latitude:  41.0752983093,
		Longitude: -88.4891967773,
		Timezone:  "America/Chicago",
	},
	"08LS": {
		ICAO:      "08LS",
		Name:      "Aeleron Airport",
		City:      "Grand Coteau",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 51,
		Latitude:  30.4036006927,
		Longitude: -92.0235977173,
		Timezone:  "America/Chicago",
	},
	"08MA": {
		ICAO:      "08MA",
		Name:      "Wormid Airport",
		City:      "Southborough",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 360,
		Latitude:  42.2939987183,
		Longitude: -71.4962005615,
		Timezone:  "America/New_York",
	},
	"08ME": {
		ICAO:      "08ME",
		Name:      "Lindbergh Airport",
		City:      "Avon",
		State:     "Maine",
		Country:   "US",
		Elevation: 554,
		Latitude:  44.814201355,
		Longitude: -70.3452987671,
		Timezone:  "America/New_York",
	},
	"08MO": {
		ICAO:      "08MO",
		Name:      "Twin Oaks Airport",
		City:      "Jasper",
		State:     "Missouri",
		Country:   "US",
		Elevation: 955,
		Latitude:  37.3955993652,
		Longitude: -94.2837982178,
		Timezone:  "America/Chicago",
	},
	"08MS": {
		ICAO:      "08MS",
		Name:      "Williams Field",
		City:      "Coldwater",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 345,
		Latitude:  34.66220093,
		Longitude: -89.86830139,
		Timezone:  "America/Chicago",
	},
	"08MT": {
		ICAO:      "08MT",
		Name:      "Clark Airport",
		City:      "Grassrange",
		State:     "Montana",
		Country:   "US",
		Elevation: 3450,
		Latitude:  47.0638999939,
		Longitude: -108.8170013428,
		Timezone:  "America/Denver",
	},
	"08NC": {
		ICAO:      "08NC",
		Name:      "Whiteheart Farm Airport",
		City:      "Lewisville",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 750,
		Latitude:  36.0676002502,
		Longitude: -80.4655990601,
		Timezone:  "America/New_York",
	},
	"08ND": {
		ICAO:      "08ND",
		Name:      "Brecht Strip",
		City:      "Golden Valley",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1990,
		Latitude:  47.3231010437,
		Longitude: -102.0979995728,
		Timezone:  "America/North_Dakota/Beulah",
	},
	"08NE": {
		ICAO:      "08NE",
		Name:      "Laurel Municipal Airport",
		City:      "Laurel",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1467,
		Latitude:  42.4317016602,
		Longitude: -97.079498291,
		Timezone:  "America/Chicago",
	},
	"08NY": {
		ICAO:      "08NY",
		Name:      "Md1 Airport",
		City:      "Middletown",
		State:     "New York",
		Country:   "US",
		Elevation: 650,
		Latitude:  41.3680992126,
		Longitude: -74.5055999756,
		Timezone:  "America/New_York",
	},
	"08OH": {
		ICAO:      "08OH",
		Name:      "Kenley Airport",
		City:      "Hartford",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1230,
		Latitude:  41.2916984558,
		Longitude: -80.5636978149,
		Timezone:  "America/New_York",
	},
	"08OK": {
		ICAO:      "08OK",
		Name:      "Parks Airport",
		City:      "Wilburton",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 635,
		Latitude:  34.9314994812,
		Longitude: -95.2583007813,
		Timezone:  "America/Chicago",
	},
	"08OR": {
		ICAO:      "08OR",
		Name:      "Saxon Sycan Airport",
		City:      "Silver Lake",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4990,
		Latitude:  42.8392982483,
		Longitude: -121.1169967651,
		Timezone:  "America/Los_Angeles",
	},
	"08TE": {
		ICAO:      "08TE",
		Name:      "B. J. Mc Combs Sisterdale Airport",
		City:      "Comfort",
		State:     "Texas",
		Country:   "US",
		Elevation: 1447,
		Latitude:  29.9759998322,
		Longitude: -98.743598938,
		Timezone:  "America/Chicago",
	},
	"08TS": {
		ICAO:      "08TS",
		Name:      "Puesta Del Sol Airport",
		City:      "La Gloria",
		State:     "Texas",
		Country:   "US",
		Elevation: 420,
		Latitude:  26.8878002167,
		Longitude: -98.5072021484,
		Timezone:  "America/Chicago",
	},
	"08TX": {
		ICAO:      "08TX",
		Name:      "Cross Wind Airport",
		City:      "Weatherford",
		State:     "Texas",
		Country:   "US",
		Elevation: 1040,
		Latitude:  32.8582000732,
		Longitude: -97.7474975586,
		Timezone:  "America/Chicago",
	},
	"08VA": {
		ICAO:      "08VA",
		Name:      "Springwood Airstrip",
		City:      "Buchanan",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1000,
		Latitude:  37.5540008545,
		Longitude: -79.7498016357,
		Timezone:  "America/New_York",
	},
	"08XS": {
		ICAO:      "08XS",
		Name:      "Peterson Airport",
		City:      "Old Ocean",
		State:     "Texas",
		Country:   "US",
		Elevation: 45,
		Latitude:  29.1081008911,
		Longitude: -95.7774963379,
		Timezone:  "America/Chicago",
	},
	"09AK": {
		ICAO:      "09AK",
		Name:      "West Beaver Airport",
		City:      "Big Lake",
		State:     "Alaska",
		Country:   "US",
		Elevation: 228,
		Latitude:  61.589361,
		Longitude: -149.847333,
		Timezone:  "America/Anchorage",
	},
	"09AZ": {
		ICAO:      "09AZ",
		Name:      "Stronghold Airport",
		City:      "St David",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4970,
		Latitude:  31.9253005981,
		Longitude: -110.0380020142,
		Timezone:  "America/Phoenix",
	},
	"09CL": {
		ICAO:      "09CL",
		Name:      "Alta Sierra Airport",
		City:      "Grass Valley",
		State:     "California",
		Country:   "US",
		Elevation: 2275,
		Latitude:  39.1138000488,
		Longitude: -121.0569992065,
		Timezone:  "America/Los_Angeles",
	},
	"09CO": {
		ICAO:      "09CO",
		Name:      "Cottonwood Field",
		City:      "Swink",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4180,
		Latitude:  38.0555992126,
		Longitude: -103.6529998779,
		Timezone:  "America/Denver",
	},
	"09FA": {
		ICAO:      "09FA",
		Name:      "Placid Lakes Airport",
		City:      "Lake Placid",
		State:     "Florida",
		Country:   "US",
		Elevation: 130,
		Latitude:  27.2455997467,
		Longitude: -81.4131011963,
		Timezone:  "America/New_York",
	},
	"09FD": {
		ICAO:      "09FD",
		Name:      "Cheryl-Lane Landings Airport",
		City:      "Bushnell",
		State:     "Florida",
		Country:   "US",
		Elevation: 71,
		Latitude:  28.6711006165,
		Longitude: -82.0873031616,
		Timezone:  "America/New_York",
	},
	"09FL": {
		ICAO:      "09FL",
		Name:      "Sunnybreeze Airport",
		City:      "Arcadia",
		State:     "Florida",
		Country:   "US",
		Elevation: 60,
		Latitude:  27.0555992126,
		Longitude: -81.9705963135,
		Timezone:  "America/New_York",
	},
	"09GA": {
		ICAO:      "09GA",
		Name:      "Sunbelt Strip",
		City:      "Moultrie",
		State:     "Georgia",
		Country:   "US",
		Elevation: 230,
		Latitude:  31.1112995148,
		Longitude: -83.68409729,
		Timezone:  "America/New_York",
	},
	"09IA": {
		ICAO:      "09IA",
		Name:      "Skydive Iowa Airport",
		City:      "Brooklyn",
		State:     "Iowa",
		Country:   "US",
		Elevation: 820,
		Latitude:  41.7458000183,
		Longitude: -92.4093017578,
		Timezone:  "America/Chicago",
	},
	"09ID": {
		ICAO:      "09ID",
		Name:      "Taylor Ranch Landing Area Airport",
		City:      "Moscow",
		State:     "Idaho",
		Country:   "US",
		Elevation: 3835,
		Latitude:  45.1039009094,
		Longitude: -114.8550033569,
		Timezone:  "America/Boise",
	},
	"09II": {
		ICAO:      "09II",
		Name:      "Gibbons Field",
		City:      "Valeene",
		State:     "Indiana",
		Country:   "US",
		Elevation: 880,
		Latitude:  38.4472999573,
		Longitude: -86.4449996948,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"09IN": {
		ICAO:      "09IN",
		Name:      "Gordon Airport",
		City:      "Columbia City",
		State:     "Indiana",
		Country:   "US",
		Elevation: 850,
		Latitude:  41.1208992004,
		Longitude: -85.4250030518,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"09KS": {
		ICAO:      "09KS",
		Name:      "Coffman Airport",
		City:      "Wellsville",
		State:     "Kansas",
		Country:   "US",
		Elevation: 970,
		Latitude:  38.7070007324,
		Longitude: -95.1382980347,
		Timezone:  "America/Chicago",
	},
	"09LL": {
		ICAO:      "09LL",
		Name:      "Pine Hill Airport",
		City:      "Capron",
		State:     "Illinois",
		Country:   "US",
		Elevation: 1000,
		Latitude:  42.4681015015,
		Longitude: -88.7432022095,
		Timezone:  "America/Chicago",
	},
	"09MA": {
		ICAO:      "09MA",
		Name:      "Kallander Field",
		City:      "Southborough",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 403,
		Latitude:  42.3042984009,
		Longitude: -71.505897522,
		Timezone:  "America/New_York",
	},
	"09ME": {
		ICAO:      "09ME",
		Name:      "Perrotti Skyranch Airfield",
		City:      "Berwick",
		State:     "Maine",
		Country:   "US",
		Elevation: 260,
		Latitude:  43.340801239,
		Longitude: -70.8546981812,
		Timezone:  "America/New_York",
	},
	"09MN": {
		ICAO:      "09MN",
		Name:      "W Johnson Field",
		City:      "Hazel Run",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1060,
		Latitude:  44.7799987793,
		Longitude: -95.6635971069,
		Timezone:  "America/Chicago",
	},
	"09MO": {
		ICAO:      "09MO",
		Name:      "Hogue Farm Airport",
		City:      "Willard",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1210,
		Latitude:  37.3783988953,
		Longitude: -93.3632965088,
		Timezone:  "America/Chicago",
	},
	"09NC": {
		ICAO:      "09NC",
		Name:      "William Irving Lewis Airport",
		City:      "Broadway",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 310,
		Latitude:  35.3292999268,
		Longitude: -79.0299987793,
		Timezone:  "America/New_York",
	},
	"09NE": {
		ICAO:      "09NE",
		Name:      "Summer Hill Farm Airport",
		City:      "Bennington",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1250,
		Latitude:  41.3333015442,
		Longitude: -96.1936035156,
		Timezone:  "America/Chicago",
	},
	"09OK": {
		ICAO:      "09OK",
		Name:      "Ragtime Aerodrome",
		City:      "Collinsville",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 700,
		Latitude:  36.3544998169,
		Longitude: -95.863899231,
		Timezone:  "America/Chicago",
	},
	"09PA": {
		ICAO:      "09PA",
		Name:      "Eagle Field",
		City:      "Port Matilda",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1780,
		Latitude:  40.8269996643,
		Longitude: -77.9880981445,
		Timezone:  "America/New_York",
	},
	"09TA": {
		ICAO:      "09TA",
		Name:      "Lazy G Bar Ranch Airport",
		City:      "Decatur",
		State:     "Texas",
		Country:   "US",
		Elevation: 923,
		Latitude:  33.2821006775,
		Longitude: -97.4970016479,
		Timezone:  "America/Chicago",
	},
	"09TE": {
		ICAO:      "09TE",
		Name:      "Running M Ranch Airport",
		City:      "Glen Rose",
		State:     "Texas",
		Country:   "US",
		Elevation: 787,
		Latitude:  32.2667999268,
		Longitude: -97.8669967651,
		Timezone:  "America/Chicago",
	},
	"09TS": {
		ICAO:      "09TS",
		Name:      "Pitcock Rosillos Mountain Ranch Airport",
		City:      "Marathon",
		State:     "Texas",
		Country:   "US",
		Elevation: 3100,
		Latitude:  29.4824008942,
		Longitude: -103.1650009155,
		Timezone:  "America/Chicago",
	},
	"09TX": {
		ICAO:      "09TX",
		Name:      "Smokey Mtn Ranch Airport",
		City:      "Sheffield",
		State:     "Texas",
		Country:   "US",
		Elevation: 2522,
		Latitude:  30.7390995026,
		Longitude: -102.0240020752,
		Timezone:  "America/Chicago",
	},
	"09VA": {
		ICAO:      "09VA",
		Name:      "Alum Ridge STOLport",
		City:      "Floyd",
		State:     "Virginia",
		Country:   "US",
		Elevation: 2375,
		Latitude:  36.9765014648,
		Longitude: -80.4938964844,
		Timezone:  "America/New_York",
	},
	"09WI": {
		ICAO:      "09WI",
		Name:      "Parkway Farm Strip",
		City:      "Holmen",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 705,
		Latitude:  44.0036010742,
		Longitude: -91.3076019287,
		Timezone:  "America/Chicago",
	},
	"0AK0": {
		ICAO:      "0AK0",
		Name:      "Scotts Airport",
		City:      "North Pole",
		State:     "Alaska",
		Country:   "US",
		Elevation: 800,
		Latitude:  64.3925018311,
		Longitude: -146.8619995117,
		Timezone:  "America/Anchorage",
	},
	"0AK1": {
		ICAO:      "0AK1",
		Name:      "Anderson Lake Airport",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 463,
		Latitude:  61.6169013977,
		Longitude: -149.3220062256,
		Timezone:  "America/Anchorage",
	},
	"0AK2": {
		ICAO:      "0AK2",
		Name:      "Yentna Bend Strip",
		City:      "Willow",
		State:     "Alaska",
		Country:   "US",
		Elevation: 81,
		Latitude:  61.725556,
		Longitude: -150.678611,
		Timezone:  "America/Anchorage",
	},
	"0AK3": {
		ICAO:      "0AK3",
		Name:      "Parker Lake Airport",
		City:      "Montana Creek",
		State:     "Alaska",
		Country:   "US",
		Elevation: 250,
		Latitude:  62.035900116,
		Longitude: -150.4940032959,
		Timezone:  "America/Anchorage",
	},
	"0AK5": {
		ICAO:      "0AK5",
		Name:      "Young Creek Airport",
		City:      "May Creek",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1580,
		Latitude:  61.3564987183,
		Longitude: -142.733001709,
		Timezone:  "America/Anchorage",
	},
	"0AK6": {
		ICAO:      "0AK6",
		Name:      "Victory Airport",
		City:      "Victory Bible Camp",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1800,
		Latitude:  61.8033981323,
		Longitude: -147.9440002441,
		Timezone:  "America/Anchorage",
	},
	"0AK7": {
		ICAO:      "0AK7",
		Name:      "Bradley Lake Hydroelectri Project Airstrip",
		City:      "Homer",
		State:     "Alaska",
		Country:   "US",
		Elevation: 18,
		Latitude:  59.7761001587,
		Longitude: -150.9589996338,
		Timezone:  "America/Anchorage",
	},
	"0AK9": {
		ICAO:      "0AK9",
		Name:      "Falcon Lake Strip",
		City:      "Point Mackenzie",
		State:     "Alaska",
		Country:   "US",
		Elevation: 110,
		Latitude:  61.3325996399,
		Longitude: -150.0590057373,
		Timezone:  "America/Anchorage",
	},
	"0AL1": {
		ICAO:      "0AL1",
		Name:      "Resort Airport",
		City:      "Foley",
		State:     "Alabama",
		Country:   "US",
		Elevation: 97,
		Latitude:  30.4438991547,
		Longitude: -87.6568984985,
		Timezone:  "America/Chicago",
	},
	"0AL5": {
		ICAO:      "0AL5",
		Name:      "Flomaton Airport",
		City:      "Flomaton",
		State:     "Alabama",
		Country:   "US",
		Elevation: 247,
		Latitude:  31.0321998596,
		Longitude: -87.2527999878,
		Timezone:  "America/Chicago",
	},
	"0AL9": {
		ICAO:      "0AL9",
		Name:      "Wilson Creek Airport",
		City:      "Florence",
		State:     "Alabama",
		Country:   "US",
		Elevation: 590,
		Latitude:  34.8499984741,
		Longitude: -87.6324996948,
		Timezone:  "America/Chicago",
	},
	"0AR1": {
		ICAO:      "0AR1",
		Name:      "Hooterville South Airport",
		City:      "Centerville",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 370,
		Latitude:  35.0946998596,
		Longitude: -93.202003479,
		Timezone:  "America/Chicago",
	},
	"0AR2": {
		ICAO:      "0AR2",
		Name:      "Mission Field-Marotti Memorial Airport",
		City:      "Crawfordsville",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 217,
		Latitude:  35.2667999268,
		Longitude: -90.3500976563,
		Timezone:  "America/Chicago",
	},
	"0AR3": {
		ICAO:      "0AR3",
		Name:      "Taylor's Air Strip",
		City:      "Deer",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 2160,
		Latitude:  35.8989982605,
		Longitude: -93.2610015869,
		Timezone:  "America/Chicago",
	},
	"0AR7": {
		ICAO:      "0AR7",
		Name:      "Yancopin Airport",
		City:      "Watson",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 150,
		Latitude:  33.9500999451,
		Longitude: -91.1811981201,
		Timezone:  "America/Chicago",
	},
	"0AZ1": {
		ICAO:      "0AZ1",
		Name:      "Taylor Field",
		City:      "Marana",
		State:     "Arizona",
		Country:   "US",
		Elevation: 2130,
		Latitude:  32.3008995056,
		Longitude: -111.3239974976,
		Timezone:  "America/Phoenix",
	},
	"0AZ2": {
		ICAO:      "0AZ2",
		Name:      "Western Sky Airport",
		City:      "Salome",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1930,
		Latitude:  33.7780990601,
		Longitude: -113.641998291,
		Timezone:  "America/Phoenix",
	},
	"0AZ4": {
		ICAO:      "0AZ4",
		Name:      "Flying Bucket Ranch Airport",
		City:      "Mobile",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1505,
		Latitude:  32.9822998047,
		Longitude: -112.2870025635,
		Timezone:  "America/Phoenix",
	},
	"0AZ5": {
		ICAO:      "0AZ5",
		Name:      "Castle Well Airport",
		City:      "Morristown",
		State:     "Arizona",
		Country:   "US",
		Elevation: 2080,
		Latitude:  33.8658981323,
		Longitude: -112.6029968262,
		Timezone:  "America/Phoenix",
	},
	"0CA1": {
		ICAO:      "0CA1",
		Name:      "Northside Airpark",
		City:      "Santa Maria",
		State:     "California",
		Country:   "US",
		Elevation: 190,
		Latitude:  34.9883003235,
		Longitude: -120.4580001831,
		Timezone:  "America/Los_Angeles",
	},
	"0CA3": {
		ICAO:      "0CA3",
		Name:      "Crawford Airport",
		City:      "Santa Ynez",
		State:     "California",
		Country:   "US",
		Elevation: 570,
		Latitude:  34.5875015259,
		Longitude: -120.0130004883,
		Timezone:  "America/Los_Angeles",
	},
	"0CA4": {
		ICAO:      "0CA4",
		Name:      "Shepherd Ranch Airport",
		City:      "Santa Ynez",
		State:     "California",
		Country:   "US",
		Elevation: 660,
		Latitude:  34.6222000122,
		Longitude: -120.0650024414,
		Timezone:  "America/Los_Angeles",
	},
	"0CA5": {
		ICAO:      "0CA5",
		Name:      "Hoffman Private Airport",
		City:      "Santa Ysabel",
		State:     "California",
		Country:   "US",
		Elevation: 3100,
		Latitude:  33.1431007385,
		Longitude: -116.7450027466,
		Timezone:  "America/Los_Angeles",
	},
	"0CA6": {
		ICAO:      "0CA6",
		Name:      "Emory Ranch Airport",
		City:      "Ocotillo",
		State:     "California",
		Country:   "US",
		Elevation: 480,
		Latitude:  32.7494444,
		Longitude: -116.0161111,
		Timezone:  "America/Los_Angeles",
	},
	"0CA8": {
		ICAO:      "0CA8",
		Name:      "Ward Ranch Airport",
		City:      "Aguanga",
		State:     "California",
		Country:   "US",
		Elevation: 2500,
		Latitude:  33.4277992249,
		Longitude: -116.843002319,
		Timezone:  "America/Los_Angeles",
	},
	"0CA9": {
		ICAO:      "0CA9",
		Name:      "Blech Ranch Airport",
		City:      "Shandon",
		State:     "California",
		Country:   "US",
		Elevation: 1140,
		Latitude:  35.5950012207,
		Longitude: -120.338996887,
		Timezone:  "America/Los_Angeles",
	},
	"0CD2": {
		ICAO:      "0CD2",
		Name:      "Foxx Valley Airport",
		City:      "Yoder",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5800,
		Latitude:  38.6525001526,
		Longitude: -104.23400116,
		Timezone:  "America/Denver",
	},
	"0CD5": {
		ICAO:      "0CD5",
		Name:      "Pinon Canyon Airport",
		City:      "Trinidad",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5686,
		Latitude:  37.4905014038,
		Longitude: -104.143997192,
		Timezone:  "America/Denver",
	},
	"0CD7": {
		ICAO:      "0CD7",
		Name:      "Fox Hole Airport",
		City:      "Lafayette",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5135,
		Latitude:  40.004699707,
		Longitude: -105.071998596,
		Timezone:  "America/Denver",
	},
	"0CL0": {
		ICAO:      "0CL0",
		Name:      "Yosemite Hidden Lake Ranch Airport",
		City:      "Raymond",
		State:     "California",
		Country:   "US",
		Elevation: 500,
		Latitude:  37.1180000305,
		Longitude: -119.913002014,
		Timezone:  "America/Los_Angeles",
	},
	"0CL1": {
		ICAO:      "0CL1",
		Name:      "Krey Field Airport",
		City:      "Adelanto",
		State:     "California",
		Country:   "US",
		Elevation: 3042,
		Latitude:  34.5694007874,
		Longitude: -117.555999756,
		Timezone:  "America/Los_Angeles",
	},
	"0CL3": {
		ICAO:      "0CL3",
		Name:      "John Nichol's Field Airport",
		City:      "Chula Vista",
		State:     "California",
		Country:   "US",
		Elevation: 490,
		Latitude:  32.6330986023,
		Longitude: -116.892997742,
		Timezone:  "America/Los_Angeles",
	},
	"0CL6": {
		ICAO:      "0CL6",
		Name:      "Bohunk's Airpark",
		City:      "Lancaster",
		State:     "California",
		Country:   "US",
		Elevation: 2410,
		Latitude:  34.6946983337,
		Longitude: -118.282997131,
		Timezone:  "America/Los_Angeles",
	},
	"0CL8": {
		ICAO:      "0CL8",
		Name:      "Tera Ultralightport",
		City:      "Ridgecrest",
		State:     "California",
		Country:   "US",
		Elevation: 2510,
		Latitude:  35.5913009644,
		Longitude: -117.634002686,
		Timezone:  "America/Los_Angeles",
	},
	"0CO1": {
		ICAO:      "0CO1",
		Name:      "Dave's Airport",
		City:      "Louisville",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5170,
		Latitude:  40.0332984924,
		Longitude: -105.124000549,
		Timezone:  "America/Denver",
	},
	"0CO2": {
		ICAO:      "0CO2",
		Name:      "Buckhorn Ranch Airport",
		City:      "Crested Butte",
		State:     "Colorado",
		Country:   "US",
		Elevation: 8980,
		Latitude:  38.8518981934,
		Longitude: -106.932998657,
		Timezone:  "America/Denver",
	},
	"0CO3": {
		ICAO:      "0CO3",
		Name:      "Greggs Nr 1 Airport",
		City:      "Bennett",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5373,
		Latitude:  39.8894004822,
		Longitude: -104.544998169,
		Timezone:  "America/Denver",
	},
	"0CO5": {
		ICAO:      "0CO5",
		Name:      "Chenoweth Airport",
		City:      "Last Chance",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4697,
		Latitude:  39.8333015442,
		Longitude: -103.589996338,
		Timezone:  "America/Denver",
	},
	"0CO6": {
		ICAO:      "0CO6",
		Name:      "Flying M & M Ranch Airport",
		City:      "Norwood",
		State:     "Colorado",
		Country:   "US",
		Elevation: 8000,
		Latitude:  38.2193984985,
		Longitude: -108.212997437,
		Timezone:  "America/Denver",
	},
	"0CO8": {
		ICAO:      "0CO8",
		Name:      "Cartwheel Airport",
		City:      "Mead",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5010,
		Latitude:  40.2083015442,
		Longitude: -105.013000488,
		Timezone:  "America/Denver",
	},
	"0CO9": {
		ICAO:      "0CO9",
		Name:      "Van Treese Airport",
		City:      "Monte Vista",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7613,
		Latitude:  37.6582984924,
		Longitude: -106.033996582,
		Timezone:  "America/Denver",
	},
	"0FA1": {
		ICAO:      "0FA1",
		Name:      "Frierson Grove Airport",
		City:      "Arcadia",
		State:     "Florida",
		Country:   "US",
		Elevation: 65,
		Latitude:  27.303899765,
		Longitude: -81.8804016113,
		Timezone:  "America/New_York",
	},
	"0FD0": {
		ICAO:      "0FD0",
		Name:      "Big 'G' Airport",
		City:      "Ormond Beach",
		State:     "Florida",
		Country:   "US",
		Elevation: 40,
		Latitude:  29.2674999237,
		Longitude: -81.2301025391,
		Timezone:  "America/New_York",
	},
	"0FD3": {
		ICAO:      "0FD3",
		Name:      "Dugger Field",
		City:      "Freeport",
		State:     "Florida",
		Country:   "US",
		Elevation: 70,
		Latitude:  30.4964008331,
		Longitude: -86.0938034058,
		Timezone:  "America/Chicago",
	},
	"0FD5": {
		ICAO:      "0FD5",
		Name:      "Breezy Knoll Airport",
		City:      "Defuniak Springs",
		State:     "Florida",
		Country:   "US",
		Elevation: 235,
		Latitude:  30.8201999664,
		Longitude: -86.3443984985,
		Timezone:  "America/Chicago",
	},
	"0FD9": {
		ICAO:      "0FD9",
		Name:      "Joy Farms Airport",
		City:      "Defuniak Springs",
		State:     "Florida",
		Country:   "US",
		Elevation: 290,
		Latitude:  30.6520996094,
		Longitude: -86.1093978882,
		Timezone:  "America/Chicago",
	},
	"0FL0": {
		ICAO:      "0FL0",
		Name:      "Harper's Fly-In Ranch Airport",
		City:      "Clewiston",
		State:     "Florida",
		Country:   "US",
		Elevation: 30,
		Latitude:  26.7306003571,
		Longitude: -81.1639022827,
		Timezone:  "America/New_York",
	},
	"0FL1": {
		ICAO:      "0FL1",
		Name:      "Thompson Airfield",
		City:      "Branford",
		State:     "Florida",
		Country:   "US",
		Elevation: 110,
		Latitude:  29.9832992554,
		Longitude: -82.8356018066,
		Timezone:  "America/New_York",
	},
	"0FL6": {
		ICAO:      "0FL6",
		Name:      "Stanchester Airport",
		City:      "Lithia",
		State:     "Florida",
		Country:   "US",
		Elevation: 94,
		Latitude:  27.8059005737,
		Longitude: -82.1400985718,
		Timezone:  "America/New_York",
	},
	"0FL9": {
		ICAO:      "0FL9",
		Name:      "Mc Daniel Ranch Inc. Airport",
		City:      "Clewiston",
		State:     "Florida",
		Country:   "US",
		Elevation: 18,
		Latitude:  26.3938999176,
		Longitude: -81.0231018066,
		Timezone:  "America/New_York",
	},
	"0GA0": {
		ICAO:      "0GA0",
		Name:      "Halls Flying Ranch Airport",
		City:      "Hampton",
		State:     "Georgia",
		Country:   "US",
		Elevation: 800,
		Latitude:  33.3562011719,
		Longitude: -84.3671035767,
		Timezone:  "America/New_York",
	},
	"0GA1": {
		ICAO:      "0GA1",
		Name:      "Mustang Field",
		City:      "Hartwell",
		State:     "Georgia",
		Country:   "US",
		Elevation: 693,
		Latitude:  34.4779014587,
		Longitude: -82.897102356,
		Timezone:  "America/New_York",
	},
	"0GA2": {
		ICAO:      "0GA2",
		Name:      "Airnautique Inc. Airport",
		City:      "Hartwell",
		State:     "Georgia",
		Country:   "US",
		Elevation: 720,
		Latitude:  34.3778991699,
		Longitude: -82.9457015991,
		Timezone:  "America/New_York",
	},
	"0GA3": {
		ICAO:      "0GA3",
		Name:      "Ayresouth Airport",
		City:      "Temple",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1287,
		Latitude:  33.7700996399,
		Longitude: -85.060798645,
		Timezone:  "America/New_York",
	},
	"0GA4": {
		ICAO:      "0GA4",
		Name:      "Paradise Falls Airport",
		City:      "High Shoals",
		State:     "Georgia",
		Country:   "US",
		Elevation: 736,
		Latitude:  33.8129005432,
		Longitude: -83.5056991577,
		Timezone:  "America/New_York",
	},
	"0GA6": {
		ICAO:      "0GA6",
		Name:      "Sunset Strip",
		City:      "Jersey",
		State:     "Georgia",
		Country:   "US",
		Elevation: 780,
		Latitude:  33.7333984375,
		Longitude: -83.8188018799,
		Timezone:  "America/New_York",
	},
	"0GA7": {
		ICAO:      "0GA7",
		Name:      "Hickory Level Airfield",
		City:      "Temple",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1080,
		Latitude:  33.6833992004,
		Longitude: -84.9999008179,
		Timezone:  "America/New_York",
	},
	"0GA8": {
		ICAO:      "0GA8",
		Name:      "Paso Fino Farm Airport",
		City:      "Valdosta",
		State:     "Georgia",
		Country:   "US",
		Elevation: 200,
		Latitude:  30.8871002197,
		Longitude: -83.4160003662,
		Timezone:  "America/New_York",
	},
	"0GA9": {
		ICAO:      "0GA9",
		Name:      "Roberta Industrial Park Airport",
		City:      "Roberta",
		State:     "Georgia",
		Country:   "US",
		Elevation: 505,
		Latitude:  32.7139015198,
		Longitude: -84.0274963379,
		Timezone:  "America/New_York",
	},
	"0GE0": {
		ICAO:      "0GE0",
		Name:      "Pinewood Airport",
		City:      "Douglasville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1085,
		Latitude:  33.7821006775,
		Longitude: -84.8007965088,
		Timezone:  "America/New_York",
	},
	"0GE1": {
		ICAO:      "0GE1",
		Name:      "Crystal Lake Airpark",
		City:      "Irwinville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 327,
		Latitude:  31.6655006409,
		Longitude: -83.4470977783,
		Timezone:  "America/New_York",
	},
	"0GE3": {
		ICAO:      "0GE3",
		Name:      "Lookout Mountain Airport",
		City:      "Trenton",
		State:     "Georgia",
		Country:   "US",
		Elevation: 563,
		Latitude:  34.9047012329,
		Longitude: -85.4597015381,
		Timezone:  "America/New_York",
	},
	"0GE4": {
		ICAO:      "0GE4",
		Name:      "Pecan Patch Airstrip",
		City:      "Sunnyside",
		State:     "Georgia",
		Country:   "US",
		Elevation: 860,
		Latitude:  33.331401825,
		Longitude: -84.2667007446,
		Timezone:  "America/New_York",
	},
	"0GE5": {
		ICAO:      "0GE5",
		Name:      "Mountain Airpark",
		City:      "Cleveland",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1438,
		Latitude:  34.5617980957,
		Longitude: -83.7135009766,
		Timezone:  "America/New_York",
	},
	"0GE7": {
		ICAO:      "0GE7",
		Name:      "Carpenter Airport",
		City:      "Appling",
		State:     "Georgia",
		Country:   "US",
		Elevation: 450,
		Latitude:  33.5203018188,
		Longitude: -82.3724975586,
		Timezone:  "America/New_York",
	},
	"0GE9": {
		ICAO:      "0GE9",
		Name:      "Pegasus Ranch Airport",
		City:      "Rocky Ford",
		State:     "Georgia",
		Country:   "US",
		Elevation: 230,
		Latitude:  32.693901062,
		Longitude: -81.7985992432,
		Timezone:  "America/New_York",
	},
	"0IA3": {
		ICAO:      "0IA3",
		Name:      "Hawkeye Airport",
		City:      "Waterloo",
		State:     "Iowa",
		Country:   "US",
		Elevation: 883,
		Latitude:  42.4347000122,
		Longitude: -92.3280029297,
		Timezone:  "America/Chicago",
	},
	"0IA4": {
		ICAO:      "0IA4",
		Name:      "Sharar Field",
		City:      "Mediapolis",
		State:     "Iowa",
		Country:   "US",
		Elevation: 700,
		Latitude:  41.0334014893,
		Longitude: -91.0501022339,
		Timezone:  "America/Chicago",
	},
	"0IA5": {
		ICAO:      "0IA5",
		Name:      "Moore Private Airport",
		City:      "Millerton",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1060,
		Latitude:  40.8722000122,
		Longitude: -93.2627029419,
		Timezone:  "America/Chicago",
	},
	"0IA6": {
		ICAO:      "0IA6",
		Name:      "Rinehart Airport",
		City:      "Victor",
		State:     "Iowa",
		Country:   "US",
		Elevation: 920,
		Latitude:  41.6596984863,
		Longitude: -92.2899017334,
		Timezone:  "America/Chicago",
	},
	"0IA8": {
		ICAO:      "0IA8",
		Name:      "Hannen Airport",
		City:      "Center Point",
		State:     "Iowa",
		Country:   "US",
		Elevation: 925,
		Latitude:  42.2280006409,
		Longitude: -91.7509994507,
		Timezone:  "America/Chicago",
	},
	"0ID0": {
		ICAO:      "0ID0",
		Name:      "South Fork Ranch Airport",
		City:      "Featherville",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4861,
		Latitude:  43.6063995361,
		Longitude: -115.1070022583,
		Timezone:  "America/Boise",
	},
	"0ID1": {
		ICAO:      "0ID1",
		Name:      "Ziggy's Airport",
		City:      "Filer",
		State:     "Idaho",
		Country:   "US",
		Elevation: 3225,
		Latitude:  42.6412010193,
		Longitude: -114.6050033569,
		Timezone:  "America/Boise",
	},
	"0ID2": {
		ICAO:      "0ID2",
		Name:      "Flying Joseph Ranch Airport",
		City:      "May",
		State:     "Idaho",
		Country:   "US",
		Elevation: 5600,
		Latitude:  44.4429016113,
		Longitude: -113.773002625,
		Timezone:  "America/Boise",
	},
	"0ID3": {
		ICAO:      "0ID3",
		Name:      "Coyote Run Airport",
		City:      "Mountain Home",
		State:     "Idaho",
		Country:   "US",
		Elevation: 3150,
		Latitude:  43.0758018494,
		Longitude: -115.672996521,
		Timezone:  "America/Boise",
	},
	"0ID4": {
		ICAO:      "0ID4",
		Name:      "Black Butte Ranch Airport",
		City:      "Shoshone",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4020,
		Latitude:  43.0476989746,
		Longitude: -114.4700012207,
		Timezone:  "America/Boise",
	},
	"0ID5": {
		ICAO:      "0ID5",
		Name:      "Ez Lope Ranch Airport",
		City:      "Oreana",
		State:     "Idaho",
		Country:   "US",
		Elevation: 3130,
		Latitude:  43.0531005859,
		Longitude: -116.4619979858,
		Timezone:  "America/Boise",
	},
	"0ID7": {
		ICAO:      "0ID7",
		Name:      "Z X Ranch Airport",
		City:      "Murphy",
		State:     "Idaho",
		Country:   "US",
		Elevation: 3820,
		Latitude:  43.230308,
		Longitude: -116.763192,
		Timezone:  "America/Boise",
	},
	"0ID8": {
		ICAO:      "0ID8",
		Name:      "Pinnacle Airport",
		City:      "Grangeville",
		State:     "Idaho",
		Country:   "US",
		Elevation: 3800,
		Latitude:  45.775197,
		Longitude: -116.178475,
		Timezone:  "America/Los_Angeles",
	},
	"0II0": {
		ICAO:      "0II0",
		Name:      "Chuck's Airport",
		City:      "Albany",
		State:     "Indiana",
		Country:   "US",
		Elevation: 955,
		Latitude:  40.2599983215,
		Longitude: -85.2533035278,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"0II1": {
		ICAO:      "0II1",
		Name:      "Unsicker Airport",
		City:      "Bremen",
		State:     "Indiana",
		Country:   "US",
		Elevation: 816,
		Latitude:  40.4281005859,
		Longitude: -86.1532974243,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"0II2": {
		ICAO:      "0II2",
		Name:      "Creighton Airport",
		City:      "Bremen",
		State:     "Indiana",
		Country:   "US",
		Elevation: 820,
		Latitude:  41.4572982788,
		Longitude: -86.1243972778,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"0II3": {
		ICAO:      "0II3",
		Name:      "Miller Strip",
		City:      "Bunker Hill",
		State:     "Indiana",
		Country:   "US",
		Elevation: 810,
		Latitude:  40.6277999878,
		Longitude: -86.0378036499,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"0II5": {
		ICAO:      "0II5",
		Name:      "Marshall Field",
		City:      "Chalmers",
		State:     "Indiana",
		Country:   "US",
		Elevation: 705,
		Latitude:  40.6595001221,
		Longitude: -86.8572006226,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"0II6": {
		ICAO:      "0II6",
		Name:      "Kropf Airport",
		City:      "Goshen",
		State:     "Indiana",
		Country:   "US",
		Elevation: 810,
		Latitude:  41.6431007385,
		Longitude: -85.8296966553,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"0II7": {
		ICAO:      "0II7",
		Name:      "Frost Field",
		City:      "Greenfield",
		State:     "Indiana",
		Country:   "US",
		Elevation: 900,
		Latitude:  39.7956008911,
		Longitude: -85.7861022949,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"0II8": {
		ICAO:      "0II8",
		Name:      "Sutton's Field",
		City:      "Lowell",
		State:     "Indiana",
		Country:   "US",
		Elevation: 740,
		Latitude:  41.324798584,
		Longitude: -87.4758987427,
		Timezone:  "America/Chicago",
	},
	"0II9": {
		ICAO:      "0II9",
		Name:      "Winters Airport",
		City:      "Hall",
		State:     "Indiana",
		Country:   "US",
		Elevation: 755,
		Latitude:  39.534198761,
		Longitude: -86.5772018433,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"0IL2": {
		ICAO:      "0IL2",
		Name:      "Griffin Airport",
		City:      "Alma",
		State:     "Illinois",
		Country:   "US",
		Elevation: 541,
		Latitude:  38.803056,
		Longitude: -88.997778,
		Timezone:  "America/Chicago",
	},
	"0IL3": {
		ICAO:      "0IL3",
		Name:      "Flying B Ranch Airport",
		City:      "Danville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 696,
		Latitude:  40.1570014954,
		Longitude: -87.6744995117,
		Timezone:  "America/Chicago",
	},
	"0IL8": {
		ICAO:      "0IL8",
		Name:      "Walter Airport",
		City:      "De Kalb",
		State:     "Illinois",
		Country:   "US",
		Elevation: 890,
		Latitude:  41.8725013733,
		Longitude: -88.7237014771,
		Timezone:  "America/Chicago",
	},
	"0IL9": {
		ICAO:      "0IL9",
		Name:      "Jack W. Watson Airport",
		City:      "De Kalb",
		State:     "Illinois",
		Country:   "US",
		Elevation: 880,
		Latitude:  41.854801178,
		Longitude: -88.7873001099,
		Timezone:  "America/Chicago",
	},
	"0IN1": {
		ICAO:      "0IN1",
		Name:      "Snider Field",
		City:      "Decker",
		State:     "Indiana",
		Country:   "US",
		Elevation: 400,
		Latitude:  38.5,
		Longitude: -87.5959014893,
		Timezone:  "America/Chicago",
	},
	"0IN2": {
		ICAO:      "0IN2",
		Name:      "The Lazy K Airport",
		City:      "Tocsin",
		State:     "Indiana",
		Country:   "US",
		Elevation: 835,
		Latitude:  40.8278007507,
		Longitude: -85.0860977173,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"0IN3": {
		ICAO:      "0IN3",
		Name:      "Graves Landing Strip",
		City:      "Winamac",
		State:     "Indiana",
		Country:   "US",
		Elevation: 704,
		Latitude:  41.1166992188,
		Longitude: -86.6667022705,
		Timezone:  "America/Indiana/Winamac",
	},
	"0IN4": {
		ICAO:      "0IN4",
		Name:      "Air Park Field",
		City:      "Woodburn",
		State:     "Indiana",
		Country:   "US",
		Elevation: 750,
		Latitude:  41.1277999878,
		Longitude: -84.9471969604,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"0IN5": {
		ICAO:      "0IN5",
		Name:      "Byrne Field",
		City:      "Byrneville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 800,
		Latitude:  38.3289985657,
		Longitude: -86.0335998535,
		Timezone:  "America/Kentucky/Louisville",
	},
	"0IN9": {
		ICAO:      "0IN9",
		Name:      "Parrish Airport",
		City:      "Mount Vernon",
		State:     "Indiana",
		Country:   "US",
		Elevation: 390,
		Latitude:  37.9930992126,
		Longitude: -87.9514007568,
		Timezone:  "America/Chicago",
	},
	"0IS3": {
		ICAO:      "0IS3",
		Name:      "Cady Aerial RLA Restricted Landing Area",
		City:      "Deer Grove",
		State:     "Illinois",
		Country:   "US",
		Elevation: 630,
		Latitude:  41.6608009338,
		Longitude: -89.6936035156,
		Timezone:  "America/Chicago",
	},
	"0IS4": {
		ICAO:      "0IS4",
		Name:      "Dawson Farms Airport",
		City:      "Lovington",
		State:     "Illinois",
		Country:   "US",
		Elevation: 670,
		Latitude:  39.7625007629,
		Longitude: -88.6715011597,
		Timezone:  "America/Chicago",
	},
	"0KS0": {
		ICAO:      "0KS0",
		Name:      "J V Ranch Airport",
		City:      "Osage City",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1180,
		Latitude:  38.6333007813,
		Longitude: -95.9253005981,
		Timezone:  "America/Chicago",
	},
	"0KS2": {
		ICAO:      "0KS2",
		Name:      "Kellie Mann Airfield",
		City:      "Ottawa",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1045,
		Latitude:  38.5600013733,
		Longitude: -95.30909729,
		Timezone:  "America/Chicago",
	},
	"0KS3": {
		ICAO:      "0KS3",
		Name:      "Camp Chippewa Airport",
		City:      "Ottawa",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1120,
		Latitude:  38.5666999817,
		Longitude: -95.366897583,
		Timezone:  "America/Chicago",
	},
	"0KS4": {
		ICAO:      "0KS4",
		Name:      "Norris Airport",
		City:      "Lawrence",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1065,
		Latitude:  38.807239,
		Longitude: -95.370056,
		Timezone:  "America/Chicago",
	},
	"0KS6": {
		ICAO:      "0KS6",
		Name:      "Kendrigan Airport",
		City:      "Peck",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1240,
		Latitude:  37.4522018433,
		Longitude: -97.3178024292,
		Timezone:  "America/Chicago",
	},
	"0KS7": {
		ICAO:      "0KS7",
		Name:      "Mono Aircraft Airport",
		City:      "Cheney",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1450,
		Latitude:  37.6927986145,
		Longitude: -97.8863983154,
		Timezone:  "America/Chicago",
	},
	"0KS8": {
		ICAO:      "0KS8",
		Name:      "Pearce Field",
		City:      "Parsons",
		State:     "Kansas",
		Country:   "US",
		Elevation: 950,
		Latitude:  37.3824996948,
		Longitude: -95.3747024536,
		Timezone:  "America/Chicago",
	},
	"0KS9": {
		ICAO:      "0KS9",
		Name:      "Lmn 120 Airport",
		City:      "Douglas",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1330,
		Latitude:  37.5158996582,
		Longitude: -96.9197998047,
		Timezone:  "America/Chicago",
	},
	"0KY0": {
		ICAO:      "0KY0",
		Name:      "Owen Air Park",
		City:      "Owenton",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 890,
		Latitude:  38.6227989197,
		Longitude: -84.7789001465,
		Timezone:  "America/New_York",
	},
	"0KY1": {
		ICAO:      "0KY1",
		Name:      "Arnemann Farms Airport",
		City:      "Franklin",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 504,
		Latitude:  36.7655982971,
		Longitude: -86.6183013916,
		Timezone:  "America/Chicago",
	},
	"0KY4": {
		ICAO:      "0KY4",
		Name:      "Cambron Field",
		City:      "Owensboro",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 427,
		Latitude:  37.5643997192,
		Longitude: -87.0819015503,
		Timezone:  "America/Chicago",
	},
	"0KY5": {
		ICAO:      "0KY5",
		Name:      "Falin Landing Strip",
		City:      "Bowling Green",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 600,
		Latitude:  36.8125,
		Longitude: -86.375,
		Timezone:  "America/Chicago",
	},
	"0KY7": {
		ICAO:      "0KY7",
		Name:      "Clinton-Hickman County Airport",
		City:      "Clinton",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 360,
		Latitude:  36.636100769,
		Longitude: -88.9985961914,
		Timezone:  "America/Chicago",
	},
	"0KY8": {
		ICAO:      "0KY8",
		Name:      "Sky Airport",
		City:      "Manchester",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 850,
		Latitude:  37.1171989441,
		Longitude: -83.7360992432,
		Timezone:  "America/New_York",
	},
	"0LA1": {
		ICAO:      "0LA1",
		Name:      "Double H Ranch Airport",
		City:      "Gonzales",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 15,
		Latitude:  30.2294006348,
		Longitude: -90.9733963013,
		Timezone:  "America/Chicago",
	},
	"0LA2": {
		ICAO:      "0LA2",
		Name:      "Don Babin-Private Airport",
		City:      "Gonzales",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 17,
		Latitude:  30.2779998779,
		Longitude: -90.9632034302,
		Timezone:  "America/Chicago",
	},
	"0LA3": {
		ICAO:      "0LA3",
		Name:      "Belcher Airpatch Airport",
		City:      "Belcher",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 280,
		Latitude:  32.7492980957,
		Longitude: -93.8734970093,
		Timezone:  "America/Chicago",
	},
	"0LA9": {
		ICAO:      "0LA9",
		Name:      "Rebel Field",
		City:      "Watson",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 74,
		Latitude:  30.5862998962,
		Longitude: -90.8615036011,
		Timezone:  "America/Chicago",
	},
	"0LL0": {
		ICAO:      "0LL0",
		Name:      "Kaufield Airport",
		City:      "Belvidere",
		State:     "Illinois",
		Country:   "US",
		Elevation: 783,
		Latitude:  42.2647018433,
		Longitude: -88.7300033569,
		Timezone:  "America/Chicago",
	},
	"0LL1": {
		ICAO:      "0LL1",
		Name:      "Jerseyville Aviation Inc Airport",
		City:      "Jerseyville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 604,
		Latitude:  39.0942001343,
		Longitude: -90.3134002686,
		Timezone:  "America/Chicago",
	},
	"0LL2": {
		ICAO:      "0LL2",
		Name:      "Murk's Strip",
		City:      "St. Augustine",
		State:     "Illinois",
		Country:   "US",
		Elevation: 700,
		Latitude:  40.738899231,
		Longitude: -90.3806991577,
		Timezone:  "America/Chicago",
	},
	"0LL3": {
		ICAO:      "0LL3",
		Name:      "Koenig Airport",
		City:      "Jerseyville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 614,
		Latitude:  39.0550003052,
		Longitude: -90.3421020508,
		Timezone:  "America/Chicago",
	},
	"0LL4": {
		ICAO:      "0LL4",
		Name:      "Flaherty Field",
		City:      "La Salle",
		State:     "Illinois",
		Country:   "US",
		Elevation: 664,
		Latitude:  41.4192008972,
		Longitude: -89.1231002808,
		Timezone:  "America/Chicago",
	},
	"0LL5": {
		ICAO:      "0LL5",
		Name:      "Busboom RLA Restricted Landing Area",
		City:      "St Joseph",
		State:     "Illinois",
		Country:   "US",
		Elevation: 675,
		Latitude:  40.1100006104,
		Longitude: -88.0783996582,
		Timezone:  "America/Chicago",
	},
	"0LL6": {
		ICAO:      "0LL6",
		Name:      "Gord Airport",
		City:      "Sandwich",
		State:     "Illinois",
		Country:   "US",
		Elevation: 710,
		Latitude:  41.6983985901,
		Longitude: -88.6075973511,
		Timezone:  "America/Chicago",
	},
	"0LS0": {
		ICAO:      "0LS0",
		Name:      "Theriot Field",
		City:      "Thibodaux",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 13,
		Latitude:  29.7497005463,
		Longitude: -90.7358016968,
		Timezone:  "America/Chicago",
	},
	"0LS4": {
		ICAO:      "0LS4",
		Name:      "Buller's Airstrip",
		City:      "Bunkie",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 47,
		Latitude:  30.8087005615,
		Longitude: -92.18800354,
		Timezone:  "America/Chicago",
	},
	"0LS5": {
		ICAO:      "0LS5",
		Name:      "Trahan Ultralightport",
		City:      "Cankton",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 40,
		Latitude:  30.3069000244,
		Longitude: -92.1297988892,
		Timezone:  "America/Chicago",
	},
	"0LS6": {
		ICAO:      "0LS6",
		Name:      "Dufour Airport",
		City:      "Morrow",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 42,
		Latitude:  30.7877006531,
		Longitude: -92.0503997803,
		Timezone:  "America/Chicago",
	},
	"0LS7": {
		ICAO:      "0LS7",
		Name:      "Morgan Field",
		City:      "New Roads",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 36,
		Latitude:  30.6128005981,
		Longitude: -91.4841995239,
		Timezone:  "America/Chicago",
	},
	"0LS8": {
		ICAO:      "0LS8",
		Name:      "Lonesome Dove Airfield",
		City:      "Woodlawn",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 34,
		Latitude:  30.2858009338,
		Longitude: -92.9412994385,
		Timezone:  "America/Chicago",
	},
	"0LS9": {
		ICAO:      "0LS9",
		Name:      "Huenefeld Airport",
		City:      "Monroe",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 72,
		Latitude:  32.5042991638,
		Longitude: -91.9821014404,
		Timezone:  "America/Chicago",
	},
	"0MD1": {
		ICAO:      "0MD1",
		Name:      "Taylor Field",
		City:      "Sudlersville",
		State:     "Maryland",
		Country:   "US",
		Elevation: 75,
		Latitude:  39.1873016357,
		Longitude: -75.7927017212,
		Timezone:  "America/New_York",
	},
	"0MD2": {
		ICAO:      "0MD2",
		Name:      "Squier Landing Airport",
		City:      "Cobb Island",
		State:     "Maryland",
		Country:   "US",
		Elevation: 16,
		Latitude:  38.2878990173,
		Longitude: -76.8675003052,
		Timezone:  "America/New_York",
	},
	"0MD4": {
		ICAO:      "0MD4",
		Name:      "Pond View Private Airport",
		City:      "Chestertown",
		State:     "Maryland",
		Country:   "US",
		Elevation: 47,
		Latitude:  39.2667999268,
		Longitude: -76.1872024536,
		Timezone:  "America/New_York",
	},
	"0MD6": {
		ICAO:      "0MD6",
		Name:      "Walters Airport",
		City:      "Mount Airy",
		State:     "Maryland",
		Country:   "US",
		Elevation: 750,
		Latitude:  39.3811988831,
		Longitude: -77.105796814,
		Timezone:  "America/New_York",
	},
	"0ME4": {
		ICAO:      "0ME4",
		Name:      "Peasley Field",
		City:      "Otis",
		State:     "Maine",
		Country:   "US",
		Elevation: 271,
		Latitude:  44.742469,
		Longitude: -68.474403,
		Timezone:  "America/New_York",
	},
	"0MI1": {
		ICAO:      "0MI1",
		Name:      "Sugar Springs Airport",
		City:      "Gladwin",
		State:     "Michigan",
		Country:   "US",
		Elevation: 940,
		Latitude:  44.1403007507,
		Longitude: -84.4375,
		Timezone:  "America/Detroit",
	},
	"0MI2": {
		ICAO:      "0MI2",
		Name:      "Stier Airstrip",
		City:      "Glennie",
		State:     "Michigan",
		Country:   "US",
		Elevation: 850,
		Latitude:  44.5222015381,
		Longitude: -83.6832962036,
		Timezone:  "America/Detroit",
	},
	"0MI3": {
		ICAO:      "0MI3",
		Name:      "Kelleys Airport",
		City:      "Grayling",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1000,
		Latitude:  44.6333999634,
		Longitude: -84.4833984375,
		Timezone:  "America/Detroit",
	},
	"0MI4": {
		ICAO:      "0MI4",
		Name:      "Lesterson Field",
		City:      "Gwinn",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1120,
		Latitude:  46.270198822,
		Longitude: -87.3864974976,
		Timezone:  "America/Detroit",
	},
	"0MI6": {
		ICAO:      "0MI6",
		Name:      "Young's Airport",
		City:      "Hulbert",
		State:     "Michigan",
		Country:   "US",
		Elevation: 800,
		Latitude:  46.3875007629,
		Longitude: -85.1839981079,
		Timezone:  "America/Detroit",
	},
	"0MI8": {
		ICAO:      "0MI8",
		Name:      "Twin Lakes Airport",
		City:      "Lewiston",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1275,
		Latitude:  44.9000015259,
		Longitude: -84.3000030518,
		Timezone:  "America/Detroit",
	},
	"0MN1": {
		ICAO:      "0MN1",
		Name:      "Brinkman Airport",
		City:      "St. Peter",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1020,
		Latitude:  44.2625007629,
		Longitude: -94.0697021484,
		Timezone:  "America/Chicago",
	},
	"0MN5": {
		ICAO:      "0MN5",
		Name:      "Pankratz Airport",
		City:      "Springfield",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1033,
		Latitude:  44.3072013855,
		Longitude: -94.9169006348,
		Timezone:  "America/Chicago",
	},
	"0MN6": {
		ICAO:      "0MN6",
		Name:      "Matson Field",
		City:      "Spring Valley",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1365,
		Latitude:  43.6622009277,
		Longitude: -92.404296875,
		Timezone:  "America/Chicago",
	},
	"0MN8": {
		ICAO:      "0MN8",
		Name:      "Keller Airport",
		City:      "Stillwater",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 990,
		Latitude:  45.135799408,
		Longitude: -92.8593978882,
		Timezone:  "America/Chicago",
	},
	"0MO0": {
		ICAO:      "0MO0",
		Name:      "Ferros Ranch-Aero Airport",
		City:      "Clinton",
		State:     "Missouri",
		Country:   "US",
		Elevation: 775,
		Latitude:  38.4103012085,
		Longitude: -93.8501968384,
		Timezone:  "America/Chicago",
	},
	"0MO1": {
		ICAO:      "0MO1",
		Name:      "Flying G Airport",
		City:      "Creighton",
		State:     "Missouri",
		Country:   "US",
		Elevation: 920,
		Latitude:  38.5444984436,
		Longitude: -94.0947036743,
		Timezone:  "America/Chicago",
	},
	"0MO2": {
		ICAO:      "0MO2",
		Name:      "Short-N-Ruff Airport",
		City:      "Caledonia",
		State:     "Missouri",
		Country:   "US",
		Elevation: 950,
		Latitude:  37.7867012024,
		Longitude: -90.7800979614,
		Timezone:  "America/Chicago",
	},
	"0MO3": {
		ICAO:      "0MO3",
		Name:      "Arvin Ranch Airport",
		City:      "Drexel",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1000,
		Latitude:  38.4402999878,
		Longitude: -94.5932998657,
		Timezone:  "America/Chicago",
	},
	"0MO5": {
		ICAO:      "0MO5",
		Name:      "Joe D Lewis Airport",
		City:      "Edina",
		State:     "Missouri",
		Country:   "US",
		Elevation: 780,
		Latitude:  40.1333999634,
		Longitude: -92.1001968384,
		Timezone:  "America/Chicago",
	},
	"0MO6": {
		ICAO:      "0MO6",
		Name:      "Hall Airport",
		City:      "Edina",
		State:     "Missouri",
		Country:   "US",
		Elevation: 850,
		Latitude:  40.1166992188,
		Longitude: -92.2668991089,
		Timezone:  "America/Chicago",
	},
	"0MO8": {
		ICAO:      "0MO8",
		Name:      "Sloan's Airport",
		City:      "Elsberry",
		State:     "Missouri",
		Country:   "US",
		Elevation: 437,
		Latitude:  39.1405982971,
		Longitude: -90.7368011475,
		Timezone:  "America/Chicago",
	},
	"0MS0": {
		ICAO:      "0MS0",
		Name:      "Topton Air Estates Airport",
		City:      "Meridian",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 453,
		Latitude:  32.4749984741,
		Longitude: -88.6166992188,
		Timezone:  "America/Chicago",
	},
	"0MS1": {
		ICAO:      "0MS1",
		Name:      "Franklin Field",
		City:      "Meadville",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 200,
		Latitude:  31.4251995087,
		Longitude: -90.9029006958,
		Timezone:  "America/Chicago",
	},
	"0MS2": {
		ICAO:      "0MS2",
		Name:      "Morgan Field",
		City:      "Seminary",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 300,
		Latitude:  31.5417995453,
		Longitude: -89.3917007446,
		Timezone:  "America/Chicago",
	},
	"0MS5": {
		ICAO:      "0MS5",
		Name:      "Lewis Air Service Airport",
		City:      "Leland",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 120,
		Latitude:  33.3917999268,
		Longitude: -90.9168014526,
		Timezone:  "America/Chicago",
	},
	"0MS7": {
		ICAO:      "0MS7",
		Name:      "Hale Field",
		City:      "Pontotoc",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 346,
		Latitude:  34.3426017761,
		Longitude: -89.1430969238,
		Timezone:  "America/Chicago",
	},
	"0MS8": {
		ICAO:      "0MS8",
		Name:      "Catfish Point Airport",
		City:      "Benoit",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 154,
		Latitude:  33.6992988586,
		Longitude: -91.1651000977,
		Timezone:  "America/Chicago",
	},
	"0MS9": {
		ICAO:      "0MS9",
		Name:      "Shenandoah Valley Farms Airport",
		City:      "Houston",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 300,
		Latitude:  33.8007011414,
		Longitude: -89.0272979736,
		Timezone:  "America/Chicago",
	},
	"0MT1": {
		ICAO:      "0MT1",
		Name:      "Williams Field",
		City:      "Augusta",
		State:     "Montana",
		Country:   "US",
		Elevation: 4070,
		Latitude:  47.4975013733,
		Longitude: -112.3909988403,
		Timezone:  "America/Denver",
	},
	"0MT4": {
		ICAO:      "0MT4",
		Name:      "Kimp Airport",
		City:      "Hamilton",
		State:     "Montana",
		Country:   "US",
		Elevation: 3510,
		Latitude:  46.2785987854,
		Longitude: -114.1409988403,
		Timezone:  "America/Denver",
	},
	"0MT5": {
		ICAO:      "0MT5",
		Name:      "Cottonwood Airport",
		City:      "Laurel",
		State:     "Montana",
		Country:   "US",
		Elevation: 3350,
		Latitude:  45.6342010498,
		Longitude: -108.8489990234,
		Timezone:  "America/Denver",
	},
	"0MT6": {
		ICAO:      "0MT6",
		Name:      "Hanson Airport",
		City:      "Marion",
		State:     "Montana",
		Country:   "US",
		Elevation: 3900,
		Latitude:  48.040599823,
		Longitude: -114.6790008545,
		Timezone:  "America/Denver",
	},
	"0MT7": {
		ICAO:      "0MT7",
		Name:      "Pinehurst Ranch Airport",
		City:      "Thompson Falls",
		State:     "Montana",
		Country:   "US",
		Elevation: 2460,
		Latitude:  47.6608009338,
		Longitude: -115.4039993286,
		Timezone:  "America/Denver",
	},
	"0MT9": {
		ICAO:      "0MT9",
		Name:      "Lone Hawk Airport",
		City:      "White Fish",
		State:     "Montana",
		Country:   "US",
		Elevation: 3284,
		Latitude:  48.4832992554,
		Longitude: -114.4929962158,
		Timezone:  "America/Denver",
	},
	"0MU0": {
		ICAO:      "0MU0",
		Name:      "Eagle Field",
		City:      "Ellington",
		State:     "Missouri",
		Country:   "US",
		Elevation: 784,
		Latitude:  37.2346992493,
		Longitude: -91.0935974121,
		Timezone:  "America/Chicago",
	},
	"0MU1": {
		ICAO:      "0MU1",
		Name:      "Sunderland Airport",
		City:      "Avilla",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1027,
		Latitude:  37.2467002869,
		Longitude: -94.1738967896,
		Timezone:  "America/Chicago",
	},
	"0MU4": {
		ICAO:      "0MU4",
		Name:      "Slaughter Airport",
		City:      "Marceline",
		State:     "Missouri",
		Country:   "US",
		Elevation: 800,
		Latitude:  39.6161003113,
		Longitude: -92.866897583,
		Timezone:  "America/Chicago",
	},
	"0MU7": {
		ICAO:      "0MU7",
		Name:      "Lambs Field",
		City:      "Lone Jack",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1070,
		Latitude:  38.875,
		Longitude: -94.1397018433,
		Timezone:  "America/Chicago",
	},
	"0MU8": {
		ICAO:      "0MU8",
		Name:      "Prince STOLport",
		City:      "Millersville",
		State:     "Missouri",
		Country:   "US",
		Elevation: 582,
		Latitude:  37.4387016296,
		Longitude: -89.7590026855,
		Timezone:  "America/Chicago",
	},
	"0NA1": {
		ICAO:      "0NA1",
		Name:      "Goerger Airport",
		City:      "Barney",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1045,
		Latitude:  46.2454986572,
		Longitude: -97.0375976563,
		Timezone:  "America/Chicago",
	},
	"0NA5": {
		ICAO:      "0NA5",
		Name:      "Sorlie Airport",
		City:      "Maddock",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1580,
		Latitude:  48.0010986328,
		Longitude: -99.5628967285,
		Timezone:  "America/Chicago",
	},
	"0NA9": {
		ICAO:      "0NA9",
		Name:      "Boyd's Turf Airport",
		City:      "Golva",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 2750,
		Latitude:  46.75,
		Longitude: -103.8010025024,
		Timezone:  "America/Denver",
	},
	"0NC1": {
		ICAO:      "0NC1",
		Name:      "Bearwallow Farm Airport",
		City:      "Hendersonville",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 3608,
		Latitude:  35.4570999146,
		Longitude: -82.3753967285,
		Timezone:  "America/New_York",
	},
	"0NC7": {
		ICAO:      "0NC7",
		Name:      "Lindsay Airport",
		City:      "Madison",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 700,
		Latitude:  36.3572006226,
		Longitude: -79.9974975586,
		Timezone:  "America/New_York",
	},
	"0NC8": {
		ICAO:      "0NC8",
		Name:      "Double Creek Ranch Airport",
		City:      "Cleveland",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 830,
		Latitude:  35.7556991577,
		Longitude: -80.7352981567,
		Timezone:  "America/New_York",
	},
	"0ND0": {
		ICAO:      "0ND0",
		Name:      "Gilbertson Field",
		City:      "Flora",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1550,
		Latitude:  47.9291992188,
		Longitude: -99.4003982544,
		Timezone:  "America/Chicago",
	},
	"0ND6": {
		ICAO:      "0ND6",
		Name:      "Schumacher Strip",
		City:      "Hague",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 2020,
		Latitude:  46.0550003052,
		Longitude: -99.917098999,
		Timezone:  "America/Chicago",
	},
	"0ND7": {
		ICAO:      "0ND7",
		Name:      "Saville Private Airport",
		City:      "Hazelton",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1961,
		Latitude:  46.4653015137,
		Longitude: -100.1050033569,
		Timezone:  "America/Chicago",
	},
	"0NE0": {
		ICAO:      "0NE0",
		Name:      "Merrihew Airport",
		City:      "Ashby",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3815,
		Latitude:  41.8499984741,
		Longitude: -101.9499969482,
		Timezone:  "America/Denver",
	},
	"0NE1": {
		ICAO:      "0NE1",
		Name:      "Runner Landing Area Airport",
		City:      "Ashby",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3760,
		Latitude:  42.4071998596,
		Longitude: -101.9319992065,
		Timezone:  "America/Denver",
	},
	"0NE2": {
		ICAO:      "0NE2",
		Name:      "Merry Airport",
		City:      "Ashby",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3791,
		Latitude:  41.8666000366,
		Longitude: -101.9169998169,
		Timezone:  "America/Denver",
	},
	"0NE5": {
		ICAO:      "0NE5",
		Name:      "Newman Airport",
		City:      "Big Springs",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3525,
		Latitude:  41.0210990906,
		Longitude: -102.0179977417,
		Timezone:  "America/Denver",
	},
	"0NE6": {
		ICAO:      "0NE6",
		Name:      "Frager Field",
		City:      "Fairfield",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1765,
		Latitude:  40.4333000183,
		Longitude: -98.070602417,
		Timezone:  "America/Chicago",
	},
	"0NJ5": {
		ICAO:      "0NJ5",
		Name:      "Fla-Net Airport",
		City:      "Netcong",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 1040,
		Latitude:  40.8858985901,
		Longitude: -74.7057037354,
		Timezone:  "America/New_York",
	},
	"0NJ6": {
		ICAO:      "0NJ6",
		Name:      "Dix Field",
		City:      "Linwood",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 8,
		Latitude:  39.3167991638,
		Longitude: -74.6328964233,
		Timezone:  "America/New_York",
	},
	"0NK0": {
		ICAO:      "0NK0",
		Name:      "Berdick Field",
		City:      "Cattaraugus",
		State:     "New York",
		Country:   "US",
		Elevation: 1690,
		Latitude:  42.3339996338,
		Longitude: -78.7909011841,
		Timezone:  "America/New_York",
	},
	"0NK1": {
		ICAO:      "0NK1",
		Name:      "Spring Brook Airport",
		City:      "Central Square",
		State:     "New York",
		Country:   "US",
		Elevation: 450,
		Latitude:  43.3616981506,
		Longitude: -76.220199585,
		Timezone:  "America/New_York",
	},
	"0NK2": {
		ICAO:      "0NK2",
		Name:      "Westwind Farm Airport",
		City:      "Charlton",
		State:     "New York",
		Country:   "US",
		Elevation: 800,
		Latitude:  42.9833984375,
		Longitude: -74.0161972046,
		Timezone:  "America/New_York",
	},
	"0NK3": {
		ICAO:      "0NK3",
		Name:      "Seven Gullies Airport",
		City:      "Groveland",
		State:     "New York",
		Country:   "US",
		Elevation: 600,
		Latitude:  42.6958999634,
		Longitude: -77.7921981812,
		Timezone:  "America/New_York",
	},
	"0NK6": {
		ICAO:      "0NK6",
		Name:      "Lapeer Flyer Airport",
		City:      "Marathon",
		State:     "New York",
		Country:   "US",
		Elevation: 1280,
		Latitude:  42.4384002686,
		Longitude: -76.0887985229,
		Timezone:  "America/New_York",
	},
	"0NK8": {
		ICAO:      "0NK8",
		Name:      "Rocky Reef Farm Airport",
		City:      "Attlebury",
		State:     "New York",
		Country:   "US",
		Elevation: 450,
		Latitude:  41.9208984375,
		Longitude: -73.6704025269,
		Timezone:  "America/New_York",
	},
	"0NK9": {
		ICAO:      "0NK9",
		Name:      "Lakeview Airport",
		City:      "Chittenango",
		State:     "New York",
		Country:   "US",
		Elevation: 1050,
		Latitude:  43.0102996826,
		Longitude: -75.8268966675,
		Timezone:  "America/New_York",
	},
	"0NM0": {
		ICAO:      "0NM0",
		Name:      "Columbus Municipal Airport",
		City:      "Columbus",
		State:     "New Mexico",
		Country:   "US",
		Elevation: 4024,
		Latitude:  31.8236999512,
		Longitude: -107.6269989014,
		Timezone:  "America/Denver",
	},
	"0NM7": {
		ICAO:      "0NM7",
		Name:      "Negrito Airstrip",
		City:      "Reserve",
		State:     "New Mexico",
		Country:   "US",
		Elevation: 8143,
		Latitude:  33.5209007263,
		Longitude: -108.5429992676,
		Timezone:  "America/Denver",
	},
	"0NY0": {
		ICAO:      "0NY0",
		Name:      "Bennetts Airport",
		City:      "North Creek",
		State:     "New York",
		Country:   "US",
		Elevation: 1040,
		Latitude:  43.7000999451,
		Longitude: -73.9829025269,
		Timezone:  "America/New_York",
	},
	"0NY1": {
		ICAO:      "0NY1",
		Name:      "Russell Field",
		City:      "Northumberland",
		State:     "New York",
		Country:   "US",
		Elevation: 260,
		Latitude:  43.1391983032,
		Longitude: -73.6531982422,
		Timezone:  "America/New_York",
	},
	"0NY3": {
		ICAO:      "0NY3",
		Name:      "Kermizian Airport",
		City:      "Ohio",
		State:     "New York",
		Country:   "US",
		Elevation: 1440,
		Latitude:  43.313999176,
		Longitude: -74.9320983887,
		Timezone:  "America/New_York",
	},
	"0NY7": {
		ICAO:      "0NY7",
		Name:      "Murphys Landing Strip",
		City:      "Perth",
		State:     "New York",
		Country:   "US",
		Elevation: 940,
		Latitude:  43.0376014709,
		Longitude: -74.1843032837,
		Timezone:  "America/New_York",
	},
	"0NY8": {
		ICAO:      "0NY8",
		Name:      "Wenskoski Field",
		City:      "Perth",
		State:     "New York",
		Country:   "US",
		Elevation: 800,
		Latitude:  42.9972991943,
		Longitude: -74.2453994751,
		Timezone:  "America/New_York",
	},
	"0OH2": {
		ICAO:      "0OH2",
		Name:      "Jenkins Field",
		City:      "North Hampton",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1080,
		Latitude:  39.9706001282,
		Longitude: -83.9115982056,
		Timezone:  "America/New_York",
	},
	"0OH6": {
		ICAO:      "0OH6",
		Name:      "Fry Field",
		City:      "Spring Valley",
		State:     "Ohio",
		Country:   "US",
		Elevation: 775,
		Latitude:  39.6006011963,
		Longitude: -84.0196990967,
		Timezone:  "America/New_York",
	},
	"0OH7": {
		ICAO:      "0OH7",
		Name:      "Apple Airport",
		City:      "Piqua",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1000,
		Latitude:  40.1432991028,
		Longitude: -84.1718978882,
		Timezone:  "America/New_York",
	},
	"0OH8": {
		ICAO:      "0OH8",
		Name:      "Fry Airport",
		City:      "Piqua",
		State:     "Ohio",
		Country:   "US",
		Elevation: 900,
		Latitude:  40.1884002686,
		Longitude: -84.2561035156,
		Timezone:  "America/New_York",
	},
	"0OH9": {
		ICAO:      "0OH9",
		Name:      "Parknavy STOLport",
		City:      "Plain City",
		State:     "Ohio",
		Country:   "US",
		Elevation: 943,
		Latitude:  40.0717010498,
		Longitude: -83.2298965454,
		Timezone:  "America/New_York",
	},
	"0OI2": {
		ICAO:      "0OI2",
		Name:      "Unger Field",
		City:      "Greenfield",
		State:     "Ohio",
		Country:   "US",
		Elevation: 940,
		Latitude:  39.3597984314,
		Longitude: -83.4177017212,
		Timezone:  "America/New_York",
	},
	"0OI3": {
		ICAO:      "0OI3",
		Name:      "Galloway Airport",
		City:      "Sandusky",
		State:     "Ohio",
		Country:   "US",
		Elevation: 643,
		Latitude:  41.3820991516,
		Longitude: -82.7161026001,
		Timezone:  "America/New_York",
	},
	"0OI4": {
		ICAO:      "0OI4",
		Name:      "Salt Box Airport",
		City:      "Hiram",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1250,
		Latitude:  41.3242645264,
		Longitude: -81.1593093872,
		Timezone:  "America/New_York",
	},
	"0OI6": {
		ICAO:      "0OI6",
		Name:      "Victor's Landing Airport",
		City:      "Kingsville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 860,
		Latitude:  41.8381004333,
		Longitude: -80.643699646,
		Timezone:  "America/New_York",
	},
	"0OI9": {
		ICAO:      "0OI9",
		Name:      "Hidden Quarry Airport",
		City:      "Lucasville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 830,
		Latitude:  38.8945007324,
		Longitude: -83.1135025024,
		Timezone:  "America/New_York",
	},
	"0OK0": {
		ICAO:      "0OK0",
		Name:      "Edmond Airport",
		City:      "Edmond",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1169,
		Latitude:  35.6833992004,
		Longitude: -97.4336013794,
		Timezone:  "America/Chicago",
	},
	"0OK1": {
		ICAO:      "0OK1",
		Name:      "Mckey Airport",
		City:      "Elmore City",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1056,
		Latitude:  34.6570014954,
		Longitude: -97.4017028809,
		Timezone:  "America/Chicago",
	},
	"0OK3": {
		ICAO:      "0OK3",
		Name:      "Mckinley Ranch Airport",
		City:      "Geary",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1420,
		Latitude:  35.6194992065,
		Longitude: -98.5225982666,
		Timezone:  "America/Chicago",
	},
	"0OK4": {
		ICAO:      "0OK4",
		Name:      "Rock Creek Farm Airport",
		City:      "Coal Gate",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 730,
		Latitude:  34.6605987549,
		Longitude: -96.2536010742,
		Timezone:  "America/Chicago",
	},
	"0OK5": {
		ICAO:      "0OK5",
		Name:      "Chappell Airport",
		City:      "Guthrie",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1070,
		Latitude:  35.8889007568,
		Longitude: -97.4002990723,
		Timezone:  "America/Chicago",
	},
	"0OK6": {
		ICAO:      "0OK6",
		Name:      "Ellis/Harvey Airport",
		City:      "Guthrie",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1220,
		Latitude:  35.801700592,
		Longitude: -97.386100769,
		Timezone:  "America/Chicago",
	},
	"0OK8": {
		ICAO:      "0OK8",
		Name:      "Cade's Airport",
		City:      "Covington",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1140,
		Latitude:  36.2924995422,
		Longitude: -97.5931015015,
		Timezone:  "America/Chicago",
	},
	"0OK9": {
		ICAO:      "0OK9",
		Name:      "Crystal Airport",
		City:      "Jennings",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1016,
		Latitude:  36.2134017944,
		Longitude: -96.6370010376,
		Timezone:  "America/Chicago",
	},
	"0OR3": {
		ICAO:      "0OR3",
		Name:      "Long Ranch Airport",
		City:      "Merrill",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4090,
		Latitude:  42.0190010071,
		Longitude: -121.7040023804,
		Timezone:  "America/Los_Angeles",
	},
	"0OR5": {
		ICAO:      "0OR5",
		Name:      "East Oregon Cattle Co Airport",
		City:      "Eagle Point",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1345,
		Latitude:  42.5035018921,
		Longitude: -122.855003357,
		Timezone:  "America/Los_Angeles",
	},
	"0OR6": {
		ICAO:      "0OR6",
		Name:      "Rome Service Airport",
		City:      "Rome",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3387,
		Latitude:  42.8339996338,
		Longitude: -117.6289978027,
		Timezone:  "America/Boise",
	},
	"0OR7": {
		ICAO:      "0OR7",
		Name:      "Marr Field",
		City:      "Monmouth",
		State:     "Oregon",
		Country:   "US",
		Elevation: 275,
		Latitude:  44.8540000916,
		Longitude: -123.263999939,
		Timezone:  "America/Los_Angeles",
	},
	"0OR8": {
		ICAO:      "0OR8",
		Name:      "Sutton on Rogue Airport",
		City:      "White City",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1260,
		Latitude:  42.4847984314,
		Longitude: -122.8659973145,
		Timezone:  "America/Los_Angeles",
	},
	"0OR9": {
		ICAO:      "0OR9",
		Name:      "Hanel Field",
		City:      "Mount Hood",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1626,
		Latitude:  45.5926017761,
		Longitude: -121.5490036011,
		Timezone:  "America/Los_Angeles",
	},
	"0PA0": {
		ICAO:      "0PA0",
		Name:      "Philadelphia Gliderport",
		City:      "Hilltown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 670,
		Latitude:  40.3283996582,
		Longitude: -75.2413024902,
		Timezone:  "America/New_York",
	},
	"0PA2": {
		ICAO:      "0PA2",
		Name:      "Haunstein Ultralightport",
		City:      "Shermans Dale",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 600,
		Latitude:  40.3167991638,
		Longitude: -77.1247024536,
		Timezone:  "America/New_York",
	},
	"0PA3": {
		ICAO:      "0PA3",
		Name:      "Robbins Farm Airport",
		City:      "Troy",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1207,
		Latitude:  41.7495002747,
		Longitude: -76.7602005005,
		Timezone:  "America/New_York",
	},
	"0PA4": {
		ICAO:      "0PA4",
		Name:      "Ecko Field",
		City:      "Hummelstown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 380,
		Latitude:  40.2375984192,
		Longitude: -76.7247009277,
		Timezone:  "America/New_York",
	},
	"0PA5": {
		ICAO:      "0PA5",
		Name:      "Fisher Airport",
		City:      "Franklin",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1500,
		Latitude:  41.4944992065,
		Longitude: -79.8309020996,
		Timezone:  "America/New_York",
	},
	"0PA6": {
		ICAO:      "0PA6",
		Name:      "Hostetler Airport",
		City:      "Huntingdon",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 760,
		Latitude:  40.6309013367,
		Longitude: -77.8503036499,
		Timezone:  "America/New_York",
	},
	"0PA7": {
		ICAO:      "0PA7",
		Name:      "Linvill Airport",
		City:      "Media",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 180,
		Latitude:  39.8847999573,
		Longitude: -75.4149017334,
		Timezone:  "America/New_York",
	},
	"0PN0": {
		ICAO:      "0PN0",
		Name:      "Fletcher Airport",
		City:      "Sheakeyville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1335,
		Latitude:  41.4435997009,
		Longitude: -80.1539001465,
		Timezone:  "America/New_York",
	},
	"0PN2": {
		ICAO:      "0PN2",
		Name:      "Flying Acres Airport",
		City:      "Slippery Rock",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1247,
		Latitude:  41.0731010437,
		Longitude: -80.0702972412,
		Timezone:  "America/New_York",
	},
	"0PN4": {
		ICAO:      "0PN4",
		Name:      "Kitner Airport",
		City:      "New Bloomfield",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 960,
		Latitude:  40.4128990173,
		Longitude: -77.2414016724,
		Timezone:  "America/New_York",
	},
	"0PN5": {
		ICAO:      "0PN5",
		Name:      "Nichols Airport",
		City:      "Spartansburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1761,
		Latitude:  41.7792015076,
		Longitude: -79.5983963013,
		Timezone:  "America/New_York",
	},
	"0PN6": {
		ICAO:      "0PN6",
		Name:      "Memmi Airport",
		City:      "Three Springs",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1138,
		Latitude:  40.1689987183,
		Longitude: -77.9925003052,
		Timezone:  "America/New_York",
	},
	"0PN7": {
		ICAO:      "0PN7",
		Name:      "Drillmore Acres Airport",
		City:      "Newburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 680,
		Latitude:  40.1282997131,
		Longitude: -77.631401062,
		Timezone:  "America/New_York",
	},
	"0PN8": {
		ICAO:      "0PN8",
		Name:      "Shriver Airport",
		City:      "Titusville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1560,
		Latitude:  41.6166992188,
		Longitude: -79.6830978394,
		Timezone:  "America/New_York",
	},
	"0PN9": {
		ICAO:      "0PN9",
		Name:      "Paul Personal Use Airport",
		City:      "Titusville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1645,
		Latitude:  41.7195014954,
		Longitude: -79.6778030396,
		Timezone:  "America/New_York",
	},
	"0PS0": {
		ICAO:      "0PS0",
		Name:      "Frederick Airpark",
		City:      "Tylersburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1560,
		Latitude:  41.3708992004,
		Longitude: -79.3416976929,
		Timezone:  "America/New_York",
	},
	"0PS3": {
		ICAO:      "0PS3",
		Name:      "Carsonville Airport",
		City:      "Carsonville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 800,
		Latitude:  40.490398407,
		Longitude: -76.7666015625,
		Timezone:  "America/New_York",
	},
	"0PS4": {
		ICAO:      "0PS4",
		Name:      "Old Orchard Airport",
		City:      "Curwensville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1980,
		Latitude:  41.0083999634,
		Longitude: -78.5608978271,
		Timezone:  "America/New_York",
	},
	"0PS7": {
		ICAO:      "0PS7",
		Name:      "Harman Airport",
		City:      "Elizabethville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 720,
		Latitude:  40.5223007202,
		Longitude: -76.8454971313,
		Timezone:  "America/New_York",
	},
	"0SD0": {
		ICAO:      "0SD0",
		Name:      "Lenling Airport",
		City:      "Glencross",
		State:     "South Dakota",
		Country:   "US",
		Elevation: 2142,
		Latitude:  45.4249992371,
		Longitude: -100.891998291,
		Timezone:  "America/Denver",
	},
	"0SD1": {
		ICAO:      "0SD1",
		Name:      "Lodi Airport",
		City:      "Wakonda",
		State:     "South Dakota",
		Country:   "US",
		Elevation: 1200,
		Latitude:  42.9721984863,
		Longitude: -96.9802017212,
		Timezone:  "America/Chicago",
	},
	"0SD3": {
		ICAO:      "0SD3",
		Name:      "Bledsoe Ranch Airport",
		City:      "Hoover",
		State:     "South Dakota",
		Country:   "US",
		Elevation: 2872,
		Latitude:  45.2694015503,
		Longitude: -103.2170028687,
		Timezone:  "America/Denver",
	},
	"0SD6": {
		ICAO:      "0SD6",
		Name:      "Oasis Ranch Airport",
		City:      "Philip",
		State:     "South Dakota",
		Country:   "US",
		Elevation: 2580,
		Latitude:  44.288898468,
		Longitude: -101.8259963989,
		Timezone:  "America/Denver",
	},
	"0SD7": {
		ICAO:      "0SD7",
		Name:      "Porch Ranch Airport",
		City:      "Wanblee",
		State:     "South Dakota",
		Country:   "US",
		Elevation: 2775,
		Latitude:  43.5243988037,
		Longitude: -101.7340011597,
		Timezone:  "America/Denver",
	},
	"0SD8": {
		ICAO:      "0SD8",
		Name:      "Juhnke Airport",
		City:      "Vivian",
		State:     "South Dakota",
		Country:   "US",
		Elevation: 1900,
		Latitude:  43.7827987671,
		Longitude: -100.2929992676,
		Timezone:  "America/Chicago",
	},
	"0TA1": {
		ICAO:      "0TA1",
		Name:      "Buddy Harmel Airport",
		City:      "Halfway",
		State:     "Texas",
		Country:   "US",
		Elevation: 3552,
		Latitude:  34.2195014954,
		Longitude: -101.9800033569,
		Timezone:  "America/Chicago",
	},
	"0TA2": {
		ICAO:      "0TA2",
		Name:      "Smith Farms Airport",
		City:      "Halfway",
		State:     "Texas",
		Country:   "US",
		Elevation: 3487,
		Latitude:  34.2167015076,
		Longitude: -101.9250030518,
		Timezone:  "America/Chicago",
	},
	"0TA3": {
		ICAO:      "0TA3",
		Name:      "Tate Ranch Airport",
		City:      "Dryden",
		State:     "Texas",
		Country:   "US",
		Elevation: 2650,
		Latitude:  30.3043003082,
		Longitude: -102.1380004883,
		Timezone:  "America/Chicago",
	},
	"0TA4": {
		ICAO:      "0TA4",
		Name:      "Erco Field",
		City:      "Kaufman",
		State:     "Texas",
		Country:   "US",
		Elevation: 395,
		Latitude:  32.4978981018,
		Longitude: -96.2705001831,
		Timezone:  "America/Chicago",
	},
	"0TA7": {
		ICAO:      "0TA7",
		Name:      "Alta Vista Ranch Airport",
		City:      "Marfa",
		State:     "Texas",
		Country:   "US",
		Elevation: 4702,
		Latitude:  30.1481990814,
		Longitude: -103.8929977417,
		Timezone:  "America/Chicago",
	},
	"0TA8": {
		ICAO:      "0TA8",
		Name:      "Womack Plantation Airport",
		City:      "Paris",
		State:     "Texas",
		Country:   "US",
		Elevation: 410,
		Latitude:  33.8487014771,
		Longitude: -95.4072036743,
		Timezone:  "America/Chicago",
	},
	"0TE0": {
		ICAO:      "0TE0",
		Name:      "Robertson Ranch Airport",
		City:      "Comstock",
		State:     "Texas",
		Country:   "US",
		Elevation: 1663,
		Latitude:  29.6930007935,
		Longitude: -101.1620025635,
		Timezone:  "America/Chicago",
	},
	"0TE3": {
		ICAO:      "0TE3",
		Name:      "Buzz Field",
		City:      "Roaring Springs",
		State:     "Texas",
		Country:   "US",
		Elevation: 2614,
		Latitude:  33.9291992188,
		Longitude: -100.8420028687,
		Timezone:  "America/Chicago",
	},
	"0TE4": {
		ICAO:      "0TE4",
		Name:      "Hilltop Lakes Airport",
		City:      "Hilltop Lakes",
		State:     "Texas",
		Country:   "US",
		Elevation: 501,
		Latitude:  31.0809993744,
		Longitude: -96.2152023315,
		Timezone:  "America/Chicago",
	},
	"0TE5": {
		ICAO:      "0TE5",
		Name:      "Santa Maria Ranch Airport",
		City:      "Laredo",
		State:     "Texas",
		Country:   "US",
		Elevation: 516,
		Latitude:  27.4337997437,
		Longitude: -99.1858978271,
		Timezone:  "America/Chicago",
	},
	"0TE6": {
		ICAO:      "0TE6",
		Name:      "Gorman Airport",
		City:      "Gorman",
		State:     "Texas",
		Country:   "US",
		Elevation: 1452,
		Latitude:  32.2042999268,
		Longitude: -98.6656036377,
		Timezone:  "America/Chicago",
	},
	"0TE7": {
		ICAO:      "0TE7",
		Name:      "Lbj Ranch Airport",
		City:      "Johnson City",
		State:     "Texas",
		Country:   "US",
		Elevation: 1515,
		Latitude:  30.2518005371,
		Longitude: -98.6224975586,
		Timezone:  "America/Chicago",
	},
	"0TN0": {
		ICAO:      "0TN0",
		Name:      "R & S Buzzard Airport",
		City:      "Spring Hill",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 810,
		Latitude:  35.676700592,
		Longitude: -86.896697998,
		Timezone:  "America/Chicago",
	},
	"0TN2": {
		ICAO:      "0TN2",
		Name:      "Village Airport",
		City:      "Greenback",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 920,
		Latitude:  35.6558990479,
		Longitude: -84.1855010986,
		Timezone:  "America/New_York",
	},
	"0TN5": {
		ICAO:      "0TN5",
		Name:      "Mc Donald Airport",
		City:      "Murfreesboro",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 600,
		Latitude:  35.867401123,
		Longitude: -86.4978027344,
		Timezone:  "America/Chicago",
	},
	"0TN6": {
		ICAO:      "0TN6",
		Name:      "Riner Farm Airport",
		City:      "Norris",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1125,
		Latitude:  36.1833992004,
		Longitude: -84.0990982056,
		Timezone:  "America/New_York",
	},
	"0TS0": {
		ICAO:      "0TS0",
		Name:      "Flower Mound Airport",
		City:      "Flower Mound",
		State:     "Texas",
		Country:   "US",
		Elevation: 625,
		Latitude:  33.0228996277,
		Longitude: -97.1224975586,
		Timezone:  "America/Chicago",
	},
	"0TS1": {
		ICAO:      "0TS1",
		Name:      "Dooley Airport",
		City:      "Justin",
		State:     "Texas",
		Country:   "US",
		Elevation: 650,
		Latitude:  33.0987014771,
		Longitude: -97.2724990845,
		Timezone:  "America/Chicago",
	},
	"0TS2": {
		ICAO:      "0TS2",
		Name:      "Ultralight International Ultralightport",
		City:      "Haslet",
		State:     "Texas",
		Country:   "US",
		Elevation: 820,
		Latitude:  32.9486999512,
		Longitude: -97.3289031982,
		Timezone:  "America/Chicago",
	},
	"0TS4": {
		ICAO:      "0TS4",
		Name:      "Ullrich Airport",
		City:      "Ledbetter",
		State:     "Texas",
		Country:   "US",
		Elevation: 410,
		Latitude:  30.0918998718,
		Longitude: -96.7949981689,
		Timezone:  "America/Chicago",
	},
	"0TS5": {
		ICAO:      "0TS5",
		Name:      "Lake Bay Gall Airport",
		City:      "Cleveland",
		State:     "Texas",
		Country:   "US",
		Elevation: 230,
		Latitude:  30.4419002533,
		Longitude: -95.1863021851,
		Timezone:  "America/Chicago",
	},
	"0TS7": {
		ICAO:      "0TS7",
		Name:      "Flying U Airport",
		City:      "Mineral Wells",
		State:     "Texas",
		Country:   "US",
		Elevation: 978,
		Latitude:  32.8334999084,
		Longitude: -98.1252975464,
		Timezone:  "America/Chicago",
	},
	"0TS8": {
		ICAO:      "0TS8",
		Name:      "Flying 'K' Ranch Ultralightport",
		City:      "Mercedes",
		State:     "Texas",
		Country:   "US",
		Elevation: 81,
		Latitude:  26.1504001617,
		Longitude: -97.8835983276,
		Timezone:  "America/Chicago",
	},
	"0TS9": {
		ICAO:      "0TS9",
		Name:      "Jim Roach Field",
		City:      "Niederwald",
		State:     "Texas",
		Country:   "US",
		Elevation: 600,
		Latitude:  30.015499115,
		Longitude: -97.737197876,
		Timezone:  "America/Chicago",
	},
	"0TX0": {
		ICAO:      "0TX0",
		Name:      "Nassau Bay Airport",
		City:      "Granbury",
		State:     "Texas",
		Country:   "US",
		Elevation: 714,
		Latitude:  32.4196014404,
		Longitude: -97.7127990723,
		Timezone:  "America/Chicago",
	},
	"0TX1": {
		ICAO:      "0TX1",
		Name:      "Pecan Plantation Airport",
		City:      "Granbury",
		State:     "Texas",
		Country:   "US",
		Elevation: 710,
		Latitude:  32.3540000916,
		Longitude: -97.676399231,
		Timezone:  "America/Chicago",
	},
	"0TX5": {
		ICAO:      "0TX5",
		Name:      "Shiloh Airport",
		City:      "Flower Mound",
		State:     "Texas",
		Country:   "US",
		Elevation: 635,
		Latitude:  33.0239982605,
		Longitude: -97.1196975708,
		Timezone:  "America/Chicago",
	},
	"0TX6": {
		ICAO:      "0TX6",
		Name:      "Elm Creek Airpark",
		City:      "Seguin",
		State:     "Texas",
		Country:   "US",
		Elevation: 550,
		Latitude:  29.5051994324,
		Longitude: -97.9969024658,
		Timezone:  "America/Chicago",
	},
	"0TX7": {
		ICAO:      "0TX7",
		Name:      "Lazy K Acres Airport",
		City:      "Grapevine",
		State:     "Texas",
		Country:   "US",
		Elevation: 700,
		Latitude:  32.9500999451,
		Longitude: -97.1502990723,
		Timezone:  "America/Chicago",
	},
	"0TX8": {
		ICAO:      "0TX8",
		Name:      "Jacobia Field",
		City:      "Greenville",
		State:     "Texas",
		Country:   "US",
		Elevation: 570,
		Latitude:  33.1804008484,
		Longitude: -96.043296814,
		Timezone:  "America/Chicago",
	},
	"0TX9": {
		ICAO:      "0TX9",
		Name:      "Card Aerodrome",
		City:      "Greenville",
		State:     "Texas",
		Country:   "US",
		Elevation: 540,
		Latitude:  33.2500991821,
		Longitude: -96.116897583,
		Timezone:  "America/Chicago",
	},
	"0VA1": {
		ICAO:      "0VA1",
		Name:      "Johnson Fox Field",
		City:      "Bedford",
		State:     "Virginia",
		Country:   "US",
		Elevation: 900,
		Latitude:  37.2335014343,
		Longitude: -79.5852966309,
		Timezone:  "America/New_York",
	},
	"0VA2": {
		ICAO:      "0VA2",
		Name:      "Riverwood Airport",
		City:      "Iron Gate",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1020,
		Latitude:  37.7965011597,
		Longitude: -79.7667007446,
		Timezone:  "America/New_York",
	},
	"0VA3": {
		ICAO:      "0VA3",
		Name:      "Hunt Airport",
		City:      "Manquin",
		State:     "Virginia",
		Country:   "US",
		Elevation: 150,
		Latitude:  37.6987991333,
		Longitude: -77.0952987671,
		Timezone:  "America/New_York",
	},
	"0VA4": {
		ICAO:      "0VA4",
		Name:      "Worley Field",
		City:      "Rocky Mount",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1219,
		Latitude:  36.9500999451,
		Longitude: -79.8795013428,
		Timezone:  "America/New_York",
	},
	"0VA5": {
		ICAO:      "0VA5",
		Name:      "Coffman Field",
		City:      "Richmond",
		State:     "Virginia",
		Country:   "US",
		Elevation: 280,
		Latitude:  37.6941443722,
		Longitude: -77.6608943939,
		Timezone:  "America/New_York",
	},
	"0VA7": {
		ICAO:      "0VA7",
		Name:      "Flatrock Air Strip",
		City:      "Richmond",
		State:     "Virginia",
		Country:   "US",
		Elevation: 425,
		Latitude:  37.5232009888,
		Longitude: -77.8193969727,
		Timezone:  "America/New_York",
	},
	"0VA8": {
		ICAO:      "0VA8",
		Name:      "Jayarz Airport",
		City:      "Mannboro",
		State:     "Virginia",
		Country:   "US",
		Elevation: 210,
		Latitude:  37.2858009338,
		Longitude: -77.7911987305,
		Timezone:  "America/New_York",
	},
	"0VA9": {
		ICAO:      "0VA9",
		Name:      "Handy Strip",
		City:      "Gloucester Court House",
		State:     "Virginia",
		Country:   "US",
		Elevation: 8,
		Latitude:  37.4473991394,
		Longitude: -76.4537963867,
		Timezone:  "America/New_York",
	},
	"0VI1": {
		ICAO:      "0VI1",
		Name:      "Brooklyn Airport",
		City:      "Manquin",
		State:     "Virginia",
		Country:   "US",
		Elevation: 145,
		Latitude:  37.6888999939,
		Longitude: -77.1014022827,
		Timezone:  "America/New_York",
	},
	"0WA1": {
		ICAO:      "0WA1",
		Name:      "Dwight Field",
		City:      "Chehalis",
		State:     "Washington",
		Country:   "US",
		Elevation: 350,
		Latitude:  46.617099762,
		Longitude: -122.8259963989,
		Timezone:  "America/Los_Angeles",
	},
	"0WA2": {
		ICAO:      "0WA2",
		Name:      "Curtis Airport",
		City:      "Curtis",
		State:     "Washington",
		Country:   "US",
		Elevation: 254,
		Latitude:  46.5904006958,
		Longitude: -123.1050033569,
		Timezone:  "America/Los_Angeles",
	},
	"0WI1": {
		ICAO:      "0WI1",
		Name:      "Mort's Landing Airport",
		City:      "Comstock",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1255,
		Latitude:  45.5071983337,
		Longitude: -92.1418991089,
		Timezone:  "America/Chicago",
	},
	"0WI2": {
		ICAO:      "0WI2",
		Name:      "Doering Farms Airfield",
		City:      "Monches",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 910,
		Latitude:  43.2014007568,
		Longitude: -88.3333969116,
		Timezone:  "America/Chicago",
	},
	"0WI4": {
		ICAO:      "0WI4",
		Name:      "Snow Crest Ranch Airport",
		City:      "Montello",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 810,
		Latitude:  43.8293991089,
		Longitude: -89.3526000977,
		Timezone:  "America/Chicago",
	},
	"0WI5": {
		ICAO:      "0WI5",
		Name:      "Crash In International Airport",
		City:      "Husher",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 755,
		Latitude:  42.7933998108,
		Longitude: -87.8906021118,
		Timezone:  "America/Chicago",
	},
	"0WI6": {
		ICAO:      "0WI6",
		Name:      "Simandl Field/Private Airport",
		City:      "Muskego",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 805,
		Latitude:  42.8736000061,
		Longitude: -88.1578979492,
		Timezone:  "America/Chicago",
	},
	"0WI7": {
		ICAO:      "0WI7",
		Name:      "With Wings And A Halo Airport",
		City:      "Neenah",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 760,
		Latitude:  44.1335983276,
		Longitude: -88.4992980957,
		Timezone:  "America/Chicago",
	},
	"0WI8": {
		ICAO:      "0WI8",
		Name:      "Oconomowoc Airport",
		City:      "Oconomowoc",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 885,
		Latitude:  43.1389007568,
		Longitude: -88.4722976685,
		Timezone:  "America/Chicago",
	},
	"0WI9": {
		ICAO:      "0WI9",
		Name:      "Mc Manus Hoonch-Na-Shee-Kaw Airport",
		City:      "Oregon",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 931,
		Latitude:  42.9314002991,
		Longitude: -89.3386993408,
		Timezone:  "America/Chicago",
	},
	"0WN0": {
		ICAO:      "0WN0",
		Name:      "Rucilla's Roost Airport",
		City:      "Sequim",
		State:     "Washington",
		Country:   "US",
		Elevation: 15,
		Latitude:  48.137298584,
		Longitude: -123.1129989624,
		Timezone:  "America/Los_Angeles",
	},
	"0WN1": {
		ICAO:      "0WN1",
		Name:      "Shangri-La Airport",
		City:      "Selah",
		State:     "Washington",
		Country:   "US",
		Elevation: 1500,
		Latitude:  46.7518997192,
		Longitude: -120.6019973755,
		Timezone:  "America/Los_Angeles",
	},
	"0WN2": {
		ICAO:      "0WN2",
		Name:      "Coopers Landing",
		City:      "Kennewick",
		State:     "Washington",
		Country:   "US",
		Elevation: 435,
		Latitude:  46.127778,
		Longitude: -119.015,
		Timezone:  "America/Los_Angeles",
	},
	"0WN5": {
		ICAO:      "0WN5",
		Name:      "Potts Field",
		City:      "North Cape",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 815,
		Latitude:  42.7997016907,
		Longitude: -88.0486984253,
		Timezone:  "America/Chicago",
	},
	"0WN9": {
		ICAO:      "0WN9",
		Name:      "Wings For Christ Airport",
		City:      "Omak",
		State:     "Washington",
		Country:   "US",
		Elevation: 1000,
		Latitude:  48.430464,
		Longitude: -119.486058,
		Timezone:  "America/Los_Angeles",
	},
	"0WY0": {
		ICAO:      "0WY0",
		Name:      "Freedom Air Ranch Airport",
		City:      "Freedom",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 5698,
		Latitude:  43.039722,
		Longitude: -111.038056,
		Timezone:  "America/Boise",
	},
	"0WY1": {
		ICAO:      "0WY1",
		Name:      "Dorsey Creek Ranch Airport",
		City:      "Basin",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4017,
		Latitude:  44.411111,
		Longitude: -108.163333,
		Timezone:  "America/Denver",
	},
	"0XA4": {
		ICAO:      "0XA4",
		Name:      "Seven C's Ranch Airport",
		City:      "Freer",
		State:     "Texas",
		Country:   "US",
		Elevation: 331,
		Latitude:  27.9969005585,
		Longitude: -98.8822021484,
		Timezone:  "America/Chicago",
	},
	"0XA5": {
		ICAO:      "0XA5",
		Name:      "74 Ranch Airport",
		City:      "Campbellton",
		State:     "Texas",
		Country:   "US",
		Elevation: 320,
		Latitude:  28.6849002838,
		Longitude: -98.3827972412,
		Timezone:  "America/Chicago",
	},
	"0XA7": {
		ICAO:      "0XA7",
		Name:      "Bar 3 Ranch Airport",
		City:      "Georgetown",
		State:     "Texas",
		Country:   "US",
		Elevation: 744,
		Latitude:  30.66270065,
		Longitude: -97.62509918,
		Timezone:  "America/Chicago",
	},
	"0XA8": {
		ICAO:      "0XA8",
		Name:      "Ohho Airport",
		City:      "Dripping Springs",
		State:     "Texas",
		Country:   "US",
		Elevation: 1340,
		Latitude:  30.2700004578,
		Longitude: -98.1082992554,
		Timezone:  "America/Chicago",
	},
	"0XS0": {
		ICAO:      "0XS0",
		Name:      "Diamond H Ranch Airport",
		City:      "Catarina",
		State:     "Texas",
		Country:   "US",
		Elevation: 511,
		Latitude:  28.4002990723,
		Longitude: -99.5587005615,
		Timezone:  "America/Chicago",
	},
	"0XS1": {
		ICAO:      "0XS1",
		Name:      "Rodgers Roost Airport",
		City:      "El Campo",
		State:     "Texas",
		Country:   "US",
		Elevation: 129,
		Latitude:  29.2877998352,
		Longitude: -96.3497009277,
		Timezone:  "America/Chicago",
	},
	"0XS2": {
		ICAO:      "0XS2",
		Name:      "John Fields Ranch Airport",
		City:      "Sonora",
		State:     "Texas",
		Country:   "US",
		Elevation: 2342,
		Latitude:  30.5548992157,
		Longitude: -100.533996582,
		Timezone:  "America/Chicago",
	},
	"0XS6": {
		ICAO:      "0XS6",
		Name:      "Lakeside Beach Airport",
		City:      "Spicewood",
		State:     "Texas",
		Country:   "US",
		Elevation: 800,
		Latitude:  30.5009994507,
		Longitude: -98.1470031738,
		Timezone:  "America/Chicago",
	},
	"0XS7": {
		ICAO:      "0XS7",
		Name:      "Anacacho Ranch Airport",
		City:      "Spofford",
		State:     "Texas",
		Country:   "US",
		Elevation: 1067,
		Latitude:  29.1919002533,
		Longitude: -100.2750015259,
		Timezone:  "America/Chicago",
	},
	"0XS8": {
		ICAO:      "0XS8",
		Name:      "Dunbar Ranch Airport",
		City:      "Spofford",
		State:     "Texas",
		Country:   "US",
		Elevation: 958,
		Latitude:  29.0760993958,
		Longitude: -100.375,
		Timezone:  "America/Chicago",
	},
	"0XS9": {
		ICAO:      "0XS9",
		Name:      "French Field",
		City:      "Bullard",
		State:     "Texas",
		Country:   "US",
		Elevation: 540,
		Latitude:  32.134042,
		Longitude: -95.339583,
		Timezone:  "America/Chicago",
	},
	"10AK": {
		ICAO:      "10AK",
		Name:      "Hunt Strip",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 200,
		Latitude:  61.5918998718,
		Longitude: -149.6779937744,
		Timezone:  "America/Anchorage",
	},
	"10AZ": {
		ICAO:      "10AZ",
		Name:      "Pierce Airport",
		City:      "Buckeye",
		State:     "Arizona",
		Country:   "US",
		Elevation: 860,
		Latitude:  33.3664016724,
		Longitude: -112.6179962158,
		Timezone:  "America/Phoenix",
	},
	"10CL": {
		ICAO:      "10CL",
		Name:      "Joe Gottlieb Field",
		City:      "Bakersfield",
		State:     "California",
		Country:   "US",
		Elevation: 347,
		Latitude:  35.3680000305,
		Longitude: -119.1949996948,
		Timezone:  "America/Los_Angeles",
	},
	"10CO": {
		ICAO:      "10CO",
		Name:      "Silkman Farms Inc. Airport",
		City:      "Burlington",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4000,
		Latitude:  39.45470047,
		Longitude: -102.212997437,
		Timezone:  "America/Denver",
	},
	"10FA": {
		ICAO:      "10FA",
		Name:      "Flying Baron Estates Airport",
		City:      "Leesburg",
		State:     "Florida",
		Country:   "US",
		Elevation: 75,
		Latitude:  28.7975006104,
		Longitude: -81.9358978271,
		Timezone:  "America/New_York",
	},
	"10FD": {
		ICAO:      "10FD",
		Name:      "Seven Feathers Airport",
		City:      "Dunnellon",
		State:     "Florida",
		Country:   "US",
		Elevation: 68,
		Latitude:  29.0065994263,
		Longitude: -82.4046020508,
		Timezone:  "America/New_York",
	},
	"10GA": {
		ICAO:      "10GA",
		Name:      "Beaverbrook Aerodrome",
		City:      "Griffin",
		State:     "Georgia",
		Country:   "US",
		Elevation: 910,
		Latitude:  33.3250999451,
		Longitude: -84.2983016968,
		Timezone:  "America/New_York",
	},
	"10IA": {
		ICAO:      "10IA",
		Name:      "Flyers Airport",
		City:      "Waterloo",
		State:     "Iowa",
		Country:   "US",
		Elevation: 830,
		Latitude:  42.4166984558,
		Longitude: -92.2501983643,
		Timezone:  "America/Chicago",
	},
	"10ID": {
		ICAO:      "10ID",
		Name:      "Flat Top Airstrip",
		City:      "Muldoon",
		State:     "Idaho",
		Country:   "US",
		Elevation: 5841,
		Latitude:  43.4944000244,
		Longitude: -113.9219970703,
		Timezone:  "America/Boise",
	},
	"10IN": {
		ICAO:      "10IN",
		Name:      "Squires Airport",
		City:      "Connersville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 1000,
		Latitude:  39.7183990479,
		Longitude: -85.0488967896,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"10LA": {
		ICAO:      "10LA",
		Name:      "Industrial Helicopters Inc. Airport",
		City:      "Scott",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 35,
		Latitude:  30.2474002838,
		Longitude: -92.0960006714,
		Timezone:  "America/Chicago",
	},
	"10LS": {
		ICAO:      "10LS",
		Name:      "G & J Fly A Way Ultralightport",
		City:      "Youngsville",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 20,
		Latitude:  30.0463008881,
		Longitude: -91.9945983887,
		Timezone:  "America/Chicago",
	},
	"10MO": {
		ICAO:      "10MO",
		Name:      "Eckerts Airstrip",
		City:      "Williamsburg",
		State:     "Missouri",
		Country:   "US",
		Elevation: 839,
		Latitude:  38.9145011902,
		Longitude: -91.7042999268,
		Timezone:  "America/Chicago",
	},
	"10MT": {
		ICAO:      "10MT",
		Name:      "Hoolie Airport",
		City:      "Haxby",
		State:     "Montana",
		Country:   "US",
		Elevation: 2526,
		Latitude:  47.8333015442,
		Longitude: -106.4840011597,
		Timezone:  "America/Denver",
	},
	"10NC": {
		ICAO:      "10NC",
		Name:      "Johnston Farm Airport",
		City:      "Jackson",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 60,
		Latitude:  36.3431015015,
		Longitude: -77.5093994141,
		Timezone:  "America/New_York",
	},
	"10ND": {
		ICAO:      "10ND",
		Name:      "Smith Strip",
		City:      "Halliday",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 2200,
		Latitude:  47.1875,
		Longitude: -102.3619995117,
		Timezone:  "America/Denver",
	},
	"10OH": {
		ICAO:      "10OH",
		Name:      "Zoellner Airport",
		City:      "Huron",
		State:     "Ohio",
		Country:   "US",
		Elevation: 617,
		Latitude:  41.4028015137,
		Longitude: -82.6290969849,
		Timezone:  "America/New_York",
	},
	"10OK": {
		ICAO:      "10OK",
		Name:      "Hickory Hills Airport",
		City:      "Norman",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1175,
		Latitude:  35.1666984558,
		Longitude: -97.2170028687,
		Timezone:  "America/Chicago",
	},
	"10OR": {
		ICAO:      "10OR",
		Name:      "Stevens Mountain Airport",
		City:      "Vernonia",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1000,
		Latitude:  45.7892990112,
		Longitude: -123.2320022583,
		Timezone:  "America/Los_Angeles",
	},
	"10TE": {
		ICAO:      "10TE",
		Name:      "Gottwald Field",
		City:      "Harwood",
		State:     "Texas",
		Country:   "US",
		Elevation: 440,
		Latitude:  29.6669006348,
		Longitude: -97.5002975464,
		Timezone:  "America/Chicago",
	},
	"10TN": {
		ICAO:      "10TN",
		Name:      "Flatwood Field",
		City:      "Lebanon",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 600,
		Latitude:  36.0718994141,
		Longitude: -86.3544006348,
		Timezone:  "America/Chicago",
	},
	"10TS": {
		ICAO:      "10TS",
		Name:      "Saint's Strip Ultralightport",
		City:      "Marion",
		State:     "Texas",
		Country:   "US",
		Elevation: 980,
		Latitude:  29.5524997711,
		Longitude: -98.1031036377,
		Timezone:  "America/Chicago",
	},
	"10VA": {
		ICAO:      "10VA",
		Name:      "Nashs Airport",
		City:      "Appomattox",
		State:     "Virginia",
		Country:   "US",
		Elevation: 785,
		Latitude:  37.3862991333,
		Longitude: -78.8131027222,
		Timezone:  "America/New_York",
	},
	"10WA": {
		ICAO:      "10WA",
		Name:      "Mullan Hill Airport",
		City:      "Spokane",
		State:     "Washington",
		Country:   "US",
		Elevation: 2310,
		Latitude:  47.554599762,
		Longitude: -117.4130020142,
		Timezone:  "America/Los_Angeles",
	},
	"10WI": {
		ICAO:      "10WI",
		Name:      "Buds Landing Airport",
		City:      "Pound",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 850,
		Latitude:  45.1157989502,
		Longitude: -88.1894989014,
		Timezone:  "America/Chicago",
	},
	"10WY": {
		ICAO:      "10WY",
		Name:      "Willow Creek Ranch Airport",
		City:      "Kaycee",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 5521,
		Latitude:  43.423333,
		Longitude: -106.823611,
		Timezone:  "America/Denver",
	},
	"10XS": {
		ICAO:      "10XS",
		Name:      "Flying S Air Ranch Airport",
		City:      "Brownwood",
		State:     "Texas",
		Country:   "US",
		Elevation: 1447,
		Latitude:  31.779499054,
		Longitude: -98.9093017578,
		Timezone:  "America/Chicago",
	},
	"11AK": {
		ICAO:      "11AK",
		Name:      "Rocking T. Ranch Airport",
		City:      "Delta Junction",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1190,
		Latitude:  63.9995994568,
		Longitude: -145.501998901,
		Timezone:  "America/Anchorage",
	},
	"11AZ": {
		ICAO:      "11AZ",
		Name:      "Whetstone Airport",
		City:      "Huachuca City",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4250,
		Latitude:  31.6837005615,
		Longitude: -110.2900009155,
		Timezone:  "America/Phoenix",
	},
	"11CA": {
		ICAO:      "11CA",
		Name:      "Turner Field",
		City:      "Fowler",
		State:     "California",
		Country:   "US",
		Elevation: 300,
		Latitude:  36.6733016968,
		Longitude: -119.7220001221,
		Timezone:  "America/Los_Angeles",
	},
	"11CL": {
		ICAO:      "11CL",
		Name:      "Hansen Airport",
		City:      "Adelanto",
		State:     "California",
		Country:   "US",
		Elevation: 2885,
		Latitude:  34.6300010681,
		Longitude: -117.6529998779,
		Timezone:  "America/Los_Angeles",
	},
	"11FA": {
		ICAO:      "11FA",
		Name:      "B & L Farms Airport",
		City:      "Homestead",
		State:     "Florida",
		Country:   "US",
		Elevation: 9,
		Latitude:  25.5837001801,
		Longitude: -80.4998016357,
		Timezone:  "America/New_York",
	},
	"11FL": {
		ICAO:      "11FL",
		Name:      "Williams Field",
		City:      "Middleburg",
		State:     "Florida",
		Country:   "US",
		Elevation: 88,
		Latitude:  30.0429992676,
		Longitude: -81.880897522,
		Timezone:  "America/New_York",
	},
	"11ID": {
		ICAO:      "11ID",
		Name:      "Riddle Airport",
		City:      "Riddle",
		State:     "Idaho",
		Country:   "US",
		Elevation: 5356,
		Latitude:  42.1855010986,
		Longitude: -116.1129989624,
		Timezone:  "America/Boise",
	},
	"11II": {
		ICAO:      "11II",
		Name:      "Atterbury Field (Camp Atterbury)",
		City:      "Nineveh",
		State:     "Indiana",
		Country:   "US",
		Elevation: 710,
		Latitude:  39.3417015076,
		Longitude: -86.0305023193,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"11IN": {
		ICAO:      "11IN",
		Name:      "Jr's Airport",
		City:      "Danville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 950,
		Latitude:  39.818901062,
		Longitude: -86.5381011963,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"11KS": {
		ICAO:      "11KS",
		Name:      "Sheller's Airport",
		City:      "Tonganoxie",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1090,
		Latitude:  39.1166992188,
		Longitude: -95.1489028931,
		Timezone:  "America/Chicago",
	},
	"11LL": {
		ICAO:      "11LL",
		Name:      "Thacker Airport",
		City:      "Chenoa",
		State:     "Illinois",
		Country:   "US",
		Elevation: 720,
		Latitude:  40.7230987549,
		Longitude: -88.7630996704,
		Timezone:  "America/Chicago",
	},
	"11LS": {
		ICAO:      "11LS",
		Name:      "Jasmine Hill Airport",
		City:      "Glenmora",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 120,
		Latitude:  30.9759998322,
		Longitude: -92.5690002441,
		Timezone:  "America/Chicago",
	},
	"11MA": {
		ICAO:      "11MA",
		Name:      "Bulljump Airport",
		City:      "Wareham",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 60,
		Latitude:  41.8073005676,
		Longitude: -70.7583999634,
		Timezone:  "America/New_York",
	},
	"11MD": {
		ICAO:      "11MD",
		Name:      "Tim's Airport",
		City:      "Salisbury",
		State:     "Maryland",
		Country:   "US",
		Elevation: 42,
		Latitude:  38.4333000183,
		Longitude: -75.6277999878,
		Timezone:  "America/New_York",
	},
	"11MO": {
		ICAO:      "11MO",
		Name:      "Redgate Ranch Airport",
		City:      "Williamsburg",
		State:     "Missouri",
		Country:   "US",
		Elevation: 820,
		Latitude:  38.9016990662,
		Longitude: -91.6352005005,
		Timezone:  "America/Chicago",
	},
	"11NE": {
		ICAO:      "11NE",
		Name:      "Kaan Airport",
		City:      "Harrison",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 4650,
		Latitude:  42.4425010681,
		Longitude: -103.9160003662,
		Timezone:  "America/Denver",
	},
	"11NK": {
		ICAO:      "11NK",
		Name:      "Hop House Airpark",
		City:      "Fort Plain",
		State:     "New York",
		Country:   "US",
		Elevation: 530,
		Latitude:  42.913898468,
		Longitude: -74.6117019653,
		Timezone:  "America/New_York",
	},
	"11NY": {
		ICAO:      "11NY",
		Name:      "Saikkonen Airport",
		City:      "Spencer",
		State:     "New York",
		Country:   "US",
		Elevation: 969,
		Latitude:  42.2089996338,
		Longitude: -76.4496994019,
		Timezone:  "America/New_York",
	},
	"11OA": {
		ICAO:      "11OA",
		Name:      "Hide-A-Way Hills Resort Airport",
		City:      "Bremen",
		State:     "Ohio",
		Country:   "US",
		Elevation: 780,
		Latitude:  39.6431007385,
		Longitude: -82.466796875,
		Timezone:  "America/New_York",
	},
	"11OG": {
		ICAO:      "11OG",
		Name:      "Unity Airport",
		City:      "Unity",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3975,
		Latitude:  44.4513015747,
		Longitude: -118.18699646,
		Timezone:  "America/Los_Angeles",
	},
	"11OK": {
		ICAO:      "11OK",
		Name:      "Beefor Ranch Airport",
		City:      "Purcell",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1150,
		Latitude:  35.0722999573,
		Longitude: -97.3835983276,
		Timezone:  "America/Chicago",
	},
	"11OR": {
		ICAO:      "11OR",
		Name:      "Holiday Sky Ranch Airport",
		City:      "Sutherlin",
		State:     "Oregon",
		Country:   "US",
		Elevation: 510,
		Latitude:  43.3935012817,
		Longitude: -123.2850036621,
		Timezone:  "America/Los_Angeles",
	},
	"11PN": {
		ICAO:      "11PN",
		Name:      "Eakin Airport",
		City:      "Cranberry",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1489,
		Latitude:  41.3525,
		Longitude: -79.732778,
		Timezone:  "America/New_York",
	},
	"11TE": {
		ICAO:      "11TE",
		Name:      "Flying M Ranch Airport",
		City:      "Grandview",
		State:     "Texas",
		Country:   "US",
		Elevation: 700,
		Latitude:  32.3000984192,
		Longitude: -97.204498291,
		Timezone:  "America/Chicago",
	},
	"11TN": {
		ICAO:      "11TN",
		Name:      "Salem Field",
		City:      "Clarksville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 425,
		Latitude:  36.4749984741,
		Longitude: -87.3666992188,
		Timezone:  "America/Chicago",
	},
	"11TS": {
		ICAO:      "11TS",
		Name:      "Pt Enterprise D&W Ranch Airport",
		City:      "Mexia",
		State:     "Texas",
		Country:   "US",
		Elevation: 526,
		Latitude:  31.6578998566,
		Longitude: -96.4113998413,
		Timezone:  "America/Chicago",
	},
	"11TX": {
		ICAO:      "11TX",
		Name:      "Butler Airport",
		City:      "Sherman",
		State:     "Texas",
		Country:   "US",
		Elevation: 840,
		Latitude:  33.6668014526,
		Longitude: -96.6905975342,
		Timezone:  "America/Chicago",
	},
	"11VG": {
		ICAO:      "11VG",
		Name:      "Devil's Reach Landing Airport",
		City:      "Oak Grove",
		State:     "Virginia",
		Country:   "US",
		Elevation: 28,
		Latitude:  38.209400177,
		Longitude: -77.0006027222,
		Timezone:  "America/New_York",
	},
	"11WA": {
		ICAO:      "11WA",
		Name:      "Homeport Airport",
		City:      "Spokane",
		State:     "Washington",
		Country:   "US",
		Elevation: 2320,
		Latitude:  47.5237998962,
		Longitude: -117.4700012207,
		Timezone:  "America/Los_Angeles",
	},
	"12AR": {
		ICAO:      "12AR",
		Name:      "Gibbons Airport",
		City:      "Marvell",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 178,
		Latitude:  34.515444,
		Longitude: -90.973722,
		Timezone:  "America/Chicago",
	},
	"12AZ": {
		ICAO:      "12AZ",
		Name:      "Ranta Strip",
		City:      "Morristown",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1840,
		Latitude:  33.8539009094,
		Longitude: -112.6529998779,
		Timezone:  "America/Phoenix",
	},
	"12CA": {
		ICAO:      "12CA",
		Name:      "Faber Vineyards Airport",
		City:      "Lodi",
		State:     "California",
		Country:   "US",
		Elevation: 70,
		Latitude:  38.1096000671,
		Longitude: -121.1689987183,
		Timezone:  "America/Los_Angeles",
	},
	"12CL": {
		ICAO:      "12CL",
		Name:      "Newton Field",
		City:      "Stratford",
		State:     "California",
		Country:   "US",
		Elevation: 194,
		Latitude:  36.1568984985,
		Longitude: -119.8499984741,
		Timezone:  "America/Los_Angeles",
	},
	"12CO": {
		ICAO:      "12CO",
		Name:      "Omega 1 STOLport",
		City:      "Hotchkiss",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6409,
		Latitude:  38.8354988098,
		Longitude: -107.8150024414,
		Timezone:  "America/Denver",
	},
	"12FA": {
		ICAO:      "12FA",
		Name:      "Hi-Acres Airport",
		City:      "Clermont",
		State:     "Florida",
		Country:   "US",
		Elevation: 105,
		Latitude:  28.4785995483,
		Longitude: -81.7145004272,
		Timezone:  "America/New_York",
	},
	"12IA": {
		ICAO:      "12IA",
		Name:      "Plueger Airfield",
		City:      "Lemars",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1245,
		Latitude:  42.8004989624,
		Longitude: -96.137802124,
		Timezone:  "America/Chicago",
	},
	"12ID": {
		ICAO:      "12ID",
		Name:      "Flying B Ranch Landing Strip",
		City:      "Salmon",
		State:     "Idaho",
		Country:   "US",
		Elevation: 3647,
		Latitude:  44.9679985046,
		Longitude: -114.733001709,
		Timezone:  "America/Boise",
	},
	"12II": {
		ICAO:      "12II",
		Name:      "Pentecost Airport",
		City:      "Centerville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 1020,
		Latitude:  39.8562011719,
		Longitude: -84.9721984863,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"12IL": {
		ICAO:      "12IL",
		Name:      "Hawker Airport",
		City:      "Kankakee",
		State:     "Illinois",
		Country:   "US",
		Elevation: 620,
		Latitude:  41.2000007629,
		Longitude: -88.0167007446,
		Timezone:  "America/Chicago",
	},
	"12IN": {
		ICAO:      "12IN",
		Name:      "J & S Field",
		City:      "Darmstadt",
		State:     "Indiana",
		Country:   "US",
		Elevation: 465,
		Latitude:  38.118598938,
		Longitude: -87.6474990845,
		Timezone:  "America/Chicago",
	},
	"12KS": {
		ICAO:      "12KS",
		Name:      "Sooter Airport",
		City:      "Sedgwick",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1420,
		Latitude:  37.9352989197,
		Longitude: -97.4878005981,
		Timezone:  "America/Chicago",
	},
	"12KY": {
		ICAO:      "12KY",
		Name:      "John M. Foree Airport",
		City:      "New Castle",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 853,
		Latitude:  38.4603004456,
		Longitude: -85.1686019897,
		Timezone:  "America/New_York",
	},
	"12LA": {
		ICAO:      "12LA",
		Name:      "Kml Airport",
		City:      "Sulphur",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 20,
		Latitude:  30.2758998871,
		Longitude: -93.3573989868,
		Timezone:  "America/Chicago",
	},
	"12LL": {
		ICAO:      "12LL",
		Name:      "Lambdins Field",
		City:      "Wolf Lake",
		State:     "Illinois",
		Country:   "US",
		Elevation: 350,
		Latitude:  37.5402984619,
		Longitude: -89.4861984253,
		Timezone:  "America/Chicago",
	},
	"12LS": {
		ICAO:      "12LS",
		Name:      "Hains Airstrip",
		City:      "Rayne",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 20,
		Latitude:  30.1786003113,
		Longitude: -92.2850036621,
		Timezone:  "America/Chicago",
	},
	"12ME": {
		ICAO:      "12ME",
		Name:      "Ridgeview Airport",
		City:      "Searsmont",
		State:     "Maine",
		Country:   "US",
		Elevation: 240,
		Latitude:  44.3222999573,
		Longitude: -69.2072982788,
		Timezone:  "America/New_York",
	},
	"12MI": {
		ICAO:      "12MI",
		Name:      "John's Airport",
		City:      "Davison",
		State:     "Michigan",
		Country:   "US",
		Elevation: 820,
		Latitude:  43.051700592,
		Longitude: -83.491897583,
		Timezone:  "America/Detroit",
	},
	"12MO": {
		ICAO:      "12MO",
		Name:      "Ferguson Farms Airport",
		City:      "Windsor",
		State:     "Missouri",
		Country:   "US",
		Elevation: 990,
		Latitude:  38.5806007385,
		Longitude: -93.5947036743,
		Timezone:  "America/Chicago",
	},
	"12MS": {
		ICAO:      "12MS",
		Name:      "Green Acres Airport",
		City:      "Woodville",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 300,
		Latitude:  31.0930995941,
		Longitude: -91.3190002441,
		Timezone:  "America/Chicago",
	},
	"12MT": {
		ICAO:      "12MT",
		Name:      "Lincolns Field",
		City:      "Hingham",
		State:     "Montana",
		Country:   "US",
		Elevation: 2931,
		Latitude:  48.7742004395,
		Longitude: -110.4420013428,
		Timezone:  "America/Denver",
	},
	"12NC": {
		ICAO:      "12NC",
		Name:      "Atlantic Field Mcolf Airport",
		City:      "Atlantic",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 20,
		Latitude:  34.8833007812,
		Longitude: -76.3499984741,
		Timezone:  "America/New_York",
	},
	"12ND": {
		ICAO:      "12ND",
		Name:      "Pietschtree Airstrip",
		City:      "Burlington",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1566,
		Latitude:  48.29529953,
		Longitude: -101.4270019531,
		Timezone:  "America/Chicago",
	},
	"12NE": {
		ICAO:      "12NE",
		Name:      "Reisig Brothers Airport",
		City:      "Scottsbluff",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3985,
		Latitude:  41.9066009521,
		Longitude: -103.6999969482,
		Timezone:  "America/Denver",
	},
	"12NH": {
		ICAO:      "12NH",
		Name:      "Gile Pond Airport",
		City:      "Sanbornton",
		State:     "New Hampshire",
		Country:   "US",
		Elevation: 485,
		Latitude:  43.495278,
		Longitude: -71.654444,
		Timezone:  "America/New_York",
	},
	"12NK": {
		ICAO:      "12NK",
		Name:      "Westport Airport",
		City:      "Westport",
		State:     "New York",
		Country:   "US",
		Elevation: 276,
		Latitude:  44.1584014893,
		Longitude: -73.4328994751,
		Timezone:  "America/New_York",
	},
	"12NM": {
		ICAO:      "12NM",
		Name:      "Columbus Stockyards Airport",
		City:      "Columbus",
		State:     "New Mexico",
		Country:   "US",
		Elevation: 4017,
		Latitude:  31.7917995453,
		Longitude: -107.6380004883,
		Timezone:  "America/Denver",
	},
	"12OH": {
		ICAO:      "12OH",
		Name:      "Mcgregor Airfield",
		City:      "Brookville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 940,
		Latitude:  39.8348007202,
		Longitude: -84.3347015381,
		Timezone:  "America/New_York",
	},
	"12OK": {
		ICAO:      "12OK",
		Name:      "Expressway Airpark",
		City:      "Oklahoma City",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1070,
		Latitude:  35.5413017273,
		Longitude: -97.4531021118,
		Timezone:  "America/Chicago",
	},
	"12OR": {
		ICAO:      "12OR",
		Name:      "Skinner Ranch Airport",
		City:      "Jordan Valley",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4273,
		Latitude:  42.9513015747,
		Longitude: -117.2819976807,
		Timezone:  "America/Boise",
	},
	"12SC": {
		ICAO:      "12SC",
		Name:      "Over the Hill Airport",
		City:      "Elgin",
		State:     "South Carolina",
		Country:   "US",
		Elevation: 250,
		Latitude:  34.1413993835,
		Longitude: -80.7466964722,
		Timezone:  "America/New_York",
	},
	"12TE": {
		ICAO:      "12TE",
		Name:      "Curtis Ranch Field",
		City:      "Brady",
		State:     "Texas",
		Country:   "US",
		Elevation: 1740,
		Latitude:  31.3167991638,
		Longitude: -99.2337036133,
		Timezone:  "America/Chicago",
	},
	"12TN": {
		ICAO:      "12TN",
		Name:      "Riley Creek Airport",
		City:      "Kingston",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 750,
		Latitude:  35.8236999512,
		Longitude: -84.5388031006,
		Timezone:  "America/New_York",
	},
	"12TS": {
		ICAO:      "12TS",
		Name:      "Blo Airport",
		City:      "Mineral Wells",
		State:     "Texas",
		Country:   "US",
		Elevation: 1050,
		Latitude:  32.9085006714,
		Longitude: -97.9773025513,
		Timezone:  "America/Chicago",
	},
	"12TX": {
		ICAO:      "12TX",
		Name:      "Griffin Airport",
		City:      "Bertram",
		State:     "Texas",
		Country:   "US",
		Elevation: 1235,
		Latitude:  30.8213996887,
		Longitude: -98.0805969238,
		Timezone:  "America/Chicago",
	},
	"12VA": {
		ICAO:      "12VA",
		Name:      "Hop-Along Airport",
		City:      "Clifton Forge",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1060,
		Latitude:  37.7881011963,
		Longitude: -79.7677993774,
		Timezone:  "America/New_York",
	},
	"12WA": {
		ICAO:      "12WA",
		Name:      "Fowlers Nw 40 Airport",
		City:      "Spokane",
		State:     "Washington",
		Country:   "US",
		Elevation: 2313,
		Latitude:  47.5206985474,
		Longitude: -117.4629974365,
		Timezone:  "America/Los_Angeles",
	},
	"12WT": {
		ICAO:      "12WT",
		Name:      "Walker Airport",
		City:      "Palouse",
		State:     "Washington",
		Country:   "US",
		Elevation: 2450,
		Latitude:  46.911,
		Longitude: -117.218667,
		Timezone:  "America/Los_Angeles",
	},
	"12WY": {
		ICAO:      "12WY",
		Name:      "Black Diamond Airport",
		City:      "Burlington",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4400,
		Latitude:  44.45,
		Longitude: -108.384667,
		Timezone:  "America/Denver",
	},
	"12XS": {
		ICAO:      "12XS",
		Name:      "Mc Croskey Field",
		City:      "Butlerville",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 253,
		Latitude:  34.95470047,
		Longitude: -91.8371963501,
		Timezone:  "America/Chicago",
	},
	"13AK": {
		ICAO:      "13AK",
		Name:      "Satterbergs Airport",
		City:      "Houston",
		State:     "Alaska",
		Country:   "US",
		Elevation: 350,
		Latitude:  61.6547012329,
		Longitude: -149.8999938965,
		Timezone:  "America/Anchorage",
	},
	"13AZ": {
		ICAO:      "13AZ",
		Name:      "Waltenberry Field Ultralightport",
		City:      "Tonopah",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1213,
		Latitude:  33.535,
		Longitude: -112.853472222,
		Timezone:  "America/Phoenix",
	},
	"13CL": {
		ICAO:      "13CL",
		Name:      "Maine Prairie Airport",
		City:      "Dixon",
		State:     "California",
		Country:   "US",
		Elevation: 40,
		Latitude:  38.3813018799,
		Longitude: -121.8239974976,
		Timezone:  "America/Los_Angeles",
	},
	"13FA": {
		ICAO:      "13FA",
		Name:      "Earle Airpark",
		City:      "Hastings",
		State:     "Florida",
		Country:   "US",
		Elevation: 25,
		Latitude:  29.6944007874,
		Longitude: -81.5137023926,
		Timezone:  "America/New_York",
	},
	"13GA": {
		ICAO:      "13GA",
		Name:      "Oak Ridge Plantation Airport",
		City:      "Sylvester",
		State:     "Georgia",
		Country:   "US",
		Elevation: 350,
		Latitude:  31.3666000366,
		Longitude: -83.7548980713,
		Timezone:  "America/New_York",
	},
	"13GE": {
		ICAO:      "13GE",
		Name:      "Holt Airpark",
		City:      "Oak Park",
		State:     "Georgia",
		Country:   "US",
		Elevation: 280,
		Latitude:  32.3608016968,
		Longitude: -82.3180999756,
		Timezone:  "America/New_York",
	},
	"13IA": {
		ICAO:      "13IA",
		Name:      "Hillside Stables Airport",
		City:      "Sabula",
		State:     "Iowa",
		Country:   "US",
		Elevation: 760,
		Latitude:  42.0457000732,
		Longitude: -90.1911010742,
		Timezone:  "America/Chicago",
	},
	"13ID": {
		ICAO:      "13ID",
		Name:      "Silva Ranch Airport",
		City:      "Stanley",
		State:     "Idaho",
		Country:   "US",
		Elevation: 6400,
		Latitude:  44.2666015625,
		Longitude: -115.0009994507,
		Timezone:  "America/Boise",
	},
	"13IN": {
		ICAO:      "13IN",
		Name:      "Gage Airport",
		City:      "Decatur",
		State:     "Indiana",
		Country:   "US",
		Elevation: 814,
		Latitude:  40.8152999878,
		Longitude: -84.927696228,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"13KS": {
		ICAO:      "13KS",
		Name:      "Daniel's Landing Airport",
		City:      "Andover",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1325,
		Latitude:  37.6977996826,
		Longitude: -97.1102981567,
		Timezone:  "America/Chicago",
	},
	"13KY": {
		ICAO:      "13KY",
		Name:      "Miles Airport",
		City:      "Owensboro",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 407,
		Latitude:  37.7056007385,
		Longitude: -87.1560974121,
		Timezone:  "America/Chicago",
	},
	"13LS": {
		ICAO:      "13LS",
		Name:      "Tee Brad's Airport",
		City:      "Kaplan",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 12,
		Latitude:  29.9939002991,
		Longitude: -92.385597229,
		Timezone:  "America/Chicago",
	},
	"13MA": {
		ICAO:      "13MA",
		Name:      "Metropolitan Airport",
		City:      "Palmer",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 418,
		Latitude:  42.2233009338,
		Longitude: -72.3114013672,
		Timezone:  "America/New_York",
	},
	"13ME": {
		ICAO:      "13ME",
		Name:      "Socatean Bay Airport",
		City:      "Rockwood",
		State:     "Maine",
		Country:   "US",
		Elevation: 1179,
		Latitude:  45.7764015198,
		Longitude: -69.8093032837,
		Timezone:  "America/New_York",
	},
	"13MN": {
		ICAO:      "13MN",
		Name:      "Melby Airport",
		City:      "Ashby",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1250,
		Latitude:  46.1007995605,
		Longitude: -95.8313980103,
		Timezone:  "America/Chicago",
	},
	"13MO": {
		ICAO:      "13MO",
		Name:      "Domeyer Airport",
		City:      "Lenox",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1260,
		Latitude:  37.676399231,
		Longitude: -91.7064971924,
		Timezone:  "America/Chicago",
	},
	"13MS": {
		ICAO:      "13MS",
		Name:      "Greener Pastures Airpark",
		City:      "Carriere",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 180,
		Latitude:  30.6333007813,
		Longitude: -89.6639022827,
		Timezone:  "America/Chicago",
	},
	"13MT": {
		ICAO:      "13MT",
		Name:      "Sorenson Airport",
		City:      "Kremlin",
		State:     "Montana",
		Country:   "US",
		Elevation: 2873,
		Latitude:  48.5208015442,
		Longitude: -110.1129989624,
		Timezone:  "America/Denver",
	},
	"13NC": {
		ICAO:      "13NC",
		Name:      "Oak Grove Mcolf Airport",
		City:      "Pollocksville",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 27,
		Latitude:  35.0335006714,
		Longitude: -77.2497024536,
		Timezone:  "America/New_York",
	},
	"13NE": {
		ICAO:      "13NE",
		Name:      "Hock Airport",
		City:      "Culbertson",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2777,
		Latitude:  40.3166999817,
		Longitude: -100.8089981079,
		Timezone:  "America/Chicago",
	},
	"13NM": {
		ICAO:      "13NM",
		Name:      "Beaverhead Airstrip",
		City:      "Silver City",
		State:     "New Mexico",
		Country:   "US",
		Elevation: 7378,
		Latitude:  33.4194984436,
		Longitude: -108.138999939,
		Timezone:  "America/Denver",
	},
	"13OH": {
		ICAO:      "13OH",
		Name:      "Rose Field",
		City:      "Jewell",
		State:     "Ohio",
		Country:   "US",
		Elevation: 728,
		Latitude:  41.3302993774,
		Longitude: -84.2983016968,
		Timezone:  "America/New_York",
	},
	"13OR": {
		ICAO:      "13OR",
		Name:      "Aubrey Mountain Airstrip",
		City:      "Oakridge",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1680,
		Latitude:  43.7406997681,
		Longitude: -122.4260025024,
		Timezone:  "America/Los_Angeles",
	},
	"13PA": {
		ICAO:      "13PA",
		Name:      "Rigrtona Airport",
		City:      "Tidioute",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1703,
		Latitude:  41.682598114,
		Longitude: -79.452003479,
		Timezone:  "America/New_York",
	},
	"13PN": {
		ICAO:      "13PN",
		Name:      "Richland Acres Ultralightport",
		City:      "Quakertown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 550,
		Latitude:  40.4542999268,
		Longitude: -75.3046035767,
		Timezone:  "America/New_York",
	},
	"13TE": {
		ICAO:      "13TE",
		Name:      "Varisco Airport",
		City:      "Bryan",
		State:     "Texas",
		Country:   "US",
		Elevation: 240,
		Latitude:  30.6560001373,
		Longitude: -96.5382995605,
		Timezone:  "America/Chicago",
	},
	"13TS": {
		ICAO:      "13TS",
		Name:      "Van Es Ranch Airport",
		City:      "Oilton",
		State:     "Texas",
		Country:   "US",
		Elevation: 830,
		Latitude:  27.4552993774,
		Longitude: -98.926399231,
		Timezone:  "America/Chicago",
	},
	"13TX": {
		ICAO:      "13TX",
		Name:      "Allison Ranch Airport",
		City:      "Sonora",
		State:     "Texas",
		Country:   "US",
		Elevation: 2350,
		Latitude:  30.5312995911,
		Longitude: -100.4509963989,
		Timezone:  "America/Chicago",
	},
	"13WI": {
		ICAO:      "13WI",
		Name:      "Nowatzski Field",
		City:      "Markesan",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 920,
		Latitude:  43.6963996887,
		Longitude: -88.9996032715,
		Timezone:  "America/Chicago",
	},
	"14AK": {
		ICAO:      "14AK",
		Name:      "Four Corners Airport",
		City:      "Palmer",
		State:     "Alaska",
		Country:   "US",
		Elevation: 300,
		Latitude:  61.6015014648,
		Longitude: -149.2480010986,
		Timezone:  "America/Anchorage",
	},
	"14AR": {
		ICAO:      "14AR",
		Name:      "Rice-Bell Field Airport",
		City:      "Forrest City",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 221,
		Latitude:  35.0527992249,
		Longitude: -90.6750030518,
		Timezone:  "America/Chicago",
	},
	"14AZ": {
		ICAO:      "14AZ",
		Name:      "Ruby Star Airpark",
		City:      "Green Valley",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4025,
		Latitude:  31.9083003998,
		Longitude: -111.1220016479,
		Timezone:  "America/Phoenix",
	},
	"14CO": {
		ICAO:      "14CO",
		Name:      "Good Pasture Airport",
		City:      "Beulah",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6000,
		Latitude:  38.0971984863,
		Longitude: -104.9100036621,
		Timezone:  "America/Denver",
	},
	"14FA": {
		ICAO:      "14FA",
		Name:      "Burrs Strip",
		City:      "Miami",
		State:     "Florida",
		Country:   "US",
		Elevation: 10,
		Latitude:  25.5701007843,
		Longitude: -80.3998031616,
		Timezone:  "America/New_York",
	},
	"14FD": {
		ICAO:      "14FD",
		Name:      "Sunshine Farms Airport",
		City:      "St Augustine",
		State:     "Florida",
		Country:   "US",
		Elevation: 23,
		Latitude:  29.9491004944,
		Longitude: -81.4904022217,
		Timezone:  "America/New_York",
	},
	"14FL": {
		ICAO:      "14FL",
		Name:      "Wells Flying Service Airport",
		City:      "Jay",
		State:     "Florida",
		Country:   "US",
		Elevation: 240,
		Latitude:  30.8654994965,
		Longitude: -87.177696228,
		Timezone:  "America/Chicago",
	},
	"14ID": {
		ICAO:      "14ID",
		Name:      "Peaceful Cove Airport",
		City:      "Boise",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2920,
		Latitude:  43.6809997559,
		Longitude: -116.2030029297,
		Timezone:  "America/Boise",
	},
	"14IN": {
		ICAO:      "14IN",
		Name:      "De Motte Airport",
		City:      "De Motte",
		State:     "Indiana",
		Country:   "US",
		Elevation: 655,
		Latitude:  41.2070007324,
		Longitude: -87.2125015259,
		Timezone:  "America/Chicago",
	},
	"14KS": {
		ICAO:      "14KS",
		Name:      "G & S Space Port Airport",
		City:      "Parker",
		State:     "Kansas",
		Country:   "US",
		Elevation: 940,
		Latitude:  38.2691993713,
		Longitude: -94.904800415,
		Timezone:  "America/Chicago",
	},
	"14KY": {
		ICAO:      "14KY",
		Name:      "Blue Haven Farm Airport",
		City:      "Paris",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 985,
		Latitude:  38.1736984253,
		Longitude: -84.3683013916,
		Timezone:  "America/New_York",
	},
	"14LA": {
		ICAO:      "14LA",
		Name:      "Killarny Farm Airport",
		City:      "St Francisville",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 53,
		Latitude:  30.8409996033,
		Longitude: -91.5082015991,
		Timezone:  "America/Chicago",
	},
	"14ME": {
		ICAO:      "14ME",
		Name:      "Ring Hill Airport",
		City:      "Carmel",
		State:     "Maine",
		Country:   "US",
		Elevation: 340,
		Latitude:  44.7925987244,
		Longitude: -69.0702972412,
		Timezone:  "America/New_York",
	},
	"14MI": {
		ICAO:      "14MI",
		Name:      "East-West Paris Airport",
		City:      "Dutton",
		State:     "Michigan",
		Country:   "US",
		Elevation: 750,
		Latitude:  42.8508987427,
		Longitude: -85.5670013428,
		Timezone:  "America/Detroit",
	},
	"14MO": {
		ICAO:      "14MO",
		Name:      "Stevinson Farm Airport",
		City:      "Blue Springs",
		State:     "Missouri",
		Country:   "US",
		Elevation: 900,
		Latitude:  39.0499992371,
		Longitude: -94.3169021606,
		Timezone:  "America/Chicago",
	},
	"14MS": {
		ICAO:      "14MS",
		Name:      "Rose Field",
		City:      "Hickory",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 400,
		Latitude:  32.3452987671,
		Longitude: -89.0342025757,
		Timezone:  "America/Chicago",
	},
	"14MT": {
		ICAO:      "14MT",
		Name:      "Holland Ranch Airport",
		City:      "Lewistown",
		State:     "Montana",
		Country:   "US",
		Elevation: 4050,
		Latitude:  47.0516014099,
		Longitude: -109.0749969482,
		Timezone:  "America/Denver",
	},
	"14NC": {
		ICAO:      "14NC",
		Name:      "Camp Davis Mcolf Airport",
		City:      "Holly Ridge",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 60,
		Latitude:  34.5167007446,
		Longitude: -77.5500030518,
		Timezone:  "America/New_York",
	},
	"14ND": {
		ICAO:      "14ND",
		Name:      "Hagen Private Airport",
		City:      "Reeder",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 2810,
		Latitude:  46.2078018188,
		Longitude: -102.9079971313,
		Timezone:  "America/Denver",
	},
	"14NE": {
		ICAO:      "14NE",
		Name:      "Aaron's Field",
		City:      "Aurora",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1825,
		Latitude:  40.8624992371,
		Longitude: -98.064201355,
		Timezone:  "America/Chicago",
	},
	"14NK": {
		ICAO:      "14NK",
		Name:      "Mountain View Airport",
		City:      "Kingsbury",
		State:     "New York",
		Country:   "US",
		Elevation: 243,
		Latitude:  43.3400001526,
		Longitude: -73.4789962769,
		Timezone:  "America/New_York",
	},
	"14OH": {
		ICAO:      "14OH",
		Name:      "Heilman Airport",
		City:      "Kenton",
		State:     "Ohio",
		Country:   "US",
		Elevation: 930,
		Latitude:  40.6688995361,
		Longitude: -83.4434967041,
		Timezone:  "America/New_York",
	},
	"14OI": {
		ICAO:      "14OI",
		Name:      "Pleasant Home Gliding Club Gliderport",
		City:      "Pleasant Home",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1050,
		Latitude:  40.9192008972,
		Longitude: -82.1139984131,
		Timezone:  "America/New_York",
	},
	"14OK": {
		ICAO:      "14OK",
		Name:      "Reherman Airport",
		City:      "Kingfisher",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1105,
		Latitude:  35.8041992188,
		Longitude: -97.9337005615,
		Timezone:  "America/Chicago",
	},
	"14PA": {
		ICAO:      "14PA",
		Name:      "Dimascio Field",
		City:      "Pottstown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 350,
		Latitude:  40.2909011841,
		Longitude: -75.6591033936,
		Timezone:  "America/New_York",
	},
	"14PS": {
		ICAO:      "14PS",
		Name:      "Hideaway Ultralightport",
		City:      "Quarryville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 630,
		Latitude:  39.8843994141,
		Longitude: -76.118598938,
		Timezone:  "America/New_York",
	},
	"14TA": {
		ICAO:      "14TA",
		Name:      "Dean Ranch Airport",
		City:      "Startzville",
		State:     "Texas",
		Country:   "US",
		Elevation: 1180,
		Latitude:  29.8985996246,
		Longitude: -98.4030990601,
		Timezone:  "America/Chicago",
	},
	"14TE": {
		ICAO:      "14TE",
		Name:      "Smith I-Ranch Airport",
		City:      "Hext",
		State:     "Texas",
		Country:   "US",
		Elevation: 1793,
		Latitude:  30.9167995453,
		Longitude: -99.4837036133,
		Timezone:  "America/Chicago",
	},
	"14TS": {
		ICAO:      "14TS",
		Name:      "O S Wyatt Airport",
		City:      "Realitos",
		State:     "Texas",
		Country:   "US",
		Elevation: 532,
		Latitude:  27.4216995239,
		Longitude: -98.6044998169,
		Timezone:  "America/Chicago",
	},
	"14TX": {
		ICAO:      "14TX",
		Name:      "Keyes Ranch Airport",
		City:      "Dripping Springs",
		State:     "Texas",
		Country:   "US",
		Elevation: 1150,
		Latitude:  30.2894001007,
		Longitude: -98.1738967896,
		Timezone:  "America/Chicago",
	},
	"14VA": {
		ICAO:      "14VA",
		Name:      "Krens Farm Airport",
		City:      "Hillsboro",
		State:     "Virginia",
		Country:   "US",
		Elevation: 790,
		Latitude:  39.2290000916,
		Longitude: -77.7466964722,
		Timezone:  "America/New_York",
	},
	"14WA": {
		ICAO:      "14WA",
		Name:      "Lz Ranch Airport",
		City:      "Olympia",
		State:     "Washington",
		Country:   "US",
		Elevation: 240,
		Latitude:  46.9706993103,
		Longitude: -122.7369995117,
		Timezone:  "America/Los_Angeles",
	},
	"14WS": {
		ICAO:      "14WS",
		Name:      "Lakewood Lodge Airport",
		City:      "Stone Lake",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1365,
		Latitude:  45.7779998779,
		Longitude: -91.5585021973,
		Timezone:  "America/Chicago",
	},
	"14XA": {
		ICAO:      "14XA",
		Name:      "Sherman (Frog Pond) Airport",
		City:      "Sherman",
		State:     "Texas",
		Country:   "US",
		Elevation: 686,
		Latitude:  33.6578884021,
		Longitude: -96.7882847786,
		Timezone:  "America/Chicago",
	},
	"14XS": {
		ICAO:      "14XS",
		Name:      "Isbell Ranch Airport",
		City:      "Davilla",
		State:     "Texas",
		Country:   "US",
		Elevation: 430,
		Latitude:  30.8085002899,
		Longitude: -97.2739028931,
		Timezone:  "America/Chicago",
	},
	"15AK": {
		ICAO:      "15AK",
		Name:      "Golden North Airfield",
		City:      "Cantwell",
		State:     "Alaska",
		Country:   "US",
		Elevation: 2250,
		Latitude:  63.3707008362,
		Longitude: -148.8500061035,
		Timezone:  "America/Anchorage",
	},
	"15AR": {
		ICAO:      "15AR",
		Name:      "Dawson's Airport",
		City:      "Brickeys",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 220,
		Latitude:  34.8259010315,
		Longitude: -90.543296814,
		Timezone:  "America/Chicago",
	},
	"15AZ": {
		ICAO:      "15AZ",
		Name:      "Quail Mesa Ranch Airport",
		City:      "Quartzsite",
		State:     "Arizona",
		Country:   "US",
		Elevation: 290,
		Latitude:  33.7444992065,
		Longitude: -114.476997375,
		Timezone:  "America/Phoenix",
	},
	"15CL": {
		ICAO:      "15CL",
		Name:      "Hunt Farms Airport",
		City:      "Merced",
		State:     "California",
		Country:   "US",
		Elevation: 223,
		Latitude:  37.3427009583,
		Longitude: -120.4039993286,
		Timezone:  "America/Los_Angeles",
	},
	"15FD": {
		ICAO:      "15FD",
		Name:      "Orange Hill Airport",
		City:      "Wausau",
		State:     "Florida",
		Country:   "US",
		Elevation: 260,
		Latitude:  30.6585006714,
		Longitude: -85.5298995972,
		Timezone:  "America/Chicago",
	},
	"15FL": {
		ICAO:      "15FL",
		Name:      "Cannon Creek Airpark",
		City:      "Lake City",
		State:     "Florida",
		Country:   "US",
		Elevation: 125,
		Latitude:  30.1536998749,
		Longitude: -82.6663970947,
		Timezone:  "America/New_York",
	},
	"15GA": {
		ICAO:      "15GA",
		Name:      "Darla's Airport",
		City:      "Kite",
		State:     "Georgia",
		Country:   "US",
		Elevation: 328,
		Latitude:  32.6801986694,
		Longitude: -82.4789962769,
		Timezone:  "America/New_York",
	},
	"15ID": {
		ICAO:      "15ID",
		Name:      "Scanlon Airport",
		City:      "Coeur D Alene",
		State:     "Washington",
		Country:   "US",
		Elevation: 2000,
		Latitude:  47.6856002808,
		Longitude: -117.0380020142,
		Timezone:  "America/Los_Angeles",
	},
	"15IL": {
		ICAO:      "15IL",
		Name:      "Gittleson Farms Airport",
		City:      "Franklin Grove",
		State:     "Illinois",
		Country:   "US",
		Elevation: 836,
		Latitude:  41.7972984314,
		Longitude: -89.2732009888,
		Timezone:  "America/Chicago",
	},
	"15IN": {
		ICAO:      "15IN",
		Name:      "Bugtown Airport",
		City:      "New Harmony",
		State:     "Indiana",
		Country:   "US",
		Elevation: 464,
		Latitude:  38.1506004333,
		Longitude: -87.8403015137,
		Timezone:  "America/Chicago",
	},
	"15KS": {
		ICAO:      "15KS",
		Name:      "Sickler Airstrip",
		City:      "Emporia",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1200,
		Latitude:  38.4305992126,
		Longitude: -96.0883026123,
		Timezone:  "America/Chicago",
	},
	"15KY": {
		ICAO:      "15KY",
		Name:      "Brennan Farm Airport",
		City:      "Paris",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 910,
		Latitude:  38.1570014954,
		Longitude: -84.3015975952,
		Timezone:  "America/New_York",
	},
	"15LL": {
		ICAO:      "15LL",
		Name:      "Cloverleaf Ranch Airport",
		City:      "Tiskilwa",
		State:     "Illinois",
		Country:   "US",
		Elevation: 807,
		Latitude:  41.2374992371,
		Longitude: -89.5366973877,
		Timezone:  "America/Chicago",
	},
	"15MA": {
		ICAO:      "15MA",
		Name:      "Canapitsit Airport",
		City:      "Gosnold",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 10,
		Latitude:  41.4222984314,
		Longitude: -70.9095001221,
		Timezone:  "America/New_York",
	},
	"15ME": {
		ICAO:      "15ME",
		Name:      "Eagle Field",
		City:      "North Yarmouth",
		State:     "Maine",
		Country:   "US",
		Elevation: 150,
		Latitude:  43.8152999878,
		Longitude: -70.2292022705,
		Timezone:  "America/New_York",
	},
	"15MN": {
		ICAO:      "15MN",
		Name:      "Tyler Farms Airport",
		City:      "Watkins",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1185,
		Latitude:  45.2621994019,
		Longitude: -94.5106964111,
		Timezone:  "America/Chicago",
	},
	"15MO": {
		ICAO:      "15MO",
		Name:      "Applegate Airport",
		City:      "Queen City",
		State:     "Missouri",
		Country:   "US",
		Elevation: 997,
		Latitude:  40.391998291,
		Longitude: -92.5654983521,
		Timezone:  "America/Chicago",
	},
	"15MT": {
		ICAO:      "15MT",
		Name:      "Saubak Airport",
		City:      "Lustre",
		State:     "Montana",
		Country:   "US",
		Elevation: 2701,
		Latitude:  48.5004005432,
		Longitude: -105.9100036621,
		Timezone:  "America/Denver",
	},
	"15NC": {
		ICAO:      "15NC",
		Name:      "Dragonfly Field",
		City:      "Ahoskie",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 49,
		Latitude:  36.2397003174,
		Longitude: -76.9738998413,
		Timezone:  "America/New_York",
	},
	"15NE": {
		ICAO:      "15NE",
		Name:      "Sindt Airport",
		City:      "Upland",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1990,
		Latitude:  40.2910995483,
		Longitude: -98.8234024048,
		Timezone:  "America/Chicago",
	},
	"15NK": {
		ICAO:      "15NK",
		Name:      "Knox Landing Airport",
		City:      "Sardinia",
		State:     "New York",
		Country:   "US",
		Elevation: 1845,
		Latitude:  42.5764007568,
		Longitude: -78.5567016602,
		Timezone:  "America/New_York",
	},
	"15OK": {
		ICAO:      "15OK",
		Name:      "Bluebird Airport",
		City:      "Muskogee",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 531,
		Latitude:  35.703889,
		Longitude: -95.310833,
		Timezone:  "America/Chicago",
	},
	"15PA": {
		ICAO:      "15PA",
		Name:      "Cavage Personal Use Airport",
		City:      "Prompton",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1400,
		Latitude:  41.5942993164,
		Longitude: -75.3554992676,
		Timezone:  "America/New_York",
	},
	"15TA": {
		ICAO:      "15TA",
		Name:      "J R Ranch Airport",
		City:      "Briggs",
		State:     "Texas",
		Country:   "US",
		Elevation: 1125,
		Latitude:  30.2544002533,
		Longitude: -99.9469985962,
		Timezone:  "America/Chicago",
	},
	"15TX": {
		ICAO:      "15TX",
		Name:      "Ed Shadle Airport",
		City:      "Stephenville",
		State:     "Texas",
		Country:   "US",
		Elevation: 1550,
		Latitude:  32.3042984009,
		Longitude: -98.325302124,
		Timezone:  "America/Chicago",
	},
	"15VA": {
		ICAO:      "15VA",
		Name:      "Fox Acres Airport",
		City:      "Warrenton",
		State:     "Virginia",
		Country:   "US",
		Elevation: 450,
		Latitude:  38.7223014832,
		Longitude: -77.8996963501,
		Timezone:  "America/New_York",
	},
	"15WA": {
		ICAO:      "15WA",
		Name:      "Sunset Airport",
		City:      "Stanwood",
		State:     "Washington",
		Country:   "US",
		Elevation: 156,
		Latitude:  48.27230072,
		Longitude: -122.3570023,
		Timezone:  "America/Los_Angeles",
	},
	"15WI": {
		ICAO:      "15WI",
		Name:      "Peterson Field",
		City:      "Oregon",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 978,
		Latitude:  42.9275016785,
		Longitude: -89.4181976318,
		Timezone:  "America/Chicago",
	},
	"15XS": {
		ICAO:      "15XS",
		Name:      "Toy Airpark",
		City:      "Liverpool",
		State:     "Texas",
		Country:   "US",
		Elevation: 36,
		Latitude:  29.3327007294,
		Longitude: -95.3340988159,
		Timezone:  "America/Chicago",
	},
	"16AK": {
		ICAO:      "16AK",
		Name:      "Gattis Strip",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 320,
		Latitude:  61.5992012024,
		Longitude: -149.3470001221,
		Timezone:  "America/Anchorage",
	},
	"16AR": {
		ICAO:      "16AR",
		Name:      "Heifer Creek Ranch Airport",
		City:      "Springfield",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 730,
		Latitude:  35.3011016846,
		Longitude: -92.5733032227,
		Timezone:  "America/Chicago",
	},
	"16AZ": {
		ICAO:      "16AZ",
		Name:      "Yav'pe Ma'ta Airport",
		City:      "Skull Valley",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4366,
		Latitude:  34.5088996887,
		Longitude: -112.6800003052,
		Timezone:  "America/Phoenix",
	},
	"16CO": {
		ICAO:      "16CO",
		Name:      "Dry Pen Airport",
		City:      "Parachute",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5331,
		Latitude:  39.40995,
		Longitude: -108.085667,
		Timezone:  "America/Denver",
	},
	"16FA": {
		ICAO:      "16FA",
		Name:      "Little Deer Airport",
		City:      "Ochopee",
		State:     "Florida",
		Country:   "US",
		Elevation: 14,
		Latitude:  26.0308990479,
		Longitude: -81.0386962891,
		Timezone:  "America/New_York",
	},
	"16FD": {
		ICAO:      "16FD",
		Name:      "Skinners Wholesale Nursery Airport",
		City:      "Crescent City",
		State:     "Florida",
		Country:   "US",
		Elevation: 20,
		Latitude:  29.4050006866,
		Longitude: -81.4959030151,
		Timezone:  "America/New_York",
	},
	"16FL": {
		ICAO:      "16FL",
		Name:      "J-22 Ranch Airport",
		City:      "Milton",
		State:     "Florida",
		Country:   "US",
		Elevation: 200,
		Latitude:  30.7252006531,
		Longitude: -87.2014007568,
		Timezone:  "America/Chicago",
	},
	"16IA": {
		ICAO:      "16IA",
		Name:      "Stangl STOLport",
		City:      "Coon Rapids",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1230,
		Latitude:  41.8807983398,
		Longitude: -94.7771987915,
		Timezone:  "America/Chicago",
	},
	"16II": {
		ICAO:      "16II",
		Name:      "Harrington Field",
		City:      "San Pierre",
		State:     "Indiana",
		Country:   "US",
		Elevation: 710,
		Latitude:  41.1831016541,
		Longitude: -86.9336013794,
		Timezone:  "America/Chicago",
	},
	"16IL": {
		ICAO:      "16IL",
		Name:      "Harold Bunger Airport",
		City:      "Kirkland",
		State:     "Illinois",
		Country:   "US",
		Elevation: 830,
		Latitude:  42.0666999817,
		Longitude: -88.8333969116,
		Timezone:  "America/Chicago",
	},
	"16IS": {
		ICAO:      "16IS",
		Name:      "Kellums Airport",
		City:      "Goreville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 720,
		Latitude:  37.5778007507,
		Longitude: -89.0092010498,
		Timezone:  "America/Chicago",
	},
	"16KY": {
		ICAO:      "16KY",
		Name:      "Praise God Airport",
		City:      "Carter",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 1070,
		Latitude:  38.4440002441,
		Longitude: -83.1227035522,
		Timezone:  "America/New_York",
	},
	"16LS": {
		ICAO:      "16LS",
		Name:      "Atchafalaya Flying Company Airport",
		City:      "Simmsport",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 41,
		Latitude:  30.9444007874,
		Longitude: -91.7938995361,
		Timezone:  "America/Chicago",
	},
	"16MD": {
		ICAO:      "16MD",
		Name:      "High Valley Airport",
		City:      "North East",
		State:     "Maryland",
		Country:   "US",
		Elevation: 363,
		Latitude:  39.668611,
		Longitude: -75.960278,
		Timezone:  "America/New_York",
	},
	"16MI": {
		ICAO:      "16MI",
		Name:      "Barnstormers 5 Airport",
		City:      "Gaines",
		State:     "Michigan",
		Country:   "US",
		Elevation: 823,
		Latitude:  42.8891983032,
		Longitude: -83.8666000366,
		Timezone:  "America/Detroit",
	},
	"16MO": {
		ICAO:      "16MO",
		Name:      "Findley Field",
		City:      "Beaufort",
		State:     "Missouri",
		Country:   "US",
		Elevation: 755,
		Latitude:  38.4056015015,
		Longitude: -91.1574020386,
		Timezone:  "America/Chicago",
	},
	"16MT": {
		ICAO:      "16MT",
		Name:      "Bangart Field",
		City:      "Roberts",
		State:     "Montana",
		Country:   "US",
		Elevation: 4320,
		Latitude:  45.4052009583,
		Longitude: -109.1299972534,
		Timezone:  "America/Denver",
	},
	"16NC": {
		ICAO:      "16NC",
		Name:      "Lee's Airport",
		City:      "Edenton",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 31,
		Latitude:  36.1960983276,
		Longitude: -76.6652984619,
		Timezone:  "America/New_York",
	},
	"16NE": {
		ICAO:      "16NE",
		Name:      "Korver Airport",
		City:      "Panama",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1390,
		Latitude:  40.5600013733,
		Longitude: -96.4916992188,
		Timezone:  "America/Chicago",
	},
	"16OI": {
		ICAO:      "16OI",
		Name:      "Trump Airport",
		City:      "Greenville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1038,
		Latitude:  40.1253013611,
		Longitude: -84.5824966431,
		Timezone:  "America/New_York",
	},
	"16OK": {
		ICAO:      "16OK",
		Name:      "Lower Forty Airport",
		City:      "Harrah",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1160,
		Latitude:  35.4667015076,
		Longitude: -97.200302124,
		Timezone:  "America/Chicago",
	},
	"16PA": {
		ICAO:      "16PA",
		Name:      "Gregg Airport",
		City:      "Eighty Four",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1200,
		Latitude:  40.1805992126,
		Longitude: -80.1276016235,
		Timezone:  "America/New_York",
	},
	"16SC": {
		ICAO:      "16SC",
		Name:      "Southern Aero Sports Airport",
		City:      "Cross Anchor",
		State:     "South Carolina",
		Country:   "US",
		Elevation: 600,
		Latitude:  34.6250991821,
		Longitude: -81.898399353,
		Timezone:  "America/New_York",
	},
	"16TA": {
		ICAO:      "16TA",
		Name:      "Seven Springs Airport",
		City:      "Toyahvale",
		State:     "Texas",
		Country:   "US",
		Elevation: 3500,
		Latitude:  30.9197006226,
		Longitude: -103.7829971313,
		Timezone:  "America/Chicago",
	},
	"16TE": {
		ICAO:      "16TE",
		Name:      "Holly Lake Ranch Airport",
		City:      "Hawkins",
		State:     "Texas",
		Country:   "US",
		Elevation: 409,
		Latitude:  32.6976013184,
		Longitude: -95.2077026367,
		Timezone:  "America/Chicago",
	},
	"16TS": {
		ICAO:      "16TS",
		Name:      "Pineridge STOLport",
		City:      "Tyler",
		State:     "Texas",
		Country:   "US",
		Elevation: 420,
		Latitude:  32.2226982117,
		Longitude: -95.3180007935,
		Timezone:  "America/Chicago",
	},
	"16TX": {
		ICAO:      "16TX",
		Name:      "Ratliff Airport",
		City:      "Stephenville",
		State:     "Texas",
		Country:   "US",
		Elevation: 1300,
		Latitude:  32.263999939,
		Longitude: -98.1072998047,
		Timezone:  "America/Chicago",
	},
	"16WA": {
		ICAO:      "16WA",
		Name:      "Tightcliff Airport",
		City:      "Nine Mile Falls",
		State:     "Washington",
		Country:   "US",
		Elevation: 1855,
		Latitude:  47.8446998596,
		Longitude: -117.6940002441,
		Timezone:  "America/Los_Angeles",
	},
	"16XS": {
		ICAO:      "16XS",
		Name:      "Outlaw Flyers Airport",
		City:      "Conroe",
		State:     "Texas",
		Country:   "US",
		Elevation: 270,
		Latitude:  30.404367,
		Longitude: -95.35985,
		Timezone:  "America/Chicago",
	},
	"17AR": {
		ICAO:      "17AR",
		Name:      "Bredlow Farm Airport",
		City:      "England",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 236,
		Latitude:  34.5531005859,
		Longitude: -92.0914001465,
		Timezone:  "America/Chicago",
	},
	"17CL": {
		ICAO:      "17CL",
		Name:      "Las Trancas Airport",
		City:      "Davenport",
		State:     "California",
		Country:   "US",
		Elevation: 125,
		Latitude:  37.0880012512,
		Longitude: -122.274002075,
		Timezone:  "America/Los_Angeles",
	},
	"17CO": {
		ICAO:      "17CO",
		Name:      "Skylane Ranch Airport",
		City:      "Berthoud",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5000,
		Latitude:  40.272258,
		Longitude: -105.032625,
		Timezone:  "America/Denver",
	},
	"17FA": {
		ICAO:      "17FA",
		Name:      "Cotton Strip",
		City:      "La Belle",
		State:     "Florida",
		Country:   "US",
		Elevation: 18,
		Latitude:  26.7133998871,
		Longitude: -81.5354003906,
		Timezone:  "America/New_York",
	},
	"17FL": {
		ICAO:      "17FL",
		Name:      "Jumbolair-Greystone Airport",
		City:      "Ocala",
		State:     "Florida",
		Country:   "US",
		Elevation: 100,
		Latitude:  29.2803001404,
		Longitude: -82.1248016357,
		Timezone:  "America/New_York",
	},
	"17GA": {
		ICAO:      "17GA",
		Name:      "Panther Creek Airport",
		City:      "Newnan",
		State:     "Georgia",
		Country:   "US",
		Elevation: 850,
		Latitude:  33.4667015076,
		Longitude: -84.8660964966,
		Timezone:  "America/New_York",
	},
	"17ID": {
		ICAO:      "17ID",
		Name:      "Coyote Ridge Airport",
		City:      "Midvale",
		State:     "Idaho",
		Country:   "US",
		Elevation: 3365,
		Latitude:  44.413898468,
		Longitude: -116.591003418,
		Timezone:  "America/Boise",
	},
	"17II": {
		ICAO:      "17II",
		Name:      "Dreessen Field",
		City:      "Walkerton",
		State:     "Indiana",
		Country:   "US",
		Elevation: 706,
		Latitude:  41.4500007629,
		Longitude: -86.5102996826,
		Timezone:  "America/Chicago",
	},
	"17IN": {
		ICAO:      "17IN",
		Name:      "Pruss Airport",
		City:      "Farmers Retreat",
		State:     "Indiana",
		Country:   "US",
		Elevation: 890,
		Latitude:  38.9784011841,
		Longitude: -85.109703064,
		Timezone:  "America/New_York",
	},
	"17KS": {
		ICAO:      "17KS",
		Name:      "Tevis Airport",
		City:      "Melvern",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1000,
		Latitude:  38.5027999878,
		Longitude: -95.6772003174,
		Timezone:  "America/Chicago",
	},
	"17KY": {
		ICAO:      "17KY",
		Name:      "Shannon Field",
		City:      "Sacramento",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 440,
		Latitude:  37.393901825,
		Longitude: -87.2589035034,
		Timezone:  "America/Chicago",
	},
	"17LL": {
		ICAO:      "17LL",
		Name:      "Oink Acres Airport",
		City:      "Mahomet",
		State:     "Illinois",
		Country:   "US",
		Elevation: 725,
		Latitude:  40.1605987549,
		Longitude: -88.4103012085,
		Timezone:  "America/Chicago",
	},
	"17LS": {
		ICAO:      "17LS",
		Name:      "Yankee Field",
		City:      "Port Barre",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 45,
		Latitude:  30.6056995392,
		Longitude: -91.9917984009,
		Timezone:  "America/Chicago",
	},
	"17ME": {
		ICAO:      "17ME",
		Name:      "Bresett'S Mountainside Airport",
		City:      "Van Buren",
		State:     "Maine",
		Country:   "US",
		Elevation: 590,
		Latitude:  47.2025,
		Longitude: -67.991667,
		Timezone:  "America/Moncton",
	},
	"17MO": {
		ICAO:      "17MO",
		Name:      "Rgl Field",
		City:      "Montgomery City",
		State:     "Missouri",
		Country:   "US",
		Elevation: 825,
		Latitude:  38.9850006104,
		Longitude: -91.5348968506,
		Timezone:  "America/Chicago",
	},
	"17MT": {
		ICAO:      "17MT",
		Name:      "Abel Ranch Airport",
		City:      "Kalispell",
		State:     "Montana",
		Country:   "US",
		Elevation: 2900,
		Latitude:  48.1071014404,
		Longitude: -114.1770019531,
		Timezone:  "America/Denver",
	},
	"17NC": {
		ICAO:      "17NC",
		Name:      "Mitchell Field",
		City:      "Elizabethtown",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 23,
		Latitude:  34.4663009644,
		Longitude: -78.3285980225,
		Timezone:  "America/New_York",
	},
	"17NK": {
		ICAO:      "17NK",
		Name:      "Re-Dun Field",
		City:      "Reading Center",
		State:     "New York",
		Country:   "US",
		Elevation: 1351,
		Latitude:  42.5395011902,
		Longitude: -76.9483032227,
		Timezone:  "America/New_York",
	},
	"17OH": {
		ICAO:      "17OH",
		Name:      "Kosik Private Airport",
		City:      "Kipton",
		State:     "Ohio",
		Country:   "US",
		Elevation: 855,
		Latitude:  41.2425994873,
		Longitude: -82.2702026367,
		Timezone:  "America/New_York",
	},
	"17OI": {
		ICAO:      "17OI",
		Name:      "Haas Airport",
		City:      "Bainbridge",
		State:     "Ohio",
		Country:   "US",
		Elevation: 750,
		Latitude:  39.2266998291,
		Longitude: -83.3177032471,
		Timezone:  "America/New_York",
	},
	"17OK": {
		ICAO:      "17OK",
		Name:      "J-B Airstrip",
		City:      "Kremlin",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1142,
		Latitude:  36.5250015259,
		Longitude: -97.7586975098,
		Timezone:  "America/Chicago",
	},
	"17OR": {
		ICAO:      "17OR",
		Name:      "Glide Aero Airport",
		City:      "Glide",
		State:     "Oregon",
		Country:   "US",
		Elevation: 980,
		Latitude:  43.2641983032,
		Longitude: -123.1100006104,
		Timezone:  "America/Los_Angeles",
	},
	"17PS": {
		ICAO:      "17PS",
		Name:      "Mountain Crest Airport",
		City:      "Tidioute",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1641,
		Latitude:  41.5792007446,
		Longitude: -79.4417037964,
		Timezone:  "America/New_York",
	},
	"17SC": {
		ICAO:      "17SC",
		Name:      "Carolina Cow Country Airport",
		City:      "Clinton",
		State:     "South Carolina",
		Country:   "US",
		Elevation: 680,
		Latitude:  34.5326004028,
		Longitude: -81.8861999512,
		Timezone:  "America/New_York",
	},
	"17TE": {
		ICAO:      "17TE",
		Name:      "Comfort Airpark",
		City:      "Comfort",
		State:     "Texas",
		Country:   "US",
		Elevation: 1470,
		Latitude:  29.9291000366,
		Longitude: -98.9402999878,
		Timezone:  "America/Chicago",
	},
	"17TN": {
		ICAO:      "17TN",
		Name:      "Murphy Field",
		City:      "Jonesborough",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1720,
		Latitude:  36.2994003296,
		Longitude: -82.6042022705,
		Timezone:  "America/New_York",
	},
	"17TX": {
		ICAO:      "17TX",
		Name:      "Kimzey Airport",
		City:      "Stephenville",
		State:     "Texas",
		Country:   "US",
		Elevation: 1100,
		Latitude:  32.4486999512,
		Longitude: -98.2549972534,
		Timezone:  "America/Chicago",
	},
	"17WI": {
		ICAO:      "17WI",
		Name:      "Ha-Rail Airport",
		City:      "Lake Mills",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 885,
		Latitude:  43.0360984802,
		Longitude: -88.8932037354,
		Timezone:  "America/Chicago",
	},
	"17XA": {
		ICAO:      "17XA",
		Name:      "Jacksonville / Hunter Field",
		City:      "Jacksonville",
		State:     "Texas",
		Country:   "US",
		Elevation: 580,
		Latitude:  32.017778,
		Longitude: -95.7969444445,
		Timezone:  "America/Chicago",
	},
	"17XS": {
		ICAO:      "17XS",
		Name:      "Quahadi Ranch Airport",
		City:      "Dublin",
		State:     "Texas",
		Country:   "US",
		Elevation: 1422,
		Latitude:  32.1777992249,
		Longitude: -98.4349975586,
		Timezone:  "America/Chicago",
	},
	"18AR": {
		ICAO:      "18AR",
		Name:      "Buck Mountain Airport",
		City:      "Eureka Springs",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1450,
		Latitude:  36.3442001343,
		Longitude: -93.831703186,
		Timezone:  "America/Chicago",
	},
	"18AZ": {
		ICAO:      "18AZ",
		Name:      "Sky Ranch At Carefree Airport",
		City:      "Carefree",
		State:     "Arizona",
		Country:   "US",
		Elevation: 2568,
		Latitude:  33.8180999756,
		Longitude: -111.8980026245,
		Timezone:  "America/Phoenix",
	},
	"18CL": {
		ICAO:      "18CL",
		Name:      "Bowles Airport",
		City:      "Live Oak",
		State:     "California",
		Country:   "US",
		Elevation: 75,
		Latitude:  39.2859992981,
		Longitude: -121.6940002441,
		Timezone:  "America/Los_Angeles",
	},
	"18FA": {
		ICAO:      "18FA",
		Name:      "Tropical Plantation Airport",
		City:      "Palm City",
		State:     "Florida",
		Country:   "US",
		Elevation: 19,
		Latitude:  27.1028003693,
		Longitude: -80.2934036255,
		Timezone:  "America/New_York",
	},
	"18FD": {
		ICAO:      "18FD",
		Name:      "Sky Ranch Airport",
		City:      "Baker",
		State:     "Florida",
		Country:   "US",
		Elevation: 250,
		Latitude:  30.8540992737,
		Longitude: -86.6671981812,
		Timezone:  "America/Chicago",
	},
	"18GA": {
		ICAO:      "18GA",
		Name:      "Sleepy Hollow Airport",
		City:      "Social Circle",
		State:     "Georgia",
		Country:   "US",
		Elevation: 750,
		Latitude:  33.6931991577,
		Longitude: -83.657699585,
		Timezone:  "America/New_York",
	},
	"18II": {
		ICAO:      "18II",
		Name:      "Mc Gill Airport",
		City:      "Cicero",
		State:     "Indiana",
		Country:   "US",
		Elevation: 855,
		Latitude:  40.1199989319,
		Longitude: -86.0682983398,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"18IN": {
		ICAO:      "18IN",
		Name:      "Kester Fly Inn Airport",
		City:      "Farmersburg",
		State:     "Indiana",
		Country:   "US",
		Elevation: 575,
		Latitude:  39.2784004211,
		Longitude: -87.4135971069,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"18JY": {
		ICAO:      "18JY",
		Name:      "Skamokawa East Valley Airport",
		City:      "Skamokawa",
		State:     "Washington",
		Country:   "US",
		Elevation: 19,
		Latitude:  46.2846984863,
		Longitude: -123.4430007935,
		Timezone:  "America/Los_Angeles",
	},
	"18KS": {
		ICAO:      "18KS",
		Name:      "Cherokee Strip",
		City:      "Udall",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1270,
		Latitude:  37.3708992004,
		Longitude: -97.1072998047,
		Timezone:  "America/Chicago",
	},
	"18ME": {
		ICAO:      "18ME",
		Name:      "Ruby Airport",
		City:      "Bristol",
		State:     "Maine",
		Country:   "US",
		Elevation: 216,
		Latitude:  43.893597,
		Longitude: -69.495406,
		Timezone:  "America/New_York",
	},
	"18MN": {
		ICAO:      "18MN",
		Name:      "Hines Farm Airport",
		City:      "Motley",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1283,
		Latitude:  46.2372016907,
		Longitude: -94.5931015015,
		Timezone:  "America/Chicago",
	},
	"18MO": {
		ICAO:      "18MO",
		Name:      "Gimlin Airport",
		City:      "Ozark",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1380,
		Latitude:  36.962600708,
		Longitude: -93.1707000732,
		Timezone:  "America/Chicago",
	},
	"18MT": {
		ICAO:      "18MT",
		Name:      "Fish Ranch Airport",
		City:      "Jackson",
		State:     "Montana",
		Country:   "US",
		Elevation: 7200,
		Latitude:  45.2159996033,
		Longitude: -113.4990005493,
		Timezone:  "America/Denver",
	},
	"18NC": {
		ICAO:      "18NC",
		Name:      "Lanni Field",
		City:      "Lenoir",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 1160,
		Latitude:  35.888999939,
		Longitude: -81.6137008667,
		Timezone:  "America/New_York",
	},
	"18NE": {
		ICAO:      "18NE",
		Name:      "Doc's Airport",
		City:      "Geneva",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1640,
		Latitude:  40.5139007568,
		Longitude: -97.6106033325,
		Timezone:  "America/Chicago",
	},
	"18NY": {
		ICAO:      "18NY",
		Name:      "Strip in the Woods Airport",
		City:      "Woodstock",
		State:     "New York",
		Country:   "US",
		Elevation: 500,
		Latitude:  42.0164985657,
		Longitude: -74.0490036011,
		Timezone:  "America/New_York",
	},
	"18OH": {
		ICAO:      "18OH",
		Name:      "Jer-Mar Airpark",
		City:      "Lagrange",
		State:     "Ohio",
		Country:   "US",
		Elevation: 800,
		Latitude:  41.2561988831,
		Longitude: -82.1437988281,
		Timezone:  "America/New_York",
	},
	"18OI": {
		ICAO:      "18OI",
		Name:      "Boggy Bottoms Airport",
		City:      "Vickery",
		State:     "Ohio",
		Country:   "US",
		Elevation: 575,
		Latitude:  41.4234008789,
		Longitude: -82.9826965332,
		Timezone:  "America/New_York",
	},
	"18OR": {
		ICAO:      "18OR",
		Name:      "Red's Field",
		City:      "Amity",
		State:     "Oregon",
		Country:   "US",
		Elevation: 190,
		Latitude:  45.0918655396,
		Longitude: -123.182350159,
		Timezone:  "America/Los_Angeles",
	},
	"18PA": {
		ICAO:      "18PA",
		Name:      "Slack Airport",
		City:      "Forest Grove",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 275,
		Latitude:  40.3000984192,
		Longitude: -75.082901001,
		Timezone:  "America/New_York",
	},
	"18PN": {
		ICAO:      "18PN",
		Name:      "Spud View Airport",
		City:      "Upper Strasburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 733,
		Latitude:  40.0653991699,
		Longitude: -77.6928024292,
		Timezone:  "America/New_York",
	},
	"18SC": {
		ICAO:      "18SC",
		Name:      "Connelly Field",
		City:      "Silverstreet",
		State:     "South Carolina",
		Country:   "US",
		Elevation: 570,
		Latitude:  34.2739982605,
		Longitude: -81.773399353,
		Timezone:  "America/New_York",
	},
	"18TA": {
		ICAO:      "18TA",
		Name:      "West Kerr Ranch Airport",
		City:      "Junction",
		State:     "Texas",
		Country:   "US",
		Elevation: 2330,
		Latitude:  30.1362991333,
		Longitude: -99.7436981201,
		Timezone:  "America/Chicago",
	},
	"18TE": {
		ICAO:      "18TE",
		Name:      "Alert Field",
		City:      "Big Sandy",
		State:     "Texas",
		Country:   "US",
		Elevation: 390,
		Latitude:  32.5852012634,
		Longitude: -95.0639038086,
		Timezone:  "America/Chicago",
	},
	"18TX": {
		ICAO:      "18TX",
		Name:      "Flying 'T' Ranch Airport",
		City:      "Caddo Mills",
		State:     "Texas",
		Country:   "US",
		Elevation: 545,
		Latitude:  32.9524993896,
		Longitude: -96.2281036377,
		Timezone:  "America/Chicago",
	},
	"18WV": {
		ICAO:      "18WV",
		Name:      "Lynn Airport",
		City:      "Newburg",
		State:     "West Virginia",
		Country:   "US",
		Elevation: 1680,
		Latitude:  39.41,
		Longitude: -79.856667,
		Timezone:  "America/New_York",
	},
	"18XS": {
		ICAO:      "18XS",
		Name:      "Gardner Farm Airport",
		City:      "Mc Adoo",
		State:     "Texas",
		Country:   "US",
		Elevation: 3000,
		Latitude:  33.8000984192,
		Longitude: -101.033996582,
		Timezone:  "America/Chicago",
	},
	"19AK": {
		ICAO:      "19AK",
		IATA:      "ICY",
		Name:      "Icy Bay Airport",
		City:      "Icy Bay",
		State:     "Alaska",
		Country:   "US",
		Elevation: 50,
		Latitude:  59.96900177,
		Longitude: -141.662002563,
		Timezone:  "America/Anchorage",
	},
	"19AR": {
		ICAO:      "19AR",
		Name:      "Naylor field",
		City:      "Naylor",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 230,
		Latitude:  35.15670013,
		Longitude: -92.22419739,
		Timezone:  "America/Chicago",
	},
	"19AZ": {
		ICAO:      "19AZ",
		Name:      "Montezuma Airport",
		City:      "Camp Verde",
		State:     "Arizona",
		Country:   "US",
		Elevation: 3370,
		Latitude:  34.6049995422,
		Longitude: -111.8649978638,
		Timezone:  "America/Phoenix",
	},
	"19CL": {
		ICAO:      "19CL",
		Name:      "Palisades Ranch Airport",
		City:      "Helendale",
		State:     "California",
		Country:   "US",
		Elevation: 2510,
		Latitude:  34.7122001648,
		Longitude: -117.3509979248,
		Timezone:  "America/Los_Angeles",
	},
	"19GA": {
		ICAO:      "19GA",
		Name:      "Willow Pond Aviation Inc Airport",
		City:      "Fayetteville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 868,
		Latitude:  33.4236984253,
		Longitude: -84.4999008179,
		Timezone:  "America/New_York",
	},
	"19IA": {
		ICAO:      "19IA",
		Name:      "Ancam Antique Airfield",
		City:      "West Burlington",
		State:     "Iowa",
		Country:   "US",
		Elevation: 690,
		Latitude:  40.8568000793,
		Longitude: -91.2037963867,
		Timezone:  "America/Chicago",
	},
	"19IL": {
		ICAO:      "19IL",
		Name:      "Eagle Pass Airport",
		City:      "Bloomington",
		State:     "Illinois",
		Country:   "US",
		Elevation: 810,
		Latitude:  40.5494003296,
		Longitude: -89.1060028076,
		Timezone:  "America/Chicago",
	},
	"19IN": {
		ICAO:      "19IN",
		Name:      "Clark Airport",
		City:      "Frankfort",
		State:     "Indiana",
		Country:   "US",
		Elevation: 880,
		Latitude:  40.1944999695,
		Longitude: -86.5231018066,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"19IS": {
		ICAO:      "19IS",
		Name:      "Skillet Fork Farm Airport",
		City:      "Wayne City",
		State:     "Illinois",
		Country:   "US",
		Elevation: 430,
		Latitude:  38.2958984375,
		Longitude: -88.5569992065,
		Timezone:  "America/Chicago",
	},
	"19KY": {
		ICAO:      "19KY",
		Name:      "Conrads Airport",
		City:      "Dry Ridge",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 940,
		Latitude:  38.7065010071,
		Longitude: -84.599899292,
		Timezone:  "America/New_York",
	},
	"19LL": {
		ICAO:      "19LL",
		Name:      "Neiner Airport",
		City:      "Manteno",
		State:     "Illinois",
		Country:   "US",
		Elevation: 650,
		Latitude:  41.2569999695,
		Longitude: -87.9195022583,
		Timezone:  "America/Chicago",
	},
	"19NC": {
		ICAO:      "19NC",
		Name:      "Double S Airport",
		City:      "Red Oak",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 195,
		Latitude:  36.0648994446,
		Longitude: -77.922996521,
		Timezone:  "America/New_York",
	},
	"19ND": {
		ICAO:      "19ND",
		Name:      "Breckheimer Airport",
		City:      "Tolna",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1458,
		Latitude:  47.7332992554,
		Longitude: -98.4887008667,
		Timezone:  "America/Chicago",
	},
	"19NE": {
		ICAO:      "19NE",
		Name:      "Hoyt Airport",
		City:      "Mc Cook/Culbertson",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2707,
		Latitude:  40.1082992554,
		Longitude: -100.8089981079,
		Timezone:  "America/Chicago",
	},
	"19NK": {
		ICAO:      "19NK",
		Name:      "Riveredge Airpark",
		City:      "Central Square",
		State:     "New York",
		Country:   "US",
		Elevation: 378,
		Latitude:  43.2439002991,
		Longitude: -76.1528015137,
		Timezone:  "America/New_York",
	},
	"19NY": {
		ICAO:      "19NY",
		Name:      "Four Seasons Airport",
		City:      "Reading",
		State:     "New York",
		Country:   "US",
		Elevation: 1650,
		Latitude:  42.4062004089,
		Longitude: -77.9608001709,
		Timezone:  "America/New_York",
	},
	"19OH": {
		ICAO:      "19OH",
		Name:      "Harris Corn Field",
		City:      "Litchfield",
		State:     "Ohio",
		Country:   "US",
		Elevation: 915,
		Latitude:  41.1730995178,
		Longitude: -82.0593032837,
		Timezone:  "America/New_York",
	},
	"19OK": {
		ICAO:      "19OK",
		Name:      "Gerhart Airport",
		City:      "Okmulgee",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 940,
		Latitude:  35.7284011841,
		Longitude: -95.9772033691,
		Timezone:  "America/Chicago",
	},
	"19OR": {
		ICAO:      "19OR",
		Name:      "Nelson Ranch Airport",
		City:      "Maupin",
		State:     "Oregon",
		Country:   "US",
		Elevation: 2000,
		Latitude:  45.1096000671,
		Longitude: -121.2239990234,
		Timezone:  "America/Los_Angeles",
	},
	"19PA": {
		ICAO:      "19PA",
		Name:      "Lake Airport",
		City:      "Millville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 750,
		Latitude:  41.1170005798,
		Longitude: -76.5207977295,
		Timezone:  "America/New_York",
	},
	"19SC": {
		ICAO:      "19SC",
		Name:      "Sexton Airport",
		City:      "Newberry",
		State:     "South Carolina",
		Country:   "US",
		Elevation: 593,
		Latitude:  34.353833,
		Longitude: -81.802358,
		Timezone:  "America/New_York",
	},
	"19TA": {
		ICAO:      "19TA",
		Name:      "Lagrone Ranch Airport",
		City:      "Mc Clendon-Chisholm",
		State:     "Texas",
		Country:   "US",
		Elevation: 567,
		Latitude:  32.8222999573,
		Longitude: -96.4169006348,
		Timezone:  "America/Chicago",
	},
	"19TE": {
		ICAO:      "19TE",
		Name:      "Cut and Shoot Airport",
		City:      "Conroe",
		State:     "Texas",
		Country:   "US",
		Elevation: 170,
		Latitude:  30.3169002533,
		Longitude: -95.3335037231,
		Timezone:  "America/Chicago",
	},
	"19TN": {
		ICAO:      "19TN",
		Name:      "Ferraraccio Field",
		City:      "Clarksville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 620,
		Latitude:  36.4994010925,
		Longitude: -87.1902999878,
		Timezone:  "America/Chicago",
	},
	"19TX": {
		ICAO:      "19TX",
		Name:      "Woody Mc Clellan Ranch Airport",
		City:      "Temple",
		State:     "Texas",
		Country:   "US",
		Elevation: 690,
		Latitude:  31.238199234,
		Longitude: -97.5141983032,
		Timezone:  "America/Chicago",
	},
	"19VA": {
		ICAO:      "19VA",
		Name:      "Flying W Airport",
		City:      "St. Paul",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1760,
		Latitude:  36.8897018433,
		Longitude: -82.2977981567,
		Timezone:  "America/New_York",
	},
	"19WA": {
		ICAO:      "19WA",
		Name:      "Key Way Airport",
		City:      "Stevenson",
		State:     "Washington",
		Country:   "US",
		Elevation: 972,
		Latitude:  45.7196006775,
		Longitude: -121.8860015869,
		Timezone:  "America/Los_Angeles",
	},
	"19WI": {
		ICAO:      "19WI",
		Name:      "Erickson Field",
		City:      "Prairie Farm",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1180,
		Latitude:  45.1952018738,
		Longitude: -92.0570983887,
		Timezone:  "America/Chicago",
	},
	"19XS": {
		ICAO:      "19XS",
		Name:      "Draggintail Acres Airport",
		City:      "Maypearl",
		State:     "Texas",
		Country:   "US",
		Elevation: 585,
		Latitude:  32.2971992493,
		Longitude: -97.0868988037,
		Timezone:  "America/Chicago",
	},
	"1AK1": {
		ICAO:      "1AK1",
		Name:      "Crevice Creek Airport",
		City:      "Crevice Creek",
		State:     "Alaska",
		Country:   "US",
		Elevation: 840,
		Latitude:  67.3729019165,
		Longitude: -152.0160064697,
		Timezone:  "America/Anchorage",
	},
	"1AK2": {
		ICAO:      "1AK2",
		Name:      "Crosswind Lake Airport",
		City:      "Crosswind Lake",
		State:     "Alaska",
		Country:   "US",
		Elevation: 2125,
		Latitude:  62.4009017944,
		Longitude: -146.0130004883,
		Timezone:  "America/Anchorage",
	},
	"1AK4": {
		ICAO:      "1AK4",
		Name:      "Kenai River Airpark",
		City:      "Soldotna",
		State:     "Alaska",
		Country:   "US",
		Elevation: 200,
		Latitude:  60.5242004395,
		Longitude: -150.7519989014,
		Timezone:  "America/Anchorage",
	},
	"1AK6": {
		ICAO:      "1AK6",
		Name:      "Talachulitna River Airport",
		City:      "Sketna",
		State:     "Alaska",
		Country:   "US",
		Elevation: 282,
		Latitude:  61.8527984619,
		Longitude: -151.4080047607,
		Timezone:  "America/Anchorage",
	},
	"1AK8": {
		ICAO:      "1AK8",
		Name:      "Talaheim Airport",
		City:      "Skwentna",
		State:     "Alaska",
		Country:   "US",
		Elevation: 610,
		Latitude:  61.6768989563,
		Longitude: -151.3869934082,
		Timezone:  "America/Anchorage",
	},
	"1AL2": {
		ICAO:      "1AL2",
		Name:      "Tri-L Acres Airport",
		City:      "Columbiana",
		State:     "Alabama",
		Country:   "US",
		Elevation: 580,
		Latitude:  33.2486000061,
		Longitude: -86.5975036621,
		Timezone:  "America/Chicago",
	},
	"1AL4": {
		ICAO:      "1AL4",
		Name:      "Elsanor Airport",
		City:      "Robertsdale",
		State:     "Alabama",
		Country:   "US",
		Elevation: 180,
		Latitude:  30.5447006226,
		Longitude: -87.5594024658,
		Timezone:  "America/Chicago",
	},
	"1AL5": {
		ICAO:      "1AL5",
		Name:      "Berry Field",
		City:      "Loxley",
		State:     "Alabama",
		Country:   "US",
		Elevation: 170,
		Latitude:  30.6364002228,
		Longitude: -87.7478027344,
		Timezone:  "America/Chicago",
	},
	"1AL8": {
		ICAO:      "1AL8",
		Name:      "Moore Field",
		City:      "Moody",
		State:     "Alabama",
		Country:   "US",
		Elevation: 730,
		Latitude:  33.6110992432,
		Longitude: -86.4744033813,
		Timezone:  "America/Chicago",
	},
	"1AR0": {
		ICAO:      "1AR0",
		Name:      "Breckenridge Airport",
		City:      "Beedeville",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 221,
		Latitude:  35.3987007141,
		Longitude: -91.1334991455,
		Timezone:  "America/Chicago",
	},
	"1AR1": {
		ICAO:      "1AR1",
		Name:      "Walls Airport",
		City:      "Coy",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 215,
		Latitude:  34.5271987915,
		Longitude: -91.8727035522,
		Timezone:  "America/Chicago",
	},
	"1AR2": {
		ICAO:      "1AR2",
		Name:      "Taylor Airstrip",
		City:      "Des Arc",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 208,
		Latitude:  34.9868011475,
		Longitude: -91.5503005981,
		Timezone:  "America/Chicago",
	},
	"1AR3": {
		ICAO:      "1AR3",
		Name:      "Bob Norman Airstrip",
		City:      "Des Arc",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 208,
		Latitude:  34.9732017517,
		Longitude: -91.5260009766,
		Timezone:  "America/Chicago",
	},
	"1AR4": {
		ICAO:      "1AR4",
		Name:      "Foggy River Airport",
		City:      "Elkins",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1240,
		Latitude:  36.0178985596,
		Longitude: -94.0184020996,
		Timezone:  "America/Chicago",
	},
	"1AR5": {
		ICAO:      "1AR5",
		Name:      "Cherokee Strip",
		City:      "Gentry",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1194,
		Latitude:  36.2994003296,
		Longitude: -94.5847015381,
		Timezone:  "America/Chicago",
	},
	"1AR6": {
		ICAO:      "1AR6",
		Name:      "Diamond Bluff Airport",
		City:      "Greers Ferry",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1000,
		Latitude:  35.5032997131,
		Longitude: -92.1967010498,
		Timezone:  "America/Chicago",
	},
	"1AR8": {
		ICAO:      "1AR8",
		Name:      "Flying W Air Park Inc Airport",
		City:      "Lavaca",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 420,
		Latitude:  35.3256988525,
		Longitude: -94.1867980957,
		Timezone:  "America/Chicago",
	},
	"1AR9": {
		ICAO:      "1AR9",
		Name:      "Country Air Estates Airport",
		City:      "Lonoke",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 250,
		Latitude:  34.8138999939,
		Longitude: -92,
		Timezone:  "America/Chicago",
	},
	"1AZ0": {
		ICAO:      "1AZ0",
		Name:      "Mobile Airport",
		City:      "Mobile",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1261,
		Latitude:  33.1119003296,
		Longitude: -112.2689971924,
		Timezone:  "America/Phoenix",
	},
	"1AZ2": {
		ICAO:      "1AZ2",
		Name:      "Paradise Air Park",
		City:      "Tolleson",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1015,
		Latitude:  33.4117012024,
		Longitude: -112.1819992065,
		Timezone:  "America/Phoenix",
	},
	"1AZ8": {
		ICAO:      "1AZ8",
		Name:      "Willow Springs Ranch Airport",
		City:      "Bullhead City",
		State:     "Arizona",
		Country:   "US",
		Elevation: 3750,
		Latitude:  35.2957992554,
		Longitude: -114.3730010986,
		Timezone:  "America/Phoenix",
	},
	"1CA1": {
		ICAO:      "1CA1",
		Name:      "Belridge Strip",
		City:      "South Belridge",
		State:     "California",
		Country:   "US",
		Elevation: 575,
		Latitude:  35.4679985046,
		Longitude: -119.7220001221,
		Timezone:  "America/Los_Angeles",
	},
	"1CA4": {
		ICAO:      "1CA4",
		Name:      "Aha-Quin Airport",
		City:      "Blythe",
		State:     "California",
		Country:   "US",
		Elevation: 300,
		Latitude:  33.8694992065,
		Longitude: -114.5270004272,
		Timezone:  "America/Los_Angeles",
	},
	"1CA6": {
		ICAO:      "1CA6",
		Name:      "On the Rocks Airport",
		City:      "Alpine",
		State:     "California",
		Country:   "US",
		Elevation: 2650,
		Latitude:  32.7650985718,
		Longitude: -116.7229995728,
		Timezone:  "America/Los_Angeles",
	},
	"1CD1": {
		ICAO:      "1CD1",
		Name:      "Reed Airport",
		City:      "Nunn",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5350,
		Latitude:  40.6902999878,
		Longitude: -104.8669967651,
		Timezone:  "America/Denver",
	},
	"1CD2": {
		ICAO:      "1CD2",
		Name:      "Tonga Airport",
		City:      "Platteville",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4925,
		Latitude:  40.1671981812,
		Longitude: -104.7929992676,
		Timezone:  "America/Denver",
	},
	"1CD4": {
		ICAO:      "1CD4",
		Name:      "Eagle Soaring Airport",
		City:      "Steam Boat Springs",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6600,
		Latitude:  40.5093002319,
		Longitude: -106.9430007935,
		Timezone:  "America/Denver",
	},
	"1CL1": {
		ICAO:      "1CL1",
		Name:      "Little Buttes Antique Airfield",
		City:      "Lancaster",
		State:     "California",
		Country:   "US",
		Elevation: 2433,
		Latitude:  34.7941017151,
		Longitude: -118.2779998779,
		Timezone:  "America/Los_Angeles",
	},
	"1CL2": {
		ICAO:      "1CL2",
		Name:      "Pontious Airport",
		City:      "Mojave",
		State:     "California",
		Country:   "US",
		Elevation: 2610,
		Latitude:  34.9425010681,
		Longitude: -118.1699981689,
		Timezone:  "America/Los_Angeles",
	},
	"1CO2": {
		ICAO:      "1CO2",
		Name:      "Williams Ranch Airport",
		City:      "Norwood",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6842,
		Latitude:  38.1624984741,
		Longitude: -108.3399963379,
		Timezone:  "America/Denver",
	},
	"1CO3": {
		ICAO:      "1CO3",
		Name:      "Bellmore Farms Airport",
		City:      "Nunn",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5225,
		Latitude:  40.7041015625,
		Longitude: -104.797996521,
		Timezone:  "America/Denver",
	},
	"1CO4": {
		ICAO:      "1CO4",
		Name:      "Clifford Field",
		City:      "Olathe",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5560,
		Latitude:  38.5750007629,
		Longitude: -107.9589996338,
		Timezone:  "America/Denver",
	},
	"1CO5": {
		ICAO:      "1CO5",
		Name:      "Melon Field",
		City:      "Rocky Ford",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4260,
		Latitude:  38.0153007507,
		Longitude: -103.6989974976,
		Timezone:  "America/Denver",
	},
	"1CO7": {
		ICAO:      "1CO7",
		Name:      "Dodsworth Airport",
		City:      "Salida",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7480,
		Latitude:  38.551399231,
		Longitude: -105.9919967651,
		Timezone:  "America/Denver",
	},
	"1CO8": {
		ICAO:      "1CO8",
		Name:      "Everitt Airport",
		City:      "Parker",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6295,
		Latitude:  39.5292015076,
		Longitude: -104.6579971313,
		Timezone:  "America/Denver",
	},
	"1DE5": {
		ICAO:      "1DE5",
		Name:      "Mckeown Airport",
		City:      "Middletown",
		State:     "Delaware",
		Country:   "US",
		Elevation: 60,
		Latitude:  39.4956016541,
		Longitude: -75.7332992554,
		Timezone:  "America/New_York",
	},
	"1FA1": {
		ICAO:      "1FA1",
		Name:      "Post Oak Ranch Airport",
		City:      "Crystal River",
		State:     "Florida",
		Country:   "US",
		Elevation: 25,
		Latitude:  28.8558006287,
		Longitude: -82.5525970459,
		Timezone:  "America/New_York",
	},
	"1FA3": {
		ICAO:      "1FA3",
		Name:      "Pine Island Airport",
		City:      "Fort Myers",
		State:     "Florida",
		Country:   "US",
		Elevation: 12,
		Latitude:  26.6562004089,
		Longitude: -82.1212005615,
		Timezone:  "America/New_York",
	},
	"1FD0": {
		ICAO:      "1FD0",
		Name:      "Lawrence Airport",
		City:      "Grand Ridge",
		State:     "Florida",
		Country:   "US",
		Elevation: 130,
		Latitude:  30.7154998779,
		Longitude: -85.0243988037,
		Timezone:  "America/Chicago",
	},
	"1FD2": {
		ICAO:      "1FD2",
		Name:      "Bass Limited Airport",
		City:      "Madison",
		State:     "Florida",
		Country:   "US",
		Elevation: 100,
		Latitude:  30.6194000244,
		Longitude: -83.3040008545,
		Timezone:  "America/New_York",
	},
	"1FD4": {
		ICAO:      "1FD4",
		Name:      "Corkscrew Trace Airpark",
		City:      "Estero",
		State:     "Florida",
		Country:   "US",
		Elevation: 29,
		Latitude:  26.445400238,
		Longitude: -81.6072998047,
		Timezone:  "America/New_York",
	},
	"1FL3": {
		ICAO:      "1FL3",
		Name:      "Z Ranch Airport",
		City:      "Lake City",
		State:     "Florida",
		Country:   "US",
		Elevation: 75,
		Latitude:  30.0344009399,
		Longitude: -82.7820968628,
		Timezone:  "America/New_York",
	},
	"1GA0": {
		ICAO:      "1GA0",
		Name:      "Eagle Neck Airport",
		City:      "Shellman Bluff",
		State:     "Georgia",
		Country:   "US",
		Elevation: 10,
		Latitude:  31.6394004822,
		Longitude: -81.3290023804,
		Timezone:  "America/New_York",
	},
	"1GA2": {
		ICAO:      "1GA2",
		Name:      "Flying N Estates Airport",
		City:      "Luthersville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 870,
		Latitude:  33.1772994995,
		Longitude: -84.7484970093,
		Timezone:  "America/New_York",
	},
	"1GA4": {
		ICAO:      "1GA4",
		Name:      "Southern Agricultural Aviation Airport",
		City:      "Statesboro",
		State:     "Georgia",
		Country:   "US",
		Elevation: 180,
		Latitude:  32.3284988403,
		Longitude: -81.7554016113,
		Timezone:  "America/New_York",
	},
	"1GA5": {
		ICAO:      "1GA5",
		Name:      "Shilo Farms Airport",
		City:      "Hahira",
		State:     "Georgia",
		Country:   "US",
		Elevation: 211,
		Latitude:  30.9309997559,
		Longitude: -83.3834991455,
		Timezone:  "America/New_York",
	},
	"1GA6": {
		ICAO:      "1GA6",
		Name:      "De De Airport",
		City:      "Barnesville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 800,
		Latitude:  33.0089988708,
		Longitude: -84.1107025146,
		Timezone:  "America/New_York",
	},
	"1GA8": {
		ICAO:      "1GA8",
		Name:      "South One Ten Airport",
		City:      "Nashville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 225,
		Latitude:  31.1784992218,
		Longitude: -83.2265014648,
		Timezone:  "America/New_York",
	},
	"1GA9": {
		ICAO:      "1GA9",
		Name:      "Aerie Airport",
		City:      "Monroe",
		State:     "Georgia",
		Country:   "US",
		Elevation: 830,
		Latitude:  33.7615013123,
		Longitude: -83.6527023315,
		Timezone:  "America/New_York",
	},
	"1GE2": {
		ICAO:      "1GE2",
		Name:      "C & W Air Park",
		City:      "Woodland",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1306,
		Latitude:  32.8443984985,
		Longitude: -84.5438995361,
		Timezone:  "America/New_York",
	},
	"1GE3": {
		ICAO:      "1GE3",
		Name:      "Answered Prayer Airport",
		City:      "Franklin",
		State:     "Georgia",
		Country:   "US",
		Elevation: 890,
		Latitude:  33.2538986206,
		Longitude: -85.1703033447,
		Timezone:  "America/New_York",
	},
	"1GE4": {
		ICAO:      "1GE4",
		Name:      "Beckley Farms Airport",
		City:      "Fort Valley",
		State:     "Georgia",
		Country:   "US",
		Elevation: 425,
		Latitude:  32.5010986328,
		Longitude: -83.9332962036,
		Timezone:  "America/New_York",
	},
	"1GE5": {
		ICAO:      "1GE5",
		Name:      "Flying D Airport",
		City:      "Concord",
		State:     "Georgia",
		Country:   "US",
		Elevation: 838,
		Latitude:  33.1012001038,
		Longitude: -84.3935012817,
		Timezone:  "America/New_York",
	},
	"1GE6": {
		ICAO:      "1GE6",
		Name:      "Forestry-Strip Airport",
		City:      "Patterson",
		State:     "Georgia",
		Country:   "US",
		Elevation: 105,
		Latitude:  31.3724002838,
		Longitude: -82.1234970093,
		Timezone:  "America/New_York",
	},
	"1GE7": {
		ICAO:      "1GE7",
		Name:      "Hay Field",
		City:      "Madison",
		State:     "Georgia",
		Country:   "US",
		Elevation: 650,
		Latitude:  33.51720047,
		Longitude: -83.4396972656,
		Timezone:  "America/New_York",
	},
	"1GE9": {
		ICAO:      "1GE9",
		Name:      "Waverly Landing Airport",
		City:      "Waverly Hall",
		State:     "Georgia",
		Country:   "US",
		Elevation: 675,
		Latitude:  32.7144012451,
		Longitude: -84.7181015015,
		Timezone:  "America/New_York",
	},
	"1IA0": {
		ICAO:      "1IA0",
		Name:      "Brown Truck Sales Airport",
		City:      "Corning",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1300,
		Latitude:  40.8972015381,
		Longitude: -94.7910995483,
		Timezone:  "America/Chicago",
	},
	"1IA4": {
		ICAO:      "1IA4",
		Name:      "Wallace Field",
		City:      "Creston",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1290,
		Latitude:  41.0555000305,
		Longitude: -94.3946990967,
		Timezone:  "America/Chicago",
	},
	"1IA6": {
		ICAO:      "1IA6",
		Name:      "Aero-Lane Airport",
		City:      "Randolph",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1134,
		Latitude:  40.8849983215,
		Longitude: -95.4852981567,
		Timezone:  "America/Chicago",
	},
	"1IA9": {
		ICAO:      "1IA9",
		Name:      "Newbrough Airport",
		City:      "Britt",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1192,
		Latitude:  43.0676994324,
		Longitude: -93.9021987915,
		Timezone:  "America/Chicago",
	},
	"1ID3": {
		ICAO:      "1ID3",
		Name:      "Beaux Ranch Field",
		City:      "Sandpoint",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2079,
		Latitude:  48.1679992676,
		Longitude: -116.7269973755,
		Timezone:  "America/Los_Angeles",
	},
	"1ID4": {
		ICAO:      "1ID4",
		Name:      "Red Baron Airpark",
		City:      "Oasis",
		State:     "Idaho",
		Country:   "US",
		Elevation: 3259,
		Latitude:  43.3105555556,
		Longitude: -115.9025,
		Timezone:  "America/Boise",
	},
	"1ID7": {
		ICAO:      "1ID7",
		Name:      "Fairbanks Airfield",
		City:      "Eden",
		State:     "Idaho",
		Country:   "US",
		Elevation: 3830,
		Latitude:  42.5877990723,
		Longitude: -114.3339996338,
		Timezone:  "America/Boise",
	},
	"1ID9": {
		ICAO:      "1ID9",
		Name:      "Skyline STOLport",
		City:      "Inkom",
		State:     "Idaho",
		Country:   "US",
		Elevation: 6150,
		Latitude:  42.7806015015,
		Longitude: -112.1709976196,
		Timezone:  "America/Boise",
	},
	"1II0": {
		ICAO:      "1II0",
		Name:      "Webster Airport",
		City:      "Homer",
		State:     "Indiana",
		Country:   "US",
		Elevation: 910,
		Latitude:  39.5766983032,
		Longitude: -85.5877990723,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"1II2": {
		ICAO:      "1II2",
		Name:      "Adams Airport",
		City:      "Geneva",
		State:     "Indiana",
		Country:   "US",
		Elevation: 850,
		Latitude:  40.6157989502,
		Longitude: -84.8348007202,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"1II3": {
		ICAO:      "1II3",
		Name:      "Davis Airport",
		City:      "Carlisle",
		State:     "Indiana",
		Country:   "US",
		Elevation: 500,
		Latitude:  38.9570007324,
		Longitude: -87.394203186,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"1II4": {
		ICAO:      "1II4",
		Name:      "Amy Airport",
		City:      "Corydon",
		State:     "Indiana",
		Country:   "US",
		Elevation: 665,
		Latitude:  38.1250991821,
		Longitude: -86.1622009277,
		Timezone:  "America/Kentucky/Louisville",
	},
	"1II5": {
		ICAO:      "1II5",
		Name:      "Van De Mark Airport",
		City:      "Culver",
		State:     "Indiana",
		Country:   "US",
		Elevation: 730,
		Latitude:  41.2472991943,
		Longitude: -86.5030975342,
		Timezone:  "America/Indiana/Knox",
	},
	"1II8": {
		ICAO:      "1II8",
		Name:      "Morris Airport",
		City:      "Greenfield",
		State:     "Indiana",
		Country:   "US",
		Elevation: 870,
		Latitude:  39.814201355,
		Longitude: -85.818901062,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"1IL0": {
		ICAO:      "1IL0",
		Name:      "Barnstorm Field",
		City:      "El Paso",
		State:     "Illinois",
		Country:   "US",
		Elevation: 731,
		Latitude:  40.7750015259,
		Longitude: -89.0010986328,
		Timezone:  "America/Chicago",
	},
	"1IL1": {
		ICAO:      "1IL1",
		Name:      "Horsefeathers Ranch Airport",
		City:      "Irving",
		State:     "Illinois",
		Country:   "US",
		Elevation: 653,
		Latitude:  39.2444992065,
		Longitude: -89.4259033203,
		Timezone:  "America/Chicago",
	},
	"1IL2": {
		ICAO:      "1IL2",
		Name:      "Hobby Hideaway RLA",
		City:      "Delavan",
		State:     "Illinois",
		Country:   "US",
		Elevation: 660,
		Latitude:  40.3470001221,
		Longitude: -89.470703125,
		Timezone:  "America/Chicago",
	},
	"1IL4": {
		ICAO:      "1IL4",
		Name:      "Heitman Aerdrome Airport",
		City:      "Millstadt",
		State:     "Illinois",
		Country:   "US",
		Elevation: 631,
		Latitude:  38.4187011719,
		Longitude: -90.1312026978,
		Timezone:  "America/Chicago",
	},
	"1IL6": {
		ICAO:      "1IL6",
		Name:      "Bally's Strip",
		City:      "Dixon",
		State:     "Illinois",
		Country:   "US",
		Elevation: 786,
		Latitude:  41.8666992188,
		Longitude: -89.4418029785,
		Timezone:  "America/Chicago",
	},
	"1IL8": {
		ICAO:      "1IL8",
		Name:      "Loy Airport",
		City:      "Donovan",
		State:     "Illinois",
		Country:   "US",
		Elevation: 665,
		Latitude:  40.9319992065,
		Longitude: -87.5959014893,
		Timezone:  "America/Chicago",
	},
	"1IL9": {
		ICAO:      "1IL9",
		Name:      "Holland Field",
		City:      "Mc Lean",
		State:     "Illinois",
		Country:   "US",
		Elevation: 715,
		Latitude:  40.3005981445,
		Longitude: -89.1243972778,
		Timezone:  "America/Chicago",
	},
	"1IN0": {
		ICAO:      "1IN0",
		Name:      "Belknap-Icarus Acres Airport",
		City:      "Parker City",
		State:     "Indiana",
		Country:   "US",
		Elevation: 1040,
		Latitude:  40.1333999634,
		Longitude: -85.2304992676,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"1IN1": {
		ICAO:      "1IN1",
		Name:      "Shearer STOLport",
		City:      "Lewisville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 750,
		Latitude:  39.4883995056,
		Longitude: -86.6530990601,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"1IN3": {
		ICAO:      "1IN3",
		Name:      "Confer's Place Airport",
		City:      "Arcola",
		State:     "Indiana",
		Country:   "US",
		Elevation: 865,
		Latitude:  41.0978012085,
		Longitude: -85.2425003052,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"1IN4": {
		ICAO:      "1IN4",
		Name:      "Robinson Airpark",
		City:      "Elizabeth",
		State:     "Indiana",
		Country:   "US",
		Elevation: 775,
		Latitude:  38.1431007385,
		Longitude: -85.9878005981,
		Timezone:  "America/Kentucky/Louisville",
	},
	"1IN5": {
		ICAO:      "1IN5",
		Name:      "Bottoms Brothers Airport",
		City:      "Somerville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 545,
		Latitude:  38.2848014832,
		Longitude: -87.3820037842,
		Timezone:  "America/Chicago",
	},
	"1IN6": {
		ICAO:      "1IN6",
		Name:      "Cottingham Airport",
		City:      "Pine Village",
		State:     "Indiana",
		Country:   "US",
		Elevation: 685,
		Latitude:  40.4016990662,
		Longitude: -87.2202987671,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"1IS0": {
		ICAO:      "1IS0",
		Name:      "Harold Emmerich Airport",
		City:      "Belleville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 515,
		Latitude:  38.5242004395,
		Longitude: -89.9322967529,
		Timezone:  "America/Chicago",
	},
	"1IS2": {
		ICAO:      "1IS2",
		Name:      "Triple Creek Airport",
		City:      "Bement",
		State:     "Illinois",
		Country:   "US",
		Elevation: 666,
		Latitude:  39.8964004517,
		Longitude: -88.5223007202,
		Timezone:  "America/Chicago",
	},
	"1IS3": {
		ICAO:      "1IS3",
		Name:      "Cribbet Airport",
		City:      "Blue Mound",
		State:     "Illinois",
		Country:   "US",
		Elevation: 620,
		Latitude:  39.7277984619,
		Longitude: -89.0595016479,
		Timezone:  "America/Chicago",
	},
	"1IS4": {
		ICAO:      "1IS4",
		Name:      "Swan Valley Farm Airport",
		City:      "Lanark",
		State:     "Illinois",
		Country:   "US",
		Elevation: 916,
		Latitude:  42.1344985962,
		Longitude: -89.8246002197,
		Timezone:  "America/Chicago",
	},
	"1IS5": {
		ICAO:      "1IS5",
		Name:      "Walder's Farm Airport",
		City:      "Paw Paw",
		State:     "Illinois",
		Country:   "US",
		Elevation: 900,
		Latitude:  41.6530990601,
		Longitude: -89.0015029907,
		Timezone:  "America/Chicago",
	},
	"1JY2": {
		ICAO:      "1JY2",
		Name:      "Mahogany Mtn. Airport",
		City:      "Silver Lake",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4620,
		Latitude:  43.0671005249,
		Longitude: -121.0530014038,
		Timezone:  "America/Los_Angeles",
	},
	"1KS0": {
		ICAO:      "1KS0",
		Name:      "Huey Airport",
		City:      "Hutchinson",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1700,
		Latitude:  38.1694984436,
		Longitude: -97.9711990356,
		Timezone:  "America/Chicago",
	},
	"1KS1": {
		ICAO:      "1KS1",
		Name:      "Cochran Airport",
		City:      "Richmond",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1005,
		Latitude:  38.4589004517,
		Longitude: -95.2260971069,
		Timezone:  "America/Chicago",
	},
	"1KS2": {
		ICAO:      "1KS2",
		Name:      "Risky Airport",
		City:      "Easton",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1133,
		Latitude:  39.313999176,
		Longitude: -95.0716018677,
		Timezone:  "America/Chicago",
	},
	"1KS3": {
		ICAO:      "1KS3",
		Name:      "Leo's Place STOLport",
		City:      "Riley",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1350,
		Latitude:  39.3041992188,
		Longitude: -96.7585983276,
		Timezone:  "America/Chicago",
	},
	"1KS4": {
		ICAO:      "1KS4",
		Name:      "Prichard Airstrip",
		City:      "Enterprise",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1164,
		Latitude:  38.9000015259,
		Longitude: -97.1169967651,
		Timezone:  "America/Chicago",
	},
	"1KS5": {
		ICAO:      "1KS5",
		Name:      "Keyser Airport",
		City:      "St Marys",
		State:     "Kansas",
		Country:   "US",
		Elevation: 950,
		Latitude:  39.1624984741,
		Longitude: -96.0963973999,
		Timezone:  "America/Chicago",
	},
	"1KS6": {
		ICAO:      "1KS6",
		Name:      "Silers Plane Valley Airport",
		City:      "Salina",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1204,
		Latitude:  38.8666992188,
		Longitude: -97.5363998413,
		Timezone:  "America/Chicago",
	},
	"1KS7": {
		ICAO:      "1KS7",
		Name:      "Hitch Feeders Ii Inc. Airport",
		City:      "Satanta",
		State:     "Kansas",
		Country:   "US",
		Elevation: 3028,
		Latitude:  37.6735992432,
		Longitude: -101.016998291,
		Timezone:  "America/Chicago",
	},
	"1KS8": {
		ICAO:      "1KS8",
		Name:      "Buehler Airport",
		City:      "Scott City",
		State:     "Kansas",
		Country:   "US",
		Elevation: 3038,
		Latitude:  38.5153007507,
		Longitude: -100.9950027466,
		Timezone:  "America/Chicago",
	},
	"1KS9": {
		ICAO:      "1KS9",
		Name:      "Huff Airport",
		City:      "Bonner Springs",
		State:     "Kansas",
		Country:   "US",
		Elevation: 935,
		Latitude:  39.0456008911,
		Longitude: -94.9489974976,
		Timezone:  "America/Chicago",
	},
	"1KY7": {
		ICAO:      "1KY7",
		Name:      "Jordan Hill Farm Airport",
		City:      "Richmond",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 910,
		Latitude:  37.8325004578,
		Longitude: -84.1808013916,
		Timezone:  "America/New_York",
	},
	"1LA0": {
		ICAO:      "1LA0",
		Name:      "Ken Guidry Nr 2 Airport",
		City:      "Gueydan",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 6,
		Latitude:  30.0168991089,
		Longitude: -92.5279006958,
		Timezone:  "America/Chicago",
	},
	"1LA1": {
		ICAO:      "1LA1",
		Name:      "Triche Field",
		City:      "Hahnville",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 10,
		Latitude:  29.983499527,
		Longitude: -90.4347991943,
		Timezone:  "America/Chicago",
	},
	"1LA3": {
		ICAO:      "1LA3",
		Name:      "Goose Island Airport",
		City:      "Hayes",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 4,
		Latitude:  30.0307998657,
		Longitude: -92.8335037231,
		Timezone:  "America/Chicago",
	},
	"1LA5": {
		ICAO:      "1LA5",
		Name:      "Morrow Strip",
		City:      "Morrow",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 40,
		Latitude:  30.799200058,
		Longitude: -92.0597000122,
		Timezone:  "America/Chicago",
	},
	"1LL0": {
		ICAO:      "1LL0",
		Name:      "Boondox Field",
		City:      "Seneca",
		State:     "Illinois",
		Country:   "US",
		Elevation: 625,
		Latitude:  41.3292007446,
		Longitude: -88.6000976563,
		Timezone:  "America/Chicago",
	},
	"1LL2": {
		ICAO:      "1LL2",
		Name:      "Spring Brook Airport",
		City:      "Seneca",
		State:     "Illinois",
		Country:   "US",
		Elevation: 500,
		Latitude:  41.3005981445,
		Longitude: -88.635597229,
		Timezone:  "America/Chicago",
	},
	"1LL4": {
		ICAO:      "1LL4",
		Name:      "Sies Landing Area Airport",
		City:      "Litchfield",
		State:     "Illinois",
		Country:   "US",
		Elevation: 690,
		Latitude:  39.1833992004,
		Longitude: -89.7001037598,
		Timezone:  "America/Chicago",
	},
	"1LL6": {
		ICAO:      "1LL6",
		Name:      "Janssen Airport",
		City:      "Shannon",
		State:     "Illinois",
		Country:   "US",
		Elevation: 877,
		Latitude:  42.228099823,
		Longitude: -89.7381973267,
		Timezone:  "America/Chicago",
	},
	"1LL7": {
		ICAO:      "1LL7",
		Name:      "Edwin G. Bennett Airport",
		City:      "Sheffield",
		State:     "Illinois",
		Country:   "US",
		Elevation: 690,
		Latitude:  41.3498001099,
		Longitude: -89.7732009888,
		Timezone:  "America/Chicago",
	},
	"1LL8": {
		ICAO:      "1LL8",
		Name:      "Zoomer Field",
		City:      "Sheldon",
		State:     "Illinois",
		Country:   "US",
		Elevation: 680,
		Latitude:  40.788898468,
		Longitude: -87.5584030151,
		Timezone:  "America/Chicago",
	},
	"1LS0": {
		ICAO:      "1LS0",
		Name:      "Cal Mire Field",
		City:      "Gonzales",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 14,
		Latitude:  30.2241001129,
		Longitude: -90.8820037842,
		Timezone:  "America/Chicago",
	},
	"1LS5": {
		ICAO:      "1LS5",
		Name:      "Cameron Airstrip",
		City:      "Cameron",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 5,
		Latitude:  29.8033008575,
		Longitude: -93.334602356,
		Timezone:  "America/Chicago",
	},
	"1LS8": {
		ICAO:      "1LS8",
		Name:      "Summerell Airport",
		City:      "Ferriday",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 64,
		Latitude:  31.682100296,
		Longitude: -91.4987030029,
		Timezone:  "America/Chicago",
	},
	"1MA5": {
		ICAO:      "1MA5",
		Name:      "Unknown Field",
		City:      "Southborough",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 432,
		Latitude:  42.2789993286,
		Longitude: -71.5477981567,
		Timezone:  "America/New_York",
	},
	"1MD1": {
		ICAO:      "1MD1",
		Name:      "Big Oak Farm Airport",
		City:      "Cambridge",
		State:     "Maryland",
		Country:   "US",
		Elevation: 4,
		Latitude:  38.5690002441,
		Longitude: -76.2863006592,
		Timezone:  "America/New_York",
	},
	"1MD3": {
		ICAO:      "1MD3",
		Name:      "Krastel Farms Airport",
		City:      "Kennedyville",
		State:     "Maryland",
		Country:   "US",
		Elevation: 75,
		Latitude:  39.3089981079,
		Longitude: -75.987701416,
		Timezone:  "America/New_York",
	},
	"1MD8": {
		ICAO:      "1MD8",
		Name:      "Mayberry Run Airport",
		City:      "Westminster/Silver Run",
		State:     "Maryland",
		Country:   "US",
		Elevation: 580,
		Latitude:  39.6828994751,
		Longitude: -77.0971984863,
		Timezone:  "America/New_York",
	},
	"1MI1": {
		ICAO:      "1MI1",
		Name:      "Nikkila Farms Airport",
		City:      "Mass",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1100,
		Latitude:  46.7027015686,
		Longitude: -88.9973983765,
		Timezone:  "America/Detroit",
	},
	"1MI3": {
		ICAO:      "1MI3",
		Name:      "Black River Ranch Airport",
		City:      "Onaway",
		State:     "Michigan",
		Country:   "US",
		Elevation: 880,
		Latitude:  45.1958007813,
		Longitude: -84.3217010498,
		Timezone:  "America/Detroit",
	},
	"1MI4": {
		ICAO:      "1MI4",
		Name:      "Ramsy Farm Airport",
		City:      "Perronville",
		State:     "Michigan",
		Country:   "US",
		Elevation: 880,
		Latitude:  45.8389015198,
		Longitude: -87.3224029541,
		Timezone:  "America/Detroit",
	},
	"1MI5": {
		ICAO:      "1MI5",
		Name:      "Mckenzie's Landing Airport",
		City:      "Howell",
		State:     "Michigan",
		Country:   "US",
		Elevation: 955,
		Latitude:  42.603099823,
		Longitude: -83.859703064,
		Timezone:  "America/Detroit",
	},
	"1MI6": {
		ICAO:      "1MI6",
		Name:      "Rosedale Airport",
		City:      "Sault Ste Marie",
		State:     "Michigan",
		Country:   "US",
		Elevation: 675,
		Latitude:  46.3777999878,
		Longitude: -84.3110961914,
		Timezone:  "America/Detroit",
	},
	"1MI7": {
		ICAO:      "1MI7",
		Name:      "Thompson Airport",
		City:      "Selkirk",
		State:     "Michigan",
		Country:   "US",
		Elevation: 857,
		Latitude:  44.3166999817,
		Longitude: -84.0667037964,
		Timezone:  "America/Detroit",
	},
	"1MI8": {
		ICAO:      "1MI8",
		Name:      "Vlachos Acres Airport",
		City:      "Hubbard",
		State:     "Michigan",
		Country:   "US",
		Elevation: 785,
		Latitude:  44.846698761,
		Longitude: -83.6369018555,
		Timezone:  "America/Detroit",
	},
	"1MI9": {
		ICAO:      "1MI9",
		Name:      "Southfork Airport",
		City:      "Marquette",
		State:     "Michigan",
		Country:   "US",
		Elevation: 660,
		Latitude:  46.4721984863,
		Longitude: -87.3500976563,
		Timezone:  "America/Detroit",
	},
	"1MN0": {
		ICAO:      "1MN0",
		Name:      "Wetherbee Farm Airport",
		City:      "Tenney",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 980,
		Latitude:  46.043598175,
		Longitude: -96.5462036133,
		Timezone:  "America/Chicago",
	},
	"1MN1": {
		ICAO:      "1MN1",
		Name:      "Beyer Airport",
		City:      "Tenney",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1000,
		Latitude:  45.9733009338,
		Longitude: -96.564201355,
		Timezone:  "America/Chicago",
	},
	"1MN2": {
		ICAO:      "1MN2",
		Name:      "Hay Acres Airport",
		City:      "Akiey",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1430,
		Latitude:  45.9382019043,
		Longitude: -94.7606964111,
		Timezone:  "America/Chicago",
	},
	"1MN5": {
		ICAO:      "1MN5",
		Name:      "Molnau Airpark",
		City:      "Waconia",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1000,
		Latitude:  44.8436012268,
		Longitude: -93.7399978638,
		Timezone:  "America/Chicago",
	},
	"1MN8": {
		ICAO:      "1MN8",
		Name:      "Sky Harbor Residential Airpark",
		City:      "Webster",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1111,
		Latitude:  44.5257987976,
		Longitude: -93.3268966675,
		Timezone:  "America/Chicago",
	},
	"1MO2": {
		ICAO:      "1MO2",
		Name:      "Hart Airport",
		City:      "Freeman",
		State:     "Missouri",
		Country:   "US",
		Elevation: 825,
		Latitude:  38.5494995117,
		Longitude: -94.476600647,
		Timezone:  "America/Chicago",
	},
	"1MO3": {
		ICAO:      "1MO3",
		Name:      "Lake Viking Airport",
		City:      "Gallatin",
		State:     "Missouri",
		Country:   "US",
		Elevation: 940,
		Latitude:  39.9333992004,
		Longitude: -94.0737991333,
		Timezone:  "America/Chicago",
	},
	"1MO4": {
		ICAO:      "1MO4",
		Name:      "Longwood Mfg Corp Airport",
		City:      "Gallatin",
		State:     "Missouri",
		Country:   "US",
		Elevation: 800,
		Latitude:  39.8917007446,
		Longitude: -93.8640975952,
		Timezone:  "America/Chicago",
	},
	"1MO6": {
		ICAO:      "1MO6",
		Name:      "Double S Ranch Airport",
		City:      "Grandin",
		State:     "Missouri",
		Country:   "US",
		Elevation: 737,
		Latitude:  36.8362007141,
		Longitude: -90.7222976685,
		Timezone:  "America/Chicago",
	},
	"1MO7": {
		ICAO:      "1MO7",
		Name:      "Hines Airport",
		City:      "Gorin",
		State:     "Missouri",
		Country:   "US",
		Elevation: 750,
		Latitude:  40.363899231,
		Longitude: -91.9751968384,
		Timezone:  "America/Chicago",
	},
	"1MO8": {
		ICAO:      "1MO8",
		Name:      "Sherlock Field",
		City:      "Gravois Mills",
		State:     "Missouri",
		Country:   "US",
		Elevation: 800,
		Latitude:  38.2527999878,
		Longitude: -92.8030014038,
		Timezone:  "America/Chicago",
	},
	"1MO9": {
		ICAO:      "1MO9",
		Name:      "Eagle's Point & Red Barn Village Airpark",
		City:      "Greenfield",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1040,
		Latitude:  37.4791984558,
		Longitude: -93.851600647,
		Timezone:  "America/Chicago",
	},
	"1MS0": {
		ICAO:      "1MS0",
		Name:      "Tapley Airport",
		City:      "Shaw",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 129,
		Latitude:  33.5998001099,
		Longitude: -90.7811965942,
		Timezone:  "America/Chicago",
	},
	"1MS2": {
		ICAO:      "1MS2",
		Name:      "Peterson Airport",
		City:      "Goodman",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 240,
		Latitude:  32.9417991638,
		Longitude: -89.9306030273,
		Timezone:  "America/Chicago",
	},
	"1MS6": {
		ICAO:      "1MS6",
		Name:      "Eagles Ridge Airport",
		City:      "Hernando",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 262,
		Latitude:  34.7972984314,
		Longitude: -89.9131011963,
		Timezone:  "America/Chicago",
	},
	"1MS8": {
		ICAO:      "1MS8",
		Name:      "Columbus Air Force Base Aux Field Airfield",
		City:      "Shuqualak",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 260,
		Latitude:  32.9401016235,
		Longitude: -88.5792007446,
		Timezone:  "America/Chicago",
	},
	"1MS9": {
		ICAO:      "1MS9",
		Name:      "Piker-Too Airport",
		City:      "Centreville",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 300,
		Latitude:  31.011800766,
		Longitude: -90.9682006836,
		Timezone:  "America/Chicago",
	},
	"1MT0": {
		ICAO:      "1MT0",
		Name:      "Nine Quarter Circle Ranch Airport",
		City:      "Gallatin Gateway",
		State:     "Montana",
		Country:   "US",
		Elevation: 6974,
		Latitude:  45.067401886,
		Longitude: -111.2969970703,
		Timezone:  "America/Denver",
	},
	"1MT2": {
		ICAO:      "1MT2",
		Name:      "Skyrider Ultralightport",
		City:      "Billings",
		State:     "Montana",
		Country:   "US",
		Elevation: 3540,
		Latitude:  45.9430999756,
		Longitude: -108.5490036011,
		Timezone:  "America/Denver",
	},
	"1MT3": {
		ICAO:      "1MT3",
		Name:      "Wood Strip",
		City:      "Elliston",
		State:     "Montana",
		Country:   "US",
		Elevation: 5100,
		Latitude:  46.5581016541,
		Longitude: -112.4120025635,
		Timezone:  "America/Denver",
	},
	"1MT4": {
		ICAO:      "1MT4",
		Name:      "Davis Airport",
		City:      "Helena",
		State:     "Montana",
		Country:   "US",
		Elevation: 3780,
		Latitude:  46.6884994507,
		Longitude: -111.8710021973,
		Timezone:  "America/Denver",
	},
	"1MT6": {
		ICAO:      "1MT6",
		Name:      "Bobcat Field",
		City:      "Philipsburg",
		State:     "Montana",
		Country:   "US",
		Elevation: 4824,
		Latitude:  46.3568992615,
		Longitude: -113.5100021362,
		Timezone:  "America/Denver",
	},
	"1MT7": {
		ICAO:      "1MT7",
		Name:      "Prill Field",
		City:      "Sand Coulee",
		State:     "Montana",
		Country:   "US",
		Elevation: 3420,
		Latitude:  47.4211006165,
		Longitude: -111.158996582,
		Timezone:  "America/Denver",
	},
	"1MT9": {
		ICAO:      "1MT9",
		Name:      "Wilcox Airport",
		City:      "Billings",
		State:     "Montana",
		Country:   "US",
		Elevation: 3390,
		Latitude:  45.7891998291,
		Longitude: -108.6839981079,
		Timezone:  "America/Denver",
	},
	"1MU0": {
		ICAO:      "1MU0",
		Name:      "Medcalf Field",
		City:      "Republic",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1260,
		Latitude:  37.1666984558,
		Longitude: -93.5626983643,
		Timezone:  "America/Chicago",
	},
	"1MU2": {
		ICAO:      "1MU2",
		Name:      "Smitty's Landing Airport",
		City:      "Ridgely",
		State:     "Missouri",
		Country:   "US",
		Elevation: 800,
		Latitude:  39.4473991394,
		Longitude: -94.6671981812,
		Timezone:  "America/Chicago",
	},
	"1MU4": {
		ICAO:      "1MU4",
		Name:      "Short Air Airport",
		City:      "Warrensburg",
		State:     "Missouri",
		Country:   "US",
		Elevation: 840,
		Latitude:  38.6516990662,
		Longitude: -93.7788009644,
		Timezone:  "America/Chicago",
	},
	"1MU6": {
		ICAO:      "1MU6",
		Name:      "Simerly Airport",
		City:      "Fillmore",
		State:     "Missouri",
		Country:   "US",
		Elevation: 971,
		Latitude:  40.1108016968,
		Longitude: -94.9689025879,
		Timezone:  "America/Chicago",
	},
	"1MU7": {
		ICAO:      "1MU7",
		Name:      "Mooseberry Airport",
		City:      "Jackson",
		State:     "Missouri",
		Country:   "US",
		Elevation: 480,
		Latitude:  37.4653015137,
		Longitude: -89.6119995117,
		Timezone:  "America/Chicago",
	},
	"1MU8": {
		ICAO:      "1MU8",
		Name:      "Church's Landing Airport",
		City:      "Rushville",
		State:     "Missouri",
		Country:   "US",
		Elevation: 826,
		Latitude:  39.5541992188,
		Longitude: -94.9985961914,
		Timezone:  "America/Chicago",
	},
	"1MY1": {
		ICAO:      "1MY1",
		Name:      "Carlson Agricultural Airport",
		City:      "Wendell",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1033,
		Latitude:  45.9852981567,
		Longitude: -96.1943969727,
		Timezone:  "America/Chicago",
	},
	"1NA0": {
		ICAO:      "1NA0",
		Name:      "Bohn Airstrip",
		City:      "Mountain",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1142,
		Latitude:  48.7322006226,
		Longitude: -97.889503479,
		Timezone:  "America/Chicago",
	},
	"1NA5": {
		ICAO:      "1NA5",
		Name:      "Gienger/Box Bar Ranch Airport",
		City:      "Streeter",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1950,
		Latitude:  46.625,
		Longitude: -99.4504013062,
		Timezone:  "America/Chicago",
	},
	"1NA8": {
		ICAO:      "1NA8",
		Name:      "Preszler Airstrip",
		City:      "Braddock",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1795,
		Latitude:  46.6904983521,
		Longitude: -100.0950012207,
		Timezone:  "America/Chicago",
	},
	"1NA9": {
		ICAO:      "1NA9",
		Name:      "Myran Airstrip",
		City:      "Taylor",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 2120,
		Latitude:  47.0332984924,
		Longitude: -102.4960021973,
		Timezone:  "America/Denver",
	},
	"1NC0": {
		ICAO:      "1NC0",
		Name:      "Delk's Airport",
		City:      "Asheboro",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 460,
		Latitude:  35.7220993042,
		Longitude: -79.9413986206,
		Timezone:  "America/New_York",
	},
	"1NC1": {
		ICAO:      "1NC1",
		Name:      "Ron's Ultralight Field",
		City:      "Pittsboro",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 525,
		Latitude:  35.8119010925,
		Longitude: -79.2386016846,
		Timezone:  "America/New_York",
	},
	"1NC2": {
		ICAO:      "1NC2",
		Name:      "Atwell Airport",
		City:      "Mooresville",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 830,
		Latitude:  35.6547012329,
		Longitude: -80.7893981934,
		Timezone:  "America/New_York",
	},
	"1NC3": {
		ICAO:      "1NC3",
		Name:      "Fletcher's Airport",
		City:      "Elizabeth City",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 7,
		Latitude:  36.1806983948,
		Longitude: -76.1491012573,
		Timezone:  "America/New_York",
	},
	"1NC4": {
		ICAO:      "1NC4",
		Name:      "Bell Strip",
		City:      "Pollocksville",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 35,
		Latitude:  34.9953994751,
		Longitude: -77.2576980591,
		Timezone:  "America/New_York",
	},
	"1NC5": {
		ICAO:      "1NC5",
		Name:      "Riley Field",
		City:      "Bunn",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 360,
		Latitude:  35.9332008362,
		Longitude: -78.3486022949,
		Timezone:  "America/New_York",
	},
	"1NC6": {
		ICAO:      "1NC6",
		Name:      "Arant Airport",
		City:      "Wingate",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 535,
		Latitude:  35.0559997559,
		Longitude: -80.4501037598,
		Timezone:  "America/New_York",
	},
	"1NC8": {
		ICAO:      "1NC8",
		Name:      "Lonesome Field",
		City:      "Richfield",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 665,
		Latitude:  35.5158004761,
		Longitude: -80.258102417,
		Timezone:  "America/New_York",
	},
	"1NC9": {
		ICAO:      "1NC9",
		Name:      "Northbrook International Ultraport Ultralightport",
		City:      "Cherryville",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 1030,
		Latitude:  35.4496994019,
		Longitude: -81.426399231,
		Timezone:  "America/New_York",
	},
	"1ND3": {
		ICAO:      "1ND3",
		Name:      "Andvik Airport",
		City:      "Kindred",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 980,
		Latitude:  46.6477012634,
		Longitude: -97.1305999756,
		Timezone:  "America/Chicago",
	},
	"1ND4": {
		ICAO:      "1ND4",
		Name:      "Walkinshaw Airport",
		City:      "Argusville",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 900,
		Latitude:  47.0583000183,
		Longitude: -97.016998291,
		Timezone:  "America/Chicago",
	},
	"1ND7": {
		ICAO:      "1ND7",
		Name:      "Dittmer Airport",
		City:      "Lynchburg",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 943,
		Latitude:  46.8291015625,
		Longitude: -97.266998291,
		Timezone:  "America/Chicago",
	},
	"1NE0": {
		ICAO:      "1NE0",
		Name:      "Higgins Bros Airport",
		City:      "Brownlee",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2860,
		Latitude:  42.3166999817,
		Longitude: -100.4169998169,
		Timezone:  "America/Chicago",
	},
	"1NE1": {
		ICAO:      "1NE1",
		Name:      "Paul Ridder Ranch Airport",
		City:      "Callaway",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2780,
		Latitude:  41.28329849,
		Longitude: -100.0500031,
		Timezone:  "America/Chicago",
	},
	"1NE2": {
		ICAO:      "1NE2",
		Name:      "Witthuhn Airport",
		City:      "Callaway",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2700,
		Latitude:  41.3250007629,
		Longitude: -99.917098999,
		Timezone:  "America/Chicago",
	},
	"1NE5": {
		ICAO:      "1NE5",
		Name:      "Sibbernsen Airport",
		City:      "Washington",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1280,
		Latitude:  41.4061012268,
		Longitude: -96.2855987549,
		Timezone:  "America/Chicago",
	},
	"1NE6": {
		ICAO:      "1NE6",
		Name:      "Miller Airstrip",
		City:      "Dalton",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 4100,
		Latitude:  41.340801239,
		Longitude: -102.7779998779,
		Timezone:  "America/Denver",
	},
	"1NK0": {
		ICAO:      "1NK0",
		Name:      "Roxbury Runway Airport",
		City:      "Roxbury",
		State:     "New York",
		Country:   "US",
		Elevation: 1600,
		Latitude:  42.2972984314,
		Longitude: -74.5482025146,
		Timezone:  "America/New_York",
	},
	"1NK5": {
		ICAO:      "1NK5",
		Name:      "Westtown Airport",
		City:      "Westtown",
		State:     "New York",
		Country:   "US",
		Elevation: 600,
		Latitude:  41.3614997864,
		Longitude: -74.5242996216,
		Timezone:  "America/New_York",
	},
	"1NK6": {
		ICAO:      "1NK6",
		Name:      "Catalano Airfield",
		City:      "Wilmington",
		State:     "New York",
		Country:   "US",
		Elevation: 840,
		Latitude:  44.4236984253,
		Longitude: -73.7496032715,
		Timezone:  "America/New_York",
	},
	"1NK7": {
		ICAO:      "1NK7",
		Name:      "Boonville Inc Airport",
		City:      "Boonville",
		State:     "New York",
		Country:   "US",
		Elevation: 1200,
		Latitude:  43.4667015076,
		Longitude: -75.2496032715,
		Timezone:  "America/New_York",
	},
	"1NK8": {
		ICAO:      "1NK8",
		Name:      "Chenango Bridge Airport",
		City:      "Binghamton",
		State:     "New York",
		Country:   "US",
		Elevation: 940,
		Latitude:  42.1916999817,
		Longitude: -75.8413009644,
		Timezone:  "America/New_York",
	},
	"1NM0": {
		ICAO:      "1NM0",
		Name:      "Me-Own Airport",
		City:      "Silver City",
		State:     "New Mexico",
		Country:   "US",
		Elevation: 7554,
		Latitude:  33.2120018005,
		Longitude: -108.0260009766,
		Timezone:  "America/Denver",
	},
	"1NV1": {
		ICAO:      "1NV1",
		Name:      "Fallon Southwest Airpark",
		City:      "Fallon",
		State:     "Nevada",
		Country:   "US",
		Elevation: 3950,
		Latitude:  39.4152984619,
		Longitude: -118.8369979858,
		Timezone:  "America/Los_Angeles",
	},
	"1NY2": {
		ICAO:      "1NY2",
		Name:      "Kidder Field",
		City:      "Pulaski",
		State:     "New York",
		Country:   "US",
		Elevation: 360,
		Latitude:  43.6166992188,
		Longitude: -76.1626968384,
		Timezone:  "America/New_York",
	},
	"1NY3": {
		ICAO:      "1NY3",
		Name:      "Richland Airport",
		City:      "Richland",
		State:     "New York",
		Country:   "US",
		Elevation: 560,
		Latitude:  43.5666999817,
		Longitude: -76.0329971313,
		Timezone:  "America/New_York",
	},
	"1NY4": {
		ICAO:      "1NY4",
		Name:      "High Banks Farm Landing Area Airport",
		City:      "Saranac",
		State:     "New York",
		Country:   "US",
		Elevation: 1520,
		Latitude:  44.6175003052,
		Longitude: -73.9110031128,
		Timezone:  "America/New_York",
	},
	"1NY5": {
		ICAO:      "1NY5",
		Name:      "Sherman Field",
		City:      "Poughkeepsie",
		State:     "New York",
		Country:   "US",
		Elevation: 625,
		Latitude:  41.638999939,
		Longitude: -73.7428970337,
		Timezone:  "America/New_York",
	},
	"1NY7": {
		ICAO:      "1NY7",
		Name:      "Minard Farms Airport",
		City:      "Clinton Dale",
		State:     "New York",
		Country:   "US",
		Elevation: 450,
		Latitude:  41.7084007263,
		Longitude: -74.0617980957,
		Timezone:  "America/New_York",
	},
	"1NY8": {
		ICAO:      "1NY8",
		Name:      "Killian Airfield",
		City:      "Scipio",
		State:     "New York",
		Country:   "US",
		Elevation: 1020,
		Latitude:  42.8000984192,
		Longitude: -76.5830001831,
		Timezone:  "America/New_York",
	},
	"1NY9": {
		ICAO:      "1NY9",
		Name:      "Caughdenoy Airport",
		City:      "Caughdenoy",
		State:     "New York",
		Country:   "US",
		Elevation: 400,
		Latitude:  43.2658996582,
		Longitude: -76.18800354,
		Timezone:  "America/New_York",
	},
	"1OA1": {
		ICAO:      "1OA1",
		Name:      "Bashore Airport",
		City:      "Pleasant Hill",
		State:     "Ohio",
		Country:   "US",
		Elevation: 940,
		Latitude:  40.0458984375,
		Longitude: -84.3343963623,
		Timezone:  "America/New_York",
	},
	"1OA2": {
		ICAO:      "1OA2",
		Name:      "Mole Airport",
		City:      "Grafton",
		State:     "Ohio",
		Country:   "US",
		Elevation: 854,
		Latitude:  41.2616996765,
		Longitude: -81.9906997681,
		Timezone:  "America/New_York",
	},
	"1OA3": {
		ICAO:      "1OA3",
		Name:      "Barnett Airpark",
		City:      "Blanchester",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1010,
		Latitude:  39.3334007263,
		Longitude: -83.8554992676,
		Timezone:  "America/New_York",
	},
	"1OA4": {
		ICAO:      "1OA4",
		Name:      "Kepes Flying Field",
		City:      "Springfield",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1080,
		Latitude:  39.9235992432,
		Longitude: -83.6616973877,
		Timezone:  "America/New_York",
	},
	"1OA5": {
		ICAO:      "1OA5",
		Name:      "Double J Airport",
		City:      "Fayetteville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 972,
		Latitude:  39.2195014954,
		Longitude: -83.9526977539,
		Timezone:  "America/New_York",
	},
	"1OA6": {
		ICAO:      "1OA6",
		Name:      "Frith Airport",
		City:      "Morrow",
		State:     "Ohio",
		Country:   "US",
		Elevation: 910,
		Latitude:  39.3362007141,
		Longitude: -84.0457992554,
		Timezone:  "America/New_York",
	},
	"1OA7": {
		ICAO:      "1OA7",
		Name:      "Yoder Airport",
		City:      "Louisville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1149,
		Latitude:  40.8417015076,
		Longitude: -81.2415008545,
		Timezone:  "America/New_York",
	},
	"1OA8": {
		ICAO:      "1OA8",
		Name:      "Erdy Farm Airport",
		City:      "Kilbourne",
		State:     "Ohio",
		Country:   "US",
		Elevation: 981,
		Latitude:  40.346698761,
		Longitude: -82.9044036865,
		Timezone:  "America/New_York",
	},
	"1OH1": {
		ICAO:      "1OH1",
		Name:      "S and S Field",
		City:      "Quincy",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1057,
		Latitude:  40.301399231,
		Longitude: -83.9760971069,
		Timezone:  "America/New_York",
	},
	"1OH2": {
		ICAO:      "1OH2",
		Name:      "Dougherty Airport",
		City:      "Bellevue",
		State:     "Ohio",
		Country:   "US",
		Elevation: 807,
		Latitude:  41.2439994812,
		Longitude: -82.8663024902,
		Timezone:  "America/New_York",
	},
	"1OH3": {
		ICAO:      "1OH3",
		Name:      "Mather Field",
		City:      "Castalia",
		State:     "Ohio",
		Country:   "US",
		Elevation: 640,
		Latitude:  41.3891983032,
		Longitude: -82.7627029419,
		Timezone:  "America/New_York",
	},
	"1OH4": {
		ICAO:      "1OH4",
		Name:      "Fortman Airport",
		City:      "St Marys",
		State:     "Ohio",
		Country:   "US",
		Elevation: 885,
		Latitude:  40.5553016663,
		Longitude: -84.3865966797,
		Timezone:  "America/New_York",
	},
	"1OH7": {
		ICAO:      "1OH7",
		Name:      "Derry Landing Strip",
		City:      "Sonora",
		State:     "Ohio",
		Country:   "US",
		Elevation: 905,
		Latitude:  39.9939994812,
		Longitude: -81.8637008667,
		Timezone:  "America/New_York",
	},
	"1OH8": {
		ICAO:      "1OH8",
		Name:      "Lisbon Airfield",
		City:      "South Charleston",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1160,
		Latitude:  39.8666992188,
		Longitude: -83.6333007813,
		Timezone:  "America/New_York",
	},
	"1OH9": {
		ICAO:      "1OH9",
		Name:      "Lewis Airport",
		City:      "Peebles",
		State:     "Ohio",
		Country:   "US",
		Elevation: 862,
		Latitude:  38.9351005554,
		Longitude: -83.3993988037,
		Timezone:  "America/New_York",
	},
	"1OI1": {
		ICAO:      "1OI1",
		Name:      "Pauls Airport",
		City:      "Middlefield",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1191,
		Latitude:  41.4283981323,
		Longitude: -81.0640029907,
		Timezone:  "America/New_York",
	},
	"1OI6": {
		ICAO:      "1OI6",
		Name:      "Stone Airport",
		City:      "New Lebanon",
		State:     "Ohio",
		Country:   "US",
		Elevation: 978,
		Latitude:  39.7700996399,
		Longitude: -84.4166030884,
		Timezone:  "America/New_York",
	},
	"1OI7": {
		ICAO:      "1OI7",
		Name:      "Knight Airport",
		City:      "Norwalk",
		State:     "Ohio",
		Country:   "US",
		Elevation: 925,
		Latitude:  41.1584014893,
		Longitude: -82.5665969849,
		Timezone:  "America/New_York",
	},
	"1OI9": {
		ICAO:      "1OI9",
		Name:      "Mc Colloch's Airport",
		City:      "Piqua",
		State:     "Ohio",
		Country:   "US",
		Elevation: 990,
		Latitude:  40.1772994995,
		Longitude: -84.2944030762,
		Timezone:  "America/New_York",
	},
	"1OK0": {
		ICAO:      "1OK0",
		Name:      "Neversweat Airport",
		City:      "Bristow",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 775,
		Latitude:  35.8945007324,
		Longitude: -96.2794036865,
		Timezone:  "America/Chicago",
	},
	"1OK1": {
		ICAO:      "1OK1",
		Name:      "Dave's Place Airport",
		City:      "Kingfisher",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1160,
		Latitude:  35.8250007629,
		Longitude: -97.8059005737,
		Timezone:  "America/Chicago",
	},
	"1OK2": {
		ICAO:      "1OK2",
		Name:      "Wyatt Airport",
		City:      "Goldsby",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1163,
		Latitude:  35.0987014771,
		Longitude: -97.4698028564,
		Timezone:  "America/Chicago",
	},
	"1OK3": {
		ICAO:      "1OK3",
		Name:      "Wolfe Field Airport",
		City:      "Bray",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1194,
		Latitude:  34.5760993958,
		Longitude: -97.8141021729,
		Timezone:  "America/Chicago",
	},
	"1OK5": {
		ICAO:      "1OK5",
		Name:      "Hohman Airport",
		City:      "Lone Wolf",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1563,
		Latitude:  34.9833984375,
		Longitude: -99.2294998169,
		Timezone:  "America/Chicago",
	},
	"1OK6": {
		ICAO:      "1OK6",
		Name:      "Stamper Ranch Airport",
		City:      "Luther",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1020,
		Latitude:  35.7219009399,
		Longitude: -97.2046966553,
		Timezone:  "America/Chicago",
	},
	"1OK7": {
		ICAO:      "1OK7",
		Name:      "Grimes Airport",
		City:      "Maysville",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1035,
		Latitude:  34.8000984192,
		Longitude: -97.4822006226,
		Timezone:  "America/Chicago",
	},
	"1OK8": {
		ICAO:      "1OK8",
		Name:      "5B Ranch Airport",
		City:      "Edmond",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1085,
		Latitude:  35.729801178,
		Longitude: -97.5413970947,
		Timezone:  "America/Chicago",
	},
	"1OK9": {
		ICAO:      "1OK9",
		Name:      "Hankins Airport",
		City:      "Marietta",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 843,
		Latitude:  33.9751014709,
		Longitude: -97.0892028809,
		Timezone:  "America/Chicago",
	},
	"1OL2": {
		ICAO:      "1OL2",
		Name:      "Steciak Strip",
		City:      "Harrah",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1100,
		Latitude:  35.4889984131,
		Longitude: -97.1920013428,
		Timezone:  "America/Chicago",
	},
	"1OR0": {
		ICAO:      "1OR0",
		Name:      "Sunnyhill Airport",
		City:      "North Bend",
		State:     "Oregon",
		Country:   "US",
		Elevation: 320,
		Latitude:  43.4832000732,
		Longitude: -124.2030029297,
		Timezone:  "America/Los_Angeles",
	},
	"1OR3": {
		ICAO:      "1OR3",
		Name:      "Sunset Air Strip",
		City:      "North Plains",
		State:     "Oregon",
		Country:   "US",
		Elevation: 207,
		Latitude:  45.5914993286,
		Longitude: -123.0100021362,
		Timezone:  "America/Los_Angeles",
	},
	"1OR4": {
		ICAO:      "1OR4",
		Name:      "North Plains Gliderport",
		City:      "North Plains",
		State:     "Oregon",
		Country:   "US",
		Elevation: 210,
		Latitude:  45.6040000916,
		Longitude: -123.0250015259,
		Timezone:  "America/Los_Angeles",
	},
	"1OR5": {
		ICAO:      "1OR5",
		Name:      "Reed Airport",
		City:      "Olex",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1550,
		Latitude:  45.4874000549,
		Longitude: -120.2259979248,
		Timezone:  "America/Los_Angeles",
	},
	"1OR6": {
		ICAO:      "1OR6",
		Name:      "Clackamas Heights Airport",
		City:      "Oregon City",
		State:     "Oregon",
		Country:   "US",
		Elevation: 545,
		Latitude:  45.3713989258,
		Longitude: -122.5540008545,
		Timezone:  "America/Los_Angeles",
	},
	"1OR7": {
		ICAO:      "1OR7",
		Name:      "Skyhill Airport",
		City:      "Oregon City",
		State:     "Oregon",
		Country:   "US",
		Elevation: 735,
		Latitude:  45.2878990173,
		Longitude: -122.4560012817,
		Timezone:  "America/Los_Angeles",
	},
	"1PA0": {
		ICAO:      "1PA0",
		Name:      "Hinaman Acres Airport",
		City:      "Jersey Shore",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 800,
		Latitude:  41.1333999634,
		Longitude: -77.1996994019,
		Timezone:  "America/New_York",
	},
	"1PA3": {
		ICAO:      "1PA3",
		Name:      "Ridgeview Airport",
		City:      "Kellers Church",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 520,
		Latitude:  40.4500999451,
		Longitude: -75.1996002197,
		Timezone:  "America/New_York",
	},
	"1PA4": {
		ICAO:      "1PA4",
		Name:      "Wagner Airport",
		City:      "Lewistown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 520,
		Latitude:  40.5526008606,
		Longitude: -77.6200027466,
		Timezone:  "America/New_York",
	},
	"1PA5": {
		ICAO:      "1PA5",
		Name:      "Beaver Creek Ultralightport",
		City:      "New Galilee",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 928,
		Latitude:  40.830244,
		Longitude: -80.413678,
		Timezone:  "America/New_York",
	},
	"1PA6": {
		ICAO:      "1PA6",
		Name:      "Whittle Airport",
		City:      "Kennett Square",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 345,
		Latitude:  39.8273010254,
		Longitude: -75.7235031128,
		Timezone:  "America/New_York",
	},
	"1PA9": {
		ICAO:      "1PA9",
		Name:      "Schadels Airport",
		City:      "Klingerstown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 700,
		Latitude:  40.6668014526,
		Longitude: -76.6829986572,
		Timezone:  "America/New_York",
	},
	"1PN0": {
		ICAO:      "1PN0",
		Name:      "Bentley Airport",
		City:      "Pottstown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 690,
		Latitude:  40.2014007568,
		Longitude: -75.7142028809,
		Timezone:  "America/New_York",
	},
	"1PN1": {
		ICAO:      "1PN1",
		Name:      "Napodano Airport",
		City:      "Transfer",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1060,
		Latitude:  41.3342018127,
		Longitude: -80.4206008911,
		Timezone:  "America/New_York",
	},
	"1PS0": {
		ICAO:      "1PS0",
		Name:      "Ickes Airport",
		City:      "Osterburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1320,
		Latitude:  40.1918983459,
		Longitude: -78.5646972656,
		Timezone:  "America/New_York",
	},
	"1PS4": {
		ICAO:      "1PS4",
		Name:      "Sankey Airport",
		City:      "Osceola Mills",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1620,
		Latitude:  40.8609008789,
		Longitude: -78.258102417,
		Timezone:  "America/New_York",
	},
	"1PS9": {
		ICAO:      "1PS9",
		Name:      "Wicker & Wings Aerodrome",
		City:      "Trumbauersville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 590,
		Latitude:  40.4034004211,
		Longitude: -75.3704986572,
		Timezone:  "America/New_York",
	},
	"1S2": {
		ICAO:      "1S2",
		Name:      "Darrington Municipal Airport",
		City:      "Darrington",
		State:     "Washington",
		Country:   "US",
		Elevation: 553,
		Latitude:  48.2586125,
		Longitude: -121.6102036,
		Timezone:  "America/Los_Angeles",
	},
	"1SC2": {
		ICAO:      "1SC2",
		Name:      "Gwinn Field",
		City:      "Hopkins",
		State:     "South Carolina",
		Country:   "US",
		Elevation: 390,
		Latitude:  33.9337997437,
		Longitude: -80.7960968018,
		Timezone:  "America/New_York",
	},
	"1SD0": {
		ICAO:      "1SD0",
		Name:      "Dorsey Ranch Airport",
		City:      "Glad Valley",
		State:     "South Dakota",
		Country:   "US",
		Elevation: 2350,
		Latitude:  45.3230018616,
		Longitude: -101.7419967651,
		Timezone:  "America/Denver",
	},
	"1SD1": {
		ICAO:      "1SD1",
		Name:      "Burke Field",
		City:      "Scotland",
		State:     "South Dakota",
		Country:   "US",
		Elevation: 1352,
		Latitude:  43.1654014587,
		Longitude: -97.7309036255,
		Timezone:  "America/Chicago",
	},
	"1SD3": {
		ICAO:      "1SD3",
		Name:      "Turkey Ridge Airport",
		City:      "Hurley",
		State:     "South Dakota",
		Country:   "US",
		Elevation: 1325,
		Latitude:  43.2332992554,
		Longitude: -97.1502990723,
		Timezone:  "America/Chicago",
	},
	"1SD4": {
		ICAO:      "1SD4",
		Name:      "Vig Limousin Airport",
		City:      "Faith",
		State:     "South Dakota",
		Country:   "US",
		Elevation: 2552,
		Latitude:  44.8666992188,
		Longitude: -102.4169998169,
		Timezone:  "America/Denver",
	},
	"1SD5": {
		ICAO:      "1SD5",
		Name:      "Drake Farm Airport",
		City:      "White Lake",
		State:     "South Dakota",
		Country:   "US",
		Elevation: 1659,
		Latitude:  43.7686004639,
		Longitude: -98.7986984253,
		Timezone:  "America/Chicago",
	},
	"1SD6": {
		ICAO:      "1SD6",
		Name:      "Bogner No Ii Airport",
		City:      "Oelrich's",
		State:     "South Dakota",
		Country:   "US",
		Elevation: 3500,
		Latitude:  43.1102981567,
		Longitude: -103.2300033569,
		Timezone:  "America/Denver",
	},
	"1SD8": {
		ICAO:      "1SD8",
		Name:      "Tribitt Airport",
		City:      "Albee",
		State:     "South Dakota",
		Country:   "US",
		Elevation: 1175,
		Latitude:  45.0476989746,
		Longitude: -96.5525970459,
		Timezone:  "America/Chicago",
	},
	"1TA2": {
		ICAO:      "1TA2",
		Name:      "Vigo Park Airport",
		City:      "Tulia",
		State:     "Texas",
		Country:   "US",
		Elevation: 3383,
		Latitude:  34.650100708,
		Longitude: -101.5,
		Timezone:  "America/Chicago",
	},
	"1TA4": {
		ICAO:      "1TA4",
		Name:      "Last Resort Airport",
		City:      "Crockett",
		State:     "Texas",
		Country:   "US",
		Elevation: 420,
		Latitude:  31.4584999084,
		Longitude: -95.2960968018,
		Timezone:  "America/Chicago",
	},
	"1TA5": {
		ICAO:      "1TA5",
		Name:      "Two Leggs Airport",
		City:      "Denver City",
		State:     "Texas",
		Country:   "US",
		Elevation: 3625,
		Latitude:  32.9417991638,
		Longitude: -103,
		Timezone:  "America/Chicago",
	},
	"1TA7": {
		ICAO:      "1TA7",
		Name:      "Thompson Field",
		City:      "Canton",
		State:     "Texas",
		Country:   "US",
		Elevation: 515,
		Latitude:  32.6160011292,
		Longitude: -95.8683013916,
		Timezone:  "America/Chicago",
	},
	"1TE0": {
		ICAO:      "1TE0",
		Name:      "Locker Brothers Airport",
		City:      "Muleshoe",
		State:     "Texas",
		Country:   "US",
		Elevation: 3802,
		Latitude:  34.2695007324,
		Longitude: -102.7170028687,
		Timezone:  "America/Chicago",
	},
	"1TE1": {
		ICAO:      "1TE1",
		Name:      "Briscoes Catarina Ranch Airport",
		City:      "Catarina",
		State:     "Texas",
		Country:   "US",
		Elevation: 650,
		Latitude:  28.2502994537,
		Longitude: -99.8170013428,
		Timezone:  "America/Chicago",
	},
	"1TE2": {
		ICAO:      "1TE2",
		Name:      "Flying F Ranch Airport",
		City:      "Crosby",
		State:     "Texas",
		Country:   "US",
		Elevation: 65,
		Latitude:  29.9862995148,
		Longitude: -95.0168991089,
		Timezone:  "America/Chicago",
	},
	"1TE3": {
		ICAO:      "1TE3",
		Name:      "Bolton Airport",
		City:      "Jacksonville",
		State:     "Texas",
		Country:   "US",
		Elevation: 450,
		Latitude:  31.9174003601,
		Longitude: -95.2071990967,
		Timezone:  "America/Chicago",
	},
	"1TE4": {
		ICAO:      "1TE4",
		Name:      "Zuehl Airport",
		City:      "Marion",
		State:     "Texas",
		Country:   "US",
		Elevation: 592,
		Latitude:  29.4955005646,
		Longitude: -98.1577987671,
		Timezone:  "America/Chicago",
	},
	"1TE5": {
		ICAO:      "1TE5",
		Name:      "Corpora Airport",
		City:      "Hearne",
		State:     "Texas",
		Country:   "US",
		Elevation: 270,
		Latitude:  30.8162994385,
		Longitude: -96.6010971069,
		Timezone:  "America/Chicago",
	},
	"1TE6": {
		ICAO:      "1TE6",
		Name:      "Moore Airport",
		City:      "Wildorado",
		State:     "Texas",
		Country:   "US",
		Elevation: 3850,
		Latitude:  35.1180992126,
		Longitude: -102.1849975586,
		Timezone:  "America/Chicago",
	},
	"1TE7": {
		ICAO:      "1TE7",
		Name:      "Ray Farm Airport",
		City:      "Floresville",
		State:     "Texas",
		Country:   "US",
		Elevation: 560,
		Latitude:  29.121099472,
		Longitude: -98.3656005859,
		Timezone:  "America/Chicago",
	},
	"1TE8": {
		ICAO:      "1TE8",
		Name:      "Kahuna Bay Airport",
		City:      "Howe",
		State:     "Texas",
		Country:   "US",
		Elevation: 702,
		Latitude:  33.5489997864,
		Longitude: -96.5582962036,
		Timezone:  "America/Chicago",
	},
	"1TE9": {
		ICAO:      "1TE9",
		Name:      "Rmr Ranch Airport",
		City:      "Franklin",
		State:     "Texas",
		Country:   "US",
		Elevation: 420,
		Latitude:  31.271900177,
		Longitude: -96.439201355,
		Timezone:  "America/Chicago",
	},
	"1TN0": {
		ICAO:      "1TN0",
		Name:      "Cedar Crest Field",
		City:      "Lebanon",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 540,
		Latitude:  36.2463989258,
		Longitude: -86.3028030396,
		Timezone:  "America/Chicago",
	},
	"1TN2": {
		ICAO:      "1TN2",
		Name:      "Kay Airport",
		City:      "Rogersville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1250,
		Latitude:  36.4458999634,
		Longitude: -83.1123962402,
		Timezone:  "America/New_York",
	},
	"1TN3": {
		ICAO:      "1TN3",
		Name:      "Wagner Field",
		City:      "Belvidere",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 975,
		Latitude:  35.1397018433,
		Longitude: -86.2221984863,
		Timezone:  "America/Chicago",
	},
	"1TN6": {
		ICAO:      "1TN6",
		Name:      "Higdon Airfield",
		City:      "Karns",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1000,
		Latitude:  35.9725990295,
		Longitude: -84.1568984985,
		Timezone:  "America/New_York",
	},
	"1TS0": {
		ICAO:      "1TS0",
		Name:      "Eagle Air Airport",
		City:      "Oak Island",
		State:     "Texas",
		Country:   "US",
		Elevation: 8,
		Latitude:  29.6704998016,
		Longitude: -94.6707992554,
		Timezone:  "America/Chicago",
	},
	"1TS1": {
		ICAO:      "1TS1",
		Name:      "Laas Farm Airport",
		City:      "Pattison",
		State:     "Texas",
		Country:   "US",
		Elevation: 165,
		Latitude:  29.9127006531,
		Longitude: -95.994102478,
		Timezone:  "America/Chicago",
	},
	"1TS2": {
		ICAO:      "1TS2",
		Name:      "Bar 16 Airport",
		City:      "Hubbard",
		State:     "Texas",
		Country:   "US",
		Elevation: 547,
		Latitude:  31.824300766,
		Longitude: -96.778503418,
		Timezone:  "America/Chicago",
	},
	"1TS3": {
		ICAO:      "1TS3",
		Name:      "B & S Ultralightport",
		City:      "San Leon",
		State:     "Texas",
		Country:   "US",
		Elevation: 16,
		Latitude:  29.496099472,
		Longitude: -94.9505004883,
		Timezone:  "America/Chicago",
	},
	"1TS6": {
		ICAO:      "1TS6",
		Name:      "Dentonio Ranch Airport",
		City:      "Carrizo Springs",
		State:     "Texas",
		Country:   "US",
		Elevation: 825,
		Latitude:  28.2693996429,
		Longitude: -99.9345016479,
		Timezone:  "America/Chicago",
	},
	"1TS8": {
		ICAO:      "1TS8",
		Name:      "Landers Ranch Airport",
		City:      "Colorado City",
		State:     "Texas",
		Country:   "US",
		Elevation: 2275,
		Latitude:  32.2876014709,
		Longitude: -100.7220001221,
		Timezone:  "America/Chicago",
	},
	"1TS9": {
		ICAO:      "1TS9",
		Name:      "Four Winds Airport",
		City:      "Celina",
		State:     "Texas",
		Country:   "US",
		Elevation: 775,
		Latitude:  33.368056,
		Longitude: -96.752778,
		Timezone:  "America/Chicago",
	},
	"1TX0": {
		ICAO:      "1TX0",
		Name:      "Lytle Airpark",
		City:      "Lytle",
		State:     "Texas",
		Country:   "US",
		Elevation: 780,
		Latitude:  29.2533340454,
		Longitude: -98.861114502,
		Timezone:  "America/Chicago",
	},
	"1TX1": {
		ICAO:      "1TX1",
		Name:      "Yoakum Airport",
		City:      "Gustine",
		State:     "Texas",
		Country:   "US",
		Elevation: 1130,
		Latitude:  31.8556995392,
		Longitude: -98.4141998291,
		Timezone:  "America/Chicago",
	},
	"1TX2": {
		ICAO:      "1TX2",
		Name:      "J Y Ranch-R B Masterson Iii Estate Airport",
		City:      "Guthrie",
		State:     "Texas",
		Country:   "US",
		Elevation: 1808,
		Latitude:  33.7168006897,
		Longitude: -100.1500015259,
		Timezone:  "America/Chicago",
	},
	"1TX3": {
		ICAO:      "1TX3",
		Name:      "Beggs Ranch Airport",
		City:      "Guthrie",
		State:     "Texas",
		Country:   "US",
		Elevation: 1870,
		Latitude:  33.4250984192,
		Longitude: -100.5419998169,
		Timezone:  "America/Chicago",
	},
	"1TX4": {
		ICAO:      "1TX4",
		Name:      "Shoreline Ranch Airport",
		City:      "Lago Vista",
		State:     "Texas",
		Country:   "US",
		Elevation: 900,
		Latitude:  30.4286003113,
		Longitude: -97.9704971313,
		Timezone:  "America/Chicago",
	},
	"1TX5": {
		ICAO:      "1TX5",
		Name:      "Laney Farm Airport",
		City:      "Hale Center",
		State:     "Texas",
		Country:   "US",
		Elevation: 3483,
		Latitude:  34.1291999817,
		Longitude: -101.908996582,
		Timezone:  "America/Chicago",
	},
	"1TX6": {
		ICAO:      "1TX6",
		Name:      "Muxworthy Airport",
		City:      "Hamilton",
		State:     "Texas",
		Country:   "US",
		Elevation: 1210,
		Latitude:  31.6984996796,
		Longitude: -98.0963973999,
		Timezone:  "America/Chicago",
	},
	"1TX7": {
		ICAO:      "1TX7",
		Name:      "Killion Ranch Airport",
		City:      "Hamlin",
		State:     "Texas",
		Country:   "US",
		Elevation: 1845,
		Latitude:  32.8056983948,
		Longitude: -100.1839981079,
		Timezone:  "America/Chicago",
	},
	"1TX8": {
		ICAO:      "1TX8",
		Name:      "Hart Aerial Airport",
		City:      "Hart",
		State:     "Texas",
		Country:   "US",
		Elevation: 3655,
		Latitude:  34.3694992065,
		Longitude: -102.0839996338,
		Timezone:  "America/Chicago",
	},
	"1TX9": {
		ICAO:      "1TX9",
		Name:      "Hawk Ranch Airport",
		City:      "Hawkins",
		State:     "Texas",
		Country:   "US",
		Elevation: 335,
		Latitude:  32.6375999451,
		Longitude: -95.1613006592,
		Timezone:  "America/Chicago",
	},
	"1VA0": {
		ICAO:      "1VA0",
		Name:      "Sabot Airport",
		City:      "Sabot",
		State:     "Virginia",
		Country:   "US",
		Elevation: 255,
		Latitude:  37.6292991638,
		Longitude: -77.7474975586,
		Timezone:  "America/New_York",
	},
	"1VA1": {
		ICAO:      "1VA1",
		Name:      "Micro Airport",
		City:      "Stuart",
		State:     "Virginia",
		Country:   "US",
		Elevation: 2923,
		Latitude:  36.7351989746,
		Longitude: -80.4487991333,
		Timezone:  "America/New_York",
	},
	"1VA3": {
		ICAO:      "1VA3",
		Name:      "Tye River Airport",
		City:      "Lovingston",
		State:     "Virginia",
		Country:   "US",
		Elevation: 650,
		Latitude:  37.6581993103,
		Longitude: -78.945602417,
		Timezone:  "America/New_York",
	},
	"1VA5": {
		ICAO:      "1VA5",
		Name:      "Hoffman's Farm Airport",
		City:      "Severn",
		State:     "Virginia",
		Country:   "US",
		Elevation: 3,
		Latitude:  37.270401001,
		Longitude: -76.3927001953,
		Timezone:  "America/New_York",
	},
	"1VA7": {
		ICAO:      "1VA7",
		Name:      "Aaron Penston Field",
		City:      "South Boston",
		State:     "Virginia",
		Country:   "US",
		Elevation: 413,
		Latitude:  36.6660003662,
		Longitude: -78.7225036621,
		Timezone:  "America/New_York",
	},
	"1VA8": {
		ICAO:      "1VA8",
		Name:      "Hazelswart Airport",
		City:      "Chase City",
		State:     "Virginia",
		Country:   "US",
		Elevation: 485,
		Latitude:  36.8619003296,
		Longitude: -78.4111022949,
		Timezone:  "America/New_York",
	},
	"1VA9": {
		ICAO:      "1VA9",
		Name:      "Flying T Farm Airport",
		City:      "Stafford",
		State:     "Virginia",
		Country:   "US",
		Elevation: 255,
		Latitude:  38.4328994751,
		Longitude: -77.4704971313,
		Timezone:  "America/New_York",
	},
	"1VG2": {
		ICAO:      "1VG2",
		Name:      "Whithall Farm Airport",
		City:      "Montross",
		State:     "Virginia",
		Country:   "US",
		Elevation: 23,
		Latitude:  38.1277008057,
		Longitude: -76.689201355,
		Timezone:  "America/New_York",
	},
	"1WA2": {
		ICAO:      "1WA2",
		Name:      "Pilot's Pastures Airport",
		City:      "Winlock",
		State:     "Washington",
		Country:   "US",
		Elevation: 450,
		Latitude:  46.5191993713,
		Longitude: -122.8899993896,
		Timezone:  "America/Los_Angeles",
	},
	"1WA6": {
		ICAO:      "1WA6",
		Name:      "Fall City Airport",
		City:      "Fall City",
		State:     "Washington",
		Country:   "US",
		Elevation: 140,
		Latitude:  47.5595016479,
		Longitude: -121.8639984131,
		Timezone:  "America/Los_Angeles",
	},
	"1WA9": {
		ICAO:      "1WA9",
		Name:      "Friday West Airport",
		City:      "Friday Harbor",
		State:     "Washington",
		Country:   "US",
		Elevation: 150,
		Latitude:  48.5350990295,
		Longitude: -123.0449981689,
		Timezone:  "America/Los_Angeles",
	},
	"1WI0": {
		ICAO:      "1WI0",
		Name:      "Spring Valley Farm Airport",
		City:      "Orfordville",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 850,
		Latitude:  42.6425018311,
		Longitude: -89.3112030029,
		Timezone:  "America/Chicago",
	},
	"1WI1": {
		ICAO:      "1WI1",
		Name:      "Williams Airport",
		City:      "Oshkosh",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 800,
		Latitude:  43.932800293,
		Longitude: -88.7045974731,
		Timezone:  "America/Chicago",
	},
	"1WI2": {
		ICAO:      "1WI2",
		Name:      "Flying S Ranch Airport",
		City:      "Port Washington",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 705,
		Latitude:  43.3652992249,
		Longitude: -87.8870010376,
		Timezone:  "America/Chicago",
	},
	"1WI3": {
		ICAO:      "1WI3",
		Name:      "Bed-Ah-Wick Field",
		City:      "Princeton",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 787,
		Latitude:  43.8499984741,
		Longitude: -89.1167984009,
		Timezone:  "America/Chicago",
	},
	"1WI6": {
		ICAO:      "1WI6",
		Name:      "Aero Estates Airport",
		City:      "Raymond Township",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 825,
		Latitude:  42.8403015137,
		Longitude: -88.0640029907,
		Timezone:  "America/Chicago",
	},
	"1WI7": {
		ICAO:      "1WI7",
		Name:      "Buzzards Roost Airport",
		City:      "Redgranite",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 805,
		Latitude:  44.0222015381,
		Longitude: -89.096496582,
		Timezone:  "America/Chicago",
	},
	"1WI8": {
		ICAO:      "1WI8",
		Name:      "Jorgensen - Stoller Airport",
		City:      "Algoma",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 690,
		Latitude:  44.635799408,
		Longitude: -87.4402008057,
		Timezone:  "America/Chicago",
	},
	"1WI9": {
		ICAO:      "1WI9",
		Name:      "Blackhawk Island Airport",
		City:      "Fort Atkinson",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 800,
		Latitude:  42.913898468,
		Longitude: -88.8667984009,
		Timezone:  "America/Chicago",
	},
	"1WN0": {
		ICAO:      "1WN0",
		Name:      "Schmidt Ranch Airport",
		City:      "Northport",
		State:     "Washington",
		Country:   "US",
		Elevation: 1335,
		Latitude:  48.933381,
		Longitude: -117.749808,
		Timezone:  "America/Los_Angeles",
	},
	"1WN1": {
		ICAO:      "1WN1",
		Name:      "Kettle Moraine Airport",
		City:      "Dundee",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1040,
		Latitude:  43.6679992676,
		Longitude: -88.1953964233,
		Timezone:  "America/Chicago",
	},
	"1WN2": {
		ICAO:      "1WN2",
		Name:      "Covered Bridge Fields Airport",
		City:      "Cedarburg",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 875,
		Latitude:  43.3292007446,
		Longitude: -87.9972991943,
		Timezone:  "America/Chicago",
	},
	"1WN3": {
		ICAO:      "1WN3",
		Name:      "Danielson Field Airport",
		City:      "Star Prairie",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1000,
		Latitude:  45.2118988037,
		Longitude: -92.5360031128,
		Timezone:  "America/Chicago",
	},
	"1WN5": {
		ICAO:      "1WN5",
		Name:      "Simons Airfield",
		City:      "Land-O-Lakes",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1735,
		Latitude:  46.1547012329,
		Longitude: -89.3174972534,
		Timezone:  "America/Chicago",
	},
	"1XA0": {
		ICAO:      "1XA0",
		Name:      "Rab Ranch Airport",
		City:      "Andice",
		State:     "Texas",
		Country:   "US",
		Elevation: 1095,
		Latitude:  30.79369926,
		Longitude: -97.90119934,
		Timezone:  "America/Chicago",
	},
	"1XA1": {
		ICAO:      "1XA1",
		Name:      "Sky Acres Airport",
		City:      "Bowie",
		State:     "Texas",
		Country:   "US",
		Elevation: 1070,
		Latitude:  33.6477012634,
		Longitude: -97.8290023804,
		Timezone:  "America/Chicago",
	},
	"1XA2": {
		ICAO:      "1XA2",
		Name:      "Chase Field Industrial Airport",
		City:      "Beeville",
		State:     "Texas",
		Country:   "US",
		Elevation: 190,
		Latitude:  28.4349994659,
		Longitude: -97.6517028809,
		Timezone:  "America/Chicago",
	},
	"1XA4": {
		ICAO:      "1XA4",
		Name:      "Mikeska Field",
		City:      "Brookshire",
		State:     "Texas",
		Country:   "US",
		Elevation: 161,
		Latitude:  29.8738002777,
		Longitude: -96.0044021606,
		Timezone:  "America/Chicago",
	},
	"1XA5": {
		ICAO:      "1XA5",
		Name:      "Flying Armadillo Field",
		City:      "Paige",
		State:     "Texas",
		Country:   "US",
		Elevation: 410,
		Latitude:  30.1350002289,
		Longitude: -97.1428985596,
		Timezone:  "America/Chicago",
	},
	"1XA6": {
		ICAO:      "1XA6",
		Name:      "Tailwheel Acres Airport",
		City:      "Valley View",
		State:     "Texas",
		Country:   "US",
		Elevation: 702,
		Latitude:  33.4737014771,
		Longitude: -97.1223983765,
		Timezone:  "America/Chicago",
	},
	"1XA7": {
		ICAO:      "1XA7",
		Name:      "Gloster Aerodrome",
		City:      "Sealy",
		State:     "Texas",
		Country:   "US",
		Elevation: 145,
		Latitude:  29.7334003448,
		Longitude: -96.0591964722,
		Timezone:  "America/Chicago",
	},
	"1XA8": {
		ICAO:      "1XA8",
		Name:      "Hackberry Airport",
		City:      "Chatfield",
		State:     "Texas",
		Country:   "US",
		Elevation: 433,
		Latitude:  32.2324981689,
		Longitude: -96.3974990845,
		Timezone:  "America/Chicago",
	},
	"1XS0": {
		ICAO:      "1XS0",
		Name:      "Double U Ranch Airport",
		City:      "Spring Branch",
		State:     "Texas",
		Country:   "US",
		Elevation: 1170,
		Latitude:  29.9223995209,
		Longitude: -98.4809036255,
		Timezone:  "America/Chicago",
	},
	"1XS1": {
		ICAO:      "1XS1",
		Name:      "Dunham Field",
		City:      "Crosby",
		State:     "Texas",
		Country:   "US",
		Elevation: 60,
		Latitude:  29.9382991791,
		Longitude: -95.0466003418,
		Timezone:  "America/Chicago",
	},
	"1XS2": {
		ICAO:      "1XS2",
		Name:      "Skye Dance Airport",
		City:      "Elgin",
		State:     "Texas",
		Country:   "US",
		Elevation: 532,
		Latitude:  30.3808002472,
		Longitude: -97.4274978638,
		Timezone:  "America/Chicago",
	},
	"1XS4": {
		ICAO:      "1XS4",
		Name:      "Barton Field",
		City:      "Taylor",
		State:     "Texas",
		Country:   "US",
		Elevation: 530,
		Latitude:  30.4535007477,
		Longitude: -97.2555007935,
		Timezone:  "America/Chicago",
	},
	"1XS7": {
		ICAO:      "1XS7",
		Name:      "Heath Canyon Airport",
		City:      "Terlingua",
		State:     "Texas",
		Country:   "US",
		Elevation: 1857,
		Latitude:  29.4501991272,
		Longitude: -102.8320007324,
		Timezone:  "America/Chicago",
	},
	"1XS8": {
		ICAO:      "1XS8",
		Name:      "Pinon Ranch Airport",
		City:      "Brackettville",
		State:     "Texas",
		Country:   "US",
		Elevation: 1640,
		Latitude:  29.6340999603,
		Longitude: -100.3740005493,
		Timezone:  "America/Chicago",
	},
	"1XS9": {
		ICAO:      "1XS9",
		Name:      "Beefmaster's Best Airport",
		City:      "Three Rivers",
		State:     "Texas",
		Country:   "US",
		Elevation: 250,
		Latitude:  28.3955993652,
		Longitude: -98.2208023071,
		Timezone:  "America/Chicago",
	},
	"20AK": {
		ICAO:      "20AK",
		Name:      "Owen Field",
		City:      "Big Lake",
		State:     "Alaska",
		Country:   "US",
		Elevation: 150,
		Latitude:  61.503789,
		Longitude: -149.896044,
		Timezone:  "America/Anchorage",
	},
	"20AR": {
		ICAO:      "20AR",
		Name:      "Rak Airport",
		City:      "Guy",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 660,
		Latitude:  35.30789948,
		Longitude: -92.32019806,
		Timezone:  "America/Chicago",
	},
	"20AZ": {
		ICAO:      "20AZ",
		Name:      "Eds Field",
		City:      "Picacho",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1644,
		Latitude:  32.6833992004,
		Longitude: -111.5009994507,
		Timezone:  "America/Phoenix",
	},
	"20CL": {
		ICAO:      "20CL",
		Name:      "Mysterious Valley Airport",
		City:      "Pope Valley",
		State:     "California",
		Country:   "US",
		Elevation: 1410,
		Latitude:  38.7492980957,
		Longitude: -122.3669967651,
		Timezone:  "America/Los_Angeles",
	},
	"20FA": {
		ICAO:      "20FA",
		Name:      "Golden Land Ranch Airport",
		City:      "Okeechobee",
		State:     "Florida",
		Country:   "US",
		Elevation: 43,
		Latitude:  27.3962001801,
		Longitude: -80.9206008911,
		Timezone:  "America/New_York",
	},
	"20FD": {
		ICAO:      "20FD",
		Name:      "Pratt Airport",
		City:      "Greenville",
		State:     "Florida",
		Country:   "US",
		Elevation: 100,
		Latitude:  30.4255008698,
		Longitude: -83.5759963989,
		Timezone:  "America/New_York",
	},
	"20FL": {
		ICAO:      "20FL",
		Name:      "Lake Suzy Estates Airport",
		City:      "Port Charlotte",
		State:     "Florida",
		Country:   "US",
		Elevation: 25,
		Latitude:  27.0426006317,
		Longitude: -82.0437011719,
		Timezone:  "America/New_York",
	},
	"20GA": {
		ICAO:      "20GA",
		Name:      "Earl L. Small Jr. Field/Stockmar Airport",
		City:      "Villa Rica",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1110,
		Latitude:  33.7565002441,
		Longitude: -84.8846969604,
		Timezone:  "America/New_York",
	},
	"20ID": {
		ICAO:      "20ID",
		Name:      "Harrington Airport",
		City:      "Idaho City",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4200,
		Latitude:  43.720439,
		Longitude: -116.005667,
		Timezone:  "America/Boise",
	},
	"20II": {
		ICAO:      "20II",
		Name:      "Kay Air Airport",
		City:      "Mooresville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 651,
		Latitude:  39.5872001648,
		Longitude: -86.2755966187,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"20IN": {
		ICAO:      "20IN",
		Name:      "Canary's Airport",
		City:      "Franklin",
		State:     "Indiana",
		Country:   "US",
		Elevation: 760,
		Latitude:  39.5139007568,
		Longitude: -86.051399231,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"20IS": {
		ICAO:      "20IS",
		Name:      "Mitek Airport",
		City:      "Winslow",
		State:     "Illinois",
		Country:   "US",
		Elevation: 777,
		Latitude:  42.4972000122,
		Longitude: -89.7918014526,
		Timezone:  "America/Chicago",
	},
	"20KS": {
		ICAO:      "20KS",
		Name:      "Airpark Estates Airport",
		City:      "Augusta",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1340,
		Latitude:  37.6758003235,
		Longitude: -96.8852996826,
		Timezone:  "America/Chicago",
	},
	"20KY": {
		ICAO:      "20KY",
		Name:      "Madi's Meadows Airport",
		City:      "Verona",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 880,
		Latitude:  38.8168983459,
		Longitude: -84.6710968018,
		Timezone:  "America/New_York",
	},
	"20LS": {
		ICAO:      "20LS",
		Name:      "Clark Field",
		City:      "Zachary",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 62,
		Latitude:  30.6229000092,
		Longitude: -91.0329971313,
		Timezone:  "America/Chicago",
	},
	"20MO": {
		ICAO:      "20MO",
		Name:      "Royal Wood Aerodrome",
		City:      "Kearney",
		State:     "Missouri",
		Country:   "US",
		Elevation: 860,
		Latitude:  39.3582992554,
		Longitude: -94.3169021606,
		Timezone:  "America/Chicago",
	},
	"20NC": {
		ICAO:      "20NC",
		Name:      "Mountain View Airport",
		City:      "Sherrills Ford",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 1020,
		Latitude:  35.5923995972,
		Longitude: -81.0559005737,
		Timezone:  "America/New_York",
	},
	"20ND": {
		ICAO:      "20ND",
		Name:      "Crooked Lake Airstrip",
		City:      "Turtle Lake",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1900,
		Latitude:  47.6539001465,
		Longitude: -100.8909988403,
		Timezone:  "America/Chicago",
	},
	"20NY": {
		ICAO:      "20NY",
		Name:      "Stafford Airport",
		City:      "Stafford",
		State:     "New York",
		Country:   "US",
		Elevation: 891,
		Latitude:  43.0088996887,
		Longitude: -78.0430984497,
		Timezone:  "America/New_York",
	},
	"20OH": {
		ICAO:      "20OH",
		Name:      "Kruggel Airport",
		City:      "Litchfield",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1055,
		Latitude:  41.1431007385,
		Longitude: -82.028503418,
		Timezone:  "America/New_York",
	},
	"20OK": {
		ICAO:      "20OK",
		Name:      "Will and Wiley's Aerodrome",
		City:      "Claremore",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 700,
		Latitude:  36.3256988525,
		Longitude: -95.6468963623,
		Timezone:  "America/Chicago",
	},
	"20OR": {
		ICAO:      "20OR",
		Name:      "Warner's Airport",
		City:      "Redland",
		State:     "Oregon",
		Country:   "US",
		Elevation: 325,
		Latitude:  45.3250007629,
		Longitude: -122.4240036011,
		Timezone:  "America/Los_Angeles",
	},
	"20SC": {
		ICAO:      "20SC",
		Name:      "O Neal Field",
		City:      "Greer",
		State:     "South Carolina",
		Country:   "US",
		Elevation: 929,
		Latitude:  35.0065002441,
		Longitude: -82.2642974854,
		Timezone:  "America/New_York",
	},
	"20TA": {
		ICAO:      "20TA",
		Name:      "Mag Drop Airport",
		City:      "Bells",
		State:     "Texas",
		Country:   "US",
		Elevation: 740,
		Latitude:  33.559167,
		Longitude: -96.409444,
		Timezone:  "America/Chicago",
	},
	"20TE": {
		ICAO:      "20TE",
		Name:      "Reece Ranch Airport",
		City:      "Itasca",
		State:     "Texas",
		Country:   "US",
		Elevation: 880,
		Latitude:  32.1959991455,
		Longitude: -97.0892028809,
		Timezone:  "America/Chicago",
	},
	"20TS": {
		ICAO:      "20TS",
		Name:      "Bains Private Airport",
		City:      "Bandera",
		State:     "Texas",
		Country:   "US",
		Elevation: 1420,
		Latitude:  29.8099002838,
		Longitude: -99.0680999756,
		Timezone:  "America/Chicago",
	},
	"20VA": {
		ICAO:      "20VA",
		Name:      "Woodford Airpark",
		City:      "Bowling Green",
		State:     "Virginia",
		Country:   "US",
		Elevation: 245,
		Latitude:  38.0824012756,
		Longitude: -77.4882965088,
		Timezone:  "America/New_York",
	},
	"20VG": {
		ICAO:      "20VG",
		Name:      "Hawk Ridge Airport",
		City:      "Bedford",
		State:     "Virginia",
		Country:   "US",
		Elevation: 760,
		Latitude:  37.2887001038,
		Longitude: -79.4468994141,
		Timezone:  "America/New_York",
	},
	"20WA": {
		ICAO:      "20WA",
		Name:      "Skatter Creek Airport",
		City:      "Rochester",
		State:     "Washington",
		Country:   "US",
		Elevation: 174,
		Latitude:  46.824798584,
		Longitude: -123.0350036621,
		Timezone:  "America/Los_Angeles",
	},
	"20WI": {
		ICAO:      "20WI",
		Name:      "Harju Airport",
		City:      "Redgranite",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 800,
		Latitude:  44.0214004517,
		Longitude: -89.0792999268,
		Timezone:  "America/Chicago",
	},
	"20XS": {
		ICAO:      "20XS",
		Name:      "Klutts Field",
		City:      "Chisholm",
		State:     "Texas",
		Country:   "US",
		Elevation: 495,
		Latitude:  32.8353996277,
		Longitude: -96.3779983521,
		Timezone:  "America/Chicago",
	},
	"21AK": {
		ICAO:      "21AK",
		Name:      "Montana Creek Airport",
		City:      "Talkeetna",
		State:     "Alaska",
		Country:   "US",
		Elevation: 250,
		Latitude:  62.0717010498,
		Longitude: -150.0690002441,
		Timezone:  "America/Anchorage",
	},
	"21AR": {
		ICAO:      "21AR",
		Name:      "Skarda/Tollville Airport",
		City:      "Hazen",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 218,
		Latitude:  34.7176017761,
		Longitude: -91.5390014648,
		Timezone:  "America/Chicago",
	},
	"21AZ": {
		ICAO:      "21AZ",
		Name:      "White Mountain Lake Airport",
		City:      "Show Low",
		State:     "Arizona",
		Country:   "US",
		Elevation: 6066,
		Latitude:  34.3522987366,
		Longitude: -109.9700012207,
		Timezone:  "America/Phoenix",
	},
	"21FA": {
		ICAO:      "21FA",
		Name:      "Rockledge Airport",
		City:      "Rockledge",
		State:     "Florida",
		Country:   "US",
		Elevation: 27,
		Latitude:  28.2982997894,
		Longitude: -80.7186965942,
		Timezone:  "America/New_York",
	},
	"21FD": {
		ICAO:      "21FD",
		Name:      "Land's Field",
		City:      "Marianna",
		State:     "Florida",
		Country:   "US",
		Elevation: 100,
		Latitude:  30.6945991516,
		Longitude: -85.310798645,
		Timezone:  "America/Chicago",
	},
	"21FL": {
		ICAO:      "21FL",
		Name:      "Kings Field",
		City:      "Clermont",
		State:     "Florida",
		Country:   "US",
		Elevation: 98,
		Latitude:  28.508600235,
		Longitude: -81.8003005981,
		Timezone:  "America/New_York",
	},
	"21GE": {
		ICAO:      "21GE",
		Name:      "Deer Crossing Airport",
		City:      "Cleveland",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1390,
		Latitude:  34.5497016907,
		Longitude: -83.8221969604,
		Timezone:  "America/New_York",
	},
	"21IN": {
		ICAO:      "21IN",
		Name:      "Minneman Airport",
		City:      "Hartford City",
		State:     "Indiana",
		Country:   "US",
		Elevation: 905,
		Latitude:  40.4183998108,
		Longitude: -85.3554992676,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"21KY": {
		ICAO:      "21KY",
		Name:      "Lincoln Farm Airport",
		City:      "Hodgenville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 780,
		Latitude:  37.5376014709,
		Longitude: -85.7375030518,
		Timezone:  "America/New_York",
	},
	"21LA": {
		ICAO:      "21LA",
		Name:      "Southern Helicopters Airport",
		City:      "Baton Rouge",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 28,
		Latitude:  30.2968997955,
		Longitude: -91.2149963379,
		Timezone:  "America/Chicago",
	},
	"21LS": {
		ICAO:      "21LS",
		Name:      "Solitude Airstrip",
		City:      "St Francisville",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 181,
		Latitude:  30.8703994751,
		Longitude: -91.431098938,
		Timezone:  "America/Chicago",
	},
	"21MO": {
		ICAO:      "21MO",
		Name:      "Martins Airport",
		City:      "Lawson",
		State:     "Missouri",
		Country:   "US",
		Elevation: 995,
		Latitude:  39.5013999939,
		Longitude: -94.1773986816,
		Timezone:  "America/Chicago",
	},
	"21MU": {
		ICAO:      "21MU",
		Name:      "The Peninsula Airport",
		City:      "Shell Knob",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1015,
		Latitude:  36.5648994446,
		Longitude: -93.5095977783,
		Timezone:  "America/Chicago",
	},
	"21ND": {
		ICAO:      "21ND",
		Name:      "Rosenau Airport",
		City:      "Upham",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1450,
		Latitude:  48.5821990967,
		Longitude: -100.8550033569,
		Timezone:  "America/Chicago",
	},
	"21NE": {
		ICAO:      "21NE",
		Name:      "Byron Airport",
		City:      "Byron",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1670,
		Latitude:  40.0041999817,
		Longitude: -97.7731018066,
		Timezone:  "America/Chicago",
	},
	"21NH": {
		ICAO:      "21NH",
		Name:      "Propwash Airport",
		City:      "Kensington",
		State:     "New Hampshire",
		Country:   "US",
		Elevation: 190,
		Latitude:  42.9169006348,
		Longitude: -70.9353027344,
		Timezone:  "America/New_York",
	},
	"21NK": {
		ICAO:      "21NK",
		Name:      "Don Kichote Airport",
		City:      "Morris",
		State:     "New York",
		Country:   "US",
		Elevation: 1750,
		Latitude:  42.5527992249,
		Longitude: -75.1911010742,
		Timezone:  "America/New_York",
	},
	"21OG": {
		ICAO:      "21OG",
		Name:      "Ames Airport",
		City:      "Jefferson",
		State:     "Oregon",
		Country:   "US",
		Elevation: 720,
		Latitude:  44.771806,
		Longitude: -122.969444,
		Timezone:  "America/Los_Angeles",
	},
	"21OI": {
		ICAO:      "21OI",
		Name:      "Clearwater Airpark",
		City:      "Owensville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 905,
		Latitude:  39.1281013489,
		Longitude: -84.0926971436,
		Timezone:  "America/New_York",
	},
	"21SC": {
		ICAO:      "21SC",
		Name:      "Graham Airport",
		City:      "Darlington",
		State:     "South Carolina",
		Country:   "US",
		Elevation: 154,
		Latitude:  34.3479003906,
		Longitude: -79.8799972534,
		Timezone:  "America/New_York",
	},
	"21TE": {
		ICAO:      "21TE",
		Name:      "Seaberg Ranch Airport",
		City:      "Dayton",
		State:     "Texas",
		Country:   "US",
		Elevation: 68,
		Latitude:  30.0126991272,
		Longitude: -94.9241027832,
		Timezone:  "America/Chicago",
	},
	"21TN": {
		ICAO:      "21TN",
		Name:      "Ray's Stall Airport",
		City:      "Dandridge",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1020,
		Latitude:  35.9852981567,
		Longitude: -83.0922012329,
		Timezone:  "America/New_York",
	},
	"21TX": {
		ICAO:      "21TX",
		Name:      "Shilling's Airport",
		City:      "Texarkana",
		State:     "Texas",
		Country:   "US",
		Elevation: 350,
		Latitude:  33.4995994568,
		Longitude: -94.0943984985,
		Timezone:  "America/Chicago",
	},
	"21VA": {
		ICAO:      "21VA",
		Name:      "Eagles Nest Airport",
		City:      "Cheriton",
		State:     "Virginia",
		Country:   "US",
		Elevation: 40,
		Latitude:  37.3031997681,
		Longitude: -75.9348983765,
		Timezone:  "America/New_York",
	},
	"21XS": {
		ICAO:      "21XS",
		Name:      "Houston Airpark",
		City:      "Houston",
		State:     "Texas",
		Country:   "US",
		Elevation: 48,
		Latitude:  29.5196990967,
		Longitude: -95.2751998901,
		Timezone:  "America/Chicago",
	},
	"22AK": {
		ICAO:      "22AK",
		Name:      "Helmericks Airport",
		City:      "Deadhorse",
		State:     "Alaska",
		Country:   "US",
		Elevation: 7,
		Latitude:  70.428338,
		Longitude: -150.402899,
		Timezone:  "America/Anchorage",
	},
	"22CL": {
		ICAO:      "22CL",
		Name:      "Double Tree Farm Airport",
		City:      "Marysville",
		State:     "California",
		Country:   "US",
		Elevation: 550,
		Latitude:  39.2999000549,
		Longitude: -121.3590011597,
		Timezone:  "America/Los_Angeles",
	},
	"22CO": {
		ICAO:      "22CO",
		Name:      "Flying E Airport",
		City:      "Brighton",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5100,
		Latitude:  39.9580574036,
		Longitude: -104.485557556,
		Timezone:  "America/Denver",
	},
	"22FA": {
		ICAO:      "22FA",
		Name:      "Hidden River Airport",
		City:      "Sarasota",
		State:     "Florida",
		Country:   "US",
		Elevation: 47,
		Latitude:  27.3031005859,
		Longitude: -82.2725982666,
		Timezone:  "America/New_York",
	},
	"22FL": {
		ICAO:      "22FL",
		Name:      "Farm Air Service Airport",
		City:      "Altha",
		State:     "Florida",
		Country:   "US",
		Elevation: 150,
		Latitude:  30.6019001007,
		Longitude: -85.1399002075,
		Timezone:  "America/Chicago",
	},
	"22GA": {
		ICAO:      "22GA",
		Name:      "Riverside Airport",
		City:      "Plainville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 620,
		Latitude:  34.4292984009,
		Longitude: -85.054397583,
		Timezone:  "America/New_York",
	},
	"22ID": {
		ICAO:      "22ID",
		Name:      "Treasure Gulch Airport",
		City:      "Placerville",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4350,
		Latitude:  43.9446983337,
		Longitude: -115.9410018921,
		Timezone:  "America/Boise",
	},
	"22II": {
		ICAO:      "22II",
		Name:      "Lewis Airport",
		City:      "Mount Vernon",
		State:     "Indiana",
		Country:   "US",
		Elevation: 370,
		Latitude:  37.9169998169,
		Longitude: -87.761100769,
		Timezone:  "America/Chicago",
	},
	"22IL": {
		ICAO:      "22IL",
		Name:      "Heller Airport",
		City:      "Galena",
		State:     "Illinois",
		Country:   "US",
		Elevation: 820,
		Latitude:  42.4119987488,
		Longitude: -90.400100708,
		Timezone:  "America/Chicago",
	},
	"22IN": {
		ICAO:      "22IN",
		Name:      "Mooney Field",
		City:      "Auburn",
		State:     "Indiana",
		Country:   "US",
		Elevation: 865,
		Latitude:  41.3483009338,
		Longitude: -85.0074996948,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"22IS": {
		ICAO:      "22IS",
		Name:      "Johnson Farm Airport",
		City:      "Carman",
		State:     "Illinois",
		Country:   "US",
		Elevation: 535,
		Latitude:  40.7262001038,
		Longitude: -91.0699005127,
		Timezone:  "America/Chicago",
	},
	"22LA": {
		ICAO:      "22LA",
		Name:      "Wd Flyers Airport",
		City:      "Simmesport",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 48,
		Latitude:  30.9974994659,
		Longitude: -91.8811035156,
		Timezone:  "America/Chicago",
	},
	"22LL": {
		ICAO:      "22LL",
		Name:      "Pike's Airport",
		City:      "Ottawa",
		State:     "Illinois",
		Country:   "US",
		Elevation: 600,
		Latitude:  41.3134002686,
		Longitude: -88.8606033325,
		Timezone:  "America/Chicago",
	},
	"22LS": {
		ICAO:      "22LS",
		Name:      "Couvillion Airport",
		City:      "Arnaudville",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 29,
		Latitude:  30.3978004456,
		Longitude: -91.9738998413,
		Timezone:  "America/Chicago",
	},
	"22MD": {
		ICAO:      "22MD",
		Name:      "Laura's Landing Airport",
		City:      "Hagerstown",
		State:     "Maryland",
		Country:   "US",
		Elevation: 550,
		Latitude:  39.5868988037,
		Longitude: -77.6449966431,
		Timezone:  "America/New_York",
	},
	"22MO": {
		ICAO:      "22MO",
		Name:      "Belly Acres Ranch Airport",
		City:      "Mountain Grove",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1285,
		Latitude:  37.2611999512,
		Longitude: -92.3667984009,
		Timezone:  "America/Chicago",
	},
	"22MS": {
		ICAO:      "22MS",
		Name:      "Sky Landings Airport",
		City:      "Lucedale",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 260,
		Latitude:  30.840278,
		Longitude: -88.574722,
		Timezone:  "America/Chicago",
	},
	"22MT": {
		ICAO:      "22MT",
		Name:      "Wounded Buck Ranch Airport",
		City:      "Kalispell",
		State:     "Montana",
		Country:   "US",
		Elevation: 2905,
		Latitude:  48.1194000244,
		Longitude: -114.1699981689,
		Timezone:  "America/Denver",
	},
	"22NC": {
		ICAO:      "22NC",
		Name:      "Flying Dove Field",
		City:      "Siler City",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 650,
		Latitude:  35.8210983276,
		Longitude: -79.4682998657,
		Timezone:  "America/New_York",
	},
	"22NE": {
		ICAO:      "22NE",
		Name:      "Grone Airport",
		City:      "Davenport",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1643,
		Latitude:  40.297000885,
		Longitude: -97.7981033325,
		Timezone:  "America/Chicago",
	},
	"22NR": {
		ICAO:      "22NR",
		Name:      "Bully Field",
		City:      "Burlington",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 550,
		Latitude:  36.153611,
		Longitude: -79.400556,
		Timezone:  "America/New_York",
	},
	"22OG": {
		ICAO:      "22OG",
		Name:      "Withrotor Airport",
		City:      "Lakeview",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4920,
		Latitude:  42.2753982544,
		Longitude: -120.3939971924,
		Timezone:  "America/Los_Angeles",
	},
	"22OI": {
		ICAO:      "22OI",
		Name:      "Giovannone Airport",
		City:      "Lordstown",
		State:     "Ohio",
		Country:   "US",
		Elevation: 950,
		Latitude:  41.1488990784,
		Longitude: -80.8401031494,
		Timezone:  "America/New_York",
	},
	"22OK": {
		ICAO:      "22OK",
		Name:      "Smith Field",
		City:      "Springer",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 860,
		Latitude:  34.315278,
		Longitude: -97.175833,
		Timezone:  "America/Chicago",
	},
	"22OR": {
		ICAO:      "22OR",
		Name:      "Iron Crown Airport",
		City:      "Silverton",
		State:     "Oregon",
		Country:   "US",
		Elevation: 790,
		Latitude:  44.9417991638,
		Longitude: -122.7630004883,
		Timezone:  "America/Los_Angeles",
	},
	"22TA": {
		ICAO:      "22TA",
		Name:      "Standard Industries Airport",
		City:      "San Antonio",
		State:     "Texas",
		Country:   "US",
		Elevation: 690,
		Latitude:  29.341299057,
		Longitude: -98.6596984863,
		Timezone:  "America/Chicago",
	},
	"22TE": {
		ICAO:      "22TE",
		Name:      "Fenner Ranch Airport",
		City:      "Cordele",
		State:     "Texas",
		Country:   "US",
		Elevation: 126,
		Latitude:  29.1933002472,
		Longitude: -96.6891021729,
		Timezone:  "America/Chicago",
	},
	"22WA": {
		ICAO:      "22WA",
		Name:      "Gollehon Airport",
		City:      "Wilbur",
		State:     "Washington",
		Country:   "US",
		Elevation: 2360,
		Latitude:  47.8202018738,
		Longitude: -118.68699646,
		Timezone:  "America/Los_Angeles",
	},
	"22WN": {
		ICAO:      "22WN",
		Name:      "Southwind Airport",
		City:      "Dodgeville",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1030,
		Latitude:  43.0685997009,
		Longitude: -90.2194976807,
		Timezone:  "America/Chicago",
	},
	"22WY": {
		ICAO:      "22WY",
		Name:      "Kissack/Reynolds Airport",
		City:      "Rozet",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4380,
		Latitude:  44.338056,
		Longitude: -105.162117,
		Timezone:  "America/Denver",
	},
	"22XS": {
		ICAO:      "22XS",
		Name:      "Longhorn Aux Landing Strip",
		City:      "Fort Hood(Killeen)",
		State:     "Texas",
		Country:   "US",
		Elevation: 720,
		Latitude:  31.3710002899,
		Longitude: -97.6669998169,
		Timezone:  "America/Chicago",
	},
	"23AK": {
		ICAO:      "23AK",
		Name:      "Yuknis Airport",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 340,
		Latitude:  61.5941009521,
		Longitude: -149.3110046387,
		Timezone:  "America/Anchorage",
	},
	"23AR": {
		ICAO:      "23AR",
		Name:      "Classic Airstrip",
		City:      "Jonesboro",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 260,
		Latitude:  35.8259010315,
		Longitude: -90.547203064,
		Timezone:  "America/Chicago",
	},
	"23AZ": {
		ICAO:      "23AZ",
		Name:      "Moreton Airpark",
		City:      "Wickenburg",
		State:     "Arizona",
		Country:   "US",
		Elevation: 2455,
		Latitude:  34.0195007324,
		Longitude: -112.8259963989,
		Timezone:  "America/Phoenix",
	},
	"23CA": {
		ICAO:      "23CA",
		Name:      "Lefko Airport",
		City:      "Fort Jones",
		State:     "California",
		Country:   "US",
		Elevation: 2860,
		Latitude:  41.6249008179,
		Longitude: -122.7649993896,
		Timezone:  "America/Los_Angeles",
	},
	"23CD": {
		ICAO:      "23CD",
		Name:      "Aviation Acres Airport",
		City:      "Burlington",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4210,
		Latitude:  39.29945,
		Longitude: -102.321817,
		Timezone:  "America/Denver",
	},
	"23CN": {
		ICAO:      "23CN",
		Name:      "Douthitt Strip",
		City:      "El Centro",
		State:     "California",
		Country:   "US",
		Elevation: 45,
		Latitude:  32.788898468,
		Longitude: -115.5299987793,
		Timezone:  "America/Los_Angeles",
	},
	"23CO": {
		ICAO:      "23CO",
		Name:      "High Mesa Airport",
		City:      "Rush",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6000,
		Latitude:  38.8438987732,
		Longitude: -103.9899978638,
		Timezone:  "America/Denver",
	},
	"23DE": {
		ICAO:      "23DE",
		Name:      "Full Throttle Farm Airport",
		City:      "Clayton",
		State:     "Delaware",
		Country:   "US",
		Elevation: 81,
		Latitude:  39.312567,
		Longitude: -75.728983,
		Timezone:  "America/New_York",
	},
	"23FD": {
		ICAO:      "23FD",
		Name:      "Scott 2000 Airport",
		City:      "Vero Beach",
		State:     "Florida",
		Country:   "US",
		Elevation: 27,
		Latitude:  27.5473003387,
		Longitude: -80.6620025635,
		Timezone:  "America/New_York",
	},
	"23FL": {
		ICAO:      "23FL",
		Name:      "Gyro Town Usa STOLport",
		City:      "Wimauma",
		State:     "Florida",
		Country:   "US",
		Elevation: 65,
		Latitude:  27.6844005585,
		Longitude: -82.2442016602,
		Timezone:  "America/New_York",
	},
	"23GE": {
		ICAO:      "23GE",
		Name:      "Lt Landing Airport",
		City:      "Dublin",
		State:     "Georgia",
		Country:   "US",
		Elevation: 286,
		Latitude:  32.383056,
		Longitude: -83.1,
		Timezone:  "America/New_York",
	},
	"23IS": {
		ICAO:      "23IS",
		Name:      "Clark Airport",
		City:      "Plymouth",
		State:     "Illinois",
		Country:   "US",
		Elevation: 570,
		Latitude:  40.3259010315,
		Longitude: -90.8275985718,
		Timezone:  "America/Chicago",
	},
	"23KS": {
		ICAO:      "23KS",
		Name:      "Ney Airport",
		City:      "Bonner Springs",
		State:     "Kansas",
		Country:   "US",
		Elevation: 950,
		Latitude:  39.0699996948,
		Longitude: -94.948600769,
		Timezone:  "America/Chicago",
	},
	"23KY": {
		ICAO:      "23KY",
		Name:      "Barnes Farm Airport",
		City:      "Burna",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 500,
		Latitude:  37.2308006287,
		Longitude: -88.3186035156,
		Timezone:  "America/Chicago",
	},
	"23LA": {
		ICAO:      "23LA",
		Name:      "Warner-Thunder Hill Airport",
		City:      "Covington",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 185,
		Latitude:  30.6301994324,
		Longitude: -90.079498291,
		Timezone:  "America/Chicago",
	},
	"23ME": {
		ICAO:      "23ME",
		Name:      "Dyer's Field",
		City:      "Rumford",
		State:     "Maine",
		Country:   "US",
		Elevation: 650,
		Latitude:  44.5264015198,
		Longitude: -70.6635971069,
		Timezone:  "America/New_York",
	},
	"23MI": {
		ICAO:      "23MI",
		Name:      "Betsie River Airstrip",
		City:      "Thompsonville",
		State:     "Michigan",
		Country:   "US",
		Elevation: 825,
		Latitude:  44.573600769,
		Longitude: -85.888999939,
		Timezone:  "America/Detroit",
	},
	"23MN": {
		ICAO:      "23MN",
		Name:      "High Grove Airport",
		City:      "Preston",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1240,
		Latitude:  43.6411018372,
		Longitude: -92.022102356,
		Timezone:  "America/Chicago",
	},
	"23MO": {
		ICAO:      "23MO",
		Name:      "Oerke Enterprises Airport",
		City:      "Butler",
		State:     "Missouri",
		Country:   "US",
		Elevation: 770,
		Latitude:  38.2200012207,
		Longitude: -94.4961013794,
		Timezone:  "America/Chicago",
	},
	"23MT": {
		ICAO:      "23MT",
		Name:      "Waterfall Airport",
		City:      "Bozeman",
		State:     "Montana",
		Country:   "US",
		Elevation: 4950,
		Latitude:  45.6571998596,
		Longitude: -111.0400009155,
		Timezone:  "America/Denver",
	},
	"23ND": {
		ICAO:      "23ND",
		Name:      "Minnkota Private Airport",
		City:      "Center",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 2110,
		Latitude:  47.0821990967,
		Longitude: -101.266998291,
		Timezone:  "America/North_Dakota/Center",
	},
	"23NE": {
		ICAO:      "23NE",
		Name:      "Stromsburg Municipal Airport",
		City:      "Stromsburg",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1675,
		Latitude:  41.1291999817,
		Longitude: -97.5775985718,
		Timezone:  "America/Chicago",
	},
	"23NH": {
		ICAO:      "23NH",
		Name:      "Windswept Airport",
		City:      "Windsor",
		State:     "New Hampshire",
		Country:   "US",
		Elevation: 1289,
		Latitude:  43.13029861,
		Longitude: -72.01499939,
		Timezone:  "America/New_York",
	},
	"23NJ": {
		ICAO:      "23NJ",
		Name:      "Alliance Airport",
		City:      "Pittsgrove",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 110,
		Latitude:  39.504699707,
		Longitude: -75.0931015015,
		Timezone:  "America/New_York",
	},
	"23NK": {
		ICAO:      "23NK",
		Name:      "Long Acre Farms Airport",
		City:      "Macedon",
		State:     "New York",
		Country:   "US",
		Elevation: 520,
		Latitude:  43.1124992371,
		Longitude: -77.3125,
		Timezone:  "America/New_York",
	},
	"23NR": {
		ICAO:      "23NR",
		Name:      "Whitfield'S East Airport",
		City:      "Fairfield",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 1,
		Latitude:  35.560556,
		Longitude: -76.234722,
		Timezone:  "America/New_York",
	},
	"23NY": {
		ICAO:      "23NY",
		Name:      "Jolamtra Landing Area Airport",
		City:      "Bath",
		State:     "New York",
		Country:   "US",
		Elevation: 1700,
		Latitude:  42.279800415,
		Longitude: -77.3399963379,
		Timezone:  "America/New_York",
	},
	"23OA": {
		ICAO:      "23OA",
		Name:      "Dade Field",
		City:      "Christianburg",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1138,
		Latitude:  40.050278,
		Longitude: -84.013056,
		Timezone:  "America/New_York",
	},
	"23PA": {
		ICAO:      "23PA",
		Name:      "Cedar Acres Private Group Airport",
		City:      "Revere",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 500,
		Latitude:  40.5000991821,
		Longitude: -75.1467971802,
		Timezone:  "America/New_York",
	},
	"23TA": {
		ICAO:      "23TA",
		Name:      "Rowland R Airfield",
		City:      "Tom Bean",
		State:     "Texas",
		Country:   "US",
		Elevation: 705,
		Latitude:  33.4873008728,
		Longitude: -96.4558029175,
		Timezone:  "America/Chicago",
	},
	"23TS": {
		ICAO:      "23TS",
		Name:      "Bertani Ranch Airport",
		City:      "Del Rio",
		State:     "Texas",
		Country:   "US",
		Elevation: 1110,
		Latitude:  29.422700882,
		Longitude: -100.7389984131,
		Timezone:  "America/Chicago",
	},
	"23TX": {
		ICAO:      "23TX",
		Name:      "The Grove/Ranch Airport",
		City:      "The Grove/Ranch",
		State:     "Texas",
		Country:   "US",
		Elevation: 700,
		Latitude:  31.2495994568,
		Longitude: -97.5227966309,
		Timezone:  "America/Chicago",
	},
	"23VA": {
		ICAO:      "23VA",
		Name:      "Wheatland Airport",
		City:      "Buckingham",
		State:     "Virginia",
		Country:   "US",
		Elevation: 490,
		Latitude:  37.5570983887,
		Longitude: -78.6200027466,
		Timezone:  "America/New_York",
	},
	"23VG": {
		ICAO:      "23VG",
		Name:      "Murdock's Holly Bu Airport",
		City:      "Boydton",
		State:     "Virginia",
		Country:   "US",
		Elevation: 465,
		Latitude:  36.7097015381,
		Longitude: -78.4281005859,
		Timezone:  "America/New_York",
	},
	"23VT": {
		ICAO:      "23VT",
		Name:      "North Windham Airport",
		City:      "Londonderry",
		State:     "Vermont",
		Country:   "US",
		Elevation: 1750,
		Latitude:  43.2215003967,
		Longitude: -72.7128982544,
		Timezone:  "America/New_York",
	},
	"23WN": {
		ICAO:      "23WN",
		Name:      "Willie's Airport",
		City:      "Springbrook",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1110,
		Latitude:  45.9110984802,
		Longitude: -91.6285018921,
		Timezone:  "America/Chicago",
	},
	"23WV": {
		ICAO:      "23WV",
		Name:      "Bocamanu Airport",
		City:      "Fayetteville",
		State:     "West Virginia",
		Country:   "US",
		Elevation: 2501,
		Latitude:  38.025833,
		Longitude: -80.982778,
		Timezone:  "America/New_York",
	},
	"23XS": {
		ICAO:      "23XS",
		Name:      "Shorthorn Aux Landing Strip",
		City:      "Fort Hood/Killeen/",
		State:     "Texas",
		Country:   "US",
		Elevation: 720,
		Latitude:  31.3586006165,
		Longitude: -97.6710968018,
		Timezone:  "America/Chicago",
	},
	"24AK": {
		ICAO:      "24AK",
		Name:      "Toad Lake Strip",
		City:      "Meadow Lakes",
		State:     "Alaska",
		Country:   "US",
		Elevation: 100,
		Latitude:  61.6156997681,
		Longitude: -149.7070007324,
		Timezone:  "America/Anchorage",
	},
	"24AR": {
		ICAO:      "24AR",
		Name:      "Griffin Agricultural Airport",
		City:      "Lundell",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 200,
		Latitude:  34.1981010437,
		Longitude: -90.952796936,
		Timezone:  "America/Chicago",
	},
	"24AZ": {
		ICAO:      "24AZ",
		Name:      "Pleasant Valley Airstrip",
		City:      "Young",
		State:     "Arizona",
		Country:   "US",
		Elevation: 5688,
		Latitude:  34.1581001282,
		Longitude: -110.9349975586,
		Timezone:  "America/Phoenix",
	},
	"24CL": {
		ICAO:      "24CL",
		Name:      "Salyer Farms Airport",
		City:      "Corcoran",
		State:     "California",
		Country:   "US",
		Elevation: 205,
		Latitude:  36.0887985229,
		Longitude: -119.5429992676,
		Timezone:  "America/Los_Angeles",
	},
	"24CT": {
		ICAO:      "24CT",
		Name:      "Bee Field",
		City:      "Jewett City",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 280,
		Latitude:  41.566886,
		Longitude: -71.966883,
		Timezone:  "America/New_York",
	},
	"24FL": {
		ICAO:      "24FL",
		Name:      "Garcon Field",
		City:      "Milton",
		State:     "Florida",
		Country:   "US",
		Elevation: 16,
		Latitude:  30.4899997711,
		Longitude: -87.0858001709,
		Timezone:  "America/Chicago",
	},
	"24IA": {
		ICAO:      "24IA",
		Name:      "Isley Field",
		City:      "Cedar Falls",
		State:     "Iowa",
		Country:   "US",
		Elevation: 982,
		Latitude:  42.5158004761,
		Longitude: -92.5197982788,
		Timezone:  "America/Chicago",
	},
	"24IN": {
		ICAO:      "24IN",
		Name:      "Kenstin Manor Airport",
		City:      "Helmsburg",
		State:     "Indiana",
		Country:   "US",
		Elevation: 658,
		Latitude:  39.2625007629,
		Longitude: -86.2874984741,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"24KS": {
		ICAO:      "24KS",
		Name:      "Blocker Field",
		City:      "Haven",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1523,
		Latitude:  37.8582992554,
		Longitude: -97.8178024292,
		Timezone:  "America/Chicago",
	},
	"24KY": {
		ICAO:      "24KY",
		Name:      "Williams Farm Airport",
		City:      "Mortons Gap",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 415,
		Latitude:  37.2167015076,
		Longitude: -87.4306030273,
		Timezone:  "America/Chicago",
	},
	"24LL": {
		ICAO:      "24LL",
		Name:      "Lambert Airport",
		City:      "Pinckneyville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 510,
		Latitude:  38.0862007141,
		Longitude: -89.4589996338,
		Timezone:  "America/Chicago",
	},
	"24ME": {
		ICAO:      "24ME",
		Name:      "Slip Knot Landing Airport",
		City:      "Windham",
		State:     "Maine",
		Country:   "US",
		Elevation: 310,
		Latitude:  43.8222007751,
		Longitude: -70.4428024292,
		Timezone:  "America/New_York",
	},
	"24MI": {
		ICAO:      "24MI",
		Name:      "Johnson Airport",
		City:      "Alto",
		State:     "Michigan",
		Country:   "US",
		Elevation: 810,
		Latitude:  42.7999992371,
		Longitude: -85.3707962036,
		Timezone:  "America/Detroit",
	},
	"24MN": {
		ICAO:      "24MN",
		Name:      "Bergemann Airport",
		City:      "Garden City",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1001,
		Latitude:  44.0041007996,
		Longitude: -94.1780014038,
		Timezone:  "America/Chicago",
	},
	"24MO": {
		ICAO:      "24MO",
		Name:      "Mistwood Airport",
		City:      "Camdenton",
		State:     "Missouri",
		Country:   "US",
		Elevation: 760,
		Latitude:  38.1333999634,
		Longitude: -92.8335037231,
		Timezone:  "America/Chicago",
	},
	"24MT": {
		ICAO:      "24MT",
		Name:      "Haggerty Airport",
		City:      "Bozeman",
		State:     "Montana",
		Country:   "US",
		Elevation: 5170,
		Latitude:  45.6194000244,
		Longitude: -111.0920028687,
		Timezone:  "America/Denver",
	},
	"24MU": {
		ICAO:      "24MU",
		Name:      "Steele Airport",
		City:      "Fairfax",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1077,
		Latitude:  40.3628005981,
		Longitude: -95.4385986328,
		Timezone:  "America/Chicago",
	},
	"24NC": {
		ICAO:      "24NC",
		Name:      "Mc Gee Field",
		City:      "Monroe",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 550,
		Latitude:  35.1272010803,
		Longitude: -80.4749984741,
		Timezone:  "America/New_York",
	},
	"24NE": {
		ICAO:      "24NE",
		Name:      "Orr Airport",
		City:      "Lakeside",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3827,
		Latitude:  42.3838996887,
		Longitude: -102.4390029907,
		Timezone:  "America/Denver",
	},
	"24NY": {
		ICAO:      "24NY",
		Name:      "Ely Air Park",
		City:      "Bergen",
		State:     "New York",
		Country:   "US",
		Elevation: 690,
		Latitude:  43.0666999817,
		Longitude: -77.9589004517,
		Timezone:  "America/New_York",
	},
	"24OH": {
		ICAO:      "24OH",
		Name:      "Milburn Airport",
		City:      "Louisville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1197,
		Latitude:  40.8805999756,
		Longitude: -81.2914962769,
		Timezone:  "America/New_York",
	},
	"24OI": {
		ICAO:      "24OI",
		Name:      "Reeds Airport",
		City:      "St Paris",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1200,
		Latitude:  40.0778007507,
		Longitude: -83.8972015381,
		Timezone:  "America/New_York",
	},
	"24PA": {
		ICAO:      "24PA",
		Name:      "Kindelberger Landing Strip",
		City:      "Freedom",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1140,
		Latitude:  40.6916999817,
		Longitude: -80.1778030396,
		Timezone:  "America/New_York",
	},
	"24SC": {
		ICAO:      "24SC",
		Name:      "The Farm Airport",
		City:      "Summit",
		State:     "South Carolina",
		Country:   "US",
		Elevation: 580,
		Latitude:  33.935798645,
		Longitude: -81.4297027588,
		Timezone:  "America/New_York",
	},
	"24TA": {
		ICAO:      "24TA",
		Name:      "Moore Ranch Airport",
		City:      "Centerville",
		State:     "Texas",
		Country:   "US",
		Elevation: 315,
		Latitude:  31.2131996155,
		Longitude: -95.8274993896,
		Timezone:  "America/Chicago",
	},
	"24TX": {
		ICAO:      "24TX",
		Name:      "Thurber Lake Airport",
		City:      "Thurber",
		State:     "Texas",
		Country:   "US",
		Elevation: 1092,
		Latitude:  32.5000991821,
		Longitude: -98.3836975098,
		Timezone:  "America/Chicago",
	},
	"24VA": {
		ICAO:      "24VA",
		Name:      "Cherrystone Airport",
		City:      "Cheriton",
		State:     "Virginia",
		Country:   "US",
		Elevation: 14,
		Latitude:  37.2882003784,
		Longitude: -75.9997024536,
		Timezone:  "America/New_York",
	},
	"24WI": {
		ICAO:      "24WI",
		Name:      "Irlbeck Airport",
		City:      "Somerset",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 990,
		Latitude:  45.095500946,
		Longitude: -92.6737976074,
		Timezone:  "America/Chicago",
	},
	"24WN": {
		ICAO:      "24WN",
		Name:      "Prairie View Farm Airport",
		City:      "Lake Geneva",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 890,
		Latitude:  42.5541000366,
		Longitude: -88.4274978638,
		Timezone:  "America/Chicago",
	},
	"24XA": {
		ICAO:      "24XA",
		Name:      "Cross-B Airport",
		City:      "San Antonio",
		State:     "Texas",
		Country:   "US",
		Elevation: 530,
		Latitude:  29.320969,
		Longitude: -98.370294,
		Timezone:  "America/Chicago",
	},
	"25AK": {
		ICAO:      "25AK",
		Name:      "Honeybee Lake Aero Park Airport",
		City:      "Willow",
		State:     "Alaska",
		Country:   "US",
		Elevation: 200,
		Latitude:  61.7122001648,
		Longitude: -150.06300354,
		Timezone:  "America/Anchorage",
	},
	"25AR": {
		ICAO:      "25AR",
		Name:      "Reeves Private Airport",
		City:      "Mount Pleasant",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 600,
		Latitude:  35.9305992126,
		Longitude: -91.7460021973,
		Timezone:  "America/Chicago",
	},
	"25AZ": {
		ICAO:      "25AZ",
		Name:      "Mystery Well Ranch Airport",
		City:      "Portal",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4173,
		Latitude:  31.8159008026,
		Longitude: -109.0569992065,
		Timezone:  "America/Denver",
	},
	"25CA": {
		ICAO:      "25CA",
		Name:      "Loma Madera Ranch Airport",
		City:      "Santa Ysabel",
		State:     "California",
		Country:   "US",
		Elevation: 3375,
		Latitude:  33.1791992188,
		Longitude: -116.7929992676,
		Timezone:  "America/Los_Angeles",
	},
	"25CD": {
		ICAO:      "25CD",
		Name:      "Lux Field",
		City:      "Hartsel",
		State:     "Colorado",
		Country:   "US",
		Elevation: 9300,
		Latitude:  39.152333,
		Longitude: -105.720167,
		Timezone:  "America/Denver",
	},
	"25CL": {
		ICAO:      "25CL",
		Name:      "Van Dyke Strip",
		City:      "Pleasant Grove",
		State:     "California",
		Country:   "US",
		Elevation: 48,
		Latitude:  38.8666000366,
		Longitude: -121.4840011597,
		Timezone:  "America/Los_Angeles",
	},
	"25CO": {
		ICAO:      "25CO",
		Name:      "Crystal Lakes Airport",
		City:      "Red Feather Lakes",
		State:     "Colorado",
		Country:   "US",
		Elevation: 8440,
		Latitude:  40.851600647,
		Longitude: -105.6330032349,
		Timezone:  "America/Denver",
	},
	"25FL": {
		ICAO:      "25FL",
		Name:      "Connell's Wahoo Airport",
		City:      "Bushnell",
		State:     "Florida",
		Country:   "US",
		Elevation: 65,
		Latitude:  28.6319007874,
		Longitude: -82.1577987671,
		Timezone:  "America/New_York",
	},
	"25GA": {
		ICAO:      "25GA",
		Name:      "Miller Farm Airport",
		City:      "Douglasville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 750,
		Latitude:  33.6595993042,
		Longitude: -84.6630020142,
		Timezone:  "America/New_York",
	},
	"25IL": {
		ICAO:      "25IL",
		Name:      "Bob Davis Flying Service Airport",
		City:      "Virden",
		State:     "Illinois",
		Country:   "US",
		Elevation: 680,
		Latitude:  39.5167007446,
		Longitude: -89.8582992554,
		Timezone:  "America/Chicago",
	},
	"25IN": {
		ICAO:      "25IN",
		Name:      "Reid-Eash Airport",
		City:      "Howe",
		State:     "Indiana",
		Country:   "US",
		Elevation: 879,
		Latitude:  41.7028007507,
		Longitude: -85.4194030762,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"25KS": {
		ICAO:      "25KS",
		Name:      "Wyrill Farming Airport",
		City:      "Kirwin",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1700,
		Latitude:  39.6833000183,
		Longitude: -99.12840271,
		Timezone:  "America/Chicago",
	},
	"25LL": {
		ICAO:      "25LL",
		Name:      "Bradbury Airport",
		City:      "Piper City",
		State:     "Illinois",
		Country:   "US",
		Elevation: 694,
		Latitude:  40.7523002625,
		Longitude: -88.2228012085,
		Timezone:  "America/Chicago",
	},
	"25MD": {
		ICAO:      "25MD",
		Name:      "Whalen Field",
		City:      "Sudlersville",
		State:     "Maryland",
		Country:   "US",
		Elevation: 70,
		Latitude:  39.1674995422,
		Longitude: -75.8402023315,
		Timezone:  "America/New_York",
	},
	"25MI": {
		ICAO:      "25MI",
		Name:      "Farver Field",
		City:      "Caseville",
		State:     "Michigan",
		Country:   "US",
		Elevation: 600,
		Latitude:  43.9166984558,
		Longitude: -83.2750015259,
		Timezone:  "America/Detroit",
	},
	"25MO": {
		ICAO:      "25MO",
		Name:      "Cook Airport",
		City:      "Carl Junction",
		State:     "Missouri",
		Country:   "US",
		Elevation: 920,
		Latitude:  37.185798645,
		Longitude: -94.5511016846,
		Timezone:  "America/Chicago",
	},
	"25MT": {
		ICAO:      "25MT",
		Name:      "Blatter Airport",
		City:      "Glasgow",
		State:     "Montana",
		Country:   "US",
		Elevation: 2800,
		Latitude:  48.4221992493,
		Longitude: -106.2529983521,
		Timezone:  "America/Denver",
	},
	"25NC": {
		ICAO:      "25NC",
		Name:      "Smith Air Strip",
		City:      "Asheboro",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 560,
		Latitude:  35.6333007813,
		Longitude: -79.7099990845,
		Timezone:  "America/New_York",
	},
	"25NE": {
		ICAO:      "25NE",
		Name:      "Corr Airport",
		City:      "Melbeta",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 4000,
		Latitude:  41.7240982056,
		Longitude: -103.5110015869,
		Timezone:  "America/Denver",
	},
	"25NK": {
		ICAO:      "25NK",
		Name:      "Loucks Airport",
		City:      "Hammondsport",
		State:     "New York",
		Country:   "US",
		Elevation: 1425,
		Latitude:  42.4361991882,
		Longitude: -77.1322021484,
		Timezone:  "America/New_York",
	},
	"25NM": {
		ICAO:      "25NM",
		Name:      "Dream Catcher Ranch Airport",
		City:      "Pie Town",
		State:     "New Mexico",
		Country:   "US",
		Elevation: 7253,
		Latitude:  34.476944,
		Longitude: -108.031111,
		Timezone:  "America/Denver",
	},
	"25OH": {
		ICAO:      "25OH",
		Name:      "Hammond Airport",
		City:      "Louisville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1050,
		Latitude:  40.9131011963,
		Longitude: -81.259803772,
		Timezone:  "America/New_York",
	},
	"25OI": {
		ICAO:      "25OI",
		Name:      "Morrison Field",
		City:      "Bristolville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 865,
		Latitude:  41.4225997925,
		Longitude: -80.8778991699,
		Timezone:  "America/New_York",
	},
	"25SC": {
		ICAO:      "25SC",
		Name:      "Mountain Ridge Airport",
		City:      "Greenville",
		State:     "South Carolina",
		Country:   "US",
		Elevation: 980,
		Latitude:  34.960556,
		Longitude: -82.339444,
		Timezone:  "America/New_York",
	},
	"25TA": {
		ICAO:      "25TA",
		Name:      "Ferris Airport",
		City:      "Baytown",
		State:     "Texas",
		Country:   "US",
		Elevation: 35,
		Latitude:  29.83659935,
		Longitude: -94.9673995972,
		Timezone:  "America/Chicago",
	},
	"25TE": {
		ICAO:      "25TE",
		Name:      "Taylor's Air Park",
		City:      "Joshua",
		State:     "Texas",
		Country:   "US",
		Elevation: 700,
		Latitude:  32.4500999451,
		Longitude: -97.4002990723,
		Timezone:  "America/Chicago",
	},
	"25TS": {
		ICAO:      "25TS",
		Name:      "Miller Airfield",
		City:      "Dalhart",
		State:     "Texas",
		Country:   "US",
		Elevation: 3961,
		Latitude:  36.0917015076,
		Longitude: -102.4160003662,
		Timezone:  "America/Chicago",
	},
	"25TX": {
		ICAO:      "25TX",
		Name:      "Hubbard Airport",
		City:      "Lufkin",
		State:     "Texas",
		Country:   "US",
		Elevation: 300,
		Latitude:  31.3127994537,
		Longitude: -94.6443023682,
		Timezone:  "America/Chicago",
	},
	"25VA": {
		ICAO:      "25VA",
		Name:      "Toddsbury Farm Airport",
		City:      "Aylett",
		State:     "Virginia",
		Country:   "US",
		Elevation: 50,
		Latitude:  37.8176002502,
		Longitude: -77.1029968262,
		Timezone:  "America/New_York",
	},
	"25WA": {
		ICAO:      "25WA",
		Name:      "Hart Ranch Airport",
		City:      "Tonasket",
		State:     "Washington",
		Country:   "US",
		Elevation: 2608,
		Latitude:  48.576499939,
		Longitude: -119.0650024414,
		Timezone:  "America/Los_Angeles",
	},
	"25WN": {
		ICAO:      "25WN",
		Name:      "Little Plum Creek Airport",
		City:      "Pepin",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 12,
		Latitude:  44.528718,
		Longitude: -92.120341,
		Timezone:  "America/Chicago",
	},
	"25XA": {
		ICAO:      "25XA",
		Name:      "Headwaters Airport",
		City:      "Fredericksburg",
		State:     "Texas",
		Country:   "US",
		Elevation: 1974,
		Latitude:  30.098869,
		Longitude: -98.704617,
		Timezone:  "America/Chicago",
	},
	"25XS": {
		ICAO:      "25XS",
		Name:      "O'Brien Airpark",
		City:      "Waxahachie",
		State:     "Texas",
		Country:   "US",
		Elevation: 650,
		Latitude:  32.4864997864,
		Longitude: -96.8463973999,
		Timezone:  "America/Chicago",
	},
	"26AL": {
		ICAO:      "26AL",
		Name:      "Richardson Field",
		City:      "Mobile",
		State:     "Alabama",
		Country:   "US",
		Elevation: 180,
		Latitude:  30.6242008209,
		Longitude: -88.2786026001,
		Timezone:  "America/Chicago",
	},
	"26AR": {
		ICAO:      "26AR",
		Name:      "Fly N K Airport",
		City:      "Searcy",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 400,
		Latitude:  35.2154998779,
		Longitude: -91.807800293,
		Timezone:  "America/Chicago",
	},
	"26AZ": {
		ICAO:      "26AZ",
		Name:      "Flying Dare's Ranch Airport",
		City:      "Aguila",
		State:     "Arizona",
		Country:   "US",
		Elevation: 2192,
		Latitude:  34.0120010376,
		Longitude: -113.2030029297,
		Timezone:  "America/Phoenix",
	},
	"26CA": {
		ICAO:      "26CA",
		Name:      "Boeckmann Ranch Airport",
		City:      "Wilton",
		State:     "California",
		Country:   "US",
		Elevation: 100,
		Latitude:  38.4268989563,
		Longitude: -121.1719970703,
		Timezone:  "America/Los_Angeles",
	},
	"26FD": {
		ICAO:      "26FD",
		Name:      "Hennessy Airport",
		City:      "Arcadia",
		State:     "Florida",
		Country:   "US",
		Elevation: 50,
		Latitude:  27.0512008667,
		Longitude: -81.8480987549,
		Timezone:  "America/New_York",
	},
	"26GA": {
		ICAO:      "26GA",
		Name:      "Murphree Airport",
		City:      "Newnan",
		State:     "Georgia",
		Country:   "US",
		Elevation: 760,
		Latitude:  33.3362007141,
		Longitude: -84.9138031006,
		Timezone:  "America/New_York",
	},
	"26ID": {
		ICAO:      "26ID",
		Name:      "Seven Shamrock Airport",
		City:      "Sandpoint",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2081,
		Latitude:  48.363889,
		Longitude: -116.403611,
		Timezone:  "America/Los_Angeles",
	},
	"26IL": {
		ICAO:      "26IL",
		Name:      "Duane E. Davis Airport",
		City:      "Leaf River",
		State:     "Illinois",
		Country:   "US",
		Elevation: 741,
		Latitude:  42.1197013855,
		Longitude: -89.3733978271,
		Timezone:  "America/Chicago",
	},
	"26IN": {
		ICAO:      "26IN",
		Name:      "Willcox Airport",
		City:      "Knightstown",
		State:     "Indiana",
		Country:   "US",
		Elevation: 990,
		Latitude:  39.8233985901,
		Longitude: -85.5919036865,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"26IS": {
		ICAO:      "26IS",
		Name:      "Ficklin-Airtech Airport",
		City:      "Fairbury",
		State:     "Illinois",
		Country:   "US",
		Elevation: 672,
		Latitude:  40.7667007446,
		Longitude: -88.5055999756,
		Timezone:  "America/Chicago",
	},
	"26LL": {
		ICAO:      "26LL",
		Name:      "Anderson Airport",
		City:      "Sycamore",
		State:     "Illinois",
		Country:   "US",
		Elevation: 830,
		Latitude:  42.0094985962,
		Longitude: -88.7175979614,
		Timezone:  "America/Chicago",
	},
	"26MA": {
		ICAO:      "26MA",
		Name:      "Pepperell Airport",
		City:      "Pepperell",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 176,
		Latitude:  42.6962013245,
		Longitude: -71.5501022339,
		Timezone:  "America/New_York",
	},
	"26MI": {
		ICAO:      "26MI",
		Name:      "Van Strien Company Airport",
		City:      "Lowell",
		State:     "Michigan",
		Country:   "US",
		Elevation: 624,
		Latitude:  42.9239006042,
		Longitude: -85.3780975342,
		Timezone:  "America/Detroit",
	},
	"26MN": {
		ICAO:      "26MN",
		Name:      "Christianson Field",
		City:      "Halstad",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 874,
		Latitude:  47.3300018311,
		Longitude: -96.7836990356,
		Timezone:  "America/Chicago",
	},
	"26MO": {
		ICAO:      "26MO",
		Name:      "Taber Field",
		City:      "Theodosia",
		State:     "Missouri",
		Country:   "US",
		Elevation: 985,
		Latitude:  36.6133995056,
		Longitude: -92.7845993042,
		Timezone:  "America/Chicago",
	},
	"26MT": {
		ICAO:      "26MT",
		Name:      "Hutchinson Airport",
		City:      "Hingham",
		State:     "Montana",
		Country:   "US",
		Elevation: 2900,
		Latitude:  48.8356018066,
		Longitude: -110.4420013428,
		Timezone:  "America/Denver",
	},
	"26NC": {
		ICAO:      "26NC",
		Name:      "Boonville Airport",
		City:      "Boonville",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 1003,
		Latitude:  36.2229003906,
		Longitude: -80.7155990601,
		Timezone:  "America/New_York",
	},
	"26ND": {
		ICAO:      "26ND",
		Name:      "Chitwood Airstrip",
		City:      "Alexander",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 2180,
		Latitude:  47.7097015381,
		Longitude: -103.6439971924,
		Timezone:  "America/Chicago",
	},
	"26NV": {
		ICAO:      "26NV",
		Name:      "Darrow Field",
		City:      "Fallon",
		State:     "Nevada",
		Country:   "US",
		Elevation: 3965,
		Latitude:  39.4534262223,
		Longitude: -118.866330385,
		Timezone:  "America/Los_Angeles",
	},
	"26OG": {
		ICAO:      "26OG",
		Name:      "Oakridge Ranch Airport",
		City:      "Eagle Point",
		State:     "Washington",
		Country:   "US",
		Elevation: 1560,
		Latitude:  46.463211,
		Longitude: -122.733972,
		Timezone:  "America/Los_Angeles",
	},
	"26OH": {
		ICAO:      "26OH",
		Name:      "Hitz Airport",
		City:      "Louisville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1180,
		Latitude:  40.8753013611,
		Longitude: -81.2950973511,
		Timezone:  "America/New_York",
	},
	"26OK": {
		ICAO:      "26OK",
		Name:      "Duck Creek Airport",
		City:      "Mounds",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 700,
		Latitude:  35.8821983337,
		Longitude: -96.0044021606,
		Timezone:  "America/Chicago",
	},
	"26OR": {
		ICAO:      "26OR",
		Name:      "Cub Port Airport",
		City:      "Happy Valley",
		State:     "Oregon",
		Country:   "US",
		Elevation: 640,
		Latitude:  45.4379005432,
		Longitude: -122.5279998779,
		Timezone:  "America/Los_Angeles",
	},
	"26TX": {
		ICAO:      "26TX",
		Name:      "Lowrance Ranch Airport",
		City:      "Truscott",
		State:     "Texas",
		Country:   "US",
		Elevation: 1692,
		Latitude:  33.7648010254,
		Longitude: -100.0179977417,
		Timezone:  "America/Chicago",
	},
	"26WI": {
		ICAO:      "26WI",
		Name:      "Kitty-Wompus Airport",
		City:      "Exeland",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1336,
		Latitude:  45.6991004944,
		Longitude: -91.3034973145,
		Timezone:  "America/Chicago",
	},
	"26XS": {
		ICAO:      "26XS",
		Name:      "Tatum Ranch Airport",
		City:      "Blanco",
		State:     "Texas",
		Country:   "US",
		Elevation: 1420,
		Latitude:  30.1140995026,
		Longitude: -98.4614028931,
		Timezone:  "America/Chicago",
	},
	"27AL": {
		ICAO:      "27AL",
		Name:      "Little Texas Airport",
		City:      "Tuskegee",
		State:     "Alabama",
		Country:   "US",
		Elevation: 500,
		Latitude:  32.4534988403,
		Longitude: -85.5581970215,
		Timezone:  "America/Chicago",
	},
	"27AZ": {
		ICAO:      "27AZ",
		Name:      "Eagle Roost Airpark",
		City:      "Aguila",
		State:     "Arizona",
		Country:   "US",
		Elevation: 2206,
		Latitude:  33.9194984436,
		Longitude: -113.1669998169,
		Timezone:  "America/Phoenix",
	},
	"27CA": {
		ICAO:      "27CA",
		Name:      "Holiday Ranch Airport",
		City:      "Apple Valley",
		State:     "California",
		Country:   "US",
		Elevation: 3260,
		Latitude:  34.5611000061,
		Longitude: -117.0810012817,
		Timezone:  "America/Los_Angeles",
	},
	"27CL": {
		ICAO:      "27CL",
		Name:      "Sopwith Farm Airport",
		City:      "Pleasant Grove",
		State:     "California",
		Country:   "US",
		Elevation: 20,
		Latitude:  38.7471008301,
		Longitude: -121.5729980469,
		Timezone:  "America/Los_Angeles",
	},
	"27CO": {
		ICAO:      "27CO",
		Name:      "Roubideau Airport",
		City:      "Delta",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4900,
		Latitude:  38.7260017395,
		Longitude: -108.1289978027,
		Timezone:  "America/Denver",
	},
	"27FL": {
		ICAO:      "27FL",
		Name:      "Watson Airport",
		City:      "Campbellton",
		State:     "Florida",
		Country:   "US",
		Elevation: 122,
		Latitude:  30.9526996613,
		Longitude: -85.4173965454,
		Timezone:  "America/Chicago",
	},
	"27GA": {
		ICAO:      "27GA",
		Name:      "Wilson International Airport",
		City:      "Carrollton",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1050,
		Latitude:  33.6584014893,
		Longitude: -85.0099029541,
		Timezone:  "America/New_York",
	},
	"27IA": {
		ICAO:      "27IA",
		Name:      "Oldfield Aero",
		City:      "Keosauqua",
		State:     "Iowa",
		Country:   "US",
		Elevation: 750,
		Latitude:  40.7513008118,
		Longitude: -92.0307006836,
		Timezone:  "America/Chicago",
	},
	"27IS": {
		ICAO:      "27IS",
		Name:      "Gelfius International Airport",
		City:      "Dahlgren",
		State:     "Illinois",
		Country:   "US",
		Elevation: 530,
		Latitude:  38.1488990784,
		Longitude: -88.6703033447,
		Timezone:  "America/Chicago",
	},
	"27KS": {
		ICAO:      "27KS",
		Name:      "Bellamy Farm Airport",
		City:      "Goodland",
		State:     "Kansas",
		Country:   "US",
		Elevation: 3650,
		Latitude:  39.1288986206,
		Longitude: -101.7170028687,
		Timezone:  "America/Denver",
	},
	"27LL": {
		ICAO:      "27LL",
		Name:      "Ralph Jacobs Airport",
		City:      "Valmeyer",
		State:     "Illinois",
		Country:   "US",
		Elevation: 400,
		Latitude:  38.3750991821,
		Longitude: -90.3192977905,
		Timezone:  "America/Chicago",
	},
	"27MI": {
		ICAO:      "27MI",
		Name:      "Miller Field",
		City:      "Vassar",
		State:     "Michigan",
		Country:   "US",
		Elevation: 705,
		Latitude:  43.3727989197,
		Longitude: -83.4866027832,
		Timezone:  "America/Detroit",
	},
	"27MN": {
		ICAO:      "27MN",
		Name:      "Lake Bronson Airport",
		City:      "Lake Bronson",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 960,
		Latitude:  48.7374992371,
		Longitude: -96.6545028687,
		Timezone:  "America/Chicago",
	},
	"27MO": {
		ICAO:      "27MO",
		Name:      "Route 66 Airfield",
		City:      "Carthage",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1055,
		Latitude:  37.193901062,
		Longitude: -94.2241973877,
		Timezone:  "America/Chicago",
	},
	"27MT": {
		ICAO:      "27MT",
		Name:      "Rahn Airport",
		City:      "Kalispell",
		State:     "Montana",
		Country:   "US",
		Elevation: 3090,
		Latitude:  48.3218994141,
		Longitude: -114.327003479,
		Timezone:  "America/Denver",
	},
	"27NE": {
		ICAO:      "27NE",
		Name:      "Hendricks Field",
		City:      "Grant",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3450,
		Latitude:  40.7228012085,
		Longitude: -101.7959976196,
		Timezone:  "America/Denver",
	},
	"27NK": {
		ICAO:      "27NK",
		Name:      "Mohawk Air Park",
		City:      "Schyler",
		State:     "New York",
		Country:   "US",
		Elevation: 1079,
		Latitude:  43.113317,
		Longitude: -75.044467,
		Timezone:  "America/New_York",
	},
	"27NR": {
		ICAO:      "27NR",
		Name:      "Angier (C A G Farms) Airport",
		City:      "Angier",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 310,
		Latitude:  35.4687616608,
		Longitude: -78.6520028114,
		Timezone:  "America/New_York",
	},
	"27OH": {
		ICAO:      "27OH",
		Name:      "Schaller Airport",
		City:      "Findlay",
		State:     "Ohio",
		Country:   "US",
		Elevation: 830,
		Latitude:  40.9467010498,
		Longitude: -83.679901123,
		Timezone:  "America/New_York",
	},
	"27OI": {
		ICAO:      "27OI",
		Name:      "Auburn Airport",
		City:      "Chagrin Falls",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1200,
		Latitude:  41.4103012085,
		Longitude: -81.2843017578,
		Timezone:  "America/New_York",
	},
	"27OK": {
		ICAO:      "27OK",
		Name:      "Thomas Landing Airport",
		City:      "Ponca City",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1110,
		Latitude:  36.7422981262,
		Longitude: -96.9638977051,
		Timezone:  "America/Chicago",
	},
	"27PN": {
		ICAO:      "27PN",
		Name:      "Shelley Private Airport",
		City:      "Richfield",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 750,
		Latitude:  40.6811981201,
		Longitude: -77.1619033813,
		Timezone:  "America/New_York",
	},
	"27WA": {
		ICAO:      "27WA",
		Name:      "J C's Airport",
		City:      "Conway",
		State:     "Washington",
		Country:   "US",
		Elevation: 236,
		Latitude:  48.3250999451,
		Longitude: -122.3160018921,
		Timezone:  "America/Los_Angeles",
	},
	"27WI": {
		ICAO:      "27WI",
		Name:      "Tegeler Airport",
		City:      "Prescott",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 950,
		Latitude:  44.8077011108,
		Longitude: -92.7137985229,
		Timezone:  "America/Chicago",
	},
	"27XA": {
		ICAO:      "27XA",
		Name:      "Arnett Landing Airport",
		City:      "Poolville",
		State:     "Texas",
		Country:   "US",
		Elevation: 1180,
		Latitude:  32.979389,
		Longitude: -97.889917,
		Timezone:  "America/Chicago",
	},
	"27XS": {
		ICAO:      "27XS",
		Name:      "Sport Flyers Airport",
		City:      "Brookshire",
		State:     "Texas",
		Country:   "US",
		Elevation: 180,
		Latitude:  29.8379993439,
		Longitude: -95.9494018555,
		Timezone:  "America/Chicago",
	},
	"28AK": {
		ICAO:      "28AK",
		Name:      "Farrars Airport",
		City:      "Sutton",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1875,
		Latitude:  61.8088989258,
		Longitude: -147.8589935303,
		Timezone:  "America/Anchorage",
	},
	"28AL": {
		ICAO:      "28AL",
		Name:      "Golden Pond Airport",
		City:      "Ashville",
		State:     "Alabama",
		Country:   "US",
		Elevation: 540,
		Latitude:  33.8625984192,
		Longitude: -86.1206970215,
		Timezone:  "America/Chicago",
	},
	"28AR": {
		ICAO:      "28AR",
		Name:      "Henley Aerodrome",
		City:      "Charlotte",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 377,
		Latitude:  35.8333015442,
		Longitude: -91.4332962036,
		Timezone:  "America/Chicago",
	},
	"28AZ": {
		ICAO:      "28AZ",
		Name:      "Sampley's Airport",
		City:      "Aguila",
		State:     "Arizona",
		Country:   "US",
		Elevation: 2208,
		Latitude:  33.9227981567,
		Longitude: -113.1809997559,
		Timezone:  "America/Phoenix",
	},
	"28CA": {
		ICAO:      "28CA",
		Name:      "Dos Palos Airport",
		City:      "Dos Palos",
		State:     "California",
		Country:   "US",
		Elevation: 118,
		Latitude:  36.9623985291,
		Longitude: -120.6299972534,
		Timezone:  "America/Los_Angeles",
	},
	"28FL": {
		ICAO:      "28FL",
		Name:      "Jim Finlay Farm Airport",
		City:      "Cresent City",
		State:     "Florida",
		Country:   "US",
		Elevation: 40,
		Latitude:  29.4419002533,
		Longitude: -81.5830993652,
		Timezone:  "America/New_York",
	},
	"28II": {
		ICAO:      "28II",
		Name:      "Cedar Farm Airport",
		City:      "Laconia",
		State:     "Indiana",
		Country:   "US",
		Elevation: 453,
		Latitude:  37.9747009277,
		Longitude: -86.0561981201,
		Timezone:  "America/Kentucky/Louisville",
	},
	"28IL": {
		ICAO:      "28IL",
		Name:      "West Grove Airport",
		City:      "Leaf River",
		State:     "Illinois",
		Country:   "US",
		Elevation: 910,
		Latitude:  42.0971984863,
		Longitude: -89.4192962646,
		Timezone:  "America/Chicago",
	},
	"28IN": {
		ICAO:      "28IN",
		Name:      "Marchino Field",
		City:      "Vincennes",
		State:     "Indiana",
		Country:   "US",
		Elevation: 540,
		Latitude:  38.6377983093,
		Longitude: -87.4347000122,
		Timezone:  "America/Indiana/Vincennes",
	},
	"28IS": {
		ICAO:      "28IS",
		Name:      "O K Flying Club Inc Airport",
		City:      "Fillmore",
		State:     "Illinois",
		Country:   "US",
		Elevation: 665,
		Latitude:  39.1948013306,
		Longitude: -89.3117980957,
		Timezone:  "America/Chicago",
	},
	"28KS": {
		ICAO:      "28KS",
		Name:      "Highland Farm Airport",
		City:      "Rantoul",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1010,
		Latitude:  38.5424995422,
		Longitude: -95.1560974121,
		Timezone:  "America/Chicago",
	},
	"28KY": {
		ICAO:      "28KY",
		Name:      "Brandon Airdrome Airport",
		City:      "Murray",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 525,
		Latitude:  36.5759010315,
		Longitude: -88.3134002686,
		Timezone:  "America/Chicago",
	},
	"28LL": {
		ICAO:      "28LL",
		Name:      "Williamson Airport",
		City:      "Westervelt",
		State:     "Illinois",
		Country:   "US",
		Elevation: 630,
		Latitude:  39.4603004456,
		Longitude: -88.8684005737,
		Timezone:  "America/Chicago",
	},
	"28MA": {
		ICAO:      "28MA",
		Name:      "Cannizzaro Field",
		City:      "Southwick",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 250,
		Latitude:  42.0797996521,
		Longitude: -72.8180999756,
		Timezone:  "America/New_York",
	},
	"28MI": {
		ICAO:      "28MI",
		Name:      "Miller-Herrold Airport",
		City:      "Cedar",
		State:     "Michigan",
		Country:   "US",
		Elevation: 624,
		Latitude:  44.8744010925,
		Longitude: -85.7926025391,
		Timezone:  "America/Detroit",
	},
	"28MO": {
		ICAO:      "28MO",
		Name:      "Pasley Airport",
		City:      "Roscoe",
		State:     "Missouri",
		Country:   "US",
		Elevation: 790,
		Latitude:  37.9738998413,
		Longitude: -93.8188018799,
		Timezone:  "America/Chicago",
	},
	"28MT": {
		ICAO:      "28MT",
		Name:      "Cain Ranch Airport",
		City:      "Lincoln",
		State:     "Montana",
		Country:   "US",
		Elevation: 4480,
		Latitude:  46.953556,
		Longitude: -112.748083,
		Timezone:  "America/Denver",
	},
	"28NE": {
		ICAO:      "28NE",
		Name:      "Thies Airport",
		City:      "Randolph",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1750,
		Latitude:  42.4416999817,
		Longitude: -97.2966995239,
		Timezone:  "America/Chicago",
	},
	"28NK": {
		ICAO:      "28NK",
		Name:      "Ritchie Airfield",
		City:      "Clayton",
		State:     "New York",
		Country:   "US",
		Elevation: 385,
		Latitude:  44.186944,
		Longitude: -76.076389,
		Timezone:  "America/New_York",
	},
	"28NY": {
		ICAO:      "28NY",
		Name:      "Spaudling Aerodrome",
		City:      "Cassadaga",
		State:     "New York",
		Country:   "US",
		Elevation: 1607,
		Latitude:  42.308333,
		Longitude: -79.225,
		Timezone:  "America/New_York",
	},
	"28OH": {
		ICAO:      "28OH",
		Name:      "Newbury Airport",
		City:      "Lyons",
		State:     "Ohio",
		Country:   "US",
		Elevation: 743,
		Latitude:  41.7033996582,
		Longitude: -84.0027008057,
		Timezone:  "America/New_York",
	},
	"28OK": {
		ICAO:      "28OK",
		Name:      "Pellar Farm Airport",
		City:      "El Reno",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1400,
		Latitude:  35.5027999878,
		Longitude: -98.0406036377,
		Timezone:  "America/Chicago",
	},
	"28OR": {
		ICAO:      "28OR",
		Name:      "Parrett Mountain Airport",
		City:      "Newberg",
		State:     "Oregon",
		Country:   "US",
		Elevation: 790,
		Latitude:  45.4790000916,
		Longitude: -122.2340011597,
		Timezone:  "America/Los_Angeles",
	},
	"28PA": {
		ICAO:      "28PA",
		Name:      "Yarrow Airport",
		City:      "Schwenksville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 330,
		Latitude:  40.2512016296,
		Longitude: -75.4455032349,
		Timezone:  "America/New_York",
	},
	"28TA": {
		ICAO:      "28TA",
		Name:      "Duval County Ranch Co Airport",
		City:      "Freer",
		State:     "Texas",
		Country:   "US",
		Elevation: 665,
		Latitude:  27.83359909,
		Longitude: -98.71700287,
		Timezone:  "America/Chicago",
	},
	"28TE": {
		ICAO:      "28TE",
		Name:      "Midlake Airport",
		City:      "Elmendorf",
		State:     "Texas",
		Country:   "US",
		Elevation: 535,
		Latitude:  29.2682991028,
		Longitude: -98.3369979858,
		Timezone:  "America/Chicago",
	},
	"28TS": {
		ICAO:      "28TS",
		Name:      "Boe-Wrinkle Airport",
		City:      "Azle",
		State:     "Texas",
		Country:   "US",
		Elevation: 802,
		Latitude:  32.904800415,
		Longitude: -97.5950012207,
		Timezone:  "America/Chicago",
	},
	"28TX": {
		ICAO:      "28TX",
		Name:      "Lehman Airport",
		City:      "Vernon",
		State:     "Texas",
		Country:   "US",
		Elevation: 1275,
		Latitude:  34.085899353,
		Longitude: -99.4551010132,
		Timezone:  "America/Chicago",
	},
	"28WA": {
		ICAO:      "28WA",
		Name:      "Robert L Delanoy Airport",
		City:      "Vancouver",
		State:     "Washington",
		Country:   "US",
		Elevation: 30,
		Latitude:  45.7304000854,
		Longitude: -122.7350006104,
		Timezone:  "America/Los_Angeles",
	},
	"28WI": {
		ICAO:      "28WI",
		Name:      "Rutherford Airport",
		City:      "Arkansaw",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 790,
		Latitude:  44.6040992737,
		Longitude: -92.0466003418,
		Timezone:  "America/Chicago",
	},
	"28XS": {
		ICAO:      "28XS",
		Name:      "Flying G Airport",
		City:      "Kaufman",
		State:     "Texas",
		Country:   "US",
		Elevation: 470,
		Latitude:  32.6459999084,
		Longitude: -96.325302124,
		Timezone:  "America/Chicago",
	},
	"29AK": {
		ICAO:      "29AK",
		Name:      "Remington Field",
		City:      "Delta Junction",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1250,
		Latitude:  64.0495986938,
		Longitude: -145.4190063477,
		Timezone:  "America/Anchorage",
	},
	"29AZ": {
		ICAO:      "29AZ",
		Name:      "Paloma Ranch Airport",
		City:      "Paloma",
		State:     "Arizona",
		Country:   "US",
		Elevation: 737,
		Latitude:  32.9056015015,
		Longitude: -112.9000015259,
		Timezone:  "America/Phoenix",
	},
	"29CN": {
		ICAO:      "29CN",
		Name:      "Dubey Airport",
		City:      "Georgetown",
		State:     "California",
		Country:   "US",
		Elevation: 2400,
		Latitude:  38.849899292,
		Longitude: -120.8339996338,
		Timezone:  "America/Los_Angeles",
	},
	"29GA": {
		ICAO:      "29GA",
		Name:      "Rolling Meadows Airfield",
		City:      "Sharpsburg",
		State:     "Georgia",
		Country:   "US",
		Elevation: 808,
		Latitude:  33.3843002319,
		Longitude: -84.6318969727,
		Timezone:  "America/New_York",
	},
	"29II": {
		ICAO:      "29II",
		Name:      "Norm's Airport",
		City:      "Michigan City",
		State:     "Indiana",
		Country:   "US",
		Elevation: 750,
		Latitude:  41.6598014832,
		Longitude: -86.8075027466,
		Timezone:  "America/Chicago",
	},
	"29IN": {
		ICAO:      "29IN",
		Name:      "Homestead Airport",
		City:      "Columbia City",
		State:     "Indiana",
		Country:   "US",
		Elevation: 863,
		Latitude:  41.0759010315,
		Longitude: -85.386100769,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"29KS": {
		ICAO:      "29KS",
		Name:      "Navrat Airport",
		City:      "Lincolnville",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1470,
		Latitude:  38.48279953,
		Longitude: -97.0113983154,
		Timezone:  "America/Chicago",
	},
	"29KY": {
		ICAO:      "29KY",
		Name:      "Cramer Aerodrome",
		City:      "Hopkinsville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 640,
		Latitude:  36.8931007385,
		Longitude: -87.4272003174,
		Timezone:  "America/Chicago",
	},
	"29MD": {
		ICAO:      "29MD",
		Name:      "Schlosser Airport",
		City:      "Galena",
		State:     "Maryland",
		Country:   "US",
		Elevation: 53,
		Latitude:  39.3628005981,
		Longitude: -75.8127975464,
		Timezone:  "America/New_York",
	},
	"29MI": {
		ICAO:      "29MI",
		Name:      "Sawyer Field",
		City:      "Mason",
		State:     "Michigan",
		Country:   "US",
		Elevation: 910,
		Latitude:  42.5999984741,
		Longitude: -84.4499969482,
		Timezone:  "America/Detroit",
	},
	"29MN": {
		ICAO:      "29MN",
		Name:      "Wetenkamp Airport",
		City:      "Tintah",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 990,
		Latitude:  46.028301239,
		Longitude: -96.3628005981,
		Timezone:  "America/Chicago",
	},
	"29MO": {
		ICAO:      "29MO",
		Name:      "Rollert Farm Airport",
		City:      "Smithville",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1000,
		Latitude:  39.3507995605,
		Longitude: -94.5186004639,
		Timezone:  "America/Chicago",
	},
	"29MT": {
		ICAO:      "29MT",
		Name:      "Craik Airport",
		City:      "Trout Creek",
		State:     "Montana",
		Country:   "US",
		Elevation: 2500,
		Latitude:  47.738899231,
		Longitude: -115.5029983521,
		Timezone:  "America/Denver",
	},
	"29NC": {
		ICAO:      "29NC",
		Name:      "29NC Airport",
		City:      "Shelby",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 930,
		Latitude:  35.3116989136,
		Longitude: -81.6252975464,
		Timezone:  "America/New_York",
	},
	"29NE": {
		ICAO:      "29NE",
		Name:      "Bartmess Airport",
		City:      "Wallace",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3150,
		Latitude:  40.761100769,
		Longitude: -101.1809997559,
		Timezone:  "America/Chicago",
	},
	"29OH": {
		ICAO:      "29OH",
		Name:      "Fox Airport",
		City:      "Magnolia",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1020,
		Latitude:  40.6864013672,
		Longitude: -81.3025970459,
		Timezone:  "America/New_York",
	},
	"29OK": {
		ICAO:      "29OK",
		Name:      "Ditch Witch Airport",
		City:      "Perry",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1120,
		Latitude:  36.2999992371,
		Longitude: -97.3170013428,
		Timezone:  "America/Chicago",
	},
	"29OR": {
		ICAO:      "29OR",
		Name:      "Smith Private Airport",
		City:      "Brooks",
		State:     "Oregon",
		Country:   "US",
		Elevation: 180,
		Latitude:  45.0317993164,
		Longitude: -122.9570007324,
		Timezone:  "America/Los_Angeles",
	},
	"29PA": {
		ICAO:      "29PA",
		Name:      "Gardner Airport",
		City:      "Breinigsville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 500,
		Latitude:  40.5568008423,
		Longitude: -75.6432037354,
		Timezone:  "America/New_York",
	},
	"29SC": {
		ICAO:      "29SC",
		Name:      "Hannah Rhea Field",
		City:      "Walterboro",
		State:     "South Carolina",
		Country:   "US",
		Elevation: 89,
		Latitude:  32.9014015198,
		Longitude: -80.8535995483,
		Timezone:  "America/New_York",
	},
	"29TA": {
		ICAO:      "29TA",
		Name:      "Cactus Hill Airport",
		City:      "Wichita Falls",
		State:     "Texas",
		Country:   "US",
		Elevation: 1040,
		Latitude:  33.873053,
		Longitude: -98.645906,
		Timezone:  "America/Chicago",
	},
	"29TX": {
		ICAO:      "29TX",
		Name:      "Lockett Airport",
		City:      "Vernon",
		State:     "Texas",
		Country:   "US",
		Elevation: 1282,
		Latitude:  34.0945014954,
		Longitude: -99.3656005859,
		Timezone:  "America/Chicago",
	},
	"29VA": {
		ICAO:      "29VA",
		Name:      "Rhynalds Ranch Airport",
		City:      "Remington",
		State:     "Virginia",
		Country:   "US",
		Elevation: 285,
		Latitude:  38.5293006897,
		Longitude: -77.738899231,
		Timezone:  "America/New_York",
	},
	"29WI": {
		ICAO:      "29WI",
		Name:      "Whittlesey Cranberry Co Airport",
		City:      "Babcock",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 982,
		Latitude:  44.3255004883,
		Longitude: -90.0273971558,
		Timezone:  "America/Chicago",
	},
	"2AK0": {
		ICAO:      "2AK0",
		Name:      "Mac Kenzie Country Airpark",
		City:      "Point Mackenzie",
		State:     "Alaska",
		Country:   "US",
		Elevation: 120,
		Latitude:  61.2559013367,
		Longitude: -149.975006104,
		Timezone:  "America/Anchorage",
	},
	"2AK1": {
		ICAO:      "2AK1",
		Name:      "Blair Lake Airport",
		City:      "Fairbanks /Ft Wainwright/",
		State:     "Alaska",
		Country:   "US",
		Elevation: 910,
		Latitude:  64.3637008667,
		Longitude: -147.3639984131,
		Timezone:  "America/Anchorage",
	},
	"2AK2": {
		ICAO:      "2AK2",
		Name:      "Clear Creek Airport",
		City:      "Fairbanks /Ft Wainwright/",
		State:     "Alaska",
		Country:   "US",
		Elevation: 660,
		Latitude:  64.4534988403,
		Longitude: -147.56300354,
		Timezone:  "America/Anchorage",
	},
	"2AK4": {
		ICAO:      "2AK4",
		Name:      "Oyster Cove Airport",
		City:      "Homer",
		State:     "Alaska",
		Country:   "US",
		Elevation: 45,
		Latitude:  59.4700012207,
		Longitude: -151.5119934082,
		Timezone:  "America/Anchorage",
	},
	"2AK6": {
		ICAO:      "2AK6",
		IATA:      "HGZ",
		Name:      "Hog River Airport",
		City:      "Hogatza",
		State:     "Alaska",
		Country:   "US",
		Elevation: 534,
		Latitude:  66.2161026,
		Longitude: -155.6690063,
		Timezone:  "America/Anchorage",
	},
	"2AK7": {
		ICAO:      "2AK7",
		Name:      "Bald Mountain Airport",
		City:      "Talkeetna",
		State:     "Alaska",
		Country:   "US",
		Elevation: 3600,
		Latitude:  62.3073997498,
		Longitude: -149.7519989014,
		Timezone:  "America/Anchorage",
	},
	"2AK8": {
		ICAO:      "2AK8",
		Name:      "Icy Cape Afs Airport",
		City:      "Icy Cape",
		State:     "Alaska",
		Country:   "US",
		Elevation: 48,
		Latitude:  70.2925033569,
		Longitude: -161.9029998779,
		Timezone:  "America/Anchorage",
	},
	"2AK9": {
		ICAO:      "2AK9",
		Name:      "Independence Creek Airport",
		City:      "Independence Creek",
		State:     "Alaska",
		Country:   "US",
		Elevation: 250,
		Latitude:  65.673500061,
		Longitude: -162.4640045166,
		Timezone:  "America/Nome",
	},
	"2AL0": {
		ICAO:      "2AL0",
		Name:      "Paulling Place Airstrip",
		City:      "Dayton",
		State:     "Alabama",
		Country:   "US",
		Elevation: 275,
		Latitude:  32.3517990112,
		Longitude: -87.5456008911,
		Timezone:  "America/Chicago",
	},
	"2AL1": {
		ICAO:      "2AL1",
		Name:      "Collier Airpark",
		City:      "Magnolia Springs",
		State:     "Alabama",
		Country:   "US",
		Elevation: 74,
		Latitude:  30.4241008759,
		Longitude: -87.7778015137,
		Timezone:  "America/Chicago",
	},
	"2AL3": {
		ICAO:      "2AL3",
		Name:      "Emerald Mountain Airport",
		City:      "Wetumpka",
		State:     "Alabama",
		Country:   "US",
		Elevation: 365,
		Latitude:  32.4553985596,
		Longitude: -86.1192016602,
		Timezone:  "America/Chicago",
	},
	"2AL5": {
		ICAO:      "2AL5",
		Name:      "Flying J Ranch Airport",
		City:      "Fort Payne",
		State:     "Alabama",
		Country:   "US",
		Elevation: 1430,
		Latitude:  34.3927993774,
		Longitude: -85.6650009155,
		Timezone:  "America/Chicago",
	},
	"2AL6": {
		ICAO:      "2AL6",
		Name:      "Dale O. Galer Aerodrome",
		City:      "Theodore",
		State:     "Alabama",
		Country:   "US",
		Elevation: 40,
		Latitude:  30.4993000031,
		Longitude: -88.1981964111,
		Timezone:  "America/Chicago",
	},
	"2AL7": {
		ICAO:      "2AL7",
		Name:      "Mount Aero Lake Farm Airport",
		City:      "Hanceville",
		State:     "Alabama",
		Country:   "US",
		Elevation: 625,
		Latitude:  34.0628013611,
		Longitude: -86.7230987549,
		Timezone:  "America/Chicago",
	},
	"2AL8": {
		ICAO:      "2AL8",
		Name:      "Finkley Farm Airport",
		City:      "Phenix City",
		State:     "Alabama",
		Country:   "US",
		Elevation: 413,
		Latitude:  32.4211006165,
		Longitude: -85.2481002808,
		Timezone:  "America/Chicago",
	},
	"2AR0": {
		ICAO:      "2AR0",
		Name:      "Bully Henry Airport",
		City:      "Lonoke",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 245,
		Latitude:  34.8680992126,
		Longitude: -91.9742965698,
		Timezone:  "America/Chicago",
	},
	"2AR1": {
		ICAO:      "2AR1",
		Name:      "Seratt Airport",
		City:      "Rogers",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1305,
		Latitude:  36.292098999,
		Longitude: -94.0952987671,
		Timezone:  "America/Chicago",
	},
	"2AR2": {
		ICAO:      "2AR2",
		Name:      "Davidson Field",
		City:      "Sage",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 835,
		Latitude:  36.0152015686,
		Longitude: -91.7966003418,
		Timezone:  "America/Chicago",
	},
	"2AR4": {
		ICAO:      "2AR4",
		Name:      "Jaynes Field",
		City:      "Trumann",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 212,
		Latitude:  35.6707000732,
		Longitude: -90.5278015137,
		Timezone:  "America/Chicago",
	},
	"2AR5": {
		ICAO:      "2AR5",
		Name:      "Ashmore Field",
		City:      "Centerton",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1362,
		Latitude:  36.3830986023,
		Longitude: -94.2938995361,
		Timezone:  "America/Chicago",
	},
	"2AR6": {
		ICAO:      "2AR6",
		Name:      "Glenn Winchester Airport",
		City:      "Mineral Springs",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 365,
		Latitude:  33.8818016052,
		Longitude: -93.9284973145,
		Timezone:  "America/Chicago",
	},
	"2AR7": {
		ICAO:      "2AR7",
		Name:      "Head Airfield",
		City:      "Prescott",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 441,
		Latitude:  33.8490982056,
		Longitude: -93.4791030884,
		Timezone:  "America/Chicago",
	},
	"2AR8": {
		ICAO:      "2AR8",
		Name:      "L C Hickman Airport",
		City:      "Centerton",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1380,
		Latitude:  36.3527984619,
		Longitude: -94.3330993652,
		Timezone:  "America/Chicago",
	},
	"2AZ1": {
		ICAO:      "2AZ1",
		Name:      "Indian Hills Airpark",
		City:      "Salome",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1866,
		Latitude:  33.7592010498,
		Longitude: -113.6149978638,
		Timezone:  "America/Phoenix",
	},
	"2AZ3": {
		ICAO:      "2AZ3",
		Name:      "El Mirage-Village Square Airport",
		City:      "El Mirage",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1150,
		Latitude:  33.6128005981,
		Longitude: -112.3349990845,
		Timezone:  "America/Phoenix",
	},
	"2AZ4": {
		ICAO:      "2AZ4",
		Name:      "Millar Airport",
		City:      "Maricopa",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1215,
		Latitude:  33.0536994934,
		Longitude: -112.1460037231,
		Timezone:  "America/Phoenix",
	},
	"2AZ5": {
		ICAO:      "2AZ5",
		Name:      "Donnelly Residence Airport",
		City:      "Maricopa",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1518,
		Latitude:  32.8863983154,
		Longitude: -112.1299972534,
		Timezone:  "America/Phoenix",
	},
	"2AZ7": {
		ICAO:      "2AZ7",
		Name:      "Mc Gill's Ultralight Field",
		City:      "Phoenix",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1360,
		Latitude:  33.6842002869,
		Longitude: -112.164001465,
		Timezone:  "America/Phoenix",
	},
	"2AZ8": {
		ICAO:      "2AZ8",
		Name:      "Tubac Ultralight Flightpark Ultralightport",
		City:      "Tubac",
		State:     "Arizona",
		Country:   "US",
		Elevation: 3200,
		Latitude:  31.6145000458,
		Longitude: -111.0360031128,
		Timezone:  "America/Phoenix",
	},
	"2AZ9": {
		ICAO:      "2AZ9",
		Name:      "Tribal Air Airport",
		City:      "Mcneal",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4200,
		Latitude:  31.6023006439,
		Longitude: -109.6510009766,
		Timezone:  "America/Phoenix",
	},
	"2CA2": {
		ICAO:      "2CA2",
		Name:      "Cones Field",
		City:      "Twentynine Palms",
		State:     "California",
		Country:   "US",
		Elevation: 1800,
		Latitude:  34.1627998352,
		Longitude: -116.047996521,
		Timezone:  "America/Los_Angeles",
	},
	"2CA3": {
		ICAO:      "2CA3",
		Name:      "Crosswinds Airport",
		City:      "Twentynine Palms",
		State:     "California",
		Country:   "US",
		Elevation: 1835,
		Latitude:  34.1599998474,
		Longitude: -115.9970016479,
		Timezone:  "America/Los_Angeles",
	},
	"2CA4": {
		ICAO:      "2CA4",
		Name:      "Blackinton Airport",
		City:      "Valley Center",
		State:     "California",
		Country:   "US",
		Elevation: 1156,
		Latitude:  33.2588996887,
		Longitude: -117.0920028687,
		Timezone:  "America/Los_Angeles",
	},
	"2CA8": {
		ICAO:      "2CA8",
		Name:      "B & E Ranch Airport",
		City:      "Yucca Valley",
		State:     "California",
		Country:   "US",
		Elevation: 2793,
		Latitude:  34.4206008911,
		Longitude: -116.611000061,
		Timezone:  "America/Los_Angeles",
	},
	"2CD3": {
		ICAO:      "2CD3",
		Name:      "Bijou Springs Ranch Airport",
		City:      "Peyton",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6600,
		Latitude:  39.119472,
		Longitude: -104.431806,
		Timezone:  "America/Denver",
	},
	"2CD6": {
		ICAO:      "2CD6",
		Name:      "Safer Airport",
		City:      "Elizabeth",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6510,
		Latitude:  39.39485,
		Longitude: -104.547033,
		Timezone:  "America/Denver",
	},
	"2CD9": {
		ICAO:      "2CD9",
		Name:      "Kellogg Airstrip",
		City:      "Laporte",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5180,
		Latitude:  40.6814,
		Longitude: -105.13855,
		Timezone:  "America/Denver",
	},
	"2CL2": {
		ICAO:      "2CL2",
		Name:      "Mc Cabe Ranch Airport",
		City:      "Arbuckle",
		State:     "California",
		Country:   "US",
		Elevation: 65,
		Latitude:  39.0110015869,
		Longitude: -122.097999573,
		Timezone:  "America/Los_Angeles",
	},
	"2CL3": {
		ICAO:      "2CL3",
		Name:      "Longbell Ranch Airport",
		City:      "Tennant",
		State:     "California",
		Country:   "US",
		Elevation: 4607,
		Latitude:  41.6421012878,
		Longitude: -121.8899993896,
		Timezone:  "America/Los_Angeles",
	},
	"2CL9": {
		ICAO:      "2CL9",
		Name:      "Mustang Airport",
		City:      "Galt",
		State:     "California",
		Country:   "US",
		Elevation: 45,
		Latitude:  38.334400177,
		Longitude: -121.3040008545,
		Timezone:  "America/Los_Angeles",
	},
	"2CN3": {
		ICAO:      "2CN3",
		Name:      "Tenaja Valley Airport",
		City:      "Murrieta",
		State:     "California",
		Country:   "US",
		Elevation: 2350,
		Latitude:  33.511111,
		Longitude: -117.328611,
		Timezone:  "America/Los_Angeles",
	},
	"2CN4": {
		ICAO:      "2CN4",
		Name:      "Paramount Farms Airport",
		City:      "Lost Hills",
		State:     "California",
		Country:   "US",
		Elevation: 617,
		Latitude:  35.6663888889,
		Longitude: -119.896388889,
		Timezone:  "America/Los_Angeles",
	},
	"2CN8": {
		ICAO:      "2CN8",
		Name:      "Lake Arrowhead Airport",
		City:      "Lake Arrowhead",
		State:     "California",
		Country:   "US",
		Elevation: 4610,
		Latitude:  34.304167,
		Longitude: -117.151389,
		Timezone:  "America/Los_Angeles",
	},
	"2CO1": {
		ICAO:      "2CO1",
		Name:      "Cherokee Trail Ranch Airport",
		City:      "Peyton",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7240,
		Latitude:  39.1105003357,
		Longitude: -104.5839996338,
		Timezone:  "America/Denver",
	},
	"2CO2": {
		ICAO:      "2CO2",
		Name:      "CMRS Airdrome Airport",
		City:      "Poncha Springs",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7872,
		Latitude:  38.5402984619,
		Longitude: -106.105003357,
		Timezone:  "America/Denver",
	},
	"2CO3": {
		ICAO:      "2CO3",
		Name:      "Jackrabbit Strip",
		City:      "Milliken",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5000,
		Latitude:  40.375,
		Longitude: -104.8730010986,
		Timezone:  "America/Denver",
	},
	"2CO5": {
		ICAO:      "2CO5",
		Name:      "Edenway Airport",
		City:      "Pueblo",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4970,
		Latitude:  38.3471984863,
		Longitude: -104.6320037842,
		Timezone:  "America/Denver",
	},
	"2DE2": {
		ICAO:      "2DE2",
		Name:      "Willaview Airport",
		City:      "Farmington",
		State:     "Delaware",
		Country:   "US",
		Elevation: 63,
		Latitude:  38.8549995422,
		Longitude: -75.5721969604,
		Timezone:  "America/New_York",
	},
	"2FA2": {
		ICAO:      "2FA2",
		Name:      "Monroe Airpark",
		City:      "Belleview",
		State:     "Florida",
		Country:   "US",
		Elevation: 75,
		Latitude:  29.0188999176,
		Longitude: -82.1137008667,
		Timezone:  "America/New_York",
	},
	"2FA4": {
		ICAO:      "2FA4",
		Name:      "Southern Ranch Airport",
		City:      "Clewiston",
		State:     "Florida",
		Country:   "US",
		Elevation: 18,
		Latitude:  26.3794994354,
		Longitude: -80.945602417,
		Timezone:  "America/New_York",
	},
	"2FA5": {
		ICAO:      "2FA5",
		Name:      "Thunderbird Air Park",
		City:      "Crescent City",
		State:     "Florida",
		Country:   "US",
		Elevation: 67,
		Latitude:  29.4722003937,
		Longitude: -81.5719985962,
		Timezone:  "America/New_York",
	},
	"2FA6": {
		ICAO:      "2FA6",
		Name:      "Freeflight International Airport",
		City:      "Coleman",
		State:     "Florida",
		Country:   "US",
		Elevation: 55,
		Latitude:  28.8111000061,
		Longitude: -82.0653991699,
		Timezone:  "America/New_York",
	},
	"2FA9": {
		ICAO:      "2FA9",
		Name:      "Mount Olive Farm Airport",
		City:      "Lamont",
		State:     "Florida",
		Country:   "US",
		Elevation: 73,
		Latitude:  30.336389,
		Longitude: -83.783611,
		Timezone:  "America/New_York",
	},
	"2FD0": {
		ICAO:      "2FD0",
		Name:      "Flints Flying Ranch Airport",
		City:      "Branford",
		State:     "Florida",
		Country:   "US",
		Elevation: 45,
		Latitude:  29.9757995605,
		Longitude: -82.9073028564,
		Timezone:  "America/New_York",
	},
	"2FD1": {
		ICAO:      "2FD1",
		Name:      "Hobby Hill STOLport",
		City:      "Weirsdale",
		State:     "Florida",
		Country:   "US",
		Elevation: 95,
		Latitude:  28.9755001068,
		Longitude: -81.9187011719,
		Timezone:  "America/New_York",
	},
	"2FD2": {
		ICAO:      "2FD2",
		Name:      "Marjorie Kennan Rawlings Airport",
		City:      "Island Grove",
		State:     "Florida",
		Country:   "US",
		Elevation: 78,
		Latitude:  29.47929955,
		Longitude: -82.05329895,
		Timezone:  "America/New_York",
	},
	"2FD6": {
		ICAO:      "2FD6",
		Name:      "Highlander Airport",
		City:      "Oak Hill",
		State:     "Florida",
		Country:   "US",
		Elevation: 15,
		Latitude:  28.8880004883,
		Longitude: -80.8664016724,
		Timezone:  "America/New_York",
	},
	"2FD8": {
		ICAO:      "2FD8",
		Name:      "Lib Field",
		City:      "Welaka",
		State:     "Florida",
		Country:   "US",
		Elevation: 50,
		Latitude:  29.5210990906,
		Longitude: -81.6323013306,
		Timezone:  "America/New_York",
	},
	"2FL0": {
		ICAO:      "2FL0",
		Name:      "Crystal Village Airport",
		City:      "Wausau",
		State:     "Florida",
		Country:   "US",
		Elevation: 113,
		Latitude:  30.4568996429,
		Longitude: -85.6859970093,
		Timezone:  "America/Chicago",
	},
	"2FL3": {
		ICAO:      "2FL3",
		Name:      "Folsom Airport",
		City:      "Cypress",
		State:     "Florida",
		Country:   "US",
		Elevation: 155,
		Latitude:  30.6725997925,
		Longitude: -85.0835037231,
		Timezone:  "America/Chicago",
	},
	"2FL8": {
		ICAO:      "2FL8",
		Name:      "Tiger Lake Airport",
		City:      "River Ranch",
		State:     "Florida",
		Country:   "US",
		Elevation: 54,
		Latitude:  27.883600235,
		Longitude: -81.3630981445,
		Timezone:  "America/New_York",
	},
	"2GA0": {
		ICAO:      "2GA0",
		Name:      "Kennedy Intranational Airport",
		City:      "Newborn",
		State:     "Georgia",
		Country:   "US",
		Elevation: 695,
		Latitude:  33.5265007019,
		Longitude: -83.6380004883,
		Timezone:  "America/New_York",
	},
	"2GA1": {
		ICAO:      "2GA1",
		Name:      "Poole Farm Airport",
		City:      "Oxford",
		State:     "Georgia",
		Country:   "US",
		Elevation: 830,
		Latitude:  33.6806983948,
		Longitude: -83.8526992798,
		Timezone:  "America/New_York",
	},
	"2GA2": {
		ICAO:      "2GA2",
		Name:      "Swaids Field",
		City:      "Springfield",
		State:     "Georgia",
		Country:   "US",
		Elevation: 85,
		Latitude:  32.3945999146,
		Longitude: -81.2807006836,
		Timezone:  "America/New_York",
	},
	"2GA3": {
		ICAO:      "2GA3",
		Name:      "Wrights Field",
		City:      "Pinehurst",
		State:     "Georgia",
		Country:   "US",
		Elevation: 383,
		Latitude:  32.2062988281,
		Longitude: -83.7863006592,
		Timezone:  "America/New_York",
	},
	"2GA4": {
		ICAO:      "2GA4",
		Name:      "Mack's Field",
		City:      "Pinehurst",
		State:     "Georgia",
		Country:   "US",
		Elevation: 400,
		Latitude:  32.1712989807,
		Longitude: -83.7556991577,
		Timezone:  "America/New_York",
	},
	"2GA5": {
		ICAO:      "2GA5",
		Name:      "Windrift Aerodrome",
		City:      "Concord",
		State:     "Georgia",
		Country:   "US",
		Elevation: 780,
		Latitude:  33.0418014526,
		Longitude: -84.4320983887,
		Timezone:  "America/New_York",
	},
	"2GA6": {
		ICAO:      "2GA6",
		Name:      "Catoosa Springs STOLport",
		City:      "Ringgold",
		State:     "Georgia",
		Country:   "US",
		Elevation: 930,
		Latitude:  34.917301178,
		Longitude: -85.0524978638,
		Timezone:  "America/New_York",
	},
	"2GA8": {
		ICAO:      "2GA8",
		Name:      "Shannon Flight Strip",
		City:      "Shannon",
		State:     "Georgia",
		Country:   "US",
		Elevation: 677,
		Latitude:  34.3498001099,
		Longitude: -85.0768966675,
		Timezone:  "America/New_York",
	},
	"2GA9": {
		ICAO:      "2GA9",
		Name:      "Lenora Airport",
		City:      "Snellville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 940,
		Latitude:  33.804599762,
		Longitude: -83.9962997437,
		Timezone:  "America/New_York",
	},
	"2GE3": {
		ICAO:      "2GE3",
		Name:      "Smith Airport",
		City:      "Greensboro",
		State:     "Georgia",
		Country:   "US",
		Elevation: 550,
		Latitude:  33.6762008667,
		Longitude: -83.2593002319,
		Timezone:  "America/New_York",
	},
	"2GE5": {
		ICAO:      "2GE5",
		Name:      "Chukkar Farm Ultralightport",
		City:      "Alpharetta",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1000,
		Latitude:  34.1977996826,
		Longitude: -84.3193969727,
		Timezone:  "America/New_York",
	},
	"2GE7": {
		ICAO:      "2GE7",
		Name:      "Petty Farms Airport",
		City:      "Crandall",
		State:     "Georgia",
		Country:   "US",
		Elevation: 760,
		Latitude:  34.9580993652,
		Longitude: -84.7791976929,
		Timezone:  "America/New_York",
	},
	"2GE8": {
		ICAO:      "2GE8",
		Name:      "Andy Fields Airport",
		City:      "Newnan",
		State:     "Georgia",
		Country:   "US",
		Elevation: 980,
		Latitude:  33.4641990662,
		Longitude: -84.6632995605,
		Timezone:  "America/New_York",
	},
	"2IA9": {
		ICAO:      "2IA9",
		Name:      "South 80 Field",
		City:      "Crystal Lake",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1210,
		Latitude:  43.1907997131,
		Longitude: -93.7707977295,
		Timezone:  "America/Chicago",
	},
	"2ID2": {
		ICAO:      "2ID2",
		Name:      "Linda's Roost Airport",
		City:      "Mackay",
		State:     "Idaho",
		Country:   "US",
		Elevation: 6250,
		Latitude:  43.9543991089,
		Longitude: -113.6470031738,
		Timezone:  "America/Boise",
	},
	"2ID3": {
		ICAO:      "2ID3",
		Name:      "Josephine Ranch Airport",
		City:      "Triangle",
		State:     "Idaho",
		Country:   "US",
		Elevation: 5100,
		Latitude:  42.751833,
		Longitude: -116.677333,
		Timezone:  "America/Boise",
	},
	"2ID4": {
		ICAO:      "2ID4",
		Name:      "Silverwood Airport",
		City:      "Athol",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2350,
		Latitude:  47.9081993103,
		Longitude: -116.7089996338,
		Timezone:  "America/Los_Angeles",
	},
	"2ID5": {
		ICAO:      "2ID5",
		Name:      "Splan Airport",
		City:      "Mccall",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4987,
		Latitude:  44.807467,
		Longitude: -116.049483,
		Timezone:  "America/Boise",
	},
	"2ID6": {
		ICAO:      "2ID6",
		Name:      "Lemons Field",
		City:      "Fruitland",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2280,
		Latitude:  43.927336,
		Longitude: -116.946475,
		Timezone:  "America/Boise",
	},
	"2ID7": {
		ICAO:      "2ID7",
		Name:      "Cayuse Creek /US Forest Service/ Airport",
		City:      "Cayuse Creek",
		State:     "Idaho",
		Country:   "US",
		Elevation: 3500,
		Latitude:  46.6665992737,
		Longitude: -115.072998047,
		Timezone:  "America/Los_Angeles",
	},
	"2IG4": {
		ICAO:      "2IG4",
		Name:      "Ed-Air Airport",
		City:      "Oaktown",
		State:     "Indiana",
		Country:   "US",
		Elevation: 426,
		Latitude:  38.851398468,
		Longitude: -87.4997024536,
		Timezone:  "America/Chicago",
	},
	"2II0": {
		ICAO:      "2II0",
		Name:      "Indian Hills Flying Field",
		City:      "Kokomo",
		State:     "Indiana",
		Country:   "US",
		Elevation: 860,
		Latitude:  40.4174003601,
		Longitude: -86.1524963379,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"2II1": {
		ICAO:      "2II1",
		Name:      "Yoder Airport",
		City:      "Emma",
		State:     "Indiana",
		Country:   "US",
		Elevation: 905,
		Latitude:  41.6097984314,
		Longitude: -85.5254974365,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"2II2": {
		ICAO:      "2II2",
		Name:      "Indian Creek Airport",
		City:      "Mount Vernon",
		State:     "Indiana",
		Country:   "US",
		Elevation: 380,
		Latitude:  37.9874992371,
		Longitude: -87.9384002686,
		Timezone:  "America/Chicago",
	},
	"2II3": {
		ICAO:      "2II3",
		Name:      "Stottlemyer Airport",
		City:      "Frankton",
		State:     "Indiana",
		Country:   "US",
		Elevation: 865,
		Latitude:  40.2266998291,
		Longitude: -85.8163986206,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"2II6": {
		ICAO:      "2II6",
		Name:      "Baird-Wolford Airport",
		City:      "Tipton",
		State:     "Indiana",
		Country:   "US",
		Elevation: 880,
		Latitude:  40.2648010254,
		Longitude: -86.1027984619,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"2IL0": {
		ICAO:      "2IL0",
		Name:      "Sneek Airport",
		City:      "Shannon",
		State:     "Illinois",
		Country:   "US",
		Elevation: 909,
		Latitude:  42.1814002991,
		Longitude: -89.676902771,
		Timezone:  "America/Chicago",
	},
	"2IL1": {
		ICAO:      "2IL1",
		Name:      "Mc Cartney Airport",
		City:      "Durand",
		State:     "Illinois",
		Country:   "US",
		Elevation: 908,
		Latitude:  42.413898468,
		Longitude: -89.3292999268,
		Timezone:  "America/Chicago",
	},
	"2IL2": {
		ICAO:      "2IL2",
		Name:      "Watters Airport",
		City:      "Dwight",
		State:     "Illinois",
		Country:   "US",
		Elevation: 660,
		Latitude:  41.0533981323,
		Longitude: -88.4611968994,
		Timezone:  "America/Chicago",
	},
	"2IL3": {
		ICAO:      "2IL3",
		Name:      "Mc Neal's Field",
		City:      "East Moline",
		State:     "Illinois",
		Country:   "US",
		Elevation: 575,
		Latitude:  41.5158996582,
		Longitude: -90.3796005249,
		Timezone:  "America/Chicago",
	},
	"2IL4": {
		ICAO:      "2IL4",
		Name:      "Mountain Airport",
		City:      "Perry",
		State:     "Illinois",
		Country:   "US",
		Elevation: 590,
		Latitude:  39.7792015076,
		Longitude: -90.7485961914,
		Timezone:  "America/Chicago",
	},
	"2IL5": {
		ICAO:      "2IL5",
		Name:      "Sutton Airport",
		City:      "Omaha",
		State:     "Illinois",
		Country:   "US",
		Elevation: 470,
		Latitude:  37.9342002869,
		Longitude: -88.2316970825,
		Timezone:  "America/Chicago",
	},
	"2IL6": {
		ICAO:      "2IL6",
		Name:      "Low and Slow Airport",
		City:      "Hardin",
		State:     "Illinois",
		Country:   "US",
		Elevation: 455,
		Latitude:  39.1375007629,
		Longitude: -90.6299972534,
		Timezone:  "America/Chicago",
	},
	"2IL7": {
		ICAO:      "2IL7",
		Name:      "Ben Emge Airport",
		City:      "Belleville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 555,
		Latitude:  38.5153007507,
		Longitude: -90.0279006958,
		Timezone:  "America/Chicago",
	},
	"2IL9": {
		ICAO:      "2IL9",
		Name:      "Meadow Creek Airport",
		City:      "Monee",
		State:     "Illinois",
		Country:   "US",
		Elevation: 770,
		Latitude:  41.432800293,
		Longitude: -87.7817001343,
		Timezone:  "America/Chicago",
	},
	"2IN0": {
		ICAO:      "2IN0",
		Name:      "Skip's Place Airport",
		City:      "Ossian",
		State:     "Indiana",
		Country:   "US",
		Elevation: 858,
		Latitude:  40.8368988037,
		Longitude: -85.1710968018,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"2IN2": {
		ICAO:      "2IN2",
		Name:      "Gerig's Field",
		City:      "Roanoke",
		State:     "Indiana",
		Country:   "US",
		Elevation: 850,
		Latitude:  41.0092010498,
		Longitude: -85.3713989258,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"2IN3": {
		ICAO:      "2IN3",
		Name:      "Tatertown Airport",
		City:      "Medaryville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 700,
		Latitude:  41.0848007202,
		Longitude: -86.9242019653,
		Timezone:  "America/Chicago",
	},
	"2IN4": {
		ICAO:      "2IN4",
		Name:      "Scott Field",
		City:      "Argos",
		State:     "Indiana",
		Country:   "US",
		Elevation: 810,
		Latitude:  41.2566986084,
		Longitude: -86.1896972656,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"2IN5": {
		ICAO:      "2IN5",
		Name:      "Midkiff Airport",
		City:      "Sulphur Springs",
		State:     "Indiana",
		Country:   "US",
		Elevation: 1038,
		Latitude:  40.0152015686,
		Longitude: -85.4729003906,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"2IN6": {
		ICAO:      "2IN6",
		Name:      "Galloway Airport",
		City:      "Noblesville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 830,
		Latitude:  40.0419006348,
		Longitude: -85.9169006348,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"2IN9": {
		ICAO:      "2IN9",
		Name:      "Berry Field",
		City:      "New Whiteland",
		State:     "Indiana",
		Country:   "US",
		Elevation: 795,
		Latitude:  39.5666999817,
		Longitude: -86.1181030273,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"2IS3": {
		ICAO:      "2IS3",
		Name:      "Jackson Field",
		City:      "Elmira",
		State:     "Illinois",
		Country:   "US",
		Elevation: 760,
		Latitude:  41.1716995239,
		Longitude: -89.8068008423,
		Timezone:  "America/Chicago",
	},
	"2IS4": {
		ICAO:      "2IS4",
		Name:      "Ritter Field",
		City:      "Illinois City",
		State:     "Illinois",
		Country:   "US",
		Elevation: 552,
		Latitude:  41.3370018005,
		Longitude: -91.0131988525,
		Timezone:  "America/Chicago",
	},
	"2IS5": {
		ICAO:      "2IS5",
		Name:      "Parrish RLA Restricted Landing Area",
		City:      "Lacon",
		State:     "Illinois",
		Country:   "US",
		Elevation: 640,
		Latitude:  40.9523010254,
		Longitude: -89.4078979492,
		Timezone:  "America/Chicago",
	},
	"2IS6": {
		ICAO:      "2IS6",
		Name:      "Red Shed Field",
		City:      "Lena",
		State:     "Illinois",
		Country:   "US",
		Elevation: 900,
		Latitude:  42.4180984497,
		Longitude: -89.8226013184,
		Timezone:  "America/Chicago",
	},
	"2IS7": {
		ICAO:      "2IS7",
		Name:      "Somers Blossom Airport",
		City:      "Lindenwood",
		State:     "Illinois",
		Country:   "US",
		Elevation: 789,
		Latitude:  42.0606002808,
		Longitude: -89.00340271,
		Timezone:  "America/Chicago",
	},
	"2IS9": {
		ICAO:      "2IS9",
		Name:      "Schertz Aerial Service - Cooksville Airport",
		City:      "Cooksville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 741,
		Latitude:  40.5780982971,
		Longitude: -88.7078018188,
		Timezone:  "America/Chicago",
	},
	"2KS0": {
		ICAO:      "2KS0",
		Name:      "Rupp Airport",
		City:      "Sedan",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1000,
		Latitude:  37.2066993713,
		Longitude: -96.0768966675,
		Timezone:  "America/Chicago",
	},
	"2KS1": {
		ICAO:      "2KS1",
		Name:      "Rush Airport",
		City:      "Severance",
		State:     "Kansas",
		Country:   "US",
		Elevation: 950,
		Latitude:  39.7743988037,
		Longitude: -95.2510986328,
		Timezone:  "America/Chicago",
	},
	"2KS2": {
		ICAO:      "2KS2",
		Name:      "Stuber Flying Ranch Airport",
		City:      "Severy",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1200,
		Latitude:  37.6556015015,
		Longitude: -96.2906036377,
		Timezone:  "America/Chicago",
	},
	"2KS3": {
		ICAO:      "2KS3",
		Name:      "Wilson Airport",
		City:      "Wilson",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1727,
		Latitude:  38.8381004333,
		Longitude: -98.4856033325,
		Timezone:  "America/Chicago",
	},
	"2KS4": {
		ICAO:      "2KS4",
		Name:      "Dick Airport",
		City:      "Spivey",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1549,
		Latitude:  37.4477996826,
		Longitude: -98.2253036499,
		Timezone:  "America/Chicago",
	},
	"2KS5": {
		ICAO:      "2KS5",
		Name:      "Plains Municipal Airport",
		City:      "Plains",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2762,
		Latitude:  37.2709007263,
		Longitude: -100.5879974365,
		Timezone:  "America/Chicago",
	},
	"2KS8": {
		ICAO:      "2KS8",
		Name:      "Vonada Airport",
		City:      "Sylvan Grove",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1629,
		Latitude:  39.1166992188,
		Longitude: -98.4002990723,
		Timezone:  "America/Chicago",
	},
	"2KS9": {
		ICAO:      "2KS9",
		Name:      "Starshire Farm Airport",
		City:      "Topeka",
		State:     "Kansas",
		Country:   "US",
		Elevation: 990,
		Latitude:  38.9000015259,
		Longitude: -95.5836029053,
		Timezone:  "America/Chicago",
	},
	"2KY1": {
		ICAO:      "2KY1",
		Name:      "Kitty Hawk Farm Ultralightport",
		City:      "Paint Lick",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 980,
		Latitude:  37.6192016602,
		Longitude: -84.381401062,
		Timezone:  "America/New_York",
	},
	"2KY3": {
		ICAO:      "2KY3",
		Name:      "Plane-O-Field Airport",
		City:      "Bowling Green",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 655,
		Latitude:  36.8347015381,
		Longitude: -86.4582977295,
		Timezone:  "America/Chicago",
	},
	"2KY4": {
		ICAO:      "2KY4",
		Name:      "Oz Airport",
		City:      "Elizabethtown",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 820,
		Latitude:  37.6549987793,
		Longitude: -85.8135986328,
		Timezone:  "America/New_York",
	},
	"2KY5": {
		ICAO:      "2KY5",
		Name:      "Womstead Field",
		City:      "Olive Hill",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 1050,
		Latitude:  38.375,
		Longitude: -83.1547012329,
		Timezone:  "America/New_York",
	},
	"2KY8": {
		ICAO:      "2KY8",
		Name:      "Seldom Scene Airport",
		City:      "Millville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 860,
		Latitude:  38.1055984497,
		Longitude: -84.8499984741,
		Timezone:  "America/New_York",
	},
	"2LA0": {
		ICAO:      "2LA0",
		Name:      "Central Industries Airport",
		City:      "Intracoastal City",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 2,
		Latitude:  29.7896995544,
		Longitude: -92.15650177,
		Timezone:  "America/Chicago",
	},
	"2LA4": {
		ICAO:      "2LA4",
		Name:      "Bunkie Flying Service Airport",
		City:      "Bunkie",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 65,
		Latitude:  30.9291000366,
		Longitude: -92.1792984009,
		Timezone:  "America/Chicago",
	},
	"2LA5": {
		ICAO:      "2LA5",
		Name:      "Reed Flying Service Inc Airport",
		City:      "Iota",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 35,
		Latitude:  30.3432998657,
		Longitude: -92.4320983887,
		Timezone:  "America/Chicago",
	},
	"2LA6": {
		ICAO:      "2LA6",
		Name:      "Howell 1 Airport",
		City:      "Jackson",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 180,
		Latitude:  30.8334999084,
		Longitude: -91.1668014526,
		Timezone:  "America/Chicago",
	},
	"2LA7": {
		ICAO:      "2LA7",
		Name:      "Costello Airport",
		City:      "Oak Grove",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 75,
		Latitude:  32.9670982361,
		Longitude: -91.4262008667,
		Timezone:  "America/Chicago",
	},
	"2LL1": {
		ICAO:      "2LL1",
		Name:      "Cwian Field",
		City:      "Sheridan",
		State:     "Illinois",
		Country:   "US",
		Elevation: 550,
		Latitude:  41.5416984558,
		Longitude: -88.6890029907,
		Timezone:  "America/Chicago",
	},
	"2LL2": {
		ICAO:      "2LL2",
		Name:      "Weihler Airport",
		City:      "Sherrard",
		State:     "Illinois",
		Country:   "US",
		Elevation: 740,
		Latitude:  41.3148002625,
		Longitude: -90.5521011353,
		Timezone:  "America/Chicago",
	},
	"2LL3": {
		ICAO:      "2LL3",
		Name:      "Roy Burden Restricted Landing Area",
		City:      "Shirley",
		State:     "Illinois",
		Country:   "US",
		Elevation: 730,
		Latitude:  40.3889007568,
		Longitude: -89.0691986084,
		Timezone:  "America/Chicago",
	},
	"2LL4": {
		ICAO:      "2LL4",
		Name:      "Snow Airport",
		City:      "Macon",
		State:     "Illinois",
		Country:   "US",
		Elevation: 728,
		Latitude:  39.7498016357,
		Longitude: -88.9794998169,
		Timezone:  "America/Chicago",
	},
	"2LL5": {
		ICAO:      "2LL5",
		Name:      "Justus Airport",
		City:      "Sidney",
		State:     "Illinois",
		Country:   "US",
		Elevation: 685,
		Latitude:  39.9986000061,
		Longitude: -88.0599975586,
		Timezone:  "America/Chicago",
	},
	"2LL7": {
		ICAO:      "2LL7",
		Name:      "Adams Restricted Landing Area Number 2",
		City:      "Pontiac",
		State:     "Illinois",
		Country:   "US",
		Elevation: 630,
		Latitude:  40.9347000122,
		Longitude: -88.7376022339,
		Timezone:  "America/Chicago",
	},
	"2LL9": {
		ICAO:      "2LL9",
		Name:      "George Airport",
		City:      "Somonauk",
		State:     "Illinois",
		Country:   "US",
		Elevation: 740,
		Latitude:  41.7042007446,
		Longitude: -88.6930999756,
		Timezone:  "America/Chicago",
	},
	"2LS2": {
		ICAO:      "2LS2",
		Name:      "Chitimacha Air Park",
		City:      "Charenton",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 14,
		Latitude:  29.872222,
		Longitude: -91.543056,
		Timezone:  "America/Chicago",
	},
	"2LS3": {
		ICAO:      "2LS3",
		Name:      "Majors Airpark",
		City:      "Melville",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 34,
		Latitude:  30.618244,
		Longitude: -91.752064,
		Timezone:  "America/Chicago",
	},
	"2MD0": {
		ICAO:      "2MD0",
		Name:      "Anderson Farm Airport",
		City:      "Marion",
		State:     "Maryland",
		Country:   "US",
		Elevation: 8,
		Latitude:  38.0486984253,
		Longitude: -75.7034988403,
		Timezone:  "America/New_York",
	},
	"2MD1": {
		ICAO:      "2MD1",
		Name:      "Recompense Farm Airport",
		City:      "Clements",
		State:     "Maryland",
		Country:   "US",
		Elevation: 30,
		Latitude:  38.3084983826,
		Longitude: -76.7247009277,
		Timezone:  "America/New_York",
	},
	"2MD3": {
		ICAO:      "2MD3",
		Name:      "Fly Away Farm Airport",
		City:      "Boyds",
		State:     "Maryland",
		Country:   "US",
		Elevation: 501,
		Latitude:  39.229,
		Longitude: -77.345333,
		Timezone:  "America/New_York",
	},
	"2MD4": {
		ICAO:      "2MD4",
		Name:      "Ennis Aerodrome",
		City:      "Salisbury",
		State:     "Maryland",
		Country:   "US",
		Elevation: 44,
		Latitude:  38.3945999146,
		Longitude: -75.5574035645,
		Timezone:  "America/New_York",
	},
	"2MD6": {
		ICAO:      "2MD6",
		Name:      "Cherry Field",
		City:      "Lexington Park",
		State:     "Maryland",
		Country:   "US",
		Elevation: 8,
		Latitude:  38.1440010071,
		Longitude: -76.4721984863,
		Timezone:  "America/New_York",
	},
	"2MD8": {
		ICAO:      "2MD8",
		Name:      "Greater Gortner Airport",
		City:      "Oakland",
		State:     "Maryland",
		Country:   "US",
		Elevation: 2500,
		Latitude:  39.3334007263,
		Longitude: -79.4413986206,
		Timezone:  "America/New_York",
	},
	"2ME3": {
		ICAO:      "2ME3",
		Name:      "Charleston / Harte Field",
		City:      "Charleston",
		State:     "Maine",
		Country:   "US",
		Elevation: 230,
		Latitude:  45.04135,
		Longitude: -68.990021,
		Timezone:  "America/New_York",
	},
	"2MI0": {
		ICAO:      "2MI0",
		Name:      "Woodside Airport",
		City:      "Chesaning",
		State:     "Michigan",
		Country:   "US",
		Elevation: 615,
		Latitude:  43.2167015076,
		Longitude: -84.047203064,
		Timezone:  "America/Detroit",
	},
	"2MI3": {
		ICAO:      "2MI3",
		Name:      "Larry D Boven Airport",
		City:      "Galesburg",
		State:     "Michigan",
		Country:   "US",
		Elevation: 890,
		Latitude:  42.3125,
		Longitude: -85.4633026123,
		Timezone:  "America/Detroit",
	},
	"2MI4": {
		ICAO:      "2MI4",
		Name:      "Mc Jilton Field",
		City:      "St Louis",
		State:     "Michigan",
		Country:   "US",
		Elevation: 729,
		Latitude:  43.4056015015,
		Longitude: -84.5714035034,
		Timezone:  "America/Detroit",
	},
	"2MI5": {
		ICAO:      "2MI5",
		Name:      "Somerville Airport",
		City:      "Ada",
		State:     "Michigan",
		Country:   "US",
		Elevation: 660,
		Latitude:  42.9416999817,
		Longitude: -85.4863967896,
		Timezone:  "America/Detroit",
	},
	"2MI7": {
		ICAO:      "2MI7",
		Name:      "Flugplatz Airport",
		City:      "Lexington",
		State:     "Michigan",
		Country:   "US",
		Elevation: 700,
		Latitude:  43.3469009399,
		Longitude: -82.5522003174,
		Timezone:  "America/Detroit",
	},
	"2MI8": {
		ICAO:      "2MI8",
		Name:      "Vassar Field",
		City:      "Vassar",
		State:     "Michigan",
		Country:   "US",
		Elevation: 675,
		Latitude:  43.3433990479,
		Longitude: -83.5352020264,
		Timezone:  "America/Detroit",
	},
	"2MI9": {
		ICAO:      "2MI9",
		Name:      "Capen Airport",
		City:      "Mecosta",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1015,
		Latitude:  43.6241989136,
		Longitude: -85.3178024292,
		Timezone:  "America/Detroit",
	},
	"2MN0": {
		ICAO:      "2MN0",
		Name:      "Pribbs Field",
		City:      "East Grand Forks",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 818,
		Latitude:  48.0820999146,
		Longitude: -97.0162963867,
		Timezone:  "America/Chicago",
	},
	"2MN1": {
		ICAO:      "2MN1",
		Name:      "Winter Strip",
		City:      "Gluek",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1040,
		Latitude:  45.0461006165,
		Longitude: -95.4645004272,
		Timezone:  "America/Chicago",
	},
	"2MN2": {
		ICAO:      "2MN2",
		Name:      "Dreamcatcher Airport",
		City:      "Jacobson",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1250,
		Latitude:  46.9403991699,
		Longitude: -93.2978973389,
		Timezone:  "America/Chicago",
	},
	"2MN6": {
		ICAO:      "2MN6",
		Name:      "Van Norman's Airport",
		City:      "St Charles",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1100,
		Latitude:  44.0471992493,
		Longitude: -92.0929031372,
		Timezone:  "America/Chicago",
	},
	"2MN7": {
		ICAO:      "2MN7",
		Name:      "Fussy Airport",
		City:      "North Prairie",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1100,
		Latitude:  45.7915992737,
		Longitude: -94.3407974243,
		Timezone:  "America/Chicago",
	},
	"2MN8": {
		ICAO:      "2MN8",
		Name:      "Trygstad Airport",
		City:      "Rochester",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1227,
		Latitude:  43.9961013794,
		Longitude: -92.3357009888,
		Timezone:  "America/Chicago",
	},
	"2MN9": {
		ICAO:      "2MN9",
		Name:      "F. Dillenburg Airport",
		City:      "Foley",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1295,
		Latitude:  45.8244018555,
		Longitude: -93.995300293,
		Timezone:  "America/Chicago",
	},
	"2MO0": {
		ICAO:      "2MO0",
		Name:      "Fletcher Field",
		City:      "Centerview",
		State:     "Missouri",
		Country:   "US",
		Elevation: 805,
		Latitude:  38.7994003296,
		Longitude: -93.8641967773,
		Timezone:  "America/Chicago",
	},
	"2MO1": {
		ICAO:      "2MO1",
		Name:      "Bird Field",
		City:      "Willard",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1250,
		Latitude:  37.3199996948,
		Longitude: -93.4198989868,
		Timezone:  "America/Chicago",
	},
	"2MO2": {
		ICAO:      "2MO2",
		Name:      "Northwood Airport",
		City:      "Holt",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1000,
		Latitude:  39.4641990662,
		Longitude: -94.3079986572,
		Timezone:  "America/Chicago",
	},
	"2MO4": {
		ICAO:      "2MO4",
		Name:      "Breckenridge Airport",
		City:      "Rockville",
		State:     "Missouri",
		Country:   "US",
		Elevation: 805,
		Latitude:  38.1143989563,
		Longitude: -94.1766967773,
		Timezone:  "America/Chicago",
	},
	"2MO5": {
		ICAO:      "2MO5",
		Name:      "Pegasus Ranch Aerodrome",
		City:      "Houston",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1430,
		Latitude:  37.3819999695,
		Longitude: -92.2107009888,
		Timezone:  "America/Chicago",
	},
	"2MO6": {
		ICAO:      "2MO6",
		Name:      "Hunziker Airport",
		City:      "Hurdland",
		State:     "Missouri",
		Country:   "US",
		Elevation: 890,
		Latitude:  40.1344985962,
		Longitude: -92.2976989746,
		Timezone:  "America/Chicago",
	},
	"2MO7": {
		ICAO:      "2MO7",
		Name:      "Fawn Lake Airport",
		City:      "Warrenton",
		State:     "Missouri",
		Country:   "US",
		Elevation: 750,
		Latitude:  38.8666992188,
		Longitude: -91.0860977173,
		Timezone:  "America/Chicago",
	},
	"2MO8": {
		ICAO:      "2MO8",
		Name:      "Frerer Strip",
		City:      "Carthage",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1020,
		Latitude:  37.1351013184,
		Longitude: -94.3640975952,
		Timezone:  "America/Chicago",
	},
	"2MO9": {
		ICAO:      "2MO9",
		Name:      "Runway Ranch Airport",
		City:      "Kansas City",
		State:     "Missouri",
		Country:   "US",
		Elevation: 929,
		Latitude:  38.9500007629,
		Longitude: -94.4502029419,
		Timezone:  "America/Chicago",
	},
	"2MS7": {
		ICAO:      "2MS7",
		Name:      "Puff Airpark",
		City:      "Puckett",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 400,
		Latitude:  32.0584983826,
		Longitude: -89.8167037964,
		Timezone:  "America/Chicago",
	},
	"2MS8": {
		ICAO:      "2MS8",
		Name:      "Spencer Field",
		City:      "Grenada",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 175,
		Latitude:  33.7896995544,
		Longitude: -89.8743972778,
		Timezone:  "America/Chicago",
	},
	"2MS9": {
		ICAO:      "2MS9",
		Name:      "Kimmel Land & Cattle Airport",
		City:      "Houston",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 265,
		Latitude:  33.8300018311,
		Longitude: -88.9227981567,
		Timezone:  "America/Chicago",
	},
	"2MT0": {
		ICAO:      "2MT0",
		Name:      "Bates Airstrip",
		City:      "Kalispell",
		State:     "Montana",
		Country:   "US",
		Elevation: 3110,
		Latitude:  48.2999992371,
		Longitude: -114.4140014648,
		Timezone:  "America/Denver",
	},
	"2MT1": {
		ICAO:      "2MT1",
		Name:      "Ryan Field",
		City:      "West Glacier",
		State:     "Montana",
		Country:   "US",
		Elevation: 3660,
		Latitude:  48.4831008911,
		Longitude: -113.9589996338,
		Timezone:  "America/Denver",
	},
	"2MT2": {
		ICAO:      "2MT2",
		Name:      "Braidwater Farm Airport",
		City:      "Kalispell",
		State:     "Montana",
		Country:   "US",
		Elevation: 2910,
		Latitude:  48.2000007629,
		Longitude: -114.2580032349,
		Timezone:  "America/Denver",
	},
	"2MT5": {
		ICAO:      "2MT5",
		Name:      "Briar Creek Airport",
		City:      "Bozeman",
		State:     "Montana",
		Country:   "US",
		Elevation: 5175,
		Latitude:  45.5941009521,
		Longitude: -111.1520004272,
		Timezone:  "America/Denver",
	},
	"2MT8": {
		ICAO:      "2MT8",
		Name:      "South Boulder Airport",
		City:      "Cardwell",
		State:     "Montana",
		Country:   "US",
		Elevation: 4560,
		Latitude:  45.8151,
		Longitude: -111.9259,
		Timezone:  "America/Denver",
	},
	"2MT9": {
		ICAO:      "2MT9",
		Name:      "Gold Creek Airport",
		City:      "Drummond",
		State:     "Montana",
		Country:   "US",
		Elevation: 4956,
		Latitude:  46.513889,
		Longitude: -112.983333,
		Timezone:  "America/Denver",
	},
	"2MU9": {
		ICAO:      "2MU9",
		Name:      "Monroe Field",
		City:      "Hawk Point",
		State:     "Missouri",
		Country:   "US",
		Elevation: 710,
		Latitude:  38.9655990601,
		Longitude: -91.1181030273,
		Timezone:  "America/Chicago",
	},
	"2MY2": {
		ICAO:      "2MY2",
		Name:      "Zarn Airport",
		City:      "Amboy",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1050,
		Latitude:  43.907417,
		Longitude: -94.274667,
		Timezone:  "America/Chicago",
	},
	"2MY4": {
		ICAO:      "2MY4",
		Name:      "Miller Airport",
		City:      "Clear Lake",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 987,
		Latitude:  45.433869,
		Longitude: -93.948714,
		Timezone:  "America/Chicago",
	},
	"2NA0": {
		ICAO:      "2NA0",
		Name:      "Soderquist Airport",
		City:      "Wilton",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1880,
		Latitude:  47.2566986084,
		Longitude: -100.7959976196,
		Timezone:  "America/Chicago",
	},
	"2NA7": {
		ICAO:      "2NA7",
		Name:      "Slater Farm Airport",
		City:      "Esmond",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1600,
		Latitude:  48.028301239,
		Longitude: -99.6165008545,
		Timezone:  "America/Chicago",
	},
	"2NC0": {
		ICAO:      "2NC0",
		Name:      "Mountain Air Airport",
		City:      "Burnsville",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 4432,
		Latitude:  35.8686981201,
		Longitude: -82.341796875,
		Timezone:  "America/New_York",
	},
	"2NC1": {
		ICAO:      "2NC1",
		Name:      "Hawk's Knoll Airport",
		City:      "Waxhaw",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 695,
		Latitude:  34.9973983765,
		Longitude: -80.8009033203,
		Timezone:  "America/New_York",
	},
	"2NC4": {
		ICAO:      "2NC4",
		Name:      "Scottbrook Farm Airport",
		City:      "Goldsboro",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 160,
		Latitude:  35.3378982544,
		Longitude: -78.1547012329,
		Timezone:  "America/New_York",
	},
	"2NC6": {
		ICAO:      "2NC6",
		Name:      "Flying M Airport",
		City:      "Lexington",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 845,
		Latitude:  35.7995986938,
		Longitude: -80.3962020874,
		Timezone:  "America/New_York",
	},
	"2NC7": {
		ICAO:      "2NC7",
		Name:      "Scotland Neck East Airport",
		City:      "Scotland Neck",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 103,
		Latitude:  36.1679000854,
		Longitude: -77.3989028931,
		Timezone:  "America/New_York",
	},
	"2NC8": {
		ICAO:      "2NC8",
		Name:      "Goodnight's Airport",
		City:      "Kannapolis",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 740,
		Latitude:  35.5253982544,
		Longitude: -80.6398010254,
		Timezone:  "America/New_York",
	},
	"2ND0": {
		ICAO:      "2ND0",
		Name:      "Kraft Airport",
		City:      "Mapleton",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 910,
		Latitude:  46.8143997192,
		Longitude: -97.0497970581,
		Timezone:  "America/Chicago",
	},
	"2ND1": {
		ICAO:      "2ND1",
		Name:      "Westerlind Airport",
		City:      "Mercer",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1910,
		Latitude:  47.4925003052,
		Longitude: -100.6999969482,
		Timezone:  "America/Chicago",
	},
	"2ND2": {
		ICAO:      "2ND2",
		Name:      "Makeeff Airport",
		City:      "Mercer",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1910,
		Latitude:  47.5668983459,
		Longitude: -100.7210006714,
		Timezone:  "America/Chicago",
	},
	"2ND3": {
		ICAO:      "2ND3",
		Name:      "Casslindan Airport",
		City:      "Grand Forks",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 832,
		Latitude:  48.0008010864,
		Longitude: -97.1223983765,
		Timezone:  "America/Chicago",
	},
	"2ND7": {
		ICAO:      "2ND7",
		Name:      "Johnson Airport",
		City:      "Newburg",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1460,
		Latitude:  48.751701355,
		Longitude: -100.8140029907,
		Timezone:  "America/Chicago",
	},
	"2ND9": {
		ICAO:      "2ND9",
		Name:      "Brekhus Field",
		City:      "Norma",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1850,
		Latitude:  48.8166999817,
		Longitude: -101.9169998169,
		Timezone:  "America/Chicago",
	},
	"2NE0": {
		ICAO:      "2NE0",
		Name:      "Johnson Lake Airport",
		City:      "Elwood",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2625,
		Latitude:  40.6967010498,
		Longitude: -99.8342971802,
		Timezone:  "America/Chicago",
	},
	"2NE4": {
		ICAO:      "2NE4",
		Name:      "Spring Lake Airport",
		City:      "Gordon",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3815,
		Latitude:  42.4500007629,
		Longitude: -102.1340026855,
		Timezone:  "America/Denver",
	},
	"2NE5": {
		ICAO:      "2NE5",
		Name:      "Fiese Airstrip",
		City:      "Gothenburg",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2535,
		Latitude:  40.9000015259,
		Longitude: -100.0950012207,
		Timezone:  "America/Chicago",
	},
	"2NE6": {
		ICAO:      "2NE6",
		Name:      "Coppersmith Airport",
		City:      "Grainton",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3240,
		Latitude:  40.9207992554,
		Longitude: -101.2969970703,
		Timezone:  "America/Denver",
	},
	"2NE7": {
		ICAO:      "2NE7",
		Name:      "Kumor Airport",
		City:      "Grant",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3520,
		Latitude:  40.851398468,
		Longitude: -101.9430007935,
		Timezone:  "America/Denver",
	},
	"2NH5": {
		ICAO:      "2NH5",
		Name:      "Pilgrim's Home Airfield",
		City:      "Westmoreland",
		State:     "New Hampshire",
		Country:   "US",
		Elevation: 600,
		Latitude:  42.9902992249,
		Longitude: -72.4017028809,
		Timezone:  "America/New_York",
	},
	"2NJ1": {
		ICAO:      "2NJ1",
		Name:      "Jugtown Mountain Airport",
		City:      "Pattenburg",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 600,
		Latitude:  40.6333999634,
		Longitude: -75.0662994385,
		Timezone:  "America/New_York",
	},
	"2NJ3": {
		ICAO:      "2NJ3",
		Name:      "Weidel/Private/ Airport",
		City:      "Pennington",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 301,
		Latitude:  40.3404006958,
		Longitude: -74.8543014526,
		Timezone:  "America/New_York",
	},
	"2NJ5": {
		ICAO:      "2NJ5",
		Name:      "Hartung Airport",
		City:      "Phillipsburg",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 350,
		Latitude:  40.6520004272,
		Longitude: -75.1859970093,
		Timezone:  "America/New_York",
	},
	"2NJ6": {
		ICAO:      "2NJ6",
		Name:      "Markle Airport",
		City:      "Phillipsburg",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 320,
		Latitude:  40.7737007141,
		Longitude: -75.1593017578,
		Timezone:  "America/New_York",
	},
	"2NK1": {
		ICAO:      "2NK1",
		Name:      "Gaskin's Hilltop Airport",
		City:      "Canisteo",
		State:     "New York",
		Country:   "US",
		Elevation: 1730,
		Latitude:  42.2961997986,
		Longitude: -77.4654998779,
		Timezone:  "America/New_York",
	},
	"2NK3": {
		ICAO:      "2NK3",
		Name:      "Rose Field",
		City:      "Orient",
		State:     "New York",
		Country:   "US",
		Elevation: 20,
		Latitude:  41.1543006897,
		Longitude: -72.2800979614,
		Timezone:  "America/New_York",
	},
	"2NK7": {
		ICAO:      "2NK7",
		Name:      "Walton Airport",
		City:      "Walton",
		State:     "New York",
		Country:   "US",
		Elevation: 1200,
		Latitude:  42.1579017639,
		Longitude: -75.1479034424,
		Timezone:  "America/New_York",
	},
	"2NK9": {
		ICAO:      "2NK9",
		Name:      "Old Orchard Airpark",
		City:      "Modena",
		State:     "New York",
		Country:   "US",
		Elevation: 660,
		Latitude:  41.6571998596,
		Longitude: -74.0772018433,
		Timezone:  "America/New_York",
	},
	"2NV2": {
		ICAO:      "2NV2",
		Name:      "Gibb Ranch Airport",
		City:      "Sparks",
		State:     "Nevada",
		Country:   "US",
		Elevation: 4242,
		Latitude:  39.8347015381,
		Longitude: -119.6750030518,
		Timezone:  "America/Los_Angeles",
	},
	"2NY0": {
		ICAO:      "2NY0",
		Name:      "Catskill Valley Airpark",
		City:      "South Cairo",
		State:     "New York",
		Country:   "US",
		Elevation: 190,
		Latitude:  42.2806015015,
		Longitude: -73.9529037476,
		Timezone:  "America/New_York",
	},
	"2NY4": {
		ICAO:      "2NY4",
		Name:      "Sackett Farms Airstrip",
		City:      "Byron",
		State:     "New York",
		Country:   "US",
		Elevation: 665,
		Latitude:  43.0736999512,
		Longitude: -78.0500030518,
		Timezone:  "America/New_York",
	},
	"2NY7": {
		ICAO:      "2NY7",
		Name:      "Towner Farm Airport",
		City:      "Addison",
		State:     "New York",
		Country:   "US",
		Elevation: 1248,
		Latitude:  42.1214981079,
		Longitude: -77.2114028931,
		Timezone:  "America/New_York",
	},
	"2NY9": {
		ICAO:      "2NY9",
		Name:      "Kayutah Lake Airport",
		City:      "Alpine",
		State:     "New York",
		Country:   "US",
		Elevation: 1385,
		Latitude:  42.370098114,
		Longitude: -76.7225036621,
		Timezone:  "America/New_York",
	},
	"2OA1": {
		ICAO:      "2OA1",
		Name:      "Bristol Airstrip",
		City:      "Bristolville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 865,
		Latitude:  41.4151000977,
		Longitude: -80.8843002319,
		Timezone:  "America/New_York",
	},
	"2OA2": {
		ICAO:      "2OA2",
		Name:      "Mackie's Airport",
		City:      "Midland",
		State:     "Ohio",
		Country:   "US",
		Elevation: 993,
		Latitude:  39.2481002808,
		Longitude: -83.9150009155,
		Timezone:  "America/New_York",
	},
	"2OA4": {
		ICAO:      "2OA4",
		Name:      "Victory Field",
		City:      "Donnellsville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1055,
		Latitude:  39.9318008423,
		Longitude: -83.9075012207,
		Timezone:  "America/New_York",
	},
	"2OA5": {
		ICAO:      "2OA5",
		Name:      "Fl-Airfield",
		City:      "Marysville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 980,
		Latitude:  40.1824989319,
		Longitude: -83.3508987427,
		Timezone:  "America/New_York",
	},
	"2OA6": {
		ICAO:      "2OA6",
		Name:      "Hilltop Airport",
		City:      "Cambridge",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1000,
		Latitude:  40.0864982605,
		Longitude: -81.5882034302,
		Timezone:  "America/New_York",
	},
	"2OA7": {
		ICAO:      "2OA7",
		Name:      "Utter Field",
		City:      "Felicity",
		State:     "Ohio",
		Country:   "US",
		Elevation: 890,
		Latitude:  38.85139847,
		Longitude: -84.10559845,
		Timezone:  "America/New_York",
	},
	"2OG4": {
		ICAO:      "2OG4",
		Name:      "El Rancho Airport",
		City:      "Fields",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4130,
		Latitude:  42.330278,
		Longitude: -118.647222,
		Timezone:  "America/Los_Angeles",
	},
	"2OG5": {
		ICAO:      "2OG5",
		Name:      "Mendenhall Airstrip",
		City:      "Willamina",
		State:     "Oregon",
		Country:   "US",
		Elevation: 350,
		Latitude:  45.117978,
		Longitude: -123.506458,
		Timezone:  "America/Los_Angeles",
	},
	"2OH0": {
		ICAO:      "2OH0",
		Name:      "Bohannan Airport",
		City:      "Kilbourne",
		State:     "Ohio",
		Country:   "US",
		Elevation: 960,
		Latitude:  40.345500946,
		Longitude: -82.9404983521,
		Timezone:  "America/New_York",
	},
	"2OH2": {
		ICAO:      "2OH2",
		Name:      "Merritt Airport",
		City:      "Staunton",
		State:     "Ohio",
		Country:   "US",
		Elevation: 990,
		Latitude:  39.4720001221,
		Longitude: -83.4898986816,
		Timezone:  "America/New_York",
	},
	"2OH4": {
		ICAO:      "2OH4",
		Name:      "Cedar Creek Airport",
		City:      "Shauck",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1220,
		Latitude:  40.6245002747,
		Longitude: -82.6354980469,
		Timezone:  "America/New_York",
	},
	"2OH6": {
		ICAO:      "2OH6",
		Name:      "Leavelle Airstrip",
		City:      "Troy",
		State:     "Ohio",
		Country:   "US",
		Elevation: 935,
		Latitude:  40.0120010376,
		Longitude: -84.2813034058,
		Timezone:  "America/New_York",
	},
	"2OH7": {
		ICAO:      "2OH7",
		Name:      "Jbr Airport",
		City:      "Hamersville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 945,
		Latitude:  38.9112014771,
		Longitude: -84.0260009766,
		Timezone:  "America/New_York",
	},
	"2OH8": {
		ICAO:      "2OH8",
		Name:      "Port-O-John Airport",
		City:      "Derby",
		State:     "Ohio",
		Country:   "US",
		Elevation: 925,
		Latitude:  39.79529953,
		Longitude: -83.223197937,
		Timezone:  "America/New_York",
	},
	"2OH9": {
		ICAO:      "2OH9",
		Name:      "Caesar Creek Gliderport",
		City:      "Waynesville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 940,
		Latitude:  39.4766998291,
		Longitude: -84.0938034058,
		Timezone:  "America/New_York",
	},
	"2OI2": {
		ICAO:      "2OI2",
		Name:      "Air Jordan Airport",
		City:      "Waynesville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 920,
		Latitude:  39.5611000061,
		Longitude: -84.051902771,
		Timezone:  "America/New_York",
	},
	"2OI4": {
		ICAO:      "2OI4",
		Name:      "Aero Lake Farm Airport",
		City:      "Westfield Center",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1195,
		Latitude:  41.04529953,
		Longitude: -81.9365005493,
		Timezone:  "America/New_York",
	},
	"2OI8": {
		ICAO:      "2OI8",
		Name:      "K & D Airways Airport",
		City:      "Mansfield",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1248,
		Latitude:  40.863899231,
		Longitude: -82.4206008911,
		Timezone:  "America/New_York",
	},
	"2OK0": {
		ICAO:      "2OK0",
		Name:      "Burford Corp. Airport",
		City:      "Maysville",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 945,
		Latitude:  34.8320007324,
		Longitude: -97.4100036621,
		Timezone:  "America/Chicago",
	},
	"2OK1": {
		ICAO:      "2OK1",
		Name:      "Frost Ranch Airport",
		City:      "Meeker",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1030,
		Latitude:  35.5900993347,
		Longitude: -97.010597229,
		Timezone:  "America/Chicago",
	},
	"2OK2": {
		ICAO:      "2OK2",
		Name:      "Twin Lakes Airport",
		City:      "Midwest City",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1199,
		Latitude:  35.3292007446,
		Longitude: -97.2697982788,
		Timezone:  "America/Chicago",
	},
	"2OK4": {
		ICAO:      "2OK4",
		Name:      "Ragwing Acres Airport",
		City:      "Mounds",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 780,
		Latitude:  35.8167991638,
		Longitude: -96.0586013794,
		Timezone:  "America/Chicago",
	},
	"2OK5": {
		ICAO:      "2OK5",
		Name:      "Ferrell Ranch Airport",
		City:      "Mountain View",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1575,
		Latitude:  34.9290008545,
		Longitude: -98.7397994995,
		Timezone:  "America/Chicago",
	},
	"2OK6": {
		ICAO:      "2OK6",
		Name:      "Flying N Ranch Airport",
		City:      "Meeker",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1095,
		Latitude:  35.4819984436,
		Longitude: -97.0205993652,
		Timezone:  "America/Chicago",
	},
	"2OK7": {
		ICAO:      "2OK7",
		Name:      "Cole Landing Area Airport",
		City:      "Newcastle",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1353,
		Latitude:  35.2583999634,
		Longitude: -97.6628036499,
		Timezone:  "America/Chicago",
	},
	"2OK8": {
		ICAO:      "2OK8",
		Name:      "D & G Farms Airport",
		City:      "Newcastle",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1300,
		Latitude:  35.2251014709,
		Longitude: -97.6461029053,
		Timezone:  "America/Chicago",
	},
	"2OK9": {
		ICAO:      "2OK9",
		Name:      "Pata Skyhaven Airport",
		City:      "Norman",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1136,
		Latitude:  35.1958999634,
		Longitude: -97.3084030151,
		Timezone:  "America/Chicago",
	},
	"2OL2": {
		ICAO:      "2OL2",
		Name:      "Myers Field Airstrip",
		City:      "Bixby",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 665,
		Latitude:  35.887833,
		Longitude: -95.919639,
		Timezone:  "America/Chicago",
	},
	"2OR0": {
		ICAO:      "2OR0",
		Name:      "Nielsen Airport",
		City:      "Oregon City",
		State:     "Oregon",
		Country:   "US",
		Elevation: 510,
		Latitude:  45.3442993164,
		Longitude: -122.5179977417,
		Timezone:  "America/Los_Angeles",
	},
	"2OR1": {
		ICAO:      "2OR1",
		Name:      "Big Muddy Ranch Airport",
		City:      "Antelope",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1641,
		Latitude:  44.8284988403,
		Longitude: -120.4950027466,
		Timezone:  "America/Los_Angeles",
	},
	"2OR3": {
		ICAO:      "2OR3",
		Name:      "Davidson Field",
		City:      "Jefferson",
		State:     "Oregon",
		Country:   "US",
		Elevation: 188,
		Latitude:  44.7946014404,
		Longitude: -123.0979995728,
		Timezone:  "America/Los_Angeles",
	},
	"2OR4": {
		ICAO:      "2OR4",
		Name:      "Heavens Gate Ranch Airport",
		City:      "Oakland",
		State:     "Oregon",
		Country:   "US",
		Elevation: 965,
		Latitude:  43.4748001099,
		Longitude: -123.3460006714,
		Timezone:  "America/Los_Angeles",
	},
	"2OR6": {
		ICAO:      "2OR6",
		Name:      "Lockhart Airport",
		City:      "Ironside",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4615,
		Latitude:  44.1813011169,
		Longitude: -118.2149963379,
		Timezone:  "America/Boise",
	},
	"2OR7": {
		ICAO:      "2OR7",
		Name:      "Sweet Home Airport",
		City:      "Sweet Home",
		State:     "Oregon",
		Country:   "US",
		Elevation: 645,
		Latitude:  44.3992996216,
		Longitude: -122.6849975586,
		Timezone:  "America/Los_Angeles",
	},
	"2PA0": {
		ICAO:      "2PA0",
		Name:      "Zettlemoyer Airport",
		City:      "Kutztown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 550,
		Latitude:  40.55680084,
		Longitude: -75.69799805,
		Timezone:  "America/New_York",
	},
	"2PA1": {
		ICAO:      "2PA1",
		Name:      "Boehm's Field",
		City:      "Lackawaxen",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1330,
		Latitude:  41.4430999756,
		Longitude: -75.0078964233,
		Timezone:  "America/New_York",
	},
	"2PA2": {
		ICAO:      "2PA2",
		Name:      "Lehman Airport",
		City:      "Dallas",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1300,
		Latitude:  41.3194999695,
		Longitude: -76.0444030762,
		Timezone:  "America/New_York",
	},
	"2PA3": {
		ICAO:      "2PA3",
		Name:      "Reed Airport",
		City:      "Lake Ariel",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1400,
		Latitude:  41.5000991821,
		Longitude: -75.4162979126,
		Timezone:  "America/New_York",
	},
	"2PA4": {
		ICAO:      "2PA4",
		Name:      "Boden Airport",
		City:      "Lakewood",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 2120,
		Latitude:  41.846698761,
		Longitude: -75.4462966919,
		Timezone:  "America/New_York",
	},
	"2PA5": {
		ICAO:      "2PA5",
		Name:      "Johnson Airport",
		City:      "Port Allegany",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1506,
		Latitude:  41.7806015015,
		Longitude: -78.269203186,
		Timezone:  "America/New_York",
	},
	"2PA6": {
		ICAO:      "2PA6",
		Name:      "The Old Commonwealth Aerodrome",
		City:      "Shartlesville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 540,
		Latitude:  40.4911994934,
		Longitude: -76.0977020264,
		Timezone:  "America/New_York",
	},
	"2PA7": {
		ICAO:      "2PA7",
		Name:      "Egolf Airport",
		City:      "Landisburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 610,
		Latitude:  40.3334007263,
		Longitude: -77.299697876,
		Timezone:  "America/New_York",
	},
	"2PA8": {
		ICAO:      "2PA8",
		Name:      "Shulls Airport",
		City:      "Landisburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 640,
		Latitude:  40.3000984192,
		Longitude: -77.299697876,
		Timezone:  "America/New_York",
	},
	"2PN0": {
		ICAO:      "2PN0",
		Name:      "Strawberry Acres Airport",
		City:      "Wattsburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1300,
		Latitude:  42.0153007507,
		Longitude: -79.8178024292,
		Timezone:  "America/New_York",
	},
	"2PN1": {
		ICAO:      "2PN1",
		Name:      "Malco Airport",
		City:      "Westfield",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1990,
		Latitude:  41.9833984375,
		Longitude: -77.5246963501,
		Timezone:  "America/New_York",
	},
	"2PN3": {
		ICAO:      "2PN3",
		Name:      "Market Garden Airport",
		City:      "York Springs",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 600,
		Latitude:  39.961101532,
		Longitude: -77.1003036499,
		Timezone:  "America/New_York",
	},
	"2PN4": {
		ICAO:      "2PN4",
		Name:      "Quemahening Flightpark Ultralightport",
		City:      "Stoystown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1997,
		Latitude:  40.0817985535,
		Longitude: -78.9455032349,
		Timezone:  "America/New_York",
	},
	"2PN5": {
		ICAO:      "2PN5",
		Name:      "Baker-Sell Airport",
		City:      "Woodbury",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1371,
		Latitude:  40.2484016418,
		Longitude: -78.3608016968,
		Timezone:  "America/New_York",
	},
	"2PN7": {
		ICAO:      "2PN7",
		Name:      "Brennan Personal Use Airport",
		City:      "Zelienople",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1210,
		Latitude:  40.8555984497,
		Longitude: -80.070602417,
		Timezone:  "America/New_York",
	},
	"2PS3": {
		ICAO:      "2PS3",
		Name:      "Mathna Airport",
		City:      "York Springs",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 490,
		Latitude:  40.0167999268,
		Longitude: -77.0413970947,
		Timezone:  "America/New_York",
	},
	"2SC5": {
		ICAO:      "2SC5",
		Name:      "Ridgewood Air Airport",
		City:      "Waterloo",
		State:     "South Carolina",
		Country:   "US",
		Elevation: 560,
		Latitude:  34.2916984558,
		Longitude: -82.0539016724,
		Timezone:  "America/New_York",
	},
	"2SC7": {
		ICAO:      "2SC7",
		Name:      "Laurel Hill Farms Airport",
		City:      "Mcclellanville",
		State:     "South Carolina",
		Country:   "US",
		Elevation: 8,
		Latitude:  33.051700592,
		Longitude: -79.5413970947,
		Timezone:  "America/New_York",
	},
	"2SC8": {
		ICAO:      "2SC8",
		Name:      "Gaston Airport",
		City:      "Gaston",
		State:     "South Carolina",
		Country:   "US",
		Elevation: 460,
		Latitude:  33.7868003845,
		Longitude: -81.0948028564,
		Timezone:  "America/New_York",
	},
	"2SC9": {
		ICAO:      "2SC9",
		Name:      "Mcintosh Airport",
		City:      "Kingstree",
		State:     "South Carolina",
		Country:   "US",
		Elevation: 76,
		Latitude:  33.750278,
		Longitude: -79.957778,
		Timezone:  "America/New_York",
	},
	"2SD0": {
		ICAO:      "2SD0",
		Name:      "Paradise Valley Airport",
		City:      "Nemo",
		State:     "South Dakota",
		Country:   "US",
		Elevation: 4500,
		Latitude:  44.1636009216,
		Longitude: -103.4789962769,
		Timezone:  "America/Denver",
	},
	"2SD2": {
		ICAO:      "2SD2",
		Name:      "Pepper Port Airport",
		City:      "Bonesteel",
		State:     "South Dakota",
		Country:   "US",
		Elevation: 1965,
		Latitude:  43.1058006287,
		Longitude: -98.9303970337,
		Timezone:  "America/Chicago",
	},
	"2SD3": {
		ICAO:      "2SD3",
		Name:      "Bollweg Farm Airport",
		City:      "Harrold",
		State:     "South Dakota",
		Country:   "US",
		Elevation: 1746,
		Latitude:  44.5633010864,
		Longitude: -99.7250976563,
		Timezone:  "America/Chicago",
	},
	"2SD4": {
		ICAO:      "2SD4",
		Name:      "Nicolaisen Airport",
		City:      "Mount Vernon",
		State:     "South Dakota",
		Country:   "US",
		Elevation: 1440,
		Latitude:  43.6833000183,
		Longitude: -98.265296936,
		Timezone:  "America/Chicago",
	},
	"2SD7": {
		ICAO:      "2SD7",
		Name:      "Dangel Airport",
		City:      "Hurley",
		State:     "South Dakota",
		Country:   "US",
		Elevation: 1284,
		Latitude:  43.3333015442,
		Longitude: -97.0627975464,
		Timezone:  "America/Chicago",
	},
	"2SD8": {
		ICAO:      "2SD8",
		Name:      "Bixler Ridge Airport",
		City:      "Centerville",
		State:     "South Dakota",
		Country:   "US",
		Elevation: 1250,
		Latitude:  43.0456008911,
		Longitude: -96.9391021729,
		Timezone:  "America/Chicago",
	},
	"2SD9": {
		ICAO:      "2SD9",
		Name:      "Tc Field",
		City:      "Ipswich",
		State:     "South Dakota",
		Country:   "US",
		Elevation: 1515,
		Latitude:  45.4522018433,
		Longitude: -99.0101013184,
		Timezone:  "America/Chicago",
	},
	"2TA1": {
		ICAO:      "2TA1",
		Name:      "Gravco STOLport",
		City:      "Lufkin",
		State:     "Texas",
		Country:   "US",
		Elevation: 284,
		Latitude:  31.3901996613,
		Longitude: -94.6910018921,
		Timezone:  "America/Chicago",
	},
	"2TA3": {
		ICAO:      "2TA3",
		Name:      "Triangle Ranch Private Airport",
		City:      "Pine Springs",
		State:     "Texas",
		Country:   "US",
		Elevation: 4400,
		Latitude:  31.7182006836,
		Longitude: -104.5979995728,
		Timezone:  "America/Chicago",
	},
	"2TA4": {
		ICAO:      "2TA4",
		Name:      "Mario's Flying Pizza Airport",
		City:      "Sealy",
		State:     "Texas",
		Country:   "US",
		Elevation: 233,
		Latitude:  29.7476997375,
		Longitude: -96.2927017212,
		Timezone:  "America/Chicago",
	},
	"2TA6": {
		ICAO:      "2TA6",
		Name:      "Pyramid Ranch Airport",
		City:      "Fairfield",
		State:     "Texas",
		Country:   "US",
		Elevation: 362,
		Latitude:  31.8626995087,
		Longitude: -96.1975021362,
		Timezone:  "America/Chicago",
	},
	"2TA8": {
		ICAO:      "2TA8",
		Name:      "El Coyote Ranch Airport",
		City:      "Encino",
		State:     "Texas",
		Country:   "US",
		Elevation: 196,
		Latitude:  26.85840034,
		Longitude: -98.22200012,
		Timezone:  "America/Chicago",
	},
	"2TA9": {
		ICAO:      "2TA9",
		Name:      "A M I G O For Christ Airport",
		City:      "Weatherford",
		State:     "Texas",
		Country:   "US",
		Elevation: 840,
		Latitude:  32.6604003906,
		Longitude: -97.8420028687,
		Timezone:  "America/Chicago",
	},
	"2TE0": {
		ICAO:      "2TE0",
		Name:      "Eagle Air Park",
		City:      "Brazoria",
		State:     "Texas",
		Country:   "US",
		Elevation: 15,
		Latitude:  28.9822006226,
		Longitude: -95.5796966553,
		Timezone:  "America/Chicago",
	},
	"2TE2": {
		ICAO:      "2TE2",
		Name:      "Flying Oaks Airport",
		City:      "Fort Worth",
		State:     "Texas",
		Country:   "US",
		Elevation: 720,
		Latitude:  32.8306999207,
		Longitude: -97.5335998535,
		Timezone:  "America/Chicago",
	},
	"2TE3": {
		ICAO:      "2TE3",
		Name:      "Weems Farm Airport",
		City:      "Aubrey",
		State:     "Texas",
		Country:   "US",
		Elevation: 590,
		Latitude:  33.2942008972,
		Longitude: -96.9281005859,
		Timezone:  "America/Chicago",
	},
	"2TE4": {
		ICAO:      "2TE4",
		Name:      "Frels Airport",
		City:      "El Campo",
		State:     "Texas",
		Country:   "US",
		Elevation: 100,
		Latitude:  29.1791992188,
		Longitude: -96.3227005005,
		Timezone:  "America/Chicago",
	},
	"2TE5": {
		ICAO:      "2TE5",
		Name:      "Piano Ranch Airport",
		City:      "Flatonia",
		State:     "Texas",
		Country:   "US",
		Elevation: 383,
		Latitude:  29.6951007843,
		Longitude: -97.1632995605,
		Timezone:  "America/Chicago",
	},
	"2TE6": {
		ICAO:      "2TE6",
		Name:      "Burris Ranch Airport",
		City:      "Karnes City",
		State:     "Texas",
		Country:   "US",
		Elevation: 448,
		Latitude:  28.8878002167,
		Longitude: -98.0503005981,
		Timezone:  "America/Chicago",
	},
	"2TE7": {
		ICAO:      "2TE7",
		Name:      "Beach Ranch Airport",
		City:      "Post",
		State:     "Texas",
		Country:   "US",
		Elevation: 2303,
		Latitude:  33.228167,
		Longitude: -101.13535,
		Timezone:  "America/Chicago",
	},
	"2TE8": {
		ICAO:      "2TE8",
		Name:      "Wagner-Braxdale Airport",
		City:      "Crystal City",
		State:     "Texas",
		Country:   "US",
		Elevation: 570,
		Latitude:  28.6783008575,
		Longitude: -99.7986984253,
		Timezone:  "America/Chicago",
	},
	"2TN2": {
		ICAO:      "2TN2",
		Name:      "Wings Field",
		City:      "Hickory Valley",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 540,
		Latitude:  35.2235984802,
		Longitude: -89.1889038086,
		Timezone:  "America/Chicago",
	},
	"2TN4": {
		ICAO:      "2TN4",
		Name:      "Shoemaker-Shelby Forest Airport",
		City:      "Millington",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 335,
		Latitude:  35.3180999756,
		Longitude: -90.0353012085,
		Timezone:  "America/Chicago",
	},
	"2TN7": {
		ICAO:      "2TN7",
		Name:      "Wolf Creek Airport",
		City:      "Kingston",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 890,
		Latitude:  35.8044013977,
		Longitude: -84.415802002,
		Timezone:  "America/New_York",
	},
	"2TN8": {
		ICAO:      "2TN8",
		Name:      "Soggy Bottom Airport",
		City:      "Dowelltown",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 580,
		Latitude:  36.0180015564,
		Longitude: -85.9359970093,
		Timezone:  "America/Chicago",
	},
	"2TS0": {
		ICAO:      "2TS0",
		Name:      "Myska Field",
		City:      "Denton",
		State:     "Texas",
		Country:   "US",
		Elevation: 595,
		Latitude:  33.1557006836,
		Longitude: -97.1380996704,
		Timezone:  "America/Chicago",
	},
	"2TS2": {
		ICAO:      "2TS2",
		Name:      "Shofner Farms Airport",
		City:      "Harlingen",
		State:     "Texas",
		Country:   "US",
		Elevation: 57,
		Latitude:  26.1683998108,
		Longitude: -97.7335968018,
		Timezone:  "America/Chicago",
	},
	"2TS3": {
		ICAO:      "2TS3",
		Name:      "Tigerbird Field",
		City:      "Waco",
		State:     "Texas",
		Country:   "US",
		Elevation: 450,
		Latitude:  31.6793003082,
		Longitude: -97.2406005859,
		Timezone:  "America/Chicago",
	},
	"2TS4": {
		ICAO:      "2TS4",
		Name:      "Circle R Ranch Airport",
		City:      "Malakoff",
		State:     "Texas",
		Country:   "US",
		Elevation: 395,
		Latitude:  32.1397018433,
		Longitude: -95.9702987671,
		Timezone:  "America/Chicago",
	},
	"2TS5": {
		ICAO:      "2TS5",
		Name:      "Hedley Airport",
		City:      "Hedley",
		State:     "Texas",
		Country:   "US",
		Elevation: 2510,
		Latitude:  34.893699646,
		Longitude: -100.6100006104,
		Timezone:  "America/Chicago",
	},
	"2TS6": {
		ICAO:      "2TS6",
		Name:      "Eagle's Nest Estates Airport",
		City:      "Midlothian",
		State:     "Texas",
		Country:   "US",
		Elevation: 780,
		Latitude:  32.5125999451,
		Longitude: -96.9266967773,
		Timezone:  "America/Chicago",
	},
	"2TS8": {
		ICAO:      "2TS8",
		Name:      "Bailey Airport",
		City:      "Stockdale",
		State:     "Texas",
		Country:   "US",
		Elevation: 452,
		Latitude:  29.220500946,
		Longitude: -97.8716964722,
		Timezone:  "America/Chicago",
	},
	"2TX0": {
		ICAO:      "2TX0",
		Name:      "Blue Sky Airfield",
		City:      "Amarillo",
		State:     "Texas",
		Country:   "US",
		Elevation: 3615,
		Latitude:  35.0644989014,
		Longitude: -101.8489990234,
		Timezone:  "America/Chicago",
	},
	"2TX1": {
		ICAO:      "2TX1",
		Name:      "Russell Paradise Airport",
		City:      "Elmendorf",
		State:     "Texas",
		Country:   "US",
		Elevation: 535,
		Latitude:  29.2252006531,
		Longitude: -98.3835983276,
		Timezone:  "America/Chicago",
	},
	"2TX2": {
		ICAO:      "2TX2",
		Name:      "Ray Smith Farm Airport",
		City:      "Hico",
		State:     "Texas",
		Country:   "US",
		Elevation: 1050,
		Latitude:  32.0265007019,
		Longitude: -98.0100021362,
		Timezone:  "America/Chicago",
	},
	"2TX3": {
		ICAO:      "2TX3",
		Name:      "La Fonda Ranch Airport",
		City:      "Brackettville",
		State:     "Texas",
		Country:   "US",
		Elevation: 1020,
		Latitude:  29.2168998718,
		Longitude: -100.6169967651,
		Timezone:  "America/Chicago",
	},
	"2TX4": {
		ICAO:      "2TX4",
		Name:      "Lewis Ranch Airport",
		City:      "Encinal",
		State:     "Texas",
		Country:   "US",
		Elevation: 540,
		Latitude:  27.9836006165,
		Longitude: -99.376701355,
		Timezone:  "America/Chicago",
	},
	"2TX5": {
		ICAO:      "2TX5",
		Name:      "Berry Airport",
		City:      "Gatesville",
		State:     "Texas",
		Country:   "US",
		Elevation: 757,
		Latitude:  31.4535007477,
		Longitude: -97.7514038086,
		Timezone:  "America/Chicago",
	},
	"2TX6": {
		ICAO:      "2TX6",
		Name:      "Everitt Airport",
		City:      "Idalou",
		State:     "Texas",
		Country:   "US",
		Elevation: 3200,
		Latitude:  33.6818008423,
		Longitude: -101.6679992676,
		Timezone:  "America/Chicago",
	},
	"2TX7": {
		ICAO:      "2TX7",
		Name:      "Jw Airport",
		City:      "Justin",
		State:     "Texas",
		Country:   "US",
		Elevation: 875,
		Latitude:  33.0946006775,
		Longitude: -97.3961029053,
		Timezone:  "America/Chicago",
	},
	"2TX8": {
		ICAO:      "2TX8",
		Name:      "Eagle's Landing Airport",
		City:      "Krum",
		State:     "Texas",
		Country:   "US",
		Elevation: 930,
		Latitude:  33.3082008362,
		Longitude: -97.379699707,
		Timezone:  "America/Chicago",
	},
	"2UT2": {
		ICAO:      "2UT2",
		Name:      "High Meadow Ranch Airport",
		City:      "Duck Creek Village",
		State:     "Utah",
		Country:   "US",
		Elevation: 8280,
		Latitude:  37.5208015442,
		Longitude: -112.641998291,
		Timezone:  "America/Denver",
	},
	"2UT3": {
		ICAO:      "2UT3",
		Name:      "Fort Ranch Airport",
		City:      "Brigham City",
		State:     "Utah",
		Country:   "US",
		Elevation: 4440,
		Latitude:  41.4921989441,
		Longitude: -112.591003418,
		Timezone:  "America/Denver",
	},
	"2VA0": {
		ICAO:      "2VA0",
		Name:      "Red Birds Airyard Airport",
		City:      "Moneta",
		State:     "Virginia",
		Country:   "US",
		Elevation: 940,
		Latitude:  37.1125984192,
		Longitude: -79.6091995239,
		Timezone:  "America/New_York",
	},
	"2VA1": {
		ICAO:      "2VA1",
		Name:      "Jett Airpark",
		City:      "Edwardsville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 90,
		Latitude:  37.8959999084,
		Longitude: -76.3552017212,
		Timezone:  "America/New_York",
	},
	"2VA2": {
		ICAO:      "2VA2",
		Name:      "Melville Airstrip",
		City:      "Surry",
		State:     "Virginia",
		Country:   "US",
		Elevation: 90,
		Latitude:  37.1459999084,
		Longitude: -76.7900009155,
		Timezone:  "America/New_York",
	},
	"2VA3": {
		ICAO:      "2VA3",
		Name:      "Merlin Aerodrome",
		City:      "Amelia Court House",
		State:     "Virginia",
		Country:   "US",
		Elevation: 235,
		Latitude:  37.3149986267,
		Longitude: -77.8660964966,
		Timezone:  "America/New_York",
	},
	"2VA5": {
		ICAO:      "2VA5",
		Name:      "Rosegill Farm Airstrip",
		City:      "Urbanna",
		State:     "Virginia",
		Country:   "US",
		Elevation: 32,
		Latitude:  37.6306991577,
		Longitude: -76.5650024414,
		Timezone:  "America/New_York",
	},
	"2VA6": {
		ICAO:      "2VA6",
		Name:      "Arrowhead Point Airport",
		City:      "Saluda",
		State:     "Virginia",
		Country:   "US",
		Elevation: 85,
		Latitude:  37.6237983704,
		Longitude: -76.6010971069,
		Timezone:  "America/New_York",
	},
	"2VA8": {
		ICAO:      "2VA8",
		Name:      "Brandywyne Farms Airport",
		City:      "Holland",
		State:     "Virginia",
		Country:   "US",
		Elevation: 66,
		Latitude:  36.6837997437,
		Longitude: -76.8300018311,
		Timezone:  "America/New_York",
	},
	"2VA9": {
		ICAO:      "2VA9",
		Name:      "Airlie Airport",
		City:      "Warrenton",
		State:     "Virginia",
		Country:   "US",
		Elevation: 522,
		Latitude:  38.757900238,
		Longitude: -77.7872009277,
		Timezone:  "America/New_York",
	},
	"2VG2": {
		ICAO:      "2VG2",
		Name:      "Upperville Airport",
		City:      "Upperville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 551,
		Latitude:  38.9718017578,
		Longitude: -77.8696975708,
		Timezone:  "America/New_York",
	},
	"2VG3": {
		ICAO:      "2VG3",
		Name:      "Cub Haven Airport",
		City:      "Harrisonburg",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1350,
		Latitude:  38.5485992432,
		Longitude: -78.8711013794,
		Timezone:  "America/New_York",
	},
	"2VG7": {
		ICAO:      "2VG7",
		Name:      "Seven Gables Airport",
		City:      "Orange",
		State:     "Virginia",
		Country:   "US",
		Elevation: 411,
		Latitude:  38.163898468,
		Longitude: -77.8499984741,
		Timezone:  "America/New_York",
	},
	"2VG8": {
		ICAO:      "2VG8",
		Name:      "Folly Neck Airport",
		City:      "Warsaw",
		State:     "Virginia",
		Country:   "US",
		Elevation: 12,
		Latitude:  37.8722000122,
		Longitude: -76.7311019897,
		Timezone:  "America/New_York",
	},
	"2WA1": {
		ICAO:      "2WA1",
		Name:      "Diamond Point Airstrip",
		City:      "Sequim",
		State:     "Washington",
		Country:   "US",
		Elevation: 262,
		Latitude:  48.0926017761,
		Longitude: -122.9290008545,
		Timezone:  "America/Los_Angeles",
	},
	"2WA3": {
		ICAO:      "2WA3",
		Name:      "Stuart Island West Airport",
		City:      "Friday Harbor",
		State:     "Washington",
		Country:   "US",
		Elevation: 200,
		Latitude:  48.6845016479,
		Longitude: -123.2099990845,
		Timezone:  "America/Los_Angeles",
	},
	"2WA6": {
		ICAO:      "2WA6",
		Name:      "Rice Ranch Airport",
		City:      "Grand Coulee",
		State:     "Washington",
		Country:   "US",
		Elevation: 2434,
		Latitude:  47.9981994629,
		Longitude: -119.0889968872,
		Timezone:  "America/Los_Angeles",
	},
	"2WA8": {
		ICAO:      "2WA8",
		Name:      "Kramer Ranch Airport",
		City:      "Harrington",
		State:     "Washington",
		Country:   "US",
		Elevation: 2181,
		Latitude:  47.3816986084,
		Longitude: -118.2969970703,
		Timezone:  "America/Los_Angeles",
	},
	"2WA9": {
		ICAO:      "2WA9",
		Name:      "Touchet Valley Airport",
		City:      "Huntsville",
		State:     "Washington",
		Country:   "US",
		Elevation: 1350,
		Latitude:  46.2891998291,
		Longitude: -118.1009979248,
		Timezone:  "America/Los_Angeles",
	},
	"2WI0": {
		ICAO:      "2WI0",
		Name:      "Bender's Airport",
		City:      "Rothschild",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1197,
		Latitude:  44.8526992798,
		Longitude: -89.6000976563,
		Timezone:  "America/Chicago",
	},
	"2WI1": {
		ICAO:      "2WI1",
		Name:      "Uff-Da Airport",
		City:      "Stoughton",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 945,
		Latitude:  42.9435997009,
		Longitude: -89.2865982056,
		Timezone:  "America/Chicago",
	},
	"2WI2": {
		ICAO:      "2WI2",
		Name:      "Shullsburg Airport",
		City:      "Shullsburg",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1080,
		Latitude:  42.5569000244,
		Longitude: -90.2276000977,
		Timezone:  "America/Chicago",
	},
	"2WI4": {
		ICAO:      "2WI4",
		Name:      "Plows & Props Airport",
		City:      "Springfield",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 840,
		Latitude:  42.6427993774,
		Longitude: -88.3986968994,
		Timezone:  "America/Chicago",
	},
	"2WI5": {
		ICAO:      "2WI5",
		Name:      "Blair Lake Airport",
		City:      "Mercer",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1625,
		Latitude:  46.0974006653,
		Longitude: -90.1632003784,
		Timezone:  "America/Chicago",
	},
	"2WI6": {
		ICAO:      "2WI6",
		Name:      "Matson Airport",
		City:      "Stoughton",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 915,
		Latitude:  42.9136009216,
		Longitude: -89.1862030029,
		Timezone:  "America/Chicago",
	},
	"2WI7": {
		ICAO:      "2WI7",
		Name:      "Hecklers' Strip",
		City:      "Mount Vernon",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1130,
		Latitude:  42.9696998596,
		Longitude: -89.665397644,
		Timezone:  "America/Chicago",
	},
	"2WI8": {
		ICAO:      "2WI8",
		Name:      "Davies Airport",
		City:      "Oostburg",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 645,
		Latitude:  43.5974998474,
		Longitude: -87.7789993286,
		Timezone:  "America/Chicago",
	},
	"2WI9": {
		ICAO:      "2WI9",
		Name:      "Bulldog Ranch Airport",
		City:      "Oxford",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 955,
		Latitude:  43.7599983215,
		Longitude: -89.634803772,
		Timezone:  "America/Chicago",
	},
	"2WN2": {
		ICAO:      "2WN2",
		Name:      "Gaffney Airport",
		City:      "Briggsville",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 827,
		Latitude:  43.6542015076,
		Longitude: -89.6303024292,
		Timezone:  "America/Chicago",
	},
	"2WN3": {
		ICAO:      "2WN3",
		Name:      "Curns Airport",
		City:      "New London",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 790,
		Latitude:  44.3578987122,
		Longitude: -88.7613983154,
		Timezone:  "America/Chicago",
	},
	"2WN4": {
		ICAO:      "2WN4",
		Name:      "Mcfaul Airport",
		City:      "New London",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 780,
		Latitude:  44.3394012451,
		Longitude: -88.7811965942,
		Timezone:  "America/Chicago",
	},
	"2WN5": {
		ICAO:      "2WN5",
		Name:      "Murmuring Springs Airport",
		City:      "Necedah",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 910,
		Latitude:  44.006401062,
		Longitude: -90.019203186,
		Timezone:  "America/Chicago",
	},
	"2WN6": {
		ICAO:      "2WN6",
		Name:      "Cunningham Airport",
		City:      "Granton",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1089,
		Latitude:  44.5297012329,
		Longitude: -90.4192962646,
		Timezone:  "America/Chicago",
	},
	"2WN7": {
		ICAO:      "2WN7",
		Name:      "Planeacres Airport",
		City:      "Fisk",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 844,
		Latitude:  43.9486999512,
		Longitude: -88.6732025146,
		Timezone:  "America/Chicago",
	},
	"2WN8": {
		ICAO:      "2WN8",
		Name:      "Oshkosh Sky Ranch Airport",
		City:      "Omro",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 770,
		Latitude:  44.015228,
		Longitude: -88.714269,
		Timezone:  "America/Chicago",
	},
	"2WN9": {
		ICAO:      "2WN9",
		Name:      "Hillcrest Airport",
		City:      "Goldendale",
		State:     "Washington",
		Country:   "US",
		Elevation: 2405,
		Latitude:  45.8451004028,
		Longitude: -120.7009963989,
		Timezone:  "America/Los_Angeles",
	},
	"2WS2": {
		ICAO:      "2WS2",
		Name:      "Connor'S Lake Landing",
		City:      "Webster",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1000,
		Latitude:  45.913686,
		Longitude: -92.293239,
		Timezone:  "America/Chicago",
	},
	"2WS3": {
		ICAO:      "2WS3",
		Name:      "C. R. Acres Airport",
		City:      "Greenleaf",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 800,
		Latitude:  44.280428,
		Longitude: -88.093578,
		Timezone:  "America/Chicago",
	},
	"2WV3": {
		ICAO:      "2WV3",
		Name:      "Hales Landing Airport",
		City:      "Elizabeth",
		State:     "West Virginia",
		Country:   "US",
		Elevation: 650,
		Latitude:  39.1171989441,
		Longitude: -81.3917007446,
		Timezone:  "America/New_York",
	},
	"2WV5": {
		ICAO:      "2WV5",
		Name:      "Willow Bend Airport",
		City:      "Union",
		State:     "West Virginia",
		Country:   "US",
		Elevation: 2150,
		Latitude:  37.5466995239,
		Longitude: -80.5183029175,
		Timezone:  "America/New_York",
	},
	"2WY3": {
		ICAO:      "2WY3",
		Name:      "Haas Airport",
		City:      "Merna",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 7980,
		Latitude:  43.0071983337,
		Longitude: -110.3170013428,
		Timezone:  "America/Denver",
	},
	"2WY8": {
		ICAO:      "2WY8",
		Name:      "Bar Flying E Airport",
		City:      "Cody",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 5550,
		Latitude:  44.4043998718,
		Longitude: -109.2819976807,
		Timezone:  "America/Denver",
	},
	"2XA0": {
		ICAO:      "2XA0",
		Name:      "Foard County Airport",
		City:      "Crowell",
		State:     "Texas",
		Country:   "US",
		Elevation: 1479,
		Latitude:  33.9793014526,
		Longitude: -99.7128982544,
		Timezone:  "America/Chicago",
	},
	"2XA1": {
		ICAO:      "2XA1",
		Name:      "El Caballero Airport",
		City:      "Cotulla",
		State:     "Texas",
		Country:   "US",
		Elevation: 317,
		Latitude:  28.1324005127,
		Longitude: -98.9321975708,
		Timezone:  "America/Chicago",
	},
	"2XA2": {
		ICAO:      "2XA2",
		Name:      "Knape Airport",
		City:      "Danbury",
		State:     "Texas",
		Country:   "US",
		Elevation: 20,
		Latitude:  29.2203998566,
		Longitude: -95.3108978271,
		Timezone:  "America/Chicago",
	},
	"2XA4": {
		ICAO:      "2XA4",
		Name:      "Flying S Ranch Airport",
		City:      "Collinsville",
		State:     "Texas",
		Country:   "US",
		Elevation: 730,
		Latitude:  33.5807991028,
		Longitude: -96.8003005981,
		Timezone:  "America/Chicago",
	},
	"2XA5": {
		ICAO:      "2XA5",
		Name:      "Someday Ranch Airport",
		City:      "Lockhart",
		State:     "Texas",
		Country:   "US",
		Elevation: 466,
		Latitude:  29.8036994934,
		Longitude: -97.6912002563,
		Timezone:  "America/Chicago",
	},
	"2XA7": {
		ICAO:      "2XA7",
		Name:      "Tailwheel Airport",
		City:      "Longview",
		State:     "Texas",
		Country:   "US",
		Elevation: 360,
		Latitude:  32.6150016785,
		Longitude: -94.7656021118,
		Timezone:  "America/Chicago",
	},
	"2XS1": {
		ICAO:      "2XS1",
		Name:      "Harris Ranch Airport",
		City:      "Johnson City",
		State:     "Texas",
		Country:   "US",
		Elevation: 1188,
		Latitude:  30.2196998596,
		Longitude: -98.3025970459,
		Timezone:  "America/Chicago",
	},
	"2XS2": {
		ICAO:      "2XS2",
		Name:      "Indio-Faith Airport",
		City:      "Carrizo Springs",
		State:     "Texas",
		Country:   "US",
		Elevation: 750,
		Latitude:  28.2628002167,
		Longitude: -100.162002563,
		Timezone:  "America/Chicago",
	},
	"2XS3": {
		ICAO:      "2XS3",
		Name:      "Glad Oaks Airport",
		City:      "Athens",
		State:     "Texas",
		Country:   "US",
		Elevation: 487,
		Latitude:  32.0167999268,
		Longitude: -95.7002029419,
		Timezone:  "America/Chicago",
	},
	"2XS4": {
		ICAO:      "2XS4",
		Name:      "Skida Patch Airport",
		City:      "Bells",
		State:     "Texas",
		Country:   "US",
		Elevation: 665,
		Latitude:  33.6613998413,
		Longitude: -96.4064025879,
		Timezone:  "America/Chicago",
	},
	"2XS5": {
		ICAO:      "2XS5",
		Name:      "Cross Triangle Ranch Airport",
		City:      "Twin Sisters",
		State:     "Texas",
		Country:   "US",
		Elevation: 1420,
		Latitude:  30.0104999542,
		Longitude: -98.4253005981,
		Timezone:  "America/Chicago",
	},
	"2XS6": {
		ICAO:      "2XS6",
		Name:      "Foster Ranch Airport",
		City:      "Utopia",
		State:     "Texas",
		Country:   "US",
		Elevation: 1687,
		Latitude:  29.7210998535,
		Longitude: -99.5711975098,
		Timezone:  "America/Chicago",
	},
	"2XS7": {
		ICAO:      "2XS7",
		Name:      "Annandale Ranch Airport",
		City:      "Uvalde",
		State:     "Texas",
		Country:   "US",
		Elevation: 1205,
		Latitude:  29.4501991272,
		Longitude: -99.6837005615,
		Timezone:  "America/Chicago",
	},
	"2XS8": {
		ICAO:      "2XS8",
		Name:      "Benson Airstrip",
		City:      "Uvalde",
		State:     "Texas",
		Country:   "US",
		Elevation: 929,
		Latitude:  29.2294006348,
		Longitude: -99.8238983154,
		Timezone:  "America/Chicago",
	},
	"30AK": {
		ICAO:      "30AK",
		Name:      "Doyle Estates Airport",
		City:      "Kenai",
		State:     "Alaska",
		Country:   "US",
		Elevation: 50,
		Latitude:  60.5027999878,
		Longitude: -151.2599945068,
		Timezone:  "America/Anchorage",
	},
	"30AR": {
		ICAO:      "30AR",
		Name:      "Phalanx Airport",
		City:      "Greers Ferry",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 584,
		Latitude:  35.54436111,
		Longitude: -92.0561676,
		Timezone:  "America/Chicago",
	},
	"30AZ": {
		ICAO:      "30AZ",
		Name:      "Gila Compressor Station Airport",
		City:      "Arlington",
		State:     "Arizona",
		Country:   "US",
		Elevation: 838,
		Latitude:  33.25,
		Longitude: -112.81300354,
		Timezone:  "America/Phoenix",
	},
	"30CO": {
		ICAO:      "30CO",
		Name:      "Coyote Creek Ranch Airport",
		City:      "Guffey",
		State:     "Colorado",
		Country:   "US",
		Elevation: 8030,
		Latitude:  38.6725006104,
		Longitude: -105.3359985352,
		Timezone:  "America/Denver",
	},
	"30GA": {
		ICAO:      "30GA",
		Name:      "Elliott Field",
		City:      "Dawsonville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1500,
		Latitude:  34.4589996338,
		Longitude: -84.1812973022,
		Timezone:  "America/New_York",
	},
	"30ID": {
		ICAO:      "30ID",
		Name:      "Mason Airport",
		City:      "Murtaugh",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4210,
		Latitude:  42.432222,
		Longitude: -114.166389,
		Timezone:  "America/Boise",
	},
	"30IN": {
		ICAO:      "30IN",
		Name:      "Wigent Airport",
		City:      "Columbia City",
		State:     "Indiana",
		Country:   "US",
		Elevation: 880,
		Latitude:  41.2092018127,
		Longitude: -85.4580001831,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"30IS": {
		ICAO:      "30IS",
		Name:      "Aero Lake Estates Airport",
		City:      "Genoa",
		State:     "Illinois",
		Country:   "US",
		Elevation: 875,
		Latitude:  42.0772018433,
		Longitude: -88.6250991821,
		Timezone:  "America/Chicago",
	},
	"30KS": {
		ICAO:      "30KS",
		Name:      "Tyler Airport",
		City:      "Arkansas City",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1200,
		Latitude:  37.1323013306,
		Longitude: -97.0734024048,
		Timezone:  "America/Chicago",
	},
	"30LL": {
		ICAO:      "30LL",
		Name:      "Williams Airpark",
		City:      "Chapin",
		State:     "Illinois",
		Country:   "US",
		Elevation: 614,
		Latitude:  39.7541999817,
		Longitude: -90.3654022217,
		Timezone:  "America/Chicago",
	},
	"30MI": {
		ICAO:      "30MI",
		Name:      "Wolverton's Field",
		City:      "Buchanan",
		State:     "Michigan",
		Country:   "US",
		Elevation: 750,
		Latitude:  41.8591995239,
		Longitude: -86.327796936,
		Timezone:  "America/Detroit",
	},
	"30MO": {
		ICAO:      "30MO",
		Name:      "Matzie Airport",
		City:      "Lebanon",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1129,
		Latitude:  37.6459007263,
		Longitude: -92.4945983887,
		Timezone:  "America/Chicago",
	},
	"30NC": {
		ICAO:      "30NC",
		Name:      "Happy Bottom Airport",
		City:      "Advance",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 705,
		Latitude:  36.0485992432,
		Longitude: -80.4569015503,
		Timezone:  "America/New_York",
	},
	"30NE": {
		ICAO:      "30NE",
		Name:      "Rs Ag-Land Airport",
		City:      "Campbell",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2000,
		Latitude:  40.3558006287,
		Longitude: -98.6830978394,
		Timezone:  "America/Chicago",
	},
	"30NY": {
		ICAO:      "30NY",
		Name:      "Coye Field",
		City:      "Canadice",
		State:     "New York",
		Country:   "US",
		Elevation: 1610,
		Latitude:  42.7500991821,
		Longitude: -77.5580978394,
		Timezone:  "America/New_York",
	},
	"30OK": {
		ICAO:      "30OK",
		Name:      "Lobo Field",
		City:      "Calvin",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 860,
		Latitude:  34.9168014526,
		Longitude: -96.2335968018,
		Timezone:  "America/Chicago",
	},
	"30OR": {
		ICAO:      "30OR",
		Name:      "Bero Field",
		City:      "Vernonia",
		State:     "Oregon",
		Country:   "US",
		Elevation: 700,
		Latitude:  45.8026008606,
		Longitude: -123.2799987793,
		Timezone:  "America/Los_Angeles",
	},
	"30PA": {
		ICAO:      "30PA",
		Name:      "Kings Airport",
		City:      "Schwenksville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 290,
		Latitude:  40.2678985596,
		Longitude: -75.4365997314,
		Timezone:  "America/New_York",
	},
	"30TA": {
		ICAO:      "30TA",
		Name:      "Tri-Modal Air Park",
		City:      "Georgetown",
		State:     "Texas",
		Country:   "US",
		Elevation: 812,
		Latitude:  30.5918998718,
		Longitude: -97.7225036621,
		Timezone:  "America/Chicago",
	},
	"30TE": {
		ICAO:      "30TE",
		Name:      "Cone Airport",
		City:      "Ralls",
		State:     "Texas",
		Country:   "US",
		Elevation: 3144,
		Latitude:  33.8111991882,
		Longitude: -101.3960037231,
		Timezone:  "America/Chicago",
	},
	"30TS": {
		ICAO:      "30TS",
		Name:      "Hall Airport",
		City:      "Ore City",
		State:     "Texas",
		Country:   "US",
		Elevation: 350,
		Latitude:  32.8284988403,
		Longitude: -94.7188034058,
		Timezone:  "America/Chicago",
	},
	"30TX": {
		ICAO:      "30TX",
		Name:      "Farmer's Co-Op Airport",
		City:      "Harlingen",
		State:     "Texas",
		Country:   "US",
		Elevation: 35,
		Latitude:  26.1355991364,
		Longitude: -97.6947021484,
		Timezone:  "America/Chicago",
	},
	"30WA": {
		ICAO:      "30WA",
		Name:      "Weller Canyon Airport",
		City:      "Waitsburg",
		State:     "Washington",
		Country:   "US",
		Elevation: 1300,
		Latitude:  46.3148994446,
		Longitude: -118.1780014038,
		Timezone:  "America/Los_Angeles",
	},
	"30WI": {
		ICAO:      "30WI",
		Name:      "Chambers Island Airport",
		City:      "Chambers Island",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 588,
		Latitude:  45.1918983459,
		Longitude: -87.3592987061,
		Timezone:  "America/Chicago",
	},
	"30XS": {
		ICAO:      "30XS",
		Name:      "Farwell Spraying Service Inc Airport",
		City:      "Farwell",
		State:     "Texas",
		Country:   "US",
		Elevation: 4115,
		Latitude:  34.3823013306,
		Longitude: -103.0179977417,
		Timezone:  "America/Chicago",
	},
	"31AK": {
		ICAO:      "31AK",
		Name:      "Dalrymple's Airport",
		City:      "Fairbanks",
		State:     "Alaska",
		Country:   "US",
		Elevation: 450,
		Latitude:  64.832901001,
		Longitude: -147.4250030518,
		Timezone:  "America/Anchorage",
	},
	"31AZ": {
		ICAO:      "31AZ",
		Name:      "Benson Airport",
		City:      "Benson",
		State:     "Arizona",
		Country:   "US",
		Elevation: 3628,
		Latitude:  31.9636993408,
		Longitude: -110.2600021362,
		Timezone:  "America/Phoenix",
	},
	"31FA": {
		ICAO:      "31FA",
		Name:      "S & S Avion Ranch Airport",
		City:      "Oxford",
		State:     "Florida",
		Country:   "US",
		Elevation: 70,
		Latitude:  28.9591999054,
		Longitude: -82.1342010498,
		Timezone:  "America/New_York",
	},
	"31FL": {
		ICAO:      "31FL",
		Name:      "Tedford Ranch Airport",
		City:      "Melbourne",
		State:     "Florida",
		Country:   "US",
		Elevation: 54,
		Latitude:  28.0433998108,
		Longitude: -81.019203186,
		Timezone:  "America/New_York",
	},
	"31II": {
		ICAO:      "31II",
		Name:      "Rising Sun Airport",
		City:      "Rising Sun",
		State:     "Indiana",
		Country:   "US",
		Elevation: 830,
		Latitude:  38.9258995056,
		Longitude: -84.9119033813,
		Timezone:  "America/Indiana/Vevay",
	},
	"31IN": {
		ICAO:      "31IN",
		Name:      "Timber House Airport",
		City:      "Lafayette",
		State:     "Indiana",
		Country:   "US",
		Elevation: 726,
		Latitude:  40.2849998474,
		Longitude: -86.8563995361,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"31KS": {
		ICAO:      "31KS",
		Name:      "Mills Field",
		City:      "South Hutchinson",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1520,
		Latitude:  37.9958000183,
		Longitude: -97.9294967651,
		Timezone:  "America/Chicago",
	},
	"31KY": {
		ICAO:      "31KY",
		Name:      "Wild Blue Airport",
		City:      "Paris",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 890,
		Latitude:  38.1105995178,
		Longitude: -84.3221969604,
		Timezone:  "America/New_York",
	},
	"31LL": {
		ICAO:      "31LL",
		Name:      "Seeman Airport",
		City:      "Chenoa",
		State:     "Illinois",
		Country:   "US",
		Elevation: 689,
		Latitude:  40.7785987854,
		Longitude: -88.6661987305,
		Timezone:  "America/Chicago",
	},
	"31MA": {
		ICAO:      "31MA",
		Name:      "Norm's Field",
		City:      "Belchertown",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 460,
		Latitude:  42.2620010376,
		Longitude: -72.4080963135,
		Timezone:  "America/New_York",
	},
	"31MI": {
		ICAO:      "31MI",
		Name:      "Rentz Ultralightport",
		City:      "Niles",
		State:     "Michigan",
		Country:   "US",
		Elevation: 800,
		Latitude:  41.779800415,
		Longitude: -86.1772003174,
		Timezone:  "America/Detroit",
	},
	"31MO": {
		ICAO:      "31MO",
		Name:      "Hannah Airport",
		City:      "Bolckow",
		State:     "Missouri",
		Country:   "US",
		Elevation: 937,
		Latitude:  40.1054992676,
		Longitude: -94.7433013916,
		Timezone:  "America/Chicago",
	},
	"31NE": {
		ICAO:      "31NE",
		Name:      "Grimm Farm Airport",
		City:      "Crofton",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1670,
		Latitude:  42.7550010681,
		Longitude: -97.7051010132,
		Timezone:  "America/Chicago",
	},
	"31TA": {
		ICAO:      "31TA",
		Name:      "Vitek Field",
		City:      "Granger",
		State:     "Texas",
		Country:   "US",
		Elevation: 580,
		Latitude:  30.7294006348,
		Longitude: -97.4253005981,
		Timezone:  "America/Chicago",
	},
	"31TS": {
		ICAO:      "31TS",
		Name:      "Flyers Field",
		City:      "Greenville",
		State:     "Texas",
		Country:   "US",
		Elevation: 640,
		Latitude:  33.1903991699,
		Longitude: -96.1891021729,
		Timezone:  "America/Chicago",
	},
	"31TX": {
		ICAO:      "31TX",
		Name:      "Scott Airport",
		City:      "Waco",
		State:     "Texas",
		Country:   "US",
		Elevation: 590,
		Latitude:  31.5967998505,
		Longitude: -97.3300018311,
		Timezone:  "America/Chicago",
	},
	"31VA": {
		ICAO:      "31VA",
		Name:      "Aberdeen Field",
		City:      "Smithfield",
		State:     "Virginia",
		Country:   "US",
		Elevation: 33,
		Latitude:  37.0237998962,
		Longitude: -76.5886001587,
		Timezone:  "America/New_York",
	},
	"31WI": {
		ICAO:      "31WI",
		Name:      "Mcdermott Air Park",
		City:      "Sullivan",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 875,
		Latitude:  43.0071983337,
		Longitude: -88.6019973755,
		Timezone:  "America/Chicago",
	},
	"31WN": {
		ICAO:      "31WN",
		Name:      "Berlin Field",
		City:      "Berlin",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 773,
		Latitude:  43.9916000366,
		Longitude: -88.962600708,
		Timezone:  "America/Chicago",
	},
	"31XS": {
		ICAO:      "31XS",
		Name:      "Fly-N-Ski Airport",
		City:      "Merit",
		State:     "Texas",
		Country:   "US",
		Elevation: 638,
		Latitude:  33.1837005615,
		Longitude: -96.2703018188,
		Timezone:  "America/Chicago",
	},
	"32AK": {
		ICAO:      "32AK",
		Name:      "Hardrock Field",
		City:      "Fairbanks",
		State:     "Alaska",
		Country:   "US",
		Elevation: 664,
		Latitude:  64.90930176,
		Longitude: -147.9140015,
		Timezone:  "America/Anchorage",
	},
	"32CA": {
		ICAO:      "32CA",
		Name:      "Stone Airstrip",
		City:      "Lemoore",
		State:     "California",
		Country:   "US",
		Elevation: 220,
		Latitude:  36.3333015442,
		Longitude: -119.8249969482,
		Timezone:  "America/Los_Angeles",
	},
	"32CL": {
		ICAO:      "32CL",
		Name:      "Bob's Flying Service Inc Airport",
		City:      "Knights Landing",
		State:     "California",
		Country:   "US",
		Elevation: 40,
		Latitude:  38.8331985474,
		Longitude: -121.7180023193,
		Timezone:  "America/Los_Angeles",
	},
	"32CO": {
		ICAO:      "32CO",
		Name:      "Braun Airport",
		City:      "Penrose",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5543,
		Latitude:  38.4552993774,
		Longitude: -105.0839996338,
		Timezone:  "America/Denver",
	},
	"32FA": {
		ICAO:      "32FA",
		Name:      "Sunset Strip Airpark",
		City:      "Okeechobee",
		State:     "Florida",
		Country:   "US",
		Elevation: 34,
		Latitude:  27.2341995239,
		Longitude: -80.9037017822,
		Timezone:  "America/New_York",
	},
	"32GA": {
		ICAO:      "32GA",
		Name:      "Sebastian Cove Airport",
		City:      "Eatonton",
		State:     "Georgia",
		Country:   "US",
		Elevation: 490,
		Latitude:  33.4542007446,
		Longitude: -83.2778015137,
		Timezone:  "America/New_York",
	},
	"32II": {
		ICAO:      "32II",
		Name:      "Roberts Airport",
		City:      "Vevay",
		State:     "Indiana",
		Country:   "US",
		Elevation: 820,
		Latitude:  38.8163986206,
		Longitude: -85.106300354,
		Timezone:  "America/Indiana/Vevay",
	},
	"32IS": {
		ICAO:      "32IS",
		Name:      "Baker Airport",
		City:      "Kilbourne",
		State:     "Illinois",
		Country:   "US",
		Elevation: 485,
		Latitude:  40.1352996826,
		Longitude: -91.0095977783,
		Timezone:  "America/Chicago",
	},
	"32KS": {
		ICAO:      "32KS",
		Name:      "Wilkens Airport",
		City:      "Healy",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2856,
		Latitude:  38.5931015015,
		Longitude: -100.6129989624,
		Timezone:  "America/Chicago",
	},
	"32LL": {
		ICAO:      "32LL",
		Name:      "The Sandbox Airport",
		City:      "Cordova",
		State:     "Illinois",
		Country:   "US",
		Elevation: 620,
		Latitude:  41.7042007446,
		Longitude: -90.259803772,
		Timezone:  "America/Chicago",
	},
	"32LS": {
		ICAO:      "32LS",
		Name:      "Stevens Strip",
		City:      "Ponchatoula",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 13,
		Latitude:  30.41959953,
		Longitude: -90.35900116,
		Timezone:  "America/Chicago",
	},
	"32MD": {
		ICAO:      "32MD",
		Name:      "Roseland Airport",
		City:      "Sudlersville",
		State:     "Maryland",
		Country:   "US",
		Elevation: 70,
		Latitude:  39.1893997192,
		Longitude: -75.8768005371,
		Timezone:  "America/New_York",
	},
	"32MN": {
		ICAO:      "32MN",
		Name:      "Don's Landing Field",
		City:      "Watkins",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1140,
		Latitude:  45.3675003052,
		Longitude: -94.3993988037,
		Timezone:  "America/Chicago",
	},
	"32MO": {
		ICAO:      "32MO",
		Name:      "Phillips Airport",
		City:      "Fredericktown",
		State:     "Missouri",
		Country:   "US",
		Elevation: 860,
		Latitude:  37.5766983032,
		Longitude: -90.2259979248,
		Timezone:  "America/Chicago",
	},
	"32MS": {
		ICAO:      "32MS",
		Name:      "Browns Landing",
		City:      "Foxworth",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 385,
		Latitude:  31.194167,
		Longitude: -89.9525,
		Timezone:  "America/Chicago",
	},
	"32NC": {
		ICAO:      "32NC",
		Name:      "Hall Field",
		City:      "Parkton",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 175,
		Latitude:  34.8964004517,
		Longitude: -79.0430984497,
		Timezone:  "America/New_York",
	},
	"32NE": {
		ICAO:      "32NE",
		Name:      "Mc Ginn Ranch Airport",
		City:      "Dunning",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2758,
		Latitude:  41.7210998535,
		Longitude: -100.087997437,
		Timezone:  "America/Chicago",
	},
	"32NK": {
		ICAO:      "32NK",
		Name:      "Schoharie Creek Airport",
		City:      "Esperance",
		State:     "New York",
		Country:   "US",
		Elevation: 620,
		Latitude:  42.7532997131,
		Longitude: -74.3110961914,
		Timezone:  "America/New_York",
	},
	"32OH": {
		ICAO:      "32OH",
		Name:      "Rall Field",
		City:      "Mansfield",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1330,
		Latitude:  40.7611999512,
		Longitude: -82.4179000854,
		Timezone:  "America/New_York",
	},
	"32OK": {
		ICAO:      "32OK",
		Name:      "Flying A Ranch Airport",
		City:      "Hartshorne",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 670,
		Latitude:  34.8317985535,
		Longitude: -95.5175018311,
		Timezone:  "America/Chicago",
	},
	"32OR": {
		ICAO:      "32OR",
		Name:      "Pine Hollow Airport",
		City:      "Wamic",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1820,
		Latitude:  45.2547988892,
		Longitude: -121.2939987183,
		Timezone:  "America/Los_Angeles",
	},
	"32PA": {
		ICAO:      "32PA",
		Name:      "Yost Personal Use Airport",
		City:      "Shermans Dale",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 560,
		Latitude:  40.3301010132,
		Longitude: -77.1772003174,
		Timezone:  "America/New_York",
	},
	"32PN": {
		ICAO:      "32PN",
		Name:      "Black Rock Airport",
		City:      "New Brighton",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1150,
		Latitude:  40.7340011597,
		Longitude: -80.2759017944,
		Timezone:  "America/New_York",
	},
	"32TA": {
		ICAO:      "32TA",
		Name:      "Wilbourn Ranch Airport",
		City:      "Harwood",
		State:     "Texas",
		Country:   "US",
		Elevation: 350,
		Latitude:  29.6919002533,
		Longitude: -97.4336013794,
		Timezone:  "America/Chicago",
	},
	"32TE": {
		ICAO:      "32TE",
		Name:      "Granite Shoals Municipal Airport",
		City:      "Granite Shoals",
		State:     "Texas",
		Country:   "US",
		Elevation: 860,
		Latitude:  30.5893993378,
		Longitude: -98.370300293,
		Timezone:  "America/Chicago",
	},
	"32TX": {
		ICAO:      "32TX",
		Name:      "Hinson Airport",
		City:      "Kenney",
		State:     "Texas",
		Country:   "US",
		Elevation: 365,
		Latitude:  30.0580997467,
		Longitude: -96.3591995239,
		Timezone:  "America/Chicago",
	},
	"32VA": {
		ICAO:      "32VA",
		Name:      "Old South Aerodrome",
		City:      "Abingdon",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1970,
		Latitude:  36.677778,
		Longitude: -81.928722,
		Timezone:  "America/New_York",
	},
	"32WA": {
		ICAO:      "32WA",
		Name:      "Whitestone Airport",
		City:      "Tonasket",
		State:     "Washington",
		Country:   "US",
		Elevation: 1310,
		Latitude:  48.7227777778,
		Longitude: -119.467222222,
		Timezone:  "America/Los_Angeles",
	},
	"32WI": {
		ICAO:      "32WI",
		Name:      "Dalonia Airport",
		City:      "Hortonville",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 850,
		Latitude:  44.3163986206,
		Longitude: -88.7008972168,
		Timezone:  "America/Chicago",
	},
	"32WN": {
		ICAO:      "32WN",
		Name:      "Bowman Airstrip",
		City:      "Suring",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 840,
		Latitude:  45.099606,
		Longitude: -88.419733,
		Timezone:  "America/Chicago",
	},
	"33AK": {
		ICAO:      "33AK",
		Name:      "Nugget Bench Airport",
		City:      "Nugget Bench",
		State:     "Alaska",
		Country:   "US",
		Elevation: 2010,
		Latitude:  62.5172996521,
		Longitude: -150.9450073242,
		Timezone:  "America/Anchorage",
	},
	"33AR": {
		ICAO:      "33AR",
		Name:      "Skypoint Estates Airport",
		City:      "Greers Ferry",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 547,
		Latitude:  35.6021995544,
		Longitude: -92.1425018311,
		Timezone:  "America/Chicago",
	},
	"33AZ": {
		ICAO:      "33AZ",
		Name:      "Yolo Ranch Airport",
		City:      "Camp Wood",
		State:     "Arizona",
		Country:   "US",
		Elevation: 5950,
		Latitude:  34.7985992432,
		Longitude: -112.9700012207,
		Timezone:  "America/Phoenix",
	},
	"33CA": {
		ICAO:      "33CA",
		Name:      "Lloyd's Landing Airport",
		City:      "Rosamond",
		State:     "California",
		Country:   "US",
		Elevation: 2690,
		Latitude:  34.9054985046,
		Longitude: -118.3020019531,
		Timezone:  "America/Los_Angeles",
	},
	"33CL": {
		ICAO:      "33CL",
		Name:      "Oak Country Ranch Airport",
		City:      "Paso Robles",
		State:     "California",
		Country:   "US",
		Elevation: 1200,
		Latitude:  35.5682983398,
		Longitude: -120.7969970703,
		Timezone:  "America/Los_Angeles",
	},
	"33CO": {
		ICAO:      "33CO",
		Name:      "Melby Ranch Airstrip",
		City:      "San Luis",
		State:     "Colorado",
		Country:   "US",
		Elevation: 8820,
		Latitude:  37.0649986267,
		Longitude: -105.4639968872,
		Timezone:  "America/Denver",
	},
	"33CT": {
		ICAO:      "33CT",
		Name:      "Irish Hills Farms Airport",
		City:      "Bethlehem",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 830,
		Latitude:  41.609444,
		Longitude: -73.239722,
		Timezone:  "America/New_York",
	},
	"33FA": {
		ICAO:      "33FA",
		Name:      "Recreation Corporation Airport",
		City:      "Okeechobee",
		State:     "Florida",
		Country:   "US",
		Elevation: 67,
		Latitude:  27.5981006622,
		Longitude: -80.8459014893,
		Timezone:  "America/New_York",
	},
	"33FD": {
		ICAO:      "33FD",
		Name:      "R O Ranch STOLport",
		City:      "Mayo",
		State:     "Florida",
		Country:   "US",
		Elevation: 50,
		Latitude:  29.9144001007,
		Longitude: -83.2665023804,
		Timezone:  "America/New_York",
	},
	"33FL": {
		ICAO:      "33FL",
		Name:      "Flying S Ranch Airport",
		City:      "Kenansville",
		State:     "Florida",
		Country:   "US",
		Elevation: 72,
		Latitude:  27.8608608246,
		Longitude: -80.9998626709,
		Timezone:  "America/New_York",
	},
	"33GA": {
		ICAO:      "33GA",
		Name:      "Hudson River Landing Airport",
		City:      "Carnesville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 665,
		Latitude:  34.2599983215,
		Longitude: -83.2866973877,
		Timezone:  "America/New_York",
	},
	"33IL": {
		ICAO:      "33IL",
		Name:      "John Scharff Airport",
		City:      "Clinton",
		State:     "Illinois",
		Country:   "US",
		Elevation: 730,
		Latitude:  40.1660995483,
		Longitude: -88.8983001709,
		Timezone:  "America/Chicago",
	},
	"33IS": {
		ICAO:      "33IS",
		Name:      "Howell Airport",
		City:      "Findlay",
		State:     "Illinois",
		Country:   "US",
		Elevation: 672,
		Latitude:  39.5444984436,
		Longitude: -88.757598877,
		Timezone:  "America/Chicago",
	},
	"33KS": {
		ICAO:      "33KS",
		Name:      "Buena Terra Airport",
		City:      "Topeka",
		State:     "Kansas",
		Country:   "US",
		Elevation: 950,
		Latitude:  39.1530990601,
		Longitude: -95.6093978882,
		Timezone:  "America/Chicago",
	},
	"33KY": {
		ICAO:      "33KY",
		Name:      "Bigger (Stol) STOLport",
		City:      "Utica",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 406,
		Latitude:  37.6123008728,
		Longitude: -87.0569000244,
		Timezone:  "America/Chicago",
	},
	"33LA": {
		ICAO:      "33LA",
		Name:      "Sky Ranch Airport",
		City:      "Youngsville",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 25,
		Latitude:  30.0695991516,
		Longitude: -91.9861984253,
		Timezone:  "America/Chicago",
	},
	"33LL": {
		ICAO:      "33LL",
		Name:      "Isley Airport",
		City:      "Wheeler",
		State:     "Illinois",
		Country:   "US",
		Elevation: 595,
		Latitude:  39.0875015259,
		Longitude: -88.2889022827,
		Timezone:  "America/Chicago",
	},
	"33MI": {
		ICAO:      "33MI",
		Name:      "Great Lakes Airport",
		City:      "Sagola",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1300,
		Latitude:  46.0666007996,
		Longitude: -88.007598877,
		Timezone:  "America/Menominee",
	},
	"33MN": {
		ICAO:      "33MN",
		Name:      "Swift Private Airport",
		City:      "Wendell",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1100,
		Latitude:  46.0900001526,
		Longitude: -96.0970001221,
		Timezone:  "America/Chicago",
	},
	"33MO": {
		ICAO:      "33MO",
		Name:      "Leaming Field",
		City:      "Golden City",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1135,
		Latitude:  37.2513999939,
		Longitude: -94.0569000244,
		Timezone:  "America/Chicago",
	},
	"33NC": {
		ICAO:      "33NC",
		Name:      "Pettigrew Moore Aerodrome",
		City:      "Wilmington",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 30,
		Latitude:  34.4268989563,
		Longitude: -78.0894012451,
		Timezone:  "America/New_York",
	},
	"33NE": {
		ICAO:      "33NE",
		Name:      "Orr Ranch Airport",
		City:      "Hay Springs",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3880,
		Latitude:  42.386100769,
		Longitude: -102.5749969482,
		Timezone:  "America/Denver",
	},
	"33NR": {
		ICAO:      "33NR",
		Name:      "Wolf Ridge Airport",
		City:      "Mars Hill",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 4605,
		Latitude:  35.941667,
		Longitude: -82.506667,
		Timezone:  "America/New_York",
	},
	"33NY": {
		ICAO:      "33NY",
		Name:      "Wayne Delp Airport",
		City:      "West Coxsackie",
		State:     "New York",
		Country:   "US",
		Elevation: 800,
		Latitude:  42.3995018005,
		Longitude: -73.8662033081,
		Timezone:  "America/New_York",
	},
	"33OH": {
		ICAO:      "33OH",
		Name:      "Sunset Strip",
		City:      "Jerome",
		State:     "Ohio",
		Country:   "US",
		Elevation: 945,
		Latitude:  40.1852989197,
		Longitude: -83.1735992432,
		Timezone:  "America/New_York",
	},
	"33OI": {
		ICAO:      "33OI",
		Name:      "Soaring Horse Airport",
		City:      "Chatham Township",
		State:     "Ohio",
		Country:   "US",
		Elevation: 950,
		Latitude:  41.0828018188,
		Longitude: -82.0431976318,
		Timezone:  "America/New_York",
	},
	"33OK": {
		ICAO:      "33OK",
		Name:      "Myrick Airport",
		City:      "Edmond",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1195,
		Latitude:  35.7432903732,
		Longitude: -97.4055790901,
		Timezone:  "America/Chicago",
	},
	"33OR": {
		ICAO:      "33OR",
		Name:      "Crow-Mag Airport",
		City:      "Elmira",
		State:     "Oregon",
		Country:   "US",
		Elevation: 450,
		Latitude:  44.0637016296,
		Longitude: -123.3909988403,
		Timezone:  "America/Los_Angeles",
	},
	"33PA": {
		ICAO:      "33PA",
		Name:      "Sutliff Private Airport",
		City:      "Shickshinny",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 800,
		Latitude:  41.1668014526,
		Longitude: -76.2497024536,
		Timezone:  "America/New_York",
	},
	"33TA": {
		ICAO:      "33TA",
		Name:      "Lake Bonanza Airport",
		City:      "Montgomery",
		State:     "Texas",
		Country:   "US",
		Elevation: 301,
		Latitude:  30.3330001831,
		Longitude: -95.612197876,
		Timezone:  "America/Chicago",
	},
	"33VA": {
		ICAO:      "33VA",
		Name:      "Fox Fire Airport",
		City:      "Halifax",
		State:     "Virginia",
		Country:   "US",
		Elevation: 535,
		Latitude:  36.7843017578,
		Longitude: -79.049697876,
		Timezone:  "America/New_York",
	},
	"33WA": {
		ICAO:      "33WA",
		Name:      "Franz Ranch Airport",
		City:      "Warden",
		State:     "Washington",
		Country:   "US",
		Elevation: 1478,
		Latitude:  47.0499000549,
		Longitude: -118.8509979248,
		Timezone:  "America/Los_Angeles",
	},
	"34AK": {
		ICAO:      "34AK",
		Name:      "Cardwell Strip",
		City:      "Palmer",
		State:     "Alaska",
		Country:   "US",
		Elevation: 425,
		Latitude:  61.625,
		Longitude: -149.2890014648,
		Timezone:  "America/Anchorage",
	},
	"34AR": {
		ICAO:      "34AR",
		Name:      "Woodbridge Field",
		City:      "Marked Tree",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 215,
		Latitude:  35.5811004639,
		Longitude: -90.3852996826,
		Timezone:  "America/Chicago",
	},
	"34AZ": {
		ICAO:      "34AZ",
		Name:      "Gila River Memorial Airport",
		City:      "Chandler",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1185,
		Latitude:  33.2434005737,
		Longitude: -111.9130020142,
		Timezone:  "America/Phoenix",
	},
	"34CD": {
		ICAO:      "34CD",
		Name:      "Elk Park Ranch Airport",
		City:      "Allenspark",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7900,
		Latitude:  40.262683,
		Longitude: -105.448011,
		Timezone:  "America/Denver",
	},
	"34CN": {
		ICAO:      "34CN",
		Name:      "Bonanza Hills Airport",
		City:      "Snelling",
		State:     "California",
		Country:   "US",
		Elevation: 295,
		Latitude:  37.5234985352,
		Longitude: -120.3980026245,
		Timezone:  "America/Los_Angeles",
	},
	"34CO": {
		ICAO:      "34CO",
		Name:      "Simons Airport",
		City:      "Aurora",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5525,
		Latitude:  39.7164001465,
		Longitude: -104.7320022583,
		Timezone:  "America/Denver",
	},
	"34FD": {
		ICAO:      "34FD",
		Name:      "Blue Ridge Flightpark Airport",
		City:      "Oakhill",
		State:     "Florida",
		Country:   "US",
		Elevation: 25,
		Latitude:  28.8572006226,
		Longitude: -80.9069976807,
		Timezone:  "America/New_York",
	},
	"34FL": {
		ICAO:      "34FL",
		Name:      "Ellis Agricultural Field",
		City:      "Paxton",
		State:     "Florida",
		Country:   "US",
		Elevation: 324,
		Latitude:  30.9654998779,
		Longitude: -86.302696228,
		Timezone:  "America/Chicago",
	},
	"34ID": {
		ICAO:      "34ID",
		Name:      "Freeman Creek Airport",
		City:      "Cavendish",
		State:     "Idaho",
		Country:   "US",
		Elevation: 3100,
		Latitude:  46.570333,
		Longitude: -116.343611,
		Timezone:  "America/Los_Angeles",
	},
	"34II": {
		ICAO:      "34II",
		Name:      "Burk Personal Use Airport",
		City:      "Anderson",
		State:     "Indiana",
		Country:   "US",
		Elevation: 930,
		Latitude:  40.0505981445,
		Longitude: -85.6027984619,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"34IL": {
		ICAO:      "34IL",
		Name:      "G. Bray Airport",
		City:      "Lewistown",
		State:     "Illinois",
		Country:   "US",
		Elevation: 550,
		Latitude:  40.3624992371,
		Longitude: -90.1084976196,
		Timezone:  "America/Chicago",
	},
	"34IN": {
		ICAO:      "34IN",
		Name:      "Windy P Ridge Airport",
		City:      "Portland",
		State:     "Indiana",
		Country:   "US",
		Elevation: 1035,
		Latitude:  40.3403015137,
		Longitude: -85.0179977417,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"34IS": {
		ICAO:      "34IS",
		Name:      "Jim Wehrli Memorial Airport",
		City:      "Marseilles",
		State:     "Illinois",
		Country:   "US",
		Elevation: 630,
		Latitude:  41.4448013306,
		Longitude: -88.7166976929,
		Timezone:  "America/Chicago",
	},
	"34KY": {
		ICAO:      "34KY",
		Name:      "Lone Pine Aerodrome",
		City:      "Auburn",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 720,
		Latitude:  36.9124984741,
		Longitude: -86.7827987671,
		Timezone:  "America/Chicago",
	},
	"34LA": {
		ICAO:      "34LA",
		Name:      "Swamp Smith Airport",
		City:      "De Ridder",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 180,
		Latitude:  30.7716007233,
		Longitude: -93.2713012695,
		Timezone:  "America/Chicago",
	},
	"34LS": {
		ICAO:      "34LS",
		Name:      "Coastal Ridge Airpark",
		City:      "Abbeville",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 20,
		Latitude:  30.0289001465,
		Longitude: -92.014503479,
		Timezone:  "America/Chicago",
	},
	"34MI": {
		ICAO:      "34MI",
		Name:      "De Witt Property Airport",
		City:      "Spring Lake",
		State:     "Michigan",
		Country:   "US",
		Elevation: 605,
		Latitude:  43.1002998352,
		Longitude: -86.2233963013,
		Timezone:  "America/Detroit",
	},
	"34MS": {
		ICAO:      "34MS",
		Name:      "Colle Field",
		City:      "Gautier",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 25,
		Latitude:  30.4738006592,
		Longitude: -88.6961975098,
		Timezone:  "America/Chicago",
	},
	"34ND": {
		ICAO:      "34ND",
		Name:      "Plath Farms Airport",
		City:      "Davenport",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 928,
		Latitude:  46.7057991028,
		Longitude: -97.15650177,
		Timezone:  "America/Chicago",
	},
	"34NE": {
		ICAO:      "34NE",
		Name:      "Evans Ranch Airport",
		City:      "Hershey",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2895,
		Latitude:  41.2238998413,
		Longitude: -100.9349975586,
		Timezone:  "America/Chicago",
	},
	"34NY": {
		ICAO:      "34NY",
		Name:      "Hendershot Airport",
		City:      "Hilton",
		State:     "New York",
		Country:   "US",
		Elevation: 300,
		Latitude:  43.2901000977,
		Longitude: -77.8273010254,
		Timezone:  "America/New_York",
	},
	"34OH": {
		ICAO:      "34OH",
		Name:      "Arend Airport",
		City:      "Mark Center",
		State:     "Ohio",
		Country:   "US",
		Elevation: 720,
		Latitude:  41.2569999695,
		Longitude: -84.6416015625,
		Timezone:  "America/New_York",
	},
	"34OI": {
		ICAO:      "34OI",
		Name:      "Snoddy Air Strip",
		City:      "Shreve",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1180,
		Latitude:  40.6944999695,
		Longitude: -82.1178970337,
		Timezone:  "America/New_York",
	},
	"34OK": {
		ICAO:      "34OK",
		Name:      "Double Bar S Ranches Airport",
		City:      "Shamrock",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 950,
		Latitude:  35.900100708,
		Longitude: -96.5503005981,
		Timezone:  "America/Chicago",
	},
	"34PA": {
		ICAO:      "34PA",
		Name:      "Waltz Airport",
		City:      "Gettysburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 542,
		Latitude:  39.7607002258,
		Longitude: -77.1930007935,
		Timezone:  "America/New_York",
	},
	"34PN": {
		ICAO:      "34PN",
		Name:      "Fox Field",
		City:      "Newport",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 730,
		Latitude:  40.485556,
		Longitude: -77.220556,
		Timezone:  "America/New_York",
	},
	"34TA": {
		ICAO:      "34TA",
		Name:      "Jsi Airport",
		City:      "Princeton",
		State:     "Texas",
		Country:   "US",
		Elevation: 590,
		Latitude:  33.167098999,
		Longitude: -96.487197876,
		Timezone:  "America/Chicago",
	},
	"34TE": {
		ICAO:      "34TE",
		Name:      "Bee Creek Airport",
		City:      "Maypearl",
		State:     "Texas",
		Country:   "US",
		Elevation: 760,
		Latitude:  32.2321014404,
		Longitude: -97.0085983276,
		Timezone:  "America/Chicago",
	},
	"34TS": {
		ICAO:      "34TS",
		Name:      "Canyon Lake Airport",
		City:      "Canyon Lake",
		State:     "Texas",
		Country:   "US",
		Elevation: 940,
		Latitude:  29.9158000946,
		Longitude: -98.2474975586,
		Timezone:  "America/Chicago",
	},
	"34WI": {
		ICAO:      "34WI",
		Name:      "Thiessen Field",
		City:      "Baraboo",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1200,
		Latitude:  43.4178009033,
		Longitude: -89.7654037476,
		Timezone:  "America/Chicago",
	},
	"34XS": {
		ICAO:      "34XS",
		Name:      "Flying Hare Airport",
		City:      "Conroe",
		State:     "Texas",
		Country:   "US",
		Elevation: 225,
		Latitude:  30.391944,
		Longitude: -95.538611,
		Timezone:  "America/Chicago",
	},
	"35AR": {
		ICAO:      "35AR",
		Name:      "Black Horse Landing Ultralightport",
		City:      "Eureka Springs",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1207,
		Latitude:  36.3563995361,
		Longitude: -93.6863021851,
		Timezone:  "America/Chicago",
	},
	"35AZ": {
		ICAO:      "35AZ",
		Name:      "Continental Airport",
		City:      "Continental",
		State:     "Arizona",
		Country:   "US",
		Elevation: 2869,
		Latitude:  31.8437004089,
		Longitude: -110.9769973755,
		Timezone:  "America/Phoenix",
	},
	"35CL": {
		ICAO:      "35CL",
		Name:      "Flying M Ranch Airport",
		City:      "Merced",
		State:     "California",
		Country:   "US",
		Elevation: 154,
		Latitude:  37.2318992615,
		Longitude: -120.4789962769,
		Timezone:  "America/Los_Angeles",
	},
	"35CN": {
		ICAO:      "35CN",
		Name:      "Farnsworth Ranch Airstrip",
		City:      "Grimes",
		State:     "California",
		Country:   "US",
		Elevation: 50,
		Latitude:  39.099899292,
		Longitude: -121.9339981079,
		Timezone:  "America/Los_Angeles",
	},
	"35CO": {
		ICAO:      "35CO",
		Name:      "Lone Tree Ranch Airport",
		City:      "Punkin Center",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5525,
		Latitude:  38.8819999695,
		Longitude: -103.8089981079,
		Timezone:  "America/Denver",
	},
	"35FA": {
		ICAO:      "35FA",
		Name:      "Rimes Lakecrest Airport",
		City:      "Cross Creek",
		State:     "Florida",
		Country:   "US",
		Elevation: 65,
		Latitude:  29.4678993225,
		Longitude: -82.1587982178,
		Timezone:  "America/New_York",
	},
	"35FD": {
		ICAO:      "35FD",
		Name:      "Rush Airport",
		City:      "Trenton",
		State:     "Florida",
		Country:   "US",
		Elevation: 80,
		Latitude:  29.6923007965,
		Longitude: -82.7292022705,
		Timezone:  "America/New_York",
	},
	"35II": {
		ICAO:      "35II",
		Name:      "Brown Airport",
		City:      "Goshen",
		State:     "Indiana",
		Country:   "US",
		Elevation: 850,
		Latitude:  41.542301178,
		Longitude: -85.8419036865,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"35IL": {
		ICAO:      "35IL",
		Name:      "Clarion Field",
		City:      "La Moille",
		State:     "Illinois",
		Country:   "US",
		Elevation: 800,
		Latitude:  41.5339012146,
		Longitude: -89.202796936,
		Timezone:  "America/Chicago",
	},
	"35IN": {
		ICAO:      "35IN",
		Name:      "Ellison Airport",
		City:      "Tipton",
		State:     "Indiana",
		Country:   "US",
		Elevation: 875,
		Latitude:  40.2944984436,
		Longitude: -85.9424972534,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"35IS": {
		ICAO:      "35IS",
		Name:      "Enoch Airport",
		City:      "Findlay",
		State:     "Illinois",
		Country:   "US",
		Elevation: 676,
		Latitude:  39.5284004211,
		Longitude: -88.763999939,
		Timezone:  "America/Chicago",
	},
	"35KS": {
		ICAO:      "35KS",
		Name:      "Selby Farm Airport",
		City:      "Derby",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1250,
		Latitude:  37.5335998535,
		Longitude: -97.3116989136,
		Timezone:  "America/Chicago",
	},
	"35KY": {
		ICAO:      "35KY",
		Name:      "Welcome Field",
		City:      "Franklin",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 695,
		Latitude:  36.7112007141,
		Longitude: -86.636100769,
		Timezone:  "America/Chicago",
	},
	"35LA": {
		ICAO:      "35LA",
		Name:      "Ms Pats Airport",
		City:      "Abbeville",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 10,
		Latitude:  29.9745006561,
		Longitude: -92.1761016846,
		Timezone:  "America/Chicago",
	},
	"35ME": {
		ICAO:      "35ME",
		Name:      "Matinicus Island Airport",
		City:      "Matinicus Island",
		State:     "Maine",
		Country:   "US",
		Elevation: 15,
		Latitude:  43.8711013794,
		Longitude: -68.8933029175,
		Timezone:  "America/New_York",
	},
	"35MI": {
		ICAO:      "35MI",
		Name:      "Law Field",
		City:      "Vermontville",
		State:     "Michigan",
		Country:   "US",
		Elevation: 920,
		Latitude:  42.6338996887,
		Longitude: -84.9858016968,
		Timezone:  "America/Detroit",
	},
	"35MN": {
		ICAO:      "35MN",
		Name:      "Wipline Airport",
		City:      "Inver Grove Heights",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 687,
		Latitude:  44.8154983521,
		Longitude: -93.0134963989,
		Timezone:  "America/Chicago",
	},
	"35NC": {
		ICAO:      "35NC",
		Name:      "Johnston Airport",
		City:      "Mooresville",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 715,
		Latitude:  35.5382995605,
		Longitude: -80.7514038086,
		Timezone:  "America/New_York",
	},
	"35NE": {
		ICAO:      "35NE",
		Name:      "Shelburnes Airport",
		City:      "Ogallala",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3475,
		Latitude:  41.0499992371,
		Longitude: -101.8050003052,
		Timezone:  "America/Denver",
	},
	"35NY": {
		ICAO:      "35NY",
		Name:      "Ciszak Airport",
		City:      "Springville",
		State:     "New York",
		Country:   "US",
		Elevation: 1397,
		Latitude:  42.5125999451,
		Longitude: -78.7352981567,
		Timezone:  "America/New_York",
	},
	"35OK": {
		ICAO:      "35OK",
		Name:      "Schumacher Field",
		City:      "Clinton",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1595,
		Latitude:  35.5,
		Longitude: -99.0355987549,
		Timezone:  "America/Chicago",
	},
	"35OL": {
		ICAO:      "35OL",
		Name:      "Henderson Farm Airport",
		City:      "Mooreland",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1750,
		Latitude:  36.6330986023,
		Longitude: -99.1511993408,
		Timezone:  "America/Chicago",
	},
	"35OR": {
		ICAO:      "35OR",
		Name:      "Freight Wagon Field",
		City:      "Redmond",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3195,
		Latitude:  44.1926002502,
		Longitude: -121.1790008545,
		Timezone:  "America/Los_Angeles",
	},
	"35TS": {
		ICAO:      "35TS",
		Name:      "Hempstead Gliderport",
		City:      "Hempstead",
		State:     "Texas",
		Country:   "US",
		Elevation: 260,
		Latitude:  30.1215991974,
		Longitude: -96.0866012573,
		Timezone:  "America/Chicago",
	},
	"35TX": {
		ICAO:      "35TX",
		Name:      "Flying B Ranch Airstrip",
		City:      "Abilene",
		State:     "Texas",
		Country:   "US",
		Elevation: 1750,
		Latitude:  32.4840011597,
		Longitude: -99.8255996704,
		Timezone:  "America/Chicago",
	},
	"35WA": {
		ICAO:      "35WA",
		Name:      "Fisher Ranch Airport",
		City:      "Washtucna",
		State:     "Washington",
		Country:   "US",
		Elevation: 1521,
		Latitude:  46.7499008179,
		Longitude: -118.4179992676,
		Timezone:  "America/Los_Angeles",
	},
	"35WI": {
		ICAO:      "35WI",
		Name:      "Barker Strip",
		City:      "East Troy",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1000,
		Latitude:  42.7456016541,
		Longitude: -88.4375991821,
		Timezone:  "America/Chicago",
	},
	"36AK": {
		ICAO:      "36AK",
		Name:      "Flyway Farm Airstrip",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 150,
		Latitude:  61.3432998657,
		Longitude: -150.0670013428,
		Timezone:  "America/Anchorage",
	},
	"36AR": {
		ICAO:      "36AR",
		Name:      "David Stanley Memorial Airport",
		City:      "Gregory",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 200,
		Latitude:  35.206401825,
		Longitude: -91.318901062,
		Timezone:  "America/Chicago",
	},
	"36AZ": {
		ICAO:      "36AZ",
		Name:      "Valley Farms Airport",
		City:      "Coolidge",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1478,
		Latitude:  32.9936981201,
		Longitude: -111.4509963989,
		Timezone:  "America/Phoenix",
	},
	"36CA": {
		ICAO:      "36CA",
		Name:      "Stone Land County Airport",
		City:      "Stratford",
		State:     "California",
		Country:   "US",
		Elevation: 265,
		Latitude:  36.1319007874,
		Longitude: -119.986000061,
		Timezone:  "America/Los_Angeles",
	},
	"36CN": {
		ICAO:      "36CN",
		Name:      "Blackwell Land Company Inc Airport",
		City:      "Lost Hills",
		State:     "California",
		Country:   "US",
		Elevation: 760,
		Latitude:  35.6363983154,
		Longitude: -120.0090026855,
		Timezone:  "America/Los_Angeles",
	},
	"36CO": {
		ICAO:      "36CO",
		Name:      "Fat Chance Airport",
		City:      "Peyton",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7100,
		Latitude:  39.106098175,
		Longitude: -104.5439987183,
		Timezone:  "America/Denver",
	},
	"36GA": {
		ICAO:      "36GA",
		Name:      "Lola Landing Airport",
		City:      "Conyers",
		State:     "Georgia",
		Country:   "US",
		Elevation: 870,
		Latitude:  33.7300987244,
		Longitude: -83.9746017456,
		Timezone:  "America/New_York",
	},
	"36II": {
		ICAO:      "36II",
		Name:      "Newby Landing Airport",
		City:      "Brownsburg",
		State:     "Indiana",
		Country:   "US",
		Elevation: 878,
		Latitude:  39.8412017822,
		Longitude: -86.3486022949,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"36IN": {
		ICAO:      "36IN",
		Name:      "Peterson Ultralightport",
		City:      "North Liberty",
		State:     "Indiana",
		Country:   "US",
		Elevation: 730,
		Latitude:  41.5362014771,
		Longitude: -86.443901062,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"36IS": {
		ICAO:      "36IS",
		Name:      "Gillen Airport",
		City:      "Monmouth",
		State:     "Illinois",
		Country:   "US",
		Elevation: 750,
		Latitude:  40.926399231,
		Longitude: -90.7312011719,
		Timezone:  "America/Chicago",
	},
	"36KS": {
		ICAO:      "36KS",
		Name:      "White Farms Airport",
		City:      "Galesburg",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1000,
		Latitude:  37.4748001099,
		Longitude: -95.3435974121,
		Timezone:  "America/Chicago",
	},
	"36KY": {
		ICAO:      "36KY",
		Name:      "Arnolds Airport",
		City:      "Springfield",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 843,
		Latitude:  37.6556015015,
		Longitude: -85.1538009644,
		Timezone:  "America/New_York",
	},
	"36MI": {
		ICAO:      "36MI",
		Name:      "Tecumseh Products Airport",
		City:      "Tecumseh",
		State:     "Michigan",
		Country:   "US",
		Elevation: 795,
		Latitude:  42.032699585,
		Longitude: -83.8783035278,
		Timezone:  "America/Detroit",
	},
	"36MN": {
		ICAO:      "36MN",
		Name:      "Wagner Farm Airport",
		City:      "Barnesville",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 935,
		Latitude:  46.6716003418,
		Longitude: -96.575302124,
		Timezone:  "America/Chicago",
	},
	"36MO": {
		ICAO:      "36MO",
		Name:      "Harrison Private Airport",
		City:      "Rolla",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1050,
		Latitude:  37.8527984619,
		Longitude: -91.6467971802,
		Timezone:  "America/Chicago",
	},
	"36NC": {
		ICAO:      "36NC",
		Name:      "Greensboro North Airport",
		City:      "Greensboro",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 900,
		Latitude:  36.2536010742,
		Longitude: -79.9080963135,
		Timezone:  "America/New_York",
	},
	"36NE": {
		ICAO:      "36NE",
		Name:      "Frevert Airstrip",
		City:      "Wausa",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1800,
		Latitude:  42.5141983032,
		Longitude: -97.5809020996,
		Timezone:  "America/Chicago",
	},
	"36NY": {
		ICAO:      "36NY",
		Name:      "Dew Airpark",
		City:      "Cuba",
		State:     "New York",
		Country:   "US",
		Elevation: 2100,
		Latitude:  42.2359008789,
		Longitude: -78.2117004395,
		Timezone:  "America/New_York",
	},
	"36OH": {
		ICAO:      "36OH",
		Name:      "Wright's Field",
		City:      "Conesville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 770,
		Latitude:  40.1902999878,
		Longitude: -81.9739990234,
		Timezone:  "America/New_York",
	},
	"36OI": {
		ICAO:      "36OI",
		Name:      "Verhoff Airport",
		City:      "Continental",
		State:     "Ohio",
		Country:   "US",
		Elevation: 728,
		Latitude:  41.0731010437,
		Longitude: -84.2126998901,
		Timezone:  "America/New_York",
	},
	"36OK": {
		ICAO:      "36OK",
		Name:      "Jones Air Park",
		City:      "Tecumseh",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1100,
		Latitude:  35.2723007202,
		Longitude: -96.8816986084,
		Timezone:  "America/Chicago",
	},
	"36OR": {
		ICAO:      "36OR",
		Name:      "Jasper Ridge Airstrip",
		City:      "Springfield/Jasper",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1125,
		Latitude:  44.0032997131,
		Longitude: -122.8799972534,
		Timezone:  "America/Los_Angeles",
	},
	"36TN": {
		ICAO:      "36TN",
		Name:      "Ruckman Field",
		City:      "Clarksville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 660,
		Latitude:  36.4770011902,
		Longitude: -87.1398010254,
		Timezone:  "America/Chicago",
	},
	"36TX": {
		ICAO:      "36TX",
		Name:      "Bevoni-Flying B Airport",
		City:      "Whitesboro",
		State:     "Texas",
		Country:   "US",
		Elevation: 763,
		Latitude:  33.7209014893,
		Longitude: -96.8613967896,
		Timezone:  "America/Chicago",
	},
	"36WA": {
		ICAO:      "36WA",
		Name:      "Bob's Field",
		City:      "Trout Lake",
		State:     "Washington",
		Country:   "US",
		Elevation: 1650,
		Latitude:  45.956401825,
		Longitude: -121.4980010986,
		Timezone:  "America/Los_Angeles",
	},
	"36WI": {
		ICAO:      "36WI",
		Name:      "Holland Air Park",
		City:      "Holmen",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 730,
		Latitude:  44.0321998596,
		Longitude: -91.2995986938,
		Timezone:  "America/Chicago",
	},
	"36XS": {
		ICAO:      "36XS",
		Name:      "Flying F Airport",
		City:      "Nocona",
		State:     "Texas",
		Country:   "US",
		Elevation: 890,
		Latitude:  33.8777008057,
		Longitude: -97.7297973633,
		Timezone:  "America/Chicago",
	},
	"37AR": {
		ICAO:      "37AR",
		Name:      "Beech Creek Airport",
		City:      "Hamburg",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 175,
		Latitude:  33.1824989319,
		Longitude: -91.6283035278,
		Timezone:  "America/Chicago",
	},
	"37AZ": {
		ICAO:      "37AZ",
		Name:      "Sarita Airport",
		City:      "Coolidge",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1464,
		Latitude:  32.9356002808,
		Longitude: -111.4850006104,
		Timezone:  "America/Phoenix",
	},
	"37CA": {
		ICAO:      "37CA",
		Name:      "Billy Joe Airport",
		City:      "Temecula",
		State:     "California",
		Country:   "US",
		Elevation: 1350,
		Latitude:  33.5069999695,
		Longitude: -117.0709991455,
		Timezone:  "America/Los_Angeles",
	},
	"37CL": {
		ICAO:      "37CL",
		Name:      "Lyall-Roberts Airport",
		City:      "Escondido",
		State:     "California",
		Country:   "US",
		Elevation: 925,
		Latitude:  33.3245010376,
		Longitude: -116.9899978638,
		Timezone:  "America/Los_Angeles",
	},
	"37CN": {
		ICAO:      "37CN",
		Name:      "Arnold Ranch Airport",
		City:      "Fresno",
		State:     "California",
		Country:   "US",
		Elevation: 382,
		Latitude:  36.9143981934,
		Longitude: -119.7860031128,
		Timezone:  "America/Los_Angeles",
	},
	"37FL": {
		ICAO:      "37FL",
		Name:      "Flying Harness Farms Airport",
		City:      "Bell",
		State:     "Florida",
		Country:   "US",
		Elevation: 78,
		Latitude:  29.7185993195,
		Longitude: -82.8445968628,
		Timezone:  "America/New_York",
	},
	"37GA": {
		ICAO:      "37GA",
		Name:      "Blue Bird Field",
		City:      "Crandall",
		State:     "Georgia",
		Country:   "US",
		Elevation: 800,
		Latitude:  34.9003982544,
		Longitude: -84.778503418,
		Timezone:  "America/New_York",
	},
	"37ID": {
		ICAO:      "37ID",
		Name:      "Hungry Ridge Ranch Airport",
		City:      "Grangeville",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4599,
		Latitude:  45.782417,
		Longitude: -115.938,
		Timezone:  "America/Los_Angeles",
	},
	"37II": {
		ICAO:      "37II",
		Name:      "Winn Field",
		City:      "Culver",
		State:     "Indiana",
		Country:   "US",
		Elevation: 808,
		Latitude:  41.1755981445,
		Longitude: -86.3735961914,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"37IL": {
		ICAO:      "37IL",
		Name:      "Minder Airport",
		City:      "Lincoln",
		State:     "Illinois",
		Country:   "US",
		Elevation: 600,
		Latitude:  40.1333999634,
		Longitude: -89.3251037598,
		Timezone:  "America/Chicago",
	},
	"37IN": {
		ICAO:      "37IN",
		Name:      "Felix Airport",
		City:      "Stockwell",
		State:     "Indiana",
		Country:   "US",
		Elevation: 750,
		Latitude:  40.2916984558,
		Longitude: -86.7802963257,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"37IS": {
		ICAO:      "37IS",
		Name:      "Hilbert Airport",
		City:      "Marengo",
		State:     "Illinois",
		Country:   "US",
		Elevation: 890,
		Latitude:  42.3321990967,
		Longitude: -88.6055984497,
		Timezone:  "America/Chicago",
	},
	"37KS": {
		ICAO:      "37KS",
		Name:      "Bar P Ranch Airport",
		City:      "Howard",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1060,
		Latitude:  37.4819984436,
		Longitude: -96.3377990723,
		Timezone:  "America/Chicago",
	},
	"37LA": {
		ICAO:      "37LA",
		Name:      "Wayne Brown Airport",
		City:      "Clayton",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 59,
		Latitude:  31.7276992798,
		Longitude: -91.582901001,
		Timezone:  "America/Chicago",
	},
	"37LL": {
		ICAO:      "37LL",
		Name:      "Dale Curten Farm Airport",
		City:      "Modoc",
		State:     "Illinois",
		Country:   "US",
		Elevation: 615,
		Latitude:  38.0477981567,
		Longitude: -89.9971008301,
		Timezone:  "America/Chicago",
	},
	"37MI": {
		ICAO:      "37MI",
		Name:      "Handleman Sky Ranch Airport",
		City:      "Oxford",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1150,
		Latitude:  42.8527984619,
		Longitude: -83.2276992798,
		Timezone:  "America/Detroit",
	},
	"37MN": {
		ICAO:      "37MN",
		Name:      "Nagel and Schultz Airport",
		City:      "Belle Plaine",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 970,
		Latitude:  44.5821990967,
		Longitude: -93.8015975952,
		Timezone:  "America/Chicago",
	},
	"37MO": {
		ICAO:      "37MO",
		Name:      "Mayes Homestead Airport",
		City:      "Polo",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1000,
		Latitude:  39.5750007629,
		Longitude: -94.1836013794,
		Timezone:  "America/Chicago",
	},
	"37ND": {
		ICAO:      "37ND",
		Name:      "Sunset Strip",
		City:      "Dunn Center",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 2360,
		Latitude:  47.2319984436,
		Longitude: -102.5090026855,
		Timezone:  "America/Denver",
	},
	"37NE": {
		ICAO:      "37NE",
		Name:      "Watermeier Airport",
		City:      "Burr",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1100,
		Latitude:  40.60779953,
		Longitude: -96.307800293,
		Timezone:  "America/Chicago",
	},
	"37OI": {
		ICAO:      "37OI",
		Name:      "Cole Airfield",
		City:      "Crestline",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1100,
		Latitude:  40.8250999451,
		Longitude: -82.7967987061,
		Timezone:  "America/New_York",
	},
	"37OR": {
		ICAO:      "37OR",
		Name:      "Vey Sheep Ranch Airport",
		City:      "Starkey",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4158,
		Latitude:  45.0932006836,
		Longitude: -118.3990020752,
		Timezone:  "America/Los_Angeles",
	},
	"37PA": {
		ICAO:      "37PA",
		Name:      "Roadcap Airport",
		City:      "Middleburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 731,
		Latitude:  40.7883987427,
		Longitude: -77.1061019897,
		Timezone:  "America/New_York",
	},
	"37TE": {
		ICAO:      "37TE",
		Name:      "Johnnie Volk Field",
		City:      "Hitchcock",
		State:     "Texas",
		Country:   "US",
		Elevation: 13,
		Latitude:  29.360200882,
		Longitude: -95.0099029541,
		Timezone:  "America/Chicago",
	},
	"37TS": {
		ICAO:      "37TS",
		Name:      "Skinner Airport",
		City:      "Greenville",
		State:     "Texas",
		Country:   "US",
		Elevation: 460,
		Latitude:  32.9906997681,
		Longitude: -96.0400009155,
		Timezone:  "America/Chicago",
	},
	"37TX": {
		ICAO:      "37TX",
		Name:      "Yates Field",
		City:      "San Saba",
		State:     "Texas",
		Country:   "US",
		Elevation: 1400,
		Latitude:  31.3215999603,
		Longitude: -98.8656005859,
		Timezone:  "America/Chicago",
	},
	"37VT": {
		ICAO:      "37VT",
		Name:      "Brisson Airport",
		City:      "Shoreham",
		State:     "Vermont",
		Country:   "US",
		Elevation: 320,
		Latitude:  43.868598938,
		Longitude: -73.3180999756,
		Timezone:  "America/New_York",
	},
	"37WA": {
		ICAO:      "37WA",
		Name:      "Baumann Farm Inc. Airport",
		City:      "Washtucna",
		State:     "Washington",
		Country:   "US",
		Elevation: 1600,
		Latitude:  46.8317985535,
		Longitude: -118.4619979858,
		Timezone:  "America/Los_Angeles",
	},
	"37WI": {
		ICAO:      "37WI",
		Name:      "Docken Field",
		City:      "Mount Horeb",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1230,
		Latitude:  42.9911003113,
		Longitude: -89.7535018921,
		Timezone:  "America/Chicago",
	},
	"37XS": {
		ICAO:      "37XS",
		Name:      "Lake Whitney Country Club Airport",
		City:      "Whitney",
		State:     "Texas",
		Country:   "US",
		Elevation: 627,
		Latitude:  32.0001983643,
		Longitude: -97.4169998169,
		Timezone:  "America/Chicago",
	},
	"38AK": {
		ICAO:      "38AK",
		Name:      "Mels Airport",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 340,
		Latitude:  61.5625,
		Longitude: -149.6679992676,
		Timezone:  "America/Anchorage",
	},
	"38AR": {
		ICAO:      "38AR",
		Name:      "Williams Field",
		City:      "Siloam Springs",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1150,
		Latitude:  36.1188011169,
		Longitude: -94.4682006836,
		Timezone:  "America/Chicago",
	},
	"38AZ": {
		ICAO:      "38AZ",
		Name:      "Peabody Bedard Field",
		City:      "Kayenta",
		State:     "Arizona",
		Country:   "US",
		Elevation: 6564,
		Latitude:  36.471698761,
		Longitude: -110.4179992676,
		Timezone:  "America/Denver",
	},
	"38CA": {
		ICAO:      "38CA",
		Name:      "Cashen Airport",
		City:      "Wasco",
		State:     "California",
		Country:   "US",
		Elevation: 225,
		Latitude:  35.6916007996,
		Longitude: -119.4889984131,
		Timezone:  "America/Los_Angeles",
	},
	"38CL": {
		ICAO:      "38CL",
		Name:      "Riego Flight Strip",
		City:      "Verona",
		State:     "California",
		Country:   "US",
		Elevation: 21,
		Latitude:  38.7541007996,
		Longitude: -121.56300354,
		Timezone:  "America/Los_Angeles",
	},
	"38CN": {
		ICAO:      "38CN",
		Name:      "Sanborn Airport",
		City:      "Meridian",
		State:     "California",
		Country:   "US",
		Elevation: 36,
		Latitude:  39.1092987061,
		Longitude: -121.8840026855,
		Timezone:  "America/Los_Angeles",
	},
	"38FA": {
		ICAO:      "38FA",
		Name:      "Blue Springs Airport",
		City:      "Madison",
		State:     "Florida",
		Country:   "US",
		Elevation: 95,
		Latitude:  30.483499527,
		Longitude: -83.2499008179,
		Timezone:  "America/New_York",
	},
	"38ID": {
		ICAO:      "38ID",
		Name:      "Sky Ranch North Airport",
		City:      "Nampa",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2750,
		Latitude:  43.5094444444,
		Longitude: -116.667916667,
		Timezone:  "America/Boise",
	},
	"38II": {
		ICAO:      "38II",
		Name:      "Hampton Field",
		City:      "Greencastle",
		State:     "Indiana",
		Country:   "US",
		Elevation: 895,
		Latitude:  39.728611,
		Longitude: -86.823056,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"38IN": {
		ICAO:      "38IN",
		Name:      "Fuller Field",
		City:      "Brownsburg",
		State:     "Indiana",
		Country:   "US",
		Elevation: 910,
		Latitude:  39.9020004272,
		Longitude: -86.3630981445,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"38IS": {
		ICAO:      "38IS",
		Name:      "Winchester Airport",
		City:      "Nauvoo",
		State:     "Illinois",
		Country:   "US",
		Elevation: 685,
		Latitude:  40.5820007324,
		Longitude: -91.3584976196,
		Timezone:  "America/Chicago",
	},
	"38KS": {
		ICAO:      "38KS",
		Name:      "Savute Airport",
		City:      "Kechi",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1400,
		Latitude:  37.7778015137,
		Longitude: -97.2774963379,
		Timezone:  "America/Chicago",
	},
	"38LA": {
		ICAO:      "38LA",
		Name:      "Whiteville Airport",
		City:      "Ville Platte",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 43,
		Latitude:  30.7844009399,
		Longitude: -92.1822967529,
		Timezone:  "America/Chicago",
	},
	"38LS": {
		ICAO:      "38LS",
		Name:      "Ace Flying Airport",
		City:      "Parks",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 23,
		Latitude:  30.236700058,
		Longitude: -91.8296966553,
		Timezone:  "America/Chicago",
	},
	"38MI": {
		ICAO:      "38MI",
		Name:      "Chicora Field",
		City:      "Allegan",
		State:     "Michigan",
		Country:   "US",
		Elevation: 700,
		Latitude:  42.4668998718,
		Longitude: -85.9792022705,
		Timezone:  "America/Detroit",
	},
	"38MN": {
		ICAO:      "38MN",
		Name:      "Chandler Personal Use Airport",
		City:      "Bemidji",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1380,
		Latitude:  47.4110984802,
		Longitude: -94.7724990845,
		Timezone:  "America/Chicago",
	},
	"38MO": {
		ICAO:      "38MO",
		Name:      "Barber Airport",
		City:      "Annada",
		State:     "Missouri",
		Country:   "US",
		Elevation: 480,
		Latitude:  39.2555999756,
		Longitude: -90.8556976318,
		Timezone:  "America/Chicago",
	},
	"38ND": {
		ICAO:      "38ND",
		Name:      "Schroeder Private Airport",
		City:      "Erie",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1140,
		Latitude:  47.0979995728,
		Longitude: -97.4011993408,
		Timezone:  "America/Chicago",
	},
	"38NE": {
		ICAO:      "38NE",
		Name:      "Boardman Aerial Airport",
		City:      "Henderson",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1720,
		Latitude:  40.7536010742,
		Longitude: -97.8514022827,
		Timezone:  "America/Chicago",
	},
	"38NY": {
		ICAO:      "38NY",
		Name:      "Greenlawn Farm Airport",
		City:      "Dundee",
		State:     "New York",
		Country:   "US",
		Elevation: 1010,
		Latitude:  42.5486984253,
		Longitude: -76.9580001831,
		Timezone:  "America/New_York",
	},
	"38OH": {
		ICAO:      "38OH",
		Name:      "Industry Air Park",
		City:      "Baltimore",
		State:     "Ohio",
		Country:   "US",
		Elevation: 877,
		Latitude:  39.8447990417,
		Longitude: -82.5795974731,
		Timezone:  "America/New_York",
	},
	"38OI": {
		ICAO:      "38OI",
		Name:      "Ronshausen Airport",
		City:      "Darbyville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 750,
		Latitude:  39.6851005554,
		Longitude: -83.0718994141,
		Timezone:  "America/New_York",
	},
	"38PA": {
		ICAO:      "38PA",
		Name:      "Aerequus Airport",
		City:      "Solebury",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 390,
		Latitude:  40.3908996582,
		Longitude: -75.0140991211,
		Timezone:  "America/New_York",
	},
	"38PN": {
		ICAO:      "38PN",
		Name:      "Waisley Airport",
		City:      "Mc Kean",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 950,
		Latitude:  42.0000991821,
		Longitude: -80.1664962769,
		Timezone:  "America/New_York",
	},
	"38SC": {
		ICAO:      "38SC",
		Name:      "La Dolce Terra Airport",
		City:      "Gaffney",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 890,
		Latitude:  35.177778,
		Longitude: -81.747778,
		Timezone:  "America/New_York",
	},
	"38TN": {
		ICAO:      "38TN",
		Name:      "Sugar Grove Airport",
		City:      "Westmoreland",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 800,
		Latitude:  36.6253013611,
		Longitude: -86.2733001709,
		Timezone:  "America/Chicago",
	},
	"38WA": {
		ICAO:      "38WA",
		IATA:      "BYW",
		Name:      "Blakely Island Airport",
		City:      "Blakely Island",
		State:     "Washington",
		Country:   "US",
		Elevation: 66,
		Latitude:  48.5789985657,
		Longitude: -122.825996399,
		Timezone:  "America/Los_Angeles",
	},
	"38WI": {
		ICAO:      "38WI",
		Name:      "Northport Airport",
		City:      "Royalton",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 800,
		Latitude:  44.3885993958,
		Longitude: -88.8526000977,
		Timezone:  "America/Chicago",
	},
	"38WV": {
		ICAO:      "38WV",
		Name:      "River's Edge Farm Airport",
		City:      "Capon Bridge",
		State:     "West Virginia",
		Country:   "US",
		Elevation: 810,
		Latitude:  39.3252983093,
		Longitude: -78.4255981445,
		Timezone:  "America/New_York",
	},
	"38XS": {
		ICAO:      "38XS",
		Name:      "San Rafael Ranch Airport",
		City:      "Santa Elena",
		State:     "Texas",
		Country:   "US",
		Elevation: 312,
		Latitude:  26.8152008057,
		Longitude: -98.4793014526,
		Timezone:  "America/Chicago",
	},
	"39AK": {
		ICAO:      "39AK",
		Name:      "Gilmore Strip",
		City:      "Palmer",
		State:     "Alaska",
		Country:   "US",
		Elevation: 340,
		Latitude:  61.5859985352,
		Longitude: -149.2359924316,
		Timezone:  "America/Anchorage",
	},
	"39AR": {
		ICAO:      "39AR",
		Name:      "Twin Cities Airport",
		City:      "Van Buren",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 410,
		Latitude:  35.4171981812,
		Longitude: -94.3264007568,
		Timezone:  "America/Chicago",
	},
	"39AZ": {
		ICAO:      "39AZ",
		Name:      "Lukachukai Airport",
		City:      "Lukachukai",
		State:     "Arizona",
		Country:   "US",
		Elevation: 6433,
		Latitude:  36.4006004333,
		Longitude: -109.2570037842,
		Timezone:  "America/Denver",
	},
	"39CO": {
		ICAO:      "39CO",
		Name:      "Flying M Ranch Airport",
		City:      "Montrose",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7200,
		Latitude:  38.4879989624,
		Longitude: -107.6740036011,
		Timezone:  "America/Denver",
	},
	"39FA": {
		ICAO:      "39FA",
		Name:      "Sanders Ranch Airport",
		City:      "Melrose",
		State:     "Florida",
		Country:   "US",
		Elevation: 150,
		Latitude:  29.6625003815,
		Longitude: -82.0093002319,
		Timezone:  "America/New_York",
	},
	"39FD": {
		ICAO:      "39FD",
		Name:      "Ancient Oaks Airport",
		City:      "Steinhatchee",
		State:     "Florida",
		Country:   "US",
		Elevation: 26,
		Latitude:  29.7294006348,
		Longitude: -83.3535003662,
		Timezone:  "America/New_York",
	},
	"39IA": {
		ICAO:      "39IA",
		Name:      "Husband Field",
		City:      "Dallas Center",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1010,
		Latitude:  41.6794013977,
		Longitude: -94.0202026367,
		Timezone:  "America/Chicago",
	},
	"39ID": {
		ICAO:      "39ID",
		Name:      "Albion Municipal Airport",
		City:      "Albion",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4777,
		Latitude:  42.3977012634,
		Longitude: -113.5579986572,
		Timezone:  "America/Boise",
	},
	"39II": {
		ICAO:      "39II",
		Name:      "Miller Field",
		City:      "Topeka",
		State:     "Indiana",
		Country:   "US",
		Elevation: 957,
		Latitude:  41.5148010254,
		Longitude: -85.5522003174,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"39IL": {
		ICAO:      "39IL",
		Name:      "Mason Airport",
		City:      "Lincoln",
		State:     "Illinois",
		Country:   "US",
		Elevation: 591,
		Latitude:  40.0938987732,
		Longitude: -89.3628997803,
		Timezone:  "America/Chicago",
	},
	"39IN": {
		ICAO:      "39IN",
		Name:      "Roberts Field",
		City:      "Sulphur Springs",
		State:     "Indiana",
		Country:   "US",
		Elevation: 1085,
		Latitude:  40.0023002625,
		Longitude: -85.4289016724,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"39IS": {
		ICAO:      "39IS",
		Name:      "Hagi Landing Area Airport",
		City:      "Streator",
		State:     "Illinois",
		Country:   "US",
		Elevation: 620,
		Latitude:  41.1100006104,
		Longitude: -88.7956008911,
		Timezone:  "America/Chicago",
	},
	"39KS": {
		ICAO:      "39KS",
		Name:      "Rolling Meadows Airport",
		City:      "Worden",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1050,
		Latitude:  38.7860984802,
		Longitude: -95.3075027466,
		Timezone:  "America/Chicago",
	},
	"39LA": {
		ICAO:      "39LA",
		Name:      "Whitaker Ultralightport",
		City:      "Chataignier",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 65,
		Latitude:  30.58833333,
		Longitude: -92.23055556,
		Timezone:  "America/Chicago",
	},
	"39LL": {
		ICAO:      "39LL",
		Name:      "Sullivan Airport",
		City:      "Hamilton",
		State:     "Illinois",
		Country:   "US",
		Elevation: 635,
		Latitude:  40.401389,
		Longitude: -91.303056,
		Timezone:  "America/Chicago",
	},
	"39MI": {
		ICAO:      "39MI",
		Name:      "Huber Airport",
		City:      "Crystal Falls",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1418,
		Latitude:  46.168800354,
		Longitude: -88.1724014282,
		Timezone:  "America/Menominee",
	},
	"39MN": {
		ICAO:      "39MN",
		Name:      "Anthony Private Airport",
		City:      "Two Harbors",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1000,
		Latitude:  47.0368995667,
		Longitude: -91.7105026245,
		Timezone:  "America/Chicago",
	},
	"39MO": {
		ICAO:      "39MO",
		Name:      "Gardner Airport",
		City:      "Springfield",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1325,
		Latitude:  37.1186981201,
		Longitude: -93.2049026489,
		Timezone:  "America/Chicago",
	},
	"39OH": {
		ICAO:      "39OH",
		Name:      "Crosswind Meadows Airport",
		City:      "Minerva",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1160,
		Latitude:  40.8123016357,
		Longitude: -81.0550994873,
		Timezone:  "America/New_York",
	},
	"39OK": {
		ICAO:      "39OK",
		Name:      "Paradise Air Haven Airport",
		City:      "Goldsby",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1205,
		Latitude:  35.1114997864,
		Longitude: -97.4793014526,
		Timezone:  "America/Chicago",
	},
	"39PA": {
		ICAO:      "39PA",
		Name:      "Gehris Airport",
		City:      "Souderton",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 570,
		Latitude:  40.3172988892,
		Longitude: -75.2904968262,
		Timezone:  "America/New_York",
	},
	"39PN": {
		ICAO:      "39PN",
		Name:      "Nelson's Run Airport",
		City:      "Mercer",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1305,
		Latitude:  41.2159004211,
		Longitude: -80.1976013184,
		Timezone:  "America/New_York",
	},
	"39TA": {
		ICAO:      "39TA",
		Name:      "Flying Tigers Airport",
		City:      "Paris",
		State:     "Texas",
		Country:   "US",
		Elevation: 588,
		Latitude:  33.6525993347,
		Longitude: -95.6557998657,
		Timezone:  "America/Chicago",
	},
	"39TE": {
		ICAO:      "39TE",
		Name:      "Seminole Spraying Service Airport",
		City:      "Seminole",
		State:     "Texas",
		Country:   "US",
		Elevation: 3372,
		Latitude:  32.7182006836,
		Longitude: -102.737998962,
		Timezone:  "America/Chicago",
	},
	"39TN": {
		ICAO:      "39TN",
		Name:      "Big Sandy Airpark",
		City:      "Big Sandy",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 632,
		Latitude:  36.279722,
		Longitude: -88.043611,
		Timezone:  "America/Chicago",
	},
	"39WA": {
		ICAO:      "39WA",
		Name:      "Tailskid Ranch Airport",
		City:      "Tum Tum",
		State:     "Washington",
		Country:   "US",
		Elevation: 1910,
		Latitude:  47.8518981934,
		Longitude: -117.7819976807,
		Timezone:  "America/Los_Angeles",
	},
	"39WI": {
		ICAO:      "39WI",
		Name:      "S & S Ranch Airport",
		City:      "Viola",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1200,
		Latitude:  43.4846992493,
		Longitude: -90.6199035645,
		Timezone:  "America/Chicago",
	},
	"3AK1": {
		ICAO:      "3AK1",
		Name:      "Deshka Landing Airport",
		City:      "Willow",
		State:     "Alaska",
		Country:   "US",
		Elevation: 110,
		Latitude:  61.7187995911,
		Longitude: -150.1959991455,
		Timezone:  "America/Anchorage",
	},
	"3AK2": {
		ICAO:      "3AK2",
		Name:      "Niklason Lake Estates Airport",
		City:      "Palmer",
		State:     "Alaska",
		Country:   "US",
		Elevation: 410,
		Latitude:  61.6274986267,
		Longitude: -149.2870025635,
		Timezone:  "America/Anchorage",
	},
	"3AK3": {
		ICAO:      "3AK3",
		Name:      "Songlo Vista Airport",
		City:      "Talkeetna",
		State:     "Alaska",
		Country:   "US",
		Elevation: 825,
		Latitude:  62.5638008118,
		Longitude: -150.220993042,
		Timezone:  "America/Anchorage",
	},
	"3AK4": {
		ICAO:      "3AK4",
		Name:      "Johnson Airport",
		City:      "Kenai",
		State:     "Alaska",
		Country:   "US",
		Elevation: 102,
		Latitude:  60.6324996948,
		Longitude: -151.3399963379,
		Timezone:  "America/Anchorage",
	},
	"3AK5": {
		ICAO:      "3AK5",
		Name:      "Drift River Airport",
		City:      "Kenai",
		State:     "Alaska",
		Country:   "US",
		Elevation: 30,
		Latitude:  60.5889015198,
		Longitude: -152.1620025635,
		Timezone:  "America/Anchorage",
	},
	"3AK6": {
		ICAO:      "3AK6",
		Name:      "B & B Boys Ranch Airport",
		City:      "Palmer",
		State:     "Alaska",
		Country:   "US",
		Elevation: 300,
		Latitude:  61.5928993225,
		Longitude: -149.3020019531,
		Timezone:  "America/Anchorage",
	},
	"3AK7": {
		ICAO:      "3AK7",
		Name:      "Laub Airport",
		City:      "Willow",
		State:     "Alaska",
		Country:   "US",
		Elevation: 100,
		Latitude:  61.7655982971,
		Longitude: -150.3390045166,
		Timezone:  "America/Anchorage",
	},
	"3AK8": {
		ICAO:      "3AK8",
		Name:      "Boisselle's Strip",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 400,
		Latitude:  61.6624984741,
		Longitude: -149.391998291,
		Timezone:  "America/Anchorage",
	},
	"3AK9": {
		ICAO:      "3AK9",
		Name:      "River John Airport",
		City:      "Skwentna",
		State:     "Alaska",
		Country:   "US",
		Elevation: 170,
		Latitude:  61.941944,
		Longitude: -151.036111,
		Timezone:  "America/Anchorage",
	},
	"3AL1": {
		ICAO:      "3AL1",
		Name:      "Flying H Ranch Airport",
		City:      "Hartford",
		State:     "Alabama",
		Country:   "US",
		Elevation: 264,
		Latitude:  31.0503997803,
		Longitude: -85.6853027344,
		Timezone:  "America/Chicago",
	},
	"3AL5": {
		ICAO:      "3AL5",
		Name:      "Edwards Farm Airport",
		City:      "Jacksonville",
		State:     "Alabama",
		Country:   "US",
		Elevation: 750,
		Latitude:  33.8353004456,
		Longitude: -85.7639007568,
		Timezone:  "America/Chicago",
	},
	"3AL6": {
		ICAO:      "3AL6",
		Name:      "Town & Country Airpark",
		City:      "Branchville",
		State:     "Alabama",
		Country:   "US",
		Elevation: 730,
		Latitude:  33.4901008606,
		Longitude: -86.4124984741,
		Timezone:  "America/Chicago",
	},
	"3AL7": {
		ICAO:      "3AL7",
		Name:      "Flowers Field",
		City:      "Thomaston",
		State:     "Alabama",
		Country:   "US",
		Elevation: 280,
		Latitude:  32.2849998474,
		Longitude: -87.5199966431,
		Timezone:  "America/Chicago",
	},
	"3AL8": {
		ICAO:      "3AL8",
		Name:      "Flint River Ranch Airport",
		City:      "Owens Crossroads",
		State:     "Alabama",
		Country:   "US",
		Elevation: 590,
		Latitude:  34.6192016602,
		Longitude: -86.4669036865,
		Timezone:  "America/Chicago",
	},
	"3AR0": {
		ICAO:      "3AR0",
		Name:      "Frost Flying Inc Airport",
		City:      "Marianna",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 240,
		Latitude:  34.8306007385,
		Longitude: -90.853302002,
		Timezone:  "America/Chicago",
	},
	"3AR2": {
		ICAO:      "3AR2",
		Name:      "Ridgeway Field",
		City:      "Harrison",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1400,
		Latitude:  36.320098877,
		Longitude: -93.2015991211,
		Timezone:  "America/Chicago",
	},
	"3AR3": {
		ICAO:      "3AR3",
		Name:      "Cypress Creek Airport",
		City:      "Cabot",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 280,
		Latitude:  35.0596008301,
		Longitude: -92.0820007324,
		Timezone:  "America/Chicago",
	},
	"3AR4": {
		ICAO:      "3AR4",
		Name:      "Four Mile Creek Ranch Airport",
		City:      "Cabot",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 500,
		Latitude:  35.0056991577,
		Longitude: -92.0774002075,
		Timezone:  "America/Chicago",
	},
	"3AR5": {
		ICAO:      "3AR5",
		Name:      "Tripp Strip",
		City:      "Griffithville",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 210,
		Latitude:  35.1179008484,
		Longitude: -91.6119003296,
		Timezone:  "America/Chicago",
	},
	"3AR6": {
		ICAO:      "3AR6",
		Name:      "Crystal Ridge Airport",
		City:      "Little Rock",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 500,
		Latitude:  34.7285995483,
		Longitude: -92.5164031982,
		Timezone:  "America/Chicago",
	},
	"3AR7": {
		ICAO:      "3AR7",
		Name:      "Taylor Field",
		City:      "Lowell",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1315,
		Latitude:  36.2877998352,
		Longitude: -94.1866989136,
		Timezone:  "America/Chicago",
	},
	"3AR8": {
		ICAO:      "3AR8",
		Name:      "River Acres Airport",
		City:      "Heber Springs",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 260,
		Latitude:  35.4667015076,
		Longitude: -91.966796875,
		Timezone:  "America/Chicago",
	},
	"3AR9": {
		ICAO:      "3AR9",
		Name:      "Quinn Field",
		City:      "Paragould",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 240,
		Latitude:  36.0519981384,
		Longitude: -90.3583984375,
		Timezone:  "America/Chicago",
	},
	"3AZ2": {
		ICAO:      "3AZ2",
		Name:      "U of A Maricopa Ag Center Airport",
		City:      "Maricopa",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1181,
		Latitude:  33.0802993774,
		Longitude: -111.983001709,
		Timezone:  "America/Phoenix",
	},
	"3AZ5": {
		ICAO:      "3AZ5",
		Name:      "Hualapai Airport",
		City:      "Peach Springs",
		State:     "Arizona",
		Country:   "US",
		Elevation: 5317,
		Latitude:  35.5722007751,
		Longitude: -113.2919998169,
		Timezone:  "America/Phoenix",
	},
	"3AZ8": {
		ICAO:      "3AZ8",
		Name:      "High Mesa Airpark",
		City:      "Solomon",
		State:     "Arizona",
		Country:   "US",
		Elevation: 3080,
		Latitude:  32.762298584,
		Longitude: -109.6529998779,
		Timezone:  "America/Phoenix",
	},
	"3CA3": {
		ICAO:      "3CA3",
		Name:      "Dixon Airport",
		City:      "Cuyama",
		State:     "California",
		Country:   "US",
		Elevation: 2440,
		Latitude:  34.9221992493,
		Longitude: -119.5260009766,
		Timezone:  "America/Los_Angeles",
	},
	"3CA5": {
		ICAO:      "3CA5",
		Name:      "Haws Airport",
		City:      "Madera",
		State:     "California",
		Country:   "US",
		Elevation: 192,
		Latitude:  36.9416007996,
		Longitude: -120.2429962158,
		Timezone:  "America/Los_Angeles",
	},
	"3CA7": {
		ICAO:      "3CA7",
		Name:      "Metz Airport",
		City:      "Greenfield",
		State:     "California",
		Country:   "US",
		Elevation: 240,
		Latitude:  36.3283004761,
		Longitude: -121.18699646,
		Timezone:  "America/Los_Angeles",
	},
	"3CA9": {
		ICAO:      "3CA9",
		Name:      "Clark Ranch Airport",
		City:      "Soledad",
		State:     "California",
		Country:   "US",
		Elevation: 276,
		Latitude:  36.3633003235,
		Longitude: -121.3099975586,
		Timezone:  "America/Los_Angeles",
	},
	"3CL2": {
		ICAO:      "3CL2",
		Name:      "Meadow Airstrip",
		City:      "Red Bluff",
		State:     "California",
		Country:   "US",
		Elevation: 345,
		Latitude:  40.1563987732,
		Longitude: -122.3020019531,
		Timezone:  "America/Los_Angeles",
	},
	"3CO0": {
		ICAO:      "3CO0",
		Name:      "Sky Island Ranch Airport",
		City:      "Delta",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5300,
		Latitude:  38.7374992371,
		Longitude: -108.0059967041,
		Timezone:  "America/Denver",
	},
	"3CO1": {
		ICAO:      "3CO1",
		Name:      "Cridler Field",
		City:      "Hotchkiss",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6460,
		Latitude:  38.8483009338,
		Longitude: -107.8330001831,
		Timezone:  "America/Denver",
	},
	"3CO2": {
		ICAO:      "3CO2",
		Name:      "Mertens Airport",
		City:      "Sterling",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4192,
		Latitude:  40.6175003052,
		Longitude: -103.3330001831,
		Timezone:  "America/Denver",
	},
	"3CO4": {
		ICAO:      "3CO4",
		Name:      "Tercio Ranch Airstrip",
		City:      "Stonewall",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7957,
		Latitude:  37.0708999634,
		Longitude: -105.0189971924,
		Timezone:  "America/Denver",
	},
	"3CO7": {
		ICAO:      "3CO7",
		Name:      "Dietrichs Airport",
		City:      "Elizabeth",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6780,
		Latitude:  39.3325004578,
		Longitude: -104.5660018921,
		Timezone:  "America/Denver",
	},
	"3FD0": {
		ICAO:      "3FD0",
		Name:      "Last Chance Ranch Airport",
		City:      "Lake Placid",
		State:     "Florida",
		Country:   "US",
		Elevation: 70,
		Latitude:  27.0216999054,
		Longitude: -81.4512023926,
		Timezone:  "America/New_York",
	},
	"3FD4": {
		ICAO:      "3FD4",
		Name:      "Florida Flying Gators Ultralightport",
		City:      "Minneola",
		State:     "Florida",
		Country:   "US",
		Elevation: 30,
		Latitude:  28.6277999878,
		Longitude: -81.8029022217,
		Timezone:  "America/New_York",
	},
	"3FD6": {
		ICAO:      "3FD6",
		Name:      "Tradewinds Aerodrome",
		City:      "Scottsmoor",
		State:     "Florida",
		Country:   "US",
		Elevation: 5,
		Latitude:  28.7642002106,
		Longitude: -80.8538970947,
		Timezone:  "America/New_York",
	},
	"3FL0": {
		ICAO:      "3FL0",
		Name:      "Mount Royal Airport",
		City:      "Welaka",
		State:     "Florida",
		Country:   "US",
		Elevation: 60,
		Latitude:  29.4361000061,
		Longitude: -81.6567001343,
		Timezone:  "America/New_York",
	},
	"3FL1": {
		ICAO:      "3FL1",
		Name:      "Two J's Flying Ranch Airport",
		City:      "Dade City",
		State:     "Florida",
		Country:   "US",
		Elevation: 60,
		Latitude:  28.4524993896,
		Longitude: -82.2078018188,
		Timezone:  "America/New_York",
	},
	"3FL5": {
		ICAO:      "3FL5",
		Name:      "Mills Ranch South Airport",
		City:      "Kenansville",
		State:     "Florida",
		Country:   "US",
		Elevation: 60,
		Latitude:  27.7782993317,
		Longitude: -80.9218978882,
		Timezone:  "America/New_York",
	},
	"3FL8": {
		ICAO:      "3FL8",
		Name:      "Hart Airport",
		City:      "Malone",
		State:     "Florida",
		Country:   "US",
		Elevation: 125,
		Latitude:  30.9463005066,
		Longitude: -85.2738037109,
		Timezone:  "America/Chicago",
	},
	"3GA0": {
		ICAO:      "3GA0",
		Name:      "Grant Airport",
		City:      "Stockbridge",
		State:     "Georgia",
		Country:   "US",
		Elevation: 790,
		Latitude:  33.5275001526,
		Longitude: -84.1568984985,
		Timezone:  "America/New_York",
	},
	"3GA1": {
		ICAO:      "3GA1",
		Name:      "Prattsburg Airport",
		City:      "Talbotton",
		State:     "Georgia",
		Country:   "US",
		Elevation: 554,
		Latitude:  32.7251014709,
		Longitude: -84.3582992554,
		Timezone:  "America/New_York",
	},
	"3GA3": {
		ICAO:      "3GA3",
		Name:      "Warren Field",
		City:      "Lookout Mountain",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1866,
		Latitude:  34.871111,
		Longitude: -85.4375,
		Timezone:  "America/New_York",
	},
	"3GA5": {
		ICAO:      "3GA5",
		Name:      "Diamond R Ranch Airport",
		City:      "Villa Rica",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1213,
		Latitude:  33.6371002197,
		Longitude: -84.9387969971,
		Timezone:  "America/New_York",
	},
	"3GA6": {
		ICAO:      "3GA6",
		Name:      "Erlen Airport",
		City:      "Whitesburg",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1044,
		Latitude:  33.5705986023,
		Longitude: -84.8983001709,
		Timezone:  "America/New_York",
	},
	"3GA7": {
		ICAO:      "3GA7",
		Name:      "Rhodes Air Ranch Airport",
		City:      "Waynesboro",
		State:     "Georgia",
		Country:   "US",
		Elevation: 195,
		Latitude:  33.1624984741,
		Longitude: -81.7683029175,
		Timezone:  "America/New_York",
	},
	"3GA8": {
		ICAO:      "3GA8",
		Name:      "Cauley's Airstrip",
		City:      "Soperton",
		State:     "Georgia",
		Country:   "US",
		Elevation: 251,
		Latitude:  32.3293991089,
		Longitude: -82.6350021362,
		Timezone:  "America/New_York",
	},
	"3GA9": {
		ICAO:      "3GA9",
		Name:      "Vintage Field",
		City:      "Zebulon",
		State:     "Georgia",
		Country:   "US",
		Elevation: 873,
		Latitude:  33.1334991455,
		Longitude: -84.3774032593,
		Timezone:  "America/New_York",
	},
	"3GE3": {
		ICAO:      "3GE3",
		Name:      "Broad River Air Park",
		City:      "Carnesville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 690,
		Latitude:  34.4118995667,
		Longitude: -83.1841964722,
		Timezone:  "America/New_York",
	},
	"3GE4": {
		ICAO:      "3GE4",
		Name:      "Fox Mountain Airport",
		City:      "Rising Fawn",
		State:     "Alabama",
		Country:   "US",
		Elevation: 925,
		Latitude:  34.7299995422,
		Longitude: -85.5419006348,
		Timezone:  "America/Chicago",
	},
	"3GE7": {
		ICAO:      "3GE7",
		Name:      "Eden Field",
		City:      "Darien",
		State:     "Georgia",
		Country:   "US",
		Elevation: 30,
		Latitude:  31.4127006531,
		Longitude: -81.4373016357,
		Timezone:  "America/New_York",
	},
	"3GE8": {
		ICAO:      "3GE8",
		Name:      "Prater Ranch Airport",
		City:      "Lincolnton",
		State:     "Georgia",
		Country:   "US",
		Elevation: 460,
		Latitude:  33.84349823,
		Longitude: -82.4789962769,
		Timezone:  "America/New_York",
	},
	"3GE9": {
		ICAO:      "3GE9",
		Name:      "Holly Farm Airport",
		City:      "Woodstock",
		State:     "Georgia",
		Country:   "US",
		Elevation: 900,
		Latitude:  34.1399993896,
		Longitude: -84.3780975342,
		Timezone:  "America/New_York",
	},
	"3IA1": {
		ICAO:      "3IA1",
		Name:      "Whites Airport",
		City:      "Melbourne",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1050,
		Latitude:  41.9608001709,
		Longitude: -93.1199035645,
		Timezone:  "America/Chicago",
	},
	"3IA4": {
		ICAO:      "3IA4",
		Name:      "Robinson Airport",
		City:      "Missouri Valley",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1000,
		Latitude:  41.5318984985,
		Longitude: -95.9420013428,
		Timezone:  "America/Chicago",
	},
	"3IA5": {
		ICAO:      "3IA5",
		Name:      "Kerr Airport",
		City:      "Marcus",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1450,
		Latitude:  42.8772010803,
		Longitude: -95.8824996948,
		Timezone:  "America/Chicago",
	},
	"3IA9": {
		ICAO:      "3IA9",
		Name:      "Rake Airport",
		City:      "Rake",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1180,
		Latitude:  43.4757995605,
		Longitude: -93.9072036743,
		Timezone:  "America/Chicago",
	},
	"3ID2": {
		ICAO:      "3ID2",
		Name:      "The Last Resort Airport",
		City:      "Weippe",
		State:     "Idaho",
		Country:   "US",
		Elevation: 3155,
		Latitude:  46.350675,
		Longitude: -115.977,
		Timezone:  "America/Los_Angeles",
	},
	"3ID7": {
		ICAO:      "3ID7",
		Name:      "Indian Creek Ranch Airport",
		City:      "Kuna",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2674,
		Latitude:  43.4760017395,
		Longitude: -116.4039993286,
		Timezone:  "America/Boise",
	},
	"3IG3": {
		ICAO:      "3IG3",
		Name:      "Terry's Airport",
		City:      "Columbia City",
		State:     "Indiana",
		Country:   "US",
		Elevation: 850,
		Latitude:  41.1702003479,
		Longitude: -85.4216003418,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"3II0": {
		ICAO:      "3II0",
		Name:      "Fremont (Murphy) Airport",
		City:      "Fremont",
		State:     "Indiana",
		Country:   "US",
		Elevation: 1050,
		Latitude:  41.6910848513,
		Longitude: -84.864256382,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"3II1": {
		ICAO:      "3II1",
		Name:      "Shenandoah Flying Field",
		City:      "Gosport",
		State:     "Indiana",
		Country:   "US",
		Elevation: 720,
		Latitude:  39.4166984558,
		Longitude: -86.6358032227,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"3II2": {
		ICAO:      "3II2",
		Name:      "Fifer Field",
		City:      "Griffin",
		State:     "Indiana",
		Country:   "US",
		Elevation: 445,
		Latitude:  38.2127990723,
		Longitude: -87.9195022583,
		Timezone:  "America/Chicago",
	},
	"3II3": {
		ICAO:      "3II3",
		Name:      "Basting Airport",
		City:      "Woodburn",
		State:     "Indiana",
		Country:   "US",
		Elevation: 755,
		Latitude:  41.0870018005,
		Longitude: -84.8411026001,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"3II5": {
		ICAO:      "3II5",
		Name:      "Stout Field",
		City:      "Medaryville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 700,
		Latitude:  41.0839004517,
		Longitude: -86.7733001709,
		Timezone:  "America/Indiana/Winamac",
	},
	"3II8": {
		ICAO:      "3II8",
		Name:      "Birkey Private Airport",
		City:      "Bremen",
		State:     "Indiana",
		Country:   "US",
		Elevation: 796,
		Latitude:  41.4422988892,
		Longitude: -86.261100769,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"3II9": {
		ICAO:      "3II9",
		Name:      "Dick's Strip",
		City:      "Arcola",
		State:     "Indiana",
		Country:   "US",
		Elevation: 850,
		Latitude:  41.0973014832,
		Longitude: -85.2422027588,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"3IL1": {
		ICAO:      "3IL1",
		Name:      "Silver Creek Gliderport",
		City:      "New Douglas",
		State:     "Illinois",
		Country:   "US",
		Elevation: 595,
		Latitude:  38.9253005981,
		Longitude: -89.6622009277,
		Timezone:  "America/Chicago",
	},
	"3IL2": {
		ICAO:      "3IL2",
		Name:      "Sweedler Airport",
		City:      "Elwood",
		State:     "Illinois",
		Country:   "US",
		Elevation: 669,
		Latitude:  41.4123001099,
		Longitude: -88.0486984253,
		Timezone:  "America/Chicago",
	},
	"3IL7": {
		ICAO:      "3IL7",
		Name:      "Home Free Airport",
		City:      "Steward",
		State:     "Illinois",
		Country:   "US",
		Elevation: 819,
		Latitude:  41.8033981323,
		Longitude: -89.0217971802,
		Timezone:  "America/Chicago",
	},
	"3IL9": {
		ICAO:      "3IL9",
		Name:      "Sugar Creek Farm Airport",
		City:      "Atlanta",
		State:     "Illinois",
		Country:   "US",
		Elevation: 600,
		Latitude:  40.2694015503,
		Longitude: -89.3022003174,
		Timezone:  "America/Chicago",
	},
	"3IN2": {
		ICAO:      "3IN2",
		Name:      "Dupouy Airport",
		City:      "Point Isabel",
		State:     "Indiana",
		Country:   "US",
		Elevation: 860,
		Latitude:  40.4192008972,
		Longitude: -85.8561019897,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"3IN3": {
		ICAO:      "3IN3",
		Name:      "Carlson Farm Airport",
		City:      "Portage",
		State:     "Indiana",
		Country:   "US",
		Elevation: 690,
		Latitude:  41.5231018066,
		Longitude: -87.1639022827,
		Timezone:  "America/Chicago",
	},
	"3IN4": {
		ICAO:      "3IN4",
		Name:      "Johnson Airport",
		City:      "Markle",
		State:     "Indiana",
		Country:   "US",
		Elevation: 830,
		Latitude:  40.7750015259,
		Longitude: -85.3388977051,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"3IN5": {
		ICAO:      "3IN5",
		Name:      "Pippenger Airport",
		City:      "Albion",
		State:     "Indiana",
		Country:   "US",
		Elevation: 980,
		Latitude:  41.3168983459,
		Longitude: -85.3713989258,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"3IN6": {
		ICAO:      "3IN6",
		Name:      "Holloway Field",
		City:      "New Washington",
		State:     "Indiana",
		Country:   "US",
		Elevation: 750,
		Latitude:  38.588333,
		Longitude: -85.505833,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"3IN7": {
		ICAO:      "3IN7",
		Name:      "Chain-O-Lakes Airport",
		City:      "South Bend",
		State:     "Indiana",
		Country:   "US",
		Elevation: 743,
		Latitude:  41.6624984741,
		Longitude: -86.3542022705,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"3IN8": {
		ICAO:      "3IN8",
		Name:      "Ddt Field",
		City:      "Culver",
		State:     "Indiana",
		Country:   "US",
		Elevation: 823,
		Latitude:  41.20470047,
		Longitude: -86.3458023071,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"3IN9": {
		ICAO:      "3IN9",
		Name:      "Thomas Airport",
		City:      "Cicero",
		State:     "Indiana",
		Country:   "US",
		Elevation: 890,
		Latitude:  40.1514015198,
		Longitude: -86.1192016602,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"3IS1": {
		ICAO:      "3IS1",
		Name:      "Mc Coy Airport",
		City:      "Blue Mound",
		State:     "Illinois",
		Country:   "US",
		Elevation: 615,
		Latitude:  39.7000007629,
		Longitude: -89.2089996338,
		Timezone:  "America/Chicago",
	},
	"3IS2": {
		ICAO:      "3IS2",
		Name:      "Earp Airport",
		City:      "Roseville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 740,
		Latitude:  40.7406005859,
		Longitude: -90.6696014404,
		Timezone:  "America/Chicago",
	},
	"3IS3": {
		ICAO:      "3IS3",
		Name:      "Noland RLA Restricted Landing Area",
		City:      "Blue Mound",
		State:     "Illinois",
		Country:   "US",
		Elevation: 600,
		Latitude:  39.7333984375,
		Longitude: -89.150100708,
		Timezone:  "America/Chicago",
	},
	"3IS4": {
		ICAO:      "3IS4",
		Name:      "Merkle Airport",
		City:      "Hanover",
		State:     "Illinois",
		Country:   "US",
		Elevation: 970,
		Latitude:  42.288898468,
		Longitude: -90.3453979492,
		Timezone:  "America/Chicago",
	},
	"3IS5": {
		ICAO:      "3IS5",
		Name:      "Holmes Airport",
		City:      "Pawnee",
		State:     "Illinois",
		Country:   "US",
		Elevation: 590,
		Latitude:  39.6845016479,
		Longitude: -89.5987014771,
		Timezone:  "America/Chicago",
	},
	"3IS6": {
		ICAO:      "3IS6",
		Name:      "Davy Jones /Private/ Airport",
		City:      "Bonnie",
		State:     "Illinois",
		Country:   "US",
		Elevation: 500,
		Latitude:  38.2228012085,
		Longitude: -88.8827972412,
		Timezone:  "America/Chicago",
	},
	"3IS7": {
		ICAO:      "3IS7",
		Name:      "Foote Airport",
		City:      "Wenona",
		State:     "Illinois",
		Country:   "US",
		Elevation: 650,
		Latitude:  41.0867004395,
		Longitude: -89.0959014893,
		Timezone:  "America/Chicago",
	},
	"3IS8": {
		ICAO:      "3IS8",
		IATA:      "BDF",
		Name:      "Rinkenberger Restricted Landing Area",
		City:      "Bradford",
		State:     "Illinois",
		Country:   "US",
		Elevation: 808,
		Latitude:  41.2308998108,
		Longitude: -89.6156997681,
		Timezone:  "America/Chicago",
	},
	"3KS1": {
		ICAO:      "3KS1",
		Name:      "Mesa Verde Airport",
		City:      "Topeka",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1080,
		Latitude:  39.1556015015,
		Longitude: -95.7433013916,
		Timezone:  "America/Chicago",
	},
	"3KS3": {
		ICAO:      "3KS3",
		Name:      "Rogers Airport",
		City:      "Udall",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1210,
		Latitude:  37.4000015259,
		Longitude: -97.1920013428,
		Timezone:  "America/Chicago",
	},
	"3KS4": {
		ICAO:      "3KS4",
		Name:      "Eveleigh Farms Airport",
		City:      "Ulysses",
		State:     "Kansas",
		Country:   "US",
		Elevation: 3207,
		Latitude:  37.7677993774,
		Longitude: -101.4349975586,
		Timezone:  "America/Chicago",
	},
	"3KS5": {
		ICAO:      "3KS5",
		Name:      "High Point Airport",
		City:      "Valley Center",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1395,
		Latitude:  37.8427155871,
		Longitude: -97.3514413834,
		Timezone:  "America/Chicago",
	},
	"3KS7": {
		ICAO:      "3KS7",
		Name:      "Berwick Airport",
		City:      "Valley Center",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1400,
		Latitude:  37.8292007446,
		Longitude: -97.3488998413,
		Timezone:  "America/Chicago",
	},
	"3KS8": {
		ICAO:      "3KS8",
		Name:      "Reed-Wilsonton Airport",
		City:      "Altamont",
		State:     "Kansas",
		Country:   "US",
		Elevation: 920,
		Latitude:  37.2565002441,
		Longitude: -95.3336029053,
		Timezone:  "America/Chicago",
	},
	"3KY0": {
		ICAO:      "3KY0",
		Name:      "Lamar Field",
		City:      "Hawesville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 410,
		Latitude:  37.8967018127,
		Longitude: -86.7893981934,
		Timezone:  "America/Chicago",
	},
	"3KY1": {
		ICAO:      "3KY1",
		Name:      "Goode Airpark",
		City:      "Utica",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 395,
		Latitude:  37.6413993835,
		Longitude: -87.1316986084,
		Timezone:  "America/Chicago",
	},
	"3KY2": {
		ICAO:      "3KY2",
		Name:      "Chesnut Knolls Airport",
		City:      "London",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 1265,
		Latitude:  37.1511001587,
		Longitude: -84.2574996948,
		Timezone:  "America/New_York",
	},
	"3KY3": {
		ICAO:      "3KY3",
		Name:      "Mason Valley Airport",
		City:      "Taylorsville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 510,
		Latitude:  38.0382995605,
		Longitude: -85.3593978882,
		Timezone:  "America/New_York",
	},
	"3KY4": {
		ICAO:      "3KY4",
		Name:      "Craw Daddy Landing Airport",
		City:      "Carrollton",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 460,
		Latitude:  38.6727981567,
		Longitude: -85.182800293,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"3KY6": {
		ICAO:      "3KY6",
		Name:      "Russell Airport",
		City:      "Battletown",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 430,
		Latitude:  38.0306015015,
		Longitude: -86.2316970825,
		Timezone:  "America/Kentucky/Louisville",
	},
	"3KY7": {
		ICAO:      "3KY7",
		Name:      "Eagle's Nest Airport",
		City:      "Battletown",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 450,
		Latitude:  38.1032981873,
		Longitude: -86.4569015503,
		Timezone:  "America/Indiana/Marengo",
	},
	"3KY9": {
		ICAO:      "3KY9",
		Name:      "Miles Field",
		City:      "Shelbyville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 850,
		Latitude:  38.125,
		Longitude: -85.125,
		Timezone:  "America/New_York",
	},
	"3LA0": {
		ICAO:      "3LA0",
		Name:      "Harrington Flying Service Airport",
		City:      "Kaplan",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 10,
		Latitude:  29.9197006226,
		Longitude: -92.2473983765,
		Timezone:  "America/Chicago",
	},
	"3LA1": {
		ICAO:      "3LA1",
		Name:      "Wilder Airport",
		City:      "Kinder",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 55,
		Latitude:  30.5585002899,
		Longitude: -92.8042984009,
		Timezone:  "America/Chicago",
	},
	"3LA3": {
		ICAO:      "3LA3",
		Name:      "La Coste Construction County Airport",
		City:      "Baton Rouge",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 34,
		Latitude:  30.5023994446,
		Longitude: -91.2428970337,
		Timezone:  "America/Chicago",
	},
	"3LA4": {
		ICAO:      "3LA4",
		Name:      "Little Pecan Island Airport",
		City:      "Lake Arthur",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 5,
		Latitude:  29.8001995087,
		Longitude: -92.8001022339,
		Timezone:  "America/Chicago",
	},
	"3LA6": {
		ICAO:      "3LA6",
		Name:      "Morgan Crop Service Nr 2 Airport",
		City:      "Lake Charles",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 12,
		Latitude:  30.1376991272,
		Longitude: -93.0792999268,
		Timezone:  "America/Chicago",
	},
	"3LA8": {
		ICAO:      "3LA8",
		Name:      "Open A-1 Ranch Airport",
		City:      "Lake Charles",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 16,
		Latitude:  30.1385002136,
		Longitude: -93.227897644,
		Timezone:  "America/Chicago",
	},
	"3LA9": {
		ICAO:      "3LA9",
		Name:      "Morgan Crop Service Airport",
		City:      "Lake Charles",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 12,
		Latitude:  30.2294006348,
		Longitude: -93.0885009766,
		Timezone:  "America/Chicago",
	},
	"3LL0": {
		ICAO:      "3LL0",
		Name:      "Miller Airport",
		City:      "South Beloit",
		State:     "Illinois",
		Country:   "US",
		Elevation: 760,
		Latitude:  42.4916992188,
		Longitude: -89.0959014893,
		Timezone:  "America/Chicago",
	},
	"3LL1": {
		ICAO:      "3LL1",
		Name:      "Herschel Hunter Airport",
		City:      "Marissa",
		State:     "Illinois",
		Country:   "US",
		Elevation: 470,
		Latitude:  38.2223014832,
		Longitude: -89.7042999268,
		Timezone:  "America/Chicago",
	},
	"3LL3": {
		ICAO:      "3LL3",
		Name:      "Kibler Airport",
		City:      "Marshall",
		State:     "Illinois",
		Country:   "US",
		Elevation: 588,
		Latitude:  39.4460983276,
		Longitude: -87.6445007324,
		Timezone:  "America/Chicago",
	},
	"3LL4": {
		ICAO:      "3LL4",
		Name:      "Pillow Hill Airport",
		City:      "Spring Grove",
		State:     "Illinois",
		Country:   "US",
		Elevation: 815,
		Latitude:  42.4472007751,
		Longitude: -88.201499939,
		Timezone:  "America/Chicago",
	},
	"3LL5": {
		ICAO:      "3LL5",
		Name:      "Richardson Field",
		City:      "Spring Grove",
		State:     "Illinois",
		Country:   "US",
		Elevation: 860,
		Latitude:  42.4645004272,
		Longitude: -88.2342987061,
		Timezone:  "America/Chicago",
	},
	"3LL6": {
		ICAO:      "3LL6",
		Name:      "Bickel Airport",
		City:      "New Athens",
		State:     "Illinois",
		Country:   "US",
		Elevation: 530,
		Latitude:  38.335,
		Longitude: -89.769722,
		Timezone:  "America/Chicago",
	},
	"3LL8": {
		ICAO:      "3LL8",
		Name:      "Mc Leansboro Airport",
		City:      "Mc Leansboro",
		State:     "Illinois",
		Country:   "US",
		Elevation: 415,
		Latitude:  38.0741996765,
		Longitude: -88.5375976562,
		Timezone:  "America/Chicago",
	},
	"3LL9": {
		ICAO:      "3LL9",
		Name:      "Compton Airport",
		City:      "Pocahontas",
		State:     "Illinois",
		Country:   "US",
		Elevation: 535,
		Latitude:  38.8041992188,
		Longitude: -89.6089019775,
		Timezone:  "America/Chicago",
	},
	"3LS7": {
		ICAO:      "3LS7",
		Name:      "Bock Farms Airport",
		City:      "Williamsville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 592,
		Latitude:  39.9864006042,
		Longitude: -89.5117034912,
		Timezone:  "America/Chicago",
	},
	"3MA2": {
		ICAO:      "3MA2",
		Name:      "Baines Airport",
		City:      "Middlefield",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 1160,
		Latitude:  42.3162002563,
		Longitude: -73.0319976807,
		Timezone:  "America/New_York",
	},
	"3MA5": {
		ICAO:      "3MA5",
		Name:      "Westport Airport",
		City:      "Westport",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 80,
		Latitude:  41.5432014465,
		Longitude: -71.0353012085,
		Timezone:  "America/New_York",
	},
	"3MD0": {
		ICAO:      "3MD0",
		Name:      "Burhans Memorial Airport",
		City:      "Frederick",
		State:     "Maryland",
		Country:   "US",
		Elevation: 490,
		Latitude:  39.3482017517,
		Longitude: -77.3360977173,
		Timezone:  "America/New_York",
	},
	"3MD4": {
		ICAO:      "3MD4",
		Name:      "Fairview Airport",
		City:      "Annapolis",
		State:     "Maryland",
		Country:   "US",
		Elevation: 150,
		Latitude:  38.9776000977,
		Longitude: -76.6393966675,
		Timezone:  "America/New_York",
	},
	"3MD5": {
		ICAO:      "3MD5",
		Name:      "Hidden Hills Airport",
		City:      "Hurlock",
		State:     "Maryland",
		Country:   "US",
		Elevation: 20,
		Latitude:  38.6212005615,
		Longitude: -75.8413009644,
		Timezone:  "America/New_York",
	},
	"3MD6": {
		ICAO:      "3MD6",
		Name:      "West St Mary's Airport",
		City:      "Drayden",
		State:     "Maryland",
		Country:   "US",
		Elevation: 50,
		Latitude:  38.188999176,
		Longitude: -76.4468994141,
		Timezone:  "America/New_York",
	},
	"3MD8": {
		ICAO:      "3MD8",
		Name:      "Pokety Airport",
		City:      "Cambridge",
		State:     "Maryland",
		Country:   "US",
		Elevation: 10,
		Latitude:  38.6123008728,
		Longitude: -76.1710968018,
		Timezone:  "America/New_York",
	},
	"3MD9": {
		ICAO:      "3MD9",
		Name:      "Chandler Airport",
		City:      "Ridge",
		State:     "Maryland",
		Country:   "US",
		Elevation: 17,
		Latitude:  38.1181983948,
		Longitude: -76.3912963867,
		Timezone:  "America/New_York",
	},
	"3MI0": {
		ICAO:      "3MI0",
		Name:      "Doss Field",
		City:      "Lake George",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1050,
		Latitude:  43.9370002747,
		Longitude: -85.0220031738,
		Timezone:  "America/Detroit",
	},
	"3MI2": {
		ICAO:      "3MI2",
		Name:      "South Fox Island Airport",
		City:      "Northport",
		State:     "Michigan",
		Country:   "US",
		Elevation: 616,
		Latitude:  45.3992004395,
		Longitude: -85.8301010132,
		Timezone:  "America/Detroit",
	},
	"3MI5": {
		ICAO:      "3MI5",
		Name:      "Innes Acres Airport",
		City:      "Decker",
		State:     "Michigan",
		Country:   "US",
		Elevation: 775,
		Latitude:  43.4500007629,
		Longitude: -83.0277023315,
		Timezone:  "America/Detroit",
	},
	"3MI7": {
		ICAO:      "3MI7",
		Name:      "Willie Run Airport",
		City:      "Ortonville",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1043,
		Latitude:  42.8292007446,
		Longitude: -83.368598938,
		Timezone:  "America/Detroit",
	},
	"3MN1": {
		ICAO:      "3MN1",
		Name:      "Stahlberg-Mohr Airport",
		City:      "Iron Junction",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1380,
		Latitude:  47.4485015869,
		Longitude: -92.6448974609,
		Timezone:  "America/Chicago",
	},
	"3MN3": {
		ICAO:      "3MN3",
		Name:      "Honker Flats Airport",
		City:      "Middle River",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1190,
		Latitude:  48.538898468,
		Longitude: -95.952003479,
		Timezone:  "America/Chicago",
	},
	"3MN5": {
		ICAO:      "3MN5",
		Name:      "Little Rock/Rock Port Airport",
		City:      "Rice",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1055,
		Latitude:  45.7229995728,
		Longitude: -94.193901062,
		Timezone:  "America/Chicago",
	},
	"3MN7": {
		ICAO:      "3MN7",
		Name:      "Blue Sky Airport",
		City:      "Sabin",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 927,
		Latitude:  46.8004989624,
		Longitude: -96.5950012207,
		Timezone:  "America/Chicago",
	},
	"3MN8": {
		ICAO:      "3MN8",
		Name:      "Aysta Field",
		City:      "St Cloud",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1024,
		Latitude:  45.5227012634,
		Longitude: -94.1228027344,
		Timezone:  "America/Chicago",
	},
	"3MN9": {
		ICAO:      "3MN9",
		Name:      "Schumacher Airport",
		City:      "Oster",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1020,
		Latitude:  44.9844017029,
		Longitude: -93.9360961914,
		Timezone:  "America/Chicago",
	},
	"3MO2": {
		ICAO:      "3MO2",
		Name:      "Ultra Flight Airpark",
		City:      "Berger",
		State:     "Missouri",
		Country:   "US",
		Elevation: 502,
		Latitude:  38.6908988953,
		Longitude: -91.341003418,
		Timezone:  "America/Chicago",
	},
	"3MO4": {
		ICAO:      "3MO4",
		Name:      "Penman Airport",
		City:      "Portageville",
		State:     "Missouri",
		Country:   "US",
		Elevation: 287,
		Latitude:  36.4505996704,
		Longitude: -89.6280975342,
		Timezone:  "America/Chicago",
	},
	"3MO5": {
		ICAO:      "3MO5",
		Name:      "Nimsick Airport",
		City:      "Carthage",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1095,
		Latitude:  37.1281013489,
		Longitude: -94.2161026001,
		Timezone:  "America/Chicago",
	},
	"3MO6": {
		ICAO:      "3MO6",
		Name:      "Kitty Hawk Estates Airport",
		City:      "Kearney",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1000,
		Latitude:  39.3356018066,
		Longitude: -94.4835968018,
		Timezone:  "America/Chicago",
	},
	"3MO7": {
		ICAO:      "3MO7",
		Name:      "Fairbanks Airport",
		City:      "King City",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1083,
		Latitude:  40.0833015442,
		Longitude: -94.616897583,
		Timezone:  "America/Chicago",
	},
	"3MO8": {
		ICAO:      "3MO8",
		Name:      "Fizzle Ridge Airport",
		City:      "King City",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1096,
		Latitude:  40.0318984985,
		Longitude: -94.5376968384,
		Timezone:  "America/Chicago",
	},
	"3MS1": {
		ICAO:      "3MS1",
		Name:      "Thomas Field",
		City:      "Holly Springs",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 485,
		Latitude:  34.8058013916,
		Longitude: -89.460899353,
		Timezone:  "America/Chicago",
	},
	"3MS2": {
		ICAO:      "3MS2",
		Name:      "Thunderfoot Ranch Airport",
		City:      "Tylertown",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 390,
		Latitude:  31.0790996552,
		Longitude: -90.0283966064,
		Timezone:  "America/Chicago",
	},
	"3MS3": {
		ICAO:      "3MS3",
		Name:      "Root Hog Airport",
		City:      "Flora",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 220,
		Latitude:  32.5158996582,
		Longitude: -90.3511962891,
		Timezone:  "America/Chicago",
	},
	"3MS5": {
		ICAO:      "3MS5",
		Name:      "Mitchell's Airport",
		City:      "Vaiden",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 323,
		Latitude:  33.3017997742,
		Longitude: -89.8143005371,
		Timezone:  "America/Chicago",
	},
	"3MS6": {
		ICAO:      "3MS6",
		Name:      "E E Lane Airport",
		City:      "Flora",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 268,
		Latitude:  32.5542984009,
		Longitude: -90.3029022217,
		Timezone:  "America/Chicago",
	},
	"3MS8": {
		ICAO:      "3MS8",
		Name:      "Fairview Farms Airport",
		City:      "Scooba",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 170,
		Latitude:  32.942199707,
		Longitude: -88.3635025024,
		Timezone:  "America/Chicago",
	},
	"3MS9": {
		ICAO:      "3MS9",
		Name:      "Union Municipal Airport",
		City:      "Union",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 512,
		Latitude:  32.5806999207,
		Longitude: -89.1361999512,
		Timezone:  "America/Chicago",
	},
	"3MT3": {
		ICAO:      "3MT3",
		Name:      "Three Cross Ranch Airport",
		City:      "Ryegate",
		State:     "Montana",
		Country:   "US",
		Elevation: 3930,
		Latitude:  46.125556,
		Longitude: -109.365,
		Timezone:  "America/Denver",
	},
	"3NA0": {
		ICAO:      "3NA0",
		Name:      "Erickson Airport",
		City:      "Clifford",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1095,
		Latitude:  47.4166984558,
		Longitude: -97.5114974976,
		Timezone:  "America/Chicago",
	},
	"3NA2": {
		ICAO:      "3NA2",
		Name:      "Lorentzen Airport",
		City:      "Washburn",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1790,
		Latitude:  47.3027992249,
		Longitude: -101.0670013428,
		Timezone:  "America/Chicago",
	},
	"3NA6": {
		ICAO:      "3NA6",
		Name:      "Risovi Ranch Strip",
		City:      "Hamberg",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1560,
		Latitude:  47.7705993652,
		Longitude: -99.4617996216,
		Timezone:  "America/Chicago",
	},
	"3NC0": {
		ICAO:      "3NC0",
		Name:      "Clyde Valley Airport",
		City:      "Morganton",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 1100,
		Latitude:  35.6903991699,
		Longitude: -81.7123031616,
		Timezone:  "America/New_York",
	},
	"3NC1": {
		ICAO:      "3NC1",
		Name:      "Welborn Farm Airport",
		City:      "Boonville",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 1030,
		Latitude:  36.2053985596,
		Longitude: -80.6584014893,
		Timezone:  "America/New_York",
	},
	"3NC3": {
		ICAO:      "3NC3",
		Name:      "Tucker Field",
		City:      "Locust",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 645,
		Latitude:  35.2468986511,
		Longitude: -80.4075012207,
		Timezone:  "America/New_York",
	},
	"3NC4": {
		ICAO:      "3NC4",
		Name:      "Mc Donald Field",
		City:      "Marshville",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 545,
		Latitude:  35.05350113,
		Longitude: -80.40899658,
		Timezone:  "America/New_York",
	},
	"3NC5": {
		ICAO:      "3NC5",
		Name:      "Flying Bj Airport",
		City:      "Stony Point",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 1000,
		Latitude:  35.8121986389,
		Longitude: -81.0699996948,
		Timezone:  "America/New_York",
	},
	"3NC6": {
		ICAO:      "3NC6",
		Name:      "Mc Cachren Field",
		City:      "Harrisburg",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 565,
		Latitude:  35.3117980957,
		Longitude: -80.644203186,
		Timezone:  "America/New_York",
	},
	"3NC7": {
		ICAO:      "3NC7",
		Name:      "Maxwell Airport",
		City:      "Sparta",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 3050,
		Latitude:  36.5192985535,
		Longitude: -81.1367034912,
		Timezone:  "America/New_York",
	},
	"3NC9": {
		ICAO:      "3NC9",
		Name:      "Womble Field",
		City:      "Chapel Hill",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 475,
		Latitude:  35.8750991821,
		Longitude: -79.0871963501,
		Timezone:  "America/New_York",
	},
	"3ND5": {
		ICAO:      "3ND5",
		Name:      "Gage Flying Farmer Airport",
		City:      "Oriska",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1235,
		Latitude:  47.0611000061,
		Longitude: -97.7712020874,
		Timezone:  "America/Chicago",
	},
	"3NE2": {
		ICAO:      "3NE2",
		Name:      "Phillips Private Airport",
		City:      "Hemingford",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 4370,
		Latitude:  42.3277015686,
		Longitude: -103.1940002441,
		Timezone:  "America/Denver",
	},
	"3NE3": {
		ICAO:      "3NE3",
		Name:      "Wells Airport",
		City:      "Holdrege",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2275,
		Latitude:  40.5,
		Longitude: -99.3170013428,
		Timezone:  "America/Chicago",
	},
	"3NE6": {
		ICAO:      "3NE6",
		Name:      "Burkinshaw Field",
		City:      "Jamison",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2200,
		Latitude:  42.9667015076,
		Longitude: -99.266998291,
		Timezone:  "America/Chicago",
	},
	"3NE7": {
		ICAO:      "3NE7",
		Name:      "Pawlet Ranch Airport",
		City:      "Lakeside",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3850,
		Latitude:  41.8291015625,
		Longitude: -102.3430023193,
		Timezone:  "America/Denver",
	},
	"3NE9": {
		ICAO:      "3NE9",
		Name:      "Phelps Airport",
		City:      "Lorenzo",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 4400,
		Latitude:  41.0410995483,
		Longitude: -103.0680007935,
		Timezone:  "America/Denver",
	},
	"3NJ1": {
		ICAO:      "3NJ1",
		Name:      "Pemberton Airport",
		City:      "Pemberton",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 65,
		Latitude:  39.9821014404,
		Longitude: -74.6927032471,
		Timezone:  "America/New_York",
	},
	"3NJ5": {
		ICAO:      "3NJ5",
		Name:      "Mock Airport",
		City:      "Quakertown",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 560,
		Latitude:  40.5334014893,
		Longitude: -74.9162979126,
		Timezone:  "America/New_York",
	},
	"3NJ6": {
		ICAO:      "3NJ6",
		Name:      "Inductotherm Airport",
		City:      "Rancocas",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 65,
		Latitude:  40.0150985718,
		Longitude: -74.8429031372,
		Timezone:  "America/New_York",
	},
	"3NJ9": {
		ICAO:      "3NJ9",
		Name:      "Allen Airstrip",
		City:      "Vincentown",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 51,
		Latitude:  39.9411125183,
		Longitude: -74.7708358765,
		Timezone:  "America/New_York",
	},
	"3NK0": {
		ICAO:      "3NK0",
		Name:      "Mc Kinney Airport",
		City:      "Brockport",
		State:     "New York",
		Country:   "US",
		Elevation: 547,
		Latitude:  43.2030982971,
		Longitude: -77.9024963379,
		Timezone:  "America/New_York",
	},
	"3NK4": {
		ICAO:      "3NK4",
		Name:      "Laska Airport",
		City:      "Jamestown",
		State:     "New York",
		Country:   "US",
		Elevation: 1480,
		Latitude:  42.0278015137,
		Longitude: -79.1941986084,
		Timezone:  "America/New_York",
	},
	"3NK5": {
		ICAO:      "3NK5",
		Name:      "Secret Spot Airport",
		City:      "Plattsburgh",
		State:     "New York",
		Country:   "US",
		Elevation: 155,
		Latitude:  44.6166992188,
		Longitude: -73.4653015137,
		Timezone:  "America/New_York",
	},
	"3NK6": {
		ICAO:      "3NK6",
		Name:      "Tuscarora Plateau Airport",
		City:      "Tuscarora",
		State:     "New York",
		Country:   "US",
		Elevation: 900,
		Latitude:  42.6370010376,
		Longitude: -77.8839035034,
		Timezone:  "America/New_York",
	},
	"3NK8": {
		ICAO:      "3NK8",
		Name:      "B Flat Farm Airport",
		City:      "Copake",
		State:     "New York",
		Country:   "US",
		Elevation: 800,
		Latitude:  42.1405982971,
		Longitude: -73.6119003296,
		Timezone:  "America/New_York",
	},
	"3NR3": {
		ICAO:      "3NR3",
		Name:      "Transylvania County Airport",
		City:      "Brevard",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 2110,
		Latitude:  35.270278,
		Longitude: -82.644167,
		Timezone:  "America/New_York",
	},
	"3NY0": {
		ICAO:      "3NY0",
		Name:      "The Ranch Airport",
		City:      "Charleston",
		State:     "New York",
		Country:   "US",
		Elevation: 1110,
		Latitude:  42.8273010254,
		Longitude: -74.4039993286,
		Timezone:  "America/New_York",
	},
	"3NY3": {
		ICAO:      "3NY3",
		Name:      "De Ronda Airport",
		City:      "Springfield",
		State:     "New York",
		Country:   "US",
		Elevation: 1650,
		Latitude:  42.8597984314,
		Longitude: -74.8243026733,
		Timezone:  "America/New_York",
	},
	"3NY4": {
		ICAO:      "3NY4",
		Name:      "Di Stefano Airpark",
		City:      "Fort Plain",
		State:     "New York",
		Country:   "US",
		Elevation: 500,
		Latitude:  42.922000885,
		Longitude: -74.6268005371,
		Timezone:  "America/New_York",
	},
	"3NY5": {
		ICAO:      "3NY5",
		Name:      "Luke Airport",
		City:      "Binghamton",
		State:     "New York",
		Country:   "US",
		Elevation: 1558,
		Latitude:  42.0237007141,
		Longitude: -75.9570999146,
		Timezone:  "America/New_York",
	},
	"3NY6": {
		ICAO:      "3NY6",
		Name:      "Six Ponds Airport",
		City:      "Duanesburg",
		State:     "New York",
		Country:   "US",
		Elevation: 1125,
		Latitude:  42.8412017822,
		Longitude: -74.1787033081,
		Timezone:  "America/New_York",
	},
	"3NY7": {
		ICAO:      "3NY7",
		Name:      "Hiserts Airpark Inc Airport",
		City:      "Eparatha",
		State:     "New York",
		Country:   "US",
		Elevation: 813,
		Latitude:  42.9832992554,
		Longitude: -74.5167007446,
		Timezone:  "America/New_York",
	},
	"3NY9": {
		ICAO:      "3NY9",
		Name:      "Hilltop Airport",
		City:      "Colden",
		State:     "New York",
		Country:   "US",
		Elevation: 1385,
		Latitude:  42.6643981934,
		Longitude: -78.6731033325,
		Timezone:  "America/New_York",
	},
	"3OA8": {
		ICAO:      "3OA8",
		Name:      "Valley Vista Airport",
		City:      "Bainbridge",
		State:     "Ohio",
		Country:   "US",
		Elevation: 720,
		Latitude:  39.2375984192,
		Longitude: -83.2606964111,
		Timezone:  "America/New_York",
	},
	"3OG3": {
		ICAO:      "3OG3",
		Name:      "Dillon Field Airport",
		City:      "Klamath Falls",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4092,
		Latitude:  42.122222,
		Longitude: -121.791667,
		Timezone:  "America/Los_Angeles",
	},
	"3OH0": {
		ICAO:      "3OH0",
		Name:      "Andy Barnhart Memorial Airport",
		City:      "New Carlisle",
		State:     "Ohio",
		Country:   "US",
		Elevation: 895,
		Latitude:  39.9309005737,
		Longitude: -84.0134963989,
		Timezone:  "America/New_York",
	},
	"3OH1": {
		ICAO:      "3OH1",
		Name:      "Morningstar North Airport",
		City:      "West Alexandria",
		State:     "Ohio",
		Country:   "US",
		Elevation: 982,
		Latitude:  39.6972999573,
		Longitude: -84.5427017212,
		Timezone:  "America/New_York",
	},
	"3OH3": {
		ICAO:      "3OH3",
		Name:      "Pelz Field",
		City:      "South Vienna",
		State:     "Ohio",
		Country:   "US",
		Elevation: 990,
		Latitude:  39.9838981628,
		Longitude: -83.6132965088,
		Timezone:  "America/New_York",
	},
	"3OH4": {
		ICAO:      "3OH4",
		Name:      "Riceland Aerodrome",
		City:      "Andover",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1115,
		Latitude:  41.6039009094,
		Longitude: -80.5897979736,
		Timezone:  "America/New_York",
	},
	"3OH5": {
		ICAO:      "3OH5",
		Name:      "Stub's Field",
		City:      "Alliance",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1130,
		Latitude:  40.9673995972,
		Longitude: -81.2058029175,
		Timezone:  "America/New_York",
	},
	"3OH6": {
		ICAO:      "3OH6",
		Name:      "Youngpeter Airport",
		City:      "Delphos",
		State:     "Ohio",
		Country:   "US",
		Elevation: 815,
		Latitude:  40.7832984924,
		Longitude: -84.3677978516,
		Timezone:  "America/New_York",
	},
	"3OH7": {
		ICAO:      "3OH7",
		Name:      "Creager Airport",
		City:      "Williamsburg",
		State:     "Ohio",
		Country:   "US",
		Elevation: 895,
		Latitude:  39.0301017761,
		Longitude: -84.0466003418,
		Timezone:  "America/New_York",
	},
	"3OH8": {
		ICAO:      "3OH8",
		Name:      "Gnadenhutten Airport",
		City:      "Gnadenhutten",
		State:     "Ohio",
		Country:   "US",
		Elevation: 842,
		Latitude:  40.3608016968,
		Longitude: -81.4233016968,
		Timezone:  "America/New_York",
	},
	"3OH9": {
		ICAO:      "3OH9",
		Name:      "Merts Field",
		City:      "Wilmington",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1010,
		Latitude:  39.5535011292,
		Longitude: -83.8520965576,
		Timezone:  "America/New_York",
	},
	"3OI6": {
		ICAO:      "3OI6",
		Name:      "Vogel Airpark",
		City:      "Savannah",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1028,
		Latitude:  40.9612007141,
		Longitude: -82.3706970215,
		Timezone:  "America/New_York",
	},
	"3OK0": {
		ICAO:      "3OK0",
		Name:      "Valley Airport",
		City:      "Ada",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 780,
		Latitude:  34.7050018311,
		Longitude: -96.5766983032,
		Timezone:  "America/Chicago",
	},
	"3OK1": {
		ICAO:      "3OK1",
		Name:      "Okarche Airport",
		City:      "Okarche",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1239,
		Latitude:  35.7299995422,
		Longitude: -97.9670028687,
		Timezone:  "America/Chicago",
	},
	"3OK2": {
		ICAO:      "3OK2",
		Name:      "Hopcus Farms Airport",
		City:      "Jones",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1110,
		Latitude:  35.591999054,
		Longitude: -97.327796936,
		Timezone:  "America/Chicago",
	},
	"3OK5": {
		ICAO:      "3OK5",
		Name:      "Bearden Private Airport",
		City:      "Lindsay",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1180,
		Latitude:  34.7285995483,
		Longitude: -97.605796814,
		Timezone:  "America/Chicago",
	},
	"3OK7": {
		ICAO:      "3OK7",
		Name:      "Double W Airport",
		City:      "Sand Springs",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 790,
		Latitude:  36.2417984009,
		Longitude: -96.1342010498,
		Timezone:  "America/Chicago",
	},
	"3OK8": {
		ICAO:      "3OK8",
		Name:      "Flying G Ranch Airport",
		City:      "Sand Springs",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 890,
		Latitude:  36.1166992188,
		Longitude: -96.2335968018,
		Timezone:  "America/Chicago",
	},
	"3OK9": {
		ICAO:      "3OK9",
		Name:      "Jazz Ranch Airport",
		City:      "Shawnee",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1080,
		Latitude:  35.321786,
		Longitude: -97.040083,
		Timezone:  "America/Chicago",
	},
	"3OL8": {
		ICAO:      "3OL8",
		Name:      "Harrison Airport",
		City:      "Welch",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 805,
		Latitude:  36.8778991699,
		Longitude: -95.1491012573,
		Timezone:  "America/Chicago",
	},
	"3OR1": {
		ICAO:      "3OR1",
		Name:      "Cubehole Airport",
		City:      "Brownsville",
		State:     "Oregon",
		Country:   "US",
		Elevation: 355,
		Latitude:  44.3646011353,
		Longitude: -122.9580001831,
		Timezone:  "America/Los_Angeles",
	},
	"3OR2": {
		ICAO:      "3OR2",
		Name:      "Maxwell Private Airport",
		City:      "La Grande",
		State:     "Oregon",
		Country:   "US",
		Elevation: 2710,
		Latitude:  45.3124008179,
		Longitude: -117.9589996338,
		Timezone:  "America/Los_Angeles",
	},
	"3OR8": {
		ICAO:      "3OR8",
		Name:      "Cline Falls Air Park",
		City:      "Redmond",
		State:     "Oregon",
		Country:   "US",
		Elevation: 2920,
		Latitude:  44.2831993103,
		Longitude: -121.2689971924,
		Timezone:  "America/Los_Angeles",
	},
	"3OR9": {
		ICAO:      "3OR9",
		Name:      "Murphy Ranch Airport",
		City:      "Juntura",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3465,
		Latitude:  43.9574012756,
		Longitude: -118.1320037842,
		Timezone:  "America/Boise",
	},
	"3PA0": {
		ICAO:      "3PA0",
		Name:      "Horst Airport",
		City:      "Lebanon",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 450,
		Latitude:  40.3833999634,
		Longitude: -76.4664001465,
		Timezone:  "America/New_York",
	},
	"3PA1": {
		ICAO:      "3PA1",
		Name:      "Navarro Airport",
		City:      "Sellersville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 540,
		Latitude:  40.3722991943,
		Longitude: -75.3376998901,
		Timezone:  "America/New_York",
	},
	"3PA2": {
		ICAO:      "3PA2",
		Name:      "Neeb Airport",
		City:      "Lehighton",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 825,
		Latitude:  40.8805999756,
		Longitude: -75.6356964111,
		Timezone:  "America/New_York",
	},
	"3PA3": {
		ICAO:      "3PA3",
		Name:      "Lehman Airport",
		City:      "Lehman",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1300,
		Latitude:  41.3166999817,
		Longitude: -76.0495986938,
		Timezone:  "America/New_York",
	},
	"3PA4": {
		ICAO:      "3PA4",
		Name:      "Giffin Airport",
		City:      "Leraysville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 840,
		Latitude:  41.7779006958,
		Longitude: -76.1371994019,
		Timezone:  "America/New_York",
	},
	"3PA6": {
		ICAO:      "3PA6",
		Name:      "Fox Hollow Airport",
		City:      "Lewisburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 540,
		Latitude:  40.9934005737,
		Longitude: -76.9001998901,
		Timezone:  "America/New_York",
	},
	"3PA8": {
		ICAO:      "3PA8",
		Name:      "Harris Airport",
		City:      "Lincoln University",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 575,
		Latitude:  39.820400238,
		Longitude: -75.9257965088,
		Timezone:  "America/New_York",
	},
	"3PA9": {
		ICAO:      "3PA9",
		Name:      "Moyer Airport",
		City:      "Line Lexington",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 330,
		Latitude:  40.2967987061,
		Longitude: -75.2480010986,
		Timezone:  "America/New_York",
	},
	"3PN0": {
		ICAO:      "3PN0",
		Name:      "Schulteis Airport",
		City:      "Biglerville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 680,
		Latitude:  39.9603996277,
		Longitude: -77.2591018677,
		Timezone:  "America/New_York",
	},
	"3PN1": {
		ICAO:      "3PN1",
		Name:      "Ashlawn Airport",
		City:      "Clifford",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1511,
		Latitude:  41.6220016479,
		Longitude: -75.5962982178,
		Timezone:  "America/New_York",
	},
	"3PN2": {
		ICAO:      "3PN2",
		Name:      "Karlindo Airport",
		City:      "Fairfield",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 530,
		Latitude:  39.7589988708,
		Longitude: -77.3475036621,
		Timezone:  "America/New_York",
	},
	"3PN3": {
		ICAO:      "3PN3",
		Name:      "Countryside Airport",
		City:      "Lehman",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1260,
		Latitude:  41.2836990356,
		Longitude: -76.0352020264,
		Timezone:  "America/New_York",
	},
	"3PN6": {
		ICAO:      "3PN6",
		Name:      "Gravel Run Airport",
		City:      "Cambridge Springs",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1240,
		Latitude:  41.7500991821,
		Longitude: -80.0830993652,
		Timezone:  "America/New_York",
	},
	"3PN7": {
		ICAO:      "3PN7",
		Name:      "Stahl's Mountain Airport",
		City:      "Union Dale",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1950,
		Latitude:  41.7478981018,
		Longitude: -75.4973983765,
		Timezone:  "America/New_York",
	},
	"3PN9": {
		ICAO:      "3PN9",
		Name:      "Phil Cain Memorial Field",
		City:      "Export",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1210,
		Latitude:  40.4417991638,
		Longitude: -79.5501022339,
		Timezone:  "America/New_York",
	},
	"3PS4": {
		ICAO:      "3PS4",
		Name:      "Mountain Hide-Away Airport",
		City:      "Millcreek",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1604,
		Latitude:  40.7235984802,
		Longitude: -77.9873962402,
		Timezone:  "America/New_York",
	},
	"3PS8": {
		ICAO:      "3PS8",
		Name:      "Yingst Airport",
		City:      "Harrisburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 701,
		Latitude:  40.3958015442,
		Longitude: -76.7457962036,
		Timezone:  "America/New_York",
	},
	"3PS9": {
		ICAO:      "3PS9",
		Name:      "Risker Field",
		City:      "Bulger",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1100,
		Latitude:  40.3783988953,
		Longitude: -80.3495025635,
		Timezone:  "America/New_York",
	},
	"3SC2": {
		ICAO:      "3SC2",
		Name:      "Yahu Field",
		City:      "Summerton",
		State:     "South Carolina",
		Country:   "US",
		Elevation: 140,
		Latitude:  33.601944,
		Longitude: -80.306111,
		Timezone:  "America/New_York",
	},
	"3SC4": {
		ICAO:      "3SC4",
		Name:      "Crooked Fence Farm Airport",
		City:      "Woodruff",
		State:     "South Carolina",
		Country:   "US",
		Elevation: 645,
		Latitude:  34.773601532,
		Longitude: -81.9863967896,
		Timezone:  "America/New_York",
	},
	"3SC7": {
		ICAO:      "3SC7",
		Name:      "Jordan Airport",
		City:      "Andrews",
		State:     "South Carolina",
		Country:   "US",
		Elevation: 35,
		Latitude:  33.5396995544,
		Longitude: -79.5318984985,
		Timezone:  "America/New_York",
	},
	"3SD3": {
		ICAO:      "3SD3",
		Name:      "Brown Field",
		City:      "Scenic",
		State:     "South Dakota",
		Country:   "US",
		Elevation: 2780,
		Latitude:  43.8117980957,
		Longitude: -102.7200012207,
		Timezone:  "America/Denver",
	},
	"3SD4": {
		ICAO:      "3SD4",
		Name:      "Chris Hofer Landing Strip",
		City:      "Tea",
		State:     "South Dakota",
		Country:   "US",
		Elevation: 1491,
		Latitude:  43.4655990601,
		Longitude: -96.8632965088,
		Timezone:  "America/Chicago",
	},
	"3SD6": {
		ICAO:      "3SD6",
		Name:      "Running Colors Airport",
		City:      "Rapid City",
		State:     "South Dakota",
		Country:   "US",
		Elevation: 2920,
		Latitude:  44.2022018433,
		Longitude: -103.0790023804,
		Timezone:  "America/Denver",
	},
	"3TA0": {
		ICAO:      "3TA0",
		Name:      "Four Square Ranch Airport",
		City:      "Rocksprings",
		State:     "Texas",
		Country:   "US",
		Elevation: 2250,
		Latitude:  30.0969009399,
		Longitude: -100.4049987793,
		Timezone:  "America/Chicago",
	},
	"3TA1": {
		ICAO:      "3TA1",
		Name:      "Diamond K Ranch Airport",
		City:      "Comfort",
		State:     "Texas",
		Country:   "US",
		Elevation: 1600,
		Latitude:  30.0366001129,
		Longitude: -98.6975021362,
		Timezone:  "America/Chicago",
	},
	"3TA2": {
		ICAO:      "3TA2",
		Name:      "Clark Sky Ranch Airport",
		City:      "Sealy",
		State:     "Texas",
		Country:   "US",
		Elevation: 145,
		Latitude:  29.7366008759,
		Longitude: -96.0641021729,
		Timezone:  "America/Chicago",
	},
	"3TA4": {
		ICAO:      "3TA4",
		Name:      "Tin Top Ranch Airport",
		City:      "Brownwood",
		State:     "Texas",
		Country:   "US",
		Elevation: 1300,
		Latitude:  31.4857997894,
		Longitude: -98.9760971069,
		Timezone:  "America/Chicago",
	},
	"3TA5": {
		ICAO:      "3TA5",
		Name:      "Blanco Landing Airport",
		City:      "Blanco",
		State:     "Texas",
		Country:   "US",
		Elevation: 1460,
		Latitude:  30.122467,
		Longitude: -98.374783,
		Timezone:  "America/Chicago",
	},
	"3TA6": {
		ICAO:      "3TA6",
		Name:      "Spring Ranch Airport",
		City:      "Brackettville",
		State:     "Texas",
		Country:   "US",
		Elevation: 1440,
		Latitude:  29.5419006348,
		Longitude: -100.2529983521,
		Timezone:  "America/Chicago",
	},
	"3TA7": {
		ICAO:      "3TA7",
		Name:      "Jim Sears Airport",
		City:      "Bridgeport",
		State:     "Texas",
		Country:   "US",
		Elevation: 900,
		Latitude:  33.143699646,
		Longitude: -97.8044967651,
		Timezone:  "America/Chicago",
	},
	"3TA8": {
		ICAO:      "3TA8",
		Name:      "Picosa Ranch Airport",
		City:      "Crystal City",
		State:     "Texas",
		Country:   "US",
		Elevation: 600,
		Latitude:  28.7196998596,
		Longitude: -100.0550003052,
		Timezone:  "America/Chicago",
	},
	"3TA9": {
		ICAO:      "3TA9",
		Name:      "Chacon Creek Ranch Airport",
		City:      "Crystal City",
		State:     "Texas",
		Country:   "US",
		Elevation: 595,
		Latitude:  28.7085990906,
		Longitude: -100.0070037842,
		Timezone:  "America/Chicago",
	},
	"3TE0": {
		ICAO:      "3TE0",
		Name:      "K Ranch Airport",
		City:      "Iola",
		State:     "Texas",
		Country:   "US",
		Elevation: 285,
		Latitude:  30.7315998077,
		Longitude: -96.1311035156,
		Timezone:  "America/Chicago",
	},
	"3TE1": {
		ICAO:      "3TE1",
		Name:      "Gum Island Airport",
		City:      "Dayton",
		State:     "Texas",
		Country:   "US",
		Elevation: 53,
		Latitude:  29.9515991211,
		Longitude: -94.9044036865,
		Timezone:  "America/Chicago",
	},
	"3TE3": {
		ICAO:      "3TE3",
		Name:      "Big Bend Ranch State Park Airport",
		City:      "Presidio",
		State:     "Texas",
		Country:   "US",
		Elevation: 4250,
		Latitude:  29.4696006775,
		Longitude: -103.93699646,
		Timezone:  "America/Chicago",
	},
	"3TE4": {
		ICAO:      "3TE4",
		Name:      "Hot Springs Airport",
		City:      "Ruidosa",
		State:     "Texas",
		Country:   "US",
		Elevation: 3071,
		Latitude:  30.0088005066,
		Longitude: -104.6650009155,
		Timezone:  "America/Chicago",
	},
	"3TE5": {
		ICAO:      "3TE5",
		Name:      "Stamps Field",
		City:      "Panhandle",
		State:     "Texas",
		Country:   "US",
		Elevation: 3445,
		Latitude:  35.3306007385,
		Longitude: -101.3850021362,
		Timezone:  "America/Chicago",
	},
	"3TE6": {
		ICAO:      "3TE6",
		Name:      "Skellytown Airport",
		City:      "Skellytown",
		State:     "Texas",
		Country:   "US",
		Elevation: 3280,
		Latitude:  35.5699996948,
		Longitude: -101.1689987183,
		Timezone:  "America/Chicago",
	},
	"3TE7": {
		ICAO:      "3TE7",
		Name:      "Mills Ranch Airport",
		City:      "Pandale",
		State:     "Texas",
		Country:   "US",
		Elevation: 1965,
		Latitude:  30.2418994904,
		Longitude: -101.4169998169,
		Timezone:  "America/Chicago",
	},
	"3TE8": {
		ICAO:      "3TE8",
		Name:      "Terlingua Airport",
		City:      "Terlingua /Alpine/",
		State:     "Texas",
		Country:   "US",
		Elevation: 2717,
		Latitude:  29.3194007874,
		Longitude: -103.5839996338,
		Timezone:  "America/Chicago",
	},
	"3TE9": {
		ICAO:      "3TE9",
		Name:      "Pinoak Airport",
		City:      "Dayton",
		State:     "Texas",
		Country:   "US",
		Elevation: 91,
		Latitude:  30.1058006287,
		Longitude: -94.9588012695,
		Timezone:  "America/Chicago",
	},
	"3TN0": {
		ICAO:      "3TN0",
		Name:      "Indian Springs Airport",
		City:      "Kingsport",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1580,
		Latitude:  36.535,
		Longitude: -82.423889,
		Timezone:  "America/New_York",
	},
	"3TN3": {
		ICAO:      "3TN3",
		Name:      "Wayne's World Airport",
		City:      "Dresden",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 440,
		Latitude:  36.2633018494,
		Longitude: -88.7572021484,
		Timezone:  "America/Chicago",
	},
	"3TN4": {
		ICAO:      "3TN4",
		Name:      "Kenner Farm Airport",
		City:      "Seymour",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 940,
		Latitude:  35.8809013367,
		Longitude: -83.7270965576,
		Timezone:  "America/New_York",
	},
	"3TN6": {
		ICAO:      "3TN6",
		Name:      "Wilson Field",
		City:      "Memphis",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 435,
		Latitude:  35.0968017578,
		Longitude: -89.2806015015,
		Timezone:  "America/Chicago",
	},
	"3TN8": {
		ICAO:      "3TN8",
		Name:      "Massingale Airport",
		City:      "Lenoir City",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 840,
		Latitude:  35.763999939,
		Longitude: -84.1874008179,
		Timezone:  "America/New_York",
	},
	"3TN9": {
		ICAO:      "3TN9",
		Name:      "Holenthawall Airport",
		City:      "Springfield",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 645,
		Latitude:  36.6133003235,
		Longitude: -86.8103027344,
		Timezone:  "America/Chicago",
	},
	"3TS0": {
		ICAO:      "3TS0",
		Name:      "East Side Airport",
		City:      "Longview",
		State:     "Texas",
		Country:   "US",
		Elevation: 373,
		Latitude:  32.5084991455,
		Longitude: -94.7002029419,
		Timezone:  "America/Chicago",
	},
	"3TS1": {
		ICAO:      "3TS1",
		Name:      "White Wings Airport",
		City:      "Wimberley",
		State:     "Texas",
		Country:   "US",
		Elevation: 1083,
		Latitude:  30.0114002228,
		Longitude: -98.0404968262,
		Timezone:  "America/Chicago",
	},
	"3TS3": {
		ICAO:      "3TS3",
		Name:      "J-D Ranch Airport",
		City:      "Alvin",
		State:     "Texas",
		Country:   "US",
		Elevation: 34,
		Latitude:  29.3747005463,
		Longitude: -95.2904968262,
		Timezone:  "America/Chicago",
	},
	"3TS5": {
		ICAO:      "3TS5",
		Name:      "Purdy-Nielsen Memorial Airpark",
		City:      "Beasley",
		State:     "Texas",
		Country:   "US",
		Elevation: 105,
		Latitude:  29.4710998535,
		Longitude: -95.8862991333,
		Timezone:  "America/Chicago",
	},
	"3TS7": {
		ICAO:      "3TS7",
		Name:      "Pavlat Airport",
		City:      "Dayton",
		State:     "Texas",
		Country:   "US",
		Elevation: 70,
		Latitude:  30.0210990906,
		Longitude: -94.9459991455,
		Timezone:  "America/Chicago",
	},
	"3TS9": {
		ICAO:      "3TS9",
		Name:      "Oak Glen Ranch Airport",
		City:      "Cooper",
		State:     "Texas",
		Country:   "US",
		Elevation: 430,
		Latitude:  33.3792991638,
		Longitude: -95.6035995483,
		Timezone:  "America/Chicago",
	},
	"3TX1": {
		ICAO:      "3TX1",
		Name:      "Paradise Point Airport",
		City:      "Berryville",
		State:     "Texas",
		Country:   "US",
		Elevation: 345,
		Latitude:  32.0917015076,
		Longitude: -95.4447021484,
		Timezone:  "America/Chicago",
	},
	"3TX2": {
		ICAO:      "3TX2",
		Name:      "Flying S Farm Airport",
		City:      "Justin",
		State:     "Texas",
		Country:   "US",
		Elevation: 780,
		Latitude:  33.0806999207,
		Longitude: -97.3488998413,
		Timezone:  "America/Chicago",
	},
	"3TX3": {
		ICAO:      "3TX3",
		Name:      "Sitton Field",
		City:      "Justin",
		State:     "Texas",
		Country:   "US",
		Elevation: 682,
		Latitude:  33.1195983887,
		Longitude: -97.3197021484,
		Timezone:  "America/Chicago",
	},
	"3TX4": {
		ICAO:      "3TX4",
		Name:      "Willow Run Airport",
		City:      "Justin",
		State:     "Texas",
		Country:   "US",
		Elevation: 820,
		Latitude:  33.0946006775,
		Longitude: -97.3803024292,
		Timezone:  "America/Chicago",
	},
	"3TX5": {
		ICAO:      "3TX5",
		Name:      "Mc Creless Farm Airport",
		City:      "Adkins",
		State:     "Texas",
		Country:   "US",
		Elevation: 600,
		Latitude:  29.371099472,
		Longitude: -98.2316970825,
		Timezone:  "America/Chicago",
	},
	"3TX6": {
		ICAO:      "3TX6",
		Name:      "Lowell Smith Jr Airport",
		City:      "Rio Vista",
		State:     "Texas",
		Country:   "US",
		Elevation: 740,
		Latitude:  32.233481,
		Longitude: -97.380849,
		Timezone:  "America/Chicago",
	},
	"3TX7": {
		ICAO:      "3TX7",
		Name:      "Flying P Airport",
		City:      "Justin",
		State:     "Texas",
		Country:   "US",
		Elevation: 805,
		Latitude:  33.1408004761,
		Longitude: -97.3711013794,
		Timezone:  "America/Chicago",
	},
	"3TX8": {
		ICAO:      "3TX8",
		Name:      "Drop Field",
		City:      "Justin",
		State:     "Texas",
		Country:   "US",
		Elevation: 700,
		Latitude:  33.1250991821,
		Longitude: -97.3460998535,
		Timezone:  "America/Chicago",
	},
	"3TX9": {
		ICAO:      "3TX9",
		Name:      "Rafter J Airport",
		City:      "Burleson",
		State:     "Texas",
		Country:   "US",
		Elevation: 710,
		Latitude:  32.5275993347,
		Longitude: -97.2807998657,
		Timezone:  "America/Chicago",
	},
	"3VA0": {
		ICAO:      "3VA0",
		Name:      "Ayres-Aicp Airport",
		City:      "Warrenton",
		State:     "Virginia",
		Country:   "US",
		Elevation: 530,
		Latitude:  38.7759017944,
		Longitude: -77.9105987549,
		Timezone:  "America/New_York",
	},
	"3VA1": {
		ICAO:      "3VA1",
		Name:      "The Meadows Airport",
		City:      "Warrenton",
		State:     "Virginia",
		Country:   "US",
		Elevation: 584,
		Latitude:  38.7742996216,
		Longitude: -77.799697876,
		Timezone:  "America/New_York",
	},
	"3VA2": {
		ICAO:      "3VA2",
		Name:      "Aviacres Airport",
		City:      "Warrenton",
		State:     "Virginia",
		Country:   "US",
		Elevation: 435,
		Latitude:  38.6242980957,
		Longitude: -77.7861022949,
		Timezone:  "America/New_York",
	},
	"3VA3": {
		ICAO:      "3VA3",
		Name:      "Flying Circus Aerodrome",
		City:      "Warrenton",
		State:     "Virginia",
		Country:   "US",
		Elevation: 315,
		Latitude:  38.5503997803,
		Longitude: -77.7157974243,
		Timezone:  "America/New_York",
	},
	"3VA4": {
		ICAO:      "3VA4",
		Name:      "Shandy Hall Farm Airport",
		City:      "Warsaw",
		State:     "Virginia",
		Country:   "US",
		Elevation: 10,
		Latitude:  37.8576011658,
		Longitude: -76.7358016968,
		Timezone:  "America/New_York",
	},
	"3VA5": {
		ICAO:      "3VA5",
		Name:      "Goose Hunt Farm Airport",
		City:      "Leesburg",
		State:     "Virginia",
		Country:   "US",
		Elevation: 340,
		Latitude:  39.028889,
		Longitude: -77.625,
		Timezone:  "America/New_York",
	},
	"3VA7": {
		ICAO:      "3VA7",
		Name:      "White Post Airport",
		City:      "White Post",
		State:     "Virginia",
		Country:   "US",
		Elevation: 610,
		Latitude:  39.0625991821,
		Longitude: -78.0914001465,
		Timezone:  "America/New_York",
	},
	"3VA8": {
		ICAO:      "3VA8",
		Name:      "Garner Airport",
		City:      "Windsor",
		State:     "Virginia",
		Country:   "US",
		Elevation: 80,
		Latitude:  36.856300354,
		Longitude: -76.6821975708,
		Timezone:  "America/New_York",
	},
	"3VG2": {
		ICAO:      "3VG2",
		Name:      "Machipongo International Airport",
		City:      "Machipongo",
		State:     "Virginia",
		Country:   "US",
		Elevation: 36,
		Latitude:  37.3883018494,
		Longitude: -75.8925018311,
		Timezone:  "America/New_York",
	},
	"3VG3": {
		ICAO:      "3VG3",
		Name:      "Serenity Farm Airport",
		City:      "Burgess",
		State:     "Virginia",
		Country:   "US",
		Elevation: 10,
		Latitude:  37.9361000061,
		Longitude: -76.3486022949,
		Timezone:  "America/New_York",
	},
	"3VG4": {
		ICAO:      "3VG4",
		Name:      "Knight'S Landing Llc",
		City:      "Virginia Beach",
		State:     "Virginia",
		Country:   "US",
		Elevation: 11,
		Latitude:  36.558333,
		Longitude: -76.116389,
		Timezone:  "America/New_York",
	},
	"3VG7": {
		ICAO:      "3VG7",
		Name:      "Hartwood Airport",
		City:      "Somerville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 390,
		Latitude:  38.4846000671,
		Longitude: -77.6128005981,
		Timezone:  "America/New_York",
	},
	"3VG9": {
		ICAO:      "3VG9",
		Name:      "Birch Creek Plantation Airport",
		City:      "South Boston",
		State:     "Virginia",
		Country:   "US",
		Elevation: 550,
		Latitude:  36.701944,
		Longitude: -79.133889,
		Timezone:  "America/New_York",
	},
	"3WA0": {
		ICAO:      "3WA0",
		Name:      "Taylor Airport",
		City:      "Rainier",
		State:     "Washington",
		Country:   "US",
		Elevation: 470,
		Latitude:  46.8975982666,
		Longitude: -122.6699981689,
		Timezone:  "America/Los_Angeles",
	},
	"3WA1": {
		ICAO:      "3WA1",
		Name:      "Johnson's Landing Ultralightport",
		City:      "Leavenworth",
		State:     "Washington",
		Country:   "US",
		Elevation: 1260,
		Latitude:  47.6035995483,
		Longitude: -120.6699981689,
		Timezone:  "America/Los_Angeles",
	},
	"3WA2": {
		ICAO:      "3WA2",
		Name:      "Hanes Airport",
		City:      "Harrington",
		State:     "Washington",
		Country:   "US",
		Elevation: 2209,
		Latitude:  47.4915008545,
		Longitude: -118.2509994507,
		Timezone:  "America/Los_Angeles",
	},
	"3WA3": {
		ICAO:      "3WA3",
		Name:      "Angel Park Airport",
		City:      "Kahlotus",
		State:     "Washington",
		Country:   "US",
		Elevation: 884,
		Latitude:  46.6484985352,
		Longitude: -118.5830001831,
		Timezone:  "America/Los_Angeles",
	},
	"3WA4": {
		ICAO:      "3WA4",
		Name:      "Watson Airport",
		City:      "Kahlotus",
		State:     "Washington",
		Country:   "US",
		Elevation: 1349,
		Latitude:  46.7164993286,
		Longitude: -118.6179962158,
		Timezone:  "America/Los_Angeles",
	},
	"3WA9": {
		ICAO:      "3WA9",
		Name:      "Flying Carpet Airport",
		City:      "Lacey",
		State:     "Washington",
		Country:   "US",
		Elevation: 255,
		Latitude:  46.9628982544,
		Longitude: -122.8000030518,
		Timezone:  "America/Los_Angeles",
	},
	"3WI0": {
		ICAO:      "3WI0",
		Name:      "Carnot Field",
		City:      "Algoma",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 655,
		Latitude:  44.64220047,
		Longitude: -87.4197998047,
		Timezone:  "America/Chicago",
	},
	"3WI1": {
		ICAO:      "3WI1",
		Name:      "Olson's Airport",
		City:      "Union Grove",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 825,
		Latitude:  42.6570014954,
		Longitude: -88.087600708,
		Timezone:  "America/Chicago",
	},
	"3WI2": {
		ICAO:      "3WI2",
		Name:      "Tesmer Airport",
		City:      "Waterloo",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 925,
		Latitude:  43.1528015137,
		Longitude: -88.9845962524,
		Timezone:  "America/Chicago",
	},
	"3WI3": {
		ICAO:      "3WI3",
		Name:      "Plover River Airfield",
		City:      "Bevent",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1185,
		Latitude:  44.7652015686,
		Longitude: -89.4073028564,
		Timezone:  "America/Chicago",
	},
	"3WI4": {
		ICAO:      "3WI4",
		Name:      "Flying 'O' Airport",
		City:      "Wausau",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1320,
		Latitude:  44.9235992432,
		Longitude: -89.4022979736,
		Timezone:  "America/Chicago",
	},
	"3WI5": {
		ICAO:      "3WI5",
		Name:      "Clover Valley Airport",
		City:      "Whitewater",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 850,
		Latitude:  42.8125,
		Longitude: -88.7265014648,
		Timezone:  "America/Chicago",
	},
	"3WI6": {
		ICAO:      "3WI6",
		Name:      "Melin Farms Airport",
		City:      "Clinton",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 885,
		Latitude:  42.5900993347,
		Longitude: -88.8746032715,
		Timezone:  "America/Chicago",
	},
	"3WI7": {
		ICAO:      "3WI7",
		Name:      "Bark River Airport",
		City:      "Dousman",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 875,
		Latitude:  43.0332984924,
		Longitude: -88.4581985474,
		Timezone:  "America/Chicago",
	},
	"3WI8": {
		ICAO:      "3WI8",
		Name:      "Plantation Pine Airport",
		City:      "Redgranite",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 825,
		Latitude:  44.0532989502,
		Longitude: -89.1218032837,
		Timezone:  "America/Chicago",
	},
	"3WI9": {
		ICAO:      "3WI9",
		Name:      "Rosenbaum Field",
		City:      "Chippewa Falls",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 940,
		Latitude:  44.9937591553,
		Longitude: -91.3777542114,
		Timezone:  "America/Chicago",
	},
	"3WN2": {
		ICAO:      "3WN2",
		Name:      "Old Dairy Airport",
		City:      "Washburn",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 725,
		Latitude:  46.6916007996,
		Longitude: -90.8706970215,
		Timezone:  "America/Chicago",
	},
	"3WN4": {
		ICAO:      "3WN4",
		Name:      "Mark's Park & Airfield",
		City:      "New London",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 855,
		Latitude:  44.3069000244,
		Longitude: -88.7711029053,
		Timezone:  "America/Chicago",
	},
	"3WN6": {
		ICAO:      "3WN6",
		Name:      "Knutson Farms Airport",
		City:      "Chetek",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1035,
		Latitude:  45.2389984131,
		Longitude: -91.7118988037,
		Timezone:  "America/Chicago",
	},
	"3WN7": {
		ICAO:      "3WN7",
		Name:      "Woodland Airstrip",
		City:      "Manitowoc",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 610,
		Latitude:  44.1500015259,
		Longitude: -87.6177978516,
		Timezone:  "America/Chicago",
	},
	"3WN8": {
		ICAO:      "3WN8",
		Name:      "Blunt Field",
		City:      "Chippewa Falls",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 930,
		Latitude:  44.9356002808,
		Longitude: -91.4307022095,
		Timezone:  "America/Chicago",
	},
	"3WN9": {
		ICAO:      "3WN9",
		Name:      "Brion Memorial Airport",
		City:      "Strum",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 900,
		Latitude:  44.5569000244,
		Longitude: -91.366897583,
		Timezone:  "America/Chicago",
	},
	"3XA0": {
		ICAO:      "3XA0",
		Name:      "Drennan Airport",
		City:      "Grandview",
		State:     "Texas",
		Country:   "US",
		Elevation: 726,
		Latitude:  32.313889,
		Longitude: -97.231111,
		Timezone:  "America/Chicago",
	},
	"3XA1": {
		ICAO:      "3XA1",
		Name:      "Hardy Field",
		City:      "Valley View",
		State:     "Texas",
		Country:   "US",
		Elevation: 670,
		Latitude:  33.47395,
		Longitude: -97.169267,
		Timezone:  "America/Chicago",
	},
	"3XA7": {
		ICAO:      "3XA7",
		Name:      "Eagle Rock Ranch Airport",
		City:      "Pontotoc",
		State:     "Texas",
		Country:   "US",
		Elevation: 1540,
		Latitude:  30.890817,
		Longitude: -99.006183,
		Timezone:  "America/Chicago",
	},
	"3XA8": {
		ICAO:      "3XA8",
		Name:      "Chicken Strip",
		City:      "Cresson",
		State:     "Texas",
		Country:   "US",
		Elevation: 1055,
		Latitude:  32.514167,
		Longitude: -97.595278,
		Timezone:  "America/Chicago",
	},
	"3XS0": {
		ICAO:      "3XS0",
		Name:      "Hartlee Field",
		City:      "Denton",
		State:     "Texas",
		Country:   "US",
		Elevation: 588,
		Latitude:  33.2685012817,
		Longitude: -97.0710983276,
		Timezone:  "America/Chicago",
	},
	"3XS1": {
		ICAO:      "3XS1",
		Name:      "Kubecka Aviation Airport",
		City:      "Palacios",
		State:     "Texas",
		Country:   "US",
		Elevation: 20,
		Latitude:  28.76720047,
		Longitude: -96.3069000244,
		Timezone:  "America/Chicago",
	},
	"3XS4": {
		ICAO:      "3XS4",
		Name:      "Jenkins Airport",
		City:      "Vidor",
		State:     "Texas",
		Country:   "US",
		Elevation: 15,
		Latitude:  30.1343994141,
		Longitude: -93.9359970093,
		Timezone:  "America/Chicago",
	},
	"3XS5": {
		ICAO:      "3XS5",
		Name:      "Star Smith Field",
		City:      "Von Ormy",
		State:     "Texas",
		Country:   "US",
		Elevation: 662,
		Latitude:  29.2980003357,
		Longitude: -98.6766967773,
		Timezone:  "America/Chicago",
	},
	"3XS8": {
		ICAO:      "3XS8",
		Name:      "Ken Ada Ranch Airport",
		City:      "Waller",
		State:     "Texas",
		Country:   "US",
		Elevation: 250,
		Latitude:  30.1308002472,
		Longitude: -95.9001998901,
		Timezone:  "America/Chicago",
	},
	"40AK": {
		ICAO:      "40AK",
		Name:      "Sterling Air Park",
		City:      "Sterling",
		State:     "Alaska",
		Country:   "US",
		Elevation: 90,
		Latitude:  60.5553016663,
		Longitude: -150.8419952393,
		Timezone:  "America/Anchorage",
	},
	"40AR": {
		ICAO:      "40AR",
		Name:      "Lost Bridge Village Airport",
		City:      "Garfield",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1440,
		Latitude:  36.3927993774,
		Longitude: -93.912399292,
		Timezone:  "America/Chicago",
	},
	"40AZ": {
		ICAO:      "40AZ",
		Name:      "Boulais Ranch Airport",
		City:      "Maricopa",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1240,
		Latitude:  33.0788993835,
		Longitude: -112.1289978027,
		Timezone:  "America/Phoenix",
	},
	"40FL": {
		ICAO:      "40FL",
		Name:      "Fred Babcock Airport",
		City:      "Punta Gorda",
		State:     "Florida",
		Country:   "US",
		Elevation: 38,
		Latitude:  26.885099411,
		Longitude: -81.7511978149,
		Timezone:  "America/New_York",
	},
	"40II": {
		ICAO:      "40II",
		Name:      "Keener Field",
		City:      "Butler",
		State:     "Indiana",
		Country:   "US",
		Elevation: 825,
		Latitude:  41.351398468,
		Longitude: -84.8169021606,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"40IL": {
		ICAO:      "40IL",
		Name:      "Antique Aerodrome",
		City:      "Cornell",
		State:     "Illinois",
		Country:   "US",
		Elevation: 683,
		Latitude:  41.0057983398,
		Longitude: -88.6580963135,
		Timezone:  "America/Chicago",
	},
	"40IN": {
		ICAO:      "40IN",
		Name:      "Cherry Hill Airport",
		City:      "Freetown",
		State:     "Indiana",
		Country:   "US",
		Elevation: 745,
		Latitude:  39.0741996765,
		Longitude: -86.1541976929,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"40KS": {
		ICAO:      "40KS",
		Name:      "Chanay Airport",
		City:      "Williamsburg",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1110,
		Latitude:  38.5400009155,
		Longitude: -95.4507980347,
		Timezone:  "America/Chicago",
	},
	"40KY": {
		ICAO:      "40KY",
		Name:      "Rooster Field",
		City:      "Eminence",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 840,
		Latitude:  38.2933998108,
		Longitude: -85.2050018311,
		Timezone:  "America/New_York",
	},
	"40LA": {
		ICAO:      "40LA",
		Name:      "Woodsland Plantation Airport",
		City:      "Monroe",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 60,
		Latitude:  32.3460006714,
		Longitude: -91.9695968628,
		Timezone:  "America/Chicago",
	},
	"40ME": {
		ICAO:      "40ME",
		Name:      "Tib Field",
		City:      "Fryeburg",
		State:     "Maine",
		Country:   "US",
		Elevation: 450,
		Latitude:  44.105892,
		Longitude: -70.955978,
		Timezone:  "America/New_York",
	},
	"40MN": {
		ICAO:      "40MN",
		Name:      "Turner Field",
		City:      "Bingham Lake",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1410,
		Latitude:  43.8666000366,
		Longitude: -95.0141983032,
		Timezone:  "America/Chicago",
	},
	"40MO": {
		ICAO:      "40MO",
		Name:      "Louise's Strip",
		City:      "Weston",
		State:     "Missouri",
		Country:   "US",
		Elevation: 945,
		Latitude:  39.4227981567,
		Longitude: -94.8376998901,
		Timezone:  "America/Chicago",
	},
	"40NE": {
		ICAO:      "40NE",
		Name:      "Landgren Ranch Airport",
		City:      "Bartlett",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2100,
		Latitude:  41.9749984741,
		Longitude: -98.5158996582,
		Timezone:  "America/Chicago",
	},
	"40OH": {
		ICAO:      "40OH",
		Name:      "Bucks Airport",
		City:      "Newbury",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1220,
		Latitude:  41.4511985779,
		Longitude: -81.2348022461,
		Timezone:  "America/New_York",
	},
	"40OI": {
		ICAO:      "40OI",
		Name:      "M.C.R. Airport",
		City:      "Lodi",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1155,
		Latitude:  41.0616989136,
		Longitude: -81.9835968018,
		Timezone:  "America/New_York",
	},
	"40OK": {
		ICAO:      "40OK",
		Name:      "Hilltop Airport",
		City:      "Oilton",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 930,
		Latitude:  36.1161003113,
		Longitude: -96.5381011963,
		Timezone:  "America/Chicago",
	},
	"40PN": {
		ICAO:      "40PN",
		Name:      "Eagles Mere Field Airport",
		City:      "Eagles Mere",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1900,
		Latitude:  41.3911556,
		Longitude: -76.6114028,
		Timezone:  "America/New_York",
	},
	"40TA": {
		ICAO:      "40TA",
		Name:      "Miles Field",
		City:      "Jewett",
		State:     "Texas",
		Country:   "US",
		Elevation: 460,
		Latitude:  31.3129997253,
		Longitude: -96.2204971313,
		Timezone:  "America/Chicago",
	},
	"40TE": {
		ICAO:      "40TE",
		Name:      "Sybert Farm Airport",
		City:      "Jarrell",
		State:     "Texas",
		Country:   "US",
		Elevation: 841,
		Latitude:  30.789100647,
		Longitude: -97.6549987793,
		Timezone:  "America/Chicago",
	},
	"40TS": {
		ICAO:      "40TS",
		Name:      "Square K Airport",
		City:      "West",
		State:     "Texas",
		Country:   "US",
		Elevation: 590,
		Latitude:  31.7723999023,
		Longitude: -97.1322021484,
		Timezone:  "America/Chicago",
	},
	"40WI": {
		ICAO:      "40WI",
		Name:      "Schubert Airstrip",
		City:      "Centerville",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 750,
		Latitude:  44.0583000183,
		Longitude: -91.4307022095,
		Timezone:  "America/Chicago",
	},
	"40XS": {
		ICAO:      "40XS",
		Name:      "Breakaway Park Airport",
		City:      "Cedar Park",
		State:     "Texas",
		Country:   "US",
		Elevation: 897,
		Latitude:  30.517999649,
		Longitude: -97.7806015015,
		Timezone:  "America/Chicago",
	},
	"41AK": {
		ICAO:      "41AK",
		Name:      "Settlers Bay Airstrip",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 75,
		Latitude:  61.5009002686,
		Longitude: -149.639694214,
		Timezone:  "America/Anchorage",
	},
	"41AR": {
		ICAO:      "41AR",
		Name:      "Ashworth Airport",
		City:      "Arkadelphia",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 501,
		Latitude:  34.123298645,
		Longitude: -93.3497009277,
		Timezone:  "America/Chicago",
	},
	"41AZ": {
		ICAO:      "41AZ",
		Name:      "Ak Chin Community Airfield",
		City:      "Maricopa",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1210,
		Latitude:  32.9861984253,
		Longitude: -112.0260009766,
		Timezone:  "America/Phoenix",
	},
	"41CA": {
		ICAO:      "41CA",
		Name:      "Silver Creek Ranch Airport",
		City:      "Forest Glen",
		State:     "California",
		Country:   "US",
		Elevation: 2511,
		Latitude:  40.3176002502,
		Longitude: -123.2539978027,
		Timezone:  "America/Los_Angeles",
	},
	"41CO": {
		ICAO:      "41CO",
		Name:      "Flying Dog Ranch Airstrip",
		City:      "Meeker",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7020,
		Latitude:  39.971111,
		Longitude: -107.611111,
		Timezone:  "America/Denver",
	},
	"41ID": {
		ICAO:      "41ID",
		Name:      "Black'S Airfield",
		City:      "Kuna",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2788,
		Latitude:  43.471383,
		Longitude: -116.326417,
		Timezone:  "America/Boise",
	},
	"41IL": {
		ICAO:      "41IL",
		Name:      "Alcock RLA Restricted Landing Area",
		City:      "Lindenwood",
		State:     "Illinois",
		Country:   "US",
		Elevation: 780,
		Latitude:  42.0694999695,
		Longitude: -89.0334014893,
		Timezone:  "America/Chicago",
	},
	"41IS": {
		ICAO:      "41IS",
		Name:      "Rees Field",
		City:      "Grand Ridge",
		State:     "Illinois",
		Country:   "US",
		Elevation: 675,
		Latitude:  41.204498291,
		Longitude: -88.8033981323,
		Timezone:  "America/Chicago",
	},
	"41KS": {
		ICAO:      "41KS",
		Name:      "Flying Z Ranch Airport",
		City:      "Spring Hill",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1075,
		Latitude:  38.7355003357,
		Longitude: -94.7973022461,
		Timezone:  "America/Chicago",
	},
	"41MN": {
		ICAO:      "41MN",
		Name:      "Shannon Field",
		City:      "Clinton",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1000,
		Latitude:  45.4268989563,
		Longitude: -96.5895004272,
		Timezone:  "America/Chicago",
	},
	"41MO": {
		ICAO:      "41MO",
		Name:      "Textor Airport",
		City:      "Willard",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1140,
		Latitude:  37.3609008789,
		Longitude: -93.4957962036,
		Timezone:  "America/Chicago",
	},
	"41MT": {
		ICAO:      "41MT",
		Name:      "R & R Field",
		City:      "Heron",
		State:     "Montana",
		Country:   "US",
		Elevation: 2274,
		Latitude:  48.08715,
		Longitude: -116.046267,
		Timezone:  "America/Denver",
	},
	"41NC": {
		ICAO:      "41NC",
		Name:      "Sloop Airport",
		City:      "Kannapolis",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 794,
		Latitude:  35.5218009949,
		Longitude: -80.5220031738,
		Timezone:  "America/New_York",
	},
	"41NE": {
		ICAO:      "41NE",
		Name:      "Vandersnick Airport",
		City:      "Ewing",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1990,
		Latitude:  42.1556015015,
		Longitude: -98.5003967285,
		Timezone:  "America/Chicago",
	},
	"41NK": {
		ICAO:      "41NK",
		Name:      "Winchell Mountain Airport",
		City:      "Millerton",
		State:     "New York",
		Country:   "US",
		Elevation: 1141,
		Latitude:  41.947778,
		Longitude: -73.518055,
		Timezone:  "America/New_York",
	},
	"41NY": {
		ICAO:      "41NY",
		Name:      "Bonebender Airport",
		City:      "Essex",
		State:     "New York",
		Country:   "US",
		Elevation: 336,
		Latitude:  44.3124008179,
		Longitude: -73.3849029541,
		Timezone:  "America/New_York",
	},
	"41OH": {
		ICAO:      "41OH",
		Name:      "Sleepy Hollow Airport",
		City:      "New Franklin",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1050,
		Latitude:  40.7836990356,
		Longitude: -81.0989990234,
		Timezone:  "America/New_York",
	},
	"41PA": {
		ICAO:      "41PA",
		Name:      "Deitch Airport",
		City:      "New Kingstown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 420,
		Latitude:  40.257900238,
		Longitude: -77.0813980103,
		Timezone:  "America/New_York",
	},
	"41TA": {
		ICAO:      "41TA",
		Name:      "Circle P Ranch Airport",
		City:      "Glen Rose",
		State:     "Texas",
		Country:   "US",
		Elevation: 800,
		Latitude:  32.2375984192,
		Longitude: -97.8780975342,
		Timezone:  "America/Chicago",
	},
	"41TN": {
		ICAO:      "41TN",
		Name:      "Stonewall Airpark",
		City:      "Milton",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 760,
		Latitude:  35.9007987976,
		Longitude: -86.2071990967,
		Timezone:  "America/Chicago",
	},
	"41TS": {
		ICAO:      "41TS",
		Name:      "Flying T Ranch Airport",
		City:      "Westminster",
		State:     "Texas",
		Country:   "US",
		Elevation: 705,
		Latitude:  33.3783988953,
		Longitude: -96.4483032227,
		Timezone:  "America/Chicago",
	},
	"41TX": {
		ICAO:      "41TX",
		Name:      "Henington Airport",
		City:      "Wolfe City",
		State:     "Texas",
		Country:   "US",
		Elevation: 645,
		Latitude:  33.3748016357,
		Longitude: -96.0821990967,
		Timezone:  "America/Chicago",
	},
	"41WI": {
		ICAO:      "41WI",
		Name:      "Paddock Field",
		City:      "Elkhorn",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 950,
		Latitude:  42.7386016846,
		Longitude: -88.5798034668,
		Timezone:  "America/Chicago",
	},
	"41XS": {
		ICAO:      "41XS",
		Name:      "Macho Grande Airport",
		City:      "Taylor",
		State:     "Texas",
		Country:   "US",
		Elevation: 600,
		Latitude:  30.539100647,
		Longitude: -97.4188995361,
		Timezone:  "America/Chicago",
	},
	"42AK": {
		ICAO:      "42AK",
		Name:      "Wicker Airport",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 314,
		Latitude:  61.568901062,
		Longitude: -149.4850006104,
		Timezone:  "America/Anchorage",
	},
	"42AZ": {
		ICAO:      "42AZ",
		Name:      "Orme School Airport",
		City:      "Mayer",
		State:     "Arizona",
		Country:   "US",
		Elevation: 3934,
		Latitude:  34.4277992249,
		Longitude: -112.0650024414,
		Timezone:  "America/Phoenix",
	},
	"42CN": {
		ICAO:      "42CN",
		Name:      "Peg Field",
		City:      "Reedley",
		State:     "California",
		Country:   "US",
		Elevation: 460,
		Latitude:  36.7090988159,
		Longitude: -119.4049987793,
		Timezone:  "America/Los_Angeles",
	},
	"42II": {
		ICAO:      "42II",
		Name:      "Norm's Airpark",
		City:      "La Porte",
		State:     "Indiana",
		Country:   "US",
		Elevation: 740,
		Latitude:  41.6845016479,
		Longitude: -86.7677993774,
		Timezone:  "America/Chicago",
	},
	"42KY": {
		ICAO:      "42KY",
		Name:      "Pirates Cove Airport",
		City:      "Benton",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 380,
		Latitude:  36.7764015198,
		Longitude: -88.2222976685,
		Timezone:  "America/Chicago",
	},
	"42MD": {
		ICAO:      "42MD",
		Name:      "Herrington Field",
		City:      "Oakland",
		State:     "Maryland",
		Country:   "US",
		Elevation: 2470,
		Latitude:  39.46829987,
		Longitude: -79.4253006,
		Timezone:  "America/New_York",
	},
	"42MI": {
		ICAO:      "42MI",
		Name:      "Midway Airport",
		City:      "Albion",
		State:     "Michigan",
		Country:   "US",
		Elevation: 963,
		Latitude:  42.2153015137,
		Longitude: -84.8069000244,
		Timezone:  "America/Detroit",
	},
	"42MN": {
		ICAO:      "42MN",
		Name:      "Kral's Personal Use Landing Field",
		City:      "Corcoran",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 990,
		Latitude:  45.0840988159,
		Longitude: -93.5643997192,
		Timezone:  "America/Chicago",
	},
	"42MO": {
		ICAO:      "42MO",
		Name:      "Aire Parque Airport",
		City:      "Festus",
		State:     "Missouri",
		Country:   "US",
		Elevation: 710,
		Latitude:  38.1231002808,
		Longitude: -90.388999939,
		Timezone:  "America/Chicago",
	},
	"42NC": {
		ICAO:      "42NC",
		Name:      "Spring Paths Airport",
		City:      "Spring Hope",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 290,
		Latitude:  35.9578018188,
		Longitude: -78.1706008911,
		Timezone:  "America/New_York",
	},
	"42NE": {
		ICAO:      "42NE",
		Name:      "Packard Ranch Airport",
		City:      "Arthur",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3524,
		Latitude:  41.4343986511,
		Longitude: -101.6809997559,
		Timezone:  "America/Denver",
	},
	"42NY": {
		ICAO:      "42NY",
		Name:      "Walter's Field",
		City:      "East Homer",
		State:     "New York",
		Country:   "US",
		Elevation: 1560,
		Latitude:  42.6689987183,
		Longitude: -76.0561981201,
		Timezone:  "America/New_York",
	},
	"42OH": {
		ICAO:      "42OH",
		Name:      "Dunn Field",
		City:      "Newton Falls",
		State:     "Ohio",
		Country:   "US",
		Elevation: 955,
		Latitude:  41.1558990479,
		Longitude: -81.0039978027,
		Timezone:  "America/New_York",
	},
	"42OR": {
		ICAO:      "42OR",
		Name:      "Shotgun Ranch Airstrip",
		City:      "Post",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3430,
		Latitude:  44.1342010498,
		Longitude: -120.3229980469,
		Timezone:  "America/Los_Angeles",
	},
	"42TE": {
		ICAO:      "42TE",
		Name:      "Ball Airport",
		City:      "Victoria",
		State:     "Texas",
		Country:   "US",
		Elevation: 124,
		Latitude:  28.8878002167,
		Longitude: -97.0211029053,
		Timezone:  "America/Chicago",
	},
	"42TN": {
		ICAO:      "42TN",
		Name:      "Oliver Landing Airport",
		City:      "Paris",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 475,
		Latitude:  36.411944,
		Longitude: -88.265556,
		Timezone:  "America/Chicago",
	},
	"42TX": {
		ICAO:      "42TX",
		Name:      "Magee Airport",
		City:      "Wolfe City",
		State:     "Texas",
		Country:   "US",
		Elevation: 620,
		Latitude:  33.3167991638,
		Longitude: -96.0669021606,
		Timezone:  "America/Chicago",
	},
	"42VA": {
		ICAO:      "42VA",
		Name:      "Virginia Beach Airport",
		City:      "Virginia Beach",
		State:     "Virginia",
		Country:   "US",
		Elevation: 15,
		Latitude:  36.6781997681,
		Longitude: -76.0329971313,
		Timezone:  "America/New_York",
	},
	"42VT": {
		ICAO:      "42VT",
		Name:      "Frogs End Airport",
		City:      "Shelburne",
		State:     "Vermont",
		Country:   "US",
		Elevation: 320,
		Latitude:  44.3669013977,
		Longitude: -73.1501998901,
		Timezone:  "America/New_York",
	},
	"42WA": {
		ICAO:      "42WA",
		Name:      "Sheffels Ranch Airport",
		City:      "Wilbur",
		State:     "Washington",
		Country:   "US",
		Elevation: 2222,
		Latitude:  47.817199707,
		Longitude: -118.8010025024,
		Timezone:  "America/Los_Angeles",
	},
	"42XS": {
		ICAO:      "42XS",
		Name:      "Temple Ranch Airport",
		City:      "Freer",
		State:     "Texas",
		Country:   "US",
		Elevation: 490,
		Latitude:  27.957444,
		Longitude: -98.403889,
		Timezone:  "America/Chicago",
	},
	"43AK": {
		ICAO:      "43AK",
		Name:      "Kalmbach Airport",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 445,
		Latitude:  61.6007995605,
		Longitude: -149.578994751,
		Timezone:  "America/Anchorage",
	},
	"43AR": {
		ICAO:      "43AR",
		Name:      "Haigwood Landing Strip",
		City:      "Newport",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 230,
		Latitude:  35.6114006042,
		Longitude: -91.2414016724,
		Timezone:  "America/Chicago",
	},
	"43CA": {
		ICAO:      "43CA",
		Name:      "Dick Taylor Airstrip",
		City:      "Kelso",
		State:     "California",
		Country:   "US",
		Elevation: 2611,
		Latitude:  34.8819007874,
		Longitude: -115.733001709,
		Timezone:  "America/Los_Angeles",
	},
	"43CL": {
		ICAO:      "43CL",
		Name:      "Dick Dale Skyranch Airport",
		City:      "Twentynine Palms",
		State:     "California",
		Country:   "US",
		Elevation: 1793,
		Latitude:  34.2061004639,
		Longitude: -115.9980010986,
		Timezone:  "America/Los_Angeles",
	},
	"43CN": {
		ICAO:      "43CN",
		Name:      "Mazza Airport",
		City:      "Petaluma",
		State:     "California",
		Country:   "US",
		Elevation: 20,
		Latitude:  38.1851997375,
		Longitude: -122.6019973755,
		Timezone:  "America/Los_Angeles",
	},
	"43CO": {
		ICAO:      "43CO",
		Name:      "Kostroski Airport",
		City:      "Franktown",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6700,
		Latitude:  39.3666992188,
		Longitude: -104.6780014038,
		Timezone:  "America/Denver",
	},
	"43FL": {
		ICAO:      "43FL",
		Name:      "Fly'N R Ranch Airport",
		City:      "Umatilla",
		State:     "Florida",
		Country:   "US",
		Elevation: 78,
		Latitude:  28.9841995239,
		Longitude: -81.8170013428,
		Timezone:  "America/New_York",
	},
	"43IG": {
		ICAO:      "43IG",
		Name:      "Grayland Intergalactic Airport",
		City:      "Grayland",
		State:     "Washington",
		Country:   "US",
		Elevation: 280,
		Latitude:  46.801528,
		Longitude: -124.072944,
		Timezone:  "America/Los_Angeles",
	},
	"43IN": {
		ICAO:      "43IN",
		Name:      "Hepler Airport",
		City:      "St Wendel",
		State:     "Indiana",
		Country:   "US",
		Elevation: 460,
		Latitude:  38.1375007629,
		Longitude: -87.676399231,
		Timezone:  "America/Chicago",
	},
	"43KS": {
		ICAO:      "43KS",
		Name:      "Smith Airport",
		City:      "Manhattan",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1100,
		Latitude:  39.1018981934,
		Longitude: -96.4178009033,
		Timezone:  "America/Chicago",
	},
	"43LA": {
		ICAO:      "43LA",
		Name:      "Slay'S Airstrip",
		City:      "Belcher",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 177,
		Latitude:  32.747643,
		Longitude: -93.874708,
		Timezone:  "America/Chicago",
	},
	"43MI": {
		ICAO:      "43MI",
		Name:      "W Gladstone Airport",
		City:      "Gladstone",
		State:     "Michigan",
		Country:   "US",
		Elevation: 720,
		Latitude:  45.8591003418,
		Longitude: -87.1167984009,
		Timezone:  "America/Detroit",
	},
	"43MO": {
		ICAO:      "43MO",
		Name:      "Riordan Airport",
		City:      "Pleasant Hill",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1000,
		Latitude:  38.7832984924,
		Longitude: -94.3683013916,
		Timezone:  "America/Chicago",
	},
	"43NC": {
		ICAO:      "43NC",
		Name:      "Bahnson Airport",
		City:      "Mocksville",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 810,
		Latitude:  36.0208015442,
		Longitude: -80.5164031982,
		Timezone:  "America/New_York",
	},
	"43NE": {
		ICAO:      "43NE",
		Name:      "Onion Crest Airpark",
		City:      "Riverdale",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2205,
		Latitude:  40.7750015259,
		Longitude: -99.1547012329,
		Timezone:  "America/Chicago",
	},
	"43NY": {
		ICAO:      "43NY",
		Name:      "Watercolor Airport",
		City:      "Cherry Valley",
		State:     "New York",
		Country:   "US",
		Elevation: 1993,
		Latitude:  42.7846984863,
		Longitude: -74.8024978638,
		Timezone:  "America/New_York",
	},
	"43OA": {
		ICAO:      "43OA",
		Name:      "Anderson Airfield",
		City:      "Bellbrook",
		State:     "Ohio",
		Country:   "US",
		Elevation: 940,
		Latitude:  39.580972,
		Longitude: -84.092194,
		Timezone:  "America/New_York",
	},
	"43OI": {
		ICAO:      "43OI",
		Name:      "Windswept Airport",
		City:      "North Baltimore",
		State:     "Ohio",
		Country:   "US",
		Elevation: 710,
		Latitude:  41.1986999512,
		Longitude: -83.7007980347,
		Timezone:  "America/New_York",
	},
	"43OK": {
		ICAO:      "43OK",
		Name:      "Biggs Skypatch Airport",
		City:      "Wellston",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1000,
		Latitude:  35.6666984558,
		Longitude: -96.9835968018,
		Timezone:  "America/Chicago",
	},
	"43OR": {
		ICAO:      "43OR",
		Name:      "Inspiration Airport",
		City:      "Bend",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3400,
		Latitude:  44.6225013733,
		Longitude: -122.7519989014,
		Timezone:  "America/Los_Angeles",
	},
	"43PA": {
		ICAO:      "43PA",
		Name:      "Kolb Airport",
		City:      "Spring City",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 290,
		Latitude:  40.1600990295,
		Longitude: -75.5463027954,
		Timezone:  "America/New_York",
	},
	"43TA": {
		ICAO:      "43TA",
		Name:      "Diamond Seven Ranch Airport",
		City:      "Morgan",
		State:     "Texas",
		Country:   "US",
		Elevation: 797,
		Latitude:  32.0251998901,
		Longitude: -97.6264038086,
		Timezone:  "America/Chicago",
	},
	"43TN": {
		ICAO:      "43TN",
		Name:      "Roseanne Airport",
		City:      "Morrison",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1050,
		Latitude:  35.618056,
		Longitude: -86.0025,
		Timezone:  "America/Chicago",
	},
	"43TS": {
		ICAO:      "43TS",
		Name:      "Fraser Farm Airport",
		City:      "Lindale",
		State:     "Texas",
		Country:   "US",
		Elevation: 570,
		Latitude:  32.5662002563,
		Longitude: -95.4682998657,
		Timezone:  "America/Chicago",
	},
	"43TX": {
		ICAO:      "43TX",
		Name:      "Mid-Valley Dusters Inc Airport",
		City:      "Alamo",
		State:     "Texas",
		Country:   "US",
		Elevation: 91,
		Latitude:  26.1530990601,
		Longitude: -98.1350021362,
		Timezone:  "America/Chicago",
	},
	"43WI": {
		ICAO:      "43WI",
		Name:      "Eagle Ridge Ultralightport",
		City:      "Colfax",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1067,
		Latitude:  45.031533,
		Longitude: -91.7325,
		Timezone:  "America/Chicago",
	},
	"44AK": {
		ICAO:      "44AK",
		Name:      "West Papoose Lake Airpark",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 230,
		Latitude:  61.5334014893,
		Longitude: -150.1020050049,
		Timezone:  "America/Anchorage",
	},
	"44AZ": {
		ICAO:      "44AZ",
		Name:      "Sandhill Ranch Airport",
		City:      "Page",
		State:     "Arizona",
		Country:   "US",
		Elevation: 5868,
		Latitude:  36.9319000244,
		Longitude: -111.9700012207,
		Timezone:  "America/Phoenix",
	},
	"44CA": {
		ICAO:      "44CA",
		Name:      "W R Byron Airport",
		City:      "Blythe",
		State:     "California",
		Country:   "US",
		Elevation: 400,
		Latitude:  33.6791992188,
		Longitude: -114.6439971924,
		Timezone:  "America/Los_Angeles",
	},
	"44CL": {
		ICAO:      "44CL",
		Name:      "Cortopassi Airport",
		City:      "Linden",
		State:     "California",
		Country:   "US",
		Elevation: 80,
		Latitude:  38.0138015747,
		Longitude: -121.1259994507,
		Timezone:  "America/Los_Angeles",
	},
	"44CO": {
		ICAO:      "44CO",
		Name:      "Redlands Airport",
		City:      "Hotchkiss",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6100,
		Latitude:  38.8367004395,
		Longitude: -107.7929992676,
		Timezone:  "America/Denver",
	},
	"44GA": {
		ICAO:      "44GA",
		Name:      "S&S Flying Service Airport",
		City:      "Pineview",
		State:     "Georgia",
		Country:   "US",
		Elevation: 260,
		Latitude:  32.1043014526,
		Longitude: -83.4829025269,
		Timezone:  "America/New_York",
	},
	"44ID": {
		ICAO:      "44ID",
		Name:      "Boulder Creek Airstrip",
		City:      "Donnelly",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4950,
		Latitude:  44.736942,
		Longitude: -116.074158,
		Timezone:  "America/Boise",
	},
	"44II": {
		ICAO:      "44II",
		Name:      "The Wolf Den Airport",
		City:      "Roanoke",
		State:     "Indiana",
		Country:   "US",
		Elevation: 815,
		Latitude:  40.9530982971,
		Longitude: -85.3865966797,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"44IN": {
		ICAO:      "44IN",
		Name:      "Rush STOLport",
		City:      "West Lafayette",
		State:     "Indiana",
		Country:   "US",
		Elevation: 675,
		Latitude:  40.4998016357,
		Longitude: -86.9550018311,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"44IS": {
		ICAO:      "44IS",
		Name:      "Potter Airport",
		City:      "Barry",
		State:     "Illinois",
		Country:   "US",
		Elevation: 700,
		Latitude:  39.7448005676,
		Longitude: -91.0748977661,
		Timezone:  "America/Chicago",
	},
	"44KS": {
		ICAO:      "44KS",
		Name:      "Vankirk Airport",
		City:      "Wichita",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1350,
		Latitude:  37.6102981567,
		Longitude: -97.1624984741,
		Timezone:  "America/Chicago",
	},
	"44KY": {
		ICAO:      "44KY",
		Name:      "Duff Airport",
		City:      "Chavies",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 820,
		Latitude:  37.3526000977,
		Longitude: -83.3423995972,
		Timezone:  "America/New_York",
	},
	"44LA": {
		ICAO:      "44LA",
		Name:      "B T & K H Ranch Airport",
		City:      "Dry Creek",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 110,
		Latitude:  30.6473999023,
		Longitude: -93.0551986694,
		Timezone:  "America/Chicago",
	},
	"44MI": {
		ICAO:      "44MI",
		Name:      "Airbatco Field",
		City:      "Mulliken",
		State:     "Michigan",
		Country:   "US",
		Elevation: 852,
		Latitude:  42.773601532,
		Longitude: -84.9000015259,
		Timezone:  "America/Detroit",
	},
	"44MO": {
		ICAO:      "44MO",
		Name:      "Butch's Strip STOLport",
		City:      "St Joseph",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1000,
		Latitude:  39.7097015381,
		Longitude: -94.8022003174,
		Timezone:  "America/Chicago",
	},
	"44MU": {
		ICAO:      "44MU",
		Name:      "Lynch Field",
		City:      "Granby",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1170,
		Latitude:  36.9081001282,
		Longitude: -94.3099975586,
		Timezone:  "America/Chicago",
	},
	"44NC": {
		ICAO:      "44NC",
		Name:      "Broadway Airfield",
		City:      "Landis",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 650,
		Latitude:  35.5433006287,
		Longitude: -80.6519012451,
		Timezone:  "America/New_York",
	},
	"44ND": {
		ICAO:      "44ND",
		Name:      "Fredericks Ranch Airport",
		City:      "Halliday",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 2225,
		Latitude:  47.5278015137,
		Longitude: -102.2409973145,
		Timezone:  "America/Denver",
	},
	"44NY": {
		ICAO:      "44NY",
		Name:      "Tomcat Airport",
		City:      "Fort Plain",
		State:     "New York",
		Country:   "US",
		Elevation: 460,
		Latitude:  42.934299469,
		Longitude: -74.6523971558,
		Timezone:  "America/New_York",
	},
	"44OK": {
		ICAO:      "44OK",
		Name:      "Sky High Airport",
		City:      "Crescent",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1100,
		Latitude:  35.9542007446,
		Longitude: -97.5492019653,
		Timezone:  "America/Chicago",
	},
	"44OR": {
		ICAO:      "44OR",
		Name:      "Compton Airport",
		City:      "Canby",
		State:     "Oregon",
		Country:   "US",
		Elevation: 160,
		Latitude:  45.2223014832,
		Longitude: -122.7269973755,
		Timezone:  "America/Los_Angeles",
	},
	"44PA": {
		ICAO:      "44PA",
		Name:      "Pennfield Farm Airport",
		City:      "Spring Mills",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1200,
		Latitude:  40.8501014709,
		Longitude: -77.6164016724,
		Timezone:  "America/New_York",
	},
	"44TE": {
		ICAO:      "44TE",
		Name:      "Shirley Williams Airport",
		City:      "Kingsland",
		State:     "Texas",
		Country:   "US",
		Elevation: 880,
		Latitude:  30.6835002899,
		Longitude: -98.4169998169,
		Timezone:  "America/Chicago",
	},
	"44TN": {
		ICAO:      "44TN",
		Name:      "Stone Field",
		City:      "Louisville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 890,
		Latitude:  35.84090042,
		Longitude: -84.07569885,
		Timezone:  "America/New_York",
	},
	"44TS": {
		ICAO:      "44TS",
		Name:      "Mc David Ranch Airport",
		City:      "Llano",
		State:     "Texas",
		Country:   "US",
		Elevation: 1407,
		Latitude:  30.5776996613,
		Longitude: -98.8337020874,
		Timezone:  "America/Chicago",
	},
	"44TX": {
		ICAO:      "44TX",
		Name:      "White Oak Airport",
		City:      "Fredericksburg",
		State:     "Texas",
		Country:   "US",
		Elevation: 1875,
		Latitude:  30.2105007172,
		Longitude: -99.0948028564,
		Timezone:  "America/Chicago",
	},
	"44VA": {
		ICAO:      "44VA",
		Name:      "Big Hill Airport",
		City:      "Eagle Rock",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1620,
		Latitude:  37.7312011719,
		Longitude: -79.8112030029,
		Timezone:  "America/New_York",
	},
	"44WI": {
		ICAO:      "44WI",
		Name:      "Stoiber Airport",
		City:      "Spencer",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1298,
		Latitude:  44.772217,
		Longitude: -90.385067,
		Timezone:  "America/Chicago",
	},
	"44WY": {
		ICAO:      "44WY",
		Name:      "Underwood Airport",
		City:      "Douglas",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4863,
		Latitude:  42.8511,
		Longitude: -105.496217,
		Timezone:  "America/Denver",
	},
	"45AK": {
		ICAO:      "45AK",
		Name:      "Cherokee Airport",
		City:      "Delta Junction",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1300,
		Latitude:  63.9617996216,
		Longitude: -145.4700012207,
		Timezone:  "America/Anchorage",
	},
	"45AR": {
		ICAO:      "45AR",
		Name:      "Thunder Ridge Ranch",
		City:      "Flippin",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 789,
		Latitude:  36.3671989441,
		Longitude: -92.6401977539,
		Timezone:  "America/Chicago",
	},
	"45AZ": {
		ICAO:      "45AZ",
		Name:      "Pine Springs Airport",
		City:      "Pine Springs",
		State:     "Arizona",
		Country:   "US",
		Elevation: 6930,
		Latitude:  35.4009017944,
		Longitude: -109.2760009766,
		Timezone:  "America/Denver",
	},
	"45CL": {
		ICAO:      "45CL",
		Name:      "Hell'er High Water Airport",
		City:      "Zenia",
		State:     "California",
		Country:   "US",
		Elevation: 2493,
		Latitude:  40.1431999207,
		Longitude: -123.3980026245,
		Timezone:  "America/Los_Angeles",
	},
	"45CN": {
		ICAO:      "45CN",
		Name:      "Hermitage Airport",
		City:      "Groveland",
		State:     "California",
		Country:   "US",
		Elevation: 2700,
		Latitude:  37.8801994324,
		Longitude: -120.1940002441,
		Timezone:  "America/Los_Angeles",
	},
	"45FL": {
		ICAO:      "45FL",
		Name:      "Moss Meadows Airport",
		City:      "Live Oak",
		State:     "Florida",
		Country:   "US",
		Elevation: 155,
		Latitude:  30.2374000549,
		Longitude: -82.90650177,
		Timezone:  "America/New_York",
	},
	"45II": {
		ICAO:      "45II",
		Name:      "Cummings Field",
		City:      "Rolling Prairie",
		State:     "Indiana",
		Country:   "US",
		Elevation: 800,
		Latitude:  41.6337013245,
		Longitude: -86.585899353,
		Timezone:  "America/Chicago",
	},
	"45IN": {
		ICAO:      "45IN",
		Name:      "Aerobatic Practice Airport",
		City:      "North Vernon",
		State:     "Indiana",
		Country:   "US",
		Elevation: 681,
		Latitude:  38.9963989258,
		Longitude: -85.7480010986,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"45KS": {
		ICAO:      "45KS",
		Name:      "Liebau Ranch Airport",
		City:      "Grenola",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1250,
		Latitude:  37.4039001465,
		Longitude: -96.4533004761,
		Timezone:  "America/Chicago",
	},
	"45MI": {
		ICAO:      "45MI",
		Name:      "Franklin's Airport",
		City:      "Barbeau",
		State:     "Michigan",
		Country:   "US",
		Elevation: 630,
		Latitude:  46.2639007568,
		Longitude: -84.1264038086,
		Timezone:  "America/Detroit",
	},
	"45MN": {
		ICAO:      "45MN",
		Name:      "Kaml Airstrip",
		City:      "Euclid",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 898,
		Latitude:  47.9261016846,
		Longitude: -96.6141967773,
		Timezone:  "America/Chicago",
	},
	"45MO": {
		ICAO:      "45MO",
		Name:      "Waldmeister Farm Airport",
		City:      "St Peters",
		State:     "Missouri",
		Country:   "US",
		Elevation: 424,
		Latitude:  38.8473014832,
		Longitude: -90.638999939,
		Timezone:  "America/Chicago",
	},
	"45MT": {
		ICAO:      "45MT",
		Name:      "Pester Airport",
		City:      "Hingham",
		State:     "Montana",
		Country:   "US",
		Elevation: 3055,
		Latitude:  48.628333,
		Longitude: -110.469667,
		Timezone:  "America/Denver",
	},
	"45NC": {
		ICAO:      "45NC",
		Name:      "Glover Airport",
		City:      "Seaboard",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 120,
		Latitude:  36.5280990601,
		Longitude: -77.4546966553,
		Timezone:  "America/New_York",
	},
	"45OG": {
		ICAO:      "45OG",
		Name:      "Rugg Ranches Airport",
		City:      "Pilot Rock",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1890,
		Latitude:  45.476389,
		Longitude: -118.791667,
		Timezone:  "America/Los_Angeles",
	},
	"45OH": {
		ICAO:      "45OH",
		Name:      "Boober Airport",
		City:      "New Richmond",
		State:     "Ohio",
		Country:   "US",
		Elevation: 854,
		Latitude:  38.975994,
		Longitude: -84.219511,
		Timezone:  "America/New_York",
	},
	"45OI": {
		ICAO:      "45OI",
		Name:      "Plane Country Airport",
		City:      "Strasburg",
		State:     "Ohio",
		Country:   "US",
		Elevation: 920,
		Latitude:  40.5611991882,
		Longitude: -81.5457000732,
		Timezone:  "America/New_York",
	},
	"45OK": {
		ICAO:      "45OK",
		Name:      "Belleview Landing Airport",
		City:      "South Coffeyville",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 775,
		Latitude:  36.9126014709,
		Longitude: -95.6029968262,
		Timezone:  "America/Chicago",
	},
	"45OL": {
		ICAO:      "45OL",
		Name:      "Stilwell Airport",
		City:      "Stilwell",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1125,
		Latitude:  35.837600708,
		Longitude: -94.6251983643,
		Timezone:  "America/Chicago",
	},
	"45OR": {
		ICAO:      "45OR",
		Name:      "Benham Airport",
		City:      "Coquille",
		State:     "Oregon",
		Country:   "US",
		Elevation: 280,
		Latitude:  43.154800415,
		Longitude: -124.1709976196,
		Timezone:  "America/Los_Angeles",
	},
	"45TE": {
		ICAO:      "45TE",
		Name:      "Deussen Field",
		City:      "Ponder",
		State:     "Texas",
		Country:   "US",
		Elevation: 790,
		Latitude:  33.1917991638,
		Longitude: -97.325302124,
		Timezone:  "America/Chicago",
	},
	"45TN": {
		ICAO:      "45TN",
		Name:      "Darnell's Landings Airport",
		City:      "Church Hill",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1150,
		Latitude:  36.5066986084,
		Longitude: -82.7117004395,
		Timezone:  "America/New_York",
	},
	"45TX": {
		ICAO:      "45TX",
		Name:      "West Airpark",
		City:      "West",
		State:     "Texas",
		Country:   "US",
		Elevation: 563,
		Latitude:  31.7523994446,
		Longitude: -97.0982971191,
		Timezone:  "America/Chicago",
	},
	"45VA": {
		ICAO:      "45VA",
		Name:      "Frog Hollow Farm Airport",
		City:      "West Point",
		State:     "Virginia",
		Country:   "US",
		Elevation: 30,
		Latitude:  37.695098877,
		Longitude: -76.9950027466,
		Timezone:  "America/New_York",
	},
	"45WI": {
		ICAO:      "45WI",
		Name:      "Windsong Farm Airport",
		City:      "Woodville",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1250,
		Latitude:  45,
		Longitude: -92.2835006714,
		Timezone:  "America/Chicago",
	},
	"45WY": {
		ICAO:      "45WY",
		Name:      "American Falconry Airport",
		City:      "Casper",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 5722,
		Latitude:  42.889733,
		Longitude: -106.674866,
		Timezone:  "America/Denver",
	},
	"45XS": {
		ICAO:      "45XS",
		Name:      "Ghost Apache Airport",
		City:      "Cotulla",
		State:     "Texas",
		Country:   "US",
		Elevation: 330,
		Latitude:  28.1403007507,
		Longitude: -98.912902832,
		Timezone:  "America/Chicago",
	},
	"46AK": {
		ICAO:      "46AK",
		Name:      "Bear Cove Farm Airport",
		City:      "Homer",
		State:     "Alaska",
		Country:   "US",
		Elevation: 115,
		Latitude:  59.7383003235,
		Longitude: -151.0299987793,
		Timezone:  "America/Anchorage",
	},
	"46AR": {
		ICAO:      "46AR",
		Name:      "Robert Chris Mc Intosh Airport",
		City:      "Sherwood",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 242,
		Latitude:  34.8461990356,
		Longitude: -92.1720962524,
		Timezone:  "America/Chicago",
	},
	"46AZ": {
		ICAO:      "46AZ",
		Name:      "Pinon Airport",
		City:      "Pinon",
		State:     "Arizona",
		Country:   "US",
		Elevation: 6315,
		Latitude:  36.0910987854,
		Longitude: -110.233001709,
		Timezone:  "America/Denver",
	},
	"46CA": {
		ICAO:      "46CA",
		Name:      "Rancho Vallecito Airport",
		City:      "Julian",
		State:     "California",
		Country:   "US",
		Elevation: 1680,
		Latitude:  32.9709014893,
		Longitude: -116.408996582,
		Timezone:  "America/Los_Angeles",
	},
	"46CN": {
		ICAO:      "46CN",
		Name:      "Crystal Airport",
		City:      "Llano",
		State:     "California",
		Country:   "US",
		Elevation: 3420,
		Latitude:  34.4846992493,
		Longitude: -117.8259963989,
		Timezone:  "America/Los_Angeles",
	},
	"46CO": {
		ICAO:      "46CO",
		Name:      "Huerfano Agricultural Airport",
		City:      "Pueblo",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4790,
		Latitude:  38.0999984741,
		Longitude: -104.4260025024,
		Timezone:  "America/Denver",
	},
	"46FD": {
		ICAO:      "46FD",
		Name:      "Tater Farms Strip",
		City:      "Hastings",
		State:     "Florida",
		Country:   "US",
		Elevation: 14,
		Latitude:  29.6679992676,
		Longitude: -81.4972991943,
		Timezone:  "America/New_York",
	},
	"46IA": {
		ICAO:      "46IA",
		Name:      "Todd Field",
		City:      "Ankeny",
		State:     "Iowa",
		Country:   "US",
		Elevation: 965,
		Latitude:  41.7557983398,
		Longitude: -93.5665969849,
		Timezone:  "America/Chicago",
	},
	"46IN": {
		ICAO:      "46IN",
		Name:      "Ames Field",
		City:      "Plymouth",
		State:     "Indiana",
		Country:   "US",
		Elevation: 825,
		Latitude:  41.3811988831,
		Longitude: -86.2228012085,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"46KS": {
		ICAO:      "46KS",
		Name:      "Hidden Valley Airport",
		City:      "Valley Center",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1345,
		Latitude:  37.8316993713,
		Longitude: -97.3630981445,
		Timezone:  "America/Chicago",
	},
	"46LA": {
		ICAO:      "46LA",
		Name:      "Davis Plantation Airport",
		City:      "Bastrop",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 150,
		Latitude:  32.7965011597,
		Longitude: -91.9468002319,
		Timezone:  "America/Chicago",
	},
	"46LS": {
		ICAO:      "46LS",
		Name:      "Venissat Airstrip",
		City:      "Gueydan",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 4,
		Latitude:  30.086111,
		Longitude: -92.532222,
		Timezone:  "America/Chicago",
	},
	"46MI": {
		ICAO:      "46MI",
		Name:      "Cloud Nine Field",
		City:      "Webberville",
		State:     "Michigan",
		Country:   "US",
		Elevation: 910,
		Latitude:  42.5461006165,
		Longitude: -84.1277999878,
		Timezone:  "America/Detroit",
	},
	"46MN": {
		ICAO:      "46MN",
		Name:      "Elmer Airport",
		City:      "Evansville",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1350,
		Latitude:  46.0055007935,
		Longitude: -95.6920013428,
		Timezone:  "America/Chicago",
	},
	"46MO": {
		ICAO:      "46MO",
		Name:      "Sontimer Airport",
		City:      "St. Peters",
		State:     "Missouri",
		Country:   "US",
		Elevation: 443,
		Latitude:  38.8083992004,
		Longitude: -90.6126022339,
		Timezone:  "America/Chicago",
	},
	"46MS": {
		ICAO:      "46MS",
		Name:      "Circle Bar Ranch Airport",
		City:      "Foxworth",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 420,
		Latitude:  31.2950000763,
		Longitude: -89.9850006104,
		Timezone:  "America/Chicago",
	},
	"46NC": {
		ICAO:      "46NC",
		Name:      "Brown Field",
		City:      "Marshville",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 400,
		Latitude:  34.8927993774,
		Longitude: -80.373298645,
		Timezone:  "America/New_York",
	},
	"46NE": {
		ICAO:      "46NE",
		Name:      "Jantzen Airport",
		City:      "Madrid",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3375,
		Latitude:  40.8964004517,
		Longitude: -101.5049972534,
		Timezone:  "America/Denver",
	},
	"46NJ": {
		ICAO:      "46NJ",
		Name:      "Perl Acres Airport",
		City:      "Clarksburg",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 150,
		Latitude:  40.1529006958,
		Longitude: -74.4552001953,
		Timezone:  "America/New_York",
	},
	"46NY": {
		ICAO:      "46NY",
		Name:      "Savannah Agri-Air Airport",
		City:      "Savannah",
		State:     "New York",
		Country:   "US",
		Elevation: 465,
		Latitude:  43.0187988281,
		Longitude: -76.7610015869,
		Timezone:  "America/New_York",
	},
	"46OH": {
		ICAO:      "46OH",
		Name:      "Wetzl Airport",
		City:      "North Jackson",
		State:     "Ohio",
		Country:   "US",
		Elevation: 997,
		Latitude:  41.1036987305,
		Longitude: -80.826499939,
		Timezone:  "America/New_York",
	},
	"46OI": {
		ICAO:      "46OI",
		Name:      "Kellers Strip",
		City:      "Sylvania",
		State:     "Ohio",
		Country:   "US",
		Elevation: 700,
		Latitude:  41.6833992004,
		Longitude: -83.7666015625,
		Timezone:  "America/New_York",
	},
	"46OK": {
		ICAO:      "46OK",
		Name:      "Steinert Lakes Airport",
		City:      "Bison",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1120,
		Latitude:  36.2210998535,
		Longitude: -97.8444976807,
		Timezone:  "America/Chicago",
	},
	"46SC": {
		ICAO:      "46SC",
		Name:      "Gaffney Airport",
		City:      "Gaffney",
		State:     "South Carolina",
		Country:   "US",
		Elevation: 840,
		Latitude:  35.1391983032,
		Longitude: -81.695602417,
		Timezone:  "America/New_York",
	},
	"46TA": {
		ICAO:      "46TA",
		Name:      "Gould Strip",
		City:      "Cotulla",
		State:     "Texas",
		Country:   "US",
		Elevation: 287,
		Latitude:  28.1261005402,
		Longitude: -98.859703064,
		Timezone:  "America/Chicago",
	},
	"46TE": {
		ICAO:      "46TE",
		Name:      "02 Ranch Airport",
		City:      "Alpine",
		State:     "Texas",
		Country:   "US",
		Elevation: 3799,
		Latitude:  29.8749008179,
		Longitude: -103.6969985962,
		Timezone:  "America/Chicago",
	},
	"46TS": {
		ICAO:      "46TS",
		Name:      "Lavon North Airport",
		City:      "Dallas",
		State:     "Texas",
		Country:   "US",
		Elevation: 565,
		Latitude:  33.128200531,
		Longitude: -96.5096969604,
		Timezone:  "America/Chicago",
	},
	"46TX": {
		ICAO:      "46TX",
		Name:      "Phillips Corporation Airport",
		City:      "Angleton",
		State:     "Texas",
		Country:   "US",
		Elevation: 30,
		Latitude:  29.150800705,
		Longitude: -95.4952011108,
		Timezone:  "America/Chicago",
	},
	"46VA": {
		ICAO:      "46VA",
		Name:      "Byrd Creek Airport",
		City:      "Caledonia",
		State:     "Virginia",
		Country:   "US",
		Elevation: 315,
		Latitude:  37.8188018799,
		Longitude: -78.047203064,
		Timezone:  "America/New_York",
	},
	"46XS": {
		ICAO:      "46XS",
		Name:      "Windy Hill Airport",
		City:      "Denton",
		State:     "Texas",
		Country:   "US",
		Elevation: 600,
		Latitude:  33.3111991882,
		Longitude: -97.1044998169,
		Timezone:  "America/Chicago",
	},
	"47AK": {
		ICAO:      "47AK",
		Name:      "Highland Airport",
		City:      "Eagle River",
		State:     "Alaska",
		Country:   "US",
		Elevation: 650,
		Latitude:  61.2924995422,
		Longitude: -149.5330047607,
		Timezone:  "America/Anchorage",
	},
	"47AR": {
		ICAO:      "47AR",
		Name:      "Mabry Field",
		City:      "Melbourne",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 550,
		Latitude:  36.043598175,
		Longitude: -91.943901062,
		Timezone:  "America/Chicago",
	},
	"47AZ": {
		ICAO:      "47AZ",
		Name:      "Ray Schnepf Ranch Airport",
		City:      "Queen Creek",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1458,
		Latitude:  33.2242012024,
		Longitude: -111.59400177,
		Timezone:  "America/Phoenix",
	},
	"47CO": {
		ICAO:      "47CO",
		Name:      "Mile Hi Airport",
		City:      "New Raymer",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5270,
		Latitude:  40.9000015259,
		Longitude: -104.8339996338,
		Timezone:  "America/Denver",
	},
	"47FD": {
		ICAO:      "47FD",
		Name:      "Strayhorn Ranch Airport",
		City:      "Fort Myers",
		State:     "Florida",
		Country:   "US",
		Elevation: 20,
		Latitude:  26.6627998352,
		Longitude: -81.7705993652,
		Timezone:  "America/New_York",
	},
	"47FL": {
		ICAO:      "47FL",
		Name:      "Flying Exotics Airport",
		City:      "Weirsdale",
		State:     "Florida",
		Country:   "US",
		Elevation: 150,
		Latitude:  28.9641990662,
		Longitude: -81.7789993286,
		Timezone:  "America/New_York",
	},
	"47GA": {
		ICAO:      "47GA",
		Name:      "Everidge Airport",
		City:      "Pinehurst",
		State:     "Georgia",
		Country:   "US",
		Elevation: 450,
		Latitude:  32.2196998596,
		Longitude: -83.8553009033,
		Timezone:  "America/New_York",
	},
	"47II": {
		ICAO:      "47II",
		Name:      "Westrick Airport",
		City:      "Liberty Mills",
		State:     "Indiana",
		Country:   "US",
		Elevation: 800,
		Latitude:  41.0561981201,
		Longitude: -85.6904983521,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"47IL": {
		ICAO:      "47IL",
		Name:      "Godbee RLA Restricted Landing Area",
		City:      "Kempton",
		State:     "Illinois",
		Country:   "US",
		Elevation: 694,
		Latitude:  40.9506988525,
		Longitude: -88.2705993652,
		Timezone:  "America/Chicago",
	},
	"47IN": {
		ICAO:      "47IN",
		Name:      "Mann Airport",
		City:      "Merom",
		State:     "Indiana",
		Country:   "US",
		Elevation: 428,
		Latitude:  38.9883995056,
		Longitude: -87.5180969238,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"47IS": {
		ICAO:      "47IS",
		Name:      "Flying Z Ranch Airport",
		City:      "Durand",
		State:     "Illinois",
		Country:   "US",
		Elevation: 945,
		Latitude:  42.4916992188,
		Longitude: -89.3569030762,
		Timezone:  "America/Chicago",
	},
	"47KS": {
		ICAO:      "47KS",
		Name:      "Maxwell Airport",
		City:      "Baldwin City",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1011,
		Latitude:  38.7705993652,
		Longitude: -95.2743988037,
		Timezone:  "America/Chicago",
	},
	"47MI": {
		ICAO:      "47MI",
		Name:      "Kenneth Hayward Airport",
		City:      "Bay City",
		State:     "Michigan",
		Country:   "US",
		Elevation: 590,
		Latitude:  43.5334014893,
		Longitude: -83.8541030884,
		Timezone:  "America/Detroit",
	},
	"47MN": {
		ICAO:      "47MN",
		Name:      "Sons Private-Commercial Airport",
		City:      "Norwood",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1000,
		Latitude:  44.7000007629,
		Longitude: -93.866897583,
		Timezone:  "America/Chicago",
	},
	"47MO": {
		ICAO:      "47MO",
		Name:      "Cliff Scott Airport",
		City:      "Hamilton",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1000,
		Latitude:  39.7402992249,
		Longitude: -93.9905014038,
		Timezone:  "America/Chicago",
	},
	"47NC": {
		ICAO:      "47NC",
		Name:      "Windemere Airport",
		City:      "Fayetteville",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 98,
		Latitude:  35.1380996704,
		Longitude: -78.8227996826,
		Timezone:  "America/New_York",
	},
	"47NY": {
		ICAO:      "47NY",
		Name:      "Elk Creek Airport",
		City:      "Borden",
		State:     "New York",
		Country:   "US",
		Elevation: 1540,
		Latitude:  42.0449981689,
		Longitude: -77.3499984741,
		Timezone:  "America/New_York",
	},
	"47OH": {
		ICAO:      "47OH",
		Name:      "Chapin Airport",
		City:      "Norwalk",
		State:     "Ohio",
		Country:   "US",
		Elevation: 885,
		Latitude:  41.2000999451,
		Longitude: -82.5735015869,
		Timezone:  "America/New_York",
	},
	"47OK": {
		ICAO:      "47OK",
		Name:      "Harman Airport",
		City:      "Calumet",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1345,
		Latitude:  35.5836982727,
		Longitude: -98.0401000977,
		Timezone:  "America/Chicago",
	},
	"47OR": {
		ICAO:      "47OR",
		Name:      "Mc Ranch Airport",
		City:      "Adel",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4555,
		Latitude:  42.1649017334,
		Longitude: -119.904998779,
		Timezone:  "America/Los_Angeles",
	},
	"47PA": {
		ICAO:      "47PA",
		Name:      "Homan Airport",
		City:      "State College",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1200,
		Latitude:  40.7167015076,
		Longitude: -77.9831008911,
		Timezone:  "America/New_York",
	},
	"47TA": {
		ICAO:      "47TA",
		Name:      "Pleasure Field",
		City:      "Prosper",
		State:     "Texas",
		Country:   "US",
		Elevation: 604,
		Latitude:  33.2560005188,
		Longitude: -96.8852996826,
		Timezone:  "America/Chicago",
	},
	"47TE": {
		ICAO:      "47TE",
		Name:      "Whites Airport",
		City:      "La Ward",
		State:     "Texas",
		Country:   "US",
		Elevation: 43,
		Latitude:  28.8516998291,
		Longitude: -96.4974975586,
		Timezone:  "America/Chicago",
	},
	"47TS": {
		ICAO:      "47TS",
		Name:      "Hensarling Airport",
		City:      "Madisonville",
		State:     "Texas",
		Country:   "US",
		Elevation: 315,
		Latitude:  31.0480003357,
		Longitude: -95.9177017212,
		Timezone:  "America/Chicago",
	},
	"47TX": {
		ICAO:      "47TX",
		Name:      "Armstrong Ranch Airport",
		City:      "Armstrong",
		State:     "Texas",
		Country:   "US",
		Elevation: 27,
		Latitude:  26.9339008331,
		Longitude: -97.7624969482,
		Timezone:  "America/Chicago",
	},
	"47WI": {
		ICAO:      "47WI",
		Name:      "Oakbrook Airport",
		City:      "Fort Atkinson",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 840,
		Latitude:  42.8899993896,
		Longitude: -88.8081970215,
		Timezone:  "America/Chicago",
	},
	"47XS": {
		ICAO:      "47XS",
		Name:      "Bartos Farm Airport",
		City:      "Lubbock",
		State:     "Texas",
		Country:   "US",
		Elevation: 3153,
		Latitude:  33.4724998474,
		Longitude: -101.7850036621,
		Timezone:  "America/Chicago",
	},
	"48AK": {
		ICAO:      "48AK",
		Name:      "Castle Mountain Airstrip",
		City:      "Sutton/Chickaloon",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1010,
		Latitude:  61.7960412696,
		Longitude: -148.494161367,
		Timezone:  "America/Anchorage",
	},
	"48AZ": {
		ICAO:      "48AZ",
		Name:      "Rimrock Airport",
		City:      "Rimrock",
		State:     "Arizona",
		Country:   "US",
		Elevation: 3575,
		Latitude:  34.6509017944,
		Longitude: -111.7880020142,
		Timezone:  "America/Phoenix",
	},
	"48CN": {
		ICAO:      "48CN",
		Name:      "Sallaberry Ranch Strip",
		City:      "Madera",
		State:     "California",
		Country:   "US",
		Elevation: 263,
		Latitude:  37.0572013855,
		Longitude: -120.1439971924,
		Timezone:  "America/Los_Angeles",
	},
	"48CO": {
		ICAO:      "48CO",
		Name:      "Shaull Farm Airstrip",
		City:      "Nunn",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5235,
		Latitude:  40.7215995789,
		Longitude: -104.7779998779,
		Timezone:  "America/Denver",
	},
	"48FL": {
		ICAO:      "48FL",
		Name:      "Mc Kinnon Airpark",
		City:      "Walnut Hill",
		State:     "Florida",
		Country:   "US",
		Elevation: 227,
		Latitude:  30.8141002655,
		Longitude: -87.4749984741,
		Timezone:  "America/Chicago",
	},
	"48II": {
		ICAO:      "48II",
		Name:      "Cedar Creek Airport",
		City:      "Cedarville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 792,
		Latitude:  41.1903991699,
		Longitude: -85.0242004395,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"48IL": {
		ICAO:      "48IL",
		Name:      "Lake West Wind Airport",
		City:      "Godfrey",
		State:     "Illinois",
		Country:   "US",
		Elevation: 601,
		Latitude:  38.9700012207,
		Longitude: -90.2472000122,
		Timezone:  "America/Chicago",
	},
	"48IS": {
		ICAO:      "48IS",
		Name:      "P.J. Killian Airport",
		City:      "Lexington",
		State:     "Illinois",
		Country:   "US",
		Elevation: 757,
		Latitude:  40.6021995544,
		Longitude: -88.8333969116,
		Timezone:  "America/Chicago",
	},
	"48KS": {
		ICAO:      "48KS",
		Name:      "R F Roesner Airport",
		City:      "Salina",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1230,
		Latitude:  38.8111000061,
		Longitude: -97.4313964844,
		Timezone:  "America/Chicago",
	},
	"48LL": {
		ICAO:      "48LL",
		Name:      "Rock Cut Farms Airport",
		City:      "Leaf River",
		State:     "Illinois",
		Country:   "US",
		Elevation: 812,
		Latitude:  42.1253013611,
		Longitude: -89.4589996338,
		Timezone:  "America/Chicago",
	},
	"48MD": {
		ICAO:      "48MD",
		Name:      "Le Champ Airport",
		City:      "Princess Anne",
		State:     "Maryland",
		Country:   "US",
		Elevation: 14,
		Latitude:  38.149167,
		Longitude: -75.662222,
		Timezone:  "America/New_York",
	},
	"48NC": {
		ICAO:      "48NC",
		Name:      "Morrison Field",
		City:      "Lexington",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 825,
		Latitude:  35.7817001343,
		Longitude: -80.4169006348,
		Timezone:  "America/New_York",
	},
	"48ND": {
		ICAO:      "48ND",
		Name:      "Schirmeister Private Airport",
		City:      "Hazelton",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1650,
		Latitude:  46.5125007629,
		Longitude: -100.5500030518,
		Timezone:  "America/Chicago",
	},
	"48OH": {
		ICAO:      "48OH",
		Name:      "Dechant Farms Airport",
		City:      "Oberlin",
		State:     "Ohio",
		Country:   "US",
		Elevation: 800,
		Latitude:  41.2541999817,
		Longitude: -82.1970977783,
		Timezone:  "America/New_York",
	},
	"48OK": {
		ICAO:      "48OK",
		Name:      "Rafter 'R' Ranch Airport",
		City:      "Checotah",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 620,
		Latitude:  35.3181991577,
		Longitude: -95.5183029175,
		Timezone:  "America/Chicago",
	},
	"48OR": {
		ICAO:      "48OR",
		Name:      "Lookingglass Airport",
		City:      "Roseburg",
		State:     "Oregon",
		Country:   "US",
		Elevation: 580,
		Latitude:  43.1593017578,
		Longitude: -123.5019989014,
		Timezone:  "America/Los_Angeles",
	},
	"48TE": {
		ICAO:      "48TE",
		Name:      "4M Ranch Airfield",
		City:      "Langtry",
		State:     "Texas",
		Country:   "US",
		Elevation: 1824,
		Latitude:  30.021,
		Longitude: -101.573056,
		Timezone:  "America/Chicago",
	},
	"48TS": {
		ICAO:      "48TS",
		Name:      "Fox Airport",
		City:      "New Braunfels",
		State:     "Texas",
		Country:   "US",
		Elevation: 650,
		Latitude:  29.6676998138,
		Longitude: -98.1988983154,
		Timezone:  "America/Chicago",
	},
	"48TX": {
		ICAO:      "48TX",
		Name:      "Tri-County Aerodrome",
		City:      "Trenton",
		State:     "Texas",
		Country:   "US",
		Elevation: 790,
		Latitude:  33.4542999268,
		Longitude: -96.3815994263,
		Timezone:  "America/Chicago",
	},
	"48VA": {
		ICAO:      "48VA",
		Name:      "Al's Field",
		City:      "Gore",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1120,
		Latitude:  39.2906990051,
		Longitude: -78.3507995605,
		Timezone:  "America/New_York",
	},
	"48WA": {
		ICAO:      "48WA",
		Name:      "West Valley Airport",
		City:      "Yakima",
		State:     "Washington",
		Country:   "US",
		Elevation: 1900,
		Latitude:  46.5625,
		Longitude: -120.7770004272,
		Timezone:  "America/Los_Angeles",
	},
	"48WI": {
		ICAO:      "48WI",
		Name:      "Circle A Ranch Airport",
		City:      "Hazelhurst",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1640,
		Latitude:  45.793800354,
		Longitude: -89.7301025391,
		Timezone:  "America/Chicago",
	},
	"48XS": {
		ICAO:      "48XS",
		Name:      "Maravillas Gap Ranch Airport",
		City:      "Marathon",
		State:     "Texas",
		Country:   "US",
		Elevation: 3325,
		Latitude:  29.9265995026,
		Longitude: -103.2949981689,
		Timezone:  "America/Chicago",
	},
	"49AK": {
		ICAO:      "49AK",
		Name:      "Secluded Lake Airport",
		City:      "Talkeetna",
		State:     "Alaska",
		Country:   "US",
		Elevation: 300,
		Latitude:  62.0245018005,
		Longitude: -149.9770050049,
		Timezone:  "America/Anchorage",
	},
	"49AZ": {
		ICAO:      "49AZ",
		Name:      "Rock Point Airport",
		City:      "Rock Point",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4999,
		Latitude:  36.7318992615,
		Longitude: -109.6190032959,
		Timezone:  "America/Denver",
	},
	"49CL": {
		ICAO:      "49CL",
		Name:      "El Peco Ranch Airport",
		City:      "Madera",
		State:     "California",
		Country:   "US",
		Elevation: 213,
		Latitude:  36.9054985046,
		Longitude: -120.1770019531,
		Timezone:  "America/Los_Angeles",
	},
	"49CN": {
		ICAO:      "49CN",
		Name:      "Rancho Tehama Airport",
		City:      "Red Bluff",
		State:     "California",
		Country:   "US",
		Elevation: 550,
		Latitude:  40.0159988403,
		Longitude: -122.3899993896,
		Timezone:  "America/Los_Angeles",
	},
	"49CO": {
		ICAO:      "49CO",
		Name:      "Air Dusters Inc Airport",
		City:      "Roggen",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4780,
		Latitude:  40.0932998657,
		Longitude: -104.375,
		Timezone:  "America/Denver",
	},
	"49FD": {
		ICAO:      "49FD",
		Name:      "Watson Farm Airport",
		City:      "Quincy",
		State:     "Florida",
		Country:   "US",
		Elevation: 275,
		Latitude:  30.611944,
		Longitude: -84.641667,
		Timezone:  "America/New_York",
	},
	"49FL": {
		ICAO:      "49FL",
		Name:      "Mike's Ag Air Airport",
		City:      "Walnut Hill",
		State:     "Florida",
		Country:   "US",
		Elevation: 210,
		Latitude:  30.9771003723,
		Longitude: -87.4950027466,
		Timezone:  "America/Chicago",
	},
	"49II": {
		ICAO:      "49II",
		Name:      "Reinoehl Field",
		City:      "Lena",
		State:     "Indiana",
		Country:   "US",
		Elevation: 800,
		Latitude:  39.6097984314,
		Longitude: -87.015296936,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"49IL": {
		ICAO:      "49IL",
		Name:      "Walnut Creek Airport",
		City:      "Somonauk",
		State:     "Illinois",
		Country:   "US",
		Elevation: 666,
		Latitude:  41.5932998657,
		Longitude: -88.6917037964,
		Timezone:  "America/Chicago",
	},
	"49IN": {
		ICAO:      "49IN",
		Name:      "Drake Airport",
		City:      "Sullivan",
		State:     "Indiana",
		Country:   "US",
		Elevation: 500,
		Latitude:  39.1316986084,
		Longitude: -87.3820037842,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"49KS": {
		ICAO:      "49KS",
		Name:      "N & N Airport",
		City:      "Manhattan",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1230,
		Latitude:  39.214099884,
		Longitude: -96.6723022461,
		Timezone:  "America/Chicago",
	},
	"49MN": {
		ICAO:      "49MN",
		Name:      "Willow Ridge Airport",
		City:      "Hastings",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 839,
		Latitude:  44.6719017029,
		Longitude: -92.8263015747,
		Timezone:  "America/Chicago",
	},
	"49NE": {
		ICAO:      "49NE",
		Name:      "Harden Airstrip",
		City:      "North Platte",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3006,
		Latitude:  41.0652999878,
		Longitude: -100.7279968262,
		Timezone:  "America/Chicago",
	},
	"49NY": {
		ICAO:      "49NY",
		Name:      "Westmoreland Airport",
		City:      "Shelter Island",
		State:     "New York",
		Country:   "US",
		Elevation: 24,
		Latitude:  41.0667991638,
		Longitude: -72.3495025635,
		Timezone:  "America/New_York",
	},
	"49OH": {
		ICAO:      "49OH",
		Name:      "Gilbert Airport",
		City:      "Olmsted Falls",
		State:     "Ohio",
		Country:   "US",
		Elevation: 790,
		Latitude:  41.3666992188,
		Longitude: -81.9664993286,
		Timezone:  "America/New_York",
	},
	"49OR": {
		ICAO:      "49OR",
		Name:      "Land's Inn Ranch Airport",
		City:      "Dayville",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3880,
		Latitude:  44.5937995911,
		Longitude: -119.5640029907,
		Timezone:  "America/Los_Angeles",
	},
	"49PA": {
		ICAO:      "49PA",
		Name:      "Gap View Airport",
		City:      "Stone Church",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 780,
		Latitude:  40.8592987061,
		Longitude: -75.1337966919,
		Timezone:  "America/New_York",
	},
	"49TA": {
		ICAO:      "49TA",
		Name:      "Roeder Airport",
		City:      "Crosby",
		State:     "Texas",
		Country:   "US",
		Elevation: 49,
		Latitude:  29.8966007233,
		Longitude: -95.0174026489,
		Timezone:  "America/Chicago",
	},
	"49TE": {
		ICAO:      "49TE",
		Name:      "Stowers Ranch Airport",
		City:      "Hunt",
		State:     "Texas",
		Country:   "US",
		Elevation: 2002,
		Latitude:  30.0576992035,
		Longitude: -99.512802124,
		Timezone:  "America/Chicago",
	},
	"49WA": {
		ICAO:      "49WA",
		Name:      "Cougar Mountain Airfield",
		City:      "Yelm",
		State:     "Washington",
		Country:   "US",
		Elevation: 470,
		Latitude:  46.8470001221,
		Longitude: -122.5220031738,
		Timezone:  "America/Los_Angeles",
	},
	"49WI": {
		ICAO:      "49WI",
		Name:      "Tri-Center Airport",
		City:      "Loganville",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1225,
		Latitude:  43.3791999817,
		Longitude: -90.0154037476,
		Timezone:  "America/Chicago",
	},
	"49XS": {
		ICAO:      "49XS",
		Name:      "McCasland Ranch Airport",
		City:      "Star",
		State:     "Texas",
		Country:   "US",
		Elevation: 1370,
		Latitude:  31.448333,
		Longitude: -98.321111,
		Timezone:  "America/Chicago",
	},
	"4AK1": {
		ICAO:      "4AK1",
		Name:      "Inigok Airport",
		City:      "Deadhorse",
		State:     "Alaska",
		Country:   "US",
		Elevation: 192,
		Latitude:  70.0037994385,
		Longitude: -153.0780029297,
		Timezone:  "America/Anchorage",
	},
	"4AK2": {
		ICAO:      "4AK2",
		Name:      "St John Homestead Airport",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 250,
		Latitude:  61.5579986572,
		Longitude: -149.4239959717,
		Timezone:  "America/Anchorage",
	},
	"4AK3": {
		ICAO:      "4AK3",
		Name:      "Long Lake Airport",
		City:      "Long Lake",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1500,
		Latitude:  61.3778991699,
		Longitude: -143.3260040283,
		Timezone:  "America/Anchorage",
	},
	"4AK5": {
		ICAO:      "4AK5",
		Name:      "Mankomen Lake Airport",
		City:      "Mankomen Lake",
		State:     "Alaska",
		Country:   "US",
		Elevation: 3050,
		Latitude:  62.9898986816,
		Longitude: -144.4799957275,
		Timezone:  "America/Anchorage",
	},
	"4AK6": {
		ICAO:      "4AK6",
		Name:      "Wolf Lake Airport",
		City:      "Palmer",
		State:     "Alaska",
		Country:   "US",
		Elevation: 572,
		Latitude:  61.6409988403,
		Longitude: -149.2890014648,
		Timezone:  "America/Anchorage",
	},
	"4AK7": {
		ICAO:      "4AK7",
		Name:      "Aleknagik Mission Lodge Airport",
		City:      "Aleknagik",
		State:     "Alaska",
		Country:   "US",
		Elevation: 150,
		Latitude:  59.2808990479,
		Longitude: -158.597000122,
		Timezone:  "America/Anchorage",
	},
	"4AK8": {
		ICAO:      "4AK8",
		Name:      "Napaimute Pioneer Airfield",
		City:      "Napaimute",
		State:     "Alaska",
		Country:   "US",
		Elevation: 200,
		Latitude:  61.5363006592,
		Longitude: -158.7389984131,
		Timezone:  "America/Anchorage",
	},
	"4AK9": {
		ICAO:      "4AK9",
		Name:      "Tibbetts Airport",
		City:      "Naknek",
		State:     "Alaska",
		Country:   "US",
		Elevation: 50,
		Latitude:  58.7342987061,
		Longitude: -157.0070037842,
		Timezone:  "America/Anchorage",
	},
	"4AL2": {
		ICAO:      "4AL2",
		Name:      "Irwin Farms Airport",
		City:      "Foley",
		State:     "Alabama",
		Country:   "US",
		Elevation: 25,
		Latitude:  30.3731994629,
		Longitude: -87.7256011963,
		Timezone:  "America/Chicago",
	},
	"4AL6": {
		ICAO:      "4AL6",
		Name:      "Bonner Field",
		City:      "Summerdale",
		State:     "Alabama",
		Country:   "US",
		Elevation: 105,
		Latitude:  30.4668998718,
		Longitude: -87.6832962036,
		Timezone:  "America/Chicago",
	},
	"4AL7": {
		ICAO:      "4AL7",
		Name:      "Leon's Landing Airport",
		City:      "Flat Rock",
		State:     "Alabama",
		Country:   "US",
		Elevation: 1430,
		Latitude:  34.7447013855,
		Longitude: -85.6897964478,
		Timezone:  "America/Chicago",
	},
	"4AL8": {
		ICAO:      "4AL8",
		Name:      "Milton Airport",
		City:      "Huntsville",
		State:     "Alabama",
		Country:   "US",
		Elevation: 639,
		Latitude:  34.4906005859,
		Longitude: -86.6733016968,
		Timezone:  "America/Chicago",
	},
	"4AL9": {
		ICAO:      "4AL9",
		Name:      "T W Spear Memorial Airport",
		City:      "Lapine",
		State:     "Alabama",
		Country:   "US",
		Elevation: 438,
		Latitude:  31.9747009277,
		Longitude: -86.2938995361,
		Timezone:  "America/Chicago",
	},
	"4AR1": {
		ICAO:      "4AR1",
		Name:      "Shurley Field Airport",
		City:      "Cabot",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 430,
		Latitude:  35.0413017273,
		Longitude: -92.1009979248,
		Timezone:  "America/Chicago",
	},
	"4AR2": {
		ICAO:      "4AR2",
		Name:      "Red Oak Airport",
		City:      "Cabot",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 315,
		Latitude:  34.9099998474,
		Longitude: -92.0066986084,
		Timezone:  "America/Chicago",
	},
	"4AR4": {
		ICAO:      "4AR4",
		Name:      "Reed-Joseph Land Co. Airport",
		City:      "Garland",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 227,
		Latitude:  33.3479003906,
		Longitude: -93.7315979004,
		Timezone:  "America/Chicago",
	},
	"4AR5": {
		ICAO:      "4AR5",
		Name:      "Lawrence Field",
		City:      "Cherry Valley",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 284,
		Latitude:  35.3708000183,
		Longitude: -90.7506027222,
		Timezone:  "America/Chicago",
	},
	"4AR6": {
		ICAO:      "4AR6",
		Name:      "Williams Ranch Airport",
		City:      "Caddo Gap",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 750,
		Latitude:  34.3792991638,
		Longitude: -93.6377029419,
		Timezone:  "America/Chicago",
	},
	"4AR7": {
		ICAO:      "4AR7",
		Name:      "Worth James Ranch Airport",
		City:      "Booneville",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 650,
		Latitude:  35.0834007263,
		Longitude: -94.116897583,
		Timezone:  "America/Chicago",
	},
	"4AR9": {
		ICAO:      "4AR9",
		Name:      "Scudder Airport",
		City:      "Jonesboro",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 238,
		Latitude:  35.7761993408,
		Longitude: -90.6467971802,
		Timezone:  "America/Chicago",
	},
	"4AZ0": {
		ICAO:      "4AZ0",
		Name:      "Palm Valley Tucson Airport",
		City:      "Tucson",
		State:     "Arizona",
		Country:   "US",
		Elevation: 2139,
		Latitude:  32.2989997864,
		Longitude: -111.3170013428,
		Timezone:  "America/Phoenix",
	},
	"4AZ2": {
		ICAO:      "4AZ2",
		Name:      "Block Ranch Airport",
		City:      "Capitan",
		State:     "New Mexico",
		Country:   "US",
		Elevation: 5966,
		Latitude:  33.7019996643,
		Longitude: -105.3840026855,
		Timezone:  "America/Denver",
	},
	"4AZ6": {
		ICAO:      "4AZ6",
		Name:      "Parsons Field",
		City:      "Kingman",
		State:     "Arizona",
		Country:   "US",
		Elevation: 3760,
		Latitude:  34.9981002808,
		Longitude: -113.4619979858,
		Timezone:  "America/Phoenix",
	},
	"4AZ7": {
		ICAO:      "4AZ7",
		Name:      "San Carlos Airport",
		City:      "San Carlos",
		State:     "Arizona",
		Country:   "US",
		Elevation: 2896,
		Latitude:  33.3777999878,
		Longitude: -110.4619979858,
		Timezone:  "America/Phoenix",
	},
	"4AZ8": {
		ICAO:      "4AZ8",
		Name:      "The Ultralight Strip Ultralightport",
		City:      "Marana",
		State:     "Arizona",
		Country:   "US",
		Elevation: 2120,
		Latitude:  32.2947006226,
		Longitude: -111.34400177,
		Timezone:  "America/Phoenix",
	},
	"4AZ9": {
		ICAO:      "4AZ9",
		Name:      "Leroy Airport",
		City:      "Willcox",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4197,
		Latitude:  32.1130981445,
		Longitude: -109.7620010376,
		Timezone:  "America/Phoenix",
	},
	"4CA2": {
		ICAO:      "4CA2",
		Name:      "Funny Farm Airport",
		City:      "Brentwood",
		State:     "California",
		Country:   "US",
		Elevation: 29,
		Latitude:  37.9468994141,
		Longitude: -121.6470031738,
		Timezone:  "America/Los_Angeles",
	},
	"4CA7": {
		ICAO:      "4CA7",
		Name:      "Johnsen Airport",
		City:      "Chico",
		State:     "California",
		Country:   "US",
		Elevation: 190,
		Latitude:  39.6707000732,
		Longitude: -121.8860015869,
		Timezone:  "America/Los_Angeles",
	},
	"4CA8": {
		ICAO:      "4CA8",
		Name:      "Yandell Ranch Airport",
		City:      "Modesto",
		State:     "California",
		Country:   "US",
		Elevation: 40,
		Latitude:  37.640499115,
		Longitude: -121.1620025635,
		Timezone:  "America/Los_Angeles",
	},
	"4CL3": {
		ICAO:      "4CL3",
		Name:      "Antelope Valley Ranch Airport",
		City:      "Williams",
		State:     "California",
		Country:   "US",
		Elevation: 530,
		Latitude:  39.1463012695,
		Longitude: -122.3539962769,
		Timezone:  "America/Los_Angeles",
	},
	"4CL4": {
		ICAO:      "4CL4",
		Name:      "Rabbit Ranch Airport",
		City:      "Lucerne",
		State:     "California",
		Country:   "US",
		Elevation: 2942,
		Latitude:  34.4543991089,
		Longitude: -117.0260009766,
		Timezone:  "America/Los_Angeles",
	},
	"4CO0": {
		ICAO:      "4CO0",
		Name:      "Glen-Aspen Airport",
		City:      "Carbondale",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6835,
		Latitude:  39.3889007568,
		Longitude: -107.1600036621,
		Timezone:  "America/Denver",
	},
	"4CO2": {
		ICAO:      "4CO2",
		Name:      "Owl Canyon Gliderport",
		City:      "Wellington",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5545,
		Latitude:  40.8046989441,
		Longitude: -104.9850006104,
		Timezone:  "America/Denver",
	},
	"4CO3": {
		ICAO:      "4CO3",
		Name:      "Griffin Field",
		City:      "Walsh",
		State:     "Colorado",
		Country:   "US",
		Elevation: 3964,
		Latitude:  37.3917007446,
		Longitude: -102.2799987793,
		Timezone:  "America/Denver",
	},
	"4CO7": {
		ICAO:      "4CO7",
		Name:      "Ambrosich Field",
		City:      "Elbert",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7028,
		Latitude:  39.2178001404,
		Longitude: -104.6800003052,
		Timezone:  "America/Denver",
	},
	"4CO8": {
		ICAO:      "4CO8",
		Name:      "Kelgun Airport",
		City:      "Castle Rock",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6800,
		Latitude:  39.3069000244,
		Longitude: -104.8160018921,
		Timezone:  "America/Denver",
	},
	"4FA3": {
		ICAO:      "4FA3",
		Name:      "Tranquility Bay Strip",
		City:      "Fort Myers",
		State:     "Florida",
		Country:   "US",
		Elevation: 8,
		Latitude:  26.6434001923,
		Longitude: -82.1178970337,
		Timezone:  "America/New_York",
	},
	"4FD2": {
		ICAO:      "4FD2",
		Name:      "Flying 'F' Farms Airport",
		City:      "O'Brien",
		State:     "Florida",
		Country:   "US",
		Elevation: 40,
		Latitude:  30.0538005829,
		Longitude: -83.018699646,
		Timezone:  "America/New_York",
	},
	"4FD3": {
		ICAO:      "4FD3",
		Name:      "The Wright Place STOLport",
		City:      "Homestead",
		State:     "Florida",
		Country:   "US",
		Elevation: 13,
		Latitude:  25.5153007507,
		Longitude: -80.5134963989,
		Timezone:  "America/New_York",
	},
	"4FD7": {
		ICAO:      "4FD7",
		Name:      "Flanders Field",
		City:      "Polk City",
		State:     "Florida",
		Country:   "US",
		Elevation: 131,
		Latitude:  28.2063999176,
		Longitude: -81.8279037476,
		Timezone:  "America/New_York",
	},
	"4FL0": {
		ICAO:      "4FL0",
		Name:      "Turkey Scratch Plantation Airport",
		City:      "Lamont",
		State:     "Florida",
		Country:   "US",
		Elevation: 115,
		Latitude:  30.4036006927,
		Longitude: -83.8099975586,
		Timezone:  "America/New_York",
	},
	"4FL1": {
		ICAO:      "4FL1",
		Name:      "Escape Ranch Airport",
		City:      "Kenansville",
		State:     "Florida",
		Country:   "US",
		Elevation: 71,
		Latitude:  27.8628005981,
		Longitude: -80.9580993652,
		Timezone:  "America/New_York",
	},
	"4FL2": {
		ICAO:      "4FL2",
		Name:      "Ko-Kee Airport",
		City:      "Bushnell",
		State:     "Florida",
		Country:   "US",
		Elevation: 72,
		Latitude:  28.5949993134,
		Longitude: -82.1498031616,
		Timezone:  "America/New_York",
	},
	"4FL3": {
		ICAO:      "4FL3",
		Name:      "Fellsmere Airport",
		City:      "Fellsmere",
		State:     "Florida",
		Country:   "US",
		Elevation: 24,
		Latitude:  27.7416992188,
		Longitude: -80.6661987305,
		Timezone:  "America/New_York",
	},
	"4FL4": {
		ICAO:      "4FL4",
		Name:      "Darrah Air Park",
		City:      "Bell",
		State:     "Florida",
		Country:   "US",
		Elevation: 54,
		Latitude:  29.7782993317,
		Longitude: -82.8904037476,
		Timezone:  "America/New_York",
	},
	"4FL5": {
		ICAO:      "4FL5",
		Name:      "Ridge Landing Airport",
		City:      "Frostproof",
		State:     "Florida",
		Country:   "US",
		Elevation: 140,
		Latitude:  27.7572002411,
		Longitude: -81.6006011963,
		Timezone:  "America/New_York",
	},
	"4FL6": {
		ICAO:      "4FL6",
		Name:      "Royal Trails Airport",
		City:      "Cassia",
		State:     "Florida",
		Country:   "US",
		Elevation: 49,
		Latitude:  28.9377994537,
		Longitude: -81.4737014771,
		Timezone:  "America/New_York",
	},
	"4FL8": {
		ICAO:      "4FL8",
		Name:      "Schwartz Farms Inc Airport",
		City:      "Sarasota",
		State:     "Florida",
		Country:   "US",
		Elevation: 53,
		Latitude:  27.3437004089,
		Longitude: -82.3172988892,
		Timezone:  "America/New_York",
	},
	"4FL9": {
		ICAO:      "4FL9",
		Name:      "Gore Airport",
		City:      "Haines City",
		State:     "Florida",
		Country:   "US",
		Elevation: 135,
		Latitude:  28.1434001923,
		Longitude: -81.6470031738,
		Timezone:  "America/New_York",
	},
	"4GA0": {
		ICAO:      "4GA0",
		Name:      "Pegasus Run Airport",
		City:      "Ball Ground",
		State:     "Georgia",
		Country:   "US",
		Elevation: 900,
		Latitude:  34.2801017761,
		Longitude: -84.4213027954,
		Timezone:  "America/New_York",
	},
	"4GA1": {
		ICAO:      "4GA1",
		Name:      "Thacker Field",
		City:      "Concord",
		State:     "Georgia",
		Country:   "US",
		Elevation: 920,
		Latitude:  33.0447998047,
		Longitude: -84.3983001709,
		Timezone:  "America/New_York",
	},
	"4GA4": {
		ICAO:      "4GA4",
		Name:      "R.M. Harris Airport",
		City:      "Eton",
		State:     "Georgia",
		Country:   "US",
		Elevation: 770,
		Latitude:  34.846698761,
		Longitude: -84.754699707,
		Timezone:  "America/New_York",
	},
	"4GA6": {
		ICAO:      "4GA6",
		Name:      "Chattahoochee Air Park",
		City:      "Douglasville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 750,
		Latitude:  33.6621017456,
		Longitude: -84.6804962158,
		Timezone:  "America/New_York",
	},
	"4GA7": {
		ICAO:      "4GA7",
		Name:      "Kitchens Field",
		City:      "Griffin",
		State:     "Georgia",
		Country:   "US",
		Elevation: 800,
		Latitude:  33.134601593,
		Longitude: -84.2540969849,
		Timezone:  "America/New_York",
	},
	"4GA8": {
		ICAO:      "4GA8",
		Name:      "Andrews Airport",
		City:      "Arlington",
		State:     "Georgia",
		Country:   "US",
		Elevation: 245,
		Latitude:  31.4155006409,
		Longitude: -84.7129974365,
		Timezone:  "America/New_York",
	},
	"4GA9": {
		ICAO:      "4GA9",
		Name:      "Ebeneezer Airport",
		City:      "Cumming",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1230,
		Latitude:  34.2422981262,
		Longitude: -84.0570983887,
		Timezone:  "America/New_York",
	},
	"4GE0": {
		ICAO:      "4GE0",
		Name:      "Millhaven Airport",
		City:      "Millhaven",
		State:     "Georgia",
		Country:   "US",
		Elevation: 164,
		Latitude:  32.936000824,
		Longitude: -81.6621017456,
		Timezone:  "America/New_York",
	},
	"4GE1": {
		ICAO:      "4GE1",
		Name:      "Viola Farm Airport",
		City:      "Attapulgus",
		State:     "Georgia",
		Country:   "US",
		Elevation: 285,
		Latitude:  30.7236995697,
		Longitude: -84.5410995483,
		Timezone:  "America/New_York",
	},
	"4GE2": {
		ICAO:      "4GE2",
		Name:      "Gorden E Bellah International Airport",
		City:      "Stockbridge",
		State:     "Georgia",
		Country:   "US",
		Elevation: 780,
		Latitude:  33.5500984192,
		Longitude: -84.1666030884,
		Timezone:  "America/New_York",
	},
	"4GE3": {
		ICAO:      "4GE3",
		Name:      "Whitehall Airport",
		City:      "Montrose",
		State:     "Georgia",
		Country:   "US",
		Elevation: 350,
		Latitude:  32.498889,
		Longitude: -83.191667,
		Timezone:  "America/New_York",
	},
	"4IA2": {
		ICAO:      "4IA2",
		Name:      "Walker Field",
		City:      "West Liberty",
		State:     "Iowa",
		Country:   "US",
		Elevation: 715,
		Latitude:  41.586101532,
		Longitude: -91.220199585,
		Timezone:  "America/Chicago",
	},
	"4IA4": {
		ICAO:      "4IA4",
		Name:      "Mccoy Airport",
		City:      "Danville",
		State:     "Iowa",
		Country:   "US",
		Elevation: 715,
		Latitude:  40.8185997009,
		Longitude: -91.3388977051,
		Timezone:  "America/Chicago",
	},
	"4IA7": {
		ICAO:      "4IA7",
		Name:      "Witcombe Field",
		City:      "Cedar Falls",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1000,
		Latitude:  42.47190094,
		Longitude: -92.49520111,
		Timezone:  "America/Chicago",
	},
	"4IA8": {
		ICAO:      "4IA8",
		Name:      "Weiss Airport",
		City:      "Williamsburg",
		State:     "Iowa",
		Country:   "US",
		Elevation: 850,
		Latitude:  41.6472015381,
		Longitude: -91.959602356,
		Timezone:  "America/Chicago",
	},
	"4IA9": {
		ICAO:      "4IA9",
		Name:      "Triple J Airport",
		City:      "Dike",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1020,
		Latitude:  42.4583015442,
		Longitude: -92.7223968506,
		Timezone:  "America/Chicago",
	},
	"4II0": {
		ICAO:      "4II0",
		Name:      "De Ford Airport",
		City:      "Delphi",
		State:     "Indiana",
		Country:   "US",
		Elevation: 640,
		Latitude:  40.609500885,
		Longitude: -86.7382965088,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"4II2": {
		ICAO:      "4II2",
		Name:      "Hangar Fly Ultralight Fly Club Ultralightport",
		City:      "Lawrence",
		State:     "Indiana",
		Country:   "US",
		Elevation: 860,
		Latitude:  39.8534011841,
		Longitude: -85.9541015625,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"4II3": {
		ICAO:      "4II3",
		Name:      "Oleo Airport",
		City:      "Fillmore",
		State:     "Indiana",
		Country:   "US",
		Elevation: 870,
		Latitude:  39.7364006042,
		Longitude: -86.7296981812,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"4II4": {
		ICAO:      "4II4",
		Name:      "Chesak Airport",
		City:      "North Judson",
		State:     "Indiana",
		Country:   "US",
		Elevation: 670,
		Latitude:  41.2555999756,
		Longitude: -86.7881011963,
		Timezone:  "America/Indiana/Knox",
	},
	"4II6": {
		ICAO:      "4II6",
		Name:      "Gettlefinger Field",
		City:      "Palmyra",
		State:     "Indiana",
		Country:   "US",
		Elevation: 820,
		Latitude:  38.3791999817,
		Longitude: -86.0815963745,
		Timezone:  "America/Kentucky/Louisville",
	},
	"4II7": {
		ICAO:      "4II7",
		Name:      "Sport Aircraft Flight Park Ultralightport",
		City:      "Thorntown",
		State:     "Indiana",
		Country:   "US",
		Elevation: 900,
		Latitude:  40.1058998108,
		Longitude: -86.5218963623,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"4II8": {
		ICAO:      "4II8",
		Name:      "Willis Airport",
		City:      "Greenfield",
		State:     "Indiana",
		Country:   "US",
		Elevation: 880,
		Latitude:  39.7223014832,
		Longitude: -85.7165985107,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"4II9": {
		ICAO:      "4II9",
		Name:      "Shultz /Private/ Airport",
		City:      "Brookston",
		State:     "Indiana",
		Country:   "US",
		Elevation: 695,
		Latitude:  40.6091995239,
		Longitude: -86.8807983398,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"4IL2": {
		ICAO:      "4IL2",
		Name:      "Wayne Ziller Jr Airport",
		City:      "Fairbury",
		State:     "Illinois",
		Country:   "US",
		Elevation: 661,
		Latitude:  40.8083992004,
		Longitude: -88.5417022705,
		Timezone:  "America/Chicago",
	},
	"4IL3": {
		ICAO:      "4IL3",
		Name:      "Thrifty Acres STOLport",
		City:      "Stillman Valley (Byron)",
		State:     "Illinois",
		Country:   "US",
		Elevation: 725,
		Latitude:  42.1427993774,
		Longitude: -89.2284011841,
		Timezone:  "America/Chicago",
	},
	"4IL4": {
		ICAO:      "4IL4",
		Name:      "Kuebler RLA Restricted Landing Area",
		City:      "St Elmo",
		State:     "Illinois",
		Country:   "US",
		Elevation: 620,
		Latitude:  39.0564002991,
		Longitude: -88.8161010742,
		Timezone:  "America/Chicago",
	},
	"4IL5": {
		ICAO:      "4IL5",
		Name:      "Frank's Flying Service Airport",
		City:      "Morrison",
		State:     "Illinois",
		Country:   "US",
		Elevation: 600,
		Latitude:  41.7249984741,
		Longitude: -90.0050964355,
		Timezone:  "America/Chicago",
	},
	"4IL8": {
		ICAO:      "4IL8",
		Name:      "Rendleman Airport",
		City:      "Murphysboro",
		State:     "Illinois",
		Country:   "US",
		Elevation: 500,
		Latitude:  37.7563018799,
		Longitude: -89.3833007813,
		Timezone:  "America/Chicago",
	},
	"4IL9": {
		ICAO:      "4IL9",
		Name:      "Unzicker Airport",
		City:      "Benson",
		State:     "Illinois",
		Country:   "US",
		Elevation: 745,
		Latitude:  40.8274993896,
		Longitude: -89.1207962036,
		Timezone:  "America/Chicago",
	},
	"4IN4": {
		ICAO:      "4IN4",
		Name:      "Wilson Airport",
		City:      "Wallace",
		State:     "Indiana",
		Country:   "US",
		Elevation: 710,
		Latitude:  39.9752998352,
		Longitude: -87.1802978516,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"4IN5": {
		ICAO:      "4IN5",
		Name:      "Ball Field",
		City:      "Bourbon",
		State:     "Indiana",
		Country:   "US",
		Elevation: 825,
		Latitude:  41.3442001343,
		Longitude: -86.1483001709,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"4IN7": {
		ICAO:      "4IN7",
		Name:      "Riley Field",
		City:      "Attica",
		State:     "Indiana",
		Country:   "US",
		Elevation: 608,
		Latitude:  40.3008995056,
		Longitude: -87.2292022705,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"4IN8": {
		ICAO:      "4IN8",
		Name:      "Culp Farms Airport",
		City:      "Rensselaer",
		State:     "Indiana",
		Country:   "US",
		Elevation: 673,
		Latitude:  40.8675003052,
		Longitude: -87.0682983398,
		Timezone:  "America/Chicago",
	},
	"4IN9": {
		ICAO:      "4IN9",
		Name:      "Wawasee Airport",
		City:      "Syracuse",
		State:     "Indiana",
		Country:   "US",
		Elevation: 880,
		Latitude:  41.4188995361,
		Longitude: -85.6917037964,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"4IS1": {
		ICAO:      "4IS1",
		Name:      "Kamm Airport",
		City:      "Atwood",
		State:     "Illinois",
		Country:   "US",
		Elevation: 672,
		Latitude:  39.7731018066,
		Longitude: -88.4486999512,
		Timezone:  "America/Chicago",
	},
	"4IS5": {
		ICAO:      "4IS5",
		Name:      "Smith Restricted Landing Area",
		City:      "Hecker",
		State:     "Illinois",
		Country:   "US",
		Elevation: 420,
		Latitude:  38.2583999634,
		Longitude: -89.9381027222,
		Timezone:  "America/Chicago",
	},
	"4IS6": {
		ICAO:      "4IS6",
		Name:      "Hattan Farms Airport",
		City:      "Minonk",
		State:     "Illinois",
		Country:   "US",
		Elevation: 710,
		Latitude:  40.9305992126,
		Longitude: -89.1205978394,
		Timezone:  "America/Chicago",
	},
	"4IS7": {
		ICAO:      "4IS7",
		Name:      "Litchfield RLA Restricted Landing Area",
		City:      "Seymour",
		State:     "Illinois",
		Country:   "US",
		Elevation: 704,
		Latitude:  40.0731010437,
		Longitude: -88.3994979858,
		Timezone:  "America/Chicago",
	},
	"4IS8": {
		ICAO:      "4IS8",
		Name:      "Leigh Farm Airport",
		City:      "Sparland",
		State:     "Illinois",
		Country:   "US",
		Elevation: 730,
		Latitude:  40.0217018127,
		Longitude: -89.5501022339,
		Timezone:  "America/Chicago",
	},
	"4IS9": {
		ICAO:      "4IS9",
		Name:      "Weidner Airport",
		City:      "Bunker Hill",
		State:     "Illinois",
		Country:   "US",
		Elevation: 667,
		Latitude:  39.085899353,
		Longitude: -89.9589004517,
		Timezone:  "America/Chicago",
	},
	"4KS1": {
		ICAO:      "4KS1",
		Name:      "Amar Farms Airport",
		City:      "Wellsville",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1020,
		Latitude:  38.6852989197,
		Longitude: -95.0329971313,
		Timezone:  "America/Chicago",
	},
	"4KS2": {
		ICAO:      "4KS2",
		Name:      "Ingels Aerodrome",
		City:      "De Soto",
		State:     "Kansas",
		Country:   "US",
		Elevation: 960,
		Latitude:  39.0186004639,
		Longitude: -94.9543991089,
		Timezone:  "America/Chicago",
	},
	"4KS6": {
		ICAO:      "4KS6",
		Name:      "Thomsen Field",
		City:      "Hartford",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1140,
		Latitude:  38.2416992188,
		Longitude: -95.9067001343,
		Timezone:  "America/Chicago",
	},
	"4KS7": {
		ICAO:      "4KS7",
		Name:      "Butler Airpark",
		City:      "Rose Hill",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1310,
		Latitude:  37.5588989258,
		Longitude: -97.0650024414,
		Timezone:  "America/Chicago",
	},
	"4KS8": {
		ICAO:      "4KS8",
		Name:      "Bursch Private Airport",
		City:      "Bird City",
		State:     "Kansas",
		Country:   "US",
		Elevation: 3650,
		Latitude:  39.5910987854,
		Longitude: -101.5930023193,
		Timezone:  "America/Chicago",
	},
	"4KY0": {
		ICAO:      "4KY0",
		Name:      "J & C Antique Airfield",
		City:      "Hardin",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 540,
		Latitude:  36.7619018555,
		Longitude: -88.376701355,
		Timezone:  "America/Chicago",
	},
	"4KY1": {
		ICAO:      "4KY1",
		Name:      "Creek Side Landing Airport",
		City:      "Glasgow",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 727,
		Latitude:  36.8950004578,
		Longitude: -85.7727966309,
		Timezone:  "America/Chicago",
	},
	"4KY4": {
		ICAO:      "4KY4",
		Name:      "Moseley Field",
		City:      "Calhoun",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 395,
		Latitude:  37.5924987793,
		Longitude: -87.2810974121,
		Timezone:  "America/Chicago",
	},
	"4KY5": {
		ICAO:      "4KY5",
		Name:      "Weavers Run Airport",
		City:      "West Point",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 550,
		Latitude:  38.0285987854,
		Longitude: -83.8899993896,
		Timezone:  "America/New_York",
	},
	"4KY7": {
		ICAO:      "4KY7",
		Name:      "Mueller Farm Airport",
		City:      "Verona",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 890,
		Latitude:  38.823299408,
		Longitude: -84.6847000122,
		Timezone:  "America/New_York",
	},
	"4KY8": {
		ICAO:      "4KY8",
		Name:      "Shady Acres Airport",
		City:      "Shepherdsville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 450,
		Latitude:  37.9458007813,
		Longitude: -85.6968994141,
		Timezone:  "America/New_York",
	},
	"4LA3": {
		ICAO:      "4LA3",
		Name:      "Jackson Airport",
		City:      "Jackson",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 204,
		Latitude:  30.807800293,
		Longitude: -91.2133026123,
		Timezone:  "America/Chicago",
	},
	"4LA8": {
		ICAO:      "4LA8",
		Name:      "Overton Private Airport",
		City:      "Clinton",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 110,
		Latitude:  30.7077007294,
		Longitude: -90.8742980957,
		Timezone:  "America/Chicago",
	},
	"4LA9": {
		ICAO:      "4LA9",
		Name:      "Blouin Flightpark Ultralightport",
		City:      "Raceland",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 10,
		Latitude:  29.7362995148,
		Longitude: -90.6342010498,
		Timezone:  "America/Chicago",
	},
	"4LL0": {
		ICAO:      "4LL0",
		Name:      "Ranken Airport",
		City:      "Steward",
		State:     "Illinois",
		Country:   "US",
		Elevation: 785,
		Latitude:  41.835899353,
		Longitude: -89.0933990479,
		Timezone:  "America/Chicago",
	},
	"4LL1": {
		ICAO:      "4LL1",
		Name:      "Staton Airport",
		City:      "Steward",
		State:     "Illinois",
		Country:   "US",
		Elevation: 770,
		Latitude:  41.8541984558,
		Longitude: -89.087600708,
		Timezone:  "America/Chicago",
	},
	"4LL3": {
		ICAO:      "4LL3",
		Name:      "John L Coppernoll Airport",
		City:      "Stockton",
		State:     "Illinois",
		Country:   "US",
		Elevation: 930,
		Latitude:  42.3083000183,
		Longitude: -89.9834976196,
		Timezone:  "America/Chicago",
	},
	"4LL4": {
		ICAO:      "4LL4",
		Name:      "Enjoy Field",
		City:      "Chebanse",
		State:     "Illinois",
		Country:   "US",
		Elevation: 685,
		Latitude:  40.9757995605,
		Longitude: -87.9985961914,
		Timezone:  "America/Chicago",
	},
	"4LL7": {
		ICAO:      "4LL7",
		Name:      "Willadae Farms Airport",
		City:      "Sycamore",
		State:     "Illinois",
		Country:   "US",
		Elevation: 850,
		Latitude:  42,
		Longitude: -88.6668014526,
		Timezone:  "America/Chicago",
	},
	"4LL8": {
		ICAO:      "4LL8",
		Name:      "Colonial Acres Airport",
		City:      "Sycamore",
		State:     "Illinois",
		Country:   "US",
		Elevation: 840,
		Latitude:  42.0027999878,
		Longitude: -88.6458969116,
		Timezone:  "America/Chicago",
	},
	"4LL9": {
		ICAO:      "4LL9",
		Name:      "Alan B. Janssen Airport",
		City:      "Morrisonville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 642,
		Latitude:  39.3437004089,
		Longitude: -89.4236984253,
		Timezone:  "America/Chicago",
	},
	"4MD0": {
		ICAO:      "4MD0",
		Name:      "Mears Creek Airfield",
		City:      "Lusby",
		State:     "Maryland",
		Country:   "US",
		Elevation: 27,
		Latitude:  38.3742980957,
		Longitude: -76.483001709,
		Timezone:  "America/New_York",
	},
	"4MD1": {
		ICAO:      "4MD1",
		Name:      "Bunting's Field",
		City:      "Berlin",
		State:     "Maryland",
		Country:   "US",
		Elevation: 5,
		Latitude:  38.3681983948,
		Longitude: -75.2306976318,
		Timezone:  "America/New_York",
	},
	"4MD2": {
		ICAO:      "4MD2",
		Name:      "Ward's Airport",
		City:      "Oakland",
		State:     "Maryland",
		Country:   "US",
		Elevation: 2500,
		Latitude:  39.4297981262,
		Longitude: -79.4539031982,
		Timezone:  "America/New_York",
	},
	"4MD3": {
		ICAO:      "4MD3",
		Name:      "Carey Field",
		City:      "Bishopville",
		State:     "Maryland",
		Country:   "US",
		Elevation: 23,
		Latitude:  38.4397010803,
		Longitude: -75.1594009399,
		Timezone:  "America/New_York",
	},
	"4MD4": {
		ICAO:      "4MD4",
		Name:      "Clements Airport",
		City:      "Clements",
		State:     "Maryland",
		Country:   "US",
		Elevation: 105,
		Latitude:  38.3405990601,
		Longitude: -76.737197876,
		Timezone:  "America/New_York",
	},
	"4MD6": {
		ICAO:      "4MD6",
		Name:      "Moxley's Airport",
		City:      "Dublin",
		State:     "Maryland",
		Country:   "US",
		Elevation: 410,
		Latitude:  39.625,
		Longitude: -76.2624969482,
		Timezone:  "America/New_York",
	},
	"4MD7": {
		ICAO:      "4MD7",
		Name:      "Catoctin Crosswind Airport",
		City:      "Thurmont",
		State:     "Maryland",
		Country:   "US",
		Elevation: 451,
		Latitude:  39.646389,
		Longitude: -77.366667,
		Timezone:  "America/New_York",
	},
	"4MD8": {
		ICAO:      "4MD8",
		Name:      "Ijamsville Airport",
		City:      "Frederick",
		State:     "Maryland",
		Country:   "US",
		Elevation: 490,
		Latitude:  39.3582992554,
		Longitude: -77.3396987915,
		Timezone:  "America/New_York",
	},
	"4MD9": {
		ICAO:      "4MD9",
		Name:      "St John Airport",
		City:      "Owings Mills",
		State:     "Maryland",
		Country:   "US",
		Elevation: 461,
		Latitude:  39.4805984497,
		Longitude: -76.760597229,
		Timezone:  "America/New_York",
	},
	"4MI0": {
		ICAO:      "4MI0",
		Name:      "Kriewall Strip",
		City:      "Romeo",
		State:     "Michigan",
		Country:   "US",
		Elevation: 765,
		Latitude:  42.8381004333,
		Longitude: -82.9688034058,
		Timezone:  "America/Detroit",
	},
	"4MI1": {
		ICAO:      "4MI1",
		Name:      "Air Rahe Airport",
		City:      "Petersburg",
		State:     "Michigan",
		Country:   "US",
		Elevation: 680,
		Latitude:  41.8306007385,
		Longitude: -83.6874008179,
		Timezone:  "America/Detroit",
	},
	"4MI3": {
		ICAO:      "4MI3",
		Name:      "Phil's Field",
		City:      "Overisel",
		State:     "Michigan",
		Country:   "US",
		Elevation: 685,
		Latitude:  42.7277984619,
		Longitude: -86.0402984619,
		Timezone:  "America/Detroit",
	},
	"4MI4": {
		ICAO:      "4MI4",
		Name:      "Whitcomb Field",
		City:      "Schoolcraft",
		State:     "Michigan",
		Country:   "US",
		Elevation: 915,
		Latitude:  42.1781005859,
		Longitude: -85.6707992554,
		Timezone:  "America/Detroit",
	},
	"4MI6": {
		ICAO:      "4MI6",
		Name:      "Benedick Airport",
		City:      "Three Oaks",
		State:     "Michigan",
		Country:   "US",
		Elevation: 660,
		Latitude:  41.85779953,
		Longitude: -86.6145019531,
		Timezone:  "America/Detroit",
	},
	"4MI7": {
		ICAO:      "4MI7",
		Name:      "Witbeck Aerodrone Airport",
		City:      "Farwell",
		State:     "Michigan",
		Country:   "US",
		Elevation: 890,
		Latitude:  43.8134002686,
		Longitude: -84.8217010498,
		Timezone:  "America/Detroit",
	},
	"4MN0": {
		ICAO:      "4MN0",
		Name:      "Skalicky Airstrip",
		City:      "Albany",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1300,
		Latitude:  45.6618995667,
		Longitude: -94.5755996704,
		Timezone:  "America/Chicago",
	},
	"4MN2": {
		ICAO:      "4MN2",
		Name:      "Coot Landing Airport",
		City:      "Barnsville",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1325,
		Latitude:  46.6637992859,
		Longitude: -96.2417984009,
		Timezone:  "America/Chicago",
	},
	"4MN4": {
		ICAO:      "4MN4",
		Name:      "Mulder Field Inc Airport",
		City:      "Ivanhoe",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1669,
		Latitude:  44.4557991028,
		Longitude: -96.2305984497,
		Timezone:  "America/Chicago",
	},
	"4MN5": {
		ICAO:      "4MN5",
		Name:      "Kapaun-Wilson Field",
		City:      "Graceville",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1122,
		Latitude:  45.5504989624,
		Longitude: -96.4514007568,
		Timezone:  "America/Chicago",
	},
	"4MN7": {
		ICAO:      "4MN7",
		Name:      "Burk Airport",
		City:      "Amboy",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1070,
		Latitude:  43.8437995911,
		Longitude: -94.2857971191,
		Timezone:  "America/Chicago",
	},
	"4MN8": {
		ICAO:      "4MN8",
		Name:      "Myers Field",
		City:      "Pequot Lakes",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1300,
		Latitude:  46.6274986267,
		Longitude: -94.3394012451,
		Timezone:  "America/Chicago",
	},
	"4MO0": {
		ICAO:      "4MO0",
		Name:      "Fender J H Airport",
		City:      "Kingsville",
		State:     "Missouri",
		Country:   "US",
		Elevation: 865,
		Latitude:  38.6995010376,
		Longitude: -94.0579986572,
		Timezone:  "America/Chicago",
	},
	"4MO2": {
		ICAO:      "4MO2",
		Name:      "Newcomb Hereford Ranch Airport",
		City:      "Lancaster",
		State:     "Missouri",
		Country:   "US",
		Elevation: 850,
		Latitude:  40.5583992004,
		Longitude: -92.4419021606,
		Timezone:  "America/Chicago",
	},
	"4MO3": {
		ICAO:      "4MO3",
		Name:      "Schneider Field",
		City:      "La Plata",
		State:     "Missouri",
		Country:   "US",
		Elevation: 939,
		Latitude:  39.9916992188,
		Longitude: -92.491897583,
		Timezone:  "America/Chicago",
	},
	"4MO4": {
		ICAO:      "4MO4",
		Name:      "Liberty Landing Airport",
		City:      "Liberty",
		State:     "Missouri",
		Country:   "US",
		Elevation: 720,
		Latitude:  39.2167015076,
		Longitude: -94.3336029053,
		Timezone:  "America/Chicago",
	},
	"4MO5": {
		ICAO:      "4MO5",
		Name:      "Boehne Field",
		City:      "Lockwood",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1120,
		Latitude:  37.3666992188,
		Longitude: -93.991897583,
		Timezone:  "America/Chicago",
	},
	"4MO6": {
		ICAO:      "4MO6",
		Name:      "Mark Twain Air Park",
		City:      "Louisiana",
		State:     "Missouri",
		Country:   "US",
		Elevation: 840,
		Latitude:  39.4388999939,
		Longitude: -91.1182022095,
		Timezone:  "America/Chicago",
	},
	"4MO7": {
		ICAO:      "4MO7",
		Name:      "Crop Care Airport",
		City:      "Mound City",
		State:     "Missouri",
		Country:   "US",
		Elevation: 875,
		Latitude:  40.1413993835,
		Longitude: -95.2407989502,
		Timezone:  "America/Chicago",
	},
	"4MO8": {
		ICAO:      "4MO8",
		Name:      "Martens Airport",
		City:      "Atherton",
		State:     "Missouri",
		Country:   "US",
		Elevation: 720,
		Latitude:  39.2066993713,
		Longitude: -94.2696990967,
		Timezone:  "America/Chicago",
	},
	"4MO9": {
		ICAO:      "4MO9",
		Name:      "Friends Field",
		City:      "Lone Jack",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1100,
		Latitude:  38.8925018311,
		Longitude: -94.1085968018,
		Timezone:  "America/Chicago",
	},
	"4MS0": {
		ICAO:      "4MS0",
		Name:      "Payne Airport",
		City:      "Pelahatchie",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 359,
		Latitude:  32.3003997803,
		Longitude: -89.8590011597,
		Timezone:  "America/Chicago",
	},
	"4MS2": {
		ICAO:      "4MS2",
		Name:      "Dogwood Acres Airport",
		City:      "Raymond",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 250,
		Latitude:  32.2121009827,
		Longitude: -90.450302124,
		Timezone:  "America/Chicago",
	},
	"4MS3": {
		ICAO:      "4MS3",
		Name:      "Jeter Field",
		City:      "Sladen",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 520,
		Latitude:  34.9560012817,
		Longitude: -89.3939971924,
		Timezone:  "America/Chicago",
	},
	"4MS4": {
		ICAO:      "4MS4",
		Name:      "Spearman Field",
		City:      "Shannon",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 270,
		Latitude:  34.157699585,
		Longitude: -88.7602005005,
		Timezone:  "America/Chicago",
	},
	"4MS5": {
		ICAO:      "4MS5",
		Name:      "Bird Nest Airport",
		City:      "Lucedale",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 311,
		Latitude:  31.0013999939,
		Longitude: -88.516998291,
		Timezone:  "America/Chicago",
	},
	"4MS7": {
		ICAO:      "4MS7",
		Name:      "Arnold Field",
		City:      "Tunica",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 195,
		Latitude:  34.7484016418,
		Longitude: -90.2807998657,
		Timezone:  "America/Chicago",
	},
	"4MS9": {
		ICAO:      "4MS9",
		Name:      "Cochran Airport",
		City:      "Canton",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 240,
		Latitude:  32.6660003662,
		Longitude: -90.0675964355,
		Timezone:  "America/Chicago",
	},
	"4MT1": {
		ICAO:      "4MT1",
		Name:      "Kreikemeier Airport",
		City:      "Belgrade",
		State:     "Montana",
		Country:   "US",
		Elevation: 4360,
		Latitude:  45.8207015991,
		Longitude: -111.1449966431,
		Timezone:  "America/Denver",
	},
	"4NA0": {
		ICAO:      "4NA0",
		Name:      "Lindvig Airstrip",
		City:      "Williston",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 2180,
		Latitude:  48.0083999634,
		Longitude: -103.591003418,
		Timezone:  "America/Chicago",
	},
	"4NA1": {
		ICAO:      "4NA1",
		Name:      "Ring Rock Ranch Airport",
		City:      "Williston",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 2256,
		Latitude:  48.254699707,
		Longitude: -103.672996521,
		Timezone:  "America/Chicago",
	},
	"4NA3": {
		ICAO:      "4NA3",
		Name:      "Peterson Airstrip",
		City:      "Antler",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1545,
		Latitude:  48.8572006226,
		Longitude: -101.3020019531,
		Timezone:  "America/Chicago",
	},
	"4NA5": {
		ICAO:      "4NA5",
		Name:      "Berg Strip",
		City:      "Berthold",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 2250,
		Latitude:  48.2882995605,
		Longitude: -101.8280029297,
		Timezone:  "America/Chicago",
	},
	"4NA6": {
		ICAO:      "4NA6",
		Name:      "Nicks Landing Airport",
		City:      "Grenora",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 2440,
		Latitude:  48.4550018311,
		Longitude: -103.8799972534,
		Timezone:  "America/Chicago",
	},
	"4NC0": {
		ICAO:      "4NC0",
		Name:      "Boyd's Hawks Creek Airport",
		City:      "Mocksville",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 815,
		Latitude:  35.8984985352,
		Longitude: -80.6162033081,
		Timezone:  "America/New_York",
	},
	"4NC2": {
		ICAO:      "4NC2",
		Name:      "Bay Creek Airport",
		City:      "Bayboro",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 2,
		Latitude:  35.207698822,
		Longitude: -76.612197876,
		Timezone:  "America/New_York",
	},
	"4NC3": {
		ICAO:      "4NC3",
		Name:      "Taylorsville Airport",
		City:      "Taylorsville",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 1090,
		Latitude:  35.9308013916,
		Longitude: -81.1967010498,
		Timezone:  "America/New_York",
	},
	"4NC5": {
		ICAO:      "4NC5",
		Name:      "Triple M Airport",
		City:      "Cape Carteret",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 30,
		Latitude:  34.7304000854,
		Longitude: -77.0504989624,
		Timezone:  "America/New_York",
	},
	"4NC6": {
		ICAO:      "4NC6",
		Name:      "Cane Creek Airport",
		City:      "Fletcher",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 2100,
		Latitude:  35.4445991516,
		Longitude: -82.479598999,
		Timezone:  "America/New_York",
	},
	"4NC7": {
		ICAO:      "4NC7",
		Name:      "Peacock STOLport",
		City:      "Garner",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 390,
		Latitude:  35.6963005066,
		Longitude: -78.5374984741,
		Timezone:  "America/New_York",
	},
	"4NC8": {
		ICAO:      "4NC8",
		Name:      "Buffalo Creek Airport",
		City:      "Concord",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 600,
		Latitude:  35.4224014282,
		Longitude: -80.6205978394,
		Timezone:  "America/New_York",
	},
	"4NC9": {
		ICAO:      "4NC9",
		Name:      "Lz Carroll STOLport",
		City:      "Gold Hill",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 675,
		Latitude:  35.4485015869,
		Longitude: -80.3425979614,
		Timezone:  "America/New_York",
	},
	"4ND1": {
		ICAO:      "4ND1",
		Name:      "Knutson Airport",
		City:      "Thompson",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 880,
		Latitude:  47.7928009033,
		Longitude: -97.1537017822,
		Timezone:  "America/Chicago",
	},
	"4ND4": {
		ICAO:      "4ND4",
		Name:      "Pfau Private Airport",
		City:      "Upham",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1455,
		Latitude:  48.5638999939,
		Longitude: -100.8799972534,
		Timezone:  "America/Chicago",
	},
	"4ND8": {
		ICAO:      "4ND8",
		Name:      "Sanden Airport",
		City:      "Wyndmere",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1065,
		Latitude:  46.3540992737,
		Longitude: -97.1169967651,
		Timezone:  "America/Chicago",
	},
	"4NE0": {
		ICAO:      "4NE0",
		Name:      "Regier Brothers Airport",
		City:      "Madrid",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3363,
		Latitude:  40.9160995483,
		Longitude: -101.5599975586,
		Timezone:  "America/Denver",
	},
	"4NE1": {
		ICAO:      "4NE1",
		Name:      "Mc Millan Ranch Airport",
		City:      "Milburn",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2500,
		Latitude:  41.7541999817,
		Longitude: -99.7546005249,
		Timezone:  "America/Chicago",
	},
	"4NE8": {
		ICAO:      "4NE8",
		Name:      "Malone M-Bar Ranch Airport",
		City:      "Palisade",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3000,
		Latitude:  40.3828010559,
		Longitude: -101.1520004272,
		Timezone:  "America/Chicago",
	},
	"4NE9": {
		ICAO:      "4NE9",
		Name:      "Holzfaster's Airport",
		City:      "Paxton",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3305,
		Latitude:  40.9991989136,
		Longitude: -101.4469985962,
		Timezone:  "America/Denver",
	},
	"4NJ0": {
		ICAO:      "4NJ0",
		Name:      "Windward Farms Airport",
		City:      "Rosemont",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 425,
		Latitude:  40.4492988586,
		Longitude: -75.0177001953,
		Timezone:  "America/New_York",
	},
	"4NJ8": {
		ICAO:      "4NJ8",
		Name:      "Peters Airport",
		City:      "Somerville",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 150,
		Latitude:  40.6001014709,
		Longitude: -74.7329025269,
		Timezone:  "America/New_York",
	},
	"4NK1": {
		ICAO:      "4NK1",
		Name:      "B-Ville Airpark",
		City:      "Baldwinsville",
		State:     "New York",
		Country:   "US",
		Elevation: 375,
		Latitude:  43.1447982788,
		Longitude: -76.3113021851,
		Timezone:  "America/New_York",
	},
	"4NK2": {
		ICAO:      "4NK2",
		Name:      "Tracy Field",
		City:      "Greenwich",
		State:     "New York",
		Country:   "US",
		Elevation: 455,
		Latitude:  43.145233,
		Longitude: -73.41965,
		Timezone:  "America/New_York",
	},
	"4NK3": {
		ICAO:      "4NK3",
		Name:      "Evans Airways Airport",
		City:      "Evans",
		State:     "New York",
		Country:   "US",
		Elevation: 940,
		Latitude:  42.6725006104,
		Longitude: -78.9670028687,
		Timezone:  "America/New_York",
	},
	"4NK4": {
		ICAO:      "4NK4",
		Name:      "Woodford Airfield",
		City:      "Fabius",
		State:     "New York",
		Country:   "US",
		Elevation: 1280,
		Latitude:  42.8272018433,
		Longitude: -76.0258026123,
		Timezone:  "America/New_York",
	},
	"4NK5": {
		ICAO:      "4NK5",
		Name:      "Harvs Airport",
		City:      "Macedon",
		State:     "New York",
		Country:   "US",
		Elevation: 500,
		Latitude:  43.1012001038,
		Longitude: -77.3677978516,
		Timezone:  "America/New_York",
	},
	"4NK6": {
		ICAO:      "4NK6",
		Name:      "Ag-Alley Airport",
		City:      "Marion",
		State:     "New York",
		Country:   "US",
		Elevation: 450,
		Latitude:  43.1610984802,
		Longitude: -77.1774978638,
		Timezone:  "America/New_York",
	},
	"4NK8": {
		ICAO:      "4NK8",
		Name:      "Longwell Airport",
		City:      "Amsterdam",
		State:     "New York",
		Country:   "US",
		Elevation: 800,
		Latitude:  43.0009002686,
		Longitude: -74.1871032715,
		Timezone:  "America/New_York",
	},
	"4NY0": {
		ICAO:      "4NY0",
		Name:      "Mountain View Airpark",
		City:      "Beekmantown",
		State:     "New York",
		Country:   "US",
		Elevation: 500,
		Latitude:  44.6688995361,
		Longitude: -73.5462036133,
		Timezone:  "America/New_York",
	},
	"4NY1": {
		ICAO:      "4NY1",
		Name:      "Orange Poultry Farm Airport",
		City:      "Chester",
		State:     "New York",
		Country:   "US",
		Elevation: 420,
		Latitude:  41.3278999329,
		Longitude: -74.3246002197,
		Timezone:  "America/New_York",
	},
	"4NY4": {
		ICAO:      "4NY4",
		Name:      "Cross' Farm Airport",
		City:      "Delmar",
		State:     "New York",
		Country:   "US",
		Elevation: 55,
		Latitude:  42.5595016479,
		Longitude: -73.772102356,
		Timezone:  "America/New_York",
	},
	"4NY8": {
		ICAO:      "4NY8",
		Name:      "Harris Hill Gliderport",
		City:      "Elmira",
		State:     "New York",
		Country:   "US",
		Elevation: 1709,
		Latitude:  42.1208992004,
		Longitude: -76.9011001587,
		Timezone:  "America/New_York",
	},
	"4OA8": {
		ICAO:      "4OA8",
		Name:      "Bowman Airport",
		City:      "Piqua",
		State:     "Ohio",
		Country:   "US",
		Elevation: 980,
		Latitude:  40.1086006165,
		Longitude: -84.299697876,
		Timezone:  "America/New_York",
	},
	"4OH1": {
		ICAO:      "4OH1",
		Name:      "Sky Haven Airport",
		City:      "Madison",
		State:     "Ohio",
		Country:   "US",
		Elevation: 940,
		Latitude:  41.716999054,
		Longitude: -81.1067962646,
		Timezone:  "America/New_York",
	},
	"4OH3": {
		ICAO:      "4OH3",
		Name:      "Bieber Field",
		City:      "North Lima",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1035,
		Latitude:  40.9831008911,
		Longitude: -80.7011032104,
		Timezone:  "America/New_York",
	},
	"4OH4": {
		ICAO:      "4OH4",
		Name:      "Millertime Airport",
		City:      "Orient",
		State:     "Ohio",
		Country:   "US",
		Elevation: 840,
		Latitude:  39.7928009033,
		Longitude: -83.1172027588,
		Timezone:  "America/New_York",
	},
	"4OH8": {
		ICAO:      "4OH8",
		Name:      "Huffman Farm Airport",
		City:      "Archbold",
		State:     "Ohio",
		Country:   "US",
		Elevation: 741,
		Latitude:  41.5,
		Longitude: -84.2499008179,
		Timezone:  "America/New_York",
	},
	"4OI7": {
		ICAO:      "4OI7",
		Name:      "R & M Aviation Airport",
		City:      "Avon",
		State:     "Ohio",
		Country:   "US",
		Elevation: 690,
		Latitude:  41.4537010193,
		Longitude: -81.9701004028,
		Timezone:  "America/New_York",
	},
	"4OI8": {
		ICAO:      "4OI8",
		Name:      "Brocker Field",
		City:      "Canfield",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1170,
		Latitude:  41.057800293,
		Longitude: -80.7545013428,
		Timezone:  "America/New_York",
	},
	"4OI9": {
		ICAO:      "4OI9",
		Name:      "Baisden Airport",
		City:      "Coalton",
		State:     "Ohio",
		Country:   "US",
		Elevation: 730,
		Latitude:  39.1021995544,
		Longitude: -82.6107025146,
		Timezone:  "America/New_York",
	},
	"4OK0": {
		ICAO:      "4OK0",
		Name:      "Zevely Climbing Zz Ranch Airport",
		City:      "Seminole",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 980,
		Latitude:  35.2251014709,
		Longitude: -96.7228012085,
		Timezone:  "America/Chicago",
	},
	"4OK1": {
		ICAO:      "4OK1",
		Name:      "Hatton Ranch Airport",
		City:      "Talihina",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 830,
		Latitude:  34.7168006897,
		Longitude: -95.175201416,
		Timezone:  "America/Chicago",
	},
	"4OK2": {
		ICAO:      "4OK2",
		Name:      "Candy Lake Estate Airport",
		City:      "Avant",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 685,
		Latitude:  36.4930992126,
		Longitude: -96.0547027588,
		Timezone:  "America/Chicago",
	},
	"4OK4": {
		ICAO:      "4OK4",
		Name:      "Low Pass Airport",
		City:      "Tuttle",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1320,
		Latitude:  35.2876014709,
		Longitude: -97.6641998291,
		Timezone:  "America/Chicago",
	},
	"4OK5": {
		ICAO:      "4OK5",
		Name:      "Newman Farm Airport",
		City:      "Velma",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1140,
		Latitude:  34.5500984192,
		Longitude: -97.6003036499,
		Timezone:  "America/Chicago",
	},
	"4OK6": {
		ICAO:      "4OK6",
		Name:      "Pitts Field",
		City:      "Wanette",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1060,
		Latitude:  34.9894981384,
		Longitude: -97.0252990723,
		Timezone:  "America/Chicago",
	},
	"4OK7": {
		ICAO:      "4OK7",
		Name:      "Flying A Ranch Airport",
		City:      "Washington",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1165,
		Latitude:  35.0912017822,
		Longitude: -97.5009002686,
		Timezone:  "America/Chicago",
	},
	"4OK8": {
		ICAO:      "4OK8",
		Name:      "Austin Airport",
		City:      "Wilburton",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 675,
		Latitude:  34.9031982422,
		Longitude: -95.3589019775,
		Timezone:  "America/Chicago",
	},
	"4OL3": {
		ICAO:      "4OL3",
		Name:      "Jerry-Wright Airfield",
		City:      "Lawton",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1100,
		Latitude:  34.5167007446,
		Longitude: -98.3337020874,
		Timezone:  "America/Chicago",
	},
	"4OR0": {
		ICAO:      "4OR0",
		Name:      "Springbrook Airport",
		City:      "Rogue River",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1560,
		Latitude:  42.5550994873,
		Longitude: -123.2040023804,
		Timezone:  "America/Los_Angeles",
	},
	"4OR3": {
		ICAO:      "4OR3",
		Name:      "Lambert Field",
		City:      "Albany",
		State:     "Oregon",
		Country:   "US",
		Elevation: 195,
		Latitude:  44.7304000854,
		Longitude: -123.0759963989,
		Timezone:  "America/Los_Angeles",
	},
	"4OR4": {
		ICAO:      "4OR4",
		Name:      "Schrock Airport",
		City:      "Corvallis",
		State:     "Oregon",
		Country:   "US",
		Elevation: 245,
		Latitude:  44.4435997009,
		Longitude: -123.2379989624,
		Timezone:  "America/Los_Angeles",
	},
	"4OR5": {
		ICAO:      "4OR5",
		Name:      "Fly 'N' W Airport",
		City:      "Salem",
		State:     "Oregon",
		Country:   "US",
		Elevation: 206,
		Latitude:  44.9678993225,
		Longitude: -122.9410018921,
		Timezone:  "America/Los_Angeles",
	},
	"4OR6": {
		ICAO:      "4OR6",
		Name:      "Auberge Des Fleurs Airport",
		City:      "Sandy",
		State:     "Oregon",
		Country:   "US",
		Elevation: 510,
		Latitude:  45.449798584,
		Longitude: -122.2539978027,
		Timezone:  "America/Los_Angeles",
	},
	"4OR7": {
		ICAO:      "4OR7",
		Name:      "Lusardi Field",
		City:      "Salem",
		State:     "Oregon",
		Country:   "US",
		Elevation: 205,
		Latitude:  44.98540115,
		Longitude: -122.9459991,
		Timezone:  "America/Los_Angeles",
	},
	"4OR8": {
		ICAO:      "4OR8",
		Name:      "Wagoner Airport",
		City:      "Salem",
		State:     "Oregon",
		Country:   "US",
		Elevation: 700,
		Latitude:  44.8484992981,
		Longitude: -123.0410003662,
		Timezone:  "America/Los_Angeles",
	},
	"4PA0": {
		ICAO:      "4PA0",
		Name:      "Millard Airport",
		City:      "Annville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 485,
		Latitude:  40.317199707,
		Longitude: -76.5363998413,
		Timezone:  "America/New_York",
	},
	"4PA1": {
		ICAO:      "4PA1",
		Name:      "Sauers-Haven Airport",
		City:      "New Berlin",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 590,
		Latitude:  40.8838996887,
		Longitude: -76.9800033569,
		Timezone:  "America/New_York",
	},
	"4PA3": {
		ICAO:      "4PA3",
		Name:      "Frymoyer Airport",
		City:      "Liverpool",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 520,
		Latitude:  40.6305999756,
		Longitude: -77.0102996826,
		Timezone:  "America/New_York",
	},
	"4PA5": {
		ICAO:      "4PA5",
		Name:      "Ponderosa Airport",
		City:      "Lock Haven",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1820,
		Latitude:  41.3083992004,
		Longitude: -77.5246963501,
		Timezone:  "America/New_York",
	},
	"4PA6": {
		ICAO:      "4PA6",
		Name:      "Hiawatha Airport",
		City:      "Starlight",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1670,
		Latitude:  41.9151000977,
		Longitude: -75.364402771,
		Timezone:  "America/New_York",
	},
	"4PA7": {
		ICAO:      "4PA7",
		Name:      "Sheepshead Airport",
		City:      "Lykens",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 610,
		Latitude:  40.5648002625,
		Longitude: -76.74659729,
		Timezone:  "America/New_York",
	},
	"4PA8": {
		ICAO:      "4PA8",
		Name:      "Russo Airstrip",
		City:      "Pipersville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 400,
		Latitude:  40.4328994751,
		Longitude: -75.149597168,
		Timezone:  "America/New_York",
	},
	"4PN0": {
		ICAO:      "4PN0",
		Name:      "Flying M Ranch Airport",
		City:      "Grove City",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1348,
		Latitude:  41.2164001465,
		Longitude: -80.0936965942,
		Timezone:  "America/New_York",
	},
	"4PN7": {
		ICAO:      "4PN7",
		Name:      "Merritt Field",
		City:      "Eagles Mere",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 2000,
		Latitude:  41.4180984497,
		Longitude: -76.5344009399,
		Timezone:  "America/New_York",
	},
	"4PN8": {
		ICAO:      "4PN8",
		Name:      "C.J.K. Airport",
		City:      "Tunkhannock",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1100,
		Latitude:  41.5308990479,
		Longitude: -75.8783035278,
		Timezone:  "America/New_York",
	},
	"4PS3": {
		ICAO:      "4PS3",
		Name:      "Paraport Airstrip",
		City:      "Warren",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1400,
		Latitude:  41.788417,
		Longitude: -79.518586,
		Timezone:  "America/New_York",
	},
	"4PS4": {
		ICAO:      "4PS4",
		Name:      "Mountain Top Airport",
		City:      "Chadds Ford",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 350,
		Latitude:  39.8543014526,
		Longitude: -75.5727005005,
		Timezone:  "America/New_York",
	},
	"4PS5": {
		ICAO:      "4PS5",
		Name:      "Muddy Creek Airport",
		City:      "Carmicheal",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1080,
		Latitude:  39.920501709,
		Longitude: -79.9486999512,
		Timezone:  "America/New_York",
	},
	"4PS6": {
		ICAO:      "4PS6",
		Name:      "Krout Airport",
		City:      "Lewistown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 600,
		Latitude:  40.6366996765,
		Longitude: -77.4835968018,
		Timezone:  "America/New_York",
	},
	"4PS7": {
		ICAO:      "4PS7",
		Name:      "Cohen Airport",
		City:      "Oxford",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 418,
		Latitude:  39.7361984253,
		Longitude: -75.9436035156,
		Timezone:  "America/New_York",
	},
	"4SC4": {
		ICAO:      "4SC4",
		Name:      "Davis Field",
		City:      "Liberty",
		State:     "South Carolina",
		Country:   "US",
		Elevation: 940,
		Latitude:  34.7653007507,
		Longitude: -82.6911010742,
		Timezone:  "America/New_York",
	},
	"4SC7": {
		ICAO:      "4SC7",
		Name:      "Wexford Landing Airport",
		City:      "Aiken",
		State:     "South Carolina",
		Country:   "US",
		Elevation: 350,
		Latitude:  33.5783004761,
		Longitude: -81.5117034912,
		Timezone:  "America/New_York",
	},
	"4SD4": {
		ICAO:      "4SD4",
		Name:      "Dan's Airport",
		City:      "Rapid City",
		State:     "South Dakota",
		Country:   "US",
		Elevation: 3166,
		Latitude:  44.0416984558,
		Longitude: -103.0459976196,
		Timezone:  "America/Denver",
	},
	"4TA0": {
		ICAO:      "4TA0",
		Name:      "Massimiliano Memorial Field",
		City:      "Damon",
		State:     "Texas",
		Country:   "US",
		Elevation: 65,
		Latitude:  29.3554992676,
		Longitude: -95.6638031006,
		Timezone:  "America/Chicago",
	},
	"4TA1": {
		ICAO:      "4TA1",
		Name:      "Warschun Ranch Airport",
		City:      "Denton",
		State:     "Texas",
		Country:   "US",
		Elevation: 600,
		Latitude:  33.302898407,
		Longitude: -97.1113967896,
		Timezone:  "America/Chicago",
	},
	"4TA3": {
		ICAO:      "4TA3",
		Name:      "Costello Island Inc Airport",
		City:      "Graham",
		State:     "Texas",
		Country:   "US",
		Elevation: 1020,
		Latitude:  32.8987007141,
		Longitude: -98.4600982666,
		Timezone:  "America/Chicago",
	},
	"4TA4": {
		ICAO:      "4TA4",
		Name:      "Pea Patch Airport",
		City:      "Hempstead",
		State:     "Texas",
		Country:   "US",
		Elevation: 174,
		Latitude:  29.9391002655,
		Longitude: -96.0597000122,
		Timezone:  "America/Chicago",
	},
	"4TA5": {
		ICAO:      "4TA5",
		Name:      "Lucky G Airport",
		City:      "Holliday",
		State:     "Texas",
		Country:   "US",
		Elevation: 1068,
		Latitude:  33.8526000977,
		Longitude: -98.6462020874,
		Timezone:  "America/Chicago",
	},
	"4TA6": {
		ICAO:      "4TA6",
		Name:      "Comanche Caves Ranch Airport",
		City:      "Hunt",
		State:     "Texas",
		Country:   "US",
		Elevation: 2000,
		Latitude:  30.0223999023,
		Longitude: -99.3836975098,
		Timezone:  "America/Chicago",
	},
	"4TA8": {
		ICAO:      "4TA8",
		Name:      "Perry Ranch Airport",
		City:      "Ozona",
		State:     "Texas",
		Country:   "US",
		Elevation: 2250,
		Latitude:  30.6168003082,
		Longitude: -101.016998291,
		Timezone:  "America/Chicago",
	},
	"4TE0": {
		ICAO:      "4TE0",
		Name:      "Lone Star Steel Company Airport",
		City:      "Lone Star",
		State:     "Texas",
		Country:   "US",
		Elevation: 285,
		Latitude:  32.9334983826,
		Longitude: -94.7334976196,
		Timezone:  "America/Chicago",
	},
	"4TE1": {
		ICAO:      "4TE1",
		Name:      "Figure 1 Ranch Airport",
		City:      "Booker",
		State:     "Texas",
		Country:   "US",
		Elevation: 2631,
		Latitude:  36.266998291,
		Longitude: -100.5350036621,
		Timezone:  "America/Chicago",
	},
	"4TE2": {
		ICAO:      "4TE2",
		Name:      "Lewis Private Airport",
		City:      "Del Rio",
		State:     "Texas",
		Country:   "US",
		Elevation: 1379,
		Latitude:  29.5440998077,
		Longitude: -100.672996521,
		Timezone:  "America/Chicago",
	},
	"4TE3": {
		ICAO:      "4TE3",
		Name:      "Finley Ranch Airport",
		City:      "Claude",
		State:     "Texas",
		Country:   "US",
		Elevation: 3400,
		Latitude:  35.1166992188,
		Longitude: -101.4420013428,
		Timezone:  "America/Chicago",
	},
	"4TE4": {
		ICAO:      "4TE4",
		Name:      "Taurus Mesa Airport",
		City:      "Alpine",
		State:     "Texas",
		Country:   "US",
		Elevation: 3576,
		Latitude:  29.6210002899,
		Longitude: -103.6790008545,
		Timezone:  "America/Chicago",
	},
	"4TE6": {
		ICAO:      "4TE6",
		Name:      "Jamison Airstrip",
		City:      "Dayton",
		State:     "Texas",
		Country:   "US",
		Elevation: 85,
		Latitude:  30.0627002716,
		Longitude: -94.9474029541,
		Timezone:  "America/Chicago",
	},
	"4TE7": {
		ICAO:      "4TE7",
		Name:      "Devil's River Ranch Airport",
		City:      "Del Rio",
		State:     "Texas",
		Country:   "US",
		Elevation: 1680,
		Latitude:  29.7408008575,
		Longitude: -100.9649963379,
		Timezone:  "America/Chicago",
	},
	"4TE8": {
		ICAO:      "4TE8",
		Name:      "Ben Bruce Memorial Airpark",
		City:      "Evadale",
		State:     "Texas",
		Country:   "US",
		Elevation: 44,
		Latitude:  30.3209991455,
		Longitude: -94.0735015869,
		Timezone:  "America/Chicago",
	},
	"4TE9": {
		ICAO:      "4TE9",
		Name:      "Squirrel Creek Ranch Airport",
		City:      "D'Hanis",
		State:     "Texas",
		Country:   "US",
		Elevation: 970,
		Latitude:  29.2640991211,
		Longitude: -99.3506011963,
		Timezone:  "America/Chicago",
	},
	"4TN0": {
		ICAO:      "4TN0",
		Name:      "Melton Field",
		City:      "Blaine",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1060,
		Latitude:  36.1519012451,
		Longitude: -83.7328033447,
		Timezone:  "America/New_York",
	},
	"4TN2": {
		ICAO:      "4TN2",
		Name:      "Buck Creek Ranch Airport",
		City:      "Crossville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1800,
		Latitude:  35.8833999634,
		Longitude: -84.9688034058,
		Timezone:  "America/Chicago",
	},
	"4TN3": {
		ICAO:      "4TN3",
		Name:      "Hawks Nest Airport",
		City:      "Moscow",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 410,
		Latitude:  35.1183013916,
		Longitude: -89.3677978516,
		Timezone:  "America/Chicago",
	},
	"4TN4": {
		ICAO:      "4TN4",
		Name:      "Cotton Field",
		City:      "Eads",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 345,
		Latitude:  35.1899986267,
		Longitude: -89.6386032104,
		Timezone:  "America/Chicago",
	},
	"4TN9": {
		ICAO:      "4TN9",
		Name:      "Southfork Airport",
		City:      "Jackson",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 387,
		Latitude:  35.6108016968,
		Longitude: -88.6947021484,
		Timezone:  "America/Chicago",
	},
	"4TS5": {
		ICAO:      "4TS5",
		Name:      "Goebel Field",
		City:      "Mountain Home",
		State:     "Texas",
		Country:   "US",
		Elevation: 2189,
		Latitude:  30.2219009399,
		Longitude: -99.4983978271,
		Timezone:  "America/Chicago",
	},
	"4TS7": {
		ICAO:      "4TS7",
		Name:      "Allison Ranch Airport",
		City:      "Fort Stockton",
		State:     "Texas",
		Country:   "US",
		Elevation: 4572,
		Latitude:  30.4501991272,
		Longitude: -102.9270019531,
		Timezone:  "America/Chicago",
	},
	"4TS8": {
		ICAO:      "4TS8",
		Name:      "Glasscock Field",
		City:      "D'Hanis",
		State:     "Texas",
		Country:   "US",
		Elevation: 1089,
		Latitude:  29.375,
		Longitude: -99.3420028687,
		Timezone:  "America/Chicago",
	},
	"4TX0": {
		ICAO:      "4TX0",
		Name:      "Slack Airport",
		City:      "Mont Belvieu",
		State:     "Texas",
		Country:   "US",
		Elevation: 35,
		Latitude:  29.8635997772,
		Longitude: -94.8343963623,
		Timezone:  "America/Chicago",
	},
	"4TX2": {
		ICAO:      "4TX2",
		Name:      "Stage Coach Hills Airport",
		City:      "Keller",
		State:     "Texas",
		Country:   "US",
		Elevation: 684,
		Latitude:  32.9668006897,
		Longitude: -97.2335968018,
		Timezone:  "America/Chicago",
	},
	"4TX3": {
		ICAO:      "4TX3",
		Name:      "Forest Hill Airport",
		City:      "Petty",
		State:     "Texas",
		Country:   "US",
		Elevation: 525,
		Latitude:  33.6612014771,
		Longitude: -95.8336029053,
		Timezone:  "America/Chicago",
	},
	"4TX4": {
		ICAO:      "4TX4",
		Name:      "Birk Airport",
		City:      "Kennedale",
		State:     "Texas",
		Country:   "US",
		Elevation: 700,
		Latitude:  32.6278991699,
		Longitude: -97.1920013428,
		Timezone:  "America/Chicago",
	},
	"4TX5": {
		ICAO:      "4TX5",
		Name:      "Hancock Airport",
		City:      "Kerens",
		State:     "Texas",
		Country:   "US",
		Elevation: 305,
		Latitude:  32.2299003601,
		Longitude: -96.2596969604,
		Timezone:  "America/Chicago",
	},
	"4TX6": {
		ICAO:      "4TX6",
		Name:      "Kilgore Airport",
		City:      "Kilgore",
		State:     "Texas",
		Country:   "US",
		Elevation: 365,
		Latitude:  32.4127006531,
		Longitude: -94.8182983398,
		Timezone:  "America/Chicago",
	},
	"4TX7": {
		ICAO:      "4TX7",
		Name:      "Oliver Airport",
		City:      "Knox City",
		State:     "Texas",
		Country:   "US",
		Elevation: 1500,
		Latitude:  33.4194984436,
		Longitude: -99.8656005859,
		Timezone:  "America/Chicago",
	},
	"4TX8": {
		ICAO:      "4TX8",
		Name:      "Addington Field",
		City:      "Krum",
		State:     "Texas",
		Country:   "US",
		Elevation: 705,
		Latitude:  33.2486991882,
		Longitude: -97.2238998413,
		Timezone:  "America/Chicago",
	},
	"4VA0": {
		ICAO:      "4VA0",
		Name:      "Wood Farm Airport",
		City:      "Wingina",
		State:     "Virginia",
		Country:   "US",
		Elevation: 350,
		Latitude:  37.6365013123,
		Longitude: -78.7300033569,
		Timezone:  "America/New_York",
	},
	"4VA1": {
		ICAO:      "4VA1",
		Name:      "Davis Field",
		City:      "Martinsville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 840,
		Latitude:  36.6867980957,
		Longitude: -79.6660995483,
		Timezone:  "America/New_York",
	},
	"4VA3": {
		ICAO:      "4VA3",
		Name:      "Flying W Airport",
		City:      "Richmond",
		State:     "Virginia",
		Country:   "US",
		Elevation: 101,
		Latitude:  37.429901123,
		Longitude: -77.3944015503,
		Timezone:  "America/New_York",
	},
	"4VA4": {
		ICAO:      "4VA4",
		Name:      "Hepner Airport",
		City:      "Woodstock",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1150,
		Latitude:  38.9347991943,
		Longitude: -78.5400009155,
		Timezone:  "America/New_York",
	},
	"4VA5": {
		ICAO:      "4VA5",
		Name:      "Starr Airport",
		City:      "Arrington",
		State:     "Virginia",
		Country:   "US",
		Elevation: 640,
		Latitude:  37.6595993042,
		Longitude: -78.922203064,
		Timezone:  "America/New_York",
	},
	"4VA6": {
		ICAO:      "4VA6",
		Name:      "Taylor Airport",
		City:      "Atlantic",
		State:     "Virginia",
		Country:   "US",
		Elevation: 48,
		Latitude:  37.8806991577,
		Longitude: -75.5059967041,
		Timezone:  "America/New_York",
	},
	"4VA8": {
		ICAO:      "4VA8",
		Name:      "Christian's Airport",
		City:      "Richmond",
		State:     "Virginia",
		Country:   "US",
		Elevation: 125,
		Latitude:  37.6138000488,
		Longitude: -77.2957992554,
		Timezone:  "America/New_York",
	},
	"4VA9": {
		ICAO:      "4VA9",
		Name:      "Pleasantdale Field",
		City:      "Rixeyville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 420,
		Latitude:  38.5690002441,
		Longitude: -77.920501709,
		Timezone:  "America/New_York",
	},
	"4VG2": {
		ICAO:      "4VG2",
		Name:      "Cool Water Airport",
		City:      "Ashland",
		State:     "Virginia",
		Country:   "US",
		Elevation: 250,
		Latitude:  37.858056,
		Longitude: -77.547222,
		Timezone:  "America/New_York",
	},
	"4WA0": {
		ICAO:      "4WA0",
		Name:      "B & G Farms Airport",
		City:      "Royal City",
		State:     "Washington",
		Country:   "US",
		Elevation: 1120,
		Latitude:  46.9402999878,
		Longitude: -119.737998962,
		Timezone:  "America/Los_Angeles",
	},
	"4WA1": {
		ICAO:      "4WA1",
		Name:      "Brown's Cape Horn Airport",
		City:      "Skamania",
		State:     "Washington",
		Country:   "US",
		Elevation: 920,
		Latitude:  45.6082992554,
		Longitude: -122.1969985962,
		Timezone:  "America/Los_Angeles",
	},
	"4WA2": {
		ICAO:      "4WA2",
		Name:      "Cricket Field",
		City:      "Little Rock",
		State:     "Washington",
		Country:   "US",
		Elevation: 184,
		Latitude:  46.9272003174,
		Longitude: -122.9680023193,
		Timezone:  "America/Los_Angeles",
	},
	"4WA4": {
		ICAO:      "4WA4",
		Name:      "Windsock Airport",
		City:      "Lopez",
		State:     "Washington",
		Country:   "US",
		Elevation: 30,
		Latitude:  48.5415000916,
		Longitude: -122.8870010376,
		Timezone:  "America/Los_Angeles",
	},
	"4WA6": {
		ICAO:      "4WA6",
		Name:      "Hille-Kimp Airstrip",
		City:      "Kahlotus",
		State:     "Washington",
		Country:   "US",
		Elevation: 1500,
		Latitude:  46.6184997559,
		Longitude: -118.5950012207,
		Timezone:  "America/Los_Angeles",
	},
	"4WA8": {
		ICAO:      "4WA8",
		Name:      "Riverside Airport",
		City:      "Deming",
		State:     "Washington",
		Country:   "US",
		Elevation: 175,
		Latitude:  48.8348007202,
		Longitude: -122.263999939,
		Timezone:  "America/Los_Angeles",
	},
	"4WA9": {
		ICAO:      "4WA9",
		Name:      "Port Orchard Airport",
		City:      "Port Orchard",
		State:     "Washington",
		Country:   "US",
		Elevation: 370,
		Latitude:  47.4323005676,
		Longitude: -122.6679992676,
		Timezone:  "America/Los_Angeles",
	},
	"4WI0": {
		ICAO:      "4WI0",
		Name:      "Wissota Airport",
		City:      "Chippewa Falls",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 936,
		Latitude:  44.9333000183,
		Longitude: -91.2723999023,
		Timezone:  "America/Chicago",
	},
	"4WI1": {
		ICAO:      "4WI1",
		Name:      "Bancroft East Airport",
		City:      "Rio",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 840,
		Latitude:  43.4832992554,
		Longitude: -89.1834030151,
		Timezone:  "America/Chicago",
	},
	"4WI2": {
		ICAO:      "4WI2",
		Name:      "Island View Airport",
		City:      "Crandon",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1640,
		Latitude:  45.606098175,
		Longitude: -89.0214996338,
		Timezone:  "America/Chicago",
	},
	"4WI3": {
		ICAO:      "4WI3",
		Name:      "Neveln Field",
		City:      "Crivitz",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 900,
		Latitude:  45.3251991272,
		Longitude: -88.137298584,
		Timezone:  "America/Chicago",
	},
	"4WI4": {
		ICAO:      "4WI4",
		Name:      "Turner Airport",
		City:      "Tomahawk",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1500,
		Latitude:  45.5741004944,
		Longitude: -89.7480010986,
		Timezone:  "America/Chicago",
	},
	"4WI5": {
		ICAO:      "4WI5",
		Name:      "Carlson Airport",
		City:      "Eau Claire",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 900,
		Latitude:  44.7235984802,
		Longitude: -91.4598999023,
		Timezone:  "America/Chicago",
	},
	"4WI6": {
		ICAO:      "4WI6",
		Name:      "Feldmann Field",
		City:      "Sheboygan Falls",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 750,
		Latitude:  43.8041992188,
		Longitude: -87.8973007202,
		Timezone:  "America/Chicago",
	},
	"4WI8": {
		ICAO:      "4WI8",
		Name:      "Carhart Farms Airport",
		City:      "Galesville",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 737,
		Latitude:  44.0513000488,
		Longitude: -91.3762969971,
		Timezone:  "America/Chicago",
	},
	"4WI9": {
		ICAO:      "4WI9",
		Name:      "Mar-O-Dae Farm Airport",
		City:      "Glenwood City",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1164,
		Latitude:  45.0069007874,
		Longitude: -92.1918029785,
		Timezone:  "America/Chicago",
	},
	"4WN2": {
		ICAO:      "4WN2",
		Name:      "Swensen Airport",
		City:      "Vesper",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1010,
		Latitude:  44.4169998169,
		Longitude: -89.9863967896,
		Timezone:  "America/Chicago",
	},
	"4WN3": {
		ICAO:      "4WN3",
		Name:      "Lakewood Airpark",
		City:      "Lakewood",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1300,
		Latitude:  45.3320999146,
		Longitude: -88.5360031128,
		Timezone:  "America/Chicago",
	},
	"4WN4": {
		ICAO:      "4WN4",
		Name:      "River Ridge Aero Estates Airport",
		City:      "Holcombe",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1050,
		Latitude:  45.2941017151,
		Longitude: -91.2329025269,
		Timezone:  "America/Chicago",
	},
	"4WN5": {
		ICAO:      "4WN5",
		Name:      "Romeo Airstrip",
		City:      "Balsam Lake",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1155,
		Latitude:  45.4570007324,
		Longitude: -92.3783035278,
		Timezone:  "America/Chicago",
	},
	"4WN9": {
		ICAO:      "4WN9",
		Name:      "Digger Dougs Airport",
		City:      "Salem",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 800,
		Latitude:  42.5975,
		Longitude: -88.146389,
		Timezone:  "America/Chicago",
	},
	"4WV4": {
		ICAO:      "4WV4",
		Name:      "Heaven's Landing Airport",
		City:      "Medley",
		State:     "West Virginia",
		Country:   "US",
		Elevation: 1220,
		Latitude:  39.1699981689,
		Longitude: -79.0616989136,
		Timezone:  "America/New_York",
	},
	"4XA2": {
		ICAO:      "4XA2",
		Name:      "Card Airfield",
		City:      "Kenedy",
		State:     "Texas",
		Country:   "US",
		Elevation: 320,
		Latitude:  28.789167,
		Longitude: -97.763389,
		Timezone:  "America/Chicago",
	},
	"4XA3": {
		ICAO:      "4XA3",
		Name:      "Owen Field",
		City:      "Clyde",
		State:     "Texas",
		Country:   "US",
		Elevation: 1971,
		Latitude:  32.253611,
		Longitude: -99.498611,
		Timezone:  "America/Chicago",
	},
	"4XA4": {
		ICAO:      "4XA4",
		Name:      "Apache Pass Airport",
		City:      "Rockdale",
		State:     "Texas",
		Country:   "US",
		Elevation: 410,
		Latitude:  30.734958,
		Longitude: -97.143378,
		Timezone:  "America/Chicago",
	},
	"4XA5": {
		ICAO:      "4XA5",
		Name:      "4-Shipp Airport",
		City:      "Burkburnett",
		State:     "Texas",
		Country:   "US",
		Elevation: 1004,
		Latitude:  34.03775,
		Longitude: -98.496,
		Timezone:  "America/Chicago",
	},
	"4XS0": {
		ICAO:      "4XS0",
		Name:      "Pfeffer & Son Farms Airport",
		City:      "Waller",
		State:     "Texas",
		Country:   "US",
		Elevation: 210,
		Latitude:  29.9297008514,
		Longitude: -95.9641036987,
		Timezone:  "America/Chicago",
	},
	"4XS1": {
		ICAO:      "4XS1",
		Name:      "Coyote Field",
		City:      "Corsicana",
		State:     "Texas",
		Country:   "US",
		Elevation: 430,
		Latitude:  32.1338996887,
		Longitude: -96.5205993652,
		Timezone:  "America/Chicago",
	},
	"4XS2": {
		ICAO:      "4XS2",
		Name:      "Teate Field",
		City:      "Paradise",
		State:     "Texas",
		Country:   "US",
		Elevation: 795,
		Latitude:  33.1706008911,
		Longitude: -97.7316970825,
		Timezone:  "America/Chicago",
	},
	"4XS5": {
		ICAO:      "4XS5",
		Name:      "Scrappin Valley Airport",
		City:      "Wiergate",
		State:     "Texas",
		Country:   "US",
		Elevation: 350,
		Latitude:  31.1219005585,
		Longitude: -93.8030014038,
		Timezone:  "America/Chicago",
	},
	"4XS6": {
		ICAO:      "4XS6",
		Name:      "H Young Ranch Airport",
		City:      "Willow City",
		State:     "Texas",
		Country:   "US",
		Elevation: 1700,
		Latitude:  30.3918991089,
		Longitude: -98.6613998413,
		Timezone:  "America/Chicago",
	},
	"4XS8": {
		ICAO:      "4XS8",
		Name:      "Winn Ranch Airport",
		City:      "Wimberley",
		State:     "Texas",
		Country:   "US",
		Elevation: 960,
		Latitude:  29.9668998718,
		Longitude: -98.1169967651,
		Timezone:  "America/Chicago",
	},
	"4XS9": {
		ICAO:      "4XS9",
		Name:      "Garnett Ranch Airport",
		City:      "Wimberley",
		State:     "Texas",
		Country:   "US",
		Elevation: 1300,
		Latitude:  30.0960006714,
		Longitude: -98.1628036499,
		Timezone:  "America/Chicago",
	},
	"50AK": {
		ICAO:      "50AK",
		Name:      "Robin'S Landing",
		City:      "Point Mackenzie",
		State:     "Alaska",
		Country:   "US",
		Elevation: 95,
		Latitude:  61.380461,
		Longitude: -150.11625,
		Timezone:  "America/Anchorage",
	},
	"50AZ": {
		ICAO:      "50AZ",
		Name:      "Rocky Ridge Airport",
		City:      "Rocky Ridge",
		State:     "Arizona",
		Country:   "US",
		Elevation: 5946,
		Latitude:  36.0597000122,
		Longitude: -110.5879974365,
		Timezone:  "America/Denver",
	},
	"50FD": {
		ICAO:      "50FD",
		Name:      "Cattle Creek Ranch Airport",
		City:      "Altha",
		State:     "Florida",
		Country:   "US",
		Elevation: 120,
		Latitude:  30.5181999207,
		Longitude: -85.1987991333,
		Timezone:  "America/Chicago",
	},
	"50FL": {
		ICAO:      "50FL",
		Name:      "Odom's Flying Service Airport",
		City:      "Jay",
		State:     "Florida",
		Country:   "US",
		Elevation: 245,
		Latitude:  30.8533000946,
		Longitude: -87.239402771,
		Timezone:  "America/Chicago",
	},
	"50GA": {
		ICAO:      "50GA",
		Name:      "Mallory Field",
		City:      "Valdosta",
		State:     "Georgia",
		Country:   "US",
		Elevation: 202,
		Latitude:  30.950799942,
		Longitude: -83.2341995239,
		Timezone:  "America/New_York",
	},
	"50II": {
		ICAO:      "50II",
		Name:      "Way West Airport",
		City:      "Bainbridge",
		State:     "Indiana",
		Country:   "US",
		Elevation: 890,
		Latitude:  39.7737007141,
		Longitude: -86.756401062,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"50IL": {
		ICAO:      "50IL",
		Name:      "Midland Airport",
		City:      "Antioch",
		State:     "Illinois",
		Country:   "US",
		Elevation: 750,
		Latitude:  42.488899231,
		Longitude: -88.1743011475,
		Timezone:  "America/Chicago",
	},
	"50IN": {
		ICAO:      "50IN",
		Name:      "Ropkey Field",
		City:      "Crawfordsville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 820,
		Latitude:  40.058086,
		Longitude: -86.795421,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"50KS": {
		ICAO:      "50KS",
		Name:      "Grandpa's Farm Airport",
		City:      "Pratt",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1820,
		Latitude:  37.665599823,
		Longitude: -98.6197967529,
		Timezone:  "America/Chicago",
	},
	"50KY": {
		ICAO:      "50KY",
		Name:      "Zanzibar Farm Airport",
		City:      "Paris",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 915,
		Latitude:  38.1297988892,
		Longitude: -84.3302001953,
		Timezone:  "America/New_York",
	},
	"50MN": {
		ICAO:      "50MN",
		Name:      "Zimmerman Airport",
		City:      "Herman",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1045,
		Latitude:  45.793598175,
		Longitude: -96.3001022339,
		Timezone:  "America/Chicago",
	},
	"50NC": {
		ICAO:      "50NC",
		Name:      "Ervin Airfield",
		City:      "Kannapolis",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 905,
		Latitude:  35.5449981689,
		Longitude: -80.6686019897,
		Timezone:  "America/New_York",
	},
	"50NE": {
		ICAO:      "50NE",
		Name:      "Elge Field",
		City:      "Aurora",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1800,
		Latitude:  40.9399986267,
		Longitude: -98.0781021118,
		Timezone:  "America/Chicago",
	},
	"50NM": {
		ICAO:      "50NM",
		Name:      "Ghost Ranch Strip",
		City:      "Abiquiu",
		State:     "New Mexico",
		Country:   "US",
		Elevation: 6440,
		Latitude:  36.3027992249,
		Longitude: -106.4879989624,
		Timezone:  "America/Denver",
	},
	"50NY": {
		ICAO:      "50NY",
		Name:      "Old Fort Farm Airport",
		City:      "Hemlock",
		State:     "New York",
		Country:   "US",
		Elevation: 1050,
		Latitude:  42.8144989014,
		Longitude: -77.5733032227,
		Timezone:  "America/New_York",
	},
	"50OH": {
		ICAO:      "50OH",
		Name:      "Culver Field",
		City:      "Oregon",
		State:     "Ohio",
		Country:   "US",
		Elevation: 620,
		Latitude:  41.6631011963,
		Longitude: -83.4154968262,
		Timezone:  "America/New_York",
	},
	"50OI": {
		ICAO:      "50OI",
		Name:      "Beckman Field",
		City:      "Ashville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 760,
		Latitude:  39.7425994873,
		Longitude: -83.0382003784,
		Timezone:  "America/New_York",
	},
	"50OR": {
		ICAO:      "50OR",
		Name:      "Goering Ranches Airport",
		City:      "Alfalfa",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3450,
		Latitude:  44.0973014832,
		Longitude: -120.9469985962,
		Timezone:  "America/Los_Angeles",
	},
	"50PA": {
		ICAO:      "50PA",
		Name:      "Pegasus Air Park",
		City:      "Stroudsburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 620,
		Latitude:  40.9333992004,
		Longitude: -75.3496017456,
		Timezone:  "America/New_York",
	},
	"50TA": {
		ICAO:      "50TA",
		Name:      "Moltz Airport",
		City:      "Geronimo",
		State:     "Texas",
		Country:   "US",
		Elevation: 600,
		Latitude:  29.6877002716,
		Longitude: -97.9961013794,
		Timezone:  "America/Chicago",
	},
	"50TE": {
		ICAO:      "50TE",
		Name:      "Keller Ranch Airport",
		City:      "Johnson City",
		State:     "Texas",
		Country:   "US",
		Elevation: 1550,
		Latitude:  30.2304992676,
		Longitude: -98.4753036499,
		Timezone:  "America/Chicago",
	},
	"50TN": {
		ICAO:      "50TN",
		Name:      "Paris Landing Airpark",
		City:      "Paris",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 400,
		Latitude:  36.352339,
		Longitude: -88.350492,
		Timezone:  "America/Chicago",
	},
	"50TX": {
		ICAO:      "50TX",
		Name:      "Kennedy Ranch Airport",
		City:      "Blanco",
		State:     "Texas",
		Country:   "US",
		Elevation: 1598,
		Latitude:  30.1375999451,
		Longitude: -98.5253982544,
		Timezone:  "America/Chicago",
	},
	"50WA": {
		ICAO:      "50WA",
		Name:      "Hartley Airport",
		City:      "Chehalis",
		State:     "Washington",
		Country:   "US",
		Elevation: 400,
		Latitude:  46.6198005676,
		Longitude: -122.9649963379,
		Timezone:  "America/Los_Angeles",
	},
	"50WI": {
		ICAO:      "50WI",
		Name:      "Earl's Air Park",
		City:      "Neenah",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 775,
		Latitude:  44.113899231,
		Longitude: -88.5270996094,
		Timezone:  "America/Chicago",
	},
	"50XS": {
		ICAO:      "50XS",
		Name:      "Hughes Ranch Airport",
		City:      "Quemado",
		State:     "Texas",
		Country:   "US",
		Elevation: 905,
		Latitude:  29.0433006287,
		Longitude: -100.5849990845,
		Timezone:  "America/Chicago",
	},
	"51AK": {
		ICAO:      "51AK",
		Name:      "Birch Creek Landing",
		City:      "Talkeetna",
		State:     "Alaska",
		Country:   "US",
		Elevation: 400,
		Latitude:  62.242289,
		Longitude: -150.065814,
		Timezone:  "America/Anchorage",
	},
	"51CA": {
		ICAO:      "51CA",
		Name:      "Kelly Airport",
		City:      "Lucerne Valley",
		State:     "California",
		Country:   "US",
		Elevation: 2806,
		Latitude:  34.4235992432,
		Longitude: -116.6169967651,
		Timezone:  "America/Los_Angeles",
	},
	"51CL": {
		ICAO:      "51CL",
		Name:      "Vetters Sky Ranch Airport",
		City:      "Galt",
		State:     "California",
		Country:   "US",
		Elevation: 80,
		Latitude:  38.2304992676,
		Longitude: -121.2220001221,
		Timezone:  "America/Los_Angeles",
	},
	"51CO": {
		ICAO:      "51CO",
		Name:      "Flying W No.2 Airport",
		City:      "Hotchkiss",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7105,
		Latitude:  38.8856010437,
		Longitude: -107.8040008545,
		Timezone:  "America/Denver",
	},
	"51FD": {
		ICAO:      "51FD",
		Name:      "Tex Merritt Private Airstrip",
		City:      "Leesburg",
		State:     "Florida",
		Country:   "US",
		Elevation: 95,
		Latitude:  28.6678009033,
		Longitude: -81.9180984497,
		Timezone:  "America/New_York",
	},
	"51FL": {
		ICAO:      "51FL",
		Name:      "Chiefland Sky Ranch Airport",
		City:      "Chiefland",
		State:     "Florida",
		Country:   "US",
		Elevation: 35,
		Latitude:  29.39489937,
		Longitude: -82.86979675,
		Timezone:  "America/New_York",
	},
	"51GA": {
		ICAO:      "51GA",
		Name:      "Smith Field",
		City:      "Tyrone",
		State:     "Georgia",
		Country:   "US",
		Elevation: 920,
		Latitude:  33.5080986023,
		Longitude: -84.6005020142,
		Timezone:  "America/New_York",
	},
	"51IL": {
		ICAO:      "51IL",
		Name:      "Marvin D Bradd Airport",
		City:      "Mansfield",
		State:     "Illinois",
		Country:   "US",
		Elevation: 797,
		Latitude:  40.2717018127,
		Longitude: -88.4944992065,
		Timezone:  "America/Chicago",
	},
	"51IN": {
		ICAO:      "51IN",
		Name:      "Dahnke Airport",
		City:      "West Lafayette",
		State:     "Indiana",
		Country:   "US",
		Elevation: 670,
		Latitude:  40.4928016663,
		Longitude: -86.9056015015,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"51IS": {
		ICAO:      "51IS",
		Name:      "Curry Airport",
		City:      "Griggsville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 620,
		Latitude:  39.659198761,
		Longitude: -90.7001037598,
		Timezone:  "America/Chicago",
	},
	"51KS": {
		ICAO:      "51KS",
		Name:      "Barnard Airport",
		City:      "Elmo",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1420,
		Latitude:  38.6370010376,
		Longitude: -97.2136001587,
		Timezone:  "America/Chicago",
	},
	"51LA": {
		ICAO:      "51LA",
		Name:      "R T Leblanc Airport",
		City:      "White Castle",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 15,
		Latitude:  30.1434993744,
		Longitude: -91.1531982422,
		Timezone:  "America/Chicago",
	},
	"51LS": {
		ICAO:      "51LS",
		Name:      "Red Oak Landing",
		City:      "Sunset",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 55,
		Latitude:  30.438053,
		Longitude: -92.116653,
		Timezone:  "America/Chicago",
	},
	"51NC": {
		ICAO:      "51NC",
		Name:      "Field of Dreams Airport",
		City:      "Zebulon",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 355,
		Latitude:  35.8619003296,
		Longitude: -78.3486022949,
		Timezone:  "America/New_York",
	},
	"51NE": {
		ICAO:      "51NE",
		Name:      "Bil Lo Airport",
		City:      "Fort Calhoun",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 997,
		Latitude:  41.4707984924,
		Longitude: -96.0211029053,
		Timezone:  "America/Chicago",
	},
	"51NY": {
		ICAO:      "51NY",
		Name:      "Maxon Field",
		City:      "Holley",
		State:     "New York",
		Country:   "US",
		Elevation: 615,
		Latitude:  43.1795005798,
		Longitude: -78.0397033691,
		Timezone:  "America/New_York",
	},
	"51OH": {
		ICAO:      "51OH",
		Name:      "Agner Airport",
		City:      "Ottawa",
		State:     "Ohio",
		Country:   "US",
		Elevation: 740,
		Latitude:  41.0703010559,
		Longitude: -84.0863037109,
		Timezone:  "America/New_York",
	},
	"51OI": {
		ICAO:      "51OI",
		Name:      "Clay's Rv Airport",
		City:      "Canal Fulton",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1035,
		Latitude:  40.8550987244,
		Longitude: -81.5989990234,
		Timezone:  "America/New_York",
	},
	"51OK": {
		ICAO:      "51OK",
		Name:      "Eagle Creek Airport",
		City:      "Beggs",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 740,
		Latitude:  35.793611,
		Longitude: -96.029722,
		Timezone:  "America/Chicago",
	},
	"51PA": {
		ICAO:      "51PA",
		Name:      "Lamberson Airport",
		City:      "New Oxford",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 540,
		Latitude:  39.8833999634,
		Longitude: -77.0830001831,
		Timezone:  "America/New_York",
	},
	"51SC": {
		ICAO:      "51SC",
		Name:      "Moccasin Creek Airport",
		City:      "Ehrhardt",
		State:     "South Carolina",
		Country:   "US",
		Elevation: 142,
		Latitude:  33.14220047,
		Longitude: -80.9555969238,
		Timezone:  "America/New_York",
	},
	"51TE": {
		ICAO:      "51TE",
		Name:      "Barstool Ranch Airport",
		City:      "Waxahachie",
		State:     "Texas",
		Country:   "US",
		Elevation: 546,
		Latitude:  32.470192,
		Longitude: -97.787119,
		Timezone:  "America/Chicago",
	},
	"51TN": {
		ICAO:      "51TN",
		Name:      "Glendale Field",
		City:      "Columbia",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 750,
		Latitude:  35.5331001282,
		Longitude: -86.9768981934,
		Timezone:  "America/Chicago",
	},
	"51TX": {
		ICAO:      "51TX",
		Name:      "N D Ranch Airport",
		City:      "Van",
		State:     "Texas",
		Country:   "US",
		Elevation: 540,
		Latitude:  32.5396003723,
		Longitude: -95.668296814,
		Timezone:  "America/Chicago",
	},
	"51VA": {
		ICAO:      "51VA",
		Name:      "Skyview Airport",
		City:      "Manassas",
		State:     "Virginia",
		Country:   "US",
		Elevation: 340,
		Latitude:  38.7162017822,
		Longitude: -77.6339035034,
		Timezone:  "America/New_York",
	},
	"51WA": {
		ICAO:      "51WA",
		Name:      "Evergreen Sky Ranch Airport",
		City:      "Black Diamond",
		State:     "Washington",
		Country:   "US",
		Elevation: 580,
		Latitude:  47.2681999207,
		Longitude: -122.0680007935,
		Timezone:  "America/Los_Angeles",
	},
	"51WI": {
		ICAO:      "51WI",
		Name:      "Gallinger Airport",
		City:      "Oshkosh",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 800,
		Latitude:  44.0628013611,
		Longitude: -88.5572967529,
		Timezone:  "America/Chicago",
	},
	"51WT": {
		ICAO:      "51WT",
		Name:      "Whiterik Field",
		City:      "Fruitland",
		State:     "Washington",
		Country:   "US",
		Elevation: 1895,
		Latitude:  48.010417,
		Longitude: -118.362028,
		Timezone:  "America/Los_Angeles",
	},
	"52AK": {
		ICAO:      "52AK",
		Name:      "Basquo Airport",
		City:      "Kenai",
		State:     "Alaska",
		Country:   "US",
		Elevation: 63,
		Latitude:  60.5057983398,
		Longitude: -151.2460021973,
		Timezone:  "America/Anchorage",
	},
	"52AR": {
		ICAO:      "52AR",
		Name:      "Ira's Airstrip",
		City:      "Greenbrier",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 590,
		Latitude:  35.2801017761,
		Longitude: -92.2415008545,
		Timezone:  "America/Chicago",
	},
	"52AZ": {
		ICAO:      "52AZ",
		Name:      "X-1 Ranch Upper Headquarters Airport",
		City:      "Seligman",
		State:     "Arizona",
		Country:   "US",
		Elevation: 5585,
		Latitude:  35.3833007813,
		Longitude: -113.3010025024,
		Timezone:  "America/Phoenix",
	},
	"52CA": {
		ICAO:      "52CA",
		Name:      "Lake Mathews Airport",
		City:      "Riverside",
		State:     "California",
		Country:   "US",
		Elevation: 1385,
		Latitude:  33.853099823,
		Longitude: -117.4240036011,
		Timezone:  "America/Los_Angeles",
	},
	"52CL": {
		ICAO:      "52CL",
		Name:      "Adelanto Airport",
		City:      "Adelanto",
		State:     "California",
		Country:   "US",
		Elevation: 3075,
		Latitude:  34.5374984741,
		Longitude: -117.4609985352,
		Timezone:  "America/Los_Angeles",
	},
	"52CN": {
		ICAO:      "52CN",
		Name:      "Ponderosa Sky Ranch Airport",
		City:      "Paynes Creek",
		State:     "California",
		Country:   "US",
		Elevation: 3454,
		Latitude:  40.3442993164,
		Longitude: -121.7750015259,
		Timezone:  "America/Los_Angeles",
	},
	"52CO": {
		ICAO:      "52CO",
		Name:      "Mountain View Ranch Airport",
		City:      "Monument",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7500,
		Latitude:  39.1096992493,
		Longitude: -104.6539993286,
		Timezone:  "America/Denver",
	},
	"52FA": {
		ICAO:      "52FA",
		Name:      "Bent Willies Airport",
		City:      "Lake Wales",
		State:     "Florida",
		Country:   "US",
		Elevation: 73,
		Latitude:  27.939611,
		Longitude: -81.432028,
		Timezone:  "America/New_York",
	},
	"52FL": {
		ICAO:      "52FL",
		Name:      "Lake Clinch Airpark",
		City:      "Frostproof",
		State:     "Florida",
		Country:   "US",
		Elevation: 140,
		Latitude:  27.7502994537,
		Longitude: -81.5595016479,
		Timezone:  "America/New_York",
	},
	"52IL": {
		ICAO:      "52IL",
		Name:      "Hasselbring Airport",
		City:      "Paxton",
		State:     "Illinois",
		Country:   "US",
		Elevation: 800,
		Latitude:  40.4103012085,
		Longitude: -88.0649032593,
		Timezone:  "America/Chicago",
	},
	"52IN": {
		ICAO:      "52IN",
		Name:      "Higginbotham Field",
		City:      "Prairieton",
		State:     "Indiana",
		Country:   "US",
		Elevation: 452,
		Latitude:  39.3414001465,
		Longitude: -87.5314025879,
		Timezone:  "America/Chicago",
	},
	"52KS": {
		ICAO:      "52KS",
		Name:      "Bland Airport",
		City:      "Eudora",
		State:     "Kansas",
		Country:   "US",
		Elevation: 935,
		Latitude:  38.9028015137,
		Longitude: -95.1322021484,
		Timezone:  "America/Chicago",
	},
	"52KY": {
		ICAO:      "52KY",
		Name:      "Air Castle Airport",
		City:      "Paris",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 900,
		Latitude:  38.1180992126,
		Longitude: -84.3023986816,
		Timezone:  "America/New_York",
	},
	"52MN": {
		ICAO:      "52MN",
		Name:      "Anderson Field",
		City:      "Kennedy",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 800,
		Latitude:  48.6430015564,
		Longitude: -96.8072967529,
		Timezone:  "America/Chicago",
	},
	"52MO": {
		ICAO:      "52MO",
		Name:      "Pleasant View Ultralightport",
		City:      "Zell",
		State:     "Missouri",
		Country:   "US",
		Elevation: 700,
		Latitude:  37.9622993469,
		Longitude: -90.1443023682,
		Timezone:  "America/Chicago",
	},
	"52NC": {
		ICAO:      "52NC",
		Name:      "Epley Airport",
		City:      "Jacksonville",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 43,
		Latitude:  34.8446006775,
		Longitude: -77.2667007446,
		Timezone:  "America/New_York",
	},
	"52ND": {
		ICAO:      "52ND",
		Name:      "Richtsmeier Airport",
		City:      "Hunter",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1047,
		Latitude:  47.1893997192,
		Longitude: -97.2894973755,
		Timezone:  "America/Chicago",
	},
	"52NE": {
		ICAO:      "52NE",
		Name:      "Svitak Airport",
		City:      "Morrill",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 4183,
		Latitude:  42.0449981689,
		Longitude: -103.966003418,
		Timezone:  "America/Denver",
	},
	"52NM": {
		ICAO:      "52NM",
		Name:      "Timberon Airport",
		City:      "Alamogordo",
		State:     "New Mexico",
		Country:   "US",
		Elevation: 6940,
		Latitude:  32.6338996887,
		Longitude: -105.6859970093,
		Timezone:  "America/Denver",
	},
	"52NY": {
		ICAO:      "52NY",
		Name:      "Bedson's Land Base Airport",
		City:      "Honeoye Falls",
		State:     "New York",
		Country:   "US",
		Elevation: 804,
		Latitude:  42.9305992126,
		Longitude: -77.644203186,
		Timezone:  "America/New_York",
	},
	"52OR": {
		ICAO:      "52OR",
		Name:      "Chinook Ultralight Airpark",
		City:      "Scappoose",
		State:     "Oregon",
		Country:   "US",
		Elevation: 40,
		Latitude:  45.7453994751,
		Longitude: -122.8460006714,
		Timezone:  "America/Los_Angeles",
	},
	"52PA": {
		ICAO:      "52PA",
		Name:      "Wildcat Airport",
		City:      "Tamaqua",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 980,
		Latitude:  40.7630996704,
		Longitude: -76.004699707,
		Timezone:  "America/New_York",
	},
	"52TA": {
		ICAO:      "52TA",
		Name:      "Red Wing Airport",
		City:      "Trenton",
		State:     "Texas",
		Country:   "US",
		Elevation: 755,
		Latitude:  33.4640007019,
		Longitude: -96.3503036499,
		Timezone:  "America/Chicago",
	},
	"52TE": {
		ICAO:      "52TE",
		Name:      "Robinson Ranch Airport",
		City:      "Junction",
		State:     "Texas",
		Country:   "US",
		Elevation: 2100,
		Latitude:  30.5168991089,
		Longitude: -99.516998291,
		Timezone:  "America/Chicago",
	},
	"52TS": {
		ICAO:      "52TS",
		Name:      "Fall Creek Air Ranch STOLport",
		City:      "Spicewood",
		State:     "Texas",
		Country:   "US",
		Elevation: 915,
		Latitude:  30.4185009003,
		Longitude: -98.1628036499,
		Timezone:  "America/Chicago",
	},
	"52WA": {
		ICAO:      "52WA",
		Name:      "Honn Farm Airport",
		City:      "Benge",
		State:     "Washington",
		Country:   "US",
		Elevation: 1620,
		Latitude:  46.9320983887,
		Longitude: -117.9759979248,
		Timezone:  "America/Los_Angeles",
	},
	"52WI": {
		ICAO:      "52WI",
		Name:      "John'S Field",
		City:      "Athens",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1450,
		Latitude:  45.159644,
		Longitude: -90.056883,
		Timezone:  "America/Chicago",
	},
	"53AK": {
		ICAO:      "53AK",
		Name:      "Lakewood Airstrip",
		City:      "Sterling",
		State:     "Alaska",
		Country:   "US",
		Elevation: 110,
		Latitude:  60.5345001221,
		Longitude: -150.9389953613,
		Timezone:  "America/Anchorage",
	},
	"53AZ": {
		ICAO:      "53AZ",
		Name:      "Shonto Airport",
		City:      "Shonto",
		State:     "Arizona",
		Country:   "US",
		Elevation: 6449,
		Latitude:  36.5881004333,
		Longitude: -110.6549987793,
		Timezone:  "America/Denver",
	},
	"53CL": {
		ICAO:      "53CL",
		Name:      "Lofty Redwoods Airport",
		City:      "Anchor Bay",
		State:     "California",
		Country:   "US",
		Elevation: 1317,
		Latitude:  38.8596000671,
		Longitude: -123.591003418,
		Timezone:  "America/Los_Angeles",
	},
	"53CN": {
		ICAO:      "53CN",
		Name:      "Thayer Aviation Airport",
		City:      "Grimes",
		State:     "California",
		Country:   "US",
		Elevation: 54,
		Latitude:  39.0749015808,
		Longitude: -121.9179992676,
		Timezone:  "America/Los_Angeles",
	},
	"53CO": {
		ICAO:      "53CO",
		Name:      "Rons Field",
		City:      "Two Buttes",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4096,
		Latitude:  37.5531005859,
		Longitude: -102.391998291,
		Timezone:  "America/Denver",
	},
	"53FD": {
		ICAO:      "53FD",
		Name:      "Charlotte's Field",
		City:      "Tallahassee",
		State:     "Florida",
		Country:   "US",
		Elevation: 80,
		Latitude:  30.4775009155,
		Longitude: -84.0346984863,
		Timezone:  "America/New_York",
	},
	"53GA": {
		ICAO:      "53GA",
		Name:      "Dawson Field",
		City:      "Cave Spring",
		State:     "Georgia",
		Country:   "US",
		Elevation: 675,
		Latitude:  34.1678009033,
		Longitude: -85.2585983276,
		Timezone:  "America/New_York",
	},
	"53IA": {
		ICAO:      "53IA",
		Name:      "Tama Airport",
		City:      "Tama",
		State:     "Iowa",
		Country:   "US",
		Elevation: 813,
		Latitude:  41.9500007629,
		Longitude: -92.5765991211,
		Timezone:  "America/Chicago",
	},
	"53II": {
		ICAO:      "53II",
		Name:      "Steinman Airport",
		City:      "Woodburn",
		State:     "Indiana",
		Country:   "US",
		Elevation: 760,
		Latitude:  41.0731010437,
		Longitude: -84.8407974243,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"53IL": {
		ICAO:      "53IL",
		Name:      "Clapper Airport",
		City:      "Mansfield",
		State:     "Illinois",
		Country:   "US",
		Elevation: 710,
		Latitude:  40.1528015137,
		Longitude: -88.5167007446,
		Timezone:  "America/Chicago",
	},
	"53IN": {
		ICAO:      "53IN",
		Name:      "Hartman Farms Field",
		City:      "Kokomo",
		State:     "Indiana",
		Country:   "US",
		Elevation: 818,
		Latitude:  40.5214004517,
		Longitude: -86.2149963379,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"53KS": {
		ICAO:      "53KS",
		Name:      "Weaver Ranch Airport",
		City:      "Hesston",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1460,
		Latitude:  38.1082992554,
		Longitude: -97.432800293,
		Timezone:  "America/Chicago",
	},
	"53KY": {
		ICAO:      "53KY",
		Name:      "Short Airport",
		City:      "Eubank",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 1130,
		Latitude:  37.2723007202,
		Longitude: -84.5571975708,
		Timezone:  "America/New_York",
	},
	"53MN": {
		ICAO:      "53MN",
		Name:      "Radloff's Cedar View Farms Airport",
		City:      "Lyle",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1240,
		Latitude:  43.5458984375,
		Longitude: -93.0266036987,
		Timezone:  "America/Chicago",
	},
	"53NC": {
		ICAO:      "53NC",
		Name:      "Mynatt Field",
		City:      "Lumberton",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 95,
		Latitude:  34.3781013489,
		Longitude: -79.0805969238,
		Timezone:  "America/New_York",
	},
	"53ND": {
		ICAO:      "53ND",
		Name:      "R. Leep Strip",
		City:      "Hurdsfield",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1894,
		Latitude:  47.5480995178,
		Longitude: -99.9470977783,
		Timezone:  "America/Chicago",
	},
	"53NE": {
		ICAO:      "53NE",
		Name:      "Starns Brothers Airport",
		City:      "Ashland",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1090,
		Latitude:  41.081401825,
		Longitude: -96.4033966064,
		Timezone:  "America/Chicago",
	},
	"53NY": {
		ICAO:      "53NY",
		Name:      "Smiths Land Base Airport",
		City:      "Honeoye Falls",
		State:     "New York",
		Country:   "US",
		Elevation: 627,
		Latitude:  42.9584007263,
		Longitude: -77.6157989502,
		Timezone:  "America/New_York",
	},
	"53OH": {
		ICAO:      "53OH",
		Name:      "Charloe Airport",
		City:      "Paulding",
		State:     "Ohio",
		Country:   "US",
		Elevation: 721,
		Latitude:  41.1344985962,
		Longitude: -84.4507980347,
		Timezone:  "America/New_York",
	},
	"53OK": {
		ICAO:      "53OK",
		Name:      "Thompson Private Airport",
		City:      "Tuttle",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1280,
		Latitude:  35.2500991821,
		Longitude: -97.766998291,
		Timezone:  "America/Chicago",
	},
	"53TA": {
		ICAO:      "53TA",
		Name:      "Shivers Private Airport",
		City:      "Vernon",
		State:     "Texas",
		Country:   "US",
		Elevation: 1215,
		Latitude:  34.1543006897,
		Longitude: -99.2641983032,
		Timezone:  "America/Chicago",
	},
	"53TS": {
		ICAO:      "53TS",
		Name:      "Bridges Field",
		City:      "Tioga",
		State:     "Texas",
		Country:   "US",
		Elevation: 715,
		Latitude:  33.4840011597,
		Longitude: -96.8392028809,
		Timezone:  "America/Chicago",
	},
	"53TX": {
		ICAO:      "53TX",
		Name:      "Cannon Field",
		City:      "Somerset",
		State:     "Texas",
		Country:   "US",
		Elevation: 610,
		Latitude:  29.2161006927,
		Longitude: -98.5494995117,
		Timezone:  "America/Chicago",
	},
	"53VA": {
		ICAO:      "53VA",
		Name:      "Horse Feathers Airport",
		City:      "Midland",
		State:     "Virginia",
		Country:   "US",
		Elevation: 390,
		Latitude:  38.6339988708,
		Longitude: -77.7574996948,
		Timezone:  "America/New_York",
	},
	"53WA": {
		ICAO:      "53WA",
		Name:      "Columbia Agricultural Airport",
		City:      "Pasco",
		State:     "Washington",
		Country:   "US",
		Elevation: 535,
		Latitude:  46.2924003601,
		Longitude: -118.983001709,
		Timezone:  "America/Los_Angeles",
	},
	"53WI": {
		ICAO:      "53WI",
		Name:      "Kelly Airport",
		City:      "Argyle",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1020,
		Latitude:  42.6981010437,
		Longitude: -89.7867965698,
		Timezone:  "America/Chicago",
	},
	"53XS": {
		ICAO:      "53XS",
		Name:      "Kornegay Private Airport",
		City:      "San Benito",
		State:     "Texas",
		Country:   "US",
		Elevation: 30,
		Latitude:  26.148399353,
		Longitude: -97.5916976929,
		Timezone:  "America/Chicago",
	},
	"54AR": {
		ICAO:      "54AR",
		Name:      "Winfield Airpark",
		City:      "Altus",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 390,
		Latitude:  35.392856,
		Longitude: -93.758706,
		Timezone:  "America/Chicago",
	},
	"54AZ": {
		ICAO:      "54AZ",
		Name:      "Somerton Airport",
		City:      "Somerton",
		State:     "Arizona",
		Country:   "US",
		Elevation: 177,
		Latitude:  32.6008987427,
		Longitude: -114.6660003662,
		Timezone:  "America/Phoenix",
	},
	"54CL": {
		ICAO:      "54CL",
		Name:      "Lake Riverside Estates Airport",
		City:      "Anza",
		State:     "California",
		Country:   "US",
		Elevation: 3410,
		Latitude:  33.5209007263,
		Longitude: -116.7969970703,
		Timezone:  "America/Los_Angeles",
	},
	"54CN": {
		ICAO:      "54CN",
		Name:      "Akin Airport",
		City:      "Placerville",
		State:     "California",
		Country:   "US",
		Elevation: 1595,
		Latitude:  38.7541007996,
		Longitude: -120.8990020752,
		Timezone:  "America/Los_Angeles",
	},
	"54GA": {
		ICAO:      "54GA",
		Name:      "Deerfield Landing Airport",
		City:      "Hampton",
		State:     "Georgia",
		Country:   "US",
		Elevation: 890,
		Latitude:  33.4642982483,
		Longitude: -84.2683029175,
		Timezone:  "America/New_York",
	},
	"54II": {
		ICAO:      "54II",
		Name:      "Caldwell Field",
		City:      "Bunker Hill",
		State:     "Indiana",
		Country:   "US",
		Elevation: 800,
		Latitude:  40.6245002747,
		Longitude: -86.1552963257,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"54IL": {
		ICAO:      "54IL",
		Name:      "Rothrock Airport",
		City:      "Streator",
		State:     "Illinois",
		Country:   "US",
		Elevation: 653,
		Latitude:  41.1286010742,
		Longitude: -88.8020019531,
		Timezone:  "America/Chicago",
	},
	"54IS": {
		ICAO:      "54IS",
		Name:      "Zea Mays Field",
		City:      "Ohio",
		State:     "Illinois",
		Country:   "US",
		Elevation: 766,
		Latitude:  41.5266990662,
		Longitude: -89.4180984497,
		Timezone:  "America/Chicago",
	},
	"54KS": {
		ICAO:      "54KS",
		Name:      "Michael Airport",
		City:      "Wellsville",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1020,
		Latitude:  38.8027992249,
		Longitude: -95.112701416,
		Timezone:  "America/Chicago",
	},
	"54KY": {
		ICAO:      "54KY",
		Name:      "Turner Field",
		City:      "Murray",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 350,
		Latitude:  36.731098175,
		Longitude: -88.2472000122,
		Timezone:  "America/Chicago",
	},
	"54LA": {
		ICAO:      "54LA",
		Name:      "Triangle J Airport",
		City:      "Baton Rouge",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 69,
		Latitude:  30.5627002716,
		Longitude: -91.0383987427,
		Timezone:  "America/Chicago",
	},
	"54ME": {
		ICAO:      "54ME",
		Name:      "Cliff Dow Airport",
		City:      "New Gloucester",
		State:     "Maine",
		Country:   "US",
		Elevation: 415,
		Latitude:  43.949444,
		Longitude: -70.320278,
		Timezone:  "America/New_York",
	},
	"54MI": {
		ICAO:      "54MI",
		Name:      "Pingston Aeroworks Airport",
		City:      "Bryon",
		State:     "Michigan",
		Country:   "US",
		Elevation: 880,
		Latitude:  42.8102989197,
		Longitude: -84.0830993652,
		Timezone:  "America/Detroit",
	},
	"54MN": {
		ICAO:      "54MN",
		Name:      "Sell's Flying Field",
		City:      "Mayer",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 980,
		Latitude:  44.8791007996,
		Longitude: -93.8854980469,
		Timezone:  "America/Chicago",
	},
	"54MO": {
		ICAO:      "54MO",
		Name:      "Ozark's Flying Patch Airport",
		City:      "Sparta",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1530,
		Latitude:  37.0013999939,
		Longitude: -92.9404983521,
		Timezone:  "America/Chicago",
	},
	"54NC": {
		ICAO:      "54NC",
		Name:      "Dillard Airport",
		City:      "Winfall",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 15,
		Latitude:  36.2393989563,
		Longitude: -76.4171981812,
		Timezone:  "America/New_York",
	},
	"54NY": {
		ICAO:      "54NY",
		Name:      "Erb Acres Airport",
		City:      "Angelica",
		State:     "New York",
		Country:   "US",
		Elevation: 1940,
		Latitude:  42.3550987244,
		Longitude: -78.0353012085,
		Timezone:  "America/New_York",
	},
	"54OH": {
		ICAO:      "54OH",
		Name:      "Buehler Airport",
		City:      "Paulding",
		State:     "Ohio",
		Country:   "US",
		Elevation: 728,
		Latitude:  41.1234016418,
		Longitude: -84.6176986694,
		Timezone:  "America/New_York",
	},
	"54OK": {
		ICAO:      "54OK",
		Name:      "Ambassador Ultralightport",
		City:      "Mounds",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 687,
		Latitude:  35.8694992065,
		Longitude: -96.0350036621,
		Timezone:  "America/Chicago",
	},
	"54TA": {
		ICAO:      "54TA",
		Name:      "George P Shanks Airport",
		City:      "Waxahachie",
		State:     "Texas",
		Country:   "US",
		Elevation: 610,
		Latitude:  32.2997016907,
		Longitude: -96.873298645,
		Timezone:  "America/Chicago",
	},
	"54TE": {
		ICAO:      "54TE",
		Name:      "Camelot Airport",
		City:      "Thrall",
		State:     "Texas",
		Country:   "US",
		Elevation: 480,
		Latitude:  30.5151996613,
		Longitude: -97.2668991089,
		Timezone:  "America/Chicago",
	},
	"54TS": {
		ICAO:      "54TS",
		Name:      "J Bar Wc Ranch Airport",
		City:      "Weatherford",
		State:     "Texas",
		Country:   "US",
		Elevation: 1235,
		Latitude:  32.8111991882,
		Longitude: -99.8517990112,
		Timezone:  "America/Chicago",
	},
	"54TX": {
		ICAO:      "54TX",
		Name:      "Flying H Ranch Airport",
		City:      "Kingsland",
		State:     "Texas",
		Country:   "US",
		Elevation: 1000,
		Latitude:  30.677400589,
		Longitude: -98.5252990723,
		Timezone:  "America/Chicago",
	},
	"54WI": {
		ICAO:      "54WI",
		Name:      "Flyplassen Airport",
		City:      "Woodville",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1300,
		Latitude:  45.0083007813,
		Longitude: -92.2751998901,
		Timezone:  "America/Chicago",
	},
	"55AK": {
		ICAO:      "55AK",
		Name:      "Lawrence Airstrip",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 200,
		Latitude:  61.4958000183,
		Longitude: -149.699005127,
		Timezone:  "America/Anchorage",
	},
	"55AZ": {
		ICAO:      "55AZ",
		Name:      "Potters Field",
		City:      "Stanfield",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1309,
		Latitude:  32.8739013672,
		Longitude: -111.9599990845,
		Timezone:  "America/Phoenix",
	},
	"55CL": {
		ICAO:      "55CL",
		Name:      "Costerisan Farms Airport",
		City:      "Bakersfield",
		State:     "California",
		Country:   "US",
		Elevation: 346,
		Latitude:  35.2633018494,
		Longitude: -119.0459976196,
		Timezone:  "America/Los_Angeles",
	},
	"55FD": {
		ICAO:      "55FD",
		Name:      "Dotson Airport",
		City:      "Baker",
		State:     "Florida",
		Country:   "US",
		Elevation: 251,
		Latitude:  30.8185005188,
		Longitude: -86.68800354,
		Timezone:  "America/Chicago",
	},
	"55GE": {
		ICAO:      "55GE",
		Name:      "Rostex Airport",
		City:      "Tennga",
		State:     "Georgia",
		Country:   "US",
		Elevation: 887,
		Latitude:  34.9805984497,
		Longitude: -84.7416000366,
		Timezone:  "America/New_York",
	},
	"55IL": {
		ICAO:      "55IL",
		Name:      "Brandt Airport",
		City:      "Manteno",
		State:     "Illinois",
		Country:   "US",
		Elevation: 680,
		Latitude:  41.2778015137,
		Longitude: -87.822303772,
		Timezone:  "America/Chicago",
	},
	"55IN": {
		ICAO:      "55IN",
		Name:      "Brenneke Airport",
		City:      "Woodburn",
		State:     "Indiana",
		Country:   "US",
		Elevation: 790,
		Latitude:  41.1446990967,
		Longitude: -84.862197876,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"55IS": {
		ICAO:      "55IS",
		Name:      "Glatthaar Airport",
		City:      "Grandview",
		State:     "Illinois",
		Country:   "US",
		Elevation: 765,
		Latitude:  39.5250015259,
		Longitude: -87.8059005737,
		Timezone:  "America/Chicago",
	},
	"55LL": {
		ICAO:      "55LL",
		Name:      "Sky Soaring Airport",
		City:      "Union",
		State:     "Illinois",
		Country:   "US",
		Elevation: 884,
		Latitude:  42.1542015076,
		Longitude: -88.5114974976,
		Timezone:  "America/Chicago",
	},
	"55ME": {
		ICAO:      "55ME",
		Name:      "HIghland View Field",
		City:      "Searsmont",
		State:     "Maine",
		Country:   "US",
		Elevation: 600,
		Latitude:  44.318056,
		Longitude: -69.138333,
		Timezone:  "America/New_York",
	},
	"55MN": {
		ICAO:      "55MN",
		Name:      "Conley Field",
		City:      "Mora",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1004,
		Latitude:  45.8352012634,
		Longitude: -93.3355026245,
		Timezone:  "America/Chicago",
	},
	"55MO": {
		ICAO:      "55MO",
		Name:      "Tightsqueeze Field",
		City:      "Jonesburg",
		State:     "Missouri",
		Country:   "US",
		Elevation: 820,
		Latitude:  38.8722991943,
		Longitude: -91.2602005005,
		Timezone:  "America/Chicago",
	},
	"55ND": {
		ICAO:      "55ND",
		Name:      "Pete's Port Airport",
		City:      "Killdeer",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 2260,
		Latitude:  47.3555984497,
		Longitude: -102.7109985352,
		Timezone:  "America/Denver",
	},
	"55NM": {
		ICAO:      "55NM",
		Name:      "Burris Ranch Nr 1 Airport",
		City:      "Belen",
		State:     "New Mexico",
		Country:   "US",
		Elevation: 5218,
		Latitude:  34.4833984375,
		Longitude: -106.6129989624,
		Timezone:  "America/Denver",
	},
	"55NY": {
		ICAO:      "55NY",
		Name:      "Ridgeview Airport",
		City:      "Jamestown",
		State:     "New York",
		Country:   "US",
		Elevation: 1645,
		Latitude:  42.0820007324,
		Longitude: -79.3123016357,
		Timezone:  "America/New_York",
	},
	"55OI": {
		ICAO:      "55OI",
		Name:      "Aero Flight Center Airport",
		City:      "East Palestine",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1140,
		Latitude:  40.8586997986,
		Longitude: -80.5353012085,
		Timezone:  "America/New_York",
	},
	"55OK": {
		ICAO:      "55OK",
		Name:      "Gilstrap Field",
		City:      "Inola",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 575,
		Latitude:  36.2239990234,
		Longitude: -95.5783004761,
		Timezone:  "America/Chicago",
	},
	"55OR": {
		ICAO:      "55OR",
		Name:      "Muddy Creek Airport",
		City:      "Corvallis",
		State:     "Oregon",
		Country:   "US",
		Elevation: 253,
		Latitude:  44.436000824,
		Longitude: -123.3079986572,
		Timezone:  "America/Los_Angeles",
	},
	"55PA": {
		ICAO:      "55PA",
		Name:      "Sency Airport",
		City:      "Treskow",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1350,
		Latitude:  40.8955993652,
		Longitude: -75.9440994263,
		Timezone:  "America/New_York",
	},
	"55TE": {
		ICAO:      "55TE",
		Name:      "Lebegue Lsa Landing",
		City:      "Elmendorf",
		State:     "Texas",
		Country:   "US",
		Elevation: 419,
		Latitude:  29.230278,
		Longitude: -98.35,
		Timezone:  "America/Chicago",
	},
	"55TX": {
		ICAO:      "55TX",
		Name:      "Stonecipher Airport",
		City:      "Beaumont",
		State:     "Texas",
		Country:   "US",
		Elevation: 39,
		Latitude:  30.1121997833,
		Longitude: -94.2563018799,
		Timezone:  "America/Chicago",
	},
	"55WA": {
		ICAO:      "55WA",
		Name:      "Wilkinson Ranch Airport",
		City:      "Yakima",
		State:     "Washington",
		Country:   "US",
		Elevation: 1670,
		Latitude:  46.5280990601,
		Longitude: -120.7610015869,
		Timezone:  "America/Los_Angeles",
	},
	"55WI": {
		ICAO:      "55WI",
		Name:      "Cranmoor Airstrip",
		City:      "Nekoosa",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 985,
		Latitude:  44.3180007935,
		Longitude: -89.9686965942,
		Timezone:  "America/Chicago",
	},
	"55XS": {
		ICAO:      "55XS",
		Name:      "Frontier Airport",
		City:      "Spofford",
		State:     "Texas",
		Country:   "US",
		Elevation: 1001,
		Latitude:  29.1662998199,
		Longitude: -100.4179992676,
		Timezone:  "America/Chicago",
	},
	"56AK": {
		ICAO:      "56AK",
		Name:      "Bechtol Field",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 300,
		Latitude:  61.5750007629,
		Longitude: -149.6060028076,
		Timezone:  "America/Anchorage",
	},
	"56AZ": {
		ICAO:      "56AZ",
		Name:      "Mauldin Airstrip",
		City:      "Tonopah",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1193,
		Latitude:  33.4819984436,
		Longitude: -113.0019989014,
		Timezone:  "America/Phoenix",
	},
	"56CL": {
		ICAO:      "56CL",
		Name:      "Cyr Aviation Airport",
		City:      "Blythe",
		State:     "California",
		Country:   "US",
		Elevation: 263,
		Latitude:  33.591999054,
		Longitude: -114.6060028076,
		Timezone:  "America/Los_Angeles",
	},
	"56FD": {
		ICAO:      "56FD",
		Name:      "North American Farms Airport",
		City:      "Malone",
		State:     "Florida",
		Country:   "US",
		Elevation: 130,
		Latitude:  30.9645996094,
		Longitude: -85.0693969727,
		Timezone:  "America/Chicago",
	},
	"56FL": {
		ICAO:      "56FL",
		Name:      "Buchanan Airport",
		City:      "Jay",
		State:     "Florida",
		Country:   "US",
		Elevation: 246,
		Latitude:  30.8759994507,
		Longitude: -87.2074966431,
		Timezone:  "America/Chicago",
	},
	"56IL": {
		ICAO:      "56IL",
		Name:      "Lee Creek Airport",
		City:      "Caledonia",
		State:     "Illinois",
		Country:   "US",
		Elevation: 965,
		Latitude:  42.4625015259,
		Longitude: -88.875,
		Timezone:  "America/Chicago",
	},
	"56IN": {
		ICAO:      "56IN",
		Name:      "Casad Industrial Park Airport",
		City:      "New Haven",
		State:     "Indiana",
		Country:   "US",
		Elevation: 766,
		Latitude:  41.0792007446,
		Longitude: -84.9427032471,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"56IS": {
		ICAO:      "56IS",
		Name:      "Schmidt Airport",
		City:      "Thomasboro",
		State:     "Illinois",
		Country:   "US",
		Elevation: 690,
		Latitude:  40.2430992126,
		Longitude: -88.0828018188,
		Timezone:  "America/Chicago",
	},
	"56LA": {
		ICAO:      "56LA",
		Name:      "Old Hickory Ranch Airport",
		City:      "Keithville",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 180,
		Latitude:  32.3452987671,
		Longitude: -93.8182983398,
		Timezone:  "America/Chicago",
	},
	"56LL": {
		ICAO:      "56LL",
		Name:      "Wade Airport",
		City:      "Waterman",
		State:     "Illinois",
		Country:   "US",
		Elevation: 810,
		Latitude:  41.7550010681,
		Longitude: -88.7712020874,
		Timezone:  "America/Chicago",
	},
	"56MN": {
		ICAO:      "56MN",
		Name:      "Lothert's Farm Strip",
		City:      "Morton",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1015,
		Latitude:  44.6455001831,
		Longitude: -94.9614028931,
		Timezone:  "America/Chicago",
	},
	"56MO": {
		ICAO:      "56MO",
		Name:      "Mc Clurg Airport",
		City:      "Monett",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1376,
		Latitude:  36.9709014893,
		Longitude: -93.9241027832,
		Timezone:  "America/Chicago",
	},
	"56NC": {
		ICAO:      "56NC",
		Name:      "Wheat Field",
		City:      "Tryon",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 960,
		Latitude:  35.2066993713,
		Longitude: -82.0318984985,
		Timezone:  "America/New_York",
	},
	"56NE": {
		ICAO:      "56NE",
		Name:      "Noble Field",
		City:      "Ravenna",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1967,
		Latitude:  41.0327987671,
		Longitude: -98.81199646,
		Timezone:  "America/Chicago",
	},
	"56NY": {
		ICAO:      "56NY",
		Name:      "Maynard's Airport",
		City:      "Kendall",
		State:     "New York",
		Country:   "US",
		Elevation: 342,
		Latitude:  43.3252983093,
		Longitude: -78.049697876,
		Timezone:  "America/New_York",
	},
	"56OH": {
		ICAO:      "56OH",
		Name:      "Richey Airport",
		City:      "Petersburg",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1195,
		Latitude:  40.9072990417,
		Longitude: -80.5514984131,
		Timezone:  "America/New_York",
	},
	"56OI": {
		ICAO:      "56OI",
		Name:      "Double S Farms Airport",
		City:      "Kipton",
		State:     "Ohio",
		Country:   "US",
		Elevation: 898,
		Latitude:  41.2056007385,
		Longitude: -82.3312988281,
		Timezone:  "America/New_York",
	},
	"56OK": {
		ICAO:      "56OK",
		Name:      "L D Airport",
		City:      "Mc Loud",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1080,
		Latitude:  35.4584007263,
		Longitude: -97.0670013428,
		Timezone:  "America/Chicago",
	},
	"56PA": {
		ICAO:      "56PA",
		Name:      "Hoge Farm Airport",
		City:      "Tinicum",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 430,
		Latitude:  40.4695014954,
		Longitude: -75.0820999146,
		Timezone:  "America/New_York",
	},
	"56TE": {
		ICAO:      "56TE",
		Name:      "Cardiff Brothers Airport",
		City:      "Katy",
		State:     "Texas",
		Country:   "US",
		Elevation: 140,
		Latitude:  29.7294006348,
		Longitude: -95.8497009277,
		Timezone:  "America/Chicago",
	},
	"56TS": {
		ICAO:      "56TS",
		Name:      "Elgin Intracontinental Airport",
		City:      "Elgin",
		State:     "Texas",
		Country:   "US",
		Elevation: 558,
		Latitude:  30.3376998901,
		Longitude: -97.3503036499,
		Timezone:  "America/Chicago",
	},
	"56TX": {
		ICAO:      "56TX",
		Name:      "Anchorage Farm Field",
		City:      "Warrenton",
		State:     "Texas",
		Country:   "US",
		Elevation: 440,
		Latitude:  30.0093994141,
		Longitude: -96.7593994141,
		Timezone:  "America/Chicago",
	},
	"56WI": {
		ICAO:      "56WI",
		Name:      "Jaks Field",
		City:      "Mosinee",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1190,
		Latitude:  44.773399353,
		Longitude: -89.5936965942,
		Timezone:  "America/Chicago",
	},
	"57AL": {
		ICAO:      "57AL",
		Name:      "Baswell Airport",
		City:      "Loxley",
		State:     "Alabama",
		Country:   "US",
		Elevation: 190,
		Latitude:  30.65320015,
		Longitude: -87.72370148,
		Timezone:  "America/Chicago",
	},
	"57AZ": {
		ICAO:      "57AZ",
		Name:      "La Cholla Airpark",
		City:      "Tucson",
		State:     "Arizona",
		Country:   "US",
		Elevation: 2940,
		Latitude:  32.4467010498,
		Longitude: -111.0029983521,
		Timezone:  "America/Phoenix",
	},
	"57CL": {
		ICAO:      "57CL",
		Name:      "Boron Airstrip",
		City:      "Boron",
		State:     "California",
		Country:   "US",
		Elevation: 2499,
		Latitude:  35.0036010742,
		Longitude: -117.6070022583,
		Timezone:  "America/Los_Angeles",
	},
	"57CN": {
		ICAO:      "57CN",
		Name:      "Van Vleck Airport",
		City:      "Rancho Murieta",
		State:     "California",
		Country:   "US",
		Elevation: 175,
		Latitude:  38.4691009521,
		Longitude: -121.0749969482,
		Timezone:  "America/Los_Angeles",
	},
	"57FA": {
		ICAO:      "57FA",
		Name:      "Lake X Airport",
		City:      "St Cloud",
		State:     "Florida",
		Country:   "US",
		Elevation: 72,
		Latitude:  28.2085990906,
		Longitude: -81.1192016602,
		Timezone:  "America/New_York",
	},
	"57FD": {
		ICAO:      "57FD",
		Name:      "Sunset Strip Airpark",
		City:      "Brooksville",
		State:     "Florida",
		Country:   "US",
		Elevation: 70,
		Latitude:  28.5396995544,
		Longitude: -82.2288970947,
		Timezone:  "America/New_York",
	},
	"57GA": {
		ICAO:      "57GA",
		Name:      "Blue Ridge Skyport Airport",
		City:      "Blue Ridge",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1946,
		Latitude:  34.8630981445,
		Longitude: -84.3937988281,
		Timezone:  "America/New_York",
	},
	"57II": {
		ICAO:      "57II",
		Name:      "Reimer Aerodrome",
		City:      "Jamestown",
		State:     "Indiana",
		Country:   "US",
		Elevation: 930,
		Latitude:  39.9641990662,
		Longitude: -86.5674972534,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"57IS": {
		ICAO:      "57IS",
		Name:      "Dozier Airport",
		City:      "Fairfield",
		State:     "Illinois",
		Country:   "US",
		Elevation: 420,
		Latitude:  38.3166999817,
		Longitude: -88.2723007202,
		Timezone:  "America/Chicago",
	},
	"57KY": {
		ICAO:      "57KY",
		Name:      "Belcher Regional Airport",
		City:      "Belcher",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 1700,
		Latitude:  37.3694000244,
		Longitude: -82.3242034912,
		Timezone:  "America/New_York",
	},
	"57ME": {
		ICAO:      "57ME",
		Name:      "Dyer's Landing Airport",
		City:      "Fryeburg",
		State:     "Maine",
		Country:   "US",
		Elevation: 441,
		Latitude:  43.9178009033,
		Longitude: -70.8544006348,
		Timezone:  "America/New_York",
	},
	"57NC": {
		ICAO:      "57NC",
		Name:      "Sossamon Field",
		City:      "Bryson City",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 1940,
		Latitude:  35.4264984131,
		Longitude: -83.4581985474,
		Timezone:  "America/New_York",
	},
	"57ND": {
		ICAO:      "57ND",
		Name:      "Odegaard Airport",
		City:      "Kindred",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1020,
		Latitude:  46.6082992554,
		Longitude: -97.0595016479,
		Timezone:  "America/Chicago",
	},
	"57NE": {
		ICAO:      "57NE",
		Name:      "Cole Memorial Airport",
		City:      "Merriman",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3259,
		Latitude:  42.9241981506,
		Longitude: -101.6940002441,
		Timezone:  "America/Denver",
	},
	"57NM": {
		ICAO:      "57NM",
		Name:      "Monte Prieto Ranch Airport",
		City:      "Claunch",
		State:     "New Mexico",
		Country:   "US",
		Elevation: 6164,
		Latitude:  34.0886993408,
		Longitude: -106.1190032959,
		Timezone:  "America/Denver",
	},
	"57OH": {
		ICAO:      "57OH",
		Name:      "Fillmans Farms Field",
		City:      "Port Washington",
		State:     "Ohio",
		Country:   "US",
		Elevation: 820,
		Latitude:  40.3362007141,
		Longitude: -81.4900970459,
		Timezone:  "America/New_York",
	},
	"57OI": {
		ICAO:      "57OI",
		Name:      "Logan's Chance Airport",
		City:      "Lawrenceville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1100,
		Latitude:  39.9967002869,
		Longitude: -83.8546981812,
		Timezone:  "America/New_York",
	},
	"57OR": {
		ICAO:      "57OR",
		Name:      "Umpqua River Farm Airport",
		City:      "Sutherlin",
		State:     "Oregon",
		Country:   "US",
		Elevation: 380,
		Latitude:  43.3400001526,
		Longitude: -123.4509963989,
		Timezone:  "America/Los_Angeles",
	},
	"57PA": {
		ICAO:      "57PA",
		Name:      "H&H Personal Use Airport",
		City:      "Tioga",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1680,
		Latitude:  41.9029006958,
		Longitude: -77.2303009033,
		Timezone:  "America/New_York",
	},
	"57PN": {
		ICAO:      "57PN",
		Name:      "Ransome STOLport",
		City:      "Solebury",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 360,
		Latitude:  40.3975982666,
		Longitude: -75.0106964111,
		Timezone:  "America/New_York",
	},
	"57TE": {
		ICAO:      "57TE",
		Name:      "Prade Ranch Airport",
		City:      "Leakey",
		State:     "Texas",
		Country:   "US",
		Elevation: 2296,
		Latitude:  29.9388008118,
		Longitude: -99.7834014893,
		Timezone:  "America/Chicago",
	},
	"57TX": {
		ICAO:      "57TX",
		Name:      "Brown Ranch Airport",
		City:      "Beeville",
		State:     "Texas",
		Country:   "US",
		Elevation: 325,
		Latitude:  28.4419002533,
		Longitude: -97.804397583,
		Timezone:  "America/Chicago",
	},
	"57VA": {
		ICAO:      "57VA",
		Name:      "Toga Airport",
		City:      "Buckingham Courthouse",
		State:     "Virginia",
		Country:   "US",
		Elevation: 725,
		Latitude:  37.4510002136,
		Longitude: -78.6785964966,
		Timezone:  "America/New_York",
	},
	"57WA": {
		ICAO:      "57WA",
		Name:      "Kent Farms Airport",
		City:      "Othello",
		State:     "Washington",
		Country:   "US",
		Elevation: 1155,
		Latitude:  46.873500824,
		Longitude: -119.1299972534,
		Timezone:  "America/Los_Angeles",
	},
	"57XS": {
		ICAO:      "57XS",
		Name:      "Hoopes Ranch Airport",
		City:      "Weir",
		State:     "Texas",
		Country:   "US",
		Elevation: 800,
		Latitude:  30.6968994141,
		Longitude: -97.5600967407,
		Timezone:  "America/Chicago",
	},
	"58AK": {
		ICAO:      "58AK",
		Name:      "Fairview East Airport",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 100,
		Latitude:  61.5526008606,
		Longitude: -149.3399963379,
		Timezone:  "America/Anchorage",
	},
	"58AZ": {
		ICAO:      "58AZ",
		Name:      "Chapman Ranch Airstrip",
		City:      "Young",
		State:     "Arizona",
		Country:   "US",
		Elevation: 5115,
		Latitude:  34.093611,
		Longitude: -110.932778,
		Timezone:  "America/Phoenix",
	},
	"58CL": {
		ICAO:      "58CL",
		Name:      "Borrego Air Ranch Airport",
		City:      "Borrego Springs",
		State:     "California",
		Country:   "US",
		Elevation: 575,
		Latitude:  33.1916999817,
		Longitude: -116.2760009766,
		Timezone:  "America/Los_Angeles",
	},
	"58FD": {
		ICAO:      "58FD",
		Name:      "Southerly Airport",
		City:      "Dundee",
		State:     "Florida",
		Country:   "US",
		Elevation: 78,
		Latitude:  28.0142002106,
		Longitude: -81.5406036377,
		Timezone:  "America/New_York",
	},
	"58II": {
		ICAO:      "58II",
		Name:      "Henneman Airport",
		City:      "Boggstown",
		State:     "Indiana",
		Country:   "US",
		Elevation: 770,
		Latitude:  39.5209007263,
		Longitude: -85.9000015259,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"58IL": {
		ICAO:      "58IL",
		Name:      "Spangler Airport",
		City:      "Manteno",
		State:     "Illinois",
		Country:   "US",
		Elevation: 675,
		Latitude:  41.2383995056,
		Longitude: -87.8594970703,
		Timezone:  "America/Chicago",
	},
	"58IN": {
		ICAO:      "58IN",
		Name:      "Hilltop Airport",
		City:      "Spencerville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 820,
		Latitude:  41.2583999634,
		Longitude: -84.863899231,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"58KS": {
		ICAO:      "58KS",
		Name:      "Burger's Valley Airport",
		City:      "Salina",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1325,
		Latitude:  38.9221992493,
		Longitude: -97.7086029053,
		Timezone:  "America/Chicago",
	},
	"58KY": {
		ICAO:      "58KY",
		Name:      "Willow Island Airpark",
		City:      "Finchville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 740,
		Latitude:  38.1208992004,
		Longitude: -85.3319015503,
		Timezone:  "America/New_York",
	},
	"58LA": {
		ICAO:      "58LA",
		Name:      "Sydal Acres Airport",
		City:      "Wilson",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 200,
		Latitude:  30.9195995331,
		Longitude: -91.1668014526,
		Timezone:  "America/Chicago",
	},
	"58MN": {
		ICAO:      "58MN",
		Name:      "Northwest Angle Airport",
		City:      "Warroad",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1071,
		Latitude:  49.351398468,
		Longitude: -95.0710983276,
		Timezone:  "America/Chicago",
	},
	"58NC": {
		ICAO:      "58NC",
		Name:      "Fox Haven Plantation STOLport",
		City:      "Rutherfordton",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 800,
		Latitude:  35.3468017578,
		Longitude: -82.0550994873,
		Timezone:  "America/New_York",
	},
	"58OH": {
		ICAO:      "58OH",
		Name:      "Rattlesnake Island Airport",
		City:      "Put In Bay",
		State:     "Ohio",
		Country:   "US",
		Elevation: 586,
		Latitude:  41.6791992188,
		Longitude: -82.8470993042,
		Timezone:  "America/New_York",
	},
	"58OR": {
		ICAO:      "58OR",
		Name:      "Umpqua Airport",
		City:      "Roseburg",
		State:     "Oregon",
		Country:   "US",
		Elevation: 700,
		Latitude:  43.3097000122,
		Longitude: -123.1269989014,
		Timezone:  "America/Los_Angeles",
	},
	"58PA": {
		ICAO:      "58PA",
		Name:      "Tallman East Airport",
		City:      "Tower City",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 842,
		Latitude:  40.5918006897,
		Longitude: -76.5143966675,
		Timezone:  "America/New_York",
	},
	"58TE": {
		ICAO:      "58TE",
		Name:      "Mc Nabb Farm Airport",
		City:      "Ropesville",
		State:     "Texas",
		Country:   "US",
		Elevation: 3315,
		Latitude:  33.40650177,
		Longitude: -102.077003479,
		Timezone:  "America/Chicago",
	},
	"58TX": {
		ICAO:      "58TX",
		Name:      "Tailspin Estates Airport",
		City:      "Weatherford",
		State:     "Texas",
		Country:   "US",
		Elevation: 830,
		Latitude:  32.6511993408,
		Longitude: -97.9336013794,
		Timezone:  "America/Chicago",
	},
	"58VA": {
		ICAO:      "58VA",
		Name:      "Walnut Hill Airport",
		City:      "Calverton",
		State:     "Virginia",
		Country:   "US",
		Elevation: 285,
		Latitude:  38.5951004028,
		Longitude: -77.6461029053,
		Timezone:  "America/New_York",
	},
	"58WA": {
		ICAO:      "58WA",
		Name:      "Dye Seed Ranch Inc. Airport",
		City:      "Pomeroy",
		State:     "Washington",
		Country:   "US",
		Elevation: 2500,
		Latitude:  46.5074005127,
		Longitude: -117.4499969482,
		Timezone:  "America/Los_Angeles",
	},
	"58WI": {
		ICAO:      "58WI",
		Name:      "Geo Jensen Airport",
		City:      "Frederic",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1350,
		Latitude:  45.6450996399,
		Longitude: -92.3664016724,
		Timezone:  "America/Chicago",
	},
	"58XS": {
		ICAO:      "58XS",
		Name:      "Mc Entire's Lazy V Ranch Airport",
		City:      "Sterling City",
		State:     "Texas",
		Country:   "US",
		Elevation: 2360,
		Latitude:  31.8306999207,
		Longitude: -101.055999756,
		Timezone:  "America/Chicago",
	},
	"59AK": {
		ICAO:      "59AK",
		Name:      "Penderosa Airport",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 520,
		Latitude:  61.63180161,
		Longitude: -149.3410034,
		Timezone:  "America/Anchorage",
	},
	"59AS": {
		ICAO:      "59AS",
		Name:      "Becker Field",
		City:      "Battle Ground",
		State:     "Washington",
		Country:   "US",
		Elevation: 1720,
		Latitude:  45.850669,
		Longitude: -122.544514,
		Timezone:  "America/Los_Angeles",
	},
	"59AZ": {
		ICAO:      "59AZ",
		Name:      "Robin STOLport",
		City:      "Chino Valley",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4772,
		Latitude:  34.7444992065,
		Longitude: -112.4449996948,
		Timezone:  "America/Phoenix",
	},
	"59CA": {
		ICAO:      "59CA",
		Name:      "Little Hands STOLport",
		City:      "San Ramon",
		State:     "California",
		Country:   "US",
		Elevation: 1008,
		Latitude:  37.8032989502,
		Longitude: -122.0289993286,
		Timezone:  "America/Los_Angeles",
	},
	"59CL": {
		ICAO:      "59CL",
		Name:      "O'Connell Brothers Airport",
		City:      "Brawley",
		State:     "California",
		Country:   "US",
		Elevation: -99,
		Latitude:  32.9625015259,
		Longitude: -115.5419998169,
		Timezone:  "America/Los_Angeles",
	},
	"59CN": {
		ICAO:      "59CN",
		Name:      "James Brothers Airport",
		City:      "Pleasant Grove",
		State:     "California",
		Country:   "US",
		Elevation: 25,
		Latitude:  38.8348999023,
		Longitude: -121.5299987793,
		Timezone:  "America/Los_Angeles",
	},
	"59CO": {
		ICAO:      "59CO",
		Name:      "Comanche Livestock Airport",
		City:      "Strasburg",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5230,
		Latitude:  39.8333015442,
		Longitude: -104.3170013428,
		Timezone:  "America/Denver",
	},
	"59FD": {
		ICAO:      "59FD",
		Name:      "Big Cypress Airfield",
		City:      "Immokalee",
		State:     "Florida",
		Country:   "US",
		Elevation: 20,
		Latitude:  26.3202991486,
		Longitude: -80.9878005981,
		Timezone:  "America/New_York",
	},
	"59II": {
		ICAO:      "59II",
		Name:      "Posey Patch Ultralightport",
		City:      "Mount Vernon",
		State:     "Indiana",
		Country:   "US",
		Elevation: 370,
		Latitude:  37.9039001465,
		Longitude: -87.7902984619,
		Timezone:  "America/Chicago",
	},
	"59IL": {
		ICAO:      "59IL",
		Name:      "Ruder Airport",
		City:      "Maple Park",
		State:     "Illinois",
		Country:   "US",
		Elevation: 855,
		Latitude:  41.9258003235,
		Longitude: -88.6464004517,
		Timezone:  "America/Chicago",
	},
	"59IN": {
		ICAO:      "59IN",
		Name:      "Stuntz & Hochstetler Pines Airport",
		City:      "Walkerton",
		State:     "Indiana",
		Country:   "US",
		Elevation: 739,
		Latitude:  41.4737014771,
		Longitude: -86.4507980347,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"59KS": {
		ICAO:      "59KS",
		Name:      "Dunn Field",
		City:      "Beagle",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1060,
		Latitude:  38.4084014893,
		Longitude: -95.0190963745,
		Timezone:  "America/Chicago",
	},
	"59KY": {
		ICAO:      "59KY",
		Name:      "Samuels Springs Airport",
		City:      "Deatsville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 690,
		Latitude:  37.8978004456,
		Longitude: -85.5551986694,
		Timezone:  "America/New_York",
	},
	"59MN": {
		ICAO:      "59MN",
		Name:      "Cox-Coyour Meml Air Field",
		City:      "Roosevelt",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1304,
		Latitude:  48.5993995667,
		Longitude: -95.1868972778,
		Timezone:  "America/Chicago",
	},
	"59NC": {
		ICAO:      "59NC",
		Name:      "Mclean Brothers Airport",
		City:      "Gibsonville",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 696,
		Latitude:  36.0974006653,
		Longitude: -79.5552978516,
		Timezone:  "America/New_York",
	},
	"59NE": {
		ICAO:      "59NE",
		Name:      "Nelson Airport",
		City:      "Boelus",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1982,
		Latitude:  41.0811004639,
		Longitude: -98.6295013428,
		Timezone:  "America/Chicago",
	},
	"59NM": {
		ICAO:      "59NM",
		Name:      "Lockmiller & Sons Airport",
		City:      "Clovis",
		State:     "New Mexico",
		Country:   "US",
		Elevation: 4559,
		Latitude:  34.5472984314,
		Longitude: -103.4639968872,
		Timezone:  "America/Denver",
	},
	"59NY": {
		ICAO:      "59NY",
		Name:      "Bent-Wing Airport",
		City:      "Lockport",
		State:     "New York",
		Country:   "US",
		Elevation: 370,
		Latitude:  43.2305984497,
		Longitude: -78.6900024414,
		Timezone:  "America/New_York",
	},
	"59OH": {
		ICAO:      "59OH",
		Name:      "Kelch Airport",
		City:      "Bethel",
		State:     "Ohio",
		Country:   "US",
		Elevation: 905,
		Latitude:  38.9752998352,
		Longitude: -84.0342025757,
		Timezone:  "America/New_York",
	},
	"59PA": {
		ICAO:      "59PA",
		Name:      "Erkes Airport",
		City:      "Upper Black Eddy",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 340,
		Latitude:  40.5376014709,
		Longitude: -75.087097168,
		Timezone:  "America/New_York",
	},
	"59PN": {
		ICAO:      "59PN",
		Name:      "Brookside Farms Airport",
		City:      "Hershey",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 489,
		Latitude:  40.3158988953,
		Longitude: -76.685798645,
		Timezone:  "America/New_York",
	},
	"59TE": {
		ICAO:      "59TE",
		Name:      "Hoffpauir Airport",
		City:      "Katy",
		State:     "Texas",
		Country:   "US",
		Elevation: 133,
		Latitude:  29.8057994843,
		Longitude: -95.7529983521,
		Timezone:  "America/Chicago",
	},
	"59TS": {
		ICAO:      "59TS",
		Name:      "Rossler Ranch Airport",
		City:      "George West",
		State:     "Texas",
		Country:   "US",
		Elevation: 250,
		Latitude:  28.1431007385,
		Longitude: -98.1583023071,
		Timezone:  "America/Chicago",
	},
	"59TX": {
		ICAO:      "59TX",
		Name:      "Benjamin Franklin Airport",
		City:      "Godley",
		State:     "Texas",
		Country:   "US",
		Elevation: 990,
		Latitude:  32.4872016907,
		Longitude: -97.4903030396,
		Timezone:  "America/Chicago",
	},
	"59WA": {
		ICAO:      "59WA",
		Name:      "Sorrell Airport",
		City:      "Tenino",
		State:     "Washington",
		Country:   "US",
		Elevation: 250,
		Latitude:  46.8503990173,
		Longitude: -122.9359970093,
		Timezone:  "America/Los_Angeles",
	},
	"59WI": {
		ICAO:      "59WI",
		Name:      "Little Wheel Field",
		City:      "Cottage Grove",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 900,
		Latitude:  43.0339012146,
		Longitude: -89.1925964355,
		Timezone:  "America/Chicago",
	},
	"5AK0": {
		ICAO:      "5AK0",
		Name:      "Trading Bay Production Airport",
		City:      "Trading Bay",
		State:     "Alaska",
		Country:   "US",
		Elevation: 200,
		Latitude:  60.8154983521,
		Longitude: -151.7989959717,
		Timezone:  "America/Anchorage",
	},
	"5AK2": {
		ICAO:      "5AK2",
		Name:      "Howards Airport",
		City:      "North Pole",
		State:     "Alaska",
		Country:   "US",
		Elevation: 490,
		Latitude:  64.7306976318,
		Longitude: -147.3480072021,
		Timezone:  "America/Anchorage",
	},
	"5AK3": {
		ICAO:      "5AK3",
		Name:      "Airway Airport",
		City:      "North Pole",
		State:     "Alaska",
		Country:   "US",
		Elevation: 480,
		Latitude:  64.7732009888,
		Longitude: -147.333999634,
		Timezone:  "America/Anchorage",
	},
	"5AK5": {
		ICAO:      "5AK5",
		Name:      "Todds Strip",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 400,
		Latitude:  61.6571006775,
		Longitude: -149.4620056152,
		Timezone:  "America/Anchorage",
	},
	"5AK6": {
		ICAO:      "5AK6",
		Name:      "Colberg Airport",
		City:      "Palmer",
		State:     "Alaska",
		Country:   "US",
		Elevation: 125,
		Latitude:  61.5569992065,
		Longitude: -149.2680053711,
		Timezone:  "America/Anchorage",
	},
	"5AK8": {
		ICAO:      "5AK8",
		Name:      "Memory Lake Airport",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 420,
		Latitude:  61.631401062,
		Longitude: -149.4320068359,
		Timezone:  "America/Anchorage",
	},
	"5AK9": {
		ICAO:      "5AK9",
		Name:      "Grandview Subdivision Airport",
		City:      "Palmer",
		State:     "Alaska",
		Country:   "US",
		Elevation: 150,
		Latitude:  61.548500061,
		Longitude: -149.1900024414,
		Timezone:  "America/Anchorage",
	},
	"5AL1": {
		ICAO:      "5AL1",
		Name:      "Mc Gowin Field",
		City:      "Chapman",
		State:     "Alabama",
		Country:   "US",
		Elevation: 400,
		Latitude:  31.6807003021,
		Longitude: -86.6819000244,
		Timezone:  "America/Chicago",
	},
	"5AL3": {
		ICAO:      "5AL3",
		Name:      "Flying X Ranch Airport",
		City:      "Montevallo",
		State:     "Alabama",
		Country:   "US",
		Elevation: 466,
		Latitude:  33.1529006958,
		Longitude: -86.8514022827,
		Timezone:  "America/Chicago",
	},
	"5AL4": {
		ICAO:      "5AL4",
		Name:      "Blueberry Hill Airport",
		City:      "Dothan",
		State:     "Alabama",
		Country:   "US",
		Elevation: 220,
		Latitude:  31.1215991974,
		Longitude: -85.3654022217,
		Timezone:  "America/Chicago",
	},
	"5AL5": {
		ICAO:      "5AL5",
		Name:      "Big River Airpark",
		City:      "Muscle Shoals",
		State:     "Alabama",
		Country:   "US",
		Elevation: 505,
		Latitude:  34.6911010742,
		Longitude: -87.6528015137,
		Timezone:  "America/Chicago",
	},
	"5AL7": {
		ICAO:      "5AL7",
		Name:      "Hataway Field",
		City:      "Kinston",
		State:     "Alabama",
		Country:   "US",
		Elevation: 300,
		Latitude:  31.25,
		Longitude: -86.1667022705,
		Timezone:  "America/Chicago",
	},
	"5AL8": {
		ICAO:      "5AL8",
		Name:      "Fairlane Airport",
		City:      "Samson",
		State:     "Alabama",
		Country:   "US",
		Elevation: 225,
		Latitude:  31.1049995422,
		Longitude: -85.9931030273,
		Timezone:  "America/Chicago",
	},
	"5AR1": {
		ICAO:      "5AR1",
		Name:      "Tillar Airport",
		City:      "Tillar",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 150,
		Latitude:  33.712600708,
		Longitude: -91.4611968994,
		Timezone:  "America/Chicago",
	},
	"5AR2": {
		ICAO:      "5AR2",
		Name:      "Turrell Flying Service Airport",
		City:      "Turrell",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 226,
		Latitude:  35.375,
		Longitude: -90.2063980103,
		Timezone:  "America/Chicago",
	},
	"5AR3": {
		ICAO:      "5AR3",
		Name:      "Wabbaseka Flying Service Airport",
		City:      "Wabbaseka",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 200,
		Latitude:  34.3501014709,
		Longitude: -91.7835006714,
		Timezone:  "America/Chicago",
	},
	"5AR7": {
		ICAO:      "5AR7",
		Name:      "Keller Airfield",
		City:      "Weiner",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 249,
		Latitude:  35.6866989136,
		Longitude: -90.7936019897,
		Timezone:  "America/Chicago",
	},
	"5AR8": {
		ICAO:      "5AR8",
		Name:      "Pine Prairie Airport",
		City:      "Ashdown",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 321,
		Latitude:  33.6167984009,
		Longitude: -94.116897583,
		Timezone:  "America/Chicago",
	},
	"5AZ3": {
		ICAO:      "5AZ3",
		Name:      "Pegasus Airpark",
		City:      "Queen Creek",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1450,
		Latitude:  33.2125015259,
		Longitude: -111.6169967651,
		Timezone:  "America/Phoenix",
	},
	"5AZ6": {
		ICAO:      "5AZ6",
		Name:      "Motown Airport",
		City:      "Mobile",
		State:     "Arizona",
		Country:   "US",
		Elevation: 80,
		Latitude:  32.925,
		Longitude: -112.259444,
		Timezone:  "America/Phoenix",
	},
	"5AZ7": {
		ICAO:      "5AZ7",
		Name:      "Carranza Farm Airstrip",
		City:      "Stanfield",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1300,
		Latitude:  32.8471984863,
		Longitude: -111.9499969482,
		Timezone:  "America/Phoenix",
	},
	"5AZ9": {
		ICAO:      "5AZ9",
		Name:      "Regeneration Airport",
		City:      "Fort Thomas",
		State:     "Arizona",
		Country:   "US",
		Elevation: 2750,
		Latitude:  33.0327987671,
		Longitude: -109.9879989624,
		Timezone:  "America/Phoenix",
	},
	"5CA4": {
		ICAO:      "5CA4",
		Name:      "Ludlow Airport",
		City:      "Barstow",
		State:     "California",
		Country:   "US",
		Elevation: 1700,
		Latitude:  34.7532997131,
		Longitude: -116.1539993286,
		Timezone:  "America/Los_Angeles",
	},
	"5CA7": {
		ICAO:      "5CA7",
		Name:      "Agro-West Airport",
		City:      "Five Points",
		State:     "California",
		Country:   "US",
		Elevation: 335,
		Latitude:  36.336101532,
		Longitude: -120.247001648,
		Timezone:  "America/Los_Angeles",
	},
	"5CA8": {
		ICAO:      "5CA8",
		Name:      "Howard /Private/ Airport",
		City:      "Ione",
		State:     "California",
		Country:   "US",
		Elevation: 480,
		Latitude:  38.2563018799,
		Longitude: -120.931999207,
		Timezone:  "America/Los_Angeles",
	},
	"5CL0": {
		ICAO:      "5CL0",
		Name:      "Grupe Ranch Airport",
		City:      "Mariposa",
		State:     "California",
		Country:   "US",
		Elevation: 1625,
		Latitude:  37.3404998779,
		Longitude: -119.8619995117,
		Timezone:  "America/Los_Angeles",
	},
	"5CL1": {
		ICAO:      "5CL1",
		Name:      "Hart Ranch Airport",
		City:      "Santa Margarita",
		State:     "California",
		Country:   "US",
		Elevation: 1380,
		Latitude:  35.40470123,
		Longitude: -120.5250015,
		Timezone:  "America/Los_Angeles",
	},
	"5CL3": {
		ICAO:      "5CL3",
		Name:      "Mapes Ranch Airport",
		City:      "Modesto",
		State:     "California",
		Country:   "US",
		Elevation: 34,
		Latitude:  37.6679992676,
		Longitude: -121.1989974976,
		Timezone:  "America/Los_Angeles",
	},
	"5CL7": {
		ICAO:      "5CL7",
		Name:      "Gene Wash Reservoir Airport",
		City:      "Parker Dam",
		State:     "California",
		Country:   "US",
		Elevation: 860,
		Latitude:  34.3081016541,
		Longitude: -114.18699646,
		Timezone:  "America/Los_Angeles",
	},
	"5CL8": {
		ICAO:      "5CL8",
		Name:      "Creekside Airport",
		City:      "Arvin",
		State:     "California",
		Country:   "US",
		Elevation: 335,
		Latitude:  35.1761016846,
		Longitude: -118.922996521,
		Timezone:  "America/Los_Angeles",
	},
	"5CL9": {
		ICAO:      "5CL9",
		Name:      "Johnston Field",
		City:      "Auberry",
		State:     "California",
		Country:   "US",
		Elevation: 1280,
		Latitude:  37.1413002014,
		Longitude: -119.5080032349,
		Timezone:  "America/Los_Angeles",
	},
	"5CO0": {
		ICAO:      "5CO0",
		Name:      "Cheyenne Wells Municipal Airport",
		City:      "Cheyenne Wells",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4184,
		Latitude:  38.8166999817,
		Longitude: -102.3059997559,
		Timezone:  "America/Denver",
	},
	"5CO1": {
		ICAO:      "5CO1",
		Name:      "Shipman Ranch Airport",
		City:      "Austin",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6200,
		Latitude:  38.8333015442,
		Longitude: -107.8590011597,
		Timezone:  "America/Denver",
	},
	"5CO4": {
		ICAO:      "5CO4",
		Name:      "Spickard Farm Airport",
		City:      "Byers",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5320,
		Latitude:  39.7000007629,
		Longitude: -104.2509994507,
		Timezone:  "America/Denver",
	},
	"5CO6": {
		ICAO:      "5CO6",
		Name:      "Meyer Ranch Airport",
		City:      "Conifer",
		State:     "Colorado",
		Country:   "US",
		Elevation: 8000,
		Latitude:  39.5499992371,
		Longitude: -105.283996582,
		Timezone:  "America/Denver",
	},
	"5CO7": {
		ICAO:      "5CO7",
		Name:      "Mesa View Ranch Airport",
		City:      "Craig",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6978,
		Latitude:  40.7750015259,
		Longitude: -107.533996582,
		Timezone:  "America/Denver",
	},
	"5CO8": {
		ICAO:      "5CO8",
		Name:      "Pleasant Valley Airport",
		City:      "Oak Creek",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7120,
		Latitude:  40.3414001465,
		Longitude: -106.8489990234,
		Timezone:  "America/Denver",
	},
	"5CT0": {
		ICAO:      "5CT0",
		Name:      "Eastford Airport",
		City:      "Eastford",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 600,
		Latitude:  41.9104003906,
		Longitude: -72.0680999756,
		Timezone:  "America/New_York",
	},
	"5CT5": {
		ICAO:      "5CT5",
		Name:      "Thomson Field",
		City:      "Bethlehem",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 820,
		Latitude:  41.6323013306,
		Longitude: -73.2162017822,
		Timezone:  "America/New_York",
	},
	"5CT6": {
		ICAO:      "5CT6",
		Name:      "Buell Farm Ultralightport",
		City:      "Eastford",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 670,
		Latitude:  41.9194984436,
		Longitude: -72.1184005737,
		Timezone:  "America/New_York",
	},
	"5CT7": {
		ICAO:      "5CT7",
		Name:      "Mile Creek Airport",
		City:      "Old Lyme",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 30,
		Latitude:  41.3022994995,
		Longitude: -72.2962036133,
		Timezone:  "America/New_York",
	},
	"5FA1": {
		ICAO:      "5FA1",
		Name:      "Flying Palomino Ranch Airport",
		City:      "Fruitland Park",
		State:     "Florida",
		Country:   "US",
		Elevation: 85,
		Latitude:  28.8780002594,
		Longitude: -81.94090271,
		Timezone:  "America/New_York",
	},
	"5FD0": {
		ICAO:      "5FD0",
		Name:      "Manatee Springs Airport",
		City:      "Old Town",
		State:     "Florida",
		Country:   "US",
		Elevation: 17,
		Latitude:  29.4927005768,
		Longitude: -82.9912033081,
		Timezone:  "America/New_York",
	},
	"5FD1": {
		ICAO:      "5FD1",
		Name:      "Ron Wood Airport",
		City:      "Clarksville",
		State:     "Florida",
		Country:   "US",
		Elevation: 140,
		Latitude:  30.432100296,
		Longitude: -85.2888031006,
		Timezone:  "America/Chicago",
	},
	"5FD3": {
		ICAO:      "5FD3",
		Name:      "County Line Airstrip",
		City:      "Altha",
		State:     "Florida",
		Country:   "US",
		Elevation: 150,
		Latitude:  30.608499527,
		Longitude: -85.0913009644,
		Timezone:  "America/Chicago",
	},
	"5FD5": {
		ICAO:      "5FD5",
		Name:      "Able Airpark",
		City:      "Blountstown",
		State:     "Florida",
		Country:   "US",
		Elevation: 100,
		Latitude:  30.595500946,
		Longitude: -84.981300354,
		Timezone:  "America/Chicago",
	},
	"5FD7": {
		ICAO:      "5FD7",
		Name:      "Chinsegut Airport",
		City:      "Brooksville",
		State:     "Florida",
		Country:   "US",
		Elevation: 80,
		Latitude:  28.611700058,
		Longitude: -82.3678970337,
		Timezone:  "America/New_York",
	},
	"5FL0": {
		ICAO:      "5FL0",
		Name:      "Dusty Airpatch Airport",
		City:      "Dade City",
		State:     "Florida",
		Country:   "US",
		Elevation: 120,
		Latitude:  28.4130992889,
		Longitude: -82.2233963013,
		Timezone:  "America/New_York",
	},
	"5FL1": {
		ICAO:      "5FL1",
		Name:      "Ezell Airport",
		City:      "Keaton Beach",
		State:     "Florida",
		Country:   "US",
		Elevation: 25,
		Latitude:  29.9032993317,
		Longitude: -83.6079025269,
		Timezone:  "America/New_York",
	},
	"5FL4": {
		ICAO:      "5FL4",
		Name:      "Byrd Air Field",
		City:      "Elkton",
		State:     "Florida",
		Country:   "US",
		Elevation: 21,
		Latitude:  29.8008003235,
		Longitude: -81.5009002686,
		Timezone:  "America/New_York",
	},
	"5FL6": {
		ICAO:      "5FL6",
		Name:      "Flat Creek Airport",
		City:      "Greensboro",
		State:     "Florida",
		Country:   "US",
		Elevation: 295,
		Latitude:  30.6044006348,
		Longitude: -84.7907028198,
		Timezone:  "America/New_York",
	},
	"5FL7": {
		ICAO:      "5FL7",
		Name:      "Twelve Oaks Airport",
		City:      "Hernando",
		State:     "Florida",
		Country:   "US",
		Elevation: 50,
		Latitude:  28.9780006409,
		Longitude: -82.3637008667,
		Timezone:  "America/New_York",
	},
	"5FL8": {
		ICAO:      "5FL8",
		Name:      "Chipola Airpark",
		City:      "Greenwood",
		State:     "Florida",
		Country:   "US",
		Elevation: 125,
		Latitude:  30.8882007599,
		Longitude: -85.1677017212,
		Timezone:  "America/Chicago",
	},
	"5FL9": {
		ICAO:      "5FL9",
		Name:      "Cedar Lane Airport",
		City:      "Defuniak Springs",
		State:     "Florida",
		Country:   "US",
		Elevation: 270,
		Latitude:  30.7362995148,
		Longitude: -86.2129974365,
		Timezone:  "America/Chicago",
	},
	"5GA0": {
		ICAO:      "5GA0",
		Name:      "Gable Branch Airport",
		City:      "Haralson",
		State:     "Georgia",
		Country:   "US",
		Elevation: 840,
		Latitude:  33.2276000977,
		Longitude: -84.5444030762,
		Timezone:  "America/New_York",
	},
	"5GA2": {
		ICAO:      "5GA2",
		Name:      "Lyons Landing Airport",
		City:      "Whitesburg",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1080,
		Latitude:  33.5732002258,
		Longitude: -84.9096984863,
		Timezone:  "America/New_York",
	},
	"5GA3": {
		ICAO:      "5GA3",
		Name:      "Eagles Landing Airport",
		City:      "Williamson",
		State:     "Georgia",
		Country:   "US",
		Elevation: 980,
		Latitude:  33.1600990295,
		Longitude: -84.365196228,
		Timezone:  "America/New_York",
	},
	"5GA4": {
		ICAO:      "5GA4",
		Name:      "Air Acres Airport",
		City:      "Woodstock",
		State:     "Georgia",
		Country:   "US",
		Elevation: 918,
		Latitude:  34.1058998108,
		Longitude: -84.4485015869,
		Timezone:  "America/New_York",
	},
	"5GA5": {
		ICAO:      "5GA5",
		Name:      "Pinebrook Estates Airport",
		City:      "Griffin",
		State:     "Georgia",
		Country:   "US",
		Elevation: 820,
		Latitude:  33.2597999573,
		Longitude: -84.3494033813,
		Timezone:  "America/New_York",
	},
	"5GA6": {
		ICAO:      "5GA6",
		Name:      "Roney Farms Airport",
		City:      "Lilly",
		State:     "Georgia",
		Country:   "US",
		Elevation: 352,
		Latitude:  32.1498985291,
		Longitude: -83.8612976074,
		Timezone:  "America/New_York",
	},
	"5GA7": {
		ICAO:      "5GA7",
		Name:      "Panacea Airport",
		City:      "Franklin/Glenn",
		State:     "Georgia",
		Country:   "US",
		Elevation: 800,
		Latitude:  33.2018013,
		Longitude: -85.2304992676,
		Timezone:  "America/Chicago",
	},
	"5GE1": {
		ICAO:      "5GE1",
		Name:      "Peachtree Landings Airport",
		City:      "Fort Valley",
		State:     "Georgia",
		Country:   "US",
		Elevation: 447,
		Latitude:  32.5703010559,
		Longitude: -83.7219009399,
		Timezone:  "America/New_York",
	},
	"5IA3": {
		ICAO:      "5IA3",
		Name:      "Watkins Private Airport",
		City:      "Humeston",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1095,
		Latitude:  40.8438987732,
		Longitude: -93.5427017212,
		Timezone:  "America/Chicago",
	},
	"5IA5": {
		ICAO:      "5IA5",
		Name:      "Folkerts Airport",
		City:      "Rudd",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1150,
		Latitude:  43.1430015564,
		Longitude: -92.8779983521,
		Timezone:  "America/Chicago",
	},
	"5IA6": {
		ICAO:      "5IA6",
		Name:      "Flying S Ranch Airport",
		City:      "Renwick",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1145,
		Latitude:  42.8305015564,
		Longitude: -94.0115966797,
		Timezone:  "America/Chicago",
	},
	"5IA7": {
		ICAO:      "5IA7",
		Name:      "Murphy Field Private Airport",
		City:      "Wellman",
		State:     "Iowa",
		Country:   "US",
		Elevation: 803,
		Latitude:  41.4216995239,
		Longitude: -91.8331985474,
		Timezone:  "America/Chicago",
	},
	"5II1": {
		ICAO:      "5II1",
		Name:      "Layne Field",
		City:      "New Winchester",
		State:     "Indiana",
		Country:   "US",
		Elevation: 905,
		Latitude:  39.7827987671,
		Longitude: -86.6175003052,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"5II2": {
		ICAO:      "5II2",
		Name:      "Miles Field",
		City:      "New Whiteland",
		State:     "Indiana",
		Country:   "US",
		Elevation: 795,
		Latitude:  39.5597991943,
		Longitude: -86.1350021362,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"5II5": {
		ICAO:      "5II5",
		Name:      "Koester Field",
		City:      "St. Wendel",
		State:     "Indiana",
		Country:   "US",
		Elevation: 480,
		Latitude:  38.1180144538,
		Longitude: -87.7020978928,
		Timezone:  "America/Chicago",
	},
	"5II8": {
		ICAO:      "5II8",
		Name:      "Jacobi Airport",
		City:      "Palmyra",
		State:     "Indiana",
		Country:   "US",
		Elevation: 750,
		Latitude:  38.4100990295,
		Longitude: -86.1288986206,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"5II9": {
		ICAO:      "5II9",
		Name:      "Aero Plaines Airport",
		City:      "Youngstown",
		State:     "Indiana",
		Country:   "US",
		Elevation: 570,
		Latitude:  39.3527984619,
		Longitude: -87.3733978271,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"5IL2": {
		ICAO:      "5IL2",
		Name:      "Panther Field",
		City:      "Pinckneyville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 432,
		Latitude:  38.0638999939,
		Longitude: -89.387802124,
		Timezone:  "America/Chicago",
	},
	"5IN3": {
		ICAO:      "5IN3",
		Name:      "Tragesser Airport",
		City:      "Tipton",
		State:     "Indiana",
		Country:   "US",
		Elevation: 875,
		Latitude:  40.3003005981,
		Longitude: -86.0503005981,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"5IN4": {
		ICAO:      "5IN4",
		Name:      "Wyckoff Airstrip",
		City:      "Valparaiso",
		State:     "Indiana",
		Country:   "US",
		Elevation: 780,
		Latitude:  41.4833984375,
		Longitude: -86.9499969482,
		Timezone:  "America/Chicago",
	},
	"5IN5": {
		ICAO:      "5IN5",
		Name:      "Berkey Field",
		City:      "Warsaw",
		State:     "Indiana",
		Country:   "US",
		Elevation: 858,
		Latitude:  41.2877998352,
		Longitude: -85.9005966187,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"5IN6": {
		ICAO:      "5IN6",
		Name:      "Orthodontic Strip",
		City:      "Westville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 805,
		Latitude:  41.5875015259,
		Longitude: -86.9028015137,
		Timezone:  "America/Chicago",
	},
	"5IN9": {
		ICAO:      "5IN9",
		Name:      "Stangland Airport",
		City:      "Wolflake",
		State:     "Indiana",
		Country:   "US",
		Elevation: 924,
		Latitude:  41.3162002563,
		Longitude: -85.4649963379,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"5IS3": {
		ICAO:      "5IS3",
		Name:      "Riverveiw Airport",
		City:      "Evansville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 490,
		Latitude:  38.0783996582,
		Longitude: -89.979598999,
		Timezone:  "America/Chicago",
	},
	"5IS4": {
		ICAO:      "5IS4",
		Name:      "Corman Acres Airport",
		City:      "Lake City",
		State:     "Illinois",
		Country:   "US",
		Elevation: 710,
		Latitude:  39.7661018372,
		Longitude: -88.7795028687,
		Timezone:  "America/Chicago",
	},
	"5IS7": {
		ICAO:      "5IS7",
		Name:      "Cast Airport",
		City:      "Fairmount",
		State:     "Illinois",
		Country:   "US",
		Elevation: 670,
		Latitude:  40.038898468,
		Longitude: -87.7817001343,
		Timezone:  "America/Chicago",
	},
	"5IS9": {
		ICAO:      "5IS9",
		Name:      "Mc Pherson Airport",
		City:      "Carthage",
		State:     "Illinois",
		Country:   "US",
		Elevation: 640,
		Latitude:  40.3306007385,
		Longitude: -91.0501022339,
		Timezone:  "America/Chicago",
	},
	"5KS1": {
		ICAO:      "5KS1",
		Name:      "Threshing Bee Airport",
		City:      "Mc Louth",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1150,
		Latitude:  39.1938018799,
		Longitude: -95.2163009644,
		Timezone:  "America/Chicago",
	},
	"5KS2": {
		ICAO:      "5KS2",
		Name:      "Fortmeyer Airport",
		City:      "Goodland",
		State:     "Kansas",
		Country:   "US",
		Elevation: 3650,
		Latitude:  39.3417015076,
		Longitude: -101.6660003662,
		Timezone:  "America/Denver",
	},
	"5KS3": {
		ICAO:      "5KS3",
		Name:      "Deweze Airport",
		City:      "Harper",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1414,
		Latitude:  37.279499054,
		Longitude: -98.0180969238,
		Timezone:  "America/Chicago",
	},
	"5KS4": {
		ICAO:      "5KS4",
		Name:      "Hall Farms Airport",
		City:      "Kanorado",
		State:     "Kansas",
		Country:   "US",
		Elevation: 3908,
		Latitude:  39.2299995422,
		Longitude: -101.983001709,
		Timezone:  "America/Denver",
	},
	"5KS6": {
		ICAO:      "5KS6",
		Name:      "Mollenkamp Field",
		City:      "Leoti",
		State:     "Kansas",
		Country:   "US",
		Elevation: 3295,
		Latitude:  38.7667007446,
		Longitude: -101.2570037842,
		Timezone:  "America/Chicago",
	},
	"5KS8": {
		ICAO:      "5KS8",
		Name:      "Jack Poore Airport",
		City:      "Mc Donald",
		State:     "Kansas",
		Country:   "US",
		Elevation: 3330,
		Latitude:  39.8180999756,
		Longitude: -101.3479995728,
		Timezone:  "America/Chicago",
	},
	"5KS9": {
		ICAO:      "5KS9",
		Name:      "Black Airport",
		City:      "Mc Donald",
		State:     "Kansas",
		Country:   "US",
		Elevation: 3345,
		Latitude:  39.8258018494,
		Longitude: -101.3629989624,
		Timezone:  "America/Chicago",
	},
	"5KY2": {
		ICAO:      "5KY2",
		Name:      "Larkins Farm Airport",
		City:      "Bardwell",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 390,
		Latitude:  36.8511009216,
		Longitude: -88.926902771,
		Timezone:  "America/Chicago",
	},
	"5KY3": {
		ICAO:      "5KY3",
		Name:      "West Kentucky Airpark",
		City:      "Paducah",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 338,
		Latitude:  36.9674987793,
		Longitude: -88.5652999878,
		Timezone:  "America/Chicago",
	},
	"5KY5": {
		ICAO:      "5KY5",
		Name:      "Lowe Airport",
		City:      "Cadiz",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 498,
		Latitude:  36.8624992371,
		Longitude: -87.7902984619,
		Timezone:  "America/Chicago",
	},
	"5KY6": {
		ICAO:      "5KY6",
		Name:      "Jake'S Field",
		City:      "Smithfield",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 903,
		Latitude:  38.420556,
		Longitude: -85.257778,
		Timezone:  "America/New_York",
	},
	"5KY7": {
		ICAO:      "5KY7",
		Name:      "Dale Hollow Regional (Petro Field) Airport",
		City:      "Albany",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 980,
		Latitude:  36.6377983093,
		Longitude: -85.1638031006,
		Timezone:  "America/Chicago",
	},
	"5KY8": {
		ICAO:      "5KY8",
		Name:      "Harold Reynolds Airport",
		City:      "Harrodsburg",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 860,
		Latitude:  37.889722,
		Longitude: -84.927778,
		Timezone:  "America/New_York",
	},
	"5KY9": {
		ICAO:      "5KY9",
		Name:      "Nolin Aero Salvage Airport",
		City:      "Bee Spring",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 700,
		Latitude:  37.3325004578,
		Longitude: -86.2542037964,
		Timezone:  "America/Chicago",
	},
	"5LA0": {
		ICAO:      "5LA0",
		Name:      "Juneau Ultralightport",
		City:      "Shreveport",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 200,
		Latitude:  32.5918006897,
		Longitude: -93.8280029297,
		Timezone:  "America/Chicago",
	},
	"5LA1": {
		ICAO:      "5LA1",
		Name:      "Kent's Flying Service Airport",
		City:      "Moreauville",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 55,
		Latitude:  31.0279998779,
		Longitude: -91.9812011719,
		Timezone:  "America/Chicago",
	},
	"5LL0": {
		ICAO:      "5LL0",
		Name:      "Beckerman Field",
		City:      "Mount Carmel",
		State:     "Illinois",
		Country:   "US",
		Elevation: 424,
		Latitude:  38.4084014893,
		Longitude: -87.8375015259,
		Timezone:  "America/Chicago",
	},
	"5LL1": {
		ICAO:      "5LL1",
		Name:      "Hilvety Airport",
		City:      "Moweaqua",
		State:     "Illinois",
		Country:   "US",
		Elevation: 674,
		Latitude:  39.6500015259,
		Longitude: -88.9750976563,
		Timezone:  "America/Chicago",
	},
	"5LL2": {
		ICAO:      "5LL2",
		Name:      "Paul E. Kroenlein Airport",
		City:      "Moweaqua",
		State:     "Illinois",
		Country:   "US",
		Elevation: 710,
		Latitude:  39.6083984375,
		Longitude: -88.8583984375,
		Timezone:  "America/Chicago",
	},
	"5LL3": {
		ICAO:      "5LL3",
		Name:      "Mayhall Airport",
		City:      "Murdock",
		State:     "Illinois",
		Country:   "US",
		Elevation: 643,
		Latitude:  39.7695007324,
		Longitude: -88.0952987671,
		Timezone:  "America/Chicago",
	},
	"5LL5": {
		ICAO:      "5LL5",
		Name:      "Gordon Brown Airport",
		City:      "Trivoli",
		State:     "Illinois",
		Country:   "US",
		Elevation: 745,
		Latitude:  40.7014007568,
		Longitude: -89.8840026855,
		Timezone:  "America/Chicago",
	},
	"5LL7": {
		ICAO:      "5LL7",
		Name:      "Hilbert's Funny Farm Airport",
		City:      "Union",
		State:     "Illinois",
		Country:   "US",
		Elevation: 850,
		Latitude:  42.2106018066,
		Longitude: -88.5150985718,
		Timezone:  "America/Chicago",
	},
	"5LL8": {
		ICAO:      "5LL8",
		Name:      "Hugh Van Voorst Airport",
		City:      "Union Hill",
		State:     "Illinois",
		Country:   "US",
		Elevation: 617,
		Latitude:  41.1123008728,
		Longitude: -88.1400985718,
		Timezone:  "America/Chicago",
	},
	"5LS6": {
		ICAO:      "5LS6",
		Name:      "Arkla Flyers Inc Airport",
		City:      "Mer Rouge",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 90,
		Latitude:  32.7650985718,
		Longitude: -91.7962036133,
		Timezone:  "America/Chicago",
	},
	"5LS9": {
		ICAO:      "5LS9",
		Name:      "Ammons Airport",
		City:      "Zwolle",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 210,
		Latitude:  31.5316009521,
		Longitude: -93.7077026367,
		Timezone:  "America/Chicago",
	},
	"5MA3": {
		ICAO:      "5MA3",
		Name:      "Pasport Airport",
		City:      "West Brookfield",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 1010,
		Latitude:  42.1875991821,
		Longitude: -72.1430969238,
		Timezone:  "America/New_York",
	},
	"5MA9": {
		ICAO:      "5MA9",
		Name:      "Allen's Pond Airport",
		City:      "Westport",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 60,
		Latitude:  41.5111999512,
		Longitude: -71.0280990601,
		Timezone:  "America/New_York",
	},
	"5MD0": {
		ICAO:      "5MD0",
		Name:      "Marble Head Farm Airport",
		City:      "Ridgely",
		State:     "Maryland",
		Country:   "US",
		Elevation: 70,
		Latitude:  38.9893989563,
		Longitude: -75.8666992188,
		Timezone:  "America/New_York",
	},
	"5MD2": {
		ICAO:      "5MD2",
		Name:      "Marsh Field",
		City:      "Frostburg",
		State:     "Maryland",
		Country:   "US",
		Elevation: 2600,
		Latitude:  39.5766983032,
		Longitude: -78.9186019897,
		Timezone:  "America/New_York",
	},
	"5MD5": {
		ICAO:      "5MD5",
		Name:      "Flying W Airport",
		City:      "Princess Anne",
		State:     "Maryland",
		Country:   "US",
		Elevation: 18,
		Latitude:  38.1960983276,
		Longitude: -75.7093963623,
		Timezone:  "America/New_York",
	},
	"5MD7": {
		ICAO:      "5MD7",
		Name:      "Bluemel Field",
		City:      "Princess Anne",
		State:     "Maryland",
		Country:   "US",
		Elevation: 3,
		Latitude:  38.1335983276,
		Longitude: -75.7630996704,
		Timezone:  "America/New_York",
	},
	"5MD8": {
		ICAO:      "5MD8",
		Name:      "Edelen Field",
		City:      "La Plata",
		State:     "Maryland",
		Country:   "US",
		Elevation: 151,
		Latitude:  38.4910011292,
		Longitude: -77.0115966797,
		Timezone:  "America/New_York",
	},
	"5MI1": {
		ICAO:      "5MI1",
		Name:      "Wilds Field",
		City:      "Cedar Springs",
		State:     "Michigan",
		Country:   "US",
		Elevation: 885,
		Latitude:  43.2513999939,
		Longitude: -85.5428009033,
		Timezone:  "America/Detroit",
	},
	"5MI2": {
		ICAO:      "5MI2",
		Name:      "Mc Question's Airport",
		City:      "Leroy",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1317,
		Latitude:  44.0772018433,
		Longitude: -85.4375991821,
		Timezone:  "America/Detroit",
	},
	"5MI3": {
		ICAO:      "5MI3",
		Name:      "Cridler Airport",
		City:      "Lowell",
		State:     "Michigan",
		Country:   "US",
		Elevation: 870,
		Latitude:  43.0511016846,
		Longitude: -85.4199981689,
		Timezone:  "America/Detroit",
	},
	"5MI4": {
		ICAO:      "5MI4",
		Name:      "Anderson Airport",
		City:      "Howard City",
		State:     "Michigan",
		Country:   "US",
		Elevation: 902,
		Latitude:  43.4283981323,
		Longitude: -85.4548034668,
		Timezone:  "America/Detroit",
	},
	"5MI7": {
		ICAO:      "5MI7",
		Name:      "Williams Field",
		City:      "Jackson",
		State:     "Michigan",
		Country:   "US",
		Elevation: 960,
		Latitude:  42.3610992432,
		Longitude: -84.4247970581,
		Timezone:  "America/Detroit",
	},
	"5MI8": {
		ICAO:      "5MI8",
		Name:      "Lakeside Airport",
		City:      "Holt",
		State:     "Michigan",
		Country:   "US",
		Elevation: 900,
		Latitude:  42.6716995239,
		Longitude: -84.4882965088,
		Timezone:  "America/Detroit",
	},
	"5MN0": {
		ICAO:      "5MN0",
		Name:      "Scrabeck Airport",
		City:      "High Forest",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1280,
		Latitude:  43.8666000366,
		Longitude: -92.5610961914,
		Timezone:  "America/Chicago",
	},
	"5MN1": {
		ICAO:      "5MN1",
		Name:      "Budde Airport",
		City:      "Mankato",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 975,
		Latitude:  44.2005004883,
		Longitude: -94.1132965088,
		Timezone:  "America/Chicago",
	},
	"5MN2": {
		ICAO:      "5MN2",
		Name:      "Hoiland Field",
		City:      "Bagley",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1535,
		Latitude:  47.3958015442,
		Longitude: -95.3675003052,
		Timezone:  "America/Chicago",
	},
	"5MN4": {
		ICAO:      "5MN4",
		Name:      "Seven Hills Airport",
		City:      "Clearwater",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1010,
		Latitude:  45.4124984741,
		Longitude: -94.0766983032,
		Timezone:  "America/Chicago",
	},
	"5MN5": {
		ICAO:      "5MN5",
		Name:      "Barnes Airport",
		City:      "Lindstrom",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 945,
		Latitude:  45.4364013672,
		Longitude: -92.8348999023,
		Timezone:  "America/Chicago",
	},
	"5MN7": {
		ICAO:      "5MN7",
		Name:      "Lindey's Landing Airport",
		City:      "Deerwood",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1295,
		Latitude:  46.3871994019,
		Longitude: -93.9319000244,
		Timezone:  "America/Chicago",
	},
	"5MN8": {
		ICAO:      "5MN8",
		Name:      "Rick Mathias Private Airport",
		City:      "Dumont",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1050,
		Latitude:  45.7193984985,
		Longitude: -96.3103027344,
		Timezone:  "America/Chicago",
	},
	"5MN9": {
		ICAO:      "5MN9",
		Name:      "Chanlin Field",
		City:      "Middle River",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1147,
		Latitude:  48.4235992432,
		Longitude: -96.1288986206,
		Timezone:  "America/Chicago",
	},
	"5MO1": {
		ICAO:      "5MO1",
		Name:      "Irons Airport",
		City:      "Moberly",
		State:     "Missouri",
		Country:   "US",
		Elevation: 860,
		Latitude:  39.3842010498,
		Longitude: -92.3762969971,
		Timezone:  "America/Chicago",
	},
	"5MO3": {
		ICAO:      "5MO3",
		Name:      "Bil-Mitch Airport",
		City:      "Monett",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1342,
		Latitude:  36.9406013489,
		Longitude: -93.9573974609,
		Timezone:  "America/Chicago",
	},
	"5MO9": {
		ICAO:      "5MO9",
		Name:      "Oak Ridge Farms Airport",
		City:      "New Boston",
		State:     "Missouri",
		Country:   "US",
		Elevation: 995,
		Latitude:  39.9824981689,
		Longitude: -92.864402771,
		Timezone:  "America/Chicago",
	},
	"5MS1": {
		ICAO:      "5MS1",
		Name:      "Rollang Field",
		City:      "Rolling Fork",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 106,
		Latitude:  32.9542999268,
		Longitude: -90.8459014893,
		Timezone:  "America/Chicago",
	},
	"5MS2": {
		ICAO:      "5MS2",
		Name:      "Turkey Bayou Airpark",
		City:      "Lakeshore",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 9,
		Latitude:  30.2507991791,
		Longitude: -89.4509963989,
		Timezone:  "America/Chicago",
	},
	"5MS3": {
		ICAO:      "5MS3",
		Name:      "Porter Airport",
		City:      "Columbia",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 158,
		Latitude:  31.1749992371,
		Longitude: -89.8031005859,
		Timezone:  "America/Chicago",
	},
	"5MS4": {
		ICAO:      "5MS4",
		Name:      "Walnut Creek Airport",
		City:      "Canton",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 315,
		Latitude:  32.5926017761,
		Longitude: -89.925201416,
		Timezone:  "America/Chicago",
	},
	"5MS5": {
		ICAO:      "5MS5",
		Name:      "Mint Julep Airpark",
		City:      "Picayune",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 100,
		Latitude:  30.5452003479,
		Longitude: -89.4627990723,
		Timezone:  "America/Chicago",
	},
	"5MS6": {
		ICAO:      "5MS6",
		Name:      "Dee's Strip",
		City:      "Brandon",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 329,
		Latitude:  32.4230003357,
		Longitude: -90.0076980591,
		Timezone:  "America/Chicago",
	},
	"5MS8": {
		ICAO:      "5MS8",
		Name:      "Ball Airport",
		City:      "Okolona",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 298,
		Latitude:  33.875,
		Longitude: -88.723333,
		Timezone:  "America/Chicago",
	},
	"5NA0": {
		ICAO:      "5NA0",
		Name:      "Semchenko Airport",
		City:      "Max",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 2065,
		Latitude:  47.773601532,
		Longitude: -101.2789993286,
		Timezone:  "America/Chicago",
	},
	"5NA2": {
		ICAO:      "5NA2",
		Name:      "Haugen Farm Airstrip",
		City:      "Portland",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1002,
		Latitude:  47.5377998352,
		Longitude: -97.3637008667,
		Timezone:  "America/Chicago",
	},
	"5NC1": {
		ICAO:      "5NC1",
		Name:      "Tara Airbase Airport",
		City:      "Mocksville",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 756,
		Latitude:  35.7932014465,
		Longitude: -80.512298584,
		Timezone:  "America/New_York",
	},
	"5NC2": {
		ICAO:      "5NC2",
		Name:      "Lathan Strip",
		City:      "Monroe",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 575,
		Latitude:  34.8763008118,
		Longitude: -80.6192016602,
		Timezone:  "America/New_York",
	},
	"5NC3": {
		ICAO:      "5NC3",
		Name:      "Gilliam-Mc Connell Airfield",
		City:      "Carthage",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 445,
		Latitude:  35.3417015076,
		Longitude: -79.4368972778,
		Timezone:  "America/New_York",
	},
	"5NC5": {
		ICAO:      "5NC5",
		Name:      "The Duchy Airport",
		City:      "Chapel Hill",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 650,
		Latitude:  35.9814987183,
		Longitude: -79.2731018066,
		Timezone:  "America/New_York",
	},
	"5ND0": {
		ICAO:      "5ND0",
		Name:      "Stiehl Airport",
		City:      "Buford",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 2277,
		Latitude:  48.0764007568,
		Longitude: -104.0319976807,
		Timezone:  "America/Chicago",
	},
	"5ND3": {
		ICAO:      "5ND3",
		Name:      "Craig Private Airport",
		City:      "Bathgate",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 800,
		Latitude:  48.8554992676,
		Longitude: -97.3219985962,
		Timezone:  "America/Chicago",
	},
	"5ND4": {
		ICAO:      "5ND4",
		Name:      "Tesch Strip",
		City:      "Lidgerwood",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1114,
		Latitude:  46.1436004639,
		Longitude: -97.2431030273,
		Timezone:  "America/Chicago",
	},
	"5ND9": {
		ICAO:      "5ND9",
		Name:      "Deck Airport",
		City:      "Hillsboro",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 916,
		Latitude:  47.4846992493,
		Longitude: -97.074798584,
		Timezone:  "America/Chicago",
	},
	"5NE1": {
		ICAO:      "5NE1",
		Name:      "Trumbull Ranch Airport",
		City:      "Ringgold",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3260,
		Latitude:  41.5499992371,
		Longitude: -100.8339996338,
		Timezone:  "America/Chicago",
	},
	"5NE2": {
		ICAO:      "5NE2",
		Name:      "Eickhoff Strip",
		City:      "Shubert",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1000,
		Latitude:  40.1958007813,
		Longitude: -95.6294021606,
		Timezone:  "America/Chicago",
	},
	"5NE3": {
		ICAO:      "5NE3",
		Name:      "Diamond Bar Jones Airport",
		City:      "Stapleton",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2900,
		Latitude:  41.4874992371,
		Longitude: -100.4879989624,
		Timezone:  "America/Chicago",
	},
	"5NE4": {
		ICAO:      "5NE4",
		Name:      "Snyder Ranch Airport",
		City:      "Sutherland",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3330,
		Latitude:  41.5722007751,
		Longitude: -101.1719970703,
		Timezone:  "America/Chicago",
	},
	"5NE5": {
		ICAO:      "5NE5",
		Name:      "Trego Airport",
		City:      "Sutherland",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2957,
		Latitude:  41.2066993713,
		Longitude: -101.1429977417,
		Timezone:  "America/Chicago",
	},
	"5NE6": {
		ICAO:      "5NE6",
		Name:      "Bakers Acres Airport",
		City:      "Trenton",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2950,
		Latitude:  40.25,
		Longitude: -101.0999984741,
		Timezone:  "America/Chicago",
	},
	"5NE9": {
		ICAO:      "5NE9",
		Name:      "Dodson Brothers Airport",
		City:      "Venango",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3575,
		Latitude:  40.7332992554,
		Longitude: -102.0500030518,
		Timezone:  "America/Denver",
	},
	"5NJ2": {
		ICAO:      "5NJ2",
		Name:      "Herr Mountain Airport",
		City:      "Stanton",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 560,
		Latitude:  40.5918006897,
		Longitude: -74.8413009644,
		Timezone:  "America/New_York",
	},
	"5NK0": {
		ICAO:      "5NK0",
		Name:      "Salubrious Point Airport",
		City:      "Chaumont",
		State:     "New York",
		Country:   "US",
		Elevation: 295,
		Latitude:  44.0442008972,
		Longitude: -76.1436004639,
		Timezone:  "America/New_York",
	},
	"5NK1": {
		ICAO:      "5NK1",
		Name:      "Toggenburg Farms Airport",
		City:      "Newark",
		State:     "New York",
		Country:   "US",
		Elevation: 435,
		Latitude:  43.1111984253,
		Longitude: -77.0539016724,
		Timezone:  "America/New_York",
	},
	"5NK5": {
		ICAO:      "5NK5",
		Name:      "Kingdom Field",
		City:      "Oswego",
		State:     "New York",
		Country:   "US",
		Elevation: 380,
		Latitude:  43.4086990356,
		Longitude: -76.4412994385,
		Timezone:  "America/New_York",
	},
	"5NK9": {
		ICAO:      "5NK9",
		Name:      "Treichler Farm Airport",
		City:      "Wales",
		State:     "New York",
		Country:   "US",
		Elevation: 1140,
		Latitude:  42.7616996765,
		Longitude: -78.4961013794,
		Timezone:  "America/New_York",
	},
	"5NY1": {
		ICAO:      "5NY1",
		Name:      "Tomahawk Hills Airport",
		City:      "Meridale",
		State:     "New York",
		Country:   "US",
		Elevation: 2380,
		Latitude:  42.3997993469,
		Longitude: -74.9063034058,
		Timezone:  "America/New_York",
	},
	"5NY3": {
		ICAO:      "5NY3",
		Name:      "Keech Airport",
		City:      "Etna",
		State:     "New York",
		Country:   "US",
		Elevation: 1070,
		Latitude:  42.4929008484,
		Longitude: -76.3869018555,
		Timezone:  "America/New_York",
	},
	"5NY4": {
		ICAO:      "5NY4",
		Name:      "Stanwix Heights Airport",
		City:      "Rome",
		State:     "New York",
		Country:   "US",
		Elevation: 610,
		Latitude:  43.1576004028,
		Longitude: -75.4307022095,
		Timezone:  "America/New_York",
	},
	"5NY5": {
		ICAO:      "5NY5",
		Name:      "Gardiner Airport",
		City:      "Gardiner",
		State:     "New York",
		Country:   "US",
		Elevation: 340,
		Latitude:  41.6668014526,
		Longitude: -74.149597168,
		Timezone:  "America/New_York",
	},
	"5NY7": {
		ICAO:      "5NY7",
		Name:      "Rolling Hills Airport",
		City:      "Goshen",
		State:     "New York",
		Country:   "US",
		Elevation: 410,
		Latitude:  41.3278999329,
		Longitude: -74.4098968506,
		Timezone:  "America/New_York",
	},
	"5OH0": {
		ICAO:      "5OH0",
		Name:      "Alge Airport",
		City:      "Arlington",
		State:     "Ohio",
		Country:   "US",
		Elevation: 857,
		Latitude:  40.904800415,
		Longitude: -83.6446990967,
		Timezone:  "America/New_York",
	},
	"5OH4": {
		ICAO:      "5OH4",
		Name:      "Armington Airport",
		City:      "Austinburg",
		State:     "Ohio",
		Country:   "US",
		Elevation: 820,
		Latitude:  41.7291984558,
		Longitude: -80.8789978027,
		Timezone:  "America/New_York",
	},
	"5OH6": {
		ICAO:      "5OH6",
		Name:      "Johnsons Field",
		City:      "Avon",
		State:     "Ohio",
		Country:   "US",
		Elevation: 700,
		Latitude:  41.4319992065,
		Longitude: -81.974899292,
		Timezone:  "America/New_York",
	},
	"5OH7": {
		ICAO:      "5OH7",
		Name:      "Keller Airport",
		City:      "Avon",
		State:     "Ohio",
		Country:   "US",
		Elevation: 692,
		Latitude:  41.4467010498,
		Longitude: -82.0071029663,
		Timezone:  "America/New_York",
	},
	"5OH9": {
		ICAO:      "5OH9",
		Name:      "Furey Airport",
		City:      "Malvern",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1050,
		Latitude:  40.6972007751,
		Longitude: -81.1791992188,
		Timezone:  "America/New_York",
	},
	"5OI0": {
		ICAO:      "5OI0",
		Name:      "Rohrer Airport",
		City:      "Loveland",
		State:     "Ohio",
		Country:   "US",
		Elevation: 800,
		Latitude:  39.284198761,
		Longitude: -84.2113037109,
		Timezone:  "America/New_York",
	},
	"5OI3": {
		ICAO:      "5OI3",
		Name:      "Sheets Field",
		City:      "Kinsman",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1225,
		Latitude:  41.4137001038,
		Longitude: -80.5248031616,
		Timezone:  "America/New_York",
	},
	"5OI5": {
		ICAO:      "5OI5",
		Name:      "Hamrick Airport",
		City:      "Willshire",
		State:     "Ohio",
		Country:   "US",
		Elevation: 810,
		Latitude:  40.6934013367,
		Longitude: -84.7805023193,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"5OI7": {
		ICAO:      "5OI7",
		Name:      "Gruetter Airport",
		City:      "Curtice",
		State:     "Ohio",
		Country:   "US",
		Elevation: 600,
		Latitude:  41.5924987793,
		Longitude: -83.37550354,
		Timezone:  "America/New_York",
	},
	"5OI8": {
		ICAO:      "5OI8",
		Name:      "Mc Knight Airport",
		City:      "Johnstown",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1172,
		Latitude:  40.1902999878,
		Longitude: -82.6546020508,
		Timezone:  "America/New_York",
	},
	"5OI9": {
		ICAO:      "5OI9",
		Name:      "Rogers Private Airport",
		City:      "Sherwood",
		State:     "Ohio",
		Country:   "US",
		Elevation: 725,
		Latitude:  41.3278007507,
		Longitude: -84.5577011108,
		Timezone:  "America/New_York",
	},
	"5OK2": {
		ICAO:      "5OK2",
		Name:      "Christopher M. Rippee Memorial Airport",
		City:      "Non",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 820,
		Latitude:  34.776667,
		Longitude: -96.245278,
		Timezone:  "America/Chicago",
	},
	"5OK3": {
		ICAO:      "5OK3",
		Name:      "Stearmans Roost Airport",
		City:      "Vinita",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 702,
		Latitude:  36.4883995056,
		Longitude: -95.1438980103,
		Timezone:  "America/Chicago",
	},
	"5OK4": {
		ICAO:      "5OK4",
		Name:      "Pacer Field",
		City:      "Goldsby",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1140,
		Latitude:  35.1170005798,
		Longitude: -97.4505996704,
		Timezone:  "America/Chicago",
	},
	"5OK6": {
		ICAO:      "5OK6",
		Name:      "Woodleaf Aero Estates Airport",
		City:      "Edmond",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1201,
		Latitude:  35.712600708,
		Longitude: -97.3669967651,
		Timezone:  "America/Chicago",
	},
	"5OK8": {
		ICAO:      "5OK8",
		Name:      "Alford A. Bratcher Airport",
		City:      "Marlow",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1100,
		Latitude:  34.6750984192,
		Longitude: -97.8559036255,
		Timezone:  "America/Chicago",
	},
	"5OK9": {
		ICAO:      "5OK9",
		Name:      "Night Hawk Airpatch Ultralightport",
		City:      "Inola",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 670,
		Latitude:  36.189201355,
		Longitude: -95.5402984619,
		Timezone:  "America/Chicago",
	},
	"5OR0": {
		ICAO:      "5OR0",
		Name:      "Backachers Ranch Airport",
		City:      "Selma",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1460,
		Latitude:  42.2850990295,
		Longitude: -123.5329971313,
		Timezone:  "America/Los_Angeles",
	},
	"5OR2": {
		ICAO:      "5OR2",
		Name:      "Bushnell Airport",
		City:      "Sheridan",
		State:     "Oregon",
		Country:   "US",
		Elevation: 220,
		Latitude:  45.1031990051,
		Longitude: -123.425003052,
		Timezone:  "America/Los_Angeles",
	},
	"5OR3": {
		ICAO:      "5OR3",
		Name:      "Siletz Airport",
		City:      "Siletz",
		State:     "Oregon",
		Country:   "US",
		Elevation: 100,
		Latitude:  44.7304000854,
		Longitude: -123.9150009155,
		Timezone:  "America/Los_Angeles",
	},
	"5OR4": {
		ICAO:      "5OR4",
		Name:      "Flying T Ranch Airport",
		City:      "Sprague River",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4340,
		Latitude:  42.4348983765,
		Longitude: -121.3499984741,
		Timezone:  "America/Los_Angeles",
	},
	"5OR5": {
		ICAO:      "5OR5",
		Name:      "Juniper Air Park",
		City:      "Bend",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3490,
		Latitude:  44.0345993042,
		Longitude: -121.1179962158,
		Timezone:  "America/Los_Angeles",
	},
	"5OR8": {
		ICAO:      "5OR8",
		Name:      "Hatch Airport",
		City:      "Stayton",
		State:     "Oregon",
		Country:   "US",
		Elevation: 380,
		Latitude:  44.7732009888,
		Longitude: -122.8499984741,
		Timezone:  "America/Los_Angeles",
	},
	"5OR9": {
		ICAO:      "5OR9",
		Name:      "Lone Oaks Ranch Airport",
		City:      "Stayton",
		State:     "Oregon",
		Country:   "US",
		Elevation: 630,
		Latitude:  44.7767982483,
		Longitude: -122.7600021362,
		Timezone:  "America/Los_Angeles",
	},
	"5PA0": {
		ICAO:      "5PA0",
		Name:      "Blomster Field Farm Airport",
		City:      "Carmichaels",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1050,
		Latitude:  39.8728485107,
		Longitude: -79.955329895,
		Timezone:  "America/New_York",
	},
	"5PA1": {
		ICAO:      "5PA1",
		Name:      "Broadt Personal Use Airport",
		City:      "Mainville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 920,
		Latitude:  40.9972991943,
		Longitude: -76.3837966919,
		Timezone:  "America/New_York",
	},
	"5PA2": {
		ICAO:      "5PA2",
		Name:      "Double D Skyranch Airport",
		City:      "St John's",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1140,
		Latitude:  41.035900116,
		Longitude: -76.0223999023,
		Timezone:  "America/New_York",
	},
	"5PA3": {
		ICAO:      "5PA3",
		Name:      "Metzler Airport",
		City:      "Manheim",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 420,
		Latitude:  40.1668014526,
		Longitude: -76.3996963501,
		Timezone:  "America/New_York",
	},
	"5PA6": {
		ICAO:      "5PA6",
		Name:      "Shield Farm Airport",
		City:      "Russell",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1240,
		Latitude:  41.9311981201,
		Longitude: -79.146697998,
		Timezone:  "America/New_York",
	},
	"5PA8": {
		ICAO:      "5PA8",
		Name:      "Chambers Airport",
		City:      "TunkHannock",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1021,
		Latitude:  41.55009842,
		Longitude: -76.04959869,
		Timezone:  "America/New_York",
	},
	"5PA9": {
		ICAO:      "5PA9",
		Name:      "Taylor Flight Park Ultralightport",
		City:      "Strattanville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1580,
		Latitude:  41.1884002686,
		Longitude: -79.2823028564,
		Timezone:  "America/New_York",
	},
	"5PN5": {
		ICAO:      "5PN5",
		Name:      "Bittner-Whitsel Airport",
		City:      "Rouzerville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 240,
		Latitude:  39.7422981262,
		Longitude: -77.5353012085,
		Timezone:  "America/New_York",
	},
	"5PN7": {
		ICAO:      "5PN7",
		Name:      "Jarrett Airport",
		City:      "Ivyland",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 210,
		Latitude:  40.2332992554,
		Longitude: -75.0749969482,
		Timezone:  "America/New_York",
	},
	"5PN8": {
		ICAO:      "5PN8",
		Name:      "Lincoln Farms Airport",
		City:      "Calvin",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1200,
		Latitude:  40.3288993835,
		Longitude: -78.0449981689,
		Timezone:  "America/New_York",
	},
	"5PS4": {
		ICAO:      "5PS4",
		Name:      "Hansen Airport",
		City:      "Sassamansville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 320,
		Latitude:  40.3426017761,
		Longitude: -75.5782012939,
		Timezone:  "America/New_York",
	},
	"5PS5": {
		ICAO:      "5PS5",
		Name:      "Chestnut Hill Airport",
		City:      "Duncannon",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 620,
		Latitude:  40.3968009949,
		Longitude: -77.0740966797,
		Timezone:  "America/New_York",
	},
	"5PS9": {
		ICAO:      "5PS9",
		Name:      "Tate Airport",
		City:      "West Rockhill",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 570,
		Latitude:  40.3851013184,
		Longitude: -75.3330001831,
		Timezone:  "America/New_York",
	},
	"5TA0": {
		ICAO:      "5TA0",
		Name:      "Hamilton Aircraft Inc Airport",
		City:      "Seminole",
		State:     "Texas",
		Country:   "US",
		Elevation: 3520,
		Latitude:  32.7319984436,
		Longitude: -102.9440002441,
		Timezone:  "America/Chicago",
	},
	"5TA1": {
		ICAO:      "5TA1",
		Name:      "Charping Airport",
		City:      "Temple",
		State:     "Texas",
		Country:   "US",
		Elevation: 683,
		Latitude:  31.1935005188,
		Longitude: -97.4072036743,
		Timezone:  "America/Chicago",
	},
	"5TA2": {
		ICAO:      "5TA2",
		Name:      "Rabbit Run Airport",
		City:      "Waco",
		State:     "Texas",
		Country:   "US",
		Elevation: 740,
		Latitude:  31.4545993805,
		Longitude: -97.2502975464,
		Timezone:  "America/Chicago",
	},
	"5TA3": {
		ICAO:      "5TA3",
		Name:      "Pearson Ranch Private Airport",
		City:      "Yoakum",
		State:     "Texas",
		Country:   "US",
		Elevation: 350,
		Latitude:  29.3586006165,
		Longitude: -97.1446990967,
		Timezone:  "America/Chicago",
	},
	"5TA4": {
		ICAO:      "5TA4",
		Name:      "Reed Airport",
		City:      "Claude",
		State:     "Texas",
		Country:   "US",
		Elevation: 3369,
		Latitude:  34.900100708,
		Longitude: -101.3170013428,
		Timezone:  "America/Chicago",
	},
	"5TA5": {
		ICAO:      "5TA5",
		Name:      "Creasy Airport",
		City:      "Santa Fe",
		State:     "Texas",
		Country:   "US",
		Elevation: 23,
		Latitude:  29.3360996246,
		Longitude: -95.115196228,
		Timezone:  "America/Chicago",
	},
	"5TA6": {
		ICAO:      "5TA6",
		Name:      "Knot 2 Shabby Airport",
		City:      "Winters",
		State:     "Texas",
		Country:   "US",
		Elevation: 1820,
		Latitude:  31.9344005585,
		Longitude: -99.8943023682,
		Timezone:  "America/Chicago",
	},
	"5TA7": {
		ICAO:      "5TA7",
		Name:      "Kami-Kazi Airport",
		City:      "Santa Fe",
		State:     "Texas",
		Country:   "US",
		Elevation: 25,
		Latitude:  29.4207992554,
		Longitude: -95.1591033936,
		Timezone:  "America/Chicago",
	},
	"5TA8": {
		ICAO:      "5TA8",
		Name:      "Deer Meadow Ranch Airport",
		City:      "Athens",
		State:     "Texas",
		Country:   "US",
		Elevation: 520,
		Latitude:  32.0133018494,
		Longitude: -95.9282989502,
		Timezone:  "America/Chicago",
	},
	"5TA9": {
		ICAO:      "5TA9",
		Name:      "Seagoville Airport",
		City:      "Seagoville",
		State:     "Texas",
		Country:   "US",
		Elevation: 415,
		Latitude:  32.6099014282,
		Longitude: -96.5269012451,
		Timezone:  "America/Chicago",
	},
	"5TE0": {
		ICAO:      "5TE0",
		Name:      "Comanche Ranch Airport",
		City:      "Eagle Pass",
		State:     "Texas",
		Country:   "US",
		Elevation: 730,
		Latitude:  28.6280002594,
		Longitude: -100.158996582,
		Timezone:  "America/Chicago",
	},
	"5TE1": {
		ICAO:      "5TE1",
		Name:      "Rawls Ranch Airport",
		City:      "Marfa",
		State:     "Texas",
		Country:   "US",
		Elevation: 4520,
		Latitude:  29.6501998901,
		Longitude: -103.9499969482,
		Timezone:  "America/Chicago",
	},
	"5TE2": {
		ICAO:      "5TE2",
		Name:      "Bleakley Ranch Airport",
		City:      "Dripping Springs",
		State:     "Texas",
		Country:   "US",
		Elevation: 1384,
		Latitude:  30.2252006531,
		Longitude: -98.141998291,
		Timezone:  "America/Chicago",
	},
	"5TE3": {
		ICAO:      "5TE3",
		Name:      "Alexander Ranch Airport",
		City:      "Dripping Springs",
		State:     "Texas",
		Country:   "US",
		Elevation: 1250,
		Latitude:  30.271900177,
		Longitude: -98.1169967651,
		Timezone:  "America/Chicago",
	},
	"5TE4": {
		ICAO:      "5TE4",
		Name:      "Perkins Prothro Cimarron Ranch Airport",
		City:      "Kerrick",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 3995,
		Latitude:  36.5083999634,
		Longitude: -102.391998291,
		Timezone:  "America/Chicago",
	},
	"5TE5": {
		ICAO:      "5TE5",
		Name:      "Iron Mountain Ranch Airport",
		City:      "Marathon",
		State:     "Texas",
		Country:   "US",
		Elevation: 4313,
		Latitude:  30.2738990784,
		Longitude: -103.2369995117,
		Timezone:  "America/Chicago",
	},
	"5TE6": {
		ICAO:      "5TE6",
		Name:      "Keystone Ranch Airport",
		City:      "San Angelo",
		State:     "Texas",
		Country:   "US",
		Elevation: 2023,
		Latitude:  31.269167,
		Longitude: -100.460833,
		Timezone:  "America/Chicago",
	},
	"5TE7": {
		ICAO:      "5TE7",
		Name:      "Renz Ranch Airport",
		City:      "Eagle Lake",
		State:     "Texas",
		Country:   "US",
		Elevation: 220,
		Latitude:  29.7194004059,
		Longitude: -96.3094024658,
		Timezone:  "America/Chicago",
	},
	"5TE8": {
		ICAO:      "5TE8",
		Name:      "Willis N Clark Airport",
		City:      "Miami",
		State:     "Texas",
		Country:   "US",
		Elevation: 3086,
		Latitude:  35.7611999512,
		Longitude: -100.7519989014,
		Timezone:  "America/Chicago",
	},
	"5TN4": {
		ICAO:      "5TN4",
		Name:      "Mcgraw's Backyard Airport",
		City:      "Greenback",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 920,
		Latitude:  35.6706008911,
		Longitude: -84.1157989502,
		Timezone:  "America/New_York",
	},
	"5TN9": {
		ICAO:      "5TN9",
		Name:      "One Grand Field",
		City:      "Cookeville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1000,
		Latitude:  36.113899231,
		Longitude: -85.5950012207,
		Timezone:  "America/Chicago",
	},
	"5TS0": {
		ICAO:      "5TS0",
		Name:      "Diehl Ranch Airport",
		City:      "Arcadia",
		State:     "Texas",
		Country:   "US",
		Elevation: 26,
		Latitude:  29.4125003815,
		Longitude: -95.1223983765,
		Timezone:  "America/Chicago",
	},
	"5TS1": {
		ICAO:      "5TS1",
		Name:      "Uno Mas Ranch Airport",
		City:      "Cotulla",
		State:     "Texas",
		Country:   "US",
		Elevation: 380,
		Latitude:  28.2057991028,
		Longitude: -99.1344985962,
		Timezone:  "America/Chicago",
	},
	"5TS2": {
		ICAO:      "5TS2",
		Name:      "Chan-C Airport",
		City:      "Coupland",
		State:     "Texas",
		Country:   "US",
		Elevation: 536,
		Latitude:  30.4794006348,
		Longitude: -97.3191986084,
		Timezone:  "America/Chicago",
	},
	"5TS3": {
		ICAO:      "5TS3",
		Name:      "Knapp Pecan Orchard Airpark",
		City:      "Dawson",
		State:     "Texas",
		Country:   "US",
		Elevation: 450,
		Latitude:  31.974199295,
		Longitude: -96.6782989502,
		Timezone:  "America/Chicago",
	},
	"5TS4": {
		ICAO:      "5TS4",
		Name:      "Mc Entire Airport",
		City:      "Decatur",
		State:     "Texas",
		Country:   "US",
		Elevation: 790,
		Latitude:  33.1445999146,
		Longitude: -97.6016998291,
		Timezone:  "America/Chicago",
	},
	"5TS5": {
		ICAO:      "5TS5",
		Name:      "Mc Donald Ranch Airport",
		City:      "Dilley",
		State:     "Texas",
		Country:   "US",
		Elevation: 555,
		Latitude:  28.6511001587,
		Longitude: -99.2886962891,
		Timezone:  "America/Chicago",
	},
	"5TS7": {
		ICAO:      "5TS7",
		Name:      "R D Williams Airport",
		City:      "Garrison",
		State:     "Texas",
		Country:   "US",
		Elevation: 360,
		Latitude:  31.776599884,
		Longitude: -94.485496521,
		Timezone:  "America/Chicago",
	},
	"5TS8": {
		ICAO:      "5TS8",
		Name:      "Bear Creek Ranch Airport",
		City:      "New Braunfels",
		State:     "Texas",
		Country:   "US",
		Elevation: 1360,
		Latitude:  29.808599472,
		Longitude: -98.2724990845,
		Timezone:  "America/Chicago",
	},
	"5TS9": {
		ICAO:      "5TS9",
		Name:      "Big Duke's Place Airport",
		City:      "Indianola",
		State:     "Texas",
		Country:   "US",
		Elevation: 5,
		Latitude:  28.5303001404,
		Longitude: -96.5274963379,
		Timezone:  "America/Chicago",
	},
	"5TX0": {
		ICAO:      "5TX0",
		Name:      "Hidden Valley Airpark",
		City:      "Lake Dallas",
		State:     "Texas",
		Country:   "US",
		Elevation: 611,
		Latitude:  33.173500061,
		Longitude: -97.051399231,
		Timezone:  "America/Chicago",
	},
	"5TX1": {
		ICAO:      "5TX1",
		Name:      "Harley White Field",
		City:      "Lamkin",
		State:     "Texas",
		Country:   "US",
		Elevation: 1120,
		Latitude:  31.8612995148,
		Longitude: -98.2594985962,
		Timezone:  "America/Chicago",
	},
	"5TX2": {
		ICAO:      "5TX2",
		Name:      "Grove Hill Airport",
		City:      "Leonard",
		State:     "Texas",
		Country:   "US",
		Elevation: 721,
		Latitude:  33.4222984314,
		Longitude: -96.2169036865,
		Timezone:  "America/Chicago",
	},
	"5TX4": {
		ICAO:      "5TX4",
		Name:      "Black Mark Strip",
		City:      "Lewisville",
		State:     "Texas",
		Country:   "US",
		Elevation: 635,
		Latitude:  33.0335006714,
		Longitude: -97.0670013428,
		Timezone:  "America/Chicago",
	},
	"5TX6": {
		ICAO:      "5TX6",
		Name:      "Hilliard Landing Area Airport",
		City:      "Lewisville",
		State:     "Texas",
		Country:   "US",
		Elevation: 635,
		Latitude:  33.0167999268,
		Longitude: -97.0725021362,
		Timezone:  "America/Chicago",
	},
	"5TX8": {
		ICAO:      "5TX8",
		Name:      "Boon/Lovelace Airport",
		City:      "Linden",
		State:     "Texas",
		Country:   "US",
		Elevation: 429,
		Latitude:  33.0321006775,
		Longitude: -94.3794021606,
		Timezone:  "America/Chicago",
	},
	"5TX9": {
		ICAO:      "5TX9",
		Name:      "Utopia on the River Airport",
		City:      "Utopia",
		State:     "Texas",
		Country:   "US",
		Elevation: 1320,
		Latitude:  29.5835990906,
		Longitude: -99.5261993408,
		Timezone:  "America/Chicago",
	},
	"5VA5": {
		ICAO:      "5VA5",
		Name:      "Chimney View Airport",
		City:      "Fredericksburg",
		State:     "Virginia",
		Country:   "US",
		Elevation: 25,
		Latitude:  38.4034996033,
		Longitude: -77.3158035278,
		Timezone:  "America/New_York",
	},
	"5VA9": {
		ICAO:      "5VA9",
		Name:      "Kilmarnock/Tolbert Field",
		City:      "Kilmarnock",
		State:     "Virginia",
		Country:   "US",
		Elevation: 7,
		Latitude:  37.6926002502,
		Longitude: -76.3182983398,
		Timezone:  "America/New_York",
	},
	"5WA0": {
		ICAO:      "5WA0",
		Name:      "Sourdough Airport",
		City:      "Tonasket",
		State:     "Washington",
		Country:   "US",
		Elevation: 2640,
		Latitude:  48.5956001282,
		Longitude: -119.1009979248,
		Timezone:  "America/Los_Angeles",
	},
	"5WA1": {
		ICAO:      "5WA1",
		Name:      "Dorman Field",
		City:      "Mesa",
		State:     "Washington",
		Country:   "US",
		Elevation: 700,
		Latitude:  46.5214996338,
		Longitude: -119.1790008545,
		Timezone:  "America/Los_Angeles",
	},
	"5WA2": {
		ICAO:      "5WA2",
		Name:      "B & M Ranch Airport",
		City:      "Monroe",
		State:     "Washington",
		Country:   "US",
		Elevation: 370,
		Latitude:  47.8665008545,
		Longitude: -121.9010009766,
		Timezone:  "America/Los_Angeles",
	},
	"5WA5": {
		ICAO:      "5WA5",
		Name:      "Ross Private Strip",
		City:      "Orondo",
		State:     "Washington",
		Country:   "US",
		Elevation: 920,
		Latitude:  47.7631988525,
		Longitude: -120.1520004272,
		Timezone:  "America/Los_Angeles",
	},
	"5WA7": {
		ICAO:      "5WA7",
		Name:      "Warwick Airport",
		City:      "Centerville",
		State:     "Washington",
		Country:   "US",
		Elevation: 1620,
		Latitude:  45.7164993286,
		Longitude: -120.9850006104,
		Timezone:  "America/Los_Angeles",
	},
	"5WA8": {
		ICAO:      "5WA8",
		Name:      "Hogan's Corner Airport",
		City:      "Ocean Shores",
		State:     "Washington",
		Country:   "US",
		Elevation: 50,
		Latitude:  47.0429000854,
		Longitude: -124.146003723,
		Timezone:  "America/Los_Angeles",
	},
	"5WA9": {
		ICAO:      "5WA9",
		Name:      "Brush Prairie Aerodrome",
		City:      "Brush Prairie",
		State:     "Washington",
		Country:   "US",
		Elevation: 290,
		Latitude:  45.7238998413,
		Longitude: -122.5439987183,
		Timezone:  "America/Los_Angeles",
	},
	"5WI0": {
		ICAO:      "5WI0",
		Name:      "Grandpa's Farm Airport",
		City:      "Augusta",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 970,
		Latitude:  44.7027015686,
		Longitude: -91.1337966919,
		Timezone:  "America/Chicago",
	},
	"5WI1": {
		ICAO:      "5WI1",
		Name:      "Springbrook Airport",
		City:      "Springbrook",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1110,
		Latitude:  45.9019012451,
		Longitude: -91.6771011353,
		Timezone:  "America/Chicago",
	},
	"5WI2": {
		ICAO:      "5WI2",
		Name:      "Plainfield International Airport",
		City:      "Plainfield",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1110,
		Latitude:  44.2224998474,
		Longitude: -89.4953994751,
		Timezone:  "America/Chicago",
	},
	"5WI3": {
		ICAO:      "5WI3",
		Name:      "Der Schwarzwald Airport",
		City:      "Waterloo",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 925,
		Latitude:  43.2078018188,
		Longitude: -89.0170974731,
		Timezone:  "America/Chicago",
	},
	"5WI4": {
		ICAO:      "5WI4",
		Name:      "Larson Airport",
		City:      "Marinette",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 615,
		Latitude:  45.081401825,
		Longitude: -87.6792984009,
		Timezone:  "America/Chicago",
	},
	"5WI5": {
		ICAO:      "5WI5",
		Name:      "Haymeadow Airport",
		City:      "Merrill",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1500,
		Latitude:  45.2750015259,
		Longitude: -89.4917984009,
		Timezone:  "America/Chicago",
	},
	"5WI6": {
		ICAO:      "5WI6",
		Name:      "Independence Airport",
		City:      "Independence",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 804,
		Latitude:  44.3669013977,
		Longitude: -91.3957977295,
		Timezone:  "America/Chicago",
	},
	"5WI8": {
		ICAO:      "5WI8",
		Name:      "Ermis-Ridgeview Airport",
		City:      "Coleman",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 715,
		Latitude:  45.0760993958,
		Longitude: -88.0043029785,
		Timezone:  "America/Chicago",
	},
	"5WI9": {
		ICAO:      "5WI9",
		Name:      "Patz Airport",
		City:      "Pound",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 860,
		Latitude:  45.129699707,
		Longitude: -88.1648025513,
		Timezone:  "America/Chicago",
	},
	"5WN2": {
		ICAO:      "5WN2",
		Name:      "Eberle Ranch Airport",
		City:      "Dane",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1050,
		Latitude:  43.2682991028,
		Longitude: -89.4843978882,
		Timezone:  "America/Chicago",
	},
	"5WN8": {
		ICAO:      "5WN8",
		Name:      "Knight Sky Airport",
		City:      "Wausau",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1475,
		Latitude:  45.0323982239,
		Longitude: -89.401802063,
		Timezone:  "America/Chicago",
	},
	"5WN9": {
		ICAO:      "5WN9",
		Name:      "Ottman Landing Airport",
		City:      "Maiden Rock",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1150,
		Latitude:  44.5943984985,
		Longitude: -92.2584991455,
		Timezone:  "America/Chicago",
	},
	"5XS0": {
		ICAO:      "5XS0",
		Name:      "Tnt Ultralightport",
		City:      "Randolph",
		State:     "Texas",
		Country:   "US",
		Elevation: 708,
		Latitude:  33.4967002869,
		Longitude: -96.2647018433,
		Timezone:  "America/Chicago",
	},
	"5XS2": {
		ICAO:      "5XS2",
		Name:      "Kimball Farm Service Inc Airport",
		City:      "Hereford",
		State:     "Texas",
		Country:   "US",
		Elevation: 3700,
		Latitude:  35.0334014893,
		Longitude: -102.1999969482,
		Timezone:  "America/Chicago",
	},
	"5XS3": {
		ICAO:      "5XS3",
		Name:      "Wilber Farms Airport",
		City:      "Winnie",
		State:     "Texas",
		Country:   "US",
		Elevation: 11,
		Latitude:  29.7696990967,
		Longitude: -94.2612991333,
		Timezone:  "America/Chicago",
	},
	"5XS4": {
		ICAO:      "5XS4",
		Name:      "Gary's Airport",
		City:      "Marion",
		State:     "Texas",
		Country:   "US",
		Elevation: 655,
		Latitude:  29.5862998962,
		Longitude: -98.1196975708,
		Timezone:  "America/Chicago",
	},
	"5XS5": {
		ICAO:      "5XS5",
		Name:      "Wits End Ranch Airport",
		City:      "Van",
		State:     "Texas",
		Country:   "US",
		Elevation: 545,
		Latitude:  32.5427777778,
		Longitude: -95.7086111111,
		Timezone:  "America/Chicago",
	},
	"5XS6": {
		ICAO:      "5XS6",
		Name:      "Old Reb Airport",
		City:      "Mercedes",
		State:     "Texas",
		Country:   "US",
		Elevation: 60,
		Latitude:  26.1753997803,
		Longitude: -97.8752975464,
		Timezone:  "America/Chicago",
	},
	"5XS7": {
		ICAO:      "5XS7",
		Name:      "Outback Airport",
		City:      "Lockney",
		State:     "Texas",
		Country:   "US",
		Elevation: 3286,
		Latitude:  34.2296981812,
		Longitude: -101.4639968872,
		Timezone:  "America/Chicago",
	},
	"5XS8": {
		ICAO:      "5XS8",
		Name:      "L Davis Ranch Airport",
		City:      "Brackettville",
		State:     "Texas",
		Country:   "US",
		Elevation: 1390,
		Latitude:  29.5168991089,
		Longitude: -100.3170013428,
		Timezone:  "America/Chicago",
	},
	"5XS9": {
		ICAO:      "5XS9",
		Name:      "Byrt Airport",
		City:      "Streetman",
		State:     "Texas",
		Country:   "US",
		Elevation: 414,
		Latitude:  31.9218006134,
		Longitude: -96.2649993896,
		Timezone:  "America/Chicago",
	},
	"60AZ": {
		ICAO:      "60AZ",
		Name:      "Wood's Airstrip",
		City:      "Wenden",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1962,
		Latitude:  33.8381004333,
		Longitude: -113.4540023804,
		Timezone:  "America/Phoenix",
	},
	"60CL": {
		ICAO:      "60CL",
		Name:      "Chp Academy Airport",
		City:      "Bryte",
		State:     "California",
		Country:   "US",
		Elevation: 20,
		Latitude:  38.5988006592,
		Longitude: -121.5609970093,
		Timezone:  "America/Los_Angeles",
	},
	"60CO": {
		ICAO:      "60CO",
		Name:      "Stevens Airport",
		City:      "Hudson",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5180,
		Latitude:  39.9667015076,
		Longitude: -104.6510009766,
		Timezone:  "America/Denver",
	},
	"60IN": {
		ICAO:      "60IN",
		Name:      "Fisher Farm Airport",
		City:      "Roanoke",
		State:     "Indiana",
		Country:   "US",
		Elevation: 750,
		Latitude:  40.9458999634,
		Longitude: -85.3700027466,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"60IS": {
		ICAO:      "60IS",
		Name:      "Nelson Private Airport",
		City:      "Browns",
		State:     "Illinois",
		Country:   "US",
		Elevation: 420,
		Latitude:  38.4006004333,
		Longitude: -87.9886016846,
		Timezone:  "America/Chicago",
	},
	"60KS": {
		ICAO:      "60KS",
		Name:      "Alley Field",
		City:      "Douglass",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1260,
		Latitude:  37.5139007568,
		Longitude: -97.0002975464,
		Timezone:  "America/Chicago",
	},
	"60MI": {
		ICAO:      "60MI",
		Name:      "Maybee Airport",
		City:      "Maybee",
		State:     "Michigan",
		Country:   "US",
		Elevation: 650,
		Latitude:  42.0414009094,
		Longitude: -83.5669021606,
		Timezone:  "America/Detroit",
	},
	"60MN": {
		ICAO:      "60MN",
		Name:      "Fuhr Flying Svc Airport",
		City:      "Seaforth",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1070,
		Latitude:  44.4679985046,
		Longitude: -95.2835998535,
		Timezone:  "America/Chicago",
	},
	"60NC": {
		ICAO:      "60NC",
		Name:      "Star Hill Golf Club Airport",
		City:      "Cape Carteret",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 40,
		Latitude:  34.7052001953,
		Longitude: -77.0511016846,
		Timezone:  "America/New_York",
	},
	"60NJ": {
		ICAO:      "60NJ",
		Name:      "O'Dwyer Airport",
		City:      "Flemington",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 200,
		Latitude:  40.4668006897,
		Longitude: -74.832901001,
		Timezone:  "America/New_York",
	},
	"60OI": {
		ICAO:      "60OI",
		Name:      "Zorn Acres Airport",
		City:      "Milan",
		State:     "Ohio",
		Country:   "US",
		Elevation: 640,
		Latitude:  41.3567008972,
		Longitude: -82.6249008179,
		Timezone:  "America/New_York",
	},
	"60OK": {
		ICAO:      "60OK",
		Name:      "Carlin Lawrence Airport",
		City:      "Carnegie",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1340,
		Latitude:  35.1222991943,
		Longitude: -98.5197982788,
		Timezone:  "America/Chicago",
	},
	"60OR": {
		ICAO:      "60OR",
		Name:      "Whitaker Airport",
		City:      "Oakland",
		State:     "Oregon",
		Country:   "US",
		Elevation: 625,
		Latitude:  43.4264984131,
		Longitude: -123.2799987793,
		Timezone:  "America/Los_Angeles",
	},
	"60TA": {
		ICAO:      "60TA",
		Name:      "Air Ranch Estates Airport",
		City:      "Waxahachie",
		State:     "Texas",
		Country:   "US",
		Elevation: 670,
		Latitude:  32.3250999451,
		Longitude: -96.9475021362,
		Timezone:  "America/Chicago",
	},
	"60TE": {
		ICAO:      "60TE",
		Name:      "Tierra Linda Ranch Airport",
		City:      "Kerrville",
		State:     "Texas",
		Country:   "US",
		Elevation: 1990,
		Latitude:  30.1334991455,
		Longitude: -99.1502990723,
		Timezone:  "America/Chicago",
	},
	"60TX": {
		ICAO:      "60TX",
		Name:      "P-K Ranch Airport",
		City:      "Bellville",
		State:     "Texas",
		Country:   "US",
		Elevation: 350,
		Latitude:  30.0335998535,
		Longitude: -96.2636032104,
		Timezone:  "America/Chicago",
	},
	"61AR": {
		ICAO:      "61AR",
		Name:      "The Valley Airport",
		City:      "Cotter",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 450,
		Latitude:  36.3083992,
		Longitude: -92.53070068,
		Timezone:  "America/Chicago",
	},
	"61AZ": {
		ICAO:      "61AZ",
		Name:      "White Mountain Ultralightport",
		City:      "Show Low",
		State:     "Arizona",
		Country:   "US",
		Elevation: 6546,
		Latitude:  34.3697013855,
		Longitude: -109.6969985962,
		Timezone:  "America/Phoenix",
	},
	"61CA": {
		ICAO:      "61CA",
		Name:      "Bauer Airport",
		City:      "Twentynine Palms",
		State:     "California",
		Country:   "US",
		Elevation: 1950,
		Latitude:  34.1747016907,
		Longitude: -116.0670013428,
		Timezone:  "America/Los_Angeles",
	},
	"61CL": {
		ICAO:      "61CL",
		Name:      "Johnson Brothers Airport",
		City:      "Calexico",
		State:     "California",
		Country:   "US",
		Elevation: -1,
		Latitude:  32.6734008789,
		Longitude: -115.5589981079,
		Timezone:  "America/Los_Angeles",
	},
	"61GA": {
		ICAO:      "61GA",
		Name:      "Pea Patch Aerodrome",
		City:      "Blythe",
		State:     "Georgia",
		Country:   "US",
		Elevation: 434,
		Latitude:  33.3017997742,
		Longitude: -82.1723022461,
		Timezone:  "America/New_York",
	},
	"61II": {
		ICAO:      "61II",
		Name:      "Kay Field",
		City:      "Wolflake",
		State:     "Indiana",
		Country:   "US",
		Elevation: 930,
		Latitude:  41.3624992371,
		Longitude: -85.4777984619,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"61IN": {
		ICAO:      "61IN",
		Name:      "Wilkerson's Airport",
		City:      "Scottsburg",
		State:     "Indiana",
		Country:   "US",
		Elevation: 730,
		Latitude:  38.7136993408,
		Longitude: -85.6035995483,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"61IS": {
		ICAO:      "61IS",
		Name:      "Jim & Peg RLA Restricted Landing Area",
		City:      "Dwight",
		State:     "Illinois",
		Country:   "US",
		Elevation: 630,
		Latitude:  41.0547981262,
		Longitude: -88.387298584,
		Timezone:  "America/Chicago",
	},
	"61KS": {
		ICAO:      "61KS",
		Name:      "Masters Field",
		City:      "Troy",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1120,
		Latitude:  39.7588996887,
		Longitude: -95.0958023071,
		Timezone:  "America/Chicago",
	},
	"61KY": {
		ICAO:      "61KY",
		Name:      "Schroder Airport",
		City:      "Owenton",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 910,
		Latitude:  38.5500984192,
		Longitude: -84.7632980347,
		Timezone:  "America/New_York",
	},
	"61LL": {
		ICAO:      "61LL",
		Name:      "Smith Restricted Landing Area",
		City:      "Peirron",
		State:     "Illinois",
		Country:   "US",
		Elevation: 530,
		Latitude:  38.7731018066,
		Longitude: -89.5935974121,
		Timezone:  "America/Chicago",
	},
	"61MA": {
		ICAO:      "61MA",
		Name:      "Shirley Airport",
		City:      "Shirley",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 400,
		Latitude:  42.5270004272,
		Longitude: -71.6644973755,
		Timezone:  "America/New_York",
	},
	"61MN": {
		ICAO:      "61MN",
		Name:      "Traverse Air Airport",
		City:      "Wheaton",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1002,
		Latitude:  45.9536018372,
		Longitude: -96.4009017944,
		Timezone:  "America/Chicago",
	},
	"61MO": {
		ICAO:      "61MO",
		Name:      "Carl Ensor Airport",
		City:      "Grainville",
		State:     "Missouri",
		Country:   "US",
		Elevation: 740,
		Latitude:  39.5485992432,
		Longitude: -92.0988006592,
		Timezone:  "America/Chicago",
	},
	"61MU": {
		ICAO:      "61MU",
		Name:      "Farris Strip",
		City:      "Faucett",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1054,
		Latitude:  39.5881996155,
		Longitude: -94.7837982178,
		Timezone:  "America/Chicago",
	},
	"61NC": {
		ICAO:      "61NC",
		Name:      "Southern Comforts Aerodrome",
		City:      "Parkton",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 175,
		Latitude:  34.8712997437,
		Longitude: -78.9655990601,
		Timezone:  "America/New_York",
	},
	"61ND": {
		ICAO:      "61ND",
		Name:      "Bakke Airport",
		City:      "Larimore",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1125,
		Latitude:  47.9474983215,
		Longitude: -97.6620025635,
		Timezone:  "America/Chicago",
	},
	"61NJ": {
		ICAO:      "61NJ",
		Name:      "Thomas Browne Airpark",
		City:      "Glassboro",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 145,
		Latitude:  39.6915016174,
		Longitude: -75.1432037354,
		Timezone:  "America/New_York",
	},
	"61NK": {
		ICAO:      "61NK",
		Name:      "Sunset Airport",
		City:      "Willsboro",
		State:     "New York",
		Country:   "US",
		Elevation: 175,
		Latitude:  44.329856,
		Longitude: -73.361793,
		Timezone:  "America/New_York",
	},
	"61NY": {
		ICAO:      "61NY",
		Name:      "Bassett Field",
		City:      "Lockport",
		State:     "New York",
		Country:   "US",
		Elevation: 629,
		Latitude:  43.1687011719,
		Longitude: -78.7823028564,
		Timezone:  "America/New_York",
	},
	"61OH": {
		ICAO:      "61OH",
		Name:      "Jetway Airport",
		City:      "Ravenna",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1130,
		Latitude:  41.1981010437,
		Longitude: -81.2095031738,
		Timezone:  "America/New_York",
	},
	"61OK": {
		ICAO:      "61OK",
		Name:      "The Highlands Airport",
		City:      "Cashion",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1160,
		Latitude:  35.7999992371,
		Longitude: -97.6087036133,
		Timezone:  "America/Chicago",
	},
	"61OR": {
		ICAO:      "61OR",
		Name:      "The Citadel Airport",
		City:      "Sisters",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3077,
		Latitude:  44.3348007202,
		Longitude: -121.3550033569,
		Timezone:  "America/Los_Angeles",
	},
	"61PA": {
		ICAO:      "61PA",
		Name:      "Hi Line Lodge Airport",
		City:      "Wellsboro",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 2202,
		Latitude:  41.6944999695,
		Longitude: -77.1650009155,
		Timezone:  "America/New_York",
	},
	"61PN": {
		ICAO:      "61PN",
		Name:      "A G A Farms Airport",
		City:      "Perkasie",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 450,
		Latitude:  40.4257011414,
		Longitude: -75.2323989868,
		Timezone:  "America/New_York",
	},
	"61TA": {
		ICAO:      "61TA",
		Name:      "Eagle Landing Airport",
		City:      "Avinger",
		State:     "Texas",
		Country:   "US",
		Elevation: 380,
		Latitude:  32.8839988708,
		Longitude: -94.6076965332,
		Timezone:  "America/Chicago",
	},
	"61TE": {
		ICAO:      "61TE",
		Name:      "Kezer Air Ranch Airport",
		City:      "Springtown",
		State:     "Texas",
		Country:   "US",
		Elevation: 830,
		Latitude:  32.9868011475,
		Longitude: -97.6196975708,
		Timezone:  "America/Chicago",
	},
	"61TX": {
		ICAO:      "61TX",
		Name:      "Traylor Tick Farm Airport",
		City:      "Bellville",
		State:     "Texas",
		Country:   "US",
		Elevation: 250,
		Latitude:  29.9724006653,
		Longitude: -96.3071975708,
		Timezone:  "America/Chicago",
	},
	"61VA": {
		ICAO:      "61VA",
		Name:      "High View Farm Airport",
		City:      "Berryville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 595,
		Latitude:  39.2354011536,
		Longitude: -78.0119018555,
		Timezone:  "America/New_York",
	},
	"61WA": {
		ICAO:      "61WA",
		Name:      "Burden Field-(Rabbit Run) Airport",
		City:      "Friday Harbor",
		State:     "Washington",
		Country:   "US",
		Elevation: 50,
		Latitude:  48.4720001221,
		Longitude: -123.0179977417,
		Timezone:  "America/Los_Angeles",
	},
	"61WI": {
		ICAO:      "61WI",
		Name:      "Dinnerbell Airport",
		City:      "Eden",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1120,
		Latitude:  43.7083015442,
		Longitude: -88.285697937,
		Timezone:  "America/Chicago",
	},
	"61XS": {
		ICAO:      "61XS",
		Name:      "Shanks Agricultural Strip",
		City:      "Wharton",
		State:     "Texas",
		Country:   "US",
		Elevation: 100,
		Latitude:  29.3243999481,
		Longitude: -96.0207977295,
		Timezone:  "America/Chicago",
	},
	"62AL": {
		ICAO:      "62AL",
		Name:      "Skywest Airpark",
		City:      "Mobile",
		State:     "Alabama",
		Country:   "US",
		Elevation: 150,
		Latitude:  30.6835002899,
		Longitude: -88.3720016479,
		Timezone:  "America/Chicago",
	},
	"62CL": {
		ICAO:      "62CL",
		Name:      "Flying S Ranch Airport",
		City:      "Caliente",
		State:     "California",
		Country:   "US",
		Elevation: 3485,
		Latitude:  35.4071998596,
		Longitude: -118.5,
		Timezone:  "America/Los_Angeles",
	},
	"62CO": {
		ICAO:      "62CO",
		Name:      "The Farm Airport",
		City:      "Keenesburg",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4820,
		Latitude:  40.0983009338,
		Longitude: -104.4250030518,
		Timezone:  "America/Denver",
	},
	"62FL": {
		ICAO:      "62FL",
		Name:      "David Wine's Airstrip",
		City:      "Lake Wales",
		State:     "Florida",
		Country:   "US",
		Elevation: 100,
		Latitude:  27.8444004059,
		Longitude: -81.4396972656,
		Timezone:  "America/New_York",
	},
	"62GA": {
		ICAO:      "62GA",
		Name:      "Seven Lakes Airport",
		City:      "Jackson",
		State:     "Georgia",
		Country:   "US",
		Elevation: 630,
		Latitude:  33.3246002197,
		Longitude: -83.9162979126,
		Timezone:  "America/New_York",
	},
	"62IA": {
		ICAO:      "62IA",
		Name:      "Lerchs Airport",
		City:      "Martelle",
		State:     "Iowa",
		Country:   "US",
		Elevation: 941,
		Latitude:  42.0666999817,
		Longitude: -91.3667984009,
		Timezone:  "America/Chicago",
	},
	"62IL": {
		ICAO:      "62IL",
		Name:      "Ferris Field",
		City:      "Marengo",
		State:     "Illinois",
		Country:   "US",
		Elevation: 841,
		Latitude:  42.1861000061,
		Longitude: -88.6043014526,
		Timezone:  "America/Chicago",
	},
	"62IN": {
		ICAO:      "62IN",
		Name:      "Fowler Field /Private/ Airport",
		City:      "Greentown",
		State:     "Indiana",
		Country:   "US",
		Elevation: 854,
		Latitude:  40.4309005737,
		Longitude: -85.978302002,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"62IS": {
		ICAO:      "62IS",
		Name:      "Wilson Airport",
		City:      "Fithian",
		State:     "Illinois",
		Country:   "US",
		Elevation: 670,
		Latitude:  40.0875015259,
		Longitude: -87.9075012207,
		Timezone:  "America/Chicago",
	},
	"62LA": {
		ICAO:      "62LA",
		Name:      "Ken Guidry Nr 4 Airport",
		City:      "Maurice",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 21,
		Latitude:  30.0751991272,
		Longitude: -92.2136993408,
		Timezone:  "America/Chicago",
	},
	"62MD": {
		ICAO:      "62MD",
		Name:      "Sandy Point Airport",
		City:      "Prince Frederick",
		State:     "Maryland",
		Country:   "US",
		Elevation: 20,
		Latitude:  38.488806,
		Longitude: -76.643633,
		Timezone:  "America/New_York",
	},
	"62MI": {
		ICAO:      "62MI",
		Name:      "Combs Airport",
		City:      "Samaria",
		State:     "Michigan",
		Country:   "US",
		Elevation: 640,
		Latitude:  41.8306007385,
		Longitude: -83.5652999878,
		Timezone:  "America/Detroit",
	},
	"62MN": {
		ICAO:      "62MN",
		Name:      "Taylors Falls Airport",
		City:      "Taylors Falls",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 920,
		Latitude:  45.381401062,
		Longitude: -92.6824035645,
		Timezone:  "America/Chicago",
	},
	"62MO": {
		ICAO:      "62MO",
		Name:      "Washburn Farm Airport",
		City:      "Grain Valley",
		State:     "Missouri",
		Country:   "US",
		Elevation: 855,
		Latitude:  39.0606002808,
		Longitude: -94.2033004761,
		Timezone:  "America/Chicago",
	},
	"62NC": {
		ICAO:      "62NC",
		Name:      "Hickory Hill Airport",
		City:      "Havelock",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 30,
		Latitude:  34.9367980957,
		Longitude: -76.9447021484,
		Timezone:  "America/New_York",
	},
	"62ND": {
		ICAO:      "62ND",
		Name:      "Morten Airport",
		City:      "Larimore",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1340,
		Latitude:  47.7792015076,
		Longitude: -97.7594985962,
		Timezone:  "America/Chicago",
	},
	"62NE": {
		ICAO:      "62NE",
		Name:      "Beebe Airport",
		City:      "Hayes Center",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2860,
		Latitude:  40.4474983215,
		Longitude: -100.7929992676,
		Timezone:  "America/Chicago",
	},
	"62NJ": {
		ICAO:      "62NJ",
		Name:      "Scheller Airport",
		City:      "Schooleys Mountain",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 1051,
		Latitude:  40.808333,
		Longitude: -74.808333,
		Timezone:  "America/New_York",
	},
	"62NM": {
		ICAO:      "62NM",
		Name:      "Seven Rivers Airport",
		City:      "Carlsbad",
		State:     "New Mexico",
		Country:   "US",
		Elevation: 3360,
		Latitude:  32.5987014771,
		Longitude: -104.4260025024,
		Timezone:  "America/Denver",
	},
	"62OH": {
		ICAO:      "62OH",
		Name:      "Willard Field",
		City:      "Bremen",
		State:     "Ohio",
		Country:   "US",
		Elevation: 820,
		Latitude:  39.6843986511,
		Longitude: -82.466796875,
		Timezone:  "America/New_York",
	},
	"62OK": {
		ICAO:      "62OK",
		Name:      "Lewis North Airport",
		City:      "Jay",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 960,
		Latitude:  36.3391990662,
		Longitude: -94.9204025269,
		Timezone:  "America/Chicago",
	},
	"62PA": {
		ICAO:      "62PA",
		Name:      "Shreveport North Airport",
		City:      "Wellsville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 550,
		Latitude:  40.0362014771,
		Longitude: -76.9954986572,
		Timezone:  "America/New_York",
	},
	"62TA": {
		ICAO:      "62TA",
		Name:      "Hawken Air One Airport",
		City:      "Elgin",
		State:     "Texas",
		Country:   "US",
		Elevation: 520,
		Latitude:  30.3183002472,
		Longitude: -97.3022003174,
		Timezone:  "America/Chicago",
	},
	"62TX": {
		ICAO:      "62TX",
		Name:      "Barge Ranch Airport",
		City:      "Belton",
		State:     "Texas",
		Country:   "US",
		Elevation: 590,
		Latitude:  31.0918006897,
		Longitude: -97.4602966309,
		Timezone:  "America/Chicago",
	},
	"62VA": {
		ICAO:      "62VA",
		Name:      "Grasso Salvage Airport",
		City:      "Holland",
		State:     "Virginia",
		Country:   "US",
		Elevation: 70,
		Latitude:  36.6698989868,
		Longitude: -76.7285995483,
		Timezone:  "America/New_York",
	},
	"62XS": {
		ICAO:      "62XS",
		Name:      "J F Ranch Airport",
		City:      "Clairette",
		State:     "Texas",
		Country:   "US",
		Elevation: 1220,
		Latitude:  32.0223999023,
		Longitude: -98.1336975098,
		Timezone:  "America/Chicago",
	},
	"63AK": {
		ICAO:      "63AK",
		Name:      "Kucera Residence Airport",
		City:      "Big Lake",
		State:     "Alaska",
		Country:   "US",
		Elevation: 189,
		Latitude:  61.5843009949,
		Longitude: -149.93800354,
		Timezone:  "America/Anchorage",
	},
	"63AR": {
		ICAO:      "63AR",
		Name:      "Mc Neely Airport",
		City:      "Earle",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 212,
		Latitude:  35.251499176,
		Longitude: -90.4598007202,
		Timezone:  "America/Chicago",
	},
	"63CA": {
		ICAO:      "63CA",
		Name:      "Desert Air Sky Ranch Airport",
		City:      "North Shore",
		State:     "California",
		Country:   "US",
		Latitude:  33.481098175,
		Longitude: -115.8740005493,
		Timezone:  "America/Los_Angeles",
	},
	"63CL": {
		ICAO:      "63CL",
		Name:      "G3 Ranch Airport",
		City:      "Capay",
		State:     "California",
		Country:   "US",
		Elevation: 360,
		Latitude:  38.7165985107,
		Longitude: -122.138999939,
		Timezone:  "America/Los_Angeles",
	},
	"63CN": {
		ICAO:      "63CN",
		Name:      "Meadowlark Field",
		City:      "Livermore",
		State:     "California",
		Country:   "US",
		Elevation: 730,
		Latitude:  37.6613006592,
		Longitude: -121.6940002441,
		Timezone:  "America/Los_Angeles",
	},
	"63CO": {
		ICAO:      "63CO",
		Name:      "Hendricks Field At West Creek Ranch Airport",
		City:      "Gateway",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4820,
		Latitude:  38.7055015564,
		Longitude: -108.93699646,
		Timezone:  "America/Denver",
	},
	"63FD": {
		ICAO:      "63FD",
		Name:      "Link Field",
		City:      "Alachua",
		State:     "Florida",
		Country:   "US",
		Elevation: 108,
		Latitude:  29.6980991364,
		Longitude: -82.4914016724,
		Timezone:  "America/New_York",
	},
	"63GA": {
		ICAO:      "63GA",
		Name:      "Zips Airport",
		City:      "Mc Donough",
		State:     "Georgia",
		Country:   "US",
		Elevation: 880,
		Latitude:  33.4528999329,
		Longitude: -84.095199585,
		Timezone:  "America/New_York",
	},
	"63II": {
		ICAO:      "63II",
		Name:      "Woods Field",
		City:      "Franklin",
		State:     "Indiana",
		Country:   "US",
		Elevation: 750,
		Latitude:  39.5051002502,
		Longitude: -86.0302963257,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"63IL": {
		ICAO:      "63IL",
		Name:      "Emerick Airport",
		City:      "Marengo",
		State:     "Illinois",
		Country:   "US",
		Elevation: 840,
		Latitude:  42.1833992004,
		Longitude: -88.5944976807,
		Timezone:  "America/Chicago",
	},
	"63IN": {
		ICAO:      "63IN",
		Name:      "Ropkey Field",
		City:      "Indianapolis",
		State:     "Indiana",
		Country:   "US",
		Elevation: 880,
		Latitude:  39.8995018005,
		Longitude: -86.271697998,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"63IS": {
		ICAO:      "63IS",
		Name:      "Corn Alley Airport",
		City:      "El Paso",
		State:     "Illinois",
		Country:   "US",
		Elevation: 715,
		Latitude:  40.7692985535,
		Longitude: -88.9831008911,
		Timezone:  "America/Chicago",
	},
	"63KS": {
		ICAO:      "63KS",
		Name:      "5-D Ranch Airport",
		City:      "Clay Center",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1190,
		Latitude:  39.3283004761,
		Longitude: -97.0608978271,
		Timezone:  "America/Chicago",
	},
	"63LA": {
		ICAO:      "63LA",
		Name:      "Leonards Airfield & Indust Park Airport",
		City:      "Delcambre",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 23,
		Latitude:  29.964099884,
		Longitude: -91.9121017456,
		Timezone:  "America/Chicago",
	},
	"63MN": {
		ICAO:      "63MN",
		Name:      "Weideman International Airport",
		City:      "Two Harbors",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 930,
		Latitude:  47.0943984985,
		Longitude: -91.6001968384,
		Timezone:  "America/Chicago",
	},
	"63NC": {
		ICAO:      "63NC",
		Name:      "Wood Airport",
		City:      "Wilson",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 170,
		Latitude:  35.8362998962,
		Longitude: -77.9963989258,
		Timezone:  "America/New_York",
	},
	"63ND": {
		ICAO:      "63ND",
		Name:      "Moellenkamp Airport",
		City:      "Lisbon",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1240,
		Latitude:  46.3652000427,
		Longitude: -97.7282028198,
		Timezone:  "America/Chicago",
	},
	"63NY": {
		ICAO:      "63NY",
		Name:      "Shear Airport",
		City:      "Ransomville",
		State:     "New York",
		Country:   "US",
		Elevation: 325,
		Latitude:  43.2611999512,
		Longitude: -78.9653015137,
		Timezone:  "America/New_York",
	},
	"63OH": {
		ICAO:      "63OH",
		Name:      "White's Airport",
		City:      "Darbyville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 815,
		Latitude:  39.7028007507,
		Longitude: -83.0939025879,
		Timezone:  "America/New_York",
	},
	"63OI": {
		ICAO:      "63OI",
		Name:      "Bowman Field",
		City:      "Delaware",
		State:     "Ohio",
		Country:   "US",
		Elevation: 965,
		Latitude:  40.3350982666,
		Longitude: -83.1360015869,
		Timezone:  "America/New_York",
	},
	"63OR": {
		ICAO:      "63OR",
		Name:      "Mountaindale Airport",
		City:      "Mountaindale",
		State:     "Oregon",
		Country:   "US",
		Elevation: 180,
		Latitude:  45.6165008545,
		Longitude: -123.0459976196,
		Timezone:  "America/Los_Angeles",
	},
	"63PA": {
		ICAO:      "63PA",
		Name:      "Boyer Airport",
		City:      "Wernersville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 510,
		Latitude:  40.3218002319,
		Longitude: -76.1005020142,
		Timezone:  "America/New_York",
	},
	"63TA": {
		ICAO:      "63TA",
		Name:      "Barnett Airport",
		City:      "Chillicothe",
		State:     "Texas",
		Country:   "US",
		Elevation: 1385,
		Latitude:  34.2695007324,
		Longitude: -99.5142974854,
		Timezone:  "America/Chicago",
	},
	"63TE": {
		ICAO:      "63TE",
		Name:      "Flying S Ranch Airport",
		City:      "Kennard",
		State:     "Texas",
		Country:   "US",
		Elevation: 370,
		Latitude:  31.3535003662,
		Longitude: -95.2058029175,
		Timezone:  "America/Chicago",
	},
	"63TX": {
		ICAO:      "63TX",
		Name:      "Grosser Airport",
		City:      "Bergheim",
		State:     "Texas",
		Country:   "US",
		Elevation: 1354,
		Latitude:  29.7751998901,
		Longitude: -98.5781021118,
		Timezone:  "America/Chicago",
	},
	"63WA": {
		ICAO:      "63WA",
		Name:      "Boyle R & D Airport",
		City:      "Colbert",
		State:     "Washington",
		Country:   "US",
		Elevation: 2350,
		Latitude:  47.8333015442,
		Longitude: -117.2740020752,
		Timezone:  "America/Los_Angeles",
	},
	"63WI": {
		ICAO:      "63WI",
		Name:      "Flying H Airport",
		City:      "Monroe",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1020,
		Latitude:  42.6657981873,
		Longitude: -89.7376022339,
		Timezone:  "America/Chicago",
	},
	"63XS": {
		ICAO:      "63XS",
		Name:      "Byram Ranch Airport",
		City:      "Henly",
		State:     "Texas",
		Country:   "US",
		Elevation: 1120,
		Latitude:  30.2241001129,
		Longitude: -98.2885971069,
		Timezone:  "America/Chicago",
	},
	"64AK": {
		ICAO:      "64AK",
		Name:      "Carpentiers Strip",
		City:      "Point Mackenzie",
		State:     "Alaska",
		Country:   "US",
		Elevation: 175,
		Latitude:  61.4398994446,
		Longitude: -150.0260009766,
		Timezone:  "America/Anchorage",
	},
	"64AR": {
		ICAO:      "64AR",
		Name:      "Lowrance Airport",
		City:      "Driver",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 232,
		Latitude:  35.6125984192,
		Longitude: -90.0208969116,
		Timezone:  "America/Chicago",
	},
	"64CL": {
		ICAO:      "64CL",
		Name:      "Goodyear Blimp Base Airport",
		City:      "Carson",
		State:     "California",
		Country:   "US",
		Elevation: 21,
		Latitude:  33.8563995361,
		Longitude: -118.2770004272,
		Timezone:  "America/Los_Angeles",
	},
	"64CN": {
		ICAO:      "64CN",
		Name:      "Sunrise Dusters Airport",
		City:      "Knights Landing",
		State:     "California",
		Country:   "US",
		Elevation: 25,
		Latitude:  38.8166007996,
		Longitude: -121.7009963989,
		Timezone:  "America/Los_Angeles",
	},
	"64CO": {
		ICAO:      "64CO",
		Name:      "Cholla Airport",
		City:      "Walsenburg",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5950,
		Latitude:  37.755556,
		Longitude: -104.756389,
		Timezone:  "America/Denver",
	},
	"64CT": {
		ICAO:      "64CT",
		Name:      "Woodstock Airport",
		City:      "South Woodstock",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 465,
		Latitude:  41.9215011597,
		Longitude: -71.9529037476,
		Timezone:  "America/New_York",
	},
	"64FA": {
		ICAO:      "64FA",
		Name:      "Naked Lady Ranch Airport",
		City:      "Stuart",
		State:     "Florida",
		Country:   "US",
		Elevation: 22,
		Latitude:  27.1427993774,
		Longitude: -80.3377990723,
		Timezone:  "America/New_York",
	},
	"64GA": {
		ICAO:      "64GA",
		Name:      "Big 'T' Airport",
		City:      "Senoia",
		State:     "Georgia",
		Country:   "US",
		Elevation: 900,
		Latitude:  33.2835006714,
		Longitude: -84.5388031006,
		Timezone:  "America/New_York",
	},
	"64IL": {
		ICAO:      "64IL",
		Name:      "Walpole Airport",
		City:      "Marengo",
		State:     "Illinois",
		Country:   "US",
		Elevation: 790,
		Latitude:  42.2416992188,
		Longitude: -88.6668014526,
		Timezone:  "America/Chicago",
	},
	"64IN": {
		ICAO:      "64IN",
		Name:      "Peacock Farms Airport",
		City:      "Fairmount",
		State:     "Indiana",
		Country:   "US",
		Elevation: 875,
		Latitude:  40.4347991943,
		Longitude: -85.62550354,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"64KY": {
		ICAO:      "64KY",
		Name:      "Hemp Ridge Airport",
		City:      "Shelbyville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 860,
		Latitude:  38.1530990601,
		Longitude: -85.1187973022,
		Timezone:  "America/New_York",
	},
	"64MO": {
		ICAO:      "64MO",
		Name:      "Booze Island Airport",
		City:      "St Joseph",
		State:     "Missouri",
		Country:   "US",
		Elevation: 794,
		Latitude:  39.6652984619,
		Longitude: -95.0132980347,
		Timezone:  "America/Chicago",
	},
	"64NC": {
		ICAO:      "64NC",
		Name:      "Fields Airport",
		City:      "Pleasant Garden",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 820,
		Latitude:  35.9019012451,
		Longitude: -79.7724990845,
		Timezone:  "America/New_York",
	},
	"64ND": {
		ICAO:      "64ND",
		Name:      "Z. P. Field",
		City:      "Mandan",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1840,
		Latitude:  46.8507995605,
		Longitude: -101.0749969482,
		Timezone:  "America/North_Dakota/New_Salem",
	},
	"64NY": {
		ICAO:      "64NY",
		Name:      "Hemlock Run Airport",
		City:      "Watkins Glen",
		State:     "New York",
		Country:   "US",
		Elevation: 1300,
		Latitude:  42.3672981262,
		Longitude: -76.931098938,
		Timezone:  "America/New_York",
	},
	"64OG": {
		ICAO:      "64OG",
		Name:      "Antone Ranch Airport",
		City:      "Mitchell",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3908,
		Latitude:  44.493436,
		Longitude: -119.843686,
		Timezone:  "America/Los_Angeles",
	},
	"64OH": {
		ICAO:      "64OH",
		Name:      "O K Dies Airport",
		City:      "Salem",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1280,
		Latitude:  40.8987007141,
		Longitude: -80.9292984009,
		Timezone:  "America/New_York",
	},
	"64OR": {
		ICAO:      "64OR",
		Name:      "Plum Valley Airport",
		City:      "Amity",
		State:     "Oregon",
		Country:   "US",
		Elevation: 230,
		Latitude:  45.0336990356,
		Longitude: -123.1719970703,
		Timezone:  "America/Los_Angeles",
	},
	"64TE": {
		ICAO:      "64TE",
		Name:      "Apache Springs Airport",
		City:      "Kerrville",
		State:     "Texas",
		Country:   "US",
		Elevation: 2016,
		Latitude:  30.1590995789,
		Longitude: -99.3373031616,
		Timezone:  "America/Chicago",
	},
	"64WI": {
		ICAO:      "64WI",
		Name:      "Sky Diving Airport",
		City:      "Omro",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 760,
		Latitude:  44.0414009094,
		Longitude: -88.7001037598,
		Timezone:  "America/Chicago",
	},
	"65AK": {
		ICAO:      "65AK",
		Name:      "Mcdonald Ridge Airport",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 285,
		Latitude:  61.5605010986,
		Longitude: -149.369003296,
		Timezone:  "America/Anchorage",
	},
	"65AR": {
		ICAO:      "65AR",
		Name:      "Bernard Manor Airport",
		City:      "Earle",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 210,
		Latitude:  35.2265014648,
		Longitude: -90.4654006958,
		Timezone:  "America/Chicago",
	},
	"65CL": {
		ICAO:      "65CL",
		Name:      "Al Divine Airport",
		City:      "Caruthers",
		State:     "California",
		Country:   "US",
		Elevation: 252,
		Latitude:  36.5144004822,
		Longitude: -119.763999939,
		Timezone:  "America/Los_Angeles",
	},
	"65CN": {
		ICAO:      "65CN",
		Name:      "Bottimore Ranch Airport",
		City:      "Herald",
		State:     "California",
		Country:   "US",
		Elevation: 70,
		Latitude:  38.3041000366,
		Longitude: -121.2509994507,
		Timezone:  "America/Los_Angeles",
	},
	"65CO": {
		ICAO:      "65CO",
		Name:      "Wkr Airport",
		City:      "Fort Collins",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4840,
		Latitude:  40.5208015442,
		Longitude: -104.9670028687,
		Timezone:  "America/Denver",
	},
	"65IL": {
		ICAO:      "65IL",
		Name:      "Far Field",
		City:      "Marengo",
		State:     "Illinois",
		Country:   "US",
		Elevation: 830,
		Latitude:  42.2009010315,
		Longitude: -88.6222991943,
		Timezone:  "America/Chicago",
	},
	"65IS": {
		ICAO:      "65IS",
		Name:      "Frings Airport",
		City:      "Leeds",
		State:     "Illinois",
		Country:   "US",
		Elevation: 675,
		Latitude:  41.0238990784,
		Longitude: -88.9837036133,
		Timezone:  "America/Chicago",
	},
	"65KS": {
		ICAO:      "65KS",
		Name:      "Griffith Field",
		City:      "Downs",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1500,
		Latitude:  39.507833,
		Longitude: -98.586167,
		Timezone:  "America/Chicago",
	},
	"65LA": {
		ICAO:      "65LA",
		IATA:      "BCS",
		Name:      "Southern Seaplane Airport",
		City:      "Belle Chasse",
		State:     "Louisiana",
		Country:   "US",
		Latitude:  29.8661003113,
		Longitude: -90.0222015381,
		Timezone:  "America/Chicago",
	},
	"65MN": {
		ICAO:      "65MN",
		Name:      "Ricks Field",
		City:      "Wendell",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1100,
		Latitude:  45.964099884,
		Longitude: -96.1498031616,
		Timezone:  "America/Chicago",
	},
	"65NM": {
		ICAO:      "65NM",
		Name:      "Curtis and Curtis Airport",
		City:      "Forrest",
		State:     "New Mexico",
		Country:   "US",
		Elevation: 4691,
		Latitude:  34.7266998291,
		Longitude: -103.6100006104,
		Timezone:  "America/Denver",
	},
	"65NY": {
		ICAO:      "65NY",
		Name:      "Chautauqua Lake Airpark",
		City:      "Mayville",
		State:     "New York",
		Country:   "US",
		Elevation: 1655,
		Latitude:  42.2839012146,
		Longitude: -79.443901062,
		Timezone:  "America/New_York",
	},
	"65OH": {
		ICAO:      "65OH",
		Name:      "Wiita Farms Airport",
		City:      "Sharon Center",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1220,
		Latitude:  41.0717010498,
		Longitude: -81.7511978149,
		Timezone:  "America/New_York",
	},
	"65PA": {
		ICAO:      "65PA",
		Name:      "Cherry Valley Airport",
		City:      "Saylorsburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 670,
		Latitude:  40.8959007263,
		Longitude: -75.2963027954,
		Timezone:  "America/New_York",
	},
	"65TA": {
		ICAO:      "65TA",
		Name:      "Flying C Ranch Airport",
		City:      "Independence",
		State:     "Texas",
		Country:   "US",
		Elevation: 250,
		Latitude:  30.3432998657,
		Longitude: -96.3071975708,
		Timezone:  "America/Chicago",
	},
	"65TE": {
		ICAO:      "65TE",
		Name:      "Windwood Farm Airport",
		City:      "Bells",
		State:     "Texas",
		Country:   "US",
		Elevation: 778,
		Latitude:  33.568167,
		Longitude: -96.397167,
		Timezone:  "America/Chicago",
	},
	"65TN": {
		ICAO:      "65TN",
		Name:      "Roach Farm Airport",
		City:      "Fall Branch",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1750,
		Latitude:  36.3342018127,
		Longitude: -82.6616973877,
		Timezone:  "America/New_York",
	},
	"65TS": {
		ICAO:      "65TS",
		Name:      "Becker Airport",
		City:      "Alvord",
		State:     "Texas",
		Country:   "US",
		Elevation: 940,
		Latitude:  33.4023017883,
		Longitude: -97.6708984375,
		Timezone:  "America/Chicago",
	},
	"65TX": {
		ICAO:      "65TX",
		Name:      "Flying Eagle Ranch Airport",
		City:      "Salt Flat",
		State:     "Texas",
		Country:   "US",
		Elevation: 4300,
		Latitude:  31.775133,
		Longitude: -105.431317,
		Timezone:  "America/Denver",
	},
	"65WA": {
		ICAO:      "65WA",
		Name:      "Wissler's Airport",
		City:      "Tenino",
		State:     "Washington",
		Country:   "US",
		Elevation: 255,
		Latitude:  46.8404006958,
		Longitude: -122.9160003662,
		Timezone:  "America/Los_Angeles",
	},
	"65WI": {
		ICAO:      "65WI",
		Name:      "Dulmes Field",
		City:      "Oostburg",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 720,
		Latitude:  43.6192016602,
		Longitude: -87.8219985962,
		Timezone:  "America/Chicago",
	},
	"65WN": {
		ICAO:      "65WN",
		Name:      "Whoopy Hollow Aerodrome",
		City:      "Wiota",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 925,
		Latitude:  42.676399231,
		Longitude: -89.93800354,
		Timezone:  "America/Chicago",
	},
	"65XS": {
		ICAO:      "65XS",
		Name:      "Birdnest Airport",
		City:      "Parker",
		State:     "Texas",
		Country:   "US",
		Elevation: 815,
		Latitude:  32.2232017517,
		Longitude: -97.2817001343,
		Timezone:  "America/Chicago",
	},
	"66CA": {
		ICAO:      "66CA",
		Name:      "Rancho San Simeon Airport",
		City:      "Cambria",
		State:     "California",
		Country:   "US",
		Elevation: 320,
		Latitude:  35.6077003479,
		Longitude: -121.1100006104,
		Timezone:  "America/Los_Angeles",
	},
	"66CL": {
		ICAO:      "66CL",
		Name:      "Triangle T Ranch Airport",
		City:      "Chowchilla",
		State:     "California",
		Country:   "US",
		Elevation: 130,
		Latitude:  36.9986000061,
		Longitude: -120.4649963379,
		Timezone:  "America/Los_Angeles",
	},
	"66FD": {
		ICAO:      "66FD",
		Name:      "J.R.'s STOLport",
		City:      "Crystal River",
		State:     "Florida",
		Country:   "US",
		Elevation: 50,
		Latitude:  29.0035991669,
		Longitude: -82.5552978516,
		Timezone:  "America/New_York",
	},
	"66GA": {
		ICAO:      "66GA",
		Name:      "Sawyer Farm Airport",
		City:      "Blakely",
		State:     "Georgia",
		Country:   "US",
		Elevation: 350,
		Latitude:  31.474722,
		Longitude: -85.002778,
		Timezone:  "America/New_York",
	},
	"66II": {
		ICAO:      "66II",
		Name:      "Pat Robinson Airport",
		City:      "Lapel",
		State:     "Indiana",
		Country:   "US",
		Elevation: 865,
		Latitude:  40.0421981812,
		Longitude: -85.8653030396,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"66IL": {
		ICAO:      "66IL",
		Name:      "Aerogrange Airport",
		City:      "Marengo",
		State:     "Illinois",
		Country:   "US",
		Elevation: 820,
		Latitude:  42.2691993713,
		Longitude: -88.5711975098,
		Timezone:  "America/Chicago",
	},
	"66IN": {
		ICAO:      "66IN",
		Name:      "Oakes Field",
		City:      "Charlottesville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 935,
		Latitude:  39.7874984741,
		Longitude: -85.6369018555,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"66KS": {
		ICAO:      "66KS",
		Name:      "Patterson Farms Airport",
		City:      "Independence",
		State:     "Kansas",
		Country:   "US",
		Elevation: 880,
		Latitude:  37.163898468,
		Longitude: -95.8722000122,
		Timezone:  "America/Chicago",
	},
	"66KY": {
		ICAO:      "66KY",
		Name:      "Mc Grew Airport",
		City:      "Clarkson",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 720,
		Latitude:  37.507598877,
		Longitude: -86.2054977417,
		Timezone:  "America/Chicago",
	},
	"66LA": {
		ICAO:      "66LA",
		Name:      "Schexnayder Airport",
		City:      "Erwinville",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 15,
		Latitude:  30.5104999542,
		Longitude: -91.3923034668,
		Timezone:  "America/Chicago",
	},
	"66MI": {
		ICAO:      "66MI",
		Name:      "East Lake Airport",
		City:      "Kalamazoo",
		State:     "Michigan",
		Country:   "US",
		Elevation: 860,
		Latitude:  42.2284011841,
		Longitude: -85.4843978882,
		Timezone:  "America/Detroit",
	},
	"66MN": {
		ICAO:      "66MN",
		Name:      "Howard's Airport",
		City:      "Clara City",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1000,
		Latitude:  44.9771995544,
		Longitude: -95.4227981567,
		Timezone:  "America/Chicago",
	},
	"66MO": {
		ICAO:      "66MO",
		Name:      "Ahlers Acres Airport",
		City:      "Foristell",
		State:     "Missouri",
		Country:   "US",
		Elevation: 660,
		Latitude:  38.8320007324,
		Longitude: -90.9512023926,
		Timezone:  "America/Chicago",
	},
	"66NC": {
		ICAO:      "66NC",
		Name:      "Hood Field",
		City:      "Dover",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 59,
		Latitude:  35.1744003296,
		Longitude: -77.4835968018,
		Timezone:  "America/New_York",
	},
	"66NY": {
		ICAO:      "66NY",
		Name:      "Maple Ridge Airport",
		City:      "Medina",
		State:     "New York",
		Country:   "US",
		Elevation: 646,
		Latitude:  43.2028007507,
		Longitude: -78.3538970947,
		Timezone:  "America/New_York",
	},
	"66OI": {
		ICAO:      "66OI",
		Name:      "Gorman-Freeman Airport",
		City:      "Wooster",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1210,
		Latitude:  40.7813987732,
		Longitude: -82.0557022095,
		Timezone:  "America/New_York",
	},
	"66OK": {
		ICAO:      "66OK",
		Name:      "Mc Laughlin Farm Airport",
		City:      "Durant",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 695,
		Latitude:  34.0890007019,
		Longitude: -96.4216995239,
		Timezone:  "America/Chicago",
	},
	"66TE": {
		ICAO:      "66TE",
		Name:      "The Landings Airport",
		City:      "Granbury",
		State:     "Texas",
		Country:   "US",
		Elevation: 684,
		Latitude:  32.366111,
		Longitude: -97.644444,
		Timezone:  "America/Chicago",
	},
	"66TX": {
		ICAO:      "66TX",
		Name:      "Loma de Cometa Airport",
		City:      "Cometa",
		State:     "Texas",
		Country:   "US",
		Elevation: 615,
		Latitude:  28.6821537,
		Longitude: -100.0237885,
		Timezone:  "America/Chicago",
	},
	"66WA": {
		ICAO:      "66WA",
		Name:      "Trout Lake Airport",
		City:      "Trout Lake",
		State:     "Washington",
		Country:   "US",
		Elevation: 1914,
		Latitude:  46.0023002625,
		Longitude: -121.5230026245,
		Timezone:  "America/Los_Angeles",
	},
	"66WI": {
		ICAO:      "66WI",
		Name:      "J-3 Cub Field",
		City:      "Watertown",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 860,
		Latitude:  43.0774993896,
		Longitude: -88.662902832,
		Timezone:  "America/Chicago",
	},
	"66XS": {
		ICAO:      "66XS",
		Name:      "Baylie Airport",
		City:      "Westminster",
		State:     "Texas",
		Country:   "US",
		Elevation: 685,
		Latitude:  33.3964996338,
		Longitude: -96.4886016846,
		Timezone:  "America/Chicago",
	},
	"67AK": {
		ICAO:      "67AK",
		Name:      "South Hollywood Airport",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 375,
		Latitude:  61.5345993042,
		Longitude: -149.6699981689,
		Timezone:  "America/Anchorage",
	},
	"67AR": {
		ICAO:      "67AR",
		Name:      "Wedington Woods Airport",
		City:      "Fayetteville",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1090,
		Latitude:  36.0978012085,
		Longitude: -94.3075027466,
		Timezone:  "America/Chicago",
	},
	"67CA": {
		ICAO:      "67CA",
		Name:      "Chapman Farms Airport",
		City:      "Chowchilla",
		State:     "California",
		Country:   "US",
		Elevation: 175,
		Latitude:  37.0727005005,
		Longitude: -120.3779983521,
		Timezone:  "America/Los_Angeles",
	},
	"67CL": {
		ICAO:      "67CL",
		Name:      "Davis Airport",
		City:      "Colusa",
		State:     "California",
		Country:   "US",
		Elevation: 45,
		Latitude:  39.2010002136,
		Longitude: -122.0490036011,
		Timezone:  "America/Los_Angeles",
	},
	"67CO": {
		ICAO:      "67CO",
		Name:      "Highline Farm Airstrip",
		City:      "Bracewell",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4745,
		Latitude:  40.4715003967,
		Longitude: -104.81300354,
		Timezone:  "America/Denver",
	},
	"67FL": {
		ICAO:      "67FL",
		Name:      "Myakka Head Airport",
		City:      "Zolfo Springs",
		State:     "Florida",
		Country:   "US",
		Elevation: 80,
		Latitude:  27.4556007385,
		Longitude: -82.1050033569,
		Timezone:  "America/New_York",
	},
	"67GA": {
		ICAO:      "67GA",
		Name:      "Apalachee Bluff Airpark",
		City:      "Monroe",
		State:     "Georgia",
		Country:   "US",
		Elevation: 735,
		Latitude:  33.8592987061,
		Longitude: -83.5876998901,
		Timezone:  "America/New_York",
	},
	"67II": {
		ICAO:      "67II",
		Name:      "Schoettmer Farm Airport",
		City:      "Greensburg",
		State:     "Indiana",
		Country:   "US",
		Elevation: 875,
		Latitude:  39.3713989258,
		Longitude: -85.5922012329,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"67IL": {
		ICAO:      "67IL",
		Name:      "Mitchell RLA Restricted Landing Area",
		City:      "Marseilles",
		State:     "Illinois",
		Country:   "US",
		Elevation: 705,
		Latitude:  41.3666992188,
		Longitude: -88.650100708,
		Timezone:  "America/Chicago",
	},
	"67IN": {
		ICAO:      "67IN",
		Name:      "Smitty's Soaring Airport",
		City:      "Knox",
		State:     "Indiana",
		Country:   "US",
		Elevation: 720,
		Latitude:  41.2364006042,
		Longitude: -86.5942001343,
		Timezone:  "America/Indiana/Knox",
	},
	"67KS": {
		ICAO:      "67KS",
		Name:      "Montezuma Coop Airport",
		City:      "Montezuma",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2780,
		Latitude:  37.5875015259,
		Longitude: -100.4349975586,
		Timezone:  "America/Chicago",
	},
	"67KY": {
		ICAO:      "67KY",
		Name:      "Estes Airport",
		City:      "Florence",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 890,
		Latitude:  38.9833984375,
		Longitude: -84.6660995483,
		Timezone:  "America/New_York",
	},
	"67LL": {
		ICAO:      "67LL",
		Name:      "County Poor Farm Airport",
		City:      "Oregon",
		State:     "Illinois",
		Country:   "US",
		Elevation: 695,
		Latitude:  41.9902992249,
		Longitude: -89.3314971924,
		Timezone:  "America/Chicago",
	},
	"67MN": {
		ICAO:      "67MN",
		Name:      "Pagel's Field",
		City:      "Atwater",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1170,
		Latitude:  45.2019004822,
		Longitude: -94.8011016846,
		Timezone:  "America/Chicago",
	},
	"67MO": {
		ICAO:      "67MO",
		Name:      "Ski Harbor Airport",
		City:      "Pittsburg",
		State:     "Missouri",
		Country:   "US",
		Elevation: 906,
		Latitude:  37.8109016418,
		Longitude: -93.3699035645,
		Timezone:  "America/Chicago",
	},
	"67NC": {
		ICAO:      "67NC",
		Name:      "Hightower Areo Plantation Airport",
		City:      "Wadesboro",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 395,
		Latitude:  34.9700012207,
		Longitude: -80.0289001465,
		Timezone:  "America/New_York",
	},
	"67ND": {
		ICAO:      "67ND",
		Name:      "Waldie Farms Airport",
		City:      "Marion",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1456,
		Latitude:  46.5433006287,
		Longitude: -98.3823013306,
		Timezone:  "America/Chicago",
	},
	"67NE": {
		ICAO:      "67NE",
		Name:      "L J Bose Airstrip",
		City:      "Orleans",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2000,
		Latitude:  40.1370010376,
		Longitude: -99.4983978271,
		Timezone:  "America/Chicago",
	},
	"67NJ": {
		ICAO:      "67NJ",
		Name:      "Mount Pleasant Landing Strip",
		City:      "Hainesburg",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 800,
		Latitude:  40.9697990417,
		Longitude: -75.0668029785,
		Timezone:  "America/New_York",
	},
	"67NY": {
		ICAO:      "67NY",
		Name:      "Ultralight Port Ultralightport",
		City:      "Little Valley",
		State:     "New York",
		Country:   "US",
		Elevation: 1860,
		Latitude:  42.2930984497,
		Longitude: -78.7900009155,
		Timezone:  "America/New_York",
	},
	"67OG": {
		ICAO:      "67OG",
		Name:      "George Airport",
		City:      "Sheridan",
		State:     "Oregon",
		Country:   "US",
		Elevation: 440,
		Latitude:  45.031389,
		Longitude: -123.406111,
		Timezone:  "America/Los_Angeles",
	},
	"67OH": {
		ICAO:      "67OH",
		Name:      "Harper Ridge Airport",
		City:      "Solon",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1235,
		Latitude:  41.413898468,
		Longitude: -81.4525985718,
		Timezone:  "America/New_York",
	},
	"67OK": {
		ICAO:      "67OK",
		Name:      "Flying J Ranch Airport",
		City:      "Peggs",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 990,
		Latitude:  36.0331001282,
		Longitude: -95.0780029297,
		Timezone:  "America/Chicago",
	},
	"67OR": {
		ICAO:      "67OR",
		Name:      "Mc Gee Airport",
		City:      "Donald",
		State:     "Oregon",
		Country:   "US",
		Elevation: 175,
		Latitude:  45.2336997986,
		Longitude: -122.856002808,
		Timezone:  "America/Los_Angeles",
	},
	"67PN": {
		ICAO:      "67PN",
		Name:      "Masser Field",
		City:      "Sacramento",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 720,
		Latitude:  40.6333999634,
		Longitude: -76.5969009399,
		Timezone:  "America/New_York",
	},
	"67TE": {
		ICAO:      "67TE",
		Name:      "Fisher Ranch Airport",
		City:      "Killeen",
		State:     "Texas",
		Country:   "US",
		Elevation: 870,
		Latitude:  30.9496002197,
		Longitude: -97.7966995239,
		Timezone:  "America/Chicago",
	},
	"67TX": {
		ICAO:      "67TX",
		Name:      "Old Hoppe Place Airport",
		City:      "Agua Dulce",
		State:     "Texas",
		Country:   "US",
		Elevation: 96,
		Latitude:  27.8003005981,
		Longitude: -97.8511962891,
		Timezone:  "America/Chicago",
	},
	"67VA": {
		ICAO:      "67VA",
		Name:      "Karmy's Airport",
		City:      "Woodstock",
		State:     "Virginia",
		Country:   "US",
		Elevation: 890,
		Latitude:  38.837600708,
		Longitude: -78.5195007324,
		Timezone:  "America/New_York",
	},
	"67WA": {
		ICAO:      "67WA",
		Name:      "Page Airport",
		City:      "Walla Walla",
		State:     "Washington",
		Country:   "US",
		Elevation: 800,
		Latitude:  46.0164985657,
		Longitude: -118.371002197,
		Timezone:  "America/Los_Angeles",
	},
	"67WI": {
		ICAO:      "67WI",
		Name:      "Accurate Airport",
		City:      "Necedah",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 890,
		Latitude:  43.9710998535,
		Longitude: -89.9971008301,
		Timezone:  "America/Chicago",
	},
	"68AZ": {
		ICAO:      "68AZ",
		Name:      "Music Mountain Air Ranch Airport",
		City:      "Hackberry",
		State:     "Arizona",
		Country:   "US",
		Elevation: 3450,
		Latitude:  35.4751014709,
		Longitude: -113.7689971924,
		Timezone:  "America/Phoenix",
	},
	"68CA": {
		ICAO:      "68CA",
		Name:      "Lake California Air Park",
		City:      "Cottonwood",
		State:     "California",
		Country:   "US",
		Elevation: 615,
		Latitude:  40.3610992432,
		Longitude: -122.216003418,
		Timezone:  "America/Los_Angeles",
	},
	"68CL": {
		ICAO:      "68CL",
		Name:      "El Rico Airport",
		City:      "Corcoran",
		State:     "California",
		Country:   "US",
		Elevation: 181,
		Latitude:  36.0457992554,
		Longitude: -119.6470031738,
		Timezone:  "America/Los_Angeles",
	},
	"68CN": {
		ICAO:      "68CN",
		Name:      "Porter Ranch Airport",
		City:      "Kennedy Meadows",
		State:     "California",
		Country:   "US",
		Elevation: 6800,
		Latitude:  36.0222015381,
		Longitude: -118.0950012207,
		Timezone:  "America/Los_Angeles",
	},
	"68CO": {
		ICAO:      "68CO",
		Name:      "Singleton Ranch Airport",
		City:      "Byers",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4940,
		Latitude:  39.8722000122,
		Longitude: -104.1529998779,
		Timezone:  "America/Denver",
	},
	"68FD": {
		ICAO:      "68FD",
		Name:      "Maran Airport",
		City:      "Fountain",
		State:     "Florida",
		Country:   "US",
		Elevation: 225,
		Latitude:  30.5221996307,
		Longitude: -85.376701355,
		Timezone:  "America/Chicago",
	},
	"68ID": {
		ICAO:      "68ID",
		Name:      "Rapoport Ranch Airport",
		City:      "Sandpoint",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2170,
		Latitude:  48.25,
		Longitude: -116.466667,
		Timezone:  "America/Los_Angeles",
	},
	"68IL": {
		ICAO:      "68IL",
		Name:      "Prairie Lake Hunt Club Airport",
		City:      "Marseilles",
		State:     "Illinois",
		Country:   "US",
		Elevation: 750,
		Latitude:  41.3950004578,
		Longitude: -88.6828994751,
		Timezone:  "America/Chicago",
	},
	"68IN": {
		ICAO:      "68IN",
		Name:      "Alley Oop Airport",
		City:      "Wheatfield",
		State:     "Indiana",
		Country:   "US",
		Elevation: 715,
		Latitude:  41.1213989258,
		Longitude: -86.9974975586,
		Timezone:  "America/Chicago",
	},
	"68IS": {
		ICAO:      "68IS",
		Name:      "Casa De Aero Park Airport",
		City:      "Hampshire",
		State:     "Illinois",
		Country:   "US",
		Elevation: 860,
		Latitude:  42.148601532,
		Longitude: -88.5597991943,
		Timezone:  "America/Chicago",
	},
	"68KS": {
		ICAO:      "68KS",
		Name:      "Cloud 9 Airport",
		City:      "Paola",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1050,
		Latitude:  38.567199707,
		Longitude: -94.7825012207,
		Timezone:  "America/Chicago",
	},
	"68KY": {
		ICAO:      "68KY",
		Name:      "Lee's Airpark",
		City:      "Somerset",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 970,
		Latitude:  37.0233001709,
		Longitude: -84.7296981812,
		Timezone:  "America/New_York",
	},
	"68LA": {
		ICAO:      "68LA",
		Name:      "Pilkinton Airstrip",
		City:      "Bossier City",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 148,
		Latitude:  32.3306007385,
		Longitude: -93.5171966553,
		Timezone:  "America/Chicago",
	},
	"68MI": {
		ICAO:      "68MI",
		Name:      "Saline Airport",
		City:      "Saline",
		State:     "Michigan",
		Country:   "US",
		Elevation: 820,
		Latitude:  42.1464004517,
		Longitude: -83.7919006348,
		Timezone:  "America/Detroit",
	},
	"68MN": {
		ICAO:      "68MN",
		Name:      "Mathew Private Field",
		City:      "Barnesville",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 965,
		Latitude:  46.6712989807,
		Longitude: -96.4508972168,
		Timezone:  "America/Chicago",
	},
	"68NC": {
		ICAO:      "68NC",
		Name:      "Winstead '76' Airport",
		City:      "Leasburg",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 615,
		Latitude:  36.3996009827,
		Longitude: -79.1280975342,
		Timezone:  "America/New_York",
	},
	"68NE": {
		ICAO:      "68NE",
		Name:      "Hall-Feld Airport",
		City:      "Davey",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1355,
		Latitude:  40.9914016724,
		Longitude: -96.6196975708,
		Timezone:  "America/Chicago",
	},
	"68NM": {
		ICAO:      "68NM",
		Name:      "Flying H Ranch Airport",
		City:      "Hope",
		State:     "New Mexico",
		Country:   "US",
		Elevation: 5162,
		Latitude:  33.0287017822,
		Longitude: -105.1269989014,
		Timezone:  "America/Denver",
	},
	"68NY": {
		ICAO:      "68NY",
		Name:      "Merrimac Farms Airport",
		City:      "Mount Morris",
		State:     "New York",
		Country:   "US",
		Elevation: 910,
		Latitude:  42.7500991821,
		Longitude: -77.7838973999,
		Timezone:  "America/New_York",
	},
	"68OR": {
		ICAO:      "68OR",
		Name:      "Lyda Ranch Airstrip",
		City:      "Dufur",
		State:     "Oregon",
		Country:   "US",
		Elevation: 2450,
		Latitude:  45.3911018372,
		Longitude: -121.2760009766,
		Timezone:  "America/Los_Angeles",
	},
	"68PA": {
		ICAO:      "68PA",
		Name:      "Don's Place Airpark",
		City:      "Hamburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 510,
		Latitude:  40.5181999207,
		Longitude: -75.9162979126,
		Timezone:  "America/New_York",
	},
	"68TE": {
		ICAO:      "68TE",
		Name:      "Norris Raun Ranch Airport",
		City:      "El Campo",
		State:     "Texas",
		Country:   "US",
		Elevation: 108,
		Latitude:  29.196100235,
		Longitude: -96.491897583,
		Timezone:  "America/Chicago",
	},
	"68TS": {
		ICAO:      "68TS",
		Name:      "Bishop Field",
		City:      "Royce City",
		State:     "Texas",
		Country:   "US",
		Elevation: 550,
		Latitude:  32.8750991821,
		Longitude: -96.2878036499,
		Timezone:  "America/Chicago",
	},
	"68TX": {
		ICAO:      "68TX",
		Name:      "San Patricio International Airport",
		City:      "San Patricio",
		State:     "Texas",
		Country:   "US",
		Elevation: 50,
		Latitude:  27.9808006287,
		Longitude: -97.7869033813,
		Timezone:  "America/Chicago",
	},
	"68WA": {
		ICAO:      "68WA",
		Name:      "Mc Mahon Field",
		City:      "Zillah",
		State:     "Washington",
		Country:   "US",
		Elevation: 850,
		Latitude:  46.393699646,
		Longitude: -120.222000122,
		Timezone:  "America/Los_Angeles",
	},
	"68XS": {
		ICAO:      "68XS",
		Name:      "Margaritaville Airport",
		City:      "Whitesboro",
		State:     "Texas",
		Country:   "US",
		Elevation: 750,
		Latitude:  33.7212982178,
		Longitude: -96.8697967529,
		Timezone:  "America/Chicago",
	},
	"69CL": {
		ICAO:      "69CL",
		Name:      "Medlock Field",
		City:      "Davis",
		State:     "California",
		Country:   "US",
		Elevation: 48,
		Latitude:  38.6155014038,
		Longitude: -121.7470016479,
		Timezone:  "America/Los_Angeles",
	},
	"69FD": {
		ICAO:      "69FD",
		Name:      "Unicorn Place Airport",
		City:      "Defuniak Springs",
		State:     "Florida",
		Country:   "US",
		Elevation: 260,
		Latitude:  30.8446998596,
		Longitude: -86.2814025879,
		Timezone:  "America/Chicago",
	},
	"69FL": {
		ICAO:      "69FL",
		Name:      "Eagle's Landing Airport",
		City:      "Fort Myers",
		State:     "Florida",
		Country:   "US",
		Elevation: 10,
		Latitude:  26.6772994995,
		Longitude: -82.1364974976,
		Timezone:  "America/New_York",
	},
	"69GE": {
		ICAO:      "69GE",
		Name:      "Great Oaks Airport",
		City:      "Tignall",
		State:     "Georgia",
		Country:   "US",
		Elevation: 530,
		Latitude:  33.9230003357,
		Longitude: -82.6529006958,
		Timezone:  "America/New_York",
	},
	"69II": {
		ICAO:      "69II",
		Name:      "Greuter Field",
		City:      "Butler",
		State:     "Indiana",
		Country:   "US",
		Elevation: 840,
		Latitude:  41.3694000244,
		Longitude: -84.8788986206,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"69IL": {
		ICAO:      "69IL",
		Name:      "David Gillespie Airport",
		City:      "Marseilles",
		State:     "Illinois",
		Country:   "US",
		Elevation: 730,
		Latitude:  41.3652992249,
		Longitude: -88.6791992188,
		Timezone:  "America/Chicago",
	},
	"69IS": {
		ICAO:      "69IS",
		Name:      "Sinele's Sunset Strip",
		City:      "Niota",
		State:     "Illinois",
		Country:   "US",
		Elevation: 690,
		Latitude:  40.579498291,
		Longitude: -91.2918014526,
		Timezone:  "America/Chicago",
	},
	"69KS": {
		ICAO:      "69KS",
		Name:      "Chiles Airpark",
		City:      "Chiles",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1050,
		Latitude:  38.6697006226,
		Longitude: -94.7425003052,
		Timezone:  "America/Chicago",
	},
	"69LL": {
		ICAO:      "69LL",
		Name:      "Maas Airstrip",
		City:      "New Boston",
		State:     "Illinois",
		Country:   "US",
		Elevation: 534,
		Latitude:  41.2869987488,
		Longitude: -91.0487976074,
		Timezone:  "America/Chicago",
	},
	"69MI": {
		ICAO:      "69MI",
		Name:      "Perry Aero Park",
		City:      "Perry",
		State:     "Michigan",
		Country:   "US",
		Elevation: 855,
		Latitude:  42.821389,
		Longitude: -84.278889,
		Timezone:  "America/Detroit",
	},
	"69MN": {
		ICAO:      "69MN",
		Name:      "Yaggie Private Airport",
		City:      "Breckenridge",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 982,
		Latitude:  46.2776985168,
		Longitude: -96.4294967651,
		Timezone:  "America/Chicago",
	},
	"69MO": {
		ICAO:      "69MO",
		Name:      "Hess-Mckeown Airport",
		City:      "Higbee",
		State:     "Missouri",
		Country:   "US",
		Elevation: 885,
		Latitude:  39.3083992004,
		Longitude: -92.4541015625,
		Timezone:  "America/Chicago",
	},
	"69MY": {
		ICAO:      "69MY",
		Name:      "Hall Airport",
		City:      "Farmington",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 875,
		Latitude:  44.6610984802,
		Longitude: -93.1001968384,
		Timezone:  "America/Chicago",
	},
	"69NC": {
		ICAO:      "69NC",
		Name:      "Clute's Hilltop Airport",
		City:      "Forest City",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 1078,
		Latitude:  35.4339981079,
		Longitude: -81.8511962891,
		Timezone:  "America/New_York",
	},
	"69ND": {
		ICAO:      "69ND",
		Name:      "Tengesdal Airport",
		City:      "Maxbass",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1495,
		Latitude:  48.7430992126,
		Longitude: -101.0419998169,
		Timezone:  "America/Chicago",
	},
	"69OH": {
		ICAO:      "69OH",
		Name:      "Dwight Field",
		City:      "Homer",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1025,
		Latitude:  40.2630996704,
		Longitude: -82.5143966675,
		Timezone:  "America/New_York",
	},
	"69OI": {
		ICAO:      "69OI",
		Name:      "Schulze's Airport",
		City:      "Chatfield",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1000,
		Latitude:  40.9500999451,
		Longitude: -82.9248962402,
		Timezone:  "America/New_York",
	},
	"69OK": {
		ICAO:      "69OK",
		Name:      "Green Country Airpark",
		City:      "Fairland",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 840,
		Latitude:  36.7286987305,
		Longitude: -94.8505020142,
		Timezone:  "America/Chicago",
	},
	"69OR": {
		ICAO:      "69OR",
		Name:      "Dick Fisher Airport",
		City:      "Gaston",
		State:     "Oregon",
		Country:   "US",
		Elevation: 300,
		Latitude:  45.4126014709,
		Longitude: -123.1370010376,
		Timezone:  "America/Los_Angeles",
	},
	"69PA": {
		ICAO:      "69PA",
		Name:      "Hurst STOLport",
		City:      "Newmanstown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 540,
		Latitude:  40.3428993225,
		Longitude: -76.2093963623,
		Timezone:  "America/New_York",
	},
	"69TA": {
		ICAO:      "69TA",
		Name:      "Dean Airport",
		City:      "Corpus Christi",
		State:     "Texas",
		Country:   "US",
		Elevation: 45,
		Latitude:  27.6550006866,
		Longitude: -97.5252990723,
		Timezone:  "America/Chicago",
	},
	"69TE": {
		ICAO:      "69TE",
		Name:      "Deer Pasture Airport",
		City:      "Lampasas",
		State:     "Texas",
		Country:   "US",
		Elevation: 1422,
		Latitude:  30.9263000488,
		Longitude: -98.2363967896,
		Timezone:  "America/Chicago",
	},
	"69TS": {
		ICAO:      "69TS",
		Name:      "White Airport",
		City:      "Killeen",
		State:     "Texas",
		Country:   "US",
		Elevation: 890,
		Latitude:  30.9988002777,
		Longitude: -97.755897522,
		Timezone:  "America/Chicago",
	},
	"69TX": {
		ICAO:      "69TX",
		Name:      "Green Lake Ranch Airport",
		City:      "Bloomington",
		State:     "Texas",
		Country:   "US",
		Elevation: 50,
		Latitude:  28.5835990906,
		Longitude: -96.8544006348,
		Timezone:  "America/Chicago",
	},
	"69VA": {
		ICAO:      "69VA",
		Name:      "Meadow Farm Airport",
		City:      "Ashland",
		State:     "Virginia",
		Country:   "US",
		Elevation: 75,
		Latitude:  37.8571014404,
		Longitude: -77.420501709,
		Timezone:  "America/New_York",
	},
	"69XA": {
		ICAO:      "69XA",
		Name:      "Richey Airfield",
		City:      "Chico",
		State:     "Texas",
		Country:   "US",
		Elevation: 950,
		Latitude:  33.349956,
		Longitude: -97.776206,
		Timezone:  "America/Chicago",
	},
	"69XS": {
		ICAO:      "69XS",
		Name:      "Brushy Creek Airport",
		City:      "Whitesboro",
		State:     "Texas",
		Country:   "US",
		Elevation: 715,
		Latitude:  33.7533988953,
		Longitude: -96.8302001953,
		Timezone:  "America/Chicago",
	},
	"6AK1": {
		ICAO:      "6AK1",
		Name:      "Platinum Mine Airport",
		City:      "Platinum",
		State:     "Alaska",
		Country:   "US",
		Elevation: 100,
		Latitude:  58.9117012024,
		Longitude: -161.7140045166,
		Timezone:  "America/Anchorage",
	},
	"6AK2": {
		ICAO:      "6AK2",
		Name:      "Sleepers Strip",
		City:      "Point Mackenzie",
		State:     "Alaska",
		Country:   "US",
		Elevation: 125,
		Latitude:  61.251499176,
		Longitude: -149.9819946289,
		Timezone:  "America/Anchorage",
	},
	"6AK3": {
		ICAO:      "6AK3",
		Name:      "Butler Aviation Airport",
		City:      "Kenai",
		State:     "Alaska",
		Country:   "US",
		Elevation: 185,
		Latitude:  60.7294006348,
		Longitude: -151.2779998779,
		Timezone:  "America/Anchorage",
	},
	"6AK5": {
		ICAO:      "6AK5",
		Name:      "Fire Island Airport",
		City:      "Anchorage",
		State:     "Alaska",
		Country:   "US",
		Elevation: 55,
		Latitude:  61.1683006287,
		Longitude: -150.1609954834,
		Timezone:  "America/Anchorage",
	},
	"6AK7": {
		ICAO:      "6AK7",
		Name:      "Rainbow Heights Estates Airstrip",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 270,
		Latitude:  61.5746994019,
		Longitude: -149.6349945068,
		Timezone:  "America/Anchorage",
	},
	"6AK8": {
		ICAO:      "6AK8",
		Name:      "Tulakes Airport",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 350,
		Latitude:  61.6180992126,
		Longitude: -149.6690063477,
		Timezone:  "America/Anchorage",
	},
	"6AK9": {
		ICAO:      "6AK9",
		Name:      "Cizek North Airport",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 300,
		Latitude:  61.6334991455,
		Longitude: -149.6820068359,
		Timezone:  "America/Anchorage",
	},
	"6AL3": {
		ICAO:      "6AL3",
		Name:      "Sweet Home Airport",
		City:      "Henagar",
		State:     "Alabama",
		Country:   "US",
		Elevation: 1340,
		Latitude:  34.582047,
		Longitude: -85.755572,
		Timezone:  "America/Chicago",
	},
	"6AL6": {
		ICAO:      "6AL6",
		Name:      "Hawthorn Pines Airport",
		City:      "Chatom",
		State:     "Alabama",
		Country:   "US",
		Elevation: 150,
		Latitude:  31.3885993958,
		Longitude: -88.0925979614,
		Timezone:  "America/Chicago",
	},
	"6AR0": {
		ICAO:      "6AR0",
		Name:      "Cypress Creek Airpark",
		City:      "El Paso",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 270,
		Latitude:  35.0542984009,
		Longitude: -92.1171035767,
		Timezone:  "America/Chicago",
	},
	"6AR1": {
		ICAO:      "6AR1",
		Name:      "Elder Airstrip",
		City:      "Cave Springs",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1300,
		Latitude:  36.2779006958,
		Longitude: -94.1966018677,
		Timezone:  "America/Chicago",
	},
	"6AR2": {
		ICAO:      "6AR2",
		Name:      "A J's Airport",
		City:      "Casa",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 370,
		Latitude:  35.020401001,
		Longitude: -93.0649032593,
		Timezone:  "America/Chicago",
	},
	"6AR5": {
		ICAO:      "6AR5",
		Name:      "Mc Donald's Airstrip",
		City:      "Calamine",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 375,
		Latitude:  36.0346984863,
		Longitude: -91.3707962036,
		Timezone:  "America/Chicago",
	},
	"6AR6": {
		ICAO:      "6AR6",
		Name:      "Two Rivers Airport",
		City:      "Little Rock",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 265,
		Latitude:  34.8405990601,
		Longitude: -92.4417037964,
		Timezone:  "America/Chicago",
	},
	"6AR8": {
		ICAO:      "6AR8",
		Name:      "Flying W Airpark II Airport",
		City:      "Charleston",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 474,
		Latitude:  35.3300018311,
		Longitude: -94.0419006348,
		Timezone:  "America/Chicago",
	},
	"6AR9": {
		ICAO:      "6AR9",
		Name:      "Pine Mountain Airpark",
		City:      "Flippin",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 880,
		Latitude:  36.4152984619,
		Longitude: -92.6943969727,
		Timezone:  "America/Chicago",
	},
	"6AZ2": {
		ICAO:      "6AZ2",
		Name:      "Wisky Ranch/Chevlon Airport",
		City:      "Heber",
		State:     "Arizona",
		Country:   "US",
		Elevation: 6172,
		Latitude:  34.6167984009,
		Longitude: -110.6279983521,
		Timezone:  "America/Phoenix",
	},
	"6AZ5": {
		ICAO:      "6AZ5",
		Name:      "Ash Creek Airport",
		City:      "Pearce",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4680,
		Latitude:  31.857483,
		Longitude: -109.542317,
		Timezone:  "America/Phoenix",
	},
	"6AZ8": {
		ICAO:      "6AZ8",
		Name:      "Flying Diamond Airport",
		City:      "Tucson",
		State:     "Arizona",
		Country:   "US",
		Elevation: 3800,
		Latitude:  31.9410991669,
		Longitude: -111.1200027466,
		Timezone:  "America/Phoenix",
	},
	"6CA1": {
		ICAO:      "6CA1",
		Name:      "Triple R Ranch Airport",
		City:      "Macdoel",
		State:     "California",
		Country:   "US",
		Elevation: 4380,
		Latitude:  41.7426986694,
		Longitude: -121.8779983521,
		Timezone:  "America/Los_Angeles",
	},
	"6CA5": {
		ICAO:      "6CA5",
		Name:      "Valley Vista Airport",
		City:      "Yucca Valley",
		State:     "California",
		Country:   "US",
		Elevation: 3370,
		Latitude:  34.3372001648,
		Longitude: -116.5800018311,
		Timezone:  "America/Los_Angeles",
	},
	"6CA6": {
		ICAO:      "6CA6",
		Name:      "Eagle Ridge Ranch Airport",
		City:      "Railroad Flat",
		State:     "California",
		Country:   "US",
		Elevation: 2870,
		Latitude:  38.3277015686,
		Longitude: -120.4769973755,
		Timezone:  "America/Los_Angeles",
	},
	"6CA7": {
		ICAO:      "6CA7",
		Name:      "J&J Crop Dusters Inc Airport",
		City:      "Arvin",
		State:     "California",
		Country:   "US",
		Elevation: 388,
		Latitude:  35.1800003052,
		Longitude: -118.8550033569,
		Timezone:  "America/Los_Angeles",
	},
	"6CA8": {
		ICAO:      "6CA8",
		Name:      "Depue Airport",
		City:      "Lenwood",
		State:     "California",
		Country:   "US",
		Elevation: 2313,
		Latitude:  34.8563995361,
		Longitude: -117.1370010376,
		Timezone:  "America/Los_Angeles",
	},
	"6CL0": {
		ICAO:      "6CL0",
		Name:      "Majors Airport",
		City:      "Bakersfield",
		State:     "California",
		Country:   "US",
		Elevation: 640,
		Latitude:  35.5261001587,
		Longitude: -119.1019973755,
		Timezone:  "America/Los_Angeles",
	},
	"6CL4": {
		ICAO:      "6CL4",
		Name:      "Manzanita Airport",
		City:      "Mariposa",
		State:     "California",
		Country:   "US",
		Elevation: 3020,
		Latitude:  37.459400177,
		Longitude: -119.7419967651,
		Timezone:  "America/Los_Angeles",
	},
	"6CL6": {
		ICAO:      "6CL6",
		Name:      "Mark Franz Private Strip",
		City:      "O'Neals",
		State:     "California",
		Country:   "US",
		Elevation: 2560,
		Latitude:  37.1749000549,
		Longitude: -119.6090011597,
		Timezone:  "America/Los_Angeles",
	},
	"6CL8": {
		ICAO:      "6CL8",
		Name:      "Harley Airport",
		City:      "Stockton",
		State:     "California",
		Country:   "US",
		Elevation: 20,
		Latitude:  37.9508018494,
		Longitude: -121.2429962158,
		Timezone:  "America/Los_Angeles",
	},
	"6CL9": {
		ICAO:      "6CL9",
		Name:      "Peoria Airport",
		City:      "Jamestown",
		State:     "California",
		Country:   "US",
		Elevation: 830,
		Latitude:  37.9216995239,
		Longitude: -120.5149993896,
		Timezone:  "America/Los_Angeles",
	},
	"6CO0": {
		ICAO:      "6CO0",
		Name:      "Doctors Mesa Airport",
		City:      "Eckert/Orchard City",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5680,
		Latitude:  38.8527984619,
		Longitude: -108.016998291,
		Timezone:  "America/Denver",
	},
	"6CO1": {
		ICAO:      "6CO1",
		Name:      "Burnham Field",
		City:      "Wellington",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5615,
		Latitude:  40.7813987732,
		Longitude: -104.9169998169,
		Timezone:  "America/Denver",
	},
	"6CO2": {
		ICAO:      "6CO2",
		Name:      "Mesawood Airpark",
		City:      "Cedaredge",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7200,
		Latitude:  38.961101532,
		Longitude: -107.9580001831,
		Timezone:  "America/Denver",
	},
	"6CO3": {
		ICAO:      "6CO3",
		Name:      "Wine Glass International Airport",
		City:      "La Junta",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4708,
		Latitude:  37.6363983154,
		Longitude: -103.6579971313,
		Timezone:  "America/Denver",
	},
	"6CO4": {
		ICAO:      "6CO4",
		Name:      "Hat-Field STOLport",
		City:      "Fort Collins",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4885,
		Latitude:  40.5102996826,
		Longitude: -105.0009994507,
		Timezone:  "America/Denver",
	},
	"6CO6": {
		ICAO:      "6CO6",
		Name:      "Skywagon Ranch Airport",
		City:      "Berthoud",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5187,
		Latitude:  40.311886,
		Longitude: -105.145833,
		Timezone:  "America/Denver",
	},
	"6FD0": {
		ICAO:      "6FD0",
		Name:      "Windy Acres Airport",
		City:      "Chiefland",
		State:     "Florida",
		Country:   "US",
		Elevation: 45,
		Latitude:  29.5657997131,
		Longitude: -82.8686981201,
		Timezone:  "America/New_York",
	},
	"6FD2": {
		ICAO:      "6FD2",
		Name:      "Big Oaks Ranch Airport",
		City:      "Chuluota",
		State:     "Florida",
		Country:   "US",
		Elevation: 25,
		Latitude:  28.6669006348,
		Longitude: -81.0845031738,
		Timezone:  "America/New_York",
	},
	"6FD5": {
		ICAO:      "6FD5",
		Name:      "Blanket Bay Airport",
		City:      "Yeehaw Junction",
		State:     "Florida",
		Country:   "US",
		Elevation: 65,
		Latitude:  27.7786006927,
		Longitude: -81.0998001099,
		Timezone:  "America/New_York",
	},
	"6FD6": {
		ICAO:      "6FD6",
		Name:      "Britt Brown & Porter Ranch Airport",
		City:      "Eustis",
		State:     "Florida",
		Country:   "US",
		Elevation: 167,
		Latitude:  28.9050006866,
		Longitude: -81.4330978394,
		Timezone:  "America/New_York",
	},
	"6FD7": {
		ICAO:      "6FD7",
		Name:      "Thomson Airfield",
		City:      "Ellenton",
		State:     "Florida",
		Country:   "US",
		Elevation: 35,
		Latitude:  27.5442008972,
		Longitude: -82.5300979614,
		Timezone:  "America/New_York",
	},
	"6FL0": {
		ICAO:      "6FL0",
		Name:      "Seminole Lake Gliderport",
		City:      "Groveland",
		State:     "Florida",
		Country:   "US",
		Elevation: 120,
		Latitude:  28.4057998657,
		Longitude: -81.8378982544,
		Timezone:  "America/New_York",
	},
	"6FL2": {
		ICAO:      "6FL2",
		Name:      "Kilpatrick Farm Airport",
		City:      "Sneads",
		State:     "Florida",
		Country:   "US",
		Elevation: 160,
		Latitude:  30.727399826,
		Longitude: -84.9177017212,
		Timezone:  "America/Chicago",
	},
	"6FL3": {
		ICAO:      "6FL3",
		Name:      "Payson Ranch Airport",
		City:      "Punta Gorda",
		State:     "Florida",
		Country:   "US",
		Elevation: 62,
		Latitude:  26.9372997284,
		Longitude: -81.5670013428,
		Timezone:  "America/New_York",
	},
	"6FL4": {
		ICAO:      "6FL4",
		Name:      "Sylvanmir Farms Airport",
		City:      "Keaton Beach",
		State:     "Florida",
		Country:   "US",
		Elevation: 8,
		Latitude:  29.821100235,
		Longitude: -83.5731964111,
		Timezone:  "America/New_York",
	},
	"6FL8": {
		ICAO:      "6FL8",
		Name:      "Ames Field",
		City:      "Trenton",
		State:     "Florida",
		Country:   "US",
		Elevation: 64,
		Latitude:  29.5874996185,
		Longitude: -82.8712005615,
		Timezone:  "America/New_York",
	},
	"6FL9": {
		ICAO:      "6FL9",
		Name:      "Saw Whet Farms Airport",
		City:      "Bell",
		State:     "Florida",
		Country:   "US",
		Elevation: 50,
		Latitude:  29.6900005341,
		Longitude: -82.8712005615,
		Timezone:  "America/New_York",
	},
	"6GA0": {
		ICAO:      "6GA0",
		Name:      "Stoney Point Field",
		City:      "Cumming",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1150,
		Latitude:  34.132598877,
		Longitude: -84.1994018555,
		Timezone:  "America/New_York",
	},
	"6GA1": {
		ICAO:      "6GA1",
		Name:      "Fagundes Field",
		City:      "Haralson",
		State:     "Georgia",
		Country:   "US",
		Elevation: 864,
		Latitude:  33.1990013123,
		Longitude: -84.5838012695,
		Timezone:  "America/New_York",
	},
	"6GA4": {
		ICAO:      "6GA4",
		Name:      "Spring Valley Farm Airport",
		City:      "Loganville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 920,
		Latitude:  33.7971000671,
		Longitude: -83.8513031006,
		Timezone:  "America/New_York",
	},
	"6GA5": {
		ICAO:      "6GA5",
		Name:      "Kolibri Airport",
		City:      "Whitesburg",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1100,
		Latitude:  33.5782012939,
		Longitude: -84.9068984985,
		Timezone:  "America/New_York",
	},
	"6GA8": {
		ICAO:      "6GA8",
		Name:      "Flying W Farms Airport",
		City:      "Carrollton",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1010,
		Latitude:  33.5078010559,
		Longitude: -85.1856002808,
		Timezone:  "America/New_York",
	},
	"6GE2": {
		ICAO:      "6GE2",
		Name:      "Rainbow Field",
		City:      "Woodland",
		State:     "Georgia",
		Country:   "US",
		Elevation: 780,
		Latitude:  32.8328018188,
		Longitude: -84.5378036499,
		Timezone:  "America/New_York",
	},
	"6IA5": {
		ICAO:      "6IA5",
		Name:      "Davis Field",
		City:      "Dunkerton",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1005,
		Latitude:  42.6166992188,
		Longitude: -92.2169036865,
		Timezone:  "America/Chicago",
	},
	"6IA6": {
		ICAO:      "6IA6",
		Name:      "Leise Airport",
		City:      "Albion",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1000,
		Latitude:  42.1500015259,
		Longitude: -92.9751968384,
		Timezone:  "America/Chicago",
	},
	"6IA7": {
		ICAO:      "6IA7",
		Name:      "R S Auto Airport",
		City:      "Lockridge",
		State:     "Iowa",
		Country:   "US",
		Elevation: 770,
		Latitude:  40.990398407,
		Longitude: -91.7442016602,
		Timezone:  "America/Chicago",
	},
	"6ID1": {
		ICAO:      "6ID1",
		Name:      "Regan Ranch Airport",
		City:      "Coeur D Alene",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2720,
		Latitude:  47.6436004639,
		Longitude: -116.698997498,
		Timezone:  "America/Los_Angeles",
	},
	"6II0": {
		ICAO:      "6II0",
		Name:      "Turnpaugh Field",
		City:      "Walton",
		State:     "Indiana",
		Country:   "US",
		Elevation: 757,
		Latitude:  40.6453018188,
		Longitude: -86.2928009033,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"6II2": {
		ICAO:      "6II2",
		Name:      "Willis Airport Site No. 2 Airport",
		City:      "Greenfield",
		State:     "Indiana",
		Country:   "US",
		Elevation: 881,
		Latitude:  39.7333984375,
		Longitude: -85.7232971191,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"6II4": {
		ICAO:      "6II4",
		Name:      "Cornell Airport",
		City:      "Boonville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 390,
		Latitude:  37.9822006226,
		Longitude: -87.2988967896,
		Timezone:  "America/Chicago",
	},
	"6II6": {
		ICAO:      "6II6",
		Name:      "David Beiswanger Ultralightport",
		City:      "South Milford",
		State:     "Indiana",
		Country:   "US",
		Elevation: 1010,
		Latitude:  41.5606002808,
		Longitude: -85.2739028931,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"6II7": {
		ICAO:      "6II7",
		Name:      "Bergs Airport",
		City:      "Lizton",
		State:     "Indiana",
		Country:   "US",
		Elevation: 975,
		Latitude:  39.8861999512,
		Longitude: -86.5813980103,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"6II8": {
		ICAO:      "6II8",
		Name:      "Godahavit Airport",
		City:      "Wheatland",
		State:     "Indiana",
		Country:   "US",
		Elevation: 495,
		Latitude:  38.654499054,
		Longitude: -87.2827987671,
		Timezone:  "America/Indiana/Vincennes",
	},
	"6IL0": {
		ICAO:      "6IL0",
		Name:      "Ramme Airport",
		City:      "Genoa",
		State:     "Illinois",
		Country:   "US",
		Elevation: 850,
		Latitude:  42.131401062,
		Longitude: -88.699798584,
		Timezone:  "America/Chicago",
	},
	"6IL2": {
		ICAO:      "6IL2",
		Name:      "Phyllis Field",
		City:      "Huntley",
		State:     "Illinois",
		Country:   "US",
		Elevation: 855,
		Latitude:  42.2210998535,
		Longitude: -88.4179000854,
		Timezone:  "America/Chicago",
	},
	"6IL3": {
		ICAO:      "6IL3",
		Name:      "Rummel Restricted Landing Area",
		City:      "German Valley",
		State:     "Illinois",
		Country:   "US",
		Elevation: 850,
		Latitude:  42.1666984558,
		Longitude: -89.5000991821,
		Timezone:  "America/Chicago",
	},
	"6IL4": {
		ICAO:      "6IL4",
		Name:      "Foxfield Aerodrome",
		City:      "Lanark",
		State:     "Illinois",
		Country:   "US",
		Elevation: 895,
		Latitude:  42.1603012085,
		Longitude: -89.8861999512,
		Timezone:  "America/Chicago",
	},
	"6IL5": {
		ICAO:      "6IL5",
		Name:      "Keil Airport",
		City:      "Dongola",
		State:     "Illinois",
		Country:   "US",
		Elevation: 525,
		Latitude:  37.3863983154,
		Longitude: -89.1604003906,
		Timezone:  "America/Chicago",
	},
	"6IL6": {
		ICAO:      "6IL6",
		Name:      "Reid Restricted Landing Area",
		City:      "Gilberts",
		State:     "Illinois",
		Country:   "US",
		Elevation: 920,
		Latitude:  42.1184005737,
		Longitude: -88.4078979492,
		Timezone:  "America/Chicago",
	},
	"6IN2": {
		ICAO:      "6IN2",
		Name:      "Whelen Airport",
		City:      "Waldron",
		State:     "Indiana",
		Country:   "US",
		Elevation: 870,
		Latitude:  39.493598938,
		Longitude: -85.6485977173,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"6IN3": {
		ICAO:      "6IN3",
		Name:      "Wyandotte Airport",
		City:      "Lafayette",
		State:     "Indiana",
		Country:   "US",
		Elevation: 700,
		Latitude:  40.3492012024,
		Longitude: -86.7639007568,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"6IN4": {
		ICAO:      "6IN4",
		Name:      "Fischer Field",
		City:      "Waldron",
		State:     "Indiana",
		Country:   "US",
		Elevation: 835,
		Latitude:  39.465801239,
		Longitude: -85.698600769,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"6IN5": {
		ICAO:      "6IN5",
		Name:      "Foghorn Farms Airport",
		City:      "Pendleton",
		State:     "Indiana",
		Country:   "US",
		Elevation: 870,
		Latitude:  40.0327987671,
		Longitude: -85.7835998535,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"6IN6": {
		ICAO:      "6IN6",
		Name:      "Mershon Airport",
		City:      "Monrovia",
		State:     "Indiana",
		Country:   "US",
		Elevation: 870,
		Latitude:  39.5489006042,
		Longitude: -86.4766998291,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"6IN8": {
		ICAO:      "6IN8",
		Name:      "Mcminn Airport",
		City:      "Rushville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 1065,
		Latitude:  39.5285987854,
		Longitude: -85.3167037964,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"6IS0": {
		ICAO:      "6IS0",
		Name:      "Day Aero-Place Airport",
		City:      "Urbana",
		State:     "Illinois",
		Country:   "US",
		Elevation: 690,
		Latitude:  40.0950012207,
		Longitude: -88.1287002563,
		Timezone:  "America/Chicago",
	},
	"6IS8": {
		ICAO:      "6IS8",
		Name:      "Trisler Airport",
		City:      "Homer",
		State:     "Illinois",
		Country:   "US",
		Elevation: 672,
		Latitude:  39.9894981384,
		Longitude: -87.9052963257,
		Timezone:  "America/Chicago",
	},
	"6KS1": {
		ICAO:      "6KS1",
		Name:      "Quinter Air Strip",
		City:      "Quinter",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2681,
		Latitude:  39.0705986023,
		Longitude: -100.25,
		Timezone:  "America/Chicago",
	},
	"6KS2": {
		ICAO:      "6KS2",
		Name:      "Stevenson Private Airport",
		City:      "Selden",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2940,
		Latitude:  39.4416999817,
		Longitude: -100.6299972534,
		Timezone:  "America/Chicago",
	},
	"6KS3": {
		ICAO:      "6KS3",
		Name:      "Walker Strip",
		City:      "Sharon Springs",
		State:     "Kansas",
		Country:   "US",
		Elevation: 3670,
		Latitude:  38.8456001282,
		Longitude: -101.7480010986,
		Timezone:  "America/Denver",
	},
	"6KS4": {
		ICAO:      "6KS4",
		Name:      "Bussen Airport",
		City:      "Sharon Springs",
		State:     "Kansas",
		Country:   "US",
		Elevation: 3475,
		Latitude:  38.9053001404,
		Longitude: -101.783996582,
		Timezone:  "America/Denver",
	},
	"6KS5": {
		ICAO:      "6KS5",
		Name:      "Yeamans Fox Nest Airport",
		City:      "Pleasanton",
		State:     "Kansas",
		Country:   "US",
		Elevation: 782,
		Latitude:  38.2249984741,
		Longitude: -94.6644973755,
		Timezone:  "America/Chicago",
	},
	"6KS6": {
		ICAO:      "6KS6",
		Name:      "Kiowa Airport",
		City:      "Kiowa",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1333,
		Latitude:  37.0153007507,
		Longitude: -98.4959030151,
		Timezone:  "America/Chicago",
	},
	"6KS7": {
		ICAO:      "6KS7",
		Name:      "Shute Airport",
		City:      "Lebanon",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1960,
		Latitude:  39.984500885,
		Longitude: -98.5102996826,
		Timezone:  "America/Chicago",
	},
	"6KS8": {
		ICAO:      "6KS8",
		Name:      "Mary's Place Airport",
		City:      "Emporia",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1138,
		Latitude:  38.3835983276,
		Longitude: -96.1461029053,
		Timezone:  "America/Chicago",
	},
	"6KY3": {
		ICAO:      "6KY3",
		Name:      "Flying C Farms Airport",
		City:      "Shelbyville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 850,
		Latitude:  38.328611,
		Longitude: -85.214167,
		Timezone:  "America/New_York",
	},
	"6KY4": {
		ICAO:      "6KY4",
		Name:      "Adair Airport",
		City:      "Fountain Run",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 870,
		Latitude:  36.753333,
		Longitude: -85.953889,
		Timezone:  "America/Chicago",
	},
	"6KY6": {
		ICAO:      "6KY6",
		Name:      "Jeffries Farm Airport",
		City:      "Louisville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 803,
		Latitude:  38.3605995178,
		Longitude: -85.3630981445,
		Timezone:  "America/New_York",
	},
	"6KY7": {
		ICAO:      "6KY7",
		Name:      "Alexander Field",
		City:      "Hickman",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 445,
		Latitude:  36.523,
		Longitude: -89.051693,
		Timezone:  "America/Chicago",
	},
	"6LA1": {
		ICAO:      "6LA1",
		Name:      "Cottonwood Airport",
		City:      "Lake Providence",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 120,
		Latitude:  32.9840011597,
		Longitude: -91.2059020996,
		Timezone:  "America/Chicago",
	},
	"6LA4": {
		ICAO:      "6LA4",
		Name:      "Dyer Airport",
		City:      "Oakdale",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 118,
		Latitude:  30.807100296,
		Longitude: -92.591796875,
		Timezone:  "America/Chicago",
	},
	"6LA5": {
		ICAO:      "6LA5",
		Name:      "Church Point Flyers Airport",
		City:      "Church Point",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 45,
		Latitude:  30.4547996521,
		Longitude: -92.2406005859,
		Timezone:  "America/Chicago",
	},
	"6LA6": {
		ICAO:      "6LA6",
		Name:      "Williams Flying Service Airport",
		City:      "Winnsboro",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 79,
		Latitude:  32.2182006836,
		Longitude: -91.8682022095,
		Timezone:  "America/Chicago",
	},
	"6LL0": {
		ICAO:      "6LL0",
		Name:      "Williamson Airport",
		City:      "Neoga",
		State:     "Illinois",
		Country:   "US",
		Elevation: 664,
		Latitude:  39.318901062,
		Longitude: -88.4250030518,
		Timezone:  "America/Chicago",
	},
	"6LL1": {
		ICAO:      "6LL1",
		Name:      "Cumberland Air Park RLA Restricted Landing Area",
		City:      "Neoga",
		State:     "Illinois",
		Country:   "US",
		Elevation: 631,
		Latitude:  39.3138999939,
		Longitude: -88.3444976807,
		Timezone:  "America/Chicago",
	},
	"6LL2": {
		ICAO:      "6LL2",
		Name:      "Young Airport",
		City:      "Viola",
		State:     "Illinois",
		Country:   "US",
		Elevation: 760,
		Latitude:  41.1645011902,
		Longitude: -90.5574035645,
		Timezone:  "America/Chicago",
	},
	"6LL4": {
		ICAO:      "6LL4",
		Name:      "Wildy Field",
		City:      "New Athens",
		State:     "Illinois",
		Country:   "US",
		Elevation: 450,
		Latitude:  38.2675018311,
		Longitude: -89.8917999268,
		Timezone:  "America/Chicago",
	},
	"6LL5": {
		ICAO:      "6LL5",
		Name:      "Smith Restricted Landing Area",
		City:      "New Berlin",
		State:     "Illinois",
		Country:   "US",
		Elevation: 632,
		Latitude:  39.7895011902,
		Longitude: -89.9912033081,
		Timezone:  "America/Chicago",
	},
	"6LL6": {
		ICAO:      "6LL6",
		Name:      "Fischer's RLA Restricted Landing Area",
		City:      "New Memphis",
		State:     "Illinois",
		Country:   "US",
		Elevation: 440,
		Latitude:  38.4916992188,
		Longitude: -89.6633987427,
		Timezone:  "America/Chicago",
	},
	"6LL8": {
		ICAO:      "6LL8",
		Name:      "Songwood Inn Airport",
		City:      "Watseka",
		State:     "Illinois",
		Country:   "US",
		Elevation: 629,
		Latitude:  40.792301178,
		Longitude: -87.7253036499,
		Timezone:  "America/Chicago",
	},
	"6MA2": {
		ICAO:      "6MA2",
		Name:      "Meadowbrook Airport",
		City:      "Amesbury",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 117,
		Latitude:  42.8600997925,
		Longitude: -70.99659729,
		Timezone:  "America/New_York",
	},
	"6MD1": {
		ICAO:      "6MD1",
		Name:      "Dileo Field",
		City:      "Denton",
		State:     "Maryland",
		Country:   "US",
		Elevation: 50,
		Latitude:  38.8363990784,
		Longitude: -75.9064025879,
		Timezone:  "America/New_York",
	},
	"6MD2": {
		ICAO:      "6MD2",
		Name:      "Spring Landing Airport",
		City:      "Crumpton",
		State:     "Maryland",
		Country:   "US",
		Elevation: 50,
		Latitude:  39.2458000183,
		Longitude: -75.8843994141,
		Timezone:  "America/New_York",
	},
	"6MD3": {
		ICAO:      "6MD3",
		Name:      "Harp Airport",
		City:      "Wolfsville",
		State:     "Maryland",
		Country:   "US",
		Elevation: 1360,
		Latitude:  39.5969009399,
		Longitude: -77.5466995239,
		Timezone:  "America/New_York",
	},
	"6MD4": {
		ICAO:      "6MD4",
		Name:      "Pembroke Farm Airport",
		City:      "Conowingo",
		State:     "Maryland",
		Country:   "US",
		Elevation: 380,
		Latitude:  39.7193984985,
		Longitude: -76.2083969116,
		Timezone:  "America/New_York",
	},
	"6MD7": {
		ICAO:      "6MD7",
		Name:      "D'Angelo Airport",
		City:      "Golts",
		State:     "Maryland",
		Country:   "US",
		Elevation: 82,
		Latitude:  39.3391990662,
		Longitude: -75.7667007446,
		Timezone:  "America/New_York",
	},
	"6MD8": {
		ICAO:      "6MD8",
		Name:      "White's Airstrip",
		City:      "Pittsville",
		State:     "Maryland",
		Country:   "US",
		Elevation: 55,
		Latitude:  38.4422988892,
		Longitude: -75.4032974243,
		Timezone:  "America/New_York",
	},
	"6MI0": {
		ICAO:      "6MI0",
		Name:      "King Trout Airport",
		City:      "Grayling",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1100,
		Latitude:  44.6147003174,
		Longitude: -84.952003479,
		Timezone:  "America/Detroit",
	},
	"6MI1": {
		ICAO:      "6MI1",
		Name:      "Jensen Field",
		City:      "Millington",
		State:     "Michigan",
		Country:   "US",
		Elevation: 755,
		Latitude:  43.2630996704,
		Longitude: -83.5637969971,
		Timezone:  "America/Detroit",
	},
	"6MI2": {
		ICAO:      "6MI2",
		Name:      "Deyoung Airport",
		City:      "Allendale",
		State:     "Michigan",
		Country:   "US",
		Elevation: 656,
		Latitude:  42.9678001404,
		Longitude: -85.9617004395,
		Timezone:  "America/Detroit",
	},
	"6MI3": {
		ICAO:      "6MI3",
		Name:      "Curt's Place Airport",
		City:      "Zeeland",
		State:     "Michigan",
		Country:   "US",
		Elevation: 630,
		Latitude:  42.7630996704,
		Longitude: -85.9828033447,
		Timezone:  "America/Detroit",
	},
	"6MI6": {
		ICAO:      "6MI6",
		Name:      "Hancock Airport",
		City:      "Buchanan",
		State:     "Michigan",
		Country:   "US",
		Elevation: 760,
		Latitude:  41.8527984619,
		Longitude: -86.4360961914,
		Timezone:  "America/Detroit",
	},
	"6MI7": {
		ICAO:      "6MI7",
		Name:      "J P's Field",
		City:      "Zeeland",
		State:     "Michigan",
		Country:   "US",
		Elevation: 708,
		Latitude:  42.806098938,
		Longitude: -85.9341964722,
		Timezone:  "America/Detroit",
	},
	"6MN0": {
		ICAO:      "6MN0",
		Name:      "Rooney Airfield",
		City:      "Ponsford",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1620,
		Latitude:  47.0555992126,
		Longitude: -95.4231033325,
		Timezone:  "America/Chicago",
	},
	"6MN2": {
		ICAO:      "6MN2",
		Name:      "Brown's Private Airport",
		City:      "Hancock",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1175,
		Latitude:  45.545501709,
		Longitude: -95.7795028687,
		Timezone:  "America/Chicago",
	},
	"6MN7": {
		ICAO:      "6MN7",
		Name:      "Mankato Farmstrip Airport",
		City:      "Mankato",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1002,
		Latitude:  44.0999984741,
		Longitude: -93.9169006348,
		Timezone:  "America/Chicago",
	},
	"6MN8": {
		ICAO:      "6MN8",
		Name:      "Underland Airstrip",
		City:      "Medford",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1145,
		Latitude:  44.1526985168,
		Longitude: -93.2723999023,
		Timezone:  "America/Chicago",
	},
	"6MN9": {
		ICAO:      "6MN9",
		Name:      "Benson Airport",
		City:      "White Bear Township",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 942,
		Latitude:  45.1166000366,
		Longitude: -92.9960021973,
		Timezone:  "America/Chicago",
	},
	"6MO0": {
		ICAO:      "6MO0",
		Name:      "Blackhawk Airport",
		City:      "Old Monroe",
		State:     "Missouri",
		Country:   "US",
		Elevation: 584,
		Latitude:  38.9556007385,
		Longitude: -90.8147964478,
		Timezone:  "America/Chicago",
	},
	"6MO1": {
		ICAO:      "6MO1",
		Name:      "Markt Air Strip",
		City:      "Oregon",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1140,
		Latitude:  39.9667015076,
		Longitude: -95.0669021606,
		Timezone:  "America/Chicago",
	},
	"6MO2": {
		ICAO:      "6MO2",
		Name:      "Ste Genevieve Flying Club Airport",
		City:      "Ste Genevieve",
		State:     "Missouri",
		Country:   "US",
		Elevation: 380,
		Latitude:  37.9861984253,
		Longitude: -90.0334014893,
		Timezone:  "America/Chicago",
	},
	"6MO3": {
		ICAO:      "6MO3",
		Name:      "Flying 'E' Airport",
		City:      "Houston",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1510,
		Latitude:  37.3492012024,
		Longitude: -92.1283035278,
		Timezone:  "America/Chicago",
	},
	"6MO4": {
		ICAO:      "6MO4",
		Name:      "Wixted Airport",
		City:      "Osceola",
		State:     "Missouri",
		Country:   "US",
		Elevation: 850,
		Latitude:  38.0416984558,
		Longitude: -93.7835006714,
		Timezone:  "America/Chicago",
	},
	"6MO6": {
		ICAO:      "6MO6",
		Name:      "Winter Field",
		City:      "Owensville",
		State:     "Missouri",
		Country:   "US",
		Elevation: 960,
		Latitude:  38.3334007263,
		Longitude: -91.5167999268,
		Timezone:  "America/Chicago",
	},
	"6MO7": {
		ICAO:      "6MO7",
		Name:      "Riverside Landings Airport",
		City:      "Ozark",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1280,
		Latitude:  37.0292015076,
		Longitude: -93.1668014526,
		Timezone:  "America/Chicago",
	},
	"6MO8": {
		ICAO:      "6MO8",
		Name:      "Brazeale Farm Airport",
		City:      "Paris",
		State:     "Missouri",
		Country:   "US",
		Elevation: 780,
		Latitude:  39.4833984375,
		Longitude: -92.0668029785,
		Timezone:  "America/Chicago",
	},
	"6MO9": {
		ICAO:      "6MO9",
		Name:      "West Aero Ranch Airport",
		City:      "Peculiar",
		State:     "Missouri",
		Country:   "US",
		Elevation: 980,
		Latitude:  38.7005996704,
		Longitude: -94.4738006592,
		Timezone:  "America/Chicago",
	},
	"6MS1": {
		ICAO:      "6MS1",
		Name:      "Woodbridge Airport",
		City:      "Brandon",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 325,
		Latitude:  32.4557991028,
		Longitude: -89.9697036743,
		Timezone:  "America/Chicago",
	},
	"6MS2": {
		ICAO:      "6MS2",
		Name:      "Wells Farm Airport",
		City:      "Crystal Springs",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 455,
		Latitude:  31.983861,
		Longitude: -90.335361,
		Timezone:  "America/Chicago",
	},
	"6MU4": {
		ICAO:      "6MU4",
		Name:      "Flying Shamrock Airport",
		City:      "Mexico",
		State:     "Missouri",
		Country:   "US",
		Elevation: 810,
		Latitude:  39.0377998352,
		Longitude: -91.725402832,
		Timezone:  "America/Chicago",
	},
	"6MU9": {
		ICAO:      "6MU9",
		Name:      "Craddock Field",
		City:      "Licking",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1199,
		Latitude:  37.5348014832,
		Longitude: -91.9652023315,
		Timezone:  "America/Chicago",
	},
	"6NA0": {
		ICAO:      "6NA0",
		Name:      "Strom Private Airport",
		City:      "Columbus",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1930,
		Latitude:  48.9930992126,
		Longitude: -102.7379989624,
		Timezone:  "America/Chicago",
	},
	"6NA2": {
		ICAO:      "6NA2",
		Name:      "South Hector Airstrip",
		City:      "Fargo",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 910,
		Latitude:  46.7924995422,
		Longitude: -96.8037033081,
		Timezone:  "America/Chicago",
	},
	"6NA5": {
		ICAO:      "6NA5",
		Name:      "Chase Airstrip",
		City:      "Hebron",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 2140,
		Latitude:  46.9127998352,
		Longitude: -102.0090026855,
		Timezone:  "America/North_Dakota/New_Salem",
	},
	"6NA6": {
		ICAO:      "6NA6",
		Name:      "Marsh Brothers Airstrip",
		City:      "Newburg",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1481,
		Latitude:  48.6192016602,
		Longitude: -100.9779968262,
		Timezone:  "America/Chicago",
	},
	"6NA7": {
		ICAO:      "6NA7",
		Name:      "Kersten Brothers Airport",
		City:      "Newburg",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1473,
		Latitude:  48.7303009033,
		Longitude: -100.9219970703,
		Timezone:  "America/Chicago",
	},
	"6NC0": {
		ICAO:      "6NC0",
		Name:      "Cox-Grantham Airfield",
		City:      "Grantham",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 157,
		Latitude:  35.289100647,
		Longitude: -78.1800003052,
		Timezone:  "America/New_York",
	},
	"6NC1": {
		ICAO:      "6NC1",
		Name:      "Little Mountain Airport",
		City:      "Denver",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 1030,
		Latitude:  35.5917015076,
		Longitude: -81.0793991089,
		Timezone:  "America/New_York",
	},
	"6NC2": {
		ICAO:      "6NC2",
		Name:      "Wilhelm Airport",
		City:      "Kannapolis",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 750,
		Latitude:  35.5111999512,
		Longitude: -80.5580978394,
		Timezone:  "America/New_York",
	},
	"6NC3": {
		ICAO:      "6NC3",
		Name:      "Eastover Air Ranch Airport",
		City:      "Fayetteville",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 160,
		Latitude:  35.0863990784,
		Longitude: -78.7722015381,
		Timezone:  "America/New_York",
	},
	"6NC4": {
		ICAO:      "6NC4",
		Name:      "Brown STOLport",
		City:      "Taylorsville",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 1170,
		Latitude:  35.9057006836,
		Longitude: -81.139503479,
		Timezone:  "America/New_York",
	},
	"6NC5": {
		ICAO:      "6NC5",
		Name:      "Thompson Farms Airport",
		City:      "Whitakers",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 100,
		Latitude:  36.0178985596,
		Longitude: -77.6619033813,
		Timezone:  "America/New_York",
	},
	"6NC7": {
		ICAO:      "6NC7",
		Name:      "Circle P Farm Airport",
		City:      "Newport",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 30,
		Latitude:  34.7513999939,
		Longitude: -76.9060974121,
		Timezone:  "America/New_York",
	},
	"6NC8": {
		ICAO:      "6NC8",
		Name:      "Marchmont Plantation Airpark",
		City:      "Advance",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 730,
		Latitude:  35.9382019043,
		Longitude: -80.3844985962,
		Timezone:  "America/New_York",
	},
	"6NC9": {
		ICAO:      "6NC9",
		Name:      "Moss Hill Airport",
		City:      "Kinston",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 30,
		Latitude:  35.2020988464,
		Longitude: -77.75050354,
		Timezone:  "America/New_York",
	},
	"6ND2": {
		ICAO:      "6ND2",
		Name:      "Larson Airport",
		City:      "Fullerton",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1442,
		Latitude:  46.2588996887,
		Longitude: -98.3879013062,
		Timezone:  "America/Chicago",
	},
	"6ND3": {
		ICAO:      "6ND3",
		Name:      "Lisburg Airport",
		City:      "Argusville",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 885,
		Latitude:  47.0696983337,
		Longitude: -96.9405975342,
		Timezone:  "America/Chicago",
	},
	"6ND6": {
		ICAO:      "6ND6",
		Name:      "Fugleberg Farm Airport",
		City:      "Portland",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1090,
		Latitude:  47.5167007446,
		Longitude: -97.5503997803,
		Timezone:  "America/Chicago",
	},
	"6ND9": {
		ICAO:      "6ND9",
		Name:      "Skinningsrud Airport",
		City:      "Berthold",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 2010,
		Latitude:  48.3927993774,
		Longitude: -101.763999939,
		Timezone:  "America/Chicago",
	},
	"6NE0": {
		ICAO:      "6NE0",
		Name:      "Van Boening Airport",
		City:      "Wallace",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3102,
		Latitude:  40.7868995667,
		Longitude: -101.1660003662,
		Timezone:  "America/Chicago",
	},
	"6NE7": {
		ICAO:      "6NE7",
		Name:      "Lee Field",
		City:      "Bartley",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2450,
		Latitude:  40.2444000244,
		Longitude: -100.3379974365,
		Timezone:  "America/Chicago",
	},
	"6NE8": {
		ICAO:      "6NE8",
		Name:      "Hoppy's Airport",
		City:      "Benkelman",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3227,
		Latitude:  40.0868988037,
		Longitude: -101.5690002441,
		Timezone:  "America/Denver",
	},
	"6NE9": {
		ICAO:      "6NE9",
		Name:      "Hoesel Airport",
		City:      "Callaway",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2600,
		Latitude:  41.2681007385,
		Longitude: -99.8879013062,
		Timezone:  "America/Chicago",
	},
	"6NJ0": {
		ICAO:      "6NJ0",
		Name:      "Lentine South Airport",
		City:      "Warren Grove",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 150,
		Latitude:  39.7871017456,
		Longitude: -74.3774032593,
		Timezone:  "America/New_York",
	},
	"6NJ1": {
		ICAO:      "6NJ1",
		Name:      "Vliet Airport",
		City:      "Washington",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 460,
		Latitude:  40.740398407,
		Longitude: -74.9568023682,
		Timezone:  "America/New_York",
	},
	"6NJ8": {
		ICAO:      "6NJ8",
		Name:      "Lance Airport",
		City:      "Whitehouse Station",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 275,
		Latitude:  40.5918006897,
		Longitude: -74.7912979126,
		Timezone:  "America/New_York",
	},
	"6NJ9": {
		ICAO:      "6NJ9",
		Name:      "Bridgeport-Cahill Field",
		City:      "Bridgeport",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 15,
		Latitude:  39.7918014526,
		Longitude: -75.3746032715,
		Timezone:  "America/New_York",
	},
	"6NK0": {
		ICAO:      "6NK0",
		Name:      "Knapp Airport",
		City:      "Schenevus",
		State:     "New York",
		Country:   "US",
		Elevation: 1300,
		Latitude:  42.6291999817,
		Longitude: -74.8208007813,
		Timezone:  "America/New_York",
	},
	"6NK1": {
		ICAO:      "6NK1",
		Name:      "Casey's Airport",
		City:      "Warsaw",
		State:     "New York",
		Country:   "US",
		Elevation: 1600,
		Latitude:  42.756401062,
		Longitude: -78.1755981445,
		Timezone:  "America/New_York",
	},
	"6NK2": {
		ICAO:      "6NK2",
		Name:      "Meerwarth Airport",
		City:      "White Creek",
		State:     "New York",
		Country:   "US",
		Elevation: 640,
		Latitude:  42.9505996704,
		Longitude: -73.3347015381,
		Timezone:  "America/New_York",
	},
	"6NK4": {
		ICAO:      "6NK4",
		Name:      "Ttt Air Airport",
		City:      "North Collins",
		State:     "New York",
		Country:   "US",
		Elevation: 891,
		Latitude:  42.5978012085,
		Longitude: -78.9689025879,
		Timezone:  "America/New_York",
	},
	"6NK8": {
		ICAO:      "6NK8",
		Name:      "Flying K Airport",
		City:      "Oneida",
		State:     "New York",
		Country:   "US",
		Elevation: 1350,
		Latitude:  43.0097999573,
		Longitude: -76.6405029297,
		Timezone:  "America/New_York",
	},
	"6NY1": {
		ICAO:      "6NY1",
		Name:      "Old Port Royal Airport",
		City:      "Corning",
		State:     "New York",
		Country:   "US",
		Elevation: 1700,
		Latitude:  42.1128997803,
		Longitude: -77.0894012451,
		Timezone:  "America/New_York",
	},
	"6NY2": {
		ICAO:      "6NY2",
		Name:      "Omni Airpark",
		City:      "Horseheads",
		State:     "New York",
		Country:   "US",
		Elevation: 970,
		Latitude:  42.2319984436,
		Longitude: -76.8469009399,
		Timezone:  "America/New_York",
	},
	"6NY3": {
		ICAO:      "6NY3",
		Name:      "Airy-Acres Airport",
		City:      "Interlaken",
		State:     "New York",
		Country:   "US",
		Elevation: 805,
		Latitude:  42.6440010071,
		Longitude: -76.7296981812,
		Timezone:  "America/New_York",
	},
	"6NY4": {
		ICAO:      "6NY4",
		Name:      "West Township Airport",
		City:      "Knox",
		State:     "New York",
		Country:   "US",
		Elevation: 1260,
		Latitude:  42.7004013062,
		Longitude: -74.1550979614,
		Timezone:  "America/New_York",
	},
	"6NY7": {
		ICAO:      "6NY7",
		Name:      "Piolis Brookside Airport",
		City:      "Kerhonkson",
		State:     "New York",
		Country:   "US",
		Elevation: 1060,
		Latitude:  41.865398407,
		Longitude: -74.332901001,
		Timezone:  "America/New_York",
	},
	"6OG3": {
		ICAO:      "6OG3",
		Name:      "Sky Wagon Ranch Airport",
		City:      "Klamath Falls",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4150,
		Latitude:  42.130283,
		Longitude: -121.531633,
		Timezone:  "America/Los_Angeles",
	},
	"6OH1": {
		ICAO:      "6OH1",
		Name:      "Missler-Bellevue Airport",
		City:      "Bellevue",
		State:     "Ohio",
		Country:   "US",
		Elevation: 760,
		Latitude:  41.2862014771,
		Longitude: -82.8666000366,
		Timezone:  "America/New_York",
	},
	"6OH3": {
		ICAO:      "6OH3",
		Name:      "R C Ford Field",
		City:      "Berkey",
		State:     "Ohio",
		Country:   "US",
		Elevation: 700,
		Latitude:  41.7051010132,
		Longitude: -83.8287963867,
		Timezone:  "America/Detroit",
	},
	"6OH4": {
		ICAO:      "6OH4",
		Name:      "Aring Field",
		City:      "Bowling Green",
		State:     "Ohio",
		Country:   "US",
		Elevation: 650,
		Latitude:  41.3583984375,
		Longitude: -83.5582962036,
		Timezone:  "America/New_York",
	},
	"6OH6": {
		ICAO:      "6OH6",
		Name:      "Shelton Airport",
		City:      "Bowling Green",
		State:     "Ohio",
		Country:   "US",
		Elevation: 675,
		Latitude:  41.3644981384,
		Longitude: -83.5474014282,
		Timezone:  "America/New_York",
	},
	"6OH7": {
		ICAO:      "6OH7",
		Name:      "Nietz Airport",
		City:      "Bowling Green",
		State:     "Ohio",
		Country:   "US",
		Elevation: 676,
		Latitude:  41.4126014709,
		Longitude: -83.6797027588,
		Timezone:  "America/New_York",
	},
	"6OH9": {
		ICAO:      "6OH9",
		Name:      "Kemps Field of Dreams Airport",
		City:      "Covington",
		State:     "Ohio",
		Country:   "US",
		Elevation: 974,
		Latitude:  40.1058998108,
		Longitude: -84.3999023438,
		Timezone:  "America/New_York",
	},
	"6OI0": {
		ICAO:      "6OI0",
		Name:      "Derecsky Airport",
		City:      "Auburn Center",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1185,
		Latitude:  41.3759002686,
		Longitude: -81.2573013306,
		Timezone:  "America/New_York",
	},
	"6OI5": {
		ICAO:      "6OI5",
		Name:      "Mitchell Airport",
		City:      "Plain City",
		State:     "Ohio",
		Country:   "US",
		Elevation: 950,
		Latitude:  40.137298584,
		Longitude: -83.2180023193,
		Timezone:  "America/New_York",
	},
	"6OK0": {
		ICAO:      "6OK0",
		Name:      "White Airport",
		City:      "Kingfisher",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1110,
		Latitude:  35.8913993835,
		Longitude: -97.9589004517,
		Timezone:  "America/Chicago",
	},
	"6OK1": {
		ICAO:      "6OK1",
		Name:      "John Reid Airport",
		City:      "Tahlequah",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 831,
		Latitude:  36.001499176,
		Longitude: -95.0029983521,
		Timezone:  "America/Chicago",
	},
	"6OK2": {
		ICAO:      "6OK2",
		Name:      "Redhills Airport",
		City:      "Chickasha",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1175,
		Latitude:  35.0628013611,
		Longitude: -97.8953018188,
		Timezone:  "America/Chicago",
	},
	"6OK3": {
		ICAO:      "6OK3",
		Name:      "Pinson's Cottonpatch Airport",
		City:      "Tipton",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1297,
		Latitude:  34.4748001099,
		Longitude: -99.1315002441,
		Timezone:  "America/Chicago",
	},
	"6OK5": {
		ICAO:      "6OK5",
		Name:      "Cary Ranch Airport",
		City:      "Reydon",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 2455,
		Latitude:  35.6286010742,
		Longitude: -99.929397583,
		Timezone:  "America/Chicago",
	},
	"6OK6": {
		ICAO:      "6OK6",
		IATA:      "BWL",
		Name:      "Earl Henry Airport",
		City:      "Blackwell",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1054,
		Latitude:  36.7958984375,
		Longitude: -97.3170013428,
		Timezone:  "America/Chicago",
	},
	"6OK7": {
		ICAO:      "6OK7",
		Name:      "Cochran Ranch Airport",
		City:      "Atoka",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 790,
		Latitude:  34.5959014893,
		Longitude: -95.8629989624,
		Timezone:  "America/Chicago",
	},
	"6OK8": {
		ICAO:      "6OK8",
		Name:      "Lasley Private Airport",
		City:      "Eakly",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1630,
		Latitude:  35.359500885,
		Longitude: -98.5711975098,
		Timezone:  "America/Chicago",
	},
	"6OK9": {
		ICAO:      "6OK9",
		Name:      "Mulberry Hill Airport",
		City:      "Stillwater",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1026,
		Latitude:  36.14170074,
		Longitude: -96.94229889,
		Timezone:  "America/Chicago",
	},
	"6OR0": {
		ICAO:      "6OR0",
		Name:      "Grells Airport",
		City:      "Tangent",
		State:     "Oregon",
		Country:   "US",
		Elevation: 253,
		Latitude:  44.5297012329,
		Longitude: -123.091003418,
		Timezone:  "America/Los_Angeles",
	},
	"6OR1": {
		ICAO:      "6OR1",
		Name:      "Pointers Airport",
		City:      "The Dalles",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1140,
		Latitude:  45.572101593,
		Longitude: -121.2720031738,
		Timezone:  "America/Los_Angeles",
	},
	"6OR2": {
		ICAO:      "6OR2",
		Name:      "Chenoweth Airpark",
		City:      "The Dalles",
		State:     "Oregon",
		Country:   "US",
		Elevation: 609,
		Latitude:  45.611000061,
		Longitude: -121.2710037231,
		Timezone:  "America/Los_Angeles",
	},
	"6OR4": {
		ICAO:      "6OR4",
		Name:      "Tailwheel Airport",
		City:      "Prineville",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3130,
		Latitude:  44.2731825,
		Longitude: -120.794741944,
		Timezone:  "America/Los_Angeles",
	},
	"6OR6": {
		ICAO:      "6OR6",
		Name:      "Wonder Airport",
		City:      "Wonder",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1250,
		Latitude:  42.3914985657,
		Longitude: -123.5350036621,
		Timezone:  "America/Los_Angeles",
	},
	"6OR7": {
		ICAO:      "6OR7",
		Name:      "Schmidt Airport",
		City:      "Boring",
		State:     "Oregon",
		Country:   "US",
		Elevation: 650,
		Latitude:  45.4528999329,
		Longitude: -122.3209991455,
		Timezone:  "America/Los_Angeles",
	},
	"6OR8": {
		ICAO:      "6OR8",
		Name:      "Holce & Oblack Airport",
		City:      "Birkenfield",
		State:     "Oregon",
		Country:   "US",
		Elevation: 500,
		Latitude:  45.9715003967,
		Longitude: -123.3499984741,
		Timezone:  "America/Los_Angeles",
	},
	"6OR9": {
		ICAO:      "6OR9",
		Name:      "Reds Wallowa Horse Ranch Airport",
		City:      "Enterprise",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3600,
		Latitude:  45.3414993286,
		Longitude: -117.6259994507,
		Timezone:  "America/Los_Angeles",
	},
	"6PA0": {
		ICAO:      "6PA0",
		Name:      "Myer Airport",
		City:      "Milford",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1345,
		Latitude:  41.3501014709,
		Longitude: -74.9328994751,
		Timezone:  "America/New_York",
	},
	"6PA1": {
		ICAO:      "6PA1",
		Name:      "Tower Airfield",
		City:      "North Washington",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1300,
		Latitude:  40.5102996826,
		Longitude: -79.5798034668,
		Timezone:  "America/New_York",
	},
	"6PA3": {
		ICAO:      "6PA3",
		Name:      "Hackenburg-Penny Hill Airport",
		City:      "Montoursville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1000,
		Latitude:  41.150100708,
		Longitude: -76.8996963501,
		Timezone:  "America/New_York",
	},
	"6PA4": {
		ICAO:      "6PA4",
		Name:      "Scandia Air Park",
		City:      "Russell",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 2050,
		Latitude:  41.9305992126,
		Longitude: -79.0342025757,
		Timezone:  "America/New_York",
	},
	"6PA5": {
		ICAO:      "6PA5",
		Name:      "Botsford Aerodrome",
		City:      "Shippensburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 615,
		Latitude:  40.1133995056,
		Longitude: -77.4542007446,
		Timezone:  "America/New_York",
	},
	"6PA6": {
		ICAO:      "6PA6",
		Name:      "Air Haven Airport",
		City:      "Moscow",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1550,
		Latitude:  41.3501014709,
		Longitude: -75.483001709,
		Timezone:  "America/New_York",
	},
	"6PA7": {
		ICAO:      "6PA7",
		Name:      "Berwick Airport",
		City:      "Berwick",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 610,
		Latitude:  41.0694999695,
		Longitude: -76.2163009644,
		Timezone:  "America/New_York",
	},
	"6PA8": {
		ICAO:      "6PA8",
		Name:      "Tall Pines Airfield",
		City:      "Lock Haven",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1500,
		Latitude:  41.2014007568,
		Longitude: -77.4408035278,
		Timezone:  "America/New_York",
	},
	"6PA9": {
		ICAO:      "6PA9",
		Name:      "Shangri La Airport",
		City:      "Burnham",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 760,
		Latitude:  40.6856002808,
		Longitude: -77.4747009277,
		Timezone:  "America/New_York",
	},
	"6PN5": {
		ICAO:      "6PN5",
		Name:      "Sam's Field",
		City:      "Bradford",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1540,
		Latitude:  41.9075012207,
		Longitude: -78.6288986206,
		Timezone:  "America/New_York",
	},
	"6PN9": {
		ICAO:      "6PN9",
		Name:      "Mc Donald's Airport",
		City:      "Brandonville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1200,
		Latitude:  40.8861999512,
		Longitude: -76.153503418,
		Timezone:  "America/New_York",
	},
	"6PS3": {
		ICAO:      "6PS3",
		Name:      "Champ Field",
		City:      "Knoxville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1220,
		Latitude:  41.9589996338,
		Longitude: -77.4261016846,
		Timezone:  "America/New_York",
	},
	"6PS8": {
		ICAO:      "6PS8",
		Name:      "Tyler Airport",
		City:      "New Milford",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1401,
		Latitude:  41.816719,
		Longitude: -75.698783,
		Timezone:  "America/New_York",
	},
	"6SC1": {
		ICAO:      "6SC1",
		Name:      "Lesesne Airport",
		City:      "Cross",
		State:     "South Carolina",
		Country:   "US",
		Elevation: 83,
		Latitude:  33.3692016602,
		Longitude: -80.2232971191,
		Timezone:  "America/New_York",
	},
	"6TA0": {
		ICAO:      "6TA0",
		Name:      "Rocking R Ranch Airport",
		City:      "Eldorado",
		State:     "Texas",
		Country:   "US",
		Elevation: 2250,
		Latitude:  30.858499527,
		Longitude: -100.3229980469,
		Timezone:  "America/Chicago",
	},
	"6TA2": {
		ICAO:      "6TA2",
		Name:      "Rockys Place Airport",
		City:      "Leonard",
		State:     "Texas",
		Country:   "US",
		Elevation: 685,
		Latitude:  33.3722000122,
		Longitude: -96.362197876,
		Timezone:  "America/Chicago",
	},
	"6TA3": {
		ICAO:      "6TA3",
		Name:      "Culp Airport",
		City:      "Lawrence",
		State:     "Texas",
		Country:   "US",
		Elevation: 470,
		Latitude:  32.7214012146,
		Longitude: -96.7238998413,
		Timezone:  "America/Chicago",
	},
	"6TA4": {
		ICAO:      "6TA4",
		Name:      "Winn Exploration Co. Inc Airport",
		City:      "Eagle Pass",
		State:     "Texas",
		Country:   "US",
		Elevation: 881,
		Latitude:  28.7168998718,
		Longitude: -100.408996582,
		Timezone:  "America/Chicago",
	},
	"6TA7": {
		ICAO:      "6TA7",
		Name:      "Flying K Airport",
		City:      "Weatherford",
		State:     "Texas",
		Country:   "US",
		Elevation: 1150,
		Latitude:  32.8284988403,
		Longitude: -97.7795028687,
		Timezone:  "America/Chicago",
	},
	"6TA9": {
		ICAO:      "6TA9",
		Name:      "Los Ebanos Ranch Airport",
		City:      "Mathis",
		State:     "Texas",
		Country:   "US",
		Elevation: 225,
		Latitude:  28.1170005798,
		Longitude: -98.0447006226,
		Timezone:  "America/Chicago",
	},
	"6TE0": {
		ICAO:      "6TE0",
		Name:      "Skalitsky Airport",
		City:      "Edcouch",
		State:     "Texas",
		Country:   "US",
		Elevation: 60,
		Latitude:  26.2903003693,
		Longitude: -97.9494018555,
		Timezone:  "America/Chicago",
	},
	"6TE1": {
		ICAO:      "6TE1",
		Name:      "Norman & White Airport",
		City:      "Edinburg",
		State:     "Texas",
		Country:   "US",
		Elevation: 90,
		Latitude:  26.3614997864,
		Longitude: -98.1524963379,
		Timezone:  "America/Chicago",
	},
	"6TE2": {
		ICAO:      "6TE2",
		Name:      "Zimmerle Airport",
		City:      "Abilene",
		State:     "Texas",
		Country:   "US",
		Elevation: 2057,
		Latitude:  32.270401001,
		Longitude: -99.5979003906,
		Timezone:  "America/Chicago",
	},
	"6TE3": {
		ICAO:      "6TE3",
		Name:      "Hahns Airport",
		City:      "Edna",
		State:     "Texas",
		Country:   "US",
		Elevation: 52,
		Latitude:  28.9489002228,
		Longitude: -96.549697876,
		Timezone:  "America/Chicago",
	},
	"6TE4": {
		ICAO:      "6TE4",
		Name:      "Coldwater Ranch Airport",
		City:      "Sunray",
		State:     "Texas",
		Country:   "US",
		Elevation: 3446,
		Latitude:  36.2333984375,
		Longitude: -101.7340011597,
		Timezone:  "America/Chicago",
	},
	"6TE5": {
		ICAO:      "6TE5",
		Name:      "Kubecka Flying Service Inc. Airport",
		City:      "Edna",
		State:     "Texas",
		Country:   "US",
		Elevation: 65,
		Latitude:  28.9808006287,
		Longitude: -96.6219024658,
		Timezone:  "America/Chicago",
	},
	"6TE6": {
		ICAO:      "6TE6",
		Name:      "6666 Ranch Airport",
		City:      "Guthrie",
		State:     "Texas",
		Country:   "US",
		Elevation: 1775,
		Latitude:  33.6412010193,
		Longitude: -100.347999573,
		Timezone:  "America/Chicago",
	},
	"6TE7": {
		ICAO:      "6TE7",
		Name:      "Mc Neill Ranch Airport",
		City:      "Wayside",
		State:     "Texas",
		Country:   "US",
		Elevation: 3413,
		Latitude:  34.837600708,
		Longitude: -101.57900238,
		Timezone:  "America/Chicago",
	},
	"6TE8": {
		ICAO:      "6TE8",
		Name:      "Douglas Flying Service Private Airport",
		City:      "Aspermont",
		State:     "Texas",
		Country:   "US",
		Elevation: 1743,
		Latitude:  33.1689987183,
		Longitude: -100.2170028687,
		Timezone:  "America/Chicago",
	},
	"6TE9": {
		ICAO:      "6TE9",
		Name:      "Mc Kenzie Field",
		City:      "Avinger",
		State:     "Texas",
		Country:   "US",
		Elevation: 360,
		Latitude:  32.8167991638,
		Longitude: -94.616897583,
		Timezone:  "America/Chicago",
	},
	"6TN1": {
		ICAO:      "6TN1",
		Name:      "Weakleys Field",
		City:      "Pleasantview",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 705,
		Latitude:  36.40620041,
		Longitude: -87.0821991,
		Timezone:  "America/Chicago",
	},
	"6TN4": {
		ICAO:      "6TN4",
		Name:      "The Aviation Valley Airport",
		City:      "Tullahoma",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 890,
		Latitude:  35.4219017029,
		Longitude: -86.2902984619,
		Timezone:  "America/Chicago",
	},
	"6TN7": {
		ICAO:      "6TN7",
		Name:      "St. Somewhere Airport",
		City:      "Grand Junction",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 550,
		Latitude:  35.0238990784,
		Longitude: -89.2050018311,
		Timezone:  "America/Chicago",
	},
	"6TN9": {
		ICAO:      "6TN9",
		Name:      "Dripping Springs Farm Airport",
		City:      "Burns",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 728,
		Latitude:  36.0614013672,
		Longitude: -87.2472000122,
		Timezone:  "America/Chicago",
	},
	"6TS0": {
		ICAO:      "6TS0",
		Name:      "True Airport",
		City:      "Plainview",
		State:     "Texas",
		Country:   "US",
		Elevation: 3385,
		Latitude:  34.0892219543,
		Longitude: -101.7404632568,
		Timezone:  "America/Chicago",
	},
	"6TS1": {
		ICAO:      "6TS1",
		Name:      "Worrell Airport",
		City:      "Kempner",
		State:     "Texas",
		Country:   "US",
		Elevation: 960,
		Latitude:  31.1779003143,
		Longitude: -98.073097229,
		Timezone:  "America/Chicago",
	},
	"6TS2": {
		ICAO:      "6TS2",
		Name:      "Dauenhauer Field Airport",
		City:      "Boyd",
		State:     "Texas",
		Country:   "US",
		Elevation: 906,
		Latitude:  29.0993995667,
		Longitude: -98.46559906,
		Timezone:  "America/Chicago",
	},
	"6TS3": {
		ICAO:      "6TS3",
		Name:      "Dale Acres Airport",
		City:      "Milford",
		State:     "Texas",
		Country:   "US",
		Elevation: 550,
		Latitude:  32.1599006653,
		Longitude: -96.9419021606,
		Timezone:  "America/Chicago",
	},
	"6TS4": {
		ICAO:      "6TS4",
		Name:      "Stampede Valley Airport",
		City:      "Moffat",
		State:     "Texas",
		Country:   "US",
		Elevation: 670,
		Latitude:  31.1863002777,
		Longitude: -97.3906021118,
		Timezone:  "America/Chicago",
	},
	"6TS6": {
		ICAO:      "6TS6",
		Name:      "Wood Crest Ranch Airport",
		City:      "Normangee",
		State:     "Texas",
		Country:   "US",
		Elevation: 410,
		Latitude:  31.0415992737,
		Longitude: -96.2015991211,
		Timezone:  "America/Chicago",
	},
	"6TS8": {
		ICAO:      "6TS8",
		Name:      "Rabb And Nobra Airport",
		City:      "Industry",
		State:     "Texas",
		Country:   "US",
		Elevation: 400,
		Latitude:  30.003333,
		Longitude: -96.581111,
		Timezone:  "America/Chicago",
	},
	"6TX2": {
		ICAO:      "6TX2",
		Name:      "Circle M Ranch Airport",
		City:      "Frelsburg",
		State:     "Texas",
		Country:   "US",
		Elevation: 340,
		Latitude:  29.8349990845,
		Longitude: -96.5143966675,
		Timezone:  "America/Chicago",
	},
	"6TX3": {
		ICAO:      "6TX3",
		Name:      "Drewery Airport",
		City:      "Mc Kinney",
		State:     "Texas",
		Country:   "US",
		Elevation: 700,
		Latitude:  33.2335014343,
		Longitude: -96.7835998535,
		Timezone:  "America/Chicago",
	},
	"6TX4": {
		ICAO:      "6TX4",
		Name:      "Whitfield Airport",
		City:      "Kress",
		State:     "Texas",
		Country:   "US",
		Elevation: 3330,
		Latitude:  34.3433990479,
		Longitude: -101.5439987183,
		Timezone:  "America/Chicago",
	},
	"6TX6": {
		ICAO:      "6TX6",
		Name:      "Jtj Ranch Airport",
		City:      "Corsicana",
		State:     "Texas",
		Country:   "US",
		Elevation: 350,
		Latitude:  32.0825996399,
		Longitude: -96.3566970825,
		Timezone:  "America/Chicago",
	},
	"6TX7": {
		ICAO:      "6TX7",
		Name:      "Flying L Airpark",
		City:      "Mansfield",
		State:     "Texas",
		Country:   "US",
		Elevation: 680,
		Latitude:  32.5376014709,
		Longitude: -97.1307983398,
		Timezone:  "America/Chicago",
	},
	"6TX8": {
		ICAO:      "6TX8",
		Name:      "Hess Airport",
		City:      "Mansfield",
		State:     "Texas",
		Country:   "US",
		Elevation: 673,
		Latitude:  32.5559997559,
		Longitude: -97.2063980103,
		Timezone:  "America/Chicago",
	},
	"6TX9": {
		ICAO:      "6TX9",
		Name:      "Stovall Ranch Nr 4 Airport",
		City:      "Marathon",
		State:     "Texas",
		Country:   "US",
		Elevation: 3010,
		Latitude:  29.8549003601,
		Longitude: -102.7919998169,
		Timezone:  "America/Chicago",
	},
	"6VA0": {
		ICAO:      "6VA0",
		Name:      "Remo Private Airport",
		City:      "Reidville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 13,
		Latitude:  37.8186988831,
		Longitude: -76.3177032471,
		Timezone:  "America/New_York",
	},
	"6VA2": {
		ICAO:      "6VA2",
		Name:      "Loury Lester Airpark",
		City:      "Martinsville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1030,
		Latitude:  36.7417984009,
		Longitude: -79.8478012085,
		Timezone:  "America/New_York",
	},
	"6VA4": {
		ICAO:      "6VA4",
		Name:      "Trussmark Airport",
		City:      "Salem",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1030,
		Latitude:  37.2849006653,
		Longitude: -80.0698013306,
		Timezone:  "America/New_York",
	},
	"6VA9": {
		ICAO:      "6VA9",
		Name:      "Bush Airport",
		City:      "Woolwine",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1500,
		Latitude:  36.8426017761,
		Longitude: -80.2384033203,
		Timezone:  "America/New_York",
	},
	"6WA2": {
		ICAO:      "6WA2",
		Name:      "Gower Field",
		City:      "Olympia",
		State:     "Washington",
		Country:   "US",
		Elevation: 130,
		Latitude:  47.1114997864,
		Longitude: -122.8209991455,
		Timezone:  "America/Los_Angeles",
	},
	"6WA3": {
		ICAO:      "6WA3",
		Name:      "Green Acres Airport",
		City:      "Basin City",
		State:     "Washington",
		Country:   "US",
		Elevation: 1000,
		Latitude:  46.6167984009,
		Longitude: -119.2740020752,
		Timezone:  "America/Los_Angeles",
	},
	"6WA4": {
		ICAO:      "6WA4",
		Name:      "Ochoa Field",
		City:      "Othello",
		State:     "Washington",
		Country:   "US",
		Elevation: 1149,
		Latitude:  46.8628997803,
		Longitude: -119.138999939,
		Timezone:  "America/Los_Angeles",
	},
	"6WA5": {
		ICAO:      "6WA5",
		Name:      "Wilding Farm Airport",
		City:      "Shaw Island",
		State:     "Washington",
		Country:   "US",
		Elevation: 60,
		Latitude:  48.5665016174,
		Longitude: -122.9489974976,
		Timezone:  "America/Los_Angeles",
	},
	"6WA6": {
		ICAO:      "6WA6",
		Name:      "Carr Airport",
		City:      "Pasco",
		State:     "Washington",
		Country:   "US",
		Elevation: 625,
		Latitude:  46.2784996033,
		Longitude: -118.9560012817,
		Timezone:  "America/Los_Angeles",
	},
	"6WI0": {
		ICAO:      "6WI0",
		Name:      "Cub Acres Airport",
		City:      "Prairie Farm",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1000,
		Latitude:  45.1805000305,
		Longitude: -91.987701416,
		Timezone:  "America/Chicago",
	},
	"6WI1": {
		ICAO:      "6WI1",
		Name:      "Winch Airfield",
		City:      "Wisconsin Rapids",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1120,
		Latitude:  44.4952011108,
		Longitude: -89.9125976563,
		Timezone:  "America/Chicago",
	},
	"6WI2": {
		ICAO:      "6WI2",
		Name:      "St Croix Riviera Airport",
		City:      "River Falls",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 895,
		Latitude:  44.8438987732,
		Longitude: -92.7557983398,
		Timezone:  "America/Chicago",
	},
	"6WI3": {
		ICAO:      "6WI3",
		Name:      "Will-Be-Gone Airport",
		City:      "Birchwood",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1290,
		Latitude:  45.6772003174,
		Longitude: -91.5625,
		Timezone:  "America/Chicago",
	},
	"6WI4": {
		ICAO:      "6WI4",
		Name:      "Saxon Airport",
		City:      "Saxon",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 950,
		Latitude:  46.515499115,
		Longitude: -90.431602478,
		Timezone:  "America/Chicago",
	},
	"6WI5": {
		ICAO:      "6WI5",
		Name:      "Dolata Airport",
		City:      "Stiles",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 615,
		Latitude:  44.8891983032,
		Longitude: -88.0208969116,
		Timezone:  "America/Chicago",
	},
	"6WI6": {
		ICAO:      "6WI6",
		Name:      "Mick Shier Field",
		City:      "Rosiere",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 650,
		Latitude:  44.6839981079,
		Longitude: -87.6279983521,
		Timezone:  "America/Chicago",
	},
	"6WI7": {
		ICAO:      "6WI7",
		Name:      "Walnut Wash Airport",
		City:      "Orfordville",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 900,
		Latitude:  42.6104011536,
		Longitude: -89.2761001587,
		Timezone:  "America/Chicago",
	},
	"6WI9": {
		ICAO:      "6WI9",
		Name:      "Rex Ranch Airport",
		City:      "Montello",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 790,
		Latitude:  43.7966003418,
		Longitude: -89.2678985596,
		Timezone:  "America/Chicago",
	},
	"6WN5": {
		ICAO:      "6WN5",
		Name:      "Airwolfe Airport",
		City:      "Stockholm",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1130,
		Latitude:  44.4998016357,
		Longitude: -92.1987991333,
		Timezone:  "America/Chicago",
	},
	"6WN6": {
		ICAO:      "6WN6",
		Name:      "Fountain Prairie Airport",
		City:      "Columbus",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 880,
		Latitude:  43.3800010681,
		Longitude: -89.0106964111,
		Timezone:  "America/Chicago",
	},
	"6XA4": {
		ICAO:      "6XA4",
		Name:      "Zadow Airstrip",
		State:     "Texas",
		Country:   "US",
		Latitude:  29.9917385509,
		Longitude: -95.954353809,
		Timezone:  "America/Chicago",
	},
	"6XS2": {
		ICAO:      "6XS2",
		Name:      "Luscombe Acres Airport",
		City:      "Alvarado",
		State:     "Texas",
		Country:   "US",
		Elevation: 700,
		Latitude:  32.3460006714,
		Longitude: -97.1992034912,
		Timezone:  "America/Chicago",
	},
	"6XS3": {
		ICAO:      "6XS3",
		Name:      "Mullins Landing Airport",
		City:      "Farmersville",
		State:     "Texas",
		Country:   "US",
		Elevation: 550,
		Latitude:  33.1385002136,
		Longitude: -96.3854980469,
		Timezone:  "America/Chicago",
	},
	"6XS4": {
		ICAO:      "6XS4",
		Name:      "Noelke Ranch Airport",
		City:      "Mertzon",
		State:     "Texas",
		Country:   "US",
		Elevation: 2500,
		Latitude:  31.1667995453,
		Longitude: -101,
		Timezone:  "America/Chicago",
	},
	"6XS5": {
		ICAO:      "6XS5",
		Name:      "Cannon Aviation Airport",
		City:      "Edcouch",
		State:     "Texas",
		Country:   "US",
		Elevation: 65,
		Latitude:  26.3087005615,
		Longitude: -97.9543991089,
		Timezone:  "America/Chicago",
	},
	"6XS7": {
		ICAO:      "6XS7",
		Name:      "Eugene's Dream Airport",
		City:      "Springtown",
		State:     "Texas",
		Country:   "US",
		Elevation: 905,
		Latitude:  33.04529953,
		Longitude: -97.6747970581,
		Timezone:  "America/Chicago",
	},
	"6XS8": {
		ICAO:      "6XS8",
		Name:      "Vultures Row Airport",
		City:      "Sanger",
		State:     "Texas",
		Country:   "US",
		Elevation: 710,
		Latitude:  33.3400001526,
		Longitude: -97.1031036377,
		Timezone:  "America/Chicago",
	},
	"6XS9": {
		ICAO:      "6XS9",
		Name:      "Harmony Field",
		City:      "Elmendorf",
		State:     "Texas",
		Country:   "US",
		Elevation: 570,
		Latitude:  29.2961006165,
		Longitude: -98.3795013428,
		Timezone:  "America/Chicago",
	},
	"70AK": {
		ICAO:      "70AK",
		Name:      "Bangerter Field",
		City:      "Soldotna",
		State:     "Alaska",
		Country:   "US",
		Elevation: 250,
		Latitude:  60.517778,
		Longitude: -150.959722,
		Timezone:  "America/Anchorage",
	},
	"70AR": {
		ICAO:      "70AR",
		Name:      "Randal Field",
		City:      "Gosnell",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 242,
		Latitude:  35.9944992065,
		Longitude: -89.9972991943,
		Timezone:  "America/Chicago",
	},
	"70CL": {
		ICAO:      "70CL",
		Name:      "Amargosa Airport",
		City:      "Death Valley Junction",
		State:     "California",
		Country:   "US",
		Elevation: 2037,
		Latitude:  36.2957992554,
		Longitude: -116.4260025024,
		Timezone:  "America/Los_Angeles",
	},
	"70II": {
		ICAO:      "70II",
		Name:      "Nasby Airport",
		City:      "Shelbyville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 830,
		Latitude:  39.5217018127,
		Longitude: -85.6864013672,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"70IL": {
		ICAO:      "70IL",
		Name:      "Murphy Farms Airport",
		City:      "Farmersville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 645,
		Latitude:  39.4235992432,
		Longitude: -89.5682983398,
		Timezone:  "America/Chicago",
	},
	"70IN": {
		ICAO:      "70IN",
		Name:      "Lewis Airfield",
		City:      "Milan",
		State:     "Indiana",
		Country:   "US",
		Elevation: 989,
		Latitude:  39.1817016602,
		Longitude: -85.1512985229,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"70KY": {
		ICAO:      "70KY",
		Name:      "Vine Grove Airport",
		City:      "Vine Grove",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 680,
		Latitude:  37.8158988953,
		Longitude: -85.9646987915,
		Timezone:  "America/New_York",
	},
	"70LA": {
		ICAO:      "70LA",
		Name:      "Roland Airport",
		City:      "Hineston",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 220,
		Latitude:  31.1690998077,
		Longitude: -92.7563018799,
		Timezone:  "America/Chicago",
	},
	"70LL": {
		ICAO:      "70LL",
		Name:      "Adams Restricted Landing Area Number 1",
		City:      "Pontiac",
		State:     "Illinois",
		Country:   "US",
		Elevation: 640,
		Latitude:  40.9416999817,
		Longitude: -88.7500991821,
		Timezone:  "America/Chicago",
	},
	"70MD": {
		ICAO:      "70MD",
		Name:      "Washington Field",
		City:      "Nanjemoy",
		State:     "Maryland",
		Country:   "US",
		Elevation: 95,
		Latitude:  38.458734,
		Longitude: -77.23498,
		Timezone:  "America/New_York",
	},
	"70MN": {
		ICAO:      "70MN",
		Name:      "Harry-Walt Airport",
		City:      "Cedar",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 940,
		Latitude:  45.3518981934,
		Longitude: -93.2461013794,
		Timezone:  "America/Chicago",
	},
	"70MO": {
		ICAO:      "70MO",
		Name:      "Andrews Airport",
		City:      "Carthage",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1070,
		Latitude:  37.2178001404,
		Longitude: -94.1463012695,
		Timezone:  "America/Chicago",
	},
	"70NC": {
		ICAO:      "70NC",
		Name:      "Martindale Executive Airpark",
		City:      "Paschall",
		State:     "Virginia",
		Country:   "US",
		Elevation: 300,
		Latitude:  36.5446014404,
		Longitude: -78.1480026245,
		Timezone:  "America/New_York",
	},
	"70NJ": {
		ICAO:      "70NJ",
		Name:      "Parker Airport",
		City:      "Asbury",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 307,
		Latitude:  40.6769981384,
		Longitude: -75.024597168,
		Timezone:  "America/New_York",
	},
	"70OH": {
		ICAO:      "70OH",
		Name:      "Shenandoah Airpark",
		City:      "Shenandoah",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1210,
		Latitude:  40.9201011658,
		Longitude: -82.4791030884,
		Timezone:  "America/New_York",
	},
	"70OK": {
		ICAO:      "70OK",
		Name:      "Old 66 Strip",
		City:      "Miami",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 853,
		Latitude:  36.8336982727,
		Longitude: -94.9040985107,
		Timezone:  "America/Chicago",
	},
	"70OR": {
		ICAO:      "70OR",
		Name:      "Goodin Creek Airport",
		City:      "Gaston",
		State:     "Oregon",
		Country:   "US",
		Elevation: 245,
		Latitude:  45.4095993042,
		Longitude: -123.1490020752,
		Timezone:  "America/Los_Angeles",
	},
	"70PN": {
		ICAO:      "70PN",
		Name:      "Sanders Personal Use Airport",
		City:      "Sugar Grove",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1455,
		Latitude:  41.989200592,
		Longitude: -79.3011016846,
		Timezone:  "America/New_York",
	},
	"70TE": {
		ICAO:      "70TE",
		Name:      "Flying Heart Ranch Airport",
		City:      "Waco",
		State:     "Texas",
		Country:   "US",
		Elevation: 415,
		Latitude:  31.5020999908,
		Longitude: -97.0808029175,
		Timezone:  "America/Chicago",
	},
	"70VA": {
		ICAO:      "70VA",
		Name:      "Burnt Chimney Airport",
		City:      "Burnt Chimney",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1034,
		Latitude:  37.0754013062,
		Longitude: -79.8273010254,
		Timezone:  "America/New_York",
	},
	"70WI": {
		ICAO:      "70WI",
		Name:      "Kanten Field",
		City:      "Baldwin",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1160,
		Latitude:  45.0321998596,
		Longitude: -92.3544006348,
		Timezone:  "America/Chicago",
	},
	"70XS": {
		ICAO:      "70XS",
		Name:      "Restoration Ranch Airport",
		City:      "Wimberley",
		State:     "Texas",
		Country:   "US",
		Elevation: 840,
		Latitude:  29.9932994843,
		Longitude: -98.043296814,
		Timezone:  "America/Chicago",
	},
	"71AK": {
		ICAO:      "71AK",
		Name:      "Bluff Park Farm Airport",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 110,
		Latitude:  61.5275993347,
		Longitude: -149.4960021973,
		Timezone:  "America/Anchorage",
	},
	"71AL": {
		ICAO:      "71AL",
		Name:      "Sells Airport",
		City:      "Greenville",
		State:     "Alabama",
		Country:   "US",
		Elevation: 380,
		Latitude:  31.808786,
		Longitude: -86.523525,
		Timezone:  "America/Chicago",
	},
	"71CL": {
		ICAO:      "71CL",
		Name:      "Gunnersfield Ranch Airport",
		City:      "Delevan",
		State:     "California",
		Country:   "US",
		Elevation: 62,
		Latitude:  39.3526992798,
		Longitude: -122.09400177,
		Timezone:  "America/Los_Angeles",
	},
	"71IN": {
		ICAO:      "71IN",
		Name:      "Morgan Airfield",
		City:      "Salem",
		State:     "Indiana",
		Country:   "US",
		Elevation: 922,
		Latitude:  38.6659011841,
		Longitude: -86.0057983398,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"71IS": {
		ICAO:      "71IS",
		Name:      "Ted's Place Airport",
		City:      "Paxton",
		State:     "Illinois",
		Country:   "US",
		Elevation: 780,
		Latitude:  40.4255981445,
		Longitude: -88.0006027222,
		Timezone:  "America/Chicago",
	},
	"71KS": {
		ICAO:      "71KS",
		Name:      "Stonehenge Airport",
		City:      "Williamstown",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1030,
		Latitude:  39.1041984558,
		Longitude: -95.2960968018,
		Timezone:  "America/Chicago",
	},
	"71KY": {
		ICAO:      "71KY",
		Name:      "Sunrise Acres Airport",
		City:      "Petersburg",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 465,
		Latitude:  39.0348014832,
		Longitude: -84.8727035522,
		Timezone:  "America/New_York",
	},
	"71LA": {
		ICAO:      "71LA",
		Name:      "Gustafson Airport",
		City:      "Tallulah",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 80,
		Latitude:  32.4331016541,
		Longitude: -91.3152008057,
		Timezone:  "America/Chicago",
	},
	"71LL": {
		ICAO:      "71LL",
		Name:      "Riverode Farms Airport",
		City:      "Pontiac",
		State:     "Illinois",
		Country:   "US",
		Elevation: 651,
		Latitude:  40.8424987793,
		Longitude: -88.5597991943,
		Timezone:  "America/Chicago",
	},
	"71ME": {
		ICAO:      "71ME",
		Name:      "Twin Eagles Airport",
		City:      "Naples",
		State:     "Maine",
		Country:   "US",
		Elevation: 280,
		Latitude:  43.9668998718,
		Longitude: -70.6010971069,
		Timezone:  "America/New_York",
	},
	"71MI": {
		ICAO:      "71MI",
		Name:      "Belleville (Johnston) Airport",
		City:      "Belleville",
		State:     "Michigan",
		Country:   "US",
		Elevation: 643,
		Latitude:  42.0843851004,
		Longitude: -83.5170149803,
		Timezone:  "America/Detroit",
	},
	"71MO": {
		ICAO:      "71MO",
		Name:      "Famuliner Farms Airport",
		City:      "Wakenda",
		State:     "Missouri",
		Country:   "US",
		Elevation: 655,
		Latitude:  39.2919998169,
		Longitude: -93.3841018677,
		Timezone:  "America/Chicago",
	},
	"71NC": {
		ICAO:      "71NC",
		Name:      "K & D Airport",
		City:      "Elon College",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 800,
		Latitude:  36.2633018494,
		Longitude: -79.4496994019,
		Timezone:  "America/New_York",
	},
	"71NE": {
		ICAO:      "71NE",
		Name:      "Neben Airport",
		City:      "Oconto",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2615,
		Latitude:  41.0355987549,
		Longitude: -99.8298034668,
		Timezone:  "America/Chicago",
	},
	"71OH": {
		ICAO:      "71OH",
		Name:      "Sattler Landing Strip",
		City:      "Swanton",
		State:     "Ohio",
		Country:   "US",
		Elevation: 700,
		Latitude:  41.6417007446,
		Longitude: -83.8749008179,
		Timezone:  "America/New_York",
	},
	"71OK": {
		ICAO:      "71OK",
		Name:      "Neuwirth Airstrip",
		City:      "Lawton",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1185,
		Latitude:  34.5761985779,
		Longitude: -98.483001709,
		Timezone:  "America/Chicago",
	},
	"71OR": {
		ICAO:      "71OR",
		Name:      "Cerny Airport",
		City:      "Seneca",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4765,
		Latitude:  44.2445,
		Longitude: -119.126242,
		Timezone:  "America/Los_Angeles",
	},
	"71PN": {
		ICAO:      "71PN",
		Name:      "Schulteis Field",
		City:      "New Oxford",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 550,
		Latitude:  39.91815,
		Longitude: -77.045533,
		Timezone:  "America/New_York",
	},
	"71TA": {
		ICAO:      "71TA",
		Name:      "Bates Field",
		City:      "Odessa",
		State:     "Texas",
		Country:   "US",
		Elevation: 2820,
		Latitude:  31.8334999084,
		Longitude: -102.2340011597,
		Timezone:  "America/Chicago",
	},
	"71TE": {
		ICAO:      "71TE",
		Name:      "Moursund Ranch Airport",
		City:      "Round Mountain",
		State:     "Texas",
		Country:   "US",
		Elevation: 1471,
		Latitude:  30.3780002594,
		Longitude: -98.3760986328,
		Timezone:  "America/Chicago",
	},
	"71TS": {
		ICAO:      "71TS",
		Name:      "Circle C Ranch Airport",
		City:      "Saginaw",
		State:     "Texas",
		Country:   "US",
		Elevation: 675,
		Latitude:  32.8959999084,
		Longitude: -97.2878036499,
		Timezone:  "America/Chicago",
	},
	"71TX": {
		ICAO:      "71TX",
		Name:      "Grier Airport",
		City:      "Boerne",
		State:     "Texas",
		Country:   "US",
		Elevation: 1450,
		Latitude:  29.8726997375,
		Longitude: -98.7322006226,
		Timezone:  "America/Chicago",
	},
	"71XS": {
		ICAO:      "71XS",
		Name:      "Flying V Airport",
		City:      "Sanger",
		State:     "Texas",
		Country:   "US",
		Elevation: 680,
		Latitude:  33.3465003967,
		Longitude: -97.2058029175,
		Timezone:  "America/Chicago",
	},
	"72CL": {
		ICAO:      "72CL",
		Name:      "Iron Mountain Pumping Plant Airport",
		City:      "Desert Center",
		State:     "California",
		Country:   "US",
		Elevation: 890,
		Latitude:  34.1342010498,
		Longitude: -115.1190032959,
		Timezone:  "America/Los_Angeles",
	},
	"72CO": {
		ICAO:      "72CO",
		Name:      "Idler Bro's Airport",
		City:      "Kirk",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4200,
		Latitude:  39.6794013977,
		Longitude: -102.591003418,
		Timezone:  "America/Denver",
	},
	"72FL": {
		ICAO:      "72FL",
		Name:      "Gator Airpark",
		City:      "St Cloud",
		State:     "Florida",
		Country:   "US",
		Elevation: 68,
		Latitude:  28.1575164795,
		Longitude: -81.2347946167,
		Timezone:  "America/New_York",
	},
	"72GA": {
		ICAO:      "72GA",
		Name:      "Barbour Island Airport",
		City:      "Shellman Bluff",
		State:     "Georgia",
		Country:   "US",
		Elevation: 10,
		Latitude:  31.5835990906,
		Longitude: -81.2332000732,
		Timezone:  "America/New_York",
	},
	"72IN": {
		ICAO:      "72IN",
		Name:      "Gutwein Airport",
		City:      "Monon",
		State:     "Indiana",
		Country:   "US",
		Elevation: 680,
		Latitude:  40.9119987488,
		Longitude: -86.8739013672,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"72IS": {
		ICAO:      "72IS",
		Name:      "Wiley Updike Airport",
		City:      "Plano",
		State:     "Illinois",
		Country:   "US",
		Elevation: 650,
		Latitude:  41.6253013611,
		Longitude: -88.5745010376,
		Timezone:  "America/Chicago",
	},
	"72KS": {
		ICAO:      "72KS",
		Name:      "Mount Muncie Airport",
		City:      "Lansing",
		State:     "Kansas",
		Country:   "US",
		Elevation: 750,
		Latitude:  39.2733001709,
		Longitude: -94.8802032471,
		Timezone:  "America/Chicago",
	},
	"72KY": {
		ICAO:      "72KY",
		Name:      "Carr Airport",
		City:      "Williamsburg",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 1105,
		Latitude:  36.7650985718,
		Longitude: -84.1994018555,
		Timezone:  "America/New_York",
	},
	"72LL": {
		ICAO:      "72LL",
		Name:      "Foley Airport",
		City:      "Princeton",
		State:     "Illinois",
		Country:   "US",
		Elevation: 690,
		Latitude:  41.3736000061,
		Longitude: -89.3444976807,
		Timezone:  "America/Chicago",
	},
	"72ME": {
		ICAO:      "72ME",
		Name:      "Philbrick Mountain Airport",
		City:      "Ellsworth",
		State:     "Maine",
		Country:   "US",
		Elevation: 667,
		Latitude:  44.635278,
		Longitude: -68.6325,
		Timezone:  "America/New_York",
	},
	"72MI": {
		ICAO:      "72MI",
		Name:      "Northwoods Airport",
		City:      "Watersmeet",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1642,
		Latitude:  46.285900116,
		Longitude: -89.2774963379,
		Timezone:  "America/Menominee",
	},
	"72MN": {
		ICAO:      "72MN",
		Name:      "Bardwell Airstrip",
		City:      "Clearbrook",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1260,
		Latitude:  47.7524986267,
		Longitude: -95.3349990845,
		Timezone:  "America/Chicago",
	},
	"72ND": {
		ICAO:      "72ND",
		Name:      "Berg Field",
		City:      "Northwood",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1111,
		Latitude:  47.6610984802,
		Longitude: -97.5920028687,
		Timezone:  "America/Chicago",
	},
	"72NE": {
		ICAO:      "72NE",
		Name:      "J&J Airport",
		City:      "Springfield",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1135,
		Latitude:  41.0792007446,
		Longitude: -96.1530990601,
		Timezone:  "America/Chicago",
	},
	"72NM": {
		ICAO:      "72NM",
		Name:      "Otero Mill Airport",
		City:      "La Luz",
		State:     "New Mexico",
		Country:   "US",
		Elevation: 4603,
		Latitude:  32.9584007263,
		Longitude: -105.9629974365,
		Timezone:  "America/Denver",
	},
	"72OH": {
		ICAO:      "72OH",
		Name:      "Zeigler Landing Strip",
		City:      "Swanton",
		State:     "Ohio",
		Country:   "US",
		Elevation: 680,
		Latitude:  41.6044998169,
		Longitude: -83.8676986694,
		Timezone:  "America/New_York",
	},
	"72OI": {
		ICAO:      "72OI",
		Name:      "Tathams' STOLport",
		City:      "Findlay",
		State:     "Ohio",
		Country:   "US",
		Elevation: 817,
		Latitude:  41.0736999512,
		Longitude: -83.6847000122,
		Timezone:  "America/New_York",
	},
	"72OR": {
		ICAO:      "72OR",
		Name:      "Ochs Private Airport",
		City:      "Madras",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1790,
		Latitude:  44.7843017578,
		Longitude: -120.9550018311,
		Timezone:  "America/Los_Angeles",
	},
	"72TA": {
		ICAO:      "72TA",
		Name:      "Pierce Field",
		City:      "Port O'Connor",
		State:     "Texas",
		Country:   "US",
		Elevation: 5,
		Latitude:  28.4585990906,
		Longitude: -96.2919006348,
		Timezone:  "America/Chicago",
	},
	"72TE": {
		ICAO:      "72TE",
		Name:      "Chaparrosa Ranch Airport",
		City:      "La Pryor",
		State:     "Texas",
		Country:   "US",
		Elevation: 680,
		Latitude:  28.8794002533,
		Longitude: -99.9906005859,
		Timezone:  "America/Chicago",
	},
	"72TS": {
		ICAO:      "72TS",
		Name:      "Galaxy Ranch Airport",
		City:      "Sheridan",
		State:     "Texas",
		Country:   "US",
		Elevation: 178,
		Latitude:  29.3607997894,
		Longitude: -96.6224975586,
		Timezone:  "America/Chicago",
	},
	"72WI": {
		ICAO:      "72WI",
		Name:      "Windhaven Airport",
		City:      "Collins",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 910,
		Latitude:  44.0461006165,
		Longitude: -87.9928970337,
		Timezone:  "America/Chicago",
	},
	"72XS": {
		ICAO:      "72XS",
		Name:      "El Paisano Airport",
		City:      "Round Top",
		State:     "Texas",
		Country:   "US",
		Elevation: 340,
		Latitude:  30.0335998535,
		Longitude: -96.6847000122,
		Timezone:  "America/Chicago",
	},
	"73AR": {
		ICAO:      "73AR",
		Name:      "Dogwood Airport",
		City:      "Cabot",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 280,
		Latitude:  35.048500061,
		Longitude: -92.0537033081,
		Timezone:  "America/Chicago",
	},
	"73AZ": {
		ICAO:      "73AZ",
		Name:      "Dunton Ranch Airport",
		City:      "Kingman",
		State:     "Arizona",
		Country:   "US",
		Elevation: 5115,
		Latitude:  35.2853012085,
		Longitude: -113.361000061,
		Timezone:  "America/Phoenix",
	},
	"73CA": {
		ICAO:      "73CA",
		Name:      "Bear Valley STOLport",
		City:      "Bear Valley",
		State:     "California",
		Country:   "US",
		Elevation: 7073,
		Latitude:  38.4588012695,
		Longitude: -120.0400009155,
		Timezone:  "America/Los_Angeles",
	},
	"73CL": {
		ICAO:      "73CL",
		Name:      "Julian Hinds Pump Plant Airstrip",
		City:      "Desert Center",
		State:     "California",
		Country:   "US",
		Elevation: 1335,
		Latitude:  33.6988983154,
		Longitude: -115.6370010376,
		Timezone:  "America/Los_Angeles",
	},
	"73CO": {
		ICAO:      "73CO",
		Name:      "Woods STOLport",
		City:      "Montrose",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6120,
		Latitude:  38.3889007568,
		Longitude: -107.8229980469,
		Timezone:  "America/Denver",
	},
	"73FD": {
		ICAO:      "73FD",
		Name:      "Brookins Air Strip",
		City:      "Chiefland",
		State:     "Florida",
		Country:   "US",
		Elevation: 45,
		Latitude:  29.510799408,
		Longitude: -82.8600997925,
		Timezone:  "America/New_York",
	},
	"73IA": {
		ICAO:      "73IA",
		Name:      "Harris Field",
		City:      "Mason City",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1180,
		Latitude:  43.23559952,
		Longitude: -93.20829773,
		Timezone:  "America/Chicago",
	},
	"73II": {
		ICAO:      "73II",
		Name:      "Nulltown Wingnuts Ultralightport",
		City:      "Connersville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 765,
		Latitude:  39.5717010498,
		Longitude: -85.1585998535,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"73IN": {
		ICAO:      "73IN",
		Name:      "Benham Airport",
		City:      "Lyons",
		State:     "Indiana",
		Country:   "US",
		Elevation: 500,
		Latitude:  38.9766998291,
		Longitude: -87.0224990845,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"73KS": {
		ICAO:      "73KS",
		Name:      "Philip Ranch Airport",
		City:      "Hays",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1923,
		Latitude:  38.7806015015,
		Longitude: -99.2211990356,
		Timezone:  "America/Chicago",
	},
	"73KY": {
		ICAO:      "73KY",
		Name:      "Brooks Field",
		City:      "Brooks",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 480,
		Latitude:  38.0332984924,
		Longitude: -85.8007965088,
		Timezone:  "America/Kentucky/Louisville",
	},
	"73MN": {
		ICAO:      "73MN",
		Name:      "Paynes Airport",
		City:      "DeGraff",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1035,
		Latitude:  45.1353988647,
		Longitude: -95.5167007446,
		Timezone:  "America/Chicago",
	},
	"73MU": {
		ICAO:      "73MU",
		Name:      "Sky-Go Farms Airport",
		City:      "Fulton",
		State:     "Missouri",
		Country:   "US",
		Elevation: 878,
		Latitude:  38.8856010437,
		Longitude: -91.9691009521,
		Timezone:  "America/Chicago",
	},
	"73ND": {
		ICAO:      "73ND",
		Name:      "M Heart Ranch Airport",
		City:      "Nortonville",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1535,
		Latitude:  46.5716018677,
		Longitude: -98.6884002686,
		Timezone:  "America/Chicago",
	},
	"73NE": {
		ICAO:      "73NE",
		Name:      "Bornmann Field",
		City:      "Waverly",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1180,
		Latitude:  41.0266990662,
		Longitude: -96.5306015015,
		Timezone:  "America/Chicago",
	},
	"73NY": {
		ICAO:      "73NY",
		Name:      "Olmstead Landing Strip",
		City:      "Masonville",
		State:     "New York",
		Country:   "US",
		Elevation: 1160,
		Latitude:  42.265767,
		Longitude: -75.310033,
		Timezone:  "America/New_York",
	},
	"73OH": {
		ICAO:      "73OH",
		Name:      "Thompson Drag Raceway Airport",
		City:      "Thompson",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1100,
		Latitude:  41.6469993591,
		Longitude: -81.0086975098,
		Timezone:  "America/New_York",
	},
	"73OI": {
		ICAO:      "73OI",
		Name:      "Rick's Airport",
		City:      "Montville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1265,
		Latitude:  41.5745010376,
		Longitude: -81.0756988525,
		Timezone:  "America/New_York",
	},
	"73OK": {
		ICAO:      "73OK",
		Name:      "Haxton Airport",
		City:      "Leedey",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 2050,
		Latitude:  35.8456001282,
		Longitude: -99.3373031616,
		Timezone:  "America/Chicago",
	},
	"73OR": {
		ICAO:      "73OR",
		Name:      "Ribbon Ridge Airport",
		City:      "Newberg",
		State:     "Oregon",
		Country:   "US",
		Elevation: 490,
		Latitude:  45.3703994751,
		Longitude: -123.0650024414,
		Timezone:  "America/Los_Angeles",
	},
	"73PA": {
		ICAO:      "73PA",
		Name:      "Gilbert Airport",
		City:      "York",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 530,
		Latitude:  39.9295005798,
		Longitude: -76.7707977295,
		Timezone:  "America/New_York",
	},
	"73PN": {
		ICAO:      "73PN",
		Name:      "Finkhaven Airport",
		City:      "Trout Run",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 640,
		Latitude:  41.3684005737,
		Longitude: -77.0710983276,
		Timezone:  "America/New_York",
	},
	"73TA": {
		ICAO:      "73TA",
		Name:      "Salado Airport",
		City:      "Salado",
		State:     "Texas",
		Country:   "US",
		Elevation: 670,
		Latitude:  30.9207000732,
		Longitude: -97.5397033691,
		Timezone:  "America/Chicago",
	},
	"73TE": {
		ICAO:      "73TE",
		Name:      "Moore Private Airport",
		City:      "Azle",
		State:     "Texas",
		Country:   "US",
		Elevation: 670,
		Latitude:  32.9249992371,
		Longitude: -97.5582962036,
		Timezone:  "America/Chicago",
	},
	"73TX": {
		ICAO:      "73TX",
		Name:      "Frerich Ranch Airport",
		City:      "Brackettville",
		State:     "Texas",
		Country:   "US",
		Elevation: 995,
		Latitude:  29.1718997955,
		Longitude: -100.641998291,
		Timezone:  "America/Chicago",
	},
	"73WA": {
		ICAO:      "73WA",
		Name:      "7 Bays Airport",
		City:      "Davenport",
		State:     "Washington",
		Country:   "US",
		Elevation: 1580,
		Latitude:  47.8510017395,
		Longitude: -118.3300018311,
		Timezone:  "America/Los_Angeles",
	},
	"73WI": {
		ICAO:      "73WI",
		Name:      "Vetterkind Strip",
		City:      "New Auburn",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1080,
		Latitude:  45.2405014038,
		Longitude: -91.5087966919,
		Timezone:  "America/Chicago",
	},
	"73WT": {
		ICAO:      "73WT",
		Name:      "Mc Gregor Airport",
		City:      "Hooper",
		State:     "Washington",
		Country:   "US",
		Elevation: 1800,
		Latitude:  46.723333,
		Longitude: -118.099444,
		Timezone:  "America/Los_Angeles",
	},
	"73XS": {
		ICAO:      "73XS",
		Name:      "Dearing Ranch Airport",
		City:      "Gordon",
		State:     "Texas",
		Country:   "US",
		Elevation: 1055,
		Latitude:  32.4459991455,
		Longitude: -98.3712005615,
		Timezone:  "America/Chicago",
	},
	"74AK": {
		ICAO:      "74AK",
		Name:      "Puviaq Airport",
		City:      "Barrow",
		State:     "Alaska",
		Country:   "US",
		Elevation: 23,
		Latitude:  70.7155990601,
		Longitude: -154.4069976807,
		Timezone:  "America/Anchorage",
	},
	"74AZ": {
		ICAO:      "74AZ",
		Name:      "Rancho San Marcos Airport",
		City:      "Tombstone",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4353,
		Latitude:  31.619611,
		Longitude: -110.047361,
		Timezone:  "America/Phoenix",
	},
	"74CL": {
		ICAO:      "74CL",
		Name:      "Di Giorgio Ranch Landing Strip",
		City:      "Di Giorgio",
		State:     "California",
		Country:   "US",
		Elevation: 517,
		Latitude:  35.2569007874,
		Longitude: -118.8330001831,
		Timezone:  "America/Los_Angeles",
	},
	"74CO": {
		ICAO:      "74CO",
		Name:      "Westberg-Rosling Farms Airport",
		City:      "Roggen",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4857,
		Latitude:  40.0471992493,
		Longitude: -104.2089996338,
		Timezone:  "America/Denver",
	},
	"74FD": {
		ICAO:      "74FD",
		Name:      "Clarksville Airport",
		City:      "Clarksville",
		State:     "Florida",
		Country:   "US",
		Elevation: 90,
		Latitude:  30.464099884,
		Longitude: -85.1771011353,
		Timezone:  "America/Chicago",
	},
	"74FL": {
		ICAO:      "74FL",
		Name:      "Jefferson Landings Airport",
		City:      "Monticello",
		State:     "Florida",
		Country:   "US",
		Elevation: 120,
		Latitude:  30.5846004486,
		Longitude: -83.707901001,
		Timezone:  "America/New_York",
	},
	"74GA": {
		ICAO:      "74GA",
		Name:      "Bowens Mill Christian Center Airport",
		City:      "Fitzgerald",
		State:     "Georgia",
		Country:   "US",
		Elevation: 362,
		Latitude:  31.8346004486,
		Longitude: -83.21849823,
		Timezone:  "America/New_York",
	},
	"74IN": {
		ICAO:      "74IN",
		Name:      "Tucker Farms Airport",
		City:      "Montpelier",
		State:     "Indiana",
		Country:   "US",
		Elevation: 865,
		Latitude:  40.5684013367,
		Longitude: -85.3443984985,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"74KS": {
		ICAO:      "74KS",
		Name:      "Ensminger Airport",
		City:      "Moran",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1070,
		Latitude:  37.9333992004,
		Longitude: -95.2172012329,
		Timezone:  "America/Chicago",
	},
	"74KY": {
		ICAO:      "74KY",
		Name:      "Terry Field",
		City:      "Cunningham",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 370,
		Latitude:  36.8987007141,
		Longitude: -88.8467025757,
		Timezone:  "America/Chicago",
	},
	"74MN": {
		ICAO:      "74MN",
		Name:      "Frisch Personal Airport",
		City:      "Dumont",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1046,
		Latitude:  45.6972007751,
		Longitude: -96.4313964844,
		Timezone:  "America/Chicago",
	},
	"74MU": {
		ICAO:      "74MU",
		Name:      "Koala-T-Field Airport",
		City:      "Hawk Point",
		State:     "Missouri",
		Country:   "US",
		Elevation: 690,
		Latitude:  39.0400009155,
		Longitude: -91.1304016113,
		Timezone:  "America/Chicago",
	},
	"74NC": {
		ICAO:      "74NC",
		Name:      "Benton Farm Airport",
		City:      "Whartonville",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 9,
		Latitude:  35.0964012146,
		Longitude: -76.629699707,
		Timezone:  "America/New_York",
	},
	"74NY": {
		ICAO:      "74NY",
		Name:      "New Salem Aerodrome",
		City:      "Farmington",
		State:     "New York",
		Country:   "US",
		Elevation: 660,
		Latitude:  43.0055999756,
		Longitude: -77.2913970947,
		Timezone:  "America/New_York",
	},
	"74OK": {
		ICAO:      "74OK",
		Name:      "W.G. Anderson Memorial Airport",
		City:      "Marietta",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 825,
		Latitude:  33.9500999451,
		Longitude: -97.1502990723,
		Timezone:  "America/Chicago",
	},
	"74OR": {
		ICAO:      "74OR",
		Name:      "Stan Jost Airport",
		City:      "Newberg",
		State:     "Oregon",
		Country:   "US",
		Elevation: 180,
		Latitude:  45.3236999512,
		Longitude: -123.0189971924,
		Timezone:  "America/Los_Angeles",
	},
	"74PA": {
		ICAO:      "74PA",
		Name:      "Shontz Airport",
		City:      "Zieglerville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 300,
		Latitude:  40.2918014526,
		Longitude: -75.4663009644,
		Timezone:  "America/New_York",
	},
	"74TA": {
		ICAO:      "74TA",
		Name:      "Glass Ranch Airport",
		City:      "Carrizo Springs",
		State:     "Texas",
		Country:   "US",
		Elevation: 800,
		Latitude:  28.4503002167,
		Longitude: -100.1500015259,
		Timezone:  "America/Chicago",
	},
	"74TE": {
		ICAO:      "74TE",
		Name:      "Flat Top Ranch Airport",
		City:      "Walnut Springs",
		State:     "Texas",
		Country:   "US",
		Elevation: 960,
		Latitude:  32.05989838,
		Longitude: -97.79470062,
		Timezone:  "America/Chicago",
	},
	"74TX": {
		ICAO:      "74TX",
		Name:      "Fort Clark Springs Airport",
		City:      "Brackettville",
		State:     "Texas",
		Country:   "US",
		Elevation: 1106,
		Latitude:  29.2866001129,
		Longitude: -100.4339981079,
		Timezone:  "America/Chicago",
	},
	"74WA": {
		ICAO:      "74WA",
		Name:      "Mattawa Air Strip",
		City:      "Mattawa",
		State:     "Washington",
		Country:   "US",
		Elevation: 750,
		Latitude:  46.7326011658,
		Longitude: -119.702003479,
		Timezone:  "America/Los_Angeles",
	},
	"74WT": {
		ICAO:      "74WT",
		Name:      "Blowout Bench Airport",
		City:      "Ellensburg",
		State:     "Washington",
		Country:   "US",
		Elevation: 2065,
		Latitude:  47.111011,
		Longitude: -120.710742,
		Timezone:  "America/Los_Angeles",
	},
	"74XA": {
		ICAO:      "74XA",
		Name:      "Gun Barrel City Airpark",
		State:     "Texas",
		Country:   "US",
		Elevation: 385,
		Latitude:  32.3551499558,
		Longitude: -96.1456650496,
		Timezone:  "America/Chicago",
	},
	"74XS": {
		ICAO:      "74XS",
		Name:      "Charles J Hughes Ranch Airport",
		City:      "Big Lake",
		State:     "Texas",
		Country:   "US",
		Elevation: 2520,
		Latitude:  31.418800354,
		Longitude: -101.4820022583,
		Timezone:  "America/Chicago",
	},
	"75AK": {
		ICAO:      "75AK",
		Name:      "Henley Airport",
		City:      "Kenai",
		State:     "Alaska",
		Country:   "US",
		Elevation: 100,
		Latitude:  60.593067,
		Longitude: -151.327956,
		Timezone:  "America/Anchorage",
	},
	"75AR": {
		ICAO:      "75AR",
		Name:      "The Bluffs Airport",
		City:      "Henderson",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 782,
		Latitude:  36.2961997986,
		Longitude: -92.216003418,
		Timezone:  "America/Chicago",
	},
	"75CL": {
		ICAO:      "75CL",
		Name:      "Willis Ranch Airport",
		City:      "Dos Palos",
		State:     "California",
		Country:   "US",
		Elevation: 115,
		Latitude:  37.0654983521,
		Longitude: -120.5670013428,
		Timezone:  "America/Los_Angeles",
	},
	"75DC": {
		ICAO:      "75DC",
		Name:      "Oakesdale Airport",
		City:      "Oakesdale",
		State:     "Washington",
		Country:   "US",
		Elevation: 2680,
		Latitude:  47.1416015625,
		Longitude: -117.236000061,
		Timezone:  "America/Los_Angeles",
	},
	"75FL": {
		ICAO:      "75FL",
		Name:      "Sandy Creek Airpark",
		City:      "Panama City",
		State:     "Florida",
		Country:   "US",
		Elevation: 13,
		Latitude:  30.102399826,
		Longitude: -85.4624023438,
		Timezone:  "America/Chicago",
	},
	"75II": {
		ICAO:      "75II",
		Name:      "Lowells Landing Airport",
		City:      "Hardinsburg",
		State:     "Indiana",
		Country:   "US",
		Elevation: 700,
		Latitude:  38.4467010498,
		Longitude: -86.2608032227,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"75IS": {
		ICAO:      "75IS",
		Name:      "Reinke Airport",
		City:      "Rochelle",
		State:     "Illinois",
		Country:   "US",
		Elevation: 790,
		Latitude:  41.8992004395,
		Longitude: -89.1704025269,
		Timezone:  "America/Chicago",
	},
	"75KS": {
		ICAO:      "75KS",
		Name:      "Esplund Farm Airport",
		City:      "Minneola",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2555,
		Latitude:  37.3866996765,
		Longitude: -99.8653030396,
		Timezone:  "America/Chicago",
	},
	"75KY": {
		ICAO:      "75KY",
		Name:      "Hisle Field",
		City:      "Winchester",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 770,
		Latitude:  37.9441986084,
		Longitude: -83.9978027344,
		Timezone:  "America/New_York",
	},
	"75LL": {
		ICAO:      "75LL",
		Name:      "Hausmann Airport",
		City:      "Wyoming",
		State:     "Illinois",
		Country:   "US",
		Elevation: 748,
		Latitude:  41.0835990906,
		Longitude: -89.7582015991,
		Timezone:  "America/Chicago",
	},
	"75MI": {
		ICAO:      "75MI",
		Name:      "Norton Field",
		City:      "Riley",
		State:     "Michigan",
		Country:   "US",
		Elevation: 795,
		Latitude:  42.9714012146,
		Longitude: -82.8478012085,
		Timezone:  "America/Detroit",
	},
	"75MN": {
		ICAO:      "75MN",
		Name:      "Galler's Airport",
		City:      "Elysian",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1070,
		Latitude:  44.1865997314,
		Longitude: -93.7141036987,
		Timezone:  "America/Chicago",
	},
	"75NC": {
		ICAO:      "75NC",
		Name:      "Jw Stone Airport",
		City:      "Middlesex",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 246,
		Latitude:  35.8493003845,
		Longitude: -78.1836013794,
		Timezone:  "America/New_York",
	},
	"75ND": {
		ICAO:      "75ND",
		Name:      "Jurgens Airstrip",
		City:      "Taylor",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 2370,
		Latitude:  46.9075012207,
		Longitude: -102.4909973145,
		Timezone:  "America/Denver",
	},
	"75NE": {
		ICAO:      "75NE",
		Name:      "Heaton Airport",
		City:      "Fort Calhoun",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1000,
		Latitude:  41.4458007813,
		Longitude: -95.956703186,
		Timezone:  "America/Chicago",
	},
	"75NY": {
		ICAO:      "75NY",
		Name:      "Reiss Game Farm Airport",
		City:      "Olean",
		State:     "New York",
		Country:   "US",
		Elevation: 1698,
		Latitude:  42.1166992188,
		Longitude: -78.3155975342,
		Timezone:  "America/New_York",
	},
	"75OH": {
		ICAO:      "75OH",
		Name:      "Howensting Airport",
		City:      "Hicksville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 740,
		Latitude:  41.2592010498,
		Longitude: -84.7936019897,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"75OI": {
		ICAO:      "75OI",
		Name:      "Soltis Field",
		City:      "Marion",
		State:     "Ohio",
		Country:   "US",
		Elevation: 950,
		Latitude:  40.5037002563,
		Longitude: -83.0902023315,
		Timezone:  "America/New_York",
	},
	"75OK": {
		ICAO:      "75OK",
		Name:      "Longhorn Field",
		City:      "Enid",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1105,
		Latitude:  36.317199707,
		Longitude: -97.8161010742,
		Timezone:  "America/Chicago",
	},
	"75PA": {
		ICAO:      "75PA",
		Name:      "Marther Field",
		City:      "Albion",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 950,
		Latitude:  41.8731002808,
		Longitude: -80.4589996338,
		Timezone:  "America/New_York",
	},
	"75TA": {
		ICAO:      "75TA",
		Name:      "Coleman Cattle Company Nr 1 Airport",
		City:      "Ackerly",
		State:     "Texas",
		Country:   "US",
		Elevation: 2820,
		Latitude:  32.5343017578,
		Longitude: -101.7249984741,
		Timezone:  "America/Chicago",
	},
	"75TE": {
		ICAO:      "75TE",
		Name:      "Womack Farm Airport",
		City:      "Waco",
		State:     "Texas",
		Country:   "US",
		Elevation: 500,
		Latitude:  31.7474002838,
		Longitude: -97.2461013794,
		Timezone:  "America/Chicago",
	},
	"75TN": {
		ICAO:      "75TN",
		Name:      "Srigley Field Airstrip",
		City:      "Greeneville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1145,
		Latitude:  36.281389,
		Longitude: -82.861389,
		Timezone:  "America/New_York",
	},
	"75TS": {
		ICAO:      "75TS",
		Name:      "Venus Airport",
		City:      "Venus",
		State:     "Texas",
		Country:   "US",
		Elevation: 665,
		Latitude:  32.4164009094,
		Longitude: -97.0919036865,
		Timezone:  "America/Chicago",
	},
	"75TX": {
		ICAO:      "75TX",
		Name:      "Leona Ranch Airport",
		City:      "Brackettville",
		State:     "Texas",
		Country:   "US",
		Elevation: 1720,
		Latitude:  29.5002002716,
		Longitude: -100.3669967651,
		Timezone:  "America/Chicago",
	},
	"75WA": {
		ICAO:      "75WA",
		Name:      "Port Elsner Airport",
		City:      "White Salmon",
		State:     "Washington",
		Country:   "US",
		Elevation: 610,
		Latitude:  45.8021011353,
		Longitude: -121.4800033569,
		Timezone:  "America/Los_Angeles",
	},
	"75WI": {
		ICAO:      "75WI",
		Name:      "Smies Airport",
		City:      "Cedar Grove",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 640,
		Latitude:  43.5889015198,
		Longitude: -87.7843017578,
		Timezone:  "America/Chicago",
	},
	"75WT": {
		ICAO:      "75WT",
		Name:      "Dickson Field",
		City:      "Oroville",
		State:     "Washington",
		Country:   "US",
		Elevation: 3214,
		Latitude:  48.984167,
		Longitude: -119.298333,
		Timezone:  "America/Los_Angeles",
	},
	"75WV": {
		ICAO:      "75WV",
		Name:      "Nicholson Airport",
		City:      "West Union",
		State:     "West Virginia",
		Country:   "US",
		Elevation: 1008,
		Latitude:  39.1659,
		Longitude: -80.650806,
		Timezone:  "America/New_York",
	},
	"75XS": {
		ICAO:      "75XS",
		Name:      "Triple R Airport",
		City:      "San Antonio",
		State:     "Texas",
		Country:   "US",
		Elevation: 510,
		Latitude:  29.2493991852,
		Longitude: -98.4760971069,
		Timezone:  "America/Chicago",
	},
	"76AK": {
		ICAO:      "76AK",
		Name:      "Pogo Mine Airstrip",
		City:      "Delta Junction",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1350,
		Latitude:  64.4674987793,
		Longitude: -144.921005249,
		Timezone:  "America/Anchorage",
	},
	"76CL": {
		ICAO:      "76CL",
		Name:      "Emmett Field",
		City:      "Dos Palos",
		State:     "California",
		Country:   "US",
		Elevation: 135,
		Latitude:  37.0887985229,
		Longitude: -120.5009994507,
		Timezone:  "America/Los_Angeles",
	},
	"76CO": {
		ICAO:      "76CO",
		Name:      "Hoy Airstrip",
		City:      "Bennett",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5310,
		Latitude:  39.8358001709,
		Longitude: -104.4219970703,
		Timezone:  "America/Denver",
	},
	"76FD": {
		ICAO:      "76FD",
		Name:      "Peavy Farms Airport",
		City:      "Havana",
		State:     "Florida",
		Country:   "US",
		Elevation: 270,
		Latitude:  30.6704998016,
		Longitude: -84.414100647,
		Timezone:  "America/New_York",
	},
	"76GA": {
		ICAO:      "76GA",
		Name:      "Landings East Sylvania Airport",
		City:      "Sylvania",
		State:     "Georgia",
		Country:   "US",
		Elevation: 155,
		Latitude:  32.7447013855,
		Longitude: -81.6130981445,
		Timezone:  "America/New_York",
	},
	"76II": {
		ICAO:      "76II",
		Name:      "Ries Airport",
		City:      "Laotto",
		State:     "Indiana",
		Country:   "US",
		Elevation: 880,
		Latitude:  41.2764015198,
		Longitude: -85.2711029053,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"76IN": {
		ICAO:      "76IN",
		Name:      "Krebs Airport",
		City:      "Boswell",
		State:     "Indiana",
		Country:   "US",
		Elevation: 780,
		Latitude:  40.5391998291,
		Longitude: -87.4694976807,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"76KS": {
		ICAO:      "76KS",
		Name:      "Prima Airport",
		City:      "Scott City",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2915,
		Latitude:  38.2583999634,
		Longitude: -100.8799972534,
		Timezone:  "America/Chicago",
	},
	"76KY": {
		ICAO:      "76KY",
		Name:      "Buzzard's Roost Airport",
		City:      "Lawrenceburg",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 770,
		Latitude:  38.0148010254,
		Longitude: -84.8415985107,
		Timezone:  "America/New_York",
	},
	"76MI": {
		ICAO:      "76MI",
		Name:      "Kozal Airfield",
		City:      "Sparta",
		State:     "Michigan",
		Country:   "US",
		Elevation: 775,
		Latitude:  43.1333007812,
		Longitude: -85.7249984741,
		Timezone:  "America/Detroit",
	},
	"76MN": {
		ICAO:      "76MN",
		Name:      "Hemmingsen Private Airport",
		City:      "Erskine",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1246,
		Latitude:  47.5964012146,
		Longitude: -96.021697998,
		Timezone:  "America/Chicago",
	},
	"76MO": {
		ICAO:      "76MO",
		Name:      "Walnut Creek Airport",
		City:      "Elmer",
		State:     "Missouri",
		Country:   "US",
		Elevation: 735,
		Latitude:  39.9693984985,
		Longitude: -92.6968994141,
		Timezone:  "America/Chicago",
	},
	"76NC": {
		ICAO:      "76NC",
		Name:      "Dunroamin Farms Airport",
		City:      "Enfield",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 100,
		Latitude:  36.1599006653,
		Longitude: -77.6247024536,
		Timezone:  "America/New_York",
	},
	"76OI": {
		ICAO:      "76OI",
		Name:      "Hull Airport",
		City:      "Bettsville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 687,
		Latitude:  41.2573013306,
		Longitude: -83.2060012817,
		Timezone:  "America/New_York",
	},
	"76OK": {
		ICAO:      "76OK",
		Name:      "Odom's Roost Airport",
		City:      "Newcastle",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1240,
		Latitude:  35.2848014832,
		Longitude: -97.6169967651,
		Timezone:  "America/Chicago",
	},
	"76TA": {
		ICAO:      "76TA",
		Name:      "Coleman Cattle Company Nr 2 Airport",
		City:      "Adrian",
		State:     "Texas",
		Country:   "US",
		Elevation: 3820,
		Latitude:  35.4016990662,
		Longitude: -102.8079986572,
		Timezone:  "America/Chicago",
	},
	"76TE": {
		ICAO:      "76TE",
		Name:      "Big Tank Ranch Airport",
		City:      "Salt Flat",
		State:     "Texas",
		Country:   "US",
		Elevation: 4800,
		Latitude:  31.5000991821,
		Longitude: -105.0839996338,
		Timezone:  "America/Denver",
	},
	"76TX": {
		ICAO:      "76TX",
		Name:      "Spanish Oaks Airport",
		City:      "Little Elm",
		State:     "Texas",
		Country:   "US",
		Elevation: 580,
		Latitude:  33.1473007202,
		Longitude: -96.9757995605,
		Timezone:  "America/Chicago",
	},
	"76WA": {
		ICAO:      "76WA",
		Name:      "Heineck Farm Airport",
		City:      "Everett",
		State:     "Washington",
		Country:   "US",
		Elevation: 10,
		Latitude:  48.0051002502,
		Longitude: -122.1340026855,
		Timezone:  "America/Los_Angeles",
	},
	"77AK": {
		ICAO:      "77AK",
		Name:      "Tidewater Bluffs Airport",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 80,
		Latitude:  61.5027008057,
		Longitude: -149.595993042,
		Timezone:  "America/Anchorage",
	},
	"77CA": {
		ICAO:      "77CA",
		Name:      "Daulton Airport",
		City:      "Madera",
		State:     "California",
		Country:   "US",
		Elevation: 425,
		Latitude:  37.1208000183,
		Longitude: -119.9879989624,
		Timezone:  "America/Los_Angeles",
	},
	"77CL": {
		ICAO:      "77CL",
		Name:      "Baker & Hall Airport",
		City:      "Dunlap",
		State:     "California",
		Country:   "US",
		Elevation: 2150,
		Latitude:  36.7177009583,
		Longitude: -119.1370010376,
		Timezone:  "America/Los_Angeles",
	},
	"77GA": {
		ICAO:      "77GA",
		Name:      "Toland Airport",
		City:      "Milner",
		State:     "Georgia",
		Country:   "US",
		Elevation: 800,
		Latitude:  33.1640014648,
		Longitude: -84.2135009766,
		Timezone:  "America/New_York",
	},
	"77KS": {
		ICAO:      "77KS",
		Name:      "Abel Ranch Airport",
		City:      "Wilmont",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1320,
		Latitude:  37.3675003052,
		Longitude: -96.87840271,
		Timezone:  "America/Chicago",
	},
	"77KY": {
		ICAO:      "77KY",
		Name:      "One Oak Airport",
		City:      "Mount Sterling/Owingsville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 1000,
		Latitude:  38.1111984253,
		Longitude: -83.8734970093,
		Timezone:  "America/New_York",
	},
	"77LL": {
		ICAO:      "77LL",
		Name:      "Briggs Brothers Airfield",
		City:      "Galena",
		State:     "Illinois",
		Country:   "US",
		Elevation: 770,
		Latitude:  42.4235992432,
		Longitude: -90.5043029785,
		Timezone:  "America/Chicago",
	},
	"77MI": {
		ICAO:      "77MI",
		Name:      "David's Airport",
		City:      "Athens",
		State:     "Michigan",
		Country:   "US",
		Elevation: 887,
		Latitude:  42.0652999878,
		Longitude: -85.2416000366,
		Timezone:  "America/Detroit",
	},
	"77MN": {
		ICAO:      "77MN",
		Name:      "Klamar Field",
		City:      "Gatzke",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1160,
		Latitude:  48.3713989258,
		Longitude: -95.7881011963,
		Timezone:  "America/Chicago",
	},
	"77MO": {
		ICAO:      "77MO",
		Name:      "Springhill Airport",
		City:      "Mount Vernon",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1305,
		Latitude:  37.0903015137,
		Longitude: -93.7694015503,
		Timezone:  "America/Chicago",
	},
	"77NC": {
		ICAO:      "77NC",
		Name:      "Winnabow Airport",
		City:      "Winnabow",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 33,
		Latitude:  34.1520996094,
		Longitude: -78.1054992676,
		Timezone:  "America/New_York",
	},
	"77NY": {
		ICAO:      "77NY",
		Name:      "Pendleton Airpark",
		City:      "Pendleton",
		State:     "New York",
		Country:   "US",
		Elevation: 582,
		Latitude:  43.0820007324,
		Longitude: -78.779800415,
		Timezone:  "America/New_York",
	},
	"77OH": {
		ICAO:      "77OH",
		Name:      "Collins-Flege Airpark",
		City:      "Mason",
		State:     "Ohio",
		Country:   "US",
		Elevation: 860,
		Latitude:  39.3628005981,
		Longitude: -84.3413009644,
		Timezone:  "America/New_York",
	},
	"77PA": {
		ICAO:      "77PA",
		Name:      "Nardo Airport",
		City:      "Allison Park",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1140,
		Latitude:  40.5834007263,
		Longitude: -79.8998031616,
		Timezone:  "America/New_York",
	},
	"77PN": {
		ICAO:      "77PN",
		Name:      "Gilfert Airport",
		City:      "Benfer",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 630,
		Latitude:  40.7840003967,
		Longitude: -77.2065963745,
		Timezone:  "America/New_York",
	},
	"77TA": {
		ICAO:      "77TA",
		Name:      "Blue Skies Airport",
		City:      "Azle",
		State:     "Texas",
		Country:   "US",
		Elevation: 960,
		Latitude:  32.8740005493,
		Longitude: -97.6613998413,
		Timezone:  "America/Chicago",
	},
	"77TN": {
		ICAO:      "77TN",
		Name:      "Darnell's Field",
		City:      "Church Hill",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1150,
		Latitude:  36.5092010498,
		Longitude: -82.6817016602,
		Timezone:  "America/New_York",
	},
	"77TS": {
		ICAO:      "77TS",
		Name:      "Flying R Ranch Airport",
		City:      "New Braunfels",
		State:     "Texas",
		Country:   "US",
		Elevation: 850,
		Latitude:  29.7994003296,
		Longitude: -98.0849990845,
		Timezone:  "America/Chicago",
	},
	"77TX": {
		ICAO:      "77TX",
		Name:      "Woods Airport",
		City:      "Brookshire",
		State:     "Texas",
		Country:   "US",
		Elevation: 160,
		Latitude:  29.7816009521,
		Longitude: -95.9147033691,
		Timezone:  "America/Chicago",
	},
	"77VA": {
		ICAO:      "77VA",
		Name:      "Camp Friendship Airfield",
		City:      "Palmyra",
		State:     "Virginia",
		Country:   "US",
		Elevation: 345,
		Latitude:  37.8881988525,
		Longitude: -78.2857971191,
		Timezone:  "America/New_York",
	},
	"77WI": {
		ICAO:      "77WI",
		Name:      "Foxewood Airport",
		City:      "New Munster",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 760,
		Latitude:  42.5717010498,
		Longitude: -88.2031021118,
		Timezone:  "America/Chicago",
	},
	"77XS": {
		ICAO:      "77XS",
		Name:      "Garrett Ranch Airport",
		City:      "Danbury",
		State:     "Texas",
		Country:   "US",
		Elevation: 30,
		Latitude:  29.2872009277,
		Longitude: -95.359703064,
		Timezone:  "America/Chicago",
	},
	"78AK": {
		ICAO:      "78AK",
		Name:      "C.T.S. Airport",
		City:      "Willow",
		State:     "Alaska",
		Country:   "US",
		Elevation: 160,
		Latitude:  61.7331008911,
		Longitude: -150.5260009766,
		Timezone:  "America/Anchorage",
	},
	"78AR": {
		ICAO:      "78AR",
		Name:      "Tucker Field",
		City:      "Hughes",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 205,
		Latitude:  34.9375991821,
		Longitude: -90.4709014893,
		Timezone:  "America/Chicago",
	},
	"78FD": {
		ICAO:      "78FD",
		Name:      "Mac's Field",
		City:      "Homestead",
		State:     "Florida",
		Country:   "US",
		Elevation: 16,
		Latitude:  25.5464992523,
		Longitude: -80.4655990601,
		Timezone:  "America/New_York",
	},
	"78FL": {
		ICAO:      "78FL",
		Name:      "Pomona Landing Airport",
		City:      "Pomona Park",
		State:     "Florida",
		Country:   "US",
		Elevation: 50,
		Latitude:  29.502199173,
		Longitude: -81.5783996582,
		Timezone:  "America/New_York",
	},
	"78GA": {
		ICAO:      "78GA",
		Name:      "C&R Farm Airport",
		City:      "Carrollton",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1013,
		Latitude:  33.5042991638,
		Longitude: -85.0171966553,
		Timezone:  "America/New_York",
	},
	"78II": {
		ICAO:      "78II",
		Name:      "Fox Station Airport",
		City:      "La Fontaine",
		State:     "Indiana",
		Country:   "US",
		Elevation: 835,
		Latitude:  40.6361999512,
		Longitude: -85.68800354,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"78KS": {
		ICAO:      "78KS",
		Name:      "Walter A Swalley Airpark",
		City:      "Baxter Springs",
		State:     "Kansas",
		Country:   "US",
		Elevation: 840,
		Latitude:  37.0167007446,
		Longitude: -94.800201416,
		Timezone:  "America/Chicago",
	},
	"78LA": {
		ICAO:      "78LA",
		Name:      "Heinsohn's Airfield",
		City:      "Shreveport",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 230,
		Latitude:  32.2667999268,
		Longitude: -93.7251968384,
		Timezone:  "America/Chicago",
	},
	"78ME": {
		ICAO:      "78ME",
		Name:      "Eric's Field Airport",
		City:      "Falmouth",
		State:     "Maine",
		Country:   "US",
		Elevation: 97,
		Latitude:  43.753056,
		Longitude: -70.259167,
		Timezone:  "America/New_York",
	},
	"78MI": {
		ICAO:      "78MI",
		Name:      "Carls Airport",
		City:      "South Rockwood",
		State:     "Michigan",
		Country:   "US",
		Elevation: 598,
		Latitude:  42.040599823,
		Longitude: -83.3034973145,
		Timezone:  "America/Detroit",
	},
	"78MN": {
		ICAO:      "78MN",
		Name:      "Hagens Private Airport",
		City:      "Gatzke",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1155,
		Latitude:  48.3788986206,
		Longitude: -95.8116989136,
		Timezone:  "America/Chicago",
	},
	"78MO": {
		ICAO:      "78MO",
		Name:      "Staggs Airport",
		City:      "Ewing",
		State:     "Missouri",
		Country:   "US",
		Elevation: 660,
		Latitude:  39.995300293,
		Longitude: -91.6890029907,
		Timezone:  "America/Chicago",
	},
	"78MU": {
		ICAO:      "78MU",
		Name:      "Stony Branch Airport",
		City:      "Jasper",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1025,
		Latitude:  37.2963981628,
		Longitude: -94.1951980591,
		Timezone:  "America/Chicago",
	},
	"78NC": {
		ICAO:      "78NC",
		Name:      "Fuquay/Angier Field",
		City:      "Fuquay/Varina",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 349,
		Latitude:  35.5443000793,
		Longitude: -78.7474975586,
		Timezone:  "America/New_York",
	},
	"78ND": {
		ICAO:      "78ND",
		Name:      "Folske Ranch Airport",
		City:      "Bowman",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 2900,
		Latitude:  46.2949981689,
		Longitude: -103.4319992065,
		Timezone:  "America/Denver",
	},
	"78NE": {
		ICAO:      "78NE",
		Name:      "Stava Airport",
		City:      "Brainard",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1655,
		Latitude:  41.1660995483,
		Longitude: -97.0314025879,
		Timezone:  "America/Chicago",
	},
	"78NY": {
		ICAO:      "78NY",
		Name:      "Flying F Airport",
		City:      "Pendleton",
		State:     "New York",
		Country:   "US",
		Elevation: 580,
		Latitude:  43.0555992126,
		Longitude: -78.8161010742,
		Timezone:  "America/New_York",
	},
	"78OH": {
		ICAO:      "78OH",
		Name:      "Valley City Flying Club Airport",
		City:      "Valley City",
		State:     "Ohio",
		Country:   "US",
		Elevation: 850,
		Latitude:  41.2569999695,
		Longitude: -81.970703125,
		Timezone:  "America/New_York",
	},
	"78OR": {
		ICAO:      "78OR",
		Name:      "Crowley Ranch Airstrip",
		City:      "Crowley",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4128,
		Latitude:  43.3078994751,
		Longitude: -117.8939971924,
		Timezone:  "America/Boise",
	},
	"78TA": {
		ICAO:      "78TA",
		Name:      "Ranch-Aero Airport",
		City:      "Bigfoot",
		State:     "Texas",
		Country:   "US",
		Elevation: 560,
		Latitude:  29.0613994598,
		Longitude: -98.8003005981,
		Timezone:  "America/Chicago",
	},
	"78TE": {
		ICAO:      "78TE",
		Name:      "Rocking A Airport",
		City:      "Whitney",
		State:     "Texas",
		Country:   "US",
		Elevation: 635,
		Latitude:  32.020401001,
		Longitude: -97.3311004639,
		Timezone:  "America/Chicago",
	},
	"78VA": {
		ICAO:      "78VA",
		Name:      "Hilldale Airport",
		City:      "Stanardsville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 490,
		Latitude:  38.2463989258,
		Longitude: -78.4753036499,
		Timezone:  "America/New_York",
	},
	"78WA": {
		ICAO:      "78WA",
		Name:      "Center Island Airport",
		City:      "Center Island",
		State:     "Washington",
		Country:   "US",
		Elevation: 115,
		Latitude:  48.4901008606,
		Longitude: -122.8320007324,
		Timezone:  "America/Los_Angeles",
	},
	"78XS": {
		ICAO:      "78XS",
		Name:      "Smith Aviation Inc Airport",
		City:      "Danevang",
		State:     "Texas",
		Country:   "US",
		Elevation: 75,
		Latitude:  29.0529994965,
		Longitude: -96.2621994019,
		Timezone:  "America/Chicago",
	},
	"79AK": {
		ICAO:      "79AK",
		Name:      "Huttunen Strip",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 120,
		Latitude:  61.481839,
		Longitude: -149.744019,
		Timezone:  "America/Anchorage",
	},
	"79AR": {
		ICAO:      "79AR",
		Name:      "Perrys Airport",
		City:      "Jacksonville",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 263,
		Latitude:  34.8529014587,
		Longitude: -92.0482025146,
		Timezone:  "America/Chicago",
	},
	"79CL": {
		ICAO:      "79CL",
		Name:      "Milhous Ranch Airport",
		City:      "North San Juan",
		State:     "California",
		Country:   "US",
		Elevation: 2625,
		Latitude:  39.3249015808,
		Longitude: -121.0759963989,
		Timezone:  "America/Los_Angeles",
	},
	"79FD": {
		ICAO:      "79FD",
		Name:      "Midway Lake Airport",
		City:      "Plant City",
		State:     "Florida",
		Country:   "US",
		Elevation: 150,
		Latitude:  28.0419998169,
		Longitude: -82.092300415,
		Timezone:  "America/New_York",
	},
	"79FL": {
		ICAO:      "79FL",
		Name:      "Neal Field",
		City:      "Chiefland",
		State:     "Florida",
		Country:   "US",
		Elevation: 45,
		Latitude:  29.5527000427,
		Longitude: -82.8686981201,
		Timezone:  "America/New_York",
	},
	"79GA": {
		ICAO:      "79GA",
		Name:      "Coates Airport",
		City:      "Fort Gaines",
		State:     "Georgia",
		Country:   "US",
		Elevation: 310,
		Latitude:  31.4913005829,
		Longitude: -85.0166015625,
		Timezone:  "America/New_York",
	},
	"79ID": {
		ICAO:      "79ID",
		Name:      "Kooskia (Clear Creek Int) Airport",
		State:     "Idaho",
		Country:   "US",
		Elevation: 1800,
		Latitude:  46.0488642914,
		Longitude: -115.869691372,
		Timezone:  "America/Los_Angeles",
	},
	"79II": {
		ICAO:      "79II",
		Name:      "Sauer-Harter Airport",
		City:      "Greenfield",
		State:     "Indiana",
		Country:   "US",
		Elevation: 930,
		Latitude:  39.8833999634,
		Longitude: -85.7024993896,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"79IL": {
		ICAO:      "79IL",
		Name:      "Miller Airport",
		City:      "Ashton",
		State:     "Illinois",
		Country:   "US",
		Elevation: 820,
		Latitude:  41.7966995239,
		Longitude: -89.2303009033,
		Timezone:  "America/Chicago",
	},
	"79IS": {
		ICAO:      "79IS",
		Name:      "Hartwell Ranch RLA Restricted Landing Area",
		City:      "Hillview",
		State:     "Illinois",
		Country:   "US",
		Elevation: 427,
		Latitude:  39.4448013306,
		Longitude: -90.5621032715,
		Timezone:  "America/Chicago",
	},
	"79KY": {
		ICAO:      "79KY",
		Name:      "Lucas Field",
		City:      "Nicholasville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 906,
		Latitude:  37.8712997437,
		Longitude: -84.610496521,
		Timezone:  "America/New_York",
	},
	"79MI": {
		ICAO:      "79MI",
		Name:      "Pike River Landing",
		City:      "Chassell",
		State:     "Michigan",
		Country:   "US",
		Elevation: 902,
		Latitude:  46.980278,
		Longitude: -88.54,
		Timezone:  "America/Detroit",
	},
	"79MN": {
		ICAO:      "79MN",
		Name:      "Paul Field",
		City:      "Gatzke",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1165,
		Latitude:  48.3777008057,
		Longitude: -95.7544021606,
		Timezone:  "America/Chicago",
	},
	"79MO": {
		ICAO:      "79MO",
		Name:      "Faries Field",
		City:      "Williamsville",
		State:     "Missouri",
		Country:   "US",
		Elevation: 450,
		Latitude:  36.9866981506,
		Longitude: -90.543296814,
		Timezone:  "America/Chicago",
	},
	"79MU": {
		ICAO:      "79MU",
		Name:      "Dunham Private Airport",
		City:      "Smithville",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1000,
		Latitude:  39.3302993774,
		Longitude: -94.556602478,
		Timezone:  "America/Chicago",
	},
	"79NC": {
		ICAO:      "79NC",
		Name:      "Ball Airport",
		City:      "Louisburg",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 366,
		Latitude:  36.1343002319,
		Longitude: -78.2818984985,
		Timezone:  "America/New_York",
	},
	"79ND": {
		ICAO:      "79ND",
		Name:      "Reimers Airport",
		City:      "Carrington",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1590,
		Latitude:  47.323299408,
		Longitude: -99.0011978149,
		Timezone:  "America/Chicago",
	},
	"79NY": {
		ICAO:      "79NY",
		Name:      "Smith Airport",
		City:      "Pendleton",
		State:     "New York",
		Country:   "US",
		Elevation: 580,
		Latitude:  43.0870018005,
		Longitude: -78.8114013672,
		Timezone:  "America/New_York",
	},
	"79OH": {
		ICAO:      "79OH",
		Name:      "Smith-Stewart Field",
		City:      "Vienna",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1092,
		Latitude:  41.2098007202,
		Longitude: -80.6830978394,
		Timezone:  "America/New_York",
	},
	"79OI": {
		ICAO:      "79OI",
		Name:      "Flying J Airport",
		City:      "Pitchin",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1070,
		Latitude:  39.8172988892,
		Longitude: -83.7557983398,
		Timezone:  "America/New_York",
	},
	"79OK": {
		ICAO:      "79OK",
		Name:      "Little River Ranch Airport",
		City:      "Pickens",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 580,
		Latitude:  35.3847999573,
		Longitude: -95.176399231,
		Timezone:  "America/Chicago",
	},
	"79PA": {
		ICAO:      "79PA",
		Name:      "Little Britain Airport",
		City:      "Quarryville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 470,
		Latitude:  39.7911987305,
		Longitude: -76.1416015625,
		Timezone:  "America/New_York",
	},
	"79TA": {
		ICAO:      "79TA",
		Name:      "Morris Ranch Airport",
		City:      "Cotulla",
		State:     "Texas",
		Country:   "US",
		Elevation: 346,
		Latitude:  28.1697006226,
		Longitude: -98.8899993896,
		Timezone:  "America/Chicago",
	},
	"79TE": {
		ICAO:      "79TE",
		Name:      "Arrowhead Airport",
		City:      "Lagarto",
		State:     "Texas",
		Country:   "US",
		Elevation: 165,
		Latitude:  28.1156005859,
		Longitude: -97.9308013916,
		Timezone:  "America/Chicago",
	},
	"79TS": {
		ICAO:      "79TS",
		Name:      "Tallows Field",
		City:      "Celina",
		State:     "Texas",
		Country:   "US",
		Elevation: 729,
		Latitude:  33.3042984009,
		Longitude: -96.7472000122,
		Timezone:  "America/Chicago",
	},
	"79TX": {
		ICAO:      "79TX",
		Name:      "Ag-Air Inc. Airport",
		City:      "Brownsville",
		State:     "Texas",
		Country:   "US",
		Elevation: 30,
		Latitude:  26.0730991364,
		Longitude: -97.5374984741,
		Timezone:  "America/Chicago",
	},
	"79WA": {
		ICAO:      "79WA",
		Name:      "Grigg Farm Airport",
		City:      "Quincy",
		State:     "Washington",
		Country:   "US",
		Elevation: 1220,
		Latitude:  47.173500061,
		Longitude: -119.7470016479,
		Timezone:  "America/Los_Angeles",
	},
	"79WT": {
		ICAO:      "79WT",
		Name:      "Ellensburg (Rotor Ranch) Airport",
		State:     "Washington",
		Country:   "US",
		Elevation: 1962,
		Latitude:  47.0914260595,
		Longitude: -120.589778423,
		Timezone:  "America/Los_Angeles",
	},
	"79XS": {
		ICAO:      "79XS",
		Name:      "Silverton Municipal Airport",
		City:      "Silverton",
		State:     "Texas",
		Country:   "US",
		Elevation: 3267,
		Latitude:  34.4667015076,
		Longitude: -101.3000030518,
		Timezone:  "America/Chicago",
	},
	"7AK0": {
		ICAO:      "7AK0",
		Name:      "Art Z Airport",
		City:      "Tanana",
		State:     "Alaska",
		Country:   "US",
		Elevation: 193,
		Latitude:  64.6473999023,
		Longitude: -151.8099975586,
		Timezone:  "America/Anchorage",
	},
	"7AK2": {
		ICAO:      "7AK2",
		Name:      "Snettisham Airport",
		City:      "Snettisham",
		State:     "Alaska",
		Country:   "US",
		Elevation: 14,
		Latitude:  58.1343994141,
		Longitude: -133.7299957275,
		Timezone:  "America/Juneau",
	},
	"7AK3": {
		ICAO:      "7AK3",
		Name:      "Gaede Airport",
		City:      "Soldotna",
		State:     "Alaska",
		Country:   "US",
		Elevation: 115,
		Latitude:  60.476398468,
		Longitude: -151.1660003662,
		Timezone:  "America/Anchorage",
	},
	"7AK4": {
		ICAO:      "7AK4",
		Name:      "Jack Fish Landing Airport",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 150,
		Latitude:  61.5415000916,
		Longitude: -149.4609985352,
		Timezone:  "America/Anchorage",
	},
	"7AK5": {
		ICAO:      "7AK5",
		Name:      "Dahler Homestead Airport",
		City:      "Soldotna",
		State:     "Alaska",
		Country:   "US",
		Elevation: 300,
		Latitude:  60.5036010742,
		Longitude: -150.9360046387,
		Timezone:  "America/Anchorage",
	},
	"7AK6": {
		ICAO:      "7AK6",
		Name:      "Dan France Airport",
		City:      "Soldotna",
		State:     "Alaska",
		Country:   "US",
		Elevation: 260,
		Latitude:  60.4777984619,
		Longitude: -150.9290008545,
		Timezone:  "America/Anchorage",
	},
	"7AK8": {
		ICAO:      "7AK8",
		Name:      "Hess Airport",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 355,
		Latitude:  61.6212997437,
		Longitude: -149.6150054932,
		Timezone:  "America/Anchorage",
	},
	"7AK9": {
		ICAO:      "7AK9",
		Name:      "Vinduska Airport",
		City:      "Palmer",
		State:     "Alaska",
		Country:   "US",
		Elevation: 700,
		Latitude:  61.6468009949,
		Longitude: -149.0489959717,
		Timezone:  "America/Anchorage",
	},
	"7AL9": {
		ICAO:      "7AL9",
		Name:      "Horak Airport",
		City:      "Elberta",
		State:     "Alabama",
		Country:   "US",
		Elevation: 110,
		Latitude:  30.4668998718,
		Longitude: -87.5416030884,
		Timezone:  "America/Chicago",
	},
	"7AR0": {
		ICAO:      "7AR0",
		Name:      "Hammer Field Airport",
		City:      "Eureka Springs",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1600,
		Latitude:  36.4314002991,
		Longitude: -93.6992034912,
		Timezone:  "America/Chicago",
	},
	"7AR1": {
		ICAO:      "7AR1",
		Name:      "Ward's Airport",
		City:      "Eudora",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 113,
		Latitude:  33.0499992371,
		Longitude: -91.3415985107,
		Timezone:  "America/Chicago",
	},
	"7AR2": {
		ICAO:      "7AR2",
		Name:      "Verser's Landing Airport",
		City:      "Greers Ferry",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 610,
		Latitude:  35.5572013855,
		Longitude: -92.1321029663,
		Timezone:  "America/Chicago",
	},
	"7AR3": {
		ICAO:      "7AR3",
		Name:      "Runsick Flying Service Airport",
		City:      "Colt",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 250,
		Latitude:  35.1511001587,
		Longitude: -90.8155975342,
		Timezone:  "America/Chicago",
	},
	"7AR4": {
		ICAO:      "7AR4",
		Name:      "Brown'S Airport",
		City:      "Searcy",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 367,
		Latitude:  35.292222,
		Longitude: -91.89,
		Timezone:  "America/Chicago",
	},
	"7CA1": {
		ICAO:      "7CA1",
		Name:      "Abraham Ranch Airport",
		City:      "Lucerne Valley",
		State:     "California",
		Country:   "US",
		Elevation: 2850,
		Latitude:  34.4146995544,
		Longitude: -116.6230010986,
		Timezone:  "America/Los_Angeles",
	},
	"7CA2": {
		ICAO:      "7CA2",
		Name:      "Paradise Lakes Airport",
		City:      "Bakersfield",
		State:     "California",
		Country:   "US",
		Elevation: 340,
		Latitude:  35.176700592,
		Longitude: -118.9270019531,
		Timezone:  "America/Los_Angeles",
	},
	"7CA6": {
		ICAO:      "7CA6",
		Name:      "West Side Field Station Airport",
		City:      "Five Points",
		State:     "California",
		Country:   "US",
		Elevation: 280,
		Latitude:  36.3387985229,
		Longitude: -120.1100006104,
		Timezone:  "America/Los_Angeles",
	},
	"7CL0": {
		ICAO:      "7CL0",
		Name:      "Du Bois Ranch Airport",
		City:      "Kerman",
		State:     "California",
		Country:   "US",
		Elevation: 195,
		Latitude:  36.7036018372,
		Longitude: -120.1350021362,
		Timezone:  "America/Los_Angeles",
	},
	"7CL8": {
		ICAO:      "7CL8",
		Name:      "Geyser Ranch Airport",
		City:      "Ely/Pioche",
		State:     "Nevada",
		Country:   "US",
		Elevation: 5977,
		Latitude:  38.664100647,
		Longitude: -114.6320037842,
		Timezone:  "America/Los_Angeles",
	},
	"7CL9": {
		ICAO:      "7CL9",
		Name:      "Perryman Airport",
		City:      "Placerville",
		State:     "California",
		Country:   "US",
		Elevation: 2600,
		Latitude:  38.6963005066,
		Longitude: -120.7310028076,
		Timezone:  "America/Los_Angeles",
	},
	"7CO0": {
		ICAO:      "7CO0",
		Name:      "Parkland Airport",
		City:      "Erie",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5050,
		Latitude:  40.0717010498,
		Longitude: -105.033996582,
		Timezone:  "America/Denver",
	},
	"7CO1": {
		ICAO:      "7CO1",
		Name:      "Dave Nash Ranch Airport",
		City:      "Guffey",
		State:     "Colorado",
		Country:   "US",
		Elevation: 8400,
		Latitude:  38.7597007751,
		Longitude: -105.3840026855,
		Timezone:  "America/Denver",
	},
	"7CO2": {
		ICAO:      "7CO2",
		Name:      "Hawkins Ranch Airport",
		City:      "Hotchkiss",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6200,
		Latitude:  38.8415985107,
		Longitude: -107.8759994507,
		Timezone:  "America/Denver",
	},
	"7CO3": {
		ICAO:      "7CO3",
		Name:      "Lindys Airpark",
		City:      "Hudson",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5040,
		Latitude:  40.038898468,
		Longitude: -104.6240005493,
		Timezone:  "America/Denver",
	},
	"7CO4": {
		ICAO:      "7CO4",
		Name:      "Williams Ranch Airport",
		City:      "New Raymer",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4860,
		Latitude:  40.6041984558,
		Longitude: -103.9390029907,
		Timezone:  "America/Denver",
	},
	"7CO5": {
		ICAO:      "7CO5",
		Name:      "Yocam Ranch Airport",
		City:      "Orchard",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4461,
		Latitude:  40.2999992371,
		Longitude: -104.2300033569,
		Timezone:  "America/Denver",
	},
	"7CO6": {
		ICAO:      "7CO6",
		Name:      "Vantage View Airport",
		City:      "Rush",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5780,
		Latitude:  38.760278,
		Longitude: -104.101389,
		Timezone:  "America/Denver",
	},
	"7CO8": {
		ICAO:      "7CO8",
		Name:      "West Pueblo Airport",
		City:      "Pueblo West",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5135,
		Latitude:  38.3633003235,
		Longitude: -104.7379989624,
		Timezone:  "America/Denver",
	},
	"7FA1": {
		ICAO:      "7FA1",
		Name:      "Sugar Loaf Shores Airport",
		City:      "Key West",
		State:     "Florida",
		Country:   "US",
		Elevation: 4,
		Latitude:  24.6487998962,
		Longitude: -81.5798034668,
		Timezone:  "America/New_York",
	},
	"7FA8": {
		ICAO:      "7FA8",
		Name:      "Berry Grove Airport",
		City:      "La Belle",
		State:     "Florida",
		Country:   "US",
		Elevation: 28,
		Latitude:  26.6956005096,
		Longitude: -81.4906005859,
		Timezone:  "America/New_York",
	},
	"7FD2": {
		ICAO:      "7FD2",
		Name:      "Drake Ranch Airport",
		City:      "Hernando",
		State:     "Florida",
		Country:   "US",
		Elevation: 50,
		Latitude:  28.9902992249,
		Longitude: -82.3303985596,
		Timezone:  "America/New_York",
	},
	"7FD6": {
		ICAO:      "7FD6",
		Name:      "Loxahatchee Airport",
		City:      "Lake Worth",
		State:     "Florida",
		Country:   "US",
		Elevation: 14,
		Latitude:  26.6226005554,
		Longitude: -80.3052978516,
		Timezone:  "America/New_York",
	},
	"7FD9": {
		ICAO:      "7FD9",
		Name:      "Estherbrook Aerodrome",
		City:      "Madison",
		State:     "Florida",
		Country:   "US",
		Elevation: 100,
		Latitude:  30.6243991852,
		Longitude: -83.2867965698,
		Timezone:  "America/New_York",
	},
	"7FL2": {
		ICAO:      "7FL2",
		Name:      "Wellborn STOLport",
		City:      "Wellborn",
		State:     "Florida",
		Country:   "US",
		Elevation: 180,
		Latitude:  30.2455005646,
		Longitude: -82.778503418,
		Timezone:  "America/New_York",
	},
	"7FL3": {
		ICAO:      "7FL3",
		Name:      "Adams Ranch Airport",
		City:      "Fort Pierce",
		State:     "Florida",
		Country:   "US",
		Elevation: 25,
		Latitude:  27.4419994354,
		Longitude: -80.584197998,
		Timezone:  "America/New_York",
	},
	"7FL4": {
		ICAO:      "7FL4",
		Name:      "Haller Airpark",
		City:      "Green Cove Springs",
		State:     "Florida",
		Country:   "US",
		Elevation: 75,
		Latitude:  29.9029998779,
		Longitude: -81.6858978271,
		Timezone:  "America/New_York",
	},
	"7FL6": {
		ICAO:      "7FL6",
		Name:      "Spruce Creek Airport",
		City:      "Daytona Beach",
		State:     "Florida",
		Country:   "US",
		Elevation: 24,
		Latitude:  29.0802001953,
		Longitude: -81.0466995239,
		Timezone:  "America/New_York",
	},
	"7FL7": {
		ICAO:      "7FL7",
		Name:      "Lazy S Farm Airport",
		City:      "High Springs",
		State:     "Florida",
		Country:   "US",
		Elevation: 80,
		Latitude:  29.8686008453,
		Longitude: -82.6345977783,
		Timezone:  "America/New_York",
	},
	"7FL8": {
		ICAO:      "7FL8",
		Name:      "Dragonfly Airport",
		City:      "Fort Pierce",
		State:     "Florida",
		Country:   "US",
		Elevation: 25,
		Latitude:  27.4438991547,
		Longitude: -80.5651016235,
		Timezone:  "America/New_York",
	},
	"7GA2": {
		ICAO:      "7GA2",
		Name:      "Mcclellan Airport",
		City:      "Valdosta",
		State:     "Georgia",
		Country:   "US",
		Elevation: 229,
		Latitude:  30.7789001465,
		Longitude: -83.2018966675,
		Timezone:  "America/New_York",
	},
	"7GA3": {
		ICAO:      "7GA3",
		Name:      "Hacienda De Gay Airstrip",
		City:      "Garfield",
		State:     "Georgia",
		Country:   "US",
		Elevation: 328,
		Latitude:  32.6632995605,
		Longitude: -82.075302124,
		Timezone:  "America/New_York",
	},
	"7GA4": {
		ICAO:      "7GA4",
		Name:      "Danville Airpark",
		City:      "Danville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 459,
		Latitude:  32.5810012817,
		Longitude: -83.2677001953,
		Timezone:  "America/New_York",
	},
	"7GA5": {
		ICAO:      "7GA5",
		Name:      "Mount Cove STOLport",
		City:      "Kensington",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1000,
		Latitude:  34.6323013306,
		Longitude: -85.4471969604,
		Timezone:  "America/New_York",
	},
	"7GA6": {
		ICAO:      "7GA6",
		Name:      "Holder's Field",
		City:      "Locust Grove",
		State:     "Georgia",
		Country:   "US",
		Elevation: 740,
		Latitude:  33.3614997864,
		Longitude: -84.1445999146,
		Timezone:  "America/New_York",
	},
	"7GA9": {
		ICAO:      "7GA9",
		Name:      "Smisson Field",
		City:      "Whitesburg",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1044,
		Latitude:  33.5705986023,
		Longitude: -84.8977966309,
		Timezone:  "America/New_York",
	},
	"7GE1": {
		ICAO:      "7GE1",
		Name:      "Davison Ranch Airport",
		City:      "Caldwell",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2370,
		Latitude:  43.6581993103,
		Longitude: -116.7509994507,
		Timezone:  "America/Boise",
	},
	"7GE5": {
		ICAO:      "7GE5",
		Name:      "Sunset Strip",
		City:      "Centralhatchee",
		State:     "Georgia",
		Country:   "US",
		Elevation: 740,
		Latitude:  33.3591995239,
		Longitude: -85.1175003052,
		Timezone:  "America/New_York",
	},
	"7IA1": {
		ICAO:      "7IA1",
		Name:      "White Pigeon Airport",
		City:      "North English",
		State:     "Iowa",
		Country:   "US",
		Elevation: 875,
		Latitude:  41.4934997559,
		Longitude: -92.1482009888,
		Timezone:  "America/Chicago",
	},
	"7IA3": {
		ICAO:      "7IA3",
		Name:      "Mc Bride Field",
		City:      "Marion",
		State:     "Iowa",
		Country:   "US",
		Elevation: 881,
		Latitude:  42.0638999939,
		Longitude: -91.6343002319,
		Timezone:  "America/Chicago",
	},
	"7IA7": {
		ICAO:      "7IA7",
		Name:      "Mitchell Field",
		City:      "New Hampton",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1160,
		Latitude:  43.0835990906,
		Longitude: -92.3099975586,
		Timezone:  "America/Chicago",
	},
	"7II0": {
		ICAO:      "7II0",
		Name:      "Porter Field",
		City:      "New Whiteland",
		State:     "Indiana",
		Country:   "US",
		Elevation: 785,
		Latitude:  39.5620002747,
		Longitude: -86.1125030518,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"7II3": {
		ICAO:      "7II3",
		Name:      "Schroeder Private Airport",
		City:      "Mount Vernon",
		State:     "Indiana",
		Country:   "US",
		Elevation: 385,
		Latitude:  38.0033988953,
		Longitude: -87.756401062,
		Timezone:  "America/Chicago",
	},
	"7II7": {
		ICAO:      "7II7",
		Name:      "Rex's Ultralightport",
		City:      "Wolflake",
		State:     "Indiana",
		Country:   "US",
		Elevation: 920,
		Latitude:  41.2972984314,
		Longitude: -85.4832992554,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"7II9": {
		ICAO:      "7II9",
		Name:      "Thorn Field",
		City:      "Bargersville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 740,
		Latitude:  39.5828018188,
		Longitude: -86.1797027588,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"7IL0": {
		ICAO:      "7IL0",
		Name:      "Mussman Airport",
		City:      "Grant Park",
		State:     "Illinois",
		Country:   "US",
		Elevation: 705,
		Latitude:  41.2709007263,
		Longitude: -87.577796936,
		Timezone:  "America/Chicago",
	},
	"7IL2": {
		ICAO:      "7IL2",
		Name:      "O John Clark Restricted Landing Area",
		City:      "Grays Lake",
		State:     "Illinois",
		Country:   "US",
		Elevation: 790,
		Latitude:  42.3684005737,
		Longitude: -87.9916992188,
		Timezone:  "America/Chicago",
	},
	"7IL3": {
		ICAO:      "7IL3",
		Name:      "Lutz RLA Restricted Landing Area",
		City:      "Greenvalley",
		State:     "Illinois",
		Country:   "US",
		Elevation: 545,
		Latitude:  40.4283981323,
		Longitude: -89.6020965576,
		Timezone:  "America/Chicago",
	},
	"7IL6": {
		ICAO:      "7IL6",
		Name:      "Dr Joseph W Esser Airport",
		City:      "Hampshire",
		State:     "Illinois",
		Country:   "US",
		Elevation: 979,
		Latitude:  42.1305999756,
		Longitude: -88.5028991699,
		Timezone:  "America/Chicago",
	},
	"7IL7": {
		ICAO:      "7IL7",
		Name:      "Edward Getzelman Airport",
		City:      "Hampshire",
		State:     "Illinois",
		Country:   "US",
		Elevation: 870,
		Latitude:  42.126701355,
		Longitude: -88.5334014893,
		Timezone:  "America/Chicago",
	},
	"7IL8": {
		ICAO:      "7IL8",
		Name:      "Cody Port RLA Restricted Landing Area",
		City:      "Harding",
		State:     "Illinois",
		Country:   "US",
		Elevation: 668,
		Latitude:  41.518901825,
		Longitude: -88.837600708,
		Timezone:  "America/Chicago",
	},
	"7IL9": {
		ICAO:      "7IL9",
		Name:      "Ambler - Cady Airport",
		City:      "Earlville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 700,
		Latitude:  41.5438995361,
		Longitude: -88.9934005737,
		Timezone:  "America/Chicago",
	},
	"7IN2": {
		ICAO:      "7IN2",
		Name:      "Flying Crown Airport",
		City:      "Hamilton",
		State:     "Indiana",
		Country:   "US",
		Elevation: 920,
		Latitude:  41.5013999939,
		Longitude: -84.8783035278,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"7IN3": {
		ICAO:      "7IN3",
		Name:      "Garrett Field",
		City:      "Poseyville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 400,
		Latitude:  38.243598938,
		Longitude: -87.7900009155,
		Timezone:  "America/Chicago",
	},
	"7IN6": {
		ICAO:      "7IN6",
		Name:      "Myers Farm Airport",
		City:      "Livonia",
		State:     "Indiana",
		Country:   "US",
		Elevation: 800,
		Latitude:  38.5819015503,
		Longitude: -86.2536010742,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"7IN7": {
		ICAO:      "7IN7",
		Name:      "Gary Johnson Field",
		City:      "West Lebanon",
		State:     "Indiana",
		Country:   "US",
		Elevation: 570,
		Latitude:  40.2358016968,
		Longitude: -87.3507995605,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"7IN9": {
		ICAO:      "7IN9",
		Name:      "The Last Resort Airport",
		City:      "Springport",
		State:     "Indiana",
		Country:   "US",
		Elevation: 1070,
		Latitude:  40.0374984741,
		Longitude: -85.4105987549,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"7IS2": {
		ICAO:      "7IS2",
		Name:      "Aero Estates Airport",
		City:      "Belleville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 565,
		Latitude:  38.5245018005,
		Longitude: -90.0569000244,
		Timezone:  "America/Chicago",
	},
	"7IS3": {
		ICAO:      "7IS3",
		Name:      "Hooterville Airport",
		City:      "Clinton",
		State:     "Illinois",
		Country:   "US",
		Elevation: 730,
		Latitude:  40.1433982849,
		Longitude: -89.091796875,
		Timezone:  "America/Chicago",
	},
	"7IS5": {
		ICAO:      "7IS5",
		Name:      "Koppie Airport",
		City:      "Gilberts",
		State:     "Illinois",
		Country:   "US",
		Elevation: 890,
		Latitude:  42.1213989258,
		Longitude: -88.3780975342,
		Timezone:  "America/Chicago",
	},
	"7IS7": {
		ICAO:      "7IS7",
		Name:      "Hammock Field",
		City:      "Alhambra",
		State:     "Illinois",
		Country:   "US",
		Elevation: 595,
		Latitude:  38.913898468,
		Longitude: -89.6700973511,
		Timezone:  "America/Chicago",
	},
	"7IS9": {
		ICAO:      "7IS9",
		Name:      "King Airport",
		City:      "Columbia",
		State:     "Illinois",
		Country:   "US",
		Elevation: 407,
		Latitude:  38.476398468,
		Longitude: -90.2361984253,
		Timezone:  "America/Chicago",
	},
	"7KS0": {
		ICAO:      "7KS0",
		Name:      "Flying T Airport",
		City:      "Oskaloosa",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1160,
		Latitude:  39.2714004517,
		Longitude: -95.3169021606,
		Timezone:  "America/Chicago",
	},
	"7KS2": {
		ICAO:      "7KS2",
		Name:      "Cloud Airport",
		City:      "Overbrook",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1020,
		Latitude:  38.84999847,
		Longitude: -95.46749878,
		Timezone:  "America/Chicago",
	},
	"7KS3": {
		ICAO:      "7KS3",
		Name:      "Youvan Airport",
		City:      "Arma",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1000,
		Latitude:  37.5292015076,
		Longitude: -94.6960983276,
		Timezone:  "America/Chicago",
	},
	"7KS4": {
		ICAO:      "7KS4",
		Name:      "Bent Nail Ranch Airport",
		City:      "Horton",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1180,
		Latitude:  39.6333007812,
		Longitude: -95.439201355,
		Timezone:  "America/Chicago",
	},
	"7KS5": {
		ICAO:      "7KS5",
		Name:      "Croisant Airport",
		City:      "Humboldt",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1000,
		Latitude:  37.7961997986,
		Longitude: -95.3419036865,
		Timezone:  "America/Chicago",
	},
	"7KS6": {
		ICAO:      "7KS6",
		Name:      "Linders Cow-Chip Airport",
		City:      "Louisburg",
		State:     "Kansas",
		Country:   "US",
		Elevation: 985,
		Latitude:  38.6263999939,
		Longitude: -94.7694015503,
		Timezone:  "America/Chicago",
	},
	"7KS7": {
		ICAO:      "7KS7",
		Name:      "Evans Airport",
		City:      "Oakley",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2950,
		Latitude:  38.9583015442,
		Longitude: -100.8000030518,
		Timezone:  "America/Chicago",
	},
	"7KS8": {
		ICAO:      "7KS8",
		Name:      "Shaw Aerial Spraying Airport",
		City:      "Oberlin",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2700,
		Latitude:  39.8306007385,
		Longitude: -100.581001282,
		Timezone:  "America/Chicago",
	},
	"7KS9": {
		ICAO:      "7KS9",
		Name:      "Eagle Field",
		City:      "Bentley",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1379,
		Latitude:  37.8488998413,
		Longitude: -97.5100021362,
		Timezone:  "America/Chicago",
	},
	"7KY2": {
		ICAO:      "7KY2",
		Name:      "Ryan Field",
		City:      "Verona",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 890,
		Latitude:  38.8036003113,
		Longitude: -84.6611022949,
		Timezone:  "America/New_York",
	},
	"7KY3": {
		ICAO:      "7KY3",
		Name:      "Little Mount International Airport",
		City:      "Tarylorsville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 750,
		Latitude:  38.0760993958,
		Longitude: -85.2367019653,
		Timezone:  "America/New_York",
	},
	"7LA0": {
		ICAO:      "7LA0",
		Name:      "Valverda Strip",
		City:      "Maringouin",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 50,
		Latitude:  30.525100708,
		Longitude: -91.5319976807,
		Timezone:  "America/Chicago",
	},
	"7LA1": {
		ICAO:      "7LA1",
		Name:      "Birdwin Airport",
		City:      "Port Sulphur",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 2,
		Latitude:  29.4582996368,
		Longitude: -89.6759033203,
		Timezone:  "America/Chicago",
	},
	"7LA3": {
		ICAO:      "7LA3",
		Name:      "Habetz Airport",
		City:      "Ragley",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 79,
		Latitude:  30.4829998016,
		Longitude: -93.2287979126,
		Timezone:  "America/Chicago",
	},
	"7LA4": {
		ICAO:      "7LA4",
		Name:      "Gladney Airport",
		City:      "Rayne",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 23,
		Latitude:  30.1252002716,
		Longitude: -92.2195968628,
		Timezone:  "America/Chicago",
	},
	"7LL3": {
		ICAO:      "7LL3",
		Name:      "Jasper County Flying Club Airport",
		City:      "Newton",
		State:     "Illinois",
		Country:   "US",
		Elevation: 575,
		Latitude:  38.9317016602,
		Longitude: -88.1569976807,
		Timezone:  "America/Chicago",
	},
	"7LL4": {
		ICAO:      "7LL4",
		Name:      "Bakers Strip",
		City:      "Weldon",
		State:     "Illinois",
		Country:   "US",
		Elevation: 715,
		Latitude:  40.1172981262,
		Longitude: -88.7590026855,
		Timezone:  "America/Chicago",
	},
	"7LL6": {
		ICAO:      "7LL6",
		Name:      "Gehant Airport",
		City:      "West Brooklyn",
		State:     "Illinois",
		Country:   "US",
		Elevation: 952,
		Latitude:  41.6974983215,
		Longitude: -89.1443023682,
		Timezone:  "America/Chicago",
	},
	"7LL7": {
		ICAO:      "7LL7",
		Name:      "Delhotal Airport",
		City:      "West Brooklyn",
		State:     "Illinois",
		Country:   "US",
		Elevation: 760,
		Latitude:  41.7360992432,
		Longitude: -89.1723022461,
		Timezone:  "America/Chicago",
	},
	"7LL8": {
		ICAO:      "7LL8",
		Name:      "Dahler Airport",
		City:      "Nokomis",
		State:     "Illinois",
		Country:   "US",
		Elevation: 659,
		Latitude:  39.3525009155,
		Longitude: -89.3009033203,
		Timezone:  "America/Chicago",
	},
	"7LL9": {
		ICAO:      "7LL9",
		Name:      "Brammeier Airport",
		City:      "Oakdale",
		State:     "Illinois",
		Country:   "US",
		Elevation: 530,
		Latitude:  38.2616996765,
		Longitude: -89.4709014893,
		Timezone:  "America/Chicago",
	},
	"7MD0": {
		ICAO:      "7MD0",
		Name:      "Our Domain Airport",
		City:      "Marydel",
		State:     "Delaware",
		Country:   "US",
		Elevation: 65,
		Latitude:  39.0666999817,
		Longitude: -75.7332992554,
		Timezone:  "America/New_York",
	},
	"7MD1": {
		ICAO:      "7MD1",
		Name:      "Magennis Farm Airport",
		City:      "Federalsburg",
		State:     "Maryland",
		Country:   "US",
		Elevation: 40,
		Latitude:  38.7033996582,
		Longitude: -75.8509979248,
		Timezone:  "America/New_York",
	},
	"7MD7": {
		ICAO:      "7MD7",
		Name:      "Boomers Field",
		City:      "Pocomoke",
		State:     "Maryland",
		Country:   "US",
		Elevation: 22,
		Latitude:  38.0004005432,
		Longitude: -75.5830001831,
		Timezone:  "America/New_York",
	},
	"7MD8": {
		ICAO:      "7MD8",
		Name:      "Kent Fort Manor Airport",
		City:      "Stevensville",
		State:     "Maryland",
		Country:   "US",
		Elevation: 10,
		Latitude:  38.8564987183,
		Longitude: -76.3616027832,
		Timezone:  "America/New_York",
	},
	"7MD9": {
		ICAO:      "7MD9",
		Name:      "Tilghman Whipp Airport",
		City:      "Tilghman",
		State:     "Maryland",
		Country:   "US",
		Elevation: 5,
		Latitude:  38.7360992432,
		Longitude: -76.3167037964,
		Timezone:  "America/New_York",
	},
	"7MI2": {
		ICAO:      "7MI2",
		Name:      "Sawyer / Southwest Lakes Airpark",
		State:     "Michigan",
		Country:   "US",
		Elevation: 650,
		Latitude:  41.8910966258,
		Longitude: -86.5867388248,
		Timezone:  "America/Detroit",
	},
	"7MI3": {
		ICAO:      "7MI3",
		Name:      "Thrall Lake Airport",
		City:      "Vicksburg",
		State:     "Michigan",
		Country:   "US",
		Elevation: 850,
		Latitude:  42.1166992188,
		Longitude: -85.5167007446,
		Timezone:  "America/Detroit",
	},
	"7MI4": {
		ICAO:      "7MI4",
		Name:      "Ludington Airport",
		City:      "Port Hope",
		State:     "Michigan",
		Country:   "US",
		Elevation: 690,
		Latitude:  43.9221992493,
		Longitude: -82.7485961914,
		Timezone:  "America/Detroit",
	},
	"7MI5": {
		ICAO:      "7MI5",
		Name:      "Lada Airport",
		City:      "Petersburg",
		State:     "Michigan",
		Country:   "US",
		Elevation: 678,
		Latitude:  41.8875007629,
		Longitude: -83.6765975952,
		Timezone:  "America/Detroit",
	},
	"7MI6": {
		ICAO:      "7MI6",
		Name:      "Wightman Airport",
		City:      "Oakley",
		State:     "Michigan",
		Country:   "US",
		Elevation: 725,
		Latitude:  43.1102981567,
		Longitude: -84.1264038086,
		Timezone:  "America/Detroit",
	},
	"7MI7": {
		ICAO:      "7MI7",
		Name:      "Walker Airport",
		City:      "Cooper",
		State:     "Michigan",
		Country:   "US",
		Elevation: 750,
		Latitude:  42.3805999756,
		Longitude: -85.5774993896,
		Timezone:  "America/Detroit",
	},
	"7MI8": {
		ICAO:      "7MI8",
		Name:      "Letts Field",
		City:      "Vassar",
		State:     "Michigan",
		Country:   "US",
		Elevation: 800,
		Latitude:  43.3391990662,
		Longitude: -83.4760971069,
		Timezone:  "America/Detroit",
	},
	"7MN0": {
		ICAO:      "7MN0",
		Name:      "Johnson Private Airport",
		City:      "Sacred Heart",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1020,
		Latitude:  44.7999992371,
		Longitude: -95.3953018188,
		Timezone:  "America/Chicago",
	},
	"7MN3": {
		ICAO:      "7MN3",
		Name:      "Caldbeck Field",
		City:      "Spring Valley",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1340,
		Latitude:  43.7052001953,
		Longitude: -92.4607009888,
		Timezone:  "America/Chicago",
	},
	"7MN4": {
		ICAO:      "7MN4",
		Name:      "Mille Lacs Lake Resort Airport",
		City:      "Garrison",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1261,
		Latitude:  46.3194007874,
		Longitude: -93.8308029175,
		Timezone:  "America/Chicago",
	},
	"7MN6": {
		ICAO:      "7MN6",
		Name:      "Johnson Airport",
		City:      "Argyle",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 820,
		Latitude:  48.3428001404,
		Longitude: -96.880897522,
		Timezone:  "America/Chicago",
	},
	"7MN8": {
		ICAO:      "7MN8",
		Name:      "Willis Airport",
		City:      "Thief River Falls",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1135,
		Latitude:  48.0699996948,
		Longitude: -96.2789001465,
		Timezone:  "America/Chicago",
	},
	"7MO1": {
		ICAO:      "7MO1",
		Name:      "Wood Acres Airport",
		City:      "Perry",
		State:     "Missouri",
		Country:   "US",
		Elevation: 720,
		Latitude:  39.4723014832,
		Longitude: -91.7113037109,
		Timezone:  "America/Chicago",
	},
	"7MO2": {
		ICAO:      "7MO2",
		Name:      "Bevill Airport",
		City:      "Philadelphia",
		State:     "Missouri",
		Country:   "US",
		Elevation: 710,
		Latitude:  39.8499984741,
		Longitude: -91.7334976196,
		Timezone:  "America/Chicago",
	},
	"7MO3": {
		ICAO:      "7MO3",
		Name:      "B S Ranch Airport",
		City:      "Pineville",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1000,
		Latitude:  36.6001014709,
		Longitude: -94.3501968384,
		Timezone:  "America/Chicago",
	},
	"7MO4": {
		ICAO:      "7MO4",
		Name:      "Flintlock Field",
		City:      "Platte City",
		State:     "Missouri",
		Country:   "US",
		Elevation: 930,
		Latitude:  39.4319000244,
		Longitude: -94.8030014038,
		Timezone:  "America/Chicago",
	},
	"7MO5": {
		ICAO:      "7MO5",
		Name:      "Elton Field",
		City:      "Platte City",
		State:     "Missouri",
		Country:   "US",
		Elevation: 780,
		Latitude:  39.3347015381,
		Longitude: -94.8126983643,
		Timezone:  "America/Chicago",
	},
	"7MO6": {
		ICAO:      "7MO6",
		Name:      "Mac's Field",
		City:      "Plattsburg",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1015,
		Latitude:  39.4832992554,
		Longitude: -94.4669036865,
		Timezone:  "America/Chicago",
	},
	"7MO7": {
		ICAO:      "7MO7",
		Name:      "Kimray Airport",
		City:      "Plattsburg",
		State:     "Missouri",
		Country:   "US",
		Elevation: 940,
		Latitude:  39.5666999817,
		Longitude: -94.4147033691,
		Timezone:  "America/Chicago",
	},
	"7MO8": {
		ICAO:      "7MO8",
		Name:      "Clark Airport",
		City:      "Plattsburg",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1010,
		Latitude:  39.5292015076,
		Longitude: -94.4502029419,
		Timezone:  "America/Chicago",
	},
	"7MS1": {
		ICAO:      "7MS1",
		Name:      "Eagle Crest Estates Airport",
		City:      "Canton",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 280,
		Latitude:  32.6068992615,
		Longitude: -89.9832992554,
		Timezone:  "America/Chicago",
	},
	"7MY9": {
		ICAO:      "7MY9",
		Name:      "Petes Airport",
		City:      "Dexter",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1337,
		Latitude:  43.746717,
		Longitude: -92.7245,
		Timezone:  "America/Chicago",
	},
	"7NA0": {
		ICAO:      "7NA0",
		Name:      "Downs Farm Private Airport",
		City:      "Hillsboro",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 900,
		Latitude:  47.3574981689,
		Longitude: -97.0280990601,
		Timezone:  "America/Chicago",
	},
	"7NA2": {
		ICAO:      "7NA2",
		Name:      "Undlin Airstrip",
		City:      "Lansford",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1575,
		Latitude:  48.6878013611,
		Longitude: -101.3499984741,
		Timezone:  "America/Chicago",
	},
	"7NA3": {
		ICAO:      "7NA3",
		Name:      "Hiam Private Airport",
		City:      "Lisbon",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1251,
		Latitude:  46.3499984741,
		Longitude: -97.6942977905,
		Timezone:  "America/Chicago",
	},
	"7NA4": {
		ICAO:      "7NA4",
		Name:      "Ingebretson Airspray Airport",
		City:      "Mayville",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 982,
		Latitude:  47.51720047,
		Longitude: -97.3337020874,
		Timezone:  "America/Chicago",
	},
	"7NA9": {
		ICAO:      "7NA9",
		Name:      "Sjule Private Airstrip",
		City:      "Souris",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1680,
		Latitude:  48.9282989502,
		Longitude: -100.5780029297,
		Timezone:  "America/Chicago",
	},
	"7NC0": {
		ICAO:      "7NC0",
		Name:      "Pamlico Airport",
		City:      "Bayboro",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 20,
		Latitude:  35.1585006714,
		Longitude: -76.7627029419,
		Timezone:  "America/New_York",
	},
	"7NC1": {
		ICAO:      "7NC1",
		Name:      "Stag Air Park",
		City:      "Burgaw",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 23,
		Latitude:  34.5279006958,
		Longitude: -77.8503036499,
		Timezone:  "America/New_York",
	},
	"7NC2": {
		ICAO:      "7NC2",
		IATA:      "DUF",
		Name:      "Pine Island Airport",
		City:      "Corolla",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 16,
		Latitude:  36.2535018921,
		Longitude: -75.7884979248,
		Timezone:  "America/New_York",
	},
	"7NC3": {
		ICAO:      "7NC3",
		Name:      "Kenly Airport",
		City:      "Kenly",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 225,
		Latitude:  35.596389,
		Longitude: -78.098611,
		Timezone:  "America/New_York",
	},
	"7NC5": {
		ICAO:      "7NC5",
		Name:      "Crooked Creek Airport",
		City:      "Bunn",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 250,
		Latitude:  35.9337997437,
		Longitude: -78.2466964722,
		Timezone:  "America/New_York",
	},
	"7NC6": {
		ICAO:      "7NC6",
		Name:      "Kimrey Airport",
		City:      "Mebane",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 624,
		Latitude:  36.0554008484,
		Longitude: -79.3266983032,
		Timezone:  "America/New_York",
	},
	"7NC7": {
		ICAO:      "7NC7",
		Name:      "Lewis Airstrip",
		City:      "Walnut Cove",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 650,
		Latitude:  36.3468017578,
		Longitude: -80.1736984253,
		Timezone:  "America/New_York",
	},
	"7NC8": {
		ICAO:      "7NC8",
		Name:      "Fish Airpark",
		City:      "Angier",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 280,
		Latitude:  35.5349006653,
		Longitude: -78.6964035034,
		Timezone:  "America/New_York",
	},
	"7NC9": {
		ICAO:      "7NC9",
		Name:      "Spencer Field",
		City:      "Asheboro",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 460,
		Latitude:  35.7274017334,
		Longitude: -79.9111022949,
		Timezone:  "America/New_York",
	},
	"7ND1": {
		ICAO:      "7ND1",
		Name:      "Wolberg's Private Airport",
		City:      "Dickinson",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 2540,
		Latitude:  47.0360984802,
		Longitude: -102.7779998779,
		Timezone:  "America/Denver",
	},
	"7ND2": {
		ICAO:      "7ND2",
		Name:      "Kelly's Field",
		City:      "Grafton",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 810,
		Latitude:  48.3727989197,
		Longitude: -97.320098877,
		Timezone:  "America/Chicago",
	},
	"7ND5": {
		ICAO:      "7ND5",
		Name:      "Buchmiller Airport",
		City:      "Bowdon",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1850,
		Latitude:  47.3596992493,
		Longitude: -99.6225967407,
		Timezone:  "America/Chicago",
	},
	"7ND7": {
		ICAO:      "7ND7",
		Name:      "Haugen's Airport",
		City:      "Alexander",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 2330,
		Latitude:  47.6824989319,
		Longitude: -103.7279968262,
		Timezone:  "America/Chicago",
	},
	"7ND8": {
		ICAO:      "7ND8",
		Name:      "Forest Airport",
		City:      "Langdon",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1570,
		Latitude:  48.8521995544,
		Longitude: -98.7481994629,
		Timezone:  "America/Chicago",
	},
	"7NE2": {
		ICAO:      "7NE2",
		Name:      "Plainsview Ranch Airport",
		City:      "Hemingford",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 4650,
		Latitude:  42.2583007813,
		Longitude: -103.5299987793,
		Timezone:  "America/Denver",
	},
	"7NE5": {
		ICAO:      "7NE5",
		Name:      "Lierley Farms Airport",
		City:      "Paxton",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3220,
		Latitude:  41.0666999817,
		Longitude: -101.3679962158,
		Timezone:  "America/Denver",
	},
	"7NE6": {
		ICAO:      "7NE6",
		Name:      "Merchant Homestead Airport",
		City:      "Polk",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1700,
		Latitude:  41.011100769,
		Longitude: -97.8114013672,
		Timezone:  "America/Chicago",
	},
	"7NJ2": {
		ICAO:      "7NJ2",
		Name:      "Stoe Creek Farm Airport",
		City:      "Canton",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 15,
		Latitude:  39.481300354,
		Longitude: -75.4052963257,
		Timezone:  "America/New_York",
	},
	"7NJ7": {
		ICAO:      "7NJ7",
		Name:      "Coombs Airport",
		City:      "Elmer",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 120,
		Latitude:  39.5606002808,
		Longitude: -75.2185974121,
		Timezone:  "America/New_York",
	},
	"7NJ9": {
		ICAO:      "7NJ9",
		Name:      "Dave's Aerodrome",
		City:      "Cedarville",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 8,
		Latitude:  39.3181991577,
		Longitude: -75.2062988281,
		Timezone:  "America/New_York",
	},
	"7NK0": {
		ICAO:      "7NK0",
		Name:      "Valley View Airport",
		City:      "Middleburgh",
		State:     "New York",
		Country:   "US",
		Elevation: 1160,
		Latitude:  42.6475982666,
		Longitude: -74.3873977661,
		Timezone:  "America/New_York",
	},
	"7NK1": {
		ICAO:      "7NK1",
		Name:      "Deer Run Air Field",
		City:      "Nelson",
		State:     "New York",
		Country:   "US",
		Elevation: 1725,
		Latitude:  42.9430999756,
		Longitude: -75.7536010742,
		Timezone:  "America/New_York",
	},
	"7NK4": {
		ICAO:      "7NK4",
		Name:      "Ridge Road West Airport",
		City:      "Spenceport",
		State:     "New York",
		Country:   "US",
		Elevation: 400,
		Latitude:  43.2225990295,
		Longitude: -77.8106002808,
		Timezone:  "America/New_York",
	},
	"7NK6": {
		ICAO:      "7NK6",
		Name:      "Blueberry Field",
		City:      "Treadwell",
		State:     "New York",
		Country:   "US",
		Elevation: 2100,
		Latitude:  42.2916984558,
		Longitude: -75.0667037964,
		Timezone:  "America/New_York",
	},
	"7NK7": {
		ICAO:      "7NK7",
		Name:      "Morin Airport",
		City:      "Sherburne",
		State:     "New York",
		Country:   "US",
		Elevation: 1068,
		Latitude:  42.7132987976,
		Longitude: -75.8899993896,
		Timezone:  "America/New_York",
	},
	"7NM1": {
		ICAO:      "7NM1",
		Name:      "Mesa Verde Ranch Strip Airport",
		City:      "Alamogordo",
		State:     "New Mexico",
		Country:   "US",
		Elevation: 4237,
		Latitude:  32.935278,
		Longitude: -106.045278,
		Timezone:  "America/Denver",
	},
	"7NY0": {
		ICAO:      "7NY0",
		Name:      "Campis Airport",
		City:      "Lake Huntington",
		State:     "New York",
		Country:   "US",
		Elevation: 1200,
		Latitude:  41.6833992004,
		Longitude: -75.0162963867,
		Timezone:  "America/New_York",
	},
	"7NY1": {
		ICAO:      "7NY1",
		Name:      "Gar Field",
		City:      "Esperance",
		State:     "New York",
		Country:   "US",
		Elevation: 640,
		Latitude:  42.7634010315,
		Longitude: -74.278503418,
		Timezone:  "America/New_York",
	},
	"7NY2": {
		ICAO:      "7NY2",
		Name:      "Butterville Airport",
		City:      "Adams",
		State:     "New York",
		Country:   "US",
		Elevation: 510,
		Latitude:  43.8433990479,
		Longitude: -76.0947036743,
		Timezone:  "America/New_York",
	},
	"7NY4": {
		ICAO:      "7NY4",
		Name:      "Culver Airfield",
		City:      "Mecklenburg",
		State:     "New York",
		Country:   "US",
		Elevation: 1455,
		Latitude:  42.4250984192,
		Longitude: -76.7149963379,
		Timezone:  "America/New_York",
	},
	"7NY6": {
		ICAO:      "7NY6",
		Name:      "Sherwood Farm Airport",
		City:      "Johnsonville",
		State:     "New York",
		Country:   "US",
		Elevation: 530,
		Latitude:  42.893699646,
		Longitude: -73.5078964233,
		Timezone:  "America/New_York",
	},
	"7NY8": {
		ICAO:      "7NY8",
		Name:      "Shaw Field",
		City:      "Glen Falls",
		State:     "New York",
		Country:   "US",
		Elevation: 190,
		Latitude:  43.1922988892,
		Longitude: -73.6297988892,
		Timezone:  "America/New_York",
	},
	"7OA7": {
		ICAO:      "7OA7",
		Name:      "Skydive Greene County Inc Airport",
		City:      "Xenia",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1055,
		Latitude:  39.6791992188,
		Longitude: -83.8707962036,
		Timezone:  "America/New_York",
	},
	"7OH1": {
		ICAO:      "7OH1",
		Name:      "Rauhaus Field",
		City:      "Valley City",
		State:     "Ohio",
		Country:   "US",
		Elevation: 912,
		Latitude:  41.2099990845,
		Longitude: -81.9736022949,
		Timezone:  "America/New_York",
	},
	"7OH2": {
		ICAO:      "7OH2",
		Name:      "Canal Fulton Airport",
		City:      "Canal Fulton",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1150,
		Latitude:  40.9002990723,
		Longitude: -81.5336990356,
		Timezone:  "America/New_York",
	},
	"7OH3": {
		ICAO:      "7OH3",
		Name:      "Obi One Airport",
		City:      "Delaware",
		State:     "Ohio",
		Country:   "US",
		Elevation: 952,
		Latitude:  40.3527984619,
		Longitude: -83.0510025024,
		Timezone:  "America/New_York",
	},
	"7OH5": {
		ICAO:      "7OH5",
		Name:      "Mahoning County Joint Vocational School Airport",
		City:      "Canfield",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1130,
		Latitude:  41.034198761,
		Longitude: -80.7817001343,
		Timezone:  "America/New_York",
	},
	"7OH7": {
		ICAO:      "7OH7",
		Name:      "Hydebrook Airport",
		City:      "Yellow Springs",
		State:     "Ohio",
		Country:   "US",
		Elevation: 960,
		Latitude:  39.7801017761,
		Longitude: -83.8999023438,
		Timezone:  "America/New_York",
	},
	"7OH8": {
		ICAO:      "7OH8",
		Name:      "Curtis Airport",
		City:      "Chardon",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1250,
		Latitude:  41.6377983093,
		Longitude: -81.1014022827,
		Timezone:  "America/New_York",
	},
	"7OI0": {
		ICAO:      "7OI0",
		Name:      "Christy's Airport",
		City:      "Napoleon",
		State:     "Ohio",
		Country:   "US",
		Elevation: 683,
		Latitude:  41.3342018127,
		Longitude: -84.1635971069,
		Timezone:  "America/New_York",
	},
	"7OI1": {
		ICAO:      "7OI1",
		Name:      "Blevins Airport",
		City:      "Ayersville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 722,
		Latitude:  41.2072982788,
		Longitude: -84.2696990967,
		Timezone:  "America/New_York",
	},
	"7OI5": {
		ICAO:      "7OI5",
		Name:      "Bea-D Airport",
		City:      "Graytown",
		State:     "Ohio",
		Country:   "US",
		Elevation: 605,
		Latitude:  41.5205993652,
		Longitude: -83.2751998901,
		Timezone:  "America/New_York",
	},
	"7OI7": {
		ICAO:      "7OI7",
		Name:      "Southcreek Airport",
		City:      "Clyde",
		State:     "Ohio",
		Country:   "US",
		Elevation: 665,
		Latitude:  41.2994995117,
		Longitude: -83.0112991333,
		Timezone:  "America/New_York",
	},
	"7OI9": {
		ICAO:      "7OI9",
		Name:      "Massengill Airport",
		City:      "Nashport",
		State:     "Ohio",
		Country:   "US",
		Elevation: 955,
		Latitude:  40.0690002441,
		Longitude: -82.1360015869,
		Timezone:  "America/New_York",
	},
	"7OK0": {
		ICAO:      "7OK0",
		Name:      "Dennis Ranch Airport",
		City:      "Grady",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 875,
		Latitude:  34.0317993164,
		Longitude: -97.6778030396,
		Timezone:  "America/Chicago",
	},
	"7OK1": {
		ICAO:      "7OK1",
		Name:      "Meadowlark Field",
		City:      "Boynton",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 746,
		Latitude:  35.6203994751,
		Longitude: -95.7442016602,
		Timezone:  "America/Chicago",
	},
	"7OK2": {
		ICAO:      "7OK2",
		Name:      "Scott Airport",
		City:      "Mangum",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1650,
		Latitude:  34.976398468,
		Longitude: -99.4998016357,
		Timezone:  "America/Chicago",
	},
	"7OK4": {
		ICAO:      "7OK4",
		Name:      "Martin Farms Airport",
		City:      "Amber",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1200,
		Latitude:  35.150100708,
		Longitude: -97.8669967651,
		Timezone:  "America/Chicago",
	},
	"7OK8": {
		ICAO:      "7OK8",
		Name:      "Myers Airport",
		City:      "Crescent",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1060,
		Latitude:  35.989200592,
		Longitude: -97.573097229,
		Timezone:  "America/Chicago",
	},
	"7OR0": {
		ICAO:      "7OR0",
		Name:      "Minam Lodge Airport",
		City:      "Cove",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3589,
		Latitude:  45.3582000732,
		Longitude: -117.6340026855,
		Timezone:  "America/Los_Angeles",
	},
	"7OR1": {
		ICAO:      "7OR1",
		Name:      "Arnold Airstrip",
		City:      "Crane",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4140,
		Latitude:  43.4193000793,
		Longitude: -118.5759963989,
		Timezone:  "America/Los_Angeles",
	},
	"7OR2": {
		ICAO:      "7OR2",
		Name:      "Basl Hill Farms Airport",
		City:      "Mehama",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1160,
		Latitude:  44.8228988647,
		Longitude: -122.6800003052,
		Timezone:  "America/Los_Angeles",
	},
	"7OR3": {
		ICAO:      "7OR3",
		Name:      "Honald Ranch Airport",
		City:      "The Dalles",
		State:     "Oregon",
		Country:   "US",
		Elevation: 245,
		Latitude:  45.5904006958,
		Longitude: -121.2139968872,
		Timezone:  "America/Los_Angeles",
	},
	"7OR4": {
		ICAO:      "7OR4",
		Name:      "Pineridge Ranch Airport",
		City:      "Sisters",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3070,
		Latitude:  44.3096008301,
		Longitude: -121.4430007935,
		Timezone:  "America/Los_Angeles",
	},
	"7OR6": {
		ICAO:      "7OR6",
		Name:      "Green Acres Air Park",
		City:      "Hood River",
		State:     "Oregon",
		Country:   "US",
		Elevation: 800,
		Latitude:  45.6367988586,
		Longitude: -121.591003418,
		Timezone:  "America/Los_Angeles",
	},
	"7OR7": {
		ICAO:      "7OR7",
		Name:      "Hollin Airport",
		City:      "Brooks",
		State:     "Oregon",
		Country:   "US",
		Elevation: 170,
		Latitude:  45.0540008545,
		Longitude: -122.9069976807,
		Timezone:  "America/Los_Angeles",
	},
	"7OR8": {
		ICAO:      "7OR8",
		Name:      "Inshallah International Airport",
		City:      "Seneca",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4560,
		Latitude:  44.0904006958,
		Longitude: -119.2900009155,
		Timezone:  "America/Los_Angeles",
	},
	"7OR9": {
		ICAO:      "7OR9",
		Name:      "Parson Landing Airport",
		City:      "Oregon City",
		State:     "Oregon",
		Country:   "US",
		Elevation: 640,
		Latitude:  45.3242988586,
		Longitude: -122.472000122,
		Timezone:  "America/Los_Angeles",
	},
	"7PA0": {
		ICAO:      "7PA0",
		Name:      "Manor Knoll Personal Use Airport",
		City:      "Mountville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 440,
		Latitude:  40.0209007263,
		Longitude: -76.4238967896,
		Timezone:  "America/New_York",
	},
	"7PA1": {
		ICAO:      "7PA1",
		Name:      "Warren Airpark",
		City:      "Warren",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1790,
		Latitude:  41.8658981323,
		Longitude: -79.206199646,
		Timezone:  "America/New_York",
	},
	"7PA3": {
		ICAO:      "7PA3",
		Name:      "Charles G. Kalko Airport",
		City:      "Honesdale",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1400,
		Latitude:  41.7056007385,
		Longitude: -75.2880020142,
		Timezone:  "America/New_York",
	},
	"7PA4": {
		ICAO:      "7PA4",
		Name:      "Orson Field",
		City:      "Orson",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 2100,
		Latitude:  41.842300415,
		Longitude: -75.4363021851,
		Timezone:  "America/New_York",
	},
	"7PA5": {
		ICAO:      "7PA5",
		Name:      "Fauser Ultralightport",
		City:      "Turbotville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 565,
		Latitude:  41.1029014587,
		Longitude: -76.8054962158,
		Timezone:  "America/New_York",
	},
	"7PA6": {
		ICAO:      "7PA6",
		Name:      "Stott Private Airport",
		City:      "Newtown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 143,
		Latitude:  40.2453994751,
		Longitude: -74.9913024902,
		Timezone:  "America/New_York",
	},
	"7PN0": {
		ICAO:      "7PN0",
		Name:      "Tidmore Airport",
		City:      "Minersville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 930,
		Latitude:  40.7050018311,
		Longitude: -76.2363967896,
		Timezone:  "America/New_York",
	},
	"7PS4": {
		ICAO:      "7PS4",
		Name:      "Sky Classics Field",
		City:      "Grantville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 500,
		Latitude:  40.3978996277,
		Longitude: -76.5988998413,
		Timezone:  "America/New_York",
	},
	"7PS7": {
		ICAO:      "7PS7",
		Name:      "Lenzner Farm Airport",
		City:      "Sewickley",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1250,
		Latitude:  40.5387001038,
		Longitude: -80.1173019409,
		Timezone:  "America/New_York",
	},
	"7TA0": {
		ICAO:      "7TA0",
		Name:      "Field's Field",
		City:      "Tomball",
		State:     "Texas",
		Country:   "US",
		Elevation: 198,
		Latitude:  30.0921993256,
		Longitude: -95.6516036987,
		Timezone:  "America/Chicago",
	},
	"7TA3": {
		ICAO:      "7TA3",
		Name:      "Quarterway Airport",
		City:      "Plainview",
		State:     "Texas",
		Country:   "US",
		Elevation: 3470,
		Latitude:  34.1809005737,
		Longitude: -101.8369979858,
		Timezone:  "America/Chicago",
	},
	"7TA5": {
		ICAO:      "7TA5",
		Name:      "Weber Ranch Airport",
		City:      "Caldwell",
		State:     "Texas",
		Country:   "US",
		Elevation: 350,
		Latitude:  30.5870990753,
		Longitude: -96.6341018677,
		Timezone:  "America/Chicago",
	},
	"7TA7": {
		ICAO:      "7TA7",
		Name:      "Flying M Ranch Airport",
		City:      "Reklaw",
		State:     "Texas",
		Country:   "US",
		Elevation: 310,
		Latitude:  31.8398990631,
		Longitude: -94.9613037109,
		Timezone:  "America/Chicago",
	},
	"7TA8": {
		ICAO:      "7TA8",
		Name:      "John Henry Key Airport",
		City:      "Boerne",
		State:     "Texas",
		Country:   "US",
		Elevation: 1400,
		Latitude:  29.8596992493,
		Longitude: -98.7414016724,
		Timezone:  "America/Chicago",
	},
	"7TE0": {
		ICAO:      "7TE0",
		Name:      "Kelley Crop Service Airport",
		City:      "Fannett",
		State:     "Texas",
		Country:   "US",
		Elevation: 21,
		Latitude:  29.9137992859,
		Longitude: -94.2081985474,
		Timezone:  "America/Chicago",
	},
	"7TE1": {
		ICAO:      "7TE1",
		Name:      "Zachry Ranch Airport",
		City:      "Escobas",
		State:     "Texas",
		Country:   "US",
		Elevation: 592,
		Latitude:  27.0725002289,
		Longitude: -98.9385986328,
		Timezone:  "America/Chicago",
	},
	"7TE2": {
		ICAO:      "7TE2",
		Name:      "Cage Ranch Airport",
		City:      "Falfurrias",
		State:     "Texas",
		Country:   "US",
		Elevation: 148,
		Latitude:  27.1058998108,
		Longitude: -98.2058029175,
		Timezone:  "America/Chicago",
	},
	"7TE3": {
		ICAO:      "7TE3",
		Name:      "Lometa Air Strip",
		City:      "Lometa",
		State:     "Texas",
		Country:   "US",
		Elevation: 1400,
		Latitude:  31.233499527,
		Longitude: -98.4670028687,
		Timezone:  "America/Chicago",
	},
	"7TE4": {
		ICAO:      "7TE4",
		Name:      "Flying J Ranch Airport",
		City:      "Leakey",
		State:     "Texas",
		Country:   "US",
		Elevation: 1620,
		Latitude:  29.5969009399,
		Longitude: -99.8452987671,
		Timezone:  "America/Chicago",
	},
	"7TE5": {
		ICAO:      "7TE5",
		Name:      "Carlisle Airport",
		City:      "Mullin",
		State:     "Texas",
		Country:   "US",
		Elevation: 1790,
		Latitude:  31.6791992188,
		Longitude: -98.6635971069,
		Timezone:  "America/Chicago",
	},
	"7TE6": {
		ICAO:      "7TE6",
		Name:      "Stovall Ranch Nr 1 Airport",
		City:      "El Campo",
		State:     "Texas",
		Country:   "US",
		Elevation: 108,
		Latitude:  29.2003002167,
		Longitude: -96.554397583,
		Timezone:  "America/Chicago",
	},
	"7TE7": {
		ICAO:      "7TE7",
		Name:      "Moore Field",
		City:      "Mission",
		State:     "Texas",
		Country:   "US",
		Elevation: 219,
		Latitude:  26.3836994171,
		Longitude: -98.3336029053,
		Timezone:  "America/Chicago",
	},
	"7TE8": {
		ICAO:      "7TE8",
		Name:      "Trulock Ranch Field",
		City:      "Colorado City",
		State:     "Texas",
		Country:   "US",
		Elevation: 2168,
		Latitude:  32.4210014343,
		Longitude: -100.8109970093,
		Timezone:  "America/Chicago",
	},
	"7TE9": {
		ICAO:      "7TE9",
		Name:      "Boening Brothers Airport",
		City:      "Floresville",
		State:     "Texas",
		Country:   "US",
		Elevation: 407,
		Latitude:  29.0272006989,
		Longitude: -98.2866973877,
		Timezone:  "America/Chicago",
	},
	"7TN0": {
		ICAO:      "7TN0",
		Name:      "Blue Bird Field",
		City:      "Pleasantview",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 705,
		Latitude:  36.4356002808,
		Longitude: -87.0914001465,
		Timezone:  "America/Chicago",
	},
	"7TN1": {
		ICAO:      "7TN1",
		Name:      "Elk River Airfield",
		City:      "Pelham",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 995,
		Latitude:  35.3205986023,
		Longitude: -85.8535995483,
		Timezone:  "America/Chicago",
	},
	"7TN3": {
		ICAO:      "7TN3",
		Name:      "Stones River Airport",
		City:      "Murfreesboro",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 537,
		Latitude:  35.9707984924,
		Longitude: -86.3930969238,
		Timezone:  "America/Chicago",
	},
	"7TN4": {
		ICAO:      "7TN4",
		Name:      "Pegasus Field",
		City:      "Oakland",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 380,
		Latitude:  35.1727981567,
		Longitude: -89.5625,
		Timezone:  "America/Chicago",
	},
	"7TN5": {
		ICAO:      "7TN5",
		Name:      "Flying D Airport",
		City:      "Hickory Valley",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 477,
		Latitude:  35.133806,
		Longitude: -89.186806,
		Timezone:  "America/Chicago",
	},
	"7TS0": {
		ICAO:      "7TS0",
		Name:      "Fairview Airport",
		City:      "Rhome",
		State:     "Texas",
		Country:   "US",
		Elevation: 920,
		Latitude:  33.0951004028,
		Longitude: -97.4270019531,
		Timezone:  "America/Chicago",
	},
	"7TS2": {
		ICAO:      "7TS2",
		Name:      "Alison Air Park",
		City:      "San Marcos",
		State:     "Texas",
		Country:   "US",
		Elevation: 630,
		Latitude:  29.8313007355,
		Longitude: -97.8891983032,
		Timezone:  "America/Chicago",
	},
	"7TS3": {
		ICAO:      "7TS3",
		Name:      "Wyatt Airport",
		City:      "San Marcos",
		State:     "Texas",
		Country:   "US",
		Elevation: 800,
		Latitude:  29.9598999023,
		Longitude: -97.9852981567,
		Timezone:  "America/Chicago",
	},
	"7TS4": {
		ICAO:      "7TS4",
		Name:      "Roma Airport",
		City:      "Ponder",
		State:     "Texas",
		Country:   "US",
		Elevation: 725,
		Latitude:  33.1832008362,
		Longitude: -97.2545013428,
		Timezone:  "America/Chicago",
	},
	"7TS7": {
		ICAO:      "7TS7",
		Name:      "Broussard Farm Airport",
		City:      "Fannett",
		State:     "Texas",
		Country:   "US",
		Elevation: 20,
		Latitude:  29.9111003876,
		Longitude: -94.3391036987,
		Timezone:  "America/Chicago",
	},
	"7TS8": {
		ICAO:      "7TS8",
		Name:      "Ott Farms Airport",
		City:      "Dimmitt",
		State:     "Texas",
		Country:   "US",
		Elevation: 3817,
		Latitude:  34.4038009644,
		Longitude: -102.4540023804,
		Timezone:  "America/Chicago",
	},
	"7TS9": {
		ICAO:      "7TS9",
		Name:      "Ag Aviation Airport",
		City:      "Bay City",
		State:     "Texas",
		Country:   "US",
		Elevation: 50,
		Latitude:  29.0541000366,
		Longitude: -95.9625015259,
		Timezone:  "America/Chicago",
	},
	"7TX0": {
		ICAO:      "7TX0",
		Name:      "Tom Danaher Airport",
		City:      "Wichita Falls",
		State:     "Texas",
		Country:   "US",
		Elevation: 986,
		Latitude:  33.8282012939,
		Longitude: -98.5727005005,
		Timezone:  "America/Chicago",
	},
	"7TX4": {
		ICAO:      "7TX4",
		Name:      "Hillcrest Airport",
		City:      "Keller",
		State:     "Texas",
		Country:   "US",
		Elevation: 710,
		Latitude:  32.9668006897,
		Longitude: -97.2752990723,
		Timezone:  "America/Chicago",
	},
	"7TX5": {
		ICAO:      "7TX5",
		Name:      "Mabee Ranch Airport",
		City:      "Midland",
		State:     "Texas",
		Country:   "US",
		Elevation: 2862,
		Latitude:  32.2170982361,
		Longitude: -102.1600036621,
		Timezone:  "America/Chicago",
	},
	"7TX7": {
		ICAO:      "7TX7",
		Name:      "Ryan Aerodrome",
		City:      "Midland",
		State:     "Texas",
		Country:   "US",
		Elevation: 2740,
		Latitude:  32.0834999084,
		Longitude: -102.016998291,
		Timezone:  "America/Chicago",
	},
	"7TX8": {
		ICAO:      "7TX8",
		Name:      "Flying K Airport",
		City:      "Amarillo",
		State:     "Texas",
		Country:   "US",
		Elevation: 3560,
		Latitude:  35.1076011658,
		Longitude: -102.0380020142,
		Timezone:  "America/Chicago",
	},
	"7TX9": {
		ICAO:      "7TX9",
		Name:      "Ashford Field",
		City:      "Old Boston",
		State:     "Texas",
		Country:   "US",
		Elevation: 305,
		Latitude:  33.3586997986,
		Longitude: -94.4376983643,
		Timezone:  "America/Chicago",
	},
	"7VA5": {
		ICAO:      "7VA5",
		Name:      "Redhouse Airfield",
		City:      "Redhouse",
		State:     "Virginia",
		Country:   "US",
		Elevation: 742,
		Latitude:  37.188999176,
		Longitude: -78.8296966553,
		Timezone:  "America/New_York",
	},
	"7VA9": {
		ICAO:      "7VA9",
		Name:      "Hannah Field",
		City:      "Monterey",
		State:     "Virginia",
		Country:   "US",
		Elevation: 3049,
		Latitude:  38.398333,
		Longitude: -79.600278,
		Timezone:  "America/New_York",
	},
	"7VG0": {
		ICAO:      "7VG0",
		Name:      "Warrenton Air Park",
		City:      "Warrenton",
		State:     "Virginia",
		Country:   "US",
		Elevation: 442,
		Latitude:  38.6557006836,
		Longitude: -77.7872009277,
		Timezone:  "America/New_York",
	},
	"7WA0": {
		ICAO:      "7WA0",
		Name:      "Big Andy Airport",
		City:      "Port Angeles",
		State:     "Washington",
		Country:   "US",
		Elevation: 830,
		Latitude:  48.098400116,
		Longitude: -123.6429977417,
		Timezone:  "America/Los_Angeles",
	},
	"7WA3": {
		ICAO:      "7WA3",
		Name:      "West Wind Airport",
		City:      "Everson",
		State:     "Washington",
		Country:   "US",
		Elevation: 160,
		Latitude:  48.886100769,
		Longitude: -122.3290023804,
		Timezone:  "America/Los_Angeles",
	},
	"7WA4": {
		ICAO:      "7WA4",
		Name:      "Humbert Airport",
		City:      "Ford",
		State:     "Washington",
		Country:   "US",
		Elevation: 1890,
		Latitude:  47.8574981689,
		Longitude: -117.7799987793,
		Timezone:  "America/Los_Angeles",
	},
	"7WA5": {
		ICAO:      "7WA5",
		Name:      "Stuart Island Airpark",
		City:      "Stuart Island",
		State:     "Washington",
		Country:   "US",
		Elevation: 10,
		Latitude:  48.6729011536,
		Longitude: -123.1760025024,
		Timezone:  "America/Los_Angeles",
	},
	"7WA7": {
		ICAO:      "7WA7",
		Name:      "Mc Whorter Ranch Airport",
		City:      "Prosser",
		State:     "Washington",
		Country:   "US",
		Elevation: 1356,
		Latitude:  46.3207015991,
		Longitude: -119.617996216,
		Timezone:  "America/Los_Angeles",
	},
	"7WI0": {
		ICAO:      "7WI0",
		Name:      "Turkey Bluff Airport",
		City:      "Ferryville",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1110,
		Latitude:  43.3730010986,
		Longitude: -91.1099014282,
		Timezone:  "America/Chicago",
	},
	"7WI2": {
		ICAO:      "7WI2",
		Name:      "Higgins Airport",
		City:      "Rio",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 965,
		Latitude:  43.4430007935,
		Longitude: -89.1781997681,
		Timezone:  "America/Chicago",
	},
	"7WI4": {
		ICAO:      "7WI4",
		Name:      "Lewis Airport",
		City:      "Black River Falls",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1041,
		Latitude:  44.3004989624,
		Longitude: -90.9878997803,
		Timezone:  "America/Chicago",
	},
	"7WI5": {
		ICAO:      "7WI5",
		Name:      "Syvrud Airport",
		City:      "Brooklyn",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1000,
		Latitude:  42.8824996948,
		Longitude: -89.4007034302,
		Timezone:  "America/Chicago",
	},
	"7WI6": {
		ICAO:      "7WI6",
		Name:      "Weatherbee Field",
		City:      "Wyocena",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 960,
		Latitude:  43.470500946,
		Longitude: -89.3590011597,
		Timezone:  "America/Chicago",
	},
	"7WI7": {
		ICAO:      "7WI7",
		Name:      "Charlies Airport",
		City:      "Medford",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1490,
		Latitude:  45.1589012146,
		Longitude: -90.1492996216,
		Timezone:  "America/Chicago",
	},
	"7WI8": {
		ICAO:      "7WI8",
		Name:      "Crispy Cedars Airport",
		City:      "Brussels",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 700,
		Latitude:  44.7806015015,
		Longitude: -87.6024017334,
		Timezone:  "America/Chicago",
	},
	"7WI9": {
		ICAO:      "7WI9",
		Name:      "Nett Construction Airport",
		City:      "Fond Du Lac",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1040,
		Latitude:  43.8305015564,
		Longitude: -88.3254013062,
		Timezone:  "America/Chicago",
	},
	"7XS0": {
		ICAO:      "7XS0",
		Name:      "Polly Ranch Airport",
		City:      "Friendswood",
		State:     "Texas",
		Country:   "US",
		Elevation: 24,
		Latitude:  29.504699707,
		Longitude: -95.1754989624,
		Timezone:  "America/Chicago",
	},
	"7XS1": {
		ICAO:      "7XS1",
		Name:      "Flying E Ranch Airport",
		City:      "Springtown",
		State:     "Texas",
		Country:   "US",
		Elevation: 850,
		Latitude:  32.9933013916,
		Longitude: -97.676902771,
		Timezone:  "America/Chicago",
	},
	"7XS2": {
		ICAO:      "7XS2",
		Name:      "Kenley's Mustang Prairie Ranch Airport",
		City:      "Crockett",
		State:     "Texas",
		Country:   "US",
		Elevation: 300,
		Latitude:  31.2423992157,
		Longitude: -95.6041030884,
		Timezone:  "America/Chicago",
	},
	"7XS3": {
		ICAO:      "7XS3",
		Name:      "W C Ranch Airport",
		City:      "Grapeland",
		State:     "Texas",
		Country:   "US",
		Elevation: 337,
		Latitude:  31.476600647,
		Longitude: -95.6350021362,
		Timezone:  "America/Chicago",
	},
	"7XS5": {
		ICAO:      "7XS5",
		Name:      "Christian Ranch Airport",
		City:      "Eldorado",
		State:     "Texas",
		Country:   "US",
		Elevation: 2425,
		Latitude:  30.8603000641,
		Longitude: -100.5670013428,
		Timezone:  "America/Chicago",
	},
	"7XS6": {
		ICAO:      "7XS6",
		Name:      "Moore Hx Ranch Airport",
		City:      "Fairfield",
		State:     "Texas",
		Country:   "US",
		Elevation: 400,
		Latitude:  31.7943992615,
		Longitude: -96.2080001831,
		Timezone:  "America/Chicago",
	},
	"7XS7": {
		ICAO:      "7XS7",
		Name:      "Indian Springs Ranch Airport",
		City:      "Fredericksburg",
		State:     "Texas",
		Country:   "US",
		Elevation: 2030,
		Latitude:  30.1147003174,
		Longitude: -98.931098938,
		Timezone:  "America/Chicago",
	},
	"80AR": {
		ICAO:      "80AR",
		Name:      "Price Field",
		City:      "Joiner",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 236,
		Latitude:  35.5000991821,
		Longitude: -90.2583999634,
		Timezone:  "America/Chicago",
	},
	"80CA": {
		ICAO:      "80CA",
		Name:      "Bacchi Valley Industries Airport",
		City:      "Lotus",
		State:     "California",
		Country:   "US",
		Elevation: 760,
		Latitude:  38.817401886,
		Longitude: -120.9219970703,
		Timezone:  "America/Los_Angeles",
	},
	"80CO": {
		ICAO:      "80CO",
		Name:      "Simonson Field",
		City:      "Pueblo",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5130,
		Latitude:  38.1516990662,
		Longitude: -104.7060012817,
		Timezone:  "America/Denver",
	},
	"80FD": {
		ICAO:      "80FD",
		Name:      "Brady Ranch Airport",
		City:      "Indiantown",
		State:     "Florida",
		Country:   "US",
		Elevation: 18,
		Latitude:  27.1133995056,
		Longitude: -80.6333999634,
		Timezone:  "America/New_York",
	},
	"80GA": {
		ICAO:      "80GA",
		Name:      "Murphy's Landing Airport",
		City:      "Grantville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 820,
		Latitude:  33.2587013245,
		Longitude: -84.8447036743,
		Timezone:  "America/New_York",
	},
	"80IN": {
		ICAO:      "80IN",
		Name:      "Gustin's /Private/ Airport",
		City:      "Harlan",
		State:     "Indiana",
		Country:   "US",
		Elevation: 740,
		Latitude:  41.2042007446,
		Longitude: -84.8051986694,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"80IS": {
		ICAO:      "80IS",
		Name:      "Martin Airport",
		City:      "Hillview",
		State:     "Illinois",
		Country:   "US",
		Elevation: 425,
		Latitude:  39.4169998169,
		Longitude: -90.5859985352,
		Timezone:  "America/Chicago",
	},
	"80KY": {
		ICAO:      "80KY",
		Name:      "David Lowe Airport",
		City:      "Sacramento",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 465,
		Latitude:  37.3961982727,
		Longitude: -87.235496521,
		Timezone:  "America/Chicago",
	},
	"80LA": {
		ICAO:      "80LA",
		Name:      "Luscombe Lane 1 Airport",
		City:      "Eunice",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 42,
		Latitude:  30.4263000488,
		Longitude: -92.3181991577,
		Timezone:  "America/Chicago",
	},
	"80MO": {
		ICAO:      "80MO",
		Name:      "Seiferd Field",
		City:      "Carthage",
		State:     "Missouri",
		Country:   "US",
		Elevation: 980,
		Latitude:  37.276536,
		Longitude: -94.351897,
		Timezone:  "America/Chicago",
	},
	"80NC": {
		ICAO:      "80NC",
		Name:      "May Airport",
		City:      "Greensboro",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 777,
		Latitude:  35.9990005493,
		Longitude: -79.6893005371,
		Timezone:  "America/New_York",
	},
	"80ND": {
		ICAO:      "80ND",
		Name:      "Troy Field",
		City:      "Crosby",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 2080,
		Latitude:  48.8786010742,
		Longitude: -103.3700027466,
		Timezone:  "America/Chicago",
	},
	"80NE": {
		ICAO:      "80NE",
		Name:      "R & R Farms Airport",
		City:      "Davenport",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1620,
		Latitude:  40.2817001343,
		Longitude: -97.8542022705,
		Timezone:  "America/Chicago",
	},
	"80OH": {
		ICAO:      "80OH",
		Name:      "Sloas Airport",
		City:      "Warren",
		State:     "Ohio",
		Country:   "US",
		Elevation: 885,
		Latitude:  41.2667007446,
		Longitude: -80.7665023804,
		Timezone:  "America/New_York",
	},
	"80OK": {
		ICAO:      "80OK",
		Name:      "Flying S Ranch Airport",
		City:      "Reydon",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 2440,
		Latitude:  35.5766983032,
		Longitude: -99.8861999512,
		Timezone:  "America/Chicago",
	},
	"80OR": {
		ICAO:      "80OR",
		Name:      "Wilderness Airport",
		City:      "Bly",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4540,
		Latitude:  42.3642997742,
		Longitude: -120.966003418,
		Timezone:  "America/Los_Angeles",
	},
	"80PN": {
		ICAO:      "80PN",
		Name:      "Hanny Beaver Airpark Inc Airport",
		City:      "Hookstown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1220,
		Latitude:  40.5416984558,
		Longitude: -80.4525985718,
		Timezone:  "America/New_York",
	},
	"80TN": {
		ICAO:      "80TN",
		Name:      "Big T Airport",
		City:      "Lenoir City",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 900,
		Latitude:  35.8266983032,
		Longitude: -84.3300018311,
		Timezone:  "America/New_York",
	},
	"80TS": {
		ICAO:      "80TS",
		Name:      "M Y Ranch Airport",
		City:      "Crockett",
		State:     "Texas",
		Country:   "US",
		Elevation: 300,
		Latitude:  31.3276996613,
		Longitude: -95.5729980469,
		Timezone:  "America/Chicago",
	},
	"80TX": {
		ICAO:      "80TX",
		Name:      "Resaca Airstrip",
		City:      "Brownsville",
		State:     "Texas",
		Country:   "US",
		Elevation: 20,
		Latitude:  25.926700592,
		Longitude: -97.4085998535,
		Timezone:  "America/Chicago",
	},
	"80VA": {
		ICAO:      "80VA",
		Name:      "Holly Springs Airport",
		City:      "Richmond",
		State:     "Virginia",
		Country:   "US",
		Elevation: 250,
		Latitude:  37.7913017273,
		Longitude: -77.8246994019,
		Timezone:  "America/New_York",
	},
	"80WA": {
		ICAO:      "80WA",
		Name:      "Sfs Airpark",
		City:      "Hadlock",
		State:     "Washington",
		Country:   "US",
		Elevation: 120,
		Latitude:  48.0358009338,
		Longitude: -122.7740020752,
		Timezone:  "America/Los_Angeles",
	},
	"80WI": {
		ICAO:      "80WI",
		Name:      "Spring Creek Airport",
		City:      "Spring Lake",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 800,
		Latitude:  44.0229988098,
		Longitude: -89.1384963989,
		Timezone:  "America/Chicago",
	},
	"80XS": {
		ICAO:      "80XS",
		Name:      "Three Acres Airport",
		City:      "Woodbine",
		State:     "Texas",
		Country:   "US",
		Elevation: 775,
		Latitude:  33.6432991028,
		Longitude: -97.0021972656,
		Timezone:  "America/Chicago",
	},
	"81AK": {
		ICAO:      "81AK",
		Name:      "Mc Kinley Country Airport",
		City:      "Trapper Creek",
		State:     "Alaska",
		Country:   "US",
		Elevation: 600,
		Latitude:  62.5676994324,
		Longitude: -150.235992432,
		Timezone:  "America/Anchorage",
	},
	"81AR": {
		ICAO:      "81AR",
		Name:      "Smith's Strip",
		City:      "Lonoke",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 250,
		Latitude:  34.7500991821,
		Longitude: -91.8487014771,
		Timezone:  "America/Chicago",
	},
	"81CA": {
		ICAO:      "81CA",
		Name:      "7-M Ranch Airport",
		City:      "Middletown",
		State:     "California",
		Country:   "US",
		Elevation: 1153,
		Latitude:  38.7316017151,
		Longitude: -122.56300354,
		Timezone:  "America/Los_Angeles",
	},
	"81CO": {
		ICAO:      "81CO",
		Name:      "Mesa 1 Airport",
		City:      "Steamboat Springs",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7000,
		Latitude:  40.4061012268,
		Longitude: -106.8209991455,
		Timezone:  "America/Denver",
	},
	"81FL": {
		ICAO:      "81FL",
		Name:      "Cox's Hammock Airport",
		City:      "Indiantown",
		State:     "Florida",
		Country:   "US",
		Elevation: 35,
		Latitude:  27.0716991425,
		Longitude: -80.5201034546,
		Timezone:  "America/New_York",
	},
	"81GA": {
		ICAO:      "81GA",
		Name:      "Lucky Lairds Landing Airport",
		City:      "Villa Rica",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1130,
		Latitude:  33.6408996582,
		Longitude: -84.9048995972,
		Timezone:  "America/New_York",
	},
	"81IL": {
		ICAO:      "81IL",
		Name:      "Illinois Valley Parachute Club Airport",
		City:      "Minier",
		State:     "Illinois",
		Country:   "US",
		Elevation: 646,
		Latitude:  40.4352989197,
		Longitude: -89.3592987061,
		Timezone:  "America/Chicago",
	},
	"81KS": {
		ICAO:      "81KS",
		Name:      "Blackhawk Airport",
		City:      "Meriden",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1015,
		Latitude:  39.1562004089,
		Longitude: -95.5774993896,
		Timezone:  "America/Chicago",
	},
	"81KY": {
		ICAO:      "81KY",
		Name:      "Hi Rise Farm Airport",
		City:      "New Liberty",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 480,
		Latitude:  38.5783004761,
		Longitude: -85.0175018311,
		Timezone:  "America/New_York",
	},
	"81LL": {
		ICAO:      "81LL",
		Name:      "Otterbach Farm Airport",
		City:      "Mendota",
		State:     "Illinois",
		Country:   "US",
		Elevation: 840,
		Latitude:  41.5691986084,
		Longitude: -89.1736984253,
		Timezone:  "America/Chicago",
	},
	"81MN": {
		ICAO:      "81MN",
		Name:      "Ewen Landing Field",
		City:      "Jeffers",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1483,
		Latitude:  44.0304985046,
		Longitude: -95.1955032349,
		Timezone:  "America/Chicago",
	},
	"81NC": {
		ICAO:      "81NC",
		Name:      "Cox Field",
		City:      "Tabor City",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 50,
		Latitude:  34.0408325195,
		Longitude: -78.7399978638,
		Timezone:  "America/New_York",
	},
	"81ND": {
		ICAO:      "81ND",
		Name:      "Millers Airstrip",
		City:      "Deering",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1545,
		Latitude:  48.3885993958,
		Longitude: -101.0449981689,
		Timezone:  "America/Chicago",
	},
	"81NE": {
		ICAO:      "81NE",
		Name:      "Bunger Field",
		City:      "Hildreth",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2170,
		Latitude:  40.273601532,
		Longitude: -99.0078964233,
		Timezone:  "America/Chicago",
	},
	"81NM": {
		ICAO:      "81NM",
		Name:      "Diamond A Ranch Airport",
		City:      "Picacho",
		State:     "New Mexico",
		Country:   "US",
		Elevation: 5056,
		Latitude:  33.3451004028,
		Longitude: -105.1800003052,
		Timezone:  "America/Denver",
	},
	"81OH": {
		ICAO:      "81OH",
		Name:      "Elbel Airport",
		City:      "Hamersville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 975,
		Latitude:  38.8917007446,
		Longitude: -83.983001709,
		Timezone:  "America/New_York",
	},
	"81OK": {
		ICAO:      "81OK",
		Name:      "Twin Lakes Ranch Airport",
		City:      "Granite",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1630,
		Latitude:  35.0264015198,
		Longitude: -99.3394012451,
		Timezone:  "America/Chicago",
	},
	"81OR": {
		ICAO:      "81OR",
		Name:      "Wagontire Airport",
		City:      "Burns",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4725,
		Latitude:  43.2499008179,
		Longitude: -119.875,
		Timezone:  "America/Los_Angeles",
	},
	"81TE": {
		ICAO:      "81TE",
		Name:      "Horn Ranch Airport",
		City:      "Del Rio",
		State:     "Texas",
		Country:   "US",
		Elevation: 1290,
		Latitude:  29.5419006348,
		Longitude: -100.6169967651,
		Timezone:  "America/Chicago",
	},
	"81TS": {
		ICAO:      "81TS",
		Name:      "Dibrell Airport",
		City:      "Crockett",
		State:     "Texas",
		Country:   "US",
		Elevation: 385,
		Latitude:  31.4437999725,
		Longitude: -95.2754974365,
		Timezone:  "America/Chicago",
	},
	"81TX": {
		ICAO:      "81TX",
		Name:      "La Leona Airport",
		City:      "Terlingua",
		State:     "Texas",
		Country:   "US",
		Elevation: 2590,
		Latitude:  29.3432998657,
		Longitude: -103.5650024414,
		Timezone:  "America/Chicago",
	},
	"81VA": {
		ICAO:      "81VA",
		Name:      "Ferguson Airport",
		City:      "Rocky Mount",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1050,
		Latitude:  37.1180992126,
		Longitude: -79.7885971069,
		Timezone:  "America/New_York",
	},
	"81WI": {
		ICAO:      "81WI",
		Name:      "Gunner Field",
		City:      "Athens",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1465,
		Latitude:  45.0786018372,
		Longitude: -90.1764984131,
		Timezone:  "America/Chicago",
	},
	"82AK": {
		ICAO:      "82AK",
		Name:      "Jim's Landing Airport",
		City:      "Palmer",
		State:     "Alaska",
		Country:   "US",
		Elevation: 425,
		Latitude:  61.6166000366,
		Longitude: -149.1909942627,
		Timezone:  "America/Anchorage",
	},
	"82CL": {
		ICAO:      "82CL",
		IATA:      "FOB",
		Name:      "Fort Bragg Airport",
		City:      "Fort Bragg",
		State:     "California",
		Country:   "US",
		Elevation: 71,
		Latitude:  39.4743003845,
		Longitude: -123.79599762,
		Timezone:  "America/Los_Angeles",
	},
	"82FL": {
		ICAO:      "82FL",
		Name:      "Br Ranch Airport",
		City:      "Jupiter",
		State:     "Florida",
		Country:   "US",
		Elevation: 20,
		Latitude:  26.9141998291,
		Longitude: -80.2052993774,
		Timezone:  "America/New_York",
	},
	"82GA": {
		ICAO:      "82GA",
		Name:      "Paces South Farms Airport",
		City:      "Wadley",
		State:     "Georgia",
		Country:   "US",
		Elevation: 275,
		Latitude:  32.8087997437,
		Longitude: -82.3852996826,
		Timezone:  "America/New_York",
	},
	"82IN": {
		ICAO:      "82IN",
		Name:      "Hunter Airport",
		City:      "North Manchester",
		State:     "Indiana",
		Country:   "US",
		Elevation: 790,
		Latitude:  40.9812011719,
		Longitude: -85.9291992188,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"82IS": {
		ICAO:      "82IS",
		Name:      "Landings Condominium Airport",
		City:      "Huntley",
		State:     "Illinois",
		Country:   "US",
		Elevation: 880,
		Latitude:  42.1431007385,
		Longitude: -88.4029006958,
		Timezone:  "America/Chicago",
	},
	"82KY": {
		ICAO:      "82KY",
		Name:      "Woodledge Farm Airport",
		City:      "Crestwood",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 680,
		Latitude:  38.2834014893,
		Longitude: -85.4819030762,
		Timezone:  "America/Kentucky/Louisville",
	},
	"82LA": {
		ICAO:      "82LA",
		Name:      "Fisher's Field",
		City:      "Chalmette",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 5,
		Latitude:  29.9323997498,
		Longitude: -89.932800293,
		Timezone:  "America/Chicago",
	},
	"82LL": {
		ICAO:      "82LL",
		Name:      "Cheechako Airport",
		City:      "Monroe Center",
		State:     "Illinois",
		Country:   "US",
		Elevation: 860,
		Latitude:  42.1295013428,
		Longitude: -88.9923019409,
		Timezone:  "America/Chicago",
	},
	"82NC": {
		ICAO:      "82NC",
		Name:      "Pineview Air Airport",
		City:      "Sanford",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 300,
		Latitude:  35.3081016541,
		Longitude: -79.0733032227,
		Timezone:  "America/New_York",
	},
	"82NM": {
		ICAO:      "82NM",
		Name:      "Skeen Ranch Airport",
		City:      "Picacho",
		State:     "New Mexico",
		Country:   "US",
		Elevation: 5524,
		Latitude:  33.1701011658,
		Longitude: -105.1340026855,
		Timezone:  "America/Denver",
	},
	"82NY": {
		ICAO:      "82NY",
		Name:      "Silvernails Field",
		City:      "Gallatin",
		State:     "New York",
		Country:   "US",
		Elevation: 571,
		Latitude:  42.001389,
		Longitude: -73.67111,
		Timezone:  "America/New_York",
	},
	"82OH": {
		ICAO:      "82OH",
		Name:      "Rocky Ridge Airpark",
		City:      "Waterville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 650,
		Latitude:  41.5078010559,
		Longitude: -83.7713012695,
		Timezone:  "America/New_York",
	},
	"82OI": {
		ICAO:      "82OI",
		Name:      "J-Em Farm Airport",
		City:      "Columbus",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1005,
		Latitude:  40.1542015076,
		Longitude: -83.2749023438,
		Timezone:  "America/New_York",
	},
	"82OR": {
		ICAO:      "82OR",
		Name:      "Lost Creek Airport",
		City:      "Dexter",
		State:     "Oregon",
		Country:   "US",
		Elevation: 700,
		Latitude:  43.8992996216,
		Longitude: -122.8170013428,
		Timezone:  "America/Los_Angeles",
	},
	"82PA": {
		ICAO:      "82PA",
		Name:      "Frame Field",
		City:      "Candor",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1210,
		Latitude:  40.3967018127,
		Longitude: -80.3052978516,
		Timezone:  "America/New_York",
	},
	"82TE": {
		ICAO:      "82TE",
		Name:      "Tarry Bank Airport",
		City:      "Leon Springs",
		State:     "Texas",
		Country:   "US",
		Elevation: 1380,
		Latitude:  29.6916007996,
		Longitude: -98.7074966431,
		Timezone:  "America/Chicago",
	},
	"82TS": {
		ICAO:      "82TS",
		Name:      "Elmdale Airpark",
		City:      "Abilene",
		State:     "Texas",
		Country:   "US",
		Elevation: 1775,
		Latitude:  32.4500999451,
		Longitude: -99.6503982544,
		Timezone:  "America/Chicago",
	},
	"82VA": {
		ICAO:      "82VA",
		Name:      "Root Field",
		City:      "Verona",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1290,
		Latitude:  38.1731987,
		Longitude: -78.9692001343,
		Timezone:  "America/New_York",
	},
	"82WI": {
		ICAO:      "82WI",
		Name:      "Triple S Ranch Airport",
		City:      "Whitelaw",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 855,
		Latitude:  44.1492004395,
		Longitude: -87.8143997192,
		Timezone:  "America/Chicago",
	},
	"82XS": {
		ICAO:      "82XS",
		Name:      "Circle P Ranch Airport",
		City:      "Floresville",
		State:     "Texas",
		Country:   "US",
		Elevation: 500,
		Latitude:  29.1026992798,
		Longitude: -98.2166976929,
		Timezone:  "America/Chicago",
	},
	"83AK": {
		ICAO:      "83AK",
		Name:      "Tolovana Hot Springs Airport",
		City:      "Fairbanks",
		State:     "Alaska",
		Country:   "US",
		Elevation: 800,
		Latitude:  65.2565002441,
		Longitude: -148.8300018311,
		Timezone:  "America/Anchorage",
	},
	"83FD": {
		ICAO:      "83FD",
		Name:      "Lake Montaza Airport",
		City:      "Okeechobee",
		State:     "Florida",
		Country:   "US",
		Elevation: 52,
		Latitude:  27.5802993774,
		Longitude: -80.8152999878,
		Timezone:  "America/New_York",
	},
	"83FL": {
		ICAO:      "83FL",
		Name:      "Nassau Airport",
		City:      "Yulee",
		State:     "Florida",
		Country:   "US",
		Elevation: 20,
		Latitude:  30.6187992096,
		Longitude: -81.5336990356,
		Timezone:  "America/New_York",
	},
	"83GA": {
		ICAO:      "83GA",
		Name:      "Brock Airpark",
		City:      "Bainbridge",
		State:     "Georgia",
		Country:   "US",
		Elevation: 130,
		Latitude:  30.9997005463,
		Longitude: -84.5061035156,
		Timezone:  "America/New_York",
	},
	"83IL": {
		ICAO:      "83IL",
		Name:      "Oltman-Shuck Airport",
		City:      "Minonk",
		State:     "Illinois",
		Country:   "US",
		Elevation: 736,
		Latitude:  40.8927993774,
		Longitude: -89.1039962769,
		Timezone:  "America/Chicago",
	},
	"83IN": {
		ICAO:      "83IN",
		Name:      "Goodenough Airport",
		City:      "Peru",
		State:     "Indiana",
		Country:   "US",
		Elevation: 685,
		Latitude:  40.7397994995,
		Longitude: -86.00050354,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"83IS": {
		ICAO:      "83IS",
		Name:      "Prairie Airport",
		City:      "Irvington",
		State:     "Illinois",
		Country:   "US",
		Elevation: 550,
		Latitude:  38.426399231,
		Longitude: -89.1312026978,
		Timezone:  "America/Chicago",
	},
	"83KS": {
		ICAO:      "83KS",
		Name:      "Miller Aeroplane Field",
		City:      "Wamego",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1020,
		Latitude:  39.2352981567,
		Longitude: -96.299697876,
		Timezone:  "America/Chicago",
	},
	"83LA": {
		ICAO:      "83LA",
		Name:      "Ledet Airfield",
		City:      "Cut Off",
		State:     "Louisiana",
		Country:   "US",
		Latitude:  29.5555000305,
		Longitude: -90.329498291,
		Timezone:  "America/Chicago",
	},
	"83LL": {
		ICAO:      "83LL",
		Name:      "Lindauer Airport",
		City:      "New Athens",
		State:     "Illinois",
		Country:   "US",
		Elevation: 437,
		Latitude:  38.3036003113,
		Longitude: -89.8551025391,
		Timezone:  "America/Chicago",
	},
	"83MU": {
		ICAO:      "83MU",
		Name:      "Dalbom Ultralightport",
		City:      "Stella",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1130,
		Latitude:  36.7690010071,
		Longitude: -94.2124023438,
		Timezone:  "America/Chicago",
	},
	"83NE": {
		ICAO:      "83NE",
		Name:      "Abbott Airport",
		City:      "Long Pine",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2560,
		Latitude:  42.3250007629,
		Longitude: -99.7559967041,
		Timezone:  "America/Chicago",
	},
	"83NM": {
		ICAO:      "83NM",
		Name:      "King Ranch Airport",
		City:      "Pie Town",
		State:     "New Mexico",
		Country:   "US",
		Elevation: 7143,
		Latitude:  34.6481018066,
		Longitude: -108.0059967041,
		Timezone:  "America/Denver",
	},
	"83OH": {
		ICAO:      "83OH",
		Name:      "Adams Strip",
		City:      "Pleasent Hill",
		State:     "Ohio",
		Country:   "US",
		Elevation: 973,
		Latitude:  40.0623016357,
		Longitude: -84.3794021606,
		Timezone:  "America/New_York",
	},
	"83OK": {
		ICAO:      "83OK",
		Name:      "Sweetbriar Airport",
		City:      "Murry Spur",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 560,
		Latitude:  35.2568016052,
		Longitude: -94.5780029297,
		Timezone:  "America/Chicago",
	},
	"83OR": {
		ICAO:      "83OR",
		Name:      "Tamarack Springs Ranch Airport",
		City:      "Lostine",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3445,
		Latitude:  45.5013008118,
		Longitude: -117.4720001221,
		Timezone:  "America/Los_Angeles",
	},
	"83TA": {
		ICAO:      "83TA",
		Name:      "Rainbow Field",
		City:      "Purdon",
		State:     "Texas",
		Country:   "US",
		Elevation: 365,
		Latitude:  31.9002990723,
		Longitude: -96.6003036499,
		Timezone:  "America/Chicago",
	},
	"83TE": {
		ICAO:      "83TE",
		Name:      "K Bar Ranch Airport",
		City:      "Sabinal",
		State:     "Texas",
		Country:   "US",
		Elevation: 1208,
		Latitude:  29.4969005585,
		Longitude: -99.4938964844,
		Timezone:  "America/Chicago",
	},
	"83TS": {
		ICAO:      "83TS",
		Name:      "Moore Ranch Airport",
		City:      "Gordon",
		State:     "Texas",
		Country:   "US",
		Elevation: 1200,
		Latitude:  32.426700592,
		Longitude: -98.4197006226,
		Timezone:  "America/Chicago",
	},
	"83TX": {
		ICAO:      "83TX",
		Name:      "Texas A And M Flight Test Station Airport",
		City:      "Bryan",
		State:     "Texas",
		Country:   "US",
		Elevation: 264,
		Latitude:  30.6334991455,
		Longitude: -96.4835968018,
		Timezone:  "America/Chicago",
	},
	"83WA": {
		ICAO:      "83WA",
		Name:      "Darcy's Air Strip",
		City:      "Walla Walla",
		State:     "Washington",
		Country:   "US",
		Elevation: 2430,
		Latitude:  46.0275993347,
		Longitude: -118.1679992676,
		Timezone:  "America/Los_Angeles",
	},
	"83WI": {
		ICAO:      "83WI",
		Name:      "Jones Airport",
		City:      "Dorchester",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1410,
		Latitude:  44.970500946,
		Longitude: -90.3461990356,
		Timezone:  "America/Chicago",
	},
	"84AL": {
		ICAO:      "84AL",
		Name:      "Blessings Landing Air Ranch",
		City:      "Pinckard",
		State:     "Alabama",
		Country:   "US",
		Elevation: 400,
		Latitude:  31.3031,
		Longitude: -85.52705,
		Timezone:  "America/Chicago",
	},
	"84IL": {
		ICAO:      "84IL",
		Name:      "Rossi's Farm Airport",
		City:      "Minooka",
		State:     "Illinois",
		Country:   "US",
		Elevation: 580,
		Latitude:  41.5069999695,
		Longitude: -88.2778015137,
		Timezone:  "America/Chicago",
	},
	"84KY": {
		ICAO:      "84KY",
		Name:      "Greene County Parachute Center Airport",
		City:      "Bardstown",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 650,
		Latitude:  37.8152999878,
		Longitude: -85.5400009155,
		Timezone:  "America/New_York",
	},
	"84LA": {
		ICAO:      "84LA",
		Name:      "Touchstone Ridge Ultralightport",
		City:      "Haughton",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 180,
		Latitude:  32.552898407,
		Longitude: -93.5932006836,
		Timezone:  "America/Chicago",
	},
	"84MI": {
		ICAO:      "84MI",
		Name:      "Timbers Sky Camp Airport",
		City:      "South Branch",
		State:     "Michigan",
		Country:   "US",
		Elevation: 945,
		Latitude:  44.4695014954,
		Longitude: -83.8818969727,
		Timezone:  "America/Detroit",
	},
	"84MN": {
		ICAO:      "84MN",
		Name:      "Nielsville Airport",
		City:      "Nielsville",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 869,
		Latitude:  47.5332984924,
		Longitude: -96.8309020996,
		Timezone:  "America/Chicago",
	},
	"84MT": {
		ICAO:      "84MT",
		Name:      "Shimmon Airport",
		City:      "Corvallis",
		State:     "Montana",
		Country:   "US",
		Elevation: 3920,
		Latitude:  46.280814,
		Longitude: -114.050214,
		Timezone:  "America/Denver",
	},
	"84NC": {
		ICAO:      "84NC",
		Name:      "Rooster Field",
		City:      "Cameron",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 340,
		Latitude:  35.273333,
		Longitude: -79.248611,
		Timezone:  "America/New_York",
	},
	"84ND": {
		ICAO:      "84ND",
		Name:      "Kyllo Airport",
		City:      "Mc Canna",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1150,
		Latitude:  48.0008010864,
		Longitude: -97.7570037842,
		Timezone:  "America/Chicago",
	},
	"84NE": {
		ICAO:      "84NE",
		Name:      "Hewetts Airport",
		City:      "Beaver City",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2275,
		Latitude:  40.1707992554,
		Longitude: -99.8171005249,
		Timezone:  "America/Chicago",
	},
	"84NM": {
		ICAO:      "84NM",
		Name:      "Nalda Ranch Airport",
		City:      "Pie Town",
		State:     "New Mexico",
		Country:   "US",
		Elevation: 7531,
		Latitude:  34.3534011841,
		Longitude: -108.1620025635,
		Timezone:  "America/Denver",
	},
	"84NY": {
		ICAO:      "84NY",
		Name:      "Skyview Airport",
		City:      "Rush",
		State:     "New York",
		Country:   "US",
		Elevation: 642,
		Latitude:  42.9861984253,
		Longitude: -77.6125030518,
		Timezone:  "America/New_York",
	},
	"84OH": {
		ICAO:      "84OH",
		Name:      "Hanshell Flying Apple Airport",
		City:      "Octa",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1055,
		Latitude:  39.581199646,
		Longitude: -83.6510009766,
		Timezone:  "America/New_York",
	},
	"84OK": {
		ICAO:      "84OK",
		Name:      "Collier Farms Airport",
		City:      "Taloga",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1740,
		Latitude:  35.9388999939,
		Longitude: -99.0531997681,
		Timezone:  "America/Chicago",
	},
	"84OL": {
		ICAO:      "84OL",
		Name:      "Cotton Field",
		City:      "Broken Arrow",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 685,
		Latitude:  36.1104011536,
		Longitude: -95.7655029297,
		Timezone:  "America/Chicago",
	},
	"84OR": {
		ICAO:      "84OR",
		Name:      "B Bar Ranch Airport",
		City:      "Selma",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1400,
		Latitude:  42.2476005554,
		Longitude: -123.5699996948,
		Timezone:  "America/Los_Angeles",
	},
	"84PN": {
		ICAO:      "84PN",
		Name:      "Draco STOLport",
		City:      "Stewartstown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 625,
		Latitude:  39.7526016235,
		Longitude: -76.5354995728,
		Timezone:  "America/New_York",
	},
	"84TE": {
		ICAO:      "84TE",
		Name:      "W4 Ranch Airport",
		City:      "Morgan",
		State:     "Texas",
		Country:   "US",
		Elevation: 833,
		Latitude:  31.988067,
		Longitude: -97.543181,
		Timezone:  "America/Chicago",
	},
	"84TX": {
		ICAO:      "84TX",
		Name:      "Camp Longhorn Airport",
		City:      "Buchanan Dam",
		State:     "Texas",
		Country:   "US",
		Elevation: 934,
		Latitude:  30.7413005829,
		Longitude: -98.3786010742,
		Timezone:  "America/Chicago",
	},
	"84WA": {
		ICAO:      "84WA",
		Name:      "D and B Airpark",
		City:      "Elma",
		State:     "Washington",
		Country:   "US",
		Elevation: 55,
		Latitude:  46.9729003906,
		Longitude: -123.3840026855,
		Timezone:  "America/Los_Angeles",
	},
	"84XS": {
		ICAO:      "84XS",
		Name:      "Lang Ranch Airport",
		City:      "Mountain Home",
		State:     "Texas",
		Country:   "US",
		Elevation: 2200,
		Latitude:  30.255,
		Longitude: -99.398889,
		Timezone:  "America/Chicago",
	},
	"85FA": {
		ICAO:      "85FA",
		Name:      "North Exuma Airport",
		City:      "Deland",
		State:     "Florida",
		Country:   "US",
		Elevation: 62,
		Latitude:  29.0949001312,
		Longitude: -81.2776031494,
		Timezone:  "America/New_York",
	},
	"85FL": {
		ICAO:      "85FL",
		Name:      "Thomas Farms Airport",
		City:      "Jay",
		State:     "Florida",
		Country:   "US",
		Elevation: 215,
		Latitude:  30.9692993164,
		Longitude: -87.0515975952,
		Timezone:  "America/Chicago",
	},
	"85GA": {
		ICAO:      "85GA",
		Name:      "Aiken Field",
		City:      "Talmo",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1020,
		Latitude:  34.1722984314,
		Longitude: -83.7310028076,
		Timezone:  "America/New_York",
	},
	"85IA": {
		ICAO:      "85IA",
		Name:      "Henry Airport",
		City:      "Floyd",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1125,
		Latitude:  43.1786003113,
		Longitude: -92.6974029541,
		Timezone:  "America/Chicago",
	},
	"85IL": {
		ICAO:      "85IL",
		Name:      "Durbin Airport",
		City:      "Sullivan",
		State:     "Illinois",
		Country:   "US",
		Elevation: 664,
		Latitude:  39.637298584,
		Longitude: -88.6359024048,
		Timezone:  "America/Chicago",
	},
	"85IN": {
		ICAO:      "85IN",
		Name:      "Leak Airport",
		City:      "Tab",
		State:     "Indiana",
		Country:   "US",
		Elevation: 720,
		Latitude:  40.4338989258,
		Longitude: -87.5049972534,
		Timezone:  "America/Chicago",
	},
	"85KS": {
		ICAO:      "85KS",
		Name:      "Sanders Airport",
		City:      "Winchester",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1070,
		Latitude:  39.3009986877,
		Longitude: -95.2145004272,
		Timezone:  "America/Chicago",
	},
	"85KY": {
		ICAO:      "85KY",
		Name:      "Deer Run Airpark",
		City:      "New Castle",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 890,
		Latitude:  38.4702987671,
		Longitude: -85.1386032104,
		Timezone:  "America/New_York",
	},
	"85LL": {
		ICAO:      "85LL",
		Name:      "Wormley Airport",
		City:      "Oswego",
		State:     "Illinois",
		Country:   "US",
		Elevation: 680,
		Latitude:  41.6838989258,
		Longitude: -88.262298584,
		Timezone:  "America/Chicago",
	},
	"85MI": {
		ICAO:      "85MI",
		Name:      "Dewind Field",
		City:      "Zeeland",
		State:     "Michigan",
		Country:   "US",
		Elevation: 640,
		Latitude:  42.9323005676,
		Longitude: -86.0138015747,
		Timezone:  "America/Detroit",
	},
	"85MN": {
		ICAO:      "85MN",
		Name:      "Christison Airport",
		City:      "Plainview",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1175,
		Latitude:  44.1585998535,
		Longitude: -92.2021026611,
		Timezone:  "America/Chicago",
	},
	"85MO": {
		ICAO:      "85MO",
		Name:      "Willhite Airport",
		City:      "Swedeborg",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1022,
		Latitude:  37.875,
		Longitude: -92.3224029541,
		Timezone:  "America/Chicago",
	},
	"85NC": {
		ICAO:      "85NC",
		Name:      "Reagans Roost Ultralightport",
		City:      "Whittier",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 2080,
		Latitude:  35.4291992188,
		Longitude: -83.1668014526,
		Timezone:  "America/New_York",
	},
	"85NE": {
		ICAO:      "85NE",
		Name:      "Meyers Freedom Flight Hardy Airport",
		City:      "Hardy",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1624,
		Latitude:  40.0208015442,
		Longitude: -97.9420013428,
		Timezone:  "America/Chicago",
	},
	"85NY": {
		ICAO:      "85NY",
		Name:      "High Acres Airport",
		City:      "Sardinia",
		State:     "New York",
		Country:   "US",
		Elevation: 1540,
		Latitude:  42.5783996582,
		Longitude: -78.5199966431,
		Timezone:  "America/New_York",
	},
	"85OH": {
		ICAO:      "85OH",
		Name:      "Tounshendeaux Airport",
		City:      "West Farmington",
		State:     "Ohio",
		Country:   "US",
		Elevation: 838,
		Latitude:  41.4137001038,
		Longitude: -80.9469985962,
		Timezone:  "America/New_York",
	},
	"85OL": {
		ICAO:      "85OL",
		Name:      "Huscher Field",
		City:      "Cache",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1242,
		Latitude:  34.6194992065,
		Longitude: -98.5955963135,
		Timezone:  "America/Chicago",
	},
	"85OR": {
		ICAO:      "85OR",
		Name:      "Allen's Airstrip",
		City:      "Bly",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4410,
		Latitude:  42.4045982361,
		Longitude: -121.0490036011,
		Timezone:  "America/Los_Angeles",
	},
	"85PA": {
		ICAO:      "85PA",
		Name:      "Krumenacker Airport",
		City:      "Carrolltown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 2040,
		Latitude:  40.5951004028,
		Longitude: -78.7783966064,
		Timezone:  "America/New_York",
	},
	"85PN": {
		ICAO:      "85PN",
		Name:      "Kiski Airport",
		City:      "Vandergrift",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1280,
		Latitude:  40.5792007446,
		Longitude: -79.6053009033,
		Timezone:  "America/New_York",
	},
	"85TA": {
		ICAO:      "85TA",
		Name:      "J-Bar Ranch Airport",
		City:      "Collinsville",
		State:     "Texas",
		Country:   "US",
		Elevation: 718,
		Latitude:  33.5798988342,
		Longitude: -96.8124008179,
		Timezone:  "America/Chicago",
	},
	"85TE": {
		ICAO:      "85TE",
		Name:      "Old Kingsbury Aerodrome",
		City:      "Kingsbury",
		State:     "Texas",
		Country:   "US",
		Elevation: 560,
		Latitude:  29.6341991425,
		Longitude: -97.8116989136,
		Timezone:  "America/Chicago",
	},
	"85TX": {
		ICAO:      "85TX",
		Name:      "Rutherford Ranch Airport",
		City:      "Buda",
		State:     "Texas",
		Country:   "US",
		Elevation: 983,
		Latitude:  30.0776996613,
		Longitude: -97.9655990601,
		Timezone:  "America/Chicago",
	},
	"85WA": {
		ICAO:      "85WA",
		Name:      "Scott Seed Farm Airport",
		City:      "Pomeroy",
		State:     "Washington",
		Country:   "US",
		Elevation: 1866,
		Latitude:  46.5265007019,
		Longitude: -117.7649993896,
		Timezone:  "America/Los_Angeles",
	},
	"85WI": {
		ICAO:      "85WI",
		Name:      "Cub Bear Airport",
		City:      "Emerald",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1240,
		Latitude:  45.0443992615,
		Longitude: -92.2928009033,
		Timezone:  "America/Chicago",
	},
	"86AR": {
		ICAO:      "86AR",
		Name:      "Goacher Airport",
		City:      "Lonoke",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 235,
		Latitude:  34.8292999268,
		Longitude: -91.9542999268,
		Timezone:  "America/Chicago",
	},
	"86CL": {
		ICAO:      "86CL",
		Name:      "Ernst Field",
		City:      "Hemet",
		State:     "California",
		Country:   "US",
		Elevation: 2860,
		Latitude:  33.5971984863,
		Longitude: -116.8830032349,
		Timezone:  "America/Los_Angeles",
	},
	"86CO": {
		ICAO:      "86CO",
		Name:      "Lazy W Airport",
		City:      "Berthoud",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5200,
		Latitude:  40.3135986328,
		Longitude: -105.1449966431,
		Timezone:  "America/Denver",
	},
	"86FD": {
		ICAO:      "86FD",
		Name:      "Country Landings Airport",
		City:      "Bell",
		State:     "Florida",
		Country:   "US",
		Elevation: 55,
		Latitude:  29.7646999359,
		Longitude: -82.8964004517,
		Timezone:  "America/New_York",
	},
	"86GA": {
		ICAO:      "86GA",
		Name:      "Flying G Ranch Airport",
		City:      "Tunnel Hill",
		State:     "Georgia",
		Country:   "US",
		Elevation: 845,
		Latitude:  34.8679008484,
		Longitude: -85.0610961914,
		Timezone:  "America/New_York",
	},
	"86IL": {
		ICAO:      "86IL",
		Name:      "Sharp Airport",
		City:      "Bethany",
		State:     "Illinois",
		Country:   "US",
		Elevation: 674,
		Latitude:  39.681667,
		Longitude: -88.764444,
		Timezone:  "America/Chicago",
	},
	"86IS": {
		ICAO:      "86IS",
		Name:      "Michael Pfister Airport",
		City:      "Roanoke",
		State:     "Illinois",
		Country:   "US",
		Elevation: 805,
		Latitude:  40.7793998718,
		Longitude: -89.2577972412,
		Timezone:  "America/Chicago",
	},
	"86KS": {
		ICAO:      "86KS",
		Name:      "Coberly Airport",
		City:      "Gove",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2540,
		Latitude:  38.786889,
		Longitude: -100.419843,
		Timezone:  "America/Chicago",
	},
	"86KY": {
		ICAO:      "86KY",
		Name:      "Hornback Airport",
		City:      "Hodgenville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 760,
		Latitude:  37.6213989258,
		Longitude: -85.7921981812,
		Timezone:  "America/New_York",
	},
	"86LA": {
		ICAO:      "86LA",
		Name:      "Spring Airport",
		City:      "Kentwood",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 400,
		Latitude:  30.9738006592,
		Longitude: -90.6657028198,
		Timezone:  "America/Chicago",
	},
	"86MI": {
		ICAO:      "86MI",
		Name:      "Downwind Acres Airport",
		City:      "Willis",
		State:     "Michigan",
		Country:   "US",
		Elevation: 683,
		Latitude:  42.1528015137,
		Longitude: -83.5673980713,
		Timezone:  "America/Detroit",
	},
	"86NC": {
		ICAO:      "86NC",
		Name:      "Craig Craft Airport",
		City:      "Hertford",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 14,
		Latitude:  36.2635993958,
		Longitude: -76.4933013916,
		Timezone:  "America/New_York",
	},
	"86NE": {
		ICAO:      "86NE",
		Name:      "Orum Aerodrome",
		City:      "Blair",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1320,
		Latitude:  41.5416984558,
		Longitude: -96.2739028931,
		Timezone:  "America/Chicago",
	},
	"86OH": {
		ICAO:      "86OH",
		Name:      "Westfield Airport",
		City:      "Westfield Center",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1044,
		Latitude:  41.0127983093,
		Longitude: -81.9200973511,
		Timezone:  "America/New_York",
	},
	"86PN": {
		ICAO:      "86PN",
		Name:      "Seesholtz Airport",
		City:      "Bloomsburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 730,
		Latitude:  41.0444984436,
		Longitude: -76.3552017212,
		Timezone:  "America/New_York",
	},
	"86TA": {
		ICAO:      "86TA",
		Name:      "Slaughter Ranch Airport",
		City:      "Marble Falls",
		State:     "Texas",
		Country:   "US",
		Elevation: 890,
		Latitude:  30.6126995087,
		Longitude: -98.2975006104,
		Timezone:  "America/Chicago",
	},
	"86TE": {
		ICAO:      "86TE",
		Name:      "Matthews Ranch Airport",
		City:      "Santo",
		State:     "Texas",
		Country:   "US",
		Elevation: 835,
		Latitude:  32.6498985291,
		Longitude: -98.1473007202,
		Timezone:  "America/Chicago",
	},
	"86TS": {
		ICAO:      "86TS",
		Name:      "Fairway Farm Airport",
		City:      "San Augustine",
		State:     "Texas",
		Country:   "US",
		Elevation: 360,
		Latitude:  31.483499527,
		Longitude: -94.0335006714,
		Timezone:  "America/Chicago",
	},
	"86TX": {
		ICAO:      "86TX",
		Name:      "Flying J Airport",
		City:      "Bulverde",
		State:     "Texas",
		Country:   "US",
		Elevation: 1055,
		Latitude:  29.7371997833,
		Longitude: -98.3788986206,
		Timezone:  "America/Chicago",
	},
	"86WA": {
		ICAO:      "86WA",
		Name:      "Kapowsin Field",
		City:      "Kapowsin",
		State:     "Washington",
		Country:   "US",
		Elevation: 720,
		Latitude:  47.0040016174,
		Longitude: -122.236000061,
		Timezone:  "America/Los_Angeles",
	},
	"86WI": {
		ICAO:      "86WI",
		Name:      "Bakers Field",
		City:      "Blanchardville",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 875,
		Latitude:  42.8252983093,
		Longitude: -89.8701019287,
		Timezone:  "America/Chicago",
	},
	"86XS": {
		ICAO:      "86XS",
		Name:      "Gesin Ranches Airport",
		City:      "Sweetwater",
		State:     "Texas",
		Country:   "US",
		Elevation: 2200,
		Latitude:  32.400100708,
		Longitude: -100.3669967651,
		Timezone:  "America/Chicago",
	},
	"87AK": {
		ICAO:      "87AK",
		Name:      "Soloy Strip",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 545,
		Latitude:  61.6515007019,
		Longitude: -149.2879943848,
		Timezone:  "America/Anchorage",
	},
	"87CA": {
		ICAO:      "87CA",
		Name:      "Swanson Ranch Nr 1 Airport",
		City:      "Hanford",
		State:     "California",
		Country:   "US",
		Elevation: 248,
		Latitude:  36.3998985291,
		Longitude: -119.6179962158,
		Timezone:  "America/Los_Angeles",
	},
	"87CO": {
		ICAO:      "87CO",
		Name:      "Young's Strip",
		City:      "Bennett",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5650,
		Latitude:  39.6911010742,
		Longitude: -104.4160003662,
		Timezone:  "America/Denver",
	},
	"87IL": {
		ICAO:      "87IL",
		Name:      "Bushby RLA Restricted Landing Area",
		City:      "Minooka",
		State:     "Illinois",
		Country:   "US",
		Elevation: 580,
		Latitude:  41.5167007446,
		Longitude: -88.2667007446,
		Timezone:  "America/Chicago",
	},
	"87IN": {
		ICAO:      "87IN",
		Name:      "May's Strip",
		City:      "Walkerton",
		State:     "Indiana",
		Country:   "US",
		Elevation: 715,
		Latitude:  41.452999115,
		Longitude: -86.4697036743,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"87IS": {
		ICAO:      "87IS",
		Name:      "Seigfried Halfpap Airport",
		City:      "Quincy",
		State:     "Illinois",
		Country:   "US",
		Elevation: 550,
		Latitude:  39.8861999512,
		Longitude: -91.3207015991,
		Timezone:  "America/Chicago",
	},
	"87KS": {
		ICAO:      "87KS",
		Name:      "Holyrood Municipal Airport",
		City:      "Holyrood",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1805,
		Latitude:  38.5875015259,
		Longitude: -98.404800415,
		Timezone:  "America/Chicago",
	},
	"87MN": {
		ICAO:      "87MN",
		Name:      "Erickson Airport",
		City:      "Roosevelt",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1155,
		Latitude:  48.8102989197,
		Longitude: -95.2015991211,
		Timezone:  "America/Chicago",
	},
	"87MO": {
		ICAO:      "87MO",
		Name:      "Richters Airport",
		City:      "East Lynne",
		State:     "Missouri",
		Country:   "US",
		Elevation: 915,
		Latitude:  38.6960983276,
		Longitude: -94.2580032349,
		Timezone:  "America/Chicago",
	},
	"87NC": {
		ICAO:      "87NC",
		Name:      "Brocks Airport",
		City:      "Roseboro",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 120,
		Latitude:  34.916111,
		Longitude: -78.600277,
		Timezone:  "America/New_York",
	},
	"87NE": {
		ICAO:      "87NE",
		Name:      "Knox Landing Airport",
		City:      "York",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1638,
		Latitude:  40.8092002869,
		Longitude: -97.5877990723,
		Timezone:  "America/Chicago",
	},
	"87OH": {
		ICAO:      "87OH",
		Name:      "Mundron Field",
		City:      "Fayette",
		State:     "Ohio",
		Country:   "US",
		Elevation: 840,
		Latitude:  41.7056007385,
		Longitude: -84.3735961914,
		Timezone:  "America/Detroit",
	},
	"87OI": {
		ICAO:      "87OI",
		Name:      "Lake Air Ranch Airport",
		City:      "Norwalk",
		State:     "Ohio",
		Country:   "US",
		Elevation: 690,
		Latitude:  41.2723007202,
		Longitude: -82.6230010986,
		Timezone:  "America/New_York",
	},
	"87OR": {
		ICAO:      "87OR",
		Name:      "Moondance Ranch Airport",
		City:      "Beatty",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4980,
		Latitude:  42.6517982483,
		Longitude: -121.3290023804,
		Timezone:  "America/Los_Angeles",
	},
	"87PA": {
		ICAO:      "87PA",
		Name:      "Waltman Airport",
		City:      "Chicora",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1360,
		Latitude:  40.9264984131,
		Longitude: -79.7387008667,
		Timezone:  "America/New_York",
	},
	"87TA": {
		ICAO:      "87TA",
		Name:      "Cielo Grande Ranch Airport",
		City:      "Mountain Home",
		State:     "Texas",
		Country:   "US",
		Elevation: 2180,
		Latitude:  30.1891002655,
		Longitude: -99.2697982788,
		Timezone:  "America/Chicago",
	},
	"87TE": {
		ICAO:      "87TE",
		Name:      "Rose Field Airport",
		City:      "Needville",
		State:     "Texas",
		Country:   "US",
		Elevation: 100,
		Latitude:  29.4519004822,
		Longitude: -95.8655014038,
		Timezone:  "America/Chicago",
	},
	"87TX": {
		ICAO:      "87TX",
		Name:      "Porter Ranch Airport",
		City:      "Burnell",
		State:     "Texas",
		Country:   "US",
		Elevation: 418,
		Latitude:  28.6900005341,
		Longitude: -97.7602996826,
		Timezone:  "America/Chicago",
	},
	"87VA": {
		ICAO:      "87VA",
		Name:      "Cottonwood Farm Airport",
		City:      "Crozet",
		State:     "Virginia",
		Country:   "US",
		Elevation: 690,
		Latitude:  38.0811004639,
		Longitude: -78.695602417,
		Timezone:  "America/New_York",
	},
	"87WI": {
		ICAO:      "87WI",
		Name:      "Quale Airport",
		City:      "Cottage Grove",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 955,
		Latitude:  43.0194015503,
		Longitude: -89.1854019165,
		Timezone:  "America/Chicago",
	},
	"87XS": {
		ICAO:      "87XS",
		Name:      "Cinco B Ranch Airport",
		City:      "Medina",
		State:     "Texas",
		Country:   "US",
		Elevation: 1457,
		Latitude:  29.7877006531,
		Longitude: -99.262802124,
		Timezone:  "America/Chicago",
	},
	"88AK": {
		ICAO:      "88AK",
		Name:      "Dutch Landing Strip",
		City:      "Sterling",
		State:     "Alaska",
		Country:   "US",
		Elevation: 300,
		Latitude:  60.5402984619,
		Longitude: -150.8679962158,
		Timezone:  "America/Anchorage",
	},
	"88CO": {
		ICAO:      "88CO",
		Name:      "Tranquila Airport",
		City:      "Yoder",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6320,
		Latitude:  38.886100769,
		Longitude: -104.2490005493,
		Timezone:  "America/Denver",
	},
	"88GA": {
		ICAO:      "88GA",
		Name:      "Beaver Creek Airport",
		City:      "Concord",
		State:     "Georgia",
		Country:   "US",
		Elevation: 810,
		Latitude:  33.1119003296,
		Longitude: -84.4447021484,
		Timezone:  "America/New_York",
	},
	"88IL": {
		ICAO:      "88IL",
		Name:      "Mary's Landing Airport",
		City:      "Capron",
		State:     "Illinois",
		Country:   "US",
		Elevation: 910,
		Latitude:  42.4131011963,
		Longitude: -88.7332992554,
		Timezone:  "America/Chicago",
	},
	"88IN": {
		ICAO:      "88IN",
		Name:      "Gardner Airport",
		City:      "Boggstown",
		State:     "Indiana",
		Country:   "US",
		Elevation: 740,
		Latitude:  39.5797996521,
		Longitude: -85.9321975708,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"88IS": {
		ICAO:      "88IS",
		Name:      "Blickhan Lndg Area Airport",
		City:      "Quincy",
		State:     "Illinois",
		Country:   "US",
		Elevation: 635,
		Latitude:  40.0083999634,
		Longitude: -91.3821029663,
		Timezone:  "America/Chicago",
	},
	"88LA": {
		ICAO:      "88LA",
		Name:      "Freebird Field",
		City:      "Lafayette",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 20,
		Latitude:  30.1294002533,
		Longitude: -92.1351013184,
		Timezone:  "America/Chicago",
	},
	"88LL": {
		ICAO:      "88LL",
		Name:      "Block Airport",
		City:      "Pearl City",
		State:     "Illinois",
		Country:   "US",
		Elevation: 917,
		Latitude:  42.1916999817,
		Longitude: -89.8382034302,
		Timezone:  "America/Chicago",
	},
	"88MN": {
		ICAO:      "88MN",
		Name:      "Fox Field",
		City:      "Young America",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1000,
		Latitude:  44.801700592,
		Longitude: -93.950302124,
		Timezone:  "America/Chicago",
	},
	"88MO": {
		ICAO:      "88MO",
		Name:      "Howell Valley Airport",
		City:      "West Plains",
		State:     "Missouri",
		Country:   "US",
		Elevation: 950,
		Latitude:  36.7016983032,
		Longitude: -91.802696228,
		Timezone:  "America/Chicago",
	},
	"88NC": {
		ICAO:      "88NC",
		Name:      "Corriher Field",
		City:      "Salisbury",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 840,
		Latitude:  35.6324996948,
		Longitude: -80.6853027344,
		Timezone:  "America/New_York",
	},
	"88ND": {
		ICAO:      "88ND",
		Name:      "Goodman Strip",
		City:      "Milton",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1560,
		Latitude:  48.7089004517,
		Longitude: -97.9940032959,
		Timezone:  "America/Chicago",
	},
	"88NE": {
		ICAO:      "88NE",
		Name:      "Ensor Field",
		City:      "Johnson",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1170,
		Latitude:  40.4039001465,
		Longitude: -96.0171966553,
		Timezone:  "America/Chicago",
	},
	"88NV": {
		ICAO:      "88NV",
		Name:      "Black Rock City Airport",
		City:      "Gerlach",
		State:     "Nevada",
		Country:   "US",
		Elevation: 3904,
		Latitude:  40.757333,
		Longitude: -119.212861,
		Timezone:  "America/Los_Angeles",
	},
	"88NY": {
		ICAO:      "88NY",
		Name:      "Zelazny Airport",
		City:      "Shelby",
		State:     "New York",
		Country:   "US",
		Elevation: 635,
		Latitude:  43.1577987671,
		Longitude: -78.3583984375,
		Timezone:  "America/New_York",
	},
	"88OH": {
		ICAO:      "88OH",
		Name:      "Chippewa Field",
		City:      "Williston",
		State:     "Ohio",
		Country:   "US",
		Elevation: 596,
		Latitude:  41.5999984741,
		Longitude: -83.3165969849,
		Timezone:  "America/New_York",
	},
	"88OK": {
		ICAO:      "88OK",
		Name:      "Hilltop Airport",
		City:      "Stillwater",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1070,
		Latitude:  36.1934013367,
		Longitude: -97.0906982422,
		Timezone:  "America/Chicago",
	},
	"88OL": {
		ICAO:      "88OL",
		Name:      "Pace Field",
		City:      "Calvin",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 825,
		Latitude:  34.9645004272,
		Longitude: -96.2711029053,
		Timezone:  "America/Chicago",
	},
	"88OR": {
		ICAO:      "88OR",
		Name:      "Tallman Airport",
		City:      "Lebanon",
		State:     "Oregon",
		Country:   "US",
		Elevation: 303,
		Latitude:  44.5682983398,
		Longitude: -122.9710006714,
		Timezone:  "America/Los_Angeles",
	},
	"88PA": {
		ICAO:      "88PA",
		Name:      "Mc Coy Airport",
		City:      "Clinton",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1200,
		Latitude:  40.5391998291,
		Longitude: -80.4131011963,
		Timezone:  "America/New_York",
	},
	"88TA": {
		ICAO:      "88TA",
		Name:      "Figure 2 Ranch Airport",
		City:      "Van Horn",
		State:     "Texas",
		Country:   "US",
		Elevation: 3700,
		Latitude:  31.4559993744,
		Longitude: -104.84400177,
		Timezone:  "America/Chicago",
	},
	"88TE": {
		ICAO:      "88TE",
		Name:      "Thunderbird Southwest Airport",
		City:      "La Grange",
		State:     "Texas",
		Country:   "US",
		Elevation: 385,
		Latitude:  29.9016990662,
		Longitude: -96.7992019653,
		Timezone:  "America/Chicago",
	},
	"88TN": {
		ICAO:      "88TN",
		Name:      "Piney Creek Airport",
		City:      "Altamont",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1960,
		Latitude:  35.4245,
		Longitude: -85.779333,
		Timezone:  "America/Chicago",
	},
	"88VA": {
		ICAO:      "88VA",
		Name:      "Belmont Farm Airport",
		City:      "Culpeper",
		State:     "Virginia",
		Country:   "US",
		Elevation: 305,
		Latitude:  38.3741989136,
		Longitude: -77.991897583,
		Timezone:  "America/New_York",
	},
	"88VT": {
		ICAO:      "88VT",
		Name:      "Hawk's Nest Airport",
		City:      "Isle Lamotte",
		State:     "Vermont",
		Country:   "US",
		Elevation: 140,
		Latitude:  44.8782997131,
		Longitude: -73.3503036499,
		Timezone:  "America/New_York",
	},
	"88WI": {
		ICAO:      "88WI",
		Name:      "Nicolet Airport",
		City:      "Green Bay",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 790,
		Latitude:  44.5083007813,
		Longitude: -87.8607025146,
		Timezone:  "America/Chicago",
	},
	"88XS": {
		ICAO:      "88XS",
		Name:      "Bogan & Fontenot Airport",
		City:      "Nome",
		State:     "Texas",
		Country:   "US",
		Elevation: 35,
		Latitude:  30.0191001892,
		Longitude: -94.3877029419,
		Timezone:  "America/Chicago",
	},
	"89AK": {
		ICAO:      "89AK",
		Name:      "Lincoln Village Airpark",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 250,
		Latitude:  61.5593986511,
		Longitude: -149.7050018311,
		Timezone:  "America/Anchorage",
	},
	"89CA": {
		ICAO:      "89CA",
		Name:      "Swanson Ranch Nr 2 Airport",
		City:      "Riverdale",
		State:     "California",
		Country:   "US",
		Elevation: 237,
		Latitude:  36.4665985107,
		Longitude: -119.8840026855,
		Timezone:  "America/Los_Angeles",
	},
	"89CO": {
		ICAO:      "89CO",
		Name:      "Vallery Airport",
		City:      "Wiggins",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4480,
		Latitude:  40.238899231,
		Longitude: -103.9919967651,
		Timezone:  "America/Denver",
	},
	"89GE": {
		ICAO:      "89GE",
		Name:      "South Fulton Airport",
		City:      "Palmetto",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1025,
		Latitude:  33.5376014709,
		Longitude: -84.6388015747,
		Timezone:  "America/New_York",
	},
	"89IA": {
		ICAO:      "89IA",
		Name:      "Chain Lakes Airpark",
		City:      "Cedar Rapids",
		State:     "Iowa",
		Country:   "US",
		Elevation: 825,
		Latitude:  42.051399231,
		Longitude: -91.716796875,
		Timezone:  "America/Chicago",
	},
	"89IN": {
		ICAO:      "89IN",
		Name:      "Milhon Airport",
		City:      "Martinsville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 615,
		Latitude:  39.5069999695,
		Longitude: -86.3897018433,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"89IS": {
		ICAO:      "89IS",
		Name:      "Voges Airstrip",
		City:      "Red Bud",
		State:     "Illinois",
		Country:   "US",
		Elevation: 424,
		Latitude:  38.2167015076,
		Longitude: -89.9750976563,
		Timezone:  "America/Chicago",
	},
	"89LL": {
		ICAO:      "89LL",
		Name:      "Norman Airport",
		City:      "Peotone",
		State:     "Illinois",
		Country:   "US",
		Elevation: 750,
		Latitude:  41.3583984375,
		Longitude: -87.7249984741,
		Timezone:  "America/Chicago",
	},
	"89MN": {
		ICAO:      "89MN",
		Name:      "Beskar Airport",
		City:      "Taylors Falls",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 900,
		Latitude:  45.3876991272,
		Longitude: -92.6968994141,
		Timezone:  "America/Chicago",
	},
	"89MO": {
		ICAO:      "89MO",
		Name:      "Bishop's Landing Airport",
		City:      "Westline",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1000,
		Latitude:  38.5833015442,
		Longitude: -94.5796966553,
		Timezone:  "America/Chicago",
	},
	"89NC": {
		ICAO:      "89NC",
		Name:      "Strickland Field",
		City:      "Fair Bluff",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 98,
		Latitude:  34.2663993835,
		Longitude: -79.0235977173,
		Timezone:  "America/New_York",
	},
	"89ND": {
		ICAO:      "89ND",
		Name:      "Poleschook Airport",
		City:      "Minot",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 2245,
		Latitude:  47.9113998413,
		Longitude: -101.2490005493,
		Timezone:  "America/Chicago",
	},
	"89NE": {
		ICAO:      "89NE",
		Name:      "George Airport",
		City:      "Hershey",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2850,
		Latitude:  41.17029953,
		Longitude: -101.0009994507,
		Timezone:  "America/Chicago",
	},
	"89NJ": {
		ICAO:      "89NJ",
		Name:      "Strawberry Fields Airport",
		City:      "Mays Landing",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 45,
		Latitude:  39.489200592,
		Longitude: -74.7236022949,
		Timezone:  "America/New_York",
	},
	"89NY": {
		ICAO:      "89NY",
		Name:      "Maxson Airfield",
		City:      "Alexandria Bay",
		State:     "New York",
		Country:   "US",
		Elevation: 340,
		Latitude:  44.3166999817,
		Longitude: -75.8996963501,
		Timezone:  "America/New_York",
	},
	"89OI": {
		ICAO:      "89OI",
		Name:      "Circle C Airport",
		City:      "Berlin Center",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1042,
		Latitude:  41.038898468,
		Longitude: -80.9840011597,
		Timezone:  "America/New_York",
	},
	"89OK": {
		ICAO:      "89OK",
		Name:      "Neversweat Too Airport",
		City:      "Sallisaw",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 565,
		Latitude:  35.4430999756,
		Longitude: -94.8974990845,
		Timezone:  "America/Chicago",
	},
	"89OR": {
		ICAO:      "89OR",
		Name:      "Mucky Flat Airport",
		City:      "Eagle Point",
		State:     "Oregon",
		Country:   "US",
		Elevation: 2000,
		Latitude:  42.5979003906,
		Longitude: -122.7129974365,
		Timezone:  "America/Los_Angeles",
	},
	"89TA": {
		ICAO:      "89TA",
		Name:      "Soaring Club of Houston Gliderport",
		City:      "Waller",
		State:     "Texas",
		Country:   "US",
		Elevation: 308,
		Latitude:  30.1926994324,
		Longitude: -95.9633026123,
		Timezone:  "America/Chicago",
	},
	"89TE": {
		ICAO:      "89TE",
		Name:      "Lajitas International Airport",
		City:      "Lajitas",
		State:     "Texas",
		Country:   "US",
		Elevation: 2630,
		Latitude:  29.27799988,
		Longitude: -103.6869965,
		Timezone:  "America/Chicago",
	},
	"89TS": {
		ICAO:      "89TS",
		Name:      "Carter Ranch Airport",
		City:      "Oakwood",
		State:     "Texas",
		Country:   "US",
		Elevation: 215,
		Latitude:  31.5667991638,
		Longitude: -95.7666015625,
		Timezone:  "America/Chicago",
	},
	"89TX": {
		ICAO:      "89TX",
		Name:      "Aero-Bee Ranch Airstrip",
		City:      "Burnet",
		State:     "Texas",
		Country:   "US",
		Elevation: 1242,
		Latitude:  30.8792991638,
		Longitude: -98.4169998169,
		Timezone:  "America/Chicago",
	},
	"89VA": {
		ICAO:      "89VA",
		Name:      "Hidden River Airport",
		City:      "Edinburg",
		State:     "Virginia",
		Country:   "US",
		Elevation: 780,
		Latitude:  38.5223007202,
		Longitude: -78.5242004395,
		Timezone:  "America/New_York",
	},
	"89WA": {
		ICAO:      "89WA",
		Name:      "Williams Airpatch Airport",
		City:      "Arlington",
		State:     "Washington",
		Country:   "US",
		Elevation: 130,
		Latitude:  48.2706985474,
		Longitude: -122.0189971924,
		Timezone:  "America/Los_Angeles",
	},
	"89WI": {
		ICAO:      "89WI",
		Name:      "Tuschen Airport",
		City:      "Jonesdale",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1060,
		Latitude:  42.9068984985,
		Longitude: -90.0320968628,
		Timezone:  "America/Chicago",
	},
	"8AK0": {
		ICAO:      "8AK0",
		Name:      "Diamond Nn Cannery Airport",
		City:      "South Naknek",
		State:     "Alaska",
		Country:   "US",
		Elevation: 60,
		Latitude:  58.710899353,
		Longitude: -156.9949951172,
		Timezone:  "America/Anchorage",
	},
	"8AK1": {
		ICAO:      "8AK1",
		Name:      "Jacobus Field",
		City:      "Gulkana",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1670,
		Latitude:  62.2803001404,
		Longitude: -145.3630065918,
		Timezone:  "America/Anchorage",
	},
	"8AK2": {
		ICAO:      "8AK2",
		Name:      "Carty's Airstrip",
		City:      "Kenai",
		State:     "Alaska",
		Country:   "US",
		Elevation: 150,
		Latitude:  60.7024993896,
		Longitude: -151.2940063477,
		Timezone:  "America/Anchorage",
	},
	"8AK3": {
		ICAO:      "8AK3",
		Name:      "Roland Norton Memorial Airstrip",
		City:      "Selawik",
		State:     "Alaska",
		Country:   "US",
		Elevation: 360,
		Latitude:  66.7659988403,
		Longitude: -160.1529998779,
		Timezone:  "America/Anchorage",
	},
	"8AK6": {
		ICAO:      "8AK6",
		Name:      "Little Susitna Airport",
		City:      "Susitna Station",
		State:     "Alaska",
		Country:   "US",
		Elevation: 70,
		Latitude:  61.3718986511,
		Longitude: -150.2619934082,
		Timezone:  "America/Anchorage",
	},
	"8AK8": {
		ICAO:      "8AK8",
		Name:      "North Cubs Strip Airport",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 350,
		Latitude:  61.6300010681,
		Longitude: -149.682998657,
		Timezone:  "America/Anchorage",
	},
	"8AK9": {
		ICAO:      "8AK9",
		Name:      "Tok 2 Airport",
		City:      "Tok",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1630,
		Latitude:  63.308904791,
		Longitude: -143.016929626,
		Timezone:  "America/Anchorage",
	},
	"8AN6": {
		ICAO:      "8AN6",
		Name:      "Isaacson Airport",
		City:      "Medical Lake",
		State:     "Washington",
		Country:   "US",
		Elevation: 2410,
		Latitude:  47.659198761,
		Longitude: -117.6819992065,
		Timezone:  "America/Los_Angeles",
	},
	"8AR0": {
		ICAO:      "8AR0",
		Name:      "Hargrove Airport",
		City:      "Stuttgart",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 211,
		Latitude:  34.5125999451,
		Longitude: -91.5640029907,
		Timezone:  "America/Chicago",
	},
	"8AR1": {
		ICAO:      "8AR1",
		Name:      "Totty Field Airport",
		City:      "Oakland",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 780,
		Latitude:  36.4893989563,
		Longitude: -92.6421966553,
		Timezone:  "America/Chicago",
	},
	"8AZ5": {
		ICAO:      "8AZ5",
		Name:      "Lizzy Lizard Airport",
		City:      "Portal",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4200,
		Latitude:  31.897944,
		Longitude: -109.056917,
		Timezone:  "America/Denver",
	},
	"8CA0": {
		ICAO:      "8CA0",
		Name:      "Osborne Airport",
		City:      "Victorville",
		State:     "California",
		Country:   "US",
		Elevation: 2900,
		Latitude:  34.5839004517,
		Longitude: -117.266998291,
		Timezone:  "America/Los_Angeles",
	},
	"8CA5": {
		ICAO:      "8CA5",
		Name:      "Pines Airpark",
		City:      "Winchester",
		State:     "California",
		Country:   "US",
		Elevation: 1420,
		Latitude:  33.6455993652,
		Longitude: -117.1100006104,
		Timezone:  "America/Los_Angeles",
	},
	"8CA8": {
		ICAO:      "8CA8",
		Name:      "Wallom Field",
		City:      "Lodi",
		State:     "California",
		Country:   "US",
		Elevation: 56,
		Latitude:  38.067401886,
		Longitude: -121.2080001831,
		Timezone:  "America/Los_Angeles",
	},
	"8CL0": {
		ICAO:      "8CL0",
		Name:      "Nichols Farms Airport",
		City:      "Palmdale",
		State:     "California",
		Country:   "US",
		Elevation: 2700,
		Latitude:  34.5889015198,
		Longitude: -117.8259963989,
		Timezone:  "America/Los_Angeles",
	},
	"8CL1": {
		ICAO:      "8CL1",
		Name:      "Lake Wohlford Resort Airport",
		City:      "Escondido",
		State:     "California",
		Country:   "US",
		Elevation: 1643,
		Latitude:  33.1753473206,
		Longitude: -117.002334595,
		Timezone:  "America/Los_Angeles",
	},
	"8CL2": {
		ICAO:      "8CL2",
		Name:      "Lucchetti Ranch Airport",
		City:      "Elk Grove",
		State:     "California",
		Country:   "US",
		Elevation: 96,
		Latitude:  38.423500061,
		Longitude: -121.2269973755,
		Timezone:  "America/Los_Angeles",
	},
	"8CL6": {
		ICAO:      "8CL6",
		Name:      "Moller Airport",
		City:      "Maxwell",
		State:     "California",
		Country:   "US",
		Elevation: 85,
		Latitude:  39.2885017395,
		Longitude: -122.1890029907,
		Timezone:  "America/Los_Angeles",
	},
	"8CL9": {
		ICAO:      "8CL9",
		Name:      "Bland Field",
		City:      "Kerman",
		State:     "California",
		Country:   "US",
		Elevation: 222,
		Latitude:  36.7123985291,
		Longitude: -120.0299987793,
		Timezone:  "America/Los_Angeles",
	},
	"8CO0": {
		ICAO:      "8CO0",
		Name:      "Kent Airport",
		City:      "Wiggins",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4480,
		Latitude:  40.2527999878,
		Longitude: -104.0029983521,
		Timezone:  "America/Denver",
	},
	"8CO2": {
		ICAO:      "8CO2",
		Name:      "William Leon Schawo Airport",
		City:      "Briggsdale",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4950,
		Latitude:  40.6707992554,
		Longitude: -104.2080001831,
		Timezone:  "America/Denver",
	},
	"8CO4": {
		ICAO:      "8CO4",
		Name:      "East Moore Field",
		City:      "Byers",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5250,
		Latitude:  39.670501709,
		Longitude: -104.1360015869,
		Timezone:  "America/Denver",
	},
	"8CO5": {
		ICAO:      "8CO5",
		Name:      "Widner Airport",
		City:      "Hotchkiss",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6020,
		Latitude:  38.8339004517,
		Longitude: -107.891998291,
		Timezone:  "America/Denver",
	},
	"8CO7": {
		ICAO:      "8CO7",
		Name:      "Colorado Antique Field",
		City:      "Niwot",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5069,
		Latitude:  40.1225013733,
		Longitude: -105.1719970703,
		Timezone:  "America/Denver",
	},
	"8CO8": {
		ICAO:      "8CO8",
		Name:      "Koenig Airport",
		City:      "Yuma",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4150,
		Latitude:  40.1082992554,
		Longitude: -102.7279968262,
		Timezone:  "America/Denver",
	},
	"8CO9": {
		ICAO:      "8CO9",
		Name:      "Pine View Airport",
		City:      "Elizabeth",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6572,
		Latitude:  39.3443984985,
		Longitude: -104.5810012817,
		Timezone:  "America/Denver",
	},
	"8FA4": {
		ICAO:      "8FA4",
		Name:      "Samsula / Coe Field",
		State:     "Florida",
		Country:   "US",
		Elevation: 40,
		Latitude:  29.0102045831,
		Longitude: -81.1328315735,
		Timezone:  "America/New_York",
	},
	"8FD1": {
		ICAO:      "8FD1",
		Name:      "Buckner Airport",
		City:      "Mc Alpin",
		State:     "Florida",
		Country:   "US",
		Elevation: 90,
		Latitude:  30.0988006592,
		Longitude: -82.9275970459,
		Timezone:  "America/New_York",
	},
	"8FD2": {
		ICAO:      "8FD2",
		Name:      "The Flying Horseman Airport",
		City:      "Melrose",
		State:     "Florida",
		Country:   "US",
		Elevation: 192,
		Latitude:  29.7152004242,
		Longitude: -82.0942993164,
		Timezone:  "America/New_York",
	},
	"8FD3": {
		ICAO:      "8FD3",
		Name:      "Blackwater Airfield",
		City:      "Munson",
		State:     "Florida",
		Country:   "US",
		Elevation: 200,
		Latitude:  30.8777008057,
		Longitude: -86.8544006348,
		Timezone:  "America/Chicago",
	},
	"8FD5": {
		ICAO:      "8FD5",
		Name:      "Lykes Brighton Airport",
		City:      "Okeechobee",
		State:     "Florida",
		Country:   "US",
		Elevation: 30,
		Latitude:  27.1888999939,
		Longitude: -81.0869979858,
		Timezone:  "America/New_York",
	},
	"8FL0": {
		ICAO:      "8FL0",
		Name:      "Hutson Airfield",
		City:      "Hastings",
		State:     "Florida",
		Country:   "US",
		Elevation: 50,
		Latitude:  29.7280006409,
		Longitude: -81.4397964478,
		Timezone:  "America/New_York",
	},
	"8FL1": {
		ICAO:      "8FL1",
		Name:      "Mc Donald's Field",
		City:      "Arcadia",
		State:     "Florida",
		Country:   "US",
		Elevation: 54,
		Latitude:  27.1970005035,
		Longitude: -81.9156036377,
		Timezone:  "America/New_York",
	},
	"8FL2": {
		ICAO:      "8FL2",
		Name:      "Norton Airport",
		City:      "Belleview",
		State:     "Florida",
		Country:   "US",
		Elevation: 67,
		Latitude:  29.0011005402,
		Longitude: -82.1006011963,
		Timezone:  "America/New_York",
	},
	"8FL3": {
		ICAO:      "8FL3",
		Name:      "Back Achers Airport",
		City:      "Belleview",
		State:     "Florida",
		Country:   "US",
		Elevation: 90,
		Latitude:  29.0729999542,
		Longitude: -81.99949646,
		Timezone:  "America/New_York",
	},
	"8FL6": {
		ICAO:      "8FL6",
		Name:      "George T Mc Cutchan Airport",
		City:      "Harold",
		State:     "Florida",
		Country:   "US",
		Elevation: 160,
		Latitude:  30.673500061,
		Longitude: -86.8518981934,
		Timezone:  "America/Chicago",
	},
	"8GA1": {
		ICAO:      "8GA1",
		Name:      "Gum Creek Airport",
		City:      "Roopville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1070,
		Latitude:  33.4211997986,
		Longitude: -85.1619033813,
		Timezone:  "America/New_York",
	},
	"8GA3": {
		ICAO:      "8GA3",
		Name:      "Leesburg Spraying Airport",
		City:      "Leesburg",
		State:     "Georgia",
		Country:   "US",
		Elevation: 260,
		Latitude:  31.7751998901,
		Longitude: -84.1249008179,
		Timezone:  "America/New_York",
	},
	"8GA6": {
		ICAO:      "8GA6",
		Name:      "S & S Landing Strip",
		City:      "Griffin",
		State:     "Georgia",
		Country:   "US",
		Elevation: 900,
		Latitude:  33.16070175,
		Longitude: -84.3184967,
		Timezone:  "America/New_York",
	},
	"8GA7": {
		ICAO:      "8GA7",
		Name:      "Etowah Bend Gliderport",
		City:      "Kingston",
		State:     "Georgia",
		Country:   "US",
		Elevation: 620,
		Latitude:  34.2019996643,
		Longitude: -84.9772033691,
		Timezone:  "America/New_York",
	},
	"8GA8": {
		ICAO:      "8GA8",
		Name:      "Falcons Aerie Airport",
		City:      "Carrollton",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1180,
		Latitude:  33.5773010254,
		Longitude: -85.0029983521,
		Timezone:  "America/New_York",
	},
	"8GA9": {
		ICAO:      "8GA9",
		Name:      "Brook Bridge Aerodrome",
		City:      "Vaughn",
		State:     "Georgia",
		Country:   "US",
		Elevation: 820,
		Latitude:  33.2767982483,
		Longitude: -84.4107971191,
		Timezone:  "America/New_York",
	},
	"8IA2": {
		ICAO:      "8IA2",
		Name:      "Port Paradise Farms Airport",
		City:      "Winfield",
		State:     "Iowa",
		Country:   "US",
		Elevation: 700,
		Latitude:  41.1739006042,
		Longitude: -91.3618011475,
		Timezone:  "America/Chicago",
	},
	"8IA9": {
		ICAO:      "8IA9",
		Name:      "Shields Airport",
		City:      "Red Oak",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1245,
		Latitude:  41.15610123,
		Longitude: -95.24420166,
		Timezone:  "America/Chicago",
	},
	"8ID8": {
		ICAO:      "8ID8",
		Name:      "Lewis And Clark Airstrip",
		City:      "Salmon",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4530,
		Latitude:  45.353333,
		Longitude: -113.865,
		Timezone:  "America/Boise",
	},
	"8II0": {
		ICAO:      "8II0",
		Name:      "Allen and Gloss Airport",
		City:      "Bennington",
		State:     "Indiana",
		Country:   "US",
		Elevation: 880,
		Latitude:  38.8569984436,
		Longitude: -85.1013031006,
		Timezone:  "America/Indiana/Vevay",
	},
	"8II1": {
		ICAO:      "8II1",
		Name:      "Careferre Acres Airport",
		City:      "Jasonville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 520,
		Latitude:  39.1831016541,
		Longitude: -87.1260986328,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"8II2": {
		ICAO:      "8II2",
		Name:      "Norris Field",
		City:      "Liberty",
		State:     "Indiana",
		Country:   "US",
		Elevation: 1085,
		Latitude:  39.6530990601,
		Longitude: -84.8660964966,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"8II3": {
		ICAO:      "8II3",
		Name:      "Thrust Industries Airport",
		City:      "Haubstadt",
		State:     "Indiana",
		Country:   "US",
		Elevation: 460,
		Latitude:  38.2125015259,
		Longitude: -87.579498291,
		Timezone:  "America/Chicago",
	},
	"8II5": {
		ICAO:      "8II5",
		Name:      "Holt Field",
		City:      "Hoagland",
		State:     "Indiana",
		Country:   "US",
		Elevation: 790,
		Latitude:  40.8995018005,
		Longitude: -85,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"8IL0": {
		ICAO:      "8IL0",
		Name:      "Adkins RLA Restricted Landing Area",
		City:      "Harvard",
		State:     "Illinois",
		Country:   "US",
		Elevation: 853,
		Latitude:  42.3527984619,
		Longitude: -88.634803772,
		Timezone:  "America/Chicago",
	},
	"8IL1": {
		ICAO:      "8IL1",
		Name:      "Twin Gardens Airport",
		City:      "Harvard",
		State:     "Illinois",
		Country:   "US",
		Elevation: 900,
		Latitude:  42.4084014893,
		Longitude: -88.6529006958,
		Timezone:  "America/Chicago",
	},
	"8IL2": {
		ICAO:      "8IL2",
		Name:      "Kirkpatrick Airport",
		City:      "Harvard",
		State:     "Illinois",
		Country:   "US",
		Elevation: 950,
		Latitude:  42.4846992493,
		Longitude: -88.6000976563,
		Timezone:  "America/Chicago",
	},
	"8IL3": {
		ICAO:      "8IL3",
		Name:      "Butler Hill RLA Restricted Landing Area",
		City:      "West Brooklyn",
		State:     "Illinois",
		Country:   "US",
		Elevation: 765,
		Latitude:  41.7380981445,
		Longitude: -89.1624984741,
		Timezone:  "America/Chicago",
	},
	"8IL5": {
		ICAO:      "8IL5",
		Name:      "O'Connor Field",
		City:      "Pecatonica",
		State:     "Illinois",
		Country:   "US",
		Elevation: 900,
		Latitude:  42.3680992126,
		Longitude: -89.3542022705,
		Timezone:  "America/Chicago",
	},
	"8IL6": {
		ICAO:      "8IL6",
		Name:      "Robertson's Roost Airport",
		City:      "Edgewood",
		State:     "Illinois",
		Country:   "US",
		Elevation: 542,
		Latitude:  38.8594017029,
		Longitude: -88.6585998535,
		Timezone:  "America/Chicago",
	},
	"8IN2": {
		ICAO:      "8IN2",
		Name:      "Robinson Field",
		City:      "Greenwood",
		State:     "Indiana",
		Country:   "US",
		Elevation: 840,
		Latitude:  39.623298645,
		Longitude: -86.0314025879,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"8IN7": {
		ICAO:      "8IN7",
		Name:      "Brush Creek Airport",
		City:      "Butlerville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 825,
		Latitude:  39.04529953,
		Longitude: -85.515296936,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"8IN9": {
		ICAO:      "8IN9",
		Name:      "Marcidale Airport",
		City:      "Amo",
		State:     "Indiana",
		Country:   "US",
		Elevation: 855,
		Latitude:  39.68560028,
		Longitude: -86.60220337,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"8IS1": {
		ICAO:      "8IS1",
		Name:      "Nelson Airport",
		City:      "Carlinville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 615,
		Latitude:  39.307800293,
		Longitude: -89.9932022095,
		Timezone:  "America/Chicago",
	},
	"8IS2": {
		ICAO:      "8IS2",
		Name:      "Kilsoquah Farm Airport",
		City:      "Roanoke",
		State:     "Indiana",
		Country:   "US",
		Elevation: 840,
		Latitude:  40.9977989197,
		Longitude: -85.3889007568,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"8IS5": {
		ICAO:      "8IS5",
		Name:      "Raymond Classen Memorial Airport",
		City:      "Danforth",
		State:     "Illinois",
		Country:   "US",
		Elevation: 650,
		Latitude:  40.8166999817,
		Longitude: -87.9916992188,
		Timezone:  "America/Chicago",
	},
	"8KS0": {
		ICAO:      "8KS0",
		Name:      "Morgan Farms Airport",
		City:      "Ulysses",
		State:     "Kansas",
		Country:   "US",
		Elevation: 3190,
		Latitude:  37.8125,
		Longitude: -101.4589996338,
		Timezone:  "America/Chicago",
	},
	"8KS1": {
		ICAO:      "8KS1",
		Name:      "Neu Field",
		City:      "Basehor",
		State:     "Kansas",
		Country:   "US",
		Elevation: 920,
		Latitude:  39.1719017029,
		Longitude: -94.980796814,
		Timezone:  "America/Chicago",
	},
	"8KS2": {
		ICAO:      "8KS2",
		Name:      "King Ranch Airport",
		City:      "Hamilton",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1160,
		Latitude:  37.9388999939,
		Longitude: -96.230796814,
		Timezone:  "America/Chicago",
	},
	"8KS3": {
		ICAO:      "8KS3",
		Name:      "Davis Airfield",
		City:      "Hiawatha",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1100,
		Latitude:  39.8527984619,
		Longitude: -95.4804992676,
		Timezone:  "America/Chicago",
	},
	"8KS4": {
		ICAO:      "8KS4",
		Name:      "Rans Airport",
		City:      "Hays",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2100,
		Latitude:  38.9049987793,
		Longitude: -99.3508987427,
		Timezone:  "America/Chicago",
	},
	"8KS5": {
		ICAO:      "8KS5",
		Name:      "Blue Sky Airport",
		City:      "Parsons",
		State:     "Kansas",
		Country:   "US",
		Elevation: 890,
		Latitude:  37.3166999817,
		Longitude: -95.2802963257,
		Timezone:  "America/Chicago",
	},
	"8KS7": {
		ICAO:      "8KS7",
		Name:      "Pine Sod Ranch Airport",
		City:      "Stilwell",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1072,
		Latitude:  38.7331008911,
		Longitude: -94.6452026367,
		Timezone:  "America/Chicago",
	},
	"8KS8": {
		ICAO:      "8KS8",
		Name:      "Prairie Cottage Airport",
		City:      "Chapman",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1222,
		Latitude:  38.9178009033,
		Longitude: -97.0077972412,
		Timezone:  "America/Chicago",
	},
	"8KS9": {
		ICAO:      "8KS9",
		Name:      "The Wilderness Airport",
		City:      "Williamsburg",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1050,
		Latitude:  38.4444999695,
		Longitude: -95.4253005981,
		Timezone:  "America/Chicago",
	},
	"8KY0": {
		ICAO:      "8KY0",
		Name:      "Alberta Ultralightport",
		City:      "Alberta",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 980,
		Latitude:  38.3967018127,
		Longitude: -84.4636001587,
		Timezone:  "America/New_York",
	},
	"8KY3": {
		ICAO:      "8KY3",
		Name:      "Battle Field",
		City:      "Perryville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 950,
		Latitude:  37.661111,
		Longitude: -84.9925,
		Timezone:  "America/New_York",
	},
	"8KY6": {
		ICAO:      "8KY6",
		Name:      "Caintuckee Airport",
		City:      "Independence",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 930,
		Latitude:  38.935528,
		Longitude: -84.568278,
		Timezone:  "America/New_York",
	},
	"8LA9": {
		ICAO:      "8LA9",
		Name:      "Koll Airport",
		City:      "Welsh",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 40,
		Latitude:  30.3570995331,
		Longitude: -92.7445983887,
		Timezone:  "America/Chicago",
	},
	"8LL0": {
		ICAO:      "8LL0",
		Name:      "Nance Airport",
		City:      "Old Ripley",
		State:     "Illinois",
		Country:   "US",
		Elevation: 570,
		Latitude:  38.8875007629,
		Longitude: -89.5572967529,
		Timezone:  "America/Chicago",
	},
	"8LL1": {
		ICAO:      "8LL1",
		Name:      "Hunter Airport",
		City:      "Woodstock",
		State:     "Illinois",
		Country:   "US",
		Elevation: 914,
		Latitude:  42.3567008972,
		Longitude: -88.5062026978,
		Timezone:  "America/Chicago",
	},
	"8LL2": {
		ICAO:      "8LL2",
		Name:      "Eckberg Airport",
		City:      "Wyanet",
		State:     "Illinois",
		Country:   "US",
		Elevation: 670,
		Latitude:  41.3694992065,
		Longitude: -89.5597991943,
		Timezone:  "America/Chicago",
	},
	"8LL3": {
		ICAO:      "8LL3",
		Name:      "Hundley Airport",
		City:      "Rochester",
		State:     "Illinois",
		Country:   "US",
		Elevation: 590,
		Latitude:  39.7092005795,
		Longitude: -89.4898438454,
		Timezone:  "America/Chicago",
	},
	"8LL7": {
		ICAO:      "8LL7",
		Name:      "Donald A. Hamilton Airport",
		City:      "Pawnee",
		State:     "Illinois",
		Country:   "US",
		Elevation: 605,
		Latitude:  39.5945014954,
		Longitude: -89.5944976807,
		Timezone:  "America/Chicago",
	},
	"8MD0": {
		ICAO:      "8MD0",
		Name:      "Suzie Field",
		City:      "Woodbine",
		State:     "Maryland",
		Country:   "US",
		Elevation: 560,
		Latitude:  39.3833007813,
		Longitude: -77.0175018311,
		Timezone:  "America/New_York",
	},
	"8MD4": {
		ICAO:      "8MD4",
		Name:      "Spiering Airport",
		City:      "Greensboro",
		State:     "Maryland",
		Country:   "US",
		Elevation: 55,
		Latitude:  38.9805984497,
		Longitude: -75.7778015137,
		Timezone:  "America/New_York",
	},
	"8MD5": {
		ICAO:      "8MD5",
		Name:      "Harrison Farm Airport",
		City:      "Libertytown",
		State:     "Maryland",
		Country:   "US",
		Elevation: 610,
		Latitude:  39.498298645,
		Longitude: -77.2217025757,
		Timezone:  "America/New_York",
	},
	"8MD6": {
		ICAO:      "8MD6",
		Name:      "Burgess Field",
		City:      "Riverside",
		State:     "Maryland",
		Country:   "US",
		Elevation: 35,
		Latitude:  38.3970985413,
		Longitude: -77.1468963623,
		Timezone:  "America/New_York",
	},
	"8MD7": {
		ICAO:      "8MD7",
		Name:      "Deerfield Airport",
		City:      "Leonardtown",
		State:     "Maryland",
		Country:   "US",
		Elevation: 25,
		Latitude:  38.2386016846,
		Longitude: -76.6596984863,
		Timezone:  "America/New_York",
	},
	"8MI0": {
		ICAO:      "8MI0",
		Name:      "Trowbridge Farms Airport",
		City:      "South Haven",
		State:     "Michigan",
		Country:   "US",
		Elevation: 640,
		Latitude:  42.4119987488,
		Longitude: -86.1708984375,
		Timezone:  "America/Detroit",
	},
	"8MI3": {
		ICAO:      "8MI3",
		Name:      "Mattawan Airpark",
		City:      "Mattawan",
		State:     "Michigan",
		Country:   "US",
		Elevation: 830,
		Latitude:  42.1958999634,
		Longitude: -85.8056030273,
		Timezone:  "America/Detroit",
	},
	"8MI4": {
		ICAO:      "8MI4",
		Name:      "Howe Airport",
		City:      "Morley",
		State:     "Michigan",
		Country:   "US",
		Elevation: 871,
		Latitude:  43.4958992004,
		Longitude: -85.5736999512,
		Timezone:  "America/Detroit",
	},
	"8MI7": {
		ICAO:      "8MI7",
		Name:      "Bass Lake Airport",
		City:      "Steuben",
		State:     "Michigan",
		Country:   "US",
		Elevation: 805,
		Latitude:  46.1618995667,
		Longitude: -86.4842987061,
		Timezone:  "America/Detroit",
	},
	"8MI8": {
		ICAO:      "8MI8",
		Name:      "Cupp/Sjvs Landing Strip",
		City:      "Mendon",
		State:     "Michigan",
		Country:   "US",
		Elevation: 850,
		Latitude:  42.9947013855,
		Longitude: -85.4542007446,
		Timezone:  "America/Detroit",
	},
	"8MI9": {
		ICAO:      "8MI9",
		Name:      "Winters Field",
		City:      "Chelsea",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1000,
		Latitude:  42.3050003052,
		Longitude: -84.1066970825,
		Timezone:  "America/Detroit",
	},
	"8MN1": {
		ICAO:      "8MN1",
		Name:      "Shimpa Airstrip",
		City:      "Euclid",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 885,
		Latitude:  47.9667812491,
		Longitude: -96.640226841,
		Timezone:  "America/Chicago",
	},
	"8MN3": {
		ICAO:      "8MN3",
		Name:      "Breezy Point Airport",
		City:      "Pequot Lakes",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1255,
		Latitude:  46.5957984924,
		Longitude: -94.2200012207,
		Timezone:  "America/Chicago",
	},
	"8MN4": {
		ICAO:      "8MN4",
		Name:      "Sethney Personal STOLport",
		City:      "Deerwood",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1289,
		Latitude:  46.379699707,
		Longitude: -93.8135986328,
		Timezone:  "America/Chicago",
	},
	"8MN5": {
		ICAO:      "8MN5",
		Name:      "Vieira Airport",
		City:      "Lino Lakes",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 900,
		Latitude:  45.2083015442,
		Longitude: -93.1335983276,
		Timezone:  "America/Chicago",
	},
	"8MN8": {
		ICAO:      "8MN8",
		Name:      "Porter Airport",
		City:      "Jacobson",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1125,
		Latitude:  47.0126991272,
		Longitude: -93.3937988281,
		Timezone:  "America/Chicago",
	},
	"8MN9": {
		ICAO:      "8MN9",
		Name:      "Mavencamp Airport",
		City:      "Maple Lake",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1027,
		Latitude:  45.2569007874,
		Longitude: -94.039100647,
		Timezone:  "America/Chicago",
	},
	"8MO3": {
		ICAO:      "8MO3",
		Name:      "Curtis Field",
		City:      "Richmond",
		State:     "Missouri",
		Country:   "US",
		Elevation: 760,
		Latitude:  39.3246994019,
		Longitude: -93.9415969849,
		Timezone:  "America/Chicago",
	},
	"8MO4": {
		ICAO:      "8MO4",
		Name:      "Joan Lake Airport",
		City:      "Richwoods",
		State:     "Missouri",
		Country:   "US",
		Elevation: 800,
		Latitude:  38.2084007263,
		Longitude: -90.8667984009,
		Timezone:  "America/Chicago",
	},
	"8MO5": {
		ICAO:      "8MO5",
		Name:      "Cayton Pony Express Airport",
		City:      "Maysville",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1043,
		Latitude:  39.8227996826,
		Longitude: -94.3841018677,
		Timezone:  "America/Chicago",
	},
	"8MO7": {
		ICAO:      "8MO7",
		Name:      "Bean Lake Airport",
		City:      "Rushville",
		State:     "Missouri",
		Country:   "US",
		Elevation: 780,
		Latitude:  39.4916992188,
		Longitude: -95.0065994263,
		Timezone:  "America/Chicago",
	},
	"8MO8": {
		ICAO:      "8MO8",
		Name:      "Frazier Airport",
		City:      "Monett",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1400,
		Latitude:  36.9112014771,
		Longitude: -93.866897583,
		Timezone:  "America/Chicago",
	},
	"8NA0": {
		ICAO:      "8NA0",
		Name:      "Tappen Airstrip",
		City:      "Tappen",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1780,
		Latitude:  46.8810997009,
		Longitude: -99.630897522,
		Timezone:  "America/Chicago",
	},
	"8NA5": {
		ICAO:      "8NA5",
		Name:      "Liechty Farm Airport",
		City:      "Montpelier",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1505,
		Latitude:  46.6629981995,
		Longitude: -98.6304016113,
		Timezone:  "America/Chicago",
	},
	"8NA9": {
		ICAO:      "8NA9",
		Name:      "Hashbarger Farm Airstrip",
		City:      "Hope",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1195,
		Latitude:  47.327999115,
		Longitude: -97.6790008545,
		Timezone:  "America/Chicago",
	},
	"8NC1": {
		ICAO:      "8NC1",
		Name:      "Dean Wings Past Airport",
		City:      "Cameron",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 480,
		Latitude:  35.3763008118,
		Longitude: -79.2408981323,
		Timezone:  "America/New_York",
	},
	"8NC2": {
		ICAO:      "8NC2",
		Name:      "Summey Airpark",
		City:      "Caroleen",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 1000,
		Latitude:  35.2892990112,
		Longitude: -81.7851028442,
		Timezone:  "America/New_York",
	},
	"8NC3": {
		ICAO:      "8NC3",
		Name:      "Winding Creek Airport",
		City:      "Sneads Ferry",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 53,
		Latitude:  34.5661010742,
		Longitude: -77.4405975342,
		Timezone:  "America/New_York",
	},
	"8NC4": {
		ICAO:      "8NC4",
		Name:      "Dead Dog Airport",
		City:      "Pittsboro",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 525,
		Latitude:  35.7299003601,
		Longitude: -79.3127975464,
		Timezone:  "America/New_York",
	},
	"8NC5": {
		ICAO:      "8NC5",
		Name:      "Adams Airport",
		City:      "Rowland",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 143,
		Latitude:  34.5760993958,
		Longitude: -79.2488021851,
		Timezone:  "America/New_York",
	},
	"8NC6": {
		ICAO:      "8NC6",
		Name:      "Brooks Field",
		City:      "Siler City",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 620,
		Latitude:  35.6896018982,
		Longitude: -79.4069976807,
		Timezone:  "America/New_York",
	},
	"8NC7": {
		ICAO:      "8NC7",
		Name:      "Parker Field",
		City:      "Maysville",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 40,
		Latitude:  34.9364013672,
		Longitude: -77.2767028809,
		Timezone:  "America/New_York",
	},
	"8NC8": {
		ICAO:      "8NC8",
		Name:      "Lake Ridge Aero Park Airport",
		City:      "Durham",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 309,
		Latitude:  36.0601005554,
		Longitude: -78.7833023071,
		Timezone:  "America/New_York",
	},
	"8NC9": {
		ICAO:      "8NC9",
		Name:      "W.N.C. Air Museum Airport",
		City:      "Hendersonville",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 2083,
		Latitude:  35.3072013855,
		Longitude: -82.4336013794,
		Timezone:  "America/New_York",
	},
	"8ND0": {
		ICAO:      "8ND0",
		Name:      "Amble-Tiger North Farms Airport",
		City:      "Sarles",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1590,
		Latitude:  48.9014015198,
		Longitude: -99.0167999268,
		Timezone:  "America/Chicago",
	},
	"8ND4": {
		ICAO:      "8ND4",
		Name:      "Heyde Airport",
		City:      "Manvel",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 820,
		Latitude:  48.073600769,
		Longitude: -97.1862030029,
		Timezone:  "America/Chicago",
	},
	"8ND5": {
		ICAO:      "8ND5",
		Name:      "Ausk Strip",
		City:      "Chaffee",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1020,
		Latitude:  46.7639007568,
		Longitude: -97.4054031372,
		Timezone:  "America/Chicago",
	},
	"8ND6": {
		ICAO:      "8ND6",
		Name:      "J Vining Airport",
		City:      "Chaffee",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1025,
		Latitude:  46.7126998901,
		Longitude: -97.3908996582,
		Timezone:  "America/Chicago",
	},
	"8NE2": {
		ICAO:      "8NE2",
		Name:      "P & R Airport",
		City:      "Cedar Creek",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1100,
		Latitude:  41.0416984558,
		Longitude: -96.0335998535,
		Timezone:  "America/Chicago",
	},
	"8NE3": {
		ICAO:      "8NE3",
		Name:      "Smith Field",
		City:      "Champion",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3380,
		Latitude:  40.4818992615,
		Longitude: -101.81300354,
		Timezone:  "America/Denver",
	},
	"8NE4": {
		ICAO:      "8NE4",
		Name:      "Bornemeier Airstrip",
		City:      "Elmwood",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1292,
		Latitude:  40.8708000183,
		Longitude: -96.2919998169,
		Timezone:  "America/Chicago",
	},
	"8NE5": {
		ICAO:      "8NE5",
		Name:      "X1 Ranch Airport",
		City:      "Ericson",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2070,
		Latitude:  41.790599823,
		Longitude: -98.7297973633,
		Timezone:  "America/Chicago",
	},
	"8NE6": {
		ICAO:      "8NE6",
		Name:      "Franklin's Plainview Airport",
		City:      "Franklin",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2000,
		Latitude:  40.1208000183,
		Longitude: -98.9253997803,
		Timezone:  "America/Chicago",
	},
	"8NE7": {
		ICAO:      "8NE7",
		Name:      "Smith Airport",
		City:      "Franklin",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1948,
		Latitude:  40.0999984741,
		Longitude: -99.0003967285,
		Timezone:  "America/Chicago",
	},
	"8NE9": {
		ICAO:      "8NE9",
		Name:      "C A M P Airport",
		City:      "Fullerton",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1675,
		Latitude:  41.3903007507,
		Longitude: -97.9645004272,
		Timezone:  "America/Chicago",
	},
	"8NJ0": {
		ICAO:      "8NJ0",
		Name:      "Winchelsea Airport",
		City:      "Port Republic",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 30,
		Latitude:  39.5178985596,
		Longitude: -74.5090026855,
		Timezone:  "America/New_York",
	},
	"8NK3": {
		ICAO:      "8NK3",
		Name:      "Harris Airport",
		City:      "Fort Ann",
		State:     "Vermont",
		Country:   "US",
		Elevation: 261,
		Latitude:  43.5,
		Longitude: -73.066667,
		Timezone:  "America/New_York",
	},
	"8NK4": {
		ICAO:      "8NK4",
		Name:      "Bethany Airpark",
		City:      "Bethany Center",
		State:     "New York",
		Country:   "US",
		Elevation: 1000,
		Latitude:  42.9432983398,
		Longitude: -78.1382980347,
		Timezone:  "America/New_York",
	},
	"8NK6": {
		ICAO:      "8NK6",
		Name:      "Suntime Airport",
		City:      "West Burlington",
		State:     "New York",
		Country:   "US",
		Elevation: 1650,
		Latitude:  42.713056,
		Longitude: -75.168333,
		Timezone:  "America/New_York",
	},
	"8NY3": {
		ICAO:      "8NY3",
		Name:      "North Fork Airport",
		City:      "Afton",
		State:     "New York",
		Country:   "US",
		Elevation: 1455,
		Latitude:  42.268699646,
		Longitude: -75.5597000122,
		Timezone:  "America/New_York",
	},
	"8NY5": {
		ICAO:      "8NY5",
		Name:      "Mariaville Aerodrome",
		City:      "Duanesburg",
		State:     "New York",
		Country:   "US",
		Elevation: 1260,
		Latitude:  42.820400238,
		Longitude: -74.1482009888,
		Timezone:  "America/New_York",
	},
	"8NY7": {
		ICAO:      "8NY7",
		Name:      "Sha-Wan-Ga Valley Airport",
		City:      "Bloomingburg",
		State:     "New York",
		Country:   "US",
		Elevation: 437,
		Latitude:  41.5714988708,
		Longitude: -74.4023971558,
		Timezone:  "America/New_York",
	},
	"8OA5": {
		ICAO:      "8OA5",
		Name:      "Camp Crook Municipal Airport",
		City:      "Camp Crook",
		State:     "South Dakota",
		Country:   "US",
		Elevation: 3140,
		Latitude:  45.5666999817,
		Longitude: -103.9840011597,
		Timezone:  "America/Denver",
	},
	"8OA6": {
		ICAO:      "8OA6",
		Name:      "Warner Airstrip",
		City:      "Vickery",
		State:     "Ohio",
		Country:   "US",
		Elevation: 611,
		Latitude:  41.3861999512,
		Longitude: -82.897102356,
		Timezone:  "America/New_York",
	},
	"8OA7": {
		ICAO:      "8OA7",
		Name:      "Bald Eagle Field",
		City:      "Stockport",
		State:     "Ohio",
		Country:   "US",
		Elevation: 680,
		Latitude:  39.5564994812,
		Longitude: -81.78099823,
		Timezone:  "America/New_York",
	},
	"8OA9": {
		ICAO:      "8OA9",
		Name:      "Margos Sky Ranch Airport",
		City:      "Seville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1045,
		Latitude:  41.0444984436,
		Longitude: -81.8156967163,
		Timezone:  "America/New_York",
	},
	"8OH3": {
		ICAO:      "8OH3",
		Name:      "J and B Sky Ranch Airport",
		City:      "Coitsville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1125,
		Latitude:  41.1172981262,
		Longitude: -80.5225982666,
		Timezone:  "America/New_York",
	},
	"8OH4": {
		ICAO:      "8OH4",
		Name:      "York Aerodrome",
		City:      "Columbiana",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1260,
		Latitude:  40.8438987732,
		Longitude: -80.7025985718,
		Timezone:  "America/New_York",
	},
	"8OH5": {
		ICAO:      "8OH5",
		Name:      "Urban Airport",
		City:      "Cortland",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1045,
		Latitude:  41.3819999695,
		Longitude: -80.7123031616,
		Timezone:  "America/New_York",
	},
	"8OH7": {
		ICAO:      "8OH7",
		Name:      "Morris Field",
		City:      "East Palestine",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1250,
		Latitude:  40.8823013306,
		Longitude: -80.6003036499,
		Timezone:  "America/New_York",
	},
	"8OH8": {
		ICAO:      "8OH8",
		Name:      "Allen Airport",
		City:      "Dorset",
		State:     "Ohio",
		Country:   "US",
		Elevation: 985,
		Latitude:  41.6680984497,
		Longitude: -80.6856002808,
		Timezone:  "America/New_York",
	},
	"8OI0": {
		ICAO:      "8OI0",
		Name:      "Brannon Field",
		City:      "Newbury",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1250,
		Latitude:  41.4542007446,
		Longitude: -81.2453994751,
		Timezone:  "America/New_York",
	},
	"8OI3": {
		ICAO:      "8OI3",
		Name:      "Allen Airport",
		City:      "Cortland",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1105,
		Latitude:  41.3734016418,
		Longitude: -80.6689987183,
		Timezone:  "America/New_York",
	},
	"8OI5": {
		ICAO:      "8OI5",
		Name:      "Clum Airport",
		City:      "Thornville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1115,
		Latitude:  39.8816986084,
		Longitude: -82.4145965576,
		Timezone:  "America/New_York",
	},
	"8OI9": {
		ICAO:      "8OI9",
		Name:      "Morkassel Field",
		City:      "Idaho",
		State:     "Ohio",
		Country:   "US",
		Elevation: 850,
		Latitude:  39.0992012024,
		Longitude: -83.1166000366,
		Timezone:  "America/New_York",
	},
	"8OK0": {
		ICAO:      "8OK0",
		Name:      "Lamle Airport",
		City:      "Okeene",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1212,
		Latitude:  36.0682983398,
		Longitude: -98.2891998291,
		Timezone:  "America/Chicago",
	},
	"8OK1": {
		ICAO:      "8OK1",
		Name:      "Warbonnet Airport",
		City:      "Blanchard",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1335,
		Latitude:  35.1683006287,
		Longitude: -97.7108001709,
		Timezone:  "America/Chicago",
	},
	"8OK2": {
		ICAO:      "8OK2",
		Name:      "Baker Airstrip",
		City:      "El Reno",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1370,
		Latitude:  35.4751014709,
		Longitude: -97.9289016724,
		Timezone:  "America/Chicago",
	},
	"8OK4": {
		ICAO:      "8OK4",
		Name:      "Brandley Airport",
		City:      "El Reno",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1400,
		Latitude:  35.5041999817,
		Longitude: -98.0406036377,
		Timezone:  "America/Chicago",
	},
	"8OK6": {
		ICAO:      "8OK6",
		Name:      "Barry Dotson Ranch Airport",
		City:      "Marble City",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 880,
		Latitude:  35.5667991638,
		Longitude: -94.7501983643,
		Timezone:  "America/Chicago",
	},
	"8OK7": {
		ICAO:      "8OK7",
		Name:      "Cc & M Airport",
		City:      "Medford",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1085,
		Latitude:  36.7972984314,
		Longitude: -97.7183990479,
		Timezone:  "America/Chicago",
	},
	"8OL1": {
		ICAO:      "8OL1",
		Name:      "Petes Airpark",
		City:      "Wetumka",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 771,
		Latitude:  35.2279014587,
		Longitude: -96.2294998169,
		Timezone:  "America/Chicago",
	},
	"8OR2": {
		ICAO:      "8OR2",
		Name:      "Kingston Airpark",
		City:      "Stayton",
		State:     "Oregon",
		Country:   "US",
		Elevation: 871,
		Latitude:  44.7784996033,
		Longitude: -122.7300033569,
		Timezone:  "America/Los_Angeles",
	},
	"8OR3": {
		ICAO:      "8OR3",
		Name:      "Riverview Ranch Airport",
		City:      "Days Creek",
		State:     "Oregon",
		Country:   "US",
		Elevation: 840,
		Latitude:  42.9337005615,
		Longitude: -123.1210021973,
		Timezone:  "America/Los_Angeles",
	},
	"8OR5": {
		ICAO:      "8OR5",
		Name:      "Pilot Butte Airport",
		City:      "Bend",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3675,
		Latitude:  44.0472984314,
		Longitude: -121.2760009766,
		Timezone:  "America/Los_Angeles",
	},
	"8OR6": {
		ICAO:      "8OR6",
		Name:      "Grabhorn's Airport",
		City:      "Scappoose",
		State:     "Oregon",
		Country:   "US",
		Elevation: 255,
		Latitude:  45.7822990417,
		Longitude: -122.8939971924,
		Timezone:  "America/Los_Angeles",
	},
	"8OR7": {
		ICAO:      "8OR7",
		Name:      "Gates Airport",
		City:      "Corvallis",
		State:     "Oregon",
		Country:   "US",
		Elevation: 220,
		Latitude:  44.6271019,
		Longitude: -123.2249985,
		Timezone:  "America/Los_Angeles",
	},
	"8PA0": {
		ICAO:      "8PA0",
		Name:      "Numidia Airport",
		City:      "Numidia",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1070,
		Latitude:  40.8661994934,
		Longitude: -76.3973999023,
		Timezone:  "America/New_York",
	},
	"8PA1": {
		ICAO:      "8PA1",
		Name:      "Dee Jay Airport",
		City:      "Ono",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 505,
		Latitude:  40.4084014893,
		Longitude: -76.5038986206,
		Timezone:  "America/New_York",
	},
	"8PA3": {
		ICAO:      "8PA3",
		Name:      "Deer Meadows Airstrip",
		City:      "Weatherly",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1210,
		Latitude:  40.9958992004,
		Longitude: -75.7405014038,
		Timezone:  "America/New_York",
	},
	"8PA4": {
		ICAO:      "8PA4",
		Name:      "J F T Airport",
		City:      "Mifflinburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 660,
		Latitude:  40.9444999695,
		Longitude: -77.0457992554,
		Timezone:  "America/New_York",
	},
	"8PA6": {
		ICAO:      "8PA6",
		Name:      "Turtle Rock Airstrip",
		City:      "Shermansdale",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 626,
		Latitude:  40.329306,
		Longitude: -77.167639,
		Timezone:  "America/New_York",
	},
	"8PA8": {
		ICAO:      "8PA8",
		Name:      "Sunny Rest Airport",
		City:      "Palmerton",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 980,
		Latitude:  40.8167991638,
		Longitude: -75.6662979126,
		Timezone:  "America/New_York",
	},
	"8PN0": {
		ICAO:      "8PN0",
		Name:      "Lost Acres Airport",
		City:      "Chambersburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 730,
		Latitude:  39.9383010864,
		Longitude: -77.6164016724,
		Timezone:  "America/New_York",
	},
	"8PN2": {
		ICAO:      "8PN2",
		Name:      "Hallett's Airport",
		City:      "Bangor",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 700,
		Latitude:  40.9104995728,
		Longitude: -75.1651000977,
		Timezone:  "America/New_York",
	},
	"8PN3": {
		ICAO:      "8PN3",
		Name:      "Mc Cauley'S Airport",
		City:      "Venus",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1571,
		Latitude:  41.383333,
		Longitude: -79.5,
		Timezone:  "America/New_York",
	},
	"8PN9": {
		ICAO:      "8PN9",
		Name:      "Marsh Creek Airport",
		City:      "Gettysburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 530,
		Latitude:  39.8219985962,
		Longitude: -77.2919006348,
		Timezone:  "America/New_York",
	},
	"8PS0": {
		ICAO:      "8PS0",
		Name:      "Middlebury Airport",
		City:      "Middlebury Center",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1170,
		Latitude:  41.8479003906,
		Longitude: -77.2814025879,
		Timezone:  "America/New_York",
	},
	"8PS2": {
		ICAO:      "8PS2",
		Name:      "Still Meadow Farm Airport",
		City:      "Jackson Center",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1370,
		Latitude:  41.3106002808,
		Longitude: -80.1628036499,
		Timezone:  "America/New_York",
	},
	"8TA0": {
		ICAO:      "8TA0",
		Name:      "John B Connally Ranch Airport",
		City:      "Floresville",
		State:     "Texas",
		Country:   "US",
		Elevation: 530,
		Latitude:  29.135799408,
		Longitude: -98.2763977051,
		Timezone:  "America/Chicago",
	},
	"8TA1": {
		ICAO:      "8TA1",
		Name:      "Whatley Flying Service Airport",
		City:      "Seadrift",
		State:     "Texas",
		Country:   "US",
		Elevation: 20,
		Latitude:  28.478099823,
		Longitude: -96.7619018555,
		Timezone:  "America/Chicago",
	},
	"8TA3": {
		ICAO:      "8TA3",
		Name:      "Flying X River Ranch Airport",
		City:      "Spicewood",
		State:     "Texas",
		Country:   "US",
		Elevation: 820,
		Latitude:  30.5168991089,
		Longitude: -98.1742019653,
		Timezone:  "America/Chicago",
	},
	"8TA4": {
		ICAO:      "8TA4",
		Name:      "Laseair Airport",
		City:      "Texas City",
		State:     "Texas",
		Country:   "US",
		Elevation: 12,
		Latitude:  29.4454994202,
		Longitude: -95.0076980591,
		Timezone:  "America/Chicago",
	},
	"8TA5": {
		ICAO:      "8TA5",
		Name:      "Short Stop Airport",
		City:      "Farmersville",
		State:     "Texas",
		Country:   "US",
		Elevation: 645,
		Latitude:  33.1632003784,
		Longitude: -96.3221969604,
		Timezone:  "America/Chicago",
	},
	"8TA7": {
		ICAO:      "8TA7",
		Name:      "Stark Field",
		City:      "Granbury",
		State:     "Texas",
		Country:   "US",
		Elevation: 900,
		Latitude:  32.5695991516,
		Longitude: -97.7530975342,
		Timezone:  "America/Chicago",
	},
	"8TA8": {
		ICAO:      "8TA8",
		Name:      "Bufords Field",
		City:      "Huntington",
		State:     "Texas",
		Country:   "US",
		Elevation: 200,
		Latitude:  31.1690998077,
		Longitude: -94.5324020386,
		Timezone:  "America/Chicago",
	},
	"8TA9": {
		ICAO:      "8TA9",
		Name:      "Star Dusters Airport",
		City:      "Washington",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 37,
		Latitude:  30.635799408,
		Longitude: -92.0584030151,
		Timezone:  "America/Chicago",
	},
	"8TE0": {
		ICAO:      "8TE0",
		Name:      "Gillingham Airport",
		City:      "Floresville",
		State:     "Texas",
		Country:   "US",
		Elevation: 450,
		Latitude:  29.1390991211,
		Longitude: -98.1132965088,
		Timezone:  "America/Chicago",
	},
	"8TE1": {
		ICAO:      "8TE1",
		Name:      "Wall Flying Service Airport",
		City:      "Floresville",
		State:     "Texas",
		Country:   "US",
		Elevation: 440,
		Latitude:  29.017999649,
		Longitude: -98.206703186,
		Timezone:  "America/Chicago",
	},
	"8TE2": {
		ICAO:      "8TE2",
		Name:      "J-Bar Ranch Airport",
		City:      "Crane",
		State:     "Texas",
		Country:   "US",
		Elevation: 2667,
		Latitude:  31.5279006958,
		Longitude: -102.5279998779,
		Timezone:  "America/Chicago",
	},
	"8TE3": {
		ICAO:      "8TE3",
		Name:      "Lopez Ranch Airport",
		City:      "Fort Mc Kavett",
		State:     "Texas",
		Country:   "US",
		Elevation: 2318,
		Latitude:  30.7182006836,
		Longitude: -100.0810012817,
		Timezone:  "America/Chicago",
	},
	"8TE4": {
		ICAO:      "8TE4",
		Name:      "H & F Properties Airport",
		City:      "Crystal City",
		State:     "Texas",
		Country:   "US",
		Elevation: 640,
		Latitude:  28.8150005341,
		Longitude: -99.758102417,
		Timezone:  "America/Chicago",
	},
	"8TE6": {
		ICAO:      "8TE6",
		Name:      "Faith Cattle Company Longfellow Ranch Airport",
		City:      "Fort Stockton",
		State:     "Texas",
		Country:   "US",
		Elevation: 3900,
		Latitude:  30.3001995087,
		Longitude: -102.717002869,
		Timezone:  "America/Chicago",
	},
	"8TE7": {
		ICAO:      "8TE7",
		Name:      "Carter Ranch STOLport",
		City:      "Fredericksburg",
		State:     "Texas",
		Country:   "US",
		Elevation: 1840,
		Latitude:  30.3208007813,
		Longitude: -98.9128036499,
		Timezone:  "America/Chicago",
	},
	"8TE8": {
		ICAO:      "8TE8",
		Name:      "Tradewind Agricultural Airport",
		City:      "El Campo",
		State:     "Texas",
		Country:   "US",
		Elevation: 95,
		Latitude:  29.1627998352,
		Longitude: -96.2251968384,
		Timezone:  "America/Chicago",
	},
	"8TN1": {
		ICAO:      "8TN1",
		Name:      "Parker Airport",
		City:      "Eads",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 375,
		Latitude:  35.1963996887,
		Longitude: -89.6268997192,
		Timezone:  "America/Chicago",
	},
	"8TN2": {
		ICAO:      "8TN2",
		Name:      "Pleasant Grove Airpark",
		City:      "Shelbyville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 850,
		Latitude:  35.3988990784,
		Longitude: -86.5539016724,
		Timezone:  "America/Chicago",
	},
	"8TN3": {
		ICAO:      "8TN3",
		Name:      "Raby Airpark",
		City:      "Farragut",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 830,
		Latitude:  35.8417015076,
		Longitude: -84.181098938,
		Timezone:  "America/New_York",
	},
	"8TN4": {
		ICAO:      "8TN4",
		Name:      "Flaglor Airport",
		City:      "Mosheim",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1135,
		Latitude:  36.2447013855,
		Longitude: -82.9689025879,
		Timezone:  "America/New_York",
	},
	"8TN5": {
		ICAO:      "8TN5",
		Name:      "Nobuzzn Airport",
		City:      "Springfield",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 650,
		Latitude:  36.4864006042,
		Longitude: -86.9111022949,
		Timezone:  "America/Chicago",
	},
	"8TN6": {
		ICAO:      "8TN6",
		Name:      "Baker Field",
		City:      "Murfreesboro",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 560,
		Latitude:  35.9211006165,
		Longitude: -86.3447036743,
		Timezone:  "America/Chicago",
	},
	"8TN7": {
		ICAO:      "8TN7",
		Name:      "Short Creek Airport",
		City:      "Dover",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 390,
		Latitude:  36.4007987976,
		Longitude: -87.9916992188,
		Timezone:  "America/Chicago",
	},
	"8TN8": {
		ICAO:      "8TN8",
		Name:      "Field of Dreams Ultralightport",
		City:      "Burlison",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 250,
		Latitude:  35.5416984558,
		Longitude: -89.8713989258,
		Timezone:  "America/Chicago",
	},
	"8TN9": {
		ICAO:      "8TN9",
		Name:      "Bull Run Airport",
		City:      "Covington",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 376,
		Latitude:  35.5155982971,
		Longitude: -89.6778030396,
		Timezone:  "America/Chicago",
	},
	"8TS0": {
		ICAO:      "8TS0",
		Name:      "Hamilton Ranch Airport",
		City:      "Bruni",
		State:     "Texas",
		Country:   "US",
		Elevation: 698,
		Latitude:  27.5531005859,
		Longitude: -98.7322006226,
		Timezone:  "America/Chicago",
	},
	"8TS1": {
		ICAO:      "8TS1",
		Name:      "Retta Airport",
		City:      "Burleson",
		State:     "Texas",
		Country:   "US",
		Elevation: 700,
		Latitude:  32.5418014526,
		Longitude: -97.2419967651,
		Timezone:  "America/Chicago",
	},
	"8TS2": {
		ICAO:      "8TS2",
		Name:      "Henrietta Airport",
		City:      "Henrietta",
		State:     "Texas",
		Country:   "US",
		Elevation: 932,
		Latitude:  33.7834014893,
		Longitude: -98.2170028687,
		Timezone:  "America/Chicago",
	},
	"8TS3": {
		ICAO:      "8TS3",
		Name:      "Bruner Airport",
		City:      "Latexo",
		State:     "Texas",
		Country:   "US",
		Elevation: 387,
		Latitude:  31.3901996613,
		Longitude: -95.4732971191,
		Timezone:  "America/Chicago",
	},
	"8TS5": {
		ICAO:      "8TS5",
		Name:      "Stol Field",
		City:      "Joshua",
		State:     "Texas",
		Country:   "US",
		Elevation: 908,
		Latitude:  32.4678993225,
		Longitude: -97.3683013916,
		Timezone:  "America/Chicago",
	},
	"8TS6": {
		ICAO:      "8TS6",
		Name:      "Moore Airport",
		City:      "Kaufman",
		State:     "Texas",
		Country:   "US",
		Elevation: 418,
		Latitude:  32.5331993103,
		Longitude: -96.3225021362,
		Timezone:  "America/Chicago",
	},
	"8TS7": {
		ICAO:      "8TS7",
		Name:      "Wyatt 3-Rivers Airport",
		City:      "Glen Rose",
		State:     "Texas",
		Country:   "US",
		Elevation: 615,
		Latitude:  32.2448997498,
		Longitude: -97.7260971069,
		Timezone:  "America/Chicago",
	},
	"8TS8": {
		ICAO:      "8TS8",
		Name:      "Rnk Ranch Airport",
		City:      "Del Rio",
		State:     "Texas",
		Country:   "US",
		Elevation: 1945,
		Latitude:  29.9174003601,
		Longitude: -100.9130020142,
		Timezone:  "America/Chicago",
	},
	"8TS9": {
		ICAO:      "8TS9",
		Name:      "Strait Ranch Airport",
		City:      "Encinal",
		State:     "Texas",
		Country:   "US",
		Elevation: 620,
		Latitude:  28.1399993896,
		Longitude: -99.5702972412,
		Timezone:  "America/Chicago",
	},
	"8TX0": {
		ICAO:      "8TX0",
		Name:      "Hub Field",
		City:      "Jewett",
		State:     "Texas",
		Country:   "US",
		Elevation: 400,
		Latitude:  31.429599762,
		Longitude: -96.1302032471,
		Timezone:  "America/Chicago",
	},
	"8TX2": {
		ICAO:      "8TX2",
		Name:      "Freeman Ranch Airport",
		City:      "Rocksprings",
		State:     "Texas",
		Country:   "US",
		Elevation: 2380,
		Latitude:  29.9785003662,
		Longitude: -100.202003479,
		Timezone:  "America/Chicago",
	},
	"8TX3": {
		ICAO:      "8TX3",
		Name:      "Edwards Airport",
		City:      "Goldthwaite",
		State:     "Texas",
		Country:   "US",
		Elevation: 1300,
		Latitude:  31.3428993225,
		Longitude: -98.6156005859,
		Timezone:  "America/Chicago",
	},
	"8TX6": {
		ICAO:      "8TX6",
		Name:      "Harper Airport",
		City:      "Palmer",
		State:     "Texas",
		Country:   "US",
		Elevation: 510,
		Latitude:  32.4276008606,
		Longitude: -96.7169036865,
		Timezone:  "America/Chicago",
	},
	"8TX7": {
		ICAO:      "8TX7",
		Name:      "Skyhaven Airport",
		City:      "Houston",
		State:     "Texas",
		Country:   "US",
		Elevation: 40,
		Latitude:  29.8335990906,
		Longitude: -95.1484985352,
		Timezone:  "America/Chicago",
	},
	"8TX8": {
		ICAO:      "8TX8",
		Name:      "Weeks Airport",
		City:      "Premont",
		State:     "Texas",
		Country:   "US",
		Elevation: 190,
		Latitude:  27.3609008789,
		Longitude: -98.1794967651,
		Timezone:  "America/Chicago",
	},
	"8VA1": {
		ICAO:      "8VA1",
		Name:      "Grand Pre Farm Airport",
		City:      "Brightwood",
		State:     "Virginia",
		Country:   "US",
		Elevation: 500,
		Latitude:  38.4542999268,
		Longitude: -78.1864013672,
		Timezone:  "America/New_York",
	},
	"8VA2": {
		ICAO:      "8VA2",
		Name:      "Shivok Airport",
		City:      "Callao",
		State:     "Virginia",
		Country:   "US",
		Elevation: 22,
		Latitude:  38.0089988708,
		Longitude: -76.56300354,
		Timezone:  "America/New_York",
	},
	"8VA4": {
		ICAO:      "8VA4",
		Name:      "Lotus International Airport",
		City:      "Buckinham",
		State:     "Virginia",
		Country:   "US",
		Elevation: 340,
		Latitude:  37.6729011536,
		Longitude: -78.7033004761,
		Timezone:  "America/New_York",
	},
	"8VA6": {
		ICAO:      "8VA6",
		Name:      "Flatwoods Airport",
		City:      "Lovingston",
		State:     "Virginia",
		Country:   "US",
		Elevation: 840,
		Latitude:  37.7382011414,
		Longitude: -78.9621963501,
		Timezone:  "America/New_York",
	},
	"8WA0": {
		ICAO:      "8WA0",
		Name:      "Flying B Airport",
		City:      "Rainier",
		State:     "Washington",
		Country:   "US",
		Elevation: 445,
		Latitude:  46.8776016235,
		Longitude: -122.6009979248,
		Timezone:  "America/Los_Angeles",
	},
	"8WA5": {
		ICAO:      "8WA5",
		Name:      "Tree Heart Ranch Airport",
		City:      "Ritzville",
		State:     "Washington",
		Country:   "US",
		Elevation: 1420,
		Latitude:  47.1414985657,
		Longitude: -118.8010025024,
		Timezone:  "America/Los_Angeles",
	},
	"8WA6": {
		ICAO:      "8WA6",
		Name:      "Christensen Field",
		City:      "Royal City",
		State:     "Washington",
		Country:   "US",
		Elevation: 1150,
		Latitude:  46.9207000732,
		Longitude: -119.5899963379,
		Timezone:  "America/Los_Angeles",
	},
	"8WA7": {
		ICAO:      "8WA7",
		Name:      "Gossard Field",
		City:      "St John",
		State:     "Washington",
		Country:   "US",
		Elevation: 1973,
		Latitude:  47.1152000427,
		Longitude: -117.5739974976,
		Timezone:  "America/Los_Angeles",
	},
	"8WI0": {
		ICAO:      "8WI0",
		Name:      "Wood Airport",
		City:      "Wisconsin Dells",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 935,
		Latitude:  43.7439002991,
		Longitude: -89.8143005371,
		Timezone:  "America/Chicago",
	},
	"8WI1": {
		ICAO:      "8WI1",
		Name:      "Dillenburg's Airport",
		City:      "Shawano",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 915,
		Latitude:  44.7536010742,
		Longitude: -88.7282028198,
		Timezone:  "America/Chicago",
	},
	"8WI2": {
		ICAO:      "8WI2",
		Name:      "Runway Leasing Inc Nr 1 Airport",
		City:      "Plainfield",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1100,
		Latitude:  44.2247009277,
		Longitude: -89.53099823,
		Timezone:  "America/Chicago",
	},
	"8WI3": {
		ICAO:      "8WI3",
		Name:      "Rwnway Leasing Inc Nr 2 Airport",
		City:      "Plover",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1070,
		Latitude:  44.4207992554,
		Longitude: -89.5553970337,
		Timezone:  "America/Chicago",
	},
	"8WI5": {
		ICAO:      "8WI5",
		Name:      "Wolf River Landing Strip",
		City:      "Black Creek",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 670,
		Latitude:  44.515496,
		Longitude: -88.54164,
		Timezone:  "America/Chicago",
	},
	"8WI6": {
		ICAO:      "8WI6",
		Name:      "Funk Aerodrome",
		City:      "Luxemburg",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 831,
		Latitude:  44.6096992493,
		Longitude: -87.6529006958,
		Timezone:  "America/Chicago",
	},
	"8WI7": {
		ICAO:      "8WI7",
		Name:      "Dunbar Airport",
		City:      "Dunbar",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1196,
		Latitude:  45.6511001587,
		Longitude: -88.1820983887,
		Timezone:  "America/Chicago",
	},
	"8WI8": {
		ICAO:      "8WI8",
		Name:      "Stupek Farms Airport",
		City:      "Fennimore",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1210,
		Latitude:  42.9766998291,
		Longitude: -90.6482009888,
		Timezone:  "America/Chicago",
	},
	"8WN8": {
		ICAO:      "8WN8",
		Name:      "Mave's Lakeview Road Airport",
		City:      "Ellison Bay",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 700,
		Latitude:  45.2350997925,
		Longitude: -87.0674972534,
		Timezone:  "America/Chicago",
	},
	"8XS0": {
		ICAO:      "8XS0",
		Name:      "Tivydale Ranch Airport",
		City:      "Fredericksburg",
		State:     "Texas",
		Country:   "US",
		Elevation: 1910,
		Latitude:  30.2544002533,
		Longitude: -99.0997009277,
		Timezone:  "America/Chicago",
	},
	"8XS1": {
		ICAO:      "8XS1",
		Name:      "Harold Freeman Farm Airport",
		City:      "Katy",
		State:     "Texas",
		Country:   "US",
		Elevation: 160,
		Latitude:  29.8826999664,
		Longitude: -95.8118972778,
		Timezone:  "America/Chicago",
	},
	"8XS2": {
		ICAO:      "8XS2",
		Name:      "Rachal Airport",
		City:      "Laredo",
		State:     "Texas",
		Country:   "US",
		Elevation: 600,
		Latitude:  27.9297008514,
		Longitude: -99.8511962891,
		Timezone:  "America/Chicago",
	},
	"8XS3": {
		ICAO:      "8XS3",
		Name:      "Pegasus Place Airstrip",
		City:      "Leander",
		State:     "Texas",
		Country:   "US",
		Elevation: 965,
		Latitude:  30.5669002533,
		Longitude: -97.829498291,
		Timezone:  "America/Chicago",
	},
	"8XS6": {
		ICAO:      "8XS6",
		Name:      "Mc Manus Field",
		City:      "Tioga",
		State:     "Texas",
		Country:   "US",
		Elevation: 695,
		Latitude:  33.4522018433,
		Longitude: -96.8328018188,
		Timezone:  "America/Chicago",
	},
	"8XS8": {
		ICAO:      "8XS8",
		Name:      "Reese Airpark",
		City:      "Lubbock",
		State:     "Texas",
		Country:   "US",
		Elevation: 3338,
		Latitude:  33.5903015137,
		Longitude: -102.0370025635,
		Timezone:  "America/Chicago",
	},
	"8XS9": {
		ICAO:      "8XS9",
		Name:      "Rust Field",
		City:      "Waring",
		State:     "Texas",
		Country:   "US",
		Elevation: 1331,
		Latitude:  29.9571990967,
		Longitude: -98.7906036377,
		Timezone:  "America/Chicago",
	},
	"90AR": {
		ICAO:      "90AR",
		Name:      "Reedville Airport",
		City:      "Dumas",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 162,
		Latitude:  33.9250984192,
		Longitude: -91.5111999512,
		Timezone:  "America/Chicago",
	},
	"90CA": {
		ICAO:      "90CA",
		Name:      "Fowler's Airport",
		City:      "Linden",
		State:     "California",
		Country:   "US",
		Elevation: 77,
		Latitude:  38.0035018921,
		Longitude: -121.1190032959,
		Timezone:  "America/Los_Angeles",
	},
	"90CL": {
		ICAO:      "90CL",
		Name:      "Diamond M Ranch Airport",
		City:      "Elk Creek",
		State:     "California",
		Country:   "US",
		Elevation: 1296,
		Latitude:  39.572101593,
		Longitude: -122.6090011597,
		Timezone:  "America/Los_Angeles",
	},
	"90FD": {
		ICAO:      "90FD",
		Name:      "Blue Head Ranch Airport",
		City:      "Lake Placid",
		State:     "Florida",
		Country:   "US",
		Elevation: 77,
		Latitude:  27.1625003815,
		Longitude: -81.5429000854,
		Timezone:  "America/New_York",
	},
	"90FL": {
		ICAO:      "90FL",
		Name:      "Paxton Airport",
		City:      "Okeechobee",
		State:     "Florida",
		Country:   "US",
		Elevation: 66,
		Latitude:  27.3612003326,
		Longitude: -80.7609024048,
		Timezone:  "America/New_York",
	},
	"90IA": {
		ICAO:      "90IA",
		Name:      "Missouri Valley Airport",
		City:      "Missouri Valley",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1000,
		Latitude:  41.5402984619,
		Longitude: -95.887802124,
		Timezone:  "America/Chicago",
	},
	"90IL": {
		ICAO:      "90IL",
		Name:      "Johnson Airport",
		City:      "Momence",
		State:     "Illinois",
		Country:   "US",
		Elevation: 630,
		Latitude:  41.1981010437,
		Longitude: -87.5588989258,
		Timezone:  "America/Chicago",
	},
	"90IN": {
		ICAO:      "90IN",
		Name:      "Mc Daniel's Field",
		City:      "Martinsville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 600,
		Latitude:  39.4089012146,
		Longitude: -86.4441986084,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"90KS": {
		ICAO:      "90KS",
		Name:      "Sunset Strip Airpark",
		City:      "Topeka",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1075,
		Latitude:  38.9824981689,
		Longitude: -95.5950012207,
		Timezone:  "America/Chicago",
	},
	"90KY": {
		ICAO:      "90KY",
		Name:      "Williams Airport",
		City:      "Hodgenville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 760,
		Latitude:  37.5825996399,
		Longitude: -85.7332992554,
		Timezone:  "America/New_York",
	},
	"90LA": {
		ICAO:      "90LA",
		Name:      "Sharp Field",
		City:      "Minden",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 165,
		Latitude:  32.613899231,
		Longitude: -93.323600769,
		Timezone:  "America/Chicago",
	},
	"90ME": {
		ICAO:      "90ME",
		Name:      "Zinck Airport",
		City:      "Rumford",
		State:     "Maine",
		Country:   "US",
		Elevation: 620,
		Latitude:  44.523056,
		Longitude: -70.545556,
		Timezone:  "America/New_York",
	},
	"90MN": {
		ICAO:      "90MN",
		Name:      "Schmidt Private Airport",
		City:      "Vesta",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1060,
		Latitude:  44.5032997131,
		Longitude: -95.4539031982,
		Timezone:  "America/Chicago",
	},
	"90NY": {
		ICAO:      "90NY",
		Name:      "Hopewell Airpark",
		City:      "Canandaigua",
		State:     "New York",
		Country:   "US",
		Elevation: 730,
		Latitude:  42.919508,
		Longitude: -77.242778,
		Timezone:  "America/New_York",
	},
	"90OI": {
		ICAO:      "90OI",
		Name:      "Joe Cimprich Airport",
		City:      "Camden",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1010,
		Latitude:  39.6041984558,
		Longitude: -84.6350021362,
		Timezone:  "America/New_York",
	},
	"90OK": {
		ICAO:      "90OK",
		Name:      "Stewart Farms Airport",
		City:      "Aline",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1460,
		Latitude:  36.5005989075,
		Longitude: -98.5019989014,
		Timezone:  "America/Chicago",
	},
	"90PA": {
		ICAO:      "90PA",
		Name:      "Adams Airport",
		City:      "Coneville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1780,
		Latitude:  41.8791999817,
		Longitude: -78.1149978638,
		Timezone:  "America/New_York",
	},
	"90PN": {
		ICAO:      "90PN",
		Name:      "Baney's Airport",
		City:      "Dover",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 470,
		Latitude:  40.0172996521,
		Longitude: -76.8197021484,
		Timezone:  "America/New_York",
	},
	"90TE": {
		ICAO:      "90TE",
		Name:      "The Homestead Airport",
		City:      "Corsicana",
		State:     "Texas",
		Country:   "US",
		Elevation: 398,
		Latitude:  32.1957015991,
		Longitude: -96.4385986328,
		Timezone:  "America/Chicago",
	},
	"90TS": {
		ICAO:      "90TS",
		Name:      "Jordan Ranch Airport",
		City:      "Bedias",
		State:     "Texas",
		Country:   "US",
		Elevation: 265,
		Latitude:  30.7791996002,
		Longitude: -95.797203064,
		Timezone:  "America/Chicago",
	},
	"90TX": {
		ICAO:      "90TX",
		Name:      "Callaghan Ranch Airport",
		City:      "Callaghan",
		State:     "Texas",
		Country:   "US",
		Elevation: 635,
		Latitude:  27.8794002533,
		Longitude: -99.396697998,
		Timezone:  "America/Chicago",
	},
	"90VA": {
		ICAO:      "90VA",
		Name:      "Hen & Bacon Airport",
		City:      "Orange",
		State:     "Virginia",
		Country:   "US",
		Elevation: 430,
		Latitude:  38.2181015015,
		Longitude: -78.2118988037,
		Timezone:  "America/New_York",
	},
	"90WA": {
		ICAO:      "90WA",
		Name:      "Waldronaire Airport",
		City:      "East Sound",
		State:     "Washington",
		Country:   "US",
		Elevation: 140,
		Latitude:  48.7117996216,
		Longitude: -123.0179977417,
		Timezone:  "America/Los_Angeles",
	},
	"90WI": {
		ICAO:      "90WI",
		Name:      "Heritage Acres Airport",
		City:      "Laona",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1608,
		Latitude:  45.5750007629,
		Longitude: -88.7083969116,
		Timezone:  "America/Chicago",
	},
	"91CL": {
		ICAO:      "91CL",
		Name:      "Sacatar Meadows Airport",
		City:      "Kennedy Meadows",
		State:     "California",
		Country:   "US",
		Elevation: 6427,
		Latitude:  35.9898986816,
		Longitude: -118.0800018311,
		Timezone:  "America/Los_Angeles",
	},
	"91GA": {
		ICAO:      "91GA",
		Name:      "Dbaks Airport",
		City:      "Moreland",
		State:     "Georgia",
		Country:   "US",
		Elevation: 910,
		Latitude:  33.273601532,
		Longitude: -84.793296814,
		Timezone:  "America/New_York",
	},
	"91IN": {
		ICAO:      "91IN",
		Name:      "Strietelmeier Flying Field",
		City:      "Columbus",
		State:     "Indiana",
		Country:   "US",
		Elevation: 625,
		Latitude:  39.2080993652,
		Longitude: -85.9747009277,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"91IS": {
		ICAO:      "91IS",
		Name:      "Stanton Airport",
		City:      "Riverton",
		State:     "Illinois",
		Country:   "US",
		Elevation: 580,
		Latitude:  39.8236999512,
		Longitude: -89.5039978027,
		Timezone:  "America/Chicago",
	},
	"91KS": {
		ICAO:      "91KS",
		Name:      "St. Joseph'S Landing",
		City:      "St. Mary'S",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1136,
		Latitude:  39.243803,
		Longitude: -96.033039,
		Timezone:  "America/Chicago",
	},
	"91LA": {
		ICAO:      "91LA",
		Name:      "Koch Airport",
		City:      "Eunice",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 51,
		Latitude:  30.5345993042,
		Longitude: -92.367401123,
		Timezone:  "America/Chicago",
	},
	"91LS": {
		ICAO:      "91LS",
		Name:      "Duclos RLA Restricted Landing Area",
		City:      "Red Bud",
		State:     "Illinois",
		Country:   "US",
		Elevation: 520,
		Latitude:  38.2089004517,
		Longitude: -90.0372009277,
		Timezone:  "America/Chicago",
	},
	"91NY": {
		ICAO:      "91NY",
		Name:      "Manitou Field",
		City:      "Spencerport",
		State:     "New York",
		Country:   "US",
		Elevation: 470,
		Latitude:  43.1955986023,
		Longitude: -77.7481002808,
		Timezone:  "America/New_York",
	},
	"91OH": {
		ICAO:      "91OH",
		Name:      "Stine Field",
		City:      "Wooster",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1172,
		Latitude:  40.7333984375,
		Longitude: -81.8928985596,
		Timezone:  "America/New_York",
	},
	"91OI": {
		ICAO:      "91OI",
		Name:      "Cross Airport",
		City:      "Canton",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1125,
		Latitude:  40.7773017883,
		Longitude: -81.4587020874,
		Timezone:  "America/New_York",
	},
	"91OK": {
		ICAO:      "91OK",
		Name:      "Stewart Airport",
		City:      "Altus",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1365,
		Latitude:  34.641998291,
		Longitude: -99.3712005615,
		Timezone:  "America/Chicago",
	},
	"91OR": {
		ICAO:      "91OR",
		Name:      "Abba's Airport",
		City:      "Sheridan",
		State:     "Oregon",
		Country:   "US",
		Elevation: 215,
		Latitude:  45.1026000977,
		Longitude: -123.418998718,
		Timezone:  "America/Los_Angeles",
	},
	"91TA": {
		ICAO:      "91TA",
		Name:      "Rhines Roost Airport",
		City:      "Canton",
		State:     "Texas",
		Country:   "US",
		Elevation: 540,
		Latitude:  32.4459991455,
		Longitude: -95.9188995361,
		Timezone:  "America/Chicago",
	},
	"91TE": {
		ICAO:      "91TE",
		Name:      "Britts Crosswind Airport",
		City:      "Liberty Hill",
		State:     "Texas",
		Country:   "US",
		Elevation: 1000,
		Latitude:  30.7247009277,
		Longitude: -97.8619995117,
		Timezone:  "America/Chicago",
	},
	"91TS": {
		ICAO:      "91TS",
		Name:      "Songbird Ranch Airport",
		City:      "Rosharon",
		State:     "Texas",
		Country:   "US",
		Elevation: 34,
		Latitude:  29.3686008453,
		Longitude: -95.3398971558,
		Timezone:  "America/Chicago",
	},
	"91TX": {
		ICAO:      "91TX",
		Name:      "Paisano Ranch Airport",
		City:      "Calliham",
		State:     "Texas",
		Country:   "US",
		Elevation: 235,
		Latitude:  28.396900177,
		Longitude: -98.3628005981,
		Timezone:  "America/Chicago",
	},
	"91VA": {
		ICAO:      "91VA",
		Name:      "Miller Airport",
		City:      "Bedford",
		State:     "Virginia",
		Country:   "US",
		Elevation: 760,
		Latitude:  37.3275985718,
		Longitude: -79.404800415,
		Timezone:  "America/New_York",
	},
	"91WA": {
		ICAO:      "91WA",
		Name:      "Christensen Bros Wahluke Strip",
		City:      "Mattawa",
		State:     "Washington",
		Country:   "US",
		Elevation: 840,
		Latitude:  46.7081985474,
		Longitude: -119.8010025024,
		Timezone:  "America/Los_Angeles",
	},
	"91WI": {
		ICAO:      "91WI",
		Name:      "Gottschalk Field",
		City:      "Nekoosa",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 976,
		Latitude:  44.2938995361,
		Longitude: -90.0320968628,
		Timezone:  "America/Chicago",
	},
	"92CA": {
		ICAO:      "92CA",
		Name:      "Westlake Farms Airport",
		City:      "Stratford",
		State:     "California",
		Country:   "US",
		Elevation: 192,
		Latitude:  36.1208000183,
		Longitude: -119.8880004883,
		Timezone:  "America/Los_Angeles",
	},
	"92CL": {
		ICAO:      "92CL",
		Name:      "Moronis Airport",
		City:      "Meridian",
		State:     "California",
		Country:   "US",
		Elevation: 35,
		Latitude:  39.1031990051,
		Longitude: -121.8499984741,
		Timezone:  "America/Los_Angeles",
	},
	"92CO": {
		ICAO:      "92CO",
		Name:      "Lake Creek Ranch Airport",
		City:      "Hillside",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7200,
		Latitude:  38.2916984558,
		Longitude: -105.611000061,
		Timezone:  "America/Denver",
	},
	"92FL": {
		ICAO:      "92FL",
		Name:      "Carlstrom Field",
		City:      "Arcadia",
		State:     "Florida",
		Country:   "US",
		Elevation: 35,
		Latitude:  27.1205997467,
		Longitude: -81.8498001099,
		Timezone:  "America/New_York",
	},
	"92GE": {
		ICAO:      "92GE",
		Name:      "Midville International Airport",
		City:      "Midville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 265,
		Latitude:  32.8452987671,
		Longitude: -82.2707977295,
		Timezone:  "America/New_York",
	},
	"92IL": {
		ICAO:      "92IL",
		Name:      "Hunter Raffety Elevators Inc Airport",
		City:      "Cairo",
		State:     "Illinois",
		Country:   "US",
		Elevation: 323,
		Latitude:  37.038898468,
		Longitude: -89.3208999634,
		Timezone:  "America/Chicago",
	},
	"92IN": {
		ICAO:      "92IN",
		Name:      "H R Weisser Airport",
		City:      "Milford",
		State:     "Indiana",
		Country:   "US",
		Elevation: 832,
		Latitude:  41.4234008789,
		Longitude: -85.8475036621,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"92IS": {
		ICAO:      "92IS",
		Name:      "Taft Airport",
		City:      "Rochester",
		State:     "Illinois",
		Country:   "US",
		Elevation: 586,
		Latitude:  39.7420005798,
		Longitude: -89.4784011841,
		Timezone:  "America/Chicago",
	},
	"92KY": {
		ICAO:      "92KY",
		Name:      "Timmons Field",
		City:      "Pewee Valley",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 690,
		Latitude:  38.284198761,
		Longitude: -85.4726028442,
		Timezone:  "America/Kentucky/Louisville",
	},
	"92MI": {
		ICAO:      "92MI",
		Name:      "Zischke Airport",
		City:      "Dewitt",
		State:     "Michigan",
		Country:   "US",
		Elevation: 863,
		Latitude:  42.861667,
		Longitude: -84.619167,
		Timezone:  "America/Detroit",
	},
	"92MN": {
		ICAO:      "92MN",
		Name:      "Brutlag Farms Airport",
		City:      "Wendell",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1127,
		Latitude:  46.0144004822,
		Longitude: -96.1016998291,
		Timezone:  "America/Chicago",
	},
	"92NJ": {
		ICAO:      "92NJ",
		Name:      "Fly-N-D Landing Strip",
		City:      "Kingwood Township",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 530,
		Latitude:  40.4683990479,
		Longitude: -75.0024032593,
		Timezone:  "America/New_York",
	},
	"92NY": {
		ICAO:      "92NY",
		Name:      "Bloecher Farm Airport",
		City:      "Strykersville",
		State:     "New York",
		Country:   "US",
		Elevation: 1430,
		Latitude:  42.7519989014,
		Longitude: -78.4614028931,
		Timezone:  "America/New_York",
	},
	"92OK": {
		ICAO:      "92OK",
		Name:      "Flying W Ranch Airport",
		City:      "Atoka",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 618,
		Latitude:  34.382900238,
		Longitude: -96.2919006348,
		Timezone:  "America/Chicago",
	},
	"92PA": {
		ICAO:      "92PA",
		Name:      "Hawkins Field",
		City:      "Cooperstown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1150,
		Latitude:  40.7042007446,
		Longitude: -79.952003479,
		Timezone:  "America/New_York",
	},
	"92TA": {
		ICAO:      "92TA",
		Name:      "Drennan Farm Airport",
		City:      "Los Fresnos",
		State:     "Texas",
		Country:   "US",
		Elevation: 27,
		Latitude:  26.1212005615,
		Longitude: -97.4280014038,
		Timezone:  "America/Chicago",
	},
	"92TE": {
		ICAO:      "92TE",
		Name:      "Chaney San Francisco Ranch Airport",
		City:      "Marathon",
		State:     "Texas",
		Country:   "US",
		Elevation: 3250,
		Latitude:  29.9668998718,
		Longitude: -102.9339981079,
		Timezone:  "America/Chicago",
	},
	"92VA": {
		ICAO:      "92VA",
		Name:      "New Quarter Farm Airport",
		City:      "Gloucester",
		State:     "Virginia",
		Country:   "US",
		Elevation: 38,
		Latitude:  37.3446998596,
		Longitude: -76.5643997192,
		Timezone:  "America/New_York",
	},
	"92WI": {
		ICAO:      "92WI",
		Name:      "Knight Aire Airport",
		City:      "Merrill",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1605,
		Latitude:  45.2961006165,
		Longitude: -89.6410980225,
		Timezone:  "America/Chicago",
	},
	"92XS": {
		ICAO:      "92XS",
		Name:      "T.R. Funk Inc Airport",
		City:      "Raymondville",
		State:     "Texas",
		Country:   "US",
		Elevation: 24,
		Latitude:  26.4790000916,
		Longitude: -97.6872024536,
		Timezone:  "America/Chicago",
	},
	"93AK": {
		ICAO:      "93AK",
		Name:      "The Queens Airport",
		City:      "Queens",
		State:     "Alaska",
		Country:   "US",
		Elevation: 25,
		Latitude:  58.8720016479,
		Longitude: -158.4720001221,
		Timezone:  "America/Anchorage",
	},
	"93AR": {
		ICAO:      "93AR",
		Name:      "Ohlendorf Airport",
		City:      "Osceola",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 232,
		Latitude:  35.6208992004,
		Longitude: -89.9861984253,
		Timezone:  "America/Chicago",
	},
	"93CO": {
		ICAO:      "93CO",
		Name:      "Antelope Airpark",
		City:      "Lake George",
		State:     "Colorado",
		Country:   "US",
		Elevation: 8700,
		Latitude:  39.0233192444,
		Longitude: -105.614997864,
		Timezone:  "America/Denver",
	},
	"93FD": {
		ICAO:      "93FD",
		Name:      "Chumuckla 20-20 Airport",
		City:      "Milton",
		State:     "Florida",
		Country:   "US",
		Elevation: 205,
		Latitude:  30.7546005249,
		Longitude: -87.1774978638,
		Timezone:  "America/Chicago",
	},
	"93FL": {
		ICAO:      "93FL",
		Name:      "Pine Lakes Farm Airport",
		City:      "Deland",
		State:     "Florida",
		Country:   "US",
		Elevation: 60,
		Latitude:  28.9447002411,
		Longitude: -81.4080963135,
		Timezone:  "America/New_York",
	},
	"93IA": {
		ICAO:      "93IA",
		Name:      "Lund Airport",
		City:      "Jewell",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1063,
		Latitude:  42.2602996826,
		Longitude: -93.6496963501,
		Timezone:  "America/Chicago",
	},
	"93IL": {
		ICAO:      "93IL",
		Name:      "Bauer Airport",
		City:      "Lindenwood",
		State:     "Illinois",
		Country:   "US",
		Elevation: 780,
		Latitude:  42.0675010681,
		Longitude: -89.0231018066,
		Timezone:  "America/Chicago",
	},
	"93IN": {
		ICAO:      "93IN",
		Name:      "Foltz Farm Airport",
		City:      "Shelbyville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 820,
		Latitude:  39.5886001587,
		Longitude: -85.7360992432,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"93KY": {
		ICAO:      "93KY",
		Name:      "Baggett Farms Airport",
		City:      "Sacramento",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 405,
		Latitude:  37.4707984924,
		Longitude: -87.2533035278,
		Timezone:  "America/Chicago",
	},
	"93LL": {
		ICAO:      "93LL",
		Name:      "Hemmingsen Airport",
		City:      "Sandwich",
		State:     "Illinois",
		Country:   "US",
		Elevation: 640,
		Latitude:  41.5945014954,
		Longitude: -88.6055984497,
		Timezone:  "America/Chicago",
	},
	"93MD": {
		ICAO:      "93MD",
		Name:      "Knollwood Farm Airport",
		City:      "Elkton",
		State:     "Maryland",
		Country:   "US",
		Elevation: 100,
		Latitude:  39.561833,
		Longitude: -75.834833,
		Timezone:  "America/New_York",
	},
	"93MN": {
		ICAO:      "93MN",
		Name:      "Hoppe Sky Ranch Airport",
		City:      "Mankato",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1010,
		Latitude:  44.0321006775,
		Longitude: -93.9484024048,
		Timezone:  "America/Chicago",
	},
	"93MO": {
		ICAO:      "93MO",
		Name:      "Kathy's Patch Airport",
		City:      "Prosperity",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1030,
		Latitude:  37.1088981628,
		Longitude: -94.4055023193,
		Timezone:  "America/Chicago",
	},
	"93MS": {
		ICAO:      "93MS",
		Name:      "Shelby Air Service Airport",
		City:      "Shelby",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 154,
		Latitude:  33.9715995789,
		Longitude: -90.7682037354,
		Timezone:  "America/Chicago",
	},
	"93MT": {
		ICAO:      "93MT",
		Name:      "Masonry Field",
		City:      "Bigfork",
		State:     "Montana",
		Country:   "US",
		Elevation: 3049,
		Latitude:  48.100833,
		Longitude: -114.089167,
		Timezone:  "America/Denver",
	},
	"93OH": {
		ICAO:      "93OH",
		Name:      "Gay Airport",
		City:      "Atwater",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1200,
		Latitude:  41.0444984436,
		Longitude: -81.1858978271,
		Timezone:  "America/New_York",
	},
	"93OK": {
		ICAO:      "93OK",
		Name:      "Jantzen Airport",
		City:      "Coweta",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 670,
		Latitude:  35.9345016479,
		Longitude: -95.5914001465,
		Timezone:  "America/Chicago",
	},
	"93PA": {
		ICAO:      "93PA",
		Name:      "Maple Cave Park Airport",
		City:      "Cooperstown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1180,
		Latitude:  41.5033988953,
		Longitude: -79.881401062,
		Timezone:  "America/New_York",
	},
	"93TE": {
		ICAO:      "93TE",
		Name:      "Mayhew Ranch Nr 1 Airport",
		City:      "Marathon",
		State:     "Texas",
		Country:   "US",
		Elevation: 2783,
		Latitude:  29.8096008301,
		Longitude: -102.9079971313,
		Timezone:  "America/Chicago",
	},
	"93TS": {
		ICAO:      "93TS",
		Name:      "Longbird Airport",
		City:      "Sealy",
		State:     "Texas",
		Country:   "US",
		Elevation: 150,
		Latitude:  29.7483005524,
		Longitude: -96.1147003174,
		Timezone:  "America/Chicago",
	},
	"93TX": {
		ICAO:      "93TX",
		Name:      "Putz Aero Inc Airport",
		City:      "Calvert",
		State:     "Texas",
		Country:   "US",
		Elevation: 279,
		Latitude:  30.914899826,
		Longitude: -96.6855010986,
		Timezone:  "America/Chicago",
	},
	"93VA": {
		ICAO:      "93VA",
		Name:      "Timberdoodle Airport",
		City:      "Amherst",
		State:     "Virginia",
		Country:   "US",
		Elevation: 810,
		Latitude:  37.5363006592,
		Longitude: -79.023399353,
		Timezone:  "America/New_York",
	},
	"93WA": {
		ICAO:      "93WA",
		Name:      "Harris Airport",
		City:      "Tyler",
		State:     "Washington",
		Country:   "US",
		Elevation: 2375,
		Latitude:  47.4478988647,
		Longitude: -117.8059997559,
		Timezone:  "America/Los_Angeles",
	},
	"93WI": {
		ICAO:      "93WI",
		Name:      "Arrowhead Springs Airport",
		City:      "Richfield",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 980,
		Latitude:  43.2536010742,
		Longitude: -88.2095031738,
		Timezone:  "America/Chicago",
	},
	"93XS": {
		ICAO:      "93XS",
		Name:      "Joseph Ross Scherdin Airport",
		City:      "Richwood",
		State:     "Texas",
		Country:   "US",
		Elevation: 10,
		Latitude:  29.0811004639,
		Longitude: -95.3826980591,
		Timezone:  "America/Chicago",
	},
	"94FL": {
		ICAO:      "94FL",
		Name:      "Pine Shadows Airpark",
		City:      "Fort Myers",
		State:     "Florida",
		Country:   "US",
		Elevation: 20,
		Latitude:  26.73279953,
		Longitude: -81.8973007202,
		Timezone:  "America/New_York",
	},
	"94GA": {
		ICAO:      "94GA",
		Name:      "Snow Hill Airstrip",
		City:      "Unadilla",
		State:     "Georgia",
		Country:   "US",
		Elevation: 465,
		Latitude:  32.2629013062,
		Longitude: -83.7871017456,
		Timezone:  "America/New_York",
	},
	"94IL": {
		ICAO:      "94IL",
		Name:      "Lutz Airport",
		City:      "Mulberry Grove",
		State:     "Illinois",
		Country:   "US",
		Elevation: 526,
		Latitude:  38.917301178,
		Longitude: -89.2256011963,
		Timezone:  "America/Chicago",
	},
	"94IS": {
		ICAO:      "94IS",
		Name:      "Adams Private Airport",
		City:      "Royalton",
		State:     "Illinois",
		Country:   "US",
		Elevation: 410,
		Latitude:  37.9028015137,
		Longitude: -89.1376037598,
		Timezone:  "America/Chicago",
	},
	"94KS": {
		ICAO:      "94KS",
		Name:      "Smith Field",
		City:      "Macksville",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2023,
		Latitude:  37.9749984741,
		Longitude: -98.9656982422,
		Timezone:  "America/Chicago",
	},
	"94KY": {
		ICAO:      "94KY",
		Name:      "Woosley Airport",
		City:      "Rumsey",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 400,
		Latitude:  37.5005989075,
		Longitude: -87.3293991089,
		Timezone:  "America/Chicago",
	},
	"94MN": {
		ICAO:      "94MN",
		Name:      "Ag Spray Inc Airport",
		City:      "Barnesville",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 960,
		Latitude:  46.5765991211,
		Longitude: -96.5120010376,
		Timezone:  "America/Chicago",
	},
	"94MO": {
		ICAO:      "94MO",
		Name:      "Ray's Roost Airport",
		City:      "West Plains",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1170,
		Latitude:  36.8053016663,
		Longitude: -91.8813018799,
		Timezone:  "America/Chicago",
	},
	"94NC": {
		ICAO:      "94NC",
		Name:      "Viking Airport",
		City:      "Raeford",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 295,
		Latitude:  35.009601593,
		Longitude: -79.2600021362,
		Timezone:  "America/New_York",
	},
	"94ND": {
		ICAO:      "94ND",
		Name:      "Michael Zurcher Farm Strip",
		City:      "Newburg",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1470,
		Latitude:  48.6659011841,
		Longitude: -100.9179992676,
		Timezone:  "America/Chicago",
	},
	"94NY": {
		ICAO:      "94NY",
		Name:      "St Bernard Field",
		City:      "Union Springs",
		State:     "New York",
		Country:   "US",
		Elevation: 540,
		Latitude:  42.8250999451,
		Longitude: -76.6912994385,
		Timezone:  "America/New_York",
	},
	"94OR": {
		ICAO:      "94OR",
		Name:      "Farm Yard Field",
		City:      "Elkton",
		State:     "Oregon",
		Country:   "US",
		Elevation: 230,
		Latitude:  43.5307006836,
		Longitude: -123.5490036011,
		Timezone:  "America/Los_Angeles",
	},
	"94TA": {
		ICAO:      "94TA",
		Name:      "Reece Field",
		City:      "San Angelo",
		State:     "Texas",
		Country:   "US",
		Elevation: 1960,
		Latitude:  31.3057003021,
		Longitude: -100.4589996338,
		Timezone:  "America/Chicago",
	},
	"94TX": {
		ICAO:      "94TX",
		Name:      "River Bend Ranch Airport",
		City:      "Brady",
		State:     "Texas",
		Country:   "US",
		Elevation: 1500,
		Latitude:  31.4377002716,
		Longitude: -99.3155975342,
		Timezone:  "America/Chicago",
	},
	"94VA": {
		ICAO:      "94VA",
		Name:      "Highview Farms Airport",
		City:      "Appomattox",
		State:     "Virginia",
		Country:   "US",
		Elevation: 765,
		Latitude:  37.2630996704,
		Longitude: -78.8494033813,
		Timezone:  "America/New_York",
	},
	"94WA": {
		ICAO:      "94WA",
		Name:      "Wishkah River Ranch Airport",
		City:      "Aberdeen",
		State:     "Washington",
		Country:   "US",
		Elevation: 36,
		Latitude:  47.0848007202,
		Longitude: -123.7740020752,
		Timezone:  "America/Los_Angeles",
	},
	"94XS": {
		ICAO:      "94XS",
		Name:      "Heritage Ranch Airport",
		City:      "Richmond",
		State:     "Texas",
		Country:   "US",
		Elevation: 103,
		Latitude:  29.6543998718,
		Longitude: -95.8293991089,
		Timezone:  "America/Chicago",
	},
	"95CA": {
		ICAO:      "95CA",
		Name:      "Bonel Airport",
		City:      "Whitley Garden",
		State:     "California",
		Country:   "US",
		Elevation: 1030,
		Latitude:  35.6691017151,
		Longitude: -120.5490036011,
		Timezone:  "America/Los_Angeles",
	},
	"95CO": {
		ICAO:      "95CO",
		Name:      "Mann Ranch Airport",
		City:      "Rye",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6000,
		Latitude:  37.8684997559,
		Longitude: -104.8359985352,
		Timezone:  "America/Denver",
	},
	"95FD": {
		ICAO:      "95FD",
		Name:      "South Point Airport",
		City:      "La Belle",
		State:     "Florida",
		Country:   "US",
		Elevation: 22,
		Latitude:  26.7682991028,
		Longitude: -81.5389022827,
		Timezone:  "America/New_York",
	},
	"95GA": {
		ICAO:      "95GA",
		Name:      "Mcintosh Field",
		City:      "Whitesburg",
		State:     "Georgia",
		Country:   "US",
		Elevation: 771,
		Latitude:  33.4558982849,
		Longitude: -84.9430007935,
		Timezone:  "America/New_York",
	},
	"95IL": {
		ICAO:      "95IL",
		Name:      "Henderson Airport",
		City:      "Rosiclare",
		State:     "Illinois",
		Country:   "US",
		Elevation: 485,
		Latitude:  37.4861984253,
		Longitude: -88.4000015259,
		Timezone:  "America/Chicago",
	},
	"95IN": {
		ICAO:      "95IN",
		Name:      "Stewart Field",
		City:      "Seymour",
		State:     "Indiana",
		Country:   "US",
		Elevation: 550,
		Latitude:  38.8875007629,
		Longitude: -85.8264007568,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"95IS": {
		ICAO:      "95IS",
		Name:      "Jan Knipe Airport",
		City:      "Warsaw",
		State:     "Illinois",
		Country:   "US",
		Elevation: 645,
		Latitude:  40.3706016541,
		Longitude: -91.3889007568,
		Timezone:  "America/Chicago",
	},
	"95KY": {
		ICAO:      "95KY",
		Name:      "Crawford Ultralightport",
		City:      "Brodhead",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 1227,
		Latitude:  37.42029953,
		Longitude: -84.3692016602,
		Timezone:  "America/New_York",
	},
	"95LL": {
		ICAO:      "95LL",
		Name:      "Warren Airport",
		City:      "Somonauk",
		State:     "Illinois",
		Country:   "US",
		Elevation: 694,
		Latitude:  41.6478004456,
		Longitude: -88.6943969727,
		Timezone:  "America/Chicago",
	},
	"95MI": {
		ICAO:      "95MI",
		Name:      "Guy Vander Jagt Airfield",
		City:      "Luther",
		State:     "Michigan",
		Country:   "US",
		Elevation: 970,
		Latitude:  44.0894012451,
		Longitude: -85.7230987549,
		Timezone:  "America/Detroit",
	},
	"95MO": {
		ICAO:      "95MO",
		Name:      "Schlemmer Airport",
		City:      "Laddonia",
		State:     "Missouri",
		Country:   "US",
		Elevation: 780,
		Latitude:  39.2594985962,
		Longitude: -91.6501998901,
		Timezone:  "America/Chicago",
	},
	"95NE": {
		ICAO:      "95NE",
		Name:      "W Meeks Ranch Airport",
		City:      "Taylor",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2335,
		Latitude:  41.9928016663,
		Longitude: -99.4553985596,
		Timezone:  "America/Chicago",
	},
	"95NY": {
		ICAO:      "95NY",
		Name:      "Fisher Airport",
		City:      "Wales Center/Cowlesville/",
		State:     "New York",
		Country:   "US",
		Elevation: 1280,
		Latitude:  42.7891998291,
		Longitude: -78.4764022827,
		Timezone:  "America/New_York",
	},
	"95OH": {
		ICAO:      "95OH",
		Name:      "Tong Farm Airport",
		City:      "Carey",
		State:     "Ohio",
		Country:   "US",
		Elevation: 840,
		Latitude:  40.9709014893,
		Longitude: -83.4582977295,
		Timezone:  "America/New_York",
	},
	"95OK": {
		ICAO:      "95OK",
		Name:      "Barcus Field",
		City:      "Claremore",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 660,
		Latitude:  36.2676010132,
		Longitude: -95.6306991577,
		Timezone:  "America/Chicago",
	},
	"95OR": {
		ICAO:      "95OR",
		Name:      "Flournoy Valley Airport",
		City:      "Roseburg",
		State:     "Oregon",
		Country:   "US",
		Elevation: 790,
		Latitude:  43.2075996399,
		Longitude: -123.5329971313,
		Timezone:  "America/Los_Angeles",
	},
	"95TA": {
		ICAO:      "95TA",
		Name:      "Thunder Creek Airport",
		City:      "Utopia",
		State:     "Texas",
		Country:   "US",
		Elevation: 1500,
		Latitude:  29.6348991394,
		Longitude: -99.4822998047,
		Timezone:  "America/Chicago",
	},
	"95TS": {
		ICAO:      "95TS",
		Name:      "Rob Airport",
		City:      "Loving",
		State:     "Texas",
		Country:   "US",
		Elevation: 1310,
		Latitude:  33.2308006287,
		Longitude: -98.4897003174,
		Timezone:  "America/Chicago",
	},
	"95TX": {
		ICAO:      "95TX",
		Name:      "Chupadera Ranch Airport",
		City:      "Carrizo Springs",
		State:     "Texas",
		Country:   "US",
		Elevation: 651,
		Latitude:  28.1924991608,
		Longitude: -100.0719985962,
		Timezone:  "America/Chicago",
	},
	"95VA": {
		ICAO:      "95VA",
		Name:      "Buck Hollar Airport",
		City:      "Danville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 450,
		Latitude:  36.6231994629,
		Longitude: -79.3559036255,
		Timezone:  "America/New_York",
	},
	"95WA": {
		ICAO:      "95WA",
		Name:      "Black Diamond Airport",
		City:      "Black Diamond",
		State:     "Washington",
		Country:   "US",
		Elevation: 595,
		Latitude:  47.315700531,
		Longitude: -122.0100021362,
		Timezone:  "America/Los_Angeles",
	},
	"95WI": {
		ICAO:      "95WI",
		Name:      "Wisersky Airport",
		City:      "Stoughton",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 965,
		Latitude:  42.9361000061,
		Longitude: -89.129699707,
		Timezone:  "America/Chicago",
	},
	"96CO": {
		ICAO:      "96CO",
		Name:      "Logan Airport",
		City:      "Bennet",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5300,
		Latitude:  39.8563995361,
		Longitude: -104.3960037231,
		Timezone:  "America/Denver",
	},
	"96FD": {
		ICAO:      "96FD",
		Name:      "Citrus Hedging Ranch Airport",
		City:      "Okeechobee",
		State:     "Florida",
		Country:   "US",
		Elevation: 60,
		Latitude:  27.3334007263,
		Longitude: -80.7361984253,
		Timezone:  "America/New_York",
	},
	"96FL": {
		ICAO:      "96FL",
		Name:      "Wings Field",
		City:      "Williston",
		State:     "Florida",
		Country:   "US",
		Elevation: 100,
		Latitude:  29.2425003052,
		Longitude: -82.5454025269,
		Timezone:  "America/New_York",
	},
	"96GA": {
		ICAO:      "96GA",
		Name:      "3-M's Airport",
		City:      "Williamson",
		State:     "Georgia",
		Country:   "US",
		Elevation: 890,
		Latitude:  33.185798645,
		Longitude: -84.3681030273,
		Timezone:  "America/New_York",
	},
	"96ID": {
		ICAO:      "96ID",
		Name:      "Flying H Ranch Airport",
		City:      "Priest River",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2230,
		Latitude:  48.2355995178,
		Longitude: -116.875,
		Timezone:  "America/Los_Angeles",
	},
	"96IL": {
		ICAO:      "96IL",
		Name:      "Kramer Airport",
		City:      "Lena",
		State:     "Illinois",
		Country:   "US",
		Elevation: 930,
		Latitude:  42.375,
		Longitude: -89.8460006714,
		Timezone:  "America/Chicago",
	},
	"96IS": {
		ICAO:      "96IS",
		Name:      "Hildreth Air Park",
		City:      "Sidell",
		State:     "Illinois",
		Country:   "US",
		Elevation: 730,
		Latitude:  39.875,
		Longitude: -87.8450012207,
		Timezone:  "America/Chicago",
	},
	"96KY": {
		ICAO:      "96KY",
		Name:      "A & L Airport",
		City:      "Calhoun",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 465,
		Latitude:  37.5522003174,
		Longitude: -87.2714004517,
		Timezone:  "America/Chicago",
	},
	"96LL": {
		ICAO:      "96LL",
		Name:      "Wichman Airport",
		City:      "Stockland",
		State:     "Illinois",
		Country:   "US",
		Elevation: 681,
		Latitude:  40.6161003113,
		Longitude: -87.6141967773,
		Timezone:  "America/Chicago",
	},
	"96MN": {
		ICAO:      "96MN",
		Name:      "Air-Ag Airport",
		City:      "Glenville",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1240,
		Latitude:  43.5680007935,
		Longitude: -93.2544021606,
		Timezone:  "America/Chicago",
	},
	"96MU": {
		ICAO:      "96MU",
		Name:      "Green Airfield",
		City:      "St. James",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1068,
		Latitude:  37.9434013367,
		Longitude: -91.6020965576,
		Timezone:  "America/Chicago",
	},
	"96OK": {
		ICAO:      "96OK",
		Name:      "Mike's Place Airport",
		City:      "Buffalo",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1787,
		Latitude:  36.8488998413,
		Longitude: -99.456703186,
		Timezone:  "America/Chicago",
	},
	"96OR": {
		ICAO:      "96OR",
		Name:      "Cable Creek Ranch Airport",
		City:      "Ukiah",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4060,
		Latitude:  45.1006011963,
		Longitude: -118.8170013428,
		Timezone:  "America/Los_Angeles",
	},
	"96PA": {
		ICAO:      "96PA",
		Name:      "Franklin Center Airport",
		City:      "Cranesville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1225,
		Latitude:  41.9208984375,
		Longitude: -80.2498016357,
		Timezone:  "America/New_York",
	},
	"96TE": {
		ICAO:      "96TE",
		Name:      "Yorktown Airport",
		City:      "Yorktown",
		State:     "Texas",
		Country:   "US",
		Elevation: 240,
		Latitude:  28.9477996826,
		Longitude: -97.4599990845,
		Timezone:  "America/Chicago",
	},
	"96TS": {
		ICAO:      "96TS",
		Name:      "Nuttall Airport",
		City:      "Whitehouse",
		State:     "Texas",
		Country:   "US",
		Elevation: 425,
		Latitude:  32.2246017456,
		Longitude: -95.1958007813,
		Timezone:  "America/Chicago",
	},
	"96TX": {
		ICAO:      "96TX",
		Name:      "Brown Field",
		City:      "Christoval",
		State:     "Texas",
		Country:   "US",
		Elevation: 2125,
		Latitude:  31.1653995514,
		Longitude: -100.4759979248,
		Timezone:  "America/Chicago",
	},
	"96VE": {
		ICAO:      "96VE",
		Name:      "Beiter Airport",
		City:      "Ellensburg",
		State:     "Washington",
		Country:   "US",
		Elevation: 1910,
		Latitude:  46.906667,
		Longitude: -120.433611,
		Timezone:  "America/Los_Angeles",
	},
	"96WA": {
		ICAO:      "96WA",
		Name:      "Jim & Julie's Airport",
		City:      "Everett",
		State:     "Washington",
		Country:   "US",
		Elevation: 15,
		Latitude:  47.8981018066,
		Longitude: -122.15599823,
		Timezone:  "America/Los_Angeles",
	},
	"96WY": {
		ICAO:      "96WY",
		Name:      "Cedar Creek Ranch Airport",
		City:      "Saratoga",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 7760,
		Latitude:  41.409198761,
		Longitude: -106.5920028687,
		Timezone:  "America/Denver",
	},
	"97AK": {
		ICAO:      "97AK",
		Name:      "High Ridge Association Airport",
		City:      "Palmer",
		State:     "Alaska",
		Country:   "US",
		Elevation: 720,
		Latitude:  61.6640014648,
		Longitude: -149.2539978027,
		Timezone:  "America/Anchorage",
	},
	"97CA": {
		ICAO:      "97CA",
		Name:      "Tejon Agricultural Airport",
		City:      "Lebec",
		State:     "California",
		Country:   "US",
		Elevation: 1065,
		Latitude:  34.9891014099,
		Longitude: -118.9150009155,
		Timezone:  "America/Los_Angeles",
	},
	"97CL": {
		ICAO:      "97CL",
		Name:      "Siller Bros Inc Airport",
		City:      "Oroville",
		State:     "California",
		Country:   "US",
		Elevation: 235,
		Latitude:  39.4602012634,
		Longitude: -121.5810012817,
		Timezone:  "America/Los_Angeles",
	},
	"97CO": {
		ICAO:      "97CO",
		Name:      "Peakview Airport",
		City:      "Yoder",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6370,
		Latitude:  38.9249992371,
		Longitude: -104.158996582,
		Timezone:  "America/Denver",
	},
	"97FL": {
		ICAO:      "97FL",
		Name:      "Love Field",
		City:      "Weirsdale",
		State:     "Florida",
		Country:   "US",
		Elevation: 80,
		Latitude:  28.9619007111,
		Longitude: -81.8915023804,
		Timezone:  "America/New_York",
	},
	"97GA": {
		ICAO:      "97GA",
		Name:      "B Tree Farms Airport",
		City:      "Gillsville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1000,
		Latitude:  34.2681999207,
		Longitude: -83.6720962524,
		Timezone:  "America/New_York",
	},
	"97IA": {
		ICAO:      "97IA",
		Name:      "Volkens Field",
		City:      "Carson",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1275,
		Latitude:  41.2557983398,
		Longitude: -95.4766998291,
		Timezone:  "America/Chicago",
	},
	"97KS": {
		ICAO:      "97KS",
		Name:      "Clear View Farm Airport",
		City:      "Olathe",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1060,
		Latitude:  38.8101005554,
		Longitude: -94.7556991577,
		Timezone:  "America/Chicago",
	},
	"97LA": {
		ICAO:      "97LA",
		Name:      "Greene Air Park",
		City:      "Milton",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 50,
		Latitude:  30.0338001251,
		Longitude: -92.0551986694,
		Timezone:  "America/Chicago",
	},
	"97MO": {
		ICAO:      "97MO",
		Name:      "Gary's Airport",
		City:      "Buffalo",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1200,
		Latitude:  37.7566986084,
		Longitude: -93.0162963867,
		Timezone:  "America/Chicago",
	},
	"97MT": {
		ICAO:      "97MT",
		Name:      "Cabin Creek Landing Airport",
		City:      "Marion",
		State:     "Montana",
		Country:   "US",
		Elevation: 3999,
		Latitude:  48.0749015808,
		Longitude: -114.6760025024,
		Timezone:  "America/Denver",
	},
	"97ND": {
		ICAO:      "97ND",
		Name:      "Walser Strip",
		City:      "Park River",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 960,
		Latitude:  48.4308013916,
		Longitude: -97.7080993652,
		Timezone:  "America/Chicago",
	},
	"97NY": {
		ICAO:      "97NY",
		Name:      "Hunter Mountain Airport",
		City:      "Hunter",
		State:     "New York",
		Country:   "US",
		Elevation: 2000,
		Latitude:  42.2342987061,
		Longitude: -74.2376022339,
		Timezone:  "America/New_York",
	},
	"97OG": {
		ICAO:      "97OG",
		Name:      "Bybee Field",
		City:      "Nyssa",
		State:     "Oregon",
		Country:   "US",
		Elevation: 2150,
		Latitude:  43.897828,
		Longitude: -116.989964,
		Timezone:  "America/Boise",
	},
	"97OR": {
		ICAO:      "97OR",
		Name:      "Hi Country No 2 Airport",
		City:      "Prairie City",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3840,
		Latitude:  44.4351997375,
		Longitude: -118.6620025635,
		Timezone:  "America/Los_Angeles",
	},
	"97PA": {
		ICAO:      "97PA",
		Name:      "Willows Airport",
		City:      "Cranesville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1225,
		Latitude:  41.9084014893,
		Longitude: -80.258102417,
		Timezone:  "America/New_York",
	},
	"97PN": {
		ICAO:      "97PN",
		Name:      "Gold Mine Field",
		City:      "Sellersville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 550,
		Latitude:  40.3750991821,
		Longitude: -75.3496017456,
		Timezone:  "America/New_York",
	},
	"97TN": {
		ICAO:      "97TN",
		Name:      "Kite Field",
		City:      "Jonesborough",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1450,
		Latitude:  36.2785568902,
		Longitude: -82.5857627392,
		Timezone:  "America/New_York",
	},
	"97TS": {
		ICAO:      "97TS",
		Name:      "Gdap Air Ranch Airport",
		City:      "Willis",
		State:     "Texas",
		Country:   "US",
		Elevation: 300,
		Latitude:  30.4801998138,
		Longitude: -95.5399017334,
		Timezone:  "America/Chicago",
	},
	"97TX": {
		ICAO:      "97TX",
		Name:      "San Pedro Ranch Airport",
		City:      "Carrizo Springs",
		State:     "Texas",
		Country:   "US",
		Elevation: 710,
		Latitude:  28.2835998535,
		Longitude: -100.0670013428,
		Timezone:  "America/Chicago",
	},
	"97VA": {
		ICAO:      "97VA",
		Name:      "Singleton Airport",
		City:      "Warm Springs",
		State:     "Virginia",
		Country:   "US",
		Elevation: 2060,
		Latitude:  38.2107009888,
		Longitude: -79.7134017944,
		Timezone:  "America/New_York",
	},
	"97WA": {
		ICAO:      "97WA",
		Name:      "Basin City Airfield",
		City:      "Mesa",
		State:     "Washington",
		Country:   "US",
		Elevation: 722,
		Latitude:  46.5857009888,
		Longitude: -119.1539993286,
		Timezone:  "America/Los_Angeles",
	},
	"97WI": {
		ICAO:      "97WI",
		Name:      "North Fork Airport",
		City:      "Thorp",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1145,
		Latitude:  44.9632987976,
		Longitude: -90.8602981567,
		Timezone:  "America/Chicago",
	},
	"97XS": {
		ICAO:      "97XS",
		Name:      "Tilghman Airport",
		City:      "Van Alstyne",
		State:     "Texas",
		Country:   "US",
		Elevation: 741,
		Latitude:  33.432833,
		Longitude: -96.479467,
		Timezone:  "America/Chicago",
	},
	"98AK": {
		ICAO:      "98AK",
		Name:      "Eastland Airport",
		City:      "Homer",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1190,
		Latitude:  59.7757987976,
		Longitude: -151.1820068359,
		Timezone:  "America/Anchorage",
	},
	"98CL": {
		ICAO:      "98CL",
		Name:      "Noltas Airport",
		City:      "Willows",
		State:     "California",
		Country:   "US",
		Elevation: 145,
		Latitude:  39.5789985657,
		Longitude: -122.2009963989,
		Timezone:  "America/Los_Angeles",
	},
	"98GA": {
		ICAO:      "98GA",
		Name:      "Circle T Airport",
		City:      "Indian Springs",
		State:     "Georgia",
		Country:   "US",
		Elevation: 545,
		Latitude:  33.2056999207,
		Longitude: -83.912399292,
		Timezone:  "America/New_York",
	},
	"98IN": {
		ICAO:      "98IN",
		Name:      "B & V Flying Ranch Airport",
		City:      "Albion",
		State:     "Indiana",
		Country:   "US",
		Elevation: 950,
		Latitude:  41.363899231,
		Longitude: -85.3886032104,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"98KS": {
		ICAO:      "98KS",
		Name:      "Rexford Airport",
		City:      "Montezuma",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2775,
		Latitude:  37.4458999634,
		Longitude: -100.4919967651,
		Timezone:  "America/Chicago",
	},
	"98LL": {
		ICAO:      "98LL",
		Name:      "Braden Farms Airport",
		City:      "Watseka",
		State:     "Illinois",
		Country:   "US",
		Elevation: 630,
		Latitude:  40.7991981506,
		Longitude: -87.7835998535,
		Timezone:  "America/Chicago",
	},
	"98ME": {
		ICAO:      "98ME",
		Name:      "Greaton Airfield",
		City:      "Saco",
		State:     "Maine",
		Country:   "US",
		Elevation: 170,
		Latitude:  43.575717,
		Longitude: -70.518858,
		Timezone:  "America/New_York",
	},
	"98MN": {
		ICAO:      "98MN",
		Name:      "Up Yonder Airport",
		City:      "Bemidji",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1446,
		Latitude:  47.29349899,
		Longitude: -94.87490082,
		Timezone:  "America/Chicago",
	},
	"98MO": {
		ICAO:      "98MO",
		Name:      "Woodliff Airpark",
		City:      "Foristell",
		State:     "Missouri",
		Country:   "US",
		Elevation: 750,
		Latitude:  38.7862014771,
		Longitude: -90.9423980713,
		Timezone:  "America/Chicago",
	},
	"98MT": {
		ICAO:      "98MT",
		Name:      "Rosemont Airport",
		City:      "Stevensville",
		State:     "Montana",
		Country:   "US",
		Elevation: 4172,
		Latitude:  46.4427986145,
		Longitude: -114.0029983521,
		Timezone:  "America/Denver",
	},
	"98ND": {
		ICAO:      "98ND",
		Name:      "Sobolik Airport",
		City:      "Pisek",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 920,
		Latitude:  48.2633018494,
		Longitude: -97.6222991943,
		Timezone:  "America/Chicago",
	},
	"98NE": {
		ICAO:      "98NE",
		Name:      "Munsterman Airport",
		City:      "Glenvil",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1860,
		Latitude:  40.4542007446,
		Longitude: -98.3044967651,
		Timezone:  "America/Chicago",
	},
	"98NM": {
		ICAO:      "98NM",
		Name:      "S & S Ranch Airport",
		City:      "Wagon Mound",
		State:     "New Mexico",
		Country:   "US",
		Elevation: 6310,
		Latitude:  36.0766983032,
		Longitude: -104.7180023193,
		Timezone:  "America/Denver",
	},
	"98NY": {
		ICAO:      "98NY",
		Name:      "Krenzers Airport",
		City:      "West Bloomfield",
		State:     "New York",
		Country:   "US",
		Elevation: 1035,
		Latitude:  42.8833999634,
		Longitude: -77.5330963135,
		Timezone:  "America/New_York",
	},
	"98OH": {
		ICAO:      "98OH",
		Name:      "Paine's Airport",
		City:      "Geneva",
		State:     "Ohio",
		Country:   "US",
		Elevation: 660,
		Latitude:  41.8250999451,
		Longitude: -80.883102417,
		Timezone:  "America/New_York",
	},
	"98OK": {
		ICAO:      "98OK",
		Name:      "Candy Lake Estate Airport",
		City:      "Avant",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 685,
		Latitude:  36.489200592,
		Longitude: -96.0528030396,
		Timezone:  "America/Chicago",
	},
	"98OL": {
		ICAO:      "98OL",
		Name:      "Copland Airport",
		City:      "Duncan",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1120,
		Latitude:  34.5000991821,
		Longitude: -97.9947967529,
		Timezone:  "America/Chicago",
	},
	"98OR": {
		ICAO:      "98OR",
		Name:      "Mach-O Acres Airport",
		City:      "Sheridan",
		State:     "Oregon",
		Country:   "US",
		Elevation: 435,
		Latitude:  45.1990013123,
		Longitude: -123.3700027466,
		Timezone:  "America/Los_Angeles",
	},
	"98PA": {
		ICAO:      "98PA",
		Name:      "Pleasant Hill Airport",
		City:      "Portersville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1180,
		Latitude:  40.8944015503,
		Longitude: -80.181098938,
		Timezone:  "America/New_York",
	},
	"98TE": {
		ICAO:      "98TE",
		Name:      "Hilltop Airport",
		City:      "Glide",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1320,
		Latitude:  43.322101593,
		Longitude: -123.1770019531,
		Timezone:  "America/Los_Angeles",
	},
	"98TN": {
		ICAO:      "98TN",
		Name:      "Landing At River'S Edge",
		City:      "Blaine",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1019,
		Latitude:  36.1425,
		Longitude: -83.604167,
		Timezone:  "America/New_York",
	},
	"98TS": {
		ICAO:      "98TS",
		Name:      "Trap Travelstead Field",
		City:      "Winona",
		State:     "Texas",
		Country:   "US",
		Elevation: 430,
		Latitude:  32.4182014465,
		Longitude: -95.0001983643,
		Timezone:  "America/Chicago",
	},
	"98VA": {
		ICAO:      "98VA",
		Name:      "Glascock Airport",
		City:      "Arcola",
		State:     "Virginia",
		Country:   "US",
		Elevation: 330,
		Latitude:  38.9422988892,
		Longitude: -77.5421981812,
		Timezone:  "America/New_York",
	},
	"99AK": {
		ICAO:      "99AK",
		Name:      "Moore Creek Airport",
		City:      "Moore Creek",
		State:     "Alaska",
		Country:   "US",
		Elevation: 980,
		Latitude:  62.5989990234,
		Longitude: -157.1519927979,
		Timezone:  "America/Anchorage",
	},
	"99AZ": {
		ICAO:      "99AZ",
		Name:      "Eagletail Ranch Airport",
		City:      "Tonopah",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1203,
		Latitude:  33.3955993652,
		Longitude: -113.2229995728,
		Timezone:  "America/Phoenix",
	},
	"99CL": {
		ICAO:      "99CL",
		Name:      "El Mirage Field Adelanto Airport",
		City:      "El Mirage",
		State:     "California",
		Country:   "US",
		Elevation: 2865,
		Latitude:  34.625,
		Longitude: -117.6050033569,
		Timezone:  "America/Los_Angeles",
	},
	"99CO": {
		ICAO:      "99CO",
		Name:      "Rock Creek Airport",
		City:      "Pueblo",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5660,
		Latitude:  38.1363983154,
		Longitude: -104.8150024414,
		Timezone:  "America/Denver",
	},
	"99FL": {
		ICAO:      "99FL",
		Name:      "Madison County Airport",
		City:      "Lee",
		State:     "Florida",
		Country:   "US",
		Elevation: 150,
		Latitude:  30.4473991394,
		Longitude: -83.3109970093,
		Timezone:  "America/New_York",
	},
	"99IA": {
		ICAO:      "99IA",
		Name:      "Courtney's Landing Airport",
		City:      "Eddyville",
		State:     "Iowa",
		Country:   "US",
		Elevation: 835,
		Latitude:  41.2165985107,
		Longitude: -92.7779998779,
		Timezone:  "America/Chicago",
	},
	"99II": {
		ICAO:      "99II",
		Name:      "Cooper Field",
		City:      "Reelsville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 880,
		Latitude:  39.5835,
		Longitude: -86.952527,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"99IL": {
		ICAO:      "99IL",
		Name:      "Providence Place Field",
		City:      "Stockton",
		State:     "Illinois",
		Country:   "US",
		Elevation: 959,
		Latitude:  42.2867012024,
		Longitude: -89.9574966431,
		Timezone:  "America/Chicago",
	},
	"99IN": {
		ICAO:      "99IN",
		Name:      "Bee-Acre Farm Strip",
		City:      "Atlanta",
		State:     "Indiana",
		Country:   "US",
		Elevation: 885,
		Latitude:  40.2061004639,
		Longitude: -86.1005020142,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"99KS": {
		ICAO:      "99KS",
		Name:      "Elm Creek Farms Airport",
		City:      "Medicine Lodge",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1600,
		Latitude:  37.4093017578,
		Longitude: -98.6493988037,
		Timezone:  "America/Chicago",
	},
	"99KY": {
		ICAO:      "99KY",
		Name:      "Gravity Zero Airport",
		City:      "Danville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 860,
		Latitude:  37.6556015015,
		Longitude: -84.6902999878,
		Timezone:  "America/New_York",
	},
	"99LA": {
		ICAO:      "99LA",
		Name:      "Lawson Field",
		City:      "Crowley",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 25,
		Latitude:  30.2894001007,
		Longitude: -92.388999939,
		Timezone:  "America/Chicago",
	},
	"99MT": {
		ICAO:      "99MT",
		Name:      "Fox Field",
		City:      "Hamilton",
		State:     "Montana",
		Country:   "US",
		Elevation: 3750,
		Latitude:  46.0974998474,
		Longitude: -114.1780014038,
		Timezone:  "America/Denver",
	},
	"99NY": {
		ICAO:      "99NY",
		Name:      "Hibbard's Airport",
		City:      "Wilson",
		State:     "New York",
		Country:   "US",
		Elevation: 410,
		Latitude:  43.2333984375,
		Longitude: -78.7873001099,
		Timezone:  "America/New_York",
	},
	"99OR": {
		ICAO:      "99OR",
		Name:      "Lazy F Ranch Airport",
		City:      "Wallowa",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3710,
		Latitude:  45.6506996155,
		Longitude: -117.669998169,
		Timezone:  "America/Los_Angeles",
	},
	"99PA": {
		ICAO:      "99PA",
		Name:      "Nielsen Airport",
		City:      "Tioga",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1040,
		Latitude:  41.919796,
		Longitude: -77.14136,
		Timezone:  "America/New_York",
	},
	"99SC": {
		ICAO:      "99SC",
		Name:      "St. Mathews Airport",
		City:      "St. Mathews",
		State:     "South Carolina",
		Country:   "US",
		Elevation: 278,
		Latitude:  33.689167,
		Longitude: -80.688611,
		Timezone:  "America/New_York",
	},
	"99TE": {
		ICAO:      "99TE",
		Name:      "Pritchard Airport",
		City:      "Bells",
		State:     "Texas",
		Country:   "US",
		Elevation: 800,
		Latitude:  33.6058006287,
		Longitude: -96.4156036377,
		Timezone:  "America/Chicago",
	},
	"99TX": {
		ICAO:      "99TX",
		Name:      "Briggs Ranch Airport",
		City:      "Catarina",
		State:     "Texas",
		Country:   "US",
		Elevation: 520,
		Latitude:  28.3003005981,
		Longitude: -99.5670013428,
		Timezone:  "America/Chicago",
	},
	"99UT": {
		ICAO:      "99UT",
		Name:      "Sundog Airport",
		City:      "Jensen",
		State:     "Utah",
		Country:   "US",
		Elevation: 4900,
		Latitude:  40.3506011963,
		Longitude: -109.4010009766,
		Timezone:  "America/Denver",
	},
	"99VA": {
		ICAO:      "99VA",
		Name:      "Anderson Airport",
		City:      "Scottsville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 450,
		Latitude:  37.7999992371,
		Longitude: -78.4666976929,
		Timezone:  "America/New_York",
	},
	"99WI": {
		ICAO:      "99WI",
		Name:      "Foscoro Airport",
		City:      "Algoma",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 680,
		Latitude:  44.6808013916,
		Longitude: -87.3805999756,
		Timezone:  "America/Chicago",
	},
	"99WY": {
		ICAO:      "99WY",
		Name:      "Xingu Airstrip",
		City:      "Dayton",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4340,
		Latitude:  44.8457984924,
		Longitude: -107.2770004272,
		Timezone:  "America/Denver",
	},
	"99XA": {
		ICAO:      "99XA",
		Name:      "Briggs / Skotz Airfield",
		State:     "Texas",
		Country:   "US",
		Elevation: 1200,
		Latitude:  30.8639760767,
		Longitude: -98.0037117004,
		Timezone:  "America/Chicago",
	},
	"99XS": {
		ICAO:      "99XS",
		Name:      "Sam Little International Airport",
		City:      "Weatherford",
		State:     "Texas",
		Country:   "US",
		Elevation: 900,
		Latitude:  32.6834983826,
		Longitude: -97.8056030273,
		Timezone:  "America/Chicago",
	},
	"9AK0": {
		ICAO:      "9AK0",
		Name:      "Sports Mans Paradise Airport",
		City:      "Tok",
		State:     "Alaska",
		Country:   "US",
		Elevation: 3120,
		Latitude:  62.5121002197,
		Longitude: -143.2350006104,
		Timezone:  "America/Anchorage",
	},
	"9AK2": {
		ICAO:      "9AK2",
		Name:      "Kako Airport",
		City:      "Russian Mission",
		State:     "Alaska",
		Country:   "US",
		Elevation: 300,
		Latitude:  61.89889908,
		Longitude: -161.4380035,
		Timezone:  "America/Anchorage",
	},
	"9AK3": {
		ICAO:      "9AK3",
		Name:      "Nikolai Creek Airport",
		City:      "Tyonek",
		State:     "Alaska",
		Country:   "US",
		Elevation: 30,
		Latitude:  61.0139007568,
		Longitude: -151.449005127,
		Timezone:  "America/Anchorage",
	},
	"9AK4": {
		ICAO:      "9AK4",
		Name:      "Arctic Angel Airport",
		City:      "Delta Junction",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1009,
		Latitude:  64.1373977661,
		Longitude: -145.8300018311,
		Timezone:  "America/Anchorage",
	},
	"9AK5": {
		ICAO:      "9AK5",
		Name:      "Sky Harbor Airport",
		City:      "Anchorage",
		State:     "Alaska",
		Country:   "US",
		Elevation: 340,
		Latitude:  61.1165008545,
		Longitude: -149.8190002441,
		Timezone:  "America/Anchorage",
	},
	"9AK6": {
		ICAO:      "9AK6",
		Name:      "Leisurewood Airstrip Owners Assoc Airport",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 366,
		Latitude:  61.6240997314,
		Longitude: -149.6479949951,
		Timezone:  "America/Anchorage",
	},
	"9AK7": {
		ICAO:      "9AK7",
		Name:      "Cubdivision Airport",
		City:      "Big Lake",
		State:     "Alaska",
		Country:   "US",
		Elevation: 250,
		Latitude:  61.5898017883,
		Longitude: -149.8170013428,
		Timezone:  "America/Anchorage",
	},
	"9AK8": {
		ICAO:      "9AK8",
		Name:      "Wolf Track Airport",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 90,
		Latitude:  61.6246986389,
		Longitude: -149.6820068359,
		Timezone:  "America/Anchorage",
	},
	"9AR0": {
		ICAO:      "9AR0",
		Name:      "Robertson Airport",
		City:      "Mc Crory",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 207,
		Latitude:  35.188999176,
		Longitude: -91.2064971924,
		Timezone:  "America/Chicago",
	},
	"9AR9": {
		ICAO:      "9AR9",
		Name:      "Henry Field",
		City:      "Cabot",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 250,
		Latitude:  34.887389,
		Longitude: -91.989486,
		Timezone:  "America/Chicago",
	},
	"9CA0": {
		ICAO:      "9CA0",
		Name:      "Psk Ranch Airport",
		City:      "Tehachapi",
		State:     "California",
		Country:   "US",
		Elevation: 3840,
		Latitude:  35.113899231,
		Longitude: -118.5979995728,
		Timezone:  "America/Los_Angeles",
	},
	"9CA6": {
		ICAO:      "9CA6",
		Name:      "North Valley Airport",
		City:      "Bishop",
		State:     "Nevada",
		Country:   "US",
		Elevation: 4960,
		Latitude:  37.8690986633,
		Longitude: -118.0950012207,
		Timezone:  "America/Los_Angeles",
	},
	"9CA7": {
		ICAO:      "9CA7",
		Name:      "Harris River Ranch Airport",
		City:      "Sanger",
		State:     "California",
		Country:   "US",
		Elevation: 450,
		Latitude:  36.76720047,
		Longitude: -119.4359970093,
		Timezone:  "America/Los_Angeles",
	},
	"9CL0": {
		ICAO:      "9CL0",
		Name:      "Turlock Airpark",
		City:      "Turlock",
		State:     "California",
		Country:   "US",
		Elevation: 100,
		Latitude:  37.4707984924,
		Longitude: -120.84400177,
		Timezone:  "America/Los_Angeles",
	},
	"9CL1": {
		ICAO:      "9CL1",
		Name:      "Johnson Ranch Airport",
		City:      "Le Grand",
		State:     "California",
		Country:   "US",
		Elevation: 240,
		Latitude:  37.1911010742,
		Longitude: -120.2760009766,
		Timezone:  "America/Los_Angeles",
	},
	"9CL2": {
		ICAO:      "9CL2",
		Name:      "Christensen Ranch Airport",
		City:      "Hollister",
		State:     "California",
		Country:   "US",
		Elevation: 380,
		Latitude:  36.8716011047,
		Longitude: -121.3369979858,
		Timezone:  "America/Los_Angeles",
	},
	"9CL3": {
		ICAO:      "9CL3",
		Name:      "Likely Airport",
		City:      "Likely",
		State:     "California",
		Country:   "US",
		Elevation: 4420,
		Latitude:  41.2402000427,
		Longitude: -120.5260009766,
		Timezone:  "America/Los_Angeles",
	},
	"9CL4": {
		ICAO:      "9CL4",
		Name:      "Hart Mine Airport",
		City:      "Ivanpah",
		State:     "California",
		Country:   "US",
		Elevation: 4620,
		Latitude:  35.2832984924,
		Longitude: -115.1920013428,
		Timezone:  "America/Los_Angeles",
	},
	"9CL5": {
		ICAO:      "9CL5",
		Name:      "Blair Strip",
		City:      "Hanford",
		State:     "California",
		Country:   "US",
		Elevation: 231,
		Latitude:  36.2751998901,
		Longitude: -119.6409988403,
		Timezone:  "America/Los_Angeles",
	},
	"9CL7": {
		ICAO:      "9CL7",
		Name:      "Old Aerodrome",
		City:      "Marysville",
		State:     "California",
		Country:   "US",
		Elevation: 700,
		Latitude:  39.3249015808,
		Longitude: -121.3590011597,
		Timezone:  "America/Los_Angeles",
	},
	"9CL9": {
		ICAO:      "9CL9",
		Name:      "Spezia Airport",
		City:      "Walnut Grove",
		State:     "California",
		Country:   "US",
		Latitude:  38.2165985107,
		Longitude: -121.533996582,
		Timezone:  "America/Los_Angeles",
	},
	"9CO2": {
		ICAO:      "9CO2",
		Name:      "Van Slyke Field",
		City:      "Watkins",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5400,
		Latitude:  39.69609833,
		Longitude: -104.4759979,
		Timezone:  "America/Denver",
	},
	"9CO3": {
		ICAO:      "9CO3",
		Name:      "Hubbard Airport",
		City:      "Gateway",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4670,
		Latitude:  38.6833000183,
		Longitude: -108.9840011597,
		Timezone:  "America/Denver",
	},
	"9CO5": {
		ICAO:      "9CO5",
		Name:      "Scherler Private Airstrip",
		City:      "Brandon",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4126,
		Latitude:  38.5766983032,
		Longitude: -102.43699646,
		Timezone:  "America/Denver",
	},
	"9CO6": {
		ICAO:      "9CO6",
		Name:      "D Bar D Airport",
		City:      "Elizabeth",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6752,
		Latitude:  39.2812995911,
		Longitude: -104.5729980469,
		Timezone:  "America/Denver",
	},
	"9CO7": {
		ICAO:      "9CO7",
		Name:      "Beaugh Airport",
		City:      "Kersey",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4770,
		Latitude:  40.3405990601,
		Longitude: -104.5729980469,
		Timezone:  "America/Denver",
	},
	"9CO8": {
		ICAO:      "9CO8",
		Name:      "Herrick Airport",
		City:      "Kiowa",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6000,
		Latitude:  39.2793,
		Longitude: -104.34315,
		Timezone:  "America/Denver",
	},
	"9CO9": {
		ICAO:      "9CO9",
		Name:      "Phylcon Ranch Airport",
		City:      "Mesa",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6585,
		Latitude:  39.1291007996,
		Longitude: -108.141998291,
		Timezone:  "America/Denver",
	},
	"9FD2": {
		ICAO:      "9FD2",
		Name:      "Blackwater Creek Ultralightport",
		City:      "Plant City",
		State:     "Florida",
		Country:   "US",
		Elevation: 80,
		Latitude:  28.1369991302,
		Longitude: -82.1464996338,
		Timezone:  "America/New_York",
	},
	"9FD5": {
		ICAO:      "9FD5",
		Name:      "Thompson's Goinbroke Aero Ranch Airport",
		City:      "Citra",
		State:     "Florida",
		Country:   "US",
		Elevation: 85,
		Latitude:  29.3866004944,
		Longitude: -82.1428985596,
		Timezone:  "America/New_York",
	},
	"9FD7": {
		ICAO:      "9FD7",
		Name:      "Fort Atkinson Plantation Airpark",
		City:      "Day",
		State:     "Florida",
		Country:   "US",
		Elevation: 90,
		Latitude:  30.196225,
		Longitude: -83.309044,
		Timezone:  "America/New_York",
	},
	"9FD9": {
		ICAO:      "9FD9",
		Name:      "Buddys Ag Service Airport",
		City:      "Mc Alpin",
		State:     "Florida",
		Country:   "US",
		Elevation: 100,
		Latitude:  30.1224002838,
		Longitude: -82.9679031372,
		Timezone:  "America/New_York",
	},
	"9FL0": {
		ICAO:      "9FL0",
		Name:      "Suwannee Belle Airport",
		City:      "Live Oak",
		State:     "Florida",
		Country:   "US",
		Elevation: 50,
		Latitude:  30.0923995972,
		Longitude: -83.0854034424,
		Timezone:  "America/New_York",
	},
	"9FL1": {
		ICAO:      "9FL1",
		Name:      "Flying W Airranch Airport",
		City:      "Bushnell",
		State:     "Florida",
		Country:   "US",
		Elevation: 58,
		Latitude:  28.683599472,
		Longitude: -82.1576004028,
		Timezone:  "America/New_York",
	},
	"9FL2": {
		ICAO:      "9FL2",
		Name:      "Saranac Farm Airport",
		City:      "Havana",
		State:     "Florida",
		Country:   "US",
		Elevation: 230,
		Latitude:  30.6525001526,
		Longitude: -84.4747009277,
		Timezone:  "America/New_York",
	},
	"9FL5": {
		ICAO:      "9FL5",
		Name:      "Shady Bend Airport",
		City:      "Bell",
		State:     "Florida",
		Country:   "US",
		Elevation: 25,
		Latitude:  29.8141002655,
		Longitude: -82.9256973267,
		Timezone:  "America/New_York",
	},
	"9FL7": {
		ICAO:      "9FL7",
		Name:      "Oasis Ranger Station-U.S. Government Airport",
		City:      "Everglades City",
		State:     "Florida",
		Country:   "US",
		Elevation: 10,
		Latitude:  25.8572998047,
		Longitude: -81.0345001221,
		Timezone:  "America/New_York",
	},
	"9FL8": {
		ICAO:      "9FL8",
		Name:      "Finlayson Farm Airport",
		City:      "Greenville",
		State:     "Florida",
		Country:   "US",
		Elevation: 150,
		Latitude:  30.6166000366,
		Longitude: -83.6523971558,
		Timezone:  "America/New_York",
	},
	"9GA0": {
		ICAO:      "9GA0",
		Name:      "Brown Field",
		City:      "Zebulon",
		State:     "Georgia",
		Country:   "US",
		Elevation: 864,
		Latitude:  33.0570983887,
		Longitude: -84.3423995972,
		Timezone:  "America/New_York",
	},
	"9GA1": {
		ICAO:      "9GA1",
		Name:      "Briar Patch Airport",
		City:      "Clyo",
		State:     "Georgia",
		Country:   "US",
		Elevation: 100,
		Latitude:  32.4488983154,
		Longitude: -81.3368988037,
		Timezone:  "America/New_York",
	},
	"9GA2": {
		ICAO:      "9GA2",
		Name:      "Berg Park Aerodrome",
		City:      "Midway",
		State:     "Georgia",
		Country:   "US",
		Elevation: 15,
		Latitude:  31.8025,
		Longitude: -81.3975,
		Timezone:  "America/New_York",
	},
	"9GA3": {
		ICAO:      "9GA3",
		Name:      "Young Airport",
		City:      "Dawson",
		State:     "Georgia",
		Country:   "US",
		Elevation: 401,
		Latitude:  31.8395996094,
		Longitude: -84.4815979004,
		Timezone:  "America/New_York",
	},
	"9GA5": {
		ICAO:      "9GA5",
		Name:      "Lowell Field",
		City:      "Americus",
		State:     "Georgia",
		Country:   "US",
		Elevation: 475,
		Latitude:  32.0582008362,
		Longitude: -84.2901992798,
		Timezone:  "America/New_York",
	},
	"9GA6": {
		ICAO:      "9GA6",
		Name:      "Brookline - Meadowmere Airport",
		City:      "Milledgeville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 500,
		Latitude:  33.0978012085,
		Longitude: -83.4047012329,
		Timezone:  "America/New_York",
	},
	"9GA8": {
		ICAO:      "9GA8",
		Name:      "Chinaberry Ranch Airport",
		City:      "East Dublin",
		State:     "Georgia",
		Country:   "US",
		Elevation: 295,
		Latitude:  32.6935997009,
		Longitude: -82.8989028931,
		Timezone:  "America/New_York",
	},
	"9GA9": {
		ICAO:      "9GA9",
		Name:      "Deerfield Landing Airport",
		City:      "Eatonton",
		State:     "Georgia",
		Country:   "US",
		Elevation: 445,
		Latitude:  33.2181015015,
		Longitude: -83.317199707,
		Timezone:  "America/New_York",
	},
	"9GE7": {
		ICAO:      "9GE7",
		Name:      "Neely Farms Airport",
		City:      "Covington",
		State:     "Georgia",
		Country:   "US",
		Elevation: 720,
		Latitude:  33.531389,
		Longitude: -83.791944,
		Timezone:  "America/New_York",
	},
	"9II0": {
		ICAO:      "9II0",
		Name:      "Shaffer Airport",
		City:      "Merriam",
		State:     "Indiana",
		Country:   "US",
		Elevation: 940,
		Latitude:  41.2825012207,
		Longitude: -85.4385986328,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"9II1": {
		ICAO:      "9II1",
		Name:      "Short Stop Airport",
		City:      "Hatfield",
		State:     "Indiana",
		Country:   "US",
		Elevation: 395,
		Latitude:  37.8997001648,
		Longitude: -87.2177963257,
		Timezone:  "America/Chicago",
	},
	"9II2": {
		ICAO:      "9II2",
		Name:      "Nelund Field",
		City:      "Mishawaka",
		State:     "Indiana",
		Country:   "US",
		Elevation: 750,
		Latitude:  41.6875,
		Longitude: -86.1549987793,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"9II3": {
		ICAO:      "9II3",
		Name:      "Dillon Airport",
		City:      "North Liberty",
		State:     "Indiana",
		Country:   "US",
		Elevation: 700,
		Latitude:  41.5625,
		Longitude: -86.4721984863,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"9II4": {
		ICAO:      "9II4",
		Name:      "Cruzan Field",
		City:      "Arcadia",
		State:     "Indiana",
		Country:   "US",
		Elevation: 845,
		Latitude:  40.1725006104,
		Longitude: -85.9447021484,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"9II8": {
		ICAO:      "9II8",
		Name:      "Francis Airport",
		City:      "Holton",
		State:     "Indiana",
		Country:   "US",
		Elevation: 920,
		Latitude:  39.086101532,
		Longitude: -85.3711013794,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"9IL0": {
		ICAO:      "9IL0",
		Name:      "Villiger Restricted Landing Area",
		City:      "Henry",
		State:     "Illinois",
		Country:   "US",
		Elevation: 500,
		Latitude:  41.0945014954,
		Longitude: -89.3861999512,
		Timezone:  "America/Chicago",
	},
	"9IL2": {
		ICAO:      "9IL2",
		Name:      "Routh Airport",
		City:      "St Joseph",
		State:     "Illinois",
		Country:   "US",
		Elevation: 668,
		Latitude:  40.0649986267,
		Longitude: -88.0263977051,
		Timezone:  "America/Chicago",
	},
	"9IL3": {
		ICAO:      "9IL3",
		Name:      "Wm Quinton Restricted Landing Area",
		City:      "Heyworth",
		State:     "Illinois",
		Country:   "US",
		Elevation: 715,
		Latitude:  40.2820014954,
		Longitude: -89.029296875,
		Timezone:  "America/Chicago",
	},
	"9IL4": {
		ICAO:      "9IL4",
		Name:      "Thompson Airport",
		City:      "Steward",
		State:     "Illinois",
		Country:   "US",
		Elevation: 790,
		Latitude:  41.8072013855,
		Longitude: -89.0481033325,
		Timezone:  "America/Chicago",
	},
	"9IL6": {
		ICAO:      "9IL6",
		Name:      "Weishaupt Airport",
		City:      "Mackinaw",
		State:     "Illinois",
		Country:   "US",
		Elevation: 650,
		Latitude:  40.5130996704,
		Longitude: -89.3673019409,
		Timezone:  "America/Chicago",
	},
	"9IL7": {
		ICAO:      "9IL7",
		Name:      "Black Airport",
		City:      "Hillsdale",
		State:     "Illinois",
		Country:   "US",
		Elevation: 580,
		Latitude:  41.5097999573,
		Longitude: -90.1820983887,
		Timezone:  "America/Chicago",
	},
	"9IN2": {
		ICAO:      "9IN2",
		Name:      "New Liberty Field",
		City:      "New Liberty",
		State:     "Indiana",
		Country:   "US",
		Elevation: 810,
		Latitude:  38.5583000183,
		Longitude: -85.8653030396,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"9IN3": {
		ICAO:      "9IN3",
		Name:      "Johnsons Strawberry Farm Airport",
		City:      "Hobart",
		State:     "Indiana",
		Country:   "US",
		Elevation: 620,
		Latitude:  41.5569000244,
		Longitude: -87.2264022827,
		Timezone:  "America/Chicago",
	},
	"9IN4": {
		ICAO:      "9IN4",
		Name:      "Bandmill Field",
		City:      "Decker",
		State:     "Indiana",
		Country:   "US",
		Elevation: 400,
		Latitude:  38.534198761,
		Longitude: -87.5919036865,
		Timezone:  "America/Chicago",
	},
	"9IN6": {
		ICAO:      "9IN6",
		Name:      "Garrard Airport",
		City:      "Montezuma",
		State:     "Indiana",
		Country:   "US",
		Elevation: 550,
		Latitude:  39.840801239,
		Longitude: -87.3394012451,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"9IN7": {
		ICAO:      "9IN7",
		Name:      "Zupancic Field",
		City:      "Morgantown",
		State:     "Indiana",
		Country:   "US",
		Elevation: 660,
		Latitude:  39.3546981812,
		Longitude: -86.3069000244,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"9IN8": {
		ICAO:      "9IN8",
		Name:      "Green Field",
		City:      "Churubusco",
		State:     "Indiana",
		Country:   "US",
		Elevation: 890,
		Latitude:  41.2291984558,
		Longitude: -85.3581008911,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"9IN9": {
		ICAO:      "9IN9",
		Name:      "Gessie Airport",
		City:      "Gessie",
		State:     "Indiana",
		Country:   "US",
		Elevation: 635,
		Latitude:  40.0778007507,
		Longitude: -87.515296936,
		Timezone:  "America/Chicago",
	},
	"9IS0": {
		ICAO:      "9IS0",
		Name:      "Cedar Ridge Airport",
		City:      "Nauvoo",
		State:     "Illinois",
		Country:   "US",
		Elevation: 680,
		Latitude:  40.5430984497,
		Longitude: -91.3307037354,
		Timezone:  "America/Chicago",
	},
	"9IS2": {
		ICAO:      "9IS2",
		Name:      "Schertz Field",
		City:      "Gibson City",
		State:     "Illinois",
		Country:   "US",
		Elevation: 758,
		Latitude:  40.4822006226,
		Longitude: -88.2676010132,
		Timezone:  "America/Chicago",
	},
	"9IS5": {
		ICAO:      "9IS5",
		Name:      "J & Y Ultralightport",
		City:      "Dixon",
		State:     "Illinois",
		Country:   "US",
		Elevation: 786,
		Latitude:  41.8581008911,
		Longitude: -89.4073028564,
		Timezone:  "America/Chicago",
	},
	"9IS9": {
		ICAO:      "9IS9",
		Name:      "Lytleville Orchard Airport",
		City:      "Heyworth",
		State:     "Illinois",
		Country:   "US",
		Elevation: 785,
		Latitude:  40.3344993591,
		Longitude: -88.9553985596,
		Timezone:  "America/Chicago",
	},
	"9KS1": {
		ICAO:      "9KS1",
		Name:      "Hartland Airport",
		City:      "Wellsville",
		State:     "Kansas",
		Country:   "US",
		Elevation: 993,
		Latitude:  38.6594444,
		Longitude: -95.0922222,
		Timezone:  "America/Chicago",
	},
	"9KS4": {
		ICAO:      "9KS4",
		Name:      "Rose Port Airport",
		City:      "Ionia",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1500,
		Latitude:  39.6363983154,
		Longitude: -98.3598022461,
		Timezone:  "America/Chicago",
	},
	"9KS5": {
		ICAO:      "9KS5",
		Name:      "Rush Field",
		City:      "Geneseo",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1675,
		Latitude:  38.5778007507,
		Longitude: -98.1003036499,
		Timezone:  "America/Chicago",
	},
	"9KS6": {
		ICAO:      "9KS6",
		Name:      "Lundgren Hereford Ranch Airport",
		City:      "Gove",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2600,
		Latitude:  38.8130989075,
		Longitude: -100.5709991455,
		Timezone:  "America/Chicago",
	},
	"9KS7": {
		ICAO:      "9KS7",
		Name:      "Miller Airport",
		City:      "Harris",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1000,
		Latitude:  38.2778015137,
		Longitude: -95.4432983398,
		Timezone:  "America/Chicago",
	},
	"9KS9": {
		ICAO:      "9KS9",
		Name:      "Dmh Airport",
		City:      "Salina",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1272,
		Latitude:  38.7907981873,
		Longitude: -97.6183013916,
		Timezone:  "America/Chicago",
	},
	"9KY9": {
		ICAO:      "9KY9",
		Name:      "Paintsville-Prestonsburg-Combs Field",
		City:      "Paintsville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 624,
		Latitude:  37.7458992004,
		Longitude: -82.7790985107,
		Timezone:  "America/New_York",
	},
	"9LA1": {
		ICAO:      "9LA1",
		Name:      "Turkey Creek Lake Airport",
		City:      "Wisner",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 67,
		Latitude:  31.9067993164,
		Longitude: -91.7593002319,
		Timezone:  "America/Chicago",
	},
	"9LA2": {
		ICAO:      "9LA2",
		Name:      "Norris Airstrip",
		City:      "Newellton",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 70,
		Latitude:  32.1403999329,
		Longitude: -91.4112014771,
		Timezone:  "America/Chicago",
	},
	"9LA6": {
		ICAO:      "9LA6",
		Name:      "Chandler Airport",
		City:      "Alexandria",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 82,
		Latitude:  31.3068008423,
		Longitude: -92.6115036011,
		Timezone:  "America/Chicago",
	},
	"9LA9": {
		ICAO:      "9LA9",
		Name:      "Skyline Airpark",
		City:      "Amite",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 200,
		Latitude:  30.733499527,
		Longitude: -90.4501037598,
		Timezone:  "America/Chicago",
	},
	"9LL0": {
		ICAO:      "9LL0",
		Name:      "Krutmeier Airport",
		City:      "Payson",
		State:     "Illinois",
		Country:   "US",
		Elevation: 715,
		Latitude:  39.8199996948,
		Longitude: -91.2210006714,
		Timezone:  "America/Chicago",
	},
	"9LL2": {
		ICAO:      "9LL2",
		Name:      "Hepp Airport",
		City:      "Cutler",
		State:     "Illinois",
		Country:   "US",
		Elevation: 505,
		Latitude:  38.0336990356,
		Longitude: -89.5792999268,
		Timezone:  "America/Chicago",
	},
	"9LL3": {
		ICAO:      "9LL3",
		Name:      "John R Reed Airport",
		City:      "Ashmore",
		State:     "Illinois",
		Country:   "US",
		Elevation: 699,
		Latitude:  39.5491981506,
		Longitude: -88.0323028564,
		Timezone:  "America/Chicago",
	},
	"9LL5": {
		ICAO:      "9LL5",
		Name:      "Tommy's Airpark",
		City:      "Edinburg",
		State:     "Illinois",
		Country:   "US",
		Elevation: 595,
		Latitude:  39.6511001587,
		Longitude: -89.4586029053,
		Timezone:  "America/Chicago",
	},
	"9LL9": {
		ICAO:      "9LL9",
		Name:      "Townley Farms Airport",
		City:      "Mattoon",
		State:     "Illinois",
		Country:   "US",
		Elevation: 700,
		Latitude:  39.4846992493,
		Longitude: -88.4693984985,
		Timezone:  "America/Chicago",
	},
	"9LS1": {
		ICAO:      "9LS1",
		Name:      "Bayou Meadows Airport",
		City:      "Darnell",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 85,
		Latitude:  32.7000999451,
		Longitude: -91.3833999634,
		Timezone:  "America/Chicago",
	},
	"9LS5": {
		ICAO:      "9LS5",
		Name:      "Red Beard Dusting Service Airport",
		City:      "Tallulah",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 86,
		Latitude:  32.2835006714,
		Longitude: -91.1278991699,
		Timezone:  "America/Chicago",
	},
	"9LS9": {
		ICAO:      "9LS9",
		Name:      "4B Ranch Airport",
		City:      "Tendal",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 80,
		Latitude:  32.4099006653,
		Longitude: -91.3487014771,
		Timezone:  "America/Chicago",
	},
	"9MD9": {
		ICAO:      "9MD9",
		Name:      "G W Farm Airport",
		City:      "Denton",
		State:     "Delaware",
		Country:   "US",
		Elevation: 50,
		Latitude:  38.8520011902,
		Longitude: -75.6893005371,
		Timezone:  "America/New_York",
	},
	"9MI2": {
		ICAO:      "9MI2",
		Name:      "Newport Woods Airport",
		City:      "Newport",
		State:     "Michigan",
		Country:   "US",
		Elevation: 593,
		Latitude:  41.9878997803,
		Longitude: -83.3076019287,
		Timezone:  "America/Detroit",
	},
	"9MI5": {
		ICAO:      "9MI5",
		Name:      "Grass Roots STOLport",
		City:      "Millington",
		State:     "Michigan",
		Country:   "US",
		Elevation: 780,
		Latitude:  43.2378005981,
		Longitude: -83.5243988037,
		Timezone:  "America/Detroit",
	},
	"9MI6": {
		ICAO:      "9MI6",
		Name:      "Trollman's Field",
		City:      "Fenton",
		State:     "Michigan",
		Country:   "US",
		Elevation: 990,
		Latitude:  42.739200592,
		Longitude: -83.7549972534,
		Timezone:  "America/Detroit",
	},
	"9MN1": {
		ICAO:      "9MN1",
		Name:      "Troll Farm Airport",
		City:      "Cambridge",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 960,
		Latitude:  45.6115989685,
		Longitude: -93.1974029541,
		Timezone:  "America/Chicago",
	},
	"9MN3": {
		ICAO:      "9MN3",
		Name:      "Barnesville Municipal Airport",
		City:      "Barnesville",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 980,
		Latitude:  46.6651992798,
		Longitude: -96.4412002563,
		Timezone:  "America/Chicago",
	},
	"9MN5": {
		ICAO:      "9MN5",
		Name:      "Janssen Airport",
		City:      "Barnesville",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1100,
		Latitude:  46.7118988037,
		Longitude: -96.3515014648,
		Timezone:  "America/Chicago",
	},
	"9MN6": {
		ICAO:      "9MN6",
		Name:      "Merill L Harris Field",
		City:      "Jordan",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 800,
		Latitude:  44.6433982849,
		Longitude: -93.6996994019,
		Timezone:  "America/Chicago",
	},
	"9MN9": {
		ICAO:      "9MN9",
		Name:      "Tofte Airport",
		City:      "Tofte",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 790,
		Latitude:  47.5880012512,
		Longitude: -90.8235015869,
		Timezone:  "America/Chicago",
	},
	"9MO3": {
		ICAO:      "9MO3",
		Name:      "Lawlor-Justus Airport",
		City:      "Faucett",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1100,
		Latitude:  39.6417007446,
		Longitude: -94.8135986328,
		Timezone:  "America/Chicago",
	},
	"9MO4": {
		ICAO:      "9MO4",
		Name:      "Worth Airport",
		City:      "Savannah",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1130,
		Latitude:  39.9791984558,
		Longitude: -94.8739013672,
		Timezone:  "America/Chicago",
	},
	"9MO5": {
		ICAO:      "9MO5",
		Name:      "Hindman Airport",
		City:      "Sparta",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1380,
		Latitude:  37.06119919,
		Longitude: -93.09929657,
		Timezone:  "America/Chicago",
	},
	"9MO6": {
		ICAO:      "9MO6",
		Name:      "Ivy Bend Airport",
		City:      "Stover",
		State:     "Missouri",
		Country:   "US",
		Elevation: 690,
		Latitude:  38.1930999756,
		Longitude: -92.992401123,
		Timezone:  "America/Chicago",
	},
	"9MO9": {
		ICAO:      "9MO9",
		Name:      "Eagle Lodge Airport",
		City:      "Sumner",
		State:     "Missouri",
		Country:   "US",
		Elevation: 650,
		Latitude:  39.5499992371,
		Longitude: -93.2501983643,
		Timezone:  "America/Chicago",
	},
	"9MT8": {
		ICAO:      "9MT8",
		Name:      "Monger Airport",
		City:      "Bozeman",
		State:     "Montana",
		Country:   "US",
		Elevation: 4550,
		Latitude:  45.7580986023,
		Longitude: -111.0830001831,
		Timezone:  "America/Denver",
	},
	"9MT9": {
		ICAO:      "9MT9",
		Name:      "Mc Kenna Airport",
		City:      "Belgrade",
		State:     "Montana",
		Country:   "US",
		Elevation: 4450,
		Latitude:  45.7907981873,
		Longitude: -111.125,
		Timezone:  "America/Denver",
	},
	"9NA4": {
		ICAO:      "9NA4",
		Name:      "Bodmer Airport",
		City:      "Kenmare",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 1885,
		Latitude:  48.6660995483,
		Longitude: -101.8909988403,
		Timezone:  "America/Chicago",
	},
	"9NC0": {
		ICAO:      "9NC0",
		Name:      "Raleigh East Airport",
		City:      "Knightdale",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 313,
		Latitude:  35.7976989746,
		Longitude: -78.43699646,
		Timezone:  "America/New_York",
	},
	"9NC2": {
		ICAO:      "9NC2",
		Name:      "Flyers Airpark",
		City:      "Linden",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 115,
		Latitude:  35.2182006836,
		Longitude: -78.7964019775,
		Timezone:  "America/New_York",
	},
	"9NC3": {
		ICAO:      "9NC3",
		Name:      "Edwards Airport",
		City:      "Monroe",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 640,
		Latitude:  35.0708007813,
		Longitude: -80.5663986206,
		Timezone:  "America/New_York",
	},
	"9NC4": {
		ICAO:      "9NC4",
		Name:      "Jiles Field",
		City:      "Gates",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 75,
		Latitude:  36.5093994141,
		Longitude: -76.6641998291,
		Timezone:  "America/New_York",
	},
	"9NC6": {
		ICAO:      "9NC6",
		Name:      "Sandy Run Acres Airport",
		City:      "Maco",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 60,
		Latitude:  34.2748985291,
		Longitude: -78.1607971191,
		Timezone:  "America/New_York",
	},
	"9NC7": {
		ICAO:      "9NC7",
		Name:      "Willow Creek Airport",
		City:      "Mt Pleasant",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 572,
		Latitude:  35.3697013855,
		Longitude: -80.4400024414,
		Timezone:  "America/New_York",
	},
	"9NC8": {
		ICAO:      "9NC8",
		Name:      "Eagles Landing Airport",
		City:      "Pittsboro",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 430,
		Latitude:  35.6869010925,
		Longitude: -79.2188034058,
		Timezone:  "America/New_York",
	},
	"9NC9": {
		ICAO:      "9NC9",
		Name:      "Meylor Field",
		City:      "Greensboro",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 910,
		Latitude:  36.2100982666,
		Longitude: -79.8949966431,
		Timezone:  "America/New_York",
	},
	"9ND1": {
		ICAO:      "9ND1",
		Name:      "Elliott Farms Airport",
		City:      "Drayton",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 808,
		Latitude:  48.6479988098,
		Longitude: -97.2684020996,
		Timezone:  "America/Chicago",
	},
	"9ND8": {
		ICAO:      "9ND8",
		Name:      "Hinkle Airport",
		City:      "Cavalier",
		State:     "North Dakota",
		Country:   "US",
		Elevation: 908,
		Latitude:  48.7854995728,
		Longitude: -97.6723022461,
		Timezone:  "America/Chicago",
	},
	"9NE1": {
		ICAO:      "9NE1",
		Name:      "El-Co Airport",
		City:      "Brule",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3650,
		Latitude:  41.1444015503,
		Longitude: -101.9270019531,
		Timezone:  "America/Denver",
	},
	"9NE2": {
		ICAO:      "9NE2",
		Name:      "Folkerts Airport",
		City:      "Carleton",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1610,
		Latitude:  40.2916984558,
		Longitude: -97.7225036621,
		Timezone:  "America/Chicago",
	},
	"9NE3": {
		ICAO:      "9NE3",
		Name:      "Davis Ranch Airport",
		City:      "Hyannis",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3768,
		Latitude:  42.1875,
		Longitude: -101.7890014648,
		Timezone:  "America/Denver",
	},
	"9NE4": {
		ICAO:      "9NE4",
		Name:      "Johnston Field",
		City:      "Lindsay",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1800,
		Latitude:  41.7033004761,
		Longitude: -97.7444992065,
		Timezone:  "America/Chicago",
	},
	"9NE6": {
		ICAO:      "9NE6",
		Name:      "Bates Airpark",
		City:      "Omaha",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1250,
		Latitude:  41.3382987976,
		Longitude: -96.0880966187,
		Timezone:  "America/Chicago",
	},
	"9NE7": {
		ICAO:      "9NE7",
		Name:      "Bay Field",
		City:      "Stapleton",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3000,
		Latitude:  41.4333000183,
		Longitude: -100.4840011597,
		Timezone:  "America/Chicago",
	},
	"9NE8": {
		ICAO:      "9NE8",
		Name:      "Brosius Field",
		City:      "Stapleton",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3010,
		Latitude:  41.4016990662,
		Longitude: -100.4789962769,
		Timezone:  "America/Chicago",
	},
	"9NE9": {
		ICAO:      "9NE9",
		Name:      "Dog Leg Airport",
		City:      "Hebron",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1500,
		Latitude:  40.1944007874,
		Longitude: -97.5670013428,
		Timezone:  "America/Chicago",
	},
	"9NJ5": {
		ICAO:      "9NJ5",
		Name:      "Stallone Airport",
		City:      "Harrisonville",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 80,
		Latitude:  39.6800994873,
		Longitude: -75.2524032593,
		Timezone:  "America/New_York",
	},
	"9NJ6": {
		ICAO:      "9NJ6",
		Name:      "Halka Nurseries Airport",
		City:      "Fairton",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 10,
		Latitude:  39.3664016724,
		Longitude: -75.2683029175,
		Timezone:  "America/New_York",
	},
	"9NJ8": {
		ICAO:      "9NJ8",
		Name:      "High Meadow Farms Airport",
		City:      "Hope",
		State:     "New Jersey",
		Country:   "US",
		Elevation: 630,
		Latitude:  40.8959007263,
		Longitude: -74.9996032715,
		Timezone:  "America/New_York",
	},
	"9NR7": {
		ICAO:      "9NR7",
		Name:      "Triple F Airpark",
		City:      "Salemburg",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 148,
		Latitude:  35.018333,
		Longitude: -78.565833,
		Timezone:  "America/New_York",
	},
	"9NR8": {
		ICAO:      "9NR8",
		Name:      "Buie Field",
		City:      "Red Springs",
		State:     "North Carolina",
		Country:   "US",
		Elevation: 203,
		Latitude:  34.793333,
		Longitude: -79.193611,
		Timezone:  "America/New_York",
	},
	"9NY4": {
		ICAO:      "9NY4",
		Name:      "Kennedy Airfield",
		City:      "Kennedy",
		State:     "New York",
		Country:   "US",
		Elevation: 1760,
		Latitude:  42.1007995605,
		Longitude: -79.0818023682,
		Timezone:  "America/New_York",
	},
	"9NY6": {
		ICAO:      "9NY6",
		Name:      "Mason Airway Airport",
		City:      "Treadwell",
		State:     "New York",
		Country:   "US",
		Elevation: 2170,
		Latitude:  42.324798584,
		Longitude: -74.9862976074,
		Timezone:  "America/New_York",
	},
	"9NY7": {
		ICAO:      "9NY7",
		Name:      "Hart Airport",
		City:      "Perth",
		State:     "New York",
		Country:   "US",
		Elevation: 925,
		Latitude:  43.0329017639,
		Longitude: -74.1510009766,
		Timezone:  "America/New_York",
	},
	"9OA3": {
		ICAO:      "9OA3",
		Name:      "Bellville Dam Landing Strip",
		City:      "Long Bottom",
		State:     "Ohio",
		Country:   "US",
		Elevation: 591,
		Latitude:  39.0834007263,
		Longitude: -81.7776031494,
		Timezone:  "America/New_York",
	},
	"9OA4": {
		ICAO:      "9OA4",
		Name:      "Gilmer Airport",
		City:      "Lewisburg",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1045,
		Latitude:  39.8302993774,
		Longitude: -84.5096969604,
		Timezone:  "America/New_York",
	},
	"9OA5": {
		ICAO:      "9OA5",
		Name:      "Buckeye Executive Airport",
		City:      "Hebron",
		State:     "Ohio",
		Country:   "US",
		Elevation: 914,
		Latitude:  39.9623184204,
		Longitude: -82.5499801636,
		Timezone:  "America/New_York",
	},
	"9OA7": {
		ICAO:      "9OA7",
		Name:      "Jenkins Airport",
		City:      "Fremont",
		State:     "Ohio",
		Country:   "US",
		Elevation: 655,
		Latitude:  41.3097991943,
		Longitude: -83.1159973145,
		Timezone:  "America/New_York",
	},
	"9OA8": {
		ICAO:      "9OA8",
		Name:      "Crocker Airport",
		City:      "Columbia Station",
		State:     "Ohio",
		Country:   "US",
		Elevation: 830,
		Latitude:  41.2822990417,
		Longitude: -81.9489974976,
		Timezone:  "America/New_York",
	},
	"9OA9": {
		ICAO:      "9OA9",
		Name:      "Bender Airport",
		City:      "Bellville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1140,
		Latitude:  40.6213989258,
		Longitude: -82.5337982178,
		Timezone:  "America/New_York",
	},
	"9OH1": {
		ICAO:      "9OH1",
		Name:      "Haar Airport",
		City:      "Elmore",
		State:     "Ohio",
		Country:   "US",
		Elevation: 617,
		Latitude:  41.4542007446,
		Longitude: -83.2651977539,
		Timezone:  "America/New_York",
	},
	"9OH2": {
		ICAO:      "9OH2",
		Name:      "Toussaint Airpark",
		City:      "Elmore",
		State:     "Ohio",
		Country:   "US",
		Elevation: 610,
		Latitude:  41.5155982971,
		Longitude: -83.3058013916,
		Timezone:  "America/New_York",
	},
	"9OH4": {
		ICAO:      "9OH4",
		Name:      "Richards Airport",
		City:      "Elyria",
		State:     "Ohio",
		Country:   "US",
		Elevation: 690,
		Latitude:  41.4039001465,
		Longitude: -82.1064987183,
		Timezone:  "America/New_York",
	},
	"9OH6": {
		ICAO:      "9OH6",
		Name:      "Weaver Airport",
		City:      "Findlay",
		State:     "Ohio",
		Country:   "US",
		Elevation: 795,
		Latitude:  41.0353012085,
		Longitude: -83.5690002441,
		Timezone:  "America/New_York",
	},
	"9OH7": {
		ICAO:      "9OH7",
		Name:      "Lutz Airport",
		City:      "Findlay",
		State:     "Ohio",
		Country:   "US",
		Elevation: 808,
		Latitude:  40.9617004395,
		Longitude: -83.595199585,
		Timezone:  "America/New_York",
	},
	"9OH8": {
		ICAO:      "9OH8",
		Name:      "Ferrell Airport",
		City:      "Findlay",
		State:     "Ohio",
		Country:   "US",
		Elevation: 815,
		Latitude:  40.976398468,
		Longitude: -83.6416015625,
		Timezone:  "America/New_York",
	},
	"9OH9": {
		ICAO:      "9OH9",
		Name:      "Forest Field",
		City:      "Forest",
		State:     "Ohio",
		Country:   "US",
		Elevation: 890,
		Latitude:  40.8287010193,
		Longitude: -83.5138015747,
		Timezone:  "America/New_York",
	},
	"9OI3": {
		ICAO:      "9OI3",
		Name:      "Johns Landing Airport",
		City:      "Zanesville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1000,
		Latitude:  39.898601532,
		Longitude: -82.1102981567,
		Timezone:  "America/New_York",
	},
	"9OI5": {
		ICAO:      "9OI5",
		Name:      "Transportation Research Center of Ohio Airport",
		City:      "East Liberty",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1100,
		Latitude:  40.3081016541,
		Longitude: -83.5417022705,
		Timezone:  "America/New_York",
	},
	"9OI6": {
		ICAO:      "9OI6",
		Name:      "Nesta Airport",
		City:      "Loudenville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 950,
		Latitude:  40.6328010559,
		Longitude: -82.1768035889,
		Timezone:  "America/New_York",
	},
	"9OI7": {
		ICAO:      "9OI7",
		Name:      "Bayes Airport",
		City:      "Magnetic Springs",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1000,
		Latitude:  40.3400993347,
		Longitude: -83.2841033936,
		Timezone:  "America/New_York",
	},
	"9OI8": {
		ICAO:      "9OI8",
		Name:      "Brothers Aviation Airport",
		City:      "Old Washington",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1040,
		Latitude:  40.0322990417,
		Longitude: -81.4651031494,
		Timezone:  "America/New_York",
	},
	"9OK3": {
		ICAO:      "9OK3",
		Name:      "Flying Eagle Estates Airport",
		City:      "Bartlesville",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 690,
		Latitude:  36.7037010193,
		Longitude: -95.8404998779,
		Timezone:  "America/Chicago",
	},
	"9OK5": {
		ICAO:      "9OK5",
		Name:      "Diamond C Ranch Airport",
		City:      "Stratford",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1140,
		Latitude:  34.778301239,
		Longitude: -96.9274978638,
		Timezone:  "America/Chicago",
	},
	"9OK7": {
		ICAO:      "9OK7",
		Name:      "Cimarron Strip",
		City:      "Jennings",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 920,
		Latitude:  36.1847991943,
		Longitude: -96.5503005981,
		Timezone:  "America/Chicago",
	},
	"9OK9": {
		ICAO:      "9OK9",
		Name:      "Strang Airpark",
		City:      "Short",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 690,
		Latitude:  35.5444984436,
		Longitude: -94.4862976074,
		Timezone:  "America/Chicago",
	},
	"9OR0": {
		ICAO:      "9OR0",
		Name:      "Lafferty Field",
		City:      "Brownsville",
		State:     "Oregon",
		Country:   "US",
		Elevation: 360,
		Latitude:  44.3151016235,
		Longitude: -123.0250015259,
		Timezone:  "America/Los_Angeles",
	},
	"9OR1": {
		ICAO:      "9OR1",
		Name:      "Shaniko Ranch Airport",
		City:      "Shaniko",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3300,
		Latitude:  45.0028991699,
		Longitude: -120.7399978638,
		Timezone:  "America/Los_Angeles",
	},
	"9OR4": {
		ICAO:      "9OR4",
		Name:      "King's Airport",
		City:      "Milton/Freewater",
		State:     "Oregon",
		Country:   "US",
		Elevation: 950,
		Latitude:  45.9337005615,
		Longitude: -118.4479980469,
		Timezone:  "America/Los_Angeles",
	},
	"9OR8": {
		ICAO:      "9OR8",
		Name:      "Wooldridge Agstrip Airport",
		City:      "Albany",
		State:     "Oregon",
		Country:   "US",
		Elevation: 205,
		Latitude:  44.7326011658,
		Longitude: -123.0559997559,
		Timezone:  "America/Los_Angeles",
	},
	"9PA0": {
		ICAO:      "9PA0",
		Name:      "Lance Airport",
		City:      "Parker Ford",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 275,
		Latitude:  40.1873016357,
		Longitude: -75.5902023315,
		Timezone:  "America/New_York",
	},
	"9PA1": {
		ICAO:      "9PA1",
		Name:      "Carbondale-Clifford Airport",
		City:      "Carbondale",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1400,
		Latitude:  41.6542015076,
		Longitude: -75.5787963867,
		Timezone:  "America/New_York",
	},
	"9PA2": {
		ICAO:      "9PA2",
		Name:      "Old Plains Airport",
		City:      "Pennsburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 660,
		Latitude:  40.3945007324,
		Longitude: -75.4412994385,
		Timezone:  "America/New_York",
	},
	"9PA3": {
		ICAO:      "9PA3",
		Name:      "Meadow Strip Ultralightport",
		City:      "Richlandtown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 499,
		Latitude:  40.4757003784,
		Longitude: -75.3096008301,
		Timezone:  "America/New_York",
	},
	"9PA5": {
		ICAO:      "9PA5",
		Name:      "Solt Field",
		City:      "Leighton",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 520,
		Latitude:  40.7801017761,
		Longitude: -75.7163009644,
		Timezone:  "America/New_York",
	},
	"9PA7": {
		ICAO:      "9PA7",
		Name:      "Keystone Airport",
		City:      "Fort Hill",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 2531,
		Latitude:  39.7728004456,
		Longitude: -79.2110977173,
		Timezone:  "America/New_York",
	},
	"9PN1": {
		ICAO:      "9PN1",
		Name:      "Seitz Field",
		City:      "Perryopolis",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1240,
		Latitude:  40.0390014648,
		Longitude: -79.7900009155,
		Timezone:  "America/New_York",
	},
	"9PN7": {
		ICAO:      "9PN7",
		Name:      "Veit Airport",
		City:      "Wysox",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1360,
		Latitude:  41.8180999756,
		Longitude: -76.4126968384,
		Timezone:  "America/New_York",
	},
	"9PN8": {
		ICAO:      "9PN8",
		Name:      "Malinchak Private Airport",
		City:      "Carbondale",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1700,
		Latitude:  41.6250991821,
		Longitude: -75.5329971313,
		Timezone:  "America/New_York",
	},
	"9PS2": {
		ICAO:      "9PS2",
		Name:      "J T Willie Airport",
		City:      "Worthington",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1200,
		Latitude:  40.8087005615,
		Longitude: -79.6697998047,
		Timezone:  "America/New_York",
	},
	"9PS8": {
		ICAO:      "9PS8",
		Name:      "Manor Landing Airport",
		City:      "Gettysburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 560,
		Latitude:  39.8130989075,
		Longitude: -77.2949981689,
		Timezone:  "America/New_York",
	},
	"9SD7": {
		ICAO:      "9SD7",
		Name:      "Beaman Airport",
		City:      "Selby",
		State:     "South Dakota",
		Country:   "US",
		Elevation: 1960,
		Latitude:  45.4749984741,
		Longitude: -100.0350036621,
		Timezone:  "America/Chicago",
	},
	"9SD8": {
		ICAO:      "9SD8",
		Name:      "Oakleaf Airport",
		City:      "Hartford",
		State:     "South Dakota",
		Country:   "US",
		Elevation: 1625,
		Latitude:  43.5833015442,
		Longitude: -96.950302124,
		Timezone:  "America/Chicago",
	},
	"9SD9": {
		ICAO:      "9SD9",
		Name:      "Weelborg Airport",
		City:      "Dell Rapids",
		State:     "South Dakota",
		Country:   "US",
		Elevation: 1553,
		Latitude:  43.8065986633,
		Longitude: -96.7089004517,
		Timezone:  "America/Chicago",
	},
	"9TA1": {
		ICAO:      "9TA1",
		Name:      "Hilltop Ranch Airport",
		City:      "New Braunfels",
		State:     "Texas",
		Country:   "US",
		Elevation: 762,
		Latitude:  29.6585998535,
		Longitude: -98.137802124,
		Timezone:  "America/Chicago",
	},
	"9TA2": {
		ICAO:      "9TA2",
		Name:      "Urschel Ranch Airport",
		City:      "Canadian",
		State:     "Texas",
		Country:   "US",
		Elevation: 2340,
		Latitude:  35.9289016724,
		Longitude: -100.3069992065,
		Timezone:  "America/Chicago",
	},
	"9TA3": {
		ICAO:      "9TA3",
		Name:      "Rogers Airport",
		City:      "Crosby",
		State:     "Texas",
		Country:   "US",
		Elevation: 38,
		Latitude:  29.8729991913,
		Longitude: -94.9896011353,
		Timezone:  "America/Chicago",
	},
	"9TA8": {
		ICAO:      "9TA8",
		Name:      "Taylor Ranch Airport",
		City:      "Alto",
		State:     "Texas",
		Country:   "US",
		Elevation: 270,
		Latitude:  31.6840991974,
		Longitude: -94.9772033691,
		Timezone:  "America/Chicago",
	},
	"9TE0": {
		ICAO:      "9TE0",
		Name:      "Twin Acres Airport",
		City:      "Scurry",
		State:     "Texas",
		Country:   "US",
		Elevation: 340,
		Latitude:  32.3992004395,
		Longitude: -96.3835983276,
		Timezone:  "America/Chicago",
	},
	"9TE2": {
		ICAO:      "9TE2",
		Name:      "Wallace Ranch Airport",
		City:      "Comfort",
		State:     "Texas",
		Country:   "US",
		Elevation: 1510,
		Latitude:  30.0168991089,
		Longitude: -98.8503036499,
		Timezone:  "America/Chicago",
	},
	"9TE3": {
		ICAO:      "9TE3",
		Name:      "Thorp Airport",
		City:      "Lamesa",
		State:     "Texas",
		Country:   "US",
		Elevation: 2900,
		Latitude:  32.5778999329,
		Longitude: -101.9309997559,
		Timezone:  "America/Chicago",
	},
	"9TE4": {
		ICAO:      "9TE4",
		Name:      "Tanner's Airport",
		City:      "Port Lavaca",
		State:     "Texas",
		Country:   "US",
		Elevation: 15,
		Latitude:  28.5697002411,
		Longitude: -96.629699707,
		Timezone:  "America/Chicago",
	},
	"9TE5": {
		ICAO:      "9TE5",
		Name:      "Kalt Ranch Airport",
		City:      "Fulton",
		State:     "Texas",
		Country:   "US",
		Elevation: 15,
		Latitude:  28.154499054,
		Longitude: -96.9768981934,
		Timezone:  "America/Chicago",
	},
	"9TE6": {
		ICAO:      "9TE6",
		Name:      "Rocky Creek Ranch Airport",
		City:      "La Grange",
		State:     "Texas",
		Country:   "US",
		Elevation: 390,
		Latitude:  29.9169006348,
		Longitude: -96.8003005981,
		Timezone:  "America/Chicago",
	},
	"9TE7": {
		ICAO:      "9TE7",
		Name:      "Flying G H Ranch Airport",
		City:      "Paige",
		State:     "Texas",
		Country:   "US",
		Elevation: 455,
		Latitude:  30.245278,
		Longitude: -97.082778,
		Timezone:  "America/Chicago",
	},
	"9TN1": {
		ICAO:      "9TN1",
		Name:      "Versailles Aerodrome",
		City:      "Eagleville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 855,
		Latitude:  35.7299995422,
		Longitude: -86.5378036499,
		Timezone:  "America/Chicago",
	},
	"9TN2": {
		ICAO:      "9TN2",
		Name:      "Meadowlark Airport",
		City:      "Camden",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 460,
		Latitude:  36.163898468,
		Longitude: -88.2052993774,
		Timezone:  "America/Chicago",
	},
	"9TN3": {
		ICAO:      "9TN3",
		Name:      "Mehrhoff Field",
		City:      "Rossville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 426,
		Latitude:  35.1136016846,
		Longitude: -89.5243988037,
		Timezone:  "America/Chicago",
	},
	"9TN4": {
		ICAO:      "9TN4",
		Name:      "Foreman Field",
		City:      "Springfield",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 705,
		Latitude:  36.457698822,
		Longitude: -86.9459991455,
		Timezone:  "America/Chicago",
	},
	"9TN5": {
		ICAO:      "9TN5",
		Name:      "Askey Field",
		City:      "Culleoka",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 810,
		Latitude:  35.4583015442,
		Longitude: -86.9569015503,
		Timezone:  "America/Chicago",
	},
	"9TN7": {
		ICAO:      "9TN7",
		Name:      "Baskin Airport",
		City:      "Covington",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 400,
		Latitude:  35.5344009399,
		Longitude: -89.7350006104,
		Timezone:  "America/Chicago",
	},
	"9TN9": {
		ICAO:      "9TN9",
		Name:      "Toy Box Airport",
		City:      "Lebanon",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 630,
		Latitude:  36.0194015503,
		Longitude: -86.2881011963,
		Timezone:  "America/Chicago",
	},
	"9TS3": {
		ICAO:      "9TS3",
		Name:      "Simaron Ranch Airport",
		City:      "Waller",
		State:     "Texas",
		Country:   "US",
		Elevation: 310,
		Latitude:  30.1483001709,
		Longitude: -95.9669036865,
		Timezone:  "America/Chicago",
	},
	"9TS6": {
		ICAO:      "9TS6",
		Name:      "Goodlett Field",
		City:      "Alvarado",
		State:     "Texas",
		Country:   "US",
		Elevation: 690,
		Latitude:  32.3788986206,
		Longitude: -97.1782989502,
		Timezone:  "America/Chicago",
	},
	"9TX1": {
		ICAO:      "9TX1",
		Name:      "Bar S Ranch Airport",
		City:      "Giddings",
		State:     "Texas",
		Country:   "US",
		Elevation: 465,
		Latitude:  30.1432991028,
		Longitude: -96.8561019897,
		Timezone:  "America/Chicago",
	},
	"9TX2": {
		ICAO:      "9TX2",
		Name:      "Bennetts Airport",
		City:      "Crandall",
		State:     "Texas",
		Country:   "US",
		Elevation: 445,
		Latitude:  32.6371002197,
		Longitude: -96.4083023071,
		Timezone:  "America/Chicago",
	},
	"9TX3": {
		ICAO:      "9TX3",
		Name:      "Horan Airport",
		City:      "Plainview",
		State:     "Texas",
		Country:   "US",
		Elevation: 3339,
		Latitude:  34.1689987183,
		Longitude: -101.6350021362,
		Timezone:  "America/Chicago",
	},
	"9TX4": {
		ICAO:      "9TX4",
		Name:      "Flying H Ranch Airport",
		City:      "Liberty Hill",
		State:     "Texas",
		Country:   "US",
		Elevation: 1000,
		Latitude:  30.6595993042,
		Longitude: -97.9111022949,
		Timezone:  "America/Chicago",
	},
	"9TX5": {
		ICAO:      "9TX5",
		Name:      "Camp Bullis Als (Cals) Airport",
		City:      "San Antonio",
		State:     "Texas",
		Country:   "US",
		Elevation: 1158,
		Latitude:  29.74469948,
		Longitude: -98.53720093,
		Timezone:  "America/Chicago",
	},
	"9TX6": {
		ICAO:      "9TX6",
		Name:      "Beggs Ranch Airport",
		City:      "Post",
		State:     "Texas",
		Country:   "US",
		Elevation: 2355,
		Latitude:  33.1750984192,
		Longitude: -101.0240020752,
		Timezone:  "America/Chicago",
	},
	"9TX7": {
		ICAO:      "9TX7",
		Name:      "Hitex Private Airport",
		City:      "Pottsboro",
		State:     "Texas",
		Country:   "US",
		Elevation: 700,
		Latitude:  33.8418006897,
		Longitude: -96.6697006226,
		Timezone:  "America/Chicago",
	},
	"9TX9": {
		ICAO:      "9TX9",
		Name:      "Canon Ranch Airport",
		City:      "Sheffield",
		State:     "Texas",
		Country:   "US",
		Elevation: 2380,
		Latitude:  30.7481994629,
		Longitude: -101.9649963379,
		Timezone:  "America/Chicago",
	},
	"9VA0": {
		ICAO:      "9VA0",
		Name:      "Bath Alum Airport",
		City:      "Warm Springs",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1779,
		Latitude:  38.0526008606,
		Longitude: -79.7288970947,
		Timezone:  "America/New_York",
	},
	"9VA1": {
		ICAO:      "9VA1",
		Name:      "Holly Point Airport",
		City:      "Mathews",
		State:     "Virginia",
		Country:   "US",
		Elevation: 12,
		Latitude:  37.4053993225,
		Longitude: -76.3882980347,
		Timezone:  "America/New_York",
	},
	"9VA3": {
		ICAO:      "9VA3",
		Name:      "Crippen Creek Farm Airport",
		City:      "Parksley",
		State:     "Virginia",
		Country:   "US",
		Elevation: 6,
		Latitude:  37.7629013062,
		Longitude: -75.5718994141,
		Timezone:  "America/New_York",
	},
	"9VA4": {
		ICAO:      "9VA4",
		Name:      "Franwood Farms Inc. Airport",
		City:      "New Market",
		State:     "Virginia",
		Country:   "US",
		Elevation: 990,
		Latitude:  38.6908988953,
		Longitude: -78.6295013428,
		Timezone:  "America/New_York",
	},
	"9VG9": {
		ICAO:      "9VG9",
		Name:      "Jucapa Farms Airport",
		City:      "Winchester",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1160,
		Latitude:  39.3721008301,
		Longitude: -78.3044967651,
		Timezone:  "America/New_York",
	},
	"9WA3": {
		ICAO:      "9WA3",
		Name:      "River Bend Airport",
		City:      "River Bend Lodge",
		State:     "Washington",
		Country:   "US",
		Elevation: 2040,
		Latitude:  48.4696006775,
		Longitude: -117.2900009155,
		Timezone:  "America/Los_Angeles",
	},
	"9WA4": {
		ICAO:      "9WA4",
		Name:      "Piper Canyon Airport",
		City:      "Goldendale",
		State:     "Washington",
		Country:   "US",
		Elevation: 1610,
		Latitude:  45.8027992249,
		Longitude: -120.9560012817,
		Timezone:  "America/Los_Angeles",
	},
	"9WA7": {
		ICAO:      "9WA7",
		Name:      "Albritton Airport",
		City:      "Buckley",
		State:     "Washington",
		Country:   "US",
		Elevation: 530,
		Latitude:  47.2155990601,
		Longitude: -122.1330032349,
		Timezone:  "America/Los_Angeles",
	},
	"9WI1": {
		ICAO:      "9WI1",
		Name:      "Black Otter Airport",
		City:      "Hortonville",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 805,
		Latitude:  44.3293991089,
		Longitude: -88.6235961914,
		Timezone:  "America/Chicago",
	},
	"9WI2": {
		ICAO:      "9WI2",
		Name:      "Flying Dollar Ranch Airport",
		City:      "Maribel",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 840,
		Latitude:  44.2874984741,
		Longitude: -87.7958984375,
		Timezone:  "America/Chicago",
	},
	"9WI3": {
		ICAO:      "9WI3",
		Name:      "Buchholz Farm Airport",
		City:      "Morrison",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 955,
		Latitude:  44.2854995728,
		Longitude: -87.9868011475,
		Timezone:  "America/Chicago",
	},
	"9WI4": {
		ICAO:      "9WI4",
		Name:      "Faken Airport",
		City:      "New Berlin",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 904,
		Latitude:  42.9785995483,
		Longitude: -88.134803772,
		Timezone:  "America/Chicago",
	},
	"9WI5": {
		ICAO:      "9WI5",
		Name:      "Tamarack Airport",
		City:      "Palmyra",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 850,
		Latitude:  42.8283004761,
		Longitude: -88.5745010376,
		Timezone:  "America/Chicago",
	},
	"9WI6": {
		ICAO:      "9WI6",
		Name:      "Kitty Hawk Estates Airport",
		City:      "Polar",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1500,
		Latitude:  45.1040992737,
		Longitude: -89.0167999268,
		Timezone:  "America/Chicago",
	},
	"9WI8": {
		ICAO:      "9WI8",
		Name:      "Fletcher Airport",
		City:      "Spring Prairie",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 980,
		Latitude:  42.693901062,
		Longitude: -88.3793029785,
		Timezone:  "America/Chicago",
	},
	"9WI9": {
		ICAO:      "9WI9",
		Name:      "Carlson Airport",
		City:      "Superior",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 680,
		Latitude:  46.6208000183,
		Longitude: -92.0819015503,
		Timezone:  "America/Chicago",
	},
	"9WN1": {
		ICAO:      "9WN1",
		Name:      "Courtney Plummer Airport",
		City:      "Winneconne",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 758,
		Latitude:  44.1068992615,
		Longitude: -88.6839981079,
		Timezone:  "America/Chicago",
	},
	"9WN2": {
		ICAO:      "9WN2",
		Name:      "Voyager Village Airstrip",
		City:      "Webster",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1020,
		Latitude:  45.9665985107,
		Longitude: -92.1460037231,
		Timezone:  "America/Chicago",
	},
	"9WN3": {
		ICAO:      "9WN3",
		Name:      "Ozaukee Airport",
		City:      "Port Washington",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 800,
		Latitude:  43.4213981628,
		Longitude: -87.888999939,
		Timezone:  "America/Chicago",
	},
	"9WN4": {
		ICAO:      "9WN4",
		Name:      "Rag Wing Airport",
		City:      "Antigo",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1500,
		Latitude:  45.0839996338,
		Longitude: -89.029800415,
		Timezone:  "America/Chicago",
	},
	"9WN5": {
		ICAO:      "9WN5",
		Name:      "Lodi Lakeland Airport",
		City:      "Lodi",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 844,
		Latitude:  43.3333015442,
		Longitude: -89.5210037231,
		Timezone:  "America/Chicago",
	},
	"9WN9": {
		ICAO:      "9WN9",
		Name:      "Coloma Municipal Airport",
		City:      "Coloma",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1100,
		Latitude:  44.0600013733,
		Longitude: -89.5668029785,
		Timezone:  "America/Chicago",
	},
	"9WS2": {
		ICAO:      "9WS2",
		Name:      "Antique Aerodrome",
		City:      "De Pere",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 655,
		Latitude:  44.367639,
		Longitude: -88.158678,
		Timezone:  "America/Chicago",
	},
	"9XS0": {
		ICAO:      "9XS0",
		Name:      "Dobbs Ranch Airport",
		City:      "Yoakum",
		State:     "Texas",
		Country:   "US",
		Elevation: 170,
		Latitude:  29.0979995728,
		Longitude: -96.9552993774,
		Timezone:  "America/Chicago",
	},
	"9XS1": {
		ICAO:      "9XS1",
		Name:      "The Landing Airport",
		City:      "Hill",
		State:     "Texas",
		Country:   "US",
		Elevation: 570,
		Latitude:  31.8355998993,
		Longitude: -96.952796936,
		Timezone:  "America/Chicago",
	},
	"9XS2": {
		ICAO:      "9XS2",
		Name:      "Beaver Creek Airport",
		City:      "Sherman",
		State:     "Texas",
		Country:   "US",
		Elevation: 708,
		Latitude:  33.6542626352,
		Longitude: -96.7763596773,
		Timezone:  "America/Chicago",
	},
	"9XS3": {
		ICAO:      "9XS3",
		Name:      "Macy Ranch Airport",
		City:      "Post",
		State:     "Texas",
		Country:   "US",
		Elevation: 2870,
		Latitude:  33.1034011841,
		Longitude: -101.4649963379,
		Timezone:  "America/Chicago",
	},
	"9XS4": {
		ICAO:      "9XS4",
		Name:      "Mc Keon Aviation Airport",
		City:      "Denison",
		State:     "Texas",
		Country:   "US",
		Elevation: 750,
		Latitude:  33.7342987061,
		Longitude: -96.626701355,
		Timezone:  "America/Chicago",
	},
	"9XS5": {
		ICAO:      "9XS5",
		Name:      "Kitten Farm Private Airport",
		City:      "Slaton",
		State:     "Texas",
		Country:   "US",
		Elevation: 3050,
		Latitude:  33.3501014709,
		Longitude: -101.641998291,
		Timezone:  "America/Chicago",
	},
	"9XS6": {
		ICAO:      "9XS6",
		Name:      "Sudan Airport",
		City:      "Sudan",
		State:     "Texas",
		Country:   "US",
		Elevation: 3769,
		Latitude:  34.0531005859,
		Longitude: -102.5220031738,
		Timezone:  "America/Chicago",
	},
	"9XS7": {
		ICAO:      "9XS7",
		Name:      "Reeder Airport",
		City:      "Lone Oak",
		State:     "Texas",
		Country:   "US",
		Elevation: 530,
		Latitude:  32.947101593,
		Longitude: -95.9586029053,
		Timezone:  "America/Chicago",
	},
	"9XS9": {
		ICAO:      "9XS9",
		Name:      "Rancho Verde Airport",
		City:      "Brenham",
		State:     "Texas",
		Country:   "US",
		Elevation: 275,
		Latitude:  30.2833003998,
		Longitude: -96.4475021362,
		Timezone:  "America/Chicago",
	},
	"AA00": {
		ICAO:      "AA00",
		Name:      "Shump Airport",
		City:      "Talkeetna",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1670,
		Latitude:  62.51015,
		Longitude: -149.926717,
		Timezone:  "America/Anchorage",
	},
	"AA05": {
		ICAO:      "AA05",
		Name:      "Shelby Strip",
		City:      "Anchorage",
		State:     "Alaska",
		Country:   "US",
		Elevation: 40,
		Latitude:  61.507811,
		Longitude: -149.078481,
		Timezone:  "America/Anchorage",
	},
	"AA06": {
		ICAO:      "AA06",
		Name:      "Sixmile Lake Airport",
		City:      "Anchorage",
		State:     "Alaska",
		Country:   "US",
		Elevation: 85,
		Latitude:  61.289722,
		Longitude: -149.806111,
		Timezone:  "America/Anchorage",
	},
	"AA22": {
		ICAO:      "AA22",
		Name:      "Delta Daves Airport",
		City:      "Delta Junction",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1070,
		Latitude:  64.132858,
		Longitude: -145.804494,
		Timezone:  "America/Anchorage",
	},
	"AA30": {
		ICAO:      "AA30",
		Name:      "Trio Estates Airport",
		City:      "Houston",
		State:     "Alaska",
		Country:   "US",
		Elevation: 315,
		Latitude:  61.648189,
		Longitude: -149.870831,
		Timezone:  "America/Anchorage",
	},
	"AA33": {
		ICAO:      "AA33",
		Name:      "Gus Landing",
		City:      "Houston",
		State:     "Alaska",
		Country:   "US",
		Elevation: 120,
		Latitude:  61.60415,
		Longitude: -149.770119,
		Timezone:  "America/Anchorage",
	},
	"AA50": {
		ICAO:      "AA50",
		Name:      "Point Thomson Sea Ice Airstrip",
		City:      "Deadhorse",
		State:     "Alaska",
		Country:   "US",
		Latitude:  70.181144,
		Longitude: -146.35035,
		Timezone:  "America/Anchorage",
	},
	"AA76": {
		ICAO:      "AA76",
		Name:      "Timber Trails Airport",
		City:      "North Pole",
		State:     "Alaska",
		Country:   "US",
		Elevation: 480,
		Latitude:  64.769444,
		Longitude: -147.329444,
		Timezone:  "America/Anchorage",
	},
	"AAXX": {
		ICAO:      "AAXX",
		Name:      "Rothera Point Airport",
		City:      "Rothera Point",
		Country:   "AQ",
		Latitude:  -67.5669411575,
		Longitude: -68.1269931793,
		Timezone:  "Antarctica/Rothera",
	},
	"ADDE": {
		ICAO:      "ADDE",
		IATA:      "ALX",
		Name:      "Thomas C Russell Field",
		City:      "Alexander City",
		State:     "Alabama",
		Country:   "US",
		Elevation: 686,
		Latitude:  32.9146995544,
		Longitude: -85.9629974365,
		Timezone:  "America/Chicago",
	},
	"AGAF": {
		ICAO:      "AGAF",
		IATA:      "AFT",
		Name:      "Afutara Aerodrome",
		City:      "Bila",
		State:     "Malaita",
		Country:   "SB",
		Elevation: 23,
		Latitude:  -9.1913888889,
		Longitude: 160.948611111,
		Timezone:  "Pacific/Guadalcanal",
	},
	"AGAR": {
		ICAO:      "AGAR",
		IATA:      "RNA",
		Name:      "Ulawa Airport",
		City:      "Arona",
		Country:   "SB",
		Elevation: 40,
		Latitude:  -9.8605435826,
		Longitude: 161.979546547,
		Timezone:  "Pacific/Guadalcanal",
	},
	"AGAT": {
		ICAO:      "AGAT",
		IATA:      "ATD",
		Name:      "Uru Harbour Airport",
		City:      "Atoifi",
		Country:   "SB",
		Latitude:  -8.8733301163,
		Longitude: 161.0110015869,
		Timezone:  "Pacific/Guadalcanal",
	},
	"AGBA": {
		ICAO:      "AGBA",
		IATA:      "VEV",
		Name:      "Barakoma Airport",
		City:      "Barakoma",
		Country:   "SB",
		Latitude:  -7.912779808,
		Longitude: 156.7059936523,
		Timezone:  "Pacific/Guadalcanal",
	},
	"AGBT": {
		ICAO:      "AGBT",
		IATA:      "BPF",
		Name:      "Batuna Aerodrome",
		City:      "Batuna Mission Station",
		State:     "Western Province",
		Country:   "SB",
		Elevation: 24,
		Latitude:  -8.5620277778,
		Longitude: 158.119305556,
		Timezone:  "Pacific/Guadalcanal",
	},
	"AGEV": {
		ICAO:      "AGEV",
		IATA:      "GEF",
		Name:      "Geva Airport",
		City:      "Liangia",
		Country:   "SB",
		Latitude:  -7.5758299828,
		Longitude: 156.5970001221,
		Timezone:  "Pacific/Guadalcanal",
	},
	"AGGA": {
		ICAO:      "AGGA",
		IATA:      "AKS",
		Name:      "Auki Airport",
		City:      "Auki",
		Country:   "SB",
		Elevation: 5,
		Latitude:  -8.7025699616,
		Longitude: 160.682006836,
		Timezone:  "Pacific/Guadalcanal",
	},
	"AGGB": {
		ICAO:      "AGGB",
		IATA:      "BNY",
		Name:      "Bellona/Anua Airport",
		City:      "Anua",
		Country:   "SB",
		Elevation: 60,
		Latitude:  -11.3022222222,
		Longitude: 159.798333333,
		Timezone:  "Pacific/Guadalcanal",
	},
	"AGGC": {
		ICAO:      "AGGC",
		IATA:      "CHY",
		Name:      "Choiseul Bay Airport",
		State:     "Choiseul",
		Country:   "SB",
		Latitude:  -6.711944,
		Longitude: 156.396111,
		Timezone:  "Pacific/Guadalcanal",
	},
	"AGGE": {
		ICAO:      "AGGE",
		IATA:      "BAS",
		Name:      "Ballalae Airport",
		City:      "Ballalae",
		Country:   "SB",
		Elevation: 5,
		Latitude:  -6.990745,
		Longitude: 155.886656,
		Timezone:  "Pacific/Guadalcanal",
	},
	"AGGF": {
		ICAO:      "AGGF",
		IATA:      "FRE",
		Name:      "Fera/Maringe Airport",
		City:      "Fera Island",
		State:     "Isabel",
		Country:   "SB",
		Latitude:  -8.1075000763,
		Longitude: 159.5769958496,
		Timezone:  "Pacific/Guadalcanal",
	},
	"AGGH": {
		ICAO:      "AGGH",
		IATA:      "HIR",
		Name:      "Honiara International Airport",
		City:      "Honiara",
		State:     "Guadalcanal",
		Country:   "SB",
		Elevation: 28,
		Latitude:  -9.4280004501,
		Longitude: 160.0549926758,
		Timezone:  "Pacific/Guadalcanal",
	},
	"AGGI": {
		ICAO:      "AGGI",
		IATA:      "MBU",
		Name:      "Babanakira Airport",
		City:      "Mbambanakira",
		State:     "Guadalcanal",
		Country:   "SB",
		Latitude:  -9.7475004196,
		Longitude: 159.8390045166,
		Timezone:  "Pacific/Guadalcanal",
	},
	"AGGJ": {
		ICAO:      "AGGJ",
		IATA:      "AVU",
		Name:      "Avu Avu Airport",
		State:     "Guadalcanal",
		Country:   "SB",
		Elevation: 25,
		Latitude:  -9.8683333333,
		Longitude: 160.410555556,
		Timezone:  "Pacific/Guadalcanal",
	},
	"AGGK": {
		ICAO:      "AGGK",
		IATA:      "IRA",
		Name:      "Ngorangora Airport",
		City:      "Kirakira",
		Country:   "SB",
		Latitude:  -10.4497003555,
		Longitude: 161.8979949951,
		Timezone:  "Pacific/Guadalcanal",
	},
	"AGGL": {
		ICAO:      "AGGL",
		IATA:      "SCZ",
		Name:      "Santa Cruz/Graciosa Bay/Luova Airport",
		City:      "Santa Cruz/Graciosa Bay/Luova",
		State:     "Temotu",
		Country:   "SB",
		Elevation: 18,
		Latitude:  -10.7202997208,
		Longitude: 165.7949981689,
		Timezone:  "Pacific/Guadalcanal",
	},
	"AGGM": {
		ICAO:      "AGGM",
		IATA:      "MUA",
		Name:      "Munda Airport",
		Country:   "SB",
		Elevation: 10,
		Latitude:  -8.3279695511,
		Longitude: 157.2630004883,
		Timezone:  "Pacific/Guadalcanal",
	},
	"AGGN": {
		ICAO:      "AGGN",
		IATA:      "GZO",
		Name:      "Nusatupe Airport",
		City:      "Gizo",
		Country:   "SB",
		Elevation: 13,
		Latitude:  -8.0977802277,
		Longitude: 156.863998413,
		Timezone:  "Pacific/Guadalcanal",
	},
	"AGGO": {
		ICAO:      "AGGO",
		IATA:      "MNY",
		Name:      "Mono Airport",
		City:      "Stirling Island",
		Country:   "SB",
		Latitude:  -7.4169402122,
		Longitude: 155.5650024414,
		Timezone:  "Pacific/Guadalcanal",
	},
	"AGGP": {
		ICAO:      "AGGP",
		IATA:      "PRS",
		Name:      "Parasi Airport",
		City:      "Parasi",
		Country:   "SB",
		Latitude:  -9.6416702271,
		Longitude: 161.4250030518,
		Timezone:  "Pacific/Guadalcanal",
	},
	"AGGR": {
		ICAO:      "AGGR",
		IATA:      "RNL",
		Name:      "Rennell/Tingoa Airport",
		City:      "Rennell Island",
		State:     "Rennell and Bellona",
		Country:   "SB",
		Latitude:  -11.5339002609,
		Longitude: 160.06300354,
		Timezone:  "Pacific/Guadalcanal",
	},
	"AGGS": {
		ICAO:      "AGGS",
		IATA:      "EGM",
		Name:      "Sege Airport",
		City:      "Sege",
		Country:   "SB",
		Latitude:  -8.5788898468,
		Longitude: 157.8760070801,
		Timezone:  "Pacific/Guadalcanal",
	},
	"AGGT": {
		ICAO:      "AGGT",
		IATA:      "NNB",
		Name:      "Santa Ana Airport",
		City:      "Santa Ana Island",
		Country:   "SB",
		Elevation: 3,
		Latitude:  -10.847994,
		Longitude: 162.454108,
		Timezone:  "Pacific/Guadalcanal",
	},
	"AGGU": {
		ICAO:      "AGGU",
		IATA:      "RUS",
		Name:      "Marau Airport",
		City:      "Marau",
		Country:   "SB",
		Latitude:  -9.8616695404,
		Longitude: 160.824996948,
		Timezone:  "Pacific/Guadalcanal",
	},
	"AGGV": {
		ICAO:      "AGGV",
		IATA:      "VAO",
		Name:      "Suavanao Airport",
		City:      "Suavanao",
		Country:   "SB",
		Latitude:  -7.585559845,
		Longitude: 158.7310028076,
		Timezone:  "Pacific/Guadalcanal",
	},
	"AGGY": {
		ICAO:      "AGGY",
		IATA:      "XYA",
		Name:      "Yandina Airport",
		City:      "Yandina",
		Country:   "SB",
		Elevation: 60,
		Latitude:  -9.092816,
		Longitude: 159.21841,
		Timezone:  "Pacific/Guadalcanal",
	},
	"AGKG": {
		ICAO:      "AGKG",
		IATA:      "KGE",
		Name:      "Kagau Island Airport",
		City:      "Kagau Island",
		Country:   "SB",
		Latitude:  -7.3330001831,
		Longitude: 157.5829925537,
		Timezone:  "Pacific/Guadalcanal",
	},
	"AGKW": {
		ICAO:      "AGKW",
		IATA:      "KWS",
		Name:      "Kwailabesi Airport",
		City:      "Kwailabesi",
		Country:   "SB",
		Elevation: 26,
		Latitude:  -8.360508,
		Longitude: 160.775127,
		Timezone:  "Pacific/Guadalcanal",
	},
	"AGOK": {
		ICAO:      "AGOK",
		IATA:      "GTA",
		Name:      "Gatokae Airport",
		City:      "Gatokae",
		Country:   "SB",
		Elevation: 70,
		Latitude:  -8.739167,
		Longitude: 158.203056,
		Timezone:  "Pacific/Guadalcanal",
	},
	"AGRC": {
		ICAO:      "AGRC",
		IATA:      "RIN",
		Name:      "Ringi Cove Airport",
		City:      "Ringi Cove",
		Country:   "SB",
		Latitude:  -8.1263904572,
		Longitude: 157.1430053711,
		Timezone:  "Pacific/Guadalcanal",
	},
	"AGRM": {
		ICAO:      "AGRM",
		IATA:      "RBV",
		Name:      "Ramata Airport",
		City:      "Ramata",
		Country:   "SB",
		Latitude:  -8.1680603027,
		Longitude: 157.6430053711,
		Timezone:  "Pacific/Guadalcanal",
	},
	"AK00": {
		ICAO:      "AK00",
		Name:      "Anchor River Airpark",
		City:      "Anchor Point",
		State:     "Alaska",
		Country:   "US",
		Elevation: 120,
		Latitude:  59.7966995239,
		Longitude: -151.8630065918,
		Timezone:  "America/Anchorage",
	},
	"AK01": {
		ICAO:      "AK01",
		Name:      "Alaska Airpark",
		City:      "Sterling",
		State:     "Alaska",
		Country:   "US",
		Elevation: 320,
		Latitude:  60.528983,
		Longitude: -150.625133,
		Timezone:  "America/Anchorage",
	},
	"AK02": {
		ICAO:      "AK02",
		Name:      "Bear Creek 1 Airport",
		City:      "Bear Creek",
		State:     "Alaska",
		Country:   "US",
		Elevation: 575,
		Latitude:  65.5399017334,
		Longitude: -161.06199646,
		Timezone:  "America/Anchorage",
	},
	"AK05": {
		ICAO:      "AK05",
		Name:      "Breeden Airport",
		City:      "Sterling",
		State:     "Alaska",
		Country:   "US",
		Elevation: 365,
		Latitude:  60.54105,
		Longitude: -150.599217,
		Timezone:  "America/Anchorage",
	},
	"AK06": {
		ICAO:      "AK06",
		Name:      "Denali Airport",
		City:      "Mckinley Park",
		State:     "Alaska",
		Country:   "US",
		Elevation: 2050,
		Latitude:  63.6450996399,
		Longitude: -148.7980041504,
		Timezone:  "America/Anchorage",
	},
	"AK07": {
		ICAO:      "AK07",
		Name:      "Dog Fish Bay Airport",
		City:      "Nanwalek",
		State:     "Alaska",
		Country:   "US",
		Elevation: 10,
		Latitude:  59.2410011292,
		Longitude: -151.8769989014,
		Timezone:  "America/Anchorage",
	},
	"AK08": {
		ICAO:      "AK08",
		Name:      "Thomas Strip",
		City:      "Willow",
		State:     "Alaska",
		Country:   "US",
		Elevation: 190,
		Latitude:  61.7899017334,
		Longitude: -150.1060028076,
		Timezone:  "America/Anchorage",
	},
	"AK09": {
		ICAO:      "AK09",
		Name:      "Wingsong Estates Airport",
		City:      "Delta Junction",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1100,
		Latitude:  64.0495986938,
		Longitude: -145.5019989014,
		Timezone:  "America/Anchorage",
	},
	"AK10": {
		ICAO:      "AK10",
		Name:      "Ernies Airstrip",
		City:      "Willow",
		State:     "Alaska",
		Country:   "US",
		Elevation: 210,
		Latitude:  61.7172012329,
		Longitude: -150.10899353,
		Timezone:  "America/Anchorage",
	},
	"AK11": {
		ICAO:      "AK11",
		Name:      "Point Mcintyre Airport",
		City:      "Point Mcintyre",
		State:     "Alaska",
		Country:   "US",
		Elevation: 15,
		Latitude:  70.4017028809,
		Longitude: -148.6820068359,
		Timezone:  "America/Anchorage",
	},
	"AK12": {
		ICAO:      "AK12",
		Name:      "Flying Crown Airport",
		City:      "Anchorage",
		State:     "Alaska",
		Country:   "US",
		Elevation: 47,
		Latitude:  61.1066017151,
		Longitude: -149.8639984131,
		Timezone:  "America/Anchorage",
	},
	"AK13": {
		ICAO:      "AK13",
		IATA:      "CEX",
		Name:      "Chena Hot Springs Airport",
		City:      "Chena Hot Springs",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1195,
		Latitude:  65.0518035889,
		Longitude: -146.04699707,
		Timezone:  "America/Anchorage",
	},
	"AK14": {
		ICAO:      "AK14",
		Name:      "Turinsky Airstrip",
		City:      "Point Mackenzie",
		State:     "Alaska",
		Country:   "US",
		Elevation: 100,
		Latitude:  61.3512992859,
		Longitude: -150.0890045166,
		Timezone:  "America/Anchorage",
	},
	"AK16": {
		ICAO:      "AK16",
		Name:      "Sunset Strip",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 303,
		Latitude:  61.52339935,
		Longitude: -149.7149963,
		Timezone:  "America/Anchorage",
	},
	"AK17": {
		ICAO:      "AK17",
		Name:      "Glacierview Strip",
		City:      "Homer",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1600,
		Latitude:  59.7635993958,
		Longitude: -151.2230072021,
		Timezone:  "America/Anchorage",
	},
	"AK18": {
		ICAO:      "AK18",
		Name:      "Camp Point Airport",
		City:      "Homer",
		State:     "Alaska",
		Country:   "US",
		Elevation: 8,
		Latitude:  59.8284988403,
		Longitude: -153.0809936523,
		Timezone:  "America/Anchorage",
	},
	"AK19": {
		ICAO:      "AK19",
		Name:      "Carl's Landing Airport",
		City:      "Talkeetna",
		State:     "Alaska",
		Country:   "US",
		Elevation: 300,
		Latitude:  62.1739006042,
		Longitude: -150.0639953613,
		Timezone:  "America/Anchorage",
	},
	"AK20": {
		ICAO:      "AK20",
		Name:      "Cd-3 Airstrip",
		City:      "Nuiqsut",
		State:     "Alaska",
		Country:   "US",
		Elevation: 7,
		Latitude:  70.420622,
		Longitude: -150.886667,
		Timezone:  "America/Anchorage",
	},
	"AK21": {
		ICAO:      "AK21",
		Name:      "Nushagak Airport",
		City:      "Dillingham",
		State:     "Alaska",
		Country:   "US",
		Elevation: 40,
		Latitude:  59.132598877,
		Longitude: -157.7769927979,
		Timezone:  "America/Anchorage",
	},
	"AK22": {
		ICAO:      "AK22",
		Name:      "Lakloey Air Park",
		City:      "Fairbanks",
		State:     "Alaska",
		Country:   "US",
		Elevation: 475,
		Latitude:  64.8217010498,
		Longitude: -147.5220031738,
		Timezone:  "America/Anchorage",
	},
	"AK24": {
		ICAO:      "AK24",
		Name:      "Hilltop Airport",
		City:      "Chugiak",
		State:     "Alaska",
		Country:   "US",
		Elevation: 420,
		Latitude:  61.41820145,
		Longitude: -149.4389954,
		Timezone:  "America/Anchorage",
	},
	"AK25": {
		ICAO:      "AK25",
		Name:      "Piper Landing Airport",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 350,
		Latitude:  61.6175003052,
		Longitude: -149.6150054932,
		Timezone:  "America/Anchorage",
	},
	"AK26": {
		ICAO:      "AK26",
		Name:      "Solomon State Field",
		City:      "Solomon",
		State:     "Alaska",
		Country:   "US",
		Elevation: 25,
		Latitude:  64.5606002808,
		Longitude: -164.4420013428,
		Timezone:  "America/Nome",
	},
	"AK27": {
		ICAO:      "AK27",
		Name:      "Valley Flying Crown Airport",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 400,
		Latitude:  61.6426010132,
		Longitude: -149.6239929199,
		Timezone:  "America/Anchorage",
	},
	"AK28": {
		ICAO:      "AK28",
		Name:      "Chena Marina Airport",
		City:      "Fairbanks",
		State:     "Alaska",
		Country:   "US",
		Elevation: 427,
		Latitude:  64.8140029907,
		Longitude: -147.9190063477,
		Timezone:  "America/Anchorage",
	},
	"AK29": {
		ICAO:      "AK29",
		Name:      "Reids Landing Airport",
		City:      "Houston",
		State:     "Alaska",
		Country:   "US",
		Elevation: 246,
		Latitude:  61.6125984192,
		Longitude: -149.8520050049,
		Timezone:  "America/Anchorage",
	},
	"AK30": {
		ICAO:      "AK30",
		Name:      "Strawberry Point Airport",
		City:      "Cordova",
		State:     "Alaska",
		Country:   "US",
		Elevation: 50,
		Latitude:  60.391153,
		Longitude: -146.097464,
		Timezone:  "America/Anchorage",
	},
	"AK31": {
		ICAO:      "AK31",
		Name:      "Swift Creek Airport",
		City:      "Mccarthy",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1225,
		Latitude:  61.4112014771,
		Longitude: -143.0010070801,
		Timezone:  "America/Anchorage",
	},
	"AK33": {
		ICAO:      "AK33",
		IATA:      "HED",
		Name:      "Herendeen Bay Airport",
		City:      "Herendeen Bay",
		State:     "Alaska",
		Country:   "US",
		Elevation: 20,
		Latitude:  55.801399231,
		Longitude: -160.899002075,
		Timezone:  "America/Anchorage",
	},
	"AK36": {
		ICAO:      "AK36",
		Name:      "Point Mac Airport",
		City:      "Point Mackenzie",
		State:     "Alaska",
		Country:   "US",
		Elevation: 300,
		Latitude:  61.436000824,
		Longitude: -150.1219940186,
		Timezone:  "America/Anchorage",
	},
	"AK39": {
		ICAO:      "AK39",
		Name:      "South Gasline Airport",
		City:      "Sterling",
		State:     "Alaska",
		Country:   "US",
		Elevation: 381,
		Latitude:  60.6972007751,
		Longitude: -150.1799926758,
		Timezone:  "America/Anchorage",
	},
	"AK40": {
		ICAO:      "AK40",
		Name:      "Nixon Fork Mine Airport",
		City:      "McGrath",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1510,
		Latitude:  63.2291984558,
		Longitude: -154.759994507,
		Timezone:  "America/Anchorage",
	},
	"AK41": {
		ICAO:      "AK41",
		Name:      "Greg'n Sage Airport",
		City:      "North Pole",
		State:     "Alaska",
		Country:   "US",
		Elevation: 925,
		Latitude:  64.5438995361,
		Longitude: -146.8439941406,
		Timezone:  "America/Anchorage",
	},
	"AK44": {
		ICAO:      "AK44",
		Name:      "Talkeetna Village Strip",
		City:      "Talkeetna",
		State:     "Alaska",
		Country:   "US",
		Elevation: 346,
		Latitude:  62.318901062,
		Longitude: -150.1159973145,
		Timezone:  "America/Anchorage",
	},
	"AK45": {
		ICAO:      "AK45",
		Name:      "Lost River 2 Airport",
		City:      "Lost River",
		State:     "Alaska",
		Country:   "US",
		Elevation: 220,
		Latitude:  65.456199646,
		Longitude: -167.1750030518,
		Timezone:  "America/Nome",
	},
	"AK46": {
		ICAO:      "AK46",
		Name:      "Abi Airport",
		City:      "Palmer",
		State:     "Alaska",
		Country:   "US",
		Elevation: 750,
		Latitude:  61.6287994385,
		Longitude: -149.0429992676,
		Timezone:  "America/Anchorage",
	},
	"AK47": {
		ICAO:      "AK47",
		Name:      "Shawn Field",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 300,
		Latitude:  61.5857009888,
		Longitude: -149.56300354,
		Timezone:  "America/Anchorage",
	},
	"AK49": {
		ICAO:      "AK49",
		IATA:      "TWE",
		Name:      "Taylor Airport",
		City:      "Taylor",
		State:     "Alaska",
		Country:   "US",
		Elevation: 440,
		Latitude:  65.6792984009,
		Longitude: -164.798995972,
		Timezone:  "America/Nome",
	},
	"AK50": {
		ICAO:      "AK50",
		Name:      "Sky Ranch At Pioneer Peak Airport",
		City:      "Palmer",
		State:     "Alaska",
		Country:   "US",
		Elevation: 120,
		Latitude:  61.554599762,
		Longitude: -149.1439971924,
		Timezone:  "America/Anchorage",
	},
	"AK51": {
		ICAO:      "AK51",
		Name:      "Wilder/Natwick Llc Airport",
		City:      "Port Alsworth",
		State:     "Alaska",
		Country:   "US",
		Elevation: 288,
		Latitude:  60.198600769,
		Longitude: -154.3249969482,
		Timezone:  "America/Anchorage",
	},
	"AK52": {
		ICAO:      "AK52",
		Name:      "Moen's Ranch Airport",
		City:      "Fairbanks",
		State:     "Alaska",
		Country:   "US",
		Elevation: 582,
		Latitude:  64.8889007568,
		Longitude: -147.5359954834,
		Timezone:  "America/Anchorage",
	},
	"AK53": {
		ICAO:      "AK53",
		Name:      "Downwind Landing Airport",
		City:      "Palmer",
		State:     "Alaska",
		Country:   "US",
		Elevation: 220,
		Latitude:  61.5822982788,
		Longitude: -148.9969940186,
		Timezone:  "America/Anchorage",
	},
	"AK54": {
		ICAO:      "AK54",
		Name:      "Stro's Airport",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 340,
		Latitude:  61.6571006775,
		Longitude: -149.3260040283,
		Timezone:  "America/Anchorage",
	},
	"AK55": {
		ICAO:      "AK55",
		Name:      "Moose Run Airstrip",
		City:      "Soldotna",
		State:     "Alaska",
		Country:   "US",
		Elevation: 290,
		Latitude:  60.489200592,
		Longitude: -150.8020019531,
		Timezone:  "America/Anchorage",
	},
	"AK58": {
		ICAO:      "AK58",
		Name:      "Fairview West Airport",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 73,
		Latitude:  61.5400009155,
		Longitude: -149.5489959717,
		Timezone:  "America/Anchorage",
	},
	"AK59": {
		ICAO:      "AK59",
		Name:      "King Ranch Airport",
		City:      "Sutton",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1350,
		Latitude:  61.7956008911,
		Longitude: -148.3549957275,
		Timezone:  "America/Anchorage",
	},
	"AK60": {
		ICAO:      "AK60",
		Name:      "Fort Jensen Airport",
		City:      "Jensens",
		State:     "Alaska",
		Country:   "US",
		Elevation: 240,
		Latitude:  57.8851013184,
		Longitude: -157.0970001221,
		Timezone:  "America/Anchorage",
	},
	"AK61": {
		ICAO:      "AK61",
		Name:      "Stephan Lake Lodge Airport",
		City:      "Talkeetna",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1900,
		Latitude:  62.6996002197,
		Longitude: -148.9069976807,
		Timezone:  "America/Anchorage",
	},
	"AK63": {
		ICAO:      "AK63",
		Name:      "Pat-Mar Strip",
		City:      "Palmer",
		State:     "Alaska",
		Country:   "US",
		Elevation: 246,
		Latitude:  61.5886001587,
		Longitude: -149.1430053711,
		Timezone:  "America/Anchorage",
	},
	"AK64": {
		ICAO:      "AK64",
		Name:      "Taylor Mountain Airport",
		City:      "Taylor Mountain Mine",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1000,
		Latitude:  60.8678016663,
		Longitude: -157.391998291,
		Timezone:  "America/Anchorage",
	},
	"AK65": {
		ICAO:      "AK65",
		Name:      "Green's Strip",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 300,
		Latitude:  61.5979003906,
		Longitude: -149.3509979248,
		Timezone:  "America/Anchorage",
	},
	"AK66": {
		ICAO:      "AK66",
		Name:      "Hunter Creek Airport",
		City:      "Palmer",
		State:     "Alaska",
		Country:   "US",
		Elevation: 100,
		Latitude:  61.4631004333,
		Longitude: -148.8040008545,
		Timezone:  "America/Anchorage",
	},
	"AK67": {
		ICAO:      "AK67",
		Name:      "Skid Marks Airport",
		City:      "Willow",
		State:     "Alaska",
		Country:   "US",
		Elevation: 64,
		Latitude:  61.7386016846,
		Longitude: -150.0729980469,
		Timezone:  "America/Anchorage",
	},
	"AK68": {
		ICAO:      "AK68",
		Name:      "Minuteman Strip",
		City:      "Willow",
		State:     "Alaska",
		Country:   "US",
		Elevation: 290,
		Latitude:  61.7202987671,
		Longitude: -150.0540008545,
		Timezone:  "America/Anchorage",
	},
	"AK69": {
		ICAO:      "AK69",
		Name:      "Long Lake Airport",
		City:      "Willow",
		State:     "Alaska",
		Country:   "US",
		Elevation: 200,
		Latitude:  61.7293014526,
		Longitude: -150.0980072021,
		Timezone:  "America/Anchorage",
	},
	"AK72": {
		ICAO:      "AK72",
		Name:      "Jewell Airport",
		City:      "Willow",
		State:     "Alaska",
		Country:   "US",
		Elevation: 113,
		Latitude:  61.70470047,
		Longitude: -150.6009979248,
		Timezone:  "America/Anchorage",
	},
	"AK73": {
		ICAO:      "AK73",
		Name:      "Mcgahan Industrial Airpark",
		City:      "Nikishka",
		State:     "Alaska",
		Country:   "US",
		Elevation: 150,
		Latitude:  60.7252998352,
		Longitude: -151.3020019531,
		Timezone:  "America/Anchorage",
	},
	"AK75": {
		ICAO:      "AK75",
		IATA:      "CDL",
		Name:      "Candle 2 Airport",
		City:      "Candle",
		State:     "Alaska",
		Country:   "US",
		Elevation: 15,
		Latitude:  65.907699585,
		Longitude: -161.925994873,
		Timezone:  "America/Anchorage",
	},
	"AK76": {
		ICAO:      "AK76",
		Name:      "East Alsek River Airport",
		City:      "Yakutat",
		State:     "Alaska",
		Country:   "US",
		Elevation: 20,
		Latitude:  59.1260986328,
		Longitude: -138.4069976807,
		Timezone:  "America/Yakutat",
	},
	"AK77": {
		ICAO:      "AK77",
		Name:      "All West Airport",
		City:      "Delta Junction",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1275,
		Latitude:  63.9413986206,
		Longitude: -145.4219970703,
		Timezone:  "America/Anchorage",
	},
	"AK79": {
		ICAO:      "AK79",
		Name:      "Jolly Field",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 100,
		Latitude:  61.59355,
		Longitude: -149.5665,
		Timezone:  "America/Anchorage",
	},
	"AK80": {
		ICAO:      "AK80",
		Name:      "American Creek Airport",
		City:      "American Creek",
		State:     "Alaska",
		Country:   "US",
		Elevation: 500,
		Latitude:  65.1016998291,
		Longitude: -151.1770019531,
		Timezone:  "America/Anchorage",
	},
	"AK83": {
		ICAO:      "AK83",
		Name:      "Gannon's Landing Airport",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 380,
		Latitude:  61.6273002625,
		Longitude: -149.6089935303,
		Timezone:  "America/Anchorage",
	},
	"AK84": {
		ICAO:      "AK84",
		Name:      "Scooter's Landing Strip",
		City:      "Sterling",
		State:     "Alaska",
		Country:   "US",
		Elevation: 259,
		Latitude:  60.5293998718,
		Longitude: -150.8309936523,
		Timezone:  "America/Anchorage",
	},
	"AK85": {
		ICAO:      "AK85",
		Name:      "Birchwater Airport",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 110,
		Latitude:  61.482333,
		Longitude: -149.763986,
		Timezone:  "America/Anchorage",
	},
	"AK86": {
		ICAO:      "AK86",
		Name:      "Kramer Airport",
		City:      "Big Lake",
		State:     "Alaska",
		Country:   "US",
		Elevation: 150,
		Latitude:  61.573539,
		Longitude: -149.91135,
		Timezone:  "America/Anchorage",
	},
	"AK88": {
		ICAO:      "AK88",
		Name:      "Eagle Nest Airport",
		City:      "Palmer",
		State:     "Alaska",
		Country:   "US",
		Elevation: 416,
		Latitude:  61.6046981812,
		Longitude: -149.0399932861,
		Timezone:  "America/Anchorage",
	},
	"AK89": {
		ICAO:      "AK89",
		Name:      "Black Spruce Airport",
		City:      "Houston",
		State:     "Alaska",
		Country:   "US",
		Elevation: 350,
		Latitude:  61.6570014954,
		Longitude: -149.8370056152,
		Timezone:  "America/Anchorage",
	},
	"AK90": {
		ICAO:      "AK90",
		Name:      "Shirley Lake Airport",
		City:      "Willow",
		State:     "Alaska",
		Country:   "US",
		Elevation: 185,
		Latitude:  61.7447013855,
		Longitude: -150.1150054932,
		Timezone:  "America/Anchorage",
	},
	"AK92": {
		ICAO:      "AK92",
		Name:      "Martin Airport",
		City:      "Palmer",
		State:     "Alaska",
		Country:   "US",
		Elevation: 820,
		Latitude:  61.63800049,
		Longitude: -149.029007,
		Timezone:  "America/Anchorage",
	},
	"AK93": {
		ICAO:      "AK93",
		Name:      "Grouse Ridge Airport",
		City:      "Palmer",
		State:     "Alaska",
		Country:   "US",
		Elevation: 535,
		Latitude:  61.6552009583,
		Longitude: -149.2729949951,
		Timezone:  "America/Anchorage",
	},
	"AK95": {
		ICAO:      "AK95",
		Name:      "Twin Lake Airport",
		City:      "Big Lake",
		State:     "Alaska",
		Country:   "US",
		Elevation: 185,
		Latitude:  61.5698013306,
		Longitude: -149.779006958,
		Timezone:  "America/Anchorage",
	},
	"AK96": {
		ICAO:      "AK96",
		IATA:      "BSZ",
		Name:      "Bartletts Airport",
		City:      "Egegik",
		State:     "Alaska",
		Country:   "US",
		Elevation: 50,
		Latitude:  58.2162017822,
		Longitude: -157.352005005,
		Timezone:  "America/Anchorage",
	},
	"AK97": {
		ICAO:      "AK97",
		IATA:      "BSW",
		Name:      "Boswell Bay Airport",
		City:      "Boswell Bay",
		State:     "Alaska",
		Country:   "US",
		Elevation: 230,
		Latitude:  60.4230995178,
		Longitude: -146.145996094,
		Timezone:  "America/Anchorage",
	},
	"AK98": {
		ICAO:      "AK98",
		Name:      "Buck Creek Airport",
		City:      "Buck Creek",
		State:     "Alaska",
		Country:   "US",
		Elevation: 560,
		Latitude:  65.6387023926,
		Longitude: -167.4859924316,
		Timezone:  "America/Nome",
	},
	"AK99": {
		ICAO:      "AK99",
		Name:      "Grand Home Airport",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 85,
		Latitude:  61.446667,
		Longitude: -149.761111,
		Timezone:  "America/Anchorage",
	},
	"AL00": {
		ICAO:      "AL00",
		Name:      "Charlie Wilkes Airport",
		City:      "Fayette",
		State:     "Alabama",
		Country:   "US",
		Elevation: 340,
		Latitude:  33.7444000244,
		Longitude: -87.7863998413,
		Timezone:  "America/Chicago",
	},
	"AL01": {
		ICAO:      "AL01",
		Name:      "Bedsole Farm Airport",
		City:      "Claiborne",
		State:     "Alabama",
		Country:   "US",
		Elevation: 180,
		Latitude:  31.5585002899,
		Longitude: -87.5333023071,
		Timezone:  "America/Chicago",
	},
	"AL02": {
		ICAO:      "AL02",
		Name:      "Rainey Field",
		City:      "Cottondale",
		State:     "Alabama",
		Country:   "US",
		Elevation: 300,
		Latitude:  33.150100708,
		Longitude: -87.4417037964,
		Timezone:  "America/Chicago",
	},
	"AL03": {
		ICAO:      "AL03",
		Name:      "Strickland/Smalley Field",
		City:      "Akron",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 369,
		Latitude:  35.853889,
		Longitude: -87.723889,
		Timezone:  "America/Chicago",
	},
	"AL04": {
		ICAO:      "AL04",
		Name:      "Pleasant View Farm Airport",
		City:      "Elberta",
		State:     "Alabama",
		Country:   "US",
		Elevation: 105,
		Latitude:  30.4626998901,
		Longitude: -87.5250015259,
		Timezone:  "America/Chicago",
	},
	"AL05": {
		ICAO:      "AL05",
		Name:      "Sehoy Airport",
		City:      "Hurtsboro",
		State:     "Alabama",
		Country:   "US",
		Elevation: 331,
		Latitude:  32.2201004028,
		Longitude: -85.4680023193,
		Timezone:  "America/Chicago",
	},
	"AL08": {
		ICAO:      "AL08",
		Name:      "Perdido Winds Airpark",
		City:      "Elberta",
		State:     "Alabama",
		Country:   "US",
		Elevation: 75,
		Latitude:  30.4244003296,
		Longitude: -87.5353012085,
		Timezone:  "America/Chicago",
	},
	"AL09": {
		ICAO:      "AL09",
		Name:      "Cloudmont Airpark",
		City:      "Mentone",
		State:     "Alabama",
		Country:   "US",
		Elevation: 1692,
		Latitude:  34.5334014893,
		Longitude: -85.5902023315,
		Timezone:  "America/Chicago",
	},
	"AL10": {
		ICAO:      "AL10",
		Name:      "Frerichs Airport",
		City:      "Gurley",
		State:     "Alabama",
		Country:   "US",
		Elevation: 650,
		Latitude:  34.6511993408,
		Longitude: -86.4169006348,
		Timezone:  "America/Chicago",
	},
	"AL12": {
		ICAO:      "AL12",
		Name:      "Kershaw Airport",
		City:      "Montgomery",
		State:     "Alabama",
		Country:   "US",
		Elevation: 225,
		Latitude:  32.3585014343,
		Longitude: -86.125,
		Timezone:  "America/Chicago",
	},
	"AL15": {
		ICAO:      "AL15",
		Name:      "Styron Airport",
		City:      "Foley",
		State:     "Alabama",
		Country:   "US",
		Elevation: 120,
		Latitude:  30.4029998779,
		Longitude: -87.7241973877,
		Timezone:  "America/Chicago",
	},
	"AL16": {
		ICAO:      "AL16",
		Name:      "Turkey Creek Airport",
		City:      "Pinson",
		State:     "Alabama",
		Country:   "US",
		Elevation: 640,
		Latitude:  33.7267990112,
		Longitude: -86.7236022949,
		Timezone:  "America/Chicago",
	},
	"AL17": {
		ICAO:      "AL17",
		Name:      "Lazy Eight Airpark Llc Airport",
		City:      "Wilsonville",
		State:     "Alabama",
		Country:   "US",
		Elevation: 450,
		Latitude:  33.2340011597,
		Longitude: -86.5130004883,
		Timezone:  "America/Chicago",
	},
	"AL18": {
		ICAO:      "AL18",
		Name:      "Parker Field",
		City:      "Gardendale",
		State:     "Alabama",
		Country:   "US",
		Elevation: 590,
		Latitude:  33.6762008667,
		Longitude: -86.8675003052,
		Timezone:  "America/Chicago",
	},
	"AL25": {
		ICAO:      "AL25",
		Name:      "Spratling Field",
		City:      "Roba",
		State:     "Alabama",
		Country:   "US",
		Elevation: 368,
		Latitude:  32.242401123,
		Longitude: -85.6010971069,
		Timezone:  "America/Chicago",
	},
	"AL27": {
		ICAO:      "AL27",
		Name:      "Headquarters Airport",
		City:      "Centre",
		State:     "Alabama",
		Country:   "US",
		Elevation: 610,
		Latitude:  34.1057014465,
		Longitude: -85.5658035278,
		Timezone:  "America/Chicago",
	},
	"AL29": {
		ICAO:      "AL29",
		Name:      "New Horizon Airport",
		City:      "Blountsville",
		State:     "Alabama",
		Country:   "US",
		Elevation: 750,
		Latitude:  34.1292991638,
		Longitude: -86.6333007813,
		Timezone:  "America/Chicago",
	},
	"AL32": {
		ICAO:      "AL32",
		Name:      "Flying M Ranch Airport",
		City:      "Cedar Bluff",
		State:     "Alabama",
		Country:   "US",
		Elevation: 670,
		Latitude:  34.3226013184,
		Longitude: -85.643699646,
		Timezone:  "America/Chicago",
	},
	"AL33": {
		ICAO:      "AL33",
		Name:      "Sturdy Oak Farm Airport",
		City:      "Perdido",
		State:     "Alabama",
		Country:   "US",
		Elevation: 230,
		Latitude:  31.0251998901,
		Longitude: -87.5858001709,
		Timezone:  "America/Chicago",
	},
	"AL34": {
		ICAO:      "AL34",
		Name:      "Henson Field",
		City:      "Moulton",
		State:     "Alabama",
		Country:   "US",
		Elevation: 685,
		Latitude:  34.4469985962,
		Longitude: -87.386100769,
		Timezone:  "America/Chicago",
	},
	"AL35": {
		ICAO:      "AL35",
		Name:      "Mc Ginnis Airport",
		City:      "Loxley",
		State:     "Alabama",
		Country:   "US",
		Elevation: 125,
		Latitude:  30.619600296,
		Longitude: -87.7043991089,
		Timezone:  "America/Chicago",
	},
	"AL37": {
		ICAO:      "AL37",
		Name:      "Benedick Airport",
		City:      "Jack",
		State:     "Alabama",
		Country:   "US",
		Elevation: 335,
		Latitude:  31.555028,
		Longitude: -85.977278,
		Timezone:  "America/Chicago",
	},
	"AL40": {
		ICAO:      "AL40",
		Name:      "Bonham Airport",
		City:      "Birmingham",
		State:     "Alabama",
		Country:   "US",
		Elevation: 600,
		Latitude:  33.5432014465,
		Longitude: -86.9858016968,
		Timezone:  "America/Chicago",
	},
	"AL41": {
		ICAO:      "AL41",
		Name:      "Freedom Field",
		City:      "Slocomb",
		State:     "Alabama",
		Country:   "US",
		Elevation: 285,
		Latitude:  31.1159992218,
		Longitude: -85.62550354,
		Timezone:  "America/Chicago",
	},
	"AL43": {
		ICAO:      "AL43",
		Name:      "Henley Ranch Airport",
		City:      "Geiger",
		State:     "Alabama",
		Country:   "US",
		Elevation: 200,
		Latitude:  32.9085006714,
		Longitude: -88.3333969116,
		Timezone:  "America/Chicago",
	},
	"AL51": {
		ICAO:      "AL51",
		Name:      "Flying C's Plantation Airport",
		City:      "Crawford",
		State:     "Alabama",
		Country:   "US",
		Elevation: 395,
		Latitude:  32.4160995483,
		Longitude: -85.2992019653,
		Timezone:  "America/Chicago",
	},
	"AL54": {
		ICAO:      "AL54",
		Name:      "Bird Nest Airport",
		City:      "Winfield",
		State:     "Alabama",
		Country:   "US",
		Elevation: 420,
		Latitude:  33.8633003235,
		Longitude: -87.8452987671,
		Timezone:  "America/Chicago",
	},
	"AL55": {
		ICAO:      "AL55",
		Name:      "Shields Airport",
		City:      "Elberta",
		State:     "Alabama",
		Country:   "US",
		Elevation: 95,
		Latitude:  30.4419002533,
		Longitude: -87.4638977051,
		Timezone:  "America/Chicago",
	},
	"AL56": {
		ICAO:      "AL56",
		Name:      "Jones Light Aviation Airport",
		City:      "Smiths",
		State:     "Alabama",
		Country:   "US",
		Elevation: 440,
		Latitude:  32.5000991821,
		Longitude: -85.0832977295,
		Timezone:  "America/New_York",
	},
	"AL60": {
		ICAO:      "AL60",
		Name:      "Dugger's Field",
		City:      "Ashville",
		State:     "Alabama",
		Country:   "US",
		Elevation: 560,
		Latitude:  33.8386993408,
		Longitude: -86.2114028931,
		Timezone:  "America/Chicago",
	},
	"AL61": {
		ICAO:      "AL61",
		Name:      "Belforest Airport",
		City:      "Daphne",
		State:     "Alabama",
		Country:   "US",
		Elevation: 140,
		Latitude:  30.6168994904,
		Longitude: -87.8582992554,
		Timezone:  "America/Chicago",
	},
	"AL62": {
		ICAO:      "AL62",
		Name:      "Striplin Airfield",
		City:      "Hartford",
		State:     "Alabama",
		Country:   "US",
		Elevation: 255,
		Latitude:  31.0830001831,
		Longitude: -85.6835021973,
		Timezone:  "America/Chicago",
	},
	"AL68": {
		ICAO:      "AL68",
		Name:      "Willis Airport",
		City:      "Piedmont",
		State:     "Alabama",
		Country:   "US",
		Elevation: 790,
		Latitude:  33.8512001038,
		Longitude: -85.6630020142,
		Timezone:  "America/Chicago",
	},
	"AL69": {
		ICAO:      "AL69",
		Name:      "Vaughn Private Airport",
		City:      "Summerdale",
		State:     "Alabama",
		Country:   "US",
		Elevation: 30,
		Latitude:  30.5084991455,
		Longitude: -87.4916000366,
		Timezone:  "America/Chicago",
	},
	"AL71": {
		ICAO:      "AL71",
		Name:      "Willow Point Airport",
		City:      "Our Town",
		State:     "Alabama",
		Country:   "US",
		Elevation: 551,
		Latitude:  32.8036994934,
		Longitude: -85.9804992676,
		Timezone:  "America/Chicago",
	},
	"AL72": {
		ICAO:      "AL72",
		Name:      "Palmer Field",
		City:      "Madison",
		State:     "Alabama",
		Country:   "US",
		Elevation: 680,
		Latitude:  34.695400238,
		Longitude: -86.7861022949,
		Timezone:  "America/Chicago",
	},
	"AL73": {
		ICAO:      "AL73",
		Name:      "Sharpe Field",
		City:      "Tuskegee",
		State:     "Alabama",
		Country:   "US",
		Elevation: 253,
		Latitude:  32.4919013977,
		Longitude: -85.7755966187,
		Timezone:  "America/Chicago",
	},
	"AL74": {
		ICAO:      "AL74",
		Name:      "Grimes Field",
		City:      "Semmes",
		State:     "Alabama",
		Country:   "US",
		Elevation: 215,
		Latitude:  30.7644004822,
		Longitude: -88.3058013916,
		Timezone:  "America/Chicago",
	},
	"AL76": {
		ICAO:      "AL76",
		Name:      "Mayfield (Private) Airport",
		City:      "Maplesville",
		State:     "Alabama",
		Country:   "US",
		Elevation: 495,
		Latitude:  32.8487014771,
		Longitude: -86.8700027466,
		Timezone:  "America/Chicago",
	},
	"AL77": {
		ICAO:      "AL77",
		Name:      "Smart Road Airport",
		City:      "Troy",
		State:     "Alabama",
		Country:   "US",
		Elevation: 425,
		Latitude:  31.7406005859,
		Longitude: -86.0070037842,
		Timezone:  "America/Chicago",
	},
	"AL78": {
		ICAO:      "AL78",
		Name:      "Klumpp Airport",
		City:      "Fairhope",
		State:     "Alabama",
		Country:   "US",
		Elevation: 111,
		Latitude:  30.5268993378,
		Longitude: -87.8542022705,
		Timezone:  "America/Chicago",
	},
	"AL79": {
		ICAO:      "AL79",
		Name:      "Bartlett Ranch Airport",
		City:      "Pike Road",
		State:     "Alabama",
		Country:   "US",
		Elevation: 305,
		Latitude:  32.2695999146,
		Longitude: -86.0988006592,
		Timezone:  "America/Chicago",
	},
	"AL81": {
		ICAO:      "AL81",
		Name:      "Wallace Field",
		City:      "Robertsdale",
		State:     "Alabama",
		Country:   "US",
		Elevation: 125,
		Latitude:  30.5501995087,
		Longitude: -87.5832977295,
		Timezone:  "America/Chicago",
	},
	"AL84": {
		ICAO:      "AL84",
		Name:      "Pecan Ponds Farm Airport",
		City:      "Robertsdale",
		State:     "Alabama",
		Country:   "US",
		Elevation: 125,
		Latitude:  30.5585002899,
		Longitude: -87.6041030884,
		Timezone:  "America/Chicago",
	},
	"AL88": {
		ICAO:      "AL88",
		Name:      "Ban Farm Airport",
		City:      "Lillian",
		State:     "Alabama",
		Country:   "US",
		Elevation: 80,
		Latitude:  30.4265995026,
		Longitude: -87.4708023071,
		Timezone:  "America/Chicago",
	},
	"AL89": {
		ICAO:      "AL89",
		Name:      "Sommerset Strip",
		City:      "Opelika",
		State:     "Alabama",
		Country:   "US",
		Elevation: 820,
		Latitude:  32.7187004089,
		Longitude: -85.366897583,
		Timezone:  "America/Chicago",
	},
	"AL92": {
		ICAO:      "AL92",
		Name:      "Hawk Field",
		City:      "Harpersville",
		State:     "Alabama",
		Country:   "US",
		Elevation: 420,
		Latitude:  33.3073005676,
		Longitude: -86.4036026001,
		Timezone:  "America/Chicago",
	},
	"AL93": {
		ICAO:      "AL93",
		Name:      "Big Sky Airport",
		City:      "Huntsville",
		State:     "Alabama",
		Country:   "US",
		Elevation: 830,
		Latitude:  34.8856010437,
		Longitude: -86.7099990845,
		Timezone:  "America/Chicago",
	},
	"AL94": {
		ICAO:      "AL94",
		Name:      "Evans Field",
		City:      "Wilmer",
		State:     "Alabama",
		Country:   "US",
		Elevation: 212,
		Latitude:  30.8127002716,
		Longitude: -88.3771972656,
		Timezone:  "America/Chicago",
	},
	"AL97": {
		ICAO:      "AL97",
		Name:      "Williamson Farm Airport",
		City:      "Loxley",
		State:     "Alabama",
		Country:   "US",
		Elevation: 162,
		Latitude:  30.6446990967,
		Longitude: -87.6694030762,
		Timezone:  "America/Chicago",
	},
	"AN01": {
		ICAO:      "AN01",
		Name:      "Av Ranch Airport",
		City:      "Klondyke",
		State:     "Arizona",
		Country:   "US",
		Elevation: 3500,
		Latitude:  32.8166999817,
		Longitude: -110.3170013428,
		Timezone:  "America/Phoenix",
	},
	"ANYN": {
		ICAO:      "ANYN",
		IATA:      "INU",
		Name:      "Nauru International Airport",
		City:      "Yaren District",
		State:     "Yaren",
		Country:   "NR",
		Elevation: 22,
		Latitude:  -0.547458,
		Longitude: 166.919006,
		Timezone:  "Pacific/Nauru",
	},
	"AR01": {
		ICAO:      "AR01",
		Name:      "Baker Flying Service Airport",
		City:      "Eudora",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 116,
		Latitude:  33.1085014343,
		Longitude: -91.2500991821,
		Timezone:  "America/Chicago",
	},
	"AR03": {
		ICAO:      "AR03",
		Name:      "Centerville Airstrip",
		City:      "Centerville",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 350,
		Latitude:  35.1186981201,
		Longitude: -93.2015991211,
		Timezone:  "America/Chicago",
	},
	"AR04": {
		ICAO:      "AR04",
		Name:      "Gunem Field",
		City:      "Cave Springs",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1214,
		Latitude:  36.2616996765,
		Longitude: -94.248298645,
		Timezone:  "America/Chicago",
	},
	"AR05": {
		ICAO:      "AR05",
		Name:      "John Harris Field",
		City:      "Chickala",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1770,
		Latitude:  35.2159004211,
		Longitude: -93.304901123,
		Timezone:  "America/Chicago",
	},
	"AR06": {
		ICAO:      "AR06",
		Name:      "Cantrell Farms Airport",
		City:      "Conway",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 325,
		Latitude:  35.2324981689,
		Longitude: -92.3432998657,
		Timezone:  "America/Chicago",
	},
	"AR08": {
		ICAO:      "AR08",
		Name:      "Flying Machines Airstrip",
		City:      "Eureka Springs",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1150,
		Latitude:  36.3152999878,
		Longitude: -93.6666030884,
		Timezone:  "America/Chicago",
	},
	"AR09": {
		ICAO:      "AR09",
		Name:      "Bella Vista Field",
		City:      "Gravette",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1200,
		Latitude:  36.413898468,
		Longitude: -94.4389038086,
		Timezone:  "America/Chicago",
	},
	"AR10": {
		ICAO:      "AR10",
		Name:      "Greenwalt Company Airport",
		City:      "Hazen",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 232,
		Latitude:  34.8067016602,
		Longitude: -91.6082992554,
		Timezone:  "America/Chicago",
	},
	"AR11": {
		ICAO:      "AR11",
		Name:      "Ozark Aerodrome",
		City:      "Rogers",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1270,
		Latitude:  36.28889847,
		Longitude: -94.09030151,
		Timezone:  "America/Chicago",
	},
	"AR12": {
		ICAO:      "AR12",
		Name:      "Mc Donald's Strip",
		City:      "Rose Bud",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 700,
		Latitude:  35.3205986023,
		Longitude: -92.018699646,
		Timezone:  "America/Chicago",
	},
	"AR13": {
		ICAO:      "AR13",
		Name:      "Pearson Field",
		City:      "Bigelow",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 300,
		Latitude:  34.9861984253,
		Longitude: -92.6334991455,
		Timezone:  "America/Chicago",
	},
	"AR17": {
		ICAO:      "AR17",
		Name:      "Burns Aerodrome",
		City:      "Palestine",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 207,
		Latitude:  34.9833984375,
		Longitude: -90.9151000977,
		Timezone:  "America/Chicago",
	},
	"AR18": {
		ICAO:      "AR18",
		Name:      "Gerrard Airport",
		City:      "Marianna",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 205,
		Latitude:  34.7714996338,
		Longitude: -90.8404006958,
		Timezone:  "America/Chicago",
	},
	"AR19": {
		ICAO:      "AR19",
		Name:      "Wesson-Davis Field",
		City:      "Paris",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 570,
		Latitude:  35.2128982544,
		Longitude: -93.7520980835,
		Timezone:  "America/Chicago",
	},
	"AR22": {
		ICAO:      "AR22",
		Name:      "Tommy's Flying Service Inc Airport",
		City:      "Sherrill",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 215,
		Latitude:  34.3815002441,
		Longitude: -91.9585037231,
		Timezone:  "America/Chicago",
	},
	"AR23": {
		ICAO:      "AR23",
		Name:      "Cedar Creek Ranch Airport",
		City:      "Belleville",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 420,
		Latitude:  35.1333999634,
		Longitude: -93.4751968384,
		Timezone:  "America/Chicago",
	},
	"AR25": {
		ICAO:      "AR25",
		Name:      "Vilonia Airport",
		City:      "Vilonia",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 590,
		Latitude:  35.1186981201,
		Longitude: -92.1893005371,
		Timezone:  "America/Chicago",
	},
	"AR27": {
		ICAO:      "AR27",
		Name:      "Lollars Creek Farm Airport",
		City:      "Wesley",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1430,
		Latitude:  35.938999176,
		Longitude: -93.845703125,
		Timezone:  "America/Chicago",
	},
	"AR28": {
		ICAO:      "AR28",
		Name:      "Odom Field",
		City:      "Cabot",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 280,
		Latitude:  34.9393005371,
		Longitude: -92.024597168,
		Timezone:  "America/Chicago",
	},
	"AR34": {
		ICAO:      "AR34",
		Name:      "Pine Village Airport",
		City:      "Mayflower",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 610,
		Latitude:  34.9819984436,
		Longitude: -92.4451980591,
		Timezone:  "America/Chicago",
	},
	"AR35": {
		ICAO:      "AR35",
		Name:      "Henson Farm Airport",
		City:      "Fayetteville",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1420,
		Latitude:  36.0867004395,
		Longitude: -94.1052017212,
		Timezone:  "America/Chicago",
	},
	"AR43": {
		ICAO:      "AR43",
		Name:      "Brickey Private Airport",
		City:      "Morrilton",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 320,
		Latitude:  35.1001014709,
		Longitude: -92.7835006714,
		Timezone:  "America/Chicago",
	},
	"AR44": {
		ICAO:      "AR44",
		Name:      "Flying W Airport",
		City:      "Texarkana",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 265,
		Latitude:  33.5125999451,
		Longitude: -94.0134963989,
		Timezone:  "America/Chicago",
	},
	"AR46": {
		ICAO:      "AR46",
		Name:      "Penrose Airport",
		City:      "Mc Crory",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 220,
		Latitude:  35.1893005371,
		Longitude: -91.048500061,
		Timezone:  "America/Chicago",
	},
	"AR50": {
		ICAO:      "AR50",
		Name:      "Hess Strip",
		City:      "Wynne",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 180,
		Latitude:  35.2667999268,
		Longitude: -90.9834976196,
		Timezone:  "America/Chicago",
	},
	"AR54": {
		ICAO:      "AR54",
		Name:      "Ralph Fulmer Field",
		City:      "Cabot",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 245,
		Latitude:  34.9187011719,
		Longitude: -91.9151000977,
		Timezone:  "America/Chicago",
	},
	"AR55": {
		ICAO:      "AR55",
		Name:      "R.V. Stewart Field",
		City:      "North Little Rock",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 251,
		Latitude:  34.7397994995,
		Longitude: -92.0901031494,
		Timezone:  "America/Chicago",
	},
	"AR56": {
		ICAO:      "AR56",
		Name:      "Amos Airport",
		City:      "Oil Trough",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 240,
		Latitude:  35.6001014709,
		Longitude: -91.4335021973,
		Timezone:  "America/Chicago",
	},
	"AR64": {
		ICAO:      "AR64",
		Name:      "White River Airport",
		City:      "Melbourne",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 310,
		Latitude:  35.9211997986,
		Longitude: -92.0050964355,
		Timezone:  "America/Chicago",
	},
	"AR79": {
		ICAO:      "AR79",
		Name:      "Flying G Ranch Airport",
		City:      "Tillar",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 150,
		Latitude:  33.6931991577,
		Longitude: -91.3796005249,
		Timezone:  "America/Chicago",
	},
	"AR81": {
		ICAO:      "AR81",
		Name:      "Katheryn's Landing Airport",
		City:      "Leola",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 250,
		Latitude:  34.1623001099,
		Longitude: -92.6037979126,
		Timezone:  "America/Chicago",
	},
	"AR82": {
		ICAO:      "AR82",
		Name:      "Gillespie Strip",
		City:      "Leola",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 492,
		Latitude:  34.1711997986,
		Longitude: -92.6959991455,
		Timezone:  "America/Chicago",
	},
	"AR83": {
		ICAO:      "AR83",
		Name:      "Heard Airport",
		City:      "Leola",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 500,
		Latitude:  34.1762008667,
		Longitude: -92.7009963989,
		Timezone:  "America/Chicago",
	},
	"AR84": {
		ICAO:      "AR84",
		Name:      "Bobwhite Hill Ranch Airport",
		City:      "Mayflower",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 350,
		Latitude:  34.9664993286,
		Longitude: -92.2993011475,
		Timezone:  "America/Chicago",
	},
	"AR88": {
		ICAO:      "AR88",
		Name:      "Poe's Airport",
		City:      "Conway",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 295,
		Latitude:  35.0547981262,
		Longitude: -92.3539962769,
		Timezone:  "America/Chicago",
	},
	"AR89": {
		ICAO:      "AR89",
		Name:      "Landers Loop Airport",
		City:      "Dover",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 730,
		Latitude:  35.400100708,
		Longitude: -93.0660018921,
		Timezone:  "America/Chicago",
	},
	"AR91": {
		ICAO:      "AR91",
		Name:      "Circle S Farms Airport",
		City:      "Lowell",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1200,
		Latitude:  36.2258987427,
		Longitude: -94.0320968628,
		Timezone:  "America/Chicago",
	},
	"AR93": {
		ICAO:      "AR93",
		Name:      "Ark-Mo Airport",
		City:      "Pollard",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 308,
		Latitude:  36.4855995178,
		Longitude: -90.2804031372,
		Timezone:  "America/Chicago",
	},
	"AR94": {
		ICAO:      "AR94",
		Name:      "Squirrel Run Airport",
		City:      "Dyer",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 415,
		Latitude:  35.4854011536,
		Longitude: -94.1382980347,
		Timezone:  "America/Chicago",
	},
	"AR97": {
		ICAO:      "AR97",
		Name:      "Chael Airport",
		City:      "Wooster",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 320,
		Latitude:  35.2201004028,
		Longitude: -92.4964981079,
		Timezone:  "America/Chicago",
	},
	"AR98": {
		ICAO:      "AR98",
		Name:      "Johnson Field",
		City:      "Mc Crory",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 208,
		Latitude:  35.2644996643,
		Longitude: -91.1890029907,
		Timezone:  "America/Chicago",
	},
	"AYAI": {
		ICAO:      "AYAI",
		IATA:      "ATP",
		Name:      "Aitape Airport",
		City:      "Aitape",
		State:     "Sandaun",
		Country:   "PG",
		Elevation: 10,
		Latitude:  -3.1436111111,
		Longitude: 142.346805556,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYAM": {
		ICAO:      "AYAM",
		IATA:      "AMU",
		Name:      "Amanab Airport",
		City:      "Amanab",
		State:     "Sandaun",
		Country:   "PG",
		Elevation: 1307,
		Latitude:  -3.586,
		Longitude: 141.214333333,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYAN": {
		ICAO:      "AYAN",
		IATA:      "ADC",
		Name:      "Andakombe Airport",
		City:      "Andekombe",
		State:     "Gulf",
		Country:   "PG",
		Elevation: 3600,
		Latitude:  -7.1372222222,
		Longitude: 145.744722222,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYAO": {
		ICAO:      "AYAO",
		IATA:      "AIE",
		Name:      "Aiome Airport",
		City:      "Aiome",
		State:     "Madang",
		Country:   "PG",
		Elevation: 350,
		Latitude:  -5.145699978,
		Longitude: 144.7307003,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYAQ": {
		ICAO:      "AYAQ",
		IATA:      "KPM",
		Name:      "Kompiam Airport",
		State:     "Enga",
		Country:   "PG",
		Elevation: 5100,
		Latitude:  -5.3816666667,
		Longitude: 143.924722222,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYAT": {
		ICAO:      "AYAT",
		IATA:      "AUJ",
		Name:      "Ambunti Airport",
		City:      "Ambunti",
		State:     "East Sepik",
		Country:   "PG",
		Elevation: 50,
		Latitude:  -4.2156666667,
		Longitude: 142.823166667,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYAW": {
		ICAO:      "AYAW",
		IATA:      "AWB",
		Name:      "Awaba Airport",
		City:      "Awaba",
		State:     "Western Province",
		Country:   "PG",
		Elevation: 32,
		Latitude:  -8.0061101914,
		Longitude: 142.748352051,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYAY": {
		ICAO:      "AYAY",
		IATA:      "AYU",
		Name:      "Aiyura Airport",
		City:      "Aiyura Valley",
		State:     "Eastern Highlands",
		Country:   "PG",
		Elevation: 5355,
		Latitude:  -6.3380555556,
		Longitude: 145.904166667,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYBA": {
		ICAO:      "AYBA",
		IATA:      "VMU",
		Name:      "Baimuru Airport",
		City:      "Baimuru",
		State:     "Gulf",
		Country:   "PG",
		Elevation: 27,
		Latitude:  -7.4968600273,
		Longitude: 144.819900513,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYBG": {
		ICAO:      "AYBG",
		IATA:      "BDZ",
		Name:      "Baindoung Airport",
		State:     "Morobe",
		Country:   "PG",
		Elevation: 4400,
		Latitude:  -6.3472222222,
		Longitude: 146.942222222,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYBK": {
		ICAO:      "AYBK",
		IATA:      "BUA",
		Name:      "Buka Airport",
		City:      "Buka Island",
		State:     "Bougainville",
		Country:   "PG",
		Elevation: 11,
		Latitude:  -5.4223198891,
		Longitude: 154.6730041504,
		Timezone:  "Pacific/Bougainville",
	},
	"AYBM": {
		ICAO:      "AYBM",
		IATA:      "OPU",
		Name:      "Balimo Airport",
		City:      "Balimo",
		State:     "Western Province",
		Country:   "PG",
		Elevation: 51,
		Latitude:  -8.0500001907,
		Longitude: 142.932998657,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYBU": {
		ICAO:      "AYBU",
		IATA:      "BUL",
		Name:      "Bulolo Airport",
		City:      "Bulolo",
		State:     "Morobe",
		Country:   "PG",
		Elevation: 2240,
		Latitude:  -7.2162866714,
		Longitude: 146.649541855,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYBW": {
		ICAO:      "AYBW",
		Name:      "Blackwara Airport",
		State:     "Sandaun",
		Country:   "PG",
		Elevation: 58,
		Latitude:  -2.618211,
		Longitude: 141.091971,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYCH": {
		ICAO:      "AYCH",
		IATA:      "CMU",
		Name:      "Chimbu Airport",
		City:      "Kundiawa",
		State:     "Chimbu",
		Country:   "PG",
		Elevation: 4974,
		Latitude:  -6.0242900848,
		Longitude: 144.970993042,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYDE": {
		ICAO:      "AYDE",
		IATA:      "DER",
		Name:      "Derim Airport",
		City:      "Derim",
		State:     "Morobe",
		Country:   "PG",
		Elevation: 4850,
		Latitude:  -6.1447222222,
		Longitude: 147.107222222,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYDS": {
		ICAO:      "AYDS",
		Name:      "Dusin Airport",
		City:      "Dusin",
		State:     "East Sepik",
		Country:   "PG",
		Elevation: 6300,
		Latitude:  -5.184135,
		Longitude: 144.411965,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYDU": {
		ICAO:      "AYDU",
		IATA:      "DAU",
		Name:      "Daru Airport",
		City:      "Daru",
		State:     "Western Province",
		Country:   "PG",
		Elevation: 20,
		Latitude:  -9.0867595673,
		Longitude: 143.207992554,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYED": {
		ICAO:      "AYED",
		IATA:      "XYR",
		Name:      "Edwaki Airport",
		City:      "Yellow River Mission",
		State:     "Sandaun",
		Country:   "PG",
		Elevation: 190,
		Latitude:  -3.883986,
		Longitude: 141.792234,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYFA": {
		ICAO:      "AYFA",
		IATA:      "FNE",
		Name:      "Fane Airport",
		City:      "Fane Mission",
		State:     "Central Province",
		Country:   "PG",
		Elevation: 4500,
		Latitude:  -8.5492777778,
		Longitude: 147.085833333,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYFI": {
		ICAO:      "AYFI",
		IATA:      "FIN",
		Name:      "Finschhafen Airport",
		City:      "Buki",
		State:     "Morobe",
		Country:   "PG",
		Elevation: 60,
		Latitude:  -6.621750109,
		Longitude: 147.85405,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYFR": {
		ICAO:      "AYFR",
		IATA:      "FAQ",
		Name:      "Frieda River Airport",
		City:      "Frieda River",
		State:     "East Sepik",
		Country:   "PG",
		Elevation: 217,
		Latitude:  -4.610468,
		Longitude: 141.960138,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYGA": {
		ICAO:      "AYGA",
		IATA:      "GKA",
		Name:      "Goroka Airport",
		City:      "Goronka",
		State:     "Eastern Highlands",
		Country:   "PG",
		Elevation: 5282,
		Latitude:  -6.0816898346,
		Longitude: 145.391998291,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYGB": {
		ICAO:      "AYGB",
		Name:      "Gobe Airport",
		City:      "Gobe",
		State:     "Gulf",
		Country:   "PG",
		Latitude:  -6.8775170598,
		Longitude: 143.721942902,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYGG": {
		ICAO:      "AYGG",
		IATA:      "GRL",
		Name:      "Garasa Airport",
		City:      "Au",
		State:     "Morobe",
		Country:   "PG",
		Elevation: 2539,
		Latitude:  -7.980713,
		Longitude: 147.213461,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYGI": {
		ICAO:      "AYGI",
		IATA:      "GAR",
		Name:      "Garaina Airport",
		City:      "Garaina",
		State:     "Morobe",
		Country:   "PG",
		Elevation: 2497,
		Latitude:  -7.8756250275,
		Longitude: 147.141265869,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYGN": {
		ICAO:      "AYGN",
		IATA:      "GUR",
		Name:      "Gurney Airport",
		City:      "Gurney",
		State:     "Milne Bay",
		Country:   "PG",
		Elevation: 88,
		Latitude:  -10.3114995956,
		Longitude: 150.333999634,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYGP": {
		ICAO:      "AYGP",
		IATA:      "GAP",
		Name:      "Gusap Airport",
		City:      "Gusap",
		State:     "Morobe",
		Country:   "PG",
		Elevation: 1504,
		Latitude:  -6.053196,
		Longitude: 145.959047,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYGR": {
		ICAO:      "AYGR",
		IATA:      "PNP",
		Name:      "Girua Airport",
		City:      "Popondetta",
		State:     "Northern Province",
		Country:   "PG",
		Elevation: 311,
		Latitude:  -8.8045396805,
		Longitude: 148.309005737,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYGT": {
		ICAO:      "AYGT",
		IATA:      "GMI",
		Name:      "Gasmata Island Airport",
		City:      "Gasmata Island",
		State:     "West New Britain",
		Country:   "PG",
		Elevation: 23,
		Latitude:  -6.2711100578,
		Longitude: 150.330993652,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYGV": {
		ICAO:      "AYGV",
		IATA:      "GVI",
		Name:      "Green River Airport",
		City:      "Green River",
		State:     "Sandaun",
		Country:   "PG",
		Elevation: 281,
		Latitude:  -3.9022428989,
		Longitude: 141.170516968,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYGW": {
		ICAO:      "AYGW",
		Name:      "Guwasa Airport",
		City:      "Guwasa",
		State:     "Eastern Highlands",
		Country:   "PG",
		Elevation: 4800,
		Latitude:  -6.465838,
		Longitude: 145.083968,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYHA": {
		ICAO:      "AYHA",
		Name:      "Haia Airport",
		State:     "Chimbu",
		Country:   "PG",
		Elevation: 2434,
		Latitude:  -6.707421,
		Longitude: 144.997398,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYHK": {
		ICAO:      "AYHK",
		IATA:      "HKN",
		Name:      "Kimbe Airport",
		City:      "Hoskins",
		State:     "West New Britain",
		Country:   "PG",
		Elevation: 66,
		Latitude:  -5.4621701241,
		Longitude: 150.4049987793,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYIQ": {
		ICAO:      "AYIQ",
		IATA:      "KIE",
		Name:      "Aropa Airport",
		City:      "Kieta",
		State:     "Bougainville Island",
		Country:   "PG",
		Elevation: 10,
		Latitude:  -6.305,
		Longitude: 155.727778,
		Timezone:  "Pacific/Bougainville",
	},
	"AYKA": {
		ICAO:      "AYKA",
		IATA:      "LSA",
		Name:      "Losuia Airport",
		City:      "Losuia",
		State:     "Milne Bay",
		Country:   "PG",
		Elevation: 27,
		Latitude:  -8.5058202744,
		Longitude: 151.0809936523,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYKB": {
		ICAO:      "AYKB",
		IATA:      "KBM",
		Name:      "Kabwum",
		State:     "Morobe",
		Country:   "PG",
		Elevation: 4450,
		Latitude:  -6.1554722222,
		Longitude: 147.191472222,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYKC": {
		ICAO:      "AYKC",
		IATA:      "KDR",
		Name:      "Kandrian Airport",
		City:      "Kandrian",
		State:     "West New Britain",
		Country:   "PG",
		Elevation: 280,
		Latitude:  -6.2659,
		Longitude: 149.5581,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYKI": {
		ICAO:      "AYKI",
		IATA:      "UNG",
		Name:      "Kiunga Airport",
		City:      "Kiunga",
		State:     "Western Province",
		Country:   "PG",
		Elevation: 88,
		Latitude:  -6.1257100105,
		Longitude: 141.2819976807,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYKK": {
		ICAO:      "AYKK",
		IATA:      "KRI",
		Name:      "Kikori Airport",
		City:      "Kikori",
		State:     "Gulf",
		Country:   "PG",
		Elevation: 50,
		Latitude:  -7.4243798256,
		Longitude: 144.2500762939,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYKM": {
		ICAO:      "AYKM",
		IATA:      "KMA",
		Name:      "Kerema Airport",
		City:      "Kerema",
		State:     "Gulf",
		Country:   "PG",
		Elevation: 10,
		Latitude:  -7.9636101723,
		Longitude: 145.770996094,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYKO": {
		ICAO:      "AYKO",
		IATA:      "KKD",
		Name:      "Kokoda Airport",
		City:      "Kokoda",
		State:     "Northern Province",
		Country:   "PG",
		Elevation: 1240,
		Latitude:  -8.8846788406,
		Longitude: 147.730957031,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYKT": {
		ICAO:      "AYKT",
		IATA:      "KZF",
		Name:      "Kaintiba Airport",
		City:      "Kaintiba",
		State:     "Gulf",
		Country:   "PG",
		Elevation: 2050,
		Latitude:  -7.50025,
		Longitude: 146.033833333,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYKU": {
		ICAO:      "AYKU",
		IATA:      "KUQ",
		Name:      "Kuri Airport",
		City:      "Kuri",
		State:     "Gulf",
		Country:   "PG",
		Elevation: 35,
		Latitude:  -7.135083,
		Longitude: 143.276111,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYKV": {
		ICAO:      "AYKV",
		IATA:      "KVG",
		Name:      "Kavieng Airport",
		City:      "Kavieng",
		State:     "New Ireland",
		Country:   "PG",
		Elevation: 7,
		Latitude:  -2.5794000626,
		Longitude: 150.807998657,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYKW": {
		ICAO:      "AYKW",
		IATA:      "KWO",
		Name:      "Kawito Airport",
		City:      "Kawito",
		State:     "Western Province",
		Country:   "PG",
		Elevation: 72,
		Latitude:  -7.9770069122,
		Longitude: 142.823577881,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYKY": {
		ICAO:      "AYKY",
		IATA:      "LNV",
		Name:      "Londolovit Airport",
		City:      "Londolovit",
		State:     "New Ireland",
		Country:   "PG",
		Elevation: 167,
		Latitude:  -3.043610096,
		Longitude: 152.628997803,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYLM": {
		ICAO:      "AYLM",
		IATA:      "LMY",
		Name:      "Lake Murray Airport",
		City:      "Lake Murray",
		State:     "Western Province",
		Country:   "PG",
		Elevation: 52,
		Latitude:  -7.0099201202,
		Longitude: 141.494003296,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYLU": {
		ICAO:      "AYLU",
		IATA:      "LMI",
		Name:      "Lumi Airport",
		City:      "Lumi",
		State:     "Sandaun",
		Country:   "PG",
		Elevation: 1750,
		Latitude:  -3.4768395113,
		Longitude: 142.041292191,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYMC": {
		ICAO:      "AYMC",
		IATA:      "MYX",
		Name:      "Menyamya Airport",
		City:      "Menyamya",
		State:     "Morobe",
		Country:   "PG",
		Elevation: 3880,
		Latitude:  -7.2116666667,
		Longitude: 146.019361111,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYMD": {
		ICAO:      "AYMD",
		IATA:      "MAG",
		Name:      "Madang Airport",
		City:      "Madang",
		State:     "Madang",
		Country:   "PG",
		Elevation: 20,
		Latitude:  -5.2070798874,
		Longitude: 145.789001465,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYMH": {
		ICAO:      "AYMH",
		IATA:      "HGU",
		Name:      "Mount Hagen Kagamuga Airport",
		City:      "Mount Hagen",
		State:     "Western Highlands",
		Country:   "PG",
		Elevation: 5388,
		Latitude:  -5.826789856,
		Longitude: 144.296005249,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYMI": {
		ICAO:      "AYMI",
		IATA:      "MXK",
		Name:      "Mindik Airport",
		City:      "Mindik",
		State:     "Morobe",
		Country:   "PG",
		Elevation: 4200,
		Latitude:  -6.4716666667,
		Longitude: 147.441138889,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYMK": {
		ICAO:      "AYMK",
		Name:      "Mok Airport",
		City:      "Mok",
		State:     "West New Britain",
		Country:   "PG",
		Elevation: 1050,
		Latitude:  -5.730238,
		Longitude: 149.056583,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYML": {
		ICAO:      "AYML",
		IATA:      "GUV",
		Name:      "Mougulu Airport",
		City:      "Mougulu",
		State:     "Western Province",
		Country:   "PG",
		Elevation: 825,
		Latitude:  -6.280859,
		Longitude: 142.420775,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYMN": {
		ICAO:      "AYMN",
		IATA:      "MDU",
		Name:      "Mendi Airport",
		State:     "Southern Highlands",
		Country:   "PG",
		Elevation: 5680,
		Latitude:  -6.1477398872,
		Longitude: 143.6569976807,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYMO": {
		ICAO:      "AYMO",
		IATA:      "MAS",
		Name:      "Momote Airport",
		State:     "Manus",
		Country:   "PG",
		Elevation: 12,
		Latitude:  -2.0618898869,
		Longitude: 147.4239959717,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYMR": {
		ICAO:      "AYMR",
		IATA:      "MXH",
		Name:      "Moro Airport",
		City:      "Moro",
		State:     "Southern Highlands",
		Country:   "PG",
		Elevation: 2740,
		Latitude:  -6.3633298874,
		Longitude: 143.238006592,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYMS": {
		ICAO:      "AYMS",
		IATA:      "MIS",
		Name:      "Misima Island Airport",
		City:      "Misima Island",
		State:     "Milne Bay",
		Country:   "PG",
		Elevation: 26,
		Latitude:  -10.6892004013,
		Longitude: 152.837997437,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYNE": {
		ICAO:      "AYNE",
		IATA:      "GBF",
		Name:      "Negarbo(Negabo) Airport",
		City:      "Negarbo",
		State:     "Chimbu",
		Country:   "PG",
		Elevation: 4500,
		Latitude:  -6.567778,
		Longitude: 144.703058,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYNG": {
		ICAO:      "AYNG",
		IATA:      "MFO",
		Name:      "Manguna Airport",
		City:      "Manguna",
		State:     "East New Britain",
		Country:   "PG",
		Elevation: 187,
		Latitude:  -5.577778,
		Longitude: 151.792333,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYNU": {
		ICAO:      "AYNU",
		IATA:      "UKU",
		Name:      "Nuku Airport",
		City:      "Nuku",
		State:     "Sandaun",
		Country:   "PG",
		Elevation: 870,
		Latitude:  -3.676651,
		Longitude: 142.484334,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYNZ": {
		ICAO:      "AYNZ",
		IATA:      "LAE",
		Name:      "Lae Nadzab Airport",
		City:      "Nadzab",
		State:     "Morobe",
		Country:   "PG",
		Elevation: 239,
		Latitude:  -6.5698299408,
		Longitude: 146.725997925,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYOG": {
		ICAO:      "AYOG",
		IATA:      "OGE",
		Name:      "Ogeranang Airport",
		State:     "Morobe",
		Country:   "PG",
		Elevation: 5850,
		Latitude:  -6.46675,
		Longitude: 147.364166667,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYOM": {
		ICAO:      "AYOM",
		IATA:      "OSE",
		Name:      "Omora Airport",
		City:      "Omora",
		State:     "Morobe",
		Country:   "PG",
		Elevation: 2540,
		Latitude:  -7.8277777778,
		Longitude: 147.080638889,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYPD": {
		ICAO:      "AYPD",
		IATA:      "PDI",
		Name:      "Pindiu Airport",
		City:      "Pindiu",
		State:     "Morobe",
		Country:   "PG",
		Elevation: 2995,
		Latitude:  -6.4451388889,
		Longitude: 147.515833333,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYPU": {
		ICAO:      "AYPU",
		Name:      "Puri Airport",
		City:      "Puri",
		State:     "Southern Highlands",
		Country:   "PG",
		Elevation: 6050,
		Latitude:  -6.4751611111,
		Longitude: 143.987222222,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYPW": {
		ICAO:      "AYPW",
		Name:      "Panakawa Airport",
		State:     "Western Province",
		Country:   "PG",
		Elevation: 42,
		Latitude:  -7.6720555556,
		Longitude: 143.124722222,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYPY": {
		ICAO:      "AYPY",
		IATA:      "POM",
		Name:      "Port Moresby Jacksons International Airport",
		City:      "Port Moresby",
		State:     "National Capital",
		Country:   "PG",
		Elevation: 146,
		Latitude:  -9.4433803558,
		Longitude: 147.2200012207,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYQA": {
		ICAO:      "AYQA",
		IATA:      "KRJ",
		Name:      "Karawari Airstrip",
		State:     "East Sepik",
		Country:   "PG",
		Elevation: 70,
		Latitude:  -4.5966666667,
		Longitude: 143.5225,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYRG": {
		ICAO:      "AYRG",
		IATA:      "RMN",
		Name:      "Rumginae Airport",
		State:     "Western Province",
		Country:   "PG",
		Elevation: 160,
		Latitude:  -5.8972222222,
		Longitude: 141.271666667,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYRI": {
		ICAO:      "AYRI",
		IATA:      "KMR",
		Name:      "Karimui Airport",
		City:      "Karimui",
		State:     "Chimbu",
		Country:   "PG",
		Elevation: 3640,
		Latitude:  -6.4921,
		Longitude: 144.823,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYSA": {
		ICAO:      "AYSA",
		IATA:      "SBE",
		Name:      "Suabi Airport",
		State:     "Western Province",
		Country:   "PG",
		Elevation: 465,
		Latitude:  -6.1044444444,
		Longitude: 142.278333333,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYSE": {
		ICAO:      "AYSE",
		IATA:      "NIS",
		Name:      "Simberi Airport",
		City:      "Simberi Island",
		State:     "New Ireland",
		Country:   "PG",
		Elevation: 10,
		Latitude:  -2.6622222222,
		Longitude: 151.997777778,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYSG": {
		ICAO:      "AYSG",
		IATA:      "SIL",
		Name:      "Sila Airport",
		City:      "Sila Mission",
		State:     "Northern Province",
		Country:   "PG",
		Elevation: 2230,
		Latitude:  -9.0735555556,
		Longitude: 148.38925,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYSJ": {
		ICAO:      "AYSJ",
		IATA:      "SIM",
		Name:      "Simbai Airport",
		City:      "Simbai",
		State:     "Western Highlands",
		Country:   "PG",
		Elevation: 5804,
		Latitude:  -5.2786111111,
		Longitude: 144.544722222,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYST": {
		ICAO:      "AYST",
		Name:      "Sturt Island Airport",
		State:     "Western Province",
		Country:   "PG",
		Elevation: 40,
		Latitude:  -8.1525,
		Longitude: 142.268444444,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYSU": {
		ICAO:      "AYSU",
		IATA:      "SKC",
		Name:      "Suki Airport",
		City:      "Suki",
		State:     "Western Province",
		Country:   "PG",
		Elevation: 24,
		Latitude:  -8.0466,
		Longitude: 141.7222,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYTA": {
		ICAO:      "AYTA",
		IATA:      "TIZ",
		Name:      "Tari Airport",
		City:      "Tari",
		State:     "Southern Highlands",
		Country:   "PG",
		Elevation: 5500,
		Latitude:  -5.8449997902,
		Longitude: 142.947998047,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYTB": {
		ICAO:      "AYTB",
		IATA:      "TBG",
		Name:      "Tabubil Airport",
		City:      "Tabubil",
		State:     "Western Province",
		Country:   "PG",
		Elevation: 1570,
		Latitude:  -5.2786102295,
		Longitude: 141.2259979248,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYTE": {
		ICAO:      "AYTE",
		IATA:      "TFM",
		Name:      "Telefomin Airport",
		City:      "Telefomin",
		State:     "Western Province",
		Country:   "PG",
		Elevation: 4950,
		Latitude:  -5.1260804642,
		Longitude: 141.641921997,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYTI": {
		ICAO:      "AYTI",
		IATA:      "TPI",
		Name:      "Tapini Airport",
		City:      "Tapini",
		State:     "Central Province",
		Country:   "PG",
		Elevation: 3100,
		Latitude:  -8.3566666667,
		Longitude: 146.989166667,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYTJ": {
		ICAO:      "AYTJ",
		IATA:      "TAJ",
		Name:      "Tadji Airport",
		City:      "Aitape",
		State:     "Sandaun",
		Country:   "PG",
		Elevation: 33,
		Latitude:  -3.1981918035,
		Longitude: 142.430963516,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYTK": {
		ICAO:      "AYTK",
		IATA:      "RAB",
		Name:      "Tokua Airport",
		City:      "Tokua",
		State:     "East New Britain",
		Country:   "PG",
		Elevation: 32,
		Latitude:  -4.3404598236,
		Longitude: 152.380004883,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYTN": {
		ICAO:      "AYTN",
		IATA:      "TKW",
		Name:      "Tekin Airport",
		City:      "Tekin",
		State:     "Sandaun",
		Country:   "PG",
		Elevation: 5785,
		Latitude:  -5.2436666667,
		Longitude: 142.165194444,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYTU": {
		ICAO:      "AYTU",
		IATA:      "TFI",
		Name:      "Tufi Airport",
		City:      "Tufi",
		State:     "Northern Province",
		Country:   "PG",
		Elevation: 85,
		Latitude:  -9.0759544373,
		Longitude: 149.319839478,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYVN": {
		ICAO:      "AYVN",
		IATA:      "VAI",
		Name:      "Vanimo Airport",
		State:     "Sandaun",
		Country:   "PG",
		Elevation: 10,
		Latitude:  -2.6971700191,
		Longitude: 141.3020019531,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYWB": {
		ICAO:      "AYWB",
		IATA:      "WAO",
		Name:      "Wabo Airport",
		City:      "Wabo",
		State:     "Gulf",
		Country:   "PG",
		Elevation: 132,
		Latitude:  -6.9894444444,
		Longitude: 145.075111111,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYWD": {
		ICAO:      "AYWD",
		IATA:      "WBM",
		Name:      "Wapenamanda Airport",
		State:     "Enga",
		Country:   "PG",
		Elevation: 5889,
		Latitude:  -5.6433000565,
		Longitude: 143.8950042725,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYWG": {
		ICAO:      "AYWG",
		IATA:      "AGL",
		Name:      "Wanigela Airport",
		State:     "Northern Province",
		Country:   "PG",
		Elevation: 53,
		Latitude:  -9.3375,
		Longitude: 149.155555556,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYWK": {
		ICAO:      "AYWK",
		IATA:      "WWK",
		Name:      "Wewak International Airport",
		City:      "Wewak",
		State:     "East Sepik",
		Country:   "PG",
		Elevation: 19,
		Latitude:  -3.5838301182,
		Longitude: 143.669006348,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYWO": {
		ICAO:      "AYWO",
		IATA:      "WOA",
		Name:      "Wonenara Airport",
		City:      "Wonenara",
		State:     "Eastern Highlands",
		Country:   "PG",
		Elevation: 5028,
		Latitude:  -6.7968611111,
		Longitude: 145.891944444,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYWS": {
		ICAO:      "AYWS",
		IATA:      "WSU",
		Name:      "Wasu Airport",
		City:      "Wasu",
		State:     "Morobe",
		Country:   "PG",
		Elevation: 34,
		Latitude:  -5.9617094492,
		Longitude: 147.19822526,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYWT": {
		ICAO:      "AYWT",
		IATA:      "WTP",
		Name:      "Woitape Airport",
		City:      "Fatima Mission",
		State:     "Central Province",
		Country:   "PG",
		Elevation: 5200,
		Latitude:  -8.5458333333,
		Longitude: 147.2525,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AYWU": {
		ICAO:      "AYWU",
		IATA:      "WUG",
		Name:      "Wau Airport",
		City:      "Wau",
		State:     "Morobe",
		Country:   "PG",
		Elevation: 3600,
		Latitude:  -7.3455555556,
		Longitude: 146.718611111,
		Timezone:  "Pacific/Port_Moresby",
	},
	"AZ00": {
		ICAO:      "AZ00",
		Name:      "Morton Field",
		City:      "Dateland",
		State:     "Arizona",
		Country:   "US",
		Elevation: 475,
		Latitude:  32.8058013916,
		Longitude: -113.5070037842,
		Timezone:  "America/Phoenix",
	},
	"AZ01": {
		ICAO:      "AZ01",
		Name:      "Outback Ranch Airstrip",
		City:      "Wenden",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1980,
		Latitude:  33.8477783203,
		Longitude: -113.570274353,
		Timezone:  "America/Phoenix",
	},
	"AZ03": {
		ICAO:      "AZ03",
		Name:      "Cliff Dwellers Lodge Airport",
		City:      "Marble Canyon",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4217,
		Latitude:  36.7344017029,
		Longitude: -111.7529983521,
		Timezone:  "America/Phoenix",
	},
	"AZ04": {
		ICAO:      "AZ04",
		Name:      "Hillair Dirt Strip",
		City:      "Aguila",
		State:     "Arizona",
		Country:   "US",
		Elevation: 2210,
		Latitude:  33.9192008972,
		Longitude: -113.1900024414,
		Timezone:  "America/Phoenix",
	},
	"AZ05": {
		ICAO:      "AZ05",
		Name:      "Lakeside Airpark",
		City:      "Buckeye",
		State:     "Arizona",
		Country:   "US",
		Elevation: 763,
		Latitude:  33.1124992371,
		Longitude: -112.6620025635,
		Timezone:  "America/Phoenix",
	},
	"AZ06": {
		ICAO:      "AZ06",
		Name:      "Dateland Airfield",
		City:      "Dateland",
		State:     "Arizona",
		Country:   "US",
		Elevation: 450,
		Latitude:  32.817199707,
		Longitude: -113.5270004272,
		Timezone:  "America/Phoenix",
	},
	"AZ09": {
		ICAO:      "AZ09",
		Name:      "Sergio Private Airport",
		City:      "Lake Havasu City",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1900,
		Latitude:  34.7999992371,
		Longitude: -114.1129989624,
		Timezone:  "America/Phoenix",
	},
	"AZ13": {
		ICAO:      "AZ13",
		Name:      "Schu Ranch Airport",
		City:      "Maricopa",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1410,
		Latitude:  33.016998291,
		Longitude: -112.28099823,
		Timezone:  "America/Phoenix",
	},
	"AZ14": {
		ICAO:      "AZ14",
		Name:      "Ammon Airport",
		City:      "Willcox",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4520,
		Latitude:  32.02470016,
		Longitude: -109.9570007,
		Timezone:  "America/Phoenix",
	},
	"AZ17": {
		ICAO:      "AZ17",
		Name:      "Circle H Ranch Airport",
		City:      "Douglas",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4680,
		Latitude:  31.502199173,
		Longitude: -109.3909988403,
		Timezone:  "America/Phoenix",
	},
	"AZ19": {
		ICAO:      "AZ19",
		Name:      "C & L Ranch Ultralightport",
		City:      "Sonoita",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4600,
		Latitude:  31.7378005981,
		Longitude: -110.6259994507,
		Timezone:  "America/Phoenix",
	},
	"AZ21": {
		ICAO:      "AZ21",
		Name:      "Four Pillars Airport",
		City:      "Huachuca City/Tombstone",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4100,
		Latitude:  31.7458000183,
		Longitude: -110.2480010986,
		Timezone:  "America/Phoenix",
	},
	"AZ25": {
		ICAO:      "AZ25",
		Name:      "Goldfield Ranch Airport",
		City:      "Fountain Hills",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1720,
		Latitude:  33.5999984741,
		Longitude: -111.6050033569,
		Timezone:  "America/Phoenix",
	},
	"AZ26": {
		ICAO:      "AZ26",
		Name:      "Evelyn Field",
		City:      "Palominas",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4242,
		Latitude:  31.3691997528,
		Longitude: -110.125,
		Timezone:  "America/Hermosillo",
	},
	"AZ27": {
		ICAO:      "AZ27",
		Name:      "Big Springs Ranch Airport",
		City:      "Prescott",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4421,
		Latitude:  34.9099998474,
		Longitude: -112.533996582,
		Timezone:  "America/Phoenix",
	},
	"AZ28": {
		ICAO:      "AZ28",
		Name:      "Thunder Ridge Airpark",
		City:      "Morristown",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1915,
		Latitude:  33.8214988708,
		Longitude: -112.5729980469,
		Timezone:  "America/Phoenix",
	},
	"AZ31": {
		ICAO:      "AZ31",
		Name:      "Serene Field",
		City:      "Maricopa",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1456,
		Latitude:  32.9967002869,
		Longitude: -112.2720031738,
		Timezone:  "America/Phoenix",
	},
	"AZ32": {
		ICAO:      "AZ32",
		Name:      "Roesner Ranch Airport",
		City:      "Morristown",
		State:     "Arizona",
		Country:   "US",
		Elevation: 2060,
		Latitude:  33.8474998474,
		Longitude: -112.5879974365,
		Timezone:  "America/Phoenix",
	},
	"AZ34": {
		ICAO:      "AZ34",
		Name:      "Massey Farm Airport",
		City:      "Yucca",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1630,
		Latitude:  34.691428,
		Longitude: -114.097739,
		Timezone:  "America/Phoenix",
	},
	"AZ40": {
		ICAO:      "AZ40",
		Name:      "Sheepy Hollow Ranch Airfield",
		City:      "Salome",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1930,
		Latitude:  33.7874183655,
		Longitude: -113.642417908,
		Timezone:  "America/Phoenix",
	},
	"AZ43": {
		ICAO:      "AZ43",
		Name:      "Hidden Valley Airport",
		City:      "Maricopa",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1245,
		Latitude:  33.0638999939,
		Longitude: -112.1640014648,
		Timezone:  "America/Phoenix",
	},
	"AZ45": {
		ICAO:      "AZ45",
		Name:      "China Peak Observatory Airport",
		City:      "Klondyke",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4800,
		Latitude:  32.7291984558,
		Longitude: -110.2959976196,
		Timezone:  "America/Phoenix",
	},
	"AZ46": {
		ICAO:      "AZ46",
		Name:      "H&H Ranch Airstrip",
		City:      "Chino Valley",
		State:     "Arizona",
		Country:   "US",
		Elevation: 5025,
		Latitude:  34.7663993835,
		Longitude: -112.5490036011,
		Timezone:  "America/Phoenix",
	},
	"AZ49": {
		ICAO:      "AZ49",
		Name:      "Walter Ranch Airport",
		City:      "Stanfield",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1615,
		Latitude:  32.8955993652,
		Longitude: -112.2539978027,
		Timezone:  "America/Phoenix",
	},
	"AZ50": {
		ICAO:      "AZ50",
		Name:      "Triangle Airpark",
		City:      "White Hills",
		State:     "Arizona",
		Country:   "US",
		Elevation: 2419,
		Latitude:  35.7146987915,
		Longitude: -114.4810028076,
		Timezone:  "America/Phoenix",
	},
	"AZ57": {
		ICAO:      "AZ57",
		Name:      "Pilots Rest Airport",
		City:      "Paulden",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4482,
		Latitude:  34.9308013916,
		Longitude: -112.5120010376,
		Timezone:  "America/Phoenix",
	},
	"AZ60": {
		ICAO:      "AZ60",
		Name:      "Coyner Airstrip",
		City:      "Litchfield Park",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1170,
		Latitude:  33.5050010681,
		Longitude: -112.4729995728,
		Timezone:  "America/Phoenix",
	},
	"AZ61": {
		ICAO:      "AZ61",
		Name:      "G.M. Ranch Airport",
		City:      "Stanfield",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1616,
		Latitude:  32.9016990662,
		Longitude: -112.2379989624,
		Timezone:  "America/Phoenix",
	},
	"AZ63": {
		ICAO:      "AZ63",
		Name:      "Twin Hawks Airpark",
		City:      "Redrock",
		State:     "Arizona",
		Country:   "US",
		Elevation: 2290,
		Latitude:  32.5278015137,
		Longitude: -111.1959991455,
		Timezone:  "America/Phoenix",
	},
	"AZ64": {
		ICAO:      "AZ64",
		Name:      "Rio Vista Hills Airport",
		City:      "Wickenburg",
		State:     "Arizona",
		Country:   "US",
		Elevation: 2225,
		Latitude:  33.9523010254,
		Longitude: -112.68699646,
		Timezone:  "America/Phoenix",
	},
	"AZ67": {
		ICAO:      "AZ67",
		Name:      "El Tiro Gliderport",
		City:      "Tucson",
		State:     "Arizona",
		Country:   "US",
		Elevation: 2100,
		Latitude:  32.4269981384,
		Longitude: -111.3899993896,
		Timezone:  "America/Phoenix",
	},
	"AZ71": {
		ICAO:      "AZ71",
		Name:      "Cooper Ranch Airport",
		City:      "Yarnell",
		State:     "Arizona",
		Country:   "US",
		Elevation: 3440,
		Latitude:  34.2153015137,
		Longitude: -112.5360031128,
		Timezone:  "America/Phoenix",
	},
	"AZ74": {
		ICAO:      "AZ74",
		Name:      "Lone Mountain International Airport",
		City:      "Patagonia",
		State:     "Arizona",
		Country:   "US",
		Elevation: 5350,
		Latitude:  31.3551006317,
		Longitude: -110.3740005493,
		Timezone:  "America/Hermosillo",
	},
	"AZ77": {
		ICAO:      "AZ77",
		Name:      "Sunrise Ranch Airport",
		City:      "Williams",
		State:     "Arizona",
		Country:   "US",
		Elevation: 6958,
		Latitude:  35.2383003235,
		Longitude: -111.9359970093,
		Timezone:  "America/Phoenix",
	},
	"AZ78": {
		ICAO:      "AZ78",
		Name:      "Mel's Ranch Airport",
		City:      "Maricopa",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1420,
		Latitude:  33.011100769,
		Longitude: -112.2819976807,
		Timezone:  "America/Phoenix",
	},
	"AZ79": {
		ICAO:      "AZ79",
		Name:      "Airscrew Performance Flightpark Ultralightport",
		City:      "Glendale",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1112,
		Latitude:  33.5299987793,
		Longitude: -112.2170028687,
		Timezone:  "America/Phoenix",
	},
	"AZ82": {
		ICAO:      "AZ82",
		Name:      "Mogollon Airpark",
		City:      "Overgaard",
		State:     "Arizona",
		Country:   "US",
		Elevation: 6657,
		Latitude:  34.3973007202,
		Longitude: -110.5299987793,
		Timezone:  "America/Phoenix",
	},
	"AZ85": {
		ICAO:      "AZ85",
		Name:      "Tonopah Airport",
		City:      "Tonopah",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1248,
		Latitude:  33.5339012146,
		Longitude: -112.9599990845,
		Timezone:  "America/Phoenix",
	},
	"AZ86": {
		ICAO:      "AZ86",
		Name:      "Coyote Run Gliderport",
		City:      "Coyote Springs",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4978,
		Latitude:  34.6842002869,
		Longitude: -112.2919998169,
		Timezone:  "America/Phoenix",
	},
	"AZ90": {
		ICAO:      "AZ90",
		Name:      "Hangar Haciendas Airport",
		City:      "Laveen",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1225,
		Latitude:  33.3534011841,
		Longitude: -112.1269989014,
		Timezone:  "America/Phoenix",
	},
	"AZ97": {
		ICAO:      "AZ97",
		Name:      "X Bar 1 Ranch (Lower) Airport",
		City:      "Kingman",
		State:     "Arizona",
		Country:   "US",
		Elevation: 3710,
		Latitude:  35.3483009338,
		Longitude: -113.6890029907,
		Timezone:  "America/Phoenix",
	},
	"BGAA": {
		ICAO:      "BGAA",
		IATA:      "JEG",
		Name:      "Aasiaat Airport",
		City:      "Aasiaat",
		State:     "Qaasuitsup",
		Country:   "GL",
		Elevation: 74,
		Latitude:  68.7218017578,
		Longitude: -52.7846984863,
		Timezone:  "America/Nuuk",
	},
	"BGBW": {
		ICAO:      "BGBW",
		IATA:      "UAK",
		Name:      "Narsarsuaq Airport",
		City:      "Narsarsuaq",
		State:     "Kujalleq",
		Country:   "GL",
		Elevation: 112,
		Latitude:  61.1604995728,
		Longitude: -45.4259986877,
		Timezone:  "America/Nuuk",
	},
	"BGCO": {
		ICAO:      "BGCO",
		IATA:      "CNP",
		Name:      "Neerlerit Inaat Airport",
		City:      "Neerlerit Inaat",
		State:     "Sermersooq",
		Country:   "GL",
		Elevation: 45,
		Latitude:  70.7431030273,
		Longitude: -22.6504993439,
		Timezone:  "America/Scoresbysund",
	},
	"BGDH": {
		ICAO:      "BGDH",
		Name:      "Danmarkshavn Landing Strip",
		City:      "Danmarkshavn",
		Country:   "GL",
		Latitude:  76.7666702271,
		Longitude: -18.6666679382,
		Timezone:  "America/Nuuk",
	},
	"BGGH": {
		ICAO:      "BGGH",
		IATA:      "GOH",
		Name:      "Godthaab / Nuuk Airport",
		City:      "Nuuk",
		Country:   "GL",
		Elevation: 283,
		Latitude:  64.19090271,
		Longitude: -51.6781005859,
		Timezone:  "America/Nuuk",
	},
	"BGJN": {
		ICAO:      "BGJN",
		IATA:      "JAV",
		Name:      "Ilulissat Airport",
		City:      "Ilulissat",
		Country:   "GL",
		Elevation: 95,
		Latitude:  69.2432022095,
		Longitude: -51.0570983887,
		Timezone:  "America/Nuuk",
	},
	"BGKK": {
		ICAO:      "BGKK",
		IATA:      "KUS",
		Name:      "Kulusuk Airport",
		City:      "Kulusuk",
		State:     "Sermersooq",
		Country:   "GL",
		Elevation: 117,
		Latitude:  65.573600769,
		Longitude: -37.1236000061,
		Timezone:  "America/Nuuk",
	},
	"BGMI": {
		ICAO:      "BGMI",
		Name:      "Station Nord Landing Strip",
		Country:   "GL",
		Elevation: 35,
		Latitude:  81.6,
		Longitude: -16.6666666667,
		Timezone:  "America/Nuuk",
	},
	"BGMQ": {
		ICAO:      "BGMQ",
		IATA:      "JSU",
		Name:      "Maniitsoq Airport",
		City:      "Maniitsoq",
		Country:   "GL",
		Elevation: 91,
		Latitude:  65.4124984741,
		Longitude: -52.9393997192,
		Timezone:  "America/Nuuk",
	},
	"BGMV": {
		ICAO:      "BGMV",
		Name:      "Mestersvig Airport",
		City:      "Mestersvig",
		Country:   "GL",
		Elevation: 52,
		Latitude:  72.2369003296,
		Longitude: -23.9319000244,
		Timezone:  "America/Scoresbysund",
	},
	"BGPT": {
		ICAO:      "BGPT",
		IATA:      "JFR",
		Name:      "Paamiut Airport",
		City:      "Paamiut",
		Country:   "GL",
		Elevation: 120,
		Latitude:  62.0147361755,
		Longitude: -49.6709365845,
		Timezone:  "America/Nuuk",
	},
	"BGQQ": {
		ICAO:      "BGQQ",
		IATA:      "NAQ",
		Name:      "Qaanaaq Airport",
		City:      "Qaanaaq",
		Country:   "GL",
		Elevation: 51,
		Latitude:  77.4886016846,
		Longitude: -69.3887023926,
		Timezone:  "America/Thule",
	},
	"BGSF": {
		ICAO:      "BGSF",
		IATA:      "SFJ",
		Name:      "Kangerlussuaq Airport",
		City:      "Kangerlussuaq",
		State:     "Qeqqata",
		Country:   "GL",
		Elevation: 165,
		Latitude:  67.0122218992,
		Longitude: -50.7116031647,
		Timezone:  "America/Nuuk",
	},
	"BGSS": {
		ICAO:      "BGSS",
		IATA:      "JHS",
		Name:      "Sisimiut Airport",
		City:      "Sisimiut",
		Country:   "GL",
		Elevation: 33,
		Latitude:  66.9513015747,
		Longitude: -53.7293014526,
		Timezone:  "America/Nuuk",
	},
	"BGTL": {
		ICAO:      "BGTL",
		IATA:      "THU",
		Name:      "Thule Air Base",
		City:      "Thule",
		State:     "Qaasuitsup",
		Country:   "GL",
		Elevation: 251,
		Latitude:  76.5311965942,
		Longitude: -68.7032012939,
		Timezone:  "America/Thule",
	},
	"BGUK": {
		ICAO:      "BGUK",
		IATA:      "JUV",
		Name:      "Upernavik Airport",
		City:      "Upernavik",
		Country:   "GL",
		Elevation: 414,
		Latitude:  72.7901992798,
		Longitude: -56.1305999756,
		Timezone:  "America/Nuuk",
	},
	"BGUQ": {
		ICAO:      "BGUQ",
		IATA:      "JQA",
		Name:      "Qaarsut Airport",
		City:      "Uummannaq",
		Country:   "GL",
		Elevation: 289,
		Latitude:  70.7341995239,
		Longitude: -52.6962013245,
		Timezone:  "America/Nuuk",
	},
	"BIAE": {
		ICAO:      "BIAE",
		Name:      "Arngerdareyri Airport",
		City:      "Arngerdareyri",
		State:     "Westfjords",
		Country:   "IS",
		Elevation: 20,
		Latitude:  65.9047012329,
		Longitude: -22.3633003235,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIAL": {
		ICAO:      "BIAL",
		Name:      "Alftaver Airport",
		City:      "Alftaver",
		State:     "South",
		Country:   "IS",
		Elevation: 160,
		Latitude:  63.5400009155,
		Longitude: -18.4493999481,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIAR": {
		ICAO:      "BIAR",
		IATA:      "AEY",
		Name:      "Akureyri Airport",
		City:      "Akureyri",
		State:     "Northeast",
		Country:   "IS",
		Elevation: 6,
		Latitude:  65.6600036621,
		Longitude: -18.0727005005,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIBA": {
		ICAO:      "BIBA",
		Name:      "Bakki Airport",
		City:      "Bakki",
		State:     "South",
		Country:   "IS",
		Elevation: 45,
		Latitude:  63.556098938,
		Longitude: -20.1375007629,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIBD": {
		ICAO:      "BIBD",
		IATA:      "BIU",
		Name:      "Bildudalur Airport",
		City:      "Bildudalur",
		State:     "Westfjords",
		Country:   "IS",
		Elevation: 18,
		Latitude:  65.6412963867,
		Longitude: -23.5461997986,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIBF": {
		ICAO:      "BIBF",
		IATA:      "BGJ",
		Name:      "Borgarfjordur eystri Airport",
		City:      "Borgarfjordur eystri",
		State:     "East",
		Country:   "IS",
		Elevation: 80,
		Latitude:  65.5164031982,
		Longitude: -13.8050003052,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIBI": {
		ICAO:      "BIBI",
		Name:      "Baeir Airport",
		City:      "Baeir",
		State:     "Westfjords",
		Country:   "IS",
		Elevation: 60,
		Latitude:  66.0999984741,
		Longitude: -22.5669994354,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIBK": {
		ICAO:      "BIBK",
		IATA:      "BJD",
		Name:      "Bakkafjordur Airport",
		City:      "Bakkafjordur",
		State:     "Northeast",
		Country:   "IS",
		Elevation: 14,
		Latitude:  66.0218963623,
		Longitude: -14.8243999481,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIBL": {
		ICAO:      "BIBL",
		IATA:      "BLO",
		Name:      "Hjaltabakki Airport",
		City:      "Blonduos",
		State:     "Northwest",
		Country:   "IS",
		Elevation: 131,
		Latitude:  65.6449966431,
		Longitude: -20.2875003815,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIBR": {
		ICAO:      "BIBR",
		IATA:      "BQD",
		Name:      "Budardalur Airport",
		City:      "Budardalur",
		State:     "West",
		Country:   "IS",
		Elevation: 131,
		Latitude:  65.075302124,
		Longitude: -21.8003005981,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIBV": {
		ICAO:      "BIBV",
		IATA:      "BXV",
		Name:      "Breiddalsvik Airport",
		City:      "Breiddalsvik",
		State:     "East",
		Country:   "IS",
		Elevation: 8,
		Latitude:  64.7900009155,
		Longitude: -14.0228004456,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIDA": {
		ICAO:      "BIDA",
		Name:      "Dagverdara Airport",
		City:      "Dagverdara",
		State:     "West",
		Country:   "IS",
		Elevation: 130,
		Latitude:  64.7450027466,
		Longitude: -23.7271995544,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIDV": {
		ICAO:      "BIDV",
		IATA:      "DJU",
		Name:      "Djupivogur Airport",
		City:      "Djupivogur",
		State:     "East",
		Country:   "IS",
		Elevation: 9,
		Latitude:  64.644203186,
		Longitude: -14.2827997208,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIEG": {
		ICAO:      "BIEG",
		IATA:      "EGS",
		Name:      "Egilsstadir Airport",
		City:      "Egilsstadir",
		State:     "East",
		Country:   "IS",
		Elevation: 76,
		Latitude:  65.2833023071,
		Longitude: -14.4013996124,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIEH": {
		ICAO:      "BIEH",
		Name:      "Einholtsmelar Airport",
		City:      "Einholtsmelar",
		State:     "South",
		Country:   "IS",
		Elevation: 361,
		Latitude:  64.25,
		Longitude: -20.2999992371,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIFF": {
		ICAO:      "BIFF",
		IATA:      "FAS",
		Name:      "Faskrudsfjordur Airport",
		City:      "Faskrudsfjordur",
		State:     "East",
		Country:   "IS",
		Elevation: 15,
		Latitude:  64.9317016602,
		Longitude: -14.0606002808,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIFL": {
		ICAO:      "BIFL",
		Name:      "Fludir Airport",
		City:      "Fludir",
		State:     "South",
		Country:   "IS",
		Elevation: 270,
		Latitude:  64.1427993774,
		Longitude: -20.3260993958,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIFM": {
		ICAO:      "BIFM",
		IATA:      "FAG",
		Name:      "Fagurholsmyri Airport",
		City:      "Fagurholsmyri",
		State:     "East",
		Country:   "IS",
		Elevation: 56,
		Latitude:  63.8746986389,
		Longitude: -16.6410999298,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIFZ": {
		ICAO:      "BIFZ",
		Name:      "Forsaeti Airport",
		City:      "Forsaeti",
		State:     "South",
		Country:   "IS",
		Elevation: 32,
		Latitude:  63.8499984741,
		Longitude: -20.716999054,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIGE": {
		ICAO:      "BIGE",
		Name:      "Geysir Airport",
		City:      "Geysir",
		State:     "South",
		Country:   "IS",
		Elevation: 350,
		Latitude:  64.310798645,
		Longitude: -20.2705993652,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIGF": {
		ICAO:      "BIGF",
		IATA:      "GUU",
		Name:      "Grundarfjordur Airport",
		City:      "Grundarfjordur",
		State:     "West",
		Country:   "IS",
		Elevation: 17,
		Latitude:  64.9914016724,
		Longitude: -23.2247009277,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIGH": {
		ICAO:      "BIGH",
		Name:      "Gunnarsholt Airport",
		City:      "Gunnarsholt",
		State:     "South",
		Country:   "IS",
		Elevation: 200,
		Latitude:  63.8532981873,
		Longitude: -20.2628002167,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIGJ": {
		ICAO:      "BIGJ",
		IATA:      "GJR",
		Name:      "Gjogur Airport",
		City:      "Gjogur",
		State:     "Westfjords",
		Country:   "IS",
		Elevation: 83,
		Latitude:  65.995300293,
		Longitude: -21.3269004822,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIGR": {
		ICAO:      "BIGR",
		IATA:      "GRY",
		Name:      "Grimsey Airport",
		City:      "Grimsey",
		State:     "Northeast",
		Country:   "IS",
		Elevation: 66,
		Latitude:  66.5458,
		Longitude: -18.0173,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIGS": {
		ICAO:      "BIGS",
		Name:      "Grimsstadir Airport",
		City:      "Grimsstadir",
		State:     "Northeast",
		Country:   "IS",
		Elevation: 1279,
		Latitude:  65.6324996948,
		Longitude: -16.1483001709,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIHE": {
		ICAO:      "BIHE",
		Name:      "Herdubreidarlindir Airport",
		City:      "Herdubreidarlindir",
		State:     "Northeast",
		Country:   "IS",
		Elevation: 1500,
		Latitude:  65.1885986328,
		Longitude: -16.1947002411,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIHI": {
		ICAO:      "BIHI",
		Name:      "Hveravellir Airport",
		City:      "Hveravellir",
		State:     "Northwest",
		Country:   "IS",
		Elevation: 2000,
		Latitude:  64.886100769,
		Longitude: -19.4925003052,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIHK": {
		ICAO:      "BIHK",
		IATA:      "HVK",
		Name:      "Holmavik Airport",
		City:      "Holmavik",
		State:     "Westfjords",
		Country:   "IS",
		Elevation: 90,
		Latitude:  65.7046966553,
		Longitude: -21.6963996887,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIHL": {
		ICAO:      "BIHL",
		Name:      "Hella Airport",
		City:      "Hella",
		State:     "South",
		Country:   "IS",
		Elevation: 66,
		Latitude:  63.8358001709,
		Longitude: -20.3775005341,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIHN": {
		ICAO:      "BIHN",
		IATA:      "HFN",
		Name:      "Hornafjordur Airport",
		City:      "Hornafjordur",
		State:     "East",
		Country:   "IS",
		Elevation: 24,
		Latitude:  64.2956008911,
		Longitude: -15.2271995544,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIHR": {
		ICAO:      "BIHR",
		Name:      "Hvolsvollur Airport",
		City:      "Hvolsvollur",
		State:     "South",
		Country:   "IS",
		Elevation: 109,
		Latitude:  63.7538986206,
		Longitude: -20.2441997528,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIHS": {
		ICAO:      "BIHS",
		Name:      "Hrafnseyri Airport",
		City:      "Hrafnseyri",
		State:     "Westfjords",
		Country:   "IS",
		Elevation: 30,
		Latitude:  65.75,
		Longitude: -23.466999054,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIHT": {
		ICAO:      "BIHT",
		IATA:      "FLI",
		Name:      "Holt Airport",
		City:      "Flateyri",
		State:     "Westfjords",
		Country:   "IS",
		Elevation: 10,
		Latitude:  66.0141804294,
		Longitude: -23.4416913986,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIHU": {
		ICAO:      "BIHU",
		IATA:      "HZK",
		Name:      "Husavik Airport",
		City:      "Husavik",
		State:     "Northeast",
		Country:   "IS",
		Elevation: 48,
		Latitude:  65.9523010254,
		Longitude: -17.4260005951,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIHV": {
		ICAO:      "BIHV",
		IATA:      "HVM",
		Name:      "Krokstadarmelar Airport",
		City:      "Hvammstangi",
		State:     "Northwest",
		Country:   "IS",
		Elevation: 164,
		Latitude:  65.2664031982,
		Longitude: -20.8469009399,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIHX": {
		ICAO:      "BIHX",
		Name:      "Hrauneyjarfoss Airport",
		City:      "Hrauneyjarfoss",
		State:     "South",
		Country:   "IS",
		Elevation: 1200,
		Latitude:  64.1999969482,
		Longitude: -19.2833003998,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIHY": {
		ICAO:      "BIHY",
		Name:      "Hrisey Airport",
		City:      "Hrisey",
		State:     "Northeast",
		Country:   "IS",
		Elevation: 15,
		Latitude:  66,
		Longitude: -18.3999996185,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIHZ": {
		ICAO:      "BIHZ",
		Name:      "Husafell Airport",
		City:      "Husafell",
		State:     "West",
		Country:   "IS",
		Elevation: 380,
		Latitude:  64.6996994019,
		Longitude: -20.883600235,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIID": {
		ICAO:      "BIID",
		IATA:      "HLO",
		Name:      "Ingjaldssandur Airport",
		City:      "Onundarfjordur",
		State:     "Westfjords",
		Country:   "IS",
		Elevation: 70,
		Latitude:  66.0500030518,
		Longitude: -23.696100235,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIIS": {
		ICAO:      "BIIS",
		IATA:      "IFJ",
		Name:      "Isafjordur Airport",
		City:      "Isafjordur",
		State:     "Westfjords",
		Country:   "IS",
		Elevation: 8,
		Latitude:  66.0580978394,
		Longitude: -23.1352996826,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIKA": {
		ICAO:      "BIKA",
		Name:      "Kaldarmelar Airport",
		City:      "Kaldarmelar",
		State:     "West",
		Country:   "IS",
		Elevation: 160,
		Latitude:  64.7789001465,
		Longitude: -22.2569007874,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIKE": {
		ICAO:      "BIKE",
		Name:      "Kerlingafjoll Airport",
		City:      "Kerlingafjoll",
		State:     "South",
		Country:   "IS",
		Elevation: 2100,
		Latitude:  64.7050018311,
		Longitude: -19.4106006622,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIKF": {
		ICAO:      "BIKF",
		IATA:      "KEF",
		Name:      "Keflavik International Airport",
		City:      "Reykjavik",
		State:     "Southern Peninsula",
		Country:   "IS",
		Elevation: 171,
		Latitude:  63.9850006104,
		Longitude: -22.6056003571,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIKJ": {
		ICAO:      "BIKJ",
		Name:      "Kroksfjard-Arnes Airport",
		City:      "Kroksfjard-arnes",
		State:     "Westfjords",
		Country:   "IS",
		Elevation: 49,
		Latitude:  65.4670028687,
		Longitude: -21.9500007629,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIKL": {
		ICAO:      "BIKL",
		Name:      "Kirkjubaejarklaustur Airport",
		City:      "Kirkjubaejarklaustur",
		State:     "South",
		Country:   "IS",
		Elevation: 66,
		Latitude:  63.7907981873,
		Longitude: -18.0013999939,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIKP": {
		ICAO:      "BIKP",
		IATA:      "OPA",
		Name:      "Kopasker Airport",
		City:      "Kopasker",
		State:     "Northeast",
		Country:   "IS",
		Elevation: 20,
		Latitude:  66.310798645,
		Longitude: -16.4666996002,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIKR": {
		ICAO:      "BIKR",
		IATA:      "SAK",
		Name:      "Saudarkrokur Airport",
		City:      "Saudarkrokur",
		State:     "Northwest",
		Country:   "IS",
		Elevation: 8,
		Latitude:  65.7316970825,
		Longitude: -19.5727996826,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIMK": {
		ICAO:      "BIMK",
		Name:      "Mulakot Airport",
		City:      "Mulakot",
		State:     "South",
		Country:   "IS",
		Elevation: 272,
		Latitude:  63.7141990662,
		Longitude: -19.8791999817,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIMM": {
		ICAO:      "BIMM",
		Name:      "Melgerdismelar Airport",
		City:      "Melgerdismelar",
		State:     "Northeast",
		Country:   "IS",
		Elevation: 35,
		Latitude:  65.4832992554,
		Longitude: -18.1667003632,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIMN": {
		ICAO:      "BIMN",
		Name:      "Melanes Airport",
		City:      "Melanes",
		State:     "Westfjords",
		Country:   "IS",
		Elevation: 148,
		Latitude:  65.516998291,
		Longitude: -22.3999996185,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIMS": {
		ICAO:      "BIMS",
		Name:      "Tungubakkar Airport",
		City:      "Tungubakkar",
		State:     "Capital Region",
		Country:   "IS",
		Elevation: 5,
		Latitude:  64.181098938,
		Longitude: -21.7077999115,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIND": {
		ICAO:      "BIND",
		Name:      "Nyjidalur Airport",
		City:      "Nyjidalur",
		State:     "South",
		Country:   "IS",
		Elevation: 2625,
		Latitude:  64.7205963135,
		Longitude: -18.0666999817,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BINF": {
		ICAO:      "BINF",
		IATA:      "NOR",
		Name:      "Nordfjordur Airport",
		City:      "Nordfjordur",
		State:     "East",
		Country:   "IS",
		Elevation: 13,
		Latitude:  65.1318969727,
		Longitude: -13.7463998795,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIOF": {
		ICAO:      "BIOF",
		IATA:      "OFJ",
		Name:      "Olafsfjordur Airport",
		City:      "Olafsfjordur",
		State:     "Northeast",
		Country:   "IS",
		Elevation: 32,
		Latitude:  66.0832977295,
		Longitude: -18.6667003632,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIPA": {
		ICAO:      "BIPA",
		IATA:      "PFJ",
		Name:      "Patreksfjordur Airport",
		City:      "Patreksfjordur",
		State:     "Westfjords",
		Country:   "IS",
		Elevation: 11,
		Latitude:  65.5558013916,
		Longitude: -23.9650001526,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIRE": {
		ICAO:      "BIRE",
		IATA:      "RHA",
		Name:      "Reykholar Airport",
		City:      "Reykholar",
		State:     "Westfjords",
		Country:   "IS",
		Elevation: 60,
		Latitude:  65.4526290894,
		Longitude: -22.2061157227,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIRF": {
		ICAO:      "BIRF",
		IATA:      "OLI",
		Name:      "Rif Airport",
		City:      "Rif",
		State:     "West",
		Country:   "IS",
		Elevation: 18,
		Latitude:  64.9113998413,
		Longitude: -23.8230991364,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIRG": {
		ICAO:      "BIRG",
		IATA:      "RFN",
		Name:      "Raufarhofn Airport",
		City:      "Raufarhofn",
		State:     "Northeast",
		Country:   "IS",
		Elevation: 39,
		Latitude:  66.4064025879,
		Longitude: -15.918299675,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIRK": {
		ICAO:      "BIRK",
		IATA:      "RKV",
		Name:      "Reykjavik Airport",
		City:      "Reykjavik",
		State:     "Capital Region",
		Country:   "IS",
		Elevation: 48,
		Latitude:  64.1299972534,
		Longitude: -21.9405994415,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIRL": {
		ICAO:      "BIRL",
		IATA:      "MVA",
		Name:      "Reykjahlid Airport",
		City:      "Myvatn",
		State:     "Northeast",
		Country:   "IS",
		Elevation: 1030,
		Latitude:  65.6557998657,
		Longitude: -16.9181003571,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIRS": {
		ICAO:      "BIRS",
		Name:      "Reykjanes Airport",
		City:      "Reykjanes",
		State:     "Westfjords",
		Country:   "IS",
		Elevation: 10,
		Latitude:  65.9141998291,
		Longitude: -22.4214000702,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BISA": {
		ICAO:      "BISA",
		Name:      "Sanda Airport",
		City:      "Sanda",
		State:     "Northwest",
		Country:   "IS",
		Elevation: 1580,
		Latitude:  65.1343994141,
		Longitude: -19.6630992889,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BISF": {
		ICAO:      "BISF",
		Name:      "Selfoss Airport",
		City:      "Selfoss",
		State:     "South",
		Country:   "IS",
		Elevation: 45,
		Latitude:  63.9291992188,
		Longitude: -21.0377998352,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BISG": {
		ICAO:      "BISG",
		Name:      "Steinasandur Airport",
		City:      "Steinasandur",
		State:     "East",
		Country:   "IS",
		Elevation: 30,
		Latitude:  64.1669998169,
		Longitude: -15.8999996185,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BISH": {
		ICAO:      "BISH",
		Name:      "Stora-Holt Airport",
		City:      "Stora-Holt",
		State:     "West",
		Country:   "IS",
		Elevation: 100,
		Latitude:  65.4000015259,
		Longitude: -21.9330005646,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BISI": {
		ICAO:      "BISI",
		IATA:      "SIJ",
		Name:      "Siglufjordur Airport",
		City:      "Siglufjordur",
		State:     "Northeast",
		Country:   "IS",
		Elevation: 10,
		Latitude:  66.1333007813,
		Longitude: -18.9167003632,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BISK": {
		ICAO:      "BISK",
		Name:      "Skogasandur Airport",
		City:      "Skogasandur",
		State:     "South",
		Country:   "IS",
		Elevation: 130,
		Latitude:  63.51720047,
		Longitude: -19.489200592,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BISN": {
		ICAO:      "BISN",
		Name:      "Svinafell Airport",
		City:      "Svinafell",
		State:     "East",
		Country:   "IS",
		Elevation: 60,
		Latitude:  64.3833007813,
		Longitude: -15.3832998276,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BISP": {
		ICAO:      "BISP",
		Name:      "Sprengisandur Airport",
		City:      "Sprengisandur",
		State:     "South",
		Country:   "IS",
		Elevation: 2050,
		Latitude:  64.6541976929,
		Longitude: -18.4969005585,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BISR": {
		ICAO:      "BISR",
		Name:      "Storikroppur Airport",
		City:      "Storikroppur",
		State:     "West",
		Country:   "IS",
		Elevation: 165,
		Latitude:  64.6343994141,
		Longitude: -21.4874992371,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BISS": {
		ICAO:      "BISS",
		Name:      "Sandskeid Airport",
		City:      "Sandskeid",
		State:     "Capital Region",
		Country:   "IS",
		Elevation: 600,
		Latitude:  64.060798645,
		Longitude: -21.5746994019,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIST": {
		ICAO:      "BIST",
		IATA:      "SYK",
		Name:      "Stykkisholmur Airport",
		City:      "Stykkisholmur",
		State:     "West",
		Country:   "IS",
		Elevation: 42,
		Latitude:  65.0580978394,
		Longitude: -22.7942008972,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BISV": {
		ICAO:      "BISV",
		Name:      "Skalavatn Airport",
		City:      "Skalavatn",
		State:     "South",
		Country:   "IS",
		Elevation: 2000,
		Latitude:  64.1157989502,
		Longitude: -18.7833003998,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BITE": {
		ICAO:      "BITE",
		IATA:      "TEY",
		Name:      "Tingeyri Airport",
		City:      "Tingeyri",
		State:     "Westfjords",
		Country:   "IS",
		Elevation: 65,
		Latitude:  65.870300293,
		Longitude: -23.5599994659,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BITM": {
		ICAO:      "BITM",
		Name:      "Torsmork Airport",
		City:      "Torsmork",
		State:     "South",
		Country:   "IS",
		Elevation: 600,
		Latitude:  63.6899986267,
		Longitude: -19.5631008148,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BITN": {
		ICAO:      "BITN",
		IATA:      "THO",
		Name:      "Thorshofn Airport",
		City:      "Thorshofn",
		State:     "Northeast",
		Country:   "IS",
		Elevation: 65,
		Latitude:  66.21849823,
		Longitude: -15.3355998993,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BITO": {
		ICAO:      "BITO",
		Name:      "Thorisos Airport",
		City:      "Thorisos",
		State:     "South",
		Country:   "IS",
		Elevation: 1840,
		Latitude:  64.3499984741,
		Longitude: -18.8500003815,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIVA": {
		ICAO:      "BIVA",
		Name:      "Vatnsnes Airport",
		City:      "Vatnsnes",
		State:     "South",
		Country:   "IS",
		Latitude:  64.0330963135,
		Longitude: -20.6511001587,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIVH": {
		ICAO:      "BIVH",
		Name:      "Varmahlid Airport",
		City:      "Varmahlid",
		State:     "Northwest",
		Country:   "IS",
		Elevation: 27,
		Latitude:  65.5569458008,
		Longitude: -19.4280567169,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIVI": {
		ICAO:      "BIVI",
		Name:      "Vik Airport",
		City:      "Vik",
		State:     "South",
		Country:   "IS",
		Elevation: 100,
		Latitude:  63.4216995239,
		Longitude: -18.888299942,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIVM": {
		ICAO:      "BIVM",
		IATA:      "VEY",
		Name:      "Vestmannaeyjar Airport",
		City:      "Vestmannaeyjar",
		State:     "South",
		Country:   "IS",
		Elevation: 326,
		Latitude:  63.4243011475,
		Longitude: -20.2789001465,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BIVO": {
		ICAO:      "BIVO",
		IATA:      "VPN",
		Name:      "Vopnafjordur Airport",
		City:      "Vopnafjordur",
		State:     "East",
		Country:   "IS",
		Elevation: 16,
		Latitude:  65.7205963135,
		Longitude: -14.8506002426,
		Timezone:  "Atlantic/Reykjavik",
	},
	"BKPR": {
		ICAO:      "BKPR",
		IATA:      "PRN",
		Name:      "Pristina International Airport",
		City:      "Prishtina",
		State:     "Pristina",
		Country:   "KS",
		Elevation: 1789,
		Latitude:  42.5727996826,
		Longitude: 21.0358009338,
		Timezone:  "Europe/Belgrade",
	},
	"CA02": {
		ICAO:      "CA02",
		Name:      "Patterson Airport",
		City:      "Patterson",
		State:     "California",
		Country:   "US",
		Elevation: 168,
		Latitude:  37.46849823,
		Longitude: -121.1689987183,
		Timezone:  "America/Los_Angeles",
	},
	"CA03": {
		ICAO:      "CA03",
		Name:      "John Myers Airport",
		City:      "Planada",
		State:     "California",
		Country:   "US",
		Elevation: 280,
		Latitude:  37.349899292,
		Longitude: -120.2939987183,
		Timezone:  "America/Los_Angeles",
	},
	"CA04": {
		ICAO:      "CA04",
		Name:      "Flying N Ranch Airport",
		City:      "Cottonwood",
		State:     "California",
		Country:   "US",
		Elevation: 530,
		Latitude:  40.3462982178,
		Longitude: -122.3389968872,
		Timezone:  "America/Los_Angeles",
	},
	"CA05": {
		ICAO:      "CA05",
		Name:      "Machado Dusters Airport",
		City:      "Stratford",
		State:     "California",
		Country:   "US",
		Elevation: 192,
		Latitude:  36.1601982117,
		Longitude: -119.81300354,
		Timezone:  "America/Los_Angeles",
	},
	"CA06": {
		ICAO:      "CA06",
		Name:      "Bates Field",
		City:      "Alturas",
		State:     "California",
		Country:   "US",
		Elevation: 4400,
		Latitude:  41.4178009033,
		Longitude: -120.5690002441,
		Timezone:  "America/Los_Angeles",
	},
	"CA07": {
		ICAO:      "CA07",
		Name:      "Scheidel Ranch Airport",
		City:      "Pleasant Grove",
		State:     "California",
		Country:   "US",
		Elevation: 28,
		Latitude:  38.8401985168,
		Longitude: -121.56199646,
		Timezone:  "America/Los_Angeles",
	},
	"CA09": {
		ICAO:      "CA09",
		Name:      "Round Mountain Airport",
		City:      "Klamath River",
		State:     "California",
		Country:   "US",
		Elevation: 3500,
		Latitude:  41.8861999512,
		Longitude: -122.84400177,
		Timezone:  "America/Los_Angeles",
	},
	"CA10": {
		ICAO:      "CA10",
		Name:      "Coonrod Ranch Airport",
		City:      "Little Shasta",
		State:     "California",
		Country:   "US",
		Elevation: 3010,
		Latitude:  41.617099762,
		Longitude: -122.3190002441,
		Timezone:  "America/Los_Angeles",
	},
	"CA11": {
		ICAO:      "CA11",
		Name:      "Mc Cloud Airstrip",
		City:      "Mc Cloud",
		State:     "California",
		Country:   "US",
		Elevation: 3540,
		Latitude:  41.2732009888,
		Longitude: -122.122001648,
		Timezone:  "America/Los_Angeles",
	},
	"CA13": {
		ICAO:      "CA13",
		Name:      "Kings River Community College Airport",
		City:      "Reedley",
		State:     "California",
		Country:   "US",
		Elevation: 350,
		Latitude:  36.6068992615,
		Longitude: -119.4629974365,
		Timezone:  "America/Los_Angeles",
	},
	"CA16": {
		ICAO:      "CA16",
		Name:      "Holtsmans Airport",
		City:      "Rio Linda",
		State:     "California",
		Country:   "US",
		Elevation: 120,
		Latitude:  38.7290992737,
		Longitude: -121.388999939,
		Timezone:  "America/Los_Angeles",
	},
	"CA17": {
		ICAO:      "CA17",
		Name:      "Peterson Airport",
		City:      "Riverbank",
		State:     "California",
		Country:   "US",
		Elevation: 158,
		Latitude:  37.7234992981,
		Longitude: -120.8899993896,
		Timezone:  "America/Los_Angeles",
	},
	"CA18": {
		ICAO:      "CA18",
		Name:      "Wagner Aviation Airport",
		City:      "Robbins",
		State:     "California",
		Country:   "US",
		Elevation: 24,
		Latitude:  38.8721008301,
		Longitude: -121.7099990845,
		Timezone:  "America/Los_Angeles",
	},
	"CA19": {
		ICAO:      "CA19",
		Name:      "Camanche Skypark Airport",
		City:      "Ione",
		State:     "California",
		Country:   "US",
		Elevation: 520,
		Latitude:  38.2638015747,
		Longitude: -120.922996521,
		Timezone:  "America/Los_Angeles",
	},
	"CA20": {
		ICAO:      "CA20",
		Name:      "Eagle's Nest Airport",
		City:      "Ione",
		State:     "California",
		Country:   "US",
		Elevation: 220,
		Latitude:  38.4305000305,
		Longitude: -121.01499939,
		Timezone:  "America/Los_Angeles",
	},
	"CA21": {
		ICAO:      "CA21",
		Name:      "Limberlost Ranch Airport",
		City:      "Rough and Ready",
		State:     "California",
		Country:   "US",
		Elevation: 1650,
		Latitude:  39.2206993103,
		Longitude: -121.2149963379,
		Timezone:  "America/Los_Angeles",
	},
	"CA22": {
		ICAO:      "CA22",
		Name:      "Lauppes Strip",
		City:      "Sacramento",
		State:     "California",
		Country:   "US",
		Elevation: 23,
		Latitude:  38.7593002319,
		Longitude: -121.5849990845,
		Timezone:  "America/Los_Angeles",
	},
	"CA32": {
		ICAO:      "CA32",
		Name:      "San Joaquin Airport",
		City:      "San Joaquin",
		State:     "California",
		Country:   "US",
		Elevation: 165,
		Latitude:  36.5833015442,
		Longitude: -120.2509994507,
		Timezone:  "America/Los_Angeles",
	},
	"CA35": {
		ICAO:      "CA35",
		Name:      "San Rafael Airport",
		City:      "San Rafael",
		State:     "California",
		Country:   "US",
		Elevation: 5,
		Latitude:  38.0168991089,
		Longitude: -122.5210037231,
		Timezone:  "America/Los_Angeles",
	},
	"CA38": {
		ICAO:      "CA38",
		Name:      "Totem Pole Ranch Airport",
		City:      "Calpine",
		State:     "California",
		Country:   "US",
		Elevation: 4986,
		Latitude:  39.6512985229,
		Longitude: -120.4390029907,
		Timezone:  "America/Los_Angeles",
	},
	"CA39": {
		ICAO:      "CA39",
		Name:      "Graywood Ranch Airport",
		City:      "Santa Rosa",
		State:     "California",
		Country:   "US",
		Elevation: 450,
		Latitude:  38.4346008301,
		Longitude: -122.5640029907,
		Timezone:  "America/Los_Angeles",
	},
	"CA40": {
		ICAO:      "CA40",
		Name:      "Central Valley Aviation Inc Airport",
		City:      "Selma",
		State:     "California",
		Country:   "US",
		Elevation: 276,
		Latitude:  36.5213012695,
		Longitude: -119.6699981689,
		Timezone:  "America/Los_Angeles",
	},
	"CA41": {
		ICAO:      "CA41",
		Name:      "Quinn Airport",
		City:      "Selma",
		State:     "California",
		Country:   "US",
		Elevation: 310,
		Latitude:  36.5833015442,
		Longitude: -119.6399993896,
		Timezone:  "America/Los_Angeles",
	},
	"CA42": {
		ICAO:      "CA42",
		Name:      "Mc Conville Airstrip",
		City:      "Lake Elsinore",
		State:     "California",
		Country:   "US",
		Elevation: 2600,
		Latitude:  33.6436004639,
		Longitude: -117.429000854,
		Timezone:  "America/Los_Angeles",
	},
	"CA44": {
		ICAO:      "CA44",
		Name:      "Sequoia Ranch Airport",
		City:      "Springville",
		State:     "California",
		Country:   "US",
		Elevation: 1153,
		Latitude:  36.1498985291,
		Longitude: -118.8020019531,
		Timezone:  "America/Los_Angeles",
	},
	"CA45": {
		ICAO:      "CA45",
		Name:      "Stevinson Strip",
		City:      "Stevinson",
		State:     "California",
		Country:   "US",
		Elevation: 79,
		Latitude:  37.3249015808,
		Longitude: -120.8679962158,
		Timezone:  "America/Los_Angeles",
	},
	"CA49": {
		ICAO:      "CA49",
		Name:      "Jones Farms Airport",
		City:      "Stratford",
		State:     "California",
		Country:   "US",
		Elevation: 199,
		Latitude:  36.2041015625,
		Longitude: -119.841003418,
		Timezone:  "America/Los_Angeles",
	},
	"CA51": {
		ICAO:      "CA51",
		Name:      "The Sea Ranch Airport",
		City:      "The Sea Ranch",
		State:     "California",
		Country:   "US",
		Elevation: 360,
		Latitude:  38.7046012878,
		Longitude: -123.4329986572,
		Timezone:  "America/Los_Angeles",
	},
	"CA53": {
		ICAO:      "CA53",
		Name:      "Tews Field",
		City:      "Redding",
		State:     "California",
		Country:   "US",
		Elevation: 740,
		Latitude:  40.6725006104,
		Longitude: -122.3420028687,
		Timezone:  "America/Los_Angeles",
	},
	"CA54": {
		ICAO:      "CA54",
		Name:      "33 Strip",
		City:      "Tracy",
		State:     "California",
		Country:   "US",
		Elevation: 81,
		Latitude:  37.670501709,
		Longitude: -121.3280029297,
		Timezone:  "America/Los_Angeles",
	},
	"CA57": {
		ICAO:      "CA57",
		Name:      "Blake Sky Park Airport",
		City:      "Vacaville",
		State:     "California",
		Country:   "US",
		Elevation: 310,
		Latitude:  38.4581985474,
		Longitude: -122.0459976196,
		Timezone:  "America/Los_Angeles",
	},
	"CA59": {
		ICAO:      "CA59",
		Name:      "Vestal Strip",
		City:      "Verona",
		State:     "California",
		Country:   "US",
		Elevation: 23,
		Latitude:  38.7915992737,
		Longitude: -121.5839996338,
		Timezone:  "America/Los_Angeles",
	},
	"CA60": {
		ICAO:      "CA60",
		Name:      "Deer Creek Ranch Airport",
		City:      "Vina",
		State:     "California",
		Country:   "US",
		Elevation: 280,
		Latitude:  39.9499015808,
		Longitude: -121.9970016479,
		Timezone:  "America/Los_Angeles",
	},
	"CA62": {
		ICAO:      "CA62",
		Name:      "Mc Millan Assault Strip",
		City:      "Camp Roberts",
		State:     "California",
		Country:   "US",
		Elevation: 920,
		Latitude:  35.71910095,
		Longitude: -120.7679977,
		Timezone:  "America/Los_Angeles",
	},
	"CA64": {
		ICAO:      "CA64",
		Name:      "Gilbreath Bros Duck Club Airport",
		City:      "Wasco",
		State:     "California",
		Country:   "US",
		Elevation: 220,
		Latitude:  35.6869010925,
		Longitude: -119.5999984741,
		Timezone:  "America/Los_Angeles",
	},
	"CA66": {
		ICAO:      "CA66",
		Name:      "Monterey Bay Academy Airport",
		City:      "Watsonville",
		State:     "California",
		Country:   "US",
		Elevation: 70,
		Latitude:  36.9061012268,
		Longitude: -121.8430023193,
		Timezone:  "America/Los_Angeles",
	},
	"CA67": {
		ICAO:      "CA67",
		Name:      "Westley Airport",
		City:      "Westley",
		State:     "California",
		Country:   "US",
		Elevation: 96,
		Latitude:  37.5480003357,
		Longitude: -121.2050018311,
		Timezone:  "America/Los_Angeles",
	},
	"CA69": {
		ICAO:      "CA69",
		Name:      "Avenal Airport",
		City:      "Avenal",
		State:     "California",
		Country:   "US",
		Elevation: 790,
		Latitude:  36.0041007996,
		Longitude: -120.141998291,
		Timezone:  "America/Los_Angeles",
	},
	"CA70": {
		ICAO:      "CA70",
		Name:      "Sun Hill Ranch Airport",
		City:      "Helendale",
		State:     "California",
		Country:   "US",
		Elevation: 2984,
		Latitude:  34.7579994202,
		Longitude: -117.4970016479,
		Timezone:  "America/Los_Angeles",
	},
	"CA71": {
		ICAO:      "CA71",
		Name:      "Horse Shoe A Ranch Airport",
		City:      "Drytown",
		State:     "California",
		Country:   "US",
		Elevation: 750,
		Latitude:  38.4399986267,
		Longitude: -120.8870010376,
		Timezone:  "America/Los_Angeles",
	},
	"CA73": {
		ICAO:      "CA73",
		Name:      "Vanderford Ranch Company Airport",
		City:      "Yuba City",
		State:     "California",
		Country:   "US",
		Elevation: 42,
		Latitude:  39.0956993103,
		Longitude: -121.716003418,
		Timezone:  "America/Los_Angeles",
	},
	"CA75": {
		ICAO:      "CA75",
		Name:      "Reider Ranch Airport",
		City:      "Potrero",
		State:     "California",
		Country:   "US",
		Elevation: 2655,
		Latitude:  32.6380996704,
		Longitude: -116.638999939,
		Timezone:  "America/Los_Angeles",
	},
	"CA76": {
		ICAO:      "CA76",
		Name:      "Flying T Ranch Airport",
		City:      "Ramona",
		State:     "California",
		Country:   "US",
		Elevation: 2500,
		Latitude:  33.0703010559,
		Longitude: -116.7519989014,
		Timezone:  "America/Los_Angeles",
	},
	"CA84": {
		ICAO:      "CA84",
		Name:      "Torrey Pines Gliderport",
		City:      "San Diego",
		State:     "California",
		Country:   "US",
		Elevation: 372,
		Latitude:  32.8897018433,
		Longitude: -117.2470016479,
		Timezone:  "America/Los_Angeles",
	},
	"CA88": {
		ICAO:      "CA88",
		Name:      "San Ardo Field",
		City:      "San Ardo",
		State:     "California",
		Country:   "US",
		Elevation: 441,
		Latitude:  36.0261001587,
		Longitude: -120.9079971313,
		Timezone:  "America/Los_Angeles",
	},
	"CA89": {
		ICAO:      "CA89",
		Name:      "Skylark Field",
		City:      "Lake Elsinore",
		State:     "California",
		Country:   "US",
		Elevation: 1253,
		Latitude:  33.6300010681,
		Longitude: -117.3020019531,
		Timezone:  "America/Los_Angeles",
	},
	"CA90": {
		ICAO:      "CA90",
		Name:      "Cadiz Airstrip",
		City:      "Cadiz",
		State:     "California",
		Country:   "US",
		Elevation: 780,
		Latitude:  34.5139007568,
		Longitude: -115.5189971924,
		Timezone:  "America/Los_Angeles",
	},
	"CA92": {
		ICAO:      "CA92",
		Name:      "Paradise Skypark Airport",
		City:      "Paradise",
		State:     "California",
		Country:   "US",
		Elevation: 1300,
		Latitude:  39.7106018066,
		Longitude: -121.616996765,
		Timezone:  "America/Los_Angeles",
	},
	"CA97": {
		ICAO:      "CA97",
		Name:      "Christy Airstrip",
		City:      "Santa Barbara",
		State:     "California",
		Country:   "US",
		Elevation: 250,
		Latitude:  34.0196990967,
		Longitude: -119.8529968262,
		Timezone:  "America/Los_Angeles",
	},
	"CAA2": {
		ICAO:      "CAA2",
		Name:      "St. Andre-Avellin",
		City:      "St. Andre Avellin",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 550,
		Latitude:  45.7463888889,
		Longitude: -75.0758333333,
		Timezone:  "America/Toronto",
	},
	"CAA8": {
		ICAO:      "CAA8",
		Name:      "Invermere Airport",
		City:      "Invermere",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2820,
		Latitude:  50.5167007446,
		Longitude: -116,
		Timezone:  "America/Edmonton",
	},
	"CAD4": {
		ICAO:      "CAD4",
		IATA:      "YZZ",
		Name:      "Trail Airport",
		City:      "Trail",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 1427,
		Latitude:  49.0555992126,
		Longitude: -117.60900116,
		Timezone:  "America/Vancouver",
	},
	"CAD5": {
		ICAO:      "CAD5",
		Name:      "Merritt Airport",
		City:      "Merritt",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2080,
		Latitude:  50.1227989197,
		Longitude: -120.7470016479,
		Timezone:  "America/Vancouver",
	},
	"CAE4": {
		ICAO:      "CAE4",
		Name:      "Tsacha Lake Airport",
		City:      "Tsacha Lake",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 3450,
		Latitude:  53.0167007446,
		Longitude: -124.8330001831,
		Timezone:  "America/Vancouver",
	},
	"CAF4": {
		ICAO:      "CAF4",
		Name:      "Tsuniah Lake Lodge Airport",
		City:      "Tsuniah Lake Lodge",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 4000,
		Latitude:  51.5332984924,
		Longitude: -124.1669998169,
		Timezone:  "America/Vancouver",
	},
	"CAG2": {
		ICAO:      "CAG2",
		Name:      "Regina / Aerogate",
		City:      "Brora",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1980,
		Latitude:  50.5977310536,
		Longitude: -104.601312876,
		Timezone:  "America/Regina",
	},
	"CAG3": {
		ICAO:      "CAG3",
		Name:      "Chilko Lake (Tsylos Park Lodge) Airport",
		City:      "Chilko Lake",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 3850,
		Latitude:  51.6260986328,
		Longitude: -124.141998291,
		Timezone:  "America/Vancouver",
	},
	"CAH3": {
		ICAO:      "CAH3",
		Name:      "Courtenay Airpark",
		City:      "Courtenay",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 9,
		Latitude:  49.6794013977,
		Longitude: -124.9820022583,
		Timezone:  "America/Vancouver",
	},
	"CAH4": {
		ICAO:      "CAH4",
		Name:      "Valemount Airport",
		City:      "Valemount",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2615,
		Latitude:  52.8532981873,
		Longitude: -119.3339996338,
		Timezone:  "America/Vancouver",
	},
	"CAJ2": {
		ICAO:      "CAJ2",
		Name:      "Wiley Airport",
		City:      "Eagle Plains",
		State:     "Yukon",
		Country:   "CA",
		Elevation: 2365,
		Latitude:  66.4910964966,
		Longitude: -136.5729980469,
		Timezone:  "America/Whitehorse",
	},
	"CAJ3": {
		ICAO:      "CAJ3",
		Name:      "Art Sutcliffe Field",
		City:      "Creston",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2070,
		Latitude:  49.0368995667,
		Longitude: -116.4980010986,
		Timezone:  "America/Creston",
	},
	"CAJ4": {
		ICAO:      "CAJ4",
		IATA:      "YAA",
		Name:      "Anahim Lake Airport",
		City:      "Anahim Lake",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 3635,
		Latitude:  52.4524993896,
		Longitude: -125.3030014038,
		Timezone:  "America/Vancouver",
	},
	"CAJ7": {
		ICAO:      "CAJ7",
		Name:      "Cayley A J Flying Ranch Airport",
		City:      "Cayley",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3450,
		Latitude:  50.4500007629,
		Longitude: -113.75,
		Timezone:  "America/Edmonton",
	},
	"CAJ9": {
		ICAO:      "CAJ9",
		Name:      "Fort Ware Airport",
		City:      "Fort Ware",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2450,
		Latitude:  57.4272003174,
		Longitude: -125.6500015259,
		Timezone:  "America/Vancouver",
	},
	"CAK3": {
		ICAO:      "CAK3",
		Name:      "Delta Heritage Air Park",
		City:      "Delta",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 10,
		Latitude:  49.0774002075,
		Longitude: -122.9410018921,
		Timezone:  "America/Vancouver",
	},
	"CAL3": {
		ICAO:      "CAL3",
		IATA:      "DGF",
		Name:      "Douglas Lake Airport",
		City:      "Douglas Lake",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2770,
		Latitude:  50.1654600485,
		Longitude: -120.171273351,
		Timezone:  "America/Vancouver",
	},
	"CAL4": {
		ICAO:      "CAL4",
		IATA:      "JHL",
		Name:      "Fort MacKay/Albian Aerodrome",
		City:      "Albian Village",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 1048,
		Latitude:  57.2238998413,
		Longitude: -111.418998718,
		Timezone:  "America/Edmonton",
	},
	"CAM3": {
		ICAO:      "CAM3",
		IATA:      "DUQ",
		Name:      "Duncan Airport",
		City:      "Duncan",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 300,
		Latitude:  48.7545336204,
		Longitude: -123.709702492,
		Timezone:  "America/Vancouver",
	},
	"CAM4": {
		ICAO:      "CAM4",
		Name:      "Alhambra / Ahlstrom",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3212,
		Latitude:  52.3461940388,
		Longitude: -114.667525291,
		Timezone:  "America/Edmonton",
	},
	"CAM5": {
		ICAO:      "CAM5",
		Name:      "Houston Airport",
		City:      "Houston",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2150,
		Latitude:  54.4392009821,
		Longitude: -126.778879166,
		Timezone:  "America/Vancouver",
	},
	"CAP3": {
		ICAO:      "CAP3",
		Name:      "Sechelt-Gibsons Airport",
		City:      "Sechelt-Gibsons",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 250,
		Latitude:  49.4606018066,
		Longitude: -123.71900177,
		Timezone:  "America/Vancouver",
	},
	"CAP6": {
		ICAO:      "CAP6",
		Name:      "Ingenika Airport",
		City:      "Ingenika",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2230,
		Latitude:  56.790599823,
		Longitude: -124.8970031738,
		Timezone:  "America/Vancouver",
	},
	"CAQ4": {
		ICAO:      "CAQ4",
		Name:      "Springhouse Airpark",
		City:      "Springhouse",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 3250,
		Latitude:  51.9556007385,
		Longitude: -122.138999939,
		Timezone:  "America/Vancouver",
	},
	"CAQ5": {
		ICAO:      "CAQ5",
		Name:      "Nakusp Airport",
		City:      "Nakusp",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 1689,
		Latitude:  50.2663993835,
		Longitude: -117.81300354,
		Timezone:  "America/Vancouver",
	},
	"CAR2": {
		ICAO:      "CAR2",
		Name:      "Crawford Bay Airport",
		City:      "Crawford Bay",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 1760,
		Latitude:  49.6666984558,
		Longitude: -116.8170013428,
		Timezone:  "America/Creston",
	},
	"CAR5": {
		ICAO:      "CAR5",
		Name:      "Arthur (Arthur South)",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1530,
		Latitude:  43.7827292694,
		Longitude: -80.4329252243,
		Timezone:  "America/Toronto",
	},
	"CAS2": {
		ICAO:      "CAS2",
		Name:      "Moose Lake (Lodge) Airport",
		City:      "Moose Lake",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 3500,
		Latitude:  53.073299408,
		Longitude: -125.408996582,
		Timezone:  "America/Vancouver",
	},
	"CAS3": {
		ICAO:      "CAS3",
		Name:      "Barkerville Airport",
		City:      "Barkerville",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 4060,
		Latitude:  53.0881004333,
		Longitude: -121.5149993896,
		Timezone:  "America/Vancouver",
	},
	"CAT1": {
		ICAO:      "CAT1",
		Name:      "Atwood / Coghlin Airport",
		City:      "Atwood",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1215,
		Latitude:  43.6833000183,
		Longitude: -81.0044021606,
		Timezone:  "America/Toronto",
	},
	"CAT4": {
		ICAO:      "CAT4",
		Name:      "Qualicum Beach Airport",
		City:      "Qualicum Beach",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 191,
		Latitude:  49.3372001648,
		Longitude: -124.3939971924,
		Timezone:  "America/Vancouver",
	},
	"CAT5": {
		ICAO:      "CAT5",
		Name:      "Port Mcneill Airport",
		City:      "Port Mcneill",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 225,
		Latitude:  50.5755996704,
		Longitude: -127.0289993286,
		Timezone:  "America/Vancouver",
	},
	"CAU3": {
		ICAO:      "CAU3",
		Name:      "Oliver Airport",
		City:      "Oliver",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 1015,
		Latitude:  49.1733016968,
		Longitude: -119.5510025024,
		Timezone:  "America/Vancouver",
	},
	"CAU4": {
		ICAO:      "CAU4",
		Name:      "Vanderhoof Airport",
		City:      "Vanderhoof",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2225,
		Latitude:  54.0461006165,
		Longitude: -124.0120010376,
		Timezone:  "America/Vancouver",
	},
	"CAV3": {
		ICAO:      "CAV3",
		Name:      "One Hundred Mile House Airport",
		City:      "One Hundred Mile House",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 3055,
		Latitude:  51.6425018311,
		Longitude: -121.3069992065,
		Timezone:  "America/Vancouver",
	},
	"CAV4": {
		ICAO:      "CAV4",
		Name:      "Mcbride Airport",
		City:      "Mcbride",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2350,
		Latitude:  53.3149986267,
		Longitude: -120.1709976196,
		Timezone:  "America/Vancouver",
	},
	"CAV6": {
		ICAO:      "CAV6",
		Name:      "Beausejour/Av-Ranch Airpark",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 800,
		Latitude:  50.041111,
		Longitude: -96.585833,
		Timezone:  "America/Winnipeg",
	},
	"CAV9": {
		ICAO:      "CAV9",
		Name:      "Oak Hammock Air Park",
		City:      "Oak Hammock",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 755,
		Latitude:  50.1377983093,
		Longitude: -97.0614013672,
		Timezone:  "America/Winnipeg",
	},
	"CAW3": {
		ICAO:      "CAW3",
		Name:      "Scum Lake Airport",
		City:      "Scum Lake",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 3950,
		Latitude:  51.7999992371,
		Longitude: -123.5830001831,
		Timezone:  "America/Vancouver",
	},
	"CAX2": {
		ICAO:      "CAX2",
		Name:      "Axe Lake",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1758,
		Latitude:  57.2691650391,
		Longitude: -109.8475036621,
		Timezone:  "America/Regina",
	},
	"CAX5": {
		ICAO:      "CAX5",
		Name:      "Likely Airport",
		City:      "Likely",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 3225,
		Latitude:  52.6166992188,
		Longitude: -121.5,
		Timezone:  "America/Vancouver",
	},
	"CAY2": {
		ICAO:      "CAY2",
		Name:      "Gang Ranch Airport",
		City:      "Gang Ranch",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2150,
		Latitude:  51.551700592,
		Longitude: -122.327003479,
		Timezone:  "America/Vancouver",
	},
	"CAZ5": {
		ICAO:      "CAZ5",
		Name:      "Cache Creek-Ashcroft Regional Airport",
		City:      "Cache Creek",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2034,
		Latitude:  50.7751091109,
		Longitude: -121.321227551,
		Timezone:  "America/Vancouver",
	},
	"CBA8": {
		ICAO:      "CBA8",
		Name:      "Beaverley Airport",
		City:      "Beaverley",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2420,
		Latitude:  53.8555984497,
		Longitude: -122.9079971313,
		Timezone:  "America/Vancouver",
	},
	"CBA9": {
		ICAO:      "CBA9",
		Name:      "Ospika Airport",
		City:      "Ospika",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2300,
		Latitude:  56.2750015259,
		Longitude: -124.0520019531,
		Timezone:  "America/Vancouver",
	},
	"CBB7": {
		ICAO:      "CBB7",
		Name:      "Tipella Airport",
		City:      "Tipella",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 55,
		Latitude:  49.7430992126,
		Longitude: -122.1630020142,
		Timezone:  "America/Vancouver",
	},
	"CBB9": {
		ICAO:      "CBB9",
		Name:      "Osoyoos Airport",
		City:      "Osoyoos",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 1100,
		Latitude:  49.0372009277,
		Longitude: -119.488998413,
		Timezone:  "America/Los_Angeles",
	},
	"CBBC": {
		ICAO:      "CBBC",
		IATA:      "ZEL",
		Name:      "Bella Bella (Campbell Island) Airport",
		City:      "Bella Bella",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 141,
		Latitude:  52.1850013733,
		Longitude: -128.1569976807,
		Timezone:  "America/Vancouver",
	},
	"CBC2": {
		ICAO:      "CBC2",
		Name:      "Ford Bay Airport",
		City:      "Ford Bay",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 673,
		Latitude:  66.0374984741,
		Longitude: -124.7149963379,
		Timezone:  "America/Inuvik",
	},
	"CBD6": {
		ICAO:      "CBD6",
		Name:      "Nahanni Butte Airport",
		City:      "Nahanni Butte",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 600,
		Latitude:  61.0297012329,
		Longitude: -123.388999939,
		Timezone:  "America/Inuvik",
	},
	"CBE2": {
		ICAO:      "CBE2",
		Name:      "Elko Airport",
		City:      "Elko",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2850,
		Latitude:  49.2832984924,
		Longitude: -115.1500015259,
		Timezone:  "America/Edmonton",
	},
	"CBF2": {
		ICAO:      "CBF2",
		Name:      "Belwood (Baird Field)",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1410,
		Latitude:  43.808333333,
		Longitude: -80.3119444444,
		Timezone:  "America/Toronto",
	},
	"CBF9": {
		ICAO:      "CBF9",
		Name:      "Mabel Lake Airport",
		City:      "Mabel Lake",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 1410,
		Latitude:  50.6088981628,
		Longitude: -118.7310028076,
		Timezone:  "America/Vancouver",
	},
	"CBG2": {
		ICAO:      "CBG2",
		Name:      "Green Lake Airport",
		City:      "Green Lake",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 3550,
		Latitude:  51.4294013977,
		Longitude: -121.2099990845,
		Timezone:  "America/Vancouver",
	},
	"CBH2": {
		ICAO:      "CBH2",
		Name:      "Helmet Airport",
		City:      "Helmet",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 1930,
		Latitude:  59.4258003235,
		Longitude: -120.797996521,
		Timezone:  "America/Fort_Nelson",
	},
	"CBH4": {
		ICAO:      "CBH4",
		Name:      "Prairie Creek Airport",
		City:      "Prairie Creek",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 2950,
		Latitude:  61.5647010803,
		Longitude: -124.8150024414,
		Timezone:  "America/Inuvik",
	},
	"CBJ4": {
		ICAO:      "CBJ4",
		Name:      "Echo Valley Airport",
		City:      "Echo Valley",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 3650,
		Latitude:  51.2416992188,
		Longitude: -121.9940032959,
		Timezone:  "America/Vancouver",
	},
	"CBK6": {
		ICAO:      "CBK6",
		Name:      "Quesnel Lake Airport",
		City:      "Quesnel Lake",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2500,
		Latitude:  52.5149993896,
		Longitude: -121.0449981689,
		Timezone:  "America/Vancouver",
	},
	"CBK7": {
		ICAO:      "CBK7",
		Name:      "Mile 422 (Alaska Highway) Airport",
		City:      "Toad River",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2400,
		Latitude:  58.8493995667,
		Longitude: -125.2399978638,
		Timezone:  "America/Fort_Nelson",
	},
	"CBL3": {
		ICAO:      "CBL3",
		Name:      "Gordon Field",
		City:      "Fort Nelson",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 1625,
		Latitude:  58.8166999817,
		Longitude: -122.7829971313,
		Timezone:  "America/Fort_Nelson",
	},
	"CBL6": {
		ICAO:      "CBL6",
		Name:      "Radium Hot Springs Airport",
		City:      "Radium Hot Springs",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2650,
		Latitude:  50.6333007813,
		Longitude: -116.0999984741,
		Timezone:  "America/Edmonton",
	},
	"CBL9": {
		ICAO:      "CBL9",
		Name:      "Elkin Creek Guest Ranch Airport",
		City:      "Elkin Creek",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 4080,
		Latitude:  51.5127983093,
		Longitude: -123.8040008545,
		Timezone:  "America/Vancouver",
	},
	"CBM5": {
		ICAO:      "CBM5",
		Name:      "Telegraph Creek Airport",
		City:      "Telegraph Creek",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 1100,
		Latitude:  57.9166984558,
		Longitude: -131.1170043945,
		Timezone:  "America/Vancouver",
	},
	"CBM6": {
		ICAO:      "CBM6",
		Name:      "Midway Airport",
		City:      "Midway",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 1896,
		Latitude:  49.0099983215,
		Longitude: -118.7900009155,
		Timezone:  "America/Los_Angeles",
	},
	"CBN9": {
		ICAO:      "CBN9",
		Name:      "Tsay Keh Airport",
		City:      "Tsay Keh",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2280,
		Latitude:  56.9061012268,
		Longitude: -124.9649963379,
		Timezone:  "America/Vancouver",
	},
	"CBQ2": {
		ICAO:      "CBQ2",
		Name:      "Fort Langley Airport",
		City:      "Fort Langley",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 30,
		Latitude:  49.1674995422,
		Longitude: -122.5550003052,
		Timezone:  "America/Vancouver",
	},
	"CBQ7": {
		ICAO:      "CBQ7",
		Name:      "Kemess Creek Airport",
		City:      "Kemess Creek",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 4191,
		Latitude:  56.9743995667,
		Longitude: -126.7409973145,
		Timezone:  "America/Vancouver",
	},
	"CBQ8": {
		ICAO:      "CBQ8",
		Name:      "Woodcock Airport",
		City:      "Woodcock",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 537,
		Latitude:  55.0666999817,
		Longitude: -128.233001709,
		Timezone:  "America/Vancouver",
	},
	"CBR2": {
		ICAO:      "CBR2",
		Name:      "Kaslo Airport",
		City:      "Kaslo",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2354,
		Latitude:  49.9035987854,
		Longitude: -116.9349975586,
		Timezone:  "America/Vancouver",
	},
	"CBR4": {
		ICAO:      "CBR4",
		Name:      "Clinton / Bleibler Ranch",
		City:      "Clinton",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 3695,
		Latitude:  51.2668021597,
		Longitude: -121.685771942,
		Timezone:  "America/Vancouver",
	},
	"CBS4": {
		ICAO:      "CBS4",
		Name:      "Mule Creek Airport",
		City:      "Mule Creek",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2900,
		Latitude:  59.7855987549,
		Longitude: -136.5829925537,
		Timezone:  "America/Vancouver",
	},
	"CBS7": {
		ICAO:      "CBS7",
		Name:      "Briercrest South Airport",
		City:      "Briercrest South",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2120,
		Latitude:  50.0663986206,
		Longitude: -105.3010025024,
		Timezone:  "America/Regina",
	},
	"CBS8": {
		ICAO:      "CBS8",
		Name:      "Port Alberni Airport",
		City:      "Port Alberni",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 250,
		Latitude:  49.3218994141,
		Longitude: -124.9309997559,
		Timezone:  "America/Vancouver",
	},
	"CBT3": {
		ICAO:      "CBT3",
		Name:      "Tsetzi Lake (Pan Phillips) Airport",
		City:      "Tsetzi Lake",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 3550,
		Latitude:  52.9719009399,
		Longitude: -125.0270004272,
		Timezone:  "America/Vancouver",
	},
	"CBT6": {
		ICAO:      "CBT6",
		Name:      "Quilchena Airport",
		City:      "Quilchena",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2100,
		Latitude:  50.1618895996,
		Longitude: -120.507745743,
		Timezone:  "America/Vancouver",
	},
	"CBU2": {
		ICAO:      "CBU2",
		Name:      "Eddontenajon / Iskut Village Airport",
		City:      "Eddontenajon",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 3100,
		Latitude:  57.847994155,
		Longitude: -129.983968735,
		Timezone:  "America/Vancouver",
	},
	"CBW2": {
		ICAO:      "CBW2",
		Name:      "Kitimat Airport",
		City:      "Kitimat",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 250,
		Latitude:  54.163898468,
		Longitude: -128.5809936523,
		Timezone:  "America/Vancouver",
	},
	"CBW3": {
		ICAO:      "CBW3",
		Name:      "Fort Grahame Airport",
		City:      "Fort Graham",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2230,
		Latitude:  56.5213877877,
		Longitude: -124.470291138,
		Timezone:  "America/Vancouver",
	},
	"CBW4": {
		ICAO:      "CBW4",
		Name:      "Bob Quinn Lake Airport",
		City:      "Bob Quinn Lake",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2000,
		Latitude:  56.9667015076,
		Longitude: -130.25,
		Timezone:  "America/Vancouver",
	},
	"CBW6": {
		ICAO:      "CBW6",
		Name:      "Belwood (Wright Field)",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1490,
		Latitude:  43.793888889,
		Longitude: -80.4025,
		Timezone:  "America/Toronto",
	},
	"CBW8": {
		ICAO:      "CBW8",
		Name:      "Prince George (North Cariboo Air Park) Airport",
		City:      "Prince George",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2500,
		Latitude:  54.0080986023,
		Longitude: -123.0230026245,
		Timezone:  "America/Vancouver",
	},
	"CBX5": {
		ICAO:      "CBX5",
		Name:      "Tungsten (Cantung) Airport",
		City:      "Tungsten",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 3500,
		Latitude:  61.9569015503,
		Longitude: -128.2030029297,
		Timezone:  "America/Inuvik",
	},
	"CBX7": {
		ICAO:      "CBX7",
		Name:      "Tumbler Ridge Airport",
		City:      "Tumbler Ridge",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 3075,
		Latitude:  55.0250015259,
		Longitude: -120.9349975586,
		Timezone:  "America/Dawson_Creek",
	},
	"CBZ9": {
		ICAO:      "CBZ9",
		Name:      "Fraser Lake Airport",
		City:      "Fraser Lake",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2690,
		Latitude:  54.0133018494,
		Longitude: -124.7679977417,
		Timezone:  "America/Vancouver",
	},
	"CCA3": {
		ICAO:      "CCA3",
		Name:      "Cable Head Airpark",
		City:      "Cable Head",
		State:     "Prince Edward Island",
		Country:   "CA",
		Elevation: 94,
		Latitude:  46.4432983398,
		Longitude: -62.5928001404,
		Timezone:  "America/Halifax",
	},
	"CCA6": {
		ICAO:      "CCA6",
		IATA:      "YWM",
		Name:      "Williams Harbour Airport",
		City:      "Williams Harbour",
		State:     "Newfoundland and Labrador",
		Country:   "CA",
		Elevation: 70,
		Latitude:  52.5668983459,
		Longitude: -55.7846984863,
		Timezone:  "America/St_Johns",
	},
	"CCA7": {
		ICAO:      "CCA7",
		Name:      "Apple River Airport",
		City:      "Apple River",
		State:     "Nova Scotia",
		Country:   "CA",
		Elevation: 135,
		Latitude:  45.4618988037,
		Longitude: -64.8214035034,
		Timezone:  "America/Halifax",
	},
	"CCC2": {
		ICAO:      "CCC2",
		Name:      "Winterland Airport",
		City:      "Winterland",
		State:     "Newfoundland and Labrador",
		Country:   "CA",
		Elevation: 156,
		Latitude:  47.1369018555,
		Longitude: -55.3292007446,
		Timezone:  "America/St_Johns",
	},
	"CCD2": {
		ICAO:      "CCD2",
		Name:      "Springdale Airport",
		City:      "Springdale",
		State:     "Newfoundland and Labrador",
		Country:   "CA",
		Elevation: 250,
		Latitude:  49.4789009094,
		Longitude: -56.1781005859,
		Timezone:  "America/St_Johns",
	},
	"CCD3": {
		ICAO:      "CCD3",
		Name:      "Woodstock Airport",
		City:      "Woodstock",
		State:     "New Brunswick",
		Country:   "CA",
		Elevation: 481,
		Latitude:  46.1522484697,
		Longitude: -67.5452327728,
		Timezone:  "America/Moncton",
	},
	"CCD4": {
		ICAO:      "CCD4",
		Name:      "Postville Airport",
		City:      "Postville",
		State:     "Newfoundland and Labrador",
		Country:   "CA",
		Elevation: 193,
		Latitude:  54.9105987549,
		Longitude: -59.7867012024,
		Timezone:  "America/Goose_Bay",
	},
	"CCE2": {
		ICAO:      "CCE2",
		Name:      "Mount Pleasant Airport",
		City:      "Mount Pleasant",
		State:     "Prince Edward Island",
		Country:   "CA",
		Latitude:  46.5999984741,
		Longitude: -64,
		Timezone:  "America/Halifax",
	},
	"CCE3": {
		ICAO:      "CCE3",
		Name:      "Juniper Airport",
		City:      "Juniper",
		State:     "New Brunswick",
		Country:   "CA",
		Elevation: 837,
		Latitude:  46.5628013611,
		Longitude: -67.168296814,
		Timezone:  "America/Moncton",
	},
	"CCE4": {
		ICAO:      "CCE4",
		Name:      "Black Tickle Airport",
		City:      "Black Tickle",
		State:     "Newfoundland and Labrador",
		Country:   "CA",
		Elevation: 57,
		Latitude:  53.4693984985,
		Longitude: -55.7849998474,
		Timezone:  "America/Goose_Bay",
	},
	"CCF4": {
		ICAO:      "CCF4",
		Name:      "Porters Lake Airport",
		City:      "Porters Lake",
		State:     "Nova Scotia",
		Country:   "CA",
		Elevation: 20,
		Latitude:  44.7099936874,
		Longitude: -63.2996821404,
		Timezone:  "America/Halifax",
	},
	"CCF6": {
		ICAO:      "CCF6",
		Name:      "Edmonton/Morinville (Currie Field)",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2374,
		Latitude:  53.819722,
		Longitude: -113.760833,
		Timezone:  "America/Edmonton",
	},
	"CCF9": {
		ICAO:      "CCF9",
		Name:      "Scottsfield Airpark",
		City:      "Scottsfield",
		State:     "New Brunswick",
		Country:   "CA",
		Elevation: 600,
		Latitude:  45.9603004456,
		Longitude: -67.0952987671,
		Timezone:  "America/Moncton",
	},
	"CCG3": {
		ICAO:      "CCG3",
		Name:      "Weyman Airpark",
		City:      "Burtt's Corner",
		State:     "New Brunswick",
		Country:   "CA",
		Elevation: 140,
		Latitude:  46.0374984741,
		Longitude: -66.8589019775,
		Timezone:  "America/Moncton",
	},
	"CCG4": {
		ICAO:      "CCG4",
		Name:      "Moncton / McEwen Airport",
		City:      "Moncton",
		State:     "New Brunswick",
		Country:   "CA",
		Elevation: 214,
		Latitude:  46.1539001465,
		Longitude: -64.7686004639,
		Timezone:  "America/Moncton",
	},
	"CCH2": {
		ICAO:      "CCH2",
		Name:      "Upper Kent Airport",
		State:     "Maine",
		Country:   "CA",
		Elevation: 246,
		Latitude:  46.587137287,
		Longitude: -67.7195549011,
		Timezone:  "America/Moncton",
	},
	"CCH4": {
		ICAO:      "CCH4",
		Name:      "Charlottetown Airport",
		City:      "Charlottetown",
		State:     "Newfoundland and Labrador",
		Country:   "CA",
		Elevation: 209,
		Latitude:  52.7649993896,
		Longitude: -56.1156005859,
		Timezone:  "America/St_Johns",
	},
	"CCI9": {
		ICAO:      "CCI9",
		Name:      "Cortes Island (Hansen Airfield) Airport",
		City:      "Cortes Island",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 164,
		Latitude:  50.0203018188,
		Longitude: -124.9840011597,
		Timezone:  "America/Vancouver",
	},
	"CCJ3": {
		ICAO:      "CCJ3",
		Name:      "Boston Brook Airport",
		City:      "Boston Brook",
		State:     "New Brunswick",
		Country:   "CA",
		Elevation: 958,
		Latitude:  47.448600769,
		Longitude: -67.6247024536,
		Timezone:  "America/Moncton",
	},
	"CCK3": {
		ICAO:      "CCK3",
		Name:      "Grand Falls Airport",
		City:      "Grand Falls",
		State:     "New Brunswick",
		Country:   "CA",
		Elevation: 712,
		Latitude:  47.0750007629,
		Longitude: -67.6849975586,
		Timezone:  "America/Moncton",
	},
	"CCK4": {
		ICAO:      "CCK4",
		IATA:      "YFX",
		Name:      "St. Lewis (Fox Harbour) Airport",
		City:      "St. Lewis",
		State:     "Newfoundland and Labrador",
		Country:   "CA",
		Elevation: 74,
		Latitude:  52.3727989197,
		Longitude: -55.6739006042,
		Timezone:  "America/St_Johns",
	},
	"CCK5": {
		ICAO:      "CCK5",
		Name:      "Owen Sound (Cook Field)",
		City:      "Owen Sound",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1040,
		Latitude:  44.6449639588,
		Longitude: -80.7456064224,
		Timezone:  "America/Toronto",
	},
	"CCL2": {
		ICAO:      "CCL2",
		Name:      "Candle Lake Airpark",
		City:      "Candle Lake",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1650,
		Latitude:  53.7682991028,
		Longitude: -105.3079986572,
		Timezone:  "America/Regina",
	},
	"CCL3": {
		ICAO:      "CCL3",
		Name:      "Christina Lake",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 1905,
		Latitude:  55.6283836075,
		Longitude: -110.751113892,
		Timezone:  "America/Edmonton",
	},
	"CCM3": {
		ICAO:      "CCM3",
		Name:      "Sevogle Airport",
		City:      "Sevogle",
		State:     "New Brunswick",
		Country:   "CA",
		Elevation: 1350,
		Latitude:  47.1910552979,
		Longitude: -66.1606140137,
		Timezone:  "America/Moncton",
	},
	"CCM4": {
		ICAO:      "CCM4",
		Name:      "Port Au Choix Airport",
		City:      "Port Au Choix",
		State:     "Newfoundland and Labrador",
		Country:   "CA",
		Elevation: 90,
		Latitude:  50.6888999939,
		Longitude: -57.331401825,
		Timezone:  "America/St_Johns",
	},
	"CCN2": {
		ICAO:      "CCN2",
		Name:      "Grand Manan Airport",
		City:      "Grand Manan",
		State:     "New Brunswick",
		Country:   "CA",
		Elevation: 238,
		Latitude:  44.7132987976,
		Longitude: -66.7964019775,
		Timezone:  "America/Moncton",
	},
	"CCP2": {
		ICAO:      "CCP2",
		Name:      "Exploits Valley Botwood Airport",
		City:      "Exploits Valley",
		State:     "Newfoundland and Labrador",
		Country:   "CA",
		Elevation: 365,
		Latitude:  49.0559997559,
		Longitude: -55.4477005005,
		Timezone:  "America/St_Johns",
	},
	"CCP3": {
		ICAO:      "CCP3",
		Name:      "Chute-St-Philippe Airport",
		City:      "Chute-St-Philippe",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 860,
		Latitude:  46.6610984802,
		Longitude: -75.2450027466,
		Timezone:  "America/Toronto",
	},
	"CCP4": {
		ICAO:      "CCP4",
		IATA:      "YHA",
		Name:      "Port Hope Simpson Airport",
		City:      "Port Hope Simpson",
		State:     "Newfoundland and Labrador",
		Country:   "CA",
		Elevation: 347,
		Latitude:  52.5280990601,
		Longitude: -56.2860984802,
		Timezone:  "America/St_Johns",
	},
	"CCP6": {
		ICAO:      "CCP6",
		Name:      "Caniapiscau Airport",
		City:      "Caniapiscau",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 1672,
		Latitude:  54.8377990723,
		Longitude: -69.8927993774,
		Timezone:  "America/Toronto",
	},
	"CCQ3": {
		ICAO:      "CCQ3",
		Name:      "Debert Airport",
		City:      "Debert",
		State:     "Nova Scotia",
		Country:   "CA",
		Elevation: 142,
		Latitude:  45.418598175,
		Longitude: -63.4606018066,
		Timezone:  "America/Halifax",
	},
	"CCR3": {
		ICAO:      "CCR3",
		Name:      "Florenceville Airport",
		City:      "Florenceville",
		State:     "New Brunswick",
		Country:   "CA",
		Elevation: 508,
		Latitude:  46.4261016846,
		Longitude: -67.6280975342,
		Timezone:  "America/Moncton",
	},
	"CCS3": {
		ICAO:      "CCS3",
		Name:      "St. Stephen Airport",
		City:      "St. Stephen",
		State:     "New Brunswick",
		Country:   "CA",
		Elevation: 96,
		Latitude:  45.2075004578,
		Longitude: -67.2506027222,
		Timezone:  "America/Moncton",
	},
	"CCS4": {
		ICAO:      "CCS4",
		Name:      "Chipman Airport",
		City:      "Chipman",
		State:     "New Brunswick",
		Country:   "CA",
		Elevation: 65,
		Latitude:  46.148601532,
		Longitude: -65.9041976929,
		Timezone:  "America/Moncton",
	},
	"CCS5": {
		ICAO:      "CCS5",
		Name:      "Havelock Airport",
		City:      "Havelock",
		State:     "New Brunswick",
		Country:   "CA",
		Elevation: 425,
		Latitude:  45.9864006042,
		Longitude: -65.301902771,
		Timezone:  "America/Moncton",
	},
	"CCS6": {
		ICAO:      "CCS6",
		Name:      "Courtenay (Smit Field) Airport",
		City:      "Courtenay",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 500,
		Latitude:  49.6824989319,
		Longitude: -125.1009979248,
		Timezone:  "America/Vancouver",
	},
	"CCT2": {
		ICAO:      "CCT2",
		Name:      "Cookstown Airport",
		City:      "Cookstown",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 750,
		Latitude:  44.238899231,
		Longitude: -79.6389007568,
		Timezone:  "America/Toronto",
	},
	"CCV4": {
		ICAO:      "CCV4",
		Name:      "Bell Island Airport",
		City:      "Bell Island",
		State:     "Newfoundland and Labrador",
		Country:   "CA",
		Elevation: 150,
		Latitude:  47.6333007813,
		Longitude: -52.9832992554,
		Timezone:  "America/St_Johns",
	},
	"CCW3": {
		ICAO:      "CCW3",
		Name:      "Waterville / Kings County Municipal Airport",
		City:      "Waterville",
		State:     "Nova Scotia",
		Country:   "CA",
		Elevation: 119,
		Latitude:  45.0518989563,
		Longitude: -64.6517028809,
		Timezone:  "America/Halifax",
	},
	"CCW4": {
		ICAO:      "CCW4",
		Name:      "Stanley Airport",
		City:      "Stanley",
		State:     "Nova Scotia",
		Country:   "CA",
		Elevation: 95,
		Latitude:  45.1006011963,
		Longitude: -63.9206008911,
		Timezone:  "America/Halifax",
	},
	"CCX3": {
		ICAO:      "CCX3",
		Name:      "Brockway Airport",
		City:      "Brockway",
		State:     "New Brunswick",
		Country:   "CA",
		Elevation: 300,
		Latitude:  45.5666999817,
		Longitude: -67.0999984741,
		Timezone:  "America/Moncton",
	},
	"CCY3": {
		ICAO:      "CCY3",
		Name:      "Sussex Airport",
		City:      "Sussex",
		State:     "New Brunswick",
		Country:   "CA",
		Elevation: 467,
		Latitude:  45.6899986267,
		Longitude: -65.5400009155,
		Timezone:  "America/Moncton",
	},
	"CCY4": {
		ICAO:      "CCY4",
		Name:      "East Gore Eco Airpark",
		City:      "East Gore",
		State:     "Nova Scotia",
		Country:   "CA",
		Elevation: 615,
		Latitude:  45.1175003052,
		Longitude: -63.7028007507,
		Timezone:  "America/Halifax",
	},
	"CCZ2": {
		ICAO:      "CCZ2",
		IATA:      "YRG",
		Name:      "Rigolet Airport",
		City:      "Rigolet",
		State:     "Newfoundland and Labrador",
		Country:   "CA",
		Elevation: 180,
		Latitude:  54.1796989441,
		Longitude: -58.4575004578,
		Timezone:  "America/Goose_Bay",
	},
	"CCZ3": {
		ICAO:      "CCZ3",
		Name:      "Clarenville Airport",
		City:      "Clarenville",
		State:     "Newfoundland and Labrador",
		Country:   "CA",
		Elevation: 199,
		Latitude:  48.2747001648,
		Longitude: -53.9239006042,
		Timezone:  "America/St_Johns",
	},
	"CCZ4": {
		ICAO:      "CCZ4",
		Name:      "Margaree Airport",
		City:      "Margaree",
		State:     "Nova Scotia",
		Country:   "CA",
		Elevation: 181,
		Latitude:  46.3410987854,
		Longitude: -60.9796981812,
		Timezone:  "America/Halifax",
	},
	"CCZ5": {
		ICAO:      "CCZ5",
		Name:      "Thorburn Airport",
		City:      "Thorburn",
		State:     "Nova Scotia",
		Country:   "CA",
		Elevation: 120,
		Latitude:  45.5606002808,
		Longitude: -62.5946998596,
		Timezone:  "America/Halifax",
	},
	"CD01": {
		ICAO:      "CD01",
		Name:      "Lowe Airstrip",
		City:      "Cotopaxi",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6200,
		Latitude:  38.3983001709,
		Longitude: -105.6200027466,
		Timezone:  "America/Denver",
	},
	"CD02": {
		ICAO:      "CD02",
		Name:      "Skyote Airport",
		City:      "Steamboat Springs",
		State:     "Colorado",
		Country:   "US",
		Elevation: 8200,
		Latitude:  40.7579994202,
		Longitude: -106.9720001221,
		Timezone:  "America/Denver",
	},
	"CD03": {
		ICAO:      "CD03",
		Name:      "Tinnes Airport",
		City:      "Lamar",
		State:     "Colorado",
		Country:   "US",
		Elevation: 3850,
		Latitude:  38.1990013123,
		Longitude: -102.5729980469,
		Timezone:  "America/Denver",
	},
	"CD05": {
		ICAO:      "CD05",
		Name:      "Jackson Airfield",
		City:      "Yuma",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4075,
		Latitude:  40.1944007874,
		Longitude: -102.697998047,
		Timezone:  "America/Denver",
	},
	"CD09": {
		ICAO:      "CD09",
		Name:      "Yoder Airstrip",
		City:      "Bennett",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5345,
		Latitude:  39.8196983337,
		Longitude: -104.408996582,
		Timezone:  "America/Denver",
	},
	"CD10": {
		ICAO:      "CD10",
		Name:      "Chapman Field",
		City:      "South Fork",
		State:     "Colorado",
		Country:   "US",
		Elevation: 8100,
		Latitude:  37.6749992371,
		Longitude: -106.5559997559,
		Timezone:  "America/Denver",
	},
	"CD13": {
		ICAO:      "CD13",
		Name:      "Morris Airport",
		City:      "Deer Trail",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4750,
		Latitude:  39.8350982666,
		Longitude: -103.736000061,
		Timezone:  "America/Denver",
	},
	"CD14": {
		ICAO:      "CD14",
		Name:      "J & S Airport",
		City:      "Bennett",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5320,
		Latitude:  39.8288993835,
		Longitude: -104.43699646,
		Timezone:  "America/Denver",
	},
	"CD15": {
		ICAO:      "CD15",
		Name:      "Schantz Airstrip",
		City:      "Simla",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5870,
		Latitude:  39.2963981628,
		Longitude: -104.125,
		Timezone:  "America/Denver",
	},
	"CD17": {
		ICAO:      "CD17",
		Name:      "Bijou Basin Airport",
		City:      "Byers",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4885,
		Latitude:  39.9082984924,
		Longitude: -104.1259994507,
		Timezone:  "America/Denver",
	},
	"CD20": {
		ICAO:      "CD20",
		Name:      "Sprague Airport",
		City:      "Loveland",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5603,
		Latitude:  40.4749984741,
		Longitude: -105.2200012207,
		Timezone:  "America/Denver",
	},
	"CD23": {
		ICAO:      "CD23",
		Name:      "Aero Bear Field",
		City:      "Calhan",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6300,
		Latitude:  39.198299408,
		Longitude: -104.3290023804,
		Timezone:  "America/Denver",
	},
	"CD28": {
		ICAO:      "CD28",
		Name:      "Tall Timber Airport",
		City:      "Indian Hills",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7360,
		Latitude:  39.643901825,
		Longitude: -105.2679977417,
		Timezone:  "America/Denver",
	},
	"CD32": {
		ICAO:      "CD32",
		Name:      "Castle Lakes Airport",
		City:      "Lake City",
		State:     "Colorado",
		Country:   "US",
		Elevation: 9300,
		Latitude:  37.909198761,
		Longitude: -107.3499984741,
		Timezone:  "America/Denver",
	},
	"CD39": {
		ICAO:      "CD39",
		Name:      "Pond's Field",
		City:      "Berthoud",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5050,
		Latitude:  40.2649993896,
		Longitude: -105.1370010376,
		Timezone:  "America/Denver",
	},
	"CD45": {
		ICAO:      "CD45",
		Name:      "Flyin' B Ranch Airport",
		City:      "Elizabeth",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6720,
		Latitude:  39.323299408,
		Longitude: -104.5729980469,
		Timezone:  "America/Denver",
	},
	"CD48": {
		ICAO:      "CD48",
		Name:      "Cuchara Ranch Airport",
		City:      "Walsenburg",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5827,
		Latitude:  37.7863998413,
		Longitude: -104.59400177,
		Timezone:  "America/Denver",
	},
	"CD69": {
		ICAO:      "CD69",
		Name:      "Morning Shadows Ranch Airport",
		City:      "South Fork",
		State:     "Colorado",
		Country:   "US",
		Elevation: 8300,
		Latitude:  37.7402992249,
		Longitude: -106.5260009766,
		Timezone:  "America/Denver",
	},
	"CD82": {
		ICAO:      "CD82",
		Name:      "Val Air Airport",
		City:      "Durango",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6548,
		Latitude:  37.3377990723,
		Longitude: -107.8519973755,
		Timezone:  "America/Denver",
	},
	"CD97": {
		ICAO:      "CD97",
		Name:      "Montemadeira Ii Airport",
		City:      "Hotchkiss",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5750,
		Latitude:  38.8055992126,
		Longitude: -107.7740020752,
		Timezone:  "America/Denver",
	},
	"CD99": {
		ICAO:      "CD99",
		Name:      "Lucky L Ranch Airport",
		City:      "Steamboat Springs",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7000,
		Latitude:  40.3582992554,
		Longitude: -106.8300018311,
		Timezone:  "America/Denver",
	},
	"CDA3": {
		ICAO:      "CDA3",
		Name:      "Valley Airport",
		City:      "Valley",
		State:     "Nova Scotia",
		Country:   "CA",
		Elevation: 175,
		Latitude:  45.3967018127,
		Longitude: -63.2155990601,
		Timezone:  "America/Halifax",
	},
	"CDA4": {
		ICAO:      "CDA4",
		Name:      "Pokemouche Airport",
		City:      "Pokemouche",
		State:     "New Brunswick",
		Country:   "CA",
		Elevation: 68,
		Latitude:  47.7164001465,
		Longitude: -64.8824996948,
		Timezone:  "America/Moncton",
	},
	"CDA5": {
		ICAO:      "CDA5",
		Name:      "St. Andrews Codroy Valley Airport",
		City:      "St. Andrews",
		State:     "Newfoundland and Labrador",
		Country:   "CA",
		Elevation: 65,
		Latitude:  47.773601532,
		Longitude: -59.3130989075,
		Timezone:  "America/St_Johns",
	},
	"CDA6": {
		ICAO:      "CDA6",
		Name:      "Bristol Airport",
		City:      "Bristol",
		State:     "New Brunswick",
		Country:   "CA",
		Elevation: 574,
		Latitude:  46.459400177,
		Longitude: -67.5646972656,
		Timezone:  "America/Moncton",
	},
	"CDC3": {
		ICAO:      "CDC3",
		Name:      "Dawson Creek (Flying L Ranch) Airport",
		City:      "Dawson Creek",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2680,
		Latitude:  55.8194885254,
		Longitude: -120.4602813721,
		Timezone:  "America/Dawson_Creek",
	},
	"CDC4": {
		ICAO:      "CDC4",
		Name:      "St-Quentin Airport",
		City:      "St-Quentin",
		State:     "New Brunswick",
		Country:   "CA",
		Elevation: 875,
		Latitude:  47.5210990906,
		Longitude: -67.4210968018,
		Timezone:  "America/Moncton",
	},
	"CDD3": {
		ICAO:      "CDD3",
		Name:      "Ste-Agnes-de-Dundee Airport",
		City:      "Ste-Agnes-de-Dundee",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 160,
		Latitude:  45.0475006104,
		Longitude: -74.3432998657,
		Timezone:  "America/Toronto",
	},
	"CDF3": {
		ICAO:      "CDF3",
		Name:      "Englehart (Dave's Field)",
		City:      "Englehart",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 700,
		Latitude:  47.809722223,
		Longitude: -79.8111111111,
		Timezone:  "America/Nipigon",
	},
	"CDG3": {
		ICAO:      "CDG3",
		Name:      "Dungannon",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 870,
		Latitude:  43.836389,
		Longitude: -81.606667,
		Timezone:  "America/Toronto",
	},
	"CDH1": {
		ICAO:      "CDH1",
		Name:      "Deerhurst Resort Airport",
		City:      "Huntsville",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1029,
		Latitude:  45.3541984558,
		Longitude: -79.1513977051,
		Timezone:  "America/Toronto",
	},
	"CDH3": {
		ICAO:      "CDH3",
		Name:      "Finlay Air Park",
		City:      "Finlay",
		State:     "Nova Scotia",
		Country:   "CA",
		Elevation: 145,
		Latitude:  43.9622001648,
		Longitude: -65.9942016602,
		Timezone:  "America/Halifax",
	},
	"CDJ4": {
		ICAO:      "CDJ4",
		Name:      "Clearwater Airport",
		City:      "Clearwater",
		State:     "New Brunswick",
		Country:   "CA",
		Elevation: 1330,
		Latitude:  46.7132987976,
		Longitude: -66.8283004761,
		Timezone:  "America/Moncton",
	},
	"CDJ5": {
		ICAO:      "CDJ5",
		Name:      "Strathmore (D.J. Murray) Airport",
		City:      "Strathmore",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3150,
		Latitude:  51.1319007874,
		Longitude: -113.5599975586,
		Timezone:  "America/Edmonton",
	},
	"CDK2": {
		ICAO:      "CDK2",
		IATA:      "DVK",
		Name:      "Diavik Airport",
		City:      "Diavik",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 1413,
		Latitude:  64.5113983154,
		Longitude: -110.289001465,
		Timezone:  "America/Yellowknife",
	},
	"CDO2": {
		ICAO:      "CDO2",
		Name:      "Ostergard's Airport",
		City:      "Drumheller",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2900,
		Latitude:  51.2943992615,
		Longitude: -112.6139984131,
		Timezone:  "America/Edmonton",
	},
	"CDT5": {
		ICAO:      "CDT5",
		Name:      "Buctouche Airport",
		City:      "Buctouche",
		State:     "New Brunswick",
		Country:   "CA",
		Elevation: 57,
		Latitude:  46.5093994141,
		Longitude: -64.693901062,
		Timezone:  "America/Moncton",
	},
	"CDU6": {
		ICAO:      "CDU6",
		Name:      "Doaktown Airport",
		City:      "Doaktown",
		State:     "New Brunswick",
		Country:   "CA",
		Elevation: 326,
		Latitude:  46.5525016785,
		Longitude: -66.0939025879,
		Timezone:  "America/Moncton",
	},
	"CDU9": {
		ICAO:      "CDU9",
		Name:      "Dunnville Airport",
		City:      "Dunnville",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 600,
		Latitude:  42.8722000122,
		Longitude: -79.5958023071,
		Timezone:  "America/Toronto",
	},
	"CDV2": {
		ICAO:      "CDV2",
		Name:      "Downs Gulch Airport",
		City:      "Downs Gulch",
		State:     "New Brunswick",
		Country:   "CA",
		Elevation: 883,
		Latitude:  47.7536010742,
		Longitude: -67.4261016846,
		Timezone:  "America/Moncton",
	},
	"CDW2": {
		ICAO:      "CDW2",
		Name:      "Baddeck (Crown Jewel) Airport",
		City:      "Baddeck",
		State:     "Nova Scotia",
		Country:   "CA",
		Elevation: 290,
		Latitude:  46.1650009155,
		Longitude: -60.7839012146,
		Timezone:  "America/Halifax",
	},
	"CDY3": {
		ICAO:      "CDY3",
		Name:      "Fogo Airport",
		City:      "Fogo",
		State:     "Newfoundland and Labrador",
		Country:   "CA",
		Elevation: 80,
		Latitude:  49.6575012207,
		Longitude: -54.2374992371,
		Timezone:  "America/St_Johns",
	},
	"CDY6": {
		ICAO:      "CDY6",
		Name:      "Bridgewater / Dayspring Airpark",
		City:      "Bridgewater",
		State:     "Nova Scotia",
		Country:   "CA",
		Elevation: 150,
		Latitude:  44.3819007874,
		Longitude: -64.456703186,
		Timezone:  "America/Halifax",
	},
	"CEA3": {
		ICAO:      "CEA3",
		Name:      "Olds-Didsbury Airport",
		City:      "Olds-Didsbury",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3360,
		Latitude:  51.7118988037,
		Longitude: -114.107002258,
		Timezone:  "America/Edmonton",
	},
	"CEA5": {
		ICAO:      "CEA5",
		Name:      "Hardisty Airport",
		City:      "Hardisty",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2326,
		Latitude:  52.646900177,
		Longitude: -111.3840026855,
		Timezone:  "America/Edmonton",
	},
	"CEA6": {
		ICAO:      "CEA6",
		Name:      "Cardston Airport",
		City:      "Cardston",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3887,
		Latitude:  49.1624984741,
		Longitude: -113.2409973145,
		Timezone:  "America/Edmonton",
	},
	"CEB3": {
		ICAO:      "CEB3",
		IATA:      "YCK",
		Name:      "Colville Lake Airport",
		City:      "Colville Lake",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 850,
		Latitude:  67.0333023071,
		Longitude: -126.0830001831,
		Timezone:  "America/Inuvik",
	},
	"CEB5": {
		ICAO:      "CEB5",
		Name:      "Fairview Airport",
		City:      "Fairview",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2166,
		Latitude:  56.081401825,
		Longitude: -118.4349975586,
		Timezone:  "America/Edmonton",
	},
	"CEC3": {
		ICAO:      "CEC3",
		Name:      "Fox Lake Airport",
		City:      "Fox Lake",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 850,
		Latitude:  58.4733009338,
		Longitude: -114.5429992676,
		Timezone:  "America/Edmonton",
	},
	"CEC4": {
		ICAO:      "CEC4",
		Name:      "Hinton / Jasper-Hinton Airport",
		City:      "Hinton",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 4006,
		Latitude:  53.3191986084,
		Longitude: -117.752998352,
		Timezone:  "America/Edmonton",
	},
	"CED3": {
		ICAO:      "CED3",
		Name:      "Oyen Municipal Airport",
		City:      "Oyen",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2498,
		Latitude:  51.3342018127,
		Longitude: -110.4909973145,
		Timezone:  "America/Edmonton",
	},
	"CED4": {
		ICAO:      "CED4",
		Name:      "Fox Creek Airport",
		City:      "Fox Creek",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2842,
		Latitude:  54.3800010681,
		Longitude: -116.7659988403,
		Timezone:  "America/Edmonton",
	},
	"CED5": {
		ICAO:      "CED5",
		Name:      "Taber Airport",
		City:      "Taber",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2648,
		Latitude:  49.8266983032,
		Longitude: -112.1849975586,
		Timezone:  "America/Edmonton",
	},
	"CED6": {
		ICAO:      "CED6",
		Name:      "Highwood Airport",
		City:      "De Winton",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3450,
		Latitude:  50.8018989563,
		Longitude: -113.888999939,
		Timezone:  "America/Edmonton",
	},
	"CEE4": {
		ICAO:      "CEE4",
		Name:      "Hinton / Entrance Airport",
		City:      "Hinton",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3450,
		Latitude:  53.381401062,
		Longitude: -117.700996399,
		Timezone:  "America/Edmonton",
	},
	"CEE5": {
		ICAO:      "CEE5",
		Name:      "Wabasca Airport",
		City:      "Wabasca",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 1827,
		Latitude:  55.9618988037,
		Longitude: -113.819000244,
		Timezone:  "America/Edmonton",
	},
	"CEE6": {
		ICAO:      "CEE6",
		Name:      "Edmonton / Twin Island Airpark",
		City:      "Edmonton",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2435,
		Latitude:  53.4710998535,
		Longitude: -113.153999329,
		Timezone:  "America/Edmonton",
	},
	"CEE8": {
		ICAO:      "CEE8",
		Name:      "Viking Airport",
		City:      "Viking",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2260,
		Latitude:  53.1089685346,
		Longitude: -111.864938736,
		Timezone:  "America/Edmonton",
	},
	"CEF2": {
		ICAO:      "CEF2",
		Name:      "Belwood (Ellen Field)",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1540,
		Latitude:  43.8372354306,
		Longitude: -80.3694534302,
		Timezone:  "America/Toronto",
	},
	"CEF3": {
		ICAO:      "CEF3",
		Name:      "Bow Island Airport",
		City:      "Bow Island",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2634,
		Latitude:  49.8833007813,
		Longitude: -111.3330001831,
		Timezone:  "America/Edmonton",
	},
	"CEF4": {
		ICAO:      "CEF4",
		Name:      "Airdrie Airport",
		City:      "Airdrie",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3648,
		Latitude:  51.2639007568,
		Longitude: -113.9339981079,
		Timezone:  "America/Edmonton",
	},
	"CEF6": {
		ICAO:      "CEF6",
		Name:      "Forestburg Airport",
		City:      "Forestburg",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2334,
		Latitude:  52.5746994019,
		Longitude: -112.0839996338,
		Timezone:  "America/Edmonton",
	},
	"CEG2": {
		ICAO:      "CEG2",
		Name:      "Acme Airport",
		City:      "Acme",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3035,
		Latitude:  51.456401825,
		Longitude: -113.5149993896,
		Timezone:  "America/Edmonton",
	},
	"CEG3": {
		ICAO:      "CEG3",
		Name:      "Lacombe Airport",
		City:      "Lacombe",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2783,
		Latitude:  52.4883003235,
		Longitude: -113.7119979858,
		Timezone:  "America/Edmonton",
	},
	"CEG4": {
		ICAO:      "CEG4",
		Name:      "Drumheller Municipal Airport",
		City:      "Drumheller",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2597,
		Latitude:  51.4963989258,
		Longitude: -112.7490005493,
		Timezone:  "America/Edmonton",
	},
	"CEG5": {
		ICAO:      "CEG5",
		Name:      "Chipewyan Lake Airport",
		City:      "Chipewyan Lake",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 1800,
		Latitude:  56.9524993896,
		Longitude: -113.4960021973,
		Timezone:  "America/Edmonton",
	},
	"CEG8": {
		ICAO:      "CEG8",
		Name:      "North Seal River Airport",
		City:      "North Seal River",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 980,
		Latitude:  58.9693984985,
		Longitude: -99.9749984741,
		Timezone:  "America/Winnipeg",
	},
	"CEH2": {
		ICAO:      "CEH2",
		Name:      "Cu Nim Airport",
		City:      "Black Diamond",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3700,
		Latitude:  50.7224998474,
		Longitude: -114.1790008545,
		Timezone:  "America/Edmonton",
	},
	"CEH3": {
		ICAO:      "CEH3",
		Name:      "Ponoka Industrial Airport",
		City:      "Ponoka Industrial",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2669,
		Latitude:  52.6516990662,
		Longitude: -113.6060028076,
		Timezone:  "America/Edmonton",
	},
	"CEH4": {
		ICAO:      "CEH4",
		Name:      "De Winton South Calgary Airport",
		City:      "De Winton",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3355,
		Latitude:  50.8218994141,
		Longitude: -113.8239974976,
		Timezone:  "America/Edmonton",
	},
	"CEH5": {
		ICAO:      "CEH5",
		Name:      "Red Earth Creek Airport",
		City:      "Red Earth Creek",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 1790,
		Latitude:  56.5463981628,
		Longitude: -115.2740020752,
		Timezone:  "America/Edmonton",
	},
	"CEH6": {
		ICAO:      "CEH6",
		Name:      "Provost Airport",
		City:      "Provost",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2197,
		Latitude:  52.3381004333,
		Longitude: -110.2789993286,
		Timezone:  "America/Edmonton",
	},
	"CEJ3": {
		ICAO:      "CEJ3",
		Name:      "Stettler Airport",
		City:      "Stettler",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2686,
		Latitude:  52.3100013733,
		Longitude: -112.7539978027,
		Timezone:  "America/Edmonton",
	},
	"CEJ4": {
		ICAO:      "CEJ4",
		Name:      "Claresholm Industrial Airport",
		City:      "Claresholm",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3325,
		Latitude:  50.004699707,
		Longitude: -113.6299972534,
		Timezone:  "America/Edmonton",
	},
	"CEJ6": {
		ICAO:      "CEJ6",
		Name:      "Elk Point Airport",
		City:      "Elk Point",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 1981,
		Latitude:  53.8931007385,
		Longitude: -110.7720031738,
		Timezone:  "America/Edmonton",
	},
	"CEK2": {
		ICAO:      "CEK2",
		Name:      "Braeburn Airport",
		City:      "Braeburn",
		State:     "Yukon",
		Country:   "CA",
		Elevation: 2400,
		Latitude:  61.4844017029,
		Longitude: -135.7760009766,
		Timezone:  "America/Whitehorse",
	},
	"CEK6": {
		ICAO:      "CEK6",
		Name:      "Killam-Sedgewick Airport",
		City:      "Killam",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2182,
		Latitude:  52.7999992371,
		Longitude: -111.766998291,
		Timezone:  "America/Edmonton",
	},
	"CEL3": {
		ICAO:      "CEL3",
		Name:      "East Linton (Kerr Field)",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 750,
		Latitude:  44.6610679784,
		Longitude: -80.9440255165,
		Timezone:  "America/Toronto",
	},
	"CEL4": {
		ICAO:      "CEL4",
		Name:      "Hanna Airport",
		City:      "Hanna",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2738,
		Latitude:  51.631401062,
		Longitude: -111.9039993286,
		Timezone:  "America/Edmonton",
	},
	"CEL5": {
		ICAO:      "CEL5",
		Name:      "Valleyview Airport",
		City:      "Valleyview",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2435,
		Latitude:  55.0325012207,
		Longitude: -117.2959976196,
		Timezone:  "America/Edmonton",
	},
	"CEL6": {
		ICAO:      "CEL6",
		Name:      "Two Hills Airport",
		City:      "Two Hills",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2010,
		Latitude:  53.7000007629,
		Longitude: -111.7829971313,
		Timezone:  "America/Edmonton",
	},
	"CEL8": {
		ICAO:      "CEL8",
		Name:      "Eleonore",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 750,
		Latitude:  52.718306623,
		Longitude: -76.0798072815,
		Timezone:  "America/Toronto",
	},
	"CEM3": {
		ICAO:      "CEM3",
		IATA:      "YLE",
		Name:      "Whati Airport",
		City:      "Whati",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 882,
		Latitude:  63.1316986084,
		Longitude: -117.2460021973,
		Timezone:  "America/Yellowknife",
	},
	"CEM4": {
		ICAO:      "CEM4",
		Name:      "Innisfail Airport",
		City:      "Innisfail",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3017,
		Latitude:  52.0786018372,
		Longitude: -114.0270004272,
		Timezone:  "America/Edmonton",
	},
	"CEM5": {
		ICAO:      "CEM5",
		Name:      "Swan Hills Airport",
		City:      "Swan Hills",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3473,
		Latitude:  54.6707992554,
		Longitude: -115.4150009155,
		Timezone:  "America/Edmonton",
	},
	"CEN2": {
		ICAO:      "CEN2",
		Name:      "Bassano Airport",
		City:      "Bassano",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2613,
		Latitude:  50.7999992371,
		Longitude: -112.4670028687,
		Timezone:  "America/Edmonton",
	},
	"CEN3": {
		ICAO:      "CEN3",
		Name:      "Three Hills Airport",
		City:      "Three Hills",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2975,
		Latitude:  51.6967010498,
		Longitude: -113.2089996338,
		Timezone:  "America/Edmonton",
	},
	"CEN4": {
		ICAO:      "CEN4",
		Name:      "High River Airport",
		City:      "High River",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3431,
		Latitude:  50.5332984924,
		Longitude: -113.8330001831,
		Timezone:  "America/Edmonton",
	},
	"CEN5": {
		ICAO:      "CEN5",
		Name:      "Cold Lake Regional Airport",
		City:      "Cold Lake",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 1786,
		Latitude:  54.4775009155,
		Longitude: -110.2689971924,
		Timezone:  "America/Edmonton",
	},
	"CEN6": {
		ICAO:      "CEN6",
		Name:      "Vauxhall Airport",
		City:      "Vauxhall",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2579,
		Latitude:  50.0332984924,
		Longitude: -112.0830001831,
		Timezone:  "America/Edmonton",
	},
	"CEP3": {
		ICAO:      "CEP3",
		Name:      "Barrhead Airport",
		City:      "Barrhead",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2121,
		Latitude:  54.0999984741,
		Longitude: -114.4329986572,
		Timezone:  "America/Edmonton",
	},
	"CEP4": {
		ICAO:      "CEP4",
		Name:      "Ross International Airport",
		City:      "Coutts",
		State:     "Montana",
		Country:   "CA",
		Elevation: 3550,
		Latitude:  49,
		Longitude: -111.9670028687,
		Timezone:  "America/Edmonton",
	},
	"CEP5": {
		ICAO:      "CEP5",
		Name:      "Janvier Airport",
		City:      "Janvier",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 1575,
		Latitude:  55.9000015259,
		Longitude: -110.733001709,
		Timezone:  "America/Edmonton",
	},
	"CEP6": {
		ICAO:      "CEP6",
		Name:      "Warner Airport",
		City:      "Warner",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3319,
		Latitude:  49.2933006287,
		Longitude: -112.1890029907,
		Timezone:  "America/Edmonton",
	},
	"CEQ3": {
		ICAO:      "CEQ3",
		Name:      "Camrose Airport",
		City:      "Camrose",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2426,
		Latitude:  53.0402984619,
		Longitude: -112.8160018921,
		Timezone:  "America/Edmonton",
	},
	"CEQ4": {
		ICAO:      "CEQ4",
		Name:      "Del Bonita / Whetstone International Airport",
		City:      "Del Bonita",
		State:     "Montana",
		Country:   "CA",
		Elevation: 4335,
		Latitude:  48.9986000061,
		Longitude: -112.776000977,
		Timezone:  "America/Denver",
	},
	"CEQ5": {
		ICAO:      "CEQ5",
		Name:      "Grande Cache Airport",
		City:      "Grande Cache",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 4117,
		Latitude:  53.9169006348,
		Longitude: -118.8740005493,
		Timezone:  "America/Edmonton",
	},
	"CER2": {
		ICAO:      "CER2",
		Name:      "Castor Airport",
		City:      "Castor",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2704,
		Latitude:  52.2209249248,
		Longitude: -111.926887035,
		Timezone:  "America/Edmonton",
	},
	"CER3": {
		ICAO:      "CER3",
		Name:      "Drayton Valley Industrial Airport",
		City:      "Drayton Valley",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2776,
		Latitude:  53.2658004761,
		Longitude: -114.9599990845,
		Timezone:  "America/Edmonton",
	},
	"CER4": {
		ICAO:      "CER4",
		IATA:      "NML",
		Name:      "Fort McMurray / Mildred Lake Airport",
		City:      "Fort McMurray",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 1046,
		Latitude:  57.0555992126,
		Longitude: -111.573997498,
		Timezone:  "America/Edmonton",
	},
	"CES2": {
		ICAO:      "CES2",
		Name:      "St-Esprit",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 192,
		Latitude:  45.914399704,
		Longitude: -73.6723959446,
		Timezone:  "America/Toronto",
	},
	"CES4": {
		ICAO:      "CES4",
		Name:      "Westlock Airport",
		City:      "Westlock",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2214,
		Latitude:  54.14220047,
		Longitude: -113.7409973145,
		Timezone:  "America/Edmonton",
	},
	"CET2": {
		ICAO:      "CET2",
		Name:      "Conklin (Leismer) Airport",
		City:      "Conklin",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 1930,
		Latitude:  55.6953010559,
		Longitude: -111.2789993286,
		Timezone:  "America/Edmonton",
	},
	"CET4": {
		ICAO:      "CET4",
		Name:      "Fort Simpson Island Airport",
		City:      "Fort Simpson Island",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 405,
		Latitude:  61.8666992188,
		Longitude: -121.3659973145,
		Timezone:  "America/Inuvik",
	},
	"CET9": {
		ICAO:      "CET9",
		Name:      "Jean Marie River Airport",
		City:      "Jean Marie River",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 470,
		Latitude:  61.5167007446,
		Longitude: -120.6169967651,
		Timezone:  "America/Inuvik",
	},
	"CEU2": {
		ICAO:      "CEU2",
		Name:      "Beaverlodge Airport",
		City:      "Beaverlodge",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2289,
		Latitude:  55.1842002869,
		Longitude: -119.4479980469,
		Timezone:  "America/Edmonton",
	},
	"CEU9": {
		ICAO:      "CEU9",
		Name:      "Trout Lake Airport",
		City:      "Trout Lake",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 1635,
		Latitude:  60.4393997192,
		Longitude: -121.2369995117,
		Timezone:  "America/Inuvik",
	},
	"CEV2": {
		ICAO:      "CEV2",
		Name:      "Edra Airport",
		City:      "Edra",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2625,
		Latitude:  57.8499984741,
		Longitude: -113.25,
		Timezone:  "America/Edmonton",
	},
	"CEV3": {
		ICAO:      "CEV3",
		Name:      "Vegreville Airport",
		City:      "Vegreville",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2072,
		Latitude:  53.5144004822,
		Longitude: -112.0270004272,
		Timezone:  "America/Edmonton",
	},
	"CEV5": {
		ICAO:      "CEV5",
		Name:      "Mayerthorpe Airport",
		City:      "Mayerthorpe",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2432,
		Latitude:  53.9375,
		Longitude: -115.1790008545,
		Timezone:  "America/Edmonton",
	},
	"CEV7": {
		ICAO:      "CEV7",
		Name:      "Tofield Airport",
		City:      "Tofield",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2311,
		Latitude:  53.3711013794,
		Longitude: -112.6969985962,
		Timezone:  "America/Edmonton",
	},
	"CEV9": {
		ICAO:      "CEV9",
		Name:      "Snare River Airport",
		City:      "Snare River",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 700,
		Latitude:  63.4333000183,
		Longitude: -116.1829986572,
		Timezone:  "America/Yellowknife",
	},
	"CEW3": {
		ICAO:      "CEW3",
		IATA:      "ZSP",
		Name:      "St. Paul Airport",
		City:      "St. Paul",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2147,
		Latitude:  53.9933013916,
		Longitude: -111.379997253,
		Timezone:  "America/Edmonton",
	},
	"CEW5": {
		ICAO:      "CEW5",
		Name:      "Milk River Airport",
		City:      "Milk River",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3449,
		Latitude:  49.1333007813,
		Longitude: -112.0500030518,
		Timezone:  "America/Edmonton",
	},
	"CEX3": {
		ICAO:      "CEX3",
		Name:      "Wetaskiwin Regional Airport",
		City:      "Wetaskiwin",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2509,
		Latitude:  52.9650001526,
		Longitude: -113.411003113,
		Timezone:  "America/Edmonton",
	},
	"CEX4": {
		ICAO:      "CEX4",
		Name:      "Carmacks Airport",
		City:      "Carmacks",
		State:     "Yukon",
		Country:   "CA",
		Elevation: 1770,
		Latitude:  62.1081008911,
		Longitude: -136.1799926758,
		Timezone:  "America/Whitehorse",
	},
	"CEX9": {
		ICAO:      "CEX9",
		Name:      "Brant (Dixon Farm) Airport",
		City:      "Brant",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3342,
		Latitude:  50.4193992615,
		Longitude: -113.5410003662,
		Timezone:  "America/Edmonton",
	},
	"CEY3": {
		ICAO:      "CEY3",
		Name:      "Fort Macleod Airport",
		City:      "Fort Macleod",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3138,
		Latitude:  49.7000007629,
		Longitude: -113.4169998169,
		Timezone:  "America/Edmonton",
	},
	"CEZ2": {
		ICAO:      "CEZ2",
		Name:      "Chapman Lake Airport",
		City:      "Chapman Lake",
		State:     "Yukon",
		Country:   "CA",
		Elevation: 3100,
		Latitude:  64.9000015259,
		Longitude: -138.266998291,
		Timezone:  "America/Dawson",
	},
	"CEZ3": {
		ICAO:      "CEZ3",
		Name:      "Edmonton / Cooking Lake Airport",
		City:      "Edmonton",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2437,
		Latitude:  53.4275016785,
		Longitude: -113.115997314,
		Timezone:  "America/Edmonton",
	},
	"CEZ4": {
		ICAO:      "CEZ4",
		Name:      "Fort Vermilion Airport",
		City:      "Fort Vermilion",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 836,
		Latitude:  58.4042015076,
		Longitude: -115.9509963989,
		Timezone:  "America/Edmonton",
	},
	"CFA4": {
		ICAO:      "CFA4",
		Name:      "Carcross Airport",
		City:      "Carcross",
		State:     "Yukon",
		Country:   "CA",
		Elevation: 2161,
		Latitude:  60.1741981506,
		Longitude: -134.6979980469,
		Timezone:  "America/Whitehorse",
	},
	"CFA5": {
		ICAO:      "CFA5",
		Name:      "Grande Airport",
		City:      "Grande",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 1750,
		Latitude:  56.2999992371,
		Longitude: -112.233001709,
		Timezone:  "America/Edmonton",
	},
	"CFA7": {
		ICAO:      "CFA7",
		Name:      "Taltheilei Narrows Airport",
		City:      "Taltheilei Narrows",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 570,
		Latitude:  62.5980987549,
		Longitude: -111.5429992676,
		Timezone:  "America/Yellowknife",
	},
	"CFB3": {
		ICAO:      "CFB3",
		Name:      "Hespero Airport",
		City:      "Hespero",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3175,
		Latitude:  52.2999992371,
		Longitude: -114.4639968872,
		Timezone:  "America/Edmonton",
	},
	"CFB4": {
		ICAO:      "CFB4",
		Name:      "Trout Lake Airport",
		City:      "Trout Lake",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2290,
		Latitude:  56.5,
		Longitude: -114.5500030518,
		Timezone:  "America/Edmonton",
	},
	"CFB5": {
		ICAO:      "CFB5",
		Name:      "Namur Lake Airport",
		City:      "Namur Lake",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2560,
		Latitude:  57.3833007813,
		Longitude: -112.8000030518,
		Timezone:  "America/Edmonton",
	},
	"CFB6": {
		ICAO:      "CFB6",
		Name:      "Edmonton / Josephburg Airport",
		City:      "Edmonton",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2069,
		Latitude:  53.727998032,
		Longitude: -113.086566925,
		Timezone:  "America/Edmonton",
	},
	"CFB7": {
		ICAO:      "CFB7",
		Name:      "Steen River Airport",
		City:      "Steen River",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 996,
		Latitude:  59.6333007813,
		Longitude: -117.1669998169,
		Timezone:  "America/Edmonton",
	},
	"CFC4": {
		ICAO:      "CFC4",
		Name:      "Macmillan Pass Airport",
		City:      "Macmillan Pass",
		State:     "Yukon",
		Country:   "CA",
		Elevation: 3810,
		Latitude:  63.181098938,
		Longitude: -130.2019958496,
		Timezone:  "America/Whitehorse",
	},
	"CFC6": {
		ICAO:      "CFC6",
		Name:      "Rockyford Airport",
		City:      "Rockyford",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2860,
		Latitude:  51.2667007446,
		Longitude: -113.1169967651,
		Timezone:  "America/Edmonton",
	},
	"CFC7": {
		ICAO:      "CFC7",
		Name:      "Rimbey Airport",
		City:      "Rimbey",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2963,
		Latitude:  52.6819000244,
		Longitude: -114.2379989624,
		Timezone:  "America/Edmonton",
	},
	"CFD4": {
		ICAO:      "CFD4",
		Name:      "Foremost Airport",
		City:      "Foremost",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2909,
		Latitude:  49.4831008911,
		Longitude: -111.4940032959,
		Timezone:  "America/Edmonton",
	},
	"CFD5": {
		ICAO:      "CFD5",
		Name:      "Grimshaw Airport",
		City:      "Grimshaw",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2050,
		Latitude:  56.19611,
		Longitude: -117.625278,
		Timezone:  "America/Edmonton",
	},
	"CFD9": {
		ICAO:      "CFD9",
		Name:      "Bjorgum Farm Airport",
		City:      "Bjorgum Farm",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2415,
		Latitude:  53.0833015442,
		Longitude: -112.8030014038,
		Timezone:  "America/Edmonton",
	},
	"CFF2": {
		ICAO:      "CFF2",
		Name:      "Christina Basin Airport",
		City:      "Christina Basin",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2365,
		Latitude:  55.5900001526,
		Longitude: -111.8229980469,
		Timezone:  "America/Edmonton",
	},
	"CFF3": {
		ICAO:      "CFF3",
		Name:      "Jean Lake Airport",
		City:      "Jean Lake",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2370,
		Latitude:  57.4832992554,
		Longitude: -113.8830032349,
		Timezone:  "America/Edmonton",
	},
	"CFF4": {
		ICAO:      "CFF4",
		IATA:      "DAS",
		Name:      "Great Bear Lake Airport",
		City:      "Great Bear Lake",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 562,
		Latitude:  66.7031021118,
		Longitude: -119.707000732,
		Timezone:  "America/Yellowknife",
	},
	"CFF9": {
		ICAO:      "CFF9",
		Name:      "Marek Farms Airport",
		City:      "Marek Farms",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2449,
		Latitude:  53.0313987732,
		Longitude: -112.7779998779,
		Timezone:  "America/Edmonton",
	},
	"CFG3": {
		ICAO:      "CFG3",
		Name:      "Consort Airport",
		City:      "Consort",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2499,
		Latitude:  52.0167007446,
		Longitude: -110.75,
		Timezone:  "America/Edmonton",
	},
	"CFG4": {
		ICAO:      "CFG4",
		Name:      "Debolt Airport",
		City:      "Debolt",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2120,
		Latitude:  55.2332992554,
		Longitude: -118.0329971313,
		Timezone:  "America/Edmonton",
	},
	"CFG5": {
		ICAO:      "CFG5",
		Name:      "John D'Or Prairie Airport",
		City:      "John D'Or Prairie",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 950,
		Latitude:  58.4914016724,
		Longitude: -115.1380004883,
		Timezone:  "America/Edmonton",
	},
	"CFG6": {
		ICAO:      "CFG6",
		IATA:      "YFI",
		Name:      "Fort Mackay / Firebag",
		City:      "Suncor Energy Site",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 1762,
		Latitude:  57.2758333344,
		Longitude: -110.976666667,
		Timezone:  "America/Edmonton",
	},
	"CFG7": {
		ICAO:      "CFG7",
		Name:      "Steen Tower Airport",
		City:      "Steen Tower",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2320,
		Latitude:  59.6333007813,
		Longitude: -117.7829971313,
		Timezone:  "America/Edmonton",
	},
	"CFH4": {
		ICAO:      "CFH4",
		Name:      "Fox Harbour Airport",
		City:      "Fox Harbour",
		State:     "Nova Scotia",
		Country:   "CA",
		Elevation: 62,
		Latitude:  45.8699989319,
		Longitude: -63.461101532,
		Timezone:  "America/Halifax",
	},
	"CFH8": {
		ICAO:      "CFH8",
		Name:      "Warburg / Zajes Airport",
		City:      "Warburg",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2530,
		Latitude:  53.2177775261,
		Longitude: -114.336605072,
		Timezone:  "America/Edmonton",
	},
	"CFK2": {
		ICAO:      "CFK2",
		Name:      "Bashaw Airport",
		City:      "Bashaw",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2610,
		Latitude:  52.5574989319,
		Longitude: -112.9649963379,
		Timezone:  "America/Edmonton",
	},
	"CFK3": {
		ICAO:      "CFK3",
		Name:      "Fontas Airport",
		City:      "Fontas",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2950,
		Latitude:  57.7999992371,
		Longitude: -119.4499969482,
		Timezone:  "America/Edmonton",
	},
	"CFK4": {
		ICAO:      "CFK4",
		Name:      "Calling Lake Airport",
		City:      "Calling Lake",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2092,
		Latitude:  55.3166999817,
		Longitude: -113.25,
		Timezone:  "America/Edmonton",
	},
	"CFK6": {
		ICAO:      "CFK6",
		Name:      "Olds (Netook) Airport",
		City:      "Olds",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3330,
		Latitude:  51.8499984741,
		Longitude: -114.0670013428,
		Timezone:  "America/Edmonton",
	},
	"CFL2": {
		ICAO:      "CFL2",
		Name:      "Empress McNeill Spectra Energy Airport",
		City:      "Empress",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2395,
		Latitude:  50.6824989319,
		Longitude: -110.041999817,
		Timezone:  "America/Edmonton",
	},
	"CFL9": {
		ICAO:      "CFL9",
		Name:      "Johnson Lake Airport",
		City:      "Johnson Lake",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2000,
		Latitude:  57.5666999817,
		Longitude: -110.3170013428,
		Timezone:  "America/Edmonton",
	},
	"CFM2": {
		ICAO:      "CFM2",
		Name:      "Birch Mountain Airport",
		City:      "Birch Mountain",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2600,
		Latitude:  57.7000007629,
		Longitude: -111.8330001831,
		Timezone:  "America/Edmonton",
	},
	"CFM4": {
		ICAO:      "CFM4",
		Name:      "Donnelly Airport",
		City:      "Donnelly",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 1949,
		Latitude:  55.709400177,
		Longitude: -117.09400177,
		Timezone:  "America/Edmonton",
	},
	"CFM5": {
		ICAO:      "CFM5",
		Name:      "Hamburg Airport",
		City:      "Apache",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2540,
		Latitude:  57.3552844682,
		Longitude: -119.765310287,
		Timezone:  "America/Edmonton",
	},
	"CFM6": {
		ICAO:      "CFM6",
		Name:      "Teepee Airport",
		City:      "Teepee",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2565,
		Latitude:  56.459400177,
		Longitude: -114.1190032959,
		Timezone:  "America/Edmonton",
	},
	"CFM7": {
		ICAO:      "CFM7",
		Name:      "Boyle Airport",
		City:      "Boyle",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2154,
		Latitude:  54.573299408,
		Longitude: -112.8199996948,
		Timezone:  "America/Edmonton",
	},
	"CFM8": {
		ICAO:      "CFM8",
		Name:      "Fort Macleod (Alcock Farm) Airport",
		City:      "Fort Macleod",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3340,
		Latitude:  49.5833015442,
		Longitude: -113.3000030518,
		Timezone:  "America/Edmonton",
	},
	"CFN5": {
		ICAO:      "CFN5",
		Name:      "La Crete Airport",
		City:      "La Crete",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 1046,
		Latitude:  58.1728751982,
		Longitude: -116.337919235,
		Timezone:  "America/Edmonton",
	},
	"CFN7": {
		ICAO:      "CFN7",
		Name:      "Sundre Airport",
		City:      "Sundre",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3656,
		Latitude:  51.7742004395,
		Longitude: -114.6770019531,
		Timezone:  "America/Edmonton",
	},
	"CFP4": {
		ICAO:      "CFP4",
		Name:      "Mcquesten Airport",
		City:      "Mcquesten",
		State:     "Yukon",
		Country:   "CA",
		Elevation: 1500,
		Latitude:  63.5999984741,
		Longitude: -137.5670013428,
		Timezone:  "America/Whitehorse",
	},
	"CFP5": {
		ICAO:      "CFP5",
		Name:      "Glendon Airport",
		City:      "Glendon",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 1835,
		Latitude:  54.2708015442,
		Longitude: -111.0670013428,
		Timezone:  "America/Edmonton",
	},
	"CFP6": {
		ICAO:      "CFP6",
		Name:      "La Biche River Airport",
		City:      "La Biche River",
		State:     "Yukon",
		Country:   "CA",
		Elevation: 1356,
		Latitude:  60.1291999817,
		Longitude: -124.0490036011,
		Timezone:  "America/Inuvik",
	},
	"CFP7": {
		ICAO:      "CFP7",
		Name:      "Wainwright Field 21 Airport",
		City:      "Wainwright",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2260,
		Latitude:  52.8306007385,
		Longitude: -111.100997925,
		Timezone:  "America/Edmonton",
	},
	"CFP8": {
		ICAO:      "CFP8",
		Name:      "Whitehorse / Cousins Airport",
		City:      "Whitehorse",
		State:     "Yukon",
		Country:   "CA",
		Elevation: 2200,
		Latitude:  60.8117343999,
		Longitude: -135.182132721,
		Timezone:  "America/Whitehorse",
	},
	"CFQ3": {
		ICAO:      "CFQ3",
		Name:      "Milk River (Madge) Airport",
		City:      "Milk River",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3400,
		Latitude:  49.1446990967,
		Longitude: -112.0839996338,
		Timezone:  "America/Edmonton",
	},
	"CFQ4": {
		ICAO:      "CFQ4",
		Name:      "Cheadle Airport",
		City:      "Cheadle",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3300,
		Latitude:  51.0574989319,
		Longitude: -113.6240005493,
		Timezone:  "America/Edmonton",
	},
	"CFQ5": {
		ICAO:      "CFQ5",
		Name:      "Silver City Airport",
		City:      "Silver City",
		State:     "Yukon",
		Country:   "CA",
		Elevation: 2570,
		Latitude:  61.0289001465,
		Longitude: -138.4080047607,
		Timezone:  "America/Dawson",
	},
	"CFQ6": {
		ICAO:      "CFQ6",
		Name:      "Pelly Crossing Airport",
		City:      "Pelly Crossing",
		State:     "Yukon",
		Country:   "CA",
		Elevation: 1870,
		Latitude:  62.8372001648,
		Longitude: -136.5350036621,
		Timezone:  "America/Whitehorse",
	},
	"CFQ7": {
		ICAO:      "CFQ7",
		Name:      "Edmonton / Gartner Airport",
		City:      "Edmonton",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2390,
		Latitude:  53.2817001343,
		Longitude: -113.455001831,
		Timezone:  "America/Edmonton",
	},
	"CFR2": {
		ICAO:      "CFR2",
		Name:      "Bawlf (Blackwells) Airport",
		City:      "Bawlf",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2325,
		Latitude:  52.9000015259,
		Longitude: -112.5500030518,
		Timezone:  "America/Edmonton",
	},
	"CFR5": {
		ICAO:      "CFR5",
		Name:      "Alban Airport",
		City:      "French River",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 700,
		Latitude:  46.0896987915,
		Longitude: -80.6044006348,
		Timezone:  "America/Toronto",
	},
	"CFR7": {
		ICAO:      "CFR7",
		Name:      "Red Deer Forestry Airport",
		City:      "Red Deer Forestry",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 4646,
		Latitude:  51.6514015198,
		Longitude: -115.2389984131,
		Timezone:  "America/Edmonton",
	},
	"CFS4": {
		ICAO:      "CFS4",
		Name:      "Ogilvie River Airport",
		City:      "Ogilvie River",
		State:     "Yukon",
		Country:   "CA",
		Elevation: 1640,
		Latitude:  65.6667022705,
		Longitude: -138.1170043945,
		Timezone:  "America/Dawson",
	},
	"CFS5": {
		ICAO:      "CFS5",
		Name:      "Spirit River Airport",
		City:      "Spirit River",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2044,
		Latitude:  55.7835998535,
		Longitude: -118.8399963379,
		Timezone:  "America/Edmonton",
	},
	"CFS6": {
		ICAO:      "CFS6",
		Name:      "Loon River Airport",
		City:      "Loon River",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 1550,
		Latitude:  57.1418991089,
		Longitude: -115.0749969482,
		Timezone:  "America/Edmonton",
	},
	"CFS7": {
		ICAO:      "CFS7",
		Name:      "Twin Creeks Airport",
		City:      "Twin Creeks",
		State:     "Yukon",
		Country:   "CA",
		Elevation: 2913,
		Latitude:  62.6194000244,
		Longitude: -131.279006958,
		Timezone:  "America/Whitehorse",
	},
	"CFS8": {
		ICAO:      "CFS8",
		Name:      "Clearwater River Airport",
		City:      "Clearwater River",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 4100,
		Latitude:  51.9880981445,
		Longitude: -115.2279968262,
		Timezone:  "America/Edmonton",
	},
	"CFT2": {
		ICAO:      "CFT2",
		Name:      "Wilderman Farm Airport",
		City:      "Blackie",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3350,
		Latitude:  50.552434748,
		Longitude: -113.583183289,
		Timezone:  "America/Edmonton",
	},
	"CFT3": {
		ICAO:      "CFT3",
		Name:      "Finlayson Lake Airport",
		City:      "Finlayson Lake",
		State:     "Yukon",
		Country:   "CA",
		Elevation: 3094,
		Latitude:  61.6913986206,
		Longitude: -130.774002075,
		Timezone:  "America/Whitehorse",
	},
	"CFT5": {
		ICAO:      "CFT5",
		Name:      "Hyland Airport",
		City:      "Hyland",
		State:     "Yukon",
		Country:   "CA",
		Elevation: 3000,
		Latitude:  61.5238990784,
		Longitude: -128.2689971924,
		Timezone:  "America/Whitehorse",
	},
	"CFT8": {
		ICAO:      "CFT8",
		Name:      "Pelican Airport",
		City:      "Pelican",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2059,
		Latitude:  56.1608009338,
		Longitude: -113.4739990234,
		Timezone:  "America/Edmonton",
	},
	"CFT9": {
		ICAO:      "CFT9",
		Name:      "Zama Lake Airport",
		City:      "Zama Lake",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 1242,
		Latitude:  59.0638999939,
		Longitude: -118.8899993896,
		Timezone:  "America/Edmonton",
	},
	"CFU3": {
		ICAO:      "CFU3",
		Name:      "Chipman Airport",
		City:      "Chipman",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2195,
		Latitude:  53.7167015076,
		Longitude: -112.6330032349,
		Timezone:  "America/Edmonton",
	},
	"CFU4": {
		ICAO:      "CFU4",
		Name:      "Garden River Airport",
		City:      "Garden River",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 790,
		Latitude:  58.7139015198,
		Longitude: -113.8759994507,
		Timezone:  "America/Edmonton",
	},
	"CFU8": {
		ICAO:      "CFU8",
		Name:      "Irma Airport",
		City:      "Irma",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2230,
		Latitude:  52.9122009277,
		Longitude: -111.1740036011,
		Timezone:  "America/Edmonton",
	},
	"CFV2": {
		ICAO:      "CFV2",
		Name:      "Beiseker Airport",
		City:      "Beiseker",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3036,
		Latitude:  51.3932991028,
		Longitude: -113.4720001221,
		Timezone:  "America/Edmonton",
	},
	"CFV3": {
		ICAO:      "CFV3",
		Name:      "Mobil Bistcho Airport",
		City:      "Mobil Bistcho",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2000,
		Latitude:  59.4747326918,
		Longitude: -119.006137848,
		Timezone:  "America/Edmonton",
	},
	"CFV6": {
		ICAO:      "CFV6",
		Name:      "Margaret Lake Airport",
		City:      "Margaret Lake",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2750,
		Latitude:  58.9458023034,
		Longitude: -115.256538391,
		Timezone:  "America/Edmonton",
	},
	"CFW2": {
		ICAO:      "CFW2",
		Name:      "Gordon Lake Airport",
		City:      "Gordon Lake",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 1600,
		Latitude:  56.6166992188,
		Longitude: -110.5,
		Timezone:  "America/Edmonton",
	},
	"CFW4": {
		ICAO:      "CFW4",
		Name:      "Muskeg Tower Airport",
		City:      "Muskeg Tower",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2020,
		Latitude:  57.1375007629,
		Longitude: -110.8929977417,
		Timezone:  "America/Edmonton",
	},
	"CFW5": {
		ICAO:      "CFW5",
		Name:      "Taltson River Airport",
		City:      "Taltson River",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 875,
		Latitude:  60.3833007813,
		Longitude: -111.3499984741,
		Timezone:  "America/Yellowknife",
	},
	"CFX2": {
		ICAO:      "CFX2",
		Name:      "Calgary / Okotoks Air Park",
		City:      "Calgary",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3601,
		Latitude:  50.7352981567,
		Longitude: -113.934997559,
		Timezone:  "America/Edmonton",
	},
	"CFX3": {
		ICAO:      "CFX3",
		Name:      "Doig Airport",
		City:      "Doig",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3500,
		Latitude:  56.9500007629,
		Longitude: -119.516998291,
		Timezone:  "America/Edmonton",
	},
	"CFX4": {
		ICAO:      "CFX4",
		Name:      "Manning Airport",
		City:      "Manning",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 1612,
		Latitude:  56.9508018494,
		Longitude: -117.6439971924,
		Timezone:  "America/Edmonton",
	},
	"CFX6": {
		ICAO:      "CFX6",
		Name:      "Vulcan Airport",
		City:      "Vulcan",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3437,
		Latitude:  50.4047012329,
		Longitude: -113.2829971313,
		Timezone:  "America/Edmonton",
	},
	"CFX8": {
		ICAO:      "CFX8",
		Name:      "Chestermere (Kirkby Field) Airport",
		City:      "Chestermere",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3339,
		Latitude:  51.0416984558,
		Longitude: -113.7519989014,
		Timezone:  "America/Edmonton",
	},
	"CFY2": {
		ICAO:      "CFY2",
		Name:      "Grist Lake Airport",
		City:      "Grist Lake",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2000,
		Latitude:  55.4000015259,
		Longitude: -110.483001709,
		Timezone:  "America/Edmonton",
	},
	"CFY4": {
		ICAO:      "CFY4",
		Name:      "Winters Aire Park Airport",
		City:      "Indus",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3370,
		Latitude:  50.9048190147,
		Longitude: -113.796215057,
		Timezone:  "America/Edmonton",
	},
	"CFY5": {
		ICAO:      "CFY5",
		Name:      "Pine Lake",
		City:      "Pine Lake",
		State:     "Yukon",
		Country:   "CA",
		Elevation: 3250,
		Latitude:  60.103099823,
		Longitude: -130.9340057373,
		Timezone:  "America/Whitehorse",
	},
	"CFY6": {
		ICAO:      "CFY6",
		Name:      "Turner Valley Bar N Ranch Airport",
		City:      "Turner Valley Bar N Ranch",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 4210,
		Latitude:  50.6543998718,
		Longitude: -114.34400177,
		Timezone:  "America/Edmonton",
	},
	"CFZ3": {
		ICAO:      "CFZ3",
		Name:      "Schlenker Airport",
		City:      "Medicine Hat",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2365,
		Latitude:  49.9667015076,
		Longitude: -110.7170028687,
		Timezone:  "America/Edmonton",
	},
	"CFZ5": {
		ICAO:      "CFZ5",
		Name:      "Sundre / Goodwins Farm Airport",
		City:      "Sundre",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3700,
		Latitude:  51.7398057147,
		Longitude: -114.662600756,
		Timezone:  "America/Edmonton",
	},
	"CGB2": {
		ICAO:      "CGB2",
		Name:      "Bishell's Airport",
		City:      "Carstairs",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3400,
		Latitude:  51.5828018188,
		Longitude: -114.0520019531,
		Timezone:  "America/Edmonton",
	},
	"CGF2": {
		ICAO:      "CGF2",
		Name:      "Edmonton / Goyer Field",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2400,
		Latitude:  53.5762892197,
		Longitude: -112.982025146,
		Timezone:  "America/Edmonton",
	},
	"CGK2": {
		ICAO:      "CGK2",
		Name:      "Gahcho Kue",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 1371,
		Latitude:  63.4330558777,
		Longitude: -109.19972229,
		Timezone:  "America/Yellowknife",
	},
	"CGL2": {
		ICAO:      "CGL2",
		Name:      "Harrow Airport",
		City:      "Harrow",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 610,
		Latitude:  42.0593986511,
		Longitude: -82.8407974243,
		Timezone:  "America/Toronto",
	},
	"CGL4": {
		ICAO:      "CGL4",
		Name:      "Eaglesham (South) Airport",
		City:      "Eaglesham",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 1920,
		Latitude:  55.6711006165,
		Longitude: -117.9359970093,
		Timezone:  "America/Edmonton",
	},
	"CGR3": {
		ICAO:      "CGR3",
		Name:      "George Lake Aerodrome",
		State:     "Nunavut",
		Country:   "CA",
		Elevation: 1150,
		Latitude:  65.928056,
		Longitude: -102.462222,
		Timezone:  "America/Cambridge_Bay",
	},
	"CGS2": {
		ICAO:      "CGS2",
		Name:      "Goose Lake Aerodrome",
		State:     "Nunavut",
		Country:   "CA",
		Elevation: 907,
		Latitude:  65.552778,
		Longitude: -106.433611,
		Timezone:  "America/Cambridge_Bay",
	},
	"CGV2": {
		ICAO:      "CGV2",
		Name:      "Grand Valley / Luther Field",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1602,
		Latitude:  43.976388889,
		Longitude: -80.38527778,
		Timezone:  "America/Toronto",
	},
	"CGV5": {
		ICAO:      "CGV5",
		Name:      "Grand Valley (Black Field)",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1580,
		Latitude:  43.8600915245,
		Longitude: -80.2886867523,
		Timezone:  "America/Toronto",
	},
	"CGV6": {
		ICAO:      "CGV6",
		Name:      "Grand Valley (Martin Field)",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1550,
		Latitude:  43.8748961118,
		Longitude: -80.2824425697,
		Timezone:  "America/Toronto",
	},
	"CHC5": {
		ICAO:      "CHC5",
		Name:      "Hayes Camp Aerodrome",
		State:     "Nunavut",
		Country:   "CA",
		Elevation: 780,
		Latitude:  66.655556,
		Longitude: -91.545,
		Timezone:  "America/Rankin_Inlet",
	},
	"CHF2": {
		ICAO:      "CHF2",
		Name:      "Ottawa / Manotick (Hope Field)",
		City:      "Manotick",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 315,
		Latitude:  45.1906013489,
		Longitude: -75.7086029053,
		Timezone:  "America/Toronto",
	},
	"CHG2": {
		ICAO:      "CHG2",
		Name:      "Harbour Grace Airport",
		City:      "Harbour Grace",
		State:     "Newfoundland and Labrador",
		Country:   "CA",
		Elevation: 325,
		Latitude:  47.6856002808,
		Longitude: -53.2538986206,
		Timezone:  "America/St_Johns",
	},
	"CHM2": {
		ICAO:      "CHM2",
		Name:      "Spiritwood / H & M Fast Farms",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2250,
		Latitude:  53.275,
		Longitude: -107.57027778,
		Timezone:  "America/Swift_Current",
	},
	"CHS3": {
		ICAO:      "CHS3",
		Name:      "Highgate (South)",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 740,
		Latitude:  42.4736111111,
		Longitude: -81.8219444444,
		Timezone:  "America/Toronto",
	},
	"CJA3": {
		ICAO:      "CJA3",
		Name:      "Morden Airport",
		City:      "Morden",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 950,
		Latitude:  49.2103004456,
		Longitude: -98.0591964722,
		Timezone:  "America/Winnipeg",
	},
	"CJA5": {
		ICAO:      "CJA5",
		Name:      "Nestor Falls Airport",
		City:      "Nestor Falls",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1195,
		Latitude:  49.1403007507,
		Longitude: -93.9169006348,
		Timezone:  "America/Rainy_River",
	},
	"CJA7": {
		ICAO:      "CJA7",
		Name:      "Arcola Airport",
		City:      "Arcola",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1985,
		Latitude:  49.6333007813,
		Longitude: -102.483001709,
		Timezone:  "America/Regina",
	},
	"CJB2": {
		ICAO:      "CJB2",
		Name:      "Friendship Field",
		City:      "Carman",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 882,
		Latitude:  49.4914016724,
		Longitude: -98.0186004639,
		Timezone:  "America/Winnipeg",
	},
	"CJB3": {
		ICAO:      "CJB3",
		Name:      "Steinbach Airport",
		City:      "Steinbach",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 850,
		Latitude:  49.5494003296,
		Longitude: -96.679397583,
		Timezone:  "America/Winnipeg",
	},
	"CJB5": {
		ICAO:      "CJB5",
		Name:      "Moosomin Airport",
		City:      "Moosomin",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1853,
		Latitude:  50.168598175,
		Longitude: -101.6439971924,
		Timezone:  "America/Regina",
	},
	"CJB6": {
		ICAO:      "CJB6",
		Name:      "Gods Lake Airport",
		City:      "Gods Lake",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 605,
		Latitude:  54.7792015076,
		Longitude: -93.7169036865,
		Timezone:  "America/Winnipeg",
	},
	"CJB8": {
		ICAO:      "CJB8",
		Name:      "Kyle Airport",
		City:      "Kyle",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2175,
		Latitude:  50.8333015442,
		Longitude: -108.0699996948,
		Timezone:  "America/Swift_Current",
	},
	"CJC2": {
		ICAO:      "CJC2",
		Name:      "Craik Airport",
		City:      "Craik",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1950,
		Latitude:  51.056098938,
		Longitude: -105.8379974365,
		Timezone:  "America/Regina",
	},
	"CJC3": {
		ICAO:      "CJC3",
		Name:      "Davidson Municipal Airport",
		City:      "Davidson",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2025,
		Latitude:  51.2477989197,
		Longitude: -105.9759979248,
		Timezone:  "America/Regina",
	},
	"CJC4": {
		ICAO:      "CJC4",
		Name:      "Central Butte Airport",
		City:      "Central Butte",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2030,
		Latitude:  50.801399231,
		Longitude: -106.4879989624,
		Timezone:  "America/Swift_Current",
	},
	"CJC5": {
		ICAO:      "CJC5",
		Name:      "Shaunavon Airport",
		City:      "Shaunavon",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 3028,
		Latitude:  49.6581001282,
		Longitude: -108.40599823,
		Timezone:  "America/Swift_Current",
	},
	"CJC6": {
		ICAO:      "CJC6",
		Name:      "Hafford Airport",
		City:      "Hafford",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1935,
		Latitude:  52.7322006226,
		Longitude: -107.3740005493,
		Timezone:  "America/Swift_Current",
	},
	"CJC8": {
		ICAO:      "CJC8",
		Name:      "Laurie River Airport",
		City:      "Laurie River",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 1200,
		Latitude:  56.2486000061,
		Longitude: -101.3040008545,
		Timezone:  "America/Winnipeg",
	},
	"CJD2": {
		ICAO:      "CJD2",
		Name:      "Cudworth Municipal Airport",
		City:      "Cudworth",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1878,
		Latitude:  52.4832992554,
		Longitude: -105.7170028687,
		Timezone:  "America/Regina",
	},
	"CJD3": {
		ICAO:      "CJD3",
		Name:      "Birch Hills Airport",
		City:      "Birch Hills",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1500,
		Latitude:  52.9907989502,
		Longitude: -105.4449996948,
		Timezone:  "America/Regina",
	},
	"CJD5": {
		ICAO:      "CJD5",
		Name:      "Leader Airport",
		City:      "Leader",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2201,
		Latitude:  50.8772010803,
		Longitude: -109.5009994507,
		Timezone:  "America/Swift_Current",
	},
	"CJE2": {
		ICAO:      "CJE2",
		Name:      "Dore Lake Airport",
		City:      "Dore Lake",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1565,
		Latitude:  54.6166992188,
		Longitude: -107.3830032349,
		Timezone:  "America/Regina",
	},
	"CJE3": {
		ICAO:      "CJE3",
		Name:      "Weyburn Airport",
		City:      "Weyburn",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1934,
		Latitude:  49.6974983215,
		Longitude: -103.8010025024,
		Timezone:  "America/Regina",
	},
	"CJE4": {
		ICAO:      "CJE4",
		Name:      "Snow Lake Airport",
		City:      "Snow Lake",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 993,
		Latitude:  54.8972015381,
		Longitude: -99.818901062,
		Timezone:  "America/Winnipeg",
	},
	"CJE5": {
		ICAO:      "CJE5",
		Name:      "Glaslyn Airport",
		City:      "Glaslyn",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2250,
		Latitude:  53.3772010803,
		Longitude: -108.3420028687,
		Timezone:  "America/Swift_Current",
	},
	"CJE6": {
		ICAO:      "CJE6",
		Name:      "Paradise Hill Airport",
		City:      "Paradise Hill",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1927,
		Latitude:  53.5353012085,
		Longitude: -109.4339981079,
		Timezone:  "America/Swift_Current",
	},
	"CJE7": {
		ICAO:      "CJE7",
		Name:      "Ashern Airport",
		City:      "Ashern",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 976,
		Latitude:  51.1585998535,
		Longitude: -98.3319015503,
		Timezone:  "America/Winnipeg",
	},
	"CJF3": {
		ICAO:      "CJF3",
		Name:      "Ile-a-la-Crosse Airport",
		City:      "Ile-a-la-Crosse",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1394,
		Latitude:  55.4897003174,
		Longitude: -107.9300003052,
		Timezone:  "America/Regina",
	},
	"CJF5": {
		ICAO:      "CJF5",
		Name:      "West Poplar Airport",
		City:      "West Poplar",
		State:     "Montana",
		Country:   "CA",
		Elevation: 2885,
		Latitude:  49.0031013489,
		Longitude: -106.3870010376,
		Timezone:  "America/Denver",
	},
	"CJF8": {
		ICAO:      "CJF8",
		Name:      "Biggar Airport",
		City:      "Biggar",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2130,
		Latitude:  52.0511016846,
		Longitude: -107.9879989624,
		Timezone:  "America/Swift_Current",
	},
	"CJG2": {
		ICAO:      "CJG2",
		Name:      "Eatonia (Elvie Smith) Municipal Airport",
		City:      "Eatonia",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2365,
		Latitude:  51.2188987732,
		Longitude: -109.3929977417,
		Timezone:  "America/Swift_Current",
	},
	"CJG4": {
		ICAO:      "CJG4",
		Name:      "Wrong Lake Airport",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 840,
		Latitude:  52.6154522074,
		Longitude: -96.184015274,
		Timezone:  "America/Winnipeg",
	},
	"CJH2": {
		ICAO:      "CJH2",
		Name:      "Gilbert Plains Airport",
		City:      "Gilbert Plains",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 1310,
		Latitude:  51.1333007813,
		Longitude: -100.5,
		Timezone:  "America/Winnipeg",
	},
	"CJH3": {
		ICAO:      "CJH3",
		Name:      "Maidstone Airport",
		City:      "Maidstone",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1975,
		Latitude:  53.0943984985,
		Longitude: -109.3249969482,
		Timezone:  "America/Edmonton",
	},
	"CJH4": {
		ICAO:      "CJH4",
		Name:      "Ferland Airport",
		City:      "Ferland",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2550,
		Latitude:  49.4449996948,
		Longitude: -106.9319992065,
		Timezone:  "America/Swift_Current",
	},
	"CJH8": {
		ICAO:      "CJH8",
		Name:      "Leask Airport",
		City:      "Leask",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1715,
		Latitude:  53.0167007446,
		Longitude: -106.75,
		Timezone:  "America/Swift_Current",
	},
	"CJJ2": {
		ICAO:      "CJJ2",
		Name:      "Glenboro Airport",
		City:      "Glenboro",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 1240,
		Latitude:  49.5499992371,
		Longitude: -99.3332977295,
		Timezone:  "America/Winnipeg",
	},
	"CJJ3": {
		ICAO:      "CJJ3",
		Name:      "Wildwood / Loche Mist Farms Airstrip",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2880,
		Latitude:  53.546529605,
		Longitude: -115.260658264,
		Timezone:  "America/Edmonton",
	},
	"CJJ4": {
		ICAO:      "CJJ4",
		Name:      "Deloraine Airport",
		City:      "Deloraine",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 1680,
		Latitude:  49.1506004333,
		Longitude: -100.5009994507,
		Timezone:  "America/Winnipeg",
	},
	"CJJ5": {
		ICAO:      "CJJ5",
		Name:      "Cabri Airport",
		City:      "Cabri",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2175,
		Latitude:  50.6194000244,
		Longitude: -108.466003418,
		Timezone:  "America/Swift_Current",
	},
	"CJJ8": {
		ICAO:      "CJJ8",
		Name:      "Macklin Airport",
		City:      "Macklin",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2280,
		Latitude:  52.3428001404,
		Longitude: -109.9189987183,
		Timezone:  "America/Swift_Current",
	},
	"CJK2": {
		ICAO:      "CJK2",
		Name:      "Gunisao Lake Airport",
		City:      "Gunisao Lake",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 886,
		Latitude:  53.5200004578,
		Longitude: -96.3711013794,
		Timezone:  "America/Winnipeg",
	},
	"CJK3": {
		ICAO:      "CJK3",
		Name:      "Beauval Airport",
		City:      "Beauval",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1436,
		Latitude:  55.1102981567,
		Longitude: -107.716003418,
		Timezone:  "America/Regina",
	},
	"CJK4": {
		ICAO:      "CJK4",
		Name:      "Esterhazy Airport",
		City:      "Esterhazy",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1700,
		Latitude:  50.6427993774,
		Longitude: -102.0999984741,
		Timezone:  "America/Regina",
	},
	"CJK5": {
		ICAO:      "CJK5",
		Name:      "Gull Lake Airport",
		City:      "Gull Lake",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2665,
		Latitude:  50.057800293,
		Longitude: -108.4899978638,
		Timezone:  "America/Swift_Current",
	},
	"CJK9": {
		ICAO:      "CJK9",
		Name:      "Preeceville Airport",
		City:      "Preeceville",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1750,
		Latitude:  51.9500007629,
		Longitude: -102.6500015259,
		Timezone:  "America/Regina",
	},
	"CJL2": {
		ICAO:      "CJL2",
		Name:      "Hatchet Lake Airport",
		City:      "Hatchet Lake",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1362,
		Latitude:  58.6624984741,
		Longitude: -103.5380020142,
		Timezone:  "America/Regina",
	},
	"CJL4": {
		ICAO:      "CJL4",
		Name:      "La Loche Airport",
		City:      "La Loche",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1501,
		Latitude:  56.4733009338,
		Longitude: -109.4039993286,
		Timezone:  "America/Regina",
	},
	"CJL5": {
		ICAO:      "CJL5",
		Name:      "Winnipeg / Lyncrest Airport",
		City:      "Winnipeg",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 775,
		Latitude:  49.8525009155,
		Longitude: -96.9736022949,
		Timezone:  "America/Winnipeg",
	},
	"CJL6": {
		ICAO:      "CJL6",
		Name:      "Altona Municipal Airport",
		City:      "Altona",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 806,
		Latitude:  49.0943984985,
		Longitude: -97.5338973999,
		Timezone:  "America/Winnipeg",
	},
	"CJL8": {
		ICAO:      "CJL8",
		Name:      "Kasba Lake Airport",
		City:      "Kasba Lake",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 1131,
		Latitude:  60.2919006348,
		Longitude: -102.5019989014,
		Timezone:  "America/Yellowknife",
	},
	"CJL9": {
		ICAO:      "CJL9",
		Name:      "Radisson Airport",
		City:      "Radisson",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1675,
		Latitude:  52.4631004333,
		Longitude: -107.3769989014,
		Timezone:  "America/Swift_Current",
	},
	"CJM2": {
		ICAO:      "CJM2",
		Name:      "Ituna Airport",
		City:      "Ituna",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2200,
		Latitude:  51.1460990906,
		Longitude: -103.4260025024,
		Timezone:  "America/Regina",
	},
	"CJM4": {
		ICAO:      "CJM4",
		Name:      "Gravelbourg Airport",
		City:      "Gravelbourg",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2296,
		Latitude:  49.8662071228,
		Longitude: -106.5710754395,
		Timezone:  "America/Swift_Current",
	},
	"CJM5": {
		ICAO:      "CJM5",
		Name:      "Frontier Airport",
		City:      "Frontier",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2960,
		Latitude:  49.1666984558,
		Longitude: -108.5670013428,
		Timezone:  "America/Swift_Current",
	},
	"CJM6": {
		ICAO:      "CJM6",
		Name:      "Arborfield Airport",
		City:      "Arborfield",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1183,
		Latitude:  53.10779953,
		Longitude: -103.6510009766,
		Timezone:  "America/Regina",
	},
	"CJN2": {
		ICAO:      "CJN2",
		Name:      "Kamsack Airport",
		City:      "Kamsack",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1510,
		Latitude:  51.5597000122,
		Longitude: -101.8789978027,
		Timezone:  "America/Regina",
	},
	"CJN4": {
		ICAO:      "CJN4",
		Name:      "Assiniboia Airport",
		City:      "Assiniboia",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2370,
		Latitude:  49.7346992493,
		Longitude: -105.9469985962,
		Timezone:  "America/Regina",
	},
	"CJN5": {
		ICAO:      "CJN5",
		Name:      "Corman Air Park",
		City:      "Saskatoon",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1690,
		Latitude:  52.0005989075,
		Longitude: -106.4629974365,
		Timezone:  "America/Regina",
	},
	"CJN7": {
		ICAO:      "CJN7",
		Name:      "Little Churchill River / Dunlop's Fly-In Lodge Airport",
		City:      "Little Churchill River",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 700,
		Latitude:  56.5800018311,
		Longitude: -96.2474975586,
		Timezone:  "America/Winnipeg",
	},
	"CJP2": {
		ICAO:      "CJP2",
		Name:      "Kerrobert Airport",
		City:      "Kerrobert",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2208,
		Latitude:  51.9268989563,
		Longitude: -109.1289978027,
		Timezone:  "America/Swift_Current",
	},
	"CJP6": {
		ICAO:      "CJP6",
		Name:      "Camsell Portage Airport",
		City:      "Camsell Portage",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 750,
		Latitude:  59.6100006104,
		Longitude: -109.266998291,
		Timezone:  "America/Regina",
	},
	"CJP7": {
		ICAO:      "CJP7",
		Name:      "Bird River (Lac Du Bonnet) Airport",
		City:      "Bird River",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 950,
		Latitude:  50.3967018127,
		Longitude: -95.7350006104,
		Timezone:  "America/Winnipeg",
	},
	"CJP9": {
		ICAO:      "CJP9",
		Name:      "Charlot River Airport",
		City:      "Charlot River",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 700,
		Latitude:  59.6016998291,
		Longitude: -109.1370010376,
		Timezone:  "America/Regina",
	},
	"CJQ2": {
		ICAO:      "CJQ2",
		Name:      "Lampman Airport",
		City:      "Lampman",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1950,
		Latitude:  49.3666992188,
		Longitude: -102.766998291,
		Timezone:  "America/Regina",
	},
	"CJQ3": {
		ICAO:      "CJQ3",
		Name:      "Carlyle Airport",
		City:      "Carlyle",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2074,
		Latitude:  49.6444015503,
		Longitude: -102.2870025635,
		Timezone:  "America/Regina",
	},
	"CJQ4": {
		ICAO:      "CJQ4",
		Name:      "Maple Creek Airport",
		City:      "Maple Creek",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2517,
		Latitude:  49.8958015442,
		Longitude: -109.4749984741,
		Timezone:  "America/Swift_Current",
	},
	"CJQ5": {
		ICAO:      "CJQ5",
		Name:      "Arnes Airport",
		City:      "Arnes",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 725,
		Latitude:  50.836101532,
		Longitude: -96.9571990967,
		Timezone:  "America/Winnipeg",
	},
	"CJQ6": {
		ICAO:      "CJQ6",
		Name:      "Tanquary Fiord Airport",
		City:      "Tanquary Fiord",
		State:     "Nunavut",
		Country:   "CA",
		Elevation: 50,
		Latitude:  81.4094009399,
		Longitude: -76.8816986084,
		Timezone:  "America/Iqaluit",
	},
	"CJQ9": {
		ICAO:      "CJQ9",
		Name:      "Big Sand Lake Airport",
		City:      "Big Sand Lake",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 1050,
		Latitude:  57.6194000244,
		Longitude: -99.8752975464,
		Timezone:  "America/Winnipeg",
	},
	"CJR2": {
		ICAO:      "CJR2",
		Name:      "Luseland Airport",
		City:      "Luseland",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2300,
		Latitude:  52.0699996948,
		Longitude: -109.3740005493,
		Timezone:  "America/Swift_Current",
	},
	"CJR3": {
		ICAO:      "CJR3",
		Name:      "The Pas / Grace Lake Airport",
		City:      "The Pas",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 873,
		Latitude:  53.8264007568,
		Longitude: -101.205001831,
		Timezone:  "America/Winnipeg",
	},
	"CJR4": {
		ICAO:      "CJR4",
		Name:      "Eston Airport",
		City:      "Eston",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2225,
		Latitude:  51.1441993713,
		Longitude: -108.7630004883,
		Timezone:  "America/Swift_Current",
	},
	"CJR5": {
		ICAO:      "CJR5",
		Name:      "Gladstone Airport",
		City:      "Gladstone",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 900,
		Latitude:  50.1636009216,
		Longitude: -98.9424972534,
		Timezone:  "America/Winnipeg",
	},
	"CJR7": {
		ICAO:      "CJR7",
		Name:      "Canora Airport",
		City:      "Canora",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1603,
		Latitude:  51.6288986206,
		Longitude: -102.4499969482,
		Timezone:  "America/Regina",
	},
	"CJR8": {
		ICAO:      "CJR8",
		Name:      "Mccreary Airport",
		City:      "Mccreary",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 1000,
		Latitude:  50.7647018433,
		Longitude: -99.4966964722,
		Timezone:  "America/Winnipeg",
	},
	"CJR9": {
		ICAO:      "CJR9",
		Name:      "Naicam Airport",
		City:      "Naicam",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1780,
		Latitude:  52.4113998413,
		Longitude: -104.4840011597,
		Timezone:  "America/Regina",
	},
	"CJS2": {
		ICAO:      "CJS2",
		Name:      "Malcolm Island Airport",
		City:      "Malcolm Island",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1120,
		Latitude:  56.9494018555,
		Longitude: -102.2389984131,
		Timezone:  "America/Regina",
	},
	"CJS3": {
		ICAO:      "CJS3",
		Name:      "Cluff Lake Airport",
		City:      "Cluff Lake",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1112,
		Latitude:  58.3911018372,
		Longitude: -109.5159988403,
		Timezone:  "America/Regina",
	},
	"CJS4": {
		ICAO:      "CJS4",
		Name:      "Moose Jaw Municipal Airport",
		City:      "Moose Jaw",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1896,
		Latitude:  50.4342002869,
		Longitude: -105.3870010376,
		Timezone:  "America/Regina",
	},
	"CJS5": {
		ICAO:      "CJS5",
		Name:      "Killarney Municipal Airport",
		City:      "Killarney",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 1680,
		Latitude:  49.1516990662,
		Longitude: -99.6902999878,
		Timezone:  "America/Winnipeg",
	},
	"CJS7": {
		ICAO:      "CJS7",
		Name:      "Carman (South) Airport",
		City:      "Carman",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 870,
		Latitude:  49.4803009033,
		Longitude: -98.0149993896,
		Timezone:  "America/Winnipeg",
	},
	"CJT2": {
		ICAO:      "CJT2",
		Name:      "Matheson Island Airport",
		City:      "Matheson Island",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 725,
		Latitude:  51.7322006226,
		Longitude: -96.9344024658,
		Timezone:  "America/Winnipeg",
	},
	"CJT3": {
		ICAO:      "CJT3",
		Name:      "Knee Lake Airport",
		City:      "Knee Lake",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 625,
		Latitude:  54.9152984619,
		Longitude: -94.7981033325,
		Timezone:  "America/Winnipeg",
	},
	"CJT4": {
		ICAO:      "CJT4",
		Name:      "Cumberland House Airport",
		City:      "Cumberland House",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 877,
		Latitude:  53.9561004639,
		Longitude: -102.297996521,
		Timezone:  "America/Regina",
	},
	"CJT5": {
		ICAO:      "CJT5",
		Name:      "Melita Airport",
		City:      "Melita",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 1475,
		Latitude:  49.2616996765,
		Longitude: -101.013999939,
		Timezone:  "America/Winnipeg",
	},
	"CJT8": {
		ICAO:      "CJT8",
		Name:      "Homewood Airport",
		City:      "Homewood",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 815,
		Latitude:  49.5092010498,
		Longitude: -97.8506011963,
		Timezone:  "America/Winnipeg",
	},
	"CJT9": {
		ICAO:      "CJT9",
		Name:      "Leoville Airport",
		City:      "Leoville",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1960,
		Latitude:  53.6206016541,
		Longitude: -107.6090011597,
		Timezone:  "America/Swift_Current",
	},
	"CJU3": {
		ICAO:      "CJU3",
		Name:      "Macdonald Airport",
		City:      "Macdonald",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 840,
		Latitude:  50.0964012146,
		Longitude: -98.5008010864,
		Timezone:  "America/Winnipeg",
	},
	"CJU4": {
		ICAO:      "CJU4",
		Name:      "Humboldt Airport",
		City:      "Humboldt",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1865,
		Latitude:  52.1761016846,
		Longitude: -105.1269989014,
		Timezone:  "America/Regina",
	},
	"CJU5": {
		ICAO:      "CJU5",
		Name:      "Minnedosa Airport",
		City:      "Minnedosa",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 1840,
		Latitude:  50.271900177,
		Longitude: -99.7630996704,
		Timezone:  "America/Winnipeg",
	},
	"CJU6": {
		ICAO:      "CJU6",
		Name:      "Arborg Airport",
		City:      "Arborg",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 765,
		Latitude:  50.9127998352,
		Longitude: -97.304397583,
		Timezone:  "America/Winnipeg",
	},
	"CJU7": {
		ICAO:      "CJU7",
		Name:      "Edam Airport",
		City:      "Edam",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1790,
		Latitude:  53.1850013733,
		Longitude: -108.7890014648,
		Timezone:  "America/Swift_Current",
	},
	"CJV2": {
		ICAO:      "CJV2",
		Name:      "Neilburg Airport",
		City:      "Neilburg",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2222,
		Latitude:  52.831401825,
		Longitude: -109.6399993896,
		Timezone:  "America/Edmonton",
	},
	"CJV4": {
		ICAO:      "CJV4",
		Name:      "Otter Lake Airport",
		City:      "Otter Lake",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1300,
		Latitude:  55.5811004639,
		Longitude: -104.7850036621,
		Timezone:  "America/Regina",
	},
	"CJV5": {
		ICAO:      "CJV5",
		Name:      "Neepawa Airport",
		City:      "Neepawa",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 1277,
		Latitude:  50.23279953,
		Longitude: -99.510597229,
		Timezone:  "America/Winnipeg",
	},
	"CJV7": {
		ICAO:      "CJV7",
		IATA:      "SUR",
		Name:      "Summer Beaver Airport",
		City:      "Summer Beaver",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 832,
		Latitude:  52.7085990906,
		Longitude: -88.5419006348,
		Timezone:  "America/Nipigon",
	},
	"CJV8": {
		ICAO:      "CJV8",
		Name:      "Grand Rapids Airport",
		City:      "Grand Rapids",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 860,
		Latitude:  53.1725006104,
		Longitude: -99.323097229,
		Timezone:  "America/Winnipeg",
	},
	"CJV9": {
		ICAO:      "CJV9",
		Name:      "Melville Municipal Airport",
		City:      "Melville",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1800,
		Latitude:  50.9413986206,
		Longitude: -102.7379989624,
		Timezone:  "America/Regina",
	},
	"CJW2": {
		ICAO:      "CJW2",
		Name:      "Oxbow Airport",
		City:      "Oxbow",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1850,
		Latitude:  49.2332992554,
		Longitude: -102.1500015259,
		Timezone:  "America/Regina",
	},
	"CJW3": {
		ICAO:      "CJW3",
		Name:      "Loon Lake Airport",
		City:      "Loon Lake",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1790,
		Latitude:  54.0182991028,
		Longitude: -109.1350021362,
		Timezone:  "America/Swift_Current",
	},
	"CJW4": {
		ICAO:      "CJW4",
		Name:      "Pelican Narrows Airport",
		City:      "Pelican Narrows",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1242,
		Latitude:  55.2868995667,
		Longitude: -102.75,
		Timezone:  "America/Regina",
	},
	"CJW5": {
		ICAO:      "CJW5",
		Name:      "Russell Airport",
		City:      "Russell",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 1825,
		Latitude:  50.7658004761,
		Longitude: -101.2949981689,
		Timezone:  "America/Winnipeg",
	},
	"CJW7": {
		ICAO:      "CJW7",
		Name:      "Cigar Lake Airport",
		City:      "Cigar Lake",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1562,
		Latitude:  58.0531005859,
		Longitude: -104.4840011597,
		Timezone:  "America/Regina",
	},
	"CJX4": {
		ICAO:      "CJX4",
		Name:      "Rosetown Airport",
		City:      "Rosetown",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1920,
		Latitude:  51.5643997192,
		Longitude: -107.9179992676,
		Timezone:  "America/Swift_Current",
	},
	"CJX5": {
		ICAO:      "CJX5",
		Name:      "Souris Glenwood Industrial Air Park",
		City:      "Souris Glenwood",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 1480,
		Latitude:  49.6333007813,
		Longitude: -100.1999969482,
		Timezone:  "America/Winnipeg",
	},
	"CJY3": {
		ICAO:      "CJY3",
		Name:      "Tisdale Airport",
		City:      "Tisdale",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1525,
		Latitude:  52.8367004395,
		Longitude: -104.0670013428,
		Timezone:  "America/Regina",
	},
	"CJY4": {
		ICAO:      "CJY4",
		Name:      "Sandy Bay Airport",
		City:      "Sandy Bay",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1001,
		Latitude:  55.5456008911,
		Longitude: -102.2720031738,
		Timezone:  "America/Regina",
	},
	"CJY5": {
		ICAO:      "CJY5",
		Name:      "Strathclair Airport",
		City:      "Strathclair",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 1875,
		Latitude:  50.3960990906,
		Longitude: -100.4250030518,
		Timezone:  "America/Winnipeg",
	},
	"CJZ2": {
		ICAO:      "CJZ2",
		Name:      "Portage La Prairie (North) Airport",
		City:      "Portage La Prairie",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 860,
		Latitude:  49.9925003052,
		Longitude: -98.3031005859,
		Timezone:  "America/Winnipeg",
	},
	"CJZ3": {
		ICAO:      "CJZ3",
		Name:      "Melfort Airport",
		City:      "Melfort",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1495,
		Latitude:  52.8666992188,
		Longitude: -104.6999969482,
		Timezone:  "America/Regina",
	},
	"CJZ4": {
		ICAO:      "CJZ4",
		Name:      "Shellbrook Airport",
		City:      "Shellbrook",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1640,
		Latitude:  53.2282981873,
		Longitude: -106.3629989624,
		Timezone:  "America/Swift_Current",
	},
	"CKA4": {
		ICAO:      "CKA4",
		Name:      "Zhoda Airport",
		City:      "Zhoda",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 950,
		Latitude:  49.2807998657,
		Longitude: -96.5010986328,
		Timezone:  "America/Winnipeg",
	},
	"CKA8": {
		ICAO:      "CKA8",
		Name:      "St. Francois Xavier Airport",
		City:      "St. Francois Xavier",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 798,
		Latitude:  49.9244003296,
		Longitude: -97.5488967896,
		Timezone:  "America/Winnipeg",
	},
	"CKA9": {
		ICAO:      "CKA9",
		Name:      "Southend Airport",
		City:      "Southend",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1120,
		Latitude:  56.3372001648,
		Longitude: -103.2929992676,
		Timezone:  "America/Regina",
	},
	"CKB2": {
		ICAO:      "CKB2",
		Name:      "Patuanak Airport",
		City:      "Patuanak",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1426,
		Latitude:  55.9000015259,
		Longitude: -107.7210006714,
		Timezone:  "America/Regina",
	},
	"CKB6": {
		ICAO:      "CKB6",
		IATA:      "YAX",
		Name:      "Wapekeka Airport",
		City:      "Angling Lake",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 712,
		Latitude:  53.8492012024,
		Longitude: -89.5793991089,
		Timezone:  "America/Rainy_River",
	},
	"CKB7": {
		ICAO:      "CKB7",
		Name:      "Roblin Airport",
		City:      "Roblin",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 1821,
		Latitude:  51.2344017029,
		Longitude: -101.3929977417,
		Timezone:  "America/Winnipeg",
	},
	"CKB8": {
		ICAO:      "CKB8",
		Name:      "Silver Falls Airport",
		City:      "Silver Falls",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 775,
		Latitude:  50.5,
		Longitude: -96.0980987549,
		Timezone:  "America/Winnipeg",
	},
	"CKC6": {
		ICAO:      "CKC6",
		Name:      "Lanigan Airport",
		City:      "Lanigan",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1750,
		Latitude:  51.8452987671,
		Longitude: -104.9919967651,
		Timezone:  "America/Regina",
	},
	"CKC7": {
		ICAO:      "CKC7",
		Name:      "Rockglen Airport",
		City:      "Rockglen",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2700,
		Latitude:  49.1666984558,
		Longitude: -105.9329986572,
		Timezone:  "America/Regina",
	},
	"CKC8": {
		ICAO:      "CKC8",
		Name:      "Somerset Airport",
		City:      "Somerset",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 1563,
		Latitude:  49.4002990723,
		Longitude: -98.6924972534,
		Timezone:  "America/Winnipeg",
	},
	"CKC9": {
		ICAO:      "CKC9",
		Name:      "Pangman Airport",
		City:      "Pangman",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2300,
		Latitude:  49.6464004517,
		Longitude: -104.6660003662,
		Timezone:  "America/Regina",
	},
	"CKD2": {
		ICAO:      "CKD2",
		Name:      "Porcupine Plain Airport",
		City:      "Porcupine Plain",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1635,
		Latitude:  52.6197013855,
		Longitude: -103.2480010986,
		Timezone:  "America/Regina",
	},
	"CKD5": {
		ICAO:      "CKD5",
		Name:      "Kipling Airport",
		City:      "Kipling",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2157,
		Latitude:  50.0992012024,
		Longitude: -102.6070022583,
		Timezone:  "America/Regina",
	},
	"CKD7": {
		ICAO:      "CKD7",
		Name:      "Roland (Graham Field) Airport",
		City:      "Roland",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 865,
		Latitude:  49.4082984924,
		Longitude: -97.9906005859,
		Timezone:  "America/Winnipeg",
	},
	"CKD8": {
		ICAO:      "CKD8",
		Name:      "Kirkfield / Balsam Lake Airfield",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 864,
		Latitude:  44.5399876549,
		Longitude: -78.8892602921,
		Timezone:  "America/Toronto",
	},
	"CKD9": {
		ICAO:      "CKD9",
		Name:      "Slate Falls Airport",
		City:      "Slate Falls",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1355,
		Latitude:  51.1300010681,
		Longitude: -91.6656036377,
		Timezone:  "America/Rainy_River",
	},
	"CKE2": {
		ICAO:      "CKE2",
		Name:      "Quill Lake Airport",
		City:      "Quill Lake",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1750,
		Latitude:  52.0666999817,
		Longitude: -104.266998291,
		Timezone:  "America/Regina",
	},
	"CKE8": {
		ICAO:      "CKE8",
		Name:      "Unity Airport",
		City:      "Unity",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2090,
		Latitude:  52.4458007813,
		Longitude: -109.1849975586,
		Timezone:  "America/Swift_Current",
	},
	"CKF2": {
		ICAO:      "CKF2",
		Name:      "Radville Airport",
		City:      "Radville",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2067,
		Latitude:  49.4606018066,
		Longitude: -104.2720031738,
		Timezone:  "America/Regina",
	},
	"CKF4": {
		ICAO:      "CKF4",
		Name:      "Goodsoil Airport",
		City:      "Goodsoil",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1750,
		Latitude:  54.4166984558,
		Longitude: -109.233001709,
		Timezone:  "America/Swift_Current",
	},
	"CKF6": {
		ICAO:      "CKF6",
		Name:      "Macgregor Airport",
		City:      "Macgregor",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 950,
		Latitude:  49.9667015076,
		Longitude: -98.75,
		Timezone:  "America/Winnipeg",
	},
	"CKF9": {
		ICAO:      "CKF9",
		Name:      "De Lesseps Lake Airport",
		City:      "De Lesseps Lake",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1312,
		Latitude:  50.7174987793,
		Longitude: -90.6836013794,
		Timezone:  "America/Rainy_River",
	},
	"CKG2": {
		ICAO:      "CKG2",
		Name:      "Riverton Airport",
		City:      "Riverton",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 725,
		Latitude:  50.971698761,
		Longitude: -97.011100769,
		Timezone:  "America/Winnipeg",
	},
	"CKG5": {
		ICAO:      "CKG5",
		Name:      "Manitou Airport",
		City:      "Manitou",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 1592,
		Latitude:  49.25,
		Longitude: -98.5333023071,
		Timezone:  "America/Winnipeg",
	},
	"CKG8": {
		ICAO:      "CKG8",
		Name:      "Kakabeka Falls Airport",
		City:      "Kakabeka Falls",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1000,
		Latitude:  48.4183006287,
		Longitude: -89.6018981934,
		Timezone:  "America/Thunder_Bay",
	},
	"CKH2": {
		ICAO:      "CKH2",
		Name:      "Rocanville Airport",
		City:      "Rocanville",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1585,
		Latitude:  50.4646987915,
		Longitude: -101.5540008545,
		Timezone:  "America/Regina",
	},
	"CKH3": {
		ICAO:      "CKH3",
		Name:      "Debden Airport",
		City:      "Debden",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1700,
		Latitude:  53.5318984985,
		Longitude: -106.8840026855,
		Timezone:  "America/Swift_Current",
	},
	"CKH7": {
		ICAO:      "CKH7",
		Name:      "Spiritwood Airport",
		City:      "Spiritwood",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1915,
		Latitude:  53.3633003235,
		Longitude: -107.547996521,
		Timezone:  "America/Swift_Current",
	},
	"CKH8": {
		ICAO:      "CKH8",
		Name:      "Lumsden (Colhoun) Airport",
		City:      "Lumsden",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1900,
		Latitude:  50.6674995422,
		Longitude: -104.7890014648,
		Timezone:  "America/Regina",
	},
	"CKJ2": {
		ICAO:      "CKJ2",
		Name:      "Rosenort Airport",
		City:      "Rosenort",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 779,
		Latitude:  49.4530982971,
		Longitude: -97.4225006104,
		Timezone:  "America/Winnipeg",
	},
	"CKJ4": {
		ICAO:      "CKJ4",
		Name:      "Hanley Airport",
		City:      "Hanley",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1925,
		Latitude:  51.6241989136,
		Longitude: -106.4420013428,
		Timezone:  "America/Regina",
	},
	"CKJ7": {
		ICAO:      "CKJ7",
		Name:      "Starbuck Airport",
		City:      "Starbuck",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 785,
		Latitude:  49.7167015076,
		Longitude: -97.6832962036,
		Timezone:  "America/Winnipeg",
	},
	"CKJ8": {
		ICAO:      "CKJ8",
		Name:      "Molson Lake Airport",
		City:      "Molson Lake",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 750,
		Latitude:  54.2580986023,
		Longitude: -97.011100769,
		Timezone:  "America/Winnipeg",
	},
	"CKJ9": {
		ICAO:      "CKJ9",
		Name:      "Lemberg Airport",
		City:      "Lemberg",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2050,
		Latitude:  50.7089004517,
		Longitude: -103.1969985962,
		Timezone:  "America/Regina",
	},
	"CKK2": {
		ICAO:      "CKK2",
		Name:      "St. Brieux Airport",
		City:      "St. Brieux",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1780,
		Latitude:  52.6500015259,
		Longitude: -104.8669967651,
		Timezone:  "America/Regina",
	},
	"CKK3": {
		ICAO:      "CKK3",
		Name:      "Scobey Border Station Airport",
		City:      "Coronach",
		State:     "Montana",
		Country:   "CA",
		Elevation: 2501,
		Latitude:  48.9994659424,
		Longitude: -105.3993377686,
		Timezone:  "America/Denver",
	},
	"CKK4": {
		ICAO:      "CKK4",
		Name:      "Estevan (South) Airport",
		City:      "Estevan",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1900,
		Latitude:  49.038898468,
		Longitude: -102.9759979248,
		Timezone:  "America/Chicago",
	},
	"CKK5": {
		ICAO:      "CKK5",
		Name:      "Eastend Airport",
		City:      "Eastend",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 3040,
		Latitude:  49.5499992371,
		Longitude: -108.8000030518,
		Timezone:  "America/Swift_Current",
	},
	"CKK7": {
		ICAO:      "CKK7",
		Name:      "Steinbach (South) Airport",
		City:      "Steinbach",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 888,
		Latitude:  49.4939002991,
		Longitude: -96.6988983154,
		Timezone:  "America/Winnipeg",
	},
	"CKL2": {
		ICAO:      "CKL2",
		Name:      "Selkirk Airport",
		City:      "Selkirk",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 747,
		Latitude:  50.1721992493,
		Longitude: -96.8722000122,
		Timezone:  "America/Winnipeg",
	},
	"CKL3": {
		ICAO:      "CKL3",
		IATA:      "WNN",
		Name:      "Wunnumin Lake Airport",
		City:      "Wunnumin Lake",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 819,
		Latitude:  52.893901825,
		Longitude: -89.2891998291,
		Timezone:  "America/Rainy_River",
	},
	"CKL5": {
		ICAO:      "CKL5",
		Name:      "Shoal Lake Airport",
		City:      "Shoal Lake",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 1836,
		Latitude:  50.4575004578,
		Longitude: -100.6090011597,
		Timezone:  "America/Winnipeg",
	},
	"CKL6": {
		ICAO:      "CKL6",
		Name:      "Little Bear Lake Airport",
		City:      "Little Bear Lake",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2100,
		Latitude:  54.2919006348,
		Longitude: -104.6709976196,
		Timezone:  "America/Regina",
	},
	"CKL9": {
		ICAO:      "CKL9",
		Name:      "Regina Beach Airport",
		City:      "Regina Beach",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1850,
		Latitude:  50.7527999878,
		Longitude: -104.9789962769,
		Timezone:  "America/Regina",
	},
	"CKM2": {
		ICAO:      "CKM2",
		Name:      "Sioux Narrows Airport",
		City:      "Sioux Narrows",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1100,
		Latitude:  49.3899993896,
		Longitude: -93.995300293,
		Timezone:  "America/Rainy_River",
	},
	"CKM4": {
		ICAO:      "CKM4",
		Name:      "Jan Lake Airport",
		City:      "Jan Lake",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1135,
		Latitude:  54.8307991028,
		Longitude: -102.7890014648,
		Timezone:  "America/Regina",
	},
	"CKM6": {
		ICAO:      "CKM6",
		Name:      "Easterville Airport",
		City:      "Easterville",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 850,
		Latitude:  53.1166992188,
		Longitude: -99.8000030518,
		Timezone:  "America/Winnipeg",
	},
	"CKM8": {
		ICAO:      "CKM8",
		Name:      "Opapimiskan Lake Airport",
		City:      "Opapimiskan Lake",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1023,
		Latitude:  52.6067008972,
		Longitude: -90.3768997192,
		Timezone:  "America/Rainy_River",
	},
	"CKN5": {
		ICAO:      "CKN5",
		Name:      "Fillmore Airport",
		City:      "Fillmore",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2000,
		Latitude:  49.8689002991,
		Longitude: -103.4449996948,
		Timezone:  "America/Regina",
	},
	"CKN8": {
		ICAO:      "CKN8",
		Name:      "Nekweaga Bay Airport",
		City:      "Nekweaga Bay",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1320,
		Latitude:  57.7425003052,
		Longitude: -103.9449996948,
		Timezone:  "America/Regina",
	},
	"CKP2": {
		ICAO:      "CKP2",
		Name:      "Spring Valley (North) Airport",
		City:      "Spring Valley",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2200,
		Latitude:  50.0600013733,
		Longitude: -105.4020004272,
		Timezone:  "America/Regina",
	},
	"CKQ2": {
		ICAO:      "CKQ2",
		Name:      "Squaw Rapids Airport",
		City:      "Squaw Rapids",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1040,
		Latitude:  53.6786003113,
		Longitude: -103.3499984741,
		Timezone:  "America/Regina",
	},
	"CKQ3": {
		ICAO:      "CKQ3",
		IATA:      "YNO",
		Name:      "North Spirit Lake Airport",
		City:      "North Spirit Lake",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1082,
		Latitude:  52.4900016785,
		Longitude: -92.9710998535,
		Timezone:  "America/Rainy_River",
	},
	"CKQ5": {
		ICAO:      "CKQ5",
		Name:      "Lucky Lake Airport",
		City:      "Lucky Lake",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2084,
		Latitude:  50.9942016602,
		Longitude: -107.1309967041,
		Timezone:  "America/Swift_Current",
	},
	"CKQ6": {
		ICAO:      "CKQ6",
		Name:      "Erickson Municipal Airport",
		City:      "Erickson",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 2114,
		Latitude:  50.4994010925,
		Longitude: -99.8977966309,
		Timezone:  "America/Winnipeg",
	},
	"CKQ7": {
		ICAO:      "CKQ7",
		Name:      "Vermilion Bay Airport",
		City:      "Vermilion Bay",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1284,
		Latitude:  49.879699707,
		Longitude: -93.4364013672,
		Timezone:  "America/Rainy_River",
	},
	"CKQ8": {
		ICAO:      "CKQ8",
		Name:      "McArthur River Airport",
		City:      "Mcarthur River",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1718,
		Latitude:  57.7675018311,
		Longitude: -105.0240020752,
		Timezone:  "America/Regina",
	},
	"CKQ9": {
		ICAO:      "CKQ9",
		Name:      "Pine Dock Airport",
		City:      "Pine Dock",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 750,
		Latitude:  51.6199989319,
		Longitude: -96.810798645,
		Timezone:  "America/Winnipeg",
	},
	"CKR3": {
		ICAO:      "CKR3",
		Name:      "Ste. Rose Du Lac Airport",
		City:      "Ste. Rose Du Lac",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 900,
		Latitude:  51.0407981873,
		Longitude: -99.495300293,
		Timezone:  "America/Winnipeg",
	},
	"CKR4": {
		ICAO:      "CKR4",
		Name:      "Lundar Airport",
		City:      "Lundar",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 830,
		Latitude:  50.7033004761,
		Longitude: -98.0567016602,
		Timezone:  "America/Winnipeg",
	},
	"CKR7": {
		ICAO:      "CKR7",
		Name:      "Virden (Gabrielle Farm) Airport",
		City:      "Virden",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 1440,
		Latitude:  49.7846984863,
		Longitude: -100.9560012817,
		Timezone:  "America/Winnipeg",
	},
	"CKR9": {
		ICAO:      "CKR9",
		Name:      "Outlook Airport",
		City:      "Outlook",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1771,
		Latitude:  51.4824981689,
		Longitude: -107.0370025635,
		Timezone:  "America/Swift_Current",
	},
	"CKS3": {
		ICAO:      "CKS3",
		Name:      "Cudworth Airport",
		City:      "Cudworth",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1875,
		Latitude:  52.4880981445,
		Longitude: -105.763999939,
		Timezone:  "America/Regina",
	},
	"CKS6": {
		ICAO:      "CKS6",
		Name:      "Bryant Airport",
		City:      "Estevan",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1970,
		Latitude:  49.4099998474,
		Longitude: -103.1439971924,
		Timezone:  "America/Regina",
	},
	"CKS7": {
		ICAO:      "CKS7",
		Name:      "Wadena Airport",
		City:      "Wadena",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1740,
		Latitude:  51.9294480759,
		Longitude: -103.833761215,
		Timezone:  "America/Regina",
	},
	"CKS8": {
		ICAO:      "CKS8",
		Name:      "Cree Lake (Crystal Lodge) Airport",
		City:      "Cree Lake",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1615,
		Latitude:  57.4641990662,
		Longitude: -106.7480010986,
		Timezone:  "America/Regina",
	},
	"CKS9": {
		ICAO:      "CKS9",
		Name:      "Shepherd's Landing Airport",
		City:      "Kincardine",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 675,
		Latitude:  44.1175003052,
		Longitude: -81.6983032227,
		Timezone:  "America/Toronto",
	},
	"CKT5": {
		ICAO:      "CKT5",
		Name:      "Hartney Airport",
		City:      "Hartney",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 1450,
		Latitude:  49.4500007629,
		Longitude: -100.5500030518,
		Timezone:  "America/Winnipeg",
	},
	"CKT7": {
		ICAO:      "CKT7",
		Name:      "Wakaw Airport",
		City:      "Wakaw",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1660,
		Latitude:  52.6490522523,
		Longitude: -105.763063431,
		Timezone:  "America/Regina",
	},
	"CKU2": {
		ICAO:      "CKU2",
		Name:      "Treherne Airport",
		City:      "Treherne",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 1200,
		Latitude:  49.6310997009,
		Longitude: -98.6663970947,
		Timezone:  "America/Winnipeg",
	},
	"CKU4": {
		ICAO:      "CKU4",
		Name:      "Cut Knife Airport",
		City:      "Cut Knife",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2100,
		Latitude:  52.7332992554,
		Longitude: -109.016998291,
		Timezone:  "America/Swift_Current",
	},
	"CKU5": {
		ICAO:      "CKU5",
		Name:      "Imperial Airport",
		City:      "Imperial",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1665,
		Latitude:  51.3499984741,
		Longitude: -105.4000015259,
		Timezone:  "America/Regina",
	},
	"CKU6": {
		ICAO:      "CKU6",
		Name:      "Grenfell Airport",
		City:      "Grenfell",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1964,
		Latitude:  50.4192008972,
		Longitude: -102.9349975586,
		Timezone:  "America/Regina",
	},
	"CKU7": {
		ICAO:      "CKU7",
		Name:      "Watrous Airport",
		City:      "Watrous",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1700,
		Latitude:  51.68713736,
		Longitude: -105.362319946,
		Timezone:  "America/Regina",
	},
	"CKV2": {
		ICAO:      "CKV2",
		Name:      "Kelvington Airport",
		City:      "Kelvington",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1860,
		Latitude:  52.1333007813,
		Longitude: -103.5350036621,
		Timezone:  "America/Regina",
	},
	"CKV4": {
		ICAO:      "CKV4",
		Name:      "North of Sixty Airport",
		City:      "Obre Lake",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 1195,
		Latitude:  60.3163986206,
		Longitude: -103.1289978027,
		Timezone:  "America/Yellowknife",
	},
	"CKV6": {
		ICAO:      "CKV6",
		Name:      "Churchbridge Airport",
		City:      "Churchbridge",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1745,
		Latitude:  51.0167007446,
		Longitude: -101.8170013428,
		Timezone:  "America/Regina",
	},
	"CKV7": {
		ICAO:      "CKV7",
		Name:      "Wawota Airport",
		City:      "Wawota",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2168,
		Latitude:  49.8944823433,
		Longitude: -102.03045845,
		Timezone:  "America/Regina",
	},
	"CKW6": {
		ICAO:      "CKW6",
		Name:      "Davin Lake Airport",
		City:      "Davin Lake",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1200,
		Latitude:  56.8833007813,
		Longitude: -103.5830001831,
		Timezone:  "America/Regina",
	},
	"CKX2": {
		ICAO:      "CKX2",
		Name:      "Warren / Woodlands Airport",
		City:      "Warren",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 817,
		Latitude:  50.1594009399,
		Longitude: -97.5911026001,
		Timezone:  "America/Winnipeg",
	},
	"CKX4": {
		ICAO:      "CKX4",
		Name:      "Fisher Branch Airport",
		City:      "Fisher Branch",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 825,
		Latitude:  51.0833015442,
		Longitude: -97.4832992554,
		Timezone:  "America/Winnipeg",
	},
	"CKX5": {
		ICAO:      "CKX5",
		Name:      "Dinsmore Airport",
		City:      "Dinsmore",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1900,
		Latitude:  51.3306007385,
		Longitude: -107.43800354,
		Timezone:  "America/Swift_Current",
	},
	"CKX8": {
		ICAO:      "CKX8",
		Name:      "Big River Airport",
		City:      "Big River",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1592,
		Latitude:  53.836101532,
		Longitude: -107.0090026855,
		Timezone:  "America/Swift_Current",
	},
	"CKY2": {
		ICAO:      "CKY2",
		Name:      "Whitewood Airport",
		City:      "Whitewood",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2000,
		Latitude:  50.3347783814,
		Longitude: -102.274475098,
		Timezone:  "America/Regina",
	},
	"CKY8": {
		ICAO:      "CKY8",
		Name:      "Arkayla Springs Airport",
		City:      "Cochrane",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 4000,
		Latitude:  51.20470047,
		Longitude: -114.7740020752,
		Timezone:  "America/Edmonton",
	},
	"CKZ2": {
		ICAO:      "CKZ2",
		Name:      "Willow Bunch Airport",
		City:      "Willow Bunch",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2427,
		Latitude:  49.4000015259,
		Longitude: -105.6669998169,
		Timezone:  "America/Regina",
	},
	"CKZ3": {
		ICAO:      "CKZ3",
		Name:      "Elk Island Airport",
		City:      "Elk Island",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 625,
		Latitude:  54.6713981628,
		Longitude: -94.1453018188,
		Timezone:  "America/Winnipeg",
	},
	"CKZ6": {
		ICAO:      "CKZ6",
		Name:      "Louise Municipal Airport",
		City:      "Crystal City-Pilot Mound",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 1525,
		Latitude:  49.1472015381,
		Longitude: -98.8807983398,
		Timezone:  "America/Winnipeg",
	},
	"CKZ7": {
		ICAO:      "CKZ7",
		Name:      "Winkler Airport",
		City:      "Winkler",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 850,
		Latitude:  49.1669006348,
		Longitude: -97.9203033447,
		Timezone:  "America/Winnipeg",
	},
	"CL01": {
		ICAO:      "CL01",
		Name:      "Eagle Field",
		City:      "Dos Palos",
		State:     "California",
		Country:   "US",
		Elevation: 153,
		Latitude:  36.8998985291,
		Longitude: -120.6679992676,
		Timezone:  "America/Los_Angeles",
	},
	"CL04": {
		ICAO:      "CL04",
		Name:      "Sky Way Estates Airport",
		City:      "Elk Grove",
		State:     "California",
		Country:   "US",
		Elevation: 92,
		Latitude:  38.4107017517,
		Longitude: -121.2350006104,
		Timezone:  "America/Los_Angeles",
	},
	"CL11": {
		ICAO:      "CL11",
		Name:      "Ferdun Ranch Airport",
		City:      "Lodi",
		State:     "California",
		Country:   "US",
		Elevation: 64,
		Latitude:  38.134601593,
		Longitude: -121.2200012207,
		Timezone:  "America/Los_Angeles",
	},
	"CL13": {
		ICAO:      "CL13",
		Name:      "Brian Ranch Airport",
		City:      "Palmdale",
		State:     "California",
		Country:   "US",
		Elevation: 3230,
		Latitude:  34.5153007507,
		Longitude: -117.7620010376,
		Timezone:  "America/Los_Angeles",
	},
	"CL18": {
		ICAO:      "CL18",
		Name:      "Danby Airstrip",
		City:      "Danby",
		State:     "California",
		Country:   "US",
		Elevation: 1400,
		Latitude:  34.5550003052,
		Longitude: -115.3560028076,
		Timezone:  "America/Los_Angeles",
	},
	"CL23": {
		ICAO:      "CL23",
		Name:      "Jones/Ag-Viation Airport",
		City:      "Biggs",
		State:     "California",
		Country:   "US",
		Elevation: 109,
		Latitude:  39.4595985413,
		Longitude: -121.6959991455,
		Timezone:  "America/Los_Angeles",
	},
	"CL24": {
		ICAO:      "CL24",
		Name:      "Kindsvater Ranch Airport",
		City:      "Clovis",
		State:     "California",
		Country:   "US",
		Elevation: 529,
		Latitude:  36.849899292,
		Longitude: -119.5100021362,
		Timezone:  "America/Los_Angeles",
	},
	"CL25": {
		ICAO:      "CL25",
		Name:      "7R Ranch Airport",
		City:      "Cuyama",
		State:     "California",
		Country:   "US",
		Elevation: 3176,
		Latitude:  34.9355010986,
		Longitude: -119.452003479,
		Timezone:  "America/Los_Angeles",
	},
	"CL29": {
		ICAO:      "CL29",
		Name:      "Camino Airstrip",
		City:      "Goffs",
		State:     "California",
		Country:   "US",
		Elevation: 2079,
		Latitude:  34.8349990845,
		Longitude: -114.9570007324,
		Timezone:  "America/Los_Angeles",
	},
	"CL33": {
		ICAO:      "CL33",
		Name:      "Pauma Valley Air Park",
		City:      "Pauma Valley",
		State:     "California",
		Country:   "US",
		Elevation: 756,
		Latitude:  33.3106002808,
		Longitude: -116.9980010986,
		Timezone:  "America/Los_Angeles",
	},
	"CL35": {
		ICAO:      "CL35",
		Name:      "Warner Springs Gliderport Airport",
		City:      "Warner Springs",
		State:     "California",
		Country:   "US",
		Elevation: 2885,
		Latitude:  33.2845001221,
		Longitude: -116.6750030518,
		Timezone:  "America/Los_Angeles",
	},
	"CL36": {
		ICAO:      "CL36",
		Name:      "Allan Ranch Flight Park Ultralightport",
		City:      "Windsor",
		State:     "California",
		Country:   "US",
		Elevation: 160,
		Latitude:  38.5694007874,
		Longitude: -122.8040008545,
		Timezone:  "America/Los_Angeles",
	},
	"CL46": {
		ICAO:      "CL46",
		Name:      "Quail Lake Sky Park Airport",
		City:      "Gorman/Lancaster",
		State:     "California",
		Country:   "US",
		Elevation: 3370,
		Latitude:  34.7677993774,
		Longitude: -118.7320022583,
		Timezone:  "America/Los_Angeles",
	},
	"CL56": {
		ICAO:      "CL56",
		Name:      "Ranchaero Airport",
		City:      "Chico",
		State:     "California",
		Country:   "US",
		Elevation: 173,
		Latitude:  39.7196006775,
		Longitude: -121.871002197,
		Timezone:  "America/Los_Angeles",
	},
	"CL74": {
		ICAO:      "CL74",
		Name:      "Skyotee Ranch Airport",
		City:      "Rosamond",
		State:     "California",
		Country:   "US",
		Elevation: 2600,
		Latitude:  34.8319015503,
		Longitude: -118.4010009766,
		Timezone:  "America/Los_Angeles",
	},
	"CL77": {
		ICAO:      "CL77",
		Name:      "Bonny Doon Village Airport",
		City:      "Santa Cruz",
		State:     "California",
		Country:   "US",
		Elevation: 2020,
		Latitude:  37.0704994202,
		Longitude: -122.1269989014,
		Timezone:  "America/Los_Angeles",
	},
	"CL84": {
		ICAO:      "CL84",
		Name:      "Ahlem Farms Airport",
		City:      "Hilmar",
		State:     "California",
		Country:   "US",
		Elevation: 73,
		Latitude:  37.3883018494,
		Longitude: -120.9270019531,
		Timezone:  "America/Los_Angeles",
	},
	"CL88": {
		ICAO:      "CL88",
		Name:      "Wesinger Ranch Airport",
		City:      "Alturas",
		State:     "California",
		Country:   "US",
		Elevation: 4300,
		Latitude:  41.447101593,
		Longitude: -120.5839996338,
		Timezone:  "America/Los_Angeles",
	},
	"CL90": {
		ICAO:      "CL90",
		Name:      "Butte Creek Hog Ranch Airport",
		City:      "Chico",
		State:     "California",
		Country:   "US",
		Elevation: 200,
		Latitude:  39.6888008118,
		Longitude: -121.783996582,
		Timezone:  "America/Los_Angeles",
	},
	"CLA2": {
		ICAO:      "CLA2",
		Name:      "L'Assomption Airport",
		City:      "L'Assomption",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 52,
		Latitude:  45.8231010437,
		Longitude: -73.4606018066,
		Timezone:  "America/Toronto",
	},
	"CLA4": {
		ICAO:      "CLA4",
		Name:      "Holland Landing Airpark",
		City:      "Holland Landing",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 855,
		Latitude:  44.0894012451,
		Longitude: -79.4950027466,
		Timezone:  "America/Toronto",
	},
	"CLA5": {
		ICAO:      "CLA5",
		Name:      "Lethbridge / Anderson Field",
		City:      "Lethbridge",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3020,
		Latitude:  49.6569444445,
		Longitude: -112.772777778,
		Timezone:  "America/Edmonton",
	},
	"CLA6": {
		ICAO:      "CLA6",
		Name:      "Lancaster Airpark",
		City:      "Bainsville",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 145,
		Latitude:  45.2004013062,
		Longitude: -74.361000061,
		Timezone:  "America/Toronto",
	},
	"CLB2": {
		ICAO:      "CLB2",
		Name:      "Plattsville (Lubitz Flying Field)",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 980,
		Latitude:  43.3058333,
		Longitude: -80.5486111,
		Timezone:  "America/Toronto",
	},
	"CLC2": {
		ICAO:      "CLC2",
		Name:      "London / Chapeskie Field",
		City:      "London",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 930,
		Latitude:  43.0682983398,
		Longitude: -81.1256027222,
		Timezone:  "America/Toronto",
	},
	"CLE4": {
		ICAO:      "CLE4",
		Name:      "Lower East Pubnico (La Field) Airport",
		City:      "Lower East Pubnico",
		State:     "Nova Scotia",
		Country:   "CA",
		Elevation: 107,
		Latitude:  43.6222000122,
		Longitude: -65.7619018555,
		Timezone:  "America/Halifax",
	},
	"CLG3": {
		ICAO:      "CLG3",
		Name:      "Liege / CNRL Airport",
		City:      "Liege",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 1728,
		Latitude:  57.0018997192,
		Longitude: -113.194000244,
		Timezone:  "America/Edmonton",
	},
	"CLG7": {
		ICAO:      "CLG7",
		Name:      "Fort McMurray / Legend Airfield",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2061,
		Latitude:  57.19694444,
		Longitude: -112.895556,
		Timezone:  "America/Edmonton",
	},
	"CLJ3": {
		ICAO:      "CLJ3",
		Name:      "Lethbridge / J3 Airfield",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2910,
		Latitude:  49.7416214711,
		Longitude: -112.738330364,
		Timezone:  "America/Edmonton",
	},
	"CLK3": {
		ICAO:      "CLK3",
		Name:      "Lucknow Airpark",
		City:      "Lucknow",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 960,
		Latitude:  43.9678001404,
		Longitude: -81.4933013916,
		Timezone:  "America/Toronto",
	},
	"CLM2": {
		ICAO:      "CLM2",
		Name:      "Leamington Airport",
		City:      "Leamington",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 576,
		Latitude:  42.0250015259,
		Longitude: -82.5250015259,
		Timezone:  "America/Toronto",
	},
	"CLN4": {
		ICAO:      "CLN4",
		Name:      "Beaverlodge / Clanachan Field",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2694,
		Latitude:  55.2344927355,
		Longitude: -119.841592312,
		Timezone:  "America/Edmonton",
	},
	"CLS3": {
		ICAO:      "CLS3",
		Name:      "Fort McMurray / South Liege Airfield",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 1707,
		Latitude:  56.8323528324,
		Longitude: -113.098840714,
		Timezone:  "America/Edmonton",
	},
	"CLV2": {
		ICAO:      "CLV2",
		Name:      "Stayner (Clearview Field)",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 870,
		Latitude:  44.404637674,
		Longitude: -80.1479780674,
		Timezone:  "America/Toronto",
	},
	"CLW2": {
		ICAO:      "CLW2",
		Name:      "Lewvan (Farr Strip) Airport",
		City:      "Lewvan",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1904,
		Latitude:  49.9850006104,
		Longitude: -104.1149978638,
		Timezone:  "America/Regina",
	},
	"CLW3": {
		ICAO:      "CLW3",
		Name:      "Laurel / Whittington",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1550,
		Latitude:  43.9791679382,
		Longitude: -80.1772232056,
		Timezone:  "America/Toronto",
	},
	"CMA2": {
		ICAO:      "CMA2",
		Name:      "Mattawa Airport",
		City:      "Mattawa",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 600,
		Latitude:  46.2997016907,
		Longitude: -78.7478027344,
		Timezone:  "America/Toronto",
	},
	"CMB2": {
		ICAO:      "CMB2",
		Name:      "Meadowbank Airport",
		City:      "Meadowbank Gold Mine",
		State:     "Nunavut",
		Country:   "CA",
		Elevation: 475,
		Latitude:  65.0246615457,
		Longitude: -96.0686588287,
		Timezone:  "America/Rankin_Inlet",
	},
	"CMF2": {
		ICAO:      "CMF2",
		Name:      "Edmonton / Calmar (Maplelane Farm)",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2520,
		Latitude:  53.1846546068,
		Longitude: -113.738236427,
		Timezone:  "America/Edmonton",
	},
	"CMF3": {
		ICAO:      "CMF3",
		Name:      "Lethbridge (Mercer Field)",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3085,
		Latitude:  49.5548112877,
		Longitude: -112.561540604,
		Timezone:  "America/Edmonton",
	},
	"CML2": {
		ICAO:      "CML2",
		Name:      "Quamichan Lake (Raven Field) Airport",
		City:      "Quamichan Lake",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 130,
		Latitude:  48.8119010925,
		Longitude: -123.6510009766,
		Timezone:  "America/Vancouver",
	},
	"CMN4": {
		ICAO:      "CMN4",
		Name:      "Minto",
		State:     "Yukon",
		Country:   "CA",
		Elevation: 2969,
		Latitude:  62.6047222222,
		Longitude: -137.221944444,
		Timezone:  "America/Whitehorse",
	},
	"CMN5": {
		ICAO:      "CMN5",
		Name:      "Manic-5",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 1332,
		Latitude:  50.6577319114,
		Longitude: -68.8310194016,
		Timezone:  "America/Toronto",
	},
	"CMN6": {
		ICAO:      "CMN6",
		Name:      "Edmonton/Morinville (Mike's Field)",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2302,
		Latitude:  53.8369444444,
		Longitude: -113.563333333,
		Timezone:  "America/Edmonton",
	},
	"CMW3": {
		ICAO:      "CMW3",
		Name:      "Matawatchan",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 990,
		Latitude:  45.1643383253,
		Longitude: -77.0955276489,
		Timezone:  "America/Toronto",
	},
	"CN12": {
		ICAO:      "CN12",
		Name:      "Williams Gliderport",
		City:      "Williams",
		State:     "California",
		Country:   "US",
		Elevation: 68,
		Latitude:  39.1635017395,
		Longitude: -122.1320037842,
		Timezone:  "America/Los_Angeles",
	},
	"CN13": {
		ICAO:      "CN13",
		Name:      "Borges - Clarksburg Airport",
		City:      "Clarksburg",
		State:     "California",
		Country:   "US",
		Elevation: 12,
		Latitude:  38.4431991577,
		Longitude: -121.509002686,
		Timezone:  "America/Los_Angeles",
	},
	"CN15": {
		ICAO:      "CN15",
		Name:      "Vineyard Oaks Farm Airport",
		City:      "Lemore",
		State:     "California",
		Country:   "US",
		Elevation: 210,
		Latitude:  36.3692016602,
		Longitude: -119.7860031128,
		Timezone:  "America/Los_Angeles",
	},
	"CN19": {
		ICAO:      "CN19",
		Name:      "Las Serpientes Airport",
		City:      "Knightsen",
		State:     "California",
		Country:   "US",
		Elevation: 10,
		Latitude:  37.9331016541,
		Longitude: -121.6129989624,
		Timezone:  "America/Los_Angeles",
	},
	"CN23": {
		ICAO:      "CN23",
		Name:      "Harvard Airport",
		City:      "Yermo",
		State:     "California",
		Country:   "US",
		Elevation: 1825,
		Latitude:  34.9617004395,
		Longitude: -116.6750030518,
		Timezone:  "America/Los_Angeles",
	},
	"CN24": {
		ICAO:      "CN24",
		Name:      "Flying R Airport",
		City:      "Galt",
		State:     "California",
		Country:   "US",
		Elevation: 70,
		Latitude:  38.283000946,
		Longitude: -121.2549972534,
		Timezone:  "America/Los_Angeles",
	},
	"CN29": {
		ICAO:      "CN29",
		Name:      "J-B Airport",
		City:      "Farmington",
		State:     "California",
		Country:   "US",
		Elevation: 103,
		Latitude:  37.9351997375,
		Longitude: -121.0149993896,
		Timezone:  "America/Los_Angeles",
	},
	"CN37": {
		ICAO:      "CN37",
		Name:      "Kelso Valley Airport",
		City:      "Mojave",
		State:     "California",
		Country:   "US",
		Elevation: 4047,
		Latitude:  35.38130188,
		Longitude: -118.2310028,
		Timezone:  "America/Los_Angeles",
	},
	"CN38": {
		ICAO:      "CN38",
		Name:      "Flying B Ranch Airport",
		City:      "Franklin",
		State:     "California",
		Country:   "US",
		Elevation: 19,
		Latitude:  38.3413009644,
		Longitude: -121.43699646,
		Timezone:  "America/Los_Angeles",
	},
	"CN42": {
		ICAO:      "CN42",
		Name:      "Double Creek Ranch Airport",
		City:      "Manton",
		State:     "California",
		Country:   "US",
		Elevation: 2030,
		Latitude:  40.4457015991,
		Longitude: -121.8820037842,
		Timezone:  "America/Los_Angeles",
	},
	"CN44": {
		ICAO:      "CN44",
		Name:      "Flying Bull Airport",
		City:      "Modesto",
		State:     "California",
		Country:   "US",
		Elevation: 40,
		Latitude:  37.6204986572,
		Longitude: -121.1630020142,
		Timezone:  "America/Los_Angeles",
	},
	"CN98": {
		ICAO:      "CN98",
		Name:      "Walter'S Camp Airport",
		City:      "Palo Verde",
		State:     "California",
		Country:   "US",
		Elevation: 309,
		Latitude:  33.245556,
		Longitude: -114.710278,
		Timezone:  "America/Los_Angeles",
	},
	"CNA2": {
		ICAO:      "CNA2",
		Name:      "Highgate Airport",
		City:      "Highgate",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 710,
		Latitude:  42.5544013977,
		Longitude: -81.8031005859,
		Timezone:  "America/Toronto",
	},
	"CNA3": {
		ICAO:      "CNA3",
		Name:      "Springwater Barrie Airpark",
		City:      "Springwater",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 963,
		Latitude:  44.4063987732,
		Longitude: -79.7316970825,
		Timezone:  "America/Toronto",
	},
	"CNA4": {
		ICAO:      "CNA4",
		Name:      "Emsdale Airport",
		City:      "Emsdale",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1150,
		Latitude:  45.5499992371,
		Longitude: -79.3499984741,
		Timezone:  "America/Nipigon",
	},
	"CNA9": {
		ICAO:      "CNA9",
		Name:      "Tomvale Airport",
		City:      "Plevna",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 890,
		Latitude:  44.9160995483,
		Longitude: -76.935798645,
		Timezone:  "America/Toronto",
	},
	"CNC3": {
		ICAO:      "CNC3",
		Name:      "Brampton Airport",
		City:      "Brampton",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 935,
		Latitude:  43.7602996826,
		Longitude: -79.875,
		Timezone:  "America/Toronto",
	},
	"CNC4": {
		ICAO:      "CNC4",
		Name:      "Guelph Airport",
		City:      "Guelph",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1100,
		Latitude:  43.5638999939,
		Longitude: -80.1960983276,
		Timezone:  "America/Toronto",
	},
	"CND4": {
		ICAO:      "CND4",
		Name:      "Stanhope Municipal Airport",
		City:      "Haliburton",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1066,
		Latitude:  45.1108333333,
		Longitude: -78.64,
		Timezone:  "America/Toronto",
	},
	"CNE3": {
		ICAO:      "CNE3",
		IATA:      "XBE",
		Name:      "Bearskin Lake Airport",
		City:      "Bearskin Lake",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 800,
		Latitude:  53.9655990601,
		Longitude: -91.0271987915,
		Timezone:  "America/Rainy_River",
	},
	"CNE4": {
		ICAO:      "CNE4",
		Name:      "Iroquois Falls Airport",
		City:      "Iroquois Falls",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1008,
		Latitude:  48.7416992188,
		Longitude: -80.793296814,
		Timezone:  "America/Nipigon",
	},
	"CNE9": {
		ICAO:      "CNE9",
		Name:      "Essex Airport",
		City:      "Essex",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 620,
		Latitude:  42.0957984924,
		Longitude: -82.8794021606,
		Timezone:  "America/Toronto",
	},
	"CNF3": {
		ICAO:      "CNF3",
		Name:      "Pendleton Airport",
		City:      "Pendleton",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 260,
		Latitude:  45.4860992432,
		Longitude: -75.0960998535,
		Timezone:  "America/Toronto",
	},
	"CNF4": {
		ICAO:      "CNF4",
		Name:      "Lindsay Airport",
		City:      "Lindsay",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 882,
		Latitude:  44.3647003174,
		Longitude: -78.7838973999,
		Timezone:  "America/Toronto",
	},
	"CNF8": {
		ICAO:      "CNF8",
		Name:      "Dwight Airport",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1100,
		Latitude:  45.3144454956,
		Longitude: -78.9725036621,
		Timezone:  "America/Toronto",
	},
	"CNF9": {
		ICAO:      "CNF9",
		Name:      "Niagara South Airport",
		City:      "Niagara Falls",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 576,
		Latitude:  42.9994010925,
		Longitude: -79.0635986328,
		Timezone:  "America/Toronto",
	},
	"CNH2": {
		ICAO:      "CNH2",
		Name:      "Natuashish Airport",
		City:      "Natuashish",
		State:     "Newfoundland and Labrador",
		Country:   "CA",
		Elevation: 30,
		Latitude:  55.913898468,
		Longitude: -61.1843986511,
		Timezone:  "America/Goose_Bay",
	},
	"CNJ3": {
		ICAO:      "CNJ3",
		Name:      "Fort Erie Airport",
		City:      "Fort Erie",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 625,
		Latitude:  42.9172121601,
		Longitude: -78.957452774,
		Timezone:  "America/Toronto",
	},
	"CNJ4": {
		ICAO:      "CNJ4",
		Name:      "Orillia Airport",
		City:      "Orillia",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 725,
		Latitude:  44.6776557839,
		Longitude: -79.3102169037,
		Timezone:  "America/Toronto",
	},
	"CNK4": {
		ICAO:      "CNK4",
		Name:      "Parry Sound Area Municipal Airport",
		City:      "Parry Sound",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 832,
		Latitude:  45.2574996948,
		Longitude: -79.8296966553,
		Timezone:  "America/Nipigon",
	},
	"CNL2": {
		ICAO:      "CNL2",
		Name:      "Fort McMurray / North Liege Airfield",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2236,
		Latitude:  57.1360995926,
		Longitude: -113.290071487,
		Timezone:  "America/Edmonton",
	},
	"CNL3": {
		ICAO:      "CNL3",
		Name:      "Brockville - Thousand Islands Regional Tackaberry Airport",
		City:      "Brockville",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 402,
		Latitude:  44.6394004822,
		Longitude: -75.7502975464,
		Timezone:  "America/Toronto",
	},
	"CNL4": {
		ICAO:      "CNL4",
		Name:      "Port Elgin Airport",
		City:      "Port Elgin",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 650,
		Latitude:  44.4152984619,
		Longitude: -81.4143981934,
		Timezone:  "America/Toronto",
	},
	"CNL7": {
		ICAO:      "CNL7",
		Name:      "Nobel / Lumsden Air Park",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 720,
		Latitude:  45.4083333,
		Longitude: -80.0780555,
		Timezone:  "America/Nipigon",
	},
	"CNL8": {
		ICAO:      "CNL8",
		Name:      "Wyevale (Boker Field) Airport",
		City:      "Wyevale",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 705,
		Latitude:  44.6553001404,
		Longitude: -79.8788986206,
		Timezone:  "America/Toronto",
	},
	"CNL9": {
		ICAO:      "CNL9",
		Name:      "Nueltin Lake Airport",
		City:      "Nueltin Lake",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 971,
		Latitude:  59.7080993652,
		Longitude: -100.1269989014,
		Timezone:  "America/Winnipeg",
	},
	"CNM2": {
		ICAO:      "CNM2",
		Name:      "Melbourne Airport",
		City:      "Melbourne",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 735,
		Latitude:  42.8241996765,
		Longitude: -81.5485992432,
		Timezone:  "America/Toronto",
	},
	"CNM5": {
		ICAO:      "CNM5",
		IATA:      "KIF",
		Name:      "Kingfisher Lake Airport",
		City:      "Kingfisher Lake",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 866,
		Latitude:  53.0125007629,
		Longitude: -89.8553009033,
		Timezone:  "America/Rainy_River",
	},
	"CNN3": {
		ICAO:      "CNN3",
		Name:      "Shelburne / Fisher Field",
		City:      "Shelburne",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1675,
		Latitude:  44.0278929436,
		Longitude: -80.2059459686,
		Timezone:  "America/Toronto",
	},
	"CNN8": {
		ICAO:      "CNN8",
		Name:      "Gananoque Airport",
		City:      "Gananoque",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 395,
		Latitude:  44.4019012451,
		Longitude: -76.2442016602,
		Timezone:  "America/Toronto",
	},
	"CNP3": {
		ICAO:      "CNP3",
		Name:      "Arnprior Airport",
		City:      "Arnprior",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 358,
		Latitude:  45.4136009216,
		Longitude: -76.3657989502,
		Timezone:  "America/Toronto",
	},
	"CNP6": {
		ICAO:      "CNP6",
		Name:      "Nampa / Hockey",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2050,
		Latitude:  55.914900566,
		Longitude: -117.136155367,
		Timezone:  "America/Edmonton",
	},
	"CNP7": {
		ICAO:      "CNP7",
		Name:      "Iroquois Airport",
		City:      "Iroquois",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 246,
		Latitude:  44.8421783447,
		Longitude: -75.3114089966,
		Timezone:  "America/New_York",
	},
	"CNP8": {
		ICAO:      "CNP8",
		Name:      "Greenbank Airport",
		City:      "Greenbank",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 975,
		Latitude:  44.1336968141,
		Longitude: -79.0130281448,
		Timezone:  "America/Toronto",
	},
	"CNQ3": {
		ICAO:      "CNQ3",
		Name:      "Welland / Niagara Central Airport",
		City:      "Welland",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 585,
		Latitude:  42.9791984558,
		Longitude: -79.3246994019,
		Timezone:  "America/Toronto",
	},
	"CNR4": {
		ICAO:      "CNR4",
		Name:      "Tobermory Airport",
		City:      "Tobermory",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 700,
		Latitude:  45.2214680842,
		Longitude: -81.6275596619,
		Timezone:  "America/Toronto",
	},
	"CNR6": {
		ICAO:      "CNR6",
		Name:      "Carleton Place Airport",
		City:      "Carleton Place",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 450,
		Latitude:  45.1039009094,
		Longitude: -76.123298645,
		Timezone:  "America/Toronto",
	},
	"CNR9": {
		ICAO:      "CNR9",
		Name:      "Arnstein Airport",
		City:      "Arnstein",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 745,
		Latitude:  45.9255981445,
		Longitude: -79.9278030396,
		Timezone:  "America/Nipigon",
	},
	"CNS4": {
		ICAO:      "CNS4",
		Name:      "Alexandria Aerodrome",
		City:      "Alexandria",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 260,
		Latitude:  45.3306999207,
		Longitude: -74.6257247925,
		Timezone:  "America/Toronto",
	},
	"CNS6": {
		ICAO:      "CNS6",
		Name:      "Straffordville Airport",
		City:      "Straffordville",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 732,
		Latitude:  42.7383003235,
		Longitude: -80.8167037964,
		Timezone:  "America/Toronto",
	},
	"CNS7": {
		ICAO:      "CNS7",
		Name:      "Kincardine Airport",
		City:      "Kincardine",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 772,
		Latitude:  44.2014007568,
		Longitude: -81.6066970825,
		Timezone:  "America/Toronto",
	},
	"CNS8": {
		ICAO:      "CNS8",
		Name:      "Morrisburg Airport",
		City:      "Morrisburg",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 254,
		Latitude:  44.9469985962,
		Longitude: -75.0774002075,
		Timezone:  "America/New_York",
	},
	"CNT3": {
		ICAO:      "CNT3",
		IATA:      "YOG",
		Name:      "Ogoki Post Airport",
		City:      "Ogoki Post",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 594,
		Latitude:  51.6585998535,
		Longitude: -85.9017028809,
		Timezone:  "America/Nipigon",
	},
	"CNT6": {
		ICAO:      "CNT6",
		Name:      "Elmira Airport",
		City:      "Elmira",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1250,
		Latitude:  43.5847015381,
		Longitude: -80.6031036377,
		Timezone:  "America/Toronto",
	},
	"CNT7": {
		ICAO:      "CNT7",
		Name:      "Picton Airport",
		City:      "Picton",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 465,
		Latitude:  43.989200592,
		Longitude: -77.1391983032,
		Timezone:  "America/Toronto",
	},
	"CNU4": {
		ICAO:      "CNU4",
		Name:      "Belleville Airport",
		City:      "Belleville",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 320,
		Latitude:  44.192199707,
		Longitude: -77.3094024658,
		Timezone:  "America/Toronto",
	},
	"CNU8": {
		ICAO:      "CNU8",
		Name:      "Markham Airport",
		City:      "Toronto",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 807,
		Latitude:  43.935798645,
		Longitude: -79.2621994019,
		Timezone:  "America/Toronto",
	},
	"CNV4": {
		ICAO:      "CNV4",
		Name:      "Hawkesbury Airport",
		City:      "Hawkesbury",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 167,
		Latitude:  45.6166992188,
		Longitude: -74.6500015259,
		Timezone:  "America/Toronto",
	},
	"CNV8": {
		ICAO:      "CNV8",
		Name:      "Edenvale Aerodrome",
		City:      "Edenvale",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 718,
		Latitude:  44.4411010742,
		Longitude: -79.9627990723,
		Timezone:  "America/Toronto",
	},
	"CNV9": {
		ICAO:      "CNV9",
		Name:      "Quebec / Neuville Airport",
		City:      "Neuville",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 314,
		Latitude:  46.7214759694,
		Longitude: -71.5854978561,
		Timezone:  "America/Toronto",
	},
	"CNW2": {
		ICAO:      "CNW2",
		Name:      "Windermere Airport",
		City:      "Windermere",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 746,
		Latitude:  45.1624984741,
		Longitude: -79.5353012085,
		Timezone:  "America/Toronto",
	},
	"CNW3": {
		ICAO:      "CNW3",
		Name:      "Bancroft Airport",
		City:      "Bancroft",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1085,
		Latitude:  45.0731010437,
		Longitude: -77.8805999756,
		Timezone:  "America/Toronto",
	},
	"CNX3": {
		ICAO:      "CNX3",
		Name:      "Carey Lake Airport",
		City:      "Carey Lake",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 837,
		Latitude:  49.75,
		Longitude: -84.0333023071,
		Timezone:  "America/Nipigon",
	},
	"CNX8": {
		ICAO:      "CNX8",
		Name:      "Nixon Airport",
		City:      "Nixon",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 780,
		Latitude:  42.8474998474,
		Longitude: -80.3953018188,
		Timezone:  "America/Toronto",
	},
	"CNY3": {
		ICAO:      "CNY3",
		Name:      "Collingwood Airport",
		City:      "Collingwood",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 730,
		Latitude:  44.4491996765,
		Longitude: -80.1583023071,
		Timezone:  "America/Toronto",
	},
	"CNY4": {
		ICAO:      "CNY4",
		Name:      "Alliston Airport",
		City:      "Alliston",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 720,
		Latitude:  44.1833000183,
		Longitude: -79.837097168,
		Timezone:  "America/Toronto",
	},
	"CNZ4": {
		ICAO:      "CNZ4",
		Name:      "Barry's Bay / Madawaska Valley Airpark",
		City:      "Barry's Bay",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1000,
		Latitude:  45.5047227536,
		Longitude: -77.6440930367,
		Timezone:  "America/Toronto",
	},
	"CNZ8": {
		ICAO:      "CNZ8",
		Name:      "Grimsby Airpark",
		City:      "Grimsby",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 640,
		Latitude:  43.1599998474,
		Longitude: -79.5991973877,
		Timezone:  "America/Toronto",
	},
	"CO00": {
		ICAO:      "CO00",
		Name:      "Flagler Aerial Spraying Inc Airport",
		City:      "Flagler",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4945,
		Latitude:  39.2799987793,
		Longitude: -103.0670013428,
		Timezone:  "America/Denver",
	},
	"CO02": {
		ICAO:      "CO02",
		Name:      "Harrington Ranch Airport",
		City:      "Bennet",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5975,
		Latitude:  39.629699707,
		Longitude: -104.5189971924,
		Timezone:  "America/Denver",
	},
	"CO03": {
		ICAO:      "CO03",
		Name:      "Aspen Gliderport",
		City:      "Aspen",
		State:     "Colorado",
		Country:   "US",
		Elevation: 8440,
		Latitude:  39.2599983215,
		Longitude: -106.9120025635,
		Timezone:  "America/Denver",
	},
	"CO06": {
		ICAO:      "CO06",
		Name:      "Wirth Field",
		City:      "New Raymer",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4881,
		Latitude:  40.5135993958,
		Longitude: -103.9120025635,
		Timezone:  "America/Denver",
	},
	"CO07": {
		ICAO:      "CO07",
		Name:      "Athanasiou Valley Airport",
		City:      "Blackhawk",
		State:     "Colorado",
		Country:   "US",
		Elevation: 8900,
		Latitude:  39.8541984558,
		Longitude: -105.4710006714,
		Timezone:  "America/Denver",
	},
	"CO09": {
		ICAO:      "CO09",
		Name:      "Youtsey Airport",
		City:      "Boone",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4500,
		Latitude:  38.2667007446,
		Longitude: -104.283996582,
		Timezone:  "America/Denver",
	},
	"CO10": {
		ICAO:      "CO10",
		Name:      "Lemons Private Strip",
		City:      "Boulder",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5230,
		Latitude:  39.9947013855,
		Longitude: -105.2239990234,
		Timezone:  "America/Denver",
	},
	"CO11": {
		ICAO:      "CO11",
		Name:      "Granite Mountain Lodge Airport",
		City:      "Salida",
		State:     "Colorado",
		Country:   "US",
		Elevation: 9000,
		Latitude:  38.4160995483,
		Longitude: -106.0579986572,
		Timezone:  "America/Denver",
	},
	"CO12": {
		ICAO:      "CO12",
		Name:      "Brighton Van-Aire Estates Airport",
		City:      "Brighton",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5055,
		Latitude:  39.983001709,
		Longitude: -104.705001831,
		Timezone:  "America/Denver",
	},
	"CO13": {
		ICAO:      "CO13",
		Name:      "Heckendorf Ranches Airport",
		City:      "Brighton",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5060,
		Latitude:  39.9632987976,
		Longitude: -104.7490005493,
		Timezone:  "America/Denver",
	},
	"CO14": {
		ICAO:      "CO14",
		Name:      "Reasoner Airport",
		City:      "Brighton",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5100,
		Latitude:  39.9166984558,
		Longitude: -104.8050003052,
		Timezone:  "America/Denver",
	},
	"CO15": {
		ICAO:      "CO15",
		Name:      "Kelly Air Park",
		City:      "Elbert",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7040,
		Latitude:  39.2247009277,
		Longitude: -104.63999939,
		Timezone:  "America/Denver",
	},
	"CO17": {
		ICAO:      "CO17",
		Name:      "Sky Haven Airport",
		City:      "Byers",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5215,
		Latitude:  39.7136001587,
		Longitude: -104.1940002441,
		Timezone:  "America/Denver",
	},
	"CO18": {
		ICAO:      "CO18",
		Name:      "Chaparral Airport",
		City:      "Kiowa",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5930,
		Latitude:  39.4640007019,
		Longitude: -104.2580032349,
		Timezone:  "America/Denver",
	},
	"CO19": {
		ICAO:      "CO19",
		Name:      "Tezak Airport",
		City:      "Canon City",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6280,
		Latitude:  38.4108009338,
		Longitude: -105.6039962769,
		Timezone:  "America/Denver",
	},
	"CO20": {
		ICAO:      "CO20",
		Name:      "Flying W Ranch Airport",
		City:      "Cedaredge",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6885,
		Latitude:  38.8624992371,
		Longitude: -107.8209991455,
		Timezone:  "America/Denver",
	},
	"CO22": {
		ICAO:      "CO22",
		Name:      "Hildebrandt Airport",
		City:      "Bethune",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4300,
		Latitude:  39.2791671753,
		Longitude: -102.4749984741,
		Timezone:  "America/Denver",
	},
	"CO25": {
		ICAO:      "CO25",
		Name:      "Metrogro Farm Airport",
		City:      "Agate",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5240,
		Latitude:  39.4961013794,
		Longitude: -103.7239990234,
		Timezone:  "America/Denver",
	},
	"CO27": {
		ICAO:      "CO27",
		Name:      "Tanner Field",
		City:      "Cortez",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6640,
		Latitude:  37.4678001404,
		Longitude: -108.6650009155,
		Timezone:  "America/Denver",
	},
	"CO38": {
		ICAO:      "CO38",
		Name:      "Comanche Airfield Llc Airport",
		City:      "Strasburg",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5350,
		Latitude:  39.7472000122,
		Longitude: -104.31300354,
		Timezone:  "America/Denver",
	},
	"CO42": {
		ICAO:      "CO42",
		Name:      "Circle 8 Ranch Airport",
		City:      "Elizabeth",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6650,
		Latitude:  39.3347015381,
		Longitude: -104.5459976196,
		Timezone:  "America/Denver",
	},
	"CO43": {
		ICAO:      "CO43",
		Name:      "Pinyon Airport",
		City:      "Glade Park",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6980,
		Latitude:  39.0027999878,
		Longitude: -108.7099990845,
		Timezone:  "America/Denver",
	},
	"CO47": {
		ICAO:      "CO47",
		Name:      "Gypsum Creek Ranch Airport",
		City:      "Gypsum",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6720,
		Latitude:  39.5999984741,
		Longitude: -106.9739990234,
		Timezone:  "America/Denver",
	},
	"CO48": {
		ICAO:      "CO48",
		Name:      "Crop Air Inc Airport",
		City:      "Eaton",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4800,
		Latitude:  40.5125007629,
		Longitude: -104.6520004272,
		Timezone:  "America/Denver",
	},
	"CO49": {
		ICAO:      "CO49",
		Name:      "Flying Lazy D Ranch Airport",
		City:      "Elbert",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6910,
		Latitude:  39.1624984741,
		Longitude: -104.5350036621,
		Timezone:  "America/Denver",
	},
	"CO52": {
		ICAO:      "CO52",
		Name:      "Marshdale STOLport",
		City:      "Evergreen",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7700,
		Latitude:  39.5833015442,
		Longitude: -105.3030014038,
		Timezone:  "America/Denver",
	},
	"CO53": {
		ICAO:      "CO53",
		Name:      "Yankee Field",
		City:      "Fort Collins",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5050,
		Latitude:  40.6347007751,
		Longitude: -104.9909973145,
		Timezone:  "America/Denver",
	},
	"CO54": {
		ICAO:      "CO54",
		Name:      "G W Flanders Ranch Strip",
		City:      "Falcon",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7260,
		Latitude:  38.9832992554,
		Longitude: -104.6340026855,
		Timezone:  "America/Denver",
	},
	"CO55": {
		ICAO:      "CO55",
		Name:      "Christman Field",
		City:      "Fort Collins",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5160,
		Latitude:  40.5971984863,
		Longitude: -105.1439971924,
		Timezone:  "America/Denver",
	},
	"CO56": {
		ICAO:      "CO56",
		Name:      "Jjs Airport",
		City:      "Keenesburg",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5075,
		Latitude:  39.964099884,
		Longitude: -104.4629974365,
		Timezone:  "America/Denver",
	},
	"CO58": {
		ICAO:      "CO58",
		Name:      "Wings N Things Airpark & Museum Airport",
		City:      "Firestone",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5000,
		Latitude:  40.1058006287,
		Longitude: -104.9550018311,
		Timezone:  "America/Denver",
	},
	"CO59": {
		ICAO:      "CO59",
		Name:      "Hay Fever Farm Airport",
		City:      "Galeton",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4840,
		Latitude:  40.5457992554,
		Longitude: -104.5640029907,
		Timezone:  "America/Denver",
	},
	"CO60": {
		ICAO:      "CO60",
		Name:      "Horseshoe Landings Airport",
		City:      "Keenesburg",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4800,
		Latitude:  40.1082992554,
		Longitude: -104.4309997559,
		Timezone:  "America/Denver",
	},
	"CO61": {
		ICAO:      "CO61",
		Name:      "Golden Field (Yellow Hat) Airport",
		City:      "Gardner",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7710,
		Latitude:  37.8422012329,
		Longitude: -105.2139968872,
		Timezone:  "America/Denver",
	},
	"CO65": {
		ICAO:      "CO65",
		Name:      "Geary Ranch Airport",
		City:      "Westcliffe",
		State:     "Colorado",
		Country:   "US",
		Elevation: 8300,
		Latitude:  38.0457992554,
		Longitude: -105.4710006714,
		Timezone:  "America/Denver",
	},
	"CO70": {
		ICAO:      "CO70",
		Name:      "Uhrich Airport",
		City:      "Greeley",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4715,
		Latitude:  40.4604988098,
		Longitude: -104.6460037231,
		Timezone:  "America/Denver",
	},
	"CO73": {
		ICAO:      "CO73",
		Name:      "Air-Sprayers Nr 2 Airport",
		City:      "Hartman",
		State:     "Colorado",
		Country:   "US",
		Elevation: 3610,
		Latitude:  38.0999984741,
		Longitude: -102.2170028687,
		Timezone:  "America/Denver",
	},
	"CO76": {
		ICAO:      "CO76",
		Name:      "Eden Ranch Airport",
		City:      "Hotchkiss",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6150,
		Latitude:  38.8363990784,
		Longitude: -107.8659973145,
		Timezone:  "America/Denver",
	},
	"CO77": {
		ICAO:      "CO77",
		Name:      "Horth Strip",
		City:      "Hudson",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5128,
		Latitude:  39.9785995483,
		Longitude: -104.59400177,
		Timezone:  "America/Denver",
	},
	"CO80": {
		ICAO:      "CO80",
		Name:      "Fowler Airport",
		City:      "Fowler",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4428,
		Latitude:  38.0724983215,
		Longitude: -104.0459976196,
		Timezone:  "America/Denver",
	},
	"CO81": {
		ICAO:      "CO81",
		Name:      "Hill Airport",
		City:      "Joes",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4345,
		Latitude:  39.6208000183,
		Longitude: -102.7440032959,
		Timezone:  "America/Denver",
	},
	"CO82": {
		ICAO:      "CO82",
		Name:      "Land Airport",
		City:      "Keenesburg",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5000,
		Latitude:  40.0957984924,
		Longitude: -104.5889968872,
		Timezone:  "America/Denver",
	},
	"CO84": {
		ICAO:      "CO84",
		Name:      "Idlers Field",
		City:      "Kirk",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4159,
		Latitude:  39.6082992554,
		Longitude: -102.5410003662,
		Timezone:  "America/Denver",
	},
	"CO86": {
		ICAO:      "CO86",
		Name:      "Granby Sports Park Ultralightport",
		City:      "Granby",
		State:     "Colorado",
		Country:   "US",
		Elevation: 8110,
		Latitude:  40.0485992432,
		Longitude: -105.93800354,
		Timezone:  "America/Denver",
	},
	"CO89": {
		ICAO:      "CO89",
		Name:      "Barber Field",
		City:      "Delta",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5050,
		Latitude:  38.7728004456,
		Longitude: -108.0439987183,
		Timezone:  "America/Denver",
	},
	"CO90": {
		ICAO:      "CO90",
		Name:      "Usaf Academy Bullseye Aux Airstrip",
		City:      "Ellicott",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6036,
		Latitude:  38.7583007813,
		Longitude: -104.3069992065,
		Timezone:  "America/Denver",
	},
	"CO92": {
		ICAO:      "CO92",
		Name:      "Frasier Ranch Airport",
		City:      "Last Chance",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5000,
		Latitude:  39.6417007446,
		Longitude: -103.6449966431,
		Timezone:  "America/Denver",
	},
	"CO93": {
		ICAO:      "CO93",
		Name:      "Perry Park Airport",
		City:      "Larkspur",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6700,
		Latitude:  39.25,
		Longitude: -104.8909988403,
		Timezone:  "America/Denver",
	},
	"CO94": {
		ICAO:      "CO94",
		Name:      "Decker Farms Airport",
		City:      "Lindon",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5050,
		Latitude:  39.7069015503,
		Longitude: -103.3519973755,
		Timezone:  "America/Denver",
	},
	"CO95": {
		ICAO:      "CO95",
		Name:      "Wilson Ranch Airport",
		City:      "Telluride",
		State:     "Colorado",
		Country:   "US",
		Elevation: 9580,
		Latitude:  38.0527992249,
		Longitude: -107.9800033569,
		Timezone:  "America/Denver",
	},
	"CO96": {
		ICAO:      "CO96",
		Name:      "Reed Hollow Ranch Airport",
		City:      "Franktown",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6200,
		Latitude:  39.3717002869,
		Longitude: -104.7429962158,
		Timezone:  "America/Denver",
	},
	"CO97": {
		ICAO:      "CO97",
		Name:      "Comanche Springs Ranch Airport",
		City:      "Strasburg",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5435,
		Latitude:  39.7207984924,
		Longitude: -104.3229980469,
		Timezone:  "America/Denver",
	},
	"CO98": {
		ICAO:      "CO98",
		Name:      "Bowen Farms Nr 1 Airport",
		City:      "Littleton",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5450,
		Latitude:  39.5499992371,
		Longitude: -105.033996582,
		Timezone:  "America/Denver",
	},
	"COB4": {
		ICAO:      "COB4",
		Name:      "Orangeville / Brundle Field",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1630,
		Latitude:  43.8745094139,
		Longitude: -80.1798963547,
		Timezone:  "America/Toronto",
	},
	"COL2": {
		ICAO:      "COL2",
		Name:      "Orangeville / Laurel Airstrip",
		City:      "Orangeville",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1620,
		Latitude:  43.9169006348,
		Longitude: -80.2058029175,
		Timezone:  "America/Toronto",
	},
	"CPA4": {
		ICAO:      "CPA4",
		Name:      "Simcoe (Dennison Field) Airport",
		City:      "Simcoe",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 722,
		Latitude:  42.8193524977,
		Longitude: -80.265212059,
		Timezone:  "America/Toronto",
	},
	"CPB5": {
		ICAO:      "CPB5",
		Name:      "Pilot Butte Airport",
		City:      "Pilot Butte",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2000,
		Latitude:  50.461101532,
		Longitude: -104.4309997559,
		Timezone:  "America/Regina",
	},
	"CPB8": {
		ICAO:      "CPB8",
		Name:      "Paramount Bistcho Airport",
		City:      "Paramount Bistcho",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 1969,
		Latitude:  59.6380996704,
		Longitude: -118.3330001831,
		Timezone:  "America/Edmonton",
	},
	"CPB9": {
		ICAO:      "CPB9",
		Name:      "Baldwin Airport",
		City:      "Baldwin",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 750,
		Latitude:  44.26720047,
		Longitude: -79.3405990601,
		Timezone:  "America/Toronto",
	},
	"CPC3": {
		ICAO:      "CPC3",
		Name:      "Arthur (Walter's Field)",
		City:      "Arthur East",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1550,
		Latitude:  43.840578507,
		Longitude: -80.4428815842,
		Timezone:  "America/Toronto",
	},
	"CPC6": {
		ICAO:      "CPC6",
		Name:      "Teeswater (Thompson Field) Airport",
		City:      "Teeswater",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1000,
		Latitude:  43.948299408,
		Longitude: -81.2718963623,
		Timezone:  "America/Toronto",
	},
	"CPD2": {
		ICAO:      "CPD2",
		Name:      "Ethel Airport",
		City:      "Ethel",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1170,
		Latitude:  43.7449989319,
		Longitude: -81.1766967773,
		Timezone:  "America/Toronto",
	},
	"CPD4": {
		ICAO:      "CPD4",
		Name:      "Brussels (Armstrong Field) Airport",
		City:      "Brussels",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1100,
		Latitude:  43.75,
		Longitude: -81.2332992554,
		Timezone:  "America/Toronto",
	},
	"CPD8": {
		ICAO:      "CPD8",
		Name:      "Hawkesbury (Windover Field) Airport",
		City:      "Hawkesbury",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 165,
		Latitude:  45.5643997192,
		Longitude: -74.8097000122,
		Timezone:  "America/Toronto",
	},
	"CPE3": {
		ICAO:      "CPE3",
		Name:      "Elk Lake Airport",
		City:      "Elk Lake",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 950,
		Latitude:  47.7285995483,
		Longitude: -80.3158035278,
		Timezone:  "America/Nipigon",
	},
	"CPE4": {
		ICAO:      "CPE4",
		Name:      "Cambridge / Reid's Field",
		City:      "Cambridge",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1050,
		Latitude:  43.4427306923,
		Longitude: -80.2325105667,
		Timezone:  "America/Toronto",
	},
	"CPE5": {
		ICAO:      "CPE5",
		Name:      "Port Colborne Airport",
		City:      "Port Colborne",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 600,
		Latitude:  42.8774986267,
		Longitude: -79.3527984619,
		Timezone:  "America/Toronto",
	},
	"CPE6": {
		ICAO:      "CPE6",
		Name:      "South River/Sundridge Airport & Float Plane Base",
		City:      "South River - Sundridge",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1160,
		Latitude:  45.8210983276,
		Longitude: -79.3233032227,
		Timezone:  "America/Nipigon",
	},
	"CPF2": {
		ICAO:      "CPF2",
		IATA:      "YEB",
		Name:      "Bar River Airport",
		City:      "Bar River",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 591,
		Latitude:  46.42029953,
		Longitude: -84.0922012329,
		Timezone:  "America/Nipigon",
	},
	"CPF4": {
		ICAO:      "CPF4",
		Name:      "Bruce McPhail Memorial Airport",
		City:      "Cobden",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 500,
		Latitude:  45.5950012207,
		Longitude: -76.8332977295,
		Timezone:  "America/Toronto",
	},
	"CPF6": {
		ICAO:      "CPF6",
		Name:      "Stoney Creek Airport",
		City:      "Stoney Creek",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 680,
		Latitude:  43.1696925259,
		Longitude: -79.7094154358,
		Timezone:  "America/Toronto",
	},
	"CPF7": {
		ICAO:      "CPF7",
		Name:      "Southampton Airport",
		City:      "Southampton",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 685,
		Latitude:  44.4939002991,
		Longitude: -81.3352966309,
		Timezone:  "America/Toronto",
	},
	"CPG4": {
		ICAO:      "CPG4",
		Name:      "Elmira (East) Airport",
		City:      "Elmira",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1150,
		Latitude:  43.5918998718,
		Longitude: -80.5121994019,
		Timezone:  "America/Toronto",
	},
	"CPG5": {
		ICAO:      "CPG5",
		Name:      "Hawkesbury (East) Airport",
		City:      "Hawkesbury",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 200,
		Latitude:  45.5828018188,
		Longitude: -74.5488967896,
		Timezone:  "America/Toronto",
	},
	"CPG6": {
		ICAO:      "CPG6",
		Name:      "Port Elgin (Pryde Field) Airport",
		City:      "Port Elgin",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 661,
		Latitude:  44.4589004517,
		Longitude: -81.379699707,
		Timezone:  "America/Toronto",
	},
	"CPG7": {
		ICAO:      "CPG7",
		Name:      "Fergus (Juergensen Field) Airport",
		City:      "Fergus",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1450,
		Latitude:  43.7350006104,
		Longitude: -80.4471969604,
		Timezone:  "America/Toronto",
	},
	"CPH2": {
		ICAO:      "CPH2",
		Name:      "Rolph Airport",
		City:      "Deep River",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 490,
		Latitude:  46.1166992188,
		Longitude: -77.5333023071,
		Timezone:  "America/Toronto",
	},
	"CPH3": {
		ICAO:      "CPH3",
		Name:      "Port Hope (Peter's Field)",
		City:      "Port Hope",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 560,
		Latitude:  44.0248843879,
		Longitude: -78.4269762039,
		Timezone:  "America/Toronto",
	},
	"CPH9": {
		ICAO:      "CPH9",
		Name:      "Fordwich Airport",
		City:      "Fordwich",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1250,
		Latitude:  43.8880996704,
		Longitude: -80.995300293,
		Timezone:  "America/Toronto",
	},
	"CPJ5": {
		ICAO:      "CPJ5",
		Name:      "Stirling Airport",
		City:      "Stirling",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 625,
		Latitude:  44.2414016724,
		Longitude: -77.5616989136,
		Timezone:  "America/Toronto",
	},
	"CPK2": {
		ICAO:      "CPK2",
		Name:      "Strathroy (Blue Yonder) Airport",
		City:      "Strathroy",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 780,
		Latitude:  42.965801239,
		Longitude: -81.5916976929,
		Timezone:  "America/Toronto",
	},
	"CPK9": {
		ICAO:      "CPK9",
		Name:      "Arthur (Peskett Field)",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1555,
		Latitude:  43.7715431648,
		Longitude: -80.4835224152,
		Timezone:  "America/Toronto",
	},
	"CPL3": {
		ICAO:      "CPL3",
		Name:      "Kars / Rideau Valley Air Park",
		City:      "Kars",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 286,
		Latitude:  45.1030620653,
		Longitude: -75.6281661987,
		Timezone:  "America/Toronto",
	},
	"CPL4": {
		ICAO:      "CPL4",
		Name:      "Grand Bend Airport",
		City:      "Grand Bend",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 642,
		Latitude:  43.2868995667,
		Longitude: -81.7142028809,
		Timezone:  "America/Toronto",
	},
	"CPL5": {
		ICAO:      "CPL5",
		Name:      "Thessalon Municipal Airport",
		City:      "Thessalon",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 750,
		Latitude:  46.3166999817,
		Longitude: -83.5333023071,
		Timezone:  "America/Nipigon",
	},
	"CPM3": {
		ICAO:      "CPM3",
		Name:      "POURVOIRIE MIRAGE AERODROME",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 1300,
		Latitude:  53.8025,
		Longitude: -72.841389,
		Timezone:  "America/Toronto",
	},
	"CPM5": {
		ICAO:      "CPM5",
		Name:      "Volk Airport",
		City:      "Tottenham",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 980,
		Latitude:  43.9913325082,
		Longitude: -79.7811698914,
		Timezone:  "America/Toronto",
	},
	"CPM7": {
		ICAO:      "CPM7",
		Name:      "Bradford Airport",
		City:      "Bradford",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 973,
		Latitude:  44.136100769,
		Longitude: -79.6288986206,
		Timezone:  "America/Toronto",
	},
	"CPN2": {
		ICAO:      "CPN2",
		Name:      "Pontiac Airpark",
		City:      "Pontiac",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 216,
		Latitude:  45.5297264228,
		Longitude: -76.1673545837,
		Timezone:  "America/Toronto",
	},
	"CPN5": {
		ICAO:      "CPN5",
		Name:      "Listowel Airport",
		City:      "Listowel",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1232,
		Latitude:  43.7428016663,
		Longitude: -80.9914016724,
		Timezone:  "America/Toronto",
	},
	"CPP6": {
		ICAO:      "CPP6",
		Name:      "York Airport",
		City:      "York",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 670,
		Latitude:  43.0374984741,
		Longitude: -79.8525009155,
		Timezone:  "America/Toronto",
	},
	"CPQ4": {
		ICAO:      "CPQ4",
		Name:      "Lefroy Airport",
		City:      "Lefroy",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 724,
		Latitude:  44.2919095924,
		Longitude: -79.5472812653,
		Timezone:  "America/Toronto",
	},
	"CPR2": {
		ICAO:      "CPR2",
		Name:      "Ottawa / Embrun Airport",
		City:      "Embrun",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 230,
		Latitude:  45.2411003113,
		Longitude: -75.2985992432,
		Timezone:  "America/Toronto",
	},
	"CPR3": {
		ICAO:      "CPR3",
		Name:      "Palmerston Airport",
		City:      "Palmerston",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1360,
		Latitude:  43.8535995483,
		Longitude: -80.7802963257,
		Timezone:  "America/Toronto",
	},
	"CPR5": {
		ICAO:      "CPR5",
		Name:      "Woodstock Airport",
		City:      "Woodstock",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1040,
		Latitude:  43.1082677518,
		Longitude: -80.8218669891,
		Timezone:  "America/Toronto",
	},
	"CPR7": {
		ICAO:      "CPR7",
		Name:      "Wingham / Richard W. LeVan Airport",
		City:      "Wingham",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1075,
		Latitude:  43.8675003052,
		Longitude: -81.2985992432,
		Timezone:  "America/Toronto",
	},
	"CPR9": {
		ICAO:      "CPR9",
		Name:      "Fergus (Royland Field)",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1400,
		Latitude:  43.7417549969,
		Longitude: -80.3862011433,
		Timezone:  "America/Toronto",
	},
	"CPS2": {
		ICAO:      "CPS2",
		Name:      "Elmhirst's Resort Airport",
		City:      "Keene",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 650,
		Latitude:  44.251470223,
		Longitude: -78.1072568893,
		Timezone:  "America/Toronto",
	},
	"CPS4": {
		ICAO:      "CPS4",
		Name:      "Lucan Airport",
		City:      "Lucan",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 960,
		Latitude:  43.1632995605,
		Longitude: -81.4124984741,
		Timezone:  "America/Toronto",
	},
	"CPS5": {
		ICAO:      "CPS5",
		Name:      "Miminiska Airport",
		City:      "Miminiska",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1000,
		Latitude:  51.6044006348,
		Longitude: -88.5821990967,
		Timezone:  "America/Nipigon",
	},
	"CPS7": {
		ICAO:      "CPS7",
		Name:      "Orton / Smith Field",
		City:      "Orton",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1450,
		Latitude:  43.7753857192,
		Longitude: -80.2280688286,
		Timezone:  "America/Toronto",
	},
	"CPT2": {
		ICAO:      "CPT2",
		Name:      "Killarney Airport",
		City:      "Killarney",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 608,
		Latitude:  45.9775009155,
		Longitude: -81.4946975708,
		Timezone:  "America/Toronto",
	},
	"CPT3": {
		ICAO:      "CPT3",
		Name:      "Rockton Airport",
		City:      "Rockton",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 846,
		Latitude:  43.3222007751,
		Longitude: -80.176399231,
		Timezone:  "America/Toronto",
	},
	"CPU3": {
		ICAO:      "CPU3",
		Name:      "Rodney (New Glasgow) Airport",
		City:      "Rodney",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 670,
		Latitude:  42.5313987732,
		Longitude: -81.6069030762,
		Timezone:  "America/Toronto",
	},
	"CPU6": {
		ICAO:      "CPU6",
		Name:      "Tyendinaga Mohawk Airport",
		City:      "Tyendinaga",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 260,
		Latitude:  44.1847000122,
		Longitude: -77.1078033447,
		Timezone:  "America/Toronto",
	},
	"CPV2": {
		ICAO:      "CPV2",
		Name:      "Orangeville / Murray Wesley Kot Field",
		City:      "Orangeville",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1550,
		Latitude:  43.9585990906,
		Longitude: -80.1549987793,
		Timezone:  "America/Toronto",
	},
	"CPV3": {
		ICAO:      "CPV3",
		Name:      "St. Joseph Island Airport",
		City:      "St. Joseph Island",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 600,
		Latitude:  46.2832984924,
		Longitude: -83.9499969482,
		Timezone:  "America/Nipigon",
	},
	"CPV4": {
		ICAO:      "CPV4",
		Name:      "Mansfield Airport",
		City:      "Mansfield",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 995,
		Latitude:  44.1439218832,
		Longitude: -80.0136208534,
		Timezone:  "America/Toronto",
	},
	"CPV7": {
		ICAO:      "CPV7",
		IATA:      "YHP",
		Name:      "Poplar Hill Airport",
		City:      "Poplar Hill",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1095,
		Latitude:  52.1133003235,
		Longitude: -94.2555999756,
		Timezone:  "America/Rainy_River",
	},
	"CPV8": {
		ICAO:      "CPV8",
		Name:      "Keewaywin Airport",
		City:      "Keewaywin",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 988,
		Latitude:  52.9911003113,
		Longitude: -92.8364028931,
		Timezone:  "America/Rainy_River",
	},
	"CPY9": {
		ICAO:      "CPY9",
		Name:      "Fergus (Holyoake Airfield)",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1380,
		Latitude:  43.728611111,
		Longitude: -80.2830555556,
		Timezone:  "America/Toronto",
	},
	"CPZ3": {
		ICAO:      "CPZ3",
		Name:      "Trenton / Mountain View Airport",
		City:      "Mountain View",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 362,
		Latitude:  44.0694007874,
		Longitude: -77.3380966187,
		Timezone:  "America/Toronto",
	},
	"CRB2": {
		ICAO:      "CRB2",
		Name:      "Cottam Airport",
		City:      "Cottam",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 635,
		Latitude:  42.1399993896,
		Longitude: -82.6528015137,
		Timezone:  "America/Toronto",
	},
	"CRB4": {
		ICAO:      "CRB4",
		Name:      "Riviere Bonnard Airport",
		City:      "Riviere Bonnard",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 1585,
		Latitude:  50.7043991089,
		Longitude: -71.1624984741,
		Timezone:  "America/Toronto",
	},
	"CRB5": {
		ICAO:      "CRB5",
		Name:      "Riviere Bell Airport",
		City:      "Riviere Bell",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 89,
		Latitude:  49.0788993835,
		Longitude: -62.2369003296,
		Timezone:  "America/Toronto",
	},
	"CRD4": {
		ICAO:      "CRD4",
		Name:      "Red Deer / South 40 Airstrip",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3040,
		Latitude:  52.261944,
		Longitude: -113.951389,
		Timezone:  "America/Edmonton",
	},
	"CRD5": {
		ICAO:      "CRD5",
		Name:      "Red Deer / Truant Airfield",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3100,
		Latitude:  52.2870428404,
		Longitude: -113.913180828,
		Timezone:  "America/Edmonton",
	},
	"CRE2": {
		ICAO:      "CRE2",
		Name:      "Edzo Airport",
		City:      "Rae",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 587,
		Latitude:  62.7667007446,
		Longitude: -116.0839996338,
		Timezone:  "America/Yellowknife",
	},
	"CRE3": {
		ICAO:      "CRE3",
		Name:      "Curries (Rand Private Airfield)",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1060,
		Latitude:  43.0643730519,
		Longitude: -80.6983566284,
		Timezone:  "America/Toronto",
	},
	"CRF3": {
		ICAO:      "CRF3",
		Name:      "Edmonton / Villeneuve (Rose Field)",
		City:      "Villeneuve",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2190,
		Latitude:  53.645555556,
		Longitude: -113.802777778,
		Timezone:  "America/Edmonton",
	},
	"CRF4": {
		ICAO:      "CRF4",
		Name:      "Calgary / Okotoks (Rowland Field)",
		City:      "Calgary",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3800,
		Latitude:  50.764722223,
		Longitude: -113.958055556,
		Timezone:  "America/Edmonton",
	},
	"CRJ2": {
		ICAO:      "CRJ2",
		Name:      "Stoney Point (Trepanier)",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 630,
		Latitude:  42.2888870239,
		Longitude: -82.5983352661,
		Timezone:  "America/Toronto",
	},
	"CRL2": {
		ICAO:      "CRL2",
		Name:      "Westport / Rideau Lakes Airport",
		City:      "Westport",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 500,
		Latitude:  44.6666651766,
		Longitude: -76.3966673613,
		Timezone:  "America/Toronto",
	},
	"CRL4": {
		ICAO:      "CRL4",
		Name:      "Kirby Lake Airport",
		City:      "Kirby Lake",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2260,
		Latitude:  55.3555984497,
		Longitude: -110.6370010376,
		Timezone:  "America/Edmonton",
	},
	"CRL7": {
		ICAO:      "CRL7",
		Name:      "Reindeer Lake / Lindbergh Lodge Airstrip",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1128,
		Latitude:  57.28805555,
		Longitude: -102.525,
		Timezone:  "America/Regina",
	},
	"CRM2": {
		ICAO:      "CRM2",
		Name:      "Riding Mountain Airport",
		City:      "Riding Mountain",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 1016,
		Latitude:  50.573600769,
		Longitude: -99.3613967896,
		Timezone:  "America/Winnipeg",
	},
	"CRML": {
		ICAO:      "CRML",
		Name:      "Stoney Point (Le Cunff) Airport",
		City:      "Stoney Point",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 625,
		Latitude:  42.2956008911,
		Longitude: -82.5350036621,
		Timezone:  "America/Toronto",
	},
	"CRP2": {
		ICAO:      "CRP2",
		Name:      "R.M. of Pipestone Airport",
		City:      "Reston",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 1488,
		Latitude:  49.5806007385,
		Longitude: -101.0530014038,
		Timezone:  "America/Winnipeg",
	},
	"CRS3": {
		ICAO:      "CRS3",
		Name:      "Calgary / Christiansen Field",
		City:      "Calgary",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3820,
		Latitude:  50.7994796471,
		Longitude: -114.051260948,
		Timezone:  "America/Edmonton",
	},
	"CRS7": {
		ICAO:      "CRS7",
		Name:      "Matoush Airport",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 2608,
		Latitude:  51.905,
		Longitude: -72.12388888,
		Timezone:  "America/Toronto",
	},
	"CRT2": {
		ICAO:      "CRT2",
		Name:      "Riviere Temiscamie (Air Roberval Ltee) Airport",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 1300,
		Latitude:  51.0092455225,
		Longitude: -72.9835939407,
		Timezone:  "America/Toronto",
	},
	"CSA2": {
		ICAO:      "CSA2",
		Name:      "Lac Agile (Mascouche) Airport",
		City:      "Lac Agile",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 160,
		Latitude:  45.7919006348,
		Longitude: -73.6380996704,
		Timezone:  "America/Toronto",
	},
	"CSB2": {
		ICAO:      "CSB2",
		Name:      "Sable Island Landing Strip",
		City:      "Sable Island",
		State:     "Nova Scotia",
		Country:   "CA",
		Latitude:  43.9303535212,
		Longitude: -59.9597740173,
		Timezone:  "America/Halifax",
	},
	"CSB3": {
		ICAO:      "CSB3",
		Name:      "St-Mathieu-De-Beloeil Airport",
		City:      "St-Mathieu-De-Beloeil",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 47,
		Latitude:  45.5900001526,
		Longitude: -73.2386016846,
		Timezone:  "America/Toronto",
	},
	"CSB5": {
		ICAO:      "CSB5",
		Name:      "Shediac Bridge Airport",
		City:      "Shediac Bridge",
		State:     "New Brunswick",
		Country:   "CA",
		Elevation: 25,
		Latitude:  46.254699707,
		Longitude: -64.5766983032,
		Timezone:  "America/Moncton",
	},
	"CSC3": {
		ICAO:      "CSC3",
		Name:      "Drummondville Airport",
		City:      "Drummondville",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 365,
		Latitude:  45.8452987671,
		Longitude: -72.3946990967,
		Timezone:  "America/Toronto",
	},
	"CSC5": {
		ICAO:      "CSC5",
		Name:      "Lac Etchemin Airport",
		City:      "Lac Etchemin",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 1500,
		Latitude:  46.4071998596,
		Longitude: -70.5014038086,
		Timezone:  "America/Toronto",
	},
	"CSC9": {
		ICAO:      "CSC9",
		Name:      "Sudbury / Coniston Airport",
		City:      "Sudbury",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 886,
		Latitude:  46.4822006226,
		Longitude: -80.8339004517,
		Timezone:  "America/Toronto",
	},
	"CSD3": {
		ICAO:      "CSD3",
		Name:      "Valleyfield Airport",
		City:      "Valleyfield",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 155,
		Latitude:  45.2083015442,
		Longitude: -74.1414031982,
		Timezone:  "America/Toronto",
	},
	"CSD4": {
		ICAO:      "CSD4",
		Name:      "Mont Laurier Airport",
		City:      "Mont-Laurier",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 815,
		Latitude:  46.5643997192,
		Longitude: -75.577796936,
		Timezone:  "America/Toronto",
	},
	"CSE3": {
		ICAO:      "CSE3",
		Name:      "Lourdes-De-Joliette Airport",
		City:      "Lourdes-De-Joliette",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 350,
		Latitude:  46.1094017029,
		Longitude: -73.452796936,
		Timezone:  "America/Toronto",
	},
	"CSE4": {
		ICAO:      "CSE4",
		Name:      "Lachute Airport",
		City:      "Lachute",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 221,
		Latitude:  45.6394004822,
		Longitude: -74.3705978394,
		Timezone:  "America/Toronto",
	},
	"CSE5": {
		ICAO:      "CSE5",
		Name:      "Montmagny Airport",
		City:      "Montmagny",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 37,
		Latitude:  47.0013999939,
		Longitude: -70.5158004761,
		Timezone:  "America/Toronto",
	},
	"CSF3": {
		ICAO:      "CSF3",
		Name:      "Poste Montagnais (Mile 134) Airport",
		City:      "Poste Montagnais",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 1987,
		Latitude:  51.8885993958,
		Longitude: -65.7149963379,
		Timezone:  "America/Toronto",
	},
	"CSF7": {
		ICAO:      "CSF7",
		Name:      "Ottawa / Casselman (Shea Field)",
		City:      "Ottawa",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 216,
		Latitude:  45.299675051,
		Longitude: -75.1715040207,
		Timezone:  "America/Toronto",
	},
	"CSG3": {
		ICAO:      "CSG3",
		Name:      "Joliette Airport",
		City:      "Joliette",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 225,
		Latitude:  46.0449981689,
		Longitude: -73.501701355,
		Timezone:  "America/Toronto",
	},
	"CSG5": {
		ICAO:      "CSG5",
		Name:      "St Jean Chrysostome Airport",
		City:      "St-Jean Chrysostome",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 325,
		Latitude:  46.6852989197,
		Longitude: -71.1517028809,
		Timezone:  "America/Toronto",
	},
	"CSH2": {
		ICAO:      "CSH2",
		Name:      "Ile-aux-Grues Airport",
		City:      "Ile-aux-Grues",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 60,
		Latitude:  47.0710983276,
		Longitude: -70.5327987671,
		Timezone:  "America/Toronto",
	},
	"CSH4": {
		ICAO:      "CSH4",
		Name:      "Lebel-sur-Quevillon Airport",
		City:      "Lebel-sur-Quevillon",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 960,
		Latitude:  49.0303001404,
		Longitude: -77.0171966553,
		Timezone:  "America/Toronto",
	},
	"CSH5": {
		ICAO:      "CSH5",
		Name:      "St-Ferdinand Airport",
		City:      "St-Ferdinand",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 1050,
		Latitude:  46.1260986328,
		Longitude: -71.5361022949,
		Timezone:  "America/Toronto",
	},
	"CSJ2": {
		ICAO:      "CSJ2",
		Name:      "Kanawata Aeroparc Airport",
		City:      "Kanawata",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 1350,
		Latitude:  47.5946998596,
		Longitude: -74.1335983276,
		Timezone:  "America/Toronto",
	},
	"CSJ4": {
		ICAO:      "CSJ4",
		Name:      "Louiseville Airport",
		City:      "Louiseville",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 40,
		Latitude:  46.2461013794,
		Longitude: -72.9044036865,
		Timezone:  "America/Toronto",
	},
	"CSJ5": {
		ICAO:      "CSJ5",
		Name:      "St-Louis-De-France Airport",
		City:      "St-Louis-De-France",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 225,
		Latitude:  46.4366989136,
		Longitude: -72.6303024292,
		Timezone:  "America/Toronto",
	},
	"CSK3": {
		ICAO:      "CSK3",
		Name:      "Montreal / Mascouche Airport",
		City:      "Montreal",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 75,
		Latitude:  45.7186012268,
		Longitude: -73.5980987549,
		Timezone:  "America/Toronto",
	},
	"CSK4": {
		ICAO:      "CSK4",
		Name:      "Mansonville Airport",
		City:      "Mansonville",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 575,
		Latitude:  45.0264015198,
		Longitude: -72.4019012451,
		Timezone:  "America/New_York",
	},
	"CSK5": {
		ICAO:      "CSK5",
		Name:      "Paquet Airport",
		City:      "St-Raymond",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 582,
		Latitude:  46.8950004578,
		Longitude: -71.7857971191,
		Timezone:  "America/Toronto",
	},
	"CSK6": {
		ICAO:      "CSK6",
		IATA:      "YNX",
		Name:      "Snap Lake Airport",
		City:      "Snap Lake",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 1557,
		Latitude:  63.5936012268,
		Longitude: -110.90599823,
		Timezone:  "America/Yellowknife",
	},
	"CSK8": {
		ICAO:      "CSK8",
		Name:      "King George Airpark",
		City:      "Surrey",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 5,
		Latitude:  49.0950012207,
		Longitude: -122.8190002441,
		Timezone:  "America/Vancouver",
	},
	"CSL3": {
		ICAO:      "CSL3",
		Name:      "Lac a la Tortue Airport",
		City:      "Lac-a-la-Tortue",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 434,
		Latitude:  46.6217002869,
		Longitude: -72.6303024292,
		Timezone:  "America/Toronto",
	},
	"CSL5": {
		ICAO:      "CSL5",
		Name:      "St-Victor-De-Beauce Airport",
		City:      "St-Victor-De-Beauce",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 1100,
		Latitude:  46.1183013916,
		Longitude: -70.8891983032,
		Timezone:  "America/Toronto",
	},
	"CSL7": {
		ICAO:      "CSL7",
		Name:      "Odessa / Strawberry Lakes Airstrip",
		City:      "Qu'Appelle",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2160,
		Latitude:  50.3645177502,
		Longitude: -103.74080658,
		Timezone:  "America/Regina",
	},
	"CSL9": {
		ICAO:      "CSL9",
		Name:      "Baie-Comeau (Manic 1) Airport",
		City:      "Baie-Comeau",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 250,
		Latitude:  49.1843986511,
		Longitude: -68.3625030518,
		Timezone:  "America/Toronto",
	},
	"CSM3": {
		ICAO:      "CSM3",
		Name:      "Thetford Mines Airport",
		City:      "Thetford Mines",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 1408,
		Latitude:  46.051700592,
		Longitude: -71.2572021484,
		Timezone:  "America/Toronto",
	},
	"CSM4": {
		ICAO:      "CSM4",
		Name:      "Ste-Julienne Airport",
		City:      "Ste-Julienne",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 280,
		Latitude:  45.9407997131,
		Longitude: -73.7238998413,
		Timezone:  "America/Toronto",
	},
	"CSM5": {
		ICAO:      "CSM5",
		Name:      "St-Michel-Des-Saints Airport",
		City:      "St-Michel-Des-Saints",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 1372,
		Latitude:  46.6808013916,
		Longitude: -73.993598938,
		Timezone:  "America/Toronto",
	},
	"CSN3": {
		ICAO:      "CSN3",
		Name:      "St-Jerome Airport",
		City:      "St-Jerome",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 550,
		Latitude:  45.7803001404,
		Longitude: -74.0618972778,
		Timezone:  "America/Toronto",
	},
	"CSN5": {
		ICAO:      "CSN5",
		Name:      "Stanhope Airport",
		City:      "Stanhope",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 1450,
		Latitude:  45.0155982971,
		Longitude: -71.7944030762,
		Timezone:  "America/New_York",
	},
	"CSN7": {
		ICAO:      "CSN7",
		Name:      "Farnham Airport",
		City:      "Farnham",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 180,
		Latitude:  45.2855987549,
		Longitude: -73.0077972412,
		Timezone:  "America/Toronto",
	},
	"CSP3": {
		ICAO:      "CSP3",
		Name:      "Stony Plain (Lichtner Farms) Airport",
		City:      "Stony Plain",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2415,
		Latitude:  53.5374984741,
		Longitude: -114.0680007935,
		Timezone:  "America/Edmonton",
	},
	"CSP5": {
		ICAO:      "CSP5",
		Name:      "St-Mathias Airport",
		City:      "St-Mathias",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 50,
		Latitude:  45.5008010864,
		Longitude: -73.2416992188,
		Timezone:  "America/Toronto",
	},
	"CSP6": {
		ICAO:      "CSP6",
		Name:      "Montreal / Aeroparc Ile Perrot",
		City:      "Montreal",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 100,
		Latitude:  45.3760986328,
		Longitude: -73.9072036743,
		Timezone:  "America/Toronto",
	},
	"CSQ2": {
		ICAO:      "CSQ2",
		Name:      "Shuswap (Skwlax Field) Airport",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 1170,
		Latitude:  50.8786689434,
		Longitude: -119.590816498,
		Timezone:  "America/Vancouver",
	},
	"CSQ3": {
		ICAO:      "CSQ3",
		Name:      "Valcourt Airport",
		City:      "Valcourt",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 740,
		Latitude:  45.481098175,
		Longitude: -72.3103027344,
		Timezone:  "America/Toronto",
	},
	"CSR3": {
		ICAO:      "CSR3",
		Name:      "Victoriaville Airport",
		City:      "Victoriaville",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 486,
		Latitude:  46.1128005981,
		Longitude: -71.9289016724,
		Timezone:  "America/Toronto",
	},
	"CSR8": {
		ICAO:      "CSR8",
		Name:      "La Sarre Airport",
		City:      "La Sarre",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 1048,
		Latitude:  48.9171981812,
		Longitude: -79.1785964966,
		Timezone:  "America/Toronto",
	},
	"CSS3": {
		ICAO:      "CSS3",
		Name:      "Montreal / Les Cedres Airport",
		City:      "Montreal",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 160,
		Latitude:  45.3474998474,
		Longitude: -74.0766983032,
		Timezone:  "America/Toronto",
	},
	"CSS4": {
		ICAO:      "CSS4",
		Name:      "St-Dominique Airport",
		City:      "St-Dominique",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 250,
		Latitude:  45.6324996948,
		Longitude: -72.818901062,
		Timezone:  "America/Toronto",
	},
	"CST3": {
		ICAO:      "CST3",
		Name:      "Montreal / St-Lazare Airport",
		City:      "Montreal",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 175,
		Latitude:  45.3903282887,
		Longitude: -74.1340041161,
		Timezone:  "America/Toronto",
	},
	"CST7": {
		ICAO:      "CST7",
		Name:      "St-Lambert-De-Lauzon Airport",
		City:      "St-Lambert-De-Lauzon",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 475,
		Latitude:  46.5602989197,
		Longitude: -71.1825027466,
		Timezone:  "America/Toronto",
	},
	"CSU2": {
		ICAO:      "CSU2",
		IATA:      "YKU",
		Name:      "Chisasibi Airport",
		City:      "Chisasibi",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 43,
		Latitude:  53.8055992126,
		Longitude: -78.9169006348,
		Timezone:  "America/Toronto",
	},
	"CSU3": {
		ICAO:      "CSU3",
		Name:      "St Hyacinthe Airport",
		City:      "St-Hyacinthe",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 115,
		Latitude:  45.6049995422,
		Longitude: -73.0141983032,
		Timezone:  "America/Toronto",
	},
	"CSU4": {
		ICAO:      "CSU4",
		Name:      "Ste-Lucie-De-Beauregard Airport",
		City:      "Ste-Lucie-De-Beauregard",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 1212,
		Latitude:  46.7407989502,
		Longitude: -70.0325012207,
		Timezone:  "America/Toronto",
	},
	"CSU5": {
		ICAO:      "CSU5",
		Name:      "Weymontachie Airport",
		City:      "Weymontachie",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 1171,
		Latitude:  47.9369010925,
		Longitude: -73.817199707,
		Timezone:  "America/Toronto",
	},
	"CSU7": {
		ICAO:      "CSU7",
		Name:      "Lac-a-la-Tortue (Water Aerodrome) Airport",
		City:      "Lac-a-la-Tortue",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 427,
		Latitude:  46.6166992188,
		Longitude: -72.6258010864,
		Timezone:  "America/Toronto",
	},
	"CSV8": {
		ICAO:      "CSV8",
		Name:      "Schomberg (Sloan Valley View Farm Airfield)",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 890,
		Latitude:  43.9861452438,
		Longitude: -79.7272896767,
		Timezone:  "America/Toronto",
	},
	"CSX3": {
		ICAO:      "CSX3",
		Name:      "Richelieu",
		City:      "Richelieu",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 115,
		Latitude:  45.4470727938,
		Longitude: -73.2326853275,
		Timezone:  "America/Toronto",
	},
	"CSX5": {
		ICAO:      "CSX5",
		Name:      "Grant Airport",
		City:      "St-Mathias",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 110,
		Latitude:  45.4719009399,
		Longitude: -73.1994018555,
		Timezone:  "America/Toronto",
	},
	"CSX7": {
		ICAO:      "CSX7",
		Name:      "Exeter Airport",
		City:      "Sexsmith",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 856,
		Latitude:  43.3772010803,
		Longitude: -81.5042037964,
		Timezone:  "America/Toronto",
	},
	"CSY3": {
		ICAO:      "CSY3",
		Name:      "Sorel Airport",
		City:      "Sorel",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 76,
		Latitude:  45.9803009033,
		Longitude: -73.0421981812,
		Timezone:  "America/Toronto",
	},
	"CSY4": {
		ICAO:      "CSY4",
		Name:      "St-Donat Airport",
		City:      "St-Donat",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 1270,
		Latitude:  46.3058013916,
		Longitude: -74.1813964844,
		Timezone:  "America/Toronto",
	},
	"CSZ3": {
		ICAO:      "CSZ3",
		Name:      "St-Jovite Airport",
		City:      "Mont-Tremblant",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 790,
		Latitude:  46.1542015076,
		Longitude: -74.5836029053,
		Timezone:  "America/Toronto",
	},
	"CSZ4": {
		ICAO:      "CSZ4",
		Name:      "St-Frederic Airport",
		City:      "St-Frederic",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 991,
		Latitude:  46.331401825,
		Longitude: -70.9633026123,
		Timezone:  "America/Toronto",
	},
	"CT01": {
		ICAO:      "CT01",
		Name:      "Whelan Farms Airport",
		City:      "Bethlehem",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 1020,
		Latitude:  41.6661987305,
		Longitude: -73.1890029907,
		Timezone:  "America/New_York",
	},
	"CT04": {
		ICAO:      "CT04",
		Name:      "Grass Land Air Field",
		City:      "North Canaan",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 705,
		Latitude:  42.0390014648,
		Longitude: -73.2871017456,
		Timezone:  "America/New_York",
	},
	"CT07": {
		ICAO:      "CT07",
		Name:      "Skis Landing Area Airport",
		City:      "Colchester",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 510,
		Latitude:  41.5736999512,
		Longitude: -72.3031005859,
		Timezone:  "America/New_York",
	},
	"CT08": {
		ICAO:      "CT08",
		Name:      "Gardner Lake Airport",
		City:      "Colchester",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 485,
		Latitude:  41.5082015991,
		Longitude: -72.2436981201,
		Timezone:  "America/New_York",
	},
	"CT09": {
		ICAO:      "CT09",
		Name:      "Heckler Field",
		City:      "Coventry",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 793,
		Latitude:  41.802898407,
		Longitude: -72.3917007446,
		Timezone:  "America/New_York",
	},
	"CT11": {
		ICAO:      "CT11",
		Name:      "Devils Hopyard Field",
		City:      "East Haddam",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 125,
		Latitude:  41.4440002441,
		Longitude: -72.3389968872,
		Timezone:  "America/New_York",
	},
	"CT13": {
		ICAO:      "CT13",
		Name:      "Yankee Airstrip",
		City:      "East Killingly",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 720,
		Latitude:  41.8833999634,
		Longitude: -71.8162002563,
		Timezone:  "America/New_York",
	},
	"CT14": {
		ICAO:      "CT14",
		Name:      "Bancroft Airport",
		City:      "East Windsor Hill",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 52,
		Latitude:  41.8667984009,
		Longitude: -72.6162033081,
		Timezone:  "America/New_York",
	},
	"CT15": {
		ICAO:      "CT15",
		Name:      "Wysocki Field",
		City:      "Ellington",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 390,
		Latitude:  41.945400238,
		Longitude: -72.4776000977,
		Timezone:  "America/New_York",
	},
	"CT19": {
		ICAO:      "CT19",
		Name:      "Laurie Field",
		City:      "Hazardville",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 129,
		Latitude:  41.9682006836,
		Longitude: -72.5398025513,
		Timezone:  "America/New_York",
	},
	"CT20": {
		ICAO:      "CT20",
		Name:      "Rankl Field",
		City:      "Marlborough",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 350,
		Latitude:  41.6001014709,
		Longitude: -72.4328994751,
		Timezone:  "America/New_York",
	},
	"CT24": {
		ICAO:      "CT24",
		Name:      "North Canaan Aviation Facilities Inc Airport",
		City:      "North Canaan",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 658,
		Latitude:  42.0453987122,
		Longitude: -73.3386993408,
		Timezone:  "America/New_York",
	},
	"CT29": {
		ICAO:      "CT29",
		Name:      "Valley Farms Airport",
		City:      "Somers",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 221,
		Latitude:  42.0136985779,
		Longitude: -72.4729003906,
		Timezone:  "America/New_York",
	},
	"CT31": {
		ICAO:      "CT31",
		Name:      "Swift Airport",
		City:      "Stafford Springs",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 515,
		Latitude:  41.9650993347,
		Longitude: -72.332901001,
		Timezone:  "America/New_York",
	},
	"CT32": {
		ICAO:      "CT32",
		Name:      "Gallup Farm Airport",
		City:      "Voluntown",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 440,
		Latitude:  41.5354003906,
		Longitude: -71.8345031738,
		Timezone:  "America/New_York",
	},
	"CT36": {
		ICAO:      "CT36",
		Name:      "Gager Field",
		City:      "Bozrah",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 465,
		Latitude:  41.5644989014,
		Longitude: -72.197303772,
		Timezone:  "America/New_York",
	},
	"CT39": {
		ICAO:      "CT39",
		Name:      "Maplewood Farm Airport",
		City:      "Durham",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 280,
		Latitude:  41.4683990479,
		Longitude: -72.7083969116,
		Timezone:  "America/New_York",
	},
	"CT42": {
		ICAO:      "CT42",
		Name:      "Wings Ago Airstrip",
		City:      "Goshen",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 1585,
		Latitude:  41.8501014709,
		Longitude: -73.2661972046,
		Timezone:  "America/New_York",
	},
	"CT43": {
		ICAO:      "CT43",
		Name:      "Spruce Airport",
		City:      "Jewett City",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 200,
		Latitude:  41.5937004089,
		Longitude: -71.9311981201,
		Timezone:  "America/New_York",
	},
	"CT44": {
		ICAO:      "CT44",
		Name:      "Ripley Field",
		City:      "Litchfield",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 1330,
		Latitude:  41.7857017517,
		Longitude: -73.3087005615,
		Timezone:  "America/New_York",
	},
	"CT48": {
		ICAO:      "CT48",
		Name:      "Wychwood Field",
		City:      "Mystic",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 316,
		Latitude:  41.4367980957,
		Longitude: -71.9253997803,
		Timezone:  "America/New_York",
	},
	"CT51": {
		ICAO:      "CT51",
		Name:      "Docktors Field",
		City:      "New Milford",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 300,
		Latitude:  41.5334014893,
		Longitude: -73.4161987305,
		Timezone:  "America/New_York",
	},
	"CT52": {
		ICAO:      "CT52",
		Name:      "Flying Ridge Airstrip",
		City:      "Newtown",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 610,
		Latitude:  41.3681983948,
		Longitude: -73.2911987305,
		Timezone:  "America/New_York",
	},
	"CT59": {
		ICAO:      "CT59",
		Name:      "Good Hill Farm Airport",
		City:      "Roxbury",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 949,
		Latitude:  41.5500984192,
		Longitude: -73.2661972046,
		Timezone:  "America/New_York",
	},
	"CT66": {
		ICAO:      "CT66",
		Name:      "Long View Landing Airport",
		City:      "Washington",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 940,
		Latitude:  41.6668014526,
		Longitude: -73.2828979492,
		Timezone:  "America/New_York",
	},
	"CT74": {
		ICAO:      "CT74",
		Name:      "Westford Airstrip",
		City:      "Ashford/Westford",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 900,
		Latitude:  41.9506988525,
		Longitude: -72.1815032959,
		Timezone:  "America/New_York",
	},
	"CT80": {
		ICAO:      "CT80",
		Name:      "Stonington Airpark",
		City:      "Stonington",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 7,
		Latitude:  41.3473014832,
		Longitude: -71.8859024048,
		Timezone:  "America/New_York",
	},
	"CT85": {
		ICAO:      "CT85",
		Name:      "Roberts Farm Airport",
		City:      "East Windsor",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 37,
		Latitude:  41.8898010254,
		Longitude: -72.6150970459,
		Timezone:  "America/New_York",
	},
	"CT96": {
		ICAO:      "CT96",
		Name:      "Green Acres Airport",
		City:      "Bristol",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 950,
		Latitude:  41.7084007263,
		Longitude: -72.9711990356,
		Timezone:  "America/New_York",
	},
	"CTA3": {
		ICAO:      "CTA3",
		Name:      "Ile aux Coudres Airport",
		City:      "Ile aux Coudres",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 110,
		Latitude:  47.3894004822,
		Longitude: -70.3843994141,
		Timezone:  "America/Toronto",
	},
	"CTA4": {
		ICAO:      "CTA4",
		Name:      "St Bruno-de-Guigues Airport",
		City:      "St-Bruno-de-Guigues",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 820,
		Latitude:  47.4491996765,
		Longitude: -79.4180984497,
		Timezone:  "America/Toronto",
	},
	"CTB6": {
		ICAO:      "CTB6",
		IATA:      "ZTB",
		Name:      "Tete-a-la-Baleine Airport",
		City:      "Tete-a-la-Baleine",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 107,
		Latitude:  50.6744003296,
		Longitude: -59.3835983276,
		Timezone:  "America/Blanc-Sablon",
	},
	"CTE5": {
		ICAO:      "CTE5",
		Name:      "Lac a la Perchaude Airport",
		City:      "Lac a la Perchaude",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 550,
		Latitude:  46.6225013733,
		Longitude: -72.8482971191,
		Timezone:  "America/Toronto",
	},
	"CTF3": {
		ICAO:      "CTF3",
		Name:      "Causapscal Airport",
		City:      "Causapscal",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 340,
		Latitude:  48.3114013672,
		Longitude: -67.2514038086,
		Timezone:  "America/Toronto",
	},
	"CTF4": {
		ICAO:      "CTF4",
		Name:      "Dundalk (Tripp Field)",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1700,
		Latitude:  44.1743863972,
		Longitude: -80.3079128265,
		Timezone:  "America/Toronto",
	},
	"CTG3": {
		ICAO:      "CTG3",
		Name:      "Du Rocher-Perce (Pabok) Airport",
		City:      "Du Rocher-Perce",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 87,
		Latitude:  48.3833007813,
		Longitude: -64.5643997192,
		Timezone:  "America/Toronto",
	},
	"CTH2": {
		ICAO:      "CTH2",
		Name:      "Thor Lake Airport",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 809,
		Latitude:  62.1007902302,
		Longitude: -112.624111176,
		Timezone:  "America/Yellowknife",
	},
	"CTH3": {
		ICAO:      "CTH3",
		Name:      "Grandes-Bergeronnes Airport",
		City:      "Grandes-Bergeronnes",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 69,
		Latitude:  48.2322006226,
		Longitude: -69.5428009033,
		Timezone:  "America/Toronto",
	},
	"CTK2": {
		ICAO:      "CTK2",
		Name:      "Senneterre Airport",
		City:      "Senneterre",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 1106,
		Latitude:  48.3405990601,
		Longitude: -77.181098938,
		Timezone:  "America/Toronto",
	},
	"CTK6": {
		ICAO:      "CTK6",
		Name:      "Kegaska Airport",
		City:      "Kegaska",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 32,
		Latitude:  50.1958007813,
		Longitude: -61.2658004761,
		Timezone:  "America/Toronto",
	},
	"CTK7": {
		ICAO:      "CTK7",
		Name:      "Selkirk / Kindy Airstrip",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 686,
		Latitude:  42.8511123657,
		Longitude: -79.8791656494,
		Timezone:  "America/Toronto",
	},
	"CTN6": {
		ICAO:      "CTN6",
		Name:      "Treherne (South Norfolk Airpark) Airport",
		City:      "Treherne",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 1190,
		Latitude:  49.6596984863,
		Longitude: -98.6652984619,
		Timezone:  "America/Winnipeg",
	},
	"CTP9": {
		ICAO:      "CTP9",
		IATA:      "YAU",
		Name:      "Donaldson Airport",
		City:      "Kattiniq",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 1902,
		Latitude:  61.6622009277,
		Longitude: -73.3214035034,
		Timezone:  "America/Toronto",
	},
	"CTQ2": {
		ICAO:      "CTQ2",
		Name:      "Weller Airport",
		City:      "Stanstead",
		State:     "Vermont",
		Country:   "CA",
		Elevation: 1250,
		Latitude:  45.0344009399,
		Longitude: -72.0350036621,
		Timezone:  "America/New_York",
	},
	"CTQ6": {
		ICAO:      "CTQ6",
		Name:      "St-Anselme Airport",
		City:      "St-Anselme",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 560,
		Latitude:  46.6218986511,
		Longitude: -70.9550018311,
		Timezone:  "America/Toronto",
	},
	"CTR6": {
		ICAO:      "CTR6",
		Name:      "St-Basile (Marcotte) Airport",
		City:      "St-Basile",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 300,
		Latitude:  46.7835998535,
		Longitude: -71.8264007568,
		Timezone:  "America/Toronto",
	},
	"CTT5": {
		ICAO:      "CTT5",
		Name:      "La Romaine Airport",
		City:      "La Romaine",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 90,
		Latitude:  50.2597007751,
		Longitude: -60.6794013977,
		Timezone:  "America/Blanc-Sablon",
	},
	"CTU2": {
		ICAO:      "CTU2",
		Name:      "Fontanges Airport",
		City:      "Fontanges",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 1550,
		Latitude:  54.5539016724,
		Longitude: -71.1733016968,
		Timezone:  "America/Toronto",
	},
	"CTU5": {
		ICAO:      "CTU5",
		IATA:      "ZLT",
		Name:      "La Tabatiere Airport",
		City:      "La Tabatiere",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 102,
		Latitude:  50.8307991028,
		Longitude: -58.9756011963,
		Timezone:  "America/Blanc-Sablon",
	},
	"CTY4": {
		ICAO:      "CTY4",
		Name:      "Olds / North 40 Ranch Airstrip",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3410,
		Latitude:  51.9001705129,
		Longitude: -114.147820473,
		Timezone:  "America/Edmonton",
	},
	"CTY5": {
		ICAO:      "CTY5",
		Name:      "Rougemont Airport",
		City:      "Rougemont",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 80,
		Latitude:  45.4383010864,
		Longitude: -73.0378036499,
		Timezone:  "America/Toronto",
	},
	"CVB2": {
		ICAO:      "CVB2",
		Name:      "Voisey's Bay",
		State:     "Newfoundland and Labrador",
		Country:   "CA",
		Latitude:  56.3447222222,
		Longitude: -62.0880555556,
		Timezone:  "America/Goose_Bay",
	},
	"CVF2": {
		ICAO:      "CVF2",
		Name:      "Fergus (Vodarek Field)",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1380,
		Latitude:  43.7232733033,
		Longitude: -80.2893733978,
		Timezone:  "America/Toronto",
	},
	"CVS2": {
		ICAO:      "CVS2",
		Name:      "Viking (South) Airport",
		City:      "Viking",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2242,
		Latitude:  53.0256004333,
		Longitude: -111.9489974976,
		Timezone:  "America/Edmonton",
	},
	"CWC1": {
		ICAO:      "CWC1",
		Name:      "White City (Radomsky) Airport",
		City:      "White City",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2020,
		Latitude:  50.4397010803,
		Longitude: -104.3059997559,
		Timezone:  "America/Regina",
	},
	"CWF2": {
		ICAO:      "CWF2",
		Name:      "Walter's Falls (Piper Way) Airfield",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1400,
		Latitude:  44.463879951,
		Longitude: -80.6214737892,
		Timezone:  "America/Toronto",
	},
	"CWJC": {
		ICAO:      "CWJC",
		Name:      "Ennadai Lake Airport",
		City:      "Ennadai",
		State:     "Nunavut",
		Country:   "CA",
		Elevation: 1021,
		Latitude:  61.1333333333,
		Longitude: -100.9,
		Timezone:  "America/Rankin_Inlet",
	},
	"CYAB": {
		ICAO:      "CYAB",
		IATA:      "YAB",
		Name:      "Arctic Bay Airport",
		State:     "Nunavut",
		Country:   "CA",
		Elevation: 72,
		Latitude:  73.0057668174,
		Longitude: -85.0425052643,
		Timezone:  "America/Rankin_Inlet",
	},
	"CYAC": {
		ICAO:      "CYAC",
		IATA:      "YAC",
		Name:      "Cat Lake Airport",
		City:      "Cat Lake",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1344,
		Latitude:  51.7271995544,
		Longitude: -91.8244018555,
		Timezone:  "America/Rainy_River",
	},
	"CYAD": {
		ICAO:      "CYAD",
		IATA:      "YAR",
		Name:      "La Grande-3 Airport",
		City:      "La Grande-3",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 775,
		Latitude:  53.5717010498,
		Longitude: -76.1964035034,
		Timezone:  "America/Toronto",
	},
	"CYAG": {
		ICAO:      "CYAG",
		IATA:      "YAG",
		Name:      "Fort Frances Municipal Airport",
		City:      "Fort Frances",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1125,
		Latitude:  48.6542015076,
		Longitude: -93.4396972656,
		Timezone:  "America/Rainy_River",
	},
	"CYAH": {
		ICAO:      "CYAH",
		IATA:      "YAH",
		Name:      "La Grande-4 Airport",
		City:      "La Grande-4",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 1005,
		Latitude:  53.754699707,
		Longitude: -73.6753005981,
		Timezone:  "America/Toronto",
	},
	"CYAL": {
		ICAO:      "CYAL",
		IATA:      "YAL",
		Name:      "Alert Bay Airport",
		City:      "Alert Bay",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 240,
		Latitude:  50.5821990967,
		Longitude: -126.9160003662,
		Timezone:  "America/Vancouver",
	},
	"CYAM": {
		ICAO:      "CYAM",
		IATA:      "YAM",
		Name:      "Sault Ste Marie Airport",
		City:      "Sault Ste Marie",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 630,
		Latitude:  46.4850006104,
		Longitude: -84.5093994141,
		Timezone:  "America/Detroit",
	},
	"CYAQ": {
		ICAO:      "CYAQ",
		IATA:      "XKS",
		Name:      "Kasabonika Airport",
		City:      "Kasabonika",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 672,
		Latitude:  53.5247001648,
		Longitude: -88.6427993774,
		Timezone:  "America/Rainy_River",
	},
	"CYAS": {
		ICAO:      "CYAS",
		IATA:      "YKG",
		Name:      "Kangirsuk Airport",
		City:      "Kangirsuk",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 403,
		Latitude:  60.0271987915,
		Longitude: -69.9991989136,
		Timezone:  "America/Toronto",
	},
	"CYAT": {
		ICAO:      "CYAT",
		IATA:      "YAT",
		Name:      "Attawapiskat Airport",
		City:      "Attawapiskat",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 31,
		Latitude:  52.9275016785,
		Longitude: -82.4319000244,
		Timezone:  "America/Nipigon",
	},
	"CYAU": {
		ICAO:      "CYAU",
		Name:      "Liverpool South Shore Regional Airport",
		City:      "Liverpool",
		State:     "Nova Scotia",
		Country:   "CA",
		Elevation: 321,
		Latitude:  44.2303009033,
		Longitude: -64.8561019897,
		Timezone:  "America/Halifax",
	},
	"CYAV": {
		ICAO:      "CYAV",
		Name:      "Winnipeg / St. Andrews Airport",
		City:      "Winnipeg",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 760,
		Latitude:  50.0564002991,
		Longitude: -97.0325012207,
		Timezone:  "America/Winnipeg",
	},
	"CYAW": {
		ICAO:      "CYAW",
		IATA:      "YAW",
		Name:      "Shearwater Airport",
		City:      "Halifax",
		State:     "Nova Scotia",
		Country:   "CA",
		Elevation: 144,
		Latitude:  44.637222,
		Longitude: -63.502222,
		Timezone:  "America/Halifax",
	},
	"CYAX": {
		ICAO:      "CYAX",
		Name:      "Lac Du Bonnet Airport",
		City:      "Lac Du Bonnet",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 850,
		Latitude:  50.2943992615,
		Longitude: -96.0100021362,
		Timezone:  "America/Winnipeg",
	},
	"CYAY": {
		ICAO:      "CYAY",
		IATA:      "YAY",
		Name:      "St. Anthony Airport",
		City:      "St. Anthony",
		State:     "Newfoundland and Labrador",
		Country:   "CA",
		Elevation: 108,
		Latitude:  51.3918991089,
		Longitude: -56.0830993652,
		Timezone:  "America/St_Johns",
	},
	"CYAZ": {
		ICAO:      "CYAZ",
		IATA:      "YAZ",
		Name:      "Tofino / Long Beach Airport",
		City:      "Tofino",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 80,
		Latitude:  49.0798255464,
		Longitude: -125.775604248,
		Timezone:  "America/Vancouver",
	},
	"CYBA": {
		ICAO:      "CYBA",
		IATA:      "YBA",
		Name:      "Banff Airport",
		City:      "Banff",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 4583,
		Latitude:  51.2073404699,
		Longitude: -115.541861057,
		Timezone:  "America/Edmonton",
	},
	"CYBB": {
		ICAO:      "CYBB",
		IATA:      "YBB",
		Name:      "Kugaaruk Airport",
		City:      "Kugaaruk",
		State:     "Nunavut",
		Country:   "CA",
		Elevation: 56,
		Latitude:  68.5344009399,
		Longitude: -89.8080978394,
		Timezone:  "America/Cambridge_Bay",
	},
	"CYBC": {
		ICAO:      "CYBC",
		IATA:      "YBC",
		Name:      "Baie Comeau Airport",
		City:      "Baie-Comeau",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 71,
		Latitude:  49.1324996948,
		Longitude: -68.2043991089,
		Timezone:  "America/Toronto",
	},
	"CYBD": {
		ICAO:      "CYBD",
		IATA:      "QBC",
		Name:      "Bella Coola Airport",
		City:      "Bella Coola",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 117,
		Latitude:  52.3875007629,
		Longitude: -126.5960006714,
		Timezone:  "America/Vancouver",
	},
	"CYBE": {
		ICAO:      "CYBE",
		IATA:      "YBE",
		Name:      "Uranium City Airport",
		City:      "Uranium City",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1044,
		Latitude:  59.5614013672,
		Longitude: -108.4810028076,
		Timezone:  "America/Regina",
	},
	"CYBF": {
		ICAO:      "CYBF",
		IATA:      "YBY",
		Name:      "Bonnyville Airport",
		City:      "Bonnyville",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 1836,
		Latitude:  54.3041992188,
		Longitude: -110.7440032959,
		Timezone:  "America/Edmonton",
	},
	"CYBG": {
		ICAO:      "CYBG",
		IATA:      "YBG",
		Name:      "CFB Bagotville",
		City:      "Bagotville",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 522,
		Latitude:  48.3306007385,
		Longitude: -70.9963989258,
		Timezone:  "America/Toronto",
	},
	"CYBK": {
		ICAO:      "CYBK",
		IATA:      "YBK",
		Name:      "Baker Lake Airport",
		City:      "Baker Lake",
		State:     "Nunavut",
		Country:   "CA",
		Elevation: 59,
		Latitude:  64.2988967896,
		Longitude: -96.077796936,
		Timezone:  "America/Rankin_Inlet",
	},
	"CYBL": {
		ICAO:      "CYBL",
		IATA:      "YBL",
		Name:      "Campbell River Airport",
		City:      "Campbell River",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 346,
		Latitude:  49.9508018494,
		Longitude: -125.2710037231,
		Timezone:  "America/Vancouver",
	},
	"CYBP": {
		ICAO:      "CYBP",
		Name:      "Brooks Regional",
		City:      "Brooks",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2490,
		Latitude:  50.6335983276,
		Longitude: -111.9260025024,
		Timezone:  "America/Edmonton",
	},
	"CYBQ": {
		ICAO:      "CYBQ",
		IATA:      "XTL",
		Name:      "Tadoule Lake Airport",
		City:      "Tadoule Lake",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 923,
		Latitude:  58.7061004639,
		Longitude: -98.5121994019,
		Timezone:  "America/Winnipeg",
	},
	"CYBR": {
		ICAO:      "CYBR",
		IATA:      "YBR",
		Name:      "Brandon Municipal Airport",
		City:      "Brandon",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 1343,
		Latitude:  49.9099998474,
		Longitude: -99.9518966675,
		Timezone:  "America/Winnipeg",
	},
	"CYBT": {
		ICAO:      "CYBT",
		IATA:      "YBT",
		Name:      "Brochet Airport",
		City:      "Brochet",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 1136,
		Latitude:  57.8894004822,
		Longitude: -101.6790008545,
		Timezone:  "America/Winnipeg",
	},
	"CYBU": {
		ICAO:      "CYBU",
		Name:      "Nipawin Airport",
		City:      "Nipawin",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1220,
		Latitude:  53.3325004578,
		Longitude: -104.0080032349,
		Timezone:  "America/Regina",
	},
	"CYBV": {
		ICAO:      "CYBV",
		IATA:      "YBV",
		Name:      "Berens River Airport",
		City:      "Berens River",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 728,
		Latitude:  52.3588981628,
		Longitude: -97.0183029175,
		Timezone:  "America/Winnipeg",
	},
	"CYBW": {
		ICAO:      "CYBW",
		Name:      "Calgary / Springbank Airport",
		City:      "Calgary",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3940,
		Latitude:  51.103099823,
		Longitude: -114.374000549,
		Timezone:  "America/Edmonton",
	},
	"CYBX": {
		ICAO:      "CYBX",
		IATA:      "YBX",
		Name:      "Lourdes de Blanc Sablon Airport",
		City:      "Lourdes-De-Blanc-Sablon",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 121,
		Latitude:  51.4435997009,
		Longitude: -57.1852989197,
		Timezone:  "America/Blanc-Sablon",
	},
	"CYCA": {
		ICAO:      "CYCA",
		IATA:      "YRF",
		Name:      "Cartwright Airport",
		City:      "Cartwright",
		State:     "Newfoundland and Labrador",
		Country:   "CA",
		Elevation: 40,
		Latitude:  53.682800293,
		Longitude: -57.0419006348,
		Timezone:  "America/Goose_Bay",
	},
	"CYCB": {
		ICAO:      "CYCB",
		IATA:      "YCB",
		Name:      "Cambridge Bay Airport",
		City:      "Cambridge Bay",
		State:     "Nunavut",
		Country:   "CA",
		Elevation: 90,
		Latitude:  69.1081008911,
		Longitude: -105.138000488,
		Timezone:  "America/Cambridge_Bay",
	},
	"CYCC": {
		ICAO:      "CYCC",
		IATA:      "YCC",
		Name:      "Cornwall Regional Airport",
		City:      "Cornwall",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 175,
		Latitude:  45.0928001404,
		Longitude: -74.5633010864,
		Timezone:  "America/Toronto",
	},
	"CYCD": {
		ICAO:      "CYCD",
		IATA:      "YCD",
		Name:      "Nanaimo Airport",
		City:      "Nanaimo",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 93,
		Latitude:  49.0522994995,
		Longitude: -123.8700027466,
		Timezone:  "America/Vancouver",
	},
	"CYCE": {
		ICAO:      "CYCE",
		IATA:      "YCE",
		Name:      "James T. Field Memorial Aerodrome",
		City:      "Centralia",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 824,
		Latitude:  43.2855987549,
		Longitude: -81.5083007813,
		Timezone:  "America/Toronto",
	},
	"CYCG": {
		ICAO:      "CYCG",
		IATA:      "YCG",
		Name:      "Castlegar/West Kootenay Regional Airport",
		City:      "Castlegar",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 1624,
		Latitude:  49.2963981628,
		Longitude: -117.632003784,
		Timezone:  "America/Vancouver",
	},
	"CYCH": {
		ICAO:      "CYCH",
		IATA:      "YCH",
		Name:      "Miramichi Airport",
		City:      "Miramichi",
		State:     "New Brunswick",
		Country:   "CA",
		Elevation: 108,
		Latitude:  47.0078010559,
		Longitude: -65.4492034912,
		Timezone:  "America/Moncton",
	},
	"CYCK": {
		ICAO:      "CYCK",
		IATA:      "XCM",
		Name:      "Chatham Kent Airport",
		City:      "Chatham-Kent",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 650,
		Latitude:  42.3064002991,
		Longitude: -82.0819015503,
		Timezone:  "America/Toronto",
	},
	"CYCL": {
		ICAO:      "CYCL",
		IATA:      "YCL",
		Name:      "Charlo Airport",
		City:      "Charlo",
		State:     "New Brunswick",
		Country:   "CA",
		Elevation: 132,
		Latitude:  47.9907989502,
		Longitude: -66.3302993774,
		Timezone:  "America/Moncton",
	},
	"CYCN": {
		ICAO:      "CYCN",
		IATA:      "YCN",
		Name:      "Cochrane Airport",
		City:      "Cochrane",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 861,
		Latitude:  49.1055984497,
		Longitude: -81.0136032104,
		Timezone:  "America/Nipigon",
	},
	"CYCO": {
		ICAO:      "CYCO",
		IATA:      "YCO",
		Name:      "Kugluktuk Airport",
		City:      "Kugluktuk",
		State:     "Nunavut",
		Country:   "CA",
		Elevation: 74,
		Latitude:  67.8167037964,
		Longitude: -115.143997192,
		Timezone:  "America/Cambridge_Bay",
	},
	"CYCP": {
		ICAO:      "CYCP",
		Name:      "Blue River Airport",
		City:      "Blue River",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2240,
		Latitude:  52.1166992188,
		Longitude: -119.2829971313,
		Timezone:  "America/Vancouver",
	},
	"CYCQ": {
		ICAO:      "CYCQ",
		IATA:      "YCQ",
		Name:      "Chetwynd Airport",
		City:      "Chetwynd",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2000,
		Latitude:  55.6871986389,
		Longitude: -121.6269989014,
		Timezone:  "America/Dawson_Creek",
	},
	"CYCR": {
		ICAO:      "CYCR",
		IATA:      "YCR",
		Name:      "Cross Lake (Charlie Sinclair Memorial) Airport",
		City:      "Cross Lake",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 709,
		Latitude:  54.6105995178,
		Longitude: -97.7608032227,
		Timezone:  "America/Winnipeg",
	},
	"CYCS": {
		ICAO:      "CYCS",
		IATA:      "YCS",
		Name:      "Chesterfield Inlet Airport",
		City:      "Chesterfield Inlet",
		State:     "Nunavut",
		Country:   "CA",
		Elevation: 32,
		Latitude:  63.3469009399,
		Longitude: -90.7311019897,
		Timezone:  "America/Rankin_Inlet",
	},
	"CYCT": {
		ICAO:      "CYCT",
		IATA:      "YCT",
		Name:      "Coronation Airport",
		City:      "Coronation",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2595,
		Latitude:  52.0750007629,
		Longitude: -111.4449996948,
		Timezone:  "America/Edmonton",
	},
	"CYCW": {
		ICAO:      "CYCW",
		IATA:      "YCW",
		Name:      "Chilliwack Airport",
		City:      "Chilliwack",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 32,
		Latitude:  49.1528015137,
		Longitude: -121.9390029907,
		Timezone:  "America/Vancouver",
	},
	"CYCY": {
		ICAO:      "CYCY",
		IATA:      "YCY",
		Name:      "Clyde River Airport",
		City:      "Clyde River",
		State:     "Nunavut",
		Country:   "CA",
		Elevation: 87,
		Latitude:  70.4860992432,
		Longitude: -68.5167007446,
		Timezone:  "America/Iqaluit",
	},
	"CYCZ": {
		ICAO:      "CYCZ",
		IATA:      "YCZ",
		Name:      "Fairmont Hot Springs Airport",
		City:      "Fairmont Hot Springs",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2661,
		Latitude:  50.3302993774,
		Longitude: -115.873001099,
		Timezone:  "America/Edmonton",
	},
	"CYDA": {
		ICAO:      "CYDA",
		IATA:      "YDA",
		Name:      "Dawson City Airport",
		City:      "Dawson City",
		State:     "Yukon",
		Country:   "CA",
		Elevation: 1215,
		Latitude:  64.0430984497,
		Longitude: -139.1280059814,
		Timezone:  "America/Dawson",
	},
	"CYDB": {
		ICAO:      "CYDB",
		IATA:      "YDB",
		Name:      "Burwash Airport",
		City:      "Burwash",
		State:     "Yukon",
		Country:   "CA",
		Elevation: 2647,
		Latitude:  61.3711013794,
		Longitude: -139.0410003662,
		Timezone:  "America/Dawson",
	},
	"CYDC": {
		ICAO:      "CYDC",
		Name:      "Princeton Airport",
		City:      "Town of Princeton",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2298,
		Latitude:  49.4681015015,
		Longitude: -120.511001587,
		Timezone:  "America/Vancouver",
	},
	"CYDF": {
		ICAO:      "CYDF",
		IATA:      "YDF",
		Name:      "Deer Lake Airport",
		City:      "Deer Lake",
		State:     "Newfoundland and Labrador",
		Country:   "CA",
		Elevation: 72,
		Latitude:  49.2108001709,
		Longitude: -57.3913993835,
		Timezone:  "America/St_Johns",
	},
	"CYDL": {
		ICAO:      "CYDL",
		IATA:      "YDL",
		Name:      "Dease Lake Airport",
		City:      "Dease Lake",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2600,
		Latitude:  58.4221992493,
		Longitude: -130.0319976807,
		Timezone:  "America/Vancouver",
	},
	"CYDM": {
		ICAO:      "CYDM",
		IATA:      "YDM",
		Name:      "Ross River Airport",
		City:      "Ross River",
		State:     "Yukon",
		Country:   "CA",
		Elevation: 2314,
		Latitude:  61.9706001282,
		Longitude: -132.4230041504,
		Timezone:  "America/Whitehorse",
	},
	"CYDN": {
		ICAO:      "CYDN",
		IATA:      "YDN",
		Name:      "Dauphin Barker Airport",
		City:      "Dauphin",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 999,
		Latitude:  51.1007995605,
		Longitude: -100.0520019531,
		Timezone:  "America/Winnipeg",
	},
	"CYDO": {
		ICAO:      "CYDO",
		IATA:      "YDO",
		Name:      "Dolbeau St Felicien Airport",
		City:      "Dolbeau-St-Felicien",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 372,
		Latitude:  48.7784996033,
		Longitude: -72.375,
		Timezone:  "America/Toronto",
	},
	"CYDP": {
		ICAO:      "CYDP",
		IATA:      "YDP",
		Name:      "Nain Airport",
		City:      "Nain",
		State:     "Newfoundland and Labrador",
		Country:   "CA",
		Elevation: 22,
		Latitude:  56.5491981506,
		Longitude: -61.6803016663,
		Timezone:  "America/Goose_Bay",
	},
	"CYDQ": {
		ICAO:      "CYDQ",
		IATA:      "YDQ",
		Name:      "Dawson Creek Airport",
		City:      "Dawson Creek",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2148,
		Latitude:  55.7422981262,
		Longitude: -120.1829986572,
		Timezone:  "America/Dawson_Creek",
	},
	"CYEA": {
		ICAO:      "CYEA",
		Name:      "Empress Airport",
		City:      "Empress",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2211,
		Latitude:  50.9333000183,
		Longitude: -110.0130004883,
		Timezone:  "America/Edmonton",
	},
	"CYEE": {
		ICAO:      "CYEE",
		Name:      "Huronia Airport",
		City:      "Midland",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 770,
		Latitude:  44.6833000183,
		Longitude: -79.9282989502,
		Timezone:  "America/Toronto",
	},
	"CYEG": {
		ICAO:      "CYEG",
		IATA:      "YEG",
		Name:      "Edmonton International Airport",
		City:      "Edmonton",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2373,
		Latitude:  53.3097000122,
		Longitude: -113.580001831,
		Timezone:  "America/Edmonton",
	},
	"CYEK": {
		ICAO:      "CYEK",
		IATA:      "YEK",
		Name:      "Arviat Airport",
		City:      "Arviat",
		State:     "Nunavut",
		Country:   "CA",
		Elevation: 32,
		Latitude:  61.0942001343,
		Longitude: -94.0708007812,
		Timezone:  "America/Rankin_Inlet",
	},
	"CYEL": {
		ICAO:      "CYEL",
		IATA:      "YEL",
		Name:      "Elliot Lake Municipal Airport",
		City:      "Elliot Lake",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1087,
		Latitude:  46.351398468,
		Longitude: -82.5614013672,
		Timezone:  "America/Nipigon",
	},
	"CYEM": {
		ICAO:      "CYEM",
		IATA:      "YEM",
		Name:      "Manitoulin East Municipal Airport",
		City:      "Manitowaning",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 869,
		Latitude:  45.8428001404,
		Longitude: -81.8581008911,
		Timezone:  "America/Nipigon",
	},
	"CYEN": {
		ICAO:      "CYEN",
		IATA:      "YEN",
		Name:      "Estevan Airport",
		City:      "Estevan",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1905,
		Latitude:  49.2103004456,
		Longitude: -102.966003418,
		Timezone:  "America/Regina",
	},
	"CYER": {
		ICAO:      "CYER",
		IATA:      "YER",
		Name:      "Fort Severn Airport",
		City:      "Fort Severn",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 48,
		Latitude:  56.018901825,
		Longitude: -87.6761016846,
		Timezone:  "America/Nipigon",
	},
	"CYES": {
		ICAO:      "CYES",
		Name:      "Edmundston Airport",
		City:      "Edmundston",
		State:     "New Brunswick",
		Country:   "CA",
		Elevation: 490,
		Latitude:  47.4906005859,
		Longitude: -68.4819030762,
		Timezone:  "America/Moncton",
	},
	"CYET": {
		ICAO:      "CYET",
		IATA:      "YET",
		Name:      "Edson Airport",
		City:      "Edson",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3043,
		Latitude:  53.5788993835,
		Longitude: -116.464996338,
		Timezone:  "America/Edmonton",
	},
	"CYEU": {
		ICAO:      "CYEU",
		IATA:      "YEU",
		Name:      "Eureka Airport",
		City:      "Eureka",
		State:     "Nunavut",
		Country:   "CA",
		Elevation: 256,
		Latitude:  79.9946975708,
		Longitude: -85.814201355,
		Timezone:  "America/Rankin_Inlet",
	},
	"CYEV": {
		ICAO:      "CYEV",
		IATA:      "YEV",
		Name:      "Inuvik Mike Zubko Airport",
		City:      "Inuvik",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 224,
		Latitude:  68.3041992188,
		Longitude: -133.483001709,
		Timezone:  "America/Inuvik",
	},
	"CYEY": {
		ICAO:      "CYEY",
		IATA:      "YEY",
		Name:      "Amos Magny Airport",
		City:      "Amos",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 1068,
		Latitude:  48.5638999939,
		Longitude: -78.2497024536,
		Timezone:  "America/Toronto",
	},
	"CYFA": {
		ICAO:      "CYFA",
		IATA:      "YFA",
		Name:      "Fort Albany Airport",
		City:      "Fort Albany",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 48,
		Latitude:  52.2014007568,
		Longitude: -81.6968994141,
		Timezone:  "America/Nipigon",
	},
	"CYFB": {
		ICAO:      "CYFB",
		IATA:      "YFB",
		Name:      "Iqaluit Airport",
		City:      "Iqaluit",
		State:     "Nunavut",
		Country:   "CA",
		Elevation: 110,
		Latitude:  63.756401062,
		Longitude: -68.5558013916,
		Timezone:  "America/Iqaluit",
	},
	"CYFC": {
		ICAO:      "CYFC",
		IATA:      "YFC",
		Name:      "Fredericton Airport",
		City:      "Fredericton",
		State:     "New Brunswick",
		Country:   "CA",
		Elevation: 68,
		Latitude:  45.8689002991,
		Longitude: -66.5372009277,
		Timezone:  "America/Moncton",
	},
	"CYFD": {
		ICAO:      "CYFD",
		Name:      "Brantford Municipal Airport",
		City:      "Brantford",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 815,
		Latitude:  43.131401062,
		Longitude: -80.3424987793,
		Timezone:  "America/Toronto",
	},
	"CYFE": {
		ICAO:      "CYFE",
		IATA:      "YFE",
		Name:      "Forestville Airport",
		City:      "Forestville",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 293,
		Latitude:  48.7461013794,
		Longitude: -69.0971984863,
		Timezone:  "America/Toronto",
	},
	"CYFH": {
		ICAO:      "CYFH",
		IATA:      "YFH",
		Name:      "Fort Hope Airport",
		City:      "Fort Hope",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 899,
		Latitude:  51.5619010925,
		Longitude: -87.9077987671,
		Timezone:  "America/Nipigon",
	},
	"CYFJ": {
		ICAO:      "CYFJ",
		IATA:      "YTM",
		Name:      "La Macaza / Mont-Tremblant International Inc Airport",
		City:      "Riviere Rouge",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 827,
		Latitude:  46.4094009399,
		Longitude: -74.7799987793,
		Timezone:  "America/Toronto",
	},
	"CYFO": {
		ICAO:      "CYFO",
		IATA:      "YFO",
		Name:      "Flin Flon Airport",
		City:      "Flin Flon",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 997,
		Latitude:  54.6781005859,
		Longitude: -101.6819992065,
		Timezone:  "America/Winnipeg",
	},
	"CYFR": {
		ICAO:      "CYFR",
		IATA:      "YFR",
		Name:      "Fort Resolution Airport",
		City:      "Fort Resolution",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 526,
		Latitude:  61.1808013916,
		Longitude: -113.690002441,
		Timezone:  "America/Yellowknife",
	},
	"CYFS": {
		ICAO:      "CYFS",
		IATA:      "YFS",
		Name:      "Fort Simpson Airport",
		City:      "Fort Simpson",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 555,
		Latitude:  61.7602005005,
		Longitude: -121.2369995117,
		Timezone:  "America/Inuvik",
	},
	"CYFT": {
		ICAO:      "CYFT",
		IATA:      "YMN",
		Name:      "Makkovik Airport",
		City:      "Makkovik",
		State:     "Newfoundland and Labrador",
		Country:   "CA",
		Elevation: 234,
		Latitude:  55.0769004822,
		Longitude: -59.1864013672,
		Timezone:  "America/Goose_Bay",
	},
	"CYGB": {
		ICAO:      "CYGB",
		IATA:      "YGB",
		Name:      "Texada Gillies Bay Airport",
		City:      "Texada",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 326,
		Latitude:  49.6941986084,
		Longitude: -124.5179977417,
		Timezone:  "America/Vancouver",
	},
	"CYGD": {
		ICAO:      "CYGD",
		IATA:      "YGD",
		Name:      "Goderich Airport",
		City:      "Goderich",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 712,
		Latitude:  43.7668991089,
		Longitude: -81.7106018066,
		Timezone:  "America/Toronto",
	},
	"CYGE": {
		ICAO:      "CYGE",
		Name:      "Golden Airport",
		City:      "Golden",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2575,
		Latitude:  51.2991981506,
		Longitude: -116.9820022583,
		Timezone:  "America/Edmonton",
	},
	"CYGH": {
		ICAO:      "CYGH",
		IATA:      "YGH",
		Name:      "Fort Good Hope Airport",
		City:      "Fort Good Hope",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 268,
		Latitude:  66.2407989502,
		Longitude: -128.6510009766,
		Timezone:  "America/Inuvik",
	},
	"CYGK": {
		ICAO:      "CYGK",
		IATA:      "YGK",
		Name:      "Kingston Norman Rogers Airport",
		City:      "Kingston",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 305,
		Latitude:  44.2252998352,
		Longitude: -76.5969009399,
		Timezone:  "America/Toronto",
	},
	"CYGL": {
		ICAO:      "CYGL",
		IATA:      "YGL",
		Name:      "La Grande Riviere Airport",
		City:      "La Grande Riviere",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 639,
		Latitude:  53.6253013611,
		Longitude: -77.7042007446,
		Timezone:  "America/Toronto",
	},
	"CYGM": {
		ICAO:      "CYGM",
		IATA:      "YGM",
		Name:      "Gimli Industrial Park Airport",
		City:      "Gimli",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 753,
		Latitude:  50.6281013489,
		Longitude: -97.043296814,
		Timezone:  "America/Winnipeg",
	},
	"CYGO": {
		ICAO:      "CYGO",
		IATA:      "YGO",
		Name:      "Gods Lake Narrows Airport",
		City:      "Gods Lake Narrows",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 617,
		Latitude:  54.5588989258,
		Longitude: -94.4914016724,
		Timezone:  "America/Winnipeg",
	},
	"CYGP": {
		ICAO:      "CYGP",
		IATA:      "YGP",
		Name:      "Gaspe (Michel-Pouliot) Airport",
		City:      "Gaspe",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 112,
		Latitude:  48.7752990723,
		Longitude: -64.4785995483,
		Timezone:  "America/Toronto",
	},
	"CYGQ": {
		ICAO:      "CYGQ",
		IATA:      "YGQ",
		Name:      "Geraldton Greenstone Regional Airport",
		City:      "Geraldton",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1144,
		Latitude:  49.778301239,
		Longitude: -86.9393997192,
		Timezone:  "America/Nipigon",
	},
	"CYGR": {
		ICAO:      "CYGR",
		IATA:      "YGR",
		Name:      "Iles-de-la-Madeleine Airport",
		City:      "Iles-de-la-Madeleine",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 35,
		Latitude:  47.4247016907,
		Longitude: -61.7780990601,
		Timezone:  "America/Halifax",
	},
	"CYGT": {
		ICAO:      "CYGT",
		IATA:      "YGT",
		Name:      "Igloolik Airport",
		City:      "Igloolik",
		State:     "Nunavut",
		Country:   "CA",
		Elevation: 174,
		Latitude:  69.3647003174,
		Longitude: -81.8161010742,
		Timezone:  "America/Iqaluit",
	},
	"CYGV": {
		ICAO:      "CYGV",
		IATA:      "YGV",
		Name:      "Havre St Pierre Airport",
		City:      "Havre St-Pierre",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 124,
		Latitude:  50.2818984985,
		Longitude: -63.6114006042,
		Timezone:  "America/Toronto",
	},
	"CYGW": {
		ICAO:      "CYGW",
		IATA:      "YGW",
		Name:      "Kuujjuarapik Airport",
		City:      "Kuujjuarapik",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 34,
		Latitude:  55.2818984985,
		Longitude: -77.765296936,
		Timezone:  "America/Iqaluit",
	},
	"CYGX": {
		ICAO:      "CYGX",
		IATA:      "YGX",
		Name:      "Gillam Airport",
		City:      "Gillam",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 476,
		Latitude:  56.3574981689,
		Longitude: -94.7106018066,
		Timezone:  "America/Winnipeg",
	},
	"CYGZ": {
		ICAO:      "CYGZ",
		IATA:      "YGZ",
		Name:      "Grise Fiord Airport",
		City:      "Grise Fiord",
		State:     "Nunavut",
		Country:   "CA",
		Elevation: 146,
		Latitude:  76.4261016846,
		Longitude: -82.9092025757,
		Timezone:  "America/Iqaluit",
	},
	"CYHA": {
		ICAO:      "CYHA",
		IATA:      "YQC",
		Name:      "Quaqtaq Airport",
		City:      "Quaqtaq",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 103,
		Latitude:  61.0463981628,
		Longitude: -69.6177978516,
		Timezone:  "America/Iqaluit",
	},
	"CYHB": {
		ICAO:      "CYHB",
		IATA:      "YHB",
		Name:      "Hudson Bay Airport",
		City:      "Hudson Bay",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1175,
		Latitude:  52.8166999817,
		Longitude: -102.3109970093,
		Timezone:  "America/Regina",
	},
	"CYHD": {
		ICAO:      "CYHD",
		IATA:      "YHD",
		Name:      "Dryden Regional Airport",
		City:      "Dryden",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1354,
		Latitude:  49.8316993713,
		Longitude: -92.7442016602,
		Timezone:  "America/Rainy_River",
	},
	"CYHE": {
		ICAO:      "CYHE",
		IATA:      "YHE",
		Name:      "Hope Airport",
		City:      "Hope",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 128,
		Latitude:  49.3683013916,
		Longitude: -121.498001099,
		Timezone:  "America/Vancouver",
	},
	"CYHF": {
		ICAO:      "CYHF",
		IATA:      "YHF",
		Name:      "Hearst Rene Fontaine Municipal Airport",
		City:      "Hearst",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 827,
		Latitude:  49.7141990662,
		Longitude: -83.6860961914,
		Timezone:  "America/Nipigon",
	},
	"CYHH": {
		ICAO:      "CYHH",
		IATA:      "YNS",
		Name:      "Nemiscau Airport",
		City:      "Nemiscau",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 802,
		Latitude:  51.6911010742,
		Longitude: -76.135597229,
		Timezone:  "America/Toronto",
	},
	"CYHI": {
		ICAO:      "CYHI",
		IATA:      "YHI",
		Name:      "Ulukhaktok Holman Airport",
		City:      "Ulukhaktok",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 117,
		Latitude:  70.762802124,
		Longitude: -117.8059997559,
		Timezone:  "America/Yellowknife",
	},
	"CYHK": {
		ICAO:      "CYHK",
		IATA:      "YHK",
		Name:      "Gjoa Haven Airport",
		City:      "Gjoa Haven",
		State:     "Nunavut",
		Country:   "CA",
		Elevation: 152,
		Latitude:  68.635597229,
		Longitude: -95.8497009277,
		Timezone:  "America/Cambridge_Bay",
	},
	"CYHM": {
		ICAO:      "CYHM",
		IATA:      "YHM",
		Name:      "John C. Munro Hamilton International Airport",
		City:      "Hamilton",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 780,
		Latitude:  43.1735992432,
		Longitude: -79.9349975586,
		Timezone:  "America/Toronto",
	},
	"CYHN": {
		ICAO:      "CYHN",
		IATA:      "YHN",
		Name:      "Hornepayne Municipal Airport",
		City:      "Hornepayne",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1099,
		Latitude:  49.1930999756,
		Longitude: -84.7589035034,
		Timezone:  "America/Nipigon",
	},
	"CYHO": {
		ICAO:      "CYHO",
		IATA:      "YHO",
		Name:      "Hopedale Airport",
		City:      "Hopedale",
		State:     "Newfoundland and Labrador",
		Country:   "CA",
		Elevation: 39,
		Latitude:  55.448299408,
		Longitude: -60.2285995483,
		Timezone:  "America/Goose_Bay",
	},
	"CYHR": {
		ICAO:      "CYHR",
		IATA:      "YHR",
		Name:      "Chevery Airport",
		City:      "Chevery",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 39,
		Latitude:  50.4688987732,
		Longitude: -59.6366996765,
		Timezone:  "America/Blanc-Sablon",
	},
	"CYHS": {
		ICAO:      "CYHS",
		Name:      "Hanover / Saugeen Municipal Airport",
		City:      "Hanover",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 939,
		Latitude:  44.1582984924,
		Longitude: -81.0627975464,
		Timezone:  "America/Toronto",
	},
	"CYHT": {
		ICAO:      "CYHT",
		IATA:      "YHT",
		Name:      "Haines Junction Airport",
		City:      "Haines Junction",
		State:     "Yukon",
		Country:   "CA",
		Elevation: 2150,
		Latitude:  60.7891998291,
		Longitude: -137.546005249,
		Timezone:  "America/Whitehorse",
	},
	"CYHU": {
		ICAO:      "CYHU",
		IATA:      "YHU",
		Name:      "Montreal / Saint-Hubert Airport",
		City:      "Montreal",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 90,
		Latitude:  45.5175018311,
		Longitude: -73.4169006348,
		Timezone:  "America/Toronto",
	},
	"CYHY": {
		ICAO:      "CYHY",
		IATA:      "YHY",
		Name:      "Hay River / Merlyn Carter Airport",
		City:      "Hay River",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 541,
		Latitude:  60.8396987915,
		Longitude: -115.782997131,
		Timezone:  "America/Yellowknife",
	},
	"CYHZ": {
		ICAO:      "CYHZ",
		IATA:      "YHZ",
		Name:      "Halifax / Stanfield International Airport",
		City:      "Halifax",
		State:     "Nova Scotia",
		Country:   "CA",
		Elevation: 477,
		Latitude:  44.8807983398,
		Longitude: -63.5085983276,
		Timezone:  "America/Halifax",
	},
	"CYIB": {
		ICAO:      "CYIB",
		IATA:      "YIB",
		Name:      "Atikokan Municipal Airport",
		City:      "Atikokan",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1408,
		Latitude:  48.7738990784,
		Longitude: -91.6386032104,
		Timezone:  "America/Atikokan",
	},
	"CYID": {
		ICAO:      "CYID",
		IATA:      "YDG",
		Name:      "Digby / Annapolis Regional Airport",
		City:      "Digby",
		State:     "Nova Scotia",
		Country:   "CA",
		Elevation: 499,
		Latitude:  44.5458450365,
		Longitude: -65.7854247093,
		Timezone:  "America/Halifax",
	},
	"CYIF": {
		ICAO:      "CYIF",
		IATA:      "YIF",
		Name:      "St Augustin Airport",
		City:      "St-Augustin",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 20,
		Latitude:  51.2117004395,
		Longitude: -58.6582984924,
		Timezone:  "America/Blanc-Sablon",
	},
	"CYIK": {
		ICAO:      "CYIK",
		IATA:      "YIK",
		Name:      "Ivujivik Airport",
		City:      "Ivujivik",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 126,
		Latitude:  62.417301178,
		Longitude: -77.9253005981,
		Timezone:  "America/Iqaluit",
	},
	"CYIO": {
		ICAO:      "CYIO",
		IATA:      "YIO",
		Name:      "Pond Inlet Airport",
		City:      "Pond Inlet",
		State:     "Nunavut",
		Country:   "CA",
		Elevation: 181,
		Latitude:  72.6832962036,
		Longitude: -77.9666976929,
		Timezone:  "America/Iqaluit",
	},
	"CYIV": {
		ICAO:      "CYIV",
		IATA:      "YIV",
		Name:      "Island Lake Airport",
		City:      "Island Lake",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 770,
		Latitude:  53.8572006226,
		Longitude: -94.6536026001,
		Timezone:  "America/Winnipeg",
	},
	"CYJA": {
		ICAO:      "CYJA",
		IATA:      "YJA",
		Name:      "Jasper Airport",
		City:      "Jasper",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3350,
		Latitude:  52.9967002869,
		Longitude: -118.0589981079,
		Timezone:  "America/Edmonton",
	},
	"CYJF": {
		ICAO:      "CYJF",
		IATA:      "YJF",
		Name:      "Fort Liard Airport",
		City:      "Fort Liard",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 708,
		Latitude:  60.2358016968,
		Longitude: -123.46900177,
		Timezone:  "America/Inuvik",
	},
	"CYJM": {
		ICAO:      "CYJM",
		Name:      "Fort St James Airport",
		City:      "Fort St. James",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2364,
		Latitude:  54.3972015381,
		Longitude: -124.2630004883,
		Timezone:  "America/Vancouver",
	},
	"CYJN": {
		ICAO:      "CYJN",
		IATA:      "YJN",
		Name:      "St Jean Airport",
		City:      "St Jean",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 136,
		Latitude:  45.2943992615,
		Longitude: -73.2810974121,
		Timezone:  "America/Toronto",
	},
	"CYJP": {
		ICAO:      "CYJP",
		Name:      "Fort Providence Airport",
		City:      "Fort Providence",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 524,
		Latitude:  61.3194007874,
		Longitude: -117.6060028076,
		Timezone:  "America/Yellowknife",
	},
	"CYJQ": {
		ICAO:      "CYJQ",
		Name:      "Denny Island Airport",
		City:      "Bella Bella",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 162,
		Latitude:  52.1397018433,
		Longitude: -128.063995361,
		Timezone:  "America/Vancouver",
	},
	"CYJT": {
		ICAO:      "CYJT",
		IATA:      "YJT",
		Name:      "Stephenville Airport",
		City:      "Stephenville",
		State:     "Newfoundland and Labrador",
		Country:   "CA",
		Elevation: 84,
		Latitude:  48.5442008972,
		Longitude: -58.5499992371,
		Timezone:  "America/St_Johns",
	},
	"CYKA": {
		ICAO:      "CYKA",
		IATA:      "YKA",
		Name:      "Kamloops Airport",
		City:      "Kamloops",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 1133,
		Latitude:  50.7022018433,
		Longitude: -120.4440002441,
		Timezone:  "America/Vancouver",
	},
	"CYKC": {
		ICAO:      "CYKC",
		IATA:      "YKC",
		Name:      "Collins Bay Airport",
		City:      "Collins Bay",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1341,
		Latitude:  58.2360992432,
		Longitude: -103.6780014038,
		Timezone:  "America/Regina",
	},
	"CYKD": {
		ICAO:      "CYKD",
		IATA:      "LAK",
		Name:      "Aklavik Airport",
		City:      "Aklavik",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 23,
		Latitude:  68.2232971191,
		Longitude: -135.0059967041,
		Timezone:  "America/Inuvik",
	},
	"CYKF": {
		ICAO:      "CYKF",
		IATA:      "YKF",
		Name:      "Waterloo Airport",
		City:      "Kitchener",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1055,
		Latitude:  43.4608001709,
		Longitude: -80.3786010742,
		Timezone:  "America/Toronto",
	},
	"CYKG": {
		ICAO:      "CYKG",
		IATA:      "YWB",
		Name:      "Kangiqsujuaq (Wakeham Bay) Airport",
		City:      "Kangiqsujuaq",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 501,
		Latitude:  61.5886001587,
		Longitude: -71.929397583,
		Timezone:  "America/Toronto",
	},
	"CYKJ": {
		ICAO:      "CYKJ",
		IATA:      "YKJ",
		Name:      "Key Lake Airport",
		City:      "Key Lake",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1679,
		Latitude:  57.2560997009,
		Longitude: -105.6179962158,
		Timezone:  "America/Regina",
	},
	"CYKL": {
		ICAO:      "CYKL",
		IATA:      "YKL",
		Name:      "Schefferville Airport",
		City:      "Schefferville",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 1709,
		Latitude:  54.8053016663,
		Longitude: -66.8052978516,
		Timezone:  "America/Toronto",
	},
	"CYKO": {
		ICAO:      "CYKO",
		IATA:      "AKV",
		Name:      "Akulivik Airport",
		City:      "Akulivik",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 75,
		Latitude:  60.8185997009,
		Longitude: -78.1485977173,
		Timezone:  "America/Iqaluit",
	},
	"CYKQ": {
		ICAO:      "CYKQ",
		IATA:      "YKQ",
		Name:      "Waskaganish Airport",
		City:      "Waskaganish",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 80,
		Latitude:  51.4733009338,
		Longitude: -78.7583007813,
		Timezone:  "America/Toronto",
	},
	"CYKX": {
		ICAO:      "CYKX",
		IATA:      "YKX",
		Name:      "Kirkland Lake Airport",
		City:      "Kirkland Lake",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1157,
		Latitude:  48.2103004456,
		Longitude: -79.9813995361,
		Timezone:  "America/Toronto",
	},
	"CYKY": {
		ICAO:      "CYKY",
		IATA:      "YKY",
		Name:      "Kindersley Airport",
		City:      "Kindersley",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2277,
		Latitude:  51.5175018311,
		Longitude: -109.180999756,
		Timezone:  "America/Swift_Current",
	},
	"CYKZ": {
		ICAO:      "CYKZ",
		IATA:      "YKZ",
		Name:      "Buttonville Municipal Airport",
		City:      "Toronto",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 650,
		Latitude:  43.8622016907,
		Longitude: -79.3700027466,
		Timezone:  "America/Toronto",
	},
	"CYLA": {
		ICAO:      "CYLA",
		IATA:      "YPJ",
		Name:      "Aupaluk Airport",
		City:      "Aupaluk",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 119,
		Latitude:  59.2966995239,
		Longitude: -69.5997009277,
		Timezone:  "America/Toronto",
	},
	"CYLB": {
		ICAO:      "CYLB",
		IATA:      "YLB",
		Name:      "Lac La Biche Airport",
		City:      "Lac La Biche",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 1884,
		Latitude:  54.7703018188,
		Longitude: -112.0319976807,
		Timezone:  "America/Edmonton",
	},
	"CYLC": {
		ICAO:      "CYLC",
		IATA:      "YLC",
		Name:      "Kimmirut Airport",
		City:      "Kimmirut",
		State:     "Nunavut",
		Country:   "CA",
		Elevation: 175,
		Latitude:  62.8499984741,
		Longitude: -69.8833007812,
		Timezone:  "America/Iqaluit",
	},
	"CYLD": {
		ICAO:      "CYLD",
		IATA:      "YLD",
		Name:      "Chapleau Airport",
		City:      "Chapleau",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1470,
		Latitude:  47.8199996948,
		Longitude: -83.3467025757,
		Timezone:  "America/Toronto",
	},
	"CYLH": {
		ICAO:      "CYLH",
		IATA:      "YLH",
		Name:      "Lansdowne House Airport",
		City:      "Lansdowne House",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 834,
		Latitude:  52.1955986023,
		Longitude: -87.9341964722,
		Timezone:  "America/Nipigon",
	},
	"CYLI": {
		ICAO:      "CYLI",
		Name:      "Lillooet Airport",
		City:      "Lillooet",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 1320,
		Latitude:  50.6747016907,
		Longitude: -121.893997192,
		Timezone:  "America/Vancouver",
	},
	"CYLJ": {
		ICAO:      "CYLJ",
		IATA:      "YLJ",
		Name:      "Meadow Lake Airport",
		City:      "Meadow Lake",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1576,
		Latitude:  54.1253013611,
		Longitude: -108.5230026245,
		Timezone:  "America/Swift_Current",
	},
	"CYLK": {
		ICAO:      "CYLK",
		IATA:      "YSG",
		Name:      "Lutselk'e Airport",
		City:      "Lutselk'e",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 596,
		Latitude:  62.4183006287,
		Longitude: -110.6819992065,
		Timezone:  "America/Yellowknife",
	},
	"CYLL": {
		ICAO:      "CYLL",
		IATA:      "YLL",
		Name:      "Lloydminster Airport",
		City:      "Lloydminster",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2193,
		Latitude:  53.3092002869,
		Longitude: -110.0729980469,
		Timezone:  "America/Edmonton",
	},
	"CYLQ": {
		ICAO:      "CYLQ",
		IATA:      "YLQ",
		Name:      "La Tuque Airport",
		City:      "La Tuque",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 548,
		Latitude:  47.4096984863,
		Longitude: -72.7889022827,
		Timezone:  "America/Toronto",
	},
	"CYLR": {
		ICAO:      "CYLR",
		IATA:      "YLR",
		Name:      "Leaf Rapids Airport",
		City:      "Leaf Rapids",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 959,
		Latitude:  56.5133018494,
		Longitude: -99.9852981567,
		Timezone:  "America/Winnipeg",
	},
	"CYLS": {
		ICAO:      "CYLS",
		IATA:      "YLK",
		Name:      "Barrie-Orillia (Lake Simcoe Regional Airport)",
		City:      "Barrie-Orillia",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 972,
		Latitude:  44.4852981567,
		Longitude: -79.5556030273,
		Timezone:  "America/Toronto",
	},
	"CYLT": {
		ICAO:      "CYLT",
		IATA:      "YLT",
		Name:      "Alert Airport",
		City:      "Alert",
		State:     "Nunavut",
		Country:   "CA",
		Elevation: 100,
		Latitude:  82.5177993774,
		Longitude: -62.2806015015,
		Timezone:  "America/Pangnirtung",
	},
	"CYLU": {
		ICAO:      "CYLU",
		IATA:      "XGR",
		Name:      "Kangiqsualujjuaq (Georges River) Airport",
		City:      "Kangiqsualujjuaq",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 215,
		Latitude:  58.7113990784,
		Longitude: -65.9927978516,
		Timezone:  "America/Toronto",
	},
	"CYLW": {
		ICAO:      "CYLW",
		IATA:      "YLW",
		Name:      "Kelowna Airport",
		City:      "Kelowna",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 1409,
		Latitude:  49.9561004639,
		Longitude: -119.3779983521,
		Timezone:  "America/Vancouver",
	},
	"CYMA": {
		ICAO:      "CYMA",
		IATA:      "YMA",
		Name:      "Mayo Airport",
		City:      "Mayo",
		State:     "Yukon",
		Country:   "CA",
		Elevation: 1653,
		Latitude:  63.6164016724,
		Longitude: -135.8679962158,
		Timezone:  "America/Whitehorse",
	},
	"CYME": {
		ICAO:      "CYME",
		IATA:      "YME",
		Name:      "Matane Airport",
		City:      "Matane",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 102,
		Latitude:  48.8568992615,
		Longitude: -67.4533004761,
		Timezone:  "America/Toronto",
	},
	"CYMG": {
		ICAO:      "CYMG",
		IATA:      "YMG",
		Name:      "Manitouwadge Airport",
		City:      "Manitouwadge",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1198,
		Latitude:  49.0839004517,
		Longitude: -85.8606033325,
		Timezone:  "America/Nipigon",
	},
	"CYMH": {
		ICAO:      "CYMH",
		IATA:      "YMH",
		Name:      "Mary's Harbour Airport",
		City:      "Mary's Harbour",
		State:     "Newfoundland and Labrador",
		Country:   "CA",
		Elevation: 38,
		Latitude:  52.3027992249,
		Longitude: -55.8471984863,
		Timezone:  "America/St_Johns",
	},
	"CYMJ": {
		ICAO:      "CYMJ",
		IATA:      "YMJ",
		Name:      "Moose Jaw Air Vice Marshal C. M. McEwen Airport",
		City:      "Moose Jaw",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1892,
		Latitude:  50.3302993774,
		Longitude: -105.5589981079,
		Timezone:  "America/Regina",
	},
	"CYML": {
		ICAO:      "CYML",
		IATA:      "YML",
		Name:      "Charlevoix Airport",
		City:      "Charlevoix",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 977,
		Latitude:  47.5974998474,
		Longitude: -70.2238998413,
		Timezone:  "America/Toronto",
	},
	"CYMM": {
		ICAO:      "CYMM",
		IATA:      "YMM",
		Name:      "Fort McMurray Airport",
		City:      "Fort McMurray",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 1211,
		Latitude:  56.653301239,
		Longitude: -111.222000122,
		Timezone:  "America/Edmonton",
	},
	"CYMO": {
		ICAO:      "CYMO",
		IATA:      "YMO",
		Name:      "Moosonee Airport",
		City:      "Moosonee",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 30,
		Latitude:  51.2910995483,
		Longitude: -80.6078033447,
		Timezone:  "America/Nipigon",
	},
	"CYMT": {
		ICAO:      "CYMT",
		IATA:      "YMT",
		Name:      "Chapais Airport",
		City:      "Chibougamau",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 1270,
		Latitude:  49.771900177,
		Longitude: -74.5280990601,
		Timezone:  "America/Toronto",
	},
	"CYMU": {
		ICAO:      "CYMU",
		IATA:      "YUD",
		Name:      "Umiujaq Airport",
		City:      "Umiujaq",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 250,
		Latitude:  56.5360984802,
		Longitude: -76.5183029175,
		Timezone:  "America/Iqaluit",
	},
	"CYMW": {
		ICAO:      "CYMW",
		IATA:      "YMW",
		Name:      "Maniwaki Airport",
		City:      "Maniwaki",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 656,
		Latitude:  46.2728004456,
		Longitude: -75.9906005859,
		Timezone:  "America/Toronto",
	},
	"CYMX": {
		ICAO:      "CYMX",
		IATA:      "YMX",
		Name:      "Montreal International (Mirabel) Airport",
		City:      "Montreal",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 270,
		Latitude:  45.6795005798,
		Longitude: -74.0386962891,
		Timezone:  "America/Toronto",
	},
	"CYMY": {
		ICAO:      "CYMY",
		Name:      "Ear Falls Airport",
		City:      "Ear Falls",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1269,
		Latitude:  50.7178001404,
		Longitude: -93.3835983276,
		Timezone:  "America/Rainy_River",
	},
	"CYNA": {
		ICAO:      "CYNA",
		IATA:      "YNA",
		Name:      "Natashquan Airport",
		City:      "Natashquan",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 39,
		Latitude:  50.1899986267,
		Longitude: -61.7891998291,
		Timezone:  "America/Toronto",
	},
	"CYNC": {
		ICAO:      "CYNC",
		IATA:      "YNC",
		Name:      "Wemindji Airport",
		City:      "Wemindji",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 66,
		Latitude:  53.0106010437,
		Longitude: -78.8311004639,
		Timezone:  "America/Toronto",
	},
	"CYND": {
		ICAO:      "CYND",
		IATA:      "YND",
		Name:      "Ottawa / Gatineau Airport",
		City:      "Gatineau",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 211,
		Latitude:  45.5217018127,
		Longitude: -75.5635986328,
		Timezone:  "America/Toronto",
	},
	"CYNE": {
		ICAO:      "CYNE",
		IATA:      "YNE",
		Name:      "Norway House Airport",
		City:      "Norway House",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 734,
		Latitude:  53.9583015442,
		Longitude: -97.8442001343,
		Timezone:  "America/Winnipeg",
	},
	"CYNH": {
		ICAO:      "CYNH",
		IATA:      "YNH",
		Name:      "Hudsons Hope Airport",
		City:      "Hudson's Hope",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2220,
		Latitude:  56.0355987549,
		Longitude: -121.9759979248,
		Timezone:  "America/Dawson_Creek",
	},
	"CYNJ": {
		ICAO:      "CYNJ",
		IATA:      "YLY",
		Name:      "Langley Airport",
		City:      "Langley",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 34,
		Latitude:  49.1007995605,
		Longitude: -122.6309967041,
		Timezone:  "America/Vancouver",
	},
	"CYNL": {
		ICAO:      "CYNL",
		IATA:      "YNL",
		Name:      "Points North Landing Airport",
		City:      "Points North Landing",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1605,
		Latitude:  58.2766990662,
		Longitude: -104.0820007324,
		Timezone:  "America/Regina",
	},
	"CYNM": {
		ICAO:      "CYNM",
		IATA:      "YNM",
		Name:      "Matagami Airport",
		City:      "Matagami",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 918,
		Latitude:  49.7616996765,
		Longitude: -77.8028030396,
		Timezone:  "America/Toronto",
	},
	"CYNN": {
		ICAO:      "CYNN",
		IATA:      "YNN",
		Name:      "Nejanilini Lake Airport",
		City:      "Nejanilini Lake",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 1000,
		Latitude:  59.4874992371,
		Longitude: -97.7802963257,
		Timezone:  "America/Winnipeg",
	},
	"CYNR": {
		ICAO:      "CYNR",
		IATA:      "HZP",
		Name:      "Fort Mackay / Horizon Airport",
		City:      "Fort Mackay",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 916,
		Latitude:  57.3816986084,
		Longitude: -111.700996399,
		Timezone:  "America/Edmonton",
	},
	"CYOA": {
		ICAO:      "CYOA",
		IATA:      "YOA",
		Name:      "Ekati Airport",
		City:      "Ekati",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 1536,
		Latitude:  64.6988983154,
		Longitude: -110.614997864,
		Timezone:  "America/Yellowknife",
	},
	"CYOC": {
		ICAO:      "CYOC",
		IATA:      "YOC",
		Name:      "Old Crow Airport",
		City:      "Old Crow",
		State:     "Yukon",
		Country:   "CA",
		Elevation: 824,
		Latitude:  67.570602417,
		Longitude: -139.8390045166,
		Timezone:  "America/Dawson",
	},
	"CYOD": {
		ICAO:      "CYOD",
		IATA:      "YOD",
		Name:      "CFB Cold Lake",
		City:      "Cold Lake",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 1775,
		Latitude:  54.4049987793,
		Longitude: -110.2789993286,
		Timezone:  "America/Edmonton",
	},
	"CYOH": {
		ICAO:      "CYOH",
		IATA:      "YOH",
		Name:      "Oxford House Airport",
		City:      "Oxford House",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 663,
		Latitude:  54.9333000183,
		Longitude: -95.2789001465,
		Timezone:  "America/Winnipeg",
	},
	"CYOJ": {
		ICAO:      "CYOJ",
		IATA:      "YOJ",
		Name:      "High Level Airport",
		City:      "High Level",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 1110,
		Latitude:  58.6213989258,
		Longitude: -117.1650009155,
		Timezone:  "America/Edmonton",
	},
	"CYOO": {
		ICAO:      "CYOO",
		IATA:      "YOO",
		Name:      "Oshawa Airport",
		City:      "Oshawa",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 460,
		Latitude:  43.9227981567,
		Longitude: -78.8949966431,
		Timezone:  "America/Toronto",
	},
	"CYOP": {
		ICAO:      "CYOP",
		IATA:      "YOP",
		Name:      "Rainbow Lake Airport",
		City:      "Rainbow Lake",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 1759,
		Latitude:  58.4914016724,
		Longitude: -119.4079971313,
		Timezone:  "America/Edmonton",
	},
	"CYOS": {
		ICAO:      "CYOS",
		IATA:      "YOS",
		Name:      "Owen Sound / Billy Bishop Regional Airport",
		City:      "Owen Sound",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1007,
		Latitude:  44.5903015137,
		Longitude: -80.8375015259,
		Timezone:  "America/Toronto",
	},
	"CYOW": {
		ICAO:      "CYOW",
		IATA:      "YOW",
		Name:      "Ottawa Macdonald-Cartier International Airport",
		City:      "Ottawa",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 374,
		Latitude:  45.3224983215,
		Longitude: -75.6691970825,
		Timezone:  "America/Toronto",
	},
	"CYPA": {
		ICAO:      "CYPA",
		IATA:      "YPA",
		Name:      "Prince Albert Glass Field",
		City:      "Prince Albert",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1405,
		Latitude:  53.2141990662,
		Longitude: -105.672996521,
		Timezone:  "America/Regina",
	},
	"CYPC": {
		ICAO:      "CYPC",
		IATA:      "YPC",
		Name:      "Paulatuk (Nora Aliqatchialuk Ruben) Airport",
		City:      "Paulatuk",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 15,
		Latitude:  69.3608381154,
		Longitude: -124.075469971,
		Timezone:  "America/Inuvik",
	},
	"CYPD": {
		ICAO:      "CYPD",
		IATA:      "YPS",
		Name:      "Port Hawkesbury Airport",
		City:      "Port Hawkesbury",
		State:     "Nova Scotia",
		Country:   "CA",
		Elevation: 377,
		Latitude:  45.6567001343,
		Longitude: -61.3680992126,
		Timezone:  "America/Halifax",
	},
	"CYPE": {
		ICAO:      "CYPE",
		IATA:      "YPE",
		Name:      "Peace River Airport",
		City:      "Peace River",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 1873,
		Latitude:  56.2268981934,
		Longitude: -117.4469985962,
		Timezone:  "America/Edmonton",
	},
	"CYPG": {
		ICAO:      "CYPG",
		IATA:      "YPG",
		Name:      "Southport Airport",
		City:      "Portage",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 885,
		Latitude:  49.9030990601,
		Longitude: -98.2739028931,
		Timezone:  "America/Winnipeg",
	},
	"CYPH": {
		ICAO:      "CYPH",
		IATA:      "YPH",
		Name:      "Inukjuak Airport",
		City:      "Inukjuak",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 83,
		Latitude:  58.4719009399,
		Longitude: -78.0768966675,
		Timezone:  "America/Toronto",
	},
	"CYPK": {
		ICAO:      "CYPK",
		Name:      "Pitt Meadows Airport",
		City:      "Pitt Meadows",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 11,
		Latitude:  49.2160987854,
		Longitude: -122.7099990845,
		Timezone:  "America/Vancouver",
	},
	"CYPL": {
		ICAO:      "CYPL",
		IATA:      "YPL",
		Name:      "Pickle Lake Airport",
		City:      "Pickle Lake",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1267,
		Latitude:  51.4463996887,
		Longitude: -90.2142028809,
		Timezone:  "America/Atikokan",
	},
	"CYPM": {
		ICAO:      "CYPM",
		IATA:      "YPM",
		Name:      "Pikangikum Airport",
		City:      "Pikangikum",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1114,
		Latitude:  51.8196983337,
		Longitude: -93.9732971191,
		Timezone:  "America/Rainy_River",
	},
	"CYPN": {
		ICAO:      "CYPN",
		IATA:      "YPN",
		Name:      "Port Menier Airport",
		City:      "Port-Menier",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 167,
		Latitude:  49.8363990784,
		Longitude: -64.2885971069,
		Timezone:  "America/Toronto",
	},
	"CYPO": {
		ICAO:      "CYPO",
		IATA:      "YPO",
		Name:      "Peawanuck Airport",
		City:      "Peawanuck",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 173,
		Latitude:  54.9880981445,
		Longitude: -85.4432983398,
		Timezone:  "America/Nipigon",
	},
	"CYPP": {
		ICAO:      "CYPP",
		Name:      "Parent Airport",
		City:      "Parent",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 1400,
		Latitude:  47.9319000244,
		Longitude: -74.6081008911,
		Timezone:  "America/Toronto",
	},
	"CYPQ": {
		ICAO:      "CYPQ",
		IATA:      "YPQ",
		Name:      "Peterborough Airport",
		City:      "Peterborough",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 628,
		Latitude:  44.2299995422,
		Longitude: -78.3632965088,
		Timezone:  "America/Toronto",
	},
	"CYPR": {
		ICAO:      "CYPR",
		IATA:      "YPR",
		Name:      "Prince Rupert Airport",
		City:      "Prince Rupert",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 116,
		Latitude:  54.2860984802,
		Longitude: -130.4450073242,
		Timezone:  "America/Vancouver",
	},
	"CYPS": {
		ICAO:      "CYPS",
		Name:      "Pemberton Airport",
		City:      "Pemberton",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 670,
		Latitude:  50.3025016785,
		Longitude: -122.7379989624,
		Timezone:  "America/Vancouver",
	},
	"CYPT": {
		ICAO:      "CYPT",
		Name:      "Pelee Island Airport",
		City:      "Pelee Island",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 572,
		Latitude:  41.7803993225,
		Longitude: -82.6780014038,
		Timezone:  "America/Toronto",
	},
	"CYPU": {
		ICAO:      "CYPU",
		Name:      "Puntzi Mountain Airport",
		City:      "Puntzi Mountain",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2985,
		Latitude:  52.1128005981,
		Longitude: -124.1449966431,
		Timezone:  "America/Vancouver",
	},
	"CYPW": {
		ICAO:      "CYPW",
		IATA:      "YPW",
		Name:      "Powell River Airport",
		City:      "Powell River",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 425,
		Latitude:  49.8342018127,
		Longitude: -124.5,
		Timezone:  "America/Vancouver",
	},
	"CYPX": {
		ICAO:      "CYPX",
		IATA:      "YPX",
		Name:      "Puvirnituq Airport",
		City:      "Puvirnituq",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 74,
		Latitude:  60.0505981445,
		Longitude: -77.2869033813,
		Timezone:  "America/Toronto",
	},
	"CYPY": {
		ICAO:      "CYPY",
		IATA:      "YPY",
		Name:      "Fort Chipewyan Airport",
		City:      "Fort Chipewyan",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 761,
		Latitude:  58.76720047,
		Longitude: -111.1169967651,
		Timezone:  "America/Edmonton",
	},
	"CYPZ": {
		ICAO:      "CYPZ",
		IATA:      "YPZ",
		Name:      "Burns Lake Airport",
		City:      "Burns Lake",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2343,
		Latitude:  54.3763999939,
		Longitude: -125.9509963989,
		Timezone:  "America/Vancouver",
	},
	"CYQA": {
		ICAO:      "CYQA",
		IATA:      "YQA",
		Name:      "Muskoka Airport",
		City:      "Muskoka",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 925,
		Latitude:  44.9747009277,
		Longitude: -79.3032989502,
		Timezone:  "America/Toronto",
	},
	"CYQB": {
		ICAO:      "CYQB",
		IATA:      "YQB",
		Name:      "Quebec Jean Lesage International Airport",
		City:      "Quebec",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 244,
		Latitude:  46.7910995483,
		Longitude: -71.3933029175,
		Timezone:  "America/Toronto",
	},
	"CYQD": {
		ICAO:      "CYQD",
		IATA:      "YQD",
		Name:      "The Pas Airport",
		City:      "The Pas",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 887,
		Latitude:  53.9714012146,
		Longitude: -101.091003418,
		Timezone:  "America/Winnipeg",
	},
	"CYQF": {
		ICAO:      "CYQF",
		IATA:      "YQF",
		Name:      "Red Deer Regional Airport",
		City:      "Red Deer",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2968,
		Latitude:  52.1822013855,
		Longitude: -113.8939971924,
		Timezone:  "America/Edmonton",
	},
	"CYQG": {
		ICAO:      "CYQG",
		IATA:      "YQG",
		Name:      "Windsor Airport",
		City:      "Windsor",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 622,
		Latitude:  42.2756004333,
		Longitude: -82.9555969238,
		Timezone:  "America/Toronto",
	},
	"CYQH": {
		ICAO:      "CYQH",
		IATA:      "YQH",
		Name:      "Watson Lake Airport",
		City:      "Watson Lake",
		State:     "Yukon",
		Country:   "CA",
		Elevation: 2255,
		Latitude:  60.1164016724,
		Longitude: -128.8220062256,
		Timezone:  "America/Whitehorse",
	},
	"CYQI": {
		ICAO:      "CYQI",
		IATA:      "YQI",
		Name:      "Yarmouth Airport",
		City:      "Yarmouth",
		State:     "Nova Scotia",
		Country:   "CA",
		Elevation: 141,
		Latitude:  43.8269004822,
		Longitude: -66.0880966187,
		Timezone:  "America/Halifax",
	},
	"CYQK": {
		ICAO:      "CYQK",
		IATA:      "YQK",
		Name:      "Kenora Airport",
		City:      "Kenora",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1332,
		Latitude:  49.7882995605,
		Longitude: -94.3630981445,
		Timezone:  "America/Rainy_River",
	},
	"CYQL": {
		ICAO:      "CYQL",
		IATA:      "YQL",
		Name:      "Lethbridge County Airport",
		City:      "Lethbridge",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3048,
		Latitude:  49.6302986145,
		Longitude: -112.800003052,
		Timezone:  "America/Edmonton",
	},
	"CYQM": {
		ICAO:      "CYQM",
		IATA:      "YQM",
		Name:      "Greater Moncton International Airport",
		City:      "Moncton",
		State:     "New Brunswick",
		Country:   "CA",
		Elevation: 232,
		Latitude:  46.1122016907,
		Longitude: -64.6785964966,
		Timezone:  "America/Moncton",
	},
	"CYQN": {
		ICAO:      "CYQN",
		IATA:      "YQN",
		Name:      "Nakina Airport",
		City:      "Nakina",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1057,
		Latitude:  50.182800293,
		Longitude: -86.6964035034,
		Timezone:  "America/Nipigon",
	},
	"CYQQ": {
		ICAO:      "CYQQ",
		IATA:      "YQQ",
		Name:      "Comox Airport",
		City:      "Comox",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 84,
		Latitude:  49.7108001709,
		Longitude: -124.8870010376,
		Timezone:  "America/Vancouver",
	},
	"CYQR": {
		ICAO:      "CYQR",
		IATA:      "YQR",
		Name:      "Regina International Airport",
		City:      "Regina",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1894,
		Latitude:  50.4319000244,
		Longitude: -104.6660003662,
		Timezone:  "America/Regina",
	},
	"CYQS": {
		ICAO:      "CYQS",
		IATA:      "YQS",
		Name:      "St Thomas Municipal Airport",
		City:      "St Thomas",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 778,
		Latitude:  42.7700004578,
		Longitude: -81.1108016968,
		Timezone:  "America/Toronto",
	},
	"CYQT": {
		ICAO:      "CYQT",
		IATA:      "YQT",
		Name:      "Thunder Bay Airport",
		City:      "Thunder Bay",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 653,
		Latitude:  48.3718986511,
		Longitude: -89.3238983154,
		Timezone:  "America/Thunder_Bay",
	},
	"CYQU": {
		ICAO:      "CYQU",
		IATA:      "YQU",
		Name:      "Grande Prairie Airport",
		City:      "Grande Prairie",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2195,
		Latitude:  55.1796989441,
		Longitude: -118.885002136,
		Timezone:  "America/Edmonton",
	},
	"CYQV": {
		ICAO:      "CYQV",
		IATA:      "YQV",
		Name:      "Yorkton Municipal Airport",
		City:      "Yorkton",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1635,
		Latitude:  51.2647018433,
		Longitude: -102.4619979858,
		Timezone:  "America/Regina",
	},
	"CYQW": {
		ICAO:      "CYQW",
		IATA:      "YQW",
		Name:      "North Battleford Airport",
		City:      "North Battleford",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1799,
		Latitude:  52.7691993713,
		Longitude: -108.2440032959,
		Timezone:  "America/Swift_Current",
	},
	"CYQX": {
		ICAO:      "CYQX",
		IATA:      "YQX",
		Name:      "Gander International Airport",
		City:      "Gander",
		State:     "Newfoundland and Labrador",
		Country:   "CA",
		Elevation: 496,
		Latitude:  48.9369010925,
		Longitude: -54.5680999756,
		Timezone:  "America/St_Johns",
	},
	"CYQY": {
		ICAO:      "CYQY",
		IATA:      "YQY",
		Name:      "Sydney / J.A. Douglas McCurdy Airport",
		City:      "Sydney",
		State:     "Nova Scotia",
		Country:   "CA",
		Elevation: 203,
		Latitude:  46.1613998413,
		Longitude: -60.0477981567,
		Timezone:  "America/Glace_Bay",
	},
	"CYQZ": {
		ICAO:      "CYQZ",
		IATA:      "YQZ",
		Name:      "Quesnel Airport",
		City:      "Quesnel",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 1789,
		Latitude:  53.0261001587,
		Longitude: -122.5100021362,
		Timezone:  "America/Vancouver",
	},
	"CYRA": {
		ICAO:      "CYRA",
		IATA:      "YRA",
		Name:      "Rae Lakes Airport",
		City:      "Gameti",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 723,
		Latitude:  64.1160964966,
		Longitude: -117.3099975586,
		Timezone:  "America/Yellowknife",
	},
	"CYRB": {
		ICAO:      "CYRB",
		IATA:      "YRB",
		Name:      "Resolute Bay Airport",
		City:      "Resolute Bay",
		State:     "Nunavut",
		Country:   "CA",
		Elevation: 215,
		Latitude:  74.7169036865,
		Longitude: -94.9693984985,
		Timezone:  "America/Resolute",
	},
	"CYRC": {
		ICAO:      "CYRC",
		Name:      "Chicoutimi St Honore Airport",
		City:      "Chicoutimi",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 543,
		Latitude:  48.5208015442,
		Longitude: -71.0505981445,
		Timezone:  "America/Toronto",
	},
	"CYRI": {
		ICAO:      "CYRI",
		IATA:      "YRI",
		Name:      "Riviere-du-Loup Airport",
		City:      "Riviere-du-Loup",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 427,
		Latitude:  47.7644004822,
		Longitude: -69.5847015381,
		Timezone:  "America/Toronto",
	},
	"CYRJ": {
		ICAO:      "CYRJ",
		IATA:      "YRJ",
		Name:      "Roberval Airport",
		City:      "Roberval",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 586,
		Latitude:  48.5200004578,
		Longitude: -72.2656021118,
		Timezone:  "America/Toronto",
	},
	"CYRL": {
		ICAO:      "CYRL",
		IATA:      "YRL",
		Name:      "Red Lake Airport",
		City:      "Red Lake",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1265,
		Latitude:  51.0668983459,
		Longitude: -93.7930984497,
		Timezone:  "America/Rainy_River",
	},
	"CYRM": {
		ICAO:      "CYRM",
		IATA:      "YRM",
		Name:      "Rocky Mountain House Airport",
		City:      "Rocky Mountain House",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3244,
		Latitude:  52.4296989441,
		Longitude: -114.9039993286,
		Timezone:  "America/Edmonton",
	},
	"CYRO": {
		ICAO:      "CYRO",
		IATA:      "YRO",
		Name:      "Ottawa / Rockcliffe Airport",
		City:      "Ottawa",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 188,
		Latitude:  45.4603004456,
		Longitude: -75.6461029053,
		Timezone:  "America/Toronto",
	},
	"CYRP": {
		ICAO:      "CYRP",
		IATA:      "YRP",
		Name:      "Ottawa / Carp Airport",
		City:      "Carp",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 382,
		Latitude:  45.3191986084,
		Longitude: -76.0222015381,
		Timezone:  "America/Toronto",
	},
	"CYRQ": {
		ICAO:      "CYRQ",
		IATA:      "YRQ",
		Name:      "Trois-Rivieres Airport",
		City:      "Trois-Rivieres",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 199,
		Latitude:  46.3527984619,
		Longitude: -72.679397583,
		Timezone:  "America/Toronto",
	},
	"CYRS": {
		ICAO:      "CYRS",
		IATA:      "YRS",
		Name:      "Red Sucker Lake Airport",
		City:      "Red Sucker Lake",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 729,
		Latitude:  54.1671981812,
		Longitude: -93.5571975708,
		Timezone:  "America/Winnipeg",
	},
	"CYRT": {
		ICAO:      "CYRT",
		IATA:      "YRT",
		Name:      "Rankin Inlet Airport",
		City:      "Rankin Inlet",
		State:     "Nunavut",
		Country:   "CA",
		Elevation: 94,
		Latitude:  62.8114013672,
		Longitude: -92.1157989502,
		Timezone:  "America/Rankin_Inlet",
	},
	"CYRV": {
		ICAO:      "CYRV",
		IATA:      "YRV",
		Name:      "Revelstoke Airport",
		City:      "Revelstoke",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 1459,
		Latitude:  50.9667015076,
		Longitude: -118.182998657,
		Timezone:  "America/Vancouver",
	},
	"CYSA": {
		ICAO:      "CYSA",
		Name:      "Stratford Municipal Airport",
		City:      "Stratford",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1215,
		Latitude:  43.415599823,
		Longitude: -80.9344024658,
		Timezone:  "America/Toronto",
	},
	"CYSB": {
		ICAO:      "CYSB",
		IATA:      "YSB",
		Name:      "Sudbury Airport",
		City:      "Sudbury",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1141,
		Latitude:  46.625,
		Longitude: -80.7988967896,
		Timezone:  "America/Toronto",
	},
	"CYSC": {
		ICAO:      "CYSC",
		IATA:      "YSC",
		Name:      "Sherbrooke Airport",
		City:      "Sherbrooke",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 792,
		Latitude:  45.4385986328,
		Longitude: -71.6913986206,
		Timezone:  "America/Toronto",
	},
	"CYSE": {
		ICAO:      "CYSE",
		IATA:      "YSE",
		Name:      "Squamish Airport",
		City:      "Squamish",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 171,
		Latitude:  49.7817001343,
		Longitude: -123.1620025635,
		Timezone:  "America/Vancouver",
	},
	"CYSF": {
		ICAO:      "CYSF",
		IATA:      "YSF",
		Name:      "Stony Rapids Airport",
		City:      "Stony Rapids",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 805,
		Latitude:  59.2503013611,
		Longitude: -105.841003418,
		Timezone:  "America/Regina",
	},
	"CYSG": {
		ICAO:      "CYSG",
		Name:      "St Georges Airport",
		City:      "St Georges",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 893,
		Latitude:  46.0964012146,
		Longitude: -70.7146987915,
		Timezone:  "America/Toronto",
	},
	"CYSH": {
		ICAO:      "CYSH",
		IATA:      "YSH",
		Name:      "Smiths Falls-Montague (Russ Beach) Airport",
		City:      "Smiths Falls",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 416,
		Latitude:  44.9458007813,
		Longitude: -75.9405975342,
		Timezone:  "America/Toronto",
	},
	"CYSJ": {
		ICAO:      "CYSJ",
		IATA:      "YSJ",
		Name:      "Saint John Airport",
		City:      "Saint John",
		State:     "New Brunswick",
		Country:   "CA",
		Elevation: 357,
		Latitude:  45.3161010742,
		Longitude: -65.890296936,
		Timezone:  "America/Moncton",
	},
	"CYSK": {
		ICAO:      "CYSK",
		IATA:      "YSK",
		Name:      "Sanikiluaq Airport",
		City:      "Sanikiluaq",
		State:     "Nunavut",
		Country:   "CA",
		Elevation: 104,
		Latitude:  56.5377998352,
		Longitude: -79.2466964722,
		Timezone:  "America/Iqaluit",
	},
	"CYSL": {
		ICAO:      "CYSL",
		IATA:      "YSL",
		Name:      "St Leonard Airport",
		City:      "St Leonard",
		State:     "New Brunswick",
		Country:   "CA",
		Elevation: 793,
		Latitude:  47.1575012207,
		Longitude: -67.8347015381,
		Timezone:  "America/Moncton",
	},
	"CYSM": {
		ICAO:      "CYSM",
		IATA:      "YSM",
		Name:      "Fort Smith Airport",
		City:      "Fort Smith",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 671,
		Latitude:  60.0203018188,
		Longitude: -111.9619979858,
		Timezone:  "America/Edmonton",
	},
	"CYSN": {
		ICAO:      "CYSN",
		IATA:      "YCM",
		Name:      "Niagara District Airport",
		City:      "St Catharines",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 321,
		Latitude:  43.1916999817,
		Longitude: -79.1716995239,
		Timezone:  "America/Toronto",
	},
	"CYSP": {
		ICAO:      "CYSP",
		IATA:      "YSP",
		Name:      "Marathon Airport",
		City:      "Marathon",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1035,
		Latitude:  48.7552986145,
		Longitude: -86.3443984985,
		Timezone:  "America/Nipigon",
	},
	"CYSQ": {
		ICAO:      "CYSQ",
		Name:      "Atlin Airport",
		City:      "Atlin",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 2348,
		Latitude:  59.5766983032,
		Longitude: -133.6690063477,
		Timezone:  "America/Vancouver",
	},
	"CYST": {
		ICAO:      "CYST",
		IATA:      "YST",
		Name:      "St. Theresa Point Airport",
		City:      "St. Theresa Point",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 773,
		Latitude:  53.8456001282,
		Longitude: -94.8518981934,
		Timezone:  "America/Winnipeg",
	},
	"CYSU": {
		ICAO:      "CYSU",
		IATA:      "YSU",
		Name:      "Summerside Airport",
		City:      "Summerside",
		State:     "Prince Edward Island",
		Country:   "CA",
		Elevation: 56,
		Latitude:  46.4406013489,
		Longitude: -63.8335990906,
		Timezone:  "America/Halifax",
	},
	"CYSW": {
		ICAO:      "CYSW",
		Name:      "Sparwood Elk Valley Airport",
		City:      "Sparwood",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 3800,
		Latitude:  49.8333015442,
		Longitude: -114.8830032349,
		Timezone:  "America/Edmonton",
	},
	"CYSY": {
		ICAO:      "CYSY",
		IATA:      "YSY",
		Name:      "Sachs Harbour (David Nasogaluak Jr. Saaryuaq) Airport",
		City:      "Sachs Harbour",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 282,
		Latitude:  71.9938964844,
		Longitude: -125.242996216,
		Timezone:  "America/Inuvik",
	},
	"CYSZ": {
		ICAO:      "CYSZ",
		Name:      "Ste Anne Des Monts Airport",
		City:      "Ste-Anne-Des-Monts",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 71,
		Latitude:  49.120300293,
		Longitude: -66.5291976929,
		Timezone:  "America/Toronto",
	},
	"CYTA": {
		ICAO:      "CYTA",
		IATA:      "YTA",
		Name:      "Pembroke Airport",
		City:      "Pembroke",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 529,
		Latitude:  45.8643989563,
		Longitude: -77.251701355,
		Timezone:  "America/Toronto",
	},
	"CYTB": {
		ICAO:      "CYTB",
		Name:      "Tillsonburg Airport",
		City:      "Tillsonburg",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 894,
		Latitude:  42.9275,
		Longitude: -80.7469444444,
		Timezone:  "America/Toronto",
	},
	"CYTE": {
		ICAO:      "CYTE",
		IATA:      "YTE",
		Name:      "Cape Dorset Airport",
		City:      "Cape Dorset",
		State:     "Nunavut",
		Country:   "CA",
		Elevation: 164,
		Latitude:  64.2300033569,
		Longitude: -76.5267028809,
		Timezone:  "America/Iqaluit",
	},
	"CYTF": {
		ICAO:      "CYTF",
		IATA:      "YTF",
		Name:      "Alma Airport",
		City:      "Alma",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 445,
		Latitude:  48.5088996887,
		Longitude: -71.6418991089,
		Timezone:  "America/Toronto",
	},
	"CYTH": {
		ICAO:      "CYTH",
		IATA:      "YTH",
		Name:      "Thompson Airport",
		City:      "Thompson",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 729,
		Latitude:  55.8011016846,
		Longitude: -97.8641967773,
		Timezone:  "America/Winnipeg",
	},
	"CYTL": {
		ICAO:      "CYTL",
		IATA:      "YTL",
		Name:      "Big Trout Lake Airport",
		City:      "Big Trout Lake",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 729,
		Latitude:  53.8177986145,
		Longitude: -89.8968963623,
		Timezone:  "America/Rainy_River",
	},
	"CYTN": {
		ICAO:      "CYTN",
		Name:      "Trenton Airport",
		City:      "Trenton",
		State:     "Nova Scotia",
		Country:   "CA",
		Elevation: 319,
		Latitude:  45.6119003296,
		Longitude: -62.6211013794,
		Timezone:  "America/Halifax",
	},
	"CYTQ": {
		ICAO:      "CYTQ",
		IATA:      "YTQ",
		Name:      "Tasiujaq Airport",
		City:      "Tasiujaq",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 122,
		Latitude:  58.6678009033,
		Longitude: -69.9558029175,
		Timezone:  "America/Toronto",
	},
	"CYTR": {
		ICAO:      "CYTR",
		IATA:      "YTR",
		Name:      "CFB Trenton",
		City:      "Trenton",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 283,
		Latitude:  44.1189002991,
		Longitude: -77.5280990601,
		Timezone:  "America/Toronto",
	},
	"CYTS": {
		ICAO:      "CYTS",
		IATA:      "YTS",
		Name:      "Timmins/Victor M. Power",
		City:      "Timmins",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 967,
		Latitude:  48.5696983337,
		Longitude: -81.376701355,
		Timezone:  "America/Toronto",
	},
	"CYTZ": {
		ICAO:      "CYTZ",
		IATA:      "YTZ",
		Name:      "Billy Bishop Toronto City Centre Airport",
		City:      "Toronto",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 252,
		Latitude:  43.6274986267,
		Longitude: -79.3962020874,
		Timezone:  "America/Toronto",
	},
	"CYUB": {
		ICAO:      "CYUB",
		IATA:      "YUB",
		Name:      "Tuktoyaktuk Airport",
		City:      "Tuktoyaktuk",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 15,
		Latitude:  69.4332962036,
		Longitude: -133.0260009766,
		Timezone:  "America/Inuvik",
	},
	"CYUL": {
		ICAO:      "CYUL",
		IATA:      "YUL",
		Name:      "Montreal / Pierre Elliott Trudeau International Airport",
		City:      "Montreal",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 118,
		Latitude:  45.4706001282,
		Longitude: -73.7407989502,
		Timezone:  "America/Toronto",
	},
	"CYUT": {
		ICAO:      "CYUT",
		IATA:      "YUT",
		Name:      "Repulse Bay Airport",
		City:      "Repulse Bay",
		State:     "Nunavut",
		Country:   "CA",
		Elevation: 80,
		Latitude:  66.5214004517,
		Longitude: -86.2247009277,
		Timezone:  "America/Rankin_Inlet",
	},
	"CYUX": {
		ICAO:      "CYUX",
		IATA:      "YUX",
		Name:      "Hall Beach Airport",
		City:      "Hall Beach",
		State:     "Nunavut",
		Country:   "CA",
		Elevation: 27,
		Latitude:  68.7761001587,
		Longitude: -81.243598938,
		Timezone:  "America/Iqaluit",
	},
	"CYUY": {
		ICAO:      "CYUY",
		IATA:      "YUY",
		Name:      "Rouyn Noranda Airport",
		City:      "Rouyn-Noranda",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 988,
		Latitude:  48.2061004639,
		Longitude: -78.8356018066,
		Timezone:  "America/Toronto",
	},
	"CYVB": {
		ICAO:      "CYVB",
		IATA:      "YVB",
		Name:      "Bonaventure Airport",
		City:      "Bonaventure",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 123,
		Latitude:  48.0710983276,
		Longitude: -65.4602966309,
		Timezone:  "America/Toronto",
	},
	"CYVC": {
		ICAO:      "CYVC",
		IATA:      "YVC",
		Name:      "La Ronge Airport",
		City:      "La Ronge",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1242,
		Latitude:  55.1514015198,
		Longitude: -105.2620010376,
		Timezone:  "America/Regina",
	},
	"CYVD": {
		ICAO:      "CYVD",
		Name:      "Virden/R.J. (Bob) Andrew Field Regional Aerodrome",
		City:      "Virden",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 1454,
		Latitude:  49.8782997131,
		Longitude: -100.917999268,
		Timezone:  "America/Winnipeg",
	},
	"CYVG": {
		ICAO:      "CYVG",
		IATA:      "YVG",
		Name:      "Vermilion Airport",
		City:      "Vermilion",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2025,
		Latitude:  53.3558006287,
		Longitude: -110.8239974976,
		Timezone:  "America/Edmonton",
	},
	"CYVK": {
		ICAO:      "CYVK",
		IATA:      "YVE",
		Name:      "Vernon Airport",
		City:      "Vernon",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 1140,
		Latitude:  50.2481002808,
		Longitude: -119.3310012817,
		Timezone:  "America/Vancouver",
	},
	"CYVM": {
		ICAO:      "CYVM",
		IATA:      "YVM",
		Name:      "Qikiqtarjuaq Airport",
		City:      "Qikiqtarjuaq",
		State:     "Nunavut",
		Country:   "CA",
		Elevation: 21,
		Latitude:  67.5457992554,
		Longitude: -64.0314025879,
		Timezone:  "America/Pangnirtung",
	},
	"CYVO": {
		ICAO:      "CYVO",
		IATA:      "YVO",
		Name:      "Val-d'Or Airport",
		City:      "Val-d'Or",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 1107,
		Latitude:  48.0532989502,
		Longitude: -77.7827987671,
		Timezone:  "America/Toronto",
	},
	"CYVP": {
		ICAO:      "CYVP",
		IATA:      "YVP",
		Name:      "Kuujjuaq Airport",
		City:      "Kuujjuaq",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 129,
		Latitude:  58.0960998535,
		Longitude: -68.426902771,
		Timezone:  "America/Toronto",
	},
	"CYVQ": {
		ICAO:      "CYVQ",
		IATA:      "YVQ",
		Name:      "Norman Wells Airport",
		City:      "Norman Wells",
		State:     "Northwest Territories",
		Country:   "CA",
		Elevation: 238,
		Latitude:  65.2816009521,
		Longitude: -126.797996521,
		Timezone:  "America/Inuvik",
	},
	"CYVR": {
		ICAO:      "CYVR",
		IATA:      "YVR",
		Name:      "Vancouver International Airport",
		City:      "Vancouver",
		State:     "British Columbia",
		Country:   "CA",
		Elevation: 14,
		Latitude:  49.193901062,
		Longitude: -123.1839981079,
		Timezone:  "America/Vancouver",
	},
	"CYVT": {
		ICAO:      "CYVT",
		IATA:      "YVT",
		Name:      "Buffalo Narrows Airport",
		City:      "Buffalo Narrows",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1423,
		Latitude:  55.8418998718,
		Longitude: -108.417999268,
		Timezone:  "America/Regina",
	},
	"CYVV": {
		ICAO:      "CYVV",
		IATA:      "YVV",
		Name:      "Wiarton Airport",
		City:      "Wiarton",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 729,
		Latitude:  44.7458000183,
		Longitude: -81.1072006226,
		Timezone:  "America/Toronto",
	},
	"CYVZ": {
		ICAO:      "CYVZ",
		IATA:      "YVZ",
		Name:      "Deer Lake Airport",
		City:      "Deer Lake",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1092,
		Latitude:  52.6557998657,
		Longitude: -94.0614013672,
		Timezone:  "America/Rainy_River",
	},
	"CYWA": {
		ICAO:      "CYWA",
		IATA:      "YWA",
		Name:      "Petawawa Airport",
		City:      "Petawawa",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 427,
		Latitude:  45.9522018433,
		Longitude: -77.3191986084,
		Timezone:  "America/Toronto",
	},
	"CYWE": {
		ICAO:      "CYWE",
		IATA:      "YFJ",
		Name:      "Wekweeti Airport",
		City:      "Wekweeti",
		State:     "Northwest-Territories",
		Country:   "CA",
		Elevation: 1208,
		Latitude:  64.1908035278,
		Longitude: -114.077003479,
		Timezone:  "America/Yellowknife",
	},
	"CYWG": {
		ICAO:      "CYWG",
		IATA:      "YWG",
		Name:      "Winnipeg / James Armstrong Richardson International Airport",
		City:      "Winnipeg",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 783,
		Latitude:  49.9099998474,
		Longitude: -97.2398986816,
		Timezone:  "America/Winnipeg",
	},
	"CYWJ": {
		ICAO:      "CYWJ",
		IATA:      "YWJ",
		Name:      "Deline Airport",
		City:      "Deline",
		State:     "Northwest-Territories",
		Country:   "CA",
		Elevation: 703,
		Latitude:  65.2110977173,
		Longitude: -123.4359970093,
		Timezone:  "America/Inuvik",
	},
	"CYWK": {
		ICAO:      "CYWK",
		IATA:      "YWK",
		Name:      "Wabush Airport",
		City:      "Wabush",
		State:     "Newfoundland-and-Labrador",
		Country:   "CA",
		Elevation: 1808,
		Latitude:  52.9219017029,
		Longitude: -66.864402771,
		Timezone:  "America/Goose_Bay",
	},
	"CYWL": {
		ICAO:      "CYWL",
		IATA:      "YWL",
		Name:      "Williams Lake Airport",
		City:      "Williams Lake",
		State:     "British-Columbia",
		Country:   "CA",
		Elevation: 3085,
		Latitude:  52.1831016541,
		Longitude: -122.0540008545,
		Timezone:  "America/Vancouver",
	},
	"CYWM": {
		ICAO:      "CYWM",
		Name:      "Athabasca Airport",
		City:      "Athabasca",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 1971,
		Latitude:  54.7430992126,
		Longitude: -113.2050018311,
		Timezone:  "America/Edmonton",
	},
	"CYWP": {
		ICAO:      "CYWP",
		IATA:      "YWP",
		Name:      "Webequie Airport",
		City:      "Webequie",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 685,
		Latitude:  52.9593933975,
		Longitude: -87.3748683929,
		Timezone:  "America/Nipigon",
	},
	"CYWV": {
		ICAO:      "CYWV",
		Name:      "Wainwright Airport",
		City:      "Wainwright",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2230,
		Latitude:  52.7957992554,
		Longitude: -110.8570022583,
		Timezone:  "America/Edmonton",
	},
	"CYWY": {
		ICAO:      "CYWY",
		IATA:      "YWY",
		Name:      "Wrigley Airport",
		City:      "Wrigley",
		State:     "Northwest-Territories",
		Country:   "CA",
		Elevation: 489,
		Latitude:  63.209400177,
		Longitude: -123.43699646,
		Timezone:  "America/Inuvik",
	},
	"CYXC": {
		ICAO:      "CYXC",
		IATA:      "YXC",
		Name:      "Cranbrook Airport",
		City:      "Cranbrook",
		State:     "British-Columbia",
		Country:   "CA",
		Elevation: 3082,
		Latitude:  49.6108016968,
		Longitude: -115.7819976807,
		Timezone:  "America/Edmonton",
	},
	"CYXD": {
		ICAO:      "CYXD",
		IATA:      "YXD",
		Name:      "Edmonton City Centre (Blatchford Field) Airport",
		City:      "Edmonton",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2202,
		Latitude:  53.5724983215,
		Longitude: -113.521003723,
		Timezone:  "America/Edmonton",
	},
	"CYXE": {
		ICAO:      "CYXE",
		IATA:      "YXE",
		Name:      "Saskatoon John G. Diefenbaker International Airport",
		City:      "Saskatoon",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1653,
		Latitude:  52.1707992554,
		Longitude: -106.6999969482,
		Timezone:  "America/Regina",
	},
	"CYXH": {
		ICAO:      "CYXH",
		IATA:      "YXH",
		Name:      "Medicine Hat Airport",
		City:      "Medicine Hat",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2352,
		Latitude:  50.018901825,
		Longitude: -110.7210006714,
		Timezone:  "America/Edmonton",
	},
	"CYXJ": {
		ICAO:      "CYXJ",
		IATA:      "YXJ",
		Name:      "Fort St John Airport",
		City:      "Fort St.John",
		State:     "British-Columbia",
		Country:   "CA",
		Elevation: 2280,
		Latitude:  56.2380981445,
		Longitude: -120.7399978638,
		Timezone:  "America/Dawson_Creek",
	},
	"CYXK": {
		ICAO:      "CYXK",
		IATA:      "YXK",
		Name:      "Rimouski Airport",
		City:      "Rimouski",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 82,
		Latitude:  48.478099823,
		Longitude: -68.4969024658,
		Timezone:  "America/Toronto",
	},
	"CYXL": {
		ICAO:      "CYXL",
		IATA:      "YXL",
		Name:      "Sioux Lookout Airport",
		City:      "Sioux Lookout",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1258,
		Latitude:  50.113899231,
		Longitude: -91.9052963257,
		Timezone:  "America/Rainy_River",
	},
	"CYXN": {
		ICAO:      "CYXN",
		IATA:      "YXN",
		Name:      "Whale Cove Airport",
		City:      "Whale Cove",
		State:     "Nunavut",
		Country:   "CA",
		Elevation: 40,
		Latitude:  62.2400016785,
		Longitude: -92.5980987549,
		Timezone:  "America/Rankin_Inlet",
	},
	"CYXP": {
		ICAO:      "CYXP",
		IATA:      "YXP",
		Name:      "Pangnirtung Airport",
		City:      "Pangnirtung",
		State:     "Nunavut",
		Country:   "CA",
		Elevation: 75,
		Latitude:  66.1449966431,
		Longitude: -65.7136001587,
		Timezone:  "America/Pangnirtung",
	},
	"CYXQ": {
		ICAO:      "CYXQ",
		IATA:      "YXQ",
		Name:      "Beaver Creek Airport",
		City:      "Beaver Creek",
		State:     "Yukon",
		Country:   "CA",
		Elevation: 2131,
		Latitude:  62.4103012085,
		Longitude: -140.8670043945,
		Timezone:  "America/Dawson",
	},
	"CYXR": {
		ICAO:      "CYXR",
		IATA:      "YXR",
		Name:      "Earlton (Timiskaming Regional) Airport",
		City:      "Earlton",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 800,
		Latitude:  47.6974006546,
		Longitude: -79.8473453522,
		Timezone:  "America/Nipigon",
	},
	"CYXS": {
		ICAO:      "CYXS",
		IATA:      "YXS",
		Name:      "Prince George Airport",
		City:      "Prince George",
		State:     "British-Columbia",
		Country:   "CA",
		Elevation: 2267,
		Latitude:  53.8894004822,
		Longitude: -122.6790008545,
		Timezone:  "America/Vancouver",
	},
	"CYXT": {
		ICAO:      "CYXT",
		IATA:      "YXT",
		Name:      "Terrace Airport",
		City:      "Terrace",
		State:     "British-Columbia",
		Country:   "CA",
		Elevation: 713,
		Latitude:  54.46849823,
		Longitude: -128.5760040283,
		Timezone:  "America/Vancouver",
	},
	"CYXU": {
		ICAO:      "CYXU",
		IATA:      "YXU",
		Name:      "London Airport",
		City:      "London",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 912,
		Latitude:  43.0355987549,
		Longitude: -81.1539001465,
		Timezone:  "America/Toronto",
	},
	"CYXX": {
		ICAO:      "CYXX",
		IATA:      "YXX",
		Name:      "Abbotsford Airport",
		City:      "Abbotsford",
		State:     "British-Columbia",
		Country:   "CA",
		Elevation: 195,
		Latitude:  49.0252990723,
		Longitude: -122.361000061,
		Timezone:  "America/Los_Angeles",
	},
	"CYXY": {
		ICAO:      "CYXY",
		IATA:      "YXY",
		Name:      "Whitehorse / Erik Nielsen International Airport",
		City:      "Whitehorse",
		State:     "Yukon",
		Country:   "CA",
		Elevation: 2317,
		Latitude:  60.7095985413,
		Longitude: -135.067001343,
		Timezone:  "America/Whitehorse",
	},
	"CYXZ": {
		ICAO:      "CYXZ",
		IATA:      "YXZ",
		Name:      "Wawa Airport",
		City:      "Wawa",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 942,
		Latitude:  47.9667015076,
		Longitude: -84.7866973877,
		Timezone:  "America/Nipigon",
	},
	"CYYB": {
		ICAO:      "CYYB",
		IATA:      "YYB",
		Name:      "North Bay Airport",
		City:      "North Bay",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1215,
		Latitude:  46.3636016846,
		Longitude: -79.4227981567,
		Timezone:  "America/Toronto",
	},
	"CYYC": {
		ICAO:      "CYYC",
		IATA:      "YYC",
		Name:      "Calgary International Airport",
		City:      "Calgary",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3557,
		Latitude:  51.113899231,
		Longitude: -114.0199966431,
		Timezone:  "America/Edmonton",
	},
	"CYYD": {
		ICAO:      "CYYD",
		IATA:      "YYD",
		Name:      "Smithers Airport",
		City:      "Smithers",
		State:     "British-Columbia",
		Country:   "CA",
		Elevation: 1712,
		Latitude:  54.8246994019,
		Longitude: -127.1829986572,
		Timezone:  "America/Vancouver",
	},
	"CYYE": {
		ICAO:      "CYYE",
		IATA:      "YYE",
		Name:      "Fort Nelson Airport",
		City:      "Fort Nelson",
		State:     "British-Columbia",
		Country:   "CA",
		Elevation: 1253,
		Latitude:  58.8363990784,
		Longitude: -122.5970001221,
		Timezone:  "America/Fort_Nelson",
	},
	"CYYF": {
		ICAO:      "CYYF",
		IATA:      "YYF",
		Name:      "Penticton Airport",
		City:      "Penticton",
		State:     "British-Columbia",
		Country:   "CA",
		Elevation: 1129,
		Latitude:  49.4631004333,
		Longitude: -119.6019973755,
		Timezone:  "America/Vancouver",
	},
	"CYYG": {
		ICAO:      "CYYG",
		IATA:      "YYG",
		Name:      "Charlottetown Airport",
		City:      "Charlottetown",
		State:     "Newfoundland-and-Labrador",
		Country:   "CA",
		Elevation: 160,
		Latitude:  46.2900009155,
		Longitude: -63.1211013794,
		Timezone:  "America/Halifax",
	},
	"CYYH": {
		ICAO:      "CYYH",
		IATA:      "YYH",
		Name:      "Taloyoak Airport",
		City:      "Taloyoak",
		State:     "Nunavut",
		Country:   "CA",
		Elevation: 92,
		Latitude:  69.5466995239,
		Longitude: -93.5766983032,
		Timezone:  "America/Cambridge_Bay",
	},
	"CYYJ": {
		ICAO:      "CYYJ",
		IATA:      "YYJ",
		Name:      "Victoria International Airport",
		City:      "Victoria",
		State:     "British-Columbia",
		Country:   "CA",
		Elevation: 63,
		Latitude:  48.646900177,
		Longitude: -123.4260025024,
		Timezone:  "America/Vancouver",
	},
	"CYYL": {
		ICAO:      "CYYL",
		IATA:      "YYL",
		Name:      "Lynn Lake Airport",
		City:      "Lynn Lake",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 1170,
		Latitude:  56.863899231,
		Longitude: -101.0759963989,
		Timezone:  "America/Winnipeg",
	},
	"CYYM": {
		ICAO:      "CYYM",
		IATA:      "YYM",
		Name:      "Cowley Airport",
		City:      "Cowley",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3876,
		Latitude:  49.6363983154,
		Longitude: -114.09400177,
		Timezone:  "America/Edmonton",
	},
	"CYYN": {
		ICAO:      "CYYN",
		IATA:      "YYN",
		Name:      "Swift Current Airport",
		City:      "Swift Current",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 2680,
		Latitude:  50.2919006348,
		Longitude: -107.691001892,
		Timezone:  "America/Swift_Current",
	},
	"CYYO": {
		ICAO:      "CYYO",
		Name:      "Wynyard / W.B. Needham Field",
		City:      "Wynyard",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1740,
		Latitude:  51.8098883726,
		Longitude: -104.169788361,
		Timezone:  "America/Regina",
	},
	"CYYQ": {
		ICAO:      "CYYQ",
		IATA:      "YYQ",
		Name:      "Churchill Airport",
		City:      "Churchill",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 94,
		Latitude:  58.739200592,
		Longitude: -94.0650024414,
		Timezone:  "America/Winnipeg",
	},
	"CYYR": {
		ICAO:      "CYYR",
		IATA:      "YYR",
		Name:      "Goose Bay Airport",
		City:      "Goose Bay",
		State:     "Newfoundland-and-Labrador",
		Country:   "CA",
		Elevation: 160,
		Latitude:  53.3191986084,
		Longitude: -60.4258003235,
		Timezone:  "America/Goose_Bay",
	},
	"CYYT": {
		ICAO:      "CYYT",
		IATA:      "YYT",
		Name:      "St. John's International Airport",
		City:      "St. John's",
		State:     "Newfoundland-and-Labrador",
		Country:   "CA",
		Elevation: 461,
		Latitude:  47.618598938,
		Longitude: -52.7518997192,
		Timezone:  "America/St_Johns",
	},
	"CYYU": {
		ICAO:      "CYYU",
		IATA:      "YYU",
		Name:      "Kapuskasing Airport",
		City:      "Kapuskasing",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 743,
		Latitude:  49.413898468,
		Longitude: -82.4674987793,
		Timezone:  "America/Nipigon",
	},
	"CYYW": {
		ICAO:      "CYYW",
		IATA:      "YYW",
		Name:      "Armstrong Airport",
		City:      "Armstrong",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1058,
		Latitude:  50.2902984619,
		Longitude: -88.9096984863,
		Timezone:  "America/Nipigon",
	},
	"CYYY": {
		ICAO:      "CYYY",
		IATA:      "YYY",
		Name:      "Mont Joli Airport",
		City:      "Mont-Joli",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 172,
		Latitude:  48.6086006165,
		Longitude: -68.2080993652,
		Timezone:  "America/Toronto",
	},
	"CYYZ": {
		ICAO:      "CYYZ",
		IATA:      "YYZ",
		Name:      "Lester B. Pearson International Airport",
		City:      "Toronto",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 569,
		Latitude:  43.6772003174,
		Longitude: -79.6305999756,
		Timezone:  "America/Toronto",
	},
	"CYZD": {
		ICAO:      "CYZD",
		IATA:      "YZD",
		Name:      "Downsview Airport",
		City:      "Toronto",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 652,
		Latitude:  43.7425003052,
		Longitude: -79.4655990601,
		Timezone:  "America/Toronto",
	},
	"CYZE": {
		ICAO:      "CYZE",
		IATA:      "YZE",
		Name:      "Gore Bay Manitoulin Airport",
		City:      "Gore Bay",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 635,
		Latitude:  45.8852996826,
		Longitude: -82.5678024292,
		Timezone:  "America/Nipigon",
	},
	"CYZF": {
		ICAO:      "CYZF",
		IATA:      "YZF",
		Name:      "Yellowknife Airport",
		City:      "Yellowknife",
		State:     "Northwest-Territories",
		Country:   "CA",
		Elevation: 675,
		Latitude:  62.4627990723,
		Longitude: -114.4400024414,
		Timezone:  "America/Yellowknife",
	},
	"CYZG": {
		ICAO:      "CYZG",
		IATA:      "YZG",
		Name:      "Salluit Airport",
		City:      "Salluit",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 743,
		Latitude:  62.1794013977,
		Longitude: -75.6671981812,
		Timezone:  "America/Toronto",
	},
	"CYZH": {
		ICAO:      "CYZH",
		IATA:      "YZH",
		Name:      "Slave Lake Airport",
		City:      "Slave Lake",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 1912,
		Latitude:  55.2930984497,
		Longitude: -114.777000427,
		Timezone:  "America/Edmonton",
	},
	"CYZP": {
		ICAO:      "CYZP",
		IATA:      "YZP",
		Name:      "Sandspit Airport",
		City:      "Sandspit",
		State:     "British-Columbia",
		Country:   "CA",
		Elevation: 21,
		Latitude:  53.2542991638,
		Longitude: -131.813995361,
		Timezone:  "America/Vancouver",
	},
	"CYZR": {
		ICAO:      "CYZR",
		IATA:      "YZR",
		Name:      "Chris Hadfield Airport",
		City:      "Sarnia",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 594,
		Latitude:  42.9994010925,
		Longitude: -82.3088989258,
		Timezone:  "America/Toronto",
	},
	"CYZS": {
		ICAO:      "CYZS",
		IATA:      "YZS",
		Name:      "Coral Harbour Airport",
		City:      "Coral Harbour",
		State:     "Nunavut",
		Country:   "CA",
		Elevation: 210,
		Latitude:  64.1932983398,
		Longitude: -83.3593978882,
		Timezone:  "America/Atikokan",
	},
	"CYZT": {
		ICAO:      "CYZT",
		IATA:      "YZT",
		Name:      "Port Hardy Airport",
		City:      "Port Hardy",
		State:     "British-Columbia",
		Country:   "CA",
		Elevation: 71,
		Latitude:  50.6805992126,
		Longitude: -127.3669967651,
		Timezone:  "America/Vancouver",
	},
	"CYZU": {
		ICAO:      "CYZU",
		IATA:      "YZU",
		Name:      "Whitecourt Airport",
		City:      "Whitecourt",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2567,
		Latitude:  54.143901825,
		Longitude: -115.7870025635,
		Timezone:  "America/Edmonton",
	},
	"CYZV": {
		ICAO:      "CYZV",
		IATA:      "YZV",
		Name:      "Sept-Iles Airport",
		City:      "Sept-Iles",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 180,
		Latitude:  50.2233009338,
		Longitude: -66.2656021118,
		Timezone:  "America/Toronto",
	},
	"CYZW": {
		ICAO:      "CYZW",
		IATA:      "YZW",
		Name:      "Teslin Airport",
		City:      "Teslin",
		State:     "Yukon",
		Country:   "CA",
		Elevation: 2313,
		Latitude:  60.1727981567,
		Longitude: -132.7429962158,
		Timezone:  "America/Whitehorse",
	},
	"CYZX": {
		ICAO:      "CYZX",
		IATA:      "YZX",
		Name:      "CFB Greenwood",
		City:      "Greenwood",
		State:     "Nova-Scotia",
		Country:   "CA",
		Elevation: 92,
		Latitude:  44.9844017029,
		Longitude: -64.9169006348,
		Timezone:  "America/Halifax",
	},
	"CYZY": {
		ICAO:      "CYZY",
		IATA:      "YZY",
		Name:      "Mackenzie Airport",
		City:      "Mackenzie",
		State:     "British-Columbia",
		Country:   "CA",
		Elevation: 2264,
		Latitude:  55.3044013977,
		Longitude: -123.132003784,
		Timezone:  "America/Vancouver",
	},
	"CZAC": {
		ICAO:      "CZAC",
		IATA:      "ZAC",
		Name:      "York Landing Airport",
		City:      "York Landing",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 621,
		Latitude:  56.0894012451,
		Longitude: -96.0892028809,
		Timezone:  "America/Winnipeg",
	},
	"CZAM": {
		ICAO:      "CZAM",
		IATA:      "YSN",
		Name:      "Salmon Arm Airport",
		City:      "Salmon Arm",
		State:     "British-Columbia",
		Country:   "CA",
		Elevation: 1751,
		Latitude:  50.682800293,
		Longitude: -119.2289962769,
		Timezone:  "America/Vancouver",
	},
	"CZBA": {
		ICAO:      "CZBA",
		IATA:      "YDT",
		Name:      "Burlington Executive",
		City:      "Burlington",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 602,
		Latitude:  43.4414531515,
		Longitude: -79.850102663,
		Timezone:  "America/Toronto",
	},
	"CZBB": {
		ICAO:      "CZBB",
		IATA:      "ZBD",
		Name:      "Vancouver / Boundary Bay Airport",
		City:      "Boundary Bay",
		State:     "British-Columbia",
		Country:   "CA",
		Elevation: 6,
		Latitude:  49.0741996765,
		Longitude: -123.012001038,
		Timezone:  "America/Vancouver",
	},
	"CZBD": {
		ICAO:      "CZBD",
		IATA:      "ILF",
		Name:      "Ilford Airport",
		City:      "Ilford",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 642,
		Latitude:  56.0614013672,
		Longitude: -95.613899231,
		Timezone:  "America/Winnipeg",
	},
	"CZBF": {
		ICAO:      "CZBF",
		IATA:      "ZBF",
		Name:      "Bathurst Airport",
		City:      "Bathurst",
		State:     "New-Brunswick",
		Country:   "CA",
		Elevation: 193,
		Latitude:  47.629699707,
		Longitude: -65.738899231,
		Timezone:  "America/Moncton",
	},
	"CZBM": {
		ICAO:      "CZBM",
		IATA:      "ZBM",
		Name:      "Bromont (Roland Desourdy) Airport",
		City:      "Bromont",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 375,
		Latitude:  45.2907981873,
		Longitude: -72.7414016724,
		Timezone:  "America/Toronto",
	},
	"CZEE": {
		ICAO:      "CZEE",
		IATA:      "ZEE",
		Name:      "Kelsey Airport",
		City:      "Kelsey",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 600,
		Latitude:  56.0374984741,
		Longitude: -96.5096969604,
		Timezone:  "America/Winnipeg",
	},
	"CZEM": {
		ICAO:      "CZEM",
		IATA:      "ZEM",
		Name:      "Eastmain River Airport",
		City:      "Eastmain River",
		State:     "Quebec",
		Country:   "CA",
		Elevation: 24,
		Latitude:  52.226398468,
		Longitude: -78.5224990845,
		Timezone:  "America/Toronto",
	},
	"CZFA": {
		ICAO:      "CZFA",
		IATA:      "ZFA",
		Name:      "Faro Airport",
		City:      "Faro",
		State:     "Yukon",
		Country:   "CA",
		Elevation: 2351,
		Latitude:  62.2075004578,
		Longitude: -133.3760070801,
		Timezone:  "America/Whitehorse",
	},
	"CZFD": {
		ICAO:      "CZFD",
		IATA:      "ZFD",
		Name:      "Fond-Du-Lac Airport",
		City:      "Fond-Du-Lac",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 814,
		Latitude:  59.334400177,
		Longitude: -107.1819992065,
		Timezone:  "America/Regina",
	},
	"CZFG": {
		ICAO:      "CZFG",
		IATA:      "ZFG",
		Name:      "Pukatawagan Airport",
		City:      "Pukatawagan",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 958,
		Latitude:  55.7491989136,
		Longitude: -101.2659988403,
		Timezone:  "America/Winnipeg",
	},
	"CZFM": {
		ICAO:      "CZFM",
		IATA:      "ZFM",
		Name:      "Fort Mcpherson Airport",
		City:      "Fort Mcpherson",
		State:     "Northwest-Territories",
		Country:   "CA",
		Elevation: 116,
		Latitude:  67.4075012207,
		Longitude: -134.8609924316,
		Timezone:  "America/Inuvik",
	},
	"CZFN": {
		ICAO:      "CZFN",
		IATA:      "ZFN",
		Name:      "Tulita Airport",
		City:      "Tulita",
		State:     "Northwest-Territories",
		Country:   "CA",
		Elevation: 332,
		Latitude:  64.9096984863,
		Longitude: -125.5729980469,
		Timezone:  "America/Inuvik",
	},
	"CZGF": {
		ICAO:      "CZGF",
		IATA:      "ZGF",
		Name:      "Grand Forks Airport",
		City:      "Grand Forks",
		State:     "British-Columbia",
		Country:   "CA",
		Elevation: 1720,
		Latitude:  49.0155982971,
		Longitude: -118.4309997559,
		Timezone:  "America/Los_Angeles",
	},
	"CZGI": {
		ICAO:      "CZGI",
		IATA:      "ZGI",
		Name:      "Gods River Airport",
		City:      "Gods River",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 627,
		Latitude:  54.8396987915,
		Longitude: -94.0785980225,
		Timezone:  "America/Winnipeg",
	},
	"CZGR": {
		ICAO:      "CZGR",
		IATA:      "ZGR",
		Name:      "Little Grand Rapids Airport",
		City:      "Little Grand Rapids",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 1005,
		Latitude:  52.0456008911,
		Longitude: -95.4657974243,
		Timezone:  "America/Winnipeg",
	},
	"CZHP": {
		ICAO:      "CZHP",
		IATA:      "ZHP",
		Name:      "High Prairie Airport",
		City:      "High Prairie",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 1974,
		Latitude:  55.3936004639,
		Longitude: -116.4749984741,
		Timezone:  "America/Edmonton",
	},
	"CZJG": {
		ICAO:      "CZJG",
		IATA:      "ZJG",
		Name:      "Jenpeg Airport",
		City:      "Jenpeg",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 729,
		Latitude:  54.518901825,
		Longitude: -98.0460968018,
		Timezone:  "America/Winnipeg",
	},
	"CZJN": {
		ICAO:      "CZJN",
		IATA:      "ZJN",
		Name:      "Swan River Airport",
		City:      "Swan River",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 1100,
		Latitude:  52.1206016541,
		Longitude: -101.236000061,
		Timezone:  "America/Winnipeg",
	},
	"CZKE": {
		ICAO:      "CZKE",
		IATA:      "ZKE",
		Name:      "Kashechewan Airport",
		City:      "Kashechewan",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 35,
		Latitude:  52.2825012207,
		Longitude: -81.6778030396,
		Timezone:  "America/Nipigon",
	},
	"CZLQ": {
		ICAO:      "CZLQ",
		IATA:      "YTD",
		Name:      "Thicket Portage Airport",
		City:      "Thicket Portage",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 678,
		Latitude:  55.318901062,
		Longitude: -97.7078018188,
		Timezone:  "America/Winnipeg",
	},
	"CZMD": {
		ICAO:      "CZMD",
		IATA:      "MSA",
		Name:      "Muskrat Dam Airport",
		City:      "Muskrat Dam",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 911,
		Latitude:  53.4413986206,
		Longitude: -91.762802124,
		Timezone:  "America/Rainy_River",
	},
	"CZML": {
		ICAO:      "CZML",
		IATA:      "ZMH",
		Name:      "South Cariboo Region / 108 Mile Airport",
		City:      "108 Mile",
		State:     "British-Columbia",
		Country:   "CA",
		Elevation: 3126,
		Latitude:  51.7360992432,
		Longitude: -121.333000183,
		Timezone:  "America/Vancouver",
	},
	"CZMN": {
		ICAO:      "CZMN",
		IATA:      "PIW",
		Name:      "Pikwitonei Airport",
		City:      "Pikwitonei",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 630,
		Latitude:  55.5889015198,
		Longitude: -97.1641998291,
		Timezone:  "America/Winnipeg",
	},
	"CZMT": {
		ICAO:      "CZMT",
		IATA:      "ZMT",
		Name:      "Masset Airport",
		City:      "Masset",
		State:     "British-Columbia",
		Country:   "CA",
		Elevation: 25,
		Latitude:  54.0275001526,
		Longitude: -132.125,
		Timezone:  "America/Vancouver",
	},
	"CZNG": {
		ICAO:      "CZNG",
		IATA:      "ZNG",
		Name:      "Poplar River Airport",
		City:      "Poplar River",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 728,
		Latitude:  53,
		Longitude: -97.2667007446,
		Timezone:  "America/Winnipeg",
	},
	"CZNL": {
		ICAO:      "CZNL",
		Name:      "Nelson Airport",
		City:      "Nelson",
		State:     "British-Columbia",
		Country:   "CA",
		Elevation: 1755,
		Latitude:  49.4942016602,
		Longitude: -117.3010025024,
		Timezone:  "America/Vancouver",
	},
	"CZPB": {
		ICAO:      "CZPB",
		IATA:      "ZPB",
		Name:      "Sachigo Lake Airport",
		City:      "Sachigo Lake",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 876,
		Latitude:  53.8911018372,
		Longitude: -92.1964035034,
		Timezone:  "America/Rainy_River",
	},
	"CZPC": {
		ICAO:      "CZPC",
		IATA:      "WPC",
		Name:      "Pincher Creek Airport",
		City:      "Pincher Creek",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 3903,
		Latitude:  49.5205993652,
		Longitude: -113.9970016479,
		Timezone:  "America/Edmonton",
	},
	"CZPO": {
		ICAO:      "CZPO",
		IATA:      "ZPO",
		Name:      "Pinehouse Lake Airport",
		City:      "Pinehouse Lake",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1278,
		Latitude:  55.5280990601,
		Longitude: -106.582000732,
		Timezone:  "America/Regina",
	},
	"CZRJ": {
		ICAO:      "CZRJ",
		IATA:      "ZRJ",
		Name:      "Round Lake (Weagamow Lake) Airport",
		City:      "Round Lake",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 974,
		Latitude:  52.9435997009,
		Longitude: -91.3127975464,
		Timezone:  "America/Rainy_River",
	},
	"CZSJ": {
		ICAO:      "CZSJ",
		IATA:      "ZSJ",
		Name:      "Sandy Lake Airport",
		City:      "Sandy Lake",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 951,
		Latitude:  53.064201355,
		Longitude: -93.3443984985,
		Timezone:  "America/Rainy_River",
	},
	"CZSN": {
		ICAO:      "CZSN",
		IATA:      "ZSN",
		Name:      "South Indian Lake Airport",
		City:      "South Indian Lake",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 951,
		Latitude:  56.7928009033,
		Longitude: -98.9072036743,
		Timezone:  "America/Winnipeg",
	},
	"CZST": {
		ICAO:      "CZST",
		IATA:      "ZST",
		Name:      "Stewart Airport",
		City:      "Stewart",
		State:     "British-Columbia",
		Country:   "CA",
		Elevation: 24,
		Latitude:  55.935410448,
		Longitude: -129.982434511,
		Timezone:  "America/Vancouver",
	},
	"CZTA": {
		ICAO:      "CZTA",
		IATA:      "YDV",
		Name:      "Bloodvein River Airport",
		City:      "Bloodvein River",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 721,
		Latitude:  51.7832984924,
		Longitude: -96.6999969482,
		Timezone:  "America/Winnipeg",
	},
	"CZTM": {
		ICAO:      "CZTM",
		IATA:      "ZTM",
		Name:      "Shamattawa Airport",
		City:      "Shamattawa",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 289,
		Latitude:  55.8656005859,
		Longitude: -92.0813980103,
		Timezone:  "America/Winnipeg",
	},
	"CZUC": {
		ICAO:      "CZUC",
		IATA:      "ZUC",
		Name:      "Ignace Municipal Airport",
		City:      "Ignace",
		State:     "Ontario",
		Country:   "CA",
		Elevation: 1435,
		Latitude:  49.4296989441,
		Longitude: -91.7177963257,
		Timezone:  "America/Rainy_River",
	},
	"CZUM": {
		ICAO:      "CZUM",
		IATA:      "ZUM",
		Name:      "Churchill Falls Airport",
		City:      "Churchill Falls",
		State:     "Newfoundland-and-Labrador",
		Country:   "CA",
		Elevation: 1442,
		Latitude:  53.5619010925,
		Longitude: -64.1063995361,
		Timezone:  "America/Goose_Bay",
	},
	"CZVL": {
		ICAO:      "CZVL",
		Name:      "Edmonton / Villeneuve Airport",
		City:      "Edmonton",
		State:     "Alberta",
		Country:   "CA",
		Elevation: 2256,
		Latitude:  53.6674995422,
		Longitude: -113.853996277,
		Timezone:  "America/Edmonton",
	},
	"CZWH": {
		ICAO:      "CZWH",
		IATA:      "ZWH",
		Name:      "Lac Brochet Airport",
		City:      "Lac Brochet",
		State:     "Manitoba",
		Country:   "CA",
		Elevation: 1211,
		Latitude:  58.6175003052,
		Longitude: -101.46900177,
		Timezone:  "America/Winnipeg",
	},
	"CZWL": {
		ICAO:      "CZWL",
		IATA:      "ZWL",
		Name:      "Wollaston Lake Airport",
		City:      "Wollaston Lake",
		State:     "Saskatchewan",
		Country:   "CA",
		Elevation: 1360,
		Latitude:  58.1068992615,
		Longitude: -103.1719970703,
		Timezone:  "America/Regina",
	},
	"DA10": {
		ICAO:      "DA10",
		Name:      "Illizi Airport",
		State:     "Illizi",
		Country:   "DZ",
		Elevation: 1778,
		Latitude:  26.573600769,
		Longitude: 8.4836597443,
		Timezone:  "Africa/Algiers",
	},
	"DA11": {
		ICAO:      "DA11",
		Name:      "Hamaguir Airport",
		State:     "Bechar",
		Country:   "DZ",
		Elevation: 2454,
		Latitude:  30.8789997101,
		Longitude: -3.0677499771,
		Timezone:  "Africa/Algiers",
	},
	"DA12": {
		ICAO:      "DA12",
		Name:      "El Abiodh Sidi Cheikh Airport",
		State:     "El-Bayadh",
		Country:   "DZ",
		Elevation: 2965,
		Latitude:  32.8984985352,
		Longitude: 0.5246940255,
		Timezone:  "Africa/Algiers",
	},
	"DA13": {
		ICAO:      "DA13",
		Name:      "Tinfouchy Airport",
		State:     "Tindouf",
		Country:   "DZ",
		Elevation: 1804,
		Latitude:  28.8792991638,
		Longitude: -5.8228998184,
		Timezone:  "Africa/Algiers",
	},
	"DA14": {
		ICAO:      "DA14",
		Name:      "Mostaganem Airport",
		State:     "Mostaganem",
		Country:   "DZ",
		Elevation: 732,
		Latitude:  35.9087982178,
		Longitude: 0.1493829936,
		Timezone:  "Africa/Algiers",
	},
	"DA15": {
		ICAO:      "DA15",
		Name:      "Saida Airport",
		State:     "Saida",
		Country:   "DZ",
		Elevation: 2444,
		Latitude:  34.8972015381,
		Longitude: 0.1516939998,
		Timezone:  "Africa/Algiers",
	},
	"DA16": {
		ICAO:      "DA16",
		Name:      "Tindouf East Airport",
		State:     "Tindouf",
		Country:   "DZ",
		Elevation: 1425,
		Latitude:  27.585899353,
		Longitude: -7.5001101494,
		Timezone:  "Africa/Algiers",
	},
	"DAAB": {
		ICAO:      "DAAB",
		IATA:      "QLD",
		Name:      "Blida Airport",
		State:     "Blida",
		Country:   "DZ",
		Elevation: 535,
		Latitude:  36.5036010742,
		Longitude: 2.8141698837,
		Timezone:  "Africa/Algiers",
	},
	"DAAD": {
		ICAO:      "DAAD",
		IATA:      "BUJ",
		Name:      "Bou Saada Airport",
		State:     "M'Sila",
		Country:   "DZ",
		Elevation: 1506,
		Latitude:  35.3325004578,
		Longitude: 4.206389904,
		Timezone:  "Africa/Algiers",
	},
	"DAAE": {
		ICAO:      "DAAE",
		IATA:      "BJA",
		Name:      "Soummam Airport",
		City:      "Bejaia",
		State:     "Bejaia",
		Country:   "DZ",
		Elevation: 20,
		Latitude:  36.7120018005,
		Longitude: 5.069920063,
		Timezone:  "Africa/Algiers",
	},
	"DAAG": {
		ICAO:      "DAAG",
		IATA:      "ALG",
		Name:      "Houari Boumediene Airport",
		City:      "Algiers",
		State:     "Algiers",
		Country:   "DZ",
		Elevation: 82,
		Latitude:  36.6910018921,
		Longitude: 3.2154099941,
		Timezone:  "Africa/Algiers",
	},
	"DAAJ": {
		ICAO:      "DAAJ",
		IATA:      "DJG",
		Name:      "Djanet Inedbirene Airport",
		City:      "Djanet",
		State:     "Illizi",
		Country:   "DZ",
		Elevation: 3176,
		Latitude:  24.2928009033,
		Longitude: 9.4524402618,
		Timezone:  "Africa/Algiers",
	},
	"DAAK": {
		ICAO:      "DAAK",
		IATA:      "QFD",
		Name:      "Boufarik Airport",
		State:     "Blida",
		Country:   "DZ",
		Elevation: 335,
		Latitude:  36.5457992554,
		Longitude: 2.8761100769,
		Timezone:  "Africa/Algiers",
	},
	"DAAM": {
		ICAO:      "DAAM",
		Name:      "Telerghma Airport",
		City:      "Telerghma",
		State:     "Mila",
		Country:   "DZ",
		Elevation: 2484,
		Latitude:  36.1086997986,
		Longitude: 6.3646001816,
		Timezone:  "Africa/Algiers",
	},
	"DAAN": {
		ICAO:      "DAAN",
		Name:      "Reggane Airport",
		State:     "Adrar",
		Country:   "DZ",
		Elevation: 955,
		Latitude:  26.710100174,
		Longitude: 0.2856470048,
		Timezone:  "Africa/Algiers",
	},
	"DAAP": {
		ICAO:      "DAAP",
		IATA:      "VVZ",
		Name:      "Illizi Takhamalt Airport",
		City:      "Illizi",
		State:     "Illizi",
		Country:   "DZ",
		Elevation: 1778,
		Latitude:  26.7234992981,
		Longitude: 8.6226501465,
		Timezone:  "Africa/Algiers",
	},
	"DAAQ": {
		ICAO:      "DAAQ",
		Name:      "Ain Oussera Airport",
		State:     "Djelfa",
		Country:   "DZ",
		Elevation: 2132,
		Latitude:  35.5253982544,
		Longitude: 2.8787100315,
		Timezone:  "Africa/Algiers",
	},
	"DAAS": {
		ICAO:      "DAAS",
		IATA:      "QSF",
		Name:      "Ain Arnat Airport",
		City:      "Setif",
		State:     "Setif",
		Country:   "DZ",
		Elevation: 3360,
		Latitude:  36.1781005859,
		Longitude: 5.3244900703,
		Timezone:  "Africa/Algiers",
	},
	"DAAT": {
		ICAO:      "DAAT",
		IATA:      "TMR",
		Name:      "Aguenar – Hadj Bey Akhamok Airport",
		City:      "Tamanrasset",
		State:     "Tamanrasset",
		Country:   "DZ",
		Elevation: 4518,
		Latitude:  22.8115005493,
		Longitude: 5.4510798454,
		Timezone:  "Africa/Algiers",
	},
	"DAAV": {
		ICAO:      "DAAV",
		IATA:      "GJL",
		Name:      "Jijel Ferhat Abbas Airport",
		City:      "Jijel",
		State:     "Jijel",
		Country:   "DZ",
		Elevation: 36,
		Latitude:  36.7951011658,
		Longitude: 5.8736100197,
		Timezone:  "Africa/Algiers",
	},
	"DAAW": {
		ICAO:      "DAAW",
		Name:      "Bordj Omar Driss Airport",
		City:      "Bordj Omar Driss",
		State:     "Illizi",
		Country:   "DZ",
		Elevation: 1207,
		Latitude:  28.1313991547,
		Longitude: 6.8336100578,
		Timezone:  "Africa/Algiers",
	},
	"DAAY": {
		ICAO:      "DAAY",
		IATA:      "MZW",
		Name:      "Mecheria Airport",
		City:      "Mecheria",
		State:     "Naama",
		Country:   "DZ",
		Elevation: 3855,
		Latitude:  33.535900116,
		Longitude: -0.2423530072,
		Timezone:  "Africa/Algiers",
	},
	"DAAZ": {
		ICAO:      "DAAZ",
		IATA:      "QZN",
		Name:      "Relizane Airport",
		State:     "Relizane",
		Country:   "DZ",
		Elevation: 282,
		Latitude:  35.7522010803,
		Longitude: 0.6262720227,
		Timezone:  "Africa/Algiers",
	},
	"DABB": {
		ICAO:      "DABB",
		IATA:      "AAE",
		Name:      "Annaba Airport",
		City:      "Annabah",
		State:     "Annaba",
		Country:   "DZ",
		Elevation: 16,
		Latitude:  36.8222007751,
		Longitude: 7.8091697693,
		Timezone:  "Africa/Algiers",
	},
	"DABC": {
		ICAO:      "DABC",
		IATA:      "CZL",
		Name:      "Mohamed Boudiaf International Airport",
		City:      "Constantine",
		State:     "Constantine",
		Country:   "DZ",
		Elevation: 2265,
		Latitude:  36.2760009766,
		Longitude: 6.6203899384,
		Timezone:  "Africa/Algiers",
	},
	"DABO": {
		ICAO:      "DABO",
		Name:      "Oum el Bouaghi airport",
		City:      "Oum El Bouaghi",
		State:     "Oum-el-Bouaghi",
		Country:   "DZ",
		Latitude:  35.879699707,
		Longitude: 7.2708001137,
		Timezone:  "Africa/Algiers",
	},
	"DABS": {
		ICAO:      "DABS",
		IATA:      "TEE",
		Name:      "Cheikh Larbi Tebessi Airport",
		City:      "Tebessi",
		State:     "Tebessa",
		Country:   "DZ",
		Elevation: 2661,
		Latitude:  35.4315986633,
		Longitude: 8.1207199097,
		Timezone:  "Africa/Algiers",
	},
	"DABT": {
		ICAO:      "DABT",
		IATA:      "BLJ",
		Name:      "Batna Airport",
		City:      "Batna",
		State:     "Batna",
		Country:   "DZ",
		Elevation: 2697,
		Latitude:  35.7521018982,
		Longitude: 6.3085899353,
		Timezone:  "Africa/Algiers",
	},
	"DAFH": {
		ICAO:      "DAFH",
		IATA:      "HRM",
		Name:      "Hassi R'Mel Airport",
		State:     "Laghouat",
		Country:   "DZ",
		Elevation: 2540,
		Latitude:  32.9304008484,
		Longitude: 3.3115398884,
		Timezone:  "Africa/Algiers",
	},
	"DAFI": {
		ICAO:      "DAFI",
		IATA:      "QDJ",
		Name:      "Tsletsi Airport",
		State:     "Djelfa",
		Country:   "DZ",
		Latitude:  34.6657,
		Longitude: 3.351,
		Timezone:  "Africa/Algiers",
	},
	"DALH": {
		ICAO:      "DALH",
		Name:      "Dalhousie Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -26.4290008545,
		Longitude: 135.5059967041,
		Timezone:  "Australia/Adelaide",
	},
	"DAOB": {
		ICAO:      "DAOB",
		IATA:      "TID",
		Name:      "Bou Chekif Airport",
		City:      "Tiaret",
		State:     "Tiaret",
		Country:   "DZ",
		Elevation: 3245,
		Latitude:  35.3410987854,
		Longitude: 1.4631500244,
		Timezone:  "Africa/Algiers",
	},
	"DAOC": {
		ICAO:      "DAOC",
		Name:      "Ouakda Airport",
		City:      "Bechar",
		State:     "Bechar",
		Country:   "DZ",
		Elevation: 2660,
		Latitude:  31.6424999237,
		Longitude: -2.1838901043,
		Timezone:  "Africa/Algiers",
	},
	"DAOE": {
		ICAO:      "DAOE",
		Name:      "Bou Sfer Airport",
		State:     "Oran",
		Country:   "DZ",
		Elevation: 187,
		Latitude:  35.7354011536,
		Longitude: -0.8053889871,
		Timezone:  "Africa/Algiers",
	},
	"DAOF": {
		ICAO:      "DAOF",
		IATA:      "TIN",
		Name:      "Tindouf Airport",
		City:      "Tindouf",
		State:     "Tindouf",
		Country:   "DZ",
		Elevation: 1453,
		Latitude:  27.7003993988,
		Longitude: -8.1670999527,
		Timezone:  "Africa/Algiers",
	},
	"DAOI": {
		ICAO:      "DAOI",
		IATA:      "QAS",
		Name:      "Ech Cheliff Airport",
		State:     "Chlef",
		Country:   "DZ",
		Elevation: 463,
		Latitude:  36.2126998901,
		Longitude: 1.3317699432,
		Timezone:  "Africa/Algiers",
	},
	"DAOL": {
		ICAO:      "DAOL",
		IATA:      "TAF",
		Name:      "Tafaraoui Airport",
		State:     "Oran",
		Country:   "DZ",
		Elevation: 367,
		Latitude:  35.5424003601,
		Longitude: -0.5322780013,
		Timezone:  "Africa/Algiers",
	},
	"DAON": {
		ICAO:      "DAON",
		IATA:      "TLM",
		Name:      "Zenata – Messali El Hadj Airport",
		City:      "Tlemcen",
		State:     "Tlemcen",
		Country:   "DZ",
		Elevation: 814,
		Latitude:  35.0167007446,
		Longitude: -1.4500000477,
		Timezone:  "Africa/Algiers",
	},
	"DAOO": {
		ICAO:      "DAOO",
		IATA:      "ORN",
		Name:      "Es Senia Airport",
		City:      "Oran",
		State:     "Oran",
		Country:   "DZ",
		Elevation: 295,
		Latitude:  35.6239013672,
		Longitude: -0.6211829782,
		Timezone:  "Africa/Algiers",
	},
	"DAOR": {
		ICAO:      "DAOR",
		IATA:      "CBH",
		Name:      "Bechar Boudghene Ben Ali Lotfi Airport",
		City:      "Bechar",
		State:     "Bechar",
		Country:   "DZ",
		Elevation: 2661,
		Latitude:  31.6457004547,
		Longitude: -2.2698600292,
		Timezone:  "Africa/Algiers",
	},
	"DAOS": {
		ICAO:      "DAOS",
		IATA:      "BFW",
		Name:      "Sidi Bel Abbes Airport",
		State:     "Sidi-Bel-Abbes",
		Country:   "DZ",
		Elevation: 1614,
		Latitude:  35.1717987061,
		Longitude: -0.5932750106,
		Timezone:  "Africa/Algiers",
	},
	"DAOV": {
		ICAO:      "DAOV",
		IATA:      "MUW",
		Name:      "Ghriss Airport",
		State:     "Mascara",
		Country:   "DZ",
		Elevation: 1686,
		Latitude:  35.207698822,
		Longitude: 0.147141993,
		Timezone:  "Africa/Algiers",
	},
	"DAOY": {
		ICAO:      "DAOY",
		IATA:      "EBH",
		Name:      "El Bayadh Airport",
		City:      "El Bayadh",
		State:     "El-Bayadh",
		Country:   "DZ",
		Elevation: 4493,
		Latitude:  33.7216666667,
		Longitude: 1.0925,
		Timezone:  "Africa/Algiers",
	},
	"DATG": {
		ICAO:      "DATG",
		IATA:      "INF",
		Name:      "In Guezzam Airport",
		City:      "In Guezzam",
		State:     "Tamanrasset",
		Country:   "DZ",
		Elevation: 1312,
		Latitude:  19.5669994354,
		Longitude: 5.75,
		Timezone:  "Africa/Algiers",
	},
	"DATM": {
		ICAO:      "DATM",
		IATA:      "BMW",
		Name:      "Bordj Badji Mokhtar Airport",
		City:      "Bordj Badji Mokhtar",
		State:     "Adrar",
		Country:   "DZ",
		Elevation: 1303,
		Latitude:  21.375,
		Longitude: 0.9238889813,
		Timezone:  "Africa/Algiers",
	},
	"DAUA": {
		ICAO:      "DAUA",
		IATA:      "AZR",
		Name:      "Touat Cheikh Sidi Mohamed Belkebir Airport",
		State:     "Adrar",
		Country:   "DZ",
		Elevation: 919,
		Latitude:  27.837600708,
		Longitude: -0.1864140034,
		Timezone:  "Africa/Algiers",
	},
	"DAUB": {
		ICAO:      "DAUB",
		IATA:      "BSK",
		Name:      "Biskra Airport",
		City:      "Biskra",
		State:     "Biskra",
		Country:   "DZ",
		Elevation: 289,
		Latitude:  34.7933006287,
		Longitude: 5.7382302284,
		Timezone:  "Africa/Algiers",
	},
	"DAUE": {
		ICAO:      "DAUE",
		IATA:      "ELG",
		Name:      "El Golea Airport",
		State:     "Ghardaia",
		Country:   "DZ",
		Elevation: 1306,
		Latitude:  30.5713005066,
		Longitude: 2.8595900536,
		Timezone:  "Africa/Algiers",
	},
	"DAUG": {
		ICAO:      "DAUG",
		IATA:      "GHA",
		Name:      "Noumerat - Moufdi Zakaria Airport",
		City:      "Ghardaia",
		State:     "Ghardaia",
		Country:   "DZ",
		Elevation: 1512,
		Latitude:  32.3841018677,
		Longitude: 3.7941100597,
		Timezone:  "Africa/Algiers",
	},
	"DAUH": {
		ICAO:      "DAUH",
		IATA:      "HME",
		Name:      "Oued Irara Airport",
		City:      "Hassi Messaoud",
		State:     "Ouargla",
		Country:   "DZ",
		Elevation: 463,
		Latitude:  31.6730003357,
		Longitude: 6.1404399872,
		Timezone:  "Africa/Algiers",
	},
	"DAUI": {
		ICAO:      "DAUI",
		IATA:      "INZ",
		Name:      "In Salah Airport",
		City:      "In Salah",
		State:     "Tamanrasset",
		Country:   "DZ",
		Elevation: 896,
		Latitude:  27.2509994507,
		Longitude: 2.5120201111,
		Timezone:  "Africa/Algiers",
	},
	"DAUK": {
		ICAO:      "DAUK",
		IATA:      "TGR",
		Name:      "Touggourt Sidi Madhi Airport",
		City:      "Touggourt",
		State:     "Ouargla",
		Country:   "DZ",
		Elevation: 279,
		Latitude:  33.0677986145,
		Longitude: 6.0886697769,
		Timezone:  "Africa/Algiers",
	},
	"DAUL": {
		ICAO:      "DAUL",
		IATA:      "LOO",
		Name:      "Laghouat Airport",
		City:      "Laghouat",
		State:     "Laghouat",
		Country:   "DZ",
		Elevation: 2510,
		Latitude:  33.7644004822,
		Longitude: 2.9283399582,
		Timezone:  "Africa/Algiers",
	},
	"DAUO": {
		ICAO:      "DAUO",
		IATA:      "ELU",
		Name:      "Guemar Airport",
		City:      "Guemar",
		State:     "El-Oued",
		Country:   "DZ",
		Elevation: 203,
		Latitude:  33.5113983154,
		Longitude: 6.7767901421,
		Timezone:  "Africa/Algiers",
	},
	"DAUT": {
		ICAO:      "DAUT",
		IATA:      "TMX",
		Name:      "Timimoun Airport",
		City:      "Timimoun",
		State:     "Adrar",
		Country:   "DZ",
		Elevation: 1027,
		Latitude:  29.2371006012,
		Longitude: 0.2760330141,
		Timezone:  "Africa/Algiers",
	},
	"DAUU": {
		ICAO:      "DAUU",
		IATA:      "OGX",
		Name:      "Ain el Beida Airport",
		City:      "Ouargla",
		State:     "Ouargla",
		Country:   "DZ",
		Elevation: 492,
		Latitude:  31.9172000885,
		Longitude: 5.412779808,
		Timezone:  "Africa/Algiers",
	},
	"DAUZ": {
		ICAO:      "DAUZ",
		IATA:      "IAM",
		Name:      "In Amenas Airport",
		City:      "Amenas",
		State:     "Illizi",
		Country:   "DZ",
		Elevation: 1847,
		Latitude:  28.0515003204,
		Longitude: 9.6429100037,
		Timezone:  "Africa/Algiers",
	},
	"DBBB": {
		ICAO:      "DBBB",
		IATA:      "COO",
		Name:      "Cadjehoun Airport",
		City:      "Cotonou",
		State:     "Atlantique",
		Country:   "BJ",
		Elevation: 19,
		Latitude:  6.3572301865,
		Longitude: 2.3843500614,
		Timezone:  "Africa/Porto-Novo",
	},
	"DBBC": {
		ICAO:      "DBBC",
		Name:      "Cana Airport",
		City:      "Bohicon",
		State:     "Zou",
		Country:   "BJ",
		Elevation: 548,
		Latitude:  7.1669998169,
		Longitude: 2.0669999123,
		Timezone:  "Africa/Porto-Novo",
	},
	"DBBD": {
		ICAO:      "DBBD",
		IATA:      "DJA",
		Name:      "Djougou Airport",
		City:      "Djougou",
		State:     "Atakora",
		Country:   "BJ",
		Elevation: 1480,
		Latitude:  9.6920833333,
		Longitude: 1.6377777778,
		Timezone:  "Africa/Porto-Novo",
	},
	"DBBK": {
		ICAO:      "DBBK",
		IATA:      "KDC",
		Name:      "Kandi Airport",
		City:      "Kandi",
		State:     "Alibori",
		Country:   "BJ",
		Elevation: 951,
		Latitude:  11.1330003738,
		Longitude: 2.9330000877,
		Timezone:  "Africa/Porto-Novo",
	},
	"DBBN": {
		ICAO:      "DBBN",
		IATA:      "NAE",
		Name:      "Natitingou Airport",
		City:      "Natitingou",
		Country:   "BJ",
		Elevation: 1411,
		Latitude:  10.3170003891,
		Longitude: 1.3830000162,
		Timezone:  "Africa/Porto-Novo",
	},
	"DBBO": {
		ICAO:      "DBBO",
		Name:      "Porga Airport",
		City:      "Porga",
		State:     "Atakora",
		Country:   "BJ",
		Elevation: 531,
		Latitude:  11.0329999924,
		Longitude: 0.9829999804,
		Timezone:  "Africa/Ouagadougou",
	},
	"DBBP": {
		ICAO:      "DBBP",
		IATA:      "PKO",
		Name:      "Parakou Airport",
		City:      "Parakou",
		State:     "Borgou",
		Country:   "BJ",
		Elevation: 1266,
		Latitude:  9.3576898575,
		Longitude: 2.6096799374,
		Timezone:  "Africa/Porto-Novo",
	},
	"DBBR": {
		ICAO:      "DBBR",
		Name:      "Bembereke Airport",
		City:      "Bembereke",
		State:     "Borgou",
		Country:   "BJ",
		Elevation: 1188,
		Latitude:  10.2829999924,
		Longitude: 2.7000000477,
		Timezone:  "Africa/Porto-Novo",
	},
	"DBBS": {
		ICAO:      "DBBS",
		IATA:      "SVF",
		Name:      "Save Airport",
		City:      "Save",
		State:     "Zou",
		Country:   "BJ",
		Elevation: 597,
		Latitude:  8.0181703568,
		Longitude: 2.4645800591,
		Timezone:  "Africa/Porto-Novo",
	},
	"DE00": {
		ICAO:      "DE00",
		Name:      "Doyle's Airport",
		City:      "Felton",
		State:     "Delaware",
		Country:   "US",
		Elevation: 51,
		Latitude:  39.0206985474,
		Longitude: -75.5771026611,
		Timezone:  "America/New_York",
	},
	"DE04": {
		ICAO:      "DE04",
		Name:      "Newberg Airport",
		City:      "Smyrna",
		State:     "Delaware",
		Country:   "US",
		Elevation: 20,
		Latitude:  39.2916984558,
		Longitude: -75.5749969482,
		Timezone:  "America/New_York",
	},
	"DE07": {
		ICAO:      "DE07",
		Name:      "Flying C Airport",
		City:      "Hartly",
		State:     "Delaware",
		Country:   "US",
		Elevation: 70,
		Latitude:  39.1442985535,
		Longitude: -75.6912994385,
		Timezone:  "America/New_York",
	},
	"DE09": {
		ICAO:      "DE09",
		Name:      "Johnsons Airport",
		City:      "Magnolia",
		State:     "Delaware",
		Country:   "US",
		Elevation: 43,
		Latitude:  39.0681991577,
		Longitude: -75.4885025024,
		Timezone:  "America/New_York",
	},
	"DE10": {
		ICAO:      "DE10",
		Name:      "Kimbowrosa Farm Airport",
		City:      "Milford",
		State:     "Delaware",
		Country:   "US",
		Elevation: 40,
		Latitude:  38.9668006897,
		Longitude: -75.4663009644,
		Timezone:  "America/New_York",
	},
	"DE11": {
		ICAO:      "DE11",
		Name:      "Drummond Airport",
		City:      "Milford",
		State:     "Delaware",
		Country:   "US",
		Elevation: 33,
		Latitude:  38.9057006836,
		Longitude: -75.4010009766,
		Timezone:  "America/New_York",
	},
	"DE12": {
		ICAO:      "DE12",
		Name:      "Owens Field",
		City:      "Ellendale",
		State:     "Delaware",
		Country:   "US",
		Elevation: 50,
		Latitude:  38.8056983948,
		Longitude: -75.4126968384,
		Timezone:  "America/New_York",
	},
	"DE14": {
		ICAO:      "DE14",
		Name:      "Huey Airport",
		City:      "Bridgeville",
		State:     "Delaware",
		Country:   "US",
		Elevation: 45,
		Latitude:  38.7439994812,
		Longitude: -75.5348968506,
		Timezone:  "America/New_York",
	},
	"DE15": {
		ICAO:      "DE15",
		Name:      "Pevey Airport",
		City:      "Seaford",
		State:     "Delaware",
		Country:   "US",
		Elevation: 40,
		Latitude:  38.6125984192,
		Longitude: -75.6996994019,
		Timezone:  "America/New_York",
	},
	"DE17": {
		ICAO:      "DE17",
		Name:      "Sugar Hill Airport",
		City:      "Greenwood",
		State:     "Delaware",
		Country:   "US",
		Elevation: 45,
		Latitude:  38.7779006958,
		Longitude: -75.5832977295,
		Timezone:  "America/New_York",
	},
	"DE19": {
		ICAO:      "DE19",
		Name:      "Duffy's Airport",
		City:      "Townsend",
		State:     "Delaware",
		Country:   "US",
		Elevation: 69,
		Latitude:  39.3258018494,
		Longitude: -75.7416000366,
		Timezone:  "America/New_York",
	},
	"DE20": {
		ICAO:      "DE20",
		Name:      "Spirit Airpark",
		City:      "Townsend",
		State:     "Delaware",
		Country:   "US",
		Elevation: 80,
		Latitude:  39.3736000061,
		Longitude: -75.7457962036,
		Timezone:  "America/New_York",
	},
	"DE21": {
		ICAO:      "DE21",
		Name:      "West Private Airport",
		City:      "Roxana",
		State:     "Delaware",
		Country:   "US",
		Elevation: 21,
		Latitude:  38.5172996521,
		Longitude: -75.1654968262,
		Timezone:  "America/New_York",
	},
	"DE23": {
		ICAO:      "DE23",
		Name:      "Ockel Farms Airport",
		City:      "Milton",
		State:     "Delaware",
		Country:   "US",
		Elevation: 47,
		Latitude:  38.7503013611,
		Longitude: -75.3600006104,
		Timezone:  "America/New_York",
	},
	"DE25": {
		ICAO:      "DE25",
		Name:      "Eagle Crest-Hudson Airport",
		City:      "Milton",
		State:     "Delaware",
		Country:   "US",
		Elevation: 28,
		Latitude:  38.7761001587,
		Longitude: -75.2335968018,
		Timezone:  "America/New_York",
	},
	"DE27": {
		ICAO:      "DE27",
		Name:      "Warrington Field",
		City:      "Selbyville",
		State:     "Delaware",
		Country:   "US",
		Elevation: 26,
		Latitude:  38.462600708,
		Longitude: -75.1748962402,
		Timezone:  "America/New_York",
	},
	"DE29": {
		ICAO:      "DE29",
		Name:      "Scotty's Place Airport",
		City:      "Smyrna",
		State:     "Delaware",
		Country:   "US",
		Elevation: 40,
		Latitude:  39.3750991821,
		Longitude: -75.6440963745,
		Timezone:  "America/New_York",
	},
	"DE32": {
		ICAO:      "DE32",
		Name:      "Belfair Airport",
		City:      "Felton",
		State:     "Delaware",
		Country:   "US",
		Elevation: 64,
		Latitude:  38.9749984741,
		Longitude: -75.6166992188,
		Timezone:  "America/New_York",
	},
	"DE33": {
		ICAO:      "DE33",
		Name:      "Okolona Plantation Airport",
		City:      "Middletown",
		State:     "Delaware",
		Country:   "US",
		Elevation: 61,
		Latitude:  39.48279953,
		Longitude: -75.6975021362,
		Timezone:  "America/New_York",
	},
	"DE34": {
		ICAO:      "DE34",
		Name:      "Townsend A Airport",
		City:      "Townsend",
		State:     "Delaware",
		Country:   "US",
		Elevation: 65,
		Latitude:  39.4142990112,
		Longitude: -75.6630020142,
		Timezone:  "America/New_York",
	},
	"DE49": {
		ICAO:      "DE49",
		Name:      "Josephs Airport",
		City:      "Georgetown",
		State:     "Delaware",
		Country:   "US",
		Elevation: 42,
		Latitude:  38.6375999451,
		Longitude: -75.3552017212,
		Timezone:  "America/New_York",
	},
	"DFCA": {
		ICAO:      "DFCA",
		IATA:      "XKY",
		Name:      "Kaya Airport",
		City:      "Kaya",
		Country:   "BF",
		Elevation: 984,
		Latitude:  13.0670003891,
		Longitude: -1.1000000238,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFCB": {
		ICAO:      "DFCB",
		Name:      "Barsalogho Airport",
		City:      "Barsalogho",
		Country:   "BF",
		Elevation: 1083,
		Latitude:  13.3999996185,
		Longitude: -1.0670000315,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFCC": {
		ICAO:      "DFCC",
		IATA:      "OUG",
		Name:      "Ouahigouya Airport",
		City:      "Ouahigouya",
		Country:   "BF",
		Elevation: 1106,
		Latitude:  13.5670003891,
		Longitude: -2.4170000553,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFCD": {
		ICAO:      "DFCD",
		Name:      "Didyr Airport",
		City:      "Didyr",
		State:     "Centre-Ouest",
		Country:   "BF",
		Elevation: 958,
		Latitude:  12.5500001907,
		Longitude: -2.617000103,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFCE": {
		ICAO:      "DFCE",
		Name:      "Batie Airport",
		City:      "Batie",
		State:     "Sud-Ouest",
		Country:   "BF",
		Elevation: 984,
		Latitude:  9.8830003738,
		Longitude: -2.9170000553,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFCG": {
		ICAO:      "DFCG",
		Name:      "Kongoussi Airport",
		City:      "Kongoussi",
		State:     "Centre-Nord",
		Country:   "BF",
		Elevation: 1056,
		Latitude:  13.3170003891,
		Longitude: -1.5329999924,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFCI": {
		ICAO:      "DFCI",
		Name:      "Titao Airport",
		City:      "Titao",
		State:     "Nord",
		Country:   "BF",
		Elevation: 1047,
		Latitude:  13.7670001984,
		Longitude: -2.0829999447,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFCJ": {
		ICAO:      "DFCJ",
		IATA:      "XDJ",
		Name:      "Djibo Airport",
		City:      "Djibo",
		State:     "Sahel",
		Country:   "BF",
		Elevation: 1001,
		Latitude:  14.1000003815,
		Longitude: -1.6330000162,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFCK": {
		ICAO:      "DFCK",
		Name:      "Koudougou Airport",
		City:      "Koudougou",
		State:     "Centre-Ouest",
		Country:   "BF",
		Elevation: 1001,
		Latitude:  12.2670001984,
		Longitude: -2.4000000954,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFCL": {
		ICAO:      "DFCL",
		IATA:      "XLU",
		Name:      "Leo Airport",
		City:      "Leo",
		Country:   "BF",
		Elevation: 1181,
		Latitude:  11.1000003815,
		Longitude: -2.0999999046,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFCM": {
		ICAO:      "DFCM",
		Name:      "Manga Airport",
		City:      "Manga",
		State:     "Centre-Sud",
		Country:   "BF",
		Elevation: 820,
		Latitude:  11.6669998169,
		Longitude: -1.0670000315,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFCP": {
		ICAO:      "DFCP",
		IATA:      "PUP",
		Name:      "Po Airport",
		City:      "Po",
		State:     "Centre-Sud",
		Country:   "BF",
		Elevation: 1050,
		Latitude:  11.1499996185,
		Longitude: -1.1499999762,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFCR": {
		ICAO:      "DFCR",
		Name:      "Poura Airport",
		City:      "Poura",
		State:     "Boucle-du-Mouhoun",
		Country:   "BF",
		Elevation: 853,
		Latitude:  11.6169996262,
		Longitude: -2.75,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFCS": {
		ICAO:      "DFCS",
		Name:      "Seguenega Airport",
		City:      "Seguenega",
		State:     "Nord",
		Country:   "BF",
		Elevation: 1139,
		Latitude:  13.4329996109,
		Longitude: -2,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFCT": {
		ICAO:      "DFCT",
		Name:      "Tenado Airport",
		City:      "Tenado",
		State:     "Centre-Ouest",
		Country:   "BF",
		Elevation: 951,
		Latitude:  12.1999998093,
		Longitude: -2.5829999447,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFCU": {
		ICAO:      "DFCU",
		Name:      "Gourcy Airport",
		City:      "Gourcy",
		State:     "Nord",
		Country:   "BF",
		Elevation: 1050,
		Latitude:  13.1999998093,
		Longitude: -2.367000103,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFCY": {
		ICAO:      "DFCY",
		Name:      "Yako Airport",
		City:      "Yako",
		State:     "Nord",
		Country:   "BF",
		Elevation: 984,
		Latitude:  12.9499998093,
		Longitude: -2.2829999924,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFEA": {
		ICAO:      "DFEA",
		IATA:      "XBO",
		Name:      "Boulsa Airport",
		City:      "Boulsa",
		Country:   "BF",
		Elevation: 984,
		Latitude:  12.6499996185,
		Longitude: -0.5669999719,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFEB": {
		ICAO:      "DFEB",
		IATA:      "XBG",
		Name:      "Bogande Airport",
		City:      "Bogande",
		Country:   "BF",
		Elevation: 984,
		Latitude:  12.9829998016,
		Longitude: -0.1669999957,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFEC": {
		ICAO:      "DFEC",
		Name:      "Komin-Yanga Airport",
		City:      "Komin-Yanga",
		State:     "Centre-Est",
		Country:   "BF",
		Elevation: 1116,
		Latitude:  11.6999998093,
		Longitude: 0.150000006,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFED": {
		ICAO:      "DFED",
		IATA:      "DIP",
		Name:      "Diapaga Airport",
		City:      "Diapaga",
		Country:   "BF",
		Elevation: 951,
		Latitude:  12.0500001907,
		Longitude: 1.7829999924,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFEE": {
		ICAO:      "DFEE",
		IATA:      "DOR",
		Name:      "Dori Airport",
		City:      "Dori",
		State:     "Sahel",
		Country:   "BF",
		Elevation: 919,
		Latitude:  14.0329999924,
		Longitude: -0.0329999998,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFEF": {
		ICAO:      "DFEF",
		IATA:      "FNG",
		Name:      "Fada N'gourma Airport",
		City:      "Fada N'gourma",
		State:     "Est",
		Country:   "BF",
		Elevation: 1011,
		Latitude:  12.0329999924,
		Longitude: 0.349999994,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFEG": {
		ICAO:      "DFEG",
		IATA:      "XGG",
		Name:      "Gorom-Gorom Airport",
		City:      "Gorom-Gorom",
		State:     "Sahel",
		Country:   "BF",
		Elevation: 935,
		Latitude:  14.4499998093,
		Longitude: -0.2329999954,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFEK": {
		ICAO:      "DFEK",
		Name:      "Koupela Airport",
		City:      "Koupela",
		State:     "Centre-Est",
		Country:   "BF",
		Elevation: 902,
		Latitude:  12.1669998169,
		Longitude: -0.3000000119,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFEL": {
		ICAO:      "DFEL",
		IATA:      "XKA",
		Name:      "Kantchari Airport",
		City:      "Kantchari",
		Country:   "BF",
		Elevation: 879,
		Latitude:  12.4670000076,
		Longitude: 1.5,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFEM": {
		ICAO:      "DFEM",
		IATA:      "TMQ",
		Name:      "Tambao Airport",
		City:      "Tambao",
		State:     "Sahel",
		Country:   "BF",
		Elevation: 820,
		Latitude:  14.8000001907,
		Longitude: 0.0500000007,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFEO": {
		ICAO:      "DFEO",
		Name:      "Zorgo Airport",
		City:      "Zorgo",
		State:     "Plateau-Central",
		Country:   "BF",
		Elevation: 978,
		Latitude:  12.25,
		Longitude: -0.6169999838,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFEP": {
		ICAO:      "DFEP",
		IATA:      "XPA",
		Name:      "Pama Airport",
		City:      "Pama",
		Country:   "BF",
		Elevation: 699,
		Latitude:  11.25,
		Longitude: 0.6999999881,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFER": {
		ICAO:      "DFER",
		IATA:      "ARL",
		Name:      "Arly Airport",
		City:      "Arly",
		Country:   "BF",
		Elevation: 761,
		Latitude:  11.5970001221,
		Longitude: 1.4830000401,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFES": {
		ICAO:      "DFES",
		IATA:      "XSE",
		Name:      "Sebba Airport",
		City:      "Sebba",
		State:     "Sahel",
		Country:   "BF",
		Elevation: 886,
		Latitude:  13.4499998093,
		Longitude: 0.5170000196,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFET": {
		ICAO:      "DFET",
		IATA:      "TEG",
		Name:      "Tenkodogo Airport",
		City:      "Tenkodogo",
		Country:   "BF",
		Elevation: 1017,
		Latitude:  11.8000001907,
		Longitude: -0.3670000136,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFEY": {
		ICAO:      "DFEY",
		Name:      "Ouargaye Airport",
		City:      "Ouargaye",
		State:     "Centre-Est",
		Country:   "BF",
		Elevation: 951,
		Latitude:  11.5170001984,
		Longitude: 0.0500000007,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFEZ": {
		ICAO:      "DFEZ",
		IATA:      "XZA",
		Name:      "Zabre Airport",
		City:      "Zabre",
		Country:   "BF",
		Elevation: 886,
		Latitude:  11.1669998169,
		Longitude: -0.6169999838,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFFD": {
		ICAO:      "DFFD",
		IATA:      "OUA",
		Name:      "Ouagadougou Airport",
		City:      "Ouagadougou",
		Country:   "BF",
		Elevation: 1037,
		Latitude:  12.3531999588,
		Longitude: -1.5124200583,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFOA": {
		ICAO:      "DFOA",
		Name:      "Dano Airport",
		City:      "Dano",
		State:     "Sud-Ouest",
		Country:   "BF",
		Elevation: 935,
		Latitude:  11.1330003738,
		Longitude: -3.0669999123,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFOB": {
		ICAO:      "DFOB",
		IATA:      "BNR",
		Name:      "Banfora Airport",
		City:      "Banfora",
		State:     "Cascades",
		Country:   "BF",
		Elevation: 984,
		Latitude:  10.6829996109,
		Longitude: -4.7170000076,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFOD": {
		ICAO:      "DFOD",
		IATA:      "DGU",
		Name:      "Dedougou Airport",
		City:      "Dedougou",
		Country:   "BF",
		Elevation: 984,
		Latitude:  12.4589996338,
		Longitude: -3.4900000095,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFOF": {
		ICAO:      "DFOF",
		Name:      "Safane Airport",
		City:      "Safane",
		State:     "Boucle-du-Mouhoun",
		Country:   "BF",
		Elevation: 1017,
		Latitude:  12.1499996185,
		Longitude: -3.1830000877,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFOG": {
		ICAO:      "DFOG",
		IATA:      "XGA",
		Name:      "Gaoua Airport",
		City:      "Gaoua",
		State:     "Sud-Ouest",
		Country:   "BF",
		Elevation: 912,
		Latitude:  10.299200058,
		Longitude: -3.2508299351,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFOH": {
		ICAO:      "DFOH",
		Name:      "Hounde Airport",
		City:      "Hounde",
		State:     "Hauts-Bassins",
		Country:   "BF",
		Elevation: 1063,
		Latitude:  11.4829998016,
		Longitude: -3.5169999599,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFOL": {
		ICAO:      "DFOL",
		Name:      "Loumana Airport",
		City:      "Loumana",
		State:     "Cascades",
		Country:   "BF",
		Elevation: 1148,
		Latitude:  10.5670003891,
		Longitude: -5.3499999046,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFON": {
		ICAO:      "DFON",
		IATA:      "XNU",
		Name:      "Nouna Airport",
		City:      "Nouna",
		Country:   "BF",
		Elevation: 886,
		Latitude:  12.75,
		Longitude: -3.867000103,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFOO": {
		ICAO:      "DFOO",
		IATA:      "BOY",
		Name:      "Bobo Dioulasso Airport",
		City:      "Bobo Dioulasso",
		State:     "Hauts-Bassins",
		Country:   "BF",
		Elevation: 1511,
		Latitude:  11.1600999832,
		Longitude: -4.3309698105,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFOR": {
		ICAO:      "DFOR",
		Name:      "Orodara Airport",
		City:      "Orodara",
		Country:   "BF",
		Elevation: 1706,
		Latitude:  10.9829998016,
		Longitude: -4.9330000877,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFOS": {
		ICAO:      "DFOS",
		Name:      "Sideradougou Airport",
		City:      "Sideradougou",
		State:     "Cascades",
		Country:   "BF",
		Elevation: 1047,
		Latitude:  10.6669998169,
		Longitude: -4.2670001984,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFOT": {
		ICAO:      "DFOT",
		IATA:      "TUQ",
		Name:      "Tougan Airport",
		City:      "Tougan",
		State:     "Boucle-du-Mouhoun",
		Country:   "BF",
		Elevation: 984,
		Latitude:  13.0670003891,
		Longitude: -3.0669999123,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFOU": {
		ICAO:      "DFOU",
		IATA:      "XDE",
		Name:      "Diebougou Airport",
		City:      "Diebougou",
		Country:   "BF",
		Elevation: 984,
		Latitude:  10.9499998093,
		Longitude: -3.25,
		Timezone:  "Africa/Ouagadougou",
	},
	"DFOY": {
		ICAO:      "DFOY",
		IATA:      "XAR",
		Name:      "Aribinda Airport",
		City:      "Aribinda",
		State:     "Sahel",
		Country:   "BF",
		Elevation: 1122,
		Latitude:  14.2170000076,
		Longitude: -0.8830000162,
		Timezone:  "Africa/Ouagadougou",
	},
	"DGAA": {
		ICAO:      "DGAA",
		IATA:      "ACC",
		Name:      "Kotoka International Airport",
		City:      "Accra",
		Country:   "GH",
		Elevation: 205,
		Latitude:  5.6051898003,
		Longitude: -0.1667860001,
		Timezone:  "Africa/Accra",
	},
	"DGLE": {
		ICAO:      "DGLE",
		IATA:      "TML",
		Name:      "Tamale Airport",
		City:      "Tamale",
		Country:   "GH",
		Elevation: 553,
		Latitude:  9.5571899414,
		Longitude: -0.863214016,
		Timezone:  "Africa/Accra",
	},
	"DGLN": {
		ICAO:      "DGLN",
		Name:      "Navrongo Airport",
		City:      "Navrongo",
		Country:   "GH",
		Elevation: 2221,
		Latitude:  10.8999996185,
		Longitude: -1.1000000238,
		Timezone:  "Africa/Accra",
	},
	"DGLW": {
		ICAO:      "DGLW",
		Name:      "Wa Airport",
		City:      "Wa",
		Country:   "GH",
		Elevation: 1060,
		Latitude:  10.0826997757,
		Longitude: -2.5076899529,
		Timezone:  "Africa/Accra",
	},
	"DGLY": {
		ICAO:      "DGLY",
		Name:      "Yendi Airport",
		City:      "Yendi",
		State:     "Northern",
		Country:   "GH",
		Elevation: 2408,
		Latitude:  9.4250001907,
		Longitude: -0.0047220001,
		Timezone:  "Africa/Accra",
	},
	"DGSI": {
		ICAO:      "DGSI",
		IATA:      "KMS",
		Name:      "Kumasi Airport",
		City:      "Kumasi",
		Country:   "GH",
		Elevation: 942,
		Latitude:  6.7145600319,
		Longitude: -1.5908199549,
		Timezone:  "Africa/Accra",
	},
	"DGSN": {
		ICAO:      "DGSN",
		IATA:      "NYI",
		Name:      "Sunyani Airport",
		City:      "Sunyani",
		State:     "Brong-Ahafo",
		Country:   "GH",
		Elevation: 1014,
		Latitude:  7.3618302345,
		Longitude: -2.3287599087,
		Timezone:  "Africa/Accra",
	},
	"DGTK": {
		ICAO:      "DGTK",
		IATA:      "TKD",
		Name:      "Takoradi Airport",
		City:      "Sekondi-Takoradi",
		State:     "Western",
		Country:   "GH",
		Elevation: 21,
		Latitude:  4.8960599899,
		Longitude: -1.7747600079,
		Timezone:  "Africa/Accra",
	},
	"DIAP": {
		ICAO:      "DIAP",
		IATA:      "ABJ",
		Name:      "Port Bouet Airport",
		City:      "Abidjan",
		State:     "Lagunes",
		Country:   "CI",
		Elevation: 21,
		Latitude:  5.2613902092,
		Longitude: -3.9262900352,
		Timezone:  "Africa/Abidjan",
	},
	"DIAU": {
		ICAO:      "DIAU",
		IATA:      "OGO",
		Name:      "Abengourou Airport",
		City:      "Abengourou",
		Country:   "CI",
		Elevation: 676,
		Latitude:  6.7155599594,
		Longitude: -3.470279932,
		Timezone:  "Africa/Abidjan",
	},
	"DIBC": {
		ICAO:      "DIBC",
		Name:      "Bocanda Airport",
		City:      "Bocanda",
		Country:   "CI",
		Elevation: 427,
		Latitude:  7.0329999924,
		Longitude: -4.5329999924,
		Timezone:  "Africa/Abidjan",
	},
	"DIBI": {
		ICAO:      "DIBI",
		IATA:      "BXI",
		Name:      "Boundiali Airport",
		City:      "Boundiali",
		Country:   "CI",
		Elevation: 1286,
		Latitude:  9.5329999924,
		Longitude: -6.4670000076,
		Timezone:  "Africa/Abidjan",
	},
	"DIBK": {
		ICAO:      "DIBK",
		IATA:      "BYK",
		Name:      "Bouake Airport",
		State:     "Vallee-du-Bandama",
		Country:   "CI",
		Elevation: 1230,
		Latitude:  7.7388000488,
		Longitude: -5.0736699104,
		Timezone:  "Africa/Abidjan",
	},
	"DIBN": {
		ICAO:      "DIBN",
		IATA:      "BQO",
		Name:      "Bouna Airport",
		City:      "Bouna",
		Country:   "CI",
		Elevation: 1148,
		Latitude:  9.2775001526,
		Longitude: -3.0252799988,
		Timezone:  "Africa/Abidjan",
	},
	"DIBU": {
		ICAO:      "DIBU",
		IATA:      "BDK",
		Name:      "Soko Airport",
		City:      "Bondoukou",
		Country:   "CI",
		Elevation: 1247,
		Latitude:  8.0172195435,
		Longitude: -2.7619400024,
		Timezone:  "Africa/Abidjan",
	},
	"DIDB": {
		ICAO:      "DIDB",
		Name:      "Dabou Airport",
		City:      "Dabou",
		Country:   "CI",
		Elevation: 141,
		Latitude:  5.3513851166,
		Longitude: -4.4034576416,
		Timezone:  "Africa/Abidjan",
	},
	"DIDK": {
		ICAO:      "DIDK",
		IATA:      "DIM",
		Name:      "Dimbokro Airport",
		City:      "Dimbokro",
		State:     "Lacs",
		Country:   "CI",
		Elevation: 344,
		Latitude:  6.6516699791,
		Longitude: -4.6405601501,
		Timezone:  "Africa/Abidjan",
	},
	"DIDL": {
		ICAO:      "DIDL",
		IATA:      "DJO",
		Name:      "Daloa Airport",
		State:     "Sassandra-Marahoue",
		Country:   "CI",
		Elevation: 823,
		Latitude:  6.7928099632,
		Longitude: -6.4731898308,
		Timezone:  "Africa/Abidjan",
	},
	"DIFK": {
		ICAO:      "DIFK",
		IATA:      "FEK",
		Name:      "Ferkessedougou Airport",
		City:      "Ferkessedougou",
		Country:   "CI",
		Elevation: 1102,
		Latitude:  9.6000003815,
		Longitude: -5.1999998093,
		Timezone:  "Africa/Abidjan",
	},
	"DIGA": {
		ICAO:      "DIGA",
		IATA:      "GGN",
		Name:      "Gagnoa Airport",
		City:      "Gagnoa",
		State:     "Goh-Djiboua",
		Country:   "CI",
		Elevation: 732,
		Latitude:  6.132999897,
		Longitude: -5.9499998093,
		Timezone:  "Africa/Abidjan",
	},
	"DIGL": {
		ICAO:      "DIGL",
		IATA:      "GGO",
		Name:      "Guiglo Airport",
		City:      "Guiglo",
		Country:   "CI",
		Elevation: 722,
		Latitude:  6.5249900818,
		Longitude: -7.4786100388,
		Timezone:  "Africa/Abidjan",
	},
	"DIGN": {
		ICAO:      "DIGN",
		IATA:      "BBV",
		Name:      "Nero-Mer Airport",
		City:      "Grand-Bereby",
		State:     "Bas-Sassandra",
		Country:   "CI",
		Elevation: 20,
		Latitude:  4.6434130669,
		Longitude: -6.9239616394,
		Timezone:  "Africa/Abidjan",
	},
	"DIKO": {
		ICAO:      "DIKO",
		IATA:      "HGO",
		Name:      "Korhogo Airport",
		State:     "Savanes",
		Country:   "CI",
		Elevation: 1214,
		Latitude:  9.3871803284,
		Longitude: -5.5566601753,
		Timezone:  "Africa/Abidjan",
	},
	"DIMN": {
		ICAO:      "DIMN",
		IATA:      "MJC",
		Name:      "Man Airport",
		State:     "Montagnes",
		Country:   "CI",
		Elevation: 1089,
		Latitude:  7.272069931,
		Longitude: -7.5873599052,
		Timezone:  "Africa/Abidjan",
	},
	"DIOD": {
		ICAO:      "DIOD",
		IATA:      "KEO",
		Name:      "Odienne Airport",
		City:      "Odienne",
		State:     "Denguele",
		Country:   "CI",
		Elevation: 1365,
		Latitude:  9.5,
		Longitude: -7.5669999123,
		Timezone:  "Africa/Abidjan",
	},
	"DIOF": {
		ICAO:      "DIOF",
		IATA:      "OFI",
		Name:      "Ouango Fitini Airport",
		City:      "Ouango Fitini",
		Country:   "CI",
		Elevation: 974,
		Latitude:  9.6000003815,
		Longitude: -4.0500001907,
		Timezone:  "Africa/Abidjan",
	},
	"DISG": {
		ICAO:      "DISG",
		IATA:      "SEO",
		Name:      "Seguela Airport",
		City:      "Seguela",
		State:     "Woroba",
		Country:   "CI",
		Elevation: 1056,
		Latitude:  7.9683299065,
		Longitude: -6.7108302116,
		Timezone:  "Africa/Abidjan",
	},
	"DISP": {
		ICAO:      "DISP",
		IATA:      "SPY",
		Name:      "San Pedro Airport",
		State:     "Bas-Sassandra",
		Country:   "CI",
		Elevation: 26,
		Latitude:  4.7467198372,
		Longitude: -6.6608200073,
		Timezone:  "Africa/Abidjan",
	},
	"DISS": {
		ICAO:      "DISS",
		IATA:      "ZSS",
		Name:      "Sassandra Airport",
		City:      "Sassandra",
		Country:   "CI",
		Elevation: 203,
		Latitude:  4.9283299446,
		Longitude: -6.1327800751,
		Timezone:  "Africa/Abidjan",
	},
	"DITB": {
		ICAO:      "DITB",
		IATA:      "TXU",
		Name:      "Tabou Airport",
		City:      "Tabou",
		Country:   "CI",
		Elevation: 39,
		Latitude:  4.4378094673,
		Longitude: -7.3627281189,
		Timezone:  "Africa/Abidjan",
	},
	"DIYO": {
		ICAO:      "DIYO",
		IATA:      "ASK",
		Name:      "Yamoussoukro Airport",
		City:      "Yamoussoukro",
		State:     "Yamoussoukro-Autonomous-District",
		Country:   "CI",
		Elevation: 699,
		Latitude:  6.9031701088,
		Longitude: -5.3655800819,
		Timezone:  "Africa/Abidjan",
	},
	"DN50": {
		ICAO:      "DN50",
		Name:      "Shiroro Airport",
		State:     "Niger",
		Country:   "NG",
		Elevation: 1305,
		Latitude:  9.8846502304,
		Longitude: 6.8186302185,
		Timezone:  "Africa/Lagos",
	},
	"DN51": {
		ICAO:      "DN51",
		Name:      "Ajaokuta Airport",
		State:     "Kogi",
		Country:   "NG",
		Elevation: 620,
		Latitude:  7.4572401047,
		Longitude: 6.4611301422,
		Timezone:  "Africa/Lagos",
	},
	"DN53": {
		ICAO:      "DN53",
		Name:      "Kaduna Old Airport",
		State:     "Kaduna",
		Country:   "NG",
		Elevation: 2126,
		Latitude:  10.5988998413,
		Longitude: 7.4487099648,
		Timezone:  "Africa/Lagos",
	},
	"DN54": {
		ICAO:      "DN54",
		Name:      "Bajoga Northeast Airport",
		City:      "Bajoga",
		State:     "Gombe",
		Country:   "NG",
		Elevation: 1000,
		Latitude:  10.9198999405,
		Longitude: 11.5011997223,
		Timezone:  "Africa/Lagos",
	},
	"DN55": {
		ICAO:      "DN55",
		Name:      "Eket Airport",
		State:     "Akwa-Ibom",
		Country:   "NG",
		Elevation: 42,
		Latitude:  4.642179966,
		Longitude: 7.9490399361,
		Timezone:  "Africa/Lagos",
	},
	"DN56": {
		ICAO:      "DN56",
		Name:      "Escravos Airport",
		State:     "Delta",
		Country:   "NG",
		Elevation: 6,
		Latitude:  5.6253199577,
		Longitude: 5.1923799515,
		Timezone:  "Africa/Lagos",
	},
	"DNAA": {
		ICAO:      "DNAA",
		IATA:      "ABV",
		Name:      "Nnamdi Azikiwe International Airport",
		City:      "Abuja",
		State:     "FCT",
		Country:   "NG",
		Elevation: 1123,
		Latitude:  9.0067901611,
		Longitude: 7.2631697655,
		Timezone:  "Africa/Lagos",
	},
	"DNAK": {
		ICAO:      "DNAK",
		IATA:      "AKR",
		Name:      "Akure Airport",
		City:      "Akure",
		State:     "Ondo",
		Country:   "NG",
		Elevation: 1100,
		Latitude:  7.2467398643,
		Longitude: 5.3010101318,
		Timezone:  "Africa/Lagos",
	},
	"DNAS": {
		ICAO:      "DNAS",
		IATA:      "ABB",
		Name:      "Asaba International Airport",
		City:      "Asaba",
		State:     "Delta",
		Country:   "NG",
		Elevation: 305,
		Latitude:  6.2033333333,
		Longitude: 6.6588888889,
		Timezone:  "Africa/Lagos",
	},
	"DNBE": {
		ICAO:      "DNBE",
		IATA:      "BNI",
		Name:      "Benin Airport",
		City:      "Benin",
		State:     "Edo",
		Country:   "NG",
		Elevation: 258,
		Latitude:  6.3169798851,
		Longitude: 5.5995001793,
		Timezone:  "Africa/Lagos",
	},
	"DNBI": {
		ICAO:      "DNBI",
		Name:      "Bida Airport",
		City:      "Bida",
		State:     "Niger",
		Country:   "NG",
		Elevation: 450,
		Latitude:  9.1000003815,
		Longitude: 6.0170001984,
		Timezone:  "Africa/Lagos",
	},
	"DNCA": {
		ICAO:      "DNCA",
		IATA:      "CBQ",
		Name:      "Margaret Ekpo International Airport",
		City:      "Calabar",
		State:     "Cross-River",
		Country:   "NG",
		Elevation: 210,
		Latitude:  4.9760198593,
		Longitude: 8.3472003937,
		Timezone:  "Africa/Lagos",
	},
	"DNEN": {
		ICAO:      "DNEN",
		IATA:      "ENU",
		Name:      "Akanu Ibiam International Airport",
		City:      "Enegu",
		State:     "Enugu",
		Country:   "NG",
		Elevation: 466,
		Latitude:  6.4742698669,
		Longitude: 7.5619602203,
		Timezone:  "Africa/Lagos",
	},
	"DNGO": {
		ICAO:      "DNGO",
		IATA:      "GMO",
		Name:      "Gombe Lawanti International Airport",
		City:      "Gombe",
		State:     "Bauchi",
		Country:   "NG",
		Elevation: 1590,
		Latitude:  10.2983333333,
		Longitude: 10.8963888889,
		Timezone:  "Africa/Lagos",
	},
	"DNGU": {
		ICAO:      "DNGU",
		IATA:      "QUS",
		Name:      "Gusau Airport",
		City:      "Gusau",
		State:     "Zamfara",
		Country:   "NG",
		Elevation: 1520,
		Latitude:  12.1716995239,
		Longitude: 6.6961097717,
		Timezone:  "Africa/Lagos",
	},
	"DNIB": {
		ICAO:      "DNIB",
		IATA:      "IBA",
		Name:      "Ibadan Airport",
		City:      "Ibadan",
		State:     "Oyo",
		Country:   "NG",
		Elevation: 725,
		Latitude:  7.3624601364,
		Longitude: 3.9783298969,
		Timezone:  "Africa/Lagos",
	},
	"DNIL": {
		ICAO:      "DNIL",
		IATA:      "ILR",
		Name:      "Ilorin International Airport",
		City:      "Ilorin",
		State:     "Kwara",
		Country:   "NG",
		Elevation: 1126,
		Latitude:  8.4402103424,
		Longitude: 4.4939198494,
		Timezone:  "Africa/Lagos",
	},
	"DNIM": {
		ICAO:      "DNIM",
		IATA:      "QOW",
		Name:      "Sam Mbakwe International Airport",
		City:      "Owerri",
		State:     "Imo",
		Country:   "NG",
		Elevation: 373,
		Latitude:  5.4270601273,
		Longitude: 7.206029892,
		Timezone:  "Africa/Lagos",
	},
	"DNJO": {
		ICAO:      "DNJO",
		IATA:      "JOS",
		Name:      "Yakubu Gowon Airport",
		City:      "Jos",
		State:     "Plateau",
		Country:   "NG",
		Elevation: 4232,
		Latitude:  9.6398296356,
		Longitude: 8.8690500259,
		Timezone:  "Africa/Lagos",
	},
	"DNKA": {
		ICAO:      "DNKA",
		IATA:      "KAD",
		Name:      "Kaduna Airport",
		City:      "Kaduna",
		State:     "Kaduna",
		Country:   "NG",
		Elevation: 2073,
		Latitude:  10.6960000992,
		Longitude: 7.3201098442,
		Timezone:  "Africa/Lagos",
	},
	"DNKN": {
		ICAO:      "DNKN",
		IATA:      "KAN",
		Name:      "Mallam Aminu International Airport",
		City:      "Kano",
		State:     "Kano",
		Country:   "NG",
		Elevation: 1562,
		Latitude:  12.0475997925,
		Longitude: 8.5246200562,
		Timezone:  "Africa/Lagos",
	},
	"DNMA": {
		ICAO:      "DNMA",
		IATA:      "MIU",
		Name:      "Maiduguri International Airport",
		City:      "Maiduguri",
		State:     "Borno",
		Country:   "NG",
		Elevation: 1099,
		Latitude:  11.8552999496,
		Longitude: 13.0809001923,
		Timezone:  "Africa/Lagos",
	},
	"DNMK": {
		ICAO:      "DNMK",
		IATA:      "MDI",
		Name:      "Makurdi Airport",
		City:      "Makurdi",
		State:     "Benue",
		Country:   "NG",
		Elevation: 371,
		Latitude:  7.7038798332,
		Longitude: 8.613940239,
		Timezone:  "Africa/Lagos",
	},
	"DNMM": {
		ICAO:      "DNMM",
		IATA:      "LOS",
		Name:      "Murtala Muhammed International Airport",
		City:      "Lagos",
		State:     "Lagos",
		Country:   "NG",
		Elevation: 135,
		Latitude:  6.5773701668,
		Longitude: 3.321160078,
		Timezone:  "Africa/Lagos",
	},
	"DNMN": {
		ICAO:      "DNMN",
		IATA:      "MXJ",
		Name:      "Minna Airport",
		City:      "Minna",
		State:     "Niger",
		Country:   "NG",
		Elevation: 834,
		Latitude:  9.6521701813,
		Longitude: 6.4622597694,
		Timezone:  "Africa/Lagos",
	},
	"DNOS": {
		ICAO:      "DNOS",
		Name:      "Oshogbo Airport",
		City:      "Oshogbo",
		State:     "Osun",
		Country:   "NG",
		Elevation: 997,
		Latitude:  7.7829999924,
		Longitude: 4.4829998016,
		Timezone:  "Africa/Lagos",
	},
	"DNPO": {
		ICAO:      "DNPO",
		IATA:      "PHC",
		Name:      "Port Harcourt International Airport",
		City:      "Port Harcourt",
		State:     "Rivers",
		Country:   "NG",
		Elevation: 87,
		Latitude:  5.0154900551,
		Longitude: 6.9495902061,
		Timezone:  "Africa/Lagos",
	},
	"DNSO": {
		ICAO:      "DNSO",
		IATA:      "SKO",
		Name:      "Sadiq Abubakar III International Airport",
		City:      "Sokoto",
		State:     "Sokoto",
		Country:   "NG",
		Elevation: 1010,
		Latitude:  12.9162998199,
		Longitude: 5.2071900368,
		Timezone:  "Africa/Lagos",
	},
	"DNTZ": {
		ICAO:      "DNTZ",
		IATA:      "NBE",
		Name:      "Enfidha Zine El Abidine Ben Ali International Airport",
		City:      "Enfidha",
		State:     "Susah",
		Country:   "TN",
		Elevation: 21,
		Latitude:  36.075833,
		Longitude: 10.438611,
		Timezone:  "Africa/Tunis",
	},
	"DNYO": {
		ICAO:      "DNYO",
		IATA:      "YOL",
		Name:      "Yola Airport",
		City:      "Yola",
		State:     "Adamawa",
		Country:   "NG",
		Elevation: 599,
		Latitude:  9.2575502396,
		Longitude: 12.4303998947,
		Timezone:  "Africa/Lagos",
	},
	"DNZA": {
		ICAO:      "DNZA",
		IATA:      "ZAR",
		Name:      "Zaria Airport",
		City:      "Zaria",
		State:     "Kaduna",
		Country:   "NG",
		Elevation: 2170,
		Latitude:  11.130200386,
		Longitude: 7.6858100891,
		Timezone:  "Africa/Lagos",
	},
	"DRRA": {
		ICAO:      "DRRA",
		Name:      "Tessaoua Airport",
		City:      "Tessaoua",
		State:     "Maradi",
		Country:   "NE",
		Elevation: 1358,
		Latitude:  13.7670001984,
		Longitude: 8.0170001984,
		Timezone:  "Africa/Niamey",
	},
	"DRRB": {
		ICAO:      "DRRB",
		Name:      "Birni-N'Konni Airport",
		City:      "Birni-N'Konni",
		State:     "Tahoua",
		Country:   "NE",
		Elevation: 846,
		Latitude:  13.8015003204,
		Longitude: 5.2423601151,
		Timezone:  "Africa/Niamey",
	},
	"DRRC": {
		ICAO:      "DRRC",
		Name:      "Dogondoutchi Airport",
		City:      "Dogondoutchi",
		State:     "Dosso",
		Country:   "NE",
		Elevation: 961,
		Latitude:  13.6669998169,
		Longitude: 4.0999999046,
		Timezone:  "Africa/Niamey",
	},
	"DRRD": {
		ICAO:      "DRRD",
		Name:      "Dosso Airport",
		City:      "Dosso",
		State:     "Dosso",
		Country:   "NE",
		Elevation: 738,
		Latitude:  13.0481247896,
		Longitude: 3.2209682465,
		Timezone:  "Africa/Niamey",
	},
	"DRRE": {
		ICAO:      "DRRE",
		Name:      "Tera Airport",
		City:      "Tera",
		State:     "Niamey",
		Country:   "NE",
		Elevation: 850,
		Latitude:  13.9499998093,
		Longitude: 0.7329999804,
		Timezone:  "Africa/Niamey",
	},
	"DRRG": {
		ICAO:      "DRRG",
		Name:      "Gaya Airport",
		City:      "Gaya",
		State:     "Dosso",
		Country:   "NE",
		Elevation: 662,
		Latitude:  11.8830003738,
		Longitude: 3.4330000877,
		Timezone:  "Africa/Niamey",
	},
	"DRRI": {
		ICAO:      "DRRI",
		Name:      "Bilma Airport",
		City:      "Bilma",
		State:     "Agadez",
		Country:   "NE",
		Elevation: 1715,
		Latitude:  18.6867008209,
		Longitude: 12.9191999435,
		Timezone:  "Africa/Niamey",
	},
	"DRRL": {
		ICAO:      "DRRL",
		Name:      "Tillabery Airport",
		City:      "Tillabery",
		State:     "Niamey",
		Country:   "NE",
		Elevation: 692,
		Latitude:  14.1999998093,
		Longitude: 1.4670000076,
		Timezone:  "Africa/Niamey",
	},
	"DRRM": {
		ICAO:      "DRRM",
		IATA:      "MFQ",
		Name:      "Maradi Airport",
		City:      "Maradi",
		State:     "Maradi",
		Country:   "NE",
		Elevation: 1240,
		Latitude:  13.5024995804,
		Longitude: 7.1267499924,
		Timezone:  "Africa/Niamey",
	},
	"DRRN": {
		ICAO:      "DRRN",
		IATA:      "NIM",
		Name:      "Diori Hamani International Airport",
		City:      "Niamey",
		State:     "Niamey",
		Country:   "NE",
		Elevation: 732,
		Latitude:  13.4814996719,
		Longitude: 2.1836099625,
		Timezone:  "Africa/Niamey",
	},
	"DRRP": {
		ICAO:      "DRRP",
		Name:      "La Tapoa Airport",
		City:      "La Tapoa",
		State:     "Niamey",
		Country:   "NE",
		Elevation: 722,
		Latitude:  12.4829998016,
		Longitude: 2.4000000954,
		Timezone:  "Africa/Niamey",
	},
	"DRRT": {
		ICAO:      "DRRT",
		IATA:      "THZ",
		Name:      "Tahoua Airport",
		City:      "Tahoua",
		State:     "Tahoua",
		Country:   "NE",
		Elevation: 1266,
		Latitude:  14.8756999969,
		Longitude: 5.2653598785,
		Timezone:  "Africa/Niamey",
	},
	"DRRU": {
		ICAO:      "DRRU",
		Name:      "Ouallam Airport",
		City:      "Ouallam",
		State:     "Tillaberi",
		Country:   "NE",
		Elevation: 892,
		Latitude:  14.3330001831,
		Longitude: 2.0829999447,
		Timezone:  "Africa/Niamey",
	},
	"DRZA": {
		ICAO:      "DRZA",
		IATA:      "AJY",
		Name:      "Mano Dayak International Airport",
		City:      "Agadez",
		State:     "Agadez",
		Country:   "NE",
		Elevation: 1657,
		Latitude:  16.9659996033,
		Longitude: 8.0001096725,
		Timezone:  "Africa/Niamey",
	},
	"DRZD": {
		ICAO:      "DRZD",
		Name:      "Dirkou Airport",
		State:     "Agadez",
		Country:   "NE",
		Elevation: 1273,
		Latitude:  18.9687004089,
		Longitude: 12.8687000275,
		Timezone:  "Africa/Niamey",
	},
	"DRZF": {
		ICAO:      "DRZF",
		Name:      "Diffa Airport",
		City:      "Diffa",
		State:     "Diffa",
		Country:   "NE",
		Elevation: 994,
		Latitude:  13.3729000092,
		Longitude: 12.6267004013,
		Timezone:  "Africa/Niamey",
	},
	"DRZG": {
		ICAO:      "DRZG",
		Name:      "Goure Airport",
		City:      "Goure",
		State:     "Zinder",
		Country:   "NE",
		Elevation: 1499,
		Latitude:  13.9835996628,
		Longitude: 10.2749004364,
		Timezone:  "Africa/Niamey",
	},
	"DRZI": {
		ICAO:      "DRZI",
		Name:      "Iferouane Airport",
		City:      "Iferouane",
		State:     "Agadez",
		Country:   "NE",
		Elevation: 2162,
		Latitude:  19.0669994354,
		Longitude: 8.3669996262,
		Timezone:  "Africa/Niamey",
	},
	"DRZL": {
		ICAO:      "DRZL",
		IATA:      "RLT",
		Name:      "Arlit Airport",
		City:      "Arlit",
		State:     "Agadez",
		Country:   "NE",
		Elevation: 1443,
		Latitude:  18.7903995514,
		Longitude: 7.3659501076,
		Timezone:  "Africa/Niamey",
	},
	"DRZM": {
		ICAO:      "DRZM",
		Name:      "Maine-Soroa Airport",
		City:      "Maine-Soroa",
		State:     "Diffa",
		Country:   "NE",
		Elevation: 944,
		Latitude:  13.205499649,
		Longitude: 12.0322999954,
		Timezone:  "Africa/Niamey",
	},
	"DRZN": {
		ICAO:      "DRZN",
		Name:      "N'Guigmi Airport",
		City:      "N'Guigmi",
		State:     "Diffa",
		Country:   "NE",
		Elevation: 977,
		Latitude:  14.2510004044,
		Longitude: 13.1289997101,
		Timezone:  "Africa/Niamey",
	},
	"DRZR": {
		ICAO:      "DRZR",
		IATA:      "ZND",
		Name:      "Zinder Airport",
		City:      "Zinder",
		State:     "Zinder",
		Country:   "NE",
		Elevation: 1516,
		Latitude:  13.7790002823,
		Longitude: 8.9837598801,
		Timezone:  "Africa/Niamey",
	},
	"DRZT": {
		ICAO:      "DRZT",
		Name:      "Tanout Airport",
		City:      "Tanout",
		State:     "Zinder",
		Country:   "NE",
		Elevation: 1751,
		Latitude:  14.9702997208,
		Longitude: 8.8916702271,
		Timezone:  "Africa/Niamey",
	},
	"DT70": {
		ICAO:      "DT70",
		Name:      "Medenine Airport",
		City:      "Medenine",
		State:     "Madanin",
		Country:   "TN",
		Elevation: 427,
		Latitude:  33.3502006531,
		Longitude: 10.4440002441,
		Timezone:  "Africa/Tunis",
	},
	"DTKA": {
		ICAO:      "DTKA",
		IATA:      "TBJ",
		Name:      "Tabarka 7 Novembre Airport",
		City:      "Tabarka",
		State:     "Jundubah",
		Country:   "TN",
		Elevation: 230,
		Latitude:  36.9799995422,
		Longitude: 8.8769397736,
		Timezone:  "Africa/Tunis",
	},
	"DTMB": {
		ICAO:      "DTMB",
		IATA:      "MIR",
		Name:      "Monastir Habib Bourguiba International Airport",
		City:      "Monastir",
		State:     "Al-Munastir",
		Country:   "TN",
		Elevation: 9,
		Latitude:  35.7580986023,
		Longitude: 10.754699707,
		Timezone:  "Africa/Tunis",
	},
	"DTTA": {
		ICAO:      "DTTA",
		IATA:      "TUN",
		Name:      "Tunis Carthage International Airport",
		City:      "Tunis",
		State:     "Tunis",
		Country:   "TN",
		Elevation: 22,
		Latitude:  36.8510017395,
		Longitude: 10.2271995544,
		Timezone:  "Africa/Tunis",
	},
	"DTTB": {
		ICAO:      "DTTB",
		IATA:      "OIZ",
		Name:      "Sidi Ahmed Air Base",
		City:      "Sidi Ahmed",
		State:     "Banzart",
		Country:   "TN",
		Elevation: 20,
		Latitude:  37.2453994751,
		Longitude: 9.7914495468,
		Timezone:  "Africa/Tunis",
	},
	"DTTD": {
		ICAO:      "DTTD",
		Name:      "Remada Air Base",
		City:      "Remada",
		State:     "Tataouine",
		Country:   "TN",
		Elevation: 1004,
		Latitude:  32.3061981201,
		Longitude: 10.3821001053,
		Timezone:  "Africa/Tunis",
	},
	"DTTF": {
		ICAO:      "DTTF",
		IATA:      "GAF",
		Name:      "Gafsa Ksar International Airport",
		City:      "Gafsa",
		State:     "Gafsa",
		Country:   "TN",
		Elevation: 1060,
		Latitude:  34.422000885,
		Longitude: 8.8225002289,
		Timezone:  "Africa/Tunis",
	},
	"DTTG": {
		ICAO:      "DTTG",
		IATA:      "GAE",
		Name:      "Gabes Matmata International Airport",
		City:      "Gabes",
		State:     "Qabis",
		Country:   "TN",
		Elevation: 26,
		Latitude:  33.8768997192,
		Longitude: 10.1033000946,
		Timezone:  "Africa/Tunis",
	},
	"DTTI": {
		ICAO:      "DTTI",
		Name:      "Borj El Amri Airport",
		City:      "Borj El Amri",
		State:     "Ariana",
		Country:   "TN",
		Elevation: 110,
		Latitude:  36.7212982178,
		Longitude: 9.9431495667,
		Timezone:  "Africa/Tunis",
	},
	"DTTJ": {
		ICAO:      "DTTJ",
		IATA:      "DJE",
		Name:      "Djerba Zarzis International Airport",
		City:      "Djerba",
		State:     "Madanin",
		Country:   "TN",
		Elevation: 19,
		Latitude:  33.875,
		Longitude: 10.7755002975,
		Timezone:  "Africa/Tunis",
	},
	"DTTR": {
		ICAO:      "DTTR",
		IATA:      "EBM",
		Name:      "El Borma Airport",
		City:      "El Borma",
		State:     "Tataouine",
		Country:   "TN",
		Elevation: 827,
		Latitude:  31.7042999268,
		Longitude: 9.2546195984,
		Timezone:  "Africa/Tunis",
	},
	"DTTX": {
		ICAO:      "DTTX",
		IATA:      "SFA",
		Name:      "Sfax Thyna International Airport",
		City:      "Sfax",
		State:     "Safaqis",
		Country:   "TN",
		Elevation: 85,
		Latitude:  34.7179985046,
		Longitude: 10.6909999847,
		Timezone:  "Africa/Tunis",
	},
	"DTTZ": {
		ICAO:      "DTTZ",
		IATA:      "TOE",
		Name:      "Tozeur Nefta International Airport",
		City:      "Tozeur",
		State:     "Tawzar",
		Country:   "TN",
		Elevation: 287,
		Latitude:  33.9397010803,
		Longitude: 8.1105604172,
		Timezone:  "Africa/Tunis",
	},
	"DXAK": {
		ICAO:      "DXAK",
		Name:      "Akpaka Airport",
		City:      "Atakpame",
		Country:   "TG",
		Elevation: 689,
		Latitude:  7.5830001831,
		Longitude: 1.1169999838,
		Timezone:  "Africa/Lome",
	},
	"DXDP": {
		ICAO:      "DXDP",
		Name:      "Djangou Airport",
		City:      "Djangou",
		Country:   "TG",
		Elevation: 919,
		Latitude:  10.8000001907,
		Longitude: 0.25,
		Timezone:  "Africa/Lome",
	},
	"DXKP": {
		ICAO:      "DXKP",
		Name:      "Kolokope Airport",
		City:      "Anie",
		Country:   "TG",
		Elevation: 591,
		Latitude:  7.8000001907,
		Longitude: 1.2829999924,
		Timezone:  "Africa/Lome",
	},
	"DXMG": {
		ICAO:      "DXMG",
		Name:      "Sansanne-Mango Airport",
		City:      "Mango",
		Country:   "TG",
		Elevation: 460,
		Latitude:  10.3730106354,
		Longitude: 0.4713821411,
		Timezone:  "Africa/Lome",
	},
	"DXNG": {
		ICAO:      "DXNG",
		IATA:      "LRL",
		Name:      "Niamtougou International Airport",
		City:      "Niamtougou",
		State:     "Kara",
		Country:   "TG",
		Elevation: 1515,
		Latitude:  9.7673301697,
		Longitude: 1.0912499428,
		Timezone:  "Africa/Lome",
	},
	"DXSK": {
		ICAO:      "DXSK",
		Name:      "Sokode Airport",
		City:      "Sokode",
		State:     "Centrale",
		Country:   "TG",
		Elevation: 1214,
		Latitude:  8.9829998016,
		Longitude: 1.1499999762,
		Timezone:  "Africa/Lome",
	},
	"DXXX": {
		ICAO:      "DXXX",
		IATA:      "LFW",
		Name:      "Lome-Tokoin Airport",
		City:      "Lome",
		State:     "Maritime",
		Country:   "TG",
		Elevation: 72,
		Latitude:  6.1656098366,
		Longitude: 1.2545100451,
		Timezone:  "Africa/Lome",
	},
	"EBAM": {
		ICAO:      "EBAM",
		Name:      "Amougies Airport",
		City:      "Mont-de-l'Enclus",
		State:     "Wallonia",
		Country:   "BE",
		Elevation: 49,
		Latitude:  50.7401008606,
		Longitude: 3.4848999977,
		Timezone:  "Europe/Brussels",
	},
	"EBAR": {
		ICAO:      "EBAR",
		Name:      "Sterpenich Airport",
		City:      "Arlon",
		State:     "Wallonia",
		Country:   "BE",
		Elevation: 1138,
		Latitude:  49.6627998352,
		Longitude: 5.8869400024,
		Timezone:  "Europe/Brussels",
	},
	"EBAV": {
		ICAO:      "EBAV",
		Name:      "Avernas-le-Bauduin Airport",
		City:      "Hannut",
		State:     "Wallonia",
		Country:   "BE",
		Elevation: 394,
		Latitude:  50.7066993713,
		Longitude: 5.0680599213,
		Timezone:  "Europe/Brussels",
	},
	"EBAW": {
		ICAO:      "EBAW",
		IATA:      "ANR",
		Name:      "Antwerp International Airport (Deurne)",
		City:      "Antwerp",
		State:     "Flanders",
		Country:   "BE",
		Elevation: 39,
		Latitude:  51.1893997192,
		Longitude: 4.4602799416,
		Timezone:  "Europe/Brussels",
	},
	"EBBE": {
		ICAO:      "EBBE",
		Name:      "Beauvechain Air Base",
		City:      "Beauvechain",
		State:     "Wallonia",
		Country:   "BE",
		Elevation: 370,
		Latitude:  50.7585983276,
		Longitude: 4.7683300972,
		Timezone:  "Europe/Brussels",
	},
	"EBBG": {
		ICAO:      "EBBG",
		Name:      "Michamps Airport",
		City:      "Bastogne",
		State:     "Wallonia",
		Country:   "BE",
		Elevation: 1770,
		Latitude:  50.0527992249,
		Longitude: 5.7986102104,
		Timezone:  "Europe/Brussels",
	},
	"EBBL": {
		ICAO:      "EBBL",
		Name:      "Kleine Brogel Air Base",
		City:      "Kleine Brogel",
		State:     "Flanders",
		Country:   "BE",
		Elevation: 200,
		Latitude:  51.1683006287,
		Longitude: 5.4699997902,
		Timezone:  "Europe/Brussels",
	},
	"EBBN": {
		ICAO:      "EBBN",
		Name:      "Bullingen Airport",
		City:      "Bullingen",
		State:     "Wallonia",
		Country:   "BE",
		Elevation: 2067,
		Latitude:  50.4150009155,
		Longitude: 6.2763900757,
		Timezone:  "Europe/Brussels",
	},
	"EBBO": {
		ICAO:      "EBBO",
		Name:      "Mogimont Airport",
		City:      "Bouillon",
		State:     "Wallonia",
		Country:   "BE",
		Elevation: 1394,
		Latitude:  49.8664016724,
		Longitude: 5.0680599213,
		Timezone:  "Europe/Brussels",
	},
	"EBBR": {
		ICAO:      "EBBR",
		IATA:      "BRU",
		Name:      "Brussels Airport",
		City:      "Brussels",
		State:     "Flanders",
		Country:   "BE",
		Elevation: 184,
		Latitude:  50.9014015198,
		Longitude: 4.4844398499,
		Timezone:  "Europe/Brussels",
	},
	"EBBT": {
		ICAO:      "EBBT",
		Name:      "Brasschaat Air Base",
		City:      "Brasschaat",
		State:     "Flanders",
		Country:   "BE",
		Elevation: 76,
		Latitude:  51.340801239,
		Longitude: 4.5043997765,
		Timezone:  "Europe/Brussels",
	},
	"EBBW": {
		ICAO:      "EBBW",
		Name:      "Wonck Airport",
		City:      "Bassenge",
		State:     "Flanders",
		Country:   "BE",
		Elevation: 367,
		Latitude:  50.778301239,
		Longitude: 5.6166701317,
		Timezone:  "Europe/Brussels",
	},
	"EBBX": {
		ICAO:      "EBBX",
		Name:      "Jehonville Air Base",
		City:      "Bertrix",
		State:     "Wallonia",
		Country:   "BE",
		Elevation: 1514,
		Latitude:  49.8917007446,
		Longitude: 5.2238898277,
		Timezone:  "Europe/Brussels",
	},
	"EBBY": {
		ICAO:      "EBBY",
		Name:      "Baisy-Thy Airport",
		City:      "Genappe",
		State:     "Wallonia",
		Country:   "BE",
		Elevation: 525,
		Latitude:  50.568611145,
		Longitude: 4.4347219467,
		Timezone:  "Europe/Brussels",
	},
	"EBBZ": {
		ICAO:      "EBBZ",
		Name:      "Buzet Airstrip",
		City:      "Pont-A-Celles",
		State:     "Wallonia",
		Country:   "BE",
		Elevation: 522,
		Latitude:  50.5416984558,
		Longitude: 4.3811101913,
		Timezone:  "Europe/Brussels",
	},
	"EBCF": {
		ICAO:      "EBCF",
		Name:      "Cerfontaine Nouv Airport",
		City:      "Cerfontaine",
		State:     "Wallonia",
		Country:   "BE",
		Elevation: 961,
		Latitude:  50.1528015137,
		Longitude: 4.3872199059,
		Timezone:  "Europe/Brussels",
	},
	"EBCI": {
		ICAO:      "EBCI",
		IATA:      "CRL",
		Name:      "Brussels South Charleroi Airport",
		City:      "Brussels",
		State:     "Wallonia",
		Country:   "BE",
		Elevation: 614,
		Latitude:  50.4592018127,
		Longitude: 4.4538202286,
		Timezone:  "Europe/Brussels",
	},
	"EBCR": {
		ICAO:      "EBCR",
		Name:      "Ronvaux Airport",
		City:      "Ciney",
		State:     "Wallonia",
		Country:   "BE",
		Elevation: 1076,
		Latitude:  50.2472229004,
		Longitude: 5.1138892174,
		Timezone:  "Europe/Brussels",
	},
	"EBCS": {
		ICAO:      "EBCS",
		Name:      "Saint-Remy Airport",
		City:      "Chimay",
		State:     "Wallonia",
		Country:   "BE",
		Elevation: 820,
		Latitude:  50.0583000183,
		Longitude: 4.2819399834,
		Timezone:  "Europe/Brussels",
	},
	"EBCV": {
		ICAO:      "EBCV",
		Name:      "Chievres Air Base",
		City:      "Chievres",
		State:     "Wallonia",
		Country:   "BE",
		Elevation: 194,
		Latitude:  50.5758018494,
		Longitude: 3.8310000896,
		Timezone:  "Europe/Brussels",
	},
	"EBDT": {
		ICAO:      "EBDT",
		Name:      "Schaffen Airport",
		City:      "Diest",
		State:     "Flanders",
		Country:   "BE",
		Elevation: 100,
		Latitude:  50.9991989136,
		Longitude: 5.065559864,
		Timezone:  "Europe/Brussels",
	},
	"EBFN": {
		ICAO:      "EBFN",
		Name:      "Koksijde Air Base",
		City:      "Koksijde",
		State:     "Flanders",
		Country:   "BE",
		Elevation: 20,
		Latitude:  51.0903015137,
		Longitude: 2.652780056,
		Timezone:  "Europe/Brussels",
	},
	"EBFS": {
		ICAO:      "EBFS",
		Name:      "Florennes Air Base",
		City:      "Florennes",
		State:     "Wallonia",
		Country:   "BE",
		Elevation: 935,
		Latitude:  50.2433013916,
		Longitude: 4.6458301544,
		Timezone:  "Europe/Brussels",
	},
	"EBGB": {
		ICAO:      "EBGB",
		Name:      "Grimbergen Airport",
		City:      "Grimbergen",
		State:     "Flanders",
		Country:   "BE",
		Elevation: 70,
		Latitude:  50.9494018555,
		Longitude: 4.3933300972,
		Timezone:  "Europe/Brussels",
	},
	"EBGG": {
		ICAO:      "EBGG",
		Name:      "Overboelare Airport",
		City:      "Geraardsbergen",
		State:     "Flanders",
		Country:   "BE",
		Elevation: 56,
		Latitude:  50.7555999756,
		Longitude: 3.8638899326,
		Timezone:  "Europe/Brussels",
	},
	"EBHE": {
		ICAO:      "EBHE",
		Name:      "Couthuin Airport",
		City:      "Heron",
		State:     "Wallonia",
		Country:   "BE",
		Elevation: 607,
		Latitude:  50.538898468,
		Longitude: 5.10916996,
		Timezone:  "Europe/Brussels",
	},
	"EBHN": {
		ICAO:      "EBHN",
		Name:      "Hoevenen Airport",
		City:      "Hoevenen",
		State:     "Flanders",
		Country:   "BE",
		Elevation: 10,
		Latitude:  51.306098938,
		Longitude: 4.3872199059,
		Timezone:  "Europe/Brussels",
	},
	"EBIS": {
		ICAO:      "EBIS",
		Name:      "Ath/Isieres Airport",
		City:      "Isieres",
		State:     "Wallonia",
		Country:   "BE",
		Elevation: 98,
		Latitude:  50.6641998291,
		Longitude: 3.8044400215,
		Timezone:  "Europe/Brussels",
	},
	"EBKH": {
		ICAO:      "EBKH",
		Name:      "Balekeiheuvel Airport",
		City:      "Balen",
		State:     "Flanders",
		Country:   "BE",
		Elevation: 131,
		Latitude:  51.1782989502,
		Longitude: 5.2209200859,
		Timezone:  "Europe/Brussels",
	},
	"EBKT": {
		ICAO:      "EBKT",
		IATA:      "KJK",
		Name:      "Wevelgem Airport",
		City:      "Wevelgem",
		State:     "Flanders",
		Country:   "BE",
		Elevation: 64,
		Latitude:  50.817199707,
		Longitude: 3.2047200203,
		Timezone:  "Europe/Brussels",
	},
	"EBLB": {
		ICAO:      "EBLB",
		Name:      "Elsenborn Airport",
		City:      "Butgenbach",
		State:     "Wallonia",
		Country:   "BE",
		Elevation: 1830,
		Latitude:  50.4817008972,
		Longitude: 6.1819400787,
		Timezone:  "Europe/Brussels",
	},
	"EBLE": {
		ICAO:      "EBLE",
		Name:      "Beverlo Air Base",
		City:      "Leopoldsburg",
		State:     "Flanders",
		Country:   "BE",
		Elevation: 207,
		Latitude:  51.1194000244,
		Longitude: 5.3008298874,
		Timezone:  "Europe/Brussels",
	},
	"EBLG": {
		ICAO:      "EBLG",
		IATA:      "LGG",
		Name:      "Liege Airport",
		City:      "Liege",
		State:     "Wallonia",
		Country:   "BE",
		Elevation: 659,
		Latitude:  50.6374015808,
		Longitude: 5.4432201385,
		Timezone:  "Europe/Brussels",
	},
	"EBLI": {
		ICAO:      "EBLI",
		Name:      "Lierneux Airport",
		City:      "Lierneux",
		State:     "Wallonia",
		Country:   "BE",
		Elevation: 1411,
		Latitude:  50.2867012024,
		Longitude: 5.8363900185,
		Timezone:  "Europe/Brussels",
	},
	"EBLN": {
		ICAO:      "EBLN",
		Name:      "Liernu Airport",
		City:      "Eghezee",
		State:     "Wallonia",
		Country:   "BE",
		Elevation: 564,
		Latitude:  50.5806007385,
		Longitude: 4.7925000191,
		Timezone:  "Europe/Brussels",
	},
	"EBMB": {
		ICAO:      "EBMB",
		Name:      "Melsbroek Air Base",
		City:      "Brussels",
		State:     "Flanders",
		Country:   "BE",
		Elevation: 184,
		Latitude:  50.901389,
		Longitude: 4.484444,
		Timezone:  "Europe/Brussels",
	},
	"EBMG": {
		ICAO:      "EBMG",
		Name:      "Matagne-la-Petite Airport",
		City:      "Doische",
		State:     "Wallonia",
		Country:   "BE",
		Elevation: 787,
		Latitude:  50.1046981812,
		Longitude: 4.6380600929,
		Timezone:  "Europe/Brussels",
	},
	"EBML": {
		ICAO:      "EBML",
		Name:      "Maillen Airport",
		City:      "Assesse",
		State:     "Wallonia",
		Country:   "BE",
		Elevation: 886,
		Latitude:  50.3741989136,
		Longitude: 4.9277801514,
		Timezone:  "Europe/Brussels",
	},
	"EBMO": {
		ICAO:      "EBMO",
		Name:      "Moorsele Airport",
		City:      "Moorsele",
		State:     "Flanders",
		Country:   "BE",
		Elevation: 66,
		Latitude:  50.8527984619,
		Longitude: 3.1473100185,
		Timezone:  "Europe/Brussels",
	},
	"EBNE": {
		ICAO:      "EBNE",
		Name:      "Neerpelt Airport",
		City:      "Neerpelt",
		State:     "Flanders",
		Country:   "BE",
		Elevation: 148,
		Latitude:  51.2119445801,
		Longitude: 5.4786109924,
		Timezone:  "Europe/Brussels",
	},
	"EBNM": {
		ICAO:      "EBNM",
		IATA:      "QNM",
		Name:      "Suarlee Airport",
		City:      "Namur",
		State:     "Wallonia",
		Country:   "BE",
		Elevation: 594,
		Latitude:  50.4879989624,
		Longitude: 4.7689199448,
		Timezone:  "Europe/Brussels",
	},
	"EBNO": {
		ICAO:      "EBNO",
		Name:      "Outer Airport",
		City:      "Ninove",
		State:     "Flanders",
		Country:   "BE",
		Elevation: 151,
		Latitude:  50.8544006348,
		Longitude: 3.9852800369,
		Timezone:  "Europe/Brussels",
	},
	"EBOR": {
		ICAO:      "EBOR",
		Name:      "Orchimont Airport",
		City:      "Vresse-Sur-Semois",
		State:     "Wallonia",
		Country:   "BE",
		Elevation: 1312,
		Latitude:  49.9071998596,
		Longitude: 4.9363899231,
		Timezone:  "Europe/Brussels",
	},
	"EBOS": {
		ICAO:      "EBOS",
		IATA:      "OST",
		Name:      "Ostend-Bruges International Airport",
		City:      "Ostend",
		State:     "Flanders",
		Country:   "BE",
		Elevation: 13,
		Latitude:  51.1988983154,
		Longitude: 2.8622200489,
		Timezone:  "Europe/Brussels",
	},
	"EBSG": {
		ICAO:      "EBSG",
		Name:      "Saint-Ghislain Airport",
		City:      "Saint Ghislain",
		State:     "Wallonia",
		Country:   "BE",
		Elevation: 75,
		Latitude:  50.4575004578,
		Longitude: 3.8202800751,
		Timezone:  "Europe/Brussels",
	},
	"EBSH": {
		ICAO:      "EBSH",
		Name:      "St Hubert Airport",
		City:      "Saint Hubert",
		State:     "Wallonia",
		Country:   "BE",
		Elevation: 1930,
		Latitude:  50.035900116,
		Longitude: 5.404250145,
		Timezone:  "Europe/Brussels",
	},
	"EBSL": {
		ICAO:      "EBSL",
		Name:      "Zutendaal Air Base",
		City:      "Zutendaal",
		State:     "Flanders",
		Country:   "BE",
		Elevation: 312,
		Latitude:  50.9474983215,
		Longitude: 5.5905599594,
		Timezone:  "Europe/Brussels",
	},
	"EBSP": {
		ICAO:      "EBSP",
		Name:      "Spa (la Sauveniere) Airport",
		City:      "Spa",
		State:     "Wallonia",
		Country:   "BE",
		Elevation: 1581,
		Latitude:  50.4824981689,
		Longitude: 5.910299778,
		Timezone:  "Europe/Brussels",
	},
	"EBST": {
		ICAO:      "EBST",
		Name:      "Sint Truiden Airport",
		City:      "Sint Truiden",
		State:     "Flanders",
		Country:   "BE",
		Elevation: 246,
		Latitude:  50.7919006348,
		Longitude: 5.2016701698,
		Timezone:  "Europe/Brussels",
	},
	"EBSU": {
		ICAO:      "EBSU",
		Name:      "Saint Hubert Air Base",
		City:      "Saint Hubert",
		State:     "Wallonia",
		Country:   "BE",
		Elevation: 1930,
		Latitude:  50.0344009399,
		Longitude: 5.4408102036,
		Timezone:  "Europe/Brussels",
	},
	"EBTN": {
		ICAO:      "EBTN",
		Name:      "Goetsenhoven Air Base",
		City:      "Tienen",
		State:     "Flanders",
		Country:   "BE",
		Elevation: 246,
		Latitude:  50.7817001343,
		Longitude: 4.9577798843,
		Timezone:  "Europe/Brussels",
	},
	"EBTX": {
		ICAO:      "EBTX",
		Name:      "Theux (Verviers) Airport",
		City:      "Verviers",
		State:     "Wallonia",
		Country:   "BE",
		Elevation: 1099,
		Latitude:  50.5526008606,
		Longitude: 5.8550300598,
		Timezone:  "Europe/Brussels",
	},
	"EBTY": {
		ICAO:      "EBTY",
		Name:      "Tournai/Maubray Airport",
		City:      "Tournai",
		State:     "Wallonia",
		Country:   "BE",
		Elevation: 161,
		Latitude:  50.5297012329,
		Longitude: 3.4945499897,
		Timezone:  "Europe/Brussels",
	},
	"EBUL": {
		ICAO:      "EBUL",
		Name:      "Ursel Air Base",
		City:      "Ursel",
		State:     "Flanders",
		Country:   "BE",
		Elevation: 95,
		Latitude:  51.1441993713,
		Longitude: 3.4755599499,
		Timezone:  "Europe/Brussels",
	},
	"EBWE": {
		ICAO:      "EBWE",
		Name:      "Weelde Air Base",
		City:      "Weelde",
		State:     "Flanders",
		Country:   "BE",
		Elevation: 105,
		Latitude:  51.3947982788,
		Longitude: 4.9601898193,
		Timezone:  "Europe/Amsterdam",
	},
	"EBZH": {
		ICAO:      "EBZH",
		IATA:      "QHA",
		Name:      "Hasselt Airport",
		City:      "Hasselt",
		State:     "Flanders",
		Country:   "BE",
		Elevation: 141,
		Latitude:  50.9700012207,
		Longitude: 5.3750700951,
		Timezone:  "Europe/Brussels",
	},
	"EBZR": {
		ICAO:      "EBZR",
		IATA:      "OBL",
		Name:      "Oostmalle Air Base",
		City:      "Zoersel",
		State:     "Flanders",
		Country:   "BE",
		Elevation: 53,
		Latitude:  51.2647018433,
		Longitude: 4.7533302307,
		Timezone:  "Europe/Brussels",
	},
	"EBZU": {
		ICAO:      "EBZU",
		Name:      "Zuienkerke Airport",
		City:      "Zuinkerke",
		State:     "Flanders",
		Country:   "BE",
		Elevation: 16,
		Latitude:  51.2566986084,
		Longitude: 3.1405599117,
		Timezone:  "Europe/Brussels",
	},
	"EBZW": {
		ICAO:      "EBZW",
		Name:      "Genk Zwartberg Airport",
		City:      "Genk",
		State:     "Flanders",
		Country:   "BE",
		Elevation: 278,
		Latitude:  51.0153999329,
		Longitude: 5.5264701843,
		Timezone:  "Europe/Brussels",
	},
	"EDAB": {
		ICAO:      "EDAB",
		Name:      "Bautzen Airport",
		City:      "Bautzen",
		State:     "Saxony",
		Country:   "DE",
		Elevation: 568,
		Latitude:  51.193611145,
		Longitude: 14.5197219849,
		Timezone:  "Europe/Berlin",
	},
	"EDAC": {
		ICAO:      "EDAC",
		IATA:      "AOC",
		Name:      "Altenburg-Nobitz Airport",
		City:      "Altenburg",
		State:     "Thuringia",
		Country:   "DE",
		Elevation: 640,
		Latitude:  50.9819450378,
		Longitude: 12.5063886642,
		Timezone:  "Europe/Berlin",
	},
	"EDAD": {
		ICAO:      "EDAD",
		Name:      "Dessau Airport",
		City:      "Dessau",
		State:     "Saxony-Anhalt",
		Country:   "DE",
		Elevation: 187,
		Latitude:  51.831693541,
		Longitude: 12.1909618378,
		Timezone:  "Europe/Berlin",
	},
	"EDAE": {
		ICAO:      "EDAE",
		Name:      "Eisenhuttenstadt Airport",
		City:      "Eisenhuttenstadt",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 144,
		Latitude:  52.1972236633,
		Longitude: 14.5855560303,
		Timezone:  "Europe/Berlin",
	},
	"EDAG": {
		ICAO:      "EDAG",
		Name:      "Grosruckerswalde Airport",
		City:      "Marienberg",
		State:     "Saxony",
		Country:   "DE",
		Elevation: 2198,
		Latitude:  50.6441688538,
		Longitude: 13.1263885498,
		Timezone:  "Europe/Berlin",
	},
	"EDAH": {
		ICAO:      "EDAH",
		IATA:      "HDF",
		Name:      "Heringsdorf Airport",
		City:      "Heringsdorf",
		State:     "Mecklenburg-Vorpommern",
		Country:   "DE",
		Elevation: 93,
		Latitude:  53.8787002563,
		Longitude: 14.152299881,
		Timezone:  "Europe/Berlin",
	},
	"EDAI": {
		ICAO:      "EDAI",
		Name:      "Segeletz Airport",
		City:      "Neustadt",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 141,
		Latitude:  52.8266677856,
		Longitude: 12.5419435501,
		Timezone:  "Europe/Berlin",
	},
	"EDAJ": {
		ICAO:      "EDAJ",
		Name:      "Gera-Leumnitz Airport",
		City:      "Gera",
		State:     "Thuringia",
		Country:   "DE",
		Elevation: 1014,
		Latitude:  50.8816680908,
		Longitude: 12.1358327866,
		Timezone:  "Europe/Berlin",
	},
	"EDAK": {
		ICAO:      "EDAK",
		Name:      "Grosenhain Airport",
		City:      "Grosenhain",
		State:     "Saxony",
		Country:   "DE",
		Elevation: 417,
		Latitude:  51.3080558777,
		Longitude: 13.5555562973,
		Timezone:  "Europe/Berlin",
	},
	"EDAM": {
		ICAO:      "EDAM",
		Name:      "Merseburg Airport",
		City:      "Merseburg",
		State:     "Saxony-Anhalt",
		Country:   "DE",
		Elevation: 341,
		Latitude:  51.363609314,
		Longitude: 11.9483327866,
		Timezone:  "Europe/Berlin",
	},
	"EDAN": {
		ICAO:      "EDAN",
		Name:      "Neustadt-Glewe Airport",
		City:      "Neustadt-Glewe",
		State:     "Mecklenburg-Vorpommern",
		Country:   "DE",
		Elevation: 115,
		Latitude:  53.3597221375,
		Longitude: 11.615278244,
		Timezone:  "Europe/Berlin",
	},
	"EDAO": {
		ICAO:      "EDAO",
		Name:      "Nordhausen Airport",
		City:      "Nordhausen",
		State:     "Thuringia",
		Country:   "DE",
		Elevation: 689,
		Latitude:  51.493057251,
		Longitude: 10.8333330154,
		Timezone:  "Europe/Berlin",
	},
	"EDAP": {
		ICAO:      "EDAP",
		Name:      "Neuhausen Airport",
		City:      "Cottbus",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 279,
		Latitude:  51.6847229004,
		Longitude: 14.4230556488,
		Timezone:  "Europe/Berlin",
	},
	"EDAQ": {
		ICAO:      "EDAQ",
		IATA:      "ZHZ",
		Name:      "Halle-Oppin Airport",
		City:      "Oppin",
		State:     "Saxony-Anhalt",
		Country:   "DE",
		Elevation: 348,
		Latitude:  51.5522232056,
		Longitude: 12.0538892746,
		Timezone:  "Europe/Berlin",
	},
	"EDAR": {
		ICAO:      "EDAR",
		Name:      "Pirna-Pratzschwitz Airport",
		City:      "Pirna",
		State:     "Saxony",
		Country:   "DE",
		Elevation: 400,
		Latitude:  50.9791679382,
		Longitude: 13.9097223282,
		Timezone:  "Europe/Berlin",
	},
	"EDAS": {
		ICAO:      "EDAS",
		Name:      "Finsterwalde/Heinrichsruh Airport",
		City:      "Finsterwalde",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 384,
		Latitude:  51.6344451904,
		Longitude: 13.6755561829,
		Timezone:  "Europe/Berlin",
	},
	"EDAT": {
		ICAO:      "EDAT",
		Name:      "Nardt Airport",
		City:      "Hoyerswerda",
		State:     "Saxony",
		Country:   "DE",
		Elevation: 384,
		Latitude:  51.4511108398,
		Longitude: 14.1994438171,
		Timezone:  "Europe/Berlin",
	},
	"EDAU": {
		ICAO:      "EDAU",
		IATA:      "IES",
		Name:      "Riesa-Gohlis Airport",
		City:      "Riesa",
		State:     "Saxony",
		Country:   "DE",
		Elevation: 322,
		Latitude:  51.2936096191,
		Longitude: 13.3561105728,
		Timezone:  "Europe/Berlin",
	},
	"EDAV": {
		ICAO:      "EDAV",
		Name:      "Eberswalde-Finow Airport",
		City:      "Eberswalde",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 121,
		Latitude:  52.8272209167,
		Longitude: 13.693611145,
		Timezone:  "Europe/Berlin",
	},
	"EDAW": {
		ICAO:      "EDAW",
		Name:      "Roitzschjora Airport",
		City:      "Roitzschjora",
		State:     "Saxony",
		Country:   "DE",
		Elevation: 289,
		Latitude:  51.5777778625,
		Longitude: 12.4944438934,
		Timezone:  "Europe/Berlin",
	},
	"EDAX": {
		ICAO:      "EDAX",
		IATA:      "REB",
		Name:      "Rechlin-Larz Airport",
		City:      "Larz",
		State:     "Mecklenburg-Vorpommern",
		Country:   "DE",
		Elevation: 220,
		Latitude:  53.306388855,
		Longitude: 12.7522220612,
		Timezone:  "Europe/Berlin",
	},
	"EDAY": {
		ICAO:      "EDAY",
		Name:      "Strausberg Airport",
		City:      "Strausberg",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 262,
		Latitude:  52.5805549622,
		Longitude: 13.9166669846,
		Timezone:  "Europe/Berlin",
	},
	"EDAZ": {
		ICAO:      "EDAZ",
		Name:      "Schonhagen Airport",
		City:      "Trebbin",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 131,
		Latitude:  52.2036094666,
		Longitude: 13.1563892365,
		Timezone:  "Europe/Berlin",
	},
	"EDBA": {
		ICAO:      "EDBA",
		Name:      "Arnstadt-Alkersleben Airport",
		City:      "Wulfershausen",
		State:     "Thuringia",
		Country:   "DE",
		Elevation: 1145,
		Latitude:  50.8416671753,
		Longitude: 11.0694437027,
		Timezone:  "Europe/Berlin",
	},
	"EDBC": {
		ICAO:      "EDBC",
		IATA:      "CSO",
		Name:      "Cochstedt Airport",
		City:      "Magdeburg",
		State:     "Saxony-Anhalt",
		Country:   "DE",
		Elevation: 594,
		Latitude:  51.8563995361,
		Longitude: 11.42029953,
		Timezone:  "Europe/Berlin",
	},
	"EDBD": {
		ICAO:      "EDBD",
		Name:      "Emmel Airfield Dedelow Airport",
		City:      "Prenzlau",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 213,
		Latitude:  53.356388092,
		Longitude: 13.7836112976,
		Timezone:  "Europe/Berlin",
	},
	"EDBE": {
		ICAO:      "EDBE",
		Name:      "Brandenburg-Muhlenfeld Airport",
		City:      "Brandenburg an der Havel",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 98,
		Latitude:  52.4375,
		Longitude: 12.5900001526,
		Timezone:  "Europe/Berlin",
	},
	"EDBF": {
		ICAO:      "EDBF",
		Name:      "Ruppiner Land Airport",
		City:      "Fehrbellin",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 138,
		Latitude:  52.7933311462,
		Longitude: 12.7602777481,
		Timezone:  "Europe/Berlin",
	},
	"EDBG": {
		ICAO:      "EDBG",
		Name:      "Burg Airport",
		City:      "Burg",
		State:     "Saxony-Anhalt",
		Country:   "DE",
		Elevation: 174,
		Latitude:  52.2416687012,
		Longitude: 11.8561105728,
		Timezone:  "Europe/Berlin",
	},
	"EDBH": {
		ICAO:      "EDBH",
		IATA:      "BBH",
		Name:      "Barth Airport",
		State:     "Mecklenburg-Vorpommern",
		Country:   "DE",
		Elevation: 23,
		Latitude:  54.3375015259,
		Longitude: 12.6997003555,
		Timezone:  "Europe/Berlin",
	},
	"EDBI": {
		ICAO:      "EDBI",
		Name:      "Zwickau Airport",
		City:      "Zwickau",
		State:     "Saxony",
		Country:   "DE",
		Elevation: 1050,
		Latitude:  50.7016677856,
		Longitude: 12.4538888931,
		Timezone:  "Europe/Berlin",
	},
	"EDBJ": {
		ICAO:      "EDBJ",
		Name:      "Jena-Schongleina Airport",
		City:      "Jena",
		State:     "Thuringia",
		Country:   "DE",
		Elevation: 1247,
		Latitude:  50.9152793884,
		Longitude: 11.7144441605,
		Timezone:  "Europe/Berlin",
	},
	"EDBK": {
		ICAO:      "EDBK",
		Name:      "Kyritz Airport",
		City:      "Kyritz",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 131,
		Latitude:  52.918888092,
		Longitude: 12.42527771,
		Timezone:  "Europe/Berlin",
	},
	"EDBL": {
		ICAO:      "EDBL",
		Name:      "Laucha Airport",
		City:      "Laucha an der Unstrut",
		State:     "Saxony-Anhalt",
		Country:   "DE",
		Elevation: 738,
		Latitude:  51.2458343506,
		Longitude: 11.6933326721,
		Timezone:  "Europe/Berlin",
	},
	"EDBM": {
		ICAO:      "EDBM",
		IATA:      "ZMG",
		Name:      "Magdeburg Airport",
		City:      "Magdeburg",
		State:     "Saxony-Anhalt",
		Country:   "DE",
		Elevation: 259,
		Latitude:  52.0736122131,
		Longitude: 11.6263885498,
		Timezone:  "Europe/Berlin",
	},
	"EDBO": {
		ICAO:      "EDBO",
		Name:      "Oehna Airport",
		City:      "Zellendorf",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 289,
		Latitude:  51.899723053,
		Longitude: 13.0522222519,
		Timezone:  "Europe/Berlin",
	},
	"EDBP": {
		ICAO:      "EDBP",
		Name:      "Pinnow Airport",
		City:      "Pinnow",
		State:     "Mecklenburg-Vorpommern",
		Country:   "DE",
		Elevation: 144,
		Latitude:  53.6152763367,
		Longitude: 11.5611114502,
		Timezone:  "Europe/Berlin",
	},
	"EDBQ": {
		ICAO:      "EDBQ",
		Name:      "Bronkow Airport",
		City:      "Bronkow",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 423,
		Latitude:  51.6705551147,
		Longitude: 13.9602775574,
		Timezone:  "Europe/Berlin",
	},
	"EDBR": {
		ICAO:      "EDBR",
		Name:      "Rothenburg/Gorlitz Airport",
		City:      "Rothenburg/Oberlausitz",
		State:     "Saxony",
		Country:   "DE",
		Elevation: 518,
		Latitude:  51.3633346558,
		Longitude: 14.9499998093,
		Timezone:  "Europe/Berlin",
	},
	"EDBS": {
		ICAO:      "EDBS",
		Name:      "Sommerda-Dermsdorf Airport",
		City:      "Kolleda",
		State:     "Thuringia",
		Country:   "DE",
		Elevation: 453,
		Latitude:  51.1991653442,
		Longitude: 11.1925001144,
		Timezone:  "Europe/Berlin",
	},
	"EDBT": {
		ICAO:      "EDBT",
		Name:      "Allstedt Airport",
		City:      "Allstedt",
		State:     "Thuringia",
		Country:   "DE",
		Elevation: 932,
		Latitude:  51.3805541992,
		Longitude: 11.4466667175,
		Timezone:  "Europe/Berlin",
	},
	"EDBU": {
		ICAO:      "EDBU",
		Name:      "Pritzwalk-Sommersberg Airport",
		City:      "Pritzwalk",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 289,
		Latitude:  53.1802787781,
		Longitude: 12.1855564117,
		Timezone:  "Europe/Berlin",
	},
	"EDBV": {
		ICAO:      "EDBV",
		Name:      "Stralsund Airport",
		City:      "Stralsund",
		State:     "Mecklenburg-Vorpommern",
		Country:   "DE",
		Elevation: 49,
		Latitude:  54.3358345032,
		Longitude: 13.0433330536,
		Timezone:  "Europe/Berlin",
	},
	"EDBW": {
		ICAO:      "EDBW",
		Name:      "Werneuchen Airport",
		City:      "Werneuchen",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 262,
		Latitude:  52.6316680908,
		Longitude: 13.7669439316,
		Timezone:  "Europe/Berlin",
	},
	"EDBX": {
		ICAO:      "EDBX",
		Name:      "Gorlitz Airport",
		City:      "Gorlitz",
		State:     "Saxony",
		Country:   "DE",
		Elevation: 778,
		Latitude:  51.1588897705,
		Longitude: 14.9502782822,
		Timezone:  "Europe/Berlin",
	},
	"EDBY": {
		ICAO:      "EDBY",
		Name:      "Schmoldow Airport",
		City:      "Schmoldow",
		State:     "Mecklenburg-Vorpommern",
		Country:   "DE",
		Elevation: 105,
		Latitude:  53.9722213745,
		Longitude: 13.3436107635,
		Timezone:  "Europe/Berlin",
	},
	"EDBZ": {
		ICAO:      "EDBZ",
		Name:      "Schwarzheide/Schipkau Airport",
		City:      "Schwarzheide",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 331,
		Latitude:  51.4897232056,
		Longitude: 13.8794441223,
		Timezone:  "Europe/Berlin",
	},
	"EDCA": {
		ICAO:      "EDCA",
		Name:      "Anklam Airport",
		City:      "Anklam",
		State:     "Mecklenburg-Vorpommern",
		Country:   "DE",
		Elevation: 16,
		Latitude:  53.8327789307,
		Longitude: 13.6686105728,
		Timezone:  "Europe/Berlin",
	},
	"EDCB": {
		ICAO:      "EDCB",
		Name:      "Ballenstedt Airport",
		City:      "Ballenstedt",
		State:     "Saxony-Anhalt",
		Country:   "DE",
		Elevation: 535,
		Latitude:  51.7458343506,
		Longitude: 11.229722023,
		Timezone:  "Europe/Berlin",
	},
	"EDCD": {
		ICAO:      "EDCD",
		IATA:      "CBU",
		Name:      "Cottbus-Drewitz Airport",
		City:      "Cottbus",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 272,
		Latitude:  51.8894424438,
		Longitude: 14.5319442749,
		Timezone:  "Europe/Berlin",
	},
	"EDCE": {
		ICAO:      "EDCE",
		Name:      "Eggersdorf Airport",
		City:      "Eggersdorf",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 223,
		Latitude:  52.4816665649,
		Longitude: 14.0874996185,
		Timezone:  "Europe/Berlin",
	},
	"EDCF": {
		ICAO:      "EDCF",
		Name:      "Friedersdorf Airport",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 115,
		Latitude:  52.2832984924,
		Longitude: 13.8069000244,
		Timezone:  "Europe/Berlin",
	},
	"EDCG": {
		ICAO:      "EDCG",
		IATA:      "GTI",
		Name:      "Rugen Airport",
		City:      "Rugen",
		State:     "Mecklenburg-Vorpommern",
		Country:   "DE",
		Elevation: 69,
		Latitude:  54.3833312988,
		Longitude: 13.3255558014,
		Timezone:  "Europe/Berlin",
	},
	"EDCH": {
		ICAO:      "EDCH",
		Name:      "Sprossen Airport",
		City:      "Rehmsdorf",
		State:     "Saxony-Anhalt",
		Country:   "DE",
		Elevation: 673,
		Latitude:  51.043056488,
		Longitude: 12.233332634,
		Timezone:  "Europe/Berlin",
	},
	"EDCI": {
		ICAO:      "EDCI",
		Name:      "Klix Airport",
		City:      "Bautzen",
		State:     "Saxony",
		Country:   "DE",
		Elevation: 486,
		Latitude:  51.2738876343,
		Longitude: 14.5063886642,
		Timezone:  "Europe/Berlin",
	},
	"EDCJ": {
		ICAO:      "EDCJ",
		Name:      "Chemnitz/Jahnsdorf Airport",
		City:      "Chemnitz",
		State:     "Saxony",
		Country:   "DE",
		Elevation: 1198,
		Latitude:  50.7475013733,
		Longitude: 12.8374996185,
		Timezone:  "Europe/Berlin",
	},
	"EDCK": {
		ICAO:      "EDCK",
		IATA:      "KOQ",
		Name:      "Kothen Airport",
		City:      "Kothen",
		State:     "Saxony-Anhalt",
		Country:   "DE",
		Elevation: 305,
		Latitude:  51.7211112976,
		Longitude: 11.9527778625,
		Timezone:  "Europe/Berlin",
	},
	"EDCL": {
		ICAO:      "EDCL",
		Name:      "Klietz/Scharlibbe Airport",
		City:      "Scharlibbe",
		State:     "Saxony-Anhalt",
		Country:   "DE",
		Elevation: 95,
		Latitude:  52.7094421387,
		Longitude: 12.0733327866,
		Timezone:  "Europe/Berlin",
	},
	"EDCM": {
		ICAO:      "EDCM",
		Name:      "Kamenz Airport",
		City:      "Kamenz",
		State:     "Saxony",
		Country:   "DE",
		Elevation: 495,
		Latitude:  51.2969436646,
		Longitude: 14.1274995804,
		Timezone:  "Europe/Berlin",
	},
	"EDCN": {
		ICAO:      "EDCN",
		Name:      "Nauen Airport",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 112,
		Latitude:  52.6277999878,
		Longitude: 12.9139003754,
		Timezone:  "Europe/Berlin",
	},
	"EDCO": {
		ICAO:      "EDCO",
		Name:      "Obermehler-Schlotheim Airport",
		City:      "Obermehler",
		State:     "Thuringia",
		Country:   "DE",
		Elevation: 909,
		Latitude:  51.2677764893,
		Longitude: 10.634721756,
		Timezone:  "Europe/Berlin",
	},
	"EDCP": {
		ICAO:      "EDCP",
		IATA:      "PEF",
		Name:      "Peenemunde Airport",
		City:      "Peenemunde",
		State:     "Mecklenburg-Vorpommern",
		Country:   "DE",
		Elevation: 7,
		Latitude:  54.1577796936,
		Longitude: 13.7744436264,
		Timezone:  "Europe/Berlin",
	},
	"EDCQ": {
		ICAO:      "EDCQ",
		Name:      "Aschersleben Airport",
		City:      "Aschersleben",
		State:     "Saxony-Anhalt",
		Country:   "DE",
		Elevation: 525,
		Latitude:  51.7669448853,
		Longitude: 11.4983329773,
		Timezone:  "Europe/Berlin",
	},
	"EDCR": {
		ICAO:      "EDCR",
		Name:      "Rerik-Zweedorf Airport",
		City:      "Rerik",
		State:     "Mecklenburg-Vorpommern",
		Country:   "DE",
		Elevation: 30,
		Latitude:  54.081943512,
		Longitude: 11.6491670609,
		Timezone:  "Europe/Berlin",
	},
	"EDCS": {
		ICAO:      "EDCS",
		Name:      "Saarmund Airport",
		City:      "Saarmund",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 174,
		Latitude:  52.3083343506,
		Longitude: 13.1005563736,
		Timezone:  "Europe/Berlin",
	},
	"EDCT": {
		ICAO:      "EDCT",
		Name:      "Taucha Airport",
		City:      "Taucha",
		State:     "Saxony",
		Country:   "DE",
		Elevation: 492,
		Latitude:  51.3950004578,
		Longitude: 12.5369443893,
		Timezone:  "Europe/Berlin",
	},
	"EDCU": {
		ICAO:      "EDCU",
		Name:      "Gustrow Airport",
		City:      "Gustrow",
		State:     "Mecklenburg-Vorpommern",
		Country:   "DE",
		Elevation: 46,
		Latitude:  53.8052787781,
		Longitude: 12.2305555344,
		Timezone:  "Europe/Berlin",
	},
	"EDCV": {
		ICAO:      "EDCV",
		Name:      "Pasewalk Airport",
		City:      "Pasewalk",
		State:     "Mecklenburg-Vorpommern",
		Country:   "DE",
		Elevation: 72,
		Latitude:  53.505279541,
		Longitude: 13.9483327866,
		Timezone:  "Europe/Berlin",
	},
	"EDCW": {
		ICAO:      "EDCW",
		Name:      "Wismar Airport",
		City:      "Wismar",
		State:     "Mecklenburg-Vorpommern",
		Country:   "DE",
		Elevation: 43,
		Latitude:  53.9144439697,
		Longitude: 11.4994440079,
		Timezone:  "Europe/Berlin",
	},
	"EDCX": {
		ICAO:      "EDCX",
		Name:      "Purkshof Airport",
		City:      "Rostock",
		State:     "Mecklenburg-Vorpommern",
		Country:   "DE",
		Elevation: 66,
		Latitude:  54.1616668701,
		Longitude: 12.2486114502,
		Timezone:  "Europe/Berlin",
	},
	"EDCY": {
		ICAO:      "EDCY",
		Name:      "Spremberg-Welzow Airport",
		City:      "Welzow",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 374,
		Latitude:  51.5755577087,
		Longitude: 14.1369438171,
		Timezone:  "Europe/Berlin",
	},
	"EDDB": {
		ICAO:      "EDDB",
		IATA:      "BER",
		Name:      "Berlin Brandenburg Airport",
		City:      "Berlin",
		Country:   "DE",
		Elevation: 156,
		Latitude:  52.366667,
		Longitude: 13.503333,
		Timezone:  "Europe/Berlin",
	},
	"EDDC": {
		ICAO:      "EDDC",
		IATA:      "DRS",
		Name:      "Dresden Airport",
		City:      "Dresden",
		State:     "Saxony",
		Country:   "DE",
		Elevation: 755,
		Latitude:  51.1328010559,
		Longitude: 13.76720047,
		Timezone:  "Europe/Berlin",
	},
	"EDDE": {
		ICAO:      "EDDE",
		IATA:      "ERF",
		Name:      "Erfurt Airport",
		City:      "Erfurt",
		State:     "Thuringia",
		Country:   "DE",
		Elevation: 1036,
		Latitude:  50.979801178,
		Longitude: 10.9581003189,
		Timezone:  "Europe/Berlin",
	},
	"EDDF": {
		ICAO:      "EDDF",
		IATA:      "FRA",
		Name:      "Frankfurt am Main International Airport",
		City:      "Frankfurt-am-Main",
		State:     "Hesse",
		Country:   "DE",
		Elevation: 364,
		Latitude:  50.0264015198,
		Longitude: 8.543129921,
		Timezone:  "Europe/Berlin",
	},
	"EDDG": {
		ICAO:      "EDDG",
		IATA:      "FMO",
		Name:      "Munster Osnabruck Airport",
		City:      "Munster",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 160,
		Latitude:  52.134601593,
		Longitude: 7.6848301888,
		Timezone:  "Europe/Berlin",
	},
	"EDDH": {
		ICAO:      "EDDH",
		IATA:      "HAM",
		Name:      "Hamburg Airport",
		City:      "Hamburg",
		State:     "Hamburg",
		Country:   "DE",
		Elevation: 53,
		Latitude:  53.6304016113,
		Longitude: 9.9882297516,
		Timezone:  "Europe/Berlin",
	},
	"EDDI": {
		ICAO:      "EDDI",
		IATA:      "THF",
		Name:      "Berlin Tempelhof Airport",
		City:      "Berlin",
		State:     "Berlin",
		Country:   "DE",
		Elevation: 164,
		Latitude:  52.473611,
		Longitude: 13.401667,
		Timezone:  "Europe/Berlin",
	},
	"EDDK": {
		ICAO:      "EDDK",
		IATA:      "CGN",
		Name:      "Cologne Bonn Airport",
		City:      "Cologne",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 302,
		Latitude:  50.8658981323,
		Longitude: 7.1427397728,
		Timezone:  "Europe/Berlin",
	},
	"EDDL": {
		ICAO:      "EDDL",
		IATA:      "DUS",
		Name:      "Dusseldorf International Airport",
		City:      "Dusseldorf",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 147,
		Latitude:  51.2895011902,
		Longitude: 6.7667798996,
		Timezone:  "Europe/Berlin",
	},
	"EDDM": {
		ICAO:      "EDDM",
		IATA:      "MUC",
		Name:      "Munich International Airport",
		City:      "Munich",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1487,
		Latitude:  48.3538017273,
		Longitude: 11.7861003876,
		Timezone:  "Europe/Berlin",
	},
	"EDDN": {
		ICAO:      "EDDN",
		IATA:      "NUE",
		Name:      "Nuremberg Airport",
		City:      "Nuremberg",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1046,
		Latitude:  49.4986991882,
		Longitude: 11.0669002533,
		Timezone:  "Europe/Berlin",
	},
	"EDDP": {
		ICAO:      "EDDP",
		IATA:      "LEJ",
		Name:      "Leipzig Halle Airport",
		City:      "Leipzig",
		State:     "Saxony",
		Country:   "DE",
		Elevation: 465,
		Latitude:  51.4323997498,
		Longitude: 12.2416000366,
		Timezone:  "Europe/Berlin",
	},
	"EDDR": {
		ICAO:      "EDDR",
		IATA:      "SCN",
		Name:      "Saarbrucken Airport",
		City:      "Saarbrucken",
		State:     "Saarland",
		Country:   "DE",
		Elevation: 1058,
		Latitude:  49.2145996094,
		Longitude: 7.1095099449,
		Timezone:  "Europe/Berlin",
	},
	"EDDS": {
		ICAO:      "EDDS",
		IATA:      "STR",
		Name:      "Stuttgart Airport",
		City:      "Stuttgart",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 1276,
		Latitude:  48.6898994446,
		Longitude: 9.2219600677,
		Timezone:  "Europe/Berlin",
	},
	"EDDT": {
		ICAO:      "EDDT",
		IATA:      "TXL",
		Name:      "Berlin-Tegel International Airport",
		City:      "Berlin",
		State:     "Berlin",
		Country:   "DE",
		Elevation: 122,
		Latitude:  52.5597000122,
		Longitude: 13.2876996994,
		Timezone:  "Europe/Berlin",
	},
	"EDDV": {
		ICAO:      "EDDV",
		IATA:      "HAJ",
		Name:      "Hannover Airport",
		City:      "Hannover",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 183,
		Latitude:  52.461101532,
		Longitude: 9.6850795746,
		Timezone:  "Europe/Berlin",
	},
	"EDDW": {
		ICAO:      "EDDW",
		IATA:      "BRE",
		Name:      "Bremen Airport",
		City:      "Bremen",
		State:     "Bremen",
		Country:   "DE",
		Elevation: 14,
		Latitude:  53.0475006104,
		Longitude: 8.7866697311,
		Timezone:  "Europe/Berlin",
	},
	"EDEB": {
		ICAO:      "EDEB",
		Name:      "Bad Langensalza Airport",
		City:      "Bad Langensalza",
		State:     "Thuringia",
		Country:   "DE",
		Elevation: 650,
		Latitude:  51.1291656494,
		Longitude: 10.6222219467,
		Timezone:  "Europe/Berlin",
	},
	"EDEF": {
		ICAO:      "EDEF",
		Name:      "Babenhausen Airport",
		City:      "Babenhausen",
		State:     "Hesse",
		Country:   "DE",
		Elevation: 928,
		Latitude:  49.9524993896,
		Longitude: 8.9680557251,
		Timezone:  "Europe/Berlin",
	},
	"EDEG": {
		ICAO:      "EDEG",
		Name:      "Gotha-Ost Airport",
		City:      "Gotha",
		State:     "Thuringia",
		Country:   "DE",
		Elevation: 991,
		Latitude:  50.9700012207,
		Longitude: 10.7277784348,
		Timezone:  "Europe/Berlin",
	},
	"EDEH": {
		ICAO:      "EDEH",
		Name:      "Herrenteich Airport",
		City:      "Hockenheim",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 308,
		Latitude:  49.3450012207,
		Longitude: 8.48777771,
		Timezone:  "Europe/Berlin",
	},
	"EDEL": {
		ICAO:      "EDEL",
		Name:      "Langenlonsheim Airport",
		City:      "Langenlonsheim",
		State:     "Rheinland-Pfalz",
		Country:   "DE",
		Elevation: 295,
		Latitude:  49.9083328247,
		Longitude: 7.9077777863,
		Timezone:  "Europe/Berlin",
	},
	"EDEM": {
		ICAO:      "EDEM",
		Name:      "Mosenberg Airport",
		City:      "Homburg",
		State:     "Hesse",
		Country:   "DE",
		Elevation: 1296,
		Latitude:  51.0627784729,
		Longitude: 9.4222221375,
		Timezone:  "Europe/Berlin",
	},
	"EDEP": {
		ICAO:      "EDEP",
		Name:      "Heppenheim Airport",
		City:      "Heppenheim",
		State:     "Hesse",
		Country:   "DE",
		Elevation: 361,
		Latitude:  49.6219444275,
		Longitude: 8.6244440079,
		Timezone:  "Europe/Berlin",
	},
	"EDEQ": {
		ICAO:      "EDEQ",
		Name:      "Muhlhausen Airport",
		City:      "Muhlhausen",
		State:     "Thuringia",
		Country:   "DE",
		Elevation: 814,
		Latitude:  51.213054657,
		Longitude: 10.5486106873,
		Timezone:  "Europe/Berlin",
	},
	"EDER": {
		ICAO:      "EDER",
		Name:      "Wasserkuppe Airport",
		City:      "Gersfeld",
		State:     "Hesse",
		Country:   "DE",
		Elevation: 2956,
		Latitude:  50.4988899231,
		Longitude: 9.9538888931,
		Timezone:  "Europe/Berlin",
	},
	"EDEW": {
		ICAO:      "EDEW",
		Name:      "Walldurn Airport",
		City:      "Walldurn",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 1312,
		Latitude:  49.5816688538,
		Longitude: 9.4022216797,
		Timezone:  "Europe/Berlin",
	},
	"EDFA": {
		ICAO:      "EDFA",
		Name:      "Anspach/Taunus Airport",
		City:      "Anspach",
		State:     "Hesse",
		Country:   "DE",
		Elevation: 1102,
		Latitude:  50.2883338928,
		Longitude: 8.5338888168,
		Timezone:  "Europe/Berlin",
	},
	"EDFB": {
		ICAO:      "EDFB",
		Name:      "Reichelsheim Airport",
		City:      "Reichelsheim",
		State:     "Hesse",
		Country:   "DE",
		Elevation: 397,
		Latitude:  50.3358345032,
		Longitude: 8.8780555725,
		Timezone:  "Europe/Berlin",
	},
	"EDFC": {
		ICAO:      "EDFC",
		Name:      "Aschaffenburg Airport",
		City:      "Aschaffenburg",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 410,
		Latitude:  49.9388885498,
		Longitude: 9.0638885498,
		Timezone:  "Europe/Berlin",
	},
	"EDFD": {
		ICAO:      "EDFD",
		Name:      "Bad Neustadt/Saale-Grasberg Airport",
		City:      "Bad Neustadt a.d.Saale",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 997,
		Latitude:  50.3061103821,
		Longitude: 10.2266674042,
		Timezone:  "Europe/Berlin",
	},
	"EDFE": {
		ICAO:      "EDFE",
		IATA:      "QEF",
		Name:      "Frankfurt-Egelsbach Airport",
		City:      "Egelsbach",
		State:     "Hesse",
		Country:   "DE",
		Elevation: 384,
		Latitude:  49.9599990845,
		Longitude: 8.6458330154,
		Timezone:  "Europe/Berlin",
	},
	"EDFG": {
		ICAO:      "EDFG",
		Name:      "Gelnhausen Airport",
		City:      "Gelnhausen",
		State:     "Hesse",
		Country:   "DE",
		Elevation: 446,
		Latitude:  50.1972236633,
		Longitude: 9.1700000763,
		Timezone:  "Europe/Berlin",
	},
	"EDFH": {
		ICAO:      "EDFH",
		IATA:      "HHN",
		Name:      "Frankfurt-Hahn Airport",
		City:      "Hahn",
		State:     "Rheinland-Pfalz",
		Country:   "DE",
		Elevation: 1649,
		Latitude:  49.9486999512,
		Longitude: 7.2638897896,
		Timezone:  "Europe/Berlin",
	},
	"EDFI": {
		ICAO:      "EDFI",
		Name:      "Hirzenhain Airport",
		City:      "Hirzenhain",
		State:     "Hesse",
		Country:   "DE",
		Elevation: 1706,
		Latitude:  50.787776947,
		Longitude: 8.3927783966,
		Timezone:  "Europe/Berlin",
	},
	"EDFJ": {
		ICAO:      "EDFJ",
		Name:      "Lager Hammelburg Airport",
		City:      "Hammelburg",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1132,
		Latitude:  50.0986099243,
		Longitude: 9.8836107254,
		Timezone:  "Europe/Berlin",
	},
	"EDFK": {
		ICAO:      "EDFK",
		Name:      "Bad Kissingen Airport",
		City:      "Bad Kissingen",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 653,
		Latitude:  50.2105560303,
		Longitude: 10.0688886642,
		Timezone:  "Europe/Berlin",
	},
	"EDFL": {
		ICAO:      "EDFL",
		Name:      "Giesen-Lutzellinden Airport",
		City:      "Giesen",
		State:     "Hesse",
		Country:   "DE",
		Elevation: 755,
		Latitude:  50.543888092,
		Longitude: 8.5902776718,
		Timezone:  "Europe/Berlin",
	},
	"EDFM": {
		ICAO:      "EDFM",
		IATA:      "MHG",
		Name:      "Mannheim-City Airport",
		City:      "Mannheim",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 308,
		Latitude:  49.4730567932,
		Longitude: 8.514166832,
		Timezone:  "Europe/Berlin",
	},
	"EDFN": {
		ICAO:      "EDFN",
		Name:      "Marburg-Schonstadt Airport",
		City:      "Marburg",
		State:     "Hesse",
		Country:   "DE",
		Elevation: 833,
		Latitude:  50.8744430542,
		Longitude: 8.8149995804,
		Timezone:  "Europe/Berlin",
	},
	"EDFO": {
		ICAO:      "EDFO",
		Name:      "Michelstadt/Odenwald Airport",
		City:      "Michelstadt",
		State:     "Hesse",
		Country:   "DE",
		Elevation: 1142,
		Latitude:  49.6786117554,
		Longitude: 8.9719438553,
		Timezone:  "Europe/Berlin",
	},
	"EDFP": {
		ICAO:      "EDFP",
		Name:      "Ober-Morlen Airport",
		City:      "Ober-Morlen",
		State:     "Hesse",
		Country:   "DE",
		Elevation: 814,
		Latitude:  50.3619422913,
		Longitude: 8.7111110687,
		Timezone:  "Europe/Berlin",
	},
	"EDFQ": {
		ICAO:      "EDFQ",
		Name:      "Allendorf/Eder Airport",
		City:      "Allendorf",
		State:     "Hesse",
		Country:   "DE",
		Elevation: 1158,
		Latitude:  51.0349998474,
		Longitude: 8.6808328629,
		Timezone:  "Europe/Berlin",
	},
	"EDFR": {
		ICAO:      "EDFR",
		Name:      "Rothenburg o. d. T. Airport",
		City:      "Rothenburg ob der Tauber",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1309,
		Latitude:  49.3883323669,
		Longitude: 10.2180557251,
		Timezone:  "Europe/Berlin",
	},
	"EDFS": {
		ICAO:      "EDFS",
		Name:      "Schweinfurt-Sud Airport",
		City:      "Schweinfurt",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 686,
		Latitude:  50.0108337402,
		Longitude: 10.2511110306,
		Timezone:  "Europe/Berlin",
	},
	"EDFT": {
		ICAO:      "EDFT",
		Name:      "Lauterbach Airport",
		City:      "Lauterbach",
		State:     "Hesse",
		Country:   "DE",
		Elevation: 1211,
		Latitude:  50.6833343506,
		Longitude: 9.4108333588,
		Timezone:  "Europe/Berlin",
	},
	"EDFU": {
		ICAO:      "EDFU",
		Name:      "Mainbullau Airport",
		City:      "Miltenberg",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1503,
		Latitude:  49.6949996948,
		Longitude: 9.1824998856,
		Timezone:  "Europe/Berlin",
	},
	"EDFV": {
		ICAO:      "EDFV",
		Name:      "Worms Airport",
		City:      "Worms",
		State:     "Rheinland-Pfalz",
		Country:   "DE",
		Elevation: 295,
		Latitude:  49.6069450378,
		Longitude: 8.3683328629,
		Timezone:  "Europe/Berlin",
	},
	"EDFW": {
		ICAO:      "EDFW",
		Name:      "Wurzburg-Schenkenturm Airport",
		City:      "Wurzburg",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 991,
		Latitude:  49.817779541,
		Longitude: 9.8975000381,
		Timezone:  "Europe/Berlin",
	},
	"EDFX": {
		ICAO:      "EDFX",
		Name:      "Hockenheim Airport",
		City:      "Hockenheim",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 315,
		Latitude:  49.3252792358,
		Longitude: 8.5286111832,
		Timezone:  "Europe/Berlin",
	},
	"EDFY": {
		ICAO:      "EDFY",
		Name:      "Elz Airport",
		City:      "Limburg an der Lahn",
		State:     "Hesse",
		Country:   "DE",
		Elevation: 699,
		Latitude:  50.4269447327,
		Longitude: 8.0111112595,
		Timezone:  "Europe/Berlin",
	},
	"EDFZ": {
		ICAO:      "EDFZ",
		IATA:      "QMZ",
		Name:      "Mainz-Finthen Airport",
		City:      "Mainz",
		State:     "Rheinland-Pfalz",
		Country:   "DE",
		Elevation: 761,
		Latitude:  49.9674987793,
		Longitude: 8.1472215652,
		Timezone:  "Europe/Berlin",
	},
	"EDGA": {
		ICAO:      "EDGA",
		Name:      "Ailertchen Airport",
		City:      "Ailertchen",
		State:     "Rheinland-Pfalz",
		Country:   "DE",
		Elevation: 1542,
		Latitude:  50.5930557251,
		Longitude: 7.9450001717,
		Timezone:  "Europe/Berlin",
	},
	"EDGB": {
		ICAO:      "EDGB",
		Name:      "Breitscheid Airport",
		City:      "Breitscheid",
		State:     "Hesse",
		Country:   "DE",
		Elevation: 1834,
		Latitude:  50.6763877869,
		Longitude: 8.1694440842,
		Timezone:  "Europe/Berlin",
	},
	"EDGE": {
		ICAO:      "EDGE",
		IATA:      "EIB",
		Name:      "Eisenach-Kindel Airport",
		City:      "Eisenach",
		State:     "Thuringia",
		Country:   "DE",
		Elevation: 1112,
		Latitude:  50.9927787781,
		Longitude: 10.4727783203,
		Timezone:  "Europe/Berlin",
	},
	"EDGF": {
		ICAO:      "EDGF",
		Name:      "Fulda-Jossa Airport",
		City:      "Fulda",
		State:     "Hesse",
		Country:   "DE",
		Elevation: 1558,
		Latitude:  50.4755554199,
		Longitude: 9.4425001144,
		Timezone:  "Europe/Berlin",
	},
	"EDGH": {
		ICAO:      "EDGH",
		Name:      "Hettstadt Airport",
		City:      "Hettstadt",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1050,
		Latitude:  49.7986106873,
		Longitude: 9.8366670609,
		Timezone:  "Europe/Berlin",
	},
	"EDGI": {
		ICAO:      "EDGI",
		Name:      "Ingelfingen-Buhlhof Airport",
		City:      "Ingelfingen",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 1375,
		Latitude:  49.3216667175,
		Longitude: 9.6630563736,
		Timezone:  "Europe/Berlin",
	},
	"EDGJ": {
		ICAO:      "EDGJ",
		Name:      "Ochsenfurt Airport",
		City:      "Ochsenfurt",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 814,
		Latitude:  49.6736106873,
		Longitude: 10.0713891983,
		Timezone:  "Europe/Berlin",
	},
	"EDGK": {
		ICAO:      "EDGK",
		Name:      "Korbach Airport",
		City:      "Korbach",
		State:     "Hesse",
		Country:   "DE",
		Elevation: 1280,
		Latitude:  51.2522201538,
		Longitude: 8.8736114502,
		Timezone:  "Europe/Berlin",
	},
	"EDGL": {
		ICAO:      "EDGL",
		Name:      "Ludwigshafen Airfield",
		City:      "Dannstadt",
		State:     "Rheinland-Pfalz",
		Country:   "DE",
		Elevation: 318,
		Latitude:  49.4131,
		Longitude: 8.3518,
		Timezone:  "Europe/Berlin",
	},
	"EDGM": {
		ICAO:      "EDGM",
		Name:      "Mosbach-Lohrbach Airport",
		City:      "Mosbach",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 1818,
		Latitude:  49.3988876343,
		Longitude: 9.1238889694,
		Timezone:  "Europe/Berlin",
	},
	"EDGP": {
		ICAO:      "EDGP",
		Name:      "Oppenheim Airport",
		City:      "Oppenheim",
		State:     "Rheinland-Pfalz",
		Country:   "DE",
		Elevation: 279,
		Latitude:  49.8416671753,
		Longitude: 8.3766670227,
		Timezone:  "Europe/Berlin",
	},
	"EDGQ": {
		ICAO:      "EDGQ",
		Name:      "Schameder Airport",
		City:      "Schameder",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 1788,
		Latitude:  51.0002784729,
		Longitude: 8.3077783585,
		Timezone:  "Europe/Berlin",
	},
	"EDGR": {
		ICAO:      "EDGR",
		Name:      "Giesen-Reiskirchen Airport",
		City:      "Giesen",
		State:     "Hesse",
		Country:   "DE",
		Elevation: 702,
		Latitude:  50.5669441223,
		Longitude: 8.8697223663,
		Timezone:  "Europe/Berlin",
	},
	"EDGS": {
		ICAO:      "EDGS",
		IATA:      "SGE",
		Name:      "Siegerland Airport",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 1966,
		Latitude:  50.707698822,
		Longitude: 8.0829696655,
		Timezone:  "Europe/Berlin",
	},
	"EDGT": {
		ICAO:      "EDGT",
		Name:      "Bottenhorn Airport",
		City:      "Bottenhorn",
		State:     "Hesse",
		Country:   "DE",
		Elevation: 1657,
		Latitude:  50.7952766418,
		Longitude: 8.4583330154,
		Timezone:  "Europe/Berlin",
	},
	"EDGU": {
		ICAO:      "EDGU",
		Name:      "Unterschupf Airport",
		City:      "Unterschupf",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 1155,
		Latitude:  49.5158348083,
		Longitude: 9.6694440842,
		Timezone:  "Europe/Berlin",
	},
	"EDGW": {
		ICAO:      "EDGW",
		Name:      "Wolfhagen „Graner Berg“ Airport",
		City:      "Wolfhagen",
		State:     "Hesse",
		Country:   "DE",
		Elevation: 1027,
		Latitude:  51.307220459,
		Longitude: 9.17527771,
		Timezone:  "Europe/Berlin",
	},
	"EDGX": {
		ICAO:      "EDGX",
		Name:      "Walldorf Airport",
		City:      "Walldorf",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 344,
		Latitude:  49.3030548096,
		Longitude: 8.6588888168,
		Timezone:  "Europe/Berlin",
	},
	"EDGZ": {
		ICAO:      "EDGZ",
		Name:      "Weinheim/Bergstrase Airport",
		City:      "Weinheim",
		State:     "Hesse",
		Country:   "DE",
		Elevation: 318,
		Latitude:  49.5675010681,
		Longitude: 8.6105556488,
		Timezone:  "Europe/Berlin",
	},
	"EDHB": {
		ICAO:      "EDHB",
		Name:      "Grube Airport",
		City:      "Grube",
		State:     "Schleswig-Holstein",
		Country:   "DE",
		Elevation: 7,
		Latitude:  54.2444458008,
		Longitude: 11.0247220993,
		Timezone:  "Europe/Berlin",
	},
	"EDHC": {
		ICAO:      "EDHC",
		Name:      "Luchow-Rehbeck Airport",
		City:      "Luchow",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 49,
		Latitude:  53.0161094666,
		Longitude: 11.1444444656,
		Timezone:  "Europe/Berlin",
	},
	"EDHE": {
		ICAO:      "EDHE",
		Name:      "Uetersen/Heist Airport",
		City:      "Uetersen",
		State:     "Schleswig-Holstein",
		Country:   "DE",
		Elevation: 23,
		Latitude:  53.6463890076,
		Longitude: 9.704167366,
		Timezone:  "Europe/Berlin",
	},
	"EDHF": {
		ICAO:      "EDHF",
		Name:      "Itzehoe/Hungriger Wolf Airport",
		City:      "Itzehoe",
		State:     "Schleswig-Holstein",
		Country:   "DE",
		Elevation: 89,
		Latitude:  53.9944458008,
		Longitude: 9.5786113739,
		Timezone:  "Europe/Berlin",
	},
	"EDHG": {
		ICAO:      "EDHG",
		Name:      "Luneburg Airport",
		City:      "Luneburg",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 161,
		Latitude:  53.2488899231,
		Longitude: 10.4616670609,
		Timezone:  "Europe/Berlin",
	},
	"EDHI": {
		ICAO:      "EDHI",
		IATA:      "XFW",
		Name:      "Hamburg-Finkenwerder Airport",
		City:      "Hamburg",
		State:     "Hamburg",
		Country:   "DE",
		Elevation: 23,
		Latitude:  53.5352783203,
		Longitude: 9.8355560303,
		Timezone:  "Europe/Berlin",
	},
	"EDHK": {
		ICAO:      "EDHK",
		IATA:      "KEL",
		Name:      "Kiel-Holtenau Airport",
		City:      "Kiel",
		State:     "Schleswig-Holstein",
		Country:   "DE",
		Elevation: 102,
		Latitude:  54.3794441223,
		Longitude: 10.145277977,
		Timezone:  "Europe/Berlin",
	},
	"EDHL": {
		ICAO:      "EDHL",
		IATA:      "LBC",
		Name:      "Lubeck Blankensee Airport",
		City:      "Lubeck",
		State:     "Schleswig-Holstein",
		Country:   "DE",
		Elevation: 53,
		Latitude:  53.8054008484,
		Longitude: 10.7192001343,
		Timezone:  "Europe/Berlin",
	},
	"EDHM": {
		ICAO:      "EDHM",
		Name:      "Hartenholm Airport",
		City:      "Hasenmoor",
		State:     "Schleswig-Holstein",
		Country:   "DE",
		Elevation: 108,
		Latitude:  53.9150009155,
		Longitude: 10.0355558395,
		Timezone:  "Europe/Berlin",
	},
	"EDHN": {
		ICAO:      "EDHN",
		IATA:      "EUM",
		Name:      "Neumunster Airport",
		City:      "Neumunster",
		State:     "Schleswig-Holstein",
		Country:   "DE",
		Elevation: 72,
		Latitude:  54.0794448853,
		Longitude: 9.9413890839,
		Timezone:  "Europe/Berlin",
	},
	"EDHO": {
		ICAO:      "EDHO",
		Name:      "Ahrenlohe Airport",
		City:      "Tornesch",
		State:     "Schleswig-Holstein",
		Country:   "DE",
		Elevation: 33,
		Latitude:  53.69972229,
		Longitude: 9.7405557632,
		Timezone:  "Europe/Berlin",
	},
	"EDHP": {
		ICAO:      "EDHP",
		Name:      "Pellworm Airport",
		City:      "Pellworm",
		State:     "Schleswig-Holstein",
		Country:   "DE",
		Elevation: 3,
		Latitude:  54.5363883972,
		Longitude: 8.6800003052,
		Timezone:  "Europe/Berlin",
	},
	"EDHS": {
		ICAO:      "EDHS",
		Name:      "Stade Airport",
		City:      "Stade",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 62,
		Latitude:  53.5611114502,
		Longitude: 9.4991674423,
		Timezone:  "Europe/Berlin",
	},
	"EDHU": {
		ICAO:      "EDHU",
		Name:      "Lauenbruck Airport",
		City:      "Lauenbruck",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 98,
		Latitude:  53.2075004578,
		Longitude: 9.5733327866,
		Timezone:  "Europe/Berlin",
	},
	"EDHW": {
		ICAO:      "EDHW",
		Name:      "Wahlstedt Airport",
		City:      "Wahlstedt",
		State:     "Schleswig-Holstein",
		Country:   "DE",
		Elevation: 128,
		Latitude:  53.9694442749,
		Longitude: 10.2216672897,
		Timezone:  "Europe/Berlin",
	},
	"EDJA": {
		ICAO:      "EDJA",
		IATA:      "FMM",
		Name:      "Memmingen Allgau Airport",
		City:      "Memmingen",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 2077,
		Latitude:  47.9888000488,
		Longitude: 10.2395000458,
		Timezone:  "Europe/Berlin",
	},
	"EDKA": {
		ICAO:      "EDKA",
		IATA:      "AAH",
		Name:      "Aachen-Merzbruck Airport",
		City:      "Aachen",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 623,
		Latitude:  50.8230552673,
		Longitude: 6.1863889694,
		Timezone:  "Europe/Berlin",
	},
	"EDKB": {
		ICAO:      "EDKB",
		IATA:      "BNJ",
		Name:      "Bonn-Hangelar Airport",
		City:      "Bonn",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 197,
		Latitude:  50.7688903809,
		Longitude: 7.1633329391,
		Timezone:  "Europe/Berlin",
	},
	"EDKD": {
		ICAO:      "EDKD",
		Name:      "Altena-Hegenscheid Airport",
		City:      "Altena",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 1552,
		Latitude:  51.3130569458,
		Longitude: 7.7041668892,
		Timezone:  "Europe/Berlin",
	},
	"EDKF": {
		ICAO:      "EDKF",
		Name:      "Bergneustadt/Auf dem Dumpel Airport",
		City:      "Bergneustadt",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 1604,
		Latitude:  51.0522232056,
		Longitude: 7.7072219849,
		Timezone:  "Europe/Berlin",
	},
	"EDKH": {
		ICAO:      "EDKH",
		Name:      "Hunsborn Airport",
		City:      "Freudenberg",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 1306,
		Latitude:  50.9286117554,
		Longitude: 7.8991670609,
		Timezone:  "Europe/Berlin",
	},
	"EDKI": {
		ICAO:      "EDKI",
		Name:      "Betzdorf-Kirchen Airport",
		City:      "Betzdorf",
		State:     "Rheinland-Pfalz",
		Country:   "DE",
		Elevation: 1125,
		Latitude:  50.8172225952,
		Longitude: 7.8305559158,
		Timezone:  "Europe/Berlin",
	},
	"EDKL": {
		ICAO:      "EDKL",
		Name:      "Leverkusen Airport",
		City:      "Leverkusen",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 157,
		Latitude:  51.0152778625,
		Longitude: 7.0055561066,
		Timezone:  "Europe/Berlin",
	},
	"EDKM": {
		ICAO:      "EDKM",
		Name:      "Meschede-Schuren Airport",
		City:      "Meschede",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 1434,
		Latitude:  51.3027763367,
		Longitude: 8.2391672134,
		Timezone:  "Europe/Berlin",
	},
	"EDKN": {
		ICAO:      "EDKN",
		Name:      "Wipperfurth-Neye Airport",
		City:      "Wipperfurth",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 863,
		Latitude:  51.124168396,
		Longitude: 7.3736109734,
		Timezone:  "Europe/Berlin",
	},
	"EDKO": {
		ICAO:      "EDKO",
		Name:      "Brilon/Hochsauerlandkreis Airport",
		City:      "Brilon",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 1509,
		Latitude:  51.4025001526,
		Longitude: 8.641667366,
		Timezone:  "Europe/Berlin",
	},
	"EDKP": {
		ICAO:      "EDKP",
		Name:      "Plettenberg-Huinghausen Airport",
		City:      "Plettenberg",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 981,
		Latitude:  51.1919441223,
		Longitude: 7.7911109924,
		Timezone:  "Europe/Berlin",
	},
	"EDKR": {
		ICAO:      "EDKR",
		Name:      "Schmallenberg-Rennefeld Airport",
		City:      "Schmallenberg",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 1529,
		Latitude:  51.1616668701,
		Longitude: 8.26222229,
		Timezone:  "Europe/Berlin",
	},
	"EDKU": {
		ICAO:      "EDKU",
		Name:      "Attendorn-Finnentrop Airport",
		City:      "Attendorn",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 1040,
		Latitude:  51.1458320618,
		Longitude: 7.9366669655,
		Timezone:  "Europe/Berlin",
	},
	"EDKV": {
		ICAO:      "EDKV",
		Name:      "Dahlemer Binz Airport",
		City:      "Dahlheim",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 1896,
		Latitude:  50.4055557251,
		Longitude: 6.5288891792,
		Timezone:  "Europe/Berlin",
	},
	"EDKW": {
		ICAO:      "EDKW",
		Name:      "Werdohl-Kuntrop Airport",
		City:      "Werdohl",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 1037,
		Latitude:  51.2972221375,
		Longitude: 7.818333149,
		Timezone:  "Europe/Berlin",
	},
	"EDKZ": {
		ICAO:      "EDKZ",
		Name:      "Meinerzhagen Airport",
		City:      "Meinerzhagen",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 1549,
		Latitude:  51.0999984741,
		Longitude: 7.5999999046,
		Timezone:  "Europe/Berlin",
	},
	"EDLA": {
		ICAO:      "EDLA",
		Name:      "Arnsberg-Menden Airport",
		City:      "Neheim-Husten",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 794,
		Latitude:  51.4838905334,
		Longitude: 7.8983330727,
		Timezone:  "Europe/Berlin",
	},
	"EDLB": {
		ICAO:      "EDLB",
		Name:      "Borkenberge Airport",
		City:      "Dulmen",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 157,
		Latitude:  51.7799987793,
		Longitude: 7.2880558968,
		Timezone:  "Europe/Berlin",
	},
	"EDLC": {
		ICAO:      "EDLC",
		Name:      "Kamp-Lintfort Airport",
		City:      "Rheinberg",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 85,
		Latitude:  51.5291671753,
		Longitude: 6.536110878,
		Timezone:  "Europe/Berlin",
	},
	"EDLD": {
		ICAO:      "EDLD",
		Name:      "Dinslaken/Schwarze Heide Airport",
		City:      "Bottrop-Kirchhellen",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 217,
		Latitude:  51.6161117554,
		Longitude: 6.8652777672,
		Timezone:  "Europe/Berlin",
	},
	"EDLE": {
		ICAO:      "EDLE",
		IATA:      "ESS",
		Name:      "Essen Mulheim Airport",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 424,
		Latitude:  51.4023017883,
		Longitude: 6.9373297691,
		Timezone:  "Europe/Berlin",
	},
	"EDLF": {
		ICAO:      "EDLF",
		Name:      "Grefrath-Niershorst Airport",
		City:      "Grefrath",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 105,
		Latitude:  51.3338890076,
		Longitude: 6.3594441414,
		Timezone:  "Europe/Berlin",
	},
	"EDLG": {
		ICAO:      "EDLG",
		Name:      "Goch-Asperden Airport",
		City:      "Goch",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 52,
		Latitude:  51.6908340454,
		Longitude: 6.1041669846,
		Timezone:  "Europe/Berlin",
	},
	"EDLH": {
		ICAO:      "EDLH",
		Name:      "Hamm-Lippewiesen Airport",
		City:      "Hamm",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 190,
		Latitude:  51.6897201538,
		Longitude: 7.8161110878,
		Timezone:  "Europe/Berlin",
	},
	"EDLI": {
		ICAO:      "EDLI",
		IATA:      "BFE",
		Name:      "Bielefeld Airport",
		City:      "Bielefeld",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 433,
		Latitude:  51.9647216797,
		Longitude: 8.5444440842,
		Timezone:  "Europe/Berlin",
	},
	"EDLJ": {
		ICAO:      "EDLJ",
		Name:      "Detmold Airport",
		City:      "Detmold",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 650,
		Latitude:  51.9408340454,
		Longitude: 8.9047222137,
		Timezone:  "Europe/Berlin",
	},
	"EDLK": {
		ICAO:      "EDLK",
		Name:      "Krefeld-Egelsberg Airport",
		City:      "Krefeld",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 141,
		Latitude:  51.3849983215,
		Longitude: 6.5877780914,
		Timezone:  "Europe/Berlin",
	},
	"EDLM": {
		ICAO:      "EDLM",
		IATA:      "ZOJ",
		Name:      "Marl-Loemuhle Airport",
		City:      "Marl",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 240,
		Latitude:  51.6472015381,
		Longitude: 7.1633300781,
		Timezone:  "Europe/Berlin",
	},
	"EDLN": {
		ICAO:      "EDLN",
		IATA:      "MGL",
		Name:      "Monchengladbach Airport",
		City:      "Monchengladbach",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 125,
		Latitude:  51.2302780151,
		Longitude: 6.5044441223,
		Timezone:  "Europe/Berlin",
	},
	"EDLO": {
		ICAO:      "EDLO",
		Name:      "Oerlinghausen Airport",
		City:      "Oerlinghausen",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 558,
		Latitude:  51.932220459,
		Longitude: 8.6616668701,
		Timezone:  "Europe/Berlin",
	},
	"EDLP": {
		ICAO:      "EDLP",
		IATA:      "PAD",
		Name:      "Paderborn Lippstadt Airport",
		City:      "Paderborn",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 699,
		Latitude:  51.6141014099,
		Longitude: 8.6163196564,
		Timezone:  "Europe/Berlin",
	},
	"EDLR": {
		ICAO:      "EDLR",
		Name:      "Paderborn-Haxterberg Airport",
		City:      "Paderborn",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 801,
		Latitude:  51.688331604,
		Longitude: 8.7752780914,
		Timezone:  "Europe/Berlin",
	},
	"EDLS": {
		ICAO:      "EDLS",
		Name:      "Stadtlohn-Vreden Airport",
		City:      "Stadtlohn",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 157,
		Latitude:  51.9958343506,
		Longitude: 6.8405561447,
		Timezone:  "Europe/Amsterdam",
	},
	"EDLT": {
		ICAO:      "EDLT",
		Name:      "Munster-Telgte Airport",
		City:      "Munster",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 177,
		Latitude:  51.944442749,
		Longitude: 7.7738890648,
		Timezone:  "Europe/Berlin",
	},
	"EDLU": {
		ICAO:      "EDLU",
		Name:      "Oelde Bergeler Airport",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 269,
		Latitude:  51.8306007385,
		Longitude: 8.1747198105,
		Timezone:  "Europe/Berlin",
	},
	"EDLV": {
		ICAO:      "EDLV",
		IATA:      "NRN",
		Name:      "Niederrhein Airport",
		City:      "Weeze",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 106,
		Latitude:  51.6024017334,
		Longitude: 6.1421699524,
		Timezone:  "Europe/Amsterdam",
	},
	"EDLW": {
		ICAO:      "EDLW",
		IATA:      "DTM",
		Name:      "Dortmund Airport",
		City:      "Dortmund",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 425,
		Latitude:  51.5182991028,
		Longitude: 7.6122398377,
		Timezone:  "Europe/Berlin",
	},
	"EDLX": {
		ICAO:      "EDLX",
		Name:      "Wesel-Romerwardt Airport",
		City:      "Wesel",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 72,
		Latitude:  51.662776947,
		Longitude: 6.5958328247,
		Timezone:  "Europe/Berlin",
	},
	"EDLY": {
		ICAO:      "EDLY",
		Name:      "Borken-Hoxfeld Airport",
		City:      "Borken",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 157,
		Latitude:  51.8533325195,
		Longitude: 6.8147220612,
		Timezone:  "Europe/Berlin",
	},
	"EDLZ": {
		ICAO:      "EDLZ",
		Name:      "Soest/Bad Sassendorf Airport",
		City:      "Bad Sassendorf",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 394,
		Latitude:  51.5780563354,
		Longitude: 8.2147216797,
		Timezone:  "Europe/Berlin",
	},
	"EDMA": {
		ICAO:      "EDMA",
		IATA:      "AGB",
		Name:      "Augsburg Airport",
		City:      "Augsburg",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1516,
		Latitude:  48.42527771,
		Longitude: 10.9316673279,
		Timezone:  "Europe/Berlin",
	},
	"EDMB": {
		ICAO:      "EDMB",
		Name:      "Biberach a.d. Ris Airport",
		City:      "Biberach an der Ris",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 1903,
		Latitude:  48.1111106873,
		Longitude: 9.7627782822,
		Timezone:  "Europe/Berlin",
	},
	"EDMC": {
		ICAO:      "EDMC",
		Name:      "Blaubeuren Airport",
		City:      "Blaubeuren",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 2218,
		Latitude:  48.4197235107,
		Longitude: 9.798333168,
		Timezone:  "Europe/Berlin",
	},
	"EDMD": {
		ICAO:      "EDMD",
		Name:      "Dachau-Grobenried Airport",
		City:      "Dachau",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1608,
		Latitude:  48.2283325195,
		Longitude: 11.4230556488,
		Timezone:  "Europe/Berlin",
	},
	"EDME": {
		ICAO:      "EDME",
		Name:      "Eggenfelden Airport",
		City:      "Eggenfelden",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1342,
		Latitude:  48.3961105347,
		Longitude: 12.723610878,
		Timezone:  "Europe/Berlin",
	},
	"EDMF": {
		ICAO:      "EDMF",
		Name:      "Furstenzell Airport",
		City:      "Furstenzell",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1345,
		Latitude:  48.5180549622,
		Longitude: 13.3458328247,
		Timezone:  "Europe/Berlin",
	},
	"EDMG": {
		ICAO:      "EDMG",
		Name:      "Gunzburg-Donauried Airport",
		City:      "Gunzburg",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1457,
		Latitude:  48.4866676331,
		Longitude: 10.2833328247,
		Timezone:  "Europe/Berlin",
	},
	"EDMH": {
		ICAO:      "EDMH",
		Name:      "Gunzenhausen-Reutberg Airport",
		City:      "Gunzenhausen",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1591,
		Latitude:  49.1119422913,
		Longitude: 10.7808332443,
		Timezone:  "Europe/Berlin",
	},
	"EDMI": {
		ICAO:      "EDMI",
		Name:      "Illertissen Airport",
		City:      "Illertissen",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1680,
		Latitude:  48.2350006104,
		Longitude: 10.1374998093,
		Timezone:  "Europe/Berlin",
	},
	"EDMJ": {
		ICAO:      "EDMJ",
		Name:      "Jesenwang Airport",
		City:      "Jesenwang",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1860,
		Latitude:  48.1741676331,
		Longitude: 11.125,
		Timezone:  "Europe/Berlin",
	},
	"EDMK": {
		ICAO:      "EDMK",
		Name:      "Kempten-Durach Airport",
		City:      "Kempten",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 2339,
		Latitude:  47.6919441223,
		Longitude: 10.3380556107,
		Timezone:  "Europe/Berlin",
	},
	"EDML": {
		ICAO:      "EDML",
		Name:      "Landshut Airport",
		City:      "Landshut",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1312,
		Latitude:  48.5116653442,
		Longitude: 12.0333328247,
		Timezone:  "Europe/Berlin",
	},
	"EDMN": {
		ICAO:      "EDMN",
		Name:      "Mindelheim-Mattsies Airport",
		City:      "Mindelheim",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1857,
		Latitude:  48.1069450378,
		Longitude: 10.5249996185,
		Timezone:  "Europe/Berlin",
	},
	"EDMO": {
		ICAO:      "EDMO",
		IATA:      "OBF",
		Name:      "Oberpfaffenhofen Airport",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1947,
		Latitude:  48.081401825,
		Longitude: 11.2831001282,
		Timezone:  "Europe/Berlin",
	},
	"EDMP": {
		ICAO:      "EDMP",
		Name:      "Vilsbiburg Airport",
		City:      "Vilsbiburg",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1450,
		Latitude:  48.426109314,
		Longitude: 12.3452777863,
		Timezone:  "Europe/Berlin",
	},
	"EDMQ": {
		ICAO:      "EDMQ",
		Name:      "Donauworth-Genderkingen Airport",
		City:      "Donauworth",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1312,
		Latitude:  48.7027778625,
		Longitude: 10.851111412,
		Timezone:  "Europe/Berlin",
	},
	"EDMS": {
		ICAO:      "EDMS",
		IATA:      "RBM",
		Name:      "Straubing Airport",
		City:      "Straubing",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1047,
		Latitude:  48.9008331299,
		Longitude: 12.516667366,
		Timezone:  "Europe/Berlin",
	},
	"EDMT": {
		ICAO:      "EDMT",
		Name:      "Tannheim Airport",
		City:      "Tannheim",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 1903,
		Latitude:  48.0099983215,
		Longitude: 10.098610878,
		Timezone:  "Europe/Berlin",
	},
	"EDMU": {
		ICAO:      "EDMU",
		Name:      "Gundelfingen Airport",
		City:      "Gundelfingen a.d.Donau",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1447,
		Latitude:  48.569442749,
		Longitude: 10.3588886261,
		Timezone:  "Europe/Berlin",
	},
	"EDMV": {
		ICAO:      "EDMV",
		Name:      "Vilshofen Airport",
		City:      "Vilshofen",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 991,
		Latitude:  48.6349983215,
		Longitude: 13.195555687,
		Timezone:  "Europe/Berlin",
	},
	"EDMW": {
		ICAO:      "EDMW",
		Name:      "Deggendorf Airport",
		City:      "Deggendorf",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1030,
		Latitude:  48.8302764893,
		Longitude: 12.8797216415,
		Timezone:  "Europe/Berlin",
	},
	"EDMY": {
		ICAO:      "EDMY",
		Name:      "Muhldorf Airport",
		City:      "Muhldorf am Inn",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1325,
		Latitude:  48.2794456482,
		Longitude: 12.5005559921,
		Timezone:  "Europe/Berlin",
	},
	"EDNA": {
		ICAO:      "EDNA",
		Name:      "Ampfing-Waldkraiburg Airport",
		City:      "Ampfing",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1362,
		Latitude:  48.2636108398,
		Longitude: 12.4116668701,
		Timezone:  "Europe/Berlin",
	},
	"EDNB": {
		ICAO:      "EDNB",
		Name:      "Arnbruck Airport",
		City:      "Arnbruck",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1716,
		Latitude:  49.1247215271,
		Longitude: 12.9855556488,
		Timezone:  "Europe/Berlin",
	},
	"EDNC": {
		ICAO:      "EDNC",
		Name:      "Beilngries Airport",
		City:      "Beilngries",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1211,
		Latitude:  49.0213890076,
		Longitude: 11.4847221375,
		Timezone:  "Europe/Berlin",
	},
	"EDND": {
		ICAO:      "EDND",
		Name:      "Dinkelsbuhl-Sinbronn Airport",
		City:      "Dinkelsbuhl",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1598,
		Latitude:  49.0649986267,
		Longitude: 10.4011106491,
		Timezone:  "Europe/Berlin",
	},
	"EDNE": {
		ICAO:      "EDNE",
		Name:      "Erbach Airport",
		City:      "Erbach",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 1558,
		Latitude:  48.3422203064,
		Longitude: 9.9161109924,
		Timezone:  "Europe/Berlin",
	},
	"EDNF": {
		ICAO:      "EDNF",
		Name:      "Elsenthal Grafe Airport",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1417,
		Latitude:  48.8224983215,
		Longitude: 13.3675003052,
		Timezone:  "Europe/Berlin",
	},
	"EDNG": {
		ICAO:      "EDNG",
		Name:      "Giengen/Brenz Airport",
		City:      "Giengen an der Brenz",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 1696,
		Latitude:  48.6341667175,
		Longitude: 10.2163887024,
		Timezone:  "Europe/Berlin",
	},
	"EDNH": {
		ICAO:      "EDNH",
		Name:      "Bad Worishofen-Nord Airport",
		City:      "Bad Worishofen",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1220,
		Latitude:  48.0163879395,
		Longitude: 10.6161108017,
		Timezone:  "Europe/Berlin",
	},
	"EDNI": {
		ICAO:      "EDNI",
		Name:      "Berching Airport",
		City:      "Berching",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1266,
		Latitude:  49.1305541992,
		Longitude: 11.4430561066,
		Timezone:  "Europe/Berlin",
	},
	"EDNJ": {
		ICAO:      "EDNJ",
		Name:      "Neuburg-Egweil Airport",
		City:      "Neuburg an der Donau",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1345,
		Latitude:  48.7819442749,
		Longitude: 11.2152776718,
		Timezone:  "Europe/Berlin",
	},
	"EDNK": {
		ICAO:      "EDNK",
		Name:      "Kirchdorf/Inn Airport",
		City:      "Kirchdorf am Inn",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1138,
		Latitude:  48.2383346558,
		Longitude: 12.9766674042,
		Timezone:  "Europe/Berlin",
	},
	"EDNL": {
		ICAO:      "EDNL",
		Name:      "Leutkirch-Unterzeil Airport",
		City:      "Leutkirch",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 2100,
		Latitude:  47.8588905334,
		Longitude: 10.014166832,
		Timezone:  "Europe/Berlin",
	},
	"EDNM": {
		ICAO:      "EDNM",
		Name:      "Nittenau-Bruck Airport",
		City:      "Nittenau",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1161,
		Latitude:  49.2224998474,
		Longitude: 12.296667099,
		Timezone:  "Europe/Berlin",
	},
	"EDNO": {
		ICAO:      "EDNO",
		Name:      "Nordlingen Airport",
		City:      "Nordlingen",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1385,
		Latitude:  48.8705558777,
		Longitude: 10.5050001144,
		Timezone:  "Europe/Berlin",
	},
	"EDNP": {
		ICAO:      "EDNP",
		Name:      "Pfarrkirchen Airport",
		City:      "Pfarrkirchen",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1266,
		Latitude:  48.4202766418,
		Longitude: 12.8647222519,
		Timezone:  "Europe/Berlin",
	},
	"EDNQ": {
		ICAO:      "EDNQ",
		Name:      "Bopfingen Airport",
		City:      "Bopfingen",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 2028,
		Latitude:  48.8480567932,
		Longitude: 10.3338890076,
		Timezone:  "Europe/Berlin",
	},
	"EDNR": {
		ICAO:      "EDNR",
		Name:      "Regensburg-Oberhub Airport",
		City:      "Regensburg",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1299,
		Latitude:  49.1419448853,
		Longitude: 12.0819444656,
		Timezone:  "Europe/Berlin",
	},
	"EDNS": {
		ICAO:      "EDNS",
		Name:      "Schwabmunchen Airport",
		City:      "Schwabmunchen",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1804,
		Latitude:  48.1791687012,
		Longitude: 10.7027778625,
		Timezone:  "Europe/Berlin",
	},
	"EDNT": {
		ICAO:      "EDNT",
		Name:      "Treuchtlingen-Bubenheim Airport",
		City:      "Treuchtlingen",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1345,
		Latitude:  48.9961128235,
		Longitude: 10.8841667175,
		Timezone:  "Europe/Berlin",
	},
	"EDNU": {
		ICAO:      "EDNU",
		Name:      "Thannhausen Airport",
		City:      "Thannhausen",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1611,
		Latitude:  48.2900009155,
		Longitude: 10.4419441223,
		Timezone:  "Europe/Berlin",
	},
	"EDNV": {
		ICAO:      "EDNV",
		Name:      "Vogtareuth Airport",
		City:      "Vogtareuth",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1535,
		Latitude:  47.9461097717,
		Longitude: 12.2047224045,
		Timezone:  "Europe/Berlin",
	},
	"EDNW": {
		ICAO:      "EDNW",
		Name:      "Weisenhorn Airport",
		City:      "Weisenhorn",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1644,
		Latitude:  48.2894439697,
		Longitude: 10.1402778625,
		Timezone:  "Europe/Berlin",
	},
	"EDNX": {
		ICAO:      "EDNX",
		Name:      "Oberschleisheim Airfield",
		City:      "Oberschleisheim",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1594,
		Latitude:  48.2394447327,
		Longitude: 11.5613889694,
		Timezone:  "Europe/Berlin",
	},
	"EDNY": {
		ICAO:      "EDNY",
		IATA:      "FDH",
		Name:      "Friedrichshafen Airport",
		City:      "Friedrichshafen",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 1367,
		Latitude:  47.6712989807,
		Longitude: 9.5114898682,
		Timezone:  "Europe/Berlin",
	},
	"EDNZ": {
		ICAO:      "EDNZ",
		Name:      "Zell-Haidberg Airport",
		City:      "Zell im Fichtelgebirge",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 2083,
		Latitude:  50.1369438171,
		Longitude: 11.7944440842,
		Timezone:  "Europe/Berlin",
	},
	"EDOA": {
		ICAO:      "EDOA",
		Name:      "Auerbach Airport",
		City:      "Auerbach/Vogtl.",
		State:     "Saxony",
		Country:   "DE",
		Elevation: 1880,
		Latitude:  50.4988899231,
		Longitude: 12.3777780533,
		Timezone:  "Europe/Berlin",
	},
	"EDOB": {
		ICAO:      "EDOB",
		Name:      "Bad Berka Airport",
		City:      "Bad Berka",
		State:     "Thuringia",
		Country:   "DE",
		Elevation: 1001,
		Latitude:  50.9044456482,
		Longitude: 11.2602777481,
		Timezone:  "Europe/Berlin",
	},
	"EDOC": {
		ICAO:      "EDOC",
		Name:      "Gardelegen Airport",
		City:      "Gardelegen",
		State:     "Saxony-Anhalt",
		Country:   "DE",
		Elevation: 230,
		Latitude:  52.5272216797,
		Longitude: 11.3513889313,
		Timezone:  "Europe/Berlin",
	},
	"EDOD": {
		ICAO:      "EDOD",
		Name:      "Reinsdorf Airport",
		City:      "Reinsdorf",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 335,
		Latitude:  51.9008331299,
		Longitude: 13.1944437027,
		Timezone:  "Europe/Berlin",
	},
	"EDOE": {
		ICAO:      "EDOE",
		Name:      "Bohlen Airport",
		City:      "Bohlen",
		State:     "Saxony",
		Country:   "DE",
		Elevation: 430,
		Latitude:  51.2138900757,
		Longitude: 12.3636112213,
		Timezone:  "Europe/Berlin",
	},
	"EDOF": {
		ICAO:      "EDOF",
		Name:      "Bad Frankenhausen Airport",
		City:      "Bad Frankenhausen",
		State:     "Thuringia",
		Country:   "DE",
		Elevation: 761,
		Latitude:  51.3725013733,
		Longitude: 11.1413888931,
		Timezone:  "Europe/Berlin",
	},
	"EDOG": {
		ICAO:      "EDOG",
		Name:      "Torgau-Beilrode Airport",
		City:      "Torgau",
		State:     "Saxony",
		Country:   "DE",
		Elevation: 269,
		Latitude:  51.5711669922,
		Longitude: 13.0521669388,
		Timezone:  "Europe/Berlin",
	},
	"EDOH": {
		ICAO:      "EDOH",
		Name:      "Langhennersdorf Airport",
		City:      "Freiberg",
		State:     "Saxony",
		Country:   "DE",
		Elevation: 1266,
		Latitude:  50.9483337402,
		Longitude: 13.2616672516,
		Timezone:  "Europe/Berlin",
	},
	"EDOI": {
		ICAO:      "EDOI",
		Name:      "Bienenfarm Airport",
		City:      "Nauen",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 131,
		Latitude:  52.6616668701,
		Longitude: 12.7458333969,
		Timezone:  "Europe/Berlin",
	},
	"EDOJ": {
		ICAO:      "EDOJ",
		Name:      "Lusse Airport",
		City:      "Lusse",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 217,
		Latitude:  52.1411094666,
		Longitude: 12.6647224426,
		Timezone:  "Europe/Berlin",
	},
	"EDOK": {
		ICAO:      "EDOK",
		Name:      "Rudolstadt-Groschwitz Airport",
		City:      "Rudolstadt",
		State:     "Thuringia",
		Country:   "DE",
		Elevation: 1535,
		Latitude:  50.7327766418,
		Longitude: 11.2361106873,
		Timezone:  "Europe/Berlin",
	},
	"EDOL": {
		ICAO:      "EDOL",
		IATA:      "FRF",
		Name:      "Oschersleben Airport",
		City:      "Oschersleben",
		State:     "Saxony-Anhalt",
		Country:   "DE",
		Elevation: 344,
		Latitude:  52.0380554199,
		Longitude: 11.2055559158,
		Timezone:  "Europe/Berlin",
	},
	"EDOM": {
		ICAO:      "EDOM",
		Name:      "Klein Muhlingen Airport",
		City:      "Calbe",
		State:     "Saxony-Anhalt",
		Country:   "DE",
		Elevation: 171,
		Latitude:  51.9474983215,
		Longitude: 11.7697219849,
		Timezone:  "Europe/Berlin",
	},
	"EDON": {
		ICAO:      "EDON",
		Name:      "Neuhardenberg Airport",
		City:      "Neuhardenberg",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 39,
		Latitude:  52.6130561829,
		Longitude: 14.2427778244,
		Timezone:  "Europe/Berlin",
	},
	"EDOP": {
		ICAO:      "EDOP",
		IATA:      "SZW",
		Name:      "Schwerin Parchim Airport",
		State:     "Mecklenburg-Vorpommern",
		Country:   "DE",
		Elevation: 166,
		Latitude:  53.4269981384,
		Longitude: 11.7833995819,
		Timezone:  "Europe/Berlin",
	},
	"EDOQ": {
		ICAO:      "EDOQ",
		Name:      "Oschatz Airport",
		City:      "Oschatz",
		State:     "Saxony",
		Country:   "DE",
		Elevation: 502,
		Latitude:  51.2966651917,
		Longitude: 13.078332901,
		Timezone:  "Europe/Berlin",
	},
	"EDOR": {
		ICAO:      "EDOR",
		Name:      "Stolln-Rhinow Airport",
		City:      "Rhinow",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 138,
		Latitude:  52.7408332825,
		Longitude: 12.3900003433,
		Timezone:  "Europe/Berlin",
	},
	"EDOS": {
		ICAO:      "EDOS",
		Name:      "Pennewitz Airport",
		City:      "Pennewitz",
		State:     "Thuringia",
		Country:   "DE",
		Elevation: 1506,
		Latitude:  50.6694450378,
		Longitude: 11.0505561829,
		Timezone:  "Europe/Berlin",
	},
	"EDOT": {
		ICAO:      "EDOT",
		Name:      "Greiz-Obergrochlitz Airport",
		City:      "Greiz",
		State:     "Thuringia",
		Country:   "DE",
		Elevation: 1266,
		Latitude:  50.644443512,
		Longitude: 12.1763887405,
		Timezone:  "Europe/Berlin",
	},
	"EDOU": {
		ICAO:      "EDOU",
		Name:      "Weimar-Umpferstedt Airport",
		City:      "Weimar",
		State:     "Thuringia",
		Country:   "DE",
		Elevation: 984,
		Latitude:  50.9647216797,
		Longitude: 11.4002780914,
		Timezone:  "Europe/Berlin",
	},
	"EDOV": {
		ICAO:      "EDOV",
		Name:      "Stendal-Borstel Airport",
		City:      "Stendal",
		State:     "Saxony-Anhalt",
		Country:   "DE",
		Elevation: 184,
		Latitude:  52.6288871765,
		Longitude: 11.818611145,
		Timezone:  "Europe/Berlin",
	},
	"EDOW": {
		ICAO:      "EDOW",
		Name:      "Waren-Vielist Airport",
		City:      "Waren",
		State:     "Mecklenburg-Vorpommern",
		Country:   "DE",
		Elevation: 282,
		Latitude:  53.5683326721,
		Longitude: 12.6527776718,
		Timezone:  "Europe/Berlin",
	},
	"EDOX": {
		ICAO:      "EDOX",
		Name:      "Renneritz Airport",
		City:      "Sandersdorf",
		State:     "Saxony-Anhalt",
		Country:   "DE",
		Elevation: 308,
		Latitude:  51.594165802,
		Longitude: 12.2372217178,
		Timezone:  "Europe/Berlin",
	},
	"EDOZ": {
		ICAO:      "EDOZ",
		Name:      "Schonebeck-Zackmunde Airport",
		City:      "Schonebeck",
		State:     "Saxony-Anhalt",
		Country:   "DE",
		Elevation: 167,
		Latitude:  51.9966659546,
		Longitude: 11.7908334732,
		Timezone:  "Europe/Berlin",
	},
	"EDPA": {
		ICAO:      "EDPA",
		Name:      "Aalen-Heidenheim/Elchingen Airport",
		City:      "Aalen",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 1916,
		Latitude:  48.7777786255,
		Longitude: 10.2647218704,
		Timezone:  "Europe/Berlin",
	},
	"EDPB": {
		ICAO:      "EDPB",
		Name:      "Bad Ditzenbach Airport",
		City:      "Bad Ditzenbach",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 2362,
		Latitude:  48.5627784729,
		Longitude: 9.7288894653,
		Timezone:  "Europe/Berlin",
	},
	"EDPC": {
		ICAO:      "EDPC",
		Name:      "Bad Endorf Joll Airport",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1690,
		Latitude:  47.9268989563,
		Longitude: 12.2871999741,
		Timezone:  "Europe/Berlin",
	},
	"EDPD": {
		ICAO:      "EDPD",
		Name:      "Dingolfing Airport",
		City:      "Dingolfing",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1165,
		Latitude:  48.6569442749,
		Longitude: 12.5005559921,
		Timezone:  "Europe/Berlin",
	},
	"EDPE": {
		ICAO:      "EDPE",
		Name:      "Eichstatt Airport",
		City:      "Eichstatt",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1713,
		Latitude:  48.8769454956,
		Longitude: 11.1822223663,
		Timezone:  "Europe/Berlin",
	},
	"EDPF": {
		ICAO:      "EDPF",
		Name:      "Schwandorf Airport",
		City:      "Schwandorf",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1270,
		Latitude:  49.3397216797,
		Longitude: 12.1880559921,
		Timezone:  "Europe/Berlin",
	},
	"EDPG": {
		ICAO:      "EDPG",
		Name:      "Griesau Airport",
		City:      "Pfatter",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1060,
		Latitude:  48.9538879395,
		Longitude: 12.4213886261,
		Timezone:  "Europe/Berlin",
	},
	"EDPH": {
		ICAO:      "EDPH",
		Name:      "Schwabach-Heidenberg Airport",
		City:      "Schwabach",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1181,
		Latitude:  49.268611908,
		Longitude: 11.009721756,
		Timezone:  "Europe/Berlin",
	},
	"EDPJ": {
		ICAO:      "EDPJ",
		Name:      "Laichingen Airport",
		City:      "Laichingen",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 2434,
		Latitude:  48.4969444275,
		Longitude: 9.6402778625,
		Timezone:  "Europe/Berlin",
	},
	"EDPK": {
		ICAO:      "EDPK",
		Name:      "Schonberg Airport",
		City:      "Seebruck",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1946,
		Latitude:  48.0477790833,
		Longitude: 12.5005559921,
		Timezone:  "Europe/Berlin",
	},
	"EDPM": {
		ICAO:      "EDPM",
		Name:      "Donzdorf Airport",
		City:      "Donzdorf",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 2274,
		Latitude:  48.6780548096,
		Longitude: 9.8436107635,
		Timezone:  "Europe/Berlin",
	},
	"EDPO": {
		ICAO:      "EDPO",
		Name:      "Neumarkt/Obf. Airport",
		City:      "Neumarkt in der Oberpfalz",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1394,
		Latitude:  49.2855567932,
		Longitude: 11.4430561066,
		Timezone:  "Europe/Berlin",
	},
	"EDPQ": {
		ICAO:      "EDPQ",
		Name:      "Schmidgaden Airport",
		City:      "Schmidgaden",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1247,
		Latitude:  49.4294433594,
		Longitude: 12.0980558395,
		Timezone:  "Europe/Berlin",
	},
	"EDPS": {
		ICAO:      "EDPS",
		Name:      "Sonnen Airport",
		City:      "Waldkirchen",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 2674,
		Latitude:  48.6827774048,
		Longitude: 13.6947221756,
		Timezone:  "Europe/Berlin",
	},
	"EDPT": {
		ICAO:      "EDPT",
		Name:      "Gerstetten Airport",
		City:      "Gerstetten",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 1975,
		Latitude:  48.6202774048,
		Longitude: 10.0583333969,
		Timezone:  "Europe/Berlin",
	},
	"EDPU": {
		ICAO:      "EDPU",
		Name:      "Bartholoma-Amalienhof Airport",
		City:      "Bartholoma",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 984,
		Latitude:  48.7466659546,
		Longitude: 10.0050001144,
		Timezone:  "Europe/Berlin",
	},
	"EDPW": {
		ICAO:      "EDPW",
		Name:      "Thalmassing-Waizenhofen Airport",
		City:      "Thalmassing",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1893,
		Latitude:  49.0641670227,
		Longitude: 11.2091665268,
		Timezone:  "Europe/Berlin",
	},
	"EDPY": {
		ICAO:      "EDPY",
		Name:      "Ellwangen Airport",
		City:      "Ellwangen",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 1650,
		Latitude:  48.9611091614,
		Longitude: 10.2363891602,
		Timezone:  "Europe/Berlin",
	},
	"EDQB": {
		ICAO:      "EDQB",
		Name:      "Bad Windsheim Airport",
		City:      "Bad Windsheim",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1220,
		Latitude:  49.5099983215,
		Longitude: 10.3663892746,
		Timezone:  "Europe/Berlin",
	},
	"EDQC": {
		ICAO:      "EDQC",
		Name:      "Coburg-Brandensteinsebene Airport",
		City:      "Coburg",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1490,
		Latitude:  50.2625007629,
		Longitude: 10.9955558777,
		Timezone:  "Europe/Berlin",
	},
	"EDQD": {
		ICAO:      "EDQD",
		IATA:      "BYU",
		Name:      "Bayreuth Airport",
		City:      "Bayreuth",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1601,
		Latitude:  49.9850006104,
		Longitude: 11.6400003433,
		Timezone:  "Europe/Berlin",
	},
	"EDQE": {
		ICAO:      "EDQE",
		IATA:      "URD",
		Name:      "Burg Feuerstein Airport",
		City:      "Ebermannstadt",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1673,
		Latitude:  49.7941665649,
		Longitude: 11.1336107254,
		Timezone:  "Europe/Berlin",
	},
	"EDQF": {
		ICAO:      "EDQF",
		IATA:      "QOB",
		Name:      "Ansbach-Petersdorf Airport",
		City:      "Ansbach",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1371,
		Latitude:  49.3611106873,
		Longitude: 10.6694440842,
		Timezone:  "Europe/Berlin",
	},
	"EDQG": {
		ICAO:      "EDQG",
		Name:      "Giebelstadt Airport",
		City:      "Giebelstadt",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 981,
		Latitude:  49.6480560303,
		Longitude: 9.9663887024,
		Timezone:  "Europe/Berlin",
	},
	"EDQH": {
		ICAO:      "EDQH",
		Name:      "Herzogenaurach Airport",
		City:      "Herzogenaurach",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1070,
		Latitude:  49.5825004578,
		Longitude: 10.8780555725,
		Timezone:  "Europe/Berlin",
	},
	"EDQI": {
		ICAO:      "EDQI",
		Name:      "Lauf-Lillinghof Airport",
		City:      "Lauf an der Pegnitz",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1788,
		Latitude:  49.6052780151,
		Longitude: 11.2838888168,
		Timezone:  "Europe/Berlin",
	},
	"EDQK": {
		ICAO:      "EDQK",
		Name:      "Kulmbach Airport",
		City:      "Kulmbach",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1660,
		Latitude:  50.1349983215,
		Longitude: 11.4597215652,
		Timezone:  "Europe/Berlin",
	},
	"EDQL": {
		ICAO:      "EDQL",
		Name:      "Lichtenfels Airport",
		City:      "Lichtenfels",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 853,
		Latitude:  50.1486091614,
		Longitude: 11.0480556488,
		Timezone:  "Europe/Berlin",
	},
	"EDQM": {
		ICAO:      "EDQM",
		IATA:      "HOQ",
		Name:      "Hof-Plauen Airport",
		City:      "Hof",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1959,
		Latitude:  50.2886123657,
		Longitude: 11.8563890457,
		Timezone:  "Europe/Berlin",
	},
	"EDQN": {
		ICAO:      "EDQN",
		Name:      "Neustadt/Aisch Airport",
		City:      "Neustadt an der Aisch",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1198,
		Latitude:  49.5875015259,
		Longitude: 10.5775003433,
		Timezone:  "Europe/Berlin",
	},
	"EDQO": {
		ICAO:      "EDQO",
		Name:      "Ottengruner Heide Airport",
		City:      "Helmbrechts",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1880,
		Latitude:  50.2258338928,
		Longitude: 11.7316665649,
		Timezone:  "Europe/Berlin",
	},
	"EDQP": {
		ICAO:      "EDQP",
		Name:      "Rosenthal-Field Plossen Airport",
		City:      "Speichersdorf",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1496,
		Latitude:  49.8633346558,
		Longitude: 11.7877779007,
		Timezone:  "Europe/Berlin",
	},
	"EDQR": {
		ICAO:      "EDQR",
		Name:      "Ebern-Sendelbach Airport",
		City:      "Ebern",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 827,
		Latitude:  50.0394439697,
		Longitude: 10.8227777481,
		Timezone:  "Europe/Berlin",
	},
	"EDQS": {
		ICAO:      "EDQS",
		Name:      "Suhl-Goldlauter Airport",
		City:      "Goldlauter-Heidersbach",
		State:     "Thuringia",
		Country:   "DE",
		Elevation: 1923,
		Latitude:  50.631942749,
		Longitude: 10.7274999619,
		Timezone:  "Europe/Berlin",
	},
	"EDQT": {
		ICAO:      "EDQT",
		Name:      "Hasfurt-Schweinfurt Airport",
		City:      "Hasfurt",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 719,
		Latitude:  50.0180549622,
		Longitude: 10.5294437408,
		Timezone:  "Europe/Berlin",
	},
	"EDQW": {
		ICAO:      "EDQW",
		Name:      "Weiden in der Oberpfalz Airport",
		City:      "Weiden in der Oberpfalz",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1329,
		Latitude:  49.6789016724,
		Longitude: 12.116399765,
		Timezone:  "Europe/Berlin",
	},
	"EDQX": {
		ICAO:      "EDQX",
		Name:      "Hetzleser Berg Airport",
		City:      "Neunkirchen am Brand",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1765,
		Latitude:  49.6422233582,
		Longitude: 11.1622219086,
		Timezone:  "Europe/Berlin",
	},
	"EDQY": {
		ICAO:      "EDQY",
		Name:      "Coburg-Steinrucken Airport",
		City:      "Coburg",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1184,
		Latitude:  50.230556488,
		Longitude: 10.9958333969,
		Timezone:  "Europe/Berlin",
	},
	"EDQZ": {
		ICAO:      "EDQZ",
		Name:      "Pegnitz-Zipser Berg Airport",
		City:      "Pegnitz",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1791,
		Latitude:  49.76222229,
		Longitude: 11.57472229,
		Timezone:  "Europe/Berlin",
	},
	"EDRA": {
		ICAO:      "EDRA",
		Name:      "Bad Neuenahr-Ahrweiler Airport",
		City:      "Bad Neuenahr",
		State:     "Rheinland-Pfalz",
		Country:   "DE",
		Elevation: 673,
		Latitude:  50.5577774048,
		Longitude: 7.1363887787,
		Timezone:  "Europe/Berlin",
	},
	"EDRB": {
		ICAO:      "EDRB",
		IATA:      "BBJ",
		Name:      "Bitburg Airport",
		City:      "Bitburg",
		State:     "Rheinland-Pfalz",
		Country:   "DE",
		Elevation: 1220,
		Latitude:  49.9452781677,
		Longitude: 6.5650000572,
		Timezone:  "Europe/Berlin",
	},
	"EDRD": {
		ICAO:      "EDRD",
		Name:      "Neumagen-Dhron Airport",
		City:      "Neumagen",
		State:     "Rheinland-Pfalz",
		Country:   "DE",
		Elevation: 879,
		Latitude:  49.8430557251,
		Longitude: 6.9161109924,
		Timezone:  "Europe/Berlin",
	},
	"EDRF": {
		ICAO:      "EDRF",
		Name:      "Bad Durkheim Airport",
		City:      "Bad Durkheim",
		State:     "Rheinland-Pfalz",
		Country:   "DE",
		Elevation: 351,
		Latitude:  49.4730567932,
		Longitude: 8.1963891983,
		Timezone:  "Europe/Berlin",
	},
	"EDRG": {
		ICAO:      "EDRG",
		Name:      "Idar-Oberstein/Gottschied Airport",
		City:      "Idar-Oberstein",
		State:     "Rheinland-Pfalz",
		Country:   "DE",
		Elevation: 1575,
		Latitude:  49.7322235107,
		Longitude: 7.3361110687,
		Timezone:  "Europe/Berlin",
	},
	"EDRH": {
		ICAO:      "EDRH",
		Name:      "Hoppstadten-Weiersbach Airport",
		City:      "Hoppstadten",
		State:     "Rheinland-Pfalz",
		Country:   "DE",
		Elevation: 1093,
		Latitude:  49.6105575562,
		Longitude: 7.1866669655,
		Timezone:  "Europe/Berlin",
	},
	"EDRI": {
		ICAO:      "EDRI",
		Name:      "Linkenheim Airport",
		City:      "Linkenheim",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 325,
		Latitude:  49.1416664124,
		Longitude: 8.3947219849,
		Timezone:  "Europe/Berlin",
	},
	"EDRJ": {
		ICAO:      "EDRJ",
		Name:      "Saarlouis-Duren Airport",
		City:      "Saarlouis",
		State:     "Saarland",
		Country:   "DE",
		Elevation: 1119,
		Latitude:  49.3125,
		Longitude: 6.6744441986,
		Timezone:  "Europe/Berlin",
	},
	"EDRK": {
		ICAO:      "EDRK",
		Name:      "Koblenz-Winningen Airport",
		City:      "Koblenz",
		State:     "Rheinland-Pfalz",
		Country:   "DE",
		Elevation: 640,
		Latitude:  50.3255577087,
		Longitude: 7.5286111832,
		Timezone:  "Europe/Berlin",
	},
	"EDRL": {
		ICAO:      "EDRL",
		Name:      "Lachen-Speyerdorf Airport",
		City:      "Neustadt an der Weinstrase",
		State:     "Rheinland-Pfalz",
		Country:   "DE",
		Elevation: 394,
		Latitude:  49.3308334351,
		Longitude: 8.2097215652,
		Timezone:  "Europe/Berlin",
	},
	"EDRM": {
		ICAO:      "EDRM",
		Name:      "Traben-Trarbach/Mont Royal Airport",
		City:      "Traben-Trarbach",
		State:     "Rheinland-Pfalz",
		Country:   "DE",
		Elevation: 919,
		Latitude:  49.9677772522,
		Longitude: 7.1119441986,
		Timezone:  "Europe/Berlin",
	},
	"EDRN": {
		ICAO:      "EDRN",
		Name:      "Nannhausen Airport",
		City:      "Simmern",
		State:     "Rheinland-Pfalz",
		Country:   "DE",
		Elevation: 1224,
		Latitude:  49.9702796936,
		Longitude: 7.4791669846,
		Timezone:  "Europe/Berlin",
	},
	"EDRO": {
		ICAO:      "EDRO",
		Name:      "Schweighofen Airport",
		City:      "Schweighofen",
		State:     "Rheinland-Pfalz",
		Country:   "DE",
		Elevation: 492,
		Latitude:  49.031665802,
		Longitude: 7.9899997711,
		Timezone:  "Europe/Berlin",
	},
	"EDRP": {
		ICAO:      "EDRP",
		Name:      "Pirmasens Airport",
		City:      "Pirmasens",
		State:     "Rheinland-Pfalz",
		Country:   "DE",
		Elevation: 1247,
		Latitude:  49.2644424438,
		Longitude: 7.4883332253,
		Timezone:  "Europe/Berlin",
	},
	"EDRS": {
		ICAO:      "EDRS",
		Name:      "Bad Sobernheim-Domberg Airport",
		City:      "Bad Sobernheim",
		State:     "Rheinland-Pfalz",
		Country:   "DE",
		Elevation: 810,
		Latitude:  49.7908325195,
		Longitude: 7.6661109924,
		Timezone:  "Europe/Berlin",
	},
	"EDRT": {
		ICAO:      "EDRT",
		Name:      "Trier-Fohren Airport",
		City:      "Trier",
		State:     "Rheinland-Pfalz",
		Country:   "DE",
		Elevation: 666,
		Latitude:  49.8638877869,
		Longitude: 6.7874999046,
		Timezone:  "Europe/Berlin",
	},
	"EDRV": {
		ICAO:      "EDRV",
		Name:      "Wershofen/Eifel Airport",
		City:      "Aremberg",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 1581,
		Latitude:  50.4513893127,
		Longitude: 6.7833328247,
		Timezone:  "Europe/Berlin",
	},
	"EDRW": {
		ICAO:      "EDRW",
		Name:      "Dierdorf-Wienau Airport",
		City:      "Dierdorf",
		State:     "Rheinland-Pfalz",
		Country:   "DE",
		Elevation: 951,
		Latitude:  50.5661125183,
		Longitude: 7.6533331871,
		Timezone:  "Europe/Berlin",
	},
	"EDRY": {
		ICAO:      "EDRY",
		Name:      "Speyer Airport",
		City:      "Speyer",
		State:     "Rheinland-Pfalz",
		Country:   "DE",
		Elevation: 312,
		Latitude:  49.3047218323,
		Longitude: 8.4513893127,
		Timezone:  "Europe/Berlin",
	},
	"EDRZ": {
		ICAO:      "EDRZ",
		IATA:      "ZQW",
		Name:      "Zweibrucken Airport",
		City:      "Zweibrucken",
		State:     "Rheinland-Pfalz",
		Country:   "DE",
		Elevation: 1132,
		Latitude:  49.209400177,
		Longitude: 7.4005599022,
		Timezone:  "Europe/Berlin",
	},
	"EDSA": {
		ICAO:      "EDSA",
		Name:      "Albstadt-Degerfeld Airport",
		City:      "Albstadt",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 2881,
		Latitude:  48.2497215271,
		Longitude: 9.0625,
		Timezone:  "Europe/Berlin",
	},
	"EDSB": {
		ICAO:      "EDSB",
		IATA:      "FKB",
		Name:      "Karlsruhe Baden-Baden Airport",
		City:      "Baden-Baden",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 408,
		Latitude:  48.7793998718,
		Longitude: 8.0804996491,
		Timezone:  "Europe/Berlin",
	},
	"EDSD": {
		ICAO:      "EDSD",
		Name:      "Leipheim Air Base",
		City:      "Leipheim",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1604,
		Latitude:  48.4399986267,
		Longitude: 10.2356004715,
		Timezone:  "Europe/Berlin",
	},
	"EDSG": {
		ICAO:      "EDSG",
		Name:      "Grabenstetten Airport",
		City:      "Grabenstetten",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 2329,
		Latitude:  48.5361099243,
		Longitude: 9.4369440079,
		Timezone:  "Europe/Berlin",
	},
	"EDSH": {
		ICAO:      "EDSH",
		Name:      "Backnang-Heiningen Airport",
		City:      "Backnang",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 965,
		Latitude:  48.9197235107,
		Longitude: 9.4552783966,
		Timezone:  "Europe/Berlin",
	},
	"EDSI": {
		ICAO:      "EDSI",
		Name:      "Binningen Airport",
		City:      "Binningen",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 1594,
		Latitude:  47.7991676331,
		Longitude: 8.720000267,
		Timezone:  "Europe/Berlin",
	},
	"EDSK": {
		ICAO:      "EDSK",
		Name:      "Kehl-Sundheim Airport",
		City:      "Kehl",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 453,
		Latitude:  48.5611114502,
		Longitude: 7.8433327675,
		Timezone:  "Europe/Berlin",
	},
	"EDSL": {
		ICAO:      "EDSL",
		Name:      "Blumberg Airport",
		City:      "Blumberg",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 2290,
		Latitude:  47.844165802,
		Longitude: 8.5649995804,
		Timezone:  "Europe/Berlin",
	},
	"EDSN": {
		ICAO:      "EDSN",
		Name:      "Neuhausen ob Eck Airport",
		City:      "Neuhausen ob Eck",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 2638,
		Latitude:  47.9766654968,
		Longitude: 8.9038887024,
		Timezone:  "Europe/Berlin",
	},
	"EDSW": {
		ICAO:      "EDSW",
		Name:      "Altdorf-Wallburg Airport",
		City:      "Altdorf",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 623,
		Latitude:  48.2700004578,
		Longitude: 7.8419442177,
		Timezone:  "Europe/Berlin",
	},
	"EDSZ": {
		ICAO:      "EDSZ",
		Name:      "Rottweil-Zepfenhahn Airport",
		City:      "Rottweil",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 2444,
		Latitude:  48.1863899231,
		Longitude: 8.7211112976,
		Timezone:  "Europe/Berlin",
	},
	"EDTB": {
		ICAO:      "EDTB",
		Name:      "Baden-Oos Airport",
		City:      "Baden-Baden",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 404,
		Latitude:  48.7919425964,
		Longitude: 8.1858329773,
		Timezone:  "Europe/Berlin",
	},
	"EDTC": {
		ICAO:      "EDTC",
		Name:      "Bruchsal Airport",
		City:      "Bruchsal",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 364,
		Latitude:  49.1349983215,
		Longitude: 8.5636110306,
		Timezone:  "Europe/Berlin",
	},
	"EDTD": {
		ICAO:      "EDTD",
		IATA:      "ZQL",
		Name:      "Donaueschingen-Villingen Airport",
		City:      "Donaueschingen",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 2231,
		Latitude:  47.9733314514,
		Longitude: 8.5222215653,
		Timezone:  "Europe/Berlin",
	},
	"EDTF": {
		ICAO:      "EDTF",
		Name:      "Freiburg i. Br. Airport",
		City:      "Freiburg im Breisgau",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 801,
		Latitude:  48.0227775574,
		Longitude: 7.8324999809,
		Timezone:  "Europe/Berlin",
	},
	"EDTG": {
		ICAO:      "EDTG",
		Name:      "Bremgarten Airport",
		City:      "Bremgarten",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 696,
		Latitude:  47.9027786255,
		Longitude: 7.6177778244,
		Timezone:  "Europe/Berlin",
	},
	"EDTH": {
		ICAO:      "EDTH",
		Name:      "Heubach Airport",
		City:      "Heubach",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 1424,
		Latitude:  48.8027763367,
		Longitude: 9.9274997711,
		Timezone:  "Europe/Berlin",
	},
	"EDTK": {
		ICAO:      "EDTK",
		Name:      "Karlsruhe-Forchheim Airport",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 381,
		Latitude:  48.9759368896,
		Longitude: 8.3423995972,
		Timezone:  "Europe/Berlin",
	},
	"EDTL": {
		ICAO:      "EDTL",
		IATA:      "LHA",
		Name:      "Lahr Airport",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 511,
		Latitude:  48.3693008423,
		Longitude: 7.8277201653,
		Timezone:  "Europe/Berlin",
	},
	"EDTM": {
		ICAO:      "EDTM",
		Name:      "Mengen-Hohentengen Airport",
		City:      "Mengen",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 1818,
		Latitude:  48.0538902283,
		Longitude: 9.3727779388,
		Timezone:  "Europe/Berlin",
	},
	"EDTN": {
		ICAO:      "EDTN",
		Name:      "Nabern/Teck Airport",
		City:      "Nabern",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 1214,
		Latitude:  48.61277771,
		Longitude: 9.4772224426,
		Timezone:  "Europe/Berlin",
	},
	"EDTO": {
		ICAO:      "EDTO",
		Name:      "Offenburg Airport",
		City:      "Offenburg",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 509,
		Latitude:  48.4500007629,
		Longitude: 7.9247221947,
		Timezone:  "Europe/Berlin",
	},
	"EDTP": {
		ICAO:      "EDTP",
		Name:      "Pfullendorf Airport",
		City:      "Pfullendorf",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 2303,
		Latitude:  47.9088897705,
		Longitude: 9.2505559921,
		Timezone:  "Europe/Berlin",
	},
	"EDTQ": {
		ICAO:      "EDTQ",
		Name:      "Pattonville Airport",
		City:      "Pattonville",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 919,
		Latitude:  48.8641662598,
		Longitude: 9.2247219086,
		Timezone:  "Europe/Berlin",
	},
	"EDTR": {
		ICAO:      "EDTR",
		Name:      "Herten-Rheinfelden Airport",
		City:      "Rheinfelden",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 925,
		Latitude:  47.5602798462,
		Longitude: 7.7483329773,
		Timezone:  "Europe/Berlin",
	},
	"EDTS": {
		ICAO:      "EDTS",
		Name:      "Schwenningen am Neckar Airport",
		City:      "Villingen-Schwenningen",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 2169,
		Latitude:  48.0658340454,
		Longitude: 8.5713891983,
		Timezone:  "Europe/Berlin",
	},
	"EDTU": {
		ICAO:      "EDTU",
		Name:      "Saulgau Airport",
		City:      "Bad Saulgau",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 1903,
		Latitude:  48.0294456482,
		Longitude: 9.5072221756,
		Timezone:  "Europe/Berlin",
	},
	"EDTW": {
		ICAO:      "EDTW",
		Name:      "Winzeln-Schramberg Airport",
		City:      "Schramberg",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 2310,
		Latitude:  48.2791671753,
		Longitude: 8.4283332825,
		Timezone:  "Europe/Berlin",
	},
	"EDTX": {
		ICAO:      "EDTX",
		Name:      "Schwabisch Hall-Weckrieden Airport",
		City:      "Schwabisch Hall",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 1299,
		Latitude:  49.1244430542,
		Longitude: 9.7811107635,
		Timezone:  "Europe/Berlin",
	},
	"EDTY": {
		ICAO:      "EDTY",
		Name:      "Adolf Wurth Airport",
		City:      "Schwabisch Hall",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 1299,
		Latitude:  49.1183319092,
		Longitude: 9.7838888168,
		Timezone:  "Europe/Berlin",
	},
	"EDTZ": {
		ICAO:      "EDTZ",
		Name:      "Konstanz Airport",
		City:      "Konstanz",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 1302,
		Latitude:  47.6819458008,
		Longitude: 9.13722229,
		Timezone:  "Europe/Berlin",
	},
	"EDUA": {
		ICAO:      "EDUA",
		Name:      "Stechow-Ferchesar Airport",
		City:      "Stechow-Ferchesar",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 148,
		Latitude:  52.650554657,
		Longitude: 12.48777771,
		Timezone:  "Europe/Berlin",
	},
	"EDUB": {
		ICAO:      "EDUB",
		Name:      "Brandenburg-Briest Airport",
		City:      "Brandenburg an der Havel",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 102,
		Latitude:  52.4386100769,
		Longitude: 12.4583330154,
		Timezone:  "Europe/Berlin",
	},
	"EDUC": {
		ICAO:      "EDUC",
		Name:      "Briesen Brand Airport",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 230,
		Latitude:  52.0327987671,
		Longitude: 13.7491998672,
		Timezone:  "Europe/Berlin",
	},
	"EDUF": {
		ICAO:      "EDUF",
		Name:      "Falkenberg-Lonnewitz Airport",
		City:      "Uebigau-Wahrenbruck",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 312,
		Latitude:  51.5477790833,
		Longitude: 13.228055954,
		Timezone:  "Europe/Berlin",
	},
	"EDUO": {
		ICAO:      "EDUO",
		Name:      "Oberrissdorf Airport",
		City:      "Lutherstadt Eisleben",
		State:     "Saxony-Anhalt",
		Country:   "DE",
		Elevation: 738,
		Latitude:  51.5424995422,
		Longitude: 11.5958328247,
		Timezone:  "Europe/Berlin",
	},
	"EDUS": {
		ICAO:      "EDUS",
		Name:      "Finsterwalde/Schacksdorf Airport",
		City:      "Finsterwalde",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 384,
		Latitude:  51.6074981689,
		Longitude: 13.7436113358,
		Timezone:  "Europe/Berlin",
	},
	"EDUW": {
		ICAO:      "EDUW",
		Name:      "Tutow Airport",
		City:      "Demmin",
		State:     "Mecklenburg-Vorpommern",
		Country:   "DE",
		Elevation: 26,
		Latitude:  53.9219436646,
		Longitude: 13.2188892365,
		Timezone:  "Europe/Berlin",
	},
	"EDUY": {
		ICAO:      "EDUY",
		Name:      "Welzow-Sedlitzer See Airport",
		City:      "Welzow",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 361,
		Latitude:  51.5577774048,
		Longitude: 14.1049995422,
		Timezone:  "Europe/Berlin",
	},
	"EDUZ": {
		ICAO:      "EDUZ",
		Name:      "Zerbst Airport",
		City:      "Zerbst/Anhalt",
		State:     "Saxony-Anhalt",
		Country:   "DE",
		Elevation: 289,
		Latitude:  52.000831604,
		Longitude: 12.1486110687,
		Timezone:  "Europe/Berlin",
	},
	"EDVA": {
		ICAO:      "EDVA",
		Name:      "Bad Gandersheim Airport",
		City:      "Bad Gandersheim",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 791,
		Latitude:  51.8541679382,
		Longitude: 10.0255556107,
		Timezone:  "Europe/Berlin",
	},
	"EDVC": {
		ICAO:      "EDVC",
		Name:      "Celle-Arloh Airport",
		City:      "Celle",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 207,
		Latitude:  52.6872215271,
		Longitude: 10.1113891602,
		Timezone:  "Europe/Berlin",
	},
	"EDVE": {
		ICAO:      "EDVE",
		IATA:      "BWE",
		Name:      "Braunschweig Wolfsburg Airport",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 295,
		Latitude:  52.3191986084,
		Longitude: 10.5560998917,
		Timezone:  "Europe/Berlin",
	},
	"EDVF": {
		ICAO:      "EDVF",
		Name:      "Blomberg-Borkhausen Airport",
		City:      "Blomberg",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 535,
		Latitude:  51.9172210693,
		Longitude: 9.1116666794,
		Timezone:  "Europe/Berlin",
	},
	"EDVG": {
		ICAO:      "EDVG",
		Name:      "Mengeringhausen Airport",
		City:      "Mengeringhausen",
		State:     "Hesse",
		Country:   "DE",
		Elevation: 1191,
		Latitude:  51.3763885498,
		Longitude: 8.9811105728,
		Timezone:  "Europe/Berlin",
	},
	"EDVH": {
		ICAO:      "EDVH",
		Name:      "Hodenhagen Airport",
		City:      "Hodenhagen",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 79,
		Latitude:  52.7611122131,
		Longitude: 9.6055555344,
		Timezone:  "Europe/Berlin",
	},
	"EDVI": {
		ICAO:      "EDVI",
		Name:      "Hoxter-Holzminden Airport",
		City:      "Hoxter",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 1263,
		Latitude:  51.8066673279,
		Longitude: 9.3783330917,
		Timezone:  "Europe/Berlin",
	},
	"EDVJ": {
		ICAO:      "EDVJ",
		Name:      "Salzgitter-Schaferstuhl Airport",
		City:      "Salzgitter",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 741,
		Latitude:  52.0302772522,
		Longitude: 10.364443779,
		Timezone:  "Europe/Berlin",
	},
	"EDVK": {
		ICAO:      "EDVK",
		IATA:      "KSF",
		Name:      "Kassel-Calden Airport",
		City:      "Kassel",
		State:     "Hesse",
		Country:   "DE",
		Elevation: 909,
		Latitude:  51.4083328247,
		Longitude: 9.3774995804,
		Timezone:  "Europe/Berlin",
	},
	"EDVL": {
		ICAO:      "EDVL",
		Name:      "Holleberg Airport",
		City:      "Trendelburg",
		State:     "Hesse",
		Country:   "DE",
		Elevation: 837,
		Latitude:  51.6108322144,
		Longitude: 9.3975000381,
		Timezone:  "Europe/Berlin",
	},
	"EDVM": {
		ICAO:      "EDVM",
		Name:      "Hildesheim Airport",
		City:      "Hildesheim",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 292,
		Latitude:  52.181388855,
		Longitude: 9.9463891983,
		Timezone:  "Europe/Berlin",
	},
	"EDVN": {
		ICAO:      "EDVN",
		Name:      "Northeim Airport",
		City:      "Northeim",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 404,
		Latitude:  51.7063903809,
		Longitude: 10.0397224426,
		Timezone:  "Europe/Berlin",
	},
	"EDVR": {
		ICAO:      "EDVR",
		Name:      "Rinteln Airport",
		City:      "Rinteln",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 180,
		Latitude:  52.17527771,
		Longitude: 9.0533332825,
		Timezone:  "Europe/Berlin",
	},
	"EDVS": {
		ICAO:      "EDVS",
		Name:      "Salzgitter-Drutte Airport",
		City:      "Salzgitter",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 328,
		Latitude:  52.1544456482,
		Longitude: 10.4266672134,
		Timezone:  "Europe/Berlin",
	},
	"EDVU": {
		ICAO:      "EDVU",
		Name:      "Uelzen Airport",
		City:      "Uelzen",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 246,
		Latitude:  52.9838905334,
		Longitude: 10.4650001526,
		Timezone:  "Europe/Berlin",
	},
	"EDVW": {
		ICAO:      "EDVW",
		Name:      "Hameln-Pyrmont Airport",
		City:      "Bad Pyrmont",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 1178,
		Latitude:  51.9666671753,
		Longitude: 9.2916669846,
		Timezone:  "Europe/Berlin",
	},
	"EDVY": {
		ICAO:      "EDVY",
		Name:      "Porta Westfalica Airport",
		City:      "Bad Oeynhausen",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 148,
		Latitude:  52.2208328247,
		Longitude: 8.859167099,
		Timezone:  "Europe/Berlin",
	},
	"EDWA": {
		ICAO:      "EDWA",
		Name:      "Bordelum Airport",
		City:      "Bredstedt",
		State:     "Schleswig-Holstein",
		Country:   "DE",
		Elevation: 3,
		Latitude:  54.6274986267,
		Longitude: 8.9302778244,
		Timezone:  "Europe/Berlin",
	},
	"EDWB": {
		ICAO:      "EDWB",
		IATA:      "BRV",
		Name:      "Bremerhaven Airport",
		City:      "Bremerhaven",
		State:     "Bremen",
		Country:   "DE",
		Elevation: 10,
		Latitude:  53.506942749,
		Longitude: 8.5727777481,
		Timezone:  "Europe/Berlin",
	},
	"EDWC": {
		ICAO:      "EDWC",
		Name:      "Damme Airport",
		City:      "Damme",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 151,
		Latitude:  52.4874992371,
		Longitude: 8.1855564117,
		Timezone:  "Europe/Berlin",
	},
	"EDWD": {
		ICAO:      "EDWD",
		IATA:      "XLW",
		Name:      "Lemwerder Airport",
		City:      "Lemwerder",
		State:     "Bremen",
		Country:   "DE",
		Elevation: 20,
		Latitude:  53.1446990967,
		Longitude: 8.6244401932,
		Timezone:  "Europe/Berlin",
	},
	"EDWE": {
		ICAO:      "EDWE",
		IATA:      "EME",
		Name:      "Emden Airport",
		City:      "Emden",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 3,
		Latitude:  53.3911094666,
		Longitude: 7.2274999619,
		Timezone:  "Europe/Berlin",
	},
	"EDWF": {
		ICAO:      "EDWF",
		Name:      "Leer-Papenburg Airport",
		City:      "Leer",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 3,
		Latitude:  53.2719421387,
		Longitude: 7.4416670799,
		Timezone:  "Europe/Berlin",
	},
	"EDWG": {
		ICAO:      "EDWG",
		IATA:      "AGE",
		Name:      "Wangerooge Airport",
		City:      "Wangerooge",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 7,
		Latitude:  53.7827796936,
		Longitude: 7.9138889313,
		Timezone:  "Europe/Berlin",
	},
	"EDWH": {
		ICAO:      "EDWH",
		Name:      "Oldenburg-Hatten Airport",
		City:      "Oldenburg",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 26,
		Latitude:  53.0688896179,
		Longitude: 8.3136110306,
		Timezone:  "Europe/Berlin",
	},
	"EDWI": {
		ICAO:      "EDWI",
		IATA:      "WVN",
		Name:      "Wilhelmshaven-Mariensiel Airport",
		City:      "Wilhelmshaven",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 16,
		Latitude:  53.5022201538,
		Longitude: 8.0522222519,
		Timezone:  "Europe/Berlin",
	},
	"EDWJ": {
		ICAO:      "EDWJ",
		IATA:      "JUI",
		Name:      "Juist Airport",
		City:      "Juist",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 7,
		Latitude:  53.6811103821,
		Longitude: 7.0558328629,
		Timezone:  "Europe/Berlin",
	},
	"EDWK": {
		ICAO:      "EDWK",
		Name:      "Karlshofen Airport",
		City:      "Karlshofen",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 20,
		Latitude:  53.3327789307,
		Longitude: 9.0283327103,
		Timezone:  "Europe/Berlin",
	},
	"EDWL": {
		ICAO:      "EDWL",
		IATA:      "LGO",
		Name:      "Langeoog Airport",
		City:      "Langeoog",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 7,
		Latitude:  53.7425003052,
		Longitude: 7.4977779388,
		Timezone:  "Europe/Berlin",
	},
	"EDWM": {
		ICAO:      "EDWM",
		Name:      "Weser-Wumme Airport",
		City:      "Rotenburg (Wumme)",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 59,
		Latitude:  53.0538902283,
		Longitude: 9.2086105347,
		Timezone:  "Europe/Berlin",
	},
	"EDWN": {
		ICAO:      "EDWN",
		IATA:      "ZOW",
		Name:      "Nordhorn-Lingen Airport",
		City:      "Klausheide",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 89,
		Latitude:  52.4575004578,
		Longitude: 7.1822218895,
		Timezone:  "Europe/Berlin",
	},
	"EDWO": {
		ICAO:      "EDWO",
		Name:      "Osnabruck-Atterheide Airport",
		City:      "Osnabruck",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 285,
		Latitude:  52.2863883972,
		Longitude: 7.9697217941,
		Timezone:  "Europe/Berlin",
	},
	"EDWP": {
		ICAO:      "EDWP",
		Name:      "Wiefelstede/Conneforde Airport",
		City:      "Wiefelstede",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 33,
		Latitude:  53.3213882446,
		Longitude: 8.0733327866,
		Timezone:  "Europe/Berlin",
	},
	"EDWQ": {
		ICAO:      "EDWQ",
		Name:      "Ganderkesee Atlas Airfield Airport",
		City:      "Ganderkesee",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 95,
		Latitude:  53.0361099243,
		Longitude: 8.5055561066,
		Timezone:  "Europe/Berlin",
	},
	"EDWR": {
		ICAO:      "EDWR",
		IATA:      "BMK",
		Name:      "Borkum Airport",
		City:      "Borkum",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 3,
		Latitude:  53.5963897705,
		Longitude: 6.7091670036,
		Timezone:  "Europe/Berlin",
	},
	"EDWS": {
		ICAO:      "EDWS",
		IATA:      "NOD",
		Name:      "Norden-Norddeich Airport",
		City:      "Norddeich",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 3,
		Latitude:  53.6330566406,
		Longitude: 7.1902780533,
		Timezone:  "Europe/Berlin",
	},
	"EDWU": {
		ICAO:      "EDWU",
		IATA:      "VAC",
		Name:      "Varrelbusch Airport",
		City:      "Cloppenburg",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 128,
		Latitude:  52.9083328247,
		Longitude: 8.040555954,
		Timezone:  "Europe/Berlin",
	},
	"EDWV": {
		ICAO:      "EDWV",
		Name:      "Verden-Scharnhorst Airport",
		City:      "Verden (Aller)",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 144,
		Latitude:  52.9652786255,
		Longitude: 9.2827777863,
		Timezone:  "Europe/Berlin",
	},
	"EDWX": {
		ICAO:      "EDWX",
		Name:      "Westerstede-Felde Airport",
		City:      "Westerstede",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 30,
		Latitude:  53.2886123657,
		Longitude: 7.9305558205,
		Timezone:  "Europe/Berlin",
	},
	"EDWY": {
		ICAO:      "EDWY",
		IATA:      "NRD",
		Name:      "Norderney Airport",
		City:      "Norderney",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 7,
		Latitude:  53.706943512,
		Longitude: 7.2300000191,
		Timezone:  "Europe/Berlin",
	},
	"EDWZ": {
		ICAO:      "EDWZ",
		IATA:      "BMR",
		Name:      "Baltrum Airport",
		City:      "Baltrum",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 7,
		Latitude:  53.7247238159,
		Longitude: 7.3733329773,
		Timezone:  "Europe/Berlin",
	},
	"EDXA": {
		ICAO:      "EDXA",
		Name:      "Achmer Airport",
		City:      "Bramsche",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 177,
		Latitude:  52.3772201538,
		Longitude: 7.9133329391,
		Timezone:  "Europe/Berlin",
	},
	"EDXB": {
		ICAO:      "EDXB",
		IATA:      "HEI",
		Name:      "Heide-Busum Airport",
		City:      "Busum",
		State:     "Schleswig-Holstein",
		Country:   "DE",
		Elevation: 7,
		Latitude:  54.1533317566,
		Longitude: 8.9016666412,
		Timezone:  "Europe/Berlin",
	},
	"EDXC": {
		ICAO:      "EDXC",
		Name:      "Schleswig-Kropp Airport",
		City:      "Schleswig",
		State:     "Schleswig-Holstein",
		Country:   "DE",
		Elevation: 52,
		Latitude:  54.42527771,
		Longitude: 9.5416669846,
		Timezone:  "Europe/Berlin",
	},
	"EDXD": {
		ICAO:      "EDXD",
		Name:      "Bohmte-Bad Essen Airport",
		City:      "Bohmte",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 148,
		Latitude:  52.3513908386,
		Longitude: 8.328332901,
		Timezone:  "Europe/Berlin",
	},
	"EDXE": {
		ICAO:      "EDXE",
		Name:      "Rheine-Eschendorf Airport",
		City:      "Rheine",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 131,
		Latitude:  52.276389,
		Longitude: 7.492778,
		Timezone:  "Europe/Berlin",
	},
	"EDXF": {
		ICAO:      "EDXF",
		IATA:      "FLF",
		Name:      "Flensburg-Schaeferhaus Airport",
		City:      "Flensburg",
		State:     "Schleswig-Holstein",
		Country:   "DE",
		Elevation: 131,
		Latitude:  54.7733345032,
		Longitude: 9.3788890839,
		Timezone:  "Europe/Berlin",
	},
	"EDXG": {
		ICAO:      "EDXG",
		Name:      "Melle-Gronegau Airport",
		City:      "Melle",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 236,
		Latitude:  52.2008323669,
		Longitude: 8.3805561066,
		Timezone:  "Europe/Berlin",
	},
	"EDXH": {
		ICAO:      "EDXH",
		IATA:      "HGL",
		Name:      "Helgoland-Dune Airport",
		City:      "Helgoland",
		State:     "Schleswig-Holstein",
		Country:   "DE",
		Elevation: 8,
		Latitude:  54.1852798462,
		Longitude: 7.9158329964,
		Timezone:  "Europe/Berlin",
	},
	"EDXI": {
		ICAO:      "EDXI",
		Name:      "Nienburg-Holzbalge Airport",
		City:      "Nienburg/Weser",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 82,
		Latitude:  52.7097206116,
		Longitude: 9.1625003815,
		Timezone:  "Europe/Berlin",
	},
	"EDXJ": {
		ICAO:      "EDXJ",
		IATA:      "QHU",
		Name:      "Husum-Schwesing Airport",
		City:      "Husum",
		State:     "Schleswig-Holstein",
		Country:   "DE",
		Elevation: 62,
		Latitude:  54.5099983215,
		Longitude: 9.1383333206,
		Timezone:  "Europe/Berlin",
	},
	"EDXK": {
		ICAO:      "EDXK",
		Name:      "Leck Airport",
		City:      "Leck",
		State:     "Schleswig-Holstein",
		Country:   "DE",
		Elevation: 23,
		Latitude:  54.7900009155,
		Longitude: 8.961388588,
		Timezone:  "Europe/Berlin",
	},
	"EDXL": {
		ICAO:      "EDXL",
		Name:      "Barsel Airport",
		City:      "Barsel",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 10,
		Latitude:  53.1644439697,
		Longitude: 7.7938890457,
		Timezone:  "Europe/Berlin",
	},
	"EDXM": {
		ICAO:      "EDXM",
		Name:      "St. Michaelisdonn Airport",
		City:      "Sankt Michaelisdonn",
		State:     "Schleswig-Holstein",
		Country:   "DE",
		Elevation: 125,
		Latitude:  53.9780578613,
		Longitude: 9.1447219849,
		Timezone:  "Europe/Berlin",
	},
	"EDXN": {
		ICAO:      "EDXN",
		Name:      "Nordholz-Spieka Airport",
		City:      "Cuxhaven",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 72,
		Latitude:  53.7672233582,
		Longitude: 8.6436109543,
		Timezone:  "Europe/Berlin",
	},
	"EDXO": {
		ICAO:      "EDXO",
		IATA:      "PSH",
		Name:      "St. Peter-Ording Airport",
		City:      "Sankt Peter-Ording",
		State:     "Schleswig-Holstein",
		Country:   "DE",
		Elevation: 7,
		Latitude:  54.3088874817,
		Longitude: 8.6869440079,
		Timezone:  "Europe/Berlin",
	},
	"EDXP": {
		ICAO:      "EDXP",
		Name:      "Harle Airport",
		City:      "Harlesiel",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 7,
		Latitude:  53.7066688538,
		Longitude: 7.8202781677,
		Timezone:  "Europe/Berlin",
	},
	"EDXQ": {
		ICAO:      "EDXQ",
		Name:      "Rotenburg (Wumme) Airport",
		City:      "Rotenburg (Wumme)",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 98,
		Latitude:  53.1283340454,
		Longitude: 9.348610878,
		Timezone:  "Europe/Berlin",
	},
	"EDXR": {
		ICAO:      "EDXR",
		Name:      "Rendsburg-Schachtholm Airport",
		City:      "Rendsburg",
		State:     "Schleswig-Holstein",
		Country:   "DE",
		Elevation: 23,
		Latitude:  54.2200012207,
		Longitude: 9.5994443893,
		Timezone:  "Europe/Berlin",
	},
	"EDXS": {
		ICAO:      "EDXS",
		Name:      "Seedorf Airport",
		City:      "Seedorf",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 72,
		Latitude:  53.3355560303,
		Longitude: 9.2594442368,
		Timezone:  "Europe/Berlin",
	},
	"EDXT": {
		ICAO:      "EDXT",
		Name:      "Sierksdorf/Hof Altona Airport",
		City:      "Sierksdorf",
		State:     "Schleswig-Holstein",
		Country:   "DE",
		Elevation: 79,
		Latitude:  54.067779541,
		Longitude: 10.7430562973,
		Timezone:  "Europe/Berlin",
	},
	"EDXU": {
		ICAO:      "EDXU",
		Name:      "Huttenbusch Airport",
		City:      "Huttenbusch",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 10,
		Latitude:  53.2866668701,
		Longitude: 8.947221756,
		Timezone:  "Europe/Berlin",
	},
	"EDXW": {
		ICAO:      "EDXW",
		IATA:      "GWT",
		Name:      "Westerland Sylt Airport",
		City:      "Westerland",
		State:     "Schleswig-Holstein",
		Country:   "DE",
		Elevation: 51,
		Latitude:  54.9132003784,
		Longitude: 8.340470314,
		Timezone:  "Europe/Berlin",
	},
	"EDXY": {
		ICAO:      "EDXY",
		IATA:      "OHR",
		Name:      "Wyk auf Fohr Airport",
		City:      "Wyk auf Fohr",
		State:     "Schleswig-Holstein",
		Country:   "DE",
		Elevation: 26,
		Latitude:  54.6844444275,
		Longitude: 8.5283327103,
		Timezone:  "Europe/Berlin",
	},
	"EDXZ": {
		ICAO:      "EDXZ",
		Name:      "Kuhrstedt-Bederkesa Airport",
		City:      "Kuhrstedt",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 26,
		Latitude:  53.5680541992,
		Longitude: 8.7894439697,
		Timezone:  "Europe/Berlin",
	},
	"EEEI": {
		ICAO:      "EEEI",
		Name:      "Amari Air Base",
		City:      "Keila",
		State:     "Harjumaa",
		Country:   "EE",
		Elevation: 65,
		Latitude:  59.2602996826,
		Longitude: 24.2084999084,
		Timezone:  "Europe/Tallinn",
	},
	"EEJI": {
		ICAO:      "EEJI",
		Name:      "Johvi Airfield",
		City:      "Johvi",
		State:     "Ida-Virumaa",
		Country:   "EE",
		Elevation: 246,
		Latitude:  59.3330001831,
		Longitude: 27.3829994202,
		Timezone:  "Europe/Tallinn",
	},
	"EEKA": {
		ICAO:      "EEKA",
		IATA:      "KDL",
		Name:      "Kardla Airport",
		City:      "Kardla",
		State:     "Hiiumaa",
		Country:   "EE",
		Elevation: 18,
		Latitude:  58.9907989502,
		Longitude: 22.8306999207,
		Timezone:  "Europe/Tallinn",
	},
	"EEKE": {
		ICAO:      "EEKE",
		IATA:      "URE",
		Name:      "Kuressaare Airport",
		City:      "Kuressaare",
		State:     "Saare",
		Country:   "EE",
		Elevation: 14,
		Latitude:  58.2299003601,
		Longitude: 22.5095005035,
		Timezone:  "Europe/Tallinn",
	},
	"EEKU": {
		ICAO:      "EEKU",
		Name:      "Kihnu Airfield",
		City:      "Saarekula",
		State:     "Paernumaa",
		Country:   "EE",
		Elevation: 10,
		Latitude:  58.1483001709,
		Longitude: 24.0025005341,
		Timezone:  "Europe/Tallinn",
	},
	"EENA": {
		ICAO:      "EENA",
		Name:      "Narva Estonia Airport",
		City:      "Narva",
		State:     "Ida-Virumaa",
		Country:   "EE",
		Elevation: 92,
		Latitude:  59.3885993958,
		Longitude: 28.1135997772,
		Timezone:  "Europe/Tallinn",
	},
	"EENI": {
		ICAO:      "EENI",
		Name:      "Nurmsi Airport",
		City:      "Nurmsi",
		State:     "Jaervamaa",
		Country:   "EE",
		Elevation: 233,
		Latitude:  58.8650016785,
		Longitude: 25.7299995422,
		Timezone:  "Europe/Tallinn",
	},
	"EEPU": {
		ICAO:      "EEPU",
		IATA:      "EPU",
		Name:      "Parnu Airport",
		City:      "Parnu",
		State:     "Paernumaa",
		Country:   "EE",
		Elevation: 47,
		Latitude:  58.4189987183,
		Longitude: 24.4727993011,
		Timezone:  "Europe/Tallinn",
	},
	"EERA": {
		ICAO:      "EERA",
		Name:      "Rapla Airfield",
		City:      "Rapla",
		State:     "Raplamaa",
		Country:   "EE",
		Elevation: 220,
		Latitude:  58.9864006042,
		Longitude: 24.7243995667,
		Timezone:  "Europe/Tallinn",
	},
	"EERI": {
		ICAO:      "EERI",
		Name:      "Ridali Airfield",
		City:      "Suurkula",
		State:     "Polvamaa",
		Country:   "EE",
		Elevation: 302,
		Latitude:  57.9399986267,
		Longitude: 26.9790000916,
		Timezone:  "Europe/Tallinn",
	},
	"EERU": {
		ICAO:      "EERU",
		Name:      "Ruhnu Airfield",
		City:      "Ringsu",
		State:     "Saare",
		Country:   "EE",
		Elevation: 10,
		Latitude:  57.7839012146,
		Longitude: 23.2660999298,
		Timezone:  "Europe/Tallinn",
	},
	"EETA": {
		ICAO:      "EETA",
		Name:      "Tapa Airfield",
		City:      "Tapa",
		State:     "Laeaene-Virumaa",
		Country:   "EE",
		Elevation: 331,
		Latitude:  59.2407989502,
		Longitude: 25.9622001648,
		Timezone:  "Europe/Tallinn",
	},
	"EETN": {
		ICAO:      "EETN",
		IATA:      "TLL",
		Name:      "Tallinn Airport",
		City:      "Tallinn",
		State:     "Harjumaa",
		Country:   "EE",
		Elevation: 131,
		Latitude:  59.4132995605,
		Longitude: 24.8327999115,
		Timezone:  "Europe/Tallinn",
	},
	"EETU": {
		ICAO:      "EETU",
		IATA:      "TAY",
		Name:      "Tartu Airport",
		City:      "Tartu",
		State:     "Tartu",
		Country:   "EE",
		Elevation: 219,
		Latitude:  58.3074989319,
		Longitude: 26.6903991699,
		Timezone:  "Europe/Tallinn",
	},
	"EEVI": {
		ICAO:      "EEVI",
		Name:      "Viljandi Airfield",
		City:      "Viljandi",
		State:     "Viljandimaa",
		Country:   "EE",
		Elevation: 249,
		Latitude:  58.3499984741,
		Longitude: 25.4930992126,
		Timezone:  "Europe/Tallinn",
	},
	"EFAA": {
		ICAO:      "EFAA",
		Name:      "Aavahelukka Airport",
		State:     "Lapland",
		Country:   "FI",
		Elevation: 738,
		Latitude:  67.6035995483,
		Longitude: 23.9717006683,
		Timezone:  "Europe/Helsinki",
	},
	"EFAH": {
		ICAO:      "EFAH",
		Name:      "Ahmosuo Airport",
		City:      "Oulu",
		State:     "Northern-Ostrobothnia",
		Country:   "FI",
		Elevation: 95,
		Latitude:  64.8953018188,
		Longitude: 25.752199173,
		Timezone:  "Europe/Helsinki",
	},
	"EFAL": {
		ICAO:      "EFAL",
		Name:      "Alavus Airport",
		City:      "Alavus",
		State:     "Southern-Ostrobothnia",
		Country:   "FI",
		Elevation: 407,
		Latitude:  62.5546989441,
		Longitude: 23.573299408,
		Timezone:  "Europe/Helsinki",
	},
	"EFET": {
		ICAO:      "EFET",
		IATA:      "ENF",
		Name:      "Enontekio Airport",
		City:      "Enontekio",
		State:     "Lapland",
		Country:   "FI",
		Elevation: 1005,
		Latitude:  68.3626022339,
		Longitude: 23.4242992401,
		Timezone:  "Europe/Helsinki",
	},
	"EFEU": {
		ICAO:      "EFEU",
		Name:      "Eura Airport",
		State:     "Satakunta",
		Country:   "FI",
		Elevation: 259,
		Latitude:  61.1161003113,
		Longitude: 22.2014007568,
		Timezone:  "Europe/Helsinki",
	},
	"EFFO": {
		ICAO:      "EFFO",
		IATA:      "QVE",
		Name:      "Forssa Airport",
		City:      "Forssa",
		State:     "Tavastia-Proper",
		Country:   "FI",
		Elevation: 325,
		Latitude:  60.8018989563,
		Longitude: 23.6455993652,
		Timezone:  "Europe/Helsinki",
	},
	"EFGE": {
		ICAO:      "EFGE",
		Name:      "Genbole Airport",
		City:      "Genbole",
		State:     "Finland-Proper",
		Country:   "FI",
		Elevation: 46,
		Latitude:  60.0868988037,
		Longitude: 22.521900177,
		Timezone:  "Europe/Helsinki",
	},
	"EFHA": {
		ICAO:      "EFHA",
		IATA:      "KEV",
		Name:      "Halli Airport",
		City:      "Halli / Kuorevesi",
		State:     "Central-Finland",
		Country:   "FI",
		Elevation: 479,
		Latitude:  61.8559989929,
		Longitude: 24.7866001129,
		Timezone:  "Europe/Helsinki",
	},
	"EFHF": {
		ICAO:      "EFHF",
		IATA:      "HEM",
		Name:      "Helsinki Malmi Airport",
		City:      "Helsinki",
		State:     "Uusimaa",
		Country:   "FI",
		Elevation: 57,
		Latitude:  60.2546005249,
		Longitude: 25.0428009033,
		Timezone:  "Europe/Helsinki",
	},
	"EFHI": {
		ICAO:      "EFHI",
		Name:      "Haapamaki Airport",
		State:     "Central-Finland",
		Country:   "FI",
		Elevation: 531,
		Latitude:  62.2578010559,
		Longitude: 24.3341999054,
		Timezone:  "Europe/Helsinki",
	},
	"EFHJ": {
		ICAO:      "EFHJ",
		Name:      "Haapajarvi Airport",
		City:      "Haapajarvi",
		State:     "Northern-Ostrobothnia",
		Country:   "FI",
		Elevation: 351,
		Latitude:  63.7122001648,
		Longitude: 25.3950004578,
		Timezone:  "Europe/Helsinki",
	},
	"EFHK": {
		ICAO:      "EFHK",
		IATA:      "HEL",
		Name:      "Helsinki Vantaa Airport",
		City:      "Helsinki",
		State:     "Uusimaa",
		Country:   "FI",
		Elevation: 179,
		Latitude:  60.317199707,
		Longitude: 24.963300705,
		Timezone:  "Europe/Helsinki",
	},
	"EFHL": {
		ICAO:      "EFHL",
		Name:      "Hailuoto Airport",
		State:     "Northern-Ostrobothnia",
		Country:   "FI",
		Elevation: 7,
		Latitude:  64.9669036865,
		Longitude: 24.7043991089,
		Timezone:  "Europe/Helsinki",
	},
	"EFHM": {
		ICAO:      "EFHM",
		Name:      "Hameenkyro Airport",
		State:     "Pirkanmaa",
		Country:   "FI",
		Elevation: 449,
		Latitude:  61.6897010803,
		Longitude: 23.0736999512,
		Timezone:  "Europe/Helsinki",
	},
	"EFHN": {
		ICAO:      "EFHN",
		Name:      "Hanko Airport",
		State:     "Uusimaa",
		Country:   "FI",
		Elevation: 20,
		Latitude:  59.8488998413,
		Longitude: 23.0835990906,
		Timezone:  "Europe/Helsinki",
	},
	"EFHP": {
		ICAO:      "EFHP",
		Name:      "Haapavesi Airport",
		City:      "Haapavesi",
		State:     "Northern-Ostrobothnia",
		Country:   "FI",
		Elevation: 315,
		Latitude:  64.1130981445,
		Longitude: 25.5041999817,
		Timezone:  "Europe/Helsinki",
	},
	"EFHV": {
		ICAO:      "EFHV",
		IATA:      "HYV",
		Name:      "Hyvinkaa Airport",
		State:     "Uusimaa",
		Country:   "FI",
		Elevation: 430,
		Latitude:  60.6543998718,
		Longitude: 24.8810997009,
		Timezone:  "Europe/Helsinki",
	},
	"EFII": {
		ICAO:      "EFII",
		Name:      "Iisalmi Airport",
		City:      "Iisalmi",
		State:     "Northern-Savo",
		Country:   "FI",
		Elevation: 384,
		Latitude:  63.6319007874,
		Longitude: 27.1222000122,
		Timezone:  "Europe/Helsinki",
	},
	"EFIK": {
		ICAO:      "EFIK",
		Name:      "Kiikala Airport",
		State:     "Finland-Proper",
		Country:   "FI",
		Elevation: 381,
		Latitude:  60.4625015259,
		Longitude: 23.6525001526,
		Timezone:  "Europe/Helsinki",
	},
	"EFIM": {
		ICAO:      "EFIM",
		Name:      "Immola Airport",
		State:     "South-Karelia",
		Country:   "FI",
		Elevation: 338,
		Latitude:  61.2491989136,
		Longitude: 28.9036998749,
		Timezone:  "Europe/Helsinki",
	},
	"EFIT": {
		ICAO:      "EFIT",
		IATA:      "KTQ",
		Name:      "Kitee Airport",
		State:     "North-Karelia",
		Country:   "FI",
		Elevation: 364,
		Latitude:  62.1660995483,
		Longitude: 30.073600769,
		Timezone:  "Europe/Helsinki",
	},
	"EFIV": {
		ICAO:      "EFIV",
		IATA:      "IVL",
		Name:      "Ivalo Airport",
		City:      "Ivalo",
		State:     "Lapland",
		Country:   "FI",
		Elevation: 481,
		Latitude:  68.6072998047,
		Longitude: 27.4053001404,
		Timezone:  "Europe/Helsinki",
	},
	"EFJM": {
		ICAO:      "EFJM",
		Name:      "Jamijarvi Airport",
		City:      "Jamijarvi",
		State:     "Satakunta",
		Country:   "FI",
		Elevation: 505,
		Latitude:  61.7785987854,
		Longitude: 22.7161006927,
		Timezone:  "Europe/Helsinki",
	},
	"EFJO": {
		ICAO:      "EFJO",
		IATA:      "JOE",
		Name:      "Joensuu Airport",
		City:      "Joensuu / Liperi",
		State:     "North-Karelia",
		Country:   "FI",
		Elevation: 398,
		Latitude:  62.6628990173,
		Longitude: 29.6075000763,
		Timezone:  "Europe/Helsinki",
	},
	"EFJP": {
		ICAO:      "EFJP",
		Name:      "Jakalapaa Airport",
		State:     "Lapland",
		Country:   "FI",
		Latitude:  68.7114028931,
		Longitude: 25.7527999878,
		Timezone:  "Europe/Helsinki",
	},
	"EFJY": {
		ICAO:      "EFJY",
		IATA:      "JYV",
		Name:      "Jyvaskyla Airport",
		City:      "Jyvaskylan Maalaiskunta",
		State:     "Central-Finland",
		Country:   "FI",
		Elevation: 459,
		Latitude:  62.3995018005,
		Longitude: 25.6783008575,
		Timezone:  "Europe/Helsinki",
	},
	"EFKA": {
		ICAO:      "EFKA",
		IATA:      "KAU",
		Name:      "Kauhava Airport",
		City:      "Kauhava",
		State:     "Southern-Ostrobothnia",
		Country:   "FI",
		Elevation: 151,
		Latitude:  63.1271018982,
		Longitude: 23.051399231,
		Timezone:  "Europe/Helsinki",
	},
	"EFKE": {
		ICAO:      "EFKE",
		IATA:      "KEM",
		Name:      "Kemi-Tornio Airport",
		City:      "Kemi / Tornio",
		State:     "Lapland",
		Country:   "FI",
		Elevation: 61,
		Latitude:  65.7787017822,
		Longitude: 24.5820999146,
		Timezone:  "Europe/Helsinki",
	},
	"EFKG": {
		ICAO:      "EFKG",
		Name:      "Kumlinge Airport",
		Country:   "FI",
		Elevation: 7,
		Latitude:  60.2468986511,
		Longitude: 20.8047008514,
		Timezone:  "Europe/Mariehamn",
	},
	"EFKH": {
		ICAO:      "EFKH",
		Name:      "Kuhmo Airport",
		State:     "Kainuu",
		Country:   "FI",
		Elevation: 571,
		Latitude:  64.1125030518,
		Longitude: 29.4386005402,
		Timezone:  "Europe/Helsinki",
	},
	"EFKI": {
		ICAO:      "EFKI",
		IATA:      "KAJ",
		Name:      "Kajaani Airport",
		City:      "Kajaani",
		State:     "Kainuu",
		Country:   "FI",
		Elevation: 483,
		Latitude:  64.2854995728,
		Longitude: 27.6923999786,
		Timezone:  "Europe/Helsinki",
	},
	"EFKJ": {
		ICAO:      "EFKJ",
		IATA:      "KHJ",
		Name:      "Kauhajoki Airport",
		State:     "Southern-Ostrobothnia",
		Country:   "FI",
		Elevation: 407,
		Latitude:  62.4625015259,
		Longitude: 22.3931007385,
		Timezone:  "Europe/Helsinki",
	},
	"EFKK": {
		ICAO:      "EFKK",
		IATA:      "KOK",
		Name:      "Kruunupyy Airport",
		City:      "Kokkola / Kruunupyy",
		State:     "Ostrobothnia",
		Country:   "FI",
		Elevation: 84,
		Latitude:  63.7211990356,
		Longitude: 23.1431007385,
		Timezone:  "Europe/Helsinki",
	},
	"EFKM": {
		ICAO:      "EFKM",
		Name:      "Kemijarvi Airport",
		State:     "Lapland",
		Country:   "FI",
		Elevation: 692,
		Latitude:  66.7128982544,
		Longitude: 27.1567993164,
		Timezone:  "Europe/Helsinki",
	},
	"EFKN": {
		ICAO:      "EFKN",
		Name:      "Kannus Airport",
		State:     "Central-Ostrobothnia",
		Country:   "FI",
		Elevation: 338,
		Latitude:  63.9206008911,
		Longitude: 24.0867004395,
		Timezone:  "Europe/Helsinki",
	},
	"EFKO": {
		ICAO:      "EFKO",
		Name:      "Kalajoki Airport",
		State:     "Northern-Ostrobothnia",
		Country:   "FI",
		Elevation: 43,
		Latitude:  64.2285995483,
		Longitude: 23.8264007568,
		Timezone:  "Europe/Helsinki",
	},
	"EFKR": {
		ICAO:      "EFKR",
		Name:      "Karsamaki Airport",
		City:      "Karsamaki",
		State:     "Northern-Ostrobothnia",
		Country:   "FI",
		Elevation: 348,
		Latitude:  63.989200592,
		Longitude: 25.7436008453,
		Timezone:  "Europe/Helsinki",
	},
	"EFKS": {
		ICAO:      "EFKS",
		IATA:      "KAO",
		Name:      "Kuusamo Airport",
		City:      "Kuusamo",
		State:     "Northern-Ostrobothnia",
		Country:   "FI",
		Elevation: 866,
		Latitude:  65.9876022339,
		Longitude: 29.2394008636,
		Timezone:  "Europe/Helsinki",
	},
	"EFKT": {
		ICAO:      "EFKT",
		IATA:      "KTT",
		Name:      "Kittila Airport",
		City:      "Kittila",
		State:     "Lapland",
		Country:   "FI",
		Elevation: 644,
		Latitude:  67.7009963989,
		Longitude: 24.8467998505,
		Timezone:  "Europe/Helsinki",
	},
	"EFKU": {
		ICAO:      "EFKU",
		IATA:      "KUO",
		Name:      "Kuopio Airport",
		City:      "Kuopio / Siilinjarvi",
		State:     "Northern-Savo",
		Country:   "FI",
		Elevation: 323,
		Latitude:  63.0070991516,
		Longitude: 27.7978000641,
		Timezone:  "Europe/Helsinki",
	},
	"EFKV": {
		ICAO:      "EFKV",
		Name:      "Kivijarvi Airport",
		State:     "Central-Finland",
		Country:   "FI",
		Elevation: 502,
		Latitude:  63.1253013611,
		Longitude: 25.1242008209,
		Timezone:  "Europe/Helsinki",
	},
	"EFKY": {
		ICAO:      "EFKY",
		Name:      "Kymi Airport",
		City:      "Peippola",
		State:     "Central-Finland",
		Country:   "FI",
		Elevation: 223,
		Latitude:  60.5713996887,
		Longitude: 26.8960990906,
		Timezone:  "Europe/Helsinki",
	},
	"EFLA": {
		ICAO:      "EFLA",
		IATA:      "QLF",
		Name:      "Lahti Vesivehmaa Airport",
		State:     "Paeijaenne-Tavastia",
		Country:   "FI",
		Elevation: 502,
		Latitude:  61.1441993713,
		Longitude: 25.6935005188,
		Timezone:  "Europe/Helsinki",
	},
	"EFLL": {
		ICAO:      "EFLL",
		Name:      "Lapinlahti Airport",
		City:      "Lammake",
		State:     "Northern-Savo",
		Country:   "FI",
		Elevation: 328,
		Latitude:  63.3993988037,
		Longitude: 27.4789009094,
		Timezone:  "Europe/Helsinki",
	},
	"EFLN": {
		ICAO:      "EFLN",
		Name:      "Lieksa Nurmes Airport",
		State:     "North-Karelia",
		Country:   "FI",
		Elevation: 374,
		Latitude:  63.5119018555,
		Longitude: 29.6291999817,
		Timezone:  "Europe/Helsinki",
	},
	"EFLP": {
		ICAO:      "EFLP",
		IATA:      "LPP",
		Name:      "Lappeenranta Airport",
		City:      "Lappeenranta",
		State:     "South-Karelia",
		Country:   "FI",
		Elevation: 349,
		Latitude:  61.0446014404,
		Longitude: 28.1443996429,
		Timezone:  "Europe/Helsinki",
	},
	"EFMA": {
		ICAO:      "EFMA",
		IATA:      "MHQ",
		Name:      "Mariehamn Airport",
		State:     "Mariehamns-stad",
		Country:   "FI",
		Elevation: 17,
		Latitude:  60.1222000122,
		Longitude: 19.8981990814,
		Timezone:  "Europe/Mariehamn",
	},
	"EFME": {
		ICAO:      "EFME",
		Name:      "Menkijarvi Airport",
		State:     "Southern-Ostrobothnia",
		Country:   "FI",
		Elevation: 331,
		Latitude:  62.9467010498,
		Longitude: 23.5188999176,
		Timezone:  "Europe/Helsinki",
	},
	"EFMI": {
		ICAO:      "EFMI",
		IATA:      "MIK",
		Name:      "Mikkeli Airport",
		City:      "Mikkeli",
		State:     "Southern-Savonia",
		Country:   "FI",
		Elevation: 329,
		Latitude:  61.6865997314,
		Longitude: 27.2017993927,
		Timezone:  "Europe/Helsinki",
	},
	"EFMN": {
		ICAO:      "EFMN",
		Name:      "Mantsala Airport",
		City:      "Mantsala",
		State:     "Uusimaa",
		Country:   "FI",
		Elevation: 135,
		Latitude:  60.5724983215,
		Longitude: 25.5088996887,
		Timezone:  "Europe/Helsinki",
	},
	"EFMP": {
		ICAO:      "EFMP",
		Name:      "Martiniiskonpalo Airport",
		State:     "Lapland",
		Country:   "FI",
		Latitude:  68.6603012085,
		Longitude: 25.7028999329,
		Timezone:  "Europe/Helsinki",
	},
	"EFNS": {
		ICAO:      "EFNS",
		Name:      "Savikko Airport",
		State:     "Uusimaa",
		Country:   "FI",
		Elevation: 233,
		Latitude:  60.5200004578,
		Longitude: 24.8316993713,
		Timezone:  "Europe/Helsinki",
	},
	"EFNU": {
		ICAO:      "EFNU",
		Name:      "Nummela Airport",
		State:     "Uusimaa",
		Country:   "FI",
		Elevation: 367,
		Latitude:  60.3339004517,
		Longitude: 24.2964000702,
		Timezone:  "Europe/Helsinki",
	},
	"EFOP": {
		ICAO:      "EFOP",
		Name:      "Oripaa Airport",
		City:      "Oripaa",
		State:     "Finland-Proper",
		Country:   "FI",
		Elevation: 331,
		Latitude:  60.8763999939,
		Longitude: 22.7446994781,
		Timezone:  "Europe/Helsinki",
	},
	"EFOU": {
		ICAO:      "EFOU",
		IATA:      "OUL",
		Name:      "Oulu Airport",
		City:      "Oulu / Oulunsalo",
		State:     "Northern-Ostrobothnia",
		Country:   "FI",
		Elevation: 47,
		Latitude:  64.9300994873,
		Longitude: 25.3546009064,
		Timezone:  "Europe/Helsinki",
	},
	"EFPA": {
		ICAO:      "EFPA",
		Name:      "Pokka Airport",
		State:     "Lapland",
		Country:   "FI",
		Elevation: 853,
		Latitude:  68.1502227783,
		Longitude: 25.829372406,
		Timezone:  "Europe/Helsinki",
	},
	"EFPH": {
		ICAO:      "EFPH",
		Name:      "Pyhoselka Airport",
		State:     "North-Karelia",
		Country:   "FI",
		Elevation: 322,
		Latitude:  62.4646987915,
		Longitude: 30.0352993011,
		Timezone:  "Europe/Helsinki",
	},
	"EFPI": {
		ICAO:      "EFPI",
		Name:      "Piikajarvi Airport",
		State:     "Satakunta",
		Country:   "FI",
		Elevation: 148,
		Latitude:  61.2456016541,
		Longitude: 22.1933994293,
		Timezone:  "Europe/Helsinki",
	},
	"EFPK": {
		ICAO:      "EFPK",
		Name:      "Pieksamaki Airport",
		City:      "Pieksamaki",
		State:     "Southern-Savonia",
		Country:   "FI",
		Elevation: 390,
		Latitude:  62.2647018433,
		Longitude: 27.0027999878,
		Timezone:  "Europe/Helsinki",
	},
	"EFPN": {
		ICAO:      "EFPN",
		Name:      "Punkaharju Airport",
		State:     "Southern-Savonia",
		Country:   "FI",
		Elevation: 253,
		Latitude:  61.7289009094,
		Longitude: 29.3936004639,
		Timezone:  "Europe/Helsinki",
	},
	"EFPO": {
		ICAO:      "EFPO",
		IATA:      "POR",
		Name:      "Pori Airport",
		City:      "Pori",
		State:     "Satakunta",
		Country:   "FI",
		Elevation: 44,
		Latitude:  61.4617004395,
		Longitude: 21.7999992371,
		Timezone:  "Europe/Helsinki",
	},
	"EFPU": {
		ICAO:      "EFPU",
		Name:      "Pudasjarvi Airport",
		State:     "Northern-Ostrobothnia",
		Country:   "FI",
		Elevation: 397,
		Latitude:  65.4021987915,
		Longitude: 26.9468994141,
		Timezone:  "Europe/Helsinki",
	},
	"EFPY": {
		ICAO:      "EFPY",
		Name:      "Pyhasalmi Airport",
		City:      "Pyhajarvi",
		State:     "Northern-Ostrobothnia",
		Country:   "FI",
		Elevation: 528,
		Latitude:  63.7318992615,
		Longitude: 25.9263000488,
		Timezone:  "Europe/Helsinki",
	},
	"EFRA": {
		ICAO:      "EFRA",
		Name:      "Rautavaara Airport",
		State:     "Northern-Savo",
		Country:   "FI",
		Elevation: 476,
		Latitude:  63.4241981506,
		Longitude: 28.1242008209,
		Timezone:  "Europe/Helsinki",
	},
	"EFRH": {
		ICAO:      "EFRH",
		Name:      "Raahe Pattijoki Airport",
		State:     "Northern-Ostrobothnia",
		Country:   "FI",
		Elevation: 118,
		Latitude:  64.6881027222,
		Longitude: 24.6958007812,
		Timezone:  "Europe/Helsinki",
	},
	"EFRN": {
		ICAO:      "EFRN",
		Name:      "Rantasalmi Airport",
		State:     "Southern-Savonia",
		Country:   "FI",
		Elevation: 292,
		Latitude:  62.0654983521,
		Longitude: 28.3565006256,
		Timezone:  "Europe/Helsinki",
	},
	"EFRO": {
		ICAO:      "EFRO",
		IATA:      "RVN",
		Name:      "Rovaniemi Airport",
		City:      "Rovaniemi",
		State:     "Lapland",
		Country:   "FI",
		Elevation: 642,
		Latitude:  66.5647964478,
		Longitude: 25.8304004669,
		Timezone:  "Europe/Helsinki",
	},
	"EFRU": {
		ICAO:      "EFRU",
		Name:      "Ranua Airport",
		State:     "Lapland",
		Country:   "FI",
		Elevation: 541,
		Latitude:  65.9730987549,
		Longitude: 26.3652992249,
		Timezone:  "Europe/Helsinki",
	},
	"EFRV": {
		ICAO:      "EFRV",
		Name:      "Kiuruvesi Airport",
		State:     "Northern-Savo",
		Country:   "FI",
		Elevation: 515,
		Latitude:  63.7056007385,
		Longitude: 26.616399765,
		Timezone:  "Europe/Helsinki",
	},
	"EFRY": {
		ICAO:      "EFRY",
		Name:      "Rayskala Airport",
		State:     "Tavastia-Proper",
		Country:   "FI",
		Elevation: 407,
		Latitude:  60.7447013855,
		Longitude: 24.10779953,
		Timezone:  "Europe/Helsinki",
	},
	"EFSA": {
		ICAO:      "EFSA",
		IATA:      "SVL",
		Name:      "Savonlinna Airport",
		City:      "Savonlinna",
		State:     "Southern-Savonia",
		Country:   "FI",
		Elevation: 311,
		Latitude:  61.9430999756,
		Longitude: 28.9451007843,
		Timezone:  "Europe/Helsinki",
	},
	"EFSE": {
		ICAO:      "EFSE",
		Name:      "Selanpaa Airport",
		State:     "Kymenlaakso",
		Country:   "FI",
		Elevation: 417,
		Latitude:  61.0624008179,
		Longitude: 26.7989006042,
		Timezone:  "Europe/Helsinki",
	},
	"EFSI": {
		ICAO:      "EFSI",
		IATA:      "SJY",
		Name:      "Seinajoki Airport",
		City:      "Seinajoki / Ilmajoki",
		State:     "Southern-Ostrobothnia",
		Country:   "FI",
		Elevation: 302,
		Latitude:  62.6921005249,
		Longitude: 22.8323001862,
		Timezone:  "Europe/Helsinki",
	},
	"EFSJ": {
		ICAO:      "EFSJ",
		Name:      "Sonkajarvi Jyrkk Airport",
		State:     "Northern-Savo",
		Country:   "FI",
		Elevation: 600,
		Latitude:  63.8194007874,
		Longitude: 27.7693996429,
		Timezone:  "Europe/Helsinki",
	},
	"EFSO": {
		ICAO:      "EFSO",
		IATA:      "SOT",
		Name:      "Sodankyla Airport",
		City:      "Sodankyla",
		State:     "Lapland",
		Country:   "FI",
		Elevation: 602,
		Latitude:  67.3949966431,
		Longitude: 26.6191005707,
		Timezone:  "Europe/Helsinki",
	},
	"EFSU": {
		ICAO:      "EFSU",
		Name:      "Suomussalmi Airport",
		State:     "Kainuu",
		Country:   "FI",
		Elevation: 541,
		Latitude:  64.8218994141,
		Longitude: 28.7103004456,
		Timezone:  "Europe/Helsinki",
	},
	"EFTO": {
		ICAO:      "EFTO",
		Name:      "Torbacka Airport",
		State:     "Uusimaa",
		Country:   "FI",
		Elevation: 16,
		Latitude:  60.0792007446,
		Longitude: 24.1721992493,
		Timezone:  "Europe/Helsinki",
	},
	"EFTP": {
		ICAO:      "EFTP",
		IATA:      "TMP",
		Name:      "Tampere-Pirkkala Airport",
		City:      "Tampere / Pirkkala",
		State:     "Pirkanmaa",
		Country:   "FI",
		Elevation: 390,
		Latitude:  61.414100647,
		Longitude: 23.6044006348,
		Timezone:  "Europe/Helsinki",
	},
	"EFTS": {
		ICAO:      "EFTS",
		Name:      "Teisko Airport",
		City:      "Tampere",
		State:     "Pirkanmaa",
		Country:   "FI",
		Elevation: 515,
		Latitude:  61.7733001709,
		Longitude: 24.0270004272,
		Timezone:  "Europe/Helsinki",
	},
	"EFTU": {
		ICAO:      "EFTU",
		IATA:      "TKU",
		Name:      "Turku Airport",
		City:      "Turku",
		State:     "Finland-Proper",
		Country:   "FI",
		Elevation: 161,
		Latitude:  60.5140991211,
		Longitude: 22.2628002167,
		Timezone:  "Europe/Helsinki",
	},
	"EFUT": {
		ICAO:      "EFUT",
		IATA:      "UTI",
		Name:      "Utti Air Base",
		City:      "Utti / Valkeala",
		State:     "Kymenlaakso",
		Country:   "FI",
		Elevation: 339,
		Latitude:  60.8964004517,
		Longitude: 26.9384002686,
		Timezone:  "Europe/Helsinki",
	},
	"EFVA": {
		ICAO:      "EFVA",
		IATA:      "VAA",
		Name:      "Vaasa Airport",
		City:      "Vaasa",
		State:     "Ostrobothnia",
		Country:   "FI",
		Elevation: 19,
		Latitude:  63.0507011414,
		Longitude: 21.7621994019,
		Timezone:  "Europe/Helsinki",
	},
	"EFVI": {
		ICAO:      "EFVI",
		Name:      "Viitasaari Airport",
		State:     "Central-Finland",
		Country:   "FI",
		Elevation: 361,
		Latitude:  63.1225013733,
		Longitude: 25.8160991669,
		Timezone:  "Europe/Helsinki",
	},
	"EFVL": {
		ICAO:      "EFVL",
		Name:      "Vaala Airport",
		State:     "Kainuu",
		Country:   "FI",
		Elevation: 443,
		Latitude:  64.5018997192,
		Longitude: 26.7600002289,
		Timezone:  "Europe/Helsinki",
	},
	"EFVP": {
		ICAO:      "EFVP",
		Name:      "Vampula Airport",
		State:     "Satakunta",
		Country:   "FI",
		Elevation: 295,
		Latitude:  61.0396995544,
		Longitude: 22.5916996002,
		Timezone:  "Europe/Helsinki",
	},
	"EFVR": {
		ICAO:      "EFVR",
		IATA:      "VRK",
		Name:      "Varkaus Airport",
		City:      "Varkaus / Joroinen",
		State:     "Southern-Savonia",
		Country:   "FI",
		Elevation: 286,
		Latitude:  62.1711006165,
		Longitude: 27.8686008453,
		Timezone:  "Europe/Helsinki",
	},
	"EFVT": {
		ICAO:      "EFVT",
		Name:      "Sulkaharju Airport",
		State:     "Central-Ostrobothnia",
		Country:   "FI",
		Elevation: 400,
		Latitude:  63.3978004456,
		Longitude: 24.0305995941,
		Timezone:  "Europe/Helsinki",
	},
	"EFVU": {
		ICAO:      "EFVU",
		Name:      "Vuotso Airport",
		City:      "Vuotso",
		State:     "Lapland",
		Country:   "FI",
		Latitude:  68.0871963501,
		Longitude: 27.1238994598,
		Timezone:  "Europe/Helsinki",
	},
	"EFWB": {
		ICAO:      "EFWB",
		Name:      "Wredeby Airport",
		State:     "Kymenlaakso",
		Country:   "FI",
		Elevation: 82,
		Latitude:  60.6636009216,
		Longitude: 26.7458000183,
		Timezone:  "Europe/Helsinki",
	},
	"EFYL": {
		ICAO:      "EFYL",
		IATA:      "YLI",
		Name:      "Ylivieska Airport",
		State:     "Northern-Ostrobothnia",
		Country:   "FI",
		Elevation: 252,
		Latitude:  64.0605010986,
		Longitude: 24.7159996033,
		Timezone:  "Europe/Helsinki",
	},
	"EG02": {
		ICAO:      "EG02",
		Name:      "Audley End Airfield",
		City:      "Safron Waldon",
		State:     "England",
		Country:   "GB",
		Elevation: 283,
		Latitude:  52.0091365357,
		Longitude: 0.224275589,
		Timezone:  "Europe/London",
	},
	"EG03": {
		ICAO:      "EG03",
		Name:      "Badminton Airfield",
		City:      "Badminton",
		Country:   "GB",
		Elevation: 495,
		Latitude:  51.5484166846,
		Longitude: -2.3031806946,
		Timezone:  "Europe/London",
	},
	"EG10": {
		ICAO:      "EG10",
		Name:      "Breighton Airfield",
		City:      "Breighton",
		State:     "England",
		Country:   "GB",
		Elevation: 163,
		Latitude:  53.8023996399,
		Longitude: -0.9159636498,
		Timezone:  "Europe/London",
	},
	"EG11": {
		ICAO:      "EG11",
		Name:      "Canterbury Airfield",
		City:      "Canterbury",
		State:     "England",
		Country:   "GB",
		Elevation: 300,
		Latitude:  51.2918749964,
		Longitude: 0.999584198,
		Timezone:  "Europe/London",
	},
	"EG12": {
		ICAO:      "EG12",
		Name:      "Cromer/Northrepps Aerodrome",
		City:      "Cromer",
		State:     "England",
		Country:   "GB",
		Elevation: 181,
		Latitude:  52.893807,
		Longitude: 1.32106,
		Timezone:  "Europe/London",
	},
	"EG18": {
		ICAO:      "EG18",
		Name:      "Eshott Airfield",
		City:      "Eshott",
		State:     "England",
		Country:   "GB",
		Elevation: 197,
		Latitude:  55.2804841148,
		Longitude: -1.7148971558,
		Timezone:  "Europe/London",
	},
	"EG19": {
		ICAO:      "EG19",
		Name:      "Fadmoor Airfield",
		City:      "Kirbymoorside",
		State:     "England",
		Country:   "GB",
		Latitude:  54.3080863149,
		Longitude: -0.9831047058,
		Timezone:  "Europe/London",
	},
	"EG34": {
		ICAO:      "EG34",
		Name:      "Ledbury Airport",
		City:      "Ledbury",
		State:     "England",
		Country:   "GB",
		Elevation: 250,
		Latitude:  52.0018997192,
		Longitude: -2.474159956,
		Timezone:  "Europe/London",
	},
	"EG3L": {
		ICAO:      "EG3L",
		Name:      "Millfield",
		State:     "England",
		Country:   "GB",
		Latitude:  55.5916364278,
		Longitude: -2.0872306824,
		Timezone:  "Europe/London",
	},
	"EG64": {
		ICAO:      "EG64",
		Name:      "RAF Rufforth",
		City:      "York",
		State:     "England",
		Country:   "GB",
		Elevation: 65,
		Latitude:  53.94829,
		Longitude: -1.185575,
		Timezone:  "Europe/London",
	},
	"EG73": {
		ICAO:      "EG73",
		Name:      "Fearn Airport",
		City:      "Fearn",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 25,
		Latitude:  57.7579994202,
		Longitude: -3.9430000782,
		Timezone:  "Europe/London",
	},
	"EG74": {
		ICAO:      "EG74",
		IATA:      "LHB",
		Name:      "Bruntingthorpe Airport",
		City:      "Bruntingthorpe",
		Country:   "GB",
		Elevation: 467,
		Latitude:  52.492971862,
		Longitude: -1.1250901222,
		Timezone:  "Europe/London",
	},
	"EG78": {
		ICAO:      "EG78",
		Name:      "Out Skerries",
		City:      "Out Skerries",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 20,
		Latitude:  60.425583,
		Longitude: -0.7466,
		Timezone:  "Europe/London",
	},
	"EGAA": {
		ICAO:      "EGAA",
		IATA:      "BFS",
		Name:      "Belfast International Airport",
		City:      "Belfast",
		State:     "Northern-Ireland",
		Country:   "GB",
		Elevation: 268,
		Latitude:  54.6575012207,
		Longitude: -6.2158298492,
		Timezone:  "Europe/London",
	},
	"EGAB": {
		ICAO:      "EGAB",
		IATA:      "ENK",
		Name:      "St Angelo Airport",
		City:      "Enniskillen",
		State:     "Northern-Ireland",
		Country:   "GB",
		Elevation: 155,
		Latitude:  54.3988990784,
		Longitude: -7.6516699791,
		Timezone:  "Europe/London",
	},
	"EGAC": {
		ICAO:      "EGAC",
		IATA:      "BHD",
		Name:      "George Best Belfast City Airport",
		City:      "Belfast",
		State:     "Northern-Ireland",
		Country:   "GB",
		Elevation: 15,
		Latitude:  54.6180992126,
		Longitude: -5.8724999428,
		Timezone:  "Europe/London",
	},
	"EGAD": {
		ICAO:      "EGAD",
		Name:      "Newtownards Airport",
		City:      "Newtownards",
		State:     "Northern-Ireland",
		Country:   "GB",
		Elevation: 9,
		Latitude:  54.5811004639,
		Longitude: -5.6919398308,
		Timezone:  "Europe/London",
	},
	"EGAE": {
		ICAO:      "EGAE",
		IATA:      "LDY",
		Name:      "City of Derry Airport",
		City:      "Derry",
		State:     "Northern-Ireland",
		Country:   "GB",
		Elevation: 22,
		Latitude:  55.0428009033,
		Longitude: -7.1611099243,
		Timezone:  "Europe/London",
	},
	"EGAH": {
		ICAO:      "EGAH",
		Name:      "Halley Research Station",
		Country:   "AQ",
		Elevation: 108,
		Latitude:  -75.5830555556,
		Longitude: -26.5669444444,
		Timezone:  "Antarctica/Rothera",
	},
	"EGAL": {
		ICAO:      "EGAL",
		Name:      "Langford Lodge Airport",
		City:      "Belfast",
		Country:   "GB",
		Latitude:  54.6231002808,
		Longitude: -6.3000001907,
		Timezone:  "Europe/London",
	},
	"EGAR": {
		ICAO:      "EGAR",
		Name:      "Rothera Research Station",
		Country:   "AQ",
		Elevation: 9,
		Latitude:  -67.5675,
		Longitude: -68.1274027778,
		Timezone:  "Antarctica/Rothera",
	},
	"EGBB": {
		ICAO:      "EGBB",
		IATA:      "BHX",
		Name:      "Birmingham International Airport",
		City:      "Birmingham",
		State:     "England",
		Country:   "GB",
		Elevation: 327,
		Latitude:  52.4538993835,
		Longitude: -1.7480299473,
		Timezone:  "Europe/London",
	},
	"EGBD": {
		ICAO:      "EGBD",
		Name:      "Derby Airfield",
		City:      "Derby",
		State:     "England",
		Country:   "GB",
		Elevation: 175,
		Latitude:  52.8596992493,
		Longitude: -1.6174999476,
		Timezone:  "Europe/London",
	},
	"EGBE": {
		ICAO:      "EGBE",
		IATA:      "CVT",
		Name:      "Coventry Airport",
		City:      "Coventry",
		State:     "England",
		Country:   "GB",
		Elevation: 267,
		Latitude:  52.3697013855,
		Longitude: -1.4797199965,
		Timezone:  "Europe/London",
	},
	"EGBF": {
		ICAO:      "EGBF",
		Name:      "Bedford Aerodrome",
		City:      "Bedford",
		State:     "England",
		Country:   "GB",
		Elevation: 273,
		Latitude:  52.2308485669,
		Longitude: -0.4624557495,
		Timezone:  "Europe/London",
	},
	"EGBG": {
		ICAO:      "EGBG",
		Name:      "Leicester Airport",
		City:      "Leicester",
		State:     "England",
		Country:   "GB",
		Elevation: 469,
		Latitude:  52.60779953,
		Longitude: -1.0319399834,
		Timezone:  "Europe/London",
	},
	"EGBJ": {
		ICAO:      "EGBJ",
		IATA:      "GLO",
		Name:      "Gloucestershire Airport",
		City:      "Staverton",
		State:     "England",
		Country:   "GB",
		Elevation: 101,
		Latitude:  51.8941993713,
		Longitude: -2.1672201157,
		Timezone:  "Europe/London",
	},
	"EGBK": {
		ICAO:      "EGBK",
		IATA:      "ORM",
		Name:      "Sywell Aerodrome",
		City:      "Northampton",
		State:     "England",
		Country:   "GB",
		Elevation: 429,
		Latitude:  52.3053016663,
		Longitude: -0.7930560112,
		Timezone:  "Europe/London",
	},
	"EGBL": {
		ICAO:      "EGBL",
		Name:      "Long Marston Airfield",
		City:      "Long Marston",
		State:     "England",
		Country:   "GB",
		Elevation: 154,
		Latitude:  52.1403007507,
		Longitude: -1.7536100149,
		Timezone:  "Europe/London",
	},
	"EGBM": {
		ICAO:      "EGBM",
		Name:      "Tatenhill Airfield",
		City:      "Burton upon Trent",
		State:     "England",
		Country:   "GB",
		Elevation: 439,
		Latitude:  52.8147010803,
		Longitude: -1.7611099482,
		Timezone:  "Europe/London",
	},
	"EGBN": {
		ICAO:      "EGBN",
		IATA:      "NQT",
		Name:      "Nottingham Airport",
		City:      "Nottingham",
		State:     "England",
		Country:   "GB",
		Elevation: 138,
		Latitude:  52.9199981689,
		Longitude: -1.0791699886,
		Timezone:  "Europe/London",
	},
	"EGBO": {
		ICAO:      "EGBO",
		Name:      "Wolverhampton Halfpenny Green Airport",
		City:      "Wolverhampton",
		State:     "England",
		Country:   "GB",
		Elevation: 283,
		Latitude:  52.5175018311,
		Longitude: -2.2594399452,
		Timezone:  "Europe/London",
	},
	"EGBP": {
		ICAO:      "EGBP",
		IATA:      "GBA",
		Name:      "Kemble Airport",
		City:      "Kemble",
		State:     "England",
		Country:   "GB",
		Elevation: 433,
		Latitude:  51.6680984497,
		Longitude: -2.0569400787,
		Timezone:  "Europe/London",
	},
	"EGBR": {
		ICAO:      "EGBR",
		Name:      "Breighton Airfield",
		City:      "Breighton",
		State:     "England",
		Country:   "GB",
		Elevation: 163,
		Latitude:  53.8023996399,
		Longitude: -0.9159636498,
		Timezone:  "Europe/London",
	},
	"EGBS": {
		ICAO:      "EGBS",
		Name:      "Shobdon Aerodrome",
		City:      "Shobdon",
		State:     "England",
		Country:   "GB",
		Elevation: 318,
		Latitude:  52.2416992188,
		Longitude: -2.8811099529,
		Timezone:  "Europe/London",
	},
	"EGBT": {
		ICAO:      "EGBT",
		Name:      "Turweston Airport",
		City:      "Turweston",
		Country:   "GB",
		Elevation: 448,
		Latitude:  52.0407981873,
		Longitude: -1.0955599546,
		Timezone:  "Europe/London",
	},
	"EGBW": {
		ICAO:      "EGBW",
		Name:      "Wellesbourne Mountford Airport",
		City:      "Stratford-on-Avon",
		State:     "England",
		Country:   "GB",
		Elevation: 159,
		Latitude:  52.192199707,
		Longitude: -1.6144399643,
		Timezone:  "Europe/London",
	},
	"EGCB": {
		ICAO:      "EGCB",
		Name:      "City Airport Manchester",
		City:      "Manchester / Salford",
		State:     "England",
		Country:   "GB",
		Elevation: 73,
		Latitude:  53.471698761,
		Longitude: -2.3897199631,
		Timezone:  "Europe/London",
	},
	"EGCC": {
		ICAO:      "EGCC",
		IATA:      "MAN",
		Name:      "Manchester Airport",
		City:      "Manchester",
		State:     "England",
		Country:   "GB",
		Elevation: 257,
		Latitude:  53.3536987305,
		Longitude: -2.2749500275,
		Timezone:  "Europe/London",
	},
	"EGCD": {
		ICAO:      "EGCD",
		Name:      "Manchester Woodford Airport",
		City:      "Manchester",
		State:     "England",
		Country:   "GB",
		Elevation: 295,
		Latitude:  53.3381004333,
		Longitude: -2.1488900185,
		Timezone:  "Europe/London",
	},
	"EGCF": {
		ICAO:      "EGCF",
		Name:      "Sandtoft Airfield",
		City:      "Belton",
		Country:   "GB",
		Elevation: 11,
		Latitude:  53.5597000122,
		Longitude: -0.8583329916,
		Timezone:  "Europe/London",
	},
	"EGCJ": {
		ICAO:      "EGCJ",
		Name:      "Sherburn-In-Elmet Airfield",
		City:      "Selby",
		State:     "England",
		Country:   "GB",
		Elevation: 26,
		Latitude:  53.7844009399,
		Longitude: -1.2172199488,
		Timezone:  "Europe/London",
	},
	"EGCK": {
		ICAO:      "EGCK",
		Name:      "Caernarfon Airport",
		City:      "Caernarfon",
		State:     "Wales",
		Country:   "GB",
		Elevation: 1,
		Latitude:  53.1041984558,
		Longitude: -4.340280056,
		Timezone:  "Europe/London",
	},
	"EGCL": {
		ICAO:      "EGCL",
		Name:      "Fenland Airfield",
		City:      "Spalding",
		Country:   "GB",
		Elevation: 8,
		Latitude:  52.739200592,
		Longitude: -0.0297219995,
		Timezone:  "Europe/London",
	},
	"EGCN": {
		ICAO:      "EGCN",
		IATA:      "DSA",
		Name:      "Robin Hood Doncaster Sheffield Airport",
		City:      "Doncaster",
		State:     "England",
		Country:   "GB",
		Elevation: 55,
		Latitude:  53.4805378105,
		Longitude: -1.0106563568,
		Timezone:  "Europe/London",
	},
	"EGCO": {
		ICAO:      "EGCO",
		Name:      "Birkdale Sands Airport",
		City:      "Southport",
		State:     "England",
		Country:   "GB",
		Elevation: 3,
		Latitude:  53.6453018188,
		Longitude: -3.0286099911,
		Timezone:  "Europe/London",
	},
	"EGCP": {
		ICAO:      "EGCP",
		Name:      "Thorne Airport",
		City:      "Thorne",
		State:     "England",
		Country:   "GB",
		Elevation: 16,
		Latitude:  53.6222000122,
		Longitude: -0.9280560017,
		Timezone:  "Europe/London",
	},
	"EGCR": {
		ICAO:      "EGCR",
		Name:      "Ashcroft Airfield",
		City:      "Winsford",
		Country:   "GB",
		Elevation: 150,
		Latitude:  53.1644238046,
		Longitude: -2.5721740723,
		Timezone:  "Europe/London",
	},
	"EGCS": {
		ICAO:      "EGCS",
		Name:      "Sturgate Airfield",
		City:      "Lincoln",
		Country:   "GB",
		Elevation: 58,
		Latitude:  53.3810997009,
		Longitude: -0.6852779984,
		Timezone:  "Europe/London",
	},
	"EGCT": {
		ICAO:      "EGCT",
		Name:      "Tilstock Airfield",
		City:      "Whitchurch",
		State:     "England",
		Country:   "GB",
		Elevation: 301,
		Latitude:  52.9308013916,
		Longitude: -2.6463899612,
		Timezone:  "Europe/London",
	},
	"EGCV": {
		ICAO:      "EGCV",
		Name:      "Sleap Airport",
		City:      "Shrewsbury",
		State:     "England",
		Country:   "GB",
		Elevation: 275,
		Latitude:  52.8339004517,
		Longitude: -2.7716701031,
		Timezone:  "Europe/London",
	},
	"EGCW": {
		ICAO:      "EGCW",
		Name:      "Welshpool Airport",
		City:      "Welshpool",
		State:     "Wales",
		Country:   "GB",
		Elevation: 233,
		Latitude:  52.6286010742,
		Longitude: -3.1533300877,
		Timezone:  "Europe/London",
	},
	"EGDC": {
		ICAO:      "EGDC",
		Name:      "Royal Marines Base Chivenor Airport",
		City:      "Chivenor",
		State:     "England",
		Country:   "GB",
		Elevation: 27,
		Latitude:  51.0872001648,
		Longitude: -4.1503400803,
		Timezone:  "Europe/London",
	},
	"EGDD": {
		ICAO:      "EGDD",
		Name:      "Bicester Airfield",
		City:      "Bicester",
		Country:   "GB",
		Elevation: 266,
		Latitude:  51.9166984558,
		Longitude: -1.1319400072,
		Timezone:  "Europe/London",
	},
	"EGDJ": {
		ICAO:      "EGDJ",
		IATA:      "UPV",
		Name:      "Upavon Aerodrome",
		City:      "Upavon",
		State:     "England",
		Country:   "GB",
		Elevation: 574,
		Latitude:  51.2862014771,
		Longitude: -1.7820199728,
		Timezone:  "Europe/London",
	},
	"EGDL": {
		ICAO:      "EGDL",
		IATA:      "LYE",
		Name:      "RAF Lyneham",
		City:      "Lyneham",
		State:     "England",
		Country:   "GB",
		Elevation: 513,
		Latitude:  51.5051002502,
		Longitude: -1.9934300184,
		Timezone:  "Europe/London",
	},
	"EGDM": {
		ICAO:      "EGDM",
		Name:      "MoD Boscombe Down Airport",
		City:      "Amesbury",
		State:     "England",
		Country:   "GB",
		Elevation: 407,
		Latitude:  51.1521987915,
		Longitude: -1.747410059,
		Timezone:  "Europe/London",
	},
	"EGDN": {
		ICAO:      "EGDN",
		Name:      "Netheravon Airfield",
		City:      "Netheravon",
		Country:   "GB",
		Elevation: 456,
		Latitude:  51.2472000122,
		Longitude: -1.7542500496,
		Timezone:  "Europe/London",
	},
	"EGDO": {
		ICAO:      "EGDO",
		Name:      "Predannack Airport",
		City:      "Predannack Wollas",
		State:     "England",
		Country:   "GB",
		Elevation: 295,
		Latitude:  50.0012016296,
		Longitude: -5.2308301926,
		Timezone:  "Europe/London",
	},
	"EGDR": {
		ICAO:      "EGDR",
		Name:      "RNAS Culdrose",
		City:      "Helston",
		State:     "England",
		Country:   "GB",
		Elevation: 267,
		Latitude:  50.086101532,
		Longitude: -5.255710125,
		Timezone:  "Europe/London",
	},
	"EGDT": {
		ICAO:      "EGDT",
		Name:      "Wroughton Airfield",
		City:      "Wroughton",
		State:     "England",
		Country:   "GB",
		Elevation: 656,
		Latitude:  51.5060997009,
		Longitude: -1.8019399643,
		Timezone:  "Europe/London",
	},
	"EGDV": {
		ICAO:      "EGDV",
		Name:      "Hullavington Airfield",
		City:      "Hullavington",
		Country:   "GB",
		Elevation: 327,
		Latitude:  51.526732,
		Longitude: -2.141555,
		Timezone:  "Europe/London",
	},
	"EGDW": {
		ICAO:      "EGDW",
		Name:      "Merryfield Airfield",
		City:      "Ilminster",
		State:     "England",
		Country:   "GB",
		Elevation: 151,
		Latitude:  50.9625015259,
		Longitude: -2.935559988,
		Timezone:  "Europe/London",
	},
	"EGDX": {
		ICAO:      "EGDX",
		IATA:      "DGX",
		Name:      "MOD St. Athan",
		City:      "St. Athan",
		State:     "Wales",
		Country:   "GB",
		Elevation: 163,
		Latitude:  51.404800415,
		Longitude: -3.4357500076,
		Timezone:  "Europe/London",
	},
	"EGDY": {
		ICAO:      "EGDY",
		IATA:      "YEO",
		Name:      "RNAS Yeovilton",
		City:      "Yeovil",
		State:     "England",
		Country:   "GB",
		Elevation: 75,
		Latitude:  51.0093994141,
		Longitude: -2.6388199329,
		Timezone:  "Europe/London",
	},
	"EGEC": {
		ICAO:      "EGEC",
		IATA:      "CAL",
		Name:      "Campbeltown Airport",
		City:      "Campbeltown",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 42,
		Latitude:  55.4371986389,
		Longitude: -5.6863899231,
		Timezone:  "Europe/London",
	},
	"EGED": {
		ICAO:      "EGED",
		IATA:      "EOI",
		Name:      "Eday Airport",
		City:      "Eday",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 10,
		Latitude:  59.1906013489,
		Longitude: -2.7722198963,
		Timezone:  "Europe/London",
	},
	"EGEF": {
		ICAO:      "EGEF",
		IATA:      "FIE",
		Name:      "Fair Isle Airport",
		City:      "Fair Isle",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 223,
		Latitude:  59.5358009338,
		Longitude: -1.6280599833,
		Timezone:  "Europe/London",
	},
	"EGEH": {
		ICAO:      "EGEH",
		IATA:      "WHS",
		Name:      "Whalsay Airport",
		City:      "Whalsay",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 100,
		Latitude:  60.3774986267,
		Longitude: -0.925556004,
		Timezone:  "Europe/London",
	},
	"EGEL": {
		ICAO:      "EGEL",
		IATA:      "COL",
		Name:      "Coll Airport",
		City:      "Coll Island",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 21,
		Latitude:  56.6018981934,
		Longitude: -6.6177802086,
		Timezone:  "Europe/London",
	},
	"EGEN": {
		ICAO:      "EGEN",
		IATA:      "NRL",
		Name:      "North Ronaldsay Airport",
		City:      "North Ronaldsay",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 40,
		Latitude:  59.3675003052,
		Longitude: -2.4344398975,
		Timezone:  "Europe/London",
	},
	"EGEO": {
		ICAO:      "EGEO",
		IATA:      "OBN",
		Name:      "Oban Airport",
		City:      "North Connel",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 20,
		Latitude:  56.4635009766,
		Longitude: -5.3996701241,
		Timezone:  "Europe/London",
	},
	"EGEP": {
		ICAO:      "EGEP",
		IATA:      "PPW",
		Name:      "Papa Westray Airport",
		City:      "Papa Westray",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 91,
		Latitude:  59.3516998291,
		Longitude: -2.9002799988,
		Timezone:  "Europe/London",
	},
	"EGER": {
		ICAO:      "EGER",
		IATA:      "SOY",
		Name:      "Stronsay Airport",
		City:      "Stronsay",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 39,
		Latitude:  59.1553001404,
		Longitude: -2.6413900852,
		Timezone:  "Europe/London",
	},
	"EGES": {
		ICAO:      "EGES",
		IATA:      "NDY",
		Name:      "Sanday Airport",
		City:      "Sanday",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 68,
		Latitude:  59.2503013611,
		Longitude: -2.5766699314,
		Timezone:  "Europe/London",
	},
	"EGET": {
		ICAO:      "EGET",
		IATA:      "LWK",
		Name:      "Lerwick / Tingwall Airport",
		City:      "Lerwick",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 43,
		Latitude:  60.192199707,
		Longitude: -1.2436100245,
		Timezone:  "Europe/London",
	},
	"EGEW": {
		ICAO:      "EGEW",
		IATA:      "WRY",
		Name:      "Westray Airport",
		City:      "Westray",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 29,
		Latitude:  59.3502998352,
		Longitude: -2.9500000477,
		Timezone:  "Europe/London",
	},
	"EGEY": {
		ICAO:      "EGEY",
		IATA:      "CSA",
		Name:      "Colonsay Airstrip",
		City:      "Colonsay",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 44,
		Latitude:  56.0574989319,
		Longitude: -6.243060112,
		Timezone:  "Europe/London",
	},
	"EGFA": {
		ICAO:      "EGFA",
		Name:      "Aberporth Airport",
		City:      "Aberporth",
		State:     "Wales",
		Country:   "GB",
		Elevation: 428,
		Latitude:  52.1152992249,
		Longitude: -4.5569400787,
		Timezone:  "Europe/London",
	},
	"EGFE": {
		ICAO:      "EGFE",
		IATA:      "HAW",
		Name:      "Haverfordwest Airport",
		City:      "Haverfordwest",
		State:     "Wales",
		Country:   "GB",
		Elevation: 159,
		Latitude:  51.8330993652,
		Longitude: -4.9611101151,
		Timezone:  "Europe/London",
	},
	"EGFF": {
		ICAO:      "EGFF",
		IATA:      "CWL",
		Name:      "Cardiff International Airport",
		City:      "Cardiff",
		State:     "Wales",
		Country:   "GB",
		Elevation: 220,
		Latitude:  51.3967018127,
		Longitude: -3.3433299065,
		Timezone:  "Europe/London",
	},
	"EGFH": {
		ICAO:      "EGFH",
		IATA:      "SWS",
		Name:      "Swansea Airport",
		City:      "Swansea",
		State:     "Wales",
		Country:   "GB",
		Elevation: 299,
		Latitude:  51.6053009033,
		Longitude: -4.0678300858,
		Timezone:  "Europe/London",
	},
	"EGFP": {
		ICAO:      "EGFP",
		Name:      "Pembrey Airport",
		City:      "Pembrey",
		State:     "Wales",
		Country:   "GB",
		Elevation: 15,
		Latitude:  51.7139015198,
		Longitude: -4.3122200966,
		Timezone:  "Europe/London",
	},
	"EGGD": {
		ICAO:      "EGGD",
		IATA:      "BRS",
		Name:      "Bristol International Airport",
		City:      "Bristol",
		State:     "England",
		Country:   "GB",
		Elevation: 622,
		Latitude:  51.3827018738,
		Longitude: -2.7190899849,
		Timezone:  "Europe/London",
	},
	"EGGP": {
		ICAO:      "EGGP",
		IATA:      "LPL",
		Name:      "Liverpool John Lennon Airport",
		City:      "Liverpool",
		State:     "England",
		Country:   "GB",
		Elevation: 80,
		Latitude:  53.3335990906,
		Longitude: -2.8497200012,
		Timezone:  "Europe/London",
	},
	"EGGW": {
		ICAO:      "EGGW",
		IATA:      "LTN",
		Name:      "London Luton Airport",
		City:      "London",
		State:     "England",
		Country:   "GB",
		Elevation: 526,
		Latitude:  51.8746986389,
		Longitude: -0.3683330119,
		Timezone:  "Europe/London",
	},
	"EGHA": {
		ICAO:      "EGHA",
		Name:      "Compton Abbas Aerodrome",
		City:      "Shaftesbury",
		State:     "England",
		Country:   "GB",
		Elevation: 811,
		Latitude:  50.9672012329,
		Longitude: -2.1536099911,
		Timezone:  "Europe/London",
	},
	"EGHB": {
		ICAO:      "EGHB",
		Name:      "Maypole Airport",
		City:      "Maypole",
		State:     "England",
		Country:   "GB",
		Elevation: 110,
		Latitude:  51.3389015198,
		Longitude: 1.1530599594,
		Timezone:  "Europe/London",
	},
	"EGHC": {
		ICAO:      "EGHC",
		IATA:      "LEQ",
		Name:      "Land's End Airport",
		City:      "Land's End",
		State:     "England",
		Country:   "GB",
		Elevation: 401,
		Latitude:  50.1027984619,
		Longitude: -5.6705598831,
		Timezone:  "Europe/London",
	},
	"EGHD": {
		ICAO:      "EGHD",
		IATA:      "PLH",
		Name:      "Plymouth City Airport",
		City:      "Plymouth",
		State:     "England",
		Country:   "GB",
		Elevation: 476,
		Latitude:  50.4227981567,
		Longitude: -4.1058301926,
		Timezone:  "Europe/London",
	},
	"EGHE": {
		ICAO:      "EGHE",
		IATA:      "ISC",
		Name:      "St. Mary's Airport",
		City:      "St. Mary's",
		State:     "England",
		Country:   "GB",
		Elevation: 116,
		Latitude:  49.9132995605,
		Longitude: -6.2916698456,
		Timezone:  "Europe/London",
	},
	"EGHF": {
		ICAO:      "EGHF",
		Name:      "RNAS Lee-On-Solent",
		City:      "Lee-on-Solent",
		State:     "England",
		Country:   "GB",
		Elevation: 32,
		Latitude:  50.814201355,
		Longitude: -1.20333004,
		Timezone:  "Europe/London",
	},
	"EGHG": {
		ICAO:      "EGHG",
		Name:      "Yeovil/Westland Aerodrome",
		City:      "Yeovil",
		State:     "England",
		Country:   "GB",
		Elevation: 202,
		Latitude:  50.9399986267,
		Longitude: -2.6586101055,
		Timezone:  "Europe/London",
	},
	"EGHH": {
		ICAO:      "EGHH",
		IATA:      "BOH",
		Name:      "Bournemouth Airport",
		City:      "Bournemouth",
		State:     "England",
		Country:   "GB",
		Elevation: 38,
		Latitude:  50.7799987793,
		Longitude: -1.8424999714,
		Timezone:  "Europe/London",
	},
	"EGHI": {
		ICAO:      "EGHI",
		IATA:      "SOU",
		Name:      "Southampton Airport",
		City:      "Southampton",
		State:     "England",
		Country:   "GB",
		Elevation: 44,
		Latitude:  50.9502983093,
		Longitude: -1.3567999601,
		Timezone:  "Europe/London",
	},
	"EGHJ": {
		ICAO:      "EGHJ",
		IATA:      "BBP",
		Name:      "Bembridge Airport",
		City:      "Bembridge",
		State:     "England",
		Country:   "GB",
		Elevation: 53,
		Latitude:  50.6781005859,
		Longitude: -1.1094399691,
		Timezone:  "Europe/London",
	},
	"EGHL": {
		ICAO:      "EGHL",
		IATA:      "QLA",
		Name:      "Lasham Airport",
		City:      "Lasham",
		State:     "England",
		Country:   "GB",
		Elevation: 618,
		Latitude:  51.1871986389,
		Longitude: -1.0334999561,
		Timezone:  "Europe/London",
	},
	"EGHN": {
		ICAO:      "EGHN",
		Name:      "Isle of Wight / Sandown Airport",
		City:      "Sandown",
		State:     "England",
		Country:   "GB",
		Elevation: 55,
		Latitude:  50.6530990601,
		Longitude: -1.1822199821,
		Timezone:  "Europe/London",
	},
	"EGHO": {
		ICAO:      "EGHO",
		Name:      "Thruxton Aerodrome",
		City:      "Andover",
		State:     "England",
		Country:   "GB",
		Elevation: 319,
		Latitude:  51.2106018066,
		Longitude: -1.6000000238,
		Timezone:  "Europe/London",
	},
	"EGHP": {
		ICAO:      "EGHP",
		Name:      "Popham Airport",
		City:      "Basingstoke",
		State:     "England",
		Country:   "GB",
		Elevation: 550,
		Latitude:  51.193901062,
		Longitude: -1.2347199917,
		Timezone:  "Europe/London",
	},
	"EGHQ": {
		ICAO:      "EGHQ",
		IATA:      "NQY",
		Name:      "Newquay Cornwall Airport",
		City:      "Newquay",
		State:     "England",
		Country:   "GB",
		Elevation: 390,
		Latitude:  50.4406013489,
		Longitude: -4.9954099655,
		Timezone:  "Europe/London",
	},
	"EGHR": {
		ICAO:      "EGHR",
		IATA:      "QUG",
		Name:      "Chichester/Goodwood Airport",
		City:      "Chichester",
		State:     "England",
		Country:   "GB",
		Elevation: 98,
		Latitude:  50.8594017029,
		Longitude: -0.7591670156,
		Timezone:  "Europe/London",
	},
	"EGHS": {
		ICAO:      "EGHS",
		Name:      "Henstridge Airfield",
		City:      "Henstridge",
		State:     "England",
		Country:   "GB",
		Elevation: 184,
		Latitude:  50.9850006104,
		Longitude: -2.3572199345,
		Timezone:  "Europe/London",
	},
	"EGHU": {
		ICAO:      "EGHU",
		Name:      "Eaglescott Airfield",
		City:      "Burrington",
		State:     "England",
		Country:   "GB",
		Elevation: 655,
		Latitude:  50.9286003113,
		Longitude: -3.9894399643,
		Timezone:  "Europe/London",
	},
	"EGHY": {
		ICAO:      "EGHY",
		Name:      "Truro Airport",
		City:      "Truro",
		State:     "England",
		Country:   "GB",
		Elevation: 127,
		Latitude:  50.2784004211,
		Longitude: -5.1422901154,
		Timezone:  "Europe/London",
	},
	"EGJA": {
		ICAO:      "EGJA",
		IATA:      "ACI",
		Name:      "Alderney Airport",
		City:      "Saint Anne",
		State:     "Alderney",
		Country:   "GG",
		Elevation: 290,
		Latitude:  49.7061004639,
		Longitude: -2.2147200108,
		Timezone:  "Europe/Guernsey",
	},
	"EGJB": {
		ICAO:      "EGJB",
		IATA:      "GCI",
		Name:      "Guernsey Airport",
		City:      "Saint Peter Port",
		State:     "Forest",
		Country:   "GG",
		Elevation: 336,
		Latitude:  49.4350013733,
		Longitude: -2.6019699574,
		Timezone:  "Europe/Guernsey",
	},
	"EGJJ": {
		ICAO:      "EGJJ",
		IATA:      "JER",
		Name:      "Jersey Airport",
		City:      "Saint Helier",
		State:     "St-Peter",
		Country:   "JE",
		Elevation: 277,
		Latitude:  49.207901001,
		Longitude: -2.1955099106,
		Timezone:  "Europe/Jersey",
	},
	"EGKA": {
		ICAO:      "EGKA",
		IATA:      "ESH",
		Name:      "Shoreham Airport",
		City:      "Brighton",
		State:     "England",
		Country:   "GB",
		Elevation: 7,
		Latitude:  50.8356018066,
		Longitude: -0.2972219884,
		Timezone:  "Europe/London",
	},
	"EGKB": {
		ICAO:      "EGKB",
		IATA:      "BQH",
		Name:      "London Biggin Hill Airport",
		City:      "London",
		State:     "England",
		Country:   "GB",
		Elevation: 598,
		Latitude:  51.3307991028,
		Longitude: 0.0324999988,
		Timezone:  "Europe/London",
	},
	"EGKE": {
		ICAO:      "EGKE",
		Name:      "Challock Airfield",
		City:      "Challock",
		State:     "England",
		Country:   "GB",
		Elevation: 600,
		Latitude:  51.2083015442,
		Longitude: 0.8291670084,
		Timezone:  "Europe/London",
	},
	"EGKH": {
		ICAO:      "EGKH",
		Name:      "Lashenden (Headcorn) Airfield",
		City:      "Maidstone",
		State:     "England",
		Country:   "GB",
		Elevation: 72,
		Latitude:  51.1568984985,
		Longitude: 0.6416670084,
		Timezone:  "Europe/London",
	},
	"EGKK": {
		ICAO:      "EGKK",
		IATA:      "LGW",
		Name:      "London Gatwick Airport",
		City:      "London",
		State:     "England",
		Country:   "GB",
		Elevation: 202,
		Latitude:  51.1481018066,
		Longitude: -0.1902779937,
		Timezone:  "Europe/London",
	},
	"EGKL": {
		ICAO:      "EGKL",
		Name:      "Deanland Lewes Airport",
		City:      "Lewes",
		State:     "England",
		Country:   "GB",
		Elevation: 69,
		Latitude:  50.8788986206,
		Longitude: 0.156388998,
		Timezone:  "Europe/London",
	},
	"EGKR": {
		ICAO:      "EGKR",
		IATA:      "KRH",
		Name:      "Redhill Aerodrome",
		City:      "Redhill",
		State:     "England",
		Country:   "GB",
		Elevation: 222,
		Latitude:  51.2136001587,
		Longitude: -0.1386110038,
		Timezone:  "Europe/London",
	},
	"EGLA": {
		ICAO:      "EGLA",
		Name:      "Bodmin Airfield",
		City:      "Bodmin",
		State:     "England",
		Country:   "GB",
		Elevation: 650,
		Latitude:  50.4996986389,
		Longitude: -4.6658301353,
		Timezone:  "Europe/London",
	},
	"EGLC": {
		ICAO:      "EGLC",
		IATA:      "LCY",
		Name:      "London City Airport",
		City:      "London",
		State:     "England",
		Country:   "GB",
		Elevation: 19,
		Latitude:  51.5052986145,
		Longitude: 0.0552779995,
		Timezone:  "Europe/London",
	},
	"EGLD": {
		ICAO:      "EGLD",
		Name:      "Denham Aerodrome",
		City:      "Gerrards Cross",
		Country:   "GB",
		Elevation: 24,
		Latitude:  51.5882987976,
		Longitude: -0.5130559802,
		Timezone:  "Europe/London",
	},
	"EGLF": {
		ICAO:      "EGLF",
		IATA:      "FAB",
		Name:      "Farnborough Airport",
		City:      "Farnborough",
		State:     "England",
		Country:   "GB",
		Elevation: 238,
		Latitude:  51.2757987976,
		Longitude: -0.7763329744,
		Timezone:  "Europe/London",
	},
	"EGLG": {
		ICAO:      "EGLG",
		Name:      "Panshanger Aerodrome",
		City:      "Hertford",
		Country:   "GB",
		Elevation: 249,
		Latitude:  51.8025016785,
		Longitude: -0.1580560058,
		Timezone:  "Europe/London",
	},
	"EGLJ": {
		ICAO:      "EGLJ",
		Name:      "Chalgrove Airport",
		City:      "Chalgrove",
		Country:   "GB",
		Elevation: 240,
		Latitude:  51.6761016846,
		Longitude: -1.080829978,
		Timezone:  "Europe/London",
	},
	"EGLK": {
		ICAO:      "EGLK",
		IATA:      "BBS",
		Name:      "Blackbushe Airport",
		City:      "Yateley",
		State:     "England",
		Country:   "GB",
		Elevation: 325,
		Latitude:  51.3238983154,
		Longitude: -0.8475000262,
		Timezone:  "Europe/London",
	},
	"EGLL": {
		ICAO:      "EGLL",
		IATA:      "LHR",
		Name:      "London Heathrow Airport",
		City:      "London",
		State:     "England",
		Country:   "GB",
		Elevation: 83,
		Latitude:  51.4706001282,
		Longitude: -0.4619410038,
		Timezone:  "Europe/London",
	},
	"EGLM": {
		ICAO:      "EGLM",
		Name:      "White Waltham Airfield",
		City:      "Maidenhead",
		Country:   "GB",
		Elevation: 131,
		Latitude:  51.5008010864,
		Longitude: -0.774443984,
		Timezone:  "Europe/London",
	},
	"EGLP": {
		ICAO:      "EGLP",
		Name:      "Brimpton Airfield",
		Country:   "GB",
		Elevation: 210,
		Latitude:  51.383889,
		Longitude: -1.169167,
		Timezone:  "Europe/London",
	},
	"EGLS": {
		ICAO:      "EGLS",
		Name:      "Old Sarum Airfield",
		City:      "Salisbury",
		Country:   "GB",
		Elevation: 285,
		Latitude:  51.0988998413,
		Longitude: -1.7841700315,
		Timezone:  "Europe/London",
	},
	"EGMA": {
		ICAO:      "EGMA",
		Name:      "Fowlmere Airfield",
		City:      "Cambridge",
		Country:   "GB",
		Elevation: 124,
		Latitude:  52.0774993896,
		Longitude: 0.061666999,
		Timezone:  "Europe/London",
	},
	"EGMC": {
		ICAO:      "EGMC",
		IATA:      "SEN",
		Name:      "Southend Airport",
		City:      "Southend",
		State:     "England",
		Country:   "GB",
		Elevation: 49,
		Latitude:  51.5713996887,
		Longitude: 0.695555985,
		Timezone:  "Europe/London",
	},
	"EGMD": {
		ICAO:      "EGMD",
		IATA:      "LYX",
		Name:      "Lydd Airport",
		City:      "Lydd",
		State:     "England",
		Country:   "GB",
		Elevation: 13,
		Latitude:  50.9561004639,
		Longitude: 0.9391670227,
		Timezone:  "Europe/London",
	},
	"EGMF": {
		ICAO:      "EGMF",
		Name:      "Farthing Corner/Stoneacre Farm Airport",
		City:      "Bredhurst",
		State:     "England",
		Country:   "GB",
		Elevation: 420,
		Latitude:  51.3306007385,
		Longitude: 0.6011109948,
		Timezone:  "Europe/London",
	},
	"EGMH": {
		ICAO:      "EGMH",
		IATA:      "MSE",
		Name:      "Kent International Airport",
		City:      "Manston",
		State:     "England",
		Country:   "GB",
		Elevation: 178,
		Latitude:  51.3422012329,
		Longitude: 1.3461099863,
		Timezone:  "Europe/London",
	},
	"EGMJ": {
		ICAO:      "EGMJ",
		Name:      "Little Gransden Airfield",
		City:      "St. Neot's",
		Country:   "GB",
		Elevation: 250,
		Latitude:  52.1666984558,
		Longitude: -0.1538890004,
		Timezone:  "Europe/London",
	},
	"EGML": {
		ICAO:      "EGML",
		Name:      "Damyns Hall Aerodrome",
		City:      "Upminster",
		Country:   "GB",
		Elevation: 59,
		Latitude:  51.5285987854,
		Longitude: 0.2455559969,
		Timezone:  "Europe/London",
	},
	"EGNA": {
		ICAO:      "EGNA",
		Name:      "Hucknall Airfield",
		City:      "Nottingham",
		Country:   "GB",
		Elevation: 279,
		Latitude:  53.0144004822,
		Longitude: -1.2183300257,
		Timezone:  "Europe/London",
	},
	"EGNB": {
		ICAO:      "EGNB",
		Name:      "Brough Airport",
		City:      "Brough",
		State:     "England",
		Country:   "GB",
		Elevation: 12,
		Latitude:  53.7196998596,
		Longitude: -0.5663329959,
		Timezone:  "Europe/London",
	},
	"EGNC": {
		ICAO:      "EGNC",
		IATA:      "CAX",
		Name:      "Carlisle Airport",
		City:      "Carlisle",
		State:     "England",
		Country:   "GB",
		Elevation: 190,
		Latitude:  54.9375,
		Longitude: -2.8091700077,
		Timezone:  "Europe/London",
	},
	"EGND": {
		ICAO:      "EGND",
		Name:      "HUDDERSFIELD (Crosland Moor)",
		City:      "Huddersfield",
		State:     "England",
		Country:   "GB",
		Elevation: 825,
		Latitude:  53.6216666667,
		Longitude: -1.8302777778,
		Timezone:  "Europe/London",
	},
	"EGNE": {
		ICAO:      "EGNE",
		Name:      "Retford Gamston Airport",
		City:      "Gamston",
		State:     "England",
		Country:   "GB",
		Elevation: 91,
		Latitude:  53.2806015015,
		Longitude: -0.9513890147,
		Timezone:  "Europe/London",
	},
	"EGNF": {
		ICAO:      "EGNF",
		Name:      "Netherthorpe Airfield",
		City:      "Worksop",
		Country:   "GB",
		Elevation: 250,
		Latitude:  53.3168983459,
		Longitude: -1.1963900328,
		Timezone:  "Europe/London",
	},
	"EGNG": {
		ICAO:      "EGNG",
		Name:      "Bagby Thirsk Prv Airport",
		City:      "Bagby",
		State:     "England",
		Country:   "GB",
		Elevation: 161,
		Latitude:  54.211101532,
		Longitude: -1.2899999619,
		Timezone:  "Europe/London",
	},
	"EGNH": {
		ICAO:      "EGNH",
		IATA:      "BLK",
		Name:      "Blackpool International Airport",
		City:      "Blackpool",
		State:     "England",
		Country:   "GB",
		Elevation: 34,
		Latitude:  53.7717018127,
		Longitude: -3.0286099911,
		Timezone:  "Europe/London",
	},
	"EGNI": {
		ICAO:      "EGNI",
		Name:      "Skegness (Ingoldmells) Aerodrome",
		City:      "Skegness",
		Country:   "GB",
		Elevation: 8,
		Latitude:  53.1716003418,
		Longitude: 0.3293749988,
		Timezone:  "Europe/London",
	},
	"EGNJ": {
		ICAO:      "EGNJ",
		IATA:      "HUY",
		Name:      "Humberside Airport",
		City:      "Grimsby",
		State:     "England",
		Country:   "GB",
		Elevation: 121,
		Latitude:  53.5744018555,
		Longitude: -0.3508329988,
		Timezone:  "Europe/London",
	},
	"EGNL": {
		ICAO:      "EGNL",
		IATA:      "BWF",
		Name:      "Barrow Walney Island Airport",
		City:      "Barrow-in-Furness",
		State:     "England",
		Country:   "GB",
		Elevation: 173,
		Latitude:  54.1311988831,
		Longitude: -3.2636699677,
		Timezone:  "Europe/London",
	},
	"EGNM": {
		ICAO:      "EGNM",
		IATA:      "LBA",
		Name:      "Leeds Bradford Airport",
		City:      "Leeds",
		State:     "England",
		Country:   "GB",
		Elevation: 681,
		Latitude:  53.8658981323,
		Longitude: -1.6605700254,
		Timezone:  "Europe/London",
	},
	"EGNO": {
		ICAO:      "EGNO",
		IATA:      "WRT",
		Name:      "Warton Airport",
		City:      "Warton",
		State:     "England",
		Country:   "GB",
		Elevation: 55,
		Latitude:  53.745098114,
		Longitude: -2.8830599785,
		Timezone:  "Europe/London",
	},
	"EGNR": {
		ICAO:      "EGNR",
		IATA:      "CEG",
		Name:      "Hawarden Airport",
		City:      "Hawarden",
		State:     "Wales",
		Country:   "GB",
		Elevation: 45,
		Latitude:  53.1781005859,
		Longitude: -2.9777801037,
		Timezone:  "Europe/London",
	},
	"EGNS": {
		ICAO:      "EGNS",
		IATA:      "IOM",
		Name:      "Isle of Man Airport",
		City:      "Castletown",
		State:     "Castletown",
		Country:   "IM",
		Elevation: 52,
		Latitude:  54.0833015442,
		Longitude: -4.6238899231,
		Timezone:  "Europe/Isle_of_Man",
	},
	"EGNT": {
		ICAO:      "EGNT",
		IATA:      "NCL",
		Name:      "Newcastle Airport",
		City:      "Newcastle",
		State:     "England",
		Country:   "GB",
		Elevation: 266,
		Latitude:  55.0374984741,
		Longitude: -1.6916699409,
		Timezone:  "Europe/London",
	},
	"EGNV": {
		ICAO:      "EGNV",
		IATA:      "MME",
		Name:      "Durham Tees Valley Airport",
		City:      "Durham",
		State:     "England",
		Country:   "GB",
		Elevation: 120,
		Latitude:  54.5092010498,
		Longitude: -1.4294099808,
		Timezone:  "Europe/London",
	},
	"EGNW": {
		ICAO:      "EGNW",
		Name:      "Wickenby Aerodrome",
		City:      "Lincoln",
		State:     "England",
		Country:   "GB",
		Elevation: 63,
		Latitude:  53.3166999817,
		Longitude: -0.349721998,
		Timezone:  "Europe/London",
	},
	"EGNX": {
		ICAO:      "EGNX",
		IATA:      "EMA",
		Name:      "East Midlands Airport",
		City:      "Nottingham",
		State:     "England",
		Country:   "GB",
		Elevation: 306,
		Latitude:  52.8311004639,
		Longitude: -1.3280600309,
		Timezone:  "Europe/London",
	},
	"EGNY": {
		ICAO:      "EGNY",
		Name:      "Beverley/Linley Hill Airport",
		City:      "Beverley",
		State:     "England",
		Country:   "GB",
		Elevation: 3,
		Latitude:  53.8983001709,
		Longitude: -0.3613890111,
		Timezone:  "Europe/London",
	},
	"EGOD": {
		ICAO:      "EGOD",
		Name:      "Llanbedr Airport",
		City:      "Llanbedr",
		State:     "Wales",
		Country:   "GB",
		Elevation: 30,
		Latitude:  52.8116989136,
		Longitude: -4.1235799789,
		Timezone:  "Europe/London",
	},
	"EGOE": {
		ICAO:      "EGOE",
		Name:      "RAF Ternhill",
		City:      "Ternhill",
		Country:   "GB",
		Elevation: 272,
		Latitude:  52.8712005615,
		Longitude: -2.5335600376,
		Timezone:  "Europe/London",
	},
	"EGOM": {
		ICAO:      "EGOM",
		Name:      "RAF Spadeadam",
		City:      "Spadeadam",
		State:     "England",
		Country:   "GB",
		Elevation: 1066,
		Latitude:  55.0499992371,
		Longitude: -2.5499999523,
		Timezone:  "Europe/London",
	},
	"EGOQ": {
		ICAO:      "EGOQ",
		Name:      "RAF Mona",
		City:      "Holyhead Island",
		State:     "Wales",
		Country:   "GB",
		Elevation: 202,
		Latitude:  53.2585983276,
		Longitude: -4.3735499382,
		Timezone:  "Europe/London",
	},
	"EGOS": {
		ICAO:      "EGOS",
		Name:      "RAF Shawbury",
		City:      "Shawbury",
		State:     "England",
		Country:   "GB",
		Elevation: 249,
		Latitude:  52.7981987,
		Longitude: -2.6680400372,
		Timezone:  "Europe/London",
	},
	"EGOV": {
		ICAO:      "EGOV",
		IATA:      "HLY",
		Name:      "Anglesey Airport",
		City:      "Angelsey",
		State:     "Wales",
		Country:   "GB",
		Elevation: 37,
		Latitude:  53.2481002808,
		Longitude: -4.5353398323,
		Timezone:  "Europe/London",
	},
	"EGOW": {
		ICAO:      "EGOW",
		Name:      "RAF Woodvale",
		City:      "Southport",
		State:     "England",
		Country:   "GB",
		Elevation: 37,
		Latitude:  53.5816001892,
		Longitude: -3.0555200577,
		Timezone:  "Europe/London",
	},
	"EGOY": {
		ICAO:      "EGOY",
		Name:      "RAF West Freugh",
		City:      "Wigtownshire",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 55,
		Latitude:  54.8511009216,
		Longitude: -4.9477801323,
		Timezone:  "Europe/London",
	},
	"EGPA": {
		ICAO:      "EGPA",
		IATA:      "KOI",
		Name:      "Kirkwall Airport",
		City:      "Orkney Islands",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 50,
		Latitude:  58.9578018188,
		Longitude: -2.9049999714,
		Timezone:  "Europe/London",
	},
	"EGPB": {
		ICAO:      "EGPB",
		IATA:      "LSI",
		Name:      "Sumburgh Airport",
		City:      "Lerwick",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 20,
		Latitude:  59.8788986206,
		Longitude: -1.2955600023,
		Timezone:  "Europe/London",
	},
	"EGPC": {
		ICAO:      "EGPC",
		IATA:      "WIC",
		Name:      "Wick Airport",
		City:      "Wick",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 126,
		Latitude:  58.4589004517,
		Longitude: -3.0930600166,
		Timezone:  "Europe/London",
	},
	"EGPD": {
		ICAO:      "EGPD",
		IATA:      "ABZ",
		Name:      "Aberdeen Dyce Airport",
		City:      "Aberdeen",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 215,
		Latitude:  57.2019004822,
		Longitude: -2.1977798939,
		Timezone:  "Europe/London",
	},
	"EGPE": {
		ICAO:      "EGPE",
		IATA:      "INV",
		Name:      "Inverness Airport",
		City:      "Inverness",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 31,
		Latitude:  57.5424995422,
		Longitude: -4.0475001335,
		Timezone:  "Europe/London",
	},
	"EGPF": {
		ICAO:      "EGPF",
		IATA:      "GLA",
		Name:      "Glasgow International Airport",
		City:      "Glasgow",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 26,
		Latitude:  55.8718986511,
		Longitude: -4.4330601692,
		Timezone:  "Europe/London",
	},
	"EGPG": {
		ICAO:      "EGPG",
		Name:      "Cumbernauld Airport",
		City:      "Cumbernauld",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 350,
		Latitude:  55.9747009277,
		Longitude: -3.9755599499,
		Timezone:  "Europe/London",
	},
	"EGPH": {
		ICAO:      "EGPH",
		IATA:      "EDI",
		Name:      "Edinburgh Airport",
		City:      "Edinburgh",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 135,
		Latitude:  55.9500007629,
		Longitude: -3.3724999428,
		Timezone:  "Europe/London",
	},
	"EGPI": {
		ICAO:      "EGPI",
		IATA:      "ILY",
		Name:      "Islay Airport",
		City:      "Port Ellen",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 56,
		Latitude:  55.6819000244,
		Longitude: -6.2566699982,
		Timezone:  "Europe/London",
	},
	"EGPJ": {
		ICAO:      "EGPJ",
		Name:      "Fife Airport",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 339,
		Latitude:  56.1833000183,
		Longitude: -3.220279932,
		Timezone:  "Europe/London",
	},
	"EGPK": {
		ICAO:      "EGPK",
		IATA:      "PIK",
		Name:      "Glasgow Prestwick Airport",
		City:      "Glasgow",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 65,
		Latitude:  55.5093994141,
		Longitude: -4.5866699219,
		Timezone:  "Europe/London",
	},
	"EGPL": {
		ICAO:      "EGPL",
		IATA:      "BEB",
		Name:      "Benbecula Airport",
		City:      "Balivanich",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 19,
		Latitude:  57.481098175,
		Longitude: -7.3627800941,
		Timezone:  "Europe/London",
	},
	"EGPM": {
		ICAO:      "EGPM",
		IATA:      "SCS",
		Name:      "Scatsta Airport",
		City:      "Shetland Islands",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 81,
		Latitude:  60.432800293,
		Longitude: -1.296110034,
		Timezone:  "Europe/London",
	},
	"EGPN": {
		ICAO:      "EGPN",
		IATA:      "DND",
		Name:      "Dundee Airport",
		City:      "Dundee",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 17,
		Latitude:  56.4524993896,
		Longitude: -3.0258300304,
		Timezone:  "Europe/London",
	},
	"EGPO": {
		ICAO:      "EGPO",
		IATA:      "SYY",
		Name:      "Stornoway Airport",
		City:      "Stornoway",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 26,
		Latitude:  58.2155990601,
		Longitude: -6.3311100006,
		Timezone:  "Europe/London",
	},
	"EGPR": {
		ICAO:      "EGPR",
		IATA:      "BRR",
		Name:      "Barra Airport",
		City:      "Eoligarry",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 5,
		Latitude:  57.0228004456,
		Longitude: -7.4430599213,
		Timezone:  "Europe/London",
	},
	"EGPT": {
		ICAO:      "EGPT",
		IATA:      "PSL",
		Name:      "Perth/Scone Airport",
		City:      "Perth",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 397,
		Latitude:  56.439201355,
		Longitude: -3.3722200394,
		Timezone:  "Europe/London",
	},
	"EGPU": {
		ICAO:      "EGPU",
		IATA:      "TRE",
		Name:      "Tiree Airport",
		City:      "Balemartine",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 38,
		Latitude:  56.4991989136,
		Longitude: -6.8691701889,
		Timezone:  "Europe/London",
	},
	"EGPW": {
		ICAO:      "EGPW",
		IATA:      "UNT",
		Name:      "Unst Airport",
		City:      "Shetland Islands",
		State:     "Scotland",
		Country:   "GB",
		Latitude:  60.7472000122,
		Longitude: -0.8538500071,
		Timezone:  "Europe/London",
	},
	"EGPY": {
		ICAO:      "EGPY",
		Name:      "Dounreay/Thurso Airport",
		City:      "Thurso",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 22,
		Latitude:  58.5839004517,
		Longitude: -3.7269399166,
		Timezone:  "Europe/London",
	},
	"EGQB": {
		ICAO:      "EGQB",
		IATA:      "BOL",
		Name:      "Ballykelly Airport",
		City:      "Ballykelly",
		State:     "Northern-Ireland",
		Country:   "GB",
		Elevation: 85,
		Latitude:  55.0602989197,
		Longitude: -7.0202798843,
		Timezone:  "Europe/London",
	},
	"EGQK": {
		ICAO:      "EGQK",
		IATA:      "FSS",
		Name:      "RAF Kinloss",
		City:      "Kinloss",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 22,
		Latitude:  57.6493988037,
		Longitude: -3.5606400967,
		Timezone:  "Europe/London",
	},
	"EGQL": {
		ICAO:      "EGQL",
		IATA:      "ADX",
		Name:      "RAF Leuchars",
		City:      "St. Andrews",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 38,
		Latitude:  56.3728981018,
		Longitude: -2.8684399128,
		Timezone:  "Europe/London",
	},
	"EGQS": {
		ICAO:      "EGQS",
		IATA:      "LMO",
		Name:      "RAF Lossiemouth",
		City:      "Lossiemouth",
		State:     "Scotland",
		Country:   "GB",
		Elevation: 42,
		Latitude:  57.7052001953,
		Longitude: -3.3391699791,
		Timezone:  "Europe/London",
	},
	"EGSA": {
		ICAO:      "EGSA",
		Name:      "Shipdham Airfield",
		City:      "East Dereham",
		State:     "England",
		Country:   "GB",
		Elevation: 210,
		Latitude:  52.6293983459,
		Longitude: 0.9280560017,
		Timezone:  "Europe/London",
	},
	"EGSC": {
		ICAO:      "EGSC",
		IATA:      "CBG",
		Name:      "Cambridge Airport",
		City:      "Cambridge",
		State:     "England",
		Country:   "GB",
		Elevation: 47,
		Latitude:  52.2050018311,
		Longitude: 0.174999997,
		Timezone:  "Europe/London",
	},
	"EGSF": {
		ICAO:      "EGSF",
		Name:      "Peterborough Business Airport",
		City:      "Conington",
		State:     "England",
		Country:   "GB",
		Elevation: 26,
		Latitude:  52.4681015015,
		Longitude: -0.2511110008,
		Timezone:  "Europe/London",
	},
	"EGSG": {
		ICAO:      "EGSG",
		Name:      "Stapleford Aerodrome",
		City:      "Stapleford Tawney",
		State:     "England",
		Country:   "GB",
		Elevation: 183,
		Latitude:  51.6525001526,
		Longitude: 0.1558330059,
		Timezone:  "Europe/London",
	},
	"EGSH": {
		ICAO:      "EGSH",
		IATA:      "NWI",
		Name:      "Norwich International Airport",
		City:      "Norwich",
		State:     "England",
		Country:   "GB",
		Elevation: 117,
		Latitude:  52.6758003235,
		Longitude: 1.2827800512,
		Timezone:  "Europe/London",
	},
	"EGSI": {
		ICAO:      "EGSI",
		Name:      "Mashland Airfield",
		City:      "Wisbech",
		State:     "England",
		Country:   "GB",
		Elevation: -6,
		Latitude:  52.6404853896,
		Longitude: 0.2918350697,
		Timezone:  "Europe/London",
	},
	"EGSJ": {
		ICAO:      "EGSJ",
		Name:      "Seething Airfield",
		City:      "Seething",
		State:     "England",
		Country:   "GB",
		Elevation: 130,
		Latitude:  52.511100769,
		Longitude: 1.4172199965,
		Timezone:  "Europe/London",
	},
	"EGSL": {
		ICAO:      "EGSL",
		Name:      "Andrewsfield Airport",
		City:      "Braintree",
		State:     "England",
		Country:   "GB",
		Elevation: 286,
		Latitude:  51.8950004578,
		Longitude: 0.4491670132,
		Timezone:  "Europe/London",
	},
	"EGSM": {
		ICAO:      "EGSM",
		Name:      "Beccles Airport",
		City:      "Beccles",
		State:     "England",
		Country:   "GB",
		Elevation: 80,
		Latitude:  52.4352989197,
		Longitude: 1.6183300018,
		Timezone:  "Europe/London",
	},
	"EGSN": {
		ICAO:      "EGSN",
		Name:      "Bourn Airport",
		City:      "Cambridge",
		Country:   "GB",
		Elevation: 226,
		Latitude:  52.2106018066,
		Longitude: -0.0425000004,
		Timezone:  "Europe/London",
	},
	"EGSO": {
		ICAO:      "EGSO",
		Name:      "Crowfield Airfield",
		City:      "Stowmarket",
		State:     "England",
		Country:   "GB",
		Elevation: 201,
		Latitude:  52.1711006165,
		Longitude: 1.111109972,
		Timezone:  "Europe/London",
	},
	"EGSP": {
		ICAO:      "EGSP",
		Name:      "Peterborough/Sibson Airport",
		City:      "Peterborough",
		State:     "England",
		Country:   "GB",
		Elevation: 130,
		Latitude:  52.5558013916,
		Longitude: -0.3863889873,
		Timezone:  "Europe/London",
	},
	"EGSQ": {
		ICAO:      "EGSQ",
		Name:      "Clacton Airport",
		City:      "Clacton-on-Sea",
		State:     "England",
		Country:   "GB",
		Elevation: 37,
		Latitude:  51.7849998474,
		Longitude: 1.1299999952,
		Timezone:  "Europe/London",
	},
	"EGSR": {
		ICAO:      "EGSR",
		Name:      "Earls Colne Airfield",
		City:      "Earles Colne",
		State:     "England",
		Country:   "GB",
		Elevation: 226,
		Latitude:  51.9143981934,
		Longitude: 0.6825000048,
		Timezone:  "Europe/London",
	},
	"EGSS": {
		ICAO:      "EGSS",
		IATA:      "STN",
		Name:      "London Stansted Airport",
		City:      "London",
		State:     "England",
		Country:   "GB",
		Elevation: 348,
		Latitude:  51.8849983215,
		Longitude: 0.2349999994,
		Timezone:  "Europe/London",
	},
	"EGST": {
		ICAO:      "EGST",
		Name:      "Elmsett Airport",
		City:      "Hadleigh",
		State:     "England",
		Country:   "GB",
		Elevation: 246,
		Latitude:  52.0772018433,
		Longitude: 0.9775000215,
		Timezone:  "Europe/London",
	},
	"EGSU": {
		ICAO:      "EGSU",
		IATA:      "QFO",
		Name:      "Duxford Airport",
		City:      "Duxford",
		State:     "England",
		Country:   "GB",
		Elevation: 125,
		Latitude:  52.090801239,
		Longitude: 0.1319440007,
		Timezone:  "Europe/London",
	},
	"EGSV": {
		ICAO:      "EGSV",
		Name:      "Old Buckenham Airport",
		City:      "Old Buckenham",
		State:     "England",
		Country:   "GB",
		Elevation: 185,
		Latitude:  52.4975013733,
		Longitude: 1.0519399643,
		Timezone:  "Europe/London",
	},
	"EGSW": {
		ICAO:      "EGSW",
		Name:      "Newmarket Heath Airfield",
		City:      "Newmarket",
		Country:   "GB",
		Elevation: 100,
		Latitude:  52.2419013977,
		Longitude: 0.3705559969,
		Timezone:  "Europe/London",
	},
	"EGSX": {
		ICAO:      "EGSX",
		Name:      "North Weald Airport",
		City:      "North Weald Bassett",
		State:     "England",
		Country:   "GB",
		Elevation: 321,
		Latitude:  51.721698761,
		Longitude: 0.1541669965,
		Timezone:  "Europe/London",
	},
	"EGTA": {
		ICAO:      "EGTA",
		Name:      "Aylesbury/Thame Airport",
		City:      "Aylesbury",
		Country:   "GB",
		Elevation: 289,
		Latitude:  51.7757987976,
		Longitude: -0.9402779937,
		Timezone:  "Europe/London",
	},
	"EGTB": {
		ICAO:      "EGTB",
		IATA:      "HYC",
		Name:      "Wycombe Air Park",
		City:      "High Wycombe",
		State:     "England",
		Country:   "GB",
		Elevation: 515,
		Latitude:  51.6116981506,
		Longitude: -0.8083329797,
		Timezone:  "Europe/London",
	},
	"EGTC": {
		ICAO:      "EGTC",
		Name:      "Cranfield Airport",
		City:      "Cranfield",
		State:     "England",
		Country:   "GB",
		Elevation: 358,
		Latitude:  52.0722007751,
		Longitude: -0.6166669726,
		Timezone:  "Europe/London",
	},
	"EGTD": {
		ICAO:      "EGTD",
		Name:      "Dunsfold Aerodrome",
		City:      "Dunsfold",
		State:     "England",
		Country:   "GB",
		Elevation: 172,
		Latitude:  51.1171989441,
		Longitude: -0.5358330011,
		Timezone:  "Europe/London",
	},
	"EGTE": {
		ICAO:      "EGTE",
		IATA:      "EXT",
		Name:      "Exeter International Airport",
		City:      "Exeter",
		State:     "England",
		Country:   "GB",
		Elevation: 102,
		Latitude:  50.7344017029,
		Longitude: -3.4138898849,
		Timezone:  "Europe/London",
	},
	"EGTF": {
		ICAO:      "EGTF",
		Name:      "Fairoaks Airport",
		City:      "Woking",
		State:     "England",
		Country:   "GB",
		Elevation: 80,
		Latitude:  51.3480987549,
		Longitude: -0.5588889718,
		Timezone:  "Europe/London",
	},
	"EGTG": {
		ICAO:      "EGTG",
		IATA:      "FZO",
		Name:      "Bristol Filton Airport",
		City:      "Bristol",
		State:     "England",
		Country:   "GB",
		Elevation: 226,
		Latitude:  51.5194015503,
		Longitude: -2.5908300877,
		Timezone:  "Europe/London",
	},
	"EGTH": {
		ICAO:      "EGTH",
		Name:      "Old Warden Airfield",
		City:      "Old Warden",
		Country:   "GB",
		Elevation: 110,
		Latitude:  52.0834007263,
		Longitude: -0.3166669905,
		Timezone:  "Europe/London",
	},
	"EGTK": {
		ICAO:      "EGTK",
		IATA:      "OXF",
		Name:      "Oxford (Kidlington) Airport",
		City:      "Kidlington",
		State:     "England",
		Country:   "GB",
		Elevation: 270,
		Latitude:  51.8368988037,
		Longitude: -1.3200000525,
		Timezone:  "Europe/London",
	},
	"EGTN": {
		ICAO:      "EGTN",
		Name:      "Enstone Aerodrome",
		City:      "Oxfordshire",
		State:     "England",
		Country:   "GB",
		Elevation: 550,
		Latitude:  51.9281666667,
		Longitude: -1.4285,
		Timezone:  "Europe/London",
	},
	"EGTO": {
		ICAO:      "EGTO",
		IATA:      "RCS",
		Name:      "Rochester Airport",
		City:      "Rochester",
		State:     "England",
		Country:   "GB",
		Elevation: 436,
		Latitude:  51.3518981934,
		Longitude: 0.5033329725,
		Timezone:  "Europe/London",
	},
	"EGTP": {
		ICAO:      "EGTP",
		Name:      "Perranporth Airfield",
		City:      "Perranporth",
		State:     "England",
		Country:   "GB",
		Elevation: 330,
		Latitude:  50.3316993713,
		Longitude: -5.1774997711,
		Timezone:  "Europe/London",
	},
	"EGTR": {
		ICAO:      "EGTR",
		Name:      "Elstree Airfield",
		City:      "Watford",
		State:     "England",
		Country:   "GB",
		Elevation: 332,
		Latitude:  51.6557998657,
		Longitude: -0.3258329928,
		Timezone:  "Europe/London",
	},
	"EGTU": {
		ICAO:      "EGTU",
		Name:      "Dunkeswell Airport",
		City:      "Dunkeswell",
		State:     "England",
		Country:   "GB",
		Elevation: 850,
		Latitude:  50.8600006104,
		Longitude: -3.2347199917,
		Timezone:  "Europe/London",
	},
	"EGTW": {
		ICAO:      "EGTW",
		Name:      "Oaksey Park Airport",
		City:      "Malmesbury",
		Country:   "GB",
		Elevation: 250,
		Latitude:  51.6321983337,
		Longitude: -2.0147199631,
		Timezone:  "Europe/London",
	},
	"EGUB": {
		ICAO:      "EGUB",
		IATA:      "BEX",
		Name:      "RAF Benson",
		City:      "Benson",
		State:     "England",
		Country:   "GB",
		Elevation: 226,
		Latitude:  51.6164016724,
		Longitude: -1.0958299637,
		Timezone:  "Europe/London",
	},
	"EGUD": {
		ICAO:      "EGUD",
		Name:      "RAF Abingdon",
		City:      "Abingdon",
		State:     "England",
		Country:   "GB",
		Elevation: 259,
		Latitude:  51.6902999878,
		Longitude: -1.316669941,
		Timezone:  "Europe/London",
	},
	"EGUK": {
		ICAO:      "EGUK",
		Name:      "Waterbeach Airfield",
		City:      "Waterbeach",
		Country:   "GB",
		Elevation: 19,
		Latitude:  52.281742,
		Longitude: 0.184532,
		Timezone:  "Europe/London",
	},
	"EGUL": {
		ICAO:      "EGUL",
		IATA:      "LKZ",
		Name:      "RAF Lakenheath",
		City:      "Lakenheath",
		State:     "England",
		Country:   "GB",
		Elevation: 32,
		Latitude:  52.4093017578,
		Longitude: 0.5609999895,
		Timezone:  "Europe/London",
	},
	"EGUN": {
		ICAO:      "EGUN",
		IATA:      "MHZ",
		Name:      "RAF Mildenhall",
		City:      "Mildenhall",
		State:     "England",
		Country:   "GB",
		Elevation: 33,
		Latitude:  52.3619003296,
		Longitude: 0.4864059985,
		Timezone:  "Europe/London",
	},
	"EGUO": {
		ICAO:      "EGUO",
		Name:      "Colerne Airport",
		City:      "Colerne",
		State:     "England",
		Country:   "GB",
		Elevation: 593,
		Latitude:  51.4390983582,
		Longitude: -2.2863900662,
		Timezone:  "Europe/London",
	},
	"EGUP": {
		ICAO:      "EGUP",
		Name:      "RAF Sculthorpe",
		City:      "Fakeham",
		State:     "England",
		Country:   "GB",
		Elevation: 213,
		Latitude:  52.846698761,
		Longitude: 0.7663890123,
		Timezone:  "Europe/London",
	},
	"EGUW": {
		ICAO:      "EGUW",
		Name:      "RAF Wattisham",
		City:      "Stowmarket",
		State:     "England",
		Country:   "GB",
		Elevation: 284,
		Latitude:  52.1273002625,
		Longitude: 0.956264019,
		Timezone:  "Europe/London",
	},
	"EGUY": {
		ICAO:      "EGUY",
		IATA:      "QUY",
		Name:      "RAF Wyton",
		City:      "St. Ives",
		State:     "England",
		Country:   "GB",
		Elevation: 135,
		Latitude:  52.3572006226,
		Longitude: -0.107832998,
		Timezone:  "Europe/London",
	},
	"EGVA": {
		ICAO:      "EGVA",
		IATA:      "FFD",
		Name:      "RAF Fairford",
		City:      "Fairford",
		State:     "England",
		Country:   "GB",
		Elevation: 286,
		Latitude:  51.6822013855,
		Longitude: -1.7900300026,
		Timezone:  "Europe/London",
	},
	"EGVN": {
		ICAO:      "EGVN",
		IATA:      "BZZ",
		Name:      "RAF Brize Norton",
		City:      "Brize Norton",
		State:     "England",
		Country:   "GB",
		Elevation: 288,
		Latitude:  51.75,
		Longitude: -1.5836199522,
		Timezone:  "Europe/London",
	},
	"EGVO": {
		ICAO:      "EGVO",
		IATA:      "ODH",
		Name:      "RAF Odiham",
		City:      "Odiham",
		State:     "England",
		Country:   "GB",
		Elevation: 405,
		Latitude:  51.2341003418,
		Longitude: -0.9428250194,
		Timezone:  "Europe/London",
	},
	"EGVP": {
		ICAO:      "EGVP",
		Name:      "Middle Wallop Airfield",
		City:      "Middle Wallop",
		State:     "England",
		Country:   "GB",
		Elevation: 297,
		Latitude:  51.1394004822,
		Longitude: -1.5686099529,
		Timezone:  "Europe/London",
	},
	"EGVT": {
		ICAO:      "EGVT",
		IATA:      "WXF",
		Name:      "Wethersfield Airport",
		City:      "Wethersfield",
		State:     "England",
		Country:   "GB",
		Elevation: 322,
		Latitude:  51.9719009399,
		Longitude: 0.5094439983,
		Timezone:  "Europe/London",
	},
	"EGWC": {
		ICAO:      "EGWC",
		Name:      "DCAE Cosford Airport",
		City:      "Cosford",
		State:     "England",
		Country:   "GB",
		Elevation: 272,
		Latitude:  52.6399993896,
		Longitude: -2.3055799007,
		Timezone:  "Europe/London",
	},
	"EGWE": {
		ICAO:      "EGWE",
		Name:      "RAF Henlow",
		City:      "Henlow",
		Country:   "GB",
		Elevation: 206,
		Latitude:  52.0181999207,
		Longitude: -0.3038499951,
		Timezone:  "Europe/London",
	},
	"EGWN": {
		ICAO:      "EGWN",
		Name:      "RAF Halton",
		City:      "Halton",
		Country:   "GB",
		Elevation: 370,
		Latitude:  51.7906990051,
		Longitude: -0.7379720211,
		Timezone:  "Europe/London",
	},
	"EGWU": {
		ICAO:      "EGWU",
		IATA:      "NHT",
		Name:      "RAF Northolt",
		City:      "London",
		State:     "England",
		Country:   "GB",
		Elevation: 124,
		Latitude:  51.5530014038,
		Longitude: -0.418166995,
		Timezone:  "Europe/London",
	},
	"EGXC": {
		ICAO:      "EGXC",
		IATA:      "QCY",
		Name:      "RAF Coningsby",
		City:      "Coningsby",
		State:     "England",
		Country:   "GB",
		Elevation: 25,
		Latitude:  53.0929985046,
		Longitude: -0.1660140008,
		Timezone:  "Europe/London",
	},
	"EGXD": {
		ICAO:      "EGXD",
		Name:      "RAF Dishforth",
		City:      "Dishforth",
		State:     "England",
		Country:   "GB",
		Elevation: 117,
		Latitude:  54.1371994019,
		Longitude: -1.4202500582,
		Timezone:  "Europe/London",
	},
	"EGXE": {
		ICAO:      "EGXE",
		Name:      "Leeming Airport",
		City:      "Leeming",
		State:     "England",
		Country:   "GB",
		Elevation: 132,
		Latitude:  54.2924003601,
		Longitude: -1.535400033,
		Timezone:  "Europe/London",
	},
	"EGXG": {
		ICAO:      "EGXG",
		Name:      "RAF Church Fenton",
		City:      "Church Fenton",
		State:     "England",
		Country:   "GB",
		Elevation: 29,
		Latitude:  53.8343009949,
		Longitude: -1.1955000162,
		Timezone:  "Europe/London",
	},
	"EGXH": {
		ICAO:      "EGXH",
		IATA:      "BEQ",
		Name:      "RAF Honington",
		City:      "Thetford",
		State:     "England",
		Country:   "GB",
		Elevation: 174,
		Latitude:  52.3426017761,
		Longitude: 0.7729390264,
		Timezone:  "Europe/London",
	},
	"EGXJ": {
		ICAO:      "EGXJ",
		IATA:      "OKH",
		Name:      "RAF Cottesmore",
		City:      "Cottesmore",
		State:     "England",
		Country:   "GB",
		Elevation: 461,
		Latitude:  52.7356987,
		Longitude: -0.648769021,
		Timezone:  "Europe/London",
	},
	"EGXP": {
		ICAO:      "EGXP",
		IATA:      "SQZ",
		Name:      "RAF Scampton",
		City:      "Scampton",
		State:     "England",
		Country:   "GB",
		Elevation: 202,
		Latitude:  53.307800293,
		Longitude: -0.5508329868,
		Timezone:  "Europe/London",
	},
	"EGXT": {
		ICAO:      "EGXT",
		Name:      "RAF Wittering",
		City:      "Wittering",
		State:     "England",
		Country:   "GB",
		Elevation: 273,
		Latitude:  52.6125984192,
		Longitude: -0.4764530063,
		Timezone:  "Europe/London",
	},
	"EGXU": {
		ICAO:      "EGXU",
		IATA:      "HRT",
		Name:      "RAF Linton-On-Ouse",
		City:      "Linton-On-Ouse",
		State:     "England",
		Country:   "GB",
		Elevation: 53,
		Latitude:  54.0489006042,
		Longitude: -1.2527500391,
		Timezone:  "Europe/London",
	},
	"EGXV": {
		ICAO:      "EGXV",
		Name:      "RAF Leconfield",
		City:      "Leconfield",
		State:     "England",
		Country:   "GB",
		Elevation: 36,
		Latitude:  53.8758010864,
		Longitude: -0.4350000024,
		Timezone:  "Europe/London",
	},
	"EGXW": {
		ICAO:      "EGXW",
		IATA:      "WTN",
		Name:      "RAF Waddington",
		City:      "Waddington",
		State:     "England",
		Country:   "GB",
		Elevation: 231,
		Latitude:  53.1661987305,
		Longitude: -0.5238109827,
		Timezone:  "Europe/London",
	},
	"EGXY": {
		ICAO:      "EGXY",
		Name:      "RAF Syerston",
		City:      "Newark",
		Country:   "GB",
		Elevation: 228,
		Latitude:  53.0228004456,
		Longitude: -0.9111109972,
		Timezone:  "Europe/London",
	},
	"EGXZ": {
		ICAO:      "EGXZ",
		Name:      "RAF Topcliffe",
		City:      "Topcliffe",
		State:     "England",
		Country:   "GB",
		Elevation: 92,
		Latitude:  54.2055015564,
		Longitude: -1.3820899725,
		Timezone:  "Europe/London",
	},
	"EGYD": {
		ICAO:      "EGYD",
		Name:      "RAF Cranwell",
		City:      "Cranwell",
		State:     "England",
		Country:   "GB",
		Elevation: 218,
		Latitude:  53.0303001404,
		Longitude: -0.4832420051,
		Timezone:  "Europe/London",
	},
	"EGYE": {
		ICAO:      "EGYE",
		Name:      "RAF Barkston Heath",
		City:      "Grantham",
		State:     "England",
		Country:   "GB",
		Elevation: 367,
		Latitude:  52.9622001648,
		Longitude: -0.5616250038,
		Timezone:  "Europe/London",
	},
	"EGYK": {
		ICAO:      "EGYK",
		Name:      "Elvington Airfield",
		City:      "York",
		State:     "England",
		Country:   "GB",
		Elevation: 48,
		Latitude:  53.9231444281,
		Longitude: -0.9913444519,
		Timezone:  "Europe/London",
	},
	"EGYM": {
		ICAO:      "EGYM",
		IATA:      "MRH",
		Name:      "RAF Marham",
		City:      "Marham",
		State:     "England",
		Country:   "GB",
		Elevation: 75,
		Latitude:  52.648399353,
		Longitude: 0.5506920218,
		Timezone:  "Europe/London",
	},
	"EGYP": {
		ICAO:      "EGYP",
		IATA:      "MPN",
		Name:      "Mount Pleasant Airport",
		City:      "Mount Pleasant",
		State:     "Tierra-del-Fuego",
		Country:   "FK",
		Elevation: 244,
		Latitude:  -51.8227996826,
		Longitude: -58.4472007751,
		Timezone:  "Atlantic/Stanley",
	},
	"EHAL": {
		ICAO:      "EHAL",
		Name:      "Ameland Airport",
		City:      "Ameland",
		State:     "Friesland",
		Country:   "NL",
		Elevation: 11,
		Latitude:  53.4516983032,
		Longitude: 5.6772198677,
		Timezone:  "Europe/Amsterdam",
	},
	"EHAM": {
		ICAO:      "EHAM",
		IATA:      "AMS",
		Name:      "Amsterdam Airport Schiphol",
		City:      "Amsterdam",
		State:     "North-Holland",
		Country:   "NL",
		Elevation: -11,
		Latitude:  52.3086013794,
		Longitude: 4.7638897896,
		Timezone:  "Europe/Amsterdam",
	},
	"EHBD": {
		ICAO:      "EHBD",
		Name:      "Budel Airport",
		City:      "Weert",
		State:     "North-Brabant",
		Country:   "NL",
		Elevation: 114,
		Latitude:  51.2552986145,
		Longitude: 5.601389885,
		Timezone:  "Europe/Amsterdam",
	},
	"EHBK": {
		ICAO:      "EHBK",
		IATA:      "MST",
		Name:      "Maastricht Aachen Airport",
		City:      "Maastricht",
		State:     "Limburg",
		Country:   "NL",
		Elevation: 375,
		Latitude:  50.9117012024,
		Longitude: 5.7701401711,
		Timezone:  "Europe/Amsterdam",
	},
	"EHDL": {
		ICAO:      "EHDL",
		IATA:      "QAR",
		Name:      "Deelen Air Base",
		City:      "Arnhem",
		State:     "Gelderland",
		Country:   "NL",
		Elevation: 158,
		Latitude:  52.0606002808,
		Longitude: 5.8730602264,
		Timezone:  "Europe/Amsterdam",
	},
	"EHDP": {
		ICAO:      "EHDP",
		Name:      "De Peel Air Base",
		City:      "Venray",
		State:     "North-Brabant",
		Country:   "NL",
		Elevation: 98,
		Latitude:  51.5172996521,
		Longitude: 5.8557200432,
		Timezone:  "Europe/Amsterdam",
	},
	"EHDR": {
		ICAO:      "EHDR",
		Name:      "Drachten Airport",
		City:      "Drachten",
		State:     "Friesland",
		Country:   "NL",
		Elevation: 14,
		Latitude:  53.1192016602,
		Longitude: 6.129720211,
		Timezone:  "Europe/Amsterdam",
	},
	"EHDS": {
		ICAO:      "EHDS",
		Name:      "Aero Club Salland",
		City:      "Lemelerveld",
		State:     "Overijssel",
		Country:   "NL",
		Elevation: 10,
		Latitude:  52.4683333333,
		Longitude: 6.3333333333,
		Timezone:  "Europe/Amsterdam",
	},
	"EHEH": {
		ICAO:      "EHEH",
		IATA:      "EIN",
		Name:      "Eindhoven Airport",
		City:      "Eindhoven",
		State:     "North-Brabant",
		Country:   "NL",
		Elevation: 74,
		Latitude:  51.4500999451,
		Longitude: 5.3745298386,
		Timezone:  "Europe/Amsterdam",
	},
	"EHGG": {
		ICAO:      "EHGG",
		IATA:      "GRQ",
		Name:      "Eelde Airport",
		City:      "Groningen",
		State:     "Drenthe",
		Country:   "NL",
		Elevation: 17,
		Latitude:  53.1197013855,
		Longitude: 6.5794401169,
		Timezone:  "Europe/Amsterdam",
	},
	"EHGR": {
		ICAO:      "EHGR",
		IATA:      "GLZ",
		Name:      "Gilze Rijen Air Base",
		City:      "Breda",
		State:     "North-Brabant",
		Country:   "NL",
		Elevation: 49,
		Latitude:  51.567401886,
		Longitude: 4.9318299294,
		Timezone:  "Europe/Amsterdam",
	},
	"EHHO": {
		ICAO:      "EHHO",
		Name:      "Hoogeveen Airport",
		City:      "Hoogeveen",
		State:     "Drenthe",
		Country:   "NL",
		Elevation: 40,
		Latitude:  52.7308006287,
		Longitude: 6.5161099434,
		Timezone:  "Europe/Amsterdam",
	},
	"EHHV": {
		ICAO:      "EHHV",
		Name:      "Hilversum Airport",
		City:      "Hilversum",
		State:     "North-Holland",
		Country:   "NL",
		Elevation: 3,
		Latitude:  52.1918983459,
		Longitude: 5.1469402313,
		Timezone:  "Europe/Amsterdam",
	},
	"EHKD": {
		ICAO:      "EHKD",
		IATA:      "DHR",
		Name:      "De Kooy Airport",
		City:      "Den Helder",
		State:     "North-Holland",
		Country:   "NL",
		Elevation: 3,
		Latitude:  52.9234008789,
		Longitude: 4.7806200981,
		Timezone:  "Europe/Amsterdam",
	},
	"EHLE": {
		ICAO:      "EHLE",
		IATA:      "LEY",
		Name:      "Lelystad Airport",
		City:      "Lelystad",
		State:     "Flevoland",
		Country:   "NL",
		Elevation: -13,
		Latitude:  52.4603004456,
		Longitude: 5.5272197723,
		Timezone:  "Europe/Amsterdam",
	},
	"EHLW": {
		ICAO:      "EHLW",
		IATA:      "LWR",
		Name:      "Leeuwarden Air Base",
		City:      "Leeuwarden",
		State:     "Friesland",
		Country:   "NL",
		Elevation: 3,
		Latitude:  53.2285995483,
		Longitude: 5.7605600357,
		Timezone:  "Europe/Amsterdam",
	},
	"EHMZ": {
		ICAO:      "EHMZ",
		Name:      "Midden-Zeeland Airport",
		City:      "Middelburg",
		State:     "Zeeland",
		Country:   "NL",
		Elevation: 6,
		Latitude:  51.5121994019,
		Longitude: 3.731110096,
		Timezone:  "Europe/Amsterdam",
	},
	"EHOW": {
		ICAO:      "EHOW",
		IATA:      "---",
		Name:      "Oostwold Airport",
		State:     "Groningen",
		Country:   "NL",
		Elevation: 3,
		Latitude:  53.2097583333,
		Longitude: 7.0365194444,
		Timezone:  "Europe/Amsterdam",
	},
	"EHRD": {
		ICAO:      "EHRD",
		IATA:      "RTM",
		Name:      "Rotterdam Airport",
		City:      "Rotterdam",
		State:     "South-Holland",
		Country:   "NL",
		Elevation: -15,
		Latitude:  51.9569015503,
		Longitude: 4.4372200966,
		Timezone:  "Europe/Amsterdam",
	},
	"EHSE": {
		ICAO:      "EHSE",
		Name:      "Seppe Airport",
		City:      "Hoeven",
		State:     "North-Brabant",
		Country:   "NL",
		Elevation: 30,
		Latitude:  51.5546989441,
		Longitude: 4.5524997711,
		Timezone:  "Europe/Amsterdam",
	},
	"EHST": {
		ICAO:      "EHST",
		Name:      "Stadskanaal Airport",
		City:      "Stadskanaal",
		State:     "Groningen",
		Country:   "NL",
		Elevation: 14,
		Latitude:  52.9986000061,
		Longitude: 7.0227799416,
		Timezone:  "Europe/Amsterdam",
	},
	"EHTE": {
		ICAO:      "EHTE",
		Name:      "Teuge Airport",
		City:      "Deventer",
		State:     "Gelderland",
		Country:   "NL",
		Elevation: 17,
		Latitude:  52.2447013855,
		Longitude: 6.04666996,
		Timezone:  "Europe/Amsterdam",
	},
	"EHTL": {
		ICAO:      "EHTL",
		Name:      "Terlet Airport",
		City:      "Terlet",
		State:     "Gelderland",
		Country:   "NL",
		Elevation: 276,
		Latitude:  52.0572013855,
		Longitude: 5.9244399071,
		Timezone:  "Europe/Amsterdam",
	},
	"EHTW": {
		ICAO:      "EHTW",
		IATA:      "ENS",
		Name:      "Twenthe Airport",
		City:      "Enschede",
		State:     "Overijssel",
		Country:   "NL",
		Elevation: 114,
		Latitude:  52.2700004578,
		Longitude: 6.8741698265,
		Timezone:  "Europe/Amsterdam",
	},
	"EHTX": {
		ICAO:      "EHTX",
		Name:      "Texel Airport",
		City:      "Texel",
		State:     "North-Holland",
		Country:   "NL",
		Elevation: 2,
		Latitude:  53.1152992249,
		Longitude: 4.8336100578,
		Timezone:  "Europe/Amsterdam",
	},
	"EHVK": {
		ICAO:      "EHVK",
		IATA:      "UDE",
		Name:      "Volkel Air Base",
		City:      "Uden",
		State:     "North-Brabant",
		Country:   "NL",
		Elevation: 72,
		Latitude:  51.6563987732,
		Longitude: 5.7086100578,
		Timezone:  "Europe/Amsterdam",
	},
	"EHWO": {
		ICAO:      "EHWO",
		IATA:      "WOE",
		Name:      "Woensdrecht Air Base",
		City:      "Bergen Op Zoom",
		State:     "North-Brabant",
		Country:   "NL",
		Elevation: 63,
		Latitude:  51.4491004944,
		Longitude: 4.3420300484,
		Timezone:  "Europe/Amsterdam",
	},
	"EIAB": {
		ICAO:      "EIAB",
		Name:      "Abbeyshrule Aerodrome",
		City:      "Abbeyshrule",
		State:     "Leinster",
		Country:   "IE",
		Elevation: 195,
		Latitude:  53.5917015076,
		Longitude: -7.6455597878,
		Timezone:  "Europe/Dublin",
	},
	"EIBA": {
		ICAO:      "EIBA",
		Name:      "Ballyboughal Aerodrome",
		City:      "Fingal",
		State:     "Leinster",
		Country:   "IE",
		Elevation: 180,
		Latitude:  53.5336990356,
		Longitude: -6.2460699081,
		Timezone:  "Europe/Dublin",
	},
	"EIBB": {
		ICAO:      "EIBB",
		Name:      "Brittas Bay Airport",
		City:      "Brittas Bay",
		State:     "Leinster",
		Country:   "IE",
		Elevation: 50,
		Latitude:  52.8511009216,
		Longitude: -6.0766401291,
		Timezone:  "Europe/Dublin",
	},
	"EIBN": {
		ICAO:      "EIBN",
		IATA:      "BYT",
		Name:      "Bantry Aerodrome",
		City:      "Bantry",
		State:     "Munster",
		Country:   "IE",
		Elevation: 7,
		Latitude:  51.668598175,
		Longitude: -9.48416996,
		Timezone:  "Europe/Dublin",
	},
	"EIBR": {
		ICAO:      "EIBR",
		Name:      "Birr Aerodrome",
		City:      "Birr",
		State:     "Leinster",
		Country:   "IE",
		Elevation: 250,
		Latitude:  53.0705986023,
		Longitude: -7.8983302116,
		Timezone:  "Europe/Dublin",
	},
	"EIBT": {
		ICAO:      "EIBT",
		IATA:      "BLY",
		Name:      "Belmullet Aerodrome",
		City:      "Belmullet",
		State:     "Connaught",
		Country:   "IE",
		Elevation: 150,
		Latitude:  54.2228012085,
		Longitude: -10.0307998657,
		Timezone:  "Europe/Dublin",
	},
	"EICA": {
		ICAO:      "EICA",
		IATA:      "NNR",
		Name:      "Connemara Regional Airport",
		City:      "Inverin",
		State:     "Connaught",
		Country:   "IE",
		Elevation: 70,
		Latitude:  53.2303009033,
		Longitude: -9.4677801132,
		Timezone:  "Europe/Dublin",
	},
	"EICB": {
		ICAO:      "EICB",
		IATA:      "CLB",
		Name:      "Castlebar Airport",
		City:      "Castlebar",
		State:     "Connaught",
		Country:   "IE",
		Latitude:  53.848400116,
		Longitude: -9.2803697586,
		Timezone:  "Europe/Dublin",
	},
	"EICD": {
		ICAO:      "EICD",
		IATA:      "WEX",
		Name:      "Castlebridge Airport",
		City:      "Wexford",
		State:     "Leinster",
		Country:   "IE",
		Elevation: 79,
		Latitude:  52.3333320618,
		Longitude: -6.4666671753,
		Timezone:  "Europe/Dublin",
	},
	"EICK": {
		ICAO:      "EICK",
		IATA:      "ORK",
		Name:      "Cork Airport",
		City:      "Cork",
		State:     "Munster",
		Country:   "IE",
		Elevation: 502,
		Latitude:  51.8413009644,
		Longitude: -8.491109848,
		Timezone:  "Europe/Dublin",
	},
	"EICL": {
		ICAO:      "EICL",
		Name:      "Clonbullogue Aerodrome",
		City:      "Clonbullogue",
		State:     "Leinster",
		Country:   "IE",
		Elevation: 240,
		Latitude:  53.2496566772,
		Longitude: -7.1227455139,
		Timezone:  "Europe/Dublin",
	},
	"EICM": {
		ICAO:      "EICM",
		IATA:      "GWY",
		Name:      "Galway Airport",
		City:      "Galway",
		State:     "Connaught",
		Country:   "IE",
		Elevation: 81,
		Latitude:  53.300201416,
		Longitude: -8.9415903091,
		Timezone:  "Europe/Dublin",
	},
	"EICN": {
		ICAO:      "EICN",
		Name:      "Coonagh Aerodrome",
		City:      "Limerick",
		State:     "Munster",
		Country:   "IE",
		Elevation: 6,
		Latitude:  52.6665000916,
		Longitude: -8.6818304062,
		Timezone:  "Europe/Dublin",
	},
	"EICS": {
		ICAO:      "EICS",
		Name:      "Castleforbes Airport",
		City:      "Castleforbes",
		State:     "Leinster",
		Country:   "IE",
		Elevation: 150,
		Latitude:  53.778301239,
		Longitude: -7.8352799416,
		Timezone:  "Europe/Dublin",
	},
	"EIDL": {
		ICAO:      "EIDL",
		IATA:      "CFN",
		Name:      "Donegal Airport",
		City:      "Donegal",
		State:     "Ulster",
		Country:   "IE",
		Elevation: 30,
		Latitude:  55.0442008972,
		Longitude: -8.3409996033,
		Timezone:  "Europe/Dublin",
	},
	"EIDW": {
		ICAO:      "EIDW",
		IATA:      "DUB",
		Name:      "Dublin Airport",
		City:      "Dublin",
		State:     "Leinster",
		Country:   "IE",
		Elevation: 242,
		Latitude:  53.4212989807,
		Longitude: -6.270070076,
		Timezone:  "Europe/Dublin",
	},
	"EIER": {
		ICAO:      "EIER",
		Name:      "Erinagh Airport",
		City:      "Nenagh",
		State:     "Munster",
		Country:   "IE",
		Elevation: 155,
		Latitude:  52.8125,
		Longitude: -8.2822217941,
		Timezone:  "Europe/Dublin",
	},
	"EIGN": {
		ICAO:      "EIGN",
		Name:      "Gowran Grange Aerodrome",
		City:      "Swordlestown",
		State:     "Leinster",
		Country:   "IE",
		Elevation: 489,
		Latitude:  53.1786003113,
		Longitude: -6.6352801323,
		Timezone:  "Europe/Dublin",
	},
	"EIHN": {
		ICAO:      "EIHN",
		Name:      "Hacketstown Aerodrome",
		City:      "Hacketstown",
		State:     "Leinster",
		Country:   "IE",
		Elevation: 610,
		Latitude:  52.8549995422,
		Longitude: -6.5472202301,
		Timezone:  "Europe/Dublin",
	},
	"EIIM": {
		ICAO:      "EIIM",
		IATA:      "IOR",
		Name:      "Inishmore Aerodrome",
		City:      "Inis Mor",
		State:     "Connaught",
		Country:   "IE",
		Elevation: 24,
		Latitude:  53.1067008972,
		Longitude: -9.6536102295,
		Timezone:  "Europe/Dublin",
	},
	"EIIR": {
		ICAO:      "EIIR",
		IATA:      "INQ",
		Name:      "Inisheer Aerodrome",
		City:      "Inis Oirr",
		State:     "Connaught",
		Country:   "IE",
		Elevation: 40,
		Latitude:  53.0647010803,
		Longitude: -9.5108995438,
		Timezone:  "Europe/Dublin",
	},
	"EIKH": {
		ICAO:      "EIKH",
		Name:      "Kildare Aerodrome",
		City:      "Kilrush",
		State:     "Leinster",
		Country:   "IE",
		Elevation: 260,
		Latitude:  53.0635986328,
		Longitude: -6.8552799225,
		Timezone:  "Europe/Dublin",
	},
	"EIKI": {
		ICAO:      "EIKI",
		Name:      "Killenaule Airport",
		City:      "Killenaule",
		State:     "Munster",
		Country:   "IE",
		Elevation: 680,
		Latitude:  52.6329994202,
		Longitude: -7.632999897,
		Timezone:  "Europe/Dublin",
	},
	"EIKK": {
		ICAO:      "EIKK",
		IATA:      "KKY",
		Name:      "Kilkenny Airport",
		City:      "Kilkenny",
		State:     "Leinster",
		Country:   "IE",
		Elevation: 319,
		Latitude:  52.6507987976,
		Longitude: -7.2961101532,
		Timezone:  "Europe/Dublin",
	},
	"EIKN": {
		ICAO:      "EIKN",
		IATA:      "NOC",
		Name:      "Ireland West Knock Airport",
		City:      "Charleston",
		State:     "Connaught",
		Country:   "IE",
		Elevation: 665,
		Latitude:  53.9103012085,
		Longitude: -8.8184900284,
		Timezone:  "Europe/Dublin",
	},
	"EIKY": {
		ICAO:      "EIKY",
		IATA:      "KIR",
		Name:      "Kerry Airport",
		City:      "Killarney",
		State:     "Munster",
		Country:   "IE",
		Elevation: 112,
		Latitude:  52.1809005737,
		Longitude: -9.5237798691,
		Timezone:  "Europe/Dublin",
	},
	"EILT": {
		ICAO:      "EILT",
		IATA:      "LTR",
		Name:      "Letterkenny Airport",
		City:      "Letterkenny",
		State:     "Ulster",
		Country:   "IE",
		Elevation: 20,
		Latitude:  54.9513015747,
		Longitude: -7.6728301048,
		Timezone:  "Europe/Dublin",
	},
	"EIME": {
		ICAO:      "EIME",
		Name:      "Casement Air Base",
		City:      "Baldonnel",
		State:     "Leinster",
		Country:   "IE",
		Elevation: 319,
		Latitude:  53.301700592,
		Longitude: -6.4513301849,
		Timezone:  "Europe/Dublin",
	},
	"EIMG": {
		ICAO:      "EIMG",
		Name:      "Moneygall Aerodrome",
		City:      "Moneygall",
		State:     "Munster",
		Country:   "IE",
		Elevation: 450,
		Latitude:  52.8499984741,
		Longitude: -7.9829998016,
		Timezone:  "Europe/Dublin",
	},
	"EIMH": {
		ICAO:      "EIMH",
		Name:      "Athboy Airport",
		City:      "Athboy",
		State:     "Leinster",
		Country:   "IE",
		Elevation: 230,
		Latitude:  53.6377792358,
		Longitude: -6.8786110878,
		Timezone:  "Europe/Dublin",
	},
	"EIMN": {
		ICAO:      "EIMN",
		IATA:      "IIA",
		Name:      "Inishmaan Aerodrome",
		City:      "Inis Meain",
		State:     "Connaught",
		Country:   "IE",
		Elevation: 15,
		Latitude:  53.0929985046,
		Longitude: -9.5680599213,
		Timezone:  "Europe/Dublin",
	},
	"EIMY": {
		ICAO:      "EIMY",
		Name:      "Moyne Aerodrome",
		City:      "Thurles",
		State:     "Munster",
		Country:   "IE",
		Elevation: 400,
		Latitude:  52.7030982971,
		Longitude: -7.7052798271,
		Timezone:  "Europe/Dublin",
	},
	"EINC": {
		ICAO:      "EINC",
		Name:      "Newcastle Aerodrome",
		City:      "Newcastle",
		State:     "Leinster",
		Country:   "IE",
		Elevation: 1,
		Latitude:  53.0710983276,
		Longitude: -6.0452799797,
		Timezone:  "Europe/Dublin",
	},
	"EINN": {
		ICAO:      "EINN",
		IATA:      "SNN",
		Name:      "Shannon Airport",
		City:      "Shannon",
		State:     "Munster",
		Country:   "IE",
		Elevation: 46,
		Latitude:  52.7019996643,
		Longitude: -8.9248199463,
		Timezone:  "Europe/Dublin",
	},
	"EIPT": {
		ICAO:      "EIPT",
		Name:      "Powerscourt Airfield",
		City:      "Powerscourt",
		State:     "Leinster",
		Country:   "IE",
		Elevation: 470,
		Latitude:  53.178301,
		Longitude: -6.196495,
		Timezone:  "Europe/Dublin",
	},
	"EIRT": {
		ICAO:      "EIRT",
		Name:      "Rathcoole Aerodrome",
		City:      "Rathcoole",
		State:     "Munster",
		Country:   "IE",
		Elevation: 281,
		Latitude:  52.1055984497,
		Longitude: -8.9833297729,
		Timezone:  "Europe/Dublin",
	},
	"EISG": {
		ICAO:      "EISG",
		IATA:      "SXL",
		Name:      "Sligo Airport",
		City:      "Sligo",
		State:     "Connaught",
		Country:   "IE",
		Elevation: 11,
		Latitude:  54.2802009583,
		Longitude: -8.5992097855,
		Timezone:  "Europe/Dublin",
	},
	"EISP": {
		ICAO:      "EISP",
		Name:      "Spanish Point Airport",
		City:      "Spanish Point",
		State:     "Munster",
		Country:   "IE",
		Latitude:  52.8497009277,
		Longitude: -9.4169998169,
		Timezone:  "Europe/Dublin",
	},
	"EITM": {
		ICAO:      "EITM",
		Name:      "Trim Aerodrome",
		City:      "Trim",
		State:     "Leinster",
		Country:   "IE",
		Elevation: 200,
		Latitude:  53.5746994019,
		Longitude: -6.7386097908,
		Timezone:  "Europe/Dublin",
	},
	"EITT": {
		ICAO:      "EITT",
		Name:      "Trevet Airfield",
		City:      "Dunshaughlin",
		State:     "Leinster",
		Country:   "IE",
		Elevation: 385,
		Latitude:  53.5486106873,
		Longitude: -6.5169439316,
		Timezone:  "Europe/Dublin",
	},
	"EIWF": {
		ICAO:      "EIWF",
		IATA:      "WAT",
		Name:      "Waterford Airport",
		City:      "Waterford",
		State:     "Munster",
		Country:   "IE",
		Elevation: 119,
		Latitude:  52.1871986389,
		Longitude: -7.0869598389,
		Timezone:  "Europe/Dublin",
	},
	"EIWT": {
		ICAO:      "EIWT",
		Name:      "Weston Airport",
		City:      "Leixlip",
		State:     "Leinster",
		Country:   "IE",
		Elevation: 150,
		Latitude:  53.3521995544,
		Longitude: -6.4861102104,
		Timezone:  "Europe/Dublin",
	},
	"EKAE": {
		ICAO:      "EKAE",
		Name:      "AEro Airport",
		City:      "AEro",
		State:     "South-Denmark",
		Country:   "DK",
		Elevation: 3,
		Latitude:  54.8527984619,
		Longitude: 10.4563999176,
		Timezone:  "Europe/Copenhagen",
	},
	"EKAH": {
		ICAO:      "EKAH",
		IATA:      "AAR",
		Name:      "Aarhus Airport",
		City:      "Aarhus",
		State:     "Central-Jutland",
		Country:   "DK",
		Elevation: 82,
		Latitude:  56.2999992371,
		Longitude: 10.6190004349,
		Timezone:  "Europe/Copenhagen",
	},
	"EKAT": {
		ICAO:      "EKAT",
		Name:      "Anholt Airport",
		City:      "Anholt",
		State:     "Central-Jutland",
		Country:   "DK",
		Elevation: 23,
		Latitude:  56.6988983154,
		Longitude: 11.5558996201,
		Timezone:  "Europe/Copenhagen",
	},
	"EKAV": {
		ICAO:      "EKAV",
		Name:      "Avno Air Base",
		City:      "Avno",
		State:     "Zealand",
		Country:   "DK",
		Elevation: 13,
		Latitude:  55.0830001831,
		Longitude: 11.7829999924,
		Timezone:  "Europe/Copenhagen",
	},
	"EKBI": {
		ICAO:      "EKBI",
		IATA:      "BLL",
		Name:      "Billund Airport",
		City:      "Billund",
		State:     "South-Denmark",
		Country:   "DK",
		Elevation: 247,
		Latitude:  55.7402992249,
		Longitude: 9.1517801285,
		Timezone:  "Europe/Copenhagen",
	},
	"EKCH": {
		ICAO:      "EKCH",
		IATA:      "CPH",
		Name:      "Copenhagen Kastrup Airport",
		City:      "Copenhagen",
		State:     "Capital-Region",
		Country:   "DK",
		Elevation: 17,
		Latitude:  55.6179008484,
		Longitude: 12.6560001373,
		Timezone:  "Europe/Copenhagen",
	},
	"EKEB": {
		ICAO:      "EKEB",
		IATA:      "EBJ",
		Name:      "Esbjerg Airport",
		City:      "Esbjerg",
		State:     "South-Denmark",
		Country:   "DK",
		Elevation: 97,
		Latitude:  55.5259017944,
		Longitude: 8.5534000397,
		Timezone:  "Europe/Copenhagen",
	},
	"EKEL": {
		ICAO:      "EKEL",
		Name:      "Endelave West Airport",
		City:      "Endelave",
		State:     "Central-Jutland",
		Country:   "DK",
		Elevation: 15,
		Latitude:  55.7565002441,
		Longitude: 10.2484998703,
		Timezone:  "Europe/Copenhagen",
	},
	"EKFU": {
		ICAO:      "EKFU",
		Name:      "Fur Airport",
		City:      "Fur",
		State:     "Central-Jutland",
		Country:   "DK",
		Elevation: 30,
		Latitude:  56.8244018555,
		Longitude: 8.9861097336,
		Timezone:  "Europe/Copenhagen",
	},
	"EKGH": {
		ICAO:      "EKGH",
		Name:      "Gronholt Hillerod Airport",
		City:      "Gronholt",
		State:     "Capital-Region",
		Country:   "DK",
		Elevation: 97,
		Latitude:  55.9413986206,
		Longitude: 12.3822002411,
		Timezone:  "Europe/Copenhagen",
	},
	"EKGR": {
		ICAO:      "EKGR",
		Name:      "Grenaa Airport",
		City:      "Grenaa",
		State:     "Central-Jutland",
		Country:   "DK",
		Elevation: 20,
		Latitude:  56.4425010681,
		Longitude: 10.9306001663,
		Timezone:  "Europe/Copenhagen",
	},
	"EKHG": {
		ICAO:      "EKHG",
		Name:      "Herning Airport",
		City:      "Herning",
		State:     "Central-Jutland",
		Country:   "DK",
		Elevation: 167,
		Latitude:  56.1847000122,
		Longitude: 9.0444498062,
		Timezone:  "Europe/Copenhagen",
	},
	"EKHK": {
		ICAO:      "EKHK",
		Name:      "Holbaek (Ny Hagested) Airport",
		City:      "Holbaek",
		State:     "Zealand",
		Country:   "DK",
		Elevation: 2,
		Latitude:  55.73279953,
		Longitude: 11.6028003693,
		Timezone:  "Europe/Copenhagen",
	},
	"EKHO": {
		ICAO:      "EKHO",
		Name:      "Lindtorp Airport",
		City:      "Holstebro",
		State:     "Central-Jutland",
		Country:   "DK",
		Elevation: 49,
		Latitude:  56.3967018127,
		Longitude: 8.4419403076,
		Timezone:  "Europe/Copenhagen",
	},
	"EKHS": {
		ICAO:      "EKHS",
		Name:      "Hadsund Airport",
		City:      "Hadsund",
		State:     "North-Denmark",
		Country:   "DK",
		Elevation: 8,
		Latitude:  56.7559242249,
		Longitude: 10.2288293839,
		Timezone:  "Europe/Copenhagen",
	},
	"EKHV": {
		ICAO:      "EKHV",
		Name:      "Haderslev Airport",
		City:      "Haderslev",
		State:     "South-Denmark",
		Country:   "DK",
		Elevation: 81,
		Latitude:  55.3022003174,
		Longitude: 9.5225000381,
		Timezone:  "Europe/Copenhagen",
	},
	"EKKA": {
		ICAO:      "EKKA",
		IATA:      "KRP",
		Name:      "Karup Airport",
		City:      "Karup",
		State:     "Central-Jutland",
		Country:   "DK",
		Elevation: 170,
		Latitude:  56.2975006104,
		Longitude: 9.1246299744,
		Timezone:  "Europe/Copenhagen",
	},
	"EKKL": {
		ICAO:      "EKKL",
		Name:      "Kalundborg Airport",
		City:      "Kalundborg",
		State:     "Zealand",
		Country:   "DK",
		Elevation: 1,
		Latitude:  55.7002983093,
		Longitude: 11.25,
		Timezone:  "Europe/Copenhagen",
	},
	"EKKO": {
		ICAO:      "EKKO",
		Name:      "Korsor Airport",
		City:      "Korsor",
		State:     "Zealand",
		Country:   "DK",
		Elevation: 39,
		Latitude:  55.3368988037,
		Longitude: 11.2417001724,
		Timezone:  "Europe/Copenhagen",
	},
	"EKLS": {
		ICAO:      "EKLS",
		IATA:      "BYR",
		Name:      "Laeso Airport",
		City:      "Laeso",
		State:     "North-Denmark",
		Country:   "DK",
		Elevation: 25,
		Latitude:  57.2771987915,
		Longitude: 11.0001001358,
		Timezone:  "Europe/Copenhagen",
	},
	"EKLV": {
		ICAO:      "EKLV",
		Name:      "Lemvig Airport",
		City:      "Lemvig",
		State:     "Central-Jutland",
		Country:   "DK",
		Elevation: 97,
		Latitude:  56.5029983521,
		Longitude: 8.3113298416,
		Timezone:  "Europe/Copenhagen",
	},
	"EKMB": {
		ICAO:      "EKMB",
		IATA:      "MRW",
		Name:      "Lolland Falster Maribo Airport",
		City:      "Lolland Falster / Maribo",
		State:     "Zealand",
		Country:   "DK",
		Elevation: 16,
		Latitude:  54.6992988586,
		Longitude: 11.4400997162,
		Timezone:  "Europe/Copenhagen",
	},
	"EKMN": {
		ICAO:      "EKMN",
		Name:      "Kostervig Mon Airport",
		City:      "Koster",
		State:     "Zealand",
		Country:   "DK",
		Elevation: 3,
		Latitude:  54.9646987915,
		Longitude: 12.1943998337,
		Timezone:  "Europe/Copenhagen",
	},
	"EKNM": {
		ICAO:      "EKNM",
		Name:      "Morso Airport Todso",
		City:      "Morso",
		State:     "North-Denmark",
		Country:   "DK",
		Elevation: 53,
		Latitude:  56.8244018555,
		Longitude: 8.7866697311,
		Timezone:  "Europe/Copenhagen",
	},
	"EKNS": {
		ICAO:      "EKNS",
		Name:      "Nakskov Airport",
		City:      "Naksnov",
		State:     "Zealand",
		Country:   "DK",
		Elevation: 3,
		Latitude:  54.817199707,
		Longitude: 11.1308002472,
		Timezone:  "Europe/Copenhagen",
	},
	"EKOD": {
		ICAO:      "EKOD",
		IATA:      "ODE",
		Name:      "Odense Airport",
		City:      "Odense",
		State:     "South-Denmark",
		Country:   "DK",
		Elevation: 56,
		Latitude:  55.4766998291,
		Longitude: 10.3309001923,
		Timezone:  "Europe/Copenhagen",
	},
	"EKPB": {
		ICAO:      "EKPB",
		Name:      "Krusa-Padborg Airport",
		City:      "Krusa / Padborg",
		State:     "South-Denmark",
		Country:   "DK",
		Elevation: 88,
		Latitude:  54.870300293,
		Longitude: 9.279009819,
		Timezone:  "Europe/Copenhagen",
	},
	"EKRA": {
		ICAO:      "EKRA",
		Name:      "Rarup Airport",
		City:      "Rarup",
		State:     "Central-Jutland",
		Country:   "DK",
		Elevation: 131,
		Latitude:  55.7785987854,
		Longitude: 9.9388904572,
		Timezone:  "Europe/Copenhagen",
	},
	"EKRD": {
		ICAO:      "EKRD",
		Name:      "Randers Airport",
		City:      "Randers",
		State:     "Central-Jutland",
		Country:   "DK",
		Elevation: 139,
		Latitude:  56.5065994263,
		Longitude: 10.0363998413,
		Timezone:  "Europe/Copenhagen",
	},
	"EKRK": {
		ICAO:      "EKRK",
		IATA:      "RKE",
		Name:      "Copenhagen Roskilde Airport",
		City:      "Copenhagen",
		State:     "Zealand",
		Country:   "DK",
		Elevation: 146,
		Latitude:  55.5856018066,
		Longitude: 12.1314001083,
		Timezone:  "Europe/Copenhagen",
	},
	"EKRN": {
		ICAO:      "EKRN",
		IATA:      "RNN",
		Name:      "Bornholm Airport",
		City:      "Ronne",
		State:     "Capital-Region",
		Country:   "DK",
		Elevation: 52,
		Latitude:  55.0633010864,
		Longitude: 14.7595996857,
		Timezone:  "Europe/Copenhagen",
	},
	"EKRS": {
		ICAO:      "EKRS",
		Name:      "Ringsted Airport",
		City:      "Ringsted",
		State:     "Zealand",
		Country:   "DK",
		Elevation: 113,
		Latitude:  55.4258003235,
		Longitude: 11.8066997528,
		Timezone:  "Europe/Copenhagen",
	},
	"EKSA": {
		ICAO:      "EKSA",
		Name:      "Saeby (Ottestrup) Airport",
		City:      "Saeby",
		State:     "North-Denmark",
		Country:   "DK",
		Elevation: 110,
		Latitude:  57.346698761,
		Longitude: 10.406999588,
		Timezone:  "Europe/Copenhagen",
	},
	"EKSB": {
		ICAO:      "EKSB",
		IATA:      "SGD",
		Name:      "Sonderborg Airport",
		City:      "Sonderborg",
		State:     "South-Denmark",
		Country:   "DK",
		Elevation: 24,
		Latitude:  54.9644012451,
		Longitude: 9.7917299271,
		Timezone:  "Europe/Copenhagen",
	},
	"EKSD": {
		ICAO:      "EKSD",
		Name:      "Spjald Airport",
		City:      "Spjald",
		State:     "Central-Jutland",
		Country:   "DK",
		Elevation: 249,
		Latitude:  56.1026992798,
		Longitude: 8.5142297745,
		Timezone:  "Europe/Copenhagen",
	},
	"EKSN": {
		ICAO:      "EKSN",
		IATA:      "CNL",
		Name:      "Sindal Airport",
		City:      "Sindal",
		State:     "North-Denmark",
		Country:   "DK",
		Elevation: 92,
		Latitude:  57.5035018921,
		Longitude: 10.2293996811,
		Timezone:  "Europe/Copenhagen",
	},
	"EKSP": {
		ICAO:      "EKSP",
		IATA:      "SKS",
		Name:      "Vojens Skrydstrup Airport",
		City:      "Vojens",
		State:     "South-Denmark",
		Country:   "DK",
		Elevation: 141,
		Latitude:  55.2256011963,
		Longitude: 9.2639303207,
		Timezone:  "Europe/Copenhagen",
	},
	"EKSS": {
		ICAO:      "EKSS",
		Name:      "Samso Airport",
		City:      "Samso",
		State:     "Central-Jutland",
		Country:   "DK",
		Elevation: 1,
		Latitude:  55.8894996643,
		Longitude: 10.613699913,
		Timezone:  "Europe/Copenhagen",
	},
	"EKST": {
		ICAO:      "EKST",
		Name:      "Sydfy (Tasinge) Airport",
		City:      "Tasinge",
		State:     "South-Denmark",
		Country:   "DK",
		Elevation: 7,
		Latitude:  55.0166015625,
		Longitude: 10.5633001328,
		Timezone:  "Europe/Copenhagen",
	},
	"EKSV": {
		ICAO:      "EKSV",
		IATA:      "SQW",
		Name:      "Skive Airport",
		City:      "Skive",
		State:     "Central-Jutland",
		Country:   "DK",
		Elevation: 74,
		Latitude:  56.550201416,
		Longitude: 9.1729803085,
		Timezone:  "Europe/Copenhagen",
	},
	"EKTD": {
		ICAO:      "EKTD",
		Name:      "Tonder Airport",
		City:      "Tonder",
		State:     "South-Denmark",
		Country:   "DK",
		Elevation: 1,
		Latitude:  54.9296989441,
		Longitude: 8.8405704498,
		Timezone:  "Europe/Berlin",
	},
	"EKTS": {
		ICAO:      "EKTS",
		IATA:      "TED",
		Name:      "Thisted Airport",
		City:      "Thisted",
		State:     "North-Denmark",
		Country:   "DK",
		Elevation: 23,
		Latitude:  57.0688018799,
		Longitude: 8.7052202225,
		Timezone:  "Europe/Copenhagen",
	},
	"EKVA": {
		ICAO:      "EKVA",
		Name:      "Vandel Air Base",
		City:      "Vandel",
		State:     "South-Denmark",
		Country:   "DK",
		Elevation: 250,
		Latitude:  55.7010002136,
		Longitude: 9.2135801315,
		Timezone:  "Europe/Copenhagen",
	},
	"EKVB": {
		ICAO:      "EKVB",
		Name:      "Viborg Airport",
		City:      "Viborg",
		State:     "Central-Jutland",
		Country:   "DK",
		Elevation: 75,
		Latitude:  56.4099998474,
		Longitude: 9.4090995789,
		Timezone:  "Europe/Copenhagen",
	},
	"EKVD": {
		ICAO:      "EKVD",
		Name:      "Kolding Vamdrup Airport",
		City:      "Kolding / Vamdrup",
		State:     "South-Denmark",
		Country:   "DK",
		Elevation: 143,
		Latitude:  55.4362983704,
		Longitude: 9.3309202194,
		Timezone:  "Europe/Copenhagen",
	},
	"EKVG": {
		ICAO:      "EKVG",
		IATA:      "FAE",
		Name:      "Vagar Airport",
		City:      "Vagar",
		State:     "Vagar",
		Country:   "FO",
		Elevation: 280,
		Latitude:  62.0635986328,
		Longitude: -7.2772197723,
		Timezone:  "Atlantic/Faroe",
	},
	"EKVH": {
		ICAO:      "EKVH",
		Name:      "Aars Airport",
		City:      "Aars",
		State:     "North-Denmark",
		Country:   "DK",
		Elevation: 119,
		Latitude:  56.8469009399,
		Longitude: 9.458609581,
		Timezone:  "Europe/Copenhagen",
	},
	"EKVJ": {
		ICAO:      "EKVJ",
		IATA:      "STA",
		Name:      "Stauning Airport",
		City:      "Skjern / Ringkobing",
		State:     "Central-Jutland",
		Country:   "DK",
		Elevation: 17,
		Latitude:  55.9901008606,
		Longitude: 8.3539104462,
		Timezone:  "Europe/Copenhagen",
	},
	"EKYT": {
		ICAO:      "EKYT",
		IATA:      "AAL",
		Name:      "Aalborg Airport",
		City:      "Aalborg",
		State:     "North-Denmark",
		Country:   "DK",
		Elevation: 10,
		Latitude:  57.0927589138,
		Longitude: 9.8492431641,
		Timezone:  "Europe/Copenhagen",
	},
	"EK_2": {
		ICAO:      "EK_2",
		Name:      "Femo Airfield",
		City:      "Femo",
		State:     "Zealand",
		Country:   "DK",
		Elevation: 2,
		Latitude:  54.9841151907,
		Longitude: 11.5376615524,
		Timezone:  "Europe/Copenhagen",
	},
	"EK_4": {
		ICAO:      "EK_4",
		IATA:      "0",
		Name:      "Vejro",
		State:     "Zealand",
		Country:   "DK",
		Latitude:  55.035,
		Longitude: 11.375,
		Timezone:  "Europe/Copenhagen",
	},
	"ELLX": {
		ICAO:      "ELLX",
		IATA:      "LUX",
		Name:      "Luxembourg-Findel International Airport",
		City:      "Luxembourg",
		State:     "Luxembourg",
		Country:   "LU",
		Elevation: 1234,
		Latitude:  49.6265983582,
		Longitude: 6.211520195,
		Timezone:  "Europe/Luxembourg",
	},
	"ELNT": {
		ICAO:      "ELNT",
		Name:      "Noertrange Airport",
		City:      "Winseler",
		State:     "Wiltz",
		Country:   "LU",
		Elevation: 1522,
		Latitude:  49.981098175,
		Longitude: 5.9177799225,
		Timezone:  "Europe/Luxembourg",
	},
	"ELUS": {
		ICAO:      "ELUS",
		Name:      "Useldange Airfield",
		City:      "Useldange",
		State:     "Redange",
		Country:   "LU",
		Elevation: 928,
		Latitude:  49.7686004639,
		Longitude: 5.9655599594,
		Timezone:  "Europe/Luxembourg",
	},
	"ENAE": {
		ICAO:      "ENAE",
		Name:      "AEra Airfield",
		City:      "Amot",
		State:     "Hedmark",
		Country:   "NO",
		Elevation: 1621,
		Latitude:  61.2574005127,
		Longitude: 11.6688995361,
		Timezone:  "Europe/Oslo",
	},
	"ENAL": {
		ICAO:      "ENAL",
		IATA:      "AES",
		Name:      "Alesund Airport",
		City:      "Alesund",
		State:     "More-og-Romsdal",
		Country:   "NO",
		Elevation: 69,
		Latitude:  62.5625,
		Longitude: 6.119699955,
		Timezone:  "Europe/Oslo",
	},
	"ENAN": {
		ICAO:      "ENAN",
		IATA:      "ANX",
		Name:      "Andoya Airport",
		City:      "Andenes",
		State:     "Nordland",
		Country:   "NO",
		Elevation: 43,
		Latitude:  69.2925033569,
		Longitude: 16.1441993713,
		Timezone:  "Europe/Oslo",
	},
	"ENAS": {
		ICAO:      "ENAS",
		Name:      "Ny-Alesund Airport Hamnerabben",
		City:      "Ny-Alesund",
		State:     "Svalbard",
		Country:   "NO",
		Elevation: 50,
		Latitude:  78.9274978638,
		Longitude: 11.8743000031,
		Timezone:  "Arctic/Longyearbyen",
	},
	"ENAT": {
		ICAO:      "ENAT",
		IATA:      "ALF",
		Name:      "Alta Airport",
		City:      "Alta",
		State:     "Finnmark",
		Country:   "NO",
		Elevation: 9,
		Latitude:  69.9760971069,
		Longitude: 23.3717002869,
		Timezone:  "Europe/Oslo",
	},
	"ENBL": {
		ICAO:      "ENBL",
		IATA:      "FDE",
		Name:      "Bringeland Airport",
		City:      "Forde",
		State:     "Sogn-og-Fjordane",
		Country:   "NO",
		Elevation: 1046,
		Latitude:  61.3911018372,
		Longitude: 5.756939888,
		Timezone:  "Europe/Oslo",
	},
	"ENBM": {
		ICAO:      "ENBM",
		Name:      "Bomoen Airport",
		City:      "Tjukkebygdi",
		State:     "Hordaland",
		Country:   "NO",
		Elevation: 300,
		Latitude:  60.6389007568,
		Longitude: 6.5015001297,
		Timezone:  "Europe/Oslo",
	},
	"ENBN": {
		ICAO:      "ENBN",
		IATA:      "BNN",
		Name:      "Bronnoysund Airport",
		City:      "Bronnoy",
		State:     "Nordland",
		Country:   "NO",
		Elevation: 25,
		Latitude:  65.4610977173,
		Longitude: 12.217499733,
		Timezone:  "Europe/Oslo",
	},
	"ENBO": {
		ICAO:      "ENBO",
		IATA:      "BOO",
		Name:      "Bodo Airport",
		City:      "Bodo",
		State:     "Nordland",
		Country:   "NO",
		Elevation: 42,
		Latitude:  67.269203186,
		Longitude: 14.3653001785,
		Timezone:  "Europe/Oslo",
	},
	"ENBR": {
		ICAO:      "ENBR",
		IATA:      "BGO",
		Name:      "Bergen Airport Flesland",
		City:      "Bergen",
		State:     "Hordaland",
		Country:   "NO",
		Elevation: 170,
		Latitude:  60.2933998108,
		Longitude: 5.2181401253,
		Timezone:  "Europe/Oslo",
	},
	"ENBS": {
		ICAO:      "ENBS",
		IATA:      "BJF",
		Name:      "Batsfjord Airport",
		City:      "Batsfjord",
		State:     "Finnmark",
		Country:   "NO",
		Elevation: 490,
		Latitude:  70.6005020142,
		Longitude: 29.691400528,
		Timezone:  "Europe/Oslo",
	},
	"ENBV": {
		ICAO:      "ENBV",
		IATA:      "BVG",
		Name:      "Berlevag Airport",
		City:      "Berlevag",
		State:     "Finnmark",
		Country:   "NO",
		Elevation: 42,
		Latitude:  70.8713989258,
		Longitude: 29.0342006683,
		Timezone:  "Europe/Oslo",
	},
	"ENCN": {
		ICAO:      "ENCN",
		IATA:      "KRS",
		Name:      "Kristiansand Airport",
		City:      "Kjevik",
		State:     "Vest-Agder",
		Country:   "NO",
		Elevation: 57,
		Latitude:  58.2042007446,
		Longitude: 8.0853700638,
		Timezone:  "Europe/Oslo",
	},
	"ENDI": {
		ICAO:      "ENDI",
		IATA:      "DLD",
		Name:      "Geilo Airport Dagali",
		City:      "Dagali",
		State:     "Buskerud",
		Country:   "NO",
		Elevation: 2618,
		Latitude:  60.417301178,
		Longitude: 8.5183496475,
		Timezone:  "Europe/Oslo",
	},
	"ENDO": {
		ICAO:      "ENDO",
		Name:      "Dokka Thomlevold Airport",
		City:      "Dokka",
		State:     "Oppland",
		Country:   "NO",
		Elevation: 512,
		Latitude:  60.8333015442,
		Longitude: 9.9166698456,
		Timezone:  "Europe/Oslo",
	},
	"ENDU": {
		ICAO:      "ENDU",
		IATA:      "BDU",
		Name:      "Bardufoss Airport",
		City:      "Malselv",
		State:     "Troms",
		Country:   "NO",
		Elevation: 252,
		Latitude:  69.0558013916,
		Longitude: 18.5403995514,
		Timezone:  "Europe/Oslo",
	},
	"ENEG": {
		ICAO:      "ENEG",
		Name:      "Eggemoen Airport",
		City:      "Honefoss",
		State:     "Buskerud",
		Country:   "NO",
		Elevation: 656,
		Latitude:  60.217300415,
		Longitude: 10.3239002228,
		Timezone:  "Europe/Oslo",
	},
	"ENEV": {
		ICAO:      "ENEV",
		IATA:      "EVE",
		Name:      "Harstad/Narvik Airport Evenes",
		City:      "Evenes",
		State:     "Nordland",
		Country:   "NO",
		Elevation: 84,
		Latitude:  68.4913024902,
		Longitude: 16.6781005859,
		Timezone:  "Europe/Oslo",
	},
	"ENFA": {
		ICAO:      "ENFA",
		Name:      "Flatval Airport",
		City:      "Froya",
		State:     "Sor-Trondelag",
		Country:   "NO",
		Elevation: 152,
		Latitude:  63.7005996704,
		Longitude: 8.7605695724,
		Timezone:  "Europe/Oslo",
	},
	"ENFG": {
		ICAO:      "ENFG",
		IATA:      "VDB",
		Name:      "Leirin Airport",
		State:     "Oppland",
		Country:   "NO",
		Elevation: 2697,
		Latitude:  61.0155982971,
		Longitude: 9.2880601883,
		Timezone:  "Europe/Oslo",
	},
	"ENFL": {
		ICAO:      "ENFL",
		IATA:      "FRO",
		Name:      "Floro Airport",
		City:      "Floro",
		State:     "Sogn-og-Fjordane",
		Country:   "NO",
		Elevation: 37,
		Latitude:  61.5835990906,
		Longitude: 5.024720192,
		Timezone:  "Europe/Oslo",
	},
	"ENFY": {
		ICAO:      "ENFY",
		Name:      "Fyresdal Airfield",
		City:      "Fyresdal",
		State:     "Telemark",
		Country:   "NO",
		Elevation: 986,
		Latitude:  59.2000007629,
		Longitude: 8.0866699219,
		Timezone:  "Europe/Oslo",
	},
	"ENGK": {
		ICAO:      "ENGK",
		Name:      "Gullknapp Flpl Airport",
		State:     "Aust-Agder",
		Country:   "NO",
		Elevation: 390,
		Latitude:  58.5167007446,
		Longitude: 8.6999998093,
		Timezone:  "Europe/Oslo",
	},
	"ENGM": {
		ICAO:      "ENGM",
		IATA:      "OSL",
		Name:      "Oslo Gardermoen Airport",
		City:      "Oslo",
		State:     "Akershus",
		Country:   "NO",
		Elevation: 681,
		Latitude:  60.193901062,
		Longitude: 11.100399971,
		Timezone:  "Europe/Oslo",
	},
	"ENGN": {
		ICAO:      "ENGN",
		Name:      "Folldal Grimsmoe Airport",
		City:      "Folldal",
		State:     "Hedmark",
		Country:   "NO",
		Elevation: 2260,
		Latitude:  62.1175003052,
		Longitude: 10.1107997894,
		Timezone:  "Europe/Oslo",
	},
	"ENGS": {
		ICAO:      "ENGS",
		Name:      "Snasa Airfield Gronora",
		City:      "Snasa",
		State:     "Nord-Trondelag",
		Country:   "NO",
		Elevation: 89,
		Latitude:  64.1835861206,
		Longitude: 12.1714782715,
		Timezone:  "Europe/Oslo",
	},
	"ENHA": {
		ICAO:      "ENHA",
		IATA:      "HMR",
		Name:      "Stafsberg Airport",
		City:      "Hamar",
		State:     "Hedmark",
		Country:   "NO",
		Elevation: 713,
		Latitude:  60.8180999756,
		Longitude: 11.0679998398,
		Timezone:  "Europe/Oslo",
	},
	"ENHD": {
		ICAO:      "ENHD",
		IATA:      "HAU",
		Name:      "Haugesund Airport",
		City:      "Karmoy",
		State:     "Rogaland",
		Country:   "NO",
		Elevation: 86,
		Latitude:  59.3452987671,
		Longitude: 5.2083601952,
		Timezone:  "Europe/Oslo",
	},
	"ENHF": {
		ICAO:      "ENHF",
		IATA:      "HFT",
		Name:      "Hammerfest Airport",
		City:      "Hammerfest",
		State:     "Finnmark",
		Country:   "NO",
		Elevation: 266,
		Latitude:  70.6797027588,
		Longitude: 23.6686000824,
		Timezone:  "Europe/Oslo",
	},
	"ENHK": {
		ICAO:      "ENHK",
		IATA:      "HAA",
		Name:      "Hasvik Airport",
		City:      "Hasvik",
		State:     "Finnmark",
		Country:   "NO",
		Elevation: 21,
		Latitude:  70.4867019653,
		Longitude: 22.1396999359,
		Timezone:  "Europe/Oslo",
	},
	"ENHN": {
		ICAO:      "ENHN",
		Name:      "Elverum Starmoen Airport",
		State:     "Hedmark",
		Country:   "NO",
		Elevation: 659,
		Latitude:  60.8800010681,
		Longitude: 11.6731004715,
		Timezone:  "Europe/Oslo",
	},
	"ENHS": {
		ICAO:      "ENHS",
		Name:      "Hokksund Airfield",
		State:     "Buskerud",
		Country:   "NO",
		Elevation: 30,
		Latitude:  59.7668991089,
		Longitude: 9.9113903046,
		Timezone:  "Europe/Oslo",
	},
	"ENHT": {
		ICAO:      "ENHT",
		Name:      "Hattfjelldal Airport",
		City:      "Hattfjelldal",
		State:     "Nordland",
		Country:   "NO",
		Elevation: 689,
		Latitude:  65.5947036743,
		Longitude: 13.9891996384,
		Timezone:  "Europe/Oslo",
	},
	"ENHV": {
		ICAO:      "ENHV",
		IATA:      "HVG",
		Name:      "Valan Airport",
		City:      "Honningsvag",
		State:     "Finnmark",
		Country:   "NO",
		Elevation: 44,
		Latitude:  71.0096969604,
		Longitude: 25.9836006165,
		Timezone:  "Europe/Oslo",
	},
	"ENJA": {
		ICAO:      "ENJA",
		Name:      "Jan Mayen Jan Mayensfield",
		State:     "Jan-Mayen",
		Country:   "NO",
		Elevation: 39,
		Latitude:  70.9611111111,
		Longitude: -8.5758333333,
		Timezone:  "Arctic/Longyearbyen",
	},
	"ENJB": {
		ICAO:      "ENJB",
		Name:      "Jarlsberg Airfield",
		City:      "Tonsberg",
		State:     "Vestfold",
		Country:   "NO",
		Elevation: 92,
		Latitude:  59.2999992371,
		Longitude: 10.3667001724,
		Timezone:  "Europe/Oslo",
	},
	"ENKA": {
		ICAO:      "ENKA",
		IATA:      "QKX",
		Name:      "Kautokeino Air Base",
		State:     "Finnmark",
		Country:   "NO",
		Elevation: 1165,
		Latitude:  69.0402984619,
		Longitude: 23.0340003967,
		Timezone:  "Europe/Oslo",
	},
	"ENKB": {
		ICAO:      "ENKB",
		IATA:      "KSU",
		Name:      "Kristiansund Airport Kvernberget",
		City:      "Kvernberget",
		State:     "More-og-Romsdal",
		Country:   "NO",
		Elevation: 204,
		Latitude:  63.1118011475,
		Longitude: 7.8245201111,
		Timezone:  "Europe/Oslo",
	},
	"ENKJ": {
		ICAO:      "ENKJ",
		Name:      "Kjeller Airport",
		City:      "Kjeller",
		State:     "Akershus",
		Country:   "NO",
		Elevation: 354,
		Latitude:  59.9692993164,
		Longitude: 11.0361003876,
		Timezone:  "Europe/Oslo",
	},
	"ENKL": {
		ICAO:      "ENKL",
		IATA:      "GLL",
		Name:      "Gol Airport",
		City:      "Klanten",
		State:     "Buskerud",
		Country:   "NO",
		Elevation: 2720,
		Latitude:  60.7914009094,
		Longitude: 9.0505599976,
		Timezone:  "Europe/Oslo",
	},
	"ENKR": {
		ICAO:      "ENKR",
		IATA:      "KKN",
		Name:      "Kirkenes Airport Hoybuktmoen",
		City:      "Kirkenes",
		State:     "Finnmark",
		Country:   "NO",
		Elevation: 283,
		Latitude:  69.7257995605,
		Longitude: 29.8913002014,
		Timezone:  "Europe/Oslo",
	},
	"ENLI": {
		ICAO:      "ENLI",
		IATA:      "FAN",
		Name:      "Lista Airport",
		City:      "Farsund",
		State:     "Vest-Agder",
		Country:   "NO",
		Elevation: 29,
		Latitude:  58.0994987488,
		Longitude: 6.6260499954,
		Timezone:  "Europe/Oslo",
	},
	"ENLK": {
		ICAO:      "ENLK",
		IATA:      "LKN",
		Name:      "Leknes Airport",
		City:      "Leknes",
		State:     "Nordland",
		Country:   "NO",
		Elevation: 78,
		Latitude:  68.1524963379,
		Longitude: 13.6093997955,
		Timezone:  "Europe/Oslo",
	},
	"ENLU": {
		ICAO:      "ENLU",
		Name:      "Lunde Nome Airfield",
		City:      "Lunde Nome",
		State:     "Telemark",
		Country:   "NO",
		Elevation: 190,
		Latitude:  59.2980995178,
		Longitude: 9.1327295303,
		Timezone:  "Europe/Oslo",
	},
	"ENMH": {
		ICAO:      "ENMH",
		IATA:      "MEH",
		Name:      "Mehamn Airport",
		City:      "Mehamn",
		State:     "Finnmark",
		Country:   "NO",
		Elevation: 39,
		Latitude:  71.0297012329,
		Longitude: 27.8267002106,
		Timezone:  "Europe/Oslo",
	},
	"ENML": {
		ICAO:      "ENML",
		IATA:      "MOL",
		Name:      "Molde Airport",
		City:      "Aro",
		State:     "More-og-Romsdal",
		Country:   "NO",
		Elevation: 10,
		Latitude:  62.7447013855,
		Longitude: 7.2624998093,
		Timezone:  "Europe/Oslo",
	},
	"ENMS": {
		ICAO:      "ENMS",
		IATA:      "MJF",
		Name:      "Mosjoen Airport Kjaerstad",
		State:     "Nordland",
		Country:   "NO",
		Elevation: 237,
		Latitude:  65.783996582,
		Longitude: 13.2149000168,
		Timezone:  "Europe/Oslo",
	},
	"ENNA": {
		ICAO:      "ENNA",
		IATA:      "LKL",
		Name:      "Banak Airport",
		City:      "Lakselv",
		State:     "Finnmark",
		Country:   "NO",
		Elevation: 25,
		Latitude:  70.0688018799,
		Longitude: 24.9734992981,
		Timezone:  "Europe/Oslo",
	},
	"ENNK": {
		ICAO:      "ENNK",
		IATA:      "NVK",
		Name:      "Narvik Framnes Airport",
		City:      "Narvik",
		State:     "Nordland",
		Country:   "NO",
		Elevation: 95,
		Latitude:  68.4368972778,
		Longitude: 17.3866996765,
		Timezone:  "Europe/Oslo",
	},
	"ENNM": {
		ICAO:      "ENNM",
		IATA:      "OSY",
		Name:      "Namsos Hoknesora Airport",
		City:      "Namsos",
		State:     "Nord-Trondelag",
		Country:   "NO",
		Elevation: 7,
		Latitude:  64.4721984863,
		Longitude: 11.5785999298,
		Timezone:  "Europe/Oslo",
	},
	"ENNO": {
		ICAO:      "ENNO",
		IATA:      "NTB",
		Name:      "Notodden Airport",
		State:     "Telemark",
		Country:   "NO",
		Elevation: 63,
		Latitude:  59.565700531,
		Longitude: 9.212220192,
		Timezone:  "Europe/Oslo",
	},
	"ENOL": {
		ICAO:      "ENOL",
		IATA:      "OLA",
		Name:      "Orland Airport",
		City:      "Orland",
		State:     "Sor-Trondelag",
		Country:   "NO",
		Elevation: 28,
		Latitude:  63.6988983154,
		Longitude: 9.6040000916,
		Timezone:  "Europe/Oslo",
	},
	"ENOP": {
		ICAO:      "ENOP",
		Name:      "Fagerhaug Airport",
		State:     "Sor-Trondelag",
		Country:   "NO",
		Elevation: 1830,
		Latitude:  62.6498985291,
		Longitude: 9.846739769,
		Timezone:  "Europe/Oslo",
	},
	"ENOV": {
		ICAO:      "ENOV",
		IATA:      "HOV",
		Name:      "Orsta-Volda Airport Hovden",
		City:      "Orsta",
		State:     "More-og-Romsdal",
		Country:   "NO",
		Elevation: 243,
		Latitude:  62.1800003052,
		Longitude: 6.0741000175,
		Timezone:  "Europe/Oslo",
	},
	"ENRA": {
		ICAO:      "ENRA",
		IATA:      "MQN",
		Name:      "Mo i Rana Airport Rossvoll",
		City:      "Mo i Rana",
		State:     "Nordland",
		Country:   "NO",
		Elevation: 229,
		Latitude:  66.363899231,
		Longitude: 14.3014001846,
		Timezone:  "Europe/Oslo",
	},
	"ENRG": {
		ICAO:      "ENRG",
		Name:      "Rognan Airport",
		City:      "Saltdal",
		State:     "Nordland",
		Country:   "NO",
		Elevation: 20,
		Latitude:  67.0982971191,
		Longitude: 15.4111003876,
		Timezone:  "Europe/Oslo",
	},
	"ENRI": {
		ICAO:      "ENRI",
		Name:      "Ringebu Airfield Frya",
		City:      "Frya",
		State:     "Oppland",
		Country:   "NO",
		Elevation: 571,
		Latitude:  61.5454406738,
		Longitude: 10.0615882874,
		Timezone:  "Europe/Oslo",
	},
	"ENRK": {
		ICAO:      "ENRK",
		Name:      "Rakkestad Astorp Airport",
		State:     "Ostfold",
		Country:   "NO",
		Elevation: 381,
		Latitude:  59.3974990845,
		Longitude: 11.3468999863,
		Timezone:  "Europe/Oslo",
	},
	"ENRM": {
		ICAO:      "ENRM",
		IATA:      "RVK",
		Name:      "Rorvik Airport Ryum",
		City:      "Rorvik",
		State:     "Nord-Trondelag",
		Country:   "NO",
		Elevation: 14,
		Latitude:  64.8383026123,
		Longitude: 11.1461000443,
		Timezone:  "Europe/Oslo",
	},
	"ENRO": {
		ICAO:      "ENRO",
		IATA:      "RRS",
		Name:      "Roros Airport",
		City:      "Roros",
		State:     "Sor-Trondelag",
		Country:   "NO",
		Elevation: 2054,
		Latitude:  62.5783996582,
		Longitude: 11.342300415,
		Timezone:  "Europe/Oslo",
	},
	"ENRS": {
		ICAO:      "ENRS",
		IATA:      "RET",
		Name:      "Rost Airport",
		State:     "Nordland",
		Country:   "NO",
		Elevation: 7,
		Latitude:  67.5278015137,
		Longitude: 12.1033000946,
		Timezone:  "Europe/Oslo",
	},
	"ENRV": {
		ICAO:      "ENRV",
		Name:      "Reinsvoll Airport",
		City:      "Reinsvoll",
		State:     "Oppland",
		Country:   "NO",
		Elevation: 1381,
		Latitude:  60.6721992493,
		Longitude: 10.5675001144,
		Timezone:  "Europe/Oslo",
	},
	"ENRY": {
		ICAO:      "ENRY",
		IATA:      "RYG",
		Name:      "Moss Airport Rygge",
		City:      "Rygge",
		State:     "Ostfold",
		Country:   "NO",
		Elevation: 174,
		Latitude:  59.3788986206,
		Longitude: 10.7855997086,
		Timezone:  "Europe/Oslo",
	},
	"ENSA": {
		ICAO:      "ENSA",
		IATA:      "SRP",
		Name:      "Svea Airport",
		City:      "Svea",
		State:     "Svalbard",
		Country:   "NO",
		Elevation: 29,
		Latitude:  77.9000015259,
		Longitude: 16.6833000183,
		Timezone:  "Arctic/Longyearbyen",
	},
	"ENSB": {
		ICAO:      "ENSB",
		IATA:      "LYR",
		Name:      "Svalbard Airport Longyear",
		City:      "Longyearbyen",
		State:     "Svalbard",
		Country:   "NO",
		Elevation: 88,
		Latitude:  78.2461013794,
		Longitude: 15.4656000137,
		Timezone:  "Arctic/Longyearbyen",
	},
	"ENSD": {
		ICAO:      "ENSD",
		IATA:      "SDN",
		Name:      "Sandane Airport Anda",
		City:      "Sandane",
		State:     "Sogn-og-Fjordane",
		Country:   "NO",
		Elevation: 196,
		Latitude:  61.8300018311,
		Longitude: 6.1058301926,
		Timezone:  "Europe/Oslo",
	},
	"ENSG": {
		ICAO:      "ENSG",
		IATA:      "SOG",
		Name:      "Sogndal Airport",
		City:      "Sogndal",
		State:     "Sogn-og-Fjordane",
		Country:   "NO",
		Latitude:  61.1561012268,
		Longitude: 7.1377801895,
		Timezone:  "Europe/Oslo",
	},
	"ENSH": {
		ICAO:      "ENSH",
		IATA:      "SVJ",
		Name:      "Svolvaer Helle Airport",
		City:      "Svolvaer",
		State:     "Nordland",
		Country:   "NO",
		Elevation: 27,
		Latitude:  68.2433013916,
		Longitude: 14.6691999435,
		Timezone:  "Europe/Oslo",
	},
	"ENSI": {
		ICAO:      "ENSI",
		Name:      "Ski Airfield",
		City:      "Ski",
		State:     "Akershus",
		Country:   "NO",
		Elevation: 350,
		Latitude:  59.7063903809,
		Longitude: 10.8819446564,
		Timezone:  "Europe/Oslo",
	},
	"ENSK": {
		ICAO:      "ENSK",
		IATA:      "SKN",
		Name:      "Stokmarknes Skagen Airport",
		City:      "Hadsel",
		State:     "Nordland",
		Country:   "NO",
		Elevation: 11,
		Latitude:  68.5788269043,
		Longitude: 15.033416748,
		Timezone:  "Europe/Oslo",
	},
	"ENSN": {
		ICAO:      "ENSN",
		IATA:      "SKE",
		Name:      "Skien Airport",
		City:      "Geiteryggen",
		State:     "Telemark",
		Country:   "NO",
		Elevation: 463,
		Latitude:  59.1850013733,
		Longitude: 9.5669403076,
		Timezone:  "Europe/Oslo",
	},
	"ENSO": {
		ICAO:      "ENSO",
		IATA:      "SRP",
		Name:      "Stord Airport",
		City:      "Leirvik",
		State:     "Hordaland",
		Country:   "NO",
		Elevation: 160,
		Latitude:  59.7919006348,
		Longitude: 5.3408498764,
		Timezone:  "Europe/Oslo",
	},
	"ENSR": {
		ICAO:      "ENSR",
		IATA:      "SOJ",
		Name:      "Sorkjosen Airport",
		City:      "Sorkjosen",
		State:     "Troms",
		Country:   "NO",
		Elevation: 16,
		Latitude:  69.7867965698,
		Longitude: 20.959400177,
		Timezone:  "Europe/Oslo",
	},
	"ENSS": {
		ICAO:      "ENSS",
		IATA:      "VAW",
		Name:      "Vardo Airport Svartnes",
		City:      "Vardo",
		State:     "Finnmark",
		Country:   "NO",
		Elevation: 42,
		Latitude:  70.3554000854,
		Longitude: 31.0449008942,
		Timezone:  "Europe/Oslo",
	},
	"ENST": {
		ICAO:      "ENST",
		IATA:      "SSJ",
		Name:      "Sandnessjoen Airport Stokka",
		City:      "Alstahaug",
		State:     "Nordland",
		Country:   "NO",
		Elevation: 56,
		Latitude:  65.9568023682,
		Longitude: 12.4688997269,
		Timezone:  "Europe/Oslo",
	},
	"ENSU": {
		ICAO:      "ENSU",
		Name:      "Vinnu Airport",
		City:      "Sunndalsora",
		State:     "More-og-Romsdal",
		Country:   "NO",
		Elevation: 249,
		Latitude:  62.6582984924,
		Longitude: 8.6649999619,
		Timezone:  "Europe/Oslo",
	},
	"ENTC": {
		ICAO:      "ENTC",
		IATA:      "TOS",
		Name:      "Tromso Airport",
		City:      "Tromso",
		State:     "Troms",
		Country:   "NO",
		Elevation: 31,
		Latitude:  69.6832962036,
		Longitude: 18.9188995361,
		Timezone:  "Europe/Oslo",
	},
	"ENTO": {
		ICAO:      "ENTO",
		IATA:      "TRF",
		Name:      "Sandefjord Airport Torp",
		City:      "Torp",
		State:     "Vestfold",
		Country:   "NO",
		Elevation: 286,
		Latitude:  59.1866989136,
		Longitude: 10.258600235,
		Timezone:  "Europe/Oslo",
	},
	"ENTS": {
		ICAO:      "ENTS",
		Name:      "Trysil Saeterasen Airport",
		State:     "Hedmark",
		Country:   "NO",
		Elevation: 1900,
		Latitude:  61.2292785645,
		Longitude: 12.2664070129,
		Timezone:  "Europe/Oslo",
	},
	"ENTY": {
		ICAO:      "ENTY",
		Name:      "Tynset Airport",
		City:      "Tynset",
		State:     "Hedmark",
		Country:   "NO",
		Elevation: 1581,
		Latitude:  62.2569007874,
		Longitude: 10.6694002151,
		Timezone:  "Europe/Oslo",
	},
	"ENUL": {
		ICAO:      "ENUL",
		Name:      "Os Vaksinen Ulve Airport",
		City:      "Ulven",
		State:     "Hordaland",
		Country:   "NO",
		Elevation: 180,
		Latitude:  60.192199707,
		Longitude: 5.4225001335,
		Timezone:  "Europe/Oslo",
	},
	"ENVA": {
		ICAO:      "ENVA",
		IATA:      "TRD",
		Name:      "Trondheim Airport Vaernes",
		City:      "Trondheim",
		State:     "Nord-Trondelag",
		Country:   "NO",
		Elevation: 56,
		Latitude:  63.4578018188,
		Longitude: 10.9239997864,
		Timezone:  "Europe/Oslo",
	},
	"ENVD": {
		ICAO:      "ENVD",
		IATA:      "VDS",
		Name:      "Vadso Airport",
		City:      "Vadso",
		State:     "Finnmark",
		Country:   "NO",
		Elevation: 127,
		Latitude:  70.0652999878,
		Longitude: 29.8446998596,
		Timezone:  "Europe/Oslo",
	},
	"ENVE": {
		ICAO:      "ENVE",
		Name:      "Valle Airfield Araksoyne",
		State:     "Aust-Agder",
		Country:   "NO",
		Elevation: 771,
		Latitude:  59.0317230225,
		Longitude: 7.5549030304,
		Timezone:  "Europe/Oslo",
	},
	"ENZV": {
		ICAO:      "ENZV",
		IATA:      "SVG",
		Name:      "Stavanger Airport Sola",
		City:      "Stavanger",
		State:     "Rogaland",
		Country:   "NO",
		Elevation: 29,
		Latitude:  58.876701355,
		Longitude: 5.6377801895,
		Timezone:  "Europe/Oslo",
	},
	"EPAR": {
		ICAO:      "EPAR",
		Name:      "Arlamow Airport",
		City:      "Bircza",
		State:     "Subcarpathian-Voivodeship",
		Country:   "PL",
		Elevation: 426,
		Latitude:  49.6575012207,
		Longitude: 22.5142993927,
		Timezone:  "Europe/Warsaw",
	},
	"EPBA": {
		ICAO:      "EPBA",
		Name:      "Bielsko Biala Airport",
		State:     "Silesia",
		Country:   "PL",
		Elevation: 1319,
		Latitude:  49.8050003052,
		Longitude: 19.0018997192,
		Timezone:  "Europe/Warsaw",
	},
	"EPBC": {
		ICAO:      "EPBC",
		Name:      "Babice Airport",
		State:     "Mazovia",
		Country:   "PL",
		Elevation: 352,
		Latitude:  52.2685012817,
		Longitude: 20.9109992981,
		Timezone:  "Europe/Warsaw",
	},
	"EPBD": {
		ICAO:      "EPBD",
		Name:      "Bydgoszcz-Biedaszkowo Airport",
		City:      "Bydgoszcz",
		State:     "Kujawsko-Pomorskie",
		Country:   "PL",
		Elevation: 230,
		Latitude:  53.103099823,
		Longitude: 17.9556007385,
		Timezone:  "Europe/Warsaw",
	},
	"EPBK": {
		ICAO:      "EPBK",
		IATA:      "QYY",
		Name:      "Bialystok-Krywlany Airport",
		City:      "Bialystok",
		State:     "Podlasie",
		Country:   "PL",
		Elevation: 502,
		Latitude:  53.101398468,
		Longitude: 23.1706008911,
		Timezone:  "Europe/Warsaw",
	},
	"EPBP": {
		ICAO:      "EPBP",
		IATA:      "BXP",
		Name:      "Biala Podlaska Airport",
		City:      "Biala Podlaska",
		State:     "Lublin",
		Country:   "PL",
		Elevation: 495,
		Latitude:  52.0057983398,
		Longitude: 23.1436004639,
		Timezone:  "Europe/Warsaw",
	},
	"EPBS": {
		ICAO:      "EPBS",
		Name:      "Borne Sulinowo Airport",
		City:      "Borne Sulinowo",
		State:     "West-Pomerania",
		Country:   "PL",
		Elevation: 469,
		Latitude:  53.5750007629,
		Longitude: 16.5242004395,
		Timezone:  "Europe/Warsaw",
	},
	"EPBY": {
		ICAO:      "EPBY",
		IATA:      "BZG",
		Name:      "Bydgoszcz Ignacy Jan Paderewski Airport",
		City:      "Bydgoszcz",
		State:     "Kujawsko-Pomorskie",
		Country:   "PL",
		Elevation: 235,
		Latitude:  53.0968017578,
		Longitude: 17.9776992798,
		Timezone:  "Europe/Warsaw",
	},
	"EPCE": {
		ICAO:      "EPCE",
		Name:      "Cewice Military Airport",
		City:      "Lebork",
		State:     "Pomerania",
		Country:   "PL",
		Elevation: 495,
		Latitude:  54.4160003662,
		Longitude: 17.763299942,
		Timezone:  "Europe/Warsaw",
	},
	"EPCH": {
		ICAO:      "EPCH",
		IATA:      "CZW",
		Name:      "Czestochowa-Rudniki",
		City:      "Czestochowa",
		State:     "Silesia",
		Country:   "PL",
		Elevation: 860,
		Latitude:  50.8849983215,
		Longitude: 19.20470047,
		Timezone:  "Europe/Warsaw",
	},
	"EPDA": {
		ICAO:      "EPDA",
		Name:      "Darlowek Naval Air Base",
		City:      "Darlowo",
		State:     "West-Pomerania",
		Country:   "PL",
		Elevation: 10,
		Latitude:  54.4047012329,
		Longitude: 16.353099823,
		Timezone:  "Europe/Warsaw",
	},
	"EPDE": {
		ICAO:      "EPDE",
		Name:      "Deblin Military Air Base",
		City:      "Deblin",
		State:     "Lublin",
		Country:   "PL",
		Elevation: 392,
		Latitude:  51.551399231,
		Longitude: 21.8936004639,
		Timezone:  "Europe/Warsaw",
	},
	"EPDR": {
		ICAO:      "EPDR",
		Name:      "Ziemsko Airport",
		City:      "Drawsko Pomorskie",
		State:     "West-Pomerania",
		Country:   "PL",
		Elevation: 377,
		Latitude:  53.4777984619,
		Longitude: 15.730799675,
		Timezone:  "Europe/Warsaw",
	},
	"EPEK": {
		ICAO:      "EPEK",
		Name:      "Elk-Makosieje Airport",
		City:      "Elk",
		State:     "Warmia-Masuria",
		Country:   "PL",
		Elevation: 416,
		Latitude:  53.8072222,
		Longitude: 22.5408333,
		Timezone:  "Europe/Warsaw",
	},
	"EPEL": {
		ICAO:      "EPEL",
		Name:      "Elblag Airport",
		State:     "Warmia-Masuria",
		Country:   "PL",
		Elevation: 10,
		Latitude:  54.1408004761,
		Longitude: 19.4232997894,
		Timezone:  "Europe/Warsaw",
	},
	"EPGD": {
		ICAO:      "EPGD",
		IATA:      "GDN",
		Name:      "Gdansk Lech Walesa Airport",
		City:      "Gdansk",
		State:     "Pomerania",
		Country:   "PL",
		Elevation: 489,
		Latitude:  54.3776016235,
		Longitude: 18.4661998749,
		Timezone:  "Europe/Warsaw",
	},
	"EPGI": {
		ICAO:      "EPGI",
		Name:      "Grudziadz Lisie Airport",
		State:     "Kujawsko-Pomorskie",
		Country:   "PL",
		Elevation: 121,
		Latitude:  53.5243988037,
		Longitude: 18.849199295,
		Timezone:  "Europe/Warsaw",
	},
	"EPGL": {
		ICAO:      "EPGL",
		IATA:      "QLC",
		Name:      "Gliwice Glider Airport",
		State:     "Silesia",
		Country:   "PL",
		Elevation: 830,
		Latitude:  50.2694015503,
		Longitude: 18.6728000641,
		Timezone:  "Europe/Warsaw",
	},
	"EPGM": {
		ICAO:      "EPGM",
		Name:      "Gizycko-Mazury Residence",
		City:      "Gizycko",
		State:     "Warmia-Masuria",
		Country:   "PL",
		Elevation: 410,
		Latitude:  54.00645,
		Longitude: 21.81824,
		Timezone:  "Europe/Warsaw",
	},
	"EPGO": {
		ICAO:      "EPGO",
		Name:      "Goraszka Airport",
		City:      "Sulejowek",
		State:     "Mazovia",
		Country:   "PL",
		Elevation: 361,
		Latitude:  52.1844406128,
		Longitude: 21.2811107635,
		Timezone:  "Europe/Warsaw",
	},
	"EPIN": {
		ICAO:      "EPIN",
		Name:      "Inowroclaw Glider Airport",
		State:     "Kujawsko-Pomorskie",
		Country:   "PL",
		Elevation: 279,
		Latitude:  52.8064002991,
		Longitude: 18.2858009338,
		Timezone:  "Europe/Warsaw",
	},
	"EPIR": {
		ICAO:      "EPIR",
		Name:      "Inowroclaw Military Air Base",
		State:     "Kujawsko-Pomorskie",
		Country:   "PL",
		Elevation: 259,
		Latitude:  52.8293991089,
		Longitude: 18.3306007385,
		Timezone:  "Europe/Warsaw",
	},
	"EPIW": {
		ICAO:      "EPIW",
		Name:      "Iwonicz Airport",
		City:      "Krosno",
		State:     "Subcarpathian-Voivodeship",
		Country:   "PL",
		Elevation: 971,
		Latitude:  49.6577987671,
		Longitude: 21.8174991608,
		Timezone:  "Europe/Warsaw",
	},
	"EPJA": {
		ICAO:      "EPJA",
		Name:      "Jastarnia Airport",
		City:      "Jastarnia",
		State:     "Pomerania",
		Country:   "PL",
		Elevation: 3,
		Latitude:  54.7102813721,
		Longitude: 18.645280838,
		Timezone:  "Europe/Warsaw",
	},
	"EPJG": {
		ICAO:      "EPJG",
		Name:      "Jelenia Gora Glider Airport",
		City:      "Jelenia Gora",
		State:     "Lower-Silesia",
		Country:   "PL",
		Elevation: 1119,
		Latitude:  50.8988990784,
		Longitude: 15.7855997086,
		Timezone:  "Europe/Warsaw",
	},
	"EPJS": {
		ICAO:      "EPJS",
		Name:      "Jezow Sudecki Airport",
		City:      "Jelenia Gora",
		State:     "Lower-Silesia",
		Country:   "PL",
		Elevation: 1834,
		Latitude:  50.9283294678,
		Longitude: 15.7597198486,
		Timezone:  "Europe/Warsaw",
	},
	"EPKA": {
		ICAO:      "EPKA",
		Name:      "Kielce Maslow Airport",
		City:      "Kielce",
		State:     "Swietokrzyskie",
		Country:   "PL",
		Elevation: 1010,
		Latitude:  50.8967018127,
		Longitude: 20.7317008972,
		Timezone:  "Europe/Warsaw",
	},
	"EPKB": {
		ICAO:      "EPKB",
		Name:      "Kazimierz Biskup Airport",
		State:     "Greater-Poland",
		Country:   "PL",
		Elevation: 361,
		Latitude:  52.3199996948,
		Longitude: 18.1700000763,
		Timezone:  "Europe/Warsaw",
	},
	"EPKE": {
		ICAO:      "EPKE",
		Name:      "Ketrzyn-Wilamowo Airport",
		City:      "Ketrzyn",
		State:     "Warmia-Masuria",
		Country:   "PL",
		Elevation: 417,
		Latitude:  54.043598175,
		Longitude: 21.4321994781,
		Timezone:  "Europe/Warsaw",
	},
	"EPKK": {
		ICAO:      "EPKK",
		IATA:      "KRK",
		Name:      "John Paul II International Airport Krakow-Balice Airport",
		City:      "Krakow",
		State:     "Lesser-Poland-Voivodeship",
		Country:   "PL",
		Elevation: 791,
		Latitude:  50.0777015686,
		Longitude: 19.7847995758,
		Timezone:  "Europe/Warsaw",
	},
	"EPKM": {
		ICAO:      "EPKM",
		Name:      "Muchowiec Airport",
		State:     "Silesia",
		Country:   "PL",
		Elevation: 909,
		Latitude:  50.2380981445,
		Longitude: 19.0342006683,
		Timezone:  "Europe/Warsaw",
	},
	"EPKN": {
		ICAO:      "EPKN",
		Name:      "Kamien Slaski Airport",
		State:     "Opole-Voivodeship",
		Country:   "PL",
		Elevation: 683,
		Latitude:  50.5292015076,
		Longitude: 18.0848999023,
		Timezone:  "Europe/Warsaw",
	},
	"EPKO": {
		ICAO:      "EPKO",
		IATA:      "OSZ",
		Name:      "Koszalin Zegrze Airport",
		State:     "West-Pomerania",
		Country:   "PL",
		Elevation: 249,
		Latitude:  54.0424995422,
		Longitude: 16.2656002045,
		Timezone:  "Europe/Warsaw",
	},
	"EPKP": {
		ICAO:      "EPKP",
		Name:      "Pobiednik Wielki Airport",
		City:      "Krakow",
		State:     "Lesser-Poland-Voivodeship",
		Country:   "PL",
		Elevation: 650,
		Latitude:  50.0896987915,
		Longitude: 20.2017002106,
		Timezone:  "Europe/Warsaw",
	},
	"EPKR": {
		ICAO:      "EPKR",
		Name:      "Krosno Airport",
		City:      "Krosno",
		State:     "Subcarpathian-Voivodeship",
		Country:   "PL",
		Elevation: 922,
		Latitude:  49.681098938,
		Longitude: 21.7371997833,
		Timezone:  "Europe/Warsaw",
	},
	"EPKS": {
		ICAO:      "EPKS",
		Name:      "Krzesiny Military Air Base",
		State:     "Greater-Poland",
		Country:   "PL",
		Elevation: 265,
		Latitude:  52.3316993713,
		Longitude: 16.9664001465,
		Timezone:  "Europe/Warsaw",
	},
	"EPKT": {
		ICAO:      "EPKT",
		IATA:      "KTW",
		Name:      "Katowice International Airport",
		City:      "Katowice",
		State:     "Silesia",
		Country:   "PL",
		Elevation: 995,
		Latitude:  50.4743003845,
		Longitude: 19.0799999237,
		Timezone:  "Europe/Warsaw",
	},
	"EPKW": {
		ICAO:      "EPKW",
		IATA:      "QEO",
		Name:      "Bielsko-Bialo Kaniow Airfield",
		City:      "Czechowice-Dziedzice",
		State:     "Silesia",
		Country:   "PL",
		Elevation: 1316,
		Latitude:  49.9399986267,
		Longitude: 19.0219993591,
		Timezone:  "Europe/Warsaw",
	},
	"EPLB": {
		ICAO:      "EPLB",
		IATA:      "LUZ",
		Name:      "Lublin Airport",
		City:      "Lublin",
		State:     "Lublin",
		Country:   "PL",
		Elevation: 633,
		Latitude:  51.240278,
		Longitude: 22.713611,
		Timezone:  "Europe/Warsaw",
	},
	"EPLE": {
		ICAO:      "EPLE",
		Name:      "Legnica Airport",
		State:     "Lower-Silesia",
		Country:   "PL",
		Elevation: 407,
		Latitude:  51.182800293,
		Longitude: 16.1781997681,
		Timezone:  "Europe/Warsaw",
	},
	"EPLK": {
		ICAO:      "EPLK",
		Name:      "Lask Military Air Base",
		State:     "Kodz-Voivodeship",
		Country:   "PL",
		Elevation: 633,
		Latitude:  51.551700592,
		Longitude: 19.1791000366,
		Timezone:  "Europe/Warsaw",
	},
	"EPLL": {
		ICAO:      "EPLL",
		IATA:      "LCJ",
		Name:      "Lodz Wladyslaw Reymont Airport",
		City:      "Lodz",
		State:     "Kodz-Voivodeship",
		Country:   "PL",
		Elevation: 604,
		Latitude:  51.7219009399,
		Longitude: 19.3980998993,
		Timezone:  "Europe/Warsaw",
	},
	"EPLN": {
		ICAO:      "EPLN",
		Name:      "Lansk / Gryzliny Airport",
		City:      "Olsztynek",
		State:     "Warmia-Masuria",
		Country:   "PL",
		Elevation: 538,
		Latitude:  53.6080589294,
		Longitude: 20.3444404602,
		Timezone:  "Europe/Warsaw",
	},
	"EPLR": {
		ICAO:      "EPLR",
		IATA:      "QLU",
		Name:      "Lublin Radwiec Airport",
		State:     "Lublin",
		Country:   "PL",
		Elevation: 791,
		Latitude:  51.2219009399,
		Longitude: 22.3946990967,
		Timezone:  "Europe/Warsaw",
	},
	"EPLS": {
		ICAO:      "EPLS",
		Name:      "Leszno Strzyzewi Airport",
		City:      "Leszno",
		State:     "Greater-Poland",
		Country:   "PL",
		Elevation: 312,
		Latitude:  51.8349990845,
		Longitude: 16.521900177,
		Timezone:  "Europe/Warsaw",
	},
	"EPLU": {
		ICAO:      "EPLU",
		Name:      "Lubin Airport",
		State:     "Lower-Silesia",
		Country:   "PL",
		Elevation: 512,
		Latitude:  51.4230003357,
		Longitude: 16.1961994171,
		Timezone:  "Europe/Warsaw",
	},
	"EPLY": {
		ICAO:      "EPLY",
		Name:      "Leczyca Military Air Base",
		City:      "Leczyca",
		State:     "Kodz-Voivodeship",
		Country:   "PL",
		Elevation: 377,
		Latitude:  52.004699707,
		Longitude: 19.1455993652,
		Timezone:  "Europe/Warsaw",
	},
	"EPMB": {
		ICAO:      "EPMB",
		Name:      "Malbork Military Air Base",
		State:     "Pomerania",
		Country:   "PL",
		Elevation: 16,
		Latitude:  54.0269012451,
		Longitude: 19.1341991425,
		Timezone:  "Europe/Warsaw",
	},
	"EPMI": {
		ICAO:      "EPMI",
		Name:      "Miroslawiec Military Air Base",
		City:      "Miroslawiec",
		State:     "West-Pomerania",
		Country:   "PL",
		Elevation: 459,
		Latitude:  53.3950996399,
		Longitude: 16.0827999115,
		Timezone:  "Europe/Warsaw",
	},
	"EPMJ": {
		ICAO:      "EPMJ",
		Name:      "Mikolajki Airport",
		City:      "Mikolajki",
		State:     "Warmia-Masuria",
		Country:   "PL",
		Elevation: 423,
		Latitude:  53.8077812195,
		Longitude: 21.5558300018,
		Timezone:  "Europe/Warsaw",
	},
	"EPML": {
		ICAO:      "EPML",
		Name:      "Mielec Airport",
		State:     "Subcarpathian-Voivodeship",
		Country:   "PL",
		Elevation: 548,
		Latitude:  50.3222999573,
		Longitude: 21.4620990753,
		Timezone:  "Europe/Warsaw",
	},
	"EPMM": {
		ICAO:      "EPMM",
		Name:      "Minsk Mazowiecki Military Air Base",
		State:     "Mazovia",
		Country:   "PL",
		Elevation: 604,
		Latitude:  52.1954994202,
		Longitude: 21.6558990479,
		Timezone:  "Europe/Warsaw",
	},
	"EPMO": {
		ICAO:      "EPMO",
		IATA:      "WMI",
		Name:      "Warsaw Modlin Airport",
		City:      "Warsaw",
		State:     "Mazovia",
		Country:   "PL",
		Elevation: 341,
		Latitude:  52.4510993958,
		Longitude: 20.6518001556,
		Timezone:  "Europe/Warsaw",
	},
	"EPMR": {
		ICAO:      "EPMR",
		Name:      "Miroslawice Private Airport",
		City:      "Sobotka",
		State:     "Lower-Silesia",
		Country:   "PL",
		Elevation: 495,
		Latitude:  50.9578018188,
		Longitude: 16.7702999115,
		Timezone:  "Europe/Warsaw",
	},
	"EPNL": {
		ICAO:      "EPNL",
		Name:      "Nowy Sacz-Lososina Dolna Airport",
		City:      "Nowy Sacz",
		State:     "Lesser-Poland-Voivodeship",
		Country:   "PL",
		Elevation: 830,
		Latitude:  49.7456016541,
		Longitude: 20.6236000061,
		Timezone:  "Europe/Warsaw",
	},
	"EPNM": {
		ICAO:      "EPNM",
		Name:      "Nowe Miasto nad Pilica Airfield",
		City:      "Nowe Miasto nad Pilica",
		State:     "Mazovia",
		Country:   "PL",
		Elevation: 512,
		Latitude:  51.625,
		Longitude: 20.5349998474,
		Timezone:  "Europe/Warsaw",
	},
	"EPNT": {
		ICAO:      "EPNT",
		IATA:      "QWS",
		Name:      "Nowy Targ Airport",
		City:      "Nowy Targ",
		State:     "Lesser-Poland-Voivodeship",
		Country:   "PL",
		Elevation: 2060,
		Latitude:  49.4627990723,
		Longitude: 20.0503005981,
		Timezone:  "Europe/Warsaw",
	},
	"EPOD": {
		ICAO:      "EPOD",
		Name:      "Olsztyn Dajtki Airport",
		State:     "Warmia-Masuria",
		Country:   "PL",
		Elevation: 440,
		Latitude:  53.7731018066,
		Longitude: 20.4150009155,
		Timezone:  "Europe/Warsaw",
	},
	"EPOK": {
		ICAO:      "EPOK",
		IATA:      "QYD",
		Name:      "Oksywie Military Air Base",
		City:      "Gdynia",
		State:     "Pomerania",
		Country:   "PL",
		Elevation: 144,
		Latitude:  54.57970047,
		Longitude: 18.51720047,
		Timezone:  "Europe/Warsaw",
	},
	"EPOM": {
		ICAO:      "EPOM",
		Name:      "Ostrow Airport",
		City:      "Ostrow",
		State:     "Greater-Poland",
		Country:   "PL",
		Elevation: 469,
		Latitude:  51.7010993958,
		Longitude: 17.8456001282,
		Timezone:  "Europe/Warsaw",
	},
	"EPOP": {
		ICAO:      "EPOP",
		IATA:      "QPM",
		Name:      "Opole-Polska Nowa Wies Airport",
		City:      "Opole",
		State:     "Opole-Voivodeship",
		Country:   "PL",
		Elevation: 620,
		Latitude:  50.6333007812,
		Longitude: 17.7817001343,
		Timezone:  "Europe/Warsaw",
	},
	"EPPB": {
		ICAO:      "EPPB",
		Name:      "Bednary Airfield",
		City:      "Pobiedziska",
		State:     "Greater-Poland",
		Country:   "PL",
		Elevation: 341,
		Latitude:  52.5344009399,
		Longitude: 17.2187995911,
		Timezone:  "Europe/Warsaw",
	},
	"EPPC": {
		ICAO:      "EPPC",
		Name:      "Pinczow Airport",
		City:      "Pinczow",
		State:     "Swietokrzyskie",
		Country:   "PL",
		Elevation: 610,
		Latitude:  50.5182991028,
		Longitude: 20.5149993896,
		Timezone:  "Europe/Warsaw",
	},
	"EPPI": {
		ICAO:      "EPPI",
		Name:      "Pila Airport",
		State:     "Greater-Poland",
		Country:   "PL",
		Elevation: 259,
		Latitude:  53.1699981689,
		Longitude: 16.7124996185,
		Timezone:  "Europe/Warsaw",
	},
	"EPPK": {
		ICAO:      "EPPK",
		Name:      "Poznan-Kobylnica Airport",
		City:      "Poznan",
		State:     "Greater-Poland",
		Country:   "PL",
		Elevation: 279,
		Latitude:  52.4338989258,
		Longitude: 17.0442008972,
		Timezone:  "Europe/Warsaw",
	},
	"EPPL": {
		ICAO:      "EPPL",
		Name:      "Plock Airport",
		State:     "Mazovia",
		Country:   "PL",
		Elevation: 331,
		Latitude:  52.5621986389,
		Longitude: 19.7213993073,
		Timezone:  "Europe/Warsaw",
	},
	"EPPO": {
		ICAO:      "EPPO",
		IATA:      "POZ",
		Name:      "Poznan-Lawica Airport",
		City:      "Poznan",
		State:     "Greater-Poland",
		Country:   "PL",
		Elevation: 308,
		Latitude:  52.4210014343,
		Longitude: 16.8262996674,
		Timezone:  "Europe/Warsaw",
	},
	"EPPR": {
		ICAO:      "EPPR",
		Name:      "Pruszcz Gdanski Airport",
		State:     "Pomerania",
		Country:   "PL",
		Elevation: 21,
		Latitude:  54.2480010986,
		Longitude: 18.6716003418,
		Timezone:  "Europe/Warsaw",
	},
	"EPPT": {
		ICAO:      "EPPT",
		Name:      "Piotrkow Trybunalski-Bujny Airport",
		City:      "Piotrkow Trybunalski",
		State:     "Kodz-Voivodeship",
		Country:   "PL",
		Elevation: 673,
		Latitude:  51.3830986023,
		Longitude: 19.6882991791,
		Timezone:  "Europe/Warsaw",
	},
	"EPPW": {
		ICAO:      "EPPW",
		Name:      "Powidz Military Air Base",
		State:     "Greater-Poland",
		Country:   "PL",
		Elevation: 371,
		Latitude:  52.3793983459,
		Longitude: 17.8539009094,
		Timezone:  "Europe/Warsaw",
	},
	"EPRA": {
		ICAO:      "EPRA",
		IATA:      "RDO",
		Name:      "Warsaw-Radom Airport",
		City:      "Radom",
		State:     "Mazovia",
		Country:   "PL",
		Elevation: 623,
		Latitude:  51.3891983032,
		Longitude: 21.213300705,
		Timezone:  "Europe/Warsaw",
	},
	"EPRG": {
		ICAO:      "EPRG",
		Name:      "Rybnik-Gotarowice Glider Airport",
		City:      "Rybnik",
		State:     "Silesia",
		Country:   "PL",
		Elevation: 840,
		Latitude:  50.0708007813,
		Longitude: 18.6282997131,
		Timezone:  "Europe/Warsaw",
	},
	"EPRJ": {
		ICAO:      "EPRJ",
		Name:      "Rzeszow Airport",
		City:      "Rzeszow",
		State:     "Subcarpathian-Voivodeship",
		Country:   "PL",
		Elevation: 659,
		Latitude:  50.1053009033,
		Longitude: 22.0480995178,
		Timezone:  "Europe/Warsaw",
	},
	"EPRP": {
		ICAO:      "EPRP",
		Name:      "Radom-Piastrow Glider Airport",
		City:      "Radom",
		State:     "Mazovia",
		Country:   "PL",
		Elevation: 479,
		Latitude:  51.4789009094,
		Longitude: 21.1100006104,
		Timezone:  "Europe/Warsaw",
	},
	"EPRU": {
		ICAO:      "EPRU",
		IATA:      "CZW",
		Name:      "Czestochowa-Rudniki Airport",
		City:      "Czestochowa",
		State:     "Silesia",
		Country:   "PL",
		Elevation: 860,
		Latitude:  50.886133,
		Longitude: 19.202472,
		Timezone:  "Europe/Warsaw",
	},
	"EPRZ": {
		ICAO:      "EPRZ",
		IATA:      "RZE",
		Name:      "Rzeszow-Jasionka Airport",
		City:      "Rzeszow",
		State:     "Subcarpathian-Voivodeship",
		Country:   "PL",
		Elevation: 675,
		Latitude:  50.1100006104,
		Longitude: 22.0189990997,
		Timezone:  "Europe/Warsaw",
	},
	"EPSC": {
		ICAO:      "EPSC",
		IATA:      "SZZ",
		Name:      "Szczecin-Goleniow Solidarnosc Airport",
		City:      "Goleniow",
		State:     "West-Pomerania",
		Country:   "PL",
		Elevation: 154,
		Latitude:  53.5847015381,
		Longitude: 14.9021997452,
		Timezone:  "Europe/Warsaw",
	},
	"EPSD": {
		ICAO:      "EPSD",
		Name:      "Szczecin-Dabie Airport",
		City:      "Szczecin",
		State:     "West-Pomerania",
		Country:   "PL",
		Elevation: 3,
		Latitude:  53.3921012878,
		Longitude: 14.6337995529,
		Timezone:  "Europe/Warsaw",
	},
	"EPSN": {
		ICAO:      "EPSN",
		Name:      "Swidwin Military Air Base",
		State:     "West-Pomerania",
		Country:   "PL",
		Elevation: 385,
		Latitude:  53.790599823,
		Longitude: 15.8262996674,
		Timezone:  "Europe/Warsaw",
	},
	"EPSO": {
		ICAO:      "EPSO",
		Name:      "Sochaczew Airport",
		State:     "Mazovia",
		Country:   "PL",
		Elevation: 259,
		Latitude:  52.198600769,
		Longitude: 20.2928009033,
		Timezone:  "Europe/Warsaw",
	},
	"EPSR": {
		ICAO:      "EPSR",
		Name:      "Slupsk-Krepa Airport",
		City:      "Slupsk",
		State:     "Pomerania",
		Country:   "PL",
		Elevation: 249,
		Latitude:  54.4082984924,
		Longitude: 17.0956001282,
		Timezone:  "Europe/Warsaw",
	},
	"EPST": {
		ICAO:      "EPST",
		Name:      "Stalowa Wola-Turbia Airport",
		City:      "Stalowa Wola",
		State:     "Subcarpathian-Voivodeship",
		Country:   "PL",
		Elevation: 489,
		Latitude:  50.6274986267,
		Longitude: 21.9983005524,
		Timezone:  "Europe/Warsaw",
	},
	"EPSU": {
		ICAO:      "EPSU",
		IATA:      "ZWK",
		Name:      "Suwalki Airport",
		City:      "Suwalki",
		State:     "Podlasie",
		Country:   "PL",
		Elevation: 581,
		Latitude:  54.0727996826,
		Longitude: 22.8992004395,
		Timezone:  "Europe/Warsaw",
	},
	"EPSW": {
		ICAO:      "EPSW",
		Name:      "Swidnik Lotnisko Airport",
		City:      "Swidnik",
		State:     "Lublin",
		Country:   "PL",
		Elevation: 659,
		Latitude:  51.2318992615,
		Longitude: 22.6902999878,
		Timezone:  "Europe/Warsaw",
	},
	"EPSY": {
		ICAO:      "EPSY",
		IATA:      "SZY",
		Name:      "Olsztyn-Mazury Airport",
		City:      "Szymany",
		State:     "Warmia-Mazury",
		Country:   "PL",
		Elevation: 464,
		Latitude:  53.481944,
		Longitude: 20.937778,
		Timezone:  "Europe/Warsaw",
	},
	"EPTM": {
		ICAO:      "EPTM",
		Name:      "Tomaszow Mazowiecki Military Air Base",
		City:      "Tomaszow Mazowiecki",
		State:     "Kodz-Voivodeship",
		Country:   "PL",
		Elevation: 571,
		Latitude:  51.584400177,
		Longitude: 20.0977993011,
		Timezone:  "Europe/Warsaw",
	},
	"EPTO": {
		ICAO:      "EPTO",
		Name:      "Torun Airport",
		State:     "Kujawsko-Pomorskie",
		Country:   "PL",
		Elevation: 164,
		Latitude:  53.0292015076,
		Longitude: 18.5459003448,
		Timezone:  "Europe/Warsaw",
	},
	"EPWA": {
		ICAO:      "EPWA",
		IATA:      "WAW",
		Name:      "Warsaw Chopin Airport",
		City:      "Warsaw",
		State:     "Mazovia",
		Country:   "PL",
		Elevation: 362,
		Latitude:  52.1656990051,
		Longitude: 20.9671001434,
		Timezone:  "Europe/Warsaw",
	},
	"EPWC": {
		ICAO:      "EPWC",
		Name:      "Swiebodzice Airfield",
		City:      "Swiebodzice",
		State:     "Lower-Silesia",
		Country:   "PL",
		Elevation: 876,
		Latitude:  50.8829994202,
		Longitude: 16.3199996948,
		Timezone:  "Europe/Warsaw",
	},
	"EPWK": {
		ICAO:      "EPWK",
		Name:      "Wloclawek-Kruszyn Airport",
		City:      "Wloclawek",
		State:     "Kujawsko-Pomorskie",
		Country:   "PL",
		Elevation: 220,
		Latitude:  52.5847015381,
		Longitude: 19.0156002045,
		Timezone:  "Europe/Warsaw",
	},
	"EPWR": {
		ICAO:      "EPWR",
		IATA:      "WRO",
		Name:      "Copernicus Wroclaw Airport",
		City:      "Wroclaw",
		State:     "Lower-Silesia",
		Country:   "PL",
		Elevation: 404,
		Latitude:  51.1026992798,
		Longitude: 16.885799408,
		Timezone:  "Europe/Warsaw",
	},
	"EPWS": {
		ICAO:      "EPWS",
		Name:      "Wroclaw-Szymanow Airport",
		City:      "Wroclaw",
		State:     "Lower-Silesia",
		Country:   "PL",
		Elevation: 390,
		Latitude:  51.2061004639,
		Longitude: 16.9986000061,
		Timezone:  "Europe/Warsaw",
	},
	"EPWT": {
		ICAO:      "EPWT",
		Name:      "Watorowo Airport",
		State:     "Kuyavian-Pomerania",
		Country:   "PL",
		Elevation: 299,
		Latitude:  53.298567500000004,
		Longitude: 18.414187352797445,
		Timezone:  "Europe/Warsaw",
	},
	"EPZA": {
		ICAO:      "EPZA",
		Name:      "Zamosc-Mokre Airport",
		City:      "Zamosc",
		State:     "Lublin",
		Country:   "PL",
		Elevation: 751,
		Latitude:  50.7016983032,
		Longitude: 23.2043991089,
		Timezone:  "Europe/Warsaw",
	},
	"EPZG": {
		ICAO:      "EPZG",
		IATA:      "IEG",
		Name:      "Zielona Gora-Babimost Airport",
		City:      "Babimost",
		State:     "Lubusz",
		Country:   "PL",
		Elevation: 194,
		Latitude:  52.1385002136,
		Longitude: 15.7986001968,
		Timezone:  "Europe/Warsaw",
	},
	"EPZN": {
		ICAO:      "EPZN",
		Name:      "Tomaszowo Airport",
		City:      "Zagan",
		State:     "Lubusz",
		Country:   "PL",
		Elevation: 459,
		Latitude:  51.6274986267,
		Longitude: 15.4083003998,
		Timezone:  "Europe/Warsaw",
	},
	"EPZP": {
		ICAO:      "EPZP",
		Name:      "Zielona Gora-Przylep Airport",
		City:      "Zielona Gora",
		State:     "Lubusz",
		Country:   "PL",
		Elevation: 249,
		Latitude:  51.9789009094,
		Longitude: 15.4638996124,
		Timezone:  "Europe/Warsaw",
	},
	"EPZR": {
		ICAO:      "EPZR",
		Name:      "Zar Airport",
		City:      "Miedzybrodzie Zywieckie",
		State:     "Silesia",
		Country:   "PL",
		Elevation: 1260,
		Latitude:  49.7710990906,
		Longitude: 19.2180995941,
		Timezone:  "Europe/Warsaw",
	},
	"ESCF": {
		ICAO:      "ESCF",
		Name:      "Malmen Air Base",
		City:      "Linkoping",
		State:     "Oestergotland",
		Country:   "SE",
		Elevation: 308,
		Latitude:  58.4023017883,
		Longitude: 15.5256996155,
		Timezone:  "Europe/Stockholm",
	},
	"ESCM": {
		ICAO:      "ESCM",
		Name:      "Uppsala Airport",
		State:     "Uppsala",
		Country:   "SE",
		Elevation: 68,
		Latitude:  59.8973007202,
		Longitude: 17.5886001587,
		Timezone:  "Europe/Stockholm",
	},
	"ESDF": {
		ICAO:      "ESDF",
		IATA:      "RNB",
		Name:      "Ronneby Airport",
		State:     "Blekinge",
		Country:   "SE",
		Elevation: 191,
		Latitude:  56.2667007446,
		Longitude: 15.2650003433,
		Timezone:  "Europe/Stockholm",
	},
	"ESFA": {
		ICAO:      "ESFA",
		IATA:      "XWP",
		Name:      "Hassleholm Bokeberg Airport",
		City:      "Hassleholm",
		State:     "Skane",
		Country:   "SE",
		Elevation: 105,
		Latitude:  56.1344069918,
		Longitude: 13.8756465912,
		Timezone:  "Europe/Stockholm",
	},
	"ESFH": {
		ICAO:      "ESFH",
		Name:      "Hasslosa Air Base",
		City:      "Hasslosa",
		State:     "Vaestra-Gotaland",
		Country:   "SE",
		Latitude:  58.4096984863,
		Longitude: 13.2630996704,
		Timezone:  "Europe/Stockholm",
	},
	"ESFM": {
		ICAO:      "ESFM",
		Name:      "Moholm Air Base",
		City:      "Moholm",
		State:     "Vaestra-Gotaland",
		Country:   "SE",
		Latitude:  58.5974998474,
		Longitude: 14.1108999252,
		Timezone:  "Europe/Stockholm",
	},
	"ESFQ": {
		ICAO:      "ESFQ",
		Name:      "Kosta Air Base",
		City:      "Kosta",
		State:     "Kronoberg",
		Country:   "SE",
		Latitude:  56.8445014954,
		Longitude: 15.4527997971,
		Timezone:  "Europe/Stockholm",
	},
	"ESFR": {
		ICAO:      "ESFR",
		Name:      "Rada Air Base",
		City:      "Rada",
		State:     "Vaestra-Gotaland",
		Country:   "SE",
		Elevation: 230,
		Latitude:  58.4981002808,
		Longitude: 13.0531997681,
		Timezone:  "Europe/Stockholm",
	},
	"ESFS": {
		ICAO:      "ESFS",
		Name:      "Sandvik Airport",
		City:      "Sandvik",
		State:     "Kalmar",
		Country:   "SE",
		Latitude:  57.0680999756,
		Longitude: 16.8612995148,
		Timezone:  "Europe/Stockholm",
	},
	"ESGA": {
		ICAO:      "ESGA",
		Name:      "Backamo Airport",
		City:      "Uddevalla",
		State:     "Vaestra-Gotaland",
		Country:   "SE",
		Latitude:  58.1772003174,
		Longitude: 11.9735002518,
		Timezone:  "Europe/Stockholm",
	},
	"ESGC": {
		ICAO:      "ESGC",
		Name:      "Alleberg Airport",
		City:      "Alleberg",
		State:     "Vaestra-Gotaland",
		Country:   "SE",
		Latitude:  58.1344985962,
		Longitude: 13.6026000977,
		Timezone:  "Europe/Stockholm",
	},
	"ESGD": {
		ICAO:      "ESGD",
		Name:      "Bammelshed Airport",
		City:      "Tidaholm",
		State:     "Vaestra-Gotaland",
		Country:   "SE",
		Latitude:  58.1917991638,
		Longitude: 13.9956998825,
		Timezone:  "Europe/Stockholm",
	},
	"ESGE": {
		ICAO:      "ESGE",
		Name:      "Viared Airport",
		City:      "Boras",
		State:     "Vaestra-Gotaland",
		Country:   "SE",
		Latitude:  57.6955986023,
		Longitude: 12.8428001404,
		Timezone:  "Europe/Stockholm",
	},
	"ESGF": {
		ICAO:      "ESGF",
		Name:      "Morup Airport",
		City:      "Falkenberg",
		State:     "Halland",
		Country:   "SE",
		Latitude:  56.9712982178,
		Longitude: 12.3891000748,
		Timezone:  "Europe/Stockholm",
	},
	"ESGG": {
		ICAO:      "ESGG",
		IATA:      "GOT",
		Name:      "Gothenburg-Landvetter Airport",
		City:      "Gothenburg",
		State:     "Vaestra-Gotaland",
		Country:   "SE",
		Elevation: 506,
		Latitude:  57.6627998352,
		Longitude: 12.279800415,
		Timezone:  "Europe/Stockholm",
	},
	"ESGH": {
		ICAO:      "ESGH",
		Name:      "Herrljunga Airport",
		City:      "Herrljunga",
		State:     "Vaestra-Gotaland",
		Country:   "SE",
		Latitude:  58.0293006897,
		Longitude: 13.1082000732,
		Timezone:  "Europe/Stockholm",
	},
	"ESGI": {
		ICAO:      "ESGI",
		Name:      "Alingsas Airport",
		City:      "Alingsas",
		State:     "Vaestra-Gotaland",
		Country:   "SE",
		Latitude:  57.949798584,
		Longitude: 12.5780000687,
		Timezone:  "Europe/Stockholm",
	},
	"ESGJ": {
		ICAO:      "ESGJ",
		IATA:      "JKG",
		Name:      "Jonkoping Airport",
		City:      "Jonkoping",
		State:     "Jonkoping",
		Country:   "SE",
		Elevation: 741,
		Latitude:  57.757598877,
		Longitude: 14.0686998367,
		Timezone:  "Europe/Stockholm",
	},
	"ESGK": {
		ICAO:      "ESGK",
		Name:      "Falkoping Airport",
		City:      "Falkoping",
		State:     "Vaestra-Gotaland",
		Country:   "SE",
		Elevation: 785,
		Latitude:  58.1697998047,
		Longitude: 13.5878000259,
		Timezone:  "Europe/Stockholm",
	},
	"ESGL": {
		ICAO:      "ESGL",
		IATA:      "LDK",
		Name:      "Lidkoping-Hovby Airport",
		City:      "Lidkoping",
		State:     "Vaestra-Gotaland",
		Country:   "SE",
		Elevation: 200,
		Latitude:  58.4654998779,
		Longitude: 13.1744003296,
		Timezone:  "Europe/Stockholm",
	},
	"ESGM": {
		ICAO:      "ESGM",
		Name:      "Oresten Airport",
		City:      "Oresten",
		State:     "Vaestra-Gotaland",
		Country:   "SE",
		Latitude:  57.4453010559,
		Longitude: 12.6490001678,
		Timezone:  "Europe/Stockholm",
	},
	"ESGN": {
		ICAO:      "ESGN",
		Name:      "Brannebrona Airport",
		City:      "Gotene",
		State:     "Vaestra-Gotaland",
		Country:   "SE",
		Elevation: 259,
		Latitude:  58.5786018372,
		Longitude: 13.6106004715,
		Timezone:  "Europe/Stockholm",
	},
	"ESGO": {
		ICAO:      "ESGO",
		Name:      "Vargarda Airport",
		City:      "Vargarda",
		State:     "Vaestra-Gotaland",
		Country:   "SE",
		Elevation: 328,
		Latitude:  58.0393981934,
		Longitude: 12.7889003754,
		Timezone:  "Europe/Stockholm",
	},
	"ESGP": {
		ICAO:      "ESGP",
		IATA:      "GSE",
		Name:      "Gothenburg City Airport",
		City:      "Gothenburg",
		State:     "Vaestra-Gotaland",
		Country:   "SE",
		Elevation: 59,
		Latitude:  57.7747001648,
		Longitude: 11.8704004288,
		Timezone:  "Europe/Stockholm",
	},
	"ESGR": {
		ICAO:      "ESGR",
		IATA:      "KVB",
		Name:      "Skovde Airport",
		City:      "Skovde",
		State:     "Vaestra-Gotaland",
		Country:   "SE",
		Elevation: 324,
		Latitude:  58.456401825,
		Longitude: 13.972700119,
		Timezone:  "Europe/Stockholm",
	},
	"ESGS": {
		ICAO:      "ESGS",
		Name:      "Nasinge Airport",
		City:      "Stromstad",
		State:     "Vaestra-Gotaland",
		Country:   "SE",
		Elevation: 115,
		Latitude:  59.0167999268,
		Longitude: 11.3437004089,
		Timezone:  "Europe/Stockholm",
	},
	"ESGT": {
		ICAO:      "ESGT",
		IATA:      "THN",
		Name:      "Trollhattan-Vanersborg Airport",
		City:      "Trollhattan",
		State:     "Vaestra-Gotaland",
		Country:   "SE",
		Elevation: 137,
		Latitude:  58.3180999756,
		Longitude: 12.345000267,
		Timezone:  "Europe/Stockholm",
	},
	"ESGU": {
		ICAO:      "ESGU",
		Name:      "Rorkarr Airport",
		City:      "Uddevalla",
		State:     "Vaestra-Gotaland",
		Country:   "SE",
		Latitude:  58.3675994873,
		Longitude: 11.7754001617,
		Timezone:  "Europe/Stockholm",
	},
	"ESGV": {
		ICAO:      "ESGV",
		Name:      "Varberg Getteron airfield",
		City:      "Varberg",
		State:     "Halland",
		Country:   "SE",
		Elevation: 5,
		Latitude:  57.1245994568,
		Longitude: 12.2283000946,
		Timezone:  "Europe/Stockholm",
	},
	"ESGY": {
		ICAO:      "ESGY",
		Name:      "Saffle Airport",
		City:      "Saffle",
		State:     "Vaermland",
		Country:   "SE",
		Latitude:  59.0912017822,
		Longitude: 12.9586000443,
		Timezone:  "Europe/Stockholm",
	},
	"ESIA": {
		ICAO:      "ESIA",
		Name:      "Karlsborg Air Base",
		City:      "Karlsborg",
		State:     "Vaestra-Gotaland",
		Country:   "SE",
		Elevation: 308,
		Latitude:  58.5138015747,
		Longitude: 14.5071001053,
		Timezone:  "Europe/Stockholm",
	},
	"ESIB": {
		ICAO:      "ESIB",
		Name:      "Satenas Air Base",
		City:      "Satenas",
		State:     "Vaestra-Gotaland",
		Country:   "SE",
		Elevation: 181,
		Latitude:  58.426399231,
		Longitude: 12.7144002914,
		Timezone:  "Europe/Stockholm",
	},
	"ESKA": {
		ICAO:      "ESKA",
		Name:      "Gimo Air Base",
		City:      "Gimo",
		State:     "Uppsala",
		Country:   "SE",
		Latitude:  60.1328010559,
		Longitude: 18.1049003601,
		Timezone:  "Europe/Stockholm",
	},
	"ESKC": {
		ICAO:      "ESKC",
		Name:      "Sundbro Airport",
		State:     "Uppsala",
		Country:   "SE",
		Latitude:  59.9226989746,
		Longitude: 17.5368003845,
		Timezone:  "Europe/Stockholm",
	},
	"ESKD": {
		ICAO:      "ESKD",
		Name:      "Dala Jarna Airport",
		City:      "Dala Jarna",
		State:     "Dalarna",
		Country:   "SE",
		Latitude:  60.556098938,
		Longitude: 14.3770999908,
		Timezone:  "Europe/Stockholm",
	},
	"ESKG": {
		ICAO:      "ESKG",
		Name:      "Gryttjom Airport",
		City:      "Gryttjom",
		State:     "Uppsala",
		Country:   "SE",
		Latitude:  60.2869987488,
		Longitude: 17.4216995239,
		Timezone:  "Europe/Stockholm",
	},
	"ESKH": {
		ICAO:      "ESKH",
		Name:      "Eksharad Airport",
		City:      "Eksharad",
		State:     "Vaermland",
		Country:   "SE",
		Latitude:  60.154800415,
		Longitude: 13.5285997391,
		Timezone:  "Europe/Stockholm",
	},
	"ESKK": {
		ICAO:      "ESKK",
		IATA:      "KSK",
		Name:      "Karlskoga Airport",
		State:     "Oerebro",
		Country:   "SE",
		Elevation: 400,
		Latitude:  59.3459014893,
		Longitude: 14.4959001541,
		Timezone:  "Europe/Stockholm",
	},
	"ESKM": {
		ICAO:      "ESKM",
		IATA:      "MXX",
		Name:      "Mora Airport",
		State:     "Dalarna",
		Country:   "SE",
		Elevation: 634,
		Latitude:  60.957901001,
		Longitude: 14.5114002228,
		Timezone:  "Europe/Stockholm",
	},
	"ESKN": {
		ICAO:      "ESKN",
		IATA:      "NYO",
		Name:      "Stockholm Skavsta Airport",
		City:      "Stockholm / Nykoping",
		State:     "Sodermanland",
		Country:   "SE",
		Elevation: 140,
		Latitude:  58.7886009216,
		Longitude: 16.9122009277,
		Timezone:  "Europe/Stockholm",
	},
	"ESKO": {
		ICAO:      "ESKO",
		Name:      "Munkfors Airport",
		City:      "Munkfors",
		State:     "Vaermland",
		Country:   "SE",
		Latitude:  59.7988014221,
		Longitude: 13.4906997681,
		Timezone:  "Europe/Stockholm",
	},
	"ESKT": {
		ICAO:      "ESKT",
		Name:      "Tierp Air Base",
		City:      "Tierp",
		State:     "Uppsala",
		Country:   "SE",
		Latitude:  60.3450012207,
		Longitude: 17.4218997955,
		Timezone:  "Europe/Stockholm",
	},
	"ESKU": {
		ICAO:      "ESKU",
		Name:      "Sunne Airport",
		State:     "Vaermland",
		Country:   "SE",
		Latitude:  59.8601989746,
		Longitude: 13.1128997803,
		Timezone:  "Europe/Stockholm",
	},
	"ESKV": {
		ICAO:      "ESKV",
		Name:      "Arvika Airport",
		State:     "Vaermland",
		Country:   "SE",
		Elevation: 237,
		Latitude:  59.6758995056,
		Longitude: 12.6393995285,
		Timezone:  "Europe/Stockholm",
	},
	"ESKX": {
		ICAO:      "ESKX",
		Name:      "Bjorkvik Air Base",
		City:      "Bjorkvik",
		State:     "Sodermanland",
		Country:   "SE",
		Latitude:  58.7907981873,
		Longitude: 16.5711994171,
		Timezone:  "Europe/Stockholm",
	},
	"ESMA": {
		ICAO:      "ESMA",
		Name:      "Emmaboda Airport",
		City:      "Emmaboda",
		State:     "Kalmar",
		Country:   "SE",
		Elevation: 442,
		Latitude:  56.6108016968,
		Longitude: 15.6048002243,
		Timezone:  "Europe/Stockholm",
	},
	"ESMB": {
		ICAO:      "ESMB",
		Name:      "Borglanda Airport",
		City:      "Borglanda",
		State:     "Kalmar",
		Country:   "SE",
		Elevation: 118,
		Latitude:  56.8629989624,
		Longitude: 16.6560993195,
		Timezone:  "Europe/Stockholm",
	},
	"ESMC": {
		ICAO:      "ESMC",
		Name:      "Ranneslatt Airport",
		City:      "Eksjo",
		State:     "Jonkoping",
		Country:   "SE",
		Elevation: 720,
		Latitude:  57.6706008911,
		Longitude: 14.9429998398,
		Timezone:  "Europe/Stockholm",
	},
	"ESME": {
		ICAO:      "ESME",
		Name:      "Eslov Airport",
		City:      "Eslov",
		State:     "Skane",
		Country:   "SE",
		Elevation: 296,
		Latitude:  55.8483009338,
		Longitude: 13.3283004761,
		Timezone:  "Europe/Stockholm",
	},
	"ESMF": {
		ICAO:      "ESMF",
		Name:      "Fagerhult Airport",
		City:      "Fagerhult",
		State:     "Skane",
		Country:   "SE",
		Elevation: 378,
		Latitude:  56.3879013062,
		Longitude: 13.4706001282,
		Timezone:  "Europe/Stockholm",
	},
	"ESMG": {
		ICAO:      "ESMG",
		Name:      "Feringe Airport",
		City:      "Ljungby",
		State:     "Kronoberg",
		Country:   "SE",
		Elevation: 538,
		Latitude:  56.9502983093,
		Longitude: 13.9216995239,
		Timezone:  "Europe/Stockholm",
	},
	"ESMH": {
		ICAO:      "ESMH",
		Name:      "Hoganas Airport",
		City:      "Hoganas",
		State:     "Skane",
		Country:   "SE",
		Elevation: 21,
		Latitude:  56.1847991943,
		Longitude: 12.5761003494,
		Timezone:  "Europe/Stockholm",
	},
	"ESMI": {
		ICAO:      "ESMI",
		Name:      "Sjobo/Sovde Airport",
		City:      "Sovdeborg",
		State:     "Skane",
		Country:   "SE",
		Elevation: 118,
		Latitude:  55.5982017517,
		Longitude: 13.6746997833,
		Timezone:  "Europe/Stockholm",
	},
	"ESMJ": {
		ICAO:      "ESMJ",
		Name:      "Kagerod Airport",
		City:      "Kagerod",
		State:     "Skane",
		Country:   "SE",
		Elevation: 276,
		Latitude:  55.9947013855,
		Longitude: 13.0509004593,
		Timezone:  "Europe/Stockholm",
	},
	"ESMK": {
		ICAO:      "ESMK",
		IATA:      "KID",
		Name:      "Kristianstad Airport",
		City:      "Kristianstad",
		State:     "Skane",
		Country:   "SE",
		Elevation: 76,
		Latitude:  55.9216995239,
		Longitude: 14.0854997635,
		Timezone:  "Europe/Stockholm",
	},
	"ESML": {
		ICAO:      "ESML",
		Name:      "Landskrona Airport",
		City:      "Landskrona",
		State:     "Skane",
		Country:   "SE",
		Elevation: 194,
		Latitude:  55.9459991455,
		Longitude: 12.8699998856,
		Timezone:  "Europe/Stockholm",
	},
	"ESMO": {
		ICAO:      "ESMO",
		IATA:      "OSK",
		Name:      "Oskarshamn Airport",
		State:     "Kalmar",
		Country:   "SE",
		Elevation: 96,
		Latitude:  57.3504981995,
		Longitude: 16.4979991913,
		Timezone:  "Europe/Stockholm",
	},
	"ESMP": {
		ICAO:      "ESMP",
		Name:      "Anderstorp Airport",
		City:      "Anderstorp",
		State:     "Jonkoping",
		Country:   "SE",
		Elevation: 507,
		Latitude:  57.2641983032,
		Longitude: 13.5993995667,
		Timezone:  "Europe/Stockholm",
	},
	"ESMQ": {
		ICAO:      "ESMQ",
		IATA:      "KLR",
		Name:      "Kalmar Airport",
		State:     "Kalmar",
		Country:   "SE",
		Elevation: 17,
		Latitude:  56.6855010986,
		Longitude: 16.2875995636,
		Timezone:  "Europe/Stockholm",
	},
	"ESMS": {
		ICAO:      "ESMS",
		IATA:      "MMX",
		Name:      "Malmo Sturup Airport",
		City:      "Malmo",
		State:     "Skane",
		Country:   "SE",
		Elevation: 236,
		Latitude:  55.536305364,
		Longitude: 13.3761978149,
		Timezone:  "Europe/Stockholm",
	},
	"ESMT": {
		ICAO:      "ESMT",
		IATA:      "HAD",
		Name:      "Halmstad Airport",
		City:      "Halmstad",
		State:     "Halland",
		Country:   "SE",
		Elevation: 101,
		Latitude:  56.6911010742,
		Longitude: 12.8201999664,
		Timezone:  "Europe/Stockholm",
	},
	"ESMU": {
		ICAO:      "ESMU",
		Name:      "Mockeln Airport",
		City:      "Almhult",
		State:     "Kronoberg",
		Country:   "SE",
		Elevation: 480,
		Latitude:  56.5707015991,
		Longitude: 14.1665000916,
		Timezone:  "Europe/Stockholm",
	},
	"ESMV": {
		ICAO:      "ESMV",
		Name:      "Hagshult Airport",
		State:     "Jonkoping",
		Country:   "SE",
		Elevation: 556,
		Latitude:  57.2921981812,
		Longitude: 14.1372003555,
		Timezone:  "Europe/Stockholm",
	},
	"ESMX": {
		ICAO:      "ESMX",
		IATA:      "VXO",
		Name:      "Vaxjo Kronoberg Airport",
		City:      "Vaxjo",
		State:     "Kronoberg",
		Country:   "SE",
		Elevation: 610,
		Latitude:  56.9291000366,
		Longitude: 14.7279996872,
		Timezone:  "Europe/Stockholm",
	},
	"ESMY": {
		ICAO:      "ESMY",
		Name:      "Smalandsstenar Smalanda Airport",
		City:      "Smalandsstenar",
		State:     "Jonkoping",
		Country:   "SE",
		Elevation: 540,
		Latitude:  57.168598175,
		Longitude: 13.440199852,
		Timezone:  "Europe/Stockholm",
	},
	"ESMZ": {
		ICAO:      "ESMZ",
		Name:      "Olanda Airport",
		City:      "Olanda",
		State:     "Kalmar",
		Country:   "SE",
		Elevation: 27,
		Latitude:  57.3282356084,
		Longitude: 17.0505237579,
		Timezone:  "Europe/Stockholm",
	},
	"ESNA": {
		ICAO:      "ESNA",
		Name:      "Hallviken Airport",
		State:     "Jaemtland",
		Country:   "SE",
		Elevation: 1119,
		Latitude:  63.7383003235,
		Longitude: 15.4582996368,
		Timezone:  "Europe/Stockholm",
	},
	"ESNB": {
		ICAO:      "ESNB",
		Name:      "Solleftea Airport",
		City:      "Solleftea",
		State:     "Vaesternorrland",
		Country:   "SE",
		Latitude:  63.1711997986,
		Longitude: 16.985200882,
		Timezone:  "Europe/Stockholm",
	},
	"ESNC": {
		ICAO:      "ESNC",
		Name:      "Hedlanda Airport",
		City:      "Hede",
		State:     "Jaemtland",
		Country:   "SE",
		Elevation: 1460,
		Latitude:  62.4089012146,
		Longitude: 13.7472000122,
		Timezone:  "Europe/Stockholm",
	},
	"ESND": {
		ICAO:      "ESND",
		IATA:      "EVG",
		Name:      "Sveg Airport",
		State:     "Jaemtland",
		Country:   "SE",
		Elevation: 1178,
		Latitude:  62.0477981567,
		Longitude: 14.4229001999,
		Timezone:  "Europe/Stockholm",
	},
	"ESNE": {
		ICAO:      "ESNE",
		Name:      "Overkalix Airport",
		City:      "Overkalix",
		State:     "Norrbotten",
		Country:   "SE",
		Latitude:  66.5287017822,
		Longitude: 22.3500003815,
		Timezone:  "Europe/Stockholm",
	},
	"ESNF": {
		ICAO:      "ESNF",
		Name:      "Farila Air Base",
		City:      "Farila",
		State:     "Gaevleborg",
		Country:   "SE",
		Latitude:  61.8979988098,
		Longitude: 15.7053003311,
		Timezone:  "Europe/Stockholm",
	},
	"ESNG": {
		ICAO:      "ESNG",
		IATA:      "GEV",
		Name:      "Gallivare Airport",
		City:      "Gallivare",
		State:     "Norrbotten",
		Country:   "SE",
		Elevation: 1027,
		Latitude:  67.1324005127,
		Longitude: 20.8145999908,
		Timezone:  "Europe/Stockholm",
	},
	"ESNI": {
		ICAO:      "ESNI",
		Name:      "Kubbe Air Base",
		City:      "Kubbe",
		State:     "Vaesternorrland",
		Country:   "SE",
		Latitude:  63.6324996948,
		Longitude: 17.936000824,
		Timezone:  "Europe/Stockholm",
	},
	"ESNJ": {
		ICAO:      "ESNJ",
		Name:      "Jokkmokk Airport",
		State:     "Norrbotten",
		Country:   "SE",
		Elevation: 904,
		Latitude:  66.4962005615,
		Longitude: 20.1471996307,
		Timezone:  "Europe/Stockholm",
	},
	"ESNK": {
		ICAO:      "ESNK",
		IATA:      "KRF",
		Name:      "Kramfors Solleftea Airport",
		City:      "Kramfors / Solleftea",
		State:     "Vaesternorrland",
		Country:   "SE",
		Elevation: 34,
		Latitude:  63.0485992432,
		Longitude: 17.7688999176,
		Timezone:  "Europe/Stockholm",
	},
	"ESNL": {
		ICAO:      "ESNL",
		IATA:      "LYC",
		Name:      "Lycksele Airport",
		State:     "Vaesterbotten",
		Country:   "SE",
		Elevation: 705,
		Latitude:  64.5483016968,
		Longitude: 18.7161998749,
		Timezone:  "Europe/Stockholm",
	},
	"ESNM": {
		ICAO:      "ESNM",
		Name:      "Optand Airport",
		State:     "Jaemtland",
		Country:   "SE",
		Elevation: 1236,
		Latitude:  63.1286010742,
		Longitude: 14.8028001785,
		Timezone:  "Europe/Stockholm",
	},
	"ESNN": {
		ICAO:      "ESNN",
		IATA:      "SDL",
		Name:      "Sundsvall-Harnosand Airport",
		City:      "Sundsvall/ Harnosand",
		State:     "Vaesternorrland",
		Country:   "SE",
		Elevation: 16,
		Latitude:  62.5280990601,
		Longitude: 17.4438991547,
		Timezone:  "Europe/Stockholm",
	},
	"ESNO": {
		ICAO:      "ESNO",
		IATA:      "OER",
		Name:      "Ornskoldsvik Airport",
		City:      "Ornskoldsvik",
		State:     "Vaesternorrland",
		Country:   "SE",
		Elevation: 354,
		Latitude:  63.4082984924,
		Longitude: 18.9899997711,
		Timezone:  "Europe/Stockholm",
	},
	"ESNP": {
		ICAO:      "ESNP",
		Name:      "Pitea Airport",
		City:      "Pitea",
		State:     "Norrbotten",
		Country:   "SE",
		Elevation: 43,
		Latitude:  65.3983001709,
		Longitude: 21.260799408,
		Timezone:  "Europe/Stockholm",
	},
	"ESNQ": {
		ICAO:      "ESNQ",
		IATA:      "KRN",
		Name:      "Kiruna Airport",
		State:     "Norrbotten",
		Country:   "SE",
		Elevation: 1508,
		Latitude:  67.8219985962,
		Longitude: 20.3367996216,
		Timezone:  "Europe/Stockholm",
	},
	"ESNR": {
		ICAO:      "ESNR",
		Name:      "Orsa Airport",
		State:     "Dalarna",
		Country:   "SE",
		Elevation: 683,
		Latitude:  61.1899986267,
		Longitude: 14.7125997543,
		Timezone:  "Europe/Stockholm",
	},
	"ESNS": {
		ICAO:      "ESNS",
		IATA:      "SFT",
		Name:      "Skelleftea Airport",
		City:      "Skelleftea",
		State:     "Vaesterbotten",
		Country:   "SE",
		Elevation: 157,
		Latitude:  64.6248016357,
		Longitude: 21.0769004822,
		Timezone:  "Europe/Stockholm",
	},
	"ESNT": {
		ICAO:      "ESNT",
		Name:      "Sattna Airport",
		State:     "Vaesternorrland",
		Country:   "SE",
		Elevation: 886,
		Latitude:  62.4813995361,
		Longitude: 17.0028991699,
		Timezone:  "Europe/Stockholm",
	},
	"ESNU": {
		ICAO:      "ESNU",
		IATA:      "UME",
		Name:      "Umea Airport",
		City:      "Umea",
		State:     "Vaesterbotten",
		Country:   "SE",
		Elevation: 24,
		Latitude:  63.7918014526,
		Longitude: 20.2828006744,
		Timezone:  "Europe/Stockholm",
	},
	"ESNV": {
		ICAO:      "ESNV",
		IATA:      "VHM",
		Name:      "Vilhelmina Airport",
		State:     "Vaesterbotten",
		Country:   "SE",
		Elevation: 1140,
		Latitude:  64.5791015625,
		Longitude: 16.8335990906,
		Timezone:  "Europe/Stockholm",
	},
	"ESNX": {
		ICAO:      "ESNX",
		IATA:      "AJR",
		Name:      "Arvidsjaur Airport",
		City:      "Arvidsjaur",
		State:     "Norrbotten",
		Country:   "SE",
		Elevation: 1245,
		Latitude:  65.5903015137,
		Longitude: 19.2819004059,
		Timezone:  "Europe/Stockholm",
	},
	"ESNY": {
		ICAO:      "ESNY",
		IATA:      "SOO",
		Name:      "Soderhamn Airport",
		City:      "Soderhamn",
		State:     "Gaevleborg",
		Country:   "SE",
		Elevation: 88,
		Latitude:  61.2615013123,
		Longitude: 17.0991001129,
		Timezone:  "Europe/Stockholm",
	},
	"ESNZ": {
		ICAO:      "ESNZ",
		IATA:      "OSD",
		Name:      "Ostersund Airport",
		City:      "Ostersund",
		State:     "Jaemtland",
		Country:   "SE",
		Elevation: 1233,
		Latitude:  63.1944007874,
		Longitude: 14.5003004074,
		Timezone:  "Europe/Stockholm",
	},
	"ESOE": {
		ICAO:      "ESOE",
		IATA:      "ORB",
		Name:      "Orebro Airport",
		City:      "Orebro",
		State:     "Oerebro",
		Country:   "SE",
		Elevation: 188,
		Latitude:  59.2237014771,
		Longitude: 15.0380001068,
		Timezone:  "Europe/Stockholm",
	},
	"ESOH": {
		ICAO:      "ESOH",
		IATA:      "HFS",
		Name:      "Hagfors Airport",
		State:     "Vaermland",
		Country:   "SE",
		Elevation: 474,
		Latitude:  60.0200996399,
		Longitude: 13.5789003372,
		Timezone:  "Europe/Stockholm",
	},
	"ESOK": {
		ICAO:      "ESOK",
		IATA:      "KSD",
		Name:      "Karlstad Airport",
		City:      "Karlstad",
		State:     "Vaermland",
		Country:   "SE",
		Elevation: 352,
		Latitude:  59.4446983337,
		Longitude: 13.3374004364,
		Timezone:  "Europe/Stockholm",
	},
	"ESOL": {
		ICAO:      "ESOL",
		Name:      "Lemstanas Airport",
		City:      "Storvik",
		State:     "Gaevleborg",
		Country:   "SE",
		Latitude:  60.5878982544,
		Longitude: 16.5867996216,
		Timezone:  "Europe/Stockholm",
	},
	"ESOW": {
		ICAO:      "ESOW",
		IATA:      "VST",
		Name:      "Stockholm Vasteras Airport",
		City:      "Stockholm / Vasteras",
		State:     "Vaestmanland",
		Country:   "SE",
		Elevation: 21,
		Latitude:  59.5894012451,
		Longitude: 16.633600235,
		Timezone:  "Europe/Stockholm",
	},
	"ESPA": {
		ICAO:      "ESPA",
		IATA:      "LLA",
		Name:      "Lulea Airport",
		City:      "Lulea",
		State:     "Norrbotten",
		Country:   "SE",
		Elevation: 65,
		Latitude:  65.543800354,
		Longitude: 22.1219997406,
		Timezone:  "Europe/Stockholm",
	},
	"ESPE": {
		ICAO:      "ESPE",
		Name:      "Vidsel Air Base",
		City:      "Vidsel",
		State:     "Norrbotten",
		Country:   "SE",
		Elevation: 597,
		Latitude:  65.8752975464,
		Longitude: 20.1499004364,
		Timezone:  "Europe/Stockholm",
	},
	"ESPG": {
		ICAO:      "ESPG",
		Name:      "Boden Army Air Base",
		City:      "Boden",
		State:     "Norrbotten",
		Country:   "SE",
		Latitude:  65.8103027344,
		Longitude: 21.6912994385,
		Timezone:  "Europe/Stockholm",
	},
	"ESPJ": {
		ICAO:      "ESPJ",
		Name:      "Hede Air Base",
		State:     "Norrbotten",
		Country:   "SE",
		Latitude:  65.8383026123,
		Longitude: 21.4687995911,
		Timezone:  "Europe/Stockholm",
	},
	"ESQO": {
		ICAO:      "ESQO",
		Name:      "Arboga Airport",
		State:     "Vaestmanland",
		Country:   "SE",
		Elevation: 33,
		Latitude:  59.3866004944,
		Longitude: 15.9240999222,
		Timezone:  "Europe/Stockholm",
	},
	"ESSA": {
		ICAO:      "ESSA",
		IATA:      "ARN",
		Name:      "Stockholm-Arlanda Airport",
		City:      "Stockholm",
		State:     "Stockholm",
		Country:   "SE",
		Elevation: 137,
		Latitude:  59.6519012451,
		Longitude: 17.9186000824,
		Timezone:  "Europe/Stockholm",
	},
	"ESSB": {
		ICAO:      "ESSB",
		IATA:      "BMA",
		Name:      "Stockholm-Bromma Airport",
		City:      "Stockholm",
		State:     "Stockholm",
		Country:   "SE",
		Elevation: 47,
		Latitude:  59.3544006348,
		Longitude: 17.9416999817,
		Timezone:  "Europe/Stockholm",
	},
	"ESSC": {
		ICAO:      "ESSC",
		Name:      "Ekeby Airport",
		City:      "Eskilstuna",
		State:     "Sodermanland",
		Country:   "SE",
		Latitude:  59.3838996887,
		Longitude: 16.4419002533,
		Timezone:  "Europe/Stockholm",
	},
	"ESSD": {
		ICAO:      "ESSD",
		IATA:      "BLE",
		Name:      "Borlange Airport",
		State:     "Dalarna",
		Country:   "SE",
		Elevation: 503,
		Latitude:  60.422000885,
		Longitude: 15.5151996613,
		Timezone:  "Europe/Stockholm",
	},
	"ESSE": {
		ICAO:      "ESSE",
		Name:      "Ska-Edeby Airport",
		City:      "Stockholm",
		State:     "Stockholm",
		Country:   "SE",
		Latitude:  59.3451004028,
		Longitude: 17.7404994965,
		Timezone:  "Europe/Stockholm",
	},
	"ESSF": {
		ICAO:      "ESSF",
		IATA:      "HLF",
		Name:      "Hultsfred Airport",
		State:     "Kalmar",
		Country:   "SE",
		Elevation: 366,
		Latitude:  57.5257987976,
		Longitude: 15.8233003616,
		Timezone:  "Europe/Stockholm",
	},
	"ESSG": {
		ICAO:      "ESSG",
		Name:      "Ludvika Airport",
		City:      "Ludvika",
		State:     "Dalarna",
		Country:   "SE",
		Latitude:  60.0882987976,
		Longitude: 15.0964002609,
		Timezone:  "Europe/Stockholm",
	},
	"ESSH": {
		ICAO:      "ESSH",
		Name:      "Laxa Airport",
		City:      "Laxa",
		State:     "Oerebro",
		Country:   "SE",
		Latitude:  58.9786987305,
		Longitude: 14.6661996841,
		Timezone:  "Europe/Stockholm",
	},
	"ESSI": {
		ICAO:      "ESSI",
		Name:      "Visingso Airport",
		City:      "Visingso",
		State:     "Jonkoping",
		Country:   "SE",
		Latitude:  58.098400116,
		Longitude: 14.4026002884,
		Timezone:  "Europe/Stockholm",
	},
	"ESSK": {
		ICAO:      "ESSK",
		IATA:      "GVX",
		Name:      "Gavle Sandviken Airport",
		City:      "Gavle / Sandviken",
		State:     "Gaevleborg",
		Country:   "SE",
		Elevation: 224,
		Latitude:  60.5932998657,
		Longitude: 16.9514007568,
		Timezone:  "Europe/Stockholm",
	},
	"ESSL": {
		ICAO:      "ESSL",
		IATA:      "LPI",
		Name:      "Linkoping SAAB Airport",
		City:      "Linkoping",
		State:     "Oestergotland",
		Country:   "SE",
		Elevation: 172,
		Latitude:  58.4062004089,
		Longitude: 15.6805000305,
		Timezone:  "Europe/Stockholm",
	},
	"ESSM": {
		ICAO:      "ESSM",
		Name:      "Brattforshede Airport",
		City:      "Lindfors",
		State:     "Vaermland",
		Country:   "SE",
		Latitude:  59.6083984375,
		Longitude: 13.9123001099,
		Timezone:  "Europe/Stockholm",
	},
	"ESSN": {
		ICAO:      "ESSN",
		Name:      "Norrtalje Airport",
		City:      "Norrtalje",
		State:     "Stockholm",
		Country:   "SE",
		Elevation: 39,
		Latitude:  59.73279953,
		Longitude: 18.6963996887,
		Timezone:  "Europe/Stockholm",
	},
	"ESSP": {
		ICAO:      "ESSP",
		IATA:      "NRK",
		Name:      "Norrkoping Airport",
		City:      "Norrkoping",
		State:     "Oestergotland",
		Country:   "SE",
		Elevation: 32,
		Latitude:  58.5862998962,
		Longitude: 16.2506008148,
		Timezone:  "Europe/Stockholm",
	},
	"ESST": {
		ICAO:      "ESST",
		IATA:      "TYF",
		Name:      "Torsby Airport",
		State:     "Vaermland",
		Country:   "SE",
		Elevation: 393,
		Latitude:  60.1576004028,
		Longitude: 12.9912996292,
		Timezone:  "Europe/Stockholm",
	},
	"ESSU": {
		ICAO:      "ESSU",
		IATA:      "EKT",
		Name:      "Eskilstuna Airport",
		City:      "Eskilstuna",
		State:     "Sodermanland",
		Country:   "SE",
		Elevation: 139,
		Latitude:  59.3511009216,
		Longitude: 16.7084007263,
		Timezone:  "Europe/Stockholm",
	},
	"ESSV": {
		ICAO:      "ESSV",
		IATA:      "VBY",
		Name:      "Visby Airport",
		City:      "Visby",
		State:     "Gotland",
		Country:   "SE",
		Elevation: 164,
		Latitude:  57.6627998352,
		Longitude: 18.346200943,
		Timezone:  "Europe/Stockholm",
	},
	"ESSW": {
		ICAO:      "ESSW",
		IATA:      "VVK",
		Name:      "Vastervik Airport",
		City:      "Vastervik",
		State:     "Kalmar",
		Country:   "SE",
		Latitude:  57.7799987793,
		Longitude: 16.5235996246,
		Timezone:  "Europe/Stockholm",
	},
	"ESSX": {
		ICAO:      "ESSX",
		Name:      "Johannisberg Airport",
		City:      "Vasteras",
		State:     "Vaestmanland",
		Country:   "SE",
		Latitude:  59.5758018494,
		Longitude: 16.503200531,
		Timezone:  "Europe/Stockholm",
	},
	"ESSZ": {
		ICAO:      "ESSZ",
		Name:      "Vangso Airport",
		City:      "Vangso",
		State:     "Sodermanland",
		Country:   "SE",
		Latitude:  59.1011009216,
		Longitude: 17.2110996246,
		Timezone:  "Europe/Stockholm",
	},
	"ESTA": {
		ICAO:      "ESTA",
		IATA:      "AGH",
		Name:      "Angelholm-Helsingborg Airport",
		City:      "Angelholm",
		State:     "Skane",
		Country:   "SE",
		Elevation: 68,
		Latitude:  56.2961006165,
		Longitude: 12.8471002579,
		Timezone:  "Europe/Stockholm",
	},
	"ESTF": {
		ICAO:      "ESTF",
		Name:      "Fjallbacka Anra Airport",
		City:      "Fjallbacka",
		State:     "Vaestra-Gotaland",
		Country:   "SE",
		Elevation: 10,
		Latitude:  58.6300086975,
		Longitude: 11.314458847,
		Timezone:  "Europe/Stockholm",
	},
	"ESTG": {
		ICAO:      "ESTG",
		Name:      "Gronhogen Airport",
		City:      "Gronhogen",
		State:     "Kalmar",
		Country:   "SE",
		Elevation: 10,
		Latitude:  56.2750015259,
		Longitude: 16.4200000763,
		Timezone:  "Europe/Stockholm",
	},
	"ESTL": {
		ICAO:      "ESTL",
		Name:      "Ljungbyhed Airport",
		City:      "Ljungbyhed",
		State:     "Skane",
		Country:   "SE",
		Elevation: 140,
		Latitude:  56.0828018188,
		Longitude: 13.2124996185,
		Timezone:  "Europe/Stockholm",
	},
	"ESTT": {
		ICAO:      "ESTT",
		Name:      "Vellinge Airfield",
		City:      "Vellinge",
		State:     "Skane",
		Country:   "SE",
		Elevation: 12,
		Latitude:  55.3912021148,
		Longitude: 13.0221033096,
		Timezone:  "Europe/Stockholm",
	},
	"ESUA": {
		ICAO:      "ESUA",
		Name:      "Amsele Air Base",
		City:      "Amsele",
		State:     "Vaesterbotten",
		Country:   "SE",
		Latitude:  64.570602417,
		Longitude: 19.3141002655,
		Timezone:  "Europe/Stockholm",
	},
	"ESUB": {
		ICAO:      "ESUB",
		Name:      "Arbra Airport",
		City:      "Arbra",
		State:     "Gaevleborg",
		Country:   "SE",
		Latitude:  61.5125007629,
		Longitude: 16.3724994659,
		Timezone:  "Europe/Stockholm",
	},
	"ESUD": {
		ICAO:      "ESUD",
		IATA:      "SQO",
		Name:      "Storuman Airport",
		State:     "Vaesterbotten",
		Country:   "SE",
		Elevation: 915,
		Latitude:  64.960899353,
		Longitude: 17.6965999603,
		Timezone:  "Europe/Stockholm",
	},
	"ESUE": {
		ICAO:      "ESUE",
		IATA:      "IDB",
		Name:      "Idre Airport",
		City:      "Idre",
		State:     "Dalarna",
		Country:   "SE",
		Latitude:  61.8697013855,
		Longitude: 12.6893997192,
		Timezone:  "Europe/Stockholm",
	},
	"ESUF": {
		ICAO:      "ESUF",
		Name:      "Fallfors Air Base",
		City:      "Fallfors",
		State:     "Vaesterbotten",
		Country:   "SE",
		Latitude:  65.1075973511,
		Longitude: 20.761100769,
		Timezone:  "Europe/Stockholm",
	},
	"ESUG": {
		ICAO:      "ESUG",
		Name:      "Gargnas Airport",
		City:      "Gargnas",
		State:     "Vaesterbotten",
		Country:   "SE",
		Latitude:  65.3052978516,
		Longitude: 17.9755001068,
		Timezone:  "Europe/Stockholm",
	},
	"ESUH": {
		ICAO:      "ESUH",
		Name:      "Myran Airport",
		City:      "Harnosand",
		State:     "Vaesternorrland",
		Country:   "SE",
		Latitude:  62.6324691772,
		Longitude: 17.9817008972,
		Timezone:  "Europe/Stockholm",
	},
	"ESUI": {
		ICAO:      "ESUI",
		Name:      "Mellansel Airport",
		City:      "Mellansel",
		State:     "Vaesternorrland",
		Country:   "SE",
		Elevation: 269,
		Latitude:  63.3983001709,
		Longitude: 18.3283004761,
		Timezone:  "Europe/Stockholm",
	},
	"ESUJ": {
		ICAO:      "ESUJ",
		Name:      "Talje Airport",
		City:      "Ange",
		State:     "Vaesternorrland",
		Country:   "SE",
		Latitude:  62.5652999878,
		Longitude: 15.8346996307,
		Timezone:  "Europe/Stockholm",
	},
	"ESUK": {
		ICAO:      "ESUK",
		Name:      "Kalixfors Airport",
		State:     "Norrbotten",
		Country:   "SE",
		Elevation: 1549,
		Latitude:  67.7648010254,
		Longitude: 20.2572002411,
		Timezone:  "Europe/Stockholm",
	},
	"ESUL": {
		ICAO:      "ESUL",
		Name:      "Ljusdal Airport",
		City:      "Ljusdal",
		State:     "Gaevleborg",
		Country:   "SE",
		Latitude:  61.8170013428,
		Longitude: 16.0042991638,
		Timezone:  "Europe/Stockholm",
	},
	"ESUM": {
		ICAO:      "ESUM",
		Name:      "Mohed Airport",
		City:      "Mohed",
		State:     "Gaevleborg",
		Country:   "SE",
		Latitude:  61.2911987305,
		Longitude: 16.8463001251,
		Timezone:  "Europe/Stockholm",
	},
	"ESUO": {
		ICAO:      "ESUO",
		Name:      "Graftavallen Airport",
		City:      "Graftavallen",
		State:     "Jaemtland",
		Country:   "SE",
		Elevation: 1640,
		Latitude:  63.0416984558,
		Longitude: 14.0013999939,
		Timezone:  "Europe/Stockholm",
	},
	"ESUP": {
		ICAO:      "ESUP",
		IATA:      "PJA",
		Name:      "Pajala Airport",
		State:     "Norrbotten",
		Country:   "SE",
		Elevation: 542,
		Latitude:  67.2455978394,
		Longitude: 23.0688991547,
		Timezone:  "Europe/Stockholm",
	},
	"ESUR": {
		ICAO:      "ESUR",
		Name:      "Ramsele Airport",
		City:      "Solleftea",
		State:     "Vaesternorrland",
		Country:   "SE",
		Latitude:  63.4902992249,
		Longitude: 16.4834003448,
		Timezone:  "Europe/Stockholm",
	},
	"ESUS": {
		ICAO:      "ESUS",
		Name:      "Asele Airport",
		City:      "Asele",
		State:     "Vaesterbotten",
		Country:   "SE",
		Latitude:  64.1591033936,
		Longitude: 17.2742996216,
		Timezone:  "Europe/Stockholm",
	},
	"ESUT": {
		ICAO:      "ESUT",
		IATA:      "HMV",
		Name:      "Hemavan Airport",
		State:     "Vaesterbotten",
		Country:   "SE",
		Elevation: 1503,
		Latitude:  65.806098938,
		Longitude: 15.0827999115,
		Timezone:  "Europe/Stockholm",
	},
	"ESUV": {
		ICAO:      "ESUV",
		Name:      "Alvsbyn Airport",
		City:      "Alvsbyn",
		State:     "Norrbotten",
		Country:   "SE",
		Latitude:  65.6456985474,
		Longitude: 21.0611000061,
		Timezone:  "Europe/Stockholm",
	},
	"ESUY": {
		ICAO:      "ESUY",
		Name:      "Edsby Airport",
		City:      "Edsbyn",
		State:     "Gaevleborg",
		Country:   "SE",
		Latitude:  61.3870010376,
		Longitude: 15.8333997726,
		Timezone:  "Europe/Stockholm",
	},
	"ESVA": {
		ICAO:      "ESVA",
		Name:      "Avesta Airport",
		City:      "Avesta",
		State:     "Dalarna",
		Country:   "SE",
		Latitude:  60.1804008484,
		Longitude: 16.122800827,
		Timezone:  "Europe/Stockholm",
	},
	"ESVB": {
		ICAO:      "ESVB",
		Name:      "Bunge Private Airport",
		City:      "Bunge",
		State:     "Gotland",
		Country:   "SE",
		Elevation: 69,
		Latitude:  57.8497009277,
		Longitude: 19.0349998474,
		Timezone:  "Europe/Stockholm",
	},
	"ESVG": {
		ICAO:      "ESVG",
		Name:      "Gagnef Airport",
		City:      "Djuras",
		State:     "Dalarna",
		Country:   "SE",
		Latitude:  60.551700592,
		Longitude: 15.0816001892,
		Timezone:  "Europe/Stockholm",
	},
	"ESVH": {
		ICAO:      "ESVH",
		Name:      "Hallefors Airport",
		City:      "Hallefors",
		Country:   "SE",
		Latitude:  59.867401123,
		Longitude: 14.4238996506,
		Timezone:  "Europe/Stockholm",
	},
	"ESVK": {
		ICAO:      "ESVK",
		Name:      "Katrineholm Airport",
		City:      "Katrineholm",
		State:     "Sodermanland",
		Country:   "SE",
		Latitude:  59.0223007202,
		Longitude: 16.2203006744,
		Timezone:  "Europe/Stockholm",
	},
	"ESVM": {
		ICAO:      "ESVM",
		Name:      "Skinnlanda Airport",
		City:      "Malung",
		State:     "Dalarna",
		Country:   "SE",
		Latitude:  60.6587982178,
		Longitude: 13.7265996933,
		Timezone:  "Europe/Stockholm",
	},
	"ESVQ": {
		ICAO:      "ESVQ",
		Name:      "Koping Airport",
		City:      "Koping",
		State:     "Vaestmanland",
		Country:   "SE",
		Latitude:  59.5275001526,
		Longitude: 15.9696998596,
		Timezone:  "Europe/Stockholm",
	},
	"ESVS": {
		ICAO:      "ESVS",
		Name:      "Siljansnas Airport",
		City:      "Siljansnas",
		State:     "Dalarna",
		Country:   "SE",
		Elevation: 611,
		Latitude:  60.7850990295,
		Longitude: 14.8271999359,
		Timezone:  "Europe/Stockholm",
	},
	"ETAD": {
		ICAO:      "ETAD",
		IATA:      "SPM",
		Name:      "Spangdahlem Air Base",
		City:      "Trier",
		State:     "Rheinland-Pfalz",
		Country:   "DE",
		Elevation: 1197,
		Latitude:  49.9726982117,
		Longitude: 6.6925001144,
		Timezone:  "Europe/Berlin",
	},
	"ETAR": {
		ICAO:      "ETAR",
		IATA:      "RMS",
		Name:      "Ramstein Air Base",
		City:      "Ramstein",
		State:     "Rheinland-Pfalz",
		Country:   "DE",
		Elevation: 776,
		Latitude:  49.4369010925,
		Longitude: 7.600279808,
		Timezone:  "Europe/Berlin",
	},
	"ETEJ": {
		ICAO:      "ETEJ",
		IATA:      "ZCD",
		Name:      "Bamberg-Breitenau Airport",
		City:      "Bamberg",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 837,
		Latitude:  49.9205551147,
		Longitude: 10.9141674042,
		Timezone:  "Europe/Berlin",
	},
	"ETEK": {
		ICAO:      "ETEK",
		Name:      "Baumholder Army Air Field",
		State:     "Rheinland-Pfalz",
		Country:   "DE",
		Latitude:  49.6502990723,
		Longitude: 7.300069809,
		Timezone:  "Europe/Berlin",
	},
	"ETEU": {
		ICAO:      "ETEU",
		IATA:      "GHF",
		Name:      "Giebelstadt Army Air Field",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 980,
		Latitude:  49.6481018066,
		Longitude: 9.9664897919,
		Timezone:  "Europe/Berlin",
	},
	"ETHA": {
		ICAO:      "ETHA",
		Name:      "Altenstadt Army Airfield",
		City:      "Altenstadt",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 2425,
		Latitude:  47.8354988098,
		Longitude: 10.8711996078,
		Timezone:  "Europe/Berlin",
	},
	"ETHB": {
		ICAO:      "ETHB",
		Name:      "Buckeburg Airport",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 230,
		Latitude:  52.2784996033,
		Longitude: 9.0821695328,
		Timezone:  "Europe/Berlin",
	},
	"ETHC": {
		ICAO:      "ETHC",
		IATA:      "ZCN",
		Name:      "Celle Airport",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 129,
		Latitude:  52.5912017822,
		Longitude: 10.0221004486,
		Timezone:  "Europe/Berlin",
	},
	"ETHE": {
		ICAO:      "ETHE",
		IATA:      "ZPQ",
		Name:      "Rheine Bentlage Airport",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 129,
		Latitude:  52.2911987305,
		Longitude: 7.3870000839,
		Timezone:  "Europe/Berlin",
	},
	"ETHF": {
		ICAO:      "ETHF",
		IATA:      "FRZ",
		Name:      "Fritzlar Airport",
		City:      "Fritzlar",
		State:     "Hesse",
		Country:   "DE",
		Elevation: 1345,
		Latitude:  51.1146,
		Longitude: 9.286,
		Timezone:  "Europe/Berlin",
	},
	"ETHL": {
		ICAO:      "ETHL",
		Name:      "Laupheim Airport",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 1766,
		Latitude:  48.2202987671,
		Longitude: 9.9100198746,
		Timezone:  "Europe/Berlin",
	},
	"ETHM": {
		ICAO:      "ETHM",
		Name:      "Mendig Airport",
		State:     "Rheinland-Pfalz",
		Country:   "DE",
		Elevation: 597,
		Latitude:  50.3660011292,
		Longitude: 7.3153300285,
		Timezone:  "Europe/Berlin",
	},
	"ETHN": {
		ICAO:      "ETHN",
		Name:      "Niederstetten Airport",
		City:      "Niederstetten",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 1339,
		Latitude:  49.3919448853,
		Longitude: 9.9588890076,
		Timezone:  "Europe/Berlin",
	},
	"ETHR": {
		ICAO:      "ETHR",
		Name:      "Roth Airport",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1268,
		Latitude:  49.2174987793,
		Longitude: 11.1001996994,
		Timezone:  "Europe/Berlin",
	},
	"ETHS": {
		ICAO:      "ETHS",
		Name:      "Fassberg Airport",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 245,
		Latitude:  52.9193992615,
		Longitude: 10.1975002289,
		Timezone:  "Europe/Berlin",
	},
	"ETHT": {
		ICAO:      "ETHT",
		Name:      "Cottbus (Cottbus Army) Airport",
		State:     "Brandenburg",
		Country:   "DE",
		Latitude:  51.7678985596,
		Longitude: 14.2919998169,
		Timezone:  "Europe/Berlin",
	},
	"ETIC": {
		ICAO:      "ETIC",
		Name:      "Grafenwohr Army Air Field",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1363,
		Latitude:  49.6986999512,
		Longitude: 11.940199852,
		Timezone:  "Europe/Berlin",
	},
	"ETID": {
		ICAO:      "ETID",
		IATA:      "ZNF",
		Name:      "Hanau Army Air Field",
		State:     "Hesse",
		Country:   "DE",
		Elevation: 368,
		Latitude:  50.1692008972,
		Longitude: 8.9615898132,
		Timezone:  "Europe/Berlin",
	},
	"ETIH": {
		ICAO:      "ETIH",
		Name:      "Hohenfels Army Air Field",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1455,
		Latitude:  49.2181015015,
		Longitude: 11.8360996246,
		Timezone:  "Europe/Berlin",
	},
	"ETIK": {
		ICAO:      "ETIK",
		IATA:      "ILH",
		Name:      "Illesheim Air Base",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1079,
		Latitude:  49.4738998413,
		Longitude: 10.3880996704,
		Timezone:  "Europe/Berlin",
	},
	"ETIN": {
		ICAO:      "ETIN",
		IATA:      "KZG",
		Name:      "Kitzingen Army Air Field",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 689,
		Latitude:  49.7430992126,
		Longitude: 10.2005996704,
		Timezone:  "Europe/Berlin",
	},
	"ETME": {
		ICAO:      "ETME",
		Name:      "Eggebeck Air Base",
		State:     "Schleswig-Holstein",
		Country:   "DE",
		Elevation: 65,
		Latitude:  54.6248016357,
		Longitude: 9.3459997177,
		Timezone:  "Europe/Berlin",
	},
	"ETMN": {
		ICAO:      "ETMN",
		IATA:      "NDZ",
		Name:      "Nordholz Airport",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 74,
		Latitude:  53.7677001953,
		Longitude: 8.6584997177,
		Timezone:  "Europe/Berlin",
	},
	"ETNA": {
		ICAO:      "ETNA",
		Name:      "Ahlhorn Air Force Base",
		City:      "Ahlhorn",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 162,
		Latitude:  52.8829994202,
		Longitude: 8.2329998016,
		Timezone:  "Europe/Berlin",
	},
	"ETND": {
		ICAO:      "ETND",
		Name:      "Diepholz Airport",
		City:      "Diepholz",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 128,
		Latitude:  52.5855560303,
		Longitude: 8.3422222137,
		Timezone:  "Europe/Berlin",
	},
	"ETNG": {
		ICAO:      "ETNG",
		IATA:      "GKE",
		Name:      "Geilenkirchen Airport",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 296,
		Latitude:  50.9608001709,
		Longitude: 6.0424199104,
		Timezone:  "Europe/Amsterdam",
	},
	"ETNH": {
		ICAO:      "ETNH",
		Name:      "Hohn Airport",
		State:     "Schleswig-Holstein",
		Country:   "DE",
		Elevation: 39,
		Latitude:  54.3121986389,
		Longitude: 9.5381698608,
		Timezone:  "Europe/Berlin",
	},
	"ETNJ": {
		ICAO:      "ETNJ",
		Name:      "Jever Airport",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 24,
		Latitude:  53.5335006714,
		Longitude: 7.8886699677,
		Timezone:  "Europe/Berlin",
	},
	"ETNL": {
		ICAO:      "ETNL",
		IATA:      "RLG",
		Name:      "Rostock-Laage Airport",
		City:      "Rostock",
		State:     "Mecklenburg-Vorpommern",
		Country:   "DE",
		Elevation: 138,
		Latitude:  53.9182014465,
		Longitude: 12.2783002853,
		Timezone:  "Europe/Berlin",
	},
	"ETNN": {
		ICAO:      "ETNN",
		IATA:      "QOE",
		Name:      "Norvenich Airport",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 386,
		Latitude:  50.831199646,
		Longitude: 6.6581702232,
		Timezone:  "Europe/Berlin",
	},
	"ETNS": {
		ICAO:      "ETNS",
		IATA:      "WBG",
		Name:      "Schleswig Airport",
		State:     "Schleswig-Holstein",
		Country:   "DE",
		Elevation: 70,
		Latitude:  54.4593009949,
		Longitude: 9.5163297653,
		Timezone:  "Europe/Berlin",
	},
	"ETNT": {
		ICAO:      "ETNT",
		Name:      "Wittmundhafen Airport",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 26,
		Latitude:  53.5477981567,
		Longitude: 7.6673297882,
		Timezone:  "Europe/Berlin",
	},
	"ETNU": {
		ICAO:      "ETNU",
		IATA:      "FNB",
		Name:      "Neubrandenburg Airport",
		State:     "Mecklenburg-Vorpommern",
		Country:   "DE",
		Elevation: 228,
		Latitude:  53.6021995544,
		Longitude: 13.3059997559,
		Timezone:  "Europe/Berlin",
	},
	"ETNW": {
		ICAO:      "ETNW",
		Name:      "Wunstorf Airport",
		State:     "Lower-Saxony",
		Country:   "DE",
		Elevation: 187,
		Latitude:  52.4572982788,
		Longitude: 9.4271697998,
		Timezone:  "Europe/Berlin",
	},
	"ETOI": {
		ICAO:      "ETOI",
		Name:      "Vilseck Army Air Field",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1353,
		Latitude:  49.6335983276,
		Longitude: 11.76720047,
		Timezone:  "Europe/Berlin",
	},
	"ETOR": {
		ICAO:      "ETOR",
		Name:      "Coleman Army Air Field",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Elevation: 309,
		Latitude:  49.5635986328,
		Longitude: 8.4633903503,
		Timezone:  "Europe/Berlin",
	},
	"ETOU": {
		ICAO:      "ETOU",
		IATA:      "WIE",
		Name:      "Wiesbaden Army Airfield",
		City:      "Wiesbaden",
		State:     "Hesse",
		Country:   "DE",
		Elevation: 461,
		Latitude:  50.0498008728,
		Longitude: 8.3254003525,
		Timezone:  "Europe/Berlin",
	},
	"ETSA": {
		ICAO:      "ETSA",
		Name:      "Landsberg Lech Airport",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 2044,
		Latitude:  48.0705986023,
		Longitude: 10.9060001373,
		Timezone:  "Europe/Berlin",
	},
	"ETSB": {
		ICAO:      "ETSB",
		Name:      "Buchel Airport",
		State:     "Rheinland-Pfalz",
		Country:   "DE",
		Elevation: 1568,
		Latitude:  50.1738014221,
		Longitude: 7.0633301735,
		Timezone:  "Europe/Berlin",
	},
	"ETSE": {
		ICAO:      "ETSE",
		Name:      "Erding Airport",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1515,
		Latitude:  48.3222999573,
		Longitude: 11.9486999512,
		Timezone:  "Europe/Berlin",
	},
	"ETSF": {
		ICAO:      "ETSF",
		IATA:      "FEL",
		Name:      "Furstenfeldbruck Airport",
		City:      "Furstenfeldbruck",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1703,
		Latitude:  48.2055549622,
		Longitude: 11.2669439316,
		Timezone:  "Europe/Berlin",
	},
	"ETSH": {
		ICAO:      "ETSH",
		Name:      "Holzdorf Airport",
		State:     "Brandenburg",
		Country:   "DE",
		Elevation: 265,
		Latitude:  51.7677993774,
		Longitude: 13.1676998138,
		Timezone:  "Europe/Berlin",
	},
	"ETSI": {
		ICAO:      "ETSI",
		IATA:      "IGS",
		Name:      "Ingolstadt Manching Airport",
		City:      "Manching",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1202,
		Latitude:  48.7156982422,
		Longitude: 11.5340003967,
		Timezone:  "Europe/Berlin",
	},
	"ETSL": {
		ICAO:      "ETSL",
		Name:      "Lechfeld Airport",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1822,
		Latitude:  48.1855010986,
		Longitude: 10.8612003326,
		Timezone:  "Europe/Berlin",
	},
	"ETSN": {
		ICAO:      "ETSN",
		Name:      "Neuburg Airport",
		State:     "Bavaria",
		Country:   "DE",
		Elevation: 1249,
		Latitude:  48.7109985352,
		Longitude: 11.2115001678,
		Timezone:  "Europe/Berlin",
	},
	"ETUO": {
		ICAO:      "ETUO",
		IATA:      "GUT",
		Name:      "Gutersloh Airport",
		City:      "Gutersloh",
		State:     "North-Rhine-Westphalia",
		Country:   "DE",
		Elevation: 236,
		Latitude:  51.9227981567,
		Longitude: 8.3063297272,
		Timezone:  "Europe/Berlin",
	},
	"ETWM": {
		ICAO:      "ETWM",
		Name:      "Meppe Airport",
		State:     "Lower-Saxony",
		Country:   "DE",
		Latitude:  52.7232017517,
		Longitude: 7.3263301849,
		Timezone:  "Europe/Berlin",
	},
	"EVAD": {
		ICAO:      "EVAD",
		Name:      "Adazi Airfield",
		City:      "Adazi",
		State:     "Adazi",
		Country:   "LV",
		Elevation: 8,
		Latitude:  57.0985984802,
		Longitude: 24.2658004761,
		Timezone:  "Europe/Riga",
	},
	"EVCA": {
		ICAO:      "EVCA",
		Name:      "Cesis Airport",
		City:      "Cesis",
		State:     "Priekuli",
		Country:   "LV",
		Elevation: 367,
		Latitude:  57.3244438171,
		Longitude: 25.3236103058,
		Timezone:  "Europe/Riga",
	},
	"EVDA": {
		ICAO:      "EVDA",
		IATA:      "DGP",
		Name:      "Daugavpils Intrenational Airport",
		City:      "Daugavpils",
		State:     "Daugavpils-municipality",
		Country:   "LV",
		Elevation: 398,
		Latitude:  55.9447212219,
		Longitude: 26.6650009155,
		Timezone:  "Europe/Riga",
	},
	"EVEA": {
		ICAO:      "EVEA",
		Name:      "Yelgava Airport",
		City:      "Yelgava",
		State:     "Jelgava",
		Country:   "LV",
		Elevation: 20,
		Latitude:  56.6727981567,
		Longitude: 23.6791992188,
		Timezone:  "Europe/Riga",
	},
	"EVFA": {
		ICAO:      "EVFA",
		Name:      "Vainode Airport",
		City:      "Vainode",
		State:     "Vainode",
		Country:   "LV",
		Elevation: 518,
		Latitude:  56.4056015015,
		Longitude: 21.8868999481,
		Timezone:  "Europe/Riga",
	},
	"EVGA": {
		ICAO:      "EVGA",
		Name:      "Lielvarde Airport",
		City:      "Lielvarde",
		State:     "Kegums",
		Country:   "LV",
		Elevation: 200,
		Latitude:  56.778301239,
		Longitude: 24.8539009094,
		Timezone:  "Europe/Riga",
	},
	"EVJA": {
		ICAO:      "EVJA",
		Name:      "Jurmala Airport",
		City:      "Jurmala",
		State:     "Engure",
		Country:   "LV",
		Elevation: 233,
		Latitude:  56.941667,
		Longitude: 23.223333,
		Timezone:  "Europe/Riga",
	},
	"EVKA": {
		ICAO:      "EVKA",
		Name:      "Jekabpils Airport",
		City:      "Jekabpils",
		State:     "Krustpils",
		Country:   "LV",
		Elevation: 289,
		Latitude:  56.5346984863,
		Longitude: 25.8924999237,
		Timezone:  "Europe/Riga",
	},
	"EVLA": {
		ICAO:      "EVLA",
		IATA:      "LPX",
		Name:      "Liepaja International Airport",
		City:      "Liepaja",
		State:     "Grobina",
		Country:   "LV",
		Elevation: 16,
		Latitude:  56.5175018311,
		Longitude: 21.0969009399,
		Timezone:  "Europe/Riga",
	},
	"EVPA": {
		ICAO:      "EVPA",
		Name:      "Ikshkile Airport",
		City:      "Ikshkile",
		State:     "Ikskile",
		Country:   "LV",
		Elevation: 64,
		Latitude:  56.8149986267,
		Longitude: 24.5277996063,
		Timezone:  "Europe/Riga",
	},
	"EVRA": {
		ICAO:      "EVRA",
		IATA:      "RIX",
		Name:      "Riga International Airport",
		City:      "Riga",
		State:     "Marupe",
		Country:   "LV",
		Elevation: 36,
		Latitude:  56.9235992432,
		Longitude: 23.9710998535,
		Timezone:  "Europe/Riga",
	},
	"EVRC": {
		ICAO:      "EVRC",
		Name:      "Rumbula Air Base",
		City:      "Riga",
		State:     "Riga",
		Country:   "LV",
		Elevation: 26,
		Latitude:  56.8833007812,
		Longitude: 24.2266998291,
		Timezone:  "Europe/Riga",
	},
	"EVRS": {
		ICAO:      "EVRS",
		Name:      "Spilve Aerodrome",
		City:      "Riga",
		State:     "Riga",
		Country:   "LV",
		Elevation: 3,
		Latitude:  56.9911003113,
		Longitude: 24.0750007629,
		Timezone:  "Europe/Riga",
	},
	"EVTA": {
		ICAO:      "EVTA",
		Name:      "Tukums Air Base",
		City:      "Tukums",
		State:     "Engure",
		Country:   "LV",
		Elevation: 220,
		Latitude:  56.942199707,
		Longitude: 23.2238998413,
		Timezone:  "Europe/Riga",
	},
	"EVTE": {
		ICAO:      "EVTE",
		Name:      "Talsi Airfield",
		City:      "Talsi",
		State:     "Talsi-Municipality",
		Country:   "LV",
		Latitude:  57.2543984491,
		Longitude: 22.5662183762,
		Timezone:  "Europe/Riga",
	},
	"EVVA": {
		ICAO:      "EVVA",
		IATA:      "VNT",
		Name:      "Ventspils International Airport",
		City:      "Ventspils",
		State:     "Ventspils",
		Country:   "LV",
		Elevation: 19,
		Latitude:  57.35779953,
		Longitude: 21.5442008972,
		Timezone:  "Europe/Riga",
	},
	"EYAL": {
		ICAO:      "EYAL",
		Name:      "Alytus Airport",
		City:      "Alytus",
		State:     "Alytus",
		Country:   "LT",
		Elevation: 269,
		Latitude:  54.412525177,
		Longitude: 24.059928894,
		Timezone:  "Europe/Vilnius",
	},
	"EYBI": {
		ICAO:      "EYBI",
		Name:      "Birzai Airport",
		City:      "Birzai",
		State:     "Panevezys",
		Country:   "LT",
		Elevation: 190,
		Latitude:  56.1758003235,
		Longitude: 24.7602996826,
		Timezone:  "Europe/Vilnius",
	},
	"EYDR": {
		ICAO:      "EYDR",
		Name:      "Druskininkai Airport",
		City:      "Druskininkai",
		State:     "Alytus",
		Country:   "LT",
		Elevation: 302,
		Latitude:  54.0153007507,
		Longitude: 23.9430999756,
		Timezone:  "Europe/Vilnius",
	},
	"EYJB": {
		ICAO:      "EYJB",
		Name:      "Jubarkas Airport",
		City:      "Jubarkas",
		State:     "Taurage-County",
		Country:   "LT",
		Latitude:  55.1180496216,
		Longitude: 22.7648735046,
		Timezone:  "Europe/Vilnius",
	},
	"EYKA": {
		ICAO:      "EYKA",
		IATA:      "KUN",
		Name:      "Kaunas International Airport",
		City:      "Kaunas",
		Country:   "LT",
		Elevation: 256,
		Latitude:  54.9639015198,
		Longitude: 24.0848007202,
		Timezone:  "Europe/Vilnius",
	},
	"EYKD": {
		ICAO:      "EYKD",
		Name:      "Kedainiai Air Base",
		City:      "Kedainiai",
		State:     "Kaunas",
		Country:   "LT",
		Elevation: 171,
		Latitude:  55.3116989136,
		Longitude: 23.9563999176,
		Timezone:  "Europe/Vilnius",
	},
	"EYKG": {
		ICAO:      "EYKG",
		Name:      "Kaunas Gamykla Airport",
		City:      "Kaunas",
		State:     "Kaunas",
		Country:   "LT",
		Elevation: 249,
		Latitude:  54.8800010681,
		Longitude: 23.9071998596,
		Timezone:  "Europe/Vilnius",
	},
	"EYKL": {
		ICAO:      "EYKL",
		IATA:      "KLJ",
		Name:      "Klaipeda Airport",
		City:      "Klaipeda",
		Country:   "LT",
		Elevation: 59,
		Latitude:  55.7118988037,
		Longitude: 21.2427997589,
		Timezone:  "Europe/Vilnius",
	},
	"EYKR": {
		ICAO:      "EYKR",
		Name:      "Kazlu Ruda Air Base",
		City:      "Kazlu Ruda",
		State:     "Marijampole-County",
		Country:   "LT",
		Elevation: 243,
		Latitude:  54.8055992126,
		Longitude: 23.5331001282,
		Timezone:  "Europe/Vilnius",
	},
	"EYKS": {
		ICAO:      "EYKS",
		Name:      "S. Darius and S. Girenas Airport",
		City:      "Kaunas",
		State:     "Kaunas",
		Country:   "LT",
		Elevation: 246,
		Latitude:  54.8797988892,
		Longitude: 23.8815002441,
		Timezone:  "Europe/Vilnius",
	},
	"EYKT": {
		ICAO:      "EYKT",
		Name:      "Kartena Airport",
		City:      "Kartena",
		State:     "Klaipeda-County",
		Country:   "LT",
		Elevation: 259,
		Latitude:  55.92029953,
		Longitude: 21.567199707,
		Timezone:  "Europe/Vilnius",
	},
	"EYLN": {
		ICAO:      "EYLN",
		Name:      "Valenciunai Airport",
		City:      "Valenciunai",
		State:     "Marijampole-County",
		Country:   "LT",
		Elevation: 171,
		Latitude:  55.0214004517,
		Longitude: 22.9738998413,
		Timezone:  "Europe/Vilnius",
	},
	"EYMA": {
		ICAO:      "EYMA",
		Name:      "Tirksliai Airport",
		City:      "Tirksliai / Mazeikiai",
		State:     "Telsiai",
		Country:   "LT",
		Elevation: 259,
		Latitude:  56.2305984497,
		Longitude: 22.2593994141,
		Timezone:  "Europe/Vilnius",
	},
	"EYMM": {
		ICAO:      "EYMM",
		Name:      "Sasnava Airport",
		City:      "Sasnava",
		State:     "Marijampole-County",
		Country:   "LT",
		Elevation: 240,
		Latitude:  54.6631011963,
		Longitude: 23.4538993835,
		Timezone:  "Europe/Vilnius",
	},
	"EYMO": {
		ICAO:      "EYMO",
		Name:      "Moletai Airport",
		City:      "Moletai",
		State:     "Utena",
		Country:   "LT",
		Elevation: 650,
		Latitude:  55.1128005981,
		Longitude: 25.3363990784,
		Timezone:  "Europe/Vilnius",
	},
	"EYNA": {
		ICAO:      "EYNA",
		Name:      "Akmene Airport",
		City:      "Akmene",
		State:     "Siauliai",
		Country:   "LT",
		Elevation: 259,
		Latitude:  56.2421989441,
		Longitude: 22.7331008911,
		Timezone:  "Europe/Vilnius",
	},
	"EYND": {
		ICAO:      "EYND",
		Name:      "Nida Airport",
		City:      "Nida",
		Country:   "LT",
		Latitude:  55.3277778625,
		Longitude: 21.0458335876,
		Timezone:  "Europe/Vilnius",
	},
	"EYPA": {
		ICAO:      "EYPA",
		IATA:      "PLQ",
		Name:      "Palanga International Airport",
		City:      "Palanga",
		Country:   "LT",
		Elevation: 33,
		Latitude:  55.9732017517,
		Longitude: 21.0939006805,
		Timezone:  "Europe/Vilnius",
	},
	"EYPI": {
		ICAO:      "EYPI",
		Name:      "Panevezys Istra Airport",
		City:      "Panevezys / \u00a0Istra",
		State:     "Panevezys",
		Country:   "LT",
		Elevation: 171,
		Latitude:  55.8274993896,
		Longitude: 24.3582992554,
		Timezone:  "Europe/Vilnius",
	},
	"EYPN": {
		ICAO:      "EYPN",
		Name:      "Panevezys Airport",
		City:      "Panevezys",
		State:     "Panevezys",
		Country:   "LT",
		Elevation: 171,
		Latitude:  55.7057418823,
		Longitude: 24.3392658234,
		Timezone:  "Europe/Vilnius",
	},
	"EYPP": {
		ICAO:      "EYPP",
		IATA:      "PNV",
		Name:      "Panevezys Air Base",
		City:      "Panevezys",
		State:     "Panevezys",
		Country:   "LT",
		Elevation: 197,
		Latitude:  55.7294006348,
		Longitude: 24.4608001709,
		Timezone:  "Europe/Vilnius",
	},
	"EYPR": {
		ICAO:      "EYPR",
		Name:      "Pociunai Airport",
		City:      "Pociunai",
		Country:   "LT",
		Elevation: 200,
		Latitude:  54.6493988037,
		Longitude: 24.0618991852,
		Timezone:  "Europe/Vilnius",
	},
	"EYRD": {
		ICAO:      "EYRD",
		Name:      "Rudiskes Airport",
		City:      "Rudiskes",
		State:     "Vilnius",
		Country:   "LT",
		Elevation: 522,
		Latitude:  54.4956016541,
		Longitude: 24.7175006866,
		Timezone:  "Europe/Vilnius",
	},
	"EYRK": {
		ICAO:      "EYRK",
		Name:      "Rokiskis Airport",
		City:      "Rokiskis",
		State:     "Panevezys",
		Country:   "LT",
		Elevation: 459,
		Latitude:  55.9719238281,
		Longitude: 25.6042385101,
		Timezone:  "Europe/Vilnius",
	},
	"EYRO": {
		ICAO:      "EYRO",
		Name:      "Rojunai Airport",
		City:      "Rojunai",
		State:     "Panevezys",
		Country:   "LT",
		Elevation: 180,
		Latitude:  55.6108016968,
		Longitude: 24.2208003998,
		Timezone:  "Europe/Vilnius",
	},
	"EYRU": {
		ICAO:      "EYRU",
		Name:      "Jonava Airport",
		City:      "Jonava",
		Country:   "LT",
		Elevation: 246,
		Latitude:  55.0099983215,
		Longitude: 24.3633003235,
		Timezone:  "Europe/Vilnius",
	},
	"EYSA": {
		ICAO:      "EYSA",
		IATA:      "SQQ",
		Name:      "Siauliai International Airport",
		City:      "Siauliai",
		Country:   "LT",
		Elevation: 443,
		Latitude:  55.893901825,
		Longitude: 23.3950004578,
		Timezone:  "Europe/Vilnius",
	},
	"EYSB": {
		ICAO:      "EYSB",
		IATA:      "HLJ",
		Name:      "Barysiai Airport",
		City:      "Barysiai",
		State:     "Siauliai",
		Country:   "LT",
		Elevation: 270,
		Latitude:  56.0705986023,
		Longitude: 23.5580997467,
		Timezone:  "Europe/Vilnius",
	},
	"EYSE": {
		ICAO:      "EYSE",
		Name:      "Seduva Airport",
		City:      "Seduva / Siauliai",
		State:     "Siauliai",
		Country:   "LT",
		Elevation: 259,
		Latitude:  55.7463989258,
		Longitude: 23.8047008514,
		Timezone:  "Europe/Vilnius",
	},
	"EYSI": {
		ICAO:      "EYSI",
		Name:      "Silute Air Base",
		City:      "Silute",
		State:     "Klaipeda-County",
		Country:   "LT",
		Elevation: 59,
		Latitude:  55.3368988037,
		Longitude: 21.5305995941,
		Timezone:  "Europe/Vilnius",
	},
	"EYTL": {
		ICAO:      "EYTL",
		Name:      "Telsiai Airport",
		City:      "Telsiai",
		State:     "Telsiai",
		Country:   "LT",
		Elevation: 410,
		Latitude:  55.9864006042,
		Longitude: 22.2877998352,
		Timezone:  "Europe/Vilnius",
	},
	"EYUT": {
		ICAO:      "EYUT",
		Name:      "Utena Airport",
		City:      "Utena",
		State:     "Utena",
		Country:   "LT",
		Elevation: 620,
		Latitude:  55.4900016785,
		Longitude: 25.7168998718,
		Timezone:  "Europe/Vilnius",
	},
	"EYVI": {
		ICAO:      "EYVI",
		IATA:      "VNO",
		Name:      "Vilnius International Airport",
		City:      "Vilnius",
		State:     "Vilnius",
		Country:   "LT",
		Elevation: 646,
		Latitude:  54.6341018677,
		Longitude: 25.2858009338,
		Timezone:  "Europe/Vilnius",
	},
	"EYVK": {
		ICAO:      "EYVK",
		Name:      "Kyviskes Airfield",
		City:      "Kyviskes",
		State:     "Vilnius",
		Country:   "LT",
		Elevation: 541,
		Latitude:  54.6683006287,
		Longitude: 25.5158004761,
		Timezone:  "Europe/Vilnius",
	},
	"EYVP": {
		ICAO:      "EYVP",
		Name:      "Paluknys Airport",
		City:      "Paluknys",
		State:     "Vilnius",
		Country:   "LT",
		Elevation: 400,
		Latitude:  54.4848480225,
		Longitude: 24.9873733521,
		Timezone:  "Europe/Vilnius",
	},
	"EYZA": {
		ICAO:      "EYZA",
		Name:      "Zarasai Airport",
		City:      "Zarasai",
		State:     "Utena",
		Country:   "LT",
		Elevation: 541,
		Latitude:  55.7524986267,
		Longitude: 26.2569007874,
		Timezone:  "Europe/Riga",
	},
	"FA03": {
		ICAO:      "FA03",
		Name:      "Southfork Airport",
		City:      "Duette",
		State:     "Florida",
		Country:   "US",
		Elevation: 109,
		Latitude:  27.6039009094,
		Longitude: -82.1929016113,
		Timezone:  "America/New_York",
	},
	"FA08": {
		ICAO:      "FA08",
		Name:      "Orlampa Inc Airport",
		City:      "Polk City",
		State:     "Florida",
		Country:   "US",
		Elevation: 139,
		Latitude:  28.1669998169,
		Longitude: -81.8080978394,
		Timezone:  "America/New_York",
	},
	"FA09": {
		ICAO:      "FA09",
		Name:      "Arliss M Airport",
		City:      "Fairfield",
		State:     "Florida",
		Country:   "US",
		Elevation: 200,
		Latitude:  29.3407993317,
		Longitude: -82.2637023926,
		Timezone:  "America/New_York",
	},
	"FA11": {
		ICAO:      "FA11",
		Name:      "Bird Field",
		City:      "Jennings",
		State:     "Florida",
		Country:   "US",
		Elevation: 150,
		Latitude:  30.6224002838,
		Longitude: -83.2593002319,
		Timezone:  "America/New_York",
	},
	"FA18": {
		ICAO:      "FA18",
		Name:      "Devils Garden Strip",
		City:      "La Belle",
		State:     "Florida",
		Country:   "US",
		Elevation: 18,
		Latitude:  28.4510993958,
		Longitude: -81.0002975464,
		Timezone:  "America/New_York",
	},
	"FA24": {
		ICAO:      "FA24",
		Name:      "Horseshoe Acres Airpark",
		City:      "Indiantown",
		State:     "Florida",
		Country:   "US",
		Elevation: 9,
		Latitude:  27.0932998657,
		Longitude: -80.612197876,
		Timezone:  "America/New_York",
	},
	"FA25": {
		ICAO:      "FA25",
		Name:      "Black Creek Pass Airport",
		City:      "Tallahasse",
		State:     "Florida",
		Country:   "US",
		Elevation: 95,
		Latitude:  30.50095,
		Longitude: -84.0781,
		Timezone:  "America/New_York",
	},
	"FA26": {
		ICAO:      "FA26",
		Name:      "Dogwood Farm Airport",
		City:      "Havana",
		State:     "Florida",
		Country:   "US",
		Elevation: 200,
		Latitude:  30.5372009277,
		Longitude: -84.4518966675,
		Timezone:  "America/New_York",
	},
	"FA27": {
		ICAO:      "FA27",
		Name:      "Ellsworth Field",
		City:      "Arcadia",
		State:     "Florida",
		Country:   "US",
		Elevation: 60,
		Latitude:  27.2056007385,
		Longitude: -81.7771987915,
		Timezone:  "America/New_York",
	},
	"FA29": {
		ICAO:      "FA29",
		Name:      "Lumar Field",
		City:      "Perry",
		State:     "Florida",
		Country:   "US",
		Elevation: 65,
		Latitude:  30.1308002472,
		Longitude: -83.5494003296,
		Timezone:  "America/New_York",
	},
	"FA30": {
		ICAO:      "FA30",
		Name:      "Redtail Airstrip",
		City:      "Morriston",
		State:     "Florida",
		Country:   "US",
		Elevation: 65,
		Latitude:  29.2791996002,
		Longitude: -82.5,
		Timezone:  "America/New_York",
	},
	"FA32": {
		ICAO:      "FA32",
		Name:      "Circle P Airport",
		City:      "Perry",
		State:     "Florida",
		Country:   "US",
		Elevation: 83,
		Latitude:  30.1033000946,
		Longitude: -83.450302124,
		Timezone:  "America/New_York",
	},
	"FA35": {
		ICAO:      "FA35",
		Name:      "Lindbergh's Landing Airport",
		City:      "Miami",
		State:     "Florida",
		Country:   "US",
		Elevation: 9,
		Latitude:  25.6194992065,
		Longitude: -80.4887008667,
		Timezone:  "America/New_York",
	},
	"FA36": {
		ICAO:      "FA36",
		Name:      "White Farms Airport",
		City:      "Chiefland",
		State:     "Florida",
		Country:   "US",
		Elevation: 46,
		Latitude:  29.5126991272,
		Longitude: -82.8748016357,
		Timezone:  "America/New_York",
	},
	"FA37": {
		ICAO:      "FA37",
		Name:      "Wing South Airpark",
		City:      "Naples",
		State:     "Florida",
		Country:   "US",
		Elevation: 9,
		Latitude:  26.1168003082,
		Longitude: -81.7031021118,
		Timezone:  "America/New_York",
	},
	"FA38": {
		ICAO:      "FA38",
		Name:      "Woods and Lakes Airpark",
		City:      "Oklawaha",
		State:     "Florida",
		Country:   "US",
		Elevation: 100,
		Latitude:  29.1236000061,
		Longitude: -81.8870010376,
		Timezone:  "America/New_York",
	},
	"FA40": {
		ICAO:      "FA40",
		Name:      "Hidden Lake Airport",
		City:      "New Port Richey",
		State:     "Florida",
		Country:   "US",
		Elevation: 30,
		Latitude:  28.2824993134,
		Longitude: -82.6456985474,
		Timezone:  "America/New_York",
	},
	"FA42": {
		ICAO:      "FA42",
		Name:      "Sheets Airport",
		City:      "Groveland",
		State:     "Florida",
		Country:   "US",
		Elevation: 118,
		Latitude:  28.5328998566,
		Longitude: -81.8467025757,
		Timezone:  "America/New_York",
	},
	"FA43": {
		ICAO:      "FA43",
		Name:      "Dog Island Airport",
		City:      "Carrabelle",
		State:     "Florida",
		Country:   "US",
		Elevation: 4,
		Latitude:  29.8085002899,
		Longitude: -84.595703125,
		Timezone:  "America/New_York",
	},
	"FA44": {
		ICAO:      "FA44",
		Name:      "Willis Gliderport",
		City:      "Boynton Beach",
		State:     "Florida",
		Country:   "US",
		Elevation: 18,
		Latitude:  26.5480995178,
		Longitude: -80.2147979736,
		Timezone:  "America/New_York",
	},
	"FA46": {
		ICAO:      "FA46",
		Name:      "Henderson Field",
		City:      "Minneola",
		State:     "Florida",
		Country:   "US",
		Elevation: 90,
		Latitude:  28.633600235,
		Longitude: -81.7080993652,
		Timezone:  "America/New_York",
	},
	"FA49": {
		ICAO:      "FA49",
		Name:      "Shady International Airport",
		City:      "Ocala",
		State:     "Florida",
		Country:   "US",
		Elevation: 102,
		Latitude:  29.0960998535,
		Longitude: -82.1790008545,
		Timezone:  "America/New_York",
	},
	"FA50": {
		ICAO:      "FA50",
		Name:      "Wings-N-Wheels Airport",
		City:      "Reddick",
		State:     "Florida",
		Country:   "US",
		Elevation: 82,
		Latitude:  29.3614006042,
		Longitude: -82.1498031616,
		Timezone:  "America/New_York",
	},
	"FA51": {
		ICAO:      "FA51",
		Name:      "William's Sky Manor Airport",
		City:      "La Belle",
		State:     "Florida",
		Country:   "US",
		Elevation: 15,
		Latitude:  26.7390003204,
		Longitude: -81.5062026978,
		Timezone:  "America/New_York",
	},
	"FA54": {
		ICAO:      "FA54",
		Name:      "Coral Creek Airport",
		City:      "Placida",
		State:     "Florida",
		Country:   "US",
		Elevation: 4,
		Latitude:  26.8544998169,
		Longitude: -82.2511978149,
		Timezone:  "America/New_York",
	},
	"FA55": {
		ICAO:      "FA55",
		Name:      "Garnair Airport",
		City:      "Bonifay",
		State:     "Florida",
		Country:   "US",
		Elevation: 105,
		Latitude:  30.7796993256,
		Longitude: -85.7683029175,
		Timezone:  "America/Chicago",
	},
	"FA60": {
		ICAO:      "FA60",
		Name:      "Vince's Condominium Association Airport",
		City:      "Lake Placid",
		State:     "Florida",
		Country:   "US",
		Elevation: 115,
		Latitude:  27.238899231,
		Longitude: -81.3859024048,
		Timezone:  "America/New_York",
	},
	"FA62": {
		ICAO:      "FA62",
		Name:      "Santa Fe River Ranch Airport",
		City:      "Alachua",
		State:     "Florida",
		Country:   "US",
		Elevation: 145,
		Latitude:  29.9169006348,
		Longitude: -82.4832000732,
		Timezone:  "America/New_York",
	},
	"FA63": {
		ICAO:      "FA63",
		Name:      "Leffler Airport",
		City:      "Alamana",
		State:     "Florida",
		Country:   "US",
		Elevation: 43,
		Latitude:  28.9636001587,
		Longitude: -81.1213989258,
		Timezone:  "America/New_York",
	},
	"FA64": {
		ICAO:      "FA64",
		Name:      "Montgomery Ranch Airport",
		City:      "Arcadia",
		State:     "Florida",
		Country:   "US",
		Elevation: 75,
		Latitude:  27.0744991302,
		Longitude: -81.5865020752,
		Timezone:  "America/New_York",
	},
	"FA69": {
		ICAO:      "FA69",
		Name:      "Duda Airstrip",
		City:      "Belle Glade",
		State:     "Florida",
		Country:   "US",
		Elevation: 14,
		Latitude:  26.5877990723,
		Longitude: -80.637298584,
		Timezone:  "America/New_York",
	},
	"FA71": {
		ICAO:      "FA71",
		Name:      "The Cedars Airfield",
		City:      "Cedar Key",
		State:     "Florida",
		Country:   "US",
		Elevation: 8,
		Latitude:  29.2280006409,
		Longitude: -82.9437026978,
		Timezone:  "America/New_York",
	},
	"FA72": {
		ICAO:      "FA72",
		Name:      "Acres of Diamonds Airpark",
		City:      "Greenwood",
		State:     "Florida",
		Country:   "US",
		Elevation: 134,
		Latitude:  30.8862991333,
		Longitude: -85.1638031006,
		Timezone:  "America/Chicago",
	},
	"FA74": {
		ICAO:      "FA74",
		Name:      "Romor Ranch Airport",
		City:      "Ochopee",
		State:     "Florida",
		Country:   "US",
		Elevation: 12,
		Latitude:  25.9808998108,
		Longitude: -81.0567016602,
		Timezone:  "America/New_York",
	},
	"FA77": {
		ICAO:      "FA77",
		Name:      "Lowe's Airport",
		City:      "Sarasota",
		State:     "Florida",
		Country:   "US",
		Elevation: 54,
		Latitude:  27.3309001923,
		Longitude: -82.2906036377,
		Timezone:  "America/New_York",
	},
	"FA80": {
		ICAO:      "FA80",
		Name:      "Lost Horn Ranch Airport",
		City:      "Ochopee",
		State:     "Florida",
		Country:   "US",
		Elevation: 12,
		Latitude:  25.9680995941,
		Longitude: -81.0814971924,
		Timezone:  "America/New_York",
	},
	"FA81": {
		ICAO:      "FA81",
		Name:      "Tavernaero Park Airport",
		City:      "Tavernier",
		State:     "Florida",
		Country:   "US",
		Elevation: 10,
		Latitude:  25.0086994171,
		Longitude: -80.5330963135,
		Timezone:  "America/New_York",
	},
	"FA83": {
		ICAO:      "FA83",
		Name:      "Orlando North Airpark",
		City:      "Zellwood",
		State:     "Florida",
		Country:   "US",
		Elevation: 80,
		Latitude:  28.7264003754,
		Longitude: -81.6509017944,
		Timezone:  "America/New_York",
	},
	"FA88": {
		ICAO:      "FA88",
		Name:      "Pittman Oaks Airport",
		City:      "Live Oak",
		State:     "Florida",
		Country:   "US",
		Elevation: 83,
		Latitude:  30.425042,
		Longitude: -83.048469,
		Timezone:  "America/New_York",
	},
	"FA99": {
		ICAO:      "FA99",
		Name:      "Fantasy Field Airstrip",
		City:      "Chehalis",
		State:     "Washington",
		Country:   "US",
		Elevation: 300,
		Latitude:  46.587833,
		Longitude: -122.865778,
		Timezone:  "America/Los_Angeles",
	},
	"FAAB": {
		ICAO:      "FAAB",
		IATA:      "ALJ",
		Name:      "Alexander Bay Airport",
		City:      "Alexander Bay",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 98,
		Latitude:  -28.5750007629,
		Longitude: 16.5333003998,
		Timezone:  "Africa/Johannesburg",
	},
	"FAAD": {
		ICAO:      "FAAD",
		Name:      "Adelaide Airport",
		City:      "Adelaide",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 1955,
		Latitude:  -32.6831016541,
		Longitude: 26.2942008972,
		Timezone:  "Africa/Johannesburg",
	},
	"FAAE": {
		ICAO:      "FAAE",
		Name:      "Aberdeen Airport",
		City:      "Aberdeen",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 2461,
		Latitude:  -32.4667015076,
		Longitude: 24.0666999817,
		Timezone:  "Africa/Johannesburg",
	},
	"FAAF": {
		ICAO:      "FAAF",
		Name:      "Andrew's Field Airport",
		City:      "Struisbaai",
		State:     "Western-Cape",
		Country:   "ZA",
		Elevation: 30,
		Latitude:  -34.7628898621,
		Longitude: 20.0349159241,
		Timezone:  "Africa/Johannesburg",
	},
	"FAAG": {
		ICAO:      "FAAG",
		IATA:      "AGZ",
		Name:      "Aggeneys Airport",
		City:      "Aggeneys",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 2648,
		Latitude:  -29.2817993164,
		Longitude: 18.8138999939,
		Timezone:  "Africa/Johannesburg",
	},
	"FAAK": {
		ICAO:      "FAAK",
		Name:      "Askham Airport",
		City:      "Askham",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 2801,
		Latitude:  -26.9869441986,
		Longitude: 20.7772216797,
		Timezone:  "Africa/Johannesburg",
	},
	"FAAL": {
		ICAO:      "FAAL",
		IATA:      "ADY",
		Name:      "Alldays Airport",
		City:      "Alldays",
		State:     "Limpopo",
		Country:   "ZA",
		Elevation: 2600,
		Latitude:  -22.6790008545,
		Longitude: 29.0555000305,
		Timezone:  "Africa/Johannesburg",
	},
	"FAAM": {
		ICAO:      "FAAM",
		Name:      "Amsterdam Airport",
		City:      "Amsterdam",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 4310,
		Latitude:  -26.6333332062,
		Longitude: 30.6000003815,
		Timezone:  "Africa/Johannesburg",
	},
	"FAAN": {
		ICAO:      "FAAN",
		Name:      "Aliwal North Airport",
		City:      "Aliwal North",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 4405,
		Latitude:  -30.6800003052,
		Longitude: 26.7299995422,
		Timezone:  "Africa/Johannesburg",
	},
	"FAAP": {
		ICAO:      "FAAP",
		Name:      "Aviators Paradise Field",
		State:     "North-West",
		Country:   "ZA",
		Elevation: 3800,
		Latitude:  -25.69389,
		Longitude: 27.7825073972,
		Timezone:  "Africa/Johannesburg",
	},
	"FAAS": {
		ICAO:      "FAAS",
		Name:      "Ashton Airport",
		City:      "Ashton",
		State:     "Western-Cape",
		Country:   "ZA",
		Elevation: 525,
		Latitude:  -33.8166656494,
		Longitude: 20.0666675568,
		Timezone:  "Africa/Johannesburg",
	},
	"FABA": {
		ICAO:      "FABA",
		Name:      "Microland Flight Park",
		City:      "Bapsfontein",
		State:     "Gauteng",
		Country:   "ZA",
		Elevation: 5476,
		Latitude:  -25.9762557716,
		Longitude: 28.3890151978,
		Timezone:  "Africa/Johannesburg",
	},
	"FABB": {
		ICAO:      "FABB",
		Name:      "Brakpan Airport",
		City:      "Brakpan",
		State:     "Gauteng",
		Country:   "ZA",
		Elevation: 5300,
		Latitude:  -26.2385997772,
		Longitude: 28.3017997742,
		Timezone:  "Africa/Johannesburg",
	},
	"FABD": {
		ICAO:      "FABD",
		Name:      "Burghersdorp Airport",
		City:      "Burgersdorp",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 4734,
		Latitude:  -30.9775009155,
		Longitude: 26.3080997467,
		Timezone:  "Africa/Johannesburg",
	},
	"FABE": {
		ICAO:      "FABE",
		IATA:      "BIY",
		Name:      "Bisho Airport",
		City:      "Bisho",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 1950,
		Latitude:  -32.8970985413,
		Longitude: 27.2791004181,
		Timezone:  "Africa/Johannesburg",
	},
	"FABF": {
		ICAO:      "FABF",
		Name:      "Barkley East Airport",
		City:      "Barkley East",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 5940,
		Latitude:  -30.9666671753,
		Longitude: 27.6041660309,
		Timezone:  "Africa/Johannesburg",
	},
	"FABG": {
		ICAO:      "FABG",
		Name:      "Buffelshoek Airport",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 1347,
		Latitude:  -24.702464,
		Longitude: 31.587875,
		Timezone:  "Africa/Johannesburg",
	},
	"FABH": {
		ICAO:      "FABH",
		Name:      "Belfast Aerodrome",
		City:      "Belfast",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 6250,
		Latitude:  -25.6587417378,
		Longitude: 30.041577816,
		Timezone:  "Africa/Johannesburg",
	},
	"FABK": {
		ICAO:      "FABK",
		Name:      "Bushmans Kloof Airport",
		City:      "Bushman's Kloof",
		State:     "Western-Cape",
		Country:   "ZA",
		Elevation: 1050,
		Latitude:  -32.035,
		Longitude: 19.0435,
		Timezone:  "Africa/Johannesburg",
	},
	"FABL": {
		ICAO:      "FABL",
		IATA:      "BFN",
		Name:      "J B M Hertzog International Airport",
		City:      "Bloemfontain",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 4458,
		Latitude:  -29.0926990509,
		Longitude: 26.302400589,
		Timezone:  "Africa/Johannesburg",
	},
	"FABM": {
		ICAO:      "FABM",
		Name:      "Bethlehem Airport",
		City:      "Bethlehem",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 5561,
		Latitude:  -28.2483997345,
		Longitude: 28.3360996246,
		Timezone:  "Africa/Johannesburg",
	},
	"FABO": {
		ICAO:      "FABO",
		Name:      "Hendrik Potgieter Airport",
		City:      "Bothaville",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 4236,
		Latitude:  -27.3668003082,
		Longitude: 26.6291999817,
		Timezone:  "Africa/Johannesburg",
	},
	"FABP": {
		ICAO:      "FABP",
		Name:      "Black Rock Airport",
		City:      "Santoy",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 3450,
		Latitude:  -27.129724,
		Longitude: 22.846255,
		Timezone:  "Africa/Johannesburg",
	},
	"FABR": {
		ICAO:      "FABR",
		Name:      "Barberton Airport",
		City:      "Umjindi (Barberton)",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 2250,
		Latitude:  -25.7168687423,
		Longitude: 30.9751796722,
		Timezone:  "Africa/Johannesburg",
	},
	"FABS": {
		ICAO:      "FABS",
		Name:      "Brits Airport",
		City:      "Brits",
		State:     "North-West",
		Country:   "ZA",
		Elevation: 3756,
		Latitude:  -25.5323009491,
		Longitude: 27.7758998871,
		Timezone:  "Africa/Johannesburg",
	},
	"FABT": {
		ICAO:      "FABT",
		Name:      "Bethesda Road Airport",
		City:      "Bethesda",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 4630,
		Latitude:  -31.8731994629,
		Longitude: 24.7968997955,
		Timezone:  "Africa/Johannesburg",
	},
	"FABU": {
		ICAO:      "FABU",
		IATA:      "UTE",
		Name:      "Bultfontein Airport",
		City:      "Bultfontein",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 4400,
		Latitude:  -28.273369,
		Longitude: 26.135835,
		Timezone:  "Africa/Johannesburg",
	},
	"FABV": {
		ICAO:      "FABV",
		Name:      "Brandviei Airport",
		City:      "Brandviei",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 2999,
		Latitude:  -30.4666996002,
		Longitude: 20.4666996002,
		Timezone:  "Africa/Johannesburg",
	},
	"FABW": {
		ICAO:      "FABW",
		Name:      "Beaufort West Airport",
		City:      "Beaufort West",
		State:     "Western-Cape",
		Country:   "ZA",
		Elevation: 2941,
		Latitude:  -32.2999992371,
		Longitude: 22.6700000763,
		Timezone:  "Africa/Johannesburg",
	},
	"FABX": {
		ICAO:      "FABX",
		Name:      "Beatrix Airport",
		City:      "Virginia",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 4518,
		Latitude:  -28.2450008392,
		Longitude: 26.771900177,
		Timezone:  "Africa/Johannesburg",
	},
	"FABZ": {
		ICAO:      "FABZ",
		Name:      "Bizana Airport",
		City:      "Bizana",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 2740,
		Latitude:  -30.8603000641,
		Longitude: 29.863899231,
		Timezone:  "Africa/Johannesburg",
	},
	"FACA": {
		ICAO:      "FACA",
		Name:      "Monte Carlo Airport",
		City:      "Monte Carlo",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 4995,
		Latitude:  -28.7999992371,
		Longitude: 27.4169998169,
		Timezone:  "Africa/Johannesburg",
	},
	"FACB": {
		ICAO:      "FACB",
		Name:      "Colesberg Airport",
		City:      "Colesberg",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 4639,
		Latitude:  -30.7339000702,
		Longitude: 25.0650005341,
		Timezone:  "Africa/Johannesburg",
	},
	"FACC": {
		ICAO:      "FACC",
		Name:      "Arathusa Safari Lodge Airport",
		City:      "Arathusa",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 1200,
		Latitude:  -24.7441673279,
		Longitude: 31.5224990845,
		Timezone:  "Africa/Johannesburg",
	},
	"FACD": {
		ICAO:      "FACD",
		IATA:      "CDO",
		Name:      "Cradock Airport",
		City:      "Cradock",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 3110,
		Latitude:  -32.1567001343,
		Longitude: 25.6455993652,
		Timezone:  "Africa/Johannesburg",
	},
	"FACE": {
		ICAO:      "FACE",
		Name:      "Ceres Airport",
		City:      "Ceres",
		State:     "Western-Cape",
		Country:   "ZA",
		Elevation: 1519,
		Latitude:  -33.3530006409,
		Longitude: 19.3113994598,
		Timezone:  "Africa/Johannesburg",
	},
	"FACF": {
		ICAO:      "FACF",
		Name:      "Cap St Francis Airport",
		City:      "St Francis",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 131,
		Latitude:  -34.1869010925,
		Longitude: 24.8349990845,
		Timezone:  "Africa/Johannesburg",
	},
	"FACH": {
		ICAO:      "FACH",
		Name:      "Cookhouse Airport",
		City:      "Cookhouse",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 1893,
		Latitude:  -32.7447013855,
		Longitude: 25.7963008881,
		Timezone:  "Africa/Johannesburg",
	},
	"FACI": {
		ICAO:      "FACI",
		Name:      "Citrusdal Airport",
		City:      "Citrusdal",
		State:     "Western-Cape",
		Country:   "ZA",
		Elevation: 532,
		Latitude:  -32.6176986694,
		Longitude: 19.005399704,
		Timezone:  "Africa/Johannesburg",
	},
	"FACK": {
		ICAO:      "FACK",
		Name:      "Christiana Airport",
		City:      "Christiana",
		State:     "North-West",
		Country:   "ZA",
		Elevation: 3978,
		Latitude:  -27.8777008057,
		Longitude: 25.2017993927,
		Timezone:  "Africa/Johannesburg",
	},
	"FACL": {
		ICAO:      "FACL",
		Name:      "Carolina Airport",
		City:      "Carolina",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 5420,
		Latitude:  -26.0781002045,
		Longitude: 30.0925006866,
		Timezone:  "Africa/Johannesburg",
	},
	"FACN": {
		ICAO:      "FACN",
		Name:      "Carnarvon Airport",
		City:      "Carnarvon",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 4160,
		Latitude:  -30.9885997772,
		Longitude: 22.1317005157,
		Timezone:  "Africa/Johannesburg",
	},
	"FACO": {
		ICAO:      "FACO",
		Name:      "Alkantpan Copper Airport",
		City:      "Cooperton",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 3589,
		Latitude:  -29.9064006805,
		Longitude: 22.3166999817,
		Timezone:  "Africa/Johannesburg",
	},
	"FACR": {
		ICAO:      "FACR",
		Name:      "Carltonville Airport",
		City:      "Carltonville",
		State:     "Gauteng",
		Country:   "ZA",
		Elevation: 5000,
		Latitude:  -26.3666667938,
		Longitude: 27.3466663361,
		Timezone:  "Africa/Johannesburg",
	},
	"FACT": {
		ICAO:      "FACT",
		IATA:      "CPT",
		Name:      "Cape Town International Airport",
		City:      "Cape Town",
		State:     "Western-Cape",
		Country:   "ZA",
		Elevation: 151,
		Latitude:  -33.9648017883,
		Longitude: 18.6016998291,
		Timezone:  "Africa/Johannesburg",
	},
	"FACV": {
		ICAO:      "FACV",
		Name:      "Calvinia Airport",
		City:      "Calvinia",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 3250,
		Latitude:  -31.5002994537,
		Longitude: 19.72590065,
		Timezone:  "Africa/Johannesburg",
	},
	"FACW": {
		ICAO:      "FACW",
		Name:      "Clanwilliam Airport",
		City:      "Clanwilliam",
		State:     "Western-Cape",
		Country:   "ZA",
		Elevation: 650,
		Latitude:  -32.1831016541,
		Longitude: 18.8885993958,
		Timezone:  "Africa/Johannesburg",
	},
	"FACX": {
		ICAO:      "FACX",
		Name:      "Cathcart Airport",
		City:      "Cathcart",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 3959,
		Latitude:  -32.2871017456,
		Longitude: 27.1387996674,
		Timezone:  "Africa/Johannesburg",
	},
	"FACY": {
		ICAO:      "FACY",
		Name:      "Stilbaai Airport",
		City:      "Stilbaai",
		State:     "Western-Cape",
		Country:   "ZA",
		Elevation: 266,
		Latitude:  -34.3517990112,
		Longitude: 21.4248008728,
		Timezone:  "Africa/Johannesburg",
	},
	"FADA": {
		ICAO:      "FADA",
		Name:      "De Aar Airport",
		City:      "De Aar",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 4154,
		Latitude:  -30.692199707,
		Longitude: 24.0205993652,
		Timezone:  "Africa/Johannesburg",
	},
	"FADB": {
		ICAO:      "FADB",
		Name:      "Dwaalboom Airport",
		City:      "Dwaalboom",
		State:     "Limpopo",
		Country:   "ZA",
		Elevation: 3652,
		Latitude:  -24.8071994781,
		Longitude: 26.82970047,
		Timezone:  "Africa/Johannesburg",
	},
	"FADD": {
		ICAO:      "FADD",
		Name:      "Dundee Airport",
		City:      "Dundee",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 4219,
		Latitude:  -28.1830997467,
		Longitude: 30.2224998474,
		Timezone:  "Africa/Johannesburg",
	},
	"FADE": {
		ICAO:      "FADE",
		Name:      "Delmas Airport",
		City:      "Delmas",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 5075,
		Latitude:  -26.1416664124,
		Longitude: 28.6916675568,
		Timezone:  "Africa/Johannesburg",
	},
	"FADG": {
		ICAO:      "FADG",
		Name:      "Dordrecht Airport",
		City:      "Dordrecht",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 5200,
		Latitude:  -31.3833332062,
		Longitude: 27.0333328247,
		Timezone:  "Africa/Johannesburg",
	},
	"FADH": {
		ICAO:      "FADH",
		Name:      "Durnacol Airport",
		City:      "Durnacol",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 4317,
		Latitude:  -28.0461006165,
		Longitude: 30.0063991547,
		Timezone:  "Africa/Johannesburg",
	},
	"FADK": {
		ICAO:      "FADK",
		IATA:      "DUK",
		Name:      "Mubatuba Airport",
		City:      "Mubatuba",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 210,
		Latitude:  -28.3684005737,
		Longitude: 32.2481002808,
		Timezone:  "Africa/Johannesburg",
	},
	"FADL": {
		ICAO:      "FADL",
		Name:      "Delareyville Airport",
		City:      "Delareyville",
		State:     "North-West",
		Country:   "ZA",
		Elevation: 4469,
		Latitude:  -26.6791992188,
		Longitude: 25.4747009277,
		Timezone:  "Africa/Johannesburg",
	},
	"FADM": {
		ICAO:      "FADM",
		Name:      "Kokstad Airport",
		City:      "Kokstad",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 4084,
		Latitude:  -30.5564994812,
		Longitude: 29.4081993103,
		Timezone:  "Africa/Johannesburg",
	},
	"FADN": {
		ICAO:      "FADN",
		Name:      "Durban International Airport",
		City:      "Durban",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 33,
		Latitude:  -29.9701004028,
		Longitude: 30.9505004883,
		Timezone:  "Africa/Johannesburg",
	},
	"FADO": {
		ICAO:      "FADO",
		Name:      "Dendron Airport",
		City:      "Dendron",
		State:     "Limpopo",
		Country:   "ZA",
		Elevation: 3399,
		Latitude:  -23.3805999756,
		Longitude: 29.3208007813,
		Timezone:  "Africa/Johannesburg",
	},
	"FADP": {
		ICAO:      "FADP",
		Name:      "Darlington Dam Lodge Airport",
		City:      "Port Elizabeth",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 847,
		Latitude:  -33.1832008362,
		Longitude: 25.1931991577,
		Timezone:  "Africa/Johannesburg",
	},
	"FADQ": {
		ICAO:      "FADQ",
		IATA:      "PZL",
		Name:      "Zulu Inyala Airport",
		City:      "Phinda",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 160,
		Latitude:  -27.8493995667,
		Longitude: 32.3097000122,
		Timezone:  "Africa/Johannesburg",
	},
	"FADR": {
		ICAO:      "FADR",
		Name:      "Dunnottar Airport",
		City:      "Dunnottar",
		State:     "Gauteng",
		Country:   "ZA",
		Elevation: 5241,
		Latitude:  -26.3497009277,
		Longitude: 28.4419002533,
		Timezone:  "Africa/Johannesburg",
	},
	"FADS": {
		ICAO:      "FADS",
		Name:      "De Doorns Airport",
		City:      "De Doorns",
		State:     "Western-Cape",
		Country:   "ZA",
		Elevation: 1626,
		Latitude:  -33.455557,
		Longitude: 19.684493,
		Timezone:  "Africa/Johannesburg",
	},
	"FADU": {
		ICAO:      "FADU",
		Name:      "Walkersons Field",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 6041,
		Latitude:  -25.36278,
		Longitude: 30.18778,
		Timezone:  "Africa/Johannesburg",
	},
	"FADV": {
		ICAO:      "FADV",
		Name:      "Devon Airport",
		City:      "Devon",
		State:     "Gauteng",
		Country:   "ZA",
		Elevation: 5460,
		Latitude:  -26.3589992523,
		Longitude: 28.7896995544,
		Timezone:  "Africa/Johannesburg",
	},
	"FADX": {
		ICAO:      "FADX",
		Name:      "Delta 200 Airstrip",
		City:      "Koeberg",
		State:     "Western-Cape",
		Country:   "ZA",
		Elevation: 220,
		Latitude:  -33.6494935368,
		Longitude: 18.4718799591,
		Timezone:  "Africa/Johannesburg",
	},
	"FADY": {
		ICAO:      "FADY",
		Name:      "De Aar Military Airport",
		City:      "De Aar",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 4037,
		Latitude:  -30.6365089417,
		Longitude: 23.9197254181,
		Timezone:  "Africa/Johannesburg",
	},
	"FADZ": {
		ICAO:      "FADZ",
		Name:      "Drakensberg Gardens Airport",
		City:      "Drakensberg Gardens",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 5948,
		Latitude:  -29.75,
		Longitude: 29.25,
		Timezone:  "Africa/Johannesburg",
	},
	"FAEC": {
		ICAO:      "FAEC",
		Name:      "Estcourt Airport",
		City:      "Estcourt",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 4262,
		Latitude:  -29.0471992493,
		Longitude: 29.9106006622,
		Timezone:  "Africa/Johannesburg",
	},
	"FAED": {
		ICAO:      "FAED",
		Name:      "Edenburg Airport",
		City:      "Edenburg",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 4509,
		Latitude:  -29.75,
		Longitude: 25.8999996185,
		Timezone:  "Africa/Johannesburg",
	},
	"FAEG": {
		ICAO:      "FAEG",
		Name:      "Egnep Airport",
		City:      "Egnep",
		State:     "Limpopo",
		Country:   "ZA",
		Elevation: 2299,
		Latitude:  -24.5669994354,
		Longitude: 30.3169994354,
		Timezone:  "Africa/Johannesburg",
	},
	"FAEL": {
		ICAO:      "FAEL",
		IATA:      "ELS",
		Name:      "Ben Schoeman Airport",
		City:      "East London",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 435,
		Latitude:  -33.0355987549,
		Longitude: 27.8258991241,
		Timezone:  "Africa/Johannesburg",
	},
	"FAEM": {
		ICAO:      "FAEM",
		IATA:      "EMG",
		Name:      "Empangeni Airport",
		City:      "Empangeni",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 251,
		Latitude:  -28.7199993134,
		Longitude: 31.8899993896,
		Timezone:  "Africa/Johannesburg",
	},
	"FAEO": {
		ICAO:      "FAEO",
		Name:      "Ermelo Airport",
		City:      "Ermelo",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 5700,
		Latitude:  -26.4955997467,
		Longitude: 29.9797992706,
		Timezone:  "Africa/Johannesburg",
	},
	"FAER": {
		ICAO:      "FAER",
		IATA:      "ELL",
		Name:      "Ellisras Matimba Airport",
		City:      "Ellisras",
		State:     "Limpopo",
		Country:   "ZA",
		Elevation: 2799,
		Latitude:  -23.7266998291,
		Longitude: 27.6882991791,
		Timezone:  "Africa/Johannesburg",
	},
	"FAES": {
		ICAO:      "FAES",
		Name:      "Eshowe Airport",
		City:      "Eshowe",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 1601,
		Latitude:  -28.8801994324,
		Longitude: 31.4549007416,
		Timezone:  "Africa/Johannesburg",
	},
	"FAET": {
		ICAO:      "FAET",
		Name:      "Elliot Airport",
		City:      "Elliot",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 5028,
		Latitude:  -31.3069000244,
		Longitude: 27.8495006561,
		Timezone:  "Africa/Johannesburg",
	},
	"FAFB": {
		ICAO:      "FAFB",
		IATA:      "FCB",
		Name:      "Ficksburg Sentraoes Airport",
		City:      "Ficksburg",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 5315,
		Latitude:  -28.8230991364,
		Longitude: 27.9088993073,
		Timezone:  "Africa/Johannesburg",
	},
	"FAFF": {
		ICAO:      "FAFF",
		Name:      "Frankfort Airport",
		City:      "Frankfort",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 5176,
		Latitude:  -27.2854003906,
		Longitude: 28.5153007507,
		Timezone:  "Africa/Johannesburg",
	},
	"FAFG": {
		ICAO:      "FAFG",
		Name:      "Flamingo Vlei Airport",
		City:      "Flamingo Vlei",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 3248,
		Latitude:  -30.4909992218,
		Longitude: 21.1217002869,
		Timezone:  "Africa/Johannesburg",
	},
	"FAFK": {
		ICAO:      "FAFK",
		Name:      "Fisantekraal Airport",
		City:      "Fisantekraal",
		State:     "Western-Cape",
		Country:   "ZA",
		Elevation: 410,
		Latitude:  -33.7700004578,
		Longitude: 18.7399997711,
		Timezone:  "Africa/Johannesburg",
	},
	"FAFO": {
		ICAO:      "FAFO",
		Name:      "Fort Beaufort Airport",
		City:      "Fort Beaufort",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 1581,
		Latitude:  -32.7905006409,
		Longitude: 26.5776996613,
		Timezone:  "Africa/Johannesburg",
	},
	"FAFR": {
		ICAO:      "FAFR",
		Name:      "Fraserburg Airport",
		City:      "Fraseburg",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 4180,
		Latitude:  -31.9339008331,
		Longitude: 21.5025005341,
		Timezone:  "Africa/Johannesburg",
	},
	"FAFU": {
		ICAO:      "FAFU",
		Name:      "Fraaiuitzicht Airport",
		City:      "Fraaiuitzicht",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 4841,
		Latitude:  -27.3169994354,
		Longitude: 27.25,
		Timezone:  "Africa/Johannesburg",
	},
	"FAFW": {
		ICAO:      "FAFW",
		Name:      "Freeway Airport",
		City:      "Kromdraai",
		State:     "Gauteng",
		Country:   "ZA",
		Elevation: 3819,
		Latitude:  -25.4771995544,
		Longitude: 28.2900009155,
		Timezone:  "Africa/Johannesburg",
	},
	"FAGA": {
		ICAO:      "FAGA",
		Name:      "Grange Airport",
		City:      "Umzimkulu",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 2600,
		Latitude:  -30.2670001984,
		Longitude: 29.966999054,
		Timezone:  "Africa/Johannesburg",
	},
	"FAGC": {
		ICAO:      "FAGC",
		IATA:      "GCJ",
		Name:      "Grand Central Airport",
		City:      "Midrand",
		State:     "Gauteng",
		Country:   "ZA",
		Elevation: 5325,
		Latitude:  -25.9862995148,
		Longitude: 28.1401004791,
		Timezone:  "Africa/Johannesburg",
	},
	"FAGG": {
		ICAO:      "FAGG",
		IATA:      "GRJ",
		Name:      "George Airport",
		City:      "George",
		State:     "Western-Cape",
		Country:   "ZA",
		Elevation: 648,
		Latitude:  -34.0055999756,
		Longitude: 22.378900528,
		Timezone:  "Africa/Johannesburg",
	},
	"FAGH": {
		ICAO:      "FAGH",
		Name:      "Glen Grey Airport",
		City:      "Emmaus",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 3500,
		Latitude:  -28.8333339691,
		Longitude: 29.4666671753,
		Timezone:  "Africa/Johannesburg",
	},
	"FAGI": {
		ICAO:      "FAGI",
		IATA:      "GIY",
		Name:      "Giyani Airport",
		City:      "Giyani",
		State:     "Limpopo",
		Country:   "ZA",
		Elevation: 1584,
		Latitude:  -23.2833003998,
		Longitude: 30.6499996185,
		Timezone:  "Africa/Johannesburg",
	},
	"FAGJ": {
		ICAO:      "FAGJ",
		Name:      "Gifvlei Airport",
		City:      "Gifvlei",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 3199,
		Latitude:  -30.3833007813,
		Longitude: 20.0333003998,
		Timezone:  "Africa/Johannesburg",
	},
	"FAGL": {
		ICAO:      "FAGL",
		Name:      "Groblersdal Kob Airport",
		City:      "Groblersdal",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 3051,
		Latitude:  -25.1868991852,
		Longitude: 29.4032993317,
		Timezone:  "Africa/Johannesburg",
	},
	"FAGM": {
		ICAO:      "FAGM",
		IATA:      "QRA",
		Name:      "Rand Airport",
		City:      "Johannesburg",
		State:     "Gauteng",
		Country:   "ZA",
		Elevation: 5483,
		Latitude:  -26.2425003052,
		Longitude: 28.1511993408,
		Timezone:  "Africa/Johannesburg",
	},
	"FAGO": {
		ICAO:      "FAGO",
		Name:      "Gowrie Airport",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 1200,
		Latitude:  -24.7371653036,
		Longitude: 31.5602874756,
		Timezone:  "Africa/Johannesburg",
	},
	"FAGR": {
		ICAO:      "FAGR",
		Name:      "Graaff Reinet Airport",
		City:      "Graaff-Reinet",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 2604,
		Latitude:  -32.1935997009,
		Longitude: 24.5414009094,
		Timezone:  "Africa/Johannesburg",
	},
	"FAGS": {
		ICAO:      "FAGS",
		Name:      "Giants Castle Airport",
		City:      "Giant'S Castle Game Reserve",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 6086,
		Latitude:  -29.2696990967,
		Longitude: 29.5660991669,
		Timezone:  "Africa/Johannesburg",
	},
	"FAGT": {
		ICAO:      "FAGT",
		Name:      "Grahamstown Airport",
		City:      "Grahamstown",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 2135,
		Latitude:  -33.2846984863,
		Longitude: 26.4981002808,
		Timezone:  "Africa/Johannesburg",
	},
	"FAGV": {
		ICAO:      "FAGV",
		Name:      "Gravelotte Airport",
		City:      "Gravelotte",
		State:     "Limpopo",
		Country:   "ZA",
		Elevation: 1650,
		Latitude:  -23.9167003632,
		Longitude: 30.7000007629,
		Timezone:  "Africa/Johannesburg",
	},
	"FAGW": {
		ICAO:      "FAGW",
		Name:      "Magwa Airport",
		City:      "Magwa",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 1823,
		Latitude:  -31.3939418793,
		Longitude: 29.6934700012,
		Timezone:  "Africa/Johannesburg",
	},
	"FAGY": {
		ICAO:      "FAGY",
		Name:      "Greytown Airport",
		City:      "Greytown",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 3531,
		Latitude:  -29.1219997406,
		Longitude: 30.5867004395,
		Timezone:  "Africa/Johannesburg",
	},
	"FAHA": {
		ICAO:      "FAHA",
		Name:      "Harmony Airport",
		City:      "Virginia",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 4399,
		Latitude:  -28.0786991119,
		Longitude: 26.8612003326,
		Timezone:  "Africa/Johannesburg",
	},
	"FAHB": {
		ICAO:      "FAHB",
		Name:      "Hartebeespoortdam Airport",
		City:      "Hartebeespoort",
		State:     "Limpopo",
		Country:   "ZA",
		Elevation: 3740,
		Latitude:  -24.5023994446,
		Longitude: 28.0790996552,
		Timezone:  "Africa/Johannesburg",
	},
	"FAHC": {
		ICAO:      "FAHC",
		Name:      "Howick Airport",
		City:      "Howick",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 3619,
		Latitude:  -29.5527992249,
		Longitude: 30.2110996246,
		Timezone:  "Africa/Johannesburg",
	},
	"FAHD": {
		ICAO:      "FAHD",
		Name:      "Humansdorp Airport",
		City:      "Humansdorp",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 345,
		Latitude:  -34.0365982056,
		Longitude: 24.7889003754,
		Timezone:  "Africa/Johannesburg",
	},
	"FAHE": {
		ICAO:      "FAHE",
		Name:      "Pullenshope Hendrina Airport",
		City:      "Hendrina",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 5171,
		Latitude:  -25.9778003693,
		Longitude: 29.6189002991,
		Timezone:  "Africa/Johannesburg",
	},
	"FAHF": {
		ICAO:      "FAHF",
		Name:      "Henrys Flats Airport",
		City:      "Henry'S Flats",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 1709,
		Latitude:  -33.0461006165,
		Longitude: 25.7168998718,
		Timezone:  "Africa/Johannesburg",
	},
	"FAHG": {
		ICAO:      "FAHG",
		Name:      "Heidelburg Airport",
		City:      "Heidelburg",
		State:     "Gauteng",
		Country:   "ZA",
		Elevation: 5089,
		Latitude:  -26.5100002289,
		Longitude: 28.3899993896,
		Timezone:  "Africa/Johannesburg",
	},
	"FAHH": {
		ICAO:      "FAHH",
		Name:      "Hibberdene Airport",
		City:      "Hibberdene",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 89,
		Latitude:  -30.6299991608,
		Longitude: 30.5200004578,
		Timezone:  "Africa/Johannesburg",
	},
	"FAHI": {
		ICAO:      "FAHI",
		Name:      "Halfweg Airport",
		City:      "Halfweg",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 3159,
		Latitude:  -30.013299942,
		Longitude: 20.1392002106,
		Timezone:  "Africa/Johannesburg",
	},
	"FAHJ": {
		ICAO:      "FAHJ",
		Name:      "Harding Airport",
		City:      "Harding",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 2890,
		Latitude:  -30.5666999817,
		Longitude: 29.8833007813,
		Timezone:  "Africa/Johannesburg",
	},
	"FAHK": {
		ICAO:      "FAHK",
		Name:      "Haakdoornboom Airport",
		City:      "Haakdoornboom",
		State:     "Gauteng",
		Country:   "ZA",
		Elevation: 4055,
		Latitude:  -25.5872001648,
		Longitude: 28.1158008575,
		Timezone:  "Africa/Johannesburg",
	},
	"FAHL": {
		ICAO:      "FAHL",
		IATA:      "HLW",
		Name:      "Hluhluwe Airport",
		City:      "Hluhluwe",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 249,
		Latitude:  -28.0166049887,
		Longitude: 32.2751712799,
		Timezone:  "Africa/Johannesburg",
	},
	"FAHO": {
		ICAO:      "FAHO",
		Name:      "Heilbron Airport",
		City:      "Heilbron",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 5200,
		Latitude:  -27.2786006927,
		Longitude: 27.9955997467,
		Timezone:  "Africa/Johannesburg",
	},
	"FAHP": {
		ICAO:      "FAHP",
		Name:      "Hoopstad Airport",
		City:      "Hoopstad",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 4131,
		Latitude:  -27.8216991425,
		Longitude: 25.9169006348,
		Timezone:  "Africa/Johannesburg",
	},
	"FAHR": {
		ICAO:      "FAHR",
		IATA:      "HRS",
		Name:      "Harrismith Airport",
		City:      "Harrismith",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 5585,
		Latitude:  -28.2350997925,
		Longitude: 29.1061992645,
		Timezone:  "Africa/Johannesburg",
	},
	"FAHS": {
		ICAO:      "FAHS",
		IATA:      "HDS",
		Name:      "Hoedspruit Air Force Base Airport",
		City:      "Hoedspruit",
		State:     "Limpopo",
		Country:   "ZA",
		Elevation: 1743,
		Latitude:  -24.3686008453,
		Longitude: 31.0487003326,
		Timezone:  "Africa/Johannesburg",
	},
	"FAHT": {
		ICAO:      "FAHT",
		Name:      "Hoedspruit Airport",
		City:      "Hoedspruit",
		State:     "Limpopo",
		Country:   "ZA",
		Elevation: 1801,
		Latitude:  -24.3516998291,
		Longitude: 30.9493999481,
		Timezone:  "Africa/Johannesburg",
	},
	"FAHU": {
		ICAO:      "FAHU",
		Name:      "H M S Bastard Memorial Airport",
		City:      "H.M.S.Bastard Memorial",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 4701,
		Latitude:  -30.5424995422,
		Longitude: 29.4990005493,
		Timezone:  "Africa/Johannesburg",
	},
	"FAHV": {
		ICAO:      "FAHV",
		Name:      "Gariep Dam Airport",
		City:      "Hendrik Verwoerd dam",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 4176,
		Latitude:  -30.5622005463,
		Longitude: 25.5282993317,
		Timezone:  "Africa/Johannesburg",
	},
	"FAIA": {
		ICAO:      "FAIA",
		Name:      "Itala Airport",
		City:      "Itala",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 2665,
		Latitude:  -27.486700058,
		Longitude: 31.1711006165,
		Timezone:  "Africa/Johannesburg",
	},
	"FAID": {
		ICAO:      "FAID",
		Name:      "Idutywa Airport",
		City:      "Idutywa",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 2530,
		Latitude:  -32.0999984741,
		Longitude: 28.3166999817,
		Timezone:  "Africa/Johannesburg",
	},
	"FAIO": {
		ICAO:      "FAIO",
		Name:      "Odi Airport",
		City:      "Lerulaneng",
		State:     "North-West",
		Country:   "ZA",
		Elevation: 3862,
		Latitude:  -25.5499992371,
		Longitude: 27.9330005646,
		Timezone:  "Africa/Johannesburg",
	},
	"FAIS": {
		ICAO:      "FAIS",
		Name:      "Isithebe Airport",
		City:      "Nyoni",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 313,
		Latitude:  -29.105556488,
		Longitude: 31.4074993134,
		Timezone:  "Africa/Johannesburg",
	},
	"FAIV": {
		ICAO:      "FAIV",
		Name:      "Ingwavuma Airport",
		City:      "Ingwavuma",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 2100,
		Latitude:  -27.117099762,
		Longitude: 32.0087013245,
		Timezone:  "Africa/Johannesburg",
	},
	"FAIW": {
		ICAO:      "FAIW",
		Name:      "Indwe Airport",
		City:      "Indwe",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 4167,
		Latitude:  -31.4833335876,
		Longitude: 27.3500003815,
		Timezone:  "Africa/Johannesburg",
	},
	"FAJF": {
		ICAO:      "FAJF",
		Name:      "Jagersfontain Airport",
		City:      "Jagersfontain",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 4714,
		Latitude:  -29.7625007629,
		Longitude: 25.4330005646,
		Timezone:  "Africa/Johannesburg",
	},
	"FAJP": {
		ICAO:      "FAJP",
		Name:      "Joubertina Airport",
		City:      "Joubertina",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 2007,
		Latitude:  -33.831199646,
		Longitude: 23.8300991058,
		Timezone:  "Africa/Johannesburg",
	},
	"FAJV": {
		ICAO:      "FAJV",
		Name:      "Jansenville Airport",
		City:      "Jansenville",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 1430,
		Latitude:  -32.9390983582,
		Longitude: 24.6707000732,
		Timezone:  "Africa/Johannesburg",
	},
	"FAKA": {
		ICAO:      "FAKA",
		Name:      "Kagga Kamma Airport",
		City:      "Kagga Kamma Private Game Reserve",
		State:     "Western-Cape",
		Country:   "ZA",
		Elevation: 3494,
		Latitude:  -32.7469940186,
		Longitude: 19.5620994568,
		Timezone:  "Africa/Johannesburg",
	},
	"FAKB": {
		ICAO:      "FAKB",
		Name:      "Kosibaai Airport",
		City:      "Kosi Bay",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 140,
		Latitude:  -27.0333328247,
		Longitude: 32.75,
		Timezone:  "Africa/Johannesburg",
	},
	"FAKD": {
		ICAO:      "FAKD",
		IATA:      "KXE",
		Name:      "P C Pelser Airport",
		City:      "Klerksdorp",
		State:     "North-West",
		Country:   "ZA",
		Elevation: 4444,
		Latitude:  -26.871099472,
		Longitude: 26.718000412,
		Timezone:  "Africa/Johannesburg",
	},
	"FAKE": {
		ICAO:      "FAKE",
		Name:      "Keimouth Airport",
		City:      "Keimouth",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 222,
		Latitude:  -32.7000007629,
		Longitude: 28.3500003815,
		Timezone:  "Africa/Johannesburg",
	},
	"FAKF": {
		ICAO:      "FAKF",
		Name:      "Koffee Bay Airport",
		City:      "Koffee Bay",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 10,
		Latitude:  -31.9638996124,
		Longitude: 29.14909935,
		Timezone:  "Africa/Johannesburg",
	},
	"FAKG": {
		ICAO:      "FAKG",
		Name:      "Komati Power Station Airport",
		City:      "Komati Power Station",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 5289,
		Latitude:  -26.0932998657,
		Longitude: 29.4556007385,
		Timezone:  "Africa/Johannesburg",
	},
	"FAKH": {
		ICAO:      "FAKH",
		Name:      "Kenhardt Airport",
		City:      "Kenhardt",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 2723,
		Latitude:  -29.3267002106,
		Longitude: 21.1886005402,
		Timezone:  "Africa/Johannesburg",
	},
	"FAKI": {
		ICAO:      "FAKI",
		Name:      "Kobb Inn Airport",
		City:      "Kobb Inn",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 147,
		Latitude:  -32.433799,
		Longitude: 28.683467,
		Timezone:  "Africa/Johannesburg",
	},
	"FAKK": {
		ICAO:      "FAKK",
		Name:      "Kakamas Airport",
		City:      "Kakamas",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 2322,
		Latitude:  -28.7999992371,
		Longitude: 20.6329994202,
		Timezone:  "Africa/Johannesburg",
	},
	"FAKL": {
		ICAO:      "FAKL",
		Name:      "Kriel Airport",
		City:      "Kriel",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 5262,
		Latitude:  -26.2511005402,
		Longitude: 29.1947002411,
		Timezone:  "Africa/Johannesburg",
	},
	"FAKM": {
		ICAO:      "FAKM",
		IATA:      "KIM",
		Name:      "Kimberley Airport",
		City:      "Kimberley",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 3950,
		Latitude:  -28.8027992249,
		Longitude: 24.7651996613,
		Timezone:  "Africa/Johannesburg",
	},
	"FAKN": {
		ICAO:      "FAKN",
		IATA:      "MQP",
		Name:      "Kruger Mpumalanga International Airport",
		City:      "Mpumalanga",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 2829,
		Latitude:  -25.3831996918,
		Longitude: 31.1056003571,
		Timezone:  "Africa/Johannesburg",
	},
	"FAKO": {
		ICAO:      "FAKO",
		Name:      "Komga Airport",
		City:      "Komga",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 2178,
		Latitude:  -32.5928001404,
		Longitude: 27.8757991791,
		Timezone:  "Africa/Johannesburg",
	},
	"FAKP": {
		ICAO:      "FAKP",
		IATA:      "KOF",
		Name:      "Komatipoort Airport",
		City:      "Komatipoort",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 541,
		Latitude:  -25.4402999878,
		Longitude: 31.9300003052,
		Timezone:  "Africa/Johannesburg",
	},
	"FAKR": {
		ICAO:      "FAKR",
		Name:      "Krugersdorp Airport",
		City:      "Krugersdorp",
		State:     "Gauteng",
		Country:   "ZA",
		Elevation: 5499,
		Latitude:  -26.0809993744,
		Longitude: 27.7257003784,
		Timezone:  "Africa/Johannesburg",
	},
	"FAKS": {
		ICAO:      "FAKS",
		Name:      "Kroonstad Airport",
		City:      "Kroonstad",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 4700,
		Latitude:  -27.6606006622,
		Longitude: 27.3157997131,
		Timezone:  "Africa/Johannesburg",
	},
	"FAKT": {
		ICAO:      "FAKT",
		Name:      "Kitty Hawk Airport",
		City:      "Boschkop",
		State:     "Gauteng",
		Country:   "ZA",
		Elevation: 4619,
		Latitude:  -25.8600006104,
		Longitude: 28.4500007629,
		Timezone:  "Africa/Johannesburg",
	},
	"FAKU": {
		ICAO:      "FAKU",
		IATA:      "KMH",
		Name:      "Johan Pienaar Airport",
		City:      "Kuruman",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 4382,
		Latitude:  -27.4566993713,
		Longitude: 23.4113998413,
		Timezone:  "Africa/Johannesburg",
	},
	"FAKV": {
		ICAO:      "FAKV",
		Name:      "Koffyfontein Min Airport",
		City:      "Koffyfontein",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 3996,
		Latitude:  -29.4393997192,
		Longitude: 24.9885997772,
		Timezone:  "Africa/Johannesburg",
	},
	"FAKW": {
		ICAO:      "FAKW",
		Name:      "Kareedouw Airport",
		City:      "Kareedouw",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 1050,
		Latitude:  -33.9666671753,
		Longitude: 24.2999992371,
		Timezone:  "Africa/Johannesburg",
	},
	"FAKX": {
		ICAO:      "FAKX",
		Name:      "Kenton on Sea Airport",
		City:      "Kenton On Sea",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 172,
		Latitude:  -33.6693000793,
		Longitude: 26.6308994293,
		Timezone:  "Africa/Johannesburg",
	},
	"FAKZ": {
		ICAO:      "FAKZ",
		IATA:      "KLZ",
		Name:      "Kleinsee Airport",
		City:      "Kleinsee",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 270,
		Latitude:  -29.6884002686,
		Longitude: 17.0939998627,
		Timezone:  "Africa/Johannesburg",
	},
	"FALA": {
		ICAO:      "FALA",
		IATA:      "HLA",
		Name:      "Lanseria Airport",
		City:      "Johannesburg",
		State:     "Gauteng",
		Country:   "ZA",
		Elevation: 4517,
		Latitude:  -25.9384994507,
		Longitude: 27.9260997772,
		Timezone:  "Africa/Johannesburg",
	},
	"FALB": {
		ICAO:      "FALB",
		Name:      "Ladybrand Af Airport",
		City:      "Ladybrand",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 5180,
		Latitude:  -29.1811008453,
		Longitude: 27.4535999298,
		Timezone:  "Africa/Johannesburg",
	},
	"FALC": {
		ICAO:      "FALC",
		IATA:      "LMR",
		Name:      "Lime Acres Finsch Mine Airport",
		City:      "Lime Acres",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 4900,
		Latitude:  -28.3600997925,
		Longitude: 23.4391002655,
		Timezone:  "Africa/Johannesburg",
	},
	"FALD": {
		ICAO:      "FALD",
		IATA:      "LDZ",
		Name:      "Londolozi Airport",
		City:      "Londolozi",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 1300,
		Latitude:  -24.7502002716,
		Longitude: 31.4757003784,
		Timezone:  "Africa/Johannesburg",
	},
	"FALE": {
		ICAO:      "FALE",
		IATA:      "DUR",
		Name:      "King Shaka International Airport",
		City:      "Durban",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 295,
		Latitude:  -29.6144444444,
		Longitude: 31.1197222222,
		Timezone:  "Africa/Johannesburg",
	},
	"FALF": {
		ICAO:      "FALF",
		Name:      "Loeriesfontein Airport",
		City:      "Loeriesfontein",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 2997,
		Latitude:  -30.9064006805,
		Longitude: 19.4253005981,
		Timezone:  "Africa/Johannesburg",
	},
	"FALH": {
		ICAO:      "FALH",
		Name:      "Lohathla Military Airport",
		City:      "Lohathla",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 4500,
		Latitude:  -28.0368003845,
		Longitude: 23.0986995697,
		Timezone:  "Africa/Johannesburg",
	},
	"FALI": {
		ICAO:      "FALI",
		Name:      "Lichtenburg Airport",
		City:      "Lichtenburg",
		State:     "North-West",
		Country:   "ZA",
		Elevation: 4875,
		Latitude:  -26.175699234,
		Longitude: 26.1846008301,
		Timezone:  "Africa/Johannesburg",
	},
	"FALK": {
		ICAO:      "FALK",
		IATA:      "LUJ",
		Name:      "Lusikisiki Airport",
		City:      "Lusikisiki",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 1831,
		Latitude:  -31.3666992188,
		Longitude: 29.5832996368,
		Timezone:  "Africa/Johannesburg",
	},
	"FALL": {
		ICAO:      "FALL",
		Name:      "Lydenburg Airport",
		City:      "Lydenburg",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 4882,
		Latitude:  -25.1033000946,
		Longitude: 30.4141998291,
		Timezone:  "Africa/Johannesburg",
	},
	"FALM": {
		ICAO:      "FALM",
		Name:      "Makhado Air Force Base Airport",
		City:      "Louis Trichardt Southwest",
		State:     "Limpopo",
		Country:   "ZA",
		Elevation: 3069,
		Latitude:  -23.1599006653,
		Longitude: 29.6965007782,
		Timezone:  "Africa/Johannesburg",
	},
	"FALO": {
		ICAO:      "FALO",
		IATA:      "LCD",
		Name:      "Louis Trichardt Airport",
		City:      "Louis Trichardt",
		State:     "Limpopo",
		Country:   "ZA",
		Elevation: 3025,
		Latitude:  -23.0618991852,
		Longitude: 29.8647003174,
		Timezone:  "Africa/Johannesburg",
	},
	"FALQ": {
		ICAO:      "FALQ",
		Name:      "El Mirador Airport",
		City:      "Ardmore",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 3897,
		Latitude:  -28.9861106873,
		Longitude: 29.4797229767,
		Timezone:  "Africa/Johannesburg",
	},
	"FALR": {
		ICAO:      "FALR",
		Name:      "Steytlerville Airport",
		City:      "Steytlerville",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 1450,
		Latitude:  -33.3370018005,
		Longitude: 24.3213996887,
		Timezone:  "Africa/Johannesburg",
	},
	"FALS": {
		ICAO:      "FALS",
		Name:      "Somersveld Airport",
		City:      "Somersveld",
		State:     "Western-Cape",
		Country:   "ZA",
		Elevation: 196,
		Latitude:  -33.2462272644,
		Longitude: 18.4799976349,
		Timezone:  "Africa/Johannesburg",
	},
	"FALW": {
		ICAO:      "FALW",
		IATA:      "SDB",
		Name:      "Langebaanweg Airport",
		City:      "Langebaanweg",
		State:     "Western-Cape",
		Country:   "ZA",
		Elevation: 108,
		Latitude:  -32.9688987732,
		Longitude: 18.1602993011,
		Timezone:  "Africa/Johannesburg",
	},
	"FALY": {
		ICAO:      "FALY",
		IATA:      "LAY",
		Name:      "Ladysmith Airport",
		City:      "Ladysmith",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 3548,
		Latitude:  -28.5816993713,
		Longitude: 29.7497005463,
		Timezone:  "Africa/Johannesburg",
	},
	"FAMA": {
		ICAO:      "FAMA",
		Name:      "Matatiele Airport",
		City:      "Matatiele",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 4762,
		Latitude:  -30.3222007751,
		Longitude: 28.7942008972,
		Timezone:  "Africa/Johannesburg",
	},
	"FAMB": {
		ICAO:      "FAMB",
		Name:      "Middelburg Airport",
		City:      "Middelburg",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 4886,
		Latitude:  -25.6847991943,
		Longitude: 29.4402008057,
		Timezone:  "Africa/Johannesburg",
	},
	"FAMC": {
		ICAO:      "FAMC",
		Name:      "Middelburg 2 Airport",
		City:      "Cape Town",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 4022,
		Latitude:  -31.5471992493,
		Longitude: 25.0293998718,
		Timezone:  "Africa/Johannesburg",
	},
	"FAMD": {
		ICAO:      "FAMD",
		IATA:      "AAM",
		Name:      "Malamala Airport",
		City:      "Malamala",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 1124,
		Latitude:  -24.8180999756,
		Longitude: 31.5445995331,
		Timezone:  "Africa/Johannesburg",
	},
	"FAMF": {
		ICAO:      "FAMF",
		Name:      "Malabar Airport",
		City:      "Malabar",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 3348,
		Latitude:  -29.0118999481,
		Longitude: 23.8474006653,
		Timezone:  "Africa/Johannesburg",
	},
	"FAMG": {
		ICAO:      "FAMG",
		IATA:      "MGH",
		Name:      "Margate Airport",
		City:      "Margate",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 495,
		Latitude:  -30.8574008942,
		Longitude: 30.343000412,
		Timezone:  "Africa/Johannesburg",
	},
	"FAMH": {
		ICAO:      "FAMH",
		IATA:      "MEZ",
		Name:      "Musina(Messina) Airport",
		City:      "Musina",
		State:     "Limpopo",
		Country:   "ZA",
		Elevation: 1904,
		Latitude:  -22.3560009003,
		Longitude: 29.9862003326,
		Timezone:  "Africa/Johannesburg",
	},
	"FAMI": {
		ICAO:      "FAMI",
		Name:      "Marble Hall Airport",
		City:      "Marble Hall",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 2980,
		Latitude:  -24.9890995026,
		Longitude: 29.2831001282,
		Timezone:  "Africa/Johannesburg",
	},
	"FAMJ": {
		ICAO:      "FAMJ",
		Name:      "Majuba Power Station Airport",
		City:      "Amerspoort",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 5600,
		Latitude:  -27.0792999268,
		Longitude: 29.7784996033,
		Timezone:  "Africa/Johannesburg",
	},
	"FAMK": {
		ICAO:      "FAMK",
		Name:      "Mafeking Airport",
		City:      "Mafeking",
		State:     "North-West",
		Country:   "ZA",
		Elevation: 4201,
		Latitude:  -25.8166675568,
		Longitude: 25.6166667938,
		Timezone:  "Africa/Johannesburg",
	},
	"FAML": {
		ICAO:      "FAML",
		Name:      "Manyani Game Lodge Airport",
		City:      "Mafeking",
		State:     "North-West",
		Country:   "ZA",
		Elevation: 4383,
		Latitude:  -25.8166999817,
		Longitude: 25.7166996002,
		Timezone:  "Africa/Johannesburg",
	},
	"FAMM": {
		ICAO:      "FAMM",
		IATA:      "MBD",
		Name:      "Mmabatho International Airport",
		City:      "Mafeking",
		State:     "North-West",
		Country:   "ZA",
		Elevation: 4181,
		Latitude:  -25.7984008789,
		Longitude: 25.5480003357,
		Timezone:  "Africa/Johannesburg",
	},
	"FAMN": {
		ICAO:      "FAMN",
		IATA:      "LLE",
		Name:      "Riverside Airport",
		City:      "Malelane",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 1024,
		Latitude:  -25.4300003052,
		Longitude: 31.5767002106,
		Timezone:  "Africa/Johannesburg",
	},
	"FAMO": {
		ICAO:      "FAMO",
		IATA:      "MZY",
		Name:      "Mossel Bay Airport",
		City:      "Mossel Bay",
		State:     "Western-Cape",
		Country:   "ZA",
		Elevation: 531,
		Latitude:  -34.1582984924,
		Longitude: 22.058599472,
		Timezone:  "Africa/Johannesburg",
	},
	"FAMP": {
		ICAO:      "FAMP",
		Name:      "Madimbo Airport",
		City:      "Matshakatini",
		State:     "Limpopo",
		Country:   "ZA",
		Elevation: 2116,
		Latitude:  -22.3747005463,
		Longitude: 30.8815994263,
		Timezone:  "Africa/Johannesburg",
	},
	"FAMQ": {
		ICAO:      "FAMQ",
		Name:      "Maclear Airport",
		City:      "Maclear",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 4106,
		Latitude:  -31.0832996368,
		Longitude: 28.3763999939,
		Timezone:  "Africa/Johannesburg",
	},
	"FAMS": {
		ICAO:      "FAMS",
		IATA:      "MEZ",
		Name:      "Morningside Farm Airport",
		City:      "Silveroaks",
		State:     "North-West",
		Country:   "ZA",
		Elevation: 4251,
		Latitude:  -25.7045001984,
		Longitude: 26.9090003967,
		Timezone:  "Africa/Johannesburg",
	},
	"FAMT": {
		ICAO:      "FAMT",
		Name:      "Molteno Airport",
		City:      "Molteno",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 5254,
		Latitude:  -31.3896007538,
		Longitude: 26.348400116,
		Timezone:  "Africa/Johannesburg",
	},
	"FAMU": {
		ICAO:      "FAMU",
		IATA:      "MZQ",
		Name:      "Mkuze Airport",
		City:      "Mkuze",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 400,
		Latitude:  -27.6261005402,
		Longitude: 32.0443000793,
		Timezone:  "Africa/Johannesburg",
	},
	"FAMV": {
		ICAO:      "FAMV",
		Name:      "Montrose Airport",
		City:      "Montrose",
		State:     "Limpopo",
		Country:   "ZA",
		Elevation: 2690,
		Latitude:  -24.625,
		Longitude: 30.1861114502,
		Timezone:  "Africa/Johannesburg",
	},
	"FAMX": {
		ICAO:      "FAMX",
		Name:      "Mbazwana Airport",
		City:      "Mbazwana",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 195,
		Latitude:  -27.4813995361,
		Longitude: 32.5938987732,
		Timezone:  "Africa/Johannesburg",
	},
	"FAMY": {
		ICAO:      "FAMY",
		Name:      "Malmesbury Airport",
		City:      "Mammesbury",
		State:     "Western-Cape",
		Country:   "ZA",
		Elevation: 971,
		Latitude:  -33.4618988037,
		Longitude: 18.700799942,
		Timezone:  "Africa/Johannesburg",
	},
	"FAMZ": {
		ICAO:      "FAMZ",
		Name:      "Msauli Airport",
		City:      "Msauli",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 2548,
		Latitude:  -26.0499992371,
		Longitude: 31.0170001984,
		Timezone:  "Africa/Johannesburg",
	},
	"FANA": {
		ICAO:      "FANA",
		Name:      "Nongoma Airport",
		City:      "Nongoma",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 2596,
		Latitude:  -27.9139995575,
		Longitude: 31.6562004089,
		Timezone:  "Africa/Johannesburg",
	},
	"FANC": {
		ICAO:      "FANC",
		IATA:      "NCS",
		Name:      "Newcastle Airport",
		City:      "Newcastle",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 4074,
		Latitude:  -27.7705993652,
		Longitude: 29.9769001007,
		Timezone:  "Africa/Johannesburg",
	},
	"FANG": {
		ICAO:      "FANG",
		IATA:      "NGL",
		Name:      "Ngala Airport",
		City:      "Ngala",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 3357,
		Latitude:  -24.3878002167,
		Longitude: 31.325799942,
		Timezone:  "Africa/Johannesburg",
	},
	"FANH": {
		ICAO:      "FANH",
		Name:      "New Hanover Airport",
		City:      "New Hanover",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 2490,
		Latitude:  -29.3560009003,
		Longitude: 30.5184001923,
		Timezone:  "Africa/Johannesburg",
	},
	"FANL": {
		ICAO:      "FANL",
		Name:      "New Largo Airport",
		City:      "Witbank",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 5038,
		Latitude:  -25.9785003662,
		Longitude: 28.9843997955,
		Timezone:  "Africa/Johannesburg",
	},
	"FANS": {
		ICAO:      "FANS",
		IATA:      "NLP",
		Name:      "Nelspruit Airport",
		City:      "Nelspruit",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 2875,
		Latitude:  -25.5,
		Longitude: 30.9137992859,
		Timezone:  "Africa/Johannesburg",
	},
	"FANV": {
		ICAO:      "FANV",
		Name:      "Nieuwoudtville Airfield",
		City:      "Nieuwoudtville",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 1890,
		Latitude:  -31.3500003815,
		Longitude: 19.1166667938,
		Timezone:  "Africa/Johannesburg",
	},
	"FANY": {
		ICAO:      "FANY",
		Name:      "Nylstroom Airfield",
		City:      "Modimolle",
		State:     "Limpopo",
		Country:   "ZA",
		Elevation: 3900,
		Latitude:  -24.6861000061,
		Longitude: 28.4349002838,
		Timezone:  "Africa/Johannesburg",
	},
	"FAOB": {
		ICAO:      "FAOB",
		IATA:      "OVG",
		Name:      "Overberg Airport",
		City:      "Overberg",
		State:     "Western-Cape",
		Country:   "ZA",
		Elevation: 52,
		Latitude:  -34.554901123,
		Longitude: 20.2506999969,
		Timezone:  "Africa/Johannesburg",
	},
	"FAOD": {
		ICAO:      "FAOD",
		Name:      "Odendaalsrus Airport",
		City:      "Odendaalsrus",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 1340,
		Latitude:  -27.8710002899,
		Longitude: 26.6933994293,
		Timezone:  "Africa/Johannesburg",
	},
	"FAOF": {
		ICAO:      "FAOF",
		Name:      "Jack Duvenhage Airport",
		City:      "Olifantshoek",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 4216,
		Latitude:  -27.9617004395,
		Longitude: 22.7166004181,
		Timezone:  "Africa/Johannesburg",
	},
	"FAOH": {
		ICAO:      "FAOH",
		IATA:      "OUH",
		Name:      "Oudtshoorn Airport",
		City:      "Oudtshoorn",
		State:     "Western-Cape",
		Country:   "ZA",
		Elevation: 1063,
		Latitude:  -33.6069984436,
		Longitude: 22.188999176,
		Timezone:  "Africa/Johannesburg",
	},
	"FAOI": {
		ICAO:      "FAOI",
		Name:      "Orient Glider Airport",
		City:      "Orient",
		State:     "Gauteng",
		Country:   "ZA",
		Elevation: 5102,
		Latitude:  -26.0394001007,
		Longitude: 27.5956001282,
		Timezone:  "Africa/Johannesburg",
	},
	"FAOL": {
		ICAO:      "FAOL",
		Name:      "Othawa Airport",
		City:      "Otthawa",
		State:     "Limpopo",
		Country:   "ZA",
		Elevation: 1098,
		Latitude:  -24.7329998016,
		Longitude: 31.3999996185,
		Timezone:  "Africa/Johannesburg",
	},
	"FAON": {
		ICAO:      "FAON",
		Name:      "Ornate Lake St L Airport",
		City:      "Hluhluwe",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 59,
		Latitude:  -28.0449008942,
		Longitude: 32.4253997803,
		Timezone:  "Africa/Johannesburg",
	},
	"FAOR": {
		ICAO:      "FAOR",
		IATA:      "JNB",
		Name:      "O. R. Tambo International Airport",
		City:      "Johannesburg",
		State:     "Gauteng",
		Country:   "ZA",
		Elevation: 5558,
		Latitude:  -26.133333,
		Longitude: 28.25,
		Timezone:  "Africa/Johannesburg",
	},
	"FAOT": {
		ICAO:      "FAOT",
		Name:      "Ottosdal Airport",
		City:      "Ottosdal",
		State:     "North-West",
		Country:   "ZA",
		Elevation: 4860,
		Latitude:  -26.7982997894,
		Longitude: 26.0002994537,
		Timezone:  "Africa/Johannesburg",
	},
	"FAOY": {
		ICAO:      "FAOY",
		Name:      "Orkney Airport",
		City:      "Orkney",
		State:     "North-West",
		Country:   "ZA",
		Elevation: 4265,
		Latitude:  -26.9839000702,
		Longitude: 26.6513996124,
		Timezone:  "Africa/Johannesburg",
	},
	"FAPA": {
		ICAO:      "FAPA",
		IATA:      "AFD",
		Name:      "Port Alfred Airport",
		City:      "Port Alfred",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 307,
		Latitude:  -33.5541992188,
		Longitude: 26.8777008057,
		Timezone:  "Africa/Johannesburg",
	},
	"FAPC": {
		ICAO:      "FAPC",
		Name:      "Prince Albert Airport",
		City:      "Prince Albert",
		State:     "Western-Cape",
		Country:   "ZA",
		Elevation: 2001,
		Latitude:  -33.2024993896,
		Longitude: 22.0321998596,
		Timezone:  "Africa/Johannesburg",
	},
	"FAPD": {
		ICAO:      "FAPD",
		Name:      "Pofadder Airport",
		City:      "Pofadder",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 3241,
		Latitude:  -29.1417007446,
		Longitude: 19.4130992889,
		Timezone:  "Africa/Johannesburg",
	},
	"FAPE": {
		ICAO:      "FAPE",
		IATA:      "PLZ",
		Name:      "Port Elizabeth Airport",
		City:      "Port Elizabeth",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 226,
		Latitude:  -33.9849014282,
		Longitude: 25.6173000336,
		Timezone:  "Africa/Johannesburg",
	},
	"FAPF": {
		ICAO:      "FAPF",
		Name:      "Piet Retief Airport",
		City:      "Piet Retief",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 4423,
		Latitude:  -26.9969005585,
		Longitude: 30.8407993317,
		Timezone:  "Africa/Johannesburg",
	},
	"FAPG": {
		ICAO:      "FAPG",
		IATA:      "PBZ",
		Name:      "Plettenberg Bay Airport",
		City:      "Plettenberg Bay",
		State:     "Western-Cape",
		Country:   "ZA",
		Elevation: 465,
		Latitude:  -34.0903015137,
		Longitude: 23.3278007507,
		Timezone:  "Africa/Johannesburg",
	},
	"FAPH": {
		ICAO:      "FAPH",
		IATA:      "PHW",
		Name:      "Hendrik Van Eck Airport",
		City:      "Phalaborwa",
		State:     "Limpopo",
		Country:   "ZA",
		Elevation: 1432,
		Latitude:  -23.9372005463,
		Longitude: 31.1553993225,
		Timezone:  "Africa/Johannesburg",
	},
	"FAPI": {
		ICAO:      "FAPI",
		Name:      "Pietersburg Municipal Airport",
		City:      "Polokwane",
		State:     "Limpopo",
		Country:   "ZA",
		Elevation: 4354,
		Latitude:  -23.9260997772,
		Longitude: 29.4843997955,
		Timezone:  "Africa/Johannesburg",
	},
	"FAPJ": {
		ICAO:      "FAPJ",
		IATA:      "JOH",
		Name:      "Port St Johns Airport",
		City:      "Port St Johns",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 1227,
		Latitude:  -31.6058998108,
		Longitude: 29.5198001862,
		Timezone:  "Africa/Johannesburg",
	},
	"FAPK": {
		ICAO:      "FAPK",
		IATA:      "PRK",
		Name:      "Prieska Airport",
		City:      "Prieska",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 3100,
		Latitude:  -29.683599472,
		Longitude: 22.7705993652,
		Timezone:  "Africa/Johannesburg",
	},
	"FAPL": {
		ICAO:      "FAPL",
		Name:      "Pongola Airport",
		City:      "Pongola",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 942,
		Latitude:  -27.3621997833,
		Longitude: 31.6067008972,
		Timezone:  "Africa/Johannesburg",
	},
	"FAPM": {
		ICAO:      "FAPM",
		IATA:      "PZB",
		Name:      "Pietermaritzburg Airport",
		City:      "Pietermaritzburg",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 2423,
		Latitude:  -29.6490001678,
		Longitude: 30.3987007141,
		Timezone:  "Africa/Johannesburg",
	},
	"FAPN": {
		ICAO:      "FAPN",
		IATA:      "NTY",
		Name:      "Pilanesberg International Airport",
		City:      "Pilanesberg",
		State:     "North-West",
		Country:   "ZA",
		Elevation: 3412,
		Latitude:  -25.3337993622,
		Longitude: 27.1734008789,
		Timezone:  "Africa/Johannesburg",
	},
	"FAPO": {
		ICAO:      "FAPO",
		Name:      "Pilgrims Rest Airport",
		City:      "Mankolehlotlo",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 3802,
		Latitude:  -24.7847003937,
		Longitude: 30.7922000885,
		Timezone:  "Africa/Johannesburg",
	},
	"FAPP": {
		ICAO:      "FAPP",
		IATA:      "PTG",
		Name:      "Polokwane International Airport",
		City:      "Potgietersrus",
		State:     "Limpopo",
		Country:   "ZA",
		Elevation: 4076,
		Latitude:  -23.8453006744,
		Longitude: 29.4585990906,
		Timezone:  "Africa/Johannesburg",
	},
	"FAPQ": {
		ICAO:      "FAPQ",
		Name:      "Punda Maria(Malia) Airport",
		City:      "Kruger National Park",
		State:     "Limpopo",
		Country:   "ZA",
		Elevation: 1500,
		Latitude:  -22.77,
		Longitude: 31.0108,
		Timezone:  "Africa/Johannesburg",
	},
	"FAPS": {
		ICAO:      "FAPS",
		IATA:      "PCF",
		Name:      "Potchefstroom Airport",
		City:      "Potchefstroom",
		State:     "North-West",
		Country:   "ZA",
		Elevation: 4520,
		Latitude:  -26.670999527,
		Longitude: 27.0818996429,
		Timezone:  "Africa/Johannesburg",
	},
	"FAPT": {
		ICAO:      "FAPT",
		Name:      "Posmasburg Soil Airport",
		City:      "Postmasburg",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 4301,
		Latitude:  -28.3386001587,
		Longitude: 23.07970047,
		Timezone:  "Africa/Johannesburg",
	},
	"FAPU": {
		ICAO:      "FAPU",
		Name:      "Paarl East Airport",
		City:      "Paarl",
		State:     "Western-Cape",
		Country:   "ZA",
		Elevation: 596,
		Latitude:  -33.7107666667,
		Longitude: 19.0243888889,
		Timezone:  "Africa/Johannesburg",
	},
	"FAPV": {
		ICAO:      "FAPV",
		Name:      "Petrusville Airport",
		City:      "Petrusville",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 4116,
		Latitude:  -30.0818004608,
		Longitude: 24.679599762,
		Timezone:  "Africa/Johannesburg",
	},
	"FAPW": {
		ICAO:      "FAPW",
		Name:      "Pietersrus Airport",
		City:      "Pietersrus",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 5472,
		Latitude:  -26.6061000824,
		Longitude: 29.3292007446,
		Timezone:  "Africa/Johannesburg",
	},
	"FAPX": {
		ICAO:      "FAPX",
		Name:      "Paradise Beach Airport",
		City:      "Jeffreysbaal",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 15,
		Latitude:  -34.1027793884,
		Longitude: 24.8827781677,
		Timezone:  "Africa/Johannesburg",
	},
	"FAPY": {
		ICAO:      "FAPY",
		Name:      "Parys Airport",
		City:      "Parys",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 4740,
		Latitude:  -26.8892993927,
		Longitude: 27.5034008026,
		Timezone:  "Africa/Johannesburg",
	},
	"FAPZ": {
		ICAO:      "FAPZ",
		Name:      "Progress Airport",
		City:      "Sunlands",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 748,
		Latitude:  -33.9300003052,
		Longitude: 25.3700008392,
		Timezone:  "Africa/Johannesburg",
	},
	"FAQF": {
		ICAO:      "FAQF",
		Name:      "Pomfret Airport",
		City:      "Pomfret",
		State:     "North-West",
		Country:   "ZA",
		Elevation: 3817,
		Latitude:  -25.847700119,
		Longitude: 23.5373001099,
		Timezone:  "Africa/Johannesburg",
	},
	"FAQR": {
		ICAO:      "FAQR",
		Name:      "Potgietersrus Airport",
		City:      "Potgietersus",
		State:     "Limpopo",
		Country:   "ZA",
		Elevation: 3500,
		Latitude:  -24.2333335876,
		Longitude: 28.9833335876,
		Timezone:  "Africa/Johannesburg",
	},
	"FAQT": {
		ICAO:      "FAQT",
		IATA:      "UTW",
		Name:      "Queenstown Airport",
		City:      "Queenstown",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 3637,
		Latitude:  -31.9202003479,
		Longitude: 26.8822002411,
		Timezone:  "Africa/Johannesburg",
	},
	"FARA": {
		ICAO:      "FARA",
		Name:      "Petit Airport",
		City:      "Daveyton",
		State:     "Gauteng",
		Country:   "ZA",
		Elevation: 5460,
		Latitude:  -26.0816669464,
		Longitude: 28.3999996185,
		Timezone:  "Africa/Johannesburg",
	},
	"FARB": {
		ICAO:      "FARB",
		IATA:      "RCB",
		Name:      "Richards Bay Airport",
		City:      "Richards Bay",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 109,
		Latitude:  -28.7409992218,
		Longitude: 32.0920982361,
		Timezone:  "Africa/Johannesburg",
	},
	"FARD": {
		ICAO:      "FARD",
		Name:      "Riversdale Airport",
		City:      "Riversdale",
		State:     "Western-Cape",
		Country:   "ZA",
		Elevation: 591,
		Latitude:  -34.1116981506,
		Longitude: 21.2628002167,
		Timezone:  "Africa/Johannesburg",
	},
	"FARG": {
		ICAO:      "FARG",
		Name:      "Rustenburg Airport",
		City:      "Rustenburg",
		State:     "North-West",
		Country:   "ZA",
		Elevation: 3700,
		Latitude:  -25.6443004608,
		Longitude: 27.2710990906,
		Timezone:  "Africa/Johannesburg",
	},
	"FARI": {
		ICAO:      "FARI",
		IATA:      "RVO",
		Name:      "Reivilo Airport",
		City:      "Reivilo",
		State:     "North-West",
		Country:   "ZA",
		Elevation: 4715,
		Latitude:  -27.5471992493,
		Longitude: 24.1725006104,
		Timezone:  "Africa/Johannesburg",
	},
	"FARM": {
		ICAO:      "FARM",
		Name:      "Richmond Airport",
		City:      "Richmond",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 4731,
		Latitude:  -31.423500061,
		Longitude: 23.9782009125,
		Timezone:  "Africa/Johannesburg",
	},
	"FARO": {
		ICAO:      "FARO",
		Name:      "Rooiberg Airport",
		City:      "Rooiberg",
		State:     "Limpopo",
		Country:   "ZA",
		Elevation: 3811,
		Latitude:  -24.7666664124,
		Longitude: 27.75,
		Timezone:  "Africa/Johannesburg",
	},
	"FARS": {
		ICAO:      "FARS",
		IATA:      "ROD",
		Name:      "Robertson Airport",
		City:      "Robertson",
		State:     "Western-Cape",
		Country:   "ZA",
		Elevation: 640,
		Latitude:  -33.8121986389,
		Longitude: 19.9027996063,
		Timezone:  "Africa/Johannesburg",
	},
	"FARZ": {
		ICAO:      "FARZ",
		Name:      "Reitz Airport",
		City:      "Maasstroom",
		State:     "Limpopo",
		Country:   "ZA",
		Elevation: 5331,
		Latitude:  -22.7842006683,
		Longitude: 28.4302997589,
		Timezone:  "Africa/Johannesburg",
	},
	"FASA": {
		ICAO:      "FASA",
		Name:      "Sani Pass Airport",
		City:      "Sani Pass",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 5933,
		Latitude:  -29.6571998596,
		Longitude: 29.4477005005,
		Timezone:  "Africa/Johannesburg",
	},
	"FASB": {
		ICAO:      "FASB",
		IATA:      "SBU",
		Name:      "Springbok Airport",
		City:      "Springbok",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 2690,
		Latitude:  -29.6893005371,
		Longitude: 17.9395999908,
		Timezone:  "Africa/Johannesburg",
	},
	"FASC": {
		ICAO:      "FASC",
		IATA:      "ZEC",
		Name:      "Secunda Airport",
		City:      "Secunda",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 5250,
		Latitude:  -26.52409935,
		Longitude: 29.1700992584,
		Timezone:  "Africa/Johannesburg",
	},
	"FASD": {
		ICAO:      "FASD",
		Name:      "Saldanha /Vredenburg Airport",
		City:      "Saldanha-Vredenburg",
		State:     "Western-Cape",
		Country:   "ZA",
		Elevation: 50,
		Latitude:  -32.964099884,
		Longitude: 17.9692993164,
		Timezone:  "Africa/Johannesburg",
	},
	"FASE": {
		ICAO:      "FASE",
		IATA:      "GSS",
		Name:      "Sabi Sabi Airport",
		City:      "Belfast",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 1276,
		Latitude:  -24.9414997101,
		Longitude: 31.4445991516,
		Timezone:  "Africa/Johannesburg",
	},
	"FASG": {
		ICAO:      "FASG",
		Name:      "Schweizer Reneke Airport",
		City:      "Schweizer Reneke",
		State:     "North-West",
		Country:   "ZA",
		Elevation: 4373,
		Latitude:  -27.1630992889,
		Longitude: 25.2861003876,
		Timezone:  "Africa/Johannesburg",
	},
	"FASH": {
		ICAO:      "FASH",
		Name:      "Stellenbosch Airport",
		City:      "Stellenbosch",
		State:     "Western-Cape",
		Country:   "ZA",
		Elevation: 295,
		Latitude:  -33.9799995422,
		Longitude: 18.8199996948,
		Timezone:  "Africa/Johannesburg",
	},
	"FASI": {
		ICAO:      "FASI",
		Name:      "Springs Airfield",
		City:      "Springs",
		State:     "Gauteng",
		Country:   "ZA",
		Elevation: 5340,
		Latitude:  -26.2483997345,
		Longitude: 28.3974990845,
		Timezone:  "Africa/Johannesburg",
	},
	"FASJ": {
		ICAO:      "FASJ",
		Name:      "Saffier Airport",
		City:      "Saffier",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 4700,
		Latitude:  -26.7600002289,
		Longitude: 27.7600002289,
		Timezone:  "Africa/Johannesburg",
	},
	"FASK": {
		ICAO:      "FASK",
		Name:      "Swartkop Air Force Base",
		City:      "Pretoria",
		State:     "Gauteng",
		Country:   "ZA",
		Elevation: 4780,
		Latitude:  -25.8097000122,
		Longitude: 28.1646003723,
		Timezone:  "Africa/Johannesburg",
	},
	"FASL": {
		ICAO:      "FASL",
		Name:      "Sutherland Airport",
		City:      "Sutherland",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 5247,
		Latitude:  -32.4882011414,
		Longitude: 20.6970996857,
		Timezone:  "Africa/Johannesburg",
	},
	"FASM": {
		ICAO:      "FASM",
		Name:      "Siteka Airport",
		City:      "Siteka",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 4400,
		Latitude:  -29.3250007629,
		Longitude: 30.1492004395,
		Timezone:  "Africa/Johannesburg",
	},
	"FASN": {
		ICAO:      "FASN",
		Name:      "Senekal Airport",
		City:      "Senekal",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 4849,
		Latitude:  -28.3122005463,
		Longitude: 27.6460990906,
		Timezone:  "Africa/Johannesburg",
	},
	"FASR": {
		ICAO:      "FASR",
		Name:      "Standerton Airport",
		City:      "Standerton",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 5200,
		Latitude:  -26.9270000458,
		Longitude: 29.2099990845,
		Timezone:  "Africa/Johannesburg",
	},
	"FASS": {
		ICAO:      "FASS",
		IATA:      "SIS",
		Name:      "Sishen Airport",
		City:      "Sishen",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 3848,
		Latitude:  -27.6485996246,
		Longitude: 22.9993000031,
		Timezone:  "Africa/Johannesburg",
	},
	"FAST": {
		ICAO:      "FAST",
		Name:      "Somerset East Airport",
		City:      "Somerset East",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 2349,
		Latitude:  -32.7501983643,
		Longitude: 25.5949993134,
		Timezone:  "Africa/Johannesburg",
	},
	"FASU": {
		ICAO:      "FASU",
		Name:      "Sace Airport",
		City:      "Sace",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 5151,
		Latitude:  -25.9568996429,
		Longitude: 29.2117004395,
		Timezone:  "Africa/Johannesburg",
	},
	"FASW": {
		ICAO:      "FASW",
		Name:      "Slurry Airport",
		City:      "Slurry",
		State:     "North-West",
		Country:   "ZA",
		Elevation: 4692,
		Latitude:  -25.8155994415,
		Longitude: 25.8864002228,
		Timezone:  "Africa/Johannesburg",
	},
	"FASX": {
		ICAO:      "FASX",
		Name:      "Hendrik Swellengrebel Airport",
		City:      "Swellendam",
		State:     "Western-Cape",
		Country:   "ZA",
		Elevation: 407,
		Latitude:  -34.0481987,
		Longitude: 20.4745998383,
		Timezone:  "Africa/Johannesburg",
	},
	"FASY": {
		ICAO:      "FASY",
		Name:      "Syferfontein Airport",
		City:      "Hiltonia",
		State:     "Gauteng",
		Country:   "ZA",
		Elevation: 5420,
		Latitude:  -26.349199295,
		Longitude: 27.7793998718,
		Timezone:  "Africa/Johannesburg",
	},
	"FASZ": {
		ICAO:      "FASZ",
		IATA:      "SZK",
		Name:      "Skukuza Airport",
		City:      "Skukuza",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 1020,
		Latitude:  -24.960899353,
		Longitude: 31.5886993408,
		Timezone:  "Africa/Johannesburg",
	},
	"FATA": {
		ICAO:      "FATA",
		Name:      "Tedderfield Air Park",
		City:      "Tedderfield",
		State:     "Gauteng",
		Country:   "ZA",
		Elevation: 5198,
		Latitude:  -26.3519992828,
		Longitude: 27.9689998627,
		Timezone:  "Africa/Johannesburg",
	},
	"FATB": {
		ICAO:      "FATB",
		Name:      "Thorny Bush Game Lodge Airport",
		City:      "Hoedspruit",
		State:     "Limpopo",
		Country:   "ZA",
		Elevation: 1900,
		Latitude:  -24.4147224426,
		Longitude: 31.1650009155,
		Timezone:  "Africa/Johannesburg",
	},
	"FATD": {
		ICAO:      "FATD",
		IATA:      "TDT",
		Name:      "Tanda Tula Airport",
		City:      "Welverdiend",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 1555,
		Latitude:  -24.5335998535,
		Longitude: 31.2999992371,
		Timezone:  "Africa/Johannesburg",
	},
	"FATF": {
		ICAO:      "FATF",
		Name:      "Tommys Field Airport",
		City:      "Beeshoek",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 4360,
		Latitude:  -28.2600002289,
		Longitude: 22.9932003021,
		Timezone:  "Africa/Johannesburg",
	},
	"FATH": {
		ICAO:      "FATH",
		IATA:      "THY",
		Name:      "Thohoyandou Airport",
		City:      "Thohoyandou",
		State:     "Limpopo",
		Country:   "ZA",
		Elevation: 2021,
		Latitude:  -23.0769004822,
		Longitude: 30.383600235,
		Timezone:  "Africa/Johannesburg",
	},
	"FATI": {
		ICAO:      "FATI",
		Name:      "Thabazimbi Airport",
		City:      "Thabazimbi",
		State:     "Limpopo",
		Country:   "ZA",
		Elevation: 3225,
		Latitude:  -24.5760850299,
		Longitude: 27.4196863174,
		Timezone:  "Africa/Johannesburg",
	},
	"FATK": {
		ICAO:      "FATK",
		Name:      "Tsitsikama Fly Airport",
		City:      "Tsitsikama",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 381,
		Latitude:  -34.0847015381,
		Longitude: 24.2896995544,
		Timezone:  "Africa/Johannesburg",
	},
	"FATM": {
		ICAO:      "FATM",
		Name:      "Stutterheim Airport",
		City:      "Stutterheim",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 2675,
		Latitude:  -32.5666656494,
		Longitude: 27.4500007629,
		Timezone:  "Africa/Johannesburg",
	},
	"FATN": {
		ICAO:      "FATN",
		IATA:      "TCU",
		Name:      "Thaba Nchu Tar Airport",
		City:      "Homeward",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 4941,
		Latitude:  -29.3178005219,
		Longitude: 26.8227996826,
		Timezone:  "Africa/Johannesburg",
	},
	"FATP": {
		ICAO:      "FATP",
		Name:      "New Tempe Airport",
		City:      "Bain's Vlei",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 4547,
		Latitude:  -29.0328998566,
		Longitude: 26.1576004028,
		Timezone:  "Africa/Johannesburg",
	},
	"FATR": {
		ICAO:      "FATR",
		Name:      "Trennery's Airport",
		City:      "Qolora",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 114,
		Latitude:  -32.6385993958,
		Longitude: 28.4242992401,
		Timezone:  "Africa/Johannesburg",
	},
	"FATT": {
		ICAO:      "FATT",
		Name:      "Tutuka Power Station Airport",
		City:      "Standerton",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 5313,
		Latitude:  -26.776599884,
		Longitude: 29.3388004303,
		Timezone:  "Africa/Johannesburg",
	},
	"FATW": {
		ICAO:      "FATW",
		Name:      "Witberg Tswalu Airport",
		City:      "Tswalo Game Reserve",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 3921,
		Latitude:  -27.2049999237,
		Longitude: 22.4818992615,
		Timezone:  "Africa/Johannesburg",
	},
	"FATZ": {
		ICAO:      "FATZ",
		IATA:      "LTA",
		Name:      "Tzaneen Airport",
		City:      "Tzaneen",
		State:     "Limpopo",
		Country:   "ZA",
		Elevation: 1914,
		Latitude:  -23.8243999481,
		Longitude: 30.3292999268,
		Timezone:  "Africa/Johannesburg",
	},
	"FAUB": {
		ICAO:      "FAUB",
		Name:      "Underberg Airport",
		City:      "Underberg",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 5057,
		Latitude:  -29.78840065,
		Longitude: 29.4983005524,
		Timezone:  "Africa/Johannesburg",
	},
	"FAUC": {
		ICAO:      "FAUC",
		Name:      "Ulco Airport",
		City:      "Ulco",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 3512,
		Latitude:  -28.3544998169,
		Longitude: 24.2290992737,
		Timezone:  "Africa/Johannesburg",
	},
	"FAUG": {
		ICAO:      "FAUG",
		Name:      "Ugie Airport",
		City:      "Ugie",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 4580,
		Latitude:  -31.2199993134,
		Longitude: 28.2099990845,
		Timezone:  "Africa/Johannesburg",
	},
	"FAUH": {
		ICAO:      "FAUH",
		Name:      "Uitenhage Airport",
		City:      "Uitenhage",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 289,
		Latitude:  -33.7853012085,
		Longitude: 25.3833007813,
		Timezone:  "Africa/Johannesburg",
	},
	"FAUL": {
		ICAO:      "FAUL",
		IATA:      "ULD",
		Name:      "Prince Mangosuthu Buthelezi Airport",
		City:      "Ulundi",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 1720,
		Latitude:  -28.3206005096,
		Longitude: 31.4165000916,
		Timezone:  "Africa/Johannesburg",
	},
	"FAUP": {
		ICAO:      "FAUP",
		IATA:      "UTN",
		Name:      "Pierre Van Ryneveld Airport",
		City:      "Upington",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 2782,
		Latitude:  -28.39909935,
		Longitude: 21.2602005005,
		Timezone:  "Africa/Johannesburg",
	},
	"FAUR": {
		ICAO:      "FAUR",
		Name:      "Utrecht Airport",
		City:      "Utrecht",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 4101,
		Latitude:  -27.6805000305,
		Longitude: 30.3167991638,
		Timezone:  "Africa/Johannesburg",
	},
	"FAUS": {
		ICAO:      "FAUS",
		IATA:      "ULX",
		Name:      "Ulusaba Airport",
		City:      "Ulusaba",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 1263,
		Latitude:  -24.7854003906,
		Longitude: 31.3549003601,
		Timezone:  "Africa/Johannesburg",
	},
	"FAUT": {
		ICAO:      "FAUT",
		IATA:      "UTT",
		Name:      "K. D. Matanzima Airport",
		City:      "Mthatha",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 2400,
		Latitude:  -31.5478992462,
		Longitude: 28.6742992401,
		Timezone:  "Africa/Johannesburg",
	},
	"FAVA": {
		ICAO:      "FAVA",
		Name:      "Vaalputs Airport",
		City:      "Vaalputs",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 3340,
		Latitude:  -30.1341991425,
		Longitude: 18.5266990662,
		Timezone:  "Africa/Johannesburg",
	},
	"FAVB": {
		ICAO:      "FAVB",
		IATA:      "VRU",
		Name:      "Vryburg Airport",
		City:      "Vyrburg",
		State:     "North-West",
		Country:   "ZA",
		Elevation: 3920,
		Latitude:  -26.9824008942,
		Longitude: 24.7287998199,
		Timezone:  "Africa/Johannesburg",
	},
	"FAVD": {
		ICAO:      "FAVD",
		Name:      "Vrede Airport",
		City:      "Vrede",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 5499,
		Latitude:  -27.4402999878,
		Longitude: 29.1530990601,
		Timezone:  "Africa/Johannesburg",
	},
	"FAVE": {
		ICAO:      "FAVE",
		Name:      "Ventersdorp Airport",
		City:      "Ventersdorp",
		State:     "North-West",
		Country:   "ZA",
		Elevation: 4917,
		Latitude:  -26.3008995056,
		Longitude: 26.8141994476,
		Timezone:  "Africa/Johannesburg",
	},
	"FAVF": {
		ICAO:      "FAVF",
		Name:      "Verborgenfontei Airport",
		City:      "Verborgenfontein",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 4360,
		Latitude:  -31.1658000946,
		Longitude: 23.8094005585,
		Timezone:  "Africa/Johannesburg",
	},
	"FAVG": {
		ICAO:      "FAVG",
		IATA:      "VIR",
		Name:      "Virginia Airport",
		City:      "Durban",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 20,
		Latitude:  -29.7705993652,
		Longitude: 31.0583992004,
		Timezone:  "Africa/Johannesburg",
	},
	"FAVI": {
		ICAO:      "FAVI",
		Name:      "Von Abo's Villa Airport",
		City:      "Von Abo's Villa",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 4304,
		Latitude:  -27.6105995178,
		Longitude: 26.6835002899,
		Timezone:  "Africa/Johannesburg",
	},
	"FAVM": {
		ICAO:      "FAVM",
		Name:      "Venetia Airport",
		City:      "Venetia Mine",
		State:     "Limpopo",
		Country:   "ZA",
		Elevation: 2333,
		Latitude:  -22.4480991364,
		Longitude: 29.3376998901,
		Timezone:  "Africa/Johannesburg",
	},
	"FAVP": {
		ICAO:      "FAVP",
		Name:      "Vanderbijlpark Airport",
		City:      "Vanderbijlpark",
		State:     "Gauteng",
		Country:   "ZA",
		Elevation: 4861,
		Latitude:  -26.6902999878,
		Longitude: 27.7770996094,
		Timezone:  "Africa/Johannesburg",
	},
	"FAVR": {
		ICAO:      "FAVR",
		IATA:      "VRE",
		Name:      "Vredendal Airport",
		City:      "Vredendal",
		State:     "Western-Cape",
		Country:   "ZA",
		Elevation: 330,
		Latitude:  -31.6410007477,
		Longitude: 18.5447998047,
		Timezone:  "Africa/Johannesburg",
	},
	"FAVS": {
		ICAO:      "FAVS",
		Name:      "Vastrap Airport",
		City:      "Upington",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 3245,
		Latitude:  -27.8413887024,
		Longitude: 21.6341667175,
		Timezone:  "Africa/Johannesburg",
	},
	"FAVU": {
		ICAO:      "FAVU",
		Name:      "Volksrust Airport",
		City:      "Volksrust",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 5610,
		Latitude:  -27.3768997192,
		Longitude: 29.8600006104,
		Timezone:  "Africa/Johannesburg",
	},
	"FAVV": {
		ICAO:      "FAVV",
		Name:      "Vereeniging Airport",
		City:      "Vereeniging",
		State:     "Gauteng",
		Country:   "ZA",
		Elevation: 4846,
		Latitude:  -26.566400528,
		Longitude: 27.9608001709,
		Timezone:  "Africa/Johannesburg",
	},
	"FAVW": {
		ICAO:      "FAVW",
		Name:      "Victoria West Airport",
		City:      "Victoria West",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 4124,
		Latitude:  -31.3994007111,
		Longitude: 23.1564006805,
		Timezone:  "Africa/Johannesburg",
	},
	"FAVY": {
		ICAO:      "FAVY",
		IATA:      "VYD",
		Name:      "Vryheid Airport",
		City:      "Vryheid",
		State:     "KwaZulu-Natal",
		Country:   "ZA",
		Elevation: 3799,
		Latitude:  -27.7868995667,
		Longitude: 30.7964000702,
		Timezone:  "Africa/Johannesburg",
	},
	"FAWA": {
		ICAO:      "FAWA",
		Name:      "Warmbaths Airport",
		City:      "Warmbaths",
		State:     "Limpopo",
		Country:   "ZA",
		Elevation: 3655,
		Latitude:  -24.9099998474,
		Longitude: 28.2999992371,
		Timezone:  "Africa/Johannesburg",
	},
	"FAWB": {
		ICAO:      "FAWB",
		IATA:      "PRY",
		Name:      "Wonderboom Airport",
		City:      "Pretoria",
		State:     "Gauteng",
		Country:   "ZA",
		Elevation: 4095,
		Latitude:  -25.6539001465,
		Longitude: 28.224199295,
		Timezone:  "Africa/Johannesburg",
	},
	"FAWC": {
		ICAO:      "FAWC",
		Name:      "Worcester Glider Airport",
		City:      "Worcester",
		State:     "Western-Cape",
		Country:   "ZA",
		Elevation: 673,
		Latitude:  -33.6631011963,
		Longitude: 19.4153003693,
		Timezone:  "Africa/Johannesburg",
	},
	"FAWD": {
		ICAO:      "FAWD",
		Name:      "Wolmeransstad Airport",
		City:      "Wolmeransstad",
		State:     "North-West",
		Country:   "ZA",
		Elevation: 4513,
		Latitude:  -27.1700000763,
		Longitude: 25.9799995422,
		Timezone:  "Africa/Johannesburg",
	},
	"FAWI": {
		ICAO:      "FAWI",
		Name:      "Witbank Airport",
		City:      "Witbank",
		State:     "Mpumalanga",
		Country:   "ZA",
		Elevation: 5078,
		Latitude:  -25.8323001862,
		Longitude: 29.1919994354,
		Timezone:  "Africa/Johannesburg",
	},
	"FAWK": {
		ICAO:      "FAWK",
		IATA:      "WKF",
		Name:      "Waterkloof Air Force Base",
		City:      "Pretoria",
		State:     "Gauteng",
		Country:   "ZA",
		Elevation: 4940,
		Latitude:  -25.8299999237,
		Longitude: 28.2224998474,
		Timezone:  "Africa/Johannesburg",
	},
	"FAWL": {
		ICAO:      "FAWL",
		Name:      "Williston Airport",
		City:      "Williston",
		State:     "Northern-Cape",
		Country:   "ZA",
		Elevation: 3484,
		Latitude:  -31.383600235,
		Longitude: 20.932800293,
		Timezone:  "Africa/Johannesburg",
	},
	"FAWM": {
		ICAO:      "FAWM",
		IATA:      "WEL",
		Name:      "Welkom Airport",
		City:      "Welkom",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 4399,
		Latitude:  -27.9979991913,
		Longitude: 26.6695995331,
		Timezone:  "Africa/Johannesburg",
	},
	"FAWN": {
		ICAO:      "FAWN",
		Name:      "Winburg Airport",
		City:      "Winburg",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 4800,
		Latitude:  -28.5,
		Longitude: 27.0166664124,
		Timezone:  "Africa/Johannesburg",
	},
	"FAWO": {
		ICAO:      "FAWO",
		Name:      "Willowmore Airport",
		City:      "Willowmore",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 2608,
		Latitude:  -33.2633018494,
		Longitude: 23.4906005859,
		Timezone:  "Africa/Johannesburg",
	},
	"FAWP": {
		ICAO:      "FAWP",
		Name:      "Wepener Airport",
		City:      "Wepener",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 4755,
		Latitude:  -29.7423992157,
		Longitude: 27.022600174,
		Timezone:  "Africa/Johannesburg",
	},
	"FAWR": {
		ICAO:      "FAWR",
		Name:      "Wavecrest Airport",
		City:      "Wavecrest",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 89,
		Latitude:  -32.5896987915,
		Longitude: 28.5191993713,
		Timezone:  "Africa/Johannesburg",
	},
	"FAWS": {
		ICAO:      "FAWS",
		Name:      "Wesselbronn Af Airport",
		City:      "Wesselsbron",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 4301,
		Latitude:  -27.8467006683,
		Longitude: 26.3505992889,
		Timezone:  "Africa/Johannesburg",
	},
	"FAWT": {
		ICAO:      "FAWT",
		Name:      "Winterveldt Mine Airport",
		City:      "Doornbosch",
		State:     "Limpopo",
		Country:   "ZA",
		Elevation: 3000,
		Latitude:  -24.665,
		Longitude: 30.18528,
		Timezone:  "Africa/Johannesburg",
	},
	"FAYP": {
		ICAO:      "FAYP",
		Name:      "Ysterplaat Air Force Base",
		City:      "Cape Town",
		State:     "Western-Cape",
		Country:   "ZA",
		Elevation: 52,
		Latitude:  -33.9001998901,
		Longitude: 18.4983005524,
		Timezone:  "Africa/Johannesburg",
	},
	"FAZA": {
		ICAO:      "FAZA",
		Name:      "Zastron Airport",
		City:      "Zastron",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 4265,
		Latitude:  -30.2999992371,
		Longitude: 27.1170005798,
		Timezone:  "Africa/Johannesburg",
	},
	"FAZP": {
		ICAO:      "FAZP",
		Name:      "Mazeppa Bay Airport",
		City:      "Mazeppa Bay",
		State:     "Eastern-Cape",
		Country:   "ZA",
		Elevation: 13,
		Latitude:  -32.4719009399,
		Longitude: 28.6527996063,
		Timezone:  "Africa/Johannesburg",
	},
	"FAZQ": {
		ICAO:      "FAZQ",
		Name:      "Star Airport",
		City:      "Star",
		State:     "Orange-Free-State",
		Country:   "ZA",
		Elevation: 4754,
		Latitude:  -26.8931007385,
		Longitude: 27.6900005341,
		Timezone:  "Africa/Johannesburg",
	},
	"FAZR": {
		ICAO:      "FAZR",
		Name:      "Zeerust Airport",
		City:      "Zeerust",
		State:     "North-West",
		Country:   "ZA",
		Elevation: 4258,
		Latitude:  -25.5990009308,
		Longitude: 26.0422992706,
		Timezone:  "Africa/Johannesburg",
	},
	"FBCO": {
		ICAO:      "FBCO",
		Name:      "Camp Okavango Airport",
		City:      "Camp Okavango",
		State:     "North-West",
		Country:   "BW",
		Elevation: 3150,
		Latitude:  -19.1170005798,
		Longitude: 23.1000003815,
		Timezone:  "Africa/Gaborone",
	},
	"FBFT": {
		ICAO:      "FBFT",
		IATA:      "FRW",
		Name:      "Francistown Airport",
		City:      "Francistown",
		State:     "North-East",
		Country:   "BW",
		Elevation: 3283,
		Latitude:  -21.1595993042,
		Longitude: 27.4745006561,
		Timezone:  "Africa/Gaborone",
	},
	"FBGM": {
		ICAO:      "FBGM",
		Name:      "Gumare Airport",
		City:      "Gumare",
		State:     "North-West",
		Country:   "BW",
		Elevation: 3100,
		Latitude:  -19.3666305542,
		Longitude: 22.1666660309,
		Timezone:  "Africa/Gaborone",
	},
	"FBGZ": {
		ICAO:      "FBGZ",
		IATA:      "GNZ",
		Name:      "Ghanzi Airport",
		City:      "Ghanzi",
		State:     "Ghanzi",
		Country:   "BW",
		Elevation: 3730,
		Latitude:  -21.6924991608,
		Longitude: 21.6581001282,
		Timezone:  "Africa/Gaborone",
	},
	"FBJW": {
		ICAO:      "FBJW",
		IATA:      "JWA",
		Name:      "Jwaneng Airport",
		State:     "Ngwaketsi",
		Country:   "BW",
		Elevation: 3900,
		Latitude:  -24.6023006439,
		Longitude: 24.6909999847,
		Timezone:  "Africa/Gaborone",
	},
	"FBKE": {
		ICAO:      "FBKE",
		IATA:      "BBK",
		Name:      "Kasane Airport",
		City:      "Kasane",
		State:     "North-West",
		Country:   "BW",
		Elevation: 3289,
		Latitude:  -17.8328990936,
		Longitude: 25.162399292,
		Timezone:  "Africa/Gaborone",
	},
	"FBKG": {
		ICAO:      "FBKG",
		Name:      "Kang Airport",
		State:     "Kgalagadi",
		Country:   "BW",
		Elevation: 3520,
		Latitude:  -23.6833000183,
		Longitude: 22.8166999817,
		Timezone:  "Africa/Gaborone",
	},
	"FBKR": {
		ICAO:      "FBKR",
		IATA:      "KHW",
		Name:      "Khwai River Lodge Airport",
		City:      "Khwai River Lodge",
		State:     "North-West",
		Country:   "BW",
		Elevation: 3000,
		Latitude:  -19.1499996185,
		Longitude: 23.783000946,
		Timezone:  "Africa/Gaborone",
	},
	"FBKY": {
		ICAO:      "FBKY",
		Name:      "Kanye Airport",
		City:      "Kanye",
		State:     "Ngwaketsi",
		Country:   "BW",
		Elevation: 4199,
		Latitude:  -25.0499992371,
		Longitude: 25.3166999817,
		Timezone:  "Africa/Gaborone",
	},
	"FBMG": {
		ICAO:      "FBMG",
		Name:      "Machaneng Airport",
		City:      "Machaneng",
		State:     "Central",
		Country:   "BW",
		Elevation: 2900,
		Latitude:  -23.1830005646,
		Longitude: 27.466999054,
		Timezone:  "Africa/Gaborone",
	},
	"FBML": {
		ICAO:      "FBML",
		Name:      "Molepolole Airport",
		City:      "Molepolole",
		State:     "Kweneng",
		Country:   "BW",
		Elevation: 3790,
		Latitude:  -24.389444444,
		Longitude: 25.498611111,
		Timezone:  "Africa/Gaborone",
	},
	"FBMM": {
		ICAO:      "FBMM",
		Name:      "Makalamabedi Airport",
		City:      "Makalamabedi",
		State:     "Central",
		Country:   "BW",
		Elevation: 3100,
		Latitude:  -20.3330001831,
		Longitude: 23.8829994202,
		Timezone:  "Africa/Gaborone",
	},
	"FBMN": {
		ICAO:      "FBMN",
		IATA:      "MUB",
		Name:      "Maun Airport",
		City:      "Maun",
		State:     "North-West",
		Country:   "BW",
		Elevation: 3093,
		Latitude:  -19.9726009369,
		Longitude: 23.4311008453,
		Timezone:  "Africa/Gaborone",
	},
	"FBNN": {
		ICAO:      "FBNN",
		Name:      "Nokaneng Airport",
		City:      "Nokaneng",
		State:     "North-West",
		Country:   "BW",
		Elevation: 3200,
		Latitude:  -19.3386001587,
		Longitude: 22.153799057,
		Timezone:  "Africa/Gaborone",
	},
	"FBNT": {
		ICAO:      "FBNT",
		Name:      "Nata Airport",
		City:      "Nata",
		State:     "Central",
		Country:   "BW",
		Elevation: 2690,
		Latitude:  -20.2161006927,
		Longitude: 26.1588993073,
		Timezone:  "Africa/Gaborone",
	},
	"FBNW": {
		ICAO:      "FBNW",
		Name:      "Gaborone Notwane Airport",
		City:      "Gaborone Notwane",
		State:     "South-East",
		Country:   "BW",
		Elevation: 3218,
		Latitude:  -24.6830005646,
		Longitude: 25.9330005646,
		Timezone:  "Africa/Gaborone",
	},
	"FBOK": {
		ICAO:      "FBOK",
		Name:      "Okwa Airport",
		City:      "Okwa",
		State:     "Ghanzi",
		Country:   "BW",
		Elevation: 3500,
		Latitude:  -23.0830001831,
		Longitude: 21.8829994202,
		Timezone:  "Africa/Gaborone",
	},
	"FBOR": {
		ICAO:      "FBOR",
		IATA:      "ORP",
		Name:      "Orapa Airport",
		State:     "Central",
		Country:   "BW",
		Elevation: 3100,
		Latitude:  -21.2667007446,
		Longitude: 25.3166999817,
		Timezone:  "Africa/Gaborone",
	},
	"FBPY": {
		ICAO:      "FBPY",
		IATA:      "QPH",
		Name:      "Palapye Airport",
		City:      "Palapye",
		State:     "Central",
		Country:   "BW",
		Elevation: 3000,
		Latitude:  -22.5669994354,
		Longitude: 27.1499996185,
		Timezone:  "Africa/Gaborone",
	},
	"FBRK": {
		ICAO:      "FBRK",
		Name:      "Rakops Airport",
		City:      "Rakops",
		State:     "Central",
		Country:   "BW",
		Elevation: 3000,
		Latitude:  -21,
		Longitude: 24.3330001831,
		Timezone:  "Africa/Gaborone",
	},
	"FBSK": {
		ICAO:      "FBSK",
		IATA:      "GBE",
		Name:      "Sir Seretse Khama International Airport",
		City:      "Gaborone",
		State:     "South-East",
		Country:   "BW",
		Elevation: 3299,
		Latitude:  -24.5552005768,
		Longitude: 25.9181995392,
		Timezone:  "Africa/Gaborone",
	},
	"FBSN": {
		ICAO:      "FBSN",
		IATA:      "SXN",
		Name:      "Sua Pan Airport",
		State:     "Central",
		Country:   "BW",
		Elevation: 2985,
		Latitude:  -20.5534000397,
		Longitude: 26.1158008575,
		Timezone:  "Africa/Gaborone",
	},
	"FBSP": {
		ICAO:      "FBSP",
		IATA:      "PKW",
		Name:      "Selebi Phikwe Airport",
		Country:   "BW",
		Elevation: 2925,
		Latitude:  -22.0583000183,
		Longitude: 27.8288002014,
		Timezone:  "Africa/Gaborone",
	},
	"FBSR": {
		ICAO:      "FBSR",
		Name:      "Serowe Airport",
		City:      "Serowe",
		State:     "Central",
		Country:   "BW",
		Elevation: 3800,
		Latitude:  -22.4232997894,
		Longitude: 26.7567005157,
		Timezone:  "Africa/Gaborone",
	},
	"FBSV": {
		ICAO:      "FBSV",
		IATA:      "SVT",
		Name:      "Savuti Airport",
		City:      "Savuti",
		State:     "North-West",
		Country:   "BW",
		Elevation: 3150,
		Latitude:  -18.5205993652,
		Longitude: 24.0767002106,
		Timezone:  "Africa/Gaborone",
	},
	"FBSW": {
		ICAO:      "FBSW",
		IATA:      "SWX",
		Name:      "Shakawe Airport",
		City:      "Shakawe",
		State:     "North-West",
		Country:   "BW",
		Elevation: 3379,
		Latitude:  -18.3738994598,
		Longitude: 21.8325996399,
		Timezone:  "Africa/Gaborone",
	},
	"FBTE": {
		ICAO:      "FBTE",
		Name:      "Tshane Airport",
		City:      "Tshane",
		State:     "Kgalagadi",
		Country:   "BW",
		Elevation: 3700,
		Latitude:  -24.0170001984,
		Longitude: 21.8829994202,
		Timezone:  "Africa/Gaborone",
	},
	"FBTL": {
		ICAO:      "FBTL",
		IATA:      "TLD",
		Name:      "Limpopo Valley Airport",
		City:      "Tuli Lodge",
		State:     "Central",
		Country:   "BW",
		Elevation: 1772,
		Latitude:  -22.1891994476,
		Longitude: 29.1268997192,
		Timezone:  "Africa/Gaborone",
	},
	"FBTP": {
		ICAO:      "FBTP",
		Name:      "Thebephatshwa Airport",
		State:     "Kweneng",
		Country:   "BW",
		Elevation: 3750,
		Latitude:  -24.2210998535,
		Longitude: 25.3472995758,
		Timezone:  "Africa/Gaborone",
	},
	"FBTS": {
		ICAO:      "FBTS",
		IATA:      "TBY",
		Name:      "Tshabong Airport",
		City:      "Tshabong",
		State:     "Kgalagadi",
		Country:   "BW",
		Elevation: 3179,
		Latitude:  -26.0333003998,
		Longitude: 22.3999996185,
		Timezone:  "Africa/Gaborone",
	},
	"FBXB": {
		ICAO:      "FBXB",
		Name:      "Xaxaba Airport",
		City:      "Xaxaba",
		Country:   "BW",
		Elevation: 3100,
		Latitude:  -19.5505008698,
		Longitude: 23.0548000336,
		Timezone:  "Africa/Gaborone",
	},
	"FBXG": {
		ICAO:      "FBXG",
		Name:      "Xugana Airport",
		State:     "North-West",
		Country:   "BW",
		Latitude:  -19.0499992371,
		Longitude: 23.0900001526,
		Timezone:  "Africa/Gaborone",
	},
	"FCBA": {
		ICAO:      "FCBA",
		Name:      "La Louila Airport",
		City:      "La Louila",
		State:     "Pool",
		Country:   "CG",
		Elevation: 984,
		Latitude:  -4.0830001831,
		Longitude: 14.2329998016,
		Timezone:  "Africa/Brazzaville",
	},
	"FCBB": {
		ICAO:      "FCBB",
		IATA:      "BZV",
		Name:      "Maya-Maya Airport",
		City:      "Brazzaville",
		State:     "Brazzaville",
		Country:   "CG",
		Elevation: 1048,
		Latitude:  -4.2516999245,
		Longitude: 15.2530002594,
		Timezone:  "Africa/Brazzaville",
	},
	"FCBD": {
		ICAO:      "FCBD",
		IATA:      "DJM",
		Name:      "Djambala Airport",
		City:      "Djambala",
		State:     "Plateaux",
		Country:   "CG",
		Elevation: 2595,
		Latitude:  -2.5329999924,
		Longitude: 14.75,
		Timezone:  "Africa/Brazzaville",
	},
	"FCBK": {
		ICAO:      "FCBK",
		IATA:      "KNJ",
		Name:      "Kindamba Airport",
		City:      "Kindamba",
		State:     "Pool",
		Country:   "CG",
		Elevation: 1460,
		Latitude:  -3.9500000477,
		Longitude: 14.5170001984,
		Timezone:  "Africa/Brazzaville",
	},
	"FCBL": {
		ICAO:      "FCBL",
		IATA:      "LCO",
		Name:      "Lague Airport",
		City:      "Lague",
		State:     "Plateaux",
		Country:   "CG",
		Elevation: 2756,
		Latitude:  -2.4500000477,
		Longitude: 14.5329999924,
		Timezone:  "Africa/Brazzaville",
	},
	"FCBM": {
		ICAO:      "FCBM",
		IATA:      "MUY",
		Name:      "Mouyondzi Airport",
		City:      "Mouyondzi",
		State:     "Bouenza",
		Country:   "CG",
		Elevation: 1670,
		Latitude:  -4.0148739815,
		Longitude: 13.9661121368,
		Timezone:  "Africa/Brazzaville",
	},
	"FCBP": {
		ICAO:      "FCBP",
		Name:      "M'passa Airport",
		City:      "M'passa",
		State:     "Pool",
		Country:   "CG",
		Elevation: 1115,
		Latitude:  -4.367000103,
		Longitude: 14.1499996185,
		Timezone:  "Africa/Brazzaville",
	},
	"FCBS": {
		ICAO:      "FCBS",
		IATA:      "SIB",
		Name:      "Sibiti Airport",
		City:      "Sibiti",
		State:     "Lekoumou",
		Country:   "CG",
		Elevation: 1883,
		Latitude:  -3.6830000877,
		Longitude: 13.3500003815,
		Timezone:  "Africa/Brazzaville",
	},
	"FCBT": {
		ICAO:      "FCBT",
		Name:      "Loutete Airport",
		City:      "Loutete",
		State:     "Bouenza",
		Country:   "CG",
		Elevation: 656,
		Latitude:  -4.2829999924,
		Longitude: 13.8669996262,
		Timezone:  "Africa/Brazzaville",
	},
	"FCBU": {
		ICAO:      "FCBU",
		Name:      "Aubeville Airport",
		City:      "Aubeville",
		State:     "Bouenza",
		Country:   "CG",
		Elevation: 1148,
		Latitude:  -4.2670001984,
		Longitude: 13.5329999924,
		Timezone:  "Africa/Brazzaville",
	},
	"FCBY": {
		ICAO:      "FCBY",
		IATA:      "NKY",
		Name:      "Yokangassi Airport",
		City:      "Nkayi",
		State:     "Bouenza",
		Country:   "CG",
		Elevation: 541,
		Latitude:  -4.2170000076,
		Longitude: 13.2829999924,
		Timezone:  "Africa/Brazzaville",
	},
	"FCBZ": {
		ICAO:      "FCBZ",
		IATA:      "ANJ",
		Name:      "Zanaga Airport",
		City:      "Zanaga",
		State:     "Lekoumou",
		Country:   "CG",
		Elevation: 1870,
		Latitude:  -2.8499999046,
		Longitude: 13.8170003891,
		Timezone:  "Africa/Brazzaville",
	},
	"FCMA": {
		ICAO:      "FCMA",
		Name:      "Mavinza Airport",
		City:      "Mavinza",
		State:     "Niari",
		Country:   "CG",
		Elevation: 509,
		Latitude:  -2.4500000477,
		Longitude: 11.6499996185,
		Timezone:  "Africa/Brazzaville",
	},
	"FCMB": {
		ICAO:      "FCMB",
		Name:      "N'Ziba Airport",
		City:      "N'Ziba",
		State:     "Niari",
		Country:   "CG",
		Elevation: 2149,
		Latitude:  -2.7170000076,
		Longitude: 12.5329999924,
		Timezone:  "Africa/Brazzaville",
	},
	"FCMD": {
		ICAO:      "FCMD",
		Name:      "Sidetra Airport",
		City:      "Sidetra",
		State:     "Niari",
		Country:   "CG",
		Elevation: 2133,
		Latitude:  -2.867000103,
		Longitude: 12.8669996262,
		Timezone:  "Africa/Brazzaville",
	},
	"FCMF": {
		ICAO:      "FCMF",
		Name:      "Loufoula Airport",
		City:      "Loufoula",
		State:     "Niari",
		Country:   "CG",
		Elevation: 650,
		Latitude:  -3,
		Longitude: 12,
		Timezone:  "Africa/Brazzaville",
	},
	"FCMG": {
		ICAO:      "FCMG",
		Name:      "Gokango Airport",
		City:      "Gokango",
		State:     "Niari",
		Country:   "CG",
		Elevation: 820,
		Latitude:  -3.0329999924,
		Longitude: 12.1330003738,
		Timezone:  "Africa/Brazzaville",
	},
	"FCMI": {
		ICAO:      "FCMI",
		Name:      "Irogo Airport",
		City:      "Irogo",
		State:     "Niari",
		Country:   "CG",
		Elevation: 919,
		Latitude:  -2.7170000076,
		Longitude: 11.8830003738,
		Timezone:  "Africa/Brazzaville",
	},
	"FCMK": {
		ICAO:      "FCMK",
		Name:      "Kele Kibangou Airport",
		City:      "Kele Kibangou",
		State:     "Niari",
		Country:   "CG",
		Elevation: 902,
		Latitude:  -3.3169999123,
		Longitude: 12.6169996262,
		Timezone:  "Africa/Brazzaville",
	},
	"FCML": {
		ICAO:      "FCML",
		Name:      "Leboulou Airport",
		City:      "Leboulou",
		State:     "Niari",
		Country:   "CG",
		Elevation: 886,
		Latitude:  -2.9000000954,
		Longitude: 12.3669996262,
		Timezone:  "Africa/Brazzaville",
	},
	"FCMM": {
		ICAO:      "FCMM",
		IATA:      "MSX",
		Name:      "Mossendjo Airport",
		City:      "Mossendjo",
		State:     "Niari",
		Country:   "CG",
		Elevation: 1519,
		Latitude:  -2.9500000477,
		Longitude: 12.6999998093,
		Timezone:  "Africa/Brazzaville",
	},
	"FCMN": {
		ICAO:      "FCMN",
		Name:      "N'gongo Airport",
		City:      "N'gongo",
		State:     "Niari",
		Country:   "CG",
		Elevation: 906,
		Latitude:  -2.9830000401,
		Longitude: 12.1999998093,
		Timezone:  "Africa/Brazzaville",
	},
	"FCMO": {
		ICAO:      "FCMO",
		Name:      "Mandoro Airport",
		City:      "Mandoro",
		State:     "Niari",
		Country:   "CG",
		Elevation: 1772,
		Latitude:  -2.6500000954,
		Longitude: 12.8830003738,
		Timezone:  "Africa/Brazzaville",
	},
	"FCMR": {
		ICAO:      "FCMR",
		Name:      "Marala Airport",
		City:      "Marala",
		State:     "Niari",
		Country:   "CG",
		Elevation: 1535,
		Latitude:  -2.9000000954,
		Longitude: 12.6330003738,
		Timezone:  "Africa/Brazzaville",
	},
	"FCMS": {
		ICAO:      "FCMS",
		Name:      "Nyanga Airport",
		City:      "Nyanga",
		State:     "Niari",
		Country:   "CG",
		Elevation: 558,
		Latitude:  -2.867000103,
		Longitude: 11.9499998093,
		Timezone:  "Africa/Brazzaville",
	},
	"FCMY": {
		ICAO:      "FCMY",
		Name:      "Legala Airport",
		City:      "Legala",
		State:     "Niari",
		Country:   "CG",
		Elevation: 2152,
		Latitude:  -2.2170000076,
		Longitude: 12.8330001831,
		Timezone:  "Africa/Brazzaville",
	},
	"FCMZ": {
		ICAO:      "FCMZ",
		Name:      "N'Zabi Airport",
		City:      "N'Zabi",
		State:     "Niari",
		Country:   "CG",
		Elevation: 1640,
		Latitude:  -3.1670000553,
		Longitude: 12.8669996262,
		Timezone:  "Africa/Brazzaville",
	},
	"FCOB": {
		ICAO:      "FCOB",
		IATA:      "BOE",
		Name:      "Boundji Airport",
		City:      "Boundji",
		State:     "Cuvette",
		Country:   "CG",
		Elevation: 1247,
		Latitude:  -1.0329999924,
		Longitude: 15.3830003738,
		Timezone:  "Africa/Brazzaville",
	},
	"FCOE": {
		ICAO:      "FCOE",
		IATA:      "EWO",
		Name:      "Ewo Airport",
		City:      "Ewo",
		State:     "Cuvette-Ouest",
		Country:   "CG",
		Elevation: 1503,
		Latitude:  -0.8830000162,
		Longitude: 14.8000001907,
		Timezone:  "Africa/Brazzaville",
	},
	"FCOG": {
		ICAO:      "FCOG",
		IATA:      "GMM",
		Name:      "Gamboma Airport",
		City:      "Gamboma",
		State:     "Plateaux",
		Country:   "CG",
		Elevation: 1509,
		Latitude:  -1.829403,
		Longitude: 15.885237,
		Timezone:  "Africa/Brazzaville",
	},
	"FCOI": {
		ICAO:      "FCOI",
		IATA:      "ION",
		Name:      "Impfondo Airport",
		City:      "Impfondo",
		State:     "Likouala",
		Country:   "CG",
		Elevation: 1099,
		Latitude:  1.6169999838,
		Longitude: 18.0669994354,
		Timezone:  "Africa/Brazzaville",
	},
	"FCOK": {
		ICAO:      "FCOK",
		IATA:      "KEE",
		Name:      "Kelle Airport",
		City:      "Kelle",
		State:     "Cuvette-Ouest",
		Country:   "CG",
		Elevation: 1526,
		Latitude:  -0.0829999968,
		Longitude: 14.5329999924,
		Timezone:  "Africa/Brazzaville",
	},
	"FCOM": {
		ICAO:      "FCOM",
		IATA:      "MKJ",
		Name:      "Makoua Airport",
		City:      "Makoua",
		State:     "Cuvette",
		Country:   "CG",
		Elevation: 1293,
		Latitude:  -0.0170000009,
		Longitude: 15.5830001831,
		Timezone:  "Africa/Brazzaville",
	},
	"FCOO": {
		ICAO:      "FCOO",
		IATA:      "FTX",
		Name:      "Owando Airport",
		City:      "Owando",
		State:     "Cuvette",
		Country:   "CG",
		Elevation: 1214,
		Latitude:  -0.5313500166,
		Longitude: 15.9500999451,
		Timezone:  "Africa/Brazzaville",
	},
	"FCOS": {
		ICAO:      "FCOS",
		IATA:      "SOE",
		Name:      "Souanke Airport",
		City:      "Souanke",
		State:     "Sangha",
		Country:   "CG",
		Elevation: 1722,
		Latitude:  2.0669999123,
		Longitude: 14.1330003738,
		Timezone:  "Africa/Brazzaville",
	},
	"FCOT": {
		ICAO:      "FCOT",
		IATA:      "BTB",
		Name:      "Betou Airport",
		City:      "Betou",
		State:     "Likouala",
		Country:   "CG",
		Elevation: 1168,
		Latitude:  3.0499999523,
		Longitude: 18.5,
		Timezone:  "Africa/Brazzaville",
	},
	"FCOU": {
		ICAO:      "FCOU",
		IATA:      "OUE",
		Name:      "Ouesso Airport",
		State:     "Sangha",
		Country:   "CG",
		Elevation: 1158,
		Latitude:  1.6159900427,
		Longitude: 16.0379009247,
		Timezone:  "Africa/Brazzaville",
	},
	"FCPA": {
		ICAO:      "FCPA",
		IATA:      "KMK",
		Name:      "Makabana Airport",
		City:      "Makabana",
		State:     "Niari",
		Country:   "CG",
		Elevation: 495,
		Latitude:  -3.4830000401,
		Longitude: 12.6169996262,
		Timezone:  "Africa/Brazzaville",
	},
	"FCPB": {
		ICAO:      "FCPB",
		Name:      "Bangamba Airport",
		City:      "Bangamba",
		State:     "Lekoumou",
		Country:   "CG",
		Elevation: 1312,
		Latitude:  -3.6830000877,
		Longitude: 13.1999998093,
		Timezone:  "Africa/Brazzaville",
	},
	"FCPD": {
		ICAO:      "FCPD",
		IATA:      "DIS",
		Name:      "Ngot Nzoungou Airport",
		City:      "Dolisie",
		State:     "Niari",
		Country:   "CG",
		Elevation: 1079,
		Latitude:  -4.20635,
		Longitude: 12.6599,
		Timezone:  "Africa/Brazzaville",
	},
	"FCPE": {
		ICAO:      "FCPE",
		Name:      "Leganda Airport",
		City:      "Leganda",
		State:     "Lekoumou",
		Country:   "CG",
		Elevation: 1198,
		Latitude:  -3.4330000877,
		Longitude: 12.9329996109,
		Timezone:  "Africa/Brazzaville",
	},
	"FCPG": {
		ICAO:      "FCPG",
		Name:      "Kibangou Airport",
		City:      "Kibangou",
		State:     "Niari",
		Country:   "CG",
		Elevation: 427,
		Latitude:  -3.4830000401,
		Longitude: 12.3000001907,
		Timezone:  "Africa/Brazzaville",
	},
	"FCPI": {
		ICAO:      "FCPI",
		Name:      "Loubetsi Airport",
		City:      "Loubetsi",
		State:     "Niari",
		Country:   "CG",
		Elevation: 328,
		Latitude:  -3.7000000477,
		Longitude: 12.1330003738,
		Timezone:  "Africa/Brazzaville",
	},
	"FCPK": {
		ICAO:      "FCPK",
		Name:      "N'komo Airport",
		City:      "N'komo",
		State:     "Kouilou",
		Country:   "CG",
		Elevation: 98,
		Latitude:  -3.9170000553,
		Longitude: 11.3830003738,
		Timezone:  "Africa/Brazzaville",
	},
	"FCPN": {
		ICAO:      "FCPN",
		Name:      "Noumbi Airport",
		City:      "Noumbi",
		State:     "Kouilou",
		Country:   "CG",
		Elevation: 39,
		Latitude:  -4.1500000954,
		Longitude: 11.3830003738,
		Timezone:  "Africa/Brazzaville",
	},
	"FCPO": {
		ICAO:      "FCPO",
		Name:      "Pemo Airport",
		City:      "Pemo",
		State:     "Niari",
		Country:   "CG",
		Elevation: 1686,
		Latitude:  -2.867000103,
		Longitude: 12.5329999924,
		Timezone:  "Africa/Brazzaville",
	},
	"FCPP": {
		ICAO:      "FCPP",
		IATA:      "PNR",
		Name:      "Pointe Noire Airport",
		City:      "Pointe Noire",
		State:     "Kouilou",
		Country:   "CG",
		Elevation: 55,
		Latitude:  -4.8160300255,
		Longitude: 11.8865995407,
		Timezone:  "Africa/Brazzaville",
	},
	"FCPY": {
		ICAO:      "FCPY",
		Name:      "Loukanyi Airport",
		City:      "Loukanyi",
		State:     "Kouilou",
		Country:   "CG",
		Elevation: 295,
		Latitude:  -4.0329999924,
		Longitude: 11.5170001984,
		Timezone:  "Africa/Brazzaville",
	},
	"FD01": {
		ICAO:      "FD01",
		Name:      "Hidden Acres Airpark",
		City:      "Monticello",
		State:     "Florida",
		Country:   "US",
		Elevation: 34,
		Latitude:  30.2796001434,
		Longitude: -84.0487976074,
		Timezone:  "America/New_York",
	},
	"FD02": {
		ICAO:      "FD02",
		Name:      "85th Avenue Airstrip",
		City:      "Orange Springs",
		State:     "Florida",
		Country:   "US",
		Elevation: 80,
		Latitude:  29.4850006104,
		Longitude: -82.0220031738,
		Timezone:  "America/New_York",
	},
	"FD03": {
		ICAO:      "FD03",
		Name:      "The Funny Farm Airport",
		City:      "Crestview",
		State:     "Florida",
		Country:   "US",
		Elevation: 200,
		Latitude:  30.8001995087,
		Longitude: -86.4355010986,
		Timezone:  "America/Chicago",
	},
	"FD04": {
		ICAO:      "FD04",
		Name:      "Leeward Air Ranch Airport",
		City:      "Ocala/Belleview",
		State:     "Florida",
		Country:   "US",
		Elevation: 86,
		Latitude:  29.0841007233,
		Longitude: -82.0323028564,
		Timezone:  "America/New_York",
	},
	"FD08": {
		ICAO:      "FD08",
		Name:      "Antiquers Aerodrome",
		City:      "Delray Beach",
		State:     "Florida",
		Country:   "US",
		Elevation: 23,
		Latitude:  26.4654006958,
		Longitude: -80.1498031616,
		Timezone:  "America/New_York",
	},
	"FD09": {
		ICAO:      "FD09",
		Name:      "Rlm Farms Airport",
		City:      "Kenansville",
		State:     "Florida",
		Country:   "US",
		Elevation: 75,
		Latitude:  27.8099994659,
		Longitude: -80.8589019775,
		Timezone:  "America/New_York",
	},
	"FD14": {
		ICAO:      "FD14",
		Name:      "Paniola Air Ranch Airport",
		City:      "Citra",
		State:     "Florida",
		Country:   "US",
		Elevation: 140,
		Latitude:  29.3752994537,
		Longitude: -82.0580978394,
		Timezone:  "America/New_York",
	},
	"FD15": {
		ICAO:      "FD15",
		Name:      "Tailwinds Airport",
		City:      "Jupiter",
		State:     "Florida",
		Country:   "US",
		Elevation: 19,
		Latitude:  26.9794998169,
		Longitude: -80.2186965942,
		Timezone:  "America/New_York",
	},
	"FD16": {
		ICAO:      "FD16",
		Name:      "Flying C Farm Airport",
		City:      "Branford",
		State:     "Florida",
		Country:   "US",
		Elevation: 75,
		Latitude:  29.9657993317,
		Longitude: -82.8728027344,
		Timezone:  "America/New_York",
	},
	"FD20": {
		ICAO:      "FD20",
		Name:      "Sundance Farms Airport",
		City:      "La Belle",
		State:     "Florida",
		Country:   "US",
		Elevation: 50,
		Latitude:  26.8003005981,
		Longitude: -81.4850997925,
		Timezone:  "America/New_York",
	},
	"FD22": {
		ICAO:      "FD22",
		Name:      "Melrose Landing Airport",
		City:      "Melrose",
		State:     "Florida",
		Country:   "US",
		Elevation: 159,
		Latitude:  29.6669006348,
		Longitude: -81.9551010132,
		Timezone:  "America/New_York",
	},
	"FD24": {
		ICAO:      "FD24",
		Name:      "Southern Fruit Groves Airport",
		City:      "Port St. Lucie",
		State:     "Florida",
		Country:   "US",
		Elevation: 26,
		Latitude:  27.221200943,
		Longitude: -80.5205993652,
		Timezone:  "America/New_York",
	},
	"FD25": {
		ICAO:      "FD25",
		Name:      "Fly In Ranches Airport",
		City:      "Vero Beach",
		State:     "Florida",
		Country:   "US",
		Elevation: 25,
		Latitude:  27.5625,
		Longitude: -80.4998016357,
		Timezone:  "America/New_York",
	},
	"FD26": {
		ICAO:      "FD26",
		Name:      "Kirkland Airport",
		City:      "Graceville",
		State:     "Florida",
		Country:   "US",
		Elevation: 180,
		Latitude:  30.9818000793,
		Longitude: -85.492401123,
		Timezone:  "America/Chicago",
	},
	"FD27": {
		ICAO:      "FD27",
		Name:      "Cuyler Field",
		City:      "Glen St Mary",
		State:     "Florida",
		Country:   "US",
		Elevation: 120,
		Latitude:  30.3626995087,
		Longitude: -82.2261962891,
		Timezone:  "America/New_York",
	},
	"FD29": {
		ICAO:      "FD29",
		Name:      "Flying Dutchman Ranch Airport",
		City:      "Ocala",
		State:     "Florida",
		Country:   "US",
		Elevation: 75,
		Latitude:  29.1308002472,
		Longitude: -82.1193008423,
		Timezone:  "America/New_York",
	},
	"FD30": {
		ICAO:      "FD30",
		Name:      "Southeastern Airport",
		City:      "Fort Pierce",
		State:     "Florida",
		Country:   "US",
		Elevation: 23,
		Latitude:  27.4113998413,
		Longitude: -80.5261993408,
		Timezone:  "America/New_York",
	},
	"FD31": {
		ICAO:      "FD31",
		Name:      "Bradley Airport",
		City:      "Fort White",
		State:     "Florida",
		Country:   "US",
		Elevation: 20,
		Latitude:  29.9279994965,
		Longitude: -82.7012023926,
		Timezone:  "America/New_York",
	},
	"FD33": {
		ICAO:      "FD33",
		Name:      "Green Swamp Aerodrome",
		City:      "Lakeland",
		State:     "Florida",
		Country:   "US",
		Elevation: 115,
		Latitude:  28.2117004395,
		Longitude: -81.9987030029,
		Timezone:  "America/New_York",
	},
	"FD35": {
		ICAO:      "FD35",
		Name:      "Redhead Airport",
		City:      "Ebro",
		State:     "Florida",
		Country:   "US",
		Elevation: 102,
		Latitude:  30.4904994965,
		Longitude: -85.8394012451,
		Timezone:  "America/Chicago",
	},
	"FD37": {
		ICAO:      "FD37",
		Name:      "Gentry Airport",
		City:      "St Cloud",
		State:     "Florida",
		Country:   "US",
		Elevation: 80,
		Latitude:  28.1371994019,
		Longitude: -81.2633972168,
		Timezone:  "America/New_York",
	},
	"FD38": {
		ICAO:      "FD38",
		Name:      "Wellington Aero Club Airport",
		City:      "West Palm Beach",
		State:     "Florida",
		Country:   "US",
		Elevation: 19,
		Latitude:  26.6464996338,
		Longitude: -80.2941970825,
		Timezone:  "America/New_York",
	},
	"FD39": {
		ICAO:      "FD39",
		Name:      "Baggett Airpark",
		City:      "Okeechobee",
		State:     "Florida",
		Country:   "US",
		Elevation: 19,
		Latitude:  27.1555995941,
		Longitude: -80.6917037964,
		Timezone:  "America/New_York",
	},
	"FD40": {
		ICAO:      "FD40",
		Name:      "Gardner Airport",
		City:      "Gardner",
		State:     "Florida",
		Country:   "US",
		Elevation: 76,
		Latitude:  27.344499588,
		Longitude: -81.7922973633,
		Timezone:  "America/New_York",
	},
	"FD42": {
		ICAO:      "FD42",
		Name:      "Cooksey Brothers Airport",
		City:      "Live Oak",
		State:     "Florida",
		Country:   "US",
		Elevation: 90,
		Latitude:  30.21159935,
		Longitude: -83.0546035767,
		Timezone:  "America/New_York",
	},
	"FD44": {
		ICAO:      "FD44",
		Name:      "Eagles Nest Aerodrome",
		City:      "Crescent City",
		State:     "Florida",
		Country:   "US",
		Elevation: 63,
		Latitude:  29.4269444,
		Longitude: -81.6041667,
		Timezone:  "America/New_York",
	},
	"FD48": {
		ICAO:      "FD48",
		Name:      "Deep Forest Airport",
		City:      "Jacksonville",
		State:     "Florida",
		Country:   "US",
		Elevation: 24,
		Latitude:  30.2418994904,
		Longitude: -81.449798584,
		Timezone:  "America/New_York",
	},
	"FD49": {
		ICAO:      "FD49",
		Name:      "Montgomery's Flying M Ranch Airport",
		City:      "Lake Panasoffkee",
		State:     "Florida",
		Country:   "US",
		Elevation: 70,
		Latitude:  28.7735996246,
		Longitude: -82.1304016113,
		Timezone:  "America/New_York",
	},
	"FD50": {
		ICAO:      "FD50",
		Name:      "The Trails Airport",
		City:      "Mayo",
		State:     "Florida",
		Country:   "US",
		Elevation: 45,
		Latitude:  30.1091003418,
		Longitude: -83.1886978149,
		Timezone:  "America/New_York",
	},
	"FD51": {
		ICAO:      "FD51",
		Name:      "Summerland Key Cove Airport",
		City:      "Summerland Key",
		State:     "Florida",
		Country:   "US",
		Elevation: 4,
		Latitude:  24.6588001251,
		Longitude: -81.4414978027,
		Timezone:  "America/New_York",
	},
	"FD53": {
		ICAO:      "FD53",
		Name:      "Cub Haven STOLport",
		City:      "Dade City",
		State:     "Florida",
		Country:   "US",
		Elevation: 70,
		Latitude:  28.4533004761,
		Longitude: -82.2149963379,
		Timezone:  "America/New_York",
	},
	"FD55": {
		ICAO:      "FD55",
		Name:      "Rutten Dusting Strip",
		City:      "Havana",
		State:     "Florida",
		Country:   "US",
		Elevation: 260,
		Latitude:  30.6793994904,
		Longitude: -84.369102478,
		Timezone:  "America/New_York",
	},
	"FD57": {
		ICAO:      "FD57",
		Name:      "Baggett STOLport",
		City:      "Fort Pierce",
		State:     "Florida",
		Country:   "US",
		Elevation: 18,
		Latitude:  27.4698009491,
		Longitude: -80.4163970947,
		Timezone:  "America/New_York",
	},
	"FD59": {
		ICAO:      "FD59",
		Name:      "Babcock H.Q. Airport",
		City:      "Punta Gorda",
		State:     "Florida",
		Country:   "US",
		Elevation: 34,
		Latitude:  26.8694992065,
		Longitude: -81.7175979614,
		Timezone:  "America/New_York",
	},
	"FD61": {
		ICAO:      "FD61",
		Name:      "Wright Farms Airport",
		City:      "Live Oak",
		State:     "Florida",
		Country:   "US",
		Elevation: 104,
		Latitude:  30.1765995026,
		Longitude: -82.9979019165,
		Timezone:  "America/New_York",
	},
	"FD63": {
		ICAO:      "FD63",
		Name:      "Squires Aviation Ranch Airport",
		City:      "Jasper",
		State:     "Florida",
		Country:   "US",
		Elevation: 110,
		Latitude:  30.5128002167,
		Longitude: -83.1983032227,
		Timezone:  "America/New_York",
	},
	"FD68": {
		ICAO:      "FD68",
		Name:      "Wakulla Club Airport",
		City:      "St. Marks",
		State:     "Florida",
		Country:   "US",
		Elevation: 15,
		Latitude:  30.1979999542,
		Longitude: -84.2535018921,
		Timezone:  "America/New_York",
	},
	"FD70": {
		ICAO:      "FD70",
		Name:      "River Acres Airport",
		City:      "Okeechobee",
		State:     "Florida",
		Country:   "US",
		Elevation: 30,
		Latitude:  27.3253002167,
		Longitude: -81.0287017822,
		Timezone:  "America/New_York",
	},
	"FD71": {
		ICAO:      "FD71",
		Name:      "O'Brien Airpark East/West Airport",
		City:      "Branford",
		State:     "Florida",
		Country:   "US",
		Elevation: 55,
		Latitude:  30.04829979,
		Longitude: -82.98390198,
		Timezone:  "America/New_York",
	},
	"FD72": {
		ICAO:      "FD72",
		Name:      "Kings Port Airport",
		City:      "Lake Placid",
		State:     "Florida",
		Country:   "US",
		Elevation: 148,
		Latitude:  27.182800293,
		Longitude: -81.3906021118,
		Timezone:  "America/New_York",
	},
	"FD74": {
		ICAO:      "FD74",
		Name:      "Gamebird Groves Airstrip",
		City:      "West Melbourne",
		State:     "Florida",
		Country:   "US",
		Elevation: 36,
		Latitude:  28.0659008026,
		Longitude: -80.9022979736,
		Timezone:  "America/New_York",
	},
	"FD77": {
		ICAO:      "FD77",
		Name:      "Wimauma Air Park",
		City:      "Wimauma",
		State:     "Florida",
		Country:   "US",
		Elevation: 100,
		Latitude:  27.7119998932,
		Longitude: -82.2828979492,
		Timezone:  "America/New_York",
	},
	"FD81": {
		ICAO:      "FD81",
		Name:      "Gleim Field",
		City:      "Gainesville",
		State:     "Florida",
		Country:   "US",
		Elevation: 170,
		Latitude:  29.7026996613,
		Longitude: -82.4256973267,
		Timezone:  "America/New_York",
	},
	"FD82": {
		ICAO:      "FD82",
		Name:      "Porter STOLport",
		City:      "Jay",
		State:     "Florida",
		Country:   "US",
		Elevation: 180,
		Latitude:  30.8404998779,
		Longitude: -87.105796814,
		Timezone:  "America/Chicago",
	},
	"FD83": {
		ICAO:      "FD83",
		Name:      "Stout Airport",
		City:      "Kissimmee",
		State:     "Florida",
		Country:   "US",
		Elevation: 65,
		Latitude:  28.258600235,
		Longitude: -81.3840026855,
		Timezone:  "America/New_York",
	},
	"FD84": {
		ICAO:      "FD84",
		Name:      "Delta Airport",
		City:      "Lake City",
		State:     "Florida",
		Country:   "US",
		Elevation: 200,
		Latitude:  30.1252002716,
		Longitude: -82.6636962891,
		Timezone:  "America/New_York",
	},
	"FD86": {
		ICAO:      "FD86",
		Name:      "Deep Woods Ranch Airport",
		City:      "Deland",
		State:     "Florida",
		Country:   "US",
		Elevation: 45,
		Latitude:  29.0303001404,
		Longitude: -81.4469985962,
		Timezone:  "America/New_York",
	},
	"FD88": {
		ICAO:      "FD88",
		Name:      "Aero Acres Airport",
		City:      "Fort Pierce",
		State:     "Florida",
		Country:   "US",
		Elevation: 25,
		Latitude:  27.341999054,
		Longitude: -80.5220031738,
		Timezone:  "America/New_York",
	},
	"FD89": {
		ICAO:      "FD89",
		Name:      "Collier/Pine Barren Airpark",
		City:      "Century",
		State:     "Florida",
		Country:   "US",
		Elevation: 220,
		Latitude:  30.8271007538,
		Longitude: -87.3582992554,
		Timezone:  "America/Chicago",
	},
	"FD90": {
		ICAO:      "FD90",
		Name:      "Lafayette Landings Airport",
		City:      "De Leon Springs",
		State:     "Florida",
		Country:   "US",
		Elevation: 35,
		Latitude:  29.1686000824,
		Longitude: -81.2648010254,
		Timezone:  "America/New_York",
	},
	"FD92": {
		ICAO:      "FD92",
		Name:      "Southerland Strip",
		City:      "Geneva",
		State:     "Florida",
		Country:   "US",
		Elevation: 15,
		Latitude:  28.73279953,
		Longitude: -81.0786972046,
		Timezone:  "America/New_York",
	},
	"FD93": {
		ICAO:      "FD93",
		Name:      "Yellow River Airstrip",
		City:      "Holt",
		State:     "Florida",
		Country:   "US",
		Elevation: 150,
		Latitude:  30.6854991913,
		Longitude: -86.7472000122,
		Timezone:  "America/Chicago",
	},
	"FD94": {
		ICAO:      "FD94",
		Name:      "Hartzog Field",
		City:      "Chipley",
		State:     "Florida",
		Country:   "US",
		Elevation: 125,
		Latitude:  30.7189006805,
		Longitude: -85.6066970825,
		Timezone:  "America/Chicago",
	},
	"FD96": {
		ICAO:      "FD96",
		Name:      "Hilliard's Private Airport",
		City:      "Moore Haven",
		State:     "Florida",
		Country:   "US",
		Elevation: 13,
		Latitude:  26.8325996399,
		Longitude: -81.0801010132,
		Timezone:  "America/New_York",
	},
	"FDBM": {
		ICAO:      "FDBM",
		Name:      "Matata Airport",
		City:      "Big Bend",
		Country:   "SZ",
		Elevation: 500,
		Latitude:  -26.8670005798,
		Longitude: 31.9330005646,
		Timezone:  "Africa/Mbabane",
	},
	"FDBS": {
		ICAO:      "FDBS",
		Name:      "Big Bend Sugar E Airport",
		City:      "Big Bend",
		State:     "Lubombo",
		Country:   "SZ",
		Elevation: 541,
		Latitude:  -26.7999992371,
		Longitude: 31.8999996185,
		Timezone:  "Africa/Mbabane",
	},
	"FDBT": {
		ICAO:      "FDBT",
		Name:      "Tambuti Airport",
		City:      "Big Bend",
		Country:   "SZ",
		Elevation: 595,
		Latitude:  -26.735200882,
		Longitude: 31.7758998871,
		Timezone:  "Africa/Mbabane",
	},
	"FDGL": {
		ICAO:      "FDGL",
		Name:      "Lavumisa Airport",
		City:      "Lavumisa",
		State:     "KwaZulu-Natal",
		Country:   "SZ",
		Elevation: 563,
		Latitude:  -27.3201007843,
		Longitude: 31.889799118,
		Timezone:  "Africa/Johannesburg",
	},
	"FDKB": {
		ICAO:      "FDKB",
		Name:      "Kubuta Airport",
		City:      "Kubuta B",
		State:     "Shiselweni",
		Country:   "SZ",
		Elevation: 1555,
		Latitude:  -26.8666667938,
		Longitude: 31.4833335876,
		Timezone:  "Africa/Mbabane",
	},
	"FDMH": {
		ICAO:      "FDMH",
		Name:      "Mhlume Airport",
		City:      "Mhlume",
		State:     "Lubombo",
		Country:   "SZ",
		Elevation: 926,
		Latitude:  -26.0242004395,
		Longitude: 31.8099994659,
		Timezone:  "Africa/Mbabane",
	},
	"FDMS": {
		ICAO:      "FDMS",
		IATA:      "MTS",
		Name:      "Matsapha Airport",
		City:      "Manzini",
		State:     "Manzini",
		Country:   "SZ",
		Elevation: 2075,
		Latitude:  -26.5289993286,
		Longitude: 31.3075008392,
		Timezone:  "Africa/Mbabane",
	},
	"FDNG": {
		ICAO:      "FDNG",
		Name:      "Piggs Peak Airport",
		City:      "Ngonini",
		State:     "Hhohho",
		Country:   "SZ",
		Elevation: 1410,
		Latitude:  -25.7666664124,
		Longitude: 31.3666667938,
		Timezone:  "Africa/Mbabane",
	},
	"FDNH": {
		ICAO:      "FDNH",
		Name:      "Nhlangano Airport",
		City:      "Nhlangano",
		State:     "Shiselweni",
		Country:   "SZ",
		Elevation: 3522,
		Latitude:  -27.1166667938,
		Longitude: 31.2166671753,
		Timezone:  "Africa/Mbabane",
	},
	"FDNS": {
		ICAO:      "FDNS",
		Name:      "Nsoko Airport",
		City:      "Nsoko",
		Country:   "SZ",
		Elevation: 614,
		Latitude:  -27.0333003998,
		Longitude: 31.9333324432,
		Timezone:  "Africa/Mbabane",
	},
	"FDSM": {
		ICAO:      "FDSM",
		Name:      "Simunye Airport",
		City:      "Simunye",
		State:     "Lubombo",
		Country:   "SZ",
		Elevation: 673,
		Latitude:  -26.2000007629,
		Longitude: 31.9333324432,
		Timezone:  "Africa/Mbabane",
	},
	"FDST": {
		ICAO:      "FDST",
		Name:      "Siteki Airport",
		City:      "Siteki",
		State:     "Lubombo",
		Country:   "SZ",
		Elevation: 2220,
		Latitude:  -26.4500007629,
		Longitude: 31.9500007629,
		Timezone:  "Africa/Mbabane",
	},
	"FDTM": {
		ICAO:      "FDTM",
		Name:      "Tambankulu Airport",
		City:      "Tambankulu",
		Country:   "SZ",
		Elevation: 811,
		Latitude:  -26.1000003815,
		Longitude: 31.9666805267,
		Timezone:  "Africa/Mbabane",
	},
	"FDTS": {
		ICAO:      "FDTS",
		Name:      "Tshaneni Airport",
		City:      "Tshaneni",
		State:     "Hhohho",
		Country:   "SZ",
		Elevation: 1020,
		Latitude:  -25.9833335876,
		Longitude: 31.2000007629,
		Timezone:  "Africa/Mbabane",
	},
	"FDUB": {
		ICAO:      "FDUB",
		Name:      "Ubombo Ranches Airport",
		City:      "Big Bend",
		Country:   "SZ",
		Latitude:  -26.7707996368,
		Longitude: 31.9363994598,
		Timezone:  "Africa/Mbabane",
	},
	"FEFA": {
		ICAO:      "FEFA",
		Name:      "Alindao Airport",
		City:      "Alindao",
		State:     "Basse-Kotto",
		Country:   "CF",
		Elevation: 1470,
		Latitude:  5.0163898468,
		Longitude: 21.1996994019,
		Timezone:  "Africa/Bangui",
	},
	"FEFB": {
		ICAO:      "FEFB",
		Name:      "Poste Airport",
		City:      "Obo",
		State:     "Haut-Mbomou",
		Country:   "CF",
		Latitude:  5.403888889,
		Longitude: 26.487222222,
		Timezone:  "Africa/Bangui",
	},
	"FEFC": {
		ICAO:      "FEFC",
		IATA:      "CRF",
		Name:      "Carnot Airport",
		City:      "Carnot",
		State:     "Mambere-Kadei",
		Country:   "CF",
		Elevation: 1985,
		Latitude:  4.9369997978,
		Longitude: 15.8940000534,
		Timezone:  "Africa/Bangui",
	},
	"FEFD": {
		ICAO:      "FEFD",
		Name:      "Damara Airport",
		City:      "Damara",
		State:     "Ombella-M'Poko",
		Country:   "CF",
		Elevation: 1427,
		Latitude:  4.9499998093,
		Longitude: 18.7000007629,
		Timezone:  "Africa/Bangui",
	},
	"FEFE": {
		ICAO:      "FEFE",
		Name:      "Mobaye Mbanga Airport",
		City:      "Mobaye Mbanga",
		State:     "Basse-Kotto",
		Country:   "CF",
		Elevation: 1332,
		Latitude:  4.3746500015,
		Longitude: 21.1319007874,
		Timezone:  "Africa/Bangui",
	},
	"FEFF": {
		ICAO:      "FEFF",
		IATA:      "BGF",
		Name:      "Bangui M'Poko International Airport",
		City:      "Bangui",
		State:     "Ombella-M'Poko",
		Country:   "CF",
		Elevation: 1208,
		Latitude:  4.3984799385,
		Longitude: 18.5188007355,
		Timezone:  "Africa/Bangui",
	},
	"FEFG": {
		ICAO:      "FEFG",
		IATA:      "BGU",
		Name:      "Bangassou Airport",
		City:      "Bangassou",
		Country:   "CF",
		Elevation: 1706,
		Latitude:  4.7849998474,
		Longitude: 22.7810001373,
		Timezone:  "Africa/Bangui",
	},
	"FEFI": {
		ICAO:      "FEFI",
		IATA:      "IRO",
		Name:      "Birao Airport",
		City:      "Birao",
		State:     "Vakaga",
		Country:   "CF",
		Elevation: 1696,
		Latitude:  10.2363996506,
		Longitude: 22.7168998718,
		Timezone:  "Africa/Bangui",
	},
	"FEFK": {
		ICAO:      "FEFK",
		Name:      "Kembe Airport",
		City:      "Kembe",
		State:     "Basse-Kotto",
		Country:   "CF",
		Elevation: 1913,
		Latitude:  4.5999999046,
		Longitude: 21.8670005798,
		Timezone:  "Africa/Bangui",
	},
	"FEFL": {
		ICAO:      "FEFL",
		IATA:      "BEM",
		Name:      "Bossembele Airport",
		City:      "Bossembele",
		State:     "Ombella-M'Poko",
		Country:   "CF",
		Elevation: 2231,
		Latitude:  5.2670001984,
		Longitude: 17.6329994202,
		Timezone:  "Africa/Bangui",
	},
	"FEFM": {
		ICAO:      "FEFM",
		IATA:      "BBY",
		Name:      "Bambari Airport",
		City:      "Bambari",
		State:     "Ouaka",
		Country:   "CF",
		Elevation: 1549,
		Latitude:  5.8469400406,
		Longitude: 20.6474990845,
		Timezone:  "Africa/Bangui",
	},
	"FEFN": {
		ICAO:      "FEFN",
		IATA:      "NDL",
		Name:      "N'Dele Airport",
		City:      "N'Dele",
		State:     "Bamingui-Bangoran",
		Country:   "CF",
		Elevation: 1631,
		Latitude:  8.4272060394,
		Longitude: 20.6351566315,
		Timezone:  "Africa/Bangui",
	},
	"FEFO": {
		ICAO:      "FEFO",
		IATA:      "BOP",
		Name:      "Bouar Airport",
		City:      "Bouar",
		State:     "Nana-Mambere",
		Country:   "CF",
		Elevation: 3360,
		Latitude:  5.9580001831,
		Longitude: 15.6370000839,
		Timezone:  "Africa/Bangui",
	},
	"FEFP": {
		ICAO:      "FEFP",
		Name:      "Paoua Airport",
		City:      "Paoua",
		State:     "Ouham-Pende",
		Country:   "CF",
		Elevation: 2037,
		Latitude:  7.25,
		Longitude: 16.4330005646,
		Timezone:  "Africa/Bangui",
	},
	"FEFQ": {
		ICAO:      "FEFQ",
		Name:      "Kaga-Bandoro Airport",
		City:      "Kaga-Bandoro",
		State:     "Nana-Grebizi",
		Country:   "CF",
		Elevation: 1476,
		Latitude:  6.9829998016,
		Longitude: 19.1830005646,
		Timezone:  "Africa/Bangui",
	},
	"FEFR": {
		ICAO:      "FEFR",
		IATA:      "BIV",
		Name:      "Bria Airport",
		City:      "Bria",
		State:     "Haute-Kotto",
		Country:   "CF",
		Elevation: 1975,
		Latitude:  6.527780056,
		Longitude: 21.9894008636,
		Timezone:  "Africa/Bangui",
	},
	"FEFS": {
		ICAO:      "FEFS",
		IATA:      "BSN",
		Name:      "Bossangoa Airport",
		City:      "Bossangoa",
		State:     "Ouham",
		Country:   "CF",
		Elevation: 1637,
		Latitude:  6.492000103,
		Longitude: 17.4290008545,
		Timezone:  "Africa/Bangui",
	},
	"FEFT": {
		ICAO:      "FEFT",
		IATA:      "BBT",
		Name:      "Berberati Airport",
		City:      "Berberati",
		State:     "Mambere-Kadei",
		Country:   "CF",
		Elevation: 1929,
		Latitude:  4.2215800285,
		Longitude: 15.7863998413,
		Timezone:  "Africa/Bangui",
	},
	"FEFU": {
		ICAO:      "FEFU",
		Name:      "Sibut Airport",
		City:      "Sibut",
		State:     "Kemo",
		Country:   "CF",
		Elevation: 1411,
		Latitude:  5.7329998016,
		Longitude: 19.0830001831,
		Timezone:  "Africa/Bangui",
	},
	"FEFW": {
		ICAO:      "FEFW",
		IATA:      "ODA",
		Name:      "Ouadda Airport",
		City:      "Ouadda",
		State:     "Haute-Kotto",
		Country:   "CF",
		Elevation: 2461,
		Latitude:  8.0105600357,
		Longitude: 22.3985996246,
		Timezone:  "Africa/Bangui",
	},
	"FEFY": {
		ICAO:      "FEFY",
		IATA:      "AIG",
		Name:      "Yalinga Airport",
		City:      "Yalinga",
		State:     "Haute-Kotto",
		Country:   "CF",
		Elevation: 1975,
		Latitude:  6.5199999809,
		Longitude: 23.2600002289,
		Timezone:  "Africa/Bangui",
	},
	"FEFZ": {
		ICAO:      "FEFZ",
		IATA:      "IMO",
		Name:      "Zemio Airport",
		City:      "Zemio",
		State:     "Haut-Mbomou",
		Country:   "CF",
		Elevation: 1995,
		Latitude:  5.0500001907,
		Longitude: 25.1499996185,
		Timezone:  "Africa/Bangui",
	},
	"FEGB": {
		ICAO:      "FEGB",
		Name:      "Bambouli Airport",
		City:      "Bambouli",
		State:     "Haut-Mbomou",
		Country:   "CF",
		Elevation: 2264,
		Latitude:  5.4330000877,
		Longitude: 27.216999054,
		Timezone:  "Africa/Bangui",
	},
	"FEGC": {
		ICAO:      "FEGC",
		Name:      "Bocaranga Airport",
		City:      "Bocaranga",
		State:     "Ouham-Pende",
		Country:   "CF",
		Elevation: 3464,
		Latitude:  6.9169998169,
		Longitude: 15.6169996262,
		Timezone:  "Africa/Bangui",
	},
	"FEGD": {
		ICAO:      "FEGD",
		Name:      "Dekoa Airport",
		City:      "Dekoa",
		State:     "Kemo",
		Country:   "CF",
		Elevation: 1804,
		Latitude:  6.3000001907,
		Longitude: 19.0830001831,
		Timezone:  "Africa/Bangui",
	},
	"FEGE": {
		ICAO:      "FEGE",
		IATA:      "MKI",
		Name:      "M'Boki Airport",
		City:      "Mboki",
		State:     "Haut-Mbomou",
		Country:   "CF",
		Elevation: 1969,
		Latitude:  5.3330101967,
		Longitude: 25.9319000244,
		Timezone:  "Africa/Bangui",
	},
	"FEGF": {
		ICAO:      "FEGF",
		IATA:      "BTG",
		Name:      "Batangafo Airport",
		City:      "Batangafo",
		State:     "Ouham",
		Country:   "CF",
		Elevation: 1378,
		Latitude:  7.3141098022,
		Longitude: 18.3087997437,
		Timezone:  "Africa/Bangui",
	},
	"FEGG": {
		ICAO:      "FEGG",
		Name:      "Gamboula Airport",
		City:      "Gamboula",
		Country:   "CF",
		Elevation: 2001,
		Latitude:  4.132999897,
		Longitude: 15.1499996185,
		Timezone:  "Africa/Bangui",
	},
	"FEGI": {
		ICAO:      "FEGI",
		Name:      "Grimari Airport",
		City:      "Grimari",
		State:     "Ouaka",
		Country:   "CF",
		Elevation: 1312,
		Latitude:  5.6830000877,
		Longitude: 20.0499992371,
		Timezone:  "Africa/Bangui",
	},
	"FEGL": {
		ICAO:      "FEGL",
		IATA:      "GDI",
		Name:      "Gordil Airport",
		City:      "Melle",
		State:     "Vakaga",
		Country:   "CF",
		Elevation: 1427,
		Latitude:  9.5830001831,
		Longitude: 21.7329998016,
		Timezone:  "Africa/Bangui",
	},
	"FEGM": {
		ICAO:      "FEGM",
		IATA:      "BMF",
		Name:      "Bakouma Airport",
		City:      "Bakouma",
		State:     "Mbomou",
		Country:   "CF",
		Elevation: 1640,
		Latitude:  5.6939997673,
		Longitude: 22.8010005951,
		Timezone:  "Africa/Bangui",
	},
	"FEGO": {
		ICAO:      "FEGO",
		IATA:      "ODJ",
		Name:      "Ouanda Djalle Airport",
		City:      "Ouanda Djalle",
		State:     "Vakaga",
		Country:   "CF",
		Elevation: 1985,
		Latitude:  8.8999996185,
		Longitude: 22.783000946,
		Timezone:  "Africa/Bangui",
	},
	"FEGR": {
		ICAO:      "FEGR",
		IATA:      "RFA",
		Name:      "Rafai Airport",
		City:      "Rafai",
		State:     "Mbomou",
		Country:   "CF",
		Elevation: 1759,
		Latitude:  4.9886097908,
		Longitude: 23.9277992249,
		Timezone:  "Africa/Bangui",
	},
	"FEGU": {
		ICAO:      "FEGU",
		IATA:      "BCF",
		Name:      "Bouca Airport",
		City:      "Bouca",
		State:     "Ouham",
		Country:   "CF",
		Elevation: 1532,
		Latitude:  6.5170001984,
		Longitude: 18.2670001984,
		Timezone:  "Africa/Bangui",
	},
	"FEGZ": {
		ICAO:      "FEGZ",
		IATA:      "BOZ",
		Name:      "Bozoum Airport",
		City:      "Bozoum",
		State:     "Ouham-Pende",
		Country:   "CF",
		Elevation: 2215,
		Latitude:  6.3441700935,
		Longitude: 16.3218994141,
		Timezone:  "Africa/Bangui",
	},
	"FGBT": {
		ICAO:      "FGBT",
		IATA:      "BSG",
		Name:      "Bata Airport",
		State:     "Litoral",
		Country:   "GQ",
		Elevation: 13,
		Latitude:  1.9054700136,
		Longitude: 9.805680275,
		Timezone:  "Africa/Malabo",
	},
	"FGSL": {
		ICAO:      "FGSL",
		IATA:      "SSG",
		Name:      "Malabo Airport",
		City:      "Malabo",
		Country:   "GQ",
		Elevation: 76,
		Latitude:  3.7552700043,
		Longitude: 8.7087202072,
		Timezone:  "Africa/Malabo",
	},
	"FHAW": {
		ICAO:      "FHAW",
		IATA:      "ASI",
		Name:      "RAF Ascension Island",
		City:      "Ascension Island",
		State:     "Ascension",
		Country:   "SH",
		Elevation: 278,
		Latitude:  -7.9696002007,
		Longitude: -14.393699646,
		Timezone:  "Atlantic/St_Helena",
	},
	"FIMA": {
		ICAO:      "FIMA",
		Name:      "Agalega Island Airstrip",
		City:      "Vingt Cinq",
		State:     "Agalega-Islands",
		Country:   "MU",
		Elevation: 4,
		Latitude:  -10.3731002808,
		Longitude: 56.6097984314,
		Timezone:  "Indian/Mauritius",
	},
	"FIMP": {
		ICAO:      "FIMP",
		IATA:      "MRU",
		Name:      "Sir Seewoosagur Ramgoolam International Airport",
		City:      "Port Louis",
		State:     "Grand-Port",
		Country:   "MU",
		Elevation: 186,
		Latitude:  -20.4302005768,
		Longitude: 57.6836013794,
		Timezone:  "Indian/Mauritius",
	},
	"FIMR": {
		ICAO:      "FIMR",
		IATA:      "RRG",
		Name:      "Sir Charles Gaetan Duval Airport",
		City:      "Port Mathurin",
		State:     "Rodrigues",
		Country:   "MU",
		Elevation: 95,
		Latitude:  -19.7576999664,
		Longitude: 63.361000061,
		Timezone:  "Indian/Mauritius",
	},
	"FJDG": {
		ICAO:      "FJDG",
		IATA:      "NKW",
		Name:      "Diego Garcia Naval Support Facility",
		City:      "Diego Garcia",
		Country:   "IO",
		Elevation: 9,
		Latitude:  -7.313270092,
		Longitude: 72.4111022949,
		Timezone:  "Indian/Chagos",
	},
	"FKAB": {
		ICAO:      "FKAB",
		Name:      "Banyo Airport",
		City:      "Banyo",
		State:     "Adamaoua",
		Country:   "CM",
		Elevation: 3642,
		Latitude:  6.7749900818,
		Longitude: 11.8070001602,
		Timezone:  "Africa/Douala",
	},
	"FKAF": {
		ICAO:      "FKAF",
		Name:      "Bafia Airport",
		City:      "Bafia",
		State:     "Centre",
		Country:   "CM",
		Elevation: 1591,
		Latitude:  4.7670001984,
		Longitude: 11.2170000076,
		Timezone:  "Africa/Douala",
	},
	"FKAG": {
		ICAO:      "FKAG",
		Name:      "Abong M'bang Airport",
		City:      "Abong M'bang",
		State:     "East",
		Country:   "CM",
		Elevation: 2297,
		Latitude:  3.9670000076,
		Longitude: 13.1999998093,
		Timezone:  "Africa/Douala",
	},
	"FKAN": {
		ICAO:      "FKAN",
		IATA:      "NKS",
		Name:      "Nkongsamba Airport",
		City:      "Nkongsamba",
		State:     "Littoral",
		Country:   "CM",
		Elevation: 2641,
		Latitude:  4.9499998093,
		Longitude: 9.9329996109,
		Timezone:  "Africa/Douala",
	},
	"FKAO": {
		ICAO:      "FKAO",
		Name:      "Betare Oya Airport",
		City:      "Betare Oya",
		State:     "East",
		Country:   "CM",
		Elevation: 3002,
		Latitude:  5.5041699409,
		Longitude: 14.0982999802,
		Timezone:  "Africa/Douala",
	},
	"FKKB": {
		ICAO:      "FKKB",
		IATA:      "KBI",
		Name:      "Kribi Airport",
		City:      "Kribi",
		State:     "South",
		Country:   "CM",
		Elevation: 148,
		Latitude:  2.8738899231,
		Longitude: 9.9777803421,
		Timezone:  "Africa/Douala",
	},
	"FKKC": {
		ICAO:      "FKKC",
		IATA:      "TKC",
		Name:      "Tiko Airport",
		City:      "Tiko",
		State:     "South-West",
		Country:   "CM",
		Elevation: 151,
		Latitude:  4.0891900063,
		Longitude: 9.3605298996,
		Timezone:  "Africa/Douala",
	},
	"FKKD": {
		ICAO:      "FKKD",
		IATA:      "DLA",
		Name:      "Douala International Airport",
		City:      "Douala",
		State:     "Littoral",
		Country:   "CM",
		Elevation: 33,
		Latitude:  4.0060801506,
		Longitude: 9.7194795609,
		Timezone:  "Africa/Douala",
	},
	"FKKE": {
		ICAO:      "FKKE",
		Name:      "Eseka Airport",
		City:      "Eseka",
		State:     "Centre",
		Country:   "CM",
		Elevation: 738,
		Latitude:  3.6500000954,
		Longitude: 10.7829999924,
		Timezone:  "Africa/Douala",
	},
	"FKKF": {
		ICAO:      "FKKF",
		IATA:      "MMF",
		Name:      "Mamfe Airport",
		City:      "Mamfe",
		State:     "South-West",
		Country:   "CM",
		Elevation: 413,
		Latitude:  5.7041702271,
		Longitude: 9.3063898087,
		Timezone:  "Africa/Douala",
	},
	"FKKG": {
		ICAO:      "FKKG",
		IATA:      "BLC",
		Name:      "Bali Airport",
		City:      "Bali",
		State:     "North-West",
		Country:   "CM",
		Elevation: 4465,
		Latitude:  5.8952777778,
		Longitude: 10.0338888889,
		Timezone:  "Africa/Douala",
	},
	"FKKH": {
		ICAO:      "FKKH",
		IATA:      "KLE",
		Name:      "Kaele Airport",
		City:      "Kaele",
		State:     "Far-North",
		Country:   "CM",
		Elevation: 1276,
		Latitude:  10.092499733,
		Longitude: 14.445599556,
		Timezone:  "Africa/Douala",
	},
	"FKKI": {
		ICAO:      "FKKI",
		IATA:      "OUR",
		Name:      "Batouri Airport",
		City:      "Batouri",
		State:     "East",
		Country:   "CM",
		Elevation: 2152,
		Latitude:  4.4749999046,
		Longitude: 14.3625001907,
		Timezone:  "Africa/Douala",
	},
	"FKKJ": {
		ICAO:      "FKKJ",
		IATA:      "GXX",
		Name:      "Yagoua Airport",
		City:      "Yagoua",
		State:     "Far-North",
		Country:   "CM",
		Latitude:  10.3561000824,
		Longitude: 15.2371997833,
		Timezone:  "Africa/Douala",
	},
	"FKKL": {
		ICAO:      "FKKL",
		IATA:      "MVR",
		Name:      "Salak Airport",
		City:      "Maroua",
		State:     "Far-North",
		Country:   "CM",
		Elevation: 1390,
		Latitude:  10.4513998032,
		Longitude: 14.257399559,
		Timezone:  "Africa/Douala",
	},
	"FKKM": {
		ICAO:      "FKKM",
		IATA:      "FOM",
		Name:      "Foumban Nkounja Airport",
		City:      "Foumban",
		State:     "West",
		Country:   "CM",
		Elevation: 3963,
		Latitude:  5.6369199753,
		Longitude: 10.7508001328,
		Timezone:  "Africa/Douala",
	},
	"FKKN": {
		ICAO:      "FKKN",
		IATA:      "NGE",
		Name:      "N'Gaoundere Airport",
		City:      "N'Gaoundere",
		State:     "Adamaoua",
		Country:   "CM",
		Elevation: 3655,
		Latitude:  7.3570098877,
		Longitude: 13.5592002869,
		Timezone:  "Africa/Douala",
	},
	"FKKO": {
		ICAO:      "FKKO",
		IATA:      "BTA",
		Name:      "Bertoua Airport",
		City:      "Bertoua",
		State:     "East",
		Country:   "CM",
		Elevation: 2133,
		Latitude:  4.5486102104,
		Longitude: 13.726099968,
		Timezone:  "Africa/Douala",
	},
	"FKKR": {
		ICAO:      "FKKR",
		IATA:      "GOU",
		Name:      "Garoua International Airport",
		City:      "Garoua",
		State:     "North",
		Country:   "CM",
		Elevation: 794,
		Latitude:  9.3358898163,
		Longitude: 13.3701000214,
		Timezone:  "Africa/Douala",
	},
	"FKKS": {
		ICAO:      "FKKS",
		IATA:      "DSC",
		Name:      "Dschang Airport",
		City:      "Dschang",
		State:     "West",
		Country:   "CM",
		Elevation: 4593,
		Latitude:  5.4499998093,
		Longitude: 10.0670003891,
		Timezone:  "Africa/Douala",
	},
	"FKKT": {
		ICAO:      "FKKT",
		Name:      "Tibati Airport",
		City:      "Tibati",
		State:     "Adamaoua",
		Country:   "CM",
		Elevation: 2854,
		Latitude:  6.4829998016,
		Longitude: 12.6330003738,
		Timezone:  "Africa/Douala",
	},
	"FKKU": {
		ICAO:      "FKKU",
		IATA:      "BFX",
		Name:      "Bafoussam Airport",
		City:      "Bafoussam",
		State:     "West",
		Country:   "CM",
		Elevation: 4347,
		Latitude:  5.5369200707,
		Longitude: 10.3545999527,
		Timezone:  "Africa/Douala",
	},
	"FKKV": {
		ICAO:      "FKKV",
		IATA:      "BPC",
		Name:      "Bamenda Airport",
		City:      "Bamenda",
		State:     "North-West",
		Country:   "CM",
		Elevation: 4065,
		Latitude:  6.0392398834,
		Longitude: 10.1225996017,
		Timezone:  "Africa/Douala",
	},
	"FKKW": {
		ICAO:      "FKKW",
		IATA:      "EBW",
		Name:      "Ebolowa Airport",
		City:      "Ebolowa",
		State:     "South",
		Country:   "CM",
		Elevation: 1975,
		Latitude:  2.8759999275,
		Longitude: 11.1850004196,
		Timezone:  "Africa/Douala",
	},
	"FKKY": {
		ICAO:      "FKKY",
		IATA:      "YAO",
		Name:      "Yaounde Airport",
		City:      "Yaounde",
		State:     "Centre",
		Country:   "CM",
		Elevation: 2464,
		Latitude:  3.83604002,
		Longitude: 11.5235004425,
		Timezone:  "Africa/Douala",
	},
	"FKYS": {
		ICAO:      "FKYS",
		IATA:      "NSI",
		Name:      "Yaounde Nsimalen International Airport",
		City:      "Yaounde",
		State:     "Centre",
		Country:   "CM",
		Elevation: 2278,
		Latitude:  3.7225599289,
		Longitude: 11.5532999039,
		Timezone:  "Africa/Douala",
	},
	"FL00": {
		ICAO:      "FL00",
		Name:      "Griffins Peace River Ranch Airport",
		City:      "Zolfo Springs",
		State:     "Florida",
		Country:   "US",
		Elevation: 65,
		Latitude:  27.4188995361,
		Longitude: -81.8326034546,
		Timezone:  "America/New_York",
	},
	"FL01": {
		ICAO:      "FL01",
		Name:      "Crews Homestead Ranch Airport",
		City:      "Avon Park",
		State:     "Florida",
		Country:   "US",
		Elevation: 90,
		Latitude:  27.621099472,
		Longitude: -81.6106033325,
		Timezone:  "America/New_York",
	},
	"FL02": {
		ICAO:      "FL02",
		Name:      "Fox Field",
		City:      "Branford",
		State:     "Florida",
		Country:   "US",
		Elevation: 72,
		Latitude:  29.8894004822,
		Longitude: -82.7986984253,
		Timezone:  "America/New_York",
	},
	"FL03": {
		ICAO:      "FL03",
		Name:      "Department of Corrections Field",
		City:      "Lake Butler",
		State:     "Florida",
		Country:   "US",
		Elevation: 131,
		Latitude:  29.9918994904,
		Longitude: -82.3665008545,
		Timezone:  "America/New_York",
	},
	"FL05": {
		ICAO:      "FL05",
		Name:      "Hill Landing Strip",
		City:      "Lake City",
		State:     "Florida",
		Country:   "US",
		Elevation: 160,
		Latitude:  30.1755008698,
		Longitude: -82.6064987183,
		Timezone:  "America/New_York",
	},
	"FL06": {
		ICAO:      "FL06",
		Name:      "Golden Harvest Flying Svc Inc Airport",
		City:      "Jay",
		State:     "Florida",
		Country:   "US",
		Elevation: 255,
		Latitude:  30.9029998779,
		Longitude: -87.0438995361,
		Timezone:  "America/Chicago",
	},
	"FL07": {
		ICAO:      "FL07",
		Name:      "Wings N Sunsets LLC Airport",
		City:      "Live Oak",
		State:     "Florida",
		Country:   "US",
		Elevation: 130,
		Latitude:  30.2502002716,
		Longitude: -82.9240036011,
		Timezone:  "America/New_York",
	},
	"FL08": {
		ICAO:      "FL08",
		Name:      "Florida Sheriffs Boys Ranch Airport",
		City:      "Live Oak",
		State:     "Florida",
		Country:   "US",
		Elevation: 93,
		Latitude:  30.4155006409,
		Longitude: -83.0165023804,
		Timezone:  "America/New_York",
	},
	"FL09": {
		ICAO:      "FL09",
		Name:      "Kittyhawk Estates Airport",
		City:      "Live Oak",
		State:     "Florida",
		Country:   "US",
		Elevation: 90,
		Latitude:  30.3388004303,
		Longitude: -83.1443023682,
		Timezone:  "America/New_York",
	},
	"FL10": {
		ICAO:      "FL10",
		Name:      "Little River Airport",
		City:      "Mc Alpin",
		State:     "Florida",
		Country:   "US",
		Elevation: 90,
		Latitude:  30.1149997711,
		Longitude: -82.908996582,
		Timezone:  "America/New_York",
	},
	"FL11": {
		ICAO:      "FL11",
		Name:      "Thrifts Airport",
		City:      "Macclenny",
		State:     "Florida",
		Country:   "US",
		Elevation: 100,
		Latitude:  30.3493995667,
		Longitude: -82.1209030151,
		Timezone:  "America/New_York",
	},
	"FL12": {
		ICAO:      "FL12",
		Name:      "Ingalls Field",
		City:      "Miccosukee",
		State:     "Florida",
		Country:   "US",
		Elevation: 183,
		Latitude:  30.5904998779,
		Longitude: -84.0304031372,
		Timezone:  "America/New_York",
	},
	"FL13": {
		ICAO:      "FL13",
		Name:      "Spencer's Airpark",
		City:      "Middleburg",
		State:     "Florida",
		Country:   "US",
		Elevation: 150,
		Latitude:  30.0685997009,
		Longitude: -81.9607009888,
		Timezone:  "America/New_York",
	},
	"FL14": {
		ICAO:      "FL14",
		Name:      "Vosika's Airport",
		City:      "Chipley",
		State:     "Florida",
		Country:   "US",
		Elevation: 69,
		Latitude:  30.7798995972,
		Longitude: -85.5979995728,
		Timezone:  "America/Chicago",
	},
	"FL15": {
		ICAO:      "FL15",
		Name:      "Steep Head Farm Airport",
		City:      "Clarksville",
		State:     "Florida",
		Country:   "US",
		Elevation: 60,
		Latitude:  30.4202003479,
		Longitude: -85.3004989624,
		Timezone:  "America/Chicago",
	},
	"FL16": {
		ICAO:      "FL16",
		Name:      "Market World Airport",
		City:      "Auburndale",
		State:     "Florida",
		Country:   "US",
		Elevation: 125,
		Latitude:  28.058599472,
		Longitude: -81.8164978027,
		Timezone:  "America/New_York",
	},
	"FL17": {
		ICAO:      "FL17",
		Name:      "Ruckel Airport",
		City:      "Niceville",
		State:     "Florida",
		Country:   "US",
		Elevation: 67,
		Latitude:  30.5195999146,
		Longitude: -86.4383010864,
		Timezone:  "America/Chicago",
	},
	"FL18": {
		ICAO:      "FL18",
		Name:      "Suwannee Farms Airport",
		City:      "O'Brien",
		State:     "Florida",
		Country:   "US",
		Elevation: 50,
		Latitude:  30.0904998779,
		Longitude: -83.0546035767,
		Timezone:  "America/New_York",
	},
	"FL19": {
		ICAO:      "FL19",
		Name:      "Crosswind Farm Airport",
		City:      "Ocala",
		State:     "Florida",
		Country:   "US",
		Elevation: 74,
		Latitude:  29.245300293,
		Longitude: -82.3536987305,
		Timezone:  "America/New_York",
	},
	"FL23": {
		ICAO:      "FL23",
		Name:      "Strazzulla Groves Airport",
		City:      "Fort Pierce",
		State:     "Florida",
		Country:   "US",
		Elevation: 27,
		Latitude:  27.4974994659,
		Longitude: -80.529800415,
		Timezone:  "America/New_York",
	},
	"FL27": {
		ICAO:      "FL27",
		Name:      "Lake City Airpark",
		City:      "Lake City",
		State:     "Florida",
		Country:   "US",
		Elevation: 120,
		Latitude:  30.043800354,
		Longitude: -82.6039962769,
		Timezone:  "America/New_York",
	},
	"FL28": {
		ICAO:      "FL28",
		Name:      "State Prison Field",
		City:      "Raiford",
		State:     "Florida",
		Country:   "US",
		Elevation: 123,
		Latitude:  30.0557994843,
		Longitude: -82.1723022461,
		Timezone:  "America/New_York",
	},
	"FL29": {
		ICAO:      "FL29",
		Name:      "Hales 700 Airport",
		City:      "Fort Pierce",
		State:     "Florida",
		Country:   "US",
		Elevation: 26,
		Latitude:  27.3612003326,
		Longitude: -80.5708999634,
		Timezone:  "America/New_York",
	},
	"FL31": {
		ICAO:      "FL31",
		Name:      "Mjd STOLport",
		City:      "Miami",
		State:     "Florida",
		Country:   "US",
		Elevation: 6,
		Latitude:  25.554599762,
		Longitude: -80.5539016724,
		Timezone:  "America/New_York",
	},
	"FL33": {
		ICAO:      "FL33",
		Name:      "Watson Flight Strip",
		City:      "Trenton",
		State:     "Florida",
		Country:   "US",
		Elevation: 55,
		Latitude:  29.6683006287,
		Longitude: -82.8262023926,
		Timezone:  "America/New_York",
	},
	"FL34": {
		ICAO:      "FL34",
		Name:      "Eglin Test Site B6 Airport",
		City:      "Valparaiso",
		State:     "Florida",
		Country:   "US",
		Elevation: 120,
		Latitude:  30.6319007874,
		Longitude: -86.7472000122,
		Timezone:  "America/Chicago",
	},
	"FL35": {
		ICAO:      "FL35",
		Name:      "Geraci Airpark",
		City:      "Lutz",
		State:     "Florida",
		Country:   "US",
		Elevation: 65,
		Latitude:  28.1749992371,
		Longitude: -82.48777771,
		Timezone:  "America/New_York",
	},
	"FL36": {
		ICAO:      "FL36",
		Name:      "Tocoi Airport",
		City:      "St. Augustine",
		State:     "Florida",
		Country:   "US",
		Elevation: 35,
		Latitude:  29.867472,
		Longitude: -81.367361,
		Timezone:  "America/New_York",
	},
	"FL37": {
		ICAO:      "FL37",
		Name:      "Treasure Coast Airpark",
		City:      "Fort Pierce",
		State:     "Florida",
		Country:   "US",
		Elevation: 30,
		Latitude:  27.2383995056,
		Longitude: -80.4914016724,
		Timezone:  "America/New_York",
	},
	"FL40": {
		ICAO:      "FL40",
		Name:      "Graham Landing Strip - Moore Haven Airport",
		City:      "Moore Haven",
		State:     "Florida",
		Country:   "US",
		Elevation: 18,
		Latitude:  26.8409004211,
		Longitude: -81.1517028809,
		Timezone:  "America/New_York",
	},
	"FL41": {
		ICAO:      "FL41",
		Name:      "Okeelanta Airport",
		City:      "South Bay",
		State:     "Florida",
		Country:   "US",
		Elevation: 14,
		Latitude:  26.581199646,
		Longitude: -80.755897522,
		Timezone:  "America/New_York",
	},
	"FL43": {
		ICAO:      "FL43",
		Name:      "Burntwood Ranch Airport",
		City:      "Lakeland",
		State:     "Florida",
		Country:   "US",
		Elevation: 119,
		Latitude:  28.2581005096,
		Longitude: -81.9306030273,
		Timezone:  "America/New_York",
	},
	"FL46": {
		ICAO:      "FL46",
		Name:      "Bartram Farms Air Park",
		City:      "Saint Augustine",
		State:     "Florida",
		Country:   "US",
		Elevation: 17,
		Latitude:  29.821178,
		Longitude: -81.497058,
		Timezone:  "America/New_York",
	},
	"FL47": {
		ICAO:      "FL47",
		Name:      "Ashley Field",
		City:      "Eustis",
		State:     "Florida",
		Country:   "US",
		Elevation: 80,
		Latitude:  28.8924999237,
		Longitude: -81.5569000244,
		Timezone:  "America/New_York",
	},
	"FL48": {
		ICAO:      "FL48",
		Name:      "Sierra Airpark",
		City:      "Worthington Springs",
		State:     "Florida",
		Country:   "US",
		Elevation: 115,
		Latitude:  28.8822002411,
		Longitude: -82.4297027588,
		Timezone:  "America/New_York",
	},
	"FL50": {
		ICAO:      "FL50",
		Name:      "Rossi Field",
		City:      "Lake City",
		State:     "Florida",
		Country:   "US",
		Elevation: 183,
		Latitude:  30.1466007233,
		Longitude: -82.572303772,
		Timezone:  "America/New_York",
	},
	"FL52": {
		ICAO:      "FL52",
		Name:      "Angel's Field",
		City:      "Tallahassee",
		State:     "Florida",
		Country:   "US",
		Elevation: 160,
		Latitude:  30.4810009,
		Longitude: -84.08679962,
		Timezone:  "America/New_York",
	},
	"FL54": {
		ICAO:      "FL54",
		Name:      "Flying Tiger Field",
		City:      "Worthington",
		State:     "Florida",
		Country:   "US",
		Elevation: 115,
		Latitude:  29.9354991913,
		Longitude: -82.410697937,
		Timezone:  "America/New_York",
	},
	"FL56": {
		ICAO:      "FL56",
		Name:      "Williams Hawgwild Airport",
		City:      "Fort Pierce",
		State:     "Florida",
		Country:   "US",
		Elevation: 20,
		Latitude:  27.4088993073,
		Longitude: -80.4278030396,
		Timezone:  "America/New_York",
	},
	"FL57": {
		ICAO:      "FL57",
		Name:      "Carter Airport",
		City:      "Apopka",
		State:     "Florida",
		Country:   "US",
		Elevation: 80,
		Latitude:  28.6189002991,
		Longitude: -81.5086975098,
		Timezone:  "America/New_York",
	},
	"FL58": {
		ICAO:      "FL58",
		Name:      "Johary Airport",
		City:      "Belleview",
		State:     "Florida",
		Country:   "US",
		Elevation: 72,
		Latitude:  29.0263996124,
		Longitude: -81.9878997803,
		Timezone:  "America/New_York",
	},
	"FL59": {
		ICAO:      "FL59",
		Name:      "Buckingham Field",
		City:      "Fort Myers",
		State:     "Florida",
		Country:   "US",
		Elevation: 23,
		Latitude:  26.6434001923,
		Longitude: -81.7104034424,
		Timezone:  "America/New_York",
	},
	"FL60": {
		ICAO:      "FL60",
		Name:      "Reynolds Airpark",
		City:      "Green Cove Springs",
		State:     "Florida",
		Country:   "US",
		Elevation: 21,
		Latitude:  29.9724998474,
		Longitude: -81.6608963013,
		Timezone:  "America/New_York",
	},
	"FL61": {
		ICAO:      "FL61",
		Name:      "Mc Ginley Airport",
		City:      "Ocala",
		State:     "Florida",
		Country:   "US",
		Elevation: 84,
		Latitude:  29.0261001587,
		Longitude: -82.2123031616,
		Timezone:  "America/New_York",
	},
	"FL62": {
		ICAO:      "FL62",
		Name:      "Bradshaw Farm Airport",
		City:      "Deland",
		State:     "Florida",
		Country:   "US",
		Elevation: 55,
		Latitude:  28.948600769,
		Longitude: -81.4131011963,
		Timezone:  "America/New_York",
	},
	"FL63": {
		ICAO:      "FL63",
		Name:      "Idle Wild Airport",
		City:      "Ocala",
		State:     "Florida",
		Country:   "US",
		Elevation: 80,
		Latitude:  29.289100647,
		Longitude: -82.322303772,
		Timezone:  "America/New_York",
	},
	"FL65": {
		ICAO:      "FL65",
		Name:      "Mac-Ivor & Friends Airstrip",
		City:      "Hollywood",
		State:     "Florida",
		Country:   "US",
		Elevation: 4,
		Latitude:  26.0447998047,
		Longitude: -80.3733978271,
		Timezone:  "America/New_York",
	},
	"FL66": {
		ICAO:      "FL66",
		Name:      "Peach Orchard Airport",
		City:      "Archer",
		State:     "Florida",
		Country:   "US",
		Elevation: 75,
		Latitude:  29.5597000122,
		Longitude: -82.49949646,
		Timezone:  "America/New_York",
	},
	"FL67": {
		ICAO:      "FL67",
		Name:      "Evans Properties Inc Airport",
		City:      "Fort Pierce",
		State:     "Florida",
		Country:   "US",
		Elevation: 24,
		Latitude:  27.2777996063,
		Longitude: -80.585899353,
		Timezone:  "America/New_York",
	},
	"FL73": {
		ICAO:      "FL73",
		Name:      "Lykes Palmdale Airport",
		City:      "Palmdale",
		State:     "Florida",
		Country:   "US",
		Elevation: 42,
		Latitude:  26.9477996826,
		Longitude: -81.3187026978,
		Timezone:  "America/New_York",
	},
	"FL74": {
		ICAO:      "FL74",
		Name:      "Indian River Aerodrome",
		City:      "Vero Beach",
		State:     "Florida",
		Country:   "US",
		Elevation: 25,
		Latitude:  27.5986003876,
		Longitude: -80.5019989014,
		Timezone:  "America/New_York",
	},
	"FL75": {
		ICAO:      "FL75",
		Name:      "Indian Hammock Airport",
		City:      "Fort Drum",
		State:     "Florida",
		Country:   "US",
		Elevation: 65,
		Latitude:  27.5419998169,
		Longitude: -80.8330993652,
		Timezone:  "America/New_York",
	},
	"FL76": {
		ICAO:      "FL76",
		Name:      "Seven Springs Ranch Airport",
		City:      "Ocala",
		State:     "Florida",
		Country:   "US",
		Elevation: 50,
		Latitude:  28.9638996124,
		Longitude: -82.2194976807,
		Timezone:  "America/New_York",
	},
	"FL77": {
		ICAO:      "FL77",
		Name:      "Calusa Ranch Airport",
		City:      "Miles City",
		State:     "Florida",
		Country:   "US",
		Elevation: 15,
		Latitude:  26.0545005798,
		Longitude: -81.0678024292,
		Timezone:  "America/New_York",
	},
	"FL78": {
		ICAO:      "FL78",
		Name:      "Lewis Airport",
		City:      "Brandon",
		State:     "Florida",
		Country:   "US",
		Elevation: 45,
		Latitude:  27.8985996246,
		Longitude: -82.1826019287,
		Timezone:  "America/New_York",
	},
	"FL80": {
		ICAO:      "FL80",
		Name:      "Lee Farms Airport",
		City:      "Lowell",
		State:     "Florida",
		Country:   "US",
		Elevation: 100,
		Latitude:  29.3346996307,
		Longitude: -82.1668014526,
		Timezone:  "America/New_York",
	},
	"FL81": {
		ICAO:      "FL81",
		Name:      "Triple B Airpark",
		City:      "Crestview",
		State:     "Florida",
		Country:   "US",
		Elevation: 210,
		Latitude:  30.7819004059,
		Longitude: -86.4672012329,
		Timezone:  "America/Chicago",
	},
	"FL82": {
		ICAO:      "FL82",
		Name:      "Oak Ridge Airport",
		City:      "Interlachen",
		State:     "Florida",
		Country:   "US",
		Elevation: 72,
		Latitude:  29.580536,
		Longitude: -81.861181,
		Timezone:  "America/New_York",
	},
	"FL84": {
		ICAO:      "FL84",
		Name:      "Flying-A-Ranch Airport",
		City:      "Fernandina Beach",
		State:     "Florida",
		Country:   "US",
		Elevation: 25,
		Latitude:  30.6011009216,
		Longitude: -81.5703964233,
		Timezone:  "America/New_York",
	},
	"FL86": {
		ICAO:      "FL86",
		Name:      "Woodstock Airport",
		City:      "Fort Myers",
		State:     "Florida",
		Country:   "US",
		Elevation: 5,
		Latitude:  26.5825996399,
		Longitude: -82.0983963013,
		Timezone:  "America/New_York",
	},
	"FL88": {
		ICAO:      "FL88",
		Name:      "Bob Paul Airport",
		City:      "La Belle",
		State:     "Florida",
		Country:   "US",
		Elevation: 28,
		Latitude:  26.6837005615,
		Longitude: -81.449798584,
		Timezone:  "America/New_York",
	},
	"FL89": {
		ICAO:      "FL89",
		Name:      "Triple M Airport",
		City:      "Vero Beach",
		State:     "Florida",
		Country:   "US",
		Elevation: 20,
		Latitude:  27.5541992188,
		Longitude: -80.4397964478,
		Timezone:  "America/New_York",
	},
	"FL90": {
		ICAO:      "FL90",
		Name:      "Salty Approach Airport",
		City:      "Fort Myers",
		State:     "Florida",
		Country:   "US",
		Elevation: 6,
		Latitude:  26.6033992767,
		Longitude: -82.2201004028,
		Timezone:  "America/New_York",
	},
	"FL91": {
		ICAO:      "FL91",
		Name:      "Hendry Correctional Institution Airport",
		City:      "Sunniland",
		State:     "Florida",
		Country:   "US",
		Elevation: 22,
		Latitude:  26.3097991943,
		Longitude: -81.2273025513,
		Timezone:  "America/New_York",
	},
	"FL95": {
		ICAO:      "FL95",
		Name:      "Broocke Air Patch Airport",
		City:      "Winter Beach",
		State:     "Florida",
		Country:   "US",
		Elevation: 22,
		Latitude:  27.7085990906,
		Longitude: -80.452796936,
		Timezone:  "America/New_York",
	},
	"FL96": {
		ICAO:      "FL96",
		Name:      "Jay Airport",
		City:      "Jay",
		State:     "Florida",
		Country:   "US",
		Elevation: 254,
		Latitude:  30.9556999207,
		Longitude: -87.1305007935,
		Timezone:  "America/Chicago",
	},
	"FL97": {
		ICAO:      "FL97",
		Name:      "Tangerine Airport",
		City:      "Zellwood",
		State:     "Florida",
		Country:   "US",
		Elevation: 125,
		Latitude:  28.7602996826,
		Longitude: -81.6059036255,
		Timezone:  "America/New_York",
	},
	"FL99": {
		ICAO:      "FL99",
		Name:      "Flying-H Airport",
		City:      "Fruitland Park",
		State:     "Florida",
		Country:   "US",
		Elevation: 80,
		Latitude:  28.8761005402,
		Longitude: -81.888999939,
		Timezone:  "America/New_York",
	},
	"FLAT": {
		ICAO:      "FLAT",
		Name:      "Katete Airport",
		City:      "Katete",
		State:     "Eastern",
		Country:   "ZM",
		Elevation: 3520,
		Latitude:  -14.120300293,
		Longitude: 32.0643997192,
		Timezone:  "Africa/Lusaka",
	},
	"FLBA": {
		ICAO:      "FLBA",
		IATA:      "MMQ",
		Name:      "Mbala Airport",
		City:      "Mbala",
		State:     "Northern",
		Country:   "ZM",
		Elevation: 5454,
		Latitude:  -8.85916996,
		Longitude: 31.3363990784,
		Timezone:  "Africa/Lusaka",
	},
	"FLCC": {
		ICAO:      "FLCC",
		Name:      "Chocha Airport",
		City:      "Chocha",
		State:     "Northern",
		Country:   "ZM",
		Elevation: 3270,
		Latitude:  -8.428730011,
		Longitude: 29.8059005737,
		Timezone:  "Africa/Lusaka",
	},
	"FLCH": {
		ICAO:      "FLCH",
		Name:      "Choma Airport",
		City:      "Choma",
		State:     "Southern",
		Country:   "ZM",
		Elevation: 4300,
		Latitude:  -16.783000946,
		Longitude: 27,
		Timezone:  "Africa/Lusaka",
	},
	"FLCP": {
		ICAO:      "FLCP",
		IATA:      "CIP",
		Name:      "Chipata Airport",
		City:      "Chipata",
		State:     "Eastern",
		Country:   "ZM",
		Elevation: 3360,
		Latitude:  -13.5583000183,
		Longitude: 32.5872001648,
		Timezone:  "Africa/Lusaka",
	},
	"FLCS": {
		ICAO:      "FLCS",
		Name:      "Chinsali Airport",
		City:      "Chinsali",
		State:     "Muchinga",
		Country:   "ZM",
		Elevation: 4350,
		Latitude:  -10.5329999924,
		Longitude: 32.0670013428,
		Timezone:  "Africa/Lusaka",
	},
	"FLEA": {
		ICAO:      "FLEA",
		Name:      "East One Airport",
		City:      "Bwambwa",
		State:     "Luapula",
		Country:   "ZM",
		Elevation: 3900,
		Latitude:  -11.6357002258,
		Longitude: 29.7359008789,
		Timezone:  "Africa/Lusaka",
	},
	"FLEB": {
		ICAO:      "FLEB",
		Name:      "East Two Airport",
		City:      "Mofu",
		State:     "Northern",
		Country:   "ZM",
		Elevation: 3900,
		Latitude:  -11.1034002304,
		Longitude: 30.3381996155,
		Timezone:  "Africa/Lusaka",
	},
	"FLEC": {
		ICAO:      "FLEC",
		Name:      "East Three Airport",
		City:      "Luano",
		State:     "Central",
		Country:   "ZM",
		Elevation: 2300,
		Latitude:  -14.1134996414,
		Longitude: 30.1294994354,
		Timezone:  "Africa/Lusaka",
	},
	"FLED": {
		ICAO:      "FLED",
		Name:      "East Four Airport",
		City:      "Mululowera",
		State:     "Lusaka",
		Country:   "ZM",
		Elevation: 1880,
		Latitude:  -14.8330001831,
		Longitude: 30.0830001831,
		Timezone:  "Africa/Lusaka",
	},
	"FLEE": {
		ICAO:      "FLEE",
		Name:      "East Five Airport",
		City:      "Chidulka",
		State:     "Muchinga",
		Country:   "ZM",
		Elevation: 4200,
		Latitude:  -10.5356998444,
		Longitude: 33.4071998596,
		Timezone:  "Africa/Lusaka",
	},
	"FLEF": {
		ICAO:      "FLEF",
		Name:      "East Six Airport",
		City:      "Chiwanangala",
		State:     "Northern",
		Country:   "ZM",
		Elevation: 3800,
		Latitude:  -11.1342000961,
		Longitude: 30.01720047,
		Timezone:  "Africa/Lusaka",
	},
	"FLEG": {
		ICAO:      "FLEG",
		Name:      "East Seven Airport",
		City:      "Kanshela",
		State:     "Haut-Katanga",
		Country:   "ZM",
		Elevation: 4000,
		Latitude:  -12.4499998093,
		Longitude: 29.4829998016,
		Timezone:  "Africa/Lubumbashi",
	},
	"FLEH": {
		ICAO:      "FLEH",
		Name:      "East Eight Airport",
		City:      "Kapu",
		State:     "Luapula",
		Country:   "ZM",
		Elevation: 4000,
		Latitude:  -12.1169996262,
		Longitude: 29.4500007629,
		Timezone:  "Africa/Lusaka",
	},
	"FLIK": {
		ICAO:      "FLIK",
		Name:      "Isoka Airport",
		City:      "Isoka",
		State:     "Muchinga",
		Country:   "ZM",
		Elevation: 4462,
		Latitude:  -10.1169996262,
		Longitude: 32.6329994202,
		Timezone:  "Africa/Lusaka",
	},
	"FLJK": {
		ICAO:      "FLJK",
		IATA:      "JEK",
		Name:      "Jeki Airstrip",
		City:      "Lower Zambezi National Park",
		State:     "Muchinga",
		Country:   "ZM",
		Elevation: 4462,
		Latitude:  -15.633332,
		Longitude: 29.603333,
		Timezone:  "Africa/Harare",
	},
	"FLKB": {
		ICAO:      "FLKB",
		Name:      "Kawambwa Airport",
		City:      "Kawambwa",
		State:     "Luapula",
		Country:   "ZM",
		Elevation: 4640,
		Latitude:  -9.7980604172,
		Longitude: 29.0916996002,
		Timezone:  "Africa/Lusaka",
	},
	"FLKE": {
		ICAO:      "FLKE",
		IATA:      "ZKP",
		Name:      "Kasompe Airport",
		City:      "Kasompe",
		State:     "Copperbelt",
		Country:   "ZM",
		Elevation: 4636,
		Latitude:  -12.5727996826,
		Longitude: 27.8938999176,
		Timezone:  "Africa/Lusaka",
	},
	"FLKG": {
		ICAO:      "FLKG",
		Name:      "Kalengwa Airport",
		City:      "Kalengwa",
		State:     "North-Western",
		Country:   "ZM",
		Elevation: 4093,
		Latitude:  -13.4741001129,
		Longitude: 25.0140991211,
		Timezone:  "Africa/Lusaka",
	},
	"FLKJ": {
		ICAO:      "FLKJ",
		Name:      "Kanja Airport",
		City:      "Kanja",
		State:     "Western",
		Country:   "ZM",
		Elevation: 3370,
		Latitude:  -16.4500007629,
		Longitude: 23.3670005798,
		Timezone:  "Africa/Lusaka",
	},
	"FLKK": {
		ICAO:      "FLKK",
		IATA:      "LUN",
		Name:      "Kenneth Kaunda International Airport",
		City:      "Lusaka",
		State:     "Lusaka",
		Country:   "ZM",
		Elevation: 3779,
		Latitude:  -15.3308000565,
		Longitude: 28.4526004791,
		Timezone:  "Africa/Lusaka",
	},
	"FLKL": {
		ICAO:      "FLKL",
		IATA:      "KLB",
		Name:      "Kalabo Airport",
		City:      "Kalabo",
		State:     "Western",
		Country:   "ZM",
		Elevation: 3450,
		Latitude:  -14.9982995987,
		Longitude: 22.6453990936,
		Timezone:  "Africa/Lusaka",
	},
	"FLKO": {
		ICAO:      "FLKO",
		IATA:      "KMZ",
		Name:      "Kaoma Airport",
		City:      "Kaoma",
		State:     "Western",
		Country:   "ZM",
		Elevation: 3670,
		Latitude:  -14.8000001907,
		Longitude: 24.783000946,
		Timezone:  "Africa/Lusaka",
	},
	"FLKS": {
		ICAO:      "FLKS",
		IATA:      "KAA",
		Name:      "Kasama Airport",
		City:      "Kasama",
		State:     "Northern",
		Country:   "ZM",
		Elevation: 4541,
		Latitude:  -10.2166996002,
		Longitude: 31.1333007813,
		Timezone:  "Africa/Lusaka",
	},
	"FLKU": {
		ICAO:      "FLKU",
		Name:      "Kanyau Airport",
		City:      "Kanyau",
		State:     "Western",
		Country:   "ZM",
		Elevation: 3448,
		Latitude:  -16.5,
		Longitude: 22.4169998169,
		Timezone:  "Africa/Lusaka",
	},
	"FLKW": {
		ICAO:      "FLKW",
		Name:      "Milliken Airport",
		City:      "Kabwe",
		State:     "Central",
		Country:   "ZM",
		Elevation: 3920,
		Latitude:  -14.4505996704,
		Longitude: 28.3791999817,
		Timezone:  "Africa/Lusaka",
	},
	"FLKY": {
		ICAO:      "FLKY",
		IATA:      "ZKB",
		Name:      "Kasaba Bay Airport",
		City:      "Kasaba Bay",
		State:     "Northern",
		Country:   "ZM",
		Elevation: 2780,
		Latitude:  -8.5249996185,
		Longitude: 30.6630001068,
		Timezone:  "Africa/Lusaka",
	},
	"FLKZ": {
		ICAO:      "FLKZ",
		Name:      "Lukuzi Airport",
		City:      "Nansolo",
		State:     "Eastern",
		Country:   "ZM",
		Elevation: 1801,
		Latitude:  -12.8119001389,
		Longitude: 32.0648994446,
		Timezone:  "Africa/Lusaka",
	},
	"FLLA": {
		ICAO:      "FLLA",
		Name:      "Luanshya Zambia Airport",
		City:      "Luanshya",
		State:     "Copperbelt",
		Country:   "ZM",
		Elevation: 4101,
		Latitude:  -13.141699791,
		Longitude: 28.4249992371,
		Timezone:  "Africa/Lusaka",
	},
	"FLLC": {
		ICAO:      "FLLC",
		Name:      "Lusaka City Airport",
		City:      "Lusaka",
		State:     "Lusaka",
		Country:   "ZM",
		Elevation: 4200,
		Latitude:  -15.4138002396,
		Longitude: 28.3306999207,
		Timezone:  "Africa/Lusaka",
	},
	"FLLD": {
		ICAO:      "FLLD",
		Name:      "Lundazi Airport",
		City:      "Lundazi",
		State:     "Eastern",
		Country:   "ZM",
		Elevation: 3750,
		Latitude:  -12.2860002518,
		Longitude: 33.186000824,
		Timezone:  "Africa/Lusaka",
	},
	"FLLG": {
		ICAO:      "FLLG",
		Name:      "Luwingu Airport",
		City:      "Luwingu",
		Country:   "ZM",
		Elevation: 4650,
		Latitude:  -10.245300293,
		Longitude: 29.9139995575,
		Timezone:  "Africa/Lusaka",
	},
	"FLLI": {
		ICAO:      "FLLI",
		IATA:      "LVI",
		Name:      "Livingstone Airport",
		City:      "Livingstone",
		State:     "Southern",
		Country:   "ZM",
		Elevation: 3302,
		Latitude:  -17.8218002319,
		Longitude: 25.8227005005,
		Timezone:  "Africa/Lusaka",
	},
	"FLLK": {
		ICAO:      "FLLK",
		IATA:      "LXU",
		Name:      "Lukulu Airport",
		City:      "Lukulu",
		State:     "Western",
		Country:   "ZM",
		Elevation: 3480,
		Latitude:  -14.3747997284,
		Longitude: 23.2495002747,
		Timezone:  "Africa/Lusaka",
	},
	"FLLO": {
		ICAO:      "FLLO",
		Name:      "Kalomo Airport",
		City:      "Kalomo",
		State:     "Southern",
		Country:   "ZM",
		Elevation: 4100,
		Latitude:  -16.9829998016,
		Longitude: 26.4829998016,
		Timezone:  "Africa/Lusaka",
	},
	"FLLS": {
		ICAO:      "FLLS",
		IATA:      "LUN",
		Name:      "Lusaka International Airport",
		City:      "Lusaka",
		State:     "Lusaka",
		Country:   "ZM",
		Elevation: 3779,
		Latitude:  -15.3308000565,
		Longitude: 28.4526004791,
		Timezone:  "Africa/Lusaka",
	},
	"FLMA": {
		ICAO:      "FLMA",
		IATA:      "MNS",
		Name:      "Mansa Airport",
		City:      "Mansa",
		State:     "Luapula",
		Country:   "ZM",
		Elevation: 4101,
		Latitude:  -11.1370000839,
		Longitude: 28.8726005554,
		Timezone:  "Africa/Lusaka",
	},
	"FLMB": {
		ICAO:      "FLMB",
		Name:      "Maamba Airport",
		City:      "Maamba",
		State:     "Southern",
		Country:   "ZM",
		Elevation: 2050,
		Latitude:  -17.3670005798,
		Longitude: 27.1830005646,
		Timezone:  "Africa/Lusaka",
	},
	"FLMF": {
		ICAO:      "FLMF",
		IATA:      "MFU",
		Name:      "Mfuwe Airport",
		City:      "Mfuwe",
		State:     "Eastern",
		Country:   "ZM",
		Elevation: 1853,
		Latitude:  -13.2588996887,
		Longitude: 31.9365997314,
		Timezone:  "Africa/Lusaka",
	},
	"FLMG": {
		ICAO:      "FLMG",
		IATA:      "MNR",
		Name:      "Mongu Airport",
		City:      "Mongu",
		State:     "Western",
		Country:   "ZM",
		Elevation: 3488,
		Latitude:  -15.2545003891,
		Longitude: 23.1623001099,
		Timezone:  "Africa/Lusaka",
	},
	"FLMK": {
		ICAO:      "FLMK",
		Name:      "Mkushi Airport",
		City:      "Mkushi",
		State:     "Central",
		Country:   "ZM",
		Elevation: 4195,
		Latitude:  -13.6169996262,
		Longitude: 29.3829994202,
		Timezone:  "Africa/Lusaka",
	},
	"FLML": {
		ICAO:      "FLML",
		Name:      "Mufulira Airport",
		City:      "Mufulira",
		State:     "Copperbelt",
		Country:   "ZM",
		Elevation: 4350,
		Latitude:  -12.565199852,
		Longitude: 28.2943000793,
		Timezone:  "Africa/Lusaka",
	},
	"FLMO": {
		ICAO:      "FLMO",
		Name:      "Monze Airport",
		City:      "Monze",
		State:     "Southern",
		Country:   "ZM",
		Elevation: 3700,
		Latitude:  -16.283000946,
		Longitude: 27.5170001984,
		Timezone:  "Africa/Lusaka",
	},
	"FLMP": {
		ICAO:      "FLMP",
		Name:      "Mpika Airport",
		City:      "Mpika",
		State:     "Northern",
		Country:   "ZM",
		Elevation: 4600,
		Latitude:  -11.8992004395,
		Longitude: 31.4347000122,
		Timezone:  "Africa/Lusaka",
	},
	"FLMU": {
		ICAO:      "FLMU",
		Name:      "Mulobezi Airport",
		City:      "Mulobezi",
		State:     "Western",
		Country:   "ZM",
		Elevation: 3175,
		Latitude:  -16.7670001984,
		Longitude: 25.1830005646,
		Timezone:  "Africa/Lusaka",
	},
	"FLMW": {
		ICAO:      "FLMW",
		Name:      "Mwinilunga Airport",
		City:      "Mwinilunga",
		State:     "North-Western",
		Country:   "ZM",
		Elevation: 4524,
		Latitude:  -11.6540002823,
		Longitude: 24.4309997559,
		Timezone:  "Africa/Lusaka",
	},
	"FLMZ": {
		ICAO:      "FLMZ",
		Name:      "Mazabuka Airport",
		City:      "Mazabuka",
		State:     "Southern",
		Country:   "ZM",
		Elevation: 3450,
		Latitude:  -15.8500003815,
		Longitude: 27.7416992188,
		Timezone:  "Africa/Lusaka",
	},
	"FLNA": {
		ICAO:      "FLNA",
		IATA:      "ZGM",
		Name:      "Ngoma Airport",
		City:      "Ngoma",
		State:     "Southern",
		Country:   "ZM",
		Elevation: 3400,
		Latitude:  -15.9658002853,
		Longitude: 25.9333000183,
		Timezone:  "Africa/Lusaka",
	},
	"FLND": {
		ICAO:      "FLND",
		IATA:      "NLA",
		Name:      "Ndola Airport",
		City:      "Ndola",
		State:     "Copperbelt",
		Country:   "ZM",
		Elevation: 4167,
		Latitude:  -12.9981002808,
		Longitude: 28.664899826,
		Timezone:  "Africa/Lusaka",
	},
	"FLNL": {
		ICAO:      "FLNL",
		Name:      "Namwala Airport",
		City:      "Namwala",
		State:     "Southern",
		Country:   "ZM",
		Elevation: 3200,
		Latitude:  -15.7650003433,
		Longitude: 26.4319000244,
		Timezone:  "Africa/Lusaka",
	},
	"FLNY": {
		ICAO:      "FLNY",
		Name:      "Nyimba Airport",
		City:      "Nyimba",
		State:     "Eastern",
		Country:   "ZM",
		Elevation: 2600,
		Latitude:  -14.5685997009,
		Longitude: 30.8341007233,
		Timezone:  "Africa/Lusaka",
	},
	"FLPA": {
		ICAO:      "FLPA",
		Name:      "Kasempa Airport",
		City:      "Kasempa",
		State:     "North-Western",
		Country:   "ZM",
		Elevation: 4150,
		Latitude:  -13.4399995804,
		Longitude: 25.7859992981,
		Timezone:  "Africa/Lusaka",
	},
	"FLPE": {
		ICAO:      "FLPE",
		Name:      "Petauke Airport",
		City:      "Petauke",
		State:     "Eastern",
		Country:   "ZM",
		Elevation: 3300,
		Latitude:  -14.1999998093,
		Longitude: 31.216999054,
		Timezone:  "Africa/Lusaka",
	},
	"FLPK": {
		ICAO:      "FLPK",
		Name:      "Mporokoso Airport",
		City:      "Mporokoso",
		State:     "Northern",
		Country:   "ZM",
		Elevation: 4570,
		Latitude:  -9.3666601181,
		Longitude: 30.1389007568,
		Timezone:  "Africa/Lusaka",
	},
	"FLPO": {
		ICAO:      "FLPO",
		Name:      "Kabompo Airport",
		City:      "Kabompo",
		State:     "North-Western",
		Country:   "ZM",
		Elevation: 3535,
		Latitude:  -13.5760002136,
		Longitude: 24.2299995422,
		Timezone:  "Africa/Lusaka",
	},
	"FLRO": {
		ICAO:      "FLRO",
		Name:      "Rosa Airport",
		City:      "Rosa",
		State:     "Northern",
		Country:   "ZM",
		Elevation: 5000,
		Latitude:  -9.5268297195,
		Longitude: 31.2374992371,
		Timezone:  "Africa/Lusaka",
	},
	"FLRU": {
		ICAO:      "FLRU",
		Name:      "Rufansa Airport",
		City:      "Rufansa",
		State:     "Lusaka",
		Country:   "ZM",
		Elevation: 3000,
		Latitude:  -15.0830001831,
		Longitude: 29.6329994202,
		Timezone:  "Africa/Lusaka",
	},
	"FLSE": {
		ICAO:      "FLSE",
		Name:      "Serenje Airport",
		City:      "Serenje",
		State:     "Central",
		Country:   "ZM",
		Elevation: 4650,
		Latitude:  -13.1829996109,
		Longitude: 30.2329998016,
		Timezone:  "Africa/Lusaka",
	},
	"FLSH": {
		ICAO:      "FLSH",
		Name:      "Shiwa n'gandu Airport",
		City:      "Shiwa n'gandu",
		State:     "Muchinga",
		Country:   "ZM",
		Elevation: 4600,
		Latitude:  -11.2009000778,
		Longitude: 31.7481994629,
		Timezone:  "Africa/Lusaka",
	},
	"FLSJ": {
		ICAO:      "FLSJ",
		Name:      "Sakeji Airport",
		City:      "Sakeji",
		State:     "North-Western",
		Country:   "ZM",
		Elevation: 4500,
		Latitude:  -11.2327003479,
		Longitude: 24.316400528,
		Timezone:  "Africa/Lusaka",
	},
	"FLSN": {
		ICAO:      "FLSN",
		IATA:      "SXG",
		Name:      "Senanga Airport",
		City:      "Senanga",
		State:     "Western",
		Country:   "ZM",
		Elevation: 3370,
		Latitude:  -16.1000003815,
		Longitude: 23.2670001984,
		Timezone:  "Africa/Lusaka",
	},
	"FLSO": {
		ICAO:      "FLSO",
		IATA:      "KIW",
		Name:      "Southdowns Airport",
		City:      "Kitwe",
		State:     "Copperbelt",
		Country:   "ZM",
		Elevation: 4145,
		Latitude:  -12.9005002975,
		Longitude: 28.1499004364,
		Timezone:  "Africa/Lusaka",
	},
	"FLSS": {
		ICAO:      "FLSS",
		IATA:      "SJQ",
		Name:      "Sesheke Airport",
		City:      "Sesheke",
		State:     "Western",
		Country:   "ZM",
		Elevation: 3100,
		Latitude:  -17.4762992859,
		Longitude: 24.3047008514,
		Timezone:  "Africa/Lusaka",
	},
	"FLSW": {
		ICAO:      "FLSW",
		IATA:      "SLI",
		Name:      "Solwesi Airport",
		City:      "Solwesi",
		State:     "North-Western",
		Country:   "ZM",
		Elevation: 4551,
		Latitude:  -12.1737003326,
		Longitude: 26.3651008606,
		Timezone:  "Africa/Lusaka",
	},
	"FLWA": {
		ICAO:      "FLWA",
		Name:      "West One Airport",
		City:      "Chinka",
		State:     "Copperbelt",
		Country:   "ZM",
		Elevation: 4000,
		Latitude:  -12.8549003601,
		Longitude: 27.0722007751,
		Timezone:  "Africa/Lusaka",
	},
	"FLWB": {
		ICAO:      "FLWB",
		Name:      "West Two Airport",
		City:      "Metamba",
		State:     "Central",
		Country:   "ZM",
		Elevation: 3760,
		Latitude:  -13.7664003372,
		Longitude: 27.5172996521,
		Timezone:  "Africa/Lusaka",
	},
	"FLWC": {
		ICAO:      "FLWC",
		Name:      "West Three Airport",
		City:      "Nyoka",
		State:     "North-Western",
		Country:   "ZM",
		Elevation: 3650,
		Latitude:  -13.3552999496,
		Longitude: 26.6005992889,
		Timezone:  "Africa/Lusaka",
	},
	"FLWD": {
		ICAO:      "FLWD",
		Name:      "West Four Airport",
		City:      "West Four",
		State:     "North-Western",
		Country:   "ZM",
		Elevation: 3400,
		Latitude:  -13.6454000473,
		Longitude: 25.4218997955,
		Timezone:  "Africa/Lusaka",
	},
	"FLWE": {
		ICAO:      "FLWE",
		Name:      "West Five Airport",
		City:      "Lipanda",
		State:     "Western",
		Country:   "ZM",
		Elevation: 3550,
		Latitude:  -13.9642000198,
		Longitude: 24.4605998993,
		Timezone:  "Africa/Lusaka",
	},
	"FLWF": {
		ICAO:      "FLWF",
		Name:      "West Six Airport",
		City:      "Kauni",
		State:     "North-Western",
		Country:   "ZM",
		Elevation: 4000,
		Latitude:  -12.2172002792,
		Longitude: 27.4904994965,
		Timezone:  "Africa/Lusaka",
	},
	"FLWG": {
		ICAO:      "FLWG",
		Name:      "West Seven Airport",
		City:      "Chanika",
		State:     "North-Western",
		Country:   "ZM",
		Elevation: 4000,
		Latitude:  -11.9126996994,
		Longitude: 25.6844005585,
		Timezone:  "Africa/Lusaka",
	},
	"FLWW": {
		ICAO:      "FLWW",
		Name:      "Waka Waka Airport",
		City:      "Waka Waka",
		State:     "Eastern",
		Country:   "ZM",
		Elevation: 1800,
		Latitude:  -12.4504003525,
		Longitude: 32.2569999695,
		Timezone:  "Africa/Lusaka",
	},
	"FLYA": {
		ICAO:      "FLYA",
		Name:      "Samfya Airport",
		City:      "Samfya",
		State:     "Luapula",
		Country:   "ZM",
		Elevation: 3800,
		Latitude:  -11.3653001785,
		Longitude: 29.5480995178,
		Timezone:  "Africa/Lusaka",
	},
	"FLZB": {
		ICAO:      "FLZB",
		IATA:      "BBZ",
		Name:      "Zambezi Airport",
		City:      "Zambezi",
		State:     "North-Western",
		Country:   "ZM",
		Elevation: 3538,
		Latitude:  -13.5361003876,
		Longitude: 23.108499527,
		Timezone:  "Africa/Lusaka",
	},
	"FM43": {
		ICAO:      "FM43",
		Name:      "Antsiranana Andrakaka Airport",
		State:     "Diana",
		Country:   "MG",
		Elevation: 200,
		Latitude:  -12.2560997009,
		Longitude: 49.2541999817,
		Timezone:  "Indian/Antananarivo",
	},
	"FMCH": {
		ICAO:      "FMCH",
		IATA:      "HAH",
		Name:      "Prince Said Ibrahim International Airport",
		City:      "Moroni",
		State:     "Grande-Comore",
		Country:   "KM",
		Elevation: 93,
		Latitude:  -11.5336999893,
		Longitude: 43.271900177,
		Timezone:  "Indian/Comoro",
	},
	"FMCI": {
		ICAO:      "FMCI",
		IATA:      "NWA",
		Name:      "Moheli Bandar Es Eslam Airport",
		State:     "Moheli",
		Country:   "KM",
		Elevation: 46,
		Latitude:  -12.2981004715,
		Longitude: 43.7663993835,
		Timezone:  "Indian/Comoro",
	},
	"FMCN": {
		ICAO:      "FMCN",
		IATA:      "YVA",
		Name:      "Iconi Airport",
		City:      "Moroni",
		State:     "Grande-Comore",
		Country:   "KM",
		Elevation: 33,
		Latitude:  -11.7108001709,
		Longitude: 43.2439002991,
		Timezone:  "Indian/Comoro",
	},
	"FMCV": {
		ICAO:      "FMCV",
		IATA:      "AJN",
		Name:      "Ouani Airport",
		City:      "Ouani",
		State:     "Anjouan",
		Country:   "KM",
		Elevation: 62,
		Latitude:  -12.1316995621,
		Longitude: 44.4303016663,
		Timezone:  "Indian/Comoro",
	},
	"FMCZ": {
		ICAO:      "FMCZ",
		IATA:      "DZA",
		Name:      "Dzaoudzi Pamandzi International Airport",
		City:      "Dzaoudzi",
		State:     "Pamandzi",
		Country:   "YT",
		Elevation: 23,
		Latitude:  -12.8046998978,
		Longitude: 45.2811012268,
		Timezone:  "Indian/Mayotte",
	},
	"FMEE": {
		ICAO:      "FMEE",
		IATA:      "RUN",
		Name:      "Roland Garros Airport",
		City:      "St Denis",
		State:     "Reunion",
		Country:   "RE",
		Elevation: 66,
		Latitude:  -20.8871002197,
		Longitude: 55.5102996826,
		Timezone:  "Indian/Reunion",
	},
	"FMEP": {
		ICAO:      "FMEP",
		IATA:      "ZSE",
		Name:      "Pierrefonds Airport",
		City:      "St Pierre",
		State:     "Reunion",
		Country:   "RE",
		Elevation: 59,
		Latitude:  -21.3208999634,
		Longitude: 55.4249992371,
		Timezone:  "Indian/Reunion",
	},
	"FMFE": {
		ICAO:      "FMFE",
		IATA:      "OHB",
		Name:      "Moramanga Aerodrome",
		City:      "Moramanga",
		Country:   "MG",
		Elevation: 2960,
		Latitude:  -18.9175,
		Longitude: 48.2181944444,
		Timezone:  "Indian/Antananarivo",
	},
	"FMFZ": {
		ICAO:      "FMFZ",
		Name:      "Betainomby Airport",
		City:      "Betainomby",
		Country:   "MG",
		Elevation: 40,
		Latitude:  -18.1958013864,
		Longitude: 49.348526001,
		Timezone:  "Indian/Antananarivo",
	},
	"FMMA": {
		ICAO:      "FMMA",
		Name:      "Antananarivo Arivonimamo Airport",
		Country:   "MG",
		Elevation: 4757,
		Latitude:  -19.0289993286,
		Longitude: 47.1717987061,
		Timezone:  "Indian/Antananarivo",
	},
	"FMMC": {
		ICAO:      "FMMC",
		IATA:      "WML",
		Name:      "Malaimbandy Airport",
		City:      "Malaimbandy",
		Country:   "MG",
		Elevation: 597,
		Latitude:  -20.3553902921,
		Longitude: 45.5433726311,
		Timezone:  "Indian/Antananarivo",
	},
	"FMME": {
		ICAO:      "FMME",
		IATA:      "ATJ",
		Name:      "Antsirabe Airport",
		City:      "Antsirabe",
		Country:   "MG",
		Elevation: 4997,
		Latitude:  -19.8392214824,
		Longitude: 47.0637130737,
		Timezone:  "Indian/Antananarivo",
	},
	"FMMG": {
		ICAO:      "FMMG",
		IATA:      "WAQ",
		Name:      "Antsalova Airport",
		City:      "Antsalova",
		Country:   "MG",
		Elevation: 551,
		Latitude:  -18.7012732424,
		Longitude: 44.6149206161,
		Timezone:  "Indian/Antananarivo",
	},
	"FMMH": {
		ICAO:      "FMMH",
		IATA:      "VVB",
		Name:      "Mahanoro Airport",
		City:      "Mahanoro",
		Country:   "MG",
		Elevation: 16,
		Latitude:  -19.8330001831,
		Longitude: 48.7999992371,
		Timezone:  "Indian/Antananarivo",
	},
	"FMMI": {
		ICAO:      "FMMI",
		IATA:      "TNR",
		Name:      "Ivato Airport",
		City:      "Antananarivo",
		Country:   "MG",
		Elevation: 4198,
		Latitude:  -18.7968997955,
		Longitude: 47.4788017273,
		Timezone:  "Indian/Antananarivo",
	},
	"FMMJ": {
		ICAO:      "FMMJ",
		Name:      "Ambohijanahary Airport",
		City:      "Ambohijanahary",
		Country:   "MG",
		Elevation: 2496,
		Latitude:  -17.4644789611,
		Longitude: 48.3267116547,
		Timezone:  "Indian/Antananarivo",
	},
	"FMMK": {
		ICAO:      "FMMK",
		IATA:      "JVA",
		Name:      "Ankavandra Airport",
		City:      "Ankavandra",
		Country:   "MG",
		Elevation: 427,
		Latitude:  -18.8050095209,
		Longitude: 45.2734673023,
		Timezone:  "Indian/Antananarivo",
	},
	"FMML": {
		ICAO:      "FMML",
		IATA:      "BMD",
		Name:      "Belo sur Tsiribihina Airport",
		City:      "Belo sur Tsiribihina",
		State:     "Menabe",
		Country:   "MG",
		Elevation: 154,
		Latitude:  -19.6867008209,
		Longitude: 44.5419006348,
		Timezone:  "Indian/Antananarivo",
	},
	"FMMN": {
		ICAO:      "FMMN",
		IATA:      "ZVA",
		Name:      "Miandrivazo Airport",
		Country:   "MG",
		Elevation: 203,
		Latitude:  -19.5627994537,
		Longitude: 45.4508018494,
		Timezone:  "Indian/Antananarivo",
	},
	"FMMO": {
		ICAO:      "FMMO",
		IATA:      "MXT",
		Name:      "Maintirano Airport",
		City:      "Maintirano",
		State:     "Melaky",
		Country:   "MG",
		Elevation: 95,
		Latitude:  -18.0499992371,
		Longitude: 44.033000946,
		Timezone:  "Indian/Antananarivo",
	},
	"FMMP": {
		ICAO:      "FMMP",
		Name:      "Amparafaravola Airport",
		City:      "Amparafaravola",
		Country:   "MG",
		Elevation: 2576,
		Latitude:  -17.6545730219,
		Longitude: 48.2144451141,
		Timezone:  "Indian/Antananarivo",
	},
	"FMMQ": {
		ICAO:      "FMMQ",
		IATA:      "ILK",
		Name:      "Atsinanana Airport",
		City:      "Ilaka",
		Country:   "MG",
		Elevation: 33,
		Latitude:  -19.5830001831,
		Longitude: 48.8030014038,
		Timezone:  "Indian/Antananarivo",
	},
	"FMMR": {
		ICAO:      "FMMR",
		IATA:      "TVA",
		Name:      "Morafenobe Airport",
		City:      "Morafenobe",
		Country:   "MG",
		Elevation: 748,
		Latitude:  -17.850083459,
		Longitude: 44.9204671383,
		Timezone:  "Indian/Antananarivo",
	},
	"FMMS": {
		ICAO:      "FMMS",
		IATA:      "SMS",
		Name:      "Sainte Marie Airport",
		Country:   "MG",
		Elevation: 7,
		Latitude:  -17.0939006805,
		Longitude: 49.8157997131,
		Timezone:  "Indian/Antananarivo",
	},
	"FMMT": {
		ICAO:      "FMMT",
		IATA:      "TMM",
		Name:      "Toamasina Airport",
		State:     "Atsinanana",
		Country:   "MG",
		Elevation: 22,
		Latitude:  -18.109500885,
		Longitude: 49.3925018311,
		Timezone:  "Indian/Antananarivo",
	},
	"FMMU": {
		ICAO:      "FMMU",
		IATA:      "WTA",
		Name:      "Tambohorano Airport",
		City:      "Tambohorano",
		Country:   "MG",
		Elevation: 23,
		Latitude:  -17.4761009216,
		Longitude: 43.9728012085,
		Timezone:  "Indian/Antananarivo",
	},
	"FMMV": {
		ICAO:      "FMMV",
		IATA:      "MOQ",
		Name:      "Morondava Airport",
		State:     "Menabe",
		Country:   "MG",
		Elevation: 30,
		Latitude:  -20.2847003937,
		Longitude: 44.3176002502,
		Timezone:  "Indian/Antananarivo",
	},
	"FMMX": {
		ICAO:      "FMMX",
		IATA:      "WTS",
		Name:      "Tsiroanomandidy Airport",
		City:      "Tsiroanomandidy",
		Country:   "MG",
		Elevation: 2776,
		Latitude:  -18.7596765564,
		Longitude: 46.0540652275,
		Timezone:  "Indian/Antananarivo",
	},
	"FMMY": {
		ICAO:      "FMMY",
		IATA:      "VAT",
		Name:      "Vatomandry Airport",
		City:      "Vatomandry",
		Country:   "MG",
		Elevation: 39,
		Latitude:  -19.383333,
		Longitude: 48.95,
		Timezone:  "Indian/Antananarivo",
	},
	"FMMZ": {
		ICAO:      "FMMZ",
		IATA:      "WAM",
		Name:      "Ambatondrazaka Airport",
		City:      "Ambatondrazaka",
		Country:   "MG",
		Elevation: 2513,
		Latitude:  -17.7953776085,
		Longitude: 48.4425830841,
		Timezone:  "Indian/Antananarivo",
	},
	"FMNA": {
		ICAO:      "FMNA",
		IATA:      "DIE",
		Name:      "Arrachart Airport",
		State:     "Diana",
		Country:   "MG",
		Elevation: 374,
		Latitude:  -12.3493995667,
		Longitude: 49.2916984558,
		Timezone:  "Indian/Antananarivo",
	},
	"FMNC": {
		ICAO:      "FMNC",
		IATA:      "WMR",
		Name:      "Mananara Nord Airport",
		City:      "Mananara Nord",
		State:     "Analanjirofo",
		Country:   "MG",
		Elevation: 9,
		Latitude:  -16.1639003754,
		Longitude: 49.7737998962,
		Timezone:  "Indian/Antananarivo",
	},
	"FMND": {
		ICAO:      "FMND",
		IATA:      "ZWA",
		Name:      "Andapa Airport",
		Country:   "MG",
		Elevation: 1552,
		Latitude:  -14.6517000198,
		Longitude: 49.6206016541,
		Timezone:  "Indian/Antananarivo",
	},
	"FMNE": {
		ICAO:      "FMNE",
		IATA:      "AMB",
		Name:      "Ambilobe Airport",
		Country:   "MG",
		Elevation: 72,
		Latitude:  -13.1884002686,
		Longitude: 48.9879989624,
		Timezone:  "Indian/Antananarivo",
	},
	"FMNF": {
		ICAO:      "FMNF",
		IATA:      "WBD",
		Name:      "Avaratra Airport",
		City:      "Befandriana",
		Country:   "MG",
		Elevation: 820,
		Latitude:  -15.1999998093,
		Longitude: 48.483001709,
		Timezone:  "Indian/Antananarivo",
	},
	"FMNG": {
		ICAO:      "FMNG",
		IATA:      "WPB",
		Name:      "Port Berge Airport",
		City:      "Port Berge",
		Country:   "MG",
		Elevation: 213,
		Latitude:  -15.5842864741,
		Longitude: 47.6235866547,
		Timezone:  "Indian/Antananarivo",
	},
	"FMNH": {
		ICAO:      "FMNH",
		IATA:      "ANM",
		Name:      "Antsirabato Airport",
		State:     "Sava",
		Country:   "MG",
		Elevation: 20,
		Latitude:  -14.9994001389,
		Longitude: 50.3202018738,
		Timezone:  "Indian/Antananarivo",
	},
	"FMNJ": {
		ICAO:      "FMNJ",
		IATA:      "IVA",
		Name:      "Ambanja Airport",
		City:      "Ambanja",
		Country:   "MG",
		Elevation: 36,
		Latitude:  -13.6457053132,
		Longitude: 48.4594273567,
		Timezone:  "Indian/Antananarivo",
	},
	"FMNL": {
		ICAO:      "FMNL",
		IATA:      "HVA",
		Name:      "Analalava Airport",
		Country:   "MG",
		Elevation: 345,
		Latitude:  -14.629699707,
		Longitude: 47.7638015747,
		Timezone:  "Indian/Antananarivo",
	},
	"FMNM": {
		ICAO:      "FMNM",
		IATA:      "MJN",
		Name:      "Amborovy Airport",
		Country:   "MG",
		Elevation: 87,
		Latitude:  -15.6668417421,
		Longitude: 46.3512325287,
		Timezone:  "Indian/Antananarivo",
	},
	"FMNN": {
		ICAO:      "FMNN",
		IATA:      "NOS",
		Name:      "Fascene Airport",
		City:      "Nosy Be",
		State:     "Diana",
		Country:   "MG",
		Elevation: 36,
		Latitude:  -13.3121004105,
		Longitude: 48.3148002625,
		Timezone:  "Indian/Antananarivo",
	},
	"FMNO": {
		ICAO:      "FMNO",
		IATA:      "DWB",
		Name:      "Soalala Airport",
		City:      "Soalala",
		Country:   "MG",
		Elevation: 141,
		Latitude:  -16.1016904207,
		Longitude: 45.3588366508,
		Timezone:  "Indian/Antananarivo",
	},
	"FMNP": {
		ICAO:      "FMNP",
		IATA:      "WMP",
		Name:      "Mampikony Airport",
		City:      "Mampikony",
		Country:   "MG",
		Latitude:  -16.0722693402,
		Longitude: 47.6441645622,
		Timezone:  "Indian/Antananarivo",
	},
	"FMNQ": {
		ICAO:      "FMNQ",
		IATA:      "BPY",
		Name:      "Besalampy Airport",
		State:     "Melaky",
		Country:   "MG",
		Elevation: 125,
		Latitude:  -16.7445302965,
		Longitude: 44.4824838638,
		Timezone:  "Indian/Antananarivo",
	},
	"FMNR": {
		ICAO:      "FMNR",
		IATA:      "WMN",
		Name:      "Maroantsetra Airport",
		Country:   "MG",
		Elevation: 13,
		Latitude:  -15.4366998672,
		Longitude: 49.6883010864,
		Timezone:  "Indian/Antananarivo",
	},
	"FMNS": {
		ICAO:      "FMNS",
		IATA:      "SVB",
		Name:      "Sambava Airport",
		Country:   "MG",
		Elevation: 20,
		Latitude:  -14.2785997391,
		Longitude: 50.1747016907,
		Timezone:  "Indian/Antananarivo",
	},
	"FMNT": {
		ICAO:      "FMNT",
		IATA:      "TTS",
		Name:      "Tsaratanana Airport",
		City:      "Tsaratanana",
		Country:   "MG",
		Elevation: 1073,
		Latitude:  -16.7510643983,
		Longitude: 47.6190161705,
		Timezone:  "Indian/Antananarivo",
	},
	"FMNV": {
		ICAO:      "FMNV",
		IATA:      "VOH",
		Name:      "Vohimarina Airport",
		Country:   "MG",
		Elevation: 19,
		Latitude:  -13.3758001328,
		Longitude: 50.0027999878,
		Timezone:  "Indian/Antananarivo",
	},
	"FMNW": {
		ICAO:      "FMNW",
		IATA:      "WAI",
		Name:      "Ambalabe Airport",
		City:      "Antsohihy",
		Country:   "MG",
		Elevation: 92,
		Latitude:  -14.8987998962,
		Longitude: 47.9939002991,
		Timezone:  "Indian/Antananarivo",
	},
	"FMNX": {
		ICAO:      "FMNX",
		IATA:      "WMA",
		Name:      "Mandritsara Airport",
		City:      "Mandritsara",
		Country:   "MG",
		Elevation: 1007,
		Latitude:  -15.8330494086,
		Longitude: 48.8332843781,
		Timezone:  "Indian/Antananarivo",
	},
	"FMNZ": {
		ICAO:      "FMNZ",
		Name:      "Ampampamena Airport",
		Country:   "MG",
		Elevation: 49,
		Latitude:  -13.4848003387,
		Longitude: 48.6327018738,
		Timezone:  "Indian/Antananarivo",
	},
	"FMSA": {
		ICAO:      "FMSA",
		Name:      "Ambalavao Airport",
		City:      "Ambalavao",
		Country:   "MG",
		Elevation: 3189,
		Latitude:  -21.8148910188,
		Longitude: 46.914024353,
		Timezone:  "Indian/Antananarivo",
	},
	"FMSB": {
		ICAO:      "FMSB",
		IATA:      "WBO",
		Name:      "Antsoa Airport",
		City:      "Beroroha",
		Country:   "MG",
		Elevation: 820,
		Latitude:  -21.6069837647,
		Longitude: 45.1360201836,
		Timezone:  "Indian/Antananarivo",
	},
	"FMSC": {
		ICAO:      "FMSC",
		IATA:      "WMD",
		Name:      "Mandabe Airport",
		City:      "Mandabe",
		Country:   "MG",
		Elevation: 951,
		Latitude:  -21.0463049303,
		Longitude: 44.9404120445,
		Timezone:  "Indian/Antananarivo",
	},
	"FMSD": {
		ICAO:      "FMSD",
		IATA:      "FTU",
		Name:      "Tolanaro Airport",
		City:      "Tolanaro",
		Country:   "MG",
		Elevation: 29,
		Latitude:  -25.0380992889,
		Longitude: 46.9561004639,
		Timezone:  "Indian/Antananarivo",
	},
	"FMSE": {
		ICAO:      "FMSE",
		Name:      "Betroka Airport",
		City:      "Betroka",
		Country:   "MG",
		Elevation: 2841,
		Latitude:  -23.2743078483,
		Longitude: 46.1249828339,
		Timezone:  "Indian/Antananarivo",
	},
	"FMSF": {
		ICAO:      "FMSF",
		IATA:      "WFI",
		Name:      "Fianarantsoa Airport",
		Country:   "MG",
		Elevation: 3658,
		Latitude:  -21.4416007996,
		Longitude: 47.1116981506,
		Timezone:  "Indian/Antananarivo",
	},
	"FMSG": {
		ICAO:      "FMSG",
		IATA:      "RVA",
		Name:      "Farafangana Airport",
		Country:   "MG",
		Elevation: 26,
		Latitude:  -22.8052997589,
		Longitude: 47.8205986023,
		Timezone:  "Indian/Antananarivo",
	},
	"FMSI": {
		ICAO:      "FMSI",
		IATA:      "IHO",
		Name:      "Ihosy Airport",
		City:      "Ihosy",
		State:     "Ihorombe",
		Country:   "MG",
		Elevation: 2500,
		Latitude:  -22.4047202024,
		Longitude: 46.1649370193,
		Timezone:  "Indian/Antananarivo",
	},
	"FMSJ": {
		ICAO:      "FMSJ",
		IATA:      "MJA",
		Name:      "Manja Airport",
		City:      "Manja",
		Country:   "MG",
		Elevation: 787,
		Latitude:  -21.4261052506,
		Longitude: 44.31650877,
		Timezone:  "Indian/Antananarivo",
	},
	"FMSK": {
		ICAO:      "FMSK",
		IATA:      "WVK",
		Name:      "Manakara Airport",
		Country:   "MG",
		Elevation: 33,
		Latitude:  -22.1196994781,
		Longitude: 48.0217018127,
		Timezone:  "Indian/Antananarivo",
	},
	"FMSL": {
		ICAO:      "FMSL",
		IATA:      "OVA",
		Name:      "Bekily Airport",
		City:      "Bekily",
		Country:   "MG",
		Elevation: 1270,
		Latitude:  -24.2356947547,
		Longitude: 45.3045272827,
		Timezone:  "Indian/Antananarivo",
	},
	"FMSM": {
		ICAO:      "FMSM",
		IATA:      "MNJ",
		Name:      "Mananjary Airport",
		Country:   "MG",
		Elevation: 20,
		Latitude:  -21.2017993927,
		Longitude: 48.3582992554,
		Timezone:  "Indian/Antananarivo",
	},
	"FMSN": {
		ICAO:      "FMSN",
		IATA:      "TDV",
		Name:      "Samangoky Airport",
		City:      "Tanandava",
		Country:   "MG",
		Elevation: 89,
		Latitude:  -21.7000007629,
		Longitude: 43.733001709,
		Timezone:  "Indian/Antananarivo",
	},
	"FMSR": {
		ICAO:      "FMSR",
		IATA:      "MXM",
		Name:      "Morombe Airport",
		State:     "Atsimo-Andrefana",
		Country:   "MG",
		Elevation: 16,
		Latitude:  -21.753900528,
		Longitude: 43.3754997253,
		Timezone:  "Indian/Antananarivo",
	},
	"FMST": {
		ICAO:      "FMST",
		IATA:      "TLE",
		Name:      "Toliara Airport",
		Country:   "MG",
		Elevation: 29,
		Latitude:  -23.3833999634,
		Longitude: 43.7285003662,
		Timezone:  "Indian/Antananarivo",
	},
	"FMSU": {
		ICAO:      "FMSU",
		IATA:      "VND",
		Name:      "Vangaindrano Airport",
		City:      "Vangaindrano",
		State:     "Atsimo-Atsinanana",
		Country:   "MG",
		Elevation: 45,
		Latitude:  -23.3507665915,
		Longitude: 47.5817012787,
		Timezone:  "Indian/Antananarivo",
	},
	"FMSV": {
		ICAO:      "FMSV",
		IATA:      "BKU",
		Name:      "Betioky Airport",
		City:      "Betioky",
		State:     "Atsimo-Andrefana",
		Country:   "MG",
		Elevation: 919,
		Latitude:  -23.7329998016,
		Longitude: 44.388999939,
		Timezone:  "Indian/Antananarivo",
	},
	"FMSY": {
		ICAO:      "FMSY",
		IATA:      "AMP",
		Name:      "Ampanihy Airport",
		City:      "Ampanihy",
		State:     "Atsimo-Andrefana",
		Country:   "MG",
		Elevation: 771,
		Latitude:  -24.6996994019,
		Longitude: 44.7341995239,
		Timezone:  "Indian/Antananarivo",
	},
	"FMSZ": {
		ICAO:      "FMSZ",
		IATA:      "WAK",
		Name:      "Ankazoabo Airport",
		City:      "Ankazoabo",
		Country:   "MG",
		Elevation: 1411,
		Latitude:  -22.2964423522,
		Longitude: 44.5315361023,
		Timezone:  "Indian/Antananarivo",
	},
	"FN17": {
		ICAO:      "FN17",
		Name:      "Cahama Airport",
		City:      "Cahama",
		State:     "Cunene",
		Country:   "AO",
		Elevation: 3970,
		Latitude:  -16.2383995056,
		Longitude: 14.3260002136,
		Timezone:  "Africa/Luanda",
	},
	"FN18": {
		ICAO:      "FN18",
		Name:      "Matala Airport",
		City:      "Matala",
		State:     "Huila",
		Country:   "AO",
		Elevation: 4120,
		Latitude:  -14.7274999619,
		Longitude: 15.013999939,
		Timezone:  "Africa/Luanda",
	},
	"FN19": {
		ICAO:      "FN19",
		Name:      "Cabo Ledo Airport",
		City:      "Cabo Ledo",
		State:     "Bengo",
		Country:   "AO",
		Elevation: 360,
		Latitude:  -9.6530504227,
		Longitude: 13.26060009,
		Timezone:  "Africa/Luanda",
	},
	"FNAM": {
		ICAO:      "FNAM",
		IATA:      "AZZ",
		Name:      "Ambriz Airport",
		City:      "Ambriz",
		State:     "Bengo",
		Country:   "AO",
		Elevation: 144,
		Latitude:  -7.8622198105,
		Longitude: 13.1161003113,
		Timezone:  "Africa/Luanda",
	},
	"FNBC": {
		ICAO:      "FNBC",
		IATA:      "SSY",
		Name:      "Mbanza Congo Airport",
		City:      "Mbanza Congo",
		State:     "Zaire",
		Country:   "AO",
		Elevation: 1860,
		Latitude:  -6.2698998451,
		Longitude: 14.2469997406,
		Timezone:  "Africa/Luanda",
	},
	"FNBG": {
		ICAO:      "FNBG",
		IATA:      "BUG",
		Name:      "Benguela Airport",
		City:      "Benguela",
		State:     "Benguela",
		Country:   "AO",
		Elevation: 118,
		Latitude:  -12.609000206,
		Longitude: 13.4036998749,
		Timezone:  "Africa/Luanda",
	},
	"FNCA": {
		ICAO:      "FNCA",
		IATA:      "CAB",
		Name:      "Cabinda Airport",
		City:      "Cabinda",
		State:     "Cabinda",
		Country:   "AO",
		Elevation: 66,
		Latitude:  -5.5969901085,
		Longitude: 12.1884002686,
		Timezone:  "Africa/Luanda",
	},
	"FNCB": {
		ICAO:      "FNCB",
		Name:      "Camembe Airport",
		City:      "Camembe",
		State:     "Bengo",
		Country:   "AO",
		Elevation: 2264,
		Latitude:  -8.1330003738,
		Longitude: 14.5,
		Timezone:  "Africa/Luanda",
	},
	"FNCC": {
		ICAO:      "FNCC",
		Name:      "Cacolo Airport",
		City:      "Cacolo",
		State:     "Lunda-Sul",
		Country:   "AO",
		Elevation: 3937,
		Latitude:  -10.2829999924,
		Longitude: 19.2670001984,
		Timezone:  "Africa/Luanda",
	},
	"FNCF": {
		ICAO:      "FNCF",
		IATA:      "CFF",
		Name:      "Cafunfo Airport",
		City:      "Cafunfo",
		State:     "Luanda-Norte",
		Country:   "AO",
		Latitude:  -8.7836103439,
		Longitude: 17.9897003174,
		Timezone:  "Africa/Luanda",
	},
	"FNCH": {
		ICAO:      "FNCH",
		IATA:      "PGI",
		Name:      "Chitato Airport",
		City:      "Chitato",
		State:     "Luanda-Norte",
		Country:   "AO",
		Elevation: 2500,
		Latitude:  -7.3588900566,
		Longitude: 20.8047008514,
		Timezone:  "Africa/Luanda",
	},
	"FNCM": {
		ICAO:      "FNCM",
		Name:      "Camabatela Airport",
		City:      "Camabatela",
		State:     "Cuanza-Norte",
		Country:   "AO",
		Elevation: 410,
		Latitude:  -8.2170000076,
		Longitude: 15.3669996262,
		Timezone:  "Africa/Luanda",
	},
	"FNCT": {
		ICAO:      "FNCT",
		IATA:      "CBT",
		Name:      "Catumbela Airport",
		City:      "Catumbela",
		State:     "Benguela",
		Country:   "AO",
		Latitude:  -12.4792003632,
		Longitude: 13.4869003296,
		Timezone:  "Africa/Luanda",
	},
	"FNCV": {
		ICAO:      "FNCV",
		IATA:      "CTI",
		Name:      "Cuito Cuanavale Airport",
		City:      "Cuito Cuanavale",
		State:     "Cuando-Cobango",
		Country:   "AO",
		Latitude:  -15.1603002548,
		Longitude: 19.1560993195,
		Timezone:  "Africa/Luanda",
	},
	"FNCX": {
		ICAO:      "FNCX",
		IATA:      "CXM",
		Name:      "Camaxilo Airport",
		City:      "Camaxilo",
		State:     "Luanda-Norte",
		Country:   "AO",
		Elevation: 3957,
		Latitude:  -8.3736095428,
		Longitude: 18.9235992432,
		Timezone:  "Africa/Luanda",
	},
	"FNCZ": {
		ICAO:      "FNCZ",
		IATA:      "CAV",
		Name:      "Cazombo Airport",
		City:      "Cazombo",
		State:     "Moxico",
		Country:   "AO",
		Elevation: 3700,
		Latitude:  -11.8930997849,
		Longitude: 22.9164009094,
		Timezone:  "Africa/Luanda",
	},
	"FNDB": {
		ICAO:      "FNDB",
		Name:      "Damba Airport",
		City:      "Damba",
		State:     "Uige",
		Country:   "AO",
		Elevation: 3648,
		Latitude:  -6.9000000954,
		Longitude: 15.1330003738,
		Timezone:  "Africa/Luanda",
	},
	"FNDU": {
		ICAO:      "FNDU",
		IATA:      "DUE",
		Name:      "Dundo Airport",
		City:      "Chitato",
		State:     "Luanda-Norte",
		Country:   "AO",
		Elevation: 2451,
		Latitude:  -7.4008898735,
		Longitude: 20.8185005188,
		Timezone:  "Africa/Luanda",
	},
	"FNGI": {
		ICAO:      "FNGI",
		IATA:      "VPE",
		Name:      "Ngjiva Pereira Airport",
		City:      "Ngiva",
		State:     "Cunene",
		Country:   "AO",
		Elevation: 3566,
		Latitude:  -17.0435009003,
		Longitude: 15.6837997437,
		Timezone:  "Africa/Luanda",
	},
	"FNHU": {
		ICAO:      "FNHU",
		IATA:      "NOV",
		Name:      "Nova Lisboa Airport",
		City:      "Huambo",
		State:     "Huambo",
		Country:   "AO",
		Elevation: 5587,
		Latitude:  -12.8088998795,
		Longitude: 15.7604999542,
		Timezone:  "Africa/Luanda",
	},
	"FNKU": {
		ICAO:      "FNKU",
		IATA:      "SVP",
		Name:      "Kuito Airport",
		City:      "Kuito",
		State:     "Bie",
		Country:   "AO",
		Elevation: 5618,
		Latitude:  -12.4046001434,
		Longitude: 16.9473991394,
		Timezone:  "Africa/Luanda",
	},
	"FNLB": {
		ICAO:      "FNLB",
		IATA:      "LLT",
		Name:      "Lobito Airport",
		City:      "Lobito",
		State:     "Benguela",
		Country:   "AO",
		Elevation: 10,
		Latitude:  -12.3711996078,
		Longitude: 13.5366001129,
		Timezone:  "Africa/Luanda",
	},
	"FNLK": {
		ICAO:      "FNLK",
		IATA:      "LBZ",
		Name:      "Lucapa Airport",
		City:      "Lucapa",
		State:     "Luanda-Norte",
		Country:   "AO",
		Elevation: 3029,
		Latitude:  -8.4457273483,
		Longitude: 20.7320861816,
		Timezone:  "Africa/Luanda",
	},
	"FNLU": {
		ICAO:      "FNLU",
		IATA:      "LAD",
		Name:      "Quatro De Fevereiro Airport",
		City:      "Luanda",
		State:     "Luanda",
		Country:   "AO",
		Elevation: 243,
		Latitude:  -8.8583698273,
		Longitude: 13.2312002182,
		Timezone:  "Africa/Luanda",
	},
	"FNLZ": {
		ICAO:      "FNLZ",
		IATA:      "LZM",
		Name:      "Luzamba Airport",
		City:      "Luzamba",
		State:     "Luanda-Norte",
		Country:   "AO",
		Elevation: 2904,
		Latitude:  -9.1159601212,
		Longitude: 18.0492992401,
		Timezone:  "Africa/Luanda",
	},
	"FNMA": {
		ICAO:      "FNMA",
		IATA:      "MEG",
		Name:      "Malanje Airport",
		City:      "Malanje",
		State:     "Malanje",
		Country:   "AO",
		Elevation: 3868,
		Latitude:  -9.5250902176,
		Longitude: 16.3124008179,
		Timezone:  "Africa/Luanda",
	},
	"FNME": {
		ICAO:      "FNME",
		IATA:      "SPP",
		Name:      "Menongue Airport",
		City:      "Menongue",
		State:     "Cuando-Cobango",
		Country:   "AO",
		Elevation: 4469,
		Latitude:  -14.6576004028,
		Longitude: 17.7198009491,
		Timezone:  "Africa/Luanda",
	},
	"FNMO": {
		ICAO:      "FNMO",
		IATA:      "MSZ",
		Name:      "Namibe Airport",
		City:      "Namibe",
		State:     "Namibe",
		Country:   "AO",
		Elevation: 210,
		Latitude:  -15.2611999512,
		Longitude: 12.1468000412,
		Timezone:  "Africa/Luanda",
	},
	"FNMQ": {
		ICAO:      "FNMQ",
		Name:      "Maquela do Zombo Airport",
		City:      "Maquela do Zombo",
		State:     "Uige",
		Country:   "AO",
		Elevation: 3051,
		Latitude:  -6.0302801132,
		Longitude: 15.1368999481,
		Timezone:  "Africa/Luanda",
	},
	"FNNG": {
		ICAO:      "FNNG",
		IATA:      "GXG",
		Name:      "Negage Airport",
		City:      "Negage",
		State:     "Uige",
		Country:   "AO",
		Elevation: 4105,
		Latitude:  -7.7545099258,
		Longitude: 15.2876996994,
		Timezone:  "Africa/Luanda",
	},
	"FNPA": {
		ICAO:      "FNPA",
		IATA:      "PBN",
		Name:      "Porto Amboim Airport",
		City:      "Port Amboim",
		State:     "Kwanza-Sul",
		Country:   "AO",
		Elevation: 16,
		Latitude:  -10.7220001221,
		Longitude: 13.7655000687,
		Timezone:  "Africa/Luanda",
	},
	"FNPB": {
		ICAO:      "FNPB",
		Name:      "Sanza Pombo Airport",
		City:      "Sanza Pombo",
		State:     "Uige",
		Country:   "AO",
		Elevation: 3251,
		Latitude:  -7.3000001907,
		Longitude: 15.9329996109,
		Timezone:  "Africa/Luanda",
	},
	"FNSA": {
		ICAO:      "FNSA",
		IATA:      "VHC",
		Name:      "Saurimo Airport",
		City:      "Saurimo",
		State:     "Lunda-Sul",
		Country:   "AO",
		Elevation: 3584,
		Latitude:  -9.6890697479,
		Longitude: 20.4319000244,
		Timezone:  "Africa/Luanda",
	},
	"FNSO": {
		ICAO:      "FNSO",
		IATA:      "SZA",
		Name:      "Soyo Airport",
		City:      "Soyo",
		State:     "Zaire",
		Country:   "AO",
		Elevation: 15,
		Latitude:  -6.1410899162,
		Longitude: 12.3718004227,
		Timezone:  "Africa/Luanda",
	},
	"FNSU": {
		ICAO:      "FNSU",
		IATA:      "NDD",
		Name:      "Sumbe Airport",
		City:      "Sumbe",
		State:     "Kwanza-Sul",
		Country:   "AO",
		Elevation: 36,
		Latitude:  -11.1679000854,
		Longitude: 13.8474998474,
		Timezone:  "Africa/Luanda",
	},
	"FNTO": {
		ICAO:      "FNTO",
		Name:      "Toto Airport",
		City:      "Toto",
		State:     "Uige",
		Country:   "AO",
		Elevation: 1775,
		Latitude:  -7.1463899612,
		Longitude: 14.2486000061,
		Timezone:  "Africa/Luanda",
	},
	"FNUA": {
		ICAO:      "FNUA",
		IATA:      "UAL",
		Name:      "Luau Airport",
		City:      "Luau",
		State:     "Moxico",
		Country:   "AO",
		Elevation: 3609,
		Latitude:  -10.7158002853,
		Longitude: 22.2311000824,
		Timezone:  "Africa/Luanda",
	},
	"FNUB": {
		ICAO:      "FNUB",
		IATA:      "SDD",
		Name:      "Lubango Airport",
		City:      "Lubango",
		State:     "Huila",
		Country:   "AO",
		Elevation: 5778,
		Latitude:  -14.9246997833,
		Longitude: 13.5749998093,
		Timezone:  "Africa/Luanda",
	},
	"FNUE": {
		ICAO:      "FNUE",
		IATA:      "LUO",
		Name:      "Luena Airport",
		City:      "Luena",
		State:     "Moxico",
		Country:   "AO",
		Elevation: 4360,
		Latitude:  -11.7680997849,
		Longitude: 19.8976993561,
		Timezone:  "Africa/Luanda",
	},
	"FNUG": {
		ICAO:      "FNUG",
		IATA:      "UGO",
		Name:      "Uige Airport",
		City:      "Uige",
		State:     "Uige",
		Country:   "AO",
		Elevation: 2720,
		Latitude:  -7.6030697823,
		Longitude: 15.0277996063,
		Timezone:  "Africa/Luanda",
	},
	"FNWK": {
		ICAO:      "FNWK",
		IATA:      "CEO",
		Name:      "Waco Kungo Airport",
		City:      "Waco Kungo",
		State:     "Benguela",
		Country:   "AO",
		Elevation: 4324,
		Latitude:  -11.4264001846,
		Longitude: 15.1014003754,
		Timezone:  "Africa/Luanda",
	},
	"FNXA": {
		ICAO:      "FNXA",
		IATA:      "XGN",
		Name:      "Xangongo Airport",
		City:      "Xangongo",
		State:     "Cunene",
		Country:   "AO",
		Elevation: 3635,
		Latitude:  -16.755399704,
		Longitude: 14.9652996063,
		Timezone:  "Africa/Luanda",
	},
	"FNZE": {
		ICAO:      "FNZE",
		IATA:      "ARZ",
		Name:      "N'zeto Airport",
		City:      "N'zeto",
		State:     "Zaire",
		Country:   "AO",
		Elevation: 69,
		Latitude:  -7.2594399452,
		Longitude: 12.8631000519,
		Timezone:  "Africa/Luanda",
	},
	"FNZG": {
		ICAO:      "FNZG",
		IATA:      "NZA",
		Name:      "Nzagi Airport",
		City:      "Nzagi",
		State:     "Luanda-Norte",
		Country:   "AO",
		Elevation: 2431,
		Latitude:  -7.7169399262,
		Longitude: 21.3582000732,
		Timezone:  "Africa/Luanda",
	},
	"FOGB": {
		ICAO:      "FOGB",
		IATA:      "BGB",
		Name:      "Booue Airport",
		City:      "Booue",
		State:     "Ogooue-Ivindo",
		Country:   "GA",
		Elevation: 604,
		Latitude:  -0.1000000015,
		Longitude: 11.9499998093,
		Timezone:  "Africa/Libreville",
	},
	"FOGE": {
		ICAO:      "FOGE",
		IATA:      "KDN",
		Name:      "Ndende Airport",
		City:      "Ndende",
		State:     "Ngouni",
		Country:   "GA",
		Elevation: 417,
		Latitude:  -2.4000000954,
		Longitude: 11.3669996262,
		Timezone:  "Africa/Libreville",
	},
	"FOGF": {
		ICAO:      "FOGF",
		IATA:      "FOU",
		Name:      "Fougamou Airport",
		City:      "Fougamou",
		State:     "Ngouni",
		Country:   "GA",
		Elevation: 263,
		Latitude:  -1.2829999924,
		Longitude: 10.6169996262,
		Timezone:  "Africa/Libreville",
	},
	"FOGG": {
		ICAO:      "FOGG",
		IATA:      "MBC",
		Name:      "M'Bigou Airport",
		City:      "M'Bigou",
		State:     "Ngouni",
		Country:   "GA",
		Elevation: 2346,
		Latitude:  -1.8830000162,
		Longitude: 11.9329996109,
		Timezone:  "Africa/Libreville",
	},
	"FOGI": {
		ICAO:      "FOGI",
		IATA:      "MGX",
		Name:      "Moabi Airport",
		City:      "Moabi",
		State:     "Nyanga",
		Country:   "GA",
		Elevation: 787,
		Latitude:  -2.4330000877,
		Longitude: 11,
		Timezone:  "Africa/Libreville",
	},
	"FOGJ": {
		ICAO:      "FOGJ",
		IATA:      "KDJ",
		Name:      "Ville Airport",
		City:      "N'Djole",
		State:     "Moyen-Ogooue",
		Country:   "GA",
		Elevation: 164,
		Latitude:  -0.1829999983,
		Longitude: 10.75,
		Timezone:  "Africa/Libreville",
	},
	"FOGK": {
		ICAO:      "FOGK",
		IATA:      "KOU",
		Name:      "Koulamoutou Airport",
		City:      "Koulamoutou",
		State:     "Ogooue-Lolo",
		Country:   "GA",
		Elevation: 1070,
		Latitude:  -1.1846100092,
		Longitude: 12.4413003922,
		Timezone:  "Africa/Libreville",
	},
	"FOGM": {
		ICAO:      "FOGM",
		IATA:      "MJL",
		Name:      "Mouilla Ville Airport",
		City:      "Mouila",
		State:     "Ngouni",
		Country:   "GA",
		Elevation: 295,
		Latitude:  -1.8451399803,
		Longitude: 11.0566997528,
		Timezone:  "Africa/Libreville",
	},
	"FOGO": {
		ICAO:      "FOGO",
		IATA:      "OYE",
		Name:      "Oyem Airport",
		City:      "Oyem",
		State:     "Woleu-Ntem",
		Country:   "GA",
		Elevation: 2158,
		Latitude:  1.543110013,
		Longitude: 11.5813999176,
		Timezone:  "Africa/Libreville",
	},
	"FOGQ": {
		ICAO:      "FOGQ",
		IATA:      "OKN",
		Name:      "Okondja Airport",
		City:      "Okondja",
		State:     "Haut-Ogooue",
		Country:   "GA",
		Elevation: 1325,
		Latitude:  -0.6652140021,
		Longitude: 13.6731004715,
		Timezone:  "Africa/Libreville",
	},
	"FOGR": {
		ICAO:      "FOGR",
		IATA:      "LBQ",
		Name:      "Lambarene Airport",
		City:      "Lambarene",
		State:     "Moyen-Ogooue",
		Country:   "GA",
		Elevation: 82,
		Latitude:  -0.7043889761,
		Longitude: 10.2456998825,
		Timezone:  "Africa/Libreville",
	},
	"FOGV": {
		ICAO:      "FOGV",
		IATA:      "MVX",
		Name:      "Minvoul Airport",
		City:      "Minvoul",
		State:     "Woleu-Ntem",
		Country:   "GA",
		Elevation: 1969,
		Latitude:  2.1500000954,
		Longitude: 12.1330003738,
		Timezone:  "Africa/Libreville",
	},
	"FOOB": {
		ICAO:      "FOOB",
		IATA:      "BMM",
		Name:      "Bitam Airport",
		City:      "Bitam",
		State:     "Woleu-Ntem",
		Country:   "GA",
		Elevation: 1969,
		Latitude:  2.0756399632,
		Longitude: 11.4932003021,
		Timezone:  "Africa/Libreville",
	},
	"FOOC": {
		ICAO:      "FOOC",
		Name:      "Cocobeach Airport",
		City:      "Cocobeach",
		State:     "Estuaire",
		Country:   "GA",
		Elevation: 69,
		Latitude:  1,
		Longitude: 9.5830001831,
		Timezone:  "Africa/Libreville",
	},
	"FOOD": {
		ICAO:      "FOOD",
		IATA:      "MFF",
		Name:      "Moanda Airport",
		City:      "Moanda",
		State:     "Haut-Ogooue",
		Country:   "GA",
		Elevation: 1877,
		Latitude:  -1.5329999924,
		Longitude: 13.2670001984,
		Timezone:  "Africa/Libreville",
	},
	"FOOE": {
		ICAO:      "FOOE",
		IATA:      "MKB",
		Name:      "Mekambo Airport",
		City:      "Mekambo",
		State:     "Ogooue-Ivindo",
		Country:   "GA",
		Elevation: 1686,
		Latitude:  1.0169999599,
		Longitude: 13.9329996109,
		Timezone:  "Africa/Libreville",
	},
	"FOOG": {
		ICAO:      "FOOG",
		IATA:      "POG",
		Name:      "Port Gentil Airport",
		City:      "Port Gentil",
		State:     "Ogooue-Maritime",
		Country:   "GA",
		Elevation: 13,
		Latitude:  -0.7117390037,
		Longitude: 8.7543802261,
		Timezone:  "Africa/Libreville",
	},
	"FOOH": {
		ICAO:      "FOOH",
		IATA:      "OMB",
		Name:      "Omboue Hopital Airport",
		City:      "Omboue",
		State:     "Ogooue-Maritime",
		Country:   "GA",
		Elevation: 33,
		Latitude:  -1.5747300386,
		Longitude: 9.2626895905,
		Timezone:  "Africa/Libreville",
	},
	"FOOK": {
		ICAO:      "FOOK",
		IATA:      "MKU",
		Name:      "Makokou Airport",
		City:      "Makokou",
		State:     "Ogooue-Ivindo",
		Country:   "GA",
		Elevation: 1726,
		Latitude:  0.5792109966,
		Longitude: 12.8908996582,
		Timezone:  "Africa/Libreville",
	},
	"FOOL": {
		ICAO:      "FOOL",
		IATA:      "LBV",
		Name:      "Leon M Ba Airport",
		City:      "Libreville",
		State:     "Estuaire",
		Country:   "GA",
		Elevation: 39,
		Latitude:  0.4586000144,
		Longitude: 9.4122800827,
		Timezone:  "Africa/Libreville",
	},
	"FOOM": {
		ICAO:      "FOOM",
		IATA:      "MZC",
		Name:      "Mitzic Airport",
		City:      "Mitzic",
		State:     "Woleu-Ntem",
		Country:   "GA",
		Elevation: 1913,
		Latitude:  0.7829999924,
		Longitude: 11.5500001907,
		Timezone:  "Africa/Libreville",
	},
	"FOON": {
		ICAO:      "FOON",
		IATA:      "MVB",
		Name:      "M'Vengue El Hadj Omar Bongo Ondimba International Airport",
		City:      "Franceville",
		State:     "Haut-Ogooue",
		Country:   "GA",
		Elevation: 1450,
		Latitude:  -1.656159997,
		Longitude: 13.4379997253,
		Timezone:  "Africa/Libreville",
	},
	"FOOR": {
		ICAO:      "FOOR",
		IATA:      "LTL",
		Name:      "Lastourville Airport",
		City:      "Lastourville",
		State:     "Ogooue-Lolo",
		Country:   "GA",
		Elevation: 1585,
		Latitude:  -0.8266670108,
		Longitude: 12.7486000061,
		Timezone:  "Africa/Libreville",
	},
	"FOOS": {
		ICAO:      "FOOS",
		IATA:      "ZKM",
		Name:      "Sette Cama Airport",
		City:      "Sette Cama",
		State:     "Ogooue-Maritime",
		Country:   "GA",
		Elevation: 10,
		Latitude:  -2.5329999924,
		Longitude: 9.7670001984,
		Timezone:  "Africa/Libreville",
	},
	"FOOT": {
		ICAO:      "FOOT",
		IATA:      "TCH",
		Name:      "Tchibanga Airport",
		City:      "Tchibanga",
		State:     "Nyanga",
		Country:   "GA",
		Elevation: 269,
		Latitude:  -2.8499999046,
		Longitude: 11.0170001984,
		Timezone:  "Africa/Libreville",
	},
	"FOOY": {
		ICAO:      "FOOY",
		IATA:      "MYB",
		Name:      "Mayumba Airport",
		City:      "Mayumba",
		State:     "Nyanga",
		Country:   "GA",
		Elevation: 13,
		Latitude:  -3.4584197998,
		Longitude: 10.6740760803,
		Timezone:  "Africa/Libreville",
	},
	"FPPR": {
		ICAO:      "FPPR",
		IATA:      "PCP",
		Name:      "Principe Airport",
		State:     "Principe",
		Country:   "ST",
		Elevation: 591,
		Latitude:  1.6629400253,
		Longitude: 7.4117398262,
		Timezone:  "Africa/Sao_Tome",
	},
	"FPST": {
		ICAO:      "FPST",
		IATA:      "TMS",
		Name:      "Sao Tome International Airport",
		City:      "Sao Tome",
		State:     "Sao-Tome-Island",
		Country:   "ST",
		Elevation: 33,
		Latitude:  0.3781749904,
		Longitude: 6.7121500969,
		Timezone:  "Africa/Sao_Tome",
	},
	"FQ49": {
		ICAO:      "FQ49",
		Name:      "Vila Franca do Save Airport",
		City:      "Vila Franca do Save",
		State:     "Inhambane",
		Country:   "MZ",
		Elevation: 98,
		Latitude:  -21.1644992828,
		Longitude: 34.5606994629,
		Timezone:  "Africa/Maputo",
	},
	"FQAG": {
		ICAO:      "FQAG",
		IATA:      "ANO",
		Name:      "Angoche Airport",
		City:      "Angoche",
		State:     "Nampula",
		Country:   "MZ",
		Elevation: 121,
		Latitude:  -16.1818695068,
		Longitude: 39.9452171326,
		Timezone:  "Africa/Maputo",
	},
	"FQBI": {
		ICAO:      "FQBI",
		Name:      "Bilene Airport",
		City:      "Bilene",
		State:     "Gaza",
		Country:   "MZ",
		Elevation: 118,
		Latitude:  -25.2659854889,
		Longitude: 33.2385978699,
		Timezone:  "Africa/Maputo",
	},
	"FQBR": {
		ICAO:      "FQBR",
		IATA:      "BEW",
		Name:      "Beira Airport",
		City:      "Beira",
		State:     "Sofala",
		Country:   "MZ",
		Elevation: 33,
		Latitude:  -19.7964000702,
		Longitude: 34.9076004028,
		Timezone:  "Africa/Maputo",
	},
	"FQCB": {
		ICAO:      "FQCB",
		IATA:      "FXO",
		Name:      "Cuamba Airport",
		City:      "Cuamba",
		State:     "Niassa",
		Country:   "MZ",
		Elevation: 1919,
		Latitude:  -14.8149995804,
		Longitude: 36.5299987793,
		Timezone:  "Africa/Maputo",
	},
	"FQCH": {
		ICAO:      "FQCH",
		IATA:      "VPY",
		Name:      "Chimoio Airport",
		City:      "Chimoio",
		State:     "Manica",
		Country:   "MZ",
		Elevation: 2287,
		Latitude:  -19.1513004303,
		Longitude: 33.4290008545,
		Timezone:  "Africa/Maputo",
	},
	"FQFU": {
		ICAO:      "FQFU",
		Name:      "Furancungo Airport",
		City:      "Furancungo",
		State:     "Tete",
		Country:   "MZ",
		Latitude:  -14.9186000824,
		Longitude: 33.6282997131,
		Timezone:  "Africa/Maputo",
	},
	"FQIA": {
		ICAO:      "FQIA",
		IATA:      "IHC",
		Name:      "Inhaca Airport",
		City:      "Inhaca",
		State:     "Maputo",
		Country:   "MZ",
		Elevation: 10,
		Latitude:  -25.9971446991,
		Longitude: 32.9293518066,
		Timezone:  "Africa/Maputo",
	},
	"FQIN": {
		ICAO:      "FQIN",
		IATA:      "INH",
		Name:      "Inhambane Airport",
		City:      "Inhambabe",
		State:     "Inhambane",
		Country:   "MZ",
		Elevation: 30,
		Latitude:  -23.8763999939,
		Longitude: 35.4085006714,
		Timezone:  "Africa/Maputo",
	},
	"FQLC": {
		ICAO:      "FQLC",
		IATA:      "VXC",
		Name:      "Lichinga Airport",
		City:      "Lichinga",
		State:     "Niassa",
		Country:   "MZ",
		Elevation: 4505,
		Latitude:  -13.2740001678,
		Longitude: 35.2663002014,
		Timezone:  "Africa/Maputo",
	},
	"FQLU": {
		ICAO:      "FQLU",
		IATA:      "LFB",
		Name:      "Lumbo Airport",
		City:      "Lumbo",
		State:     "Nampula",
		Country:   "MZ",
		Elevation: 33,
		Latitude:  -15.0331001282,
		Longitude: 40.6716995239,
		Timezone:  "Africa/Maputo",
	},
	"FQMA": {
		ICAO:      "FQMA",
		IATA:      "MPM",
		Name:      "Maputo Airport",
		City:      "Maputo",
		State:     "Maputo",
		Country:   "MZ",
		Elevation: 145,
		Latitude:  -25.9207992554,
		Longitude: 32.5726013184,
		Timezone:  "Africa/Maputo",
	},
	"FQMD": {
		ICAO:      "FQMD",
		IATA:      "MUD",
		Name:      "Mueda Airport",
		City:      "Mueda",
		State:     "Cabo-Delgado",
		Country:   "MZ",
		Elevation: 2789,
		Latitude:  -11.6729001999,
		Longitude: 39.5630989075,
		Timezone:  "Africa/Maputo",
	},
	"FQMP": {
		ICAO:      "FQMP",
		IATA:      "MZB",
		Name:      "Mocimboa da Praia Airport",
		City:      "Mocimboa da Praia",
		State:     "Cabo-Delgado",
		Country:   "MZ",
		Elevation: 89,
		Latitude:  -11.3618001938,
		Longitude: 40.3549003601,
		Timezone:  "Africa/Maputo",
	},
	"FQMR": {
		ICAO:      "FQMR",
		Name:      "Marrupa Airport",
		City:      "Marrupa",
		State:     "Niassa",
		Country:   "MZ",
		Elevation: 2480,
		Latitude:  -13.2250995636,
		Longitude: 37.5521011353,
		Timezone:  "Africa/Maputo",
	},
	"FQNC": {
		ICAO:      "FQNC",
		IATA:      "MNC",
		Name:      "Nacala Airport",
		City:      "Nacala",
		State:     "Nampula",
		Country:   "MZ",
		Elevation: 410,
		Latitude:  -14.4882001877,
		Longitude: 40.7122001648,
		Timezone:  "Africa/Maputo",
	},
	"FQNP": {
		ICAO:      "FQNP",
		IATA:      "APL",
		Name:      "Nampula Airport",
		City:      "Nampula",
		State:     "Nampula",
		Country:   "MZ",
		Elevation: 1444,
		Latitude:  -15.1056003571,
		Longitude: 39.2817993164,
		Timezone:  "Africa/Maputo",
	},
	"FQPB": {
		ICAO:      "FQPB",
		IATA:      "POL",
		Name:      "Pemba Airport",
		City:      "Pemba / Porto Amelia",
		State:     "Cabo-Delgado",
		Country:   "MZ",
		Elevation: 331,
		Latitude:  -12.9917621613,
		Longitude: 40.5240135193,
		Timezone:  "Africa/Maputo",
	},
	"FQPO": {
		ICAO:      "FQPO",
		IATA:      "PDD",
		Name:      "Ponta do Ouro Airport",
		City:      "Ponta do Ouro",
		State:     "Maputo",
		Country:   "MZ",
		Elevation: 92,
		Latitude:  -26.8285522461,
		Longitude: 32.8377075195,
		Timezone:  "Africa/Maputo",
	},
	"FQQL": {
		ICAO:      "FQQL",
		IATA:      "UEL",
		Name:      "Quelimane Airport",
		City:      "Quelimane",
		State:     "Zambezia",
		Country:   "MZ",
		Elevation: 36,
		Latitude:  -17.8554992676,
		Longitude: 36.8690986633,
		Timezone:  "Africa/Maputo",
	},
	"FQSG": {
		ICAO:      "FQSG",
		Name:      "Songo Airport",
		City:      "Songo",
		State:     "Tete",
		Country:   "MZ",
		Elevation: 2904,
		Latitude:  -15.6027002335,
		Longitude: 32.7732009888,
		Timezone:  "Africa/Maputo",
	},
	"FQTT": {
		ICAO:      "FQTT",
		IATA:      "TET",
		Name:      "Chingozi Airport",
		City:      "Tete",
		State:     "Tete",
		Country:   "MZ",
		Elevation: 525,
		Latitude:  -16.1047992706,
		Longitude: 33.6402015686,
		Timezone:  "Africa/Maputo",
	},
	"FQUG": {
		ICAO:      "FQUG",
		Name:      "Ulongwe Airport",
		City:      "Ulongwe",
		State:     "Tete",
		Country:   "MZ",
		Elevation: 4265,
		Latitude:  -14.7046003342,
		Longitude: 34.3524017334,
		Timezone:  "Africa/Maputo",
	},
	"FQVL": {
		ICAO:      "FQVL",
		IATA:      "VNX",
		Name:      "Vilankulo Airport",
		City:      "Vilanculo",
		State:     "Inhambane",
		Country:   "MZ",
		Elevation: 46,
		Latitude:  -22.0184001923,
		Longitude: 35.3133010864,
		Timezone:  "Africa/Maputo",
	},
	"FQXA": {
		ICAO:      "FQXA",
		IATA:      "VJB",
		Name:      "Xai-Xai Airport",
		City:      "Xai-Xai",
		State:     "Gaza",
		Country:   "MZ",
		Latitude:  -25.0377998352,
		Longitude: 33.6273994446,
		Timezone:  "Africa/Maputo",
	},
	"FSAL": {
		ICAO:      "FSAL",
		Name:      "Alphonse Airport",
		City:      "Alphonse Island",
		Country:   "SC",
		Elevation: 10,
		Latitude:  -7.00478,
		Longitude: 52.7262,
		Timezone:  "Indian/Mahe",
	},
	"FSAS": {
		ICAO:      "FSAS",
		Name:      "Assumption Island Airport",
		City:      "Assumption Island",
		Country:   "SC",
		Elevation: 10,
		Latitude:  -9.74222,
		Longitude: 46.506802,
		Timezone:  "Indian/Mahe",
	},
	"FSDA": {
		ICAO:      "FSDA",
		Name:      "Darros Airport",
		City:      "Darros Island",
		Country:   "SC",
		Elevation: 10,
		Latitude:  -5.417,
		Longitude: 53.299999,
		Timezone:  "Indian/Mahe",
	},
	"FSDR": {
		ICAO:      "FSDR",
		IATA:      "DES",
		Name:      "Desroches Airport",
		City:      "Desroches Island",
		Country:   "SC",
		Elevation: 10,
		Latitude:  -5.6967,
		Longitude: 53.6558,
		Timezone:  "Indian/Mahe",
	},
	"FSFA": {
		ICAO:      "FSFA",
		Name:      "Farquhar Airport",
		City:      "Farquhar Group",
		Country:   "SC",
		Elevation: 10,
		Latitude:  -10.1096,
		Longitude: 51.176102,
		Timezone:  "Indian/Mahe",
	},
	"FSIA": {
		ICAO:      "FSIA",
		IATA:      "SEZ",
		Name:      "Seychelles International Airport",
		City:      "Mahe Island",
		State:     "Pointe-Larue",
		Country:   "SC",
		Elevation: 10,
		Latitude:  -4.6743397713,
		Longitude: 55.5218009949,
		Timezone:  "Indian/Mahe",
	},
	"FSMA": {
		ICAO:      "FSMA",
		Name:      "Marie-Louise Airport",
		City:      "Marie-Louise Island",
		Country:   "SC",
		Elevation: 9,
		Latitude:  -6.17416,
		Longitude: 53.144402,
		Timezone:  "Indian/Mahe",
	},
	"FSPL": {
		ICAO:      "FSPL",
		Name:      "Platte Airport",
		City:      "Platte Island",
		Country:   "SC",
		Elevation: 9,
		Latitude:  -5.85916,
		Longitude: 55.383301,
		Timezone:  "Indian/Mahe",
	},
	"FSPP": {
		ICAO:      "FSPP",
		IATA:      "PRI",
		Name:      "Praslin Airport",
		City:      "Praslin Island",
		State:     "Grand-Anse-Praslin",
		Country:   "SC",
		Elevation: 10,
		Latitude:  -4.3192901611,
		Longitude: 55.6913986206,
		Timezone:  "Indian/Mahe",
	},
	"FSSB": {
		ICAO:      "FSSB",
		IATA:      "BDI",
		Name:      "Bird Island Airport",
		City:      "Bird Island",
		Country:   "SC",
		Elevation: 6,
		Latitude:  -3.72472,
		Longitude: 55.205299,
		Timezone:  "Indian/Mahe",
	},
	"FSSC": {
		ICAO:      "FSSC",
		Name:      "Coetivy Airport",
		City:      "Coetivy Island",
		Country:   "SC",
		Elevation: 10,
		Latitude:  -7.13457,
		Longitude: 56.278198,
		Timezone:  "Indian/Mahe",
	},
	"FSSD": {
		ICAO:      "FSSD",
		IATA:      "DEI",
		Name:      "Denis Island Airport",
		City:      "Denis Island",
		Country:   "SC",
		Elevation: 10,
		Latitude:  -3.80222,
		Longitude: 55.666901,
		Timezone:  "Indian/Mahe",
	},
	"FSSF": {
		ICAO:      "FSSF",
		IATA:      "FRK",
		Name:      "Fregate Island Airport",
		City:      "Fregate Island",
		State:     "Inner-Islands",
		Country:   "SC",
		Elevation: 610,
		Latitude:  -4.583,
		Longitude: 55.950001,
		Timezone:  "Indian/Mahe",
	},
	"FSSR": {
		ICAO:      "FSSR",
		Name:      "Remire Island Airport",
		City:      "Remire Island",
		Country:   "SC",
		Elevation: 9,
		Latitude:  -5.11722,
		Longitude: 53.312199,
		Timezone:  "Indian/Mahe",
	},
	"FTTA": {
		ICAO:      "FTTA",
		IATA:      "SRH",
		Name:      "Sarh Airport",
		City:      "Sarh",
		State:     "Moyen-Chari",
		Country:   "TD",
		Elevation: 1198,
		Latitude:  9.1444396973,
		Longitude: 18.3743991852,
		Timezone:  "Africa/Ndjamena",
	},
	"FTTB": {
		ICAO:      "FTTB",
		IATA:      "OGR",
		Name:      "Bongor Airport",
		City:      "Bongor",
		State:     "Mayo-Kebbi-Est",
		Country:   "TD",
		Elevation: 1076,
		Latitude:  10.2881002426,
		Longitude: 15.3810997009,
		Timezone:  "Africa/Ndjamena",
	},
	"FTTC": {
		ICAO:      "FTTC",
		IATA:      "AEH",
		Name:      "Abeche Airport",
		State:     "Ouadai",
		Country:   "TD",
		Elevation: 1788,
		Latitude:  13.8470001221,
		Longitude: 20.8442993164,
		Timezone:  "Africa/Ndjamena",
	},
	"FTTD": {
		ICAO:      "FTTD",
		IATA:      "MQQ",
		Name:      "Moundou Airport",
		State:     "Logone-Occidental",
		Country:   "TD",
		Elevation: 1407,
		Latitude:  8.6244096756,
		Longitude: 16.0713996887,
		Timezone:  "Africa/Ndjamena",
	},
	"FTTE": {
		ICAO:      "FTTE",
		Name:      "Biltine Airport",
		City:      "Biltine",
		State:     "Wadi-Fira",
		Country:   "TD",
		Elevation: 1680,
		Latitude:  14.5170001984,
		Longitude: 20.8999996185,
		Timezone:  "Africa/Ndjamena",
	},
	"FTTF": {
		ICAO:      "FTTF",
		Name:      "Fada Airport",
		City:      "Fada",
		State:     "Ennedi-Ouest",
		Country:   "TD",
		Elevation: 1788,
		Latitude:  17.1669998169,
		Longitude: 21.5669994354,
		Timezone:  "Africa/Ndjamena",
	},
	"FTTG": {
		ICAO:      "FTTG",
		Name:      "Goz-Beida Airport",
		City:      "Goz-Beida",
		State:     "Ouadai",
		Country:   "TD",
		Elevation: 1765,
		Latitude:  12.1999998093,
		Longitude: 21.466999054,
		Timezone:  "Africa/Ndjamena",
	},
	"FTTH": {
		ICAO:      "FTTH",
		IATA:      "LTC",
		Name:      "Lai Airport",
		City:      "Lai",
		State:     "Tandjile",
		Country:   "TD",
		Elevation: 1171,
		Latitude:  9.3999996185,
		Longitude: 16.2999992371,
		Timezone:  "Africa/Ndjamena",
	},
	"FTTI": {
		ICAO:      "FTTI",
		IATA:      "ATV",
		Name:      "Ati Airport",
		City:      "Ati",
		State:     "Batha",
		Country:   "TD",
		Elevation: 1089,
		Latitude:  13.2389001846,
		Longitude: 18.3132991791,
		Timezone:  "Africa/Ndjamena",
	},
	"FTTJ": {
		ICAO:      "FTTJ",
		IATA:      "NDJ",
		Name:      "N'Djamena International Airport",
		City:      "N'Djamena",
		State:     "Chari-Baguirmi",
		Country:   "TD",
		Elevation: 968,
		Latitude:  12.1337003708,
		Longitude: 15.0340003967,
		Timezone:  "Africa/Ndjamena",
	},
	"FTTK": {
		ICAO:      "FTTK",
		IATA:      "BKR",
		Name:      "Bokoro Airport",
		City:      "Bokoro",
		State:     "Hadjer-Lamis",
		Country:   "TD",
		Elevation: 984,
		Latitude:  12.3830003738,
		Longitude: 17.0669994354,
		Timezone:  "Africa/Ndjamena",
	},
	"FTTL": {
		ICAO:      "FTTL",
		IATA:      "OTC",
		Name:      "Bol Airport",
		City:      "Bol",
		State:     "Lac",
		Country:   "TD",
		Elevation: 955,
		Latitude:  13.4433002472,
		Longitude: 14.73939991,
		Timezone:  "Africa/Ndjamena",
	},
	"FTTM": {
		ICAO:      "FTTM",
		IATA:      "MVO",
		Name:      "Mongo Airport",
		City:      "Mongo",
		State:     "Guera",
		Country:   "TD",
		Elevation: 1414,
		Latitude:  12.1669998169,
		Longitude: 18.6749992371,
		Timezone:  "Africa/Ndjamena",
	},
	"FTTN": {
		ICAO:      "FTTN",
		IATA:      "AMC",
		Name:      "Am Timan Airport",
		City:      "Am Timan",
		State:     "Salamat",
		Country:   "TD",
		Elevation: 1420,
		Latitude:  11.0340003967,
		Longitude: 20.2740001678,
		Timezone:  "Africa/Ndjamena",
	},
	"FTTP": {
		ICAO:      "FTTP",
		IATA:      "PLF",
		Name:      "Pala Airport",
		City:      "Pala",
		State:     "Mayo-Kebbi-Ouest",
		Country:   "TD",
		Elevation: 1532,
		Latitude:  9.3780603409,
		Longitude: 14.9250001907,
		Timezone:  "Africa/Ndjamena",
	},
	"FTTR": {
		ICAO:      "FTTR",
		Name:      "Zouar Airport",
		City:      "Zouar",
		Country:   "TD",
		Elevation: 2655,
		Latitude:  20.4500007629,
		Longitude: 16.5830001831,
		Timezone:  "Africa/Ndjamena",
	},
	"FTTS": {
		ICAO:      "FTTS",
		IATA:      "OUT",
		Name:      "Bousso Airport",
		City:      "Bousso",
		State:     "Chari-Baguirmi",
		Country:   "TD",
		Elevation: 1099,
		Latitude:  10.4829998016,
		Longitude: 16.716999054,
		Timezone:  "Africa/Ndjamena",
	},
	"FTTU": {
		ICAO:      "FTTU",
		IATA:      "AMO",
		Name:      "Mao Airport",
		City:      "Mao",
		State:     "Kanem",
		Country:   "TD",
		Elevation: 1072,
		Latitude:  14.1456003189,
		Longitude: 15.3143997192,
		Timezone:  "Africa/Ndjamena",
	},
	"FTTY": {
		ICAO:      "FTTY",
		IATA:      "FYT",
		Name:      "Faya Largeau Airport",
		State:     "Borkou",
		Country:   "TD",
		Elevation: 771,
		Latitude:  17.9171009064,
		Longitude: 19.1110992432,
		Timezone:  "Africa/Ndjamena",
	},
	"FTTZ": {
		ICAO:      "FTTZ",
		Name:      "Bardai Zougra Airport",
		City:      "Bardai Zougra",
		Country:   "TD",
		Elevation: 3524,
		Latitude:  21.4500007629,
		Longitude: 17.0499992371,
		Timezone:  "Africa/Ndjamena",
	},
	"FV76": {
		ICAO:      "FV76",
		Name:      "Kwekwe East Airport",
		State:     "Midlands",
		Country:   "ZW",
		Elevation: 4025,
		Latitude:  -19.010799408,
		Longitude: 30.0251998901,
		Timezone:  "Africa/Harare",
	},
	"FV77": {
		ICAO:      "FV77",
		Name:      "Mhangura Airport",
		Country:   "ZW",
		Elevation: 4078,
		Latitude:  -16.9101009369,
		Longitude: 30.2423000336,
		Timezone:  "Africa/Harare",
	},
	"FVAB": {
		ICAO:      "FVAB",
		Name:      "Aberdeen Airport",
		City:      "Aberdeen",
		State:     "Manicaland",
		Country:   "ZW",
		Elevation: 4495,
		Latitude:  -18.1669998169,
		Longitude: 32.6669998169,
		Timezone:  "Africa/Harare",
	},
	"FVAE": {
		ICAO:      "FVAE",
		Name:      "Braebourne Airport",
		City:      "Braebourne",
		Country:   "ZW",
		Elevation: 3600,
		Latitude:  -16.6669998169,
		Longitude: 31.2329998016,
		Timezone:  "Africa/Harare",
	},
	"FVBB": {
		ICAO:      "FVBB",
		Name:      "Beit Bridge Airport",
		Country:   "ZW",
		Elevation: 1509,
		Latitude:  -22.2000007629,
		Longitude: 30.0167007446,
		Timezone:  "Africa/Harare",
	},
	"FVBI": {
		ICAO:      "FVBI",
		Name:      "Binga Airport",
		City:      "Binga",
		State:     "Matabeleland-North",
		Country:   "ZW",
		Elevation: 1950,
		Latitude:  -17.6480998993,
		Longitude: 27.3174991608,
		Timezone:  "Africa/Harare",
	},
	"FVBL": {
		ICAO:      "FVBL",
		Name:      "Mabalauta Airport",
		City:      "Mabalauta",
		Country:   "ZW",
		Elevation: 950,
		Latitude:  -21.9169998169,
		Longitude: 31.466999054,
		Timezone:  "Africa/Harare",
	},
	"FVBM": {
		ICAO:      "FVBM",
		Name:      "Bumi Airport",
		City:      "Bumi",
		Country:   "ZW",
		Elevation: 1650,
		Latitude:  -16.8169994354,
		Longitude: 28.3500003815,
		Timezone:  "Africa/Harare",
	},
	"FVBO": {
		ICAO:      "FVBO",
		Name:      "Bosbury Airport",
		City:      "Bosbury",
		State:     "Mashonaland-West",
		Country:   "ZW",
		Elevation: 3940,
		Latitude:  -18.216999054,
		Longitude: 30.1329994202,
		Timezone:  "Africa/Harare",
	},
	"FVBU": {
		ICAO:      "FVBU",
		IATA:      "BUQ",
		Name:      "Joshua Mqabuko Nkomo International Airport",
		City:      "Bulawayo",
		State:     "Matabeleland-North",
		Country:   "ZW",
		Elevation: 4359,
		Latitude:  -20.0174007416,
		Longitude: 28.6179008484,
		Timezone:  "Africa/Harare",
	},
	"FVCC": {
		ICAO:      "FVCC",
		Name:      "C.C. Strip",
		City:      "C.C. Strip",
		Country:   "ZW",
		Elevation: 4050,
		Latitude:  -17.4330005646,
		Longitude: 30.2999992371,
		Timezone:  "Africa/Harare",
	},
	"FVCD": {
		ICAO:      "FVCD",
		Name:      "Chirundu Airport",
		City:      "Chirundu",
		State:     "Mashonaland-West",
		Country:   "ZW",
		Elevation: 1310,
		Latitude:  -16,
		Longitude: 28.8999996185,
		Timezone:  "Africa/Harare",
	},
	"FVCE": {
		ICAO:      "FVCE",
		Name:      "Celina Airport",
		City:      "Celina",
		Country:   "ZW",
		Elevation: 4058,
		Latitude:  -17.5170001984,
		Longitude: 30.8500003815,
		Timezone:  "Africa/Harare",
	},
	"FVCH": {
		ICAO:      "FVCH",
		IATA:      "CHJ",
		Name:      "Chipinge Airport",
		City:      "Chipinge",
		State:     "Manicaland",
		Country:   "ZW",
		Elevation: 3720,
		Latitude:  -20.2066993713,
		Longitude: 32.6282997131,
		Timezone:  "Africa/Harare",
	},
	"FVCM": {
		ICAO:      "FVCM",
		Name:      "Cam+Motor Airport",
		City:      "Cam+Motor",
		State:     "Mashonaland-West",
		Country:   "ZW",
		Elevation: 3720,
		Latitude:  -18.2999992371,
		Longitude: 29.966999054,
		Timezone:  "Africa/Harare",
	},
	"FVCN": {
		ICAO:      "FVCN",
		Name:      "Centenary Airport",
		State:     "Mashonaland-Central",
		Country:   "ZW",
		Elevation: 4058,
		Latitude:  -16.7341003418,
		Longitude: 31.1219997406,
		Timezone:  "Africa/Harare",
	},
	"FVCP": {
		ICAO:      "FVCP",
		Name:      "Charles Prince Airport",
		City:      "Harare",
		State:     "Mashonaland-East",
		Country:   "ZW",
		Elevation: 4845,
		Latitude:  -17.7516002655,
		Longitude: 30.9246997833,
		Timezone:  "Africa/Harare",
	},
	"FVCR": {
		ICAO:      "FVCR",
		Name:      "Chizarira Airport",
		City:      "Chizarira",
		State:     "Matabeleland-North",
		Country:   "ZW",
		Elevation: 3280,
		Latitude:  -17.6830005646,
		Longitude: 27.8999996185,
		Timezone:  "Africa/Harare",
	},
	"FVCV": {
		ICAO:      "FVCV",
		Name:      "Chivu Airport",
		City:      "Chivu",
		State:     "Mashonaland-East",
		Country:   "ZW",
		Elevation: 4790,
		Latitude:  -19.033000946,
		Longitude: 30.8999996185,
		Timezone:  "Africa/Harare",
	},
	"FVCZ": {
		ICAO:      "FVCZ",
		IATA:      "BFO",
		Name:      "Buffalo Range Airport",
		City:      "Chiredzi",
		State:     "Masvingo",
		Country:   "ZW",
		Elevation: 1421,
		Latitude:  -21.0081005096,
		Longitude: 31.5785999298,
		Timezone:  "Africa/Harare",
	},
	"FVDA": {
		ICAO:      "FVDA",
		Name:      "Dawsons Airport",
		City:      "Dawsons",
		Country:   "ZW",
		Elevation: 4800,
		Latitude:  -17.0170001984,
		Longitude: 30.8999996185,
		Timezone:  "Africa/Harare",
	},
	"FVDE": {
		ICAO:      "FVDE",
		Name:      "Deka Airport",
		City:      "Deka",
		Country:   "ZW",
		Elevation: 1750,
		Latitude:  -18.1000003815,
		Longitude: 26.716999054,
		Timezone:  "Africa/Harare",
	},
	"FVDU": {
		ICAO:      "FVDU",
		Name:      "Dudley Airport",
		City:      "Dudley",
		State:     "Mashonaland-East",
		Country:   "ZW",
		Elevation: 4986,
		Latitude:  -18.283000946,
		Longitude: 31.4829998016,
		Timezone:  "Africa/Harare",
	},
	"FVED": {
		ICAO:      "FVED",
		Name:      "Eduan Airport",
		City:      "Eduan",
		State:     "Midlands",
		Country:   "ZW",
		Elevation: 3900,
		Latitude:  -18.7670001984,
		Longitude: 29.783000946,
		Timezone:  "Africa/Harare",
	},
	"FVFA": {
		ICAO:      "FVFA",
		IATA:      "VFA",
		Name:      "Victoria Falls International Airport",
		City:      "Victoria Falls",
		Country:   "ZW",
		Elevation: 3490,
		Latitude:  -18.0958995819,
		Longitude: 25.8390007019,
		Timezone:  "Africa/Harare",
	},
	"FVFG": {
		ICAO:      "FVFG",
		Name:      "Fothergill Airport",
		City:      "Fothergill",
		Country:   "ZW",
		Elevation: 1630,
		Latitude:  -16.7000007629,
		Longitude: 28.6669998169,
		Timezone:  "Africa/Harare",
	},
	"FVFI": {
		ICAO:      "FVFI",
		Name:      "Filabusi Airport",
		City:      "Filabusi",
		State:     "Matabeleland-South",
		Country:   "ZW",
		Elevation: 3525,
		Latitude:  -20.533000946,
		Longitude: 29.2670001984,
		Timezone:  "Africa/Harare",
	},
	"FVGD": {
		ICAO:      "FVGD",
		Name:      "Gwanda Airport",
		City:      "Gwanda",
		State:     "Matabeleland-South",
		Country:   "ZW",
		Elevation: 3328,
		Latitude:  -20.8999996185,
		Longitude: 29,
		Timezone:  "Africa/Harare",
	},
	"FVGO": {
		ICAO:      "FVGO",
		Name:      "Gokwe Airport",
		State:     "Midlands",
		Country:   "ZW",
		Elevation: 4199,
		Latitude:  -18.2000007629,
		Longitude: 28.9167003632,
		Timezone:  "Africa/Harare",
	},
	"FVGR": {
		ICAO:      "FVGR",
		Name:      "Mutara Grand Ref Airport",
		Country:   "ZW",
		Elevation: 3333,
		Latitude:  -18.9775009155,
		Longitude: 32.4508018494,
		Timezone:  "Africa/Harare",
	},
	"FVGT": {
		ICAO:      "FVGT",
		Name:      "Gaths Mine Airport",
		City:      "Gaths Mine",
		State:     "Masvingo",
		Country:   "ZW",
		Elevation: 3454,
		Latitude:  -19.9829998016,
		Longitude: 30.5,
		Timezone:  "Africa/Harare",
	},
	"FVGW": {
		ICAO:      "FVGW",
		Name:      "Gweru Airport",
		City:      "Gweru",
		Country:   "ZW",
		Elevation: 4660,
		Latitude:  -19.5499992371,
		Longitude: 29.7329998016,
		Timezone:  "Africa/Harare",
	},
	"FVHA": {
		ICAO:      "FVHA",
		IATA:      "HRE",
		Name:      "Harare International Airport",
		City:      "Harare",
		State:     "Harare",
		Country:   "ZW",
		Elevation: 4887,
		Latitude:  -17.9318008423,
		Longitude: 31.0928001404,
		Timezone:  "Africa/Harare",
	},
	"FVHP": {
		ICAO:      "FVHP",
		Name:      "Home Park Airport",
		City:      "Home Park",
		State:     "Mashonaland-East",
		Country:   "ZW",
		Elevation: 5150,
		Latitude:  -18.2000007629,
		Longitude: 31.7329998016,
		Timezone:  "Africa/Harare",
	},
	"FVHY": {
		ICAO:      "FVHY",
		Name:      "Hippo Valley Airport",
		City:      "Chiredzi",
		Country:   "ZW",
		Elevation: 1420,
		Latitude:  -21.0499992371,
		Longitude: 31.6669998169,
		Timezone:  "Africa/Harare",
	},
	"FVIN": {
		ICAO:      "FVIN",
		Name:      "Induna Airport",
		City:      "Induna",
		State:     "Matabeleland-North",
		Country:   "ZW",
		Elevation: 4489,
		Latitude:  -20.0830001831,
		Longitude: 28.7000007629,
		Timezone:  "Africa/Harare",
	},
	"FVIT": {
		ICAO:      "FVIT",
		Name:      "Itafa Airport",
		City:      "Itafa",
		State:     "Mashonaland-West",
		Country:   "ZW",
		Elevation: 3800,
		Latitude:  -18.2999992371,
		Longitude: 29.8829994202,
		Timezone:  "Africa/Harare",
	},
	"FVKB": {
		ICAO:      "FVKB",
		IATA:      "KAB",
		Name:      "Kariba International Airport",
		City:      "Kariba",
		Country:   "ZW",
		Elevation: 1706,
		Latitude:  -16.5198001862,
		Longitude: 28.8850002289,
		Timezone:  "Africa/Harare",
	},
	"FVKK": {
		ICAO:      "FVKK",
		Name:      "Kwekwe Airport",
		City:      "Kwekwe",
		Country:   "ZW",
		Elevation: 4000,
		Latitude:  -18.9333000183,
		Longitude: 29.8428001404,
		Timezone:  "Africa/Harare",
	},
	"FVKW": {
		ICAO:      "FVKW",
		Name:      "Mkwasine Airport",
		City:      "Mkwasine",
		State:     "Masvingo",
		Country:   "ZW",
		Elevation: 1620,
		Latitude:  -20.7999992371,
		Longitude: 31.8169994354,
		Timezone:  "Africa/Harare",
	},
	"FVKZ": {
		ICAO:      "FVKZ",
		Name:      "Kezi Airport",
		City:      "Kezi",
		Country:   "ZW",
		Elevation: 3300,
		Latitude:  -20.9169998169,
		Longitude: 28.4829998016,
		Timezone:  "Africa/Harare",
	},
	"FVLA": {
		ICAO:      "FVLA",
		Name:      "Langford Airport",
		City:      "Langford",
		Country:   "ZW",
		Elevation: 4600,
		Latitude:  -17.9330005646,
		Longitude: 30.9500007629,
		Timezone:  "Africa/Harare",
	},
	"FVLG": {
		ICAO:      "FVLG",
		Name:      "Longuiel Airport",
		City:      "Longuiel",
		State:     "Mashonaland-West",
		Country:   "ZW",
		Elevation: 4200,
		Latitude:  -17,
		Longitude: 29.6329994202,
		Timezone:  "Africa/Harare",
	},
	"FVLU": {
		ICAO:      "FVLU",
		Name:      "Lusulu Airport",
		City:      "Lusulu",
		State:     "Matabeleland-North",
		Country:   "ZW",
		Elevation: 3240,
		Latitude:  -18.0830001831,
		Longitude: 27.8670005798,
		Timezone:  "Africa/Harare",
	},
	"FVMA": {
		ICAO:      "FVMA",
		Name:      "Marondera Airport",
		City:      "Marondera",
		State:     "Mashonaland-East",
		Country:   "ZW",
		Elevation: 5370,
		Latitude:  -18.1830005646,
		Longitude: 31.466999054,
		Timezone:  "Africa/Harare",
	},
	"FVMB": {
		ICAO:      "FVMB",
		Name:      "Mashumbi Airport",
		City:      "Mashumbi",
		State:     "Mashonaland-Central",
		Country:   "ZW",
		Elevation: 1240,
		Latitude:  -16.1669998169,
		Longitude: 30.5669994354,
		Timezone:  "Africa/Harare",
	},
	"FVMD": {
		ICAO:      "FVMD",
		Name:      "Mount Darwin Airport",
		State:     "Mashonaland-Central",
		Country:   "ZW",
		Elevation: 3240,
		Latitude:  -16.7691993713,
		Longitude: 31.5597991943,
		Timezone:  "Africa/Harare",
	},
	"FVMF": {
		ICAO:      "FVMF",
		Name:      "Mabikwa Airport",
		City:      "Mabikwa",
		Country:   "ZW",
		Elevation: 3450,
		Latitude:  -18.7329998016,
		Longitude: 27.533000946,
		Timezone:  "Africa/Harare",
	},
	"FVMK": {
		ICAO:      "FVMK",
		Name:      "Mkonono Airport",
		City:      "Mkonono",
		Country:   "ZW",
		Elevation: 4650,
		Latitude:  -17.716999054,
		Longitude: 30.4169998169,
		Timezone:  "Africa/Harare",
	},
	"FVMN": {
		ICAO:      "FVMN",
		Name:      "Mana Pools Airport",
		City:      "Mana Pools",
		State:     "Mashonaland-West",
		Country:   "ZW",
		Elevation: 1300,
		Latitude:  -15.7670001984,
		Longitude: 29.3829994202,
		Timezone:  "Africa/Harare",
	},
	"FVMS": {
		ICAO:      "FVMS",
		Name:      "Middle Sabi Airport",
		City:      "Middle Sabi",
		State:     "Manicaland",
		Country:   "ZW",
		Elevation: 1510,
		Latitude:  -20.216999054,
		Longitude: 32.3670005798,
		Timezone:  "Africa/Harare",
	},
	"FVMT": {
		ICAO:      "FVMT",
		Name:      "Mutoko Airport",
		City:      "Mutoko",
		State:     "Mashonaland-East",
		Country:   "ZW",
		Elevation: 3950,
		Latitude:  -17.4319000244,
		Longitude: 32.1845016479,
		Timezone:  "Africa/Harare",
	},
	"FVMU": {
		ICAO:      "FVMU",
		IATA:      "UTA",
		Name:      "Mutare Airport",
		State:     "Manicaland",
		Country:   "ZW",
		Elevation: 3410,
		Latitude:  -18.9974994659,
		Longitude: 32.6272010803,
		Timezone:  "Africa/Harare",
	},
	"FVMV": {
		ICAO:      "FVMV",
		IATA:      "MVZ",
		Name:      "Masvingo International Airport",
		City:      "Masvingo",
		State:     "Masvingo",
		Country:   "ZW",
		Elevation: 3595,
		Latitude:  -20.0552997589,
		Longitude: 30.8591003418,
		Timezone:  "Africa/Harare",
	},
	"FVMW": {
		ICAO:      "FVMW",
		Name:      "Murewa Airport",
		City:      "Murewa",
		State:     "Mashonaland-East",
		Country:   "ZW",
		Elevation: 4579,
		Latitude:  -17.6499996185,
		Longitude: 31.7999992371,
		Timezone:  "Africa/Harare",
	},
	"FVNY": {
		ICAO:      "FVNY",
		Name:      "Nyanyadzi Airport",
		City:      "Nyanyadzi",
		Country:   "ZW",
		Elevation: 1850,
		Latitude:  -19.7329998016,
		Longitude: 32.4329986572,
		Timezone:  "Africa/Harare",
	},
	"FVOT": {
		ICAO:      "FVOT",
		Name:      "Kotwa Airport",
		State:     "Mashonaland-East",
		Country:   "ZW",
		Elevation: 2450,
		Latitude:  -16.9848003387,
		Longitude: 32.6726989746,
		Timezone:  "Africa/Harare",
	},
	"FVPL": {
		ICAO:      "FVPL",
		Name:      "Plumtree Airport",
		City:      "Plumtree",
		Country:   "ZW",
		Elevation: 4527,
		Latitude:  -20.4829998016,
		Longitude: 27.783000946,
		Timezone:  "Africa/Harare",
	},
	"FVRA": {
		ICAO:      "FVRA",
		Name:      "Ratelshoek Airport",
		City:      "Ratelshoek",
		State:     "Manicaland",
		Country:   "ZW",
		Elevation: 3250,
		Latitude:  -20.1669998169,
		Longitude: 32.8330001831,
		Timezone:  "Africa/Harare",
	},
	"FVRE": {
		ICAO:      "FVRE",
		Name:      "Renroc Airport",
		City:      "Renroc",
		State:     "Mashonaland-West",
		Country:   "ZW",
		Elevation: 4200,
		Latitude:  -16.966999054,
		Longitude: 29.5669994354,
		Timezone:  "Africa/Harare",
	},
	"FVRT": {
		ICAO:      "FVRT",
		Name:      "Rutenga Airport",
		City:      "Rutenga",
		State:     "Masvingo",
		Country:   "ZW",
		Elevation: 1825,
		Latitude:  -21.2329998016,
		Longitude: 30.7329998016,
		Timezone:  "Africa/Harare",
	},
	"FVRU": {
		ICAO:      "FVRU",
		Name:      "Rusape Airport",
		City:      "Rusape",
		State:     "Manicaland",
		Country:   "ZW",
		Elevation: 4560,
		Latitude:  -18.533000946,
		Longitude: 32.1329994202,
		Timezone:  "Africa/Harare",
	},
	"FVSC": {
		ICAO:      "FVSC",
		Name:      "Zisco Airport",
		Country:   "ZW",
		Elevation: 3990,
		Latitude:  -19.0291996002,
		Longitude: 29.7219009399,
		Timezone:  "Africa/Harare",
	},
	"FVSE": {
		ICAO:      "FVSE",
		Name:      "Sanyati Estate Airport",
		City:      "Sanyati Estate",
		Country:   "ZW",
		Elevation: 2790,
		Latitude:  -17.966999054,
		Longitude: 29.216999054,
		Timezone:  "Africa/Harare",
	},
	"FVSH": {
		ICAO:      "FVSH",
		Name:      "Zvishavane Airport",
		State:     "Midlands",
		Country:   "ZW",
		Elevation: 3012,
		Latitude:  -20.2894992828,
		Longitude: 30.0881996155,
		Timezone:  "Africa/Harare",
	},
	"FVSN": {
		ICAO:      "FVSN",
		Name:      "Sun Yet Sen Airport",
		City:      "Sun Yet Sen",
		Country:   "ZW",
		Elevation: 2891,
		Latitude:  -21.3330001831,
		Longitude: 28.5170001984,
		Timezone:  "Africa/Harare",
	},
	"FVSV": {
		ICAO:      "FVSV",
		Name:      "Spray View Airport",
		City:      "Spray View",
		State:     "Matabeleland-North",
		Country:   "ZW",
		Elevation: 3210,
		Latitude:  -17.9169998169,
		Longitude: 25.8169994354,
		Timezone:  "Africa/Harare",
	},
	"FVSX": {
		ICAO:      "FVSX",
		Name:      "Sengwa Gorge Airport",
		City:      "Sengwa Gorge",
		Country:   "ZW",
		Elevation: 2900,
		Latitude:  -18.1669998169,
		Longitude: 28.8500003815,
		Timezone:  "Africa/Harare",
	},
	"FVSY": {
		ICAO:      "FVSY",
		Name:      "Siyalima Airport",
		City:      "Siyalima",
		State:     "Mashonaland-Central",
		Country:   "ZW",
		Elevation: 4540,
		Latitude:  -16.6329994202,
		Longitude: 30.6499996185,
		Timezone:  "Africa/Harare",
	},
	"FVTA": {
		ICAO:      "FVTA",
		Name:      "Tashinga Airport",
		City:      "Tashinga",
		Country:   "ZW",
		Elevation: 1630,
		Latitude:  -16.8169994354,
		Longitude: 28.4330005646,
		Timezone:  "Africa/Harare",
	},
	"FVTD": {
		ICAO:      "FVTD",
		Name:      "Tinfields Airport",
		City:      "Tinfields",
		State:     "Masvingo",
		Country:   "ZW",
		Elevation: 3600,
		Latitude:  -19.9500007629,
		Longitude: 31.4330005646,
		Timezone:  "Africa/Harare",
	},
	"FVTE": {
		ICAO:      "FVTE",
		Name:      "Tengwe Airport",
		City:      "Tengwe",
		Country:   "ZW",
		Elevation: 4000,
		Latitude:  -17.1170005798,
		Longitude: 29.6170005798,
		Timezone:  "Africa/Harare",
	},
	"FVTJ": {
		ICAO:      "FVTJ",
		Name:      "Tonje Airport",
		City:      "Tonje",
		State:     "Manicaland",
		Country:   "ZW",
		Elevation: 1500,
		Latitude:  -20.3169994354,
		Longitude: 32.3499984741,
		Timezone:  "Africa/Harare",
	},
	"FVTL": {
		ICAO:      "FVTL",
		IATA:      "GWE",
		Name:      "Thornhill Air Base",
		City:      "Gweru",
		Country:   "ZW",
		Elevation: 4680,
		Latitude:  -19.4363994598,
		Longitude: 29.8619003296,
		Timezone:  "Africa/Harare",
	},
	"FVTS": {
		ICAO:      "FVTS",
		Name:      "Tsholothso Airport",
		City:      "Tsholothso",
		State:     "Matabeleland-North",
		Country:   "ZW",
		Elevation: 3580,
		Latitude:  -19.7670001984,
		Longitude: 27.7670001984,
		Timezone:  "Africa/Harare",
	},
	"FVTU": {
		ICAO:      "FVTU",
		Name:      "Tuli Airport",
		City:      "Tuli",
		State:     "Matabeleland-South",
		Country:   "ZW",
		Elevation: 1940,
		Latitude:  -21.9349250793,
		Longitude: 29.1963386536,
		Timezone:  "Africa/Harare",
	},
	"FVWD": {
		ICAO:      "FVWD",
		Name:      "Wedza Airport",
		City:      "Wedza",
		State:     "Mashonaland-East",
		Country:   "ZW",
		Elevation: 4600,
		Latitude:  -18.6170005798,
		Longitude: 31.5830001831,
		Timezone:  "Africa/Harare",
	},
	"FVWN": {
		ICAO:      "FVWN",
		IATA:      "HWN",
		Name:      "Hwange National Park Airport",
		City:      "Hwange",
		State:     "Matabeleland-North",
		Country:   "ZW",
		Elevation: 3543,
		Latitude:  -18.6298999786,
		Longitude: 27.0209999084,
		Timezone:  "Africa/Harare",
	},
	"FVWT": {
		ICAO:      "FVWT",
		IATA:      "WKI",
		Name:      "Hwange Airport",
		City:      "Hwange",
		State:     "Matabeleland-North",
		Country:   "ZW",
		Elevation: 2500,
		Latitude:  -18.3624992371,
		Longitude: 26.5167007446,
		Timezone:  "Africa/Harare",
	},
	"FVYT": {
		ICAO:      "FVYT",
		Name:      "Inyati Airport",
		City:      "Inyati",
		State:     "Matabeleland-North",
		Country:   "ZW",
		Elevation: 4400,
		Latitude:  -19.7000007629,
		Longitude: 28.8500003815,
		Timezone:  "Africa/Harare",
	},
	"FVZC": {
		ICAO:      "FVZC",
		Name:      "Zisco Airport",
		City:      "Zisco",
		Country:   "ZW",
		Elevation: 3975,
		Latitude:  -19.033000946,
		Longitude: 29.716999054,
		Timezone:  "Africa/Harare",
	},
	"FVZK": {
		ICAO:      "FVZK",
		Name:      "Zaka Airport",
		City:      "Zaka",
		State:     "Masvingo",
		Country:   "ZW",
		Elevation: 2550,
		Latitude:  -20.3500003815,
		Longitude: 31.4169998169,
		Timezone:  "Africa/Harare",
	},
	"FWBG": {
		ICAO:      "FWBG",
		Name:      "Bangula Airport",
		City:      "Bangula",
		State:     "Southern-Region",
		Country:   "MW",
		Elevation: 302,
		Latitude:  -16.5832996368,
		Longitude: 35.1166992188,
		Timezone:  "Africa/Blantyre",
	},
	"FWCB": {
		ICAO:      "FWCB",
		Name:      "Chilumba Prv Airport",
		City:      "Chilumba",
		State:     "Northern-Region",
		Country:   "MW",
		Elevation: 1611,
		Latitude:  -10.4333000183,
		Longitude: 34.25,
		Timezone:  "Africa/Blantyre",
	},
	"FWCD": {
		ICAO:      "FWCD",
		IATA:      "CEH",
		Name:      "Chelinda Malawi Airport",
		State:     "Northern-Region",
		Country:   "MW",
		Elevation: 7759,
		Latitude:  -10.5500001907,
		Longitude: 33.7999992371,
		Timezone:  "Africa/Blantyre",
	},
	"FWCL": {
		ICAO:      "FWCL",
		IATA:      "BLZ",
		Name:      "Chileka International Airport",
		City:      "Blantyre",
		State:     "Southern-Region",
		Country:   "MW",
		Elevation: 2555,
		Latitude:  -15.6791000366,
		Longitude: 34.9739990234,
		Timezone:  "Africa/Blantyre",
	},
	"FWCM": {
		ICAO:      "FWCM",
		IATA:      "CMK",
		Name:      "Club Makokola Airport",
		City:      "Club Makokola",
		State:     "Southern-Region",
		Country:   "MW",
		Elevation: 1587,
		Latitude:  -14.3069000244,
		Longitude: 35.1324996948,
		Timezone:  "Africa/Blantyre",
	},
	"FWCS": {
		ICAO:      "FWCS",
		Name:      "Ntchisi Airport",
		City:      "Ntchisi",
		State:     "Central-Region",
		Country:   "MW",
		Elevation: 4301,
		Latitude:  -13.3758001328,
		Longitude: 33.8644981384,
		Timezone:  "Africa/Blantyre",
	},
	"FWCT": {
		ICAO:      "FWCT",
		Name:      "Chitipa Airport",
		City:      "Chitipa",
		State:     "Northern-Region",
		Country:   "MW",
		Elevation: 4270,
		Latitude:  -9.6999998093,
		Longitude: 33.266998291,
		Timezone:  "Africa/Blantyre",
	},
	"FWDW": {
		ICAO:      "FWDW",
		IATA:      "DWA",
		Name:      "Dwangwa Airport",
		City:      "Dwangwa",
		State:     "Central-Region",
		Country:   "MW",
		Elevation: 1605,
		Latitude:  -12.5183000565,
		Longitude: 34.1319007874,
		Timezone:  "Africa/Blantyre",
	},
	"FWDZ": {
		ICAO:      "FWDZ",
		Name:      "Dedza Airport",
		City:      "Dedza",
		State:     "Central-Region",
		Country:   "MW",
		Elevation: 5240,
		Latitude:  -14.3830003738,
		Longitude: 34.3170013428,
		Timezone:  "Africa/Blantyre",
	},
	"FWKA": {
		ICAO:      "FWKA",
		IATA:      "KGJ",
		Name:      "Karonga Airport",
		City:      "Karonga",
		Country:   "MW",
		Elevation: 1765,
		Latitude:  -9.9535703659,
		Longitude: 33.8930015564,
		Timezone:  "Africa/Blantyre",
	},
	"FWKB": {
		ICAO:      "FWKB",
		Name:      "Katumbi Airport",
		City:      "Katumbi",
		State:     "Northern-Region",
		Country:   "MW",
		Elevation: 3986,
		Latitude:  -10.8170003891,
		Longitude: 33.516998291,
		Timezone:  "Africa/Blantyre",
	},
	"FWKG": {
		ICAO:      "FWKG",
		IATA:      "KBQ",
		Name:      "Kasungu Airport",
		City:      "Kasungu",
		State:     "Central-Region",
		Country:   "MW",
		Elevation: 3470,
		Latitude:  -13.0145998001,
		Longitude: 33.4686012268,
		Timezone:  "Africa/Blantyre",
	},
	"FWKI": {
		ICAO:      "FWKI",
		IATA:      "LLW",
		Name:      "Lilongwe International Airport",
		City:      "Lilongwe",
		State:     "Central-Region",
		Country:   "MW",
		Elevation: 4035,
		Latitude:  -13.7894001007,
		Longitude: 33.78099823,
		Timezone:  "Africa/Blantyre",
	},
	"FWKK": {
		ICAO:      "FWKK",
		Name:      "Nkhota Kota Airport",
		City:      "Nkhota Kota",
		State:     "Central-Region",
		Country:   "MW",
		Elevation: 1720,
		Latitude:  -12.9169998169,
		Longitude: 34.266998291,
		Timezone:  "Africa/Blantyre",
	},
	"FWLE": {
		ICAO:      "FWLE",
		Name:      "Lilongwe Airport",
		City:      "Lilongwe",
		State:     "Central-Region",
		Country:   "MW",
		Elevation: 3722,
		Latitude:  -13.9659996033,
		Longitude: 33.7018013,
		Timezone:  "Africa/Blantyre",
	},
	"FWLK": {
		ICAO:      "FWLK",
		IATA:      "LIX",
		Name:      "Likoma Island Airport",
		City:      "Likoma Island",
		State:     "Northern-Region",
		Country:   "MW",
		Elevation: 1600,
		Latitude:  -12.0830001831,
		Longitude: 34.733001709,
		Timezone:  "Africa/Blantyre",
	},
	"FWLP": {
		ICAO:      "FWLP",
		Name:      "Lifupa Airport",
		City:      "Lifupa",
		State:     "Central-Region",
		Country:   "MW",
		Elevation: 3326,
		Latitude:  -13.0500001907,
		Longitude: 33.1500015259,
		Timezone:  "Africa/Blantyre",
	},
	"FWMC": {
		ICAO:      "FWMC",
		Name:      "Mchinji Airport",
		City:      "Mchinji",
		State:     "Central-Region",
		Country:   "MW",
		Elevation: 3901,
		Latitude:  -13.8000001907,
		Longitude: 32.9000015259,
		Timezone:  "Africa/Blantyre",
	},
	"FWMG": {
		ICAO:      "FWMG",
		IATA:      "MAI",
		Name:      "Mangochi Airport",
		City:      "Mangochi",
		State:     "Southern-Region",
		Country:   "MW",
		Elevation: 1580,
		Latitude:  -14.4829998016,
		Longitude: 35.266998291,
		Timezone:  "Africa/Blantyre",
	},
	"FWMY": {
		ICAO:      "FWMY",
		IATA:      "MYZ",
		Name:      "Monkey Bay Airport",
		City:      "Monkey Bay",
		State:     "Southern-Region",
		Country:   "MW",
		Elevation: 1580,
		Latitude:  -14.0836000443,
		Longitude: 34.9197006226,
		Timezone:  "Africa/Blantyre",
	},
	"FWMZ": {
		ICAO:      "FWMZ",
		Name:      "Mzimba Airport",
		City:      "Mzimba",
		State:     "Northern-Region",
		Country:   "MW",
		Elevation: 4440,
		Latitude:  -11.8830003738,
		Longitude: 33.6170005798,
		Timezone:  "Africa/Blantyre",
	},
	"FWNB": {
		ICAO:      "FWNB",
		Name:      "Ngabu Airport",
		City:      "Ngabu",
		State:     "Southern-Region",
		Country:   "MW",
		Elevation: 450,
		Latitude:  -16.4829998016,
		Longitude: 34.8499984741,
		Timezone:  "Africa/Blantyre",
	},
	"FWSJ": {
		ICAO:      "FWSJ",
		Name:      "Nsanje Airport",
		City:      "Nsanje",
		State:     "Southern-Region",
		Country:   "MW",
		Elevation: 200,
		Latitude:  -16.9169998169,
		Longitude: 35.25,
		Timezone:  "Africa/Blantyre",
	},
	"FWSM": {
		ICAO:      "FWSM",
		IATA:      "LMB",
		Name:      "Salima Airport",
		City:      "Salima",
		State:     "Central-Region",
		Country:   "MW",
		Elevation: 1688,
		Latitude:  -13.7558927025,
		Longitude: 34.5841884613,
		Timezone:  "Africa/Blantyre",
	},
	"FWSU": {
		ICAO:      "FWSU",
		Name:      "Nchalo Sucoma Airport",
		State:     "Southern-Region",
		Country:   "MW",
		Elevation: 230,
		Latitude:  -16.2667007446,
		Longitude: 34.9166984558,
		Timezone:  "Africa/Blantyre",
	},
	"FWTK": {
		ICAO:      "FWTK",
		Name:      "Mtakatata Airport",
		City:      "Mtakatata",
		State:     "Central-Region",
		Country:   "MW",
		Elevation: 1725,
		Latitude:  -14.2170000076,
		Longitude: 34.533000946,
		Timezone:  "Africa/Blantyre",
	},
	"FWUU": {
		ICAO:      "FWUU",
		IATA:      "ZZU",
		Name:      "Mzuzu Airport",
		City:      "Mzuzu",
		State:     "Northern-Region",
		Country:   "MW",
		Elevation: 4115,
		Latitude:  -11.4447002411,
		Longitude: 34.0117988586,
		Timezone:  "Africa/Blantyre",
	},
	"FWZA": {
		ICAO:      "FWZA",
		Name:      "Zomba Airport",
		City:      "Zomba",
		State:     "Southern-Region",
		Country:   "MW",
		Elevation: 2650,
		Latitude:  -15.3853998184,
		Longitude: 35.3844985962,
		Timezone:  "Africa/Blantyre",
	},
	"FXBB": {
		ICAO:      "FXBB",
		Name:      "Bobete Airport",
		City:      "Bobete",
		Country:   "LS",
		Elevation: 7100,
		Latitude:  -29.4249992371,
		Longitude: 28.6669998169,
		Timezone:  "Africa/Maseru",
	},
	"FXKA": {
		ICAO:      "FXKA",
		Name:      "Katse Airport",
		City:      "Katse",
		State:     "Thaba-Tseka",
		Country:   "LS",
		Elevation: 7000,
		Latitude:  -29.3637008667,
		Longitude: 28.527299881,
		Timezone:  "Africa/Maseru",
	},
	"FXKB": {
		ICAO:      "FXKB",
		Name:      "Kolberg Airport",
		City:      "Kolberg",
		State:     "Thaba-Tseka",
		Country:   "LS",
		Elevation: 7000,
		Latitude:  -29.3803005219,
		Longitude: 28.4906997681,
		Timezone:  "Africa/Maseru",
	},
	"FXLK": {
		ICAO:      "FXLK",
		IATA:      "LEF",
		Name:      "Lebakeng Airport",
		City:      "Lebakeng",
		State:     "Thaba-Tseka",
		Country:   "LS",
		Elevation: 6000,
		Latitude:  -29.8908004761,
		Longitude: 28.6555995941,
		Timezone:  "Africa/Maseru",
	},
	"FXLR": {
		ICAO:      "FXLR",
		IATA:      "LRB",
		Name:      "Leribe Airport",
		City:      "Leribe",
		State:     "Leribe",
		Country:   "LS",
		Elevation: 5350,
		Latitude:  -28.8556003571,
		Longitude: 28.0527992249,
		Timezone:  "Africa/Johannesburg",
	},
	"FXLS": {
		ICAO:      "FXLS",
		IATA:      "LES",
		Name:      "Lesobeng Airport",
		City:      "Lesobeng",
		State:     "Thaba-Tseka",
		Country:   "LS",
		Elevation: 7130,
		Latitude:  -29.7828998566,
		Longitude: 28.3166999817,
		Timezone:  "Africa/Maseru",
	},
	"FXLT": {
		ICAO:      "FXLT",
		Name:      "Letseng Airport",
		City:      "Letseng",
		State:     "Mokhotlong",
		Country:   "LS",
		Elevation: 10400,
		Latitude:  -29.0093002319,
		Longitude: 28.8547000885,
		Timezone:  "Africa/Maseru",
	},
	"FXMA": {
		ICAO:      "FXMA",
		IATA:      "MSG",
		Name:      "Matsaile Airport",
		City:      "Matsaile",
		State:     "Thaba-Tseka",
		Country:   "LS",
		Elevation: 6200,
		Latitude:  -29.8405990601,
		Longitude: 28.7763996124,
		Timezone:  "Africa/Maseru",
	},
	"FXMF": {
		ICAO:      "FXMF",
		IATA:      "MFC",
		Name:      "Mafeteng Airport",
		City:      "Mafeteng",
		State:     "Mafeteng",
		Country:   "LS",
		Elevation: 5350,
		Latitude:  -29.8010997772,
		Longitude: 27.2436008453,
		Timezone:  "Africa/Maseru",
	},
	"FXMH": {
		ICAO:      "FXMH",
		Name:      "Mohale's Hoek Airport",
		City:      "Mohale's Hoek",
		Country:   "LS",
		Elevation: 5146,
		Latitude:  -30.1445007324,
		Longitude: 27.4708003998,
		Timezone:  "Africa/Maseru",
	},
	"FXMK": {
		ICAO:      "FXMK",
		IATA:      "MKH",
		Name:      "Mokhotlong Airport",
		City:      "Mokhotlong",
		State:     "Mokhotlong",
		Country:   "LS",
		Elevation: 7200,
		Latitude:  -29.2817993164,
		Longitude: 29.0727996826,
		Timezone:  "Africa/Maseru",
	},
	"FXML": {
		ICAO:      "FXML",
		Name:      "Malefiloane Airstrip",
		City:      "Malefiloane",
		Country:   "LS",
		Elevation: 8267,
		Latitude:  -29.3355007172,
		Longitude: 29.185300827,
		Timezone:  "Africa/Maseru",
	},
	"FXMM": {
		ICAO:      "FXMM",
		IATA:      "MSU",
		Name:      "Moshoeshoe I International Airport",
		City:      "Maseru",
		State:     "Maseru",
		Country:   "LS",
		Elevation: 5348,
		Latitude:  -29.4622993469,
		Longitude: 27.5524997711,
		Timezone:  "Africa/Maseru",
	},
	"FXMN": {
		ICAO:      "FXMN",
		Name:      "Mantsonyane Airport",
		City:      "Mantsonyane",
		State:     "Thaba-Tseka",
		Country:   "LS",
		Elevation: 7100,
		Latitude:  -29.5461006165,
		Longitude: 28.2709999084,
		Timezone:  "Africa/Maseru",
	},
	"FXMP": {
		ICAO:      "FXMP",
		Name:      "Mohlanapeng Airport",
		City:      "Mohlanapeng",
		Country:   "LS",
		Elevation: 7294,
		Latitude:  -29.5830001831,
		Longitude: 28.6830005646,
		Timezone:  "Africa/Maseru",
	},
	"FXMS": {
		ICAO:      "FXMS",
		Name:      "Mashai Store Airport",
		City:      "Mashai Store",
		State:     "Thaba-Tseka",
		Country:   "LS",
		Elevation: 7300,
		Latitude:  -29.6830005646,
		Longitude: 28.7999992371,
		Timezone:  "Africa/Maseru",
	},
	"FXMT": {
		ICAO:      "FXMT",
		Name:      "Matabeng Store Airport",
		City:      "Matabeng Store",
		Country:   "LS",
		Elevation: 6300,
		Latitude:  -29.783000946,
		Longitude: 28.7670001984,
		Timezone:  "Africa/Maseru",
	},
	"FXMU": {
		ICAO:      "FXMU",
		Name:      "Mejametalana Airbase",
		City:      "Maseru",
		State:     "Maseru",
		Country:   "LS",
		Elevation: 5105,
		Latitude:  -29.3041000366,
		Longitude: 27.5034999847,
		Timezone:  "Africa/Johannesburg",
	},
	"FXMV": {
		ICAO:      "FXMV",
		Name:      "Matabeng Village Airport",
		City:      "Matabeng Village",
		Country:   "LS",
		Elevation: 6150,
		Latitude:  -29.814699173,
		Longitude: 28.8082008362,
		Timezone:  "Africa/Maseru",
	},
	"FXNH": {
		ICAO:      "FXNH",
		Name:      "Nohanas Airport",
		City:      "Nohanas",
		Country:   "LS",
		Elevation: 5400,
		Latitude:  -30.0669994354,
		Longitude: 27.8670005798,
		Timezone:  "Africa/Maseru",
	},
	"FXNK": {
		ICAO:      "FXNK",
		IATA:      "NKU",
		Name:      "Nkaus Airport",
		City:      "Nkaus",
		State:     "Mohaleʼs-Hoek",
		Country:   "LS",
		Elevation: 5621,
		Latitude:  -30.0216999054,
		Longitude: 28.1968994141,
		Timezone:  "Africa/Maseru",
	},
	"FXPG": {
		ICAO:      "FXPG",
		IATA:      "PEL",
		Name:      "Pelaneng Airport",
		City:      "Pelaneng",
		State:     "Leribe",
		Country:   "LS",
		Elevation: 7200,
		Latitude:  -29.1205997467,
		Longitude: 28.5053005219,
		Timezone:  "Africa/Maseru",
	},
	"FXQG": {
		ICAO:      "FXQG",
		IATA:      "UTG",
		Name:      "Quthing Airport",
		City:      "Quthing",
		State:     "Quthing",
		Country:   "LS",
		Elevation: 5350,
		Latitude:  -30.4074993134,
		Longitude: 27.6933002472,
		Timezone:  "Africa/Maseru",
	},
	"FXQN": {
		ICAO:      "FXQN",
		IATA:      "UNE",
		Name:      "Qacha's Nek Airport",
		City:      "Qacha's Nek",
		State:     "Qachaʼs-Nek",
		Country:   "LS",
		Elevation: 6100,
		Latitude:  -30.111700058,
		Longitude: 28.6718997955,
		Timezone:  "Africa/Johannesburg",
	},
	"FXSE": {
		ICAO:      "FXSE",
		Name:      "Sehlabathebe Airport",
		City:      "Sehlabathebe",
		Country:   "LS",
		Elevation: 7300,
		Latitude:  -29.9158000946,
		Longitude: 29.0387992859,
		Timezone:  "Africa/Maseru",
	},
	"FXSH": {
		ICAO:      "FXSH",
		IATA:      "SHK",
		Name:      "Sehonghong Airport",
		City:      "Sehonghong",
		State:     "Thaba-Tseka",
		Country:   "LS",
		Elevation: 6500,
		Latitude:  -29.7308998108,
		Longitude: 28.7688999176,
		Timezone:  "Africa/Maseru",
	},
	"FXSK": {
		ICAO:      "FXSK",
		IATA:      "SKQ",
		Name:      "Sekakes Airport",
		City:      "Sekakes",
		State:     "Mohaleʼs-Hoek",
		Country:   "LS",
		Elevation: 5700,
		Latitude:  -30.0389003754,
		Longitude: 28.370300293,
		Timezone:  "Africa/Maseru",
	},
	"FXSM": {
		ICAO:      "FXSM",
		IATA:      "SOK",
		Name:      "Semonkong Airport",
		City:      "Semonkong",
		State:     "Maseru",
		Country:   "LS",
		Elevation: 7200,
		Latitude:  -29.8386001587,
		Longitude: 28.0599994659,
		Timezone:  "Africa/Maseru",
	},
	"FXSS": {
		ICAO:      "FXSS",
		IATA:      "SHZ",
		Name:      "Seshutes Airport",
		City:      "Seshutes",
		State:     "Leribe",
		Country:   "LS",
		Elevation: 7000,
		Latitude:  -29.2675991058,
		Longitude: 28.5522994995,
		Timezone:  "Africa/Maseru",
	},
	"FXST": {
		ICAO:      "FXST",
		Name:      "St. Theresa Airport",
		City:      "St. Theresa",
		Country:   "LS",
		Elevation: 6800,
		Latitude:  -29.6170005798,
		Longitude: 28.783000946,
		Timezone:  "Africa/Maseru",
	},
	"FXTA": {
		ICAO:      "FXTA",
		IATA:      "THB",
		Name:      "Thaba-Tseka Airport",
		City:      "Thaba-Tseka",
		State:     "Thaba-Tseka",
		Country:   "LS",
		Elevation: 7500,
		Latitude:  -29.5228004456,
		Longitude: 28.6158008575,
		Timezone:  "Africa/Maseru",
	},
	"FXTB": {
		ICAO:      "FXTB",
		Name:      "Tebellong Airport",
		City:      "Tebellong",
		Country:   "LS",
		Elevation: 5600,
		Latitude:  -30.0499992371,
		Longitude: 28.4330005646,
		Timezone:  "Africa/Maseru",
	},
	"FXTK": {
		ICAO:      "FXTK",
		IATA:      "TKO",
		Name:      "Tlokoeng Airport",
		City:      "Tlokoeng",
		State:     "Mokhotlong",
		Country:   "LS",
		Elevation: 7000,
		Latitude:  -29.2329998016,
		Longitude: 28.8829994202,
		Timezone:  "Africa/Maseru",
	},
	"FYAB": {
		ICAO:      "FYAB",
		Name:      "Aroab B Airport",
		City:      "Aroab",
		State:     "Karas",
		Country:   "NA",
		Elevation: 3235,
		Latitude:  -26.7761001587,
		Longitude: 19.6331005096,
		Timezone:  "Africa/Windhoek",
	},
	"FYAM": {
		ICAO:      "FYAM",
		Name:      "Aminuis Airstrip",
		City:      "Aminuis",
		State:     "Omaheke",
		Country:   "NA",
		Elevation: 4012,
		Latitude:  -23.6557998657,
		Longitude: 19.3516998291,
		Timezone:  "Africa/Windhoek",
	},
	"FYAR": {
		ICAO:      "FYAR",
		IATA:      "ADI",
		Name:      "Arandis Airport",
		City:      "Arandis",
		Country:   "NA",
		Elevation: 1905,
		Latitude:  -22.4622001648,
		Longitude: 14.9799995422,
		Timezone:  "Africa/Windhoek",
	},
	"FYAS": {
		ICAO:      "FYAS",
		Name:      "Aus Airport",
		City:      "Aus",
		State:     "Karas",
		Country:   "NA",
		Elevation: 4856,
		Latitude:  -26.6938991547,
		Longitude: 16.3188991547,
		Timezone:  "Africa/Windhoek",
	},
	"FYAV": {
		ICAO:      "FYAV",
		Name:      "Ariamsvley Airport",
		City:      "Ariamsvley",
		State:     "Karas",
		Country:   "NA",
		Elevation: 2549,
		Latitude:  -28.1189002991,
		Longitude: 19.8339004517,
		Timezone:  "Africa/Windhoek",
	},
	"FYBC": {
		ICAO:      "FYBC",
		Name:      "Bethanien Airport",
		City:      "Bethanien",
		State:     "Karas",
		Country:   "NA",
		Elevation: 3222,
		Latitude:  -26.5897006989,
		Longitude: 17.1674995422,
		Timezone:  "Africa/Windhoek",
	},
	"FYBJ": {
		ICAO:      "FYBJ",
		Name:      "Bitterwasser Lodge & Flying Club Airfield",
		City:      "Bitterwasser",
		State:     "Hardap",
		Country:   "NA",
		Elevation: 4167,
		Latitude:  -23.875,
		Longitude: 17.9911003113,
		Timezone:  "Africa/Windhoek",
	},
	"FYEK": {
		ICAO:      "FYEK",
		Name:      "Epukiro Airport",
		City:      "Epukiro",
		State:     "Omaheke",
		Country:   "NA",
		Elevation: 4892,
		Latitude:  -21.786699295,
		Longitude: 19.1061000824,
		Timezone:  "Africa/Windhoek",
	},
	"FYEN": {
		ICAO:      "FYEN",
		Name:      "Eenhana Airport",
		City:      "Eenhana",
		Country:   "NA",
		Elevation: 3660,
		Latitude:  -17.4829998016,
		Longitude: 16.3220005035,
		Timezone:  "Africa/Windhoek",
	},
	"FYGB": {
		ICAO:      "FYGB",
		IATA:      "GOG",
		Name:      "Gobabis Airport",
		City:      "Gobabis",
		Country:   "NA",
		Elevation: 4731,
		Latitude:  -22.5044002533,
		Longitude: 18.9731006622,
		Timezone:  "Africa/Windhoek",
	},
	"FYGC": {
		ICAO:      "FYGC",
		Name:      "Gochas Airstrip",
		City:      "Gochas",
		State:     "Hardap",
		Country:   "NA",
		Elevation: 3714,
		Latitude:  -24.8563995361,
		Longitude: 18.8178005219,
		Timezone:  "Africa/Windhoek",
	},
	"FYGF": {
		ICAO:      "FYGF",
		IATA:      "GFY",
		Name:      "Grootfontein Airport",
		City:      "Grootfontein",
		State:     "Otjozondjupa",
		Country:   "NA",
		Elevation: 4636,
		Latitude:  -19.6021995544,
		Longitude: 18.1226997375,
		Timezone:  "Africa/Windhoek",
	},
	"FYGK": {
		ICAO:      "FYGK",
		Name:      "Geluk Kuala Lodge Airport",
		State:     "Hardap",
		Country:   "NA",
		Elevation: 857,
		Latitude:  -24.6299991608,
		Longitude: 15.9392004013,
		Timezone:  "Africa/Windhoek",
	},
	"FYGL": {
		ICAO:      "FYGL",
		Name:      "Omaruru Game Lodge Airport",
		City:      "Omaruru",
		State:     "Erongo",
		Country:   "NA",
		Elevation: 4226,
		Latitude:  -21.3260993958,
		Longitude: 16.0893993378,
		Timezone:  "Africa/Windhoek",
	},
	"FYGV": {
		ICAO:      "FYGV",
		Name:      "Gravenstein Prv Airport",
		State:     "Khomas",
		Country:   "NA",
		Elevation: 4370,
		Latitude:  -23.4589004517,
		Longitude: 17.497800827,
		Timezone:  "Africa/Windhoek",
	},
	"FYHH": {
		ICAO:      "FYHH",
		Name:      "Helmeringhausen Airport",
		City:      "Helmeringhausen",
		State:     "Karas",
		Country:   "NA",
		Elevation: 4603,
		Latitude:  -25.8633003235,
		Longitude: 16.8099994659,
		Timezone:  "Africa/Windhoek",
	},
	"FYHS": {
		ICAO:      "FYHS",
		Name:      "Hobas Airport",
		City:      "Hobas",
		State:     "Karas",
		Country:   "NA",
		Elevation: 2313,
		Latitude:  -27.6243991852,
		Longitude: 17.6933002472,
		Timezone:  "Africa/Windhoek",
	},
	"FYIA": {
		ICAO:      "FYIA",
		Name:      "Intu Africa Pan Airport",
		State:     "Hardap",
		Country:   "NA",
		Elevation: 4012,
		Latitude:  -24.088300705,
		Longitude: 17.9517002106,
		Timezone:  "Africa/Windhoek",
	},
	"FYKA": {
		ICAO:      "FYKA",
		Name:      "Karibib Airport",
		City:      "Karibib",
		State:     "Erongo",
		Country:   "NA",
		Elevation: 3829,
		Latitude:  -21.8477993011,
		Longitude: 15.9027996063,
		Timezone:  "Africa/Windhoek",
	},
	"FYKD": {
		ICAO:      "FYKD",
		Name:      "Kalkfeld Airport",
		City:      "Kalkfeld",
		State:     "Otjozondjupa",
		Country:   "NA",
		Elevation: 5059,
		Latitude:  -20.9013996124,
		Longitude: 16.2077999115,
		Timezone:  "Africa/Windhoek",
	},
	"FYKE": {
		ICAO:      "FYKE",
		Name:      "Kalahari Game Lodge Airport",
		State:     "Karas",
		Country:   "NA",
		Elevation: 3182,
		Latitude:  -25.6511001587,
		Longitude: 19.8780994415,
		Timezone:  "Africa/Windhoek",
	},
	"FYKJ": {
		ICAO:      "FYKJ",
		Name:      "Kamanjab Airport",
		City:      "Kamanjab",
		State:     "Kunene",
		Country:   "NA",
		Elevation: 4259,
		Latitude:  -19.5205993652,
		Longitude: 14.8233003616,
		Timezone:  "Africa/Windhoek",
	},
	"FYKM": {
		ICAO:      "FYKM",
		IATA:      "MPA",
		Name:      "Katima Mulilo Airport",
		City:      "Mpacha",
		State:     "Zambezi",
		Country:   "NA",
		Elevation: 3144,
		Latitude:  -17.6343994141,
		Longitude: 24.176700592,
		Timezone:  "Africa/Windhoek",
	},
	"FYKT": {
		ICAO:      "FYKT",
		IATA:      "KMP",
		Name:      "Keetmanshoop Airport",
		City:      "Keetmanshoop",
		State:     "Karas",
		Country:   "NA",
		Elevation: 3506,
		Latitude:  -26.5398006439,
		Longitude: 18.1114006042,
		Timezone:  "Africa/Windhoek",
	},
	"FYLS": {
		ICAO:      "FYLS",
		IATA:      "LHU",
		Name:      "Lianshulu Airport",
		City:      "Muneambuanas",
		Country:   "NA",
		Elevation: 3143,
		Latitude:  -18.1166992188,
		Longitude: 23.3932991028,
		Timezone:  "Africa/Gaborone",
	},
	"FYLZ": {
		ICAO:      "FYLZ",
		IATA:      "LUD",
		Name:      "Luderitz Airport",
		City:      "Luderitz",
		State:     "Karas",
		Country:   "NA",
		Elevation: 457,
		Latitude:  -26.6874008179,
		Longitude: 15.2428998947,
		Timezone:  "Africa/Windhoek",
	},
	"FYMB": {
		ICAO:      "FYMB",
		Name:      "Meob Bay Landing Site",
		City:      "Meob Bay",
		State:     "Hardap",
		Country:   "NA",
		Elevation: 20,
		Latitude:  -24.6170005798,
		Longitude: 14.6829996109,
		Timezone:  "Africa/Windhoek",
	},
	"FYME": {
		ICAO:      "FYME",
		IATA:      "MJO",
		Name:      "Mount Etjo Airport",
		State:     "Otjozondjupa",
		Country:   "NA",
		Elevation: 5000,
		Latitude:  -21.0233001709,
		Longitude: 16.4528007507,
		Timezone:  "Africa/Windhoek",
	},
	"FYMH": {
		ICAO:      "FYMH",
		Name:      "Maltahoehe Airstrip",
		City:      "Maltahoehe",
		State:     "Hardap",
		Country:   "NA",
		Elevation: 4511,
		Latitude:  -24.771900177,
		Longitude: 16.9794006348,
		Timezone:  "Africa/Windhoek",
	},
	"FYML": {
		ICAO:      "FYML",
		Name:      "Mariental Airport",
		City:      "Mariental",
		State:     "Hardap",
		Country:   "NA",
		Elevation: 3650,
		Latitude:  -24.6054000854,
		Longitude: 17.9253997803,
		Timezone:  "Africa/Windhoek",
	},
	"FYMO": {
		ICAO:      "FYMO",
		IATA:      "OKU",
		Name:      "Mokuti Lodge Airport",
		City:      "Mokuti Lodge",
		Country:   "NA",
		Elevation: 3665,
		Latitude:  -18.8127994537,
		Longitude: 17.0594005585,
		Timezone:  "Africa/Windhoek",
	},
	"FYNA": {
		ICAO:      "FYNA",
		IATA:      "NNI",
		Name:      "Namutoni Airport",
		City:      "Namutoni",
		Country:   "NA",
		Elevation: 3579,
		Latitude:  -18.8064002991,
		Longitude: 16.9272003174,
		Timezone:  "Africa/Windhoek",
	},
	"FYOA": {
		ICAO:      "FYOA",
		IATA:      "OND",
		Name:      "Ondangwa Airport",
		City:      "Ondangwa",
		Country:   "NA",
		Elevation: 3599,
		Latitude:  -17.878200531,
		Longitude: 15.9525995255,
		Timezone:  "Africa/Windhoek",
	},
	"FYOE": {
		ICAO:      "FYOE",
		IATA:      "OMG",
		Name:      "Omega Airport",
		City:      "Omega",
		Country:   "NA",
		Elevation: 3346,
		Latitude:  -18.0303001404,
		Longitude: 22.189699173,
		Timezone:  "Africa/Windhoek",
	},
	"FYOG": {
		ICAO:      "FYOG",
		IATA:      "OMD",
		Name:      "Oranjemund Airport",
		City:      "Oranjemund",
		State:     "Karas",
		Country:   "NA",
		Elevation: 14,
		Latitude:  -28.5846996307,
		Longitude: 16.4466991425,
		Timezone:  "Africa/Johannesburg",
	},
	"FYOJ": {
		ICAO:      "FYOJ",
		Name:      "Outjo Airport",
		City:      "Outjo",
		State:     "Kunene",
		Country:   "NA",
		Elevation: 4334,
		Latitude:  -20.0753002167,
		Longitude: 16.1247005463,
		Timezone:  "Africa/Windhoek",
	},
	"FYOM": {
		ICAO:      "FYOM",
		Name:      "Omaruru Airport",
		City:      "Omaruru",
		State:     "Erongo",
		Country:   "NA",
		Elevation: 3993,
		Latitude:  -21.4150009155,
		Longitude: 15.9380998611,
		Timezone:  "Africa/Windhoek",
	},
	"FYON": {
		ICAO:      "FYON",
		Name:      "Okahandja Airstrip",
		City:      "Okahandja",
		State:     "Otjozondjupa",
		Country:   "NA",
		Elevation: 4321,
		Latitude:  -22.0139007568,
		Longitude: 16.8974990845,
		Timezone:  "Africa/Windhoek",
	},
	"FYOO": {
		ICAO:      "FYOO",
		IATA:      "OKF",
		Name:      "Okaukuejo Airport",
		City:      "Okaukuejo",
		State:     "Kunene",
		Country:   "NA",
		Elevation: 3911,
		Latitude:  -19.1492004395,
		Longitude: 15.9118995667,
		Timezone:  "Africa/Windhoek",
	},
	"FYOU": {
		ICAO:      "FYOU",
		Name:      "Operet Airport",
		City:      "Operet",
		State:     "Oshikoto",
		Country:   "NA",
		Elevation: 3625,
		Latitude:  -18.6093997955,
		Longitude: 17.1497001648,
		Timezone:  "Africa/Windhoek",
	},
	"FYOW": {
		ICAO:      "FYOW",
		IATA:      "OTJ",
		Name:      "Otjiwarongo Airport",
		City:      "Otjiwarongo",
		Country:   "NA",
		Elevation: 4859,
		Latitude:  -20.4347000122,
		Longitude: 16.6608009338,
		Timezone:  "Africa/Windhoek",
	},
	"FYPO": {
		ICAO:      "FYPO",
		Name:      "Pokweni Glider Airport",
		State:     "Hardap",
		Country:   "NA",
		Elevation: 4177,
		Latitude:  -23.6499996185,
		Longitude: 17.7299995422,
		Timezone:  "Africa/Windhoek",
	},
	"FYRC": {
		ICAO:      "FYRC",
		Name:      "Ruacana Airport",
		City:      "Ruacana",
		State:     "Kunene",
		Country:   "NA",
		Elevation: 3765,
		Latitude:  -17.4206008911,
		Longitude: 14.3717002869,
		Timezone:  "Africa/Windhoek",
	},
	"FYRP": {
		ICAO:      "FYRP",
		Name:      "Rosh Pinah Airport",
		City:      "Rosh Pinah",
		State:     "Karas",
		Country:   "NA",
		Elevation: 1274,
		Latitude:  -27.9643001556,
		Longitude: 16.753900528,
		Timezone:  "Africa/Windhoek",
	},
	"FYRR": {
		ICAO:      "FYRR",
		Name:      "Rag Rock Airport",
		State:     "Kunene",
		Country:   "NA",
		Elevation: 2100,
		Latitude:  -20.5380992889,
		Longitude: 14.4330997467,
		Timezone:  "Africa/Windhoek",
	},
	"FYRU": {
		ICAO:      "FYRU",
		IATA:      "NDU",
		Name:      "Rundu Airport",
		City:      "Rundu",
		Country:   "NA",
		Elevation: 3627,
		Latitude:  -17.9564990997,
		Longitude: 19.7194004059,
		Timezone:  "Africa/Windhoek",
	},
	"FYSA": {
		ICAO:      "FYSA",
		IATA:      "RHN",
		Name:      "Skorpion Mine Airport",
		City:      "Rosh Pinah",
		Country:   "NA",
		Elevation: 1870,
		Latitude:  -27.8763999939,
		Longitude: 16.6478004456,
		Timezone:  "Africa/Windhoek",
	},
	"FYSL": {
		ICAO:      "FYSL",
		Name:      "Sossusvlei Moun Airport",
		State:     "Hardap",
		Country:   "NA",
		Elevation: 2844,
		Latitude:  -24.8033008575,
		Longitude: 15.8910999298,
		Timezone:  "Africa/Windhoek",
	},
	"FYSM": {
		ICAO:      "FYSM",
		IATA:      "SWP",
		Name:      "Swakopmund Airport",
		City:      "Swakopmund",
		State:     "Erongo",
		Country:   "NA",
		Elevation: 207,
		Latitude:  -22.6618995667,
		Longitude: 14.5680999756,
		Timezone:  "Africa/Windhoek",
	},
	"FYSN": {
		ICAO:      "FYSN",
		Name:      "Osona Airstrip",
		City:      "Osona",
		State:     "Otjozondjupa",
		Country:   "NA",
		Elevation: 4449,
		Latitude:  -22.10779953,
		Longitude: 16.9797000885,
		Timezone:  "Africa/Windhoek",
	},
	"FYSO": {
		ICAO:      "FYSO",
		Name:      "Solitaire Airport",
		City:      "Solitaire",
		State:     "Khomas",
		Country:   "NA",
		Elevation: 3488,
		Latitude:  -23.900800705,
		Longitude: 16.004699707,
		Timezone:  "Africa/Windhoek",
	},
	"FYSP": {
		ICAO:      "FYSP",
		Name:      "Stampriet Pan Airport",
		City:      "Stampriet",
		State:     "Hardap",
		Country:   "NA",
		Elevation: 3819,
		Latitude:  -24.3521995544,
		Longitude: 18.4333000183,
		Timezone:  "Africa/Windhoek",
	},
	"FYSS": {
		ICAO:      "FYSS",
		IATA:      "SZM",
		Name:      "Sesriem Airstrip",
		State:     "Hardap",
		Country:   "NA",
		Elevation: 2454,
		Latitude:  -24.5128002167,
		Longitude: 15.7467002869,
		Timezone:  "Africa/Windhoek",
	},
	"FYST": {
		ICAO:      "FYST",
		Name:      "Strate Airport",
		State:     "Hardap",
		Country:   "NA",
		Elevation: 4019,
		Latitude:  -23.9678001404,
		Longitude: 18.5489006042,
		Timezone:  "Africa/Windhoek",
	},
	"FYTK": {
		ICAO:      "FYTK",
		Name:      "Tsumkwe Airport",
		City:      "Tsumkwe",
		State:     "Otjozondjupa",
		Country:   "NA",
		Elevation: 3780,
		Latitude:  -19.5849990845,
		Longitude: 20.4528007507,
		Timezone:  "Africa/Windhoek",
	},
	"FYTM": {
		ICAO:      "FYTM",
		IATA:      "TSB",
		Name:      "Tsumeb Airport",
		City:      "Tsumeb",
		Country:   "NA",
		Elevation: 4353,
		Latitude:  -19.2618999481,
		Longitude: 17.7325000763,
		Timezone:  "Africa/Windhoek",
	},
	"FYUS": {
		ICAO:      "FYUS",
		Name:      "Uis Mine Airport",
		City:      "Uis Mine",
		State:     "Erongo",
		Country:   "NA",
		Elevation: 2644,
		Latitude:  -21.2299995422,
		Longitude: 14.8671998978,
		Timezone:  "Africa/Windhoek",
	},
	"FYWB": {
		ICAO:      "FYWB",
		IATA:      "WVB",
		Name:      "Walvis Bay Airport",
		City:      "Walvis Bay",
		State:     "Erongo",
		Country:   "NA",
		Elevation: 299,
		Latitude:  -22.9799003601,
		Longitude: 14.6452999115,
		Timezone:  "Africa/Windhoek",
	},
	"FYWD": {
		ICAO:      "FYWD",
		Name:      "Wolwedans Airport",
		State:     "Hardap",
		Country:   "NA",
		Elevation: 3271,
		Latitude:  -25.1168994904,
		Longitude: 16.0006008148,
		Timezone:  "Africa/Windhoek",
	},
	"FYWE": {
		ICAO:      "FYWE",
		IATA:      "ERS",
		Name:      "Eros Airport",
		City:      "Windhoek",
		State:     "Khomas",
		Country:   "NA",
		Elevation: 5575,
		Latitude:  -22.6121997833,
		Longitude: 17.0804004669,
		Timezone:  "Africa/Windhoek",
	},
	"FYWH": {
		ICAO:      "FYWH",
		IATA:      "WDH",
		Name:      "Hosea Kutako International Airport",
		City:      "Windhoek",
		State:     "Khomas",
		Country:   "NA",
		Elevation: 5640,
		Latitude:  -22.4799003601,
		Longitude: 17.4708995819,
		Timezone:  "Africa/Windhoek",
	},
	"FYWI": {
		ICAO:      "FYWI",
		Name:      "Witvlei Airport",
		City:      "Witvlei",
		State:     "Omaheke",
		Country:   "NA",
		Elevation: 4800,
		Latitude:  -22.4053001404,
		Longitude: 18.4591999054,
		Timezone:  "Africa/Windhoek",
	},
	"FYXX": {
		ICAO:      "FYXX",
		Name:      "Canon Lodge Airport",
		State:     "Karas",
		Country:   "NA",
		Elevation: 3058,
		Latitude:  -27.6597003937,
		Longitude: 17.8377990723,
		Timezone:  "Africa/Windhoek",
	},
	"FZAA": {
		ICAO:      "FZAA",
		IATA:      "FIH",
		Name:      "Ndjili International Airport",
		City:      "Kinshasa",
		Country:   "CD",
		Elevation: 1027,
		Latitude:  -4.3857498169,
		Longitude: 15.4446001053,
		Timezone:  "Africa/Kinshasa",
	},
	"FZAB": {
		ICAO:      "FZAB",
		IATA:      "NLO",
		Name:      "Ndolo Airport",
		State:     "Kinshasa",
		Country:   "CD",
		Elevation: 915,
		Latitude:  -4.3266601563,
		Longitude: 15.3275003433,
		Timezone:  "Africa/Kinshasa",
	},
	"FZAD": {
		ICAO:      "FZAD",
		Name:      "Celo Zongo Airport",
		City:      "Celo Zongo",
		State:     "Bas-Congo",
		Country:   "CD",
		Elevation: 1660,
		Latitude:  -4.7829999924,
		Longitude: 14.9169998169,
		Timezone:  "Africa/Kinshasa",
	},
	"FZAE": {
		ICAO:      "FZAE",
		Name:      "Kimpoko Airport",
		City:      "Kimpoko",
		State:     "Kinshasa",
		Country:   "CD",
		Elevation: 1017,
		Latitude:  -4.2170000076,
		Longitude: 15.5670003891,
		Timezone:  "Africa/Kinshasa",
	},
	"FZAF": {
		ICAO:      "FZAF",
		Name:      "Nsangi Airport",
		City:      "Nsangi",
		State:     "Bas-Congo",
		Country:   "CD",
		Elevation: 2297,
		Latitude:  -5.5999999046,
		Longitude: 15.3170003891,
		Timezone:  "Africa/Kinshasa",
	},
	"FZAG": {
		ICAO:      "FZAG",
		IATA:      "MNB",
		Name:      "Muanda Airport",
		State:     "Bas-Congo",
		Country:   "CD",
		Elevation: 89,
		Latitude:  -5.9308600426,
		Longitude: 12.3517999649,
		Timezone:  "Africa/Kinshasa",
	},
	"FZAH": {
		ICAO:      "FZAH",
		Name:      "Tshela Airport",
		City:      "Tshela",
		State:     "Bas-Congo",
		Country:   "CD",
		Elevation: 361,
		Latitude:  -4.9829998016,
		Longitude: 12.9329996109,
		Timezone:  "Africa/Kinshasa",
	},
	"FZAI": {
		ICAO:      "FZAI",
		Name:      "Kitona Base Airport",
		State:     "Bas-Congo",
		Country:   "CD",
		Elevation: 394,
		Latitude:  -5.9180598259,
		Longitude: 12.4476995468,
		Timezone:  "Africa/Kinshasa",
	},
	"FZAJ": {
		ICAO:      "FZAJ",
		IATA:      "BOA",
		Name:      "Boma Airport",
		City:      "Boma",
		State:     "Bas-Congo",
		Country:   "CD",
		Elevation: 26,
		Latitude:  -5.8540000916,
		Longitude: 13.0640001297,
		Timezone:  "Africa/Kinshasa",
	},
	"FZAL": {
		ICAO:      "FZAL",
		IATA:      "LZI",
		Name:      "Luozi Airport",
		City:      "Luozi",
		State:     "Bas-Congo",
		Country:   "CD",
		Elevation: 722,
		Latitude:  -4.9499998093,
		Longitude: 14.1330003738,
		Timezone:  "Africa/Kinshasa",
	},
	"FZAM": {
		ICAO:      "FZAM",
		IATA:      "MAT",
		Name:      "Tshimpi Airport",
		City:      "Matadi",
		State:     "Bas-Congo",
		Country:   "CD",
		Elevation: 1115,
		Latitude:  -5.7996101379,
		Longitude: 13.4404001236,
		Timezone:  "Africa/Kinshasa",
	},
	"FZAN": {
		ICAO:      "FZAN",
		Name:      "Inga Airport",
		City:      "Inga",
		State:     "Bas-Congo",
		Country:   "CD",
		Elevation: 741,
		Latitude:  -5.5316700935,
		Longitude: 13.5794000626,
		Timezone:  "Africa/Kinshasa",
	},
	"FZAP": {
		ICAO:      "FZAP",
		Name:      "Lukala Airport",
		City:      "Lukala",
		State:     "Bas-Congo",
		Country:   "CD",
		Elevation: 1312,
		Latitude:  -5.5170001984,
		Longitude: 14.5,
		Timezone:  "Africa/Kinshasa",
	},
	"FZAR": {
		ICAO:      "FZAR",
		IATA:      "NKL",
		Name:      "Nkolo Fuma Airport",
		City:      "Nkolo Fuma",
		State:     "Bas-Congo",
		Country:   "CD",
		Elevation: 1476,
		Latitude:  -5.4499998093,
		Longitude: 14.8330001831,
		Timezone:  "Africa/Kinshasa",
	},
	"FZAS": {
		ICAO:      "FZAS",
		Name:      "Inkisi Airport",
		City:      "Inkisi",
		State:     "Bas-Congo",
		Country:   "CD",
		Elevation: 1968,
		Latitude:  -5.1669998169,
		Longitude: 15,
		Timezone:  "Africa/Kinshasa",
	},
	"FZAU": {
		ICAO:      "FZAU",
		Name:      "Konde Airport",
		City:      "Konde",
		Country:   "CD",
		Elevation: 7,
		Latitude:  -5.75,
		Longitude: 12.25,
		Timezone:  "Africa/Kinshasa",
	},
	"FZAW": {
		ICAO:      "FZAW",
		Name:      "Kwilu-Ngongo Airport",
		City:      "Kwilu-Ngongo",
		State:     "Bas-Congo",
		Country:   "CD",
		Elevation: 1296,
		Latitude:  -5.5,
		Longitude: 14.6999998093,
		Timezone:  "Africa/Kinshasa",
	},
	"FZAX": {
		ICAO:      "FZAX",
		Name:      "Luheki Airport",
		City:      "Luheki",
		State:     "Bas-Congo",
		Country:   "CD",
		Elevation: 984,
		Latitude:  -4.8499999046,
		Longitude: 13.7670001984,
		Timezone:  "Africa/Kinshasa",
	},
	"FZAY": {
		ICAO:      "FZAY",
		Name:      "Mvula Sanda Airport",
		City:      "Mvula Sanda",
		State:     "Bas-Congo",
		Country:   "CD",
		Elevation: 1148,
		Latitude:  -5.632999897,
		Longitude: 13.4169998169,
		Timezone:  "Africa/Kinshasa",
	},
	"FZBA": {
		ICAO:      "FZBA",
		IATA:      "INO",
		Name:      "Inongo Airport",
		City:      "Inongo",
		Country:   "CD",
		Elevation: 1040,
		Latitude:  -1.9472199678,
		Longitude: 18.2858009338,
		Timezone:  "Africa/Kinshasa",
	},
	"FZBB": {
		ICAO:      "FZBB",
		Name:      "Bongimba Airport",
		City:      "Bongimba",
		State:     "Mai-Ndombe",
		Country:   "CD",
		Elevation: 1476,
		Latitude:  -3.382999897,
		Longitude: 20,
		Timezone:  "Africa/Kinshasa",
	},
	"FZBC": {
		ICAO:      "FZBC",
		Name:      "Bikoro Airport",
		City:      "Bikoro",
		State:     "Equateur",
		Country:   "CD",
		Elevation: 1312,
		Latitude:  -0.7329999804,
		Longitude: 18.1329994202,
		Timezone:  "Africa/Kinshasa",
	},
	"FZBD": {
		ICAO:      "FZBD",
		Name:      "Oshwe Airport",
		City:      "Oshwe",
		Country:   "CD",
		Elevation: 1542,
		Latitude:  -3.382999897,
		Longitude: 19.5,
		Timezone:  "Africa/Kinshasa",
	},
	"FZBE": {
		ICAO:      "FZBE",
		Name:      "Beno Airport",
		City:      "Beno",
		State:     "Kwilu",
		Country:   "CD",
		Elevation: 1345,
		Latitude:  -3.5999999046,
		Longitude: 17.783000946,
		Timezone:  "Africa/Kinshasa",
	},
	"FZBF": {
		ICAO:      "FZBF",
		Name:      "Bonkita Airport",
		City:      "Bonkita",
		State:     "Mai-Ndombe",
		Country:   "CD",
		Elevation: 931,
		Latitude:  -3.0999999046,
		Longitude: 18.7329998016,
		Timezone:  "Africa/Kinshasa",
	},
	"FZBG": {
		ICAO:      "FZBG",
		Name:      "Kempa Airport",
		City:      "Kempa",
		State:     "Mai-Ndombe",
		Country:   "CD",
		Elevation: 1148,
		Latitude:  -2.9330000877,
		Longitude: 18.3999996185,
		Timezone:  "Africa/Kinshasa",
	},
	"FZBH": {
		ICAO:      "FZBH",
		Name:      "Isongo Airport",
		City:      "Isongo",
		State:     "Mai-Ndombe",
		Country:   "CD",
		Elevation: 1020,
		Latitude:  -1.3999999762,
		Longitude: 18.3999996185,
		Timezone:  "Africa/Kinshasa",
	},
	"FZBI": {
		ICAO:      "FZBI",
		IATA:      "NIO",
		Name:      "Nioki Airport",
		City:      "Nioki",
		Country:   "CD",
		Elevation: 1043,
		Latitude:  -2.7174999714,
		Longitude: 17.6847000122,
		Timezone:  "Africa/Kinshasa",
	},
	"FZBJ": {
		ICAO:      "FZBJ",
		Name:      "Mushie Airport",
		City:      "Mushie",
		State:     "Mai-Ndombe",
		Country:   "CD",
		Elevation: 1214,
		Latitude:  -3,
		Longitude: 16.9330005646,
		Timezone:  "Africa/Kinshasa",
	},
	"FZBK": {
		ICAO:      "FZBK",
		Name:      "Boshwe Airport",
		City:      "Boshwe",
		State:     "Mai-Ndombe",
		Country:   "CD",
		Elevation: 1181,
		Latitude:  -3.0666599274,
		Longitude: 18.6417007446,
		Timezone:  "Africa/Kinshasa",
	},
	"FZBL": {
		ICAO:      "FZBL",
		Name:      "Djokele Airport",
		City:      "Djokele",
		State:     "Mai-Ndombe",
		Country:   "CD",
		Elevation: 1007,
		Latitude:  -3.0499999523,
		Longitude: 17.1000003815,
		Timezone:  "Africa/Kinshasa",
	},
	"FZBN": {
		ICAO:      "FZBN",
		Name:      "Malebo Airport",
		City:      "Malebo",
		State:     "Mai-Ndombe",
		Country:   "CD",
		Elevation: 1411,
		Latitude:  -2.4670000076,
		Longitude: 16.5499992371,
		Timezone:  "Africa/Kinshasa",
	},
	"FZBO": {
		ICAO:      "FZBO",
		IATA:      "FDU",
		Name:      "Bandundu Airport",
		State:     "Kwilu",
		Country:   "CD",
		Elevation: 1063,
		Latitude:  -3.3113200665,
		Longitude: 17.3817005157,
		Timezone:  "Africa/Kinshasa",
	},
	"FZBP": {
		ICAO:      "FZBP",
		Name:      "Bolongonkele Airport",
		City:      "Bolongonkele",
		State:     "Mai-Ndombe",
		Country:   "CD",
		Elevation: 1158,
		Latitude:  -2.7999899387,
		Longitude: 19.9083003998,
		Timezone:  "Africa/Kinshasa",
	},
	"FZBQ": {
		ICAO:      "FZBQ",
		Name:      "Bindja Airport",
		City:      "Bindja",
		State:     "Mai-Ndombe",
		Country:   "CD",
		Elevation: 1201,
		Latitude:  -3.3833301067,
		Longitude: 19.6597003937,
		Timezone:  "Africa/Kinshasa",
	},
	"FZBS": {
		ICAO:      "FZBS",
		Name:      "Semendua Airport",
		City:      "Semendua",
		State:     "Mai-Ndombe",
		Country:   "CD",
		Elevation: 1148,
		Latitude:  -3.1830000877,
		Longitude: 18.0830001831,
		Timezone:  "Africa/Kinshasa",
	},
	"FZBT": {
		ICAO:      "FZBT",
		IATA:      "KRZ",
		Name:      "Basango Mboliasa Airport",
		City:      "Kiri",
		Country:   "CD",
		Elevation: 1013,
		Latitude:  -1.4349999428,
		Longitude: 19.0240001678,
		Timezone:  "Africa/Kinshasa",
	},
	"FZBU": {
		ICAO:      "FZBU",
		Name:      "Ipeke Airport",
		City:      "Ipeke",
		State:     "Mai-Ndombe",
		Country:   "CD",
		Elevation: 1017,
		Latitude:  -2.4830000401,
		Longitude: 18.25,
		Timezone:  "Africa/Kinshasa",
	},
	"FZBV": {
		ICAO:      "FZBV",
		Name:      "Kempile Airport",
		City:      "Kempile",
		State:     "Mai-Ndombe",
		Country:   "CD",
		Elevation: 984,
		Latitude:  -2.7170000076,
		Longitude: 18.1329994202,
		Timezone:  "Africa/Kinshasa",
	},
	"FZBW": {
		ICAO:      "FZBW",
		Name:      "Basengele Airport",
		City:      "Basengele",
		State:     "Mai-Ndombe",
		Country:   "CD",
		Elevation: 1309,
		Latitude:  -1.9170000553,
		Longitude: 17.9169998169,
		Timezone:  "Africa/Kinshasa",
	},
	"FZCA": {
		ICAO:      "FZCA",
		IATA:      "KKW",
		Name:      "Kikwit Airport",
		State:     "Kwilu",
		Country:   "CD",
		Elevation: 1572,
		Latitude:  -5.0357699394,
		Longitude: 18.7856006622,
		Timezone:  "Africa/Kinshasa",
	},
	"FZCB": {
		ICAO:      "FZCB",
		IATA:      "IDF",
		Name:      "Idiofa Airport",
		City:      "Idiofa",
		Country:   "CD",
		Elevation: 2299,
		Latitude:  -5,
		Longitude: 19.6000003815,
		Timezone:  "Africa/Kinshasa",
	},
	"FZCD": {
		ICAO:      "FZCD",
		Name:      "Vanga Airport",
		City:      "Vanga",
		State:     "Kwilu",
		Country:   "CD",
		Elevation: 1312,
		Latitude:  -4.4000000954,
		Longitude: 18.466999054,
		Timezone:  "Africa/Kinshasa",
	},
	"FZCE": {
		ICAO:      "FZCE",
		IATA:      "LUS",
		Name:      "Lusanga Airport",
		City:      "Lusanga",
		Country:   "CD",
		Elevation: 1365,
		Latitude:  -4.8000001907,
		Longitude: 18.716999054,
		Timezone:  "Africa/Kinshasa",
	},
	"FZCF": {
		ICAO:      "FZCF",
		Name:      "Kahemba Airport",
		City:      "Kahemba",
		Country:   "CD",
		Elevation: 3425,
		Latitude:  -7.3330001831,
		Longitude: 19.0170001984,
		Timezone:  "Africa/Kinshasa",
	},
	"FZCI": {
		ICAO:      "FZCI",
		Name:      "Banga Airport",
		City:      "Banga",
		Country:   "CD",
		Elevation: 2493,
		Latitude:  -5.4499998093,
		Longitude: 20.4500007629,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZCK": {
		ICAO:      "FZCK",
		Name:      "Kajiji Airport",
		City:      "Kajiji",
		State:     "Kwango",
		Country:   "CD",
		Elevation: 3510,
		Latitude:  -7.617000103,
		Longitude: 18.5499992371,
		Timezone:  "Africa/Kinshasa",
	},
	"FZCL": {
		ICAO:      "FZCL",
		Name:      "Banza Lute Airport",
		City:      "Banza Lute",
		State:     "Kwilu",
		Country:   "CD",
		Elevation: 1476,
		Latitude:  -4.1999998093,
		Longitude: 17.8330001831,
		Timezone:  "Africa/Kinshasa",
	},
	"FZCM": {
		ICAO:      "FZCM",
		Name:      "Mangai Ii Airport",
		City:      "Mangai Ii",
		State:     "Kwilu",
		Country:   "CD",
		Elevation: 1410,
		Latitude:  -4.0830001831,
		Longitude: 19.5,
		Timezone:  "Africa/Kinshasa",
	},
	"FZCO": {
		ICAO:      "FZCO",
		Name:      "Boko Airport",
		City:      "Boko",
		State:     "Kwango",
		Country:   "CD",
		Elevation: 2297,
		Latitude:  -5.3000001907,
		Longitude: 16.8500003815,
		Timezone:  "Africa/Kinshasa",
	},
	"FZCP": {
		ICAO:      "FZCP",
		Name:      "Popokabaka Airport",
		City:      "Popokabaka",
		Country:   "CD",
		Elevation: 1575,
		Latitude:  -5.6999998093,
		Longitude: 16.5830001831,
		Timezone:  "Africa/Kinshasa",
	},
	"FZCR": {
		ICAO:      "FZCR",
		Name:      "Busala Airport",
		City:      "Busala",
		State:     "Kwilu",
		Country:   "CD",
		Elevation: 1312,
		Latitude:  -4.132999897,
		Longitude: 18.716999054,
		Timezone:  "Africa/Kinshasa",
	},
	"FZCS": {
		ICAO:      "FZCS",
		Name:      "Kenge Airport",
		City:      "Kenge",
		State:     "Kwango",
		Country:   "CD",
		Elevation: 1808,
		Latitude:  -4.8388900757,
		Longitude: 17.0291996002,
		Timezone:  "Africa/Kinshasa",
	},
	"FZCT": {
		ICAO:      "FZCT",
		Name:      "Fatundu Airport",
		City:      "Fatundu",
		State:     "Kwilu",
		Country:   "CD",
		Elevation: 1526,
		Latitude:  -4.132999897,
		Longitude: 17.2999992371,
		Timezone:  "Africa/Kinshasa",
	},
	"FZCU": {
		ICAO:      "FZCU",
		Name:      "Ito Airport",
		City:      "Ito",
		State:     "Kwilu",
		Country:   "CD",
		Elevation: 1148,
		Latitude:  -3.3329999447,
		Longitude: 17.466999054,
		Timezone:  "Africa/Kinshasa",
	},
	"FZCV": {
		ICAO:      "FZCV",
		IATA:      "MSM",
		Name:      "Masi Manimba Airport",
		City:      "Masi Manimba",
		Country:   "CD",
		Elevation: 1952,
		Latitude:  -4.7829999924,
		Longitude: 17.8500003815,
		Timezone:  "Africa/Kinshasa",
	},
	"FZCW": {
		ICAO:      "FZCW",
		Name:      "Kikongo Sur Wamba Airport",
		City:      "Kikongo Sur Wamba",
		State:     "Kwilu",
		Country:   "CD",
		Elevation: 1312,
		Latitude:  -4.2670001984,
		Longitude: 17.2670001984,
		Timezone:  "Africa/Kinshasa",
	},
	"FZCX": {
		ICAO:      "FZCX",
		Name:      "Kimafu Airport",
		City:      "Kimafu",
		State:     "Kwango",
		Country:   "CD",
		Elevation: 1312,
		Latitude:  -4.5999999046,
		Longitude: 17.5830001831,
		Timezone:  "Africa/Kinshasa",
	},
	"FZCY": {
		ICAO:      "FZCY",
		Name:      "Yuki Airport",
		City:      "Yuki",
		State:     "Kwilu",
		Country:   "CD",
		Elevation: 1398,
		Latitude:  -4.0830001831,
		Longitude: 19.4330005646,
		Timezone:  "Africa/Kinshasa",
	},
	"FZDA": {
		ICAO:      "FZDA",
		Name:      "Malanga Airport",
		City:      "Malanga",
		State:     "Bas-Congo",
		Country:   "CD",
		Elevation: 1247,
		Latitude:  -5.5329999924,
		Longitude: 14.8500003815,
		Timezone:  "Africa/Kinshasa",
	},
	"FZDB": {
		ICAO:      "FZDB",
		Name:      "Kimbau Airport",
		City:      "Kimbau",
		State:     "Kwilu",
		Country:   "CD",
		Elevation: 1640,
		Latitude:  -5.617000103,
		Longitude: 17.6000003815,
		Timezone:  "Africa/Kinshasa",
	},
	"FZDD": {
		ICAO:      "FZDD",
		Name:      "Wamba Luadi Airport",
		City:      "Wamba Luadi",
		State:     "Kwango",
		Country:   "CD",
		Elevation: 2297,
		Latitude:  -6.5500001907,
		Longitude: 17.3999996185,
		Timezone:  "Africa/Kinshasa",
	},
	"FZDE": {
		ICAO:      "FZDE",
		Name:      "Tono Airport",
		City:      "Tono",
		State:     "Kwango",
		Country:   "CD",
		Elevation: 2461,
		Latitude:  -6.5500001907,
		Longitude: 18.2000007629,
		Timezone:  "Africa/Kinshasa",
	},
	"FZDF": {
		ICAO:      "FZDF",
		Name:      "Nzamba Airport",
		City:      "Nzamba",
		State:     "Kwango",
		Country:   "CD",
		Elevation: 2953,
		Latitude:  -6.8330001831,
		Longitude: 17.6669998169,
		Timezone:  "Africa/Kinshasa",
	},
	"FZDG": {
		ICAO:      "FZDG",
		Name:      "Nyanga Airport",
		City:      "Nyanga",
		State:     "Kasai",
		Country:   "CD",
		Elevation: 2231,
		Latitude:  -5.9670000076,
		Longitude: 20.4169998169,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZDH": {
		ICAO:      "FZDH",
		Name:      "Ngi Airport",
		City:      "Ngi",
		State:     "Kwilu",
		Country:   "CD",
		Elevation: 1509,
		Latitude:  -4.4169998169,
		Longitude: 17.1669998169,
		Timezone:  "Africa/Kinshasa",
	},
	"FZDJ": {
		ICAO:      "FZDJ",
		Name:      "Mutena Airport",
		City:      "Mutena",
		State:     "Kasai",
		Country:   "CD",
		Elevation: 2395,
		Latitude:  -6.7670001984,
		Longitude: 21.1499996185,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZDK": {
		ICAO:      "FZDK",
		Name:      "Kipata Katika Airport",
		City:      "Kipata Katika",
		State:     "Kwilu",
		Country:   "CD",
		Elevation: 2133,
		Latitude:  -5.0329999924,
		Longitude: 17.6499996185,
		Timezone:  "Africa/Kinshasa",
	},
	"FZDL": {
		ICAO:      "FZDL",
		Name:      "Kolokoso Airport",
		City:      "Kolokoso",
		State:     "Kwango",
		Country:   "CD",
		Elevation: 1312,
		Latitude:  -4.4499998093,
		Longitude: 17.4169998169,
		Timezone:  "Africa/Kinshasa",
	},
	"FZDM": {
		ICAO:      "FZDM",
		Name:      "Masamuna Airport",
		City:      "Masamuna",
		State:     "Kwilu",
		Country:   "CD",
		Elevation: 1476,
		Latitude:  -4.8000001907,
		Longitude: 17.5830001831,
		Timezone:  "Africa/Kinshasa",
	},
	"FZDN": {
		ICAO:      "FZDN",
		Name:      "Mongo Wa Kenda Airport",
		City:      "Mongo Wa Kenda",
		State:     "Kwango",
		Country:   "CD",
		Elevation: 1804,
		Latitude:  -6.9000000954,
		Longitude: 16.9500007629,
		Timezone:  "Africa/Kinshasa",
	},
	"FZDO": {
		ICAO:      "FZDO",
		Name:      "Moanza Airport",
		City:      "Moanza",
		State:     "Kwilu",
		Country:   "CD",
		Elevation: 2297,
		Latitude:  -5.5329999924,
		Longitude: 17.6170005798,
		Timezone:  "Africa/Kinshasa",
	},
	"FZDP": {
		ICAO:      "FZDP",
		Name:      "Mukedi Airport",
		City:      "Mukedi",
		State:     "Kwilu",
		Country:   "CD",
		Elevation: 1804,
		Latitude:  -5.6999998093,
		Longitude: 19.7670001984,
		Timezone:  "Africa/Kinshasa",
	},
	"FZDQ": {
		ICAO:      "FZDQ",
		Name:      "Mazelele Airport",
		City:      "Mazelele",
		State:     "Kwango",
		Country:   "CD",
		Elevation: 1650,
		Latitude:  -7.2829999924,
		Longitude: 17.033000946,
		Timezone:  "Africa/Kinshasa",
	},
	"FZDR": {
		ICAO:      "FZDR",
		Name:      "Bokela Airport",
		City:      "Bokela",
		State:     "Tshuapa",
		Country:   "CD",
		Elevation: 1210,
		Latitude:  -1.1499999762,
		Longitude: 21.8999996185,
		Timezone:  "Africa/Kinshasa",
	},
	"FZDS": {
		ICAO:      "FZDS",
		Name:      "Yasa Bongo Airport",
		City:      "Yasa Bongo",
		State:     "Kwilu",
		Country:   "CD",
		Elevation: 2008,
		Latitude:  -4.4499998093,
		Longitude: 17.783000946,
		Timezone:  "Africa/Kinshasa",
	},
	"FZDT": {
		ICAO:      "FZDT",
		Name:      "Matari Airport",
		City:      "Matari",
		State:     "Kwango",
		Country:   "CD",
		Elevation: 2559,
		Latitude:  -6.1999998093,
		Longitude: 17.6499996185,
		Timezone:  "Africa/Kinshasa",
	},
	"FZDU": {
		ICAO:      "FZDU",
		Name:      "Kimpangu Airport",
		City:      "Kimpangu",
		State:     "Bas-Congo",
		Country:   "CD",
		Elevation: 2133,
		Latitude:  -5.8330001831,
		Longitude: 15,
		Timezone:  "Africa/Kinshasa",
	},
	"FZDY": {
		ICAO:      "FZDY",
		Name:      "Missayi Airport",
		City:      "Missayi",
		State:     "Kwilu",
		Country:   "CD",
		Elevation: 1280,
		Latitude:  -3.882999897,
		Longitude: 17.3500003815,
		Timezone:  "Africa/Kinshasa",
	},
	"FZEA": {
		ICAO:      "FZEA",
		IATA:      "MDK",
		Name:      "Mbandaka Airport",
		City:      "Mbandaka",
		State:     "Equateur",
		Country:   "CD",
		Elevation: 1040,
		Latitude:  0.0226000007,
		Longitude: 18.2887001038,
		Timezone:  "Africa/Kinshasa",
	},
	"FZEB": {
		ICAO:      "FZEB",
		Name:      "Monieka Airport",
		City:      "Monieka",
		State:     "Equateur",
		Country:   "CD",
		Elevation: 1253,
		Latitude:  -0.0670000017,
		Longitude: 19.9829998016,
		Timezone:  "Africa/Kinshasa",
	},
	"FZEI": {
		ICAO:      "FZEI",
		Name:      "Ingende Airport",
		City:      "Ingende",
		State:     "Equateur",
		Country:   "CD",
		Elevation: 1246,
		Latitude:  0.25,
		Longitude: 18.9330005646,
		Timezone:  "Africa/Kinshasa",
	},
	"FZEM": {
		ICAO:      "FZEM",
		Name:      "Yembe Moke Airport",
		City:      "Yembe Moke",
		State:     "Kwilu",
		Country:   "CD",
		Elevation: 1220,
		Latitude:  -4.6830000877,
		Longitude: 18.216999054,
		Timezone:  "Africa/Kinshasa",
	},
	"FZEN": {
		ICAO:      "FZEN",
		IATA:      "BSU",
		Name:      "Basankusu Airport",
		City:      "Basankusu",
		State:     "Equateur",
		Country:   "CD",
		Elevation: 1217,
		Latitude:  1.2247200012,
		Longitude: 19.7889003754,
		Timezone:  "Africa/Kinshasa",
	},
	"FZEO": {
		ICAO:      "FZEO",
		Name:      "Beongo Airport",
		City:      "Beongo",
		State:     "Equateur",
		Country:   "CD",
		Elevation: 1279,
		Latitude:  1.0169999599,
		Longitude: 20.6000003815,
		Timezone:  "Africa/Kinshasa",
	},
	"FZEP": {
		ICAO:      "FZEP",
		Name:      "Mentole Airport",
		City:      "Mentole",
		State:     "Mongala",
		Country:   "CD",
		Elevation: 1295,
		Latitude:  1.3170000315,
		Longitude: 20.7000007629,
		Timezone:  "Africa/Kinshasa",
	},
	"FZER": {
		ICAO:      "FZER",
		Name:      "Kodoro Airport",
		City:      "Kodoro",
		State:     "Equateur",
		Country:   "CD",
		Elevation: 1312,
		Latitude:  1.2829999924,
		Longitude: 20.3330001831,
		Timezone:  "Africa/Kinshasa",
	},
	"FZES": {
		ICAO:      "FZES",
		Name:      "Ngumu Airport",
		City:      "Ngumu",
		State:     "Equateur",
		Country:   "CD",
		Elevation: 1227,
		Latitude:  1.4670000076,
		Longitude: 20.2999992371,
		Timezone:  "Africa/Kinshasa",
	},
	"FZFA": {
		ICAO:      "FZFA",
		IATA:      "LIE",
		Name:      "Libenge Airport",
		City:      "Libenge",
		State:     "Equateur",
		Country:   "CD",
		Elevation: 1125,
		Latitude:  3.632999897,
		Longitude: 18.6329994202,
		Timezone:  "Africa/Kinshasa",
	},
	"FZFB": {
		ICAO:      "FZFB",
		Name:      "Imesse Airport",
		City:      "Imesse",
		State:     "Sud-Ubangi",
		Country:   "CD",
		Elevation: 1110,
		Latitude:  2.5169999599,
		Longitude: 18.283000946,
		Timezone:  "Africa/Kinshasa",
	},
	"FZFC": {
		ICAO:      "FZFC",
		Name:      "Engengele Airport",
		City:      "Engengele",
		State:     "Mongala",
		Country:   "CD",
		Elevation: 1279,
		Latitude:  2.0999999046,
		Longitude: 22.6972007751,
		Timezone:  "Africa/Kinshasa",
	},
	"FZFD": {
		ICAO:      "FZFD",
		IATA:      "BDT",
		Name:      "Gbadolite Airport",
		State:     "Equateur",
		Country:   "CD",
		Elevation: 1509,
		Latitude:  4.2532100677,
		Longitude: 20.9752998352,
		Timezone:  "Africa/Kinshasa",
	},
	"FZFE": {
		ICAO:      "FZFE",
		Name:      "Abumumbazi Airport",
		City:      "Abumumbazi",
		State:     "Nord-Ubangi",
		Country:   "CD",
		Elevation: 1499,
		Latitude:  3.6830000877,
		Longitude: 22.1499996185,
		Timezone:  "Africa/Kinshasa",
	},
	"FZFF": {
		ICAO:      "FZFF",
		Name:      "Bau Airport",
		City:      "Bau",
		State:     "Sud-Ubangi",
		Country:   "CD",
		Elevation: 1640,
		Latitude:  3.7330000401,
		Longitude: 19.0830001831,
		Timezone:  "Africa/Kinshasa",
	},
	"FZFG": {
		ICAO:      "FZFG",
		Name:      "Bokada Airport",
		City:      "Bokada",
		State:     "Nord-Ubangi",
		Country:   "CD",
		Elevation: 1647,
		Latitude:  4.117000103,
		Longitude: 19.4169998169,
		Timezone:  "Africa/Kinshasa",
	},
	"FZFH": {
		ICAO:      "FZFH",
		Name:      "Mokaria-Yamoleka Airport",
		City:      "Mokaria-Yamoleka",
		State:     "Tshopo",
		Country:   "CD",
		Elevation: 1378,
		Latitude:  2.117000103,
		Longitude: 23.283000946,
		Timezone:  "Africa/Kinshasa",
	},
	"FZFJ": {
		ICAO:      "FZFJ",
		Name:      "Goyongo Airport",
		City:      "Goyongo",
		State:     "Nord-Ubangi",
		Country:   "CD",
		Elevation: 1640,
		Latitude:  4.1830000877,
		Longitude: 19.783000946,
		Timezone:  "Africa/Kinshasa",
	},
	"FZFK": {
		ICAO:      "FZFK",
		IATA:      "GMA",
		Name:      "Gemena Airport",
		City:      "Gemena",
		State:     "Sud-Ubangi",
		Country:   "CD",
		Elevation: 1378,
		Latitude:  3.2353699207,
		Longitude: 19.7712993622,
		Timezone:  "Africa/Kinshasa",
	},
	"FZFL": {
		ICAO:      "FZFL",
		Name:      "Kala Airport",
		City:      "Kala",
		State:     "Sud-Ubangi",
		Country:   "CD",
		Elevation: 1640,
		Latitude:  3.383611111,
		Longitude: 18.654722222,
		Timezone:  "Africa/Brazzaville",
	},
	"FZFN": {
		ICAO:      "FZFN",
		Name:      "Lombo Airport",
		City:      "Lombo",
		State:     "Equateur",
		Country:   "CD",
		Elevation: 2331,
		Latitude:  4.4388899803,
		Longitude: 19.5389003754,
		Timezone:  "Africa/Kinshasa",
	},
	"FZFP": {
		ICAO:      "FZFP",
		IATA:      "KLI",
		Name:      "Kotakoli Airport",
		State:     "Equateur",
		Country:   "CD",
		Elevation: 1801,
		Latitude:  4.1576399803,
		Longitude: 21.6508998871,
		Timezone:  "Africa/Kinshasa",
	},
	"FZFQ": {
		ICAO:      "FZFQ",
		Name:      "Mpaka Airport",
		City:      "Mpaka",
		State:     "Sud-Ubangi",
		Country:   "CD",
		Elevation: 1969,
		Latitude:  4.117000103,
		Longitude: 19.216999054,
		Timezone:  "Africa/Kinshasa",
	},
	"FZFR": {
		ICAO:      "FZFR",
		Name:      "Mombongo Airport",
		City:      "Mombongo",
		State:     "Tshopo",
		Country:   "CD",
		Elevation: 1476,
		Latitude:  1.6499999762,
		Longitude: 23.1499996185,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZFS": {
		ICAO:      "FZFS",
		Name:      "Karawa Airport",
		City:      "Karawa",
		State:     "Nord-Ubangi",
		Country:   "CD",
		Elevation: 1640,
		Latitude:  3.367000103,
		Longitude: 20.2999992371,
		Timezone:  "Africa/Kinshasa",
	},
	"FZFT": {
		ICAO:      "FZFT",
		Name:      "Tandala Airport",
		City:      "Tandala",
		State:     "Sud-Ubangi",
		Country:   "CD",
		Elevation: 1640,
		Latitude:  2.9775845186,
		Longitude: 19.3514084816,
		Timezone:  "Africa/Kinshasa",
	},
	"FZFU": {
		ICAO:      "FZFU",
		IATA:      "BMB",
		Name:      "Bumbar Airport",
		City:      "Bumbar",
		State:     "Equateur",
		Country:   "CD",
		Latitude:  2.1827800274,
		Longitude: 22.4817008972,
		Timezone:  "Africa/Kinshasa",
	},
	"FZFV": {
		ICAO:      "FZFV",
		Name:      "Gbado Airport",
		City:      "Gbado",
		State:     "Nord-Ubangi",
		Country:   "CD",
		Elevation: 1476,
		Latitude:  3.882999897,
		Longitude: 20.783000946,
		Timezone:  "Africa/Kinshasa",
	},
	"FZFW": {
		ICAO:      "FZFW",
		Name:      "Gwaka Airport",
		City:      "Gwaka",
		State:     "Sud-Ubangi",
		Country:   "CD",
		Elevation: 1476,
		Latitude:  2.4670000076,
		Longitude: 20.1000003815,
		Timezone:  "Africa/Kinshasa",
	},
	"FZGA": {
		ICAO:      "FZGA",
		IATA:      "LIQ",
		Name:      "Lisala Airport",
		State:     "Equateur",
		Country:   "CD",
		Elevation: 1509,
		Latitude:  2.1706600189,
		Longitude: 21.4969005585,
		Timezone:  "Africa/Kinshasa",
	},
	"FZGB": {
		ICAO:      "FZGB",
		Name:      "Bosondjo Airport",
		City:      "Bosondjo",
		State:     "Mongala",
		Country:   "CD",
		Elevation: 1312,
		Latitude:  1.8669999838,
		Longitude: 21.783000946,
		Timezone:  "Africa/Kinshasa",
	},
	"FZGC": {
		ICAO:      "FZGC",
		Name:      "Bolila Airport",
		City:      "Bolila",
		Country:   "CD",
		Elevation: 1279,
		Latitude:  1.8500000238,
		Longitude: 23.1170005798,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZGE": {
		ICAO:      "FZGE",
		Name:      "Binga Airport",
		City:      "Binga",
		State:     "Mongala",
		Country:   "CD",
		Elevation: 1476,
		Latitude:  2.4330000877,
		Longitude: 20.5,
		Timezone:  "Africa/Kinshasa",
	},
	"FZGF": {
		ICAO:      "FZGF",
		Name:      "Bokungu Airport",
		City:      "Bokungu",
		State:     "Equateur",
		Country:   "CD",
		Elevation: 1214,
		Latitude:  -0.6830000281,
		Longitude: 22.3330001831,
		Timezone:  "Africa/Kinshasa",
	},
	"FZGG": {
		ICAO:      "FZGG",
		Name:      "Mondombe Airport",
		City:      "Mondombe",
		State:     "Tshuapa",
		Country:   "CD",
		Elevation: 1457,
		Latitude:  -0.8830000162,
		Longitude: 22.8169994354,
		Timezone:  "Africa/Kinshasa",
	},
	"FZGH": {
		ICAO:      "FZGH",
		Name:      "Wema Airport",
		City:      "Wema",
		State:     "Tshuapa",
		Country:   "CD",
		Elevation: 1368,
		Latitude:  -0.4329999983,
		Longitude: 21.6499996185,
		Timezone:  "Africa/Kinshasa",
	},
	"FZGI": {
		ICAO:      "FZGI",
		Name:      "Yalingimba Airport",
		City:      "Yalingimba",
		State:     "Mongala",
		Country:   "CD",
		Elevation: 1427,
		Latitude:  2.2829999924,
		Longitude: 22.8500003815,
		Timezone:  "Africa/Kinshasa",
	},
	"FZGN": {
		ICAO:      "FZGN",
		IATA:      "BNB",
		Name:      "Boende Airport",
		City:      "Boende",
		State:     "Equateur",
		Country:   "CD",
		Elevation: 1168,
		Latitude:  -0.2169999927,
		Longitude: 20.8500003815,
		Timezone:  "Africa/Kinshasa",
	},
	"FZGT": {
		ICAO:      "FZGT",
		Name:      "Boteka Airport",
		City:      "Boteka",
		State:     "Equateur",
		Country:   "CD",
		Elevation: 1247,
		Latitude:  -0.3170000017,
		Longitude: 19.0669994354,
		Timezone:  "Africa/Kinshasa",
	},
	"FZGV": {
		ICAO:      "FZGV",
		IATA:      "IKL",
		Name:      "Ikela Airport",
		City:      "Ikela",
		State:     "Equateur",
		Country:   "CD",
		Elevation: 1283,
		Latitude:  -1.2000000477,
		Longitude: 23.283000946,
		Timezone:  "Africa/Kinshasa",
	},
	"FZGX": {
		ICAO:      "FZGX",
		Name:      "Monkoto Airport",
		City:      "Monkoto",
		State:     "Tshuapa",
		Country:   "CD",
		Elevation: 1282,
		Latitude:  -1.6000000238,
		Longitude: 20.6499996185,
		Timezone:  "Africa/Kinshasa",
	},
	"FZGY": {
		ICAO:      "FZGY",
		Name:      "Yemo Airport",
		City:      "Yemo",
		State:     "Tshuapa",
		Country:   "CD",
		Elevation: 1525,
		Latitude:  -0.4670000076,
		Longitude: 21.9330005646,
		Timezone:  "Africa/Kinshasa",
	},
	"FZIA": {
		ICAO:      "FZIA",
		Name:      "Kisangani Simisini Airport",
		State:     "Tshopo",
		Country:   "CD",
		Elevation: 1289,
		Latitude:  0.5174999833,
		Longitude: 25.1550006866,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZIC": {
		ICAO:      "FZIC",
		IATA:      "FKI",
		Name:      "Bangoka International Airport",
		City:      "Kisangani",
		State:     "Tshopo",
		Country:   "CD",
		Elevation: 1417,
		Latitude:  0.4816389978,
		Longitude: 25.3379993439,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZIG": {
		ICAO:      "FZIG",
		Name:      "KM 95 CFL Airport",
		City:      "KM 95 CFL",
		State:     "Tshopo",
		Country:   "CD",
		Elevation: 1591,
		Latitude:  -0.150000006,
		Longitude: 25.3500003815,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZIK": {
		ICAO:      "FZIK",
		Name:      "Katende Airport",
		City:      "Katende",
		State:     "Tshopo",
		Country:   "CD",
		Elevation: 1476,
		Latitude:  0.3330000043,
		Longitude: 25.5,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZIR": {
		ICAO:      "FZIR",
		IATA:      "YAN",
		Name:      "Yangambi Airport",
		City:      "Yangambi",
		Country:   "CD",
		Elevation: 1378,
		Latitude:  0.7829999924,
		Longitude: 24.466999054,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZIZ": {
		ICAO:      "FZIZ",
		Name:      "Lokutu Airport",
		City:      "Lokutu",
		State:     "Tshopo",
		Country:   "CD",
		Elevation: 1214,
		Latitude:  1.1169999838,
		Longitude: 23.6170005798,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZJB": {
		ICAO:      "FZJB",
		Name:      "Doko Airport",
		City:      "Doko",
		State:     "Haut-Uele",
		Country:   "CD",
		Elevation: 2874,
		Latitude:  3.2330000401,
		Longitude: 29.5669994354,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZJC": {
		ICAO:      "FZJC",
		Name:      "Dungu-Uye Airport",
		City:      "Dungu-Uye",
		State:     "Haut-Uele",
		Country:   "CD",
		Elevation: 2378,
		Latitude:  3.5999999046,
		Longitude: 28.8999996185,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZJD": {
		ICAO:      "FZJD",
		Name:      "Doruma Airport",
		City:      "Doruma",
		State:     "Haut-Uele",
		Country:   "CD",
		Elevation: 2378,
		Latitude:  4.7329998016,
		Longitude: 27.6830005646,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZJF": {
		ICAO:      "FZJF",
		Name:      "Aba Airport",
		City:      "Aba",
		State:     "Haut-Uele",
		Country:   "CD",
		Elevation: 3051,
		Latitude:  3.9000000954,
		Longitude: 30.25,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZJH": {
		ICAO:      "FZJH",
		IATA:      "IRP",
		Name:      "Matari Airport",
		Country:   "CD",
		Elevation: 2438,
		Latitude:  2.8276100159,
		Longitude: 27.588300705,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZJI": {
		ICAO:      "FZJI",
		Name:      "Watsa Airport",
		City:      "Watsa",
		State:     "Haut-Uele",
		Country:   "CD",
		Elevation: 3199,
		Latitude:  3.0169401169,
		Longitude: 29.5592002869,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZJK": {
		ICAO:      "FZJK",
		Name:      "Faradje Airport",
		City:      "Faradje",
		State:     "Haut-Uele",
		Country:   "CD",
		Elevation: 2690,
		Latitude:  3.7330000401,
		Longitude: 29.7000007629,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZJN": {
		ICAO:      "FZJN",
		Name:      "Luniemu Airport",
		City:      "Luniemu",
		State:     "Haut-Lomani",
		Country:   "CD",
		Elevation: 3272,
		Latitude:  -8.1999998093,
		Longitude: 24.7329998016,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZJR": {
		ICAO:      "FZJR",
		Name:      "Kere Kere Airport",
		City:      "Kere Kere",
		State:     "Ituri",
		Country:   "CD",
		Elevation: 4429,
		Latitude:  2.7330000401,
		Longitude: 30.533000946,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZKA": {
		ICAO:      "FZKA",
		IATA:      "BUX",
		Name:      "Bunia Airport",
		Country:   "CD",
		Elevation: 4045,
		Latitude:  1.5657199621,
		Longitude: 30.2208003998,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZKB": {
		ICAO:      "FZKB",
		Name:      "Bambili-Dingila Airport",
		City:      "Bambili-Dingila",
		State:     "Bas-Uele",
		Country:   "CD",
		Elevation: 2050,
		Latitude:  3.6500000954,
		Longitude: 26.1170005798,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZKC": {
		ICAO:      "FZKC",
		Name:      "Mahagi Airport",
		City:      "Mahagi",
		State:     "Ituri",
		Country:   "CD",
		Elevation: 2555,
		Latitude:  2.1670000553,
		Longitude: 31.1499996185,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZKF": {
		ICAO:      "FZKF",
		Name:      "Kilomines Airport",
		City:      "Kilomines",
		State:     "Ituri",
		Country:   "CD",
		Elevation: 4593,
		Latitude:  1.8170000315,
		Longitude: 30.2329998016,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZKI": {
		ICAO:      "FZKI",
		Name:      "Yedi Airport",
		City:      "Yedi",
		State:     "Tshopo",
		Country:   "CD",
		Elevation: 3642,
		Latitude:  2.0329999924,
		Longitude: 24.7999992371,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZKJ": {
		ICAO:      "FZKJ",
		IATA:      "BZU",
		Name:      "Buta Zega Airport",
		Country:   "CD",
		Elevation: 1378,
		Latitude:  2.8183500767,
		Longitude: 24.7936992645,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZKN": {
		ICAO:      "FZKN",
		Name:      "Aketi Airport",
		City:      "Aketi",
		State:     "Bas-Uele",
		Country:   "CD",
		Elevation: 1230,
		Latitude:  2.7000000477,
		Longitude: 23.8330001831,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZKO": {
		ICAO:      "FZKO",
		Name:      "Ango Airport",
		City:      "Ango",
		Country:   "CD",
		Elevation: 2133,
		Latitude:  4.0289998055,
		Longitude: 25.8619995117,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZMA": {
		ICAO:      "FZMA",
		IATA:      "BKY",
		Name:      "Bukavu Kavumu Airport",
		Country:   "CD",
		Elevation: 5643,
		Latitude:  -2.3089799881,
		Longitude: 28.8087997437,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZMB": {
		ICAO:      "FZMB",
		IATA:      "RUE",
		Name:      "Rughenda Airfield",
		City:      "Butembo",
		State:     "Nord-Kivu",
		Country:   "CD",
		Elevation: 5757,
		Latitude:  0.117142,
		Longitude: 29.312992,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZMC": {
		ICAO:      "FZMC",
		Name:      "Mulungu Airport",
		City:      "Mulungu",
		State:     "South-Kivu",
		Country:   "CD",
		Elevation: 2400,
		Latitude:  -2.9830000401,
		Longitude: 27.8500003815,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZMD": {
		ICAO:      "FZMD",
		Name:      "Nzovu Airport",
		City:      "Nzovu",
		State:     "South-Kivu",
		Country:   "CD",
		Elevation: 1970,
		Latitude:  -2.5829999447,
		Longitude: 27.9829998016,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZMK": {
		ICAO:      "FZMK",
		Name:      "Bulongo Kigogo Airport",
		City:      "Bulongo Kigogo",
		State:     "South-Kivu",
		Country:   "CD",
		Elevation: 5249,
		Latitude:  -2.6670000553,
		Longitude: 28.7999992371,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZMP": {
		ICAO:      "FZMP",
		Name:      "Kimano Ii Airport",
		City:      "Kimano Ii",
		State:     "Maniema",
		Country:   "CD",
		Elevation: 2461,
		Latitude:  -4.382999897,
		Longitude: 28.3500003815,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZMW": {
		ICAO:      "FZMW",
		Name:      "Shabunda Airport",
		City:      "Shabunda",
		State:     "South-Kivu",
		Country:   "CD",
		Elevation: 1837,
		Latitude:  -2.6830000877,
		Longitude: 27.3330001831,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZNA": {
		ICAO:      "FZNA",
		IATA:      "GOM",
		Name:      "Goma International Airport",
		City:      "Goma",
		Country:   "CD",
		Elevation: 5089,
		Latitude:  -1.6708099842,
		Longitude: 29.2385005951,
		Timezone:  "Africa/Kigali",
	},
	"FZNB": {
		ICAO:      "FZNB",
		Name:      "Katale Airport",
		City:      "Katale",
		State:     "Nord-Kivu",
		Country:   "CD",
		Elevation: 4589,
		Latitude:  -1.3047200441,
		Longitude: 29.3738994598,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZNC": {
		ICAO:      "FZNC",
		Name:      "Rutshuru Airport",
		City:      "Rutshuru",
		Country:   "CD",
		Elevation: 3707,
		Latitude:  -1.1670000553,
		Longitude: 29.4169998169,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZNF": {
		ICAO:      "FZNF",
		Name:      "Lubero Airport",
		City:      "Lubero",
		State:     "Nord-Kivu",
		Country:   "CD",
		Elevation: 5906,
		Latitude:  -0.1330000013,
		Longitude: 29.25,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZNI": {
		ICAO:      "FZNI",
		Name:      "Ishasha Airport",
		City:      "Ishasha",
		State:     "Nord-Kivu",
		Country:   "CD",
		Elevation: 820,
		Latitude:  -0.75,
		Longitude: 29.6329994202,
		Timezone:  "Africa/Kampala",
	},
	"FZNK": {
		ICAO:      "FZNK",
		Name:      "Katanda Rusthuru Airport",
		City:      "Katanda Rusthuru",
		State:     "Nord-Kivu",
		Country:   "CD",
		Elevation: 2297,
		Latitude:  -0.8000000119,
		Longitude: 29.3670005798,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZNP": {
		ICAO:      "FZNP",
		IATA:      "BNC",
		Name:      "Beni Airport",
		City:      "Beni",
		State:     "Nord-Kivu",
		Country:   "CD",
		Elevation: 3517,
		Latitude:  0.5749999881,
		Longitude: 29.4738998413,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZNQ": {
		ICAO:      "FZNQ",
		Name:      "Obaye Airport",
		City:      "Obaye",
		State:     "Nord-Kivu",
		Country:   "CD",
		Elevation: 2000,
		Latitude:  -1.3329999447,
		Longitude: 27.7329998016,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZNR": {
		ICAO:      "FZNR",
		Name:      "Rwindi Airport",
		City:      "Rwindi",
		State:     "Nord-Kivu",
		Country:   "CD",
		Elevation: 3412,
		Latitude:  -0.7902780175,
		Longitude: 29.2749996185,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZOA": {
		ICAO:      "FZOA",
		IATA:      "KND",
		Name:      "Kindu Airport",
		City:      "Kindu",
		State:     "Maniema",
		Country:   "CD",
		Elevation: 1630,
		Latitude:  -2.9191799164,
		Longitude: 25.9153995514,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZOB": {
		ICAO:      "FZOB",
		Name:      "Tingi-Tingi Airport",
		City:      "Tingi-Tingi",
		State:     "Maniema",
		Country:   "CD",
		Elevation: 862,
		Latitude:  -0.7624999881,
		Longitude: 26.6096992493,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZOC": {
		ICAO:      "FZOC",
		Name:      "Kamisuku Airport",
		City:      "Kalima",
		State:     "Maniema",
		Country:   "CD",
		Latitude:  -2.5499999523,
		Longitude: 26.6166667938,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZOD": {
		ICAO:      "FZOD",
		IATA:      "KLY",
		Name:      "Kinkungwa Airport",
		City:      "Kalima",
		Country:   "CD",
		Elevation: 1808,
		Latitude:  -2.5780000687,
		Longitude: 26.7339992523,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZOE": {
		ICAO:      "FZOE",
		Name:      "Kampene Airport",
		City:      "Kampene",
		State:     "Maniema",
		Country:   "CD",
		Elevation: 2034,
		Latitude:  -3.5829999447,
		Longitude: 26.7000007629,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZOF": {
		ICAO:      "FZOF",
		Name:      "Kiapupe Airport",
		City:      "Kiapupe",
		State:     "South-Kivu",
		Country:   "CD",
		Elevation: 3281,
		Latitude:  -2.9000000954,
		Longitude: 27.2999992371,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZOG": {
		ICAO:      "FZOG",
		Name:      "Lulingu Tshionka Airport",
		City:      "Lulingu Tshionka",
		State:     "South-Kivu",
		Country:   "CD",
		Elevation: 1968,
		Latitude:  -2.3169999123,
		Longitude: 27.5499992371,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZOH": {
		ICAO:      "FZOH",
		Name:      "Moga Airport",
		City:      "Moga",
		State:     "Maniema",
		Country:   "CD",
		Elevation: 2018,
		Latitude:  -2.4670000076,
		Longitude: 26.7999992371,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZOJ": {
		ICAO:      "FZOJ",
		Name:      "Obokote Airport",
		City:      "Obokote",
		State:     "Maniema",
		Country:   "CD",
		Elevation: 1378,
		Latitude:  -0.8500000238,
		Longitude: 26.3330001831,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZOK": {
		ICAO:      "FZOK",
		IATA:      "KGN",
		Name:      "Kasongo Airport",
		City:      "Kasongo",
		State:     "Maniema",
		Country:   "CD",
		Elevation: 1785,
		Latitude:  -4.5329999924,
		Longitude: 26.6170005798,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZOO": {
		ICAO:      "FZOO",
		Name:      "Kailo Airport",
		City:      "Kailo",
		State:     "Maniema",
		Country:   "CD",
		Elevation: 1804,
		Latitude:  -2.632999897,
		Longitude: 26.1000003815,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZOP": {
		ICAO:      "FZOP",
		IATA:      "PUN",
		Name:      "Punia Airport",
		City:      "Punia",
		Country:   "CD",
		Elevation: 1742,
		Latitude:  -1.3669999838,
		Longitude: 26.3330001831,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZOQ": {
		ICAO:      "FZOQ",
		Name:      "Punia-Basenge Airport",
		City:      "Punia-Basenge",
		State:     "Maniema",
		Country:   "CD",
		Elevation: 1738,
		Latitude:  -1.4670000076,
		Longitude: 26.4330005646,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZOR": {
		ICAO:      "FZOR",
		Name:      "Saulia Airport",
		City:      "Saulia",
		State:     "Maniema",
		Country:   "CD",
		Elevation: 1870,
		Latitude:  -1.5329999924,
		Longitude: 26.533000946,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZOS": {
		ICAO:      "FZOS",
		Name:      "Kasese Airport",
		City:      "Kasese",
		State:     "Maniema",
		Country:   "CD",
		Elevation: 1863,
		Latitude:  -1.6330000162,
		Longitude: 27.0499992371,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZOT": {
		ICAO:      "FZOT",
		Name:      "Phibraki Airport",
		City:      "Phibraki",
		State:     "South-Kivu",
		Country:   "CD",
		Elevation: 2100,
		Latitude:  -2.9330000877,
		Longitude: 27.533000946,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZPB": {
		ICAO:      "FZPB",
		Name:      "Kamituga Airport",
		City:      "Kamituga",
		State:     "South-Kivu",
		Country:   "CD",
		Elevation: 3871,
		Latitude:  -3.0329999924,
		Longitude: 28.1170005798,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZPC": {
		ICAO:      "FZPC",
		Name:      "Lugushwa Airport",
		City:      "Lugushwa",
		State:     "South-Kivu",
		Country:   "CD",
		Elevation: 2300,
		Latitude:  -3.3169999123,
		Longitude: 27.8829994202,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZQA": {
		ICAO:      "FZQA",
		IATA:      "FBM",
		Name:      "Lubumbashi International Airport",
		City:      "Lubumbashi",
		Country:   "CD",
		Elevation: 4295,
		Latitude:  -11.5913000107,
		Longitude: 27.5308990479,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZQC": {
		ICAO:      "FZQC",
		IATA:      "PWO",
		Name:      "Pweto Airport",
		City:      "Pweto",
		Country:   "CD",
		Elevation: 3425,
		Latitude:  -8.4670000076,
		Longitude: 28.8829994202,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZQD": {
		ICAO:      "FZQD",
		Name:      "Mulungwishi Airport",
		City:      "Mulungwishi",
		State:     "Haut-Katanga",
		Country:   "CD",
		Elevation: 3500,
		Latitude:  -10.75,
		Longitude: 26.6329994202,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZQF": {
		ICAO:      "FZQF",
		Name:      "Fungurume Airport",
		City:      "Fungurume",
		State:     "Lualaba",
		Country:   "CD",
		Elevation: 3855,
		Latitude:  -10.5333003998,
		Longitude: 26.3250007629,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZQG": {
		ICAO:      "FZQG",
		IATA:      "KEC",
		Name:      "Kasenga Airport",
		City:      "Kasenga",
		State:     "Haut-Katanga",
		Country:   "CD",
		Elevation: 3146,
		Latitude:  -10.3500003815,
		Longitude: 28.6329994202,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZQH": {
		ICAO:      "FZQH",
		Name:      "Katwe Airport",
		City:      "Katwe",
		State:     "Haut-Katanga",
		Country:   "CD",
		Elevation: 5577,
		Latitude:  -10.5500001907,
		Longitude: 27.8500003815,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZQI": {
		ICAO:      "FZQI",
		Name:      "Kamatanda Airport",
		City:      "Kamatanda",
		State:     "Haut-Katanga",
		Country:   "CD",
		Elevation: 4261,
		Latitude:  -10.8330001831,
		Longitude: 26.75,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZQJ": {
		ICAO:      "FZQJ",
		Name:      "Mwadingusha Airport",
		City:      "Mwadingusha",
		State:     "Haut-Katanga",
		Country:   "CD",
		Elevation: 3707,
		Latitude:  -10.75,
		Longitude: 27.2000007629,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZQM": {
		ICAO:      "FZQM",
		IATA:      "KWZ",
		Name:      "Kolwezi Airport",
		State:     "Lualaba",
		Country:   "CD",
		Elevation: 5007,
		Latitude:  -10.7658996582,
		Longitude: 25.5056991577,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZQN": {
		ICAO:      "FZQN",
		Name:      "Mutshatsha Airport",
		City:      "Mutshatsha",
		State:     "Lualaba",
		Country:   "CD",
		Elevation: 3806,
		Latitude:  -10.5670003891,
		Longitude: 24.3999996185,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZQP": {
		ICAO:      "FZQP",
		Name:      "Kisenge Airport",
		City:      "Kisenge",
		State:     "Lualaba",
		Country:   "CD",
		Elevation: 3412,
		Latitude:  -10.6669998169,
		Longitude: 23.1669998169,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZQU": {
		ICAO:      "FZQU",
		Name:      "Lubudi Airport",
		City:      "Lubudi",
		Country:   "CD",
		Elevation: 4541,
		Latitude:  -9.9329996109,
		Longitude: 26,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZQV": {
		ICAO:      "FZQV",
		Name:      "Mitwaba Airport",
		City:      "Mitwaba",
		State:     "Haut-Katanga",
		Country:   "CD",
		Elevation: 5240,
		Latitude:  -8.6450004578,
		Longitude: 27.3449993134,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZRA": {
		ICAO:      "FZRA",
		IATA:      "MNO",
		Name:      "Manono Airport",
		City:      "Manono",
		State:     "Tanganika",
		Country:   "CD",
		Elevation: 2077,
		Latitude:  -7.2888898849,
		Longitude: 27.3943996429,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZRB": {
		ICAO:      "FZRB",
		IATA:      "BDV",
		Name:      "Moba Airport",
		City:      "Moba",
		Country:   "CD",
		Elevation: 2953,
		Latitude:  -7.0669999123,
		Longitude: 29.783000946,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZRC": {
		ICAO:      "FZRC",
		Name:      "Mukoy Airport",
		City:      "Mukoy",
		State:     "Tanganika",
		Country:   "CD",
		Elevation: 5249,
		Latitude:  -7.5329999924,
		Longitude: 28.7000007629,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZRD": {
		ICAO:      "FZRD",
		Name:      "Kabombo Airport",
		City:      "Kabombo",
		State:     "Tanganika",
		Country:   "CD",
		Elevation: 1969,
		Latitude:  -7.3499999046,
		Longitude: 28.033000946,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZRE": {
		ICAO:      "FZRE",
		Name:      "Bukena Airport",
		City:      "Bukena",
		State:     "Haut-Lomani",
		Country:   "CD",
		Elevation: 3868,
		Latitude:  -7.75,
		Longitude: 27.2000007629,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZRF": {
		ICAO:      "FZRF",
		IATA:      "FMI",
		Name:      "Kalemie Airport",
		State:     "Tanganika",
		Country:   "CD",
		Elevation: 2569,
		Latitude:  -5.8755598068,
		Longitude: 29.25,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZRG": {
		ICAO:      "FZRG",
		Name:      "Sominka Airport",
		City:      "Sominka",
		State:     "Tanganika",
		Country:   "CD",
		Elevation: 2066,
		Latitude:  -7.4169998169,
		Longitude: 27.1499996185,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZRJ": {
		ICAO:      "FZRJ",
		Name:      "Pepa Airport",
		City:      "Pepa",
		Country:   "CD",
		Elevation: 6562,
		Latitude:  -7.7170000076,
		Longitude: 29.7999992371,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZRK": {
		ICAO:      "FZRK",
		Name:      "Kansimba Airport",
		City:      "Kansimba",
		State:     "Tanganika",
		Country:   "CD",
		Elevation: 5413,
		Latitude:  -7.3169999123,
		Longitude: 29.1669998169,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZRL": {
		ICAO:      "FZRL",
		Name:      "Lusinga Airport",
		City:      "Lusinga",
		State:     "Haut-Katanga",
		Country:   "CD",
		Elevation: 5840,
		Latitude:  -8.9329996109,
		Longitude: 27.1830005646,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZRM": {
		ICAO:      "FZRM",
		IATA:      "KBO",
		Name:      "Kabalo Airport",
		City:      "Kabalo",
		Country:   "CD",
		Elevation: 1840,
		Latitude:  -6.0830001831,
		Longitude: 26.9169998169,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZRN": {
		ICAO:      "FZRN",
		Name:      "Nyunzu Airport",
		City:      "Nyunzu",
		Country:   "CD",
		Elevation: 2297,
		Latitude:  -5.9330000877,
		Longitude: 28,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZRO": {
		ICAO:      "FZRO",
		Name:      "Luvua Airport",
		City:      "Luvua",
		State:     "Haut-Katanga",
		Country:   "CD",
		Elevation: 4298,
		Latitude:  -7.9330000877,
		Longitude: 28.533000946,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZRQ": {
		ICAO:      "FZRQ",
		IATA:      "KOO",
		Name:      "Kongolo Airport",
		City:      "Kongolo",
		Country:   "CD",
		Elevation: 1850,
		Latitude:  -5.3944401741,
		Longitude: 26.9899997711,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZSA": {
		ICAO:      "FZSA",
		IATA:      "KMN",
		Name:      "Kamina Base Airport",
		Country:   "CD",
		Elevation: 3543,
		Latitude:  -8.6420202255,
		Longitude: 25.2528991699,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZSB": {
		ICAO:      "FZSB",
		IATA:      "KMN",
		Name:      "Ville Airport",
		City:      "Kamina",
		Country:   "CD",
		Elevation: 3475,
		Latitude:  -8.7286100388,
		Longitude: 24.991399765,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZSC": {
		ICAO:      "FZSC",
		Name:      "Songa Airport",
		City:      "Songa",
		State:     "Haut-Lomani",
		Country:   "CD",
		Elevation: 3526,
		Latitude:  -8.1000003815,
		Longitude: 25.033000946,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZSD": {
		ICAO:      "FZSD",
		Name:      "Sandoa Airport",
		City:      "Sandoa",
		State:     "Lualaba",
		Country:   "CD",
		Elevation: 3022,
		Latitude:  -9.6330003738,
		Longitude: 22.8500003815,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZSE": {
		ICAO:      "FZSE",
		Name:      "Kanene Airport",
		City:      "Kanene",
		State:     "Haut-Lomani",
		Country:   "CD",
		Elevation: 3707,
		Latitude:  -9.3000001907,
		Longitude: 24.6669998169,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZSI": {
		ICAO:      "FZSI",
		Name:      "Dilolo Airport",
		City:      "Dilolo",
		State:     "Lualaba",
		Country:   "CD",
		Elevation: 3378,
		Latitude:  -10.7170000076,
		Longitude: 22.3500003815,
		Timezone:  "Africa/Luanda",
	},
	"FZSJ": {
		ICAO:      "FZSJ",
		Name:      "Kasaji Airport",
		City:      "Kasaji",
		State:     "Lualaba",
		Country:   "CD",
		Elevation: 3297,
		Latitude:  -10.3500003815,
		Longitude: 23.4169998169,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZSK": {
		ICAO:      "FZSK",
		IATA:      "KAP",
		Name:      "Kapanga Airport",
		City:      "Kapanga",
		Country:   "CD",
		Elevation: 3025,
		Latitude:  -8.3500003815,
		Longitude: 22.5830001831,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZTK": {
		ICAO:      "FZTK",
		IATA:      "KNM",
		Name:      "Kaniama Airport",
		City:      "Kaniama",
		State:     "Haut-Lomani",
		Country:   "CD",
		Elevation: 2772,
		Latitude:  -7.5830001831,
		Longitude: 24.1499996185,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZTL": {
		ICAO:      "FZTL",
		Name:      "Luena Airport",
		City:      "Luena",
		State:     "Haut-Lomani",
		Country:   "CD",
		Elevation: 2349,
		Latitude:  -9.4670000076,
		Longitude: 25.75,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZTS": {
		ICAO:      "FZTS",
		Name:      "Kaniama Airport",
		City:      "Kaniama",
		State:     "Haut-Lomani",
		Country:   "CD",
		Elevation: 2871,
		Latitude:  -7.6999998093,
		Longitude: 24.0499992371,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZUA": {
		ICAO:      "FZUA",
		IATA:      "KGA",
		Name:      "Kananga Airport",
		City:      "Kananga",
		State:     "Kasai-Central",
		Country:   "CD",
		Elevation: 2139,
		Latitude:  -5.9000501633,
		Longitude: 22.4692001343,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZUE": {
		ICAO:      "FZUE",
		Name:      "Lubondaie Airport",
		City:      "Lubondaie",
		State:     "Kasai-Central",
		Country:   "CD",
		Elevation: 2657,
		Latitude:  -6.5830001831,
		Longitude: 22.5,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZUF": {
		ICAO:      "FZUF",
		Name:      "Kasonga Airport",
		City:      "Kasonga",
		State:     "Kasai-Central",
		Country:   "CD",
		Elevation: 2707,
		Latitude:  -6.6500000954,
		Longitude: 22.3829994202,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZUG": {
		ICAO:      "FZUG",
		IATA:      "LZA",
		Name:      "Luiza Airport",
		City:      "Luiza",
		Country:   "CD",
		Elevation: 2890,
		Latitude:  -7.1830000877,
		Longitude: 22.3999996185,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZUH": {
		ICAO:      "FZUH",
		Name:      "Moma Airport",
		City:      "Moma",
		State:     "Kasai-Central",
		Country:   "CD",
		Elevation: 2789,
		Latitude:  -7.2329998016,
		Longitude: 22.6000003815,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZUI": {
		ICAO:      "FZUI",
		Name:      "Mboi Airport",
		City:      "Mboi",
		State:     "Kasai-Central",
		Country:   "CD",
		Elevation: 2789,
		Latitude:  -6.8330001831,
		Longitude: 22,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZUJ": {
		ICAO:      "FZUJ",
		Name:      "Muambi Airport",
		City:      "Muambi",
		State:     "Kasai-Central",
		Country:   "CD",
		Elevation: 2493,
		Latitude:  -6.6830000877,
		Longitude: 22.533000946,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZUK": {
		ICAO:      "FZUK",
		IATA:      "TSH",
		Name:      "Tshikapa Airport",
		City:      "Tshikapa",
		Country:   "CD",
		Elevation: 1595,
		Latitude:  -6.4383301735,
		Longitude: 20.7947006226,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZUL": {
		ICAO:      "FZUL",
		Name:      "Bulape Airport",
		City:      "Bulape",
		State:     "Kasai",
		Country:   "CD",
		Elevation: 1640,
		Latitude:  -4.617000103,
		Longitude: 21.6000003815,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZUM": {
		ICAO:      "FZUM",
		Name:      "Mutoto Airport",
		City:      "Mutoto",
		State:     "Kasai-Oriental",
		Country:   "CD",
		Elevation: 2297,
		Latitude:  -5.6999998093,
		Longitude: 23.716999054,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZUN": {
		ICAO:      "FZUN",
		Name:      "Luebo Airport",
		City:      "Luebo",
		State:     "Kasai",
		Country:   "CD",
		Elevation: 1640,
		Latitude:  -5.3499999046,
		Longitude: 21.3330001831,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZUO": {
		ICAO:      "FZUO",
		Name:      "Musese Airport",
		City:      "Musese",
		State:     "Kasai",
		Country:   "CD",
		Elevation: 1870,
		Latitude:  -5.5,
		Longitude: 21.4330005646,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZUP": {
		ICAO:      "FZUP",
		Name:      "Diboko Airport",
		City:      "Diboko",
		State:     "Kasai",
		Country:   "CD",
		Elevation: 2431,
		Latitude:  -7.0036101341,
		Longitude: 21.2441997528,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZUR": {
		ICAO:      "FZUR",
		Name:      "Tshibala Airport",
		City:      "Tshibala",
		State:     "Kasai-Central",
		Country:   "CD",
		Elevation: 2287,
		Latitude:  -6.9330000877,
		Longitude: 22,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZUS": {
		ICAO:      "FZUS",
		Name:      "Tshikaji Airport",
		City:      "Tshikaji",
		State:     "Kasai-Central",
		Country:   "CD",
		Elevation: 2287,
		Latitude:  -5.9670000076,
		Longitude: 22.4169998169,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZUT": {
		ICAO:      "FZUT",
		Name:      "Katubwe Airport",
		City:      "Katubwe",
		State:     "Kasai-Central",
		Country:   "CD",
		Elevation: 2461,
		Latitude:  -6.0500001907,
		Longitude: 22.6000003815,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZUU": {
		ICAO:      "FZUU",
		Name:      "Lutshatsha Airport",
		City:      "Lutshatsha",
		State:     "Kasai-Central",
		Country:   "CD",
		Elevation: 2329,
		Latitude:  -6.2170000076,
		Longitude: 22.0830001831,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZUV": {
		ICAO:      "FZUV",
		Name:      "Kalonda Airport",
		City:      "Kalonda",
		State:     "Kasai",
		Country:   "CD",
		Elevation: 1873,
		Latitude:  -6.4670000076,
		Longitude: 20.7999992371,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZVA": {
		ICAO:      "FZVA",
		IATA:      "LJA",
		Name:      "Lodja Airport",
		City:      "Lodja",
		State:     "Kasai-Oriental",
		Country:   "CD",
		Elevation: 1647,
		Latitude:  -3.4170000553,
		Longitude: 23.4500007629,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZVC": {
		ICAO:      "FZVC",
		Name:      "Kole Sur Lukenie Airport",
		City:      "Kole Sur Lukenie",
		State:     "Sankuru",
		Country:   "CD",
		Elevation: 1542,
		Latitude:  -3.4000000954,
		Longitude: 22.533000946,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZVD": {
		ICAO:      "FZVD",
		Name:      "Dingele Airport",
		City:      "Dingele",
		State:     "Sankuru",
		Country:   "CD",
		Elevation: 1985,
		Latitude:  -3.5999999046,
		Longitude: 24.5830001831,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZVE": {
		ICAO:      "FZVE",
		Name:      "Lomela Airport",
		City:      "Lomela",
		State:     "Sankuru",
		Country:   "CD",
		Elevation: 1434,
		Latitude:  -2.2999999523,
		Longitude: 23.2670001984,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZVF": {
		ICAO:      "FZVF",
		Name:      "Kutusongo Airport",
		City:      "Kutusongo",
		State:     "Sankuru",
		Country:   "CD",
		Elevation: 1722,
		Latitude:  -2.6670000553,
		Longitude: 23.1669998169,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZVG": {
		ICAO:      "FZVG",
		Name:      "Katako'kombe Airport",
		City:      "Katako'kombe",
		State:     "Sankuru",
		Country:   "CD",
		Elevation: 1978,
		Latitude:  -3.4670000076,
		Longitude: 24.4330005646,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZVH": {
		ICAO:      "FZVH",
		Name:      "Shongamba Airport",
		City:      "Shongamba",
		State:     "Kasai",
		Country:   "CD",
		Elevation: 2133,
		Latitude:  -4.3499999046,
		Longitude: 21.283000946,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZVI": {
		ICAO:      "FZVI",
		IATA:      "LBO",
		Name:      "Lusambo Airport",
		City:      "Lusambo",
		State:     "Sankuru",
		Country:   "CD",
		Elevation: 1407,
		Latitude:  -4.9616699219,
		Longitude: 23.3782997131,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZVJ": {
		ICAO:      "FZVJ",
		Name:      "Tshumbe Airport",
		City:      "Tshumbe",
		State:     "Sankuru",
		Country:   "CD",
		Elevation: 1804,
		Latitude:  -4.0999999046,
		Longitude: 24.3670005798,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZVK": {
		ICAO:      "FZVK",
		Name:      "Lukombe-Batwa Airport",
		City:      "Lukombe-Batwa",
		State:     "Kasai",
		Country:   "CD",
		Elevation: 1640,
		Latitude:  -4.3330001831,
		Longitude: 22.0830001831,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZVL": {
		ICAO:      "FZVL",
		Name:      "Wasolo Airport",
		City:      "Wasolo",
		State:     "Sankuru",
		Country:   "CD",
		Elevation: 1673,
		Latitude:  -3.9500000477,
		Longitude: 22.5170001984,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZVM": {
		ICAO:      "FZVM",
		IATA:      "MEW",
		Name:      "Mweka Airport",
		City:      "Mweka",
		Country:   "CD",
		Elevation: 1968,
		Latitude:  -4.8499999046,
		Longitude: 21.5499992371,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZVN": {
		ICAO:      "FZVN",
		Name:      "Wembo-Nyama Airport",
		City:      "Wembo-Nyama",
		State:     "Sankuru",
		Country:   "CD",
		Elevation: 1801,
		Latitude:  -4.4829998016,
		Longitude: 24.4500007629,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZVO": {
		ICAO:      "FZVO",
		Name:      "Beni-Dibele Airport",
		City:      "Beni-Dibele",
		State:     "Kasai-Oriental",
		Country:   "CD",
		Elevation: 1738,
		Latitude:  -4.0830001831,
		Longitude: 22.8330001831,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZVP": {
		ICAO:      "FZVP",
		Name:      "Dikungu Airport",
		City:      "Dikungu",
		State:     "Sankuru",
		Country:   "CD",
		Elevation: 1833,
		Latitude:  -4.0329999924,
		Longitude: 24.466999054,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZVR": {
		ICAO:      "FZVR",
		IATA:      "BAN",
		Name:      "Basongo Airport",
		City:      "Basongo",
		Country:   "CD",
		Elevation: 1640,
		Latitude:  -4.3169999123,
		Longitude: 20.4330005646,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZVS": {
		ICAO:      "FZVS",
		IATA:      "PFR",
		Name:      "Ilebo Airport",
		City:      "Ilebo",
		State:     "Kasai",
		Country:   "CD",
		Elevation: 1450,
		Latitude:  -4.3330001831,
		Longitude: 20.5830001831,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZVT": {
		ICAO:      "FZVT",
		Name:      "Dekese Airport",
		City:      "Dekese",
		Country:   "CD",
		Elevation: 1279,
		Latitude:  -3.4670000076,
		Longitude: 21.4330005646,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZVU": {
		ICAO:      "FZVU",
		Name:      "Idumbe Airport",
		City:      "Idumbe",
		State:     "Kasai",
		Country:   "CD",
		Elevation: 1847,
		Latitude:  -3.9170000553,
		Longitude: 21.5830001831,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZWA": {
		ICAO:      "FZWA",
		IATA:      "MJM",
		Name:      "Mbuji Mayi Airport",
		City:      "Mbuji Mayi",
		State:     "Kasai-Oriental",
		Country:   "CD",
		Elevation: 2221,
		Latitude:  -6.121240139,
		Longitude: 23.5690002441,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZWB": {
		ICAO:      "FZWB",
		Name:      "Bibanga Airport",
		City:      "Bibanga",
		State:     "Kasai-Oriental",
		Country:   "CD",
		Elevation: 2953,
		Latitude:  -6.25,
		Longitude: 23.9829998016,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZWC": {
		ICAO:      "FZWC",
		IATA:      "GDJ",
		Name:      "Gandajika Airport",
		City:      "Gandajika",
		State:     "Lomami",
		Country:   "CD",
		Elevation: 2618,
		Latitude:  -6.7329998016,
		Longitude: 23.9500007629,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZWE": {
		ICAO:      "FZWE",
		Name:      "Mwene-Ditu Airport",
		City:      "Mwene-Ditu",
		State:     "Kasai-Central",
		Country:   "CD",
		Elevation: 3198,
		Latitude:  -6.9829998016,
		Longitude: 23.0830001831,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZWF": {
		ICAO:      "FZWF",
		Name:      "Kipushi Airport",
		City:      "Kipushi",
		State:     "Lomami",
		Country:   "CD",
		Elevation: 2953,
		Latitude:  -6.1669998169,
		Longitude: 25.1830005646,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZWI": {
		ICAO:      "FZWI",
		Name:      "Kashia Airport",
		City:      "Kashia",
		State:     "Lomami",
		Country:   "CD",
		Elevation: 2887,
		Latitude:  -7.2170000076,
		Longitude: 23.75,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZWL": {
		ICAO:      "FZWL",
		Name:      "Munkamba Airport",
		City:      "Munkamba",
		Country:   "CD",
		Elevation: 2230,
		Latitude:  -5.7670001984,
		Longitude: 23.0499992371,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZWR": {
		ICAO:      "FZWR",
		Name:      "Kisengwa Airport",
		City:      "Kisengwa",
		State:     "Lomami",
		Country:   "CD",
		Elevation: 2428,
		Latitude:  -6.0170001984,
		Longitude: 25.8829994202,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZWS": {
		ICAO:      "FZWS",
		Name:      "Lubao Airport",
		City:      "Lubao",
		State:     "Lomami",
		Country:   "CD",
		Elevation: 2625,
		Latitude:  -5.3000001907,
		Longitude: 25.7329998016,
		Timezone:  "Africa/Lubumbashi",
	},
	"FZWT": {
		ICAO:      "FZWT",
		IATA:      "KBN",
		Name:      "Tunta Airport",
		City:      "Kabinda",
		State:     "Kasai-Oriental",
		Country:   "CD",
		Elevation: 2766,
		Latitude:  -6.132999897,
		Longitude: 24.4829998016,
		Timezone:  "Africa/Lubumbashi",
	},
	"GA00": {
		ICAO:      "GA00",
		Name:      "Kintail Farm Airport",
		City:      "Monroe",
		State:     "Georgia",
		Country:   "US",
		Elevation: 810,
		Latitude:  33.8255996704,
		Longitude: -83.6397018433,
		Timezone:  "America/New_York",
	},
	"GA01": {
		ICAO:      "GA01",
		Name:      "Flying H Ranch Inc Airport",
		City:      "Jackson",
		State:     "Georgia",
		Country:   "US",
		Elevation: 722,
		Latitude:  33.1977996826,
		Longitude: -84.0843963623,
		Timezone:  "America/New_York",
	},
	"GA02": {
		ICAO:      "GA02",
		Name:      "Howard Private Airport",
		City:      "Jackson",
		State:     "Georgia",
		Country:   "US",
		Elevation: 720,
		Latitude:  33.3515014648,
		Longitude: -83.9637985229,
		Timezone:  "America/New_York",
	},
	"GA03": {
		ICAO:      "GA03",
		Name:      "Wilson Airport",
		City:      "Ringgold",
		State:     "Georgia",
		Country:   "US",
		Elevation: 795,
		Latitude:  34.8694992065,
		Longitude: -85.1996994019,
		Timezone:  "America/New_York",
	},
	"GA04": {
		ICAO:      "GA04",
		Name:      "Mallards Landing Airport",
		City:      "Locust Grove",
		State:     "Georgia",
		Country:   "US",
		Elevation: 837,
		Latitude:  33.3656997681,
		Longitude: -84.1651992798,
		Timezone:  "America/New_York",
	},
	"GA06": {
		ICAO:      "GA06",
		Name:      "Monticello Sky Ranch Airport",
		City:      "Monticello",
		State:     "Georgia",
		Country:   "US",
		Elevation: 690,
		Latitude:  33.3325004578,
		Longitude: -83.7268981934,
		Timezone:  "America/New_York",
	},
	"GA07": {
		ICAO:      "GA07",
		Name:      "Crawford Hendrix Farm Airport",
		City:      "Statesboro",
		State:     "Georgia",
		Country:   "US",
		Elevation: 100,
		Latitude:  32.2372016907,
		Longitude: -81.6449966431,
		Timezone:  "America/New_York",
	},
	"GA08": {
		ICAO:      "GA08",
		Name:      "Jumpin J Airport",
		City:      "Talbotton",
		State:     "Georgia",
		Country:   "US",
		Elevation: 659,
		Latitude:  32.6917991638,
		Longitude: -84.4971008301,
		Timezone:  "America/New_York",
	},
	"GA09": {
		ICAO:      "GA09",
		Name:      "Fly-N-S Ranch Airport",
		City:      "Clermont",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1330,
		Latitude:  34.4516983032,
		Longitude: -83.7885971069,
		Timezone:  "America/New_York",
	},
	"GA10": {
		ICAO:      "GA10",
		Name:      "Ridgeview Farm Airport",
		City:      "Zebulon",
		State:     "Georgia",
		Country:   "US",
		Elevation: 854,
		Latitude:  33.0834999084,
		Longitude: -84.3582992554,
		Timezone:  "America/New_York",
	},
	"GA12": {
		ICAO:      "GA12",
		Name:      "Tallassee Plantation Airport",
		City:      "Albany",
		State:     "Georgia",
		Country:   "US",
		Elevation: 240,
		Latitude:  31.5834999084,
		Longitude: -84.3905029297,
		Timezone:  "America/New_York",
	},
	"GA13": {
		ICAO:      "GA13",
		Name:      "Double 'O' Farm Airport",
		City:      "Albany",
		State:     "Georgia",
		Country:   "US",
		Elevation: 297,
		Latitude:  31.5349006653,
		Longitude: -84.0020980835,
		Timezone:  "America/New_York",
	},
	"GA14": {
		ICAO:      "GA14",
		Name:      "Pinebloom Plantation Airport",
		City:      "Albany",
		State:     "Georgia",
		Country:   "US",
		Elevation: 170,
		Latitude:  31.4029998779,
		Longitude: -84.3227005005,
		Timezone:  "America/New_York",
	},
	"GA15": {
		ICAO:      "GA15",
		Name:      "Klockner Airport",
		City:      "Bostwick",
		State:     "Georgia",
		Country:   "US",
		Elevation: 740,
		Latitude:  33.7086982727,
		Longitude: -83.5791015625,
		Timezone:  "America/New_York",
	},
	"GA16": {
		ICAO:      "GA16",
		Name:      "Taylor Field",
		City:      "Bostwick",
		State:     "Georgia",
		Country:   "US",
		Elevation: 710,
		Latitude:  33.7243003845,
		Longitude: -83.5668029785,
		Timezone:  "America/New_York",
	},
	"GA17": {
		ICAO:      "GA17",
		Name:      "Flint River Nursery Airport",
		City:      "Byromville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 320,
		Latitude:  32.168800354,
		Longitude: -83.9735031128,
		Timezone:  "America/New_York",
	},
	"GA18": {
		ICAO:      "GA18",
		Name:      "Big Creek Flying Ranch Airport",
		City:      "Clayton",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1650,
		Latitude:  34.832901001,
		Longitude: -83.4095993042,
		Timezone:  "America/New_York",
	},
	"GA19": {
		ICAO:      "GA19",
		Name:      "Hearn Airport",
		City:      "Claxton",
		State:     "Georgia",
		Country:   "US",
		Elevation: 185,
		Latitude:  32.1571006775,
		Longitude: -81.8887023926,
		Timezone:  "America/New_York",
	},
	"GA1A": {
		ICAO:      "GA1A",
		Name:      "Kayes Airport",
		State:     "Kayes",
		Country:   "ML",
		Elevation: 161,
		Latitude:  14.4313001633,
		Longitude: -11.4397001266,
		Timezone:  "Africa/Bamako",
	},
	"GA20": {
		ICAO:      "GA20",
		Name:      "Stafford Airport",
		City:      "Cumberland Island",
		State:     "Georgia",
		Country:   "US",
		Elevation: 28,
		Latitude:  30.8111000061,
		Longitude: -81.4627990723,
		Timezone:  "America/New_York",
	},
	"GA21": {
		ICAO:      "GA21",
		Name:      "Patterson Island Airport",
		City:      "Darien",
		State:     "Georgia",
		Country:   "US",
		Elevation: 10,
		Latitude:  31.4612998962,
		Longitude: -81.3386993408,
		Timezone:  "America/New_York",
	},
	"GA22": {
		ICAO:      "GA22",
		Name:      "Jordans Airport",
		City:      "Arlington",
		State:     "Georgia",
		Country:   "US",
		Elevation: 250,
		Latitude:  31.4384994507,
		Longitude: -84.7082977295,
		Timezone:  "America/New_York",
	},
	"GA23": {
		ICAO:      "GA23",
		Name:      "Wyatt Airport",
		City:      "Summerville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 720,
		Latitude:  34.5783004761,
		Longitude: -85.3839035034,
		Timezone:  "America/New_York",
	},
	"GA25": {
		ICAO:      "GA25",
		Name:      "Fargo Airport",
		City:      "Fargo",
		State:     "Georgia",
		Country:   "US",
		Elevation: 118,
		Latitude:  30.6912994385,
		Longitude: -82.5675964355,
		Timezone:  "America/New_York",
	},
	"GA27": {
		ICAO:      "GA27",
		Name:      "Mathis Airport",
		City:      "Cumming",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1170,
		Latitude:  34.1012001038,
		Longitude: -84.1610031128,
		Timezone:  "America/New_York",
	},
	"GA29": {
		ICAO:      "GA29",
		Name:      "B & L Strip",
		City:      "Hollonville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 843,
		Latitude:  33.1562004089,
		Longitude: -84.4309997559,
		Timezone:  "America/New_York",
	},
	"GA30": {
		ICAO:      "GA30",
		Name:      "Eliott Barrow Airport",
		City:      "Matthews",
		State:     "Georgia",
		Country:   "US",
		Elevation: 400,
		Latitude:  33.2000999451,
		Longitude: -82.2665023804,
		Timezone:  "America/New_York",
	},
	"GA31": {
		ICAO:      "GA31",
		Name:      "Two Rocks Airport",
		City:      "Palmetto",
		State:     "Georgia",
		Country:   "US",
		Elevation: 964,
		Latitude:  33.4735984802,
		Longitude: -84.6735992432,
		Timezone:  "America/New_York",
	},
	"GA34": {
		ICAO:      "GA34",
		Name:      "Tootle Airport",
		City:      "Reidsville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 202,
		Latitude:  32.0321006775,
		Longitude: -82.0414962769,
		Timezone:  "America/New_York",
	},
	"GA35": {
		ICAO:      "GA35",
		Name:      "Cypress Lakes Airport",
		City:      "Savannah",
		State:     "Georgia",
		Country:   "US",
		Elevation: 30,
		Latitude:  32.1575012207,
		Longitude: -81.396697998,
		Timezone:  "America/New_York",
	},
	"GA36": {
		ICAO:      "GA36",
		Name:      "Foothills-Holcomb Airport",
		City:      "Marble Hill",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1425,
		Latitude:  34.3917007446,
		Longitude: -84.2416992188,
		Timezone:  "America/New_York",
	},
	"GA39": {
		ICAO:      "GA39",
		Name:      "Hodges Airpark",
		City:      "Savannah",
		State:     "Georgia",
		Country:   "US",
		Elevation: 10,
		Latitude:  31.9843997955,
		Longitude: -81.2462005615,
		Timezone:  "America/New_York",
	},
	"GA41": {
		ICAO:      "GA41",
		Name:      "Daniels Airport",
		City:      "Swainsboro",
		State:     "Georgia",
		Country:   "US",
		Elevation: 288,
		Latitude:  32.6141014099,
		Longitude: -82.2970962524,
		Timezone:  "America/New_York",
	},
	"GA42": {
		ICAO:      "GA42",
		Name:      "Crowe Airport",
		City:      "Sylvester",
		State:     "Georgia",
		Country:   "US",
		Elevation: 330,
		Latitude:  31.6168003082,
		Longitude: -83.8832015991,
		Timezone:  "America/New_York",
	},
	"GA43": {
		ICAO:      "GA43",
		Name:      "Briggs Field",
		City:      "Guyton",
		State:     "Georgia",
		Country:   "US",
		Elevation: 78,
		Latitude:  32.3213996887,
		Longitude: -81.4266967773,
		Timezone:  "America/New_York",
	},
	"GA44": {
		ICAO:      "GA44",
		Name:      "Richards Airport",
		City:      "Lagrange",
		State:     "Georgia",
		Country:   "US",
		Elevation: 650,
		Latitude:  33.0792999268,
		Longitude: -85.0316009521,
		Timezone:  "America/New_York",
	},
	"GA45": {
		ICAO:      "GA45",
		Name:      "Townsend Air Strip",
		City:      "Townsend",
		State:     "Georgia",
		Country:   "US",
		Elevation: 26,
		Latitude:  31.5433006287,
		Longitude: -81.5347976685,
		Timezone:  "America/New_York",
	},
	"GA47": {
		ICAO:      "GA47",
		Name:      "Bivins Airport",
		City:      "Waycross",
		State:     "Georgia",
		Country:   "US",
		Elevation: 134,
		Latitude:  31.1851997375,
		Longitude: -82.2734985352,
		Timezone:  "America/New_York",
	},
	"GA48": {
		ICAO:      "GA48",
		Name:      "Mclendon Airport",
		City:      "Edison",
		State:     "Georgia",
		Country:   "US",
		Elevation: 360,
		Latitude:  31.586111,
		Longitude: -84.790556,
		Timezone:  "America/New_York",
	},
	"GA49": {
		ICAO:      "GA49",
		Name:      "Thistle Field",
		City:      "Yatesville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 730,
		Latitude:  32.927898407,
		Longitude: -84.1412963867,
		Timezone:  "America/New_York",
	},
	"GA50": {
		ICAO:      "GA50",
		Name:      "Dream Team Airport",
		City:      "Madison",
		State:     "Georgia",
		Country:   "US",
		Elevation: 604,
		Latitude:  33.5974998474,
		Longitude: -83.5177993774,
		Timezone:  "America/New_York",
	},
	"GA53": {
		ICAO:      "GA53",
		Name:      "Rollins STOLport",
		City:      "Atlanta",
		State:     "Georgia",
		Country:   "US",
		Elevation: 810,
		Latitude:  33.8151016235,
		Longitude: -84.3719024658,
		Timezone:  "America/New_York",
	},
	"GA59": {
		ICAO:      "GA59",
		Name:      "Antique Acres Airport",
		City:      "Barnesville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 800,
		Latitude:  33.1001014709,
		Longitude: -84.0585021973,
		Timezone:  "America/New_York",
	},
	"GA61": {
		ICAO:      "GA61",
		Name:      "Kenley Field",
		City:      "Brooks",
		State:     "Georgia",
		Country:   "US",
		Elevation: 780,
		Latitude:  33.2690010071,
		Longitude: -84.4971008301,
		Timezone:  "America/New_York",
	},
	"GA62": {
		ICAO:      "GA62",
		Name:      "Cedar Ridge Airport",
		City:      "Griffin",
		State:     "Georgia",
		Country:   "US",
		Elevation: 820,
		Latitude:  33.2551002502,
		Longitude: -84.4001998901,
		Timezone:  "America/New_York",
	},
	"GA63": {
		ICAO:      "GA63",
		Name:      "Byromville Aerodrome",
		City:      "Byromville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 354,
		Latitude:  32.1792984009,
		Longitude: -83.8999023438,
		Timezone:  "America/New_York",
	},
	"GA65": {
		ICAO:      "GA65",
		Name:      "Mercer Airfield",
		City:      "Calhoun",
		State:     "Georgia",
		Country:   "US",
		Elevation: 660,
		Latitude:  34.4370002747,
		Longitude: -84.9197006226,
		Timezone:  "America/New_York",
	},
	"GA67": {
		ICAO:      "GA67",
		Name:      "King Sky Ranch Airport",
		City:      "Clarkesville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1840,
		Latitude:  34.7681999207,
		Longitude: -83.5929031372,
		Timezone:  "America/New_York",
	},
	"GA68": {
		ICAO:      "GA68",
		Name:      "Flying 'H' Ranch Airport",
		City:      "Concord",
		State:     "Georgia",
		Country:   "US",
		Elevation: 800,
		Latitude:  33.1147994995,
		Longitude: -84.4490966797,
		Timezone:  "America/New_York",
	},
	"GA72": {
		ICAO:      "GA72",
		Name:      "Pratermill Flight Park Airport",
		City:      "Dalton",
		State:     "Georgia",
		Country:   "US",
		Elevation: 780,
		Latitude:  34.8816986084,
		Longitude: -84.8897018433,
		Timezone:  "America/New_York",
	},
	"GA73": {
		ICAO:      "GA73",
		Name:      "Shade Tree Airport",
		City:      "Concord",
		State:     "Georgia",
		Country:   "US",
		Elevation: 850,
		Latitude:  33.1618003845,
		Longitude: -84.4533004761,
		Timezone:  "America/New_York",
	},
	"GA74": {
		ICAO:      "GA74",
		Name:      "Takle Field",
		City:      "Concord",
		State:     "Georgia",
		Country:   "US",
		Elevation: 850,
		Latitude:  33.0418014526,
		Longitude: -84.412399292,
		Timezone:  "America/New_York",
	},
	"GA75": {
		ICAO:      "GA75",
		Name:      "Meadowlark Airport",
		City:      "Concord",
		State:     "Georgia",
		Country:   "US",
		Elevation: 784,
		Latitude:  33.0872993469,
		Longitude: -84.4051971436,
		Timezone:  "America/New_York",
	},
	"GA76": {
		ICAO:      "GA76",
		Name:      "Broken Ranch Airport",
		City:      "Forsyth",
		State:     "Georgia",
		Country:   "US",
		Elevation: 620,
		Latitude:  33.1548995972,
		Longitude: -83.8988037109,
		Timezone:  "America/New_York",
	},
	"GA77": {
		ICAO:      "GA77",
		Name:      "Wallace Field",
		City:      "Rome",
		State:     "Georgia",
		Country:   "US",
		Elevation: 810,
		Latitude:  34.135799408,
		Longitude: -85.112197876,
		Timezone:  "America/New_York",
	},
	"GA79": {
		ICAO:      "GA79",
		Name:      "Dresden Airport",
		City:      "Newnan",
		State:     "Georgia",
		Country:   "US",
		Elevation: 800,
		Latitude:  33.3447990417,
		Longitude: -84.9113006592,
		Timezone:  "America/New_York",
	},
	"GA80": {
		ICAO:      "GA80",
		Name:      "Whispering Pines Airport",
		City:      "Conyers",
		State:     "Georgia",
		Country:   "US",
		Elevation: 940,
		Latitude:  33.7822990417,
		Longitude: -83.9888000488,
		Timezone:  "America/New_York",
	},
	"GA81": {
		ICAO:      "GA81",
		Name:      "Cameron Field",
		City:      "Ft Valley",
		State:     "Georgia",
		Country:   "US",
		Elevation: 380,
		Latitude:  32.5477981567,
		Longitude: -83.8268966675,
		Timezone:  "America/New_York",
	},
	"GA82": {
		ICAO:      "GA82",
		Name:      "Morgan Farm Field",
		City:      "Covington",
		State:     "Georgia",
		Country:   "US",
		Elevation: 700,
		Latitude:  33.5959014893,
		Longitude: -83.8041000366,
		Timezone:  "America/New_York",
	},
	"GA83": {
		ICAO:      "GA83",
		Name:      "Windy Hill Airport",
		City:      "Covington",
		State:     "Georgia",
		Country:   "US",
		Elevation: 740,
		Latitude:  33.5467987061,
		Longitude: -83.8054962158,
		Timezone:  "America/New_York",
	},
	"GA86": {
		ICAO:      "GA86",
		Name:      "Little Tobesofkee Creek Ranch Airport",
		City:      "Barnesville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 755,
		Latitude:  32.9838981628,
		Longitude: -84.0955963135,
		Timezone:  "America/New_York",
	},
	"GA87": {
		ICAO:      "GA87",
		Name:      "High Valley Airpark",
		City:      "Suches",
		State:     "Georgia",
		Country:   "US",
		Elevation: 2800,
		Latitude:  34.6981010437,
		Longitude: -84.0127029419,
		Timezone:  "America/New_York",
	},
	"GA88": {
		ICAO:      "GA88",
		Name:      "Wade Plantation Airport",
		City:      "Sylvania",
		State:     "Georgia",
		Country:   "US",
		Elevation: 146,
		Latitude:  32.9771003723,
		Longitude: -81.5362014771,
		Timezone:  "America/New_York",
	},
	"GA89": {
		ICAO:      "GA89",
		Name:      "Diamond S Airport",
		City:      "Lovejoy",
		State:     "Georgia",
		Country:   "US",
		Elevation: 910,
		Latitude:  33.4757995605,
		Longitude: -84.2769012451,
		Timezone:  "America/New_York",
	},
	"GA90": {
		ICAO:      "GA90",
		Name:      "Walker Field",
		City:      "Fayetteville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 875,
		Latitude:  33.4520988464,
		Longitude: -84.4094009399,
		Timezone:  "America/New_York",
	},
	"GA91": {
		ICAO:      "GA91",
		Name:      "Adams Airport",
		City:      "Fayetteville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 885,
		Latitude:  33.3970985413,
		Longitude: -84.4601974487,
		Timezone:  "America/New_York",
	},
	"GA92": {
		ICAO:      "GA92",
		Name:      "Beck Field",
		City:      "Fayetteville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 990,
		Latitude:  33.5194015503,
		Longitude: -84.5143966675,
		Timezone:  "America/New_York",
	},
	"GA93": {
		ICAO:      "GA93",
		Name:      "Bishops Airport",
		City:      "Fayetteville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 870,
		Latitude:  33.4453010559,
		Longitude: -84.4067001343,
		Timezone:  "America/New_York",
	},
	"GA94": {
		ICAO:      "GA94",
		Name:      "Mc Lendon Airport",
		City:      "Fayetteville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 900,
		Latitude:  33.4915008545,
		Longitude: -84.489402771,
		Timezone:  "America/New_York",
	},
	"GA95": {
		ICAO:      "GA95",
		Name:      "Coleman Field",
		City:      "Fayetteville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1010,
		Latitude:  33.5261993408,
		Longitude: -84.5021972656,
		Timezone:  "America/New_York",
	},
	"GA98": {
		ICAO:      "GA98",
		Name:      "Grayhill Airport",
		City:      "LaGrange",
		State:     "Georgia",
		Country:   "US",
		Elevation: 750,
		Latitude:  32.9489021301,
		Longitude: -85.05052948,
		Timezone:  "America/New_York",
	},
	"GA99": {
		ICAO:      "GA99",
		Name:      "Miami Valley Farm Airport",
		City:      "Fort Valley",
		State:     "Georgia",
		Country:   "US",
		Elevation: 468,
		Latitude:  32.5488014221,
		Longitude: -83.8018035889,
		Timezone:  "America/New_York",
	},
	"GAAO": {
		ICAO:      "GAAO",
		Name:      "Ansongo Airport",
		City:      "Ansongo",
		State:     "Gao",
		Country:   "ML",
		Elevation: 853,
		Latitude:  15.6999998093,
		Longitude: 0.5,
		Timezone:  "Africa/Bamako",
	},
	"GABD": {
		ICAO:      "GABD",
		Name:      "Bandiagara Airport",
		City:      "Bandiagara",
		State:     "Mopti",
		Country:   "ML",
		Elevation: 1312,
		Latitude:  14.3330001831,
		Longitude: -3.5999999046,
		Timezone:  "Africa/Bamako",
	},
	"GABF": {
		ICAO:      "GABF",
		Name:      "Bafoulabe Airport",
		City:      "Bafoulabe",
		State:     "Kayes",
		Country:   "ML",
		Elevation: 380,
		Latitude:  13.8000001907,
		Longitude: -10.8500003815,
		Timezone:  "Africa/Bamako",
	},
	"GABG": {
		ICAO:      "GABG",
		Name:      "Bougouni Airport",
		City:      "Bougouni",
		State:     "Sikasso",
		Country:   "ML",
		Elevation: 1139,
		Latitude:  11.4499998093,
		Longitude: -7.5170001984,
		Timezone:  "Africa/Bamako",
	},
	"GABR": {
		ICAO:      "GABR",
		Name:      "Bourem Airport",
		City:      "Bourem",
		State:     "Gao",
		Country:   "ML",
		Elevation: 941,
		Latitude:  17.033000946,
		Longitude: -0.400000006,
		Timezone:  "Africa/Bamako",
	},
	"GABS": {
		ICAO:      "GABS",
		IATA:      "BKO",
		Name:      "Senou Airport",
		City:      "Senou",
		State:     "Bamako",
		Country:   "ML",
		Elevation: 1247,
		Latitude:  12.5334997177,
		Longitude: -7.9499402046,
		Timezone:  "Africa/Bamako",
	},
	"GADA": {
		ICAO:      "GADA",
		Name:      "Dioila Airport",
		City:      "Dioila",
		State:     "Koulikoro",
		Country:   "ML",
		Elevation: 1050,
		Latitude:  12.5,
		Longitude: -6.8000001907,
		Timezone:  "Africa/Bamako",
	},
	"GADZ": {
		ICAO:      "GADZ",
		Name:      "Douentza Airport",
		City:      "Douentza",
		State:     "Mopti",
		Country:   "ML",
		Elevation: 984,
		Latitude:  15,
		Longitude: -2.9170000553,
		Timezone:  "Africa/Bamako",
	},
	"GAFD": {
		ICAO:      "GAFD",
		Name:      "Faladie Airport",
		City:      "Faladie",
		State:     "Koulikoro",
		Country:   "ML",
		Elevation: 1115,
		Latitude:  13.1499996185,
		Longitude: -8.3330001831,
		Timezone:  "Africa/Bamako",
	},
	"GAGM": {
		ICAO:      "GAGM",
		IATA:      "GUD",
		Name:      "Goundam Airport",
		City:      "Goundam",
		State:     "Gao",
		Country:   "ML",
		Elevation: 866,
		Latitude:  16.3614006042,
		Longitude: -3.5997200012,
		Timezone:  "Africa/Bamako",
	},
	"GAGO": {
		ICAO:      "GAGO",
		IATA:      "GAQ",
		Name:      "Gao Airport",
		State:     "Gao",
		Country:   "ML",
		Elevation: 870,
		Latitude:  16.2483997345,
		Longitude: -0.0054560001,
		Timezone:  "Africa/Bamako",
	},
	"GAKA": {
		ICAO:      "GAKA",
		IATA:      "KNZ",
		Name:      "Kenieba Airport",
		City:      "Kenieba",
		State:     "Kayes",
		Country:   "ML",
		Elevation: 449,
		Latitude:  12.8330001831,
		Longitude: -11.25,
		Timezone:  "Africa/Bamako",
	},
	"GAKL": {
		ICAO:      "GAKL",
		Name:      "Kidal Airport",
		City:      "Kidal",
		State:     "Gao",
		Country:   "ML",
		Elevation: 1496,
		Latitude:  18.4330005646,
		Longitude: 1.4170000553,
		Timezone:  "Africa/Bamako",
	},
	"GAKM": {
		ICAO:      "GAKM",
		Name:      "Ke-Macina Airport",
		City:      "Ke-Macina",
		State:     "Segou",
		Country:   "ML",
		Elevation: 902,
		Latitude:  13.9670000076,
		Longitude: -5.382999897,
		Timezone:  "Africa/Bamako",
	},
	"GAKN": {
		ICAO:      "GAKN",
		Name:      "Kolokani Airport",
		City:      "Kolokani",
		State:     "Bamako",
		Country:   "ML",
		Elevation: 1457,
		Latitude:  13.5329999924,
		Longitude: -8.0500001907,
		Timezone:  "Africa/Bamako",
	},
	"GAKO": {
		ICAO:      "GAKO",
		IATA:      "KTX",
		Name:      "Koutiala Airport",
		City:      "Koutiala",
		State:     "Sikasso",
		Country:   "ML",
		Elevation: 1240,
		Latitude:  12.3830003738,
		Longitude: -5.4670000076,
		Timezone:  "Africa/Bamako",
	},
	"GAKT": {
		ICAO:      "GAKT",
		Name:      "Kita Airport",
		City:      "Kita",
		State:     "Kayes",
		Country:   "ML",
		Elevation: 1122,
		Latitude:  13.0670003891,
		Longitude: -9.4829998016,
		Timezone:  "Africa/Bamako",
	},
	"GAKY": {
		ICAO:      "GAKY",
		IATA:      "KYS",
		Name:      "Kayes Dag Dag Airport",
		State:     "Kayes",
		Country:   "ML",
		Elevation: 164,
		Latitude:  14.4812002182,
		Longitude: -11.4043998718,
		Timezone:  "Africa/Bamako",
	},
	"GAMA": {
		ICAO:      "GAMA",
		Name:      "Markala Airport",
		City:      "Markala",
		State:     "Segou",
		Country:   "ML",
		Elevation: 1251,
		Latitude:  13.6999998093,
		Longitude: -6.0669999123,
		Timezone:  "Africa/Bamako",
	},
	"GAMB": {
		ICAO:      "GAMB",
		IATA:      "MZI",
		Name:      "Ambodedjo Airport",
		State:     "Mopti",
		Country:   "ML",
		Elevation: 906,
		Latitude:  14.5128002167,
		Longitude: -4.079559803,
		Timezone:  "Africa/Bamako",
	},
	"GAMK": {
		ICAO:      "GAMK",
		Name:      "Menaka Airport",
		City:      "Menaka",
		State:     "Gao",
		Country:   "ML",
		Elevation: 899,
		Latitude:  15.8500003815,
		Longitude: 2.4330000877,
		Timezone:  "Africa/Bamako",
	},
	"GANF": {
		ICAO:      "GANF",
		Name:      "Niafunke Airport",
		City:      "Niafunke",
		State:     "Mopti",
		Country:   "ML",
		Elevation: 866,
		Latitude:  15.9329996109,
		Longitude: -4.0170001984,
		Timezone:  "Africa/Bamako",
	},
	"GANK": {
		ICAO:      "GANK",
		IATA:      "NRM",
		Name:      "Nara Airport",
		City:      "Nara",
		State:     "Koulikoro",
		Country:   "ML",
		Elevation: 889,
		Latitude:  15.2170000076,
		Longitude: -7.2670001984,
		Timezone:  "Africa/Bamako",
	},
	"GANR": {
		ICAO:      "GANR",
		IATA:      "NIX",
		Name:      "Nioro du Sahel Airport",
		City:      "Nioro du Sahel",
		State:     "Kayes",
		Country:   "ML",
		Elevation: 778,
		Latitude:  15.2381000519,
		Longitude: -9.5761098862,
		Timezone:  "Africa/Bamako",
	},
	"GASK": {
		ICAO:      "GASK",
		IATA:      "KSS",
		Name:      "Sikasso Airport",
		City:      "Sikasso",
		State:     "Sikasso",
		Country:   "ML",
		Elevation: 1378,
		Latitude:  11.3330001831,
		Longitude: -5.6999998093,
		Timezone:  "Africa/Bamako",
	},
	"GASO": {
		ICAO:      "GASO",
		Name:      "Dignangan Airport",
		State:     "Sikasso",
		Country:   "ML",
		Elevation: 1301,
		Latitude:  11.5979995728,
		Longitude: -5.7997097969,
		Timezone:  "Africa/Bamako",
	},
	"GATB": {
		ICAO:      "GATB",
		IATA:      "TOM",
		Name:      "Timbuktu Airport",
		City:      "Timbuktu",
		State:     "Tombouctou",
		Country:   "ML",
		Elevation: 863,
		Latitude:  16.7304992676,
		Longitude: -3.0075800419,
		Timezone:  "Africa/Bamako",
	},
	"GATS": {
		ICAO:      "GATS",
		Name:      "Tessalit Airport",
		State:     "Kidal",
		Country:   "ML",
		Elevation: 1621,
		Latitude:  20.2430000305,
		Longitude: 0.9773079753,
		Timezone:  "Africa/Bamako",
	},
	"GAYE": {
		ICAO:      "GAYE",
		IATA:      "EYL",
		Name:      "Yelimane Airport",
		City:      "Yelimane",
		State:     "Kayes",
		Country:   "ML",
		Elevation: 325,
		Latitude:  15.1330003738,
		Longitude: -10.5670003891,
		Timezone:  "Africa/Bamako",
	},
	"GBYD": {
		ICAO:      "GBYD",
		IATA:      "BJL",
		Name:      "Banjul International Airport",
		City:      "Banjul",
		State:     "Western",
		Country:   "GM",
		Elevation: 95,
		Latitude:  13.3380002975,
		Longitude: -16.6522006989,
		Timezone:  "Africa/Banjul",
	},
	"GCFV": {
		ICAO:      "GCFV",
		IATA:      "FUE",
		Name:      "Fuerteventura Airport",
		City:      "Fuerteventura Island",
		State:     "Canary-Islands",
		Country:   "ES",
		Elevation: 85,
		Latitude:  28.4526996613,
		Longitude: -13.8638000488,
		Timezone:  "Atlantic/Canary",
	},
	"GCGM": {
		ICAO:      "GCGM",
		IATA:      "GMZ",
		Name:      "La Gomera Airport",
		City:      "Alajero",
		State:     "Canary-Islands",
		Country:   "ES",
		Elevation: 716,
		Latitude:  28.0296001434,
		Longitude: -17.2145996094,
		Timezone:  "Atlantic/Canary",
	},
	"GCHI": {
		ICAO:      "GCHI",
		IATA:      "VDE",
		Name:      "Hierro Airport",
		City:      "El Hierro Island",
		State:     "Canary-Islands",
		Country:   "ES",
		Elevation: 103,
		Latitude:  27.8148002625,
		Longitude: -17.8871002197,
		Timezone:  "Atlantic/Canary",
	},
	"GCLA": {
		ICAO:      "GCLA",
		IATA:      "SPC",
		Name:      "La Palma Airport",
		City:      "Sta Cruz de la Palma",
		State:     "Canary-Islands",
		Country:   "ES",
		Elevation: 107,
		Latitude:  28.626499176,
		Longitude: -17.7555999756,
		Timezone:  "Atlantic/Canary",
	},
	"GCLB": {
		ICAO:      "GCLB",
		Name:      "El Berriel Aeroc Airport",
		City:      "Gran Canaria Island",
		State:     "Canary-Islands",
		Country:   "ES",
		Elevation: 20,
		Latitude:  27.7824993134,
		Longitude: -15.5072002411,
		Timezone:  "Atlantic/Canary",
	},
	"GCLP": {
		ICAO:      "GCLP",
		IATA:      "LPA",
		Name:      "Gran Canaria Airport",
		City:      "Gran Canaria Island",
		State:     "Canary-Islands",
		Country:   "ES",
		Elevation: 78,
		Latitude:  27.9319000244,
		Longitude: -15.3865995407,
		Timezone:  "Atlantic/Canary",
	},
	"GCRR": {
		ICAO:      "GCRR",
		IATA:      "ACE",
		Name:      "Lanzarote Airport",
		City:      "Lanzarote Island",
		State:     "Canary-Islands",
		Country:   "ES",
		Elevation: 46,
		Latitude:  28.9454994202,
		Longitude: -13.6051998138,
		Timezone:  "Atlantic/Canary",
	},
	"GCTS": {
		ICAO:      "GCTS",
		IATA:      "TFS",
		Name:      "Tenerife South Airport",
		City:      "Tenerife Island",
		State:     "Canary-Islands",
		Country:   "ES",
		Elevation: 209,
		Latitude:  28.044500351,
		Longitude: -16.5725002289,
		Timezone:  "Atlantic/Canary",
	},
	"GCXO": {
		ICAO:      "GCXO",
		IATA:      "TFN",
		Name:      "Tenerife Norte Airport",
		City:      "Tenerife Island",
		State:     "Canary-Islands",
		Country:   "ES",
		Elevation: 2076,
		Latitude:  28.4827003479,
		Longitude: -16.3414993286,
		Timezone:  "Atlantic/Canary",
	},
	"GE01": {
		ICAO:      "GE01",
		Name:      "Christians Folly Airport",
		City:      "Naylor",
		State:     "Georgia",
		Country:   "US",
		Elevation: 208,
		Latitude:  30.9190006256,
		Longitude: -83.0688018799,
		Timezone:  "America/New_York",
	},
	"GE04": {
		ICAO:      "GE04",
		Name:      "Mclendon Field",
		City:      "Mulberry Grove",
		State:     "Georgia",
		Country:   "US",
		Elevation: 650,
		Latitude:  32.666944,
		Longitude: -84.965556,
		Timezone:  "America/New_York",
	},
	"GE05": {
		ICAO:      "GE05",
		Name:      "Gibson Field",
		City:      "Lafayette",
		State:     "Georgia",
		Country:   "US",
		Elevation: 750,
		Latitude:  34.6156005859,
		Longitude: -85.3431015015,
		Timezone:  "America/New_York",
	},
	"GE11": {
		ICAO:      "GE11",
		Name:      "Hogjowl Airport",
		City:      "Layfayette",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1790,
		Latitude:  34.6058006287,
		Longitude: -85.4618988037,
		Timezone:  "America/New_York",
	},
	"GE12": {
		ICAO:      "GE12",
		Name:      "Richter Airpark",
		City:      "Ashburn",
		State:     "Georgia",
		Country:   "US",
		Elevation: 357,
		Latitude:  31.7486000061,
		Longitude: -83.573600769,
		Timezone:  "America/New_York",
	},
	"GE23": {
		ICAO:      "GE23",
		Name:      "Winge Farms Airport",
		City:      "Lyons",
		State:     "Georgia",
		Country:   "US",
		Elevation: 195,
		Latitude:  32.1935997009,
		Longitude: -82.2906036377,
		Timezone:  "America/New_York",
	},
	"GE25": {
		ICAO:      "GE25",
		Name:      "Hutto Farm Airport",
		City:      "Fort Valley",
		State:     "Georgia",
		Country:   "US",
		Elevation: 620,
		Latitude:  32.6833000183,
		Longitude: -83.8569030762,
		Timezone:  "America/New_York",
	},
	"GE26": {
		ICAO:      "GE26",
		Name:      "Duke Strip 2",
		City:      "Dublin",
		State:     "Georgia",
		Country:   "US",
		Elevation: 340,
		Latitude:  32.381667,
		Longitude: -83.059722,
		Timezone:  "America/New_York",
	},
	"GE27": {
		ICAO:      "GE27",
		Name:      "Smith Field",
		City:      "Byron",
		State:     "Georgia",
		Country:   "US",
		Elevation: 475,
		Latitude:  32.6021995544,
		Longitude: -83.7542037964,
		Timezone:  "America/New_York",
	},
	"GE30": {
		ICAO:      "GE30",
		Name:      "Sandy Hill Airport",
		City:      "Waynesboro",
		State:     "Georgia",
		Country:   "US",
		Elevation: 310,
		Latitude:  33.2210998535,
		Longitude: -81.995300293,
		Timezone:  "America/New_York",
	},
	"GE33": {
		ICAO:      "GE33",
		Name:      "River Bend Airport",
		City:      "Locust Grove",
		State:     "Georgia",
		Country:   "US",
		Elevation: 718,
		Latitude:  33.304444,
		Longitude: -84.136667,
		Timezone:  "America/New_York",
	},
	"GE35": {
		ICAO:      "GE35",
		Name:      "Southern Oaks Airport",
		City:      "Bostwick",
		State:     "Georgia",
		Country:   "US",
		Elevation: 121,
		Latitude:  33.731389,
		Longitude: -83.469722,
		Timezone:  "America/New_York",
	},
	"GE99": {
		ICAO:      "GE99",
		Name:      "Heaven's Landing Airport",
		City:      "Clayton",
		State:     "Georgia",
		Country:   "US",
		Elevation: 2724,
		Latitude:  34.9143981934,
		Longitude: -83.4597015381,
		Timezone:  "America/New_York",
	},
	"GEML": {
		ICAO:      "GEML",
		IATA:      "MLN",
		Name:      "Melilla Airport",
		City:      "Melilla Island",
		State:     "Melilla",
		Country:   "ES",
		Elevation: 156,
		Latitude:  35.279800415,
		Longitude: -2.9562599659,
		Timezone:  "Africa/Casablanca",
	},
	"GFBN": {
		ICAO:      "GFBN",
		IATA:      "BTE",
		Name:      "Sherbro International Airport",
		City:      "Bonthe",
		State:     "Southern-Province",
		Country:   "SL",
		Elevation: 14,
		Latitude:  7.5324201584,
		Longitude: -12.5188999176,
		Timezone:  "Africa/Freetown",
	},
	"GFBO": {
		ICAO:      "GFBO",
		IATA:      "KBS",
		Name:      "Bo Airport",
		City:      "Bo",
		State:     "Southern-Province",
		Country:   "SL",
		Elevation: 328,
		Latitude:  7.9443998337,
		Longitude: -11.7609996796,
		Timezone:  "Africa/Freetown",
	},
	"GFGK": {
		ICAO:      "GFGK",
		IATA:      "GBK",
		Name:      "Gbangbatok Airport",
		City:      "Gbangbatok",
		State:     "Southern-Province",
		Country:   "SL",
		Elevation: 75,
		Latitude:  7.7670001984,
		Longitude: -12.3830003738,
		Timezone:  "Africa/Freetown",
	},
	"GFHA": {
		ICAO:      "GFHA",
		IATA:      "HGS",
		Name:      "Hastings Airport",
		City:      "Freetown",
		State:     "Western-Area",
		Country:   "SL",
		Elevation: 60,
		Latitude:  8.3971300125,
		Longitude: -13.1290998459,
		Timezone:  "Africa/Freetown",
	},
	"GFKB": {
		ICAO:      "GFKB",
		IATA:      "KBA",
		Name:      "Kabala Airport",
		City:      "Kabala",
		State:     "Northern-Province",
		Country:   "SL",
		Elevation: 1012,
		Latitude:  9.6383229134,
		Longitude: -11.5155601501,
		Timezone:  "Africa/Freetown",
	},
	"GFKE": {
		ICAO:      "GFKE",
		IATA:      "KEN",
		Name:      "Kenema Airport",
		City:      "Kenema",
		State:     "Eastern-Province",
		Country:   "SL",
		Elevation: 485,
		Latitude:  7.8912901878,
		Longitude: -11.1766004562,
		Timezone:  "Africa/Freetown",
	},
	"GFLL": {
		ICAO:      "GFLL",
		IATA:      "FNA",
		Name:      "Lungi International Airport",
		City:      "Freetown",
		State:     "Northern-Province",
		Country:   "SL",
		Elevation: 84,
		Latitude:  8.6164398193,
		Longitude: -13.1955003738,
		Timezone:  "Africa/Freetown",
	},
	"GFTO": {
		ICAO:      "GFTO",
		Name:      "Tongo Airport",
		City:      "Tongo",
		State:     "Eastern-Province",
		Country:   "SL",
		Elevation: 750,
		Latitude:  8.2170000076,
		Longitude: -11.0670003891,
		Timezone:  "Africa/Freetown",
	},
	"GFYE": {
		ICAO:      "GFYE",
		IATA:      "WYE",
		Name:      "Yengema Airport",
		City:      "Yengema",
		State:     "Eastern-Province",
		Country:   "SL",
		Elevation: 1300,
		Latitude:  8.6104698181,
		Longitude: -11.0453996658,
		Timezone:  "Africa/Freetown",
	},
	"GG64": {
		ICAO:      "GG64",
		Name:      "Quebo Airport",
		City:      "Quebo",
		Country:   "GW",
		Elevation: 165,
		Latitude:  11.5355997086,
		Longitude: -14.7622995377,
		Timezone:  "Africa/Bissau",
	},
	"GGBF": {
		ICAO:      "GGBF",
		Name:      "Bafata Airport",
		City:      "Bafata",
		Country:   "GW",
		Latitude:  12.1666669846,
		Longitude: -14.6666669846,
		Timezone:  "Africa/Bissau",
	},
	"GGBU": {
		ICAO:      "GGBU",
		IATA:      "BQE",
		Name:      "Bubaque Airport",
		City:      "Bubaque",
		State:     "Bolama-and-Bijagos",
		Country:   "GW",
		Latitude:  11.2973556519,
		Longitude: -15.8380794525,
		Timezone:  "Africa/Bissau",
	},
	"GGCF": {
		ICAO:      "GGCF",
		Name:      "Cufar Airport",
		City:      "Cufar",
		Country:   "GW",
		Elevation: 65,
		Latitude:  11.287899971,
		Longitude: -15.1805000305,
		Timezone:  "Africa/Bissau",
	},
	"GGOV": {
		ICAO:      "GGOV",
		IATA:      "OXB",
		Name:      "Osvaldo Vieira International Airport",
		City:      "Bissau",
		Country:   "GW",
		Elevation: 129,
		Latitude:  11.8948001862,
		Longitude: -15.6536998749,
		Timezone:  "Africa/Bissau",
	},
	"GLBU": {
		ICAO:      "GLBU",
		IATA:      "UCN",
		Name:      "Buchanan Airport",
		City:      "Buchanan",
		State:     "Sinoe",
		Country:   "LR",
		Elevation: 41,
		Latitude:  5.1676502228,
		Longitude: -9.2834701538,
		Timezone:  "Africa/Monrovia",
	},
	"GLCM": {
		ICAO:      "GLCM",
		Name:      "Robertsport/Cape Mount Airport",
		City:      "Robertsport/Cape Mount",
		State:     "Grand-Cape-Mount",
		Country:   "LR",
		Elevation: 10,
		Latitude:  6.8000001907,
		Longitude: -11.3669996262,
		Timezone:  "Africa/Monrovia",
	},
	"GLCP": {
		ICAO:      "GLCP",
		IATA:      "CPA",
		Name:      "Cape Palmas Airport",
		City:      "Harper",
		State:     "Maryland",
		Country:   "LR",
		Elevation: 20,
		Latitude:  4.3790202141,
		Longitude: -7.6969499588,
		Timezone:  "Africa/Monrovia",
	},
	"GLGE": {
		ICAO:      "GLGE",
		IATA:      "SNI",
		Name:      "Greenville Sinoe Airport",
		City:      "Greenville",
		State:     "Sinoe",
		Country:   "LR",
		Elevation: 10,
		Latitude:  5.034309864,
		Longitude: -9.0668001175,
		Timezone:  "Africa/Monrovia",
	},
	"GLLB": {
		ICAO:      "GLLB",
		Name:      "Lamco Airport",
		City:      "Buchanan",
		State:     "Grand-Bassa",
		Country:   "LR",
		Elevation: 30,
		Latitude:  5.867000103,
		Longitude: -10.0500001907,
		Timezone:  "Africa/Monrovia",
	},
	"GLMR": {
		ICAO:      "GLMR",
		IATA:      "MLW",
		Name:      "Spriggs Payne Airport",
		City:      "Monrovia",
		State:     "Montserrado",
		Country:   "LR",
		Elevation: 25,
		Latitude:  6.2890601158,
		Longitude: -10.7587003708,
		Timezone:  "Africa/Monrovia",
	},
	"GLNA": {
		ICAO:      "GLNA",
		IATA:      "NIA",
		Name:      "Nimba Airport",
		City:      "Nimba",
		State:     "Nimba",
		Country:   "LR",
		Elevation: 1632,
		Latitude:  7.5,
		Longitude: -8.6000003815,
		Timezone:  "Africa/Monrovia",
	},
	"GLRB": {
		ICAO:      "GLRB",
		IATA:      "ROB",
		Name:      "Roberts International Airport",
		City:      "Monrovia",
		State:     "Margibi",
		Country:   "LR",
		Elevation: 31,
		Latitude:  6.2337899208,
		Longitude: -10.3622999191,
		Timezone:  "Africa/Monrovia",
	},
	"GLST": {
		ICAO:      "GLST",
		IATA:      "SAZ",
		Name:      "Sasstown Airport",
		City:      "Sasstown",
		State:     "Sinoe",
		Country:   "LR",
		Elevation: 6,
		Latitude:  4.6666669846,
		Longitude: -8.4333333969,
		Timezone:  "Africa/Monrovia",
	},
	"GLTN": {
		ICAO:      "GLTN",
		IATA:      "THC",
		Name:      "Tchien Airport",
		City:      "Tchien",
		State:     "Grand-Gedeh",
		Country:   "LR",
		Elevation: 790,
		Latitude:  6.0456504822,
		Longitude: -8.1387233734,
		Timezone:  "Africa/Monrovia",
	},
	"GLVA": {
		ICAO:      "GLVA",
		IATA:      "VOI",
		Name:      "Voinjama Airport",
		City:      "Voinjama",
		State:     "Lofa",
		Country:   "LR",
		Elevation: 1400,
		Latitude:  8.3999996185,
		Longitude: -9.7670001984,
		Timezone:  "Africa/Monrovia",
	},
	"GMAA": {
		ICAO:      "GMAA",
		Name:      "Inezgane Airport",
		State:     "Souss-Massa",
		Country:   "MA",
		Elevation: 89,
		Latitude:  30.3813991547,
		Longitude: -9.5463104248,
		Timezone:  "Africa/Casablanca",
	},
	"GMAD": {
		ICAO:      "GMAD",
		IATA:      "AGA",
		Name:      "Al Massira Airport",
		City:      "Agadir",
		State:     "Souss-Massa",
		Country:   "MA",
		Elevation: 250,
		Latitude:  30.3250007629,
		Longitude: -9.413069725,
		Timezone:  "Africa/Casablanca",
	},
	"GMAT": {
		ICAO:      "GMAT",
		IATA:      "TTA",
		Name:      "Tan Tan Airport",
		City:      "Tan Tan",
		State:     "Guelmim-Oued-Noun",
		Country:   "MA",
		Elevation: 653,
		Latitude:  28.4482002258,
		Longitude: -11.1612997055,
		Timezone:  "Africa/Casablanca",
	},
	"GMAZ": {
		ICAO:      "GMAZ",
		IATA:      "OZG",
		Name:      "Zagora Airport",
		City:      "Zagora",
		State:     "Draa-Tafilalet",
		Country:   "MA",
		Elevation: 2631,
		Latitude:  30.3202991486,
		Longitude: -5.8666701317,
		Timezone:  "Africa/Casablanca",
	},
	"GMFA": {
		ICAO:      "GMFA",
		Name:      "Ouezzane Airport",
		City:      "Beni Malek",
		State:     "Tanger-Tetouan-Al-Hoceima",
		Country:   "MA",
		Elevation: 571,
		Latitude:  34.7832984924,
		Longitude: -5.6333298683,
		Timezone:  "Africa/Casablanca",
	},
	"GMFB": {
		ICAO:      "GMFB",
		IATA:      "UAR",
		Name:      "Bouarfa Airport",
		City:      "Bouarfa",
		Country:   "MA",
		Elevation: 3630,
		Latitude:  32.5143055556,
		Longitude: -1.9830555556,
		Timezone:  "Africa/Casablanca",
	},
	"GMFF": {
		ICAO:      "GMFF",
		IATA:      "FEZ",
		Name:      "Saiss Airport",
		City:      "Fes",
		State:     "Fes-Meknes",
		Country:   "MA",
		Elevation: 1900,
		Latitude:  33.9272994995,
		Longitude: -4.9779601097,
		Timezone:  "Africa/Casablanca",
	},
	"GMFI": {
		ICAO:      "GMFI",
		Name:      "Ifrane Airport",
		State:     "Fes-Meknes",
		Country:   "MA",
		Elevation: 5459,
		Latitude:  33.5052986145,
		Longitude: -5.152900219,
		Timezone:  "Africa/Casablanca",
	},
	"GMFK": {
		ICAO:      "GMFK",
		IATA:      "ERH",
		Name:      "Moulay Ali Cherif Airport",
		City:      "Errachidia",
		Country:   "MA",
		Elevation: 3428,
		Latitude:  31.9475002289,
		Longitude: -4.3983302116,
		Timezone:  "Africa/Casablanca",
	},
	"GMFM": {
		ICAO:      "GMFM",
		IATA:      "MEK",
		Name:      "Bassatine Airport",
		City:      "Meknes",
		State:     "Fes-Meknes",
		Country:   "MA",
		Elevation: 1890,
		Latitude:  33.8791007996,
		Longitude: -5.5151200294,
		Timezone:  "Africa/Casablanca",
	},
	"GMFN": {
		ICAO:      "GMFN",
		Name:      "Taouima Nador Airport",
		City:      "Nador",
		State:     "Oriental",
		Country:   "MA",
		Elevation: 10,
		Latitude:  35.1500015259,
		Longitude: -2.9170000553,
		Timezone:  "Africa/Casablanca",
	},
	"GMFO": {
		ICAO:      "GMFO",
		IATA:      "OUD",
		Name:      "Angads Airport",
		City:      "Oujda",
		State:     "Oriental",
		Country:   "MA",
		Elevation: 1535,
		Latitude:  34.7872009277,
		Longitude: -1.9239900112,
		Timezone:  "Africa/Casablanca",
	},
	"GMFU": {
		ICAO:      "GMFU",
		Name:      "Fes Sefrou Airport",
		City:      "Fes",
		State:     "Fes-Meknes",
		Country:   "MA",
		Elevation: 1539,
		Latitude:  34.0080986023,
		Longitude: -4.9655599594,
		Timezone:  "Africa/Casablanca",
	},
	"GMFZ": {
		ICAO:      "GMFZ",
		Name:      "Taza Airport",
		City:      "Taza",
		State:     "Fes-Meknes",
		Country:   "MA",
		Elevation: 1890,
		Latitude:  34.2332992554,
		Longitude: -3.9500000477,
		Timezone:  "Africa/Casablanca",
	},
	"GMMA": {
		ICAO:      "GMMA",
		IATA:      "SMW",
		Name:      "Smara Airport",
		City:      "Smara",
		State:     "Laayoune-Sakia-El-Hamra",
		Country:   "EH",
		Elevation: 350,
		Latitude:  26.7318000793,
		Longitude: -11.6847000122,
		Timezone:  "Africa/El_Aaiun",
	},
	"GMMB": {
		ICAO:      "GMMB",
		IATA:      "GMD",
		Name:      "Ben Slimane Airport",
		City:      "Ben Slimane",
		State:     "Casablanca-Settat",
		Country:   "MA",
		Elevation: 627,
		Latitude:  33.6553993225,
		Longitude: -7.221449852,
		Timezone:  "Africa/Casablanca",
	},
	"GMMC": {
		ICAO:      "GMMC",
		IATA:      "CAS",
		Name:      "Anfa Airport",
		City:      "Casablanca",
		State:     "Casablanca-Settat",
		Country:   "MA",
		Elevation: 200,
		Latitude:  33.5532989502,
		Longitude: -7.6613898277,
		Timezone:  "Africa/Casablanca",
	},
	"GMMD": {
		ICAO:      "GMMD",
		IATA:      "BEM",
		Name:      "Beni Mellal Airport",
		State:     "Beni-Mellal-Khenifra",
		Country:   "MA",
		Elevation: 1670,
		Latitude:  32.4000015259,
		Longitude: -6.3333301544,
		Timezone:  "Africa/Casablanca",
	},
	"GMME": {
		ICAO:      "GMME",
		IATA:      "RBA",
		Name:      "Rabat-Sale Airport",
		City:      "Rabat",
		Country:   "MA",
		Elevation: 276,
		Latitude:  34.0514984131,
		Longitude: -6.7515201569,
		Timezone:  "Africa/Casablanca",
	},
	"GMMF": {
		ICAO:      "GMMF",
		IATA:      "SII",
		Name:      "Sidi Ifni Xx Airport",
		City:      "Sidi Ifni",
		State:     "Guelmim-Oued-Noun",
		Country:   "MA",
		Elevation: 190,
		Latitude:  29.3666992188,
		Longitude: -10.1878004074,
		Timezone:  "Africa/Casablanca",
	},
	"GMMH": {
		ICAO:      "GMMH",
		IATA:      "VIL",
		Name:      "Dakhla Airport",
		City:      "Dakhla",
		State:     "Dakhla-Oued-Ed-Dahab",
		Country:   "EH",
		Elevation: 36,
		Latitude:  23.7182998657,
		Longitude: -15.9320001602,
		Timezone:  "Africa/El_Aaiun",
	},
	"GMMI": {
		ICAO:      "GMMI",
		IATA:      "ESU",
		Name:      "Mogador Airport",
		City:      "Essaouira",
		Country:   "MA",
		Elevation: 384,
		Latitude:  31.3974990845,
		Longitude: -9.6816701889,
		Timezone:  "Africa/Casablanca",
	},
	"GMMJ": {
		ICAO:      "GMMJ",
		Name:      "El Jadida Airport",
		City:      "El Jadida",
		State:     "Casablanca-Settat",
		Country:   "MA",
		Elevation: 92,
		Latitude:  33.2332992554,
		Longitude: -8.5166702271,
		Timezone:  "Africa/Casablanca",
	},
	"GMML": {
		ICAO:      "GMML",
		IATA:      "EUN",
		Name:      "Hassan I Airport",
		City:      "El Aaiun",
		State:     "Laayoune-Sakia-El-Hamra",
		Country:   "EH",
		Elevation: 207,
		Latitude:  27.1516990662,
		Longitude: -13.2192001343,
		Timezone:  "Africa/El_Aaiun",
	},
	"GMMN": {
		ICAO:      "GMMN",
		IATA:      "CMN",
		Name:      "Mohammed V International Airport",
		City:      "Casablanca",
		Country:   "MA",
		Elevation: 656,
		Latitude:  33.3675003052,
		Longitude: -7.5899701118,
		Timezone:  "Africa/Casablanca",
	},
	"GMMO": {
		ICAO:      "GMMO",
		Name:      "Taroudant Airport",
		City:      "Taroudant",
		State:     "Souss-Massa",
		Country:   "MA",
		Elevation: 869,
		Latitude:  30.5016994476,
		Longitude: -8.8275003433,
		Timezone:  "Africa/Casablanca",
	},
	"GMMS": {
		ICAO:      "GMMS",
		IATA:      "SFI",
		Name:      "Safi Airport",
		City:      "Safi",
		State:     "Marrakesh-Safi",
		Country:   "MA",
		Elevation: 171,
		Latitude:  32.2832984924,
		Longitude: -9.2333297729,
		Timezone:  "Africa/Casablanca",
	},
	"GMMT": {
		ICAO:      "GMMT",
		Name:      "Tit Mellil Airport",
		City:      "Casablanca",
		State:     "Casablanca-Settat",
		Country:   "MA",
		Elevation: 322,
		Latitude:  33.5974998474,
		Longitude: -7.4650001526,
		Timezone:  "Africa/Casablanca",
	},
	"GMMW": {
		ICAO:      "GMMW",
		IATA:      "NDR",
		Name:      "Nador International Airport",
		City:      "Nador",
		State:     "Oriental",
		Country:   "MA",
		Elevation: 574,
		Latitude:  34.9888000488,
		Longitude: -3.0282099247,
		Timezone:  "Africa/Casablanca",
	},
	"GMMX": {
		ICAO:      "GMMX",
		IATA:      "RAK",
		Name:      "Menara Airport",
		City:      "Marrakech",
		State:     "Marrakesh-Safi",
		Country:   "MA",
		Elevation: 1545,
		Latitude:  31.6068992615,
		Longitude: -8.0362997055,
		Timezone:  "Africa/Casablanca",
	},
	"GMMY": {
		ICAO:      "GMMY",
		IATA:      "NNA",
		Name:      "Kenitra Airport",
		Country:   "MA",
		Elevation: 16,
		Latitude:  34.2989006042,
		Longitude: -6.5958800316,
		Timezone:  "Africa/Casablanca",
	},
	"GMMZ": {
		ICAO:      "GMMZ",
		IATA:      "OZZ",
		Name:      "Ouarzazate Airport",
		City:      "Ouarzazate",
		State:     "Draa-Tafilalet",
		Country:   "MA",
		Elevation: 3782,
		Latitude:  30.9391002655,
		Longitude: -6.909430027,
		Timezone:  "Africa/Casablanca",
	},
	"GMSL": {
		ICAO:      "GMSL",
		Name:      "Sidi Slimane Airport",
		City:      "Sidi Slimane",
		State:     "Rabat-Sale-Kenitra",
		Country:   "MA",
		Elevation: 179,
		Latitude:  34.2305984497,
		Longitude: -6.050139904,
		Timezone:  "Africa/Casablanca",
	},
	"GMTA": {
		ICAO:      "GMTA",
		IATA:      "AHU",
		Name:      "Cherif Al Idrissi Airport",
		City:      "Al Hoceima",
		State:     "Tanger-Tetouan-Al-Hoceima",
		Country:   "MA",
		Elevation: 95,
		Latitude:  35.1771011353,
		Longitude: -3.8395199776,
		Timezone:  "Africa/Casablanca",
	},
	"GMTN": {
		ICAO:      "GMTN",
		IATA:      "TTU",
		Name:      "Saniat Rmel Airport",
		Country:   "MA",
		Elevation: 10,
		Latitude:  35.5942993164,
		Longitude: -5.3200201988,
		Timezone:  "Africa/Casablanca",
	},
	"GMTT": {
		ICAO:      "GMTT",
		IATA:      "TNG",
		Name:      "Ibn Batouta Airport",
		City:      "Tangier",
		Country:   "MA",
		Elevation: 62,
		Latitude:  35.7268981934,
		Longitude: -5.9168901444,
		Timezone:  "Africa/Casablanca",
	},
	"GO66": {
		ICAO:      "GO66",
		Name:      "Dodji Airport",
		City:      "Dodji",
		State:     "Louga",
		Country:   "SN",
		Elevation: 66,
		Latitude:  15.5438995361,
		Longitude: -14.9582996368,
		Timezone:  "Africa/Dakar",
	},
	"GOBD": {
		ICAO:      "GOBD",
		IATA:      "DSS",
		Name:      "Blaise Diagne International Airport",
		City:      "Diass",
		State:     "M'bour",
		Country:   "SN",
		Elevation: 289,
		Latitude:  14.671111,
		Longitude: -17.066944,
		Timezone:  "Africa/Dakar",
	},
	"GODK": {
		ICAO:      "GODK",
		IATA:      "KDA",
		Name:      "Kolda North Airport",
		City:      "Kolda",
		State:     "Kolda",
		Country:   "SN",
		Elevation: 33,
		Latitude:  12.8985004425,
		Longitude: -14.9680995941,
		Timezone:  "Africa/Dakar",
	},
	"GOGG": {
		ICAO:      "GOGG",
		IATA:      "ZIG",
		Name:      "Ziguinchor Airport",
		City:      "Ziguinchor",
		State:     "Ziguinchor",
		Country:   "SN",
		Elevation: 75,
		Latitude:  12.5556001663,
		Longitude: -16.2817993164,
		Timezone:  "Africa/Dakar",
	},
	"GOGS": {
		ICAO:      "GOGS",
		IATA:      "CSK",
		Name:      "Cap Skirring Airport",
		City:      "Cap Skirring",
		State:     "Ziguinchor",
		Country:   "SN",
		Elevation: 52,
		Latitude:  12.410200119,
		Longitude: -16.746099472,
		Timezone:  "Africa/Dakar",
	},
	"GOOK": {
		ICAO:      "GOOK",
		IATA:      "KLC",
		Name:      "Kaolack Airport",
		City:      "Kaolack",
		State:     "Kaolack",
		Country:   "SN",
		Elevation: 26,
		Latitude:  14.146900177,
		Longitude: -16.0513000488,
		Timezone:  "Africa/Dakar",
	},
	"GOOY": {
		ICAO:      "GOOY",
		IATA:      "DKR",
		Name:      "Leopold Sedar Senghor International Airport",
		City:      "Dakar",
		State:     "Dakar",
		Country:   "SN",
		Elevation: 85,
		Latitude:  14.7397003174,
		Longitude: -17.4902000427,
		Timezone:  "Africa/Dakar",
	},
	"GOSM": {
		ICAO:      "GOSM",
		IATA:      "MAX",
		Name:      "Ouro Sogui Airport",
		City:      "Matam",
		State:     "Saint-Louis",
		Country:   "SN",
		Elevation: 85,
		Latitude:  15.5936002731,
		Longitude: -13.3227996826,
		Timezone:  "Africa/Dakar",
	},
	"GOSP": {
		ICAO:      "GOSP",
		IATA:      "POD",
		Name:      "Podor Airport",
		City:      "Podor",
		State:     "Saint-Louis",
		Country:   "SN",
		Elevation: 20,
		Latitude:  16.6830005646,
		Longitude: -14.9670000076,
		Timezone:  "Africa/Dakar",
	},
	"GOSR": {
		ICAO:      "GOSR",
		IATA:      "RDT",
		Name:      "Richard Toll Airport",
		City:      "Richard Toll",
		State:     "Saint-Louis",
		Country:   "SN",
		Elevation: 20,
		Latitude:  16.4330005646,
		Longitude: -15.6499996185,
		Timezone:  "Africa/Dakar",
	},
	"GOSS": {
		ICAO:      "GOSS",
		IATA:      "XLS",
		Name:      "Saint Louis Airport",
		City:      "Saint Louis",
		State:     "Saint-Louis",
		Country:   "SN",
		Elevation: 9,
		Latitude:  16.0508003235,
		Longitude: -16.4631996155,
		Timezone:  "Africa/Dakar",
	},
	"GOTB": {
		ICAO:      "GOTB",
		IATA:      "BXE",
		Name:      "Bakel Airport",
		City:      "Bakel",
		State:     "Tambacounda",
		Country:   "SN",
		Elevation: 98,
		Latitude:  14.8472995758,
		Longitude: -12.4682998657,
		Timezone:  "Africa/Dakar",
	},
	"GOTK": {
		ICAO:      "GOTK",
		IATA:      "KGG",
		Name:      "Kedougou Airport",
		City:      "Kedougou",
		State:     "Kedougou",
		Country:   "SN",
		Elevation: 584,
		Latitude:  12.5722999573,
		Longitude: -12.2202997208,
		Timezone:  "Africa/Dakar",
	},
	"GOTS": {
		ICAO:      "GOTS",
		IATA:      "SMY",
		Name:      "Simenti Airport",
		City:      "Simenti",
		State:     "Tambacounda",
		Country:   "SN",
		Elevation: 171,
		Latitude:  13.0467996597,
		Longitude: -13.2953996658,
		Timezone:  "Africa/Dakar",
	},
	"GOTT": {
		ICAO:      "GOTT",
		IATA:      "TUD",
		Name:      "Tambacounda Airport",
		City:      "Tambacounda",
		State:     "Tambacounda",
		Country:   "SN",
		Elevation: 161,
		Latitude:  13.7368001938,
		Longitude: -13.6531000137,
		Timezone:  "Africa/Dakar",
	},
	"GQNA": {
		ICAO:      "GQNA",
		IATA:      "AEO",
		Name:      "Aioun el Atrouss Airport",
		City:      "Aioun El Atrouss",
		Country:   "MR",
		Elevation: 951,
		Latitude:  16.7112998962,
		Longitude: -9.6378803253,
		Timezone:  "Africa/Nouakchott",
	},
	"GQNB": {
		ICAO:      "GQNB",
		IATA:      "OTL",
		Name:      "Boutilimit Airport",
		City:      "Boutilimit",
		State:     "Trarza",
		Country:   "MR",
		Elevation: 121,
		Latitude:  17.533000946,
		Longitude: -14.6829996109,
		Timezone:  "Africa/Nouakchott",
	},
	"GQNC": {
		ICAO:      "GQNC",
		IATA:      "THI",
		Name:      "Tichitt Airport",
		City:      "Tichitt",
		State:     "Tagant",
		Country:   "MR",
		Elevation: 561,
		Latitude:  18.4500007629,
		Longitude: -9.5170001984,
		Timezone:  "Africa/Nouakchott",
	},
	"GQND": {
		ICAO:      "GQND",
		IATA:      "TIY",
		Name:      "Tidjikja Airport",
		City:      "Tidjikja",
		State:     "Tagant",
		Country:   "MR",
		Elevation: 1363,
		Latitude:  18.5701007843,
		Longitude: -11.423500061,
		Timezone:  "Africa/Nouakchott",
	},
	"GQNE": {
		ICAO:      "GQNE",
		IATA:      "BGH",
		Name:      "Abbaye Airport",
		City:      "Boghe",
		State:     "Brakna",
		Country:   "MR",
		Elevation: 66,
		Latitude:  16.6333141327,
		Longitude: -14.2000007629,
		Timezone:  "Africa/Nouakchott",
	},
	"GQNF": {
		ICAO:      "GQNF",
		IATA:      "KFA",
		Name:      "Kiffa Airport",
		City:      "Kiffa",
		State:     "Assaba",
		Country:   "MR",
		Elevation: 424,
		Latitude:  16.5900001526,
		Longitude: -11.4062004089,
		Timezone:  "Africa/Nouakchott",
	},
	"GQNH": {
		ICAO:      "GQNH",
		IATA:      "TMD",
		Name:      "Timbedra Airport",
		City:      "Timbedra",
		State:     "Hodh-ech-Chargui",
		Country:   "MR",
		Elevation: 692,
		Latitude:  16.2329998016,
		Longitude: -8.1669998169,
		Timezone:  "Africa/Nouakchott",
	},
	"GQNI": {
		ICAO:      "GQNI",
		IATA:      "EMN",
		Name:      "Nema Airport",
		City:      "Nema",
		State:     "Hodh-ech-Chargui",
		Country:   "MR",
		Elevation: 751,
		Latitude:  16.6219997406,
		Longitude: -7.3165998459,
		Timezone:  "Africa/Nouakchott",
	},
	"GQNJ": {
		ICAO:      "GQNJ",
		IATA:      "AJJ",
		Name:      "Akjoujt Airport",
		City:      "Akjoujt",
		Country:   "MR",
		Elevation: 403,
		Latitude:  19.7330169678,
		Longitude: -14.3832492828,
		Timezone:  "Africa/Nouakchott",
	},
	"GQNK": {
		ICAO:      "GQNK",
		IATA:      "KED",
		Name:      "Kaedi Airport",
		City:      "Kaedi",
		State:     "Gorgol",
		Country:   "MR",
		Elevation: 66,
		Latitude:  16.1595001221,
		Longitude: -13.5075998306,
		Timezone:  "Africa/Dakar",
	},
	"GQNL": {
		ICAO:      "GQNL",
		IATA:      "MOM",
		Name:      "Letfotar Airport",
		City:      "Moudjeria",
		State:     "Brakna",
		Country:   "MR",
		Elevation: 256,
		Latitude:  17.75,
		Longitude: -12.5,
		Timezone:  "Africa/Nouakchott",
	},
	"GQNM": {
		ICAO:      "GQNM",
		Name:      "Dahara Airport",
		City:      "Timbreda",
		State:     "Hodh-ech-Chargui",
		Country:   "MR",
		Elevation: 722,
		Latitude:  16.2999992371,
		Longitude: -8.0500001907,
		Timezone:  "Africa/Nouakchott",
	},
	"GQNN": {
		ICAO:      "GQNN",
		IATA:      "NKC",
		Name:      "Nouakchott International Airport",
		City:      "Nouakchott",
		State:     "Trarza",
		Country:   "MR",
		Elevation: 13,
		Latitude:  18.0981998444,
		Longitude: -15.9484996796,
		Timezone:  "Africa/Nouakchott",
	},
	"GQNS": {
		ICAO:      "GQNS",
		IATA:      "SEY",
		Name:      "Selibaby Airport",
		City:      "Selibaby",
		State:     "Guidimaka",
		Country:   "MR",
		Elevation: 219,
		Latitude:  15.1796998978,
		Longitude: -12.2073001862,
		Timezone:  "Africa/Nouakchott",
	},
	"GQNT": {
		ICAO:      "GQNT",
		IATA:      "THT",
		Name:      "Tamchakett Airport",
		City:      "Tamchakett",
		State:     "Hodh-El-Gharbi",
		Country:   "MR",
		Elevation: 620,
		Latitude:  17.2329998016,
		Longitude: -10.8170003891,
		Timezone:  "Africa/Nouakchott",
	},
	"GQPA": {
		ICAO:      "GQPA",
		IATA:      "ATR",
		Name:      "Atar International Airport",
		City:      "Atar",
		State:     "Adrar",
		Country:   "MR",
		Elevation: 734,
		Latitude:  20.5067996979,
		Longitude: -13.0431995392,
		Timezone:  "Africa/Nouakchott",
	},
	"GQPF": {
		ICAO:      "GQPF",
		IATA:      "FGD",
		Name:      "Fderik Airport",
		City:      "Fderik",
		State:     "Tiris-Zemmour",
		Country:   "MR",
		Elevation: 961,
		Latitude:  22.6669998169,
		Longitude: -12.7329998016,
		Timezone:  "Africa/Nouakchott",
	},
	"GQPP": {
		ICAO:      "GQPP",
		IATA:      "NDB",
		Name:      "Nouadhibou International Airport",
		City:      "Nouadhibou",
		State:     "Dakhlet-Nouadhibou",
		Country:   "MR",
		Elevation: 24,
		Latitude:  20.9330997467,
		Longitude: -17.0300006866,
		Timezone:  "Africa/El_Aaiun",
	},
	"GQPT": {
		ICAO:      "GQPT",
		Name:      "Bir Moghrein Airport",
		City:      "Bir Moghrein",
		State:     "Tiris-Zemmour",
		Country:   "MR",
		Elevation: 1206,
		Latitude:  25.236700058,
		Longitude: -11.5887002945,
		Timezone:  "Africa/Nouakchott",
	},
	"GQPZ": {
		ICAO:      "GQPZ",
		IATA:      "OUZ",
		Name:      "Tazadit Airport",
		City:      "Zouerate",
		State:     "Tiris-Zemmour",
		Country:   "MR",
		Elevation: 1129,
		Latitude:  22.7563991547,
		Longitude: -12.4835996628,
		Timezone:  "Africa/Nouakchott",
	},
	"GUCY": {
		ICAO:      "GUCY",
		IATA:      "CKY",
		Name:      "Conakry Airport",
		City:      "Conakry",
		State:     "Conakry",
		Country:   "GN",
		Elevation: 72,
		Latitude:  9.57689,
		Longitude: -13.612,
		Timezone:  "Africa/Conakry",
	},
	"GUFA": {
		ICAO:      "GUFA",
		IATA:      "FIG",
		Name:      "Fria Airport",
		State:     "Boke",
		Country:   "GN",
		Elevation: 499,
		Latitude:  10.3506002426,
		Longitude: -13.5691995621,
		Timezone:  "Africa/Conakry",
	},
	"GUFH": {
		ICAO:      "GUFH",
		IATA:      "FAA",
		Name:      "Faranah Airport",
		State:     "Faranah",
		Country:   "GN",
		Elevation: 1476,
		Latitude:  10.0354995728,
		Longitude: -10.7698001862,
		Timezone:  "Africa/Conakry",
	},
	"GUGO": {
		ICAO:      "GUGO",
		Name:      "Gbenko Airport",
		City:      "Banankoro",
		State:     "Faranah",
		Country:   "GN",
		Elevation: 2133,
		Latitude:  9.2460298538,
		Longitude: -9.2951402664,
		Timezone:  "Africa/Conakry",
	},
	"GUKR": {
		ICAO:      "GUKR",
		Name:      "Kawass Airport",
		City:      "Kawass",
		State:     "Boke",
		Country:   "GN",
		Elevation: 36,
		Latitude:  10.6510000229,
		Longitude: -14.5335998535,
		Timezone:  "Africa/Conakry",
	},
	"GUKU": {
		ICAO:      "GUKU",
		IATA:      "KSI",
		Name:      "Kissidougou Airport",
		City:      "Kissidougou",
		State:     "Faranah",
		Country:   "GN",
		Elevation: 1808,
		Latitude:  9.1605596542,
		Longitude: -10.1244001389,
		Timezone:  "Africa/Conakry",
	},
	"GULB": {
		ICAO:      "GULB",
		IATA:      "LEK",
		Name:      "Labe Airport",
		State:     "Labe",
		Country:   "GN",
		Elevation: 3396,
		Latitude:  11.3261003494,
		Longitude: -12.2868003845,
		Timezone:  "Africa/Conakry",
	},
	"GUMA": {
		ICAO:      "GUMA",
		IATA:      "MCA",
		Name:      "Macenta Airport",
		City:      "Macenta",
		State:     "Nzerekore",
		Country:   "GN",
		Elevation: 1690,
		Latitude:  8.5329999924,
		Longitude: -9.4670000076,
		Timezone:  "Africa/Conakry",
	},
	"GUNZ": {
		ICAO:      "GUNZ",
		IATA:      "NZE",
		Name:      "Nzerekore Airport",
		City:      "Nzerekore",
		State:     "Nzerekore",
		Country:   "GN",
		Elevation: 1657,
		Latitude:  7.8060193062,
		Longitude: -8.7017974854,
		Timezone:  "Africa/Conakry",
	},
	"GUOK": {
		ICAO:      "GUOK",
		IATA:      "BKJ",
		Name:      "Boke Airport",
		City:      "Boke",
		State:     "Boke",
		Country:   "GN",
		Elevation: 164,
		Latitude:  10.9658002853,
		Longitude: -14.2811002731,
		Timezone:  "Africa/Conakry",
	},
	"GUSA": {
		ICAO:      "GUSA",
		Name:      "Sangaredi Airport",
		City:      "Sangaredi",
		State:     "Boke",
		Country:   "GN",
		Elevation: 686,
		Latitude:  11.1169996262,
		Longitude: -13.8330001831,
		Timezone:  "Africa/Conakry",
	},
	"GUSB": {
		ICAO:      "GUSB",
		IATA:      "SBI",
		Name:      "Sambailo Airport",
		City:      "Koundara",
		State:     "Boke",
		Country:   "GN",
		Elevation: 295,
		Latitude:  12.5726995468,
		Longitude: -13.358499527,
		Timezone:  "Africa/Conakry",
	},
	"GUSI": {
		ICAO:      "GUSI",
		IATA:      "GII",
		Name:      "Siguiri Airport",
		City:      "Siguiri",
		State:     "Kankan",
		Country:   "GN",
		Elevation: 1296,
		Latitude:  11.4329996109,
		Longitude: -9.1669998169,
		Timezone:  "Africa/Conakry",
	},
	"GUXN": {
		ICAO:      "GUXN",
		IATA:      "KNN",
		Name:      "Kankan Airport",
		City:      "Kankan",
		State:     "Kankan",
		Country:   "GN",
		Elevation: 1234,
		Latitude:  10.3940000534,
		Longitude: -9.3037996292,
		Timezone:  "Africa/Conakry",
	},
	"GVAC": {
		ICAO:      "GVAC",
		IATA:      "SID",
		Name:      "Amilcar Cabral International Airport",
		City:      "Espargos",
		State:     "Sal",
		Country:   "CV",
		Elevation: 177,
		Latitude:  16.741399765,
		Longitude: -22.9493999481,
		Timezone:  "Atlantic/Cape_Verde",
	},
	"GVAN": {
		ICAO:      "GVAN",
		IATA:      "NTO",
		Name:      "Agostinho Neto Airport",
		City:      "Ponta do Sol",
		State:     "Ribeira-Grande",
		Country:   "CV",
		Elevation: 32,
		Latitude:  17.2028007507,
		Longitude: -25.0905990601,
		Timezone:  "Atlantic/Cape_Verde",
	},
	"GVBA": {
		ICAO:      "GVBA",
		IATA:      "BVC",
		Name:      "Rabil Airport",
		City:      "Rabil",
		State:     "Boa-Vista",
		Country:   "CV",
		Elevation: 69,
		Latitude:  16.1364994049,
		Longitude: -22.8889007568,
		Timezone:  "Atlantic/Cape_Verde",
	},
	"GVBR": {
		ICAO:      "GVBR",
		IATA:      "BVR",
		Name:      "Esperadinha Airport",
		City:      "Brava Island",
		State:     "Brava",
		Country:   "CV",
		Elevation: 64,
		Latitude:  14.8643055556,
		Longitude: -24.746,
		Timezone:  "Atlantic/Cape_Verde",
	},
	"GVMA": {
		ICAO:      "GVMA",
		IATA:      "MMO",
		Name:      "Maio Airport",
		City:      "Vila do Maio",
		State:     "Maio",
		Country:   "CV",
		Elevation: 36,
		Latitude:  15.1559000015,
		Longitude: -23.2136993408,
		Timezone:  "Atlantic/Cape_Verde",
	},
	"GVMT": {
		ICAO:      "GVMT",
		IATA:      "MTI",
		Name:      "Mosteiros Airport",
		City:      "Vila do Mosteiros",
		State:     "Mosteiros",
		Country:   "CV",
		Elevation: 66,
		Latitude:  15.0450000763,
		Longitude: -24.3391990662,
		Timezone:  "Atlantic/Cape_Verde",
	},
	"GVNP": {
		ICAO:      "GVNP",
		IATA:      "RAI",
		Name:      "Praia International Airport",
		City:      "Praia",
		State:     "Praia",
		Country:   "CV",
		Elevation: 230,
		Latitude:  14.9245004654,
		Longitude: -23.4934997559,
		Timezone:  "Atlantic/Cape_Verde",
	},
	"GVSF": {
		ICAO:      "GVSF",
		IATA:      "SFL",
		Name:      "Sao Filipe Airport",
		City:      "Sao Filipe",
		State:     "Sao-Filipe",
		Country:   "CV",
		Elevation: 617,
		Latitude:  14.8850002289,
		Longitude: -24.4799995422,
		Timezone:  "Atlantic/Cape_Verde",
	},
	"GVSN": {
		ICAO:      "GVSN",
		IATA:      "SNE",
		Name:      "Preguica Airport",
		City:      "Preguica",
		State:     "Ribeira-Brava",
		Country:   "CV",
		Elevation: 669,
		Latitude:  16.5883998871,
		Longitude: -24.2847003937,
		Timezone:  "Atlantic/Cape_Verde",
	},
	"GVSV": {
		ICAO:      "GVSV",
		IATA:      "VXE",
		Name:      "Sao Pedro Airport",
		City:      "Sao Pedro",
		State:     "Sao-Vicente",
		Country:   "CV",
		Elevation: 66,
		Latitude:  16.8332004547,
		Longitude: -25.0552997589,
		Timezone:  "Atlantic/Cape_Verde",
	},
	"HAAB": {
		ICAO:      "HAAB",
		IATA:      "ADD",
		Name:      "Bole International Airport",
		City:      "Addis Ababa",
		State:     "Addis-Ababa",
		Country:   "ET",
		Elevation: 7630,
		Latitude:  8.9778900147,
		Longitude: 38.7993011475,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HAAD": {
		ICAO:      "HAAD",
		Name:      "Adaba Airport",
		City:      "Adaba",
		State:     "Oromiya",
		Country:   "ET",
		Elevation: 7896,
		Latitude:  7.0500001907,
		Longitude: 39.4000015259,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HAAL": {
		ICAO:      "HAAL",
		Name:      "Lideta Army Airport",
		City:      "Addis Ababa",
		Country:   "ET",
		Elevation: 7749,
		Latitude:  9.0036802292,
		Longitude: 38.7260017395,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HAAM": {
		ICAO:      "HAAM",
		IATA:      "AMH",
		Name:      "Arba Minch Airport",
		State:     "SNNPR",
		Country:   "ET",
		Elevation: 3901,
		Latitude:  6.0393900871,
		Longitude: 37.5904998779,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HAAX": {
		ICAO:      "HAAX",
		IATA:      "AXU",
		Name:      "Axum Airport",
		State:     "Tigray",
		Country:   "ET",
		Elevation: 6959,
		Latitude:  14.1468000412,
		Longitude: 38.7728004456,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HABC": {
		ICAO:      "HABC",
		IATA:      "BCO",
		Name:      "Baco Airport",
		City:      "Baco",
		Country:   "ET",
		Latitude:  5.7828698158,
		Longitude: 36.5620002747,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HABD": {
		ICAO:      "HABD",
		IATA:      "BJR",
		Name:      "Bahir Dar Airport",
		City:      "Bahir Dar",
		State:     "Amhara",
		Country:   "ET",
		Elevation: 5978,
		Latitude:  11.6080999374,
		Longitude: 37.3216018677,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HABE": {
		ICAO:      "HABE",
		IATA:      "BEI",
		Name:      "Beica Airport",
		City:      "Beica",
		Country:   "ET",
		Elevation: 5410,
		Latitude:  9.3863897324,
		Longitude: 34.521900177,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HADC": {
		ICAO:      "HADC",
		IATA:      "DSE",
		Name:      "Combolcha Airport",
		City:      "Dessie",
		Country:   "ET",
		Elevation: 6117,
		Latitude:  11.0825004578,
		Longitude: 39.7113990784,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HADD": {
		ICAO:      "HADD",
		IATA:      "DEM",
		Name:      "Dembidollo Airport",
		City:      "Dembidollo",
		Country:   "ET",
		Elevation: 5200,
		Latitude:  8.5539999008,
		Longitude: 34.858001709,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HADM": {
		ICAO:      "HADM",
		IATA:      "DBM",
		Name:      "Debra Marcos Airport",
		City:      "Debra Marcos",
		State:     "Amhara",
		Country:   "ET",
		Elevation: 8136,
		Latitude:  10.3500003815,
		Longitude: 37.716999054,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HADO": {
		ICAO:      "HADO",
		Name:      "Dodola Airport",
		City:      "Dodola",
		State:     "Oromiya",
		Country:   "ET",
		Elevation: 8234,
		Latitude:  7.0205597878,
		Longitude: 39.0511016846,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HADR": {
		ICAO:      "HADR",
		IATA:      "DIR",
		Name:      "Aba Tenna Dejazmach Yilma International Airport",
		City:      "Dire Dawa",
		State:     "Dire-Dawa",
		Country:   "ET",
		Elevation: 3827,
		Latitude:  9.6246995926,
		Longitude: 41.8541984558,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HADT": {
		ICAO:      "HADT",
		IATA:      "DBT",
		Name:      "Debre Tabor Airport",
		City:      "Debre Tabor",
		Country:   "ET",
		Elevation: 8490,
		Latitude:  11.9670000076,
		Longitude: 38,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HAFN": {
		ICAO:      "HAFN",
		IATA:      "FNH",
		Name:      "Fincha Airport",
		City:      "Fincha",
		Country:   "ET",
		Elevation: 7600,
		Latitude:  9.5830001831,
		Longitude: 37.3499984741,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HAGB": {
		ICAO:      "HAGB",
		IATA:      "GOB",
		Name:      "Robe Airport",
		City:      "Goba",
		State:     "Oromiya",
		Country:   "ET",
		Elevation: 7892,
		Latitude:  7.0170001984,
		Longitude: 40,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HAGH": {
		ICAO:      "HAGH",
		IATA:      "GNN",
		Name:      "Ghinnir Airport",
		City:      "Ghinnir",
		State:     "Oromiya",
		Country:   "ET",
		Elevation: 6499,
		Latitude:  7.1500000954,
		Longitude: 40.716999054,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HAGM": {
		ICAO:      "HAGM",
		IATA:      "GMB",
		Name:      "Gambella Airport",
		City:      "Gambela",
		Country:   "ET",
		Elevation: 1614,
		Latitude:  8.1287603378,
		Longitude: 34.5630989075,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HAGN": {
		ICAO:      "HAGN",
		IATA:      "GDQ",
		Name:      "Gonder Airport",
		City:      "Gondar",
		State:     "Amhara",
		Country:   "ET",
		Elevation: 6449,
		Latitude:  12.519900322,
		Longitude: 37.4339981079,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HAGO": {
		ICAO:      "HAGO",
		IATA:      "GDE",
		Name:      "Gode Airport",
		City:      "Gode",
		State:     "Somali",
		Country:   "ET",
		Elevation: 834,
		Latitude:  5.9351301193,
		Longitude: 43.5786018372,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HAGR": {
		ICAO:      "HAGR",
		IATA:      "GOR",
		Name:      "Gore Airport",
		City:      "Gore",
		Country:   "ET",
		Elevation: 6580,
		Latitude:  8.1614,
		Longitude: 35.5529,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HAHM": {
		ICAO:      "HAHM",
		IATA:      "QHR",
		Name:      "Harar Meda Airport",
		City:      "Debre Zeyit",
		Country:   "ET",
		Latitude:  8.7163,
		Longitude: 39.0059,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HAHU": {
		ICAO:      "HAHU",
		IATA:      "HUE",
		Name:      "Humera Airport",
		City:      "Humera",
		Country:   "ET",
		Elevation: 1930,
		Latitude:  14.25,
		Longitude: 36.5830001831,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HAJJ": {
		ICAO:      "HAJJ",
		IATA:      "JIJ",
		Name:      "Jijiga Airport",
		City:      "Jijiga",
		State:     "Somali",
		Country:   "ET",
		Elevation: 5413,
		Latitude:  9.330833,
		Longitude: 42.911111,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HAJM": {
		ICAO:      "HAJM",
		IATA:      "JIM",
		Name:      "Jimma Airport",
		City:      "Jimma",
		State:     "Oromiya",
		Country:   "ET",
		Elevation: 5500,
		Latitude:  7.6660900116,
		Longitude: 36.8166007996,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HAKD": {
		ICAO:      "HAKD",
		IATA:      "ABK",
		Name:      "Kabri Dehar Airport",
		City:      "Kabri Dehar",
		Country:   "ET",
		Elevation: 1800,
		Latitude:  6.734000206,
		Longitude: 44.2529983521,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HAKL": {
		ICAO:      "HAKL",
		IATA:      "LFO",
		Name:      "Kelafo East Airport",
		City:      "Kelafo",
		Country:   "ET",
		Elevation: 1730,
		Latitude:  5.6570000648,
		Longitude: 44.3499984741,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HALA": {
		ICAO:      "HALA",
		IATA:      "AWA",
		Name:      "Awassa Airport",
		City:      "Awassa",
		State:     "SNNPR",
		Country:   "ET",
		Elevation: 5450,
		Latitude:  7.0669999123,
		Longitude: 38.5,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HALL": {
		ICAO:      "HALL",
		IATA:      "LLI",
		Name:      "Lalibella Airport",
		City:      "Lalibela",
		State:     "Amhara",
		Country:   "ET",
		Elevation: 6506,
		Latitude:  11.9750003815,
		Longitude: 38.9799995422,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HAMJ": {
		ICAO:      "HAMJ",
		IATA:      "TUJ",
		Name:      "Tume Airport",
		City:      "Maji",
		State:     "SNNPR",
		Country:   "ET",
		Elevation: 2500,
		Latitude:  5.8330001831,
		Longitude: 35.533000946,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HAMK": {
		ICAO:      "HAMK",
		IATA:      "MQX",
		Name:      "Mekele Airport",
		State:     "Tigray",
		Country:   "ET",
		Elevation: 7396,
		Latitude:  13.4673995972,
		Longitude: 39.5335006714,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HAML": {
		ICAO:      "HAML",
		IATA:      "MZX",
		Name:      "Masslo Airport",
		City:      "Masslo",
		State:     "Oromiya",
		Country:   "ET",
		Elevation: 4180,
		Latitude:  6.4080600739,
		Longitude: 39.7238998413,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HAMM": {
		ICAO:      "HAMM",
		IATA:      "ETE",
		Name:      "Metema Airport",
		City:      "Metema",
		State:     "Amhara",
		Country:   "ET",
		Elevation: 2650,
		Latitude:  12.9329996109,
		Longitude: 36.1669998169,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HAMN": {
		ICAO:      "HAMN",
		IATA:      "NDM",
		Name:      "Mendi Airport",
		City:      "Mendi",
		Country:   "ET",
		Elevation: 5500,
		Latitude:  9.7670001984,
		Longitude: 35.0999984741,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HAMT": {
		ICAO:      "HAMT",
		IATA:      "MTF",
		Name:      "Mizan Teferi Airport",
		City:      "Mizan Teferi",
		Country:   "ET",
		Elevation: 4396,
		Latitude:  6.9571,
		Longitude: 35.5547,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HANJ": {
		ICAO:      "HANJ",
		IATA:      "NEJ",
		Name:      "Nejjo Airport",
		City:      "Nejjo",
		Country:   "ET",
		Elevation: 6150,
		Latitude:  9.5500001907,
		Longitude: 35.466999054,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HANK": {
		ICAO:      "HANK",
		IATA:      "NEK",
		Name:      "Nekemte Airport",
		City:      "Nekemte",
		Country:   "ET",
		Elevation: 6500,
		Latitude:  9.0500001907,
		Longitude: 36.5999984741,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HASC": {
		ICAO:      "HASC",
		IATA:      "SHC",
		Name:      "Shire Airport",
		City:      "Shire",
		State:     "Tigray",
		Country:   "ET",
		Elevation: 6207,
		Latitude:  14.079444,
		Longitude: 38.270833,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HASD": {
		ICAO:      "HASD",
		IATA:      "SXU",
		Name:      "Soddu Airport",
		City:      "Soddu",
		Country:   "ET",
		Elevation: 6400,
		Latitude:  6.8169999123,
		Longitude: 37.75,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HASH": {
		ICAO:      "HASH",
		Name:      "Sheik Hussein Airport",
		City:      "Sheik Hussein",
		Country:   "ET",
		Elevation: 6500,
		Latitude:  7.1500000954,
		Longitude: 40.5670013428,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HASM": {
		ICAO:      "HASM",
		IATA:      "SZE",
		Name:      "Semera Airport",
		City:      "Semera",
		State:     "Afar",
		Country:   "ET",
		Elevation: 1390,
		Latitude:  11.7875,
		Longitude: 40.991667,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HASO": {
		ICAO:      "HASO",
		IATA:      "ASO",
		Name:      "Asosa Airport",
		City:      "Asosa",
		State:     "Binshangul-Gumuz",
		Country:   "ET",
		Elevation: 5100,
		Latitude:  10.0185003281,
		Longitude: 34.5862998962,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HATP": {
		ICAO:      "HATP",
		IATA:      "TIE",
		Name:      "Tippi Airport",
		City:      "Tippi",
		Country:   "ET",
		Elevation: 1100,
		Latitude:  7.2024,
		Longitude: 35.415,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HAWC": {
		ICAO:      "HAWC",
		IATA:      "WAC",
		Name:      "Waca Airport",
		City:      "Waca",
		Country:   "ET",
		Elevation: 4200,
		Latitude:  7.1669998169,
		Longitude: 37.1669998169,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HBBA": {
		ICAO:      "HBBA",
		IATA:      "BJM",
		Name:      "Bujumbura International Airport",
		City:      "Bujumbura",
		State:     "Bujumbura-Mairie",
		Country:   "BI",
		Elevation: 2582,
		Latitude:  -3.3240199089,
		Longitude: 29.3185005188,
		Timezone:  "Africa/Bujumbura",
	},
	"HBBE": {
		ICAO:      "HBBE",
		IATA:      "GID",
		Name:      "Gitega Airport",
		City:      "Gitega",
		State:     "Gitega",
		Country:   "BI",
		Elevation: 5741,
		Latitude:  -3.4172093868,
		Longitude: 29.9113082886,
		Timezone:  "Africa/Bujumbura",
	},
	"HBBO": {
		ICAO:      "HBBO",
		IATA:      "KRE",
		Name:      "Kirundo Airport",
		City:      "Kirundo",
		State:     "Kirundo",
		Country:   "BI",
		Elevation: 4511,
		Latitude:  -2.544772,
		Longitude: 30.094575,
		Timezone:  "Africa/Bujumbura",
	},
	"HCMA": {
		ICAO:      "HCMA",
		IATA:      "ALU",
		Name:      "Alula Airport",
		City:      "Alula",
		State:     "Bari",
		Country:   "SO",
		Elevation: 6,
		Latitude:  11.9582,
		Longitude: 50.748,
		Timezone:  "Africa/Mogadishu",
	},
	"HCMB": {
		ICAO:      "HCMB",
		IATA:      "BIB",
		Name:      "Baidoa Airport",
		City:      "Baidoa",
		State:     "Bay",
		Country:   "SO",
		Elevation: 1820,
		Latitude:  3.1022200584,
		Longitude: 43.6286010742,
		Timezone:  "Africa/Mogadishu",
	},
	"HCMC": {
		ICAO:      "HCMC",
		IATA:      "CXN",
		Name:      "Candala Airport",
		City:      "Candala",
		State:     "Bari",
		Country:   "SO",
		Elevation: 9,
		Latitude:  11.5,
		Longitude: 49.9169998169,
		Timezone:  "Africa/Mogadishu",
	},
	"HCMD": {
		ICAO:      "HCMD",
		IATA:      "BSY",
		Name:      "Bardera Airport",
		State:     "Gedo",
		Country:   "SO",
		Elevation: 4200,
		Latitude:  2.336111,
		Longitude: 42.307778,
		Timezone:  "Africa/Mogadishu",
	},
	"HCME": {
		ICAO:      "HCME",
		IATA:      "HCM",
		Name:      "Eil Airport",
		City:      "Eil",
		State:     "Nugaal",
		Country:   "SO",
		Elevation: 812,
		Latitude:  7.9169998169,
		Longitude: 49.7999992371,
		Timezone:  "Africa/Mogadishu",
	},
	"HCMF": {
		ICAO:      "HCMF",
		IATA:      "BSA",
		Name:      "Bosaso Airport",
		City:      "Bosaso",
		State:     "Bari",
		Country:   "SO",
		Elevation: 3,
		Latitude:  11.2753000259,
		Longitude: 49.1493988037,
		Timezone:  "Africa/Mogadishu",
	},
	"HCMG": {
		ICAO:      "HCMG",
		IATA:      "GSR",
		Name:      "Gardo Airport",
		City:      "Gardo",
		State:     "Bari",
		Country:   "SO",
		Elevation: 2632,
		Latitude:  9.5170001984,
		Longitude: 49.0830001831,
		Timezone:  "Africa/Mogadishu",
	},
	"HCMH": {
		ICAO:      "HCMH",
		IATA:      "HGA",
		Name:      "Egal International Airport",
		City:      "Hargeisa",
		State:     "Woqooyi-Galbeed",
		Country:   "SO",
		Elevation: 4423,
		Latitude:  9.5181703568,
		Longitude: 44.0887985229,
		Timezone:  "Africa/Mogadishu",
	},
	"HCMI": {
		ICAO:      "HCMI",
		IATA:      "BBO",
		Name:      "Berbera Airport",
		City:      "Berbera",
		State:     "Woqooyi-Galbeed",
		Country:   "SO",
		Elevation: 30,
		Latitude:  10.3892002106,
		Longitude: 44.9411010742,
		Timezone:  "Africa/Mogadishu",
	},
	"HCMK": {
		ICAO:      "HCMK",
		IATA:      "KMU",
		Name:      "Kisimayu Airport",
		State:     "Lower-Juba",
		Country:   "SO",
		Elevation: 49,
		Latitude:  -0.3773530126,
		Longitude: 42.4592018127,
		Timezone:  "Africa/Mogadishu",
	},
	"HCMM": {
		ICAO:      "HCMM",
		IATA:      "MGQ",
		Name:      "Aden Adde International Airport",
		City:      "Mogadishu",
		State:     "Banaadir",
		Country:   "SO",
		Elevation: 29,
		Latitude:  2.0144400597,
		Longitude: 45.3046989441,
		Timezone:  "Africa/Mogadishu",
	},
	"HCMN": {
		ICAO:      "HCMN",
		Name:      "Belet Uen Airport",
		City:      "Belet Uen",
		State:     "Hiiraan",
		Country:   "SO",
		Elevation: 559,
		Latitude:  4.6999998093,
		Longitude: 45.216999054,
		Timezone:  "Africa/Mogadishu",
	},
	"HCMO": {
		ICAO:      "HCMO",
		IATA:      "CMO",
		Name:      "Obbia Airport",
		City:      "Obbia",
		State:     "Mudug",
		Country:   "SO",
		Elevation: 65,
		Latitude:  5.3666667938,
		Longitude: 48.5166664124,
		Timezone:  "Africa/Mogadishu",
	},
	"HCMR": {
		ICAO:      "HCMR",
		IATA:      "GLK",
		Name:      "Galcaio Airport",
		City:      "Galcaio",
		State:     "Mudug",
		Country:   "SO",
		Elevation: 975,
		Latitude:  6.7808299065,
		Longitude: 47.45470047,
		Timezone:  "Africa/Mogadishu",
	},
	"HCMS": {
		ICAO:      "HCMS",
		IATA:      "CMS",
		Name:      "Scusciuban Airport",
		City:      "Scusciuban",
		State:     "Bari",
		Country:   "SO",
		Elevation: 1121,
		Latitude:  10.3000001907,
		Longitude: 50.233001709,
		Timezone:  "Africa/Mogadishu",
	},
	"HCMU": {
		ICAO:      "HCMU",
		IATA:      "ERA",
		Name:      "Erigavo Airport",
		City:      "Erigavo",
		State:     "Sanaag",
		Country:   "SO",
		Elevation: 5720,
		Latitude:  10.642050549,
		Longitude: 47.3879814148,
		Timezone:  "Africa/Mogadishu",
	},
	"HCMV": {
		ICAO:      "HCMV",
		IATA:      "BUO",
		Name:      "Burao Airport",
		City:      "Burao",
		State:     "Togdheer",
		Country:   "SO",
		Elevation: 3400,
		Latitude:  9.5275,
		Longitude: 45.5549,
		Timezone:  "Africa/Mogadishu",
	},
	"HDAG": {
		ICAO:      "HDAG",
		Name:      "Assa-Gueyla Airport",
		City:      "Assa-Gueyla",
		State:     "Tadjourah",
		Country:   "DJ",
		Latitude:  12.1999998093,
		Longitude: 42.6333007813,
		Timezone:  "Africa/Djibouti",
	},
	"HDAM": {
		ICAO:      "HDAM",
		IATA:      "JIB",
		Name:      "Djibouti-Ambouli Airport",
		City:      "Djibouti City",
		State:     "Djibouti",
		Country:   "DJ",
		Elevation: 49,
		Latitude:  11.5473003387,
		Longitude: 43.1595001221,
		Timezone:  "Africa/Djibouti",
	},
	"HDAS": {
		ICAO:      "HDAS",
		IATA:      "AII",
		Name:      "Ali-Sabieh Airport",
		City:      "Ali-Sabieh",
		State:     "Ali-Sabieh",
		Country:   "DJ",
		Elevation: 2320,
		Latitude:  11.1468888889,
		Longitude: 42.72,
		Timezone:  "Africa/Djibouti",
	},
	"HDCH": {
		ICAO:      "HDCH",
		Name:      "Chabelley Airport",
		City:      "Chabelley",
		State:     "Ali-Sabieh",
		Country:   "DJ",
		Elevation: 279,
		Latitude:  11.5167999268,
		Longitude: 43.0614013672,
		Timezone:  "Africa/Djibouti",
	},
	"HDDK": {
		ICAO:      "HDDK",
		Name:      "Dikhil Airport",
		City:      "Dikhil",
		Country:   "DJ",
		Latitude:  11.1000003815,
		Longitude: 42.3499984741,
		Timezone:  "Africa/Djibouti",
	},
	"HDHE": {
		ICAO:      "HDHE",
		Name:      "Herkale Airport",
		City:      "Herkale",
		State:     "Obock",
		Country:   "DJ",
		Latitude:  12.4333333969,
		Longitude: 43.2999992371,
		Timezone:  "Africa/Djibouti",
	},
	"HDMO": {
		ICAO:      "HDMO",
		IATA:      "MHI",
		Name:      "Moucha Airport",
		City:      "Moucha Island",
		State:     "Djibouti",
		Country:   "DJ",
		Latitude:  11.7166671753,
		Longitude: 43.2000007629,
		Timezone:  "Africa/Djibouti",
	},
	"HDOB": {
		ICAO:      "HDOB",
		IATA:      "OBC",
		Name:      "Obock Airport",
		City:      "Obock",
		State:     "Obock",
		Country:   "DJ",
		Elevation: 69,
		Latitude:  11.9670000076,
		Longitude: 43.266998291,
		Timezone:  "Africa/Djibouti",
	},
	"HDTJ": {
		ICAO:      "HDTJ",
		IATA:      "TDJ",
		Name:      "Tadjoura Airport",
		City:      "Tadjoura",
		State:     "Tadjourah",
		Country:   "DJ",
		Elevation: 246,
		Latitude:  11.7829999924,
		Longitude: 42.9169998169,
		Timezone:  "Africa/Djibouti",
	},
	"HE15": {
		ICAO:      "HE15",
		Name:      "Hulwan",
		State:     "Cairo",
		Country:   "EG",
		Latitude:  29.8225002289,
		Longitude: 31.3309001923,
		Timezone:  "Africa/Cairo",
	},
	"HE16": {
		ICAO:      "HE16",
		Name:      "Birma",
		State:     "Gharbia",
		Country:   "EG",
		Latitude:  30.83659935,
		Longitude: 30.936000824,
		Timezone:  "Africa/Cairo",
	},
	"HE24": {
		ICAO:      "HE24",
		IATA:      "SEW",
		Name:      "Siwa Oasis North Airport",
		City:      "Siwa",
		State:     "Matruh",
		Country:   "EG",
		Elevation: 330,
		Latitude:  29.345500946,
		Longitude: 25.5067005157,
		Timezone:  "Africa/Cairo",
	},
	"HE26": {
		ICAO:      "HE26",
		Name:      "Wadi Abu Shihat",
		State:     "Red-Sea",
		Country:   "EG",
		Latitude:  26.5572185516,
		Longitude: 33.1219291687,
		Timezone:  "Africa/Cairo",
	},
	"HE45": {
		ICAO:      "HE45",
		Name:      "Ras Sudr",
		State:     "South-Sinai",
		Country:   "EG",
		Latitude:  29.6030006409,
		Longitude: 32.6896018982,
		Timezone:  "Africa/Cairo",
	},
	"HEAL": {
		ICAO:      "HEAL",
		IATA:      "DBB",
		Name:      "El Alamein International Airport",
		City:      "El Alamein",
		State:     "Matruh",
		Country:   "EG",
		Elevation: 143,
		Latitude:  30.9244995117,
		Longitude: 28.4613990784,
		Timezone:  "Africa/Cairo",
	},
	"HEAR": {
		ICAO:      "HEAR",
		IATA:      "AAC",
		Name:      "El Arish International Airport",
		City:      "El Arish",
		State:     "North-Sinai",
		Country:   "EG",
		Elevation: 121,
		Latitude:  31.073299408,
		Longitude: 33.8358001709,
		Timezone:  "Africa/Cairo",
	},
	"HEAT": {
		ICAO:      "HEAT",
		IATA:      "ATZ",
		Name:      "Assiut International Airport",
		City:      "Assiut",
		State:     "Asyut",
		Country:   "EG",
		Elevation: 772,
		Latitude:  27.0464992523,
		Longitude: 31.0119991302,
		Timezone:  "Africa/Cairo",
	},
	"HEAX": {
		ICAO:      "HEAX",
		IATA:      "ALY",
		Name:      "El Nouzha Airport",
		City:      "Alexandria",
		Country:   "EG",
		Elevation: -6,
		Latitude:  31.1839008331,
		Longitude: 29.9489002228,
		Timezone:  "Africa/Cairo",
	},
	"HEAZ": {
		ICAO:      "HEAZ",
		Name:      "Almaza Air Force Base",
		City:      "Cairo",
		State:     "Cairo",
		Country:   "EG",
		Elevation: 300,
		Latitude:  30.0918006897,
		Longitude: 31.3600006104,
		Timezone:  "Africa/Cairo",
	},
	"HEBA": {
		ICAO:      "HEBA",
		IATA:      "HBE",
		Name:      "Borg El Arab International Airport",
		City:      "Alexandria",
		State:     "Alexandria",
		Country:   "EG",
		Elevation: 177,
		Latitude:  30.9176998138,
		Longitude: 29.6963996887,
		Timezone:  "Africa/Cairo",
	},
	"HEBL": {
		ICAO:      "HEBL",
		IATA:      "ABS",
		Name:      "Abu Simbel Airport",
		City:      "Abu Simbel",
		State:     "Aswan",
		Country:   "EG",
		Elevation: 616,
		Latitude:  22.3759994507,
		Longitude: 31.611700058,
		Timezone:  "Africa/Cairo",
	},
	"HEBS": {
		ICAO:      "HEBS",
		Name:      "Beni Suef Air Base",
		City:      "Beni Suef",
		State:     "Faiyum",
		Country:   "EG",
		Elevation: 108,
		Latitude:  29.2082996368,
		Longitude: 31.0165996552,
		Timezone:  "Africa/Cairo",
	},
	"HECA": {
		ICAO:      "HECA",
		IATA:      "CAI",
		Name:      "Cairo International Airport",
		City:      "Cairo",
		State:     "Cairo",
		Country:   "EG",
		Elevation: 382,
		Latitude:  30.1219005585,
		Longitude: 31.4055995941,
		Timezone:  "Africa/Cairo",
	},
	"HECW": {
		ICAO:      "HECW",
		IATA:      "CWE",
		Name:      "Cairo West Airport",
		City:      "El Cairo",
		State:     "Giza",
		Country:   "EG",
		Elevation: 550,
		Latitude:  30.116399765,
		Longitude: 30.9153995514,
		Timezone:  "Africa/Cairo",
	},
	"HEDK": {
		ICAO:      "HEDK",
		IATA:      "DAK",
		Name:      "Dakhla Airport",
		State:     "New-Valley",
		Country:   "EG",
		Elevation: 613,
		Latitude:  25.4116001129,
		Longitude: 29.0030994415,
		Timezone:  "Africa/Cairo",
	},
	"HEEM": {
		ICAO:      "HEEM",
		Name:      "Giza Embaba Airport",
		City:      "Cairo",
		State:     "Giza",
		Country:   "EG",
		Elevation: 59,
		Latitude:  30.0746994019,
		Longitude: 31.191400528,
		Timezone:  "Africa/Cairo",
	},
	"HEGN": {
		ICAO:      "HEGN",
		IATA:      "HRG",
		Name:      "Hurghada International Airport",
		City:      "Hurghada",
		State:     "Red-Sea",
		Country:   "EG",
		Elevation: 52,
		Latitude:  27.1783008575,
		Longitude: 33.7994003296,
		Timezone:  "Africa/Cairo",
	},
	"HEGO": {
		ICAO:      "HEGO",
		Name:      "El Gouna Airport",
		City:      "El Gouna",
		State:     "Red-Sea",
		Country:   "EG",
		Elevation: 56,
		Latitude:  27.3668994904,
		Longitude: 33.6682014465,
		Timezone:  "Africa/Cairo",
	},
	"HEGR": {
		ICAO:      "HEGR",
		IATA:      "EGH",
		Name:      "El Gora Airport",
		State:     "North-Sinai",
		Country:   "EG",
		Elevation: 324,
		Latitude:  31.0690002441,
		Longitude: 34.1291999817,
		Timezone:  "Africa/Cairo",
	},
	"HEGS": {
		ICAO:      "HEGS",
		Name:      "Jiyanklis Air Base",
		City:      "Jiyanklis",
		State:     "Beheira",
		Country:   "EG",
		Elevation: 49,
		Latitude:  30.8197994232,
		Longitude: 30.1912002563,
		Timezone:  "Africa/Cairo",
	},
	"HEKG": {
		ICAO:      "HEKG",
		IATA:      "UVL",
		Name:      "El Kharga Airport",
		State:     "New-Valley",
		Country:   "EG",
		Elevation: 192,
		Latitude:  25.4736003876,
		Longitude: 30.5907001495,
		Timezone:  "Africa/Cairo",
	},
	"HELX": {
		ICAO:      "HELX",
		IATA:      "LXR",
		Name:      "Luxor International Airport",
		City:      "Luxor",
		State:     "Qena",
		Country:   "EG",
		Elevation: 294,
		Latitude:  25.670999527,
		Longitude: 32.7066001892,
		Timezone:  "Africa/Cairo",
	},
	"HEMA": {
		ICAO:      "HEMA",
		IATA:      "RMF",
		Name:      "Marsa Alam International Airport",
		City:      "Marsa Alam",
		State:     "Red-Sea",
		Country:   "EG",
		Elevation: 251,
		Latitude:  25.557100296,
		Longitude: 34.5836982727,
		Timezone:  "Africa/Cairo",
	},
	"HEMK": {
		ICAO:      "HEMK",
		IATA:      "HMB",
		Name:      "Sohag International Airport",
		City:      "Sohag",
		State:     "New-Valley",
		Country:   "EG",
		Elevation: 310,
		Latitude:  26.3427777778,
		Longitude: 31.7427777778,
		Timezone:  "Africa/Cairo",
	},
	"HEMM": {
		ICAO:      "HEMM",
		IATA:      "MUH",
		Name:      "Mersa Matruh Airport",
		City:      "Mersa Matruh",
		State:     "Matruh",
		Country:   "EG",
		Elevation: 94,
		Latitude:  31.3253993988,
		Longitude: 27.2217006683,
		Timezone:  "Africa/Cairo",
	},
	"HEOC": {
		ICAO:      "HEOC",
		Name:      "October Airport",
		State:     "Faiyum",
		Country:   "EG",
		Elevation: 807,
		Latitude:  29.8120994568,
		Longitude: 30.8234004974,
		Timezone:  "Africa/Cairo",
	},
	"HEOW": {
		ICAO:      "HEOW",
		IATA:      "GSQ",
		Name:      "Shark El Oweinat International Airport",
		State:     "New-Valley",
		Country:   "EG",
		Elevation: 859,
		Latitude:  22.5856990814,
		Longitude: 28.7166004181,
		Timezone:  "Africa/Cairo",
	},
	"HEPS": {
		ICAO:      "HEPS",
		IATA:      "PSD",
		Name:      "Port Said Airport",
		City:      "Port Said",
		State:     "Port-Said",
		Country:   "EG",
		Elevation: 8,
		Latitude:  31.2793998718,
		Longitude: 32.2400016785,
		Timezone:  "Africa/Cairo",
	},
	"HESC": {
		ICAO:      "HESC",
		IATA:      "SKV",
		Name:      "St Catherine International Airport",
		State:     "South-Sinai",
		Country:   "EG",
		Elevation: 4368,
		Latitude:  28.685300827,
		Longitude: 34.0625,
		Timezone:  "Africa/Cairo",
	},
	"HESH": {
		ICAO:      "HESH",
		IATA:      "SSH",
		Name:      "Sharm El Sheikh International Airport",
		City:      "Sharm el-Sheikh",
		State:     "South-Sinai",
		Country:   "EG",
		Elevation: 143,
		Latitude:  27.9773006439,
		Longitude: 34.3950004578,
		Timezone:  "Africa/Cairo",
	},
	"HESN": {
		ICAO:      "HESN",
		IATA:      "ASW",
		Name:      "Aswan International Airport",
		City:      "Aswan",
		State:     "Aswan",
		Country:   "EG",
		Elevation: 662,
		Latitude:  23.9643993378,
		Longitude: 32.8199996948,
		Timezone:  "Africa/Cairo",
	},
	"HESX": {
		ICAO:      "HESX",
		IATA:      "SPX",
		Name:      "Sphinx International Airport",
		City:      "Giza",
		State:     "Giza ",
		Country:   "EG",
		Elevation: 553,
		Latitude:  30.1147222,
		Longitude: 30.8933333,
		Timezone:  "Africa/Cairo",
	},
	"HETB": {
		ICAO:      "HETB",
		IATA:      "TCP",
		Name:      "Taba International Airport",
		City:      "Taba",
		State:     "South-Sinai",
		Country:   "EG",
		Elevation: 2415,
		Latitude:  29.5877990723,
		Longitude: 34.7780990601,
		Timezone:  "Africa/Cairo",
	},
	"HETR": {
		ICAO:      "HETR",
		IATA:      "ELT",
		Name:      "El Tor Airport",
		State:     "South-Sinai",
		Country:   "EG",
		Elevation: 115,
		Latitude:  28.2089996338,
		Longitude: 33.6455001831,
		Timezone:  "Africa/Cairo",
	},
	"HHAS": {
		ICAO:      "HHAS",
		IATA:      "ASM",
		Name:      "Asmara International Airport",
		City:      "Asmara",
		State:     "Maekel",
		Country:   "ER",
		Elevation: 7661,
		Latitude:  15.2918996811,
		Longitude: 38.9107017517,
		Timezone:  "Africa/Asmara",
	},
	"HHMS": {
		ICAO:      "HHMS",
		IATA:      "MSW",
		Name:      "Massawa International Airport",
		City:      "Massawa",
		State:     "Northern-Red-Sea",
		Country:   "ER",
		Elevation: 194,
		Latitude:  15.6700000763,
		Longitude: 39.370098114,
		Timezone:  "Africa/Asmara",
	},
	"HHSB": {
		ICAO:      "HHSB",
		IATA:      "ASA",
		Name:      "Assab International Airport",
		City:      "Asab",
		Country:   "ER",
		Elevation: 46,
		Latitude:  13.0718002319,
		Longitude: 42.6450004578,
		Timezone:  "Africa/Asmara",
	},
	"HHTS": {
		ICAO:      "HHTS",
		IATA:      "TES",
		Name:      "Tessenei Airport",
		City:      "Tessenei",
		Country:   "ER",
		Elevation: 2018,
		Latitude:  15.1169996262,
		Longitude: 36.6829986572,
		Timezone:  "Africa/Asmara",
	},
	"HI01": {
		ICAO:      "HI01",
		Name:      "Princeville Airport",
		City:      "Hanalei",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 344,
		Latitude:  22.2091999054,
		Longitude: -159.4459991455,
		Timezone:  "Pacific/Honolulu",
	},
	"HI02": {
		ICAO:      "HI02",
		Name:      "Peleau Airport",
		City:      "Hakalau",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 1088,
		Latitude:  19.9022006989,
		Longitude: -155.1660003662,
		Timezone:  "Pacific/Honolulu",
	},
	"HI03": {
		ICAO:      "HI03",
		Name:      "Hanamaulu Airstrip",
		City:      "Hanamaulu",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 404,
		Latitude:  22.0412998199,
		Longitude: -159.3899993896,
		Timezone:  "Pacific/Honolulu",
	},
	"HI05": {
		ICAO:      "HI05",
		Name:      "Honokaa Airstrip",
		City:      "Honokaa",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 1440,
		Latitude:  20.0835990906,
		Longitude: -155.5019989014,
		Timezone:  "Pacific/Honolulu",
	},
	"HI13": {
		ICAO:      "HI13",
		Name:      "Puu Waa Waa Ranch Airport",
		City:      "Kailua/Kona",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 2250,
		Latitude:  19.7886009216,
		Longitude: -155.8500061035,
		Timezone:  "Pacific/Honolulu",
	},
	"HI23": {
		ICAO:      "HI23",
		Name:      "Mountain View Airstrip",
		City:      "Mountain View",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 1500,
		Latitude:  19.5475006104,
		Longitude: -155.108001709,
		Timezone:  "Pacific/Honolulu",
	},
	"HI25": {
		ICAO:      "HI25",
		Name:      "Kaalaiki Airstrip",
		City:      "Naalehu",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 1964,
		Latitude:  19.122800827,
		Longitude: -155.5780029297,
		Timezone:  "Pacific/Honolulu",
	},
	"HI27": {
		ICAO:      "HI27",
		Name:      "Upper Paauilo Airstrip",
		City:      "Paauilo",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 1520,
		Latitude:  20.0282993317,
		Longitude: -155.3880004883,
		Timezone:  "Pacific/Honolulu",
	},
	"HI28": {
		ICAO:      "HI28",
		Name:      "Pahala Airstrip",
		City:      "Pahala",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 1195,
		Latitude:  19.2161998749,
		Longitude: -155.4689941406,
		Timezone:  "Pacific/Honolulu",
	},
	"HI29": {
		ICAO:      "HI29",
		Name:      "Upper Paauau Airport",
		City:      "Pahala",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 2600,
		Latitude:  19.2275009155,
		Longitude: -155.5090026855,
		Timezone:  "Pacific/Honolulu",
	},
	"HI31": {
		ICAO:      "HI31",
		Name:      "Mauna Kea-Honolii Airport",
		City:      "Papaikou",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 1400,
		Latitude:  19.7639007568,
		Longitude: -155.1380004883,
		Timezone:  "Pacific/Honolulu",
	},
	"HI32": {
		ICAO:      "HI32",
		Name:      "Pepeekeo Airstrip",
		City:      "Pepeekeo",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 675,
		Latitude:  19.8446998596,
		Longitude: -155.1130065918,
		Timezone:  "Pacific/Honolulu",
	},
	"HI33": {
		ICAO:      "HI33",
		Name:      "Haiku Airstrip",
		City:      "Puhi",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 385,
		Latitude:  21.9423999786,
		Longitude: -159.4400024414,
		Timezone:  "Pacific/Honolulu",
	},
	"HI46": {
		ICAO:      "HI46",
		Name:      "Hi 23 Airstrip",
		City:      "Puhi",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 480,
		Latitude:  21.9657993317,
		Longitude: -159.4270019531,
		Timezone:  "Pacific/Honolulu",
	},
	"HI49": {
		ICAO:      "HI49",
		Name:      "Panda Airport",
		City:      "Kaunakakai",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 250,
		Latitude:  21.1478996277,
		Longitude: -157.2590026855,
		Timezone:  "Pacific/Honolulu",
	},
	"HKAM": {
		ICAO:      "HKAM",
		IATA:      "ASV",
		Name:      "Amboseli Airport",
		City:      "Amboseli National Park",
		State:     "Kajiado",
		Country:   "KE",
		Elevation: 3755,
		Latitude:  -2.6450500488,
		Longitude: 37.2531013489,
		Timezone:  "Africa/Nairobi",
	},
	"HKBR": {
		ICAO:      "HKBR",
		Name:      "Bura East Airport",
		City:      "Bura",
		State:     "Tana-River",
		Country:   "KE",
		Elevation: 345,
		Latitude:  -1.1000000238,
		Longitude: 39.9500007629,
		Timezone:  "Africa/Nairobi",
	},
	"HKEL": {
		ICAO:      "HKEL",
		IATA:      "EDL",
		Name:      "Eldoret International Airport",
		City:      "Eldoret",
		State:     "Uasin-Gishu",
		Country:   "KE",
		Elevation: 6941,
		Latitude:  0.4044579864,
		Longitude: 35.238899231,
		Timezone:  "Africa/Nairobi",
	},
	"HKEM": {
		ICAO:      "HKEM",
		Name:      "Embu Airport",
		City:      "Embu",
		State:     "Embu",
		Country:   "KE",
		Elevation: 4150,
		Latitude:  -0.5669999719,
		Longitude: 37.483001709,
		Timezone:  "Africa/Nairobi",
	},
	"HKES": {
		ICAO:      "HKES",
		IATA:      "EYS",
		Name:      "Eliye Springs Airport",
		City:      "Eliye Springs",
		Country:   "KE",
		Elevation: 1395,
		Latitude:  3.2166700363,
		Longitude: 35.9667015076,
		Timezone:  "Africa/Nairobi",
	},
	"HKEW": {
		ICAO:      "HKEW",
		Name:      "El Wak Airport",
		City:      "El Wak",
		State:     "Mandera",
		Country:   "KE",
		Elevation: 1295,
		Latitude:  2.7330000401,
		Longitude: 40.9329986572,
		Timezone:  "Africa/Nairobi",
	},
	"HKFG": {
		ICAO:      "HKFG",
		IATA:      "KLK",
		Name:      "Kalokol Airport",
		City:      "Kalokol",
		State:     "Turkana",
		Country:   "KE",
		Elevation: 1245,
		Latitude:  3.4830000401,
		Longitude: 35.8330001831,
		Timezone:  "Africa/Nairobi",
	},
	"HKGA": {
		ICAO:      "HKGA",
		IATA:      "GAS",
		Name:      "Garissa Airport",
		City:      "Garissa",
		State:     "Garissa",
		Country:   "KE",
		Elevation: 475,
		Latitude:  -0.4635080099,
		Longitude: 39.6483001709,
		Timezone:  "Africa/Nairobi",
	},
	"HKGT": {
		ICAO:      "HKGT",
		Name:      "Garba Tula Airport",
		City:      "Garba Tula",
		State:     "Isiolo",
		Country:   "KE",
		Elevation: 2000,
		Latitude:  0.5329999924,
		Longitude: 38.516998291,
		Timezone:  "Africa/Nairobi",
	},
	"HKHB": {
		ICAO:      "HKHB",
		Name:      "Homa Bay Airport",
		City:      "Homa Bay",
		State:     "Homa-Bay",
		Country:   "KE",
		Elevation: 4280,
		Latitude:  -0.6000000238,
		Longitude: 34.466999054,
		Timezone:  "Africa/Nairobi",
	},
	"HKHO": {
		ICAO:      "HKHO",
		IATA:      "HOA",
		Name:      "Hola Airport",
		City:      "Hola",
		State:     "Tana-River",
		Country:   "KE",
		Elevation: 195,
		Latitude:  -1.5219999552,
		Longitude: 40.0040016174,
		Timezone:  "Africa/Nairobi",
	},
	"HKIS": {
		ICAO:      "HKIS",
		Name:      "Isiolo Airport",
		City:      "Isiolo",
		State:     "Isiolo",
		Country:   "KE",
		Elevation: 3495,
		Latitude:  0.3381709456,
		Longitude: 37.5916938782,
		Timezone:  "Africa/Nairobi",
	},
	"HKJK": {
		ICAO:      "HKJK",
		IATA:      "NBO",
		Name:      "Jomo Kenyatta International Airport",
		City:      "Nairobi",
		State:     "Nairobi-Area",
		Country:   "KE",
		Elevation: 5330,
		Latitude:  -1.319239974,
		Longitude: 36.9277992249,
		Timezone:  "Africa/Nairobi",
	},
	"HKKB": {
		ICAO:      "HKKB",
		Name:      "Kiambere Airport",
		City:      "Kiambere",
		State:     "Kitui",
		Country:   "KE",
		Elevation: 2450,
		Latitude:  -0.6330000162,
		Longitude: 37.8829994202,
		Timezone:  "Africa/Nairobi",
	},
	"HKKE": {
		ICAO:      "HKKE",
		IATA:      "KEU",
		Name:      "Keekorok Airport",
		City:      "Keekorok",
		State:     "Narok",
		Country:   "KE",
		Elevation: 5800,
		Latitude:  -1.5829999447,
		Longitude: 35.25,
		Timezone:  "Africa/Nairobi",
	},
	"HKKG": {
		ICAO:      "HKKG",
		IATA:      "GGM",
		Name:      "Kakamega Airport",
		City:      "Kakamega",
		State:     "Kakamega",
		Country:   "KE",
		Elevation: 5020,
		Latitude:  0.2713420093,
		Longitude: 34.7873001099,
		Timezone:  "Africa/Nairobi",
	},
	"HKKI": {
		ICAO:      "HKKI",
		IATA:      "KIS",
		Name:      "Kisumu Airport",
		City:      "Kisumu",
		State:     "Kisumu",
		Country:   "KE",
		Elevation: 3734,
		Latitude:  -0.086139001,
		Longitude: 34.7289009094,
		Timezone:  "Africa/Nairobi",
	},
	"HKKL": {
		ICAO:      "HKKL",
		IATA:      "ILU",
		Name:      "Kilaguni Airport",
		City:      "Kilaguni",
		State:     "Taita-Taveta",
		Country:   "KE",
		Elevation: 2750,
		Latitude:  -2.9106099606,
		Longitude: 38.0652008057,
		Timezone:  "Africa/Nairobi",
	},
	"HKKR": {
		ICAO:      "HKKR",
		IATA:      "KEY",
		Name:      "Kericho Airport",
		City:      "Kericho",
		State:     "Kericho",
		Country:   "KE",
		Elevation: 6562,
		Latitude:  -0.4169999957,
		Longitude: 35.25,
		Timezone:  "Africa/Nairobi",
	},
	"HKKS": {
		ICAO:      "HKKS",
		Name:      "Kisii Airport",
		City:      "Kisii",
		State:     "Kisii",
		Country:   "KE",
		Elevation: 4905,
		Latitude:  -0.6669999957,
		Longitude: 34.7000007629,
		Timezone:  "Africa/Nairobi",
	},
	"HKKT": {
		ICAO:      "HKKT",
		IATA:      "KTL",
		Name:      "Kitale Airport",
		City:      "Kitale",
		State:     "Trans-Nzoia",
		Country:   "KE",
		Elevation: 6070,
		Latitude:  0.971988976,
		Longitude: 34.9585990906,
		Timezone:  "Africa/Nairobi",
	},
	"HKKU": {
		ICAO:      "HKKU",
		Name:      "Kitui Airport",
		City:      "Kitui",
		State:     "Kitui",
		Country:   "KE",
		Elevation: 3790,
		Latitude:  -1.372789979,
		Longitude: 37.978099823,
		Timezone:  "Africa/Nairobi",
	},
	"HKLG": {
		ICAO:      "HKLG",
		Name:      "Lokitaung Airport",
		City:      "Lokitaung",
		State:     "Turkana",
		Country:   "KE",
		Elevation: 1805,
		Latitude:  4.3789000511,
		Longitude: 35.6589012146,
		Timezone:  "Africa/Nairobi",
	},
	"HKLK": {
		ICAO:      "HKLK",
		IATA:      "LKG",
		Name:      "Lokichoggio Airport",
		City:      "Lokichoggio",
		State:     "Turkana",
		Country:   "KE",
		Elevation: 2074,
		Latitude:  4.2041201591,
		Longitude: 34.3482017517,
		Timezone:  "Africa/Nairobi",
	},
	"HKLO": {
		ICAO:      "HKLO",
		IATA:      "LOK",
		Name:      "Lodwar Airport",
		City:      "Lodwar",
		State:     "Turkana",
		Country:   "KE",
		Elevation: 1715,
		Latitude:  3.1219699383,
		Longitude: 35.6086997986,
		Timezone:  "Africa/Nairobi",
	},
	"HKLU": {
		ICAO:      "HKLU",
		IATA:      "LAU",
		Name:      "Manda Airstrip",
		City:      "Lamu",
		State:     "Lamu",
		Country:   "KE",
		Elevation: 20,
		Latitude:  -2.2524199486,
		Longitude: 40.9131011963,
		Timezone:  "Africa/Nairobi",
	},
	"HKLY": {
		ICAO:      "HKLY",
		IATA:      "LOY",
		Name:      "Loyengalani Airport",
		City:      "Loyengalani",
		State:     "Marsabit",
		Country:   "KE",
		Elevation: 1195,
		Latitude:  2.75,
		Longitude: 36.716999054,
		Timezone:  "Africa/Nairobi",
	},
	"HKMA": {
		ICAO:      "HKMA",
		IATA:      "NDE",
		Name:      "Mandera Airport",
		City:      "Mandera",
		State:     "Mandera",
		Country:   "KE",
		Elevation: 805,
		Latitude:  3.9330000877,
		Longitude: 41.8499984741,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HKMB": {
		ICAO:      "HKMB",
		IATA:      "RBT",
		Name:      "Segel Airport",
		City:      "Marsabit",
		State:     "Marsabit",
		Country:   "KE",
		Elevation: 4395,
		Latitude:  2.3499999046,
		Longitude: 37.983001709,
		Timezone:  "Africa/Nairobi",
	},
	"HKMI": {
		ICAO:      "HKMI",
		Name:      "Maralal (Kisima) Airport",
		City:      "Maralal",
		State:     "Samburu",
		Country:   "KE",
		Elevation: 5940,
		Latitude:  0.9499999881,
		Longitude: 36.7999992371,
		Timezone:  "Africa/Nairobi",
	},
	"HKMK": {
		ICAO:      "HKMK",
		IATA:      "JJM",
		Name:      "Mulika Lodge Airport",
		City:      "Meru-Kinna",
		State:     "Isiolo",
		Country:   "KE",
		Elevation: 2000,
		Latitude:  0.165083006,
		Longitude: 38.1951408386,
		Timezone:  "Africa/Nairobi",
	},
	"HKML": {
		ICAO:      "HKML",
		IATA:      "MYD",
		Name:      "Malindi Airport",
		City:      "Malindi",
		State:     "Kilifi",
		Country:   "KE",
		Elevation: 80,
		Latitude:  -3.2293100357,
		Longitude: 40.1016998291,
		Timezone:  "Africa/Nairobi",
	},
	"HKMM": {
		ICAO:      "HKMM",
		Name:      "Migori Airport",
		City:      "Migori",
		State:     "Migori",
		Country:   "KE",
		Elevation: 4575,
		Latitude:  -1.1169999838,
		Longitude: 34.483001709,
		Timezone:  "Africa/Nairobi",
	},
	"HKMO": {
		ICAO:      "HKMO",
		IATA:      "MBA",
		Name:      "Mombasa Moi International Airport",
		City:      "Mombasa",
		State:     "Mombasa",
		Country:   "KE",
		Elevation: 200,
		Latitude:  -4.0348300934,
		Longitude: 39.5942001343,
		Timezone:  "Africa/Nairobi",
	},
	"HKMS": {
		ICAO:      "HKMS",
		IATA:      "MRE",
		Name:      "Mara Serena Lodge Airstrip",
		City:      "Masai Mara",
		State:     "Narok",
		Country:   "KE",
		Elevation: 5200,
		Latitude:  -1.406111002,
		Longitude: 35.0080566406,
		Timezone:  "Africa/Nairobi",
	},
	"HKMY": {
		ICAO:      "HKMY",
		IATA:      "OYL",
		Name:      "Moyale Airport",
		City:      "Moyale (Lower)",
		State:     "Marsabit",
		Country:   "KE",
		Elevation: 2790,
		Latitude:  3.4697198868,
		Longitude: 39.101398468,
		Timezone:  "Africa/Nairobi",
	},
	"HKNI": {
		ICAO:      "HKNI",
		IATA:      "NYE",
		Name:      "Nyeri Airport",
		City:      "Nyeri",
		State:     "Nyeri",
		Country:   "KE",
		Elevation: 5830,
		Latitude:  -0.3644140065,
		Longitude: 36.9784851074,
		Timezone:  "Africa/Nairobi",
	},
	"HKNK": {
		ICAO:      "HKNK",
		IATA:      "NUU",
		Name:      "Nakuru Airport",
		City:      "Nakuru",
		State:     "Nakuru",
		Country:   "KE",
		Elevation: 6200,
		Latitude:  -0.2980670035,
		Longitude: 36.1593017578,
		Timezone:  "Africa/Nairobi",
	},
	"HKNO": {
		ICAO:      "HKNO",
		Name:      "Narok Airport",
		City:      "Narok",
		State:     "Narok",
		Country:   "KE",
		Elevation: 6070,
		Latitude:  -1.1499999762,
		Longitude: 35.766998291,
		Timezone:  "Africa/Nairobi",
	},
	"HKNV": {
		ICAO:      "HKNV",
		Name:      "Naivasha Airport",
		City:      "Naivasha",
		State:     "Nakuru",
		Country:   "KE",
		Elevation: 6380,
		Latitude:  -0.7879530191,
		Longitude: 36.4334983826,
		Timezone:  "Africa/Nairobi",
	},
	"HKNW": {
		ICAO:      "HKNW",
		IATA:      "WIL",
		Name:      "Nairobi Wilson Airport",
		City:      "Nairobi",
		State:     "Nairobi-Area",
		Country:   "KE",
		Elevation: 5536,
		Latitude:  -1.3217200041,
		Longitude: 36.8148002625,
		Timezone:  "Africa/Nairobi",
	},
	"HKNY": {
		ICAO:      "HKNY",
		IATA:      "NYK",
		Name:      "Nanyuki Airport",
		City:      "Nanyuki",
		State:     "Nyeri",
		Country:   "KE",
		Elevation: 6250,
		Latitude:  -0.0623988993,
		Longitude: 37.0410079956,
		Timezone:  "Africa/Nairobi",
	},
	"HKRE": {
		ICAO:      "HKRE",
		Name:      "Moi Air Base",
		City:      "Nairobi",
		State:     "Nairobi-Area",
		Country:   "KE",
		Elevation: 5336,
		Latitude:  -1.2772699594,
		Longitude: 36.8623008728,
		Timezone:  "Africa/Nairobi",
	},
	"HKSB": {
		ICAO:      "HKSB",
		IATA:      "UAS",
		Name:      "Samburu South Airport",
		City:      "Samburu South",
		Country:   "KE",
		Elevation: 3295,
		Latitude:  0.530583024,
		Longitude: 37.5341949463,
		Timezone:  "Africa/Nairobi",
	},
	"HKUK": {
		ICAO:      "HKUK",
		IATA:      "UKA",
		Name:      "Ukunda Airstrip",
		City:      "Ukunda",
		Country:   "KE",
		Elevation: 98,
		Latitude:  -4.2933301926,
		Longitude: 39.5710983276,
		Timezone:  "Africa/Nairobi",
	},
	"HKVO": {
		ICAO:      "HKVO",
		Name:      "Voi Airport",
		City:      "Voi",
		State:     "Taita-Taveta",
		Country:   "KE",
		Elevation: 1900,
		Latitude:  -3.3745388985,
		Longitude: 38.5347480774,
		Timezone:  "Africa/Nairobi",
	},
	"HKWE": {
		ICAO:      "HKWE",
		Name:      "Webuye Airport",
		City:      "Webuye",
		State:     "Bungoma",
		Country:   "KE",
		Elevation: 4300,
		Latitude:  0.6169999838,
		Longitude: 34.783000946,
		Timezone:  "Africa/Nairobi",
	},
	"HKWJ": {
		ICAO:      "HKWJ",
		IATA:      "WJR",
		Name:      "Wajir Airport",
		City:      "Wajir",
		State:     "Wajir",
		Country:   "KE",
		Elevation: 770,
		Latitude:  1.7332400084,
		Longitude: 40.0915985107,
		Timezone:  "Africa/Nairobi",
	},
	"HL50": {
		ICAO:      "HL50",
		Name:      "Zelten Sw New Airport",
		Country:   "LY",
		Elevation: 550,
		Latitude:  28.5870990753,
		Longitude: 19.3034000397,
		Timezone:  "Africa/Tripoli",
	},
	"HL54": {
		ICAO:      "HL54",
		Name:      "Beni Walid Airport",
		State:     "Mişratah",
		Country:   "LY",
		Elevation: 985,
		Latitude:  31.739200592,
		Longitude: 13.954000473,
		Timezone:  "Africa/Tripoli",
	},
	"HL56": {
		ICAO:      "HL56",
		Name:      "Matan al-Sarra Air Base",
		Country:   "LY",
		Elevation: 1722,
		Latitude:  21.6877002716,
		Longitude: 21.8309001923,
		Timezone:  "Africa/Tripoli",
	},
	"HL57": {
		ICAO:      "HL57",
		Name:      "Al Wigh Airport",
		Country:   "LY",
		Elevation: 1558,
		Latitude:  24.1858997345,
		Longitude: 14.5327997208,
		Timezone:  "Africa/Tripoli",
	},
	"HL58": {
		ICAO:      "HL58",
		Name:      "Agedabia Airport",
		Country:   "LY",
		Elevation: 50,
		Latitude:  30.7651004791,
		Longitude: 20.191400528,
		Timezone:  "Africa/Tripoli",
	},
	"HL59": {
		ICAO:      "HL59",
		Name:      "Al Khadim Airport",
		Country:   "LY",
		Elevation: 800,
		Latitude:  31.998500824,
		Longitude: 21.1917991638,
		Timezone:  "Africa/Tripoli",
	},
	"HL62": {
		ICAO:      "HL62",
		Name:      "Zuetina Airport",
		Country:   "LY",
		Elevation: 16,
		Latitude:  30.8701992035,
		Longitude: 20.0755004883,
		Timezone:  "Africa/Tripoli",
	},
	"HL64": {
		ICAO:      "HL64",
		Name:      "Qaryat Al Karmal Airport",
		Country:   "LY",
		Elevation: 85,
		Latitude:  31.9757003784,
		Longitude: 20.0268993378,
		Timezone:  "Africa/Tripoli",
	},
	"HL66": {
		ICAO:      "HL66",
		Name:      "Taminhint Airport",
		Country:   "LY",
		Elevation: 1325,
		Latitude:  27.2401008606,
		Longitude: 14.6562995911,
		Timezone:  "Africa/Tripoli",
	},
	"HL69": {
		ICAO:      "HL69",
		Name:      "Al Jufra Airport",
		Country:   "LY",
		Elevation: 846,
		Latitude:  29.1980991364,
		Longitude: 16.0009994507,
		Timezone:  "Africa/Tripoli",
	},
	"HL70": {
		ICAO:      "HL70",
		Name:      "Al Khuwaymat Airport",
		Country:   "LY",
		Elevation: 500,
		Latitude:  27.2572994232,
		Longitude: 21.6180992126,
		Timezone:  "Africa/Tripoli",
	},
	"HL71": {
		ICAO:      "HL71",
		Name:      "Bir Umran Airport",
		Country:   "LY",
		Elevation: 1400,
		Latitude:  26.3323993683,
		Longitude: 13.4221000671,
		Timezone:  "Africa/Tripoli",
	},
	"HL72": {
		ICAO:      "HL72",
		Name:      "Waddan Airport",
		Country:   "LY",
		Elevation: 910,
		Latitude:  29.1392002106,
		Longitude: 16.1602993011,
		Timezone:  "Africa/Tripoli",
	},
	"HL78": {
		ICAO:      "HL78",
		Name:      "Habit Awlad Muhammad Airport",
		Country:   "LY",
		Elevation: 2000,
		Latitude:  30.7019004822,
		Longitude: 12.484000206,
		Timezone:  "Africa/Tripoli",
	},
	"HL79": {
		ICAO:      "HL79",
		Name:      "Waw Al Kabir Airport",
		Country:   "LY",
		Elevation: 1465,
		Latitude:  25.3568000793,
		Longitude: 16.8099994659,
		Timezone:  "Africa/Tripoli",
	},
	"HL80": {
		ICAO:      "HL80",
		Name:      "Matratin Airport",
		Country:   "LY",
		Elevation: 100,
		Latitude:  30.642999649,
		Longitude: 18.3208007813,
		Timezone:  "Africa/Tripoli",
	},
	"HL81": {
		ICAO:      "HL81",
		Name:      "Al Hamada Con 66 East Airport",
		Country:   "LY",
		Elevation: 2090,
		Latitude:  29.5310993195,
		Longitude: 12.9436998367,
		Timezone:  "Africa/Tripoli",
	},
	"HL82": {
		ICAO:      "HL82",
		Name:      "Wadi Buzanad Sw Airport",
		Country:   "LY",
		Elevation: 381,
		Latitude:  28.9619007111,
		Longitude: 17.5881004333,
		Timezone:  "Africa/Tripoli",
	},
	"HL83": {
		ICAO:      "HL83",
		Name:      "Nanur Airport",
		Country:   "LY",
		Elevation: 185,
		Latitude:  31.7052001953,
		Longitude: 14.9116001129,
		Timezone:  "Africa/Tripoli",
	},
	"HL84": {
		ICAO:      "HL84",
		Name:      "Sarir Nw Airport",
		Country:   "LY",
		Elevation: 330,
		Latitude:  27.9757003784,
		Longitude: 22.3574008942,
		Timezone:  "Africa/Tripoli",
	},
	"HLAM": {
		ICAO:      "HLAM",
		Name:      "Amal V12 Airport",
		City:      "Amal V12",
		Country:   "LY",
		Elevation: 145,
		Latitude:  29.4794998169,
		Longitude: 21.1224002838,
		Timezone:  "Africa/Tripoli",
	},
	"HLBD": {
		ICAO:      "HLBD",
		Name:      "Beda (M-3) Airport",
		City:      "Beda M3",
		Country:   "LY",
		Elevation: 499,
		Latitude:  28.5032997131,
		Longitude: 19.0027999878,
		Timezone:  "Africa/Tripoli",
	},
	"HLFL": {
		ICAO:      "HLFL",
		Name:      "Bu Attifel Airport",
		Country:   "LY",
		Elevation: 161,
		Latitude:  28.7954006195,
		Longitude: 22.0809001923,
		Timezone:  "Africa/Tripoli",
	},
	"HLGD": {
		ICAO:      "HLGD",
		IATA:      "SRX",
		Name:      "Gardabya Airport",
		City:      "Sirt",
		Country:   "LY",
		Elevation: 267,
		Latitude:  31.0634994507,
		Longitude: 16.5949993134,
		Timezone:  "Africa/Tripoli",
	},
	"HLGL": {
		ICAO:      "HLGL",
		Name:      "Warehouse 59e Airport",
		Country:   "LY",
		Elevation: 325,
		Latitude:  28.6385002136,
		Longitude: 21.4379997253,
		Timezone:  "Africa/Tripoli",
	},
	"HLGN": {
		ICAO:      "HLGN",
		IATA:      "TOB",
		Name:      "Gamal Abdel Nasser Airport",
		City:      "Tobruk",
		Country:   "LY",
		Elevation: 519,
		Latitude:  31.861000061,
		Longitude: 23.906999588,
		Timezone:  "Africa/Tripoli",
	},
	"HLGT": {
		ICAO:      "HLGT",
		IATA:      "GHT",
		Name:      "Ghat Airport",
		City:      "Ghat",
		State:     "Sha‘biyat-Ghat",
		Country:   "LY",
		Elevation: 2296,
		Latitude:  25.1455993652,
		Longitude: 10.1426000595,
		Timezone:  "Africa/Tripoli",
	},
	"HLKF": {
		ICAO:      "HLKF",
		IATA:      "AKF",
		Name:      "Kufra Airport",
		City:      "Kufra",
		State:     "Al-Kufrah",
		Country:   "LY",
		Elevation: 1367,
		Latitude:  24.1786994934,
		Longitude: 23.313999176,
		Timezone:  "Africa/Tripoli",
	},
	"HLLB": {
		ICAO:      "HLLB",
		IATA:      "BEN",
		Name:      "Benina International Airport",
		City:      "Benghazi",
		State:     "Banghazi",
		Country:   "LY",
		Elevation: 433,
		Latitude:  32.0968017578,
		Longitude: 20.2695007324,
		Timezone:  "Africa/Tripoli",
	},
	"HLLM": {
		ICAO:      "HLLM",
		IATA:      "MJI",
		Name:      "Mitiga Airport",
		City:      "Tripoli",
		State:     "Tripoli",
		Country:   "LY",
		Elevation: 36,
		Latitude:  32.8941001892,
		Longitude: 13.2760000229,
		Timezone:  "Africa/Tripoli",
	},
	"HLLQ": {
		ICAO:      "HLLQ",
		IATA:      "LAQ",
		Name:      "La Abraq Airport",
		City:      "Al Bayda'",
		Country:   "LY",
		Elevation: 2157,
		Latitude:  32.7887001038,
		Longitude: 21.9643001556,
		Timezone:  "Africa/Tripoli",
	},
	"HLLS": {
		ICAO:      "HLLS",
		IATA:      "SEB",
		Name:      "Sabha Airport",
		City:      "Sabha",
		Country:   "LY",
		Elevation: 1427,
		Latitude:  26.9869995117,
		Longitude: 14.4724998474,
		Timezone:  "Africa/Tripoli",
	},
	"HLLT": {
		ICAO:      "HLLT",
		IATA:      "TIP",
		Name:      "Tripoli International Airport",
		City:      "Tripoli",
		Country:   "LY",
		Elevation: 263,
		Latitude:  32.6635017395,
		Longitude: 13.1590003967,
		Timezone:  "Africa/Tripoli",
	},
	"HLMB": {
		ICAO:      "HLMB",
		IATA:      "LMQ",
		Name:      "Marsa Brega Airport",
		Country:   "LY",
		Elevation: 50,
		Latitude:  30.3780994415,
		Longitude: 19.5764007568,
		Timezone:  "Africa/Tripoli",
	},
	"HLNF": {
		ICAO:      "HLNF",
		Name:      "Ras Lanuf Oil Airport",
		Country:   "LY",
		Elevation: 42,
		Latitude:  30.5,
		Longitude: 18.5272006989,
		Timezone:  "Africa/Tripoli",
	},
	"HLNR": {
		ICAO:      "HLNR",
		IATA:      "NFR",
		Name:      "Nafurah 1 Airport",
		City:      "Nafurah 1",
		Country:   "LY",
		Elevation: 122,
		Latitude:  29.2131996155,
		Longitude: 21.5923995972,
		Timezone:  "Africa/Tripoli",
	},
	"HLON": {
		ICAO:      "HLON",
		IATA:      "HUQ",
		Name:      "Hon Airport",
		Country:   "LY",
		Elevation: 919,
		Latitude:  29.1100997925,
		Longitude: 15.9656000137,
		Timezone:  "Africa/Tripoli",
	},
	"HLRA": {
		ICAO:      "HLRA",
		Name:      "Dahra Airport",
		Country:   "LY",
		Elevation: 1050,
		Latitude:  29.4726009369,
		Longitude: 17.9349002838,
		Timezone:  "Africa/Tripoli",
	},
	"HLSA": {
		ICAO:      "HLSA",
		Name:      "Sarir (C-4) Airport",
		City:      "Sarir",
		Country:   "LY",
		Elevation: 400,
		Latitude:  27.662399292,
		Longitude: 22.508600235,
		Timezone:  "Africa/Tripoli",
	},
	"HLSD": {
		ICAO:      "HLSD",
		Name:      "Essider Airport",
		Country:   "LY",
		Elevation: 20,
		Latitude:  30.625,
		Longitude: 18.3516998291,
		Timezone:  "Africa/Tripoli",
	},
	"HLTD": {
		ICAO:      "HLTD",
		IATA:      "LTD",
		Name:      "Ghadames East Airport",
		City:      "Ghadames",
		Country:   "LY",
		Elevation: 1122,
		Latitude:  30.1516990662,
		Longitude: 9.7153100967,
		Timezone:  "Africa/Tripoli",
	},
	"HLWA": {
		ICAO:      "HLWA",
		Name:      "Warehouse 59A Airport",
		City:      "Warehouse 59A",
		Country:   "LY",
		Elevation: 488,
		Latitude:  28.3223991394,
		Longitude: 19.9300003052,
		Timezone:  "Africa/Tripoli",
	},
	"HLZA": {
		ICAO:      "HLZA",
		Name:      "Zella 74 Airport",
		Country:   "LY",
		Elevation: 1085,
		Latitude:  28.5898990631,
		Longitude: 17.2938995361,
		Timezone:  "Africa/Tripoli",
	},
	"HLZG": {
		ICAO:      "HLZG",
		Name:      "Oxy A 103 Airport",
		City:      "Oxy A 103",
		Country:   "LY",
		Elevation: 318,
		Latitude:  29.0062999725,
		Longitude: 20.7861003876,
		Timezone:  "Africa/Tripoli",
	},
	"HLZU": {
		ICAO:      "HLZU",
		Name:      "Zueitina Airport",
		Country:   "LY",
		Elevation: 20,
		Latitude:  30.875,
		Longitude: 20.0699996948,
		Timezone:  "Africa/Tripoli",
	},
	"HLZW": {
		ICAO:      "HLZW",
		IATA:      "WAX",
		Name:      "Zwara Airport",
		City:      "Zuwara",
		Country:   "LY",
		Latitude:  32.9523010254,
		Longitude: 12.0155000687,
		Timezone:  "Africa/Tripoli",
	},
	"HRYG": {
		ICAO:      "HRYG",
		IATA:      "GYI",
		Name:      "Gisenyi Airport",
		City:      "Gisenyi",
		State:     "Western-Province",
		Country:   "RW",
		Elevation: 5082,
		Latitude:  -1.6771999598,
		Longitude: 29.2588996887,
		Timezone:  "Africa/Kigali",
	},
	"HRYI": {
		ICAO:      "HRYI",
		IATA:      "BTQ",
		Name:      "Butare Airport",
		City:      "Butare",
		State:     "Southern-Province",
		Country:   "RW",
		Elevation: 5801,
		Latitude:  -2.5958299637,
		Longitude: 29.736700058,
		Timezone:  "Africa/Kigali",
	},
	"HRYN": {
		ICAO:      "HRYN",
		Name:      "Nemba Airport",
		City:      "Nemba",
		State:     "Eastern-Province",
		Country:   "RW",
		Elevation: 4905,
		Latitude:  -2.3329999447,
		Longitude: 30.2000007629,
		Timezone:  "Africa/Bujumbura",
	},
	"HRYO": {
		ICAO:      "HRYO",
		Name:      "Gabiro Airport",
		City:      "Gabiro",
		State:     "Eastern-Province",
		Country:   "RW",
		Elevation: 4806,
		Latitude:  -1.5499999523,
		Longitude: 30.3829994202,
		Timezone:  "Africa/Kigali",
	},
	"HRYR": {
		ICAO:      "HRYR",
		IATA:      "KGL",
		Name:      "Kigali International Airport",
		City:      "Kigali",
		State:     "Kigali",
		Country:   "RW",
		Elevation: 4859,
		Latitude:  -1.9686299563,
		Longitude: 30.1394996643,
		Timezone:  "Africa/Kigali",
	},
	"HRYU": {
		ICAO:      "HRYU",
		IATA:      "RHG",
		Name:      "Ruhengeri Airport",
		City:      "Ruhengeri",
		State:     "Northern-Province",
		Country:   "RW",
		Elevation: 6102,
		Latitude:  -1.5,
		Longitude: 29.6329994202,
		Timezone:  "Africa/Kigali",
	},
	"HRZA": {
		ICAO:      "HRZA",
		IATA:      "KME",
		Name:      "Kamembe Airport",
		City:      "Kamembe",
		State:     "Western-Province",
		Country:   "RW",
		Elevation: 5192,
		Latitude:  -2.4622399807,
		Longitude: 28.9078998566,
		Timezone:  "Africa/Kigali",
	},
	"HSAK": {
		ICAO:      "HSAK",
		Name:      "Akobo Airport",
		City:      "Akobo",
		Country:   "SS",
		Latitude:  7.7833299637,
		Longitude: 33,
		Timezone:  "Africa/Juba",
	},
	"HSAT": {
		ICAO:      "HSAT",
		IATA:      "ATB",
		Name:      "Atbara Airport",
		City:      "Atbara",
		Country:   "SD",
		Elevation: 1181,
		Latitude:  17.7103443146,
		Longitude: 34.05701828,
		Timezone:  "Africa/Khartoum",
	},
	"HSAW": {
		ICAO:      "HSAW",
		Name:      "Aweil Airport",
		City:      "Aweil",
		Country:   "SS",
		Elevation: 1394,
		Latitude:  8.7670001984,
		Longitude: 27.3999996185,
		Timezone:  "Africa/Juba",
	},
	"HSBR": {
		ICAO:      "HSBR",
		Name:      "Bor Airport",
		City:      "Bor",
		Country:   "SS",
		Elevation: 1394,
		Latitude:  6.1999998093,
		Longitude: 31.5669994354,
		Timezone:  "Africa/Juba",
	},
	"HSBT": {
		ICAO:      "HSBT",
		Name:      "Bentu Airport",
		City:      "Bentu",
		Country:   "SS",
		Elevation: 2000,
		Latitude:  9.2170000076,
		Longitude: 29.8169994354,
		Timezone:  "Africa/Juba",
	},
	"HSCG": {
		ICAO:      "HSCG",
		Name:      "Carthago Airport",
		City:      "Carthago",
		State:     "Red-Sea",
		Country:   "SD",
		Elevation: 3230,
		Latitude:  18.75,
		Longitude: 37,
		Timezone:  "Africa/Khartoum",
	},
	"HSDB": {
		ICAO:      "HSDB",
		IATA:      "EDB",
		Name:      "El Debba Airport",
		City:      "El Debba",
		Country:   "SD",
		Elevation: 843,
		Latitude:  18.0249996185,
		Longitude: 30.9582996368,
		Timezone:  "Africa/Khartoum",
	},
	"HSDL": {
		ICAO:      "HSDL",
		Name:      "Dilling Airport",
		City:      "Dilling",
		Country:   "SD",
		Elevation: 2206,
		Latitude:  11.9905996323,
		Longitude: 29.6732006073,
		Timezone:  "Africa/Khartoum",
	},
	"HSDN": {
		ICAO:      "HSDN",
		IATA:      "DOG",
		Name:      "Dongola Airport",
		City:      "Dongola",
		State:     "Northern-State",
		Country:   "SD",
		Elevation: 772,
		Latitude:  19.1539001465,
		Longitude: 30.4300994873,
		Timezone:  "Africa/Khartoum",
	},
	"HSDZ": {
		ICAO:      "HSDZ",
		IATA:      "RSS",
		Name:      "Damazin Airport",
		City:      "Ad Damazin",
		State:     "Blue-Nile",
		Country:   "SD",
		Elevation: 1582,
		Latitude:  11.785900116,
		Longitude: 34.3367004395,
		Timezone:  "Africa/Khartoum",
	},
	"HSFS": {
		ICAO:      "HSFS",
		IATA:      "ELF",
		Name:      "El Fasher Airport",
		City:      "El Fasher",
		State:     "Northern-Darfur",
		Country:   "SD",
		Elevation: 2393,
		Latitude:  13.6148996353,
		Longitude: 25.3246002197,
		Timezone:  "Africa/Khartoum",
	},
	"HSGF": {
		ICAO:      "HSGF",
		IATA:      "GSU",
		Name:      "Azaza Airport",
		City:      "Gedaref",
		Country:   "SD",
		Elevation: 1640,
		Latitude:  14.1330003738,
		Longitude: 35.3170013428,
		Timezone:  "Africa/Khartoum",
	},
	"HSGG": {
		ICAO:      "HSGG",
		IATA:      "DNX",
		Name:      "Galegu Airport",
		City:      "Dinder",
		Country:   "SD",
		Elevation: 1640,
		Latitude:  12.5329999924,
		Longitude: 35.0670013428,
		Timezone:  "Africa/Khartoum",
	},
	"HSGN": {
		ICAO:      "HSGN",
		IATA:      "EGN",
		Name:      "Geneina Airport",
		City:      "Geneina",
		State:     "Western-Darfur",
		Country:   "SD",
		Elevation: 2650,
		Latitude:  13.4816999435,
		Longitude: 22.4671993256,
		Timezone:  "Africa/Khartoum",
	},
	"HSGO": {
		ICAO:      "HSGO",
		Name:      "Gogrial Airport",
		City:      "Gogrial",
		State:     "Northern-State",
		Country:   "SD",
		Elevation: 1640,
		Latitude:  18.8670005798,
		Longitude: 28.1170005798,
		Timezone:  "Africa/Khartoum",
	},
	"HSHG": {
		ICAO:      "HSHG",
		IATA:      "HEG",
		Name:      "Heglig Airport",
		City:      "Heglig Oilfield",
		Country:   "SD",
		Elevation: 1325,
		Latitude:  9.994933,
		Longitude: 29.397718,
		Timezone:  "Africa/Khartoum",
	},
	"HSKA": {
		ICAO:      "HSKA",
		IATA:      "KSL",
		Name:      "Kassala Airport",
		City:      "Kassala",
		State:     "Kassala",
		Country:   "SD",
		Elevation: 1671,
		Latitude:  15.3874998093,
		Longitude: 36.3288002014,
		Timezone:  "Africa/Khartoum",
	},
	"HSKG": {
		ICAO:      "HSKG",
		IATA:      "GBU",
		Name:      "Khashm El Girba Airport",
		City:      "Khashm El Girba",
		Country:   "SD",
		Elevation: 1560,
		Latitude:  14.9250001907,
		Longitude: 35.8779983521,
		Timezone:  "Africa/Khartoum",
	},
	"HSKI": {
		ICAO:      "HSKI",
		IATA:      "KST",
		Name:      "Kosti Airport",
		City:      "Kosti",
		State:     "White-Nile",
		Country:   "SD",
		Elevation: 1289,
		Latitude:  13.1829996109,
		Longitude: 32.733001709,
		Timezone:  "Africa/Khartoum",
	},
	"HSKJ": {
		ICAO:      "HSKJ",
		Name:      "Kago Kaju Airport",
		City:      "Kago Kaju",
		Country:   "SS",
		Elevation: 2500,
		Latitude:  3.9170000553,
		Longitude: 31.6669998169,
		Timezone:  "Africa/Juba",
	},
	"HSKP": {
		ICAO:      "HSKP",
		Name:      "Kapoeta Airport",
		City:      "Kapoeta",
		Country:   "SS",
		Elevation: 2220,
		Latitude:  4.7829999924,
		Longitude: 33.5830001831,
		Timezone:  "Africa/Juba",
	},
	"HSLI": {
		ICAO:      "HSLI",
		IATA:      "KDX",
		Name:      "Kadugli Airport",
		City:      "Kadugli",
		State:     "Southern-Kordofan",
		Country:   "SD",
		Elevation: 1848,
		Latitude:  11.1379995346,
		Longitude: 29.7010993958,
		Timezone:  "Africa/Khartoum",
	},
	"HSMD": {
		ICAO:      "HSMD",
		Name:      "Marida Airport",
		City:      "Marida",
		Country:   "SS",
		Elevation: 1901,
		Latitude:  4.9169998169,
		Longitude: 29.466999054,
		Timezone:  "Africa/Juba",
	},
	"HSMK": {
		ICAO:      "HSMK",
		IATA:      "RBX",
		Name:      "Rumbek Airport",
		City:      "Rumbek",
		Country:   "SS",
		Elevation: 1378,
		Latitude:  6.8249998093,
		Longitude: 29.6690006256,
		Timezone:  "Africa/Juba",
	},
	"HSMN": {
		ICAO:      "HSMN",
		Name:      "Merowe (Merowe New)",
		State:     "Northern-State",
		Country:   "SD",
		Elevation: 897,
		Latitude:  18.4433333333,
		Longitude: 31.8433333333,
		Timezone:  "Africa/Khartoum",
	},
	"HSMR": {
		ICAO:      "HSMR",
		IATA:      "MWE",
		Name:      "Merowe Airport",
		City:      "Merowe",
		State:     "Northern-State",
		Country:   "SD",
		Elevation: 846,
		Latitude:  18.4500007629,
		Longitude: 31.8330001831,
		Timezone:  "Africa/Khartoum",
	},
	"HSND": {
		ICAO:      "HSND",
		Name:      "Shendi Airport",
		City:      "Shendi",
		Country:   "SD",
		Elevation: 1181,
		Latitude:  16.7000007629,
		Longitude: 33.4329986572,
		Timezone:  "Africa/Khartoum",
	},
	"HSNH": {
		ICAO:      "HSNH",
		IATA:      "NUD",
		Name:      "En Nahud Airport",
		City:      "En Nahud",
		State:     "West-Kordofan-State",
		Country:   "SD",
		Elevation: 1955,
		Latitude:  12.6669998169,
		Longitude: 28.3330001831,
		Timezone:  "Africa/Khartoum",
	},
	"HSNM": {
		ICAO:      "HSNM",
		Name:      "Nimuli Airport",
		City:      "Nimuli",
		Country:   "SS",
		Elevation: 2059,
		Latitude:  3.617000103,
		Longitude: 32.1329994202,
		Timezone:  "Africa/Juba",
	},
	"HSNN": {
		ICAO:      "HSNN",
		IATA:      "UYL",
		Name:      "Nyala Airport",
		City:      "Nyala",
		State:     "Southern-Darfur",
		Country:   "SD",
		Elevation: 2106,
		Latitude:  12.0535001755,
		Longitude: 24.956199646,
		Timezone:  "Africa/Khartoum",
	},
	"HSNW": {
		ICAO:      "HSNW",
		IATA:      "NHF",
		Name:      "New Halfa Airport",
		City:      "New Halfa",
		State:     "Kassala",
		Country:   "SD",
		Elevation: 1480,
		Latitude:  15.3556003571,
		Longitude: 35.7277984619,
		Timezone:  "Africa/Khartoum",
	},
	"HSOB": {
		ICAO:      "HSOB",
		IATA:      "EBD",
		Name:      "El Obeid Airport",
		City:      "Al-Ubayyid",
		State:     "North-Kordofan",
		Country:   "SD",
		Elevation: 1927,
		Latitude:  13.1532001495,
		Longitude: 30.2327003479,
		Timezone:  "Africa/Khartoum",
	},
	"HSPA": {
		ICAO:      "HSPA",
		Name:      "Pachella Airport",
		City:      "Pachella",
		Country:   "SS",
		Elevation: 2000,
		Latitude:  7.178563118,
		Longitude: 34.0973587036,
		Timezone:  "Africa/Addis_Ababa",
	},
	"HSPI": {
		ICAO:      "HSPI",
		Name:      "Pibor Airport",
		City:      "Pibor Post",
		Country:   "SS",
		Elevation: 1377,
		Latitude:  6.7933333333,
		Longitude: 33.1305555556,
		Timezone:  "Africa/Juba",
	},
	"HSPN": {
		ICAO:      "HSPN",
		IATA:      "PZU",
		Name:      "Port Sudan New International Airport",
		City:      "Port Sudan",
		State:     "Red-Sea",
		Country:   "SD",
		Elevation: 135,
		Latitude:  19.433599472,
		Longitude: 37.2341003418,
		Timezone:  "Africa/Khartoum",
	},
	"HSRJ": {
		ICAO:      "HSRJ",
		Name:      "Raga Airport",
		City:      "Raga",
		Country:   "SS",
		Elevation: 1788,
		Latitude:  8.4611101151,
		Longitude: 25.6811008453,
		Timezone:  "Africa/Juba",
	},
	"HSRN": {
		ICAO:      "HSRN",
		Name:      "Renk Airport",
		City:      "Renk",
		Country:   "SS",
		Elevation: 1278,
		Latitude:  11.8324692723,
		Longitude: 32.8104972839,
		Timezone:  "Africa/Juba",
	},
	"HSSJ": {
		ICAO:      "HSSJ",
		IATA:      "JUB",
		Name:      "Juba Airport",
		City:      "Juba",
		Country:   "SS",
		Elevation: 1513,
		Latitude:  4.872010231,
		Longitude: 31.6011009216,
		Timezone:  "Africa/Juba",
	},
	"HSSM": {
		ICAO:      "HSSM",
		IATA:      "MAK",
		Name:      "Malakal Airport",
		City:      "Malakal",
		Country:   "SS",
		Elevation: 1291,
		Latitude:  9.5589704514,
		Longitude: 31.6522006989,
		Timezone:  "Africa/Juba",
	},
	"HSSP": {
		ICAO:      "HSSP",
		Name:      "Port Sudan Air Base",
		City:      "Port Sudan",
		State:     "Red-Sea",
		Country:   "SD",
		Elevation: 10,
		Latitude:  19.576499939,
		Longitude: 37.2159004211,
		Timezone:  "Africa/Khartoum",
	},
	"HSSS": {
		ICAO:      "HSSS",
		IATA:      "KRT",
		Name:      "Khartoum International Airport",
		City:      "Khartoum",
		State:     "Khartoum",
		Country:   "SD",
		Elevation: 1265,
		Latitude:  15.5895004272,
		Longitude: 32.5531997681,
		Timezone:  "Africa/Khartoum",
	},
	"HSSW": {
		ICAO:      "HSSW",
		IATA:      "WHF",
		Name:      "Wadi Halfa Airport",
		City:      "Wadi Halfa",
		State:     "Northern-State",
		Country:   "SD",
		Elevation: 961,
		Latitude:  21.8026981354,
		Longitude: 31.5215778351,
		Timezone:  "Africa/Khartoum",
	},
	"HSTO": {
		ICAO:      "HSTO",
		Name:      "Tong Airport",
		City:      "Tong",
		Country:   "SS",
		Elevation: 1413,
		Latitude:  7.2670001984,
		Longitude: 28.9829998016,
		Timezone:  "Africa/Juba",
	},
	"HSTR": {
		ICAO:      "HSTR",
		Name:      "Torit Airport",
		City:      "Torit",
		Country:   "SS",
		Elevation: 2050,
		Latitude:  4.4000000954,
		Longitude: 32.5830001831,
		Timezone:  "Africa/Juba",
	},
	"HSTU": {
		ICAO:      "HSTU",
		Name:      "Tumbura Airport",
		City:      "Tumbura",
		Country:   "SS",
		Elevation: 2230,
		Latitude:  5.5999999046,
		Longitude: 27.4500007629,
		Timezone:  "Africa/Juba",
	},
	"HSWW": {
		ICAO:      "HSWW",
		IATA:      "WUU",
		Name:      "Wau Airport",
		City:      "Wau",
		Country:   "SS",
		Elevation: 1529,
		Latitude:  7.7258300781,
		Longitude: 27.9750003815,
		Timezone:  "Africa/Juba",
	},
	"HSYA": {
		ICAO:      "HSYA",
		Name:      "Yambio Airport",
		City:      "Yambio",
		Country:   "SS",
		Elevation: 2375,
		Latitude:  4.567220211,
		Longitude: 28.4239006042,
		Timezone:  "Africa/Juba",
	},
	"HSYE": {
		ICAO:      "HSYE",
		Name:      "Yei Airport",
		City:      "Yei",
		Country:   "SS",
		Elevation: 2000,
		Latitude:  4.0830001831,
		Longitude: 30.6499996185,
		Timezone:  "Africa/Juba",
	},
	"HSYL": {
		ICAO:      "HSYL",
		Name:      "Yirol Airport",
		City:      "Yirol",
		Country:   "SS",
		Elevation: 1424,
		Latitude:  6.5669999123,
		Longitude: 30.5,
		Timezone:  "Africa/Juba",
	},
	"HSZA": {
		ICAO:      "HSZA",
		IATA:      "ZLX",
		Name:      "Zalingei Airport",
		City:      "Zalingei",
		State:     "Central-Darfur",
		Country:   "SD",
		Elevation: 2953,
		Latitude:  12.8999996185,
		Longitude: 23.4829998016,
		Timezone:  "Africa/Khartoum",
	},
	"HTAG": {
		ICAO:      "HTAG",
		Name:      "Amani Gomvu Airport",
		City:      "Kutani",
		State:     "Dar-es-Salaam",
		Country:   "TZ",
		Elevation: 114,
		Latitude:  -6.935353,
		Longitude: 39.493046,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTAR": {
		ICAO:      "HTAR",
		IATA:      "ARK",
		Name:      "Arusha Airport",
		City:      "Arusha",
		State:     "Arusha",
		Country:   "TZ",
		Elevation: 4550,
		Latitude:  -3.3677899837,
		Longitude: 36.6333007813,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTBB": {
		ICAO:      "HTBB",
		Name:      "Kibambawe Airstrip",
		City:      "Selous Game Reserve",
		State:     "Pwani",
		Country:   "TZ",
		Elevation: 485,
		Latitude:  -7.748241,
		Longitude: 38.001833,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTBS": {
		ICAO:      "HTBS",
		Name:      "Mbesa Airport",
		City:      "Mbesa Mission",
		State:     "Ruvuma",
		Country:   "TZ",
		Elevation: 1980,
		Latitude:  -11.331675,
		Longitude: 37.067816,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTBU": {
		ICAO:      "HTBU",
		IATA:      "BKZ",
		Name:      "Bukoba Airport",
		City:      "Bukoba",
		State:     "Kagera",
		Country:   "TZ",
		Elevation: 3745,
		Latitude:  -1.332,
		Longitude: 31.8212,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTCH": {
		ICAO:      "HTCH",
		Name:      "Chunya Airport",
		City:      "Chunya",
		State:     "Mbeya",
		Country:   "TZ",
		Elevation: 5000,
		Latitude:  -8.5500001907,
		Longitude: 33.4329986572,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTDA": {
		ICAO:      "HTDA",
		IATA:      "DAR",
		Name:      "Mwalimu Julius K. Nyerere International Airport",
		City:      "Dar es Salaam",
		State:     "Dar-es-Salaam",
		Country:   "TZ",
		Elevation: 182,
		Latitude:  -6.8781099319,
		Longitude: 39.2025985718,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTDO": {
		ICAO:      "HTDO",
		IATA:      "DOD",
		Name:      "Dodoma Airport",
		City:      "Dodoma",
		State:     "Dodoma",
		Country:   "TZ",
		Elevation: 3673,
		Latitude:  -6.170440197,
		Longitude: 35.7526016235,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTIR": {
		ICAO:      "HTIR",
		IATA:      "IRI",
		Name:      "Iringa Airport",
		City:      "Nduli",
		State:     "Iringa",
		Country:   "TZ",
		Elevation: 4678,
		Latitude:  -7.6686301231,
		Longitude: 35.7521018982,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTIY": {
		ICAO:      "HTIY",
		Name:      "Inyonga Airport",
		City:      "Inyonga",
		State:     "Katavi",
		Country:   "TZ",
		Elevation: 3790,
		Latitude:  -6.7166700363,
		Longitude: 32.0999984741,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTKA": {
		ICAO:      "HTKA",
		IATA:      "TKQ",
		Name:      "Kigoma Airport",
		City:      "Kigoma",
		State:     "Kigoma",
		Country:   "TZ",
		Elevation: 2700,
		Latitude:  -4.8862,
		Longitude: 29.6709,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTKB": {
		ICAO:      "HTKB",
		Name:      "Kibondo Airport",
		City:      "Kibondo",
		State:     "Kigoma",
		Country:   "TZ",
		Elevation: 4262,
		Latitude:  -3.5259799957,
		Longitude: 30.6504001617,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTKD": {
		ICAO:      "HTKD",
		Name:      "Kondoa Airport",
		City:      "Kondoa",
		State:     "Dodoma",
		Country:   "TZ",
		Elevation: 4578,
		Latitude:  -4.8937101364,
		Longitude: 35.7682991028,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTKI": {
		ICAO:      "HTKI",
		IATA:      "KIY",
		Name:      "Kilwa Masoko Airport",
		City:      "Kilwa Masoko",
		State:     "Lindi",
		Country:   "TZ",
		Elevation: 50,
		Latitude:  -8.9329996109,
		Longitude: 39.516998291,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTKJ": {
		ICAO:      "HTKJ",
		IATA:      "JRO",
		Name:      "Kilimanjaro International Airport",
		City:      "Arusha",
		State:     "Kilimanjaro",
		Country:   "TZ",
		Elevation: 2932,
		Latitude:  -3.4294099808,
		Longitude: 37.0745010376,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTKL": {
		ICAO:      "HTKL",
		Name:      "Kirondatal Airport",
		City:      "Kirondatal",
		State:     "Singida",
		Country:   "TZ",
		Elevation: 5033,
		Latitude:  -4.3143720627,
		Longitude: 34.3333206177,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTKO": {
		ICAO:      "HTKO",
		Name:      "Kongwa Airport",
		City:      "Kongwa",
		State:     "Dodoma",
		Country:   "TZ",
		Elevation: 3300,
		Latitude:  -6.1500000954,
		Longitude: 36.4169998169,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTKS": {
		ICAO:      "HTKS",
		Name:      "Kilosa Airport",
		City:      "Kilosa",
		State:     "Morogoro",
		Country:   "TZ",
		Elevation: 1567,
		Latitude:  -6.9000000954,
		Longitude: 37.0166664124,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTKT": {
		ICAO:      "HTKT",
		Name:      "Kilimatinde Airport",
		City:      "Kilimatinde",
		State:     "Singida",
		Country:   "TZ",
		Elevation: 3608,
		Latitude:  -5.8330001831,
		Longitude: 34.9329986572,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTLD": {
		ICAO:      "HTLD",
		Name:      "Loliondo Airport",
		City:      "Loliondo",
		State:     "Arusha",
		Country:   "TZ",
		Elevation: 6620,
		Latitude:  -2.0696899891,
		Longitude: 35.5457000732,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTLI": {
		ICAO:      "HTLI",
		IATA:      "LDI",
		Name:      "Kikwetu Airport",
		City:      "Lindi",
		State:     "Lindi",
		Country:   "TZ",
		Elevation: 100,
		Latitude:  -9.8511104584,
		Longitude: 39.7578010559,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTLL": {
		ICAO:      "HTLL",
		Name:      "Liuli Airport",
		City:      "Liuli",
		State:     "Ruvuma",
		Country:   "TZ",
		Elevation: 1667,
		Latitude:  -11.116399765,
		Longitude: 34.6521987915,
		Timezone:  "Africa/Blantyre",
	},
	"HTLM": {
		ICAO:      "HTLM",
		IATA:      "LKY",
		Name:      "Lake Manyara Airport",
		City:      "Lake Manyara National Park",
		State:     "Arusha",
		Country:   "TZ",
		Elevation: 4150,
		Latitude:  -3.3763101101,
		Longitude: 35.8182983398,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTMA": {
		ICAO:      "HTMA",
		IATA:      "MFA",
		Name:      "Mafia Island Airport",
		City:      "Mafia Island",
		State:     "Pwani",
		Country:   "TZ",
		Elevation: 60,
		Latitude:  -7.9169998169,
		Longitude: 39.6669998169,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTMB": {
		ICAO:      "HTMB",
		IATA:      "MBI",
		Name:      "Mbeya Airport",
		City:      "Mbeya",
		State:     "Mbeya",
		Country:   "TZ",
		Elevation: 5600,
		Latitude:  -8.9169998169,
		Longitude: 33.466999054,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTMD": {
		ICAO:      "HTMD",
		IATA:      "MWN",
		Name:      "Mwadui Airport",
		City:      "Mwadui",
		State:     "Shinyanga",
		Country:   "TZ",
		Elevation: 3970,
		Latitude:  -3.5,
		Longitude: 33.6170005798,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTMF": {
		ICAO:      "HTMF",
		Name:      "Mufindi Airport",
		City:      "Mufindi",
		State:     "Iringa",
		Country:   "TZ",
		Elevation: 4175,
		Latitude:  -8.7320404053,
		Longitude: 35.3026008606,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTMG": {
		ICAO:      "HTMG",
		Name:      "Morogoro Airport",
		City:      "Morogoro",
		State:     "Morogoro",
		Country:   "TZ",
		Elevation: 1676,
		Latitude:  -6.7972202301,
		Longitude: 37.6530990601,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTMI": {
		ICAO:      "HTMI",
		IATA:      "XMI",
		Name:      "Masasi Airport",
		City:      "Masasi",
		State:     "Mtwara",
		Country:   "TZ",
		Elevation: 1700,
		Latitude:  -10.7329998016,
		Longitude: 38.766998291,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTMK": {
		ICAO:      "HTMK",
		Name:      "Mikumi Airport",
		City:      "Mikumi",
		State:     "Morogoro",
		Country:   "TZ",
		Elevation: 1737,
		Latitude:  -7.333427906,
		Longitude: 37.1163597107,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTML": {
		ICAO:      "HTML",
		Name:      "Malya Airport",
		City:      "Malya",
		State:     "Shinyanga",
		Country:   "TZ",
		Elevation: 4079,
		Latitude:  -2.9778599739,
		Longitude: 33.5163002014,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTMO": {
		ICAO:      "HTMO",
		Name:      "Mombo Airport",
		City:      "Mombo",
		State:     "Tanga",
		Country:   "TZ",
		Elevation: 1350,
		Latitude:  -4.882999897,
		Longitude: 38.283000946,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTMP": {
		ICAO:      "HTMP",
		Name:      "Mpanda Airport",
		City:      "Mpanda",
		State:     "Rukwa",
		Country:   "TZ",
		Elevation: 3520,
		Latitude:  -6.3499999046,
		Longitude: 31.0830001831,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTMS": {
		ICAO:      "HTMS",
		IATA:      "QSI",
		Name:      "Moshi Airport",
		City:      "Moshi",
		State:     "Kilimanjaro",
		Country:   "TZ",
		Elevation: 2801,
		Latitude:  -3.3633298874,
		Longitude: 37.3269004822,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTMT": {
		ICAO:      "HTMT",
		IATA:      "MYW",
		Name:      "Mtwara Airport",
		City:      "Mtwara",
		State:     "Mtwara",
		Country:   "TZ",
		Elevation: 371,
		Latitude:  -10.339099884,
		Longitude: 40.1818008423,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTMU": {
		ICAO:      "HTMU",
		IATA:      "MUZ",
		Name:      "Musoma Airport",
		City:      "Musoma",
		State:     "Mara",
		Country:   "TZ",
		Elevation: 3806,
		Latitude:  -1.503,
		Longitude: 33.8021,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTMW": {
		ICAO:      "HTMW",
		IATA:      "MWZ",
		Name:      "Mwanza Airport",
		City:      "Mwanza",
		State:     "Mwanza",
		Country:   "TZ",
		Elevation: 3763,
		Latitude:  -2.4444899559,
		Longitude: 32.9327011108,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTMX": {
		ICAO:      "HTMX",
		Name:      "Mpwapwa Airport",
		City:      "Mpwapwa",
		State:     "Dodoma",
		Country:   "TZ",
		Elevation: 3240,
		Latitude:  -6.3499999046,
		Longitude: 36.483001709,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTNA": {
		ICAO:      "HTNA",
		IATA:      "NCH",
		Name:      "Nachingwea Airport",
		City:      "Nachingwea",
		State:     "Lindi",
		Country:   "TZ",
		Elevation: 1400,
		Latitude:  -10.3575000763,
		Longitude: 38.7792015076,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTNJ": {
		ICAO:      "HTNJ",
		IATA:      "JOM",
		Name:      "Njombe Airport",
		City:      "Njombe",
		State:     "Njombe",
		Country:   "TZ",
		Elevation: 6400,
		Latitude:  -9.3500003815,
		Longitude: 34.7999992371,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTNR": {
		ICAO:      "HTNR",
		Name:      "Ngara Airport",
		City:      "Ngara",
		State:     "Kagera",
		Country:   "TZ",
		Elevation: 5030,
		Latitude:  -2.5417900085,
		Longitude: 30.7031002045,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTOL": {
		ICAO:      "HTOL",
		Name:      "Oltipesi Airport",
		State:     "Manyara",
		Country:   "TZ",
		Elevation: 4668,
		Latitude:  -4.287732,
		Longitude: 36.616115,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTPE": {
		ICAO:      "HTPE",
		IATA:      "PMA",
		Name:      "Pemba Airport",
		City:      "Chake",
		State:     "Pemba-South",
		Country:   "TZ",
		Elevation: 80,
		Latitude:  -5.2572598457,
		Longitude: 39.8114013672,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTSD": {
		ICAO:      "HTSD",
		Name:      "Singida Airport",
		City:      "Singida",
		State:     "Singida",
		Country:   "TZ",
		Elevation: 5000,
		Latitude:  -4.8169999123,
		Longitude: 34.733001709,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTSE": {
		ICAO:      "HTSE",
		Name:      "Same Airport",
		City:      "Same",
		State:     "Kilimanjaro",
		Country:   "TZ",
		Elevation: 3000,
		Latitude:  -4.0830001831,
		Longitude: 37.716999054,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTSH": {
		ICAO:      "HTSH",
		Name:      "Mafinga Airport",
		City:      "Mafinga",
		State:     "Iringa",
		Country:   "TZ",
		Elevation: 6300,
		Latitude:  -8.3500003815,
		Longitude: 35.2999992371,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTSN": {
		ICAO:      "HTSN",
		IATA:      "SEU",
		Name:      "Seronera Airport",
		City:      "Seronera",
		State:     "Mara",
		Country:   "TZ",
		Elevation: 5080,
		Latitude:  -2.4580600262,
		Longitude: 34.8224983215,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTSO": {
		ICAO:      "HTSO",
		IATA:      "SGX",
		Name:      "Songea Airport",
		City:      "Songea",
		State:     "Ruvuma",
		Country:   "TZ",
		Elevation: 3445,
		Latitude:  -10.6829996109,
		Longitude: 35.5830001831,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTSU": {
		ICAO:      "HTSU",
		IATA:      "SUT",
		Name:      "Sumbawanga Airport",
		City:      "Sumbawanga",
		State:     "Rukwa",
		Country:   "TZ",
		Elevation: 5920,
		Latitude:  -7.9670000076,
		Longitude: 31.6669998169,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTSY": {
		ICAO:      "HTSY",
		IATA:      "SHY",
		Name:      "Shinyanga Airport",
		City:      "Shinyanga",
		State:     "Shinyanga",
		Country:   "TZ",
		Elevation: 3800,
		Latitude:  -3.6093,
		Longitude: 33.5035,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTTB": {
		ICAO:      "HTTB",
		IATA:      "TBO",
		Name:      "Tabora Airport",
		City:      "Tabora",
		State:     "Tabora",
		Country:   "TZ",
		Elevation: 3868,
		Latitude:  -5.0763897896,
		Longitude: 32.8333015442,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTTG": {
		ICAO:      "HTTG",
		IATA:      "TGT",
		Name:      "Tanga Airport",
		City:      "Tanga",
		State:     "Tanga",
		Country:   "TZ",
		Elevation: 129,
		Latitude:  -5.0923600197,
		Longitude: 39.0712013245,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTTU": {
		ICAO:      "HTTU",
		Name:      "Tunduru Airport",
		City:      "Tunduru",
		State:     "Ruvuma",
		Country:   "TZ",
		Elevation: 2200,
		Latitude:  -11.1000003815,
		Longitude: 37.3670005798,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTUK": {
		ICAO:      "HTUK",
		Name:      "Nansio Airport",
		City:      "Ukerewe",
		State:     "Mara",
		Country:   "TZ",
		Elevation: 4010,
		Latitude:  -2.0435299873,
		Longitude: 33.0643997192,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTUR": {
		ICAO:      "HTUR",
		Name:      "Urambo Airport",
		City:      "Urambo",
		State:     "Tabora",
		Country:   "TZ",
		Elevation: 3600,
		Latitude:  -5.0669999123,
		Longitude: 32.0499992371,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTUT": {
		ICAO:      "HTUT",
		Name:      "Utete Airport",
		City:      "Utete",
		State:     "Pwani",
		Country:   "TZ",
		Elevation: 208,
		Latitude:  -8.002579689,
		Longitude: 38.7616996765,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTUV": {
		ICAO:      "HTUV",
		Name:      "Uvinza Airport",
		City:      "Uvinza",
		State:     "Kigoma",
		Country:   "TZ",
		Elevation: 563,
		Latitude:  -5.0879502296,
		Longitude: 30.4060001373,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTWK": {
		ICAO:      "HTWK",
		Name:      "West Kilimanjaro Airport",
		City:      "West Kilimanjaro",
		State:     "Kilimanjaro",
		Country:   "TZ",
		Elevation: 4300,
		Latitude:  -3.0499999523,
		Longitude: 37,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HTZA": {
		ICAO:      "HTZA",
		IATA:      "ZNZ",
		Name:      "Zanzibar Airport",
		City:      "Kiembi Samaki",
		State:     "Zanzibar-Urban/West",
		Country:   "TZ",
		Elevation: 54,
		Latitude:  -6.2220201492,
		Longitude: 39.224899292,
		Timezone:  "Africa/Dar_es_Salaam",
	},
	"HUAJ": {
		ICAO:      "HUAJ",
		Name:      "Adjumani Airport",
		City:      "Adjumani",
		State:     "Northern-Region",
		Country:   "UG",
		Elevation: 2611,
		Latitude:  3.3392300606,
		Longitude: 31.7646999359,
		Timezone:  "Africa/Kampala",
	},
	"HUAR": {
		ICAO:      "HUAR",
		IATA:      "RUA",
		Name:      "Arua Airport",
		City:      "Arua",
		State:     "Northern-Region",
		Country:   "UG",
		Elevation: 3951,
		Latitude:  3.0499999523,
		Longitude: 30.9169998169,
		Timezone:  "Africa/Kampala",
	},
	"HUEN": {
		ICAO:      "HUEN",
		IATA:      "EBB",
		Name:      "Entebbe International Airport",
		City:      "Kampala",
		State:     "Central-Region",
		Country:   "UG",
		Elevation: 3782,
		Latitude:  0.0423859991,
		Longitude: 32.4435005188,
		Timezone:  "Africa/Kampala",
	},
	"HUGU": {
		ICAO:      "HUGU",
		IATA:      "ULU",
		Name:      "Gulu Airport",
		City:      "Gulu",
		State:     "Northern-Region",
		Country:   "UG",
		Elevation: 3510,
		Latitude:  2.805560112,
		Longitude: 32.2718009949,
		Timezone:  "Africa/Kampala",
	},
	"HUJI": {
		ICAO:      "HUJI",
		IATA:      "JIN",
		Name:      "Jinja Airport",
		City:      "Jinja",
		State:     "Eastern-Region",
		Country:   "UG",
		Elevation: 3855,
		Latitude:  0.4499999881,
		Longitude: 33.2000007629,
		Timezone:  "Africa/Kampala",
	},
	"HUKB": {
		ICAO:      "HUKB",
		Name:      "Kabale Airport",
		State:     "Western-Region",
		Country:   "UG",
		Elevation: 6000,
		Latitude:  -1.226111,
		Longitude: 29.96,
		Timezone:  "Africa/Kampala",
	},
	"HUKF": {
		ICAO:      "HUKF",
		IATA:      "KBG",
		Name:      "Kabalega Falls Airport",
		City:      "Kabalega Falls",
		State:     "Northern-Region",
		Country:   "UG",
		Elevation: 2365,
		Latitude:  2.326390028,
		Longitude: 31.497800827,
		Timezone:  "Africa/Kampala",
	},
	"HUKS": {
		ICAO:      "HUKS",
		IATA:      "KSE",
		Name:      "Kasese Airport",
		City:      "Kasese",
		State:     "Western-Region",
		Country:   "UG",
		Elevation: 3146,
		Latitude:  0.1829999983,
		Longitude: 30.1000003815,
		Timezone:  "Africa/Kampala",
	},
	"HULI": {
		ICAO:      "HULI",
		Name:      "Lira Airport",
		City:      "Lira",
		State:     "Northern-Region",
		Country:   "UG",
		Elevation: 3580,
		Latitude:  2.25,
		Longitude: 32.9169998169,
		Timezone:  "Africa/Kampala",
	},
	"HUMA": {
		ICAO:      "HUMA",
		IATA:      "MBQ",
		Name:      "Mbarara Airport",
		City:      "Mbarara",
		State:     "Western-Region",
		Country:   "UG",
		Elevation: 4600,
		Latitude:  -0.5552780032,
		Longitude: 30.5993995667,
		Timezone:  "Africa/Kampala",
	},
	"HUMI": {
		ICAO:      "HUMI",
		IATA:      "KCU",
		Name:      "Masindi Airport",
		City:      "Masindi",
		State:     "Western-Region",
		Country:   "UG",
		Elevation: 3850,
		Latitude:  1.7580599785,
		Longitude: 31.736700058,
		Timezone:  "Africa/Kampala",
	},
	"HUPA": {
		ICAO:      "HUPA",
		IATA:      "PAF",
		Name:      "Pakuba Airport",
		State:     "Western-Region",
		Country:   "UG",
		Latitude:  2.202222,
		Longitude: 31.554444,
		Timezone:  "Africa/Kampala",
	},
	"HUSO": {
		ICAO:      "HUSO",
		IATA:      "SRT",
		Name:      "Soroti Airport",
		City:      "Soroti",
		State:     "Eastern-Region",
		Country:   "UG",
		Elevation: 3697,
		Latitude:  1.7276899815,
		Longitude: 33.6227989197,
		Timezone:  "Africa/Kampala",
	},
	"HUTO": {
		ICAO:      "HUTO",
		IATA:      "TRY",
		Name:      "Tororo Airport",
		City:      "Tororo",
		State:     "Eastern-Region",
		Country:   "UG",
		Elevation: 3840,
		Latitude:  0.6830000281,
		Longitude: 34.1669998169,
		Timezone:  "Africa/Kampala",
	},
	"IA00": {
		ICAO:      "IA00",
		Name:      "Jukam's Landing Airport",
		City:      "Menlo",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1230,
		Latitude:  41.5200004578,
		Longitude: -94.362701416,
		Timezone:  "America/Chicago",
	},
	"IA01": {
		ICAO:      "IA01",
		Name:      "Ridge Airport",
		City:      "Oakland",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1300,
		Latitude:  41.351398468,
		Longitude: -95.4664001465,
		Timezone:  "America/Chicago",
	},
	"IA02": {
		ICAO:      "IA02",
		Name:      "Hoff Airport",
		City:      "Bellevue",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1025,
		Latitude:  42.2620010376,
		Longitude: -90.5801010132,
		Timezone:  "America/Chicago",
	},
	"IA03": {
		ICAO:      "IA03",
		Name:      "Lemons Airport",
		City:      "Cedar Falls",
		State:     "Iowa",
		Country:   "US",
		Elevation: 970,
		Latitude:  42.5222015381,
		Longitude: -92.4945983887,
		Timezone:  "America/Chicago",
	},
	"IA05": {
		ICAO:      "IA05",
		Name:      "Rathbun Lake Airport",
		City:      "Centerville",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1200,
		Latitude:  40.8516998291,
		Longitude: -92.8584976196,
		Timezone:  "America/Chicago",
	},
	"IA06": {
		ICAO:      "IA06",
		Name:      "Sig-Nor Airport",
		City:      "Sigourney",
		State:     "Iowa",
		Country:   "US",
		Elevation: 818,
		Latitude:  41.3833007813,
		Longitude: -92.1835021973,
		Timezone:  "America/Chicago",
	},
	"IA07": {
		ICAO:      "IA07",
		Name:      "Grismore Airport",
		City:      "Corydon",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1100,
		Latitude:  40.7667007446,
		Longitude: -93.300201416,
		Timezone:  "America/Chicago",
	},
	"IA08": {
		ICAO:      "IA08",
		Name:      "Riedesel Private Airport",
		City:      "Lohrville",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1150,
		Latitude:  42.2780990601,
		Longitude: -94.5516967773,
		Timezone:  "America/Chicago",
	},
	"IA11": {
		ICAO:      "IA11",
		Name:      "Too Short Airport",
		City:      "Indianola",
		State:     "Iowa",
		Country:   "US",
		Elevation: 980,
		Latitude:  41.2938995361,
		Longitude: -93.6654968262,
		Timezone:  "America/Chicago",
	},
	"IA12": {
		ICAO:      "IA12",
		Name:      "Lloyd's Field",
		City:      "Clear Lake",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1220,
		Latitude:  43.0971984863,
		Longitude: -93.4021987915,
		Timezone:  "America/Chicago",
	},
	"IA15": {
		ICAO:      "IA15",
		Name:      "Hawk Field",
		City:      "Hawkeye",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1115,
		Latitude:  42.8869018555,
		Longitude: -91.9360961914,
		Timezone:  "America/Chicago",
	},
	"IA16": {
		ICAO:      "IA16",
		Name:      "Picayune Airport",
		City:      "Iowa City",
		State:     "Iowa",
		Country:   "US",
		Elevation: 800,
		Latitude:  41.7083015442,
		Longitude: -91.5001983643,
		Timezone:  "America/Chicago",
	},
	"IA17": {
		ICAO:      "IA17",
		Name:      "Bluebird Airport",
		City:      "Janesville",
		State:     "Iowa",
		Country:   "US",
		Elevation: 924,
		Latitude:  42.6230010986,
		Longitude: -92.5012969971,
		Timezone:  "America/Chicago",
	},
	"IA18": {
		ICAO:      "IA18",
		Name:      "Poyner Airport",
		City:      "Jesup",
		State:     "Iowa",
		Country:   "US",
		Elevation: 960,
		Latitude:  42.4707984924,
		Longitude: -92.0885009766,
		Timezone:  "America/Chicago",
	},
	"IA21": {
		ICAO:      "IA21",
		Name:      "Schurr Airport",
		City:      "Leclaire",
		State:     "Iowa",
		Country:   "US",
		Elevation: 720,
		Latitude:  41.5999984741,
		Longitude: -90.3834991455,
		Timezone:  "America/Chicago",
	},
	"IA22": {
		ICAO:      "IA22",
		Name:      "Orr-Port Airport",
		City:      "New London",
		State:     "Iowa",
		Country:   "US",
		Elevation: 790,
		Latitude:  40.9384002686,
		Longitude: -91.403503418,
		Timezone:  "America/Chicago",
	},
	"IA23": {
		ICAO:      "IA23",
		Name:      "Gaa Private Airport",
		City:      "Guttenberg",
		State:     "Iowa",
		Country:   "US",
		Elevation: 620,
		Latitude:  42.8088989258,
		Longitude: -91.098197937,
		Timezone:  "America/Chicago",
	},
	"IA24": {
		ICAO:      "IA24",
		Name:      "Green Castle Airport",
		City:      "Oxford",
		State:     "Iowa",
		Country:   "US",
		Elevation: 750,
		Latitude:  41.7550010681,
		Longitude: -91.7276992798,
		Timezone:  "America/Chicago",
	},
	"IA25": {
		ICAO:      "IA25",
		Name:      "Bartlett Field",
		City:      "Solon",
		State:     "Iowa",
		Country:   "US",
		Elevation: 820,
		Latitude:  41.8005981445,
		Longitude: -91.4399032593,
		Timezone:  "America/Chicago",
	},
	"IA26": {
		ICAO:      "IA26",
		Name:      "Carter Field",
		City:      "Stockport",
		State:     "Iowa",
		Country:   "US",
		Elevation: 750,
		Latitude:  40.8083992004,
		Longitude: -91.8085021973,
		Timezone:  "America/Chicago",
	},
	"IA27": {
		ICAO:      "IA27",
		Name:      "Antique Airfield",
		City:      "Blakesburg",
		State:     "Iowa",
		Country:   "US",
		Elevation: 890,
		Latitude:  40.9777984619,
		Longitude: -92.5876998901,
		Timezone:  "America/Chicago",
	},
	"IA28": {
		ICAO:      "IA28",
		Name:      "Sherman Airport",
		City:      "Hiawatha",
		State:     "Iowa",
		Country:   "US",
		Elevation: 860,
		Latitude:  42.0583000183,
		Longitude: -91.6902008057,
		Timezone:  "America/Chicago",
	},
	"IA30": {
		ICAO:      "IA30",
		Name:      "Kleis Airport",
		City:      "Zwingle",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1050,
		Latitude:  42.2999992371,
		Longitude: -90.7500991821,
		Timezone:  "America/Chicago",
	},
	"IA31": {
		ICAO:      "IA31",
		Name:      "Beeds Lake Airport",
		City:      "Hampton",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1148,
		Latitude:  42.7666015625,
		Longitude: -93.2460021973,
		Timezone:  "America/Chicago",
	},
	"IA32": {
		ICAO:      "IA32",
		Name:      "Pierson Field",
		City:      "Oskaloosa",
		State:     "Iowa",
		Country:   "US",
		Elevation: 845,
		Latitude:  41.3250007629,
		Longitude: -92.7171020508,
		Timezone:  "America/Chicago",
	},
	"IA33": {
		ICAO:      "IA33",
		Name:      "Dad's Field",
		City:      "Rockwell",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1200,
		Latitude:  43.0119018555,
		Longitude: -93.242401123,
		Timezone:  "America/Chicago",
	},
	"IA35": {
		ICAO:      "IA35",
		Name:      "Ruckl Airport",
		City:      "Council Bluffs",
		State:     "Iowa",
		Country:   "US",
		Elevation: 970,
		Latitude:  41.1871986389,
		Longitude: -95.9041976929,
		Timezone:  "America/Chicago",
	},
	"IA36": {
		ICAO:      "IA36",
		Name:      "Mercer Field",
		City:      "Creston",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1320,
		Latitude:  41.0750007629,
		Longitude: -94.426902771,
		Timezone:  "America/Chicago",
	},
	"IA38": {
		ICAO:      "IA38",
		Name:      "De Louis Field",
		City:      "Des Moines",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1250,
		Latitude:  41.5480995178,
		Longitude: -93.4291992188,
		Timezone:  "America/Chicago",
	},
	"IA41": {
		ICAO:      "IA41",
		Name:      "Laverty Field",
		City:      "Indianola",
		State:     "Iowa",
		Country:   "US",
		Elevation: 950,
		Latitude:  41.4152984619,
		Longitude: -93.5613021851,
		Timezone:  "America/Chicago",
	},
	"IA44": {
		ICAO:      "IA44",
		Name:      "Solly's Strip Ultralightport",
		City:      "Lamoni",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1150,
		Latitude:  40.6514015198,
		Longitude: -93.9608001709,
		Timezone:  "America/Chicago",
	},
	"IA45": {
		ICAO:      "IA45",
		Name:      "Willie's Bomber Patch Airport",
		City:      "Marcus",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1446,
		Latitude:  42.7750015259,
		Longitude: -95.8460998535,
		Timezone:  "America/Chicago",
	},
	"IA47": {
		ICAO:      "IA47",
		Name:      "Day Field",
		City:      "Grimes",
		State:     "Iowa",
		Country:   "US",
		Elevation: 980,
		Latitude:  41.6402015686,
		Longitude: -93.8087997437,
		Timezone:  "America/Chicago",
	},
	"IA48": {
		ICAO:      "IA48",
		Name:      "Kern Field",
		City:      "Polk City",
		State:     "Iowa",
		Country:   "US",
		Elevation: 900,
		Latitude:  41.7416992188,
		Longitude: -93.7251968384,
		Timezone:  "America/Chicago",
	},
	"IA49": {
		ICAO:      "IA49",
		Name:      "Jirak Airport",
		City:      "Fort Atkinson",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1180,
		Latitude:  43.1450004578,
		Longitude: -92.0534973145,
		Timezone:  "America/Chicago",
	},
	"IA50": {
		ICAO:      "IA50",
		Name:      "Sigourney Iowa Airport",
		City:      "Sigourney",
		State:     "Iowa",
		Country:   "US",
		Elevation: 697,
		Latitude:  41.2907981873,
		Longitude: -92.18800354,
		Timezone:  "America/Chicago",
	},
	"IA51": {
		ICAO:      "IA51",
		Name:      "De Soto Airport",
		City:      "De Soto",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1000,
		Latitude:  41.5416984558,
		Longitude: -94.0085983276,
		Timezone:  "America/Chicago",
	},
	"IA52": {
		ICAO:      "IA52",
		Name:      "Freedom Field",
		City:      "Estherville",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1350,
		Latitude:  43.4430007935,
		Longitude: -94.8479995728,
		Timezone:  "America/Chicago",
	},
	"IA54": {
		ICAO:      "IA54",
		Name:      "Anderson Airport",
		City:      "Dubuque",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1135,
		Latitude:  42.5022010803,
		Longitude: -90.8432006836,
		Timezone:  "America/Chicago",
	},
	"IA56": {
		ICAO:      "IA56",
		Name:      "Farrar Airport",
		City:      "Farrar",
		State:     "Iowa",
		Country:   "US",
		Elevation: 916,
		Latitude:  41.7804985046,
		Longitude: -93.3794021606,
		Timezone:  "America/Chicago",
	},
	"IA58": {
		ICAO:      "IA58",
		Name:      "B-8 Farms Airport",
		City:      "Whiting",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1063,
		Latitude:  42.1150016785,
		Longitude: -96.1719970703,
		Timezone:  "America/Chicago",
	},
	"IA59": {
		ICAO:      "IA59",
		Name:      "Bickel STOLport",
		City:      "Keokuk",
		State:     "Iowa",
		Country:   "US",
		Elevation: 680,
		Latitude:  40.3917007446,
		Longitude: -91.4168014526,
		Timezone:  "America/Chicago",
	},
	"IA61": {
		ICAO:      "IA61",
		Name:      "Estle Field",
		City:      "Fairfield",
		State:     "Iowa",
		Country:   "US",
		Elevation: 800,
		Latitude:  41.0802993774,
		Longitude: -91.9466018677,
		Timezone:  "America/Chicago",
	},
	"IA62": {
		ICAO:      "IA62",
		Name:      "Wheatley Farms Airport",
		City:      "Elkhorn",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1310,
		Latitude:  41.5332984924,
		Longitude: -95.075302124,
		Timezone:  "America/Chicago",
	},
	"IA63": {
		ICAO:      "IA63",
		Name:      "Edgren Airport",
		City:      "Oskaloosa",
		State:     "Iowa",
		Country:   "US",
		Elevation: 840,
		Latitude:  41.2583007813,
		Longitude: -92.5515975952,
		Timezone:  "America/Chicago",
	},
	"IA65": {
		ICAO:      "IA65",
		Name:      "Donnellson Airport",
		City:      "Donnellson",
		State:     "Iowa",
		Country:   "US",
		Elevation: 700,
		Latitude:  40.6208992004,
		Longitude: -91.5418014526,
		Timezone:  "America/Chicago",
	},
	"IA66": {
		ICAO:      "IA66",
		Name:      "Nash Field Indianola Airport",
		City:      "Indianola",
		State:     "Iowa",
		Country:   "US",
		Elevation: 991,
		Latitude:  41.3039016724,
		Longitude: -93.5677032471,
		Timezone:  "America/Chicago",
	},
	"IA68": {
		ICAO:      "IA68",
		Name:      "Freedom Field",
		City:      "La Motte",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1055,
		Latitude:  42.2667007446,
		Longitude: -90.6007003784,
		Timezone:  "America/Chicago",
	},
	"IA69": {
		ICAO:      "IA69",
		Name:      "Sand Field Airport",
		City:      "Marcus",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1520,
		Latitude:  42.8736000061,
		Longitude: -95.8031005859,
		Timezone:  "America/Chicago",
	},
	"IA70": {
		ICAO:      "IA70",
		Name:      "Friesenborg & Larson Airport",
		City:      "Buffalo Center",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1220,
		Latitude:  43.4258003235,
		Longitude: -93.8358001709,
		Timezone:  "America/Chicago",
	},
	"IA76": {
		ICAO:      "IA76",
		Name:      "Cobb Farm Airport",
		City:      "Corydon",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1100,
		Latitude:  40.728099823,
		Longitude: -93.3432998657,
		Timezone:  "America/Chicago",
	},
	"IA77": {
		ICAO:      "IA77",
		Name:      "Keitzer Field",
		City:      "Mediapolis",
		State:     "Iowa",
		Country:   "US",
		Elevation: 755,
		Latitude:  41.0264015198,
		Longitude: -91.1417999268,
		Timezone:  "America/Chicago",
	},
	"IA80": {
		ICAO:      "IA80",
		Name:      "Dyersville Area Airport",
		City:      "Dyersville",
		State:     "Iowa",
		Country:   "US",
		Elevation: 980,
		Latitude:  42.4961013794,
		Longitude: -91.179901123,
		Timezone:  "America/Chicago",
	},
	"IA83": {
		ICAO:      "IA83",
		Name:      "Kohlhaas Airport",
		City:      "Livermore",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1153,
		Latitude:  42.9538993835,
		Longitude: -94.1677017212,
		Timezone:  "America/Chicago",
	},
	"IA84": {
		ICAO:      "IA84",
		Name:      "Lawton Airport",
		City:      "Lawton",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1235,
		Latitude:  42.4832992554,
		Longitude: -96.200302124,
		Timezone:  "America/Chicago",
	},
	"IA85": {
		ICAO:      "IA85",
		Name:      "Tuinstra Airfield",
		City:      "Indianola",
		State:     "Iowa",
		Country:   "US",
		Elevation: 900,
		Latitude:  41.4351005554,
		Longitude: -93.5404968262,
		Timezone:  "America/Chicago",
	},
	"IA86": {
		ICAO:      "IA86",
		Name:      "Hedgewood Landing Airport",
		City:      "Indianola",
		State:     "Iowa",
		Country:   "US",
		Elevation: 990,
		Latitude:  41.2667007446,
		Longitude: -93.5093994141,
		Timezone:  "America/Chicago",
	},
	"IA87": {
		ICAO:      "IA87",
		Name:      "Robel Field",
		City:      "Grimes",
		State:     "Iowa",
		Country:   "US",
		Elevation: 965,
		Latitude:  41.6575012207,
		Longitude: -93.8210983276,
		Timezone:  "America/Chicago",
	},
	"IA93": {
		ICAO:      "IA93",
		Name:      "Olsen Airport",
		City:      "Missouri Valley",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1030,
		Latitude:  41.6128005981,
		Longitude: -95.8955993652,
		Timezone:  "America/Chicago",
	},
	"IA94": {
		ICAO:      "IA94",
		Name:      "Nesler Field",
		City:      "Moorland",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1147,
		Latitude:  42.4333000183,
		Longitude: -94.2835998535,
		Timezone:  "America/Chicago",
	},
	"IA97": {
		ICAO:      "IA97",
		Name:      "Nichols Airport",
		City:      "La Porte City",
		State:     "Iowa",
		Country:   "US",
		Elevation: 950,
		Latitude:  42.3166999817,
		Longitude: -92.2835006714,
		Timezone:  "America/Chicago",
	},
	"ID00": {
		ICAO:      "ID00",
		Name:      "Hubler Field",
		City:      "Caldwell",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2385,
		Latitude:  43.6957015991,
		Longitude: -116.6380004883,
		Timezone:  "America/Boise",
	},
	"ID01": {
		ICAO:      "ID01",
		Name:      "Cx Ranch Airport",
		City:      "Clark Fork",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2071,
		Latitude:  48.1380004883,
		Longitude: -116.1979980469,
		Timezone:  "America/Los_Angeles",
	},
	"ID04": {
		ICAO:      "ID04",
		Name:      "J-Lazy-M Ranch Airport",
		City:      "Fish Haven",
		State:     "Idaho",
		Country:   "US",
		Elevation: 6190,
		Latitude:  42.0290985107,
		Longitude: -111.422996521,
		Timezone:  "America/Boise",
	},
	"ID05": {
		ICAO:      "ID05",
		Name:      "Hackney Airpark",
		City:      "Athol",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2445,
		Latitude:  47.9567985535,
		Longitude: -116.6770019531,
		Timezone:  "America/Los_Angeles",
	},
	"ID06": {
		ICAO:      "ID06",
		Name:      "Ranch Aero Airport",
		City:      "Hayden Lake",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2315,
		Latitude:  47.8288002014,
		Longitude: -116.783996582,
		Timezone:  "America/Los_Angeles",
	},
	"ID07": {
		ICAO:      "ID07",
		Name:      "Nichols Ranch Airport",
		City:      "Post Falls",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2430,
		Latitude:  47.6781005859,
		Longitude: -117.0250015259,
		Timezone:  "America/Los_Angeles",
	},
	"ID09": {
		ICAO:      "ID09",
		Name:      "Otterson Ranch Airport",
		City:      "Post Falls",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2150,
		Latitude:  47.7402000427,
		Longitude: -116.9980010986,
		Timezone:  "America/Los_Angeles",
	},
	"ID10": {
		ICAO:      "ID10",
		Name:      "Anderson-Plummer Airport",
		City:      "Potlatch",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2580,
		Latitude:  46.9243011475,
		Longitude: -116.9580001831,
		Timezone:  "America/Los_Angeles",
	},
	"ID11": {
		ICAO:      "ID11",
		Name:      "Wood Brothers Ranch Airport",
		City:      "Reubens",
		State:     "Idaho",
		Country:   "US",
		Elevation: 3400,
		Latitude:  46.3820991516,
		Longitude: -116.5130004883,
		Timezone:  "America/Los_Angeles",
	},
	"ID12": {
		ICAO:      "ID12",
		Name:      "Russell W Anderson Strip",
		City:      "Riverside",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4450,
		Latitude:  43.1843986511,
		Longitude: -112.466003418,
		Timezone:  "America/Boise",
	},
	"ID13": {
		ICAO:      "ID13",
		Name:      "Sky Island Ranch Airport",
		City:      "St Maries",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2880,
		Latitude:  47.322101593,
		Longitude: -116.6399993896,
		Timezone:  "America/Los_Angeles",
	},
	"ID16": {
		ICAO:      "ID16",
		Name:      "Sluder Airstrip",
		City:      "Bellevue",
		State:     "Idaho",
		Country:   "US",
		Elevation: 5017,
		Latitude:  43.4034996033,
		Longitude: -114.2730026245,
		Timezone:  "America/Boise",
	},
	"ID17": {
		ICAO:      "ID17",
		Name:      "Seven Devils Airport",
		City:      "Council",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4487,
		Latitude:  45.0110015869,
		Longitude: -116.6890029907,
		Timezone:  "America/Boise",
	},
	"ID19": {
		ICAO:      "ID19",
		Name:      "Bird Nr 2 Airport",
		City:      "Sandpoint",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2192,
		Latitude:  48.2309989929,
		Longitude: -116.388999939,
		Timezone:  "America/Los_Angeles",
	},
	"ID21": {
		ICAO:      "ID21",
		Name:      "Smith Ranch Airport",
		City:      "Hauser Lake",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2370,
		Latitude:  47.7563018799,
		Longitude: -117.0240020752,
		Timezone:  "America/Los_Angeles",
	},
	"ID22": {
		ICAO:      "ID22",
		Name:      "Treeport Airport",
		City:      "Spirit Lake",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2500,
		Latitude:  47.9780006409,
		Longitude: -116.7929992676,
		Timezone:  "America/Los_Angeles",
	},
	"ID23": {
		ICAO:      "ID23",
		Name:      "Rock Creek Farm Airport",
		City:      "Worley",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2500,
		Latitude:  47.3923988342,
		Longitude: -116.8730010986,
		Timezone:  "America/Los_Angeles",
	},
	"ID24": {
		ICAO:      "ID24",
		Name:      "Timber Basin Airpark Inc Airport",
		City:      "Sagle",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2220,
		Latitude:  48.2234992981,
		Longitude: -116.4390029907,
		Timezone:  "America/Los_Angeles",
	},
	"ID25": {
		ICAO:      "ID25",
		Name:      "Olmstead Sky Ranch Airport",
		City:      "Sandpoint",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2140,
		Latitude:  48.3507995605,
		Longitude: -116.5540008545,
		Timezone:  "America/Los_Angeles",
	},
	"ID26": {
		ICAO:      "ID26",
		Name:      "P And R Field",
		City:      "Mayfield",
		State:     "Idaho",
		Country:   "US",
		Elevation: 3125,
		Latitude:  43.224167,
		Longitude: -115.966944,
		Timezone:  "America/Boise",
	},
	"ID27": {
		ICAO:      "ID27",
		Name:      "Hawk Haven Airport",
		City:      "Coeur d Alene",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2333,
		Latitude:  47.75550079,
		Longitude: -116.8590012,
		Timezone:  "America/Los_Angeles",
	},
	"ID28": {
		ICAO:      "ID28",
		Name:      "Mackay Bar Airport",
		City:      "Dixie",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2172,
		Latitude:  45.3791007996,
		Longitude: -115.5049972534,
		Timezone:  "America/Boise",
	},
	"ID29": {
		ICAO:      "ID29",
		Name:      "Big Island Airport",
		City:      "Orofino",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2249,
		Latitude:  46.6973991394,
		Longitude: -115.9840011597,
		Timezone:  "America/Los_Angeles",
	},
	"ID32": {
		ICAO:      "ID32",
		Name:      "Tuka STOLport",
		City:      "Clark Fork",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2086,
		Latitude:  48.1054992676,
		Longitude: -116.1370010376,
		Timezone:  "America/Los_Angeles",
	},
	"ID33": {
		ICAO:      "ID33",
		Name:      "Stocking Meadows Airport",
		City:      "Clarkia",
		State:     "Idaho",
		Country:   "US",
		Elevation: 3850,
		Latitude:  46.936000824,
		Longitude: -115.8649978638,
		Timezone:  "America/Los_Angeles",
	},
	"ID34": {
		ICAO:      "ID34",
		Name:      "Granite Airport",
		City:      "Athol",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2260,
		Latitude:  47.9846000671,
		Longitude: -116.6829986572,
		Timezone:  "America/Los_Angeles",
	},
	"ID35": {
		ICAO:      "ID35",
		Name:      "High Valley Swanson Airport",
		City:      "Cascade",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4883,
		Latitude:  44.2391014099,
		Longitude: -116.1429977417,
		Timezone:  "America/Boise",
	},
	"ID36": {
		ICAO:      "ID36",
		Name:      "King Mountain Glider Park",
		City:      "Moore",
		State:     "Idaho",
		Country:   "US",
		Elevation: 5500,
		Latitude:  43.7630556,
		Longitude: -113.3438889,
		Timezone:  "America/Boise",
	},
	"ID39": {
		ICAO:      "ID39",
		Name:      "Owen Ranches Inc Airport",
		City:      "Bruneau",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2620,
		Latitude:  42.7957000732,
		Longitude: -115.7340011597,
		Timezone:  "America/Boise",
	},
	"ID40": {
		ICAO:      "ID40",
		Name:      "Sunrise Skypark Airport",
		City:      "Marsing",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2240,
		Latitude:  43.4178009033,
		Longitude: -116.7060012817,
		Timezone:  "America/Boise",
	},
	"ID41": {
		ICAO:      "ID41",
		Name:      "Stibnite Airport",
		City:      "Yellow Pine",
		State:     "Idaho",
		Country:   "US",
		Elevation: 6539,
		Latitude:  44.8998985291,
		Longitude: -115.3330001831,
		Timezone:  "America/Boise",
	},
	"ID43": {
		ICAO:      "ID43",
		Name:      "Carlin Bay Airport",
		City:      "Coeur D'Alene",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2702,
		Latitude:  47.5513000488,
		Longitude: -116.763999939,
		Timezone:  "America/Los_Angeles",
	},
	"ID44": {
		ICAO:      "ID44",
		Name:      "Hidden Lakes Airport",
		City:      "Cascade",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4845,
		Latitude:  44.224899292,
		Longitude: -116.1800003052,
		Timezone:  "America/Boise",
	},
	"ID48": {
		ICAO:      "ID48",
		Name:      "Western Spur Airport",
		City:      "Athol",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2350,
		Latitude:  47.9287986755,
		Longitude: -116.7109985352,
		Timezone:  "America/Los_Angeles",
	},
	"ID50": {
		ICAO:      "ID50",
		Name:      "Q.B. One Airport",
		City:      "Idaho Falls",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4875,
		Latitude:  43.6012992859,
		Longitude: -112.2429962158,
		Timezone:  "America/Boise",
	},
	"ID52": {
		ICAO:      "ID52",
		Name:      "Bowman Field",
		City:      "Chester",
		State:     "Idaho",
		Country:   "US",
		Elevation: 5100,
		Latitude:  43.9930000305,
		Longitude: -111.5510025024,
		Timezone:  "America/Boise",
	},
	"ID56": {
		ICAO:      "ID56",
		Name:      "Star 's' Ranch Airport",
		City:      "Mackay",
		State:     "Idaho",
		Country:   "US",
		Elevation: 6660,
		Latitude:  43.9818992615,
		Longitude: -114.0439987183,
		Timezone:  "America/Boise",
	},
	"ID59": {
		ICAO:      "ID59",
		Name:      "Flying A Ranch Airport",
		City:      "Lake Fork",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4918,
		Latitude:  44.8176002502,
		Longitude: -116.0680007935,
		Timezone:  "America/Boise",
	},
	"ID60": {
		ICAO:      "ID60",
		Name:      "Fountains Airport",
		City:      "Moscow",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2550,
		Latitude:  46.7131996155,
		Longitude: -116.9970016479,
		Timezone:  "America/Los_Angeles",
	},
	"ID61": {
		ICAO:      "ID61",
		Name:      "Valenov Ranch Airport",
		City:      "Priest River",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2425,
		Latitude:  48.2966003418,
		Longitude: -117.0149993896,
		Timezone:  "America/Los_Angeles",
	},
	"ID62": {
		ICAO:      "ID62",
		Name:      "Simpson Airport",
		City:      "Grace",
		State:     "Idaho",
		Country:   "US",
		Elevation: 5598,
		Latitude:  42.6082992554,
		Longitude: -111.7289962769,
		Timezone:  "America/Boise",
	},
	"ID63": {
		ICAO:      "ID63",
		Name:      "Richards Airport",
		City:      "Homedale",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2290,
		Latitude:  43.6710014343,
		Longitude: -117.0250015259,
		Timezone:  "America/Boise",
	},
	"ID65": {
		ICAO:      "ID65",
		Name:      "Pisch's Place Airport",
		City:      "Coeur D'Alene",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2775,
		Latitude:  47.5810012817,
		Longitude: -116.7720031738,
		Timezone:  "America/Los_Angeles",
	},
	"ID67": {
		ICAO:      "ID67",
		Name:      "Lower Loon Creek Airport",
		City:      "Challis",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4200,
		Latitude:  44.8082008362,
		Longitude: -114.8089981079,
		Timezone:  "America/Boise",
	},
	"ID68": {
		ICAO:      "ID68",
		Name:      "Green Acres Airport",
		City:      "Kuna",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2865,
		Latitude:  43.4846000671,
		Longitude: -116.4589996338,
		Timezone:  "America/Boise",
	},
	"ID72": {
		ICAO:      "ID72",
		Name:      "Huskey Airport",
		City:      "Irwin",
		State:     "Idaho",
		Country:   "US",
		Elevation: 5200,
		Latitude:  43.4174003601,
		Longitude: -111.2850036621,
		Timezone:  "America/Boise",
	},
	"ID74": {
		ICAO:      "ID74",
		Name:      "Sulphur Creek Ranch Airport",
		City:      "Cascade",
		State:     "Idaho",
		Country:   "US",
		Elevation: 5835,
		Latitude:  44.5365982056,
		Longitude: -115.3509979248,
		Timezone:  "America/Boise",
	},
	"ID75": {
		ICAO:      "ID75",
		Name:      "Riverlake Airport",
		City:      "Clark Fork",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2076,
		Latitude:  48.1259994507,
		Longitude: -116.1650009155,
		Timezone:  "America/Los_Angeles",
	},
	"ID76": {
		ICAO:      "ID76",
		Name:      "Wilson Bar US Forest Service Airport",
		City:      "Dixie",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2250,
		Latitude:  45.3967018127,
		Longitude: -115.483001709,
		Timezone:  "America/Boise",
	},
	"ID77": {
		ICAO:      "ID77",
		Name:      "Cuddy Meadows Airport",
		City:      "Cambridge",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4580,
		Latitude:  44.7052993774,
		Longitude: -116.8069992065,
		Timezone:  "America/Boise",
	},
	"ID78": {
		ICAO:      "ID78",
		Name:      "Cx Ranch Nr 2 Airport",
		City:      "Clark Fork",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2070,
		Latitude:  48.1416015625,
		Longitude: -116.1819992065,
		Timezone:  "America/Los_Angeles",
	},
	"ID79": {
		ICAO:      "ID79",
		Name:      "Sky Ranch South Airport",
		City:      "Nampa",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2750,
		Latitude:  43.5083007812,
		Longitude: -116.667999268,
		Timezone:  "America/Boise",
	},
	"ID82": {
		ICAO:      "ID82",
		Name:      "Picabo Airport",
		City:      "Picabo",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4828,
		Latitude:  43.3082008362,
		Longitude: -114.06300354,
		Timezone:  "America/Boise",
	},
	"ID84": {
		ICAO:      "ID84",
		Name:      "Cptpa Headquarters Airport",
		City:      "Headquarters",
		State:     "Idaho",
		Country:   "US",
		Elevation: 3314,
		Latitude:  46.617099762,
		Longitude: -115.8000030518,
		Timezone:  "America/Los_Angeles",
	},
	"ID85": {
		ICAO:      "ID85",
		Name:      "Elk River Airport",
		City:      "Elk River",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2827,
		Latitude:  46.787399292,
		Longitude: -116.1679992676,
		Timezone:  "America/Los_Angeles",
	},
	"ID86": {
		ICAO:      "ID86",
		Name:      "Deadwood Dam Airstrip",
		City:      "Cascade",
		State:     "Idaho",
		Country:   "US",
		Elevation: 5489,
		Latitude:  44.2977981567,
		Longitude: -115.6409988403,
		Timezone:  "America/Boise",
	},
	"ID87": {
		ICAO:      "ID87",
		Name:      "Rainbow Ranch Airport",
		City:      "Idaho Falls",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4750,
		Latitude:  43.4068984985,
		Longitude: -111.9739990234,
		Timezone:  "America/Boise",
	},
	"ID88": {
		ICAO:      "ID88",
		Name:      "Tracy Ranch Airport",
		City:      "Hill City",
		State:     "Idaho",
		Country:   "US",
		Elevation: 5071,
		Latitude:  43.287399292,
		Longitude: -115.0800018311,
		Timezone:  "America/Boise",
	},
	"ID90": {
		ICAO:      "ID90",
		Name:      "Spencer Ranch Landing Strip",
		City:      "Keuterville",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4284,
		Latitude:  45.8387985229,
		Longitude: -116.658996582,
		Timezone:  "America/Boise",
	},
	"ID92": {
		ICAO:      "ID92",
		Name:      "Snake River Skydiving Airport",
		City:      "Star",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2550,
		Latitude:  43.7288017273,
		Longitude: -116.5230026245,
		Timezone:  "America/Boise",
	},
	"ID93": {
		ICAO:      "ID93",
		Name:      "Reed Ranch Airport",
		City:      "Yellow Pine",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4153,
		Latitude:  44.8942985535,
		Longitude: -115.7129974365,
		Timezone:  "America/Boise",
	},
	"ID94": {
		ICAO:      "ID94",
		Name:      "Friendly Persuasion Farm Airport",
		City:      "Troy",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2860,
		Latitude:  46.7766990662,
		Longitude: -116.6910018921,
		Timezone:  "America/Los_Angeles",
	},
	"ID95": {
		ICAO:      "ID95",
		Name:      "Loomis Airport",
		City:      "Horseshoe Bend",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4180,
		Latitude:  43.861000061,
		Longitude: -116.2409973145,
		Timezone:  "America/Boise",
	},
	"ID96": {
		ICAO:      "ID96",
		Name:      "Bear Air Airport",
		City:      "Donnelly",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4904,
		Latitude:  44.7851982117,
		Longitude: -116.0670013428,
		Timezone:  "America/Boise",
	},
	"ID97": {
		ICAO:      "ID97",
		Name:      "Hangman Creek Ranch Airport",
		City:      "De Smet",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2635,
		Latitude:  47.1071014404,
		Longitude: -116.8109970093,
		Timezone:  "America/Los_Angeles",
	},
	"IG02": {
		ICAO:      "IG02",
		Name:      "Ratcliff Airport",
		City:      "Lafayette",
		State:     "Indiana",
		Country:   "US",
		Elevation: 730,
		Latitude:  40.2731018066,
		Longitude: -86.9075012207,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IG03": {
		ICAO:      "IG03",
		Name:      "Bluebird Airport",
		City:      "Paragon",
		State:     "Indiana",
		Country:   "US",
		Elevation: 580,
		Latitude:  39.3917007446,
		Longitude: -86.5999984741,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IG04": {
		ICAO:      "IG04",
		Name:      "Litzinger Ultralightport",
		City:      "Versailles",
		State:     "Indiana",
		Country:   "US",
		Elevation: 970,
		Latitude:  39.0461006165,
		Longitude: -85.1567001343,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IG05": {
		ICAO:      "IG05",
		Name:      "Wheeler Airport",
		City:      "Knox",
		State:     "Indiana",
		Country:   "US",
		Elevation: 720,
		Latitude:  41.189763,
		Longitude: -86.606956,
		Timezone:  "America/Indiana/Knox",
	},
	"IG07": {
		ICAO:      "IG07",
		Name:      "Josephs Field",
		City:      "Moores Hill",
		State:     "Indiana",
		Country:   "US",
		Elevation: 860,
		Latitude:  39.0569000244,
		Longitude: -85.0355987549,
		Timezone:  "America/New_York",
	},
	"II01": {
		ICAO:      "II01",
		Name:      "Grandlienard-Hogg Airport",
		City:      "Bluffton",
		State:     "Indiana",
		Country:   "US",
		Elevation: 830,
		Latitude:  40.7561988831,
		Longitude: -85.25050354,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II02": {
		ICAO:      "II02",
		Name:      "Kline Field",
		City:      "Butler",
		State:     "Indiana",
		Country:   "US",
		Elevation: 825,
		Latitude:  41.3856010437,
		Longitude: -84.8268966675,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II03": {
		ICAO:      "II03",
		Name:      "Buell Airport",
		City:      "Milan",
		State:     "Indiana",
		Country:   "US",
		Elevation: 990,
		Latitude:  39.198600769,
		Longitude: -85.1363983154,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II04": {
		ICAO:      "II04",
		Name:      "North West Indiana Air Airport",
		City:      "Brook",
		State:     "Indiana",
		Country:   "US",
		Elevation: 660,
		Latitude:  40.8838996887,
		Longitude: -87.3692016602,
		Timezone:  "America/Chicago",
	},
	"II05": {
		ICAO:      "II05",
		Name:      "Woodcock Airport",
		City:      "Akron",
		State:     "Indiana",
		Country:   "US",
		Elevation: 858,
		Latitude:  41.0741996765,
		Longitude: -86.064201355,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II06": {
		ICAO:      "II06",
		Name:      "Ferrell Airport",
		City:      "Cadiz",
		State:     "Indiana",
		Country:   "US",
		Elevation: 1070,
		Latitude:  39.9700012207,
		Longitude: -85.4708023071,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II07": {
		ICAO:      "II07",
		Name:      "Clover Knoll Airport",
		City:      "Cloverdale",
		State:     "Indiana",
		Country:   "US",
		Elevation: 880,
		Latitude:  39.5322990417,
		Longitude: -86.8321990967,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II08": {
		ICAO:      "II08",
		Name:      "Rheude Airport",
		City:      "Earl Park",
		State:     "Indiana",
		Country:   "US",
		Elevation: 745,
		Latitude:  40.660900116,
		Longitude: -87.4396972656,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II09": {
		ICAO:      "II09",
		Name:      "Patrum Field",
		City:      "Eminence",
		State:     "Indiana",
		Country:   "US",
		Elevation: 760,
		Latitude:  39.5736999512,
		Longitude: -86.6181030273,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II10": {
		ICAO:      "II10",
		Name:      "Rockey's Air Strip",
		City:      "Galveston",
		State:     "Indiana",
		Country:   "US",
		Elevation: 810,
		Latitude:  40.5820007324,
		Longitude: -86.1675033569,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II11": {
		ICAO:      "II11",
		Name:      "Pelz Port Airport",
		City:      "Grabill",
		State:     "Indiana",
		Country:   "US",
		Elevation: 825,
		Latitude:  41.2106018066,
		Longitude: -84.9860992432,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II12": {
		ICAO:      "II12",
		Name:      "Dague Strip",
		City:      "Grass Creek",
		State:     "Indiana",
		Country:   "US",
		Elevation: 790,
		Latitude:  40.9645004272,
		Longitude: -86.3719024658,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II13": {
		ICAO:      "II13",
		Name:      "Sugar Creek Air Park",
		City:      "Markleville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 1000,
		Latitude:  39.920278,
		Longitude: -85.598611,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II14": {
		ICAO:      "II14",
		Name:      "Beck Private Airport",
		City:      "Huntington",
		State:     "Indiana",
		Country:   "US",
		Elevation: 840,
		Latitude:  40.9612007141,
		Longitude: -85.5625,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II15": {
		ICAO:      "II15",
		Name:      "Friedrich Airport",
		City:      "Kewanna",
		State:     "Indiana",
		Country:   "US",
		Elevation: 770,
		Latitude:  40.9936981201,
		Longitude: -86.3416976929,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II16": {
		ICAO:      "II16",
		Name:      "Pigeon Airport",
		City:      "Angola",
		State:     "Indiana",
		Country:   "US",
		Elevation: 1000,
		Latitude:  41.6370010376,
		Longitude: -84.9494018555,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II17": {
		ICAO:      "II17",
		Name:      "Bickel's Cow Patch Airport",
		City:      "Knox",
		State:     "Indiana",
		Country:   "US",
		Elevation: 720,
		Latitude:  41.2681007385,
		Longitude: -86.5595016479,
		Timezone:  "America/Indiana/Knox",
	},
	"II18": {
		ICAO:      "II18",
		Name:      "Lou Abbett Farms Airport",
		City:      "La Crosse",
		State:     "Indiana",
		Country:   "US",
		Elevation: 685,
		Latitude:  41.3356018066,
		Longitude: -86.9285964966,
		Timezone:  "America/Chicago",
	},
	"II19": {
		ICAO:      "II19",
		Name:      "Etter Airport",
		City:      "Lafayette",
		State:     "Indiana",
		Country:   "US",
		Elevation: 660,
		Latitude:  40.4061012268,
		Longitude: -86.7536010742,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II20": {
		ICAO:      "II20",
		Name:      "Mossburg Airport",
		City:      "Liberty Center",
		State:     "Indiana",
		Country:   "US",
		Elevation: 845,
		Latitude:  40.6736984253,
		Longitude: -85.2632980347,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II21": {
		ICAO:      "II21",
		Name:      "Zollinger Strip",
		City:      "Ligonier",
		State:     "Indiana",
		Country:   "US",
		Elevation: 860,
		Latitude:  41.4752998352,
		Longitude: -85.6947021484,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II22": {
		ICAO:      "II22",
		Name:      "Antonian Airport",
		City:      "Medaryville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 695,
		Latitude:  41.1020011902,
		Longitude: -86.9321975708,
		Timezone:  "America/Chicago",
	},
	"II25": {
		ICAO:      "II25",
		Name:      "Timber Trails Airport",
		City:      "Spencer",
		State:     "Indiana",
		Country:   "US",
		Elevation: 800,
		Latitude:  39.3372993469,
		Longitude: -86.7797012329,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II26": {
		ICAO:      "II26",
		Name:      "Ashby Airport",
		City:      "Remington",
		State:     "Indiana",
		Country:   "US",
		Elevation: 750,
		Latitude:  40.7038993835,
		Longitude: -87.1877975464,
		Timezone:  "America/Chicago",
	},
	"II27": {
		ICAO:      "II27",
		Name:      "Ward Airport",
		City:      "Arcadia",
		State:     "Indiana",
		Country:   "US",
		Elevation: 830,
		Latitude:  40.1666984558,
		Longitude: -85.9408035278,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II28": {
		ICAO:      "II28",
		Name:      "Stettler Strip",
		City:      "Leo",
		State:     "Indiana",
		Country:   "US",
		Elevation: 860,
		Latitude:  41.2106018066,
		Longitude: -85.0635986328,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II29": {
		ICAO:      "II29",
		Name:      "Owens Field",
		City:      "Greencastle",
		State:     "Indiana",
		Country:   "US",
		Elevation: 840,
		Latitude:  39.6109008789,
		Longitude: -86.7561035156,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II30": {
		ICAO:      "II30",
		Name:      "Ridgway Flying Service Airport",
		City:      "Carlisle",
		State:     "Indiana",
		Country:   "US",
		Elevation: 510,
		Latitude:  38.9067001343,
		Longitude: -87.4017028809,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II31": {
		ICAO:      "II31",
		Name:      "Jurassic Landings Ultralightport",
		City:      "Essex",
		State:     "Illinois",
		Country:   "US",
		Elevation: 585,
		Latitude:  41.1879005432,
		Longitude: -88.167098999,
		Timezone:  "America/Chicago",
	},
	"II32": {
		ICAO:      "II32",
		Name:      "Raceway Airport",
		City:      "Chandler",
		State:     "Indiana",
		Country:   "US",
		Elevation: 400,
		Latitude:  38.0491981506,
		Longitude: -87.381401062,
		Timezone:  "America/Chicago",
	},
	"II33": {
		ICAO:      "II33",
		Name:      "Eickholtz Airport",
		City:      "Charleston",
		State:     "Indiana",
		Country:   "US",
		Elevation: 620,
		Latitude:  38.460899353,
		Longitude: -87.6549987793,
		Timezone:  "America/Chicago",
	},
	"II34": {
		ICAO:      "II34",
		Name:      "Booe Airport",
		City:      "Clay City",
		State:     "Indiana",
		Country:   "US",
		Elevation: 600,
		Latitude:  39.2731018066,
		Longitude: -87.1222000122,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II35": {
		ICAO:      "II35",
		Name:      "Lindley Private Airport",
		City:      "Danville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 925,
		Latitude:  39.8087005615,
		Longitude: -86.4955978394,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II36": {
		ICAO:      "II36",
		Name:      "Mc Neil Field",
		City:      "Fountaintown",
		State:     "Indiana",
		Country:   "US",
		Elevation: 862,
		Latitude:  39.6652984619,
		Longitude: -85.8167037964,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II37": {
		ICAO:      "II37",
		Name:      "Arthur Airport",
		City:      "Greenfield",
		State:     "Indiana",
		Country:   "US",
		Elevation: 905,
		Latitude:  39.7463989258,
		Longitude: -85.7746963501,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II38": {
		ICAO:      "II38",
		Name:      "Sherk Field",
		City:      "Lapaz",
		State:     "Indiana",
		Country:   "US",
		Elevation: 860,
		Latitude:  41.426399231,
		Longitude: -86.3163986206,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II39": {
		ICAO:      "II39",
		Name:      "Hollingsworth Airport",
		City:      "Francisco",
		State:     "Indiana",
		Country:   "US",
		Elevation: 480,
		Latitude:  38.3362007141,
		Longitude: -87.4677963257,
		Timezone:  "America/Chicago",
	},
	"II40": {
		ICAO:      "II40",
		Name:      "Hopkins Farms Airport",
		City:      "Francisco",
		State:     "Indiana",
		Country:   "US",
		Elevation: 460,
		Latitude:  38.3083992004,
		Longitude: -87.4085998535,
		Timezone:  "America/Chicago",
	},
	"II42": {
		ICAO:      "II42",
		Name:      "Creekside Farm Airport",
		City:      "Noblesville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 800,
		Latitude:  40.0685997009,
		Longitude: -85.922203064,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II43": {
		ICAO:      "II43",
		Name:      "C. V. Airport",
		City:      "Granger",
		State:     "Indiana",
		Country:   "US",
		Elevation: 780,
		Latitude:  41.75,
		Longitude: -86.192199707,
		Timezone:  "America/Detroit",
	},
	"II45": {
		ICAO:      "II45",
		Name:      "Renshaw Airport",
		City:      "Hatfield",
		State:     "Indiana",
		Country:   "US",
		Elevation: 380,
		Latitude:  37.8925018311,
		Longitude: -87.1900024414,
		Timezone:  "America/Chicago",
	},
	"II49": {
		ICAO:      "II49",
		Name:      "Foertsch Airport",
		City:      "Lamar",
		State:     "Indiana",
		Country:   "US",
		Elevation: 476,
		Latitude:  38.0369987488,
		Longitude: -86.9111022949,
		Timezone:  "America/Chicago",
	},
	"II50": {
		ICAO:      "II50",
		Name:      "Jack Oak Airport",
		City:      "Lewisville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 790,
		Latitude:  39.48279953,
		Longitude: -86.676902771,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II51": {
		ICAO:      "II51",
		Name:      "Hull Airport",
		City:      "Ligonier",
		State:     "Indiana",
		Country:   "US",
		Elevation: 930,
		Latitude:  41.4847984314,
		Longitude: -85.5930023193,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II52": {
		ICAO:      "II52",
		Name:      "Haffner Airport",
		City:      "Lizton",
		State:     "Indiana",
		Country:   "US",
		Elevation: 960,
		Latitude:  39.8774986267,
		Longitude: -86.5278015137,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II53": {
		ICAO:      "II53",
		Name:      "Burke's Airport",
		City:      "Madison",
		State:     "Indiana",
		Country:   "US",
		Elevation: 810,
		Latitude:  38.7703018188,
		Longitude: -85.484703064,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II54": {
		ICAO:      "II54",
		Name:      "Giltner Airport",
		City:      "Madison",
		State:     "Indiana",
		Country:   "US",
		Elevation: 850,
		Latitude:  38.8158988953,
		Longitude: -85.4464035034,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II55": {
		ICAO:      "II55",
		Name:      "Hodges Airport",
		City:      "Martinsville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 625,
		Latitude:  39.5270004272,
		Longitude: -86.3003005981,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II59": {
		ICAO:      "II59",
		Name:      "Strip Airport",
		City:      "Newport",
		State:     "Indiana",
		Country:   "US",
		Elevation: 650,
		Latitude:  39.8400001526,
		Longitude: -87.3936004639,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II62": {
		ICAO:      "II62",
		Name:      "Berger Airport",
		City:      "Plymouth",
		State:     "Indiana",
		Country:   "US",
		Elevation: 800,
		Latitude:  41.3877983093,
		Longitude: -86.2585983276,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II65": {
		ICAO:      "II65",
		Name:      "Gilmore Airport",
		City:      "Rensselaer",
		State:     "Indiana",
		Country:   "US",
		Elevation: 680,
		Latitude:  40.8875007629,
		Longitude: -87.1235961914,
		Timezone:  "America/Chicago",
	},
	"II67": {
		ICAO:      "II67",
		Name:      "Hickory Hills Airport",
		City:      "Hartford City",
		State:     "Indiana",
		Country:   "US",
		Elevation: 900,
		Latitude:  40.4277992249,
		Longitude: -85.3734970093,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II68": {
		ICAO:      "II68",
		Name:      "Durham Airport",
		City:      "Russellville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 820,
		Latitude:  39.85499954,
		Longitude: -86.959198,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II69": {
		ICAO:      "II69",
		Name:      "Harrod/Rose Airport",
		City:      "Scottsburg",
		State:     "Indiana",
		Country:   "US",
		Elevation: 580,
		Latitude:  38.7181015015,
		Longitude: -85.7602005005,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II70": {
		ICAO:      "II70",
		Name:      "Salsbery Airport",
		City:      "Sharpsville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 870,
		Latitude:  40.3844985962,
		Longitude: -86.0633010864,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II71": {
		ICAO:      "II71",
		Name:      "Smith Airport",
		City:      "St. Paul",
		State:     "Indiana",
		Country:   "US",
		Elevation: 900,
		Latitude:  39.3863983154,
		Longitude: -85.6172027588,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II72": {
		ICAO:      "II72",
		Name:      "Anderson Airport",
		City:      "Sunman",
		State:     "Indiana",
		Country:   "US",
		Elevation: 965,
		Latitude:  39.1958007813,
		Longitude: -85.2055969238,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II73": {
		ICAO:      "II73",
		Name:      "Good Earth Farm Strip",
		City:      "Union City",
		State:     "Indiana",
		Country:   "US",
		Elevation: 1080,
		Latitude:  40.2262001038,
		Longitude: -84.8349990845,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II74": {
		ICAO:      "II74",
		Name:      "Eby Field",
		City:      "Wakarusa",
		State:     "Indiana",
		Country:   "US",
		Elevation: 825,
		Latitude:  41.5597991943,
		Longitude: -86.0531005859,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II75": {
		ICAO:      "II75",
		Name:      "Daugherty Field",
		City:      "Warren",
		State:     "Indiana",
		Country:   "US",
		Elevation: 839,
		Latitude:  40.6791992188,
		Longitude: -85.4152984619,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II76": {
		ICAO:      "II76",
		Name:      "Clay Hill Farms Airport",
		City:      "Medora",
		State:     "Indiana",
		Country:   "US",
		Elevation: 528,
		Latitude:  38.8507003784,
		Longitude: -86.1368026733,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II77": {
		ICAO:      "II77",
		Name:      "Finney's Airpark",
		City:      "Albany",
		State:     "Indiana",
		Country:   "US",
		Elevation: 965,
		Latitude:  40.2594985962,
		Longitude: -85.2604980469,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II78": {
		ICAO:      "II78",
		Name:      "Wallace Field",
		City:      "Wilkinson",
		State:     "Indiana",
		Country:   "US",
		Elevation: 995,
		Latitude:  39.8583984375,
		Longitude: -85.601600647,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II79": {
		ICAO:      "II79",
		Name:      "Sommers Airport",
		City:      "Winamac",
		State:     "Indiana",
		Country:   "US",
		Elevation: 710,
		Latitude:  41.0769996643,
		Longitude: -86.7245025635,
		Timezone:  "America/Indiana/Winamac",
	},
	"II81": {
		ICAO:      "II81",
		Name:      "Richardson Field",
		City:      "Winslow",
		State:     "Indiana",
		Country:   "US",
		Elevation: 425,
		Latitude:  38.3805999756,
		Longitude: -87.2205963135,
		Timezone:  "America/Indiana/Petersburg",
	},
	"II82": {
		ICAO:      "II82",
		Name:      "Meadors Field",
		City:      "Danville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 1000,
		Latitude:  39.7942008972,
		Longitude: -86.577796936,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II83": {
		ICAO:      "II83",
		Name:      "Bramble Airport",
		City:      "Brownsburg",
		State:     "Indiana",
		Country:   "US",
		Elevation: 870,
		Latitude:  39.8342018127,
		Longitude: -86.4341964722,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II84": {
		ICAO:      "II84",
		Name:      "Hilakos Airport",
		City:      "Mount Vernon",
		State:     "Indiana",
		Country:   "US",
		Elevation: 381,
		Latitude:  37.9319992065,
		Longitude: -87.7714004517,
		Timezone:  "America/Chicago",
	},
	"II85": {
		ICAO:      "II85",
		Name:      "Blomenberg Airport",
		City:      "Decatur",
		State:     "Indiana",
		Country:   "US",
		Elevation: 800,
		Latitude:  40.8961982727,
		Longitude: -85.0491027832,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II86": {
		ICAO:      "II86",
		Name:      "Poole Airport",
		City:      "Noblesville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 860,
		Latitude:  40.0155982971,
		Longitude: -85.8874969482,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II87": {
		ICAO:      "II87",
		Name:      "Twelve Oaks Airport",
		City:      "Martinsville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 715,
		Latitude:  39.4514007568,
		Longitude: -86.3283004761,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II89": {
		ICAO:      "II89",
		Name:      "Yelverton Airport",
		City:      "St Wendel",
		State:     "Indiana",
		Country:   "US",
		Elevation: 525,
		Latitude:  38.0870018005,
		Longitude: -87.6817016602,
		Timezone:  "America/Chicago",
	},
	"II90": {
		ICAO:      "II90",
		Name:      "Crawford Field",
		City:      "North Judson",
		State:     "Indiana",
		Country:   "US",
		Elevation: 705,
		Latitude:  41.199798584,
		Longitude: -86.8364028931,
		Timezone:  "America/Indiana/Knox",
	},
	"II91": {
		ICAO:      "II91",
		Name:      "Dunbar Field",
		City:      "Pittsboro",
		State:     "Indiana",
		Country:   "US",
		Elevation: 940,
		Latitude:  39.8438987732,
		Longitude: -86.5053024292,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II93": {
		ICAO:      "II93",
		Name:      "Bronson Airport",
		City:      "Bargersville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 780,
		Latitude:  39.4838981628,
		Longitude: -86.2268981934,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II94": {
		ICAO:      "II94",
		Name:      "Irion Airport",
		City:      "Noblesville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 800,
		Latitude:  40.1283988953,
		Longitude: -85.886100769,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II95": {
		ICAO:      "II95",
		Name:      "Rust's Landing Airport",
		City:      "Bourbon",
		State:     "Indiana",
		Country:   "US",
		Elevation: 820,
		Latitude:  41.3644981384,
		Longitude: -86.0633010864,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II96": {
		ICAO:      "II96",
		Name:      "Buchta Airport",
		City:      "Haysville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 520,
		Latitude:  38.4928016663,
		Longitude: -86.9330978394,
		Timezone:  "America/Indiana/Vincennes",
	},
	"II97": {
		ICAO:      "II97",
		Name:      "Solenberg Airport",
		City:      "Shelbyville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 720,
		Latitude:  39.4777984619,
		Longitude: -85.8833007813,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II98": {
		ICAO:      "II98",
		Name:      "Shrum Field",
		City:      "Sullivan",
		State:     "Indiana",
		Country:   "US",
		Elevation: 480,
		Latitude:  39.0438995361,
		Longitude: -87.4219970703,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"II99": {
		ICAO:      "II99",
		Name:      "Drake Field",
		City:      "Walkerton",
		State:     "Indiana",
		Country:   "US",
		Elevation: 730,
		Latitude:  41.4364013672,
		Longitude: -86.4708023071,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IL01": {
		ICAO:      "IL01",
		Name:      "Wolford's Airport",
		City:      "Abingdon",
		State:     "Illinois",
		Country:   "US",
		Elevation: 720,
		Latitude:  40.7924995422,
		Longitude: -90.4382019043,
		Timezone:  "America/Chicago",
	},
	"IL02": {
		ICAO:      "IL02",
		Name:      "Herbert C. Maas Airport",
		City:      "Zion",
		State:     "Illinois",
		Country:   "US",
		Elevation: 719,
		Latitude:  42.4721984863,
		Longitude: -87.9084014893,
		Timezone:  "America/Chicago",
	},
	"IL05": {
		ICAO:      "IL05",
		Name:      "Bingham Airport",
		City:      "Alden",
		State:     "Illinois",
		Country:   "US",
		Elevation: 980,
		Latitude:  42.4402999878,
		Longitude: -88.4889984131,
		Timezone:  "America/Chicago",
	},
	"IL07": {
		ICAO:      "IL07",
		Name:      "Taylor Airport",
		City:      "Amboy",
		State:     "Illinois",
		Country:   "US",
		Elevation: 760,
		Latitude:  41.7235984802,
		Longitude: -89.2918014526,
		Timezone:  "America/Chicago",
	},
	"IL09": {
		ICAO:      "IL09",
		Name:      "Albrecht /2/ Airport",
		City:      "Amboy",
		State:     "Illinois",
		Country:   "US",
		Elevation: 802,
		Latitude:  41.631401062,
		Longitude: -89.4050979614,
		Timezone:  "America/Chicago",
	},
	"IL11": {
		ICAO:      "IL11",
		Name:      "Donald Alfred Gade Airport",
		City:      "Antioch",
		State:     "Illinois",
		Country:   "US",
		Elevation: 805,
		Latitude:  42.4653015137,
		Longitude: -88.040397644,
		Timezone:  "America/Chicago",
	},
	"IL12": {
		ICAO:      "IL12",
		Name:      "Kinsey RLA Restricted Landing Area",
		City:      "Arenzville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 455,
		Latitude:  39.8800010681,
		Longitude: -90.4651031494,
		Timezone:  "America/Chicago",
	},
	"IL13": {
		ICAO:      "IL13",
		Name:      "Funfsinn Airport",
		City:      "Arlington",
		State:     "Illinois",
		Country:   "US",
		Elevation: 715,
		Latitude:  41.4707984924,
		Longitude: -89.1808013916,
		Timezone:  "America/Chicago",
	},
	"IL15": {
		ICAO:      "IL15",
		Name:      "Runyan Aviation Airport",
		City:      "Bushnell",
		State:     "Illinois",
		Country:   "US",
		Elevation: 660,
		Latitude:  40.5694007874,
		Longitude: -90.5486984253,
		Timezone:  "America/Chicago",
	},
	"IL16": {
		ICAO:      "IL16",
		Name:      "Houseman Airport",
		City:      "Cobden",
		State:     "Illinois",
		Country:   "US",
		Elevation: 800,
		Latitude:  37.5401000977,
		Longitude: -89.1225967407,
		Timezone:  "America/Chicago",
	},
	"IL18": {
		ICAO:      "IL18",
		Name:      "Crook Restricted Landing Area",
		City:      "Columbia",
		State:     "Illinois",
		Country:   "US",
		Elevation: 580,
		Latitude:  38.3870010376,
		Longitude: -90.1918029785,
		Timezone:  "America/Chicago",
	},
	"IL22": {
		ICAO:      "IL22",
		Name:      "Heeg Airport",
		City:      "Oquawka",
		State:     "Illinois",
		Country:   "US",
		Elevation: 600,
		Latitude:  41.0070991516,
		Longitude: -90.8828964233,
		Timezone:  "America/Chicago",
	},
	"IL23": {
		ICAO:      "IL23",
		Name:      "Ellis Airport",
		City:      "Davis",
		State:     "Illinois",
		Country:   "US",
		Elevation: 900,
		Latitude:  42.4425010681,
		Longitude: -89.4501037598,
		Timezone:  "America/Chicago",
	},
	"IL24": {
		ICAO:      "IL24",
		Name:      "Richardson Airport",
		City:      "New Berlin",
		State:     "Illinois",
		Country:   "US",
		Elevation: 653,
		Latitude:  39.7075004578,
		Longitude: -89.883102417,
		Timezone:  "America/Chicago",
	},
	"IL29": {
		ICAO:      "IL29",
		Name:      "Von Alvens Airview Airport",
		City:      "Beecher",
		State:     "Illinois",
		Country:   "US",
		Elevation: 715,
		Latitude:  41.2999992371,
		Longitude: -87.6500015259,
		Timezone:  "America/Chicago",
	},
	"IL31": {
		ICAO:      "IL31",
		Name:      "Cooch Landing Area Airport",
		City:      "Atwood",
		State:     "Illinois",
		Country:   "US",
		Elevation: 695,
		Latitude:  39.8432998657,
		Longitude: -88.4533004761,
		Timezone:  "America/Chicago",
	},
	"IL32": {
		ICAO:      "IL32",
		Name:      "Adkisson Airport",
		City:      "Hammond",
		State:     "Illinois",
		Country:   "US",
		Elevation: 669,
		Latitude:  39.7832984924,
		Longitude: -88.6213989258,
		Timezone:  "America/Chicago",
	},
	"IL34": {
		ICAO:      "IL34",
		Name:      "Henderson Airport",
		City:      "Belvidere",
		State:     "Illinois",
		Country:   "US",
		Elevation: 815,
		Latitude:  42.2145004272,
		Longitude: -88.7570037842,
		Timezone:  "America/Chicago",
	},
	"IL35": {
		ICAO:      "IL35",
		Name:      "Redpath Restricted Landing Area",
		City:      "Baldwin",
		State:     "Illinois",
		Country:   "US",
		Elevation: 470,
		Latitude:  38.1819992065,
		Longitude: -89.8084030151,
		Timezone:  "America/Chicago",
	},
	"IL36": {
		ICAO:      "IL36",
		Name:      "Bob Walberg Field",
		City:      "Belvidere",
		State:     "Illinois",
		Country:   "US",
		Elevation: 840,
		Latitude:  42.3208999634,
		Longitude: -88.7583999634,
		Timezone:  "America/Chicago",
	},
	"IL38": {
		ICAO:      "IL38",
		Name:      "J Maddock Airport",
		City:      "Big Rock",
		State:     "Illinois",
		Country:   "US",
		Elevation: 705,
		Latitude:  41.766998291,
		Longitude: -88.5319976807,
		Timezone:  "America/Chicago",
	},
	"IL45": {
		ICAO:      "IL45",
		Name:      "Busboom Airport",
		City:      "Gifford",
		State:     "Illinois",
		Country:   "US",
		Elevation: 780,
		Latitude:  40.3111000061,
		Longitude: -88.015296936,
		Timezone:  "America/Chicago",
	},
	"IL46": {
		ICAO:      "IL46",
		Name:      "Brooks Ranch Airport",
		City:      "Burnside",
		State:     "Illinois",
		Country:   "US",
		Elevation: 700,
		Latitude:  40.4667015076,
		Longitude: -91.0335006714,
		Timezone:  "America/Chicago",
	},
	"IL51": {
		ICAO:      "IL51",
		Name:      "Aero Acres Airport",
		City:      "Campus",
		State:     "Illinois",
		Country:   "US",
		Elevation: 645,
		Latitude:  41.0381011963,
		Longitude: -88.3375015259,
		Timezone:  "America/Chicago",
	},
	"IL52": {
		ICAO:      "IL52",
		Name:      "Williams Airport",
		City:      "Carthage",
		State:     "Illinois",
		Country:   "US",
		Elevation: 679,
		Latitude:  40.439201355,
		Longitude: -91.1439971924,
		Timezone:  "America/Chicago",
	},
	"IL53": {
		ICAO:      "IL53",
		Name:      "Lung RLA Restricted Landing Area",
		City:      "Carthage",
		State:     "Illinois",
		Country:   "US",
		Elevation: 617,
		Latitude:  40.4612007141,
		Longitude: -91.0737991333,
		Timezone:  "America/Chicago",
	},
	"IL54": {
		ICAO:      "IL54",
		Name:      "Richmond Airport",
		City:      "Table Grove",
		State:     "Illinois",
		Country:   "US",
		Elevation: 680,
		Latitude:  40.3442001343,
		Longitude: -90.4428024292,
		Timezone:  "America/Chicago",
	},
	"IL55": {
		ICAO:      "IL55",
		Name:      "Andrew RLA Restricted Landing Area Airport",
		City:      "Champaign",
		State:     "Illinois",
		Country:   "US",
		Elevation: 810,
		Latitude:  40.1603012085,
		Longitude: -88.3044967651,
		Timezone:  "America/Chicago",
	},
	"IL56": {
		ICAO:      "IL56",
		Name:      "Mc Culley Airport",
		City:      "Champaign",
		State:     "Illinois",
		Country:   "US",
		Elevation: 750,
		Latitude:  40.2202987671,
		Longitude: -88.2537002563,
		Timezone:  "America/Chicago",
	},
	"IL57": {
		ICAO:      "IL57",
		Name:      "Cottonwood Airport",
		City:      "Bloomington",
		State:     "Illinois",
		Country:   "US",
		Elevation: 765,
		Latitude:  40.422000885,
		Longitude: -89.0205993652,
		Timezone:  "America/Chicago",
	},
	"IL58": {
		ICAO:      "IL58",
		Name:      "Aero Four Airport",
		City:      "Channahon",
		State:     "Illinois",
		Country:   "US",
		Elevation: 600,
		Latitude:  41.4361000061,
		Longitude: -88.2487030029,
		Timezone:  "America/Chicago",
	},
	"IL59": {
		ICAO:      "IL59",
		Name:      "Chicago Glider Club Gliderport",
		City:      "Channahon",
		State:     "Illinois",
		Country:   "US",
		Elevation: 590,
		Latitude:  41.43199921,
		Longitude: -88.24729919,
		Timezone:  "America/Chicago",
	},
	"IL60": {
		ICAO:      "IL60",
		Name:      "Chatsworth Restricted Landing Area",
		City:      "Chatsworth",
		State:     "Illinois",
		Country:   "US",
		Elevation: 742,
		Latitude:  40.7430992126,
		Longitude: -88.2911987305,
		Timezone:  "America/Chicago",
	},
	"IL63": {
		ICAO:      "IL63",
		Name:      "Mays Aviation Airport",
		City:      "Chenoa",
		State:     "Illinois",
		Country:   "US",
		Elevation: 720,
		Latitude:  40.7000007629,
		Longitude: -88.6751022339,
		Timezone:  "America/Chicago",
	},
	"IL64": {
		ICAO:      "IL64",
		Name:      "Martin Airport",
		City:      "Lincoln",
		State:     "Illinois",
		Country:   "US",
		Elevation: 558,
		Latitude:  40.1805992126,
		Longitude: -89.5000991821,
		Timezone:  "America/Chicago",
	},
	"IL65": {
		ICAO:      "IL65",
		Name:      "M.A.M Trail Airport",
		City:      "Little York",
		State:     "Illinois",
		Country:   "US",
		Elevation: 605,
		Latitude:  41.0050010681,
		Longitude: -90.7085037231,
		Timezone:  "America/Chicago",
	},
	"IL68": {
		ICAO:      "IL68",
		Name:      "Mill Rose Farm Restricted Landing Area",
		City:      "Chicago/Barrington",
		State:     "Illinois",
		Country:   "US",
		Elevation: 850,
		Latitude:  42.07720184,
		Longitude: -88.15979767,
		Timezone:  "America/Chicago",
	},
	"IL71": {
		ICAO:      "IL71",
		Name:      "Dury Estates Airport",
		City:      "Hurst",
		State:     "Illinois",
		Country:   "US",
		Elevation: 395,
		Latitude:  37.81919861,
		Longitude: -89.15219879,
		Timezone:  "America/Chicago",
	},
	"IL72": {
		ICAO:      "IL72",
		Name:      "Entwistle Airport",
		City:      "Lostant",
		State:     "Illinois",
		Country:   "US",
		Elevation: 700,
		Latitude:  41.1156005859,
		Longitude: -89.0447998047,
		Timezone:  "America/Chicago",
	},
	"IL78": {
		ICAO:      "IL78",
		Name:      "Benoit Airport",
		City:      "Kankakee",
		State:     "Illinois",
		Country:   "US",
		Elevation: 660,
		Latitude:  41.1310997009,
		Longitude: -87.7922973633,
		Timezone:  "America/Chicago",
	},
	"IL81": {
		ICAO:      "IL81",
		Name:      "Berns Airport",
		City:      "Clifton",
		State:     "Illinois",
		Country:   "US",
		Elevation: 658,
		Latitude:  40.9361000061,
		Longitude: -88.0417022705,
		Timezone:  "America/Chicago",
	},
	"IL82": {
		ICAO:      "IL82",
		Name:      "Martin RLA Restricted Landing Area",
		City:      "Clinton",
		State:     "Illinois",
		Country:   "US",
		Elevation: 725,
		Latitude:  40.109500885,
		Longitude: -88.8286972046,
		Timezone:  "America/Chicago",
	},
	"IL84": {
		ICAO:      "IL84",
		Name:      "Douglas Airport",
		City:      "Colusa",
		State:     "Illinois",
		Country:   "US",
		Elevation: 650,
		Latitude:  40.5764007568,
		Longitude: -91.102897644,
		Timezone:  "America/Chicago",
	},
	"IL86": {
		ICAO:      "IL86",
		Name:      "Krohe Airport",
		City:      "Beardstown",
		State:     "Illinois",
		Country:   "US",
		Elevation: 450,
		Latitude:  39.9361991882,
		Longitude: -90.466796875,
		Timezone:  "America/Chicago",
	},
	"IL87": {
		ICAO:      "IL87",
		Name:      "W Davis Airport",
		City:      "Compton",
		State:     "Illinois",
		Country:   "US",
		Elevation: 930,
		Latitude:  41.711101532,
		Longitude: -89.1122970581,
		Timezone:  "America/Chicago",
	},
	"IL88": {
		ICAO:      "IL88",
		Name:      "Earl Barnickel Airport",
		City:      "Compton",
		State:     "Illinois",
		Country:   "US",
		Elevation: 929,
		Latitude:  41.6749992371,
		Longitude: -89.1167984009,
		Timezone:  "America/Chicago",
	},
	"IL91": {
		ICAO:      "IL91",
		Name:      "Testoni Farms Airport",
		City:      "Cornell",
		State:     "Illinois",
		Country:   "US",
		Elevation: 688,
		Latitude:  41.0209007263,
		Longitude: -88.6533966064,
		Timezone:  "America/Chicago",
	},
	"IL93": {
		ICAO:      "IL93",
		Name:      "Hendrickson Flying Service Airport",
		City:      "Creston",
		State:     "Illinois",
		Country:   "US",
		Elevation: 889,
		Latitude:  41.9453010559,
		Longitude: -88.9240036011,
		Timezone:  "America/Chicago",
	},
	"IL94": {
		ICAO:      "IL94",
		Name:      "Hoblit Farms Airport",
		City:      "Atlanta",
		State:     "Illinois",
		Country:   "US",
		Elevation: 623,
		Latitude:  40.2262992859,
		Longitude: -89.2428970337,
		Timezone:  "America/Chicago",
	},
	"IL95": {
		ICAO:      "IL95",
		Name:      "Corn Field",
		City:      "Custer Park",
		State:     "Illinois",
		Country:   "US",
		Elevation: 585,
		Latitude:  41.2230987549,
		Longitude: -88.1453018188,
		Timezone:  "America/Chicago",
	},
	"IL97": {
		ICAO:      "IL97",
		Name:      "Springfield Southwest Airpark",
		City:      "New Berlin",
		State:     "Illinois",
		Country:   "US",
		Elevation: 635,
		Latitude:  39.7369995117,
		Longitude: -89.8097991943,
		Timezone:  "America/Chicago",
	},
	"IL99": {
		ICAO:      "IL99",
		Name:      "B & C Airport",
		City:      "Earlville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 715,
		Latitude:  41.5928001404,
		Longitude: -88.967300415,
		Timezone:  "America/Chicago",
	},
	"IN00": {
		ICAO:      "IN00",
		Name:      "Sutton Airport",
		City:      "Montmorenci",
		State:     "Indiana",
		Country:   "US",
		Elevation: 740,
		Latitude:  40.54529953,
		Longitude: -87.0522003174,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN01": {
		ICAO:      "IN01",
		Name:      "Best Lock Corp Airstrip",
		City:      "Indianapolis",
		State:     "Indiana",
		Country:   "US",
		Elevation: 815,
		Latitude:  39.8880996704,
		Longitude: -86.0661010742,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN03": {
		ICAO:      "IN03",
		Name:      "Alford Airpark",
		City:      "Petersburg",
		State:     "Indiana",
		Country:   "US",
		Elevation: 500,
		Latitude:  38.4639015198,
		Longitude: -87.25,
		Timezone:  "America/Indiana/Petersburg",
	},
	"IN04": {
		ICAO:      "IN04",
		Name:      "N'Meier Airport",
		City:      "Westphalia",
		State:     "Indiana",
		Country:   "US",
		Elevation: 495,
		Latitude:  38.8464012146,
		Longitude: -87.2403030396,
		Timezone:  "America/Indiana/Vincennes",
	},
	"IN05": {
		ICAO:      "IN05",
		Name:      "Stevens Farms Airport",
		City:      "Rushville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 930,
		Latitude:  39.6605987549,
		Longitude: -85.5483016968,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN08": {
		ICAO:      "IN08",
		Name:      "Shakamak Airport",
		City:      "Jasonville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 613,
		Latitude:  39.1694984436,
		Longitude: -87.1902999878,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN09": {
		ICAO:      "IN09",
		Name:      "Clifton Airport",
		City:      "Boswell",
		State:     "Indiana",
		Country:   "US",
		Elevation: 760,
		Latitude:  40.5242004395,
		Longitude: -87.4119033813,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN10": {
		ICAO:      "IN10",
		Name:      "Plummer Airport",
		City:      "Kewanna",
		State:     "Indiana",
		Country:   "US",
		Elevation: 760,
		Latitude:  40.9758987427,
		Longitude: -86.4199981689,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN11": {
		ICAO:      "IN11",
		Name:      "Arrowhead Farm Airport",
		City:      "Bourbon",
		State:     "Indiana",
		Country:   "US",
		Elevation: 825,
		Latitude:  41.2928009033,
		Longitude: -86.0710983276,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN13": {
		ICAO:      "IN13",
		Name:      "Lanesville Skyways Airport",
		City:      "Lanesville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 885,
		Latitude:  38.2237014771,
		Longitude: -85.9749984741,
		Timezone:  "America/Kentucky/Louisville",
	},
	"IN14": {
		ICAO:      "IN14",
		Name:      "Greenridge RLA Restricted Landing Area",
		City:      "Lanesville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 858,
		Latitude:  38.2389984131,
		Longitude: -86.0194015503,
		Timezone:  "America/Kentucky/Louisville",
	},
	"IN15": {
		ICAO:      "IN15",
		Name:      "T & T Airport",
		City:      "Vincennes",
		State:     "Indiana",
		Country:   "US",
		Elevation: 550,
		Latitude:  38.604801178,
		Longitude: -87.4630966187,
		Timezone:  "America/Indiana/Vincennes",
	},
	"IN18": {
		ICAO:      "IN18",
		Name:      "Hook Field",
		City:      "Harlan",
		State:     "Indiana",
		Country:   "US",
		Elevation: 790,
		Latitude:  41.2745018005,
		Longitude: -84.8050003052,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN19": {
		ICAO:      "IN19",
		Name:      "Marshall Field",
		City:      "Acton",
		State:     "Indiana",
		Country:   "US",
		Elevation: 800,
		Latitude:  39.6431007385,
		Longitude: -85.9816970825,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN20": {
		ICAO:      "IN20",
		Name:      "Jungclaus Airport",
		City:      "Martinsville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 765,
		Latitude:  39.4537010193,
		Longitude: -86.2802963257,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN21": {
		ICAO:      "IN21",
		Name:      "Starkey's Airport",
		City:      "Mooreland",
		State:     "Indiana",
		Country:   "US",
		Elevation: 1110,
		Latitude:  40.0430984497,
		Longitude: -85.2604980469,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN23": {
		ICAO:      "IN23",
		Name:      "Zeller Elev County Airport",
		City:      "Mount Vernon",
		State:     "Indiana",
		Country:   "US",
		Elevation: 378,
		Latitude:  37.9084014893,
		Longitude: -87.7583999634,
		Timezone:  "America/Chicago",
	},
	"IN25": {
		ICAO:      "IN25",
		Name:      "Hood Field",
		City:      "Jamestown",
		State:     "Indiana",
		Country:   "US",
		Elevation: 930,
		Latitude:  39.9981002808,
		Longitude: -86.5772018433,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN26": {
		ICAO:      "IN26",
		Name:      "Archangels Landing Airport",
		City:      "San Pierre",
		State:     "Indiana",
		Country:   "US",
		Elevation: 669,
		Latitude:  41.2613983154,
		Longitude: -86.8724975586,
		Timezone:  "America/Chicago",
	},
	"IN27": {
		ICAO:      "IN27",
		Name:      "Skyridge Airport",
		City:      "Arcadia",
		State:     "Indiana",
		Country:   "US",
		Elevation: 895,
		Latitude:  40.1791992188,
		Longitude: -86.1157989502,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN29": {
		ICAO:      "IN29",
		Name:      "Durflinger Airport",
		City:      "Otterbein",
		State:     "Indiana",
		Country:   "US",
		Elevation: 747,
		Latitude:  40.573600769,
		Longitude: -87.1035995483,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN30": {
		ICAO:      "IN30",
		Name:      "Hull Airport",
		City:      "Patoka",
		State:     "Indiana",
		Country:   "US",
		Elevation: 423,
		Latitude:  38.4023017883,
		Longitude: -87.5995025635,
		Timezone:  "America/Chicago",
	},
	"IN31": {
		ICAO:      "IN31",
		Name:      "North Lakeland Airport",
		City:      "Pendleton",
		State:     "Indiana",
		Country:   "US",
		Elevation: 870,
		Latitude:  40.0223007202,
		Longitude: -85.7536010742,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN32": {
		ICAO:      "IN32",
		Name:      "Stephenson Airport",
		City:      "Pendleton",
		State:     "Indiana",
		Country:   "US",
		Elevation: 880,
		Latitude:  39.9772987366,
		Longitude: -85.7444000244,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN33": {
		ICAO:      "IN33",
		Name:      "Robison Airport",
		City:      "Peru",
		State:     "Indiana",
		Country:   "US",
		Elevation: 735,
		Latitude:  40.7361984253,
		Longitude: -86.1153030396,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN34": {
		ICAO:      "IN34",
		Name:      "Rush Strip",
		City:      "Peru",
		State:     "Indiana",
		Country:   "US",
		Elevation: 770,
		Latitude:  40.8344993591,
		Longitude: -86.0522003174,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN35": {
		ICAO:      "IN35",
		Name:      "Shinn Bone Lane Airport",
		City:      "Peru",
		State:     "Indiana",
		Country:   "US",
		Elevation: 809,
		Latitude:  40.6542015076,
		Longitude: -86.0227966309,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN36": {
		ICAO:      "IN36",
		Name:      "Plugger Airport",
		City:      "Evansville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 426,
		Latitude:  38.0355987549,
		Longitude: -87.7240982056,
		Timezone:  "America/Chicago",
	},
	"IN38": {
		ICAO:      "IN38",
		Name:      "Stewarts Green Acres Airport",
		City:      "North Liberty",
		State:     "Indiana",
		Country:   "US",
		Elevation: 725,
		Latitude:  41.54529953,
		Longitude: -86.452796936,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN39": {
		ICAO:      "IN39",
		Name:      "Klopfenstein Airport",
		City:      "Reynolds",
		State:     "Indiana",
		Country:   "US",
		Elevation: 685,
		Latitude:  40.7672996521,
		Longitude: -86.9207992554,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN40": {
		ICAO:      "IN40",
		Name:      "Brauns Airport",
		City:      "Richland",
		State:     "Indiana",
		Country:   "US",
		Elevation: 400,
		Latitude:  37.9099998474,
		Longitude: -87.182800293,
		Timezone:  "America/Chicago",
	},
	"IN41": {
		ICAO:      "IN41",
		Name:      "Nuckols Airport",
		City:      "Redkey",
		State:     "Indiana",
		Country:   "US",
		Elevation: 1025,
		Latitude:  40.354801178,
		Longitude: -85.0491027832,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN42": {
		ICAO:      "IN42",
		Name:      "Hackbarth Airport",
		City:      "Bremen",
		State:     "Indiana",
		Country:   "US",
		Elevation: 835,
		Latitude:  41.4902992249,
		Longitude: -86.1994018555,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN43": {
		ICAO:      "IN43",
		Name:      "Spring Lake Airport",
		City:      "Pekin",
		State:     "Indiana",
		Country:   "US",
		Elevation: 830,
		Latitude:  38.4751014709,
		Longitude: -85.9972000122,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN44": {
		ICAO:      "IN44",
		Name:      "Hardin Airport",
		City:      "Salem",
		State:     "Indiana",
		Country:   "US",
		Elevation: 800,
		Latitude:  38.564201355,
		Longitude: -86.0575027466,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN45": {
		ICAO:      "IN45",
		Name:      "4 Winds Aerodrome",
		City:      "Roachdale",
		State:     "Indiana",
		Country:   "US",
		Elevation: 810,
		Latitude:  39.8250007629,
		Longitude: -86.8833007813,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN46": {
		ICAO:      "IN46",
		Name:      "Butler Field",
		City:      "Rockville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 687,
		Latitude:  39.738899231,
		Longitude: -87.2194976807,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN47": {
		ICAO:      "IN47",
		Name:      "White Airport",
		City:      "Richmond",
		State:     "Indiana",
		Country:   "US",
		Elevation: 1140,
		Latitude:  39.773601532,
		Longitude: -84.9227981567,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN49": {
		ICAO:      "IN49",
		Name:      "Pherigo Airport",
		City:      "Shelbyville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 760,
		Latitude:  39.4874992371,
		Longitude: -85.8052978516,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN50": {
		ICAO:      "IN50",
		Name:      "Siefert Airport",
		City:      "Shelbyville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 750,
		Latitude:  39.4402999878,
		Longitude: -85.765296936,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN51": {
		ICAO:      "IN51",
		Name:      "Windy Knoll Airport",
		City:      "Sheridan",
		State:     "Indiana",
		Country:   "US",
		Elevation: 920,
		Latitude:  40.1399993896,
		Longitude: -86.1524963379,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN53": {
		ICAO:      "IN53",
		Name:      "Miller Airport",
		City:      "Spencer",
		State:     "Indiana",
		Country:   "US",
		Elevation: 710,
		Latitude:  39.2916984558,
		Longitude: -86.823600769,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN54": {
		ICAO:      "IN54",
		Name:      "I & C Field",
		City:      "Culver",
		State:     "Indiana",
		Country:   "US",
		Elevation: 745,
		Latitude:  41.2836990356,
		Longitude: -86.4627990723,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN55": {
		ICAO:      "IN55",
		Name:      "Songer Airport",
		City:      "Veedersburg",
		State:     "Indiana",
		Country:   "US",
		Elevation: 652,
		Latitude:  40.0792007446,
		Longitude: -87.261100769,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN56": {
		ICAO:      "IN56",
		Name:      "Carroll's Airpark",
		City:      "Muncie",
		State:     "Indiana",
		Country:   "US",
		Elevation: 940,
		Latitude:  40.1268997192,
		Longitude: -85.4319000244,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN57": {
		ICAO:      "IN57",
		Name:      "Shure Airport",
		City:      "Farmersburg",
		State:     "Indiana",
		Country:   "US",
		Elevation: 550,
		Latitude:  39.2613983154,
		Longitude: -87.4208984375,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN58": {
		ICAO:      "IN58",
		Name:      "Sealscott Airport",
		City:      "Monroeville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 820,
		Latitude:  40.9037017822,
		Longitude: -84.8475036621,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN60": {
		ICAO:      "IN60",
		Name:      "Wilderness Field",
		City:      "Westfield",
		State:     "Indiana",
		Country:   "US",
		Elevation: 908,
		Latitude:  40.0177993774,
		Longitude: -86.1603012085,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN61": {
		ICAO:      "IN61",
		Name:      "Flying J Airport",
		City:      "Wheatfield",
		State:     "Indiana",
		Country:   "US",
		Elevation: 653,
		Latitude:  41.2641983032,
		Longitude: -87.1343994141,
		Timezone:  "America/Chicago",
	},
	"IN62": {
		ICAO:      "IN62",
		Name:      "Tropria Airport",
		City:      "Churubusco",
		State:     "Indiana",
		Country:   "US",
		Elevation: 905,
		Latitude:  41.2448005676,
		Longitude: -85.2975006104,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN63": {
		ICAO:      "IN63",
		Name:      "Horizon Field",
		City:      "Eaton",
		State:     "Indiana",
		Country:   "US",
		Elevation: 940,
		Latitude:  40.3666992188,
		Longitude: -85.3386001587,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN64": {
		ICAO:      "IN64",
		Name:      "Beck Airport",
		City:      "Bargersville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 800,
		Latitude:  39.5609016418,
		Longitude: -86.2040023804,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN65": {
		ICAO:      "IN65",
		Name:      "Wolfe Field",
		City:      "Shipshewana",
		State:     "Indiana",
		Country:   "US",
		Elevation: 896,
		Latitude:  41.6913986206,
		Longitude: -85.5954971313,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN67": {
		ICAO:      "IN67",
		Name:      "Greener Pastures Airport",
		City:      "North Vernon",
		State:     "Indiana",
		Country:   "US",
		Elevation: 618,
		Latitude:  38.9373016357,
		Longitude: -85.7375030518,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN68": {
		ICAO:      "IN68",
		Name:      "Stewart Airport",
		City:      "Elizabeth",
		State:     "Indiana",
		Country:   "US",
		Elevation: 700,
		Latitude:  38.1355018616,
		Longitude: -86.0186004639,
		Timezone:  "America/Kentucky/Louisville",
	},
	"IN69": {
		ICAO:      "IN69",
		Name:      "Hatfield Airport",
		City:      "Bristol",
		State:     "Indiana",
		Country:   "US",
		Elevation: 860,
		Latitude:  41.665599823,
		Longitude: -85.8321990967,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN70": {
		ICAO:      "IN70",
		Name:      "Bodin Airport",
		City:      "Chesterton",
		State:     "Indiana",
		Country:   "US",
		Elevation: 675,
		Latitude:  41.6335983276,
		Longitude: -87.0270004272,
		Timezone:  "America/Chicago",
	},
	"IN71": {
		ICAO:      "IN71",
		Name:      "Plew Airport",
		City:      "Columbia City",
		State:     "Indiana",
		Country:   "US",
		Elevation: 910,
		Latitude:  41.1856002808,
		Longitude: -85.5215988159,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN72": {
		ICAO:      "IN72",
		Name:      "Mayer Airport",
		City:      "Union Dale",
		State:     "Indiana",
		Country:   "US",
		Elevation: 820,
		Latitude:  40.785900116,
		Longitude: -85.2442016602,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN73": {
		ICAO:      "IN73",
		Name:      "Fleet Field",
		City:      "Culver",
		State:     "Indiana",
		Country:   "US",
		Elevation: 821,
		Latitude:  41.2223014832,
		Longitude: -86.386100769,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN74": {
		ICAO:      "IN74",
		Name:      "H.J.Umbaugh Airport",
		City:      "Culver",
		State:     "Indiana",
		Country:   "US",
		Elevation: 755,
		Latitude:  41.3069992065,
		Longitude: -86.4375,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN76": {
		ICAO:      "IN76",
		Name:      "Podell Airport",
		City:      "Denham",
		State:     "Indiana",
		Country:   "US",
		Elevation: 710,
		Latitude:  41.1263999939,
		Longitude: -86.6932983398,
		Timezone:  "America/Indiana/Winamac",
	},
	"IN80": {
		ICAO:      "IN80",
		Name:      "Roberson Airport",
		City:      "English",
		State:     "Indiana",
		Country:   "US",
		Elevation: 690,
		Latitude:  38.3213996887,
		Longitude: -86.4907989502,
		Timezone:  "America/Indiana/Marengo",
	},
	"IN81": {
		ICAO:      "IN81",
		Name:      "Small Field",
		City:      "Carthage",
		State:     "Indiana",
		Country:   "US",
		Elevation: 920,
		Latitude:  39.7518997192,
		Longitude: -85.5575027466,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN82": {
		ICAO:      "IN82",
		Name:      "Foos Field",
		City:      "Granger",
		State:     "Indiana",
		Country:   "US",
		Elevation: 790,
		Latitude:  41.7495002747,
		Longitude: -86.0880966187,
		Timezone:  "America/Detroit",
	},
	"IN83": {
		ICAO:      "IN83",
		Name:      "Lautzenhiser Airpark",
		City:      "Hamilton",
		State:     "Indiana",
		Country:   "US",
		Elevation: 928,
		Latitude:  41.5023002625,
		Longitude: -84.9301986694,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN84": {
		ICAO:      "IN84",
		Name:      "Shamrock Airport",
		City:      "Hanna",
		State:     "Indiana",
		Country:   "US",
		Elevation: 680,
		Latitude:  41.4081001282,
		Longitude: -86.7472000122,
		Timezone:  "America/Chicago",
	},
	"IN85": {
		ICAO:      "IN85",
		Name:      "Bowlin Airport",
		City:      "Huntington",
		State:     "Indiana",
		Country:   "US",
		Elevation: 810,
		Latitude:  40.8372993469,
		Longitude: -85.4835968018,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN86": {
		ICAO:      "IN86",
		Name:      "Wilson Airport",
		City:      "Knox",
		State:     "Indiana",
		Country:   "US",
		Elevation: 714,
		Latitude:  41.2803001404,
		Longitude: -86.6320037842,
		Timezone:  "America/Indiana/Knox",
	},
	"IN87": {
		ICAO:      "IN87",
		Name:      "Singleton's Landing Strip",
		City:      "Knox",
		State:     "Indiana",
		Country:   "US",
		Elevation: 685,
		Latitude:  41.351398468,
		Longitude: -86.6264038086,
		Timezone:  "America/Indiana/Knox",
	},
	"IN88": {
		ICAO:      "IN88",
		Name:      "Sanders Gyroport Airport",
		City:      "Cloverdale",
		State:     "Indiana",
		Country:   "US",
		Elevation: 760,
		Latitude:  39.5134010315,
		Longitude: -86.7639007568,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN89": {
		ICAO:      "IN89",
		Name:      "Ligonier Airport",
		City:      "Ligonier",
		State:     "Indiana",
		Country:   "US",
		Elevation: 920,
		Latitude:  41.4500999451,
		Longitude: -85.6054992676,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN90": {
		ICAO:      "IN90",
		Name:      "Wietbrock Airport",
		City:      "Lowell",
		State:     "Indiana",
		Country:   "US",
		Elevation: 695,
		Latitude:  41.2597999573,
		Longitude: -87.4813995361,
		Timezone:  "America/Chicago",
	},
	"IN91": {
		ICAO:      "IN91",
		Name:      "Valhalla Airport",
		City:      "Maples",
		State:     "Indiana",
		Country:   "US",
		Elevation: 810,
		Latitude:  41.0023002625,
		Longitude: -84.991897583,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN92": {
		ICAO:      "IN92",
		Name:      "Klein Airport",
		City:      "Vincennes",
		State:     "Indiana",
		Country:   "US",
		Elevation: 412,
		Latitude:  38.6528015137,
		Longitude: -87.5635986328,
		Timezone:  "America/Indiana/Vincennes",
	},
	"IN93": {
		ICAO:      "IN93",
		Name:      "Hustons Airport",
		City:      "New Carlisle",
		State:     "Indiana",
		Country:   "US",
		Elevation: 760,
		Latitude:  41.7120018005,
		Longitude: -86.432800293,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN95": {
		ICAO:      "IN95",
		Name:      "Long Airport",
		City:      "North Judson",
		State:     "Indiana",
		Country:   "US",
		Elevation: 700,
		Latitude:  41.2275009155,
		Longitude: -86.7902984619,
		Timezone:  "America/Indiana/Knox",
	},
	"IN98": {
		ICAO:      "IN98",
		Name:      "Farm Strip",
		City:      "Finly",
		State:     "Indiana",
		Country:   "US",
		Elevation: 825,
		Latitude:  39.6831016541,
		Longitude: -85.8442001343,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IN99": {
		ICAO:      "IN99",
		Name:      "Donica Field",
		City:      "Morgantown",
		State:     "Indiana",
		Country:   "US",
		Elevation: 870,
		Latitude:  39.4067001343,
		Longitude: -86.3054962158,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"IS00": {
		ICAO:      "IS00",
		Name:      "Jay Schertz Farm Airport",
		City:      "Lowpoint",
		State:     "Illinois",
		Country:   "US",
		Elevation: 780,
		Latitude:  40.8417015076,
		Longitude: -89.2744979858,
		Timezone:  "America/Chicago",
	},
	"IS01": {
		ICAO:      "IS01",
		Name:      "Denby Airport",
		City:      "Carlinville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 620,
		Latitude:  39.268901825,
		Longitude: -89.9039993286,
		Timezone:  "America/Chicago",
	},
	"IS02": {
		ICAO:      "IS02",
		Name:      "Dietchweiler Airport",
		City:      "Watseka",
		State:     "Illinois",
		Country:   "US",
		Elevation: 640,
		Latitude:  40.7784004211,
		Longitude: -87.7917022705,
		Timezone:  "America/Chicago",
	},
	"IS08": {
		ICAO:      "IS08",
		Name:      "Curless Airport",
		City:      "Summum",
		State:     "Illinois",
		Country:   "US",
		Elevation: 620,
		Latitude:  40.2627983093,
		Longitude: -90.2368011475,
		Timezone:  "America/Chicago",
	},
	"IS13": {
		ICAO:      "IS13",
		Name:      "Bussart Airport",
		City:      "Dudley",
		State:     "Illinois",
		Country:   "US",
		Elevation: 713,
		Latitude:  39.5792007446,
		Longitude: -87.8486022949,
		Timezone:  "America/Chicago",
	},
	"IS15": {
		ICAO:      "IS15",
		Name:      "Vodden Airport",
		City:      "Esmond",
		State:     "Illinois",
		Country:   "US",
		Elevation: 850,
		Latitude:  42.0325012207,
		Longitude: -88.9026031494,
		Timezone:  "America/Chicago",
	},
	"IS16": {
		ICAO:      "IS16",
		Name:      "Hartline Air Strip STOLport",
		City:      "Anna",
		State:     "Illinois",
		Country:   "US",
		Elevation: 645,
		Latitude:  37.4667015076,
		Longitude: -89.2284011841,
		Timezone:  "America/Chicago",
	},
	"IS19": {
		ICAO:      "IS19",
		Name:      "Carlson RLA Restricted Landing Area",
		City:      "Pecatonica",
		State:     "Illinois",
		Country:   "US",
		Elevation: 740,
		Latitude:  42.3353004456,
		Longitude: -89.323600769,
		Timezone:  "America/Chicago",
	},
	"IS24": {
		ICAO:      "IS24",
		Name:      "Harris Airport",
		City:      "Ramsey",
		State:     "Illinois",
		Country:   "US",
		Elevation: 551,
		Latitude:  39.077778,
		Longitude: -89.081667,
		Timezone:  "America/Chicago",
	},
	"IS26": {
		ICAO:      "IS26",
		Name:      "Niklaus RLA Restricted Landing Area",
		City:      "Farmer City",
		State:     "Illinois",
		Country:   "US",
		Elevation: 725,
		Latitude:  40.1833992004,
		Longitude: -88.6153030396,
		Timezone:  "America/Chicago",
	},
	"IS33": {
		ICAO:      "IS33",
		Name:      "Willhoit Airport",
		City:      "Staunton",
		State:     "Illinois",
		Country:   "US",
		Elevation: 614,
		Latitude:  39.0456008911,
		Longitude: -89.8406982422,
		Timezone:  "America/Chicago",
	},
	"IS34": {
		ICAO:      "IS34",
		Name:      "Piper's Landing Airport",
		City:      "Lawrenceville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 425,
		Latitude:  38.7066993713,
		Longitude: -87.6333999634,
		Timezone:  "America/Chicago",
	},
	"IS37": {
		ICAO:      "IS37",
		Name:      "Stockton Airport",
		City:      "Stockton",
		State:     "Illinois",
		Country:   "US",
		Elevation: 970,
		Latitude:  42.3238983154,
		Longitude: -89.9928970337,
		Timezone:  "America/Chicago",
	},
	"IS40": {
		ICAO:      "IS40",
		Name:      "Eudy Airport",
		City:      "Monroe Center",
		State:     "Illinois",
		Country:   "US",
		Elevation: 785,
		Latitude:  42.0943984985,
		Longitude: -89.0271987915,
		Timezone:  "America/Chicago",
	},
	"IS41": {
		ICAO:      "IS41",
		Name:      "Vogen Restricted Landing Area",
		City:      "Newark",
		State:     "Illinois",
		Country:   "US",
		Elevation: 740,
		Latitude:  41.460899353,
		Longitude: -88.5920028687,
		Timezone:  "America/Chicago",
	},
	"IS43": {
		ICAO:      "IS43",
		Name:      "Riggins Farms Airport",
		City:      "Villa Glove",
		State:     "Illinois",
		Country:   "US",
		Elevation: 699,
		Latitude:  39.8699989319,
		Longitude: -88.0749969482,
		Timezone:  "America/Chicago",
	},
	"IS45": {
		ICAO:      "IS45",
		Name:      "Mast Field",
		City:      "Quincy",
		State:     "Illinois",
		Country:   "US",
		Elevation: 738,
		Latitude:  39.9766998291,
		Longitude: -91.3339996338,
		Timezone:  "America/Chicago",
	},
	"IS47": {
		ICAO:      "IS47",
		Name:      "Willis Airport",
		City:      "Sycamore",
		State:     "Illinois",
		Country:   "US",
		Elevation: 856,
		Latitude:  42.0306015015,
		Longitude: -88.7017974854,
		Timezone:  "America/Chicago",
	},
	"IS51": {
		ICAO:      "IS51",
		Name:      "Schilson Field",
		City:      "Carthage",
		State:     "Illinois",
		Country:   "US",
		Elevation: 705,
		Latitude:  40.4514007568,
		Longitude: -91.2037963867,
		Timezone:  "America/Chicago",
	},
	"IS52": {
		ICAO:      "IS52",
		Name:      "Russell Airport",
		City:      "Donovan",
		State:     "Illinois",
		Country:   "US",
		Elevation: 710,
		Latitude:  40.9277992249,
		Longitude: -87.6224975586,
		Timezone:  "America/Chicago",
	},
	"IS54": {
		ICAO:      "IS54",
		Name:      "Mc Christy Airport",
		City:      "Stonington",
		State:     "Illinois",
		Country:   "US",
		Elevation: 630,
		Latitude:  39.6431007385,
		Longitude: -89.1445007324,
		Timezone:  "America/Chicago",
	},
	"IS55": {
		ICAO:      "IS55",
		Name:      "Morton Airport",
		City:      "La Harpe",
		State:     "Illinois",
		Country:   "US",
		Elevation: 705,
		Latitude:  40.5909004211,
		Longitude: -90.9423980713,
		Timezone:  "America/Chicago",
	},
	"IS56": {
		ICAO:      "IS56",
		Name:      "Herren Airport",
		City:      "Timewell",
		State:     "Illinois",
		Country:   "US",
		Elevation: 654,
		Latitude:  40.0881004333,
		Longitude: -90.8853988647,
		Timezone:  "America/Chicago",
	},
	"IS57": {
		ICAO:      "IS57",
		Name:      "Wind Rose Farm Airport",
		City:      "Marengo",
		State:     "Illinois",
		Country:   "US",
		Elevation: 815,
		Latitude:  42.1796989441,
		Longitude: -88.6327972412,
		Timezone:  "America/Chicago",
	},
	"IS59": {
		ICAO:      "IS59",
		Name:      "Rotstein Airport",
		City:      "Kirkland",
		State:     "Illinois",
		Country:   "US",
		Elevation: 760,
		Latitude:  42.1338996887,
		Longitude: -88.9242019653,
		Timezone:  "America/Chicago",
	},
	"IS60": {
		ICAO:      "IS60",
		Name:      "Nixon Restricted Landing Area",
		City:      "Medora",
		State:     "Illinois",
		Country:   "US",
		Elevation: 610,
		Latitude:  39.19419861,
		Longitude: -90.15609741,
		Timezone:  "America/Chicago",
	},
	"IS62": {
		ICAO:      "IS62",
		Name:      "John D Rennick Airport",
		City:      "Spring Bay",
		State:     "Illinois",
		Country:   "US",
		Elevation: 450,
		Latitude:  40.8672981262,
		Longitude: -89.4531021118,
		Timezone:  "America/Chicago",
	},
	"IS63": {
		ICAO:      "IS63",
		Name:      "Hoffman Airport",
		City:      "De Kalb",
		State:     "Illinois",
		Country:   "US",
		Elevation: 865,
		Latitude:  41.9702987671,
		Longitude: -88.7509002686,
		Timezone:  "America/Chicago",
	},
	"IS64": {
		ICAO:      "IS64",
		Name:      "Kuntz Field",
		City:      "Gridley",
		State:     "Illinois",
		Country:   "US",
		Elevation: 735,
		Latitude:  40.7230987549,
		Longitude: -88.8666992188,
		Timezone:  "America/Chicago",
	},
	"IS65": {
		ICAO:      "IS65",
		Name:      "Woodlake Landing Airport",
		City:      "Sandwich",
		State:     "Illinois",
		Country:   "US",
		Elevation: 677,
		Latitude:  41.6384010315,
		Longitude: -88.6445007324,
		Timezone:  "America/Chicago",
	},
	"IS66": {
		ICAO:      "IS66",
		Name:      "Rhea Restricted Landing Area",
		City:      "Pontoosuc",
		State:     "Illinois",
		Country:   "US",
		Elevation: 684,
		Latitude:  40.592300415,
		Longitude: -91.201499939,
		Timezone:  "America/Chicago",
	},
	"IS69": {
		ICAO:      "IS69",
		Name:      "Kloker Airport",
		City:      "Meredosia",
		State:     "Illinois",
		Country:   "US",
		Elevation: 440,
		Latitude:  39.9188995361,
		Longitude: -90.5110015869,
		Timezone:  "America/Chicago",
	},
	"IS71": {
		ICAO:      "IS71",
		Name:      "Koch Airport",
		City:      "Shirland",
		State:     "Illinois",
		Country:   "US",
		Elevation: 800,
		Latitude:  42.4583015442,
		Longitude: -89.1949996948,
		Timezone:  "America/Chicago",
	},
	"IS73": {
		ICAO:      "IS73",
		Name:      "Woodley Aerial Spray Airport",
		City:      "New Bedford",
		State:     "Illinois",
		Country:   "US",
		Elevation: 635,
		Latitude:  41.5325012207,
		Longitude: -89.7018966675,
		Timezone:  "America/Chicago",
	},
	"IS77": {
		ICAO:      "IS77",
		Name:      "Hoehn RLA Restricted Landing Area",
		City:      "Witt",
		State:     "Illinois",
		Country:   "US",
		Elevation: 650,
		Latitude:  39.2207984924,
		Longitude: -89.3328018188,
		Timezone:  "America/Chicago",
	},
	"IS78": {
		ICAO:      "IS78",
		Name:      "Early - Merkel Field",
		City:      "Pecatonica",
		State:     "Illinois",
		Country:   "US",
		Elevation: 794,
		Latitude:  42.3666992188,
		Longitude: -89.2319030762,
		Timezone:  "America/Chicago",
	},
	"IS80": {
		ICAO:      "IS80",
		Name:      "Uncle Chuck's Airport",
		City:      "De Kalb",
		State:     "Illinois",
		Country:   "US",
		Elevation: 880,
		Latitude:  41.9178009033,
		Longitude: -88.7892990112,
		Timezone:  "America/Chicago",
	},
	"IS82": {
		ICAO:      "IS82",
		Name:      "Arras RLA Restricted Landing Area",
		City:      "Greenfield",
		State:     "Illinois",
		Country:   "US",
		Elevation: 598,
		Latitude:  39.3381004333,
		Longitude: -90.1781005859,
		Timezone:  "America/Chicago",
	},
	"IS83": {
		ICAO:      "IS83",
		Name:      "Untied Acres Airport",
		City:      "Belvidere",
		State:     "Illinois",
		Country:   "US",
		Elevation: 800,
		Latitude:  42.2275009155,
		Longitude: -88.7671966553,
		Timezone:  "America/Chicago",
	},
	"IS85": {
		ICAO:      "IS85",
		Name:      "Newton Airport",
		City:      "Patterson",
		State:     "Illinois",
		Country:   "US",
		Elevation: 625,
		Latitude:  39.5047988892,
		Longitude: -90.5054016113,
		Timezone:  "America/Chicago",
	},
	"IS86": {
		ICAO:      "IS86",
		Name:      "Hendrix Airport",
		City:      "Buckingham",
		State:     "Illinois",
		Country:   "US",
		Elevation: 656,
		Latitude:  41.0416984558,
		Longitude: -88.1917037964,
		Timezone:  "America/Chicago",
	},
	"IS88": {
		ICAO:      "IS88",
		Name:      "Melody Field",
		City:      "Danville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 660,
		Latitude:  40.148601532,
		Longitude: -87.6945037842,
		Timezone:  "America/Chicago",
	},
	"IS93": {
		ICAO:      "IS93",
		Name:      "Sue Rock International Airport",
		City:      "Rock City",
		State:     "Illinois",
		Country:   "US",
		Elevation: 950,
		Latitude:  42.4425010681,
		Longitude: -89.4992980957,
		Timezone:  "America/Chicago",
	},
	"IS94": {
		ICAO:      "IS94",
		Name:      "Johnson Airport",
		City:      "Sandwich",
		State:     "Illinois",
		Country:   "US",
		Elevation: 667,
		Latitude:  41.6055984497,
		Longitude: -88.6612014771,
		Timezone:  "America/Chicago",
	},
	"IS95": {
		ICAO:      "IS95",
		Name:      "Harrington Farms Airport",
		City:      "Sheridan",
		State:     "Illinois",
		Country:   "US",
		Elevation: 640,
		Latitude:  41.5586013794,
		Longitude: -88.6644973755,
		Timezone:  "America/Chicago",
	},
	"JB01": {
		ICAO:      "JB01",
		Name:      "Clearwater Aero Estates Airport",
		City:      "Wisconsin Dells",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 944,
		Latitude:  43.7571983337,
		Longitude: -89.6526031494,
		Timezone:  "America/Chicago",
	},
	"JY04": {
		ICAO:      "JY04",
		Name:      "Paramount Air Airport",
		City:      "Green Creek",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 8,
		Latitude:  39.0651016235,
		Longitude: -74.9095993042,
		Timezone:  "America/New_York",
	},
	"JY07": {
		ICAO:      "JY07",
		Name:      "Air-List-Ads Airport",
		City:      "Stewartsville",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 360,
		Latitude:  40.6957015991,
		Longitude: -75.1324005127,
		Timezone:  "America/New_York",
	},
	"JY08": {
		ICAO:      "JY08",
		Name:      "Var-Sky Airport",
		City:      "Woodstown",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 135,
		Latitude:  39.650100708,
		Longitude: -75.2841033936,
		Timezone:  "America/New_York",
	},
	"JY17": {
		ICAO:      "JY17",
		Name:      "Woodcrest Farms Airstrip",
		City:      "Shiloh",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 90,
		Latitude:  39.4721984863,
		Longitude: -75.2774963379,
		Timezone:  "America/New_York",
	},
	"JY24": {
		ICAO:      "JY24",
		Name:      "Weiss Farm Airport",
		City:      "Allamuchy",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 532,
		Latitude:  40.9236984253,
		Longitude: -74.8703994751,
		Timezone:  "America/New_York",
	},
	"JY31": {
		ICAO:      "JY31",
		Name:      "Wide Sky Airpark",
		City:      "Bridgeton",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 100,
		Latitude:  39.4468002319,
		Longitude: -75.3132019043,
		Timezone:  "America/New_York",
	},
	"JY39": {
		ICAO:      "JY39",
		Name:      "Rainbow'S End Airport",
		City:      "Salem",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 11,
		Latitude:  39.661111,
		Longitude: -75.465278,
		Timezone:  "America/New_York",
	},
	"JY43": {
		ICAO:      "JY43",
		Name:      "Hill Top Airport",
		City:      "West Milford",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 921,
		Latitude:  41.08322144,
		Longitude: -74.33855438,
		Timezone:  "America/New_York",
	},
	"K00C": {
		ICAO:      "K00C",
		IATA:      "AMK",
		Name:      "Animas Air Park",
		City:      "Durango",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6684,
		Latitude:  37.2032012939,
		Longitude: -107.869003296,
		Timezone:  "America/Denver",
	},
	"K00F": {
		ICAO:      "K00F",
		IATA:      "BDX",
		Name:      "Broadus Airport",
		City:      "Broadus",
		State:     "Montana",
		Country:   "US",
		Elevation: 3280,
		Latitude:  45.47249985,
		Longitude: -105.4540024,
		Timezone:  "America/Denver",
	},
	"K00M": {
		ICAO:      "K00M",
		Name:      "Thigpen Field",
		City:      "Bay Springs",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 351,
		Latitude:  31.9538002014,
		Longitude: -89.2344970703,
		Timezone:  "America/Chicago",
	},
	"K00R": {
		ICAO:      "K00R",
		Name:      "Livingston Municipal Airport",
		City:      "Livingston",
		State:     "Texas",
		Country:   "US",
		Elevation: 151,
		Latitude:  30.6858997345,
		Longitude: -95.0178985596,
		Timezone:  "America/Chicago",
	},
	"K01G": {
		ICAO:      "K01G",
		Name:      "Perry-Warsaw Airport",
		City:      "Perry",
		State:     "New-York",
		Country:   "US",
		Elevation: 1559,
		Latitude:  42.7412986755,
		Longitude: -78.0521011353,
		Timezone:  "America/New_York",
	},
	"K01M": {
		ICAO:      "K01M",
		Name:      "Tishomingo County Airport",
		City:      "Belmont",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 578,
		Latitude:  34.4916992188,
		Longitude: -88.2011032104,
		Timezone:  "America/Chicago",
	},
	"K02A": {
		ICAO:      "K02A",
		Name:      "Gragg-Wade Field",
		City:      "Clanton",
		State:     "Alabama",
		Country:   "US",
		Elevation: 585,
		Latitude:  32.8504981995,
		Longitude: -86.6113967896,
		Timezone:  "America/Chicago",
	},
	"K02C": {
		ICAO:      "K02C",
		Name:      "Capitol Airport",
		City:      "Brookfield",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 850,
		Latitude:  43.0902222222,
		Longitude: -88.1781602778,
		Timezone:  "America/Chicago",
	},
	"K02G": {
		ICAO:      "K02G",
		Name:      "Columbiana County Airport",
		City:      "East Liverpool",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1160,
		Latitude:  40.6733016968,
		Longitude: -80.6414031982,
		Timezone:  "America/New_York",
	},
	"K03B": {
		ICAO:      "K03B",
		Name:      "Mansfield Municipal Airport",
		City:      "Mansfield",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1500,
		Latitude:  37.1277999878,
		Longitude: -92.6210021973,
		Timezone:  "America/Chicago",
	},
	"K03D": {
		ICAO:      "K03D",
		Name:      "Memphis Memorial Airport",
		City:      "Memphis",
		State:     "Missouri",
		Country:   "US",
		Elevation: 813,
		Latitude:  40.4472999573,
		Longitude: -92.2269973755,
		Timezone:  "America/Chicago",
	},
	"K04A": {
		ICAO:      "K04A",
		Name:      "Frank Sikes Airport",
		City:      "Luverne",
		State:     "Alabama",
		Country:   "US",
		Elevation: 300,
		Latitude:  31.7362995148,
		Longitude: -86.2624969482,
		Timezone:  "America/Chicago",
	},
	"K04G": {
		ICAO:      "K04G",
		Name:      "Lansdowne Airport",
		City:      "Youngstown",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1044,
		Latitude:  41.1305007935,
		Longitude: -80.6195983887,
		Timezone:  "America/New_York",
	},
	"K04M": {
		ICAO:      "K04M",
		Name:      "Calhoun County Airport",
		City:      "Pittsboro",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 383,
		Latitude:  33.9300994873,
		Longitude: -89.3427963257,
		Timezone:  "America/Chicago",
	},
	"K04Y": {
		ICAO:      "K04Y",
		Name:      "Hawley Municipal Airport",
		City:      "Hawley",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1208,
		Latitude:  46.8838005066,
		Longitude: -96.3503036499,
		Timezone:  "America/Chicago",
	},
	"K05C": {
		ICAO:      "K05C",
		Name:      "Griffith-Merrillville Airport",
		City:      "Griffith",
		State:     "Indiana",
		Country:   "US",
		Elevation: 634,
		Latitude:  41.5197982788,
		Longitude: -87.3994979858,
		Timezone:  "America/Chicago",
	},
	"K05D": {
		ICAO:      "K05D",
		Name:      "New Town Municipal Airport",
		City:      "New Town",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1925,
		Latitude:  47.966999054,
		Longitude: -102.4779968262,
		Timezone:  "America/Chicago",
	},
	"K05U": {
		ICAO:      "K05U",
		Name:      "Eureka Airport",
		City:      "Eureka",
		State:     "Nevada",
		Country:   "US",
		Elevation: 5954,
		Latitude:  39.6041984558,
		Longitude: -116.0049972534,
		Timezone:  "America/Los_Angeles",
	},
	"K06A": {
		ICAO:      "K06A",
		Name:      "Moton Field Municipal Airport",
		City:      "Tuskegee",
		State:     "Alabama",
		Country:   "US",
		Elevation: 264,
		Latitude:  32.4604988098,
		Longitude: -85.6800003052,
		Timezone:  "America/Chicago",
	},
	"K06C": {
		ICAO:      "K06C",
		Name:      "Schaumburg Regional Airport",
		City:      "Chicago/Schaumburg",
		State:     "Illinois",
		Country:   "US",
		Elevation: 801,
		Latitude:  41.9892997742,
		Longitude: -88.1011962891,
		Timezone:  "America/Chicago",
	},
	"K06D": {
		ICAO:      "K06D",
		Name:      "Rolla Municipal Airport",
		City:      "Rolla",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1823,
		Latitude:  48.88430023,
		Longitude: -99.62090302,
		Timezone:  "America/Chicago",
	},
	"K06M": {
		ICAO:      "K06M",
		Name:      "Eupora Airport",
		City:      "Eupora",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 450,
		Latitude:  33.5345993042,
		Longitude: -89.3125991821,
		Timezone:  "America/Chicago",
	},
	"K06U": {
		ICAO:      "K06U",
		Name:      "Jackpot/Hayden Field",
		City:      "Jackpot",
		State:     "Nevada",
		Country:   "US",
		Elevation: 5213,
		Latitude:  41.9760017395,
		Longitude: -114.657997131,
		Timezone:  "America/Boise",
	},
	"K07A": {
		ICAO:      "K07A",
		Name:      "Franklin Field",
		City:      "Union Springs",
		State:     "Alabama",
		Country:   "US",
		Elevation: 300,
		Latitude:  32.1668014526,
		Longitude: -85.8097000122,
		Timezone:  "America/Chicago",
	},
	"K07F": {
		ICAO:      "K07F",
		Name:      "Gladewater Municipal Airport",
		City:      "Gladewater",
		State:     "Texas",
		Country:   "US",
		Elevation: 296,
		Latitude:  32.5288009644,
		Longitude: -94.9717025757,
		Timezone:  "America/Chicago",
	},
	"K07R": {
		ICAO:      "K07R",
		Name:      "Bishop Municipal Airport",
		City:      "Bishop",
		State:     "Texas",
		Country:   "US",
		Elevation: 55,
		Latitude:  27.6103000641,
		Longitude: -97.7518997192,
		Timezone:  "America/Chicago",
	},
	"K07V": {
		ICAO:      "K07V",
		Name:      "Cuchara Valley At La Veta Airport",
		City:      "La Veta",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7153,
		Latitude:  37.5237998962,
		Longitude: -105.0090026855,
		Timezone:  "America/Denver",
	},
	"K08C": {
		ICAO:      "K08C",
		Name:      "Riverview Airport",
		City:      "Jenison",
		State:     "Michigan",
		Country:   "US",
		Elevation: 603,
		Latitude:  42.9359016418,
		Longitude: -85.8050003052,
		Timezone:  "America/Detroit",
	},
	"K08D": {
		ICAO:      "K08D",
		Name:      "Stanley Municipal Airport",
		City:      "Stanley",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 2245,
		Latitude:  48.3008003235,
		Longitude: -102.40599823,
		Timezone:  "America/Chicago",
	},
	"K08K": {
		ICAO:      "K08K",
		Name:      "Harvard State Airport",
		City:      "Harvard",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1815,
		Latitude:  40.6514015198,
		Longitude: -98.0798034668,
		Timezone:  "America/Chicago",
	},
	"K08M": {
		ICAO:      "K08M",
		Name:      "Carthage-Leake County Airport",
		City:      "Carthage",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 454,
		Latitude:  32.7611999512,
		Longitude: -89.5300979614,
		Timezone:  "America/Chicago",
	},
	"K09A": {
		ICAO:      "K09A",
		Name:      "Butler-Choctaw County Airport",
		City:      "Butler",
		State:     "Alabama",
		Country:   "US",
		Elevation: 134,
		Latitude:  32.1193008423,
		Longitude: -88.1275024414,
		Timezone:  "America/Chicago",
	},
	"K09K": {
		ICAO:      "K09K",
		Name:      "Sargent Municipal Airport",
		City:      "Sargent",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2313,
		Latitude:  41.6370010376,
		Longitude: -99.3404006958,
		Timezone:  "America/Chicago",
	},
	"K09M": {
		ICAO:      "K09M",
		Name:      "Charleston Municipal Airport",
		City:      "Charleston",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 175,
		Latitude:  33.9915008545,
		Longitude: -90.0781021118,
		Timezone:  "America/Chicago",
	},
	"K09R": {
		ICAO:      "K09R",
		Name:      "Tyler County Airport",
		City:      "Woodville",
		State:     "Texas",
		Country:   "US",
		Elevation: 388,
		Latitude:  30.7751998901,
		Longitude: -94.4585037231,
		Timezone:  "America/Chicago",
	},
	"K0A2": {
		ICAO:      "K0A2",
		Name:      "Hester Memorial Airport",
		City:      "Calhoun Falls",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 517,
		Latitude:  34.0904006958,
		Longitude: -82.5679016113,
		Timezone:  "America/New_York",
	},
	"K0A3": {
		ICAO:      "K0A3",
		Name:      "Smithville Municipal Airport",
		City:      "Smithville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1084,
		Latitude:  35.9852981567,
		Longitude: -85.8093032837,
		Timezone:  "America/Chicago",
	},
	"K0A4": {
		ICAO:      "K0A4",
		Name:      "Johnson City Stolport Airport",
		City:      "Johnson City",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1550,
		Latitude:  36.3637008667,
		Longitude: -82.3089981079,
		Timezone:  "America/New_York",
	},
	"K0A7": {
		ICAO:      "K0A7",
		Name:      "Hendersonville Airport",
		City:      "Hendersonville",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 2084,
		Latitude:  35.307598114,
		Longitude: -82.4331970215,
		Timezone:  "America/New_York",
	},
	"K0A8": {
		ICAO:      "K0A8",
		Name:      "Bibb County Airport",
		City:      "Centreville",
		State:     "Alabama",
		Country:   "US",
		Elevation: 251,
		Latitude:  32.9367980957,
		Longitude: -87.0888977051,
		Timezone:  "America/Chicago",
	},
	"K0A9": {
		ICAO:      "K0A9",
		Name:      "Elizabethton Municipal Airport",
		City:      "Elizabethton",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1593,
		Latitude:  36.3712005615,
		Longitude: -82.1733016968,
		Timezone:  "America/New_York",
	},
	"K0B1": {
		ICAO:      "K0B1",
		Name:      "Bethel Regional Airport",
		City:      "Bethel",
		State:     "Maine",
		Country:   "US",
		Elevation: 654,
		Latitude:  44.425201416,
		Longitude: -70.8098983765,
		Timezone:  "America/New_York",
	},
	"K0B4": {
		ICAO:      "K0B4",
		Name:      "Hartington Municipal Airport",
		City:      "Hartington",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1387,
		Latitude:  42.6035995483,
		Longitude: -97.2526016235,
		Timezone:  "America/Chicago",
	},
	"K0B5": {
		ICAO:      "K0B5",
		Name:      "Turners Falls Airport",
		City:      "Montague",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 356,
		Latitude:  42.5914001465,
		Longitude: -72.5227966309,
		Timezone:  "America/New_York",
	},
	"K0B8": {
		ICAO:      "K0B8",
		IATA:      "FID",
		Name:      "Elizabeth Field",
		City:      "Fishers Island",
		State:     "New-York",
		Country:   "US",
		Elevation: 9,
		Latitude:  41.2513008118,
		Longitude: -72.0316009521,
		Timezone:  "America/New_York",
	},
	"K0C0": {
		ICAO:      "K0C0",
		Name:      "Dacy Airport",
		City:      "Harvard",
		State:     "Illinois",
		Country:   "US",
		Elevation: 913,
		Latitude:  42.4025001526,
		Longitude: -88.6324005127,
		Timezone:  "America/Chicago",
	},
	"K0C4": {
		ICAO:      "K0C4",
		Name:      "Pender Municipal Airport",
		City:      "Pender",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1345,
		Latitude:  42.11389923,
		Longitude: -96.72889709,
		Timezone:  "America/Chicago",
	},
	"K0D8": {
		ICAO:      "K0D8",
		Name:      "Gettysburg Municipal Airport",
		City:      "Gettysburg",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 2062,
		Latitude:  44.98669815,
		Longitude: -99.95279694,
		Timezone:  "America/Chicago",
	},
	"K0F2": {
		ICAO:      "K0F2",
		Name:      "Bowie Municipal Airport",
		City:      "Bowie",
		State:     "Texas",
		Country:   "US",
		Elevation: 1101,
		Latitude:  33.6016998291,
		Longitude: -97.7755966187,
		Timezone:  "America/Chicago",
	},
	"K0F4": {
		ICAO:      "K0F4",
		Name:      "Loup City Municipal Airport",
		City:      "Loup City",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2071,
		Latitude:  41.2902984619,
		Longitude: -98.9906005859,
		Timezone:  "America/Chicago",
	},
	"K0F7": {
		ICAO:      "K0F7",
		Name:      "Fountainhead Lodge Airpark",
		City:      "Eufaula",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 670,
		Latitude:  35.3886985779,
		Longitude: -95.5999984741,
		Timezone:  "America/Chicago",
	},
	"K0F9": {
		ICAO:      "K0F9",
		Name:      "Tishomingo Airpark",
		City:      "Tishomingo",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 647,
		Latitude:  34.1985015869,
		Longitude: -96.6744995117,
		Timezone:  "America/Chicago",
	},
	"K0G3": {
		ICAO:      "K0G3",
		Name:      "Tecumseh Municipal Airport",
		City:      "Tecumseh",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1306,
		Latitude:  40.3993988,
		Longitude: -96.17140198,
		Timezone:  "America/Chicago",
	},
	"K0G6": {
		ICAO:      "K0G6",
		Name:      "Williams County Airport",
		City:      "Bryan",
		State:     "Ohio",
		Country:   "US",
		Elevation: 730,
		Latitude:  41.4673995972,
		Longitude: -84.5065994263,
		Timezone:  "America/New_York",
	},
	"K0G7": {
		ICAO:      "K0G7",
		Name:      "Finger Lakes Regional Airport",
		City:      "Seneca Falls",
		State:     "New-York",
		Country:   "US",
		Elevation: 492,
		Latitude:  42.8835983276,
		Longitude: -76.7810974121,
		Timezone:  "America/New_York",
	},
	"K0H1": {
		ICAO:      "K0H1",
		Name:      "Trego Wakeeney Airport",
		City:      "Wakeeney",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2435,
		Latitude:  39.0045013428,
		Longitude: -99.8928985596,
		Timezone:  "America/Chicago",
	},
	"K0I8": {
		ICAO:      "K0I8",
		Name:      "Cynthiana-Harrison County Airport",
		City:      "Cynthiana",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 721,
		Latitude:  38.3661994934,
		Longitude: -84.2833023071,
		Timezone:  "America/New_York",
	},
	"K0J4": {
		ICAO:      "K0J4",
		Name:      "Florala Municipal Airport",
		City:      "Florala",
		State:     "Alabama",
		Country:   "US",
		Elevation: 314,
		Latitude:  31.0424995422,
		Longitude: -86.3115997314,
		Timezone:  "America/Chicago",
	},
	"K0J6": {
		ICAO:      "K0J6",
		Name:      "Headland Municipal Airport",
		City:      "Headland",
		State:     "Alabama",
		Country:   "US",
		Elevation: 359,
		Latitude:  31.36429977,
		Longitude: -85.3117981,
		Timezone:  "America/Chicago",
	},
	"K0J9": {
		ICAO:      "K0J9",
		Name:      "Flying V Airport",
		City:      "Utica",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1585,
		Latitude:  40.8678016663,
		Longitude: -97.3542022705,
		Timezone:  "America/Chicago",
	},
	"K0K7": {
		ICAO:      "K0K7",
		Name:      "Humboldt Municipal Airport",
		City:      "Humboldt",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1093,
		Latitude:  42.7360992432,
		Longitude: -94.2452011108,
		Timezone:  "America/Chicago",
	},
	"K0L7": {
		ICAO:      "K0L7",
		Name:      "Jean Airport",
		City:      "Jean",
		State:     "Nevada",
		Country:   "US",
		Elevation: 2832,
		Latitude:  35.7682991028,
		Longitude: -115.3300018311,
		Timezone:  "America/Los_Angeles",
	},
	"K0L9": {
		ICAO:      "K0L9",
		Name:      "Echo Bay Airport",
		City:      "Overton",
		State:     "Nevada",
		Country:   "US",
		Elevation: 1535,
		Latitude:  36.3111000061,
		Longitude: -114.4639968872,
		Timezone:  "America/Los_Angeles",
	},
	"K0M0": {
		ICAO:      "K0M0",
		Name:      "Billy Free Municipal Airport",
		City:      "Dumas",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 163,
		Latitude:  33.8847999573,
		Longitude: -91.5345001221,
		Timezone:  "America/Chicago",
	},
	"K0M1": {
		ICAO:      "K0M1",
		Name:      "Scott Field",
		City:      "Parsons",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 520,
		Latitude:  35.6377983093,
		Longitude: -88.1279983521,
		Timezone:  "America/Chicago",
	},
	"K0M2": {
		ICAO:      "K0M2",
		Name:      "Reelfoot Lake Airport",
		City:      "Tiptonville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 289,
		Latitude:  36.4752998352,
		Longitude: -89.3461990356,
		Timezone:  "America/Chicago",
	},
	"K0M3": {
		ICAO:      "K0M3",
		Name:      "John A Baker Field",
		City:      "Hohenwald",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 972,
		Latitude:  35.5460014343,
		Longitude: -87.5972976685,
		Timezone:  "America/Chicago",
	},
	"K0M4": {
		ICAO:      "K0M4",
		Name:      "Benton County Airport",
		City:      "Camden",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 468,
		Latitude:  36.011100769,
		Longitude: -88.123298645,
		Timezone:  "America/Chicago",
	},
	"K0M5": {
		ICAO:      "K0M5",
		Name:      "Humphreys County Airport",
		City:      "Waverly",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 756,
		Latitude:  36.1166000366,
		Longitude: -87.7381973267,
		Timezone:  "America/Chicago",
	},
	"K0M8": {
		ICAO:      "K0M8",
		Name:      "Byerley Airport",
		City:      "Lake Providence",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 106,
		Latitude:  32.8259010315,
		Longitude: -91.1876983643,
		Timezone:  "America/Chicago",
	},
	"K0M9": {
		ICAO:      "K0M9",
		Name:      "Delhi Municipal Airport",
		City:      "Delhi",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 91,
		Latitude:  32.4107017517,
		Longitude: -91.4987030029,
		Timezone:  "America/Chicago",
	},
	"K0O2": {
		ICAO:      "K0O2",
		Name:      "Baker Airport",
		City:      "Baker",
		State:     "California",
		Country:   "US",
		Elevation: 922,
		Latitude:  35.2854995728,
		Longitude: -116.0830001831,
		Timezone:  "America/Los_Angeles",
	},
	"K0Q5": {
		ICAO:      "K0Q5",
		Name:      "Shelter Cove Airport",
		City:      "Shelter Cove",
		State:     "California",
		Country:   "US",
		Elevation: 69,
		Latitude:  40.0275993347,
		Longitude: -124.0729980469,
		Timezone:  "America/Los_Angeles",
	},
	"K0R0": {
		ICAO:      "K0R0",
		Name:      "Columbia-Marion County Airport",
		City:      "Columbia",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 265,
		Latitude:  31.297000885,
		Longitude: -89.8127975464,
		Timezone:  "America/Chicago",
	},
	"K0R1": {
		ICAO:      "K0R1",
		Name:      "Atmore Municipal Airport",
		City:      "Atmore",
		State:     "Alabama",
		Country:   "US",
		Elevation: 287,
		Latitude:  31.0161991119,
		Longitude: -87.4468002319,
		Timezone:  "America/Chicago",
	},
	"K0R3": {
		ICAO:      "K0R3",
		Name:      "Abbeville Chris Crusta Memorial Airport",
		City:      "Abbeville",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 16,
		Latitude:  29.97579956,
		Longitude: -92.084198,
		Timezone:  "America/Chicago",
	},
	"K0R4": {
		ICAO:      "K0R4",
		Name:      "Concordia Parish Airport",
		City:      "Vidalia",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 54,
		Latitude:  31.5620002747,
		Longitude: -91.5065002441,
		Timezone:  "America/Chicago",
	},
	"K0R5": {
		ICAO:      "K0R5",
		Name:      "David G Joyce Airport",
		City:      "Winnfield",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 146,
		Latitude:  31.9636993408,
		Longitude: -92.6603012085,
		Timezone:  "America/Chicago",
	},
	"K0R6": {
		ICAO:      "K0R6",
		Name:      "Hampton Municipal Airport",
		City:      "Hampton",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 178,
		Latitude:  33.5225982666,
		Longitude: -92.4602966309,
		Timezone:  "America/Chicago",
	},
	"K0S0": {
		ICAO:      "K0S0",
		Name:      "Lind Airport",
		City:      "Lind",
		State:     "Washington",
		Country:   "US",
		Elevation: 1507,
		Latitude:  46.9767990112,
		Longitude: -118.5869979858,
		Timezone:  "America/Los_Angeles",
	},
	"K0S7": {
		ICAO:      "K0S7",
		Name:      "Dorothy Scott Airport",
		City:      "Oroville",
		State:     "Washington",
		Country:   "US",
		Elevation: 1064,
		Latitude:  48.9589996338,
		Longitude: -119.4120025635,
		Timezone:  "America/Los_Angeles",
	},
	"K0S9": {
		ICAO:      "K0S9",
		Name:      "Jefferson County International Airport",
		City:      "Port Townsend",
		State:     "Washington",
		Country:   "US",
		Elevation: 108,
		Latitude:  48.0537986755,
		Longitude: -122.8109970093,
		Timezone:  "America/Los_Angeles",
	},
	"K0V3": {
		ICAO:      "K0V3",
		Name:      "Pioneer Village Field",
		City:      "Minden",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2160,
		Latitude:  40.5149002075,
		Longitude: -98.945602417,
		Timezone:  "America/Chicago",
	},
	"K0V4": {
		ICAO:      "K0V4",
		Name:      "Brookneal/Campbell County Airport",
		City:      "Brookneal",
		State:     "Virginia",
		Country:   "US",
		Elevation: 596,
		Latitude:  37.1417007446,
		Longitude: -79.0164031982,
		Timezone:  "America/New_York",
	},
	"K0V6": {
		ICAO:      "K0V6",
		Name:      "Mission Sioux Airport",
		City:      "Mission",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 2605,
		Latitude:  43.3069000244,
		Longitude: -100.6279983521,
		Timezone:  "America/Chicago",
	},
	"K0V7": {
		ICAO:      "K0V7",
		Name:      "Kayenta Airport",
		City:      "Kayenta",
		State:     "Arizona",
		Country:   "US",
		Elevation: 5710,
		Latitude:  36.7097015381,
		Longitude: -110.2369995117,
		Timezone:  "America/Denver",
	},
	"K0VG": {
		ICAO:      "K0VG",
		Name:      "Lee County Airport",
		City:      "Jonesville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1411,
		Latitude:  36.6540985107,
		Longitude: -83.2177963257,
		Timezone:  "America/New_York",
	},
	"K0W3": {
		ICAO:      "K0W3",
		Name:      "Harford County Airport",
		City:      "Churchville",
		State:     "Maryland",
		Country:   "US",
		Elevation: 409,
		Latitude:  39.5667991638,
		Longitude: -76.2024002075,
		Timezone:  "America/New_York",
	},
	"K10C": {
		ICAO:      "K10C",
		Name:      "Galt Field",
		City:      "Greenwood/Wonder Lake",
		State:     "Illinois",
		Country:   "US",
		Elevation: 875,
		Latitude:  42.4029006958,
		Longitude: -88.3750991821,
		Timezone:  "America/Chicago",
	},
	"K10G": {
		ICAO:      "K10G",
		Name:      "Holmes County Airport",
		City:      "Millersburg",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1218,
		Latitude:  40.5372009277,
		Longitude: -81.9543991089,
		Timezone:  "America/New_York",
	},
	"K10U": {
		ICAO:      "K10U",
		Name:      "Owyhee Airport",
		City:      "Owyhee",
		State:     "Nevada",
		Country:   "US",
		Elevation: 5374,
		Latitude:  41.9532012939,
		Longitude: -116.18800354,
		Timezone:  "America/Los_Angeles",
	},
	"K11A": {
		ICAO:      "K11A",
		Name:      "Clayton Municipal Airport",
		City:      "Clayton",
		State:     "Alabama",
		Country:   "US",
		Elevation: 435,
		Latitude:  31.8833007813,
		Longitude: -85.4849014282,
		Timezone:  "America/Chicago",
	},
	"K11R": {
		ICAO:      "K11R",
		Name:      "Brenham Municipal Airport",
		City:      "Brenham",
		State:     "Texas",
		Country:   "US",
		Elevation: 307,
		Latitude:  30.21899986,
		Longitude: -96.3742981,
		Timezone:  "America/Chicago",
	},
	"K11V": {
		ICAO:      "K11V",
		Name:      "Easton/Valley View Airport",
		City:      "Greeley",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4820,
		Latitude:  40.3305015564,
		Longitude: -104.60900116,
		Timezone:  "America/Denver",
	},
	"K12D": {
		ICAO:      "K12D",
		Name:      "Tower Municipal Airport",
		City:      "Tower",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1369,
		Latitude:  47.8182983398,
		Longitude: -92.2917022705,
		Timezone:  "America/Chicago",
	},
	"K12G": {
		ICAO:      "K12G",
		Name:      "Shelby Community Airport",
		City:      "Shelby",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1120,
		Latitude:  40.8728981018,
		Longitude: -82.6974029541,
		Timezone:  "America/New_York",
	},
	"K12J": {
		ICAO:      "K12J",
		Name:      "Brewton Municipal Airport",
		City:      "Brewton",
		State:     "Alabama",
		Country:   "US",
		Elevation: 99,
		Latitude:  31.0506000519,
		Longitude: -87.0655975342,
		Timezone:  "America/Chicago",
	},
	"K12K": {
		ICAO:      "K12K",
		Name:      "Superior Municipal Airport",
		City:      "Superior",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1691,
		Latitude:  40.0463981628,
		Longitude: -98.0600967407,
		Timezone:  "America/Chicago",
	},
	"K12N": {
		ICAO:      "K12N",
		Name:      "Aeroflex-Andover Airport",
		City:      "Andover",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 583,
		Latitude:  41.0085983276,
		Longitude: -74.7379989624,
		Timezone:  "America/New_York",
	},
	"K12V": {
		ICAO:      "K12V",
		Name:      "Ona Airpark",
		City:      "Milton",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 572,
		Latitude:  38.4408988953,
		Longitude: -82.2007980347,
		Timezone:  "America/New_York",
	},
	"K12Y": {
		ICAO:      "K12Y",
		Name:      "Le Sueur Municipal Airport",
		City:      "Le Sueur",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 868,
		Latitude:  44.4375,
		Longitude: -93.9126968384,
		Timezone:  "America/Chicago",
	},
	"K13C": {
		ICAO:      "K13C",
		Name:      "Lakeview-Airport-Griffith Field",
		City:      "Lakeview",
		State:     "Michigan",
		Country:   "US",
		Elevation: 969,
		Latitude:  43.4520988464,
		Longitude: -85.2648010254,
		Timezone:  "America/Detroit",
	},
	"K13K": {
		ICAO:      "K13K",
		Name:      "Eureka Municipal Airport",
		City:      "Eureka",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1206,
		Latitude:  37.851600647,
		Longitude: -96.2917022705,
		Timezone:  "America/Chicago",
	},
	"K14A": {
		ICAO:      "K14A",
		Name:      "Lake Norman Airpark",
		City:      "Mooresville",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 839,
		Latitude:  35.6129989624,
		Longitude: -80.8993988037,
		Timezone:  "America/New_York",
	},
	"K14F": {
		ICAO:      "K14F",
		Name:      "Hamlin Municipal Airport",
		City:      "Hamlin",
		State:     "Texas",
		Country:   "US",
		Elevation: 1753,
		Latitude:  32.8501014709,
		Longitude: -100.1380004883,
		Timezone:  "America/Chicago",
	},
	"K14G": {
		ICAO:      "K14G",
		Name:      "Fremont Airport",
		City:      "Fremont",
		State:     "Ohio",
		Country:   "US",
		Elevation: 663,
		Latitude:  41.3330993652,
		Longitude: -83.1612014771,
		Timezone:  "America/New_York",
	},
	"K14J": {
		ICAO:      "K14J",
		Name:      "Carl Folsom Airport",
		City:      "Elba",
		State:     "Alabama",
		Country:   "US",
		Elevation: 258,
		Latitude:  31.4099998474,
		Longitude: -86.0903015137,
		Timezone:  "America/Chicago",
	},
	"K14M": {
		ICAO:      "K14M",
		Name:      "Hollandale Municipal Airport",
		City:      "Hollandale",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 114,
		Latitude:  33.182598114,
		Longitude: -90.8307037354,
		Timezone:  "America/Chicago",
	},
	"K14Y": {
		ICAO:      "K14Y",
		Name:      "Todd Field",
		City:      "Long Prairie",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1333,
		Latitude:  45.898601532,
		Longitude: -94.8739013672,
		Timezone:  "America/Chicago",
	},
	"K15": {
		ICAO:      "K15",
		Name:      "Grand Glaize-Osage Beach Airport",
		City:      "Osage Beach, Missouri",
		State:     "Missouri",
		Country:   "US",
		Elevation: 876,
		Latitude:  38.1104444,
		Longitude: -92.6805556,
		Timezone:  "America/Chicago",
	},
	"K15F": {
		ICAO:      "K15F",
		Name:      "Haskell Municipal Airport",
		City:      "Haskell",
		State:     "Texas",
		Country:   "US",
		Elevation: 1625,
		Latitude:  33.1915016174,
		Longitude: -99.7179031372,
		Timezone:  "America/Chicago",
	},
	"K15J": {
		ICAO:      "K15J",
		Name:      "Cook County Airport",
		City:      "Adel",
		State:     "Georgia",
		Country:   "US",
		Elevation: 236,
		Latitude:  31.1378002167,
		Longitude: -83.4533004761,
		Timezone:  "America/New_York",
	},
	"K15M": {
		ICAO:      "K15M",
		Name:      "Iuka Airport",
		City:      "Iuka",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 630,
		Latitude:  34.7723007202,
		Longitude: -88.1659011841,
		Timezone:  "America/Chicago",
	},
	"K16D": {
		ICAO:      "K16D",
		Name:      "Perham Municipal Airport",
		City:      "Perham",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1371,
		Latitude:  46.6040992737,
		Longitude: -95.6044998169,
		Timezone:  "America/Chicago",
	},
	"K16G": {
		ICAO:      "K16G",
		Name:      "Seneca County Airport",
		City:      "Tiffin",
		State:     "Ohio",
		Country:   "US",
		Elevation: 786,
		Latitude:  41.0941009521,
		Longitude: -83.2125015259,
		Timezone:  "America/New_York",
	},
	"K16J": {
		ICAO:      "K16J",
		Name:      "Dawson Municipal Airport",
		City:      "Dawson",
		State:     "Georgia",
		Country:   "US",
		Elevation: 333,
		Latitude:  31.7432994843,
		Longitude: -84.4192962646,
		Timezone:  "America/New_York",
	},
	"K17G": {
		ICAO:      "K17G",
		Name:      "Port-Bucyrus-Crawford County Airport",
		City:      "Bucyrus",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1003,
		Latitude:  40.7816009521,
		Longitude: -82.9748001099,
		Timezone:  "America/New_York",
	},
	"K17J": {
		ICAO:      "K17J",
		Name:      "Donalsonville Municipal Airport",
		City:      "Donalsonville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 148,
		Latitude:  31.00690079,
		Longitude: -84.87760162,
		Timezone:  "America/New_York",
	},
	"K17K": {
		ICAO:      "K17K",
		Name:      "Boise City Airport",
		City:      "Boise City",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 4178,
		Latitude:  36.7742996216,
		Longitude: -102.5100021362,
		Timezone:  "America/Chicago",
	},
	"K17M": {
		ICAO:      "K17M",
		Name:      "Magee Municipal Airport",
		City:      "Magee",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 555,
		Latitude:  31.8626995087,
		Longitude: -89.8005981445,
		Timezone:  "America/Chicago",
	},
	"K17N": {
		ICAO:      "K17N",
		Name:      "Cross Keys Airport",
		City:      "Cross Keys",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 162,
		Latitude:  39.7055015564,
		Longitude: -75.0329971313,
		Timezone:  "America/New_York",
	},
	"K17V": {
		ICAO:      "K17V",
		Name:      "Haxtun Municipal Airport",
		City:      "Haxtun",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4035,
		Latitude:  40.625,
		Longitude: -102.6029968262,
		Timezone:  "America/Denver",
	},
	"K18A": {
		ICAO:      "K18A",
		Name:      "Franklin County Airport",
		City:      "Canon",
		State:     "Georgia",
		Country:   "US",
		Elevation: 890,
		Latitude:  34.34009933,
		Longitude: -83.13349915,
		Timezone:  "America/New_York",
	},
	"K18I": {
		ICAO:      "K18I",
		Name:      "Mc Creary County Airport",
		City:      "Pine Knot",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 1370,
		Latitude:  36.6958999634,
		Longitude: -84.3916015625,
		Timezone:  "America/New_York",
	},
	"K19M": {
		ICAO:      "K19M",
		Name:      "C. A. Moore Airport",
		City:      "Lexington",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 340,
		Latitude:  33.1254997253,
		Longitude: -90.0255966187,
		Timezone:  "America/Chicago",
	},
	"K19N": {
		ICAO:      "K19N",
		Name:      "Camden County Airport",
		City:      "Berlin",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 149,
		Latitude:  39.77840042,
		Longitude: -74.94779968,
		Timezone:  "America/New_York",
	},
	"K19S": {
		ICAO:      "K19S",
		Name:      "Sublette Flying Club Airport",
		City:      "Sublette",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2908,
		Latitude:  37.49140167,
		Longitude: -100.8300018,
		Timezone:  "America/Chicago",
	},
	"K1A0": {
		ICAO:      "K1A0",
		Name:      "Dallas Bay Sky Park Airport",
		City:      "Chattanooga",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 690,
		Latitude:  35.1875991821,
		Longitude: -85.177696228,
		Timezone:  "America/New_York",
	},
	"K1A3": {
		ICAO:      "K1A3",
		Name:      "Martin Campbell Field",
		City:      "Copperhill",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1790,
		Latitude:  35.01620102,
		Longitude: -84.34629822,
		Timezone:  "America/New_York",
	},
	"K1A4": {
		ICAO:      "K1A4",
		Name:      "Logan Field",
		City:      "Samson",
		State:     "Alabama",
		Country:   "US",
		Elevation: 200,
		Latitude:  31.1028003693,
		Longitude: -86.0622024536,
		Timezone:  "America/Chicago",
	},
	"K1A5": {
		ICAO:      "K1A5",
		Name:      "Macon County Airport",
		City:      "Franklin",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 2020,
		Latitude:  35.2225990295,
		Longitude: -83.4189987183,
		Timezone:  "America/New_York",
	},
	"K1A6": {
		ICAO:      "K1A6",
		Name:      "Middlesboro-Bell County Airport",
		City:      "Middlesboro",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 1154,
		Latitude:  36.6105995178,
		Longitude: -83.7373962402,
		Timezone:  "America/New_York",
	},
	"K1A7": {
		ICAO:      "K1A7",
		Name:      "Jackson County Airport",
		City:      "Gainesboro",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 515,
		Latitude:  36.4002990723,
		Longitude: -85.6417007446,
		Timezone:  "America/Chicago",
	},
	"K1A9": {
		ICAO:      "K1A9",
		Name:      "Prattville - Grouby Field",
		City:      "Prattville",
		State:     "Alabama",
		Country:   "US",
		Elevation: 225,
		Latitude:  32.4387016296,
		Longitude: -86.5127029419,
		Timezone:  "America/Chicago",
	},
	"K1B0": {
		ICAO:      "K1B0",
		Name:      "Dexter Regional Airport",
		City:      "Dexter",
		State:     "Maine",
		Country:   "US",
		Elevation: 533,
		Latitude:  45.0041007996,
		Longitude: -69.2369995117,
		Timezone:  "America/New_York",
	},
	"K1B6": {
		ICAO:      "K1B6",
		Name:      "Hopedale Industrial Park Airport",
		City:      "Hopedale",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 269,
		Latitude:  42.1064987183,
		Longitude: -71.5101013184,
		Timezone:  "America/New_York",
	},
	"K1B9": {
		ICAO:      "K1B9",
		Name:      "Mansfield Municipal Airport",
		City:      "Mansfield",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 122,
		Latitude:  42.0000991821,
		Longitude: -71.1968002319,
		Timezone:  "America/New_York",
	},
	"K1C1": {
		ICAO:      "K1C1",
		Name:      "Paxton Airport",
		City:      "Paxton",
		State:     "Illinois",
		Country:   "US",
		Elevation: 779,
		Latitude:  40.4490013123,
		Longitude: -88.1277008057,
		Timezone:  "America/Chicago",
	},
	"K1C5": {
		ICAO:      "K1C5",
		Name:      "Bolingbrook's Clow International Airport",
		City:      "Bolingbrook",
		State:     "Illinois",
		Country:   "US",
		Elevation: 670,
		Latitude:  41.6959991455,
		Longitude: -88.1292037964,
		Timezone:  "America/Chicago",
	},
	"K1D1": {
		ICAO:      "K1D1",
		Name:      "Milbank Municipal Airport",
		City:      "Milbank",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1118,
		Latitude:  45.23049927,
		Longitude: -96.56600189,
		Timezone:  "America/Chicago",
	},
	"K1D3": {
		ICAO:      "K1D3",
		Name:      "Platte Municipal Airport",
		City:      "Platte",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1618,
		Latitude:  43.403301239,
		Longitude: -98.829498291,
		Timezone:  "America/Chicago",
	},
	"K1D7": {
		ICAO:      "K1D7",
		Name:      "The Sigurd Anderson Airport",
		City:      "Webster",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1854,
		Latitude:  45.2925987244,
		Longitude: -97.5138015747,
		Timezone:  "America/Chicago",
	},
	"K1D8": {
		ICAO:      "K1D8",
		Name:      "Redfield Municipal Airport",
		City:      "Redfield",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1307,
		Latitude:  44.8624992371,
		Longitude: -98.5295028687,
		Timezone:  "America/Chicago",
	},
	"K1F0": {
		ICAO:      "K1F0",
		Name:      "Ardmore Downtown Executive Airport",
		City:      "Ardmore",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 845,
		Latitude:  34.1469722,
		Longitude: -97.1226667,
		Timezone:  "America/Chicago",
	},
	"K1F4": {
		ICAO:      "K1F4",
		Name:      "Madill Municipal Airport",
		City:      "Madill",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 890,
		Latitude:  34.1403999329,
		Longitude: -96.81199646,
		Timezone:  "America/Chicago",
	},
	"K1F5": {
		ICAO:      "K1F5",
		Name:      "Hoxie-Sheridan County Airport",
		City:      "Hoxie",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2733,
		Latitude:  39.3648986816,
		Longitude: -100.439002991,
		Timezone:  "America/Chicago",
	},
	"K1G0": {
		ICAO:      "K1G0",
		Name:      "Wood County Airport",
		City:      "Bowling Green",
		State:     "Ohio",
		Country:   "US",
		Elevation: 673,
		Latitude:  41.3909988403,
		Longitude: -83.6300964355,
		Timezone:  "America/New_York",
	},
	"K1G1": {
		ICAO:      "K1G1",
		Name:      "Elyria Airport",
		City:      "Elyria",
		State:     "Ohio",
		Country:   "US",
		Elevation: 758,
		Latitude:  41.3315010071,
		Longitude: -82.1003036499,
		Timezone:  "America/New_York",
	},
	"K1G3": {
		ICAO:      "K1G3",
		Name:      "Kent State University Airport",
		City:      "Kent",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1134,
		Latitude:  41.15140152,
		Longitude: -81.4151001,
		Timezone:  "America/New_York",
	},
	"K1G4": {
		ICAO:      "K1G4",
		IATA:      "GCW",
		Name:      "Grand Canyon West Airport",
		City:      "Peach Springs",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4825,
		Latitude:  35.990398407,
		Longitude: -113.816001892,
		Timezone:  "America/Phoenix",
	},
	"K1G5": {
		ICAO:      "K1G5",
		Name:      "Medina Municipal Airport",
		City:      "Medina",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1190,
		Latitude:  41.131401062,
		Longitude: -81.7649002075,
		Timezone:  "America/New_York",
	},
	"K1H0": {
		ICAO:      "K1H0",
		Name:      "Creve Coeur Airport",
		City:      "St Louis",
		State:     "Missouri",
		Country:   "US",
		Elevation: 462,
		Latitude:  38.7273333,
		Longitude: -90.5084167,
		Timezone:  "America/Chicago",
	},
	"K1H2": {
		ICAO:      "K1H2",
		Name:      "Effingham County Memorial Airport",
		City:      "Effingham",
		State:     "Illinois",
		Country:   "US",
		Elevation: 587,
		Latitude:  39.070400238,
		Longitude: -88.5335006714,
		Timezone:  "America/Chicago",
	},
	"K1H3": {
		ICAO:      "K1H3",
		Name:      "Linn State Technical College Airport",
		City:      "Linn",
		State:     "Missouri",
		Country:   "US",
		Elevation: 952,
		Latitude:  38.4715003967,
		Longitude: -91.8152999878,
		Timezone:  "America/Chicago",
	},
	"K1H5": {
		ICAO:      "K1H5",
		Name:      "Willow Springs Memorial Airport",
		City:      "Willow Springs",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1247,
		Latitude:  36.9896011353,
		Longitude: -91.9541015625,
		Timezone:  "America/Chicago",
	},
	"K1H8": {
		ICAO:      "K1H8",
		Name:      "Casey Municipal Airport",
		City:      "Casey",
		State:     "Illinois",
		Country:   "US",
		Elevation: 654,
		Latitude:  39.3025016785,
		Longitude: -88.0040969849,
		Timezone:  "America/Chicago",
	},
	"K1I5": {
		ICAO:      "K1I5",
		Name:      "Freehold Airport",
		City:      "Freehold",
		State:     "New-York",
		Country:   "US",
		Elevation: 440,
		Latitude:  42.3642997742,
		Longitude: -74.0660018921,
		Timezone:  "America/New_York",
	},
	"K1I7": {
		ICAO:      "K1I7",
		Name:      "Clinton Airport",
		City:      "Clinton",
		State:     "Indiana",
		Country:   "US",
		Elevation: 526,
		Latitude:  39.7125015259,
		Longitude: -87.4013977051,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"K1J0": {
		ICAO:      "K1J0",
		Name:      "Tri-County Airport",
		City:      "Bonifay",
		State:     "Florida",
		Country:   "US",
		Elevation: 85,
		Latitude:  30.8458003998,
		Longitude: -85.6014022827,
		Timezone:  "America/Chicago",
	},
	"K1J6": {
		ICAO:      "K1J6",
		Name:      "Bob Lee Flight Strip",
		City:      "Deland",
		State:     "Florida",
		Country:   "US",
		Elevation: 94,
		Latitude:  29.1044006348,
		Longitude: -81.3136978149,
		Timezone:  "America/New_York",
	},
	"K1K1": {
		ICAO:      "K1K1",
		Name:      "Lloyd Stearman Field Airport",
		City:      "Benton",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1364,
		Latitude:  37.7744444,
		Longitude: -97.1132222,
		Timezone:  "America/Chicago",
	},
	"K1K2": {
		ICAO:      "K1K2",
		Name:      "Lindsay Municipal Airport",
		City:      "Lindsay",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 968,
		Latitude:  34.8493003845,
		Longitude: -97.5852966309,
		Timezone:  "America/Chicago",
	},
	"K1K4": {
		ICAO:      "K1K4",
		Name:      "David Jay Perry Airport",
		City:      "Goldsby",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1168,
		Latitude:  35.1551017761,
		Longitude: -97.4703979492,
		Timezone:  "America/Chicago",
	},
	"K1K7": {
		ICAO:      "K1K7",
		Name:      "Fredonia Airport",
		City:      "Fredonia",
		State:     "Kansas",
		Country:   "US",
		Elevation: 880,
		Latitude:  37.5792007446,
		Longitude: -95.8377990723,
		Timezone:  "America/Chicago",
	},
	"K1K9": {
		ICAO:      "K1K9",
		Name:      "Satanta Municipal Airport",
		City:      "Satanta",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2976,
		Latitude:  37.456401825,
		Longitude: -100.9840011597,
		Timezone:  "America/Chicago",
	},
	"K1L0": {
		ICAO:      "K1L0",
		Name:      "St John The Baptist Parish Airport",
		City:      "Reserve",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 7,
		Latitude:  30.0869998932,
		Longitude: -90.5826034546,
		Timezone:  "America/Chicago",
	},
	"K1L3": {
		ICAO:      "K1L3",
		Name:      "Searchlight Airport",
		City:      "Searchlight",
		State:     "Nevada",
		Country:   "US",
		Elevation: 3410,
		Latitude:  35.4444007874,
		Longitude: -114.908996582,
		Timezone:  "America/Los_Angeles",
	},
	"K1L7": {
		ICAO:      "K1L7",
		Name:      "Escalante Municipal Airport",
		City:      "Escalante",
		State:     "Utah",
		Country:   "US",
		Elevation: 5733,
		Latitude:  37.745300293,
		Longitude: -111.5699996948,
		Timezone:  "America/Denver",
	},
	"K1L8": {
		ICAO:      "K1L8",
		Name:      "General Dick Stout Field",
		City:      "Hurricane",
		State:     "Utah",
		Country:   "US",
		Elevation: 3347,
		Latitude:  37.1389007568,
		Longitude: -113.3059997559,
		Timezone:  "America/Denver",
	},
	"K1L9": {
		ICAO:      "K1L9",
		Name:      "Parowan Airport",
		City:      "Parowan",
		State:     "Utah",
		Country:   "US",
		Elevation: 5930,
		Latitude:  37.8596992493,
		Longitude: -112.8160018921,
		Timezone:  "America/Denver",
	},
	"K1M2": {
		ICAO:      "K1M2",
		Name:      "Belzoni Municipal Airport",
		City:      "Belzoni",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 110,
		Latitude:  33.145198822,
		Longitude: -90.515296936,
		Timezone:  "America/Chicago",
	},
	"K1M4": {
		ICAO:      "K1M4",
		Name:      "Posey Field",
		City:      "Haleyville",
		State:     "Alabama",
		Country:   "US",
		Elevation: 930,
		Latitude:  34.2803001404,
		Longitude: -87.600402832,
		Timezone:  "America/Chicago",
	},
	"K1M5": {
		ICAO:      "K1M5",
		Name:      "Portland Municipal Airport",
		City:      "Portland",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 818,
		Latitude:  36.5928001404,
		Longitude: -86.4766998291,
		Timezone:  "America/Chicago",
	},
	"K1M9": {
		ICAO:      "K1M9",
		Name:      "Lake Barkley State Park Airport",
		City:      "Cadiz",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 570,
		Latitude:  36.8177986145,
		Longitude: -87.9075012207,
		Timezone:  "America/Chicago",
	},
	"K1MO": {
		ICAO:      "K1MO",
		Name:      "Mountain Grove Memorial Airport",
		City:      "Mountain Grove",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1476,
		Latitude:  37.1207008362,
		Longitude: -92.3112030029,
		Timezone:  "America/Chicago",
	},
	"K1N1": {
		ICAO:      "K1N1",
		Name:      "Sandia Airpark Estates East Airport",
		City:      "Edgewood",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 6550,
		Latitude:  35.0945014954,
		Longitude: -106.1669998169,
		Timezone:  "America/Denver",
	},
	"K1O1": {
		ICAO:      "K1O1",
		Name:      "Grandfield Municipal Airport",
		City:      "Grandfield",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1128,
		Latitude:  34.2375984192,
		Longitude: -98.7419967651,
		Timezone:  "America/Chicago",
	},
	"K1O2": {
		ICAO:      "K1O2",
		Name:      "Lampson Field",
		City:      "Lakeport",
		State:     "California",
		Country:   "US",
		Elevation: 1379,
		Latitude:  38.9906005859,
		Longitude: -122.901000977,
		Timezone:  "America/Los_Angeles",
	},
	"K1O3": {
		ICAO:      "K1O3",
		Name:      "Lodi Airport",
		City:      "Lodi",
		State:     "California",
		Country:   "US",
		Elevation: 60,
		Latitude:  38.20240021,
		Longitude: -121.2679977,
		Timezone:  "America/Los_Angeles",
	},
	"K1O4": {
		ICAO:      "K1O4",
		Name:      "Thomas Municipal Airport",
		City:      "Thomas",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1731,
		Latitude:  35.7344017029,
		Longitude: -98.7302017212,
		Timezone:  "America/Chicago",
	},
	"K1O5": {
		ICAO:      "K1O5",
		IATA:      "RKC",
		Name:      "Montague-Yreka Rohrer Field",
		City:      "Montague",
		State:     "California",
		Country:   "US",
		Elevation: 2527,
		Latitude:  41.7304000854,
		Longitude: -122.54599762,
		Timezone:  "America/Los_Angeles",
	},
	"K1O8": {
		ICAO:      "K1O8",
		Name:      "Tipton Municipal Airport",
		City:      "Tipton",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1248,
		Latitude:  34.4585990906,
		Longitude: -99.1713027954,
		Timezone:  "America/Chicago",
	},
	"K1Q2": {
		ICAO:      "K1Q2",
		Name:      "Spaulding Airport",
		City:      "Susanville",
		State:     "California",
		Country:   "US",
		Elevation: 5116,
		Latitude:  40.6502990723,
		Longitude: -120.7689971924,
		Timezone:  "America/Los_Angeles",
	},
	"K1Q4": {
		ICAO:      "K1Q4",
		Name:      "New Jerusalem Airport",
		City:      "Tracy",
		State:     "California",
		Country:   "US",
		Elevation: 62,
		Latitude:  37.6791000366,
		Longitude: -121.3000030518,
		Timezone:  "America/Los_Angeles",
	},
	"K1R1": {
		ICAO:      "K1R1",
		Name:      "Jena Airport",
		City:      "Jena",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 212,
		Latitude:  31.6569004059,
		Longitude: -92.1575012207,
		Timezone:  "America/Chicago",
	},
	"K1R4": {
		ICAO:      "K1R4",
		Name:      "Woodworth Airport",
		City:      "Woodworth",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 140,
		Latitude:  31.1263999939,
		Longitude: -92.5014038086,
		Timezone:  "America/Chicago",
	},
	"K1R7": {
		ICAO:      "K1R7",
		Name:      "Brookhaven-Lincoln County Airport",
		City:      "Brookhaven",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 489,
		Latitude:  31.6058006287,
		Longitude: -90.4093017578,
		Timezone:  "America/Chicago",
	},
	"K1R8": {
		ICAO:      "K1R8",
		Name:      "Bay Minette Municipal Airport",
		City:      "Bay Minette",
		State:     "Alabama",
		Country:   "US",
		Elevation: 248,
		Latitude:  30.8703994751,
		Longitude: -87.8192977905,
		Timezone:  "America/Chicago",
	},
	"K1S3": {
		ICAO:      "K1S3",
		Name:      "Tillitt Field",
		City:      "Forsyth",
		State:     "Montana",
		Country:   "US",
		Elevation: 2727,
		Latitude:  46.2710990906,
		Longitude: -106.6240005493,
		Timezone:  "America/Denver",
	},
	"K1S5": {
		ICAO:      "K1S5",
		Name:      "Sunnyside Municipal Airport",
		City:      "Sunnyside",
		State:     "Washington",
		Country:   "US",
		Elevation: 768,
		Latitude:  46.32709885,
		Longitude: -119.9700012,
		Timezone:  "America/Los_Angeles",
	},
	"K1S9": {
		ICAO:      "K1S9",
		Name:      "Sand Canyon Airport",
		City:      "Chewelah",
		State:     "Washington",
		Country:   "US",
		Elevation: 2084,
		Latitude:  48.3140983582,
		Longitude: -117.7429962158,
		Timezone:  "America/Los_Angeles",
	},
	"K1T7": {
		ICAO:      "K1T7",
		Name:      "Kestrel Airpark",
		City:      "San Antonio",
		State:     "Texas",
		Country:   "US",
		Elevation: 1250,
		Latitude:  29.8127002716,
		Longitude: -98.4253005981,
		Timezone:  "America/Chicago",
	},
	"K1U2": {
		ICAO:      "K1U2",
		Name:      "Mud Lake/West Jefferson County/ Airport",
		City:      "Mud Lake",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4787,
		Latitude:  43.8482017517,
		Longitude: -112.499000549,
		Timezone:  "America/Boise",
	},
	"K1U7": {
		ICAO:      "K1U7",
		Name:      "Bear Lake County Airport",
		City:      "Paris",
		State:     "Idaho",
		Country:   "US",
		Elevation: 5928,
		Latitude:  42.2472000122,
		Longitude: -111.3379974365,
		Timezone:  "America/Boise",
	},
	"K1V0": {
		ICAO:      "K1V0",
		Name:      "Navajo Lake Airport",
		City:      "Navajo Dam",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 6475,
		Latitude:  36.8083000183,
		Longitude: -107.6510009766,
		Timezone:  "America/Denver",
	},
	"K1V2": {
		ICAO:      "K1V2",
		Name:      "Grant County Airport",
		City:      "Hyannis",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3710,
		Latitude:  42.0093994141,
		Longitude: -101.7689971924,
		Timezone:  "America/Denver",
	},
	"K1V6": {
		ICAO:      "K1V6",
		Name:      "Fremont County Airport",
		City:      "Canon City",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5439,
		Latitude:  38.4280014038,
		Longitude: -105.1060028076,
		Timezone:  "America/Denver",
	},
	"K1V8": {
		ICAO:      "K1V8",
		Name:      "Leach Airport",
		City:      "Center",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7598,
		Latitude:  37.7846984863,
		Longitude: -106.0380020142,
		Timezone:  "America/Denver",
	},
	"K1X1": {
		ICAO:      "K1X1",
		Name:      "Higgins-Lipscomb County Airport",
		City:      "Higgins",
		State:     "Texas",
		Country:   "US",
		Elevation: 2566,
		Latitude:  36.1058998108,
		Longitude: -100.026000977,
		Timezone:  "America/Chicago",
	},
	"K20A": {
		ICAO:      "K20A",
		Name:      "Robbins Field",
		City:      "Oneonta",
		State:     "Alabama",
		Country:   "US",
		Elevation: 1140,
		Latitude:  33.9723014832,
		Longitude: -86.3794021606,
		Timezone:  "America/Chicago",
	},
	"K20M": {
		ICAO:      "K20M",
		Name:      "Macon Municipal Airport",
		City:      "Macon",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 238,
		Latitude:  33.1334991455,
		Longitude: -88.5355987549,
		Timezone:  "America/Chicago",
	},
	"K20N": {
		ICAO:      "K20N",
		Name:      "Kingston-Ulster Airport",
		City:      "Kingston",
		State:     "New-York",
		Country:   "US",
		Elevation: 149,
		Latitude:  41.9852981567,
		Longitude: -73.9641036987,
		Timezone:  "America/New_York",
	},
	"K20R": {
		ICAO:      "K20R",
		Name:      "Crystal City Municipal Airport",
		City:      "Crystal City",
		State:     "Texas",
		Country:   "US",
		Elevation: 608,
		Latitude:  28.6977996826,
		Longitude: -99.8178024292,
		Timezone:  "America/Chicago",
	},
	"K20U": {
		ICAO:      "K20U",
		Name:      "Beach Airport",
		City:      "Beach",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 2756,
		Latitude:  46.925201416,
		Longitude: -103.9820022583,
		Timezone:  "America/Denver",
	},
	"K20V": {
		ICAO:      "K20V",
		Name:      "Mc Elroy Airfield",
		City:      "Kremmling",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7411,
		Latitude:  40.0536994934,
		Longitude: -106.369003296,
		Timezone:  "America/Denver",
	},
	"K21D": {
		ICAO:      "K21D",
		Name:      "Lake Elmo Airport",
		City:      "St Paul",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 933,
		Latitude:  44.99750137,
		Longitude: -92.85569763,
		Timezone:  "America/Chicago",
	},
	"K21F": {
		ICAO:      "K21F",
		Name:      "Jacksboro Municipal Airport",
		City:      "Jacksboro",
		State:     "Texas",
		Country:   "US",
		Elevation: 1055,
		Latitude:  33.2286987305,
		Longitude: -98.146697998,
		Timezone:  "America/Chicago",
	},
	"K22F": {
		ICAO:      "K22F",
		Name:      "Kent County Airport",
		City:      "Jayton",
		State:     "Texas",
		Country:   "US",
		Elevation: 2006,
		Latitude:  33.2293014526,
		Longitude: -100.569000244,
		Timezone:  "America/Chicago",
	},
	"K22I": {
		ICAO:      "K22I",
		Name:      "Vinton County Airport",
		City:      "Mc Arthur",
		State:     "Ohio",
		Country:   "US",
		Elevation: 958,
		Latitude:  39.3280982971,
		Longitude: -82.4418029785,
		Timezone:  "America/New_York",
	},
	"K22M": {
		ICAO:      "K22M",
		Name:      "Pontotoc County Airport",
		City:      "Pontotoc",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 440,
		Latitude:  34.2759017944,
		Longitude: -89.0383987427,
		Timezone:  "America/Chicago",
	},
	"K22N": {
		ICAO:      "K22N",
		Name:      "Jake Arner Memorial Airport",
		City:      "Lehighton",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 534,
		Latitude:  40.8095016479,
		Longitude: -75.7614974976,
		Timezone:  "America/New_York",
	},
	"K22S": {
		ICAO:      "K22S",
		Name:      "Paisley Airport",
		City:      "Paisley",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4395,
		Latitude:  42.7178993225,
		Longitude: -120.56300354,
		Timezone:  "America/Los_Angeles",
	},
	"K22W": {
		ICAO:      "K22W",
		Name:      "Transylvania County Airport",
		City:      "Brevard",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 2110,
		Latitude:  35.2703018188,
		Longitude: -82.644203186,
		Timezone:  "America/New_York",
	},
	"K23R": {
		ICAO:      "K23R",
		Name:      "Devine Municipal Airport",
		City:      "Devine",
		State:     "Texas",
		Country:   "US",
		Elevation: 703,
		Latitude:  29.1383991241,
		Longitude: -98.9419021606,
		Timezone:  "America/Chicago",
	},
	"K24A": {
		ICAO:      "K24A",
		Name:      "Jackson County Airport",
		City:      "Sylva",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 2857,
		Latitude:  35.317401886,
		Longitude: -83.2098999023,
		Timezone:  "America/New_York",
	},
	"K24F": {
		ICAO:      "K24F",
		Name:      "Cypress River Airport",
		City:      "Jefferson",
		State:     "Texas",
		Country:   "US",
		Elevation: 221,
		Latitude:  32.7459983826,
		Longitude: -94.304397583,
		Timezone:  "America/Chicago",
	},
	"K24J": {
		ICAO:      "K24J",
		Name:      "Suwannee County Airport",
		City:      "Live Oak",
		State:     "Florida",
		Country:   "US",
		Elevation: 104,
		Latitude:  30.3001003265,
		Longitude: -83.0246963501,
		Timezone:  "America/New_York",
	},
	"K24N": {
		ICAO:      "K24N",
		Name:      "Jicarilla Apache Nation Airport",
		City:      "Dulce",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 7618,
		Latitude:  36.8284988403,
		Longitude: -106.8840026855,
		Timezone:  "America/Denver",
	},
	"K24R": {
		ICAO:      "K24R",
		Name:      "Dilley Airpark",
		City:      "Dilley",
		State:     "Texas",
		Country:   "US",
		Elevation: 542,
		Latitude:  28.6854991913,
		Longitude: -99.189201355,
		Timezone:  "America/Chicago",
	},
	"K25D": {
		ICAO:      "K25D",
		Name:      "Forest Lake Airport",
		City:      "Forest Lake",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 925,
		Latitude:  45.2476997375,
		Longitude: -92.9944000244,
		Timezone:  "America/Chicago",
	},
	"K25J": {
		ICAO:      "K25J",
		Name:      "Cuthbert-Randolph Airport",
		City:      "Cuthbert",
		State:     "Georgia",
		Country:   "US",
		Elevation: 457,
		Latitude:  31.7001991272,
		Longitude: -84.8248977661,
		Timezone:  "America/New_York",
	},
	"K25M": {
		ICAO:      "K25M",
		Name:      "Ripley Airport",
		City:      "Ripley",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 465,
		Latitude:  34.7223014832,
		Longitude: -89.0150985718,
		Timezone:  "America/Chicago",
	},
	"K26A": {
		ICAO:      "K26A",
		Name:      "Ashland/Lineville Airport",
		City:      "Ashland/Lineville",
		State:     "Alabama",
		Country:   "US",
		Elevation: 1065,
		Latitude:  33.2876014709,
		Longitude: -85.8041000366,
		Timezone:  "America/Chicago",
	},
	"K26R": {
		ICAO:      "K26R",
		Name:      "Jackson County Airport",
		City:      "Edna",
		State:     "Texas",
		Country:   "US",
		Elevation: 61,
		Latitude:  29.0009994507,
		Longitude: -96.5820007324,
		Timezone:  "America/Chicago",
	},
	"K26U": {
		ICAO:      "K26U",
		Name:      "Mc Dermitt State Airport",
		City:      "Mc Dermitt",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4478,
		Latitude:  42.0021018982,
		Longitude: -117.722999573,
		Timezone:  "America/Los_Angeles",
	},
	"K27A": {
		ICAO:      "K27A",
		Name:      "Elbert-County-Patz Field",
		City:      "Elberton",
		State:     "Georgia",
		Country:   "US",
		Elevation: 603,
		Latitude:  34.095199585,
		Longitude: -82.81590271,
		Timezone:  "America/New_York",
	},
	"K27K": {
		ICAO:      "K27K",
		Name:      "Georgetown Scott County - Marshall Field",
		City:      "Georgetown",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 947,
		Latitude:  38.2344017029,
		Longitude: -84.4347000122,
		Timezone:  "America/New_York",
	},
	"K27R": {
		ICAO:      "K27R",
		Name:      "Eldorado Airport",
		City:      "Eldorado",
		State:     "Texas",
		Country:   "US",
		Elevation: 2448,
		Latitude:  30.8621997833,
		Longitude: -100.611000061,
		Timezone:  "America/Chicago",
	},
	"K28J": {
		ICAO:      "K28J",
		Name:      "Palatka Municipal - Lt. Kay Larkin Field",
		City:      "Palatka",
		State:     "Florida",
		Country:   "US",
		Elevation: 48,
		Latitude:  29.65859985,
		Longitude: -81.68890381,
		Timezone:  "America/New_York",
	},
	"K29D": {
		ICAO:      "K29D",
		Name:      "Grove City Airport",
		City:      "Grove City",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1371,
		Latitude:  41.1459999084,
		Longitude: -80.1677017212,
		Timezone:  "America/New_York",
	},
	"K29G": {
		ICAO:      "K29G",
		Name:      "Portage County Airport",
		City:      "Ravenna",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1197,
		Latitude:  41.2102012634,
		Longitude: -81.2516021729,
		Timezone:  "America/New_York",
	},
	"K29S": {
		ICAO:      "K29S",
		Name:      "Gardiner Airport",
		City:      "Gardiner",
		State:     "Montana",
		Country:   "US",
		Elevation: 5286,
		Latitude:  45.0499000549,
		Longitude: -110.7470016479,
		Timezone:  "America/Denver",
	},
	"K2A0": {
		ICAO:      "K2A0",
		Name:      "Mark Anton Airport",
		City:      "Dayton",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 719,
		Latitude:  35.4861984253,
		Longitude: -84.931098938,
		Timezone:  "America/New_York",
	},
	"K2A1": {
		ICAO:      "K2A1",
		Name:      "Jamestown Municipal Airport",
		City:      "Jamestown",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1694,
		Latitude:  36.3497009277,
		Longitude: -84.9467010498,
		Timezone:  "America/Chicago",
	},
	"K2A2": {
		ICAO:      "K2A2",
		Name:      "Holley Mountain Airpark",
		City:      "Clinton",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1269,
		Latitude:  35.6506996155,
		Longitude: -92.4038009644,
		Timezone:  "America/Chicago",
	},
	"K2A5": {
		ICAO:      "K2A5",
		Name:      "Causey Airport",
		City:      "Liberty",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 723,
		Latitude:  35.9118003845,
		Longitude: -79.6175994873,
		Timezone:  "America/New_York",
	},
	"K2A6": {
		ICAO:      "K2A6",
		Name:      "Holly Grove Municipal Airport",
		City:      "Holly Grove",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 176,
		Latitude:  34.5824012756,
		Longitude: -91.1651992798,
		Timezone:  "America/Chicago",
	},
	"K2B3": {
		ICAO:      "K2B3",
		Name:      "Parlin Field",
		City:      "Newport",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 784,
		Latitude:  43.3880996704,
		Longitude: -72.1893005371,
		Timezone:  "America/New_York",
	},
	"K2B7": {
		ICAO:      "K2B7",
		Name:      "Pittsfield Municipal Airport",
		City:      "Pittsfield",
		State:     "Maine",
		Country:   "US",
		Elevation: 198,
		Latitude:  44.7685012817,
		Longitude: -69.3743972778,
		Timezone:  "America/New_York",
	},
	"K2C8": {
		ICAO:      "K2C8",
		Name:      "Cavalier Municipal Airport",
		City:      "Cavalier",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 892,
		Latitude:  48.7837982178,
		Longitude: -97.6318969727,
		Timezone:  "America/Chicago",
	},
	"K2CB": {
		ICAO:      "K2CB",
		Name:      "Camp Blanding Army Air Field/NG Airfield",
		City:      "Camp Blanding Mil Res(Starke)",
		State:     "Florida",
		Country:   "US",
		Elevation: 196,
		Latitude:  29.9524993896,
		Longitude: -81.9796981812,
		Timezone:  "America/New_York",
	},
	"K2D5": {
		ICAO:      "K2D5",
		Name:      "Oakes Municipal Airport",
		City:      "Oakes",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1335,
		Latitude:  46.1730003357,
		Longitude: -98.0799026489,
		Timezone:  "America/Chicago",
	},
	"K2E3": {
		ICAO:      "K2E3",
		Name:      "Cluck Ranch Airport",
		City:      "Gruver",
		State:     "Texas",
		Country:   "US",
		Elevation: 3423,
		Latitude:  36.1772003174,
		Longitude: -101.6940002441,
		Timezone:  "America/Chicago",
	},
	"K2E5": {
		ICAO:      "K2E5",
		Name:      "Dell City Municipal Airport",
		City:      "Dell City",
		State:     "Texas",
		Country:   "US",
		Elevation: 3701,
		Latitude:  31.947599411,
		Longitude: -105.1920013428,
		Timezone:  "America/Denver",
	},
	"K2E7": {
		ICAO:      "K2E7",
		Name:      "Mc Lean Gray County Airport",
		City:      "Mc Lean",
		State:     "Texas",
		Country:   "US",
		Elevation: 2835,
		Latitude:  35.2470016479,
		Longitude: -100.543998718,
		Timezone:  "America/Chicago",
	},
	"K2F0": {
		ICAO:      "K2F0",
		Name:      "Iraan Municipal Airport",
		City:      "Iraan",
		State:     "Texas",
		Country:   "US",
		Elevation: 2200,
		Latitude:  30.9057006836,
		Longitude: -101.891998291,
		Timezone:  "America/Chicago",
	},
	"K2F1": {
		ICAO:      "K2F1",
		Name:      "Shamrock Municipal Airport",
		City:      "Shamrock",
		State:     "Texas",
		Country:   "US",
		Elevation: 2369,
		Latitude:  35.2361984253,
		Longitude: -100.1849975586,
		Timezone:  "America/Chicago",
	},
	"K2F4": {
		ICAO:      "K2F4",
		Name:      "T Bar Airport",
		City:      "Tahoka",
		State:     "Texas",
		Country:   "US",
		Elevation: 3126,
		Latitude:  33.1769981384,
		Longitude: -101.8199996948,
		Timezone:  "America/Chicago",
	},
	"K2F5": {
		ICAO:      "K2F5",
		Name:      "Lamesa Municipal Airport",
		City:      "Lamesa",
		State:     "Texas",
		Country:   "US",
		Elevation: 2999,
		Latitude:  32.7563018799,
		Longitude: -101.9199981689,
		Timezone:  "America/Chicago",
	},
	"K2F7": {
		ICAO:      "K2F7",
		Name:      "Commerce Municipal Airport",
		City:      "Commerce",
		State:     "Texas",
		Country:   "US",
		Elevation: 515,
		Latitude:  33.2929000854,
		Longitude: -95.8964004517,
		Timezone:  "America/Chicago",
	},
	"K2G1": {
		ICAO:      "K2G1",
		Name:      "Concord Airpark",
		City:      "Painesville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 999,
		Latitude:  41.6669998169,
		Longitude: -81.1971969604,
		Timezone:  "America/New_York",
	},
	"K2G2": {
		ICAO:      "K2G2",
		Name:      "Jefferson County Airpark",
		City:      "Steubenville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1196,
		Latitude:  40.3594017029,
		Longitude: -80.7001037598,
		Timezone:  "America/New_York",
	},
	"K2G4": {
		ICAO:      "K2G4",
		Name:      "Garrett County Airport",
		City:      "Oakland",
		State:     "Maryland",
		Country:   "US",
		Elevation: 2933,
		Latitude:  39.5802993774,
		Longitude: -79.3394012451,
		Timezone:  "America/New_York",
	},
	"K2G9": {
		ICAO:      "K2G9",
		Name:      "Somerset County Airport",
		City:      "Somerset",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 2275,
		Latitude:  40.03910065,
		Longitude: -79.01460266,
		Timezone:  "America/New_York",
	},
	"K2GC": {
		ICAO:      "K2GC",
		Name:      "Grays Creek Airport",
		City:      "Fayetteville",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 160,
		Latitude:  34.893699646,
		Longitude: -78.84349823,
		Timezone:  "America/New_York",
	},
	"K2H0": {
		ICAO:      "K2H0",
		Name:      "Shelby County Airport",
		City:      "Shelbyville",
		State:     "Alabama",
		Country:   "US",
		Elevation: 618,
		Latitude:  39.4104003906,
		Longitude: -88.8453979492,
		Timezone:  "America/Chicago",
	},
	"K2H2": {
		ICAO:      "K2H2",
		Name:      "Jerry Sumners Sr Aurora Municipal Airport",
		City:      "Aurora",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1434,
		Latitude:  36.9622993469,
		Longitude: -93.6952972412,
		Timezone:  "America/Chicago",
	},
	"K2H9": {
		ICAO:      "K2H9",
		Name:      "Rolette Airport",
		City:      "Rolette",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1620,
		Latitude:  48.6652984619,
		Longitude: -99.8532028198,
		Timezone:  "America/Chicago",
	},
	"K2I0": {
		ICAO:      "K2I0",
		Name:      "Madisonville Municipal Airport",
		City:      "Madisonville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 439,
		Latitude:  37.3549995422,
		Longitude: -87.399597168,
		Timezone:  "America/Chicago",
	},
	"K2I3": {
		ICAO:      "K2I3",
		Name:      "Rough River State Park Airport",
		City:      "Falls-Of-Rough",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 577,
		Latitude:  37.6097984314,
		Longitude: -86.5068969727,
		Timezone:  "America/Chicago",
	},
	"K2IS": {
		ICAO:      "K2IS",
		Name:      "Airglades Airport",
		City:      "Clewiston",
		State:     "Florida",
		Country:   "US",
		Elevation: 20,
		Latitude:  26.7420005798,
		Longitude: -81.0497970581,
		Timezone:  "America/New_York",
	},
	"K2J2": {
		ICAO:      "K2J2",
		Name:      "Liberty County Airport",
		City:      "Hinesville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 98,
		Latitude:  31.7845993042,
		Longitude: -81.6411972046,
		Timezone:  "America/New_York",
	},
	"K2J3": {
		ICAO:      "K2J3",
		Name:      "Louisville Municipal Airport",
		City:      "Louisville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 328,
		Latitude:  32.9864997864,
		Longitude: -82.3856964111,
		Timezone:  "America/New_York",
	},
	"K2J5": {
		ICAO:      "K2J5",
		Name:      "Millen Airport",
		City:      "Millen",
		State:     "Georgia",
		Country:   "US",
		Elevation: 237,
		Latitude:  32.8936004639,
		Longitude: -81.9654006958,
		Timezone:  "America/New_York",
	},
	"K2K3": {
		ICAO:      "K2K3",
		Name:      "Stanton County Municipal Airport",
		City:      "Johnson",
		State:     "Kansas",
		Country:   "US",
		Elevation: 3324,
		Latitude:  37.582698822,
		Longitude: -101.733001709,
		Timezone:  "America/Chicago",
	},
	"K2K4": {
		ICAO:      "K2K4",
		Name:      "Scott Field",
		City:      "Mangum",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1643,
		Latitude:  34.8926010132,
		Longitude: -99.5281982422,
		Timezone:  "America/Chicago",
	},
	"K2K7": {
		ICAO:      "K2K7",
		Name:      "Neodesha Municipal Airport",
		City:      "Neodesha",
		State:     "Kansas",
		Country:   "US",
		Elevation: 841,
		Latitude:  37.4353981018,
		Longitude: -95.6461029053,
		Timezone:  "America/Chicago",
	},
	"K2K9": {
		ICAO:      "K2K9",
		Name:      "Haskell Airport",
		City:      "Haskell",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 588,
		Latitude:  35.832901001,
		Longitude: -95.6673965454,
		Timezone:  "America/Chicago",
	},
	"K2L0": {
		ICAO:      "K2L0",
		Name:      "Pineville Municipal Airport",
		City:      "Pineville",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 100,
		Latitude:  31.3460998535,
		Longitude: -92.4436035156,
		Timezone:  "America/Chicago",
	},
	"K2M0": {
		ICAO:      "K2M0",
		Name:      "Princeton Caldwell County Airport",
		City:      "Princeton",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 584,
		Latitude:  37.1151008606,
		Longitude: -87.8571014404,
		Timezone:  "America/Chicago",
	},
	"K2M2": {
		ICAO:      "K2M2",
		Name:      "Lawrenceburg Lawrence County Airport",
		City:      "Lawrenceburg",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 936,
		Latitude:  35.2342987061,
		Longitude: -87.2578964233,
		Timezone:  "America/Chicago",
	},
	"K2M4": {
		ICAO:      "K2M4",
		Name:      "G. V. Montgomery Airport",
		City:      "Forest",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 517,
		Latitude:  32.3535003662,
		Longitude: -89.4887008667,
		Timezone:  "America/Chicago",
	},
	"K2M8": {
		ICAO:      "K2M8",
		Name:      "Charles W. Baker Airport",
		City:      "Millington",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 247,
		Latitude:  35.2789993286,
		Longitude: -89.9315032959,
		Timezone:  "America/Chicago",
	},
	"K2MO": {
		ICAO:      "K2MO",
		Name:      "Mount Vernon Municipal Airport",
		City:      "Mount Vernon",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1244,
		Latitude:  37.0684013367,
		Longitude: -93.8849029541,
		Timezone:  "America/Chicago",
	},
	"K2O1": {
		ICAO:      "K2O1",
		IATA:      "GNF",
		Name:      "Gansner Field",
		City:      "Quincy",
		State:     "California",
		Country:   "US",
		Elevation: 3415,
		Latitude:  39.943901062,
		Longitude: -120.944999695,
		Timezone:  "America/Los_Angeles",
	},
	"K2O3": {
		ICAO:      "K2O3",
		Name:      "Angwin Parrett Field",
		City:      "Angwin",
		State:     "California",
		Country:   "US",
		Elevation: 1848,
		Latitude:  38.5784988403,
		Longitude: -122.4349975586,
		Timezone:  "America/Los_Angeles",
	},
	"K2O6": {
		ICAO:      "K2O6",
		Name:      "Chowchilla Airport",
		City:      "Chowchilla",
		State:     "California",
		Country:   "US",
		Elevation: 242,
		Latitude:  37.1124000549,
		Longitude: -120.2470016479,
		Timezone:  "America/Los_Angeles",
	},
	"K2O7": {
		ICAO:      "K2O7",
		Name:      "Independence Airport",
		City:      "Independence",
		State:     "California",
		Country:   "US",
		Elevation: 3908,
		Latitude:  36.81380081,
		Longitude: -118.2050018,
		Timezone:  "America/Los_Angeles",
	},
	"K2O8": {
		ICAO:      "K2O8",
		Name:      "Hinton Municipal Airport",
		City:      "Hinton",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1587,
		Latitude:  35.50590134,
		Longitude: -98.3423996,
		Timezone:  "America/Chicago",
	},
	"K2R0": {
		ICAO:      "K2R0",
		Name:      "Waynesboro Municipal Airport",
		City:      "Waynesboro",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 164,
		Latitude:  31.6459999084,
		Longitude: -88.634803772,
		Timezone:  "America/Chicago",
	},
	"K2R1": {
		ICAO:      "K2R1",
		Name:      "Le Maire Memorial Airport",
		City:      "Jeanerette",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 14,
		Latitude:  29.89909935,
		Longitude: -91.6660003662,
		Timezone:  "America/Chicago",
	},
	"K2R2": {
		ICAO:      "K2R2",
		Name:      "Hendricks County Gordon Graham Field",
		City:      "Indianapolis",
		State:     "Indiana",
		Country:   "US",
		Elevation: 897,
		Latitude:  39.74810028,
		Longitude: -86.47380066,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"K2R4": {
		ICAO:      "K2R4",
		Name:      "Peter Prince Field",
		City:      "Milton",
		State:     "Florida",
		Country:   "US",
		Elevation: 82,
		Latitude:  30.6375999451,
		Longitude: -86.9936981201,
		Timezone:  "America/Chicago",
	},
	"K2R5": {
		ICAO:      "K2R5",
		Name:      "St Elmo Airport",
		City:      "St Elmo",
		State:     "Alabama",
		Country:   "US",
		Elevation: 132,
		Latitude:  30.50189972,
		Longitude: -88.27510071,
		Timezone:  "America/Chicago",
	},
	"K2R6": {
		ICAO:      "K2R6",
		Name:      "Bunkie Municipal Airport",
		City:      "Bunkie",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 62,
		Latitude:  30.9566993713,
		Longitude: -92.2341003418,
		Timezone:  "America/Chicago",
	},
	"K2R7": {
		ICAO:      "K2R7",
		Name:      "Franklinton Airport",
		City:      "Franklinton",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 175,
		Latitude:  30.8194007874,
		Longitude: -90.1125030518,
		Timezone:  "America/Chicago",
	},
	"K2R9": {
		ICAO:      "K2R9",
		Name:      "Karnes County Airport",
		City:      "Kenedy",
		State:     "Texas",
		Country:   "US",
		Elevation: 289,
		Latitude:  28.8250007629,
		Longitude: -97.8656005859,
		Timezone:  "America/Chicago",
	},
	"K2RR": {
		ICAO:      "K2RR",
		Name:      "River Ranch Resort Airport",
		City:      "River Ranch",
		State:     "Florida",
		Country:   "US",
		Elevation: 55,
		Latitude:  27.7821998596,
		Longitude: -81.2052993774,
		Timezone:  "America/New_York",
	},
	"K2S4": {
		ICAO:      "K2S4",
		Name:      "Warden Airport",
		City:      "Warden",
		State:     "Washington",
		Country:   "US",
		Elevation: 1276,
		Latitude:  46.9654006958,
		Longitude: -119.066001892,
		Timezone:  "America/Los_Angeles",
	},
	"K2S7": {
		ICAO:      "K2S7",
		Name:      "Chiloquin State Airport",
		City:      "Chiloquin",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4217,
		Latitude:  42.579440493,
		Longitude: -121.879062653,
		Timezone:  "America/Los_Angeles",
	},
	"K2S8": {
		ICAO:      "K2S8",
		Name:      "Wilbur Airport",
		City:      "Wilbur",
		State:     "Washington",
		Country:   "US",
		Elevation: 2182,
		Latitude:  47.7541999817,
		Longitude: -118.7429962158,
		Timezone:  "America/Los_Angeles",
	},
	"K2S9": {
		ICAO:      "K2S9",
		Name:      "Willapa Harbor Airport",
		City:      "South Bend/Raymond/",
		State:     "Washington",
		Country:   "US",
		Elevation: 13,
		Latitude:  46.6976013184,
		Longitude: -123.8229980469,
		Timezone:  "America/Los_Angeles",
	},
	"K2T1": {
		ICAO:      "K2T1",
		Name:      "Muleshoe Municipal Airport",
		City:      "Muleshoe",
		State:     "Texas",
		Country:   "US",
		Elevation: 3779,
		Latitude:  34.1851005554,
		Longitude: -102.6409988403,
		Timezone:  "America/Chicago",
	},
	"K2V5": {
		ICAO:      "K2V5",
		Name:      "Wray Municipal Airport",
		City:      "Wray",
		State:     "Colorado",
		Country:   "US",
		Elevation: 3667,
		Latitude:  40.10029984,
		Longitude: -102.2409973,
		Timezone:  "America/Denver",
	},
	"K2V6": {
		ICAO:      "K2V6",
		Name:      "Yuma Municipal Airport",
		City:      "Yuma",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4138,
		Latitude:  40.1057105,
		Longitude: -102.7144377,
		Timezone:  "America/Denver",
	},
	"K2W5": {
		ICAO:      "K2W5",
		Name:      "Maryland Airport",
		City:      "Indian Head",
		State:     "Maryland",
		Country:   "US",
		Elevation: 170,
		Latitude:  38.6004981995,
		Longitude: -77.0729980469,
		Timezone:  "America/New_York",
	},
	"K2W6": {
		ICAO:      "K2W6",
		Name:      "St. Mary's County Regional Airport",
		City:      "Leonardtown",
		State:     "Maryland",
		Country:   "US",
		Elevation: 142,
		Latitude:  38.3153991699,
		Longitude: -76.5501022339,
		Timezone:  "America/New_York",
	},
	"K2Y4": {
		ICAO:      "K2Y4",
		Name:      "Rockwell City Municipal Airport",
		City:      "Rockwell City",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1217,
		Latitude:  42.3875007629,
		Longitude: -94.6179962158,
		Timezone:  "America/Chicago",
	},
	"K30K": {
		ICAO:      "K30K",
		Name:      "Ingalls Municipal Airport",
		City:      "Ingalls",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2814,
		Latitude:  37.9070014954,
		Longitude: -100.53099823,
		Timezone:  "America/Chicago",
	},
	"K31E": {
		ICAO:      "K31E",
		Name:      "Eagles Nest Airport",
		City:      "West Creek",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 35,
		Latitude:  39.6654014587,
		Longitude: -74.3078994751,
		Timezone:  "America/New_York",
	},
	"K32A": {
		ICAO:      "K32A",
		Name:      "Danville Municipal Airport",
		City:      "Danville",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 387,
		Latitude:  35.0870018005,
		Longitude: -93.4274978638,
		Timezone:  "America/Chicago",
	},
	"K32S": {
		ICAO:      "K32S",
		Name:      "Stevensville Airport",
		City:      "Stevensville",
		State:     "Montana",
		Country:   "US",
		Elevation: 3610,
		Latitude:  46.525100708,
		Longitude: -114.0530014038,
		Timezone:  "America/Denver",
	},
	"K33J": {
		ICAO:      "K33J",
		Name:      "Geneva Municipal Airport",
		City:      "Geneva",
		State:     "Alabama",
		Country:   "US",
		Elevation: 101,
		Latitude:  31.0552997589,
		Longitude: -85.8803024292,
		Timezone:  "America/Chicago",
	},
	"K33K": {
		ICAO:      "K33K",
		Name:      "Kinsley Municipal Airport",
		City:      "Kinsley",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2171,
		Latitude:  37.9090003967,
		Longitude: -99.4030990601,
		Timezone:  "America/Chicago",
	},
	"K33M": {
		ICAO:      "K33M",
		Name:      "Water Valley Municipal Airport",
		City:      "Water Valley",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 270,
		Latitude:  34.1668014526,
		Longitude: -89.6862030029,
		Timezone:  "America/Chicago",
	},
	"K33R": {
		ICAO:      "K33R",
		Name:      "Groveton Trinity County Airport",
		City:      "Groveton",
		State:     "Texas",
		Country:   "US",
		Elevation: 340,
		Latitude:  31.0848999023,
		Longitude: -95.164100647,
		Timezone:  "America/Chicago",
	},
	"K33S": {
		ICAO:      "K33S",
		Name:      "Pru Field",
		City:      "Ritzville",
		State:     "Washington",
		Country:   "US",
		Elevation: 1801,
		Latitude:  47.1231994629,
		Longitude: -118.3899993896,
		Timezone:  "America/Los_Angeles",
	},
	"K33U": {
		ICAO:      "K33U",
		Name:      "Dutch John Airport",
		City:      "Dutch John",
		State:     "Utah",
		Country:   "US",
		Elevation: 6561,
		Latitude:  40.9177017212,
		Longitude: -109.3909988403,
		Timezone:  "America/Denver",
	},
	"K33V": {
		ICAO:      "K33V",
		Name:      "Walden Jackson County Airport",
		City:      "Walden",
		State:     "Colorado",
		Country:   "US",
		Elevation: 8153,
		Latitude:  40.7504005432,
		Longitude: -106.2710037231,
		Timezone:  "America/Denver",
	},
	"K34M": {
		ICAO:      "K34M",
		Name:      "Campbell Municipal Airport",
		City:      "Campbell",
		State:     "Missouri",
		Country:   "US",
		Elevation: 284,
		Latitude:  36.4861984253,
		Longitude: -90.0083999634,
		Timezone:  "America/Chicago",
	},
	"K34R": {
		ICAO:      "K34R",
		Name:      "Hallettsville Municipal Airport",
		City:      "Hallettsville",
		State:     "Texas",
		Country:   "US",
		Elevation: 278,
		Latitude:  29.3899993896,
		Longitude: -96.9561004639,
		Timezone:  "America/Chicago",
	},
	"K35A": {
		ICAO:      "K35A",
		Name:      "Union County Troy Shelton Field",
		City:      "Union",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 610,
		Latitude:  34.6870002747,
		Longitude: -81.6411972046,
		Timezone:  "America/New_York",
	},
	"K35D": {
		ICAO:      "K35D",
		Name:      "Padgham Field",
		City:      "Allegan",
		State:     "Michigan",
		Country:   "US",
		Elevation: 706,
		Latitude:  42.53099823,
		Longitude: -85.82510376,
		Timezone:  "America/Detroit",
	},
	"K35S": {
		ICAO:      "K35S",
		Name:      "Wasco State Airport",
		City:      "Wasco",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1503,
		Latitude:  45.5894012451,
		Longitude: -120.6740036011,
		Timezone:  "America/Los_Angeles",
	},
	"K36K": {
		ICAO:      "K36K",
		Name:      "Lakin Airport",
		City:      "Lakin",
		State:     "Kansas",
		Country:   "US",
		Elevation: 3077,
		Latitude:  37.9695014954,
		Longitude: -101.2549972534,
		Timezone:  "America/Chicago",
	},
	"K36S": {
		ICAO:      "K36S",
		Name:      "Happy Camp Airport",
		City:      "Happy Camp",
		State:     "California",
		Country:   "US",
		Elevation: 1209,
		Latitude:  41.7906990051,
		Longitude: -123.388999939,
		Timezone:  "America/Los_Angeles",
	},
	"K36U": {
		ICAO:      "K36U",
		Name:      "Heber City Municipal Russ Mcdonald Field",
		City:      "Heber",
		State:     "Utah",
		Country:   "US",
		Elevation: 5637,
		Latitude:  40.4818000793,
		Longitude: -111.4290008545,
		Timezone:  "America/Denver",
	},
	"K37F": {
		ICAO:      "K37F",
		Name:      "Munday Municipal Airport",
		City:      "Munday",
		State:     "Texas",
		Country:   "US",
		Elevation: 1473,
		Latitude:  33.4847984314,
		Longitude: -99.5861968994,
		Timezone:  "America/Chicago",
	},
	"K37K": {
		ICAO:      "K37K",
		Name:      "Falconhead Airport",
		City:      "Burneyville",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 690,
		Latitude:  33.9259986877,
		Longitude: -97.2855987549,
		Timezone:  "America/Chicago",
	},
	"K37T": {
		ICAO:      "K37T",
		Name:      "Calico Rock Izard County Airport",
		City:      "Calico Rock",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 733,
		Latitude:  36.1645011902,
		Longitude: -92.1445007324,
		Timezone:  "America/Chicago",
	},
	"K38D": {
		ICAO:      "K38D",
		Name:      "Salem Airpark Inc Airport",
		City:      "Salem",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1162,
		Latitude:  40.9481010437,
		Longitude: -80.8620986938,
		Timezone:  "America/New_York",
	},
	"K38J": {
		ICAO:      "K38J",
		Name:      "Hemingway Stuckey Airport",
		City:      "Hemingway",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 54,
		Latitude:  33.7285995483,
		Longitude: -79.5159988403,
		Timezone:  "America/New_York",
	},
	"K38S": {
		ICAO:      "K38S",
		Name:      "Deer Lodge City County Airport",
		City:      "Deer Lodge",
		State:     "Montana",
		Country:   "US",
		Elevation: 4693,
		Latitude:  46.38819885,
		Longitude: -112.7659988,
		Timezone:  "America/Denver",
	},
	"K38U": {
		ICAO:      "K38U",
		Name:      "Wayne Wonderland Airport",
		City:      "Loa",
		State:     "Utah",
		Country:   "US",
		Elevation: 7023,
		Latitude:  38.3624992371,
		Longitude: -111.5960006714,
		Timezone:  "America/Denver",
	},
	"K3A1": {
		ICAO:      "K3A1",
		Name:      "Folsom Field",
		City:      "Cullman",
		State:     "Alabama",
		Country:   "US",
		Elevation: 969,
		Latitude:  34.268699646,
		Longitude: -86.858001709,
		Timezone:  "America/Chicago",
	},
	"K3A2": {
		ICAO:      "K3A2",
		Name:      "New Tazewell Municipal Airport",
		City:      "Tazewell",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1179,
		Latitude:  36.4100990295,
		Longitude: -83.5554962158,
		Timezone:  "America/New_York",
	},
	"K3A4": {
		ICAO:      "K3A4",
		Name:      "Southeast Greensboro Airport",
		City:      "Greensboro",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 737,
		Latitude:  35.9420013428,
		Longitude: -79.6855010986,
		Timezone:  "America/New_York",
	},
	"K3A7": {
		ICAO:      "K3A7",
		Name:      "Eutaw Municipal Airport",
		City:      "Eutaw",
		State:     "Alabama",
		Country:   "US",
		Elevation: 170,
		Latitude:  32.8209991455,
		Longitude: -87.8625030518,
		Timezone:  "America/Chicago",
	},
	"K3AU": {
		ICAO:      "K3AU",
		Name:      "Augusta Municipal Airport",
		City:      "Augusta",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1328,
		Latitude:  37.6716003418,
		Longitude: -97.0779037476,
		Timezone:  "America/Chicago",
	},
	"K3B0": {
		ICAO:      "K3B0",
		Name:      "Southbridge Municipal Airport",
		City:      "Southbridge",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 699,
		Latitude:  42.1008987427,
		Longitude: -72.0383987427,
		Timezone:  "America/New_York",
	},
	"K3B1": {
		ICAO:      "K3B1",
		Name:      "Greenville Municipal Airport",
		City:      "Greenville",
		State:     "Maine",
		Country:   "US",
		Elevation: 1401,
		Latitude:  45.4630012512,
		Longitude: -69.5515975952,
		Timezone:  "America/New_York",
	},
	"K3B4": {
		ICAO:      "K3B4",
		Name:      "Littlebrook Air Park",
		City:      "Eliot",
		State:     "Maine",
		Country:   "US",
		Elevation: 130,
		Latitude:  43.1431007385,
		Longitude: -70.7723007202,
		Timezone:  "America/New_York",
	},
	"K3B5": {
		ICAO:      "K3B5",
		Name:      "Twitchell Airport",
		City:      "Turner",
		State:     "Maine",
		Country:   "US",
		Elevation: 356,
		Latitude:  44.1888999939,
		Longitude: -70.2329025269,
		Timezone:  "America/New_York",
	},
	"K3C8": {
		ICAO:      "K3C8",
		Name:      "Calverton Executive Airpark",
		City:      "Calverton",
		State:     "New-York",
		Country:   "US",
		Elevation: 75,
		Latitude:  40.9151000977,
		Longitude: -72.7919006348,
		Timezone:  "America/New_York",
	},
	"K3CK": {
		ICAO:      "K3CK",
		Name:      "Lake in the Hills Airport",
		City:      "Chicago/Lake in the Hills",
		State:     "Illinois",
		Country:   "US",
		Elevation: 888,
		Latitude:  42.2067985535,
		Longitude: -88.3229980469,
		Timezone:  "America/Chicago",
	},
	"K3CM": {
		ICAO:      "K3CM",
		Name:      "James Clements Municipal Airport",
		City:      "Bay City",
		State:     "Michigan",
		Country:   "US",
		Elevation: 585,
		Latitude:  43.5469017,
		Longitude: -83.89550018,
		Timezone:  "America/Detroit",
	},
	"K3CU": {
		ICAO:      "K3CU",
		Name:      "Cable Union Airport",
		City:      "Cable",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1360,
		Latitude:  46.1941986084,
		Longitude: -91.2463989258,
		Timezone:  "America/Chicago",
	},
	"K3DW": {
		ICAO:      "K3DW",
		Name:      "Downtown Airport",
		City:      "Springfield",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1374,
		Latitude:  37.221698761,
		Longitude: -93.2477035522,
		Timezone:  "America/Chicago",
	},
	"K3E0": {
		ICAO:      "K3E0",
		Name:      "Miami Roberts County Airport",
		City:      "Miami",
		State:     "Texas",
		Country:   "US",
		Elevation: 2720,
		Latitude:  35.7139015198,
		Longitude: -100.6029968262,
		Timezone:  "America/Chicago",
	},
	"K3F2": {
		ICAO:      "K3F2",
		Name:      "Cisco Municipal Airport",
		City:      "Cisco",
		State:     "Texas",
		Country:   "US",
		Elevation: 1612,
		Latitude:  32.4168014526,
		Longitude: -99.0002975464,
		Timezone:  "America/Chicago",
	},
	"K3F3": {
		ICAO:      "K3F3",
		Name:      "C E 'Rusty' Williams Airport",
		City:      "Mansfield",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 324,
		Latitude:  32.0735015869,
		Longitude: -93.7655029297,
		Timezone:  "America/Chicago",
	},
	"K3F4": {
		ICAO:      "K3F4",
		Name:      "Vivian Airport",
		City:      "Vivian",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 260,
		Latitude:  32.8613014221,
		Longitude: -94.0102005005,
		Timezone:  "America/Chicago",
	},
	"K3F6": {
		ICAO:      "K3F6",
		Name:      "Dan E Richards Municipal Airport",
		City:      "Paducah",
		State:     "Texas",
		Country:   "US",
		Elevation: 1860,
		Latitude:  34.0275993347,
		Longitude: -100.2819976807,
		Timezone:  "America/Chicago",
	},
	"K3F7": {
		ICAO:      "K3F7",
		Name:      "Jones Memorial Airport",
		City:      "Bristow",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 851,
		Latitude:  35.8069000244,
		Longitude: -96.4218978882,
		Timezone:  "America/Chicago",
	},
	"K3F9": {
		ICAO:      "K3F9",
		Name:      "Mineola Wisener Field",
		City:      "Mineola",
		State:     "Texas",
		Country:   "US",
		Elevation: 429,
		Latitude:  32.676700592,
		Longitude: -95.5108032227,
		Timezone:  "America/Chicago",
	},
	"K3FU": {
		ICAO:      "K3FU",
		Name:      "Faulkton Municipal Airport",
		City:      "Faulkton",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1569,
		Latitude:  45.0318984985,
		Longitude: -99.1156997681,
		Timezone:  "America/Chicago",
	},
	"K3G1": {
		ICAO:      "K3G1",
		Name:      "Erie County Airport",
		City:      "Wattsburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1450,
		Latitude:  42.0444984436,
		Longitude: -79.8538970947,
		Timezone:  "America/New_York",
	},
	"K3G2": {
		ICAO:      "K3G2",
		Name:      "Grygla Municipal Mel Wilkens Field",
		City:      "Grygla",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1177,
		Latitude:  48.2966995239,
		Longitude: -95.6271972656,
		Timezone:  "America/Chicago",
	},
	"K3G3": {
		ICAO:      "K3G3",
		Name:      "Wadsworth Municipal Airport",
		City:      "Wadsworth",
		State:     "Ohio",
		Country:   "US",
		Elevation: 974,
		Latitude:  41.0031013489,
		Longitude: -81.7565002441,
		Timezone:  "America/New_York",
	},
	"K3G4": {
		ICAO:      "K3G4",
		Name:      "Ashland County Airport",
		City:      "Ashland",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1206,
		Latitude:  40.9029998779,
		Longitude: -82.2555999756,
		Timezone:  "America/New_York",
	},
	"K3G5": {
		ICAO:      "K3G5",
		Name:      "Dawson Army Air Field",
		City:      "Camp Dawson (Kingwood)",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 1265,
		Latitude:  39.4500999451,
		Longitude: -79.6663970947,
		Timezone:  "America/New_York",
	},
	"K3G6": {
		ICAO:      "K3G6",
		Name:      "Tri City Airport",
		City:      "Sebring",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1188,
		Latitude:  40.90599823,
		Longitude: -81,
		Timezone:  "America/New_York",
	},
	"K3GM": {
		ICAO:      "K3GM",
		Name:      "Grand Haven Memorial Airpark",
		City:      "Grand Haven",
		State:     "Michigan",
		Country:   "US",
		Elevation: 604,
		Latitude:  43.0340003967,
		Longitude: -86.1981964111,
		Timezone:  "America/Detroit",
	},
	"K3GV": {
		ICAO:      "K3GV",
		Name:      "East Kansas City Airport",
		City:      "Grain Valley",
		State:     "Missouri",
		Country:   "US",
		Elevation: 835,
		Latitude:  39.0155982971,
		Longitude: -94.2133026123,
		Timezone:  "America/Chicago",
	},
	"K3H4": {
		ICAO:      "K3H4",
		Name:      "Hillsboro Municipal Airport",
		City:      "Hillsboro",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 905,
		Latitude:  47.3594017029,
		Longitude: -97.0604019165,
		Timezone:  "America/Chicago",
	},
	"K3HO": {
		ICAO:      "K3HO",
		Name:      "Hobart Sky Ranch Airport",
		City:      "Hobart",
		State:     "Indiana",
		Country:   "US",
		Elevation: 644,
		Latitude:  41.5541992188,
		Longitude: -87.2624969482,
		Timezone:  "America/Chicago",
	},
	"K3I2": {
		ICAO:      "K3I2",
		Name:      "Mason County Airport",
		City:      "Point Pleasant",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 643,
		Latitude:  38.9146003723,
		Longitude: -82.0986022949,
		Timezone:  "America/New_York",
	},
	"K3I3": {
		ICAO:      "K3I3",
		Name:      "Sky King Airport",
		City:      "Terre Haute",
		State:     "Indiana",
		Country:   "US",
		Elevation: 496,
		Latitude:  39.5477981567,
		Longitude: -87.3772964478,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"K3I4": {
		ICAO:      "K3I4",
		Name:      "Richwood Municipal Airport",
		City:      "Richwood",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 2486,
		Latitude:  38.2583999634,
		Longitude: -80.6507034302,
		Timezone:  "America/New_York",
	},
	"K3I7": {
		ICAO:      "K3I7",
		Name:      "Phillipsburg Airport",
		City:      "Phillipsburg",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1031,
		Latitude:  39.9132995605,
		Longitude: -84.4003982544,
		Timezone:  "America/New_York",
	},
	"K3J0": {
		ICAO:      "K3J0",
		Name:      "Hampton Varnville Airport",
		City:      "Hampton",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 113,
		Latitude:  32.8678016663,
		Longitude: -81.0831985474,
		Timezone:  "America/New_York",
	},
	"K3J1": {
		ICAO:      "K3J1",
		Name:      "Ridgeland Airport",
		City:      "Ridgeland",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 79,
		Latitude:  32.4926986694,
		Longitude: -80.9923019409,
		Timezone:  "America/New_York",
	},
	"K3J7": {
		ICAO:      "K3J7",
		Name:      "Greene County Regional Airport",
		City:      "Greensboro",
		State:     "Georgia",
		Country:   "US",
		Elevation: 677,
		Latitude:  33.5976982117,
		Longitude: -83.138999939,
		Timezone:  "America/New_York",
	},
	"K3JC": {
		ICAO:      "K3JC",
		Name:      "Freeman Field",
		City:      "Junction City",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1101,
		Latitude:  39.0433006287,
		Longitude: -96.8432998657,
		Timezone:  "America/Chicago",
	},
	"K3K3": {
		ICAO:      "K3K3",
		Name:      "Syracuse Hamilton County Municipal Airport",
		City:      "Syracuse",
		State:     "Kansas",
		Country:   "US",
		Elevation: 3322,
		Latitude:  37.9916992188,
		Longitude: -101.7460021973,
		Timezone:  "America/Denver",
	},
	"K3K6": {
		ICAO:      "K3K6",
		Name:      "St Louis Metro East Shafer Field",
		City:      "St Jacob",
		State:     "Illinois",
		Country:   "US",
		Elevation: 477,
		Latitude:  38.7328987122,
		Longitude: -89.806602478,
		Timezone:  "America/Chicago",
	},
	"K3K7": {
		ICAO:      "K3K7",
		Name:      "Mark Hoard Memorial Airport",
		City:      "Leoti",
		State:     "Kansas",
		Country:   "US",
		Elevation: 3303,
		Latitude:  38.4570007324,
		Longitude: -101.3529968262,
		Timezone:  "America/Chicago",
	},
	"K3K8": {
		ICAO:      "K3K8",
		Name:      "Comanche County Airport",
		City:      "Coldwater",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2085,
		Latitude:  37.228099823,
		Longitude: -99.3309020996,
		Timezone:  "America/Chicago",
	},
	"K3L2": {
		ICAO:      "K3L2",
		Name:      "Sky Ranch Airport",
		City:      "Sandy Valley",
		State:     "Nevada",
		Country:   "US",
		Elevation: 2599,
		Latitude:  35.79529953,
		Longitude: -115.6269989014,
		Timezone:  "America/Los_Angeles",
	},
	"K3LF": {
		ICAO:      "K3LF",
		Name:      "Litchfield Municipal Airport",
		City:      "Litchfield",
		State:     "Illinois",
		Country:   "US",
		Elevation: 690,
		Latitude:  39.1624984741,
		Longitude: -89.6745986938,
		Timezone:  "America/Chicago",
	},
	"K3M0": {
		ICAO:      "K3M0",
		Name:      "Gastons Airport",
		City:      "Lakeview, Arkansas",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 479,
		Latitude:  36.3490556,
		Longitude: -92.5558056,
		Timezone:  "America/Chicago",
	},
	"K3M2": {
		ICAO:      "K3M2",
		Name:      "Double Springs Winston County Airport",
		City:      "Double Springs",
		State:     "Alabama",
		Country:   "US",
		Elevation: 753,
		Latitude:  34.1445007324,
		Longitude: -87.327796936,
		Timezone:  "America/Chicago",
	},
	"K3M3": {
		ICAO:      "K3M3",
		Name:      "Collegedale Municipal Airport",
		City:      "Collegedale",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 860,
		Latitude:  35.0430984497,
		Longitude: -85.0218963623,
		Timezone:  "America/New_York",
	},
	"K3M7": {
		ICAO:      "K3M7",
		Name:      "Lafayette Municipal Airport",
		City:      "Lafayette",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 969,
		Latitude:  36.51839828,
		Longitude: -86.0582962,
		Timezone:  "America/Chicago",
	},
	"K3M8": {
		ICAO:      "K3M8",
		Name:      "North Pickens Airport",
		City:      "Reform",
		State:     "Alabama",
		Country:   "US",
		Elevation: 237,
		Latitude:  33.3867988586,
		Longitude: -88.0065994263,
		Timezone:  "America/Chicago",
	},
	"K3M9": {
		ICAO:      "K3M9",
		Name:      "Warren Municipal Airport",
		City:      "Warren",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 235,
		Latitude:  33.5603981018,
		Longitude: -92.0854034424,
		Timezone:  "America/Chicago",
	},
	"K3MY": {
		ICAO:      "K3MY",
		Name:      "Mount Hawley Auxiliary Airport",
		City:      "Peoria",
		State:     "Illinois",
		Country:   "US",
		Elevation: 786,
		Latitude:  40.79529953,
		Longitude: -89.61340332,
		Timezone:  "America/Chicago",
	},
	"K3N6": {
		ICAO:      "K3N6",
		Name:      "Old Bridge Airport",
		City:      "Old Bridge",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 87,
		Latitude:  40.3298988342,
		Longitude: -74.3468017578,
		Timezone:  "America/New_York",
	},
	"K3O1": {
		ICAO:      "K3O1",
		Name:      "Gustine Airport",
		City:      "Gustine",
		State:     "California",
		Country:   "US",
		Elevation: 75,
		Latitude:  37.2626991272,
		Longitude: -120.9629974365,
		Timezone:  "America/Los_Angeles",
	},
	"K3O3": {
		ICAO:      "K3O3",
		Name:      "Purcell Municipal - Steven E. Shephard field",
		City:      "Purcell",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1143,
		Latitude:  34.98450089,
		Longitude: -97.38279724,
		Timezone:  "America/Chicago",
	},
	"K3O4": {
		ICAO:      "K3O4",
		Name:      "Sayre Municipal Airport",
		City:      "Sayre",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1937,
		Latitude:  35.1675987244,
		Longitude: -99.6578979492,
		Timezone:  "America/Chicago",
	},
	"K3O9": {
		ICAO:      "K3O9",
		Name:      "Grand Lake Regional Airport",
		City:      "Afton",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 792,
		Latitude:  36.5775985718,
		Longitude: -94.8619003296,
		Timezone:  "America/Chicago",
	},
	"K3P3": {
		ICAO:      "K3P3",
		Name:      "Mott Municipal Airport",
		City:      "Mott",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 2411,
		Latitude:  46.3596992493,
		Longitude: -102.3229980469,
		Timezone:  "America/Denver",
	},
	"K3R2": {
		ICAO:      "K3R2",
		Name:      "Le Gros Memorial Airport",
		City:      "Crowley",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 17,
		Latitude:  30.1609992981,
		Longitude: -92.4849014282,
		Timezone:  "America/Chicago",
	},
	"K3R4": {
		ICAO:      "K3R4",
		Name:      "Hart Airport",
		City:      "Many",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 319,
		Latitude:  31.54490089,
		Longitude: -93.4865036,
		Timezone:  "America/Chicago",
	},
	"K3R7": {
		ICAO:      "K3R7",
		Name:      "Jennings Airport",
		City:      "Jennings",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 23,
		Latitude:  30.2427005768,
		Longitude: -92.6734008789,
		Timezone:  "America/Chicago",
	},
	"K3R9": {
		ICAO:      "K3R9",
		Name:      "Lakeway Airpark",
		City:      "Austin",
		State:     "Texas",
		Country:   "US",
		Elevation: 909,
		Latitude:  30.3575000763,
		Longitude: -97.9944992065,
		Timezone:  "America/Chicago",
	},
	"K3RC": {
		ICAO:      "K3RC",
		Name:      "Roscommon Conservation Airport",
		City:      "Roscommon",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1156,
		Latitude:  44.4749984741,
		Longitude: -84.5667037964,
		Timezone:  "America/Detroit",
	},
	"K3S4": {
		ICAO:      "K3S4",
		Name:      "Illinois Valley Airport",
		City:      "Cave Junction",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1394,
		Latitude:  42.1036987305,
		Longitude: -123.6819992065,
		Timezone:  "America/Los_Angeles",
	},
	"K3S8": {
		ICAO:      "K3S8",
		IATA:      "GTP",
		Name:      "Grants Pass Airport",
		City:      "Grants Pass",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1126,
		Latitude:  42.5101013184,
		Longitude: -123.388000488,
		Timezone:  "America/Los_Angeles",
	},
	"K3S9": {
		ICAO:      "K3S9",
		Name:      "Condon State Pauling Field",
		City:      "Condon",
		State:     "Oregon",
		Country:   "US",
		Elevation: 2911,
		Latitude:  45.2466011047,
		Longitude: -120.1660003662,
		Timezone:  "America/Los_Angeles",
	},
	"K3SQ": {
		ICAO:      "K3SQ",
		Name:      "St Charles Airport",
		City:      "St Charles",
		State:     "Missouri",
		Country:   "US",
		Elevation: 442,
		Latitude:  38.8487014771,
		Longitude: -90.5000991821,
		Timezone:  "America/Chicago",
	},
	"K3T3": {
		ICAO:      "K3T3",
		Name:      "Boyceville Municipal Airport",
		City:      "Boyceville",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 967,
		Latitude:  45.0439987183,
		Longitude: -92.020401001,
		Timezone:  "America/Chicago",
	},
	"K3T5": {
		ICAO:      "K3T5",
		Name:      "Fayette Regional Air Center Airport",
		City:      "La Grange",
		State:     "Texas",
		Country:   "US",
		Elevation: 324,
		Latitude:  29.908000946,
		Longitude: -96.9499969482,
		Timezone:  "America/Chicago",
	},
	"K3TR": {
		ICAO:      "K3TR",
		Name:      "Jerry Tyler Memorial Airport",
		City:      "Niles",
		State:     "Michigan",
		Country:   "US",
		Elevation: 750,
		Latitude:  41.835899353,
		Longitude: -86.2251968384,
		Timezone:  "America/Detroit",
	},
	"K3U3": {
		ICAO:      "K3U3",
		Name:      "Bowman Field",
		City:      "Anaconda",
		State:     "Montana",
		Country:   "US",
		Elevation: 5034,
		Latitude:  46.1530990601,
		Longitude: -112.8679962158,
		Timezone:  "America/Denver",
	},
	"K3U4": {
		ICAO:      "K3U4",
		Name:      "St Labre Mission Airport",
		City:      "Ashland",
		State:     "Montana",
		Country:   "US",
		Elevation: 2909,
		Latitude:  45.5999984741,
		Longitude: -106.266998291,
		Timezone:  "America/Denver",
	},
	"K3U7": {
		ICAO:      "K3U7",
		Name:      "Benchmark Airport",
		City:      "Benchmark",
		State:     "Montana",
		Country:   "US",
		Elevation: 5434,
		Latitude:  47.481300354,
		Longitude: -112.8700027466,
		Timezone:  "America/Denver",
	},
	"K3U8": {
		ICAO:      "K3U8",
		Name:      "Big Sandy Airport",
		City:      "Big Sandy",
		State:     "Montana",
		Country:   "US",
		Elevation: 2700,
		Latitude:  48.1624984741,
		Longitude: -110.1129989624,
		Timezone:  "America/Denver",
	},
	"K3V0": {
		ICAO:      "K3V0",
		Name:      "Custer State Park Airport",
		City:      "Fairburn",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 3980,
		Latitude:  43.7249984741,
		Longitude: -103.3499984741,
		Timezone:  "America/Denver",
	},
	"K3V7": {
		ICAO:      "K3V7",
		Name:      "Belle Creek Airport",
		City:      "Belle Creek",
		State:     "Montana",
		Country:   "US",
		Elevation: 3678,
		Latitude:  45.125,
		Longitude: -105.0920028687,
		Timezone:  "America/Denver",
	},
	"K3W7": {
		ICAO:      "K3W7",
		IATA:      "GCD",
		Name:      "Grand Coulee Dam Airport",
		City:      "Electric City",
		State:     "Washington",
		Country:   "US",
		Elevation: 1588,
		Latitude:  47.922000885,
		Longitude: -119.083000183,
		Timezone:  "America/Los_Angeles",
	},
	"K3W8": {
		ICAO:      "K3W8",
		Name:      "Eureka Municipal Airport",
		City:      "Eureka",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1935,
		Latitude:  45.7999992371,
		Longitude: -99.6420974731,
		Timezone:  "America/Chicago",
	},
	"K3Y2": {
		ICAO:      "K3Y2",
		Name:      "George L Scott Municipal Airport",
		City:      "West Union",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1232,
		Latitude:  42.9850997925,
		Longitude: -91.7906036377,
		Timezone:  "America/Chicago",
	},
	"K3Y3": {
		ICAO:      "K3Y3",
		Name:      "Winterset Madison County Airport",
		City:      "Winterset",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1110,
		Latitude:  41.3628005981,
		Longitude: -94.0211029053,
		Timezone:  "America/Chicago",
	},
	"K40G": {
		ICAO:      "K40G",
		Name:      "Valle Airport",
		City:      "Grand Canyon",
		State:     "Arizona",
		Country:   "US",
		Elevation: 5999,
		Latitude:  35.6506004333,
		Longitude: -112.1480026245,
		Timezone:  "America/Phoenix",
	},
	"K40J": {
		ICAO:      "K40J",
		IATA:      "FPY",
		Name:      "Perry Foley Airport",
		City:      "Perry",
		State:     "Florida",
		Country:   "US",
		Elevation: 44,
		Latitude:  30.0692997,
		Longitude: -83.58059692,
		Timezone:  "America/New_York",
	},
	"K40U": {
		ICAO:      "K40U",
		Name:      "Manila Airport",
		City:      "Manila",
		State:     "Utah",
		Country:   "US",
		Elevation: 6175,
		Latitude:  40.9860992432,
		Longitude: -109.6780014038,
		Timezone:  "America/Denver",
	},
	"K41A": {
		ICAO:      "K41A",
		Name:      "Reeves Airport",
		City:      "Tallassee",
		State:     "Alabama",
		Country:   "US",
		Elevation: 326,
		Latitude:  32.5147018433,
		Longitude: -85.8759994507,
		Timezone:  "America/Chicago",
	},
	"K41F": {
		ICAO:      "K41F",
		Name:      "Floydada Municipal Airport",
		City:      "Floydada",
		State:     "Texas",
		Country:   "US",
		Elevation: 3187,
		Latitude:  34.00230026,
		Longitude: -101.3300018,
		Timezone:  "America/Chicago",
	},
	"K41N": {
		ICAO:      "K41N",
		Name:      "Braceville Airport",
		City:      "Newton Falls",
		State:     "Ohio",
		Country:   "US",
		Elevation: 900,
		Latitude:  41.2112007141,
		Longitude: -80.9692993164,
		Timezone:  "America/New_York",
	},
	"K41U": {
		ICAO:      "K41U",
		Name:      "Manti Ephraim Airport",
		City:      "Manti",
		State:     "Utah",
		Country:   "US",
		Elevation: 5500,
		Latitude:  39.32910156,
		Longitude: -111.6149979,
		Timezone:  "America/Denver",
	},
	"K42A": {
		ICAO:      "K42A",
		Name:      "Melbourne Municipal John E Miller Field",
		City:      "Melbourne",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 735,
		Latitude:  36.07099915,
		Longitude: -91.83010101,
		Timezone:  "America/Chicago",
	},
	"K42I": {
		ICAO:      "K42I",
		Name:      "Parr Airport",
		City:      "Zanesville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 790,
		Latitude:  40.0069999695,
		Longitude: -82.0123977661,
		Timezone:  "America/New_York",
	},
	"K42M": {
		ICAO:      "K42M",
		Name:      "Thayer Memorial Airport",
		City:      "Thayer",
		State:     "Missouri",
		Country:   "US",
		Elevation: 790,
		Latitude:  36.5223007202,
		Longitude: -91.5719985962,
		Timezone:  "America/Chicago",
	},
	"K42S": {
		ICAO:      "K42S",
		Name:      "Poplar Airport",
		City:      "Poplar",
		State:     "Montana",
		Country:   "US",
		Elevation: 2005,
		Latitude:  48.1160011292,
		Longitude: -105.1819992065,
		Timezone:  "America/Denver",
	},
	"K42U": {
		ICAO:      "K42U",
		Name:      "Morgan County Airport",
		City:      "Morgan",
		State:     "Utah",
		Country:   "US",
		Elevation: 5020,
		Latitude:  41.1487998962,
		Longitude: -111.766998291,
		Timezone:  "America/Denver",
	},
	"K42V": {
		ICAO:      "K42V",
		Name:      "Jones Airport",
		City:      "Benkelman",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3126,
		Latitude:  40.0579986572,
		Longitude: -101.5469970703,
		Timezone:  "America/Denver",
	},
	"K43A": {
		ICAO:      "K43A",
		Name:      "Montgomery County Airport",
		City:      "Star",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 628,
		Latitude:  35.3881988525,
		Longitude: -79.7928009033,
		Timezone:  "America/New_York",
	},
	"K43B": {
		ICAO:      "K43B",
		Name:      "Deblois Flight Strip",
		City:      "Deblois",
		State:     "Maine",
		Country:   "US",
		Elevation: 217,
		Latitude:  44.726398468,
		Longitude: -67.9906997681,
		Timezone:  "America/New_York",
	},
	"K43D": {
		ICAO:      "K43D",
		Name:      "Odessa Municipal Airport",
		City:      "Odessa",
		State:     "Washington",
		Country:   "US",
		Elevation: 1737,
		Latitude:  47.3475990295,
		Longitude: -118.6770019531,
		Timezone:  "America/Los_Angeles",
	},
	"K43U": {
		ICAO:      "K43U",
		Name:      "Mount Pleasant Airport",
		City:      "Mount Pleasant",
		State:     "Utah",
		Country:   "US",
		Elevation: 5830,
		Latitude:  39.52470016,
		Longitude: -111.4749985,
		Timezone:  "America/Denver",
	},
	"K44C": {
		ICAO:      "K44C",
		Name:      "Beloit Airport",
		City:      "Beloit",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 817,
		Latitude:  42.4977989197,
		Longitude: -88.9675979614,
		Timezone:  "America/Chicago",
	},
	"K44U": {
		ICAO:      "K44U",
		Name:      "Salina Gunnison Airport",
		City:      "Salina",
		State:     "Utah",
		Country:   "US",
		Elevation: 5159,
		Latitude:  39.0290985107,
		Longitude: -111.8379974365,
		Timezone:  "America/Denver",
	},
	"K45G": {
		ICAO:      "K45G",
		Name:      "Brighton Airport",
		City:      "Brighton",
		State:     "Michigan",
		Country:   "US",
		Elevation: 973,
		Latitude:  42.5698013306,
		Longitude: -83.778503418,
		Timezone:  "America/Detroit",
	},
	"K45K": {
		ICAO:      "K45K",
		Name:      "Minneapolis City County Airport",
		City:      "Minneapolis",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1245,
		Latitude:  39.0946998596,
		Longitude: -97.7205963135,
		Timezone:  "America/Chicago",
	},
	"K45R": {
		ICAO:      "K45R",
		Name:      "Hawthorne Field",
		City:      "Kountze/Silsbee",
		State:     "Texas",
		Country:   "US",
		Elevation: 71,
		Latitude:  30.3362998962,
		Longitude: -94.2574996948,
		Timezone:  "America/Chicago",
	},
	"K46D": {
		ICAO:      "K46D",
		Name:      "Carrington Municipal Airport",
		City:      "Carrington",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1607,
		Latitude:  47.4510994,
		Longitude: -99.15110016,
		Timezone:  "America/Chicago",
	},
	"K46U": {
		ICAO:      "K46U",
		Name:      "Alpine Airport",
		City:      "Alpine",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 5634,
		Latitude:  43.1846008301,
		Longitude: -111.0419998169,
		Timezone:  "America/Boise",
	},
	"K47A": {
		ICAO:      "K47A",
		Name:      "Cherokee County Airport",
		City:      "Canton",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1219,
		Latitude:  34.3106002808,
		Longitude: -84.4238967896,
		Timezone:  "America/New_York",
	},
	"K47K": {
		ICAO:      "K47K",
		Name:      "Moundridge Municipal Airport",
		City:      "Moundridge",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1489,
		Latitude:  38.2090988159,
		Longitude: -97.5027008057,
		Timezone:  "America/Chicago",
	},
	"K47V": {
		ICAO:      "K47V",
		Name:      "Curtis Municipal Airport",
		City:      "Curtis",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2678,
		Latitude:  40.63750076,
		Longitude: -100.4710007,
		Timezone:  "America/Chicago",
	},
	"K48A": {
		ICAO:      "K48A",
		Name:      "Cochran Airport",
		City:      "Cochran",
		State:     "Georgia",
		Country:   "US",
		Elevation: 377,
		Latitude:  32.3993988,
		Longitude: -83.27590179,
		Timezone:  "America/New_York",
	},
	"K48D": {
		ICAO:      "K48D",
		Name:      "Clare Municipal Airport",
		City:      "Clare",
		State:     "Michigan",
		Country:   "US",
		Elevation: 857,
		Latitude:  43.8344993591,
		Longitude: -84.740196228,
		Timezone:  "America/Detroit",
	},
	"K48I": {
		ICAO:      "K48I",
		Name:      "Braxton County Airport",
		City:      "Sutton",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 1276,
		Latitude:  38.68700027,
		Longitude: -80.65180206,
		Timezone:  "America/New_York",
	},
	"K48K": {
		ICAO:      "K48K",
		Name:      "Ness City Municipal Airport",
		City:      "Ness City",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2308,
		Latitude:  38.4710998535,
		Longitude: -99.9080963135,
		Timezone:  "America/Chicago",
	},
	"K48S": {
		ICAO:      "K48S",
		Name:      "Harlem Airport",
		City:      "Harlem",
		State:     "Montana",
		Country:   "US",
		Elevation: 2643,
		Latitude:  48.5661010742,
		Longitude: -108.7730026245,
		Timezone:  "America/Denver",
	},
	"K49A": {
		ICAO:      "K49A",
		Name:      "Gilmer County Airport",
		City:      "Ellijay",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1486,
		Latitude:  34.6278991699,
		Longitude: -84.5249023438,
		Timezone:  "America/New_York",
	},
	"K49B": {
		ICAO:      "K49B",
		Name:      "Sturgis Municipal Airport",
		City:      "Sturgis",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 3243,
		Latitude:  44.4179992676,
		Longitude: -103.375,
		Timezone:  "America/Denver",
	},
	"K49R": {
		ICAO:      "K49R",
		Name:      "Real County Airport",
		City:      "Leakey",
		State:     "Texas",
		Country:   "US",
		Elevation: 1640,
		Latitude:  29.7455005646,
		Longitude: -99.7609024048,
		Timezone:  "America/Chicago",
	},
	"K49X": {
		ICAO:      "K49X",
		Name:      "Chemehuevi Valley Airport",
		City:      "Chemehuevi Valley",
		State:     "California",
		Country:   "US",
		Elevation: 631,
		Latitude:  34.5275001526,
		Longitude: -114.4309997559,
		Timezone:  "America/Los_Angeles",
	},
	"K4A0": {
		ICAO:      "K4A0",
		Name:      "Berry Hill Airport",
		City:      "Stockbridge",
		State:     "Georgia",
		Country:   "US",
		Elevation: 770,
		Latitude:  33.5357017517,
		Longitude: -84.1791000366,
		Timezone:  "America/New_York",
	},
	"K4A4": {
		ICAO:      "K4A4",
		Name:      "Polk County Airport- Cornelius Moore Field",
		City:      "Cedartown",
		State:     "Georgia",
		Country:   "US",
		Elevation: 974,
		Latitude:  34.018699646,
		Longitude: -85.1464996338,
		Timezone:  "America/New_York",
	},
	"K4A5": {
		ICAO:      "K4A5",
		Name:      "Searcy County Airport",
		City:      "Marshall",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 963,
		Latitude:  35.8969993591,
		Longitude: -92.658996582,
		Timezone:  "America/Chicago",
	},
	"K4A6": {
		ICAO:      "K4A6",
		Name:      "Scottsboro Municipal Word Field",
		City:      "Scottsboro",
		State:     "Alabama",
		Country:   "US",
		Elevation: 650,
		Latitude:  34.68870163,
		Longitude: -86.00589752,
		Timezone:  "America/Chicago",
	},
	"K4A7": {
		ICAO:      "K4A7",
		Name:      "Clayton County Tara Field",
		City:      "Hampton",
		State:     "Georgia",
		Country:   "US",
		Elevation: 874,
		Latitude:  33.3890991211,
		Longitude: -84.3323974609,
		Timezone:  "America/New_York",
	},
	"K4A9": {
		ICAO:      "K4A9",
		Name:      "Isbell Field",
		City:      "Fort Payne",
		State:     "Alabama",
		Country:   "US",
		Elevation: 877,
		Latitude:  34.4737014771,
		Longitude: -85.7213973999,
		Timezone:  "America/Chicago",
	},
	"K4B6": {
		ICAO:      "K4B6",
		Name:      "Ticonderoga Municipal Airport",
		City:      "Ticonderoga",
		State:     "New-York",
		Country:   "US",
		Elevation: 273,
		Latitude:  43.87730026,
		Longitude: -73.4131012,
		Timezone:  "America/New_York",
	},
	"K4B7": {
		ICAO:      "K4B7",
		Name:      "Schroon Lake Airport",
		City:      "Schroon Lake",
		State:     "New-York",
		Country:   "US",
		Elevation: 830,
		Latitude:  43.8625984192,
		Longitude: -73.7425994873,
		Timezone:  "America/New_York",
	},
	"K4B8": {
		ICAO:      "K4B8",
		Name:      "Robertson Field",
		City:      "Plainville",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 200,
		Latitude:  41.6903991699,
		Longitude: -72.8647994995,
		Timezone:  "America/New_York",
	},
	"K4C8": {
		ICAO:      "K4C8",
		Name:      "Albia Municipal Airport",
		City:      "Albia",
		State:     "Iowa",
		Country:   "US",
		Elevation: 963,
		Latitude:  40.9944992065,
		Longitude: -92.7630004883,
		Timezone:  "America/Chicago",
	},
	"K4D0": {
		ICAO:      "K4D0",
		Name:      "Abrams Municipal Airport",
		City:      "Grand Ledge",
		State:     "Michigan",
		Country:   "US",
		Elevation: 842,
		Latitude:  42.7742004395,
		Longitude: -84.7331008911,
		Timezone:  "America/Detroit",
	},
	"K4E7": {
		ICAO:      "K4E7",
		Name:      "Ellendale Municipal Airport",
		City:      "Ellendale",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1455,
		Latitude:  46.0125007629,
		Longitude: -98.5129013062,
		Timezone:  "America/Chicago",
	},
	"K4F2": {
		ICAO:      "K4F2",
		Name:      "Panola County Sharpe Field",
		City:      "Carthage",
		State:     "Texas",
		Country:   "US",
		Elevation: 248,
		Latitude:  32.1761016846,
		Longitude: -94.2987976074,
		Timezone:  "America/Chicago",
	},
	"K4F7": {
		ICAO:      "K4F7",
		Name:      "Kizer Field",
		City:      "Prescott",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 319,
		Latitude:  33.8039016724,
		Longitude: -93.362197876,
		Timezone:  "America/Chicago",
	},
	"K4F8": {
		ICAO:      "K4F8",
		Name:      "Wilson Airport",
		City:      "Stephens",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 230,
		Latitude:  33.4435997009,
		Longitude: -93.0541000366,
		Timezone:  "America/Chicago",
	},
	"K4F9": {
		ICAO:      "K4F9",
		Name:      "La Moure Rott Municipal Airport",
		City:      "La Moure",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1310,
		Latitude:  46.3465995789,
		Longitude: -98.2836990356,
		Timezone:  "America/Chicago",
	},
	"K4G4": {
		ICAO:      "K4G4",
		Name:      "Youngstown Elser Metro Airport",
		City:      "Youngstown",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1070,
		Latitude:  40.9617996216,
		Longitude: -80.6772994995,
		Timezone:  "America/New_York",
	},
	"K4G5": {
		ICAO:      "K4G5",
		Name:      "Monroe County Airport",
		City:      "Woodsfield",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1197,
		Latitude:  39.7789993286,
		Longitude: -81.1027984619,
		Timezone:  "America/New_York",
	},
	"K4G6": {
		ICAO:      "K4G6",
		Name:      "Hornell Municipal Airport",
		City:      "Hornell",
		State:     "New-York",
		Country:   "US",
		Elevation: 1220,
		Latitude:  42.3820991516,
		Longitude: -77.6820983887,
		Timezone:  "America/New_York",
	},
	"K4G8": {
		ICAO:      "K4G8",
		Name:      "Columbia Airport",
		City:      "Columbia Station",
		State:     "Ohio",
		Country:   "US",
		Elevation: 813,
		Latitude:  41.3188018799,
		Longitude: -81.9601974487,
		Timezone:  "America/New_York",
	},
	"K4I0": {
		ICAO:      "K4I0",
		Name:      "Mingo County Airport",
		City:      "Williamson",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 1575,
		Latitude:  37.6875991821,
		Longitude: -82.2610015869,
		Timezone:  "America/New_York",
	},
	"K4I3": {
		ICAO:      "K4I3",
		Name:      "Knox County Airport",
		City:      "Mount Vernon",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1191,
		Latitude:  40.32870102,
		Longitude: -82.52380371,
		Timezone:  "America/New_York",
	},
	"K4I7": {
		ICAO:      "K4I7",
		Name:      "Putnam County Airport",
		City:      "Greencastle",
		State:     "Indiana",
		Country:   "US",
		Elevation: 842,
		Latitude:  39.6302986145,
		Longitude: -86.8139038086,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"K4I9": {
		ICAO:      "K4I9",
		Name:      "Morrow County Airport",
		City:      "Mount Gilead",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1085,
		Latitude:  40.5245018005,
		Longitude: -82.8500976563,
		Timezone:  "America/New_York",
	},
	"K4J1": {
		ICAO:      "K4J1",
		Name:      "Brantley County Airport",
		City:      "Nahunta",
		State:     "Georgia",
		Country:   "US",
		Elevation: 83,
		Latitude:  31.2073993683,
		Longitude: -81.9057998657,
		Timezone:  "America/New_York",
	},
	"K4J2": {
		ICAO:      "K4J2",
		Name:      "Berrien County Airport",
		City:      "Nashville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 240,
		Latitude:  31.212600708,
		Longitude: -83.2263031006,
		Timezone:  "America/New_York",
	},
	"K4J5": {
		ICAO:      "K4J5",
		Name:      "Quitman Brooks County Airport",
		City:      "Quitman",
		State:     "Georgia",
		Country:   "US",
		Elevation: 185,
		Latitude:  30.8057994843,
		Longitude: -83.5865020752,
		Timezone:  "America/New_York",
	},
	"K4J6": {
		ICAO:      "K4J6",
		Name:      "St Marys Airport",
		City:      "St Marys",
		State:     "Georgia",
		Country:   "US",
		Elevation: 23,
		Latitude:  30.75300026,
		Longitude: -81.55879974,
		Timezone:  "America/New_York",
	},
	"K4J8": {
		ICAO:      "K4J8",
		Name:      "Treutlen County Airport",
		City:      "Soperton",
		State:     "Georgia",
		Country:   "US",
		Elevation: 345,
		Latitude:  32.3876991272,
		Longitude: -82.5636978149,
		Timezone:  "America/New_York",
	},
	"K4K3": {
		ICAO:      "K4K3",
		Name:      "Lexington Municipal Airport",
		City:      "Lexington",
		State:     "Missouri",
		Country:   "US",
		Elevation: 691,
		Latitude:  39.2098007202,
		Longitude: -93.9280014038,
		Timezone:  "America/Chicago",
	},
	"K4K6": {
		ICAO:      "K4K6",
		Name:      "Bloomfield Municipal Airport",
		City:      "Bloomfield",
		State:     "Iowa",
		Country:   "US",
		Elevation: 888,
		Latitude:  40.7321014404,
		Longitude: -92.4282989502,
		Timezone:  "America/Chicago",
	},
	"K4M1": {
		ICAO:      "K4M1",
		Name:      "Carroll County Airport",
		City:      "Berryville",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1205,
		Latitude:  36.38130188,
		Longitude: -93.62460327,
		Timezone:  "America/Chicago",
	},
	"K4M2": {
		ICAO:      "K4M2",
		Name:      "Booneville Municipal Airport",
		City:      "Booneville",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 465,
		Latitude:  35.1495018005,
		Longitude: -93.8620986938,
		Timezone:  "America/Chicago",
	},
	"K4M3": {
		ICAO:      "K4M3",
		Name:      "Carlisle Municipal Airport",
		City:      "Carlisle",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 241,
		Latitude:  34.80820084,
		Longitude: -91.71209717,
		Timezone:  "America/Chicago",
	},
	"K4M7": {
		ICAO:      "K4M7",
		Name:      "Russellville Logan County Airport",
		City:      "Russellville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 689,
		Latitude:  36.7999000549,
		Longitude: -86.8102035522,
		Timezone:  "America/Chicago",
	},
	"K4M9": {
		ICAO:      "K4M9",
		Name:      "Corning Municipal Airport",
		City:      "Corning",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 293,
		Latitude:  36.4042015076,
		Longitude: -90.6479034424,
		Timezone:  "America/Chicago",
	},
	"K4O4": {
		ICAO:      "K4O4",
		Name:      "Mc Curtain County Regional Airport",
		City:      "Idabel",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 472,
		Latitude:  33.9094009399,
		Longitude: -94.8593978882,
		Timezone:  "America/Chicago",
	},
	"K4O5": {
		ICAO:      "K4O5",
		Name:      "Cherokee Municipal Airport",
		City:      "Cherokee",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1177,
		Latitude:  36.7879981995,
		Longitude: -98.3584976196,
		Timezone:  "America/Chicago",
	},
	"K4P3": {
		ICAO:      "K4P3",
		Name:      "Flandreau Municipal Airport",
		City:      "Flandreau",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1645,
		Latitude:  44.0038986206,
		Longitude: -96.5931015015,
		Timezone:  "America/Chicago",
	},
	"K4R1": {
		ICAO:      "K4R1",
		Name:      "I H Bass Jr Memorial Airport",
		City:      "Lumberton",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 310,
		Latitude:  31.015499115,
		Longitude: -89.4825973511,
		Timezone:  "America/Chicago",
	},
	"K4R3": {
		ICAO:      "K4R3",
		Name:      "Jackson Municipal Airport",
		City:      "Jackson",
		State:     "Alabama",
		Country:   "US",
		Elevation: 62,
		Latitude:  31.4720993,
		Longitude: -87.8946991,
		Timezone:  "America/Chicago",
	},
	"K4R4": {
		ICAO:      "K4R4",
		Name:      "H L Sonny Callahan Airport",
		City:      "Fairhope",
		State:     "Alabama",
		Country:   "US",
		Elevation: 91,
		Latitude:  30.46050072,
		Longitude: -87.8769989,
		Timezone:  "America/Chicago",
	},
	"K4R5": {
		ICAO:      "K4R5",
		Name:      "Madeline Island Airport",
		City:      "La Pointe",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 649,
		Latitude:  46.7887001038,
		Longitude: -90.7586975098,
		Timezone:  "America/Chicago",
	},
	"K4R7": {
		ICAO:      "K4R7",
		Name:      "Eunice Airport",
		City:      "Eunice",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 42,
		Latitude:  30.466299057,
		Longitude: -92.4237976074,
		Timezone:  "America/Chicago",
	},
	"K4R9": {
		ICAO:      "K4R9",
		Name:      "Dauphin Island Airport",
		City:      "Dauphin Island",
		State:     "Alabama",
		Country:   "US",
		Elevation: 5,
		Latitude:  30.2604999542,
		Longitude: -88.1275024414,
		Timezone:  "America/Chicago",
	},
	"K4S1": {
		ICAO:      "K4S1",
		Name:      "Gold Beach Municipal Airport",
		City:      "Gold Beach",
		State:     "Oregon",
		Country:   "US",
		Elevation: 20,
		Latitude:  42.41339874,
		Longitude: -124.4240036,
		Timezone:  "America/Los_Angeles",
	},
	"K4S2": {
		ICAO:      "K4S2",
		Name:      "Ken Jernstedt Airfield",
		City:      "Hood River",
		State:     "Oregon",
		Country:   "US",
		Elevation: 631,
		Latitude:  45.67259979,
		Longitude: -121.5360031,
		Timezone:  "America/Los_Angeles",
	},
	"K4S9": {
		ICAO:      "K4S9",
		Name:      "Portland Mulino Airport",
		City:      "Portland-Mulino",
		State:     "Oregon",
		Country:   "US",
		Elevation: 260,
		Latitude:  45.2163009644,
		Longitude: -122.5899963379,
		Timezone:  "America/Los_Angeles",
	},
	"K4T2": {
		ICAO:      "K4T2",
		Name:      "Kenneth Copeland Airport",
		City:      "Fort Worth",
		State:     "Texas",
		Country:   "US",
		Elevation: 688,
		Latitude:  32.9772987366,
		Longitude: -97.4884033203,
		Timezone:  "America/Chicago",
	},
	"K4U6": {
		ICAO:      "K4U6",
		Name:      "Circle Town County Airport",
		City:      "Circle",
		State:     "Montana",
		Country:   "US",
		Elevation: 2426,
		Latitude:  47.418598175,
		Longitude: -105.56199646,
		Timezone:  "America/Denver",
	},
	"K4U9": {
		ICAO:      "K4U9",
		Name:      "Dell Flight Strip",
		City:      "Dell",
		State:     "Montana",
		Country:   "US",
		Elevation: 6007,
		Latitude:  44.7356987,
		Longitude: -112.720001221,
		Timezone:  "America/Denver",
	},
	"K4V1": {
		ICAO:      "K4V1",
		Name:      "Spanish Peaks Airfield",
		City:      "Walsenburg",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6047,
		Latitude:  37.69660187,
		Longitude: -104.7839966,
		Timezone:  "America/Denver",
	},
	"K4V4": {
		ICAO:      "K4V4",
		Name:      "Northwood Municipal Vince Field",
		City:      "Northwood",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1117,
		Latitude:  47.7242012,
		Longitude: -97.5904007,
		Timezone:  "America/Chicago",
	},
	"K4V9": {
		ICAO:      "K4V9",
		Name:      "Antelope County Airport",
		City:      "Neligh",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1774,
		Latitude:  42.11220169,
		Longitude: -98.03869629,
		Timezone:  "America/Chicago",
	},
	"K4X4": {
		ICAO:      "K4X4",
		Name:      "Wessington Springs Airport",
		City:      "Wessington Springs",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1546,
		Latitude:  44.061000824,
		Longitude: -98.5308990479,
		Timezone:  "America/Chicago",
	},
	"K4Y4": {
		ICAO:      "K4Y4",
		Name:      "Lakes of the North Airport",
		City:      "Gaylord",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1286,
		Latitude:  44.9124984741,
		Longitude: -84.8764038086,
		Timezone:  "America/Detroit",
	},
	"K50D": {
		ICAO:      "K50D",
		Name:      "Iron County Airport",
		City:      "Crystal Falls",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1340,
		Latitude:  46.0091018677,
		Longitude: -88.2739028931,
		Timezone:  "America/Menominee",
	},
	"K50F": {
		ICAO:      "K50F",
		Name:      "Bourland Field",
		City:      "Fort Worth",
		State:     "Texas",
		Country:   "US",
		Elevation: 873,
		Latitude:  32.5817985535,
		Longitude: -97.5907974243,
		Timezone:  "America/Chicago",
	},
	"K50I": {
		ICAO:      "K50I",
		Name:      "Kentland Municipal Airport",
		City:      "Kentland",
		State:     "Indiana",
		Country:   "US",
		Elevation: 698,
		Latitude:  40.7587013245,
		Longitude: -87.4281997681,
		Timezone:  "America/Chicago",
	},
	"K50R": {
		ICAO:      "K50R",
		Name:      "Lockhart Municipal Airport",
		City:      "Lockhart",
		State:     "Texas",
		Country:   "US",
		Elevation: 532,
		Latitude:  29.8502998352,
		Longitude: -97.6724014282,
		Timezone:  "America/Chicago",
	},
	"K51A": {
		ICAO:      "K51A",
		Name:      "Hawkinsville Pulaski County Airport",
		City:      "Hawkinsville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 270,
		Latitude:  32.2835006714,
		Longitude: -83.4384994507,
		Timezone:  "America/New_York",
	},
	"K51D": {
		ICAO:      "K51D",
		Name:      "Edgeley Municipal Airport",
		City:      "Edgeley",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1601,
		Latitude:  46.34859848,
		Longitude: -98.73560333,
		Timezone:  "America/Chicago",
	},
	"K51J": {
		ICAO:      "K51J",
		Name:      "Lake City Municipal CJ Evans Field",
		City:      "Lake City",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 80,
		Latitude:  33.8535995483,
		Longitude: -79.7680969238,
		Timezone:  "America/New_York",
	},
	"K51R": {
		ICAO:      "K51R",
		Name:      "Madisonville Municipal Airport",
		City:      "Madisonville",
		State:     "Texas",
		Country:   "US",
		Elevation: 287,
		Latitude:  30.9127998352,
		Longitude: -95.952003479,
		Timezone:  "America/Chicago",
	},
	"K52F": {
		ICAO:      "K52F",
		Name:      "Northwest Regional Airport",
		City:      "Roanoke",
		State:     "Texas",
		Country:   "US",
		Elevation: 643,
		Latitude:  33.0498008728,
		Longitude: -97.2322006226,
		Timezone:  "America/Chicago",
	},
	"K52J": {
		ICAO:      "K52J",
		Name:      "Lee County Butters Field",
		City:      "Bishopville",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 211,
		Latitude:  34.2444992065,
		Longitude: -80.236000061,
		Timezone:  "America/New_York",
	},
	"K53A": {
		ICAO:      "K53A",
		Name:      "Dr. C P Savage Sr. Airport",
		City:      "Montezuma",
		State:     "Georgia",
		Country:   "US",
		Elevation: 337,
		Latitude:  32.3019981384,
		Longitude: -84.0074996948,
		Timezone:  "America/New_York",
	},
	"K54F": {
		ICAO:      "K54F",
		Name:      "Robert Lee Airport",
		City:      "Robert Lee",
		State:     "Texas",
		Country:   "US",
		Elevation: 1922,
		Latitude:  31.8817996979,
		Longitude: -100.5400009155,
		Timezone:  "America/Chicago",
	},
	"K54J": {
		ICAO:      "K54J",
		Name:      "Defuniak Springs Airport",
		City:      "Defuniak Springs",
		State:     "Florida",
		Country:   "US",
		Elevation: 289,
		Latitude:  30.731300354,
		Longitude: -86.1516036987,
		Timezone:  "America/Chicago",
	},
	"K54T": {
		ICAO:      "K54T",
		Name:      "RWJ Airpark",
		City:      "Baytown",
		State:     "Texas",
		Country:   "US",
		Elevation: 30,
		Latitude:  29.7616996765,
		Longitude: -94.846496582,
		Timezone:  "America/Chicago",
	},
	"K55K": {
		ICAO:      "K55K",
		Name:      "Oxford Municipal Airport",
		City:      "Oxford",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1189,
		Latitude:  37.2696990967,
		Longitude: -97.0914001465,
		Timezone:  "America/Chicago",
	},
	"K55M": {
		ICAO:      "K55M",
		Name:      "Star City Municipal Airport",
		City:      "Star City",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 398,
		Latitude:  33.9261016846,
		Longitude: -91.8774032593,
		Timezone:  "America/Chicago",
	},
	"K55Y": {
		ICAO:      "K55Y",
		Name:      "Rushford Municipal Airport",
		City:      "Rushford",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1211,
		Latitude:  43.8157997131,
		Longitude: -91.8301010132,
		Timezone:  "America/Chicago",
	},
	"K56D": {
		ICAO:      "K56D",
		Name:      "Wyandot County Airport",
		City:      "Upper Sandusky",
		State:     "Ohio",
		Country:   "US",
		Elevation: 830,
		Latitude:  40.8833999634,
		Longitude: -83.3144989014,
		Timezone:  "America/New_York",
	},
	"K56F": {
		ICAO:      "K56F",
		Name:      "Fisher County Airport",
		City:      "Rotan/Roby",
		State:     "Texas",
		Country:   "US",
		Elevation: 1941,
		Latitude:  32.8256988525,
		Longitude: -100.4150009155,
		Timezone:  "America/Chicago",
	},
	"K57C": {
		ICAO:      "K57C",
		Name:      "East Troy Municipal Airport",
		City:      "East Troy",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 860,
		Latitude:  42.7971992493,
		Longitude: -88.3725967407,
		Timezone:  "America/Chicago",
	},
	"K57S": {
		ICAO:      "K57S",
		Name:      "Troy Airport",
		City:      "Troy",
		State:     "Montana",
		Country:   "US",
		Elevation: 2017,
		Latitude:  48.4802017212,
		Longitude: -115.9039993286,
		Timezone:  "America/Denver",
	},
	"K58M": {
		ICAO:      "K58M",
		Name:      "Cecil County Airport",
		City:      "Elkton",
		State:     "Maryland",
		Country:   "US",
		Elevation: 106,
		Latitude:  39.5741996765,
		Longitude: -75.8697967529,
		Timezone:  "America/New_York",
	},
	"K5A1": {
		ICAO:      "K5A1",
		Name:      "Norwalk Huron County Airport",
		City:      "Norwalk",
		State:     "Ohio",
		Country:   "US",
		Elevation: 852,
		Latitude:  41.2448005676,
		Longitude: -82.5512008667,
		Timezone:  "America/New_York",
	},
	"K5A4": {
		ICAO:      "K5A4",
		Name:      "Okolona Municipal-Richard Stovall Field",
		City:      "Okolona",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 335,
		Latitude:  34.0158004761,
		Longitude: -88.7261962891,
		Timezone:  "America/Chicago",
	},
	"K5A6": {
		ICAO:      "K5A6",
		Name:      "Winona Montgomery County Airport",
		City:      "Winona",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 364,
		Latitude:  33.4654007,
		Longitude: -89.72920227,
		Timezone:  "America/Chicago",
	},
	"K5A9": {
		ICAO:      "K5A9",
		Name:      "Roosevelt Memorial Airport",
		City:      "Warm Springs",
		State:     "Georgia",
		Country:   "US",
		Elevation: 880,
		Latitude:  32.9334983826,
		Longitude: -84.6887969971,
		Timezone:  "America/New_York",
	},
	"K5B2": {
		ICAO:      "K5B2",
		Name:      "Saratoga County Airport",
		City:      "Saratoga Springs",
		State:     "New-York",
		Country:   "US",
		Elevation: 434,
		Latitude:  43.05130005,
		Longitude: -73.86119843,
		Timezone:  "America/New_York",
	},
	"K5B5": {
		ICAO:      "K5B5",
		Name:      "Napoleon Municipal Airport",
		City:      "Napoleon",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1983,
		Latitude:  46.4944000244,
		Longitude: -99.7601013184,
		Timezone:  "America/Chicago",
	},
	"K5C1": {
		ICAO:      "K5C1",
		Name:      "Boerne Stage Field",
		City:      "San Antonio",
		State:     "Texas",
		Country:   "US",
		Elevation: 1385,
		Latitude:  29.7238998413,
		Longitude: -98.6946029663,
		Timezone:  "America/Chicago",
	},
	"K5C3": {
		ICAO:      "K5C3",
		Name:      "Nary National Shefland Field",
		City:      "Nary",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1389,
		Latitude:  47.3760986328,
		Longitude: -94.797996521,
		Timezone:  "America/Chicago",
	},
	"K5C8": {
		ICAO:      "K5C8",
		Name:      "Washburn Municipal Airport",
		City:      "Washburn",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1905,
		Latitude:  47.353099823,
		Longitude: -101.0270004272,
		Timezone:  "America/Chicago",
	},
	"K5F0": {
		ICAO:      "K5F0",
		Name:      "Arcadia Bienville Parish Airport",
		City:      "Arcadia",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 440,
		Latitude:  32.5307006836,
		Longitude: -92.9540023804,
		Timezone:  "America/Chicago",
	},
	"K5F1": {
		ICAO:      "K5F1",
		Name:      "Post Garza County Municipal Airport",
		City:      "Post",
		State:     "Texas",
		Country:   "US",
		Elevation: 2545,
		Latitude:  33.2037010193,
		Longitude: -101.3399963379,
		Timezone:  "America/Chicago",
	},
	"K5F4": {
		ICAO:      "K5F4",
		Name:      "Homer Municipal Airport",
		City:      "Homer",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 244,
		Latitude:  32.7885017395,
		Longitude: -93.0037002563,
		Timezone:  "America/Chicago",
	},
	"K5G6": {
		ICAO:      "K5G6",
		Name:      "Cherry Springs Airport",
		City:      "Galeton",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 2330,
		Latitude:  41.6646995544,
		Longitude: -77.8182983398,
		Timezone:  "America/New_York",
	},
	"K5G7": {
		ICAO:      "K5G7",
		Name:      "Bluffton Airport",
		City:      "Bluffton",
		State:     "Ohio",
		Country:   "US",
		Elevation: 851,
		Latitude:  40.8853988647,
		Longitude: -83.868598938,
		Timezone:  "America/New_York",
	},
	"K5H4": {
		ICAO:      "K5H4",
		Name:      "Harvey Municipal Airport",
		City:      "Harvey",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1607,
		Latitude:  47.79119873,
		Longitude: -99.93170166,
		Timezone:  "America/Chicago",
	},
	"K5I4": {
		ICAO:      "K5I4",
		Name:      "Sheridan Airport",
		City:      "Sheridan",
		State:     "Indiana",
		Country:   "US",
		Elevation: 936,
		Latitude:  40.177898407,
		Longitude: -86.217300415,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"K5J9": {
		ICAO:      "K5J9",
		Name:      "Twin City Airport",
		City:      "Loris",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 99,
		Latitude:  34.0884017944,
		Longitude: -78.8648986816,
		Timezone:  "America/New_York",
	},
	"K5K1": {
		ICAO:      "K5K1",
		Name:      "Zelmer Memorial Airpark Inc Airport",
		City:      "Palmyra",
		State:     "Illinois",
		Country:   "US",
		Elevation: 663,
		Latitude:  39.4199981689,
		Longitude: -89.9912033081,
		Timezone:  "America/Chicago",
	},
	"K5L0": {
		ICAO:      "K5L0",
		Name:      "Lakota Municipal Airport",
		City:      "Lakota",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1512,
		Latitude:  48.0293998718,
		Longitude: -98.325302124,
		Timezone:  "America/Chicago",
	},
	"K5M3": {
		ICAO:      "K5M3",
		Name:      "Moultonboro Airport",
		City:      "Moultonboro",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 571,
		Latitude:  43.7683982849,
		Longitude: -71.3867034912,
		Timezone:  "America/New_York",
	},
	"K5M4": {
		ICAO:      "K5M4",
		Name:      "Fordyce Municipal Airport",
		City:      "Fordyce",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 193,
		Latitude:  33.8457984924,
		Longitude: -92.3655014038,
		Timezone:  "America/Chicago",
	},
	"K5M5": {
		ICAO:      "K5M5",
		Name:      "Crystal Lake Airport",
		City:      "Decatur",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1180,
		Latitude:  36.3436012268,
		Longitude: -94.4448013306,
		Timezone:  "America/Chicago",
	},
	"K5M8": {
		ICAO:      "K5M8",
		Name:      "Gurdon Lowe Field",
		City:      "Gurdon",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 229,
		Latitude:  33.9239006042,
		Longitude: -93.1687011719,
		Timezone:  "America/Chicago",
	},
	"K5M9": {
		ICAO:      "K5M9",
		Name:      "Marion Crittenden County Airport",
		City:      "Marion",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 650,
		Latitude:  37.33430099,
		Longitude: -88.11509705,
		Timezone:  "America/Chicago",
	},
	"K5N2": {
		ICAO:      "K5N2",
		Name:      "Prentice Airport",
		City:      "Prentice",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1578,
		Latitude:  45.5429992676,
		Longitude: -90.279296875,
		Timezone:  "America/Chicago",
	},
	"K5N8": {
		ICAO:      "K5N8",
		Name:      "Casselton Robert Miller Regional Airport",
		City:      "Casselton",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 933,
		Latitude:  46.8540000916,
		Longitude: -97.2087020874,
		Timezone:  "America/Chicago",
	},
	"K5P2": {
		ICAO:      "K5P2",
		Name:      "Mc Laughlin Municipal Airport",
		City:      "Mc Laughlin",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 2006,
		Latitude:  45.7967987061,
		Longitude: -100.783996582,
		Timezone:  "America/Denver",
	},
	"K5R1": {
		ICAO:      "K5R1",
		Name:      "Roy Wilcox Airport",
		City:      "Chatom",
		State:     "Alabama",
		Country:   "US",
		Elevation: 165,
		Latitude:  31.4517993927,
		Longitude: -88.1945037842,
		Timezone:  "America/Chicago",
	},
	"K5R2": {
		ICAO:      "K5R2",
		Name:      "Ocean Springs Airport",
		City:      "Ocean Springs",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 20,
		Latitude:  30.3894004822,
		Longitude: -88.75340271,
		Timezone:  "America/Chicago",
	},
	"K5R4": {
		ICAO:      "K5R4",
		Name:      "Foley Municipal Airport",
		City:      "Foley",
		State:     "Alabama",
		Country:   "US",
		Elevation: 74,
		Latitude:  30.4277000427,
		Longitude: -87.7007980347,
		Timezone:  "America/Chicago",
	},
	"K5R8": {
		ICAO:      "K5R8",
		Name:      "De Quincy Industrial Airpark",
		City:      "De Quincy",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 81,
		Latitude:  30.4412002563,
		Longitude: -93.4735031128,
		Timezone:  "America/Chicago",
	},
	"K5S0": {
		ICAO:      "K5S0",
		Name:      "Oakridge State Airport",
		City:      "Oakridge",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1393,
		Latitude:  43.7526016235,
		Longitude: -122.5029983521,
		Timezone:  "America/Los_Angeles",
	},
	"K5S2": {
		ICAO:      "K5S2",
		Name:      "Crescent Lake State Airport",
		City:      "Crescent Lake",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4810,
		Latitude:  43.5326004028,
		Longitude: -121.9499969482,
		Timezone:  "America/Los_Angeles",
	},
	"K5S6": {
		ICAO:      "K5S6",
		Name:      "Cape Blanco State Airport",
		City:      "Denmark",
		State:     "Oregon",
		Country:   "US",
		Elevation: 214,
		Latitude:  42.8578987122,
		Longitude: -124.5179977417,
		Timezone:  "America/Los_Angeles",
	},
	"K5S9": {
		ICAO:      "K5S9",
		Name:      "Valley View Airport",
		City:      "Estacada",
		State:     "Oregon",
		Country:   "US",
		Elevation: 735,
		Latitude:  45.3082008362,
		Longitude: -122.3190002441,
		Timezone:  "America/Los_Angeles",
	},
	"K5T6": {
		ICAO:      "K5T6",
		Name:      "Dona Ana County At Santa Teresa Airport",
		City:      "Santa Teresa",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4112,
		Latitude:  31.8810005188,
		Longitude: -106.7050018311,
		Timezone:  "America/Denver",
	},
	"K5T9": {
		ICAO:      "K5T9",
		Name:      "Maverick County Memorial International Airport",
		City:      "Eagle Pass",
		State:     "Texas",
		Country:   "US",
		Elevation: 887,
		Latitude:  28.85720062,
		Longitude: -100.512001,
		Timezone:  "America/Chicago",
	},
	"K5U1": {
		ICAO:      "K5U1",
		Name:      "Dutton Airport",
		City:      "Dutton",
		State:     "Montana",
		Country:   "US",
		Elevation: 3699,
		Latitude:  47.8471984863,
		Longitude: -111.6979980469,
		Timezone:  "America/Denver",
	},
	"K5U5": {
		ICAO:      "K5U5",
		Name:      "Fairfield Airport",
		City:      "Fairfield",
		State:     "Montana",
		Country:   "US",
		Elevation: 3989,
		Latitude:  47.6291007996,
		Longitude: -111.9800033569,
		Timezone:  "America/Denver",
	},
	"K5U7": {
		ICAO:      "K5U7",
		Name:      "Fort Smith Landing Strip",
		City:      "Fort Smith",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 3242,
		Latitude:  45.3208007813,
		Longitude: -107.9309997559,
		Timezone:  "America/Denver",
	},
	"K5V5": {
		ICAO:      "K5V5",
		Name:      "Shiprock Airstrip",
		City:      "Shiprock",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 5270,
		Latitude:  36.6977996826,
		Longitude: -108.7009963989,
		Timezone:  "America/Denver",
	},
	"K5W1": {
		ICAO:      "K5W1",
		Name:      "Wilson Creek Airport",
		City:      "Wilson Creek",
		State:     "Washington",
		Country:   "US",
		Elevation: 1440,
		Latitude:  47.4249000549,
		Longitude: -119.1149978638,
		Timezone:  "America/Los_Angeles",
	},
	"K5W4": {
		ICAO:      "K5W4",
		Name:      "P K Airpark",
		City:      "Raeford",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 304,
		Latitude:  35.0199012756,
		Longitude: -79.1910018921,
		Timezone:  "America/New_York",
	},
	"K5W5": {
		ICAO:      "K5W5",
		Name:      "Triple W Airport",
		City:      "Raleigh",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 244,
		Latitude:  35.620300293,
		Longitude: -78.700302124,
		Timezone:  "America/New_York",
	},
	"K5W8": {
		ICAO:      "K5W8",
		Name:      "Siler City Municipal Airport",
		City:      "Siler City",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 614,
		Latitude:  35.7042999268,
		Longitude: -79.5042037964,
		Timezone:  "America/New_York",
	},
	"K5Y1": {
		ICAO:      "K5Y1",
		Name:      "Albert J Lindberg Airport",
		City:      "Hessel",
		State:     "Michigan",
		Country:   "US",
		Elevation: 760,
		Latitude:  46.035900116,
		Longitude: -84.4197998047,
		Timezone:  "America/Detroit",
	},
	"K60C": {
		ICAO:      "K60C",
		Name:      "Elroy Municipal Airport",
		City:      "Elroy",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 944,
		Latitude:  43.706401825,
		Longitude: -90.257598877,
		Timezone:  "America/Chicago",
	},
	"K60F": {
		ICAO:      "K60F",
		Name:      "Seymour Municipal Airport",
		City:      "Seymour",
		State:     "Texas",
		Country:   "US",
		Elevation: 1344,
		Latitude:  33.6487007141,
		Longitude: -99.260597229,
		Timezone:  "America/Chicago",
	},
	"K60J": {
		ICAO:      "K60J",
		Name:      "Odell Williamson Municipal Airport",
		City:      "Ocean Isle Beach",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 32,
		Latitude:  33.9085006714,
		Longitude: -78.4366989136,
		Timezone:  "America/New_York",
	},
	"K60R": {
		ICAO:      "K60R",
		Name:      "Navasota Municipal Airport",
		City:      "Navasota",
		State:     "Texas",
		Country:   "US",
		Elevation: 228,
		Latitude:  30.3719005585,
		Longitude: -96.1132965088,
		Timezone:  "America/Chicago",
	},
	"K61A": {
		ICAO:      "K61A",
		Name:      "Camden Municipal Airport",
		City:      "Camden",
		State:     "Alabama",
		Country:   "US",
		Elevation: 143,
		Latitude:  31.9797000885,
		Longitude: -87.3391036987,
		Timezone:  "America/Chicago",
	},
	"K61C": {
		ICAO:      "K61C",
		Name:      "Fort Atkinson Municipal Airport",
		City:      "Fort Atkinson",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 800,
		Latitude:  42.9631996155,
		Longitude: -88.8175964355,
		Timezone:  "America/Chicago",
	},
	"K61R": {
		ICAO:      "K61R",
		Name:      "Newton Municipal Airport",
		City:      "Newton",
		State:     "Texas",
		Country:   "US",
		Elevation: 320,
		Latitude:  30.8843994141,
		Longitude: -93.7417984009,
		Timezone:  "America/Chicago",
	},
	"K61S": {
		ICAO:      "K61S",
		Name:      "Cottage Grove State Airport",
		City:      "Cottage Grove",
		State:     "Oregon",
		Country:   "US",
		Elevation: 641,
		Latitude:  43.7998008728,
		Longitude: -123.0289993286,
		Timezone:  "America/Los_Angeles",
	},
	"K62D": {
		ICAO:      "K62D",
		Name:      "Warren Airport",
		City:      "Warren",
		State:     "Ohio",
		Country:   "US",
		Elevation: 905,
		Latitude:  41.266998291,
		Longitude: -80.9273986816,
		Timezone:  "America/New_York",
	},
	"K62S": {
		ICAO:      "K62S",
		Name:      "Christmas Valley Airport",
		City:      "Christmas Valley",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4317,
		Latitude:  43.2364997864,
		Longitude: -120.6660003662,
		Timezone:  "America/Los_Angeles",
	},
	"K63B": {
		ICAO:      "K63B",
		Name:      "Limington Harmon Airport",
		City:      "Limington",
		State:     "Maine",
		Country:   "US",
		Elevation: 291,
		Latitude:  43.7630004883,
		Longitude: -70.6725006104,
		Timezone:  "America/New_York",
	},
	"K63C": {
		ICAO:      "K63C",
		Name:      "Adams County Legion Field",
		City:      "Friendship (Adams)",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 976,
		Latitude:  43.9612007141,
		Longitude: -89.7880020142,
		Timezone:  "America/Chicago",
	},
	"K63F": {
		ICAO:      "K63F",
		Name:      "Stanton Municipal Airport",
		City:      "Stanton",
		State:     "Texas",
		Country:   "US",
		Elevation: 2731,
		Latitude:  32.1735992432,
		Longitude: -101.8219985962,
		Timezone:  "America/Chicago",
	},
	"K63M": {
		ICAO:      "K63M",
		Name:      "Hermann Municipal Airport",
		City:      "Hermann",
		State:     "Missouri",
		Country:   "US",
		Elevation: 507,
		Latitude:  38.7052001953,
		Longitude: -91.4906005859,
		Timezone:  "America/Chicago",
	},
	"K64S": {
		ICAO:      "K64S",
		Name:      "Prospect State Airport",
		City:      "Prospect",
		State:     "Oregon",
		Country:   "US",
		Elevation: 2578,
		Latitude:  42.7431983948,
		Longitude: -122.4879989624,
		Timezone:  "America/Los_Angeles",
	},
	"K65J": {
		ICAO:      "K65J",
		Name:      "Wrens Memorial Airport",
		City:      "Wrens",
		State:     "Georgia",
		Country:   "US",
		Elevation: 450,
		Latitude:  33.2225990295,
		Longitude: -82.3836975098,
		Timezone:  "America/New_York",
	},
	"K65S": {
		ICAO:      "K65S",
		Name:      "Boundary County Airport",
		City:      "Bonners Ferry",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2337,
		Latitude:  48.72579956,
		Longitude: -116.2949982,
		Timezone:  "America/Los_Angeles",
	},
	"K66R": {
		ICAO:      "K66R",
		Name:      "Robert R Wells Jr Airport",
		City:      "Columbus",
		State:     "Texas",
		Country:   "US",
		Elevation: 242,
		Latitude:  29.6410999298,
		Longitude: -96.5158004761,
		Timezone:  "America/Chicago",
	},
	"K66V": {
		ICAO:      "K66V",
		Name:      "Bluff Airport",
		City:      "Bluff",
		State:     "Utah",
		Country:   "US",
		Elevation: 4476,
		Latitude:  37.25,
		Longitude: -109.6340026855,
		Timezone:  "America/Denver",
	},
	"K66Y": {
		ICAO:      "K66Y",
		Name:      "Diamondhead Airport",
		City:      "Diamondhead",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 14,
		Latitude:  30.3630008698,
		Longitude: -89.3877029419,
		Timezone:  "America/Chicago",
	},
	"K67A": {
		ICAO:      "K67A",
		Name:      "Fort Deposit Lowndes County Airport",
		City:      "Fort Deposit",
		State:     "Alabama",
		Country:   "US",
		Elevation: 490,
		Latitude:  31.9724006653,
		Longitude: -86.5915985107,
		Timezone:  "America/Chicago",
	},
	"K67L": {
		ICAO:      "K67L",
		Name:      "Mesquite Airport",
		City:      "Mesquite",
		State:     "Nevada",
		Country:   "US",
		Elevation: 1978,
		Latitude:  36.83499908,
		Longitude: -114.0550003,
		Timezone:  "America/Los_Angeles",
	},
	"K67R": {
		ICAO:      "K67R",
		Name:      "Rio Grande City Municipal Airport",
		City:      "Rio Grande City",
		State:     "Texas",
		Country:   "US",
		Elevation: 290,
		Latitude:  26.4242992401,
		Longitude: -98.8460998535,
		Timezone:  "America/Chicago",
	},
	"K68F": {
		ICAO:      "K68F",
		Name:      "Teague Municipal Airport",
		City:      "Teague",
		State:     "Texas",
		Country:   "US",
		Elevation: 525,
		Latitude:  31.6613006592,
		Longitude: -96.3099975586,
		Timezone:  "America/Chicago",
	},
	"K68J": {
		ICAO:      "K68J",
		Name:      "Tallahassee Commercial Airport",
		City:      "Tallahassee /Havana/",
		State:     "Florida",
		Country:   "US",
		Elevation: 155,
		Latitude:  30.5473003387,
		Longitude: -84.3738021851,
		Timezone:  "America/New_York",
	},
	"K68S": {
		ICAO:      "K68S",
		Name:      "Davenport Airport",
		City:      "Davenport",
		State:     "Washington",
		Country:   "US",
		Elevation: 2421,
		Latitude:  47.6535987854,
		Longitude: -118.1679992676,
		Timezone:  "America/Los_Angeles",
	},
	"K69K": {
		ICAO:      "K69K",
		Name:      "Wamego Municipal Airport",
		City:      "Wamego",
		State:     "Kansas",
		Country:   "US",
		Elevation: 966,
		Latitude:  39.1972007751,
		Longitude: -96.2589035034,
		Timezone:  "America/Chicago",
	},
	"K69V": {
		ICAO:      "K69V",
		Name:      "Huntington Municipal Airport",
		City:      "Huntington",
		State:     "Utah",
		Country:   "US",
		Elevation: 5915,
		Latitude:  39.3611984253,
		Longitude: -110.9169998169,
		Timezone:  "America/Denver",
	},
	"K6A1": {
		ICAO:      "K6A1",
		Name:      "Butler Municipal Airport",
		City:      "Butler",
		State:     "Georgia",
		Country:   "US",
		Elevation: 667,
		Latitude:  32.5676002502,
		Longitude: -84.2507019043,
		Timezone:  "America/New_York",
	},
	"K6A2": {
		ICAO:      "K6A2",
		Name:      "Griffin Spalding County Airport",
		City:      "Griffin",
		State:     "Georgia",
		Country:   "US",
		Elevation: 958,
		Latitude:  33.2270011902,
		Longitude: -84.2749023438,
		Timezone:  "America/New_York",
	},
	"K6A4": {
		ICAO:      "K6A4",
		Name:      "Johnson County Airport",
		City:      "Mountain City",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 2240,
		Latitude:  36.4178009033,
		Longitude: -81.8251037598,
		Timezone:  "America/New_York",
	},
	"K6B9": {
		ICAO:      "K6B9",
		Name:      "Skaneateles Aero Drome Airport",
		City:      "Skaneateles",
		State:     "New-York",
		Country:   "US",
		Elevation: 1038,
		Latitude:  42.9140014648,
		Longitude: -76.4408035278,
		Timezone:  "America/New_York",
	},
	"K6D1": {
		ICAO:      "K6D1",
		Name:      "Brooten Municipal Airport",
		City:      "Brooten",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1305,
		Latitude:  45.5,
		Longitude: -95.1128005981,
		Timezone:  "America/Chicago",
	},
	"K6D6": {
		ICAO:      "K6D6",
		Name:      "Greenville Municipal Airport",
		City:      "Greenville",
		State:     "Michigan",
		Country:   "US",
		Elevation: 855,
		Latitude:  43.1422996521,
		Longitude: -85.2537994385,
		Timezone:  "America/Detroit",
	},
	"K6D8": {
		ICAO:      "K6D8",
		Name:      "Barnes County Municipal Airport",
		City:      "Valley City",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1402,
		Latitude:  46.94100189,
		Longitude: -98.01760101,
		Timezone:  "America/Chicago",
	},
	"K6D9": {
		ICAO:      "K6D9",
		Name:      "Iosco County Airport",
		City:      "East Tawas",
		State:     "Michigan",
		Country:   "US",
		Elevation: 606,
		Latitude:  44.3128013611,
		Longitude: -83.4223022461,
		Timezone:  "America/Detroit",
	},
	"K6E5": {
		ICAO:      "K6E5",
		Name:      "Wilder Airport",
		City:      "Desmet",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1729,
		Latitude:  44.4308013916,
		Longitude: -97.5612030029,
		Timezone:  "America/Chicago",
	},
	"K6G0": {
		ICAO:      "K6G0",
		Name:      "Athelone Williams Memorial Airport",
		City:      "Davison",
		State:     "Michigan",
		Country:   "US",
		Elevation: 780,
		Latitude:  43.0290985107,
		Longitude: -83.5297012329,
		Timezone:  "America/Detroit",
	},
	"K6G1": {
		ICAO:      "K6G1",
		Name:      "Titusville Airport",
		City:      "Titusville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1600,
		Latitude:  41.6087989807,
		Longitude: -79.7413024902,
		Timezone:  "America/New_York",
	},
	"K6G4": {
		ICAO:      "K6G4",
		Name:      "Wynkoop Airport",
		City:      "Mount Vernon",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1041,
		Latitude:  40.3651008606,
		Longitude: -82.4956970215,
		Timezone:  "America/New_York",
	},
	"K6G5": {
		ICAO:      "K6G5",
		Name:      "Barnesville Bradfield Airport",
		City:      "Barnesville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1312,
		Latitude:  40.0023994446,
		Longitude: -81.1918029785,
		Timezone:  "America/New_York",
	},
	"K6I2": {
		ICAO:      "K6I2",
		Name:      "Lebanon Springfield Airport",
		City:      "Springfield",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 866,
		Latitude:  37.6334991455,
		Longitude: -85.2417984009,
		Timezone:  "America/New_York",
	},
	"K6I4": {
		ICAO:      "K6I4",
		Name:      "Boone County Airport",
		City:      "Lebanon",
		State:     "Indiana",
		Country:   "US",
		Elevation: 959,
		Latitude:  40.0073013306,
		Longitude: -86.4405975342,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"K6I6": {
		ICAO:      "K6I6",
		Name:      "Darby Dan Airport",
		City:      "Columbus",
		State:     "Ohio",
		Country:   "US",
		Elevation: 928,
		Latitude:  39.9420013428,
		Longitude: -83.2049026489,
		Timezone:  "America/New_York",
	},
	"K6J0": {
		ICAO:      "K6J0",
		Name:      "Lexington County At Pelion Airport",
		City:      "Pelion",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 452,
		Latitude:  33.7946014404,
		Longitude: -81.2459030151,
		Timezone:  "America/New_York",
	},
	"K6J2": {
		ICAO:      "K6J2",
		Name:      "St George Airport",
		City:      "St George",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 85,
		Latitude:  33.19549942,
		Longitude: -80.50849915,
		Timezone:  "America/New_York",
	},
	"K6J4": {
		ICAO:      "K6J4",
		Name:      "Saluda County Airport",
		City:      "Saluda",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 555,
		Latitude:  33.9267997742,
		Longitude: -81.7946014404,
		Timezone:  "America/New_York",
	},
	"K6J7": {
		ICAO:      "K6J7",
		Name:      "Branhams Airport",
		City:      "Darlington",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 165,
		Latitude:  34.2821006775,
		Longitude: -79.9287033081,
		Timezone:  "America/New_York",
	},
	"K6K3": {
		ICAO:      "K6K3",
		Name:      "Creighton Municipal Airport",
		City:      "Creighton",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1654,
		Latitude:  42.47079849,
		Longitude: -97.88369751,
		Timezone:  "America/Chicago",
	},
	"K6K4": {
		ICAO:      "K6K4",
		Name:      "Fairview Municipal Airport",
		City:      "Fairview",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1272,
		Latitude:  36.2901000977,
		Longitude: -98.4757995605,
		Timezone:  "America/Chicago",
	},
	"K6K5": {
		ICAO:      "K6K5",
		Name:      "Sisters Eagle Air Airport",
		City:      "Sisters",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3168,
		Latitude:  44.304599762,
		Longitude: -121.5390014648,
		Timezone:  "America/Los_Angeles",
	},
	"K6L3": {
		ICAO:      "K6L3",
		Name:      "Lisbon Municipal Airport",
		City:      "Lisbon",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1232,
		Latitude:  46.44660187,
		Longitude: -97.72730255,
		Timezone:  "America/Chicago",
	},
	"K6L5": {
		ICAO:      "K6L5",
		Name:      "Wishek Municipal Airport",
		City:      "Wishek",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 2035,
		Latitude:  46.2463989258,
		Longitude: -99.537902832,
		Timezone:  "America/Chicago",
	},
	"K6M0": {
		ICAO:      "K6M0",
		Name:      "Hazen Municipal Airport",
		City:      "Hazen",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 230,
		Latitude:  34.7593994141,
		Longitude: -91.6380996704,
		Timezone:  "America/Chicago",
	},
	"K6M2": {
		ICAO:      "K6M2",
		Name:      "Horseshoe Bend Airport",
		City:      "Horseshoe Bend",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 782,
		Latitude:  36.2214012146,
		Longitude: -91.7555007935,
		Timezone:  "America/Chicago",
	},
	"K6M6": {
		ICAO:      "K6M6",
		Name:      "Lewis County Regional Airport",
		City:      "Monticello",
		State:     "Missouri",
		Country:   "US",
		Elevation: 675,
		Latitude:  40.12919998,
		Longitude: -91.67829895,
		Timezone:  "America/Chicago",
	},
	"K6M7": {
		ICAO:      "K6M7",
		Name:      "Marianna Lee County Steve Edwards Field",
		City:      "Marianna",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 219,
		Latitude:  34.78030014,
		Longitude: -90.81060028,
		Timezone:  "America/Chicago",
	},
	"K6R3": {
		ICAO:      "K6R3",
		Name:      "Cleveland Municipal Airport",
		City:      "Cleveland",
		State:     "Texas",
		Country:   "US",
		Elevation: 150,
		Latitude:  30.3563995361,
		Longitude: -95.0080032349,
		Timezone:  "America/Chicago",
	},
	"K6R6": {
		ICAO:      "K6R6",
		Name:      "Terrell County Airport",
		City:      "Dryden",
		State:     "Texas",
		Country:   "US",
		Elevation: 2322,
		Latitude:  30.045999527,
		Longitude: -102.2129974365,
		Timezone:  "America/Chicago",
	},
	"K6S0": {
		ICAO:      "K6S0",
		Name:      "Big Timber Airport",
		City:      "Big Timber",
		State:     "Montana",
		Country:   "US",
		Elevation: 4492,
		Latitude:  45.8064002991,
		Longitude: -109.9810028076,
		Timezone:  "America/Denver",
	},
	"K6S1": {
		ICAO:      "K6S1",
		Name:      "Bridger Municipal Airport",
		City:      "Bridger",
		State:     "Montana",
		Country:   "US",
		Elevation: 3720,
		Latitude:  45.2915992737,
		Longitude: -108.9260025024,
		Timezone:  "America/Denver",
	},
	"K6S2": {
		ICAO:      "K6S2",
		IATA:      "FMU",
		Name:      "Florence Municipal Airport",
		City:      "Florence",
		State:     "Oregon",
		Country:   "US",
		Elevation: 51,
		Latitude:  43.98279953,
		Longitude: -124.111000061,
		Timezone:  "America/Los_Angeles",
	},
	"K6S3": {
		ICAO:      "K6S3",
		Name:      "Woltermann Memorial Airport",
		City:      "Columbus",
		State:     "Montana",
		Country:   "US",
		Elevation: 3575,
		Latitude:  45.6305007935,
		Longitude: -109.238998413,
		Timezone:  "America/Denver",
	},
	"K6S5": {
		ICAO:      "K6S5",
		Name:      "Ravalli County Airport",
		City:      "Hamilton",
		State:     "Montana",
		Country:   "US",
		Elevation: 3642,
		Latitude:  46.251499176,
		Longitude: -114.1259994507,
		Timezone:  "America/Denver",
	},
	"K6S8": {
		ICAO:      "K6S8",
		Name:      "Laurel Municipal Airport",
		City:      "Laurel",
		State:     "Montana",
		Country:   "US",
		Elevation: 3517,
		Latitude:  45.7030982971,
		Longitude: -108.7610015869,
		Timezone:  "America/Denver",
	},
	"K6U7": {
		ICAO:      "K6U7",
		Name:      "Hysham Airport",
		City:      "Hysham",
		State:     "Montana",
		Country:   "US",
		Elevation: 2624,
		Latitude:  46.2893981934,
		Longitude: -107.1959991455,
		Timezone:  "America/Denver",
	},
	"K6V4": {
		ICAO:      "K6V4",
		Name:      "Wall Municipal Airport",
		City:      "Wall",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 2813,
		Latitude:  43.9995002747,
		Longitude: -102.2549972534,
		Timezone:  "America/Denver",
	},
	"K6V5": {
		ICAO:      "K6V5",
		Name:      "Bison Municipal Airport",
		City:      "Bison",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 2785,
		Latitude:  45.5186004639,
		Longitude: -102.4670028687,
		Timezone:  "America/Denver",
	},
	"K6Y1": {
		ICAO:      "K6Y1",
		Name:      "Bois Blanc Airport",
		City:      "Bois Blanc Island",
		State:     "Michigan",
		Country:   "US",
		Elevation: 664,
		Latitude:  45.7663002,
		Longitude: -84.50379944,
		Timezone:  "America/Detroit",
	},
	"K70A": {
		ICAO:      "K70A",
		Name:      "Freddie Jones Field",
		City:      "Linden",
		State:     "Alabama",
		Country:   "US",
		Elevation: 161,
		Latitude:  32.2667999268,
		Longitude: -87.7181015015,
		Timezone:  "America/Chicago",
	},
	"K70J": {
		ICAO:      "K70J",
		Name:      "Cairo Grady County Airport",
		City:      "Cairo",
		State:     "Georgia",
		Country:   "US",
		Elevation: 265,
		Latitude:  30.8880004883,
		Longitude: -84.1547012329,
		Timezone:  "America/New_York",
	},
	"K71A": {
		ICAO:      "K71A",
		Name:      "Pine Hill Municipal Airport",
		City:      "Pine Hill",
		State:     "Alabama",
		Country:   "US",
		Elevation: 123,
		Latitude:  31.9668006897,
		Longitude: -87.5832977295,
		Timezone:  "America/Chicago",
	},
	"K71J": {
		ICAO:      "K71J",
		Name:      "Blackwell Field",
		City:      "Ozark",
		State:     "Alabama",
		Country:   "US",
		Elevation: 356,
		Latitude:  31.4311008453,
		Longitude: -85.6192016602,
		Timezone:  "America/Chicago",
	},
	"K71N": {
		ICAO:      "K71N",
		Name:      "Sunbury Airport",
		City:      "Sunbury",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 450,
		Latitude:  40.8917999268,
		Longitude: -76.7789001465,
		Timezone:  "America/New_York",
	},
	"K72F": {
		ICAO:      "K72F",
		Name:      "Throckmorton Municipal Airport",
		City:      "Throckmorton",
		State:     "Texas",
		Country:   "US",
		Elevation: 1273,
		Latitude:  33.1792984009,
		Longitude: -99.1498031616,
		Timezone:  "America/Chicago",
	},
	"K73C": {
		ICAO:      "K73C",
		Name:      "Lancaster Municipal Airport",
		City:      "Lancaster",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1008,
		Latitude:  42.7804985046,
		Longitude: -90.6809997559,
		Timezone:  "America/Chicago",
	},
	"K74S": {
		ICAO:      "K74S",
		Name:      "Anacortes Airport",
		City:      "Anacortes",
		State:     "Washington",
		Country:   "US",
		Elevation: 241,
		Latitude:  48.4990005493,
		Longitude: -122.6620025635,
		Timezone:  "America/Los_Angeles",
	},
	"K74V": {
		ICAO:      "K74V",
		Name:      "Roosevelt Municipal Airport",
		City:      "Roosevelt",
		State:     "Utah",
		Country:   "US",
		Elevation: 5172,
		Latitude:  40.278301239,
		Longitude: -110.0510025024,
		Timezone:  "America/Denver",
	},
	"K75J": {
		ICAO:      "K75J",
		Name:      "Turner County Airport",
		City:      "Ashburn",
		State:     "Georgia",
		Country:   "US",
		Elevation: 389,
		Latitude:  31.6854991913,
		Longitude: -83.6321029663,
		Timezone:  "America/New_York",
	},
	"K76F": {
		ICAO:      "K76F",
		Name:      "Van Zandt County Regional Airport",
		City:      "Wills Point",
		State:     "Texas",
		Country:   "US",
		Elevation: 518,
		Latitude:  32.6814994812,
		Longitude: -95.9841003418,
		Timezone:  "America/Chicago",
	},
	"K76G": {
		ICAO:      "K76G",
		Name:      "Marine City Airport",
		City:      "Marine City",
		State:     "Michigan",
		Country:   "US",
		Elevation: 613,
		Latitude:  42.7209014893,
		Longitude: -82.595703125,
		Timezone:  "America/Detroit",
	},
	"K77F": {
		ICAO:      "K77F",
		Name:      "Winters Municipal Airport",
		City:      "Winters",
		State:     "Texas",
		Country:   "US",
		Elevation: 1871,
		Latitude:  31.9472007751,
		Longitude: -99.9858016968,
		Timezone:  "America/Chicago",
	},
	"K77G": {
		ICAO:      "K77G",
		Name:      "Marlette Airport",
		City:      "Marlette",
		State:     "Michigan",
		Country:   "US",
		Elevation: 895,
		Latitude:  43.3117980957,
		Longitude: -83.0908966064,
		Timezone:  "America/Detroit",
	},
	"K77S": {
		ICAO:      "K77S",
		Name:      "Hobby Field",
		City:      "Creswell",
		State:     "Oregon",
		Country:   "US",
		Elevation: 538,
		Latitude:  43.9308013916,
		Longitude: -123.0070037842,
		Timezone:  "America/Los_Angeles",
	},
	"K78R": {
		ICAO:      "K78R",
		Name:      "San Augustine County Airport",
		City:      "San Augustine",
		State:     "Texas",
		Country:   "US",
		Elevation: 443,
		Latitude:  31.5396003723,
		Longitude: -94.170501709,
		Timezone:  "America/Chicago",
	},
	"K78Y": {
		ICAO:      "K78Y",
		Name:      "Rankin Airport",
		City:      "Maryville",
		State:     "Missouri",
		Country:   "US",
		Elevation: 976,
		Latitude:  40.3333015442,
		Longitude: -94.8336029053,
		Timezone:  "America/Chicago",
	},
	"K79D": {
		ICAO:      "K79D",
		Name:      "Philippi Barbour County Regional Airport",
		City:      "Philippi",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 1755,
		Latitude:  39.16619873,
		Longitude: -80.06259918,
		Timezone:  "America/New_York",
	},
	"K79J": {
		ICAO:      "K79J",
		Name:      "South Alabama Regional At Bill Benton Field Airport",
		City:      "Andalusia/Opp",
		State:     "Alabama",
		Country:   "US",
		Elevation: 310,
		Latitude:  31.3087997437,
		Longitude: -86.3937988281,
		Timezone:  "America/Chicago",
	},
	"K79S": {
		ICAO:      "K79S",
		Name:      "Fort Benton Airport",
		City:      "Fort Benton",
		State:     "Montana",
		Country:   "US",
		Elevation: 2869,
		Latitude:  47.8451004028,
		Longitude: -110.6360015869,
		Timezone:  "America/Denver",
	},
	"K7A0": {
		ICAO:      "K7A0",
		Name:      "Greensboro Municipal Airport",
		City:      "Greensboro",
		State:     "Alabama",
		Country:   "US",
		Elevation: 180,
		Latitude:  32.6814994812,
		Longitude: -87.6621017456,
		Timezone:  "America/Chicago",
	},
	"K7A3": {
		ICAO:      "K7A3",
		Name:      "Lanett Municipal Airport",
		City:      "Lanett",
		State:     "Alabama",
		Country:   "US",
		Elevation: 624,
		Latitude:  32.8120002747,
		Longitude: -85.229598999,
		Timezone:  "America/Chicago",
	},
	"K7A4": {
		ICAO:      "K7A4",
		Name:      "Foster Field",
		City:      "Apple River",
		State:     "Illinois",
		Country:   "US",
		Elevation: 990,
		Latitude:  42.4664001465,
		Longitude: -90.1694030762,
		Timezone:  "America/Chicago",
	},
	"K7A5": {
		ICAO:      "K7A5",
		Name:      "Roanoke Municipal Airport",
		City:      "Roanoke",
		State:     "Alabama",
		Country:   "US",
		Elevation: 907,
		Latitude:  33.1292991638,
		Longitude: -85.3666000366,
		Timezone:  "America/Chicago",
	},
	"K7A6": {
		ICAO:      "K7A6",
		Name:      "Stevenson Airport",
		City:      "Stevenson",
		State:     "Alabama",
		Country:   "US",
		Elevation: 644,
		Latitude:  34.8862991333,
		Longitude: -85.8032989502,
		Timezone:  "America/Chicago",
	},
	"K7A8": {
		ICAO:      "K7A8",
		Name:      "Avery County Morrison Field",
		City:      "Spruce Pine",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 2750,
		Latitude:  35.9445991516,
		Longitude: -81.9956970215,
		Timezone:  "America/New_York",
	},
	"K7B2": {
		ICAO:      "K7B2",
		Name:      "Northampton Airport",
		City:      "Northampton",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 121,
		Latitude:  42.3280983,
		Longitude: -72.61139679,
		Timezone:  "America/New_York",
	},
	"K7B6": {
		ICAO:      "K7B6",
		Name:      "Skylark Airpark",
		City:      "Warehouse Point",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 120,
		Latitude:  41.9292984009,
		Longitude: -72.5745010376,
		Timezone:  "America/New_York",
	},
	"K7D3": {
		ICAO:      "K7D3",
		Name:      "Baldwin Municipal Airport",
		City:      "Baldwin",
		State:     "Michigan",
		Country:   "US",
		Elevation: 828,
		Latitude:  43.8754997253,
		Longitude: -85.8421020508,
		Timezone:  "America/Detroit",
	},
	"K7D9": {
		ICAO:      "K7D9",
		Name:      "Germack Airport",
		City:      "Geneva",
		State:     "Ohio",
		Country:   "US",
		Elevation: 820,
		Latitude:  41.7778015137,
		Longitude: -80.9039993286,
		Timezone:  "America/New_York",
	},
	"K7F3": {
		ICAO:      "K7F3",
		Name:      "Caddo Mills Municipal Airport",
		City:      "Caddo Mills",
		State:     "Texas",
		Country:   "US",
		Elevation: 542,
		Latitude:  33.0362014771,
		Longitude: -96.2431030273,
		Timezone:  "America/Chicago",
	},
	"K7F5": {
		ICAO:      "K7F5",
		Name:      "Canton Hackney Airport",
		City:      "Canton",
		State:     "Texas",
		Country:   "US",
		Elevation: 500,
		Latitude:  32.5862007141,
		Longitude: -95.862701416,
		Timezone:  "America/Chicago",
	},
	"K7F7": {
		ICAO:      "K7F7",
		Name:      "Clifton Municipal Isenhower Field",
		City:      "Clifton",
		State:     "Texas",
		Country:   "US",
		Elevation: 760,
		Latitude:  31.81679916,
		Longitude: -97.56700134,
		Timezone:  "America/Chicago",
	},
	"K7G0": {
		ICAO:      "K7G0",
		Name:      "Ledgedale Airpark",
		City:      "Brockport",
		State:     "New-York",
		Country:   "US",
		Elevation: 665,
		Latitude:  43.18109894,
		Longitude: -77.915802,
		Timezone:  "America/New_York",
	},
	"K7G8": {
		ICAO:      "K7G8",
		Name:      "Geauga County Airport",
		City:      "Middlefield",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1174,
		Latitude:  41.4496002197,
		Longitude: -81.0628967285,
		Timezone:  "America/New_York",
	},
	"K7G9": {
		ICAO:      "K7G9",
		Name:      "Canton Municipal Airport",
		City:      "Canton",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1290,
		Latitude:  43.3088989258,
		Longitude: -96.5709991455,
		Timezone:  "America/Chicago",
	},
	"K7I4": {
		ICAO:      "K7I4",
		Name:      "Orleans Airport",
		City:      "Orleans",
		State:     "Indiana",
		Country:   "US",
		Elevation: 655,
		Latitude:  38.6584014893,
		Longitude: -86.4430007935,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"K7K4": {
		ICAO:      "K7K4",
		Name:      "Ohio County Airport",
		City:      "Hartford",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 535,
		Latitude:  37.4583015442,
		Longitude: -86.8499984741,
		Timezone:  "America/Chicago",
	},
	"K7K5": {
		ICAO:      "K7K5",
		Name:      "Kenmare Municipal Airport",
		City:      "Kenmare",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1962,
		Latitude:  48.6675987244,
		Longitude: -102.047996521,
		Timezone:  "America/Chicago",
	},
	"K7K8": {
		ICAO:      "K7K8",
		Name:      "Martin Field",
		City:      "So Sioux City",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1100,
		Latitude:  42.4542007446,
		Longitude: -96.4725036621,
		Timezone:  "America/Chicago",
	},
	"K7L2": {
		ICAO:      "K7L2",
		Name:      "Linton Municipal Airport",
		City:      "Linton",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1779,
		Latitude:  46.21829987,
		Longitude: -100.2450027,
		Timezone:  "America/Chicago",
	},
	"K7L8": {
		ICAO:      "K7L8",
		Name:      "Post Air Airport",
		City:      "Indianapolis",
		State:     "Indiana",
		Country:   "US",
		Elevation: 861,
		Latitude:  39.75,
		Longitude: -86.0139007568,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"K7M1": {
		ICAO:      "K7M1",
		Name:      "Mc Gehee Municipal Airport",
		City:      "Mc Gehee",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 141,
		Latitude:  33.62020111,
		Longitude: -91.3647995,
		Timezone:  "America/Chicago",
	},
	"K7M2": {
		ICAO:      "K7M2",
		Name:      "Mountain View Wilcox Memorial Field",
		City:      "Mountain View",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 805,
		Latitude:  35.8644981384,
		Longitude: -92.0903015137,
		Timezone:  "America/Chicago",
	},
	"K7M3": {
		ICAO:      "K7M3",
		Name:      "Bearce Airport",
		City:      "Mount Ida",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 644,
		Latitude:  34.52930069,
		Longitude: -93.52709961,
		Timezone:  "America/Chicago",
	},
	"K7M4": {
		ICAO:      "K7M4",
		Name:      "Osceola Municipal Airport",
		City:      "Osceola",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 234,
		Latitude:  35.6911010742,
		Longitude: -90.0101013184,
		Timezone:  "America/Chicago",
	},
	"K7M5": {
		ICAO:      "K7M5",
		Name:      "Ozark Franklin County Airport",
		City:      "Ozark",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 648,
		Latitude:  35.5107002258,
		Longitude: -93.839302063,
		Timezone:  "America/Chicago",
	},
	"K7M8": {
		ICAO:      "K7M8",
		Name:      "Rector Airport",
		City:      "Rector",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 281,
		Latitude:  36.2500991821,
		Longitude: -90.3195037842,
		Timezone:  "America/Chicago",
	},
	"K7M9": {
		ICAO:      "K7M9",
		Name:      "Salem Airport",
		City:      "Salem",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 787,
		Latitude:  36.3558998108,
		Longitude: -91.8310012817,
		Timezone:  "America/Chicago",
	},
	"K7N1": {
		ICAO:      "K7N1",
		Name:      "Corning Painted Post Airport",
		City:      "Corning",
		State:     "New-York",
		Country:   "US",
		Elevation: 962,
		Latitude:  42.17589951,
		Longitude: -77.11219788,
		Timezone:  "America/New_York",
	},
	"K7S0": {
		ICAO:      "K7S0",
		Name:      "Ronan Airport",
		City:      "Ronan",
		State:     "Montana",
		Country:   "US",
		Elevation: 3086,
		Latitude:  47.567199707,
		Longitude: -114.1009979248,
		Timezone:  "America/Denver",
	},
	"K7S1": {
		ICAO:      "K7S1",
		Name:      "Twin Bridges Airport",
		City:      "Twin Bridges",
		State:     "Montana",
		Country:   "US",
		Elevation: 4777,
		Latitude:  45.5337982178,
		Longitude: -112.3030014038,
		Timezone:  "America/Denver",
	},
	"K7S5": {
		ICAO:      "K7S5",
		Name:      "Independence State Airport",
		City:      "Independence",
		State:     "Oregon",
		Country:   "US",
		Elevation: 180,
		Latitude:  44.8670005798,
		Longitude: -123.1979980469,
		Timezone:  "America/Los_Angeles",
	},
	"K7S6": {
		ICAO:      "K7S6",
		Name:      "White Sulphur Springs Airport",
		City:      "White Sulphur Springs",
		State:     "Montana",
		Country:   "US",
		Elevation: 5061,
		Latitude:  46.5041007996,
		Longitude: -110.9130020142,
		Timezone:  "America/Denver",
	},
	"K7S7": {
		ICAO:      "K7S7",
		Name:      "Valier Airport",
		City:      "Valier",
		State:     "Montana",
		Country:   "US",
		Elevation: 3820,
		Latitude:  48.2999992371,
		Longitude: -112.2509994507,
		Timezone:  "America/Denver",
	},
	"K7S9": {
		ICAO:      "K7S9",
		Name:      "Lenhardt Airpark",
		City:      "Hubbard",
		State:     "Oregon",
		Country:   "US",
		Elevation: 165,
		Latitude:  45.1804008484,
		Longitude: -122.7429962158,
		Timezone:  "America/Los_Angeles",
	},
	"K7SP": {
		ICAO:      "K7SP",
		Name:      "Seven Springs Airport",
		City:      "Seven Springs Borough",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 2907,
		Latitude:  40.0099983215,
		Longitude: -79.3218994141,
		Timezone:  "America/New_York",
	},
	"K7T7": {
		ICAO:      "K7T7",
		Name:      "Skywest Inc Airport",
		City:      "Midland",
		State:     "Texas",
		Country:   "US",
		Elevation: 2800,
		Latitude:  31.858499527,
		Longitude: -102.0739974976,
		Timezone:  "America/Chicago",
	},
	"K7V2": {
		ICAO:      "K7V2",
		Name:      "North Fork Valley Airport",
		City:      "Paonia",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5798,
		Latitude:  38.8316993713,
		Longitude: -107.6460037231,
		Timezone:  "America/Denver",
	},
	"K7V3": {
		ICAO:      "K7V3",
		Name:      "Big Foot Airfield",
		City:      "Walworth",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 951,
		Latitude:  42.5256996155,
		Longitude: -88.6529998779,
		Timezone:  "America/Chicago",
	},
	"K7V6": {
		ICAO:      "K7V6",
		Name:      "Camp Guernsey Airport",
		City:      "Guernsey",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4400,
		Latitude:  42.259601593,
		Longitude: -104.7279968262,
		Timezone:  "America/Denver",
	},
	"K7V7": {
		ICAO:      "K7V7",
		Name:      "Red Cloud Municipal Airport",
		City:      "Red Cloud",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1744,
		Latitude:  40.08470154,
		Longitude: -98.54060364,
		Timezone:  "America/Chicago",
	},
	"K7V8": {
		ICAO:      "K7V8",
		Name:      "Julesburg Municipal Airport",
		City:      "Julesburg",
		State:     "Colorado",
		Country:   "US",
		Elevation: 3495,
		Latitude:  40.97079849,
		Longitude: -102.3150024,
		Timezone:  "America/Denver",
	},
	"K7V9": {
		ICAO:      "K7V9",
		Name:      "City of Las Animas Bent County Airport",
		City:      "Las Animas",
		State:     "Colorado",
		Country:   "US",
		Elevation: 3915,
		Latitude:  38.0527992249,
		Longitude: -103.2369995117,
		Timezone:  "America/Denver",
	},
	"K7W5": {
		ICAO:      "K7W5",
		Name:      "Henry County Airport",
		City:      "Napoleon",
		State:     "Ohio",
		Country:   "US",
		Elevation: 683,
		Latitude:  41.3742980957,
		Longitude: -84.0679016113,
		Timezone:  "America/New_York",
	},
	"K7W6": {
		ICAO:      "K7W6",
		Name:      "Hyde County Airport",
		City:      "Engelhard",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 8,
		Latitude:  35.5624008179,
		Longitude: -75.9552001953,
		Timezone:  "America/New_York",
	},
	"K7Y4": {
		ICAO:      "K7Y4",
		Name:      "Bagley Municipal Airport",
		City:      "Bagley",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1500,
		Latitude:  47.5250015259,
		Longitude: -95.3608016968,
		Timezone:  "America/Chicago",
	},
	"K80D": {
		ICAO:      "K80D",
		Name:      "Clare County Airport",
		City:      "Harrison",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1142,
		Latitude:  44.0527992249,
		Longitude: -84.8125,
		Timezone:  "America/Detroit",
	},
	"K80F": {
		ICAO:      "K80F",
		IATA:      "ATE",
		Name:      "Antlers Municipal Airport",
		City:      "Antlers",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 575,
		Latitude:  34.1926002502,
		Longitude: -95.6499023438,
		Timezone:  "America/Chicago",
	},
	"K80T": {
		ICAO:      "K80T",
		Name:      "Quincy Municipal Airport",
		City:      "Quincy",
		State:     "Washington",
		Country:   "US",
		Elevation: 1271,
		Latitude:  47.2116012573,
		Longitude: -119.8399963379,
		Timezone:  "America/Los_Angeles",
	},
	"K81B": {
		ICAO:      "K81B",
		Name:      "Oxford County Regional Airport",
		City:      "Oxford",
		State:     "Maine",
		Country:   "US",
		Elevation: 346,
		Latitude:  44.1573982239,
		Longitude: -70.481300354,
		Timezone:  "America/New_York",
	},
	"K81R": {
		ICAO:      "K81R",
		Name:      "San Saba County Municipal Airport",
		City:      "San Saba",
		State:     "Texas",
		Country:   "US",
		Elevation: 1249,
		Latitude:  31.235200882,
		Longitude: -98.7170028687,
		Timezone:  "America/Chicago",
	},
	"K82A": {
		ICAO:      "K82A",
		Name:      "Marion County Airport",
		City:      "Buena Vista",
		State:     "Georgia",
		Country:   "US",
		Elevation: 682,
		Latitude:  32.2823982239,
		Longitude: -84.5035018921,
		Timezone:  "America/New_York",
	},
	"K82C": {
		ICAO:      "K82C",
		Name:      "Mauston New Lisbon Union Airport",
		City:      "New Lisbon",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 906,
		Latitude:  43.8386993408,
		Longitude: -90.1377029419,
		Timezone:  "America/Chicago",
	},
	"K82J": {
		ICAO:      "K82J",
		Name:      "Ferguson Airport",
		City:      "Pensacola",
		State:     "Florida",
		Country:   "US",
		Elevation: 27,
		Latitude:  30.3987998962,
		Longitude: -87.3486022949,
		Timezone:  "America/Chicago",
	},
	"K83D": {
		ICAO:      "K83D",
		Name:      "Mackinac County Airport",
		City:      "St Ignace",
		State:     "Michigan",
		Country:   "US",
		Elevation: 624,
		Latitude:  45.89170074,
		Longitude: -84.73809814,
		Timezone:  "America/Detroit",
	},
	"K84D": {
		ICAO:      "K84D",
		Name:      "Cheyenne Eagle Butte Airport",
		City:      "Eagle Butte",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 2448,
		Latitude:  44.9844017,
		Longitude: -101.2509995,
		Timezone:  "America/Denver",
	},
	"K84R": {
		ICAO:      "K84R",
		Name:      "Smithville Crawford Municipal Airport",
		City:      "Smithville",
		State:     "Texas",
		Country:   "US",
		Elevation: 323,
		Latitude:  30.030500412,
		Longitude: -97.1669006348,
		Timezone:  "America/Chicago",
	},
	"K86F": {
		ICAO:      "K86F",
		Name:      "Carnegie Municipal Airport",
		City:      "Carnegie",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1354,
		Latitude:  35.1236000061,
		Longitude: -98.5752029419,
		Timezone:  "America/Chicago",
	},
	"K87I": {
		ICAO:      "K87I",
		Name:      "Yazoo County Airport",
		City:      "Yazoo City",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 104,
		Latitude:  32.8832015991,
		Longitude: -90.4636001587,
		Timezone:  "America/Chicago",
	},
	"K87K": {
		ICAO:      "K87K",
		Name:      "El Dorado Springs Memorial Airport",
		City:      "El Dorado Springs",
		State:     "Missouri",
		Country:   "US",
		Elevation: 931,
		Latitude:  37.8567008972,
		Longitude: -93.9990997314,
		Timezone:  "America/Chicago",
	},
	"K87Y": {
		ICAO:      "K87Y",
		Name:      "Blackhawk Airfield",
		City:      "Madison",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 920,
		Latitude:  43.10490036,
		Longitude: -89.1855011,
		Timezone:  "America/Chicago",
	},
	"K88J": {
		ICAO:      "K88J",
		Name:      "Allendale County Airport",
		City:      "Allendale",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 161,
		Latitude:  32.995098114,
		Longitude: -81.2702026367,
		Timezone:  "America/New_York",
	},
	"K88M": {
		ICAO:      "K88M",
		Name:      "Eureka Airport",
		City:      "Eureka",
		State:     "Montana",
		Country:   "US",
		Elevation: 2668,
		Latitude:  48.9738006592,
		Longitude: -115.0759963989,
		Timezone:  "America/Denver",
	},
	"K88R": {
		ICAO:      "K88R",
		Name:      "Spicewood Airport",
		City:      "Spicewood",
		State:     "Texas",
		Country:   "US",
		Elevation: 830,
		Latitude:  30.4780006409,
		Longitude: -98.1211013794,
		Timezone:  "America/Chicago",
	},
	"K8A0": {
		ICAO:      "K8A0",
		Name:      "Albertville Regional Thomas J Brumlik Field",
		City:      "Albertville",
		State:     "Alabama",
		Country:   "US",
		Elevation: 1032,
		Latitude:  34.22909927,
		Longitude: -86.25579834,
		Timezone:  "America/Chicago",
	},
	"K8A1": {
		ICAO:      "K8A1",
		Name:      "Guntersville Municipal Joe Starnes Field",
		City:      "Guntersville",
		State:     "Alabama",
		Country:   "US",
		Elevation: 615,
		Latitude:  34.3993988037,
		Longitude: -86.2702026367,
		Timezone:  "America/Chicago",
	},
	"K8A3": {
		ICAO:      "K8A3",
		Name:      "Livingston Municipal Airport",
		City:      "Livingston",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1372,
		Latitude:  36.4121017456,
		Longitude: -85.3115997314,
		Timezone:  "America/Chicago",
	},
	"K8A6": {
		ICAO:      "K8A6",
		Name:      "Wilgrove Air Park",
		City:      "Charlotte",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 799,
		Latitude:  35.2137985229,
		Longitude: -80.6700973511,
		Timezone:  "America/New_York",
	},
	"K8B0": {
		ICAO:      "K8B0",
		Name:      "Steven A. Bean Municipal Airport",
		City:      "Rangeley",
		State:     "Maine",
		Country:   "US",
		Elevation: 1825,
		Latitude:  44.9919014,
		Longitude: -70.66459656,
		Timezone:  "America/New_York",
	},
	"K8B1": {
		ICAO:      "K8B1",
		Name:      "Hawthorne Feather Airpark",
		City:      "Hillsboro",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 600,
		Latitude:  43.0666999817,
		Longitude: -71.8994979858,
		Timezone:  "America/New_York",
	},
	"K8B5": {
		ICAO:      "K8B5",
		Name:      "Tanner Hiller Airport",
		City:      "Barre/Barre Plains",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 584,
		Latitude:  42.3563995361,
		Longitude: -72.1300964355,
		Timezone:  "America/New_York",
	},
	"K8C4": {
		ICAO:      "K8C4",
		Name:      "Mathews Memorial Airport",
		City:      "Tipton",
		State:     "Iowa",
		Country:   "US",
		Elevation: 840,
		Latitude:  41.7634010315,
		Longitude: -91.1529006958,
		Timezone:  "America/Chicago",
	},
	"K8D1": {
		ICAO:      "K8D1",
		Name:      "New Holstein Municipal Airport",
		City:      "New Holstein",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 992,
		Latitude:  43.9441986084,
		Longitude: -88.1135025024,
		Timezone:  "America/Chicago",
	},
	"K8D3": {
		ICAO:      "K8D3",
		Name:      "Sisseton Municipal Airport",
		City:      "Sisseton",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1161,
		Latitude:  45.6707992554,
		Longitude: -96.9962005615,
		Timezone:  "America/Chicago",
	},
	"K8D7": {
		ICAO:      "K8D7",
		Name:      "Clark County Airport",
		City:      "Clark",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1792,
		Latitude:  44.8950004578,
		Longitude: -97.7108001709,
		Timezone:  "America/Chicago",
	},
	"K8F3": {
		ICAO:      "K8F3",
		Name:      "Crosbyton Municipal Airport",
		City:      "Crosbyton",
		State:     "Texas",
		Country:   "US",
		Elevation: 3018,
		Latitude:  33.6237983704,
		Longitude: -101.2409973145,
		Timezone:  "America/Chicago",
	},
	"K8F5": {
		ICAO:      "K8F5",
		Name:      "Greater Morris County Airport",
		City:      "Daingerfield",
		State:     "Texas",
		Country:   "US",
		Elevation: 402,
		Latitude:  33.1268005371,
		Longitude: -94.6987991333,
		Timezone:  "America/Chicago",
	},
	"K8F6": {
		ICAO:      "K8F6",
		Name:      "Murdo Municipal Airport",
		City:      "Murdo",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 2263,
		Latitude:  43.8516998291,
		Longitude: -100.7119979858,
		Timezone:  "America/Chicago",
	},
	"K8G1": {
		ICAO:      "K8G1",
		Name:      "Willard Airport",
		City:      "Willard",
		State:     "Ohio",
		Country:   "US",
		Elevation: 967,
		Latitude:  41.0387001038,
		Longitude: -82.7246017456,
		Timezone:  "America/New_York",
	},
	"K8G2": {
		ICAO:      "K8G2",
		Name:      "Corry Lawrence Airport",
		City:      "Corry",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1766,
		Latitude:  41.9076004028,
		Longitude: -79.6410980225,
		Timezone:  "America/New_York",
	},
	"K8G6": {
		ICAO:      "K8G6",
		Name:      "Harrison County Airport",
		City:      "Cadiz",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1174,
		Latitude:  40.2383995056,
		Longitude: -81.0129013062,
		Timezone:  "America/New_York",
	},
	"K8J7": {
		ICAO:      "K8J7",
		Name:      "Tomlinson Field",
		City:      "New Rockford",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1533,
		Latitude:  47.6963996887,
		Longitude: -99.1312026978,
		Timezone:  "America/Chicago",
	},
	"K8K2": {
		ICAO:      "K8K2",
		Name:      "Harper Municipal Airport",
		City:      "Harper",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1427,
		Latitude:  37.2781982422,
		Longitude: -98.0435028076,
		Timezone:  "America/Chicago",
	},
	"K8M1": {
		ICAO:      "K8M1",
		Name:      "Booneville Baldwyn Airport",
		City:      "Booneville/Baldwyn",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 384,
		Latitude:  34.590801239,
		Longitude: -88.6475982666,
		Timezone:  "America/Chicago",
	},
	"K8M8": {
		ICAO:      "K8M8",
		Name:      "Garland Airport",
		City:      "Lewiston",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1218,
		Latitude:  44.8064994812,
		Longitude: -84.2761993408,
		Timezone:  "America/Detroit",
	},
	"K8M9": {
		ICAO:      "K8M9",
		Name:      "Providence Webster County Airport",
		City:      "Providence",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 393,
		Latitude:  37.4248008728,
		Longitude: -87.7360992432,
		Timezone:  "America/Chicago",
	},
	"K8N2": {
		ICAO:      "K8N2",
		Name:      "Skydive Chicago Airport",
		City:      "Ottawa",
		State:     "Illinois",
		Country:   "US",
		Elevation: 616,
		Latitude:  41.3997993469,
		Longitude: -88.7938995361,
		Timezone:  "America/Chicago",
	},
	"K8N8": {
		ICAO:      "K8N8",
		Name:      "Danville Airport",
		City:      "Danville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 559,
		Latitude:  40.951499939,
		Longitude: -76.6427001953,
		Timezone:  "America/New_York",
	},
	"K8S0": {
		ICAO:      "K8S0",
		Name:      "Starr Browning Airstrip",
		City:      "Browning",
		State:     "Montana",
		Country:   "US",
		Elevation: 4655,
		Latitude:  48.601398468,
		Longitude: -113.1149978638,
		Timezone:  "America/Denver",
	},
	"K8S1": {
		ICAO:      "K8S1",
		Name:      "Polson Airport",
		City:      "Polson",
		State:     "Montana",
		Country:   "US",
		Elevation: 2941,
		Latitude:  47.695400238,
		Longitude: -114.1849975586,
		Timezone:  "America/Denver",
	},
	"K8T6": {
		ICAO:      "K8T6",
		Name:      "Live Oak County Airport",
		City:      "George West",
		State:     "Texas",
		Country:   "US",
		Elevation: 129,
		Latitude:  28.3628005981,
		Longitude: -98.1165008545,
		Timezone:  "America/Chicago",
	},
	"K8T8": {
		ICAO:      "K8T8",
		Name:      "San Geronimo Airpark",
		City:      "San Antonio",
		State:     "Texas",
		Country:   "US",
		Elevation: 1040,
		Latitude:  29.5104999542,
		Longitude: -98.7984008789,
		Timezone:  "America/Chicago",
	},
	"K8U6": {
		ICAO:      "K8U6",
		Name:      "Terry Airport",
		City:      "Terry",
		State:     "Montana",
		Country:   "US",
		Elevation: 2283,
		Latitude:  46.7770996094,
		Longitude: -105.31300354,
		Timezone:  "America/Denver",
	},
	"K8U8": {
		ICAO:      "K8U8",
		Name:      "Townsend Airport",
		City:      "Townsend",
		State:     "Montana",
		Country:   "US",
		Elevation: 3893,
		Latitude:  46.331199646,
		Longitude: -111.483001709,
		Timezone:  "America/Denver",
	},
	"K8V1": {
		ICAO:      "K8V1",
		Name:      "Del Norte Municipal & County Airport",
		City:      "Del Norte",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7949,
		Latitude:  37.71360016,
		Longitude: -106.3539963,
		Timezone:  "America/Denver",
	},
	"K8V2": {
		ICAO:      "K8V2",
		Name:      "Stuart Atkinson Municipal Airport",
		City:      "Atkinson",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2130,
		Latitude:  42.5625,
		Longitude: -99.037902832,
		Timezone:  "America/Chicago",
	},
	"K8V3": {
		ICAO:      "K8V3",
		Name:      "Parkston Municipal Airport",
		City:      "Parkston",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1415,
		Latitude:  43.3791999817,
		Longitude: -97.9711990356,
		Timezone:  "America/Chicago",
	},
	"K8V7": {
		ICAO:      "K8V7",
		Name:      "Springfield Municipal Airport",
		City:      "Springfield",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4390,
		Latitude:  37.4586982727,
		Longitude: -102.6179962158,
		Timezone:  "America/Denver",
	},
	"K8W2": {
		ICAO:      "K8W2",
		Name:      "New Market Airport",
		City:      "New Market",
		State:     "Virginia",
		Country:   "US",
		Elevation: 975,
		Latitude:  38.6557006836,
		Longitude: -78.7080993652,
		Timezone:  "America/New_York",
	},
	"K8WC": {
		ICAO:      "K8WC",
		Name:      "Washington County Airport",
		City:      "Potosi",
		State:     "Missouri",
		Country:   "US",
		Elevation: 959,
		Latitude:  37.9291992188,
		Longitude: -90.7314987183,
		Timezone:  "America/Chicago",
	},
	"K8Y6": {
		ICAO:      "K8Y6",
		Name:      "Leaders Clear Lake Airport",
		City:      "Clear Lake",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 990,
		Latitude:  45.4444007874,
		Longitude: -93.9710998535,
		Timezone:  "America/Chicago",
	},
	"K90F": {
		ICAO:      "K90F",
		Name:      "Broken Bow Airport",
		City:      "Broken Bow",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 404,
		Latitude:  34.013999939,
		Longitude: -94.7585983276,
		Timezone:  "America/Chicago",
	},
	"K91F": {
		ICAO:      "K91F",
		Name:      "Arrowhead Airport",
		City:      "Canadian",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 851,
		Latitude:  35.1562995911,
		Longitude: -95.6212997437,
		Timezone:  "America/Chicago",
	},
	"K92F": {
		ICAO:      "K92F",
		Name:      "Chattanooga Sky Harbor Airport",
		City:      "Chattanooga",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1135,
		Latitude:  34.3686981201,
		Longitude: -98.6820983887,
		Timezone:  "America/Chicago",
	},
	"K93C": {
		ICAO:      "K93C",
		Name:      "Richland Airport",
		City:      "Richland Center",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 742,
		Latitude:  43.2834014893,
		Longitude: -90.2983016968,
		Timezone:  "America/Chicago",
	},
	"K93F": {
		ICAO:      "K93F",
		Name:      "Mignon Laird Municipal Airport",
		City:      "Cheyenne",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 2084,
		Latitude:  35.6074981689,
		Longitude: -99.7046966553,
		Timezone:  "America/Chicago",
	},
	"K93Y": {
		ICAO:      "K93Y",
		Name:      "David City Municipal Airport",
		City:      "David City",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1617,
		Latitude:  41.23089981,
		Longitude: -97.12290192,
		Timezone:  "America/Chicago",
	},
	"K94E": {
		ICAO:      "K94E",
		Name:      "Whiskey Creek Airport",
		City:      "Silver City",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 6126,
		Latitude:  32.7620010376,
		Longitude: -108.2080001831,
		Timezone:  "America/Denver",
	},
	"K94K": {
		ICAO:      "K94K",
		Name:      "Cassville Municipal Airport",
		City:      "Cassville",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1482,
		Latitude:  36.6973991394,
		Longitude: -93.9004974365,
		Timezone:  "America/Chicago",
	},
	"K94R": {
		ICAO:      "K94R",
		Name:      "Lackey Aviation Airport",
		City:      "Wharton",
		State:     "Texas",
		Country:   "US",
		Elevation: 90,
		Latitude:  29.2660999298,
		Longitude: -96.0076980591,
		Timezone:  "America/Chicago",
	},
	"K95D": {
		ICAO:      "K95D",
		Name:      "Beulah Airport",
		City:      "Beulah",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1791,
		Latitude:  47.2508010864,
		Longitude: -101.8140029907,
		Timezone:  "America/North_Dakota/Beulah",
	},
	"K95E": {
		ICAO:      "K95E",
		Name:      "Stallion Army Air Field",
		City:      "Socorro",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4925,
		Latitude:  33.81900024,
		Longitude: -106.6449966,
		Timezone:  "America/Denver",
	},
	"K95F": {
		ICAO:      "K95F",
		Name:      "Cleveland Municipal Airport",
		City:      "Cleveland",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 912,
		Latitude:  36.2837982178,
		Longitude: -96.4633026123,
		Timezone:  "America/Chicago",
	},
	"K96D": {
		ICAO:      "K96D",
		Name:      "Walhalla Municipal Airport",
		City:      "Walhalla",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 953,
		Latitude:  48.9406013489,
		Longitude: -97.9028015137,
		Timezone:  "America/Chicago",
	},
	"K97M": {
		ICAO:      "K97M",
		Name:      "Ekalaka Airport",
		City:      "Ekalaka",
		State:     "Montana",
		Country:   "US",
		Elevation: 3503,
		Latitude:  45.8777999878,
		Longitude: -104.5370025635,
		Timezone:  "America/Denver",
	},
	"K98D": {
		ICAO:      "K98D",
		Name:      "Onida Municipal Airport",
		City:      "Onida",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1874,
		Latitude:  44.7008018494,
		Longitude: -100.1009979248,
		Timezone:  "America/Chicago",
	},
	"K99N": {
		ICAO:      "K99N",
		Name:      "Bamberg County Airport",
		City:      "Bamberg",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 231,
		Latitude:  33.3045005798,
		Longitude: -81.1083984375,
		Timezone:  "America/New_York",
	},
	"K99V": {
		ICAO:      "K99V",
		Name:      "Crawford Airport",
		City:      "Crawford",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6470,
		Latitude:  38.7042007446,
		Longitude: -107.6439971924,
		Timezone:  "America/Denver",
	},
	"K9A0": {
		ICAO:      "K9A0",
		Name:      "Lumpkin County Wimpys Airport",
		City:      "Dahlonega",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1311,
		Latitude:  34.5792999268,
		Longitude: -84.0206985474,
		Timezone:  "America/New_York",
	},
	"K9A4": {
		ICAO:      "K9A4",
		Name:      "Lawrence County Airport",
		City:      "Courtland",
		State:     "Alabama",
		Country:   "US",
		Elevation: 588,
		Latitude:  34.6594009399,
		Longitude: -87.3488006592,
		Timezone:  "America/Chicago",
	},
	"K9A5": {
		ICAO:      "K9A5",
		Name:      "Barwick Lafayette Airport",
		City:      "Lafayette",
		State:     "Georgia",
		Country:   "US",
		Elevation: 777,
		Latitude:  34.6885986328,
		Longitude: -85.290397644,
		Timezone:  "America/New_York",
	},
	"K9C8": {
		ICAO:      "K9C8",
		Name:      "Evart Municipal Airport",
		City:      "Evart",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1018,
		Latitude:  43.8959007263,
		Longitude: -85.2791976929,
		Timezone:  "America/Detroit",
	},
	"K9D0": {
		ICAO:      "K9D0",
		Name:      "Highmore Municipal Airport",
		City:      "Highmore",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1854,
		Latitude:  44.5415992737,
		Longitude: -99.4461975098,
		Timezone:  "America/Chicago",
	},
	"K9D1": {
		ICAO:      "K9D1",
		Name:      "Gregory Municipal Flynn Field",
		City:      "Gregory",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 2168,
		Latitude:  43.22169876,
		Longitude: -99.40329742,
		Timezone:  "America/Chicago",
	},
	"K9D2": {
		ICAO:      "K9D2",
		Name:      "Harding County Airport",
		City:      "Buffalo",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 2889,
		Latitude:  45.5806007385,
		Longitude: -103.5299987793,
		Timezone:  "America/Denver",
	},
	"K9D4": {
		ICAO:      "K9D4",
		Name:      "Deck Airport",
		City:      "Myerstown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 523,
		Latitude:  40.35179901,
		Longitude: -76.32959747,
		Timezone:  "America/New_York",
	},
	"K9D7": {
		ICAO:      "K9D7",
		Name:      "Cando Municipal Airport",
		City:      "Cando",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1481,
		Latitude:  48.47999954,
		Longitude: -99.23590088,
		Timezone:  "America/Chicago",
	},
	"K9D9": {
		ICAO:      "K9D9",
		Name:      "Hastings Airport",
		City:      "Hastings",
		State:     "Michigan",
		Country:   "US",
		Elevation: 801,
		Latitude:  42.6636009216,
		Longitude: -85.3462982178,
		Timezone:  "America/Detroit",
	},
	"K9F0": {
		ICAO:      "K9F0",
		Name:      "Dublin Municipal Airport",
		City:      "Dublin",
		State:     "Texas",
		Country:   "US",
		Elevation: 1495,
		Latitude:  32.0681991577,
		Longitude: -98.325302124,
		Timezone:  "America/Chicago",
	},
	"K9F8": {
		ICAO:      "K9F8",
		Name:      "Hoven Municipal Airport",
		City:      "Hoven",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1884,
		Latitude:  45.257598877,
		Longitude: -99.7977981567,
		Timezone:  "America/Chicago",
	},
	"K9F9": {
		ICAO:      "K9F9",
		Name:      "Sycamore Strip",
		City:      "Fort Worth",
		State:     "Texas",
		Country:   "US",
		Elevation: 760,
		Latitude:  32.6285018921,
		Longitude: -97.3535995483,
		Timezone:  "America/Chicago",
	},
	"K9G2": {
		ICAO:      "K9G2",
		Name:      "Prices Airport",
		City:      "Linden",
		State:     "Michigan",
		Country:   "US",
		Elevation: 920,
		Latitude:  42.807598114,
		Longitude: -83.7698974609,
		Timezone:  "America/Detroit",
	},
	"K9G3": {
		ICAO:      "K9G3",
		Name:      "Akron Airport",
		City:      "Akron",
		State:     "New-York",
		Country:   "US",
		Elevation: 840,
		Latitude:  43.0210990906,
		Longitude: -78.4824981689,
		Timezone:  "America/New_York",
	},
	"K9G8": {
		ICAO:      "K9G8",
		Name:      "Ebensburg Airport",
		City:      "Ebensburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 2099,
		Latitude:  40.4612007141,
		Longitude: -78.7751998901,
		Timezone:  "America/New_York",
	},
	"K9K7": {
		ICAO:      "K9K7",
		Name:      "Ellsworth Municipal Airport",
		City:      "Ellsworth",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1615,
		Latitude:  38.7504005432,
		Longitude: -98.2293014526,
		Timezone:  "America/Chicago",
	},
	"K9K8": {
		ICAO:      "K9K8",
		Name:      "Kingman Airport Clyde Cessna Field",
		City:      "Kingman",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1607,
		Latitude:  37.66899872,
		Longitude: -98.12390137,
		Timezone:  "America/Chicago",
	},
	"K9L2": {
		ICAO:      "K9L2",
		Name:      "Edwards Af Aux North Base Airport",
		City:      "Edwards",
		State:     "California",
		Country:   "US",
		Elevation: 2299,
		Latitude:  34.9907989502,
		Longitude: -117.8629989624,
		Timezone:  "America/Los_Angeles",
	},
	"K9M4": {
		ICAO:      "K9M4",
		Name:      "Ackerman Choctaw County Airport",
		City:      "Ackerman",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 552,
		Latitude:  33.3035011292,
		Longitude: -89.2284011841,
		Timezone:  "America/Chicago",
	},
	"K9M6": {
		ICAO:      "K9M6",
		Name:      "Kelly Airport",
		City:      "Oak Grove",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 112,
		Latitude:  32.8492012024,
		Longitude: -91.4039001465,
		Timezone:  "America/Chicago",
	},
	"K9M8": {
		ICAO:      "K9M8",
		Name:      "Sheridan Municipal Airport",
		City:      "Sheridan",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 232,
		Latitude:  34.3283996582,
		Longitude: -92.3509979248,
		Timezone:  "America/Chicago",
	},
	"K9S2": {
		ICAO:      "K9S2",
		Name:      "Scobey Airport",
		City:      "Scobey",
		State:     "Montana",
		Country:   "US",
		Elevation: 2432,
		Latitude:  48.8077011108,
		Longitude: -105.4390029907,
		Timezone:  "America/Denver",
	},
	"K9S4": {
		ICAO:      "K9S4",
		Name:      "Mineral County Airport",
		City:      "Superior",
		State:     "Montana",
		Country:   "US",
		Elevation: 2787,
		Latitude:  47.1683006287,
		Longitude: -114.8539962769,
		Timezone:  "America/Denver",
	},
	"K9S5": {
		ICAO:      "K9S5",
		Name:      "Three Forks Airport",
		City:      "Three Forks",
		State:     "Montana",
		Country:   "US",
		Elevation: 4089,
		Latitude:  45.8781013489,
		Longitude: -111.5690002441,
		Timezone:  "America/Denver",
	},
	"K9S9": {
		ICAO:      "K9S9",
		Name:      "Lexington Airport",
		City:      "Lexington",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1634,
		Latitude:  45.4541015625,
		Longitude: -119.6900024414,
		Timezone:  "America/Los_Angeles",
	},
	"K9U0": {
		ICAO:      "K9U0",
		Name:      "Turner Airport",
		City:      "Turner",
		State:     "Montana",
		Country:   "US",
		Elevation: 3049,
		Latitude:  48.8541984558,
		Longitude: -108.408996582,
		Timezone:  "America/Denver",
	},
	"K9U4": {
		ICAO:      "K9U4",
		Name:      "Dixon Airport",
		City:      "Dixon",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 6520,
		Latitude:  41.0382995605,
		Longitude: -107.4970016479,
		Timezone:  "America/Denver",
	},
	"K9U7": {
		ICAO:      "K9U7",
		Name:      "Currant Ranch Airport",
		City:      "Currant",
		State:     "Nevada",
		Country:   "US",
		Elevation: 5181,
		Latitude:  38.736000061,
		Longitude: -115.4800033569,
		Timezone:  "America/Los_Angeles",
	},
	"K9V5": {
		ICAO:      "K9V5",
		Name:      "Modisett Airport",
		City:      "Rushville",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3751,
		Latitude:  42.73749924,
		Longitude: -102.4449997,
		Timezone:  "America/Denver",
	},
	"K9V6": {
		ICAO:      "K9V6",
		Name:      "Martin Municipal Airport",
		City:      "Martin",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 3293,
		Latitude:  43.165599823,
		Longitude: -101.7129974365,
		Timezone:  "America/Denver",
	},
	"K9V7": {
		ICAO:      "K9V7",
		Name:      "Eads Municipal Airport",
		City:      "Eads",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4245,
		Latitude:  38.4751014709,
		Longitude: -102.8109970093,
		Timezone:  "America/Denver",
	},
	"K9V9": {
		ICAO:      "K9V9",
		Name:      "Chamberlain Municipal Airport",
		City:      "Chamberlain",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1695,
		Latitude:  43.7661018372,
		Longitude: -99.3212966919,
		Timezone:  "America/Chicago",
	},
	"K9X1": {
		ICAO:      "K9X1",
		Name:      "Williams Airport",
		City:      "Porter",
		State:     "Texas",
		Country:   "US",
		Elevation: 122,
		Latitude:  30.1534004211,
		Longitude: -95.3219985962,
		Timezone:  "America/Chicago",
	},
	"K9Y1": {
		ICAO:      "K9Y1",
		Name:      "Weydahl Field",
		City:      "Killdeer",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 2256,
		Latitude:  47.3927993774,
		Longitude: -102.7710037231,
		Timezone:  "America/Denver",
	},
	"KA08": {
		ICAO:      "KA08",
		Name:      "Vaiden Field",
		City:      "Marion",
		State:     "Alabama",
		Country:   "US",
		Elevation: 225,
		Latitude:  32.5124015808,
		Longitude: -87.385597229,
		Timezone:  "America/Chicago",
	},
	"KA09": {
		ICAO:      "KA09",
		Name:      "Eagle Airpark",
		City:      "Bullhead City",
		State:     "Arizona",
		Country:   "US",
		Elevation: 485,
		Latitude:  34.8875007629,
		Longitude: -114.6159973145,
		Timezone:  "America/Los_Angeles",
	},
	"KA20": {
		ICAO:      "KA20",
		Name:      "Sun Valley Airport",
		City:      "Bullhead City",
		State:     "Arizona",
		Country:   "US",
		Elevation: 725,
		Latitude:  35.0125007629,
		Longitude: -114.56300354,
		Timezone:  "America/Phoenix",
	},
	"KA24": {
		ICAO:      "KA24",
		Name:      "California Pines Airport",
		City:      "Alturas",
		State:     "California",
		Country:   "US",
		Elevation: 4398,
		Latitude:  41.412399292,
		Longitude: -120.6839981079,
		Timezone:  "America/Los_Angeles",
	},
	"KA30": {
		ICAO:      "KA30",
		Name:      "Scott Valley Airport",
		City:      "Fort Jones",
		State:     "California",
		Country:   "US",
		Elevation: 2728,
		Latitude:  41.5582008362,
		Longitude: -122.8550033569,
		Timezone:  "America/Los_Angeles",
	},
	"KA32": {
		ICAO:      "KA32",
		Name:      "Butte Valley Airport",
		City:      "Dorris",
		State:     "California",
		Country:   "US",
		Elevation: 4243,
		Latitude:  41.8871994019,
		Longitude: -121.9759979248,
		Timezone:  "America/Los_Angeles",
	},
	"KA34": {
		ICAO:      "KA34",
		Name:      "Dayton Valley Airpark",
		City:      "Dayton/Carson City",
		State:     "Nevada",
		Country:   "US",
		Elevation: 4414,
		Latitude:  39.2383995056,
		Longitude: -119.5550003052,
		Timezone:  "America/Los_Angeles",
	},
	"KA39": {
		ICAO:      "KA39",
		Name:      "Phoenix Regional Airport",
		City:      "Phoenix",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1300,
		Latitude:  32.9916992188,
		Longitude: -111.9209976196,
		Timezone:  "America/Phoenix",
	},
	"KA50": {
		ICAO:      "KA50",
		Name:      "Colorado Springs East Airport",
		City:      "Ellicott",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6145,
		Latitude:  38.8744010925,
		Longitude: -104.4100036621,
		Timezone:  "America/Denver",
	},
	"KAAA": {
		ICAO:      "KAAA",
		Name:      "Logan County Airport",
		City:      "Lincoln",
		State:     "Illinois",
		Country:   "US",
		Elevation: 597,
		Latitude:  40.1586990356,
		Longitude: -89.3349990845,
		Timezone:  "America/Chicago",
	},
	"KAAF": {
		ICAO:      "KAAF",
		IATA:      "AAF",
		Name:      "Apalachicola Regional Airport",
		City:      "Apalachicola",
		State:     "Florida",
		Country:   "US",
		Elevation: 20,
		Latitude:  29.72750092,
		Longitude: -85.02749634,
		Timezone:  "America/New_York",
	},
	"KAAO": {
		ICAO:      "KAAO",
		Name:      "Colonel James Jabara Airport",
		City:      "Wichita",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1421,
		Latitude:  37.74760056,
		Longitude: -97.22109985,
		Timezone:  "America/Chicago",
	},
	"KAAS": {
		ICAO:      "KAAS",
		Name:      "Taylor County Airport",
		City:      "Campbellsville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 921,
		Latitude:  37.3582992554,
		Longitude: -85.3094024658,
		Timezone:  "America/New_York",
	},
	"KAAT": {
		ICAO:      "KAAT",
		Name:      "Alturas Municipal Airport",
		City:      "Alturas",
		State:     "California",
		Country:   "US",
		Elevation: 4378,
		Latitude:  41.48300171,
		Longitude: -120.5650024,
		Timezone:  "America/Los_Angeles",
	},
	"KABE": {
		ICAO:      "KABE",
		IATA:      "ABE",
		Name:      "Lehigh Valley International Airport",
		City:      "Allentown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 393,
		Latitude:  40.6520996094,
		Longitude: -75.4408035278,
		Timezone:  "America/New_York",
	},
	"KABI": {
		ICAO:      "KABI",
		IATA:      "ABI",
		Name:      "Abilene Regional Airport",
		City:      "Abilene",
		State:     "Texas",
		Country:   "US",
		Elevation: 1791,
		Latitude:  32.4113006592,
		Longitude: -99.6819000244,
		Timezone:  "America/Chicago",
	},
	"KABQ": {
		ICAO:      "KABQ",
		IATA:      "ABQ",
		Name:      "Albuquerque International Sunport Airport",
		City:      "Albuquerque",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 5355,
		Latitude:  35.0401992798,
		Longitude: -106.6090011597,
		Timezone:  "America/Denver",
	},
	"KABR": {
		ICAO:      "KABR",
		IATA:      "ABR",
		Name:      "Aberdeen Regional Airport",
		City:      "Aberdeen",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1302,
		Latitude:  45.4491004944,
		Longitude: -98.4217987061,
		Timezone:  "America/Chicago",
	},
	"KABY": {
		ICAO:      "KABY",
		IATA:      "ABY",
		Name:      "Southwest Georgia Regional Airport",
		City:      "Albany",
		State:     "Georgia",
		Country:   "US",
		Elevation: 197,
		Latitude:  31.5354995728,
		Longitude: -84.1945037842,
		Timezone:  "America/New_York",
	},
	"KACB": {
		ICAO:      "KACB",
		IATA:      "ACB",
		Name:      "Antrim County Airport",
		City:      "Bellaire",
		State:     "Michigan",
		Country:   "US",
		Elevation: 623,
		Latitude:  44.9886016846,
		Longitude: -85.1984024048,
		Timezone:  "America/Detroit",
	},
	"KACJ": {
		ICAO:      "KACJ",
		Name:      "Souther Field",
		City:      "Americus",
		State:     "Georgia",
		Country:   "US",
		Elevation: 468,
		Latitude:  32.1108017,
		Longitude: -84.18890381,
		Timezone:  "America/New_York",
	},
	"KACK": {
		ICAO:      "KACK",
		IATA:      "ACK",
		Name:      "Nantucket Memorial Airport",
		City:      "Nantucket",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 47,
		Latitude:  41.25310135,
		Longitude: -70.06020355,
		Timezone:  "America/New_York",
	},
	"KACP": {
		ICAO:      "KACP",
		Name:      "Allen Parish Airport",
		City:      "Oakdale",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 107,
		Latitude:  30.7502994537,
		Longitude: -92.6883010864,
		Timezone:  "America/Chicago",
	},
	"KACQ": {
		ICAO:      "KACQ",
		Name:      "Waseca Municipal Airport",
		City:      "Waseca",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1126,
		Latitude:  44.0735015869,
		Longitude: -93.5529022217,
		Timezone:  "America/Chicago",
	},
	"KACT": {
		ICAO:      "KACT",
		IATA:      "ACT",
		Name:      "Waco Regional Airport",
		City:      "Waco",
		State:     "Texas",
		Country:   "US",
		Elevation: 516,
		Latitude:  31.6112995148,
		Longitude: -97.2304992676,
		Timezone:  "America/Chicago",
	},
	"KACV": {
		ICAO:      "KACV",
		IATA:      "ACV",
		Name:      "Arcata Airport",
		City:      "Arcata/Eureka",
		State:     "California",
		Country:   "US",
		Elevation: 221,
		Latitude:  40.978099823,
		Longitude: -124.1090011597,
		Timezone:  "America/Los_Angeles",
	},
	"KACY": {
		ICAO:      "KACY",
		IATA:      "ACY",
		Name:      "Atlantic City International Airport",
		City:      "Atlantic City",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 75,
		Latitude:  39.4575996399,
		Longitude: -74.5772018433,
		Timezone:  "America/New_York",
	},
	"KACZ": {
		ICAO:      "KACZ",
		Name:      "Henderson Field",
		City:      "Wallace",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 39,
		Latitude:  34.7178993225,
		Longitude: -78.0036010742,
		Timezone:  "America/New_York",
	},
	"KADC": {
		ICAO:      "KADC",
		Name:      "Wadena Municipal Airport",
		City:      "Wadena",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1369,
		Latitude:  46.4502983093,
		Longitude: -95.2109985352,
		Timezone:  "America/Chicago",
	},
	"KADG": {
		ICAO:      "KADG",
		IATA:      "ADG",
		Name:      "Lenawee County Airport",
		City:      "Adrian",
		State:     "Michigan",
		Country:   "US",
		Elevation: 798,
		Latitude:  41.8676986694,
		Longitude: -84.0773010254,
		Timezone:  "America/Detroit",
	},
	"KADH": {
		ICAO:      "KADH",
		IATA:      "ADT",
		Name:      "Ada Municipal Airport",
		City:      "Ada",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1016,
		Latitude:  34.8042984009,
		Longitude: -96.6713027954,
		Timezone:  "America/Chicago",
	},
	"KADM": {
		ICAO:      "KADM",
		IATA:      "ADM",
		Name:      "Ardmore Municipal Airport",
		City:      "Ardmore",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 777,
		Latitude:  34.30301,
		Longitude: -97.0196342,
		Timezone:  "America/Chicago",
	},
	"KADS": {
		ICAO:      "KADS",
		IATA:      "ADS",
		Name:      "Addison Airport",
		City:      "Dallas",
		State:     "Texas",
		Country:   "US",
		Elevation: 644,
		Latitude:  32.9686012268,
		Longitude: -96.8364028931,
		Timezone:  "America/Chicago",
	},
	"KADT": {
		ICAO:      "KADT",
		Name:      "Atwood-Rawlins County City Airport",
		City:      "Atwood",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2991,
		Latitude:  39.84009933,
		Longitude: -101.0419998,
		Timezone:  "America/Chicago",
	},
	"KADU": {
		ICAO:      "KADU",
		Name:      "Audubon County Airport",
		City:      "Audubon",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1287,
		Latitude:  41.7014007568,
		Longitude: -94.920501709,
		Timezone:  "America/Chicago",
	},
	"KADW": {
		ICAO:      "KADW",
		IATA:      "ADW",
		Name:      "Andrews Air Force Base",
		City:      "Camp Springs",
		State:     "Maryland",
		Country:   "US",
		Elevation: 280,
		Latitude:  38.810798645,
		Longitude: -76.8669967651,
		Timezone:  "America/New_York",
	},
	"KAEG": {
		ICAO:      "KAEG",
		Name:      "Double Eagle II Airport",
		City:      "Albuquerque",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 5837,
		Latitude:  35.145198822,
		Longitude: -106.7949981689,
		Timezone:  "America/Denver",
	},
	"KAEJ": {
		ICAO:      "KAEJ",
		Name:      "Central Colorado Regional Airport",
		City:      "Buena Vista",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7946,
		Latitude:  38.814201355,
		Longitude: -106.1210021973,
		Timezone:  "America/Denver",
	},
	"KAEL": {
		ICAO:      "KAEL",
		IATA:      "AEL",
		Name:      "Albert Lea Municipal Airport",
		City:      "Albert Lea",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1260,
		Latitude:  43.68149948,
		Longitude: -93.36720276,
		Timezone:  "America/Chicago",
	},
	"KAEX": {
		ICAO:      "KAEX",
		IATA:      "AEX",
		Name:      "Alexandria International Airport",
		City:      "Alexandria",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 89,
		Latitude:  31.3274002075,
		Longitude: -92.5497970581,
		Timezone:  "America/Chicago",
	},
	"KAFF": {
		ICAO:      "KAFF",
		IATA:      "AFF",
		Name:      "USAF Academy Airfield",
		City:      "Colorado Springs",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6572,
		Latitude:  38.96969986,
		Longitude: -104.8130035,
		Timezone:  "America/Denver",
	},
	"KAFJ": {
		ICAO:      "KAFJ",
		IATA:      "WSG",
		Name:      "Washington County Airport",
		City:      "Washington",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1184,
		Latitude:  40.1365013123,
		Longitude: -80.2901992798,
		Timezone:  "America/New_York",
	},
	"KAFK": {
		ICAO:      "KAFK",
		Name:      "Nebraska City Municipal Airport",
		City:      "Nebraska City",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1165,
		Latitude:  40.60689926,
		Longitude: -95.86569977,
		Timezone:  "America/Chicago",
	},
	"KAFN": {
		ICAO:      "KAFN",
		IATA:      "AFN",
		Name:      "Jaffrey Airport Silver Ranch Airport",
		City:      "Jaffrey",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 1040,
		Latitude:  42.8050994873,
		Longitude: -72.0029983521,
		Timezone:  "America/New_York",
	},
	"KAFO": {
		ICAO:      "KAFO",
		IATA:      "AFO",
		Name:      "Afton Municipal Airport",
		City:      "Afton",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 6221,
		Latitude:  42.7112007141,
		Longitude: -110.942001343,
		Timezone:  "America/Denver",
	},
	"KAFP": {
		ICAO:      "KAFP",
		Name:      "Anson County Airport",
		City:      "Wadesboro",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 300,
		Latitude:  35.0205993652,
		Longitude: -80.0771026611,
		Timezone:  "America/New_York",
	},
	"KAFW": {
		ICAO:      "KAFW",
		IATA:      "AFW",
		Name:      "Fort Worth Alliance Airport",
		City:      "Fort Worth",
		State:     "Texas",
		Country:   "US",
		Elevation: 722,
		Latitude:  32.9875984192,
		Longitude: -97.3188018799,
		Timezone:  "America/Chicago",
	},
	"KAGC": {
		ICAO:      "KAGC",
		IATA:      "AGC",
		Name:      "Allegheny County Airport",
		City:      "Pittsburgh",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1252,
		Latitude:  40.3544006348,
		Longitude: -79.9301986694,
		Timezone:  "America/New_York",
	},
	"KAGO": {
		ICAO:      "KAGO",
		IATA:      "AGO",
		Name:      "Magnolia Municipal Airport",
		City:      "Magnolia",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 319,
		Latitude:  33.22800064,
		Longitude: -93.21700287,
		Timezone:  "America/Chicago",
	},
	"KAGR": {
		ICAO:      "KAGR",
		Name:      "MacDill Air Force Base Auxiliary Field",
		City:      "Avon Park",
		State:     "Florida",
		Country:   "US",
		Elevation: 68,
		Latitude:  27.6506004333,
		Longitude: -81.3494033813,
		Timezone:  "America/New_York",
	},
	"KAGS": {
		ICAO:      "KAGS",
		IATA:      "AGS",
		Name:      "Augusta Regional At Bush Field",
		City:      "Augusta",
		State:     "Georgia",
		Country:   "US",
		Elevation: 144,
		Latitude:  33.3698997498,
		Longitude: -81.9645004272,
		Timezone:  "America/New_York",
	},
	"KAGZ": {
		ICAO:      "KAGZ",
		Name:      "Wagner Municipal Airport",
		City:      "Wagner",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1475,
		Latitude:  43.0633010864,
		Longitude: -98.2962036133,
		Timezone:  "America/Chicago",
	},
	"KAHC": {
		ICAO:      "KAHC",
		IATA:      "AHC",
		Name:      "Amedee Army Air Field",
		City:      "Herlong",
		State:     "California",
		Country:   "US",
		Elevation: 4012,
		Latitude:  40.26620102,
		Longitude: -120.1529999,
		Timezone:  "America/Los_Angeles",
	},
	"KAHH": {
		ICAO:      "KAHH",
		IATA:      "AHH",
		Name:      "Amery Municipal Airport",
		City:      "Amery",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1088,
		Latitude:  45.2811012268,
		Longitude: -92.3753967285,
		Timezone:  "America/Chicago",
	},
	"KAHN": {
		ICAO:      "KAHN",
		IATA:      "AHN",
		Name:      "Athens Ben Epps Airport",
		City:      "Athens",
		State:     "Georgia",
		Country:   "US",
		Elevation: 808,
		Latitude:  33.948600769,
		Longitude: -83.3263015747,
		Timezone:  "America/New_York",
	},
	"KAHQ": {
		ICAO:      "KAHQ",
		Name:      "Wahoo Municipal Airport",
		City:      "Wahoo",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1224,
		Latitude:  41.2412986755,
		Longitude: -96.59400177,
		Timezone:  "America/Chicago",
	},
	"KAIA": {
		ICAO:      "KAIA",
		IATA:      "AIA",
		Name:      "Alliance Municipal Airport",
		City:      "Alliance",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3931,
		Latitude:  42.0531997681,
		Longitude: -102.804000854,
		Timezone:  "America/Denver",
	},
	"KAIB": {
		ICAO:      "KAIB",
		Name:      "Hopkins Field",
		City:      "Nucla",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5940,
		Latitude:  38.2388000488,
		Longitude: -108.56300354,
		Timezone:  "America/Denver",
	},
	"KAID": {
		ICAO:      "KAID",
		IATA:      "AID",
		Name:      "Anderson Municipal Darlington Field",
		City:      "Anderson",
		State:     "Indiana",
		Country:   "US",
		Elevation: 919,
		Latitude:  40.1086006165,
		Longitude: -85.6129989624,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KAIG": {
		ICAO:      "KAIG",
		Name:      "Langlade County Airport",
		City:      "Antigo",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1521,
		Latitude:  45.1542015076,
		Longitude: -89.1107025146,
		Timezone:  "America/Chicago",
	},
	"KAIK": {
		ICAO:      "KAIK",
		IATA:      "AIK",
		Name:      "Aiken Municipal Airport",
		City:      "Aiken",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 528,
		Latitude:  33.6493988037,
		Longitude: -81.6849975586,
		Timezone:  "America/New_York",
	},
	"KAIO": {
		ICAO:      "KAIO",
		IATA:      "AIO",
		Name:      "Atlantic Municipal Airport",
		City:      "Atlantic",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1165,
		Latitude:  41.40729904,
		Longitude: -95.04689789,
		Timezone:  "America/Chicago",
	},
	"KAIT": {
		ICAO:      "KAIT",
		Name:      "Aitkin Municipal Steve Kurtz Field",
		City:      "Aitkin",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1205,
		Latitude:  46.5484008789,
		Longitude: -93.6768035889,
		Timezone:  "America/Chicago",
	},
	"KAIV": {
		ICAO:      "KAIV",
		IATA:      "AIV",
		Name:      "George Downer Airport",
		City:      "Aliceville",
		State:     "Alabama",
		Country:   "US",
		Elevation: 150,
		Latitude:  33.1064987183,
		Longitude: -88.1977996826,
		Timezone:  "America/Chicago",
	},
	"KAIZ": {
		ICAO:      "KAIZ",
		IATA:      "AIZ",
		Name:      "Lee C Fine Memorial Airport",
		City:      "Kaiser Lake Ozark",
		State:     "Missouri",
		Country:   "US",
		Elevation: 869,
		Latitude:  38.0960006714,
		Longitude: -92.5494995117,
		Timezone:  "America/Chicago",
	},
	"KAJG": {
		ICAO:      "KAJG",
		Name:      "Mount Carmel Municipal Airport",
		City:      "Mount Carmel",
		State:     "Illinois",
		Country:   "US",
		Elevation: 429,
		Latitude:  38.60649872,
		Longitude: -87.72669983,
		Timezone:  "America/Chicago",
	},
	"KAJO": {
		ICAO:      "KAJO",
		Name:      "Corona Municipal Airport",
		City:      "Corona",
		State:     "California",
		Country:   "US",
		Elevation: 533,
		Latitude:  33.8977012634,
		Longitude: -117.6019973755,
		Timezone:  "America/Los_Angeles",
	},
	"KAJR": {
		ICAO:      "KAJR",
		Name:      "Habersham County Airport",
		City:      "Cornelia",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1448,
		Latitude:  34.49990082,
		Longitude: -83.55670166,
		Timezone:  "America/New_York",
	},
	"KAJZ": {
		ICAO:      "KAJZ",
		Name:      "Blake Field",
		City:      "Delta",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5193,
		Latitude:  38.7863998413,
		Longitude: -108.0640029907,
		Timezone:  "America/Denver",
	},
	"KAKH": {
		ICAO:      "KAKH",
		Name:      "Gastonia Municipal Airport",
		City:      "Gastonia",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 798,
		Latitude:  35.2025985718,
		Longitude: -81.1499023438,
		Timezone:  "America/New_York",
	},
	"KAKO": {
		ICAO:      "KAKO",
		IATA:      "AKO",
		Name:      "Colorado Plains Regional Airport",
		City:      "Akron",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4714,
		Latitude:  40.1755981445,
		Longitude: -103.222000122,
		Timezone:  "America/Denver",
	},
	"KAKQ": {
		ICAO:      "KAKQ",
		Name:      "Wakefield Municipal Airport",
		City:      "Wakefield",
		State:     "Virginia",
		Country:   "US",
		Elevation: 113,
		Latitude:  36.9872016907,
		Longitude: -77.0010986328,
		Timezone:  "America/New_York",
	},
	"KAKR": {
		ICAO:      "KAKR",
		IATA:      "AKC",
		Name:      "Akron Fulton International Airport",
		City:      "Akron",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1067,
		Latitude:  41.0374984741,
		Longitude: -81.4669036865,
		Timezone:  "America/New_York",
	},
	"KALB": {
		ICAO:      "KALB",
		IATA:      "ALB",
		Name:      "Albany International Airport",
		City:      "Albany",
		State:     "New-York",
		Country:   "US",
		Elevation: 285,
		Latitude:  42.748298645,
		Longitude: -73.8016967773,
		Timezone:  "America/New_York",
	},
	"KALI": {
		ICAO:      "KALI",
		IATA:      "ALI",
		Name:      "Alice International Airport",
		City:      "Alice",
		State:     "Texas",
		Country:   "US",
		Elevation: 178,
		Latitude:  27.7409000397,
		Longitude: -98.0269012451,
		Timezone:  "America/Chicago",
	},
	"KALM": {
		ICAO:      "KALM",
		IATA:      "ALM",
		Name:      "Alamogordo White Sands Regional Airport",
		City:      "Alamogordo",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4200,
		Latitude:  32.8399009705,
		Longitude: -105.9909973145,
		Timezone:  "America/Denver",
	},
	"KALN": {
		ICAO:      "KALN",
		IATA:      "ALN",
		Name:      "St Louis Regional Airport",
		City:      "Alton/St Louis",
		State:     "Illinois",
		Country:   "US",
		Elevation: 544,
		Latitude:  38.8903007507,
		Longitude: -90.0459976196,
		Timezone:  "America/Chicago",
	},
	"KALO": {
		ICAO:      "KALO",
		IATA:      "ALO",
		Name:      "Waterloo Regional Airport",
		City:      "Waterloo",
		State:     "Iowa",
		Country:   "US",
		Elevation: 873,
		Latitude:  42.5570983887,
		Longitude: -92.4002990723,
		Timezone:  "America/Chicago",
	},
	"KALS": {
		ICAO:      "KALS",
		IATA:      "ALS",
		Name:      "San Luis Valley Regional Bergman Field",
		City:      "Alamosa",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7539,
		Latitude:  37.4348983765,
		Longitude: -105.866996765,
		Timezone:  "America/Denver",
	},
	"KALW": {
		ICAO:      "KALW",
		IATA:      "ALW",
		Name:      "Walla Walla Regional Airport",
		City:      "Walla Walla",
		State:     "Washington",
		Country:   "US",
		Elevation: 1194,
		Latitude:  46.09489822,
		Longitude: -118.288002,
		Timezone:  "America/Los_Angeles",
	},
	"KAMA": {
		ICAO:      "KAMA",
		IATA:      "AMA",
		Name:      "Rick Husband Amarillo International Airport",
		City:      "Amarillo",
		State:     "Texas",
		Country:   "US",
		Elevation: 3607,
		Latitude:  35.2193984985,
		Longitude: -101.7060012817,
		Timezone:  "America/Chicago",
	},
	"KAMG": {
		ICAO:      "KAMG",
		Name:      "Bacon County Airport",
		City:      "Alma",
		State:     "Georgia",
		Country:   "US",
		Elevation: 200,
		Latitude:  31.5361003876,
		Longitude: -82.5065994263,
		Timezone:  "America/New_York",
	},
	"KAMN": {
		ICAO:      "KAMN",
		IATA:      "AMN",
		Name:      "RPD Airport",
		City:      "Alma",
		State:     "Michigan",
		Country:   "US",
		Elevation: 754,
		Latitude:  43.322101593,
		Longitude: -84.68800354,
		Timezone:  "America/Detroit",
	},
	"KAMT": {
		ICAO:      "KAMT",
		Name:      "Alexander Salamon Airport",
		City:      "West Union",
		State:     "Ohio",
		Country:   "US",
		Elevation: 896,
		Latitude:  38.8515014648,
		Longitude: -83.5662994385,
		Timezone:  "America/New_York",
	},
	"KAMW": {
		ICAO:      "KAMW",
		IATA:      "AMW",
		Name:      "Ames Municipal Airport",
		City:      "Ames",
		State:     "Iowa",
		Country:   "US",
		Elevation: 956,
		Latitude:  41.99200058,
		Longitude: -93.62180328,
		Timezone:  "America/Chicago",
	},
	"KANB": {
		ICAO:      "KANB",
		IATA:      "ANB",
		Name:      "Anniston Metropolitan Airport",
		City:      "Anniston",
		State:     "Alabama",
		Country:   "US",
		Elevation: 612,
		Latitude:  33.58819962,
		Longitude: -85.85810089,
		Timezone:  "America/Chicago",
	},
	"KAND": {
		ICAO:      "KAND",
		IATA:      "AND",
		Name:      "Anderson Regional Airport",
		City:      "Anderson",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 782,
		Latitude:  34.4945983887,
		Longitude: -82.7093963623,
		Timezone:  "America/New_York",
	},
	"KANE": {
		ICAO:      "KANE",
		Name:      "Anoka County-Blaine Arpt(Janes Field) Airport",
		City:      "Minneapolis",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 912,
		Latitude:  45.14500046,
		Longitude: -93.21140289,
		Timezone:  "America/Chicago",
	},
	"KANJ": {
		ICAO:      "KANJ",
		Name:      "Sault Ste Marie Municipal Sanderson Field",
		City:      "Sault Ste Marie",
		State:     "Michigan",
		Country:   "US",
		Elevation: 716,
		Latitude:  46.47919846,
		Longitude: -84.36840057,
		Timezone:  "America/Detroit",
	},
	"KANK": {
		ICAO:      "KANK",
		IATA:      "SLT",
		Name:      "Harriet Alexander Field",
		City:      "Salida",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7523,
		Latitude:  38.5382995605,
		Longitude: -106.0490036011,
		Timezone:  "America/Denver",
	},
	"KANP": {
		ICAO:      "KANP",
		IATA:      "ANP",
		Name:      "Lee Airport",
		City:      "Annapolis",
		State:     "Maryland",
		Country:   "US",
		Elevation: 34,
		Latitude:  38.9429016113,
		Longitude: -76.568397522,
		Timezone:  "America/New_York",
	},
	"KANQ": {
		ICAO:      "KANQ",
		IATA:      "ANQ",
		Name:      "Tri State Steuben County Airport",
		City:      "Angola",
		State:     "Indiana",
		Country:   "US",
		Elevation: 995,
		Latitude:  41.6397018433,
		Longitude: -85.0835037231,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KANW": {
		ICAO:      "KANW",
		IATA:      "ANW",
		Name:      "Ainsworth Municipal Airport",
		City:      "Ainsworth",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2589,
		Latitude:  42.5792007446,
		Longitude: -99.9929962158,
		Timezone:  "America/Chicago",
	},
	"KANY": {
		ICAO:      "KANY",
		IATA:      "ANY",
		Name:      "Anthony Municipal Airport",
		City:      "Anthony",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1340,
		Latitude:  37.1585006714,
		Longitude: -98.0795974731,
		Timezone:  "America/Chicago",
	},
	"KAOC": {
		ICAO:      "KAOC",
		Name:      "Arco Butte County Airport",
		City:      "Arco",
		State:     "Idaho",
		Country:   "US",
		Elevation: 5332,
		Latitude:  43.6035003662,
		Longitude: -113.3339996338,
		Timezone:  "America/Boise",
	},
	"KAOH": {
		ICAO:      "KAOH",
		IATA:      "AOH",
		Name:      "Lima Allen County Airport",
		City:      "Lima",
		State:     "Ohio",
		Country:   "US",
		Elevation: 975,
		Latitude:  40.7069015503,
		Longitude: -84.0267028809,
		Timezone:  "America/New_York",
	},
	"KAOO": {
		ICAO:      "KAOO",
		IATA:      "AOO",
		Name:      "Altoona Blair County Airport",
		City:      "Altoona",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1503,
		Latitude:  40.29639816,
		Longitude: -78.31999969,
		Timezone:  "America/New_York",
	},
	"KAOV": {
		ICAO:      "KAOV",
		Name:      "Ava Bill Martin Memorial Airport",
		City:      "Ava",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1311,
		Latitude:  36.9719009399,
		Longitude: -92.6819000244,
		Timezone:  "America/Chicago",
	},
	"KAPA": {
		ICAO:      "KAPA",
		IATA:      "APA",
		Name:      "Centennial Airport",
		City:      "Denver",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5885,
		Latitude:  39.57009888,
		Longitude: -104.848999,
		Timezone:  "America/Denver",
	},
	"KAPC": {
		ICAO:      "KAPC",
		IATA:      "APC",
		Name:      "Napa County Airport",
		City:      "Napa",
		State:     "California",
		Country:   "US",
		Elevation: 35,
		Latitude:  38.2131996155,
		Longitude: -122.28099823,
		Timezone:  "America/Los_Angeles",
	},
	"KAPF": {
		ICAO:      "KAPF",
		IATA:      "APF",
		Name:      "Naples Municipal Airport",
		City:      "Naples",
		State:     "Florida",
		Country:   "US",
		Elevation: 8,
		Latitude:  26.1525993347,
		Longitude: -81.7752990723,
		Timezone:  "America/New_York",
	},
	"KAPG": {
		ICAO:      "KAPG",
		IATA:      "APG",
		Name:      "Phillips Army Air Field",
		City:      "Aberdeen Proving Grounds(Aberdeen)",
		State:     "Maryland",
		Country:   "US",
		Elevation: 57,
		Latitude:  39.46620178,
		Longitude: -76.16880035,
		Timezone:  "America/New_York",
	},
	"KAPH": {
		ICAO:      "KAPH",
		IATA:      "APH",
		Name:      "A P Hill Aaf (Fort A P Hill) Airport",
		City:      "Fort A. P. Hill",
		State:     "Virginia",
		Country:   "US",
		Elevation: 220,
		Latitude:  38.06890106,
		Longitude: -77.31829834,
		Timezone:  "America/New_York",
	},
	"KAPN": {
		ICAO:      "KAPN",
		IATA:      "APN",
		Name:      "Alpena County Regional Airport",
		City:      "Alpena",
		State:     "Michigan",
		Country:   "US",
		Elevation: 690,
		Latitude:  45.0780983,
		Longitude: -83.56030273,
		Timezone:  "America/Detroit",
	},
	"KAPT": {
		ICAO:      "KAPT",
		IATA:      "APT",
		Name:      "Marion County Brown Field",
		City:      "Jasper",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 641,
		Latitude:  35.0606994629,
		Longitude: -85.5852966309,
		Timezone:  "America/Chicago",
	},
	"KAPV": {
		ICAO:      "KAPV",
		IATA:      "APV",
		Name:      "Apple Valley Airport",
		City:      "Apple Valley",
		State:     "California",
		Country:   "US",
		Elevation: 3062,
		Latitude:  34.5752983093,
		Longitude: -117.185997009,
		Timezone:  "America/Los_Angeles",
	},
	"KAPY": {
		ICAO:      "KAPY",
		Name:      "Zapata County Airport",
		City:      "Zapata",
		State:     "Texas",
		Country:   "US",
		Elevation: 422,
		Latitude:  26.9687995911,
		Longitude: -99.2489013672,
		Timezone:  "America/Chicago",
	},
	"KAQO": {
		ICAO:      "KAQO",
		Name:      "Llano Municipal Airport",
		City:      "Llano",
		State:     "Texas",
		Country:   "US",
		Elevation: 1102,
		Latitude:  30.783700943,
		Longitude: -98.6620025635,
		Timezone:  "America/Chicago",
	},
	"KAQP": {
		ICAO:      "KAQP",
		Name:      "Appleton Municipal Airport",
		City:      "Appleton",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1020,
		Latitude:  45.2275009155,
		Longitude: -96.0043029785,
		Timezone:  "America/Chicago",
	},
	"KAQR": {
		ICAO:      "KAQR",
		Name:      "Atoka Municipal Airport",
		City:      "Atoka",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 590,
		Latitude:  34.3983001709,
		Longitude: -96.1481018066,
		Timezone:  "America/Chicago",
	},
	"KAQW": {
		ICAO:      "KAQW",
		Name:      "Harriman and West Airport",
		City:      "North Adams",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 654,
		Latitude:  42.6958999634,
		Longitude: -73.1704025269,
		Timezone:  "America/New_York",
	},
	"KARA": {
		ICAO:      "KARA",
		IATA:      "ARA",
		Name:      "Acadiana Regional Airport",
		City:      "New Iberia",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 24,
		Latitude:  30.0377998352,
		Longitude: -91.8839035034,
		Timezone:  "America/Chicago",
	},
	"KARB": {
		ICAO:      "KARB",
		IATA:      "ARB",
		Name:      "Ann Arbor Municipal Airport",
		City:      "Ann Arbor",
		State:     "Michigan",
		Country:   "US",
		Elevation: 839,
		Latitude:  42.2229995728,
		Longitude: -83.7455978394,
		Timezone:  "America/Detroit",
	},
	"KARG": {
		ICAO:      "KARG",
		IATA:      "ARG",
		Name:      "Walnut Ridge Regional Airport",
		City:      "Walnut Ridge",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 279,
		Latitude:  36.1246667,
		Longitude: -90.9251111,
		Timezone:  "America/Chicago",
	},
	"KARM": {
		ICAO:      "KARM",
		IATA:      "WHT",
		Name:      "Wharton Regional Airport",
		City:      "Wharton",
		State:     "Texas",
		Country:   "US",
		Elevation: 100,
		Latitude:  29.2542991638,
		Longitude: -96.1544036865,
		Timezone:  "America/Chicago",
	},
	"KARR": {
		ICAO:      "KARR",
		IATA:      "AUZ",
		Name:      "Aurora Municipal Airport",
		City:      "Chicago/Aurora",
		State:     "Illinois",
		Country:   "US",
		Elevation: 712,
		Latitude:  41.771900177,
		Longitude: -88.4757003784,
		Timezone:  "America/Chicago",
	},
	"KART": {
		ICAO:      "KART",
		IATA:      "ART",
		Name:      "Watertown International Airport",
		City:      "Watertown",
		State:     "New-York",
		Country:   "US",
		Elevation: 325,
		Latitude:  43.9919013977,
		Longitude: -76.021697998,
		Timezone:  "America/New_York",
	},
	"KARV": {
		ICAO:      "KARV",
		IATA:      "ARV",
		Name:      "Lakeland-Noble F. Lee Memorial field",
		City:      "Minocqua-Woodruff",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1629,
		Latitude:  45.92789841,
		Longitude: -89.73090363,
		Timezone:  "America/Chicago",
	},
	"KARW": {
		ICAO:      "KARW",
		IATA:      "BFT",
		Name:      "Beaufort County Airport",
		City:      "Beaufort",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 10,
		Latitude:  32.4122009277,
		Longitude: -80.6343994141,
		Timezone:  "America/New_York",
	},
	"KASD": {
		ICAO:      "KASD",
		Name:      "Slidell Airport",
		City:      "Slidell",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 29,
		Latitude:  30.3451004,
		Longitude: -89.82080078,
		Timezone:  "America/Chicago",
	},
	"KASE": {
		ICAO:      "KASE",
		IATA:      "ASE",
		Name:      "Aspen-Pitkin Co/Sardy Field",
		City:      "Aspen",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7820,
		Latitude:  39.22320175,
		Longitude: -106.8690033,
		Timezone:  "America/Denver",
	},
	"KASG": {
		ICAO:      "KASG",
		IATA:      "SPZ",
		Name:      "Springdale Municipal Airport",
		City:      "Springdale",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1353,
		Latitude:  36.176399231,
		Longitude: -94.1193008423,
		Timezone:  "America/Chicago",
	},
	"KASH": {
		ICAO:      "KASH",
		IATA:      "ASH",
		Name:      "Boire Field",
		City:      "Nashua",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 199,
		Latitude:  42.7817001343,
		Longitude: -71.5148010254,
		Timezone:  "America/New_York",
	},
	"KASJ": {
		ICAO:      "KASJ",
		Name:      "Tri County Airport",
		City:      "Ahoskie",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 68,
		Latitude:  36.2975006104,
		Longitude: -77.1708984375,
		Timezone:  "America/New_York",
	},
	"KASL": {
		ICAO:      "KASL",
		IATA:      "ASL",
		Name:      "Harrison County Airport",
		City:      "Marshall",
		State:     "Texas",
		Country:   "US",
		Elevation: 357,
		Latitude:  32.5205001831,
		Longitude: -94.307800293,
		Timezone:  "America/Chicago",
	},
	"KASN": {
		ICAO:      "KASN",
		IATA:      "ASN",
		Name:      "Talladega Municipal Airport",
		City:      "Talladega",
		State:     "Alabama",
		Country:   "US",
		Elevation: 529,
		Latitude:  33.5699005127,
		Longitude: -86.0509033203,
		Timezone:  "America/Chicago",
	},
	"KAST": {
		ICAO:      "KAST",
		IATA:      "AST",
		Name:      "Astoria Regional Airport",
		City:      "Astoria",
		State:     "Oregon",
		Country:   "US",
		Elevation: 15,
		Latitude:  46.158000946,
		Longitude: -123.878997803,
		Timezone:  "America/Los_Angeles",
	},
	"KASW": {
		ICAO:      "KASW",
		Name:      "Warsaw Municipal Airport",
		City:      "Warsaw",
		State:     "Indiana",
		Country:   "US",
		Elevation: 850,
		Latitude:  41.2747001648,
		Longitude: -85.8401031494,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KASX": {
		ICAO:      "KASX",
		IATA:      "ASX",
		Name:      "John F Kennedy Memorial Airport",
		City:      "Ashland",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 827,
		Latitude:  46.54850006,
		Longitude: -90.91899872,
		Timezone:  "America/Chicago",
	},
	"KASY": {
		ICAO:      "KASY",
		IATA:      "ASY",
		Name:      "Ashley Municipal Airport",
		City:      "Ashley",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 2032,
		Latitude:  46.0238990784,
		Longitude: -99.3526000977,
		Timezone:  "America/Chicago",
	},
	"KATA": {
		ICAO:      "KATA",
		Name:      "Hall Miller Municipal Airport",
		City:      "Atlanta",
		State:     "Texas",
		Country:   "US",
		Elevation: 280,
		Latitude:  33.1017990112,
		Longitude: -94.1952972412,
		Timezone:  "America/Chicago",
	},
	"KATL": {
		ICAO:      "KATL",
		IATA:      "ATL",
		Name:      "Hartsfield Jackson Atlanta International Airport",
		City:      "Atlanta",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1026,
		Latitude:  33.6366996765,
		Longitude: -84.4281005859,
		Timezone:  "America/New_York",
	},
	"KATS": {
		ICAO:      "KATS",
		IATA:      "ATS",
		Name:      "Artesia Municipal Airport",
		City:      "Artesia",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 3541,
		Latitude:  32.8525009155,
		Longitude: -104.468002319,
		Timezone:  "America/Denver",
	},
	"KATW": {
		ICAO:      "KATW",
		IATA:      "ATW",
		Name:      "Appleton International Airport",
		City:      "Appleton",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 918,
		Latitude:  44.2580986023,
		Longitude: -88.5190963745,
		Timezone:  "America/Chicago",
	},
	"KATY": {
		ICAO:      "KATY",
		IATA:      "ATY",
		Name:      "Watertown Regional Airport",
		City:      "Watertown",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1749,
		Latitude:  44.91400146,
		Longitude: -97.15470123,
		Timezone:  "America/Chicago",
	},
	"KAUG": {
		ICAO:      "KAUG",
		IATA:      "AUG",
		Name:      "Augusta State Airport",
		City:      "Augusta",
		State:     "Maine",
		Country:   "US",
		Elevation: 352,
		Latitude:  44.3205986023,
		Longitude: -69.7973022461,
		Timezone:  "America/New_York",
	},
	"KAUH": {
		ICAO:      "KAUH",
		Name:      "Aurora Municipal Al Potter Field",
		City:      "Aurora",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1803,
		Latitude:  40.89410019,
		Longitude: -97.99459839,
		Timezone:  "America/Chicago",
	},
	"KAUM": {
		ICAO:      "KAUM",
		IATA:      "AUM",
		Name:      "Austin Municipal Airport",
		City:      "Austin",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1234,
		Latitude:  43.66500092,
		Longitude: -92.93340302,
		Timezone:  "America/Chicago",
	},
	"KAUN": {
		ICAO:      "KAUN",
		IATA:      "AUN",
		Name:      "Auburn Municipal Airport",
		City:      "Auburn",
		State:     "California",
		Country:   "US",
		Elevation: 1539,
		Latitude:  38.95479965,
		Longitude: -121.0820007,
		Timezone:  "America/Los_Angeles",
	},
	"KAUO": {
		ICAO:      "KAUO",
		IATA:      "AUO",
		Name:      "Auburn Opelika Robert G. Pitts Airport",
		City:      "Auburn",
		State:     "Alabama",
		Country:   "US",
		Elevation: 777,
		Latitude:  32.61510086,
		Longitude: -85.43399811,
		Timezone:  "America/Chicago",
	},
	"KAUS": {
		ICAO:      "KAUS",
		IATA:      "AUS",
		Name:      "Austin Bergstrom International Airport",
		City:      "Austin",
		State:     "Texas",
		Country:   "US",
		Elevation: 542,
		Latitude:  30.1944999695,
		Longitude: -97.6698989868,
		Timezone:  "America/Chicago",
	},
	"KAUW": {
		ICAO:      "KAUW",
		IATA:      "AUW",
		Name:      "Wausau Downtown Airport",
		City:      "Wausau",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1201,
		Latitude:  44.9262008667,
		Longitude: -89.6266021729,
		Timezone:  "America/Chicago",
	},
	"KAVC": {
		ICAO:      "KAVC",
		Name:      "Mecklenburg Brunswick Regional Airport",
		City:      "South Hill",
		State:     "Virginia",
		Country:   "US",
		Elevation: 441,
		Latitude:  36.68830109,
		Longitude: -78.05449677,
		Timezone:  "America/New_York",
	},
	"KAVK": {
		ICAO:      "KAVK",
		Name:      "Alva Regional Airport",
		City:      "Alva",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1474,
		Latitude:  36.7732009888,
		Longitude: -98.6698989868,
		Timezone:  "America/Chicago",
	},
	"KAVL": {
		ICAO:      "KAVL",
		IATA:      "AVL",
		Name:      "Asheville Regional Airport",
		City:      "Asheville",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 2165,
		Latitude:  35.4361991882,
		Longitude: -82.5418014526,
		Timezone:  "America/New_York",
	},
	"KAVO": {
		ICAO:      "KAVO",
		IATA:      "AVO",
		Name:      "Avon Park Executive Airport",
		City:      "Avon Park",
		State:     "Florida",
		Country:   "US",
		Elevation: 160,
		Latitude:  27.59119987,
		Longitude: -81.52780151,
		Timezone:  "America/New_York",
	},
	"KAVP": {
		ICAO:      "KAVP",
		IATA:      "AVP",
		Name:      "Wilkes Barre Scranton International Airport",
		City:      "Wilkes-Barre/Scranton",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 962,
		Latitude:  41.3385009766,
		Longitude: -75.7233963013,
		Timezone:  "America/New_York",
	},
	"KAVQ": {
		ICAO:      "KAVQ",
		IATA:      "AVW",
		Name:      "Marana Regional Airport",
		City:      "Tucson",
		State:     "Arizona",
		Country:   "US",
		Elevation: 2031,
		Latitude:  32.4095993042,
		Longitude: -111.218002319,
		Timezone:  "America/Phoenix",
	},
	"KAVX": {
		ICAO:      "KAVX",
		IATA:      "AVX",
		Name:      "Catalina Airport",
		City:      "Avalon",
		State:     "California",
		Country:   "US",
		Elevation: 1602,
		Latitude:  33.4048995972,
		Longitude: -118.4160003662,
		Timezone:  "America/Los_Angeles",
	},
	"KAWG": {
		ICAO:      "KAWG",
		Name:      "Washington Municipal Airport",
		City:      "Washington",
		State:     "Iowa",
		Country:   "US",
		Elevation: 754,
		Latitude:  41.2761001587,
		Longitude: -91.6734008789,
		Timezone:  "America/Chicago",
	},
	"KAWM": {
		ICAO:      "KAWM",
		IATA:      "AWM",
		Name:      "West Memphis Municipal Airport",
		City:      "West Memphis",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 212,
		Latitude:  35.1351013184,
		Longitude: -90.2343978882,
		Timezone:  "America/Chicago",
	},
	"KAWO": {
		ICAO:      "KAWO",
		Name:      "Arlington Municipal Airport",
		City:      "Arlington",
		State:     "Washington",
		Country:   "US",
		Elevation: 142,
		Latitude:  48.16070175,
		Longitude: -122.1589966,
		Timezone:  "America/Los_Angeles",
	},
	"KAXA": {
		ICAO:      "KAXA",
		IATA:      "AXG",
		Name:      "Algona Municipal Airport",
		City:      "Algona",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1219,
		Latitude:  43.0778999329,
		Longitude: -94.2720031738,
		Timezone:  "America/Chicago",
	},
	"KAXH": {
		ICAO:      "KAXH",
		Name:      "Houston Southwest Airport",
		City:      "Houston",
		State:     "Texas",
		Country:   "US",
		Elevation: 68,
		Latitude:  29.5060997009,
		Longitude: -95.4768981934,
		Timezone:  "America/Chicago",
	},
	"KAXN": {
		ICAO:      "KAXN",
		IATA:      "AXN",
		Name:      "Chandler Field",
		City:      "Alexandria",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1425,
		Latitude:  45.8662986755,
		Longitude: -95.3946990967,
		Timezone:  "America/Chicago",
	},
	"KAXQ": {
		ICAO:      "KAXQ",
		Name:      "Clarion County Airport",
		City:      "Clarion",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1458,
		Latitude:  41.2257995605,
		Longitude: -79.4410018921,
		Timezone:  "America/New_York",
	},
	"KAXS": {
		ICAO:      "KAXS",
		IATA:      "AXS",
		Name:      "Altus Quartz Mountain Regional Airport",
		City:      "Altus",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1433,
		Latitude:  34.6963996887,
		Longitude: -99.3403015137,
		Timezone:  "America/Chicago",
	},
	"KAXV": {
		ICAO:      "KAXV",
		IATA:      "AXV",
		Name:      "Neil Armstrong Airport",
		City:      "Wapakoneta",
		State:     "Ohio",
		Country:   "US",
		Elevation: 913,
		Latitude:  40.49340057,
		Longitude: -84.29889679,
		Timezone:  "America/New_York",
	},
	"KAXX": {
		ICAO:      "KAXX",
		IATA:      "AXX",
		Name:      "Angel Fire Airport",
		City:      "Angel Fire",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 8380,
		Latitude:  36.422000885,
		Longitude: -105.290000916,
		Timezone:  "America/Denver",
	},
	"KAYS": {
		ICAO:      "KAYS",
		IATA:      "AYS",
		Name:      "Waycross Ware County Airport",
		City:      "Waycross",
		State:     "Georgia",
		Country:   "US",
		Elevation: 142,
		Latitude:  31.2490997314,
		Longitude: -82.3955001831,
		Timezone:  "America/New_York",
	},
	"KAYX": {
		ICAO:      "KAYX",
		Name:      "Arnold Air Force Base",
		City:      "Tullahoma",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1067,
		Latitude:  35.39260101,
		Longitude: -86.08580017,
		Timezone:  "America/Chicago",
	},
	"KAZC": {
		ICAO:      "KAZC",
		Name:      "Colorado City Municipal Airport",
		City:      "Colorado City",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4874,
		Latitude:  36.9598999023,
		Longitude: -113.013999939,
		Timezone:  "America/Denver",
	},
	"KAZE": {
		ICAO:      "KAZE",
		Name:      "Hazlehurst Airport",
		City:      "Hazlehurst",
		State:     "Georgia",
		Country:   "US",
		Elevation: 255,
		Latitude:  31.8847007751,
		Longitude: -82.6473999023,
		Timezone:  "America/New_York",
	},
	"KAZO": {
		ICAO:      "KAZO",
		IATA:      "AZO",
		Name:      "Kalamazoo Battle Creek International Airport",
		City:      "Kalamazoo",
		State:     "Michigan",
		Country:   "US",
		Elevation: 874,
		Latitude:  42.2349014282,
		Longitude: -85.5521011353,
		Timezone:  "America/Detroit",
	},
	"KB16": {
		ICAO:      "KB16",
		Name:      "Whitfords Airport",
		City:      "Weedsport",
		State:     "New-York",
		Country:   "US",
		Elevation: 400,
		Latitude:  43.0802993774,
		Longitude: -76.5383987427,
		Timezone:  "America/New_York",
	},
	"KB19": {
		ICAO:      "KB19",
		Name:      "Biddeford Municipal Airport",
		City:      "Biddeford",
		State:     "Maine",
		Country:   "US",
		Elevation: 157,
		Latitude:  43.464099884,
		Longitude: -70.4723968506,
		Timezone:  "America/New_York",
	},
	"KB21": {
		ICAO:      "KB21",
		Name:      "Sugarloaf Regional Airport",
		City:      "Carrabassett",
		State:     "Maine",
		Country:   "US",
		Elevation: 885,
		Latitude:  45.0862007141,
		Longitude: -70.2162017822,
		Timezone:  "America/New_York",
	},
	"KBAB": {
		ICAO:      "KBAB",
		IATA:      "BAB",
		Name:      "Beale Air Force Base",
		City:      "Marysville",
		State:     "California",
		Country:   "US",
		Elevation: 113,
		Latitude:  39.136100769,
		Longitude: -121.43699646,
		Timezone:  "America/Los_Angeles",
	},
	"KBAD": {
		ICAO:      "KBAD",
		IATA:      "BAD",
		Name:      "Barksdale Air Force Base",
		City:      "Bossier City",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 166,
		Latitude:  32.5018005371,
		Longitude: -93.6626968384,
		Timezone:  "America/Chicago",
	},
	"KBAF": {
		ICAO:      "KBAF",
		IATA:      "BAF",
		Name:      "Barnes Municipal Airport",
		City:      "Westfield/Springfield",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 271,
		Latitude:  42.1577987671,
		Longitude: -72.7155990601,
		Timezone:  "America/New_York",
	},
	"KBAK": {
		ICAO:      "KBAK",
		IATA:      "CLU",
		Name:      "Columbus Municipal Airport",
		City:      "Columbus",
		State:     "Indiana",
		Country:   "US",
		Elevation: 656,
		Latitude:  39.2619018555,
		Longitude: -85.8963012695,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KBAM": {
		ICAO:      "KBAM",
		IATA:      "BAM",
		Name:      "Battle Mountain Airport",
		City:      "Battle Mountain",
		State:     "Nevada",
		Country:   "US",
		Elevation: 4532,
		Latitude:  40.5989990234,
		Longitude: -116.8740005493,
		Timezone:  "America/Los_Angeles",
	},
	"KBAX": {
		ICAO:      "KBAX",
		Name:      "Huron County Memorial Airport",
		City:      "Bad Axe",
		State:     "Michigan",
		Country:   "US",
		Elevation: 763,
		Latitude:  43.78020096,
		Longitude: -82.98539734,
		Timezone:  "America/Detroit",
	},
	"KBAZ": {
		ICAO:      "KBAZ",
		Name:      "New Braunfels Municipal Airport",
		City:      "New Braunfels",
		State:     "Texas",
		Country:   "US",
		Elevation: 651,
		Latitude:  29.7045001984,
		Longitude: -98.0421981812,
		Timezone:  "America/Chicago",
	},
	"KBBB": {
		ICAO:      "KBBB",
		IATA:      "BBB",
		Name:      "Benson Municipal Airport",
		City:      "Benson",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1039,
		Latitude:  45.3319015503,
		Longitude: -95.6505966187,
		Timezone:  "America/Chicago",
	},
	"KBBD": {
		ICAO:      "KBBD",
		IATA:      "BBD",
		Name:      "Curtis Field",
		City:      "Brady",
		State:     "Texas",
		Country:   "US",
		Elevation: 1827,
		Latitude:  31.1793003082,
		Longitude: -99.3238983154,
		Timezone:  "America/Chicago",
	},
	"KBBG": {
		ICAO:      "KBBG",
		IATA:      "BKG",
		Name:      "Branson Airport",
		City:      "Branson",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1302,
		Latitude:  36.532082,
		Longitude: -93.200544,
		Timezone:  "America/Chicago",
	},
	"KBBP": {
		ICAO:      "KBBP",
		IATA:      "BTN",
		Name:      "Marlboro County Jetport H.E. Avent Field",
		City:      "Bennettsville",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 147,
		Latitude:  34.62170029,
		Longitude: -79.73439789,
		Timezone:  "America/New_York",
	},
	"KBBW": {
		ICAO:      "KBBW",
		IATA:      "BBW",
		Name:      "Broken Bow Municipal Airport",
		City:      "Broken Bow",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2547,
		Latitude:  41.4365005493,
		Longitude: -99.6421966553,
		Timezone:  "America/Chicago",
	},
	"KBCB": {
		ICAO:      "KBCB",
		IATA:      "BCB",
		Name:      "Virginia Tech Montgomery Executive Airport",
		City:      "Blacksburg",
		State:     "Virginia",
		Country:   "US",
		Elevation: 2132,
		Latitude:  37.2075996399,
		Longitude: -80.4077987671,
		Timezone:  "America/New_York",
	},
	"KBCE": {
		ICAO:      "KBCE",
		IATA:      "BCE",
		Name:      "Bryce Canyon Airport",
		City:      "Bryce Canyon",
		State:     "Utah",
		Country:   "US",
		Elevation: 7590,
		Latitude:  37.706401825,
		Longitude: -112.144996643,
		Timezone:  "America/Denver",
	},
	"KBCK": {
		ICAO:      "KBCK",
		Name:      "Black River Falls Area Airport",
		City:      "Black River Falls",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 836,
		Latitude:  44.2507019043,
		Longitude: -90.8553009033,
		Timezone:  "America/Chicago",
	},
	"KBCT": {
		ICAO:      "KBCT",
		IATA:      "BCT",
		Name:      "Boca Raton Airport",
		City:      "Boca Raton",
		State:     "Florida",
		Country:   "US",
		Elevation: 13,
		Latitude:  26.3784999847,
		Longitude: -80.1076965332,
		Timezone:  "America/New_York",
	},
	"KBDE": {
		ICAO:      "KBDE",
		IATA:      "BDE",
		Name:      "Baudette International Airport",
		City:      "Baudette",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1086,
		Latitude:  48.7284011841,
		Longitude: -94.612197876,
		Timezone:  "America/Chicago",
	},
	"KBDG": {
		ICAO:      "KBDG",
		IATA:      "BDG",
		Name:      "Blanding Municipal Airport",
		City:      "Blanding",
		State:     "Utah",
		Country:   "US",
		Elevation: 5868,
		Latitude:  37.58330154,
		Longitude: -109.4830017,
		Timezone:  "America/Denver",
	},
	"KBDH": {
		ICAO:      "KBDH",
		IATA:      "ILL",
		Name:      "Willmar Municipal -John L Rice Field",
		City:      "Willmar",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1126,
		Latitude:  45.11769867,
		Longitude: -95.13040161,
		Timezone:  "America/Chicago",
	},
	"KBDJ": {
		ICAO:      "KBDJ",
		Name:      "Boulder Junction Airport",
		City:      "Boulder Junction",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1666,
		Latitude:  46.1374015808,
		Longitude: -89.6460037231,
		Timezone:  "America/Chicago",
	},
	"KBDL": {
		ICAO:      "KBDL",
		IATA:      "BDL",
		Name:      "Bradley International Airport",
		City:      "Hartford",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 173,
		Latitude:  41.9388999939,
		Longitude: -72.6831970215,
		Timezone:  "America/New_York",
	},
	"KBDN": {
		ICAO:      "KBDN",
		Name:      "Bend Municipal Airport",
		City:      "Bend",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3460,
		Latitude:  44.09479904,
		Longitude: -121.2009964,
		Timezone:  "America/Los_Angeles",
	},
	"KBDQ": {
		ICAO:      "KBDQ",
		Name:      "Morrilton Municipal Airport",
		City:      "Morrilton",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 321,
		Latitude:  35.1361999512,
		Longitude: -92.7135009766,
		Timezone:  "America/Chicago",
	},
	"KBDR": {
		ICAO:      "KBDR",
		IATA:      "BDR",
		Name:      "Igor I Sikorsky Memorial Airport",
		City:      "Bridgeport",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 9,
		Latitude:  41.1635017395,
		Longitude: -73.1261978149,
		Timezone:  "America/New_York",
	},
	"KBDU": {
		ICAO:      "KBDU",
		IATA:      "WBU",
		Name:      "Boulder Municipal Airport",
		City:      "Boulder",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5288,
		Latitude:  40.0393981934,
		Longitude: -105.225997925,
		Timezone:  "America/Denver",
	},
	"KBEA": {
		ICAO:      "KBEA",
		Name:      "Beeville Municipal Airport",
		City:      "Beeville",
		State:     "Texas",
		Country:   "US",
		Elevation: 268,
		Latitude:  28.3619003296,
		Longitude: -97.7910003662,
		Timezone:  "America/Chicago",
	},
	"KBEC": {
		ICAO:      "KBEC",
		IATA:      "BEC",
		Name:      "Beech Factory Airport",
		City:      "Wichita",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1408,
		Latitude:  37.6944999695,
		Longitude: -97.2149963379,
		Timezone:  "America/Chicago",
	},
	"KBED": {
		ICAO:      "KBED",
		IATA:      "BED",
		Name:      "Laurence G Hanscom Field",
		City:      "Bedford",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 133,
		Latitude:  42.47000122,
		Longitude: -71.28900146,
		Timezone:  "America/New_York",
	},
	"KBEH": {
		ICAO:      "KBEH",
		IATA:      "BEH",
		Name:      "Southwest Michigan Regional Airport",
		City:      "Benton Harbor",
		State:     "Michigan",
		Country:   "US",
		Elevation: 643,
		Latitude:  42.1286010742,
		Longitude: -86.4284973145,
		Timezone:  "America/Detroit",
	},
	"KBFA": {
		ICAO:      "KBFA",
		Name:      "Boyne Mountain Airport",
		City:      "Boyne Falls",
		State:     "Michigan",
		Country:   "US",
		Elevation: 719,
		Latitude:  45.1657981873,
		Longitude: -84.9241027832,
		Timezone:  "America/Detroit",
	},
	"KBFD": {
		ICAO:      "KBFD",
		IATA:      "BFD",
		Name:      "Bradford Regional Airport",
		City:      "Bradford",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 2143,
		Latitude:  41.8031005859,
		Longitude: -78.6400985718,
		Timezone:  "America/New_York",
	},
	"KBFE": {
		ICAO:      "KBFE",
		Name:      "Terry County Airport",
		City:      "Brownfield",
		State:     "Texas",
		Country:   "US",
		Elevation: 3264,
		Latitude:  33.1730995178,
		Longitude: -102.1930007935,
		Timezone:  "America/Chicago",
	},
	"KBFF": {
		ICAO:      "KBFF",
		IATA:      "BFF",
		Name:      "Western Neb. Rgnl/William B. Heilig Airport",
		City:      "Scottsbluff",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3967,
		Latitude:  41.87400055,
		Longitude: -103.5960007,
		Timezone:  "America/Denver",
	},
	"KBFI": {
		ICAO:      "KBFI",
		IATA:      "BFI",
		Name:      "Boeing Field King County International Airport",
		City:      "Seattle",
		State:     "Washington",
		Country:   "US",
		Elevation: 21,
		Latitude:  47.5299987793,
		Longitude: -122.3020019531,
		Timezone:  "America/Los_Angeles",
	},
	"KBFK": {
		ICAO:      "KBFK",
		Name:      "Buffalo Municipal Airport",
		City:      "Buffalo",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1822,
		Latitude:  36.8633003235,
		Longitude: -99.6186981201,
		Timezone:  "America/Chicago",
	},
	"KBFL": {
		ICAO:      "KBFL",
		IATA:      "BFL",
		Name:      "Meadows Field",
		City:      "Bakersfield",
		State:     "California",
		Country:   "US",
		Elevation: 510,
		Latitude:  35.43360138,
		Longitude: -119.0569992,
		Timezone:  "America/Los_Angeles",
	},
	"KBFM": {
		ICAO:      "KBFM",
		IATA:      "BFM",
		Name:      "Mobile Downtown Airport",
		City:      "Mobile",
		State:     "Alabama",
		Country:   "US",
		Elevation: 26,
		Latitude:  30.6268005371,
		Longitude: -88.0680999756,
		Timezone:  "America/Chicago",
	},
	"KBFR": {
		ICAO:      "KBFR",
		IATA:      "BFR",
		Name:      "Virgil I Grissom Municipal Airport",
		City:      "Bedford",
		State:     "Indiana",
		Country:   "US",
		Elevation: 727,
		Latitude:  38.84000015,
		Longitude: -86.44539642,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KBFW": {
		ICAO:      "KBFW",
		Name:      "Silver Bay Municipal Airport",
		City:      "Silver Bay",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1089,
		Latitude:  47.2490005493,
		Longitude: -91.4156036377,
		Timezone:  "America/Chicago",
	},
	"KBGD": {
		ICAO:      "KBGD",
		IATA:      "BGD",
		Name:      "Hutchinson County Airport",
		City:      "Borger",
		State:     "Texas",
		Country:   "US",
		Elevation: 3055,
		Latitude:  35.7009010315,
		Longitude: -101.393997192,
		Timezone:  "America/Chicago",
	},
	"KBGE": {
		ICAO:      "KBGE",
		IATA:      "BGE",
		Name:      "Decatur County Industrial Air Park",
		City:      "Bainbridge",
		State:     "Georgia",
		Country:   "US",
		Elevation: 141,
		Latitude:  30.9715004,
		Longitude: -84.63739777,
		Timezone:  "America/New_York",
	},
	"KBGF": {
		ICAO:      "KBGF",
		Name:      "Winchester Municipal Airport",
		City:      "Winchester",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 979,
		Latitude:  35.1775016785,
		Longitude: -86.0662002563,
		Timezone:  "America/Chicago",
	},
	"KBGM": {
		ICAO:      "KBGM",
		IATA:      "BGM",
		Name:      "Greater Binghamton/Edwin A Link field",
		City:      "Binghamton",
		State:     "New-York",
		Country:   "US",
		Elevation: 1636,
		Latitude:  42.20869827,
		Longitude: -75.97979736,
		Timezone:  "America/New_York",
	},
	"KBGR": {
		ICAO:      "KBGR",
		IATA:      "BGR",
		Name:      "Bangor International Airport",
		City:      "Bangor",
		State:     "Maine",
		Country:   "US",
		Elevation: 192,
		Latitude:  44.8073997498,
		Longitude: -68.8281021118,
		Timezone:  "America/New_York",
	},
	"KBHB": {
		ICAO:      "KBHB",
		IATA:      "BHB",
		Name:      "Hancock County-Bar Harbor Airport",
		City:      "Bar Harbor",
		State:     "Maine",
		Country:   "US",
		Elevation: 83,
		Latitude:  44.45000076,
		Longitude: -68.3615036,
		Timezone:  "America/New_York",
	},
	"KBHC": {
		ICAO:      "KBHC",
		Name:      "Baxley Municipal Airport",
		City:      "Baxley",
		State:     "Georgia",
		Country:   "US",
		Elevation: 201,
		Latitude:  31.7138004303,
		Longitude: -82.3937988281,
		Timezone:  "America/New_York",
	},
	"KBHK": {
		ICAO:      "KBHK",
		Name:      "Baker Municipal Airport",
		City:      "Baker",
		State:     "Montana",
		Country:   "US",
		Elevation: 2975,
		Latitude:  46.34759903,
		Longitude: -104.2590027,
		Timezone:  "America/Denver",
	},
	"KBHM": {
		ICAO:      "KBHM",
		IATA:      "BHM",
		Name:      "Birmingham-Shuttlesworth International Airport",
		City:      "Birmingham",
		State:     "Alabama",
		Country:   "US",
		Elevation: 650,
		Latitude:  33.56290054,
		Longitude: -86.75350189,
		Timezone:  "America/Chicago",
	},
	"KBID": {
		ICAO:      "KBID",
		IATA:      "BID",
		Name:      "Block Island State Airport",
		City:      "Block Island",
		State:     "Rhode-Island",
		Country:   "US",
		Elevation: 108,
		Latitude:  41.1680984497,
		Longitude: -71.577796936,
		Timezone:  "America/New_York",
	},
	"KBIE": {
		ICAO:      "KBIE",
		IATA:      "BIE",
		Name:      "Beatrice Municipal Airport",
		City:      "Beatrice",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1324,
		Latitude:  40.3013000488,
		Longitude: -96.7540969849,
		Timezone:  "America/Chicago",
	},
	"KBIF": {
		ICAO:      "KBIF",
		IATA:      "BIF",
		Name:      "Biggs Army Air Field (Fort Bliss)",
		City:      "Fort Bliss/El Paso",
		State:     "Texas",
		Country:   "US",
		Elevation: 3946,
		Latitude:  31.84950066,
		Longitude: -106.3799973,
		Timezone:  "America/Denver",
	},
	"KBIH": {
		ICAO:      "KBIH",
		IATA:      "BIH",
		Name:      "Eastern Sierra Regional Airport",
		City:      "Bishop",
		State:     "California",
		Country:   "US",
		Elevation: 4124,
		Latitude:  37.3731002808,
		Longitude: -118.363998413,
		Timezone:  "America/Los_Angeles",
	},
	"KBIJ": {
		ICAO:      "KBIJ",
		Name:      "Early County Airport",
		City:      "Blakely",
		State:     "Georgia",
		Country:   "US",
		Elevation: 214,
		Latitude:  31.3974990845,
		Longitude: -84.8947982788,
		Timezone:  "America/New_York",
	},
	"KBIL": {
		ICAO:      "KBIL",
		IATA:      "BIL",
		Name:      "Billings Logan International Airport",
		City:      "Billings",
		State:     "Montana",
		Country:   "US",
		Elevation: 3652,
		Latitude:  45.8077011108,
		Longitude: -108.5429992676,
		Timezone:  "America/Denver",
	},
	"KBIS": {
		ICAO:      "KBIS",
		IATA:      "BIS",
		Name:      "Bismarck Municipal Airport",
		City:      "Bismarck",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1661,
		Latitude:  46.7727012634,
		Longitude: -100.7460021973,
		Timezone:  "America/Chicago",
	},
	"KBIV": {
		ICAO:      "KBIV",
		Name:      "Tulip City Airport",
		City:      "Holland",
		State:     "Michigan",
		Country:   "US",
		Elevation: 698,
		Latitude:  42.7429008484,
		Longitude: -86.1073989868,
		Timezone:  "America/Detroit",
	},
	"KBIX": {
		ICAO:      "KBIX",
		IATA:      "BIX",
		Name:      "Keesler Air Force Base",
		City:      "Biloxi",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 33,
		Latitude:  30.4104003906,
		Longitude: -88.9244003296,
		Timezone:  "America/Chicago",
	},
	"KBJC": {
		ICAO:      "KBJC",
		IATA:      "BJC",
		Name:      "Rocky Mountain Metropolitan Airport",
		City:      "Denver",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5673,
		Latitude:  39.90879822,
		Longitude: -105.1169968,
		Timezone:  "America/Denver",
	},
	"KBJI": {
		ICAO:      "KBJI",
		IATA:      "BJI",
		Name:      "Bemidji Regional Airport",
		City:      "Bemidji",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1391,
		Latitude:  47.50939941,
		Longitude: -94.93370056,
		Timezone:  "America/Chicago",
	},
	"KBJJ": {
		ICAO:      "KBJJ",
		IATA:      "BJJ",
		Name:      "Wayne County Airport",
		City:      "Wooster",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1136,
		Latitude:  40.8748016357,
		Longitude: -81.8882980347,
		Timezone:  "America/New_York",
	},
	"KBKD": {
		ICAO:      "KBKD",
		IATA:      "BKD",
		Name:      "Stephens County Airport",
		City:      "Breckenridge",
		State:     "Texas",
		Country:   "US",
		Elevation: 1284,
		Latitude:  32.71900177,
		Longitude: -98.8909988403,
		Timezone:  "America/Chicago",
	},
	"KBKE": {
		ICAO:      "KBKE",
		IATA:      "BKE",
		Name:      "Baker City Municipal Airport",
		City:      "Baker City",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3373,
		Latitude:  44.8372993469,
		Longitude: -117.808998108,
		Timezone:  "America/Los_Angeles",
	},
	"KBKF": {
		ICAO:      "KBKF",
		IATA:      "BFK",
		Name:      "Buckley Air Force Base",
		City:      "Aurora",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5662,
		Latitude:  39.7016983032,
		Longitude: -104.751998901,
		Timezone:  "America/Denver",
	},
	"KBKL": {
		ICAO:      "KBKL",
		IATA:      "BKL",
		Name:      "Burke Lakefront Airport",
		City:      "Cleveland",
		State:     "Ohio",
		Country:   "US",
		Elevation: 583,
		Latitude:  41.5175018311,
		Longitude: -81.6832962036,
		Timezone:  "America/New_York",
	},
	"KBKN": {
		ICAO:      "KBKN",
		IATA:      "BWL",
		Name:      "Blackwell Tonkawa Municipal Airport",
		City:      "Blackwell",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1030,
		Latitude:  36.74509811,
		Longitude: -97.34960175,
		Timezone:  "America/Chicago",
	},
	"KBKS": {
		ICAO:      "KBKS",
		Name:      "Brooks County Airport",
		City:      "Falfurrias",
		State:     "Texas",
		Country:   "US",
		Elevation: 113,
		Latitude:  27.20680046,
		Longitude: -98.12120056,
		Timezone:  "America/Chicago",
	},
	"KBKT": {
		ICAO:      "KBKT",
		IATA:      "BKT",
		Name:      "Allen C Perkinson Blackstone Army Air Field",
		City:      "Blackstone",
		State:     "Virginia",
		Country:   "US",
		Elevation: 439,
		Latitude:  37.0741996765,
		Longitude: -77.9574966431,
		Timezone:  "America/New_York",
	},
	"KBKV": {
		ICAO:      "KBKV",
		Name:      "Hernando County Airport",
		City:      "Brooksville",
		State:     "Florida",
		Country:   "US",
		Elevation: 76,
		Latitude:  28.47360039,
		Longitude: -82.45539856,
		Timezone:  "America/New_York",
	},
	"KBKW": {
		ICAO:      "KBKW",
		IATA:      "BKW",
		Name:      "Raleigh County Memorial Airport",
		City:      "Beckley",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 2504,
		Latitude:  37.7873001099,
		Longitude: -81.1241989136,
		Timezone:  "America/New_York",
	},
	"KBKX": {
		ICAO:      "KBKX",
		IATA:      "BKX",
		Name:      "Brookings Regional Airport",
		City:      "Brookings",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1648,
		Latitude:  44.3047981262,
		Longitude: -96.8169021606,
		Timezone:  "America/Chicago",
	},
	"KBLF": {
		ICAO:      "KBLF",
		IATA:      "BLF",
		Name:      "Mercer County Airport",
		City:      "Bluefield",
		State:     "Illinois",
		Country:   "US",
		Elevation: 2857,
		Latitude:  37.2957992554,
		Longitude: -81.2077026367,
		Timezone:  "America/New_York",
	},
	"KBLH": {
		ICAO:      "KBLH",
		IATA:      "BLH",
		Name:      "Blythe Airport",
		City:      "Blythe",
		State:     "California",
		Country:   "US",
		Elevation: 399,
		Latitude:  33.6192016602,
		Longitude: -114.717002869,
		Timezone:  "America/Los_Angeles",
	},
	"KBLI": {
		ICAO:      "KBLI",
		IATA:      "BLI",
		Name:      "Bellingham International Airport",
		City:      "Bellingham",
		State:     "Washington",
		Country:   "US",
		Elevation: 170,
		Latitude:  48.7928009033,
		Longitude: -122.5380020142,
		Timezone:  "America/Los_Angeles",
	},
	"KBLM": {
		ICAO:      "KBLM",
		IATA:      "BLM",
		Name:      "Monmouth Executive Airport",
		City:      "Belmar/Farmingdale",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 153,
		Latitude:  40.18690109,
		Longitude: -74.12490082,
		Timezone:  "America/New_York",
	},
	"KBLU": {
		ICAO:      "KBLU",
		IATA:      "BLU",
		Name:      "Blue Canyon Nyack Airport",
		City:      "Emigrant Gap",
		State:     "California",
		Country:   "US",
		Elevation: 5284,
		Latitude:  39.2750015259,
		Longitude: -120.709999084,
		Timezone:  "America/Los_Angeles",
	},
	"KBLV": {
		ICAO:      "KBLV",
		IATA:      "BLV",
		Name:      "Scott AFB/Midamerica Airport",
		City:      "Belleville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 459,
		Latitude:  38.54520035,
		Longitude: -89.83519745,
		Timezone:  "America/Chicago",
	},
	"KBMC": {
		ICAO:      "KBMC",
		IATA:      "BMC",
		Name:      "Brigham City Airport",
		City:      "Brigham City",
		State:     "Utah",
		Country:   "US",
		Elevation: 4229,
		Latitude:  41.5523986816,
		Longitude: -112.06199646,
		Timezone:  "America/Denver",
	},
	"KBMG": {
		ICAO:      "KBMG",
		IATA:      "BMG",
		Name:      "Monroe County Airport",
		City:      "Bloomington",
		State:     "Alabama",
		Country:   "US",
		Elevation: 846,
		Latitude:  39.1459999084,
		Longitude: -86.6166992188,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KBMI": {
		ICAO:      "KBMI",
		IATA:      "BMI",
		Name:      "Central Illinois Regional Airport at Bloomington-Normal",
		City:      "Bloomington-Normal",
		State:     "Illinois",
		Country:   "US",
		Elevation: 871,
		Latitude:  40.47710037,
		Longitude: -88.91590118,
		Timezone:  "America/Chicago",
	},
	"KBML": {
		ICAO:      "KBML",
		IATA:      "BML",
		Name:      "Berlin Regional Airport",
		City:      "Berlin",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 1161,
		Latitude:  44.57540131,
		Longitude: -71.17590332,
		Timezone:  "America/New_York",
	},
	"KBMQ": {
		ICAO:      "KBMQ",
		Name:      "Burnet Municipal Kate Craddock Field",
		City:      "Burnet",
		State:     "Texas",
		Country:   "US",
		Elevation: 1284,
		Latitude:  30.738899231,
		Longitude: -98.2386016846,
		Timezone:  "America/Chicago",
	},
	"KBMT": {
		ICAO:      "KBMT",
		IATA:      "BMT",
		Name:      "Beaumont Municipal Airport",
		City:      "Beaumont",
		State:     "Texas",
		Country:   "US",
		Elevation: 32,
		Latitude:  30.0706996918,
		Longitude: -94.2157974243,
		Timezone:  "America/Chicago",
	},
	"KBNA": {
		ICAO:      "KBNA",
		IATA:      "BNA",
		Name:      "Nashville International Airport",
		City:      "Nashville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 599,
		Latitude:  36.1245002747,
		Longitude: -86.6781997681,
		Timezone:  "America/Chicago",
	},
	"KBNG": {
		ICAO:      "KBNG",
		IATA:      "BNG",
		Name:      "Banning Municipal Airport",
		City:      "Banning",
		State:     "California",
		Country:   "US",
		Elevation: 2219,
		Latitude:  33.9230995178,
		Longitude: -116.850997925,
		Timezone:  "America/Los_Angeles",
	},
	"KBNL": {
		ICAO:      "KBNL",
		IATA:      "BNL",
		Name:      "Barnwell Regional Airport",
		City:      "Barnwell",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 246,
		Latitude:  33.25780106,
		Longitude: -81.38829803,
		Timezone:  "America/New_York",
	},
	"KBNO": {
		ICAO:      "KBNO",
		IATA:      "BNO",
		Name:      "Burns Municipal Airport",
		City:      "Burns",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4148,
		Latitude:  43.5918998718,
		Longitude: -118.955001831,
		Timezone:  "America/Los_Angeles",
	},
	"KBNW": {
		ICAO:      "KBNW",
		IATA:      "BNW",
		Name:      "Boone Municipal Airport",
		City:      "Boone",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1160,
		Latitude:  42.0495986938,
		Longitude: -93.8476028442,
		Timezone:  "America/Chicago",
	},
	"KBOI": {
		ICAO:      "KBOI",
		IATA:      "BOI",
		Name:      "Boise Air Terminal/Gowen field",
		City:      "Boise",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2871,
		Latitude:  43.56439972,
		Longitude: -116.2229996,
		Timezone:  "America/Boise",
	},
	"KBOS": {
		ICAO:      "KBOS",
		IATA:      "BOS",
		Name:      "General Edward Lawrence Logan International Airport",
		City:      "Boston",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 20,
		Latitude:  42.36429977,
		Longitude: -71.00520325,
		Timezone:  "America/New_York",
	},
	"KBOW": {
		ICAO:      "KBOW",
		IATA:      "BOW",
		Name:      "Bartow Municipal Airport",
		City:      "Bartow",
		State:     "Florida",
		Country:   "US",
		Elevation: 125,
		Latitude:  27.9433994293,
		Longitude: -81.7834014893,
		Timezone:  "America/New_York",
	},
	"KBPG": {
		ICAO:      "KBPG",
		IATA:      "BGS",
		Name:      "Big Spring Mc Mahon-Wrinkle Airport",
		City:      "Big Spring",
		State:     "Texas",
		Country:   "US",
		Elevation: 2573,
		Latitude:  32.21260071,
		Longitude: -101.5220032,
		Timezone:  "America/Chicago",
	},
	"KBPI": {
		ICAO:      "KBPI",
		IATA:      "BPI",
		Name:      "Miley Memorial Field",
		City:      "Big Piney",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 6990,
		Latitude:  42.58509827,
		Longitude: -110.1110001,
		Timezone:  "America/Denver",
	},
	"KBPK": {
		ICAO:      "KBPK",
		IATA:      "WMH",
		Name:      "Ozark Regional Airport",
		City:      "Mountain Home",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 928,
		Latitude:  36.3689002991,
		Longitude: -92.4704971313,
		Timezone:  "America/Chicago",
	},
	"KBPP": {
		ICAO:      "KBPP",
		IATA:      "BWM",
		Name:      "Bowman Municipal Airport",
		City:      "Bowman",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 2958,
		Latitude:  46.1870002747,
		Longitude: -103.428001404,
		Timezone:  "America/Denver",
	},
	"KBPT": {
		ICAO:      "KBPT",
		IATA:      "BPT",
		Name:      "Southeast Texas Regional Airport",
		City:      "Beaumont/Port Arthur",
		State:     "Texas",
		Country:   "US",
		Elevation: 15,
		Latitude:  29.950799942,
		Longitude: -94.0206985474,
		Timezone:  "America/Chicago",
	},
	"KBQK": {
		ICAO:      "KBQK",
		IATA:      "BQK",
		Name:      "Brunswick Golden Isles Airport",
		City:      "Brunswick",
		State:     "Georgia",
		Country:   "US",
		Elevation: 26,
		Latitude:  31.2588005066,
		Longitude: -81.4664993286,
		Timezone:  "America/New_York",
	},
	"KBQP": {
		ICAO:      "KBQP",
		Name:      "Morehouse Memorial Airport",
		City:      "Bastrop",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 168,
		Latitude:  32.75510025,
		Longitude: -91.88189697,
		Timezone:  "America/Chicago",
	},
	"KBQR": {
		ICAO:      "KBQR",
		Name:      "Buffalo Lancaster Regional Airport",
		City:      "Lancaster",
		State:     "New-York",
		Country:   "US",
		Elevation: 750,
		Latitude:  42.92229843,
		Longitude: -78.61229706,
		Timezone:  "America/New_York",
	},
	"KBRD": {
		ICAO:      "KBRD",
		IATA:      "BRD",
		Name:      "Brainerd Lakes Regional Airport",
		City:      "Brainerd",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1232,
		Latitude:  46.39830017,
		Longitude: -94.13809967,
		Timezone:  "America/Chicago",
	},
	"KBRL": {
		ICAO:      "KBRL",
		IATA:      "BRL",
		Name:      "Southeast Iowa Regional Airport",
		City:      "Burlington",
		State:     "Iowa",
		Country:   "US",
		Elevation: 698,
		Latitude:  40.7831993103,
		Longitude: -91.12550354,
		Timezone:  "America/Chicago",
	},
	"KBRO": {
		ICAO:      "KBRO",
		IATA:      "BRO",
		Name:      "Brownsville South Padre Island International Airport",
		City:      "Brownsville",
		State:     "Texas",
		Country:   "US",
		Elevation: 22,
		Latitude:  25.9067993164,
		Longitude: -97.4259033203,
		Timezone:  "America/Chicago",
	},
	"KBRY": {
		ICAO:      "KBRY",
		IATA:      "BRY",
		Name:      "Samuels Field",
		City:      "Bardstown",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 669,
		Latitude:  37.8143005371,
		Longitude: -85.4996032715,
		Timezone:  "America/New_York",
	},
	"KBST": {
		ICAO:      "KBST",
		Name:      "Belfast Municipal Airport",
		City:      "Belfast",
		State:     "Maine",
		Country:   "US",
		Elevation: 198,
		Latitude:  44.4094009399,
		Longitude: -69.0119018555,
		Timezone:  "America/New_York",
	},
	"KBTA": {
		ICAO:      "KBTA",
		Name:      "Blair Municipal Airport",
		City:      "Blair",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1325,
		Latitude:  41.4180984497,
		Longitude: -96.1136016846,
		Timezone:  "America/Chicago",
	},
	"KBTF": {
		ICAO:      "KBTF",
		IATA:      "BTF",
		Name:      "Skypark Airport",
		City:      "Bountiful",
		State:     "Utah",
		Country:   "US",
		Elevation: 4234,
		Latitude:  40.8694000244,
		Longitude: -111.927001953,
		Timezone:  "America/Denver",
	},
	"KBTL": {
		ICAO:      "KBTL",
		IATA:      "BTL",
		Name:      "W K Kellogg Airport",
		City:      "Battle Creek",
		State:     "Michigan",
		Country:   "US",
		Elevation: 952,
		Latitude:  42.3073005676,
		Longitude: -85.2515029907,
		Timezone:  "America/Detroit",
	},
	"KBTM": {
		ICAO:      "KBTM",
		IATA:      "BTM",
		Name:      "Bert Mooney Airport",
		City:      "Butte",
		State:     "Montana",
		Country:   "US",
		Elevation: 5550,
		Latitude:  45.9547996521,
		Longitude: -112.4970016479,
		Timezone:  "America/Denver",
	},
	"KBTN": {
		ICAO:      "KBTN",
		IATA:      "TTO",
		Name:      "Britton Municipal Airport",
		City:      "Britton",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1318,
		Latitude:  45.8152008057,
		Longitude: -97.7431030273,
		Timezone:  "America/Chicago",
	},
	"KBTP": {
		ICAO:      "KBTP",
		IATA:      "BTP",
		Name:      "Butler County-K W Scholter Field",
		City:      "Butler",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1248,
		Latitude:  40.77690125,
		Longitude: -79.9496994,
		Timezone:  "America/New_York",
	},
	"KBTR": {
		ICAO:      "KBTR",
		IATA:      "BTR",
		Name:      "Baton Rouge Metropolitan Ryan Field",
		City:      "Baton Rouge",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 70,
		Latitude:  30.53319931,
		Longitude: -91.14959717,
		Timezone:  "America/Chicago",
	},
	"KBTV": {
		ICAO:      "KBTV",
		IATA:      "BTV",
		Name:      "Burlington International Airport",
		City:      "Burlington",
		State:     "Vermont",
		Country:   "US",
		Elevation: 335,
		Latitude:  44.4719009399,
		Longitude: -73.1532974243,
		Timezone:  "America/New_York",
	},
	"KBTY": {
		ICAO:      "KBTY",
		IATA:      "BTY",
		Name:      "Beatty Airport",
		City:      "Beatty",
		State:     "Nevada",
		Country:   "US",
		Elevation: 3170,
		Latitude:  36.8610992432,
		Longitude: -116.787002563,
		Timezone:  "America/Los_Angeles",
	},
	"KBUB": {
		ICAO:      "KBUB",
		IATA:      "BUB",
		Name:      "Cram Field",
		City:      "Burwell",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2182,
		Latitude:  41.7766990662,
		Longitude: -99.1496963501,
		Timezone:  "America/Chicago",
	},
	"KBUF": {
		ICAO:      "KBUF",
		IATA:      "BUF",
		Name:      "Buffalo Niagara International Airport",
		City:      "Buffalo",
		State:     "New-York",
		Country:   "US",
		Elevation: 728,
		Latitude:  42.94049835,
		Longitude: -78.73220062,
		Timezone:  "America/New_York",
	},
	"KBUM": {
		ICAO:      "KBUM",
		IATA:      "BUM",
		Name:      "Butler Memorial Airport",
		City:      "Butler",
		State:     "Missouri",
		Country:   "US",
		Elevation: 892,
		Latitude:  38.2897987366,
		Longitude: -94.3401031494,
		Timezone:  "America/Chicago",
	},
	"KBUR": {
		ICAO:      "KBUR",
		IATA:      "BUR",
		Name:      "Bob Hope Airport",
		City:      "Burbank",
		State:     "California",
		Country:   "US",
		Elevation: 778,
		Latitude:  34.2006988525,
		Longitude: -118.3590011597,
		Timezone:  "America/Los_Angeles",
	},
	"KBUU": {
		ICAO:      "KBUU",
		Name:      "Burlington Municipal Airport",
		City:      "Burlington",
		State:     "Iowa",
		Country:   "US",
		Elevation: 779,
		Latitude:  42.690700531,
		Longitude: -88.3046035767,
		Timezone:  "America/Chicago",
	},
	"KBUY": {
		ICAO:      "KBUY",
		Name:      "Burlington Alamance Regional Airport",
		City:      "Burlington",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 617,
		Latitude:  36.048500061,
		Longitude: -79.474899292,
		Timezone:  "America/New_York",
	},
	"KBVI": {
		ICAO:      "KBVI",
		IATA:      "BFP",
		Name:      "Beaver County Airport",
		City:      "Beaver Falls",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1253,
		Latitude:  40.7724990845,
		Longitude: -80.3914031982,
		Timezone:  "America/New_York",
	},
	"KBVN": {
		ICAO:      "KBVN",
		Name:      "Albion Municipal Airport",
		City:      "Albion",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1806,
		Latitude:  41.7285995483,
		Longitude: -98.0558013916,
		Timezone:  "America/Chicago",
	},
	"KBVO": {
		ICAO:      "KBVO",
		IATA:      "BVO",
		Name:      "Bartlesville Municipal Airport",
		City:      "Bartlesville",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 711,
		Latitude:  36.76250076,
		Longitude: -96.01119995,
		Timezone:  "America/Chicago",
	},
	"KBVS": {
		ICAO:      "KBVS",
		IATA:      "MVW",
		Name:      "Skagit Regional Airport",
		City:      "Burlington/Mount Vernon",
		State:     "Washington",
		Country:   "US",
		Elevation: 144,
		Latitude:  48.4709014893,
		Longitude: -122.4209976196,
		Timezone:  "America/Los_Angeles",
	},
	"KBVU": {
		ICAO:      "KBVU",
		IATA:      "BLD",
		Name:      "Boulder City Municipal Airport",
		City:      "Boulder City",
		State:     "Nevada",
		Country:   "US",
		Elevation: 2201,
		Latitude:  35.9474983215,
		Longitude: -114.861000061,
		Timezone:  "America/Los_Angeles",
	},
	"KBVX": {
		ICAO:      "KBVX",
		IATA:      "BVX",
		Name:      "Batesville Regional Airport",
		City:      "Batesville",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 465,
		Latitude:  35.7262001,
		Longitude: -91.64730072,
		Timezone:  "America/Chicago",
	},
	"KBVY": {
		ICAO:      "KBVY",
		IATA:      "BVY",
		Name:      "Beverly Municipal Airport",
		City:      "Beverly",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 107,
		Latitude:  42.5842018127,
		Longitude: -70.9164962769,
		Timezone:  "America/New_York",
	},
	"KBWC": {
		ICAO:      "KBWC",
		IATA:      "BWC",
		Name:      "Brawley Municipal Airport",
		City:      "Brawley",
		State:     "California",
		Country:   "US",
		Elevation: -128,
		Latitude:  32.99290085,
		Longitude: -115.5169983,
		Timezone:  "America/Los_Angeles",
	},
	"KBWD": {
		ICAO:      "KBWD",
		IATA:      "BWD",
		Name:      "Brownwood Regional Airport",
		City:      "Brownwood",
		State:     "Texas",
		Country:   "US",
		Elevation: 1387,
		Latitude:  31.7936000824,
		Longitude: -98.9564971924,
		Timezone:  "America/Chicago",
	},
	"KBWG": {
		ICAO:      "KBWG",
		IATA:      "BWG",
		Name:      "Bowling Green Warren County Regional Airport",
		City:      "Bowling Green",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 547,
		Latitude:  36.9645004272,
		Longitude: -86.4197006226,
		Timezone:  "America/Chicago",
	},
	"KBWI": {
		ICAO:      "KBWI",
		IATA:      "BWI",
		Name:      "Baltimore/Washington International Thurgood Marshal Airport",
		City:      "Baltimore",
		State:     "Maryland",
		Country:   "US",
		Elevation: 146,
		Latitude:  39.17539978,
		Longitude: -76.66829681,
		Timezone:  "America/New_York",
	},
	"KBWP": {
		ICAO:      "KBWP",
		IATA:      "WAH",
		Name:      "Harry Stern Airport",
		City:      "Wahpeton",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 968,
		Latitude:  46.2440986633,
		Longitude: -96.6073989868,
		Timezone:  "America/Chicago",
	},
	"KBXA": {
		ICAO:      "KBXA",
		IATA:      "BXA",
		Name:      "George R Carr Memorial Air Field",
		City:      "Bogalusa",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 119,
		Latitude:  30.8136997223,
		Longitude: -89.8649978638,
		Timezone:  "America/Chicago",
	},
	"KBXG": {
		ICAO:      "KBXG",
		Name:      "Burke County Airport",
		City:      "Waynesboro",
		State:     "Georgia",
		Country:   "US",
		Elevation: 302,
		Latitude:  33.0413017273,
		Longitude: -82.0027008057,
		Timezone:  "America/New_York",
	},
	"KBXK": {
		ICAO:      "KBXK",
		IATA:      "BXK",
		Name:      "Buckeye Municipal Airport",
		City:      "Buckeye",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1033,
		Latitude:  33.42039871,
		Longitude: -112.685997,
		Timezone:  "America/Phoenix",
	},
	"KBXM": {
		ICAO:      "KBXM",
		IATA:      "NHZ",
		Name:      "Brunswick Executive Airport",
		City:      "Brunswick",
		State:     "Maine",
		Country:   "US",
		Elevation: 75,
		Latitude:  43.8923551,
		Longitude: -69.9388297,
		Timezone:  "America/New_York",
	},
	"KBYG": {
		ICAO:      "KBYG",
		IATA:      "BYG",
		Name:      "Johnson County Airport",
		City:      "Buffalo",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4968,
		Latitude:  44.3810997009,
		Longitude: -106.7220001221,
		Timezone:  "America/Denver",
	},
	"KBYH": {
		ICAO:      "KBYH",
		IATA:      "BYH",
		Name:      "Arkansas International Airport",
		City:      "Blytheville",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 254,
		Latitude:  35.9642982483,
		Longitude: -89.9440002441,
		Timezone:  "America/Chicago",
	},
	"KBYI": {
		ICAO:      "KBYI",
		IATA:      "BYI",
		Name:      "Burley Municipal Airport",
		City:      "Burley",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4150,
		Latitude:  42.5425987244,
		Longitude: -113.7720031738,
		Timezone:  "America/Boise",
	},
	"KBYS": {
		ICAO:      "KBYS",
		IATA:      "BYS",
		Name:      "Bicycle Lake Army Air Field",
		City:      "Fort Irwin/Barstow",
		State:     "California",
		Country:   "US",
		Elevation: 2350,
		Latitude:  35.2804985046,
		Longitude: -116.629997253,
		Timezone:  "America/Los_Angeles",
	},
	"KBYY": {
		ICAO:      "KBYY",
		IATA:      "BBC",
		Name:      "Bay City Municipal Airport",
		City:      "Bay City",
		State:     "Texas",
		Country:   "US",
		Elevation: 45,
		Latitude:  28.9733009338,
		Longitude: -95.8635025024,
		Timezone:  "America/Chicago",
	},
	"KBZN": {
		ICAO:      "KBZN",
		IATA:      "BZN",
		Name:      "Gallatin Field",
		City:      "Bozeman",
		State:     "Montana",
		Country:   "US",
		Elevation: 4473,
		Latitude:  45.77750015,
		Longitude: -111.1529999,
		Timezone:  "America/Denver",
	},
	"KC02": {
		ICAO:      "KC02",
		Name:      "Grand Geneva Resort Airport",
		City:      "Lake Geneva",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 835,
		Latitude:  42.6148986816,
		Longitude: -88.3896026611,
		Timezone:  "America/Chicago",
	},
	"KC03": {
		ICAO:      "KC03",
		Name:      "Nappanee Municipal Airport",
		City:      "Nappanee",
		State:     "Indiana",
		Country:   "US",
		Elevation: 860,
		Latitude:  41.4462013245,
		Longitude: -85.9347991943,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KC04": {
		ICAO:      "KC04",
		Name:      "Oceana County Airport",
		City:      "Hart/Shelby",
		State:     "Michigan",
		Country:   "US",
		Elevation: 910,
		Latitude:  43.6417007446,
		Longitude: -86.3292007446,
		Timezone:  "America/Detroit",
	},
	"KC08": {
		ICAO:      "KC08",
		Name:      "Silver West Airport",
		City:      "Westcliffe",
		State:     "Colorado",
		Country:   "US",
		Elevation: 8290,
		Latitude:  38.011100769,
		Longitude: -105.3730010986,
		Timezone:  "America/Denver",
	},
	"KC09": {
		ICAO:      "KC09",
		Name:      "Morris Municipal James R. Washburn field",
		City:      "Morris",
		State:     "Illinois",
		Country:   "US",
		Elevation: 584,
		Latitude:  41.42539978,
		Longitude: -88.41870117,
		Timezone:  "America/Chicago",
	},
	"KC15": {
		ICAO:      "KC15",
		Name:      "Pekin Municipal Airport",
		City:      "Pekin",
		State:     "Illinois",
		Country:   "US",
		Elevation: 530,
		Latitude:  40.4882011414,
		Longitude: -89.6759033203,
		Timezone:  "America/Chicago",
	},
	"KC16": {
		ICAO:      "KC16",
		Name:      "Frasca Field",
		City:      "Urbana",
		State:     "Illinois",
		Country:   "US",
		Elevation: 735,
		Latitude:  40.1464004517,
		Longitude: -88.1985015869,
		Timezone:  "America/Chicago",
	},
	"KC17": {
		ICAO:      "KC17",
		Name:      "Marion Airport",
		City:      "Marion",
		State:     "Iowa",
		Country:   "US",
		Elevation: 862,
		Latitude:  42.0311012268,
		Longitude: -91.529296875,
		Timezone:  "America/Chicago",
	},
	"KC20": {
		ICAO:      "KC20",
		Name:      "Andrews University Airpark",
		City:      "Berrien Springs",
		State:     "Michigan",
		Country:   "US",
		Elevation: 668,
		Latitude:  41.9516983032,
		Longitude: -86.3676986694,
		Timezone:  "America/Detroit",
	},
	"KC22": {
		ICAO:      "KC22",
		Name:      "Centre Municipal Airport",
		City:      "Centre",
		State:     "Alabama",
		Country:   "US",
		Elevation: 619,
		Latitude:  34.1599006653,
		Longitude: -85.6351013184,
		Timezone:  "America/Chicago",
	},
	"KC27": {
		ICAO:      "KC27",
		Name:      "Manchester Municipal Airport",
		City:      "Manchester",
		State:     "Iowa",
		Country:   "US",
		Elevation: 987,
		Latitude:  42.4933013916,
		Longitude: -91.4984970093,
		Timezone:  "America/Chicago",
	},
	"KC29": {
		ICAO:      "KC29",
		Name:      "Middleton Municipal Morey Field",
		City:      "Middleton",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 928,
		Latitude:  43.1142997742,
		Longitude: -89.53150177,
		Timezone:  "America/Chicago",
	},
	"KC35": {
		ICAO:      "KC35",
		Name:      "Reedsburg Municipal Airport",
		City:      "Reedsburg",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 907,
		Latitude:  43.5259017944,
		Longitude: -89.9832000732,
		Timezone:  "America/Chicago",
	},
	"KC47": {
		ICAO:      "KC47",
		Name:      "Portage Municipal Airport",
		City:      "Portage",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 825,
		Latitude:  43.5602989197,
		Longitude: -89.4829025269,
		Timezone:  "America/Chicago",
	},
	"KC62": {
		ICAO:      "KC62",
		Name:      "Kendallville Municipal Airport",
		City:      "Kendallville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 1005,
		Latitude:  41.4726982117,
		Longitude: -85.2608032227,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KC65": {
		ICAO:      "KC65",
		Name:      "Plymouth Municipal Airport",
		City:      "Plymouth",
		State:     "Indiana",
		Country:   "US",
		Elevation: 800,
		Latitude:  41.3651008606,
		Longitude: -86.3004989624,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KC71": {
		ICAO:      "KC71",
		Name:      "Crosby Municipal Airport",
		City:      "Crosby",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 336,
		Latitude:  31.295999527,
		Longitude: -91.0529022217,
		Timezone:  "America/Chicago",
	},
	"KC73": {
		ICAO:      "KC73",
		Name:      "Dixon Municipal Charles R. Walgreen Field",
		City:      "Dixon",
		State:     "Illinois",
		Country:   "US",
		Elevation: 785,
		Latitude:  41.83369827,
		Longitude: -89.44619751,
		Timezone:  "America/Chicago",
	},
	"KC74": {
		ICAO:      "KC74",
		Name:      "Cassville Municipal Airport",
		City:      "Cassville",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 627,
		Latitude:  42.7042007446,
		Longitude: -90.9645996094,
		Timezone:  "America/Chicago",
	},
	"KC75": {
		ICAO:      "KC75",
		Name:      "Marshall County Airport",
		City:      "Lacon",
		State:     "Illinois",
		Country:   "US",
		Elevation: 568,
		Latitude:  41.0192985535,
		Longitude: -89.3863983154,
		Timezone:  "America/Chicago",
	},
	"KC77": {
		ICAO:      "KC77",
		Name:      "Poplar Grove Airport",
		City:      "Poplar Grove",
		State:     "Illinois",
		Country:   "US",
		Elevation: 858,
		Latitude:  42.3228988647,
		Longitude: -88.8363037109,
		Timezone:  "America/Chicago",
	},
	"KC81": {
		ICAO:      "KC81",
		Name:      "Campbell Airport",
		City:      "Grayslake",
		State:     "Illinois",
		Country:   "US",
		Elevation: 788,
		Latitude:  42.3246002197,
		Longitude: -88.0740966797,
		Timezone:  "America/Chicago",
	},
	"KC83": {
		ICAO:      "KC83",
		Name:      "Byron Airport",
		City:      "Byron",
		State:     "California",
		Country:   "US",
		Elevation: 79,
		Latitude:  37.8283996582,
		Longitude: -121.6259994507,
		Timezone:  "America/Los_Angeles",
	},
	"KC91": {
		ICAO:      "KC91",
		Name:      "Dowagiac Municipal Airport",
		City:      "Dowagiac",
		State:     "Michigan",
		Country:   "US",
		Elevation: 747,
		Latitude:  41.99290085,
		Longitude: -86.12799835,
		Timezone:  "America/Detroit",
	},
	"KCAD": {
		ICAO:      "KCAD",
		IATA:      "CAD",
		Name:      "Wexford County Airport",
		City:      "Cadillac",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1307,
		Latitude:  44.2752990723,
		Longitude: -85.4188995361,
		Timezone:  "America/Detroit",
	},
	"KCAE": {
		ICAO:      "KCAE",
		IATA:      "CAE",
		Name:      "Columbia Metropolitan Airport",
		City:      "Columbia",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 236,
		Latitude:  33.9388008118,
		Longitude: -81.1194992065,
		Timezone:  "America/New_York",
	},
	"KCAG": {
		ICAO:      "KCAG",
		IATA:      "CIG",
		Name:      "Craig Moffat Airport",
		City:      "Craig",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6193,
		Latitude:  40.4952011108,
		Longitude: -107.5220031738,
		Timezone:  "America/Denver",
	},
	"KCAK": {
		ICAO:      "KCAK",
		IATA:      "CAK",
		Name:      "Akron Canton Regional Airport",
		City:      "Akron",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1228,
		Latitude:  40.9160995483,
		Longitude: -81.442199707,
		Timezone:  "America/New_York",
	},
	"KCAO": {
		ICAO:      "KCAO",
		IATA:      "CAO",
		Name:      "Clayton Municipal Airpark",
		City:      "Clayton",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4965,
		Latitude:  36.4462013245,
		Longitude: -103.166999817,
		Timezone:  "America/Denver",
	},
	"KCAR": {
		ICAO:      "KCAR",
		IATA:      "CAR",
		Name:      "Caribou Municipal Airport",
		City:      "Caribou",
		State:     "Maine",
		Country:   "US",
		Elevation: 626,
		Latitude:  46.8714981079,
		Longitude: -68.0178985596,
		Timezone:  "America/New_York",
	},
	"KCAV": {
		ICAO:      "KCAV",
		Name:      "Clarion Municipal Airport",
		City:      "Clarion",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1162,
		Latitude:  42.7419013977,
		Longitude: -93.7589035034,
		Timezone:  "America/Chicago",
	},
	"KCBE": {
		ICAO:      "KCBE",
		IATA:      "CBE",
		Name:      "Greater Cumberland Regional Airport",
		City:      "Cumberland",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 775,
		Latitude:  39.615398407,
		Longitude: -78.7609024048,
		Timezone:  "America/New_York",
	},
	"KCBF": {
		ICAO:      "KCBF",
		IATA:      "CBF",
		Name:      "Council Bluffs Municipal Airport",
		City:      "Council Bluffs",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1253,
		Latitude:  41.2592010498,
		Longitude: -95.760597229,
		Timezone:  "America/Chicago",
	},
	"KCBG": {
		ICAO:      "KCBG",
		Name:      "Cambridge Municipal Airport",
		City:      "Cambridge",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 945,
		Latitude:  45.5574989319,
		Longitude: -93.2641983032,
		Timezone:  "America/Chicago",
	},
	"KCBK": {
		ICAO:      "KCBK",
		IATA:      "CBK",
		Name:      "Shalz Field",
		City:      "Colby",
		State:     "Kansas",
		Country:   "US",
		Elevation: 3187,
		Latitude:  39.42750168,
		Longitude: -101.0469971,
		Timezone:  "America/Chicago",
	},
	"KCBM": {
		ICAO:      "KCBM",
		IATA:      "CBM",
		Name:      "Columbus Air Force Base",
		City:      "Columbus",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 219,
		Latitude:  33.6437988281,
		Longitude: -88.4438018799,
		Timezone:  "America/Chicago",
	},
	"KCCA": {
		ICAO:      "KCCA",
		Name:      "Clinton Municipal Airport",
		City:      "Clinton",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 514,
		Latitude:  35.5978012085,
		Longitude: -92.4515991211,
		Timezone:  "America/Chicago",
	},
	"KCCB": {
		ICAO:      "KCCB",
		IATA:      "CCB",
		Name:      "Cable Airport",
		City:      "Upland",
		State:     "California",
		Country:   "US",
		Elevation: 1444,
		Latitude:  34.1115989685,
		Longitude: -117.68800354,
		Timezone:  "America/Los_Angeles",
	},
	"KCCO": {
		ICAO:      "KCCO",
		Name:      "Newnan Coweta County Airport",
		City:      "Atlanta",
		State:     "Georgia",
		Country:   "US",
		Elevation: 970,
		Latitude:  33.3115997314,
		Longitude: -84.7697982788,
		Timezone:  "America/New_York",
	},
	"KCCR": {
		ICAO:      "KCCR",
		IATA:      "CCR",
		Name:      "Buchanan Field",
		City:      "Concord",
		State:     "California",
		Country:   "US",
		Elevation: 26,
		Latitude:  37.9897003174,
		Longitude: -122.056999207,
		Timezone:  "America/Los_Angeles",
	},
	"KCCY": {
		ICAO:      "KCCY",
		IATA:      "CCY",
		Name:      "Northeast Iowa Regional Airport",
		City:      "Charles City",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1125,
		Latitude:  43.0726013184,
		Longitude: -92.6108016968,
		Timezone:  "America/Chicago",
	},
	"KCDA": {
		ICAO:      "KCDA",
		IATA:      "LLX",
		Name:      "Caledonia County Airport",
		City:      "Lyndonville",
		State:     "Vermont",
		Country:   "US",
		Elevation: 1188,
		Latitude:  44.5690994263,
		Longitude: -72.0179977417,
		Timezone:  "America/New_York",
	},
	"KCDC": {
		ICAO:      "KCDC",
		IATA:      "CDC",
		Name:      "Cedar City Regional Airport",
		City:      "Cedar City",
		State:     "Utah",
		Country:   "US",
		Elevation: 5622,
		Latitude:  37.7010002136,
		Longitude: -113.0989990234,
		Timezone:  "America/Denver",
	},
	"KCDH": {
		ICAO:      "KCDH",
		IATA:      "CDH",
		Name:      "Harrell Field",
		City:      "Camden",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 130,
		Latitude:  33.6227989197,
		Longitude: -92.7633972168,
		Timezone:  "America/Chicago",
	},
	"KCDI": {
		ICAO:      "KCDI",
		Name:      "Cambridge Municipal Airport",
		City:      "Cambridge",
		State:     "Ohio",
		Country:   "US",
		Elevation: 799,
		Latitude:  39.9749984741,
		Longitude: -81.5775985718,
		Timezone:  "America/New_York",
	},
	"KCDK": {
		ICAO:      "KCDK",
		IATA:      "CDK",
		Name:      "George T Lewis Airport",
		City:      "Cedar Key",
		State:     "Florida",
		Country:   "US",
		Elevation: 11,
		Latitude:  29.1352567648,
		Longitude: -83.0483901501,
		Timezone:  "America/New_York",
	},
	"KCDN": {
		ICAO:      "KCDN",
		IATA:      "CDN",
		Name:      "Woodward Field",
		City:      "Camden",
		State:     "New-York",
		Country:   "US",
		Elevation: 302,
		Latitude:  34.2835998535,
		Longitude: -80.5649032593,
		Timezone:  "America/New_York",
	},
	"KCDR": {
		ICAO:      "KCDR",
		IATA:      "CDR",
		Name:      "Chadron Municipal Airport",
		City:      "Chadron",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3297,
		Latitude:  42.837600708,
		Longitude: -103.095001221,
		Timezone:  "America/Denver",
	},
	"KCDS": {
		ICAO:      "KCDS",
		IATA:      "CDS",
		Name:      "Childress Municipal Airport",
		City:      "Childress",
		State:     "Texas",
		Country:   "US",
		Elevation: 1954,
		Latitude:  34.4337997437,
		Longitude: -100.288002014,
		Timezone:  "America/Chicago",
	},
	"KCDW": {
		ICAO:      "KCDW",
		IATA:      "CDW",
		Name:      "Essex County Airport",
		City:      "Caldwell",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 173,
		Latitude:  40.8751983643,
		Longitude: -74.2814025879,
		Timezone:  "America/New_York",
	},
	"KCEA": {
		ICAO:      "KCEA",
		IATA:      "CEA",
		Name:      "Cessna Acft Field",
		City:      "Wichita",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1378,
		Latitude:  37.648601532,
		Longitude: -97.2506027222,
		Timezone:  "America/Chicago",
	},
	"KCEC": {
		ICAO:      "KCEC",
		IATA:      "CEC",
		Name:      "Jack Mc Namara Field Airport",
		City:      "Crescent City",
		State:     "California",
		Country:   "US",
		Elevation: 61,
		Latitude:  41.78020096,
		Longitude: -124.2369995,
		Timezone:  "America/Los_Angeles",
	},
	"KCEF": {
		ICAO:      "KCEF",
		IATA:      "CEF",
		Name:      "Westover ARB/Metropolitan Airport",
		City:      "Springfield/Chicopee",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 241,
		Latitude:  42.19400024,
		Longitude: -72.53479767,
		Timezone:  "America/New_York",
	},
	"KCEK": {
		ICAO:      "KCEK",
		Name:      "Crete Municipal Airport",
		City:      "Crete",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1500,
		Latitude:  40.61790085,
		Longitude: -96.92489624,
		Timezone:  "America/Chicago",
	},
	"KCEU": {
		ICAO:      "KCEU",
		IATA:      "CEU",
		Name:      "Oconee County Regional Airport",
		City:      "Clemson",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 892,
		Latitude:  34.6719017,
		Longitude: -82.8864975,
		Timezone:  "America/New_York",
	},
	"KCEV": {
		ICAO:      "KCEV",
		IATA:      "CEV",
		Name:      "Mettel Field",
		City:      "Connersville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 867,
		Latitude:  39.6985015869,
		Longitude: -85.129699707,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KCEW": {
		ICAO:      "KCEW",
		IATA:      "CEW",
		Name:      "Bob Sikes Airport",
		City:      "Crestview",
		State:     "Florida",
		Country:   "US",
		Elevation: 213,
		Latitude:  30.778799057,
		Longitude: -86.522102356,
		Timezone:  "America/Chicago",
	},
	"KCEY": {
		ICAO:      "KCEY",
		IATA:      "CEY",
		Name:      "Kyle Oakley Field",
		City:      "Murray",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 577,
		Latitude:  36.66460037,
		Longitude: -88.37280273,
		Timezone:  "America/Chicago",
	},
	"KCEZ": {
		ICAO:      "KCEZ",
		IATA:      "CEZ",
		Name:      "Cortez Municipal Airport",
		City:      "Cortez",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5918,
		Latitude:  37.3030014038,
		Longitude: -108.627998352,
		Timezone:  "America/Denver",
	},
	"KCFD": {
		ICAO:      "KCFD",
		IATA:      "CFD",
		Name:      "Coulter Field",
		City:      "Bryan",
		State:     "Texas",
		Country:   "US",
		Elevation: 367,
		Latitude:  30.7157001495,
		Longitude: -96.3313980103,
		Timezone:  "America/Chicago",
	},
	"KCFE": {
		ICAO:      "KCFE",
		Name:      "Buffalo Municipal Airport",
		City:      "Buffalo",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 967,
		Latitude:  45.1590003967,
		Longitude: -93.8432998657,
		Timezone:  "America/Chicago",
	},
	"KCFJ": {
		ICAO:      "KCFJ",
		Name:      "Crawfordsville Municipal Airport",
		City:      "Crawfordsville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 799,
		Latitude:  39.9756011963,
		Longitude: -86.9198989868,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KCFS": {
		ICAO:      "KCFS",
		IATA:      "TZC",
		Name:      "Tuscola Area Airport",
		City:      "Caro",
		State:     "Michigan",
		Country:   "US",
		Elevation: 701,
		Latitude:  43.4588012695,
		Longitude: -83.4455032349,
		Timezone:  "America/Detroit",
	},
	"KCFT": {
		ICAO:      "KCFT",
		IATA:      "CFT",
		Name:      "Greenlee County Airport",
		City:      "Clifton/Morenci",
		State:     "Arizona",
		Country:   "US",
		Elevation: 3798,
		Latitude:  32.95280075,
		Longitude: -109.2099991,
		Timezone:  "America/Phoenix",
	},
	"KCFV": {
		ICAO:      "KCFV",
		IATA:      "CFV",
		Name:      "Coffeyville Municipal Airport",
		City:      "Coffeyville",
		State:     "Kansas",
		Country:   "US",
		Elevation: 754,
		Latitude:  37.09400177,
		Longitude: -95.5718994141,
		Timezone:  "America/Chicago",
	},
	"KCGC": {
		ICAO:      "KCGC",
		Name:      "Crystal River Airport",
		City:      "Crystal River",
		State:     "Florida",
		Country:   "US",
		Elevation: 9,
		Latitude:  28.8673000336,
		Longitude: -82.5712966919,
		Timezone:  "America/New_York",
	},
	"KCGE": {
		ICAO:      "KCGE",
		IATA:      "CGE",
		Name:      "Cambridge Dorchester Airport",
		City:      "Cambridge",
		State:     "Maryland",
		Country:   "US",
		Elevation: 20,
		Latitude:  38.53929901,
		Longitude: -76.03040314,
		Timezone:  "America/New_York",
	},
	"KCGF": {
		ICAO:      "KCGF",
		IATA:      "CGF",
		Name:      "Cuyahoga County Airport",
		City:      "Cleveland",
		State:     "Ohio",
		Country:   "US",
		Elevation: 879,
		Latitude:  41.5651016235,
		Longitude: -81.4863967896,
		Timezone:  "America/New_York",
	},
	"KCGI": {
		ICAO:      "KCGI",
		IATA:      "CGI",
		Name:      "Cape Girardeau Regional Airport",
		City:      "Cape Girardeau",
		State:     "Missouri",
		Country:   "US",
		Elevation: 342,
		Latitude:  37.2252998352,
		Longitude: -89.5708007813,
		Timezone:  "America/Chicago",
	},
	"KCGS": {
		ICAO:      "KCGS",
		IATA:      "CGS",
		Name:      "College Park Airport",
		City:      "College Park",
		State:     "Maryland",
		Country:   "US",
		Elevation: 48,
		Latitude:  38.9805984497,
		Longitude: -76.9223022461,
		Timezone:  "America/New_York",
	},
	"KCGZ": {
		ICAO:      "KCGZ",
		IATA:      "CGZ",
		Name:      "Casa Grande Municipal Airport",
		City:      "Casa Grande",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1464,
		Latitude:  32.9548988342,
		Longitude: -111.766998291,
		Timezone:  "America/Phoenix",
	},
	"KCHA": {
		ICAO:      "KCHA",
		IATA:      "CHA",
		Name:      "Lovell Field",
		City:      "Chattanooga",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 683,
		Latitude:  35.0353012085,
		Longitude: -85.2037963867,
		Timezone:  "America/New_York",
	},
	"KCHD": {
		ICAO:      "KCHD",
		Name:      "Chandler Municipal Airport",
		City:      "Chandler",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1243,
		Latitude:  33.2691001892,
		Longitude: -111.8109970093,
		Timezone:  "America/Phoenix",
	},
	"KCHK": {
		ICAO:      "KCHK",
		IATA:      "CHK",
		Name:      "Chickasha Municipal Airport",
		City:      "Chickasha",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1152,
		Latitude:  35.09740067,
		Longitude: -97.96769714,
		Timezone:  "America/Chicago",
	},
	"KCHN": {
		ICAO:      "KCHN",
		Name:      "Wauchula Municipal Airport",
		City:      "Wauchula",
		State:     "Florida",
		Country:   "US",
		Elevation: 106,
		Latitude:  27.5149002075,
		Longitude: -81.8805007935,
		Timezone:  "America/New_York",
	},
	"KCHO": {
		ICAO:      "KCHO",
		IATA:      "CHO",
		Name:      "Charlottesville Albemarle Airport",
		City:      "Charlottesville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 639,
		Latitude:  38.1385993958,
		Longitude: -78.4529037476,
		Timezone:  "America/New_York",
	},
	"KCHQ": {
		ICAO:      "KCHQ",
		Name:      "Mississippi County Airport",
		City:      "Charleston",
		State:     "Missouri",
		Country:   "US",
		Elevation: 313,
		Latitude:  36.8420982361,
		Longitude: -89.359703064,
		Timezone:  "America/Chicago",
	},
	"KCHS": {
		ICAO:      "KCHS",
		IATA:      "CHS",
		Name:      "Charleston Air Force Base-International Airport",
		City:      "Charleston",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 46,
		Latitude:  32.89860153,
		Longitude: -80.04049683,
		Timezone:  "America/New_York",
	},
	"KCHT": {
		ICAO:      "KCHT",
		Name:      "Chillicothe Municipal Airport",
		City:      "Chillicothe",
		State:     "Missouri",
		Country:   "US",
		Elevation: 783,
		Latitude:  39.7821998596,
		Longitude: -93.4956970215,
		Timezone:  "America/Chicago",
	},
	"KCHU": {
		ICAO:      "KCHU",
		Name:      "Houston County Airport",
		City:      "Caledonia",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1179,
		Latitude:  43.5964012146,
		Longitude: -91.5038986206,
		Timezone:  "America/Chicago",
	},
	"KCIC": {
		ICAO:      "KCIC",
		IATA:      "CIC",
		Name:      "Chico Municipal Airport",
		City:      "Chico",
		State:     "California",
		Country:   "US",
		Elevation: 240,
		Latitude:  39.79539871,
		Longitude: -121.8580017,
		Timezone:  "America/Los_Angeles",
	},
	"KCID": {
		ICAO:      "KCID",
		IATA:      "CID",
		Name:      "The Eastern Iowa Airport",
		City:      "Cedar Rapids",
		State:     "Iowa",
		Country:   "US",
		Elevation: 869,
		Latitude:  41.8847007751,
		Longitude: -91.7108001709,
		Timezone:  "America/Chicago",
	},
	"KCII": {
		ICAO:      "KCII",
		Name:      "Choteau Airport",
		City:      "Choteau",
		State:     "Montana",
		Country:   "US",
		Elevation: 3947,
		Latitude:  47.8283004761,
		Longitude: -112.1679992676,
		Timezone:  "America/Denver",
	},
	"KCIN": {
		ICAO:      "KCIN",
		IATA:      "CIN",
		Name:      "Arthur N Neu Airport",
		City:      "Carroll",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1204,
		Latitude:  42.0461997986,
		Longitude: -94.7890014648,
		Timezone:  "America/Chicago",
	},
	"KCIR": {
		ICAO:      "KCIR",
		IATA:      "CIR",
		Name:      "Cairo Regional Airport",
		City:      "Cairo",
		State:     "Illinois",
		Country:   "US",
		Elevation: 321,
		Latitude:  37.0644989014,
		Longitude: -89.2195968628,
		Timezone:  "America/Chicago",
	},
	"KCIU": {
		ICAO:      "KCIU",
		IATA:      "CIU",
		Name:      "Chippewa County International Airport",
		City:      "Sault Ste Marie",
		State:     "Michigan",
		Country:   "US",
		Elevation: 800,
		Latitude:  46.2508010864,
		Longitude: -84.4723968506,
		Timezone:  "America/Detroit",
	},
	"KCJJ": {
		ICAO:      "KCJJ",
		Name:      "Ellen Church Field",
		City:      "Cresco",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1279,
		Latitude:  43.3652992249,
		Longitude: -92.133102417,
		Timezone:  "America/Chicago",
	},
	"KCJR": {
		ICAO:      "KCJR",
		Name:      "Culpeper Regional Airport",
		City:      "Culpeper",
		State:     "Virginia",
		Country:   "US",
		Elevation: 316,
		Latitude:  38.5266990662,
		Longitude: -77.8589019775,
		Timezone:  "America/New_York",
	},
	"KCKA": {
		ICAO:      "KCKA",
		IATA:      "CKA",
		Name:      "Kegelman Af Aux Field",
		City:      "Cherokee",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1202,
		Latitude:  36.7439002991,
		Longitude: -98.1231002808,
		Timezone:  "America/Chicago",
	},
	"KCKB": {
		ICAO:      "KCKB",
		IATA:      "CKB",
		Name:      "North Central West Virginia Airport",
		City:      "Clarksburg",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 1217,
		Latitude:  39.2966003418,
		Longitude: -80.2281036377,
		Timezone:  "America/New_York",
	},
	"KCKC": {
		ICAO:      "KCKC",
		IATA:      "GRM",
		Name:      "Grand Marais Cook County Airport",
		City:      "Grand Marais",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1799,
		Latitude:  47.8382987976,
		Longitude: -90.3828964233,
		Timezone:  "America/Chicago",
	},
	"KCKF": {
		ICAO:      "KCKF",
		Name:      "Crisp County Cordele Airport",
		City:      "Cordele",
		State:     "Georgia",
		Country:   "US",
		Elevation: 310,
		Latitude:  31.98880005,
		Longitude: -83.77390289,
		Timezone:  "America/New_York",
	},
	"KCKI": {
		ICAO:      "KCKI",
		Name:      "Williamsburg Regional Airport",
		City:      "Kingstree",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 67,
		Latitude:  33.7172012329,
		Longitude: -79.8570022583,
		Timezone:  "America/New_York",
	},
	"KCKM": {
		ICAO:      "KCKM",
		IATA:      "CKM",
		Name:      "Fletcher Field",
		City:      "Clarksdale",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 173,
		Latitude:  34.2997016907,
		Longitude: -90.512298584,
		Timezone:  "America/Chicago",
	},
	"KCKN": {
		ICAO:      "KCKN",
		IATA:      "CKN",
		Name:      "Crookston Municipal Kirkwood Field",
		City:      "Crookston",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 899,
		Latitude:  47.8417015076,
		Longitude: -96.62159729,
		Timezone:  "America/Chicago",
	},
	"KCKP": {
		ICAO:      "KCKP",
		Name:      "Cherokee County Regional Airport",
		City:      "Cherokee",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1227,
		Latitude:  42.7317009,
		Longitude: -95.55590057,
		Timezone:  "America/Chicago",
	},
	"KCKV": {
		ICAO:      "KCKV",
		IATA:      "CKV",
		Name:      "Outlaw Field",
		City:      "Clarksville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 550,
		Latitude:  36.6218986511,
		Longitude: -87.4150009155,
		Timezone:  "America/Chicago",
	},
	"KCKZ": {
		ICAO:      "KCKZ",
		Name:      "Pennridge Airport",
		City:      "Perkasie",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 568,
		Latitude:  40.3891983032,
		Longitude: -75.2904968262,
		Timezone:  "America/New_York",
	},
	"KCLE": {
		ICAO:      "KCLE",
		IATA:      "CLE",
		Name:      "Cleveland Hopkins International Airport",
		City:      "Cleveland",
		State:     "Ohio",
		Country:   "US",
		Elevation: 791,
		Latitude:  41.4117012024,
		Longitude: -81.8498001099,
		Timezone:  "America/New_York",
	},
	"KCLI": {
		ICAO:      "KCLI",
		IATA:      "CLI",
		Name:      "Clintonville Municipal Airport",
		City:      "Clintonville",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 822,
		Latitude:  44.6138000488,
		Longitude: -88.731300354,
		Timezone:  "America/Chicago",
	},
	"KCLK": {
		ICAO:      "KCLK",
		IATA:      "CLK",
		Name:      "Clinton Regional Airport",
		City:      "Clinton",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1616,
		Latitude:  35.53829956,
		Longitude: -98.93270111,
		Timezone:  "America/Chicago",
	},
	"KCLL": {
		ICAO:      "KCLL",
		IATA:      "CLL",
		Name:      "Easterwood Field",
		City:      "College Station",
		State:     "Texas",
		Country:   "US",
		Elevation: 320,
		Latitude:  30.58860016,
		Longitude: -96.36380005,
		Timezone:  "America/Chicago",
	},
	"KCLM": {
		ICAO:      "KCLM",
		IATA:      "CLM",
		Name:      "William R Fairchild International Airport",
		City:      "Port Angeles",
		State:     "Washington",
		Country:   "US",
		Elevation: 291,
		Latitude:  48.1202011108,
		Longitude: -123.5,
		Timezone:  "America/Los_Angeles",
	},
	"KCLR": {
		ICAO:      "KCLR",
		IATA:      "CLR",
		Name:      "Cliff Hatfield Memorial Airport",
		City:      "Calipatria",
		State:     "California",
		Country:   "US",
		Elevation: -182,
		Latitude:  33.1315002441,
		Longitude: -115.5210037231,
		Timezone:  "America/Los_Angeles",
	},
	"KCLS": {
		ICAO:      "KCLS",
		IATA:      "CLS",
		Name:      "Chehalis Centralia Airport",
		City:      "Chehalis",
		State:     "Washington",
		Country:   "US",
		Elevation: 176,
		Latitude:  46.6769981384,
		Longitude: -122.983001709,
		Timezone:  "America/Los_Angeles",
	},
	"KCLT": {
		ICAO:      "KCLT",
		IATA:      "CLT",
		Name:      "Charlotte Douglas International Airport",
		City:      "Charlotte",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 748,
		Latitude:  35.2140007019,
		Longitude: -80.9430999756,
		Timezone:  "America/New_York",
	},
	"KCLW": {
		ICAO:      "KCLW",
		IATA:      "CLW",
		Name:      "Clearwater Air Park",
		City:      "Clearwater",
		State:     "Florida",
		Country:   "US",
		Elevation: 71,
		Latitude:  27.9766998291,
		Longitude: -82.7586975098,
		Timezone:  "America/New_York",
	},
	"KCMA": {
		ICAO:      "KCMA",
		Name:      "Camarillo Airport",
		City:      "Camarillo",
		State:     "California",
		Country:   "US",
		Elevation: 77,
		Latitude:  34.2136993408,
		Longitude: -119.09400177,
		Timezone:  "America/Los_Angeles",
	},
	"KCMH": {
		ICAO:      "KCMH",
		IATA:      "CMH",
		Name:      "Port Columbus International Airport",
		City:      "Columbus",
		State:     "Ohio",
		Country:   "US",
		Elevation: 815,
		Latitude:  39.9980010986,
		Longitude: -82.8918991089,
		Timezone:  "America/New_York",
	},
	"KCMI": {
		ICAO:      "KCMI",
		IATA:      "CMI",
		Name:      "University of Illinois Willard Airport",
		City:      "Champaign/Urbana",
		State:     "Illinois",
		Country:   "US",
		Elevation: 755,
		Latitude:  40.03919983,
		Longitude: -88.27809906,
		Timezone:  "America/Chicago",
	},
	"KCMR": {
		ICAO:      "KCMR",
		Name:      "H.A. Clark Memorial Field",
		City:      "Williams",
		State:     "Arizona",
		Country:   "US",
		Elevation: 6680,
		Latitude:  35.30220032,
		Longitude: -112.1940002,
		Timezone:  "America/Phoenix",
	},
	"KCMX": {
		ICAO:      "KCMX",
		IATA:      "CMX",
		Name:      "Houghton County Memorial Airport",
		City:      "Hancock",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1095,
		Latitude:  47.1683998108,
		Longitude: -88.4890975952,
		Timezone:  "America/Detroit",
	},
	"KCMY": {
		ICAO:      "KCMY",
		IATA:      "CMY",
		Name:      "Sparta Fort Mc Coy Airport",
		City:      "Sparta",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 837,
		Latitude:  43.9583015442,
		Longitude: -90.7378997803,
		Timezone:  "America/Chicago",
	},
	"KCNB": {
		ICAO:      "KCNB",
		Name:      "Myers Field",
		City:      "Canby",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1194,
		Latitude:  44.72949982,
		Longitude: -96.26599884,
		Timezone:  "America/Chicago",
	},
	"KCNC": {
		ICAO:      "KCNC",
		Name:      "Chariton Municipal Airport",
		City:      "Chariton",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1050,
		Latitude:  41.0195999146,
		Longitude: -93.359703064,
		Timezone:  "America/Chicago",
	},
	"KCNH": {
		ICAO:      "KCNH",
		IATA:      "CNH",
		Name:      "Claremont Municipal Airport",
		City:      "Claremont",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 545,
		Latitude:  43.3703994751,
		Longitude: -72.3686981201,
		Timezone:  "America/New_York",
	},
	"KCNK": {
		ICAO:      "KCNK",
		IATA:      "CNK",
		Name:      "Blosser Municipal Airport",
		City:      "Concordia",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1486,
		Latitude:  39.5493011475,
		Longitude: -97.6522979736,
		Timezone:  "America/Chicago",
	},
	"KCNM": {
		ICAO:      "KCNM",
		IATA:      "CNM",
		Name:      "Cavern City Air Terminal",
		City:      "Carlsbad",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 3295,
		Latitude:  32.3375015259,
		Longitude: -104.2630004883,
		Timezone:  "America/Denver",
	},
	"KCNO": {
		ICAO:      "KCNO",
		IATA:      "CNO",
		Name:      "Chino Airport",
		City:      "Chino",
		State:     "California",
		Country:   "US",
		Elevation: 650,
		Latitude:  33.97470093,
		Longitude: -117.637001,
		Timezone:  "America/Los_Angeles",
	},
	"KCNP": {
		ICAO:      "KCNP",
		Name:      "Billy G Ray Field",
		City:      "Chappell",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3682,
		Latitude:  41.0774993896,
		Longitude: -102.4639968872,
		Timezone:  "America/Denver",
	},
	"KCNU": {
		ICAO:      "KCNU",
		IATA:      "CNU",
		Name:      "Chanute Martin Johnson Airport",
		City:      "Chanute",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1002,
		Latitude:  37.668800354,
		Longitude: -95.4850997925,
		Timezone:  "America/Chicago",
	},
	"KCNW": {
		ICAO:      "KCNW",
		IATA:      "CNW",
		Name:      "TSTC Waco Airport",
		City:      "Waco",
		State:     "Texas",
		Country:   "US",
		Elevation: 470,
		Latitude:  31.6378002167,
		Longitude: -97.0740966797,
		Timezone:  "America/Chicago",
	},
	"KCNY": {
		ICAO:      "KCNY",
		IATA:      "CNY",
		Name:      "Canyonlands Field",
		City:      "Moab",
		State:     "Utah",
		Country:   "US",
		Elevation: 4557,
		Latitude:  38.75500107,
		Longitude: -109.7549973,
		Timezone:  "America/Denver",
	},
	"KCOD": {
		ICAO:      "KCOD",
		IATA:      "COD",
		Name:      "Yellowstone Regional Airport",
		City:      "Cody",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 5102,
		Latitude:  44.520198822,
		Longitude: -109.0240020752,
		Timezone:  "America/Denver",
	},
	"KCOE": {
		ICAO:      "KCOE",
		IATA:      "COE",
		Name:      "Coeur D'Alene - Pappy Boyington Field",
		City:      "Coeur d'Alene",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2320,
		Latitude:  47.77429962,
		Longitude: -116.8199997,
		Timezone:  "America/Los_Angeles",
	},
	"KCOF": {
		ICAO:      "KCOF",
		IATA:      "COF",
		Name:      "Patrick Air Force Base",
		City:      "Cocoa Beach",
		State:     "Florida",
		Country:   "US",
		Elevation: 8,
		Latitude:  28.2348995209,
		Longitude: -80.6100997925,
		Timezone:  "America/New_York",
	},
	"KCOI": {
		ICAO:      "KCOI",
		IATA:      "COI",
		Name:      "Merritt Island Airport",
		City:      "Merritt Island",
		State:     "Florida",
		Country:   "US",
		Elevation: 6,
		Latitude:  28.3416004181,
		Longitude: -80.6855010986,
		Timezone:  "America/New_York",
	},
	"KCOM": {
		ICAO:      "KCOM",
		IATA:      "COM",
		Name:      "Coleman Municipal Airport",
		City:      "Coleman",
		State:     "Texas",
		Country:   "US",
		Elevation: 1697,
		Latitude:  31.8411006927,
		Longitude: -99.4036026001,
		Timezone:  "America/Chicago",
	},
	"KCON": {
		ICAO:      "KCON",
		IATA:      "CON",
		Name:      "Concord Municipal Airport",
		City:      "Concord",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 342,
		Latitude:  43.20270157,
		Longitude: -71.50229645,
		Timezone:  "America/New_York",
	},
	"KCOQ": {
		ICAO:      "KCOQ",
		Name:      "Cloquet Carlton County Airport",
		City:      "Cloquet",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1279,
		Latitude:  46.7010993958,
		Longitude: -92.5036010742,
		Timezone:  "America/Chicago",
	},
	"KCOS": {
		ICAO:      "KCOS",
		IATA:      "COS",
		Name:      "City of Colorado Springs Municipal Airport",
		City:      "Colorado Springs",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6187,
		Latitude:  38.8058013916,
		Longitude: -104.7009963989,
		Timezone:  "America/Denver",
	},
	"KCOT": {
		ICAO:      "KCOT",
		IATA:      "COT",
		Name:      "Cotulla-La Salle County",
		City:      "Cotulla",
		State:     "Texas",
		Country:   "US",
		Elevation: 474,
		Latitude:  28.45669937,
		Longitude: -99.22029877,
		Timezone:  "America/Chicago",
	},
	"KCOU": {
		ICAO:      "KCOU",
		IATA:      "COU",
		Name:      "Columbia Regional Airport",
		City:      "Columbia",
		State:     "Missouri",
		Country:   "US",
		Elevation: 889,
		Latitude:  38.8180999756,
		Longitude: -92.2195968628,
		Timezone:  "America/Chicago",
	},
	"KCPC": {
		ICAO:      "KCPC",
		Name:      "Columbus County Municipal Airport",
		City:      "Whiteville",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 99,
		Latitude:  34.27289963,
		Longitude: -78.71499634,
		Timezone:  "America/New_York",
	},
	"KCPK": {
		ICAO:      "KCPK",
		Name:      "Chesapeake Regional Airport",
		City:      "Norfolk",
		State:     "Virginia",
		Country:   "US",
		Elevation: 19,
		Latitude:  36.66559982,
		Longitude: -76.3207016,
		Timezone:  "America/New_York",
	},
	"KCPM": {
		ICAO:      "KCPM",
		IATA:      "CPM",
		Name:      "Compton Woodley Airport",
		City:      "Compton",
		State:     "California",
		Country:   "US",
		Elevation: 97,
		Latitude:  33.8899993896,
		Longitude: -118.2440032959,
		Timezone:  "America/Los_Angeles",
	},
	"KCPR": {
		ICAO:      "KCPR",
		IATA:      "CPR",
		Name:      "Casper-Natrona County International Airport",
		City:      "Casper",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 5350,
		Latitude:  42.90800095,
		Longitude: -106.4639969,
		Timezone:  "America/Denver",
	},
	"KCPS": {
		ICAO:      "KCPS",
		IATA:      "CPS",
		Name:      "St Louis Downtown Airport",
		City:      "Cahokia/St Louis",
		State:     "Illinois",
		Country:   "US",
		Elevation: 413,
		Latitude:  38.5707015991,
		Longitude: -90.1561965942,
		Timezone:  "America/Chicago",
	},
	"KCPT": {
		ICAO:      "KCPT",
		Name:      "Cleburne Municipal Airport",
		City:      "Cleburne",
		State:     "Texas",
		Country:   "US",
		Elevation: 854,
		Latitude:  32.3538017273,
		Longitude: -97.4337005615,
		Timezone:  "America/Chicago",
	},
	"KCPU": {
		ICAO:      "KCPU",
		Name:      "Calaveras Co Maury Rasmussen Field",
		City:      "San Andreas",
		State:     "California",
		Country:   "US",
		Elevation: 1325,
		Latitude:  38.1460990906,
		Longitude: -120.6480026245,
		Timezone:  "America/Los_Angeles",
	},
	"KCQA": {
		ICAO:      "KCQA",
		Name:      "Lakefield Airport",
		City:      "Celina",
		State:     "Ohio",
		Country:   "US",
		Elevation: 894,
		Latitude:  40.4841003418,
		Longitude: -84.5600967407,
		Timezone:  "America/New_York",
	},
	"KCQB": {
		ICAO:      "KCQB",
		Name:      "Chandler Regional Airport",
		City:      "Chandler",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 984,
		Latitude:  35.7238006592,
		Longitude: -96.8202972412,
		Timezone:  "America/Chicago",
	},
	"KCQM": {
		ICAO:      "KCQM",
		Name:      "Cook Municipal Airport",
		City:      "Cook",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1327,
		Latitude:  47.8218994141,
		Longitude: -92.6893997192,
		Timezone:  "America/Chicago",
	},
	"KCQW": {
		ICAO:      "KCQW",
		IATA:      "HCW",
		Name:      "Cheraw Municipal Lynch Bellinger Field",
		City:      "Cheraw",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 239,
		Latitude:  34.71289825,
		Longitude: -79.95700073,
		Timezone:  "America/New_York",
	},
	"KCQX": {
		ICAO:      "KCQX",
		Name:      "Chatham Municipal Airport",
		City:      "Chatham",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 68,
		Latitude:  41.6884002686,
		Longitude: -69.9895019531,
		Timezone:  "America/New_York",
	},
	"KCRE": {
		ICAO:      "KCRE",
		IATA:      "CRE",
		Name:      "Grand Strand Airport",
		City:      "North Myrtle Beach",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 32,
		Latitude:  33.8116989136,
		Longitude: -78.7238998413,
		Timezone:  "America/New_York",
	},
	"KCRG": {
		ICAO:      "KCRG",
		IATA:      "CRG",
		Name:      "Craig Municipal Airport",
		City:      "Jacksonville",
		State:     "Florida",
		Country:   "US",
		Elevation: 41,
		Latitude:  30.3362998962,
		Longitude: -81.5143966675,
		Timezone:  "America/New_York",
	},
	"KCRO": {
		ICAO:      "KCRO",
		IATA:      "CRO",
		Name:      "Corcoran Airport",
		City:      "Corcoran",
		State:     "California",
		Country:   "US",
		Elevation: 197,
		Latitude:  36.1025009155,
		Longitude: -119.595001221,
		Timezone:  "America/Los_Angeles",
	},
	"KCRP": {
		ICAO:      "KCRP",
		IATA:      "CRP",
		Name:      "Corpus Christi International Airport",
		City:      "Corpus Christi",
		State:     "Texas",
		Country:   "US",
		Elevation: 44,
		Latitude:  27.7703990936,
		Longitude: -97.5011978149,
		Timezone:  "America/Chicago",
	},
	"KCRQ": {
		ICAO:      "KCRQ",
		IATA:      "CLD",
		Name:      "Mc Clellan-Palomar Airport",
		City:      "Carlsbad",
		State:     "California",
		Country:   "US",
		Elevation: 331,
		Latitude:  33.12829971,
		Longitude: -117.2799988,
		Timezone:  "America/Los_Angeles",
	},
	"KCRS": {
		ICAO:      "KCRS",
		IATA:      "CRS",
		Name:      "C David Campbell Field Corsicana Municipal Airport",
		City:      "Corsicana",
		State:     "Texas",
		Country:   "US",
		Elevation: 449,
		Latitude:  32.0280990601,
		Longitude: -96.4005966187,
		Timezone:  "America/Chicago",
	},
	"KCRT": {
		ICAO:      "KCRT",
		IATA:      "CRT",
		Name:      "Z M Jack Stell Field",
		City:      "Crossett",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 184,
		Latitude:  33.1782989502,
		Longitude: -91.8802032471,
		Timezone:  "America/Chicago",
	},
	"KCRW": {
		ICAO:      "KCRW",
		IATA:      "CRW",
		Name:      "Yeager Airport",
		City:      "Charleston",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 981,
		Latitude:  38.3731002808,
		Longitude: -81.5932006836,
		Timezone:  "America/New_York",
	},
	"KCRX": {
		ICAO:      "KCRX",
		IATA:      "CRX",
		Name:      "Roscoe Turner Airport",
		City:      "Corinth",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 425,
		Latitude:  34.9150009155,
		Longitude: -88.6035003662,
		Timezone:  "America/Chicago",
	},
	"KCRZ": {
		ICAO:      "KCRZ",
		Name:      "Corning Municipal Airport",
		City:      "Corning",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1274,
		Latitude:  40.9940986633,
		Longitude: -94.7549972534,
		Timezone:  "America/Chicago",
	},
	"KCSB": {
		ICAO:      "KCSB",
		Name:      "Cambridge Municipal Airport",
		City:      "Cambridge",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2414,
		Latitude:  40.3065986633,
		Longitude: -100.1620025635,
		Timezone:  "America/Chicago",
	},
	"KCSG": {
		ICAO:      "KCSG",
		IATA:      "CSG",
		Name:      "Columbus Metropolitan Airport",
		City:      "Columbus",
		State:     "Georgia",
		Country:   "US",
		Elevation: 397,
		Latitude:  32.5163002014,
		Longitude: -84.9389038086,
		Timezone:  "America/New_York",
	},
	"KCSM": {
		ICAO:      "KCSM",
		IATA:      "CSM",
		Name:      "Clinton Sherman Airport",
		City:      "Clinton",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1922,
		Latitude:  35.3398017883,
		Longitude: -99.2005004883,
		Timezone:  "America/Chicago",
	},
	"KCSQ": {
		ICAO:      "KCSQ",
		IATA:      "CSQ",
		Name:      "Creston Municipal Airport",
		City:      "Creston",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1300,
		Latitude:  41.0214004517,
		Longitude: -94.3632965088,
		Timezone:  "America/Chicago",
	},
	"KCSV": {
		ICAO:      "KCSV",
		IATA:      "CSV",
		Name:      "Crossville Memorial Whitson Field",
		City:      "Crossville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1881,
		Latitude:  35.9513015747,
		Longitude: -85.0849990845,
		Timezone:  "America/Chicago",
	},
	"KCTB": {
		ICAO:      "KCTB",
		IATA:      "CTB",
		Name:      "Cut Bank Municipal Airport",
		City:      "Cut Bank",
		State:     "Montana",
		Country:   "US",
		Elevation: 3854,
		Latitude:  48.6083984375,
		Longitude: -112.3759994507,
		Timezone:  "America/Denver",
	},
	"KCTJ": {
		ICAO:      "KCTJ",
		Name:      "West Georgia Regional O V Gray Field",
		City:      "Carrollton",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1161,
		Latitude:  33.6310005188,
		Longitude: -85.1520004272,
		Timezone:  "America/New_York",
	},
	"KCTK": {
		ICAO:      "KCTK",
		Name:      "Ingersoll Airport",
		City:      "Canton",
		State:     "Illinois",
		Country:   "US",
		Elevation: 684,
		Latitude:  40.5690994263,
		Longitude: -90.074798584,
		Timezone:  "America/Chicago",
	},
	"KCTY": {
		ICAO:      "KCTY",
		IATA:      "CTY",
		Name:      "Cross City Airport",
		City:      "Cross City",
		State:     "Florida",
		Country:   "US",
		Elevation: 42,
		Latitude:  29.6354999542,
		Longitude: -83.1047973633,
		Timezone:  "America/New_York",
	},
	"KCTZ": {
		ICAO:      "KCTZ",
		IATA:      "CTZ",
		Name:      "Sampson County Airport",
		City:      "Clinton",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 148,
		Latitude:  34.9756011963,
		Longitude: -78.3646011353,
		Timezone:  "America/New_York",
	},
	"KCUB": {
		ICAO:      "KCUB",
		IATA:      "CUB",
		Name:      "Jim Hamilton L.B. Owens Airport",
		City:      "Columbia",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 193,
		Latitude:  33.970500946,
		Longitude: -80.9952011108,
		Timezone:  "America/New_York",
	},
	"KCUH": {
		ICAO:      "KCUH",
		IATA:      "CUH",
		Name:      "Cushing Municipal Airport",
		City:      "Cushing",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 916,
		Latitude:  35.9499015808,
		Longitude: -96.7731018066,
		Timezone:  "America/Chicago",
	},
	"KCUL": {
		ICAO:      "KCUL",
		Name:      "Carmi Municipal Airport",
		City:      "Carmi",
		State:     "Illinois",
		Country:   "US",
		Elevation: 388,
		Latitude:  38.0895004272,
		Longitude: -88.1231002808,
		Timezone:  "America/Chicago",
	},
	"KCUT": {
		ICAO:      "KCUT",
		Name:      "Custer County Airport",
		City:      "Custer",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 5602,
		Latitude:  43.7332992554,
		Longitude: -103.6179962158,
		Timezone:  "America/Denver",
	},
	"KCVB": {
		ICAO:      "KCVB",
		Name:      "Castroville Municipal Airport",
		City:      "Castroville",
		State:     "Texas",
		Country:   "US",
		Elevation: 771,
		Latitude:  29.3418998718,
		Longitude: -98.8508987427,
		Timezone:  "America/Chicago",
	},
	"KCVC": {
		ICAO:      "KCVC",
		Name:      "Covington Municipal Airport",
		City:      "Atlanta",
		State:     "Georgia",
		Country:   "US",
		Elevation: 820,
		Latitude:  33.6322475,
		Longitude: -83.8466189,
		Timezone:  "America/New_York",
	},
	"KCVG": {
		ICAO:      "KCVG",
		IATA:      "CVG",
		Name:      "Cincinnati Northern Kentucky International Airport",
		City:      "Hebron",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 896,
		Latitude:  39.0488014221,
		Longitude: -84.6678009033,
		Timezone:  "America/New_York",
	},
	"KCVH": {
		ICAO:      "KCVH",
		IATA:      "HLI",
		Name:      "Hollister Municipal Airport",
		City:      "Hollister",
		State:     "California",
		Country:   "US",
		Elevation: 230,
		Latitude:  36.8932991028,
		Longitude: -121.410003662,
		Timezone:  "America/Los_Angeles",
	},
	"KCVK": {
		ICAO:      "KCVK",
		IATA:      "CKK",
		Name:      "Sharp County Regional Airport",
		City:      "Ash Flat",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 716,
		Latitude:  36.26490021,
		Longitude: -91.56259918,
		Timezone:  "America/Chicago",
	},
	"KCVN": {
		ICAO:      "KCVN",
		IATA:      "CVN",
		Name:      "Clovis Municipal Airport",
		City:      "Clovis",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4216,
		Latitude:  34.4250984192,
		Longitude: -103.07900238,
		Timezone:  "America/Denver",
	},
	"KCVO": {
		ICAO:      "KCVO",
		IATA:      "CVO",
		Name:      "Corvallis Municipal Airport",
		City:      "Corvallis",
		State:     "Oregon",
		Country:   "US",
		Elevation: 250,
		Latitude:  44.49720001,
		Longitude: -123.2900009,
		Timezone:  "America/Los_Angeles",
	},
	"KCVS": {
		ICAO:      "KCVS",
		IATA:      "CVS",
		Name:      "Cannon Air Force Base",
		City:      "Clovis",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4295,
		Latitude:  34.3828010559,
		Longitude: -103.3219985962,
		Timezone:  "America/Denver",
	},
	"KCVX": {
		ICAO:      "KCVX",
		Name:      "Charlevoix Municipal Airport",
		City:      "Charlevoix",
		State:     "Michigan",
		Country:   "US",
		Elevation: 669,
		Latitude:  45.3047981262,
		Longitude: -85.2748031616,
		Timezone:  "America/Detroit",
	},
	"KCWA": {
		ICAO:      "KCWA",
		IATA:      "CWA",
		Name:      "Central Wisconsin Airport",
		City:      "Mosinee",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1277,
		Latitude:  44.7775993347,
		Longitude: -89.6668014526,
		Timezone:  "America/Chicago",
	},
	"KCWC": {
		ICAO:      "KCWC",
		IATA:      "KIP",
		Name:      "Kickapoo Downtown Airport",
		City:      "Wichita Falls",
		State:     "Texas",
		Country:   "US",
		Elevation: 1003,
		Latitude:  33.85779953,
		Longitude: -98.49040222,
		Timezone:  "America/Chicago",
	},
	"KCWF": {
		ICAO:      "KCWF",
		IATA:      "CWF",
		Name:      "Chennault International Airport",
		City:      "Lake Charles",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 17,
		Latitude:  30.2105998993,
		Longitude: -93.1432037354,
		Timezone:  "America/Chicago",
	},
	"KCWI": {
		ICAO:      "KCWI",
		IATA:      "CWI",
		Name:      "Clinton Municipal Airport",
		City:      "Clinton",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 708,
		Latitude:  41.8311004639,
		Longitude: -90.3291015625,
		Timezone:  "America/Chicago",
	},
	"KCWS": {
		ICAO:      "KCWS",
		Name:      "Dennis F Cantrell Field",
		City:      "Conway",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 316,
		Latitude:  35.0807991,
		Longitude: -92.42500305,
		Timezone:  "America/Chicago",
	},
	"KCWV": {
		ICAO:      "KCWV",
		Name:      "Claxton Evans County Airport",
		City:      "Claxton",
		State:     "Georgia",
		Country:   "US",
		Elevation: 112,
		Latitude:  32.195098877,
		Longitude: -81.8695983887,
		Timezone:  "America/New_York",
	},
	"KCXE": {
		ICAO:      "KCXE",
		Name:      "Chase City Municipal Airport",
		City:      "Chase City",
		State:     "Virginia",
		Country:   "US",
		Elevation: 503,
		Latitude:  36.7882995605,
		Longitude: -78.5016021729,
		Timezone:  "America/New_York",
	},
	"KCXL": {
		ICAO:      "KCXL",
		IATA:      "CXL",
		Name:      "Calexico International Airport",
		City:      "Calexico",
		State:     "California",
		Country:   "US",
		Elevation: 4,
		Latitude:  32.6694984436,
		Longitude: -115.5130004883,
		Timezone:  "America/Los_Angeles",
	},
	"KCXO": {
		ICAO:      "KCXO",
		IATA:      "CXO",
		Name:      "Lone Star Executive Airport",
		City:      "Houston",
		State:     "Texas",
		Country:   "US",
		Elevation: 245,
		Latitude:  30.3518009186,
		Longitude: -95.4144973755,
		Timezone:  "America/Chicago",
	},
	"KCXP": {
		ICAO:      "KCXP",
		IATA:      "CSN",
		Name:      "Carson Airport",
		City:      "Carson City",
		State:     "Nevada",
		Country:   "US",
		Elevation: 4697,
		Latitude:  39.192199707,
		Longitude: -119.7340011597,
		Timezone:  "America/Los_Angeles",
	},
	"KCXU": {
		ICAO:      "KCXU",
		Name:      "Camilla Mitchell County Airport",
		City:      "Camilla",
		State:     "Georgia",
		Country:   "US",
		Elevation: 175,
		Latitude:  31.2129001617,
		Longitude: -84.2368011475,
		Timezone:  "America/New_York",
	},
	"KCXY": {
		ICAO:      "KCXY",
		IATA:      "HAR",
		Name:      "Capital City Airport",
		City:      "Harrisburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 347,
		Latitude:  40.2170982361,
		Longitude: -76.8515014648,
		Timezone:  "America/New_York",
	},
	"KCYO": {
		ICAO:      "KCYO",
		Name:      "Pickaway County Memorial Airport",
		City:      "Circleville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 684,
		Latitude:  39.51599884,
		Longitude: -82.98210144,
		Timezone:  "America/New_York",
	},
	"KCYS": {
		ICAO:      "KCYS",
		IATA:      "CYS",
		Name:      "Cheyenne Regional Jerry Olson Field",
		City:      "Cheyenne",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 6159,
		Latitude:  41.15570068,
		Longitude: -104.8119965,
		Timezone:  "America/Denver",
	},
	"KCYW": {
		ICAO:      "KCYW",
		Name:      "Clay Center Municipal Airport",
		City:      "Clay Center",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1208,
		Latitude:  39.3871002197,
		Longitude: -97.1572036743,
		Timezone:  "America/Chicago",
	},
	"KCZD": {
		ICAO:      "KCZD",
		Name:      "Cozad Municipal Airport",
		City:      "Cozad",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2503,
		Latitude:  40.86909866,
		Longitude: -100.0039978,
		Timezone:  "America/Chicago",
	},
	"KCZG": {
		ICAO:      "KCZG",
		Name:      "Tri Cities Airport",
		City:      "Endicott",
		State:     "Alabama",
		Country:   "US",
		Elevation: 833,
		Latitude:  42.0784988403,
		Longitude: -76.0962982178,
		Timezone:  "America/New_York",
	},
	"KCZL": {
		ICAO:      "KCZL",
		Name:      "Tom B. David Field",
		City:      "Calhoun",
		State:     "Georgia",
		Country:   "US",
		Elevation: 647,
		Latitude:  34.45539856,
		Longitude: -84.93920135,
		Timezone:  "America/New_York",
	},
	"KCZT": {
		ICAO:      "KCZT",
		IATA:      "CZT",
		Name:      "Dimmit County Airport",
		City:      "Carrizo Springs",
		State:     "Texas",
		Country:   "US",
		Elevation: 599,
		Latitude:  28.5221996307,
		Longitude: -99.823600769,
		Timezone:  "America/Chicago",
	},
	"KD00": {
		ICAO:      "KD00",
		Name:      "Norman County Ada Twin Valley Airport",
		City:      "Ada/Twin Valley",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 954,
		Latitude:  47.26050186,
		Longitude: -96.40029907,
		Timezone:  "America/Chicago",
	},
	"KD02": {
		ICAO:      "KD02",
		Name:      "Osage Municipal Airport",
		City:      "Osage",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1168,
		Latitude:  43.2924995422,
		Longitude: -92.7959976196,
		Timezone:  "America/Chicago",
	},
	"KD05": {
		ICAO:      "KD05",
		Name:      "Garrison Municipal Airport",
		City:      "Garrison",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1935,
		Latitude:  47.6558990479,
		Longitude: -101.43699646,
		Timezone:  "America/Chicago",
	},
	"KD07": {
		ICAO:      "KD07",
		Name:      "Faith Municipal Airport",
		City:      "Faith",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 2582,
		Latitude:  45.0360984802,
		Longitude: -102.0199966431,
		Timezone:  "America/Denver",
	},
	"KD09": {
		ICAO:      "KD09",
		Name:      "Bottineau Municipal Airport",
		City:      "Bottineau",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1681,
		Latitude:  48.83039856,
		Longitude: -100.4169998,
		Timezone:  "America/Chicago",
	},
	"KD11": {
		ICAO:      "KD11",
		Name:      "Ojibwa Airpark",
		City:      "Weidman",
		State:     "Michigan",
		Country:   "US",
		Elevation: 950,
		Latitude:  43.7200012207,
		Longitude: -85.0027999878,
		Timezone:  "America/Detroit",
	},
	"KD14": {
		ICAO:      "KD14",
		Name:      "Fertile Municipal Airport",
		City:      "Fertile",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1135,
		Latitude:  47.5518989563,
		Longitude: -96.2919006348,
		Timezone:  "America/Chicago",
	},
	"KD17": {
		ICAO:      "KD17",
		Name:      "Westwinds Airport",
		City:      "Delta",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5000,
		Latitude:  38.7574996948,
		Longitude: -108.1480026245,
		Timezone:  "America/Denver",
	},
	"KD23": {
		ICAO:      "KD23",
		Name:      "Arcade Tri County Airport",
		City:      "Arcade",
		State:     "New-York",
		Country:   "US",
		Elevation: 1745,
		Latitude:  42.5666999817,
		Longitude: -78.4261016846,
		Timezone:  "America/New_York",
	},
	"KD25": {
		ICAO:      "KD25",
		Name:      "Manitowish Waters Airport",
		City:      "Manitowish Waters",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1610,
		Latitude:  46.1220016479,
		Longitude: -89.8823013306,
		Timezone:  "America/Chicago",
	},
	"KD31": {
		ICAO:      "KD31",
		Name:      "Leeds Municipal Airport",
		City:      "Leeds",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1508,
		Latitude:  48.2849998474,
		Longitude: -99.403503418,
		Timezone:  "America/Chicago",
	},
	"KD37": {
		ICAO:      "KD37",
		Name:      "Warren Municipal Airport",
		City:      "Warren",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 886,
		Latitude:  48.1911010742,
		Longitude: -96.7110977173,
		Timezone:  "America/Chicago",
	},
	"KD39": {
		ICAO:      "KD39",
		Name:      "Sauk Centre Municipal Airport",
		City:      "Sauk Centre",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1244,
		Latitude:  45.7066001892,
		Longitude: -94.9334030151,
		Timezone:  "America/Chicago",
	},
	"KD42": {
		ICAO:      "KD42",
		Name:      "Springfield Municipal Airport",
		City:      "Springfield",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1072,
		Latitude:  44.231098175,
		Longitude: -94.9989013672,
		Timezone:  "America/Chicago",
	},
	"KD50": {
		ICAO:      "KD50",
		Name:      "Crosby Municipal Airport",
		City:      "Crosby",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1950,
		Latitude:  48.9285011292,
		Longitude: -103.2969970703,
		Timezone:  "America/Chicago",
	},
	"KD54": {
		ICAO:      "KD54",
		Name:      "West Fargo Municipal Airport",
		City:      "Fargo",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 896,
		Latitude:  46.9007987976,
		Longitude: -96.9187011719,
		Timezone:  "America/Chicago",
	},
	"KD55": {
		ICAO:      "KD55",
		Name:      "Robertson Field",
		City:      "Langdon",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1608,
		Latitude:  48.7529983521,
		Longitude: -98.3933029175,
		Timezone:  "America/Chicago",
	},
	"KD56": {
		ICAO:      "KD56",
		Name:      "Mayville Municipal Airport",
		City:      "Mayville",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 975,
		Latitude:  47.4749984741,
		Longitude: -97.3337020874,
		Timezone:  "America/Chicago",
	},
	"KD57": {
		ICAO:      "KD57",
		Name:      "Glen Ullin Regional Airport",
		City:      "Glen Ullin",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 2089,
		Latitude:  46.8128013611,
		Longitude: -101.8600006104,
		Timezone:  "America/North_Dakota/New_Salem",
	},
	"KD60": {
		ICAO:      "KD60",
		Name:      "Tioga Municipal Airport",
		City:      "Tioga",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 2271,
		Latitude:  48.38050079,
		Longitude: -102.8980026,
		Timezone:  "America/Chicago",
	},
	"KD64": {
		ICAO:      "KD64",
		Name:      "Westhope Municipal Airport",
		City:      "Westhope",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1494,
		Latitude:  48.9133987427,
		Longitude: -101.0329971313,
		Timezone:  "America/Chicago",
	},
	"KD68": {
		ICAO:      "KD68",
		Name:      "Springerville Municipal Airport",
		City:      "Springerville",
		State:     "Arizona",
		Country:   "US",
		Elevation: 7055,
		Latitude:  34.13539886,
		Longitude: -109.3099976,
		Timezone:  "America/Phoenix",
	},
	"KD74": {
		ICAO:      "KD74",
		Name:      "Chorman Airport",
		City:      "Farmington",
		State:     "Delaware",
		Country:   "US",
		Elevation: 66,
		Latitude:  38.848400116,
		Longitude: -75.6123962402,
		Timezone:  "America/New_York",
	},
	"KD83": {
		ICAO:      "KD83",
		Name:      "Boonville Airport",
		City:      "Boonville",
		State:     "California",
		Country:   "US",
		Elevation: 371,
		Latitude:  39.0126991272,
		Longitude: -123.3830032349,
		Timezone:  "America/Los_Angeles",
	},
	"KD86": {
		ICAO:      "KD86",
		Name:      "Sequoia Field",
		City:      "Visalia",
		State:     "California",
		Country:   "US",
		Elevation: 313,
		Latitude:  36.448600769,
		Longitude: -119.3190002441,
		Timezone:  "America/Los_Angeles",
	},
	"KD95": {
		ICAO:      "KD95",
		Name:      "Dupont Lapeer Airport",
		City:      "Lapeer",
		State:     "Michigan",
		Country:   "US",
		Elevation: 834,
		Latitude:  43.0666007996,
		Longitude: -83.2723007202,
		Timezone:  "America/Detroit",
	},
	"KD98": {
		ICAO:      "KD98",
		Name:      "Romeo State Airport",
		City:      "Romeo",
		State:     "Michigan",
		Country:   "US",
		Elevation: 739,
		Latitude:  42.79610062,
		Longitude: -82.97499847,
		Timezone:  "America/Detroit",
	},
	"KDAA": {
		ICAO:      "KDAA",
		IATA:      "DAA",
		Name:      "Davison Army Air Field",
		City:      "Fort Belvoir",
		State:     "Virginia",
		Country:   "US",
		Elevation: 73,
		Latitude:  38.7150001526,
		Longitude: -77.1809997559,
		Timezone:  "America/New_York",
	},
	"KDAB": {
		ICAO:      "KDAB",
		IATA:      "DAB",
		Name:      "Daytona Beach International Airport",
		City:      "Daytona Beach",
		State:     "Florida",
		Country:   "US",
		Elevation: 34,
		Latitude:  29.1798992157,
		Longitude: -81.0580978394,
		Timezone:  "America/New_York",
	},
	"KDAF": {
		ICAO:      "KDAF",
		Name:      "Necedah Airport",
		City:      "Necedah",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 919,
		Latitude:  44.0334014893,
		Longitude: -90.0850982666,
		Timezone:  "America/Chicago",
	},
	"KDAG": {
		ICAO:      "KDAG",
		IATA:      "DAG",
		Name:      "Barstow Daggett Airport",
		City:      "Daggett",
		State:     "California",
		Country:   "US",
		Elevation: 1930,
		Latitude:  34.85369873,
		Longitude: -116.7870026,
		Timezone:  "America/Los_Angeles",
	},
	"KDAL": {
		ICAO:      "KDAL",
		IATA:      "DAL",
		Name:      "Dallas Love Field",
		City:      "Dallas",
		State:     "Texas",
		Country:   "US",
		Elevation: 487,
		Latitude:  32.8470993042,
		Longitude: -96.8517990112,
		Timezone:  "America/Chicago",
	},
	"KDAN": {
		ICAO:      "KDAN",
		IATA:      "DAN",
		Name:      "Danville Regional Airport",
		City:      "Danville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 571,
		Latitude:  36.5728988647,
		Longitude: -79.3360977173,
		Timezone:  "America/New_York",
	},
	"KDAW": {
		ICAO:      "KDAW",
		Name:      "Skyhaven Airport",
		City:      "Rochester",
		State:     "Missouri",
		Country:   "US",
		Elevation: 322,
		Latitude:  43.2840995789,
		Longitude: -70.9292984009,
		Timezone:  "America/New_York",
	},
	"KDAY": {
		ICAO:      "KDAY",
		IATA:      "DAY",
		Name:      "James M Cox Dayton International Airport",
		City:      "Dayton",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1009,
		Latitude:  39.9024009705,
		Longitude: -84.2193984985,
		Timezone:  "America/New_York",
	},
	"KDBN": {
		ICAO:      "KDBN",
		IATA:      "DBN",
		Name:      "W H 'Bud' Barron Airport",
		City:      "Dublin",
		State:     "Georgia",
		Country:   "US",
		Elevation: 309,
		Latitude:  32.56439972,
		Longitude: -82.98529816,
		Timezone:  "America/New_York",
	},
	"KDBQ": {
		ICAO:      "KDBQ",
		IATA:      "DBQ",
		Name:      "Dubuque Regional Airport",
		City:      "Dubuque",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1077,
		Latitude:  42.40200043,
		Longitude: -90.70950317,
		Timezone:  "America/Chicago",
	},
	"KDCA": {
		ICAO:      "KDCA",
		IATA:      "DCA",
		Name:      "Ronald Reagan Washington National Airport",
		City:      "Washington",
		State:     "Virginia",
		Country:   "US",
		Elevation: 15,
		Latitude:  38.8521003723,
		Longitude: -77.0376968384,
		Timezone:  "America/New_York",
	},
	"KDCM": {
		ICAO:      "KDCM",
		Name:      "Chester Catawba Regional Airport",
		City:      "Chester",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 656,
		Latitude:  34.7892990112,
		Longitude: -81.1958007813,
		Timezone:  "America/New_York",
	},
	"KDCU": {
		ICAO:      "KDCU",
		IATA:      "DCU",
		Name:      "Pryor Field Regional Airport",
		City:      "Decatur",
		State:     "Alabama",
		Country:   "US",
		Elevation: 592,
		Latitude:  34.6526985168,
		Longitude: -86.9453964233,
		Timezone:  "America/Chicago",
	},
	"KDCY": {
		ICAO:      "KDCY",
		Name:      "Daviess County Airport",
		City:      "Washington",
		State:     "Indiana",
		Country:   "US",
		Elevation: 473,
		Latitude:  38.7004013062,
		Longitude: -87.129699707,
		Timezone:  "America/Indiana/Vincennes",
	},
	"KDDC": {
		ICAO:      "KDDC",
		IATA:      "DDC",
		Name:      "Dodge City Regional Airport",
		City:      "Dodge City",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2594,
		Latitude:  37.7634010315,
		Longitude: -99.9655990601,
		Timezone:  "America/Chicago",
	},
	"KDDH": {
		ICAO:      "KDDH",
		Name:      "William H. Morse State Airport",
		City:      "Bennington",
		State:     "Vermont",
		Country:   "US",
		Elevation: 827,
		Latitude:  42.8913002,
		Longitude: -73.24639893,
		Timezone:  "America/New_York",
	},
	"KDEC": {
		ICAO:      "KDEC",
		IATA:      "DEC",
		Name:      "Decatur Airport",
		City:      "Decatur",
		State:     "Illinois",
		Country:   "US",
		Elevation: 682,
		Latitude:  39.8345985413,
		Longitude: -88.8656997681,
		Timezone:  "America/Chicago",
	},
	"KDED": {
		ICAO:      "KDED",
		Name:      "Deland Municipal Sidney H Taylor Field",
		City:      "Deland",
		State:     "Florida",
		Country:   "US",
		Elevation: 79,
		Latitude:  29.06699944,
		Longitude: -81.28379822,
		Timezone:  "America/New_York",
	},
	"KDEH": {
		ICAO:      "KDEH",
		IATA:      "DEH",
		Name:      "Decorah Municipal Airport",
		City:      "Decorah",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1158,
		Latitude:  43.27550125,
		Longitude: -91.73940277,
		Timezone:  "America/Chicago",
	},
	"KDEN": {
		ICAO:      "KDEN",
		IATA:      "DEN",
		Name:      "Denver International Airport",
		City:      "Denver",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5431,
		Latitude:  39.8616981506,
		Longitude: -104.672996521,
		Timezone:  "America/Denver",
	},
	"KDEQ": {
		ICAO:      "KDEQ",
		Name:      "J Lynn Helms Sevier County Airport",
		City:      "De Queen",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 355,
		Latitude:  34.047000885,
		Longitude: -94.3993988037,
		Timezone:  "America/Chicago",
	},
	"KDET": {
		ICAO:      "KDET",
		IATA:      "DET",
		Name:      "Coleman A. Young Municipal Airport",
		City:      "Detroit",
		State:     "Michigan",
		Country:   "US",
		Elevation: 626,
		Latitude:  42.40919876,
		Longitude: -83.00990295,
		Timezone:  "America/Detroit",
	},
	"KDEW": {
		ICAO:      "KDEW",
		Name:      "Deer Park Airport",
		City:      "Deer Park",
		State:     "Washington",
		Country:   "US",
		Elevation: 2211,
		Latitude:  47.96659851,
		Longitude: -117.427002,
		Timezone:  "America/Los_Angeles",
	},
	"KDFI": {
		ICAO:      "KDFI",
		IATA:      "DFI",
		Name:      "Defiance Memorial Airport",
		City:      "Defiance",
		State:     "Ohio",
		Country:   "US",
		Elevation: 707,
		Latitude:  41.3375015259,
		Longitude: -84.4288024902,
		Timezone:  "America/New_York",
	},
	"KDFW": {
		ICAO:      "KDFW",
		IATA:      "DFW",
		Name:      "Dallas Fort Worth International Airport",
		City:      "Dallas-Fort Worth",
		State:     "Texas",
		Country:   "US",
		Elevation: 607,
		Latitude:  32.8968009949,
		Longitude: -97.0380020142,
		Timezone:  "America/Chicago",
	},
	"KDGL": {
		ICAO:      "KDGL",
		IATA:      "DGL",
		Name:      "Douglas Municipal Airport",
		City:      "Douglas",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4173,
		Latitude:  31.3425998688,
		Longitude: -109.505996704,
		Timezone:  "America/Hermosillo",
	},
	"KDGW": {
		ICAO:      "KDGW",
		IATA:      "DGW",
		Name:      "Converse County Airport",
		City:      "Douglas",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4933,
		Latitude:  42.79719925,
		Longitude: -105.3860016,
		Timezone:  "America/Denver",
	},
	"KDHN": {
		ICAO:      "KDHN",
		IATA:      "DHN",
		Name:      "Dothan Regional Airport",
		City:      "Dothan",
		State:     "Alabama",
		Country:   "US",
		Elevation: 401,
		Latitude:  31.3213005066,
		Longitude: -85.4496002197,
		Timezone:  "America/Chicago",
	},
	"KDHT": {
		ICAO:      "KDHT",
		IATA:      "DHT",
		Name:      "Dalhart Municipal Airport",
		City:      "Dalhart",
		State:     "Texas",
		Country:   "US",
		Elevation: 3991,
		Latitude:  36.0225982666,
		Longitude: -102.54699707,
		Timezone:  "America/Chicago",
	},
	"KDIJ": {
		ICAO:      "KDIJ",
		Name:      "Driggs Reed Memorial Airport",
		City:      "Driggs",
		State:     "Idaho",
		Country:   "US",
		Elevation: 6229,
		Latitude:  43.742401123,
		Longitude: -111.0979995728,
		Timezone:  "America/Boise",
	},
	"KDIK": {
		ICAO:      "KDIK",
		IATA:      "DIK",
		Name:      "Dickinson Theodore Roosevelt Regional Airport",
		City:      "Dickinson",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 2592,
		Latitude:  46.7974014282,
		Longitude: -102.802001953,
		Timezone:  "America/Denver",
	},
	"KDKB": {
		ICAO:      "KDKB",
		Name:      "De Kalb Taylor Municipal Airport",
		City:      "De Kalb",
		State:     "Illinois",
		Country:   "US",
		Elevation: 914,
		Latitude:  41.93370056,
		Longitude: -88.70570374,
		Timezone:  "America/Chicago",
	},
	"KDKK": {
		ICAO:      "KDKK",
		IATA:      "DKK",
		Name:      "Chautauqua County-Dunkirk Airport",
		City:      "Dunkirk",
		State:     "New-York",
		Country:   "US",
		Elevation: 693,
		Latitude:  42.49330139,
		Longitude: -79.27200317,
		Timezone:  "America/New_York",
	},
	"KDKR": {
		ICAO:      "KDKR",
		Name:      "Houston County Airport",
		City:      "Crockett",
		State:     "Texas",
		Country:   "US",
		Elevation: 348,
		Latitude:  31.3069992065,
		Longitude: -95.4038009644,
		Timezone:  "America/Chicago",
	},
	"KDKX": {
		ICAO:      "KDKX",
		Name:      "Knoxville Downtown Island Airport",
		City:      "Knoxville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 833,
		Latitude:  35.9639015198,
		Longitude: -83.8739013672,
		Timezone:  "America/New_York",
	},
	"KDLC": {
		ICAO:      "KDLC",
		IATA:      "DLL",
		Name:      "Dillon County Airport",
		City:      "Dillon",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 133,
		Latitude:  34.4491004944,
		Longitude: -79.368598938,
		Timezone:  "America/New_York",
	},
	"KDLF": {
		ICAO:      "KDLF",
		IATA:      "DLF",
		Name:      "Laughlin Air Force Base",
		City:      "Del Rio",
		State:     "Texas",
		Country:   "US",
		Elevation: 1082,
		Latitude:  29.359500885,
		Longitude: -100.7779998779,
		Timezone:  "America/Chicago",
	},
	"KDLH": {
		ICAO:      "KDLH",
		IATA:      "DLH",
		Name:      "Duluth International Airport",
		City:      "Duluth",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1428,
		Latitude:  46.8420982361,
		Longitude: -92.1936035156,
		Timezone:  "America/Chicago",
	},
	"KDLL": {
		ICAO:      "KDLL",
		Name:      "Baraboo Wisconsin Dells Airport",
		City:      "Baraboo",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 979,
		Latitude:  43.52270126,
		Longitude: -89.77020264,
		Timezone:  "America/Chicago",
	},
	"KDLN": {
		ICAO:      "KDLN",
		IATA:      "DLN",
		Name:      "Dillon Airport",
		City:      "Dillon",
		State:     "Montana",
		Country:   "US",
		Elevation: 5241,
		Latitude:  45.2554016113,
		Longitude: -112.5530014038,
		Timezone:  "America/Denver",
	},
	"KDLO": {
		ICAO:      "KDLO",
		Name:      "Delano Municipal Airport",
		City:      "Delano",
		State:     "California",
		Country:   "US",
		Elevation: 314,
		Latitude:  35.7456016541,
		Longitude: -119.2369995117,
		Timezone:  "America/Los_Angeles",
	},
	"KDLS": {
		ICAO:      "KDLS",
		IATA:      "DLS",
		Name:      "Columbia Gorge Regional the Dalles Municipal Airport",
		City:      "The Dalles",
		State:     "Washington",
		Country:   "US",
		Elevation: 247,
		Latitude:  45.6184997559,
		Longitude: -121.1669998169,
		Timezone:  "America/Los_Angeles",
	},
	"KDLZ": {
		ICAO:      "KDLZ",
		Name:      "Delaware Municipal Airport",
		City:      "Delaware",
		State:     "Ohio",
		Country:   "US",
		Elevation: 945,
		Latitude:  40.2797012329,
		Longitude: -83.1147994995,
		Timezone:  "America/New_York",
	},
	"KDMA": {
		ICAO:      "KDMA",
		IATA:      "DMA",
		Name:      "Davis Monthan Air Force Base",
		City:      "Tucson",
		State:     "Arizona",
		Country:   "US",
		Elevation: 2704,
		Latitude:  32.1665000916,
		Longitude: -110.8830032349,
		Timezone:  "America/Phoenix",
	},
	"KDMN": {
		ICAO:      "KDMN",
		IATA:      "DMN",
		Name:      "Deming Municipal Airport",
		City:      "Deming",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4314,
		Latitude:  32.262298584,
		Longitude: -107.7210006714,
		Timezone:  "America/Denver",
	},
	"KDMO": {
		ICAO:      "KDMO",
		IATA:      "DMO",
		Name:      "Sedalia Memorial Airport",
		City:      "Sedalia",
		State:     "Missouri",
		Country:   "US",
		Elevation: 909,
		Latitude:  38.7074012756,
		Longitude: -93.1759033203,
		Timezone:  "America/Chicago",
	},
	"KDMW": {
		ICAO:      "KDMW",
		Name:      "Carroll County Regional Jack B Poage Field",
		City:      "Westminster",
		State:     "Maryland",
		Country:   "US",
		Elevation: 789,
		Latitude:  39.60829926,
		Longitude: -77.00769806,
		Timezone:  "America/New_York",
	},
	"KDNL": {
		ICAO:      "KDNL",
		IATA:      "DNL",
		Name:      "Daniel Field",
		City:      "Augusta",
		State:     "Georgia",
		Country:   "US",
		Elevation: 423,
		Latitude:  33.4664993286,
		Longitude: -82.0393981934,
		Timezone:  "America/New_York",
	},
	"KDNN": {
		ICAO:      "KDNN",
		IATA:      "DNN",
		Name:      "Dalton Municipal Airport",
		City:      "Dalton",
		State:     "Georgia",
		Country:   "US",
		Elevation: 709,
		Latitude:  34.72290039,
		Longitude: -84.87020111,
		Timezone:  "America/New_York",
	},
	"KDNS": {
		ICAO:      "KDNS",
		IATA:      "DNS",
		Name:      "Denison Municipal Airport",
		City:      "Denison",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1274,
		Latitude:  41.9864006,
		Longitude: -95.38069916,
		Timezone:  "America/Chicago",
	},
	"KDNV": {
		ICAO:      "KDNV",
		IATA:      "DNV",
		Name:      "Vermilion Regional Airport",
		City:      "Danville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 697,
		Latitude:  40.19919968,
		Longitude: -87.59590149,
		Timezone:  "America/Chicago",
	},
	"KDOV": {
		ICAO:      "KDOV",
		IATA:      "DOV",
		Name:      "Dover Air Force Base",
		City:      "Dover",
		State:     "Delaware",
		Country:   "US",
		Elevation: 24,
		Latitude:  39.12950134,
		Longitude: -75.46600342,
		Timezone:  "America/New_York",
	},
	"KDPA": {
		ICAO:      "KDPA",
		IATA:      "DPA",
		Name:      "Dupage Airport",
		City:      "Chicago/West Chicago",
		State:     "Illinois",
		Country:   "US",
		Elevation: 759,
		Latitude:  41.90779877,
		Longitude: -88.24859619,
		Timezone:  "America/Chicago",
	},
	"KDPG": {
		ICAO:      "KDPG",
		IATA:      "DPG",
		Name:      "Michael AAF (Dugway Proving Ground) Airport",
		City:      "Dugway Proving Ground",
		State:     "Utah",
		Country:   "US",
		Elevation: 4349,
		Latitude:  40.19940186,
		Longitude: -112.9369965,
		Timezone:  "America/Denver",
	},
	"KDPL": {
		ICAO:      "KDPL",
		Name:      "Duplin County Airport",
		City:      "Kenansville",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 136,
		Latitude:  35.00009918,
		Longitude: -77.98169708,
		Timezone:  "America/New_York",
	},
	"KDQH": {
		ICAO:      "KDQH",
		Name:      "Douglas Municipal Airport",
		City:      "Douglas",
		State:     "Arizona",
		Country:   "US",
		Elevation: 257,
		Latitude:  31.4766998291,
		Longitude: -82.860496521,
		Timezone:  "America/New_York",
	},
	"KDRA": {
		ICAO:      "KDRA",
		IATA:      "DRA",
		Name:      "Desert Rock Airport",
		City:      "Mercury",
		State:     "Nevada",
		Country:   "US",
		Elevation: 3314,
		Latitude:  36.6194000244,
		Longitude: -116.032997131,
		Timezone:  "America/Los_Angeles",
	},
	"KDRI": {
		ICAO:      "KDRI",
		IATA:      "DRI",
		Name:      "Beauregard Regional Airport",
		City:      "De Ridder",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 202,
		Latitude:  30.8316993713,
		Longitude: -93.3398971558,
		Timezone:  "America/Chicago",
	},
	"KDRM": {
		ICAO:      "KDRM",
		IATA:      "DRE",
		Name:      "Drummond Island Airport",
		City:      "Drummond Island",
		State:     "Michigan",
		Country:   "US",
		Elevation: 668,
		Latitude:  46.0093002319,
		Longitude: -83.7438964844,
		Timezone:  "America/Detroit",
	},
	"KDRO": {
		ICAO:      "KDRO",
		IATA:      "DRO",
		Name:      "Durango La Plata County Airport",
		City:      "Durango",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6685,
		Latitude:  37.1515007019,
		Longitude: -107.753997803,
		Timezone:  "America/Denver",
	},
	"KDRT": {
		ICAO:      "KDRT",
		IATA:      "DRT",
		Name:      "Del Rio International Airport",
		City:      "Del Rio",
		State:     "Texas",
		Country:   "US",
		Elevation: 1002,
		Latitude:  29.3742008209,
		Longitude: -100.927001953,
		Timezone:  "America/Chicago",
	},
	"KDSM": {
		ICAO:      "KDSM",
		IATA:      "DSM",
		Name:      "Des Moines International Airport",
		City:      "Des Moines",
		State:     "Iowa",
		Country:   "US",
		Elevation: 958,
		Latitude:  41.5340003967,
		Longitude: -93.6631011963,
		Timezone:  "America/Chicago",
	},
	"KDSV": {
		ICAO:      "KDSV",
		IATA:      "DSV",
		Name:      "Dansville Municipal Airport",
		City:      "Dansville",
		State:     "New-York",
		Country:   "US",
		Elevation: 662,
		Latitude:  42.5708999634,
		Longitude: -77.7130966187,
		Timezone:  "America/New_York",
	},
	"KDTA": {
		ICAO:      "KDTA",
		IATA:      "DTA",
		Name:      "Delta Municipal Airport",
		City:      "Delta",
		State:     "Utah",
		Country:   "US",
		Elevation: 4759,
		Latitude:  39.3805999756,
		Longitude: -112.508003235,
		Timezone:  "America/Denver",
	},
	"KDTG": {
		ICAO:      "KDTG",
		Name:      "Dwight Airport",
		City:      "Dwight",
		State:     "Illinois",
		Country:   "US",
		Elevation: 632,
		Latitude:  41.1333007812,
		Longitude: -88.4408035278,
		Timezone:  "America/Chicago",
	},
	"KDTL": {
		ICAO:      "KDTL",
		IATA:      "DTL",
		Name:      "Detroit Lakes Airport - Wething Field",
		City:      "Detroit Lakes",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1397,
		Latitude:  46.82519913,
		Longitude: -95.88569641,
		Timezone:  "America/Chicago",
	},
	"KDTN": {
		ICAO:      "KDTN",
		IATA:      "DTN",
		Name:      "Shreveport Downtown Airport",
		City:      "Shreveport",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 179,
		Latitude:  32.5401992798,
		Longitude: -93.7450027466,
		Timezone:  "America/Chicago",
	},
	"KDTO": {
		ICAO:      "KDTO",
		Name:      "Denton Municipal Airport",
		City:      "Denton",
		State:     "Texas",
		Country:   "US",
		Elevation: 642,
		Latitude:  33.2006988525,
		Longitude: -97.1979980469,
		Timezone:  "America/Chicago",
	},
	"KDTS": {
		ICAO:      "KDTS",
		IATA:      "DSI",
		Name:      "Destin Fort Walton Beach Airport",
		City:      "Destin",
		State:     "Florida",
		Country:   "US",
		Elevation: 23,
		Latitude:  30.40010071,
		Longitude: -86.47149658,
		Timezone:  "America/Chicago",
	},
	"KDTW": {
		ICAO:      "KDTW",
		IATA:      "DTW",
		Name:      "Detroit Metropolitan Wayne County Airport",
		City:      "Detroit",
		State:     "Michigan",
		Country:   "US",
		Elevation: 645,
		Latitude:  42.2123985291,
		Longitude: -83.3534011841,
		Timezone:  "America/Detroit",
	},
	"KDUA": {
		ICAO:      "KDUA",
		IATA:      "DUA",
		Name:      "Eaker Field",
		City:      "Durant",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 699,
		Latitude:  33.94229889,
		Longitude: -96.39450073,
		Timezone:  "America/Chicago",
	},
	"KDUC": {
		ICAO:      "KDUC",
		IATA:      "DUC",
		Name:      "Halliburton Field",
		City:      "Duncan",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1114,
		Latitude:  34.47090149,
		Longitude: -97.9598999,
		Timezone:  "America/Chicago",
	},
	"KDUG": {
		ICAO:      "KDUG",
		IATA:      "DUG",
		Name:      "Bisbee Douglas International Airport",
		City:      "Douglas Bisbee",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4154,
		Latitude:  31.4689998627,
		Longitude: -109.603996277,
		Timezone:  "America/Phoenix",
	},
	"KDUH": {
		ICAO:      "KDUH",
		Name:      "Toledo Suburban Airport",
		City:      "Lambertville",
		State:     "Michigan",
		Country:   "US",
		Elevation: 669,
		Latitude:  41.7359008789,
		Longitude: -83.6554031372,
		Timezone:  "America/Detroit",
	},
	"KDUJ": {
		ICAO:      "KDUJ",
		IATA:      "DUJ",
		Name:      "DuBois Regional Airport",
		City:      "Dubois",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1817,
		Latitude:  41.17829895,
		Longitude: -78.8986969,
		Timezone:  "America/New_York",
	},
	"KDUX": {
		ICAO:      "KDUX",
		Name:      "Moore County Airport",
		City:      "Dumas",
		State:     "Texas",
		Country:   "US",
		Elevation: 3705,
		Latitude:  35.8578987122,
		Longitude: -102.0130004883,
		Timezone:  "America/Chicago",
	},
	"KDVK": {
		ICAO:      "KDVK",
		Name:      "Stuart Powell Field",
		City:      "Danville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 1022,
		Latitude:  37.5778999329,
		Longitude: -84.7696990967,
		Timezone:  "America/New_York",
	},
	"KDVL": {
		ICAO:      "KDVL",
		IATA:      "DVL",
		Name:      "Devils Lake Regional Airport",
		City:      "Devils Lake",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1456,
		Latitude:  48.11420059,
		Longitude: -98.90879822,
		Timezone:  "America/Chicago",
	},
	"KDVN": {
		ICAO:      "KDVN",
		IATA:      "DVN",
		Name:      "Davenport Municipal Airport",
		City:      "Davenport",
		State:     "Iowa",
		Country:   "US",
		Elevation: 751,
		Latitude:  41.61029816,
		Longitude: -90.58830261,
		Timezone:  "America/Chicago",
	},
	"KDVO": {
		ICAO:      "KDVO",
		IATA:      "NOT",
		Name:      "Gnoss Field",
		City:      "Novato",
		State:     "California",
		Country:   "US",
		Elevation: 2,
		Latitude:  38.1436004639,
		Longitude: -122.5559997559,
		Timezone:  "America/Los_Angeles",
	},
	"KDVP": {
		ICAO:      "KDVP",
		IATA:      "NSL",
		Name:      "Slayton Municipal Airport",
		City:      "Slayton",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1623,
		Latitude:  43.9868011475,
		Longitude: -95.7826004028,
		Timezone:  "America/Chicago",
	},
	"KDVT": {
		ICAO:      "KDVT",
		IATA:      "DVT",
		Name:      "Phoenix Deer Valley Airport",
		City:      "Phoenix",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1478,
		Latitude:  33.6883010864,
		Longitude: -112.083000183,
		Timezone:  "America/Phoenix",
	},
	"KDWA": {
		ICAO:      "KDWA",
		Name:      "Yolo County Davis Woodland Winters Airport",
		City:      "Davis/Woodland/Winters",
		State:     "California",
		Country:   "US",
		Elevation: 100,
		Latitude:  38.57910156,
		Longitude: -121.8570023,
		Timezone:  "America/Los_Angeles",
	},
	"KDWH": {
		ICAO:      "KDWH",
		IATA:      "DWH",
		Name:      "David Wayne Hooks Memorial Airport",
		City:      "Houston",
		State:     "Texas",
		Country:   "US",
		Elevation: 152,
		Latitude:  30.0618000031,
		Longitude: -95.5528030396,
		Timezone:  "America/Chicago",
	},
	"KDWU": {
		ICAO:      "KDWU",
		Name:      "Ashland Regional Airport",
		City:      "Ashland",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 546,
		Latitude:  38.5545005798,
		Longitude: -82.7379989624,
		Timezone:  "America/New_York",
	},
	"KDWX": {
		ICAO:      "KDWX",
		Name:      "Dixon Airport",
		City:      "Dixon",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 6520,
		Latitude:  41.0382995605,
		Longitude: -107.4970016479,
		Timezone:  "America/Denver",
	},
	"KDXE": {
		ICAO:      "KDXE",
		Name:      "Dexter Municipal Airport",
		City:      "Dexter",
		State:     "Missouri",
		Country:   "US",
		Elevation: 304,
		Latitude:  36.77750015,
		Longitude: -89.94120026,
		Timezone:  "America/Chicago",
	},
	"KDXR": {
		ICAO:      "KDXR",
		IATA:      "DXR",
		Name:      "Danbury Municipal Airport",
		City:      "Danbury",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 458,
		Latitude:  41.3714981079,
		Longitude: -73.4822006226,
		Timezone:  "America/New_York",
	},
	"KDXX": {
		ICAO:      "KDXX",
		Name:      "Lac Qui Parle County Airport",
		City:      "Madison",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1082,
		Latitude:  44.9861984253,
		Longitude: -96.177696228,
		Timezone:  "America/Chicago",
	},
	"KDYA": {
		ICAO:      "KDYA",
		Name:      "Demopolis Municipal Airport",
		City:      "Demopolis",
		State:     "Alabama",
		Country:   "US",
		Elevation: 113,
		Latitude:  32.4637985229,
		Longitude: -87.9541015625,
		Timezone:  "America/Chicago",
	},
	"KDYB": {
		ICAO:      "KDYB",
		Name:      "Summerville Airport",
		City:      "Summerville",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 56,
		Latitude:  33.0634002686,
		Longitude: -80.279296875,
		Timezone:  "America/New_York",
	},
	"KDYL": {
		ICAO:      "KDYL",
		IATA:      "DYL",
		Name:      "Doylestown Airport",
		City:      "Doylestown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 394,
		Latitude:  40.3330001831,
		Longitude: -75.1222991943,
		Timezone:  "America/New_York",
	},
	"KDYR": {
		ICAO:      "KDYR",
		Name:      "Dyersburg Regional Airport",
		City:      "Dyersburg",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 338,
		Latitude:  35.9980010986,
		Longitude: -89.4066009521,
		Timezone:  "America/Chicago",
	},
	"KDYS": {
		ICAO:      "KDYS",
		IATA:      "DYS",
		Name:      "Dyess Air Force Base",
		City:      "Abilene",
		State:     "Texas",
		Country:   "US",
		Elevation: 1789,
		Latitude:  32.4207992554,
		Longitude: -99.854598999,
		Timezone:  "America/Chicago",
	},
	"KDYT": {
		ICAO:      "KDYT",
		Name:      "Sky Harbor Airport",
		City:      "Duluth",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 610,
		Latitude:  46.7219009399,
		Longitude: -92.0434036255,
		Timezone:  "America/Chicago",
	},
	"KDZB": {
		ICAO:      "KDZB",
		Name:      "Horseshoe Bay Resort Airpark",
		City:      "Horseshoe Bay",
		State:     "Texas",
		Country:   "US",
		Elevation: 1093,
		Latitude:  30.52709961,
		Longitude: -98.35870361,
		Timezone:  "America/Chicago",
	},
	"KDZJ": {
		ICAO:      "KDZJ",
		Name:      "Blairsville Airport",
		City:      "Blairsville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1911,
		Latitude:  34.8550987244,
		Longitude: -83.9969024658,
		Timezone:  "America/New_York",
	},
	"KE01": {
		ICAO:      "KE01",
		Name:      "Roy Hurd Memorial Airport",
		City:      "Monahans",
		State:     "Texas",
		Country:   "US",
		Elevation: 2615,
		Latitude:  31.5825004578,
		Longitude: -102.908996582,
		Timezone:  "America/Chicago",
	},
	"KE05": {
		ICAO:      "KE05",
		Name:      "Hatch Municipal Airport",
		City:      "Hatch",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4080,
		Latitude:  32.6610984802,
		Longitude: -107.1979980469,
		Timezone:  "America/Denver",
	},
	"KE06": {
		ICAO:      "KE06",
		Name:      "Lea County-Zip Franklin Memorial Airport",
		City:      "Lovington",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 3979,
		Latitude:  32.95389938,
		Longitude: -103.4089966,
		Timezone:  "America/Denver",
	},
	"KE11": {
		ICAO:      "KE11",
		Name:      "Andrews County Airport",
		City:      "Andrews",
		State:     "Texas",
		Country:   "US",
		Elevation: 3174,
		Latitude:  32.3311004639,
		Longitude: -102.5299987793,
		Timezone:  "America/Chicago",
	},
	"KE13": {
		ICAO:      "KE13",
		IATA:      "CCG",
		Name:      "Crane County Airport",
		City:      "Crane",
		State:     "Texas",
		Country:   "US",
		Elevation: 2552,
		Latitude:  31.4151000977,
		Longitude: -102.362998962,
		Timezone:  "America/Chicago",
	},
	"KE14": {
		ICAO:      "KE14",
		Name:      "Ohkay Owingeh Airport",
		City:      "Espanola",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 5790,
		Latitude:  36.0299987793,
		Longitude: -106.04599762,
		Timezone:  "America/Denver",
	},
	"KE16": {
		ICAO:      "KE16",
		Name:      "South County Airport of Santa Clara County Airport",
		City:      "San Martin",
		State:     "California",
		Country:   "US",
		Elevation: 281,
		Latitude:  37.08160019,
		Longitude: -121.5970001,
		Timezone:  "America/Los_Angeles",
	},
	"KE19": {
		ICAO:      "KE19",
		Name:      "Gruver Municipal Airport",
		City:      "Gruver",
		State:     "Texas",
		Country:   "US",
		Elevation: 3205,
		Latitude:  36.2336997986,
		Longitude: -101.4319992065,
		Timezone:  "America/Chicago",
	},
	"KE24": {
		ICAO:      "KE24",
		Name:      "Whiteriver Airport",
		City:      "Whiteriver",
		State:     "Arizona",
		Country:   "US",
		Elevation: 5153,
		Latitude:  33.8125,
		Longitude: -109.9869995,
		Timezone:  "America/Phoenix",
	},
	"KE25": {
		ICAO:      "KE25",
		Name:      "Wickenburg Municipal Airport",
		City:      "Wickenburg",
		State:     "Arizona",
		Country:   "US",
		Elevation: 2377,
		Latitude:  33.96889877,
		Longitude: -112.7990036,
		Timezone:  "America/Phoenix",
	},
	"KE26": {
		ICAO:      "KE26",
		Name:      "Lea County Jal Airport",
		City:      "Jal",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 3118,
		Latitude:  32.1310997009,
		Longitude: -103.1549987793,
		Timezone:  "America/Denver",
	},
	"KE30": {
		ICAO:      "KE30",
		Name:      "Bruce Field",
		City:      "Ballinger",
		State:     "Texas",
		Country:   "US",
		Elevation: 1738,
		Latitude:  31.6744995117,
		Longitude: -99.9769973755,
		Timezone:  "America/Chicago",
	},
	"KE34": {
		ICAO:      "KE34",
		Name:      "Smiley Johnson Municipal-Bass Field",
		City:      "Clarendon",
		State:     "Texas",
		Country:   "US",
		Elevation: 2833,
		Latitude:  34.91149902,
		Longitude: -100.8690033,
		Timezone:  "America/Chicago",
	},
	"KE35": {
		ICAO:      "KE35",
		Name:      "Fabens Airport",
		City:      "Fabens",
		State:     "Texas",
		Country:   "US",
		Elevation: 3679,
		Latitude:  31.5156993866,
		Longitude: -106.1470031738,
		Timezone:  "America/Denver",
	},
	"KE38": {
		ICAO:      "KE38",
		Name:      "Alpine Casparis Municipal Airport",
		City:      "Alpine",
		State:     "Texas",
		Country:   "US",
		Elevation: 4515,
		Latitude:  30.3841991425,
		Longitude: -103.683998108,
		Timezone:  "America/Chicago",
	},
	"KE41": {
		ICAO:      "KE41",
		IATA:      "BGQ",
		Name:      "Reagan County Airport",
		City:      "Big Lake",
		State:     "Texas",
		Country:   "US",
		Elevation: 2706,
		Latitude:  31.1989002228,
		Longitude: -101.472999573,
		Timezone:  "America/Chicago",
	},
	"KE42": {
		ICAO:      "KE42",
		Name:      "Spearman Municipal Airport",
		City:      "Spearman",
		State:     "Texas",
		Country:   "US",
		Elevation: 3090,
		Latitude:  36.2210006714,
		Longitude: -101.1949996948,
		Timezone:  "America/Chicago",
	},
	"KE45": {
		ICAO:      "KE45",
		Name:      "Pine Mountain Lake Airport",
		City:      "Groveland",
		State:     "California",
		Country:   "US",
		Elevation: 2930,
		Latitude:  37.8616981506,
		Longitude: -120.1780014038,
		Timezone:  "America/Los_Angeles",
	},
	"KE48": {
		ICAO:      "KE48",
		Name:      "Upton County Airport",
		City:      "Mc Camey",
		State:     "Texas",
		Country:   "US",
		Elevation: 2433,
		Latitude:  31.1250991821,
		Longitude: -102.2249984741,
		Timezone:  "America/Chicago",
	},
	"KE51": {
		ICAO:      "KE51",
		IATA:      "BGT",
		Name:      "Bagdad Airport",
		City:      "Bagdad",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4183,
		Latitude:  34.5959014893,
		Longitude: -113.1699981689,
		Timezone:  "America/Phoenix",
	},
	"KE52": {
		ICAO:      "KE52",
		Name:      "Oldham County Airport",
		City:      "Vega",
		State:     "Texas",
		Country:   "US",
		Elevation: 3995,
		Latitude:  35.2319984436,
		Longitude: -102.3990020752,
		Timezone:  "America/Chicago",
	},
	"KE57": {
		ICAO:      "KE57",
		Name:      "Denver City Airport",
		City:      "Denver City",
		State:     "Texas",
		Country:   "US",
		Elevation: 3575,
		Latitude:  32.9751014709,
		Longitude: -102.8420028687,
		Timezone:  "America/Chicago",
	},
	"KE60": {
		ICAO:      "KE60",
		Name:      "Eloy Municipal Airport",
		City:      "Eloy",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1513,
		Latitude:  32.8069992065,
		Longitude: -111.5869979858,
		Timezone:  "America/Phoenix",
	},
	"KE63": {
		ICAO:      "KE63",
		Name:      "Gila Bend Municipal Airport",
		City:      "Gila Bend",
		State:     "Arizona",
		Country:   "US",
		Elevation: 789,
		Latitude:  32.95809937,
		Longitude: -112.6780014,
		Timezone:  "America/Phoenix",
	},
	"KE67": {
		ICAO:      "KE67",
		Name:      "Kearny Airport",
		City:      "Kearny",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1833,
		Latitude:  33.0475997925,
		Longitude: -110.908996582,
		Timezone:  "America/Phoenix",
	},
	"KE77": {
		ICAO:      "KE77",
		Name:      "San Manuel Airport",
		City:      "San Manuel",
		State:     "Arizona",
		Country:   "US",
		Elevation: 3274,
		Latitude:  32.6365013123,
		Longitude: -110.6470031738,
		Timezone:  "America/Phoenix",
	},
	"KE78": {
		ICAO:      "KE78",
		Name:      "Sells Airport",
		City:      "Sells",
		State:     "Arizona",
		Country:   "US",
		Elevation: 2409,
		Latitude:  31.9328994751,
		Longitude: -111.8939971924,
		Timezone:  "America/Phoenix",
	},
	"KE79": {
		ICAO:      "KE79",
		Name:      "Sierra Sky Park Airport",
		City:      "Fresno",
		State:     "California",
		Country:   "US",
		Elevation: 321,
		Latitude:  36.84016667,
		Longitude: -119.86916667,
		Timezone:  "America/Los_Angeles",
	},
	"KE80": {
		ICAO:      "KE80",
		Name:      "Alexander Municipal Airport",
		City:      "Belen",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 5194,
		Latitude:  34.645198822,
		Longitude: -106.8339996338,
		Timezone:  "America/Denver",
	},
	"KE89": {
		ICAO:      "KE89",
		Name:      "Conchas Lake Airport",
		City:      "Conchas Dam",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4230,
		Latitude:  35.3643989563,
		Longitude: -104.1819992065,
		Timezone:  "America/Denver",
	},
	"KE91": {
		ICAO:      "KE91",
		Name:      "Chinle Municipal Airport",
		City:      "Chinle",
		State:     "Arizona",
		Country:   "US",
		Elevation: 5547,
		Latitude:  36.1109008789,
		Longitude: -109.5749969482,
		Timezone:  "America/Denver",
	},
	"KE95": {
		ICAO:      "KE95",
		Name:      "Benson Municipal Airport",
		City:      "Benson",
		State:     "Arizona",
		Country:   "US",
		Elevation: 3829,
		Latitude:  31.9997005463,
		Longitude: -110.3570022583,
		Timezone:  "America/Phoenix",
	},
	"KE98": {
		ICAO:      "KE98",
		Name:      "Mid Valley Airpark",
		City:      "Los Lunas",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4830,
		Latitude:  34.7597999573,
		Longitude: -106.7450027466,
		Timezone:  "America/Denver",
	},
	"KEAG": {
		ICAO:      "KEAG",
		Name:      "Eagle Grove Municipal Airport",
		City:      "Eagle Grove",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1133,
		Latitude:  42.7098007202,
		Longitude: -93.9160995483,
		Timezone:  "America/Chicago",
	},
	"KEAN": {
		ICAO:      "KEAN",
		IATA:      "EAN",
		Name:      "Phifer Airfield",
		City:      "Wheatland",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4776,
		Latitude:  42.05550003,
		Longitude: -104.9290009,
		Timezone:  "America/Denver",
	},
	"KEAR": {
		ICAO:      "KEAR",
		IATA:      "EAR",
		Name:      "Kearney Regional Airport",
		City:      "Kearney",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2131,
		Latitude:  40.72700119,
		Longitude: -99.00679779,
		Timezone:  "America/Chicago",
	},
	"KEAT": {
		ICAO:      "KEAT",
		IATA:      "EAT",
		Name:      "Pangborn Memorial Airport",
		City:      "Wenatchee",
		State:     "Washington",
		Country:   "US",
		Elevation: 1249,
		Latitude:  47.3988990784,
		Longitude: -120.2070007324,
		Timezone:  "America/Los_Angeles",
	},
	"KEAU": {
		ICAO:      "KEAU",
		IATA:      "EAU",
		Name:      "Chippewa Valley Regional Airport",
		City:      "Eau Claire",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 913,
		Latitude:  44.8657989502,
		Longitude: -91.4842987061,
		Timezone:  "America/Chicago",
	},
	"KEBG": {
		ICAO:      "KEBG",
		Name:      "South Texas International At Edinburg Airport",
		City:      "Edinburg",
		State:     "Texas",
		Country:   "US",
		Elevation: 75,
		Latitude:  26.44169998,
		Longitude: -98.12220001,
		Timezone:  "America/Chicago",
	},
	"KEBS": {
		ICAO:      "KEBS",
		IATA:      "EBS",
		Name:      "Webster City Municipal Airport",
		City:      "Webster City",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1122,
		Latitude:  42.43659973,
		Longitude: -93.86889648,
		Timezone:  "America/Chicago",
	},
	"KECG": {
		ICAO:      "KECG",
		IATA:      "ECG",
		Name:      "Elizabeth City Regional Airport & Coast Guard Air Station",
		City:      "Elizabeth City",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 12,
		Latitude:  36.26060104,
		Longitude: -76.17459869,
		Timezone:  "America/New_York",
	},
	"KECP": {
		ICAO:      "KECP",
		IATA:      "ECP",
		Name:      "Northwest Florida Beaches International Airport",
		City:      "Panama City Beach",
		State:     "Florida",
		Country:   "US",
		Elevation: 69,
		Latitude:  30.3417,
		Longitude: -85.7973,
		Timezone:  "America/Chicago",
	},
	"KECS": {
		ICAO:      "KECS",
		IATA:      "ECS",
		Name:      "Mondell Field",
		City:      "Newcastle",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4174,
		Latitude:  43.8853988647,
		Longitude: -104.3180007935,
		Timezone:  "America/Denver",
	},
	"KECU": {
		ICAO:      "KECU",
		Name:      "Edwards County Airport",
		City:      "Rocksprings",
		State:     "Texas",
		Country:   "US",
		Elevation: 2372,
		Latitude:  29.9468994141,
		Longitude: -100.1740036011,
		Timezone:  "America/Chicago",
	},
	"KEDC": {
		ICAO:      "KEDC",
		Name:      "Austin Executive Airport",
		City:      "Austin",
		State:     "Texas",
		Country:   "US",
		Elevation: 189,
		Latitude:  30.4000778,
		Longitude: -97.5760335,
		Timezone:  "America/Chicago",
	},
	"KEDE": {
		ICAO:      "KEDE",
		IATA:      "EDE",
		Name:      "Northeastern Regional Airport",
		City:      "Edenton",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 20,
		Latitude:  36.0276985168,
		Longitude: -76.5671005249,
		Timezone:  "America/New_York",
	},
	"KEDG": {
		ICAO:      "KEDG",
		Name:      "Weide Ahp (Aberdeen Proving Ground) Heliport",
		City:      "Edgewood Arsenal",
		State:     "Maryland",
		Country:   "US",
		Elevation: 21,
		Latitude:  39.39160156,
		Longitude: -76.29109955,
		Timezone:  "America/New_York",
	},
	"KEDJ": {
		ICAO:      "KEDJ",
		Name:      "Bellefontaine Regional Airport",
		City:      "Bellefontaine",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1122,
		Latitude:  40.37229919,
		Longitude: -83.81900024,
		Timezone:  "America/New_York",
	},
	"KEDN": {
		ICAO:      "KEDN",
		IATA:      "ETS",
		Name:      "Enterprise Municipal Airport",
		City:      "Enterprise",
		State:     "Alabama",
		Country:   "US",
		Elevation: 361,
		Latitude:  31.29969978,
		Longitude: -85.89990234,
		Timezone:  "America/Chicago",
	},
	"KEDU": {
		ICAO:      "KEDU",
		Name:      "University Airport",
		City:      "Davis",
		State:     "California",
		Country:   "US",
		Elevation: 69,
		Latitude:  38.53150177,
		Longitude: -121.7860031128,
		Timezone:  "America/Los_Angeles",
	},
	"KEDW": {
		ICAO:      "KEDW",
		IATA:      "EDW",
		Name:      "Edwards Air Force Base",
		City:      "Edwards",
		State:     "California",
		Country:   "US",
		Elevation: 2312,
		Latitude:  34.90539932,
		Longitude: -117.8840027,
		Timezone:  "America/Los_Angeles",
	},
	"KEED": {
		ICAO:      "KEED",
		IATA:      "EED",
		Name:      "Needles Airport",
		City:      "Needles",
		State:     "California",
		Country:   "US",
		Elevation: 983,
		Latitude:  34.7663002014,
		Longitude: -114.6230010986,
		Timezone:  "America/Los_Angeles",
	},
	"KEEN": {
		ICAO:      "KEEN",
		IATA:      "EEN",
		Name:      "Dillant Hopkins Airport",
		City:      "Keene",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 488,
		Latitude:  42.898399353,
		Longitude: -72.2707977295,
		Timezone:  "America/New_York",
	},
	"KEEO": {
		ICAO:      "KEEO",
		Name:      "Meeker Airport",
		City:      "Meeker",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6421,
		Latitude:  40.0488014221,
		Longitude: -107.8860015869,
		Timezone:  "America/Denver",
	},
	"KEET": {
		ICAO:      "KEET",
		Name:      "Shelby County Airport",
		City:      "Alabaster",
		State:     "Alabama",
		Country:   "US",
		Elevation: 586,
		Latitude:  33.17699814,
		Longitude: -86.78279877,
		Timezone:  "America/Chicago",
	},
	"KEFC": {
		ICAO:      "KEFC",
		Name:      "Belle Fourche Municipal Airport",
		City:      "Belle Fourche",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 3191,
		Latitude:  44.73419952,
		Longitude: -103.8619995,
		Timezone:  "America/Denver",
	},
	"KEFD": {
		ICAO:      "KEFD",
		IATA:      "EFD",
		Name:      "Ellington Field",
		City:      "Houston",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 32,
		Latitude:  29.6072998047,
		Longitude: -95.1587982178,
		Timezone:  "America/Chicago",
	},
	"KEFK": {
		ICAO:      "KEFK",
		IATA:      "EFK",
		Name:      "Newport State Airport",
		City:      "Newport",
		State:     "Vermont",
		Country:   "US",
		Elevation: 930,
		Latitude:  44.8888015747,
		Longitude: -72.2292022705,
		Timezone:  "America/New_York",
	},
	"KEFT": {
		ICAO:      "KEFT",
		Name:      "Monroe Municipal Airport",
		City:      "Monroe",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1086,
		Latitude:  42.6148986816,
		Longitude: -89.5904006958,
		Timezone:  "America/Chicago",
	},
	"KEFW": {
		ICAO:      "KEFW",
		IATA:      "EFW",
		Name:      "Jefferson Municipal Airport",
		City:      "Jefferson",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1044,
		Latitude:  42.0102005,
		Longitude: -94.34259796,
		Timezone:  "America/Chicago",
	},
	"KEGE": {
		ICAO:      "KEGE",
		IATA:      "EGE",
		Name:      "Eagle County Regional Airport",
		City:      "Eagle",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6548,
		Latitude:  39.64260101,
		Longitude: -106.9179993,
		Timezone:  "America/Denver",
	},
	"KEGI": {
		ICAO:      "KEGI",
		IATA:      "EGI",
		Name:      "Duke Field(Eglin Af Aux Nr 3) Airport",
		City:      "Crestview",
		State:     "Florida",
		Country:   "US",
		Elevation: 191,
		Latitude:  30.65040016,
		Longitude: -86.52290344,
		Timezone:  "America/Chicago",
	},
	"KEGQ": {
		ICAO:      "KEGQ",
		Name:      "Emmetsburg Municipal Airport",
		City:      "Emmetsburg",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1205,
		Latitude:  43.1020011902,
		Longitude: -94.7046966553,
		Timezone:  "America/Chicago",
	},
	"KEGT": {
		ICAO:      "KEGT",
		Name:      "Wellington Municipal Airport",
		City:      "Wellington",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1277,
		Latitude:  37.323600769,
		Longitude: -97.3882980347,
		Timezone:  "America/Chicago",
	},
	"KEGV": {
		ICAO:      "KEGV",
		IATA:      "EGV",
		Name:      "Eagle River Union Airport",
		City:      "Eagle River",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1642,
		Latitude:  45.9323005676,
		Longitude: -89.2683029175,
		Timezone:  "America/Chicago",
	},
	"KEHA": {
		ICAO:      "KEHA",
		Name:      "Elkhart Morton County Airport",
		City:      "Elkhart",
		State:     "Kansas",
		Country:   "US",
		Elevation: 3622,
		Latitude:  37.0007019043,
		Longitude: -101.8799972534,
		Timezone:  "America/Chicago",
	},
	"KEHO": {
		ICAO:      "KEHO",
		Name:      "Shelby-Cleveland County Regional Airport",
		City:      "Shelby",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 847,
		Latitude:  35.25559998,
		Longitude: -81.60099792,
		Timezone:  "America/New_York",
	},
	"KEHR": {
		ICAO:      "KEHR",
		Name:      "Henderson City County Airport",
		City:      "Henderson",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 387,
		Latitude:  37.80780029,
		Longitude: -87.68569946,
		Timezone:  "America/Chicago",
	},
	"KEIK": {
		ICAO:      "KEIK",
		Name:      "Erie Municipal Airport",
		City:      "Erie",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5130,
		Latitude:  40.0102005005,
		Longitude: -105.047996521,
		Timezone:  "America/Denver",
	},
	"KEIW": {
		ICAO:      "KEIW",
		Name:      "County Memorial Airport",
		City:      "New Madrid",
		State:     "Missouri",
		Country:   "US",
		Elevation: 296,
		Latitude:  36.53530121,
		Longitude: -89.59970093,
		Timezone:  "America/Chicago",
	},
	"KEKA": {
		ICAO:      "KEKA",
		IATA:      "EKA",
		Name:      "Murray Field",
		City:      "Eureka",
		State:     "California",
		Country:   "US",
		Elevation: 7,
		Latitude:  40.8033981323,
		Longitude: -124.1129989624,
		Timezone:  "America/Los_Angeles",
	},
	"KEKM": {
		ICAO:      "KEKM",
		IATA:      "EKI",
		Name:      "Elkhart Municipal Airport",
		City:      "Elkhart",
		State:     "Indiana",
		Country:   "US",
		Elevation: 778,
		Latitude:  41.7193984985,
		Longitude: -86.0031967163,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KEKN": {
		ICAO:      "KEKN",
		IATA:      "EKN",
		Name:      "Elkins-Randolph Co-Jennings Randolph Field",
		City:      "Elkins",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 1987,
		Latitude:  38.88940048,
		Longitude: -79.85710144,
		Timezone:  "America/New_York",
	},
	"KEKO": {
		ICAO:      "KEKO",
		IATA:      "EKO",
		Name:      "Elko Regional Airport",
		City:      "Elko",
		State:     "Nevada",
		Country:   "US",
		Elevation: 5140,
		Latitude:  40.8249015808,
		Longitude: -115.7919998169,
		Timezone:  "America/Los_Angeles",
	},
	"KEKQ": {
		ICAO:      "KEKQ",
		Name:      "Wayne County Airport",
		City:      "Monticello",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 963,
		Latitude:  36.8553009033,
		Longitude: -84.8561019897,
		Timezone:  "America/Kentucky/Monticello",
	},
	"KEKS": {
		ICAO:      "KEKS",
		Name:      "Ennis Big Sky Airport",
		City:      "Ennis",
		State:     "Montana",
		Country:   "US",
		Elevation: 5423,
		Latitude:  45.27180099,
		Longitude: -111.6490021,
		Timezone:  "America/Denver",
	},
	"KEKX": {
		ICAO:      "KEKX",
		IATA:      "EKX",
		Name:      "Addington Field",
		City:      "Elizabethtown",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 775,
		Latitude:  37.686000824,
		Longitude: -85.9250030518,
		Timezone:  "America/New_York",
	},
	"KEKY": {
		ICAO:      "KEKY",
		Name:      "Bessemer Airport",
		City:      "Bessemer",
		State:     "Alabama",
		Country:   "US",
		Elevation: 700,
		Latitude:  33.31290054,
		Longitude: -86.92590332,
		Timezone:  "America/Chicago",
	},
	"KELA": {
		ICAO:      "KELA",
		IATA:      "ELA",
		Name:      "Eagle Lake Airport",
		City:      "Eagle Lake",
		State:     "Texas",
		Country:   "US",
		Elevation: 184,
		Latitude:  29.6005992889,
		Longitude: -96.3218994141,
		Timezone:  "America/Chicago",
	},
	"KELD": {
		ICAO:      "KELD",
		IATA:      "ELD",
		Name:      "South Arkansas Regional At Goodwin Field",
		City:      "El Dorado",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 277,
		Latitude:  33.2210006714,
		Longitude: -92.8133010864,
		Timezone:  "America/Chicago",
	},
	"KELK": {
		ICAO:      "KELK",
		IATA:      "ELK",
		Name:      "Elk City Regional Business Airport",
		City:      "Elk City",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 2013,
		Latitude:  35.43080139,
		Longitude: -99.39430237,
		Timezone:  "America/Chicago",
	},
	"KELM": {
		ICAO:      "KELM",
		IATA:      "ELM",
		Name:      "Elmira Corning Regional Airport",
		City:      "Elmira/Corning",
		State:     "New-York",
		Country:   "US",
		Elevation: 954,
		Latitude:  42.1599006653,
		Longitude: -76.8916015625,
		Timezone:  "America/New_York",
	},
	"KELN": {
		ICAO:      "KELN",
		IATA:      "ELN",
		Name:      "Bowers Field",
		City:      "Ellensburg",
		State:     "Washington",
		Country:   "US",
		Elevation: 1764,
		Latitude:  47.03300095,
		Longitude: -120.5309982,
		Timezone:  "America/Los_Angeles",
	},
	"KELO": {
		ICAO:      "KELO",
		IATA:      "LYU",
		Name:      "Ely Municipal Airport",
		City:      "Ely",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1456,
		Latitude:  47.82450104,
		Longitude: -91.83070374,
		Timezone:  "America/Chicago",
	},
	"KELP": {
		ICAO:      "KELP",
		IATA:      "ELP",
		Name:      "El Paso International Airport",
		City:      "El Paso",
		State:     "Texas",
		Country:   "US",
		Elevation: 3959,
		Latitude:  31.80719948,
		Longitude: -106.3779984,
		Timezone:  "America/Denver",
	},
	"KELY": {
		ICAO:      "KELY",
		IATA:      "ELY",
		Name:      "Ely Airport Yelland Field",
		City:      "Ely",
		State:     "Nevada",
		Country:   "US",
		Elevation: 6259,
		Latitude:  39.29970169,
		Longitude: -114.8420029,
		Timezone:  "America/Los_Angeles",
	},
	"KELZ": {
		ICAO:      "KELZ",
		IATA:      "ELZ",
		Name:      "Wellsville Municipal ArptTarantine Field",
		City:      "Wellsville",
		State:     "New-York",
		Country:   "US",
		Elevation: 2124,
		Latitude:  42.10950089,
		Longitude: -77.98999786,
		Timezone:  "America/New_York",
	},
	"KEMM": {
		ICAO:      "KEMM",
		IATA:      "EMM",
		Name:      "Kemmerer Municipal Airport",
		City:      "Kemmerer",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 7285,
		Latitude:  41.8241004944,
		Longitude: -110.5569992065,
		Timezone:  "America/Denver",
	},
	"KEMP": {
		ICAO:      "KEMP",
		IATA:      "EMP",
		Name:      "Emporia Municipal Airport",
		City:      "Emporia",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1208,
		Latitude:  38.3320999146,
		Longitude: -96.1912002563,
		Timezone:  "America/Chicago",
	},
	"KEMT": {
		ICAO:      "KEMT",
		IATA:      "EMT",
		Name:      "El Monte Airport",
		City:      "El Monte",
		State:     "California",
		Country:   "US",
		Elevation: 296,
		Latitude:  34.086101532,
		Longitude: -118.0350036621,
		Timezone:  "America/Los_Angeles",
	},
	"KEMV": {
		ICAO:      "KEMV",
		Name:      "Emporia Greensville Regional Airport",
		City:      "Emporia",
		State:     "Virginia",
		Country:   "US",
		Elevation: 127,
		Latitude:  36.6869010925,
		Longitude: -77.4828033447,
		Timezone:  "America/New_York",
	},
	"KEND": {
		ICAO:      "KEND",
		IATA:      "END",
		Name:      "Vance Air Force Base",
		City:      "Enid",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1307,
		Latitude:  36.3391990662,
		Longitude: -97.9164962769,
		Timezone:  "America/Chicago",
	},
	"KENL": {
		ICAO:      "KENL",
		IATA:      "ENL",
		Name:      "Centralia Municipal Airport",
		City:      "Centralia",
		State:     "Illinois",
		Country:   "US",
		Elevation: 534,
		Latitude:  38.5150985718,
		Longitude: -89.0911026001,
		Timezone:  "America/Chicago",
	},
	"KENV": {
		ICAO:      "KENV",
		IATA:      "ENV",
		Name:      "Wendover Airport",
		City:      "Wendover",
		State:     "Utah",
		Country:   "US",
		Elevation: 4237,
		Latitude:  40.7187004089,
		Longitude: -114.03099823,
		Timezone:  "America/Denver",
	},
	"KENW": {
		ICAO:      "KENW",
		IATA:      "ENW",
		Name:      "Kenosha Regional Airport",
		City:      "Kenosha",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 742,
		Latitude:  42.59569931,
		Longitude: -87.92780304,
		Timezone:  "America/Chicago",
	},
	"KEOE": {
		ICAO:      "KEOE",
		Name:      "Newberry County Airport",
		City:      "Newberry",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 570,
		Latitude:  34.30929947,
		Longitude: -81.63970184,
		Timezone:  "America/New_York",
	},
	"KEOK": {
		ICAO:      "KEOK",
		IATA:      "EOK",
		Name:      "Keokuk Municipal Airport",
		City:      "Keokuk",
		State:     "Iowa",
		Country:   "US",
		Elevation: 671,
		Latitude:  40.4598999023,
		Longitude: -91.4284973145,
		Timezone:  "America/Chicago",
	},
	"KEOP": {
		ICAO:      "KEOP",
		Name:      "Pike County Airport",
		City:      "Waverly",
		State:     "Ohio",
		Country:   "US",
		Elevation: 660,
		Latitude:  39.1669006348,
		Longitude: -82.9281997681,
		Timezone:  "America/New_York",
	},
	"KEOS": {
		ICAO:      "KEOS",
		IATA:      "EOS",
		Name:      "Neosho Hugh Robinson Airport",
		City:      "Neosho",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1255,
		Latitude:  36.810798645,
		Longitude: -94.3917007446,
		Timezone:  "America/Chicago",
	},
	"KEPH": {
		ICAO:      "KEPH",
		IATA:      "EPH",
		Name:      "Ephrata Municipal Airport",
		City:      "Ephrata",
		State:     "Washington",
		Country:   "US",
		Elevation: 1276,
		Latitude:  47.30759811,
		Longitude: -119.5159988,
		Timezone:  "America/Los_Angeles",
	},
	"KEPM": {
		ICAO:      "KEPM",
		Name:      "Eastport Municipal Airport",
		City:      "Eastport",
		State:     "Maine",
		Country:   "US",
		Elevation: 45,
		Latitude:  44.9100990295,
		Longitude: -67.0127029419,
		Timezone:  "America/Moncton",
	},
	"KEQA": {
		ICAO:      "KEQA",
		IATA:      "EDK",
		Name:      "Captain Jack Thomas El Dorado Airport",
		City:      "El Dorado",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1378,
		Latitude:  37.7741012573,
		Longitude: -96.8175964355,
		Timezone:  "America/Chicago",
	},
	"KEQY": {
		ICAO:      "KEQY",
		Name:      "Charlotte-Monroe Executive Airport",
		City:      "Monroe",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 679,
		Latitude:  35.01879883,
		Longitude: -80.62020111,
		Timezone:  "America/New_York",
	},
	"KERI": {
		ICAO:      "KERI",
		IATA:      "ERI",
		Name:      "Erie International Tom Ridge Field",
		City:      "Erie",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 732,
		Latitude:  42.0831270134,
		Longitude: -80.1738667488,
		Timezone:  "America/New_York",
	},
	"KERR": {
		ICAO:      "KERR",
		IATA:      "ERR",
		Name:      "Errol Airport",
		City:      "Errol",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 1245,
		Latitude:  44.7924995422,
		Longitude: -71.1641998291,
		Timezone:  "America/New_York",
	},
	"KERV": {
		ICAO:      "KERV",
		IATA:      "ERV",
		Name:      "Kerrville Municipal Louis Schreiner Field",
		City:      "Kerrville",
		State:     "Texas",
		Country:   "US",
		Elevation: 1617,
		Latitude:  29.9766998291,
		Longitude: -99.0857009888,
		Timezone:  "America/Chicago",
	},
	"KERY": {
		ICAO:      "KERY",
		Name:      "Luce County Airport",
		City:      "Newberry",
		State:     "Michigan",
		Country:   "US",
		Elevation: 869,
		Latitude:  46.3111991882,
		Longitude: -85.4572982788,
		Timezone:  "America/Detroit",
	},
	"KESC": {
		ICAO:      "KESC",
		IATA:      "ESC",
		Name:      "Delta County Airport",
		City:      "Escanaba",
		State:     "Michigan",
		Country:   "US",
		Elevation: 609,
		Latitude:  45.7226982117,
		Longitude: -87.0936965942,
		Timezone:  "America/Detroit",
	},
	"KESF": {
		ICAO:      "KESF",
		IATA:      "ESF",
		Name:      "Esler Regional Airport",
		City:      "Alexandria",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 112,
		Latitude:  31.3948993683,
		Longitude: -92.2957992554,
		Timezone:  "America/Chicago",
	},
	"KESN": {
		ICAO:      "KESN",
		IATA:      "ESN",
		Name:      "Easton Newnam Field",
		City:      "Easton",
		State:     "Maryland",
		Country:   "US",
		Elevation: 72,
		Latitude:  38.8041992188,
		Longitude: -76.0690002441,
		Timezone:  "America/New_York",
	},
	"KEST": {
		ICAO:      "KEST",
		IATA:      "EST",
		Name:      "Estherville Municipal Airport",
		City:      "Estherville",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1319,
		Latitude:  43.40739822,
		Longitude: -94.74639893,
		Timezone:  "America/Chicago",
	},
	"KESW": {
		ICAO:      "KESW",
		IATA:      "ESW",
		Name:      "Easton State Airport",
		City:      "Easton",
		State:     "Washington",
		Country:   "US",
		Elevation: 2226,
		Latitude:  47.2541999817,
		Longitude: -121.1859970093,
		Timezone:  "America/Los_Angeles",
	},
	"KETB": {
		ICAO:      "KETB",
		IATA:      "ETB",
		Name:      "West Bend Municipal Airport",
		City:      "West Bend",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 887,
		Latitude:  43.4221992493,
		Longitude: -88.1278991699,
		Timezone:  "America/Chicago",
	},
	"KETC": {
		ICAO:      "KETC",
		Name:      "Tarboro Edgecombe Airport",
		City:      "Tarboro",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 53,
		Latitude:  35.9370994568,
		Longitude: -77.5466003418,
		Timezone:  "America/New_York",
	},
	"KETH": {
		ICAO:      "KETH",
		Name:      "Wheaton Municipal Airport",
		City:      "Wheaton",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1025,
		Latitude:  45.7804985046,
		Longitude: -96.5435028076,
		Timezone:  "America/Chicago",
	},
	"KETN": {
		ICAO:      "KETN",
		IATA:      "ETN",
		Name:      "Eastland Municipal Airport",
		City:      "Eastland",
		State:     "Texas",
		Country:   "US",
		Elevation: 1464,
		Latitude:  32.4135017395,
		Longitude: -98.8097991943,
		Timezone:  "America/Chicago",
	},
	"KEUF": {
		ICAO:      "KEUF",
		IATA:      "EUF",
		Name:      "Weedon Field",
		City:      "Eufaula",
		State:     "Alabama",
		Country:   "US",
		Elevation: 285,
		Latitude:  31.9512996674,
		Longitude: -85.1288986206,
		Timezone:  "America/Chicago",
	},
	"KEUG": {
		ICAO:      "KEUG",
		IATA:      "EUG",
		Name:      "Mahlon Sweet Field",
		City:      "Eugene",
		State:     "Oregon",
		Country:   "US",
		Elevation: 374,
		Latitude:  44.1245994568,
		Longitude: -123.2119979858,
		Timezone:  "America/Los_Angeles",
	},
	"KEUL": {
		ICAO:      "KEUL",
		Name:      "Caldwell Industrial Airport",
		City:      "Caldwell",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2432,
		Latitude:  43.64189911,
		Longitude: -116.6360016,
		Timezone:  "America/Boise",
	},
	"KEVB": {
		ICAO:      "KEVB",
		Name:      "New Smyrna Beach Municipal Airport",
		City:      "New Smyrna Beach",
		State:     "Florida",
		Country:   "US",
		Elevation: 10,
		Latitude:  29.0557003021,
		Longitude: -80.9488983154,
		Timezone:  "America/New_York",
	},
	"KEVM": {
		ICAO:      "KEVM",
		IATA:      "EVM",
		Name:      "Eveleth Virginia Municipal Airport",
		City:      "Eveleth",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1379,
		Latitude:  47.42509842,
		Longitude: -92.49849701,
		Timezone:  "America/Chicago",
	},
	"KEVU": {
		ICAO:      "KEVU",
		Name:      "Northwest Missouri Regional Airport",
		City:      "Maryville",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1145,
		Latitude:  40.35250092,
		Longitude: -94.91500092,
		Timezone:  "America/Chicago",
	},
	"KEVV": {
		ICAO:      "KEVV",
		IATA:      "EVV",
		Name:      "Evansville Regional Airport",
		City:      "Evansville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 418,
		Latitude:  38.0369987488,
		Longitude: -87.5324020386,
		Timezone:  "America/Chicago",
	},
	"KEVW": {
		ICAO:      "KEVW",
		IATA:      "EVW",
		Name:      "Evanston-Uinta County Airport-Burns Field",
		City:      "Evanston",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 7143,
		Latitude:  41.27479935,
		Longitude: -111.0350037,
		Timezone:  "America/Denver",
	},
	"KEVY": {
		ICAO:      "KEVY",
		Name:      "Summit Airport",
		City:      "Middletown",
		State:     "Delaware",
		Country:   "US",
		Elevation: 70,
		Latitude:  39.520401001,
		Longitude: -75.7203979492,
		Timezone:  "America/New_York",
	},
	"KEWB": {
		ICAO:      "KEWB",
		IATA:      "EWB",
		Name:      "New Bedford Regional Airport",
		City:      "New Bedford",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 80,
		Latitude:  41.6761016846,
		Longitude: -70.9569015503,
		Timezone:  "America/New_York",
	},
	"KEWK": {
		ICAO:      "KEWK",
		IATA:      "EWK",
		Name:      "Newton City-County Airport",
		City:      "Newton",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1533,
		Latitude:  38.0582008362,
		Longitude: -97.2744979858,
		Timezone:  "America/Chicago",
	},
	"KEWN": {
		ICAO:      "KEWN",
		IATA:      "EWN",
		Name:      "Coastal Carolina Regional Airport",
		City:      "New Bern",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 18,
		Latitude:  35.0730018616,
		Longitude: -77.0429000854,
		Timezone:  "America/New_York",
	},
	"KEWR": {
		ICAO:      "KEWR",
		IATA:      "EWR",
		Name:      "Newark Liberty International Airport",
		City:      "Newark",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 18,
		Latitude:  40.6925010681,
		Longitude: -74.1687011719,
		Timezone:  "America/New_York",
	},
	"KEXX": {
		ICAO:      "KEXX",
		Name:      "Davidson County Airport",
		City:      "Lexington",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 733,
		Latitude:  35.7811012268,
		Longitude: -80.3038024902,
		Timezone:  "America/New_York",
	},
	"KEYE": {
		ICAO:      "KEYE",
		Name:      "Eagle Creek Airpark",
		City:      "Indianapolis",
		State:     "Indiana",
		Country:   "US",
		Elevation: 823,
		Latitude:  39.8306999207,
		Longitude: -86.2944030762,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KEYF": {
		ICAO:      "KEYF",
		Name:      "Curtis L Brown Jr Field",
		City:      "Elizabethtown",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 131,
		Latitude:  34.60179901,
		Longitude: -78.57929993,
		Timezone:  "America/New_York",
	},
	"KEYQ": {
		ICAO:      "KEYQ",
		Name:      "Weiser Air Park",
		City:      "Houston",
		State:     "Texas",
		Country:   "US",
		Elevation: 137,
		Latitude:  29.9351997375,
		Longitude: -95.6396026611,
		Timezone:  "America/Chicago",
	},
	"KEYW": {
		ICAO:      "KEYW",
		IATA:      "EYW",
		Name:      "Key West International Airport",
		City:      "Key West",
		State:     "Florida",
		Country:   "US",
		Elevation: 3,
		Latitude:  24.5561008453,
		Longitude: -81.7595977783,
		Timezone:  "America/New_York",
	},
	"KEZF": {
		ICAO:      "KEZF",
		Name:      "Shannon Airport",
		City:      "Fredericksburg",
		State:     "Virginia",
		Country:   "US",
		Elevation: 85,
		Latitude:  38.2667999268,
		Longitude: -77.4492034912,
		Timezone:  "America/New_York",
	},
	"KEZI": {
		ICAO:      "KEZI",
		Name:      "Kewanee Municipal Airport",
		City:      "Kewanee",
		State:     "Illinois",
		Country:   "US",
		Elevation: 858,
		Latitude:  41.2052001953,
		Longitude: -89.9638977051,
		Timezone:  "America/Chicago",
	},
	"KEZM": {
		ICAO:      "KEZM",
		Name:      "Heart of Georgia Regional Airport",
		City:      "Eastman",
		State:     "Georgia",
		Country:   "US",
		Elevation: 304,
		Latitude:  32.2141990662,
		Longitude: -83.1279983521,
		Timezone:  "America/New_York",
	},
	"KEZS": {
		ICAO:      "KEZS",
		Name:      "Shawano Municipal Airport",
		City:      "Shawano",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 813,
		Latitude:  44.7869987488,
		Longitude: -88.5589981079,
		Timezone:  "America/Chicago",
	},
	"KEZZ": {
		ICAO:      "KEZZ",
		Name:      "Cameron Memorial Airport",
		City:      "Cameron",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1040,
		Latitude:  39.7276001,
		Longitude: -94.27639771,
		Timezone:  "America/Chicago",
	},
	"KF00": {
		ICAO:      "KF00",
		Name:      "Jones Field",
		City:      "Bonham",
		State:     "Texas",
		Country:   "US",
		Elevation: 618,
		Latitude:  33.6128005981,
		Longitude: -96.1792984009,
		Timezone:  "America/Chicago",
	},
	"KF01": {
		ICAO:      "KF01",
		Name:      "Quanah Municipal Airport",
		City:      "Quanah",
		State:     "Texas",
		Country:   "US",
		Elevation: 1602,
		Latitude:  34.2770996094,
		Longitude: -99.7593002319,
		Timezone:  "America/Chicago",
	},
	"KF02": {
		ICAO:      "KF02",
		Name:      "Fairgrounds Airpark",
		City:      "Hardin",
		State:     "Montana",
		Country:   "US",
		Elevation: 2911,
		Latitude:  45.7249984741,
		Longitude: -107.6009979248,
		Timezone:  "America/Denver",
	},
	"KF05": {
		ICAO:      "KF05",
		Name:      "Wilbarger County Airport",
		City:      "Vernon",
		State:     "Texas",
		Country:   "US",
		Elevation: 1265,
		Latitude:  34.2257003784,
		Longitude: -99.2837982178,
		Timezone:  "America/Chicago",
	},
	"KF06": {
		ICAO:      "KF06",
		Name:      "Marian Airpark",
		City:      "Wellington",
		State:     "Texas",
		Country:   "US",
		Elevation: 2008,
		Latitude:  34.8456001282,
		Longitude: -100.1959991455,
		Timezone:  "America/Chicago",
	},
	"KF08": {
		ICAO:      "KF08",
		Name:      "Eufaula Municipal Airport",
		City:      "Eufaula",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 635,
		Latitude:  35.2958984375,
		Longitude: -95.6252975464,
		Timezone:  "America/Chicago",
	},
	"KF10": {
		ICAO:      "KF10",
		Name:      "Henryetta Municipal Airport",
		City:      "Henryetta",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 849,
		Latitude:  35.4068984985,
		Longitude: -96.0158004761,
		Timezone:  "America/Chicago",
	},
	"KF14": {
		ICAO:      "KF14",
		Name:      "Wichita Valley Airport",
		City:      "Wichita Falls",
		State:     "Texas",
		Country:   "US",
		Elevation: 1005,
		Latitude:  33.9477005005,
		Longitude: -98.6166992188,
		Timezone:  "America/Chicago",
	},
	"KF17": {
		ICAO:      "KF17",
		Name:      "Center Municipal Airport",
		City:      "Center",
		State:     "Texas",
		Country:   "US",
		Elevation: 319,
		Latitude:  31.8316001892,
		Longitude: -94.1564025879,
		Timezone:  "America/Chicago",
	},
	"KF21": {
		ICAO:      "KF21",
		Name:      "Memphis Municipal Airport",
		City:      "Memphis",
		State:     "Texas",
		Country:   "US",
		Elevation: 2102,
		Latitude:  34.7396011353,
		Longitude: -100.5299987793,
		Timezone:  "America/Chicago",
	},
	"KF22": {
		ICAO:      "KF22",
		Name:      "Perry Municipal Airport",
		City:      "Perry",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1002,
		Latitude:  36.3856010437,
		Longitude: -97.2771987915,
		Timezone:  "America/Chicago",
	},
	"KF24": {
		ICAO:      "KF24",
		Name:      "Minden Webster Airport",
		City:      "Minden",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 278,
		Latitude:  32.6459999084,
		Longitude: -93.2981033325,
		Timezone:  "America/Chicago",
	},
	"KF30": {
		ICAO:      "KF30",
		Name:      "Sulphur Municipal Airport",
		City:      "Sulphur",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1051,
		Latitude:  34.5245018005,
		Longitude: -96.9897003174,
		Timezone:  "America/Chicago",
	},
	"KF31": {
		ICAO:      "KF31",
		Name:      "Lake Texoma State Park Airport",
		City:      "Kingston",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 693,
		Latitude:  33.9910011292,
		Longitude: -96.6427993774,
		Timezone:  "America/Chicago",
	},
	"KF32": {
		ICAO:      "KF32",
		Name:      "Healdton Municipal Airport",
		City:      "Healdton",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 956,
		Latitude:  34.2492980957,
		Longitude: -97.4738998413,
		Timezone:  "America/Chicago",
	},
	"KF34": {
		ICAO:      "KF34",
		Name:      "Firebaugh Airport",
		City:      "Firebaugh",
		State:     "California",
		Country:   "US",
		Elevation: 157,
		Latitude:  36.8600006104,
		Longitude: -120.4639968872,
		Timezone:  "America/Los_Angeles",
	},
	"KF35": {
		ICAO:      "KF35",
		Name:      "Possum Kingdom Airport",
		City:      "Graford",
		State:     "Texas",
		Country:   "US",
		Elevation: 1008,
		Latitude:  32.9231987,
		Longitude: -98.4364013672,
		Timezone:  "America/Chicago",
	},
	"KF36": {
		ICAO:      "KF36",
		Name:      "Cordell Municipal Airport",
		City:      "Cordell",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1589,
		Latitude:  35.2975997925,
		Longitude: -98.9673995972,
		Timezone:  "America/Chicago",
	},
	"KF37": {
		ICAO:      "KF37",
		Name:      "Carrizozo Municipal Airport",
		City:      "Carrizozo",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 5371,
		Latitude:  33.6488990784,
		Longitude: -105.8960037231,
		Timezone:  "America/Denver",
	},
	"KF41": {
		ICAO:      "KF41",
		Name:      "Ennis Municipal Airport",
		City:      "Ennis",
		State:     "Texas",
		Country:   "US",
		Elevation: 500,
		Latitude:  32.32970047,
		Longitude: -96.6639022827,
		Timezone:  "America/Chicago",
	},
	"KF43": {
		ICAO:      "KF43",
		Name:      "El Dorado Downtown-Stevens field",
		City:      "El Dorado",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 256,
		Latitude:  33.19120026,
		Longitude: -92.66320038,
		Timezone:  "America/Chicago",
	},
	"KF44": {
		ICAO:      "KF44",
		Name:      "Athens Municipal Airport",
		City:      "Athens",
		State:     "Texas",
		Country:   "US",
		Elevation: 444,
		Latitude:  32.1638478,
		Longitude: -95.8283531,
		Timezone:  "America/Chicago",
	},
	"KF45": {
		ICAO:      "KF45",
		Name:      "North Palm Beach County General Aviation Airport",
		City:      "West Palm Beach",
		State:     "Florida",
		Country:   "US",
		Elevation: 22,
		Latitude:  26.84440041,
		Longitude: -80.22129822,
		Timezone:  "America/New_York",
	},
	"KF47": {
		ICAO:      "KF47",
		Name:      "St George Island Airport",
		City:      "Apalachicola",
		State:     "Florida",
		Country:   "US",
		Elevation: 3,
		Latitude:  29.6459999084,
		Longitude: -84.9166030884,
		Timezone:  "America/New_York",
	},
	"KF48": {
		ICAO:      "KF48",
		Name:      "Nocona Airport",
		City:      "Nocona",
		State:     "Texas",
		Country:   "US",
		Elevation: 905,
		Latitude:  33.7739982605,
		Longitude: -97.7380981445,
		Timezone:  "America/Chicago",
	},
	"KF49": {
		ICAO:      "KF49",
		Name:      "Slaton Municipal Airport",
		City:      "Slaton",
		State:     "Texas",
		Country:   "US",
		Elevation: 3123,
		Latitude:  33.4847984314,
		Longitude: -101.6610031128,
		Timezone:  "America/Chicago",
	},
	"KF51": {
		ICAO:      "KF51",
		Name:      "Winnsboro Municipal Airport",
		City:      "Winnsboro",
		State:     "Texas",
		Country:   "US",
		Elevation: 513,
		Latitude:  32.9388008118,
		Longitude: -95.2789001465,
		Timezone:  "America/Chicago",
	},
	"KF53": {
		ICAO:      "KF53",
		Name:      "Franklin County Airport",
		City:      "Mount Vernon",
		State:     "Texas",
		Country:   "US",
		Elevation: 412,
		Latitude:  33.2154007,
		Longitude: -95.23739624,
		Timezone:  "America/Chicago",
	},
	"KF56": {
		ICAO:      "KF56",
		Name:      "Arledge Field",
		City:      "Stamford",
		State:     "Texas",
		Country:   "US",
		Elevation: 1561,
		Latitude:  32.9090995789,
		Longitude: -99.736000061,
		Timezone:  "America/Chicago",
	},
	"KF62": {
		ICAO:      "KF62",
		Name:      "Hayfork Airport",
		City:      "Hayfork",
		State:     "California",
		Country:   "US",
		Elevation: 2321,
		Latitude:  40.5471000671,
		Longitude: -123.1819992065,
		Timezone:  "America/Los_Angeles",
	},
	"KF68": {
		ICAO:      "KF68",
		Name:      "Anadarko Municipal Airport",
		City:      "Anadarko",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1286,
		Latitude:  35.0522003174,
		Longitude: -98.2642974854,
		Timezone:  "America/Chicago",
	},
	"KF69": {
		ICAO:      "KF69",
		Name:      "Air Park Dallas Airport",
		City:      "Dallas",
		State:     "Texas",
		Country:   "US",
		Elevation: 695,
		Latitude:  33.0234985352,
		Longitude: -96.8368988037,
		Timezone:  "America/Chicago",
	},
	"KF70": {
		ICAO:      "KF70",
		Name:      "French Valley Airport",
		City:      "Murrieta/Temecula",
		State:     "California",
		Country:   "US",
		Elevation: 1350,
		Latitude:  33.5741996765,
		Longitude: -117.1279983521,
		Timezone:  "America/Los_Angeles",
	},
	"KF72": {
		ICAO:      "KF72",
		Name:      "Franklin Field",
		City:      "Franklin",
		State:     "California",
		Country:   "US",
		Elevation: 21,
		Latitude:  38.304901123,
		Longitude: -121.4300003052,
		Timezone:  "America/Los_Angeles",
	},
	"KF75": {
		ICAO:      "KF75",
		Name:      "Harrison Field of Knox City Airport",
		City:      "Knox City",
		State:     "Texas",
		Country:   "US",
		Elevation: 1500,
		Latitude:  33.4380989075,
		Longitude: -99.8128967285,
		Timezone:  "America/Chicago",
	},
	"KF82": {
		ICAO:      "KF82",
		Name:      "Town & Country Airpark",
		City:      "Lubbock",
		State:     "Texas",
		Country:   "US",
		Elevation: 3200,
		Latitude:  33.48559952,
		Longitude: -101.8130035,
		Timezone:  "America/Chicago",
	},
	"KF83": {
		ICAO:      "KF83",
		Name:      "Abernathy Municipal Airport",
		City:      "Abernathy",
		State:     "Texas",
		Country:   "US",
		Elevation: 3327,
		Latitude:  33.8459014893,
		Longitude: -101.7630004883,
		Timezone:  "America/Chicago",
	},
	"KF84": {
		ICAO:      "KF84",
		Name:      "Stigler Regional Airport",
		City:      "Stigler",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 599,
		Latitude:  35.28910065,
		Longitude: -95.09390259,
		Timezone:  "America/Chicago",
	},
	"KF85": {
		ICAO:      "KF85",
		Name:      "Cochran County Airport",
		City:      "Morton",
		State:     "Texas",
		Country:   "US",
		Elevation: 3746,
		Latitude:  33.7293014526,
		Longitude: -102.7340011597,
		Timezone:  "America/Chicago",
	},
	"KF86": {
		ICAO:      "KF86",
		Name:      "Columbia Airport",
		City:      "Columbia",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 67,
		Latitude:  32.12239838,
		Longitude: -92.05290222,
		Timezone:  "America/Chicago",
	},
	"KF87": {
		ICAO:      "KF87",
		Name:      "Union Parish Airport",
		City:      "Farmerville",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 121,
		Latitude:  32.7249984741,
		Longitude: -92.3371963501,
		Timezone:  "America/Chicago",
	},
	"KF88": {
		ICAO:      "KF88",
		Name:      "Jonesboro Airport",
		City:      "Jonesboro",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 256,
		Latitude:  32.2019996643,
		Longitude: -92.7329025269,
		Timezone:  "America/Chicago",
	},
	"KF89": {
		ICAO:      "KF89",
		Name:      "Winnsboro Municipal Airport",
		City:      "Winnsboro",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 76,
		Latitude:  32.1529998779,
		Longitude: -91.6985015869,
		Timezone:  "America/Chicago",
	},
	"KF97": {
		ICAO:      "KF97",
		Name:      "Seagraves Airport",
		City:      "Seagraves",
		State:     "Texas",
		Country:   "US",
		Elevation: 3366,
		Latitude:  32.9546012878,
		Longitude: -102.5410003662,
		Timezone:  "America/Chicago",
	},
	"KF98": {
		ICAO:      "KF98",
		Name:      "Yoakum County Airport",
		City:      "Plains",
		State:     "Texas",
		Country:   "US",
		Elevation: 3704,
		Latitude:  33.21730042,
		Longitude: -102.8300018,
		Timezone:  "America/Chicago",
	},
	"KF99": {
		ICAO:      "KF99",
		Name:      "Holdenville Municipal Airport",
		City:      "Holdenville",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 861,
		Latitude:  35.08589935,
		Longitude: -96.41670227,
		Timezone:  "America/Chicago",
	},
	"KFAF": {
		ICAO:      "KFAF",
		IATA:      "FAF",
		Name:      "Felker Army Air Field",
		City:      "Fort Eustis",
		State:     "Virginia",
		Country:   "US",
		Elevation: 12,
		Latitude:  37.1324996948,
		Longitude: -76.6088027954,
		Timezone:  "America/New_York",
	},
	"KFAM": {
		ICAO:      "KFAM",
		IATA:      "FAM",
		Name:      "Farmington Regional Airport",
		City:      "Farmington",
		State:     "Missouri",
		Country:   "US",
		Elevation: 946,
		Latitude:  37.76110077,
		Longitude: -90.4285965,
		Timezone:  "America/Chicago",
	},
	"KFAR": {
		ICAO:      "KFAR",
		IATA:      "FAR",
		Name:      "Hector International Airport",
		City:      "Fargo",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 902,
		Latitude:  46.9207000732,
		Longitude: -96.8158035278,
		Timezone:  "America/Chicago",
	},
	"KFAT": {
		ICAO:      "KFAT",
		IATA:      "FAT",
		Name:      "Fresno Yosemite International Airport",
		City:      "Fresno",
		State:     "California",
		Country:   "US",
		Elevation: 336,
		Latitude:  36.7761993408,
		Longitude: -119.7180023193,
		Timezone:  "America/Los_Angeles",
	},
	"KFAY": {
		ICAO:      "KFAY",
		IATA:      "FAY",
		Name:      "Fayetteville Regional Grannis Field",
		City:      "Fayetteville",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 189,
		Latitude:  34.9911994934,
		Longitude: -78.8803024292,
		Timezone:  "America/New_York",
	},
	"KFBG": {
		ICAO:      "KFBG",
		IATA:      "FBG",
		Name:      "Simmons Army Air Field",
		City:      "Fort Bragg",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 244,
		Latitude:  35.13180161,
		Longitude: -78.93669891,
		Timezone:  "America/New_York",
	},
	"KFBL": {
		ICAO:      "KFBL",
		IATA:      "FBL",
		Name:      "Faribault Municipal Airport",
		City:      "Faribault",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1060,
		Latitude:  44.3246994019,
		Longitude: -93.310798645,
		Timezone:  "America/Chicago",
	},
	"KFBR": {
		ICAO:      "KFBR",
		IATA:      "FBR",
		Name:      "Fort Bridger Airport",
		City:      "Fort Bridger",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 7034,
		Latitude:  41.3918991089,
		Longitude: -110.4069976807,
		Timezone:  "America/Denver",
	},
	"KFBY": {
		ICAO:      "KFBY",
		IATA:      "FBY",
		Name:      "Fairbury Municipal Airport",
		City:      "Fairbury",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1479,
		Latitude:  40.1829986572,
		Longitude: -97.1692962646,
		Timezone:  "America/Chicago",
	},
	"KFCH": {
		ICAO:      "KFCH",
		IATA:      "FCH",
		Name:      "Fresno Chandler Executive Airport",
		City:      "Fresno",
		State:     "California",
		Country:   "US",
		Elevation: 279,
		Latitude:  36.73239899,
		Longitude: -119.8199997,
		Timezone:  "America/Los_Angeles",
	},
	"KFCI": {
		ICAO:      "KFCI",
		Name:      "Chesterfield County Airport",
		City:      "Richmond",
		State:     "Virginia",
		Country:   "US",
		Elevation: 236,
		Latitude:  37.40650177,
		Longitude: -77.52500153,
		Timezone:  "America/New_York",
	},
	"KFCM": {
		ICAO:      "KFCM",
		IATA:      "FCM",
		Name:      "Flying Cloud Airport",
		City:      "Minneapolis",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 906,
		Latitude:  44.8272018433,
		Longitude: -93.4570999146,
		Timezone:  "America/Chicago",
	},
	"KFCS": {
		ICAO:      "KFCS",
		IATA:      "FCS",
		Name:      "Butts AAF (Fort Carson) Air Field",
		City:      "Fort Carson",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5838,
		Latitude:  38.67839813,
		Longitude: -104.7570038,
		Timezone:  "America/Denver",
	},
	"KFCY": {
		ICAO:      "KFCY",
		IATA:      "FCY",
		Name:      "Forrest City Municipal Airport",
		City:      "Forrest City",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 249,
		Latitude:  34.9420013428,
		Longitude: -90.7750015259,
		Timezone:  "America/Chicago",
	},
	"KFDK": {
		ICAO:      "KFDK",
		IATA:      "FDK",
		Name:      "Frederick Municipal Airport",
		City:      "Frederick",
		State:     "Maryland",
		Country:   "US",
		Elevation: 303,
		Latitude:  39.4175987244,
		Longitude: -77.3742980957,
		Timezone:  "America/New_York",
	},
	"KFDR": {
		ICAO:      "KFDR",
		IATA:      "FDR",
		Name:      "Frederick Regional Airport",
		City:      "Frederick",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1258,
		Latitude:  34.35200119,
		Longitude: -98.98390198,
		Timezone:  "America/Chicago",
	},
	"KFDW": {
		ICAO:      "KFDW",
		Name:      "Fairfield County Airport",
		City:      "Winnsboro",
		State:     "Ohio",
		Country:   "US",
		Elevation: 577,
		Latitude:  34.3154983521,
		Longitude: -81.1088027954,
		Timezone:  "America/New_York",
	},
	"KFDY": {
		ICAO:      "KFDY",
		IATA:      "FDY",
		Name:      "Findlay Airport",
		City:      "Findlay",
		State:     "Ohio",
		Country:   "US",
		Elevation: 813,
		Latitude:  41.0135002136,
		Longitude: -83.6687011719,
		Timezone:  "America/New_York",
	},
	"KFEP": {
		ICAO:      "KFEP",
		IATA:      "FEP",
		Name:      "Albertus Airport",
		City:      "Freeport",
		State:     "Illinois",
		Country:   "US",
		Elevation: 859,
		Latitude:  42.2462005615,
		Longitude: -89.5820007324,
		Timezone:  "America/Chicago",
	},
	"KFES": {
		ICAO:      "KFES",
		Name:      "Festus Memorial Airport",
		City:      "Festus",
		State:     "Missouri",
		Country:   "US",
		Elevation: 433,
		Latitude:  38.1949005127,
		Longitude: -90.3853988647,
		Timezone:  "America/Chicago",
	},
	"KFET": {
		ICAO:      "KFET",
		IATA:      "FET",
		Name:      "Fremont Municipal Airport",
		City:      "Fremont",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1204,
		Latitude:  41.44910049,
		Longitude: -96.52020264,
		Timezone:  "America/Chicago",
	},
	"KFFA": {
		ICAO:      "KFFA",
		IATA:      "FFA",
		Name:      "First Flight Airport",
		City:      "Kill Devil Hills",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 13,
		Latitude:  36.0181999207,
		Longitude: -75.6713027954,
		Timezone:  "America/New_York",
	},
	"KFFC": {
		ICAO:      "KFFC",
		Name:      "Peachtree City Falcon Field",
		City:      "Atlanta",
		State:     "Georgia",
		Country:   "US",
		Elevation: 808,
		Latitude:  33.3572998047,
		Longitude: -84.5718002319,
		Timezone:  "America/New_York",
	},
	"KFFL": {
		ICAO:      "KFFL",
		IATA:      "FFL",
		Name:      "Fairfield Municipal Airport",
		City:      "Fairfield",
		State:     "Iowa",
		Country:   "US",
		Elevation: 799,
		Latitude:  41.0532989502,
		Longitude: -91.9788970947,
		Timezone:  "America/Chicago",
	},
	"KFFM": {
		ICAO:      "KFFM",
		IATA:      "FFM",
		Name:      "Fergus Falls Municipal Einar Mickelson Field",
		City:      "Fergus Falls",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1183,
		Latitude:  46.28440094,
		Longitude: -96.15670013,
		Timezone:  "America/Chicago",
	},
	"KFFO": {
		ICAO:      "KFFO",
		IATA:      "FFO",
		Name:      "Wright Patterson Air Force Base",
		City:      "Dayton",
		State:     "Ohio",
		Country:   "US",
		Elevation: 823,
		Latitude:  39.8260993958,
		Longitude: -84.0483016968,
		Timezone:  "America/New_York",
	},
	"KFFT": {
		ICAO:      "KFFT",
		IATA:      "FFT",
		Name:      "Capital City Airport",
		City:      "Frankfort",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 806,
		Latitude:  38.18249893,
		Longitude: -84.90470123,
		Timezone:  "America/New_York",
	},
	"KFFX": {
		ICAO:      "KFFX",
		Name:      "Fremont Municipal Airport",
		City:      "Fremont",
		State:     "Michigan",
		Country:   "US",
		Elevation: 772,
		Latitude:  43.43930054,
		Longitude: -85.99490356,
		Timezone:  "America/Detroit",
	},
	"KFFZ": {
		ICAO:      "KFFZ",
		IATA:      "MSC",
		Name:      "Falcon Field",
		City:      "Mesa",
		State:     "Alabama",
		Country:   "US",
		Elevation: 1394,
		Latitude:  33.4608001709,
		Longitude: -111.7279968262,
		Timezone:  "America/Phoenix",
	},
	"KFGX": {
		ICAO:      "KFGX",
		Name:      "Fleming Mason Airport",
		City:      "Flemingsburg",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 913,
		Latitude:  38.5418014526,
		Longitude: -83.7434005737,
		Timezone:  "America/New_York",
	},
	"KFHB": {
		ICAO:      "KFHB",
		IATA:      "FHB",
		Name:      "Fernandina Beach Municipal Airport",
		City:      "Fernandina Beach",
		State:     "Florida",
		Country:   "US",
		Elevation: 16,
		Latitude:  30.6117992401,
		Longitude: -81.4611968994,
		Timezone:  "America/New_York",
	},
	"KFHR": {
		ICAO:      "KFHR",
		IATA:      "FRD",
		Name:      "Friday Harbor Airport",
		City:      "Friday Harbor",
		State:     "Washington",
		Country:   "US",
		Elevation: 113,
		Latitude:  48.5219993591,
		Longitude: -123.024002075,
		Timezone:  "America/Los_Angeles",
	},
	"KFHU": {
		ICAO:      "KFHU",
		IATA:      "FHU",
		Name:      "Sierra Vista Municipal Libby Army Air Field",
		City:      "Fort Huachuca Sierra Vista",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4719,
		Latitude:  31.5884990692,
		Longitude: -110.34400177,
		Timezone:  "America/Phoenix",
	},
	"KFIG": {
		ICAO:      "KFIG",
		Name:      "Clearfield Lawrence Airport",
		City:      "Clearfield",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1516,
		Latitude:  41.0485992432,
		Longitude: -78.4131011963,
		Timezone:  "America/New_York",
	},
	"KFIT": {
		ICAO:      "KFIT",
		Name:      "Fitchburg Municipal Airport",
		City:      "Fitchburg",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 348,
		Latitude:  42.5541000366,
		Longitude: -71.7590026855,
		Timezone:  "America/New_York",
	},
	"KFKA": {
		ICAO:      "KFKA",
		Name:      "Fillmore County Airport",
		City:      "Preston",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1277,
		Latitude:  43.67679977,
		Longitude: -92.17970276,
		Timezone:  "America/Chicago",
	},
	"KFKL": {
		ICAO:      "KFKL",
		IATA:      "FKL",
		Name:      "Venango Regional Airport",
		City:      "Franklin",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1540,
		Latitude:  41.3778991699,
		Longitude: -79.8603973389,
		Timezone:  "America/New_York",
	},
	"KFKN": {
		ICAO:      "KFKN",
		IATA:      "FKN",
		Name:      "Franklin Municipal John Beverly Rose Airport",
		City:      "Franklin",
		State:     "Virginia",
		Country:   "US",
		Elevation: 41,
		Latitude:  36.6981010437,
		Longitude: -76.9038009644,
		Timezone:  "America/New_York",
	},
	"KFKR": {
		ICAO:      "KFKR",
		Name:      "Frankfort Municipal Airport",
		City:      "Frankfort",
		State:     "Indiana",
		Country:   "US",
		Elevation: 861,
		Latitude:  40.273399353,
		Longitude: -86.5622024536,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KFKS": {
		ICAO:      "KFKS",
		Name:      "Frankfort Dow Memorial Field",
		City:      "Frankfort",
		State:     "Michigan",
		Country:   "US",
		Elevation: 633,
		Latitude:  44.6251983643,
		Longitude: -86.2007980347,
		Timezone:  "America/Detroit",
	},
	"KFLD": {
		ICAO:      "KFLD",
		IATA:      "FLD",
		Name:      "Fond du Lac County Airport",
		City:      "Fond du Lac",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 808,
		Latitude:  43.7711982727,
		Longitude: -88.4884033203,
		Timezone:  "America/Chicago",
	},
	"KFLG": {
		ICAO:      "KFLG",
		IATA:      "FLG",
		Name:      "Flagstaff Pulliam Airport",
		City:      "Flagstaff",
		State:     "Arizona",
		Country:   "US",
		Elevation: 7014,
		Latitude:  35.13850021,
		Longitude: -111.6709976,
		Timezone:  "America/Phoenix",
	},
	"KFLL": {
		ICAO:      "KFLL",
		IATA:      "FLL",
		Name:      "Fort Lauderdale Hollywood International Airport",
		City:      "Fort Lauderdale",
		State:     "Florida",
		Country:   "US",
		Elevation: 9,
		Latitude:  26.072599411,
		Longitude: -80.1527023315,
		Timezone:  "America/New_York",
	},
	"KFLO": {
		ICAO:      "KFLO",
		IATA:      "FLO",
		Name:      "Florence Regional Airport",
		City:      "Florence",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 146,
		Latitude:  34.1853981018,
		Longitude: -79.7238998413,
		Timezone:  "America/New_York",
	},
	"KFLP": {
		ICAO:      "KFLP",
		IATA:      "FLP",
		Name:      "Marion County Regional Airport",
		City:      "Flippin",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 719,
		Latitude:  36.29090118,
		Longitude: -92.59030151,
		Timezone:  "America/Chicago",
	},
	"KFLV": {
		ICAO:      "KFLV",
		IATA:      "FLV",
		Name:      "Sherman Army Air Field",
		City:      "Fort Leavenworth",
		State:     "Kansas",
		Country:   "US",
		Elevation: 772,
		Latitude:  39.3683013916,
		Longitude: -94.9147033691,
		Timezone:  "America/Chicago",
	},
	"KFLX": {
		ICAO:      "KFLX",
		IATA:      "FLX",
		Name:      "Fallon Municipal Airport",
		City:      "Fallon",
		State:     "Nevada",
		Country:   "US",
		Elevation: 3963,
		Latitude:  39.4990997314,
		Longitude: -118.7490005493,
		Timezone:  "America/Los_Angeles",
	},
	"KFLY": {
		ICAO:      "KFLY",
		Name:      "Meadow Lake Airport",
		City:      "Colorado Springs",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6874,
		Latitude:  38.9458007812,
		Longitude: -104.569999695,
		Timezone:  "America/Denver",
	},
	"KFME": {
		ICAO:      "KFME",
		IATA:      "FME",
		Name:      "Tipton Airport",
		City:      "Fort Meade(Odenton)",
		State:     "Maryland",
		Country:   "US",
		Elevation: 150,
		Latitude:  39.0853996277,
		Longitude: -76.7593994141,
		Timezone:  "America/New_York",
	},
	"KFMH": {
		ICAO:      "KFMH",
		IATA:      "FMH",
		Name:      "Cape Cod Coast Guard Air Station",
		City:      "Falmouth",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 131,
		Latitude:  41.6584014893,
		Longitude: -70.5214004517,
		Timezone:  "America/New_York",
	},
	"KFMM": {
		ICAO:      "KFMM",
		Name:      "Fort Morgan Municipal Airport",
		City:      "Fort Morgan",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4569,
		Latitude:  40.3335990906,
		Longitude: -103.8040008545,
		Timezone:  "America/Denver",
	},
	"KFMN": {
		ICAO:      "KFMN",
		IATA:      "FMN",
		Name:      "Four Corners Regional Airport",
		City:      "Farmington",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 5506,
		Latitude:  36.7411994934,
		Longitude: -108.2300033569,
		Timezone:  "America/Denver",
	},
	"KFMY": {
		ICAO:      "KFMY",
		IATA:      "FMY",
		Name:      "Page Field",
		City:      "Fort Myers",
		State:     "Florida",
		Country:   "US",
		Elevation: 17,
		Latitude:  26.58659935,
		Longitude: -81.8632965088,
		Timezone:  "America/New_York",
	},
	"KFMZ": {
		ICAO:      "KFMZ",
		Name:      "Fairmont State Airfield",
		City:      "Fairmont",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1636,
		Latitude:  40.586101532,
		Longitude: -97.573097229,
		Timezone:  "America/Chicago",
	},
	"KFNB": {
		ICAO:      "KFNB",
		Name:      "Brenner Field",
		City:      "Falls City",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 984,
		Latitude:  40.0788002014,
		Longitude: -95.5920028687,
		Timezone:  "America/Chicago",
	},
	"KFNL": {
		ICAO:      "KFNL",
		IATA:      "FNL",
		Name:      "Fort Collins Loveland Municipal Airport",
		City:      "Fort Collins/Loveland",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5016,
		Latitude:  40.4518013,
		Longitude: -105.011001587,
		Timezone:  "America/Denver",
	},
	"KFNT": {
		ICAO:      "KFNT",
		IATA:      "FNT",
		Name:      "Bishop International Airport",
		City:      "Flint",
		State:     "Michigan",
		Country:   "US",
		Elevation: 782,
		Latitude:  42.9654006958,
		Longitude: -83.743598938,
		Timezone:  "America/Detroit",
	},
	"KFOA": {
		ICAO:      "KFOA",
		Name:      "Flora Municipal Airport",
		City:      "Flora",
		State:     "Illinois",
		Country:   "US",
		Elevation: 473,
		Latitude:  38.6649017334,
		Longitude: -88.4530029297,
		Timezone:  "America/Chicago",
	},
	"KFOD": {
		ICAO:      "KFOD",
		IATA:      "FOD",
		Name:      "Fort Dodge Regional Airport",
		City:      "Fort Dodge",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1156,
		Latitude:  42.55149841,
		Longitude: -94.19259644,
		Timezone:  "America/Chicago",
	},
	"KFOE": {
		ICAO:      "KFOE",
		IATA:      "FOE",
		Name:      "Forbes Field",
		City:      "Topeka",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1078,
		Latitude:  38.9509010315,
		Longitude: -95.6635971069,
		Timezone:  "America/Chicago",
	},
	"KFOK": {
		ICAO:      "KFOK",
		IATA:      "FOK",
		Name:      "Francis S Gabreski Airport",
		City:      "Westhampton Beach",
		State:     "New-York",
		Country:   "US",
		Elevation: 67,
		Latitude:  40.8437004089,
		Longitude: -72.6317977905,
		Timezone:  "America/New_York",
	},
	"KFOM": {
		ICAO:      "KFOM",
		IATA:      "FIL",
		Name:      "Fillmore Municipal Airport",
		City:      "Fillmore",
		State:     "Utah",
		Country:   "US",
		Elevation: 4985,
		Latitude:  38.95830154,
		Longitude: -112.362999,
		Timezone:  "America/Denver",
	},
	"KFOT": {
		ICAO:      "KFOT",
		Name:      "Rohnerville Airport",
		City:      "Fortuna",
		State:     "California",
		Country:   "US",
		Elevation: 393,
		Latitude:  40.55390167,
		Longitude: -124.1330032,
		Timezone:  "America/Los_Angeles",
	},
	"KFOZ": {
		ICAO:      "KFOZ",
		Name:      "Bigfork Municipal Airport",
		City:      "Bigfork",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1348,
		Latitude:  47.7827987671,
		Longitude: -93.6501998901,
		Timezone:  "America/Chicago",
	},
	"KFPK": {
		ICAO:      "KFPK",
		Name:      "Fitch H Beach Airport",
		City:      "Charlotte",
		State:     "Michigan",
		Country:   "US",
		Elevation: 891,
		Latitude:  42.5745010376,
		Longitude: -84.8114013672,
		Timezone:  "America/Detroit",
	},
	"KFPR": {
		ICAO:      "KFPR",
		IATA:      "FPR",
		Name:      "St Lucie County International Airport",
		City:      "Fort Pierce",
		State:     "Florida",
		Country:   "US",
		Elevation: 24,
		Latitude:  27.49510002,
		Longitude: -80.36830139,
		Timezone:  "America/New_York",
	},
	"KFQD": {
		ICAO:      "KFQD",
		Name:      "Rutherford Co Marchman Field",
		City:      "Rutherfordton",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 1077,
		Latitude:  35.42819977,
		Longitude: -81.93509674,
		Timezone:  "America/New_York",
	},
	"KFRG": {
		ICAO:      "KFRG",
		IATA:      "FRG",
		Name:      "Republic Airport",
		City:      "Farmingdale",
		State:     "New-York",
		Country:   "US",
		Elevation: 82,
		Latitude:  40.7288017273,
		Longitude: -73.4133987427,
		Timezone:  "America/New_York",
	},
	"KFRH": {
		ICAO:      "KFRH",
		IATA:      "FRH",
		Name:      "French Lick Municipal Airport",
		City:      "French Lick",
		State:     "Indiana",
		Country:   "US",
		Elevation: 792,
		Latitude:  38.5061988831,
		Longitude: -86.6369018555,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KFRI": {
		ICAO:      "KFRI",
		IATA:      "FRI",
		Name:      "Marshall Army Air Field",
		City:      "Fort Riley(Junction City)",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1065,
		Latitude:  39.05530167,
		Longitude: -96.76450348,
		Timezone:  "America/Chicago",
	},
	"KFRM": {
		ICAO:      "KFRM",
		IATA:      "FRM",
		Name:      "Fairmont Municipal Airport",
		City:      "Fairmont",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1162,
		Latitude:  43.643901825,
		Longitude: -94.4156036377,
		Timezone:  "America/Chicago",
	},
	"KFRR": {
		ICAO:      "KFRR",
		IATA:      "FRR",
		Name:      "Front Royal Warren County Airport",
		City:      "Front Royal",
		State:     "Virginia",
		Country:   "US",
		Elevation: 709,
		Latitude:  38.9174995422,
		Longitude: -78.2535018921,
		Timezone:  "America/New_York",
	},
	"KFSD": {
		ICAO:      "KFSD",
		IATA:      "FSD",
		Name:      "Joe Foss Field Airport",
		City:      "Sioux Falls",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1429,
		Latitude:  43.5820007324,
		Longitude: -96.741897583,
		Timezone:  "America/Chicago",
	},
	"KFSE": {
		ICAO:      "KFSE",
		Name:      "Fosston Municipal Airport",
		City:      "Fosston",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1277,
		Latitude:  47.5928001404,
		Longitude: -95.7734985352,
		Timezone:  "America/Chicago",
	},
	"KFSI": {
		ICAO:      "KFSI",
		IATA:      "FSI",
		Name:      "Henry Post Army Air Field (Fort Sill)",
		City:      "Fort Sill",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1189,
		Latitude:  34.64979935,
		Longitude: -98.40219879,
		Timezone:  "America/Chicago",
	},
	"KFSK": {
		ICAO:      "KFSK",
		IATA:      "FSK",
		Name:      "Fort Scott Municipal Airport",
		City:      "Fort Scott",
		State:     "Kansas",
		Country:   "US",
		Elevation: 918,
		Latitude:  37.7984008789,
		Longitude: -94.7694015503,
		Timezone:  "America/Chicago",
	},
	"KFSM": {
		ICAO:      "KFSM",
		IATA:      "FSM",
		Name:      "Fort Smith Regional Airport",
		City:      "Fort Smith",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 469,
		Latitude:  35.3366012573,
		Longitude: -94.367401123,
		Timezone:  "America/Chicago",
	},
	"KFSO": {
		ICAO:      "KFSO",
		Name:      "Franklin County State Airport",
		City:      "Highgate",
		State:     "Vermont",
		Country:   "US",
		Elevation: 228,
		Latitude:  44.94029999,
		Longitude: -73.09750366,
		Timezone:  "America/New_York",
	},
	"KFST": {
		ICAO:      "KFST",
		IATA:      "FST",
		Name:      "Fort Stockton Pecos County Airport",
		City:      "Fort Stockton",
		State:     "Texas",
		Country:   "US",
		Elevation: 3011,
		Latitude:  30.9157009125,
		Longitude: -102.9160003662,
		Timezone:  "America/Chicago",
	},
	"KFSU": {
		ICAO:      "KFSU",
		IATA:      "FSU",
		Name:      "Fort Sumner Municipal Airport",
		City:      "Fort Sumner",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4165,
		Latitude:  34.4833984375,
		Longitude: -104.2170028687,
		Timezone:  "America/Denver",
	},
	"KFSW": {
		ICAO:      "KFSW",
		IATA:      "FMS",
		Name:      "Fort Madison Municipal Airport",
		City:      "Fort Madison",
		State:     "Iowa",
		Country:   "US",
		Elevation: 724,
		Latitude:  40.6593017578,
		Longitude: -91.3267974854,
		Timezone:  "America/Chicago",
	},
	"KFTG": {
		ICAO:      "KFTG",
		Name:      "Front Range Airport",
		City:      "Denver",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5512,
		Latitude:  39.7853012085,
		Longitude: -104.5429992676,
		Timezone:  "America/Denver",
	},
	"KFTK": {
		ICAO:      "KFTK",
		IATA:      "FTK",
		Name:      "Godman Army Air Field",
		City:      "Fort Knox",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 756,
		Latitude:  37.9071006775,
		Longitude: -85.9720993042,
		Timezone:  "America/New_York",
	},
	"KFTT": {
		ICAO:      "KFTT",
		Name:      "Elton Hensley Memorial Airport",
		City:      "Fulton",
		State:     "Missouri",
		Country:   "US",
		Elevation: 886,
		Latitude:  38.8381004333,
		Longitude: -92.0026016235,
		Timezone:  "America/Chicago",
	},
	"KFTW": {
		ICAO:      "KFTW",
		IATA:      "FTW",
		Name:      "Fort Worth Meacham International Airport",
		City:      "Fort Worth",
		State:     "Texas",
		Country:   "US",
		Elevation: 710,
		Latitude:  32.8198013306,
		Longitude: -97.3623962402,
		Timezone:  "America/Chicago",
	},
	"KFTY": {
		ICAO:      "KFTY",
		IATA:      "FTY",
		Name:      "Fulton County Airport Brown Field",
		City:      "Atlanta",
		State:     "Georgia",
		Country:   "US",
		Elevation: 841,
		Latitude:  33.7790985107,
		Longitude: -84.5214004517,
		Timezone:  "America/New_York",
	},
	"KFUL": {
		ICAO:      "KFUL",
		IATA:      "FUL",
		Name:      "Fullerton Municipal Airport",
		City:      "Fullerton",
		State:     "California",
		Country:   "US",
		Elevation: 96,
		Latitude:  33.8720016479,
		Longitude: -117.980003357,
		Timezone:  "America/Los_Angeles",
	},
	"KFVE": {
		ICAO:      "KFVE",
		IATA:      "WFK",
		Name:      "Northern Aroostook Regional Airport",
		City:      "Frenchville",
		State:     "Maine",
		Country:   "US",
		Elevation: 988,
		Latitude:  47.2854995728,
		Longitude: -68.3127975464,
		Timezone:  "America/New_York",
	},
	"KFVX": {
		ICAO:      "KFVX",
		Name:      "Farmville Regional Airport",
		City:      "Farmville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 416,
		Latitude:  37.35749817,
		Longitude: -78.43779755,
		Timezone:  "America/New_York",
	},
	"KFWA": {
		ICAO:      "KFWA",
		IATA:      "FWA",
		Name:      "Fort Wayne International Airport",
		City:      "Fort Wayne",
		State:     "Indiana",
		Country:   "US",
		Elevation: 814,
		Latitude:  40.97850037,
		Longitude: -85.19509888,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KFWB": {
		ICAO:      "KFWB",
		Name:      "Branson West Airport",
		City:      "Branson West",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1348,
		Latitude:  36.698497,
		Longitude: -93.402249,
		Timezone:  "America/Chicago",
	},
	"KFWC": {
		ICAO:      "KFWC",
		Name:      "Fairfield Municipal Airport",
		City:      "Fairfield",
		State:     "Illinois",
		Country:   "US",
		Elevation: 436,
		Latitude:  38.3786010742,
		Longitude: -88.4126968384,
		Timezone:  "America/Chicago",
	},
	"KFWN": {
		ICAO:      "KFWN",
		Name:      "Sussex Airport",
		City:      "Sussex",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 421,
		Latitude:  41.2001991272,
		Longitude: -74.6230010986,
		Timezone:  "America/New_York",
	},
	"KFWQ": {
		ICAO:      "KFWQ",
		Name:      "Rostraver Airport",
		City:      "Monongahela",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1228,
		Latitude:  40.2097015381,
		Longitude: -79.8313980103,
		Timezone:  "America/New_York",
	},
	"KFWS": {
		ICAO:      "KFWS",
		Name:      "Fort Worth Spinks Airport",
		City:      "Fort Worth",
		State:     "Texas",
		Country:   "US",
		Elevation: 700,
		Latitude:  32.5652008057,
		Longitude: -97.3080978394,
		Timezone:  "America/Chicago",
	},
	"KFXE": {
		ICAO:      "KFXE",
		IATA:      "FXE",
		Name:      "Fort Lauderdale Executive Airport",
		City:      "Fort Lauderdale",
		State:     "Florida",
		Country:   "US",
		Elevation: 13,
		Latitude:  26.1972999573,
		Longitude: -80.1707000732,
		Timezone:  "America/New_York",
	},
	"KFXY": {
		ICAO:      "KFXY",
		IATA:      "FXY",
		Name:      "Forest City Municipal Airport",
		City:      "Forest City",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1229,
		Latitude:  43.23469925,
		Longitude: -93.62409973,
		Timezone:  "America/Chicago",
	},
	"KFYE": {
		ICAO:      "KFYE",
		Name:      "Fayette County Airport",
		City:      "Somerville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 436,
		Latitude:  35.207698822,
		Longitude: -89.3945007324,
		Timezone:  "America/Chicago",
	},
	"KFYJ": {
		ICAO:      "KFYJ",
		Name:      "Middle Peninsula Regional Airport",
		City:      "West Point",
		State:     "Virginia",
		Country:   "US",
		Elevation: 20,
		Latitude:  37.52119827,
		Longitude: -76.76470184,
		Timezone:  "America/New_York",
	},
	"KFYM": {
		ICAO:      "KFYM",
		IATA:      "FYM",
		Name:      "Fayetteville Municipal Airport",
		City:      "Fayetteville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 984,
		Latitude:  35.0597000122,
		Longitude: -86.5640029907,
		Timezone:  "America/Chicago",
	},
	"KFYV": {
		ICAO:      "KFYV",
		IATA:      "FYV",
		Name:      "Drake Field",
		City:      "Fayetteville",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1251,
		Latitude:  36.0051002502,
		Longitude: -94.1700973511,
		Timezone:  "America/Chicago",
	},
	"KFZG": {
		ICAO:      "KFZG",
		Name:      "Fitzgerald Municipal Airport",
		City:      "Fitzgerald",
		State:     "Georgia",
		Country:   "US",
		Elevation: 365,
		Latitude:  31.6837005615,
		Longitude: -83.2705001831,
		Timezone:  "America/New_York",
	},
	"KFZI": {
		ICAO:      "KFZI",
		Name:      "Fostoria Metropolitan Airport",
		City:      "Fostoria",
		State:     "Ohio",
		Country:   "US",
		Elevation: 752,
		Latitude:  41.19079971,
		Longitude: -83.39450073,
		Timezone:  "America/New_York",
	},
	"KFZY": {
		ICAO:      "KFZY",
		Name:      "Oswego County Airport",
		City:      "Fulton",
		State:     "New-York",
		Country:   "US",
		Elevation: 475,
		Latitude:  43.3507995605,
		Longitude: -76.3880996704,
		Timezone:  "America/New_York",
	},
	"KGAB": {
		ICAO:      "KGAB",
		IATA:      "GAB",
		Name:      "Gabbs Airport",
		City:      "Gabbs",
		State:     "Nevada",
		Country:   "US",
		Elevation: 4700,
		Latitude:  38.9240989685,
		Longitude: -117.958999634,
		Timezone:  "America/Los_Angeles",
	},
	"KGAD": {
		ICAO:      "KGAD",
		IATA:      "GAD",
		Name:      "Northeast Alabama Regional Airport",
		City:      "Gadsden",
		State:     "Alabama",
		Country:   "US",
		Elevation: 569,
		Latitude:  33.9725990295,
		Longitude: -86.0889968872,
		Timezone:  "America/Chicago",
	},
	"KGAF": {
		ICAO:      "KGAF",
		Name:      "Hutson Field",
		City:      "Grafton",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 824,
		Latitude:  48.4047012329,
		Longitude: -97.3709030151,
		Timezone:  "America/Chicago",
	},
	"KGAG": {
		ICAO:      "KGAG",
		IATA:      "GAG",
		Name:      "Gage Airport",
		City:      "Gage",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 2223,
		Latitude:  36.295501709,
		Longitude: -99.7763977051,
		Timezone:  "America/Chicago",
	},
	"KGAI": {
		ICAO:      "KGAI",
		IATA:      "GAI",
		Name:      "Montgomery County Airpark",
		City:      "Gaithersburg",
		State:     "Maryland",
		Country:   "US",
		Elevation: 539,
		Latitude:  39.1683006287,
		Longitude: -77.1660003662,
		Timezone:  "America/New_York",
	},
	"KGAO": {
		ICAO:      "KGAO",
		Name:      "South Lafourche Leonard Miller Jr Airport",
		City:      "Galliano",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 1,
		Latitude:  29.44479942,
		Longitude: -90.26110077,
		Timezone:  "America/Chicago",
	},
	"KGAS": {
		ICAO:      "KGAS",
		Name:      "Gallia Meigs Regional Airport",
		City:      "Gallipolis",
		State:     "Ohio",
		Country:   "US",
		Elevation: 566,
		Latitude:  38.8340988159,
		Longitude: -82.1633987427,
		Timezone:  "America/New_York",
	},
	"KGBD": {
		ICAO:      "KGBD",
		IATA:      "GBD",
		Name:      "Great Bend Municipal Airport",
		City:      "Great Bend",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1887,
		Latitude:  38.3442993164,
		Longitude: -98.8591995239,
		Timezone:  "America/Chicago",
	},
	"KGBG": {
		ICAO:      "KGBG",
		IATA:      "GBG",
		Name:      "Galesburg Municipal Airport",
		City:      "Galesburg",
		State:     "Illinois",
		Country:   "US",
		Elevation: 764,
		Latitude:  40.9379997253,
		Longitude: -90.431098938,
		Timezone:  "America/Chicago",
	},
	"KGBN": {
		ICAO:      "KGBN",
		Name:      "Gila Bend Af Aux Airport",
		City:      "Gila Bend",
		State:     "Arizona",
		Country:   "US",
		Elevation: 858,
		Latitude:  32.88750076,
		Longitude: -112.7200012,
		Timezone:  "America/Phoenix",
	},
	"KGBR": {
		ICAO:      "KGBR",
		IATA:      "GBR",
		Name:      "Walter J. Koladza Airport",
		City:      "Great Barrington",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 739,
		Latitude:  42.18420029,
		Longitude: -73.40319824,
		Timezone:  "America/New_York",
	},
	"KGCC": {
		ICAO:      "KGCC",
		IATA:      "GCC",
		Name:      "Gillette Campbell County Airport",
		City:      "Gillette",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4365,
		Latitude:  44.3488998413,
		Longitude: -105.5390014648,
		Timezone:  "America/Denver",
	},
	"KGCD": {
		ICAO:      "KGCD",
		IATA:      "JDA",
		Name:      "Grant Co Regional/Ogilvie Field",
		City:      "John Day",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3703,
		Latitude:  44.40420151,
		Longitude: -118.9629974,
		Timezone:  "America/Los_Angeles",
	},
	"KGCK": {
		ICAO:      "KGCK",
		IATA:      "GCK",
		Name:      "Garden City Regional Airport",
		City:      "Garden City",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2891,
		Latitude:  37.9275016785,
		Longitude: -100.7239990234,
		Timezone:  "America/Chicago",
	},
	"KGCM": {
		ICAO:      "KGCM",
		Name:      "Claremore Regional Airport",
		City:      "Claremore",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 733,
		Latitude:  36.2927017212,
		Longitude: -95.479598999,
		Timezone:  "America/Chicago",
	},
	"KGCN": {
		ICAO:      "KGCN",
		IATA:      "GCN",
		Name:      "Grand Canyon National Park Airport",
		City:      "Grand Canyon",
		State:     "Arizona",
		Country:   "US",
		Elevation: 6609,
		Latitude:  35.9524002075,
		Longitude: -112.1470031738,
		Timezone:  "America/Phoenix",
	},
	"KGCT": {
		ICAO:      "KGCT",
		Name:      "Guthrie County Regional Airport",
		City:      "Guthrie Center",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1230,
		Latitude:  41.6878013611,
		Longitude: -94.4352035522,
		Timezone:  "America/Chicago",
	},
	"KGCY": {
		ICAO:      "KGCY",
		IATA:      "GCY",
		Name:      "Greeneville Greene County Municipal Airport",
		City:      "Greeneville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1608,
		Latitude:  36.19300079,
		Longitude: -82.81510162,
		Timezone:  "America/New_York",
	},
	"KGDB": {
		ICAO:      "KGDB",
		Name:      "Granite Falls Municipal-Lenzen-Roe Memorial Field",
		City:      "Granite Falls",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1047,
		Latitude:  44.75329971,
		Longitude: -95.55599976,
		Timezone:  "America/Chicago",
	},
	"KGDJ": {
		ICAO:      "KGDJ",
		Name:      "Granbury Regional Airport",
		City:      "Granbury",
		State:     "Texas",
		Country:   "US",
		Elevation: 778,
		Latitude:  32.44440079,
		Longitude: -97.81690216,
		Timezone:  "America/Chicago",
	},
	"KGDM": {
		ICAO:      "KGDM",
		IATA:      "GDM",
		Name:      "Gardner Municipal Airport",
		City:      "Gardner",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 955,
		Latitude:  42.5499992371,
		Longitude: -72.0160980225,
		Timezone:  "America/New_York",
	},
	"KGDV": {
		ICAO:      "KGDV",
		IATA:      "GDV",
		Name:      "Dawson Community Airport",
		City:      "Glendive",
		State:     "Montana",
		Country:   "US",
		Elevation: 2458,
		Latitude:  47.13869858,
		Longitude: -104.8069992,
		Timezone:  "America/Denver",
	},
	"KGDW": {
		ICAO:      "KGDW",
		IATA:      "GDW",
		Name:      "Gladwin Zettel Memorial Airport",
		City:      "Gladwin",
		State:     "Michigan",
		Country:   "US",
		Elevation: 776,
		Latitude:  43.9706001282,
		Longitude: -84.4749984741,
		Timezone:  "America/Detroit",
	},
	"KGDY": {
		ICAO:      "KGDY",
		IATA:      "GDY",
		Name:      "Grundy Municipal Airport",
		City:      "Grundy",
		State:     "Virginia",
		Country:   "US",
		Elevation: 2304,
		Latitude:  37.2323989868,
		Longitude: -82.125,
		Timezone:  "America/New_York",
	},
	"KGED": {
		ICAO:      "KGED",
		IATA:      "GED",
		Name:      "Sussex County Airport",
		City:      "Georgetown",
		State:     "Delaware",
		Country:   "US",
		Elevation: 53,
		Latitude:  38.68920135,
		Longitude: -75.35890198,
		Timezone:  "America/New_York",
	},
	"KGEG": {
		ICAO:      "KGEG",
		IATA:      "GEG",
		Name:      "Spokane International Airport",
		City:      "Spokane",
		State:     "Washington",
		Country:   "US",
		Elevation: 2376,
		Latitude:  47.6198997498,
		Longitude: -117.533996582,
		Timezone:  "America/Los_Angeles",
	},
	"KGEO": {
		ICAO:      "KGEO",
		Name:      "Brown County Airport",
		City:      "Georgetown",
		State:     "Ohio",
		Country:   "US",
		Elevation: 958,
		Latitude:  38.8819007874,
		Longitude: -83.8826980591,
		Timezone:  "America/New_York",
	},
	"KGEU": {
		ICAO:      "KGEU",
		Name:      "Glendale Municipal Airport",
		City:      "Glendale",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1071,
		Latitude:  33.5269012451,
		Longitude: -112.2949981689,
		Timezone:  "America/Phoenix",
	},
	"KGEV": {
		ICAO:      "KGEV",
		Name:      "Ashe County Airport",
		City:      "Jefferson",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 3180,
		Latitude:  36.4323997498,
		Longitude: -81.4197006226,
		Timezone:  "America/New_York",
	},
	"KGEY": {
		ICAO:      "KGEY",
		IATA:      "GEY",
		Name:      "South Big Horn County Airport",
		City:      "Greybull",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 3939,
		Latitude:  44.51679993,
		Longitude: -108.0830002,
		Timezone:  "America/Denver",
	},
	"KGEZ": {
		ICAO:      "KGEZ",
		Name:      "Shelbyville Municipal Airport",
		City:      "Shelbyville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 803,
		Latitude:  39.58319855,
		Longitude: -85.80480194,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KGFK": {
		ICAO:      "KGFK",
		IATA:      "GFK",
		Name:      "Grand Forks International Airport",
		City:      "Grand Forks",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 845,
		Latitude:  47.9492988586,
		Longitude: -97.1761016846,
		Timezone:  "America/Chicago",
	},
	"KGFL": {
		ICAO:      "KGFL",
		IATA:      "GFL",
		Name:      "Floyd Bennett Memorial Airport",
		City:      "Glens Falls",
		State:     "New-York",
		Country:   "US",
		Elevation: 328,
		Latitude:  43.3412017822,
		Longitude: -73.6102981567,
		Timezone:  "America/New_York",
	},
	"KGFZ": {
		ICAO:      "KGFZ",
		Name:      "Greenfield Municipal Airport",
		City:      "Greenfield",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1364,
		Latitude:  41.32699966,
		Longitude: -94.4457016,
		Timezone:  "America/Chicago",
	},
	"KGGE": {
		ICAO:      "KGGE",
		IATA:      "GGE",
		Name:      "Georgetown County Airport",
		City:      "Georgetown",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 39,
		Latitude:  33.3116989136,
		Longitude: -79.3196029663,
		Timezone:  "America/New_York",
	},
	"KGGF": {
		ICAO:      "KGGF",
		Name:      "Grant Municipal Airport",
		City:      "Grant",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3425,
		Latitude:  40.86949921,
		Longitude: -101.7330017,
		Timezone:  "America/Denver",
	},
	"KGGG": {
		ICAO:      "KGGG",
		IATA:      "GGG",
		Name:      "East Texas Regional Airport",
		City:      "Longview",
		State:     "Texas",
		Country:   "US",
		Elevation: 365,
		Latitude:  32.3839988708,
		Longitude: -94.7115020752,
		Timezone:  "America/Chicago",
	},
	"KGGI": {
		ICAO:      "KGGI",
		Name:      "Grinnell Regional Airport",
		City:      "Grinnell",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1008,
		Latitude:  41.7092018127,
		Longitude: -92.7349014282,
		Timezone:  "America/Chicago",
	},
	"KGGP": {
		ICAO:      "KGGP",
		Name:      "Logansport Cass County Airport",
		City:      "Logansport",
		State:     "Indiana",
		Country:   "US",
		Elevation: 738,
		Latitude:  40.7112007141,
		Longitude: -86.3749008179,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KGGW": {
		ICAO:      "KGGW",
		IATA:      "GGW",
		Name:      "Wokal Field Glasgow International Airport",
		City:      "Glasgow",
		State:     "Montana",
		Country:   "US",
		Elevation: 2296,
		Latitude:  48.2125015259,
		Longitude: -106.6149978638,
		Timezone:  "America/Denver",
	},
	"KGHG": {
		ICAO:      "KGHG",
		Name:      "Marshfield Municipal George Harlow Field",
		City:      "Marshfield",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 11,
		Latitude:  42.0983009338,
		Longitude: -70.672203064,
		Timezone:  "America/New_York",
	},
	"KGHM": {
		ICAO:      "KGHM",
		IATA:      "GHM",
		Name:      "Centerville Municipal Airport",
		City:      "Centerville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 768,
		Latitude:  35.8373985291,
		Longitude: -87.4453964233,
		Timezone:  "America/Chicago",
	},
	"KGHW": {
		ICAO:      "KGHW",
		Name:      "Glenwood Municipal Airport",
		City:      "Glenwood",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1393,
		Latitude:  45.643901825,
		Longitude: -95.3203964233,
		Timezone:  "America/Chicago",
	},
	"KGIC": {
		ICAO:      "KGIC",
		IATA:      "IDH",
		Name:      "Idaho County Airport",
		City:      "Grangeville",
		State:     "Idaho",
		Country:   "US",
		Elevation: 3314,
		Latitude:  45.94260025,
		Longitude: -116.1230011,
		Timezone:  "America/Los_Angeles",
	},
	"KGIF": {
		ICAO:      "KGIF",
		IATA:      "GIF",
		Name:      "Winter Haven's Gilbert Airport",
		City:      "Winter Haven",
		State:     "Florida",
		Country:   "US",
		Elevation: 145,
		Latitude:  28.06290054,
		Longitude: -81.75330353,
		Timezone:  "America/New_York",
	},
	"KGJT": {
		ICAO:      "KGJT",
		IATA:      "GJT",
		Name:      "Grand Junction Regional Airport",
		City:      "Grand Junction",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4858,
		Latitude:  39.1223983765,
		Longitude: -108.527000427,
		Timezone:  "America/Denver",
	},
	"KGKJ": {
		ICAO:      "KGKJ",
		IATA:      "MEJ",
		Name:      "Port Meadville Airport",
		City:      "Meadville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1399,
		Latitude:  41.62649918,
		Longitude: -80.21469879,
		Timezone:  "America/New_York",
	},
	"KGKT": {
		ICAO:      "KGKT",
		IATA:      "GKT",
		Name:      "Gatlinburg Pigeon Forge Airport",
		City:      "Sevierville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1014,
		Latitude:  35.85779953,
		Longitude: -83.5287017822,
		Timezone:  "America/New_York",
	},
	"KGKY": {
		ICAO:      "KGKY",
		Name:      "Arlington Municipal Airport",
		City:      "Arlington",
		State:     "Oregon",
		Country:   "US",
		Elevation: 628,
		Latitude:  32.663898468,
		Longitude: -97.0942993164,
		Timezone:  "America/Chicago",
	},
	"KGLD": {
		ICAO:      "KGLD",
		IATA:      "GLD",
		Name:      "Renner Field-Goodland Municipal Airport",
		City:      "Goodland",
		State:     "Kansas",
		Country:   "US",
		Elevation: 3656,
		Latitude:  39.37060165,
		Longitude: -101.6989975,
		Timezone:  "America/Denver",
	},
	"KGLE": {
		ICAO:      "KGLE",
		IATA:      "GLE",
		Name:      "Gainesville Municipal Airport",
		City:      "Gainesville",
		State:     "Texas",
		Country:   "US",
		Elevation: 845,
		Latitude:  33.6514015198,
		Longitude: -97.1969985962,
		Timezone:  "America/Chicago",
	},
	"KGLH": {
		ICAO:      "KGLH",
		IATA:      "GLH",
		Name:      "Mid Delta Regional Airport",
		City:      "Greenville",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 131,
		Latitude:  33.4828987122,
		Longitude: -90.9856033325,
		Timezone:  "America/Chicago",
	},
	"KGLR": {
		ICAO:      "KGLR",
		IATA:      "GLR",
		Name:      "Gaylord Regional Airport",
		City:      "Gaylord",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1328,
		Latitude:  45.0135002136,
		Longitude: -84.7035980225,
		Timezone:  "America/Detroit",
	},
	"KGLS": {
		ICAO:      "KGLS",
		IATA:      "GLS",
		Name:      "Scholes International At Galveston Airport",
		City:      "Galveston",
		State:     "Texas",
		Country:   "US",
		Elevation: 6,
		Latitude:  29.2653007507,
		Longitude: -94.8603973389,
		Timezone:  "America/Chicago",
	},
	"KGLW": {
		ICAO:      "KGLW",
		IATA:      "GLW",
		Name:      "Glasgow Municipal Airport",
		City:      "Glasgow",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 716,
		Latitude:  37.03179932,
		Longitude: -85.9536972,
		Timezone:  "America/Chicago",
	},
	"KGLY": {
		ICAO:      "KGLY",
		Name:      "Clinton Memorial Airport",
		City:      "Clinton",
		State:     "Missouri",
		Country:   "US",
		Elevation: 822,
		Latitude:  38.3566017151,
		Longitude: -93.6841964722,
		Timezone:  "America/Chicago",
	},
	"KGMJ": {
		ICAO:      "KGMJ",
		Name:      "Grove Municipal Airport",
		City:      "Grove",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 831,
		Latitude:  36.60680008,
		Longitude: -94.73860168,
		Timezone:  "America/Chicago",
	},
	"KGMU": {
		ICAO:      "KGMU",
		IATA:      "GMU",
		Name:      "Greenville Downtown Airport",
		City:      "Greenville",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 1048,
		Latitude:  34.8479003906,
		Longitude: -82.3499984741,
		Timezone:  "America/New_York",
	},
	"KGNB": {
		ICAO:      "KGNB",
		Name:      "Granby Grand County Airport",
		City:      "Granby",
		State:     "Colorado",
		Country:   "US",
		Elevation: 8203,
		Latitude:  40.0896987915,
		Longitude: -105.9169998169,
		Timezone:  "America/Denver",
	},
	"KGNC": {
		ICAO:      "KGNC",
		Name:      "Gaines County Airport",
		City:      "Seminole",
		State:     "Texas",
		Country:   "US",
		Elevation: 3315,
		Latitude:  32.6753005981,
		Longitude: -102.6529998779,
		Timezone:  "America/Chicago",
	},
	"KGNF": {
		ICAO:      "KGNF",
		Name:      "Grenada Municipal Airport",
		City:      "Grenada",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 208,
		Latitude:  33.8325004578,
		Longitude: -89.7982025146,
		Timezone:  "America/Chicago",
	},
	"KGNG": {
		ICAO:      "KGNG",
		IATA:      "GNG",
		Name:      "Gooding Municipal Airport",
		City:      "Gooding",
		State:     "Idaho",
		Country:   "US",
		Elevation: 3732,
		Latitude:  42.91719818,
		Longitude: -114.7649994,
		Timezone:  "America/Boise",
	},
	"KGNT": {
		ICAO:      "KGNT",
		IATA:      "GNT",
		Name:      "Grants Milan Municipal Airport",
		City:      "Grants",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 6537,
		Latitude:  35.167301178,
		Longitude: -107.9020004272,
		Timezone:  "America/Denver",
	},
	"KGNV": {
		ICAO:      "KGNV",
		IATA:      "GNV",
		Name:      "Gainesville Regional Airport",
		City:      "Gainesville",
		State:     "Florida",
		Country:   "US",
		Elevation: 152,
		Latitude:  29.6900997162,
		Longitude: -82.2717971802,
		Timezone:  "America/New_York",
	},
	"KGOK": {
		ICAO:      "KGOK",
		IATA:      "GOK",
		Name:      "Guthrie-Edmond Regional Airport",
		City:      "Guthrie",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1069,
		Latitude:  35.84980011,
		Longitude: -97.41560364,
		Timezone:  "America/Chicago",
	},
	"KGON": {
		ICAO:      "KGON",
		IATA:      "GON",
		Name:      "Groton New London Airport",
		City:      "Groton (New London)",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 9,
		Latitude:  41.3301010132,
		Longitude: -72.0450973511,
		Timezone:  "America/New_York",
	},
	"KGOO": {
		ICAO:      "KGOO",
		Name:      "Nevada County Air Park",
		City:      "Grass Valley",
		State:     "California",
		Country:   "US",
		Elevation: 3152,
		Latitude:  39.2239990234,
		Longitude: -121.0029983521,
		Timezone:  "America/Los_Angeles",
	},
	"KGOP": {
		ICAO:      "KGOP",
		Name:      "Gatesville Municipal Airport",
		City:      "Gatesville",
		State:     "Texas",
		Country:   "US",
		Elevation: 905,
		Latitude:  31.42130089,
		Longitude: -97.79699707,
		Timezone:  "America/Chicago",
	},
	"KGOV": {
		ICAO:      "KGOV",
		Name:      "Grayling Army Air Field",
		City:      "Grayling",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1158,
		Latitude:  44.6803016663,
		Longitude: -84.7288970947,
		Timezone:  "America/Detroit",
	},
	"KGPH": {
		ICAO:      "KGPH",
		Name:      "Midwest National Air Center Airport",
		City:      "Mosby",
		State:     "Missouri",
		Country:   "US",
		Elevation: 777,
		Latitude:  39.33250046,
		Longitude: -94.30960083,
		Timezone:  "America/Chicago",
	},
	"KGPI": {
		ICAO:      "KGPI",
		IATA:      "FCA",
		Name:      "Glacier Park International Airport",
		City:      "Kalispell",
		State:     "Montana",
		Country:   "US",
		Elevation: 2977,
		Latitude:  48.3105010986,
		Longitude: -114.2559967041,
		Timezone:  "America/Denver",
	},
	"KGPM": {
		ICAO:      "KGPM",
		Name:      "Grand Prairie Municipal Airport",
		City:      "Grand Prairie",
		State:     "Texas",
		Country:   "US",
		Elevation: 588,
		Latitude:  32.6987991333,
		Longitude: -97.0468978882,
		Timezone:  "America/Chicago",
	},
	"KGPT": {
		ICAO:      "KGPT",
		IATA:      "GPT",
		Name:      "Gulfport Biloxi International Airport",
		City:      "Gulfport",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 28,
		Latitude:  30.4073009491,
		Longitude: -89.070098877,
		Timezone:  "America/Chicago",
	},
	"KGPZ": {
		ICAO:      "KGPZ",
		IATA:      "GPZ",
		Name:      "Grand Rapids Itasca Co-Gordon Newstrom field",
		City:      "Grand Rapids",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1355,
		Latitude:  47.21110153,
		Longitude: -93.50980377,
		Timezone:  "America/Chicago",
	},
	"KGQQ": {
		ICAO:      "KGQQ",
		IATA:      "GQQ",
		Name:      "Galion Municipal Airport",
		City:      "Galion",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1224,
		Latitude:  40.7533988953,
		Longitude: -82.7238006592,
		Timezone:  "America/New_York",
	},
	"KGRB": {
		ICAO:      "KGRB",
		IATA:      "GRB",
		Name:      "Austin Straubel International Airport",
		City:      "Green Bay",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 695,
		Latitude:  44.4850997925,
		Longitude: -88.1296005249,
		Timezone:  "America/Chicago",
	},
	"KGRD": {
		ICAO:      "KGRD",
		IATA:      "GRD",
		Name:      "Greenwood County Airport",
		City:      "Greenwood",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 631,
		Latitude:  34.2486991882,
		Longitude: -82.1591033936,
		Timezone:  "America/New_York",
	},
	"KGRE": {
		ICAO:      "KGRE",
		IATA:      "GRE",
		Name:      "Greenville Airport",
		City:      "Greenville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 541,
		Latitude:  38.8362007141,
		Longitude: -89.37840271,
		Timezone:  "America/Chicago",
	},
	"KGRF": {
		ICAO:      "KGRF",
		IATA:      "GRF",
		Name:      "Gray Army Air Field",
		City:      "Fort Lewis/Tacoma",
		State:     "Washington",
		Country:   "US",
		Elevation: 300,
		Latitude:  47.07920074,
		Longitude: -122.5810013,
		Timezone:  "America/Los_Angeles",
	},
	"KGRI": {
		ICAO:      "KGRI",
		IATA:      "GRI",
		Name:      "Central Nebraska Regional Airport",
		City:      "Grand Island",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1847,
		Latitude:  40.9674987793,
		Longitude: -98.3096008301,
		Timezone:  "America/Chicago",
	},
	"KGRK": {
		ICAO:      "KGRK",
		IATA:      "GRK",
		Name:      "Robert Gray  Army Air Field Airport",
		City:      "Fort Hood/Killeen",
		State:     "Texas",
		Country:   "US",
		Elevation: 1015,
		Latitude:  31.067199707,
		Longitude: -97.8289031982,
		Timezone:  "America/Chicago",
	},
	"KGRN": {
		ICAO:      "KGRN",
		IATA:      "GRN",
		Name:      "Gordon Municipal Airport",
		City:      "Gordon",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3562,
		Latitude:  42.8059997559,
		Longitude: -102.1750030518,
		Timezone:  "America/Denver",
	},
	"KGRR": {
		ICAO:      "KGRR",
		IATA:      "GRR",
		Name:      "Gerald R. Ford International Airport",
		City:      "Grand Rapids",
		State:     "Michigan",
		Country:   "US",
		Elevation: 794,
		Latitude:  42.88079834,
		Longitude: -85.52279663,
		Timezone:  "America/Detroit",
	},
	"KGSB": {
		ICAO:      "KGSB",
		IATA:      "GSB",
		Name:      "Seymour Johnson Air Force Base",
		City:      "Goldsboro",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 109,
		Latitude:  35.33940125,
		Longitude: -77.96060181,
		Timezone:  "America/New_York",
	},
	"KGSH": {
		ICAO:      "KGSH",
		IATA:      "GSH",
		Name:      "Goshen Municipal Airport",
		City:      "Goshen",
		State:     "Indiana",
		Country:   "US",
		Elevation: 827,
		Latitude:  41.5264015198,
		Longitude: -85.7929000854,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KGSO": {
		ICAO:      "KGSO",
		IATA:      "GSO",
		Name:      "Piedmont Triad International Airport",
		City:      "Greensboro",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 925,
		Latitude:  36.0978012085,
		Longitude: -79.9373016357,
		Timezone:  "America/New_York",
	},
	"KGSP": {
		ICAO:      "KGSP",
		IATA:      "GSP",
		Name:      "Greenville Spartanburg International Airport",
		City:      "Greenville",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 964,
		Latitude:  34.8956985474,
		Longitude: -82.2189025879,
		Timezone:  "America/New_York",
	},
	"KGTB": {
		ICAO:      "KGTB",
		IATA:      "GTB",
		Name:      "Wheeler Sack Army Air Field",
		City:      "Fort Drum",
		State:     "New-York",
		Country:   "US",
		Elevation: 688,
		Latitude:  44.05559921,
		Longitude: -75.71949768,
		Timezone:  "America/New_York",
	},
	"KGTE": {
		ICAO:      "KGTE",
		Name:      "Quinn Field",
		City:      "Gothenburg",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2559,
		Latitude:  40.9263000488,
		Longitude: -100.1480026245,
		Timezone:  "America/Chicago",
	},
	"KGTF": {
		ICAO:      "KGTF",
		IATA:      "GTF",
		Name:      "Great Falls International Airport",
		City:      "Great Falls",
		State:     "Montana",
		Country:   "US",
		Elevation: 3680,
		Latitude:  47.48199844,
		Longitude: -111.3710022,
		Timezone:  "America/Denver",
	},
	"KGTG": {
		ICAO:      "KGTG",
		IATA:      "GTG",
		Name:      "Grantsburg Municipal Airport",
		City:      "Grantsburg",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 927,
		Latitude:  45.7980995178,
		Longitude: -92.6643981934,
		Timezone:  "America/Chicago",
	},
	"KGTR": {
		ICAO:      "KGTR",
		IATA:      "GTR",
		Name:      "Golden Triangle Regional Airport",
		City:      "Columbus/W Point/Starkville",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 264,
		Latitude:  33.4502983093,
		Longitude: -88.5914001465,
		Timezone:  "America/Chicago",
	},
	"KGTU": {
		ICAO:      "KGTU",
		Name:      "Georgetown Municipal Airport",
		City:      "Georgetown",
		State:     "Texas",
		Country:   "US",
		Elevation: 790,
		Latitude:  30.6788005829,
		Longitude: -97.679397583,
		Timezone:  "America/Chicago",
	},
	"KGUC": {
		ICAO:      "KGUC",
		IATA:      "GUC",
		Name:      "Gunnison Crested Butte Regional Airport",
		City:      "Gunnison",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7680,
		Latitude:  38.53390121,
		Longitude: -106.9329987,
		Timezone:  "America/Denver",
	},
	"KGUP": {
		ICAO:      "KGUP",
		IATA:      "GUP",
		Name:      "Gallup Municipal Airport",
		City:      "Gallup",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 6472,
		Latitude:  35.511100769,
		Longitude: -108.789001465,
		Timezone:  "America/Denver",
	},
	"KGUS": {
		ICAO:      "KGUS",
		IATA:      "GUS",
		Name:      "Grissom ARB Airport",
		City:      "Peru",
		State:     "Indiana",
		Country:   "US",
		Elevation: 812,
		Latitude:  40.6481018066,
		Longitude: -86.1520996094,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KGUY": {
		ICAO:      "KGUY",
		IATA:      "GUY",
		Name:      "Guymon Municipal Airport",
		City:      "Guymon",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 3123,
		Latitude:  36.6851005554,
		Longitude: -101.5080032349,
		Timezone:  "America/Chicago",
	},
	"KGVL": {
		ICAO:      "KGVL",
		IATA:      "GVL",
		Name:      "Lee Gilmer Memorial Airport",
		City:      "Gainesville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1276,
		Latitude:  34.27259827,
		Longitude: -83.8302002,
		Timezone:  "America/New_York",
	},
	"KGVQ": {
		ICAO:      "KGVQ",
		Name:      "Genesee County Airport",
		City:      "Batavia",
		State:     "New-York",
		Country:   "US",
		Elevation: 914,
		Latitude:  43.03170013,
		Longitude: -78.16760254,
		Timezone:  "America/New_York",
	},
	"KGVT": {
		ICAO:      "KGVT",
		IATA:      "GVT",
		Name:      "Majors Airport",
		City:      "Greenville",
		State:     "California",
		Country:   "US",
		Elevation: 535,
		Latitude:  33.0677986145,
		Longitude: -96.0652999878,
		Timezone:  "America/Chicago",
	},
	"KGWB": {
		ICAO:      "KGWB",
		Name:      "De Kalb County Airport",
		City:      "Auburn",
		State:     "Indiana",
		Country:   "US",
		Elevation: 880,
		Latitude:  41.3072013855,
		Longitude: -85.0643997192,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KGWO": {
		ICAO:      "KGWO",
		IATA:      "GWO",
		Name:      "Greenwood Leflore Airport",
		City:      "Greenwood",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 162,
		Latitude:  33.4943008423,
		Longitude: -90.0847015381,
		Timezone:  "America/Chicago",
	},
	"KGWR": {
		ICAO:      "KGWR",
		Name:      "Gwinner Roger Melroe Field",
		City:      "Gwinner",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1266,
		Latitude:  46.2187004089,
		Longitude: -97.6432037354,
		Timezone:  "America/Chicago",
	},
	"KGWS": {
		ICAO:      "KGWS",
		IATA:      "GWS",
		Name:      "Glenwood Springs Municipal Airport",
		City:      "Glenwood Springs",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5916,
		Latitude:  39.5083007813,
		Longitude: -107.3109970093,
		Timezone:  "America/Denver",
	},
	"KGWW": {
		ICAO:      "KGWW",
		Name:      "Goldsboro-Wayne Municipal Airport",
		City:      "Goldsboro",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 134,
		Latitude:  35.4606018066,
		Longitude: -77.9648971558,
		Timezone:  "America/New_York",
	},
	"KGXY": {
		ICAO:      "KGXY",
		IATA:      "GXY",
		Name:      "Greeley Weld County Airport",
		City:      "Greeley",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4697,
		Latitude:  40.4374008179,
		Longitude: -104.6330032349,
		Timezone:  "America/Denver",
	},
	"KGYB": {
		ICAO:      "KGYB",
		Name:      "Giddings Lee County Airport",
		City:      "Giddings",
		State:     "Texas",
		Country:   "US",
		Elevation: 485,
		Latitude:  30.1693000793,
		Longitude: -96.9800033569,
		Timezone:  "America/Chicago",
	},
	"KGYH": {
		ICAO:      "KGYH",
		IATA:      "GDC",
		Name:      "Donaldson Center Airport",
		City:      "Greenville",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 955,
		Latitude:  34.7583007813,
		Longitude: -82.3764038086,
		Timezone:  "America/New_York",
	},
	"KGYI": {
		ICAO:      "KGYI",
		IATA:      "PNX",
		Name:      "North Texas Rgnl/Perrin Field",
		City:      "Sherman/Denison",
		State:     "Texas",
		Country:   "US",
		Elevation: 749,
		Latitude:  33.714099884,
		Longitude: -96.6736984253,
		Timezone:  "America/Chicago",
	},
	"KGYL": {
		ICAO:      "KGYL",
		Name:      "Glencoe Municipal Airport",
		City:      "Glencoe",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 992,
		Latitude:  44.7560005188,
		Longitude: -94.081199646,
		Timezone:  "America/Chicago",
	},
	"KGYR": {
		ICAO:      "KGYR",
		IATA:      "GYR",
		Name:      "Phoenix Goodyear Airport",
		City:      "Goodyear",
		State:     "Arizona",
		Country:   "US",
		Elevation: 968,
		Latitude:  33.4225006104,
		Longitude: -112.3759994507,
		Timezone:  "America/Phoenix",
	},
	"KGYY": {
		ICAO:      "KGYY",
		IATA:      "GYY",
		Name:      "Gary Chicago International Airport",
		City:      "Gary",
		State:     "Indiana",
		Country:   "US",
		Elevation: 591,
		Latitude:  41.6162986755,
		Longitude: -87.4128036499,
		Timezone:  "America/Chicago",
	},
	"KGZH": {
		ICAO:      "KGZH",
		Name:      "Middleton Field",
		City:      "Evergreen",
		State:     "Alabama",
		Country:   "US",
		Elevation: 259,
		Latitude:  31.4158000946,
		Longitude: -87.0439987183,
		Timezone:  "America/Chicago",
	},
	"KGZS": {
		ICAO:      "KGZS",
		Name:      "Abernathy Field",
		City:      "Pulaski",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 685,
		Latitude:  35.1537017822,
		Longitude: -87.0568008423,
		Timezone:  "America/Chicago",
	},
	"KH04": {
		ICAO:      "KH04",
		Name:      "Vinita Municipal Airport",
		City:      "Vinita",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 696,
		Latitude:  36.6329994202,
		Longitude: -95.1513977051,
		Timezone:  "America/Chicago",
	},
	"KH05": {
		ICAO:      "KH05",
		Name:      "Wilburton Municipal Airport",
		City:      "Wilburton",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 670,
		Latitude:  34.9199981689,
		Longitude: -95.3939971924,
		Timezone:  "America/Chicago",
	},
	"KH17": {
		ICAO:      "KH17",
		Name:      "Buffalo Municipal Airport",
		City:      "Buffalo",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1154,
		Latitude:  37.6542015076,
		Longitude: -93.0876998901,
		Timezone:  "America/Chicago",
	},
	"KH19": {
		ICAO:      "KH19",
		Name:      "Bowling Green Municipal Airport",
		City:      "Bowling Green",
		State:     "Missouri",
		Country:   "US",
		Elevation: 886,
		Latitude:  39.3698997498,
		Longitude: -91.2192993164,
		Timezone:  "America/Chicago",
	},
	"KH21": {
		ICAO:      "KH21",
		Name:      "Camdenton Memorial Airport",
		City:      "Camdenton",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1062,
		Latitude:  37.9739990234,
		Longitude: -92.6912002563,
		Timezone:  "America/Chicago",
	},
	"KH34": {
		ICAO:      "KH34",
		Name:      "Huntsville Municipal Airport",
		City:      "Huntsville",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1749,
		Latitude:  36.0782012939,
		Longitude: -93.7547988892,
		Timezone:  "America/Chicago",
	},
	"KH35": {
		ICAO:      "KH35",
		Name:      "Clarksville Municipal Airport",
		City:      "Clarksville",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 481,
		Latitude:  35.4706993103,
		Longitude: -93.4272003174,
		Timezone:  "America/Chicago",
	},
	"KH37": {
		ICAO:      "KH37",
		Name:      "Herlong Airport",
		City:      "Herlong",
		State:     "California",
		Country:   "US",
		Elevation: 4055,
		Latitude:  40.1388015747,
		Longitude: -120.1790008545,
		Timezone:  "America/Los_Angeles",
	},
	"KH58": {
		ICAO:      "KH58",
		Name:      "Owen Field",
		City:      "Seymour",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1600,
		Latitude:  37.1847000122,
		Longitude: -92.7376022339,
		Timezone:  "America/Chicago",
	},
	"KH68": {
		ICAO:      "KH68",
		Name:      "Hefner Easley Airport",
		City:      "Wagoner",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 609,
		Latitude:  35.962600708,
		Longitude: -95.3419036865,
		Timezone:  "America/Chicago",
	},
	"KH70": {
		ICAO:      "KH70",
		Name:      "Stratford Field",
		City:      "Stratford",
		State:     "Texas",
		Country:   "US",
		Elevation: 3668,
		Latitude:  36.3456001282,
		Longitude: -102.0490036011,
		Timezone:  "America/Chicago",
	},
	"KH71": {
		ICAO:      "KH71",
		Name:      "Mid America Industrial Airport",
		City:      "Pryor",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 622,
		Latitude:  36.2253990173,
		Longitude: -95.3301010132,
		Timezone:  "America/Chicago",
	},
	"KH76": {
		ICAO:      "KH76",
		Name:      "Pawhuska Municipal Airport",
		City:      "Pawhuska",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1000,
		Latitude:  36.6716995239,
		Longitude: -96.4056015015,
		Timezone:  "America/Chicago",
	},
	"KH79": {
		ICAO:      "KH79",
		Name:      "Eldon Model Airpark",
		City:      "Eldon",
		State:     "Missouri",
		Country:   "US",
		Elevation: 909,
		Latitude:  38.3605995178,
		Longitude: -92.5716018677,
		Timezone:  "America/Chicago",
	},
	"KH88": {
		ICAO:      "KH88",
		Name:      "Fredericktown Regional Airport",
		City:      "Fredericktown",
		State:     "Missouri",
		Country:   "US",
		Elevation: 880,
		Latitude:  37.6058006287,
		Longitude: -90.2873001099,
		Timezone:  "America/Chicago",
	},
	"KH92": {
		ICAO:      "KH92",
		Name:      "Hominy Municipal Airport",
		City:      "Hominy",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 825,
		Latitude:  36.4426994324,
		Longitude: -96.3871994019,
		Timezone:  "America/Chicago",
	},
	"KH96": {
		ICAO:      "KH96",
		Name:      "Benton Municipal Airport",
		City:      "Benton",
		State:     "Illinois",
		Country:   "US",
		Elevation: 444,
		Latitude:  38.0066986084,
		Longitude: -88.9344024658,
		Timezone:  "America/Chicago",
	},
	"KHAB": {
		ICAO:      "KHAB",
		IATA:      "HAB",
		Name:      "Marion County Rankin Fite Airport",
		City:      "Hamilton",
		State:     "Alabama",
		Country:   "US",
		Elevation: 436,
		Latitude:  34.11759949,
		Longitude: -87.99819946,
		Timezone:  "America/Chicago",
	},
	"KHAE": {
		ICAO:      "KHAE",
		Name:      "Hannibal Regional Airport",
		City:      "Hannibal",
		State:     "Missouri",
		Country:   "US",
		Elevation: 769,
		Latitude:  39.72449875,
		Longitude: -91.4437027,
		Timezone:  "America/Chicago",
	},
	"KHAF": {
		ICAO:      "KHAF",
		IATA:      "HAF",
		Name:      "Half Moon Bay Airport",
		City:      "Half Moon Bay",
		State:     "California",
		Country:   "US",
		Elevation: 66,
		Latitude:  37.5134010315,
		Longitude: -122.5009994507,
		Timezone:  "America/Los_Angeles",
	},
	"KHAI": {
		ICAO:      "KHAI",
		IATA:      "HAI",
		Name:      "Three Rivers Municipal Dr Haines Airport",
		City:      "Three Rivers",
		State:     "Michigan",
		Country:   "US",
		Elevation: 824,
		Latitude:  41.9598007202,
		Longitude: -85.5933990479,
		Timezone:  "America/Detroit",
	},
	"KHAO": {
		ICAO:      "KHAO",
		IATA:      "HAO",
		Name:      "Butler Co Regional Airport",
		City:      "Hamilton",
		State:     "Ohio",
		Country:   "US",
		Elevation: 633,
		Latitude:  39.3638000488,
		Longitude: -84.5220031738,
		Timezone:  "America/New_York",
	},
	"KHBC": {
		ICAO:      "KHBC",
		Name:      "Mohall Municipal Airport",
		City:      "Mohall",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1649,
		Latitude:  48.7683982849,
		Longitude: -101.5370025635,
		Timezone:  "America/Chicago",
	},
	"KHBG": {
		ICAO:      "KHBG",
		IATA:      "HBG",
		Name:      "Hattiesburg Bobby L Chain Municipal Airport",
		City:      "Hattiesburg",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 151,
		Latitude:  31.26479912,
		Longitude: -89.25279999,
		Timezone:  "America/Chicago",
	},
	"KHBI": {
		ICAO:      "KHBI",
		Name:      "Asheboro Regional Airport",
		City:      "Asheboro",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 671,
		Latitude:  35.65449905,
		Longitude: -79.8946991,
		Timezone:  "America/New_York",
	},
	"KHBR": {
		ICAO:      "KHBR",
		IATA:      "HBR",
		Name:      "Hobart Regional Airport",
		City:      "Hobart",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1563,
		Latitude:  34.99129868,
		Longitude: -99.05139923,
		Timezone:  "America/Chicago",
	},
	"KHBV": {
		ICAO:      "KHBV",
		Name:      "Jim Hogg County Airport",
		City:      "Hebbronville",
		State:     "Texas",
		Country:   "US",
		Elevation: 663,
		Latitude:  27.3495998383,
		Longitude: -98.7369995117,
		Timezone:  "America/Chicago",
	},
	"KHBW": {
		ICAO:      "KHBW",
		Name:      "Joshua Sanford Field",
		City:      "Hillsboro",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 938,
		Latitude:  43.6566009521,
		Longitude: -90.3282012939,
		Timezone:  "America/Chicago",
	},
	"KHBZ": {
		ICAO:      "KHBZ",
		Name:      "Heber Springs Municipal Airport",
		City:      "Heber Springs",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 632,
		Latitude:  35.5116996765,
		Longitude: -92.0130004883,
		Timezone:  "America/Chicago",
	},
	"KHCD": {
		ICAO:      "KHCD",
		Name:      "Hutchinson Municipal Butler Field",
		City:      "Hutchinson",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1062,
		Latitude:  44.85990143,
		Longitude: -94.38249969,
		Timezone:  "America/Chicago",
	},
	"KHCO": {
		ICAO:      "KHCO",
		Name:      "Hallock Municipal Airport",
		City:      "Hallock",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 819,
		Latitude:  48.7527008057,
		Longitude: -96.9430007935,
		Timezone:  "America/Chicago",
	},
	"KHCR": {
		ICAO:      "KHCR",
		Name:      "Heber City Municipal Airport - Russ McDonald Field",
		City:      "Heber",
		State:     "Utah",
		Country:   "US",
		Elevation: 1718,
		Latitude:  40.4818056,
		Longitude: -111.4288056,
		Timezone:  "America/Denver",
	},
	"KHDC": {
		ICAO:      "KHDC",
		Name:      "Hammond Northshore Regional Airport",
		City:      "Hammond",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 47,
		Latitude:  30.52160072,
		Longitude: -90.41840363,
		Timezone:  "America/Chicago",
	},
	"KHDE": {
		ICAO:      "KHDE",
		IATA:      "HDE",
		Name:      "Brewster Field",
		City:      "Holdrege",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2313,
		Latitude:  40.45209885,
		Longitude: -99.33650208,
		Timezone:  "America/Chicago",
	},
	"KHDI": {
		ICAO:      "KHDI",
		Name:      "Hardwick Field",
		City:      "Cleveland",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 874,
		Latitude:  35.2201004028,
		Longitude: -84.8323974609,
		Timezone:  "America/New_York",
	},
	"KHDN": {
		ICAO:      "KHDN",
		IATA:      "HDN",
		Name:      "Yampa Valley Airport",
		City:      "Hayden",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6606,
		Latitude:  40.48120117,
		Longitude: -107.2180023,
		Timezone:  "America/Denver",
	},
	"KHDO": {
		ICAO:      "KHDO",
		Name:      "Hondo Municipal Airport",
		City:      "Hondo",
		State:     "Texas",
		Country:   "US",
		Elevation: 930,
		Latitude:  29.359500885,
		Longitude: -99.1766967773,
		Timezone:  "America/Chicago",
	},
	"KHEE": {
		ICAO:      "KHEE",
		IATA:      "HEE",
		Name:      "Thompson Robbins Airport",
		City:      "Helena/West Helena",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 242,
		Latitude:  34.576499939,
		Longitude: -90.6759033203,
		Timezone:  "America/Chicago",
	},
	"KHEF": {
		ICAO:      "KHEF",
		IATA:      "MNZ",
		Name:      "Manassas Regional Harry P. Davis field",
		City:      "Manassas",
		State:     "Virginia",
		Country:   "US",
		Elevation: 192,
		Latitude:  38.72140121,
		Longitude: -77.51540375,
		Timezone:  "America/New_York",
	},
	"KHEG": {
		ICAO:      "KHEG",
		Name:      "Herlong Airport",
		City:      "Jacksonville",
		State:     "Florida",
		Country:   "US",
		Elevation: 87,
		Latitude:  30.2777996063,
		Longitude: -81.8059005737,
		Timezone:  "America/New_York",
	},
	"KHEI": {
		ICAO:      "KHEI",
		Name:      "Hettinger Municipal Airport",
		City:      "Hettinger",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 2705,
		Latitude:  46.0149002075,
		Longitude: -102.65599823,
		Timezone:  "America/Denver",
	},
	"KHEQ": {
		ICAO:      "KHEQ",
		Name:      "Holyoke Airport",
		City:      "Holyoke",
		State:     "Colorado",
		Country:   "US",
		Elevation: 3730,
		Latitude:  40.5694007874,
		Longitude: -102.2730026245,
		Timezone:  "America/Denver",
	},
	"KHES": {
		ICAO:      "KHES",
		Name:      "Healdsburg Municipal Airport",
		City:      "Healdsburg",
		State:     "California",
		Country:   "US",
		Elevation: 278,
		Latitude:  38.6534996033,
		Longitude: -122.899002075,
		Timezone:  "America/Los_Angeles",
	},
	"KHEZ": {
		ICAO:      "KHEZ",
		IATA:      "HEZ",
		Name:      "Hardy Anders Field Natchez Adams County Airport",
		City:      "Natchez",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 272,
		Latitude:  31.6137008667,
		Longitude: -91.2973022461,
		Timezone:  "America/Chicago",
	},
	"KHFD": {
		ICAO:      "KHFD",
		IATA:      "HFD",
		Name:      "Hartford Brainard Airport",
		City:      "Hartford",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 18,
		Latitude:  41.7366981506,
		Longitude: -72.6493988037,
		Timezone:  "America/New_York",
	},
	"KHFF": {
		ICAO:      "KHFF",
		IATA:      "HFF",
		Name:      "Mackall Army Air Field",
		City:      "Camp Mackall",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 376,
		Latitude:  35.03649902,
		Longitude: -79.49749756,
		Timezone:  "America/New_York",
	},
	"KHFJ": {
		ICAO:      "KHFJ",
		Name:      "Monett Municipal Airport",
		City:      "Monett",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1314,
		Latitude:  36.90620041,
		Longitude: -94.01280212,
		Timezone:  "America/Chicago",
	},
	"KHFY": {
		ICAO:      "KHFY",
		Name:      "Greenwood Municipal Airport",
		City:      "Indianapolis",
		State:     "Indiana",
		Country:   "US",
		Elevation: 822,
		Latitude:  39.6283988953,
		Longitude: -86.0878982544,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KHGR": {
		ICAO:      "KHGR",
		IATA:      "HGR",
		Name:      "Hagerstown Regional Richard A Henson Field",
		City:      "Hagerstown",
		State:     "Maryland",
		Country:   "US",
		Elevation: 703,
		Latitude:  39.707901,
		Longitude: -77.72949982,
		Timezone:  "America/New_York",
	},
	"KHHF": {
		ICAO:      "KHHF",
		Name:      "Hemphill County Airport",
		City:      "Canadian",
		State:     "Texas",
		Country:   "US",
		Elevation: 2396,
		Latitude:  35.89530182,
		Longitude: -100.4039993,
		Timezone:  "America/Chicago",
	},
	"KHHG": {
		ICAO:      "KHHG",
		Name:      "Huntington Municipal Airport",
		City:      "Huntington",
		State:     "Indiana",
		Country:   "US",
		Elevation: 806,
		Latitude:  40.8529014587,
		Longitude: -85.4570999146,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KHHR": {
		ICAO:      "KHHR",
		IATA:      "HHR",
		Name:      "Jack Northrop Field Hawthorne Municipal Airport",
		City:      "Hawthorne",
		State:     "California",
		Country:   "US",
		Elevation: 66,
		Latitude:  33.9227981567,
		Longitude: -118.3349990845,
		Timezone:  "America/Los_Angeles",
	},
	"KHHW": {
		ICAO:      "KHHW",
		IATA:      "HUJ",
		Name:      "Stan Stamper Municipal Airport",
		City:      "Hugo",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 572,
		Latitude:  34.03480148,
		Longitude: -95.54190063,
		Timezone:  "America/Chicago",
	},
	"KHIB": {
		ICAO:      "KHIB",
		IATA:      "HIB",
		Name:      "Chisholm Hibbing Airport",
		City:      "Hibbing",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1354,
		Latitude:  47.38660049,
		Longitude: -92.83899689,
		Timezone:  "America/Chicago",
	},
	"KHIE": {
		ICAO:      "KHIE",
		IATA:      "HIE",
		Name:      "Mount Washington Regional Airport",
		City:      "Whitefield",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 1074,
		Latitude:  44.3675994873,
		Longitude: -71.5445022583,
		Timezone:  "America/New_York",
	},
	"KHIF": {
		ICAO:      "KHIF",
		IATA:      "HIF",
		Name:      "Hill Air Force Base",
		City:      "Ogden",
		State:     "Utah",
		Country:   "US",
		Elevation: 4789,
		Latitude:  41.1239013672,
		Longitude: -111.9729995728,
		Timezone:  "America/Denver",
	},
	"KHIG": {
		ICAO:      "KHIG",
		Name:      "Higginsville Industrial Municipal Airport",
		City:      "Higginsville",
		State:     "Missouri",
		Country:   "US",
		Elevation: 840,
		Latitude:  39.07289886,
		Longitude: -93.67749786,
		Timezone:  "America/Chicago",
	},
	"KHII": {
		ICAO:      "KHII",
		IATA:      "HII",
		Name:      "Lake Havasu City Airport",
		City:      "Lake Havasu City",
		State:     "Arizona",
		Country:   "US",
		Elevation: 783,
		Latitude:  34.57109833,
		Longitude: -114.3580017,
		Timezone:  "America/Los_Angeles",
	},
	"KHIO": {
		ICAO:      "KHIO",
		IATA:      "HIO",
		Name:      "Portland Hillsboro Airport",
		City:      "Portland",
		State:     "Oregon",
		Country:   "US",
		Elevation: 208,
		Latitude:  45.54040146,
		Longitude: -122.9499969,
		Timezone:  "America/Los_Angeles",
	},
	"KHJH": {
		ICAO:      "KHJH",
		Name:      "Hebron Municipal Airport",
		City:      "Hebron",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1466,
		Latitude:  40.1521987915,
		Longitude: -97.5869979858,
		Timezone:  "America/Chicago",
	},
	"KHJO": {
		ICAO:      "KHJO",
		Name:      "Hanford Municipal Airport",
		City:      "Hanford",
		State:     "California",
		Country:   "US",
		Elevation: 240,
		Latitude:  36.31669998,
		Longitude: -119.6279984,
		Timezone:  "America/Los_Angeles",
	},
	"KHKA": {
		ICAO:      "KHKA",
		IATA:      "HKA",
		Name:      "Blytheville Municipal Airport",
		City:      "Blytheville",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 255,
		Latitude:  35.9403991699,
		Longitude: -89.8308029175,
		Timezone:  "America/Chicago",
	},
	"KHKS": {
		ICAO:      "KHKS",
		IATA:      "HKS",
		Name:      "Hawkins Field",
		City:      "Jackson",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 341,
		Latitude:  32.33449936,
		Longitude: -90.22219849,
		Timezone:  "America/Chicago",
	},
	"KHKY": {
		ICAO:      "KHKY",
		IATA:      "HKY",
		Name:      "Hickory Regional Airport",
		City:      "Hickory",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 1190,
		Latitude:  35.74110031,
		Longitude: -81.38950348,
		Timezone:  "America/New_York",
	},
	"KHLB": {
		ICAO:      "KHLB",
		IATA:      "HLB",
		Name:      "Hillenbrand Industries Airport",
		City:      "Batesville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 973,
		Latitude:  39.3445014954,
		Longitude: -85.2583007813,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KHLC": {
		ICAO:      "KHLC",
		IATA:      "HLC",
		Name:      "Hill City Municipal Airport",
		City:      "Hill City",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2238,
		Latitude:  39.37879944,
		Longitude: -99.83149719,
		Timezone:  "America/Chicago",
	},
	"KHLG": {
		ICAO:      "KHLG",
		IATA:      "HLG",
		Name:      "Wheeling Ohio County Airport",
		City:      "Wheeling",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 1195,
		Latitude:  40.1749992371,
		Longitude: -80.6463012695,
		Timezone:  "America/New_York",
	},
	"KHLM": {
		ICAO:      "KHLM",
		IATA:      "HLM",
		Name:      "Park Township Airport",
		City:      "Holland",
		State:     "Michigan",
		Country:   "US",
		Elevation: 603,
		Latitude:  42.7958984375,
		Longitude: -86.1620025635,
		Timezone:  "America/Detroit",
	},
	"KHLN": {
		ICAO:      "KHLN",
		IATA:      "HLN",
		Name:      "Helena Regional Airport",
		City:      "Helena",
		State:     "Montana",
		Country:   "US",
		Elevation: 3877,
		Latitude:  46.6068000793,
		Longitude: -111.983001709,
		Timezone:  "America/Denver",
	},
	"KHLR": {
		ICAO:      "KHLR",
		IATA:      "HLR",
		Name:      "Hood Army Air Field",
		City:      "Fort Hood(Killeen)",
		State:     "Texas",
		Country:   "US",
		Elevation: 924,
		Latitude:  31.1387004852,
		Longitude: -97.7145004272,
		Timezone:  "America/Chicago",
	},
	"KHLX": {
		ICAO:      "KHLX",
		Name:      "Twin County Airport",
		City:      "Galax Hillsville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 2693,
		Latitude:  36.7661018372,
		Longitude: -80.823600769,
		Timezone:  "America/New_York",
	},
	"KHMN": {
		ICAO:      "KHMN",
		IATA:      "HMN",
		Name:      "Holloman Air Force Base",
		City:      "Alamogordo",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4093,
		Latitude:  32.8525009155,
		Longitude: -106.1070022583,
		Timezone:  "America/Denver",
	},
	"KHMT": {
		ICAO:      "KHMT",
		IATA:      "HMT",
		Name:      "Hemet Ryan Airport",
		City:      "Hemet",
		State:     "California",
		Country:   "US",
		Elevation: 1512,
		Latitude:  33.7340011597,
		Longitude: -117.0230026245,
		Timezone:  "America/Los_Angeles",
	},
	"KHMZ": {
		ICAO:      "KHMZ",
		Name:      "Bedford County Airport",
		City:      "Bedford",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1162,
		Latitude:  40.08530045,
		Longitude: -78.5121994,
		Timezone:  "America/New_York",
	},
	"KHNB": {
		ICAO:      "KHNB",
		IATA:      "HNB",
		Name:      "Huntingburg Airport",
		City:      "Huntingburg",
		State:     "Indiana",
		Country:   "US",
		Elevation: 529,
		Latitude:  38.2490005493,
		Longitude: -86.9536972046,
		Timezone:  "America/Indiana/Vincennes",
	},
	"KHND": {
		ICAO:      "KHND",
		IATA:      "HSH",
		Name:      "Henderson Executive Airport",
		City:      "Las Vegas",
		State:     "Nevada",
		Country:   "US",
		Elevation: 2492,
		Latitude:  35.9728012085,
		Longitude: -115.1340026855,
		Timezone:  "America/Los_Angeles",
	},
	"KHNR": {
		ICAO:      "KHNR",
		Name:      "Harlan Municipal Airport",
		City:      "Harlan",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1231,
		Latitude:  41.584400177,
		Longitude: -95.3395996094,
		Timezone:  "America/Chicago",
	},
	"KHNZ": {
		ICAO:      "KHNZ",
		Name:      "Henderson Oxford Airport",
		City:      "Oxford",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 526,
		Latitude:  36.36159897,
		Longitude: -78.52919769,
		Timezone:  "America/New_York",
	},
	"KHOB": {
		ICAO:      "KHOB",
		IATA:      "HOB",
		Name:      "Lea County Regional Airport",
		City:      "Hobbs",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 3661,
		Latitude:  32.6875,
		Longitude: -103.2170029,
		Timezone:  "America/Denver",
	},
	"KHOC": {
		ICAO:      "KHOC",
		Name:      "Highland County Airport",
		City:      "Hillsboro",
		State:     "Ohio",
		Country:   "US",
		Elevation: 977,
		Latitude:  39.1888008118,
		Longitude: -83.5388031006,
		Timezone:  "America/New_York",
	},
	"KHOE": {
		ICAO:      "KHOE",
		Name:      "Homerville Airport",
		City:      "Homerville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 186,
		Latitude:  31.0559005737,
		Longitude: -82.7741012573,
		Timezone:  "America/New_York",
	},
	"KHON": {
		ICAO:      "KHON",
		IATA:      "HON",
		Name:      "Huron Regional Airport",
		City:      "Huron",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1289,
		Latitude:  44.3852005005,
		Longitude: -98.2285003662,
		Timezone:  "America/Chicago",
	},
	"KHOP": {
		ICAO:      "KHOP",
		IATA:      "HOP",
		Name:      "Campbell AAF (Fort Campbell) Air Field",
		City:      "Fort Campbell/Hopkinsville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 573,
		Latitude:  36.668598175,
		Longitude: -87.4962005615,
		Timezone:  "America/Chicago",
	},
	"KHOT": {
		ICAO:      "KHOT",
		IATA:      "HOT",
		Name:      "Memorial Field",
		City:      "Hot Springs",
		State:     "Iowa",
		Country:   "US",
		Elevation: 540,
		Latitude:  34.4780006409,
		Longitude: -93.0961990356,
		Timezone:  "America/Chicago",
	},
	"KHOU": {
		ICAO:      "KHOU",
		IATA:      "HOU",
		Name:      "William P Hobby Airport",
		City:      "Houston",
		State:     "Texas",
		Country:   "US",
		Elevation: 46,
		Latitude:  29.64539909,
		Longitude: -95.27890015,
		Timezone:  "America/Chicago",
	},
	"KHPN": {
		ICAO:      "KHPN",
		IATA:      "HPN",
		Name:      "Westchester County Airport",
		City:      "White Plains",
		State:     "New-York",
		Country:   "US",
		Elevation: 439,
		Latitude:  41.0670013428,
		Longitude: -73.7076034546,
		Timezone:  "America/New_York",
	},
	"KHPT": {
		ICAO:      "KHPT",
		IATA:      "HPT",
		Name:      "Hampton Municipal Airport",
		City:      "Hampton",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1176,
		Latitude:  42.7237014771,
		Longitude: -93.2263031006,
		Timezone:  "America/Chicago",
	},
	"KHPY": {
		ICAO:      "KHPY",
		IATA:      "HPY",
		Name:      "Baytown Airport",
		City:      "Baytown",
		State:     "Texas",
		Country:   "US",
		Elevation: 34,
		Latitude:  29.7861003876,
		Longitude: -94.9526977539,
		Timezone:  "America/Chicago",
	},
	"KHQG": {
		ICAO:      "KHQG",
		Name:      "Hugoton Municipal Airport",
		City:      "Hugoton",
		State:     "Kansas",
		Country:   "US",
		Elevation: 3134,
		Latitude:  37.1631012,
		Longitude: -101.3710022,
		Timezone:  "America/Chicago",
	},
	"KHQM": {
		ICAO:      "KHQM",
		IATA:      "HQM",
		Name:      "Bowerman Airport",
		City:      "Hoquiam",
		State:     "Washington",
		Country:   "US",
		Elevation: 18,
		Latitude:  46.9711990356,
		Longitude: -123.93699646,
		Timezone:  "America/Los_Angeles",
	},
	"KHQU": {
		ICAO:      "KHQU",
		Name:      "Thomson Mcduffie County Airport",
		City:      "Thomson",
		State:     "Georgia",
		Country:   "US",
		Elevation: 501,
		Latitude:  33.52970123,
		Longitude: -82.51650238,
		Timezone:  "America/New_York",
	},
	"KHQZ": {
		ICAO:      "KHQZ",
		Name:      "Mesquite Metro Airport",
		City:      "Mesquite",
		State:     "Texas",
		Country:   "US",
		Elevation: 447,
		Latitude:  32.7470016479,
		Longitude: -96.5304031372,
		Timezone:  "America/Chicago",
	},
	"KHRI": {
		ICAO:      "KHRI",
		IATA:      "HES",
		Name:      "Hermiston Municipal Airport",
		City:      "Hermiston",
		State:     "Oregon",
		Country:   "US",
		Elevation: 644,
		Latitude:  45.8282012939,
		Longitude: -119.2590026855,
		Timezone:  "America/Los_Angeles",
	},
	"KHRJ": {
		ICAO:      "KHRJ",
		Name:      "Harnett Regional Jetport Airport",
		City:      "Erwin",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 202,
		Latitude:  35.37939835,
		Longitude: -78.73300171,
		Timezone:  "America/New_York",
	},
	"KHRL": {
		ICAO:      "KHRL",
		IATA:      "HRL",
		Name:      "Valley International Airport",
		City:      "Harlingen",
		State:     "Texas",
		Country:   "US",
		Elevation: 36,
		Latitude:  26.2285003662,
		Longitude: -97.6544036865,
		Timezone:  "America/Chicago",
	},
	"KHRO": {
		ICAO:      "KHRO",
		IATA:      "HRO",
		Name:      "Boone County Airport",
		City:      "Harrison",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1365,
		Latitude:  36.2615013123,
		Longitude: -93.1547012329,
		Timezone:  "America/Chicago",
	},
	"KHRT": {
		ICAO:      "KHRT",
		Name:      "Hurlburt Field",
		City:      "Mary Esther",
		State:     "Florida",
		Country:   "US",
		Elevation: 38,
		Latitude:  30.4277992249,
		Longitude: -86.6893005371,
		Timezone:  "America/Chicago",
	},
	"KHRU": {
		ICAO:      "KHRU",
		Name:      "Herington Regional Airport",
		City:      "Herington",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1480,
		Latitude:  38.6946983337,
		Longitude: -96.8079986572,
		Timezone:  "America/Chicago",
	},
	"KHRX": {
		ICAO:      "KHRX",
		Name:      "Hereford Municipal Airport",
		City:      "Hereford",
		State:     "Texas",
		Country:   "US",
		Elevation: 3788,
		Latitude:  34.85779953,
		Longitude: -102.3259964,
		Timezone:  "America/Chicago",
	},
	"KHSA": {
		ICAO:      "KHSA",
		Name:      "Stennis International Airport",
		City:      "Bay St Louis",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 23,
		Latitude:  30.3677997589,
		Longitude: -89.4545974731,
		Timezone:  "America/Chicago",
	},
	"KHSB": {
		ICAO:      "KHSB",
		IATA:      "HSB",
		Name:      "Harrisburg Raleigh Airport",
		City:      "Harrisburg",
		State:     "Illinois",
		Country:   "US",
		Elevation: 398,
		Latitude:  37.81129837,
		Longitude: -88.5503006,
		Timezone:  "America/Chicago",
	},
	"KHSD": {
		ICAO:      "KHSD",
		Name:      "Sundance Airpark",
		City:      "Oklahoma City",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1193,
		Latitude:  35.6017990112,
		Longitude: -97.706199646,
		Timezone:  "America/Chicago",
	},
	"KHSE": {
		ICAO:      "KHSE",
		IATA:      "HNC",
		Name:      "Billy Mitchell Airport",
		City:      "Hatteras",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 17,
		Latitude:  35.23279953,
		Longitude: -75.6177978516,
		Timezone:  "America/New_York",
	},
	"KHSG": {
		ICAO:      "KHSG",
		Name:      "Hot Springs County Airport",
		City:      "Thermopolis",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4892,
		Latitude:  43.7136111,
		Longitude: -108.3896944,
		Timezone:  "America/Denver",
	},
	"KHSI": {
		ICAO:      "KHSI",
		IATA:      "HSI",
		Name:      "Hastings Municipal Airport",
		City:      "Hastings",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1961,
		Latitude:  40.6053009033,
		Longitude: -98.4279022217,
		Timezone:  "America/Chicago",
	},
	"KHSP": {
		ICAO:      "KHSP",
		IATA:      "HSP",
		Name:      "Ingalls Field",
		City:      "Hot Springs",
		State:     "Virginia",
		Country:   "US",
		Elevation: 3793,
		Latitude:  37.95140076,
		Longitude: -79.83390045,
		Timezone:  "America/New_York",
	},
	"KHSR": {
		ICAO:      "KHSR",
		Name:      "Hot Springs Municipal Airport",
		City:      "Hot Springs",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 3150,
		Latitude:  43.36819839,
		Longitude: -103.3880005,
		Timezone:  "America/Denver",
	},
	"KHST": {
		ICAO:      "KHST",
		IATA:      "HST",
		Name:      "Homestead ARB Airport",
		City:      "Homestead",
		State:     "Florida",
		Country:   "US",
		Elevation: 5,
		Latitude:  25.48859978,
		Longitude: -80.38359833,
		Timezone:  "America/New_York",
	},
	"KHSV": {
		ICAO:      "KHSV",
		IATA:      "HSV",
		Name:      "Huntsville International Carl T Jones Field",
		City:      "Huntsville",
		State:     "Alabama",
		Country:   "US",
		Elevation: 629,
		Latitude:  34.6371994019,
		Longitude: -86.775100708,
		Timezone:  "America/Chicago",
	},
	"KHTH": {
		ICAO:      "KHTH",
		IATA:      "HTH",
		Name:      "Hawthorne Industrial Airport",
		City:      "Hawthorne",
		State:     "Nevada",
		Country:   "US",
		Elevation: 4215,
		Latitude:  38.5443992615,
		Longitude: -118.6340026855,
		Timezone:  "America/Los_Angeles",
	},
	"KHTL": {
		ICAO:      "KHTL",
		IATA:      "HTL",
		Name:      "Roscommon County - Blodgett Memorial Airport",
		City:      "Houghton Lake",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1150,
		Latitude:  44.35979843,
		Longitude: -84.6710968,
		Timezone:  "America/Detroit",
	},
	"KHTO": {
		ICAO:      "KHTO",
		IATA:      "HTO",
		Name:      "East Hampton Airport",
		City:      "East Hampton",
		State:     "New-York",
		Country:   "US",
		Elevation: 55,
		Latitude:  40.95959854,
		Longitude: -72.25180054,
		Timezone:  "America/New_York",
	},
	"KHTS": {
		ICAO:      "KHTS",
		IATA:      "HTS",
		Name:      "Tri-State/Milton J. Ferguson Field",
		City:      "Huntington",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 828,
		Latitude:  38.36669922,
		Longitude: -82.55799866,
		Timezone:  "America/New_York",
	},
	"KHTW": {
		ICAO:      "KHTW",
		IATA:      "HTW",
		Name:      "Lawrence County Airpark",
		City:      "Chesapeake/Huntington Wva",
		State:     "Ohio",
		Country:   "US",
		Elevation: 568,
		Latitude:  38.4193000793,
		Longitude: -82.4943008423,
		Timezone:  "America/New_York",
	},
	"KHUA": {
		ICAO:      "KHUA",
		IATA:      "HUA",
		Name:      "Redstone Army Air Field",
		City:      "Redstone Arsnl Huntsville",
		State:     "Alabama",
		Country:   "US",
		Elevation: 684,
		Latitude:  34.67869949,
		Longitude: -86.68479919,
		Timezone:  "America/Chicago",
	},
	"KHUF": {
		ICAO:      "KHUF",
		IATA:      "HUF",
		Name:      "Terre Haute International Hulman Field",
		City:      "Terre Haute",
		State:     "Indiana",
		Country:   "US",
		Elevation: 589,
		Latitude:  39.451499939,
		Longitude: -87.3076019287,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KHUL": {
		ICAO:      "KHUL",
		IATA:      "HUL",
		Name:      "Houlton International Airport",
		City:      "Houlton",
		State:     "Maine",
		Country:   "US",
		Elevation: 489,
		Latitude:  46.1231002808,
		Longitude: -67.792098999,
		Timezone:  "America/Moncton",
	},
	"KHUM": {
		ICAO:      "KHUM",
		IATA:      "HUM",
		Name:      "Houma Terrebonne Airport",
		City:      "Houma",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 10,
		Latitude:  29.5664997101,
		Longitude: -90.6604003906,
		Timezone:  "America/Chicago",
	},
	"KHUT": {
		ICAO:      "KHUT",
		IATA:      "HUT",
		Name:      "Hutchinson Municipal Airport",
		City:      "Hutchinson",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1543,
		Latitude:  38.0654983521,
		Longitude: -97.8606033325,
		Timezone:  "America/Chicago",
	},
	"KHVC": {
		ICAO:      "KHVC",
		Name:      "Hopkinsville Christian County Airport",
		City:      "Hopkinsville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 564,
		Latitude:  36.8569984436,
		Longitude: -87.4551010132,
		Timezone:  "America/Chicago",
	},
	"KHVE": {
		ICAO:      "KHVE",
		IATA:      "HVE",
		Name:      "Hanksville Airport",
		City:      "Hanksville",
		State:     "Utah",
		Country:   "US",
		Elevation: 4444,
		Latitude:  38.4179992676,
		Longitude: -110.7040023804,
		Timezone:  "America/Denver",
	},
	"KHVN": {
		ICAO:      "KHVN",
		IATA:      "HVN",
		Name:      "Tweed New Haven Airport",
		City:      "New Haven",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 12,
		Latitude:  41.26369858,
		Longitude: -72.88680267,
		Timezone:  "America/New_York",
	},
	"KHVR": {
		ICAO:      "KHVR",
		IATA:      "HVR",
		Name:      "Havre City County Airport",
		City:      "Havre",
		State:     "Montana",
		Country:   "US",
		Elevation: 2591,
		Latitude:  48.54299927,
		Longitude: -109.762001,
		Timezone:  "America/Denver",
	},
	"KHVS": {
		ICAO:      "KHVS",
		IATA:      "HVS",
		Name:      "Hartsville Regional Airport",
		City:      "Hartsville",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 364,
		Latitude:  34.4030990601,
		Longitude: -80.1192016602,
		Timezone:  "America/New_York",
	},
	"KHWD": {
		ICAO:      "KHWD",
		IATA:      "HWD",
		Name:      "Hayward Executive Airport",
		City:      "Hayward",
		State:     "California",
		Country:   "US",
		Elevation: 52,
		Latitude:  37.659198761,
		Longitude: -122.1220016479,
		Timezone:  "America/Los_Angeles",
	},
	"KHWO": {
		ICAO:      "KHWO",
		IATA:      "HWO",
		Name:      "North Perry Airport",
		City:      "Hollywood",
		State:     "Florida",
		Country:   "US",
		Elevation: 8,
		Latitude:  26.0011997223,
		Longitude: -80.2406997681,
		Timezone:  "America/New_York",
	},
	"KHWQ": {
		ICAO:      "KHWQ",
		Name:      "Wheatland County At Harlowton Airport",
		City:      "Harlowton",
		State:     "Montana",
		Country:   "US",
		Elevation: 4311,
		Latitude:  46.44860077,
		Longitude: -109.8529968,
		Timezone:  "America/Denver",
	},
	"KHWV": {
		ICAO:      "KHWV",
		IATA:      "WSH",
		Name:      "Brookhaven Airport",
		City:      "Shirley",
		State:     "New-York",
		Country:   "US",
		Elevation: 81,
		Latitude:  40.8218994141,
		Longitude: -72.8694000244,
		Timezone:  "America/New_York",
	},
	"KHWY": {
		ICAO:      "KHWY",
		Name:      "Warrenton Fauquier Airport",
		City:      "Warrenton",
		State:     "Virginia",
		Country:   "US",
		Elevation: 336,
		Latitude:  38.5862998962,
		Longitude: -77.7106018066,
		Timezone:  "America/New_York",
	},
	"KHXD": {
		ICAO:      "KHXD",
		IATA:      "HHH",
		Name:      "Hilton Head Airport",
		City:      "Hilton Head Island",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 19,
		Latitude:  32.2243995667,
		Longitude: -80.6975021362,
		Timezone:  "America/New_York",
	},
	"KHXF": {
		ICAO:      "KHXF",
		Name:      "Hartford Municipal Airport",
		City:      "Hartford",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1069,
		Latitude:  43.34930038,
		Longitude: -88.39109802,
		Timezone:  "America/Chicago",
	},
	"KHYA": {
		ICAO:      "KHYA",
		IATA:      "HYA",
		Name:      "Barnstable Municipal Boardman Polando Field",
		City:      "Hyannis",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 54,
		Latitude:  41.66930008,
		Longitude: -70.28040314,
		Timezone:  "America/New_York",
	},
	"KHYI": {
		ICAO:      "KHYI",
		Name:      "San Marcos Municipal Airport",
		City:      "San Marcos",
		State:     "Texas",
		Country:   "US",
		Elevation: 597,
		Latitude:  29.8927001953,
		Longitude: -97.8629989624,
		Timezone:  "America/Chicago",
	},
	"KHYR": {
		ICAO:      "KHYR",
		IATA:      "HYR",
		Name:      "Sawyer County Airport",
		City:      "Hayward",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1216,
		Latitude:  46.0251998901,
		Longitude: -91.4442977905,
		Timezone:  "America/Chicago",
	},
	"KHYS": {
		ICAO:      "KHYS",
		IATA:      "HYS",
		Name:      "Hays Regional Airport",
		City:      "Hays",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1999,
		Latitude:  38.84220123,
		Longitude: -99.27320099,
		Timezone:  "America/Chicago",
	},
	"KHYW": {
		ICAO:      "KHYW",
		Name:      "Conway Horry County Airport",
		City:      "Conway",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 35,
		Latitude:  33.82849884,
		Longitude: -79.12220001,
		Timezone:  "America/New_York",
	},
	"KHYX": {
		ICAO:      "KHYX",
		Name:      "Saginaw County H.W. Browne Airport",
		City:      "Saginaw",
		State:     "Michigan",
		Country:   "US",
		Elevation: 601,
		Latitude:  43.4333992,
		Longitude: -83.86229706,
		Timezone:  "America/Detroit",
	},
	"KHZD": {
		ICAO:      "KHZD",
		Name:      "Carroll County Airport",
		City:      "Huntingdon",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 497,
		Latitude:  36.08929825,
		Longitude: -88.46330261,
		Timezone:  "America/Chicago",
	},
	"KHZE": {
		ICAO:      "KHZE",
		Name:      "Mercer County Regional Airport",
		City:      "Hazen",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1814,
		Latitude:  47.28990173,
		Longitude: -101.5810013,
		Timezone:  "America/North_Dakota/Beulah",
	},
	"KHZL": {
		ICAO:      "KHZL",
		IATA:      "HZL",
		Name:      "Hazleton Municipal Airport",
		City:      "Hazleton",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1603,
		Latitude:  40.9868011475,
		Longitude: -75.9949035645,
		Timezone:  "America/New_York",
	},
	"KHZR": {
		ICAO:      "KHZR",
		Name:      "False River Regional Airport",
		City:      "New Roads",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 40,
		Latitude:  30.71829987,
		Longitude: -91.47869873,
		Timezone:  "America/Chicago",
	},
	"KHZX": {
		ICAO:      "KHZX",
		Name:      "Isedor Iverson Airport",
		City:      "Mc Gregor",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1228,
		Latitude:  46.6188011169,
		Longitude: -93.3097991943,
		Timezone:  "America/Chicago",
	},
	"KHZY": {
		ICAO:      "KHZY",
		IATA:      "JFN",
		Name:      "Ashtabula County Airport",
		City:      "Ashtabula",
		State:     "Ohio",
		Country:   "US",
		Elevation: 924,
		Latitude:  41.7779998779,
		Longitude: -80.6955032349,
		Timezone:  "America/New_York",
	},
	"KI06": {
		ICAO:      "KI06",
		Name:      "City of Tulia-Swisher County Municipal Airport",
		City:      "Tulia",
		State:     "Texas",
		Country:   "US",
		Elevation: 3503,
		Latitude:  34.56679916,
		Longitude: -101.7809982,
		Timezone:  "America/Chicago",
	},
	"KI10": {
		ICAO:      "KI10",
		Name:      "Noble County Airport",
		City:      "Caldwell",
		State:     "Ohio",
		Country:   "US",
		Elevation: 879,
		Latitude:  39.8009986877,
		Longitude: -81.5363006592,
		Timezone:  "America/New_York",
	},
	"KI12": {
		ICAO:      "KI12",
		Name:      "Sidney Municipal Airport",
		City:      "Sidney",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1044,
		Latitude:  40.2414016724,
		Longitude: -84.1509017944,
		Timezone:  "America/New_York",
	},
	"KI16": {
		ICAO:      "KI16",
		Name:      "Kee Field",
		City:      "Pineville",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 1783,
		Latitude:  37.6003990173,
		Longitude: -81.5593032837,
		Timezone:  "America/New_York",
	},
	"KI17": {
		ICAO:      "KI17",
		Name:      "Piqua Airport-Hartzell Field",
		City:      "Piqua",
		State:     "Ohio",
		Country:   "US",
		Elevation: 994,
		Latitude:  40.16469955,
		Longitude: -84.30840302,
		Timezone:  "America/New_York",
	},
	"KI18": {
		ICAO:      "KI18",
		Name:      "Jackson County Airport",
		City:      "Ravenswood",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 758,
		Latitude:  38.9297981262,
		Longitude: -81.8195037842,
		Timezone:  "America/New_York",
	},
	"KI22": {
		ICAO:      "KI22",
		Name:      "Randolph County Airport",
		City:      "Winchester",
		State:     "Indiana",
		Country:   "US",
		Elevation: 1119,
		Latitude:  40.1688995361,
		Longitude: -84.9256973267,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KI23": {
		ICAO:      "KI23",
		Name:      "Fayette County Airport",
		City:      "Washington Court House",
		State:     "Ohio",
		Country:   "US",
		Elevation: 980,
		Latitude:  39.570400238,
		Longitude: -83.420501709,
		Timezone:  "America/New_York",
	},
	"KI34": {
		ICAO:      "KI34",
		Name:      "Greensburg Municipal Airport",
		City:      "Greensburg",
		State:     "Indiana",
		Country:   "US",
		Elevation: 912,
		Latitude:  39.32690048,
		Longitude: -85.52249908,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KI35": {
		ICAO:      "KI35",
		Name:      "Tucker Guthrie Memorial Airport",
		City:      "Harlan",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 1551,
		Latitude:  36.8592987061,
		Longitude: -83.3584976196,
		Timezone:  "America/New_York",
	},
	"KI39": {
		ICAO:      "KI39",
		Name:      "Madison Airport",
		City:      "Richmond",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 1002,
		Latitude:  37.63079834,
		Longitude: -84.33229828,
		Timezone:  "America/New_York",
	},
	"KI40": {
		ICAO:      "KI40",
		Name:      "Richard Downing Airport",
		City:      "Coshocton",
		State:     "Ohio",
		Country:   "US",
		Elevation: 979,
		Latitude:  40.3092002869,
		Longitude: -81.8534011841,
		Timezone:  "America/New_York",
	},
	"KI43": {
		ICAO:      "KI43",
		Name:      "James A Rhodes Airport",
		City:      "Jackson",
		State:     "Ohio",
		Country:   "US",
		Elevation: 726,
		Latitude:  38.9813995361,
		Longitude: -82.5779037476,
		Timezone:  "America/New_York",
	},
	"KI50": {
		ICAO:      "KI50",
		Name:      "Stanton Airport",
		City:      "Stanton",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 651,
		Latitude:  37.8501014709,
		Longitude: -83.8458023071,
		Timezone:  "America/New_York",
	},
	"KI53": {
		ICAO:      "KI53",
		Name:      "Liberty Casey County Airport",
		City:      "Liberty",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 1040,
		Latitude:  37.3095016479,
		Longitude: -85.0587997437,
		Timezone:  "America/Chicago",
	},
	"KI54": {
		ICAO:      "KI54",
		Name:      "Mad River Inc Airport",
		City:      "Tremont City",
		State:     "Ohio",
		Country:   "US",
		Elevation: 958,
		Latitude:  40.0200996399,
		Longitude: -83.8284988403,
		Timezone:  "America/New_York",
	},
	"KI63": {
		ICAO:      "KI63",
		Name:      "Mount Sterling Municipal Airport",
		City:      "Mount Sterling",
		State:     "Illinois",
		Country:   "US",
		Elevation: 734,
		Latitude:  39.98749924,
		Longitude: -90.80419922,
		Timezone:  "America/Chicago",
	},
	"KI64": {
		ICAO:      "KI64",
		Name:      "Wakeman Airport",
		City:      "Wakeman",
		State:     "Ohio",
		Country:   "US",
		Elevation: 848,
		Latitude:  41.2930984497,
		Longitude: -82.3706970215,
		Timezone:  "America/New_York",
	},
	"KI66": {
		ICAO:      "KI66",
		Name:      "Clinton Field",
		City:      "Wilmington",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1033,
		Latitude:  39.5032997131,
		Longitude: -83.8628005981,
		Timezone:  "America/New_York",
	},
	"KI67": {
		ICAO:      "KI67",
		Name:      "Cincinnati West Airport",
		City:      "Harrison",
		State:     "Ohio",
		Country:   "US",
		Elevation: 584,
		Latitude:  39.2588996887,
		Longitude: -84.7742996216,
		Timezone:  "America/New_York",
	},
	"KI68": {
		ICAO:      "KI68",
		Name:      "Lebanon Warren County Airport",
		City:      "Lebanon",
		State:     "Ohio",
		Country:   "US",
		Elevation: 898,
		Latitude:  39.4622001648,
		Longitude: -84.2518005371,
		Timezone:  "America/New_York",
	},
	"KI69": {
		ICAO:      "KI69",
		Name:      "Clermont County Airport",
		City:      "Batavia",
		State:     "Ohio",
		Country:   "US",
		Elevation: 843,
		Latitude:  39.07839966,
		Longitude: -84.21019745,
		Timezone:  "America/New_York",
	},
	"KI71": {
		ICAO:      "KI71",
		Name:      "Morgan County Airport",
		City:      "Mc Connelsville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1000,
		Latitude:  39.6542015076,
		Longitude: -81.8031997681,
		Timezone:  "America/New_York",
	},
	"KI73": {
		ICAO:      "KI73",
		Name:      "Moraine Air Park",
		City:      "Dayton",
		State:     "Ohio",
		Country:   "US",
		Elevation: 720,
		Latitude:  39.682598114,
		Longitude: -84.2429962158,
		Timezone:  "America/New_York",
	},
	"KI74": {
		ICAO:      "KI74",
		Name:      "Grimes Field",
		City:      "Urbana",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1068,
		Latitude:  40.13259888,
		Longitude: -83.75340271,
		Timezone:  "America/New_York",
	},
	"KI75": {
		ICAO:      "KI75",
		Name:      "Osceola Municipal Airport",
		City:      "Osceola",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1110,
		Latitude:  41.0522003174,
		Longitude: -93.6896972656,
		Timezone:  "America/Chicago",
	},
	"KI76": {
		ICAO:      "KI76",
		Name:      "Peru Municipal Airport",
		City:      "Peru",
		State:     "Indiana",
		Country:   "US",
		Elevation: 779,
		Latitude:  40.7863006592,
		Longitude: -86.1464004517,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KI86": {
		ICAO:      "KI86",
		Name:      "Perry County Airport",
		City:      "New Lexington",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1051,
		Latitude:  39.6916007996,
		Longitude: -82.1977996826,
		Timezone:  "America/New_York",
	},
	"KI93": {
		ICAO:      "KI93",
		Name:      "Breckinridge County Airport",
		City:      "Hardinsburg",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 735,
		Latitude:  37.7850990295,
		Longitude: -86.4419021606,
		Timezone:  "America/Chicago",
	},
	"KI95": {
		ICAO:      "KI95",
		Name:      "Hardin County Airport",
		City:      "Kenton",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1024,
		Latitude:  40.6106987,
		Longitude: -83.6436004639,
		Timezone:  "America/New_York",
	},
	"KIAB": {
		ICAO:      "KIAB",
		IATA:      "IAB",
		Name:      "Mc Connell Air Force Base",
		City:      "Wichita",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1371,
		Latitude:  37.62189865,
		Longitude: -97.26820374,
		Timezone:  "America/Chicago",
	},
	"KIAD": {
		ICAO:      "KIAD",
		IATA:      "IAD",
		Name:      "Washington Dulles International Airport",
		City:      "Dulles",
		State:     "Virginia",
		Country:   "US",
		Elevation: 312,
		Latitude:  38.94449997,
		Longitude: -77.45580292,
		Timezone:  "America/New_York",
	},
	"KIAG": {
		ICAO:      "KIAG",
		IATA:      "IAG",
		Name:      "Niagara Falls International Airport",
		City:      "Niagara Falls",
		State:     "New-York",
		Country:   "US",
		Elevation: 589,
		Latitude:  43.1072998047,
		Longitude: -78.9461975098,
		Timezone:  "America/New_York",
	},
	"KIAH": {
		ICAO:      "KIAH",
		IATA:      "IAH",
		Name:      "George Bush Intercontinental Houston Airport",
		City:      "Houston",
		State:     "Texas",
		Country:   "US",
		Elevation: 97,
		Latitude:  29.9843997955,
		Longitude: -95.3414001465,
		Timezone:  "America/Chicago",
	},
	"KIBM": {
		ICAO:      "KIBM",
		Name:      "Kimball Municipal Robert E Arraj Field",
		City:      "Kimball",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 4926,
		Latitude:  41.1880989075,
		Longitude: -103.6770019531,
		Timezone:  "America/Denver",
	},
	"KICL": {
		ICAO:      "KICL",
		IATA:      "ICL",
		Name:      "Schenck Field",
		City:      "Clarinda",
		State:     "Iowa",
		Country:   "US",
		Elevation: 996,
		Latitude:  40.72180176,
		Longitude: -95.02639771,
		Timezone:  "America/Chicago",
	},
	"KICR": {
		ICAO:      "KICR",
		Name:      "Winner Regional Airport",
		City:      "Winner",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 2033,
		Latitude:  43.39020157,
		Longitude: -99.84210205,
		Timezone:  "America/Chicago",
	},
	"KICT": {
		ICAO:      "KICT",
		IATA:      "ICT",
		Name:      "Wichita Mid Continent Airport",
		City:      "Wichita",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1333,
		Latitude:  37.6498985291,
		Longitude: -97.4330978394,
		Timezone:  "America/Chicago",
	},
	"KIDA": {
		ICAO:      "KIDA",
		IATA:      "IDA",
		Name:      "Idaho Falls Regional Airport",
		City:      "Idaho Falls",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4744,
		Latitude:  43.5145988464,
		Longitude: -112.0709991455,
		Timezone:  "America/Boise",
	},
	"KIDG": {
		ICAO:      "KIDG",
		IATA:      "IDG",
		Name:      "Ida Grove Municipal Airport",
		City:      "Ida Grove",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1245,
		Latitude:  42.3325996399,
		Longitude: -95.4449005127,
		Timezone:  "America/Chicago",
	},
	"KIDI": {
		ICAO:      "KIDI",
		IATA:      "IDI",
		Name:      "Indiana County/Jimmy Stewart Fld/ Airport",
		City:      "Indiana",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1405,
		Latitude:  40.63219833,
		Longitude: -79.10549927,
		Timezone:  "America/New_York",
	},
	"KIDL": {
		ICAO:      "KIDL",
		Name:      "Indianola Municipal Airport",
		City:      "Indianola",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 126,
		Latitude:  33.4856987,
		Longitude: -90.6789016724,
		Timezone:  "America/Chicago",
	},
	"KIDP": {
		ICAO:      "KIDP",
		IATA:      "IDP",
		Name:      "Independence Municipal Airport",
		City:      "Independence",
		State:     "Kansas",
		Country:   "US",
		Elevation: 825,
		Latitude:  37.1584014893,
		Longitude: -95.7783966064,
		Timezone:  "America/Chicago",
	},
	"KIEN": {
		ICAO:      "KIEN",
		IATA:      "XPR",
		Name:      "Pine Ridge Airport",
		City:      "Pine Ridge",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 3333,
		Latitude:  43.0224990845,
		Longitude: -102.511001587,
		Timezone:  "America/Denver",
	},
	"KIER": {
		ICAO:      "KIER",
		Name:      "Natchitoches Regional Airport",
		City:      "Natchitoches",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 121,
		Latitude:  31.7357006073,
		Longitude: -93.0990982056,
		Timezone:  "America/Chicago",
	},
	"KIFA": {
		ICAO:      "KIFA",
		IATA:      "IFA",
		Name:      "Iowa Falls Municipal Airport",
		City:      "Iowa Falls",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1137,
		Latitude:  42.4707984924,
		Longitude: -93.2699966431,
		Timezone:  "America/Chicago",
	},
	"KIFP": {
		ICAO:      "KIFP",
		IATA:      "IFP",
		Name:      "Laughlin Bullhead International Airport",
		City:      "Bullhead City",
		State:     "Arizona",
		Country:   "US",
		Elevation: 701,
		Latitude:  35.15739822,
		Longitude: -114.5599976,
		Timezone:  "America/Phoenix",
	},
	"KIGM": {
		ICAO:      "KIGM",
		IATA:      "IGM",
		Name:      "Kingman Airport",
		City:      "Kingman",
		State:     "Arizona",
		Country:   "US",
		Elevation: 3449,
		Latitude:  35.2594985962,
		Longitude: -113.93800354,
		Timezone:  "America/Phoenix",
	},
	"KIGQ": {
		ICAO:      "KIGQ",
		Name:      "Lansing Municipal Airport",
		City:      "Chicago",
		State:     "Illinois",
		Country:   "US",
		Elevation: 620,
		Latitude:  41.5349006653,
		Longitude: -87.5295028687,
		Timezone:  "America/Chicago",
	},
	"KIGX": {
		ICAO:      "KIGX",
		Name:      "Horace Williams Airport",
		City:      "Chapel Hill",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 512,
		Latitude:  35.9350013733,
		Longitude: -79.06590271,
		Timezone:  "America/New_York",
	},
	"KIIB": {
		ICAO:      "KIIB",
		Name:      "Independence Municipal Airport",
		City:      "Independence",
		State:     "Iowa",
		Country:   "US",
		Elevation: 979,
		Latitude:  42.45360184,
		Longitude: -91.94760132,
		Timezone:  "America/Chicago",
	},
	"KIIY": {
		ICAO:      "KIIY",
		Name:      "Washington Wilkes County Airport",
		City:      "Washington",
		State:     "Georgia",
		Country:   "US",
		Elevation: 646,
		Latitude:  33.77939987,
		Longitude: -82.81580353,
		Timezone:  "America/New_York",
	},
	"KIJD": {
		ICAO:      "KIJD",
		Name:      "Windham Airport",
		City:      "Willimantic",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 247,
		Latitude:  41.7439994812,
		Longitude: -72.1802978516,
		Timezone:  "America/New_York",
	},
	"KIJX": {
		ICAO:      "KIJX",
		IATA:      "IJX",
		Name:      "Jacksonville Municipal Airport",
		City:      "Jacksonville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 624,
		Latitude:  39.7746009827,
		Longitude: -90.2382965088,
		Timezone:  "America/Chicago",
	},
	"KIKG": {
		ICAO:      "KIKG",
		Name:      "Kleberg County Airport",
		City:      "Kingsville",
		State:     "Texas",
		Country:   "US",
		Elevation: 130,
		Latitude:  27.55089951,
		Longitude: -98.03089905,
		Timezone:  "America/Chicago",
	},
	"KIKK": {
		ICAO:      "KIKK",
		IATA:      "IKK",
		Name:      "Greater Kankakee Airport",
		City:      "Kankakee",
		State:     "Illinois",
		Country:   "US",
		Elevation: 630,
		Latitude:  41.0713996887,
		Longitude: -87.8462982178,
		Timezone:  "America/Chicago",
	},
	"KIKV": {
		ICAO:      "KIKV",
		Name:      "Ankeny Regional Airport",
		City:      "Ankeny",
		State:     "Iowa",
		Country:   "US",
		Elevation: 910,
		Latitude:  41.6913986206,
		Longitude: -93.5663986206,
		Timezone:  "America/Chicago",
	},
	"KIKW": {
		ICAO:      "KIKW",
		Name:      "Jack Barstow Airport",
		City:      "Midland",
		State:     "Michigan",
		Country:   "US",
		Elevation: 635,
		Latitude:  43.66289902,
		Longitude: -84.26129913,
		Timezone:  "America/Detroit",
	},
	"KILE": {
		ICAO:      "KILE",
		IATA:      "ILE",
		Name:      "Skylark Field",
		City:      "Killeen",
		State:     "Texas",
		Country:   "US",
		Elevation: 848,
		Latitude:  31.0858001709,
		Longitude: -97.6865005493,
		Timezone:  "America/Chicago",
	},
	"KILG": {
		ICAO:      "KILG",
		IATA:      "ILG",
		Name:      "New Castle Airport",
		City:      "Wilmington",
		State:     "Delaware",
		Country:   "US",
		Elevation: 80,
		Latitude:  39.67869949,
		Longitude: -75.60649872,
		Timezone:  "America/New_York",
	},
	"KILM": {
		ICAO:      "KILM",
		IATA:      "ILM",
		Name:      "Wilmington International Airport",
		City:      "Wilmington",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 32,
		Latitude:  34.2705993652,
		Longitude: -77.9026031494,
		Timezone:  "America/New_York",
	},
	"KILN": {
		ICAO:      "KILN",
		IATA:      "ILN",
		Name:      "Airborne Airpark",
		City:      "Wilmington",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1077,
		Latitude:  39.427898407,
		Longitude: -83.792098999,
		Timezone:  "America/New_York",
	},
	"KIML": {
		ICAO:      "KIML",
		IATA:      "IML",
		Name:      "Imperial Municipal Airport",
		City:      "Imperial",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3275,
		Latitude:  40.50930023,
		Longitude: -101.6210022,
		Timezone:  "America/Denver",
	},
	"KIMM": {
		ICAO:      "KIMM",
		IATA:      "IMM",
		Name:      "Immokalee Regional Airport",
		City:      "Immokalee",
		State:     "Florida",
		Country:   "US",
		Elevation: 37,
		Latitude:  26.43320084,
		Longitude: -81.40100098,
		Timezone:  "America/New_York",
	},
	"KIMS": {
		ICAO:      "KIMS",
		IATA:      "MDN",
		Name:      "Madison Municipal Airport",
		City:      "Madison",
		State:     "Indiana",
		Country:   "US",
		Elevation: 819,
		Latitude:  38.75889969,
		Longitude: -85.46549988,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KIMT": {
		ICAO:      "KIMT",
		IATA:      "IMT",
		Name:      "Ford Airport",
		City:      "Iron Mountain Kingsford",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1182,
		Latitude:  45.8184013367,
		Longitude: -88.1145019531,
		Timezone:  "America/Chicago",
	},
	"KIND": {
		ICAO:      "KIND",
		IATA:      "IND",
		Name:      "Indianapolis International Airport",
		City:      "Indianapolis",
		State:     "Indiana",
		Country:   "US",
		Elevation: 797,
		Latitude:  39.717300415,
		Longitude: -86.2944030762,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KINJ": {
		ICAO:      "KINJ",
		Name:      "Hillsboro Municipal Airport",
		City:      "Hillsboro",
		State:     "Illinois",
		Country:   "US",
		Elevation: 686,
		Latitude:  32.08349991,
		Longitude: -97.09719849,
		Timezone:  "America/Chicago",
	},
	"KINK": {
		ICAO:      "KINK",
		IATA:      "INK",
		Name:      "Winkler County Airport",
		City:      "Wink",
		State:     "Texas",
		Country:   "US",
		Elevation: 2822,
		Latitude:  31.7796001434,
		Longitude: -103.2009963989,
		Timezone:  "America/Chicago",
	},
	"KINL": {
		ICAO:      "KINL",
		IATA:      "INL",
		Name:      "Falls International Airport",
		City:      "International Falls",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1185,
		Latitude:  48.5662002563,
		Longitude: -93.4030990601,
		Timezone:  "America/Chicago",
	},
	"KINS": {
		ICAO:      "KINS",
		IATA:      "INS",
		Name:      "Creech Air Force Base",
		City:      "Indian Springs",
		State:     "Nevada",
		Country:   "US",
		Elevation: 3133,
		Latitude:  36.5872001648,
		Longitude: -115.672996521,
		Timezone:  "America/Los_Angeles",
	},
	"KINT": {
		ICAO:      "KINT",
		IATA:      "INT",
		Name:      "Smith Reynolds Airport",
		City:      "Winston Salem",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 969,
		Latitude:  36.1337013245,
		Longitude: -80.2220001221,
		Timezone:  "America/New_York",
	},
	"KINW": {
		ICAO:      "KINW",
		IATA:      "INW",
		Name:      "Winslow Lindbergh Regional Airport",
		City:      "Winslow",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4941,
		Latitude:  35.021900177,
		Longitude: -110.7229995728,
		Timezone:  "America/Phoenix",
	},
	"KIOB": {
		ICAO:      "KIOB",
		Name:      "Mount Sterling Montgomery County Airport",
		City:      "Mount Sterling",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 1019,
		Latitude:  38.05810165,
		Longitude: -83.979599,
		Timezone:  "America/New_York",
	},
	"KIOW": {
		ICAO:      "KIOW",
		IATA:      "IOW",
		Name:      "Iowa City Municipal Airport",
		City:      "Iowa City",
		State:     "Iowa",
		Country:   "US",
		Elevation: 668,
		Latitude:  41.6391983032,
		Longitude: -91.5465011597,
		Timezone:  "America/Chicago",
	},
	"KIPJ": {
		ICAO:      "KIPJ",
		Name:      "Lincolnton Lincoln County Regional Airport",
		City:      "Lincolnton",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 877,
		Latitude:  35.48329926,
		Longitude: -81.16130066,
		Timezone:  "America/New_York",
	},
	"KIPL": {
		ICAO:      "KIPL",
		IATA:      "IPL",
		Name:      "Imperial County Airport",
		City:      "Imperial",
		State:     "California",
		Country:   "US",
		Elevation: -54,
		Latitude:  32.8342018127,
		Longitude: -115.57900238,
		Timezone:  "America/Los_Angeles",
	},
	"KIPT": {
		ICAO:      "KIPT",
		IATA:      "IPT",
		Name:      "Williamsport Regional Airport",
		City:      "Williamsport",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 529,
		Latitude:  41.2417984009,
		Longitude: -76.9210968018,
		Timezone:  "America/New_York",
	},
	"KIRK": {
		ICAO:      "KIRK",
		IATA:      "IRK",
		Name:      "Kirksville Regional Airport",
		City:      "Kirksville",
		State:     "Missouri",
		Country:   "US",
		Elevation: 966,
		Latitude:  40.09349823,
		Longitude: -92.5448989868,
		Timezone:  "America/Chicago",
	},
	"KIRS": {
		ICAO:      "KIRS",
		IATA:      "IRS",
		Name:      "Kirsch Municipal Airport",
		City:      "Sturgis",
		State:     "Michigan",
		Country:   "US",
		Elevation: 924,
		Latitude:  41.81330109,
		Longitude: -85.43900299,
		Timezone:  "America/Detroit",
	},
	"KISB": {
		ICAO:      "KISB",
		Name:      "Sibley Municipal Airport",
		City:      "Sibley",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1538,
		Latitude:  43.36949921,
		Longitude: -95.75980377,
		Timezone:  "America/Chicago",
	},
	"KISM": {
		ICAO:      "KISM",
		IATA:      "ISM",
		Name:      "Kissimmee Gateway Airport",
		City:      "Orlando",
		State:     "Florida",
		Country:   "US",
		Elevation: 82,
		Latitude:  28.2898006439,
		Longitude: -81.4371032715,
		Timezone:  "America/New_York",
	},
	"KISN": {
		ICAO:      "KISN",
		IATA:      "ISN",
		Name:      "Sloulin Field International Airport",
		City:      "Williston",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1982,
		Latitude:  48.177898407,
		Longitude: -103.641998291,
		Timezone:  "America/Chicago",
	},
	"KISO": {
		ICAO:      "KISO",
		IATA:      "ISO",
		Name:      "Kinston Regional Jetport At Stallings Field",
		City:      "Kinston",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 93,
		Latitude:  35.331401825,
		Longitude: -77.6088027954,
		Timezone:  "America/New_York",
	},
	"KISP": {
		ICAO:      "KISP",
		IATA:      "ISP",
		Name:      "Long Island Mac Arthur Airport",
		City:      "Islip",
		State:     "New-York",
		Country:   "US",
		Elevation: 99,
		Latitude:  40.79520035,
		Longitude: -73.10019684,
		Timezone:  "America/New_York",
	},
	"KISQ": {
		ICAO:      "KISQ",
		IATA:      "ISQ",
		Name:      "Schoolcraft County Airport",
		City:      "Manistique",
		State:     "Michigan",
		Country:   "US",
		Elevation: 684,
		Latitude:  45.97460175,
		Longitude: -86.17179871,
		Timezone:  "America/Detroit",
	},
	"KISW": {
		ICAO:      "KISW",
		IATA:      "ISW",
		Name:      "Alexander Field South Wood County Airport",
		City:      "Wisconsin Rapids",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1021,
		Latitude:  44.3602981567,
		Longitude: -89.8389968872,
		Timezone:  "America/Chicago",
	},
	"KITH": {
		ICAO:      "KITH",
		IATA:      "ITH",
		Name:      "Ithaca Tompkins Regional Airport",
		City:      "Ithaca",
		State:     "New-York",
		Country:   "US",
		Elevation: 1099,
		Latitude:  42.4910011292,
		Longitude: -76.4583969116,
		Timezone:  "America/New_York",
	},
	"KITR": {
		ICAO:      "KITR",
		Name:      "Kit Carson County Airport",
		City:      "Burlington",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4219,
		Latitude:  39.2425003052,
		Longitude: -102.2850036621,
		Timezone:  "America/Denver",
	},
	"KIUA": {
		ICAO:      "KIUA",
		Name:      "Canandaigua Airport",
		City:      "Canandaigua",
		State:     "New York",
		Country:   "US",
		Elevation: 814,
		Latitude:  42.9089019,
		Longitude: -77.3252263,
		Timezone:  "America/New_York",
	},
	"KIWA": {
		ICAO:      "KIWA",
		IATA:      "AZA",
		Name:      "Phoenix-Mesa-Gateway Airport",
		City:      "Phoenix",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1382,
		Latitude:  33.30780029,
		Longitude: -111.6549988,
		Timezone:  "America/Phoenix",
	},
	"KIWD": {
		ICAO:      "KIWD",
		IATA:      "IWD",
		Name:      "Gogebic Iron County Airport",
		City:      "Ironwood",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1230,
		Latitude:  46.5275001526,
		Longitude: -90.131401062,
		Timezone:  "America/Menominee",
	},
	"KIWH": {
		ICAO:      "KIWH",
		Name:      "Wabash Municipal Airport",
		City:      "Wabash",
		State:     "Indiana",
		Country:   "US",
		Elevation: 796,
		Latitude:  40.7620010376,
		Longitude: -85.7987976074,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KIWI": {
		ICAO:      "KIWI",
		IATA:      "ISS",
		Name:      "Wiscasset Airport",
		City:      "Wiscasset",
		State:     "Maine",
		Country:   "US",
		Elevation: 70,
		Latitude:  43.9613990784,
		Longitude: -69.712600708,
		Timezone:  "America/New_York",
	},
	"KIWS": {
		ICAO:      "KIWS",
		IATA:      "IWS",
		Name:      "West Houston Airport",
		City:      "Houston",
		State:     "Texas",
		Country:   "US",
		Elevation: 111,
		Latitude:  29.8181991577,
		Longitude: -95.6725997925,
		Timezone:  "America/Chicago",
	},
	"KIXA": {
		ICAO:      "KIXA",
		Name:      "Halifax-Northampton Regional Airport",
		City:      "Roanoke Rapids",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 145,
		Latitude:  36.32979965,
		Longitude: -77.63523102,
		Timezone:  "America/New_York",
	},
	"KIXD": {
		ICAO:      "KIXD",
		IATA:      "JCI",
		Name:      "New Century Aircenter Airport",
		City:      "Olathe",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1087,
		Latitude:  38.8308982849,
		Longitude: -94.890296936,
		Timezone:  "America/Chicago",
	},
	"KIYK": {
		ICAO:      "KIYK",
		IATA:      "IYK",
		Name:      "Inyokern Airport",
		City:      "Inyokern",
		State:     "California",
		Country:   "US",
		Elevation: 2457,
		Latitude:  35.65879822,
		Longitude: -117.8300018,
		Timezone:  "America/Los_Angeles",
	},
	"KIZA": {
		ICAO:      "KIZA",
		IATA:      "SQA",
		Name:      "Santa Ynez Airport",
		City:      "Santa Ynez",
		State:     "California",
		Country:   "US",
		Elevation: 674,
		Latitude:  34.60680008,
		Longitude: -120.0759964,
		Timezone:  "America/Los_Angeles",
	},
	"KIZG": {
		ICAO:      "KIZG",
		IATA:      "FRY",
		Name:      "Eastern Slopes Regional Airport",
		City:      "Fryeburg",
		State:     "Maine",
		Country:   "US",
		Elevation: 454,
		Latitude:  43.9911003113,
		Longitude: -70.9478988647,
		Timezone:  "America/New_York",
	},
	"KJAC": {
		ICAO:      "KJAC",
		IATA:      "JAC",
		Name:      "Jackson Hole Airport",
		City:      "Jackson",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 6451,
		Latitude:  43.6072998047,
		Longitude: -110.7379989624,
		Timezone:  "America/Denver",
	},
	"KJAN": {
		ICAO:      "KJAN",
		IATA:      "JAN",
		Name:      "Jackson Evers International Airport",
		City:      "Jackson",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 346,
		Latitude:  32.3111991882,
		Longitude: -90.0758972168,
		Timezone:  "America/Chicago",
	},
	"KJAQ": {
		ICAO:      "KJAQ",
		Name:      "Westover Field Amador County Airport",
		City:      "Jackson",
		State:     "California",
		Country:   "US",
		Elevation: 1690,
		Latitude:  38.3768005371,
		Longitude: -120.793998718,
		Timezone:  "America/Los_Angeles",
	},
	"KJAS": {
		ICAO:      "KJAS",
		IATA:      "JAS",
		Name:      "Jasper County-Bell Field",
		City:      "Jasper",
		State:     "Texas",
		Country:   "US",
		Elevation: 213,
		Latitude:  30.88570023,
		Longitude: -94.03489685,
		Timezone:  "America/Chicago",
	},
	"KJAU": {
		ICAO:      "KJAU",
		Name:      "Campbell County Airport",
		City:      "Jacksboro",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1180,
		Latitude:  36.3345985413,
		Longitude: -84.1623001099,
		Timezone:  "America/New_York",
	},
	"KJAX": {
		ICAO:      "KJAX",
		IATA:      "JAX",
		Name:      "Jacksonville International Airport",
		City:      "Jacksonville",
		State:     "Florida",
		Country:   "US",
		Elevation: 30,
		Latitude:  30.4941005707,
		Longitude: -81.6878967285,
		Timezone:  "America/New_York",
	},
	"KJBR": {
		ICAO:      "KJBR",
		IATA:      "JBR",
		Name:      "Jonesboro Municipal Airport",
		City:      "Jonesboro",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 262,
		Latitude:  35.8316993713,
		Longitude: -90.6464004517,
		Timezone:  "America/Chicago",
	},
	"KJCT": {
		ICAO:      "KJCT",
		IATA:      "JCT",
		Name:      "Kimble County Airport",
		City:      "Junction",
		State:     "Texas",
		Country:   "US",
		Elevation: 1749,
		Latitude:  30.5112991333,
		Longitude: -99.7634963989,
		Timezone:  "America/Chicago",
	},
	"KJDD": {
		ICAO:      "KJDD",
		Name:      "Wood County Airport",
		City:      "Mineola/Quitman",
		State:     "Texas",
		Country:   "US",
		Elevation: 434,
		Latitude:  32.7421989441,
		Longitude: -95.4964981079,
		Timezone:  "America/Chicago",
	},
	"KJDN": {
		ICAO:      "KJDN",
		IATA:      "JDN",
		Name:      "Jordan Airport",
		City:      "Jordan",
		State:     "Montana",
		Country:   "US",
		Elevation: 2662,
		Latitude:  47.3288002014,
		Longitude: -106.9530029297,
		Timezone:  "America/Denver",
	},
	"KJEF": {
		ICAO:      "KJEF",
		IATA:      "JEF",
		Name:      "Jefferson City Memorial Airport",
		City:      "Jefferson City",
		State:     "Missouri",
		Country:   "US",
		Elevation: 549,
		Latitude:  38.5912017822,
		Longitude: -92.1560974121,
		Timezone:  "America/Chicago",
	},
	"KJER": {
		ICAO:      "KJER",
		Name:      "Jerome County Airport",
		City:      "Jerome",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4053,
		Latitude:  42.7266998291,
		Longitude: -114.4570007324,
		Timezone:  "America/Boise",
	},
	"KJES": {
		ICAO:      "KJES",
		Name:      "Jesup Wayne County Airport",
		City:      "Jesup",
		State:     "Georgia",
		Country:   "US",
		Elevation: 107,
		Latitude:  31.55400085,
		Longitude: -81.88249969,
		Timezone:  "America/New_York",
	},
	"KJFK": {
		ICAO:      "KJFK",
		IATA:      "JFK",
		Name:      "John F Kennedy International Airport",
		City:      "New York",
		State:     "New-York",
		Country:   "US",
		Elevation: 13,
		Latitude:  40.63980103,
		Longitude: -73.77890015,
		Timezone:  "America/New_York",
	},
	"KJFX": {
		ICAO:      "KJFX",
		Name:      "Walker County Bevill Field",
		City:      "Jasper",
		State:     "Alabama",
		Country:   "US",
		Elevation: 482,
		Latitude:  33.90200043,
		Longitude: -87.31420135,
		Timezone:  "America/Chicago",
	},
	"KJFZ": {
		ICAO:      "KJFZ",
		Name:      "Tazewell County Airport",
		City:      "Richlands",
		State:     "Virginia",
		Country:   "US",
		Elevation: 2653,
		Latitude:  37.06370163,
		Longitude: -81.7983017,
		Timezone:  "America/New_York",
	},
	"KJGG": {
		ICAO:      "KJGG",
		Name:      "Williamsburg Jamestown Airport",
		City:      "Williamsburg",
		State:     "Virginia",
		Country:   "US",
		Elevation: 49,
		Latitude:  37.239200592,
		Longitude: -76.7161026001,
		Timezone:  "America/New_York",
	},
	"KJHW": {
		ICAO:      "KJHW",
		IATA:      "JHW",
		Name:      "Chautauqua County-Jamestown Airport",
		City:      "Jamestown",
		State:     "New-York",
		Country:   "US",
		Elevation: 1723,
		Latitude:  42.15340042,
		Longitude: -79.25800323,
		Timezone:  "America/New_York",
	},
	"KJKA": {
		ICAO:      "KJKA",
		IATA:      "GUF",
		Name:      "Jack Edwards Airport",
		City:      "Gulf Shores",
		State:     "Alabama",
		Country:   "US",
		Elevation: 17,
		Latitude:  30.29050064,
		Longitude: -87.67179871,
		Timezone:  "America/Chicago",
	},
	"KJKJ": {
		ICAO:      "KJKJ",
		Name:      "Moorhead Municipal Airport",
		City:      "Moorhead",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 918,
		Latitude:  46.83929825,
		Longitude: -96.66369629,
		Timezone:  "America/Chicago",
	},
	"KJKL": {
		ICAO:      "KJKL",
		Name:      "Julian Carroll Airport",
		City:      "Jackson",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 1381,
		Latitude:  37.5938987732,
		Longitude: -83.3172988892,
		Timezone:  "America/New_York",
	},
	"KJLN": {
		ICAO:      "KJLN",
		IATA:      "JLN",
		Name:      "Joplin Regional Airport",
		City:      "Joplin",
		State:     "Missouri",
		Country:   "US",
		Elevation: 981,
		Latitude:  37.1517982483,
		Longitude: -94.498298645,
		Timezone:  "America/Chicago",
	},
	"KJMR": {
		ICAO:      "KJMR",
		Name:      "Mora Municipal Airport",
		City:      "Mora",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1012,
		Latitude:  45.886100769,
		Longitude: -93.2717971802,
		Timezone:  "America/Chicago",
	},
	"KJMS": {
		ICAO:      "KJMS",
		IATA:      "JMS",
		Name:      "Jamestown Regional Airport",
		City:      "Jamestown",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1500,
		Latitude:  46.92969894,
		Longitude: -98.67819977,
		Timezone:  "America/Chicago",
	},
	"KJNX": {
		ICAO:      "KJNX",
		Name:      "Johnston County Airport",
		City:      "Smithfield",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 164,
		Latitude:  35.54090118,
		Longitude: -78.39029694,
		Timezone:  "America/New_York",
	},
	"KJOT": {
		ICAO:      "KJOT",
		IATA:      "JOT",
		Name:      "Joliet Regional Airport",
		City:      "Joliet",
		State:     "Illinois",
		Country:   "US",
		Elevation: 582,
		Latitude:  41.51779938,
		Longitude: -88.17549896,
		Timezone:  "America/Chicago",
	},
	"KJQF": {
		ICAO:      "KJQF",
		IATA:      "USA",
		Name:      "Concord Regional Airport",
		City:      "Concord",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 705,
		Latitude:  35.3877983093,
		Longitude: -80.7090988159,
		Timezone:  "America/New_York",
	},
	"KJSO": {
		ICAO:      "KJSO",
		IATA:      "JKV",
		Name:      "Cherokee County Airport",
		City:      "Jacksonville",
		State:     "Texas",
		Country:   "US",
		Elevation: 677,
		Latitude:  31.8693008423,
		Longitude: -95.2173995972,
		Timezone:  "America/Chicago",
	},
	"KJST": {
		ICAO:      "KJST",
		IATA:      "JST",
		Name:      "John Murtha Johnstown Cambria County Airport",
		City:      "Johnstown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 2284,
		Latitude:  40.3161010742,
		Longitude: -78.8339004517,
		Timezone:  "America/New_York",
	},
	"KJSV": {
		ICAO:      "KJSV",
		Name:      "Sallisaw Municipal Airport",
		City:      "Sallisaw",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 527,
		Latitude:  35.4382019,
		Longitude: -94.80280304,
		Timezone:  "America/Chicago",
	},
	"KJSY": {
		ICAO:      "KJSY",
		Name:      "Joseph State Airport",
		City:      "Joseph",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4121,
		Latitude:  45.3596000671,
		Longitude: -117.253997803,
		Timezone:  "America/Los_Angeles",
	},
	"KJVL": {
		ICAO:      "KJVL",
		IATA:      "JVL",
		Name:      "Southern Wisconsin Regional Airport",
		City:      "Janesville",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 808,
		Latitude:  42.620300293,
		Longitude: -89.0416030884,
		Timezone:  "America/Chicago",
	},
	"KJVW": {
		ICAO:      "KJVW",
		Name:      "John Bell Williams Airport",
		City:      "Raymond",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 247,
		Latitude:  32.30329895,
		Longitude: -90.40850067,
		Timezone:  "America/Chicago",
	},
	"KJVY": {
		ICAO:      "KJVY",
		Name:      "Clark Regional Airport",
		City:      "Jeffersonville",
		State:     "Indiana",
		Country:   "US",
		Elevation: 474,
		Latitude:  38.365398407,
		Longitude: -85.7381973267,
		Timezone:  "America/Kentucky/Louisville",
	},
	"KJWG": {
		ICAO:      "KJWG",
		Name:      "Watonga Regional Airport",
		City:      "Watonga",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1550,
		Latitude:  35.86470032,
		Longitude: -98.42079926,
		Timezone:  "America/Chicago",
	},
	"KJWN": {
		ICAO:      "KJWN",
		Name:      "John C Tune Airport",
		City:      "Nashville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 495,
		Latitude:  36.1823997498,
		Longitude: -86.8867034912,
		Timezone:  "America/Chicago",
	},
	"KJWY": {
		ICAO:      "KJWY",
		Name:      "Mid Way Regional Airport",
		City:      "Midlothian/Waxahachie",
		State:     "Texas",
		Country:   "US",
		Elevation: 713,
		Latitude:  32.4561004639,
		Longitude: -96.912399292,
		Timezone:  "America/Chicago",
	},
	"KJXI": {
		ICAO:      "KJXI",
		Name:      "Fox Stephens Field Gilmer Municipal Airport",
		City:      "Gilmer",
		State:     "Texas",
		Country:   "US",
		Elevation: 415,
		Latitude:  32.6990013123,
		Longitude: -94.9488983154,
		Timezone:  "America/Chicago",
	},
	"KJXN": {
		ICAO:      "KJXN",
		IATA:      "JXN",
		Name:      "Jackson County Reynolds Field",
		City:      "Jackson",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1001,
		Latitude:  42.2597999573,
		Longitude: -84.4593963623,
		Timezone:  "America/Detroit",
	},
	"KJYG": {
		ICAO:      "KJYG",
		Name:      "St James Municipal Airport",
		City:      "St James",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1067,
		Latitude:  43.9863014221,
		Longitude: -94.5578994751,
		Timezone:  "America/Chicago",
	},
	"KJYL": {
		ICAO:      "KJYL",
		Name:      "Plantation Airpark",
		City:      "Sylvania",
		State:     "Georgia",
		Country:   "US",
		Elevation: 188,
		Latitude:  32.6453018188,
		Longitude: -81.5970993042,
		Timezone:  "America/New_York",
	},
	"KJYM": {
		ICAO:      "KJYM",
		Name:      "Hillsdale Municipal Airport",
		City:      "Hillsdale",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1182,
		Latitude:  41.9212989807,
		Longitude: -84.5858001709,
		Timezone:  "America/Detroit",
	},
	"KJYO": {
		ICAO:      "KJYO",
		Name:      "Leesburg Executive Airport",
		City:      "Leesburg",
		State:     "Virginia",
		Country:   "US",
		Elevation: 389,
		Latitude:  39.077999115,
		Longitude: -77.5575027466,
		Timezone:  "America/New_York",
	},
	"KJYR": {
		ICAO:      "KJYR",
		Name:      "York Municipal Airport",
		City:      "York",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1670,
		Latitude:  40.8968009949,
		Longitude: -97.6228027344,
		Timezone:  "America/Chicago",
	},
	"KJZI": {
		ICAO:      "KJZI",
		Name:      "Charleston Executive Airport",
		City:      "Charleston",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 17,
		Latitude:  32.7009010315,
		Longitude: -80.0028991699,
		Timezone:  "America/New_York",
	},
	"KJZP": {
		ICAO:      "KJZP",
		Name:      "Pickens County Airport",
		City:      "Jasper",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1535,
		Latitude:  34.4533996582,
		Longitude: -84.4573974609,
		Timezone:  "America/New_York",
	},
	"KK02": {
		ICAO:      "KK02",
		Name:      "Perryville Municipal Airport",
		City:      "Perryville",
		State:     "Missouri",
		Country:   "US",
		Elevation: 372,
		Latitude:  37.86869812,
		Longitude: -89.86209869,
		Timezone:  "America/Chicago",
	},
	"KK06": {
		ICAO:      "KK06",
		Name:      "Greater Beardstown Airport",
		City:      "Beardstown",
		State:     "Illinois",
		Country:   "US",
		Elevation: 465,
		Latitude:  39.973400116,
		Longitude: -90.4037017822,
		Timezone:  "America/Chicago",
	},
	"KK09": {
		ICAO:      "KK09",
		Name:      "Piseco Airport",
		City:      "Piseco",
		State:     "New-York",
		Country:   "US",
		Elevation: 1703,
		Latitude:  43.4533996582,
		Longitude: -74.5176010132,
		Timezone:  "America/New_York",
	},
	"KK16": {
		ICAO:      "KK16",
		Name:      "Becks Grove Airport",
		City:      "Rome",
		State:     "New-York",
		Country:   "US",
		Elevation: 450,
		Latitude:  43.2583999634,
		Longitude: -75.6037979126,
		Timezone:  "America/New_York",
	},
	"KK19": {
		ICAO:      "KK19",
		Name:      "Albany Municipal Airport",
		City:      "Albany",
		State:     "Missouri",
		Country:   "US",
		Elevation: 886,
		Latitude:  40.2621994019,
		Longitude: -94.3389968872,
		Timezone:  "America/Chicago",
	},
	"KK20": {
		ICAO:      "KK20",
		Name:      "Wendell H Ford Airport",
		City:      "Hazard",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 1253,
		Latitude:  37.3877983093,
		Longitude: -83.2621002197,
		Timezone:  "America/New_York",
	},
	"KK22": {
		ICAO:      "KK22",
		Name:      "Big Sandy Regional Airport",
		City:      "Prestonsburg",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 1221,
		Latitude:  37.7509994507,
		Longitude: -82.6367034912,
		Timezone:  "America/New_York",
	},
	"KK24": {
		ICAO:      "KK24",
		Name:      "Russell County Airport",
		City:      "Jamestown",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 1011,
		Latitude:  37.0097007751,
		Longitude: -85.1026992798,
		Timezone:  "America/Chicago",
	},
	"KK32": {
		ICAO:      "KK32",
		Name:      "Riverside Airport",
		City:      "Wichita",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1335,
		Latitude:  37.7481002808,
		Longitude: -97.4067001343,
		Timezone:  "America/Chicago",
	},
	"KK33": {
		ICAO:      "KK33",
		Name:      "Salem Memorial Airport",
		City:      "Salem",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1241,
		Latitude:  37.6152000427,
		Longitude: -91.6044006348,
		Timezone:  "America/Chicago",
	},
	"KK36": {
		ICAO:      "KK36",
		Name:      "Onawa Municipal Airport",
		City:      "Onawa",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1047,
		Latitude:  42.0043983459,
		Longitude: -96.1035995483,
		Timezone:  "America/Chicago",
	},
	"KK38": {
		ICAO:      "KK38",
		Name:      "Washington County Memorial Airport",
		City:      "Washington",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1435,
		Latitude:  39.73350143,
		Longitude: -97.04769897,
		Timezone:  "America/Chicago",
	},
	"KK39": {
		ICAO:      "KK39",
		Name:      "St Clair Regional Airport",
		City:      "St Clair",
		State:     "Missouri",
		Country:   "US",
		Elevation: 656,
		Latitude:  38.3759002686,
		Longitude: -90.970703125,
		Timezone:  "America/Chicago",
	},
	"KK44": {
		ICAO:      "KK44",
		Name:      "Beaver Municipal Airport",
		City:      "Beaver",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 2491,
		Latitude:  36.7989006042,
		Longitude: -100.5299987793,
		Timezone:  "America/Chicago",
	},
	"KK49": {
		ICAO:      "KK49",
		Name:      "Municipal Airport",
		City:      "Texhoma",
		State:     "Texas",
		Country:   "US",
		Elevation: 3462,
		Latitude:  36.5055999756,
		Longitude: -101.8140029907,
		Timezone:  "America/Chicago",
	},
	"KK51": {
		ICAO:      "KK51",
		Name:      "Medicine Lodge Airport",
		City:      "Medicine Lodge",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1543,
		Latitude:  37.2625007629,
		Longitude: -98.5462036133,
		Timezone:  "America/Chicago",
	},
	"KK52": {
		ICAO:      "KK52",
		Name:      "Capt. Ben Smith Airfield - Monroe City Airport",
		City:      "Monroe City",
		State:     "Missouri",
		Country:   "US",
		Elevation: 737,
		Latitude:  39.63439941,
		Longitude: -91.72699738,
		Timezone:  "America/Chicago",
	},
	"KK57": {
		ICAO:      "KK57",
		Name:      "Gould Peterson Municipal Airport",
		City:      "Tarkio",
		State:     "Missouri",
		Country:   "US",
		Elevation: 913,
		Latitude:  40.44580078,
		Longitude: -95.3628006,
		Timezone:  "America/Chicago",
	},
	"KK58": {
		ICAO:      "KK58",
		Name:      "Harold Krier Field",
		City:      "Ashland",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1951,
		Latitude:  37.1666984558,
		Longitude: -99.775100708,
		Timezone:  "America/Chicago",
	},
	"KK59": {
		ICAO:      "KK59",
		Name:      "Amelia Earhart Airport",
		City:      "Atchison",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1073,
		Latitude:  39.5704994202,
		Longitude: -95.1802978516,
		Timezone:  "America/Chicago",
	},
	"KK61": {
		ICAO:      "KK61",
		Name:      "Moritz Memorial Airport",
		City:      "Beloit",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1416,
		Latitude:  39.4711990356,
		Longitude: -98.1287994385,
		Timezone:  "America/Chicago",
	},
	"KK62": {
		ICAO:      "KK62",
		Name:      "Gene Snyder Airport",
		City:      "Falmouth",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 899,
		Latitude:  38.70420074,
		Longitude: -84.39160156,
		Timezone:  "America/New_York",
	},
	"KK74": {
		ICAO:      "KK74",
		Name:      "Hamry Field",
		City:      "Kindred",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 947,
		Latitude:  46.6487999,
		Longitude: -96.99859619,
		Timezone:  "America/Chicago",
	},
	"KK75": {
		ICAO:      "KK75",
		Name:      "Osborne Municipal Airport",
		City:      "Osborne",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1565,
		Latitude:  39.4291992188,
		Longitude: -98.6794967651,
		Timezone:  "America/Chicago",
	},
	"KK77": {
		ICAO:      "KK77",
		Name:      "Freedom Municipal Airport",
		City:      "Freedom",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1517,
		Latitude:  36.7584991455,
		Longitude: -99.1019973755,
		Timezone:  "America/Chicago",
	},
	"KK78": {
		ICAO:      "KK78",
		Name:      "Abilene Municipal Airport",
		City:      "Abilene",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1152,
		Latitude:  38.9040985107,
		Longitude: -97.2359008789,
		Timezone:  "America/Chicago",
	},
	"KK79": {
		ICAO:      "KK79",
		Name:      "Jetmore Municipal Airport",
		City:      "Jetmore",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2466,
		Latitude:  37.984500885,
		Longitude: -99.8943023682,
		Timezone:  "America/Chicago",
	},
	"KK81": {
		ICAO:      "KK81",
		Name:      "Miami County Airport",
		City:      "Paola",
		State:     "Kansas",
		Country:   "US",
		Elevation: 940,
		Latitude:  38.540599823,
		Longitude: -94.920501709,
		Timezone:  "America/Chicago",
	},
	"KK82": {
		ICAO:      "KK82",
		Name:      "Smith Center Municipal Airport",
		City:      "Smith Center",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1799,
		Latitude:  39.761100769,
		Longitude: -98.7934036255,
		Timezone:  "America/Chicago",
	},
	"KK83": {
		ICAO:      "KK83",
		Name:      "Sabetha Municipal Airport",
		City:      "Sabetha",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1330,
		Latitude:  39.9042015076,
		Longitude: -95.7794036865,
		Timezone:  "America/Chicago",
	},
	"KK88": {
		ICAO:      "KK88",
		Name:      "Allen County Airport",
		City:      "Iola",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1015,
		Latitude:  37.870098114,
		Longitude: -95.3863983154,
		Timezone:  "America/Chicago",
	},
	"KK89": {
		ICAO:      "KK89",
		Name:      "Macon Fower Memorial Airport",
		City:      "Macon",
		State:     "Missouri",
		Country:   "US",
		Elevation: 874,
		Latitude:  39.7286987305,
		Longitude: -92.4645004272,
		Timezone:  "America/Chicago",
	},
	"KK94": {
		ICAO:      "KK94",
		Name:      "Rush County Airport",
		City:      "La Crosse",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2070,
		Latitude:  38.5483016968,
		Longitude: -99.2886962891,
		Timezone:  "America/Chicago",
	},
	"KKIC": {
		ICAO:      "KKIC",
		IATA:      "KIC",
		Name:      "Mesa Del Rey Airport",
		City:      "King City",
		State:     "California",
		Country:   "US",
		Elevation: 370,
		Latitude:  36.2280006409,
		Longitude: -121.1220016479,
		Timezone:  "America/Los_Angeles",
	},
	"KKLS": {
		ICAO:      "KKLS",
		IATA:      "KLS",
		Name:      "Southwest Washington Regional Airport",
		City:      "Kelso",
		State:     "Washington",
		Country:   "US",
		Elevation: 20,
		Latitude:  46.1180000305,
		Longitude: -122.898002625,
		Timezone:  "America/Los_Angeles",
	},
	"KKNB": {
		ICAO:      "KKNB",
		IATA:      "KNB",
		Name:      "Kanab Municipal Airport",
		City:      "Kanab",
		State:     "Utah",
		Country:   "US",
		Elevation: 4868,
		Latitude:  37.011100769,
		Longitude: -112.53099823,
		Timezone:  "America/Denver",
	},
	"KL00": {
		ICAO:      "KL00",
		Name:      "Rosamond Skypark Airport",
		City:      "Rosamond",
		State:     "California",
		Country:   "US",
		Elevation: 2415,
		Latitude:  34.8708000183,
		Longitude: -118.2089996338,
		Timezone:  "America/Los_Angeles",
	},
	"KL04": {
		ICAO:      "KL04",
		Name:      "Holtville Airport",
		City:      "Holtville",
		State:     "California",
		Country:   "US",
		Elevation: 59,
		Latitude:  32.8403015137,
		Longitude: -115.266998291,
		Timezone:  "America/Los_Angeles",
	},
	"KL05": {
		ICAO:      "KL05",
		Name:      "Kern Valley Airport",
		City:      "Kernville",
		State:     "California",
		Country:   "US",
		Elevation: 2614,
		Latitude:  35.7282981873,
		Longitude: -118.4199981689,
		Timezone:  "America/Los_Angeles",
	},
	"KL06": {
		ICAO:      "KL06",
		Name:      "Furnace Creek Airport",
		City:      "Death Valley National Park",
		State:     "California",
		Country:   "US",
		Elevation: -210,
		Latitude:  36.4637985229,
		Longitude: -116.8809967041,
		Timezone:  "America/Los_Angeles",
	},
	"KL08": {
		ICAO:      "KL08",
		IATA:      "BXS",
		Name:      "Borrego Valley Airport",
		City:      "Borrego Springs",
		State:     "California",
		Country:   "US",
		Elevation: 520,
		Latitude:  33.2589988708,
		Longitude: -116.320999146,
		Timezone:  "America/Los_Angeles",
	},
	"KL09": {
		ICAO:      "KL09",
		Name:      "Stovepipe Wells Airport",
		City:      "Death Valley National Park",
		State:     "California",
		Country:   "US",
		Elevation: 25,
		Latitude:  36.6040992737,
		Longitude: -117.158996582,
		Timezone:  "America/Los_Angeles",
	},
	"KL17": {
		ICAO:      "KL17",
		Name:      "Taft Kern County Airport",
		City:      "Taft",
		State:     "California",
		Country:   "US",
		Elevation: 875,
		Latitude:  35.1411018372,
		Longitude: -119.4410018921,
		Timezone:  "America/Los_Angeles",
	},
	"KL19": {
		ICAO:      "KL19",
		Name:      "Wasco Kern County Airport",
		City:      "Wasco",
		State:     "California",
		Country:   "US",
		Elevation: 313,
		Latitude:  35.6197013855,
		Longitude: -119.3539962769,
		Timezone:  "America/Los_Angeles",
	},
	"KL22": {
		ICAO:      "KL22",
		Name:      "Yucca Valley Airport",
		City:      "Yucca Valley",
		State:     "California",
		Country:   "US",
		Elevation: 3224,
		Latitude:  34.1277999878,
		Longitude: -116.4079971313,
		Timezone:  "America/Los_Angeles",
	},
	"KL23": {
		ICAO:      "KL23",
		Name:      "Pahute Mesa Airstrip",
		City:      "Mercury",
		State:     "Nevada",
		Country:   "US",
		Elevation: 5068,
		Latitude:  37.1026992798,
		Longitude: -116.31300354,
		Timezone:  "America/Los_Angeles",
	},
	"KL26": {
		ICAO:      "KL26",
		Name:      "Hesperia Airport",
		City:      "Hesperia",
		State:     "California",
		Country:   "US",
		Elevation: 3390,
		Latitude:  34.3772010803,
		Longitude: -117.3160018921,
		Timezone:  "America/Los_Angeles",
	},
	"KL31": {
		ICAO:      "KL31",
		Name:      "St. Tammany Regional Airport",
		City:      "Covington",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 39,
		Latitude:  30.44510078,
		Longitude: -89.98889923,
		Timezone:  "America/Chicago",
	},
	"KL32": {
		ICAO:      "KL32",
		Name:      "Jonesville Airport",
		City:      "Jonesville",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 56,
		Latitude:  31.6252994537,
		Longitude: -91.8292007446,
		Timezone:  "America/Chicago",
	},
	"KL33": {
		ICAO:      "KL33",
		Name:      "Tensas Parish Airport",
		City:      "St Joseph",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 74,
		Latitude:  31.9974002838,
		Longitude: -91.2472991943,
		Timezone:  "America/Chicago",
	},
	"KL35": {
		ICAO:      "KL35",
		Name:      "Big Bear City Airport",
		City:      "Big Bear City",
		State:     "California",
		Country:   "US",
		Elevation: 6752,
		Latitude:  34.2638015747,
		Longitude: -116.8560028076,
		Timezone:  "America/Los_Angeles",
	},
	"KL38": {
		ICAO:      "KL38",
		Name:      "Louisiana Regional Airport",
		City:      "Gonzales",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 14,
		Latitude:  30.17270088,
		Longitude: -90.94059753,
		Timezone:  "America/Chicago",
	},
	"KL39": {
		ICAO:      "KL39",
		Name:      "Leesville Airport",
		City:      "Leesville",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 282,
		Latitude:  31.16860008,
		Longitude: -93.34249878,
		Timezone:  "America/Chicago",
	},
	"KL41": {
		ICAO:      "KL41",
		Name:      "Marble Canyon Airport",
		City:      "Marble Canyon",
		State:     "Arizona",
		Country:   "US",
		Elevation: 3603,
		Latitude:  36.8125,
		Longitude: -111.6470031738,
		Timezone:  "America/Denver",
	},
	"KL45": {
		ICAO:      "KL45",
		Name:      "Bakersfield Municipal Airport",
		City:      "Bakersfield",
		State:     "California",
		Country:   "US",
		Elevation: 378,
		Latitude:  35.32479858,
		Longitude: -118.9960022,
		Timezone:  "America/Los_Angeles",
	},
	"KL47": {
		ICAO:      "KL47",
		Name:      "Olla Airport",
		City:      "Olla",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 130,
		Latitude:  31.8953990936,
		Longitude: -92.2179031372,
		Timezone:  "America/Chicago",
	},
	"KL52": {
		ICAO:      "KL52",
		Name:      "Oceano County Airport",
		City:      "Oceano",
		State:     "California",
		Country:   "US",
		Elevation: 14,
		Latitude:  35.1015292,
		Longitude: -120.6223153,
		Timezone:  "America/Los_Angeles",
	},
	"KL62": {
		ICAO:      "KL62",
		Name:      "Elk Hills Buttonwillow Airport",
		City:      "Buttonwillow",
		State:     "California",
		Country:   "US",
		Elevation: 326,
		Latitude:  35.3526992798,
		Longitude: -119.4789962769,
		Timezone:  "America/Los_Angeles",
	},
	"KL64": {
		ICAO:      "KL64",
		Name:      "Desert Center Airport",
		City:      "Desert Center",
		State:     "California",
		Country:   "US",
		Elevation: 559,
		Latitude:  33.7486000061,
		Longitude: -115.3229980469,
		Timezone:  "America/Los_Angeles",
	},
	"KL65": {
		ICAO:      "KL65",
		Name:      "Perris Valley Airport",
		City:      "Perris",
		State:     "California",
		Country:   "US",
		Elevation: 1413,
		Latitude:  33.7608985901,
		Longitude: -117.2180023193,
		Timezone:  "America/Los_Angeles",
	},
	"KL66": {
		ICAO:      "KL66",
		Name:      "Pollock Municipal Airport",
		City:      "Pollock",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 203,
		Latitude:  31.47750092,
		Longitude: -92.46109772,
		Timezone:  "America/Chicago",
	},
	"KL67": {
		ICAO:      "KL67",
		Name:      "Rialto Municipal Miro Field",
		City:      "Rialto",
		State:     "California",
		Country:   "US",
		Elevation: 1455,
		Latitude:  34.1292991638,
		Longitude: -117.4020004272,
		Timezone:  "America/Los_Angeles",
	},
	"KL70": {
		ICAO:      "KL70",
		Name:      "Agua Dulce Airpark",
		City:      "Agua Dulce",
		State:     "California",
		Country:   "US",
		Elevation: 2660,
		Latitude:  34.5041999817,
		Longitude: -118.31300354,
		Timezone:  "America/Los_Angeles",
	},
	"KL71": {
		ICAO:      "KL71",
		Name:      "California City Municipal Airport",
		City:      "California City",
		State:     "California",
		Country:   "US",
		Elevation: 2454,
		Latitude:  35.1511993408,
		Longitude: -118.016998291,
		Timezone:  "America/Los_Angeles",
	},
	"KL72": {
		ICAO:      "KL72",
		Name:      "Trona Airport",
		City:      "Trona",
		State:     "California",
		Country:   "US",
		Elevation: 1716,
		Latitude:  35.8125,
		Longitude: -117.327003479,
		Timezone:  "America/Los_Angeles",
	},
	"KL73": {
		ICAO:      "KL73",
		Name:      "Poso Kern County Airport",
		City:      "Famoso",
		State:     "California",
		Country:   "US",
		Elevation: 635,
		Latitude:  35.5965995789,
		Longitude: -119.1279983521,
		Timezone:  "America/Los_Angeles",
	},
	"KL77": {
		ICAO:      "KL77",
		Name:      "Chiriaco Summit Airport",
		City:      "Chiriaco Summit",
		State:     "California",
		Country:   "US",
		Elevation: 1713,
		Latitude:  33.6652984619,
		Longitude: -115.7109985352,
		Timezone:  "America/Los_Angeles",
	},
	"KL83": {
		ICAO:      "KL83",
		Name:      "Thibodaux Municipal Airport",
		City:      "Thibodaux",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 9,
		Latitude:  29.747800827,
		Longitude: -90.832901001,
		Timezone:  "America/Chicago",
	},
	"KL84": {
		ICAO:      "KL84",
		Name:      "Lost Hills Kern County Airport",
		City:      "Lost Hills",
		State:     "California",
		Country:   "US",
		Elevation: 274,
		Latitude:  35.6236000061,
		Longitude: -119.6859970093,
		Timezone:  "America/Los_Angeles",
	},
	"KL88": {
		ICAO:      "KL88",
		Name:      "New Cuyama Airport",
		City:      "New Cuyama",
		State:     "California",
		Country:   "US",
		Elevation: 2203,
		Latitude:  34.9375,
		Longitude: -119.68800354,
		Timezone:  "America/Los_Angeles",
	},
	"KL94": {
		ICAO:      "KL94",
		Name:      "Mountain Valley Airport",
		City:      "Tehachapi",
		State:     "California",
		Country:   "US",
		Elevation: 4220,
		Latitude:  35.1011009216,
		Longitude: -118.422996521,
		Timezone:  "America/Los_Angeles",
	},
	"KLAA": {
		ICAO:      "KLAA",
		IATA:      "LAA",
		Name:      "Lamar Municipal Airport",
		City:      "Lamar",
		State:     "Colorado",
		Country:   "US",
		Elevation: 3706,
		Latitude:  38.0696983337,
		Longitude: -102.68800354,
		Timezone:  "America/Denver",
	},
	"KLAF": {
		ICAO:      "KLAF",
		IATA:      "LAF",
		Name:      "Purdue University Airport",
		City:      "Lafayette",
		State:     "Indiana",
		Country:   "US",
		Elevation: 606,
		Latitude:  40.4123001099,
		Longitude: -86.9368972778,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KLAL": {
		ICAO:      "KLAL",
		IATA:      "LAL",
		Name:      "Lakeland Linder Regional Airport",
		City:      "Lakeland",
		State:     "Florida",
		Country:   "US",
		Elevation: 142,
		Latitude:  27.988899231,
		Longitude: -82.0186004639,
		Timezone:  "America/New_York",
	},
	"KLAM": {
		ICAO:      "KLAM",
		IATA:      "LAM",
		Name:      "Los Alamos Airport",
		City:      "Los Alamos",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 7171,
		Latitude:  35.8797988892,
		Longitude: -106.2689971924,
		Timezone:  "America/Denver",
	},
	"KLAN": {
		ICAO:      "KLAN",
		IATA:      "LAN",
		Name:      "Capital City Airport",
		City:      "Lansing",
		State:     "Michigan",
		Country:   "US",
		Elevation: 861,
		Latitude:  42.7787017822,
		Longitude: -84.5874023438,
		Timezone:  "America/Detroit",
	},
	"KLAR": {
		ICAO:      "KLAR",
		IATA:      "LAR",
		Name:      "Laramie Regional Airport",
		City:      "Laramie",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 7284,
		Latitude:  41.3120994568,
		Longitude: -105.6750030518,
		Timezone:  "America/Denver",
	},
	"KLAS": {
		ICAO:      "KLAS",
		IATA:      "LAS",
		Name:      "Harry Reid International Airport",
		City:      "Las Vegas",
		State:     "Nevada",
		Country:   "US",
		Elevation: 2181,
		Latitude:  36.08010101,
		Longitude: -115.1520004,
		Timezone:  "America/Los_Angeles",
	},
	"KLAW": {
		ICAO:      "KLAW",
		IATA:      "LAW",
		Name:      "Lawton Fort Sill Regional Airport",
		City:      "Lawton",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1110,
		Latitude:  34.5676994324,
		Longitude: -98.4166030884,
		Timezone:  "America/Chicago",
	},
	"KLAX": {
		ICAO:      "KLAX",
		IATA:      "LAX",
		Name:      "Los Angeles International Airport",
		City:      "Los Angeles",
		State:     "California",
		Country:   "US",
		Elevation: 125,
		Latitude:  33.94250107,
		Longitude: -118.4079971,
		Timezone:  "America/Los_Angeles",
	},
	"KLBB": {
		ICAO:      "KLBB",
		IATA:      "LBB",
		Name:      "Lubbock Preston Smith International Airport",
		City:      "Lubbock",
		State:     "Texas",
		Country:   "US",
		Elevation: 3282,
		Latitude:  33.6636009216,
		Longitude: -101.8229980469,
		Timezone:  "America/Chicago",
	},
	"KLBE": {
		ICAO:      "KLBE",
		IATA:      "LBE",
		Name:      "Arnold Palmer Regional Airport",
		City:      "Latrobe",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1199,
		Latitude:  40.27590179,
		Longitude: -79.40480042,
		Timezone:  "America/New_York",
	},
	"KLBF": {
		ICAO:      "KLBF",
		IATA:      "LBF",
		Name:      "North Platte Regional Airport Lee Bird Field",
		City:      "North Platte",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2777,
		Latitude:  41.12620163,
		Longitude: -100.6839981,
		Timezone:  "America/Chicago",
	},
	"KLBL": {
		ICAO:      "KLBL",
		IATA:      "LBL",
		Name:      "Liberal Mid-America Regional Airport",
		City:      "Liberal",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2885,
		Latitude:  37.0442009,
		Longitude: -100.9599991,
		Timezone:  "America/Chicago",
	},
	"KLBO": {
		ICAO:      "KLBO",
		Name:      "Floyd W. Jones Lebanon Airport",
		City:      "Lebanon",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1321,
		Latitude:  37.64830017,
		Longitude: -92.65239716,
		Timezone:  "America/Chicago",
	},
	"KLBR": {
		ICAO:      "KLBR",
		Name:      "Clarksville Red River City-J D Trissell Field",
		City:      "Clarksville",
		State:     "Texas",
		Country:   "US",
		Elevation: 440,
		Latitude:  33.59320068,
		Longitude: -95.06359863,
		Timezone:  "America/Chicago",
	},
	"KLBT": {
		ICAO:      "KLBT",
		IATA:      "LBT",
		Name:      "Lumberton Municipal Airport",
		City:      "Lumberton",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 126,
		Latitude:  34.6099014282,
		Longitude: -79.0594024658,
		Timezone:  "America/New_York",
	},
	"KLBX": {
		ICAO:      "KLBX",
		IATA:      "LJN",
		Name:      "Brazoria County Airport",
		City:      "Angleton/Lake Jackson",
		State:     "Texas",
		Country:   "US",
		Elevation: 25,
		Latitude:  29.1086006165,
		Longitude: -95.462097168,
		Timezone:  "America/Chicago",
	},
	"KLCG": {
		ICAO:      "KLCG",
		Name:      "Wayne Municipal Airport",
		City:      "Wayne",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1431,
		Latitude:  42.2419013977,
		Longitude: -96.9813995361,
		Timezone:  "America/Chicago",
	},
	"KLCH": {
		ICAO:      "KLCH",
		IATA:      "LCH",
		Name:      "Lake Charles Regional Airport",
		City:      "Lake Charles",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 15,
		Latitude:  30.1261005402,
		Longitude: -93.2232971191,
		Timezone:  "America/Chicago",
	},
	"KLCI": {
		ICAO:      "KLCI",
		IATA:      "LCI",
		Name:      "Laconia Municipal Airport",
		City:      "Laconia",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 545,
		Latitude:  43.5727005005,
		Longitude: -71.4188995361,
		Timezone:  "America/New_York",
	},
	"KLCK": {
		ICAO:      "KLCK",
		IATA:      "LCK",
		Name:      "Rickenbacker International Airport",
		City:      "Columbus",
		State:     "Ohio",
		Country:   "US",
		Elevation: 744,
		Latitude:  39.8138008118,
		Longitude: -82.9278030396,
		Timezone:  "America/New_York",
	},
	"KLCQ": {
		ICAO:      "KLCQ",
		IATA:      "LCQ",
		Name:      "Lake City Municipal Airport",
		City:      "Lake City",
		State:     "Florida",
		Country:   "US",
		Elevation: 201,
		Latitude:  30.1819992065,
		Longitude: -82.5768966675,
		Timezone:  "America/New_York",
	},
	"KLDJ": {
		ICAO:      "KLDJ",
		IATA:      "LDJ",
		Name:      "Linden Airport",
		City:      "Linden",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 23,
		Latitude:  40.617401123,
		Longitude: -74.2445983887,
		Timezone:  "America/New_York",
	},
	"KLDM": {
		ICAO:      "KLDM",
		IATA:      "LDM",
		Name:      "Mason County Airport",
		City:      "Ludington",
		State:     "Michigan",
		Country:   "US",
		Elevation: 646,
		Latitude:  43.96250153,
		Longitude: -86.40789795,
		Timezone:  "America/Detroit",
	},
	"KLEB": {
		ICAO:      "KLEB",
		IATA:      "LEB",
		Name:      "Lebanon Municipal Airport",
		City:      "Lebanon",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 603,
		Latitude:  43.6260986328,
		Longitude: -72.3041992188,
		Timezone:  "America/New_York",
	},
	"KLEE": {
		ICAO:      "KLEE",
		IATA:      "LEE",
		Name:      "Leesburg International Airport",
		City:      "Leesburg",
		State:     "Florida",
		Country:   "US",
		Elevation: 76,
		Latitude:  28.82309914,
		Longitude: -81.80870056,
		Timezone:  "America/New_York",
	},
	"KLEM": {
		ICAO:      "KLEM",
		IATA:      "LEM",
		Name:      "Lemmon Municipal Airport",
		City:      "Lemmon",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 2571,
		Latitude:  45.9187011719,
		Longitude: -102.1060028076,
		Timezone:  "America/Denver",
	},
	"KLEW": {
		ICAO:      "KLEW",
		IATA:      "LEW",
		Name:      "Auburn Lewiston Municipal Airport",
		City:      "Auburn/Lewiston",
		State:     "Maine",
		Country:   "US",
		Elevation: 288,
		Latitude:  44.048500061,
		Longitude: -70.2835006714,
		Timezone:  "America/New_York",
	},
	"KLEX": {
		ICAO:      "KLEX",
		IATA:      "LEX",
		Name:      "Blue Grass Airport",
		City:      "Lexington",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 979,
		Latitude:  38.0364990234,
		Longitude: -84.6059036255,
		Timezone:  "America/New_York",
	},
	"KLFI": {
		ICAO:      "KLFI",
		IATA:      "LFI",
		Name:      "Langley Air Force Base",
		City:      "Hampton",
		State:     "Virginia",
		Country:   "US",
		Elevation: 11,
		Latitude:  37.082901001,
		Longitude: -76.360496521,
		Timezone:  "America/New_York",
	},
	"KLFK": {
		ICAO:      "KLFK",
		IATA:      "LFK",
		Name:      "Angelina County Airport",
		City:      "Lufkin",
		State:     "Texas",
		Country:   "US",
		Elevation: 296,
		Latitude:  31.2339992523,
		Longitude: -94.75,
		Timezone:  "America/Chicago",
	},
	"KLFT": {
		ICAO:      "KLFT",
		IATA:      "LFT",
		Name:      "Lafayette Regional Airport",
		City:      "Lafayette",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 42,
		Latitude:  30.20529938,
		Longitude: -91.98760223,
		Timezone:  "America/Chicago",
	},
	"KLGA": {
		ICAO:      "KLGA",
		IATA:      "LGA",
		Name:      "La Guardia Airport",
		City:      "New York",
		State:     "New-York",
		Country:   "US",
		Elevation: 21,
		Latitude:  40.77719879,
		Longitude: -73.87259674,
		Timezone:  "America/New_York",
	},
	"KLGB": {
		ICAO:      "KLGB",
		IATA:      "LGB",
		Name:      "Long Beach /Daugherty Field/ Airport",
		City:      "Long Beach",
		State:     "California",
		Country:   "US",
		Elevation: 60,
		Latitude:  33.81769943,
		Longitude: -118.1520004,
		Timezone:  "America/Los_Angeles",
	},
	"KLGC": {
		ICAO:      "KLGC",
		IATA:      "LGC",
		Name:      "Lagrange Callaway Airport",
		City:      "Lagrange",
		State:     "Georgia",
		Country:   "US",
		Elevation: 693,
		Latitude:  33.0088996887,
		Longitude: -85.0726013184,
		Timezone:  "America/New_York",
	},
	"KLGD": {
		ICAO:      "KLGD",
		IATA:      "LGD",
		Name:      "La Grande Union County Airport",
		City:      "La Grande",
		State:     "Oregon",
		Country:   "US",
		Elevation: 2717,
		Latitude:  45.2901992798,
		Longitude: -118.0070037842,
		Timezone:  "America/Los_Angeles",
	},
	"KLGF": {
		ICAO:      "KLGF",
		IATA:      "LGF",
		Name:      "Laguna AAF (Yuma Proving Ground) Airfield",
		City:      "Yuma Proving Ground(Yuma)",
		State:     "Arizona",
		Country:   "US",
		Elevation: 433,
		Latitude:  32.86000061,
		Longitude: -114.3970032,
		Timezone:  "America/Phoenix",
	},
	"KLGU": {
		ICAO:      "KLGU",
		IATA:      "LGU",
		Name:      "Logan Cache Airport",
		City:      "Logan",
		State:     "Utah",
		Country:   "US",
		Elevation: 4457,
		Latitude:  41.7911987305,
		Longitude: -111.8519973755,
		Timezone:  "America/Denver",
	},
	"KLHB": {
		ICAO:      "KLHB",
		Name:      "Hearne Municipal Airport",
		City:      "Hearne",
		State:     "Texas",
		Country:   "US",
		Elevation: 285,
		Latitude:  30.871799469,
		Longitude: -96.6222000122,
		Timezone:  "America/Chicago",
	},
	"KLHM": {
		ICAO:      "KLHM",
		Name:      "Lincoln Regional Karl Harder Field",
		City:      "Lincoln",
		State:     "California",
		Country:   "US",
		Elevation: 121,
		Latitude:  38.909198761,
		Longitude: -121.3509979248,
		Timezone:  "America/Los_Angeles",
	},
	"KLHQ": {
		ICAO:      "KLHQ",
		Name:      "Fairfield County Airport",
		City:      "Lancaster",
		State:     "Ohio",
		Country:   "US",
		Elevation: 868,
		Latitude:  39.7555999756,
		Longitude: -82.6570968628,
		Timezone:  "America/New_York",
	},
	"KLHV": {
		ICAO:      "KLHV",
		IATA:      "LHV",
		Name:      "William T. Piper Memorial Airport",
		City:      "Lock Haven",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 556,
		Latitude:  41.13560104,
		Longitude: -77.42230225,
		Timezone:  "America/New_York",
	},
	"KLHW": {
		ICAO:      "KLHW",
		IATA:      "LIY",
		Name:      "Wright Aaf (Fort Stewart)/Midcoast Regional Airport",
		City:      "Fort Stewart(Hinesville)",
		State:     "Georgia",
		Country:   "US",
		Elevation: 45,
		Latitude:  31.88909912,
		Longitude: -81.56230164,
		Timezone:  "America/New_York",
	},
	"KLHX": {
		ICAO:      "KLHX",
		Name:      "La Junta Municipal Airport",
		City:      "La Junta",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4229,
		Latitude:  38.04970169,
		Longitude: -103.5090027,
		Timezone:  "America/Denver",
	},
	"KLHZ": {
		ICAO:      "KLHZ",
		IATA:      "LFN",
		Name:      "Franklin County Airport",
		City:      "Louisburg",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 369,
		Latitude:  36.0233001709,
		Longitude: -78.3302993774,
		Timezone:  "America/New_York",
	},
	"KLIC": {
		ICAO:      "KLIC",
		IATA:      "LIC",
		Name:      "Limon Municipal Airport",
		City:      "Limon",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5374,
		Latitude:  39.2747993469,
		Longitude: -103.6660003662,
		Timezone:  "America/Denver",
	},
	"KLIT": {
		ICAO:      "KLIT",
		IATA:      "LIT",
		Name:      "Adams Field",
		City:      "Little Rock",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 262,
		Latitude:  34.7294006348,
		Longitude: -92.2242965698,
		Timezone:  "America/Chicago",
	},
	"KLIU": {
		ICAO:      "KLIU",
		Name:      "Littlefield Municipal Airport",
		City:      "Littlefield",
		State:     "Texas",
		Country:   "US",
		Elevation: 3616,
		Latitude:  33.9239006042,
		Longitude: -102.3870010376,
		Timezone:  "America/Chicago",
	},
	"KLJF": {
		ICAO:      "KLJF",
		Name:      "Litchfield Municipal Airport",
		City:      "Litchfield",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1140,
		Latitude:  45.0970993042,
		Longitude: -94.5073013306,
		Timezone:  "America/Chicago",
	},
	"KLKP": {
		ICAO:      "KLKP",
		IATA:      "LKP",
		Name:      "Lake Placid Airport",
		City:      "Lake Placid",
		State:     "New-York",
		Country:   "US",
		Elevation: 1747,
		Latitude:  44.2644996643,
		Longitude: -73.9618988037,
		Timezone:  "America/New_York",
	},
	"KLKR": {
		ICAO:      "KLKR",
		Name:      "Lancaster County-Mc Whirter Field",
		City:      "Lancaster",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 486,
		Latitude:  34.72290039,
		Longitude: -80.854599,
		Timezone:  "America/New_York",
	},
	"KLKU": {
		ICAO:      "KLKU",
		IATA:      "LOW",
		Name:      "Louisa County-Freeman field",
		City:      "Louisa",
		State:     "Virginia",
		Country:   "US",
		Elevation: 493,
		Latitude:  38.00979996,
		Longitude: -77.9701004,
		Timezone:  "America/New_York",
	},
	"KLKV": {
		ICAO:      "KLKV",
		IATA:      "LKV",
		Name:      "Lake County Airport",
		City:      "Lakeview",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4733,
		Latitude:  42.1610984802,
		Longitude: -120.3990020752,
		Timezone:  "America/Los_Angeles",
	},
	"KLLJ": {
		ICAO:      "KLLJ",
		IATA:      "CHL",
		Name:      "Challis Airport",
		City:      "Challis",
		State:     "Idaho",
		Country:   "US",
		Elevation: 5072,
		Latitude:  44.5229988098,
		Longitude: -114.2180023193,
		Timezone:  "America/Boise",
	},
	"KLLN": {
		ICAO:      "KLLN",
		Name:      "Levelland Municipal Airport",
		City:      "Levelland",
		State:     "Texas",
		Country:   "US",
		Elevation: 3514,
		Latitude:  33.5525016785,
		Longitude: -102.3720016479,
		Timezone:  "America/Chicago",
	},
	"KLLQ": {
		ICAO:      "KLLQ",
		Name:      "Monticello Municipal Ellis Field",
		City:      "Monticello",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 270,
		Latitude:  33.6385994,
		Longitude: -91.75099945,
		Timezone:  "America/Chicago",
	},
	"KLLR": {
		ICAO:      "KLLR",
		Name:      "Little River Airport",
		City:      "Little River",
		State:     "California",
		Country:   "US",
		Elevation: 572,
		Latitude:  39.2620010376,
		Longitude: -123.7539978027,
		Timezone:  "America/Los_Angeles",
	},
	"KLLU": {
		ICAO:      "KLLU",
		Name:      "Lamar Municipal Airport",
		City:      "Lamar",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1009,
		Latitude:  37.4894981384,
		Longitude: -94.3115005493,
		Timezone:  "America/Chicago",
	},
	"KLMO": {
		ICAO:      "KLMO",
		Name:      "Vance Brand Airport",
		City:      "Longmont",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5055,
		Latitude:  40.1637001,
		Longitude: -105.163002,
		Timezone:  "America/Denver",
	},
	"KLMS": {
		ICAO:      "KLMS",
		IATA:      "LMS",
		Name:      "Louisville Winston County Airport",
		City:      "Louisville",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 575,
		Latitude:  33.1461982727,
		Longitude: -89.0625,
		Timezone:  "America/Chicago",
	},
	"KLMT": {
		ICAO:      "KLMT",
		IATA:      "LMT",
		Name:      "Klamath Falls Airport",
		City:      "Klamath Falls",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4095,
		Latitude:  42.1561012268,
		Longitude: -121.733001709,
		Timezone:  "America/Los_Angeles",
	},
	"KLNA": {
		ICAO:      "KLNA",
		IATA:      "LNA",
		Name:      "Palm Beach County Park Airport",
		City:      "West Palm Beach",
		State:     "Florida",
		Country:   "US",
		Elevation: 14,
		Latitude:  26.59300041,
		Longitude: -80.08509827,
		Timezone:  "America/New_York",
	},
	"KLNC": {
		ICAO:      "KLNC",
		Name:      "Lancaster Airport",
		City:      "Lancaster",
		State:     "Texas",
		Country:   "US",
		Elevation: 501,
		Latitude:  32.5792007446,
		Longitude: -96.71900177,
		Timezone:  "America/Chicago",
	},
	"KLND": {
		ICAO:      "KLND",
		IATA:      "LND",
		Name:      "Hunt Field",
		City:      "Lander",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 5586,
		Latitude:  42.8152008057,
		Longitude: -108.7300033569,
		Timezone:  "America/Denver",
	},
	"KLNK": {
		ICAO:      "KLNK",
		IATA:      "LNK",
		Name:      "Lincoln Airport",
		City:      "Lincoln",
		State:     "Montana",
		Country:   "US",
		Elevation: 1219,
		Latitude:  40.8510017395,
		Longitude: -96.7592010498,
		Timezone:  "America/Chicago",
	},
	"KLNL": {
		ICAO:      "KLNL",
		Name:      "Kings Land O' Lakes Airport",
		City:      "Land O' Lakes",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1704,
		Latitude:  46.15399933,
		Longitude: -89.21209717,
		Timezone:  "America/Chicago",
	},
	"KLNN": {
		ICAO:      "KLNN",
		IATA:      "LNN",
		Name:      "Willoughby Lost Nation Municipal Airport",
		City:      "Willoughby",
		State:     "Ohio",
		Country:   "US",
		Elevation: 626,
		Latitude:  41.6839981079,
		Longitude: -81.3897018433,
		Timezone:  "America/New_York",
	},
	"KLNP": {
		ICAO:      "KLNP",
		IATA:      "LNP",
		Name:      "Lonesome Pine Airport",
		City:      "Wise",
		State:     "Virginia",
		Country:   "US",
		Elevation: 2684,
		Latitude:  36.9874992371,
		Longitude: -82.5299987793,
		Timezone:  "America/New_York",
	},
	"KLNR": {
		ICAO:      "KLNR",
		IATA:      "LNR",
		Name:      "Tri-County Regional Airport",
		City:      "Lone Rock",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 717,
		Latitude:  43.2117004395,
		Longitude: -90.181602478,
		Timezone:  "America/Chicago",
	},
	"KLNS": {
		ICAO:      "KLNS",
		IATA:      "LNS",
		Name:      "Lancaster Airport",
		City:      "Lancaster",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 403,
		Latitude:  40.1217002869,
		Longitude: -76.2960968018,
		Timezone:  "America/New_York",
	},
	"KLOL": {
		ICAO:      "KLOL",
		IATA:      "LOL",
		Name:      "Derby Field",
		City:      "Lovelock",
		State:     "Nevada",
		Country:   "US",
		Elevation: 3904,
		Latitude:  40.0663986206,
		Longitude: -118.5650024414,
		Timezone:  "America/Los_Angeles",
	},
	"KLOM": {
		ICAO:      "KLOM",
		IATA:      "BBX",
		Name:      "Wings Field",
		City:      "Philadelphia",
		State:     "Florida",
		Country:   "US",
		Elevation: 302,
		Latitude:  40.1375007629,
		Longitude: -75.2650985718,
		Timezone:  "America/New_York",
	},
	"KLOT": {
		ICAO:      "KLOT",
		IATA:      "LOT",
		Name:      "Lewis University Airport",
		City:      "Chicago/Romeoville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 679,
		Latitude:  41.6072998,
		Longitude: -88.09619904,
		Timezone:  "America/Chicago",
	},
	"KLOU": {
		ICAO:      "KLOU",
		IATA:      "LOU",
		Name:      "Bowman Field",
		City:      "Louisville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 546,
		Latitude:  38.2280006409,
		Longitude: -85.6636962891,
		Timezone:  "America/Kentucky/Louisville",
	},
	"KLOZ": {
		ICAO:      "KLOZ",
		IATA:      "LOZ",
		Name:      "London Corbin Airport Magee Field",
		City:      "London",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 1212,
		Latitude:  37.0821990967,
		Longitude: -84.0848999023,
		Timezone:  "America/New_York",
	},
	"KLPC": {
		ICAO:      "KLPC",
		IATA:      "LPC",
		Name:      "Lompoc Airport",
		City:      "Lompoc",
		State:     "California",
		Country:   "US",
		Elevation: 88,
		Latitude:  34.665599823,
		Longitude: -120.4680023193,
		Timezone:  "America/Los_Angeles",
	},
	"KLPR": {
		ICAO:      "KLPR",
		Name:      "Lorain County Regional Airport",
		City:      "Lorain/Elyria",
		State:     "Ohio",
		Country:   "US",
		Elevation: 793,
		Latitude:  41.34429932,
		Longitude: -82.17759705,
		Timezone:  "America/New_York",
	},
	"KLQK": {
		ICAO:      "KLQK",
		IATA:      "LQK",
		Name:      "Pickens County Airport",
		City:      "Pickens",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 1013,
		Latitude:  34.8100013733,
		Longitude: -82.7029037476,
		Timezone:  "America/New_York",
	},
	"KLQR": {
		ICAO:      "KLQR",
		Name:      "Larned Pawnee County Airport",
		City:      "Larned",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2012,
		Latitude:  38.20859909,
		Longitude: -99.08599854,
		Timezone:  "America/Chicago",
	},
	"KLRD": {
		ICAO:      "KLRD",
		IATA:      "LRD",
		Name:      "Laredo International Airport",
		City:      "Laredo",
		State:     "Texas",
		Country:   "US",
		Elevation: 508,
		Latitude:  27.543800354,
		Longitude: -99.4616012573,
		Timezone:  "America/Chicago",
	},
	"KLRF": {
		ICAO:      "KLRF",
		IATA:      "LRF",
		Name:      "Little Rock Air Force Base",
		City:      "Jacksonville",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 311,
		Latitude:  34.9169006348,
		Longitude: -92.1496963501,
		Timezone:  "America/Chicago",
	},
	"KLRG": {
		ICAO:      "KLRG",
		Name:      "Lincoln Regional Airport",
		City:      "Lincoln",
		State:     "Maine",
		Country:   "US",
		Elevation: 208,
		Latitude:  45.3622016907,
		Longitude: -68.5346984863,
		Timezone:  "America/New_York",
	},
	"KLRJ": {
		ICAO:      "KLRJ",
		IATA:      "LRJ",
		Name:      "Le Mars Municipal Airport",
		City:      "Le Mars",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1197,
		Latitude:  42.77799988,
		Longitude: -96.1937027,
		Timezone:  "America/Chicago",
	},
	"KLRO": {
		ICAO:      "KLRO",
		Name:      "Mt Pleasant Regional-Faison field",
		City:      "Mount Pleasant",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 12,
		Latitude:  32.89780045,
		Longitude: -79.78289795,
		Timezone:  "America/New_York",
	},
	"KLRU": {
		ICAO:      "KLRU",
		IATA:      "LRU",
		Name:      "Las Cruces International Airport",
		City:      "Las Cruces",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4456,
		Latitude:  32.2893981934,
		Longitude: -106.9219970703,
		Timezone:  "America/Denver",
	},
	"KLRY": {
		ICAO:      "KLRY",
		Name:      "Lawrence Smith Memorial Airport",
		City:      "Harrisonville",
		State:     "Missouri",
		Country:   "US",
		Elevation: 915,
		Latitude:  38.611000061,
		Longitude: -94.3421020508,
		Timezone:  "America/Chicago",
	},
	"KLSB": {
		ICAO:      "KLSB",
		IATA:      "LSB",
		Name:      "Lordsburg Municipal Airport",
		City:      "Lordsburg",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4289,
		Latitude:  32.3334999084,
		Longitude: -108.6920013428,
		Timezone:  "America/Denver",
	},
	"KLSE": {
		ICAO:      "KLSE",
		IATA:      "LSE",
		Name:      "La Crosse Municipal Airport",
		City:      "La Crosse",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 655,
		Latitude:  43.87900162,
		Longitude: -91.25669861,
		Timezone:  "America/Chicago",
	},
	"KLSF": {
		ICAO:      "KLSF",
		IATA:      "LSF",
		Name:      "Lawson Army Air Field (Fort Benning)",
		City:      "Fort Benning(Columbus)",
		State:     "Georgia",
		Country:   "US",
		Elevation: 232,
		Latitude:  32.3372993469,
		Longitude: -84.9913024902,
		Timezone:  "America/Chicago",
	},
	"KLSK": {
		ICAO:      "KLSK",
		IATA:      "LSK",
		Name:      "Lusk Municipal Airport",
		City:      "Lusk",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4964,
		Latitude:  42.7537994385,
		Longitude: -104.4049987793,
		Timezone:  "America/Denver",
	},
	"KLSN": {
		ICAO:      "KLSN",
		IATA:      "LSN",
		Name:      "Los Banos Municipal Airport",
		City:      "Los Banos",
		State:     "California",
		Country:   "US",
		Elevation: 121,
		Latitude:  37.06290054,
		Longitude: -120.8690033,
		Timezone:  "America/Los_Angeles",
	},
	"KLSV": {
		ICAO:      "KLSV",
		IATA:      "LSV",
		Name:      "Nellis Air Force Base",
		City:      "Las Vegas",
		State:     "Nevada",
		Country:   "US",
		Elevation: 1870,
		Latitude:  36.2361984253,
		Longitude: -115.033996582,
		Timezone:  "America/Los_Angeles",
	},
	"KLTS": {
		ICAO:      "KLTS",
		IATA:      "LTS",
		Name:      "Altus Air Force Base",
		City:      "Altus",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1382,
		Latitude:  34.667098999,
		Longitude: -99.2667007446,
		Timezone:  "America/Chicago",
	},
	"KLTY": {
		ICAO:      "KLTY",
		Name:      "Liberty County Airport",
		City:      "Chester",
		State:     "Montana",
		Country:   "US",
		Elevation: 3160,
		Latitude:  48.5107002258,
		Longitude: -110.9909973145,
		Timezone:  "America/Denver",
	},
	"KLUD": {
		ICAO:      "KLUD",
		Name:      "Decatur Municipal Airport",
		City:      "Decatur",
		State:     "Texas",
		Country:   "US",
		Elevation: 1047,
		Latitude:  33.2546005249,
		Longitude: -97.5805969238,
		Timezone:  "America/Chicago",
	},
	"KLUF": {
		ICAO:      "KLUF",
		IATA:      "LUF",
		Name:      "Luke Air Force Base",
		City:      "Glendale",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1085,
		Latitude:  33.5349998474,
		Longitude: -112.3830032349,
		Timezone:  "America/Phoenix",
	},
	"KLUG": {
		ICAO:      "KLUG",
		Name:      "Ellington Airport",
		City:      "Lewisburg",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 717,
		Latitude:  35.5069999695,
		Longitude: -86.8039016724,
		Timezone:  "America/Chicago",
	},
	"KLUK": {
		ICAO:      "KLUK",
		IATA:      "LUK",
		Name:      "Cincinnati Municipal Airport Lunken Field",
		City:      "Cincinnati",
		State:     "Ohio",
		Country:   "US",
		Elevation: 483,
		Latitude:  39.10329819,
		Longitude: -84.41860199,
		Timezone:  "America/New_York",
	},
	"KLUL": {
		ICAO:      "KLUL",
		IATA:      "LUL",
		Name:      "Hesler Noble Field",
		City:      "Laurel",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 238,
		Latitude:  31.6725997925,
		Longitude: -89.172203064,
		Timezone:  "America/Chicago",
	},
	"KLUM": {
		ICAO:      "KLUM",
		Name:      "Menomonie Municipal Score Field",
		City:      "Menomonie",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 895,
		Latitude:  44.8922996521,
		Longitude: -91.8678970337,
		Timezone:  "America/Chicago",
	},
	"KLUX": {
		ICAO:      "KLUX",
		Name:      "Laurens County Airport",
		City:      "Laurens",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 697,
		Latitude:  34.5070991516,
		Longitude: -81.9471969604,
		Timezone:  "America/New_York",
	},
	"KLVJ": {
		ICAO:      "KLVJ",
		Name:      "Pearland Regional Airport",
		City:      "Houston",
		State:     "Texas",
		Country:   "US",
		Elevation: 44,
		Latitude:  29.5212993622,
		Longitude: -95.2421035767,
		Timezone:  "America/Chicago",
	},
	"KLVK": {
		ICAO:      "KLVK",
		IATA:      "LVK",
		Name:      "Livermore Municipal Airport",
		City:      "Livermore",
		State:     "California",
		Country:   "US",
		Elevation: 400,
		Latitude:  37.6934013367,
		Longitude: -121.8199996948,
		Timezone:  "America/Los_Angeles",
	},
	"KLVL": {
		ICAO:      "KLVL",
		IATA:      "LVL",
		Name:      "Lawrenceville Brunswick Municipal Airport",
		City:      "Lawrenceville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 329,
		Latitude:  36.7728004456,
		Longitude: -77.7942962646,
		Timezone:  "America/New_York",
	},
	"KLVM": {
		ICAO:      "KLVM",
		IATA:      "LVM",
		Name:      "Mission Field",
		City:      "Livingston",
		State:     "Montana",
		Country:   "US",
		Elevation: 4660,
		Latitude:  45.6994018555,
		Longitude: -110.4479980469,
		Timezone:  "America/Denver",
	},
	"KLVN": {
		ICAO:      "KLVN",
		Name:      "Airlake Airport",
		City:      "Minneapolis",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 960,
		Latitude:  44.6278991699,
		Longitude: -93.2281036377,
		Timezone:  "America/Chicago",
	},
	"KLVS": {
		ICAO:      "KLVS",
		IATA:      "LVS",
		Name:      "Las Vegas Municipal Airport",
		City:      "Las Vegas",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 6877,
		Latitude:  35.6542015076,
		Longitude: -105.141998291,
		Timezone:  "America/Denver",
	},
	"KLWA": {
		ICAO:      "KLWA",
		Name:      "South Haven Area Regional Airport",
		City:      "South Haven",
		State:     "Michigan",
		Country:   "US",
		Elevation: 666,
		Latitude:  42.3512001038,
		Longitude: -86.2556991577,
		Timezone:  "America/Detroit",
	},
	"KLWB": {
		ICAO:      "KLWB",
		IATA:      "LWB",
		Name:      "Greenbrier Valley Airport",
		City:      "Lewisburg",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 2302,
		Latitude:  37.8582992554,
		Longitude: -80.3994979858,
		Timezone:  "America/New_York",
	},
	"KLWC": {
		ICAO:      "KLWC",
		IATA:      "LWC",
		Name:      "Lawrence Municipal Airport",
		City:      "Lawrence",
		State:     "Kansas",
		Country:   "US",
		Elevation: 833,
		Latitude:  39.01119995,
		Longitude: -95.21659851,
		Timezone:  "America/Chicago",
	},
	"KLWL": {
		ICAO:      "KLWL",
		IATA:      "LWL",
		Name:      "Wells Municipal Harriet Field",
		City:      "Wells",
		State:     "Nevada",
		Country:   "US",
		Elevation: 5772,
		Latitude:  41.117099762,
		Longitude: -114.9219970703,
		Timezone:  "America/Los_Angeles",
	},
	"KLWM": {
		ICAO:      "KLWM",
		IATA:      "LWM",
		Name:      "Lawrence Municipal Airport",
		City:      "Lawrence",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 148,
		Latitude:  42.7172012329,
		Longitude: -71.1233978271,
		Timezone:  "America/New_York",
	},
	"KLWS": {
		ICAO:      "KLWS",
		IATA:      "LWS",
		Name:      "Lewiston Nez Perce County Airport",
		City:      "Lewiston",
		State:     "Idaho",
		Country:   "US",
		Elevation: 1442,
		Latitude:  46.3745002747,
		Longitude: -117.0149993896,
		Timezone:  "America/Los_Angeles",
	},
	"KLWT": {
		ICAO:      "KLWT",
		IATA:      "LWT",
		Name:      "Lewistown Municipal Airport",
		City:      "Lewistown",
		State:     "Montana",
		Country:   "US",
		Elevation: 4170,
		Latitude:  47.0493011475,
		Longitude: -109.4670028687,
		Timezone:  "America/Denver",
	},
	"KLWV": {
		ICAO:      "KLWV",
		IATA:      "LWV",
		Name:      "Lawrenceville Vincennes International Airport",
		City:      "Lawrenceville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 430,
		Latitude:  38.7643013,
		Longitude: -87.6054992676,
		Timezone:  "America/Chicago",
	},
	"KLXL": {
		ICAO:      "KLXL",
		Name:      "Little Falls-Morrison County-Lindbergh field",
		City:      "Little Falls",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1123,
		Latitude:  45.9496994,
		Longitude: -94.34739685,
		Timezone:  "America/Chicago",
	},
	"KLXN": {
		ICAO:      "KLXN",
		IATA:      "LXN",
		Name:      "Jim Kelly Field",
		City:      "Lexington",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2413,
		Latitude:  40.7910003662,
		Longitude: -99.7772979736,
		Timezone:  "America/Chicago",
	},
	"KLXT": {
		ICAO:      "KLXT",
		Name:      "Lee's Summit Municipal Airport",
		City:      "Lee's Summit",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1004,
		Latitude:  38.95970154,
		Longitude: -94.37139893,
		Timezone:  "America/Chicago",
	},
	"KLXV": {
		ICAO:      "KLXV",
		IATA:      "LXV",
		Name:      "Lake County Airport",
		City:      "Leadville",
		State:     "Colorado",
		Country:   "US",
		Elevation: 9927,
		Latitude:  39.2202987671,
		Longitude: -106.3170013428,
		Timezone:  "America/Denver",
	},
	"KLXY": {
		ICAO:      "KLXY",
		Name:      "Mexia Limestone County Airport",
		City:      "Mexia",
		State:     "Texas",
		Country:   "US",
		Elevation: 544,
		Latitude:  31.639799118,
		Longitude: -96.5147018433,
		Timezone:  "America/Chicago",
	},
	"KLYH": {
		ICAO:      "KLYH",
		IATA:      "LYH",
		Name:      "Lynchburg Regional Preston Glenn Field",
		City:      "Lynchburg",
		State:     "Virginia",
		Country:   "US",
		Elevation: 938,
		Latitude:  37.3266983032,
		Longitude: -79.2004013062,
		Timezone:  "America/New_York",
	},
	"KLYO": {
		ICAO:      "KLYO",
		IATA:      "LYO",
		Name:      "Lyons Rice County Municipal Airport",
		City:      "Lyons",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1691,
		Latitude:  38.34280014,
		Longitude: -98.22689819,
		Timezone:  "America/Chicago",
	},
	"KLYV": {
		ICAO:      "KLYV",
		Name:      "Quentin Aanenson Field",
		City:      "Luverne",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1431,
		Latitude:  43.6212005615,
		Longitude: -96.2157974243,
		Timezone:  "America/Chicago",
	},
	"KLZD": {
		ICAO:      "KLZD",
		Name:      "Danielson Airport",
		City:      "Danielson",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 238,
		Latitude:  41.8196983337,
		Longitude: -71.9010009766,
		Timezone:  "America/New_York",
	},
	"KLZU": {
		ICAO:      "KLZU",
		IATA:      "LZU",
		Name:      "Gwinnett County Briscoe Field",
		City:      "Lawrenceville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1061,
		Latitude:  33.97809982,
		Longitude: -83.96240234,
		Timezone:  "America/New_York",
	},
	"KLZZ": {
		ICAO:      "KLZZ",
		Name:      "Lampasas Airport",
		City:      "Lampasas",
		State:     "Texas",
		Country:   "US",
		Elevation: 1215,
		Latitude:  31.1061992645,
		Longitude: -98.1958999634,
		Timezone:  "America/Chicago",
	},
	"KM01": {
		ICAO:      "KM01",
		Name:      "General Dewitt Spain Airport",
		City:      "Memphis",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 225,
		Latitude:  35.20069885,
		Longitude: -90.05400085,
		Timezone:  "America/Chicago",
	},
	"KM02": {
		ICAO:      "KM02",
		Name:      "Dickson Municipal Airport",
		City:      "Dickson",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 892,
		Latitude:  36.1279983521,
		Longitude: -87.4298019409,
		Timezone:  "America/Chicago",
	},
	"KM04": {
		ICAO:      "KM04",
		Name:      "Covington Municipal Airport",
		City:      "Covington",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 280,
		Latitude:  35.5834007263,
		Longitude: -89.5871963501,
		Timezone:  "America/Chicago",
	},
	"KM05": {
		ICAO:      "KM05",
		Name:      "Caruthersville Memorial Airport",
		City:      "Caruthersville",
		State:     "Missouri",
		Country:   "US",
		Elevation: 268,
		Latitude:  36.1691017151,
		Longitude: -89.6764984131,
		Timezone:  "America/Chicago",
	},
	"KM08": {
		ICAO:      "KM08",
		Name:      "William L. Whitehurst Field",
		City:      "Bolivar",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 499,
		Latitude:  35.21450043,
		Longitude: -89.04340363,
		Timezone:  "America/Chicago",
	},
	"KM11": {
		ICAO:      "KM11",
		Name:      "Copiah County Airport",
		City:      "Crystal Springs",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 443,
		Latitude:  31.9029006958,
		Longitude: -90.3686981201,
		Timezone:  "America/Chicago",
	},
	"KM12": {
		ICAO:      "KM12",
		Name:      "Steele Municipal Airport",
		City:      "Steele",
		State:     "Missouri",
		Country:   "US",
		Elevation: 258,
		Latitude:  36.09590149,
		Longitude: -89.86340332,
		Timezone:  "America/Chicago",
	},
	"KM13": {
		ICAO:      "KM13",
		Name:      "Poplarville Pearl River County Airport",
		City:      "Poplarville",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 320,
		Latitude:  30.7859992981,
		Longitude: -89.5045013428,
		Timezone:  "America/Chicago",
	},
	"KM15": {
		ICAO:      "KM15",
		Name:      "Perry County Airport",
		City:      "Linden",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 740,
		Latitude:  35.5957984924,
		Longitude: -87.876701355,
		Timezone:  "America/Chicago",
	},
	"KM17": {
		ICAO:      "KM17",
		Name:      "Bolivar Municipal Airport",
		City:      "Bolivar",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1092,
		Latitude:  37.5960998535,
		Longitude: -93.3477020264,
		Timezone:  "America/Chicago",
	},
	"KM18": {
		ICAO:      "KM18",
		Name:      "Hope Municipal Airport",
		City:      "Hope",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 359,
		Latitude:  33.7201004028,
		Longitude: -93.6587982178,
		Timezone:  "America/Chicago",
	},
	"KM19": {
		ICAO:      "KM19",
		Name:      "Newport Municipal Airport",
		City:      "Newport",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 239,
		Latitude:  35.6376991272,
		Longitude: -91.176399231,
		Timezone:  "America/Chicago",
	},
	"KM20": {
		ICAO:      "KM20",
		Name:      "Grayson County Airport",
		City:      "Leitchfield",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 760,
		Latitude:  37.3995018005,
		Longitude: -86.2602005005,
		Timezone:  "America/Chicago",
	},
	"KM21": {
		ICAO:      "KM21",
		Name:      "Muhlenberg County Airport",
		City:      "Greenville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 422,
		Latitude:  37.2262001038,
		Longitude: -87.1564025879,
		Timezone:  "America/Chicago",
	},
	"KM22": {
		ICAO:      "KM22",
		Name:      "Russellville Municipal Airport",
		City:      "Russellville",
		State:     "Alabama",
		Country:   "US",
		Elevation: 723,
		Latitude:  34.4453010559,
		Longitude: -87.7113037109,
		Timezone:  "America/Chicago",
	},
	"KM23": {
		ICAO:      "KM23",
		Name:      "James H Easom Field",
		City:      "Newton",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 369,
		Latitude:  32.3117980957,
		Longitude: -89.1359024048,
		Timezone:  "America/Chicago",
	},
	"KM24": {
		ICAO:      "KM24",
		Name:      "Dean Griffin Memorial Airport",
		City:      "Wiggins",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 270,
		Latitude:  30.8432006836,
		Longitude: -89.1597976685,
		Timezone:  "America/Chicago",
	},
	"KM25": {
		ICAO:      "KM25",
		Name:      "Mayfield Graves County Airport",
		City:      "Mayfield",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 522,
		Latitude:  36.7696389,
		Longitude: -88.5846667,
		Timezone:  "America/Chicago",
	},
	"KM27": {
		ICAO:      "KM27",
		Name:      "Waldron Municipal Airport",
		City:      "Waldron",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 695,
		Latitude:  34.8759994507,
		Longitude: -94.1092987061,
		Timezone:  "America/Chicago",
	},
	"KM29": {
		ICAO:      "KM29",
		Name:      "Hassell Field",
		City:      "Clifton",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 401,
		Latitude:  35.3851013184,
		Longitude: -87.9674987793,
		Timezone:  "America/Chicago",
	},
	"KM30": {
		ICAO:      "KM30",
		Name:      "Metropolis Municipal Airport",
		City:      "Metropolis",
		State:     "Illinois",
		Country:   "US",
		Elevation: 384,
		Latitude:  37.1859016418,
		Longitude: -88.7506027222,
		Timezone:  "America/Chicago",
	},
	"KM31": {
		ICAO:      "KM31",
		Name:      "Arnold Field",
		City:      "Halls",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 292,
		Latitude:  35.9034004211,
		Longitude: -89.3973007202,
		Timezone:  "America/Chicago",
	},
	"KM32": {
		ICAO:      "KM32",
		Name:      "Lake Village Municipal Airport",
		City:      "Lake Village",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 125,
		Latitude:  33.3460006714,
		Longitude: -91.3156967163,
		Timezone:  "America/Chicago",
	},
	"KM33": {
		ICAO:      "KM33",
		Name:      "Sumner County Regional Airport",
		City:      "Gallatin",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 583,
		Latitude:  36.37680054,
		Longitude: -86.40879822,
		Timezone:  "America/Chicago",
	},
	"KM34": {
		ICAO:      "KM34",
		Name:      "Kentucky Dam State Park Airport",
		City:      "Gilbertsville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 349,
		Latitude:  37.0094985962,
		Longitude: -88.2958984375,
		Timezone:  "America/Chicago",
	},
	"KM36": {
		ICAO:      "KM36",
		Name:      "Frank Federer Memorial Airport",
		City:      "Brinkley",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 194,
		Latitude:  34.8802986145,
		Longitude: -91.176399231,
		Timezone:  "America/Chicago",
	},
	"KM37": {
		ICAO:      "KM37",
		Name:      "Ruleville Drew Airport",
		City:      "Drew",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 137,
		Latitude:  33.7764015198,
		Longitude: -90.5250015259,
		Timezone:  "America/Chicago",
	},
	"KM40": {
		ICAO:      "KM40",
		Name:      "Monroe County Airport",
		City:      "Aberdeen/Amory",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 226,
		Latitude:  33.8736991882,
		Longitude: -88.4897003174,
		Timezone:  "America/Chicago",
	},
	"KM41": {
		ICAO:      "KM41",
		Name:      "Holly Springs Marshall County Airport",
		City:      "Holly Springs",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 553,
		Latitude:  34.8042984009,
		Longitude: -89.5211029053,
		Timezone:  "America/Chicago",
	},
	"KM43": {
		ICAO:      "KM43",
		Name:      "Prentiss Jefferson Davis County Airport",
		City:      "Prentiss",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 455,
		Latitude:  31.59539986,
		Longitude: -89.90619659,
		Timezone:  "America/Chicago",
	},
	"KM44": {
		ICAO:      "KM44",
		Name:      "Houston Municipal Airport",
		City:      "Houston",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 337,
		Latitude:  33.8917999268,
		Longitude: -89.0236968994,
		Timezone:  "America/Chicago",
	},
	"KM45": {
		ICAO:      "KM45",
		Name:      "Alpine County Airport",
		City:      "Markleeville",
		State:     "California",
		Country:   "US",
		Elevation: 5867,
		Latitude:  38.7346992493,
		Longitude: -119.766998291,
		Timezone:  "America/Los_Angeles",
	},
	"KM46": {
		ICAO:      "KM46",
		Name:      "Colstrip Airport",
		City:      "Colstrip",
		State:     "Montana",
		Country:   "US",
		Elevation: 3426,
		Latitude:  45.8529014587,
		Longitude: -106.7089996338,
		Timezone:  "America/Denver",
	},
	"KM48": {
		ICAO:      "KM48",
		Name:      "Houston Memorial Airport",
		City:      "Houston",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1196,
		Latitude:  37.3301010132,
		Longitude: -91.973197937,
		Timezone:  "America/Chicago",
	},
	"KM50": {
		ICAO:      "KM50",
		Name:      "Boardman Airport",
		City:      "Boardman",
		State:     "Oregon",
		Country:   "US",
		Elevation: 396,
		Latitude:  45.8148002625,
		Longitude: -119.8209991455,
		Timezone:  "America/Los_Angeles",
	},
	"KM51": {
		ICAO:      "KM51",
		Name:      "Oktibbeha Airport",
		City:      "Starkville",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 250,
		Latitude:  33.4975013733,
		Longitude: -88.6812973022,
		Timezone:  "America/Chicago",
	},
	"KM52": {
		ICAO:      "KM52",
		Name:      "Franklin Wilkins Airport",
		City:      "Lexington",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 514,
		Latitude:  35.6512985229,
		Longitude: -88.3788986206,
		Timezone:  "America/Chicago",
	},
	"KM53": {
		ICAO:      "KM53",
		Name:      "Humboldt Municipal Airport",
		City:      "Humboldt",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 421,
		Latitude:  35.8022003174,
		Longitude: -88.8749008179,
		Timezone:  "America/Chicago",
	},
	"KM54": {
		ICAO:      "KM54",
		Name:      "Lebanon Municipal Airport",
		City:      "Lebanon",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 588,
		Latitude:  36.19039917,
		Longitude: -86.31569672,
		Timezone:  "America/Chicago",
	},
	"KM55": {
		ICAO:      "KM55",
		Name:      "Lamar County Airport",
		City:      "Vernon",
		State:     "Alabama",
		Country:   "US",
		Elevation: 463,
		Latitude:  33.846698761,
		Longitude: -88.1155014038,
		Timezone:  "America/Chicago",
	},
	"KM59": {
		ICAO:      "KM59",
		Name:      "Richton Perry County Airport",
		City:      "Richton",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 167,
		Latitude:  31.3173999786,
		Longitude: -88.9349975586,
		Timezone:  "America/Chicago",
	},
	"KM60": {
		ICAO:      "KM60",
		Name:      "Woodruff County Airport",
		City:      "Augusta",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 200,
		Latitude:  35.271900177,
		Longitude: -91.2696990967,
		Timezone:  "America/Chicago",
	},
	"KM65": {
		ICAO:      "KM65",
		Name:      "Wynne Municipal Airport",
		City:      "Wynne",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 370,
		Latitude:  35.2316017151,
		Longitude: -90.7615966797,
		Timezone:  "America/Chicago",
	},
	"KM66": {
		ICAO:      "KM66",
		Name:      "Alfred Schroeder Field",
		City:      "Hillsboro",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1434,
		Latitude:  38.3431015015,
		Longitude: -97.2142028809,
		Timezone:  "America/Chicago",
	},
	"KM70": {
		ICAO:      "KM70",
		Name:      "Pocahontas Municipal Airport",
		City:      "Pocahontas",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 273,
		Latitude:  36.2454986572,
		Longitude: -90.9552001953,
		Timezone:  "America/Chicago",
	},
	"KM71": {
		ICAO:      "KM71",
		Name:      "Greensfield Airport",
		City:      "Moscow Mills",
		State:     "Missouri",
		Country:   "US",
		Elevation: 549,
		Latitude:  38.9017982483,
		Longitude: -90.9602966309,
		Timezone:  "America/Chicago",
	},
	"KM72": {
		ICAO:      "KM72",
		Name:      "New Albany Union County Airport",
		City:      "New Albany",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 413,
		Latitude:  34.5486984253,
		Longitude: -89.0242004395,
		Timezone:  "America/Chicago",
	},
	"KM73": {
		ICAO:      "KM73",
		Name:      "Almyra Municipal Airport",
		City:      "Almyra",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 211,
		Latitude:  34.4123001099,
		Longitude: -91.4663009644,
		Timezone:  "America/Chicago",
	},
	"KM75": {
		ICAO:      "KM75",
		Name:      "Malta Airport",
		City:      "Malta",
		State:     "Montana",
		Country:   "US",
		Elevation: 2254,
		Latitude:  48.3669013977,
		Longitude: -107.9189987183,
		Timezone:  "America/Denver",
	},
	"KM77": {
		ICAO:      "KM77",
		Name:      "Howard County Airport",
		City:      "Nashville",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 553,
		Latitude:  33.9967002869,
		Longitude: -93.8380966187,
		Timezone:  "America/Chicago",
	},
	"KM78": {
		ICAO:      "KM78",
		Name:      "Malvern Municipal Airport",
		City:      "Malvern",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 538,
		Latitude:  34.3333015442,
		Longitude: -92.7614974976,
		Timezone:  "America/Chicago",
	},
	"KM79": {
		ICAO:      "KM79",
		Name:      "John H Hooks Jr Memorial Airport",
		City:      "Rayville",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 83,
		Latitude:  32.4852981567,
		Longitude: -91.7703018188,
		Timezone:  "America/Chicago",
	},
	"KM80": {
		ICAO:      "KM80",
		Name:      "Scott Airport",
		City:      "Tallulah",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 84,
		Latitude:  32.4163017273,
		Longitude: -91.1489028931,
		Timezone:  "America/Chicago",
	},
	"KM83": {
		ICAO:      "KM83",
		Name:      "Mccharen Field",
		City:      "West Point",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 205,
		Latitude:  33.5839996338,
		Longitude: -88.6667022705,
		Timezone:  "America/Chicago",
	},
	"KM85": {
		ICAO:      "KM85",
		Name:      "Gideon Memorial Airport",
		City:      "Gideon",
		State:     "Missouri",
		Country:   "US",
		Elevation: 268,
		Latitude:  36.4438018799,
		Longitude: -89.9038009644,
		Timezone:  "America/Chicago",
	},
	"KM88": {
		ICAO:      "KM88",
		Name:      "Cornelia Fort Airpark",
		City:      "Nashville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 418,
		Latitude:  36.1902999878,
		Longitude: -86.6996994019,
		Timezone:  "America/Chicago",
	},
	"KM89": {
		ICAO:      "KM89",
		Name:      "Dexter B Florence Memorial Field",
		City:      "Arkadelphia",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 182,
		Latitude:  34.0998001099,
		Longitude: -93.0661010742,
		Timezone:  "America/Chicago",
	},
	"KM90": {
		ICAO:      "KM90",
		Name:      "William Robert Johnston Municipal Airport",
		City:      "Mendota",
		State:     "California",
		Country:   "US",
		Elevation: 162,
		Latitude:  36.7585983276,
		Longitude: -120.371002197,
		Timezone:  "America/Los_Angeles",
	},
	"KM91": {
		ICAO:      "KM91",
		Name:      "Springfield Robertson County Airport",
		City:      "Springfield",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 706,
		Latitude:  36.53720093,
		Longitude: -86.92060089,
		Timezone:  "America/Chicago",
	},
	"KM93": {
		ICAO:      "KM93",
		Name:      "Houston County Airport",
		City:      "Mc Kinnon",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 370,
		Latitude:  36.3166999817,
		Longitude: -87.9167022705,
		Timezone:  "America/Chicago",
	},
	"KM94": {
		ICAO:      "KM94",
		Name:      "Desert Aire Airport",
		City:      "Mattawa",
		State:     "Washington",
		Country:   "US",
		Elevation: 586,
		Latitude:  46.6874008179,
		Longitude: -119.9209976196,
		Timezone:  "America/Los_Angeles",
	},
	"KM99": {
		ICAO:      "KM99",
		Name:      "Saline County/Watts Field",
		City:      "Benton",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 318,
		Latitude:  34.5565986633,
		Longitude: -92.6069030762,
		Timezone:  "America/Chicago",
	},
	"KMAC": {
		ICAO:      "KMAC",
		IATA:      "MAC",
		Name:      "Macon Downtown Airport",
		City:      "Macon",
		State:     "Georgia",
		Country:   "US",
		Elevation: 437,
		Latitude:  32.82210159,
		Longitude: -83.56199646,
		Timezone:  "America/New_York",
	},
	"KMAE": {
		ICAO:      "KMAE",
		IATA:      "MAE",
		Name:      "Madera Municipal Airport",
		City:      "Madera",
		State:     "California",
		Country:   "US",
		Elevation: 255,
		Latitude:  36.9886016846,
		Longitude: -120.1119995117,
		Timezone:  "America/Los_Angeles",
	},
	"KMAF": {
		ICAO:      "KMAF",
		IATA:      "MAF",
		Name:      "Midland International Airport",
		City:      "Midland",
		State:     "Texas",
		Country:   "US",
		Elevation: 2871,
		Latitude:  31.9424991608,
		Longitude: -102.202003479,
		Timezone:  "America/Chicago",
	},
	"KMAI": {
		ICAO:      "KMAI",
		Name:      "Marianna Municipal Airport",
		City:      "Marianna",
		State:     "Florida",
		Country:   "US",
		Elevation: 110,
		Latitude:  30.8377990723,
		Longitude: -85.1819000244,
		Timezone:  "America/Chicago",
	},
	"KMAL": {
		ICAO:      "KMAL",
		Name:      "Malone Dufort Airport",
		City:      "Malone",
		State:     "New-York",
		Country:   "US",
		Elevation: 790,
		Latitude:  44.8536987305,
		Longitude: -74.3289031982,
		Timezone:  "America/New_York",
	},
	"KMAO": {
		ICAO:      "KMAO",
		Name:      "Marion County Airport",
		City:      "Marion",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 92,
		Latitude:  34.1811981201,
		Longitude: -79.3347015381,
		Timezone:  "America/New_York",
	},
	"KMAW": {
		ICAO:      "KMAW",
		IATA:      "MAW",
		Name:      "Malden Regional Airport",
		City:      "Malden",
		State:     "Missouri",
		Country:   "US",
		Elevation: 294,
		Latitude:  36.6006012,
		Longitude: -89.99220276,
		Timezone:  "America/Chicago",
	},
	"KMBG": {
		ICAO:      "KMBG",
		IATA:      "MBG",
		Name:      "Mobridge Municipal Airport",
		City:      "Mobridge",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1716,
		Latitude:  45.54650116,
		Longitude: -100.4079971,
		Timezone:  "America/Chicago",
	},
	"KMBL": {
		ICAO:      "KMBL",
		IATA:      "MBL",
		Name:      "Manistee Co Blacker Airport",
		City:      "Manistee",
		State:     "Michigan",
		Country:   "US",
		Elevation: 621,
		Latitude:  44.2723999,
		Longitude: -86.24690247,
		Timezone:  "America/Detroit",
	},
	"KMBO": {
		ICAO:      "KMBO",
		IATA:      "DXE",
		Name:      "Bruce Campbell Field",
		City:      "Madison",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 326,
		Latitude:  32.4387016296,
		Longitude: -90.1031036377,
		Timezone:  "America/Chicago",
	},
	"KMBS": {
		ICAO:      "KMBS",
		IATA:      "MBS",
		Name:      "MBS International Airport",
		City:      "Saginaw",
		State:     "Michigan",
		Country:   "US",
		Elevation: 668,
		Latitude:  43.5329017639,
		Longitude: -84.0795974731,
		Timezone:  "America/Detroit",
	},
	"KMBT": {
		ICAO:      "KMBT",
		Name:      "Murfreesboro Municipal Airport",
		City:      "Murfreesboro",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 614,
		Latitude:  35.87749863,
		Longitude: -86.37750244,
		Timezone:  "America/Chicago",
	},
	"KMBY": {
		ICAO:      "KMBY",
		IATA:      "MBY",
		Name:      "Omar N Bradley Airport",
		City:      "Moberly",
		State:     "Missouri",
		Country:   "US",
		Elevation: 867,
		Latitude:  39.4639015198,
		Longitude: -92.4270019531,
		Timezone:  "America/Chicago",
	},
	"KMCB": {
		ICAO:      "KMCB",
		IATA:      "MCB",
		Name:      "Mc Comb/Pike County/John E Lewis Field",
		City:      "Mc Comb",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 413,
		Latitude:  31.17849922,
		Longitude: -90.47190094,
		Timezone:  "America/Chicago",
	},
	"KMCC": {
		ICAO:      "KMCC",
		IATA:      "MCC",
		Name:      "Mc Clellan Airfield",
		City:      "Sacramento",
		State:     "California",
		Country:   "US",
		Elevation: 77,
		Latitude:  38.66759872,
		Longitude: -121.401001,
		Timezone:  "America/Los_Angeles",
	},
	"KMCD": {
		ICAO:      "KMCD",
		IATA:      "MCD",
		Name:      "Mackinac Island Airport",
		City:      "Mackinac Island",
		State:     "Michigan",
		Country:   "US",
		Elevation: 739,
		Latitude:  45.86489868,
		Longitude: -84.63729858,
		Timezone:  "America/Detroit",
	},
	"KMCE": {
		ICAO:      "KMCE",
		IATA:      "MCE",
		Name:      "Merced Regional Macready Field",
		City:      "Merced",
		State:     "California",
		Country:   "US",
		Elevation: 155,
		Latitude:  37.28469849,
		Longitude: -120.5139999,
		Timezone:  "America/Los_Angeles",
	},
	"KMCF": {
		ICAO:      "KMCF",
		IATA:      "MCF",
		Name:      "MacDill Air Force Base",
		City:      "Tampa",
		State:     "Florida",
		Country:   "US",
		Elevation: 14,
		Latitude:  27.84930038,
		Longitude: -82.52120209,
		Timezone:  "America/New_York",
	},
	"KMCI": {
		ICAO:      "KMCI",
		IATA:      "MCI",
		Name:      "Kansas City International Airport",
		City:      "Kansas City",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1026,
		Latitude:  39.2975997925,
		Longitude: -94.7138977051,
		Timezone:  "America/Chicago",
	},
	"KMCK": {
		ICAO:      "KMCK",
		IATA:      "MCK",
		Name:      "Mc Cook Ben Nelson Regional Airport",
		City:      "Mc Cook",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2583,
		Latitude:  40.20629883,
		Longitude: -100.5920029,
		Timezone:  "America/Chicago",
	},
	"KMCN": {
		ICAO:      "KMCN",
		IATA:      "MCN",
		Name:      "Middle Georgia Regional Airport",
		City:      "Macon",
		State:     "Georgia",
		Country:   "US",
		Elevation: 354,
		Latitude:  32.6927986145,
		Longitude: -83.6492004395,
		Timezone:  "America/New_York",
	},
	"KMCO": {
		ICAO:      "KMCO",
		IATA:      "MCO",
		Name:      "Orlando International Airport",
		City:      "Orlando",
		State:     "Florida",
		Country:   "US",
		Elevation: 96,
		Latitude:  28.4293994904,
		Longitude: -81.3089981079,
		Timezone:  "America/New_York",
	},
	"KMCW": {
		ICAO:      "KMCW",
		IATA:      "MCW",
		Name:      "Mason City Municipal Airport",
		City:      "Mason City",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1213,
		Latitude:  43.1577987671,
		Longitude: -93.3312988281,
		Timezone:  "America/Chicago",
	},
	"KMCX": {
		ICAO:      "KMCX",
		Name:      "White County Airport",
		City:      "Monticello",
		State:     "Indiana",
		Country:   "US",
		Elevation: 676,
		Latitude:  40.7088012695,
		Longitude: -86.7667999268,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KMCZ": {
		ICAO:      "KMCZ",
		Name:      "Martin County Airport",
		City:      "Williamston",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 76,
		Latitude:  35.8622016907,
		Longitude: -77.1781997681,
		Timezone:  "America/New_York",
	},
	"KMDD": {
		ICAO:      "KMDD",
		IATA:      "MDD",
		Name:      "Midland Airpark",
		City:      "Midland",
		State:     "Texas",
		Country:   "US",
		Elevation: 2803,
		Latitude:  32.0364990234,
		Longitude: -102.1009979248,
		Timezone:  "America/Chicago",
	},
	"KMDF": {
		ICAO:      "KMDF",
		Name:      "Mooreland Municipal Airport",
		City:      "Mooreland",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1970,
		Latitude:  36.4847984314,
		Longitude: -99.1941986084,
		Timezone:  "America/Chicago",
	},
	"KMDH": {
		ICAO:      "KMDH",
		IATA:      "MDH",
		Name:      "Southern Illinois Airport",
		City:      "Carbondale/Murphysboro",
		State:     "Illinois",
		Country:   "US",
		Elevation: 411,
		Latitude:  37.7780990601,
		Longitude: -89.2519989014,
		Timezone:  "America/Chicago",
	},
	"KMDQ": {
		ICAO:      "KMDQ",
		Name:      "Madison County Executive -Tom Sharp Jr Airport",
		City:      "Huntsville",
		State:     "Alabama",
		Country:   "US",
		Elevation: 756,
		Latitude:  34.8614006,
		Longitude: -86.55750275,
		Timezone:  "America/Chicago",
	},
	"KMDS": {
		ICAO:      "KMDS",
		IATA:      "XMD",
		Name:      "Madison Municipal Airport",
		City:      "Madison",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1718,
		Latitude:  44.01599884,
		Longitude: -97.08589935,
		Timezone:  "America/Chicago",
	},
	"KMDT": {
		ICAO:      "KMDT",
		IATA:      "MDT",
		Name:      "Harrisburg International Airport",
		City:      "Harrisburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 310,
		Latitude:  40.1935005188,
		Longitude: -76.7633972168,
		Timezone:  "America/New_York",
	},
	"KMDW": {
		ICAO:      "KMDW",
		IATA:      "MDW",
		Name:      "Chicago Midway International Airport",
		City:      "Chicago",
		State:     "Illinois",
		Country:   "US",
		Elevation: 620,
		Latitude:  41.7859992981,
		Longitude: -87.7524032593,
		Timezone:  "America/Chicago",
	},
	"KMDZ": {
		ICAO:      "KMDZ",
		IATA:      "MDF",
		Name:      "Taylor County Airport",
		City:      "Medford",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1478,
		Latitude:  45.10100174,
		Longitude: -90.30329895,
		Timezone:  "America/Chicago",
	},
	"KMEB": {
		ICAO:      "KMEB",
		IATA:      "MXE",
		Name:      "Laurinburg Maxton Airport",
		City:      "Maxton",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 216,
		Latitude:  34.79190063,
		Longitude: -79.36579895,
		Timezone:  "America/New_York",
	},
	"KMEI": {
		ICAO:      "KMEI",
		IATA:      "MEI",
		Name:      "Key Field",
		City:      "Meridian",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 297,
		Latitude:  32.3325996399,
		Longitude: -88.7518997192,
		Timezone:  "America/Chicago",
	},
	"KMEJ": {
		ICAO:      "KMEJ",
		Name:      "Meade Municipal Airport",
		City:      "Meade",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2529,
		Latitude:  37.2769012451,
		Longitude: -100.3560028076,
		Timezone:  "America/Chicago",
	},
	"KMEM": {
		ICAO:      "KMEM",
		IATA:      "MEM",
		Name:      "Memphis International Airport",
		City:      "Memphis",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 341,
		Latitude:  35.0424003601,
		Longitude: -89.9766998291,
		Timezone:  "America/Chicago",
	},
	"KMER": {
		ICAO:      "KMER",
		IATA:      "MER",
		Name:      "Castle Airport",
		City:      "Merced",
		State:     "California",
		Country:   "US",
		Elevation: 191,
		Latitude:  37.38050079,
		Longitude: -120.5680008,
		Timezone:  "America/Los_Angeles",
	},
	"KMEV": {
		ICAO:      "KMEV",
		IATA:      "MEV",
		Name:      "Minden Tahoe Airport",
		City:      "Minden",
		State:     "Nevada",
		Country:   "US",
		Elevation: 4722,
		Latitude:  39.00030136,
		Longitude: -119.7509995,
		Timezone:  "America/Los_Angeles",
	},
	"KMEY": {
		ICAO:      "KMEY",
		Name:      "James G. Whiting Memorial Field",
		City:      "Mapleton",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1116,
		Latitude:  42.1782989502,
		Longitude: -95.7936019897,
		Timezone:  "America/Chicago",
	},
	"KMEZ": {
		ICAO:      "KMEZ",
		Name:      "Mena Intermountain Municipal Airport",
		City:      "Mena",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1080,
		Latitude:  34.54539871,
		Longitude: -94.20269775,
		Timezone:  "America/Chicago",
	},
	"KMFD": {
		ICAO:      "KMFD",
		IATA:      "MFD",
		Name:      "Mansfield Lahm Regional Airport",
		City:      "Mansfield",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1297,
		Latitude:  40.8213996887,
		Longitude: -82.5166015625,
		Timezone:  "America/New_York",
	},
	"KMFE": {
		ICAO:      "KMFE",
		IATA:      "MFE",
		Name:      "Mc Allen Miller International Airport",
		City:      "Mc Allen",
		State:     "Texas",
		Country:   "US",
		Elevation: 107,
		Latitude:  26.17580032,
		Longitude: -98.23860168,
		Timezone:  "America/Chicago",
	},
	"KMFI": {
		ICAO:      "KMFI",
		IATA:      "MFI",
		Name:      "Marshfield Municipal Airport",
		City:      "Marshfield",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1277,
		Latitude:  44.6369018555,
		Longitude: -90.1893005371,
		Timezone:  "America/Chicago",
	},
	"KMFR": {
		ICAO:      "KMFR",
		IATA:      "MFR",
		Name:      "Rogue Valley International Medford Airport",
		City:      "Medford",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1335,
		Latitude:  42.3741989136,
		Longitude: -122.8730010986,
		Timezone:  "America/Los_Angeles",
	},
	"KMFV": {
		ICAO:      "KMFV",
		IATA:      "MFV",
		Name:      "Accomack County Airport",
		City:      "Melfa",
		State:     "Virginia",
		Country:   "US",
		Elevation: 47,
		Latitude:  37.646900177,
		Longitude: -75.761100769,
		Timezone:  "America/New_York",
	},
	"KMGC": {
		ICAO:      "KMGC",
		IATA:      "MGC",
		Name:      "Michigan City Municipal Airport",
		City:      "Michigan City",
		State:     "Indiana",
		Country:   "US",
		Elevation: 655,
		Latitude:  41.7033004761,
		Longitude: -86.8211975098,
		Timezone:  "America/Chicago",
	},
	"KMGE": {
		ICAO:      "KMGE",
		IATA:      "MGE",
		Name:      "Dobbins Air Reserve Base Airport",
		City:      "Marietta",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1068,
		Latitude:  33.91540146,
		Longitude: -84.51629639,
		Timezone:  "America/New_York",
	},
	"KMGG": {
		ICAO:      "KMGG",
		Name:      "Maple Lake Municipal Airport",
		City:      "Maple Lake",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1028,
		Latitude:  45.236000061,
		Longitude: -93.9856033325,
		Timezone:  "America/Chicago",
	},
	"KMGJ": {
		ICAO:      "KMGJ",
		IATA:      "MGJ",
		Name:      "Orange County Airport",
		City:      "Montgomery",
		State:     "New-York",
		Country:   "US",
		Elevation: 364,
		Latitude:  41.50999832,
		Longitude: -74.26460266,
		Timezone:  "America/New_York",
	},
	"KMGM": {
		ICAO:      "KMGM",
		IATA:      "MGM",
		Name:      "Montgomery Regional (Dannelly Field) Airport",
		City:      "Montgomery",
		State:     "Alabama",
		Country:   "US",
		Elevation: 221,
		Latitude:  32.30059814,
		Longitude: -86.39399719,
		Timezone:  "America/Chicago",
	},
	"KMGN": {
		ICAO:      "KMGN",
		Name:      "Harbor Springs Airport",
		City:      "Harbor Springs",
		State:     "Michigan",
		Country:   "US",
		Elevation: 686,
		Latitude:  45.4255981445,
		Longitude: -84.9133987427,
		Timezone:  "America/Detroit",
	},
	"KMGR": {
		ICAO:      "KMGR",
		IATA:      "MGR",
		Name:      "Moultrie Municipal Airport",
		City:      "Moultrie",
		State:     "Georgia",
		Country:   "US",
		Elevation: 294,
		Latitude:  31.0848999023,
		Longitude: -83.8032989502,
		Timezone:  "America/New_York",
	},
	"KMGW": {
		ICAO:      "KMGW",
		IATA:      "MGW",
		Name:      "Morgantown Municipal Walter L. Bill Hart Field",
		City:      "Morgantown",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 1248,
		Latitude:  39.64289856,
		Longitude: -79.91629791,
		Timezone:  "America/New_York",
	},
	"KMGY": {
		ICAO:      "KMGY",
		IATA:      "MGY",
		Name:      "Dayton Wright Brothers Airport",
		City:      "Dayton",
		State:     "Ohio",
		Country:   "US",
		Elevation: 957,
		Latitude:  39.5890007019,
		Longitude: -84.224899292,
		Timezone:  "America/New_York",
	},
	"KMHE": {
		ICAO:      "KMHE",
		IATA:      "MHE",
		Name:      "Mitchell Municipal Airport",
		City:      "Mitchell",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1304,
		Latitude:  43.7747993469,
		Longitude: -98.0385971069,
		Timezone:  "America/Chicago",
	},
	"KMHK": {
		ICAO:      "KMHK",
		IATA:      "MHK",
		Name:      "Manhattan Regional Airport",
		City:      "Manhattan",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1057,
		Latitude:  39.1409988403,
		Longitude: -96.6707992554,
		Timezone:  "America/Chicago",
	},
	"KMHL": {
		ICAO:      "KMHL",
		IATA:      "MHL",
		Name:      "Marshall Memorial Municipal Airport",
		City:      "Marshall",
		State:     "Missouri",
		Country:   "US",
		Elevation: 779,
		Latitude:  39.0957984924,
		Longitude: -93.2029037476,
		Timezone:  "America/Chicago",
	},
	"KMHP": {
		ICAO:      "KMHP",
		Name:      "Metter Municipal Airport",
		City:      "Metter",
		State:     "Georgia",
		Country:   "US",
		Elevation: 197,
		Latitude:  32.3739013672,
		Longitude: -82.0792007446,
		Timezone:  "America/New_York",
	},
	"KMHR": {
		ICAO:      "KMHR",
		IATA:      "MHR",
		Name:      "Sacramento Mather Airport",
		City:      "Sacramento",
		State:     "California",
		Country:   "US",
		Elevation: 99,
		Latitude:  38.55390167,
		Longitude: -121.2979965,
		Timezone:  "America/Los_Angeles",
	},
	"KMHT": {
		ICAO:      "KMHT",
		IATA:      "MHT",
		Name:      "Manchester Airport",
		City:      "Manchester",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 266,
		Latitude:  42.932598114,
		Longitude: -71.4356994629,
		Timezone:  "America/New_York",
	},
	"KMHV": {
		ICAO:      "KMHV",
		IATA:      "MHV",
		Name:      "Mojave Airport",
		City:      "Mojave",
		State:     "California",
		Country:   "US",
		Elevation: 2801,
		Latitude:  35.05939865,
		Longitude: -118.1520004,
		Timezone:  "America/Los_Angeles",
	},
	"KMIA": {
		ICAO:      "KMIA",
		IATA:      "MIA",
		Name:      "Miami International Airport",
		City:      "Miami",
		State:     "Florida",
		Country:   "US",
		Elevation: 8,
		Latitude:  25.7931995392,
		Longitude: -80.2906036377,
		Timezone:  "America/New_York",
	},
	"KMIB": {
		ICAO:      "KMIB",
		IATA:      "MIB",
		Name:      "Minot Air Force Base",
		City:      "Minot",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1667,
		Latitude:  48.41559982,
		Longitude: -101.3580017,
		Timezone:  "America/Chicago",
	},
	"KMIC": {
		ICAO:      "KMIC",
		IATA:      "MIC",
		Name:      "Crystal Airport",
		City:      "Minneapolis",
		State:     "California",
		Country:   "US",
		Elevation: 869,
		Latitude:  45.0620002747,
		Longitude: -93.3538970947,
		Timezone:  "America/Chicago",
	},
	"KMIE": {
		ICAO:      "KMIE",
		IATA:      "MIE",
		Name:      "Delaware County Johnson Field",
		City:      "Muncie",
		State:     "Indiana",
		Country:   "US",
		Elevation: 937,
		Latitude:  40.2422981262,
		Longitude: -85.3958969116,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KMIO": {
		ICAO:      "KMIO",
		IATA:      "MIO",
		Name:      "Miami Municipal Airport",
		City:      "Miami",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 808,
		Latitude:  36.909198761,
		Longitude: -94.8874969482,
		Timezone:  "America/Chicago",
	},
	"KMIT": {
		ICAO:      "KMIT",
		IATA:      "MIT",
		Name:      "Shafter Minter Field",
		City:      "Shafter",
		State:     "California",
		Country:   "US",
		Elevation: 424,
		Latitude:  35.50740051,
		Longitude: -119.1920013,
		Timezone:  "America/Los_Angeles",
	},
	"KMIV": {
		ICAO:      "KMIV",
		IATA:      "MIV",
		Name:      "Millville Municipal Airport",
		City:      "Millville",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 85,
		Latitude:  39.3678016663,
		Longitude: -75.0721969604,
		Timezone:  "America/New_York",
	},
	"KMIW": {
		ICAO:      "KMIW",
		IATA:      "MIW",
		Name:      "Marshalltown Municipal Airport",
		City:      "Marshalltown",
		State:     "Iowa",
		Country:   "US",
		Elevation: 974,
		Latitude:  42.112701416,
		Longitude: -92.9178009033,
		Timezone:  "America/Chicago",
	},
	"KMJD": {
		ICAO:      "KMJD",
		Name:      "Picayune Municipal Airport",
		City:      "Picayune",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 55,
		Latitude:  30.48749924,
		Longitude: -89.65119934,
		Timezone:  "America/Chicago",
	},
	"KMJQ": {
		ICAO:      "KMJQ",
		IATA:      "MJQ",
		Name:      "Jackson Municipal Airport",
		City:      "Jackson",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1446,
		Latitude:  43.6500015259,
		Longitude: -94.9865036011,
		Timezone:  "America/Chicago",
	},
	"KMJX": {
		ICAO:      "KMJX",
		IATA:      "MJX",
		Name:      "Robert J. Miller Air Park",
		City:      "Toms River",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 82,
		Latitude:  39.92750168,
		Longitude: -74.29239655,
		Timezone:  "America/New_York",
	},
	"KMKA": {
		ICAO:      "KMKA",
		Name:      "Miller Municipal Airport",
		City:      "Miller",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1569,
		Latitude:  44.5251998901,
		Longitude: -98.9580993652,
		Timezone:  "America/Chicago",
	},
	"KMKC": {
		ICAO:      "KMKC",
		IATA:      "MKC",
		Name:      "Charles B. Wheeler Downtown Airport",
		City:      "Kansas City",
		State:     "Missouri",
		Country:   "US",
		Elevation: 759,
		Latitude:  39.1231994629,
		Longitude: -94.5927963257,
		Timezone:  "America/Chicago",
	},
	"KMKE": {
		ICAO:      "KMKE",
		IATA:      "MKE",
		Name:      "General Mitchell International Airport",
		City:      "Milwaukee",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 723,
		Latitude:  42.9472007751,
		Longitude: -87.8965988159,
		Timezone:  "America/Chicago",
	},
	"KMKG": {
		ICAO:      "KMKG",
		IATA:      "MKG",
		Name:      "Muskegon County Airport",
		City:      "Muskegon",
		State:     "Michigan",
		Country:   "US",
		Elevation: 629,
		Latitude:  43.16949844,
		Longitude: -86.23819733,
		Timezone:  "America/Detroit",
	},
	"KMKJ": {
		ICAO:      "KMKJ",
		Name:      "Mountain Empire Airport",
		City:      "Marion/Wytheville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 2558,
		Latitude:  36.8949012756,
		Longitude: -81.349899292,
		Timezone:  "America/New_York",
	},
	"KMKL": {
		ICAO:      "KMKL",
		IATA:      "MKL",
		Name:      "Mc Kellar Sipes Regional Airport",
		City:      "Jackson",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 434,
		Latitude:  35.59989929,
		Longitude: -88.91560364,
		Timezone:  "America/Chicago",
	},
	"KMKN": {
		ICAO:      "KMKN",
		Name:      "Comanche County City Airport",
		City:      "Comanche",
		State:     "Texas",
		Country:   "US",
		Elevation: 1385,
		Latitude:  31.91679955,
		Longitude: -98.60030365,
		Timezone:  "America/Chicago",
	},
	"KMKO": {
		ICAO:      "KMKO",
		IATA:      "MKO",
		Name:      "Davis Field",
		City:      "Muskogee",
		State:     "Florida",
		Country:   "US",
		Elevation: 611,
		Latitude:  35.65650177,
		Longitude: -95.36669922,
		Timezone:  "America/Chicago",
	},
	"KMKS": {
		ICAO:      "KMKS",
		Name:      "Berkeley County Airport",
		City:      "Moncks Corner",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 73,
		Latitude:  33.1855010986,
		Longitude: -80.0363006592,
		Timezone:  "America/New_York",
	},
	"KMKT": {
		ICAO:      "KMKT",
		IATA:      "MKT",
		Name:      "Mankato Regional Airport",
		City:      "Mankato",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1021,
		Latitude:  44.22159958,
		Longitude: -93.91870117,
		Timezone:  "America/Chicago",
	},
	"KMKV": {
		ICAO:      "KMKV",
		Name:      "Marksville Municipal Airport",
		City:      "Marksville",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 79,
		Latitude:  31.0946998596,
		Longitude: -92.0690994263,
		Timezone:  "America/Chicago",
	},
	"KMKY": {
		ICAO:      "KMKY",
		IATA:      "MRK",
		Name:      "Marco Island Airport",
		City:      "Marco Island",
		State:     "Florida",
		Country:   "US",
		Elevation: 5,
		Latitude:  25.9950008392,
		Longitude: -81.6725006104,
		Timezone:  "America/New_York",
	},
	"KMLB": {
		ICAO:      "KMLB",
		IATA:      "MLB",
		Name:      "Melbourne International Airport",
		City:      "Melbourne",
		State:     "Florida",
		Country:   "US",
		Elevation: 33,
		Latitude:  28.1028003693,
		Longitude: -80.6453018188,
		Timezone:  "America/New_York",
	},
	"KMLC": {
		ICAO:      "KMLC",
		IATA:      "MLC",
		Name:      "Mc Alester Regional Airport",
		City:      "Mc Alester",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 770,
		Latitude:  34.88240051,
		Longitude: -95.78350067,
		Timezone:  "America/Chicago",
	},
	"KMLD": {
		ICAO:      "KMLD",
		IATA:      "MLD",
		Name:      "Malad City Airport",
		City:      "Malad City",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4503,
		Latitude:  42.1665992737,
		Longitude: -112.2969970703,
		Timezone:  "America/Boise",
	},
	"KMLE": {
		ICAO:      "KMLE",
		IATA:      "MIQ",
		Name:      "Millard Airport",
		City:      "Omaha",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1051,
		Latitude:  41.1959991455,
		Longitude: -96.1119995117,
		Timezone:  "America/Chicago",
	},
	"KMLF": {
		ICAO:      "KMLF",
		IATA:      "MLF",
		Name:      "Milford Municipal-Ben and Judy Briscoe Field",
		City:      "Milford",
		State:     "Utah",
		Country:   "US",
		Elevation: 5039,
		Latitude:  38.42660141,
		Longitude: -113.012001,
		Timezone:  "America/Denver",
	},
	"KMLI": {
		ICAO:      "KMLI",
		IATA:      "MLI",
		Name:      "Quad City International Airport",
		City:      "Moline",
		State:     "Illinois",
		Country:   "US",
		Elevation: 590,
		Latitude:  41.4485015869,
		Longitude: -90.5074996948,
		Timezone:  "America/Chicago",
	},
	"KMLJ": {
		ICAO:      "KMLJ",
		IATA:      "MLJ",
		Name:      "Baldwin County Airport",
		City:      "Milledgeville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 385,
		Latitude:  33.15420151,
		Longitude: -83.24069977,
		Timezone:  "America/New_York",
	},
	"KMLS": {
		ICAO:      "KMLS",
		IATA:      "MLS",
		Name:      "Frank Wiley Field",
		City:      "Miles City",
		State:     "Montana",
		Country:   "US",
		Elevation: 2630,
		Latitude:  46.4280014038,
		Longitude: -105.8860015869,
		Timezone:  "America/Denver",
	},
	"KMLT": {
		ICAO:      "KMLT",
		IATA:      "MLT",
		Name:      "Millinocket Municipal Airport",
		City:      "Millinocket",
		State:     "Maine",
		Country:   "US",
		Elevation: 408,
		Latitude:  45.6478004456,
		Longitude: -68.6856002808,
		Timezone:  "America/New_York",
	},
	"KMLU": {
		ICAO:      "KMLU",
		IATA:      "MLU",
		Name:      "Monroe Regional Airport",
		City:      "Monroe",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 79,
		Latitude:  32.5108985901,
		Longitude: -92.0376968384,
		Timezone:  "America/Chicago",
	},
	"KMMH": {
		ICAO:      "KMMH",
		IATA:      "MMH",
		Name:      "Mammoth Yosemite Airport",
		City:      "Mammoth Lakes",
		State:     "California",
		Country:   "US",
		Elevation: 7135,
		Latitude:  37.62409973,
		Longitude: -118.8379974,
		Timezone:  "America/Los_Angeles",
	},
	"KMMI": {
		ICAO:      "KMMI",
		IATA:      "MMI",
		Name:      "McMinn County Airport",
		City:      "Athens",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 875,
		Latitude:  35.39730072,
		Longitude: -84.56259918,
		Timezone:  "America/New_York",
	},
	"KMMK": {
		ICAO:      "KMMK",
		Name:      "Meriden Markham Municipal Airport",
		City:      "Meriden",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 103,
		Latitude:  41.5087013245,
		Longitude: -72.829498291,
		Timezone:  "America/New_York",
	},
	"KMML": {
		ICAO:      "KMML",
		IATA:      "MML",
		Name:      "Southwest Minnesota Regional-Marshall-Ryan Field",
		City:      "Marshall",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1183,
		Latitude:  44.45050049,
		Longitude: -95.82189941,
		Timezone:  "America/Chicago",
	},
	"KMMS": {
		ICAO:      "KMMS",
		IATA:      "MMS",
		Name:      "Selfs Airport",
		City:      "Marks",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 162,
		Latitude:  34.2314987183,
		Longitude: -90.2895965576,
		Timezone:  "America/Chicago",
	},
	"KMMT": {
		ICAO:      "KMMT",
		IATA:      "MMT",
		Name:      "Mc Entire JNGB Airport",
		City:      "Eastover",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 254,
		Latitude:  33.92079926,
		Longitude: -80.80130005,
		Timezone:  "America/New_York",
	},
	"KMMU": {
		ICAO:      "KMMU",
		IATA:      "MMU",
		Name:      "Morristown Municipal Airport",
		City:      "Morristown",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 187,
		Latitude:  40.7994003296,
		Longitude: -74.4149017334,
		Timezone:  "America/New_York",
	},
	"KMMV": {
		ICAO:      "KMMV",
		Name:      "Mc Minnville Municipal Airport",
		City:      "Mc Minnville",
		State:     "Oregon",
		Country:   "US",
		Elevation: 163,
		Latitude:  45.19440079,
		Longitude: -123.1360016,
		Timezone:  "America/Los_Angeles",
	},
	"KMNE": {
		ICAO:      "KMNE",
		Name:      "Minden Airport",
		City:      "Minden",
		State:     "Luisiana",
		Country:   "US",
		Elevation: 278,
		Latitude:  32.6460278,
		Longitude: -93.2980833,
		Timezone:  "America/Chicago",
	},
	"KMNF": {
		ICAO:      "KMNF",
		Name:      "Mountain View Airport",
		City:      "Mountain View",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1181,
		Latitude:  36.9928016663,
		Longitude: -91.7145004272,
		Timezone:  "America/Chicago",
	},
	"KMNI": {
		ICAO:      "KMNI",
		Name:      "Santee Cooper Regional Airport",
		City:      "Manning",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 103,
		Latitude:  33.5871009827,
		Longitude: -80.2087020874,
		Timezone:  "America/New_York",
	},
	"KMNM": {
		ICAO:      "KMNM",
		IATA:      "MNM",
		Name:      "Menominee Marinette Twin County Airport",
		City:      "Menominee",
		State:     "Michigan",
		Country:   "US",
		Elevation: 625,
		Latitude:  45.126701355,
		Longitude: -87.6383972168,
		Timezone:  "America/Chicago",
	},
	"KMNN": {
		ICAO:      "KMNN",
		IATA:      "MNN",
		Name:      "Marion Municipal Airport",
		City:      "Marion",
		State:     "Ohio",
		Country:   "US",
		Elevation: 993,
		Latitude:  40.6161994934,
		Longitude: -83.0634994507,
		Timezone:  "America/New_York",
	},
	"KMNV": {
		ICAO:      "KMNV",
		Name:      "Monroe County Airport",
		City:      "Madisonville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1031,
		Latitude:  35.5453987122,
		Longitude: -84.3802032471,
		Timezone:  "America/New_York",
	},
	"KMNZ": {
		ICAO:      "KMNZ",
		Name:      "Hamilton Municipal Airport",
		City:      "Hamilton",
		State:     "New-York",
		Country:   "US",
		Elevation: 1299,
		Latitude:  31.6658992767,
		Longitude: -98.1485977173,
		Timezone:  "America/Chicago",
	},
	"KMO1": {
		ICAO:      "KMO1",
		Name:      "Richland Municipal Airport",
		City:      "Richland",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1110,
		Latitude:  37.875,
		Longitude: -92.4085006714,
		Timezone:  "America/Chicago",
	},
	"KMO3": {
		ICAO:      "KMO3",
		Name:      "Stockton Municipal Airport",
		City:      "Stockton",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1042,
		Latitude:  37.6603012085,
		Longitude: -93.8168029785,
		Timezone:  "America/Chicago",
	},
	"KMO6": {
		ICAO:      "KMO6",
		Name:      "Washington Regional Airport",
		City:      "Washington",
		State:     "Missouri",
		Country:   "US",
		Elevation: 488,
		Latitude:  38.58760071,
		Longitude: -90.9937973,
		Timezone:  "America/Chicago",
	},
	"KMO8": {
		ICAO:      "KMO8",
		Name:      "North Central Missouri Regional Airport",
		City:      "Brookfield",
		State:     "Missouri",
		Country:   "US",
		Elevation: 843,
		Latitude:  39.7700004578,
		Longitude: -93.0127029419,
		Timezone:  "America/Chicago",
	},
	"KMOB": {
		ICAO:      "KMOB",
		IATA:      "MOB",
		Name:      "Mobile Regional Airport",
		City:      "Mobile",
		State:     "Alabama",
		Country:   "US",
		Elevation: 219,
		Latitude:  30.6912002563,
		Longitude: -88.2427978516,
		Timezone:  "America/Chicago",
	},
	"KMOD": {
		ICAO:      "KMOD",
		IATA:      "MOD",
		Name:      "Modesto City Co-Harry Sham Field",
		City:      "Modesto",
		State:     "California",
		Country:   "US",
		Elevation: 97,
		Latitude:  37.62580109,
		Longitude: -120.9540024,
		Timezone:  "America/Los_Angeles",
	},
	"KMOP": {
		ICAO:      "KMOP",
		IATA:      "MOP",
		Name:      "Mount Pleasant Municipal Airport",
		City:      "Mount Pleasant",
		State:     "Michigan",
		Country:   "US",
		Elevation: 755,
		Latitude:  43.6217002869,
		Longitude: -84.7375030518,
		Timezone:  "America/Detroit",
	},
	"KMOR": {
		ICAO:      "KMOR",
		IATA:      "MOR",
		Name:      "Moore Murrell Airport",
		City:      "Morristown",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1313,
		Latitude:  36.1794013977,
		Longitude: -83.37550354,
		Timezone:  "America/New_York",
	},
	"KMOT": {
		ICAO:      "KMOT",
		IATA:      "MOT",
		Name:      "Minot International Airport",
		City:      "Minot",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1716,
		Latitude:  48.2593994141,
		Longitude: -101.2799987793,
		Timezone:  "America/Chicago",
	},
	"KMOX": {
		ICAO:      "KMOX",
		IATA:      "MOX",
		Name:      "Morris Municipal - Charlie Schmidt Airport",
		City:      "Morris",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1138,
		Latitude:  45.56600189,
		Longitude: -95.96720123,
		Timezone:  "America/Chicago",
	},
	"KMPE": {
		ICAO:      "KMPE",
		Name:      "Philadelphia Municipal Airport",
		City:      "Philadelphia",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 458,
		Latitude:  32.79940033,
		Longitude: -89.12599945,
		Timezone:  "America/Chicago",
	},
	"KMPG": {
		ICAO:      "KMPG",
		Name:      "Marshall County Airport",
		City:      "Moundsville",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 1214,
		Latitude:  39.8807983398,
		Longitude: -80.7358016968,
		Timezone:  "America/New_York",
	},
	"KMPI": {
		ICAO:      "KMPI",
		IATA:      "RMY",
		Name:      "Mariposa Yosemite Airport",
		City:      "Mariposa",
		State:     "California",
		Country:   "US",
		Elevation: 2254,
		Latitude:  37.5108985901,
		Longitude: -120.040000916,
		Timezone:  "America/Los_Angeles",
	},
	"KMPJ": {
		ICAO:      "KMPJ",
		IATA:      "MPJ",
		Name:      "Petit Jean Park Airport",
		City:      "Morrilton",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 923,
		Latitude:  35.1389007568,
		Longitude: -92.9092025757,
		Timezone:  "America/Chicago",
	},
	"KMPO": {
		ICAO:      "KMPO",
		IATA:      "MPO",
		Name:      "Pocono Mountains Municipal Airport",
		City:      "Mount Pocono",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1915,
		Latitude:  41.13750076,
		Longitude: -75.37889862,
		Timezone:  "America/New_York",
	},
	"KMPR": {
		ICAO:      "KMPR",
		IATA:      "MPR",
		Name:      "Mc Pherson Airport",
		City:      "Mc Pherson",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1498,
		Latitude:  38.35240173,
		Longitude: -97.69129944,
		Timezone:  "America/Chicago",
	},
	"KMPV": {
		ICAO:      "KMPV",
		IATA:      "MPV",
		Name:      "Edward F Knapp State Airport",
		City:      "Barre/Montpelier",
		State:     "Vermont",
		Country:   "US",
		Elevation: 1166,
		Latitude:  44.20349884,
		Longitude: -72.56230164,
		Timezone:  "America/New_York",
	},
	"KMPZ": {
		ICAO:      "KMPZ",
		IATA:      "MPZ",
		Name:      "Mount Pleasant Municipal Airport",
		City:      "Mount Pleasant",
		State:     "Iowa",
		Country:   "US",
		Elevation: 730,
		Latitude:  40.94660187,
		Longitude: -91.51110077,
		Timezone:  "America/Chicago",
	},
	"KMQB": {
		ICAO:      "KMQB",
		IATA:      "MQB",
		Name:      "Macomb Municipal Airport",
		City:      "Macomb",
		State:     "Illinois",
		Country:   "US",
		Elevation: 707,
		Latitude:  40.5200996399,
		Longitude: -90.6523971558,
		Timezone:  "America/Chicago",
	},
	"KMQI": {
		ICAO:      "KMQI",
		IATA:      "MEO",
		Name:      "Dare County Regional Airport",
		City:      "Manteo",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 13,
		Latitude:  35.91899872,
		Longitude: -75.69550323,
		Timezone:  "America/New_York",
	},
	"KMQJ": {
		ICAO:      "KMQJ",
		Name:      "Mount Comfort Airport",
		City:      "Indianapolis",
		State:     "Indiana",
		Country:   "US",
		Elevation: 862,
		Latitude:  39.84349823,
		Longitude: -85.89710236,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KMQS": {
		ICAO:      "KMQS",
		IATA:      "CTH",
		Name:      "Chester County G O Carlson Airport",
		City:      "Coatesville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 660,
		Latitude:  39.97900009,
		Longitude: -75.8655014,
		Timezone:  "America/New_York",
	},
	"KMQW": {
		ICAO:      "KMQW",
		IATA:      "MQW",
		Name:      "Telfair Wheeler Airport",
		City:      "Mc Rae",
		State:     "Georgia",
		Country:   "US",
		Elevation: 202,
		Latitude:  32.0957984924,
		Longitude: -82.8799972534,
		Timezone:  "America/New_York",
	},
	"KMQY": {
		ICAO:      "KMQY",
		IATA:      "MQY",
		Name:      "Smyrna Airport",
		City:      "Smyrna",
		State:     "Delaware",
		Country:   "US",
		Elevation: 543,
		Latitude:  36.0089988708,
		Longitude: -86.5201034546,
		Timezone:  "America/Chicago",
	},
	"KMRB": {
		ICAO:      "KMRB",
		IATA:      "MRB",
		Name:      "Eastern Wv Regional Shepherd Field",
		City:      "Martinsburg",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 565,
		Latitude:  39.40190125,
		Longitude: -77.98459625,
		Timezone:  "America/New_York",
	},
	"KMRC": {
		ICAO:      "KMRC",
		IATA:      "MRC",
		Name:      "Maury County Airport",
		City:      "Columbia/Mount Pleasant",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 681,
		Latitude:  35.5541000366,
		Longitude: -87.1789016724,
		Timezone:  "America/Chicago",
	},
	"KMRF": {
		ICAO:      "KMRF",
		IATA:      "MRF",
		Name:      "Marfa Municipal Airport",
		City:      "Marfa",
		State:     "Texas",
		Country:   "US",
		Elevation: 4849,
		Latitude:  30.371099472,
		Longitude: -104.0179977417,
		Timezone:  "America/Chicago",
	},
	"KMRH": {
		ICAO:      "KMRH",
		Name:      "Michael J. Smith Field",
		City:      "Beaufort",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 11,
		Latitude:  34.73360062,
		Longitude: -76.66059875,
		Timezone:  "America/New_York",
	},
	"KMRJ": {
		ICAO:      "KMRJ",
		Name:      "Iowa County Airport",
		City:      "Mineral Point",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1171,
		Latitude:  42.8867988586,
		Longitude: -90.2361984253,
		Timezone:  "America/Chicago",
	},
	"KMRN": {
		ICAO:      "KMRN",
		IATA:      "MRN",
		Name:      "Foothills Regional Airport",
		City:      "Morganton",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 1270,
		Latitude:  35.8202018738,
		Longitude: -81.6113967896,
		Timezone:  "America/New_York",
	},
	"KMRT": {
		ICAO:      "KMRT",
		Name:      "Union County Airport",
		City:      "Marysville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1021,
		Latitude:  40.2247009277,
		Longitude: -83.351600647,
		Timezone:  "America/New_York",
	},
	"KMRY": {
		ICAO:      "KMRY",
		IATA:      "MRY",
		Name:      "Monterey Peninsula Airport",
		City:      "Monterey",
		State:     "California",
		Country:   "US",
		Elevation: 257,
		Latitude:  36.5870018005,
		Longitude: -121.8430023193,
		Timezone:  "America/Los_Angeles",
	},
	"KMSL": {
		ICAO:      "KMSL",
		IATA:      "MSL",
		Name:      "Northwest Alabama Regional Airport",
		City:      "Muscle Shoals",
		State:     "Alabama",
		Country:   "US",
		Elevation: 551,
		Latitude:  34.74530029,
		Longitude: -87.61019897,
		Timezone:  "America/Chicago",
	},
	"KMSN": {
		ICAO:      "KMSN",
		IATA:      "MSN",
		Name:      "Dane County Regional Truax Field",
		City:      "Madison",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 887,
		Latitude:  43.1399002075,
		Longitude: -89.3375015259,
		Timezone:  "America/Chicago",
	},
	"KMSO": {
		ICAO:      "KMSO",
		IATA:      "MSO",
		Name:      "Missoula International Airport",
		City:      "Missoula",
		State:     "Montana",
		Country:   "US",
		Elevation: 3206,
		Latitude:  46.91630173,
		Longitude: -114.0910034,
		Timezone:  "America/Denver",
	},
	"KMSP": {
		ICAO:      "KMSP",
		IATA:      "MSP",
		Name:      "Minneapolis-St Paul International/Wold-Chamberlain Airport",
		City:      "Minneapolis",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 841,
		Latitude:  44.8819999695,
		Longitude: -93.2218017578,
		Timezone:  "America/Chicago",
	},
	"KMSS": {
		ICAO:      "KMSS",
		IATA:      "MSS",
		Name:      "Massena International Richards Field",
		City:      "Massena",
		State:     "New-York",
		Country:   "US",
		Elevation: 215,
		Latitude:  44.935798645,
		Longitude: -74.8455963135,
		Timezone:  "America/New_York",
	},
	"KMSV": {
		ICAO:      "KMSV",
		IATA:      "MSV",
		Name:      "Sullivan County International Airport",
		City:      "Monticello",
		State:     "New-York",
		Country:   "US",
		Elevation: 1403,
		Latitude:  41.70159912,
		Longitude: -74.79499817,
		Timezone:  "America/New_York",
	},
	"KMSY": {
		ICAO:      "KMSY",
		IATA:      "MSY",
		Name:      "Louis Armstrong New Orleans International Airport",
		City:      "New Orleans",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 4,
		Latitude:  29.9934005737,
		Longitude: -90.2580032349,
		Timezone:  "America/Chicago",
	},
	"KMTC": {
		ICAO:      "KMTC",
		IATA:      "MTC",
		Name:      "Selfridge Angb Airport",
		City:      "Mount Clemens",
		State:     "Michigan",
		Country:   "US",
		Elevation: 580,
		Latitude:  42.6082992554,
		Longitude: -82.8355026245,
		Timezone:  "America/Detroit",
	},
	"KMTH": {
		ICAO:      "KMTH",
		IATA:      "MTH",
		Name:      "The Florida Keys Marathon Airport",
		City:      "Marathon",
		State:     "Florida",
		Country:   "US",
		Elevation: 5,
		Latitude:  24.72610092,
		Longitude: -81.05139923,
		Timezone:  "America/New_York",
	},
	"KMTJ": {
		ICAO:      "KMTJ",
		IATA:      "MTJ",
		Name:      "Montrose Regional Airport",
		City:      "Montrose",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5759,
		Latitude:  38.5097999573,
		Longitude: -107.893997192,
		Timezone:  "America/Denver",
	},
	"KMTN": {
		ICAO:      "KMTN",
		IATA:      "MTN",
		Name:      "Martin State Airport",
		City:      "Baltimore",
		State:     "Maryland",
		Country:   "US",
		Elevation: 21,
		Latitude:  39.32569885,
		Longitude: -76.4138031,
		Timezone:  "America/New_York",
	},
	"KMTO": {
		ICAO:      "KMTO",
		IATA:      "MTO",
		Name:      "Coles County Memorial Airport",
		City:      "Mattoon/Charleston",
		State:     "Illinois",
		Country:   "US",
		Elevation: 722,
		Latitude:  39.47790146,
		Longitude: -88.27919769,
		Timezone:  "America/Chicago",
	},
	"KMTP": {
		ICAO:      "KMTP",
		IATA:      "MTP",
		Name:      "Montauk Airport",
		City:      "Montauk",
		State:     "New-York",
		Country:   "US",
		Elevation: 6,
		Latitude:  41.076499939,
		Longitude: -71.9207992554,
		Timezone:  "America/New_York",
	},
	"KMTV": {
		ICAO:      "KMTV",
		Name:      "Blue Ridge Airport",
		City:      "Martinsville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 941,
		Latitude:  36.6306991577,
		Longitude: -80.0183029175,
		Timezone:  "America/New_York",
	},
	"KMTW": {
		ICAO:      "KMTW",
		IATA:      "MTW",
		Name:      "Manitowoc County Airport",
		City:      "Manitowoc",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 651,
		Latitude:  44.1287994385,
		Longitude: -87.6806030273,
		Timezone:  "America/Chicago",
	},
	"KMUI": {
		ICAO:      "KMUI",
		IATA:      "MUI",
		Name:      "Muir Army Air Field (Fort Indiantown Gap) Airport",
		City:      "Fort Indiantown Gap(Annville)",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 488,
		Latitude:  40.43479919,
		Longitude: -76.56939697,
		Timezone:  "America/New_York",
	},
	"KMUL": {
		ICAO:      "KMUL",
		IATA:      "MUL",
		Name:      "Spence Airport",
		City:      "Moultrie",
		State:     "Georgia",
		Country:   "US",
		Elevation: 292,
		Latitude:  31.1376991272,
		Longitude: -83.7041015625,
		Timezone:  "America/New_York",
	},
	"KMUO": {
		ICAO:      "KMUO",
		IATA:      "MUO",
		Name:      "Mountain Home Air Force Base",
		City:      "Mountain Home",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2996,
		Latitude:  43.043598175,
		Longitude: -115.8720016479,
		Timezone:  "America/Boise",
	},
	"KMUT": {
		ICAO:      "KMUT",
		IATA:      "MUT",
		Name:      "Muscatine Municipal Airport",
		City:      "Muscatine",
		State:     "Iowa",
		Country:   "US",
		Elevation: 547,
		Latitude:  41.3678016663,
		Longitude: -91.1482009888,
		Timezone:  "America/Chicago",
	},
	"KMVC": {
		ICAO:      "KMVC",
		IATA:      "MVC",
		Name:      "Monroe County Airport",
		City:      "Monroeville",
		State:     "Alabama",
		Country:   "US",
		Elevation: 419,
		Latitude:  31.4580001831,
		Longitude: -87.3509979248,
		Timezone:  "America/Chicago",
	},
	"KMVE": {
		ICAO:      "KMVE",
		IATA:      "MVE",
		Name:      "Montevideo Chippewa County Airport",
		City:      "Montevideo",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1034,
		Latitude:  44.9691009521,
		Longitude: -95.7102966309,
		Timezone:  "America/Chicago",
	},
	"KMVI": {
		ICAO:      "KMVI",
		Name:      "Monte Vista Municipal Airport",
		City:      "Monte Vista",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7611,
		Latitude:  37.5273017883,
		Longitude: -106.047996521,
		Timezone:  "America/Denver",
	},
	"KMVL": {
		ICAO:      "KMVL",
		IATA:      "MVL",
		Name:      "Morrisville Stowe State Airport",
		City:      "Morrisville",
		State:     "Vermont",
		Country:   "US",
		Elevation: 732,
		Latitude:  44.5345993042,
		Longitude: -72.6139984131,
		Timezone:  "America/New_York",
	},
	"KMVM": {
		ICAO:      "KMVM",
		Name:      "Machias Valley Airport",
		City:      "Machias",
		State:     "Maine",
		Country:   "US",
		Elevation: 96,
		Latitude:  44.7030982971,
		Longitude: -67.4785995483,
		Timezone:  "America/New_York",
	},
	"KMVN": {
		ICAO:      "KMVN",
		IATA:      "MVN",
		Name:      "Mount Vernon Airport",
		City:      "Mount Vernon",
		State:     "Illinois",
		Country:   "US",
		Elevation: 480,
		Latitude:  38.3233985901,
		Longitude: -88.8588027954,
		Timezone:  "America/Chicago",
	},
	"KMVY": {
		ICAO:      "KMVY",
		IATA:      "MVY",
		Name:      "Martha's Vineyard Airport",
		City:      "Martha's Vineyard",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 67,
		Latitude:  41.3931007385,
		Longitude: -70.6143035889,
		Timezone:  "America/New_York",
	},
	"KMWA": {
		ICAO:      "KMWA",
		IATA:      "MWA",
		Name:      "Williamson County Regional Airport",
		City:      "Marion",
		State:     "Illinois",
		Country:   "US",
		Elevation: 472,
		Latitude:  37.75500107,
		Longitude: -89.01110077,
		Timezone:  "America/Chicago",
	},
	"KMWC": {
		ICAO:      "KMWC",
		IATA:      "MWC",
		Name:      "Lawrence J Timmerman Airport",
		City:      "Milwaukee",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 745,
		Latitude:  43.1104011536,
		Longitude: -88.0344009399,
		Timezone:  "America/Chicago",
	},
	"KMWH": {
		ICAO:      "KMWH",
		IATA:      "MWH",
		Name:      "Grant Co International Airport",
		City:      "Moses Lake",
		State:     "Washington",
		Country:   "US",
		Elevation: 1189,
		Latitude:  47.20769882,
		Longitude: -119.3199997,
		Timezone:  "America/Los_Angeles",
	},
	"KMWK": {
		ICAO:      "KMWK",
		Name:      "Mount Airy Surry County Airport",
		City:      "Mount Airy",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 1249,
		Latitude:  36.45970154,
		Longitude: -80.5530014,
		Timezone:  "America/New_York",
	},
	"KMWL": {
		ICAO:      "KMWL",
		IATA:      "MWL",
		Name:      "Mineral Wells Airport",
		City:      "Mineral Wells",
		State:     "Texas",
		Country:   "US",
		Elevation: 974,
		Latitude:  32.7816009521,
		Longitude: -98.0602035522,
		Timezone:  "America/Chicago",
	},
	"KMWM": {
		ICAO:      "KMWM",
		IATA:      "MWM",
		Name:      "Windom Municipal Airport",
		City:      "Windom",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1410,
		Latitude:  43.9133987427,
		Longitude: -95.1093978882,
		Timezone:  "America/Chicago",
	},
	"KMWO": {
		ICAO:      "KMWO",
		IATA:      "MWO",
		Name:      "Middletown Regional-Hook Field Municipal Airport",
		City:      "Middletown",
		State:     "Ohio",
		Country:   "US",
		Elevation: 650,
		Latitude:  39.53099823,
		Longitude: -84.39530182,
		Timezone:  "America/New_York",
	},
	"KMXA": {
		ICAO:      "KMXA",
		IATA:      "MXA",
		Name:      "Manila Municipal Airport",
		City:      "Manila",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 242,
		Latitude:  35.8944015503,
		Longitude: -90.1546020508,
		Timezone:  "America/Chicago",
	},
	"KMXF": {
		ICAO:      "KMXF",
		IATA:      "MXF",
		Name:      "Maxwell Air Force Base",
		City:      "Montgomery",
		State:     "Alabama",
		Country:   "US",
		Elevation: 171,
		Latitude:  32.382900238,
		Longitude: -86.3657989502,
		Timezone:  "America/Chicago",
	},
	"KMXO": {
		ICAO:      "KMXO",
		IATA:      "MXO",
		Name:      "Monticello Regional Airport",
		City:      "Monticello",
		State:     "Iowa",
		Country:   "US",
		Elevation: 832,
		Latitude:  42.22040176,
		Longitude: -91.16329956,
		Timezone:  "America/Chicago",
	},
	"KMYF": {
		ICAO:      "KMYF",
		IATA:      "MYF",
		Name:      "Montgomery Field",
		City:      "San Diego",
		State:     "California",
		Country:   "US",
		Elevation: 427,
		Latitude:  32.815700531,
		Longitude: -117.1399993896,
		Timezone:  "America/Los_Angeles",
	},
	"KMYJ": {
		ICAO:      "KMYJ",
		Name:      "Mexico Memorial Airport",
		City:      "Mexico",
		State:     "Missouri",
		Country:   "US",
		Elevation: 823,
		Latitude:  39.1575012207,
		Longitude: -91.8182983398,
		Timezone:  "America/Chicago",
	},
	"KMYL": {
		ICAO:      "KMYL",
		IATA:      "MYL",
		Name:      "Mc Call Municipal Airport",
		City:      "Mc Call",
		State:     "Idaho",
		Country:   "US",
		Elevation: 5024,
		Latitude:  44.88970184,
		Longitude: -116.1009979,
		Timezone:  "America/Boise",
	},
	"KMYR": {
		ICAO:      "KMYR",
		IATA:      "MYR",
		Name:      "Myrtle Beach International Airport",
		City:      "Myrtle Beach",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 25,
		Latitude:  33.6796989441,
		Longitude: -78.9282989502,
		Timezone:  "America/New_York",
	},
	"KMYV": {
		ICAO:      "KMYV",
		IATA:      "MYV",
		Name:      "Yuba County Airport",
		City:      "Marysville",
		State:     "California",
		Country:   "US",
		Elevation: 64,
		Latitude:  39.09780121,
		Longitude: -121.5699997,
		Timezone:  "America/Los_Angeles",
	},
	"KMYZ": {
		ICAO:      "KMYZ",
		Name:      "Marysville Municipal Airport",
		City:      "Marysville",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1283,
		Latitude:  39.8553009033,
		Longitude: -96.6305999756,
		Timezone:  "America/Chicago",
	},
	"KMZH": {
		ICAO:      "KMZH",
		Name:      "Moose Lake Carlton County Airport",
		City:      "Moose Lake",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1076,
		Latitude:  46.418800354,
		Longitude: -92.8047027588,
		Timezone:  "America/Chicago",
	},
	"KMZJ": {
		ICAO:      "KMZJ",
		IATA:      "MZJ",
		Name:      "Pinal Airpark",
		City:      "Marana",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1893,
		Latitude:  32.5106010437,
		Longitude: -111.3280029297,
		Timezone:  "America/Phoenix",
	},
	"KMZZ": {
		ICAO:      "KMZZ",
		IATA:      "MZZ",
		Name:      "Marion Municipal Airport",
		City:      "Marion",
		State:     "Indiana",
		Country:   "US",
		Elevation: 859,
		Latitude:  40.4898986816,
		Longitude: -85.6797027588,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KN03": {
		ICAO:      "KN03",
		Name:      "Cortland County Chase Field",
		City:      "Cortland",
		State:     "New-York",
		Country:   "US",
		Elevation: 1198,
		Latitude:  42.59260178,
		Longitude: -76.21489716,
		Timezone:  "America/New_York",
	},
	"KN06": {
		ICAO:      "KN06",
		Name:      "Laurel Airport",
		City:      "Laurel",
		State:     "Delaware",
		Country:   "US",
		Elevation: 30,
		Latitude:  38.5421981812,
		Longitude: -75.5943984985,
		Timezone:  "America/New_York",
	},
	"KN12": {
		ICAO:      "KN12",
		Name:      "Lakewood Airport",
		City:      "Lakewood",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 43,
		Latitude:  40.0667991638,
		Longitude: -74.177696228,
		Timezone:  "America/New_York",
	},
	"KN13": {
		ICAO:      "KN13",
		Name:      "Bloomsburg Municipal Airport",
		City:      "Bloomsburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 481,
		Latitude:  40.9977989197,
		Longitude: -76.4360961914,
		Timezone:  "America/New_York",
	},
	"KN14": {
		ICAO:      "KN14",
		Name:      "Flying W Airport",
		City:      "Lumberton",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 49,
		Latitude:  39.934299469,
		Longitude: -74.8072967529,
		Timezone:  "America/New_York",
	},
	"KN19": {
		ICAO:      "KN19",
		Name:      "Aztec Municipal Airport",
		City:      "Aztec",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 5877,
		Latitude:  36.8347015381,
		Longitude: -108.0279998779,
		Timezone:  "America/Denver",
	},
	"KN23": {
		ICAO:      "KN23",
		Name:      "Sidney Municipal Airport",
		City:      "Sidney",
		State:     "New-York",
		Country:   "US",
		Elevation: 1027,
		Latitude:  42.3026008606,
		Longitude: -75.4160003662,
		Timezone:  "America/New_York",
	},
	"KN24": {
		ICAO:      "KN24",
		Name:      "Questa Municipal Nr 2 Airport",
		City:      "Questa",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 7700,
		Latitude:  36.8003005981,
		Longitude: -105.5979995728,
		Timezone:  "America/Denver",
	},
	"KN27": {
		ICAO:      "KN27",
		Name:      "Bradford County Airport",
		City:      "Towanda",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 730,
		Latitude:  41.7400016785,
		Longitude: -76.4470977783,
		Timezone:  "America/New_York",
	},
	"KN35": {
		ICAO:      "KN35",
		Name:      "Punxsutawney Municipal Airport",
		City:      "Punxsutawney",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1439,
		Latitude:  40.9667015076,
		Longitude: -78.9300003052,
		Timezone:  "America/New_York",
	},
	"KN37": {
		ICAO:      "KN37",
		Name:      "Monticello Airport",
		City:      "Monticello",
		State:     "New-York",
		Country:   "US",
		Elevation: 1545,
		Latitude:  41.6225013733,
		Longitude: -74.7013015747,
		Timezone:  "America/New_York",
	},
	"KN38": {
		ICAO:      "KN38",
		Name:      "Wellsboro Johnston Airport",
		City:      "Wellsboro",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1892,
		Latitude:  41.72790146,
		Longitude: -77.39649963,
		Timezone:  "America/New_York",
	},
	"KN40": {
		ICAO:      "KN40",
		Name:      "Sky Manor Airport",
		City:      "Pittstown",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 560,
		Latitude:  40.5662994385,
		Longitude: -74.9785995483,
		Timezone:  "America/New_York",
	},
	"KN47": {
		ICAO:      "KN47",
		Name:      "Pottstown Municipal Airport",
		City:      "Pottstown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 256,
		Latitude:  40.2602996826,
		Longitude: -75.6707992554,
		Timezone:  "America/New_York",
	},
	"KN51": {
		ICAO:      "KN51",
		Name:      "Solberg Hunterdon Airport",
		City:      "Readington",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 195,
		Latitude:  40.5836982727,
		Longitude: -74.7361984253,
		Timezone:  "America/New_York",
	},
	"KN52": {
		ICAO:      "KN52",
		Name:      "Jaars Townsend Airport",
		City:      "Waxhaw",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 602,
		Latitude:  34.8638000488,
		Longitude: -80.7480010986,
		Timezone:  "America/New_York",
	},
	"KN53": {
		ICAO:      "KN53",
		Name:      "Stroudsburg Pocono Airport",
		City:      "East Stroudsburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 480,
		Latitude:  41.0358009338,
		Longitude: -75.1605987549,
		Timezone:  "America/New_York",
	},
	"KN57": {
		ICAO:      "KN57",
		Name:      "New Garden Airport",
		City:      "Toughkenamon",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 436,
		Latitude:  39.8305015564,
		Longitude: -75.7696990967,
		Timezone:  "America/New_York",
	},
	"KN58": {
		ICAO:      "KN58",
		Name:      "Tiger Field",
		City:      "Fernley",
		State:     "Nevada",
		Country:   "US",
		Elevation: 4346,
		Latitude:  39.5596008301,
		Longitude: -119.2409973145,
		Timezone:  "America/Los_Angeles",
	},
	"KN59": {
		ICAO:      "KN59",
		Name:      "Rosaschi Air Park",
		City:      "Smith",
		State:     "Nevada",
		Country:   "US",
		Elevation: 4809,
		Latitude:  38.8394012451,
		Longitude: -119.3379974365,
		Timezone:  "America/Los_Angeles",
	},
	"KN66": {
		ICAO:      "KN66",
		Name:      "Oneonta Municipal Airport",
		City:      "Oneonta",
		State:     "New-York",
		Country:   "US",
		Elevation: 1763,
		Latitude:  42.52470016,
		Longitude: -75.06439972,
		Timezone:  "America/New_York",
	},
	"KN68": {
		ICAO:      "KN68",
		Name:      "Franklin County Regional Airport",
		City:      "Chambersburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 697,
		Latitude:  39.97309875,
		Longitude: -77.64330292,
		Timezone:  "America/New_York",
	},
	"KN69": {
		ICAO:      "KN69",
		Name:      "Stormville Airport",
		City:      "Stormville",
		State:     "New-York",
		Country:   "US",
		Elevation: 358,
		Latitude:  41.5769996643,
		Longitude: -73.7323989868,
		Timezone:  "America/New_York",
	},
	"KN71": {
		ICAO:      "KN71",
		Name:      "Donegal Springs Airpark",
		City:      "Mount Joy/Marietta",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 458,
		Latitude:  40.0922012329,
		Longitude: -76.5744018555,
		Timezone:  "America/New_York",
	},
	"KN73": {
		ICAO:      "KN73",
		Name:      "Red Lion Airport",
		City:      "Vincentown",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 54,
		Latitude:  39.9042015076,
		Longitude: -74.74949646,
		Timezone:  "America/New_York",
	},
	"KN79": {
		ICAO:      "KN79",
		Name:      "Northumberland County Airport",
		City:      "Shamokin",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 881,
		Latitude:  40.8368988037,
		Longitude: -76.5524978638,
		Timezone:  "America/New_York",
	},
	"KN81": {
		ICAO:      "KN81",
		Name:      "Hammonton Municipal Airport",
		City:      "Hammonton",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 65,
		Latitude:  39.66749954,
		Longitude: -74.75769806,
		Timezone:  "America/New_York",
	},
	"KN82": {
		ICAO:      "KN82",
		Name:      "Wurtsboro Sullivan County Airport",
		City:      "Wurtsboro",
		State:     "New-York",
		Country:   "US",
		Elevation: 560,
		Latitude:  41.5971984863,
		Longitude: -74.4582977295,
		Timezone:  "America/New_York",
	},
	"KN94": {
		ICAO:      "KN94",
		Name:      "Carlisle Airport",
		City:      "Carlisle",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 510,
		Latitude:  40.1879005432,
		Longitude: -77.1743011475,
		Timezone:  "America/New_York",
	},
	"KN96": {
		ICAO:      "KN96",
		Name:      "Bellefonte Airport",
		City:      "Bellefonte",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1071,
		Latitude:  40.8855018616,
		Longitude: -77.8162994385,
		Timezone:  "America/New_York",
	},
	"KN98": {
		ICAO:      "KN98",
		Name:      "Boyne City Municipal Airport",
		City:      "Boyne City",
		State:     "Michigan",
		Country:   "US",
		Elevation: 657,
		Latitude:  45.2083015442,
		Longitude: -84.9903030396,
		Timezone:  "America/Detroit",
	},
	"KNBC": {
		ICAO:      "KNBC",
		Name:      "Beaufort MCAS - Merritt Field",
		City:      "Beaufort",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 37,
		Latitude:  32.4774017334,
		Longitude: -80.723197937,
		Timezone:  "America/New_York",
	},
	"KNBG": {
		ICAO:      "KNBG",
		IATA:      "NBG",
		Name:      "New Orleans Nas Jrb/Alvin Callender Field/ Airport",
		City:      "New Orleans",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 2,
		Latitude:  29.82530022,
		Longitude: -90.03500366,
		Timezone:  "America/Chicago",
	},
	"KNBJ": {
		ICAO:      "KNBJ",
		IATA:      "NHX",
		Name:      "Barin Nolf Airport",
		City:      "Foley",
		State:     "Alabama",
		Country:   "US",
		Elevation: 54,
		Latitude:  30.3890991211,
		Longitude: -87.6352996826,
		Timezone:  "America/Chicago",
	},
	"KNCA": {
		ICAO:      "KNCA",
		Name:      "New River MCAS /H/ /Mccutcheon Fld/ Airport",
		City:      "Jacksonville",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 26,
		Latitude:  34.70840073,
		Longitude: -77.43969727,
		Timezone:  "America/New_York",
	},
	"KNDY": {
		ICAO:      "KNDY",
		IATA:      "DGN",
		Name:      "Dahlgren Naval Surface Warfare Center Airport",
		City:      "Dahlgren",
		State:     "Virginia",
		Country:   "US",
		Elevation: 18,
		Latitude:  38.33250046,
		Longitude: -77.03720093,
		Timezone:  "America/New_York",
	},
	"KNDZ": {
		ICAO:      "KNDZ",
		Name:      "Whiting Field Naval Air Station South Airport",
		City:      "Milton",
		State:     "Florida",
		Country:   "US",
		Elevation: 177,
		Latitude:  30.7043991089,
		Longitude: -87.0230026245,
		Timezone:  "America/Chicago",
	},
	"KNEL": {
		ICAO:      "KNEL",
		IATA:      "NEL",
		Name:      "Lakehurst NAES /Maxfield Field",
		City:      "Lakehurst",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 101,
		Latitude:  40.03329849,
		Longitude: -74.353302,
		Timezone:  "America/New_York",
	},
	"KNEN": {
		ICAO:      "KNEN",
		IATA:      "NEN",
		Name:      "Whitehouse Nolf Airport",
		City:      "Jacksonville",
		State:     "Florida",
		Country:   "US",
		Elevation: 99,
		Latitude:  30.3539009094,
		Longitude: -81.8719024658,
		Timezone:  "America/New_York",
	},
	"KNEW": {
		ICAO:      "KNEW",
		IATA:      "NEW",
		Name:      "Lakefront Airport",
		City:      "New Orleans",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 8,
		Latitude:  30.0424003601,
		Longitude: -90.0282974243,
		Timezone:  "America/Chicago",
	},
	"KNFD": {
		ICAO:      "KNFD",
		Name:      "Summerdale Nolf Airport",
		City:      "Summerdale",
		State:     "Alabama",
		Country:   "US",
		Elevation: 149,
		Latitude:  30.5076999664,
		Longitude: -87.6455001831,
		Timezone:  "America/Chicago",
	},
	"KNFE": {
		ICAO:      "KNFE",
		Name:      "Fentress Naval Auxiliary Landing Field",
		City:      "Fentress",
		State:     "Virginia",
		Country:   "US",
		Elevation: 16,
		Latitude:  36.69509888,
		Longitude: -76.13549805,
		Timezone:  "America/New_York",
	},
	"KNFG": {
		ICAO:      "KNFG",
		Name:      "Camp Pendleton MCAS (Munn Field) Airport",
		City:      "Oceanside",
		State:     "California",
		Country:   "US",
		Elevation: 78,
		Latitude:  33.30130005,
		Longitude: -117.3550034,
		Timezone:  "America/Los_Angeles",
	},
	"KNFJ": {
		ICAO:      "KNFJ",
		Name:      "Choctaw Nolf Airport",
		City:      "Milton",
		State:     "Florida",
		Country:   "US",
		Elevation: 102,
		Latitude:  30.5069007874,
		Longitude: -86.9597015381,
		Timezone:  "America/Chicago",
	},
	"KNFL": {
		ICAO:      "KNFL",
		IATA:      "NFL",
		Name:      "Fallon Naval Air Station /Van Voorhis Field",
		City:      "Fallon",
		State:     "Nevada",
		Country:   "US",
		Elevation: 3934,
		Latitude:  39.41659927,
		Longitude: -118.7009964,
		Timezone:  "America/Los_Angeles",
	},
	"KNFW": {
		ICAO:      "KNFW",
		IATA:      "FWH",
		Name:      "Fort Worth Nas Jrb/Carswell field",
		City:      "Fort Worth",
		State:     "Texas",
		Country:   "US",
		Elevation: 650,
		Latitude:  32.76919937,
		Longitude: -97.4414978,
		Timezone:  "America/Chicago",
	},
	"KNGP": {
		ICAO:      "KNGP",
		IATA:      "NGP",
		Name:      "Corpus Christi Naval Air Station/Truax Field",
		City:      "Corpus Christi",
		State:     "Texas",
		Country:   "US",
		Elevation: 18,
		Latitude:  27.69260025,
		Longitude: -97.29109955,
		Timezone:  "America/Chicago",
	},
	"KNGS": {
		ICAO:      "KNGS",
		Name:      "Santa Rosa Nolf Airport",
		City:      "Milton",
		State:     "Florida",
		Country:   "US",
		Elevation: 150,
		Latitude:  30.6107997894,
		Longitude: -86.9400024414,
		Timezone:  "America/Chicago",
	},
	"KNGU": {
		ICAO:      "KNGU",
		IATA:      "NGU",
		Name:      "Norfolk Ns (Chambers Fld) Airport",
		City:      "Norfolk",
		State:     "Virginia",
		Country:   "US",
		Elevation: 17,
		Latitude:  36.93759918,
		Longitude: -76.28929901,
		Timezone:  "America/New_York",
	},
	"KNGW": {
		ICAO:      "KNGW",
		IATA:      "NGW",
		Name:      "Cabaniss Field Nolf Airport",
		City:      "Corpus Christi",
		State:     "Texas",
		Country:   "US",
		Elevation: 30,
		Latitude:  27.7003002167,
		Longitude: -97.4344024658,
		Timezone:  "America/Chicago",
	},
	"KNHK": {
		ICAO:      "KNHK",
		IATA:      "NHK",
		Name:      "Patuxent River Naval Air Station/Trapnell Field Aiport",
		City:      "Patuxent River",
		State:     "Maryland",
		Country:   "US",
		Elevation: 39,
		Latitude:  38.2859993,
		Longitude: -76.41179657,
		Timezone:  "America/New_York",
	},
	"KNHL": {
		ICAO:      "KNHL",
		Name:      "Wolf Nolf Airport",
		City:      "Joesphine",
		State:     "Alabama",
		Country:   "US",
		Elevation: 61,
		Latitude:  30.3460998535,
		Longitude: -87.5417022705,
		Timezone:  "America/Chicago",
	},
	"KNHZ": {
		ICAO:      "KNHZ",
		Name:      "Brunswick Naval Air Station",
		City:      "Brunswick",
		State:     "Maine",
		Country:   "US",
		Elevation: 72,
		Latitude:  43.89220047,
		Longitude: -69.93859863,
		Timezone:  "America/New_York",
	},
	"KNID": {
		ICAO:      "KNID",
		Name:      "China Lake Naws (Armitage Field) Airport",
		City:      "China Lake",
		State:     "California",
		Country:   "US",
		Elevation: 2283,
		Latitude:  35.6853981,
		Longitude: -117.6920013,
		Timezone:  "America/Los_Angeles",
	},
	"KNIP": {
		ICAO:      "KNIP",
		IATA:      "NIP",
		Name:      "Jacksonville Naval Air Station",
		City:      "Jacksonville",
		State:     "Florida",
		Country:   "US",
		Elevation: 21,
		Latitude:  30.23579979,
		Longitude: -81.68060303,
		Timezone:  "America/New_York",
	},
	"KNJK": {
		ICAO:      "KNJK",
		IATA:      "NJK",
		Name:      "El Centro Naf Airport",
		City:      "El Centro",
		State:     "California",
		Country:   "US",
		Elevation: -42,
		Latitude:  32.8292007446,
		Longitude: -115.6719970703,
		Timezone:  "America/Los_Angeles",
	},
	"KNJM": {
		ICAO:      "KNJM",
		Name:      "Bogue Field Mcalf Airport",
		City:      "Swansboro",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 21,
		Latitude:  34.69039917,
		Longitude: -77.02970123,
		Timezone:  "America/New_York",
	},
	"KNJW": {
		ICAO:      "KNJW",
		Name:      "Joe Williams Nolf Airport",
		City:      "Meridian",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 539,
		Latitude:  32.7961998,
		Longitude: -88.83170319,
		Timezone:  "America/Chicago",
	},
	"KNKL": {
		ICAO:      "KNKL",
		Name:      "Holley Nolf Airport",
		City:      "Fort Walton Beach",
		State:     "Florida",
		Country:   "US",
		Elevation: 39,
		Latitude:  30.4253005981,
		Longitude: -86.8938980103,
		Timezone:  "America/Chicago",
	},
	"KNKT": {
		ICAO:      "KNKT",
		Name:      "Cherry Point MCAS /Cunningham Field/",
		City:      "Cherry Point",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 29,
		Latitude:  34.90090179,
		Longitude: -76.88069916,
		Timezone:  "America/New_York",
	},
	"KNKX": {
		ICAO:      "KNKX",
		IATA:      "NKX",
		Name:      "Miramar Marine Corps Air Station",
		City:      "San Diego",
		State:     "California",
		Country:   "US",
		Elevation: 477,
		Latitude:  32.86840057,
		Longitude: -117.1429977,
		Timezone:  "America/Los_Angeles",
	},
	"KNLC": {
		ICAO:      "KNLC",
		IATA:      "NLC",
		Name:      "Lemoore Naval Air Station (Reeves Field) Airport",
		City:      "Lemoore",
		State:     "California",
		Country:   "US",
		Elevation: 232,
		Latitude:  36.33300018,
		Longitude: -119.9520035,
		Timezone:  "America/Los_Angeles",
	},
	"KNMM": {
		ICAO:      "KNMM",
		Name:      "Meridian Naval Air Station",
		City:      "Meridian",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 316,
		Latitude:  32.55210114,
		Longitude: -88.55560303,
		Timezone:  "America/Chicago",
	},
	"KNOG": {
		ICAO:      "KNOG",
		Name:      "Orange Grove Naval Auxiliary Landing Field",
		City:      "Orange Grove",
		State:     "Texas",
		Country:   "US",
		Elevation: 257,
		Latitude:  27.9011001587,
		Longitude: -98.0516967773,
		Timezone:  "America/Chicago",
	},
	"KNOW": {
		ICAO:      "KNOW",
		Name:      "Port Angeles Cgas Airport",
		City:      "Port Angeles",
		State:     "Washington",
		Country:   "US",
		Elevation: 13,
		Latitude:  48.1414985657,
		Longitude: -123.4140014648,
		Timezone:  "America/Los_Angeles",
	},
	"KNPA": {
		ICAO:      "KNPA",
		IATA:      "NPA",
		Name:      "Pensacola Naval Air Station/Forrest Sherman field",
		City:      "Pensacola",
		State:     "Florida",
		Country:   "US",
		Elevation: 28,
		Latitude:  30.35269928,
		Longitude: -87.31860352,
		Timezone:  "America/Chicago",
	},
	"KNQA": {
		ICAO:      "KNQA",
		IATA:      "NQA",
		Name:      "Millington Regional Jetport Airport",
		City:      "Millington",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 320,
		Latitude:  35.3567008972,
		Longitude: -89.870300293,
		Timezone:  "America/Chicago",
	},
	"KNQB": {
		ICAO:      "KNQB",
		Name:      "Silverhill Nolf Airport",
		State:     "Alabama",
		Country:   "US",
		Elevation: 129,
		Latitude:  30.5636005402,
		Longitude: -87.8097000122,
		Timezone:  "America/Chicago",
	},
	"KNQI": {
		ICAO:      "KNQI",
		IATA:      "NQI",
		Name:      "Kingsville Naval Air Station",
		City:      "Kingsville",
		State:     "Texas",
		Country:   "US",
		Elevation: 50,
		Latitude:  27.5072002411,
		Longitude: -97.8097000122,
		Timezone:  "America/Chicago",
	},
	"KNQX": {
		ICAO:      "KNQX",
		IATA:      "NQX",
		Name:      "Key West Nas /Boca Chica Field/ Airport",
		City:      "Key West",
		State:     "Florida",
		Country:   "US",
		Elevation: 6,
		Latitude:  24.57579994,
		Longitude: -81.68890381,
		Timezone:  "America/New_York",
	},
	"KNRA": {
		ICAO:      "KNRA",
		Name:      "Coupeville Nolf Airport",
		City:      "Coupeville",
		State:     "Washington",
		Country:   "US",
		Elevation: 199,
		Latitude:  48.1883010864,
		Longitude: -122.6320037842,
		Timezone:  "America/Los_Angeles",
	},
	"KNRB": {
		ICAO:      "KNRB",
		IATA:      "NRB",
		Name:      "Mayport Ns (Adm David L. Mcdonald Field) Airport",
		City:      "Mayport",
		State:     "Florida",
		Country:   "US",
		Elevation: 15,
		Latitude:  30.39109993,
		Longitude: -81.42469788,
		Timezone:  "America/New_York",
	},
	"KNRN": {
		ICAO:      "KNRN",
		Name:      "Norton Municipal Airport",
		City:      "Norton",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2383,
		Latitude:  39.8503990173,
		Longitude: -99.8946990967,
		Timezone:  "America/Chicago",
	},
	"KNRQ": {
		ICAO:      "KNRQ",
		Name:      "Spencer Nolf Airport",
		City:      "Pace",
		State:     "Florida",
		Country:   "US",
		Elevation: 151,
		Latitude:  30.6252994537,
		Longitude: -87.1399993896,
		Timezone:  "America/Chicago",
	},
	"KNRS": {
		ICAO:      "KNRS",
		IATA:      "NRS",
		Name:      "Imperial Beach Nolf (Ream Fld) Airport",
		City:      "Imperial Beach",
		State:     "California",
		Country:   "US",
		Elevation: 24,
		Latitude:  32.56669998,
		Longitude: -117.1169968,
		Timezone:  "America/Los_Angeles",
	},
	"KNSE": {
		ICAO:      "KNSE",
		IATA:      "NSE",
		Name:      "Whiting Field Naval Air Station North Airport",
		City:      "Milton",
		State:     "Florida",
		Country:   "US",
		Elevation: 199,
		Latitude:  30.7241993,
		Longitude: -87.02189636,
		Timezone:  "America/Chicago",
	},
	"KNSI": {
		ICAO:      "KNSI",
		Name:      "San Nicolas Island Nolf Airport",
		City:      "San Nicolas Island",
		State:     "California",
		Country:   "US",
		Elevation: 506,
		Latitude:  33.2397994995,
		Longitude: -119.4580001831,
		Timezone:  "America/Los_Angeles",
	},
	"KNTD": {
		ICAO:      "KNTD",
		IATA:      "NTD",
		Name:      "Point Mugu Naval Air Station (Naval Base Ventura Co)",
		City:      "Point Mugu",
		State:     "California",
		Country:   "US",
		Elevation: 13,
		Latitude:  34.120300293,
		Longitude: -119.121002197,
		Timezone:  "America/Los_Angeles",
	},
	"KNTU": {
		ICAO:      "KNTU",
		IATA:      "NTU",
		Name:      "Oceana Nas /Apollo Soucek Field/ Station",
		City:      "Virginia Beach",
		State:     "Virginia",
		Country:   "US",
		Elevation: 23,
		Latitude:  36.8207016,
		Longitude: -76.03350067,
		Timezone:  "America/New_York",
	},
	"KNUC": {
		ICAO:      "KNUC",
		Name:      "San Clemente Island Naval Auxiliary Landing Field",
		City:      "San Clemente Island",
		State:     "California",
		Country:   "US",
		Elevation: 184,
		Latitude:  33.02270126,
		Longitude: -118.5879974,
		Timezone:  "America/Los_Angeles",
	},
	"KNUI": {
		ICAO:      "KNUI",
		Name:      "Webster Nolf Airport",
		City:      "St Inigoes",
		State:     "Maryland",
		Country:   "US",
		Elevation: 22,
		Latitude:  38.1459999084,
		Longitude: -76.4285964966,
		Timezone:  "America/New_York",
	},
	"KNUN": {
		ICAO:      "KNUN",
		IATA:      "NUN",
		Name:      "Saufley Field Nolf Airport",
		City:      "Pensacola",
		State:     "Florida",
		Country:   "US",
		Elevation: 85,
		Latitude:  30.4696998596,
		Longitude: -87.3380966187,
		Timezone:  "America/Chicago",
	},
	"KNUQ": {
		ICAO:      "KNUQ",
		IATA:      "NUQ",
		Name:      "Moffett Federal Airfield",
		City:      "Mountain View",
		State:     "California",
		Country:   "US",
		Elevation: 32,
		Latitude:  37.4160995483,
		Longitude: -122.0490036011,
		Timezone:  "America/Los_Angeles",
	},
	"KNUW": {
		ICAO:      "KNUW",
		IATA:      "NUW",
		Name:      "Whidbey Island Naval Air Station /Ault Field/ Airport",
		City:      "Oak Harbor",
		State:     "Washington",
		Country:   "US",
		Elevation: 47,
		Latitude:  48.35179901,
		Longitude: -122.6559982,
		Timezone:  "America/Los_Angeles",
	},
	"KNVD": {
		ICAO:      "KNVD",
		IATA:      "NVD",
		Name:      "Nevada Municipal Airport",
		City:      "Nevada",
		State:     "Missouri",
		Country:   "US",
		Elevation: 892,
		Latitude:  37.8521003723,
		Longitude: -94.304901123,
		Timezone:  "America/Chicago",
	},
	"KNWL": {
		ICAO:      "KNWL",
		Name:      "Waldron Field Nolf Airport",
		City:      "Corpus Christi",
		State:     "Texas",
		Country:   "US",
		Elevation: 25,
		Latitude:  27.6350002289,
		Longitude: -97.3122024536,
		Timezone:  "America/Chicago",
	},
	"KNXF": {
		ICAO:      "KNXF",
		Name:      "MCOLF Camp Pendleton (Red Beach) Airport",
		City:      "Oceanside",
		State:     "California",
		Country:   "US",
		Elevation: 89,
		Latitude:  33.285644,
		Longitude: -117.456478,
		Timezone:  "America/Los_Angeles",
	},
	"KNXP": {
		ICAO:      "KNXP",
		Name:      "Twentynine Palms (Self) Airport",
		City:      "Twentynine Palms",
		State:     "California",
		Country:   "US",
		Elevation: 2051,
		Latitude:  34.2961998,
		Longitude: -116.1620026,
		Timezone:  "America/Los_Angeles",
	},
	"KNXX": {
		ICAO:      "KNXX",
		Name:      "Willow Grove Naval Air Station/Joint Reserve Base",
		City:      "Willow Grove",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 358,
		Latitude:  40.19979858,
		Longitude: -75.14820099,
		Timezone:  "America/New_York",
	},
	"KNY0": {
		ICAO:      "KNY0",
		Name:      "Fulton County Airport",
		City:      "Johnstown",
		State:     "New-York",
		Country:   "US",
		Elevation: 881,
		Latitude:  42.9981994629,
		Longitude: -74.3295974731,
		Timezone:  "America/New_York",
	},
	"KNY2": {
		ICAO:      "KNY2",
		Name:      "Camillus Airport",
		City:      "Camillus",
		State:     "New-York",
		Country:   "US",
		Elevation: 405,
		Latitude:  43.0834007263,
		Longitude: -76.2829971313,
		Timezone:  "America/New_York",
	},
	"KNYG": {
		ICAO:      "KNYG",
		IATA:      "NYG",
		Name:      "Quantico MCAF /Turner field",
		City:      "Quantico",
		State:     "Virginia",
		Country:   "US",
		Elevation: 10,
		Latitude:  38.50170135,
		Longitude: -77.30529785,
		Timezone:  "America/New_York",
	},
	"KNYL": {
		ICAO:      "KNYL",
		IATA:      "YUM",
		Name:      "Yuma MCAS/Yuma International Airport",
		City:      "Yuma",
		State:     "Arizona",
		Country:   "US",
		Elevation: 213,
		Latitude:  32.65660095,
		Longitude: -114.6060028,
		Timezone:  "America/Phoenix",
	},
	"KNZY": {
		ICAO:      "KNZY",
		IATA:      "NZY",
		Name:      "North Island Naval Air Station-Halsey Field",
		City:      "San Diego",
		State:     "California",
		Country:   "US",
		Elevation: 26,
		Latitude:  32.69919968,
		Longitude: -117.2149963,
		Timezone:  "America/Los_Angeles",
	},
	"KO02": {
		ICAO:      "KO02",
		IATA:      "NVN",
		Name:      "Nervino Airport",
		City:      "Beckwourth",
		State:     "California",
		Country:   "US",
		Elevation: 4900,
		Latitude:  39.8185005188,
		Longitude: -120.352996826,
		Timezone:  "America/Los_Angeles",
	},
	"KO05": {
		ICAO:      "KO05",
		Name:      "Rogers Field",
		City:      "Chester",
		State:     "California",
		Country:   "US",
		Elevation: 4528,
		Latitude:  40.2823982239,
		Longitude: -121.2409973145,
		Timezone:  "America/Los_Angeles",
	},
	"KO08": {
		ICAO:      "KO08",
		Name:      "Colusa County Airport",
		City:      "Colusa",
		State:     "California",
		Country:   "US",
		Elevation: 50,
		Latitude:  39.17900085,
		Longitude: -121.9929962,
		Timezone:  "America/Los_Angeles",
	},
	"KO09": {
		ICAO:      "KO09",
		Name:      "Round Valley Airport",
		City:      "Covelo",
		State:     "California",
		Country:   "US",
		Elevation: 1434,
		Latitude:  39.7901992798,
		Longitude: -123.2659988403,
		Timezone:  "America/Los_Angeles",
	},
	"KO11": {
		ICAO:      "KO11",
		Name:      "Stilwell Cherokee Nation Airport",
		City:      "Stilwell",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1084,
		Latitude:  35.7569999695,
		Longitude: -94.6499023438,
		Timezone:  "America/Chicago",
	},
	"KO15": {
		ICAO:      "KO15",
		Name:      "Turlock Municipal Airport",
		City:      "Turlock",
		State:     "California",
		Country:   "US",
		Elevation: 159,
		Latitude:  37.4874000549,
		Longitude: -120.6969985962,
		Timezone:  "America/Los_Angeles",
	},
	"KO16": {
		ICAO:      "KO16",
		Name:      "Garberville Airport",
		City:      "Garberville",
		State:     "California",
		Country:   "US",
		Elevation: 546,
		Latitude:  40.0859985352,
		Longitude: -123.8140029907,
		Timezone:  "America/Los_Angeles",
	},
	"KO20": {
		ICAO:      "KO20",
		Name:      "Kingdon Airpark",
		City:      "Lodi",
		State:     "California",
		Country:   "US",
		Elevation: 15,
		Latitude:  38.0915985107,
		Longitude: -121.3590011597,
		Timezone:  "America/Los_Angeles",
	},
	"KO22": {
		ICAO:      "KO22",
		Name:      "Columbia Airport",
		City:      "Columbia",
		State:     "California",
		Country:   "US",
		Elevation: 2118,
		Latitude:  38.0303993225,
		Longitude: -120.4150009155,
		Timezone:  "America/Los_Angeles",
	},
	"KO26": {
		ICAO:      "KO26",
		Name:      "Lone Pine Airport",
		City:      "Lone Pine",
		State:     "California",
		Country:   "US",
		Elevation: 3680,
		Latitude:  36.5882987976,
		Longitude: -118.0520019531,
		Timezone:  "America/Los_Angeles",
	},
	"KO27": {
		ICAO:      "KO27",
		IATA:      "ODC",
		Name:      "Oakdale Airport",
		City:      "Oakdale",
		State:     "California",
		Country:   "US",
		Elevation: 237,
		Latitude:  37.75630188,
		Longitude: -120.8000031,
		Timezone:  "America/Los_Angeles",
	},
	"KO28": {
		ICAO:      "KO28",
		Name:      "Ells Field Willits Municipal Airport",
		City:      "Willits",
		State:     "California",
		Country:   "US",
		Elevation: 2063,
		Latitude:  39.4513015747,
		Longitude: -123.3720016479,
		Timezone:  "America/Los_Angeles",
	},
	"KO32": {
		ICAO:      "KO32",
		Name:      "Reedley Municipal Airport",
		City:      "Reedley",
		State:     "California",
		Country:   "US",
		Elevation: 383,
		Latitude:  36.6663017273,
		Longitude: -119.4499969482,
		Timezone:  "America/Los_Angeles",
	},
	"KO35": {
		ICAO:      "KO35",
		Name:      "Hollis Municipal Airport",
		City:      "Hollis",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1658,
		Latitude:  34.7084007263,
		Longitude: -99.9086990356,
		Timezone:  "America/Chicago",
	},
	"KO37": {
		ICAO:      "KO37",
		Name:      "Haigh Field",
		City:      "Orland",
		State:     "California",
		Country:   "US",
		Elevation: 215,
		Latitude:  39.7211990356,
		Longitude: -122.1470031738,
		Timezone:  "America/Los_Angeles",
	},
	"KO41": {
		ICAO:      "KO41",
		Name:      "Watts Woodland Airport",
		City:      "Woodland",
		State:     "California",
		Country:   "US",
		Elevation: 125,
		Latitude:  38.6739006042,
		Longitude: -121.8720016479,
		Timezone:  "America/Los_Angeles",
	},
	"KO42": {
		ICAO:      "KO42",
		Name:      "Woodlake Airport",
		City:      "Woodlake",
		State:     "California",
		Country:   "US",
		Elevation: 425,
		Latitude:  36.3987998962,
		Longitude: -119.1070022583,
		Timezone:  "America/Los_Angeles",
	},
	"KO43": {
		ICAO:      "KO43",
		IATA:      "EYR",
		Name:      "Yerington Municipal Airport",
		City:      "Yerington",
		State:     "Nevada",
		Country:   "US",
		Elevation: 4378,
		Latitude:  39.0041007996,
		Longitude: -119.157997131,
		Timezone:  "America/Los_Angeles",
	},
	"KO46": {
		ICAO:      "KO46",
		Name:      "Weed Airport",
		City:      "Weed",
		State:     "California",
		Country:   "US",
		Elevation: 2938,
		Latitude:  41.474899292,
		Longitude: -122.4530029297,
		Timezone:  "America/Los_Angeles",
	},
	"KO47": {
		ICAO:      "KO47",
		Name:      "Prague Municipal Airport",
		City:      "Prague",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1042,
		Latitude:  35.4822998047,
		Longitude: -96.7186965942,
		Timezone:  "America/Chicago",
	},
	"KO51": {
		ICAO:      "KO51",
		Name:      "Laverne Municipal Airport",
		City:      "Laverne",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 2112,
		Latitude:  36.7444992065,
		Longitude: -99.9086990356,
		Timezone:  "America/Chicago",
	},
	"KO52": {
		ICAO:      "KO52",
		Name:      "Sutter County Airport",
		City:      "Yuba City",
		State:     "California",
		Country:   "US",
		Elevation: 58,
		Latitude:  39.1236991882,
		Longitude: -121.6050033569,
		Timezone:  "America/Los_Angeles",
	},
	"KO53": {
		ICAO:      "KO53",
		Name:      "Medford Municipal Airport",
		City:      "Medford",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1092,
		Latitude:  36.790599823,
		Longitude: -97.7490005493,
		Timezone:  "America/Chicago",
	},
	"KO54": {
		ICAO:      "KO54",
		Name:      "Lonnie Pool Field Weaverville Airport",
		City:      "Weaverville",
		State:     "California",
		Country:   "US",
		Elevation: 2350,
		Latitude:  40.7457008362,
		Longitude: -122.9219970703,
		Timezone:  "America/Los_Angeles",
	},
	"KO57": {
		ICAO:      "KO57",
		Name:      "Bryant Field",
		City:      "Bridgeport",
		State:     "California",
		Country:   "US",
		Elevation: 6468,
		Latitude:  38.2624015808,
		Longitude: -119.2259979248,
		Timezone:  "America/Los_Angeles",
	},
	"KO59": {
		ICAO:      "KO59",
		Name:      "Cedarville Airport",
		City:      "Cedarville",
		State:     "California",
		Country:   "US",
		Elevation: 4623,
		Latitude:  41.5527000427,
		Longitude: -120.1660003662,
		Timezone:  "America/Los_Angeles",
	},
	"KO61": {
		ICAO:      "KO61",
		Name:      "Cameron Airpark",
		City:      "Cameron Park",
		State:     "California",
		Country:   "US",
		Elevation: 1286,
		Latitude:  38.6839981079,
		Longitude: -120.9879989624,
		Timezone:  "America/Los_Angeles",
	},
	"KO65": {
		ICAO:      "KO65",
		Name:      "Christman Airfield",
		City:      "Okeene",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1205,
		Latitude:  36.1124992371,
		Longitude: -98.3087005615,
		Timezone:  "America/Chicago",
	},
	"KO69": {
		ICAO:      "KO69",
		Name:      "Petaluma Municipal Airport",
		City:      "Petaluma",
		State:     "California",
		Country:   "US",
		Elevation: 90,
		Latitude:  38.2578333333,
		Longitude: -122.6053333333,
		Timezone:  "America/Los_Angeles",
	},
	"KO79": {
		ICAO:      "KO79",
		Name:      "Sierraville Dearwater Airport",
		City:      "Sierraville",
		State:     "California",
		Country:   "US",
		Elevation: 4984,
		Latitude:  39.5810012817,
		Longitude: -120.3539962769,
		Timezone:  "America/Los_Angeles",
	},
	"KO81": {
		ICAO:      "KO81",
		Name:      "Tulelake Municipal Airport",
		City:      "Tulelake",
		State:     "California",
		Country:   "US",
		Elevation: 4044,
		Latitude:  41.8874015808,
		Longitude: -121.3590011597,
		Timezone:  "America/Los_Angeles",
	},
	"KO86": {
		ICAO:      "KO86",
		Name:      "Trinity Center Airport",
		City:      "Trinity Center",
		State:     "California",
		Country:   "US",
		Elevation: 2390,
		Latitude:  40.9832000732,
		Longitude: -122.6940002441,
		Timezone:  "America/Los_Angeles",
	},
	"KO88": {
		ICAO:      "KO88",
		Name:      "Rio Vista Municipal Airport",
		City:      "Rio Vista",
		State:     "California",
		Country:   "US",
		Elevation: 20,
		Latitude:  38.1934013367,
		Longitude: -121.7040023804,
		Timezone:  "America/Los_Angeles",
	},
	"KO89": {
		ICAO:      "KO89",
		Name:      "Fall River Mills Airport",
		City:      "Fall River Mills",
		State:     "California",
		Country:   "US",
		Elevation: 3323,
		Latitude:  41.0187988281,
		Longitude: -121.4329986572,
		Timezone:  "America/Los_Angeles",
	},
	"KOAJ": {
		ICAO:      "KOAJ",
		IATA:      "OAJ",
		Name:      "Albert J Ellis Airport",
		City:      "Jacksonville",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 94,
		Latitude:  34.8292007446,
		Longitude: -77.6120986938,
		Timezone:  "America/New_York",
	},
	"KOAK": {
		ICAO:      "KOAK",
		IATA:      "OAK",
		Name:      "Metropolitan Oakland International Airport",
		City:      "Oakland",
		State:     "California",
		Country:   "US",
		Elevation: 9,
		Latitude:  37.7212982178,
		Longitude: -122.2210006714,
		Timezone:  "America/Los_Angeles",
	},
	"KOAR": {
		ICAO:      "KOAR",
		IATA:      "OAR",
		Name:      "Marina Municipal Airport",
		City:      "Marina",
		State:     "California",
		Country:   "US",
		Elevation: 137,
		Latitude:  36.68190002,
		Longitude: -121.762001,
		Timezone:  "America/Los_Angeles",
	},
	"KOBE": {
		ICAO:      "KOBE",
		IATA:      "OBE",
		Name:      "Okeechobee County Airport",
		City:      "Okeechobee",
		State:     "Florida",
		Country:   "US",
		Elevation: 34,
		Latitude:  27.2628002167,
		Longitude: -80.8498001099,
		Timezone:  "America/New_York",
	},
	"KOBI": {
		ICAO:      "KOBI",
		Name:      "Woodbine Municipal Airport",
		City:      "Woodbine",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 42,
		Latitude:  39.2192001343,
		Longitude: -74.7947998047,
		Timezone:  "America/New_York",
	},
	"KOCF": {
		ICAO:      "KOCF",
		IATA:      "OCF",
		Name:      "Ocala International Jim Taylor Field",
		City:      "Ocala",
		State:     "Florida",
		Country:   "US",
		Elevation: 90,
		Latitude:  29.17259979,
		Longitude: -82.22419739,
		Timezone:  "America/New_York",
	},
	"KOCH": {
		ICAO:      "KOCH",
		IATA:      "OCH",
		Name:      "A L Mangham Jr. Regional Airport",
		City:      "Nacogdoches",
		State:     "Texas",
		Country:   "US",
		Elevation: 355,
		Latitude:  31.57799911,
		Longitude: -94.70950317,
		Timezone:  "America/Chicago",
	},
	"KOCQ": {
		ICAO:      "KOCQ",
		Name:      "J. Douglas Bake Memorial Airport",
		City:      "Oconto",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 604,
		Latitude:  44.87419891,
		Longitude: -87.9095993,
		Timezone:  "America/Chicago",
	},
	"KOCW": {
		ICAO:      "KOCW",
		IATA:      "OCW",
		Name:      "Warren Field",
		City:      "Washington",
		State:     "Georgia",
		Country:   "US",
		Elevation: 38,
		Latitude:  35.5704994202,
		Longitude: -77.0497970581,
		Timezone:  "America/New_York",
	},
	"KODO": {
		ICAO:      "KODO",
		Name:      "Odessa Schlemeyer Field",
		City:      "Odessa",
		State:     "Texas",
		Country:   "US",
		Elevation: 3004,
		Latitude:  31.9206008911,
		Longitude: -102.3870010376,
		Timezone:  "America/Chicago",
	},
	"KODX": {
		ICAO:      "KODX",
		Name:      "Evelyn Sharp Field",
		City:      "Ord",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2070,
		Latitude:  41.6241989136,
		Longitude: -98.9524002075,
		Timezone:  "America/Chicago",
	},
	"KOEA": {
		ICAO:      "KOEA",
		Name:      "O'Neal Airport",
		City:      "Vincennes",
		State:     "Illinois",
		Country:   "US",
		Elevation: 414,
		Latitude:  38.6913986206,
		Longitude: -87.5522003174,
		Timezone:  "America/Chicago",
	},
	"KOEB": {
		ICAO:      "KOEB",
		Name:      "Branch County Memorial Airport",
		City:      "Coldwater",
		State:     "Michigan",
		Country:   "US",
		Elevation: 959,
		Latitude:  41.9333992,
		Longitude: -85.05259705,
		Timezone:  "America/Detroit",
	},
	"KOEL": {
		ICAO:      "KOEL",
		Name:      "Oakley Municipal Airport",
		City:      "Oakley",
		State:     "Kansas",
		Country:   "US",
		Elevation: 3045,
		Latitude:  39.1099014282,
		Longitude: -100.8160018921,
		Timezone:  "America/Chicago",
	},
	"KOEO": {
		ICAO:      "KOEO",
		IATA:      "OEO",
		Name:      "L O Simenstad Municipal Airport",
		City:      "Osceola",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 903,
		Latitude:  45.3100013733,
		Longitude: -92.6919021606,
		Timezone:  "America/Chicago",
	},
	"KOFF": {
		ICAO:      "KOFF",
		IATA:      "OFF",
		Name:      "Offutt Air Force Base",
		City:      "Omaha",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1052,
		Latitude:  41.1183013916,
		Longitude: -95.9124984741,
		Timezone:  "America/Chicago",
	},
	"KOFK": {
		ICAO:      "KOFK",
		IATA:      "OFK",
		Name:      "Karl Stefan Memorial Airport",
		City:      "Norfolk",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1573,
		Latitude:  41.9855003357,
		Longitude: -97.4350967407,
		Timezone:  "America/Chicago",
	},
	"KOFP": {
		ICAO:      "KOFP",
		Name:      "Hanover County Municipal Airport",
		City:      "Richmond/Ashland",
		State:     "Virginia",
		Country:   "US",
		Elevation: 207,
		Latitude:  37.70899963,
		Longitude: -77.43669891,
		Timezone:  "America/New_York",
	},
	"KOGA": {
		ICAO:      "KOGA",
		IATA:      "OGA",
		Name:      "Searle Field",
		City:      "Ogallala",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3279,
		Latitude:  41.11949921,
		Longitude: -101.7699966,
		Timezone:  "America/Denver",
	},
	"KOGB": {
		ICAO:      "KOGB",
		IATA:      "OGB",
		Name:      "Orangeburg Municipal Airport",
		City:      "Orangeburg",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 195,
		Latitude:  33.4567985535,
		Longitude: -80.8594970703,
		Timezone:  "America/New_York",
	},
	"KOGD": {
		ICAO:      "KOGD",
		IATA:      "OGD",
		Name:      "Ogden Hinckley Airport",
		City:      "Ogden",
		State:     "Utah",
		Country:   "US",
		Elevation: 4473,
		Latitude:  41.1958999634,
		Longitude: -112.0120010376,
		Timezone:  "America/Denver",
	},
	"KOGM": {
		ICAO:      "KOGM",
		Name:      "Ontonagon County Schuster Field",
		City:      "Ontonagon",
		State:     "Michigan",
		Country:   "US",
		Elevation: 669,
		Latitude:  46.84550095,
		Longitude: -89.36710358,
		Timezone:  "America/Detroit",
	},
	"KOGS": {
		ICAO:      "KOGS",
		IATA:      "OGS",
		Name:      "Ogdensburg International Airport",
		City:      "Ogdensburg",
		State:     "New-York",
		Country:   "US",
		Elevation: 297,
		Latitude:  44.6819000244,
		Longitude: -75.4654998779,
		Timezone:  "America/New_York",
	},
	"KOIC": {
		ICAO:      "KOIC",
		IATA:      "OIC",
		Name:      "Lt Warren Eaton Airport",
		City:      "Norwich",
		State:     "New-York",
		Country:   "US",
		Elevation: 1025,
		Latitude:  42.5666007996,
		Longitude: -75.5241012573,
		Timezone:  "America/New_York",
	},
	"KOIN": {
		ICAO:      "KOIN",
		Name:      "Oberlin Municipal Airport",
		City:      "Oberlin",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2703,
		Latitude:  39.8339004517,
		Longitude: -100.5390014648,
		Timezone:  "America/Chicago",
	},
	"KOJA": {
		ICAO:      "KOJA",
		Name:      "Thomas P Stafford Airport",
		City:      "Weatherford",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1605,
		Latitude:  35.5447998047,
		Longitude: -98.6685028076,
		Timezone:  "America/Chicago",
	},
	"KOJC": {
		ICAO:      "KOJC",
		IATA:      "OJC",
		Name:      "Johnson County Executive Airport",
		City:      "Olathe",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1096,
		Latitude:  38.84759903,
		Longitude: -94.73760223,
		Timezone:  "America/Chicago",
	},
	"KOKB": {
		ICAO:      "KOKB",
		IATA:      "OCN",
		Name:      "Oceanside Municipal Airport",
		City:      "Oceanside",
		State:     "California",
		Country:   "US",
		Elevation: 28,
		Latitude:  33.217300415,
		Longitude: -117.3539962769,
		Timezone:  "America/Los_Angeles",
	},
	"KOKC": {
		ICAO:      "KOKC",
		IATA:      "OKC",
		Name:      "Will Rogers World Airport",
		City:      "Oklahoma City",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1295,
		Latitude:  35.3931007385,
		Longitude: -97.6007003784,
		Timezone:  "America/Chicago",
	},
	"KOKH": {
		ICAO:      "KOKH",
		IATA:      "ODW",
		Name:      "AJ Eisenberg Airport",
		City:      "Oak Harbor",
		State:     "Washington",
		Country:   "US",
		Elevation: 193,
		Latitude:  48.251499176,
		Longitude: -122.674003601,
		Timezone:  "America/Los_Angeles",
	},
	"KOKK": {
		ICAO:      "KOKK",
		IATA:      "OKK",
		Name:      "Kokomo Municipal Airport",
		City:      "Kokomo",
		State:     "Indiana",
		Country:   "US",
		Elevation: 830,
		Latitude:  40.5281982422,
		Longitude: -86.0589981079,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KOKM": {
		ICAO:      "KOKM",
		IATA:      "OKM",
		Name:      "Okmulgee Regional Airport",
		City:      "Okmulgee",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 720,
		Latitude:  35.6680984497,
		Longitude: -95.9486999512,
		Timezone:  "America/Chicago",
	},
	"KOKS": {
		ICAO:      "KOKS",
		IATA:      "OKS",
		Name:      "Garden County Airport",
		City:      "Oshkosh",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3394,
		Latitude:  41.4010009766,
		Longitude: -102.3550033569,
		Timezone:  "America/Denver",
	},
	"KOKV": {
		ICAO:      "KOKV",
		IATA:      "WGO",
		Name:      "Winchester Regional Airport",
		City:      "Winchester",
		State:     "Virginia",
		Country:   "US",
		Elevation: 726,
		Latitude:  39.14350128,
		Longitude: -78.14440155,
		Timezone:  "America/New_York",
	},
	"KOKZ": {
		ICAO:      "KOKZ",
		Name:      "Kaolin Field",
		City:      "Sandersville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 438,
		Latitude:  32.9667015076,
		Longitude: -82.8382034302,
		Timezone:  "America/New_York",
	},
	"KOLD": {
		ICAO:      "KOLD",
		IATA:      "OLD",
		Name:      "Dewitt FieldOld Town Municipal Airport",
		City:      "Old Town",
		State:     "Maine",
		Country:   "US",
		Elevation: 126,
		Latitude:  44.95280075,
		Longitude: -68.67430115,
		Timezone:  "America/New_York",
	},
	"KOLE": {
		ICAO:      "KOLE",
		IATA:      "OLE",
		Name:      "Cattaraugus County-Olean Airport",
		City:      "Olean",
		State:     "New-York",
		Country:   "US",
		Elevation: 2135,
		Latitude:  42.24119949,
		Longitude: -78.37139893,
		Timezone:  "America/New_York",
	},
	"KOLF": {
		ICAO:      "KOLF",
		IATA:      "OLF",
		Name:      "L M Clayton Airport",
		City:      "Wolf Point",
		State:     "Montana",
		Country:   "US",
		Elevation: 1986,
		Latitude:  48.0945014954,
		Longitude: -105.5749969482,
		Timezone:  "America/Denver",
	},
	"KOLG": {
		ICAO:      "KOLG",
		Name:      "Solon Springs Municipal Airport",
		City:      "Solon Springs",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1102,
		Latitude:  46.3148002625,
		Longitude: -91.8163986206,
		Timezone:  "America/Chicago",
	},
	"KOLM": {
		ICAO:      "KOLM",
		IATA:      "OLM",
		Name:      "Olympia Regional Airport",
		City:      "Olympia",
		State:     "Washington",
		Country:   "US",
		Elevation: 209,
		Latitude:  46.9693985,
		Longitude: -122.9029999,
		Timezone:  "America/Los_Angeles",
	},
	"KOLS": {
		ICAO:      "KOLS",
		IATA:      "OLS",
		Name:      "Nogales International Airport",
		City:      "Nogales",
		State:     "Arizona",
		Country:   "US",
		Elevation: 3955,
		Latitude:  31.4176998138,
		Longitude: -110.8479995728,
		Timezone:  "America/Phoenix",
	},
	"KOLU": {
		ICAO:      "KOLU",
		IATA:      "OLU",
		Name:      "Columbus Municipal Airport",
		City:      "Columbus",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1447,
		Latitude:  41.44800186,
		Longitude: -97.34259796,
		Timezone:  "America/Chicago",
	},
	"KOLV": {
		ICAO:      "KOLV",
		IATA:      "OLV",
		Name:      "Olive Branch Airport",
		City:      "Olive Branch",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 402,
		Latitude:  34.9786987305,
		Longitude: -89.7869033813,
		Timezone:  "America/Chicago",
	},
	"KOLY": {
		ICAO:      "KOLY",
		IATA:      "OLY",
		Name:      "Olney Noble Airport",
		City:      "Olney-Noble",
		State:     "Illinois",
		Country:   "US",
		Elevation: 482,
		Latitude:  38.7218017578,
		Longitude: -88.176399231,
		Timezone:  "America/Chicago",
	},
	"KOLZ": {
		ICAO:      "KOLZ",
		Name:      "Oelwein Municipal Airport",
		City:      "Oelwein",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1076,
		Latitude:  42.6808013916,
		Longitude: -91.9745025635,
		Timezone:  "America/Chicago",
	},
	"KOMA": {
		ICAO:      "KOMA",
		IATA:      "OMA",
		Name:      "Eppley Airfield",
		City:      "Omaha",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 984,
		Latitude:  41.3031997681,
		Longitude: -95.8940963745,
		Timezone:  "America/Chicago",
	},
	"KOMH": {
		ICAO:      "KOMH",
		Name:      "Orange County Airport",
		City:      "Orange",
		State:     "Virginia",
		Country:   "US",
		Elevation: 465,
		Latitude:  38.2472000122,
		Longitude: -78.0456008911,
		Timezone:  "America/New_York",
	},
	"KOMK": {
		ICAO:      "KOMK",
		IATA:      "OMK",
		Name:      "Omak Airport",
		City:      "Omak",
		State:     "Washington",
		Country:   "US",
		Elevation: 1305,
		Latitude:  48.4644012451,
		Longitude: -119.5179977417,
		Timezone:  "America/Los_Angeles",
	},
	"KOMN": {
		ICAO:      "KOMN",
		Name:      "Ormond Beach Municipal Airport",
		City:      "Ormond Beach",
		State:     "Florida",
		Country:   "US",
		Elevation: 29,
		Latitude:  29.3006000519,
		Longitude: -81.1136016846,
		Timezone:  "America/New_York",
	},
	"KONA": {
		ICAO:      "KONA",
		IATA:      "ONA",
		Name:      "Winona Municipal-Max Conrad Field",
		City:      "Winona",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 656,
		Latitude:  44.07720184,
		Longitude: -91.70829773,
		Timezone:  "America/Chicago",
	},
	"KONL": {
		ICAO:      "KONL",
		IATA:      "ONL",
		Name:      "The O'Neill Municipal John L Baker Field",
		City:      "O'Neill",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2031,
		Latitude:  42.46989822,
		Longitude: -98.68810272,
		Timezone:  "America/Chicago",
	},
	"KONM": {
		ICAO:      "KONM",
		IATA:      "ONM",
		Name:      "Socorro Municipal Airport",
		City:      "Socorro",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4875,
		Latitude:  34.0224990845,
		Longitude: -106.9029998779,
		Timezone:  "America/Denver",
	},
	"KONO": {
		ICAO:      "KONO",
		IATA:      "ONO",
		Name:      "Ontario Municipal Airport",
		City:      "Ontario",
		State:     "Oregon",
		Country:   "US",
		Elevation: 2193,
		Latitude:  44.0205001831,
		Longitude: -117.013999939,
		Timezone:  "America/Boise",
	},
	"KONP": {
		ICAO:      "KONP",
		IATA:      "ONP",
		Name:      "Newport Municipal Airport",
		City:      "Newport",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 160,
		Latitude:  44.5803985596,
		Longitude: -124.0579986572,
		Timezone:  "America/Los_Angeles",
	},
	"KONT": {
		ICAO:      "KONT",
		IATA:      "ONT",
		Name:      "Ontario International Airport",
		City:      "Ontario",
		State:     "California",
		Country:   "US",
		Elevation: 944,
		Latitude:  34.0559997559,
		Longitude: -117.6009979248,
		Timezone:  "America/Los_Angeles",
	},
	"KONX": {
		ICAO:      "KONX",
		Name:      "Currituck County Regional Airport",
		City:      "Currituck",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 18,
		Latitude:  36.39889908,
		Longitude: -76.01629639,
		Timezone:  "America/New_York",
	},
	"KONY": {
		ICAO:      "KONY",
		IATA:      "ONY",
		Name:      "Olney Municipal Airport",
		City:      "Olney",
		State:     "Texas",
		Country:   "US",
		Elevation: 1275,
		Latitude:  33.3508987427,
		Longitude: -98.8191986084,
		Timezone:  "America/Chicago",
	},
	"KONZ": {
		ICAO:      "KONZ",
		Name:      "Grosse Ile Municipal Airport",
		City:      "Detroit/Grosse Ile",
		State:     "Michigan",
		Country:   "US",
		Elevation: 591,
		Latitude:  42.0990982056,
		Longitude: -83.1614990234,
		Timezone:  "America/Detroit",
	},
	"KOOA": {
		ICAO:      "KOOA",
		IATA:      "OOA",
		Name:      "Oskaloosa Municipal Airport",
		City:      "Oskaloosa",
		State:     "Iowa",
		Country:   "US",
		Elevation: 841,
		Latitude:  41.2262001,
		Longitude: -92.49389648,
		Timezone:  "America/Chicago",
	},
	"KOPF": {
		ICAO:      "KOPF",
		IATA:      "OPF",
		Name:      "Opa-locka Executive Airport",
		City:      "Miami",
		State:     "Florida",
		Country:   "US",
		Elevation: 8,
		Latitude:  25.90699959,
		Longitude: -80.27839661,
		Timezone:  "America/New_York",
	},
	"KOPL": {
		ICAO:      "KOPL",
		IATA:      "OPL",
		Name:      "St Landry Parish Ahart Field",
		City:      "Opelousas",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 75,
		Latitude:  30.5583992004,
		Longitude: -92.0994033813,
		Timezone:  "America/Chicago",
	},
	"KOPN": {
		ICAO:      "KOPN",
		Name:      "Thomaston Upson County Airport",
		City:      "Thomaston",
		State:     "Georgia",
		Country:   "US",
		Elevation: 798,
		Latitude:  32.95460129,
		Longitude: -84.26319885,
		Timezone:  "America/New_York",
	},
	"KOQN": {
		ICAO:      "KOQN",
		Name:      "Brandywine Airport",
		City:      "West Chester",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 466,
		Latitude:  39.9901008606,
		Longitude: -75.5819015503,
		Timezone:  "America/New_York",
	},
	"KOQU": {
		ICAO:      "KOQU",
		IATA:      "NCO",
		Name:      "Quonset State Airport",
		City:      "North Kingstown",
		State:     "Rhode-Island",
		Country:   "US",
		Elevation: 18,
		Latitude:  41.5970993042,
		Longitude: -71.4121017456,
		Timezone:  "America/New_York",
	},
	"KOQW": {
		ICAO:      "KOQW",
		Name:      "Maquoketa Municipal Airport",
		City:      "Maquoketa",
		State:     "Iowa",
		Country:   "US",
		Elevation: 769,
		Latitude:  42.0500984192,
		Longitude: -90.7388000488,
		Timezone:  "America/Chicago",
	},
	"KORB": {
		ICAO:      "KORB",
		Name:      "Orr Regional Airport",
		City:      "Orr",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1311,
		Latitude:  48.0158996582,
		Longitude: -92.8561019897,
		Timezone:  "America/Chicago",
	},
	"KORC": {
		ICAO:      "KORC",
		Name:      "Orange City Municipal Airport",
		City:      "Orange City",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1414,
		Latitude:  42.9902992249,
		Longitude: -96.0627975464,
		Timezone:  "America/Chicago",
	},
	"KORD": {
		ICAO:      "KORD",
		IATA:      "ORD",
		Name:      "Chicago O'Hare International Airport",
		City:      "Chicago",
		State:     "Illinois",
		Country:   "US",
		Elevation: 672,
		Latitude:  41.97859955,
		Longitude: -87.90480042,
		Timezone:  "America/Chicago",
	},
	"KORE": {
		ICAO:      "KORE",
		Name:      "Orange Municipal Airport",
		City:      "Orange",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 555,
		Latitude:  42.570098877,
		Longitude: -72.2885971069,
		Timezone:  "America/New_York",
	},
	"KORF": {
		ICAO:      "KORF",
		IATA:      "ORF",
		Name:      "Norfolk International Airport",
		City:      "Norfolk",
		State:     "Virginia",
		Country:   "US",
		Elevation: 26,
		Latitude:  36.8945999146,
		Longitude: -76.2012023926,
		Timezone:  "America/New_York",
	},
	"KORG": {
		ICAO:      "KORG",
		Name:      "Orange County Airport",
		City:      "Orange",
		State:     "Texas",
		Country:   "US",
		Elevation: 13,
		Latitude:  30.0692005157,
		Longitude: -93.8009033203,
		Timezone:  "America/Chicago",
	},
	"KORH": {
		ICAO:      "KORH",
		IATA:      "ORH",
		Name:      "Worcester Regional Airport",
		City:      "Worcester",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 1009,
		Latitude:  42.2672996521,
		Longitude: -71.8757019043,
		Timezone:  "America/New_York",
	},
	"KORK": {
		ICAO:      "KORK",
		Name:      "North Little Rock Municipal Airport",
		City:      "North Little Rock",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 545,
		Latitude:  34.83309937,
		Longitude: -92.25409698,
		Timezone:  "America/Chicago",
	},
	"KORL": {
		ICAO:      "KORL",
		IATA:      "ORL",
		Name:      "Orlando Executive Airport",
		City:      "Orlando",
		State:     "Florida",
		Country:   "US",
		Elevation: 113,
		Latitude:  28.5454998016,
		Longitude: -81.332901001,
		Timezone:  "America/New_York",
	},
	"KORS": {
		ICAO:      "KORS",
		IATA:      "ESD",
		Name:      "Orcas Island Airport",
		City:      "Eastsound",
		State:     "Washington",
		Country:   "US",
		Elevation: 31,
		Latitude:  48.7081985474,
		Longitude: -122.910003662,
		Timezone:  "America/Los_Angeles",
	},
	"KOSA": {
		ICAO:      "KOSA",
		IATA:      "MPS",
		Name:      "Mount Pleasant Regional Airport",
		City:      "Mount Pleasant",
		State:     "Texas",
		Country:   "US",
		Elevation: 364,
		Latitude:  33.09550095,
		Longitude: -94.96150208,
		Timezone:  "America/Chicago",
	},
	"KOSC": {
		ICAO:      "KOSC",
		IATA:      "OSC",
		Name:      "Oscoda Wurtsmith Airport",
		City:      "Oscoda",
		State:     "Michigan",
		Country:   "US",
		Elevation: 633,
		Latitude:  44.45159912,
		Longitude: -83.39409637,
		Timezone:  "America/Detroit",
	},
	"KOSH": {
		ICAO:      "KOSH",
		IATA:      "OSH",
		Name:      "Wittman Regional Airport",
		City:      "Oshkosh",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 808,
		Latitude:  43.9844017029,
		Longitude: -88.5569992065,
		Timezone:  "America/Chicago",
	},
	"KOSU": {
		ICAO:      "KOSU",
		IATA:      "OSU",
		Name:      "Ohio State University Airport",
		City:      "Columbus",
		State:     "Ohio",
		Country:   "US",
		Elevation: 905,
		Latitude:  40.0797996521,
		Longitude: -83.0729980469,
		Timezone:  "America/New_York",
	},
	"KOSX": {
		ICAO:      "KOSX",
		IATA:      "OSX",
		Name:      "Kosciusko Attala County Airport",
		City:      "Kosciusko",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 480,
		Latitude:  33.0903015137,
		Longitude: -89.5419998169,
		Timezone:  "America/Chicago",
	},
	"KOTG": {
		ICAO:      "KOTG",
		IATA:      "OTG",
		Name:      "Worthington Municipal Airport",
		City:      "Worthington",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1574,
		Latitude:  43.6551017761,
		Longitude: -95.5792007446,
		Timezone:  "America/Chicago",
	},
	"KOTH": {
		ICAO:      "KOTH",
		IATA:      "OTH",
		Name:      "Southwest Oregon Regional Airport",
		City:      "North Bend",
		State:     "Oregon",
		Country:   "US",
		Elevation: 17,
		Latitude:  43.417098999,
		Longitude: -124.2460021973,
		Timezone:  "America/Los_Angeles",
	},
	"KOTM": {
		ICAO:      "KOTM",
		IATA:      "OTM",
		Name:      "Ottumwa Regional Airport",
		City:      "Ottumwa",
		State:     "Iowa",
		Country:   "US",
		Elevation: 845,
		Latitude:  41.10660172,
		Longitude: -92.44789886,
		Timezone:  "America/Chicago",
	},
	"KOUN": {
		ICAO:      "KOUN",
		IATA:      "OUN",
		Name:      "University of Oklahoma Westheimer Airport",
		City:      "Norman",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1182,
		Latitude:  35.2456016541,
		Longitude: -97.4720993042,
		Timezone:  "America/Chicago",
	},
	"KOVE": {
		ICAO:      "KOVE",
		IATA:      "OVE",
		Name:      "Oroville Municipal Airport",
		City:      "Oroville",
		State:     "California",
		Country:   "US",
		Elevation: 192,
		Latitude:  39.4878005981,
		Longitude: -121.6220016479,
		Timezone:  "America/Los_Angeles",
	},
	"KOVL": {
		ICAO:      "KOVL",
		Name:      "Olivia Regional Airport",
		City:      "Olivia",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1076,
		Latitude:  44.7785987854,
		Longitude: -95.0327987671,
		Timezone:  "America/Chicago",
	},
	"KOVO": {
		ICAO:      "KOVO",
		Name:      "North Vernon Airport",
		City:      "North Vernon",
		State:     "Indiana",
		Country:   "US",
		Elevation: 757,
		Latitude:  39.04560089,
		Longitude: -85.6053009,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KOVS": {
		ICAO:      "KOVS",
		Name:      "Boscobel Airport",
		City:      "Boscobel",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 673,
		Latitude:  43.16019821,
		Longitude: -90.67549896,
		Timezone:  "America/Chicago",
	},
	"KOWA": {
		ICAO:      "KOWA",
		IATA:      "OWA",
		Name:      "Owatonna Degner Regional Airport",
		City:      "Owatonna",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1145,
		Latitude:  44.12340164,
		Longitude: -93.26059723,
		Timezone:  "America/Chicago",
	},
	"KOWB": {
		ICAO:      "KOWB",
		IATA:      "OWB",
		Name:      "Owensboro Daviess County Airport",
		City:      "Owensboro",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 407,
		Latitude:  37.74010086,
		Longitude: -87.16680145,
		Timezone:  "America/Chicago",
	},
	"KOWD": {
		ICAO:      "KOWD",
		IATA:      "OWD",
		Name:      "Norwood Memorial Airport",
		City:      "Norwood",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 49,
		Latitude:  42.1904983521,
		Longitude: -71.1728973389,
		Timezone:  "America/New_York",
	},
	"KOWI": {
		ICAO:      "KOWI",
		Name:      "Ottawa Municipal Airport",
		City:      "Ottawa",
		State:     "Kansas",
		Country:   "US",
		Elevation: 966,
		Latitude:  38.5387001038,
		Longitude: -95.2529983521,
		Timezone:  "America/Chicago",
	},
	"KOWK": {
		ICAO:      "KOWK",
		IATA:      "OWK",
		Name:      "Central Maine Arpt of Norridgewock Airport",
		City:      "Norridgewock",
		State:     "Maine",
		Country:   "US",
		Elevation: 270,
		Latitude:  44.71549988,
		Longitude: -69.86650085,
		Timezone:  "America/New_York",
	},
	"KOWP": {
		ICAO:      "KOWP",
		Name:      "William R. Pogue Municipal Airport",
		City:      "Sand Springs",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 892,
		Latitude:  36.1753006,
		Longitude: -96.15180206,
		Timezone:  "America/Chicago",
	},
	"KOWX": {
		ICAO:      "KOWX",
		Name:      "Putnam County Airport",
		City:      "Ottawa",
		State:     "Ohio",
		Country:   "US",
		Elevation: 764,
		Latitude:  41.03559875,
		Longitude: -83.98200226,
		Timezone:  "America/New_York",
	},
	"KOXB": {
		ICAO:      "KOXB",
		IATA:      "OCE",
		Name:      "Ocean City Municipal Airport",
		City:      "Ocean City",
		State:     "Maryland",
		Country:   "US",
		Elevation: 11,
		Latitude:  38.3103981018,
		Longitude: -75.1240005493,
		Timezone:  "America/New_York",
	},
	"KOXC": {
		ICAO:      "KOXC",
		IATA:      "OXC",
		Name:      "Waterbury Oxford Airport",
		City:      "Oxford",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 726,
		Latitude:  41.4785995483,
		Longitude: -73.1352005005,
		Timezone:  "America/New_York",
	},
	"KOXD": {
		ICAO:      "KOXD",
		IATA:      "OXD",
		Name:      "Miami University Airport",
		City:      "Oxford",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1041,
		Latitude:  39.50230026,
		Longitude: -84.78440094,
		Timezone:  "America/New_York",
	},
	"KOXI": {
		ICAO:      "KOXI",
		Name:      "Starke County Airport",
		City:      "Knox",
		State:     "Indiana",
		Country:   "US",
		Elevation: 685,
		Latitude:  41.3302002,
		Longitude: -86.66470337,
		Timezone:  "America/Indiana/Knox",
	},
	"KOXR": {
		ICAO:      "KOXR",
		IATA:      "OXR",
		Name:      "Oxnard Airport",
		City:      "Oxnard",
		State:     "California",
		Country:   "US",
		Elevation: 45,
		Latitude:  34.2008018494,
		Longitude: -119.2070007324,
		Timezone:  "America/Los_Angeles",
	},
	"KOXV": {
		ICAO:      "KOXV",
		Name:      "Knoxville Municipal Airport",
		City:      "Knoxville",
		State:     "Iowa",
		Country:   "US",
		Elevation: 928,
		Latitude:  41.2989006042,
		Longitude: -93.1138000488,
		Timezone:  "America/Chicago",
	},
	"KOYM": {
		ICAO:      "KOYM",
		IATA:      "STQ",
		Name:      "St Marys Municipal Airport",
		City:      "St Marys",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1934,
		Latitude:  41.4124984741,
		Longitude: -78.5026016235,
		Timezone:  "America/New_York",
	},
	"KOZA": {
		ICAO:      "KOZA",
		IATA:      "OZA",
		Name:      "Ozona Municipal Airport",
		City:      "Ozona",
		State:     "Texas",
		Country:   "US",
		Elevation: 2381,
		Latitude:  30.7353000641,
		Longitude: -101.2030029297,
		Timezone:  "America/Chicago",
	},
	"KOZR": {
		ICAO:      "KOZR",
		IATA:      "OZR",
		Name:      "Cairns AAF (Fort Rucker) Air Field",
		City:      "Fort Rucker/Ozark",
		State:     "Alabama",
		Country:   "US",
		Elevation: 301,
		Latitude:  31.27569962,
		Longitude: -85.71340179,
		Timezone:  "America/Chicago",
	},
	"KOZW": {
		ICAO:      "KOZW",
		Name:      "Livingston County Spencer J. Hardy Airport",
		City:      "Howell",
		State:     "Michigan",
		Country:   "US",
		Elevation: 962,
		Latitude:  42.62919998,
		Longitude: -83.98210144,
		Timezone:  "America/Detroit",
	},
	"KP01": {
		ICAO:      "KP01",
		Name:      "Eric Marcus Municipal Airport",
		City:      "Ajo",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1458,
		Latitude:  32.4500999451,
		Longitude: -112.8669967651,
		Timezone:  "America/Phoenix",
	},
	"KP03": {
		ICAO:      "KP03",
		Name:      "Cochise College Airport",
		City:      "Douglas",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4124,
		Latitude:  31.371099472,
		Longitude: -109.6900024414,
		Timezone:  "America/Hermosillo",
	},
	"KP04": {
		ICAO:      "KP04",
		IATA:      "BSQ",
		Name:      "Bisbee Municipal Airport",
		City:      "Bisbee",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4780,
		Latitude:  31.3640003204,
		Longitude: -109.883003235,
		Timezone:  "America/Hermosillo",
	},
	"KP08": {
		ICAO:      "KP08",
		Name:      "Coolidge Municipal Airport",
		City:      "Coolidge",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1574,
		Latitude:  32.9359016418,
		Longitude: -111.4270019531,
		Timezone:  "America/Phoenix",
	},
	"KP10": {
		ICAO:      "KP10",
		Name:      "Polacca Airport",
		City:      "Polacca",
		State:     "Arizona",
		Country:   "US",
		Elevation: 5573,
		Latitude:  35.7916984558,
		Longitude: -110.422996521,
		Timezone:  "America/Phoenix",
	},
	"KP13": {
		ICAO:      "KP13",
		Name:      "San Carlos Apache Airport",
		City:      "Globe",
		State:     "Arizona",
		Country:   "US",
		Elevation: 3261,
		Latitude:  33.353099823,
		Longitude: -110.6669998169,
		Timezone:  "America/Phoenix",
	},
	"KP14": {
		ICAO:      "KP14",
		IATA:      "HBK",
		Name:      "Holbrook Municipal Airport",
		City:      "Holbrook",
		State:     "Arizona",
		Country:   "US",
		Elevation: 5262,
		Latitude:  34.940700531,
		Longitude: -110.138000488,
		Timezone:  "America/Phoenix",
	},
	"KP19": {
		ICAO:      "KP19",
		Name:      "Stellar Airpark",
		City:      "Chandler",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1177,
		Latitude:  33.2989006042,
		Longitude: -111.9150009155,
		Timezone:  "America/Phoenix",
	},
	"KP20": {
		ICAO:      "KP20",
		Name:      "Avi Suquilla Airport",
		City:      "Parker",
		State:     "Arizona",
		Country:   "US",
		Elevation: 452,
		Latitude:  34.1506004333,
		Longitude: -114.2710037231,
		Timezone:  "America/Los_Angeles",
	},
	"KP23": {
		ICAO:      "KP23",
		Name:      "Seligman Airport",
		City:      "Seligman",
		State:     "Arizona",
		Country:   "US",
		Elevation: 5235,
		Latitude:  35.3349990845,
		Longitude: -112.8870010376,
		Timezone:  "America/Phoenix",
	},
	"KP29": {
		ICAO:      "KP29",
		Name:      "Tombstone Municipal Airport",
		City:      "Tombstone",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4743,
		Latitude:  31.6709003448,
		Longitude: -110.0230026245,
		Timezone:  "America/Phoenix",
	},
	"KP33": {
		ICAO:      "KP33",
		IATA:      "CWX",
		Name:      "Cochise County Airport",
		City:      "Willcox",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4187,
		Latitude:  32.24539948,
		Longitude: -109.8949966,
		Timezone:  "America/Phoenix",
	},
	"KP52": {
		ICAO:      "KP52",
		Name:      "Cottonwood Airport",
		City:      "Cottonwood",
		State:     "Arizona",
		Country:   "US",
		Elevation: 3550,
		Latitude:  34.7299995422,
		Longitude: -112.0350036621,
		Timezone:  "America/Phoenix",
	},
	"KPAE": {
		ICAO:      "KPAE",
		IATA:      "PAE",
		Name:      "Snohomish County (Paine Field) Airport",
		City:      "Everett",
		State:     "Washington",
		Country:   "US",
		Elevation: 606,
		Latitude:  47.90629959,
		Longitude: -122.2819977,
		Timezone:  "America/Los_Angeles",
	},
	"KPAH": {
		ICAO:      "KPAH",
		IATA:      "PAH",
		Name:      "Barkley Regional Airport",
		City:      "Paducah",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 410,
		Latitude:  37.060798645,
		Longitude: -88.7738037109,
		Timezone:  "America/Chicago",
	},
	"KPAM": {
		ICAO:      "KPAM",
		IATA:      "PAM",
		Name:      "Tyndall Air Force Base",
		City:      "Panama City",
		State:     "Florida",
		Country:   "US",
		Elevation: 17,
		Latitude:  30.0695991516,
		Longitude: -85.5754013062,
		Timezone:  "America/Chicago",
	},
	"KPAN": {
		ICAO:      "KPAN",
		IATA:      "PJB",
		Name:      "Payson Airport",
		City:      "Payson",
		State:     "Arizona",
		Country:   "US",
		Elevation: 5157,
		Latitude:  34.2568016052,
		Longitude: -111.3389968872,
		Timezone:  "America/Phoenix",
	},
	"KPAO": {
		ICAO:      "KPAO",
		IATA:      "PAO",
		Name:      "Palo Alto Airport of Santa Clara County",
		City:      "Palo Alto",
		State:     "California",
		Country:   "US",
		Elevation: 4,
		Latitude:  37.461101532,
		Longitude: -122.1149978638,
		Timezone:  "America/Los_Angeles",
	},
	"KPBF": {
		ICAO:      "KPBF",
		IATA:      "PBF",
		Name:      "Grider Field",
		City:      "Pine Bluff",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 206,
		Latitude:  34.1730995178,
		Longitude: -91.9356002808,
		Timezone:  "America/Chicago",
	},
	"KPBG": {
		ICAO:      "KPBG",
		IATA:      "PBG",
		Name:      "Plattsburgh International Airport",
		City:      "Plattsburgh",
		State:     "New-York",
		Country:   "US",
		Elevation: 234,
		Latitude:  44.6509017944,
		Longitude: -73.4681015015,
		Timezone:  "America/New_York",
	},
	"KPBH": {
		ICAO:      "KPBH",
		Name:      "Price County Airport",
		City:      "Phillips",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1497,
		Latitude:  45.7089996338,
		Longitude: -90.4024963379,
		Timezone:  "America/Chicago",
	},
	"KPBI": {
		ICAO:      "KPBI",
		IATA:      "PBI",
		Name:      "Palm Beach International Airport",
		City:      "West Palm Beach",
		State:     "Florida",
		Country:   "US",
		Elevation: 19,
		Latitude:  26.6832008362,
		Longitude: -80.0955963135,
		Timezone:  "America/New_York",
	},
	"KPBX": {
		ICAO:      "KPBX",
		IATA:      "PVL",
		Name:      "Pike County-Hatcher Field",
		City:      "Pikeville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 1473,
		Latitude:  37.5617981,
		Longitude: -82.56639862,
		Timezone:  "America/New_York",
	},
	"KPCD": {
		ICAO:      "KPCD",
		Name:      "Perryville Regional Airport",
		City:      "Perryville, Missouri",
		State:     "Missouri",
		Country:   "US",
		Elevation: 372,
		Latitude:  37.8686667,
		Longitude: -89.8621389,
		Timezone:  "America/Chicago",
	},
	"KPCM": {
		ICAO:      "KPCM",
		Name:      "Plant City Municipal Airport",
		City:      "Plant City",
		State:     "Florida",
		Country:   "US",
		Elevation: 153,
		Latitude:  28.00020027,
		Longitude: -82.16419983,
		Timezone:  "America/New_York",
	},
	"KPCW": {
		ICAO:      "KPCW",
		Name:      "Carl R Keller Field",
		City:      "Port Clinton",
		State:     "Ohio",
		Country:   "US",
		Elevation: 590,
		Latitude:  41.5163002014,
		Longitude: -82.8686981201,
		Timezone:  "America/New_York",
	},
	"KPCZ": {
		ICAO:      "KPCZ",
		Name:      "Waupaca Municipal Airport",
		City:      "Waupaca",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 840,
		Latitude:  44.33330154,
		Longitude: -89.01979828,
		Timezone:  "America/Chicago",
	},
	"KPDC": {
		ICAO:      "KPDC",
		IATA:      "PCD",
		Name:      "Prairie Du Chien Municipal Airport",
		City:      "Prairie Du Chien",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 661,
		Latitude:  43.0192985535,
		Longitude: -91.1237030029,
		Timezone:  "America/Chicago",
	},
	"KPDK": {
		ICAO:      "KPDK",
		IATA:      "PDK",
		Name:      "DeKalb Peachtree Airport",
		City:      "Atlanta",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1003,
		Latitude:  33.8755989075,
		Longitude: -84.3020019531,
		Timezone:  "America/New_York",
	},
	"KPDT": {
		ICAO:      "KPDT",
		IATA:      "PDT",
		Name:      "Eastern Oregon Regional At Pendleton Airport",
		City:      "Pendleton",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1497,
		Latitude:  45.695098877,
		Longitude: -118.841003418,
		Timezone:  "America/Los_Angeles",
	},
	"KPDX": {
		ICAO:      "KPDX",
		IATA:      "PDX",
		Name:      "Portland International Airport",
		City:      "Portland",
		State:     "Oregon",
		Country:   "US",
		Elevation: 31,
		Latitude:  45.58869934,
		Longitude: -122.5979996,
		Timezone:  "America/Los_Angeles",
	},
	"KPEA": {
		ICAO:      "KPEA",
		Name:      "Pella Municipal Airport",
		City:      "Pella",
		State:     "Iowa",
		Country:   "US",
		Elevation: 885,
		Latitude:  41.400100708,
		Longitude: -92.9458999634,
		Timezone:  "America/Chicago",
	},
	"KPEO": {
		ICAO:      "KPEO",
		Name:      "Penn Yan Airport",
		City:      "Penn Yan",
		State:     "New-York",
		Country:   "US",
		Elevation: 990,
		Latitude:  42.6371002197,
		Longitude: -77.0529022217,
		Timezone:  "America/New_York",
	},
	"KPEQ": {
		ICAO:      "KPEQ",
		IATA:      "PEQ",
		Name:      "Pecos Municipal Airport",
		City:      "Pecos",
		State:     "Texas",
		Country:   "US",
		Elevation: 2613,
		Latitude:  31.3824005127,
		Longitude: -103.5110015869,
		Timezone:  "America/Chicago",
	},
	"KPEX": {
		ICAO:      "KPEX",
		Name:      "Paynesville Municipal Airport",
		City:      "Paynesville",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1180,
		Latitude:  45.3722991943,
		Longitude: -94.74659729,
		Timezone:  "America/Chicago",
	},
	"KPEZ": {
		ICAO:      "KPEZ",
		Name:      "Pleasanton Municipal Airport",
		City:      "Pleasanton",
		State:     "Texas",
		Country:   "US",
		Elevation: 430,
		Latitude:  28.9542007446,
		Longitude: -98.5199966431,
		Timezone:  "America/Chicago",
	},
	"KPFC": {
		ICAO:      "KPFC",
		IATA:      "PFC",
		Name:      "Pacific City State Airport",
		City:      "Pacific City",
		State:     "Oregon",
		Country:   "US",
		Elevation: 5,
		Latitude:  45.199798584,
		Longitude: -123.961997986,
		Timezone:  "America/Los_Angeles",
	},
	"KPFN": {
		ICAO:      "KPFN",
		IATA:      "PFN",
		Name:      "Panama City-Bay Co International Airport",
		City:      "Panama City",
		State:     "Florida",
		Country:   "US",
		Elevation: 20,
		Latitude:  30.2120990753,
		Longitude: -85.682800293,
		Timezone:  "America/Chicago",
	},
	"KPGA": {
		ICAO:      "KPGA",
		IATA:      "PGA",
		Name:      "Page Municipal Airport",
		City:      "Page",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4316,
		Latitude:  36.92610168,
		Longitude: -111.447998,
		Timezone:  "America/Phoenix",
	},
	"KPGD": {
		ICAO:      "KPGD",
		IATA:      "PGD",
		Name:      "Charlotte County Airport",
		City:      "Punta Gorda",
		State:     "Florida",
		Country:   "US",
		Elevation: 26,
		Latitude:  26.92020035,
		Longitude: -81.9905014,
		Timezone:  "America/New_York",
	},
	"KPGR": {
		ICAO:      "KPGR",
		IATA:      "PGR",
		Name:      "Kirk Field",
		City:      "Paragould",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 290,
		Latitude:  36.06290054,
		Longitude: -90.50779724,
		Timezone:  "America/Chicago",
	},
	"KPGV": {
		ICAO:      "KPGV",
		IATA:      "PGV",
		Name:      "Pitt Greenville Airport",
		City:      "Greenville",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 26,
		Latitude:  35.6352005,
		Longitude: -77.38529968,
		Timezone:  "America/New_York",
	},
	"KPHD": {
		ICAO:      "KPHD",
		IATA:      "PHD",
		Name:      "Harry Clever Field",
		City:      "New Philadelphia",
		State:     "Ohio",
		Country:   "US",
		Elevation: 894,
		Latitude:  40.4709014893,
		Longitude: -81.4197006226,
		Timezone:  "America/New_York",
	},
	"KPHF": {
		ICAO:      "KPHF",
		IATA:      "PHF",
		Name:      "Newport News Williamsburg International Airport",
		City:      "Newport News",
		State:     "Virginia",
		Country:   "US",
		Elevation: 42,
		Latitude:  37.13190079,
		Longitude: -76.49299622,
		Timezone:  "America/New_York",
	},
	"KPHG": {
		ICAO:      "KPHG",
		Name:      "Phillipsburg Municipal Airport",
		City:      "Phillipsburg",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1907,
		Latitude:  39.7358016968,
		Longitude: -99.3171005249,
		Timezone:  "America/Chicago",
	},
	"KPHH": {
		ICAO:      "KPHH",
		IATA:      "ADR",
		Name:      "Robert F Swinnie Airport",
		City:      "Andrews",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 26,
		Latitude:  33.4516983032,
		Longitude: -79.5261993408,
		Timezone:  "America/New_York",
	},
	"KPHK": {
		ICAO:      "KPHK",
		IATA:      "PHK",
		Name:      "Palm Beach Co Glades Airport",
		City:      "Pahokee",
		State:     "Florida",
		Country:   "US",
		Elevation: 16,
		Latitude:  26.78499985,
		Longitude: -80.69339752,
		Timezone:  "America/New_York",
	},
	"KPHL": {
		ICAO:      "KPHL",
		IATA:      "PHL",
		Name:      "Philadelphia International Airport",
		City:      "Philadelphia",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 36,
		Latitude:  39.8718986511,
		Longitude: -75.2410964966,
		Timezone:  "America/New_York",
	},
	"KPHN": {
		ICAO:      "KPHN",
		IATA:      "PHN",
		Name:      "St Clair County International Airport",
		City:      "Port Huron",
		State:     "Michigan",
		Country:   "US",
		Elevation: 650,
		Latitude:  42.9109993,
		Longitude: -82.52890015,
		Timezone:  "America/Detroit",
	},
	"KPHP": {
		ICAO:      "KPHP",
		IATA:      "PHP",
		Name:      "Philip Airport",
		City:      "Philip",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 2207,
		Latitude:  44.0485992432,
		Longitude: -101.5989990234,
		Timezone:  "America/Denver",
	},
	"KPHT": {
		ICAO:      "KPHT",
		IATA:      "PHT",
		Name:      "Henry County Airport",
		City:      "Paris",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 580,
		Latitude:  36.3381996155,
		Longitude: -88.3828964233,
		Timezone:  "America/Chicago",
	},
	"KPHX": {
		ICAO:      "KPHX",
		IATA:      "PHX",
		Name:      "Phoenix Sky Harbor International Airport",
		City:      "Phoenix",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1135,
		Latitude:  33.434299469,
		Longitude: -112.0120010376,
		Timezone:  "America/Phoenix",
	},
	"KPIA": {
		ICAO:      "KPIA",
		IATA:      "PIA",
		Name:      "Greater Peoria Regional Airport",
		City:      "Peoria",
		State:     "Illinois",
		Country:   "US",
		Elevation: 660,
		Latitude:  40.6641998291,
		Longitude: -89.6932983398,
		Timezone:  "America/Chicago",
	},
	"KPIB": {
		ICAO:      "KPIB",
		IATA:      "PIB",
		Name:      "Hattiesburg Laurel Regional Airport",
		City:      "Hattiesburg/Laurel",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 298,
		Latitude:  31.4671001434,
		Longitude: -89.337097168,
		Timezone:  "America/Chicago",
	},
	"KPIE": {
		ICAO:      "KPIE",
		IATA:      "PIE",
		Name:      "St Petersburg Clearwater International Airport",
		City:      "St Petersburg-Clearwater",
		State:     "Florida",
		Country:   "US",
		Elevation: 11,
		Latitude:  27.91020012,
		Longitude: -82.68740082,
		Timezone:  "America/New_York",
	},
	"KPIH": {
		ICAO:      "KPIH",
		IATA:      "PIH",
		Name:      "Pocatello Regional Airport",
		City:      "Pocatello",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4452,
		Latitude:  42.9098014832,
		Longitude: -112.5960006714,
		Timezone:  "America/Boise",
	},
	"KPIL": {
		ICAO:      "KPIL",
		Name:      "Port Isabel Cameron County Airport",
		City:      "Port Isabel",
		State:     "Texas",
		Country:   "US",
		Elevation: 19,
		Latitude:  26.1662006378,
		Longitude: -97.3459014893,
		Timezone:  "America/Chicago",
	},
	"KPIM": {
		ICAO:      "KPIM",
		IATA:      "PIM",
		Name:      "Harris County Airport",
		City:      "Pine Mountain",
		State:     "Georgia",
		Country:   "US",
		Elevation: 902,
		Latitude:  32.8406982422,
		Longitude: -84.8824005127,
		Timezone:  "America/New_York",
	},
	"KPIR": {
		ICAO:      "KPIR",
		IATA:      "PIR",
		Name:      "Pierre Regional Airport",
		City:      "Pierre",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1744,
		Latitude:  44.38270187,
		Longitude: -100.2860031,
		Timezone:  "America/Chicago",
	},
	"KPIT": {
		ICAO:      "KPIT",
		IATA:      "PIT",
		Name:      "Pittsburgh International Airport",
		City:      "Pittsburgh",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1203,
		Latitude:  40.49150085,
		Longitude: -80.23290253,
		Timezone:  "America/New_York",
	},
	"KPJC": {
		ICAO:      "KPJC",
		Name:      "Zelienople Municipal Airport",
		City:      "Zelienople",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 898,
		Latitude:  40.80160141,
		Longitude: -80.16069794,
		Timezone:  "America/New_York",
	},
	"KPJY": {
		ICAO:      "KPJY",
		Name:      "Pinckneyville Du Quoin Airport",
		City:      "Pinckneyville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 400,
		Latitude:  37.9779014587,
		Longitude: -89.360496521,
		Timezone:  "America/Chicago",
	},
	"KPKB": {
		ICAO:      "KPKB",
		IATA:      "PKB",
		Name:      "Mid Ohio Valley Regional Airport",
		City:      "Parkersburg",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 858,
		Latitude:  39.3451004028,
		Longitude: -81.439201355,
		Timezone:  "America/New_York",
	},
	"KPKD": {
		ICAO:      "KPKD",
		IATA:      "PKD",
		Name:      "Park Rapids Municipal Konshok Field",
		City:      "Park Rapids",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1445,
		Latitude:  46.90060043,
		Longitude: -95.07309723,
		Timezone:  "America/Chicago",
	},
	"KPKF": {
		ICAO:      "KPKF",
		IATA:      "PKF",
		Name:      "Park Falls Municipal Airport",
		City:      "Park Falls",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1500,
		Latitude:  45.9550018311,
		Longitude: -90.4244003296,
		Timezone:  "America/Chicago",
	},
	"KPKV": {
		ICAO:      "KPKV",
		Name:      "Calhoun County Airport",
		City:      "Port Lavaca",
		State:     "Texas",
		Country:   "US",
		Elevation: 32,
		Latitude:  28.65399933,
		Longitude: -96.6812973,
		Timezone:  "America/Chicago",
	},
	"KPLD": {
		ICAO:      "KPLD",
		Name:      "Portland Municipal Airport",
		City:      "Portland",
		State:     "Indiana",
		Country:   "US",
		Elevation: 925,
		Latitude:  40.45080185,
		Longitude: -84.99009705,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KPLK": {
		ICAO:      "KPLK",
		IATA:      "PLK",
		Name:      "M. Graham Clark-Taney County Airport",
		City:      "Point Lookout",
		State:     "Missouri",
		Country:   "US",
		Elevation: 940,
		Latitude:  36.62590027,
		Longitude: -93.22889709,
		Timezone:  "America/Chicago",
	},
	"KPLN": {
		ICAO:      "KPLN",
		IATA:      "PLN",
		Name:      "Pellston Regional Airport of Emmet County Airport",
		City:      "Pellston",
		State:     "Michigan",
		Country:   "US",
		Elevation: 721,
		Latitude:  45.57089996,
		Longitude: -84.79669952,
		Timezone:  "America/Detroit",
	},
	"KPLR": {
		ICAO:      "KPLR",
		IATA:      "PLR",
		Name:      "St Clair County Airport",
		City:      "Pell City",
		State:     "Alabama",
		Country:   "US",
		Elevation: 485,
		Latitude:  33.5587997437,
		Longitude: -86.2490997314,
		Timezone:  "America/Chicago",
	},
	"KPLU": {
		ICAO:      "KPLU",
		Name:      "Pierce County-Thun Field",
		City:      "Puyallup",
		State:     "Washington",
		Country:   "US",
		Elevation: 538,
		Latitude:  47.1039009094,
		Longitude: -122.2870025635,
		Timezone:  "America/Los_Angeles",
	},
	"KPMB": {
		ICAO:      "KPMB",
		IATA:      "PMB",
		Name:      "Pembina Municipal Airport",
		City:      "Pembina",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 795,
		Latitude:  48.9425010681,
		Longitude: -97.2407989502,
		Timezone:  "America/Chicago",
	},
	"KPMD": {
		ICAO:      "KPMD",
		IATA:      "PMD",
		Name:      "Palmdale Regional/USAF Plant 42 Airport",
		City:      "Palmdale",
		State:     "California",
		Country:   "US",
		Elevation: 2543,
		Latitude:  34.62939835,
		Longitude: -118.0849991,
		Timezone:  "America/Los_Angeles",
	},
	"KPMH": {
		ICAO:      "KPMH",
		IATA:      "PMH",
		Name:      "Greater Portsmouth Regional Airport",
		City:      "Portsmouth",
		State:     "Ohio",
		Country:   "US",
		Elevation: 663,
		Latitude:  38.8404998779,
		Longitude: -82.8472976685,
		Timezone:  "America/New_York",
	},
	"KPMP": {
		ICAO:      "KPMP",
		IATA:      "PPM",
		Name:      "Pompano Beach Airpark",
		City:      "Pompano Beach",
		State:     "Florida",
		Country:   "US",
		Elevation: 19,
		Latitude:  26.2471008301,
		Longitude: -80.1110992432,
		Timezone:  "America/New_York",
	},
	"KPMU": {
		ICAO:      "KPMU",
		Name:      "Panola County Airport",
		City:      "Batesville",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 221,
		Latitude:  34.36349869,
		Longitude: -89.89260101,
		Timezone:  "America/Chicago",
	},
	"KPMV": {
		ICAO:      "KPMV",
		Name:      "Plattsmouth Municipal Airport",
		City:      "Plattsmouth",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1204,
		Latitude:  40.95019913,
		Longitude: -95.91790009,
		Timezone:  "America/Chicago",
	},
	"KPMZ": {
		ICAO:      "KPMZ",
		Name:      "Plymouth Municipal Airport",
		City:      "Plymouth",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 39,
		Latitude:  35.8083992004,
		Longitude: -76.7593002319,
		Timezone:  "America/New_York",
	},
	"KPNA": {
		ICAO:      "KPNA",
		IATA:      "PWY",
		Name:      "Ralph Wenz Field",
		City:      "Pinedale",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 7102,
		Latitude:  42.79550171,
		Longitude: -109.8069992,
		Timezone:  "America/Denver",
	},
	"KPNC": {
		ICAO:      "KPNC",
		IATA:      "PNC",
		Name:      "Ponca City Regional Airport",
		City:      "Ponca City",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1008,
		Latitude:  36.73199844,
		Longitude: -97.09980011,
		Timezone:  "America/Chicago",
	},
	"KPNE": {
		ICAO:      "KPNE",
		IATA:      "PNE",
		Name:      "Northeast Philadelphia Airport",
		City:      "Philadelphia",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 120,
		Latitude:  40.08190155,
		Longitude: -75.01059723,
		Timezone:  "America/New_York",
	},
	"KPNM": {
		ICAO:      "KPNM",
		Name:      "Princeton Municipal Airport",
		City:      "Princeton",
		State:     "Maine",
		Country:   "US",
		Elevation: 980,
		Latitude:  45.55989838,
		Longitude: -93.60820007,
		Timezone:  "America/Chicago",
	},
	"KPNN": {
		ICAO:      "KPNN",
		IATA:      "PNN",
		Name:      "Princeton Municipal Airport",
		City:      "Princeton",
		State:     "Maine",
		Country:   "US",
		Elevation: 266,
		Latitude:  45.2006988525,
		Longitude: -67.5643997192,
		Timezone:  "America/New_York",
	},
	"KPNS": {
		ICAO:      "KPNS",
		IATA:      "PNS",
		Name:      "Pensacola Regional Airport",
		City:      "Pensacola",
		State:     "Florida",
		Country:   "US",
		Elevation: 121,
		Latitude:  30.473400116,
		Longitude: -87.1865997314,
		Timezone:  "America/Chicago",
	},
	"KPNT": {
		ICAO:      "KPNT",
		Name:      "Pontiac Municipal Airport",
		City:      "Pontiac",
		State:     "Illinois",
		Country:   "US",
		Elevation: 666,
		Latitude:  40.9244003296,
		Longitude: -88.6239013672,
		Timezone:  "America/Chicago",
	},
	"KPOB": {
		ICAO:      "KPOB",
		IATA:      "POB",
		Name:      "Pope Air Force Base",
		City:      "Fayetteville",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 217,
		Latitude:  35.1708984375,
		Longitude: -79.014503479,
		Timezone:  "America/New_York",
	},
	"KPOC": {
		ICAO:      "KPOC",
		IATA:      "POC",
		Name:      "Brackett Field",
		City:      "La Verne",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1011,
		Latitude:  34.0915985107,
		Longitude: -117.7819976807,
		Timezone:  "America/Los_Angeles",
	},
	"KPOE": {
		ICAO:      "KPOE",
		IATA:      "POE",
		Name:      "Polk Army Air Field",
		City:      "Fort Polk",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 330,
		Latitude:  31.0447998,
		Longitude: -93.1917038,
		Timezone:  "America/Chicago",
	},
	"KPOF": {
		ICAO:      "KPOF",
		IATA:      "POF",
		Name:      "Poplar Bluff Municipal Airport",
		City:      "Poplar Bluff",
		State:     "Missouri",
		Country:   "US",
		Elevation: 331,
		Latitude:  36.7738990784,
		Longitude: -90.3248977661,
		Timezone:  "America/Chicago",
	},
	"KPOH": {
		ICAO:      "KPOH",
		IATA:      "POH",
		Name:      "Pocahontas Municipal Airport",
		City:      "Pocahontas",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1226,
		Latitude:  42.74280167,
		Longitude: -94.64730072,
		Timezone:  "America/Chicago",
	},
	"KPOU": {
		ICAO:      "KPOU",
		IATA:      "POU",
		Name:      "Dutchess County Airport",
		City:      "Poughkeepsie",
		State:     "New-York",
		Country:   "US",
		Elevation: 165,
		Latitude:  41.6265983582,
		Longitude: -73.8842010498,
		Timezone:  "America/New_York",
	},
	"KPOY": {
		ICAO:      "KPOY",
		IATA:      "POY",
		Name:      "Powell Municipal Airport",
		City:      "Powell",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 5092,
		Latitude:  44.8671989441,
		Longitude: -108.7929992676,
		Timezone:  "America/Denver",
	},
	"KPPA": {
		ICAO:      "KPPA",
		IATA:      "PPA",
		Name:      "Perry Lefors Field",
		City:      "Pampa",
		State:     "Texas",
		Country:   "US",
		Elevation: 3245,
		Latitude:  35.6129989624,
		Longitude: -100.9960021973,
		Timezone:  "America/Chicago",
	},
	"KPPF": {
		ICAO:      "KPPF",
		IATA:      "PPF",
		Name:      "Tri City Airport",
		City:      "Parsons",
		State:     "Kansas",
		Country:   "US",
		Elevation: 900,
		Latitude:  37.32989883,
		Longitude: -95.5062027,
		Timezone:  "America/Chicago",
	},
	"KPPO": {
		ICAO:      "KPPO",
		IATA:      "LPO",
		Name:      "La Porte Municipal Airport",
		City:      "La Porte",
		State:     "Indiana",
		Country:   "US",
		Elevation: 812,
		Latitude:  41.5724983215,
		Longitude: -86.7344970703,
		Timezone:  "America/Chicago",
	},
	"KPPQ": {
		ICAO:      "KPPQ",
		Name:      "Pittsfield Penstone Municipal Airport",
		City:      "Pittsfield",
		State:     "Illinois",
		Country:   "US",
		Elevation: 710,
		Latitude:  39.6389007568,
		Longitude: -90.7783966064,
		Timezone:  "America/Chicago",
	},
	"KPQI": {
		ICAO:      "KPQI",
		IATA:      "PQI",
		Name:      "Northern Maine Regional Airport at Presque Isle",
		City:      "Presque Isle",
		State:     "Maine",
		Country:   "US",
		Elevation: 534,
		Latitude:  46.68899918,
		Longitude: -68.0447998,
		Timezone:  "America/New_York",
	},
	"KPQL": {
		ICAO:      "KPQL",
		IATA:      "PGL",
		Name:      "Trent Lott International Airport",
		City:      "Pascagoula",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 17,
		Latitude:  30.4627990723,
		Longitude: -88.5291976929,
		Timezone:  "America/Chicago",
	},
	"KPQN": {
		ICAO:      "KPQN",
		Name:      "Pipestone Municipal Airport",
		City:      "Pipestone",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1736,
		Latitude:  43.9832992554,
		Longitude: -96.3003005981,
		Timezone:  "America/Chicago",
	},
	"KPRB": {
		ICAO:      "KPRB",
		IATA:      "PRB",
		Name:      "Paso Robles Municipal Airport",
		City:      "Paso Robles",
		State:     "California",
		Country:   "US",
		Elevation: 840,
		Latitude:  35.67290115,
		Longitude: -120.6269989,
		Timezone:  "America/Los_Angeles",
	},
	"KPRC": {
		ICAO:      "KPRC",
		IATA:      "PRC",
		Name:      "Ernest A. Love Field",
		City:      "Prescott",
		State:     "Arizona",
		Country:   "US",
		Elevation: 5045,
		Latitude:  34.65449905,
		Longitude: -112.4199982,
		Timezone:  "America/Phoenix",
	},
	"KPRG": {
		ICAO:      "KPRG",
		Name:      "Edgar County Airport",
		City:      "Paris",
		State:     "Illinois",
		Country:   "US",
		Elevation: 654,
		Latitude:  39.7001991272,
		Longitude: -87.6696014404,
		Timezone:  "America/Chicago",
	},
	"KPRN": {
		ICAO:      "KPRN",
		Name:      "Mac Crenshaw Memorial Airport",
		City:      "Greenville",
		State:     "Alabama",
		Country:   "US",
		Elevation: 451,
		Latitude:  31.84569931,
		Longitude: -86.61070251,
		Timezone:  "America/Chicago",
	},
	"KPRO": {
		ICAO:      "KPRO",
		IATA:      "PRO",
		Name:      "Perry Municipal Airport",
		City:      "Perry",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1013,
		Latitude:  41.82799911,
		Longitude: -94.15989685,
		Timezone:  "America/Chicago",
	},
	"KPRX": {
		ICAO:      "KPRX",
		IATA:      "PRX",
		Name:      "Cox Field",
		City:      "Paris",
		State:     "Texas",
		Country:   "US",
		Elevation: 547,
		Latitude:  33.6366004944,
		Longitude: -95.4507980347,
		Timezone:  "America/Chicago",
	},
	"KPRZ": {
		ICAO:      "KPRZ",
		Name:      "Portales Municipal Airport",
		City:      "Portales",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4078,
		Latitude:  34.1455001831,
		Longitude: -103.4100036621,
		Timezone:  "America/Denver",
	},
	"KPSB": {
		ICAO:      "KPSB",
		IATA:      "PSB",
		Name:      "Mid State Airport",
		City:      "Philipsburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1948,
		Latitude:  40.8843994141,
		Longitude: -78.0873031616,
		Timezone:  "America/New_York",
	},
	"KPSC": {
		ICAO:      "KPSC",
		IATA:      "PSC",
		Name:      "Tri Cities Airport",
		City:      "Pasco",
		State:     "Washington",
		Country:   "US",
		Elevation: 410,
		Latitude:  46.2647018433,
		Longitude: -119.1190032959,
		Timezone:  "America/Los_Angeles",
	},
	"KPSF": {
		ICAO:      "KPSF",
		IATA:      "PSF",
		Name:      "Pittsfield Municipal Airport",
		City:      "Pittsfield",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 1194,
		Latitude:  42.4267997742,
		Longitude: -73.2929000854,
		Timezone:  "America/New_York",
	},
	"KPSK": {
		ICAO:      "KPSK",
		IATA:      "PSK",
		Name:      "New River Valley Airport",
		City:      "Dublin",
		State:     "Virginia",
		Country:   "US",
		Elevation: 2105,
		Latitude:  37.137298584,
		Longitude: -80.6784973145,
		Timezone:  "America/New_York",
	},
	"KPSM": {
		ICAO:      "KPSM",
		IATA:      "PSM",
		Name:      "Portsmouth International at Pease Airport",
		City:      "Portsmouth",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 100,
		Latitude:  43.0778999329,
		Longitude: -70.8233032227,
		Timezone:  "America/New_York",
	},
	"KPSN": {
		ICAO:      "KPSN",
		IATA:      "PSN",
		Name:      "Palestine Municipal Airport",
		City:      "Palestine",
		State:     "Texas",
		Country:   "US",
		Elevation: 423,
		Latitude:  31.7796993256,
		Longitude: -95.7062988281,
		Timezone:  "America/Chicago",
	},
	"KPSO": {
		ICAO:      "KPSO",
		IATA:      "PGO",
		Name:      "Stevens Field",
		City:      "Pagosa Springs",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7664,
		Latitude:  37.28630066,
		Longitude: -107.0559998,
		Timezone:  "America/Denver",
	},
	"KPSP": {
		ICAO:      "KPSP",
		IATA:      "PSP",
		Name:      "Palm Springs International Airport",
		City:      "Palm Springs",
		State:     "California",
		Country:   "US",
		Elevation: 477,
		Latitude:  33.82970047,
		Longitude: -116.5070037842,
		Timezone:  "America/Los_Angeles",
	},
	"KPSX": {
		ICAO:      "KPSX",
		IATA:      "PSX",
		Name:      "Palacios Municipal Airport",
		City:      "Palacios",
		State:     "Texas",
		Country:   "US",
		Elevation: 14,
		Latitude:  28.7275009155,
		Longitude: -96.2509994507,
		Timezone:  "America/Chicago",
	},
	"KPTB": {
		ICAO:      "KPTB",
		IATA:      "PTB",
		Name:      "Dinwiddie County Airport",
		City:      "Petersburg",
		State:     "Virginia",
		Country:   "US",
		Elevation: 193,
		Latitude:  37.1837997437,
		Longitude: -77.5074005127,
		Timezone:  "America/New_York",
	},
	"KPTD": {
		ICAO:      "KPTD",
		Name:      "Potsdam Municipal-Damon field",
		City:      "Potsdam",
		State:     "New-York",
		Country:   "US",
		Elevation: 474,
		Latitude:  44.67670059,
		Longitude: -74.9484024,
		Timezone:  "America/New_York",
	},
	"KPTK": {
		ICAO:      "KPTK",
		IATA:      "PTK",
		Name:      "Oakland County International Airport",
		City:      "Pontiac",
		State:     "Michigan",
		Country:   "US",
		Elevation: 980,
		Latitude:  42.6655006409,
		Longitude: -83.4200973511,
		Timezone:  "America/Detroit",
	},
	"KPTN": {
		ICAO:      "KPTN",
		IATA:      "PTN",
		Name:      "Harry P Williams Memorial Airport",
		City:      "Patterson",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 9,
		Latitude:  29.7094993591,
		Longitude: -91.3389968872,
		Timezone:  "America/Chicago",
	},
	"KPTS": {
		ICAO:      "KPTS",
		IATA:      "PTS",
		Name:      "Atkinson Municipal Airport",
		City:      "Pittsburg",
		State:     "Kansas",
		Country:   "US",
		Elevation: 950,
		Latitude:  37.4495010376,
		Longitude: -94.7311019897,
		Timezone:  "America/Chicago",
	},
	"KPTT": {
		ICAO:      "KPTT",
		IATA:      "PTT",
		Name:      "Pratt Industrial Airport",
		City:      "Pratt",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1953,
		Latitude:  37.70159912,
		Longitude: -98.74690247,
		Timezone:  "America/Chicago",
	},
	"KPTV": {
		ICAO:      "KPTV",
		IATA:      "PTV",
		Name:      "Porterville Municipal Airport",
		City:      "Porterville",
		State:     "California",
		Country:   "US",
		Elevation: 442,
		Latitude:  36.0295982361,
		Longitude: -119.06300354,
		Timezone:  "America/Los_Angeles",
	},
	"KPTW": {
		ICAO:      "KPTW",
		IATA:      "PTW",
		Name:      "Pottstown Limerick Airport",
		City:      "Pottstown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 309,
		Latitude:  40.2396011353,
		Longitude: -75.5567016602,
		Timezone:  "America/New_York",
	},
	"KPUB": {
		ICAO:      "KPUB",
		IATA:      "PUB",
		Name:      "Pueblo Memorial Airport",
		City:      "Pueblo",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4726,
		Latitude:  38.289100647,
		Longitude: -104.4970016479,
		Timezone:  "America/Denver",
	},
	"KPUC": {
		ICAO:      "KPUC",
		IATA:      "PUC",
		Name:      "Carbon County Regional/Buck Davis Field",
		City:      "Price",
		State:     "Utah",
		Country:   "US",
		Elevation: 5957,
		Latitude:  39.61389923,
		Longitude: -110.7509995,
		Timezone:  "America/Denver",
	},
	"KPUJ": {
		ICAO:      "KPUJ",
		Name:      "Paulding County Regional Airport",
		City:      "Dallas",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1289,
		Latitude:  33.913306439,
		Longitude: -84.9420833588,
		Timezone:  "America/New_York",
	},
	"KPUW": {
		ICAO:      "KPUW",
		IATA:      "PUW",
		Name:      "Pullman Moscow Regional Airport",
		City:      "Pullman/Moscow",
		State:     "Washington",
		Country:   "US",
		Elevation: 2556,
		Latitude:  46.7439002991,
		Longitude: -117.1100006104,
		Timezone:  "America/Los_Angeles",
	},
	"KPVB": {
		ICAO:      "KPVB",
		Name:      "Platteville Municipal Airport",
		City:      "Platteville",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1025,
		Latitude:  42.68939972,
		Longitude: -90.44439697,
		Timezone:  "America/Chicago",
	},
	"KPVC": {
		ICAO:      "KPVC",
		IATA:      "PVC",
		Name:      "Provincetown Municipal Airport",
		City:      "Provincetown",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 9,
		Latitude:  42.0718994141,
		Longitude: -70.2213973999,
		Timezone:  "America/New_York",
	},
	"KPVD": {
		ICAO:      "KPVD",
		IATA:      "PVD",
		Name:      "Theodore Francis Green State Airport",
		City:      "Providence",
		State:     "Rhode-Island",
		Country:   "US",
		Elevation: 55,
		Latitude:  41.7326011658,
		Longitude: -71.4204025269,
		Timezone:  "America/New_York",
	},
	"KPVE": {
		ICAO:      "KPVE",
		Name:      "Beech River Regional Airport",
		City:      "Lexington-Parsons",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 488,
		Latitude:  35.6563987732,
		Longitude: -88.1953964233,
		Timezone:  "America/Chicago",
	},
	"KPVF": {
		ICAO:      "KPVF",
		IATA:      "PVF",
		Name:      "Placerville Airport",
		City:      "Placerville",
		State:     "California",
		Country:   "US",
		Elevation: 2585,
		Latitude:  38.7242012024,
		Longitude: -120.7529983521,
		Timezone:  "America/Los_Angeles",
	},
	"KPVG": {
		ICAO:      "KPVG",
		Name:      "Hampton Roads Executive Airport",
		City:      "Norfolk",
		State:     "Virginia",
		Country:   "US",
		Elevation: 23,
		Latitude:  36.7802009583,
		Longitude: -76.4487991333,
		Timezone:  "America/New_York",
	},
	"KPVJ": {
		ICAO:      "KPVJ",
		Name:      "Pauls Valley Municipal Airport",
		City:      "Pauls Valley",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 971,
		Latitude:  34.71110153,
		Longitude: -97.22319794,
		Timezone:  "America/Chicago",
	},
	"KPVU": {
		ICAO:      "KPVU",
		IATA:      "PVU",
		Name:      "Provo Municipal Airport",
		City:      "Provo",
		State:     "Utah",
		Country:   "US",
		Elevation: 4497,
		Latitude:  40.2192001343,
		Longitude: -111.7229995728,
		Timezone:  "America/Denver",
	},
	"KPVW": {
		ICAO:      "KPVW",
		IATA:      "PVW",
		Name:      "Hale County Airport",
		City:      "Plainview",
		State:     "Texas",
		Country:   "US",
		Elevation: 3374,
		Latitude:  34.1680984497,
		Longitude: -101.7170028687,
		Timezone:  "America/Chicago",
	},
	"KPWA": {
		ICAO:      "KPWA",
		IATA:      "PWA",
		Name:      "Wiley Post Airport",
		City:      "Oklahoma City",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1300,
		Latitude:  35.53419876,
		Longitude: -97.64710236,
		Timezone:  "America/Chicago",
	},
	"KPWC": {
		ICAO:      "KPWC",
		Name:      "Pine River Regional Airport",
		City:      "Pine River",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1295,
		Latitude:  46.7248001099,
		Longitude: -94.3816986084,
		Timezone:  "America/Chicago",
	},
	"KPWD": {
		ICAO:      "KPWD",
		IATA:      "PWD",
		Name:      "Sher Wood Airport",
		City:      "Plentywood",
		State:     "Montana",
		Country:   "US",
		Elevation: 2250,
		Latitude:  48.7902984619,
		Longitude: -104.533996582,
		Timezone:  "America/Denver",
	},
	"KPWG": {
		ICAO:      "KPWG",
		Name:      "Mc Gregor Executive Airport",
		City:      "Waco",
		State:     "Texas",
		Country:   "US",
		Elevation: 592,
		Latitude:  31.48489952,
		Longitude: -97.3164978,
		Timezone:  "America/Chicago",
	},
	"KPWK": {
		ICAO:      "KPWK",
		Name:      "Chicago Executive Airport",
		City:      "Chicago/Prospect Heights/Wheeling",
		State:     "Illinois",
		Country:   "US",
		Elevation: 647,
		Latitude:  42.114222,
		Longitude: -87.901494,
		Timezone:  "America/Chicago",
	},
	"KPWM": {
		ICAO:      "KPWM",
		IATA:      "PWM",
		Name:      "Portland International Jetport Airport",
		City:      "Portland",
		State:     "Maine",
		Country:   "US",
		Elevation: 76,
		Latitude:  43.64619827,
		Longitude: -70.30930328,
		Timezone:  "America/New_York",
	},
	"KPWT": {
		ICAO:      "KPWT",
		IATA:      "PWT",
		Name:      "Bremerton National Airport",
		City:      "Bremerton",
		State:     "Washington",
		Country:   "US",
		Elevation: 444,
		Latitude:  47.4902000427,
		Longitude: -122.7649993896,
		Timezone:  "America/Los_Angeles",
	},
	"KPXE": {
		ICAO:      "KPXE",
		Name:      "Perry Houston County Airport",
		City:      "Perry",
		State:     "Georgia",
		Country:   "US",
		Elevation: 418,
		Latitude:  32.5106010437,
		Longitude: -83.7673034668,
		Timezone:  "America/New_York",
	},
	"KPYG": {
		ICAO:      "KPYG",
		Name:      "Pageland Airport",
		City:      "Pageland",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 575,
		Latitude:  34.742099762,
		Longitude: -80.345199585,
		Timezone:  "America/New_York",
	},
	"KPYM": {
		ICAO:      "KPYM",
		IATA:      "PYM",
		Name:      "Plymouth Municipal Airport",
		City:      "Plymouth",
		State:     "Indiana",
		Country:   "US",
		Elevation: 148,
		Latitude:  41.9090003967,
		Longitude: -70.7287979126,
		Timezone:  "America/New_York",
	},
	"KPYN": {
		ICAO:      "KPYN",
		Name:      "Piedmont Municipal Airport",
		City:      "Piedmont",
		State:     "Missouri",
		Country:   "US",
		Elevation: 467,
		Latitude:  37.126701355,
		Longitude: -90.7128982544,
		Timezone:  "America/Chicago",
	},
	"KPYP": {
		ICAO:      "KPYP",
		Name:      "Centre-Piedmont-Cherokee County Regional Airport",
		City:      "Centre",
		State:     "Alabama",
		Country:   "US",
		Elevation: 596,
		Latitude:  34.08997726,
		Longitude: -85.61006927,
		Timezone:  "America/Chicago",
	},
	"KPYX": {
		ICAO:      "KPYX",
		Name:      "Perryton Ochiltree County Airport",
		City:      "Perryton",
		State:     "Texas",
		Country:   "US",
		Elevation: 2918,
		Latitude:  36.4128990173,
		Longitude: -100.7519989014,
		Timezone:  "America/Chicago",
	},
	"KPZQ": {
		ICAO:      "KPZQ",
		Name:      "Presque Isle County Airport",
		City:      "Rogers City",
		State:     "Michigan",
		Country:   "US",
		Elevation: 670,
		Latitude:  45.4071006775,
		Longitude: -83.8128967285,
		Timezone:  "America/Detroit",
	},
	"KQMG": {
		ICAO:      "KQMG",
		Name:      "Mudaysis Air Base",
		State:     "Anbar",
		Country:   "IQ",
		Elevation: 1210,
		Latitude:  32.4106712341,
		Longitude: 41.9474945068,
		Timezone:  "Asia/Baghdad",
	},
	"KR47": {
		ICAO:      "KR47",
		Name:      "Ruhe's Airport",
		City:      "Leipsic",
		State:     "Ohio",
		Country:   "US",
		Elevation: 801,
		Latitude:  41.10309982,
		Longitude: -84.05190277,
		Timezone:  "America/New_York",
	},
	"KR49": {
		ICAO:      "KR49",
		Name:      "Ferry County Airport",
		City:      "Republic",
		State:     "Washington",
		Country:   "US",
		Elevation: 2522,
		Latitude:  48.7182006836,
		Longitude: -118.65599823,
		Timezone:  "America/Los_Angeles",
	},
	"KRAC": {
		ICAO:      "KRAC",
		IATA:      "RAC",
		Name:      "John H Batten Airport",
		City:      "Racine",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 674,
		Latitude:  42.7606010437,
		Longitude: -87.8152008057,
		Timezone:  "America/Chicago",
	},
	"KRAL": {
		ICAO:      "KRAL",
		IATA:      "RAL",
		Name:      "Riverside Municipal Airport",
		City:      "Riverside",
		State:     "California",
		Country:   "US",
		Elevation: 819,
		Latitude:  33.95190048,
		Longitude: -117.4449997,
		Timezone:  "America/Los_Angeles",
	},
	"KRAP": {
		ICAO:      "KRAP",
		IATA:      "RAP",
		Name:      "Rapid City Regional Airport",
		City:      "Rapid City",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 3204,
		Latitude:  44.04529953,
		Longitude: -103.0569992065,
		Timezone:  "America/Denver",
	},
	"KRAS": {
		ICAO:      "KRAS",
		Name:      "Mustang Beach Airport",
		City:      "Port Aransas",
		State:     "Texas",
		Country:   "US",
		Elevation: 5,
		Latitude:  27.8118000031,
		Longitude: -97.0887985229,
		Timezone:  "America/Chicago",
	},
	"KRAW": {
		ICAO:      "KRAW",
		Name:      "Warsaw Municipal Airport",
		City:      "Warsaw",
		State:     "Missouri",
		Country:   "US",
		Elevation: 936,
		Latitude:  38.3469009399,
		Longitude: -93.3453979492,
		Timezone:  "America/Chicago",
	},
	"KRBD": {
		ICAO:      "KRBD",
		IATA:      "RBD",
		Name:      "Dallas Executive Airport",
		City:      "Dallas",
		State:     "Texas",
		Country:   "US",
		Elevation: 660,
		Latitude:  32.6809005737,
		Longitude: -96.8682022095,
		Timezone:  "America/Chicago",
	},
	"KRBE": {
		ICAO:      "KRBE",
		Name:      "Rock County Airport",
		City:      "Bassett",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2349,
		Latitude:  42.5696983337,
		Longitude: -99.568397522,
		Timezone:  "America/Chicago",
	},
	"KRBG": {
		ICAO:      "KRBG",
		IATA:      "RBG",
		Name:      "Roseburg Regional Airport",
		City:      "Roseburg",
		State:     "Oregon",
		Country:   "US",
		Elevation: 529,
		Latitude:  43.2388000488,
		Longitude: -123.3560028076,
		Timezone:  "America/Los_Angeles",
	},
	"KRBL": {
		ICAO:      "KRBL",
		IATA:      "RBL",
		Name:      "Red Bluff Municipal Airport",
		City:      "Red Bluff",
		State:     "California",
		Country:   "US",
		Elevation: 352,
		Latitude:  40.1506996155,
		Longitude: -122.2519989014,
		Timezone:  "America/Los_Angeles",
	},
	"KRBM": {
		ICAO:      "KRBM",
		Name:      "Robinson Army Air Field",
		City:      "Camp Robinson/Little Rock",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 587,
		Latitude:  34.85010147,
		Longitude: -92.30020142,
		Timezone:  "America/Chicago",
	},
	"KRBO": {
		ICAO:      "KRBO",
		Name:      "Nueces County Airport",
		City:      "Robstown",
		State:     "Texas",
		Country:   "US",
		Elevation: 79,
		Latitude:  27.7784996033,
		Longitude: -97.6904983521,
		Timezone:  "America/Chicago",
	},
	"KRBW": {
		ICAO:      "KRBW",
		IATA:      "RBW",
		Name:      "Lowcountry Regional Airport",
		City:      "Walterboro",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 101,
		Latitude:  32.9210014343,
		Longitude: -80.6406021118,
		Timezone:  "America/New_York",
	},
	"KRCA": {
		ICAO:      "KRCA",
		IATA:      "RCA",
		Name:      "Ellsworth Air Force Base",
		City:      "Rapid City",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 3276,
		Latitude:  44.14500046,
		Longitude: -103.1039963,
		Timezone:  "America/Denver",
	},
	"KRCE": {
		ICAO:      "KRCE",
		Name:      "Clarence E Page Municipal Airport",
		City:      "Oklahoma City",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1354,
		Latitude:  35.48809814,
		Longitude: -97.82360077,
		Timezone:  "America/Chicago",
	},
	"KRCK": {
		ICAO:      "KRCK",
		IATA:      "RCK",
		Name:      "H H Coffield Regional Airport",
		City:      "Rockdale",
		State:     "Texas",
		Country:   "US",
		Elevation: 474,
		Latitude:  30.6315994263,
		Longitude: -96.9897003174,
		Timezone:  "America/Chicago",
	},
	"KRCM": {
		ICAO:      "KRCM",
		Name:      "Skyhaven Airport",
		City:      "Warrensburg",
		State:     "Missouri",
		Country:   "US",
		Elevation: 798,
		Latitude:  38.784198761,
		Longitude: -93.8029022217,
		Timezone:  "America/Chicago",
	},
	"KRCP": {
		ICAO:      "KRCP",
		Name:      "Stockton / Rooks County Regional",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1998,
		Latitude:  39.346592,
		Longitude: -99.304649,
		Timezone:  "America/Chicago",
	},
	"KRCR": {
		ICAO:      "KRCR",
		IATA:      "RCR",
		Name:      "Fulton County Airport",
		City:      "Rochester",
		State:     "Indiana",
		Country:   "US",
		Elevation: 790,
		Latitude:  41.0656013489,
		Longitude: -86.1817016602,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KRCT": {
		ICAO:      "KRCT",
		IATA:      "RCT",
		Name:      "Nartron Field",
		City:      "Reed City",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1055,
		Latitude:  43.9000015259,
		Longitude: -85.5167007446,
		Timezone:  "America/Detroit",
	},
	"KRCV": {
		ICAO:      "KRCV",
		Name:      "Astronaut Kent Rominger Airport",
		City:      "Del Norte",
		State:     "Colorado",
		Country:   "US",
		Elevation: 7955,
		Latitude:  37.7137838,
		Longitude: -106.3520185,
		Timezone:  "America/Denver",
	},
	"KRCX": {
		ICAO:      "KRCX",
		Name:      "Rusk County Airport",
		City:      "Ladysmith",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1238,
		Latitude:  45.496799469,
		Longitude: -91.00050354,
		Timezone:  "America/Chicago",
	},
	"KRCZ": {
		ICAO:      "KRCZ",
		Name:      "Richmond County Airport",
		City:      "Rockingham",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 358,
		Latitude:  34.8913002,
		Longitude: -79.75959778,
		Timezone:  "America/New_York",
	},
	"KRDD": {
		ICAO:      "KRDD",
		IATA:      "RDD",
		Name:      "Redding Municipal Airport",
		City:      "Redding",
		State:     "California",
		Country:   "US",
		Elevation: 505,
		Latitude:  40.50899887,
		Longitude: -122.2929993,
		Timezone:  "America/Los_Angeles",
	},
	"KRDG": {
		ICAO:      "KRDG",
		IATA:      "RDG",
		Name:      "Reading Regional Carl A Spaatz Field",
		City:      "Reading",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 344,
		Latitude:  40.3785018921,
		Longitude: -75.9652023315,
		Timezone:  "America/New_York",
	},
	"KRDK": {
		ICAO:      "KRDK",
		Name:      "Red Oak Municipal Airport",
		City:      "Red Oak",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1045,
		Latitude:  41.01050186,
		Longitude: -95.25990295,
		Timezone:  "America/Chicago",
	},
	"KRDM": {
		ICAO:      "KRDM",
		IATA:      "RDM",
		Name:      "Roberts Field",
		City:      "Redmond",
		State:     "Indiana",
		Country:   "US",
		Elevation: 3080,
		Latitude:  44.2541008,
		Longitude: -121.1500015,
		Timezone:  "America/Los_Angeles",
	},
	"KRDR": {
		ICAO:      "KRDR",
		IATA:      "RDR",
		Name:      "Grand Forks Air Force Base",
		City:      "Grand Forks",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 913,
		Latitude:  47.961101532,
		Longitude: -97.4011993408,
		Timezone:  "America/Chicago",
	},
	"KRDU": {
		ICAO:      "KRDU",
		IATA:      "RDU",
		Name:      "Raleigh Durham International Airport",
		City:      "Raleigh/Durham",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 435,
		Latitude:  35.8776016235,
		Longitude: -78.7874984741,
		Timezone:  "America/New_York",
	},
	"KRED": {
		ICAO:      "KRED",
		Name:      "Red Lodge Airport",
		City:      "Red Lodge",
		State:     "Montana",
		Country:   "US",
		Elevation: 5763,
		Latitude:  45.1874008179,
		Longitude: -109.266998291,
		Timezone:  "America/Denver",
	},
	"KREG": {
		ICAO:      "KREG",
		Name:      "Louisiana Regional Airport",
		City:      "Gonzales",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 4,
		Latitude:  30.1713667,
		Longitude: -90.9403861,
		Timezone:  "America/Chicago",
	},
	"KREI": {
		ICAO:      "KREI",
		Name:      "Redlands Municipal Airport",
		City:      "Redlands",
		State:     "California",
		Country:   "US",
		Elevation: 1571,
		Latitude:  34.0853004456,
		Longitude: -117.1460037231,
		Timezone:  "America/Los_Angeles",
	},
	"KREO": {
		ICAO:      "KREO",
		IATA:      "REO",
		Name:      "Rome State Airport",
		City:      "Rome",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4053,
		Latitude:  42.5777015686,
		Longitude: -117.8850021362,
		Timezone:  "America/Boise",
	},
	"KRFD": {
		ICAO:      "KRFD",
		IATA:      "RFD",
		Name:      "Chicago Rockford International Airport",
		City:      "Chicago/Rockford",
		State:     "Illinois",
		Country:   "US",
		Elevation: 742,
		Latitude:  42.195400238,
		Longitude: -89.0971984863,
		Timezone:  "America/Chicago",
	},
	"KRFG": {
		ICAO:      "KRFG",
		IATA:      "RFG",
		Name:      "Rooke Field",
		City:      "Refugio",
		State:     "Texas",
		Country:   "US",
		Elevation: 56,
		Latitude:  28.2936000824,
		Longitude: -97.3229980469,
		Timezone:  "America/Chicago",
	},
	"KRFI": {
		ICAO:      "KRFI",
		Name:      "Rusk County Airport",
		City:      "Henderson",
		State:     "Texas",
		Country:   "US",
		Elevation: 442,
		Latitude:  32.1417007446,
		Longitude: -94.8516998291,
		Timezone:  "America/Chicago",
	},
	"KRGK": {
		ICAO:      "KRGK",
		Name:      "Red Wing Regional Airport",
		City:      "Red Wing",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 778,
		Latitude:  44.58940125,
		Longitude: -92.48500061,
		Timezone:  "America/Chicago",
	},
	"KRHI": {
		ICAO:      "KRHI",
		IATA:      "RHI",
		Name:      "Rhinelander Oneida County Airport",
		City:      "Rhinelander",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1624,
		Latitude:  45.6311988831,
		Longitude: -89.4674987793,
		Timezone:  "America/Chicago",
	},
	"KRHP": {
		ICAO:      "KRHP",
		Name:      "Western Carolina Regional Airport",
		City:      "Andrews",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 1697,
		Latitude:  35.19520187,
		Longitude: -83.86299896,
		Timezone:  "America/New_York",
	},
	"KRHV": {
		ICAO:      "KRHV",
		IATA:      "RHV",
		Name:      "Reid Hillview of Santa Clara County Airport",
		City:      "San Jose",
		State:     "California",
		Country:   "US",
		Elevation: 135,
		Latitude:  37.332901001,
		Longitude: -121.8190002441,
		Timezone:  "America/Los_Angeles",
	},
	"KRIC": {
		ICAO:      "KRIC",
		IATA:      "RIC",
		Name:      "Richmond International Airport",
		City:      "Richmond",
		State:     "Virginia",
		Country:   "US",
		Elevation: 167,
		Latitude:  37.5051994324,
		Longitude: -77.3197021484,
		Timezone:  "America/New_York",
	},
	"KRID": {
		ICAO:      "KRID",
		IATA:      "RID",
		Name:      "Richmond Municipal Airport",
		City:      "Richmond",
		State:     "Indiana",
		Country:   "US",
		Elevation: 1140,
		Latitude:  39.7571983337,
		Longitude: -84.8427963257,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KRIE": {
		ICAO:      "KRIE",
		Name:      "Rice Lake Airport",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1142,
		Latitude:  45.4785995483,
		Longitude: -91.7221984863,
		Timezone:  "America/Chicago",
	},
	"KRIF": {
		ICAO:      "KRIF",
		IATA:      "RIF",
		Name:      "Richfield Municipal Airport",
		City:      "Richfield",
		State:     "Utah",
		Country:   "US",
		Elevation: 5301,
		Latitude:  38.7364006042,
		Longitude: -112.0989990234,
		Timezone:  "America/Denver",
	},
	"KRIL": {
		ICAO:      "KRIL",
		IATA:      "RIL",
		Name:      "Garfield County Regional Airport",
		City:      "Rifle",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5548,
		Latitude:  39.52629852,
		Longitude: -107.7269974,
		Timezone:  "America/Denver",
	},
	"KRIR": {
		ICAO:      "KRIR",
		IATA:      "RIR",
		Name:      "Flabob Airport",
		City:      "Riverside/Rubidoux/",
		State:     "California",
		Country:   "US",
		Elevation: 764,
		Latitude:  33.9897003174,
		Longitude: -117.4110031128,
		Timezone:  "America/Los_Angeles",
	},
	"KRIU": {
		ICAO:      "KRIU",
		Name:      "Rancho Murieta Airport",
		City:      "Rancho Murieta",
		State:     "California",
		Country:   "US",
		Elevation: 141,
		Latitude:  38.4868011475,
		Longitude: -121.1029968262,
		Timezone:  "America/Los_Angeles",
	},
	"KRIV": {
		ICAO:      "KRIV",
		IATA:      "RIV",
		Name:      "March ARB Airport",
		City:      "Riverside",
		State:     "California",
		Country:   "US",
		Elevation: 1536,
		Latitude:  33.88069916,
		Longitude: -117.2590027,
		Timezone:  "America/Los_Angeles",
	},
	"KRIW": {
		ICAO:      "KRIW",
		IATA:      "RIW",
		Name:      "Riverton Regional Airport",
		City:      "Riverton",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 5525,
		Latitude:  43.064201355,
		Longitude: -108.4599990845,
		Timezone:  "America/Denver",
	},
	"KRJD": {
		ICAO:      "KRJD",
		Name:      "Ridgely Airpark",
		City:      "Ridgely",
		State:     "Maryland",
		Country:   "US",
		Elevation: 64,
		Latitude:  38.9701004028,
		Longitude: -75.8663024902,
		Timezone:  "America/New_York",
	},
	"KRKD": {
		ICAO:      "KRKD",
		IATA:      "RKD",
		Name:      "Knox County Regional Airport",
		City:      "Rockland",
		State:     "Maine",
		Country:   "US",
		Elevation: 56,
		Latitude:  44.06010056,
		Longitude: -69.09919739,
		Timezone:  "America/New_York",
	},
	"KRKP": {
		ICAO:      "KRKP",
		IATA:      "RKP",
		Name:      "Aransas County Airport",
		City:      "Rockport",
		State:     "Texas",
		Country:   "US",
		Elevation: 24,
		Latitude:  28.0867996216,
		Longitude: -97.0446014404,
		Timezone:  "America/Chicago",
	},
	"KRKR": {
		ICAO:      "KRKR",
		IATA:      "RKR",
		Name:      "Robert S Kerr Airport",
		City:      "Poteau",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 451,
		Latitude:  35.0215988159,
		Longitude: -94.6212997437,
		Timezone:  "America/Chicago",
	},
	"KRKS": {
		ICAO:      "KRKS",
		IATA:      "RKS",
		Name:      "Rock Springs Sweetwater County Airport",
		City:      "Rock Springs",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 6764,
		Latitude:  41.59420013,
		Longitude: -109.0650024,
		Timezone:  "America/Denver",
	},
	"KRKW": {
		ICAO:      "KRKW",
		IATA:      "RKW",
		Name:      "Rockwood Municipal Airport",
		City:      "Rockwood",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1664,
		Latitude:  35.9222984314,
		Longitude: -84.6896972656,
		Timezone:  "America/Chicago",
	},
	"KRLD": {
		ICAO:      "KRLD",
		IATA:      "RLD",
		Name:      "Richland Airport",
		City:      "Richland",
		State:     "Washington",
		Country:   "US",
		Elevation: 394,
		Latitude:  46.3055992126,
		Longitude: -119.3040008545,
		Timezone:  "America/Los_Angeles",
	},
	"KRME": {
		ICAO:      "KRME",
		IATA:      "RME",
		Name:      "Griffiss International Airport",
		City:      "Rome",
		State:     "New-York",
		Country:   "US",
		Elevation: 504,
		Latitude:  43.23379898,
		Longitude: -75.40699768,
		Timezone:  "America/New_York",
	},
	"KRMG": {
		ICAO:      "KRMG",
		IATA:      "RMG",
		Name:      "Richard B Russell Airport",
		City:      "Rome",
		State:     "Georgia",
		Country:   "US",
		Elevation: 644,
		Latitude:  34.3506011963,
		Longitude: -85.1579971313,
		Timezone:  "America/New_York",
	},
	"KRMN": {
		ICAO:      "KRMN",
		Name:      "Stafford Regional Airport",
		City:      "Stafford",
		State:     "Virginia",
		Country:   "US",
		Elevation: 211,
		Latitude:  38.3981018066,
		Longitude: -77.4554977417,
		Timezone:  "America/New_York",
	},
	"KRMY": {
		ICAO:      "KRMY",
		Name:      "Brooks Field",
		City:      "Marshall",
		State:     "Colorado",
		Country:   "US",
		Elevation: 941,
		Latitude:  42.2512016296,
		Longitude: -84.9554977417,
		Timezone:  "America/Detroit",
	},
	"KRNC": {
		ICAO:      "KRNC",
		IATA:      "RNC",
		Name:      "Warren County Memorial Airport",
		City:      "Mc Minnville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1032,
		Latitude:  35.69869995,
		Longitude: -85.84380341,
		Timezone:  "America/Chicago",
	},
	"KRND": {
		ICAO:      "KRND",
		IATA:      "RND",
		Name:      "Randolph Air Force Base",
		City:      "Universal City",
		State:     "Texas",
		Country:   "US",
		Elevation: 761,
		Latitude:  29.52969933,
		Longitude: -98.27890015,
		Timezone:  "America/Chicago",
	},
	"KRNH": {
		ICAO:      "KRNH",
		IATA:      "RNH",
		Name:      "New Richmond Regional Airport",
		City:      "New Richmond",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 998,
		Latitude:  45.14830017,
		Longitude: -92.5381012,
		Timezone:  "America/Chicago",
	},
	"KRNM": {
		ICAO:      "KRNM",
		Name:      "Ramona Airport",
		City:      "Ramona",
		State:     "California",
		Country:   "US",
		Elevation: 1395,
		Latitude:  33.0391998291,
		Longitude: -116.9150009155,
		Timezone:  "America/Los_Angeles",
	},
	"KRNO": {
		ICAO:      "KRNO",
		IATA:      "RNO",
		Name:      "Reno Tahoe International Airport",
		City:      "Reno",
		State:     "Nevada",
		Country:   "US",
		Elevation: 4415,
		Latitude:  39.4990997314,
		Longitude: -119.7679977417,
		Timezone:  "America/Los_Angeles",
	},
	"KRNP": {
		ICAO:      "KRNP",
		Name:      "Owosso Community Airport",
		City:      "Owosso",
		State:     "Michigan",
		Country:   "US",
		Elevation: 736,
		Latitude:  42.9930000305,
		Longitude: -84.1389007568,
		Timezone:  "America/Detroit",
	},
	"KRNT": {
		ICAO:      "KRNT",
		IATA:      "RNT",
		Name:      "Renton Municipal Airport",
		City:      "Renton",
		State:     "Washington",
		Country:   "US",
		Elevation: 32,
		Latitude:  47.4930992126,
		Longitude: -122.216003418,
		Timezone:  "America/Los_Angeles",
	},
	"KRNV": {
		ICAO:      "KRNV",
		Name:      "Cleveland Municipal Airport",
		City:      "Cleveland",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 139,
		Latitude:  33.76110077,
		Longitude: -90.75789642,
		Timezone:  "America/Chicago",
	},
	"KROA": {
		ICAO:      "KROA",
		IATA:      "ROA",
		Name:      "Roanoke Regional Woodrum Field",
		City:      "Roanoke",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1175,
		Latitude:  37.3255004883,
		Longitude: -79.975402832,
		Timezone:  "America/New_York",
	},
	"KROC": {
		ICAO:      "KROC",
		IATA:      "ROC",
		Name:      "Greater Rochester International Airport",
		City:      "Rochester",
		State:     "New-York",
		Country:   "US",
		Elevation: 559,
		Latitude:  43.1189002991,
		Longitude: -77.6724014282,
		Timezone:  "America/New_York",
	},
	"KROG": {
		ICAO:      "KROG",
		IATA:      "ROG",
		Name:      "Rogers Municipal Carter Field",
		City:      "Rogers",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1359,
		Latitude:  36.37229919,
		Longitude: -94.10690308,
		Timezone:  "America/Chicago",
	},
	"KROS": {
		ICAO:      "KROS",
		Name:      "Rush City Regional Airport",
		City:      "Rush City",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 926,
		Latitude:  45.69800186,
		Longitude: -92.95300293,
		Timezone:  "America/Chicago",
	},
	"KROW": {
		ICAO:      "KROW",
		IATA:      "ROW",
		Name:      "Roswell International Air Center Airport",
		City:      "Roswell",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 3671,
		Latitude:  33.3016014099,
		Longitude: -104.53099823,
		Timezone:  "America/Denver",
	},
	"KROX": {
		ICAO:      "KROX",
		IATA:      "ROX",
		Name:      "Roseau Municipal Rudy Billberg Field",
		City:      "Roseau",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1060,
		Latitude:  48.85599899,
		Longitude: -95.6969986,
		Timezone:  "America/Chicago",
	},
	"KRPB": {
		ICAO:      "KRPB",
		Name:      "Belleville Municipal Airport",
		City:      "Belleville",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1537,
		Latitude:  39.8179016113,
		Longitude: -97.6595993042,
		Timezone:  "America/Chicago",
	},
	"KRPD": {
		ICAO:      "KRPD",
		IATA:      "RIE",
		Name:      "Rice Lake Regional Carl's Field",
		City:      "Rice Lake",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1109,
		Latitude:  45.41899872,
		Longitude: -91.77349854,
		Timezone:  "America/Chicago",
	},
	"KRPH": {
		ICAO:      "KRPH",
		Name:      "Graham Municipal Airport",
		City:      "Graham",
		State:     "Texas",
		Country:   "US",
		Elevation: 1123,
		Latitude:  33.1101989746,
		Longitude: -98.5552978516,
		Timezone:  "America/Chicago",
	},
	"KRPJ": {
		ICAO:      "KRPJ",
		Name:      "Rochelle Municipal Airport - Koritz Field",
		City:      "Rochelle",
		State:     "Illinois",
		Country:   "US",
		Elevation: 781,
		Latitude:  41.8930015564,
		Longitude: -89.0783004761,
		Timezone:  "America/Chicago",
	},
	"KRPX": {
		ICAO:      "KRPX",
		IATA:      "RPX",
		Name:      "Roundup Airport",
		City:      "Roundup",
		State:     "Montana",
		Country:   "US",
		Elevation: 3491,
		Latitude:  46.4749984741,
		Longitude: -108.5429992676,
		Timezone:  "America/Denver",
	},
	"KRQB": {
		ICAO:      "KRQB",
		IATA:      "WBR",
		Name:      "Roben Hood Airport",
		City:      "Big Rapids",
		State:     "Michigan",
		Country:   "US",
		Elevation: 990,
		Latitude:  43.7225990295,
		Longitude: -85.5040969849,
		Timezone:  "America/Detroit",
	},
	"KRQE": {
		ICAO:      "KRQE",
		Name:      "Window Rock Airport",
		City:      "Window Rock",
		State:     "Arizona",
		Country:   "US",
		Elevation: 6742,
		Latitude:  35.6520996094,
		Longitude: -109.0670013428,
		Timezone:  "America/Denver",
	},
	"KRQO": {
		ICAO:      "KRQO",
		IATA:      "RQO",
		Name:      "El Reno Regional Airport",
		City:      "El Reno",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1420,
		Latitude:  35.47269821,
		Longitude: -98.00579834,
		Timezone:  "America/Chicago",
	},
	"KRRL": {
		ICAO:      "KRRL",
		IATA:      "RRL",
		Name:      "Merrill Municipal Airport",
		City:      "Merrill",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1318,
		Latitude:  45.1988983154,
		Longitude: -89.7128982544,
		Timezone:  "America/Chicago",
	},
	"KRRQ": {
		ICAO:      "KRRQ",
		Name:      "Rock Rapids Municipal Airport",
		City:      "Rock Rapids",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1363,
		Latitude:  43.4522018433,
		Longitude: -96.1798019409,
		Timezone:  "America/Chicago",
	},
	"KRRT": {
		ICAO:      "KRRT",
		IATA:      "RRT",
		Name:      "Warroad International Memorial Field",
		City:      "Warroad",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1076,
		Latitude:  48.94139862,
		Longitude: -95.3483963,
		Timezone:  "America/Chicago",
	},
	"KRSL": {
		ICAO:      "KRSL",
		IATA:      "RSL",
		Name:      "Russell Municipal Airport",
		City:      "Russell",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1862,
		Latitude:  38.8721008301,
		Longitude: -98.8117980957,
		Timezone:  "America/Chicago",
	},
	"KRSN": {
		ICAO:      "KRSN",
		IATA:      "RSN",
		Name:      "Ruston Regional Airport",
		City:      "Ruston",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 311,
		Latitude:  32.5144004822,
		Longitude: -92.5916976929,
		Timezone:  "America/Chicago",
	},
	"KRST": {
		ICAO:      "KRST",
		IATA:      "RST",
		Name:      "Rochester International Airport",
		City:      "Rochester",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1317,
		Latitude:  43.9082984924,
		Longitude: -92.5,
		Timezone:  "America/Chicago",
	},
	"KRSV": {
		ICAO:      "KRSV",
		Name:      "Robinson Municipal Airport",
		City:      "Robinson",
		State:     "Illinois",
		Country:   "US",
		Elevation: 462,
		Latitude:  39.0159988403,
		Longitude: -87.6498031616,
		Timezone:  "America/Chicago",
	},
	"KRSW": {
		ICAO:      "KRSW",
		IATA:      "RSW",
		Name:      "Southwest Florida International Airport",
		City:      "Fort Myers",
		State:     "Florida",
		Country:   "US",
		Elevation: 30,
		Latitude:  26.5361995697,
		Longitude: -81.7552032471,
		Timezone:  "America/New_York",
	},
	"KRTN": {
		ICAO:      "KRTN",
		IATA:      "RTN",
		Name:      "Raton Municipal-Crews Field",
		City:      "Raton",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 6352,
		Latitude:  36.74150085,
		Longitude: -104.5019989,
		Timezone:  "America/Denver",
	},
	"KRTS": {
		ICAO:      "KRTS",
		Name:      "Reno Stead Airport",
		City:      "Reno",
		State:     "Nevada",
		Country:   "US",
		Elevation: 5050,
		Latitude:  39.6674003601,
		Longitude: -119.875999451,
		Timezone:  "America/Los_Angeles",
	},
	"KRUE": {
		ICAO:      "KRUE",
		Name:      "Russellville Regional Airport",
		City:      "Russellville",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 404,
		Latitude:  35.25910187,
		Longitude: -93.09329987,
		Timezone:  "America/Chicago",
	},
	"KRUG": {
		ICAO:      "KRUG",
		Name:      "Rugby Municipal Airport",
		City:      "Rugby",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1548,
		Latitude:  48.3903999329,
		Longitude: -100.0240020752,
		Timezone:  "America/Chicago",
	},
	"KRUQ": {
		ICAO:      "KRUQ",
		IATA:      "SRW",
		Name:      "Rowan County Airport",
		City:      "Salisbury",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 772,
		Latitude:  35.64590073,
		Longitude: -80.52030182,
		Timezone:  "America/New_York",
	},
	"KRUT": {
		ICAO:      "KRUT",
		IATA:      "RUT",
		Name:      "Rutland - Southern Vermont Regional Airport",
		City:      "Rutland",
		State:     "Vermont",
		Country:   "US",
		Elevation: 787,
		Latitude:  43.52939987,
		Longitude: -72.94960022,
		Timezone:  "America/New_York",
	},
	"KRVJ": {
		ICAO:      "KRVJ",
		Name:      "Swinton Smith Field at Reidsville Municipal Airport",
		City:      "Reidsville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 195,
		Latitude:  32.0589981079,
		Longitude: -82.1517028809,
		Timezone:  "America/New_York",
	},
	"KRVL": {
		ICAO:      "KRVL",
		IATA:      "RED",
		Name:      "Mifflin County Airport",
		City:      "Reedsville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 819,
		Latitude:  40.6773986816,
		Longitude: -77.6268005371,
		Timezone:  "America/New_York",
	},
	"KRVN": {
		ICAO:      "KRVN",
		Name:      "Hawkins County Airport",
		City:      "Rogersville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1255,
		Latitude:  36.4575996399,
		Longitude: -82.8850021362,
		Timezone:  "America/New_York",
	},
	"KRVS": {
		ICAO:      "KRVS",
		IATA:      "RVS",
		Name:      "Richard Lloyd Jones Jr Airport",
		City:      "Tulsa",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 638,
		Latitude:  36.0396003723,
		Longitude: -95.9845962524,
		Timezone:  "America/Chicago",
	},
	"KRWF": {
		ICAO:      "KRWF",
		IATA:      "RWF",
		Name:      "Redwood Falls Municipal Airport",
		City:      "Redwood Falls",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1024,
		Latitude:  44.54719925,
		Longitude: -95.08229828,
		Timezone:  "America/Chicago",
	},
	"KRWI": {
		ICAO:      "KRWI",
		IATA:      "RWI",
		Name:      "Rocky Mount Wilson Regional Airport",
		City:      "Rocky Mount",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 159,
		Latitude:  35.856300354,
		Longitude: -77.8918991089,
		Timezone:  "America/New_York",
	},
	"KRWL": {
		ICAO:      "KRWL",
		IATA:      "RWL",
		Name:      "Rawlins Municipal Harvey Field",
		City:      "Rawlins",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 6813,
		Latitude:  41.80559921,
		Longitude: -107.1999969,
		Timezone:  "America/Denver",
	},
	"KRWN": {
		ICAO:      "KRWN",
		Name:      "Arens Field",
		City:      "Winamac",
		State:     "Indiana",
		Country:   "US",
		Elevation: 711,
		Latitude:  41.092300415,
		Longitude: -86.6128997803,
		Timezone:  "America/Indiana/Winamac",
	},
	"KRWV": {
		ICAO:      "KRWV",
		Name:      "Caldwell Municipal Airport",
		City:      "Caldwell",
		State:     "Texas",
		Country:   "US",
		Elevation: 391,
		Latitude:  30.515499115,
		Longitude: -96.7041015625,
		Timezone:  "America/Chicago",
	},
	"KRXE": {
		ICAO:      "KRXE",
		IATA:      "RXE",
		Name:      "Rexburg Madison County Airport",
		City:      "Rexburg",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4858,
		Latitude:  43.8339004517,
		Longitude: -111.805000305,
		Timezone:  "America/Boise",
	},
	"KRYM": {
		ICAO:      "KRYM",
		Name:      "Ray S Miller Army Air Field",
		City:      "Camp Ripley",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1150,
		Latitude:  46.0912017822,
		Longitude: -94.360496521,
		Timezone:  "America/Chicago",
	},
	"KRYN": {
		ICAO:      "KRYN",
		Name:      "Ryan Field",
		City:      "Tucson",
		State:     "Arizona",
		Country:   "US",
		Elevation: 2417,
		Latitude:  32.14220047,
		Longitude: -111.1750030518,
		Timezone:  "America/Phoenix",
	},
	"KRYV": {
		ICAO:      "KRYV",
		Name:      "Watertown Municipal Airport",
		City:      "Watertown",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 833,
		Latitude:  43.1696014404,
		Longitude: -88.723197937,
		Timezone:  "America/Chicago",
	},
	"KRYW": {
		ICAO:      "KRYW",
		Name:      "Lago Vista Tx Rusty Allen Airport",
		City:      "Lago Vista",
		State:     "Texas",
		Country:   "US",
		Elevation: 1231,
		Latitude:  30.4986000061,
		Longitude: -97.9694976807,
		Timezone:  "America/Chicago",
	},
	"KRYY": {
		ICAO:      "KRYY",
		Name:      "Cobb County-Mc Collum Field",
		City:      "Atlanta",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1040,
		Latitude:  34.01319885,
		Longitude: -84.59860229,
		Timezone:  "America/New_York",
	},
	"KRZL": {
		ICAO:      "KRZL",
		IATA:      "RNZ",
		Name:      "Jasper County Airport",
		City:      "Rensselaer",
		State:     "Indiana",
		Country:   "US",
		Elevation: 698,
		Latitude:  40.9478988647,
		Longitude: -87.1826019287,
		Timezone:  "America/Chicago",
	},
	"KRZN": {
		ICAO:      "KRZN",
		Name:      "Burnett County Airport",
		City:      "Siren",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 989,
		Latitude:  45.8227005005,
		Longitude: -92.3724975586,
		Timezone:  "America/Chicago",
	},
	"KRZR": {
		ICAO:      "KRZR",
		Name:      "Cleveland Regional Jetport C",
		City:      "Cleveland",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 866,
		Latitude:  35.2123364,
		Longitude: -84.7991994,
		Timezone:  "America/New_York",
	},
	"KRZT": {
		ICAO:      "KRZT",
		Name:      "Ross County Airport",
		City:      "Chillicothe",
		State:     "Ohio",
		Country:   "US",
		Elevation: 725,
		Latitude:  39.4403991699,
		Longitude: -83.0231018066,
		Timezone:  "America/New_York",
	},
	"KRZZ": {
		ICAO:      "KRZZ",
		IATA:      "RZZ",
		Name:      "Halifax County Airport",
		City:      "Roanoke Rapids",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 256,
		Latitude:  36.4394989014,
		Longitude: -77.7092971802,
		Timezone:  "America/New_York",
	},
	"KS01": {
		ICAO:      "KS01",
		Name:      "Conrad Airport",
		City:      "Conrad",
		State:     "Montana",
		Country:   "US",
		Elevation: 3545,
		Latitude:  48.168598175,
		Longitude: -111.9759979248,
		Timezone:  "America/Denver",
	},
	"KS03": {
		ICAO:      "KS03",
		IATA:      "AHM",
		Name:      "Ashland Municipal Sumner Parker Field",
		City:      "Ashland",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1885,
		Latitude:  42.1902999878,
		Longitude: -122.661003113,
		Timezone:  "America/Los_Angeles",
	},
	"KS05": {
		ICAO:      "KS05",
		IATA:      "BDY",
		Name:      "Bandon State Airport",
		City:      "Bandon",
		State:     "Oregon",
		Country:   "US",
		Elevation: 122,
		Latitude:  43.08649826,
		Longitude: -124.4079971,
		Timezone:  "America/Los_Angeles",
	},
	"KS10": {
		ICAO:      "KS10",
		Name:      "Lake Chelan Airport",
		City:      "Chelan",
		State:     "Washington",
		Country:   "US",
		Elevation: 1263,
		Latitude:  47.8660011292,
		Longitude: -119.9430007935,
		Timezone:  "America/Los_Angeles",
	},
	"KS12": {
		ICAO:      "KS12",
		Name:      "Albany Municipal Airport",
		City:      "Albany",
		State:     "Oregon",
		Country:   "US",
		Elevation: 226,
		Latitude:  44.6377983093,
		Longitude: -123.0589981079,
		Timezone:  "America/Los_Angeles",
	},
	"KS17": {
		ICAO:      "KS17",
		Name:      "Twin Lakes Airport",
		City:      "Graniteville",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 540,
		Latitude:  33.6456985474,
		Longitude: -81.8671035767,
		Timezone:  "America/New_York",
	},
	"KS19": {
		ICAO:      "KS19",
		Name:      "Mc Cormick County Airport",
		City:      "Mc Cormick",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 459,
		Latitude:  33.90810013,
		Longitude: -82.26689911,
		Timezone:  "America/New_York",
	},
	"KS20": {
		ICAO:      "KS20",
		Name:      "Goldendale Airport",
		City:      "Goldendale",
		State:     "Washington",
		Country:   "US",
		Elevation: 1678,
		Latitude:  45.8320999146,
		Longitude: -120.8450012207,
		Timezone:  "America/Los_Angeles",
	},
	"KS21": {
		ICAO:      "KS21",
		IATA:      "SUO",
		Name:      "Sunriver Airport",
		City:      "Sunriver",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4164,
		Latitude:  43.8763008118,
		Longitude: -121.45300293,
		Timezone:  "America/Los_Angeles",
	},
	"KS23": {
		ICAO:      "KS23",
		Name:      "Ione Municipal Airport",
		City:      "Ione",
		State:     "Washington",
		Country:   "US",
		Elevation: 2108,
		Latitude:  48.7080993652,
		Longitude: -117.4130020142,
		Timezone:  "America/Los_Angeles",
	},
	"KS24": {
		ICAO:      "KS24",
		Name:      "Sandusky County Regional Airport",
		City:      "Fremont, Ohio",
		State:     "Ohio",
		Country:   "US",
		Elevation: 665,
		Latitude:  41.29575,
		Longitude: -83.0372222,
		Timezone:  "America/New_York",
	},
	"KS25": {
		ICAO:      "KS25",
		Name:      "Watford City Municipal Airport",
		City:      "Watford City",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 2111,
		Latitude:  47.79570007,
		Longitude: -103.2539978,
		Timezone:  "America/Chicago",
	},
	"KS27": {
		ICAO:      "KS27",
		Name:      "Kalispell City Airport",
		City:      "Kalispell",
		State:     "Montana",
		Country:   "US",
		Elevation: 2932,
		Latitude:  48.1786003113,
		Longitude: -114.3040008545,
		Timezone:  "America/Denver",
	},
	"KS32": {
		ICAO:      "KS32",
		Name:      "Cooperstown Municipal Airport",
		City:      "Cooperstown",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1424,
		Latitude:  47.4227981567,
		Longitude: -98.1059036255,
		Timezone:  "America/Chicago",
	},
	"KS33": {
		ICAO:      "KS33",
		Name:      "Madras Municipal Airport",
		City:      "Madras",
		State:     "Oregon",
		Country:   "US",
		Elevation: 2437,
		Latitude:  44.67020035,
		Longitude: -121.1549988,
		Timezone:  "America/Los_Angeles",
	},
	"KS34": {
		ICAO:      "KS34",
		Name:      "Plains Airport",
		City:      "Plains",
		State:     "Montana",
		Country:   "US",
		Elevation: 2467,
		Latitude:  47.47240067,
		Longitude: -114.9000015,
		Timezone:  "America/Denver",
	},
	"KS36": {
		ICAO:      "KS36",
		Name:      "Crest Airpark",
		City:      "Kent",
		State:     "Washington",
		Country:   "US",
		Elevation: 472,
		Latitude:  47.3371009827,
		Longitude: -122.1039962769,
		Timezone:  "America/Los_Angeles",
	},
	"KS39": {
		ICAO:      "KS39",
		Name:      "Prineville Airport",
		City:      "Prineville",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3250,
		Latitude:  44.2869987488,
		Longitude: -120.9039993286,
		Timezone:  "America/Los_Angeles",
	},
	"KS40": {
		ICAO:      "KS40",
		Name:      "Prosser Airport",
		City:      "Prosser",
		State:     "Washington",
		Country:   "US",
		Elevation: 697,
		Latitude:  46.21340179,
		Longitude: -119.7910004,
		Timezone:  "America/Los_Angeles",
	},
	"KS42": {
		ICAO:      "KS42",
		Name:      "Springer Municipal Airport",
		City:      "Springer",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 5891,
		Latitude:  36.3269996643,
		Longitude: -104.6200027466,
		Timezone:  "America/Denver",
	},
	"KS45": {
		ICAO:      "KS45",
		Name:      "Siletz Bay State Airport",
		City:      "Gleneden Beach",
		State:     "Oregon",
		Country:   "US",
		Elevation: 69,
		Latitude:  44.8768997192,
		Longitude: -124.0289993286,
		Timezone:  "America/Los_Angeles",
	},
	"KS48": {
		ICAO:      "KS48",
		Name:      "Country Squire Airpark",
		City:      "Sandy",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1175,
		Latitude:  45.3544006348,
		Longitude: -122.2679977417,
		Timezone:  "America/Los_Angeles",
	},
	"KS50": {
		ICAO:      "KS50",
		Name:      "Auburn Municipal Airport",
		City:      "Auburn",
		State:     "Washington",
		Country:   "US",
		Elevation: 63,
		Latitude:  47.3277015686,
		Longitude: -122.2269973755,
		Timezone:  "America/Los_Angeles",
	},
	"KS52": {
		ICAO:      "KS52",
		Name:      "Methow Valley State Airport",
		City:      "Winthrop",
		State:     "Washington",
		Country:   "US",
		Elevation: 1706,
		Latitude:  48.4249000549,
		Longitude: -120.1460037231,
		Timezone:  "America/Los_Angeles",
	},
	"KS59": {
		ICAO:      "KS59",
		Name:      "Libby Airport",
		City:      "Libby",
		State:     "Montana",
		Country:   "US",
		Elevation: 2601,
		Latitude:  48.2837982178,
		Longitude: -115.4899978638,
		Timezone:  "America/Denver",
	},
	"KS64": {
		ICAO:      "KS64",
		Name:      "Stanford Airport",
		City:      "Stanford",
		State:     "Montana",
		Country:   "US",
		Elevation: 4327,
		Latitude:  47.14720154,
		Longitude: -110.2300034,
		Timezone:  "America/Denver",
	},
	"KS67": {
		ICAO:      "KS67",
		Name:      "Nampa Municipal Airport",
		City:      "Nampa",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2537,
		Latitude:  43.5812988281,
		Longitude: -116.5230026245,
		Timezone:  "America/Boise",
	},
	"KS69": {
		ICAO:      "KS69",
		Name:      "Lincoln Airport",
		City:      "Lincoln",
		State:     "Montana",
		Country:   "US",
		Elevation: 4603,
		Latitude:  46.95439911,
		Longitude: -112.6500015,
		Timezone:  "America/Denver",
	},
	"KS70": {
		ICAO:      "KS70",
		Name:      "Othello Municipal Airport",
		City:      "Othello",
		State:     "Washington",
		Country:   "US",
		Elevation: 1139,
		Latitude:  46.79489899,
		Longitude: -119.0790024,
		Timezone:  "America/Los_Angeles",
	},
	"KS71": {
		ICAO:      "KS71",
		Name:      "Edgar G Obie Airport",
		City:      "Chinook",
		State:     "Montana",
		Country:   "US",
		Elevation: 2416,
		Latitude:  48.5920982361,
		Longitude: -109.2509994507,
		Timezone:  "America/Denver",
	},
	"KS72": {
		ICAO:      "KS72",
		Name:      "St Maries Municipal Airport",
		City:      "St Maries",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2127,
		Latitude:  47.3277015686,
		Longitude: -116.577003479,
		Timezone:  "America/Los_Angeles",
	},
	"KS73": {
		ICAO:      "KS73",
		Name:      "Kamiah Municipal Airport",
		City:      "Kamiah",
		State:     "Idaho",
		Country:   "US",
		Elevation: 1194,
		Latitude:  46.2192993164,
		Longitude: -116.0130004883,
		Timezone:  "America/Los_Angeles",
	},
	"KS75": {
		ICAO:      "KS75",
		Name:      "Payette Municipal Airport",
		City:      "Payette",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2228,
		Latitude:  44.0915985107,
		Longitude: -116.9010009766,
		Timezone:  "America/Boise",
	},
	"KS78": {
		ICAO:      "KS78",
		Name:      "Emmett Municipal Airport",
		City:      "Emmett",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2350,
		Latitude:  43.849899292,
		Longitude: -116.5429992676,
		Timezone:  "America/Boise",
	},
	"KS81": {
		ICAO:      "KS81",
		Name:      "Indian Creek USFS Airport",
		City:      "Indian Creek, Idaho",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4718,
		Latitude:  44.7611389,
		Longitude: -115.1073611,
		Timezone:  "America/Boise",
	},
	"KS83": {
		ICAO:      "KS83",
		Name:      "Shoshone County Airport",
		City:      "Kellogg",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2223,
		Latitude:  47.5476989746,
		Longitude: -116.18800354,
		Timezone:  "America/Los_Angeles",
	},
	"KS84": {
		ICAO:      "KS84",
		Name:      "Cottonwood Municipal Airport",
		City:      "Cottonwood",
		State:     "Idaho",
		Country:   "US",
		Elevation: 3474,
		Latitude:  46.0387992859,
		Longitude: -116.3320007324,
		Timezone:  "America/Los_Angeles",
	},
	"KS85": {
		ICAO:      "KS85",
		Name:      "Big Sky Field",
		City:      "Culbertson",
		State:     "Montana",
		Country:   "US",
		Elevation: 1954,
		Latitude:  48.153301239,
		Longitude: -104.5039978027,
		Timezone:  "America/Denver",
	},
	"KS87": {
		ICAO:      "KS87",
		Name:      "Weiser Municipal Airport",
		City:      "Weiser",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2120,
		Latitude:  44.20679855,
		Longitude: -116.961998,
		Timezone:  "America/Boise",
	},
	"KS94": {
		ICAO:      "KS94",
		Name:      "Port of Whitman Business Air Center Airport",
		City:      "Colfax",
		State:     "Washington",
		Country:   "US",
		Elevation: 2181,
		Latitude:  46.8586997986,
		Longitude: -117.4140014648,
		Timezone:  "America/Los_Angeles",
	},
	"KS95": {
		ICAO:      "KS95",
		Name:      "Martin Field",
		City:      "College Place",
		State:     "Washington",
		Country:   "US",
		Elevation: 746,
		Latitude:  46.0469017029,
		Longitude: -118.4169998169,
		Timezone:  "America/Los_Angeles",
	},
	"KS97": {
		ICAO:      "KS97",
		Name:      "Anderson Field",
		City:      "Brewster",
		State:     "Washington",
		Country:   "US",
		Elevation: 914,
		Latitude:  48.1049003601,
		Longitude: -119.7210006714,
		Timezone:  "America/Los_Angeles",
	},
	"KS98": {
		ICAO:      "KS98",
		IATA:      "VSK",
		Name:      "Vista Field",
		City:      "Kennewick",
		State:     "Washington",
		Country:   "US",
		Elevation: 534,
		Latitude:  46.2186012268,
		Longitude: -119.209999084,
		Timezone:  "America/Los_Angeles",
	},
	"KSAA": {
		ICAO:      "KSAA",
		IATA:      "SAA",
		Name:      "Shively Field",
		City:      "Saratoga",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 7012,
		Latitude:  41.4449005127,
		Longitude: -106.8239974976,
		Timezone:  "America/Denver",
	},
	"KSAC": {
		ICAO:      "KSAC",
		IATA:      "SAC",
		Name:      "Sacramento Executive Airport",
		City:      "Sacramento",
		State:     "California",
		Country:   "US",
		Elevation: 24,
		Latitude:  38.5125007629,
		Longitude: -121.4929962158,
		Timezone:  "America/Los_Angeles",
	},
	"KSAD": {
		ICAO:      "KSAD",
		IATA:      "SAD",
		Name:      "Safford Regional Airport",
		City:      "Safford",
		State:     "Arizona",
		Country:   "US",
		Elevation: 3179,
		Latitude:  32.85480118,
		Longitude: -109.6350021,
		Timezone:  "America/Phoenix",
	},
	"KSAF": {
		ICAO:      "KSAF",
		IATA:      "SAF",
		Name:      "Santa Fe Municipal Airport",
		City:      "Santa Fe",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 6348,
		Latitude:  35.617099762,
		Longitude: -106.088996887,
		Timezone:  "America/Denver",
	},
	"KSAN": {
		ICAO:      "KSAN",
		IATA:      "SAN",
		Name:      "San Diego International Airport",
		City:      "San Diego",
		State:     "California",
		Country:   "US",
		Elevation: 17,
		Latitude:  32.7336006165,
		Longitude: -117.1900024414,
		Timezone:  "America/Los_Angeles",
	},
	"KSAR": {
		ICAO:      "KSAR",
		IATA:      "SAR",
		Name:      "Sparta Community Hunter Field",
		City:      "Sparta",
		State:     "Illinois",
		Country:   "US",
		Elevation: 538,
		Latitude:  38.1488990784,
		Longitude: -89.6986999512,
		Timezone:  "America/Chicago",
	},
	"KSAS": {
		ICAO:      "KSAS",
		IATA:      "SAS",
		Name:      "Salton Sea Airport",
		City:      "Salton City",
		State:     "California",
		Country:   "US",
		Elevation: -84,
		Latitude:  33.2414016724,
		Longitude: -115.952003479,
		Timezone:  "America/Los_Angeles",
	},
	"KSAT": {
		ICAO:      "KSAT",
		IATA:      "SAT",
		Name:      "San Antonio International Airport",
		City:      "San Antonio",
		State:     "Texas",
		Country:   "US",
		Elevation: 809,
		Latitude:  29.533700943,
		Longitude: -98.4698028564,
		Timezone:  "America/Chicago",
	},
	"KSAV": {
		ICAO:      "KSAV",
		IATA:      "SAV",
		Name:      "Savannah Hilton Head International Airport",
		City:      "Savannah",
		State:     "Georgia",
		Country:   "US",
		Elevation: 50,
		Latitude:  32.12760162,
		Longitude: -81.20210266,
		Timezone:  "America/New_York",
	},
	"KSAW": {
		ICAO:      "KSAW",
		IATA:      "MQT",
		Name:      "Sawyer International Airport",
		City:      "Marquette",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1221,
		Latitude:  46.3535995483,
		Longitude: -87.395401001,
		Timezone:  "America/Detroit",
	},
	"KSAZ": {
		ICAO:      "KSAZ",
		Name:      "Staples Municipal Airport",
		City:      "Staples",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1287,
		Latitude:  46.3809013367,
		Longitude: -94.806602478,
		Timezone:  "America/Chicago",
	},
	"KSBA": {
		ICAO:      "KSBA",
		IATA:      "SBA",
		Name:      "Santa Barbara Municipal Airport",
		City:      "Santa Barbara",
		State:     "California",
		Country:   "US",
		Elevation: 13,
		Latitude:  34.42620087,
		Longitude: -119.8399963,
		Timezone:  "America/Los_Angeles",
	},
	"KSBD": {
		ICAO:      "KSBD",
		IATA:      "SBD",
		Name:      "San Bernardino International Airport",
		City:      "San Bernardino",
		State:     "California",
		Country:   "US",
		Elevation: 1159,
		Latitude:  34.0954017639,
		Longitude: -117.2350006104,
		Timezone:  "America/Los_Angeles",
	},
	"KSBM": {
		ICAO:      "KSBM",
		IATA:      "SBM",
		Name:      "Sheboygan County Memorial Airport",
		City:      "Sheboygan",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 755,
		Latitude:  43.76959991,
		Longitude: -87.85140228,
		Timezone:  "America/Chicago",
	},
	"KSBN": {
		ICAO:      "KSBN",
		IATA:      "SBN",
		Name:      "South Bend Regional Airport",
		City:      "South Bend",
		State:     "Indiana",
		Country:   "US",
		Elevation: 799,
		Latitude:  41.7086982727,
		Longitude: -86.3172988892,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KSBO": {
		ICAO:      "KSBO",
		Name:      "Emanuel County Airport",
		City:      "Swainsboro",
		State:     "Georgia",
		Country:   "US",
		Elevation: 327,
		Latitude:  32.6091003418,
		Longitude: -82.3699035645,
		Timezone:  "America/New_York",
	},
	"KSBP": {
		ICAO:      "KSBP",
		IATA:      "SBP",
		Name:      "San Luis County Regional Airport",
		City:      "San Luis Obispo",
		State:     "California",
		Country:   "US",
		Elevation: 212,
		Latitude:  35.2368011475,
		Longitude: -120.641998291,
		Timezone:  "America/Los_Angeles",
	},
	"KSBS": {
		ICAO:      "KSBS",
		IATA:      "SBS",
		Name:      "Steamboat Springs Bob Adams Field",
		City:      "Steamboat Springs",
		State:     "Colorado",
		Country:   "US",
		Elevation: 6882,
		Latitude:  40.5163002,
		Longitude: -106.8659973,
		Timezone:  "America/Denver",
	},
	"KSBU": {
		ICAO:      "KSBU",
		Name:      "Blue Earth Municipal Airport",
		City:      "Blue Earth",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1107,
		Latitude:  43.5952987671,
		Longitude: -94.0927963257,
		Timezone:  "America/Chicago",
	},
	"KSBX": {
		ICAO:      "KSBX",
		IATA:      "SBX",
		Name:      "Shelby Airport",
		City:      "Shelby",
		State:     "Montana",
		Country:   "US",
		Elevation: 3443,
		Latitude:  48.5406990051,
		Longitude: -111.8710021973,
		Timezone:  "America/Denver",
	},
	"KSBY": {
		ICAO:      "KSBY",
		IATA:      "SBY",
		Name:      "Salisbury Ocean City Wicomico Regional Airport",
		City:      "Salisbury",
		State:     "Maryland",
		Country:   "US",
		Elevation: 52,
		Latitude:  38.3404998779,
		Longitude: -75.5102996826,
		Timezone:  "America/New_York",
	},
	"KSCB": {
		ICAO:      "KSCB",
		IATA:      "SCB",
		Name:      "Scribner State Airport",
		City:      "Scribner",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1325,
		Latitude:  41.6102981567,
		Longitude: -96.6298980713,
		Timezone:  "America/Chicago",
	},
	"KSCD": {
		ICAO:      "KSCD",
		Name:      "Merkel Field Sylacauga Municipal Airport",
		City:      "Sylacauga",
		State:     "Alabama",
		Country:   "US",
		Elevation: 569,
		Latitude:  33.1717987061,
		Longitude: -86.3054962158,
		Timezone:  "America/Chicago",
	},
	"KSCH": {
		ICAO:      "KSCH",
		IATA:      "SCH",
		Name:      "Schenectady County Airport",
		City:      "Schenectady",
		State:     "New-York",
		Country:   "US",
		Elevation: 378,
		Latitude:  42.8525009155,
		Longitude: -73.9289016724,
		Timezone:  "America/New_York",
	},
	"KSCK": {
		ICAO:      "KSCK",
		IATA:      "SCK",
		Name:      "Stockton Metropolitan Airport",
		City:      "Stockton",
		State:     "California",
		Country:   "US",
		Elevation: 33,
		Latitude:  37.8941993713,
		Longitude: -121.2379989624,
		Timezone:  "America/Los_Angeles",
	},
	"KSCX": {
		ICAO:      "KSCX",
		Name:      "Scott Municipal Airport",
		City:      "Oneida",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1545,
		Latitude:  36.4556007385,
		Longitude: -84.5857009888,
		Timezone:  "America/New_York",
	},
	"KSDA": {
		ICAO:      "KSDA",
		Name:      "Shenandoah Municipal Airport",
		City:      "Shenandoah",
		State:     "Iowa",
		Country:   "US",
		Elevation: 971,
		Latitude:  40.7515983582,
		Longitude: -95.4136962891,
		Timezone:  "America/Chicago",
	},
	"KSDC": {
		ICAO:      "KSDC",
		Name:      "Williamson Sodus Airport",
		City:      "Williamson/Sodus",
		State:     "New-York",
		Country:   "US",
		Elevation: 424,
		Latitude:  43.23469925,
		Longitude: -77.1210022,
		Timezone:  "America/New_York",
	},
	"KSDF": {
		ICAO:      "KSDF",
		IATA:      "SDF",
		Name:      "Louisville International Standiford Field",
		City:      "Louisville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 501,
		Latitude:  38.1744003296,
		Longitude: -85.736000061,
		Timezone:  "America/Kentucky/Louisville",
	},
	"KSDL": {
		ICAO:      "KSDL",
		IATA:      "SCF",
		Name:      "Scottsdale Airport",
		City:      "Scottsdale",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1510,
		Latitude:  33.6228981018,
		Longitude: -111.9110031128,
		Timezone:  "America/Phoenix",
	},
	"KSDM": {
		ICAO:      "KSDM",
		IATA:      "SDM",
		Name:      "Brown Field Municipal Airport",
		City:      "San Diego",
		State:     "California",
		Country:   "US",
		Elevation: 526,
		Latitude:  32.5722999573,
		Longitude: -116.9800033569,
		Timezone:  "America/Los_Angeles",
	},
	"KSDY": {
		ICAO:      "KSDY",
		IATA:      "SDY",
		Name:      "Sidney Richland Municipal Airport",
		City:      "Sidney",
		State:     "Montana",
		Country:   "US",
		Elevation: 1985,
		Latitude:  47.70690155,
		Longitude: -104.1930008,
		Timezone:  "America/Denver",
	},
	"KSEA": {
		ICAO:      "KSEA",
		IATA:      "SEA",
		Name:      "Seattle Tacoma International Airport",
		City:      "Seattle",
		State:     "Washington",
		Country:   "US",
		Elevation: 433,
		Latitude:  47.4490013123,
		Longitude: -122.3089981079,
		Timezone:  "America/Los_Angeles",
	},
	"KSEE": {
		ICAO:      "KSEE",
		IATA:      "SEE",
		Name:      "Gillespie Field",
		City:      "San Diego/El Cajon",
		State:     "California",
		Country:   "US",
		Elevation: 388,
		Latitude:  32.8261985779,
		Longitude: -116.9720001221,
		Timezone:  "America/Los_Angeles",
	},
	"KSEF": {
		ICAO:      "KSEF",
		IATA:      "SEF",
		Name:      "Sebring Regional Airport",
		City:      "Sebring",
		State:     "Florida",
		Country:   "US",
		Elevation: 62,
		Latitude:  27.45639992,
		Longitude: -81.3423996,
		Timezone:  "America/New_York",
	},
	"KSEG": {
		ICAO:      "KSEG",
		IATA:      "SEG",
		Name:      "Penn Valley Airport",
		City:      "Selinsgrove",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 450,
		Latitude:  40.8205986023,
		Longitude: -76.863899231,
		Timezone:  "America/New_York",
	},
	"KSEM": {
		ICAO:      "KSEM",
		IATA:      "SEM",
		Name:      "Craig Field",
		City:      "Selma",
		State:     "Alabama",
		Country:   "US",
		Elevation: 166,
		Latitude:  32.3438987732,
		Longitude: -86.9878005981,
		Timezone:  "America/Chicago",
	},
	"KSEP": {
		ICAO:      "KSEP",
		IATA:      "SEP",
		Name:      "Clark Field Municipal Airport",
		City:      "Stephenville",
		State:     "Texas",
		Country:   "US",
		Elevation: 1321,
		Latitude:  32.2153015137,
		Longitude: -98.177696228,
		Timezone:  "America/Chicago",
	},
	"KSEQ": {
		ICAO:      "KSEQ",
		Name:      "Randolph Air Force Base Auxiliary Airport",
		City:      "Seguin",
		State:     "Texas",
		Country:   "US",
		Elevation: 525,
		Latitude:  29.56579971,
		Longitude: -97.90830231,
		Timezone:  "America/Chicago",
	},
	"KSER": {
		ICAO:      "KSER",
		IATA:      "SER",
		Name:      "Freeman Municipal Airport",
		City:      "Seymour",
		State:     "Indiana",
		Country:   "US",
		Elevation: 583,
		Latitude:  38.9235992432,
		Longitude: -85.9074020386,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KSET": {
		ICAO:      "KSET",
		Name:      "St Charles County Smartt Airport",
		City:      "St Charles",
		State:     "Missouri",
		Country:   "US",
		Elevation: 437,
		Latitude:  38.92969894,
		Longitude: -90.43000031,
		Timezone:  "America/Chicago",
	},
	"KSEZ": {
		ICAO:      "KSEZ",
		IATA:      "SDX",
		Name:      "Sedona Airport",
		City:      "Sedona",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4830,
		Latitude:  34.8485984802,
		Longitude: -111.7880020142,
		Timezone:  "America/Phoenix",
	},
	"KSFB": {
		ICAO:      "KSFB",
		IATA:      "SFB",
		Name:      "Orlando Sanford International Airport",
		City:      "Orlando",
		State:     "Florida",
		Country:   "US",
		Elevation: 55,
		Latitude:  28.7775993347,
		Longitude: -81.2375030518,
		Timezone:  "America/New_York",
	},
	"KSFF": {
		ICAO:      "KSFF",
		IATA:      "SFF",
		Name:      "Felts Field",
		City:      "Spokane",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1953,
		Latitude:  47.682800293,
		Longitude: -117.3229980469,
		Timezone:  "America/Los_Angeles",
	},
	"KSFM": {
		ICAO:      "KSFM",
		IATA:      "SFM",
		Name:      "Sanford Regional Airport",
		City:      "Sanford",
		State:     "Maine",
		Country:   "US",
		Elevation: 244,
		Latitude:  43.393901825,
		Longitude: -70.7080001831,
		Timezone:  "America/New_York",
	},
	"KSFO": {
		ICAO:      "KSFO",
		IATA:      "SFO",
		Name:      "San Francisco International Airport",
		City:      "San Francisco",
		State:     "California",
		Country:   "US",
		Elevation: 13,
		Latitude:  37.6189994812,
		Longitude: -122.375,
		Timezone:  "America/Los_Angeles",
	},
	"KSFQ": {
		ICAO:      "KSFQ",
		Name:      "Suffolk Executive Airport",
		City:      "Suffolk",
		State:     "Virginia",
		Country:   "US",
		Elevation: 72,
		Latitude:  36.6823997498,
		Longitude: -76.6018981934,
		Timezone:  "America/New_York",
	},
	"KSFY": {
		ICAO:      "KSFY",
		Name:      "Tri Township Airport",
		City:      "Savanna",
		State:     "Illinois",
		Country:   "US",
		Elevation: 616,
		Latitude:  42.0457992554,
		Longitude: -90.1079025269,
		Timezone:  "America/Chicago",
	},
	"KSFZ": {
		ICAO:      "KSFZ",
		IATA:      "SFZ",
		Name:      "North Central State Airport",
		City:      "Pawtucket",
		State:     "Rhode-Island",
		Country:   "US",
		Elevation: 441,
		Latitude:  41.9207992554,
		Longitude: -71.4914016724,
		Timezone:  "America/New_York",
	},
	"KSGF": {
		ICAO:      "KSGF",
		IATA:      "SGF",
		Name:      "Springfield Branson National Airport",
		City:      "Springfield",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1268,
		Latitude:  37.24570084,
		Longitude: -93.38860321,
		Timezone:  "America/Chicago",
	},
	"KSGH": {
		ICAO:      "KSGH",
		IATA:      "SGH",
		Name:      "Springfield Beckley Municipal Airport",
		City:      "Springfield",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1051,
		Latitude:  39.8403015137,
		Longitude: -83.8402023315,
		Timezone:  "America/New_York",
	},
	"KSGJ": {
		ICAO:      "KSGJ",
		IATA:      "UST",
		Name:      "St Augustine Airport",
		City:      "St Augustine",
		State:     "Florida",
		Country:   "US",
		Elevation: 10,
		Latitude:  29.9591999054,
		Longitude: -81.3397979736,
		Timezone:  "America/New_York",
	},
	"KSGR": {
		ICAO:      "KSGR",
		IATA:      "SGR",
		Name:      "Sugar Land Regional Airport",
		City:      "Houston",
		State:     "Texas",
		Country:   "US",
		Elevation: 82,
		Latitude:  29.6222991943,
		Longitude: -95.65650177,
		Timezone:  "America/Chicago",
	},
	"KSGS": {
		ICAO:      "KSGS",
		Name:      "South St Paul Municipal Richard E Fleming field",
		City:      "South St Paul",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 821,
		Latitude:  44.85710144,
		Longitude: -93.03289795,
		Timezone:  "America/Chicago",
	},
	"KSGT": {
		ICAO:      "KSGT",
		IATA:      "SGT",
		Name:      "Stuttgart Municipal Airport",
		City:      "Stuttgart",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 224,
		Latitude:  34.5994987488,
		Longitude: -91.5749969482,
		Timezone:  "America/Chicago",
	},
	"KSGU": {
		ICAO:      "KSGU",
		IATA:      "SGU",
		Name:      "St George Municipal Airport",
		City:      "St George",
		State:     "Utah",
		Country:   "US",
		Elevation: 2941,
		Latitude:  37.0363888889,
		Longitude: -113.510305556,
		Timezone:  "America/Denver",
	},
	"KSHD": {
		ICAO:      "KSHD",
		IATA:      "SHD",
		Name:      "Shenandoah Valley Regional Airport",
		City:      "Staunton/Waynesboro/Harrisonburg",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1201,
		Latitude:  38.2638015747,
		Longitude: -78.8964004517,
		Timezone:  "America/New_York",
	},
	"KSHL": {
		ICAO:      "KSHL",
		Name:      "Sheldon Municipal Airport",
		City:      "Sheldon",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1419,
		Latitude:  43.2084007263,
		Longitude: -95.8333969116,
		Timezone:  "America/Chicago",
	},
	"KSHN": {
		ICAO:      "KSHN",
		IATA:      "SHN",
		Name:      "Sanderson Field",
		City:      "Shelton",
		State:     "Washington",
		Country:   "US",
		Elevation: 273,
		Latitude:  47.2336006165,
		Longitude: -123.1480026245,
		Timezone:  "America/Los_Angeles",
	},
	"KSHR": {
		ICAO:      "KSHR",
		IATA:      "SHR",
		Name:      "Sheridan County Airport",
		City:      "Sheridan",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4021,
		Latitude:  44.7691993713,
		Longitude: -106.9800033569,
		Timezone:  "America/Denver",
	},
	"KSHV": {
		ICAO:      "KSHV",
		IATA:      "SHV",
		Name:      "Shreveport Regional Airport",
		City:      "Shreveport",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 258,
		Latitude:  32.4466018677,
		Longitude: -93.8255996704,
		Timezone:  "America/Chicago",
	},
	"KSIF": {
		ICAO:      "KSIF",
		Name:      "Rockingham County Nc Shiloh Airport",
		City:      "Reidsville",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 694,
		Latitude:  36.43719864,
		Longitude: -79.85099792,
		Timezone:  "America/New_York",
	},
	"KSIK": {
		ICAO:      "KSIK",
		IATA:      "SIK",
		Name:      "Sikeston Memorial Municipal Airport",
		City:      "Sikeston",
		State:     "Missouri",
		Country:   "US",
		Elevation: 315,
		Latitude:  36.8988990784,
		Longitude: -89.5617980957,
		Timezone:  "America/Chicago",
	},
	"KSIV": {
		ICAO:      "KSIV",
		IATA:      "SIV",
		Name:      "Sullivan County Airport",
		City:      "Sullivan",
		State:     "Indiana",
		Country:   "US",
		Elevation: 540,
		Latitude:  39.1147003174,
		Longitude: -87.4483032227,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KSIY": {
		ICAO:      "KSIY",
		IATA:      "SIY",
		Name:      "Siskiyou County Airport",
		City:      "Montague",
		State:     "California",
		Country:   "US",
		Elevation: 2648,
		Latitude:  41.7813987732,
		Longitude: -122.4680023193,
		Timezone:  "America/Los_Angeles",
	},
	"KSJC": {
		ICAO:      "KSJC",
		IATA:      "SJC",
		Name:      "Norman Y. Mineta San Jose International Airport",
		City:      "San Jose",
		State:     "California",
		Country:   "US",
		Elevation: 62,
		Latitude:  37.3625984192,
		Longitude: -121.9290008545,
		Timezone:  "America/Los_Angeles",
	},
	"KSJN": {
		ICAO:      "KSJN",
		IATA:      "SJN",
		Name:      "St Johns Industrial Air Park",
		City:      "St Johns",
		State:     "Arizona",
		Country:   "US",
		Elevation: 5737,
		Latitude:  34.51860046,
		Longitude: -109.3789978,
		Timezone:  "America/Phoenix",
	},
	"KSJT": {
		ICAO:      "KSJT",
		IATA:      "SJT",
		Name:      "San Angelo Regional Mathis Field",
		City:      "San Angelo",
		State:     "Texas",
		Country:   "US",
		Elevation: 1919,
		Latitude:  31.3577003479,
		Longitude: -100.4960021973,
		Timezone:  "America/Chicago",
	},
	"KSJX": {
		ICAO:      "KSJX",
		Name:      "Beaver Island Airport",
		City:      "Beaver Island",
		State:     "Michigan",
		Country:   "US",
		Elevation: 669,
		Latitude:  45.6922988892,
		Longitude: -85.5665969849,
		Timezone:  "America/Detroit",
	},
	"KSKA": {
		ICAO:      "KSKA",
		IATA:      "SKA",
		Name:      "Fairchild Air Force Base",
		City:      "Spokane",
		State:     "Washington",
		Country:   "US",
		Elevation: 2461,
		Latitude:  47.6151008606,
		Longitude: -117.65599823,
		Timezone:  "America/Los_Angeles",
	},
	"KSKF": {
		ICAO:      "KSKF",
		IATA:      "SKF",
		Name:      "Lackland Air Force Base (Kelly Field Annex) Airport",
		City:      "San Antonio",
		State:     "Texas",
		Country:   "US",
		Elevation: 691,
		Latitude:  29.38419914,
		Longitude: -98.58110046,
		Timezone:  "America/Chicago",
	},
	"KSKI": {
		ICAO:      "KSKI",
		Name:      "Sac City Municipal Airport",
		City:      "Sac City",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1250,
		Latitude:  42.3791007996,
		Longitude: -94.9796981812,
		Timezone:  "America/Chicago",
	},
	"KSKX": {
		ICAO:      "KSKX",
		IATA:      "TSM",
		Name:      "Taos Regional Airport",
		City:      "Taos",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 7095,
		Latitude:  36.45819855,
		Longitude: -105.6719971,
		Timezone:  "America/Denver",
	},
	"KSKY": {
		ICAO:      "KSKY",
		Name:      "Griffing Sandusky Airport",
		City:      "Sandusky",
		State:     "Ohio",
		Country:   "US",
		Elevation: 580,
		Latitude:  41.4333992004,
		Longitude: -82.6522979736,
		Timezone:  "America/New_York",
	},
	"KSLB": {
		ICAO:      "KSLB",
		IATA:      "SLB",
		Name:      "Storm Lake Municipal Airport",
		City:      "Storm Lake",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1488,
		Latitude:  42.5973014832,
		Longitude: -95.2406997681,
		Timezone:  "America/Chicago",
	},
	"KSLC": {
		ICAO:      "KSLC",
		IATA:      "SLC",
		Name:      "Salt Lake City International Airport",
		City:      "Salt Lake City",
		State:     "Utah",
		Country:   "US",
		Elevation: 4227,
		Latitude:  40.7883987427,
		Longitude: -111.9779968262,
		Timezone:  "America/Denver",
	},
	"KSLE": {
		ICAO:      "KSLE",
		IATA:      "SLE",
		Name:      "McNary Field",
		City:      "Salem",
		State:     "Oregon",
		Country:   "US",
		Elevation: 214,
		Latitude:  44.90950012,
		Longitude: -123.0029984,
		Timezone:  "America/Los_Angeles",
	},
	"KSLG": {
		ICAO:      "KSLG",
		IATA:      "SLG",
		Name:      "Smith Field",
		City:      "Siloam Springs",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1191,
		Latitude:  36.19189835,
		Longitude: -94.48999786,
		Timezone:  "America/Chicago",
	},
	"KSLH": {
		ICAO:      "KSLH",
		Name:      "Cheboygan County Airport",
		City:      "Cheboygan",
		State:     "Michigan",
		Country:   "US",
		Elevation: 639,
		Latitude:  45.65370178,
		Longitude: -84.51930237,
		Timezone:  "America/Detroit",
	},
	"KSLI": {
		ICAO:      "KSLI",
		Name:      "Los Alamitos Army Air Field",
		City:      "Los Alamitos",
		State:     "California",
		Country:   "US",
		Elevation: 32,
		Latitude:  33.79000092,
		Longitude: -118.052002,
		Timezone:  "America/Los_Angeles",
	},
	"KSLJ": {
		ICAO:      "KSLJ",
		Name:      "Hagler Army Air Field",
		City:      "Camp Shelby",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 280,
		Latitude:  31.17379951,
		Longitude: -89.19120026,
		Timezone:  "America/Chicago",
	},
	"KSLK": {
		ICAO:      "KSLK",
		IATA:      "SLK",
		Name:      "Adirondack Regional Airport",
		City:      "Saranac Lake",
		State:     "New-York",
		Country:   "US",
		Elevation: 1663,
		Latitude:  44.3852996826,
		Longitude: -74.206199646,
		Timezone:  "America/New_York",
	},
	"KSLN": {
		ICAO:      "KSLN",
		IATA:      "SLN",
		Name:      "Salina Municipal Airport",
		City:      "Salina",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1288,
		Latitude:  38.7910003662,
		Longitude: -97.6521987915,
		Timezone:  "America/Chicago",
	},
	"KSLO": {
		ICAO:      "KSLO",
		IATA:      "SLO",
		Name:      "Salem Leckrone Airport",
		City:      "Salem",
		State:     "Illinois",
		Country:   "US",
		Elevation: 573,
		Latitude:  38.6428985596,
		Longitude: -88.9642028809,
		Timezone:  "America/Chicago",
	},
	"KSLR": {
		ICAO:      "KSLR",
		IATA:      "SLR",
		Name:      "Sulphur Springs Municipal Airport",
		City:      "Sulphur Springs",
		State:     "Texas",
		Country:   "US",
		Elevation: 489,
		Latitude:  33.1598014832,
		Longitude: -95.6211013794,
		Timezone:  "America/Chicago",
	},
	"KSMD": {
		ICAO:      "KSMD",
		IATA:      "SMD",
		Name:      "Smith Field",
		City:      "Fort Wayne",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 835,
		Latitude:  41.14339828,
		Longitude: -85.15280151,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KSME": {
		ICAO:      "KSME",
		IATA:      "SME",
		Name:      "Lake Cumberland Regional Airport",
		City:      "Somerset",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 927,
		Latitude:  37.0533981323,
		Longitude: -84.6158981323,
		Timezone:  "America/New_York",
	},
	"KSMF": {
		ICAO:      "KSMF",
		IATA:      "SMF",
		Name:      "Sacramento International Airport",
		City:      "Sacramento",
		State:     "California",
		Country:   "US",
		Elevation: 27,
		Latitude:  38.695400238,
		Longitude: -121.591003418,
		Timezone:  "America/Los_Angeles",
	},
	"KSMN": {
		ICAO:      "KSMN",
		IATA:      "SMN",
		Name:      "Lemhi County Airport",
		City:      "Salmon",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4043,
		Latitude:  45.1237983704,
		Longitude: -113.8809967041,
		Timezone:  "America/Boise",
	},
	"KSMO": {
		ICAO:      "KSMO",
		IATA:      "SMO",
		Name:      "Santa Monica Municipal Airport",
		City:      "Santa Monica",
		State:     "California",
		Country:   "US",
		Elevation: 177,
		Latitude:  34.0158004761,
		Longitude: -118.4509963989,
		Timezone:  "America/Los_Angeles",
	},
	"KSMQ": {
		ICAO:      "KSMQ",
		Name:      "Somerset Airport",
		City:      "Somerville",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 105,
		Latitude:  40.6259994507,
		Longitude: -74.6701965332,
		Timezone:  "America/New_York",
	},
	"KSMS": {
		ICAO:      "KSMS",
		IATA:      "SUM",
		Name:      "Sumter Airport",
		City:      "Sumter",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 182,
		Latitude:  33.9949989319,
		Longitude: -80.3612976074,
		Timezone:  "America/New_York",
	},
	"KSMX": {
		ICAO:      "KSMX",
		IATA:      "SMX",
		Name:      "Santa Maria Pub/Capt G Allan Hancock Field",
		City:      "Santa Maria",
		State:     "California",
		Country:   "US",
		Elevation: 261,
		Latitude:  34.89889908,
		Longitude: -120.4570007,
		Timezone:  "America/Los_Angeles",
	},
	"KSNA": {
		ICAO:      "KSNA",
		IATA:      "SNA",
		Name:      "John Wayne Airport-Orange County Airport",
		City:      "Santa Ana",
		State:     "California",
		Country:   "US",
		Elevation: 56,
		Latitude:  33.67570114,
		Longitude: -117.8679962,
		Timezone:  "America/Los_Angeles",
	},
	"KSNC": {
		ICAO:      "KSNC",
		Name:      "Chester Airport",
		City:      "Chester",
		State:     "Connecticut",
		Country:   "US",
		Elevation: 416,
		Latitude:  41.3838996887,
		Longitude: -72.505897522,
		Timezone:  "America/New_York",
	},
	"KSNH": {
		ICAO:      "KSNH",
		Name:      "Savannah Hardin County Airport",
		City:      "Savannah",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 473,
		Latitude:  35.1703987122,
		Longitude: -88.2158966064,
		Timezone:  "America/Chicago",
	},
	"KSNK": {
		ICAO:      "KSNK",
		IATA:      "SNK",
		Name:      "Winston Field",
		City:      "Snyder",
		State:     "Texas",
		Country:   "US",
		Elevation: 2430,
		Latitude:  32.6934013367,
		Longitude: -100.9499969482,
		Timezone:  "America/Chicago",
	},
	"KSNL": {
		ICAO:      "KSNL",
		IATA:      "SNL",
		Name:      "Shawnee Regional Airport",
		City:      "Shawnee",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1073,
		Latitude:  35.3578987122,
		Longitude: -96.9428024292,
		Timezone:  "America/Chicago",
	},
	"KSNS": {
		ICAO:      "KSNS",
		IATA:      "SNS",
		Name:      "Salinas Municipal Airport",
		City:      "Salinas",
		State:     "California",
		Country:   "US",
		Elevation: 85,
		Latitude:  36.6627998352,
		Longitude: -121.6060028076,
		Timezone:  "America/Los_Angeles",
	},
	"KSNY": {
		ICAO:      "KSNY",
		IATA:      "SNY",
		Name:      "Sidney Municipal-Lloyd W Carr Field",
		City:      "Sidney",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 4313,
		Latitude:  41.10129929,
		Longitude: -102.9850006,
		Timezone:  "America/Denver",
	},
	"KSOA": {
		ICAO:      "KSOA",
		Name:      "Sonora Municipal Airport",
		City:      "Sonora",
		State:     "Texas",
		Country:   "US",
		Elevation: 2140,
		Latitude:  30.5856990814,
		Longitude: -100.6490020752,
		Timezone:  "America/Chicago",
	},
	"KSOP": {
		ICAO:      "KSOP",
		IATA:      "SOP",
		Name:      "Moore County Airport",
		City:      "Pinehurst/Southern Pines",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 455,
		Latitude:  35.23740005,
		Longitude: -79.3911972,
		Timezone:  "America/New_York",
	},
	"KSOW": {
		ICAO:      "KSOW",
		IATA:      "SOW",
		Name:      "Show Low Regional Airport",
		City:      "Show Low",
		State:     "Arizona",
		Country:   "US",
		Elevation: 6415,
		Latitude:  34.265499115,
		Longitude: -110.005996704,
		Timezone:  "America/Phoenix",
	},
	"KSOY": {
		ICAO:      "KSOY",
		Name:      "Sioux Center Municipal Airport",
		City:      "Sioux Center",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1448,
		Latitude:  43.1343994141,
		Longitude: -96.1875,
		Timezone:  "America/Chicago",
	},
	"KSPA": {
		ICAO:      "KSPA",
		IATA:      "SPA",
		Name:      "Spartanburg Downtown Memorial Airport",
		City:      "Spartanburg",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 801,
		Latitude:  34.9156990051,
		Longitude: -81.9564971924,
		Timezone:  "America/New_York",
	},
	"KSPB": {
		ICAO:      "KSPB",
		Name:      "Scappoose Industrial Airpark",
		City:      "Scappoose",
		State:     "Oregon",
		Country:   "US",
		Elevation: 58,
		Latitude:  45.7709999084,
		Longitude: -122.8619995117,
		Timezone:  "America/Los_Angeles",
	},
	"KSPF": {
		ICAO:      "KSPF",
		IATA:      "SPF",
		Name:      "Black Hills Clyde Ice Field",
		City:      "Spearfish",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 3931,
		Latitude:  44.4803009033,
		Longitude: -103.7829971313,
		Timezone:  "America/Denver",
	},
	"KSPG": {
		ICAO:      "KSPG",
		IATA:      "SPG",
		Name:      "Albert Whitted Airport",
		City:      "St Petersburg",
		State:     "Florida",
		Country:   "US",
		Elevation: 7,
		Latitude:  27.7651004791,
		Longitude: -82.6269989014,
		Timezone:  "America/New_York",
	},
	"KSPH": {
		ICAO:      "KSPH",
		Name:      "Springhill Airport",
		City:      "Springhill",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 218,
		Latitude:  32.9833984375,
		Longitude: -93.4092025757,
		Timezone:  "America/Chicago",
	},
	"KSPI": {
		ICAO:      "KSPI",
		IATA:      "SPI",
		Name:      "Abraham Lincoln Capital Airport",
		City:      "Springfield",
		State:     "Illinois",
		Country:   "US",
		Elevation: 598,
		Latitude:  39.84410095,
		Longitude: -89.67790222,
		Timezone:  "America/Chicago",
	},
	"KSPS": {
		ICAO:      "KSPS",
		IATA:      "SPS",
		Name:      "Sheppard Air Force Base-Wichita Falls Municipal Airport",
		City:      "Wichita Falls",
		State:     "Texas",
		Country:   "US",
		Elevation: 1019,
		Latitude:  33.98880005,
		Longitude: -98.49189758,
		Timezone:  "America/Chicago",
	},
	"KSPW": {
		ICAO:      "KSPW",
		IATA:      "SPW",
		Name:      "Spencer Municipal Airport",
		City:      "Spencer",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1339,
		Latitude:  43.1655006409,
		Longitude: -95.202796936,
		Timezone:  "America/Chicago",
	},
	"KSPZ": {
		ICAO:      "KSPZ",
		Name:      "Silver Springs Airport",
		City:      "Silver Springs",
		State:     "Nevada",
		Country:   "US",
		Elevation: 4269,
		Latitude:  39.4029998779,
		Longitude: -119.2509994507,
		Timezone:  "America/Los_Angeles",
	},
	"KSQI": {
		ICAO:      "KSQI",
		IATA:      "SQI",
		Name:      "Whiteside Co Arpt-Jos H Bittorf field",
		City:      "Sterling/Rockfalls",
		State:     "Illinois",
		Country:   "US",
		Elevation: 648,
		Latitude:  41.74280167,
		Longitude: -89.67630005,
		Timezone:  "America/Chicago",
	},
	"KSQL": {
		ICAO:      "KSQL",
		IATA:      "SQL",
		Name:      "San Carlos Airport",
		City:      "San Carlos",
		State:     "Arizona",
		Country:   "US",
		Elevation: 5,
		Latitude:  37.5119018555,
		Longitude: -122.25,
		Timezone:  "America/Los_Angeles",
	},
	"KSRB": {
		ICAO:      "KSRB",
		Name:      "Upper Cumberland Regional Airport",
		City:      "Sparta",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1025,
		Latitude:  36.0559005737,
		Longitude: -85.5307006836,
		Timezone:  "America/Chicago",
	},
	"KSRC": {
		ICAO:      "KSRC",
		IATA:      "SRC",
		Name:      "Searcy Municipal Airport",
		City:      "Searcy",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 265,
		Latitude:  35.21060181,
		Longitude: -91.73750305,
		Timezone:  "America/Chicago",
	},
	"KSRE": {
		ICAO:      "KSRE",
		Name:      "Seminole Municipal Airport",
		City:      "Seminole",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1024,
		Latitude:  35.2747001648,
		Longitude: -96.675201416,
		Timezone:  "America/Chicago",
	},
	"KSRQ": {
		ICAO:      "KSRQ",
		IATA:      "SRQ",
		Name:      "Sarasota Bradenton International Airport",
		City:      "Sarasota/Bradenton",
		State:     "Florida",
		Country:   "US",
		Elevation: 30,
		Latitude:  27.3953990936,
		Longitude: -82.554397583,
		Timezone:  "America/New_York",
	},
	"KSRR": {
		ICAO:      "KSRR",
		IATA:      "RUI",
		Name:      "Sierra Blanca Regional Airport",
		City:      "Ruidoso",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 6814,
		Latitude:  33.4627990723,
		Longitude: -105.5350036621,
		Timezone:  "America/Denver",
	},
	"KSSC": {
		ICAO:      "KSSC",
		IATA:      "SSC",
		Name:      "Shaw Air Force Base",
		City:      "Sumter",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 241,
		Latitude:  33.97269821,
		Longitude: -80.47059631,
		Timezone:  "America/New_York",
	},
	"KSSF": {
		ICAO:      "KSSF",
		IATA:      "SSF",
		Name:      "Stinson Municipal Airport",
		City:      "San Antonio",
		State:     "Texas",
		Country:   "US",
		Elevation: 577,
		Latitude:  29.3369998932,
		Longitude: -98.4710998535,
		Timezone:  "America/Chicago",
	},
	"KSSI": {
		ICAO:      "KSSI",
		IATA:      "SSI",
		Name:      "Malcolm Mc Kinnon Airport",
		City:      "Brunswick",
		State:     "Georgia",
		Country:   "US",
		Elevation: 19,
		Latitude:  31.15180016,
		Longitude: -81.39129639,
		Timezone:  "America/New_York",
	},
	"KSSQ": {
		ICAO:      "KSSQ",
		Name:      "Shell Lake Municipal Airport",
		City:      "Shell Lake",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1233,
		Latitude:  45.73139954,
		Longitude: -91.92070007,
		Timezone:  "America/Chicago",
	},
	"KSTC": {
		ICAO:      "KSTC",
		IATA:      "STC",
		Name:      "St Cloud Regional Airport",
		City:      "St Cloud",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1031,
		Latitude:  45.5466003418,
		Longitude: -94.0598983765,
		Timezone:  "America/Chicago",
	},
	"KSTE": {
		ICAO:      "KSTE",
		IATA:      "STE",
		Name:      "Stevens Point Municipal Airport",
		City:      "Stevens Point",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1110,
		Latitude:  44.5452003479,
		Longitude: -89.5302963257,
		Timezone:  "America/Chicago",
	},
	"KSTF": {
		ICAO:      "KSTF",
		Name:      "George M Bryan Airport",
		City:      "Starkville",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 333,
		Latitude:  33.43310165,
		Longitude: -88.84860229,
		Timezone:  "America/Chicago",
	},
	"KSTJ": {
		ICAO:      "KSTJ",
		IATA:      "STJ",
		Name:      "Rosecrans Memorial Airport",
		City:      "St Joseph",
		State:     "Missouri",
		Country:   "US",
		Elevation: 826,
		Latitude:  39.771900177,
		Longitude: -94.9096984863,
		Timezone:  "America/Chicago",
	},
	"KSTK": {
		ICAO:      "KSTK",
		IATA:      "STK",
		Name:      "Sterling Municipal Airport",
		City:      "Sterling",
		State:     "Colorado",
		Country:   "US",
		Elevation: 4040,
		Latitude:  40.61529922,
		Longitude: -103.2649994,
		Timezone:  "America/Denver",
	},
	"KSTL": {
		ICAO:      "KSTL",
		IATA:      "STL",
		Name:      "Lambert St Louis International Airport",
		City:      "St Louis",
		State:     "Missouri",
		Country:   "US",
		Elevation: 618,
		Latitude:  38.7486991882,
		Longitude: -90.3700027466,
		Timezone:  "America/Chicago",
	},
	"KSTP": {
		ICAO:      "KSTP",
		IATA:      "STP",
		Name:      "St Paul Downtown Holman Field",
		City:      "St Paul",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 705,
		Latitude:  44.9345016479,
		Longitude: -93.0599975586,
		Timezone:  "America/Chicago",
	},
	"KSTS": {
		ICAO:      "KSTS",
		IATA:      "STS",
		Name:      "Charles M. Schulz Sonoma County Airport",
		City:      "Santa Rosa",
		State:     "California",
		Country:   "US",
		Elevation: 128,
		Latitude:  38.50899887,
		Longitude: -122.8130035,
		Timezone:  "America/Los_Angeles",
	},
	"KSUA": {
		ICAO:      "KSUA",
		IATA:      "SUA",
		Name:      "Witham Field",
		City:      "Stuart",
		State:     "Florida",
		Country:   "US",
		Elevation: 16,
		Latitude:  27.18169975,
		Longitude: -80.22109985,
		Timezone:  "America/New_York",
	},
	"KSUD": {
		ICAO:      "KSUD",
		IATA:      "SUD",
		Name:      "Stroud Municipal Airport",
		City:      "Stroud",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 900,
		Latitude:  35.7896003723,
		Longitude: -96.6557006836,
		Timezone:  "America/Chicago",
	},
	"KSUE": {
		ICAO:      "KSUE",
		IATA:      "SUE",
		Name:      "Door County Cherryland Airport",
		City:      "Sturgeon Bay",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 725,
		Latitude:  44.84370041,
		Longitude: -87.42150116,
		Timezone:  "America/Chicago",
	},
	"KSUN": {
		ICAO:      "KSUN",
		IATA:      "SUN",
		Name:      "Friedman Memorial Airport",
		City:      "Hailey",
		State:     "Idaho",
		Country:   "US",
		Elevation: 5318,
		Latitude:  43.50439835,
		Longitude: -114.2959976,
		Timezone:  "America/Boise",
	},
	"KSUS": {
		ICAO:      "KSUS",
		IATA:      "SUS",
		Name:      "Spirit of St Louis Airport",
		City:      "St Louis",
		State:     "Missouri",
		Country:   "US",
		Elevation: 463,
		Latitude:  38.6621017456,
		Longitude: -90.6520004272,
		Timezone:  "America/Chicago",
	},
	"KSUT": {
		ICAO:      "KSUT",
		Name:      "Brunswick County Airport",
		City:      "Oak Island",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 24,
		Latitude:  33.9292984,
		Longitude: -78.07499695,
		Timezone:  "America/New_York",
	},
	"KSUU": {
		ICAO:      "KSUU",
		IATA:      "SUU",
		Name:      "Travis Air Force Base",
		City:      "Fairfield",
		State:     "California",
		Country:   "US",
		Elevation: 62,
		Latitude:  38.2626991272,
		Longitude: -121.9270019531,
		Timezone:  "America/Los_Angeles",
	},
	"KSUW": {
		ICAO:      "KSUW",
		IATA:      "SUW",
		Name:      "Richard I Bong Airport",
		City:      "Superior",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 674,
		Latitude:  46.6897010803,
		Longitude: -92.0947036743,
		Timezone:  "America/Chicago",
	},
	"KSUX": {
		ICAO:      "KSUX",
		IATA:      "SUX",
		Name:      "Sioux Gateway Col. Bud Day Field",
		City:      "Sioux City",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1098,
		Latitude:  42.40259933,
		Longitude: -96.38439941,
		Timezone:  "America/Chicago",
	},
	"KSUZ": {
		ICAO:      "KSUZ",
		Name:      "Saline County Regional Airport",
		City:      "Benton",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 390,
		Latitude:  34.59059906,
		Longitude: -92.47940063,
		Timezone:  "America/Chicago",
	},
	"KSVC": {
		ICAO:      "KSVC",
		IATA:      "SVC",
		Name:      "Grant County Airport",
		City:      "Silver City",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 5446,
		Latitude:  32.6365013123,
		Longitude: -108.15599823,
		Timezone:  "America/Denver",
	},
	"KSVE": {
		ICAO:      "KSVE",
		IATA:      "SVE",
		Name:      "Susanville Municipal Airport",
		City:      "Susanville",
		State:     "California",
		Country:   "US",
		Elevation: 4149,
		Latitude:  40.3757019043,
		Longitude: -120.5729980469,
		Timezone:  "America/Los_Angeles",
	},
	"KSVH": {
		ICAO:      "KSVH",
		IATA:      "SVH",
		Name:      "Statesville Regional Airport",
		City:      "Statesville",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 968,
		Latitude:  35.7653007507,
		Longitude: -80.9539031982,
		Timezone:  "America/New_York",
	},
	"KSVN": {
		ICAO:      "KSVN",
		IATA:      "SVN",
		Name:      "Hunter Army Air Field",
		City:      "Savannah",
		State:     "Georgia",
		Country:   "US",
		Elevation: 41,
		Latitude:  32.00999832,
		Longitude: -81.14569855,
		Timezone:  "America/New_York",
	},
	"KSWF": {
		ICAO:      "KSWF",
		IATA:      "SWF",
		Name:      "Stewart International Airport",
		City:      "Newburgh",
		State:     "New-York",
		Country:   "US",
		Elevation: 491,
		Latitude:  41.5041007996,
		Longitude: -74.1047973633,
		Timezone:  "America/New_York",
	},
	"KSWI": {
		ICAO:      "KSWI",
		Name:      "Sherman Municipal Airport",
		City:      "Sherman",
		State:     "Texas",
		Country:   "US",
		Elevation: 745,
		Latitude:  33.6241989136,
		Longitude: -96.5860977173,
		Timezone:  "America/Chicago",
	},
	"KSWO": {
		ICAO:      "KSWO",
		IATA:      "SWO",
		Name:      "Stillwater Regional Airport",
		City:      "Stillwater",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1000,
		Latitude:  36.1612014771,
		Longitude: -97.0857009888,
		Timezone:  "America/Chicago",
	},
	"KSWT": {
		ICAO:      "KSWT",
		Name:      "Seward Municipal Airport",
		City:      "Seward",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1506,
		Latitude:  40.86470032,
		Longitude: -97.10919952,
		Timezone:  "America/Chicago",
	},
	"KSWW": {
		ICAO:      "KSWW",
		IATA:      "SWW",
		Name:      "Avenger Field",
		City:      "Sweetwater",
		State:     "Texas",
		Country:   "US",
		Elevation: 2380,
		Latitude:  32.4673995972,
		Longitude: -100.4670028687,
		Timezone:  "America/Chicago",
	},
	"KSXL": {
		ICAO:      "KSXL",
		Name:      "Summersville Airport",
		City:      "Summersville",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 1820,
		Latitude:  38.2316017151,
		Longitude: -80.8707962036,
		Timezone:  "America/New_York",
	},
	"KSXU": {
		ICAO:      "KSXU",
		Name:      "Santa Rosa Route 66 Airport",
		City:      "Santa Rosa",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4792,
		Latitude:  34.93439865,
		Longitude: -104.6429977,
		Timezone:  "America/Denver",
	},
	"KSYF": {
		ICAO:      "KSYF",
		Name:      "Cheyenne County Municipal Airport",
		City:      "St Francis",
		State:     "Kansas",
		Country:   "US",
		Elevation: 3413,
		Latitude:  39.76110077,
		Longitude: -101.7959976,
		Timezone:  "America/Chicago",
	},
	"KSYI": {
		ICAO:      "KSYI",
		IATA:      "SYI",
		Name:      "Bomar Field Shelbyville Municipal Airport",
		City:      "Shelbyville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 801,
		Latitude:  35.56010056,
		Longitude: -86.44249725,
		Timezone:  "America/Chicago",
	},
	"KSYN": {
		ICAO:      "KSYN",
		IATA:      "SYN",
		Name:      "Stanton Airfield",
		City:      "Stanton",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 920,
		Latitude:  44.4754981995,
		Longitude: -93.0162963867,
		Timezone:  "America/Chicago",
	},
	"KSYR": {
		ICAO:      "KSYR",
		IATA:      "SYR",
		Name:      "Syracuse Hancock International Airport",
		City:      "Syracuse",
		State:     "New-York",
		Country:   "US",
		Elevation: 421,
		Latitude:  43.1111984253,
		Longitude: -76.106300354,
		Timezone:  "America/New_York",
	},
	"KSYV": {
		ICAO:      "KSYV",
		IATA:      "SYV",
		Name:      "Sylvester Airport",
		City:      "Sylvester",
		State:     "Georgia",
		Country:   "US",
		Elevation: 403,
		Latitude:  31.5585002899,
		Longitude: -83.8956985474,
		Timezone:  "America/New_York",
	},
	"KSZL": {
		ICAO:      "KSZL",
		IATA:      "SZL",
		Name:      "Whiteman Air Force Base",
		City:      "Knob Noster",
		State:     "Missouri",
		Country:   "US",
		Elevation: 870,
		Latitude:  38.7303009033,
		Longitude: -93.5478973389,
		Timezone:  "America/Chicago",
	},
	"KSZN": {
		ICAO:      "KSZN",
		IATA:      "SZN",
		Name:      "Santa Cruz Island Airport",
		City:      "Santa Barbara",
		State:     "California",
		Country:   "US",
		Elevation: 50,
		Latitude:  34.0606002808,
		Longitude: -119.915000916,
		Timezone:  "America/Los_Angeles",
	},
	"KSZP": {
		ICAO:      "KSZP",
		IATA:      "SZP",
		Name:      "Santa Paula Airport",
		City:      "Santa Paula",
		State:     "California",
		Country:   "US",
		Elevation: 243,
		Latitude:  34.34719849,
		Longitude: -119.060997,
		Timezone:  "America/Los_Angeles",
	},
	"KSZT": {
		ICAO:      "KSZT",
		Name:      "Sandpoint Airport",
		City:      "Sandpoint",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2131,
		Latitude:  48.2994995117,
		Longitude: -116.5599975586,
		Timezone:  "America/Los_Angeles",
	},
	"KSZY": {
		ICAO:      "KSZY",
		Name:      "Robert Sibley Airport",
		City:      "Selmer",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 610,
		Latitude:  35.2028999329,
		Longitude: -88.4983978271,
		Timezone:  "America/Chicago",
	},
	"KT00": {
		ICAO:      "KT00",
		Name:      "Chambers County Airport",
		City:      "Anahuac",
		State:     "Texas",
		Country:   "US",
		Elevation: 21,
		Latitude:  29.7700996399,
		Longitude: -94.662399292,
		Timezone:  "America/Chicago",
	},
	"KT03": {
		ICAO:      "KT03",
		Name:      "Tuba City Airport",
		City:      "Tuba City",
		State:     "Arizona",
		Country:   "US",
		Elevation: 4513,
		Latitude:  36.0928001404,
		Longitude: -111.3830032349,
		Timezone:  "America/Denver",
	},
	"KT05": {
		ICAO:      "KT05",
		Name:      "Charles R Johnson Airport",
		City:      "Port Mansfield",
		State:     "Texas",
		Country:   "US",
		Elevation: 10,
		Latitude:  26.5627994537,
		Longitude: -97.4377975464,
		Timezone:  "America/Chicago",
	},
	"KT12": {
		ICAO:      "KT12",
		Name:      "Kirbyville Airport",
		City:      "Kirbyville",
		State:     "Texas",
		Country:   "US",
		Elevation: 121,
		Latitude:  30.6466007233,
		Longitude: -93.9149017334,
		Timezone:  "America/Chicago",
	},
	"KT15": {
		ICAO:      "KT15",
		Name:      "Marlin Airport",
		City:      "Marlin",
		State:     "Texas",
		Country:   "US",
		Elevation: 411,
		Latitude:  31.3407001495,
		Longitude: -96.8519973755,
		Timezone:  "America/Chicago",
	},
	"KT16": {
		ICAO:      "KT16",
		Name:      "Reserve Airport",
		City:      "Reserve",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 6360,
		Latitude:  33.6941986084,
		Longitude: -108.8489990234,
		Timezone:  "America/Denver",
	},
	"KT17": {
		ICAO:      "KT17",
		Name:      "New Gulf Airport",
		City:      "New Gulf",
		State:     "Texas",
		Country:   "US",
		Elevation: 100,
		Latitude:  29.2761001587,
		Longitude: -95.8886032104,
		Timezone:  "America/Chicago",
	},
	"KT19": {
		ICAO:      "KT19",
		Name:      "Duval Freer Airport",
		City:      "Freer",
		State:     "Texas",
		Country:   "US",
		Elevation: 564,
		Latitude:  27.883600235,
		Longitude: -98.6003036499,
		Timezone:  "America/Chicago",
	},
	"KT20": {
		ICAO:      "KT20",
		Name:      "Roger M. Dreyer Memorial Airport",
		City:      "Gonzales",
		State:     "Texas",
		Country:   "US",
		Elevation: 354,
		Latitude:  29.5279998779,
		Longitude: -97.4614028931,
		Timezone:  "America/Chicago",
	},
	"KT23": {
		ICAO:      "KT23",
		Name:      "Albany Municipal Airport",
		City:      "Albany",
		State:     "Texas",
		Country:   "US",
		Elevation: 1425,
		Latitude:  32.71900177,
		Longitude: -99.2676010132,
		Timezone:  "America/Chicago",
	},
	"KT24": {
		ICAO:      "KT24",
		Name:      "Pineland Municipal Airport",
		City:      "Pineland",
		State:     "Texas",
		Country:   "US",
		Elevation: 260,
		Latitude:  31.233499527,
		Longitude: -93.9819030762,
		Timezone:  "America/Chicago",
	},
	"KT27": {
		ICAO:      "KT27",
		Name:      "Horizon Airport",
		City:      "El Paso",
		State:     "Texas",
		Country:   "US",
		Elevation: 4007,
		Latitude:  31.7198009491,
		Longitude: -106.2369995117,
		Timezone:  "America/Denver",
	},
	"KT30": {
		ICAO:      "KT30",
		Name:      "Mc Kinley Field",
		City:      "Pearsall",
		State:     "Texas",
		Country:   "US",
		Elevation: 586,
		Latitude:  28.82229996,
		Longitude: -99.10900116,
		Timezone:  "America/Chicago",
	},
	"KT31": {
		ICAO:      "KT31",
		Name:      "Aero Country Airport",
		City:      "Mc Kinney",
		State:     "Texas",
		Country:   "US",
		Elevation: 792,
		Latitude:  33.2084999084,
		Longitude: -96.741897583,
		Timezone:  "America/Chicago",
	},
	"KT35": {
		ICAO:      "KT35",
		Name:      "Cameron Municipal Airpark",
		City:      "Cameron",
		State:     "Texas",
		Country:   "US",
		Elevation: 402,
		Latitude:  30.8794002533,
		Longitude: -96.9710998535,
		Timezone:  "America/Chicago",
	},
	"KT36": {
		ICAO:      "KT36",
		Name:      "Paul Pittman Memorial Airport",
		City:      "Tylertown",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 384,
		Latitude:  31.1459999084,
		Longitude: -90.1680984497,
		Timezone:  "America/Chicago",
	},
	"KT39": {
		ICAO:      "KT39",
		Name:      "Archer City Municipal Airport",
		City:      "Archer City",
		State:     "Texas",
		Country:   "US",
		Elevation: 1065,
		Latitude:  33.5822982788,
		Longitude: -98.6186981201,
		Timezone:  "America/Chicago",
	},
	"KT41": {
		ICAO:      "KT41",
		Name:      "La Porte Municipal Airport",
		City:      "La Porte",
		State:     "Texas",
		Country:   "US",
		Elevation: 25,
		Latitude:  29.6693000793,
		Longitude: -95.064201355,
		Timezone:  "America/Chicago",
	},
	"KT42": {
		ICAO:      "KT42",
		Name:      "Ruth Airport",
		City:      "Ruth",
		State:     "California",
		Country:   "US",
		Elevation: 2781,
		Latitude:  40.2112998962,
		Longitude: -123.297996521,
		Timezone:  "America/Los_Angeles",
	},
	"KT45": {
		ICAO:      "KT45",
		Name:      "Panhandle Carson County Airport",
		City:      "Panhandle",
		State:     "Texas",
		Country:   "US",
		Elevation: 3454,
		Latitude:  35.3616981506,
		Longitude: -101.3649978638,
		Timezone:  "America/Chicago",
	},
	"KT50": {
		ICAO:      "KT50",
		Name:      "Menard County Airport",
		City:      "Menard",
		State:     "Texas",
		Country:   "US",
		Elevation: 1930,
		Latitude:  30.9335002899,
		Longitude: -99.8170013428,
		Timezone:  "America/Chicago",
	},
	"KT51": {
		ICAO:      "KT51",
		Name:      "Dan Jones International Airport",
		City:      "Houston",
		State:     "Texas",
		Country:   "US",
		Elevation: 166,
		Latitude:  30.0428009033,
		Longitude: -95.6671981812,
		Timezone:  "America/Chicago",
	},
	"KT54": {
		ICAO:      "KT54",
		Name:      "Lane Airpark",
		City:      "Rosenberg",
		State:     "Texas",
		Country:   "US",
		Elevation: 94,
		Latitude:  29.5244007111,
		Longitude: -95.7751998901,
		Timezone:  "America/Chicago",
	},
	"KT55": {
		ICAO:      "KT55",
		Name:      "Dimmitt Municipal Airport",
		City:      "Dimmitt",
		State:     "Texas",
		Country:   "US",
		Elevation: 3883,
		Latitude:  34.5666999817,
		Longitude: -102.3229980469,
		Timezone:  "America/Chicago",
	},
	"KT59": {
		ICAO:      "KT59",
		Name:      "Wheeler Municipal Airport",
		City:      "Wheeler",
		State:     "Texas",
		Country:   "US",
		Elevation: 2470,
		Latitude:  35.4510993958,
		Longitude: -100.1999969482,
		Timezone:  "America/Chicago",
	},
	"KT60": {
		ICAO:      "KT60",
		Name:      "Stonewall County Airport",
		City:      "Aspermont",
		State:     "Texas",
		Country:   "US",
		Elevation: 1744,
		Latitude:  33.1722984314,
		Longitude: -100.1979980469,
		Timezone:  "America/Chicago",
	},
	"KT65": {
		ICAO:      "KT65",
		Name:      "Mid Valley Airport",
		City:      "Weslaco",
		State:     "Texas",
		Country:   "US",
		Elevation: 70,
		Latitude:  26.1776008606,
		Longitude: -97.9730987549,
		Timezone:  "America/Chicago",
	},
	"KT67": {
		ICAO:      "KT67",
		Name:      "Hicks Air Field",
		City:      "Fort Worth",
		State:     "Texas",
		Country:   "US",
		Elevation: 855,
		Latitude:  32.93119812,
		Longitude: -97.41169739,
		Timezone:  "America/Chicago",
	},
	"KT69": {
		ICAO:      "KT69",
		Name:      "Alfred C 'Bubba' Thomas Airport",
		City:      "Sinton",
		State:     "Texas",
		Country:   "US",
		Elevation: 48,
		Latitude:  28.0391998291,
		Longitude: -97.5423965454,
		Timezone:  "America/Chicago",
	},
	"KT70": {
		ICAO:      "KT70",
		Name:      "Laughlin Air Force Base Auxiliary Nr 1 Airport",
		City:      "Spofford",
		State:     "Texas",
		Country:   "US",
		Elevation: 976,
		Latitude:  29.1259994507,
		Longitude: -100.481002808,
		Timezone:  "America/Chicago",
	},
	"KT74": {
		ICAO:      "KT74",
		Name:      "Taylor Municipal Airport",
		City:      "Taylor",
		State:     "Texas",
		Country:   "US",
		Elevation: 600,
		Latitude:  30.572599411,
		Longitude: -97.4431991577,
		Timezone:  "America/Chicago",
	},
	"KT77": {
		ICAO:      "KT77",
		Name:      "Presidio Lely International Airport",
		City:      "Presidio",
		State:     "Texas",
		Country:   "US",
		Elevation: 2932,
		Latitude:  29.6340999603,
		Longitude: -104.3619995117,
		Timezone:  "America/Chicago",
	},
	"KT78": {
		ICAO:      "KT78",
		Name:      "Liberty Municipal Airport",
		City:      "Liberty",
		State:     "Texas",
		Country:   "US",
		Elevation: 70,
		Latitude:  30.0778007507,
		Longitude: -94.698600769,
		Timezone:  "America/Chicago",
	},
	"KT82": {
		ICAO:      "KT82",
		Name:      "Gillespie County Airport",
		City:      "Fredericksburg",
		State:     "Texas",
		Country:   "US",
		Elevation: 1695,
		Latitude:  30.2432003021,
		Longitude: -98.9092025757,
		Timezone:  "America/Chicago",
	},
	"KT85": {
		ICAO:      "KT85",
		Name:      "Yoakum Municipal Airport",
		City:      "Yoakum",
		State:     "Texas",
		Country:   "US",
		Elevation: 365,
		Latitude:  29.3131999969,
		Longitude: -97.1383972168,
		Timezone:  "America/Chicago",
	},
	"KT88": {
		ICAO:      "KT88",
		Name:      "Colorado City Airport",
		City:      "Colorado City",
		State:     "Texas",
		Country:   "US",
		Elevation: 2214,
		Latitude:  32.4683990479,
		Longitude: -100.9209976196,
		Timezone:  "America/Chicago",
	},
	"KT90": {
		ICAO:      "KT90",
		Name:      "Chambers County Winnie Stowell Airport",
		City:      "Winnie/Stowell",
		State:     "Texas",
		Country:   "US",
		Elevation: 25,
		Latitude:  29.80410004,
		Longitude: -94.43099976,
		Timezone:  "America/Chicago",
	},
	"KT92": {
		ICAO:      "KT92",
		Name:      "Mason County Airport",
		City:      "Mason",
		State:     "Texas",
		Country:   "US",
		Elevation: 1502,
		Latitude:  30.7322006226,
		Longitude: -99.1843032837,
		Timezone:  "America/Chicago",
	},
	"KT93": {
		ICAO:      "KT93",
		Name:      "Follett Lipscomb County Airport",
		City:      "Follett",
		State:     "Texas",
		Country:   "US",
		Elevation: 2601,
		Latitude:  36.4407997131,
		Longitude: -100.1240005493,
		Timezone:  "America/Chicago",
	},
	"KTAD": {
		ICAO:      "KTAD",
		IATA:      "TAD",
		Name:      "Perry Stokes Airport",
		City:      "Trinidad",
		State:     "Colorado",
		Country:   "US",
		Elevation: 5762,
		Latitude:  37.2593994141,
		Longitude: -104.341003418,
		Timezone:  "America/Denver",
	},
	"KTAN": {
		ICAO:      "KTAN",
		Name:      "Taunton Municipal King Field",
		City:      "Taunton",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 43,
		Latitude:  41.8744010925,
		Longitude: -71.0166015625,
		Timezone:  "America/New_York",
	},
	"KTAZ": {
		ICAO:      "KTAZ",
		Name:      "Taylorville Municipal Airport",
		City:      "Taylorville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 622,
		Latitude:  39.534198761,
		Longitude: -89.327796936,
		Timezone:  "America/Chicago",
	},
	"KTBN": {
		ICAO:      "KTBN",
		IATA:      "TBN",
		Name:      "Waynesville-St. Robert Regional Forney field",
		City:      "Fort Leonard Wood",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1159,
		Latitude:  37.74160004,
		Longitude: -92.14070129,
		Timezone:  "America/Chicago",
	},
	"KTBR": {
		ICAO:      "KTBR",
		IATA:      "TBR",
		Name:      "Statesboro Bulloch County Airport",
		City:      "Statesboro",
		State:     "Georgia",
		Country:   "US",
		Elevation: 187,
		Latitude:  32.4827003479,
		Longitude: -81.7369003296,
		Timezone:  "America/New_York",
	},
	"KTCC": {
		ICAO:      "KTCC",
		IATA:      "TCC",
		Name:      "Tucumcari Municipal Airport",
		City:      "Tucumcari",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4065,
		Latitude:  35.182800293,
		Longitude: -103.6029968262,
		Timezone:  "America/Denver",
	},
	"KTCL": {
		ICAO:      "KTCL",
		IATA:      "TCL",
		Name:      "Tuscaloosa Regional Airport",
		City:      "Tuscaloosa",
		State:     "Alabama",
		Country:   "US",
		Elevation: 170,
		Latitude:  33.2206001282,
		Longitude: -87.6113967896,
		Timezone:  "America/Chicago",
	},
	"KTCM": {
		ICAO:      "KTCM",
		IATA:      "TCM",
		Name:      "McChord Air Force Base",
		City:      "Tacoma",
		State:     "Washington",
		Country:   "US",
		Elevation: 322,
		Latitude:  47.1376991272,
		Longitude: -122.4759979248,
		Timezone:  "America/Los_Angeles",
	},
	"KTCS": {
		ICAO:      "KTCS",
		IATA:      "TCS",
		Name:      "Truth Or Consequences Municipal Airport",
		City:      "Truth Or Consequences",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4853,
		Latitude:  33.2369003296,
		Longitude: -107.2720031738,
		Timezone:  "America/Denver",
	},
	"KTCY": {
		ICAO:      "KTCY",
		Name:      "Tracy Municipal Airport",
		City:      "Tracy",
		State:     "California",
		Country:   "US",
		Elevation: 193,
		Latitude:  37.688999176,
		Longitude: -121.4420013428,
		Timezone:  "America/Los_Angeles",
	},
	"KTDF": {
		ICAO:      "KTDF",
		Name:      "Person County Airport",
		City:      "Roxboro",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 609,
		Latitude:  36.28490067,
		Longitude: -78.98419952,
		Timezone:  "America/New_York",
	},
	"KTDO": {
		ICAO:      "KTDO",
		IATA:      "TDO",
		Name:      "Ed Carlson Memorial Field South Lewis County Airport",
		City:      "Toledo",
		State:     "Washington",
		Country:   "US",
		Elevation: 374,
		Latitude:  46.4771995544,
		Longitude: -122.8059997559,
		Timezone:  "America/Los_Angeles",
	},
	"KTDW": {
		ICAO:      "KTDW",
		IATA:      "TDW",
		Name:      "Tradewind Airport",
		City:      "Amarillo",
		State:     "Texas",
		Country:   "US",
		Elevation: 3649,
		Latitude:  35.1698989868,
		Longitude: -101.8259963989,
		Timezone:  "America/Chicago",
	},
	"KTDZ": {
		ICAO:      "KTDZ",
		IATA:      "TDZ",
		Name:      "Metcalf Field",
		City:      "Toledo",
		State:     "Ohio",
		Country:   "US",
		Elevation: 623,
		Latitude:  41.56489944,
		Longitude: -83.4822998,
		Timezone:  "America/New_York",
	},
	"KTEB": {
		ICAO:      "KTEB",
		IATA:      "TEB",
		Name:      "Teterboro Airport",
		City:      "Teterboro",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 9,
		Latitude:  40.8501014709,
		Longitude: -74.060798645,
		Timezone:  "America/New_York",
	},
	"KTEL": {
		ICAO:      "KTEL",
		Name:      "Perry County Municipal Airport",
		City:      "Tell City",
		State:     "Indiana",
		Country:   "US",
		Elevation: 660,
		Latitude:  38.0177002,
		Longitude: -86.69090271,
		Timezone:  "America/Indiana/Tell_City",
	},
	"KTEW": {
		ICAO:      "KTEW",
		Name:      "Mason Jewett Field",
		City:      "Mason",
		State:     "Michigan",
		Country:   "US",
		Elevation: 920,
		Latitude:  42.56579971,
		Longitude: -84.42320251,
		Timezone:  "America/Detroit",
	},
	"KTEX": {
		ICAO:      "KTEX",
		IATA:      "TEX",
		Name:      "Telluride Regional Airport",
		City:      "Telluride",
		State:     "Colorado",
		Country:   "US",
		Elevation: 9070,
		Latitude:  37.9538002,
		Longitude: -107.9079971,
		Timezone:  "America/Denver",
	},
	"KTFP": {
		ICAO:      "KTFP",
		Name:      "T P Mc Campbell Airport",
		City:      "Ingleside",
		State:     "Texas",
		Country:   "US",
		Elevation: 18,
		Latitude:  27.9130001068,
		Longitude: -97.2115020752,
		Timezone:  "America/Chicago",
	},
	"KTGC": {
		ICAO:      "KTGC",
		Name:      "Gibson County Airport",
		City:      "Trenton",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 359,
		Latitude:  35.9324989319,
		Longitude: -88.8488998413,
		Timezone:  "America/Chicago",
	},
	"KTGI": {
		ICAO:      "KTGI",
		Name:      "Tangier Island Airport",
		City:      "Tangier",
		State:     "Virginia",
		Country:   "US",
		Elevation: 5,
		Latitude:  37.8250999451,
		Longitude: -75.9978027344,
		Timezone:  "America/New_York",
	},
	"KTHA": {
		ICAO:      "KTHA",
		IATA:      "THA",
		Name:      "Tullahoma Regional Arpt/Wm Northern Field",
		City:      "Tullahoma",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1083,
		Latitude:  35.38010025,
		Longitude: -86.24639893,
		Timezone:  "America/Chicago",
	},
	"KTHM": {
		ICAO:      "KTHM",
		IATA:      "THM",
		Name:      "Thompson Falls Airport",
		City:      "Thompson Falls",
		State:     "Montana",
		Country:   "US",
		Elevation: 2467,
		Latitude:  47.5735015869,
		Longitude: -115.28099823,
		Timezone:  "America/Denver",
	},
	"KTHP": {
		ICAO:      "KTHP",
		IATA:      "THP",
		Name:      "Hot Springs Co Thermopolis Municipal Airport",
		City:      "Thermopolis",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4592,
		Latitude:  43.6582984924,
		Longitude: -108.2129974365,
		Timezone:  "America/Denver",
	},
	"KTHV": {
		ICAO:      "KTHV",
		IATA:      "THV",
		Name:      "York Airport",
		City:      "York",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 495,
		Latitude:  39.91699982,
		Longitude: -76.8730011,
		Timezone:  "America/New_York",
	},
	"KTIF": {
		ICAO:      "KTIF",
		Name:      "Thomas County Airport",
		City:      "Thedford",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2925,
		Latitude:  41.96220016,
		Longitude: -100.5690002,
		Timezone:  "America/Chicago",
	},
	"KTIK": {
		ICAO:      "KTIK",
		IATA:      "TIK",
		Name:      "Tinker Air Force Base",
		City:      "Oklahoma City",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1291,
		Latitude:  35.4146995544,
		Longitude: -97.3865966797,
		Timezone:  "America/Chicago",
	},
	"KTIP": {
		ICAO:      "KTIP",
		Name:      "Rantoul National Avn Center-Frank Elliot field",
		City:      "Rantoul",
		State:     "Illinois",
		Country:   "US",
		Elevation: 737,
		Latitude:  40.29359818,
		Longitude: -88.14240265,
		Timezone:  "America/Chicago",
	},
	"KTIW": {
		ICAO:      "KTIW",
		IATA:      "TIW",
		Name:      "Tacoma Narrows Airport",
		City:      "Tacoma",
		State:     "Washington",
		Country:   "US",
		Elevation: 294,
		Latitude:  47.26789856,
		Longitude: -122.5780029,
		Timezone:  "America/Los_Angeles",
	},
	"KTIX": {
		ICAO:      "KTIX",
		IATA:      "TIX",
		Name:      "Space Coast Regional Airport",
		City:      "Titusville",
		State:     "Florida",
		Country:   "US",
		Elevation: 34,
		Latitude:  28.514799118,
		Longitude: -80.7992019653,
		Timezone:  "America/New_York",
	},
	"KTKC": {
		ICAO:      "KTKC",
		Name:      "Tracy Municipal Airport",
		City:      "Tracy",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1340,
		Latitude:  44.2490997314,
		Longitude: -95.6072998047,
		Timezone:  "America/Chicago",
	},
	"KTKI": {
		ICAO:      "KTKI",
		Name:      "Collin County Regional At Mc Kinney Airport",
		City:      "Dallas",
		State:     "Texas",
		Country:   "US",
		Elevation: 585,
		Latitude:  33.17789841,
		Longitude: -96.59049988,
		Timezone:  "America/Chicago",
	},
	"KTKO": {
		ICAO:      "KTKO",
		Name:      "Mankato Airport",
		City:      "Mankato",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1859,
		Latitude:  39.8027992249,
		Longitude: -98.2211990356,
		Timezone:  "America/Chicago",
	},
	"KTKV": {
		ICAO:      "KTKV",
		Name:      "Tomahawk Regional Airport",
		City:      "Tomahawk",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1487,
		Latitude:  45.46910095,
		Longitude: -89.80570221,
		Timezone:  "America/Chicago",
	},
	"KTKX": {
		ICAO:      "KTKX",
		IATA:      "KNT",
		Name:      "Kennett Memorial Airport",
		City:      "Kennett",
		State:     "Missouri",
		Country:   "US",
		Elevation: 262,
		Latitude:  36.2258987427,
		Longitude: -90.0365982056,
		Timezone:  "America/Chicago",
	},
	"KTLH": {
		ICAO:      "KTLH",
		IATA:      "TLH",
		Name:      "Tallahassee Regional Airport",
		City:      "Tallahassee",
		State:     "Florida",
		Country:   "US",
		Elevation: 81,
		Latitude:  30.3964996338,
		Longitude: -84.3503036499,
		Timezone:  "America/New_York",
	},
	"KTLR": {
		ICAO:      "KTLR",
		IATA:      "TLR",
		Name:      "Mefford Field",
		City:      "Tulare",
		State:     "California",
		Country:   "US",
		Elevation: 265,
		Latitude:  36.15629959,
		Longitude: -119.3259964,
		Timezone:  "America/Los_Angeles",
	},
	"KTMA": {
		ICAO:      "KTMA",
		IATA:      "TMA",
		Name:      "Henry Tift Myers Airport",
		City:      "Tifton",
		State:     "Georgia",
		Country:   "US",
		Elevation: 355,
		Latitude:  31.4290008545,
		Longitude: -83.4885025024,
		Timezone:  "America/New_York",
	},
	"KTMB": {
		ICAO:      "KTMB",
		IATA:      "TMB",
		Name:      "Miami Executive Airport",
		City:      "Miami",
		State:     "Miami",
		Country:   "US",
		Elevation: 10,
		Latitude:  25.6475637,
		Longitude: -80.4332246,
		Timezone:  "America/New_York",
	},
	"KTME": {
		ICAO:      "KTME",
		Name:      "Houston Executive Airport",
		City:      "Houston",
		State:     "Texas",
		Country:   "US",
		Elevation: 166,
		Latitude:  29.8071994781,
		Longitude: -95.8979034424,
		Timezone:  "America/Chicago",
	},
	"KTMK": {
		ICAO:      "KTMK",
		IATA:      "OTK",
		Name:      "Tillamook Airport",
		City:      "Tillamook",
		State:     "Oregon",
		Country:   "US",
		Elevation: 36,
		Latitude:  45.4182014465,
		Longitude: -123.814002991,
		Timezone:  "America/Los_Angeles",
	},
	"KTMT": {
		ICAO:      "KTMT",
		IATA:      "ASQ",
		Name:      "Austin Airport",
		City:      "Austin",
		State:     "Nevada",
		Country:   "US",
		Elevation: 5730,
		Latitude:  39.4679985046,
		Longitude: -117.194999695,
		Timezone:  "America/Los_Angeles",
	},
	"KTNP": {
		ICAO:      "KTNP",
		IATA:      "TNP",
		Name:      "Twentynine Palms Airport",
		City:      "Twentynine Palms",
		State:     "California",
		Country:   "US",
		Elevation: 1888,
		Latitude:  34.13159943,
		Longitude: -115.9459991,
		Timezone:  "America/Los_Angeles",
	},
	"KTNT": {
		ICAO:      "KTNT",
		IATA:      "TNT",
		Name:      "Dade Collier Training and Transition Airport",
		City:      "Miami",
		State:     "Florida",
		Country:   "US",
		Elevation: 13,
		Latitude:  25.8617992401,
		Longitude: -80.8970031738,
		Timezone:  "America/New_York",
	},
	"KTNU": {
		ICAO:      "KTNU",
		IATA:      "TNU",
		Name:      "Newton Municipal Airport",
		City:      "Newton",
		State:     "Iowa",
		Country:   "US",
		Elevation: 953,
		Latitude:  41.6744003296,
		Longitude: -93.021697998,
		Timezone:  "America/Chicago",
	},
	"KTNX": {
		ICAO:      "KTNX",
		IATA:      "XSD",
		Name:      "Tonopah Test Range Airport",
		City:      "Tonopah",
		State:     "Nevada",
		Country:   "US",
		Elevation: 5549,
		Latitude:  37.7988014221,
		Longitude: -116.78099823,
		Timezone:  "America/Los_Angeles",
	},
	"KTOA": {
		ICAO:      "KTOA",
		IATA:      "TOA",
		Name:      "Zamperini Field",
		City:      "Torrance",
		State:     "California",
		Country:   "US",
		Elevation: 103,
		Latitude:  33.8033981323,
		Longitude: -118.3399963379,
		Timezone:  "America/Los_Angeles",
	},
	"KTOB": {
		ICAO:      "KTOB",
		Name:      "Dodge Center Airport",
		City:      "Dodge Center",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1305,
		Latitude:  44.0180015564,
		Longitude: -92.8314971924,
		Timezone:  "America/Chicago",
	},
	"KTOC": {
		ICAO:      "KTOC",
		IATA:      "TOC",
		Name:      "Toccoa Rg Letourneau Field",
		City:      "Toccoa",
		State:     "Georgia",
		Country:   "US",
		Elevation: 996,
		Latitude:  34.59379959,
		Longitude: -83.29579926,
		Timezone:  "America/New_York",
	},
	"KTOI": {
		ICAO:      "KTOI",
		IATA:      "TOI",
		Name:      "Troy Municipal Airport",
		City:      "Troy",
		State:     "Alabama",
		Country:   "US",
		Elevation: 398,
		Latitude:  31.8603992462,
		Longitude: -86.0121002197,
		Timezone:  "America/Chicago",
	},
	"KTOL": {
		ICAO:      "KTOL",
		IATA:      "TOL",
		Name:      "Toledo Express Airport",
		City:      "Toledo",
		State:     "Ohio",
		Country:   "US",
		Elevation: 683,
		Latitude:  41.58679962,
		Longitude: -83.80780029,
		Timezone:  "America/New_York",
	},
	"KTOP": {
		ICAO:      "KTOP",
		IATA:      "TOP",
		Name:      "Philip Billard Municipal Airport",
		City:      "Topeka",
		State:     "Kansas",
		Country:   "US",
		Elevation: 881,
		Latitude:  39.0686988831,
		Longitude: -95.6224975586,
		Timezone:  "America/Chicago",
	},
	"KTOR": {
		ICAO:      "KTOR",
		IATA:      "TOR",
		Name:      "Torrington Municipal Airport",
		City:      "Torrington",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4207,
		Latitude:  42.0644989,
		Longitude: -104.1529999,
		Timezone:  "America/Denver",
	},
	"KTPA": {
		ICAO:      "KTPA",
		IATA:      "TPA",
		Name:      "Tampa International Airport",
		City:      "Tampa",
		State:     "Florida",
		Country:   "US",
		Elevation: 26,
		Latitude:  27.9755001068,
		Longitude: -82.533203125,
		Timezone:  "America/New_York",
	},
	"KTPF": {
		ICAO:      "KTPF",
		IATA:      "TPF",
		Name:      "Peter O Knight Airport",
		City:      "Tampa",
		State:     "Florida",
		Country:   "US",
		Elevation: 8,
		Latitude:  27.915599823,
		Longitude: -82.4493026733,
		Timezone:  "America/New_York",
	},
	"KTPH": {
		ICAO:      "KTPH",
		IATA:      "TPH",
		Name:      "Tonopah Airport",
		City:      "Tonopah",
		State:     "Nevada",
		Country:   "US",
		Elevation: 5430,
		Latitude:  38.06019974,
		Longitude: -117.086998,
		Timezone:  "America/Los_Angeles",
	},
	"KTPL": {
		ICAO:      "KTPL",
		IATA:      "TPL",
		Name:      "Draughon Miller Central Texas Regional Airport",
		City:      "Temple",
		State:     "Texas",
		Country:   "US",
		Elevation: 682,
		Latitude:  31.1525001526,
		Longitude: -97.4077987671,
		Timezone:  "America/Chicago",
	},
	"KTQE": {
		ICAO:      "KTQE",
		Name:      "Tekamah Municipal Airport",
		City:      "Tekamah",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1027,
		Latitude:  41.7635002136,
		Longitude: -96.1779022217,
		Timezone:  "America/Chicago",
	},
	"KTQH": {
		ICAO:      "KTQH",
		Name:      "Tahlequah Municipal Airport",
		City:      "Tahlequah",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 874,
		Latitude:  35.92890167,
		Longitude: -95.00450134,
		Timezone:  "America/Chicago",
	},
	"KTQK": {
		ICAO:      "KTQK",
		Name:      "Scott City Municipal Airport",
		City:      "Scott City",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2963,
		Latitude:  38.4743003845,
		Longitude: -100.8850021362,
		Timezone:  "America/Chicago",
	},
	"KTRI": {
		ICAO:      "KTRI",
		IATA:      "TRI",
		Name:      "Tri Cities Regional Tn Va Airport",
		City:      "Bristol/Johnson/Kingsport",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1519,
		Latitude:  36.4752006531,
		Longitude: -82.4074020386,
		Timezone:  "America/New_York",
	},
	"KTRK": {
		ICAO:      "KTRK",
		IATA:      "TKF",
		Name:      "Truckee Tahoe Airport",
		City:      "Truckee",
		State:     "California",
		Country:   "US",
		Elevation: 5900,
		Latitude:  39.3199996948,
		Longitude: -120.1399993896,
		Timezone:  "America/Los_Angeles",
	},
	"KTRL": {
		ICAO:      "KTRL",
		IATA:      "TRL",
		Name:      "Terrell Municipal Airport",
		City:      "Terrell",
		State:     "Texas",
		Country:   "US",
		Elevation: 474,
		Latitude:  32.7092018127,
		Longitude: -96.2674026489,
		Timezone:  "America/Chicago",
	},
	"KTRM": {
		ICAO:      "KTRM",
		IATA:      "TRM",
		Name:      "Jacqueline Cochran Regional Airport",
		City:      "Palm Springs",
		State:     "California",
		Country:   "US",
		Elevation: -115,
		Latitude:  33.626701355,
		Longitude: -116.1600036621,
		Timezone:  "America/Los_Angeles",
	},
	"KTRX": {
		ICAO:      "KTRX",
		IATA:      "TRX",
		Name:      "Trenton Municipal Airport",
		City:      "Trenton",
		State:     "Missouri",
		Country:   "US",
		Elevation: 758,
		Latitude:  40.08349991,
		Longitude: -93.59059906,
		Timezone:  "America/Chicago",
	},
	"KTSO": {
		ICAO:      "KTSO",
		Name:      "Carroll County-Tolson Airport",
		City:      "Carrollton",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1163,
		Latitude:  40.56190109,
		Longitude: -81.07749939,
		Timezone:  "America/New_York",
	},
	"KTSP": {
		ICAO:      "KTSP",
		IATA:      "TSP",
		Name:      "Tehachapi Municipal Airport",
		City:      "Tehachapi",
		State:     "California",
		Country:   "US",
		Elevation: 4001,
		Latitude:  35.1349983215,
		Longitude: -118.4390029907,
		Timezone:  "America/Los_Angeles",
	},
	"KTTA": {
		ICAO:      "KTTA",
		Name:      "Sanford-Lee County Regional Airport",
		City:      "Sanford",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 246,
		Latitude:  35.58369827,
		Longitude: -79.10079956,
		Timezone:  "America/New_York",
	},
	"KTTD": {
		ICAO:      "KTTD",
		IATA:      "TTD",
		Name:      "Portland Troutdale Airport",
		City:      "Portland",
		State:     "Oregon",
		Country:   "US",
		Elevation: 39,
		Latitude:  45.5494003296,
		Longitude: -122.4010009766,
		Timezone:  "America/Los_Angeles",
	},
	"KTTF": {
		ICAO:      "KTTF",
		Name:      "Custer Airport",
		City:      "Monroe",
		State:     "Michigan",
		Country:   "US",
		Elevation: 616,
		Latitude:  41.9398994446,
		Longitude: -83.4347000122,
		Timezone:  "America/Detroit",
	},
	"KTTN": {
		ICAO:      "KTTN",
		IATA:      "TTN",
		Name:      "Trenton Mercer Airport",
		City:      "Trenton",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 213,
		Latitude:  40.2766990662,
		Longitude: -74.8134994507,
		Timezone:  "America/New_York",
	},
	"KTTS": {
		ICAO:      "KTTS",
		Name:      "Nasa Shuttle Landing Facility Airport",
		City:      "Titusville",
		State:     "Florida",
		Country:   "US",
		Elevation: 10,
		Latitude:  28.6149997711,
		Longitude: -80.6945037842,
		Timezone:  "America/New_York",
	},
	"KTUL": {
		ICAO:      "KTUL",
		IATA:      "TUL",
		Name:      "Tulsa International Airport",
		City:      "Tulsa",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 677,
		Latitude:  36.1983985901,
		Longitude: -95.8880996704,
		Timezone:  "America/Chicago",
	},
	"KTUP": {
		ICAO:      "KTUP",
		IATA:      "TUP",
		Name:      "Tupelo Regional Airport",
		City:      "Tupelo",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 346,
		Latitude:  34.2681007385,
		Longitude: -88.7698974609,
		Timezone:  "America/Chicago",
	},
	"KTUS": {
		ICAO:      "KTUS",
		IATA:      "TUS",
		Name:      "Tucson International Airport",
		City:      "Tucson",
		State:     "Arizona",
		Country:   "US",
		Elevation: 2643,
		Latitude:  32.1161003113,
		Longitude: -110.9410018921,
		Timezone:  "America/Phoenix",
	},
	"KTVB": {
		ICAO:      "KTVB",
		Name:      "Cabool Memorial Airport",
		City:      "Cabool",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1220,
		Latitude:  37.1324005127,
		Longitude: -92.0839996338,
		Timezone:  "America/Chicago",
	},
	"KTVC": {
		ICAO:      "KTVC",
		IATA:      "TVC",
		Name:      "Cherry Capital Airport",
		City:      "Traverse City",
		State:     "Michigan",
		Country:   "US",
		Elevation: 624,
		Latitude:  44.7414016724,
		Longitude: -85.5821990967,
		Timezone:  "America/Detroit",
	},
	"KTVF": {
		ICAO:      "KTVF",
		IATA:      "TVF",
		Name:      "Thief River Falls Regional Airport",
		City:      "Thief River Falls",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1119,
		Latitude:  48.06570053,
		Longitude: -96.18499756,
		Timezone:  "America/Chicago",
	},
	"KTVI": {
		ICAO:      "KTVI",
		IATA:      "TVI",
		Name:      "Thomasville Regional Airport",
		City:      "Thomasville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 264,
		Latitude:  30.901599884,
		Longitude: -83.8813018799,
		Timezone:  "America/New_York",
	},
	"KTVK": {
		ICAO:      "KTVK",
		Name:      "Centerville Municipal Airport",
		City:      "Centerville",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1023,
		Latitude:  40.68389893,
		Longitude: -92.90100098,
		Timezone:  "America/Chicago",
	},
	"KTVL": {
		ICAO:      "KTVL",
		IATA:      "TVL",
		Name:      "Lake Tahoe Airport",
		City:      "South Lake Tahoe",
		State:     "California",
		Country:   "US",
		Elevation: 6264,
		Latitude:  38.893901825,
		Longitude: -119.9950027466,
		Timezone:  "America/Los_Angeles",
	},
	"KTVR": {
		ICAO:      "KTVR",
		Name:      "Vicksburg Tallulah Regional Airport",
		City:      "Tallulah/Vicksburg",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 86,
		Latitude:  32.351600647,
		Longitude: -91.0277023315,
		Timezone:  "America/Chicago",
	},
	"KTVY": {
		ICAO:      "KTVY",
		Name:      "Bolinder Field Tooele Valley Airport",
		City:      "Tooele",
		State:     "Utah",
		Country:   "US",
		Elevation: 4322,
		Latitude:  40.61230087,
		Longitude: -112.3509979,
		Timezone:  "America/Denver",
	},
	"KTWF": {
		ICAO:      "KTWF",
		IATA:      "TWF",
		Name:      "Joslin Field Magic Valley Regional Airport",
		City:      "Twin Falls",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4154,
		Latitude:  42.48180008,
		Longitude: -114.487999,
		Timezone:  "America/Boise",
	},
	"KTWM": {
		ICAO:      "KTWM",
		Name:      "Richard B Helgeson Airport",
		City:      "Two Harbors",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1073,
		Latitude:  47.04919815,
		Longitude: -91.74510193,
		Timezone:  "America/Chicago",
	},
	"KTWT": {
		ICAO:      "KTWT",
		Name:      "Sturgis Municipal Airport",
		City:      "Sturgis",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 372,
		Latitude:  37.5407981873,
		Longitude: -87.9517974854,
		Timezone:  "America/Chicago",
	},
	"KTXK": {
		ICAO:      "KTXK",
		IATA:      "TXK",
		Name:      "Texarkana Regional Webb Field",
		City:      "Texarkana",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 390,
		Latitude:  33.4537010193,
		Longitude: -93.9909973145,
		Timezone:  "America/Chicago",
	},
	"KTYL": {
		ICAO:      "KTYL",
		IATA:      "TYZ",
		Name:      "Taylor Airport",
		City:      "Taylor",
		State:     "Arizona",
		Country:   "US",
		Elevation: 5823,
		Latitude:  34.45280075,
		Longitude: -110.1149979,
		Timezone:  "America/Phoenix",
	},
	"KTYQ": {
		ICAO:      "KTYQ",
		Name:      "Indianapolis Executive Airport",
		City:      "Indianapolis",
		State:     "Indiana",
		Country:   "US",
		Elevation: 922,
		Latitude:  40.0307006836,
		Longitude: -86.2514038086,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KTYR": {
		ICAO:      "KTYR",
		IATA:      "TYR",
		Name:      "Tyler Pounds Regional Airport",
		City:      "Tyler",
		State:     "Texas",
		Country:   "US",
		Elevation: 544,
		Latitude:  32.3540992737,
		Longitude: -95.4023971558,
		Timezone:  "America/Chicago",
	},
	"KTYS": {
		ICAO:      "KTYS",
		IATA:      "TYS",
		Name:      "McGhee Tyson Airport",
		City:      "Knoxville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 981,
		Latitude:  35.81100082,
		Longitude: -83.9940033,
		Timezone:  "America/New_York",
	},
	"KTZR": {
		ICAO:      "KTZR",
		Name:      "Bolton Field",
		City:      "Columbus",
		State:     "Ohio",
		Country:   "US",
		Elevation: 905,
		Latitude:  39.9011993408,
		Longitude: -83.1369018555,
		Timezone:  "America/New_York",
	},
	"KTZT": {
		ICAO:      "KTZT",
		Name:      "Belle Plaine Municipal Airport",
		City:      "Belle Plaine",
		State:     "Iowa",
		Country:   "US",
		Elevation: 771,
		Latitude:  41.8787994385,
		Longitude: -92.2845993042,
		Timezone:  "America/Chicago",
	},
	"KTZV": {
		ICAO:      "KTZV",
		Name:      "Tompkinsville Monroe County Airport",
		City:      "Tompkinsville",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 1036,
		Latitude:  36.7290000916,
		Longitude: -85.6523971558,
		Timezone:  "America/Chicago",
	},
	"KU00": {
		ICAO:      "KU00",
		Name:      "Leadore Airport",
		City:      "Leadore",
		State:     "Idaho",
		Country:   "US",
		Elevation: 6018,
		Latitude:  44.6738014221,
		Longitude: -113.3529968262,
		Timezone:  "America/Boise",
	},
	"KU01": {
		ICAO:      "KU01",
		Name:      "American Falls Airport",
		City:      "American Falls",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4419,
		Latitude:  42.7972984314,
		Longitude: -112.8249969482,
		Timezone:  "America/Boise",
	},
	"KU02": {
		ICAO:      "KU02",
		Name:      "Mccarley Field",
		City:      "Blackfoot",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4488,
		Latitude:  43.2093009949,
		Longitude: -112.3499984741,
		Timezone:  "America/Boise",
	},
	"KU03": {
		ICAO:      "KU03",
		Name:      "Buhl Municipal Airport",
		City:      "Buhl",
		State:     "Idaho",
		Country:   "US",
		Elevation: 3660,
		Latitude:  42.5915985107,
		Longitude: -114.7969970703,
		Timezone:  "America/Boise",
	},
	"KU05": {
		ICAO:      "KU05",
		Name:      "Riddick Field",
		City:      "Philipsburg",
		State:     "Montana",
		Country:   "US",
		Elevation: 5212,
		Latitude:  46.3194007874,
		Longitude: -113.3050003052,
		Timezone:  "America/Denver",
	},
	"KU06": {
		ICAO:      "KU06",
		Name:      "Cokeville Municipal Airport",
		City:      "Cokeville",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 6270,
		Latitude:  42.0457992554,
		Longitude: -110.966003418,
		Timezone:  "America/Denver",
	},
	"KU07": {
		ICAO:      "KU07",
		IATA:      "BFG",
		Name:      "Bullfrog Basin Airport",
		City:      "Glen Canyon Natl Rec Area",
		State:     "Utah",
		Country:   "US",
		Elevation: 4167,
		Latitude:  37.54579926,
		Longitude: -110.7129974,
		Timezone:  "America/Denver",
	},
	"KU08": {
		ICAO:      "KU08",
		Name:      "Perkins Field",
		City:      "Overton",
		State:     "Nevada",
		Country:   "US",
		Elevation: 1358,
		Latitude:  36.5680007935,
		Longitude: -114.4430007935,
		Timezone:  "America/Los_Angeles",
	},
	"KU09": {
		ICAO:      "KU09",
		Name:      "Fort Belknap Agency Airport",
		City:      "Harlem",
		State:     "Montana",
		Country:   "US",
		Elevation: 2374,
		Latitude:  48.481098175,
		Longitude: -108.7689971924,
		Timezone:  "America/Denver",
	},
	"KU10": {
		ICAO:      "KU10",
		Name:      "Preston Airport",
		City:      "Preston",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4728,
		Latitude:  42.1068992615,
		Longitude: -111.9130020142,
		Timezone:  "America/Boise",
	},
	"KU12": {
		ICAO:      "KU12",
		Name:      "Stanford Field",
		City:      "St Anthony",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4966,
		Latitude:  43.9457015991,
		Longitude: -111.6839981079,
		Timezone:  "America/Boise",
	},
	"KU13": {
		ICAO:      "KU13",
		Name:      "Junction Airport",
		City:      "Junction",
		State:     "Utah",
		Country:   "US",
		Elevation: 6069,
		Latitude:  38.25,
		Longitude: -112.2249984741,
		Timezone:  "America/Denver",
	},
	"KU14": {
		ICAO:      "KU14",
		Name:      "Nephi Municipal Airport",
		City:      "Nephi",
		State:     "Utah",
		Country:   "US",
		Elevation: 5022,
		Latitude:  39.73880005,
		Longitude: -111.8720016,
		Timezone:  "America/Denver",
	},
	"KU25": {
		ICAO:      "KU25",
		Name:      "Dubois Municipal Airport",
		City:      "Dubois",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 7291,
		Latitude:  43.5483016968,
		Longitude: -109.6900024414,
		Timezone:  "America/Denver",
	},
	"KU30": {
		ICAO:      "KU30",
		Name:      "Temple Bar Airport",
		City:      "Temple Bar",
		State:     "Arizona",
		Country:   "US",
		Elevation: 1549,
		Latitude:  36.0205001831,
		Longitude: -114.3349990845,
		Timezone:  "America/Phoenix",
	},
	"KU34": {
		ICAO:      "KU34",
		IATA:      "RVR",
		Name:      "Green River Municipal Airport",
		City:      "Green River",
		State:     "Utah",
		Country:   "US",
		Elevation: 4225,
		Latitude:  38.9613990784,
		Longitude: -110.226997375,
		Timezone:  "America/Denver",
	},
	"KU36": {
		ICAO:      "KU36",
		Name:      "Aberdeen Municipal Airport",
		City:      "Aberdeen",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4470,
		Latitude:  42.9210014343,
		Longitude: -112.8809967041,
		Timezone:  "America/Boise",
	},
	"KU42": {
		ICAO:      "KU42",
		Name:      "South Valley Regional Airport",
		City:      "West Jordan",
		State:     "Utah",
		Country:   "US",
		Elevation: 4607,
		Latitude:  40.6195556,
		Longitude: -111.9928889,
		Timezone:  "America/Denver",
	},
	"KU43": {
		ICAO:      "KU43",
		Name:      "Monticello Airport",
		City:      "Monticello",
		State:     "Utah",
		Country:   "US",
		Elevation: 6998,
		Latitude:  37.9371986389,
		Longitude: -109.3470001221,
		Timezone:  "America/Denver",
	},
	"KU52": {
		ICAO:      "KU52",
		Name:      "Beaver Municipal Airport",
		City:      "Beaver",
		State:     "Utah",
		Country:   "US",
		Elevation: 5863,
		Latitude:  38.23070145,
		Longitude: -112.6750031,
		Timezone:  "America/Denver",
	},
	"KU55": {
		ICAO:      "KU55",
		Name:      "Panguitch Municipal Airport",
		City:      "Panguitch",
		State:     "Utah",
		Country:   "US",
		Elevation: 6763,
		Latitude:  37.84519958,
		Longitude: -112.3919983,
		Timezone:  "America/Denver",
	},
	"KU56": {
		ICAO:      "KU56",
		Name:      "Rigby Jefferson County Airport",
		City:      "Rigby",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4845,
		Latitude:  43.6444015503,
		Longitude: -111.9290008545,
		Timezone:  "America/Boise",
	},
	"KU58": {
		ICAO:      "KU58",
		Name:      "Downey/Hyde Memorial Airport",
		City:      "Downey",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4906,
		Latitude:  42.42630005,
		Longitude: -112.1090012,
		Timezone:  "America/Boise",
	},
	"KU62": {
		ICAO:      "KU62",
		Name:      "Mackay Airport",
		City:      "Mackay",
		State:     "Idaho",
		Country:   "US",
		Elevation: 5892,
		Latitude:  43.90409851,
		Longitude: -113.6009979,
		Timezone:  "America/Boise",
	},
	"KU63": {
		ICAO:      "KU63",
		Name:      "Bruce Meadows Airport",
		City:      "Stanley",
		State:     "Idaho",
		Country:   "US",
		Elevation: 6370,
		Latitude:  44.4155006409,
		Longitude: -115.3170013428,
		Timezone:  "America/Boise",
	},
	"KU64": {
		ICAO:      "KU64",
		Name:      "Monticello Airport",
		City:      "Monticello",
		State:     "Utah",
		Country:   "US",
		Elevation: 6966,
		Latitude:  37.93243,
		Longitude: -109.341225,
		Timezone:  "America/Denver",
	},
	"KU68": {
		ICAO:      "KU68",
		Name:      "North Big Horn County Airport",
		City:      "Cowley/Lovell/Byron",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4090,
		Latitude:  44.9117012024,
		Longitude: -108.4459991455,
		Timezone:  "America/Denver",
	},
	"KU69": {
		ICAO:      "KU69",
		Name:      "Duchesne Municipal Airport",
		City:      "Duchesne",
		State:     "Utah",
		Country:   "US",
		Elevation: 5826,
		Latitude:  40.1918983459,
		Longitude: -110.3809967041,
		Timezone:  "America/Denver",
	},
	"KU70": {
		ICAO:      "KU70",
		IATA:      "ICS",
		Name:      "Cascade Airport",
		City:      "Cascade",
		State:     "Idaho",
		Country:   "US",
		Elevation: 4742,
		Latitude:  44.4938011169,
		Longitude: -116.01599884,
		Timezone:  "America/Boise",
	},
	"KU76": {
		ICAO:      "KU76",
		Name:      "Mountain Home Municipal Airport",
		City:      "Mountain Home",
		State:     "Idaho",
		Country:   "US",
		Elevation: 3167,
		Latitude:  43.13130188,
		Longitude: -115.7300034,
		Timezone:  "America/Boise",
	},
	"KU77": {
		ICAO:      "KU77",
		Name:      "Spanish Fork Springville Airport",
		City:      "Spanish Fork",
		State:     "Utah",
		Country:   "US",
		Elevation: 4529,
		Latitude:  40.1416015625,
		Longitude: -111.6610031128,
		Timezone:  "America/Denver",
	},
	"KU78": {
		ICAO:      "KU78",
		Name:      "Allen H Tigert Airport",
		City:      "Soda Springs",
		State:     "Idaho",
		Country:   "US",
		Elevation: 5839,
		Latitude:  42.6416015625,
		Longitude: -111.5800018311,
		Timezone:  "America/Boise",
	},
	"KU79": {
		ICAO:      "KU79",
		Name:      "Chamberlain Usfs Airport",
		City:      "Chamberlain Guard Station",
		State:     "Idaho",
		Country:   "US",
		Elevation: 5765,
		Latitude:  45.3791007996,
		Longitude: -115.1969985962,
		Timezone:  "America/Boise",
	},
	"KU81": {
		ICAO:      "KU81",
		Name:      "Cold Meadows US Forest Service Airport",
		City:      "Cold Meadows Guard Station",
		State:     "Idaho",
		Country:   "US",
		Elevation: 7030,
		Latitude:  45.2915992737,
		Longitude: -114.949996948,
		Timezone:  "America/Boise",
	},
	"KU82": {
		ICAO:      "KU82",
		Name:      "Council Municipal Airport",
		City:      "Council",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2963,
		Latitude:  44.7499008179,
		Longitude: -116.4469985962,
		Timezone:  "America/Boise",
	},
	"KU89": {
		ICAO:      "KU89",
		Name:      "Glenns Ferry Municipal Airport",
		City:      "Glenns Ferry",
		State:     "Idaho",
		Country:   "US",
		Elevation: 2536,
		Latitude:  42.9457015991,
		Longitude: -115.3300018311,
		Timezone:  "America/Boise",
	},
	"KU96": {
		ICAO:      "KU96",
		Name:      "Cal Black Memorial Airport",
		City:      "Halls Crossing",
		State:     "Utah",
		Country:   "US",
		Elevation: 4388,
		Latitude:  37.442199707,
		Longitude: -110.5699996948,
		Timezone:  "America/Denver",
	},
	"KUAO": {
		ICAO:      "KUAO",
		Name:      "Aurora State Airport",
		City:      "Aurora",
		State:     "Oregon",
		Country:   "US",
		Elevation: 200,
		Latitude:  45.2471008301,
		Longitude: -122.7699966431,
		Timezone:  "America/Los_Angeles",
	},
	"KUBE": {
		ICAO:      "KUBE",
		Name:      "Cumberland Municipal Airport",
		City:      "Cumberland",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1241,
		Latitude:  45.5060005188,
		Longitude: -91.9811019897,
		Timezone:  "America/Chicago",
	},
	"KUBS": {
		ICAO:      "KUBS",
		IATA:      "UBS",
		Name:      "Columbus Lowndes County Airport",
		City:      "Columbus",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 188,
		Latitude:  33.4654006958,
		Longitude: -88.3803024292,
		Timezone:  "America/Chicago",
	},
	"KUBX": {
		ICAO:      "KUBX",
		Name:      "Cuba Municipal Airport",
		City:      "Cuba",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1023,
		Latitude:  38.0688018799,
		Longitude: -91.4289016724,
		Timezone:  "America/Chicago",
	},
	"KUCP": {
		ICAO:      "KUCP",
		Name:      "New Castle Municipal Airport",
		City:      "New Castle",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1070,
		Latitude:  41.02529907,
		Longitude: -80.41339874,
		Timezone:  "America/New_York",
	},
	"KUCY": {
		ICAO:      "KUCY",
		IATA:      "UCY",
		Name:      "Everett-Stewart Regional Airport",
		City:      "Union City",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 336,
		Latitude:  36.38180161,
		Longitude: -88.98539734,
		Timezone:  "America/Chicago",
	},
	"KUDD": {
		ICAO:      "KUDD",
		IATA:      "UDD",
		Name:      "Bermuda Dunes Airport",
		City:      "Palm Springs",
		State:     "California",
		Country:   "US",
		Elevation: 73,
		Latitude:  33.7484016418,
		Longitude: -116.2750015259,
		Timezone:  "America/Los_Angeles",
	},
	"KUDG": {
		ICAO:      "KUDG",
		Name:      "Darlington County Jetport Airport",
		City:      "Darlington",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 192,
		Latitude:  34.44940186,
		Longitude: -79.89009857,
		Timezone:  "America/New_York",
	},
	"KUES": {
		ICAO:      "KUES",
		IATA:      "UES",
		Name:      "Waukesha County Airport",
		City:      "Waukesha",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 911,
		Latitude:  43.0410003662,
		Longitude: -88.2370986938,
		Timezone:  "America/Chicago",
	},
	"KUGN": {
		ICAO:      "KUGN",
		IATA:      "UGN",
		Name:      "Waukegan Regional Airport",
		City:      "Chicago/Waukegan",
		State:     "Illinois",
		Country:   "US",
		Elevation: 727,
		Latitude:  42.4221992493,
		Longitude: -87.8678970337,
		Timezone:  "America/Chicago",
	},
	"KUIL": {
		ICAO:      "KUIL",
		IATA:      "UIL",
		Name:      "Quillayute Airport",
		City:      "Quillayute",
		State:     "Washington",
		Country:   "US",
		Elevation: 194,
		Latitude:  47.9365997314,
		Longitude: -124.56300354,
		Timezone:  "America/Los_Angeles",
	},
	"KUIN": {
		ICAO:      "KUIN",
		IATA:      "UIN",
		Name:      "Quincy Regional Baldwin Field",
		City:      "Quincy",
		State:     "Illinois",
		Country:   "US",
		Elevation: 768,
		Latitude:  39.94269943,
		Longitude: -91.19460297,
		Timezone:  "America/Chicago",
	},
	"KUKF": {
		ICAO:      "KUKF",
		IATA:      "IKB",
		Name:      "Wilkes County Airport",
		City:      "North Wilkesboro",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 1301,
		Latitude:  36.2228012085,
		Longitude: -81.0982971191,
		Timezone:  "America/New_York",
	},
	"KUKI": {
		ICAO:      "KUKI",
		IATA:      "UKI",
		Name:      "Ukiah Municipal Airport",
		City:      "Ukiah",
		State:     "California",
		Country:   "US",
		Elevation: 614,
		Latitude:  39.1259994507,
		Longitude: -123.2009963989,
		Timezone:  "America/Los_Angeles",
	},
	"KUKL": {
		ICAO:      "KUKL",
		Name:      "Coffey County Airport",
		City:      "Burlington",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1174,
		Latitude:  38.30250168,
		Longitude: -95.72499847,
		Timezone:  "America/Chicago",
	},
	"KUKT": {
		ICAO:      "KUKT",
		IATA:      "UKT",
		Name:      "Quakertown Airport",
		City:      "Quakertown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 526,
		Latitude:  40.4351997375,
		Longitude: -75.3818969727,
		Timezone:  "America/New_York",
	},
	"KULM": {
		ICAO:      "KULM",
		IATA:      "ULM",
		Name:      "New Ulm Municipal Airport",
		City:      "New Ulm",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1011,
		Latitude:  44.3195991516,
		Longitude: -94.5022964478,
		Timezone:  "America/Chicago",
	},
	"KULS": {
		ICAO:      "KULS",
		Name:      "Ulysses Airport",
		City:      "Ulysses",
		State:     "Kansas",
		Country:   "US",
		Elevation: 3071,
		Latitude:  37.60400009,
		Longitude: -101.3740005,
		Timezone:  "America/Chicago",
	},
	"KUMP": {
		ICAO:      "KUMP",
		Name:      "Indianapolis Metropolitan Airport",
		City:      "Indianapolis",
		State:     "Indiana",
		Country:   "US",
		Elevation: 811,
		Latitude:  39.93519974,
		Longitude: -86.04499817,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KUNI": {
		ICAO:      "KUNI",
		IATA:      "ATO",
		Name:      "Ohio University Snyder Field",
		City:      "Athens/Albany",
		State:     "Ohio",
		Country:   "US",
		Elevation: 766,
		Latitude:  39.2109985352,
		Longitude: -82.2313995361,
		Timezone:  "America/New_York",
	},
	"KUNO": {
		ICAO:      "KUNO",
		Name:      "West Plains Municipal Airport",
		City:      "West Plains",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1228,
		Latitude:  36.8783988953,
		Longitude: -91.9027023315,
		Timezone:  "America/Chicago",
	},
	"KUNU": {
		ICAO:      "KUNU",
		IATA:      "UNU",
		Name:      "Dodge County Airport",
		City:      "Juneau",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 934,
		Latitude:  43.42660141,
		Longitude: -88.70320129,
		Timezone:  "America/Chicago",
	},
	"KUNV": {
		ICAO:      "KUNV",
		IATA:      "SCE",
		Name:      "University Park Airport",
		City:      "State College",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1239,
		Latitude:  40.8493003845,
		Longitude: -77.8487014771,
		Timezone:  "America/New_York",
	},
	"KUOS": {
		ICAO:      "KUOS",
		IATA:      "UOS",
		Name:      "Franklin County Airport",
		City:      "Sewanee",
		State:     "Georgia",
		Country:   "US",
		Elevation: 1953,
		Latitude:  35.2051010132,
		Longitude: -85.8981018066,
		Timezone:  "America/Chicago",
	},
	"KUOX": {
		ICAO:      "KUOX",
		IATA:      "UOX",
		Name:      "University Oxford Airport",
		City:      "Oxford",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 452,
		Latitude:  34.3843002319,
		Longitude: -89.5367965698,
		Timezone:  "America/Chicago",
	},
	"KUSE": {
		ICAO:      "KUSE",
		Name:      "Fulton County Airport",
		City:      "Wauseon",
		State:     "Ohio",
		Country:   "US",
		Elevation: 779,
		Latitude:  41.6100997925,
		Longitude: -84.1271972656,
		Timezone:  "America/New_York",
	},
	"KUTA": {
		ICAO:      "KUTA",
		IATA:      "UTM",
		Name:      "Tunica Municipal Airport",
		City:      "Tunica",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 194,
		Latitude:  34.6809997559,
		Longitude: -90.3467025757,
		Timezone:  "America/Chicago",
	},
	"KUTS": {
		ICAO:      "KUTS",
		IATA:      "HTV",
		Name:      "Huntsville Municipal Airport",
		City:      "Huntsville",
		State:     "Texas",
		Country:   "US",
		Elevation: 363,
		Latitude:  30.7469005585,
		Longitude: -95.5871963501,
		Timezone:  "America/Chicago",
	},
	"KUUU": {
		ICAO:      "KUUU",
		IATA:      "NPT",
		Name:      "Newport State Airport",
		City:      "Newport",
		State:     "Rhode-Island",
		Country:   "US",
		Elevation: 172,
		Latitude:  41.5323982239,
		Longitude: -71.28150177,
		Timezone:  "America/New_York",
	},
	"KUUV": {
		ICAO:      "KUUV",
		Name:      "Sullivan Regional Airport",
		City:      "Sullivan",
		State:     "Missouri",
		Country:   "US",
		Elevation: 933,
		Latitude:  38.2335014343,
		Longitude: -91.1642990112,
		Timezone:  "America/Chicago",
	},
	"KUVA": {
		ICAO:      "KUVA",
		IATA:      "UVA",
		Name:      "Garner Field",
		City:      "Uvalde",
		State:     "Florida",
		Country:   "US",
		Elevation: 942,
		Latitude:  29.2112998962,
		Longitude: -99.743598938,
		Timezone:  "America/Chicago",
	},
	"KUWL": {
		ICAO:      "KUWL",
		Name:      "New Castle Henry Co. Municipal Airport",
		City:      "New Castle",
		State:     "Indiana",
		Country:   "US",
		Elevation: 1088,
		Latitude:  39.87590027,
		Longitude: -85.32649994,
		Timezone:  "America/Indiana/Indianapolis",
	},
	"KUXL": {
		ICAO:      "KUXL",
		Name:      "Southland Field",
		City:      "Sulphur",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 10,
		Latitude:  30.13139915,
		Longitude: -93.37609863,
		Timezone:  "America/Chicago",
	},
	"KUYF": {
		ICAO:      "KUYF",
		Name:      "Madison County Airport",
		City:      "London",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1082,
		Latitude:  39.93270111,
		Longitude: -83.46199799,
		Timezone:  "America/New_York",
	},
	"KUZA": {
		ICAO:      "KUZA",
		IATA:      "RKH",
		Name:      "Rock Hill York Co Bryant field",
		City:      "Rock Hill",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 666,
		Latitude:  34.9878006,
		Longitude: -81.05719757,
		Timezone:  "America/New_York",
	},
	"KVAD": {
		ICAO:      "KVAD",
		IATA:      "VAD",
		Name:      "Moody Air Force Base",
		City:      "Valdosta",
		State:     "Georgia",
		Country:   "US",
		Elevation: 233,
		Latitude:  30.9678001404,
		Longitude: -83.1930007935,
		Timezone:  "America/New_York",
	},
	"KVAY": {
		ICAO:      "KVAY",
		IATA:      "LLY",
		Name:      "South Jersey Regional Airport",
		City:      "Mount Holly",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 53,
		Latitude:  39.9429016113,
		Longitude: -74.845703125,
		Timezone:  "America/New_York",
	},
	"KVBG": {
		ICAO:      "KVBG",
		IATA:      "VBG",
		Name:      "Vandenberg Air Force Base",
		City:      "Lompoc",
		State:     "California",
		Country:   "US",
		Elevation: 369,
		Latitude:  34.7373008728,
		Longitude: -120.5839996338,
		Timezone:  "America/Los_Angeles",
	},
	"KVBT": {
		ICAO:      "KVBT",
		Name:      "Bentonville Municipal-Louise M Thaden Field",
		City:      "Bentonville",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1296,
		Latitude:  36.34569931,
		Longitude: -94.21929932,
		Timezone:  "America/Chicago",
	},
	"KVCB": {
		ICAO:      "KVCB",
		Name:      "Nut Tree Airport",
		City:      "Vacaville",
		State:     "California",
		Country:   "US",
		Elevation: 117,
		Latitude:  38.3768005371,
		Longitude: -121.9619979858,
		Timezone:  "America/Los_Angeles",
	},
	"KVCT": {
		ICAO:      "KVCT",
		IATA:      "VCT",
		Name:      "Victoria Regional Airport",
		City:      "Victoria",
		State:     "Texas",
		Country:   "US",
		Elevation: 115,
		Latitude:  28.8526000977,
		Longitude: -96.9185028076,
		Timezone:  "America/Chicago",
	},
	"KVCV": {
		ICAO:      "KVCV",
		IATA:      "VCV",
		Name:      "Southern California Logistics Airport",
		City:      "Victorville",
		State:     "California",
		Country:   "US",
		Elevation: 2885,
		Latitude:  34.5974998474,
		Longitude: -117.3830032349,
		Timezone:  "America/Los_Angeles",
	},
	"KVDF": {
		ICAO:      "KVDF",
		Name:      "Tampa Executive Airport",
		City:      "Tampa",
		State:     "Florida",
		Country:   "US",
		Elevation: 22,
		Latitude:  28.013999939,
		Longitude: -82.3452987671,
		Timezone:  "America/New_York",
	},
	"KVDI": {
		ICAO:      "KVDI",
		IATA:      "VDI",
		Name:      "Vidalia Regional Airport",
		City:      "Vidalia",
		State:     "Georgia",
		Country:   "US",
		Elevation: 275,
		Latitude:  32.1926994324,
		Longitude: -82.3712005615,
		Timezone:  "America/New_York",
	},
	"KVEL": {
		ICAO:      "KVEL",
		IATA:      "VEL",
		Name:      "Vernal Regional Airport",
		City:      "Vernal",
		State:     "Utah",
		Country:   "US",
		Elevation: 5278,
		Latitude:  40.4408989,
		Longitude: -109.5100021,
		Timezone:  "America/Denver",
	},
	"KVER": {
		ICAO:      "KVER",
		Name:      "Jesse Viertel Memorial Airport",
		City:      "Boonville",
		State:     "Missouri",
		Country:   "US",
		Elevation: 715,
		Latitude:  38.9467010498,
		Longitude: -92.6827011108,
		Timezone:  "America/Chicago",
	},
	"KVES": {
		ICAO:      "KVES",
		Name:      "Darke County Airport",
		City:      "Versailles",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1007,
		Latitude:  40.2043991089,
		Longitude: -84.5318984985,
		Timezone:  "America/New_York",
	},
	"KVGC": {
		ICAO:      "KVGC",
		Name:      "Hamilton Municipal Airport",
		City:      "Hamilton",
		State:     "New-York",
		Country:   "US",
		Elevation: 1137,
		Latitude:  42.84379959,
		Longitude: -75.56140137,
		Timezone:  "America/New_York",
	},
	"KVGT": {
		ICAO:      "KVGT",
		IATA:      "VGT",
		Name:      "North Las Vegas Airport",
		City:      "Las Vegas",
		State:     "Nevada",
		Country:   "US",
		Elevation: 2205,
		Latitude:  36.2107009888,
		Longitude: -115.1940002441,
		Timezone:  "America/Los_Angeles",
	},
	"KVHN": {
		ICAO:      "KVHN",
		IATA:      "VHN",
		Name:      "Culberson County Airport",
		City:      "Van Horn",
		State:     "Texas",
		Country:   "US",
		Elevation: 3957,
		Latitude:  31.057800293,
		Longitude: -104.783996582,
		Timezone:  "America/Chicago",
	},
	"KVIH": {
		ICAO:      "KVIH",
		IATA:      "VIH",
		Name:      "Rolla National Airport",
		City:      "Rolla/Vichy",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1148,
		Latitude:  38.1273994446,
		Longitude: -91.7695007324,
		Timezone:  "America/Chicago",
	},
	"KVIQ": {
		ICAO:      "KVIQ",
		Name:      "Neillsville Municipal Airport",
		City:      "Neillsville",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1237,
		Latitude:  44.5581016541,
		Longitude: -90.5121994019,
		Timezone:  "America/Chicago",
	},
	"KVIS": {
		ICAO:      "KVIS",
		IATA:      "VIS",
		Name:      "Visalia Municipal Airport",
		City:      "Visalia",
		State:     "California",
		Country:   "US",
		Elevation: 295,
		Latitude:  36.3186988831,
		Longitude: -119.3929977417,
		Timezone:  "America/Los_Angeles",
	},
	"KVJI": {
		ICAO:      "KVJI",
		IATA:      "VJI",
		Name:      "Virginia Highlands Airport",
		City:      "Abingdon",
		State:     "Virginia",
		Country:   "US",
		Elevation: 2087,
		Latitude:  36.6870994568,
		Longitude: -82.0333023071,
		Timezone:  "America/New_York",
	},
	"KVKS": {
		ICAO:      "KVKS",
		IATA:      "VKS",
		Name:      "Vicksburg Municipal Airport",
		City:      "Vicksburg",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 106,
		Latitude:  32.2392997742,
		Longitude: -90.9283981323,
		Timezone:  "America/Chicago",
	},
	"KVKX": {
		ICAO:      "KVKX",
		IATA:      "VKX",
		Name:      "Potomac Airfield",
		City:      "Friendly",
		State:     "Maryland",
		Country:   "US",
		Elevation: 118,
		Latitude:  38.74760056,
		Longitude: -76.9571991,
		Timezone:  "America/New_York",
	},
	"KVLA": {
		ICAO:      "KVLA",
		IATA:      "VLA",
		Name:      "Vandalia Municipal Airport",
		City:      "Vandalia",
		State:     "Illinois",
		Country:   "US",
		Elevation: 537,
		Latitude:  38.9915008545,
		Longitude: -89.1661987305,
		Timezone:  "America/Chicago",
	},
	"KVLD": {
		ICAO:      "KVLD",
		IATA:      "VLD",
		Name:      "Valdosta Regional Airport",
		City:      "Valdosta",
		State:     "Georgia",
		Country:   "US",
		Elevation: 203,
		Latitude:  30.7824993134,
		Longitude: -83.2767028809,
		Timezone:  "America/New_York",
	},
	"KVLL": {
		ICAO:      "KVLL",
		Name:      "Oakland Troy Airport",
		City:      "Troy",
		State:     "Michigan",
		Country:   "US",
		Elevation: 727,
		Latitude:  42.54290009,
		Longitude: -83.17790222,
		Timezone:  "America/Detroit",
	},
	"KVMR": {
		ICAO:      "KVMR",
		Name:      "Harold Davidson Field",
		City:      "Vermillion",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1146,
		Latitude:  42.76530075,
		Longitude: -96.93430328,
		Timezone:  "America/Chicago",
	},
	"KVNC": {
		ICAO:      "KVNC",
		IATA:      "VNC",
		Name:      "Venice Municipal Airport",
		City:      "Venice",
		State:     "Florida",
		Country:   "US",
		Elevation: 18,
		Latitude:  27.0715999603,
		Longitude: -82.4402999878,
		Timezone:  "America/New_York",
	},
	"KVNW": {
		ICAO:      "KVNW",
		Name:      "Van Wert County Airport",
		City:      "Van Wert",
		State:     "Ohio",
		Country:   "US",
		Elevation: 785,
		Latitude:  40.8647003174,
		Longitude: -84.6093978882,
		Timezone:  "America/New_York",
	},
	"KVNY": {
		ICAO:      "KVNY",
		IATA:      "VNY",
		Name:      "Van Nuys Airport",
		City:      "Van Nuys",
		State:     "California",
		Country:   "US",
		Elevation: 802,
		Latitude:  34.2098007202,
		Longitude: -118.4899978638,
		Timezone:  "America/Los_Angeles",
	},
	"KVOK": {
		ICAO:      "KVOK",
		IATA:      "VOK",
		Name:      "Volk Field",
		City:      "Camp Douglas",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 912,
		Latitude:  43.938999176,
		Longitude: -90.25340271,
		Timezone:  "America/Chicago",
	},
	"KVPC": {
		ICAO:      "KVPC",
		Name:      "Cartersville Airport",
		City:      "Cartersville",
		State:     "Georgia",
		Country:   "US",
		Elevation: 759,
		Latitude:  34.1231002808,
		Longitude: -84.8487014771,
		Timezone:  "America/New_York",
	},
	"KVPS": {
		ICAO:      "KVPS",
		IATA:      "VPS",
		Name:      "Eglin Air Force Base",
		City:      "Valparaiso",
		State:     "Florida",
		Country:   "US",
		Elevation: 87,
		Latitude:  30.4832000732,
		Longitude: -86.5253982544,
		Timezone:  "America/Chicago",
	},
	"KVPZ": {
		ICAO:      "KVPZ",
		IATA:      "VPZ",
		Name:      "Porter County Municipal Airport",
		City:      "Valparaiso",
		State:     "Indiana",
		Country:   "US",
		Elevation: 770,
		Latitude:  41.45399857,
		Longitude: -87.00710297,
		Timezone:  "America/Chicago",
	},
	"KVQQ": {
		ICAO:      "KVQQ",
		IATA:      "VQQ",
		Name:      "Cecil Field",
		City:      "Jacksonville",
		State:     "Florida",
		Country:   "US",
		Elevation: 81,
		Latitude:  30.2187004089,
		Longitude: -81.876701355,
		Timezone:  "America/New_York",
	},
	"KVRB": {
		ICAO:      "KVRB",
		IATA:      "VRB",
		Name:      "Vero Beach Municipal Airport",
		City:      "Vero Beach",
		State:     "Florida",
		Country:   "US",
		Elevation: 24,
		Latitude:  27.6555995941,
		Longitude: -80.4179000854,
		Timezone:  "America/New_York",
	},
	"KVSF": {
		ICAO:      "KVSF",
		IATA:      "VSF",
		Name:      "Hartness State (Springfield) Airport",
		City:      "Springfield",
		State:     "Vermont",
		Country:   "US",
		Elevation: 577,
		Latitude:  43.34360123,
		Longitude: -72.51730347,
		Timezone:  "America/New_York",
	},
	"KVTA": {
		ICAO:      "KVTA",
		Name:      "Newark Heath Airport",
		City:      "Newark",
		State:     "Ohio",
		Country:   "US",
		Elevation: 884,
		Latitude:  40.0247001648,
		Longitude: -82.4617996216,
		Timezone:  "America/New_York",
	},
	"KVTI": {
		ICAO:      "KVTI",
		Name:      "Vinton Veterans Memorial Airpark",
		City:      "Vinton",
		State:     "Iowa",
		Country:   "US",
		Elevation: 842,
		Latitude:  42.21860123,
		Longitude: -92.02590179,
		Timezone:  "America/Chicago",
	},
	"KVTN": {
		ICAO:      "KVTN",
		IATA:      "VTN",
		Name:      "Miller Field",
		City:      "Valentine",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2596,
		Latitude:  42.85779953,
		Longitude: -100.5479965,
		Timezone:  "America/Chicago",
	},
	"KVUJ": {
		ICAO:      "KVUJ",
		Name:      "Stanly County Airport",
		City:      "Albemarle",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 609,
		Latitude:  35.4166984558,
		Longitude: -80.1508026123,
		Timezone:  "America/New_York",
	},
	"KVUO": {
		ICAO:      "KVUO",
		Name:      "Pearson Field",
		City:      "Vancouver",
		State:     "Washington",
		Country:   "US",
		Elevation: 25,
		Latitude:  45.6204986572,
		Longitude: -122.65599823,
		Timezone:  "America/Los_Angeles",
	},
	"KVVS": {
		ICAO:      "KVVS",
		Name:      "Joseph A. Hardy Connellsville Airport",
		City:      "Connellsville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1267,
		Latitude:  39.95920181,
		Longitude: -79.65709686,
		Timezone:  "America/New_York",
	},
	"KVVV": {
		ICAO:      "KVVV",
		Name:      "Ortonville Municipal Martinson Field",
		City:      "Ortonville",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1100,
		Latitude:  45.30569839,
		Longitude: -96.42440033,
		Timezone:  "America/Chicago",
	},
	"KVWU": {
		ICAO:      "KVWU",
		Name:      "Waskish Municipal Airport",
		City:      "Waskish",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1181,
		Latitude:  48.1540985107,
		Longitude: -94.5168991089,
		Timezone:  "America/Chicago",
	},
	"KVYS": {
		ICAO:      "KVYS",
		IATA:      "VYS",
		Name:      "Illinois Valley Regional-Walter A Duncan field",
		City:      "Peru",
		State:     "Illinois",
		Country:   "US",
		Elevation: 654,
		Latitude:  41.35189819,
		Longitude: -89.15309906,
		Timezone:  "America/Chicago",
	},
	"KW01": {
		ICAO:      "KW01",
		Name:      "Tonasket Municipal Airport",
		City:      "Tonasket",
		State:     "Washington",
		Country:   "US",
		Elevation: 1311,
		Latitude:  48.7248683333,
		Longitude: -119.465634722,
		Timezone:  "America/Los_Angeles",
	},
	"KW03": {
		ICAO:      "KW03",
		Name:      "Wilson Industrial Air Center Airport",
		City:      "Wilson",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 161,
		Latitude:  35.770401001,
		Longitude: -77.9698028564,
		Timezone:  "America/New_York",
	},
	"KW05": {
		ICAO:      "KW05",
		Name:      "Gettysburg Regional Airport",
		City:      "Gettysburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 590,
		Latitude:  39.84090042,
		Longitude: -77.27420044,
		Timezone:  "America/New_York",
	},
	"KW22": {
		ICAO:      "KW22",
		Name:      "Upshur County Regional Airport",
		City:      "Buckhannon",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 1635,
		Latitude:  39.00049973,
		Longitude: -80.2736969,
		Timezone:  "America/New_York",
	},
	"KW28": {
		ICAO:      "KW28",
		Name:      "Sequim Valley Airport",
		City:      "Sequim",
		State:     "Washington",
		Country:   "US",
		Elevation: 144,
		Latitude:  48.0980987549,
		Longitude: -123.18699646,
		Timezone:  "America/Los_Angeles",
	},
	"KW31": {
		ICAO:      "KW31",
		Name:      "Lunenburg County Airport",
		City:      "Kenbridge",
		State:     "Virginia",
		Country:   "US",
		Elevation: 526,
		Latitude:  36.9602012634,
		Longitude: -78.1849975586,
		Timezone:  "America/New_York",
	},
	"KW32": {
		ICAO:      "KW32",
		Name:      "Washington Executive Hyde Field",
		City:      "Clinton",
		State:     "Maryland",
		Country:   "US",
		Elevation: 249,
		Latitude:  38.748298645,
		Longitude: -76.932800293,
		Timezone:  "America/New_York",
	},
	"KW35": {
		ICAO:      "KW35",
		Name:      "Potomac Airpark",
		City:      "Berkeley Springs",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 412,
		Latitude:  39.6926002502,
		Longitude: -78.1660995483,
		Timezone:  "America/New_York",
	},
	"KW38": {
		ICAO:      "KW38",
		Name:      "Williamsburg Whitley County Airport",
		City:      "Williamsburg",
		State:     "Kentucky",
		Country:   "US",
		Elevation: 1178,
		Latitude:  36.7949981689,
		Longitude: -84.1995010376,
		Timezone:  "America/New_York",
	},
	"KW40": {
		ICAO:      "KW40",
		Name:      "Mount Olive Municipal Airport",
		City:      "Mount Olive",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 168,
		Latitude:  35.22219849,
		Longitude: -78.03780365,
		Timezone:  "America/New_York",
	},
	"KW41": {
		ICAO:      "KW41",
		Name:      "Crisfield Municipal Airport",
		City:      "Crisfield",
		State:     "Maryland",
		Country:   "US",
		Elevation: 4,
		Latitude:  38.0176010132,
		Longitude: -75.827796936,
		Timezone:  "America/New_York",
	},
	"KW43": {
		ICAO:      "KW43",
		Name:      "Hulett Municipal Airport",
		City:      "Hulett",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4264,
		Latitude:  44.6628990173,
		Longitude: -104.5680007935,
		Timezone:  "America/Denver",
	},
	"KW63": {
		ICAO:      "KW63",
		Name:      "Lake Country Regional Airport",
		City:      "Clarksville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 421,
		Latitude:  36.5957984924,
		Longitude: -78.5600967407,
		Timezone:  "America/New_York",
	},
	"KW78": {
		ICAO:      "KW78",
		Name:      "William M Tuck Airport",
		City:      "South Boston",
		State:     "Virginia",
		Country:   "US",
		Elevation: 420,
		Latitude:  36.7099990845,
		Longitude: -78.8479995728,
		Timezone:  "America/New_York",
	},
	"KW81": {
		ICAO:      "KW81",
		Name:      "Crewe Municipal Airport",
		City:      "Crewe",
		State:     "Virginia",
		Country:   "US",
		Elevation: 420,
		Latitude:  37.1809997559,
		Longitude: -78.0982971191,
		Timezone:  "America/New_York",
	},
	"KW90": {
		ICAO:      "KW90",
		Name:      "New London Airport",
		City:      "Forest",
		State:     "Virginia",
		Country:   "US",
		Elevation: 849,
		Latitude:  37.2718009949,
		Longitude: -79.335899353,
		Timezone:  "America/New_York",
	},
	"KW91": {
		ICAO:      "KW91",
		Name:      "Smith Mountain Lake Airport",
		City:      "Moneta",
		State:     "Virginia",
		Country:   "US",
		Elevation: 892,
		Latitude:  37.1077003479,
		Longitude: -79.5924987793,
		Timezone:  "America/New_York",
	},
	"KW94": {
		ICAO:      "KW94",
		Name:      "Camp Peary Landing Strip",
		City:      "Williamsburg",
		State:     "Virginia",
		Country:   "US",
		Elevation: 41,
		Latitude:  37.30559921,
		Longitude: -76.63749695,
		Timezone:  "America/New_York",
	},
	"KW95": {
		ICAO:      "KW95",
		Name:      "Ocracoke Island Airport",
		City:      "Ocracoke",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 5,
		Latitude:  35.1012001038,
		Longitude: -75.966003418,
		Timezone:  "America/New_York",
	},
	"KW96": {
		ICAO:      "KW96",
		Name:      "New Kent County Airport",
		City:      "Quinton",
		State:     "Virginia",
		Country:   "US",
		Elevation: 123,
		Latitude:  37.503200531,
		Longitude: -77.12550354,
		Timezone:  "America/New_York",
	},
	"KWAL": {
		ICAO:      "KWAL",
		IATA:      "WAL",
		Name:      "Wallops Flight Facility Airport",
		City:      "Wallops Island",
		State:     "Virginia",
		Country:   "US",
		Elevation: 40,
		Latitude:  37.9402008057,
		Longitude: -75.4664001465,
		Timezone:  "America/New_York",
	},
	"KWAY": {
		ICAO:      "KWAY",
		IATA:      "WAY",
		Name:      "Greene County Airport",
		City:      "Waynesburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1069,
		Latitude:  39.90010071,
		Longitude: -80.13310242,
		Timezone:  "America/New_York",
	},
	"KWBW": {
		ICAO:      "KWBW",
		IATA:      "WBW",
		Name:      "Wilkes Barre Wyoming Valley Airport",
		City:      "Wilkes-Barre",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 545,
		Latitude:  41.2971992493,
		Longitude: -75.8511962891,
		Timezone:  "America/New_York",
	},
	"KWDG": {
		ICAO:      "KWDG",
		IATA:      "WDG",
		Name:      "Enid Woodring Regional Airport",
		City:      "Enid",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1167,
		Latitude:  36.3791999817,
		Longitude: -97.7910995483,
		Timezone:  "America/Chicago",
	},
	"KWDR": {
		ICAO:      "KWDR",
		IATA:      "WDR",
		Name:      "Barrow County Airport",
		City:      "Winder",
		State:     "Georgia",
		Country:   "US",
		Elevation: 943,
		Latitude:  33.98289871,
		Longitude: -83.66739655,
		Timezone:  "America/New_York",
	},
	"KWHP": {
		ICAO:      "KWHP",
		IATA:      "WHP",
		Name:      "Whiteman Airport",
		City:      "Los Angeles",
		State:     "California",
		Country:   "US",
		Elevation: 1003,
		Latitude:  34.2593002319,
		Longitude: -118.4130020142,
		Timezone:  "America/Los_Angeles",
	},
	"KWJF": {
		ICAO:      "KWJF",
		IATA:      "WJF",
		Name:      "General WM J Fox Airfield",
		City:      "Lancaster",
		State:     "California",
		Country:   "US",
		Elevation: 2351,
		Latitude:  34.74110031,
		Longitude: -118.2190018,
		Timezone:  "America/Los_Angeles",
	},
	"KWLD": {
		ICAO:      "KWLD",
		IATA:      "WLD",
		Name:      "Strother Field",
		City:      "Winfield/Arkansas City",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1160,
		Latitude:  37.168598175,
		Longitude: -97.0375976563,
		Timezone:  "America/Chicago",
	},
	"KWLW": {
		ICAO:      "KWLW",
		IATA:      "WLW",
		Name:      "Willows Glenn County Airport",
		City:      "Willows",
		State:     "California",
		Country:   "US",
		Elevation: 141,
		Latitude:  39.51639938,
		Longitude: -122.2180023,
		Timezone:  "America/Los_Angeles",
	},
	"KWMC": {
		ICAO:      "KWMC",
		IATA:      "WMC",
		Name:      "Winnemucca Municipal Airport",
		City:      "Winnemucca",
		State:     "Nevada",
		Country:   "US",
		Elevation: 4308,
		Latitude:  40.8965988159,
		Longitude: -117.8059997559,
		Timezone:  "America/Los_Angeles",
	},
	"KWRB": {
		ICAO:      "KWRB",
		IATA:      "WRB",
		Name:      "Robins Air Force Base",
		City:      "Warner Robins",
		State:     "Georgia",
		Country:   "US",
		Elevation: 294,
		Latitude:  32.6400985718,
		Longitude: -83.5919036865,
		Timezone:  "America/New_York",
	},
	"KWRI": {
		ICAO:      "KWRI",
		IATA:      "WRI",
		Name:      "Mc Guire Air Force Base",
		City:      "Wrightstown",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 131,
		Latitude:  40.0155983,
		Longitude: -74.59169769,
		Timezone:  "America/New_York",
	},
	"KWRL": {
		ICAO:      "KWRL",
		IATA:      "WRL",
		Name:      "Worland Municipal Airport",
		City:      "Worland",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4227,
		Latitude:  43.9656982422,
		Longitude: -107.9509963989,
		Timezone:  "America/Denver",
	},
	"KWSD": {
		ICAO:      "KWSD",
		IATA:      "WSD",
		Name:      "Condron Army Air Field",
		City:      "White Sands",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 3934,
		Latitude:  32.34149933,
		Longitude: -106.4029999,
		Timezone:  "America/Denver",
	},
	"KWST": {
		ICAO:      "KWST",
		IATA:      "WST",
		Name:      "Westerly State Airport",
		City:      "Westerly",
		State:     "Rhode-Island",
		Country:   "US",
		Elevation: 81,
		Latitude:  41.3496017456,
		Longitude: -71.8033981323,
		Timezone:  "America/New_York",
	},
	"KWVI": {
		ICAO:      "KWVI",
		IATA:      "WVI",
		Name:      "Watsonville Municipal Airport",
		City:      "Watsonville",
		State:     "California",
		Country:   "US",
		Elevation: 163,
		Latitude:  36.9356994629,
		Longitude: -121.7900009155,
		Timezone:  "America/Los_Angeles",
	},
	"KWVL": {
		ICAO:      "KWVL",
		IATA:      "WVL",
		Name:      "Waterville Robert Lafleur Airport",
		City:      "Waterville",
		State:     "Maine",
		Country:   "US",
		Elevation: 333,
		Latitude:  44.5331993103,
		Longitude: -69.6754989624,
		Timezone:  "America/New_York",
	},
	"KWWD": {
		ICAO:      "KWWD",
		IATA:      "WWD",
		Name:      "Cape May County Airport",
		City:      "Wildwood",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 23,
		Latitude:  39.0084991455,
		Longitude: -74.9083023071,
		Timezone:  "America/New_York",
	},
	"KWWR": {
		ICAO:      "KWWR",
		IATA:      "WWR",
		Name:      "West Woodward Airport",
		City:      "Woodward",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 2189,
		Latitude:  36.438,
		Longitude: -99.5226667,
		Timezone:  "America/Chicago",
	},
	"KWYS": {
		ICAO:      "KWYS",
		IATA:      "WYS",
		Name:      "Yellowstone Airport",
		City:      "West Yellowstone",
		State:     "Montana",
		Country:   "US",
		Elevation: 6649,
		Latitude:  44.68840027,
		Longitude: -111.1179962,
		Timezone:  "America/Denver",
	},
	"KX04": {
		ICAO:      "KX04",
		Name:      "Orlando Apopka Airport",
		City:      "Apopka",
		State:     "Florida",
		Country:   "US",
		Elevation: 143,
		Latitude:  28.7071990967,
		Longitude: -81.581703186,
		Timezone:  "America/New_York",
	},
	"KX05": {
		ICAO:      "KX05",
		Name:      "Pilot Country Airport",
		City:      "Brooksville",
		State:     "Florida",
		Country:   "US",
		Elevation: 80,
		Latitude:  28.3302993774,
		Longitude: -82.4964981079,
		Timezone:  "America/New_York",
	},
	"KX06": {
		ICAO:      "KX06",
		Name:      "Arcadia Municipal Airport",
		City:      "Arcadia",
		State:     "Florida",
		Country:   "US",
		Elevation: 60,
		Latitude:  27.1919994354,
		Longitude: -81.8373031616,
		Timezone:  "America/New_York",
	},
	"KX07": {
		ICAO:      "KX07",
		Name:      "Lake Wales Municipal Airport",
		City:      "Lake Wales",
		State:     "Florida",
		Country:   "US",
		Elevation: 127,
		Latitude:  27.8938007355,
		Longitude: -81.6203994751,
		Timezone:  "America/New_York",
	},
	"KX10": {
		ICAO:      "KX10",
		Name:      "Belle Glade State Municipal Airport",
		City:      "Belle Glade",
		State:     "Florida",
		Country:   "US",
		Elevation: 14,
		Latitude:  26.7008991241,
		Longitude: -80.6623001099,
		Timezone:  "America/New_York",
	},
	"KX13": {
		ICAO:      "KX13",
		Name:      "Carrabelle Thompson Airport",
		City:      "Carrabelle",
		State:     "Florida",
		Country:   "US",
		Elevation: 20,
		Latitude:  29.8421993256,
		Longitude: -84.7009963989,
		Timezone:  "America/New_York",
	},
	"KX21": {
		ICAO:      "KX21",
		Name:      "Arthur Dunn Air Park",
		City:      "Titusville",
		State:     "Florida",
		Country:   "US",
		Elevation: 30,
		Latitude:  28.6222991943,
		Longitude: -80.8357009888,
		Timezone:  "America/New_York",
	},
	"KX26": {
		ICAO:      "KX26",
		Name:      "Sebastian Municipal Airport",
		City:      "Sebastian",
		State:     "Florida",
		Country:   "US",
		Elevation: 21,
		Latitude:  27.8125991821,
		Longitude: -80.4959030151,
		Timezone:  "America/New_York",
	},
	"KX39": {
		ICAO:      "KX39",
		Name:      "Tampa North Aero Park Airport",
		City:      "Tampa",
		State:     "Florida",
		Country:   "US",
		Elevation: 68,
		Latitude:  28.2213001251,
		Longitude: -82.37449646,
		Timezone:  "America/New_York",
	},
	"KX43": {
		ICAO:      "KX43",
		Name:      "Sunray Airport",
		City:      "Sunray",
		State:     "Texas",
		Country:   "US",
		Elevation: 3507,
		Latitude:  36.0292015076,
		Longitude: -101.8290023804,
		Timezone:  "America/Chicago",
	},
	"KX49": {
		ICAO:      "KX49",
		Name:      "South Lakeland Airport",
		City:      "Lakeland",
		State:     "Florida",
		Country:   "US",
		Elevation: 110,
		Latitude:  27.9333992004,
		Longitude: -82.0439987183,
		Timezone:  "America/New_York",
	},
	"KX50": {
		ICAO:      "KX50",
		Name:      "Massey Ranch Airpark",
		City:      "New Smyrna Beach",
		State:     "Florida",
		Country:   "US",
		Elevation: 12,
		Latitude:  28.9789009094,
		Longitude: -80.9251022339,
		Timezone:  "America/New_York",
	},
	"KX54": {
		ICAO:      "KX54",
		Name:      "Benger Air Park",
		City:      "Friona",
		State:     "Texas",
		Country:   "US",
		Elevation: 4003,
		Latitude:  34.6542015076,
		Longitude: -102.6920013428,
		Timezone:  "America/Chicago",
	},
	"KX58": {
		ICAO:      "KX58",
		Name:      "Indiantown Airport",
		City:      "Indiantown",
		State:     "Florida",
		Country:   "US",
		Elevation: 30,
		Latitude:  27.0363998413,
		Longitude: -80.4401016235,
		Timezone:  "America/New_York",
	},
	"KX59": {
		ICAO:      "KX59",
		Name:      "Valkaria Airport",
		City:      "Malabar",
		State:     "Florida",
		Country:   "US",
		Elevation: 26,
		Latitude:  27.96199989,
		Longitude: -80.55979919,
		Timezone:  "America/New_York",
	},
	"KX60": {
		ICAO:      "KX60",
		Name:      "Williston Municipal Airport",
		City:      "Williston",
		State:     "Florida",
		Country:   "US",
		Elevation: 76,
		Latitude:  29.3542003632,
		Longitude: -82.4729003906,
		Timezone:  "America/New_York",
	},
	"KXBP": {
		ICAO:      "KXBP",
		Name:      "Bridgeport Municipal Airport",
		City:      "Bridgeport",
		State:     "Texas",
		Country:   "US",
		Elevation: 852,
		Latitude:  33.1753005981,
		Longitude: -97.8283996582,
		Timezone:  "America/Chicago",
	},
	"KXFL": {
		ICAO:      "KXFL",
		Name:      "Flagler County Airport",
		City:      "Palm Coast",
		State:     "Florida",
		Country:   "US",
		Elevation: 33,
		Latitude:  29.4673996,
		Longitude: -81.20629883,
		Timezone:  "America/New_York",
	},
	"KXLL": {
		ICAO:      "KXLL",
		Name:      "Allentown Queen City Municipal Airport",
		City:      "Allentown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 399,
		Latitude:  40.5703010559,
		Longitude: -75.4882965088,
		Timezone:  "America/New_York",
	},
	"KXMR": {
		ICAO:      "KXMR",
		Name:      "Cape Canaveral AFS Skid Strip",
		City:      "Cocoa Beach",
		State:     "Florida",
		Country:   "US",
		Elevation: 10,
		Latitude:  28.4675998688,
		Longitude: -80.5665969849,
		Timezone:  "America/New_York",
	},
	"KXNA": {
		ICAO:      "KXNA",
		IATA:      "XNA",
		Name:      "Northwest Arkansas Regional Airport",
		City:      "Fayetteville/Springdale/",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 1287,
		Latitude:  36.2818984985,
		Longitude: -94.3068008423,
		Timezone:  "America/Chicago",
	},
	"KXNO": {
		ICAO:      "KXNO",
		Name:      "North Af Aux Airport",
		City:      "North",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 321,
		Latitude:  33.6150016785,
		Longitude: -81.0843963623,
		Timezone:  "America/New_York",
	},
	"KXSA": {
		ICAO:      "KXSA",
		Name:      "Tappahannock-Essex County Airport",
		City:      "Tappahannock",
		State:     "Virginia",
		Country:   "US",
		Elevation: 135,
		Latitude:  37.8596000671,
		Longitude: -76.8940963745,
		Timezone:  "America/New_York",
	},
	"KXTA": {
		ICAO:      "KXTA",
		Name:      "Homey (Area 51) Airport",
		City:      "Groom Lake",
		State:     "Nevada",
		Country:   "US",
		Elevation: 4462,
		Latitude:  37.2350006104,
		Longitude: -115.8109970093,
		Timezone:  "America/Los_Angeles",
	},
	"KXVG": {
		ICAO:      "KXVG",
		Name:      "Longville Municipal Airport",
		City:      "Longville",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1334,
		Latitude:  46.9902000427,
		Longitude: -94.2040023804,
		Timezone:  "America/Chicago",
	},
	"KY03": {
		ICAO:      "KY03",
		Name:      "Springfield Municipal Airport",
		City:      "Springfield",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1324,
		Latitude:  42.8800010681,
		Longitude: -97.9011993408,
		Timezone:  "America/Chicago",
	},
	"KY14": {
		ICAO:      "KY14",
		Name:      "Marv Skie Lincoln County Airport",
		City:      "Tea",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1515,
		Latitude:  43.4575004578,
		Longitude: -96.8020019531,
		Timezone:  "America/Chicago",
	},
	"KY19": {
		ICAO:      "KY19",
		Name:      "Mandan Municipal Airport",
		City:      "Mandan",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1944,
		Latitude:  46.7681999207,
		Longitude: -100.8939971924,
		Timezone:  "America/North_Dakota/New_Salem",
	},
	"KY23": {
		ICAO:      "KY23",
		Name:      "Chetek Municipal Southworth Airport",
		City:      "Chetek",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1055,
		Latitude:  45.30619812,
		Longitude: -91.63619995,
		Timezone:  "America/Chicago",
	},
	"KY27": {
		ICAO:      "KY27",
		Name:      "Standing Rock Airport",
		City:      "Fort Yates",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1633,
		Latitude:  46.0663986206,
		Longitude: -100.6350021362,
		Timezone:  "America/Chicago",
	},
	"KY31": {
		ICAO:      "KY31",
		Name:      "West Branch Community Airport",
		City:      "West Branch",
		State:     "Michigan",
		Country:   "US",
		Elevation: 882,
		Latitude:  44.2448005676,
		Longitude: -84.1798019409,
		Timezone:  "America/Detroit",
	},
	"KY37": {
		ICAO:      "KY37",
		Name:      "Park River W C Skjerven Field",
		City:      "Park River",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1104,
		Latitude:  48.39400101,
		Longitude: -97.78079987,
		Timezone:  "America/Chicago",
	},
	"KY43": {
		ICAO:      "KY43",
		Name:      "Anita Municipal Kevin Burke Memorial Field",
		City:      "Anita",
		State:     "Iowa",
		Country:   "US",
		Elevation: 1251,
		Latitude:  41.4402999878,
		Longitude: -94.7696990967,
		Timezone:  "America/Chicago",
	},
	"KY47": {
		ICAO:      "KY47",
		Name:      "Oakland Southwest Airport",
		City:      "New Hudson",
		State:     "Michigan",
		Country:   "US",
		Elevation: 926,
		Latitude:  42.5031013489,
		Longitude: -83.6237030029,
		Timezone:  "America/Detroit",
	},
	"KY50": {
		ICAO:      "KY50",
		Name:      "Wautoma Municipal Airport",
		City:      "Wautoma",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 859,
		Latitude:  44.0415992737,
		Longitude: -89.3044967651,
		Timezone:  "America/Chicago",
	},
	"KY51": {
		ICAO:      "KY51",
		Name:      "Viroqua Municipal Airport",
		City:      "Viroqua",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1292,
		Latitude:  43.5793991089,
		Longitude: -90.9131011963,
		Timezone:  "America/Chicago",
	},
	"KY55": {
		ICAO:      "KY55",
		Name:      "Crandon Municipal Airport",
		City:      "Crandon",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1646,
		Latitude:  45.5166015625,
		Longitude: -88.9334030151,
		Timezone:  "America/Chicago",
	},
	"KY65": {
		ICAO:      "KY65",
		Name:      "Calvin Campbell Municipal Airport",
		City:      "Indian River",
		State:     "Michigan",
		Country:   "US",
		Elevation: 602,
		Latitude:  45.4082984924,
		Longitude: -84.5999984741,
		Timezone:  "America/Detroit",
	},
	"KY70": {
		ICAO:      "KY70",
		Name:      "Ionia County Airport",
		City:      "Ionia",
		State:     "Michigan",
		Country:   "US",
		Elevation: 818,
		Latitude:  42.9379997253,
		Longitude: -85.0605010986,
		Timezone:  "America/Detroit",
	},
	"KY72": {
		ICAO:      "KY72",
		Name:      "Bloyer Field",
		City:      "Tomah",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 966,
		Latitude:  43.97499847,
		Longitude: -90.48349762,
		Timezone:  "America/Chicago",
	},
	"KY74": {
		ICAO:      "KY74",
		Name:      "Parshall Hankins Airport",
		City:      "Parshall",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 2031,
		Latitude:  47.9364013672,
		Longitude: -102.141998291,
		Timezone:  "America/Chicago",
	},
	"KY83": {
		ICAO:      "KY83",
		Name:      "Sandusky City Airport",
		City:      "Sandusky",
		State:     "Michigan",
		Country:   "US",
		Elevation: 776,
		Latitude:  43.4542007446,
		Longitude: -82.8494033813,
		Timezone:  "America/Detroit",
	},
	"KY89": {
		ICAO:      "KY89",
		Name:      "Kalkaska City Airport",
		City:      "Kalkaska",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1030,
		Latitude:  44.7252006531,
		Longitude: -85.2031021118,
		Timezone:  "America/Detroit",
	},
	"KY93": {
		ICAO:      "KY93",
		Name:      "Atlanta Municipal Airport",
		City:      "Atlanta",
		State:     "Michigan",
		Country:   "US",
		Elevation: 875,
		Latitude:  45,
		Longitude: -84.1333007813,
		Timezone:  "America/Detroit",
	},
	"KY94": {
		ICAO:      "KY94",
		Name:      "East Jordan City Airport",
		City:      "East Jordan",
		State:     "Michigan",
		Country:   "US",
		Elevation: 642,
		Latitude:  45.1301002502,
		Longitude: -85.1065979004,
		Timezone:  "America/Detroit",
	},
	"KY95": {
		ICAO:      "KY95",
		Name:      "Hillman Airport",
		City:      "Hillman",
		State:     "Michigan",
		Country:   "US",
		Elevation: 850,
		Latitude:  45.0833015442,
		Longitude: -83.9402999878,
		Timezone:  "America/Detroit",
	},
	"KYIP": {
		ICAO:      "KYIP",
		IATA:      "YIP",
		Name:      "Willow Run Airport",
		City:      "Detroit",
		State:     "Michigan",
		Country:   "US",
		Elevation: 716,
		Latitude:  42.23789978,
		Longitude: -83.53040314,
		Timezone:  "America/Detroit",
	},
	"KYKM": {
		ICAO:      "KYKM",
		IATA:      "YKM",
		Name:      "Yakima Air Terminal McAllister Field",
		City:      "Yakima",
		State:     "Washington",
		Country:   "US",
		Elevation: 1099,
		Latitude:  46.56819916,
		Longitude: -120.5439987,
		Timezone:  "America/Los_Angeles",
	},
	"KYKN": {
		ICAO:      "KYKN",
		IATA:      "YKN",
		Name:      "Chan Gurney Municipal Airport",
		City:      "Yankton",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1306,
		Latitude:  42.9166984558,
		Longitude: -97.3859024048,
		Timezone:  "America/Chicago",
	},
	"KYNG": {
		ICAO:      "KYNG",
		IATA:      "YNG",
		Name:      "Youngstown Warren Regional Airport",
		City:      "Youngstown/Warren",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1192,
		Latitude:  41.26070023,
		Longitude: -80.67910004,
		Timezone:  "America/New_York",
	},
	"KZ01": {
		ICAO:      "KZ01",
		Name:      "Eglin Auxiliary Field 6 Airport",
		City:      "Valparaiso",
		State:     "Florida",
		Country:   "US",
		Elevation: 120,
		Latitude:  30.6336994171,
		Longitude: -86.7426986694,
		Timezone:  "America/Chicago",
	},
	"KZ10": {
		ICAO:      "KZ10",
		Name:      "Pacemaker Landing Zone Airport",
		State:     "Washington",
		Country:   "US",
		Elevation: 397,
		Latitude:  47.0217018127,
		Longitude: -122.4459991455,
		Timezone:  "America/Los_Angeles",
	},
	"KZ98": {
		ICAO:      "KZ98",
		Name:      "Ottawa Executive Airport",
		City:      "Zeeland",
		State:     "Michigan",
		Country:   "US",
		Elevation: 740,
		Latitude:  42.817199707,
		Longitude: -85.9281005859,
		Timezone:  "America/Detroit",
	},
	"KZEF": {
		ICAO:      "KZEF",
		Name:      "Elkin Municipal Airport",
		City:      "Elkin",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 1068,
		Latitude:  36.2799987793,
		Longitude: -80.7861022949,
		Timezone:  "America/New_York",
	},
	"KZER": {
		ICAO:      "KZER",
		Name:      "Schuylkill County /Joe Zerbey/ Airport",
		City:      "Pottsville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1729,
		Latitude:  40.70650101,
		Longitude: -76.37310028,
		Timezone:  "America/New_York",
	},
	"KZPH": {
		ICAO:      "KZPH",
		IATA:      "ZPH",
		Name:      "Zephyrhills Municipal Airport",
		City:      "Zephyrhills",
		State:     "Florida",
		Country:   "US",
		Elevation: 90,
		Latitude:  28.2282009125,
		Longitude: -82.1558990479,
		Timezone:  "America/New_York",
	},
	"KZUN": {
		ICAO:      "KZUN",
		Name:      "Black Rock Airport",
		City:      "Zuni Pueblo",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 6454,
		Latitude:  35.0831985474,
		Longitude: -108.7919998169,
		Timezone:  "America/Denver",
	},
	"KZZV": {
		ICAO:      "KZZV",
		IATA:      "ZZV",
		Name:      "Zanesville Municipal Airport",
		City:      "Zanesville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 900,
		Latitude:  39.9444007874,
		Longitude: -81.8920974731,
		Timezone:  "America/New_York",
	},
	"LA01": {
		ICAO:      "LA01",
		Name:      "Millers Flying Service Airport",
		City:      "Basile",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 44,
		Latitude:  30.483499527,
		Longitude: -92.5467987061,
		Timezone:  "America/Chicago",
	},
	"LA03": {
		ICAO:      "LA03",
		Name:      "Daniel Airport",
		City:      "Addis",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 18,
		Latitude:  30.3532009125,
		Longitude: -91.2820968628,
		Timezone:  "America/Chicago",
	},
	"LA04": {
		ICAO:      "LA04",
		Name:      "Cane-Air Airport",
		City:      "Belle Rose",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 17,
		Latitude:  30.0501995087,
		Longitude: -91.0668029785,
		Timezone:  "America/Chicago",
	},
	"LA07": {
		ICAO:      "LA07",
		Name:      "Price Ultralightport",
		City:      "Abita Springs",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 50,
		Latitude:  30.5174999237,
		Longitude: -90.0083007813,
		Timezone:  "America/Chicago",
	},
	"LA13": {
		ICAO:      "LA13",
		Name:      "Shaw Crop Service Airport",
		City:      "Hammond",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 95,
		Latitude:  30.5781993866,
		Longitude: -90.31590271,
		Timezone:  "America/Chicago",
	},
	"LA15": {
		ICAO:      "LA15",
		Name:      "Wilbert Airport",
		City:      "Plaquemine",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 12,
		Latitude:  30.2598991394,
		Longitude: -91.2862014771,
		Timezone:  "America/Chicago",
	},
	"LA16": {
		ICAO:      "LA16",
		Name:      "Castille Field",
		City:      "Breaux Bridge",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 25,
		Latitude:  30.2460002899,
		Longitude: -91.8667984009,
		Timezone:  "America/Chicago",
	},
	"LA17": {
		ICAO:      "LA17",
		Name:      "Pioneer Field Flight Park Ultralightport",
		City:      "Bossier City",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 173,
		Latitude:  32.6017990112,
		Longitude: -93.6940994263,
		Timezone:  "America/Chicago",
	},
	"LA21": {
		ICAO:      "LA21",
		Name:      "Chloe Airport",
		City:      "Lake Charles",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 10,
		Latitude:  30.2548999786,
		Longitude: -93.1352005005,
		Timezone:  "America/Chicago",
	},
	"LA25": {
		ICAO:      "LA25",
		Name:      "Central Farmers Coop Airport",
		City:      "Mamou",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 68,
		Latitude:  30.6973991394,
		Longitude: -92.4140014648,
		Timezone:  "America/Chicago",
	},
	"LA26": {
		ICAO:      "LA26",
		Name:      "Unicorn Airport",
		City:      "Folsom",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 147,
		Latitude:  30.6611003876,
		Longitude: -90.2332992554,
		Timezone:  "America/Chicago",
	},
	"LA30": {
		ICAO:      "LA30",
		Name:      "Phoenix Airport",
		City:      "Rayne",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 31,
		Latitude:  30.2481994629,
		Longitude: -92.2416992188,
		Timezone:  "America/Chicago",
	},
	"LA31": {
		ICAO:      "LA31",
		Name:      "Gotreaux Strip",
		City:      "Lacassine",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 16,
		Latitude:  30.2000007629,
		Longitude: -92.9332962036,
		Timezone:  "America/Chicago",
	},
	"LA32": {
		ICAO:      "LA32",
		Name:      "Lake Air Service Airport",
		City:      "Lake Arthur",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 11,
		Latitude:  30.0834999084,
		Longitude: -92.6836013794,
		Timezone:  "America/Chicago",
	},
	"LA33": {
		ICAO:      "LA33",
		Name:      "Cuba Farm Airport",
		City:      "Monroe",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 80,
		Latitude:  32.576499939,
		Longitude: -92.1137008667,
		Timezone:  "America/Chicago",
	},
	"LA34": {
		ICAO:      "LA34",
		Name:      "Reno Flight Park Airport",
		City:      "Monroe",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 68,
		Latitude:  32.4052009583,
		Longitude: -92.0663986206,
		Timezone:  "America/Chicago",
	},
	"LA35": {
		ICAO:      "LA35",
		Name:      "W & E Air Service Airport",
		City:      "Monterey",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 54,
		Latitude:  31.447599411,
		Longitude: -91.6838989258,
		Timezone:  "America/Chicago",
	},
	"LA36": {
		ICAO:      "LA36",
		Name:      "Whitehall Field Ultralightport",
		City:      "Convent",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 16,
		Latitude:  30.07980537,
		Longitude: -90.8932724,
		Timezone:  "America/Chicago",
	},
	"LA40": {
		ICAO:      "LA40",
		Name:      "F L Braughton Airport",
		City:      "Vidalia",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 50,
		Latitude:  31.3439006805,
		Longitude: -91.5927963257,
		Timezone:  "America/Chicago",
	},
	"LA41": {
		ICAO:      "LA41",
		Name:      "Dupuis Airport",
		City:      "Breaux Bridge",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 20,
		Latitude:  30.315667,
		Longitude: -91.940778,
		Timezone:  "America/Chicago",
	},
	"LA42": {
		ICAO:      "LA42",
		Name:      "Capozzoli Airport",
		City:      "Prairieville",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 24,
		Latitude:  30.3502998352,
		Longitude: -90.922203064,
		Timezone:  "America/Chicago",
	},
	"LA46": {
		ICAO:      "LA46",
		Name:      "Omni Airport",
		City:      "Port Allen",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 24,
		Latitude:  30.5041007996,
		Longitude: -91.3022994995,
		Timezone:  "America/Chicago",
	},
	"LA48": {
		ICAO:      "LA48",
		Name:      "Max Airport",
		City:      "Cottonport",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 55,
		Latitude:  30.9939002991,
		Longitude: -92.0446014404,
		Timezone:  "America/Chicago",
	},
	"LA51": {
		ICAO:      "LA51",
		Name:      "Stuart Airstrip",
		City:      "Coushatta",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 130,
		Latitude:  32.0181999207,
		Longitude: -93.4307022095,
		Timezone:  "America/Chicago",
	},
	"LA52": {
		ICAO:      "LA52",
		Name:      "T & M Ag Aviation Airport",
		City:      "Dixie",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 185,
		Latitude:  32.6987991333,
		Longitude: -93.8237991333,
		Timezone:  "America/Chicago",
	},
	"LA53": {
		ICAO:      "LA53",
		Name:      "Tensas Flying Service Airport",
		City:      "St. Joseph",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 96,
		Latitude:  31.9775,
		Longitude: -91.263889,
		Timezone:  "America/Chicago",
	},
	"LA54": {
		ICAO:      "LA54",
		Name:      "Gilliam Airport",
		City:      "Gilliam",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 194,
		Latitude:  32.837600708,
		Longitude: -93.8376998901,
		Timezone:  "America/Chicago",
	},
	"LA56": {
		ICAO:      "LA56",
		Name:      "Naylor Airport",
		City:      "Robson",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 155,
		Latitude:  32.3418006897,
		Longitude: -93.6585006714,
		Timezone:  "America/Chicago",
	},
	"LA57": {
		ICAO:      "LA57",
		Name:      "Mowata Flying Service-Eunice Airport",
		City:      "Eunice",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 45,
		Latitude:  30.39909935,
		Longitude: -92.408996582,
		Timezone:  "America/Chicago",
	},
	"LA58": {
		ICAO:      "LA58",
		Name:      "Peter Creek Ranch Airport",
		City:      "Spearsville",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 200,
		Latitude:  32.9790000916,
		Longitude: -92.5932006836,
		Timezone:  "America/Chicago",
	},
	"LA60": {
		ICAO:      "LA60",
		Name:      "Live Oak Landing Strip",
		City:      "Abbeville",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 6,
		Latitude:  29.8169002533,
		Longitude: -92.1167984009,
		Timezone:  "America/Chicago",
	},
	"LA61": {
		ICAO:      "LA61",
		Name:      "Kenan Airstrip",
		City:      "Kaplan",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 20,
		Latitude:  30.00839996,
		Longitude: -92.24120331,
		Timezone:  "America/Chicago",
	},
	"LA63": {
		ICAO:      "LA63",
		Name:      "Travis Airport",
		City:      "Bastrop",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 122,
		Latitude:  32.971698761,
		Longitude: -91.8308029175,
		Timezone:  "America/Chicago",
	},
	"LA67": {
		ICAO:      "LA67",
		Name:      "Angola Airstrip",
		City:      "Angola",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 45,
		Latitude:  30.9515991211,
		Longitude: -91.5848007202,
		Timezone:  "America/Chicago",
	},
	"LA68": {
		ICAO:      "LA68",
		Name:      "Zaunbrecher Strip",
		City:      "Iota",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 26,
		Latitude:  30.3157997131,
		Longitude: -92.5198974609,
		Timezone:  "America/Chicago",
	},
	"LA71": {
		ICAO:      "LA71",
		Name:      "O'Brien Flying Service Airport",
		City:      "Iowa",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 26,
		Latitude:  30.2243995667,
		Longitude: -93.0001983643,
		Timezone:  "America/Chicago",
	},
	"LA73": {
		ICAO:      "LA73",
		Name:      "Huffaker Field",
		City:      "Houma",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 6,
		Latitude:  29.6086006165,
		Longitude: -90.8022994995,
		Timezone:  "America/Chicago",
	},
	"LA75": {
		ICAO:      "LA75",
		Name:      "Glenn's Strip Ultralightport",
		City:      "Duson",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 29,
		Latitude:  30.2052784,
		Longitude: -92.21444702,
		Timezone:  "America/Chicago",
	},
	"LA76": {
		ICAO:      "LA76",
		Name:      "Spillers Club Airport",
		City:      "Breaux Bridge",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 16,
		Latitude:  30.2294006348,
		Longitude: -91.9084014893,
		Timezone:  "America/Chicago",
	},
	"LA77": {
		ICAO:      "LA77",
		Name:      "Mixon Airport",
		City:      "Bunkie",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 60,
		Latitude:  30.9631996155,
		Longitude: -92.1745986938,
		Timezone:  "America/Chicago",
	},
	"LA79": {
		ICAO:      "LA79",
		Name:      "Sycamore Airport",
		City:      "Amite",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 180,
		Latitude:  30.7185001373,
		Longitude: -90.3579025269,
		Timezone:  "America/Chicago",
	},
	"LA81": {
		ICAO:      "LA81",
		Name:      "Cheneyville Airport",
		City:      "Cheneyville",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 63,
		Latitude:  30.0016002655,
		Longitude: -92.2800979614,
		Timezone:  "America/Chicago",
	},
	"LA83": {
		ICAO:      "LA83",
		Name:      "Mc Cutcheon Field",
		City:      "Clinton",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 272,
		Latitude:  30.8959999084,
		Longitude: -91.0458984375,
		Timezone:  "America/Chicago",
	},
	"LA87": {
		ICAO:      "LA87",
		Name:      "Covington-Vincent Airport",
		City:      "Covington",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 35,
		Latitude:  30.5095996857,
		Longitude: -90.1529006958,
		Timezone:  "America/Chicago",
	},
	"LA88": {
		ICAO:      "LA88",
		Name:      "Hensgens Strip",
		City:      "Crowley",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 35,
		Latitude:  30.3446006775,
		Longitude: -92.3432006836,
		Timezone:  "America/Chicago",
	},
	"LA90": {
		ICAO:      "LA90",
		Name:      "Walsh Airport",
		City:      "Crowley",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 26,
		Latitude:  30.4610004425,
		Longitude: -92.4262008667,
		Timezone:  "America/Chicago",
	},
	"LA91": {
		ICAO:      "LA91",
		Name:      "Gary Landing Strip Ultralightport",
		City:      "Parks",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 21,
		Latitude:  30.2213001251,
		Longitude: -91.8335037231,
		Timezone:  "America/Chicago",
	},
	"LA94": {
		ICAO:      "LA94",
		Name:      "Corkern Airport",
		City:      "Enon",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 105,
		Latitude:  30.7224006653,
		Longitude: -90.0653991699,
		Timezone:  "America/Chicago",
	},
	"LAGJ": {
		ICAO:      "LAGJ",
		Name:      "Gjader Air Base",
		City:      "Gjader",
		State:     "Lezhe",
		Country:   "AL",
		Elevation: 23,
		Latitude:  41.895198822,
		Longitude: 19.5986995697,
		Timezone:  "Europe/Tirane",
	},
	"LAKO": {
		ICAO:      "LAKO",
		Name:      "Korce Northwest Airport",
		City:      "Korce",
		State:     "Korce",
		Country:   "AL",
		Elevation: 665,
		Latitude:  40.6456985474,
		Longitude: 20.7415008545,
		Timezone:  "Europe/Tirane",
	},
	"LAKU": {
		ICAO:      "LAKU",
		Name:      "Kukes Airport",
		City:      "Kukes",
		State:     "Kukes",
		Country:   "AL",
		Elevation: 1120,
		Latitude:  42.0336990356,
		Longitude: 20.4158992767,
		Timezone:  "Europe/Tirane",
	},
	"LAKV": {
		ICAO:      "LAKV",
		Name:      "Kucove Air Base",
		City:      "Kucove",
		State:     "Berat",
		Country:   "AL",
		Elevation: 135,
		Latitude:  40.771900177,
		Longitude: 19.9018993378,
		Timezone:  "Europe/Tirane",
	},
	"LATI": {
		ICAO:      "LATI",
		IATA:      "TIA",
		Name:      "Tirana International Airport Mother Teresa",
		City:      "Tirana",
		Country:   "AL",
		Elevation: 126,
		Latitude:  41.4146995544,
		Longitude: 19.7206001282,
		Timezone:  "Europe/Tirane",
	},
	"LAVL": {
		ICAO:      "LAVL",
		Name:      "Vlore Air Base",
		City:      "Vlore",
		State:     "Vlore",
		Country:   "AL",
		Elevation: 3,
		Latitude:  40.4761009216,
		Longitude: 19.474199295,
		Timezone:  "Europe/Tirane",
	},
	"LB11": {
		ICAO:      "LB11",
		Name:      "Tsalapita Airport",
		City:      "Tsalapita",
		State:     "Plovdiv",
		Country:   "BG",
		Elevation: 630,
		Latitude:  42.1876983643,
		Longitude: 24.5349006653,
		Timezone:  "Europe/Sofia",
	},
	"LB13": {
		ICAO:      "LB13",
		Name:      "Radomir Dolni Rakovets Airfield",
		City:      "Radomir",
		State:     "Pernik",
		Country:   "BG",
		Elevation: 625,
		Latitude:  42.4441986084,
		Longitude: 22.983499527,
		Timezone:  "Europe/Sofia",
	},
	"LB18": {
		ICAO:      "LB18",
		Name:      "Tenevo Airport",
		City:      "Tenevo",
		State:     "Yambol",
		Country:   "BG",
		Elevation: 390,
		Latitude:  42.3464012146,
		Longitude: 26.5739002228,
		Timezone:  "Europe/Sofia",
	},
	"LB19": {
		ICAO:      "LB19",
		Name:      "Zimnitsa Airport",
		City:      "Zimnitsa",
		State:     "Yambol",
		Country:   "BG",
		Elevation: 518,
		Latitude:  42.6004981995,
		Longitude: 26.6336994171,
		Timezone:  "Europe/Sofia",
	},
	"LB22": {
		ICAO:      "LB22",
		Name:      "Stanke Dimitrov Airport",
		City:      "Sapareva Banya",
		State:     "Kyustendil",
		Country:   "BG",
		Elevation: 2149,
		Latitude:  42.3116989136,
		Longitude: 23.2469005585,
		Timezone:  "Europe/Sofia",
	},
	"LB27": {
		ICAO:      "LB27",
		Name:      "Podem Airport",
		City:      "Podem",
		State:     "Pleven",
		Country:   "BG",
		Elevation: 150,
		Latitude:  43.5481987,
		Longitude: 24.5904006958,
		Timezone:  "Europe/Sofia",
	},
	"LB29": {
		ICAO:      "LB29",
		Name:      "Maslarevo Airport",
		City:      "Maslarevo",
		State:     "Veliko-Turnovo",
		Country:   "BG",
		Elevation: 349,
		Latitude:  43.4053993225,
		Longitude: 25.4740009308,
		Timezone:  "Europe/Sofia",
	},
	"LB30": {
		ICAO:      "LB30",
		Name:      "Stryama Airport",
		City:      "Stryama",
		State:     "Plovdiv",
		Country:   "BG",
		Elevation: 560,
		Latitude:  42.2853012085,
		Longitude: 24.8684997559,
		Timezone:  "Europe/Sofia",
	},
	"LB32": {
		ICAO:      "LB32",
		Name:      "Konush Airport",
		City:      "Konush",
		State:     "Plovdiv",
		Country:   "BG",
		Elevation: 500,
		Latitude:  42.0723991394,
		Longitude: 25.0533008575,
		Timezone:  "Europe/Sofia",
	},
	"LB33": {
		ICAO:      "LB33",
		Name:      "Leskovo Airport",
		City:      "Leskovo",
		State:     "Dobrich",
		Country:   "BG",
		Elevation: 700,
		Latitude:  43.726600647,
		Longitude: 27.686000824,
		Timezone:  "Europe/Sofia",
	},
	"LB34": {
		ICAO:      "LB34",
		Name:      "Stefanovo Airport",
		City:      "Stefanovo",
		State:     "Dobrich",
		Country:   "BG",
		Elevation: 905,
		Latitude:  43.5069007874,
		Longitude: 27.8449001312,
		Timezone:  "Europe/Sofia",
	},
	"LB35": {
		ICAO:      "LB35",
		Name:      "Golyama Smolnitsa Airport",
		City:      "Smolnitsa",
		State:     "Dobrich",
		Country:   "BG",
		Elevation: 810,
		Latitude:  43.6278991699,
		Longitude: 27.6940994263,
		Timezone:  "Europe/Sofia",
	},
	"LB36": {
		ICAO:      "LB36",
		Name:      "Sokolovo Airport",
		City:      "Sokolovo",
		State:     "Dobrich",
		Country:   "BG",
		Elevation: 760,
		Latitude:  43.4756011963,
		Longitude: 28.1026992798,
		Timezone:  "Europe/Sofia",
	},
	"LB37": {
		ICAO:      "LB37",
		Name:      "Stanke Dimitrov Highway Strip",
		City:      "Boboshevo",
		State:     "Kyustendil",
		Country:   "BG",
		Elevation: 3447,
		Latitude:  42.1757011414,
		Longitude: 23.0508995056,
		Timezone:  "Europe/Sofia",
	},
	"LB38": {
		ICAO:      "LB38",
		Name:      "Mustrak Airport",
		City:      "Mustrak",
		State:     "Khaskovo",
		Country:   "BG",
		Elevation: 675,
		Latitude:  41.8400001526,
		Longitude: 26.3097000122,
		Timezone:  "Europe/Sofia",
	},
	"LB39": {
		ICAO:      "LB39",
		Name:      "Graf Ignatievo South Airport",
		City:      "Graf Ignatievo",
		State:     "Plovdiv",
		Country:   "BG",
		Elevation: 623,
		Latitude:  42.2631988525,
		Longitude: 24.7192993164,
		Timezone:  "Europe/Sofia",
	},
	"LB42": {
		ICAO:      "LB42",
		Name:      "Kamenets Airport",
		City:      "Kamenets",
		State:     "Pleven",
		Country:   "BG",
		Elevation: 420,
		Latitude:  43.3273010254,
		Longitude: 25.0034008026,
		Timezone:  "Europe/Sofia",
	},
	"LB45": {
		ICAO:      "LB45",
		Name:      "Petrich Airport",
		City:      "Petrich",
		State:     "Blagoevgrad",
		Country:   "BG",
		Elevation: 371,
		Latitude:  41.448299408,
		Longitude: 23.2171001434,
		Timezone:  "Europe/Sofia",
	},
	"LBBG": {
		ICAO:      "LBBG",
		IATA:      "BOJ",
		Name:      "Burgas Airport",
		City:      "Burgas",
		State:     "Burgas",
		Country:   "BG",
		Elevation: 135,
		Latitude:  42.5695991516,
		Longitude: 27.5151996613,
		Timezone:  "Europe/Sofia",
	},
	"LBBO": {
		ICAO:      "LBBO",
		Name:      "Bohot Airport",
		City:      "Bohot",
		State:     "Pleven",
		Country:   "BG",
		Elevation: 1162,
		Latitude:  43.306388855,
		Longitude: 24.6913890839,
		Timezone:  "Europe/Sofia",
	},
	"LBDB": {
		ICAO:      "LBDB",
		Name:      "Dolna Banya Airport",
		City:      "Dolna Banya",
		State:     "Sofia",
		Country:   "BG",
		Elevation: 1779,
		Latitude:  42.3085289001,
		Longitude: 23.8204174042,
		Timezone:  "Europe/Sofia",
	},
	"LBGO": {
		ICAO:      "LBGO",
		IATA:      "GOZ",
		Name:      "Gorna Oryahovitsa Airport",
		City:      "Gorna Oryahovitsa",
		State:     "Veliko-Turnovo",
		Country:   "BG",
		Elevation: 285,
		Latitude:  43.1514015198,
		Longitude: 25.7129001617,
		Timezone:  "Europe/Sofia",
	},
	"LBGR": {
		ICAO:      "LBGR",
		Name:      "Grivitsa Airport",
		City:      "Grivitsa",
		State:     "Lovech",
		Country:   "BG",
		Elevation: 919,
		Latitude:  43.4000015259,
		Longitude: 24.7666664124,
		Timezone:  "Europe/Sofia",
	},
	"LBHS": {
		ICAO:      "LBHS",
		IATA:      "HKV",
		Name:      "Uzundzhovo Air Base",
		City:      "Haskovo",
		State:     "Khaskovo",
		Country:   "BG",
		Elevation: 160,
		Latitude:  41.976398468,
		Longitude: 25.589799881,
		Timezone:  "Europe/Sofia",
	},
	"LBHT": {
		ICAO:      "LBHT",
		Name:      "Ihtiman Airport",
		City:      "Ihtiman",
		State:     "Sofia",
		Country:   "BG",
		Elevation: 2113,
		Latitude:  42.4218902588,
		Longitude: 23.7672214508,
		Timezone:  "Europe/Sofia",
	},
	"LBIA": {
		ICAO:      "LBIA",
		IATA:      "JAM",
		Name:      "Bezmer Air Base",
		City:      "Yambol",
		State:     "Burgas",
		Country:   "BG",
		Elevation: 509,
		Latitude:  42.4548988342,
		Longitude: 26.3521995544,
		Timezone:  "Europe/Sofia",
	},
	"LBKJ": {
		ICAO:      "LBKJ",
		Name:      "Kainardzha Airport",
		City:      "Kainardzha",
		State:     "Silistra",
		Country:   "BG",
		Elevation: 575,
		Latitude:  43.9704170227,
		Longitude: 27.4695568085,
		Timezone:  "Europe/Sofia",
	},
	"LBKL": {
		ICAO:      "LBKL",
		Name:      "Kalvacha Airport",
		City:      "Kalvacha",
		State:     "Khaskovo",
		Country:   "BG",
		Elevation: 1083,
		Latitude:  42.5862503052,
		Longitude: 25.4262504578,
		Timezone:  "Europe/Sofia",
	},
	"LBLS": {
		ICAO:      "LBLS",
		Name:      "Lesnovo Airport",
		City:      "Lesnovo",
		State:     "Sofia",
		Country:   "BG",
		Elevation: 1825,
		Latitude:  42.6344985962,
		Longitude: 23.6464443207,
		Timezone:  "Europe/Sofia",
	},
	"LBMG": {
		ICAO:      "LBMG",
		Name:      "Gabrovnitsa Air Base",
		City:      "Gabrovnitsa",
		State:     "Montana",
		Country:   "BG",
		Elevation: 626,
		Latitude:  43.5443000793,
		Longitude: 23.2724990845,
		Timezone:  "Europe/Sofia",
	},
	"LBPD": {
		ICAO:      "LBPD",
		IATA:      "PDV",
		Name:      "Plovdiv International Airport",
		City:      "Plovdiv",
		State:     "Plovdiv",
		Country:   "BG",
		Elevation: 597,
		Latitude:  42.0677986145,
		Longitude: 24.8507995605,
		Timezone:  "Europe/Sofia",
	},
	"LBPG": {
		ICAO:      "LBPG",
		Name:      "Graf Ignatievo Air Base",
		City:      "Graf Ignatievo",
		State:     "Plovdiv",
		Country:   "BG",
		Elevation: 190,
		Latitude:  42.2904014587,
		Longitude: 24.7140007019,
		Timezone:  "Europe/Sofia",
	},
	"LBPL": {
		ICAO:      "LBPL",
		IATA:      "PVN",
		Name:      "Dolna Mitropoliya Air Base",
		City:      "Dolna Mitropoliya",
		State:     "Lovech",
		Country:   "BG",
		Elevation: 330,
		Latitude:  43.4514007568,
		Longitude: 24.5027999878,
		Timezone:  "Europe/Sofia",
	},
	"LBPR": {
		ICAO:      "LBPR",
		Name:      "Primorsko Airport",
		City:      "Primorsko",
		State:     "Burgas",
		Country:   "BG",
		Elevation: 14,
		Latitude:  42.2633323669,
		Longitude: 27.7016677856,
		Timezone:  "Europe/Sofia",
	},
	"LBRD": {
		ICAO:      "LBRD",
		Name:      "Erden Airport",
		City:      "Erden",
		State:     "Montana",
		Country:   "BG",
		Elevation: 617,
		Latitude:  43.499168396,
		Longitude: 23.3047218323,
		Timezone:  "Europe/Sofia",
	},
	"LBSF": {
		ICAO:      "LBSF",
		IATA:      "SOF",
		Name:      "Sofia Airport",
		City:      "Sofia",
		State:     "Sofia-Capital",
		Country:   "BG",
		Elevation: 1742,
		Latitude:  42.6966934204,
		Longitude: 23.4114360809,
		Timezone:  "Europe/Sofia",
	},
	"LBSL": {
		ICAO:      "LBSL",
		Name:      "Sliven Airfield",
		City:      "Sliven",
		State:     "Sliven",
		Country:   "BG",
		Elevation: 656,
		Latitude:  42.6463012695,
		Longitude: 26.3593997955,
		Timezone:  "Europe/Sofia",
	},
	"LBSS": {
		ICAO:      "LBSS",
		IATA:      "SLS",
		Name:      "Silistra Polkovnik Lambrinovo Airfield",
		City:      "Silistra",
		State:     "Razgrad",
		Country:   "BG",
		Elevation: 351,
		Latitude:  44.0551986694,
		Longitude: 27.1788005829,
		Timezone:  "Europe/Sofia",
	},
	"LBST": {
		ICAO:      "LBST",
		Name:      "Stryama Airport",
		City:      "Stryama",
		State:     "Plovdiv",
		Country:   "BG",
		Elevation: 916,
		Latitude:  42.5402793884,
		Longitude: 24.8252220154,
		Timezone:  "Europe/Sofia",
	},
	"LBSZ": {
		ICAO:      "LBSZ",
		IATA:      "SZR",
		Name:      "Stara Zagora Airport",
		City:      "Stara Zagora",
		State:     "Khaskovo",
		Country:   "BG",
		Elevation: 558,
		Latitude:  42.376701355,
		Longitude: 25.6551990509,
		Timezone:  "Europe/Sofia",
	},
	"LBTG": {
		ICAO:      "LBTG",
		IATA:      "TGV",
		Name:      "Bukhovtsi Airfield",
		City:      "Targovishte",
		State:     "Razgrad",
		Country:   "BG",
		Elevation: 666,
		Latitude:  43.3065986633,
		Longitude: 26.7008991241,
		Timezone:  "Europe/Sofia",
	},
	"LBVD": {
		ICAO:      "LBVD",
		IATA:      "VID",
		Name:      "Vidin Smurdan Airfield",
		City:      "Vidin",
		State:     "Montana",
		Country:   "BG",
		Elevation: 200,
		Latitude:  44.0223999023,
		Longitude: 22.8160991669,
		Timezone:  "Europe/Sofia",
	},
	"LBWB": {
		ICAO:      "LBWB",
		Name:      "Balchik Air Base",
		City:      "Balchik",
		State:     "Varna",
		Country:   "BG",
		Elevation: 660,
		Latitude:  43.4238014221,
		Longitude: 28.1812992096,
		Timezone:  "Europe/Sofia",
	},
	"LBWN": {
		ICAO:      "LBWN",
		IATA:      "VAR",
		Name:      "Varna Airport",
		City:      "Varna",
		State:     "Varna",
		Country:   "BG",
		Elevation: 230,
		Latitude:  43.2321014404,
		Longitude: 27.8250999451,
		Timezone:  "Europe/Sofia",
	},
	"LBWV": {
		ICAO:      "LBWV",
		Name:      "Izgrev Airport",
		City:      "Izgrev",
		State:     "Varna",
		Country:   "BG",
		Elevation: 1119,
		Latitude:  43.2772216797,
		Longitude: 27.7019443512,
		Timezone:  "Europe/Sofia",
	},
	"LCEN": {
		ICAO:      "LCEN",
		IATA:      "ECN",
		Name:      "Ercan International Airport",
		City:      "Nicosia",
		State:     "Nicosia",
		Country:   "CY",
		Elevation: 404,
		Latitude:  35.1547012329,
		Longitude: 33.4961013794,
		Timezone:  "Asia/Famagusta",
	},
	"LCLK": {
		ICAO:      "LCLK",
		IATA:      "LCA",
		Name:      "Larnaca International Airport",
		City:      "Larnarca",
		State:     "Larnaka",
		Country:   "CY",
		Elevation: 8,
		Latitude:  34.8750991821,
		Longitude: 33.6249008179,
		Timezone:  "Asia/Nicosia",
	},
	"LCPH": {
		ICAO:      "LCPH",
		IATA:      "PFO",
		Name:      "Paphos International Airport",
		City:      "Paphos",
		State:     "Pafos",
		Country:   "CY",
		Elevation: 41,
		Latitude:  34.7179985046,
		Longitude: 32.4856987,
		Timezone:  "Asia/Nicosia",
	},
	"LCRA": {
		ICAO:      "LCRA",
		IATA:      "AKT",
		Name:      "RAF Akrotiri",
		Country:   "CY",
		Elevation: 76,
		Latitude:  34.5904006958,
		Longitude: 32.9878997803,
		Timezone:  "Asia/Nicosia",
	},
	"LD57": {
		ICAO:      "LD57",
		Name:      "Sepurine Training Base",
		City:      "Zaton",
		State:     "Zadarska",
		Country:   "HR",
		Elevation: 60,
		Latitude:  44.2099990845,
		Longitude: 15.1632003784,
		Timezone:  "Europe/Zagreb",
	},
	"LDDU": {
		ICAO:      "LDDU",
		IATA:      "DBV",
		Name:      "Dubrovnik Airport",
		City:      "Dubrovnik",
		State:     "Dubrovacko-Neretvanska",
		Country:   "HR",
		Elevation: 527,
		Latitude:  42.5614013672,
		Longitude: 18.2681999207,
		Timezone:  "Europe/Zagreb",
	},
	"LDLO": {
		ICAO:      "LDLO",
		IATA:      "LSZ",
		Name:      "Losinj Island Airport",
		City:      "Losinj",
		State:     "Primorsko-Goranska",
		Country:   "HR",
		Elevation: 151,
		Latitude:  44.5657997131,
		Longitude: 14.3930997849,
		Timezone:  "Europe/Zagreb",
	},
	"LDOB": {
		ICAO:      "LDOB",
		Name:      "Vukovar Borovo N Airport",
		City:      "Vukovar",
		State:     "Vukovar-Sirmium",
		Country:   "HR",
		Elevation: 289,
		Latitude:  45.3863983154,
		Longitude: 18.9627990723,
		Timezone:  "Europe/Belgrade",
	},
	"LDOC": {
		ICAO:      "LDOC",
		Name:      "Cepin Airport",
		City:      "Cepin",
		State:     "Osjecko-Baranjska",
		Country:   "HR",
		Elevation: 299,
		Latitude:  45.5421981812,
		Longitude: 18.6361999512,
		Timezone:  "Europe/Zagreb",
	},
	"LDOR": {
		ICAO:      "LDOR",
		Name:      "Slavonski Jelas Airport",
		City:      "Slavonski",
		State:     "Slavonski-Brod-Posavina",
		Country:   "HR",
		Elevation: 276,
		Latitude:  45.1561012268,
		Longitude: 17.9881000519,
		Timezone:  "Europe/Sarajevo",
	},
	"LDOS": {
		ICAO:      "LDOS",
		IATA:      "OSI",
		Name:      "Osijek Airport",
		City:      "Osijek",
		State:     "Osjecko-Baranjska",
		Country:   "HR",
		Elevation: 290,
		Latitude:  45.4626998901,
		Longitude: 18.8101997375,
		Timezone:  "Europe/Zagreb",
	},
	"LDOV": {
		ICAO:      "LDOV",
		Name:      "Vinkovci Sopot Airport",
		City:      "Vinkovci",
		State:     "Vukovar-Sirmium",
		Country:   "HR",
		Elevation: 266,
		Latitude:  45.2510986328,
		Longitude: 18.7591991425,
		Timezone:  "Europe/Zagreb",
	},
	"LDPL": {
		ICAO:      "LDPL",
		IATA:      "PUY",
		Name:      "Pula Airport",
		City:      "Pula",
		State:     "Istria",
		Country:   "HR",
		Elevation: 274,
		Latitude:  44.8935012817,
		Longitude: 13.9222002029,
		Timezone:  "Europe/Zagreb",
	},
	"LDPM": {
		ICAO:      "LDPM",
		Name:      "Medulin Campanoz Airport",
		City:      "Medulin",
		State:     "Istria",
		Country:   "HR",
		Elevation: 171,
		Latitude:  44.8432998657,
		Longitude: 13.9041996002,
		Timezone:  "Europe/Zagreb",
	},
	"LDPN": {
		ICAO:      "LDPN",
		Name:      "Unije Airport",
		City:      "Unije Island",
		State:     "Primorsko-Goranska",
		Country:   "HR",
		Elevation: 39,
		Latitude:  44.6282997131,
		Longitude: 14.2411003113,
		Timezone:  "Europe/Zagreb",
	},
	"LDPV": {
		ICAO:      "LDPV",
		Name:      "Vrsar Crljenka Airport",
		City:      "Vrsar",
		State:     "Istria",
		Country:   "HR",
		Elevation: 121,
		Latitude:  45.1417007446,
		Longitude: 13.6305999756,
		Timezone:  "Europe/Zagreb",
	},
	"LDRG": {
		ICAO:      "LDRG",
		Name:      "Grobnicko Polje Airport",
		City:      "Grobnicko",
		State:     "Primorsko-Goranska",
		Country:   "HR",
		Elevation: 951,
		Latitude:  45.3795013428,
		Longitude: 14.5038003922,
		Timezone:  "Europe/Zagreb",
	},
	"LDRI": {
		ICAO:      "LDRI",
		IATA:      "RJK",
		Name:      "Rijeka Airport",
		City:      "Rijeka",
		State:     "Primorsko-Goranska",
		Country:   "HR",
		Elevation: 278,
		Latitude:  45.2168998718,
		Longitude: 14.5703001022,
		Timezone:  "Europe/Zagreb",
	},
	"LDRO": {
		ICAO:      "LDRO",
		Name:      "Otocac Airport",
		City:      "Otocac",
		State:     "Licko-Senjska",
		Country:   "HR",
		Elevation: 1539,
		Latitude:  44.8477745056,
		Longitude: 15.2866172791,
		Timezone:  "Europe/Zagreb",
	},
	"LDSB": {
		ICAO:      "LDSB",
		IATA:      "BWK",
		Name:      "Brac Airport",
		City:      "Brac Island",
		State:     "Split-Dalmatia",
		Country:   "HR",
		Elevation: 1776,
		Latitude:  43.2857017517,
		Longitude: 16.6797008514,
		Timezone:  "Europe/Zagreb",
	},
	"LDSH": {
		ICAO:      "LDSH",
		Name:      "Hvar Private Airport",
		City:      "Hvar Island",
		State:     "Split-Dalmatia",
		Country:   "HR",
		Elevation: 92,
		Latitude:  43.1817016602,
		Longitude: 16.6333007813,
		Timezone:  "Europe/Zagreb",
	},
	"LDSP": {
		ICAO:      "LDSP",
		IATA:      "SPU",
		Name:      "Split Airport",
		City:      "Split",
		State:     "Split-Dalmatia",
		Country:   "HR",
		Elevation: 79,
		Latitude:  43.538898468,
		Longitude: 16.2980003357,
		Timezone:  "Europe/Zagreb",
	},
	"LDSS": {
		ICAO:      "LDSS",
		Name:      "Sinj Glider Airport",
		City:      "Sinj",
		State:     "Split-Dalmatia",
		Country:   "HR",
		Elevation: 981,
		Latitude:  43.7002983093,
		Longitude: 16.6714000702,
		Timezone:  "Europe/Zagreb",
	},
	"LDVA": {
		ICAO:      "LDVA",
		Name:      "Varazdin Airport",
		City:      "Varazdin",
		State:     "Varazdinska",
		Country:   "HR",
		Elevation: 548,
		Latitude:  46.2946472168,
		Longitude: 16.382932663,
		Timezone:  "Europe/Zagreb",
	},
	"LDVC": {
		ICAO:      "LDVC",
		Name:      "Cakovec Pribisla Airport",
		City:      "Cakovec",
		State:     "Medimurska",
		Country:   "HR",
		Elevation: 512,
		Latitude:  46.3918991089,
		Longitude: 16.5002994537,
		Timezone:  "Europe/Zagreb",
	},
	"LDVD": {
		ICAO:      "LDVD",
		Name:      "Daruvar Blagorod Airport",
		City:      "Daruvar",
		State:     "Bjelovarsko-Bilogorska",
		Country:   "HR",
		Elevation: 410,
		Latitude:  45.5593986511,
		Longitude: 17.0331001282,
		Timezone:  "Europe/Zagreb",
	},
	"LDZA": {
		ICAO:      "LDZA",
		IATA:      "ZAG",
		Name:      "Zagreb Airport",
		City:      "Zagreb",
		State:     "Zagrebacka",
		Country:   "HR",
		Elevation: 353,
		Latitude:  45.7429008484,
		Longitude: 16.0687999725,
		Timezone:  "Europe/Zagreb",
	},
	"LDZB": {
		ICAO:      "LDZB",
		Name:      "Busevec Velika Glider Airport",
		City:      "Busevec",
		State:     "Zagrebacka",
		Country:   "HR",
		Elevation: 341,
		Latitude:  45.6474990845,
		Longitude: 16.1243991852,
		Timezone:  "Europe/Zagreb",
	},
	"LDZD": {
		ICAO:      "LDZD",
		IATA:      "ZAD",
		Name:      "Zemunik Airport",
		City:      "Zadar",
		State:     "Zadarska",
		Country:   "HR",
		Elevation: 289,
		Latitude:  44.1082992554,
		Longitude: 15.3466997147,
		Timezone:  "Europe/Zagreb",
	},
	"LDZJ": {
		ICAO:      "LDZJ",
		Name:      "Bjelovar Brezova Airport",
		City:      "Bjelovar",
		State:     "Bjelovarsko-Bilogorska",
		Country:   "HR",
		Elevation: 430,
		Latitude:  45.8608016968,
		Longitude: 16.8358001709,
		Timezone:  "Europe/Zagreb",
	},
	"LDZL": {
		ICAO:      "LDZL",
		Name:      "Lucko Airport",
		City:      "Zagreb",
		State:     "City-of-Zagreb",
		Country:   "HR",
		Elevation: 400,
		Latitude:  45.7668991089,
		Longitude: 15.8486003876,
		Timezone:  "Europe/Zagreb",
	},
	"LE84": {
		ICAO:      "LE84",
		Name:      "Jayena Airport",
		City:      "Jayena",
		State:     "Andalusia",
		Country:   "ES",
		Elevation: 3654,
		Latitude:  36.9306983948,
		Longitude: -3.8353800774,
		Timezone:  "Europe/Madrid",
	},
	"LEAB": {
		ICAO:      "LEAB",
		IATA:      "ABC",
		Name:      "Albacete-Los Llanos Airport",
		City:      "Albacete",
		State:     "Castille-La-Mancha",
		Country:   "ES",
		Elevation: 2302,
		Latitude:  38.9485015869,
		Longitude: -1.8635200262,
		Timezone:  "Europe/Madrid",
	},
	"LEAE": {
		ICAO:      "LEAE",
		Name:      "Alcolea Airport",
		City:      "Alcolea",
		State:     "Andalusia",
		Country:   "ES",
		Elevation: 463,
		Latitude:  37.9460983276,
		Longitude: -4.5894398689,
		Timezone:  "Europe/Madrid",
	},
	"LEAL": {
		ICAO:      "LEAL",
		IATA:      "ALC",
		Name:      "Alicante International Airport",
		City:      "Alicante",
		State:     "Valencia",
		Country:   "ES",
		Elevation: 142,
		Latitude:  38.2821998596,
		Longitude: -0.5581560135,
		Timezone:  "Europe/Madrid",
	},
	"LEAM": {
		ICAO:      "LEAM",
		IATA:      "LEI",
		Name:      "Almeria International Airport",
		City:      "Almeria",
		State:     "Andalusia",
		Country:   "ES",
		Elevation: 70,
		Latitude:  36.8438987732,
		Longitude: -2.3701000214,
		Timezone:  "Europe/Madrid",
	},
	"LEAP": {
		ICAO:      "LEAP",
		Name:      "Ampuriabrava Airport",
		City:      "Ampuriabraba",
		State:     "Catalonia",
		Country:   "ES",
		Elevation: 2,
		Latitude:  42.2599983215,
		Longitude: 3.109721899,
		Timezone:  "Europe/Madrid",
	},
	"LEAS": {
		ICAO:      "LEAS",
		IATA:      "OVD",
		Name:      "Asturias Airport",
		City:      "Ranon",
		State:     "Asturias",
		Country:   "ES",
		Elevation: 416,
		Latitude:  43.5635986328,
		Longitude: -6.0346198082,
		Timezone:  "Europe/Madrid",
	},
	"LEAT": {
		ICAO:      "LEAT",
		Name:      "Alfes Airport",
		City:      "Alfes",
		State:     "Catalonia",
		Country:   "ES",
		Elevation: 705,
		Latitude:  41.5499992371,
		Longitude: 0.6499999762,
		Timezone:  "Europe/Madrid",
	},
	"LEAX": {
		ICAO:      "LEAX",
		Name:      "La Axarquia-Leoni Benabu Airport",
		City:      "Malaga",
		State:     "Andalusia",
		Country:   "ES",
		Elevation: 118,
		Latitude:  36.8016662598,
		Longitude: -4.135556221,
		Timezone:  "Europe/Madrid",
	},
	"LEBA": {
		ICAO:      "LEBA",
		IATA:      "ODB",
		Name:      "Cordoba Airport",
		City:      "Cordoba",
		State:     "Andalusia",
		Country:   "ES",
		Elevation: 297,
		Latitude:  37.841999054,
		Longitude: -4.8488798141,
		Timezone:  "Europe/Madrid",
	},
	"LEBB": {
		ICAO:      "LEBB",
		IATA:      "BIO",
		Name:      "Bilbao Airport",
		City:      "Bilbao",
		State:     "Basque-Country",
		Country:   "ES",
		Elevation: 138,
		Latitude:  43.3011016846,
		Longitude: -2.9106099606,
		Timezone:  "Europe/Madrid",
	},
	"LEBE": {
		ICAO:      "LEBE",
		Name:      "Beas De Segura Airport",
		City:      "Jaen",
		State:     "Andalusia",
		Country:   "ES",
		Elevation: 1780,
		Latitude:  38.2711105347,
		Longitude: -2.9488890171,
		Timezone:  "Europe/Madrid",
	},
	"LEBG": {
		ICAO:      "LEBG",
		IATA:      "RGS",
		Name:      "Burgos Airport",
		City:      "Burgos",
		State:     "Castille-and-Leon",
		Country:   "ES",
		Elevation: 2945,
		Latitude:  42.3576011658,
		Longitude: -3.620759964,
		Timezone:  "Europe/Madrid",
	},
	"LEBL": {
		ICAO:      "LEBL",
		IATA:      "BCN",
		Name:      "Barcelona International Airport",
		City:      "Barcelona",
		State:     "Catalonia",
		Country:   "ES",
		Elevation: 12,
		Latitude:  41.2971000671,
		Longitude: 2.0784599781,
		Timezone:  "Europe/Madrid",
	},
	"LEBZ": {
		ICAO:      "LEBZ",
		IATA:      "BJZ",
		Name:      "Badajoz Airport",
		City:      "Badajoz",
		State:     "Extremadura",
		Country:   "ES",
		Elevation: 609,
		Latitude:  38.8913002014,
		Longitude: -6.8213300705,
		Timezone:  "Europe/Madrid",
	},
	"LECA": {
		ICAO:      "LECA",
		Name:      "La Nava - Corral De Ayllon Airport",
		City:      "Corral dev Ayllon",
		State:     "Castille-and-Leon",
		Country:   "ES",
		Elevation: 3280,
		Latitude:  41.4108009338,
		Longitude: -3.4483299255,
		Timezone:  "Europe/Madrid",
	},
	"LECD": {
		ICAO:      "LECD",
		Name:      "La Cerdanya Airport",
		City:      "Alp",
		State:     "Catalonia",
		Country:   "ES",
		Elevation: 3586,
		Latitude:  42.3863983154,
		Longitude: 1.8666700125,
		Timezone:  "Europe/Madrid",
	},
	"LECF": {
		ICAO:      "LECF",
		Name:      "Calaf-Sallavinera Airport",
		City:      "Sant Pere Sallavinera",
		State:     "Catalonia",
		Country:   "ES",
		Elevation: 2355,
		Latitude:  41.7442016602,
		Longitude: 1.5569399595,
		Timezone:  "Europe/Madrid",
	},
	"LECH": {
		ICAO:      "LECH",
		IATA:      "CDT",
		Name:      "Calamocha Airport",
		City:      "Calamocha",
		State:     "Aragon",
		Country:   "ES",
		Elevation: 2936,
		Latitude:  40.9000015259,
		Longitude: -1.3041199446,
		Timezone:  "Europe/Madrid",
	},
	"LECI": {
		ICAO:      "LECI",
		Name:      "Santa Cilia De Jaca Airport",
		City:      "Huesca",
		State:     "Aragon",
		Country:   "ES",
		Elevation: 2241,
		Latitude:  42.5691986084,
		Longitude: -0.7277780175,
		Timezone:  "Europe/Madrid",
	},
	"LECJ": {
		ICAO:      "LECJ",
		Name:      "Castejon de los Monegros Airport",
		City:      "Huesca",
		State:     "Aragon",
		Country:   "ES",
		Elevation: 1424,
		Latitude:  41.60779953,
		Longitude: -0.2183270007,
		Timezone:  "Europe/Madrid",
	},
	"LECN": {
		ICAO:      "LECN",
		IATA:      "CDT",
		Name:      "Castellon De La Plana Airport",
		City:      "Castellon de La Plana",
		State:     "Valencia",
		Country:   "ES",
		Elevation: 20,
		Latitude:  39.9991989136,
		Longitude: 0.0261109993,
		Timezone:  "Europe/Madrid",
	},
	"LECO": {
		ICAO:      "LECO",
		IATA:      "LCG",
		Name:      "A Coruna Airport",
		City:      "Culleredo",
		State:     "Galicia",
		Country:   "ES",
		Elevation: 326,
		Latitude:  43.3021011353,
		Longitude: -8.3772602081,
		Timezone:  "Europe/Madrid",
	},
	"LECT": {
		ICAO:      "LECT",
		Name:      "El Castano Airport",
		City:      "Luciana",
		State:     "Castille-La-Mancha",
		Country:   "ES",
		Elevation: 2116,
		Latitude:  39.0107002258,
		Longitude: -4.3863601685,
		Timezone:  "Europe/Madrid",
	},
	"LECU": {
		ICAO:      "LECU",
		IATA:      "ECV",
		Name:      "Cuatro Vientos Airport",
		City:      "Madrid",
		State:     "Madrid",
		Country:   "ES",
		Elevation: 2269,
		Latitude:  40.3707008362,
		Longitude: -3.7851400375,
		Timezone:  "Europe/Madrid",
	},
	"LECX": {
		ICAO:      "LECX",
		Name:      "Campolara Airport",
		City:      "Segovia",
		State:     "Castille-and-Leon",
		Country:   "ES",
		Elevation: 3353,
		Latitude:  40.9030990601,
		Longitude: -4.5202798843,
		Timezone:  "Europe/Madrid",
	},
	"LEDA": {
		ICAO:      "LEDA",
		IATA:      "ILD",
		Name:      "Lleida-Alguaire Airport",
		City:      "Lleida",
		State:     "Catalonia",
		Country:   "ES",
		Elevation: 1170,
		Latitude:  41.728185,
		Longitude: 0.535023,
		Timezone:  "Europe/Madrid",
	},
	"LEDS": {
		ICAO:      "LEDS",
		Name:      "Castellon-Costa Azahar Airport",
		City:      "Castellon de la Plana",
		State:     "Valencia",
		Country:   "ES",
		Elevation: 1145,
		Latitude:  40.2138888889,
		Longitude: 0.0733333333,
		Timezone:  "Europe/Madrid",
	},
	"LEEM": {
		ICAO:      "LEEM",
		Name:      "El Manantio Airport",
		City:      "Badajoz",
		State:     "Extremadura",
		Country:   "ES",
		Elevation: 708,
		Latitude:  38.777507782,
		Longitude: -6.9911241531,
		Timezone:  "Europe/Madrid",
	},
	"LEEV": {
		ICAO:      "LEEV",
		Name:      "E. Castellanos-Villacastin Airport",
		City:      "Villacastin",
		State:     "Castille-and-Leon",
		Country:   "ES",
		Elevation: 3637,
		Latitude:  40.7839012146,
		Longitude: -4.4627799988,
		Timezone:  "Europe/Madrid",
	},
	"LEFM": {
		ICAO:      "LEFM",
		Name:      "Fuentemilanos Airport",
		City:      "Segovia",
		State:     "Castille-and-Leon",
		Country:   "ES",
		Elevation: 3307,
		Latitude:  40.8886108398,
		Longitude: -4.2375001907,
		Timezone:  "Europe/Madrid",
	},
	"LEGA": {
		ICAO:      "LEGA",
		Name:      "Armilla Airport",
		City:      "Armilla",
		State:     "Andalusia",
		Country:   "ES",
		Elevation: 2297,
		Latitude:  37.1332015991,
		Longitude: -3.6356899738,
		Timezone:  "Europe/Madrid",
	},
	"LEGC": {
		ICAO:      "LEGC",
		Name:      "Altarejos-Guadalcanal Airport",
		City:      "Sevilla",
		State:     "Andalusia",
		Country:   "ES",
		Elevation: 1928,
		Latitude:  38.1716995239,
		Longitude: -5.7395300865,
		Timezone:  "Europe/Madrid",
	},
	"LEGE": {
		ICAO:      "LEGE",
		IATA:      "GRO",
		Name:      "Girona Airport",
		City:      "Girona",
		State:     "Catalonia",
		Country:   "ES",
		Elevation: 468,
		Latitude:  41.9010009766,
		Longitude: 2.7605500221,
		Timezone:  "Europe/Madrid",
	},
	"LEGR": {
		ICAO:      "LEGR",
		IATA:      "GRX",
		Name:      "Federico Garcia Lorca Airport",
		City:      "Granada",
		State:     "Andalusia",
		Country:   "ES",
		Elevation: 1860,
		Latitude:  37.1887016296,
		Longitude: -3.7773599625,
		Timezone:  "Europe/Madrid",
	},
	"LEGT": {
		ICAO:      "LEGT",
		Name:      "Getafe Air Base",
		City:      "Getafe",
		State:     "Madrid",
		Country:   "ES",
		Elevation: 2031,
		Latitude:  40.2941017151,
		Longitude: -3.7238299847,
		Timezone:  "Europe/Madrid",
	},
	"LEGU": {
		ICAO:      "LEGU",
		Name:      "Guadalupe Airport",
		City:      "Guadalupe",
		State:     "Extremadura",
		Country:   "ES",
		Elevation: 1408,
		Latitude:  39.3456993103,
		Longitude: -5.1973500252,
		Timezone:  "Europe/Madrid",
	},
	"LEGY": {
		ICAO:      "LEGY",
		Name:      "Garray Airport",
		City:      "Soria",
		State:     "Castille-and-Leon",
		Country:   "ES",
		Elevation: 3360,
		Latitude:  41.8231010437,
		Longitude: -2.4786100388,
		Timezone:  "Europe/Madrid",
	},
	"LEHC": {
		ICAO:      "LEHC",
		IATA:      "HSK",
		Name:      "Huesca/Pirineos Airport",
		City:      "Monflorite/Alcala del Obispo",
		State:     "Aragon",
		Country:   "ES",
		Elevation: 1768,
		Latitude:  42.0760993958,
		Longitude: -0.3166669905,
		Timezone:  "Europe/Madrid",
	},
	"LEHI": {
		ICAO:      "LEHI",
		Name:      "Hinojosa del Duque Airport",
		City:      "Hinojosa del Duque",
		State:     "Andalusia",
		Country:   "ES",
		Elevation: 1772,
		Latitude:  38.5,
		Longitude: -5.117000103,
		Timezone:  "Europe/Madrid",
	},
	"LEIB": {
		ICAO:      "LEIB",
		IATA:      "IBZ",
		Name:      "Ibiza Airport",
		City:      "Ibiza",
		State:     "Balearic-Islands",
		Country:   "ES",
		Elevation: 24,
		Latitude:  38.8728981018,
		Longitude: 1.3731199503,
		Timezone:  "Europe/Madrid",
	},
	"LEIG": {
		ICAO:      "LEIG",
		Name:      "Igualada/Odena Airport",
		City:      "Barcelona",
		State:     "Catalonia",
		Country:   "ES",
		Elevation: 1148,
		Latitude:  41.5856018066,
		Longitude: 1.6530599594,
		Timezone:  "Europe/Madrid",
	},
	"LEIZ": {
		ICAO:      "LEIZ",
		Name:      "La Perdiz - Torre De Juan Abad Airport",
		City:      "Torre de Juan Abad",
		State:     "Castille-La-Mancha",
		Country:   "ES",
		Elevation: 2902,
		Latitude:  38.5130996704,
		Longitude: -3.3643300533,
		Timezone:  "Europe/Madrid",
	},
	"LEJO": {
		ICAO:      "LEJO",
		Name:      "Madrigalejo Del Monte Airport",
		City:      "Madrigalejo Del Monte",
		State:     "Castille-and-Leon",
		Country:   "ES",
		Elevation: 3030,
		Latitude:  42.1343994141,
		Longitude: -3.7310299873,
		Timezone:  "Europe/Madrid",
	},
	"LEJR": {
		ICAO:      "LEJR",
		IATA:      "XRY",
		Name:      "Jerez Airport",
		City:      "Jerez de la Forntera",
		State:     "Andalusia",
		Country:   "ES",
		Elevation: 93,
		Latitude:  36.7445983887,
		Longitude: -6.0601100922,
		Timezone:  "Europe/Madrid",
	},
	"LEJU": {
		ICAO:      "LEJU",
		Name:      "La Juliana Airport",
		City:      "Sevilla",
		State:     "Andalusia",
		Country:   "ES",
		Elevation: 148,
		Latitude:  37.2949981689,
		Longitude: -6.1624999046,
		Timezone:  "Europe/Madrid",
	},
	"LELA": {
		ICAO:      "LELA",
		Name:      "La Calderera Airport",
		City:      "Valdepenas",
		State:     "Castille-La-Mancha",
		Country:   "ES",
		Elevation: 2644,
		Latitude:  38.7478981018,
		Longitude: -3.5145599842,
		Timezone:  "Europe/Madrid",
	},
	"LELC": {
		ICAO:      "LELC",
		IATA:      "MJV",
		Name:      "San Javier Airport",
		City:      "San Javier",
		State:     "Murcia",
		Country:   "ES",
		Elevation: 11,
		Latitude:  37.7750015259,
		Longitude: -0.8123890162,
		Timezone:  "Europe/Madrid",
	},
	"LELH": {
		ICAO:      "LELH",
		Name:      "Alhama De Murcia Airport",
		City:      "Canovas",
		State:     "Murcia",
		Country:   "ES",
		Elevation: 669,
		Latitude:  37.7489013672,
		Longitude: -1.3021600246,
		Timezone:  "Europe/Madrid",
	},
	"LELI": {
		ICAO:      "LELI",
		Name:      "Linares Airport",
		City:      "Linares",
		State:     "Andalusia",
		Country:   "ES",
		Elevation: 1033,
		Latitude:  38.090801239,
		Longitude: -3.7066700459,
		Timezone:  "Europe/Madrid",
	},
	"LELL": {
		ICAO:      "LELL",
		IATA:      "QSA",
		Name:      "Sabadell Airport",
		City:      "Sabadell",
		State:     "Catalonia",
		Country:   "ES",
		Elevation: 485,
		Latitude:  41.5209007263,
		Longitude: 2.1050798893,
		Timezone:  "Europe/Madrid",
	},
	"LELM": {
		ICAO:      "LELM",
		Name:      "Almansa Airport",
		City:      "Albacete",
		State:     "Castille-La-Mancha",
		Country:   "ES",
		Elevation: 2248,
		Latitude:  38.8950004578,
		Longitude: -1.1131800413,
		Timezone:  "Europe/Madrid",
	},
	"LELN": {
		ICAO:      "LELN",
		IATA:      "LEN",
		Name:      "Leon Airport",
		City:      "Leon",
		State:     "Castille-and-Leon",
		Country:   "ES",
		Elevation: 3006,
		Latitude:  42.5890007019,
		Longitude: -5.6555600166,
		Timezone:  "Europe/Madrid",
	},
	"LELO": {
		ICAO:      "LELO",
		IATA:      "RJL",
		Name:      "Logrono-Agoncillo Airport",
		City:      "Logrono",
		State:     "La-Rioja",
		Country:   "ES",
		Elevation: 1161,
		Latitude:  42.4609534888,
		Longitude: -2.3222351074,
		Timezone:  "Europe/Madrid",
	},
	"LELT": {
		ICAO:      "LELT",
		Name:      "Lillo Airport",
		City:      "Lillo",
		State:     "Castille-La-Mancha",
		Country:   "ES",
		Elevation: 2272,
		Latitude:  39.7168998718,
		Longitude: -3.3205599785,
		Timezone:  "Europe/Madrid",
	},
	"LEMD": {
		ICAO:      "LEMD",
		IATA:      "MAD",
		Name:      "Madrid Barajas International Airport",
		City:      "Madrid",
		State:     "Madrid",
		Country:   "ES",
		Elevation: 2001,
		Latitude:  40.4936,
		Longitude: -3.56676,
		Timezone:  "Europe/Madrid",
	},
	"LEMF": {
		ICAO:      "LEMF",
		IATA:      "HEV",
		Name:      "Mafe - Gibraleon Airport",
		City:      "Gibraleon",
		State:     "Andalusia",
		Country:   "ES",
		Elevation: 86,
		Latitude:  37.3642997742,
		Longitude: -6.9209399223,
		Timezone:  "Europe/Madrid",
	},
	"LEMG": {
		ICAO:      "LEMG",
		IATA:      "AGP",
		Name:      "Malaga Airport",
		City:      "Malaga",
		State:     "Andalusia",
		Country:   "ES",
		Elevation: 53,
		Latitude:  36.6749000549,
		Longitude: -4.4991102219,
		Timezone:  "Europe/Madrid",
	},
	"LEMH": {
		ICAO:      "LEMH",
		IATA:      "MAH",
		Name:      "Menorca Airport",
		City:      "Menorca Island",
		State:     "Balearic-Islands",
		Country:   "ES",
		Elevation: 302,
		Latitude:  39.8625984192,
		Longitude: 4.2186498642,
		Timezone:  "Europe/Madrid",
	},
	"LEMI": {
		ICAO:      "LEMI",
		IATA:      "RMU",
		Name:      "Región de Murcia International Airport",
		City:      "Corvera",
		State:     "Corvera",
		Country:   "ES",
		Elevation: 633,
		Latitude:  37.803,
		Longitude: -1.125,
		Timezone:  "Europe/Madrid",
	},
	"LEMO": {
		ICAO:      "LEMO",
		IATA:      "OZP",
		Name:      "Moron Air Base",
		City:      "Moron",
		State:     "Andalusia",
		Country:   "ES",
		Elevation: 285,
		Latitude:  37.1749000549,
		Longitude: -5.615940094,
		Timezone:  "Europe/Madrid",
	},
	"LEMP": {
		ICAO:      "LEMP",
		Name:      "Los Martinez Del Puerto Airport",
		City:      "Murcia",
		State:     "Murcia",
		Country:   "ES",
		Elevation: 534,
		Latitude:  37.7989006042,
		Longitude: -1.0974999666,
		Timezone:  "Europe/Madrid",
	},
	"LEMR": {
		ICAO:      "LEMR",
		Name:      "La Morgal Airport",
		City:      "Oviedo",
		State:     "Asturias",
		Country:   "ES",
		Elevation: 541,
		Latitude:  43.4291992188,
		Longitude: -5.8305602074,
		Timezone:  "Europe/Madrid",
	},
	"LEMS": {
		ICAO:      "LEMS",
		Name:      "Manresa Airport",
		City:      "Manresa",
		State:     "Catalonia",
		Country:   "ES",
		Elevation: 897,
		Latitude:  41.7653007507,
		Longitude: 1.8616700172,
		Timezone:  "Europe/Madrid",
	},
	"LEMT": {
		ICAO:      "LEMT",
		Name:      "Casarrubios Del Monte Airport",
		City:      "Toledo",
		State:     "Castille-La-Mancha",
		Country:   "ES",
		Elevation: 2050,
		Latitude:  40.2350006104,
		Longitude: -4.0263900757,
		Timezone:  "Europe/Madrid",
	},
	"LEMU": {
		ICAO:      "LEMU",
		Name:      "Muchamiel Airport",
		City:      "Muchamiel",
		State:     "Valencia",
		Country:   "ES",
		Elevation: 427,
		Latitude:  38.4406013489,
		Longitude: -0.4752779901,
		Timezone:  "Europe/Madrid",
	},
	"LEMX": {
		ICAO:      "LEMX",
		Name:      "La Mancha-Toledo Airport",
		City:      "Toledo",
		State:     "Castille-La-Mancha",
		Country:   "ES",
		Elevation: 2405,
		Latitude:  39.5620994568,
		Longitude: -3.2507500648,
		Timezone:  "Europe/Madrid",
	},
	"LENA": {
		ICAO:      "LENA",
		Name:      "Benabarre Airport",
		City:      "Huesca",
		State:     "Aragon",
		Country:   "ES",
		Elevation: 2450,
		Latitude:  42.0228004456,
		Longitude: 0.4822219908,
		Timezone:  "Europe/Madrid",
	},
	"LENE": {
		ICAO:      "LENE",
		Name:      "La Caminera Airport",
		City:      "Torrenueva",
		State:     "Ciudad Real",
		Country:   "ES",
		Elevation: 2513,
		Latitude:  38.669739,
		Longitude: -3.303389,
		Timezone:  "Europe/Madrid",
	},
	"LEOC": {
		ICAO:      "LEOC",
		Name:      "Ocana Airport",
		City:      "Toledo",
		State:     "Castille-La-Mancha",
		Country:   "ES",
		Elevation: 2405,
		Latitude:  39.9375,
		Longitude: -3.5033299923,
		Timezone:  "Europe/Madrid",
	},
	"LEOT": {
		ICAO:      "LEOT",
		Name:      "Ontur Airport",
		City:      "Ontur",
		State:     "Castille-La-Mancha",
		Country:   "ES",
		Elevation: 2200,
		Latitude:  38.6169013977,
		Longitude: -1.5249999762,
		Timezone:  "Europe/Madrid",
	},
	"LEPA": {
		ICAO:      "LEPA",
		IATA:      "PMI",
		Name:      "Palma De Mallorca Airport",
		City:      "Palma De Mallorca",
		State:     "Balearic-Islands",
		Country:   "ES",
		Elevation: 27,
		Latitude:  39.551700592,
		Longitude: 2.7388100624,
		Timezone:  "Europe/Madrid",
	},
	"LEPI": {
		ICAO:      "LEPI",
		Name:      "Casas de los Pinos Airport",
		City:      "Cuenca",
		State:     "Castille-La-Mancha",
		Country:   "ES",
		Elevation: 2370,
		Latitude:  39.2981987,
		Longitude: -2.3787200451,
		Timezone:  "Europe/Madrid",
	},
	"LEPP": {
		ICAO:      "LEPP",
		IATA:      "PNA",
		Name:      "Pamplona Airport",
		City:      "Pamplona",
		State:     "Navarre",
		Country:   "ES",
		Elevation: 1504,
		Latitude:  42.7700004578,
		Longitude: -1.646329999,
		Timezone:  "Europe/Madrid",
	},
	"LEPR": {
		ICAO:      "LEPR",
		Name:      "Palma Del Rio Airport",
		City:      "Palma del Rio",
		State:     "Andalusia",
		Country:   "ES",
		Elevation: 408,
		Latitude:  37.7153015137,
		Longitude: -5.2135601044,
		Timezone:  "Europe/Madrid",
	},
	"LEPZ": {
		ICAO:      "LEPZ",
		Name:      "Los Pozuelos De Calatrava Airport",
		City:      "Los Pozuelos De Calatrava",
		State:     "Castille-La-Mancha",
		Country:   "ES",
		Elevation: 1850,
		Latitude:  38.9122009277,
		Longitude: -4.1911101341,
		Timezone:  "Europe/Madrid",
	},
	"LERE": {
		ICAO:      "LERE",
		Name:      "Requena Airport",
		City:      "Requena",
		State:     "Valencia",
		Country:   "ES",
		Elevation: 2330,
		Latitude:  39.4747009277,
		Longitude: -1.0344400406,
		Timezone:  "Europe/Madrid",
	},
	"LERI": {
		ICAO:      "LERI",
		Name:      "Alcantarilla Airport",
		City:      "Alcantarilla",
		State:     "Murcia",
		Country:   "ES",
		Elevation: 250,
		Latitude:  37.9510993958,
		Longitude: -1.2303199768,
		Timezone:  "Europe/Madrid",
	},
	"LERM": {
		ICAO:      "LERM",
		Name:      "Robledillo De Mohernando Airport",
		City:      "Robledillo De Mohernando",
		State:     "Castille-La-Mancha",
		Country:   "ES",
		Elevation: 3097,
		Latitude:  40.8652992249,
		Longitude: -3.2477800846,
		Timezone:  "Europe/Madrid",
	},
	"LERO": {
		ICAO:      "LERO",
		Name:      "Rozas Airport",
		City:      "Lugo",
		State:     "Galicia",
		Country:   "ES",
		Elevation: 1444,
		Latitude:  43.1169013977,
		Longitude: -7.4702801704,
		Timezone:  "Europe/Madrid",
	},
	"LERS": {
		ICAO:      "LERS",
		IATA:      "REU",
		Name:      "Reus Air Base",
		City:      "Reus",
		State:     "Catalonia",
		Country:   "ES",
		Elevation: 233,
		Latitude:  41.1473999023,
		Longitude: 1.1671700478,
		Timezone:  "Europe/Madrid",
	},
	"LERT": {
		ICAO:      "LERT",
		IATA:      "ROZ",
		Name:      "Rota Naval Station Airport",
		City:      "Rota",
		State:     "Andalusia",
		Country:   "ES",
		Elevation: 86,
		Latitude:  36.645198822,
		Longitude: -6.349460125,
		Timezone:  "Europe/Madrid",
	},
	"LESA": {
		ICAO:      "LESA",
		IATA:      "SLM",
		Name:      "Salamanca Airport",
		City:      "Salamanca",
		State:     "Castille-and-Leon",
		Country:   "ES",
		Elevation: 2595,
		Latitude:  40.9520988464,
		Longitude: -5.5019898415,
		Timezone:  "Europe/Madrid",
	},
	"LESB": {
		ICAO:      "LESB",
		Name:      "Son Bonet Airport",
		City:      "Mallorca Island",
		State:     "Balearic-Islands",
		Country:   "ES",
		Elevation: 157,
		Latitude:  39.5988998413,
		Longitude: 2.7027800083,
		Timezone:  "Europe/Madrid",
	},
	"LESE": {
		ICAO:      "LESE",
		Name:      "San Enrique Airport",
		City:      "Almodovar del Campo",
		State:     "Castille-La-Mancha",
		Country:   "ES",
		Elevation: 2263,
		Latitude:  38.7308006287,
		Longitude: -4.3130598068,
		Timezone:  "Europe/Madrid",
	},
	"LESG": {
		ICAO:      "LESG",
		Name:      "Sanguesa Airport",
		City:      "Sanguesa",
		State:     "Navarre",
		Country:   "ES",
		Elevation: 1297,
		Latitude:  42.5656013489,
		Longitude: -1.284719944,
		Timezone:  "Europe/Madrid",
	},
	"LESL": {
		ICAO:      "LESL",
		Name:      "San Luis Airport",
		City:      "Menorca Island",
		State:     "Balearic-Islands",
		Country:   "ES",
		Elevation: 197,
		Latitude:  39.8622016907,
		Longitude: 4.2583298683,
		Timezone:  "Europe/Madrid",
	},
	"LESO": {
		ICAO:      "LESO",
		IATA:      "EAS",
		Name:      "San Sebastian Airport",
		City:      "Hondarribia",
		State:     "Basque-Country",
		Country:   "ES",
		Elevation: 16,
		Latitude:  43.3564987183,
		Longitude: -1.7906099558,
		Timezone:  "Europe/Madrid",
	},
	"LESS": {
		ICAO:      "LESS",
		Name:      "Sotos Airport",
		City:      "Sotos",
		State:     "Castille-La-Mancha",
		Country:   "ES",
		Elevation: 3170,
		Latitude:  40.2042007446,
		Longitude: -2.143889904,
		Timezone:  "Europe/Madrid",
	},
	"LEST": {
		ICAO:      "LEST",
		IATA:      "SCQ",
		Name:      "Santiago de Compostela Airport",
		City:      "Santiago de Compostela",
		State:     "Galicia",
		Country:   "ES",
		Elevation: 1213,
		Latitude:  42.8963012695,
		Longitude: -8.415140152,
		Timezone:  "Europe/Madrid",
	},
	"LESU": {
		ICAO:      "LESU",
		IATA:      "LEU",
		Name:      "Aerodrom dels Pirineus-Alt Urgell Airport",
		City:      "Montferrer / Castellbo",
		State:     "Catalonia",
		Country:   "ES",
		Elevation: 2625,
		Latitude:  42.3386001587,
		Longitude: 1.4091700315,
		Timezone:  "Europe/Madrid",
	},
	"LETC": {
		ICAO:      "LETC",
		Name:      "Matilla De Los Canos Airport",
		City:      "Valladolid",
		State:     "Castille-and-Leon",
		Country:   "ES",
		Elevation: 2300,
		Latitude:  41.5306015015,
		Longitude: -4.9250001907,
		Timezone:  "Europe/Madrid",
	},
	"LETE": {
		ICAO:      "LETE",
		Name:      "Morante Airport",
		City:      "Badajoz",
		State:     "Extremadura",
		Country:   "ES",
		Elevation: 703,
		Latitude:  39.0368995667,
		Longitude: -6.690559864,
		Timezone:  "Europe/Madrid",
	},
	"LETF": {
		ICAO:      "LETF",
		Name:      "Tomas Fernandez Espada Airport",
		City:      "Villamartin",
		State:     "Andalusia",
		Country:   "ES",
		Elevation: 470,
		Latitude:  36.8717934415,
		Longitude: -5.6487751007,
		Timezone:  "Europe/Madrid",
	},
	"LETI": {
		ICAO:      "LETI",
		Name:      "El Tietar Airport",
		City:      "La Iglesuela",
		State:     "Castille-La-Mancha",
		Country:   "ES",
		Elevation: 1401,
		Latitude:  40.2439002991,
		Longitude: -4.7944397926,
		Timezone:  "Europe/Madrid",
	},
	"LETJ": {
		ICAO:      "LETJ",
		Name:      "Trebujena Airfield",
		City:      "Trebujena",
		State:     "Andalusia",
		Country:   "ES",
		Elevation: 12,
		Latitude:  36.8588888889,
		Longitude: -6.1391666667,
		Timezone:  "Europe/Madrid",
	},
	"LETO": {
		ICAO:      "LETO",
		IATA:      "TOJ",
		Name:      "Torrejon Airport",
		City:      "Madrid",
		State:     "Madrid",
		Country:   "ES",
		Elevation: 2026,
		Latitude:  40.4967002869,
		Longitude: -3.4458699226,
		Timezone:  "Europe/Madrid",
	},
	"LETP": {
		ICAO:      "LETP",
		Name:      "Santo Tome Del Puerto Airport",
		City:      "Segovia",
		State:     "Castille-and-Leon",
		Country:   "ES",
		Elevation: 3638,
		Latitude:  41.2042007446,
		Longitude: -3.5947198868,
		Timezone:  "Europe/Madrid",
	},
	"LETZ": {
		ICAO:      "LETZ",
		Name:      "Torozos Airport",
		City:      "Valladolid",
		State:     "Castille-and-Leon",
		Country:   "ES",
		Elevation: 2784,
		Latitude:  41.7853012085,
		Longitude: -4.8647298813,
		Timezone:  "Europe/Madrid",
	},
	"LEVB": {
		ICAO:      "LEVB",
		Name:      "El Carrascal Airport",
		City:      "Valladolid",
		State:     "Castille-and-Leon",
		Country:   "ES",
		Elevation: 2788,
		Latitude:  41.8246994019,
		Longitude: -4.8930602074,
		Timezone:  "Europe/Madrid",
	},
	"LEVC": {
		ICAO:      "LEVC",
		IATA:      "VLC",
		Name:      "Valencia Airport",
		City:      "Valencia",
		State:     "Valencia",
		Country:   "ES",
		Elevation: 240,
		Latitude:  39.4892997742,
		Longitude: -0.4816249907,
		Timezone:  "Europe/Madrid",
	},
	"LEVD": {
		ICAO:      "LEVD",
		IATA:      "VLL",
		Name:      "Valladolid Airport",
		City:      "Valladolid",
		State:     "Castille-and-Leon",
		Country:   "ES",
		Elevation: 2776,
		Latitude:  41.7061004639,
		Longitude: -4.851940155,
		Timezone:  "Europe/Madrid",
	},
	"LEVF": {
		ICAO:      "LEVF",
		Name:      "Villaframil Airport",
		City:      "Lugo",
		State:     "Galicia",
		Country:   "ES",
		Elevation: 80,
		Latitude:  43.5525016785,
		Longitude: -7.0877799988,
		Timezone:  "Europe/Madrid",
	},
	"LEVI": {
		ICAO:      "LEVI",
		Name:      "Viso del Marques Airport",
		City:      "Viso del Marques",
		State:     "Castille-La-Mancha",
		Country:   "ES",
		Elevation: 2672,
		Latitude:  38.5055999756,
		Longitude: -3.4261500835,
		Timezone:  "Europe/Madrid",
	},
	"LEVT": {
		ICAO:      "LEVT",
		IATA:      "VIT",
		Name:      "Vitoria/Foronda Airport",
		City:      "Alava",
		State:     "Basque-Country",
		Country:   "ES",
		Elevation: 1682,
		Latitude:  42.8828010559,
		Longitude: -2.7244699001,
		Timezone:  "Europe/Madrid",
	},
	"LEVX": {
		ICAO:      "LEVX",
		IATA:      "VGO",
		Name:      "Vigo Airport",
		City:      "Vigo",
		State:     "Galicia",
		Country:   "ES",
		Elevation: 856,
		Latitude:  42.2318000793,
		Longitude: -8.6267700195,
		Timezone:  "Europe/Madrid",
	},
	"LEXJ": {
		ICAO:      "LEXJ",
		IATA:      "SDR",
		Name:      "Santander Airport",
		City:      "Santander",
		State:     "Cantabria",
		Country:   "ES",
		Elevation: 16,
		Latitude:  43.4271011353,
		Longitude: -3.8200099468,
		Timezone:  "Europe/Madrid",
	},
	"LEZG": {
		ICAO:      "LEZG",
		IATA:      "ZAZ",
		Name:      "Zaragoza Air Base",
		City:      "Zaragoza",
		State:     "Aragon",
		Country:   "ES",
		Elevation: 863,
		Latitude:  41.6661987305,
		Longitude: -1.0415500402,
		Timezone:  "Europe/Madrid",
	},
	"LEZL": {
		ICAO:      "LEZL",
		IATA:      "SVQ",
		Name:      "Sevilla Airport",
		City:      "Sevilla",
		State:     "Andalusia",
		Country:   "ES",
		Elevation: 112,
		Latitude:  37.4179992676,
		Longitude: -5.8931097984,
		Timezone:  "Europe/Madrid",
	},
	"LF50": {
		ICAO:      "LF50",
		Name:      "Soucelles Airport",
		City:      "Soucelles",
		State:     "Pays-de-la-Loire",
		Country:   "FR",
		Elevation: 255,
		Latitude:  47.5803985596,
		Longitude: -0.4122689962,
		Timezone:  "Europe/Paris",
	},
	"LF51": {
		ICAO:      "LF51",
		Name:      "Plan-de-Dieu Airport",
		City:      "Plan-de-Dieu",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Elevation: 305,
		Latitude:  44.1800003052,
		Longitude: 4.9188899994,
		Timezone:  "Europe/Paris",
	},
	"LF52": {
		ICAO:      "LF52",
		Name:      "Chambley Airport",
		City:      "Chambley",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 866,
		Latitude:  49.0255012512,
		Longitude: 5.8760700226,
		Timezone:  "Europe/Paris",
	},
	"LFAB": {
		ICAO:      "LFAB",
		IATA:      "DPE",
		Name:      "St Aubin Airport",
		City:      "Dieppe",
		State:     "Normandy",
		Country:   "FR",
		Elevation: 344,
		Latitude:  49.8824996948,
		Longitude: 1.0852799416,
		Timezone:  "Europe/Paris",
	},
	"LFAC": {
		ICAO:      "LFAC",
		IATA:      "CQF",
		Name:      "Calais-Dunkerque Airport",
		City:      "Marck",
		State:     "Hauts-de-France",
		Country:   "FR",
		Elevation: 12,
		Latitude:  50.9621009827,
		Longitude: 1.9547599554,
		Timezone:  "Europe/Paris",
	},
	"LFAD": {
		ICAO:      "LFAD",
		IATA:      "XCP",
		Name:      "Compiegne Margny Airport",
		City:      "Compiegne",
		State:     "Hauts-de-France",
		Country:   "FR",
		Elevation: 315,
		Latitude:  49.4343986511,
		Longitude: 2.8061099052,
		Timezone:  "Europe/Paris",
	},
	"LFAE": {
		ICAO:      "LFAE",
		Name:      "Eu Mers Le Treport Airport",
		City:      "Le Treport",
		State:     "Normandy",
		Country:   "FR",
		Elevation: 328,
		Latitude:  50.0691986084,
		Longitude: 1.4266699553,
		Timezone:  "Europe/Paris",
	},
	"LFAF": {
		ICAO:      "LFAF",
		IATA:      "XLN",
		Name:      "Laon - Chambry Airport",
		City:      "Laon-Chambry",
		State:     "Hauts-de-France",
		Country:   "FR",
		Elevation: 256,
		Latitude:  49.5957984924,
		Longitude: 3.6316699982,
		Timezone:  "Europe/Paris",
	},
	"LFAG": {
		ICAO:      "LFAG",
		IATA:      "XSJ",
		Name:      "Peronne-Saint-Quentin Airport",
		City:      "Peronne/Saint-Quentin",
		State:     "Hauts-de-France",
		Country:   "FR",
		Elevation: 295,
		Latitude:  49.8684997559,
		Longitude: 3.0295801163,
		Timezone:  "Europe/Paris",
	},
	"LFAH": {
		ICAO:      "LFAH",
		Name:      "Soissons-Cuffies Airport",
		City:      "Soissons-Cuffies",
		State:     "Hauts-de-France",
		Country:   "FR",
		Elevation: 138,
		Latitude:  49.4000015259,
		Longitude: 3.3169999123,
		Timezone:  "Europe/Paris",
	},
	"LFAI": {
		ICAO:      "LFAI",
		Name:      "Nangis-Les Loges Airport",
		City:      "Nangis/Les Loges",
		State:     "Ile-de-France",
		Country:   "FR",
		Elevation: 428,
		Latitude:  48.5961990356,
		Longitude: 3.0067899227,
		Timezone:  "Europe/Paris",
	},
	"LFAJ": {
		ICAO:      "LFAJ",
		Name:      "Argentan Airport",
		City:      "Argentan",
		State:     "Normandy",
		Country:   "FR",
		Elevation: 581,
		Latitude:  48.7106018066,
		Longitude: 0.003889,
		Timezone:  "Europe/Paris",
	},
	"LFAK": {
		ICAO:      "LFAK",
		IATA:      "XDK",
		Name:      "Dunkerque Les Moeres Airport",
		City:      "Les Moeres",
		State:     "Hauts-de-France",
		Country:   "FR",
		Elevation: -3,
		Latitude:  51.040599823,
		Longitude: 2.5502800941,
		Timezone:  "Europe/Brussels",
	},
	"LFAL": {
		ICAO:      "LFAL",
		Name:      "La Fleche Thoree Les Pins Airport",
		City:      "La Fleche",
		State:     "Pays-de-la-Loire",
		Country:   "FR",
		Elevation: 121,
		Latitude:  47.6941986084,
		Longitude: 0.003333,
		Timezone:  "Europe/Paris",
	},
	"LFAM": {
		ICAO:      "LFAM",
		Name:      "Berck sur Mer Airport",
		City:      "Berck",
		State:     "Hauts-de-France",
		Country:   "FR",
		Elevation: 30,
		Latitude:  50.4230995178,
		Longitude: 1.5919400454,
		Timezone:  "Europe/Paris",
	},
	"LFAN": {
		ICAO:      "LFAN",
		Name:      "Conde/Noireau Airport",
		City:      "Conde-sur-noireau",
		State:     "Normandy",
		Country:   "FR",
		Elevation: 833,
		Latitude:  48.8917007446,
		Longitude: -0.5019440055,
		Timezone:  "Europe/Paris",
	},
	"LFAO": {
		ICAO:      "LFAO",
		Name:      "Bagnoles-de-l'Orne-Couterne Airport",
		City:      "Bagnoles-de-l'Orne/Couterne",
		State:     "Normandy",
		Country:   "FR",
		Elevation: 718,
		Latitude:  48.5457992554,
		Longitude: -0.3874439895,
		Timezone:  "Europe/Paris",
	},
	"LFAP": {
		ICAO:      "LFAP",
		Name:      "Rethel Airport",
		City:      "Rethel",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 410,
		Latitude:  49.4818992615,
		Longitude: 4.3647198677,
		Timezone:  "Europe/Paris",
	},
	"LFAQ": {
		ICAO:      "LFAQ",
		IATA:      "BYF",
		Name:      "Albert-Bray Airport",
		City:      "Albert/Bray",
		State:     "Hauts-de-France",
		Country:   "FR",
		Elevation: 364,
		Latitude:  49.9715003967,
		Longitude: 2.6976599693,
		Timezone:  "Europe/Paris",
	},
	"LFAR": {
		ICAO:      "LFAR",
		Name:      "Montdidier Airport",
		City:      "Montdidier",
		State:     "Hauts-de-France",
		Country:   "FR",
		Elevation: 358,
		Latitude:  49.6730995178,
		Longitude: 2.5691699982,
		Timezone:  "Europe/Paris",
	},
	"LFAS": {
		ICAO:      "LFAS",
		Name:      "Falaise Airport",
		City:      "Falaise",
		State:     "Normandy",
		Country:   "FR",
		Elevation: 512,
		Latitude:  48.9272003174,
		Longitude: -0.1447219998,
		Timezone:  "Europe/Paris",
	},
	"LFAT": {
		ICAO:      "LFAT",
		IATA:      "LTQ",
		Name:      "Le Touquet-Cote d'Opale Airport",
		City:      "Le Touquet-Paris-Plage",
		State:     "Hauts-de-France",
		Country:   "FR",
		Elevation: 36,
		Latitude:  50.5173988342,
		Longitude: 1.6205899715,
		Timezone:  "Europe/Paris",
	},
	"LFAU": {
		ICAO:      "LFAU",
		Name:      "Vauville Airport",
		City:      "Le Touquet",
		State:     "Normandy",
		Country:   "FR",
		Elevation: 456,
		Latitude:  49.6241989136,
		Longitude: -1.8291699886,
		Timezone:  "Europe/Paris",
	},
	"LFAV": {
		ICAO:      "LFAV",
		IATA:      "XVS",
		Name:      "Valenciennes-Denain Airport",
		City:      "Valenciennes/Denain",
		State:     "Hauts-de-France",
		Country:   "FR",
		Elevation: 177,
		Latitude:  50.3258018494,
		Longitude: 3.4612600803,
		Timezone:  "Europe/Paris",
	},
	"LFAW": {
		ICAO:      "LFAW",
		Name:      "Villerupt Airport",
		City:      "Valenciennes",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 1299,
		Latitude:  49.4113998413,
		Longitude: 5.8905601501,
		Timezone:  "Europe/Paris",
	},
	"LFAX": {
		ICAO:      "LFAX",
		Name:      "Mortagne Au Perche Airport",
		City:      "Valenciennes",
		State:     "Normandy",
		Country:   "FR",
		Elevation: 886,
		Latitude:  48.5402984619,
		Longitude: 0.5338889956,
		Timezone:  "Europe/Paris",
	},
	"LFAY": {
		ICAO:      "LFAY",
		IATA:      "QAM",
		Name:      "Amiens-Glisy Airport",
		City:      "Amiens/Glisy",
		State:     "Hauts-de-France",
		Country:   "FR",
		Elevation: 208,
		Latitude:  49.8730010986,
		Longitude: 2.3870699406,
		Timezone:  "Europe/Paris",
	},
	"LFBA": {
		ICAO:      "LFBA",
		IATA:      "AGF",
		Name:      "Agen-La Garenne Airport",
		City:      "Agen/La Garenne",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 204,
		Latitude:  44.1747016907,
		Longitude: 0.5905560255,
		Timezone:  "Europe/Paris",
	},
	"LFBC": {
		ICAO:      "LFBC",
		Name:      "Cazaux (BA 120) Air Base",
		City:      "Cazaux (La Teste-de-Buch)",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 84,
		Latitude:  44.5332984924,
		Longitude: -1.125,
		Timezone:  "Europe/Paris",
	},
	"LFBD": {
		ICAO:      "LFBD",
		IATA:      "BOD",
		Name:      "Bordeaux-Merignac (BA 106) Airport",
		City:      "Bordeaux/Merignac",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 162,
		Latitude:  44.8283004761,
		Longitude: -0.7155560255,
		Timezone:  "Europe/Paris",
	},
	"LFBE": {
		ICAO:      "LFBE",
		IATA:      "EGC",
		Name:      "Bergerac-Roumaniere Airport",
		City:      "Bergerac/Roumaniere",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 171,
		Latitude:  44.8252983093,
		Longitude: 0.5186110139,
		Timezone:  "Europe/Paris",
	},
	"LFBF": {
		ICAO:      "LFBF",
		Name:      "Toulouse-Francazal (BA 101) Air Base",
		City:      "Toulouse/Francazal",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 535,
		Latitude:  43.5456008911,
		Longitude: 1.3674999475,
		Timezone:  "Europe/Paris",
	},
	"LFBG": {
		ICAO:      "LFBG",
		IATA:      "CNG",
		Name:      "Cognac-Chateaubernard (BA 709) Air Base",
		City:      "Cognac/Chateaubernard",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 102,
		Latitude:  45.6582984924,
		Longitude: -0.3174999952,
		Timezone:  "Europe/Paris",
	},
	"LFBH": {
		ICAO:      "LFBH",
		IATA:      "LRH",
		Name:      "La Rochelle-Ile de Re Airport",
		City:      "La Rochelle/Ile de Re",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 74,
		Latitude:  46.1791992188,
		Longitude: -1.1952799559,
		Timezone:  "Europe/Paris",
	},
	"LFBI": {
		ICAO:      "LFBI",
		IATA:      "PIS",
		Name:      "Poitiers-Biard Airport",
		City:      "Poitiers/Biard",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 423,
		Latitude:  46.5876998901,
		Longitude: 0.3066659868,
		Timezone:  "Europe/Paris",
	},
	"LFBJ": {
		ICAO:      "LFBJ",
		Name:      "St Junien Airport",
		City:      "Poitiers",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 902,
		Latitude:  45.903301239,
		Longitude: 0.9200000167,
		Timezone:  "Europe/Paris",
	},
	"LFBK": {
		ICAO:      "LFBK",
		IATA:      "MCU",
		Name:      "Montlucon-Gueret Airport",
		City:      "Montlucon/Gueret",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 1497,
		Latitude:  46.2225990295,
		Longitude: 2.3639600277,
		Timezone:  "Europe/Paris",
	},
	"LFBL": {
		ICAO:      "LFBL",
		IATA:      "LIG",
		Name:      "Limoges Airport",
		City:      "Limoges/Bellegarde",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 1300,
		Latitude:  45.8628005981,
		Longitude: 1.1794400215,
		Timezone:  "Europe/Paris",
	},
	"LFBM": {
		ICAO:      "LFBM",
		IATA:      "XMJ",
		Name:      "Mont-de-Marsan (BA 118) Air Base",
		City:      "Mont-de-Marsan",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 203,
		Latitude:  43.9117012024,
		Longitude: -0.5074999928,
		Timezone:  "Europe/Paris",
	},
	"LFBN": {
		ICAO:      "LFBN",
		IATA:      "NIT",
		Name:      "Niort-Souche Airport",
		City:      "Niort/Souche",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 203,
		Latitude:  46.3112983704,
		Longitude: -0.4015029967,
		Timezone:  "Europe/Paris",
	},
	"LFBO": {
		ICAO:      "LFBO",
		IATA:      "TLS",
		Name:      "Toulouse-Blagnac Airport",
		City:      "Toulouse/Blagnac",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 499,
		Latitude:  43.6291007996,
		Longitude: 1.3638199568,
		Timezone:  "Europe/Paris",
	},
	"LFBP": {
		ICAO:      "LFBP",
		IATA:      "PUF",
		Name:      "Pau Pyrenees Airport",
		City:      "Pau/Pyrenees (Uzein)",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 616,
		Latitude:  43.3800010681,
		Longitude: -0.41861099,
		Timezone:  "Europe/Paris",
	},
	"LFBR": {
		ICAO:      "LFBR",
		Name:      "Muret-Lherm Airport",
		City:      "Muret/Lherm",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 622,
		Latitude:  43.4488983154,
		Longitude: 1.2633299828,
		Timezone:  "Europe/Paris",
	},
	"LFBS": {
		ICAO:      "LFBS",
		Name:      "Biscarrosse Parentis Airport",
		City:      "Muret",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 98,
		Latitude:  44.3694000244,
		Longitude: -1.1305600405,
		Timezone:  "Europe/Paris",
	},
	"LFBT": {
		ICAO:      "LFBT",
		IATA:      "LDE",
		Name:      "Tarbes-Lourdes-Pyrenees Airport",
		City:      "Tarbes/Lourdes/Pyrenees",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 1260,
		Latitude:  43.1786994934,
		Longitude: -0.0064389999,
		Timezone:  "Europe/Paris",
	},
	"LFBU": {
		ICAO:      "LFBU",
		IATA:      "ANG",
		Name:      "Angouleme-Brie-Champniers Airport",
		City:      "Angouleme/Brie/Champniers",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 436,
		Latitude:  45.7291984558,
		Longitude: 0.2214560062,
		Timezone:  "Europe/Paris",
	},
	"LFBV": {
		ICAO:      "LFBV",
		IATA:      "BVE",
		Name:      "Brive-La Roche Airport",
		City:      "Brive-la-Gaillarde",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 379,
		Latitude:  45.1507987976,
		Longitude: 1.4691699743,
		Timezone:  "Europe/Paris",
	},
	"LFBX": {
		ICAO:      "LFBX",
		IATA:      "PGX",
		Name:      "Perigueux-Bassillac Airport",
		City:      "Perigueux/Bassillac",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 328,
		Latitude:  45.1981010437,
		Longitude: 0.8155559897,
		Timezone:  "Europe/Paris",
	},
	"LFBY": {
		ICAO:      "LFBY",
		IATA:      "XDA",
		Name:      "Dax Seyresse Airport",
		City:      "Perigueux",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 106,
		Latitude:  43.689201355,
		Longitude: -1.0688899755,
		Timezone:  "Europe/Paris",
	},
	"LFBZ": {
		ICAO:      "LFBZ",
		IATA:      "BIQ",
		Name:      "Biarritz-Anglet-Bayonne Airport",
		City:      "Biarritz/Anglet/Bayonne",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 245,
		Latitude:  43.4683990479,
		Longitude: -1.5233199596,
		Timezone:  "Europe/Paris",
	},
	"LFCA": {
		ICAO:      "LFCA",
		IATA:      "XCX",
		Name:      "Chatellerault Airport",
		City:      "Biarritz",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 207,
		Latitude:  46.7813987732,
		Longitude: 0.5519440174,
		Timezone:  "Europe/Paris",
	},
	"LFCB": {
		ICAO:      "LFCB",
		Name:      "Bagneres De Luchon Airport",
		City:      "Biarritz",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 2028,
		Latitude:  42.7999992371,
		Longitude: 0.6000000238,
		Timezone:  "Europe/Paris",
	},
	"LFCC": {
		ICAO:      "LFCC",
		IATA:      "ZAO",
		Name:      "Cahors-Lalbenque Airport",
		City:      "Cahors/Lalbenque",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 912,
		Latitude:  44.351398468,
		Longitude: 1.4752800465,
		Timezone:  "Europe/Paris",
	},
	"LFCD": {
		ICAO:      "LFCD",
		Name:      "Andernos Les Bains Airport",
		City:      "Cahors",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 66,
		Latitude:  44.7560997009,
		Longitude: -1.0633300543,
		Timezone:  "Europe/Paris",
	},
	"LFCE": {
		ICAO:      "LFCE",
		IATA:      "XGT",
		Name:      "Gueret St Laurent Airport",
		City:      "Cahors",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 1207,
		Latitude:  46.1791992188,
		Longitude: 1.9583300352,
		Timezone:  "Europe/Paris",
	},
	"LFCF": {
		ICAO:      "LFCF",
		Name:      "Figeac Livernon Airport",
		City:      "Cahors",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 1086,
		Latitude:  44.6733016968,
		Longitude: 1.7891700268,
		Timezone:  "Europe/Paris",
	},
	"LFCG": {
		ICAO:      "LFCG",
		Name:      "Saint-Girons-Antichan Airport",
		City:      "Saint-Girons/Antichan",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 1368,
		Latitude:  43.0078010559,
		Longitude: 1.1031500101,
		Timezone:  "Europe/Paris",
	},
	"LFCH": {
		ICAO:      "LFCH",
		IATA:      "XAC",
		Name:      "Arcachon-La Teste-de-Buch Airport",
		City:      "Arcachon/La Teste-de-Buch",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 49,
		Latitude:  44.5964012146,
		Longitude: -1.1108299494,
		Timezone:  "Europe/Paris",
	},
	"LFCI": {
		ICAO:      "LFCI",
		IATA:      "LBI",
		Name:      "Albi-Le Sequestre Airport",
		City:      "Albi/Le Sequestre",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 564,
		Latitude:  43.913898468,
		Longitude: 2.1130599976,
		Timezone:  "Europe/Paris",
	},
	"LFCJ": {
		ICAO:      "LFCJ",
		Name:      "Jonzac Neulles Airport",
		City:      "Albi",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 128,
		Latitude:  45.4841995239,
		Longitude: -0.4213890135,
		Timezone:  "Europe/Paris",
	},
	"LFCK": {
		ICAO:      "LFCK",
		IATA:      "DCM",
		Name:      "Castres-Mazamet Airport",
		City:      "Castres/Mazamet",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 788,
		Latitude:  43.5563011169,
		Longitude: 2.2891800404,
		Timezone:  "Europe/Paris",
	},
	"LFCL": {
		ICAO:      "LFCL",
		Name:      "Toulouse-Lasbordes Airport",
		City:      "Toulouse/Lasbordes",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 459,
		Latitude:  43.586101532,
		Longitude: 1.4991699457,
		Timezone:  "Europe/Paris",
	},
	"LFCM": {
		ICAO:      "LFCM",
		Name:      "Millau-Larzac Airport",
		City:      "Millau/Larzac",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 2606,
		Latitude:  43.9892997742,
		Longitude: 3.1830000877,
		Timezone:  "Europe/Paris",
	},
	"LFCN": {
		ICAO:      "LFCN",
		Name:      "Nogaro Airport",
		City:      "Millau",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 302,
		Latitude:  43.7696990967,
		Longitude: -0.0327779986,
		Timezone:  "Europe/Paris",
	},
	"LFCO": {
		ICAO:      "LFCO",
		Name:      "Oloron Herrere Airport",
		City:      "Millau",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 984,
		Latitude:  43.1646995544,
		Longitude: -0.5602779984,
		Timezone:  "Europe/Paris",
	},
	"LFCP": {
		ICAO:      "LFCP",
		Name:      "Pons Avy Airport",
		City:      "Millau",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 115,
		Latitude:  45.5699996948,
		Longitude: -0.5149999857,
		Timezone:  "Europe/Paris",
	},
	"LFCQ": {
		ICAO:      "LFCQ",
		Name:      "Graulhet-Montdragon Airport",
		City:      "Graulhet/Montdragon",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 581,
		Latitude:  43.7710990906,
		Longitude: 2.0108299255,
		Timezone:  "Europe/Paris",
	},
	"LFCR": {
		ICAO:      "LFCR",
		IATA:      "RDZ",
		Name:      "Rodez-Marcillac Airport",
		City:      "Rodez/Marcillac",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 1910,
		Latitude:  44.4079017639,
		Longitude: 2.4826700687,
		Timezone:  "Europe/Paris",
	},
	"LFCS": {
		ICAO:      "LFCS",
		Name:      "Bordeaux Leognan Saucats Airport",
		City:      "Rodez",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 190,
		Latitude:  44.7002983093,
		Longitude: -0.5955560207,
		Timezone:  "Europe/Paris",
	},
	"LFCT": {
		ICAO:      "LFCT",
		Name:      "Thouars Airport",
		City:      "Rodez",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 341,
		Latitude:  46.9618988037,
		Longitude: -0.1527779996,
		Timezone:  "Europe/Paris",
	},
	"LFCU": {
		ICAO:      "LFCU",
		Name:      "Ussel-Thalamy Airport",
		City:      "Ussel/Thalamy",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 2428,
		Latitude:  45.5346984863,
		Longitude: 2.4238901138,
		Timezone:  "Europe/Paris",
	},
	"LFCV": {
		ICAO:      "LFCV",
		Name:      "Villefranche De Rouergue Airport",
		City:      "Rodez",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 1096,
		Latitude:  44.3699989319,
		Longitude: 2.0280599594,
		Timezone:  "Europe/Paris",
	},
	"LFCW": {
		ICAO:      "LFCW",
		Name:      "Villeneuve-sur-Lot Airport",
		City:      "Villeneuve-sur-Lot",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 190,
		Latitude:  44.396900177,
		Longitude: 0.7588890195,
		Timezone:  "Europe/Paris",
	},
	"LFCX": {
		ICAO:      "LFCX",
		Name:      "Castelsarrasin Moissac Airport",
		City:      "Rodez",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 243,
		Latitude:  44.0868988037,
		Longitude: 1.1283299923,
		Timezone:  "Europe/Paris",
	},
	"LFCY": {
		ICAO:      "LFCY",
		IATA:      "RYN",
		Name:      "Royan-Medis Airport",
		City:      "Royan/Medis",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 72,
		Latitude:  45.6281013489,
		Longitude: -0.9725000262,
		Timezone:  "Europe/Paris",
	},
	"LFCZ": {
		ICAO:      "LFCZ",
		Name:      "Mimizan Airport",
		City:      "Mimizan",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 164,
		Latitude:  44.1460990906,
		Longitude: -1.1744400263,
		Timezone:  "Europe/Paris",
	},
	"LFDA": {
		ICAO:      "LFDA",
		Name:      "Aire-sur-l'Adour Airport",
		City:      "Aire-sur-Adour",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 259,
		Latitude:  43.709400177,
		Longitude: -0.2452780008,
		Timezone:  "Europe/Paris",
	},
	"LFDB": {
		ICAO:      "LFDB",
		IATA:      "XMW",
		Name:      "Montauban Airport",
		City:      "Montauban",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 351,
		Latitude:  44.0256996155,
		Longitude: 1.3780399561,
		Timezone:  "Europe/Paris",
	},
	"LFDC": {
		ICAO:      "LFDC",
		Name:      "Montendre Marcillac Airport",
		City:      "Royan",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 148,
		Latitude:  45.2743988037,
		Longitude: -0.4522219896,
		Timezone:  "Europe/Paris",
	},
	"LFDE": {
		ICAO:      "LFDE",
		Name:      "Egletons Airport",
		City:      "Royan",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 1857,
		Latitude:  45.4213981628,
		Longitude: 2.0688900948,
		Timezone:  "Europe/Paris",
	},
	"LFDF": {
		ICAO:      "LFDF",
		Name:      "Ste Foy La Grande Airport",
		City:      "Royan",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 282,
		Latitude:  44.8535995483,
		Longitude: 0.1766670048,
		Timezone:  "Europe/Paris",
	},
	"LFDG": {
		ICAO:      "LFDG",
		Name:      "Gaillac Lisle Sur Tarn Airport",
		City:      "Royan",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 446,
		Latitude:  43.8838996887,
		Longitude: 1.8755600452,
		Timezone:  "Europe/Paris",
	},
	"LFDH": {
		ICAO:      "LFDH",
		Name:      "Auch-Lamothe Airport",
		City:      "Auch/Lamothe",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 411,
		Latitude:  43.6878013611,
		Longitude: 0.6016669869,
		Timezone:  "Europe/Paris",
	},
	"LFDI": {
		ICAO:      "LFDI",
		IATA:      "XLR",
		Name:      "Libourne-Artigues-de-Lussac Airport",
		City:      "Libourne/Artigues-de-Lussac",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 157,
		Latitude:  44.9824981689,
		Longitude: -0.1347219944,
		Timezone:  "Europe/Paris",
	},
	"LFDJ": {
		ICAO:      "LFDJ",
		Name:      "Pamiers-Les Pujols Airport",
		City:      "Pamiers/Les Pujols",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 1115,
		Latitude:  43.0905990601,
		Longitude: 1.6958299875,
		Timezone:  "Europe/Paris",
	},
	"LFDK": {
		ICAO:      "LFDK",
		Name:      "Soulac Sur Mer Airport",
		City:      "Pamiers",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 7,
		Latitude:  45.4949989319,
		Longitude: -1.0822199583,
		Timezone:  "Europe/Paris",
	},
	"LFDL": {
		ICAO:      "LFDL",
		Name:      "Loudun Airport",
		City:      "Pamiers",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 315,
		Latitude:  47.0372009277,
		Longitude: 0.1013889983,
		Timezone:  "Europe/Paris",
	},
	"LFDM": {
		ICAO:      "LFDM",
		Name:      "Marmande-Virazeil Airport",
		City:      "Marmande/Virazeil",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 105,
		Latitude:  44.4989013672,
		Longitude: 0.2005140036,
		Timezone:  "Europe/Paris",
	},
	"LFDN": {
		ICAO:      "LFDN",
		IATA:      "RCO",
		Name:      "Rochefort-Saint-Agnant (BA 721) Airport",
		City:      "Rochefort/Saint-Agnant",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 60,
		Latitude:  45.8877983093,
		Longitude: -0.9830560088,
		Timezone:  "Europe/Paris",
	},
	"LFDO": {
		ICAO:      "LFDO",
		Name:      "Bordeaux Souge Airport",
		City:      "Rochefort",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 135,
		Latitude:  44.8499984741,
		Longitude: -0.8000000119,
		Timezone:  "Europe/Paris",
	},
	"LFDP": {
		ICAO:      "LFDP",
		Name:      "St Pierre D'oleron Airport",
		City:      "Rochefort",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 20,
		Latitude:  45.9592018127,
		Longitude: -1.3161100149,
		Timezone:  "Europe/Paris",
	},
	"LFDQ": {
		ICAO:      "LFDQ",
		Name:      "Castelnau Magnoac Airport",
		City:      "Rochefort",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 984,
		Latitude:  43.2793998718,
		Longitude: 0.5216670036,
		Timezone:  "Europe/Paris",
	},
	"LFDR": {
		ICAO:      "LFDR",
		Name:      "La Reole Floudes Airport",
		City:      "Rochefort",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 43,
		Latitude:  44.5680999756,
		Longitude: -0.0561110005,
		Timezone:  "Europe/Paris",
	},
	"LFDS": {
		ICAO:      "LFDS",
		IATA:      "XSL",
		Name:      "Sarlat Domme Airport",
		City:      "Rochefort",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 978,
		Latitude:  44.7933006287,
		Longitude: 1.2447199821,
		Timezone:  "Europe/Paris",
	},
	"LFDT": {
		ICAO:      "LFDT",
		IATA:      "XTB",
		Name:      "Tarbes Laloubere Airport",
		City:      "Rochefort",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 1076,
		Latitude:  43.2160987854,
		Longitude: 0.0786110014,
		Timezone:  "Europe/Paris",
	},
	"LFDU": {
		ICAO:      "LFDU",
		Name:      "Lesparre St Laurent Medoc Airport",
		City:      "Rochefort",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 105,
		Latitude:  45.1977996826,
		Longitude: -0.8822219968,
		Timezone:  "Europe/Paris",
	},
	"LFDV": {
		ICAO:      "LFDV",
		Name:      "Couhe Verac Airport",
		City:      "Rochefort",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 499,
		Latitude:  46.2728004456,
		Longitude: 0.1905560046,
		Timezone:  "Europe/Paris",
	},
	"LFDW": {
		ICAO:      "LFDW",
		Name:      "Chauvigny Airport",
		City:      "Rochefort",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 440,
		Latitude:  46.5835990906,
		Longitude: 0.6424999833,
		Timezone:  "Europe/Paris",
	},
	"LFDX": {
		ICAO:      "LFDX",
		Name:      "Fumel Montayral Airport",
		City:      "Rochefort",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 692,
		Latitude:  44.4636001587,
		Longitude: 1.0077799559,
		Timezone:  "Europe/Paris",
	},
	"LFDY": {
		ICAO:      "LFDY",
		Name:      "Bordeaux Yvrac Airport",
		City:      "Rochefort",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 240,
		Latitude:  44.8772010803,
		Longitude: -0.4791670144,
		Timezone:  "Europe/Paris",
	},
	"LFDZ": {
		ICAO:      "LFDZ",
		Name:      "Condat-Sur-Vezere Airport",
		City:      "Condat-Sur-Vezere",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 499,
		Latitude:  45.0999984741,
		Longitude: 1.2170000076,
		Timezone:  "Europe/Paris",
	},
	"LFEA": {
		ICAO:      "LFEA",
		Name:      "Belle Ile Airport",
		City:      "Rochefort",
		State:     "Brittany",
		Country:   "FR",
		Elevation: 171,
		Latitude:  47.3266983032,
		Longitude: -3.1983299255,
		Timezone:  "Europe/Paris",
	},
	"LFEB": {
		ICAO:      "LFEB",
		Name:      "Dinan - Trelivan Airport",
		City:      "Dinan",
		State:     "Brittany",
		Country:   "FR",
		Elevation: 394,
		Latitude:  48.4444007874,
		Longitude: -2.1013898849,
		Timezone:  "Europe/Paris",
	},
	"LFEC": {
		ICAO:      "LFEC",
		Name:      "Ouessant Airport",
		City:      "Ushant",
		State:     "Brittany",
		Country:   "FR",
		Elevation: 142,
		Latitude:  48.4631996155,
		Longitude: -5.0635800362,
		Timezone:  "Europe/Paris",
	},
	"LFED": {
		ICAO:      "LFED",
		Name:      "Pontivy Airport",
		City:      "Pontivy",
		State:     "Brittany",
		Country:   "FR",
		Elevation: 407,
		Latitude:  48.0584983826,
		Longitude: -2.9218299389,
		Timezone:  "Europe/Paris",
	},
	"LFEF": {
		ICAO:      "LFEF",
		Name:      "Amboise Dierre Airport",
		City:      "Pontivy",
		State:     "Centre",
		Country:   "FR",
		Elevation: 180,
		Latitude:  47.3414001465,
		Longitude: 0.9424999952,
		Timezone:  "Europe/Paris",
	},
	"LFEG": {
		ICAO:      "LFEG",
		Name:      "Argenton Sur Creuse Airport",
		City:      "Pontivy",
		State:     "Centre",
		Country:   "FR",
		Elevation: 663,
		Latitude:  46.5969009399,
		Longitude: 1.6024999619,
		Timezone:  "Europe/Paris",
	},
	"LFEH": {
		ICAO:      "LFEH",
		Name:      "Aubigny-sur-Nere Airport",
		City:      "Aubigny-sur-Nere",
		State:     "Centre",
		Country:   "FR",
		Elevation: 630,
		Latitude:  47.4742012024,
		Longitude: 2.3930499554,
		Timezone:  "Europe/Paris",
	},
	"LFEI": {
		ICAO:      "LFEI",
		Name:      "Briare Chatillon Airport",
		City:      "Pontivy",
		State:     "Centre",
		Country:   "FR",
		Elevation: 539,
		Latitude:  47.6143989563,
		Longitude: 2.7819399834,
		Timezone:  "Europe/Paris",
	},
	"LFEJ": {
		ICAO:      "LFEJ",
		Name:      "Chateauroux Villers Airport",
		City:      "Pontivy",
		State:     "Centre",
		Country:   "FR",
		Elevation: 541,
		Latitude:  46.8418998718,
		Longitude: 1.6211099625,
		Timezone:  "Europe/Paris",
	},
	"LFEK": {
		ICAO:      "LFEK",
		Name:      "Issoudun Le Fay Airport",
		City:      "Pontivy",
		State:     "Centre",
		Country:   "FR",
		Elevation: 531,
		Latitude:  46.8885993958,
		Longitude: 2.0413899422,
		Timezone:  "Europe/Paris",
	},
	"LFEL": {
		ICAO:      "LFEL",
		Name:      "Le Blanc Airport",
		City:      "Pontivy",
		State:     "Centre",
		Country:   "FR",
		Elevation: 377,
		Latitude:  46.6208000183,
		Longitude: 1.0874999762,
		Timezone:  "Europe/Paris",
	},
	"LFEM": {
		ICAO:      "LFEM",
		Name:      "Montargis Vimory Airport",
		City:      "Pontivy",
		State:     "Centre",
		Country:   "FR",
		Elevation: 308,
		Latitude:  47.9606018066,
		Longitude: 2.6858301163,
		Timezone:  "Europe/Paris",
	},
	"LFEN": {
		ICAO:      "LFEN",
		Name:      "Tours Sorigny Airport",
		City:      "Pontivy",
		State:     "Centre",
		Country:   "FR",
		Elevation: 299,
		Latitude:  47.2675018311,
		Longitude: 0.7011110187,
		Timezone:  "Europe/Paris",
	},
	"LFEO": {
		ICAO:      "LFEO",
		Name:      "St Malo St Serva Airport",
		State:     "Brittany",
		Country:   "FR",
		Elevation: 151,
		Latitude:  48.6136016846,
		Longitude: -1.9736100435,
		Timezone:  "Europe/Paris",
	},
	"LFEP": {
		ICAO:      "LFEP",
		Name:      "Pouilly Maconge Airport",
		City:      "Pontivy",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 1411,
		Latitude:  47.2214012146,
		Longitude: 4.5611100197,
		Timezone:  "Europe/Paris",
	},
	"LFEQ": {
		ICAO:      "LFEQ",
		Name:      "Quiberon Airport",
		City:      "Pontivy",
		State:     "Brittany",
		Country:   "FR",
		Elevation: 37,
		Latitude:  47.4822006226,
		Longitude: -3.0999999046,
		Timezone:  "Europe/Paris",
	},
	"LFER": {
		ICAO:      "LFER",
		Name:      "Redon Bains-sur-Oust Airport",
		City:      "Redon",
		State:     "Brittany",
		Country:   "FR",
		Elevation: 223,
		Latitude:  47.6994018555,
		Longitude: -2.0366699696,
		Timezone:  "Europe/Paris",
	},
	"LFES": {
		ICAO:      "LFES",
		Name:      "Guiscriff Scaer Airport",
		City:      "Guiscriff",
		State:     "Brittany",
		Country:   "FR",
		Elevation: 574,
		Latitude:  48.0525016785,
		Longitude: -3.6647200584,
		Timezone:  "Europe/Paris",
	},
	"LFET": {
		ICAO:      "LFET",
		Name:      "Til Chatel Airport",
		City:      "Guiscriff",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 938,
		Latitude:  47.5475006104,
		Longitude: 5.2119398117,
		Timezone:  "Europe/Paris",
	},
	"LFEU": {
		ICAO:      "LFEU",
		Name:      "Bar Le Duc Airport",
		City:      "Guiscriff",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 909,
		Latitude:  48.8683013916,
		Longitude: 5.1858301163,
		Timezone:  "Europe/Paris",
	},
	"LFEV": {
		ICAO:      "LFEV",
		Name:      "Gray St Adrien Airport",
		City:      "Guiscriff",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 679,
		Latitude:  47.4333000183,
		Longitude: 5.6227798462,
		Timezone:  "Europe/Paris",
	},
	"LFEW": {
		ICAO:      "LFEW",
		Name:      "Saulieu Liernais Airport",
		City:      "Guiscriff",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 1722,
		Latitude:  47.2393989563,
		Longitude: 4.26583004,
		Timezone:  "Europe/Paris",
	},
	"LFEX": {
		ICAO:      "LFEX",
		Name:      "Nancy Azelot Airport",
		City:      "Guiscriff",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 961,
		Latitude:  48.5928001404,
		Longitude: 6.241109848,
		Timezone:  "Europe/Paris",
	},
	"LFEY": {
		ICAO:      "LFEY",
		IATA:      "IDY",
		Name:      "Ile d'Yeu Airport",
		City:      "Ile d'Yeu",
		State:     "Pays-de-la-Loire",
		Country:   "FR",
		Elevation: 79,
		Latitude:  46.7186012268,
		Longitude: -2.3911099434,
		Timezone:  "Europe/Paris",
	},
	"LFEZ": {
		ICAO:      "LFEZ",
		Name:      "Nancy Malzeville Airport",
		City:      "Guiscriff",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 1253,
		Latitude:  48.7243995667,
		Longitude: 6.2077798843,
		Timezone:  "Europe/Paris",
	},
	"LFFB": {
		ICAO:      "LFFB",
		Name:      "Buno Bonnevaux Airport",
		City:      "Guiscriff",
		State:     "Ile-de-France",
		Country:   "FR",
		Elevation: 420,
		Latitude:  48.3511009216,
		Longitude: 2.4255599976,
		Timezone:  "Europe/Paris",
	},
	"LFFC": {
		ICAO:      "LFFC",
		Name:      "Mantes Cherence Airport",
		City:      "Guiscriff",
		State:     "Ile-de-France",
		Country:   "FR",
		Elevation: 512,
		Latitude:  49.0788993835,
		Longitude: 1.6897200346,
		Timezone:  "Europe/Paris",
	},
	"LFFD": {
		ICAO:      "LFFD",
		Name:      "St Andre De L'eure Airport",
		City:      "Guiscriff",
		State:     "Normandy",
		Country:   "FR",
		Elevation: 489,
		Latitude:  48.898601532,
		Longitude: 1.2505600452,
		Timezone:  "Europe/Paris",
	},
	"LFFE": {
		ICAO:      "LFFE",
		Name:      "Enghien Moisselles Airport",
		City:      "Guiscriff",
		State:     "Ile-de-France",
		Country:   "FR",
		Elevation: 335,
		Latitude:  49.0463981628,
		Longitude: 2.3530600071,
		Timezone:  "Europe/Paris",
	},
	"LFFG": {
		ICAO:      "LFFG",
		Name:      "La Ferte Gaucher Airport",
		City:      "Guiscriff",
		State:     "Ile-de-France",
		Country:   "FR",
		Elevation: 538,
		Latitude:  48.7557983398,
		Longitude: 3.2766699791,
		Timezone:  "Europe/Paris",
	},
	"LFFH": {
		ICAO:      "LFFH",
		Name:      "Chateau-Thierry - Belleau Airport",
		City:      "Guiscriff",
		State:     "Hauts-de-France",
		Country:   "FR",
		Elevation: 728,
		Latitude:  49.067199707,
		Longitude: 3.3569400311,
		Timezone:  "Europe/Paris",
	},
	"LFFI": {
		ICAO:      "LFFI",
		Name:      "Ancenis Airport",
		City:      "Ancenis",
		State:     "Pays-de-la-Loire",
		Country:   "FR",
		Elevation: 111,
		Latitude:  47.4081001282,
		Longitude: -1.1775000095,
		Timezone:  "Europe/Paris",
	},
	"LFFJ": {
		ICAO:      "LFFJ",
		Name:      "Joinville Mussey Airport",
		City:      "Guiscriff",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 1024,
		Latitude:  48.386100769,
		Longitude: 5.1449999809,
		Timezone:  "Europe/Paris",
	},
	"LFFK": {
		ICAO:      "LFFK",
		Name:      "Fontenay Le Comte Airport",
		City:      "Guiscriff",
		State:     "Pays-de-la-Loire",
		Country:   "FR",
		Elevation: 85,
		Latitude:  46.4413986206,
		Longitude: -0.7927780151,
		Timezone:  "Europe/Paris",
	},
	"LFFL": {
		ICAO:      "LFFL",
		Name:      "Bailleau Armenonville Airport",
		City:      "Guiscriff",
		State:     "Centre",
		Country:   "FR",
		Elevation: 509,
		Latitude:  48.5158004761,
		Longitude: 1.6399999857,
		Timezone:  "Europe/Paris",
	},
	"LFFM": {
		ICAO:      "LFFM",
		Name:      "Lamotte Beuvron Airport",
		City:      "Guiscriff",
		State:     "Centre",
		Country:   "FR",
		Elevation: 413,
		Latitude:  47.6567001343,
		Longitude: 1.9891699553,
		Timezone:  "Europe/Paris",
	},
	"LFFN": {
		ICAO:      "LFFN",
		Name:      "Brienne-le-Chateau Airport",
		City:      "Brienne-le-Chateau",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 381,
		Latitude:  48.4297981262,
		Longitude: 4.4822201729,
		Timezone:  "Europe/Paris",
	},
	"LFFP": {
		ICAO:      "LFFP",
		Name:      "Pithiviers Airport",
		City:      "Guiscriff",
		State:     "Centre",
		Country:   "FR",
		Elevation: 384,
		Latitude:  48.1571998596,
		Longitude: 2.1925001144,
		Timezone:  "Europe/Paris",
	},
	"LFFQ": {
		ICAO:      "LFFQ",
		Name:      "La Ferte Alais Airport",
		City:      "Guiscriff",
		State:     "Ile-de-France",
		Country:   "FR",
		Elevation: 453,
		Latitude:  48.4977989197,
		Longitude: 2.3433299065,
		Timezone:  "Europe/Paris",
	},
	"LFFR": {
		ICAO:      "LFFR",
		Name:      "Bar Sur Seine Airport",
		City:      "Guiscriff",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 938,
		Latitude:  48.0668983459,
		Longitude: 4.4136099815,
		Timezone:  "Europe/Paris",
	},
	"LFFT": {
		ICAO:      "LFFT",
		Name:      "Neufchateau Airport",
		City:      "Guiscriff",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 1224,
		Latitude:  48.3624992371,
		Longitude: 5.7213897705,
		Timezone:  "Europe/Paris",
	},
	"LFFU": {
		ICAO:      "LFFU",
		Name:      "Chateauneuf Sur Cher Airport",
		City:      "Guiscriff",
		State:     "Centre",
		Country:   "FR",
		Elevation: 551,
		Latitude:  46.8711013794,
		Longitude: 2.376940012,
		Timezone:  "Europe/Paris",
	},
	"LFFV": {
		ICAO:      "LFFV",
		Name:      "Vierzon Mereau Airport",
		City:      "Guiscriff",
		State:     "Centre",
		Country:   "FR",
		Elevation: 430,
		Latitude:  47.1946983337,
		Longitude: 2.0666699409,
		Timezone:  "Europe/Paris",
	},
	"LFFW": {
		ICAO:      "LFFW",
		Name:      "Montaigu St Georges Airport",
		City:      "Guiscriff",
		State:     "Pays-de-la-Loire",
		Country:   "FR",
		Elevation: 184,
		Latitude:  46.9331016541,
		Longitude: -1.3255599737,
		Timezone:  "Europe/Paris",
	},
	"LFFX": {
		ICAO:      "LFFX",
		Name:      "Tournus Cuisery Airport",
		City:      "Guiscriff",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 682,
		Latitude:  46.5628013611,
		Longitude: 4.9766697884,
		Timezone:  "Europe/Paris",
	},
	"LFFY": {
		ICAO:      "LFFY",
		Name:      "Etrepagny Airport",
		City:      "Guiscriff",
		State:     "Normandy",
		Country:   "FR",
		Elevation: 308,
		Latitude:  49.306098938,
		Longitude: 1.6386100054,
		Timezone:  "Europe/Paris",
	},
	"LFFZ": {
		ICAO:      "LFFZ",
		Name:      "Sezanne St Remy Airport",
		City:      "Guiscriff",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 354,
		Latitude:  48.7106018066,
		Longitude: 3.7641699314,
		Timezone:  "Europe/Paris",
	},
	"LFGA": {
		ICAO:      "LFGA",
		IATA:      "CMR",
		Name:      "Colmar-Houssen Airport",
		City:      "Colmar/Houssen",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 628,
		Latitude:  48.1099014282,
		Longitude: 7.3590102196,
		Timezone:  "Europe/Paris",
	},
	"LFGB": {
		ICAO:      "LFGB",
		Name:      "Mulhouse-Habsheim Airport",
		City:      "Mulhouse/Habsheim",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 788,
		Latitude:  47.7412986755,
		Longitude: 7.4322099686,
		Timezone:  "Europe/Paris",
	},
	"LFGC": {
		ICAO:      "LFGC",
		Name:      "Strasbourg Neuhof Airport",
		City:      "Colmar",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 456,
		Latitude:  48.5544013977,
		Longitude: 7.7780599594,
		Timezone:  "Europe/Paris",
	},
	"LFGD": {
		ICAO:      "LFGD",
		Name:      "Arbois Airport",
		City:      "Colmar",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 876,
		Latitude:  46.9199981689,
		Longitude: 5.7600002289,
		Timezone:  "Europe/Paris",
	},
	"LFGE": {
		ICAO:      "LFGE",
		Name:      "Avallon Airport",
		City:      "Colmar",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 781,
		Latitude:  47.5031013489,
		Longitude: 3.8994400501,
		Timezone:  "Europe/Paris",
	},
	"LFGF": {
		ICAO:      "LFGF",
		IATA:      "XBV",
		Name:      "Beaune-Challanges Airport",
		City:      "Beaune/Challanges",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 656,
		Latitude:  47.0059013367,
		Longitude: 4.8934202194,
		Timezone:  "Europe/Paris",
	},
	"LFGG": {
		ICAO:      "LFGG",
		Name:      "Belfort Chaux Airport",
		City:      "Colmar",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 1368,
		Latitude:  47.7022018433,
		Longitude: 6.8324999809,
		Timezone:  "Europe/Paris",
	},
	"LFGH": {
		ICAO:      "LFGH",
		Name:      "Cosne Sur Loire Airport",
		City:      "Colmar",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 581,
		Latitude:  47.3605995178,
		Longitude: 2.9194400311,
		Timezone:  "Europe/Paris",
	},
	"LFGI": {
		ICAO:      "LFGI",
		Name:      "Dijon Darois Airport",
		City:      "Colmar",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 1585,
		Latitude:  47.3869018555,
		Longitude: 4.9480600357,
		Timezone:  "Europe/Paris",
	},
	"LFGJ": {
		ICAO:      "LFGJ",
		IATA:      "DLE",
		Name:      "Dole-Tavaux Airport",
		City:      "Dole/Tavaux",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 645,
		Latitude:  47.0390014648,
		Longitude: 5.4272499084,
		Timezone:  "Europe/Paris",
	},
	"LFGK": {
		ICAO:      "LFGK",
		Name:      "Joigny Airport",
		City:      "Joigny",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 732,
		Latitude:  47.9921989441,
		Longitude: 3.3922200203,
		Timezone:  "Europe/Paris",
	},
	"LFGL": {
		ICAO:      "LFGL",
		Name:      "Lons Le Saulnier Courlaoux Airport",
		City:      "Dole",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 761,
		Latitude:  46.6761016846,
		Longitude: 5.4711098671,
		Timezone:  "Europe/Paris",
	},
	"LFGM": {
		ICAO:      "LFGM",
		Name:      "Montceau Les Mines Airport",
		City:      "Dole",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 1030,
		Latitude:  46.6041984558,
		Longitude: 4.3338899612,
		Timezone:  "Europe/Paris",
	},
	"LFGN": {
		ICAO:      "LFGN",
		Name:      "Paray Le Monial Airport",
		City:      "Dole",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 997,
		Latitude:  46.4678001404,
		Longitude: 4.1350002289,
		Timezone:  "Europe/Paris",
	},
	"LFGO": {
		ICAO:      "LFGO",
		Name:      "Pont Sur Yonne Airport",
		City:      "Dole",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 236,
		Latitude:  48.290599823,
		Longitude: 3.2508299351,
		Timezone:  "Europe/Paris",
	},
	"LFGP": {
		ICAO:      "LFGP",
		Name:      "St Florentin Cheu Airport",
		City:      "Dole",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 351,
		Latitude:  47.9822006226,
		Longitude: 3.7783300877,
		Timezone:  "Europe/Paris",
	},
	"LFGQ": {
		ICAO:      "LFGQ",
		Name:      "Semur En Auxois Airport",
		City:      "Dole",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 1053,
		Latitude:  47.4818992615,
		Longitude: 4.3441700935,
		Timezone:  "Europe/Paris",
	},
	"LFGR": {
		ICAO:      "LFGR",
		Name:      "Doncourt Les Conflans Airport",
		City:      "Dole",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 804,
		Latitude:  49.1528015137,
		Longitude: 5.932779789,
		Timezone:  "Europe/Paris",
	},
	"LFGS": {
		ICAO:      "LFGS",
		Name:      "Longuyon Villette Airport",
		City:      "Dole",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 1148,
		Latitude:  49.4844017029,
		Longitude: 5.5727801323,
		Timezone:  "Europe/Brussels",
	},
	"LFGT": {
		ICAO:      "LFGT",
		Name:      "Sarrebourg Buhl Airport",
		City:      "Dole",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 873,
		Latitude:  48.7188987732,
		Longitude: 7.0794401169,
		Timezone:  "Europe/Paris",
	},
	"LFGU": {
		ICAO:      "LFGU",
		Name:      "Sarreguemines Neunkirch Airport",
		City:      "Dole",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 853,
		Latitude:  49.1281013489,
		Longitude: 7.1083297729,
		Timezone:  "Europe/Berlin",
	},
	"LFGV": {
		ICAO:      "LFGV",
		Name:      "Thionville Yutz Airport",
		City:      "Dole",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 518,
		Latitude:  49.3546981812,
		Longitude: 6.2013897896,
		Timezone:  "Europe/Paris",
	},
	"LFGW": {
		ICAO:      "LFGW",
		IATA:      "XVN",
		Name:      "Verdun-Le Rozelier Airport",
		City:      "Verdun/Le Rozelier",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 1230,
		Latitude:  49.1223983765,
		Longitude: 5.4690499306,
		Timezone:  "Europe/Paris",
	},
	"LFGX": {
		ICAO:      "LFGX",
		Name:      "Champagnole Crotenay Airport",
		City:      "Verdun",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 1745,
		Latitude:  46.7644004822,
		Longitude: 5.8208298683,
		Timezone:  "Europe/Paris",
	},
	"LFGY": {
		ICAO:      "LFGY",
		Name:      "St Die Remomeix Airport",
		City:      "Verdun",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 1184,
		Latitude:  48.26720047,
		Longitude: 7.0086097717,
		Timezone:  "Europe/Paris",
	},
	"LFGZ": {
		ICAO:      "LFGZ",
		Name:      "Nuits St Georges Airport",
		City:      "Verdun",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 797,
		Latitude:  47.1431007385,
		Longitude: 4.9691700935,
		Timezone:  "Europe/Paris",
	},
	"LFHA": {
		ICAO:      "LFHA",
		Name:      "Issoire Le Broc Airport",
		City:      "Verdun",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 1240,
		Latitude:  45.5149993896,
		Longitude: 3.2674999237,
		Timezone:  "Europe/Paris",
	},
	"LFHC": {
		ICAO:      "LFHC",
		Name:      "Perouges - Meximieux Airport",
		City:      "Verdun",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 702,
		Latitude:  45.8697013855,
		Longitude: 5.1872200966,
		Timezone:  "Europe/Paris",
	},
	"LFHD": {
		ICAO:      "LFHD",
		Name:      "Pierrelatte Airport",
		City:      "Verdun",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 197,
		Latitude:  44.3988990784,
		Longitude: 4.7180600166,
		Timezone:  "Europe/Paris",
	},
	"LFHE": {
		ICAO:      "LFHE",
		Name:      "Romans - Saint-Paul Airport",
		City:      "Verdun",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 594,
		Latitude:  45.0661010742,
		Longitude: 5.1033301353,
		Timezone:  "Europe/Paris",
	},
	"LFHF": {
		ICAO:      "LFHF",
		Name:      "Ruoms Airport",
		City:      "Verdun",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 361,
		Latitude:  44.4453010559,
		Longitude: 4.3338899612,
		Timezone:  "Europe/Paris",
	},
	"LFHG": {
		ICAO:      "LFHG",
		Name:      "St Chamond L'horme Airport",
		City:      "Verdun",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 1296,
		Latitude:  45.4930992126,
		Longitude: 4.5355601311,
		Timezone:  "Europe/Paris",
	},
	"LFHH": {
		ICAO:      "LFHH",
		Name:      "Vienne Reventin Airport",
		City:      "Verdun",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 719,
		Latitude:  45.4641990662,
		Longitude: 4.8294401169,
		Timezone:  "Europe/Paris",
	},
	"LFHI": {
		ICAO:      "LFHI",
		Name:      "Morestel Airport",
		City:      "Verdun",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 804,
		Latitude:  45.6878013611,
		Longitude: 5.4536099434,
		Timezone:  "Europe/Paris",
	},
	"LFHJ": {
		ICAO:      "LFHJ",
		Name:      "Lyon Corbas Airport",
		City:      "Verdun",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 650,
		Latitude:  45.6542015076,
		Longitude: 4.9136099815,
		Timezone:  "Europe/Paris",
	},
	"LFHL": {
		ICAO:      "LFHL",
		Name:      "Langogne - Lesperon Airport",
		City:      "Verdun",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 3330,
		Latitude:  44.706401825,
		Longitude: 3.8883299828,
		Timezone:  "Europe/Paris",
	},
	"LFHM": {
		ICAO:      "LFHM",
		IATA:      "MVV",
		Name:      "Megeve Airport",
		City:      "Verdun",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 4823,
		Latitude:  45.8208007813,
		Longitude: 6.6522197723,
		Timezone:  "Europe/Paris",
	},
	"LFHN": {
		ICAO:      "LFHN",
		Name:      "Bellegarde-Vouvray Airport",
		City:      "Verdun",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 1624,
		Latitude:  46.1241989136,
		Longitude: 5.8061099052,
		Timezone:  "Europe/Paris",
	},
	"LFHO": {
		ICAO:      "LFHO",
		IATA:      "OBS",
		Name:      "Aubenas-Ardeche Meridional Airport",
		City:      "Aubenas/Ardeche Meridional",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 923,
		Latitude:  44.5442008972,
		Longitude: 4.3721899986,
		Timezone:  "Europe/Paris",
	},
	"LFHP": {
		ICAO:      "LFHP",
		IATA:      "LPY",
		Name:      "Le Puy-Loudes Airport",
		City:      "Le Puy/Loudes",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 2731,
		Latitude:  45.0806999207,
		Longitude: 3.7628901005,
		Timezone:  "Europe/Paris",
	},
	"LFHQ": {
		ICAO:      "LFHQ",
		Name:      "Saint-Flour-Coltines Airport",
		City:      "Saint-Flour/Coltines",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 3218,
		Latitude:  45.0764007568,
		Longitude: 2.9936099052,
		Timezone:  "Europe/Paris",
	},
	"LFHR": {
		ICAO:      "LFHR",
		Name:      "Brioude Beaumont Airport",
		City:      "St Flour",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 1483,
		Latitude:  45.3250007629,
		Longitude: 3.35916996,
		Timezone:  "Europe/Paris",
	},
	"LFHS": {
		ICAO:      "LFHS",
		IATA:      "XBK",
		Name:      "Bourg-Ceyzeriat Airport",
		City:      "Bourg/Ceyzeriat",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 857,
		Latitude:  46.2009010315,
		Longitude: 5.2920298576,
		Timezone:  "Europe/Paris",
	},
	"LFHT": {
		ICAO:      "LFHT",
		Name:      "Ambert Le Poyet Airport",
		City:      "Bourg",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 1847,
		Latitude:  45.5168991089,
		Longitude: 3.7463901043,
		Timezone:  "Europe/Paris",
	},
	"LFHU": {
		ICAO:      "LFHU",
		IATA:      "AHZ",
		Name:      "L'alpe D'huez Airport",
		City:      "Bourg",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 6102,
		Latitude:  45.0882987976,
		Longitude: 6.0847201347,
		Timezone:  "Europe/Paris",
	},
	"LFHV": {
		ICAO:      "LFHV",
		Name:      "Villefranche-Tarare Airport",
		City:      "Villefranche/Tarare",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 1076,
		Latitude:  45.919983,
		Longitude: 4.634931,
		Timezone:  "Europe/Paris",
	},
	"LFHW": {
		ICAO:      "LFHW",
		Name:      "Belleville Villie Morgon Airport",
		City:      "Villefranche",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 705,
		Latitude:  46.1427993774,
		Longitude: 4.7147197723,
		Timezone:  "Europe/Paris",
	},
	"LFHX": {
		ICAO:      "LFHX",
		Name:      "Lapalisse - Perigny Airport",
		City:      "Villefranche",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 1040,
		Latitude:  46.2538986206,
		Longitude: 3.5886099339,
		Timezone:  "Europe/Paris",
	},
	"LFHY": {
		ICAO:      "LFHY",
		IATA:      "XMU",
		Name:      "Moulins-Montbeugny Airport",
		City:      "Moulins/Montbeugny",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 915,
		Latitude:  46.5345993042,
		Longitude: 3.423719883,
		Timezone:  "Europe/Paris",
	},
	"LFHZ": {
		ICAO:      "LFHZ",
		Name:      "Sallanches Airport",
		City:      "Moulins",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 1755,
		Latitude:  45.9538993835,
		Longitude: 6.6391701698,
		Timezone:  "Europe/Paris",
	},
	"LFIB": {
		ICAO:      "LFIB",
		Name:      "Belves St Pardoux Airport",
		City:      "Moulins",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 791,
		Latitude:  44.7825012207,
		Longitude: 0.9588890076,
		Timezone:  "Europe/Paris",
	},
	"LFID": {
		ICAO:      "LFID",
		Name:      "Condom Valence Sur Baise Airport",
		City:      "Moulins",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 444,
		Latitude:  43.9103012085,
		Longitude: 0.387221992,
		Timezone:  "Europe/Paris",
	},
	"LFIF": {
		ICAO:      "LFIF",
		Name:      "Saint-Affrique-Belmont Airport",
		City:      "Saint-Affrique/Belmont",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 1686,
		Latitude:  43.823299408,
		Longitude: 2.7452800274,
		Timezone:  "Europe/Paris",
	},
	"LFIG": {
		ICAO:      "LFIG",
		Name:      "Cassagnes-Begonhes Airport",
		City:      "Cassagnes-Begonhes",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 2024,
		Latitude:  44.1777992249,
		Longitude: 2.5150001049,
		Timezone:  "Europe/Paris",
	},
	"LFIH": {
		ICAO:      "LFIH",
		Name:      "Chalais Airport",
		City:      "Moulins",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 285,
		Latitude:  45.2681007385,
		Longitude: 0.0169440005,
		Timezone:  "Europe/Paris",
	},
	"LFIK": {
		ICAO:      "LFIK",
		Name:      "Riberac St Aulaye Airport",
		City:      "Moulins",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 351,
		Latitude:  45.2402992249,
		Longitude: 0.2669439912,
		Timezone:  "Europe/Paris",
	},
	"LFIL": {
		ICAO:      "LFIL",
		Name:      "Rion Des Landes Airport",
		City:      "Moulins",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 256,
		Latitude:  43.9157981873,
		Longitude: -0.9491670132,
		Timezone:  "Europe/Paris",
	},
	"LFIM": {
		ICAO:      "LFIM",
		Name:      "St Gaudens Montrejeau Airport",
		City:      "Moulins",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 1325,
		Latitude:  43.1086006165,
		Longitude: 0.6202780008,
		Timezone:  "Europe/Paris",
	},
	"LFIP": {
		ICAO:      "LFIP",
		Name:      "Peyresourde Balestas Airport",
		City:      "Moulins",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 5190,
		Latitude:  42.7969017029,
		Longitude: 0.4355559945,
		Timezone:  "Europe/Paris",
	},
	"LFIR": {
		ICAO:      "LFIR",
		Name:      "Revel Montgey Airport",
		City:      "Moulins",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 643,
		Latitude:  43.4813995361,
		Longitude: 1.9800000191,
		Timezone:  "Europe/Paris",
	},
	"LFIT": {
		ICAO:      "LFIT",
		Name:      "Toulouse Bourg St Bernard Airport",
		City:      "Moulins",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 525,
		Latitude:  43.6122016907,
		Longitude: 1.7252800465,
		Timezone:  "Europe/Paris",
	},
	"LFIV": {
		ICAO:      "LFIV",
		Name:      "Vendays Montalivet Airport",
		City:      "Moulins",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 16,
		Latitude:  45.3805999756,
		Longitude: -1.1158299446,
		Timezone:  "Europe/Paris",
	},
	"LFIX": {
		ICAO:      "LFIX",
		Name:      "Itxassou Airport",
		City:      "Moulins",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 607,
		Latitude:  43.3375015259,
		Longitude: -1.4222199917,
		Timezone:  "Europe/Paris",
	},
	"LFIY": {
		ICAO:      "LFIY",
		Name:      "St Jean D'angely Airport",
		City:      "Moulins",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 246,
		Latitude:  45.9664001465,
		Longitude: -0.5252779722,
		Timezone:  "Europe/Paris",
	},
	"LFJA": {
		ICAO:      "LFJA",
		Name:      "Chaumont-Semoutiers Airport",
		City:      "Chaumont/Semoutiers",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 1001,
		Latitude:  48.0862998962,
		Longitude: 5.0490198135,
		Timezone:  "Europe/Paris",
	},
	"LFJB": {
		ICAO:      "LFJB",
		Name:      "Mauleon Airport",
		City:      "Mauleon",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 576,
		Latitude:  46.9028015137,
		Longitude: -0.6977779865,
		Timezone:  "Europe/Paris",
	},
	"LFJC": {
		ICAO:      "LFJC",
		Name:      "Clamecy Airport",
		City:      "Moulins",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 713,
		Latitude:  47.4383010864,
		Longitude: 3.5086100101,
		Timezone:  "Europe/Paris",
	},
	"LFJD": {
		ICAO:      "LFJD",
		Name:      "Corlier Airport",
		City:      "Moulins",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 2762,
		Latitude:  46.0396995544,
		Longitude: 5.496940136,
		Timezone:  "Europe/Paris",
	},
	"LFJE": {
		ICAO:      "LFJE",
		Name:      "La Motte Chalancon Airport",
		City:      "Moulins",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 2887,
		Latitude:  44.4958000183,
		Longitude: 5.4028501511,
		Timezone:  "Europe/Paris",
	},
	"LFJF": {
		ICAO:      "LFJF",
		Name:      "Aubenasson Airport",
		City:      "Moulins",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 810,
		Latitude:  44.6944999695,
		Longitude: 5.1511101723,
		Timezone:  "Europe/Paris",
	},
	"LFJH": {
		ICAO:      "LFJH",
		Name:      "Cazeres Palaminy Airport",
		City:      "Moulins",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 811,
		Latitude:  43.2022018433,
		Longitude: 1.0511100292,
		Timezone:  "Europe/Paris",
	},
	"LFJI": {
		ICAO:      "LFJI",
		Name:      "Marennes Le Bournet Airport",
		City:      "Moulins",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 16,
		Latitude:  45.8250007629,
		Longitude: -1.0766700506,
		Timezone:  "Europe/Paris",
	},
	"LFJL": {
		ICAO:      "LFJL",
		IATA:      "ETZ",
		Name:      "Metz-Nancy-Lorraine Airport",
		City:      "Metz / Nancy",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 870,
		Latitude:  48.9821014404,
		Longitude: 6.2513198853,
		Timezone:  "Europe/Paris",
	},
	"LFJM": {
		ICAO:      "LFJM",
		Name:      "Chailley Airport",
		City:      "Metz-Nancy/Lorraine",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Latitude:  48.0807151794,
		Longitude: 3.7142372131,
		Timezone:  "Europe/Paris",
	},
	"LFJR": {
		ICAO:      "LFJR",
		IATA:      "ANE",
		Name:      "Angers-Loire Airport",
		City:      "Angers/Marce",
		State:     "Pays-de-la-Loire",
		Country:   "FR",
		Elevation: 194,
		Latitude:  47.5602989197,
		Longitude: -0.3122220039,
		Timezone:  "Europe/Paris",
	},
	"LFJS": {
		ICAO:      "LFJS",
		Name:      "Soissons - Courmelles Airport",
		City:      "Angers",
		State:     "Hauts-de-France",
		Country:   "FR",
		Elevation: 509,
		Latitude:  49.3457984924,
		Longitude: 3.2841699123,
		Timezone:  "Europe/Paris",
	},
	"LFJT": {
		ICAO:      "LFJT",
		Name:      "Tours Le Louroux Airport",
		City:      "Angers",
		State:     "Centre",
		Country:   "FR",
		Elevation: 413,
		Latitude:  47.1500015259,
		Longitude: 0.7127779722,
		Timezone:  "Europe/Paris",
	},
	"LFJU": {
		ICAO:      "LFJU",
		Name:      "Lurcy-Levis Airport",
		City:      "Lurcy-Levis",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 746,
		Latitude:  46.7136001587,
		Longitude: 2.9458999634,
		Timezone:  "Europe/Paris",
	},
	"LFKA": {
		ICAO:      "LFKA",
		IATA:      "XAV",
		Name:      "Albertville Airport",
		City:      "Angers",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 1033,
		Latitude:  45.6272010803,
		Longitude: 6.3297200203,
		Timezone:  "Europe/Paris",
	},
	"LFKB": {
		ICAO:      "LFKB",
		IATA:      "BIA",
		Name:      "Bastia-Poretta Airport",
		City:      "Bastia/Poretta",
		State:     "Corsica",
		Country:   "FR",
		Elevation: 26,
		Latitude:  42.5527000427,
		Longitude: 9.4837303162,
		Timezone:  "Europe/Paris",
	},
	"LFKC": {
		ICAO:      "LFKC",
		IATA:      "CLY",
		Name:      "Calvi-Sainte-Catherine Airport",
		City:      "Calvi/Sainte-Catherine",
		State:     "Corsica",
		Country:   "FR",
		Elevation: 209,
		Latitude:  42.5307998657,
		Longitude: 8.7931900024,
		Timezone:  "Europe/Paris",
	},
	"LFKD": {
		ICAO:      "LFKD",
		Name:      "Sollieres Sardieres Airport",
		City:      "Calvi",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 4255,
		Latitude:  45.256401062,
		Longitude: 6.8013901711,
		Timezone:  "Europe/Paris",
	},
	"LFKE": {
		ICAO:      "LFKE",
		Name:      "Saint-Jean-en-Royans Airport",
		City:      "Calvi",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 866,
		Latitude:  45.0278015137,
		Longitude: 5.3099999428,
		Timezone:  "Europe/Paris",
	},
	"LFKF": {
		ICAO:      "LFKF",
		IATA:      "FSC",
		Name:      "Figari Sud-Corse Airport",
		City:      "Figari Sud-Corse",
		State:     "Corsica",
		Country:   "FR",
		Elevation: 87,
		Latitude:  41.5005989075,
		Longitude: 9.0977802277,
		Timezone:  "Europe/Paris",
	},
	"LFKG": {
		ICAO:      "LFKG",
		Name:      "Ghisonaccia Alzitone Airport",
		City:      "Figari/Sud Corse",
		State:     "Corsica",
		Country:   "FR",
		Elevation: 177,
		Latitude:  42.0550003052,
		Longitude: 9.4019403458,
		Timezone:  "Europe/Paris",
	},
	"LFKH": {
		ICAO:      "LFKH",
		Name:      "St Jean D'avelanne Airport",
		City:      "Figari/Sud Corse",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 968,
		Latitude:  45.5167007446,
		Longitude: 5.680560112,
		Timezone:  "Europe/Paris",
	},
	"LFKJ": {
		ICAO:      "LFKJ",
		IATA:      "AJA",
		Name:      "Ajaccio-Napoleon Bonaparte Airport",
		City:      "Ajaccio/Napoleon Bonaparte",
		State:     "Corsica",
		Country:   "FR",
		Elevation: 18,
		Latitude:  41.9235992432,
		Longitude: 8.8029203415,
		Timezone:  "Europe/Paris",
	},
	"LFKL": {
		ICAO:      "LFKL",
		Name:      "Lyon Brindas Airport",
		City:      "Ajaccio",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 1040,
		Latitude:  45.7117004395,
		Longitude: 4.6977801323,
		Timezone:  "Europe/Paris",
	},
	"LFKM": {
		ICAO:      "LFKM",
		Name:      "St Galmier Airport",
		City:      "Ajaccio",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 1266,
		Latitude:  45.6072006226,
		Longitude: 4.3058300018,
		Timezone:  "Europe/Paris",
	},
	"LFKO": {
		ICAO:      "LFKO",
		IATA:      "PRP",
		Name:      "Propriano Airport",
		City:      "Propriano",
		State:     "Corsica",
		Country:   "FR",
		Elevation: 13,
		Latitude:  41.6605987549,
		Longitude: 8.889749527,
		Timezone:  "Europe/Paris",
	},
	"LFKP": {
		ICAO:      "LFKP",
		Name:      "La Tour Du Pin Airport",
		City:      "Ajaccio",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 1050,
		Latitude:  45.5600013733,
		Longitude: 5.3847198486,
		Timezone:  "Europe/Paris",
	},
	"LFKS": {
		ICAO:      "LFKS",
		IATA:      "SOZ",
		Name:      "Solenzara (BA 126) Air Base",
		City:      "Solenzara",
		State:     "Corsica",
		Country:   "FR",
		Elevation: 28,
		Latitude:  41.9244003296,
		Longitude: 9.4060001373,
		Timezone:  "Europe/Paris",
	},
	"LFKT": {
		ICAO:      "LFKT",
		Name:      "Corte Airport",
		City:      "Corte",
		State:     "Corsica",
		Country:   "FR",
		Elevation: 1132,
		Latitude:  42.293598175,
		Longitude: 9.1930599213,
		Timezone:  "Europe/Paris",
	},
	"LFKX": {
		ICAO:      "LFKX",
		IATA:      "MFX",
		Name:      "Meribel Airport",
		City:      "Ajaccio",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 5636,
		Latitude:  45.4068984985,
		Longitude: 6.5805602074,
		Timezone:  "Europe/Paris",
	},
	"LFKY": {
		ICAO:      "LFKY",
		Name:      "Belley - Peyrieu Airport",
		City:      "Ajaccio",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 738,
		Latitude:  45.6949996948,
		Longitude: 5.6927800179,
		Timezone:  "Europe/Paris",
	},
	"LFKZ": {
		ICAO:      "LFKZ",
		Name:      "St Claude Pratz Airport",
		City:      "Ajaccio",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 2051,
		Latitude:  46.3889007568,
		Longitude: 5.7722201347,
		Timezone:  "Europe/Paris",
	},
	"LFLA": {
		ICAO:      "LFLA",
		IATA:      "AUF",
		Name:      "Auxerre-Branches Airport",
		City:      "Auxerre/Branches",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 523,
		Latitude:  47.8502006531,
		Longitude: 3.49710989,
		Timezone:  "Europe/Paris",
	},
	"LFLB": {
		ICAO:      "LFLB",
		IATA:      "CMF",
		Name:      "Chambery-Savoie Airport",
		City:      "Chambery/Aix-les-Bains",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 779,
		Latitude:  45.6380996704,
		Longitude: 5.88022995,
		Timezone:  "Europe/Paris",
	},
	"LFLC": {
		ICAO:      "LFLC",
		IATA:      "CFE",
		Name:      "Clermont-Ferrand Auvergne Airport",
		City:      "Clermont-Ferrand/Auvergne",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 1090,
		Latitude:  45.7867012024,
		Longitude: 3.1691699028,
		Timezone:  "Europe/Paris",
	},
	"LFLD": {
		ICAO:      "LFLD",
		IATA:      "BOU",
		Name:      "Bourges Airport",
		City:      "Bourges",
		State:     "Centre",
		Country:   "FR",
		Elevation: 529,
		Latitude:  47.0581016541,
		Longitude: 2.3702800274,
		Timezone:  "Europe/Paris",
	},
	"LFLE": {
		ICAO:      "LFLE",
		Name:      "Chambery-Challes-les-Eaux Airport",
		City:      "Chambery/Challes-les-Eaux",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 973,
		Latitude:  45.5611000061,
		Longitude: 5.9757599831,
		Timezone:  "Europe/Paris",
	},
	"LFLG": {
		ICAO:      "LFLG",
		Name:      "Grenoble Le Versoud Airport",
		City:      "Bourges",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 722,
		Latitude:  45.2192001343,
		Longitude: 5.8497200012,
		Timezone:  "Europe/Paris",
	},
	"LFLH": {
		ICAO:      "LFLH",
		Name:      "Chalon-Champforgeuil Airport",
		City:      "Chalon/Champforgueil",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 623,
		Latitude:  46.8260993958,
		Longitude: 4.8176298141,
		Timezone:  "Europe/Paris",
	},
	"LFLI": {
		ICAO:      "LFLI",
		IATA:      "QNJ",
		Name:      "Annemasse Airport",
		City:      "Annemasse",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 1620,
		Latitude:  46.1920013428,
		Longitude: 6.2683901787,
		Timezone:  "Europe/Paris",
	},
	"LFLJ": {
		ICAO:      "LFLJ",
		IATA:      "CVF",
		Name:      "Courchevel Airport",
		City:      "Courcheval",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 6588,
		Latitude:  45.3967018127,
		Longitude: 6.6347198486,
		Timezone:  "Europe/Paris",
	},
	"LFLK": {
		ICAO:      "LFLK",
		Name:      "Oyonnax-Arbent Airport",
		City:      "Chalon",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 1755,
		Latitude:  46.2792015076,
		Longitude: 5.6675000191,
		Timezone:  "Europe/Paris",
	},
	"LFLL": {
		ICAO:      "LFLL",
		IATA:      "LYS",
		Name:      "Lyon Saint-Exupery Airport",
		City:      "Lyon",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 821,
		Latitude:  45.726398468,
		Longitude: 5.0908298492,
		Timezone:  "Europe/Paris",
	},
	"LFLM": {
		ICAO:      "LFLM",
		IATA:      "QNX",
		Name:      "Macon-Charnay Airport",
		City:      "Macon/Charnay",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 728,
		Latitude:  46.2951011658,
		Longitude: 4.7957701683,
		Timezone:  "Europe/Paris",
	},
	"LFLN": {
		ICAO:      "LFLN",
		IATA:      "SYT",
		Name:      "Saint-Yan Airport",
		City:      "Saint-Yan",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 796,
		Latitude:  46.4124984741,
		Longitude: 4.0132598877,
		Timezone:  "Europe/Paris",
	},
	"LFLO": {
		ICAO:      "LFLO",
		IATA:      "RNE",
		Name:      "Roanne-Renaison Airport",
		City:      "Roanne/Renaison",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 1106,
		Latitude:  46.0583000183,
		Longitude: 4.0013899803,
		Timezone:  "Europe/Paris",
	},
	"LFLP": {
		ICAO:      "LFLP",
		IATA:      "NCY",
		Name:      "Annecy-Haute-Savoie-Mont Blanc Airport",
		City:      "Annecy/Meythet",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 1521,
		Latitude:  45.9291992188,
		Longitude: 6.098760128,
		Timezone:  "Europe/Paris",
	},
	"LFLQ": {
		ICAO:      "LFLQ",
		IATA:      "XMK",
		Name:      "Montelimar - Ancone Airport",
		City:      "Annecy",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 240,
		Latitude:  44.5835990906,
		Longitude: 4.7405600548,
		Timezone:  "Europe/Paris",
	},
	"LFLR": {
		ICAO:      "LFLR",
		Name:      "Saint-Rambert-d'Albon Airport",
		City:      "Annecy",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 509,
		Latitude:  45.2560997009,
		Longitude: 4.8258299828,
		Timezone:  "Europe/Paris",
	},
	"LFLS": {
		ICAO:      "LFLS",
		IATA:      "GNB",
		Name:      "Grenoble-Isere Airport",
		City:      "Grenoble/Saint-Geoirs",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 1302,
		Latitude:  45.3628997803,
		Longitude: 5.3293700218,
		Timezone:  "Europe/Paris",
	},
	"LFLT": {
		ICAO:      "LFLT",
		Name:      "Montlucon-Domerat Airport",
		City:      "Montlucon/Domerat",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 771,
		Latitude:  46.3525009155,
		Longitude: 2.5704898834,
		Timezone:  "Europe/Paris",
	},
	"LFLU": {
		ICAO:      "LFLU",
		IATA:      "VAF",
		Name:      "Valence-Chabeuil Airport",
		City:      "Valence/Chabeuil",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 525,
		Latitude:  44.9216003418,
		Longitude: 4.9699001312,
		Timezone:  "Europe/Paris",
	},
	"LFLV": {
		ICAO:      "LFLV",
		IATA:      "VHY",
		Name:      "Vichy-Charmeil Airport",
		City:      "Vichy/Charmeil",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 817,
		Latitude:  46.1697006226,
		Longitude: 3.4037399292,
		Timezone:  "Europe/Paris",
	},
	"LFLW": {
		ICAO:      "LFLW",
		IATA:      "AUR",
		Name:      "Aurillac Airport",
		City:      "Aurillac",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 2096,
		Latitude:  44.8913993835,
		Longitude: 2.4219400883,
		Timezone:  "Europe/Paris",
	},
	"LFLX": {
		ICAO:      "LFLX",
		IATA:      "CHR",
		Name:      "Chateauroux-Deols Marcel Dassault Airport",
		City:      "Chateauroux/Deols",
		State:     "Centre",
		Country:   "FR",
		Elevation: 529,
		Latitude:  46.8622016907,
		Longitude: 1.7306699753,
		Timezone:  "Europe/Paris",
	},
	"LFLY": {
		ICAO:      "LFLY",
		IATA:      "LYN",
		Name:      "Lyon-Bron Airport",
		City:      "Lyon/Bron",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 659,
		Latitude:  45.7271995544,
		Longitude: 4.944270134,
		Timezone:  "Europe/Paris",
	},
	"LFLZ": {
		ICAO:      "LFLZ",
		Name:      "Feurs Chambeon Airport",
		City:      "Lyon",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 1096,
		Latitude:  45.7036018372,
		Longitude: 4.2011098862,
		Timezone:  "Europe/Paris",
	},
	"LFMA": {
		ICAO:      "LFMA",
		IATA:      "QXB",
		Name:      "Aix-en-Provence (BA 114) Airport",
		City:      "Lyon",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Elevation: 367,
		Latitude:  43.5055999756,
		Longitude: 5.3677802086,
		Timezone:  "Europe/Paris",
	},
	"LFMC": {
		ICAO:      "LFMC",
		Name:      "Le Luc-Le Cannet Airport",
		City:      "Le Luc/Le Cannet",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Elevation: 265,
		Latitude:  43.3847007751,
		Longitude: 6.3871397972,
		Timezone:  "Europe/Paris",
	},
	"LFMD": {
		ICAO:      "LFMD",
		IATA:      "CEQ",
		Name:      "Cannes-Mandelieu Airport",
		City:      "Cannes/Mandelieu",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Elevation: 13,
		Latitude:  43.5419998169,
		Longitude: 6.9534797668,
		Timezone:  "Europe/Paris",
	},
	"LFME": {
		ICAO:      "LFME",
		Name:      "Nimes Courbessac Airport",
		City:      "Cannes",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 197,
		Latitude:  43.8539009094,
		Longitude: 4.4136099815,
		Timezone:  "Europe/Paris",
	},
	"LFMF": {
		ICAO:      "LFMF",
		Name:      "Fayence Airport",
		City:      "Cannes",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Elevation: 741,
		Latitude:  43.6088981628,
		Longitude: 6.7027797699,
		Timezone:  "Europe/Paris",
	},
	"LFMG": {
		ICAO:      "LFMG",
		Name:      "Montagne Noire Airport",
		City:      "Cannes",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 1470,
		Latitude:  43.4075012207,
		Longitude: 1.9902800322,
		Timezone:  "Europe/Paris",
	},
	"LFMH": {
		ICAO:      "LFMH",
		IATA:      "EBU",
		Name:      "Saint-Etienne-Boutheon Airport",
		City:      "Saint-Etienne/Boutheon",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 1325,
		Latitude:  45.540599823,
		Longitude: 4.2963900566,
		Timezone:  "Europe/Paris",
	},
	"LFMI": {
		ICAO:      "LFMI",
		IATA:      "QIE",
		Name:      "Istres Le Tube/Istres Air Base (BA 125) Airport",
		City:      "Istres/Le Tube",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Elevation: 82,
		Latitude:  43.5227012634,
		Longitude: 4.9238400459,
		Timezone:  "Europe/Paris",
	},
	"LFMK": {
		ICAO:      "LFMK",
		IATA:      "CCF",
		Name:      "Carcassonne Airport",
		City:      "Carcassonne/Salvaza",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 433,
		Latitude:  43.2159996033,
		Longitude: 2.306319952,
		Timezone:  "Europe/Paris",
	},
	"LFML": {
		ICAO:      "LFML",
		IATA:      "MRS",
		Name:      "Marseille Provence Airport",
		City:      "Marseille",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Elevation: 74,
		Latitude:  43.439271922,
		Longitude: 5.2214241028,
		Timezone:  "Europe/Paris",
	},
	"LFMN": {
		ICAO:      "LFMN",
		IATA:      "NCE",
		Name:      "Nice-Cote d'Azur Airport",
		City:      "Nice",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Elevation: 12,
		Latitude:  43.6584014893,
		Longitude: 7.2158699036,
		Timezone:  "Europe/Paris",
	},
	"LFMO": {
		ICAO:      "LFMO",
		IATA:      "XOG",
		Name:      "Orange-Caritat (BA 115) Air Base",
		City:      "Orange/Caritat",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Elevation: 197,
		Latitude:  44.140499115,
		Longitude: 4.8667201996,
		Timezone:  "Europe/Paris",
	},
	"LFMP": {
		ICAO:      "LFMP",
		IATA:      "PGF",
		Name:      "Perpignan-Rivesaltes (Llabanere) Airport",
		City:      "Perpignan/Rivesaltes",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 144,
		Latitude:  42.740398407,
		Longitude: 2.8706700802,
		Timezone:  "Europe/Paris",
	},
	"LFMQ": {
		ICAO:      "LFMQ",
		IATA:      "CTT",
		Name:      "Le Castellet Airport",
		City:      "Le Castellet",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Elevation: 1391,
		Latitude:  43.2524986267,
		Longitude: 5.7851901054,
		Timezone:  "Europe/Paris",
	},
	"LFMR": {
		ICAO:      "LFMR",
		IATA:      "BAE",
		Name:      "Barcelonnette - Saint-Pons Airport",
		City:      "Le Castellet",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Elevation: 3714,
		Latitude:  44.3883018494,
		Longitude: 6.6102800369,
		Timezone:  "Europe/Paris",
	},
	"LFMS": {
		ICAO:      "LFMS",
		IATA:      "XAS",
		Name:      "Ales-Deaux Airport",
		City:      "Ales/Deaux",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 668,
		Latitude:  44.0696983337,
		Longitude: 4.1421198845,
		Timezone:  "Europe/Paris",
	},
	"LFMT": {
		ICAO:      "LFMT",
		IATA:      "MPL",
		Name:      "Montpellier-Mediterranee Airport",
		City:      "Montpellier/Mediterranee",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 17,
		Latitude:  43.5761985779,
		Longitude: 3.9630100727,
		Timezone:  "Europe/Paris",
	},
	"LFMU": {
		ICAO:      "LFMU",
		IATA:      "BZR",
		Name:      "Beziers-Vias Airport",
		City:      "Beziers/Vias",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 56,
		Latitude:  43.3235015869,
		Longitude: 3.3538999557,
		Timezone:  "Europe/Paris",
	},
	"LFMV": {
		ICAO:      "LFMV",
		IATA:      "AVN",
		Name:      "Avignon-Caumont Airport",
		City:      "Avignon/Caumont",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Elevation: 124,
		Latitude:  43.9072990417,
		Longitude: 4.9018301964,
		Timezone:  "Europe/Paris",
	},
	"LFMW": {
		ICAO:      "LFMW",
		Name:      "Castelnaudary Villeneuve Airport",
		City:      "Avignon",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 551,
		Latitude:  43.3121986389,
		Longitude: 1.921110034,
		Timezone:  "Europe/Paris",
	},
	"LFMX": {
		ICAO:      "LFMX",
		Name:      "Chateau-Arnoux-Saint-Auban Airport",
		City:      "Avignon",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Elevation: 1509,
		Latitude:  44.0600013733,
		Longitude: 5.9913902283,
		Timezone:  "Europe/Paris",
	},
	"LFMY": {
		ICAO:      "LFMY",
		Name:      "Salon-de-Provence (BA 701) Air Base",
		City:      "Salon",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Elevation: 195,
		Latitude:  43.6063995361,
		Longitude: 5.1092500687,
		Timezone:  "Europe/Paris",
	},
	"LFMZ": {
		ICAO:      "LFMZ",
		Name:      "Lezignan-Corbieres Airport",
		City:      "Lezignan-Corbieres",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 207,
		Latitude:  43.1758003235,
		Longitude: 2.73416996,
		Timezone:  "Europe/Paris",
	},
	"LFNA": {
		ICAO:      "LFNA",
		IATA:      "GAT",
		Name:      "Gap - Tallard Airport",
		City:      "Avignon",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Elevation: 1986,
		Latitude:  44.4550018311,
		Longitude: 6.037779808,
		Timezone:  "Europe/Paris",
	},
	"LFNB": {
		ICAO:      "LFNB",
		IATA:      "MEN",
		Name:      "Mende-Brenoux Airport",
		City:      "Mende/Brenoux",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 3362,
		Latitude:  44.5021018982,
		Longitude: 3.5328199863,
		Timezone:  "Europe/Paris",
	},
	"LFNC": {
		ICAO:      "LFNC",
		IATA:      "SCP",
		Name:      "Mont-Dauphin - St-Crepin Airport",
		City:      "Mende",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Elevation: 2963,
		Latitude:  44.7016983032,
		Longitude: 6.600279808,
		Timezone:  "Europe/Paris",
	},
	"LFND": {
		ICAO:      "LFND",
		Name:      "Pont St Esprit Airport",
		City:      "Mende",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Elevation: 144,
		Latitude:  44.2694015503,
		Longitude: 4.6533298492,
		Timezone:  "Europe/Paris",
	},
	"LFNE": {
		ICAO:      "LFNE",
		Name:      "Salon Eyguieres Airport",
		City:      "Mende",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Elevation: 246,
		Latitude:  43.6582984924,
		Longitude: 5.0136098862,
		Timezone:  "Europe/Paris",
	},
	"LFNF": {
		ICAO:      "LFNF",
		Name:      "Vinon Sur Verdon Airport",
		City:      "Mende",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Elevation: 902,
		Latitude:  43.7378005981,
		Longitude: 5.7841701508,
		Timezone:  "Europe/Paris",
	},
	"LFNG": {
		ICAO:      "LFNG",
		Name:      "Montpellier Candillargues Airport",
		City:      "Mende",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 5,
		Latitude:  43.6102981567,
		Longitude: 4.0702800751,
		Timezone:  "Europe/Paris",
	},
	"LFNH": {
		ICAO:      "LFNH",
		Name:      "Carpentras Airport",
		City:      "Carpentras",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Elevation: 394,
		Latitude:  44.029800415,
		Longitude: 5.0780601501,
		Timezone:  "Europe/Paris",
	},
	"LFNJ": {
		ICAO:      "LFNJ",
		Name:      "Aspres-sur-Buech Airport",
		City:      "Mende",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Elevation: 2726,
		Latitude:  44.518901825,
		Longitude: 5.7375001907,
		Timezone:  "Europe/Paris",
	},
	"LFNL": {
		ICAO:      "LFNL",
		Name:      "St Martin De Londres Airport",
		City:      "Mende",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 597,
		Latitude:  43.8008003235,
		Longitude: 3.7825000286,
		Timezone:  "Europe/Paris",
	},
	"LFNO": {
		ICAO:      "LFNO",
		Name:      "Florac Ste Enimie Airport",
		City:      "Mende",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 3054,
		Latitude:  44.2863998413,
		Longitude: 3.4666700363,
		Timezone:  "Europe/Paris",
	},
	"LFNP": {
		ICAO:      "LFNP",
		Name:      "Pezenas Nizas Airport",
		City:      "Mende",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 322,
		Latitude:  43.5057983398,
		Longitude: 3.4136099815,
		Timezone:  "Europe/Paris",
	},
	"LFNQ": {
		ICAO:      "LFNQ",
		Name:      "Mont Louis La Quillane Airport",
		City:      "Mende",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 5610,
		Latitude:  42.543598175,
		Longitude: 2.1199998856,
		Timezone:  "Europe/Paris",
	},
	"LFNR": {
		ICAO:      "LFNR",
		Name:      "Berre La Fare Airport",
		City:      "Mende",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Elevation: 108,
		Latitude:  43.5377998352,
		Longitude: 5.1783299446,
		Timezone:  "Europe/Paris",
	},
	"LFNS": {
		ICAO:      "LFNS",
		Name:      "Sisteron - Theze Airport",
		City:      "Mende",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Elevation: 1772,
		Latitude:  44.2874984741,
		Longitude: 5.9302802086,
		Timezone:  "Europe/Paris",
	},
	"LFNT": {
		ICAO:      "LFNT",
		Name:      "Avignon Pujaut Airport",
		City:      "Mende",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 148,
		Latitude:  43.9968986511,
		Longitude: 4.7555599213,
		Timezone:  "Europe/Paris",
	},
	"LFNU": {
		ICAO:      "LFNU",
		Name:      "Uzes Airport",
		City:      "Mende",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 886,
		Latitude:  44.0847015381,
		Longitude: 4.3952798843,
		Timezone:  "Europe/Paris",
	},
	"LFNV": {
		ICAO:      "LFNV",
		Name:      "Valreas Visan Airport",
		City:      "Mende",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Elevation: 472,
		Latitude:  44.3368988037,
		Longitude: 4.9080600739,
		Timezone:  "Europe/Paris",
	},
	"LFNW": {
		ICAO:      "LFNW",
		Name:      "Puivert Airport",
		City:      "Mende",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 1608,
		Latitude:  42.9113998413,
		Longitude: 2.0561099052,
		Timezone:  "Europe/Paris",
	},
	"LFNX": {
		ICAO:      "LFNX",
		Name:      "Bedarieux La Tour Airport",
		City:      "Mende",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 1237,
		Latitude:  43.6408004761,
		Longitude: 3.1455600262,
		Timezone:  "Europe/Paris",
	},
	"LFNY": {
		ICAO:      "LFNY",
		Name:      "St Etienne En Devoluy Airport",
		City:      "Mende",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Elevation: 6201,
		Latitude:  44.6688995361,
		Longitude: 5.9061098099,
		Timezone:  "Europe/Paris",
	},
	"LFNZ": {
		ICAO:      "LFNZ",
		Name:      "Le Mazet De Romanin Airport",
		City:      "Mende",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Elevation: 387,
		Latitude:  43.768901825,
		Longitude: 4.8936100006,
		Timezone:  "Europe/Paris",
	},
	"LFOA": {
		ICAO:      "LFOA",
		Name:      "Avord (BA 702) Air Base",
		City:      "Avord",
		State:     "Centre",
		Country:   "FR",
		Elevation: 580,
		Latitude:  47.0532989502,
		Longitude: 2.6324999332,
		Timezone:  "Europe/Paris",
	},
	"LFOB": {
		ICAO:      "LFOB",
		IATA:      "BVA",
		Name:      "Paris Beauvais Tille Airport",
		City:      "Beauvais/Tille",
		State:     "Hauts-de-France",
		Country:   "FR",
		Elevation: 359,
		Latitude:  49.4543991089,
		Longitude: 2.1127800941,
		Timezone:  "Europe/Paris",
	},
	"LFOC": {
		ICAO:      "LFOC",
		Name:      "Chateaudun (BA 279) Airport",
		City:      "Chateaudun",
		State:     "Centre",
		Country:   "FR",
		Elevation: 433,
		Latitude:  48.0581016541,
		Longitude: 1.3766200542,
		Timezone:  "Europe/Paris",
	},
	"LFOD": {
		ICAO:      "LFOD",
		IATA:      "XSU",
		Name:      "Saumur-Saint-Florent Airport",
		City:      "Saumur/Saint-Florent",
		State:     "Pays-de-la-Loire",
		Country:   "FR",
		Elevation: 269,
		Latitude:  47.2568016052,
		Longitude: -0.1151420027,
		Timezone:  "Europe/Paris",
	},
	"LFOE": {
		ICAO:      "LFOE",
		IATA:      "EVX",
		Name:      "Evreux-Fauville (BA 105) Air Base",
		City:      "Evreux/Fauville",
		State:     "Normandy",
		Country:   "FR",
		Elevation: 464,
		Latitude:  49.0287017822,
		Longitude: 1.2198599577,
		Timezone:  "Europe/Paris",
	},
	"LFOF": {
		ICAO:      "LFOF",
		IATA:      "XAN",
		Name:      "Alencon Valframbert Airport",
		City:      "Evreux",
		State:     "Normandy",
		Country:   "FR",
		Elevation: 479,
		Latitude:  48.4474983215,
		Longitude: 0.1091670021,
		Timezone:  "Europe/Paris",
	},
	"LFOG": {
		ICAO:      "LFOG",
		Name:      "Flers Saint Paul Airport",
		City:      "Evreux",
		State:     "Normandy",
		Country:   "FR",
		Elevation: 656,
		Latitude:  48.7524986267,
		Longitude: -0.5894439816,
		Timezone:  "Europe/Paris",
	},
	"LFOH": {
		ICAO:      "LFOH",
		IATA:      "LEH",
		Name:      "Le Havre Octeville Airport",
		City:      "Le Havre/Octeville",
		State:     "Normandy",
		Country:   "FR",
		Elevation: 313,
		Latitude:  49.5339012146,
		Longitude: 0.0880559981,
		Timezone:  "Europe/Paris",
	},
	"LFOI": {
		ICAO:      "LFOI",
		IATA:      "XAB",
		Name:      "Abbeville-Buigny-Saint-Maclou Airport",
		City:      "Abbeville (Buigny/Saint-Maclou)",
		State:     "Hauts-de-France",
		Country:   "FR",
		Elevation: 220,
		Latitude:  50.1435012817,
		Longitude: 1.831889987,
		Timezone:  "Europe/Paris",
	},
	"LFOJ": {
		ICAO:      "LFOJ",
		IATA:      "ORE",
		Name:      "Orleans-Bricy (BA 123) Air Base",
		City:      "Orleans/Bricy",
		State:     "Centre",
		Country:   "FR",
		Elevation: 412,
		Latitude:  47.9878005981,
		Longitude: 1.7605600357,
		Timezone:  "Europe/Paris",
	},
	"LFOK": {
		ICAO:      "LFOK",
		IATA:      "XCR",
		Name:      "Chalons-Vatry Air Base",
		City:      "Chalons/Vatry",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 587,
		Latitude:  48.7761001587,
		Longitude: 4.1844902039,
		Timezone:  "Europe/Paris",
	},
	"LFOL": {
		ICAO:      "LFOL",
		Name:      "L'aigle St Michel Airport",
		City:      "Chalons",
		State:     "Normandy",
		Country:   "FR",
		Elevation: 787,
		Latitude:  48.7597007751,
		Longitude: 0.6591669917,
		Timezone:  "Europe/Paris",
	},
	"LFOM": {
		ICAO:      "LFOM",
		Name:      "Lessay Airport",
		City:      "Chalons",
		State:     "Normandy",
		Country:   "FR",
		Elevation: 92,
		Latitude:  49.2030982971,
		Longitude: -1.5066699982,
		Timezone:  "Europe/Paris",
	},
	"LFON": {
		ICAO:      "LFON",
		Name:      "Dreux Vernouillet Airport",
		City:      "Chalons",
		State:     "Centre",
		Country:   "FR",
		Elevation: 443,
		Latitude:  48.7066993713,
		Longitude: 1.3627799749,
		Timezone:  "Europe/Paris",
	},
	"LFOO": {
		ICAO:      "LFOO",
		IATA:      "LSO",
		Name:      "Les Sables-d'Olonne Talmont Airport",
		City:      "Les Sables-d'Olonne",
		State:     "Pays-de-la-Loire",
		Country:   "FR",
		Elevation: 105,
		Latitude:  46.4768981934,
		Longitude: -1.7227799892,
		Timezone:  "Europe/Paris",
	},
	"LFOP": {
		ICAO:      "LFOP",
		IATA:      "URO",
		Name:      "Rouen Airport",
		City:      "Rouen/Vallee de Seine",
		State:     "Normandy",
		Country:   "FR",
		Elevation: 512,
		Latitude:  49.3842010498,
		Longitude: 1.1748000383,
		Timezone:  "Europe/Paris",
	},
	"LFOQ": {
		ICAO:      "LFOQ",
		IATA:      "XBQ",
		Name:      "Blois-Le Breuil Airport",
		City:      "Blois/Le Breuil",
		State:     "Centre",
		Country:   "FR",
		Elevation: 398,
		Latitude:  47.6785011292,
		Longitude: 1.2088400126,
		Timezone:  "Europe/Paris",
	},
	"LFOR": {
		ICAO:      "LFOR",
		IATA:      "QTJ",
		Name:      "Chartres Champhol Airport",
		City:      "Blois",
		State:     "Centre",
		Country:   "FR",
		Elevation: 509,
		Latitude:  48.4589004517,
		Longitude: 1.5238900185,
		Timezone:  "Europe/Paris",
	},
	"LFOS": {
		ICAO:      "LFOS",
		Name:      "St Valery Vittefleur Airport",
		City:      "Blois",
		State:     "Normandy",
		Country:   "FR",
		Elevation: 272,
		Latitude:  49.836101532,
		Longitude: 0.6549999714,
		Timezone:  "Europe/Paris",
	},
	"LFOT": {
		ICAO:      "LFOT",
		IATA:      "TUF",
		Name:      "Tours-Val-de-Loire Airport",
		City:      "Tours/Val de Loire (Loire Valley)",
		State:     "Centre",
		Country:   "FR",
		Elevation: 357,
		Latitude:  47.4322013855,
		Longitude: 0.7276059985,
		Timezone:  "Europe/Paris",
	},
	"LFOU": {
		ICAO:      "LFOU",
		IATA:      "CET",
		Name:      "Cholet Le Pontreau Airport",
		City:      "Cholet/Le Pontreau",
		State:     "Pays-de-la-Loire",
		Country:   "FR",
		Elevation: 443,
		Latitude:  47.0820999146,
		Longitude: -0.8770639896,
		Timezone:  "Europe/Paris",
	},
	"LFOV": {
		ICAO:      "LFOV",
		IATA:      "LVA",
		Name:      "Laval-Entrammes Airport",
		City:      "Laval/Entrammes",
		State:     "Pays-de-la-Loire",
		Country:   "FR",
		Elevation: 330,
		Latitude:  48.0313987732,
		Longitude: -0.7429860234,
		Timezone:  "Europe/Paris",
	},
	"LFOW": {
		ICAO:      "LFOW",
		Name:      "Saint-Quentin - Roupy Airport",
		City:      "Laval",
		State:     "Hauts-de-France",
		Country:   "FR",
		Elevation: 325,
		Latitude:  49.8168983459,
		Longitude: 3.2066700459,
		Timezone:  "Europe/Paris",
	},
	"LFOX": {
		ICAO:      "LFOX",
		Name:      "Etampes Mondesir Airport",
		City:      "Laval",
		State:     "Ile-de-France",
		Country:   "FR",
		Elevation: 489,
		Latitude:  48.3819007874,
		Longitude: 2.0752799511,
		Timezone:  "Europe/Paris",
	},
	"LFOY": {
		ICAO:      "LFOY",
		Name:      "Le Havre St Romain Airport",
		City:      "Laval",
		State:     "Normandy",
		Country:   "FR",
		Elevation: 423,
		Latitude:  49.5447006226,
		Longitude: 0.3611109853,
		Timezone:  "Europe/Paris",
	},
	"LFOZ": {
		ICAO:      "LFOZ",
		IATA:      "ORE",
		Name:      "Orleans-Saint-Denis-de-l'Hotel Airport",
		City:      "Orleans/Saint-Denis-de-l'Hotel",
		State:     "Centre",
		Country:   "FR",
		Elevation: 396,
		Latitude:  47.896900177,
		Longitude: 2.1633300781,
		Timezone:  "Europe/Paris",
	},
	"LFPA": {
		ICAO:      "LFPA",
		Name:      "Persan Beaumont Airport",
		City:      "Orleans",
		State:     "Ile-de-France",
		Country:   "FR",
		Elevation: 148,
		Latitude:  49.1657981873,
		Longitude: 2.3130600452,
		Timezone:  "Europe/Paris",
	},
	"LFPB": {
		ICAO:      "LFPB",
		IATA:      "LBG",
		Name:      "Paris-Le Bourget Airport",
		City:      "Paris",
		State:     "Ile-de-France",
		Country:   "FR",
		Elevation: 218,
		Latitude:  48.9693984985,
		Longitude: 2.4413900375,
		Timezone:  "Europe/Paris",
	},
	"LFPC": {
		ICAO:      "LFPC",
		IATA:      "CSF",
		Name:      "Creil Air Base",
		City:      "Creil",
		State:     "Hauts-de-France",
		Country:   "FR",
		Elevation: 291,
		Latitude:  49.2535018921,
		Longitude: 2.5191400051,
		Timezone:  "Europe/Paris",
	},
	"LFPD": {
		ICAO:      "LFPD",
		Name:      "Bernay St Martin Airport",
		City:      "Creil",
		State:     "Normandy",
		Country:   "FR",
		Elevation: 554,
		Latitude:  49.1027984619,
		Longitude: 0.5666670203,
		Timezone:  "Europe/Paris",
	},
	"LFPE": {
		ICAO:      "LFPE",
		Name:      "Meaux Esbly Airport",
		City:      "Creil",
		State:     "Ile-de-France",
		Country:   "FR",
		Elevation: 217,
		Latitude:  48.9277992249,
		Longitude: 2.8352799416,
		Timezone:  "Europe/Paris",
	},
	"LFPF": {
		ICAO:      "LFPF",
		Name:      "Beynes Thiverval Airport",
		City:      "Creil",
		State:     "Ile-de-France",
		Country:   "FR",
		Elevation: 371,
		Latitude:  48.8436012268,
		Longitude: 1.9088900089,
		Timezone:  "Europe/Paris",
	},
	"LFPG": {
		ICAO:      "LFPG",
		IATA:      "CDG",
		Name:      "Charles de Gaulle International Airport",
		City:      "Paris",
		State:     "Ile-de-France",
		Country:   "FR",
		Elevation: 392,
		Latitude:  49.0127983093,
		Longitude: 2.5499999523,
		Timezone:  "Europe/Paris",
	},
	"LFPH": {
		ICAO:      "LFPH",
		Name:      "Chelles le Pin Airport",
		City:      "Paris",
		State:     "Ile-de-France",
		Country:   "FR",
		Elevation: 207,
		Latitude:  48.8978004456,
		Longitude: 2.6075000763,
		Timezone:  "Europe/Paris",
	},
	"LFPK": {
		ICAO:      "LFPK",
		Name:      "Coulommiers-Voisins Airport",
		City:      "Coulommiers/Voisins",
		State:     "Ile-de-France",
		Country:   "FR",
		Elevation: 470,
		Latitude:  48.8376998901,
		Longitude: 3.016119957,
		Timezone:  "Europe/Paris",
	},
	"LFPL": {
		ICAO:      "LFPL",
		Name:      "Lognes Emerainville Airport",
		City:      "Paris",
		State:     "Ile-de-France",
		Country:   "FR",
		Elevation: 354,
		Latitude:  48.8231010437,
		Longitude: 2.6238899231,
		Timezone:  "Europe/Paris",
	},
	"LFPM": {
		ICAO:      "LFPM",
		Name:      "Melun-Villaroche Air Base",
		City:      "Melun/Villaroche",
		State:     "Ile-de-France",
		Country:   "FR",
		Elevation: 302,
		Latitude:  48.6046981812,
		Longitude: 2.6711199284,
		Timezone:  "Europe/Paris",
	},
	"LFPN": {
		ICAO:      "LFPN",
		IATA:      "TNF",
		Name:      "Toussus-le-Noble Airport",
		City:      "Toussus-le-Noble",
		State:     "Ile-de-France",
		Country:   "FR",
		Elevation: 538,
		Latitude:  48.7518997192,
		Longitude: 2.1061899662,
		Timezone:  "Europe/Paris",
	},
	"LFPO": {
		ICAO:      "LFPO",
		IATA:      "ORY",
		Name:      "Paris-Orly Airport",
		City:      "Paris",
		State:     "Ile-de-France",
		Country:   "FR",
		Elevation: 291,
		Latitude:  48.7252998352,
		Longitude: 2.3594400883,
		Timezone:  "Europe/Paris",
	},
	"LFPP": {
		ICAO:      "LFPP",
		Name:      "Le Plessis Belleville Airport",
		City:      "Paris",
		State:     "Hauts-de-France",
		Country:   "FR",
		Elevation: 387,
		Latitude:  49.1100006104,
		Longitude: 2.7380599976,
		Timezone:  "Europe/Paris",
	},
	"LFPQ": {
		ICAO:      "LFPQ",
		Name:      "Fontenay Tresigny Airport",
		City:      "Paris",
		State:     "Ile-de-France",
		Country:   "FR",
		Elevation: 371,
		Latitude:  48.7071990967,
		Longitude: 2.9044399261,
		Timezone:  "Europe/Paris",
	},
	"LFPT": {
		ICAO:      "LFPT",
		IATA:      "POX",
		Name:      "Pontoise - Cormeilles-en-Vexin Airport",
		City:      "Pontoise",
		State:     "Ile-de-France",
		Country:   "FR",
		Elevation: 325,
		Latitude:  49.0965995789,
		Longitude: 2.0408298969,
		Timezone:  "Europe/Paris",
	},
	"LFPU": {
		ICAO:      "LFPU",
		Name:      "Moret Episy Airport",
		City:      "Pontoise",
		State:     "Ile-de-France",
		Country:   "FR",
		Elevation: 253,
		Latitude:  48.3418998718,
		Longitude: 2.7994399071,
		Timezone:  "Europe/Paris",
	},
	"LFPV": {
		ICAO:      "LFPV",
		IATA:      "VIY",
		Name:      "Villacoublay-Velizy (BA 107) Air Base",
		City:      "Villacoublay/Velizy",
		State:     "Ile-de-France",
		Country:   "FR",
		Elevation: 584,
		Latitude:  48.7743988037,
		Longitude: 2.2015399933,
		Timezone:  "Europe/Paris",
	},
	"LFPX": {
		ICAO:      "LFPX",
		Name:      "Chavenay Villepreux Airport",
		City:      "Villacoublay",
		State:     "Ile-de-France",
		Country:   "FR",
		Elevation: 426,
		Latitude:  48.8436012268,
		Longitude: 1.9822200537,
		Timezone:  "Europe/Paris",
	},
	"LFPY": {
		ICAO:      "LFPY",
		Name:      "Bretigny sur Orge Air Base",
		City:      "Villacoublay",
		State:     "Ile-de-France",
		Country:   "FR",
		Elevation: 270,
		Latitude:  48.5960998535,
		Longitude: 2.3322200775,
		Timezone:  "Europe/Paris",
	},
	"LFPZ": {
		ICAO:      "LFPZ",
		Name:      "Saint-Cyr-l'Ecole Airport",
		City:      "Villacoublay",
		State:     "Ile-de-France",
		Country:   "FR",
		Elevation: 371,
		Latitude:  48.8114013672,
		Longitude: 2.0747199059,
		Timezone:  "Europe/Paris",
	},
	"LFQA": {
		ICAO:      "LFQA",
		Name:      "Reims-Prunay Airport",
		City:      "Reims/Prunay",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 313,
		Latitude:  49.2086982727,
		Longitude: 4.1565799713,
		Timezone:  "Europe/Paris",
	},
	"LFQB": {
		ICAO:      "LFQB",
		IATA:      "QYR",
		Name:      "Troyes-Barberey Airport",
		City:      "Troyes/Barberey",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 388,
		Latitude:  48.322101593,
		Longitude: 4.016699791,
		Timezone:  "Europe/Paris",
	},
	"LFQC": {
		ICAO:      "LFQC",
		Name:      "Luneville-Croismare Airport",
		City:      "Luneville/Croismare",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 790,
		Latitude:  48.5932998657,
		Longitude: 6.5434598923,
		Timezone:  "Europe/Paris",
	},
	"LFQD": {
		ICAO:      "LFQD",
		Name:      "Arras Roclincourt Airport",
		City:      "Troyes",
		State:     "Hauts-de-France",
		Country:   "FR",
		Elevation: 338,
		Latitude:  50.3238983154,
		Longitude: 2.8027799129,
		Timezone:  "Europe/Paris",
	},
	"LFQE": {
		ICAO:      "LFQE",
		Name:      "Etain-Rouvres Air Base",
		City:      "Etain/Rouvres",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 770,
		Latitude:  49.2268981934,
		Longitude: 5.6722202301,
		Timezone:  "Europe/Paris",
	},
	"LFQF": {
		ICAO:      "LFQF",
		Name:      "Autun-Bellevue Airport",
		City:      "Autun/Bellevue",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 997,
		Latitude:  46.967300415,
		Longitude: 4.2605700493,
		Timezone:  "Europe/Paris",
	},
	"LFQG": {
		ICAO:      "LFQG",
		IATA:      "NVS",
		Name:      "Nevers-Fourchambault Airport",
		City:      "Nevers/Fourchambault",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 602,
		Latitude:  47.0026016235,
		Longitude: 3.1133298874,
		Timezone:  "Europe/Paris",
	},
	"LFQH": {
		ICAO:      "LFQH",
		Name:      "Chatillon Sur Seine Airport",
		City:      "Nevers",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 905,
		Latitude:  47.8464012146,
		Longitude: 4.5805602074,
		Timezone:  "Europe/Paris",
	},
	"LFQI": {
		ICAO:      "LFQI",
		IATA:      "XCB",
		Name:      "Cambrai-Epinoy (BA 103) Air Base",
		City:      "Cambrai/Epinoy",
		State:     "Hauts-de-France",
		Country:   "FR",
		Elevation: 257,
		Latitude:  50.2218017578,
		Longitude: 3.154239893,
		Timezone:  "Europe/Paris",
	},
	"LFQJ": {
		ICAO:      "LFQJ",
		IATA:      "XME",
		Name:      "Maubeuge-Elesmes Airport",
		City:      "Maubeuge/Elesmes",
		State:     "Hauts-de-France",
		Country:   "FR",
		Elevation: 452,
		Latitude:  50.3105010986,
		Longitude: 4.0331201553,
		Timezone:  "Europe/Paris",
	},
	"LFQK": {
		ICAO:      "LFQK",
		Name:      "Chalons Ecury Sur Coole Airport",
		City:      "Cambrai",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 319,
		Latitude:  48.9061012268,
		Longitude: 4.3541698456,
		Timezone:  "Europe/Paris",
	},
	"LFQL": {
		ICAO:      "LFQL",
		Name:      "Lens Benifontaine Airport",
		City:      "Cambrai",
		State:     "Hauts-de-France",
		Country:   "FR",
		Elevation: 167,
		Latitude:  50.4664001465,
		Longitude: 2.8197200298,
		Timezone:  "Europe/Paris",
	},
	"LFQM": {
		ICAO:      "LFQM",
		Name:      "Besancon-La Veze Airport",
		City:      "Besancon/La Veze",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 1271,
		Latitude:  47.2066001892,
		Longitude: 6.0836801529,
		Timezone:  "Europe/Paris",
	},
	"LFQN": {
		ICAO:      "LFQN",
		Name:      "Saint Omer Wizernes Airport",
		City:      "Besancon",
		State:     "Hauts-de-France",
		Country:   "FR",
		Elevation: 249,
		Latitude:  50.7294006348,
		Longitude: 2.2358300686,
		Timezone:  "Europe/Paris",
	},
	"LFQO": {
		ICAO:      "LFQO",
		Name:      "Lille/Marcq-en-Baroeul Airport",
		City:      "Besancon",
		State:     "Hauts-de-France",
		Country:   "FR",
		Elevation: 69,
		Latitude:  50.6871986389,
		Longitude: 3.0755600929,
		Timezone:  "Europe/Paris",
	},
	"LFQP": {
		ICAO:      "LFQP",
		Name:      "Phalsbourg-Bourscheid Air Base",
		City:      "Phalsbourg/Bourscheid",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 1017,
		Latitude:  48.7663002014,
		Longitude: 7.2005200386,
		Timezone:  "Europe/Paris",
	},
	"LFQQ": {
		ICAO:      "LFQQ",
		IATA:      "LIL",
		Name:      "Lille-Lesquin Airport",
		City:      "Lille/Lesquin",
		State:     "Hauts-de-France",
		Country:   "FR",
		Elevation: 157,
		Latitude:  50.5619010925,
		Longitude: 3.0894401073,
		Timezone:  "Europe/Paris",
	},
	"LFQR": {
		ICAO:      "LFQR",
		Name:      "Romilly Sur Seine Airport",
		City:      "Lille",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 267,
		Latitude:  48.5005989075,
		Longitude: 3.7633299828,
		Timezone:  "Europe/Paris",
	},
	"LFQS": {
		ICAO:      "LFQS",
		Name:      "Vitry en Artois Airport",
		City:      "Lille",
		State:     "Hauts-de-France",
		Country:   "FR",
		Elevation: 174,
		Latitude:  50.3382987976,
		Longitude: 2.9933300018,
		Timezone:  "Europe/Paris",
	},
	"LFQT": {
		ICAO:      "LFQT",
		IATA:      "HZB",
		Name:      "Merville-Calonne Airport",
		City:      "Merville/Calonne",
		State:     "Hauts-de-France",
		Country:   "FR",
		Elevation: 61,
		Latitude:  50.6184005737,
		Longitude: 2.6422400475,
		Timezone:  "Europe/Paris",
	},
	"LFQU": {
		ICAO:      "LFQU",
		Name:      "Sarre Union Airport",
		City:      "Merville",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 837,
		Latitude:  48.9514007568,
		Longitude: 7.0777797699,
		Timezone:  "Europe/Paris",
	},
	"LFQV": {
		ICAO:      "LFQV",
		IATA:      "XCZ",
		Name:      "Charleville-Mezieres Airport",
		City:      "Charleville-Mezieres",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 492,
		Latitude:  49.7839012146,
		Longitude: 4.6470799446,
		Timezone:  "Europe/Paris",
	},
	"LFQW": {
		ICAO:      "LFQW",
		IATA:      "XVO",
		Name:      "Vesoul-Frotey Airport",
		City:      "Vesoul/Frotey",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 1249,
		Latitude:  47.6375999451,
		Longitude: 6.2039198875,
		Timezone:  "Europe/Paris",
	},
	"LFQX": {
		ICAO:      "LFQX",
		Name:      "Juvancourt Airport",
		City:      "Charleville-Mezieres",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 1145,
		Latitude:  48.1150016785,
		Longitude: 4.8208298683,
		Timezone:  "Europe/Paris",
	},
	"LFQY": {
		ICAO:      "LFQY",
		Name:      "Saverne Steinbourg Airport",
		City:      "Charleville-Mezieres",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 630,
		Latitude:  48.7541999817,
		Longitude: 7.4263901711,
		Timezone:  "Europe/Paris",
	},
	"LFQZ": {
		ICAO:      "LFQZ",
		Name:      "Dieuze Gueblange Airport",
		City:      "Charleville-Mezieres",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 722,
		Latitude:  48.7752990723,
		Longitude: 6.715280056,
		Timezone:  "Europe/Paris",
	},
	"LFRA": {
		ICAO:      "LFRA",
		Name:      "Angers Avrille Airport",
		City:      "Charleville-Mezieres",
		State:     "Pays-de-la-Loire",
		Country:   "FR",
		Elevation: 187,
		Latitude:  47.4991989136,
		Longitude: -0.5727779865,
		Timezone:  "Europe/Paris",
	},
	"LFRB": {
		ICAO:      "LFRB",
		IATA:      "BES",
		Name:      "Brest Bretagne Airport",
		City:      "Brest/Guipavas",
		State:     "Brittany",
		Country:   "FR",
		Elevation: 325,
		Latitude:  48.4478988647,
		Longitude: -4.4185400009,
		Timezone:  "Europe/Paris",
	},
	"LFRC": {
		ICAO:      "LFRC",
		IATA:      "CER",
		Name:      "Cherbourg-Maupertus Airport",
		City:      "Cherbourg/Maupertus",
		State:     "Normandy",
		Country:   "FR",
		Elevation: 459,
		Latitude:  49.650100708,
		Longitude: -1.4702800512,
		Timezone:  "Europe/Paris",
	},
	"LFRD": {
		ICAO:      "LFRD",
		IATA:      "DNR",
		Name:      "Dinard-Pleurtuit-Saint-Malo Airport",
		City:      "Dinard/Pleurtuit/Saint-Malo",
		State:     "Brittany",
		Country:   "FR",
		Elevation: 219,
		Latitude:  48.5876998901,
		Longitude: -2.0799601078,
		Timezone:  "Europe/Paris",
	},
	"LFRE": {
		ICAO:      "LFRE",
		IATA:      "LBY",
		Name:      "La Baule-Escoublac Airport",
		City:      "La Baule-Escoublac",
		State:     "Pays-de-la-Loire",
		Country:   "FR",
		Elevation: 105,
		Latitude:  47.2893981934,
		Longitude: -2.3463900089,
		Timezone:  "Europe/Paris",
	},
	"LFRF": {
		ICAO:      "LFRF",
		IATA:      "GFR",
		Name:      "Granville Airport",
		City:      "Granville",
		State:     "Normandy",
		Country:   "FR",
		Elevation: 45,
		Latitude:  48.8830986023,
		Longitude: -1.5641700029,
		Timezone:  "Europe/Paris",
	},
	"LFRG": {
		ICAO:      "LFRG",
		IATA:      "DOL",
		Name:      "Deauville-Saint-Gatien Airport",
		City:      "Deauville",
		State:     "Normandy",
		Country:   "FR",
		Elevation: 479,
		Latitude:  49.3652992249,
		Longitude: 0.1543059945,
		Timezone:  "Europe/Paris",
	},
	"LFRH": {
		ICAO:      "LFRH",
		IATA:      "LRT",
		Name:      "Lorient South Brittany (Bretagne Sud) Airport",
		City:      "Lorient/Lann/Bihoue",
		State:     "Brittany",
		Country:   "FR",
		Elevation: 160,
		Latitude:  47.7606010437,
		Longitude: -3.4400000572,
		Timezone:  "Europe/Paris",
	},
	"LFRI": {
		ICAO:      "LFRI",
		IATA:      "EDM",
		Name:      "La Roche-sur-Yon Airport",
		City:      "La Roche-sur-Yon/Les Ajoncs",
		State:     "Pays-de-la-Loire",
		Country:   "FR",
		Elevation: 299,
		Latitude:  46.7019004822,
		Longitude: -1.3786300421,
		Timezone:  "Europe/Paris",
	},
	"LFRJ": {
		ICAO:      "LFRJ",
		IATA:      "LDV",
		Name:      "Landivisiau Air Base",
		City:      "Landivisiau",
		State:     "Brittany",
		Country:   "FR",
		Elevation: 348,
		Latitude:  48.5303001404,
		Longitude: -4.1516399384,
		Timezone:  "Europe/Paris",
	},
	"LFRK": {
		ICAO:      "LFRK",
		IATA:      "CFR",
		Name:      "Caen-Carpiquet Airport",
		City:      "Caen/Carpiquet",
		State:     "Normandy",
		Country:   "FR",
		Elevation: 256,
		Latitude:  49.1733016968,
		Longitude: -0.4499999881,
		Timezone:  "Europe/Paris",
	},
	"LFRL": {
		ICAO:      "LFRL",
		Name:      "Lanveoc-Poulmic Air Base",
		City:      "Lanveoc/Poulmic",
		State:     "Brittany",
		Country:   "FR",
		Elevation: 287,
		Latitude:  48.2817001343,
		Longitude: -4.4450201988,
		Timezone:  "Europe/Paris",
	},
	"LFRM": {
		ICAO:      "LFRM",
		IATA:      "LME",
		Name:      "Le Mans-Arnage Airport",
		City:      "Le Mans/Arnage",
		State:     "Pays-de-la-Loire",
		Country:   "FR",
		Elevation: 194,
		Latitude:  47.948600769,
		Longitude: 0.2016669959,
		Timezone:  "Europe/Paris",
	},
	"LFRN": {
		ICAO:      "LFRN",
		IATA:      "RNS",
		Name:      "Rennes-Saint-Jacques Airport",
		City:      "Rennes/Saint-Jacques",
		State:     "Brittany",
		Country:   "FR",
		Elevation: 124,
		Latitude:  48.0694999695,
		Longitude: -1.7347899675,
		Timezone:  "Europe/Paris",
	},
	"LFRO": {
		ICAO:      "LFRO",
		IATA:      "LAI",
		Name:      "Lannion-Cote de Granit Airport",
		City:      "Lannion",
		State:     "Brittany",
		Country:   "FR",
		Elevation: 290,
		Latitude:  48.7543983459,
		Longitude: -3.4716598988,
		Timezone:  "Europe/Paris",
	},
	"LFRP": {
		ICAO:      "LFRP",
		Name:      "Ploermel Loyat Airport",
		City:      "Rennes",
		State:     "Brittany",
		Country:   "FR",
		Elevation: 236,
		Latitude:  48.0027999878,
		Longitude: -2.3772199154,
		Timezone:  "Europe/Paris",
	},
	"LFRQ": {
		ICAO:      "LFRQ",
		IATA:      "UIP",
		Name:      "Quimper-Cornouaille Airport",
		City:      "Quimper/Pluguffan",
		State:     "Brittany",
		Country:   "FR",
		Elevation: 297,
		Latitude:  47.9749984741,
		Longitude: -4.1677899361,
		Timezone:  "Europe/Paris",
	},
	"LFRS": {
		ICAO:      "LFRS",
		IATA:      "NTE",
		Name:      "Nantes Atlantique Airport",
		City:      "Nantes",
		State:     "Pays-de-la-Loire",
		Country:   "FR",
		Elevation: 90,
		Latitude:  47.1531982422,
		Longitude: -1.610730052,
		Timezone:  "Europe/Paris",
	},
	"LFRT": {
		ICAO:      "LFRT",
		IATA:      "SBK",
		Name:      "Saint-Brieuc-Armor Airport",
		City:      "Saint-Brieuc/Armor",
		State:     "Brittany",
		Country:   "FR",
		Elevation: 453,
		Latitude:  48.5377998352,
		Longitude: -2.8544399738,
		Timezone:  "Europe/Paris",
	},
	"LFRU": {
		ICAO:      "LFRU",
		IATA:      "MXN",
		Name:      "Morlaix-Ploujean Airport",
		City:      "Morlaix/Ploujean",
		State:     "Brittany",
		Country:   "FR",
		Elevation: 272,
		Latitude:  48.6031990051,
		Longitude: -3.8157799244,
		Timezone:  "Europe/Paris",
	},
	"LFRV": {
		ICAO:      "LFRV",
		IATA:      "VNE",
		Name:      "Vannes-Meucon Airport",
		City:      "Vannes/Meucon",
		State:     "Brittany",
		Country:   "FR",
		Elevation: 446,
		Latitude:  47.7233009338,
		Longitude: -2.7185599804,
		Timezone:  "Europe/Paris",
	},
	"LFRW": {
		ICAO:      "LFRW",
		Name:      "Avranches Le Val St Pere Airport",
		City:      "Vannes",
		State:     "Normandy",
		Country:   "FR",
		Elevation: 26,
		Latitude:  48.6617012024,
		Longitude: -1.4044400454,
		Timezone:  "Europe/Paris",
	},
	"LFRZ": {
		ICAO:      "LFRZ",
		IATA:      "SNR",
		Name:      "Saint-Nazaire-Montoir Airport",
		City:      "Saint-Nazaire/Montoir",
		State:     "Pays-de-la-Loire",
		Country:   "FR",
		Elevation: 13,
		Latitude:  47.3121986389,
		Longitude: -2.1491799355,
		Timezone:  "Europe/Paris",
	},
	"LFSA": {
		ICAO:      "LFSA",
		IATA:      "QBQ",
		Name:      "Besancon Thise Airport",
		City:      "St Nazaire",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 807,
		Latitude:  47.2747001648,
		Longitude: 6.0841698647,
		Timezone:  "Europe/Paris",
	},
	"LFSB": {
		ICAO:      "LFSB",
		IATA:      "BSL",
		Name:      "EuroAirport Basel-Mulhouse-Freiburg Airport",
		City:      "Bale/Mulhouse",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 885,
		Latitude:  47.5895996094,
		Longitude: 7.5299100876,
		Timezone:  "Europe/Paris",
	},
	"LFSC": {
		ICAO:      "LFSC",
		Name:      "Colmar-Meyenheim Air Base",
		City:      "Colmar/Meyenheim",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 693,
		Latitude:  47.922000885,
		Longitude: 7.3996701241,
		Timezone:  "Europe/Paris",
	},
	"LFSD": {
		ICAO:      "LFSD",
		IATA:      "DIJ",
		Name:      "Dijon-Bourgogne Airport",
		City:      "Dijon/Longvic",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 726,
		Latitude:  47.268901825,
		Longitude: 5.0900001526,
		Timezone:  "Europe/Paris",
	},
	"LFSE": {
		ICAO:      "LFSE",
		Name:      "Epinal Dogneville Airport",
		City:      "Dijon",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 1040,
		Latitude:  48.2118988037,
		Longitude: 6.4491701126,
		Timezone:  "Europe/Paris",
	},
	"LFSF": {
		ICAO:      "LFSF",
		IATA:      "MZM",
		Name:      "Metz-Frescaty (BA 128) Air Base",
		City:      "Metz/Frescaty",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 629,
		Latitude:  49.0717010498,
		Longitude: 6.1316699982,
		Timezone:  "Europe/Paris",
	},
	"LFSG": {
		ICAO:      "LFSG",
		IATA:      "EPL",
		Name:      "Epinal-Mirecourt Airport",
		City:      "Epinal/Mirecourt",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 1084,
		Latitude:  48.3250007629,
		Longitude: 6.0699801445,
		Timezone:  "Europe/Paris",
	},
	"LFSH": {
		ICAO:      "LFSH",
		Name:      "Haguenau Airport",
		City:      "Haguenau",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 491,
		Latitude:  48.7943000793,
		Longitude: 7.817609787,
		Timezone:  "Europe/Paris",
	},
	"LFSI": {
		ICAO:      "LFSI",
		Name:      "Saint-Dizier-Robinson (BA 113) Air Base",
		City:      "Saint-Dizier/Robinson",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 458,
		Latitude:  48.6360015869,
		Longitude: 4.8994197845,
		Timezone:  "Europe/Paris",
	},
	"LFSJ": {
		ICAO:      "LFSJ",
		Name:      "Sedan Douzy Airport",
		City:      "St Dizier",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 512,
		Latitude:  49.6596984863,
		Longitude: 5.037779808,
		Timezone:  "Europe/Paris",
	},
	"LFSK": {
		ICAO:      "LFSK",
		Name:      "Vitry Le Francois Vauclerc Airport",
		City:      "St Dizier",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 410,
		Latitude:  48.7033004761,
		Longitude: 4.684440136,
		Timezone:  "Europe/Paris",
	},
	"LFSL": {
		ICAO:      "LFSL",
		IATA:      "BVE",
		Name:      "Toul Rosieres Air Base",
		City:      "St Dizier",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 936,
		Latitude:  48.7799987793,
		Longitude: 5.9800000191,
		Timezone:  "Europe/Paris",
	},
	"LFSM": {
		ICAO:      "LFSM",
		IATA:      "XMF",
		Name:      "Montbeliard-Courcelles Airport",
		City:      "Montbeliard/Courcelles",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 1041,
		Latitude:  47.4869995117,
		Longitude: 6.7905402184,
		Timezone:  "Europe/Paris",
	},
	"LFSN": {
		ICAO:      "LFSN",
		IATA:      "ENC",
		Name:      "Nancy-Essey Airport",
		City:      "Nancy/Essey",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 751,
		Latitude:  48.6921005249,
		Longitude: 6.2304601669,
		Timezone:  "Europe/Paris",
	},
	"LFSO": {
		ICAO:      "LFSO",
		Name:      "Nancy-Ochey (BA 133) Air Base",
		City:      "Nancy/Ochey",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 1106,
		Latitude:  48.5830993652,
		Longitude: 5.9549999237,
		Timezone:  "Europe/Paris",
	},
	"LFSP": {
		ICAO:      "LFSP",
		Name:      "Pontarlier Airport",
		City:      "Pontarlier",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 2683,
		Latitude:  46.9039993286,
		Longitude: 6.3273701668,
		Timezone:  "Europe/Paris",
	},
	"LFSQ": {
		ICAO:      "LFSQ",
		IATA:      "BOR",
		Name:      "Fontaine Airport",
		City:      "Belfort",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 1208,
		Latitude:  47.6556015015,
		Longitude: 7.0108299255,
		Timezone:  "Europe/Paris",
	},
	"LFSR": {
		ICAO:      "LFSR",
		IATA:      "RHE",
		Name:      "Reims-Champagne (BA 112) Airport",
		City:      "Reims/Champagne",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 312,
		Latitude:  49.3100013733,
		Longitude: 4.0500001907,
		Timezone:  "Europe/Paris",
	},
	"LFST": {
		ICAO:      "LFST",
		IATA:      "SXB",
		Name:      "Strasbourg Airport",
		City:      "Strasbourg",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 505,
		Latitude:  48.5382995605,
		Longitude: 7.6282300949,
		Timezone:  "Europe/Paris",
	},
	"LFSU": {
		ICAO:      "LFSU",
		Name:      "Langres Rolampont Airport",
		City:      "Strasbourg",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 1378,
		Latitude:  47.9655990601,
		Longitude: 5.2950000763,
		Timezone:  "Europe/Paris",
	},
	"LFSV": {
		ICAO:      "LFSV",
		Name:      "Pont St Vincent Airport",
		City:      "Strasbourg",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 1306,
		Latitude:  48.601398468,
		Longitude: 6.057779789,
		Timezone:  "Europe/Paris",
	},
	"LFSW": {
		ICAO:      "LFSW",
		Name:      "Epernay Plivot Airport",
		City:      "Strasbourg",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 289,
		Latitude:  49.0052986145,
		Longitude: 4.0869398117,
		Timezone:  "Europe/Paris",
	},
	"LFSX": {
		ICAO:      "LFSX",
		Name:      "Luxeuil-Saint-Sauveur (BA 116) Air Base",
		City:      "Luxeuil/Saint-Sauveur",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 913,
		Latitude:  47.7831001282,
		Longitude: 6.3640599251,
		Timezone:  "Europe/Paris",
	},
	"LFSY": {
		ICAO:      "LFSY",
		Name:      "Chaumont Airport",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 1175,
		Latitude:  48.0769004822,
		Longitude: 5.1513900757,
		Timezone:  "Europe/Paris",
	},
	"LFSZ": {
		ICAO:      "LFSZ",
		IATA:      "VTL",
		Name:      "Vittel Champ De Course Airport",
		City:      "Luxeuil",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 1076,
		Latitude:  48.2238998413,
		Longitude: 5.9352798462,
		Timezone:  "Europe/Paris",
	},
	"LFTF": {
		ICAO:      "LFTF",
		Name:      "Cuers-Pierrefeu Airport",
		City:      "Cuers/Pierrefeu",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Elevation: 266,
		Latitude:  43.2477989197,
		Longitude: 6.1266999245,
		Timezone:  "Europe/Paris",
	},
	"LFTH": {
		ICAO:      "LFTH",
		IATA:      "TLN",
		Name:      "Toulon-Hyeres Airport",
		City:      "Toulon/Hyeres/Le Palyvestre",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Elevation: 7,
		Latitude:  43.0973014832,
		Longitude: 6.1460299492,
		Timezone:  "Europe/Paris",
	},
	"LFTM": {
		ICAO:      "LFTM",
		Name:      "Serres - La Batie-Montsaleon Airport",
		City:      "Hyeres",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Elevation: 2345,
		Latitude:  44.4583015442,
		Longitude: 5.7283301353,
		Timezone:  "Europe/Paris",
	},
	"LFTN": {
		ICAO:      "LFTN",
		Name:      "La Grand'combe Airport",
		City:      "Hyeres",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 1647,
		Latitude:  44.2444000244,
		Longitude: 4.0122199059,
		Timezone:  "Europe/Paris",
	},
	"LFTP": {
		ICAO:      "LFTP",
		Name:      "Puimoisson Airport",
		City:      "Hyeres",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Elevation: 2523,
		Latitude:  43.870300293,
		Longitude: 6.1647200584,
		Timezone:  "Europe/Paris",
	},
	"LFTQ": {
		ICAO:      "LFTQ",
		Name:      "Chateaubriant Pouance Airport",
		City:      "Hyeres",
		State:     "Pays-de-la-Loire",
		Country:   "FR",
		Elevation: 322,
		Latitude:  47.7406005859,
		Longitude: -1.1880600452,
		Timezone:  "Europe/Paris",
	},
	"LFTR": {
		ICAO:      "LFTR",
		Name:      "Toulo(St Mandrier Navy) Airport",
		City:      "Hyeres",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Latitude:  43.082698822,
		Longitude: 5.9338197708,
		Timezone:  "Europe/Paris",
	},
	"LFTU": {
		ICAO:      "LFTU",
		Name:      "Frejus Airport",
		City:      "Hyeres",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Latitude:  43.4146995544,
		Longitude: 6.7455601692,
		Timezone:  "Europe/Paris",
	},
	"LFTW": {
		ICAO:      "LFTW",
		IATA:      "FNI",
		Name:      "Nimes-Arles-Camargue Airport",
		City:      "Nimes/Garons",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 309,
		Latitude:  43.7574005127,
		Longitude: 4.4163498878,
		Timezone:  "Europe/Paris",
	},
	"LFTZ": {
		ICAO:      "LFTZ",
		IATA:      "LTT",
		Name:      "La Mole Airport",
		City:      "La Mole",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Elevation: 59,
		Latitude:  43.2053985596,
		Longitude: 6.4819998741,
		Timezone:  "Europe/Paris",
	},
	"LFVM": {
		ICAO:      "LFVM",
		IATA:      "MQC",
		Name:      "Miquelon Airport",
		City:      "Miquelon",
		State:     "Miquelon-Langlade",
		Country:   "PM",
		Elevation: 10,
		Latitude:  47.095500946,
		Longitude: -56.3802986145,
		Timezone:  "America/Miquelon",
	},
	"LFVP": {
		ICAO:      "LFVP",
		IATA:      "FSP",
		Name:      "St Pierre Airport",
		City:      "Saint-Pierre",
		Country:   "PM",
		Elevation: 27,
		Latitude:  46.7629013062,
		Longitude: -56.1730995178,
		Timezone:  "America/Miquelon",
	},
	"LFXA": {
		ICAO:      "LFXA",
		Name:      "Amberieu Airport (BA 278)",
		City:      "Amberieu",
		State:     "Auvergne-Rhone-Alpes",
		Country:   "FR",
		Elevation: 823,
		Latitude:  45.9873008728,
		Longitude: 5.3284401894,
		Timezone:  "Europe/Paris",
	},
	"LFXB": {
		ICAO:      "LFXB",
		Name:      "Saintes Thenac Airport",
		City:      "St Pierre I",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 118,
		Latitude:  45.7019004822,
		Longitude: -0.636111021,
		Timezone:  "Europe/Paris",
	},
	"LFXC": {
		ICAO:      "LFXC",
		Name:      "Vittel (Auzainvilliers) Airport",
		City:      "St Pierre I",
		State:     "Grand-Est",
		Country:   "FR",
		Latitude:  48.2285995483,
		Longitude: 5.8430199623,
		Timezone:  "Europe/Paris",
	},
	"LFXG": {
		ICAO:      "LFXG",
		Name:      "Bitche Airport",
		City:      "St Pierre I",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 984,
		Latitude:  49.0666999817,
		Longitude: 7.4499998093,
		Timezone:  "Europe/Paris",
	},
	"LFXH": {
		ICAO:      "LFXH",
		Name:      "Le Valdahon Air Base",
		City:      "St Pierre I",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 2067,
		Latitude:  47.1666984558,
		Longitude: 6.3499999046,
		Timezone:  "Europe/Paris",
	},
	"LFXI": {
		ICAO:      "LFXI",
		Name:      "Apt St Christol Airport",
		City:      "St Pierre I",
		State:     "Provence-Alpes-Cote-d'Azur",
		Country:   "FR",
		Elevation: 2736,
		Latitude:  44.0532989502,
		Longitude: 5.4949998856,
		Timezone:  "Europe/Paris",
	},
	"LFXM": {
		ICAO:      "LFXM",
		Name:      "Mourmelon Airport",
		City:      "St Pierre I",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 387,
		Latitude:  49.1114006042,
		Longitude: 4.3669400215,
		Timezone:  "Europe/Paris",
	},
	"LFXQ": {
		ICAO:      "LFXQ",
		Name:      "Coetquidan Air Base",
		City:      "Guer",
		State:     "Brittany",
		Country:   "FR",
		Latitude:  47.9466018677,
		Longitude: -2.1923000813,
		Timezone:  "Europe/Paris",
	},
	"LFXR": {
		ICAO:      "LFXR",
		Name:      "Rochefort Soubise Airport",
		City:      "St Pierre I",
		State:     "Nouvelle-Aquitaine",
		Country:   "FR",
		Elevation: 13,
		Latitude:  45.9308013916,
		Longitude: -0.9952780008,
		Timezone:  "Europe/Paris",
	},
	"LFXU": {
		ICAO:      "LFXU",
		Name:      "Les Mureaux Airport",
		City:      "St Pierre I",
		State:     "Ile-de-France",
		Country:   "FR",
		Elevation: 89,
		Latitude:  48.998298645,
		Longitude: 1.9427800179,
		Timezone:  "Europe/Paris",
	},
	"LFYD": {
		ICAO:      "LFYD",
		Name:      "Damblain Airport",
		City:      "Damblain",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 1280,
		Latitude:  48.0862998962,
		Longitude: 5.6640601158,
		Timezone:  "Europe/Paris",
	},
	"LFYG": {
		ICAO:      "LFYG",
		Name:      "Cambrai Niergnies Airport",
		City:      "St Pierre I",
		State:     "Hauts-de-France",
		Country:   "FR",
		Elevation: 312,
		Latitude:  50.1425018311,
		Longitude: 3.2650001049,
		Timezone:  "Europe/Paris",
	},
	"LFYH": {
		ICAO:      "LFYH",
		Name:      "Broye Les Pesmes Airport",
		City:      "St Pierre I",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 682,
		Latitude:  47.3166999817,
		Longitude: 5.5166702271,
		Timezone:  "Europe/Paris",
	},
	"LFYK": {
		ICAO:      "LFYK",
		Name:      "Marville Montmedy Airport",
		City:      "St Pierre I",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 909,
		Latitude:  49.4592018127,
		Longitude: 5.4025001526,
		Timezone:  "Europe/Paris",
	},
	"LFYL": {
		ICAO:      "LFYL",
		Name:      "Lure Malbouhans Airport",
		City:      "St Pierre I",
		State:     "Bourgogne-Franche-Comte",
		Country:   "FR",
		Elevation: 1040,
		Latitude:  47.70470047,
		Longitude: 6.5458297729,
		Timezone:  "Europe/Paris",
	},
	"LFYM": {
		ICAO:      "LFYM",
		Name:      "Marigny Le Grand Airport",
		City:      "St Pierre I",
		State:     "Grand-Est",
		Country:   "FR",
		Elevation: 329,
		Latitude:  48.6599998474,
		Longitude: 3.8338899612,
		Timezone:  "Europe/Paris",
	},
	"LFYR": {
		ICAO:      "LFYR",
		Name:      "Romorantin Pruniers Airport",
		City:      "St Pierre I",
		State:     "Centre",
		Country:   "FR",
		Elevation: 289,
		Latitude:  47.3208007813,
		Longitude: 1.6888899803,
		Timezone:  "Europe/Paris",
	},
	"LFYS": {
		ICAO:      "LFYS",
		Name:      "Ste Leocadie Airport",
		City:      "St Pierre I",
		State:     "Occitanie",
		Country:   "FR",
		Elevation: 4331,
		Latitude:  42.4472007751,
		Longitude: 2.0108299255,
		Timezone:  "Europe/Madrid",
	},
	"LFYT": {
		ICAO:      "LFYT",
		Name:      "Saint-Simon - Clastres Air Base",
		City:      "St Pierre I",
		State:     "Hauts-de-France",
		Country:   "FR",
		Elevation: 295,
		Latitude:  49.75,
		Longitude: 3.2166700363,
		Timezone:  "Europe/Paris",
	},
	"LG53": {
		ICAO:      "LG53",
		Name:      "Lamia Airport",
		City:      "Lamia",
		State:     "Central-Greece",
		Country:   "GR",
		Elevation: 38,
		Latitude:  38.8745994568,
		Longitude: 22.4349994659,
		Timezone:  "Europe/Athens",
	},
	"LG54": {
		ICAO:      "LG54",
		Name:      "Tympaki Airport",
		City:      "Crete Island",
		State:     "Crete",
		Country:   "GR",
		Elevation: 7,
		Latitude:  35.0637016296,
		Longitude: 24.7674007416,
		Timezone:  "Europe/Athens",
	},
	"LG55": {
		ICAO:      "LG55",
		Name:      "Triodhon Airport",
		City:      "Triodhon",
		State:     "Peloponnese",
		Country:   "GR",
		Elevation: 36,
		Latitude:  37.0941009521,
		Longitude: 21.9904003143,
		Timezone:  "Europe/Athens",
	},
	"LG56": {
		ICAO:      "LG56",
		Name:      "Olimboi Airport",
		City:      "Chios Island",
		State:     "North-Aegean",
		Country:   "GR",
		Elevation: 656,
		Latitude:  38.2360992432,
		Longitude: 25.9391994476,
		Timezone:  "Europe/Athens",
	},
	"LGAD": {
		ICAO:      "LGAD",
		IATA:      "PYR",
		Name:      "Andravida Airport",
		City:      "Andravida",
		State:     "West-Greece",
		Country:   "GR",
		Elevation: 55,
		Latitude:  37.9207000732,
		Longitude: 21.2926006317,
		Timezone:  "Europe/Athens",
	},
	"LGAG": {
		ICAO:      "LGAG",
		IATA:      "AGQ",
		Name:      "Agrinion Airport",
		City:      "Agrinion",
		State:     "West-Greece",
		Country:   "GR",
		Elevation: 154,
		Latitude:  38.6020011902,
		Longitude: 21.3512001038,
		Timezone:  "Europe/Athens",
	},
	"LGAL": {
		ICAO:      "LGAL",
		IATA:      "AXD",
		Name:      "Dimokritos Airport",
		City:      "Alexandroupolis",
		State:     "East-Macedonia-and-Thrace",
		Country:   "GR",
		Elevation: 24,
		Latitude:  40.8558998108,
		Longitude: 25.9563007355,
		Timezone:  "Europe/Athens",
	},
	"LGAT": {
		ICAO:      "LGAT",
		IATA:      "HEW",
		Name:      "Athen Helenikon Airport",
		City:      "Athens",
		State:     "Attica",
		Country:   "GR",
		Elevation: 69,
		Latitude:  37.8932991028,
		Longitude: 23.7261009216,
		Timezone:  "Europe/Athens",
	},
	"LGAV": {
		ICAO:      "LGAV",
		IATA:      "ATH",
		Name:      "Eleftherios Venizelos International Airport",
		City:      "Athens",
		State:     "Attica",
		Country:   "GR",
		Elevation: 308,
		Latitude:  37.9364013672,
		Longitude: 23.9444999695,
		Timezone:  "Europe/Athens",
	},
	"LGAX": {
		ICAO:      "LGAX",
		Name:      "Alexandria Airport",
		City:      "Alexandria",
		State:     "Central-Macedonia",
		Country:   "GR",
		Elevation: 27,
		Latitude:  40.6511001587,
		Longitude: 22.4887008667,
		Timezone:  "Europe/Athens",
	},
	"LGBL": {
		ICAO:      "LGBL",
		IATA:      "VOL",
		Name:      "Nea Anchialos Airport",
		City:      "Nea Anchialos",
		State:     "Thessaly",
		Country:   "GR",
		Elevation: 83,
		Latitude:  39.2196006775,
		Longitude: 22.7943000793,
		Timezone:  "Europe/Athens",
	},
	"LGEL": {
		ICAO:      "LGEL",
		Name:      "Elefsis Airport",
		City:      "Elefsina",
		State:     "Attica",
		Country:   "GR",
		Elevation: 143,
		Latitude:  38.0638008118,
		Longitude: 23.5559997559,
		Timezone:  "Europe/Athens",
	},
	"LGEP": {
		ICAO:      "LGEP",
		Name:      "Epitalion Airport",
		City:      "Pyrgos",
		State:     "West-Greece",
		Country:   "GR",
		Elevation: 3,
		Latitude:  37.613899231,
		Longitude: 21.4981002808,
		Timezone:  "Europe/Athens",
	},
	"LGHI": {
		ICAO:      "LGHI",
		IATA:      "JKH",
		Name:      "Chios Island National Airport",
		City:      "Chios Island",
		State:     "North-Aegean",
		Country:   "GR",
		Elevation: 15,
		Latitude:  38.3432006836,
		Longitude: 26.1406002045,
		Timezone:  "Europe/Athens",
	},
	"LGHL": {
		ICAO:      "LGHL",
		IATA:      "PKH",
		Name:      "Porto Cheli Airport",
		City:      "Porto Cheli",
		State:     "Peloponnese",
		Country:   "GR",
		Elevation: 69,
		Latitude:  37.2975006104,
		Longitude: 23.1478004456,
		Timezone:  "Europe/Athens",
	},
	"LGIK": {
		ICAO:      "LGIK",
		IATA:      "JIK",
		Name:      "Ikaria Airport",
		City:      "Ikaria Island",
		State:     "North-Aegean",
		Country:   "GR",
		Elevation: 79,
		Latitude:  37.6827011108,
		Longitude: 26.3470993042,
		Timezone:  "Europe/Athens",
	},
	"LGIO": {
		ICAO:      "LGIO",
		IATA:      "IOA",
		Name:      "Ioannina Airport",
		City:      "Ioannina",
		State:     "Epirus",
		Country:   "GR",
		Elevation: 1558,
		Latitude:  39.6963996887,
		Longitude: 20.8225002289,
		Timezone:  "Europe/Athens",
	},
	"LGIR": {
		ICAO:      "LGIR",
		IATA:      "HER",
		Name:      "Heraklion International Nikos Kazantzakis Airport",
		City:      "Heraklion",
		State:     "Crete",
		Country:   "GR",
		Elevation: 115,
		Latitude:  35.3396987915,
		Longitude: 25.1802997589,
		Timezone:  "Europe/Athens",
	},
	"LGKA": {
		ICAO:      "LGKA",
		IATA:      "KSO",
		Name:      "Kastoria National Airport",
		City:      "Kastoria",
		State:     "West-Macedonia",
		Country:   "GR",
		Elevation: 2167,
		Latitude:  40.4463005066,
		Longitude: 21.2821998596,
		Timezone:  "Europe/Athens",
	},
	"LGKC": {
		ICAO:      "LGKC",
		IATA:      "KIT",
		Name:      "Kithira Airport",
		City:      "Kithira Island",
		State:     "Attica",
		Country:   "GR",
		Elevation: 1045,
		Latitude:  36.2742996216,
		Longitude: 23.0170001984,
		Timezone:  "Europe/Athens",
	},
	"LGKF": {
		ICAO:      "LGKF",
		IATA:      "EFL",
		Name:      "Kefallinia Airport",
		City:      "Kefallinia Island",
		State:     "Ionian-Islands",
		Country:   "GR",
		Elevation: 59,
		Latitude:  38.120098114,
		Longitude: 20.5004997253,
		Timezone:  "Europe/Athens",
	},
	"LGKJ": {
		ICAO:      "LGKJ",
		IATA:      "KZS",
		Name:      "Kastelorizo Airport",
		City:      "Kastelorizo Island",
		State:     "South-Aegean",
		Country:   "GR",
		Elevation: 489,
		Latitude:  36.1417007446,
		Longitude: 29.5764007568,
		Timezone:  "Europe/Athens",
	},
	"LGKL": {
		ICAO:      "LGKL",
		IATA:      "KLX",
		Name:      "Kalamata Airport",
		City:      "Kalamata",
		State:     "Peloponnese",
		Country:   "GR",
		Elevation: 26,
		Latitude:  37.0682983398,
		Longitude: 22.0254993439,
		Timezone:  "Europe/Athens",
	},
	"LGKM": {
		ICAO:      "LGKM",
		Name:      "Amigdhaleon Airport",
		City:      "Filipos",
		State:     "East-Macedonia-and-Thrace",
		Country:   "GR",
		Elevation: 203,
		Latitude:  40.9728012085,
		Longitude: 24.3414001465,
		Timezone:  "Europe/Athens",
	},
	"LGKN": {
		ICAO:      "LGKN",
		Name:      "Marathon Kotroni Airport",
		City:      "Marathon",
		State:     "Attica",
		Country:   "GR",
		Elevation: 682,
		Latitude:  38.1377983093,
		Longitude: 23.9510993958,
		Timezone:  "Europe/Athens",
	},
	"LGKO": {
		ICAO:      "LGKO",
		IATA:      "KGS",
		Name:      "Kos Airport",
		City:      "Kos Island",
		State:     "South-Aegean",
		Country:   "GR",
		Elevation: 412,
		Latitude:  36.7933006287,
		Longitude: 27.0916996002,
		Timezone:  "Europe/Athens",
	},
	"LGKP": {
		ICAO:      "LGKP",
		IATA:      "AOK",
		Name:      "Karpathos Airport",
		City:      "Karpathos Island",
		State:     "South-Aegean",
		Country:   "GR",
		Elevation: 66,
		Latitude:  35.4213981628,
		Longitude: 27.1459999084,
		Timezone:  "Europe/Athens",
	},
	"LGKR": {
		ICAO:      "LGKR",
		IATA:      "CFU",
		Name:      "Ioannis Kapodistrias International Airport",
		City:      "Kerkyra Island",
		State:     "Ionian-Islands",
		Country:   "GR",
		Elevation: 6,
		Latitude:  39.6018981934,
		Longitude: 19.911699295,
		Timezone:  "Europe/Athens",
	},
	"LGKS": {
		ICAO:      "LGKS",
		IATA:      "KSJ",
		Name:      "Kasos Airport",
		City:      "Kasos Island",
		State:     "South-Aegean",
		Country:   "GR",
		Elevation: 35,
		Latitude:  35.4213981628,
		Longitude: 26.9099998474,
		Timezone:  "Europe/Athens",
	},
	"LGKV": {
		ICAO:      "LGKV",
		IATA:      "KVA",
		Name:      "Alexander the Great International Airport",
		City:      "Kavala",
		State:     "East-Macedonia-and-Thrace",
		Country:   "GR",
		Elevation: 18,
		Latitude:  40.9132995605,
		Longitude: 24.6191997528,
		Timezone:  "Europe/Athens",
	},
	"LGKY": {
		ICAO:      "LGKY",
		IATA:      "JKL",
		Name:      "Kalymnos Airport",
		City:      "Kalymnos Island",
		State:     "South-Aegean",
		Country:   "GR",
		Elevation: 771,
		Latitude:  36.9632987976,
		Longitude: 26.9405994415,
		Timezone:  "Europe/Athens",
	},
	"LGKZ": {
		ICAO:      "LGKZ",
		IATA:      "KZI",
		Name:      "Filippos Airport",
		City:      "Kozani",
		State:     "West-Macedonia",
		Country:   "GR",
		Elevation: 2059,
		Latitude:  40.2860984802,
		Longitude: 21.8407993317,
		Timezone:  "Europe/Athens",
	},
	"LGLE": {
		ICAO:      "LGLE",
		IATA:      "LRS",
		Name:      "Leros Airport",
		City:      "Leros Island",
		State:     "South-Aegean",
		Country:   "GR",
		Elevation: 39,
		Latitude:  37.1848983765,
		Longitude: 26.8003005981,
		Timezone:  "Europe/Athens",
	},
	"LGLM": {
		ICAO:      "LGLM",
		IATA:      "LXS",
		Name:      "Limnos Airport",
		City:      "Limnos Island",
		State:     "North-Aegean",
		Country:   "GR",
		Elevation: 14,
		Latitude:  39.917098999,
		Longitude: 25.2362995148,
		Timezone:  "Europe/Athens",
	},
	"LGLR": {
		ICAO:      "LGLR",
		IATA:      "LRA",
		Name:      "Larisa Airport",
		City:      "Larisa",
		State:     "Thessaly",
		Country:   "GR",
		Elevation: 241,
		Latitude:  39.6562825886,
		Longitude: 22.3400008678,
		Timezone:  "Europe/Athens",
	},
	"LGMG": {
		ICAO:      "LGMG",
		Name:      "Megara Airport",
		City:      "Megara",
		State:     "Attica",
		Country:   "GR",
		Elevation: 12,
		Latitude:  37.981098175,
		Longitude: 23.3654003143,
		Timezone:  "Europe/Athens",
	},
	"LGMK": {
		ICAO:      "LGMK",
		IATA:      "JMK",
		Name:      "Mikonos Airport",
		City:      "Mykonos Island",
		State:     "South-Aegean",
		Country:   "GR",
		Elevation: 405,
		Latitude:  37.4351005554,
		Longitude: 25.3481006622,
		Timezone:  "Europe/Athens",
	},
	"LGML": {
		ICAO:      "LGML",
		IATA:      "MLO",
		Name:      "Milos Airport",
		City:      "Milos Island",
		State:     "South-Aegean",
		Country:   "GR",
		Elevation: 10,
		Latitude:  36.6968994141,
		Longitude: 24.4769001007,
		Timezone:  "Europe/Athens",
	},
	"LGMR": {
		ICAO:      "LGMR",
		Name:      "Marathon Wasser Airport",
		City:      "Marathon",
		State:     "Attica",
		Country:   "GR",
		Elevation: 3,
		Latitude:  38.143901825,
		Longitude: 24.0128002167,
		Timezone:  "Europe/Athens",
	},
	"LGMT": {
		ICAO:      "LGMT",
		IATA:      "MJT",
		Name:      "Mytilene International Airport",
		City:      "Mytilene",
		State:     "North-Aegean",
		Country:   "GR",
		Elevation: 60,
		Latitude:  39.0567016602,
		Longitude: 26.5983009338,
		Timezone:  "Europe/Athens",
	},
	"LGNX": {
		ICAO:      "LGNX",
		IATA:      "JNX",
		Name:      "Naxos Airport",
		City:      "Naxos Island",
		State:     "South-Aegean",
		Country:   "GR",
		Elevation: 10,
		Latitude:  37.0811004639,
		Longitude: 25.3680992126,
		Timezone:  "Europe/Athens",
	},
	"LGPA": {
		ICAO:      "LGPA",
		IATA:      "PAS",
		Name:      "Paros Airport",
		City:      "Paros Island",
		State:     "South-Aegean",
		Country:   "GR",
		Elevation: 131,
		Latitude:  37.0102996826,
		Longitude: 25.1280994415,
		Timezone:  "Europe/Athens",
	},
	"LGPL": {
		ICAO:      "LGPL",
		IATA:      "JTY",
		Name:      "Astypalaia Airport",
		City:      "Astypalaia Island",
		State:     "South-Aegean",
		Country:   "GR",
		Elevation: 165,
		Latitude:  36.5798988342,
		Longitude: 26.3757991791,
		Timezone:  "Europe/Athens",
	},
	"LGPZ": {
		ICAO:      "LGPZ",
		IATA:      "PVK",
		Name:      "Aktion National Airport",
		City:      "Preveza/Lefkada",
		State:     "West-Greece",
		Country:   "GR",
		Elevation: 11,
		Latitude:  38.9254989624,
		Longitude: 20.7653007507,
		Timezone:  "Europe/Athens",
	},
	"LGRD": {
		ICAO:      "LGRD",
		Name:      "Maritsa Airport",
		City:      "Rodes Island",
		State:     "South-Aegean",
		Country:   "GR",
		Elevation: 204,
		Latitude:  36.3830986023,
		Longitude: 28.1089000702,
		Timezone:  "Europe/Athens",
	},
	"LGRP": {
		ICAO:      "LGRP",
		IATA:      "RHO",
		Name:      "Diagoras Airport",
		City:      "Rodes Island",
		State:     "South-Aegean",
		Country:   "GR",
		Elevation: 17,
		Latitude:  36.4053993225,
		Longitude: 28.0862007141,
		Timezone:  "Europe/Athens",
	},
	"LGRS": {
		ICAO:      "LGRS",
		Name:      "Logan Reserve Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -27.7082996368,
		Longitude: 153.1049957275,
		Timezone:  "Australia/Brisbane",
	},
	"LGRX": {
		ICAO:      "LGRX",
		IATA:      "GPA",
		Name:      "Araxos Airport",
		City:      "Patras",
		State:     "West-Greece",
		Country:   "GR",
		Elevation: 46,
		Latitude:  38.1511001587,
		Longitude: 21.4256000519,
		Timezone:  "Europe/Athens",
	},
	"LGSA": {
		ICAO:      "LGSA",
		IATA:      "CHQ",
		Name:      "Chania International Airport",
		City:      "Souda",
		State:     "Crete",
		Country:   "GR",
		Elevation: 490,
		Latitude:  35.5317001343,
		Longitude: 24.1497001648,
		Timezone:  "Europe/Athens",
	},
	"LGSD": {
		ICAO:      "LGSD",
		Name:      "Sedes Airport",
		City:      "Sedes",
		State:     "Central-Macedonia",
		Country:   "GR",
		Elevation: 85,
		Latitude:  40.5331001282,
		Longitude: 23.0252990723,
		Timezone:  "Europe/Athens",
	},
	"LGSK": {
		ICAO:      "LGSK",
		IATA:      "JSI",
		Name:      "Skiathos Island National Airport",
		City:      "Skiathos",
		State:     "Thessaly",
		Country:   "GR",
		Elevation: 54,
		Latitude:  39.1771011353,
		Longitude: 23.5037002563,
		Timezone:  "Europe/Athens",
	},
	"LGSM": {
		ICAO:      "LGSM",
		IATA:      "SMI",
		Name:      "Samos Airport",
		City:      "Samos Island",
		State:     "North-Aegean",
		Country:   "GR",
		Elevation: 19,
		Latitude:  37.6899986267,
		Longitude: 26.911699295,
		Timezone:  "Europe/Athens",
	},
	"LGSO": {
		ICAO:      "LGSO",
		IATA:      "JSY",
		Name:      "Syros Airport",
		City:      "Syros Island",
		State:     "South-Aegean",
		Country:   "GR",
		Elevation: 236,
		Latitude:  37.4227981567,
		Longitude: 24.9508991241,
		Timezone:  "Europe/Athens",
	},
	"LGSP": {
		ICAO:      "LGSP",
		IATA:      "SPJ",
		Name:      "Sparti Airport",
		City:      "Sparti",
		State:     "Peloponnese",
		Country:   "GR",
		Elevation: 500,
		Latitude:  36.9738998413,
		Longitude: 22.5263004303,
		Timezone:  "Europe/Athens",
	},
	"LGSR": {
		ICAO:      "LGSR",
		IATA:      "JTR",
		Name:      "Santorini Airport",
		City:      "Santorini Island",
		State:     "South-Aegean",
		Country:   "GR",
		Elevation: 127,
		Latitude:  36.3992004395,
		Longitude: 25.4792995453,
		Timezone:  "Europe/Athens",
	},
	"LGST": {
		ICAO:      "LGST",
		IATA:      "JSH",
		Name:      "Sitia Airport",
		City:      "Crete Island",
		State:     "Crete",
		Country:   "GR",
		Elevation: 376,
		Latitude:  35.2160987854,
		Longitude: 26.1012992859,
		Timezone:  "Europe/Athens",
	},
	"LGSV": {
		ICAO:      "LGSV",
		Name:      "Stefanovikion Airport",
		City:      "Stefanovilion",
		State:     "Thessaly",
		Country:   "GR",
		Elevation: 146,
		Latitude:  39.4799995422,
		Longitude: 22.76720047,
		Timezone:  "Europe/Athens",
	},
	"LGSY": {
		ICAO:      "LGSY",
		IATA:      "SKU",
		Name:      "Skiros Airport",
		City:      "Skiros Island",
		State:     "Central-Greece",
		Country:   "GR",
		Elevation: 44,
		Latitude:  38.9676017761,
		Longitude: 24.4871997833,
		Timezone:  "Europe/Athens",
	},
	"LGTG": {
		ICAO:      "LGTG",
		Name:      "Tanagra Airport",
		City:      "Tanagra",
		State:     "Central-Greece",
		Country:   "GR",
		Elevation: 485,
		Latitude:  38.3398017883,
		Longitude: 23.5650005341,
		Timezone:  "Europe/Athens",
	},
	"LGTL": {
		ICAO:      "LGTL",
		Name:      "Kasteli Airport",
		City:      "Crete Island",
		State:     "Crete",
		Country:   "GR",
		Elevation: 1180,
		Latitude:  35.1920013428,
		Longitude: 25.3269996643,
		Timezone:  "Europe/Athens",
	},
	"LGTP": {
		ICAO:      "LGTP",
		Name:      "Tripolis Airport",
		City:      "Tripoli",
		State:     "Peloponnese",
		Country:   "GR",
		Elevation: 2113,
		Latitude:  37.5306015015,
		Longitude: 22.4036006927,
		Timezone:  "Europe/Athens",
	},
	"LGTS": {
		ICAO:      "LGTS",
		IATA:      "SKG",
		Name:      "Thessaloniki Macedonia International Airport",
		City:      "Thessaloniki",
		State:     "Central-Macedonia",
		Country:   "GR",
		Elevation: 22,
		Latitude:  40.5196990967,
		Longitude: 22.9708995819,
		Timezone:  "Europe/Athens",
	},
	"LGTT": {
		ICAO:      "LGTT",
		Name:      "Tatoi Airport",
		City:      "Tatoi",
		State:     "Attica",
		Country:   "GR",
		Elevation: 785,
		Latitude:  38.1088981628,
		Longitude: 23.7838001251,
		Timezone:  "Europe/Athens",
	},
	"LGZA": {
		ICAO:      "LGZA",
		IATA:      "ZTH",
		Name:      "Dionysios Solomos Airport",
		City:      "Zakynthos Island",
		State:     "Ionian-Islands",
		Country:   "GR",
		Elevation: 15,
		Latitude:  37.7509002686,
		Longitude: 20.8843002319,
		Timezone:  "Europe/Athens",
	},
	"LHBA": {
		ICAO:      "LHBA",
		Name:      "Balassagyarmat Airport",
		City:      "Balassagyarmat",
		State:     "Nograd",
		Country:   "HU",
		Elevation: 466,
		Latitude:  48.078037262,
		Longitude: 19.3217487335,
		Timezone:  "Europe/Bratislava",
	},
	"LHBC": {
		ICAO:      "LHBC",
		Name:      "Bekescsaba Glider Airport",
		City:      "Bekescsaba",
		State:     "Bekes",
		Country:   "HU",
		Elevation: 279,
		Latitude:  46.6758003235,
		Longitude: 21.1622009277,
		Timezone:  "Europe/Budapest",
	},
	"LHBP": {
		ICAO:      "LHBP",
		IATA:      "BUD",
		Name:      "Budapest Liszt Ferenc International Airport",
		City:      "Budapest",
		State:     "Budapest",
		Country:   "HU",
		Elevation: 495,
		Latitude:  47.4369010925,
		Longitude: 19.2555999756,
		Timezone:  "Europe/Budapest",
	},
	"LHBS": {
		ICAO:      "LHBS",
		Name:      "Budaors Glider Airport",
		City:      "Budaors",
		State:     "Pest",
		Country:   "HU",
		Elevation: 397,
		Latitude:  47.4510993958,
		Longitude: 18.9806003571,
		Timezone:  "Europe/Budapest",
	},
	"LHDC": {
		ICAO:      "LHDC",
		IATA:      "DEB",
		Name:      "Debrecen International Airport",
		City:      "Debrecen",
		State:     "Hajdu-Bihar",
		Country:   "HU",
		Elevation: 359,
		Latitude:  47.488899231,
		Longitude: 21.6152992249,
		Timezone:  "Europe/Budapest",
	},
	"LHDK": {
		ICAO:      "LHDK",
		Name:      "Dunakeszi Airport",
		City:      "Dunakeszi",
		State:     "Pest",
		Country:   "HU",
		Elevation: 397,
		Latitude:  47.613899231,
		Longitude: 19.1432991028,
		Timezone:  "Europe/Budapest",
	},
	"LHDV": {
		ICAO:      "LHDV",
		Name:      "Dunaujvaros Glider Airport",
		City:      "Dunaujvaros",
		State:     "Fejer",
		Country:   "HU",
		Elevation: 394,
		Latitude:  46.8958015442,
		Longitude: 18.9139003754,
		Timezone:  "Europe/Budapest",
	},
	"LHEC": {
		ICAO:      "LHEC",
		Name:      "Ersekcsanad Airport",
		City:      "Ersekcsanad",
		State:     "Bacs-Kiskun",
		Country:   "HU",
		Elevation: 285,
		Latitude:  46.2462425232,
		Longitude: 18.9210224152,
		Timezone:  "Europe/Budapest",
	},
	"LHEM": {
		ICAO:      "LHEM",
		Name:      "Esztergom Airport",
		City:      "Esztergom",
		State:     "Komarom-Esztergom",
		Country:   "HU",
		Elevation: 342,
		Latitude:  47.7728004456,
		Longitude: 18.7217006683,
		Timezone:  "Europe/Bratislava",
	},
	"LHFH": {
		ICAO:      "LHFH",
		Name:      "Farkashegy Airport",
		City:      "Farkashegy",
		State:     "Pest",
		Country:   "HU",
		Elevation: 690,
		Latitude:  47.4869003296,
		Longitude: 18.9146995544,
		Timezone:  "Europe/Budapest",
	},
	"LHFM": {
		ICAO:      "LHFM",
		Name:      "Fertoszentmiklos Airport",
		City:      "Fertoszentmiklos",
		State:     "Gyor-Moson-Sopron",
		Country:   "HU",
		Elevation: 440,
		Latitude:  47.5828018188,
		Longitude: 16.8453998566,
		Timezone:  "Europe/Budapest",
	},
	"LHGD": {
		ICAO:      "LHGD",
		Name:      "Godollo Airport",
		City:      "Godollo",
		State:     "Pest",
		Country:   "HU",
		Elevation: 600,
		Latitude:  47.5709991455,
		Longitude: 19.3386001587,
		Timezone:  "Europe/Budapest",
	},
	"LHGM": {
		ICAO:      "LHGM",
		Name:      "Gyomai Airport",
		City:      "Gyomai",
		State:     "Bekes",
		Country:   "HU",
		Elevation: 265,
		Latitude:  46.9561653137,
		Longitude: 20.8831233978,
		Timezone:  "Europe/Budapest",
	},
	"LHGY": {
		ICAO:      "LHGY",
		Name:      "Gyongyos Pipishegy Airport",
		City:      "Gyongyos Pipishegy",
		State:     "Heves",
		Country:   "HU",
		Elevation: 1150,
		Latitude:  47.8144989014,
		Longitude: 19.9776000977,
		Timezone:  "Europe/Budapest",
	},
	"LHHH": {
		ICAO:      "LHHH",
		Name:      "Harmashatarhegy Airport",
		City:      "Budapest",
		State:     "Budapest",
		Country:   "HU",
		Elevation: 960,
		Latitude:  47.5503005981,
		Longitude: 18.9787006378,
		Timezone:  "Europe/Budapest",
	},
	"LHHO": {
		ICAO:      "LHHO",
		Name:      "Hajduszoboszlo Airport",
		City:      "Hajduszoboszlo",
		State:     "Hajdu-Bihar",
		Country:   "HU",
		Elevation: 335,
		Latitude:  47.4592018127,
		Longitude: 21.3910999298,
		Timezone:  "Europe/Budapest",
	},
	"LHJK": {
		ICAO:      "LHJK",
		Name:      "Jakabszallas Airport",
		City:      "Jakabszallas",
		State:     "Bacs-Kiskun",
		Country:   "HU",
		Elevation: 348,
		Latitude:  46.7472000122,
		Longitude: 19.6056003571,
		Timezone:  "Europe/Budapest",
	},
	"LHKD": {
		ICAO:      "LHKD",
		Name:      "Kecsked Airport",
		City:      "Kecsked",
		State:     "Komarom-Esztergom",
		Country:   "HU",
		Elevation: 570,
		Latitude:  47.507900238,
		Longitude: 18.3227005005,
		Timezone:  "Europe/Budapest",
	},
	"LHKE": {
		ICAO:      "LHKE",
		Name:      "Kecskemet Airport",
		City:      "Kecskemet",
		State:     "Bacs-Kiskun",
		Country:   "HU",
		Elevation: 376,
		Latitude:  46.9174995422,
		Longitude: 19.7492008209,
		Timezone:  "Europe/Budapest",
	},
	"LHKH": {
		ICAO:      "LHKH",
		Name:      "Kiskunfelegyhaza Airport",
		City:      "Kiskunfelegyhaza",
		State:     "Bacs-Kiskun",
		Country:   "HU",
		Elevation: 289,
		Latitude:  46.7331008911,
		Longitude: 19.8932991028,
		Timezone:  "Europe/Budapest",
	},
	"LHKK": {
		ICAO:      "LHKK",
		Name:      "Kiskunlachaza-airport",
		City:      "Budapest",
		State:     "Pest",
		Country:   "HU",
		Elevation: 322,
		Latitude:  47.1688852045,
		Longitude: 19.0849685669,
		Timezone:  "Europe/Budapest",
	},
	"LHKV": {
		ICAO:      "LHKV",
		Name:      "Kaposvar Kaposujlak Glider Airport",
		City:      "Kaposujlak",
		State:     "Somogy",
		Country:   "HU",
		Elevation: 499,
		Latitude:  46.3863983154,
		Longitude: 17.7339000702,
		Timezone:  "Europe/Budapest",
	},
	"LHMC": {
		ICAO:      "LHMC",
		IATA:      "MCQ",
		Name:      "Miskolc Airport",
		City:      "Miskolc",
		State:     "Borsod-Abauj-Zemplen",
		Country:   "HU",
		Elevation: 390,
		Latitude:  48.1369018555,
		Longitude: 20.7914009094,
		Timezone:  "Europe/Budapest",
	},
	"LHMR": {
		ICAO:      "LHMR",
		Name:      "Eger Maklar Airport",
		City:      "Eger",
		Country:   "HU",
		Elevation: 407,
		Latitude:  47.8111000061,
		Longitude: 20.4228000641,
		Timezone:  "Europe/Budapest",
	},
	"LHNK": {
		ICAO:      "LHNK",
		Name:      "Nagykanizsa Airport",
		City:      "Nagykanizsa",
		State:     "Zala",
		Country:   "HU",
		Elevation: 466,
		Latitude:  46.4320007859,
		Longitude: 16.9581699371,
		Timezone:  "Europe/Budapest",
	},
	"LHNY": {
		ICAO:      "LHNY",
		Name:      "Nyiregyhaza Airport",
		City:      "Nyiregyhaza",
		State:     "Szabolcs-Szatmar-Bereg",
		Country:   "HU",
		Elevation: 338,
		Latitude:  47.9838981628,
		Longitude: 21.6923007965,
		Timezone:  "Europe/Budapest",
	},
	"LHOY": {
		ICAO:      "LHOY",
		Name:      "Ocseny Airport",
		City:      "Ocseny",
		State:     "Tolna",
		Country:   "HU",
		Elevation: 295,
		Latitude:  46.3039016724,
		Longitude: 18.7691993713,
		Timezone:  "Europe/Budapest",
	},
	"LHPA": {
		ICAO:      "LHPA",
		Name:      "Papa Airport",
		City:      "Papa",
		State:     "Veszprem",
		Country:   "HU",
		Elevation: 466,
		Latitude:  47.3636016846,
		Longitude: 17.5007991791,
		Timezone:  "Europe/Budapest",
	},
	"LHPP": {
		ICAO:      "LHPP",
		IATA:      "PEV",
		Name:      "Pecs-Pogany Airport",
		City:      "Pecs-Pogany",
		State:     "Baranya",
		Country:   "HU",
		Elevation: 1000,
		Latitude:  45.9908981323,
		Longitude: 18.2409992218,
		Timezone:  "Europe/Budapest",
	},
	"LHPR": {
		ICAO:      "LHPR",
		IATA:      "QGY",
		Name:      "Gyor-Per International Airport",
		City:      "Gyor",
		State:     "Gyor-Moson-Sopron",
		Country:   "HU",
		Elevation: 424,
		Latitude:  47.6244010925,
		Longitude: 17.8136005402,
		Timezone:  "Europe/Budapest",
	},
	"LHSA": {
		ICAO:      "LHSA",
		Name:      "Szentkiralyszabadja Airport",
		City:      "Szentkiralyszabadja",
		State:     "Veszprem",
		Country:   "HU",
		Elevation: 919,
		Latitude:  47.0778999329,
		Longitude: 17.9683990479,
		Timezone:  "Europe/Budapest",
	},
	"LHSK": {
		ICAO:      "LHSK",
		Name:      "Siofok-Kiliti Airport",
		City:      "Siofok-Kiliti",
		State:     "Somogy",
		Country:   "HU",
		Elevation: 417,
		Latitude:  46.8568992615,
		Longitude: 18.0958003998,
		Timezone:  "Europe/Budapest",
	},
	"LHSM": {
		ICAO:      "LHSM",
		IATA:      "SOB",
		Name:      "Sarmellek International Airport",
		City:      "Sarmellek",
		State:     "Zala",
		Country:   "HU",
		Elevation: 408,
		Latitude:  46.6864013672,
		Longitude: 17.1590995789,
		Timezone:  "Europe/Budapest",
	},
	"LHSN": {
		ICAO:      "LHSN",
		Name:      "Szolnok Airport",
		City:      "Szolnok",
		State:     "Jasz-Nagykun-Szolnok",
		Country:   "HU",
		Elevation: 322,
		Latitude:  47.1228981018,
		Longitude: 20.2355003357,
		Timezone:  "Europe/Budapest",
	},
	"LHSS": {
		ICAO:      "LHSS",
		Name:      "Szolnok-Szandaszolos Sport Airport",
		City:      "Szolnok-Szandaszolos",
		State:     "Jasz-Nagykun-Szolnok",
		Country:   "HU",
		Elevation: 266,
		Latitude:  47.1444015503,
		Longitude: 20.1977996826,
		Timezone:  "Europe/Budapest",
	},
	"LHSY": {
		ICAO:      "LHSY",
		Name:      "Szombathely Info Airport",
		City:      "Szombathely",
		State:     "Vas",
		Country:   "HU",
		Elevation: 732,
		Latitude:  47.2799987793,
		Longitude: 16.6263999939,
		Timezone:  "Europe/Budapest",
	},
	"LHSZ": {
		ICAO:      "LHSZ",
		Name:      "Szentes Aerodrome",
		City:      "Szentes",
		State:     "Csongrad",
		Country:   "HU",
		Elevation: 259,
		Latitude:  46.6133003235,
		Longitude: 20.2828006744,
		Timezone:  "Europe/Budapest",
	},
	"LHTA": {
		ICAO:      "LHTA",
		IATA:      "TZR",
		Name:      "Taszar Air Base",
		City:      "Taszar",
		State:     "Somogy",
		Country:   "HU",
		Elevation: 531,
		Latitude:  46.3931007385,
		Longitude: 17.9174995422,
		Timezone:  "Europe/Budapest",
	},
	"LHTL": {
		ICAO:      "LHTL",
		Name:      "Tokol Air Base",
		City:      "Tokol",
		State:     "Pest",
		Country:   "HU",
		Elevation: 318,
		Latitude:  47.3452987671,
		Longitude: 18.9808006287,
		Timezone:  "Europe/Budapest",
	},
	"LHUD": {
		ICAO:      "LHUD",
		IATA:      "QZD",
		Name:      "Szeged Glider Airport",
		City:      "Szeged",
		State:     "Csongrad",
		Country:   "HU",
		Elevation: 249,
		Latitude:  46.2468986511,
		Longitude: 20.0907993317,
		Timezone:  "Europe/Budapest",
	},
	"LHZA": {
		ICAO:      "LHZA",
		Name:      "Zalaegerszeg Airport",
		City:      "Zalaegerszeg",
		State:     "Zala",
		Country:   "HU",
		Elevation: 617,
		Latitude:  46.8847007751,
		Longitude: 16.7889003754,
		Timezone:  "Europe/Budapest",
	},
	"LIAF": {
		ICAO:      "LIAF",
		Name:      "Foligno Airport",
		City:      "Foligno",
		State:     "Umbria",
		Country:   "IT",
		Elevation: 730,
		Latitude:  42.932800293,
		Longitude: 12.710100174,
		Timezone:  "Europe/Rome",
	},
	"LIAP": {
		ICAO:      "LIAP",
		IATA:      "QAQ",
		Name:      "L'Aquila / Preturo Airport",
		City:      "L'Aquila",
		State:     "Abruzzo",
		Country:   "IT",
		Elevation: 2211,
		Latitude:  42.379901886,
		Longitude: 13.3092002869,
		Timezone:  "Europe/Rome",
	},
	"LIAQ": {
		ICAO:      "LIAQ",
		Name:      "Aquino Airport",
		City:      "Aquino",
		State:     "Latium",
		Country:   "IT",
		Elevation: 361,
		Latitude:  41.4863014221,
		Longitude: 13.7187995911,
		Timezone:  "Europe/Rome",
	},
	"LIAU": {
		ICAO:      "LIAU",
		Name:      "Capua Airport",
		City:      "Capua",
		State:     "Campania",
		Country:   "IT",
		Elevation: 64,
		Latitude:  41.1160011292,
		Longitude: 14.1780004501,
		Timezone:  "Europe/Rome",
	},
	"LIBA": {
		ICAO:      "LIBA",
		Name:      "Amendola Airport",
		City:      "Foggia",
		State:     "Apulia",
		Country:   "IT",
		Elevation: 183,
		Latitude:  41.5414009094,
		Longitude: 15.7180995941,
		Timezone:  "Europe/Rome",
	},
	"LIBC": {
		ICAO:      "LIBC",
		IATA:      "CRV",
		Name:      "Crotone Airport",
		City:      "Crotone",
		State:     "Calabria",
		Country:   "IT",
		Elevation: 522,
		Latitude:  38.9972000122,
		Longitude: 17.0802001953,
		Timezone:  "Europe/Rome",
	},
	"LIBD": {
		ICAO:      "LIBD",
		IATA:      "BRI",
		Name:      "Bari / Palese International Airport",
		City:      "Bari",
		State:     "Apulia",
		Country:   "IT",
		Elevation: 177,
		Latitude:  41.1389007568,
		Longitude: 16.7605991364,
		Timezone:  "Europe/Rome",
	},
	"LIBF": {
		ICAO:      "LIBF",
		IATA:      "FOG",
		Name:      "Foggia / Gino Lisa Airport",
		City:      "Foggia",
		State:     "Apulia",
		Country:   "IT",
		Elevation: 265,
		Latitude:  41.4328994751,
		Longitude: 15.5349998474,
		Timezone:  "Europe/Rome",
	},
	"LIBG": {
		ICAO:      "LIBG",
		IATA:      "TAR",
		Name:      "Taranto / Grottaglie Airport",
		City:      "Grottaglie",
		State:     "Apulia",
		Country:   "IT",
		Elevation: 215,
		Latitude:  40.5175018311,
		Longitude: 17.4032001495,
		Timezone:  "Europe/Rome",
	},
	"LIBN": {
		ICAO:      "LIBN",
		IATA:      "LCC",
		Name:      "Lecce Airport",
		State:     "Apulia",
		Country:   "IT",
		Elevation: 156,
		Latitude:  40.239200592,
		Longitude: 18.1333007813,
		Timezone:  "Europe/Rome",
	},
	"LIBP": {
		ICAO:      "LIBP",
		IATA:      "PSR",
		Name:      "Pescara International Airport",
		City:      "Pescara",
		State:     "Abruzzo",
		Country:   "IT",
		Elevation: 48,
		Latitude:  42.4317016602,
		Longitude: 14.1810998917,
		Timezone:  "Europe/Rome",
	},
	"LIBR": {
		ICAO:      "LIBR",
		IATA:      "BDS",
		Name:      "Brindisi / Casale Airport",
		City:      "Brindisi",
		State:     "Apulia",
		Country:   "IT",
		Elevation: 47,
		Latitude:  40.6576004028,
		Longitude: 17.9470005035,
		Timezone:  "Europe/Rome",
	},
	"LIBV": {
		ICAO:      "LIBV",
		Name:      "Gioia Del Colle Airport",
		City:      "Gioia Del Colle",
		State:     "Apulia",
		Country:   "IT",
		Elevation: 1187,
		Latitude:  40.7677993774,
		Longitude: 16.9333000183,
		Timezone:  "Europe/Rome",
	},
	"LIBX": {
		ICAO:      "LIBX",
		Name:      "Martina Franca Air Force Base",
		City:      "Martina Franca",
		State:     "Apulia",
		Country:   "IT",
		Elevation: 1467,
		Latitude:  40.7000007629,
		Longitude: 17.3330001831,
		Timezone:  "Europe/Rome",
	},
	"LICA": {
		ICAO:      "LICA",
		IATA:      "SUF",
		Name:      "Lamezia Terme Airport",
		City:      "Lamezia Terme",
		State:     "Calabria",
		Country:   "IT",
		Elevation: 39,
		Latitude:  38.9053993225,
		Longitude: 16.2423000336,
		Timezone:  "Europe/Rome",
	},
	"LICB": {
		ICAO:      "LICB",
		IATA:      "CIY",
		Name:      "Comiso Airport Vincenzo Magliocco",
		City:      "Comiso",
		State:     "Sicily",
		Country:   "IT",
		Elevation: 623,
		Latitude:  36.9946010208,
		Longitude: 14.6071815491,
		Timezone:  "Europe/Rome",
	},
	"LICC": {
		ICAO:      "LICC",
		IATA:      "CTA",
		Name:      "Catania / Fontanarossa Airport",
		City:      "Catania",
		State:     "Sicily",
		Country:   "IT",
		Elevation: 39,
		Latitude:  37.4668006897,
		Longitude: 15.0663995743,
		Timezone:  "Europe/Rome",
	},
	"LICD": {
		ICAO:      "LICD",
		IATA:      "LMP",
		Name:      "Lampedusa Airport",
		City:      "Lampedusa",
		State:     "Sicily",
		Country:   "IT",
		Elevation: 70,
		Latitude:  35.4978981018,
		Longitude: 12.6181001663,
		Timezone:  "Europe/Rome",
	},
	"LICG": {
		ICAO:      "LICG",
		IATA:      "PNL",
		Name:      "Pantelleria Airport",
		City:      "Pantelleria",
		State:     "Sicily",
		Country:   "IT",
		Elevation: 628,
		Latitude:  36.8165016174,
		Longitude: 11.9688997269,
		Timezone:  "Europe/Rome",
	},
	"LICJ": {
		ICAO:      "LICJ",
		IATA:      "PMO",
		Name:      "Palermo / Punta Raisi Airport",
		City:      "Palermo",
		State:     "Sicily",
		Country:   "IT",
		Elevation: 65,
		Latitude:  38.1759986877,
		Longitude: 13.0909996033,
		Timezone:  "Europe/Rome",
	},
	"LICP": {
		ICAO:      "LICP",
		Name:      "Palermo / Bocca Di Falco Airport",
		City:      "Palermo",
		State:     "Sicily",
		Country:   "IT",
		Elevation: 345,
		Latitude:  38.1108016968,
		Longitude: 13.3133001328,
		Timezone:  "Europe/Rome",
	},
	"LICR": {
		ICAO:      "LICR",
		IATA:      "REG",
		Name:      "Reggio Calabria Airport",
		City:      "Reggio Calabria",
		State:     "Calabria",
		Country:   "IT",
		Elevation: 96,
		Latitude:  38.0712013245,
		Longitude: 15.651599884,
		Timezone:  "Europe/Rome",
	},
	"LICT": {
		ICAO:      "LICT",
		IATA:      "TPS",
		Name:      "Trapani / Birgi Airport",
		City:      "Trapani",
		State:     "Sicily",
		Country:   "IT",
		Elevation: 25,
		Latitude:  37.9113998413,
		Longitude: 12.4879999161,
		Timezone:  "Europe/Rome",
	},
	"LICZ": {
		ICAO:      "LICZ",
		IATA:      "NSY",
		Name:      "Sigonella Airport",
		State:     "Sicily",
		Country:   "IT",
		Elevation: 79,
		Latitude:  37.4016990662,
		Longitude: 14.9224004745,
		Timezone:  "Europe/Rome",
	},
	"LIDA": {
		ICAO:      "LIDA",
		Name:      "Asiago Airport",
		City:      "Vicenza",
		State:     "Veneto",
		Country:   "IT",
		Elevation: 3409,
		Latitude:  45.8869018555,
		Longitude: 11.5169000626,
		Timezone:  "Europe/Rome",
	},
	"LIDB": {
		ICAO:      "LIDB",
		IATA:      "BLX",
		Name:      "Belluno Airport",
		City:      "Belluno",
		State:     "Veneto",
		Country:   "IT",
		Elevation: 1240,
		Latitude:  46.1674995422,
		Longitude: 12.2479000092,
		Timezone:  "Europe/Rome",
	},
	"LIDC": {
		ICAO:      "LIDC",
		Name:      "Ca Negra Airport",
		City:      "Ca Negra",
		State:     "Veneto",
		Country:   "IT",
		Elevation: 1,
		Latitude:  45.0830001831,
		Longitude: 12.1499996185,
		Timezone:  "Europe/Rome",
	},
	"LIDE": {
		ICAO:      "LIDE",
		Name:      "Reggio Emilia Airport",
		City:      "Reggio Emilia",
		State:     "Emilia-Romagna",
		Country:   "IT",
		Elevation: 152,
		Latitude:  44.698299408,
		Longitude: 10.6627998352,
		Timezone:  "Europe/Rome",
	},
	"LIDF": {
		ICAO:      "LIDF",
		Name:      "Fano Airport",
		City:      "Fano",
		State:     "The-Marches",
		Country:   "IT",
		Elevation: 54,
		Latitude:  43.8260002136,
		Longitude: 13.0275001526,
		Timezone:  "Europe/Rome",
	},
	"LIDG": {
		ICAO:      "LIDG",
		Name:      "Lugo Di Romagna Airport",
		City:      "Lugo",
		State:     "Emilia-Romagna",
		Country:   "IT",
		Elevation: 62,
		Latitude:  44.3982009888,
		Longitude: 11.8548002243,
		Timezone:  "Europe/Rome",
	},
	"LIDH": {
		ICAO:      "LIDH",
		Name:      "Thiene Airport",
		City:      "Vicenza",
		State:     "Veneto",
		Country:   "IT",
		Elevation: 335,
		Latitude:  45.6757011414,
		Longitude: 11.4965000153,
		Timezone:  "Europe/Rome",
	},
	"LIDI": {
		ICAO:      "LIDI",
		IATA:      "CDF",
		Name:      "Cortina Airport",
		City:      "Cortina D'Ampezzo",
		State:     "Veneto",
		Country:   "IT",
		Elevation: 4213,
		Latitude:  46.5730018616,
		Longitude: 12.1180000305,
		Timezone:  "Europe/Rome",
	},
	"LIDL": {
		ICAO:      "LIDL",
		Name:      "Legnago Airport",
		City:      "Legnago",
		State:     "Veneto",
		Country:   "IT",
		Elevation: 30,
		Latitude:  45.1332015991,
		Longitude: 11.2922000885,
		Timezone:  "Europe/Rome",
	},
	"LIDM": {
		ICAO:      "LIDM",
		Name:      "Mantova Airport",
		City:      "Mantova",
		State:     "Lombardy",
		Country:   "IT",
		Elevation: 57,
		Latitude:  45.1358337402,
		Longitude: 10.7944440842,
		Timezone:  "Europe/Rome",
	},
	"LIDP": {
		ICAO:      "LIDP",
		Name:      "Pavullo Airport",
		City:      "Pavullo",
		State:     "Emilia-Romagna",
		Country:   "IT",
		Elevation: 2244,
		Latitude:  44.3223991394,
		Longitude: 10.831700325,
		Timezone:  "Europe/Rome",
	},
	"LIDR": {
		ICAO:      "LIDR",
		IATA:      "RAN",
		Name:      "Ravenna Airport",
		City:      "Ravenna",
		State:     "Emilia-Romagna",
		Country:   "IT",
		Elevation: 1,
		Latitude:  44.3642997742,
		Longitude: 12.2249002457,
		Timezone:  "Europe/Rome",
	},
	"LIDT": {
		ICAO:      "LIDT",
		IATA:      "ZIA",
		Name:      "Trento / Mattarello Airport",
		City:      "Trento",
		State:     "Trentino-Alto-Adige",
		Country:   "IT",
		Elevation: 610,
		Latitude:  46.0214004517,
		Longitude: 11.1241998672,
		Timezone:  "Europe/Rome",
	},
	"LIDU": {
		ICAO:      "LIDU",
		Name:      "Carpi / Budrione Airport",
		City:      "Modena",
		State:     "Emilia-Romagna",
		Country:   "IT",
		Elevation: 69,
		Latitude:  44.8352012634,
		Longitude: 10.8716001511,
		Timezone:  "Europe/Rome",
	},
	"LIDV": {
		ICAO:      "LIDV",
		Name:      "Prati Vecchi D'Aguscello Airport",
		City:      "Ferrara",
		State:     "Emilia-Romagna",
		Country:   "IT",
		Elevation: 20,
		Latitude:  44.7902984619,
		Longitude: 11.6691999435,
		Timezone:  "Europe/Rome",
	},
	"LIDW": {
		ICAO:      "LIDW",
		Name:      "Salgareda Carrer Airport",
		City:      "Salgareda",
		State:     "Veneto",
		Country:   "IT",
		Elevation: 7,
		Latitude:  45.6991996765,
		Longitude: 12.5431003571,
		Timezone:  "Europe/Rome",
	},
	"LIEA": {
		ICAO:      "LIEA",
		IATA:      "AHO",
		Name:      "Alghero / Fertilia Airport",
		City:      "Alghero",
		State:     "Sardinia",
		Country:   "IT",
		Elevation: 87,
		Latitude:  40.6320991516,
		Longitude: 8.290769577,
		Timezone:  "Europe/Rome",
	},
	"LIED": {
		ICAO:      "LIED",
		IATA:      "DCI",
		Name:      "Decimomannu Airport",
		City:      "Decimomannu",
		State:     "Sardinia",
		Country:   "IT",
		Elevation: 100,
		Latitude:  39.3541984558,
		Longitude: 8.9724798203,
		Timezone:  "Europe/Rome",
	},
	"LIEE": {
		ICAO:      "LIEE",
		IATA:      "CAG",
		Name:      "Cagliari / Elmas Airport",
		City:      "Cagliari",
		State:     "Sardinia",
		Country:   "IT",
		Elevation: 13,
		Latitude:  39.251499176,
		Longitude: 9.0542802811,
		Timezone:  "Europe/Rome",
	},
	"LIEO": {
		ICAO:      "LIEO",
		IATA:      "OLB",
		Name:      "Olbia / Costa Smeralda Airport",
		City:      "Olbia",
		State:     "Sardinia",
		Country:   "IT",
		Elevation: 37,
		Latitude:  40.8987007141,
		Longitude: 9.5176296234,
		Timezone:  "Europe/Rome",
	},
	"LIER": {
		ICAO:      "LIER",
		IATA:      "FNU",
		Name:      "Oristano / Fenosu Airport",
		City:      "Oristano",
		State:     "Sardinia",
		Country:   "IT",
		Elevation: 36,
		Latitude:  39.8942985535,
		Longitude: 8.6403999329,
		Timezone:  "Europe/Rome",
	},
	"LIET": {
		ICAO:      "LIET",
		IATA:      "TTB",
		Name:      "Tortoli' / Arbatax Airport",
		City:      "Arbatax",
		State:     "Sardinia",
		Country:   "IT",
		Elevation: 23,
		Latitude:  39.918800354,
		Longitude: 9.6829795837,
		Timezone:  "Europe/Rome",
	},
	"LILA": {
		ICAO:      "LILA",
		Name:      "Alessandria Airport",
		City:      "Alessandria",
		State:     "Piedmont",
		Country:   "IT",
		Elevation: 299,
		Latitude:  44.925201416,
		Longitude: 8.6251296997,
		Timezone:  "Europe/Rome",
	},
	"LILB": {
		ICAO:      "LILB",
		Name:      "Alzate Brianza Airport",
		City:      "Como",
		State:     "Lombardy",
		Country:   "IT",
		Elevation: 1260,
		Latitude:  45.770198822,
		Longitude: 9.1607198715,
		Timezone:  "Europe/Rome",
	},
	"LILC": {
		ICAO:      "LILC",
		Name:      "Calcinate Del Pesce Airport",
		City:      "Varese",
		State:     "Lombardy",
		Country:   "IT",
		Elevation: 797,
		Latitude:  45.8097000122,
		Longitude: 8.7680597305,
		Timezone:  "Europe/Rome",
	},
	"LILE": {
		ICAO:      "LILE",
		Name:      "Biella / Cerrione Airport",
		City:      "Biella",
		State:     "Piedmont",
		Country:   "IT",
		Elevation: 920,
		Latitude:  45.495300293,
		Longitude: 8.1027803421,
		Timezone:  "Europe/Rome",
	},
	"LILG": {
		ICAO:      "LILG",
		Name:      "Vergiate Airport",
		City:      "Milano",
		State:     "Lombardy",
		Country:   "IT",
		Elevation: 863,
		Latitude:  45.7131996155,
		Longitude: 8.6991195679,
		Timezone:  "Europe/Rome",
	},
	"LILH": {
		ICAO:      "LILH",
		Name:      "Voghera-Rivanazzano Airport",
		City:      "Voghera",
		State:     "Lombardy",
		Country:   "IT",
		Elevation: 423,
		Latitude:  44.9519996643,
		Longitude: 9.0165300369,
		Timezone:  "Europe/Rome",
	},
	"LILI": {
		ICAO:      "LILI",
		Name:      "Vercelli Airport",
		City:      "Vercelli",
		State:     "Piedmont",
		Country:   "IT",
		Elevation: 417,
		Latitude:  45.3111991882,
		Longitude: 8.4174203873,
		Timezone:  "Europe/Rome",
	},
	"LILM": {
		ICAO:      "LILM",
		Name:      "Casale Monferrato Airport",
		City:      "Casale Monferrato",
		State:     "Piedmont",
		Country:   "IT",
		Elevation: 377,
		Latitude:  45.1111984253,
		Longitude: 8.456029892,
		Timezone:  "Europe/Rome",
	},
	"LILN": {
		ICAO:      "LILN",
		IATA:      "QVA",
		Name:      "Varese / Venegono Airport",
		City:      "Varese",
		State:     "Lombardy",
		Country:   "IT",
		Elevation: 1050,
		Latitude:  45.7421989441,
		Longitude: 8.8882303238,
		Timezone:  "Europe/Rome",
	},
	"LILO": {
		ICAO:      "LILO",
		Name:      "Sondrio Caiolo Airport",
		City:      "Caiolo",
		State:     "Lombardy",
		Country:   "IT",
		Elevation: 899,
		Latitude:  46.1542015076,
		Longitude: 9.7980604172,
		Timezone:  "Europe/Rome",
	},
	"LILQ": {
		ICAO:      "LILQ",
		IATA:      "QMM",
		Name:      "Massa Cinquale Airport",
		City:      "Marina Di Massa",
		State:     "Tuscany",
		Country:   "IT",
		Elevation: 13,
		Latitude:  43.986000061,
		Longitude: 10.1428003311,
		Timezone:  "Europe/Rome",
	},
	"LILR": {
		ICAO:      "LILR",
		Name:      "Cremona / Migliaro Airport",
		City:      "Cremona",
		State:     "Lombardy",
		Country:   "IT",
		Elevation: 161,
		Latitude:  45.1674003601,
		Longitude: 10.0018997192,
		Timezone:  "Europe/Rome",
	},
	"LILV": {
		ICAO:      "LILV",
		Name:      "Valbrembo Airport",
		City:      "Bergamo",
		State:     "Lombardy",
		Country:   "IT",
		Elevation: 745,
		Latitude:  45.7207984924,
		Longitude: 9.5934696198,
		Timezone:  "Europe/Rome",
	},
	"LIMA": {
		ICAO:      "LIMA",
		Name:      "Torino / Aeritalia Airport",
		City:      "Torino",
		State:     "Piedmont",
		Country:   "IT",
		Elevation: 943,
		Latitude:  45.0863990784,
		Longitude: 7.6033701897,
		Timezone:  "Europe/Rome",
	},
	"LIMB": {
		ICAO:      "LIMB",
		Name:      "Milano / Bresso Airport",
		City:      "Milano",
		State:     "Lombardy",
		Country:   "IT",
		Elevation: 484,
		Latitude:  45.5421981812,
		Longitude: 9.20333004,
		Timezone:  "Europe/Rome",
	},
	"LIMC": {
		ICAO:      "LIMC",
		IATA:      "MXP",
		Name:      "Malpensa International Airport",
		City:      "Milan",
		State:     "Lombardy",
		Country:   "IT",
		Elevation: 768,
		Latitude:  45.6305999756,
		Longitude: 8.7281103134,
		Timezone:  "Europe/Rome",
	},
	"LIME": {
		ICAO:      "LIME",
		IATA:      "BGY",
		Name:      "Bergamo / Orio Al Serio Airport",
		City:      "Bergamo",
		State:     "Lombardy",
		Country:   "IT",
		Elevation: 782,
		Latitude:  45.6739006042,
		Longitude: 9.7041702271,
		Timezone:  "Europe/Rome",
	},
	"LIMF": {
		ICAO:      "LIMF",
		IATA:      "TRN",
		Name:      "Torino / Caselle International Airport",
		City:      "Torino",
		State:     "Piedmont",
		Country:   "IT",
		Elevation: 989,
		Latitude:  45.2008018494,
		Longitude: 7.6496300697,
		Timezone:  "Europe/Rome",
	},
	"LIMG": {
		ICAO:      "LIMG",
		IATA:      "ALL",
		Name:      "Villanova D'Albenga International Airport",
		City:      "Albenga",
		State:     "Liguria",
		Country:   "IT",
		Elevation: 148,
		Latitude:  44.0505981445,
		Longitude: 8.1274299622,
		Timezone:  "Europe/Rome",
	},
	"LIMJ": {
		ICAO:      "LIMJ",
		IATA:      "GOA",
		Name:      "Genova / Sestri Cristoforo Colombo Airport",
		City:      "Genova",
		State:     "Liguria",
		Country:   "IT",
		Elevation: 13,
		Latitude:  44.4132995605,
		Longitude: 8.8374996185,
		Timezone:  "Europe/Rome",
	},
	"LIML": {
		ICAO:      "LIML",
		IATA:      "LIN",
		Name:      "Linate Airport",
		City:      "Milan",
		State:     "Lombardy",
		Country:   "IT",
		Elevation: 353,
		Latitude:  45.445098877,
		Longitude: 9.2767400742,
		Timezone:  "Europe/Rome",
	},
	"LIMN": {
		ICAO:      "LIMN",
		Name:      "Cameri Airport",
		City:      "Cameri",
		State:     "Piedmont",
		Country:   "IT",
		Elevation: 586,
		Latitude:  45.5295982361,
		Longitude: 8.6692199707,
		Timezone:  "Europe/Rome",
	},
	"LIMP": {
		ICAO:      "LIMP",
		IATA:      "PMF",
		Name:      "Parma Airport",
		City:      "Parma",
		State:     "Emilia-Romagna",
		Country:   "IT",
		Elevation: 161,
		Latitude:  44.8245010376,
		Longitude: 10.2964000702,
		Timezone:  "Europe/Rome",
	},
	"LIMR": {
		ICAO:      "LIMR",
		Name:      "Novi Ligure Glider Airport",
		City:      "Novi Ligure",
		State:     "Piedmont",
		Country:   "IT",
		Elevation: 607,
		Latitude:  44.7799987793,
		Longitude: 8.7863903046,
		Timezone:  "Europe/Rome",
	},
	"LIMS": {
		ICAO:      "LIMS",
		IATA:      "QPZ",
		Name:      "Piacenza Airport",
		City:      "Piacenza",
		State:     "Emilia-Romagna",
		Country:   "IT",
		Elevation: 456,
		Latitude:  44.9131011963,
		Longitude: 9.7233295441,
		Timezone:  "Europe/Rome",
	},
	"LIMW": {
		ICAO:      "LIMW",
		IATA:      "AOT",
		Name:      "Aosta Airport",
		City:      "Aosta",
		State:     "Aosta-Valley",
		Country:   "IT",
		Elevation: 1791,
		Latitude:  45.7384986877,
		Longitude: 7.3687200546,
		Timezone:  "Europe/Rome",
	},
	"LIMZ": {
		ICAO:      "LIMZ",
		IATA:      "CUF",
		Name:      "Cuneo / Levaldigi Airport",
		City:      "Cuneo",
		State:     "Piedmont",
		Country:   "IT",
		Elevation: 1267,
		Latitude:  44.547000885,
		Longitude: 7.6232199669,
		Timezone:  "Europe/Rome",
	},
	"LINL": {
		ICAO:      "LINL",
		Name:      "Lecce / Lepore Airport",
		City:      "Lecce",
		State:     "Apulia",
		Country:   "IT",
		Elevation: 59,
		Latitude:  40.3576011658,
		Longitude: 18.2938995361,
		Timezone:  "Europe/Rome",
	},
	"LIPA": {
		ICAO:      "LIPA",
		IATA:      "AVB",
		Name:      "Aviano Air Base",
		City:      "Aviano",
		State:     "Friuli-Venezia-Giulia",
		Country:   "IT",
		Elevation: 410,
		Latitude:  46.0318984985,
		Longitude: 12.5965003967,
		Timezone:  "Europe/Rome",
	},
	"LIPB": {
		ICAO:      "LIPB",
		IATA:      "BZO",
		Name:      "Bolzano Airport",
		City:      "Bolzano",
		State:     "Trentino-Alto-Adige",
		Country:   "IT",
		Elevation: 789,
		Latitude:  46.4602012634,
		Longitude: 11.3263998032,
		Timezone:  "Europe/Rome",
	},
	"LIPC": {
		ICAO:      "LIPC",
		Name:      "Cervia Air Force Base",
		City:      "Cervia",
		State:     "Emilia-Romagna",
		Country:   "IT",
		Elevation: 18,
		Latitude:  44.2242012024,
		Longitude: 12.3072004318,
		Timezone:  "Europe/Rome",
	},
	"LIPD": {
		ICAO:      "LIPD",
		IATA:      "UDN",
		Name:      "Udine / Campoformido Air Base",
		City:      "Udine",
		State:     "Friuli-Venezia-Giulia",
		Country:   "IT",
		Elevation: 305,
		Latitude:  46.0321998596,
		Longitude: 13.1868000031,
		Timezone:  "Europe/Rome",
	},
	"LIPE": {
		ICAO:      "LIPE",
		IATA:      "BLQ",
		Name:      "Bologna / Borgo Panigale Airport",
		City:      "Bologna",
		State:     "Emilia-Romagna",
		Country:   "IT",
		Elevation: 123,
		Latitude:  44.5354003906,
		Longitude: 11.2887001038,
		Timezone:  "Europe/Rome",
	},
	"LIPF": {
		ICAO:      "LIPF",
		Name:      "Ferrara Airport",
		City:      "Ferrara",
		State:     "Emilia-Romagna",
		Country:   "IT",
		Elevation: 20,
		Latitude:  44.8160018921,
		Longitude: 11.6134004593,
		Timezone:  "Europe/Rome",
	},
	"LIPG": {
		ICAO:      "LIPG",
		Name:      "Gorizia Airport",
		City:      "Gorizia",
		State:     "Friuli-Venezia-Giulia",
		Country:   "IT",
		Elevation: 207,
		Latitude:  45.9068984985,
		Longitude: 13.5993003845,
		Timezone:  "Europe/Ljubljana",
	},
	"LIPH": {
		ICAO:      "LIPH",
		IATA:      "TSF",
		Name:      "Treviso / Sant'Angelo Airport",
		City:      "Treviso",
		State:     "Veneto",
		Country:   "IT",
		Elevation: 59,
		Latitude:  45.648399353,
		Longitude: 12.1943998337,
		Timezone:  "Europe/Rome",
	},
	"LIPI": {
		ICAO:      "LIPI",
		Name:      "Rivolto Air Base",
		City:      "Codroipo",
		State:     "Friuli-Venezia-Giulia",
		Country:   "IT",
		Elevation: 179,
		Latitude:  45.9786987305,
		Longitude: 13.0493001938,
		Timezone:  "Europe/Rome",
	},
	"LIPK": {
		ICAO:      "LIPK",
		IATA:      "FRL",
		Name:      "Forli Airport",
		City:      "Forli",
		State:     "Emilia-Romagna",
		Country:   "IT",
		Elevation: 97,
		Latitude:  44.1948013306,
		Longitude: 12.0700998306,
		Timezone:  "Europe/Rome",
	},
	"LIPL": {
		ICAO:      "LIPL",
		Name:      "Ghedi Airport",
		City:      "Ghedi",
		State:     "Lombardy",
		Country:   "IT",
		Elevation: 333,
		Latitude:  45.4322013855,
		Longitude: 10.2677001953,
		Timezone:  "Europe/Rome",
	},
	"LIPM": {
		ICAO:      "LIPM",
		Name:      "Modena / Marzaglia Airport",
		City:      "Modena",
		State:     "Emilia-Romagna",
		Country:   "IT",
		Elevation: 184,
		Latitude:  44.6348991394,
		Longitude: 10.8101997375,
		Timezone:  "Europe/Rome",
	},
	"LIPN": {
		ICAO:      "LIPN",
		Name:      "Verona / Boscomantico Airport",
		City:      "Verona",
		State:     "Veneto",
		Country:   "IT",
		Elevation: 286,
		Latitude:  45.4720001221,
		Longitude: 10.9279003143,
		Timezone:  "Europe/Rome",
	},
	"LIPO": {
		ICAO:      "LIPO",
		IATA:      "VBS",
		Name:      "Brescia / Montichiari Airport",
		City:      "Brescia",
		State:     "Lombardy",
		Country:   "IT",
		Elevation: 355,
		Latitude:  45.4289016724,
		Longitude: 10.3305997849,
		Timezone:  "Europe/Rome",
	},
	"LIPQ": {
		ICAO:      "LIPQ",
		IATA:      "TRS",
		Name:      "Trieste / Ronchi Dei Legionari",
		City:      "Trieste",
		State:     "Friuli-Venezia-Giulia",
		Country:   "IT",
		Elevation: 39,
		Latitude:  45.8274993896,
		Longitude: 13.4722003937,
		Timezone:  "Europe/Rome",
	},
	"LIPR": {
		ICAO:      "LIPR",
		IATA:      "RMI",
		Name:      "Rimini / Miramare - Federico Fellini International Airport",
		City:      "Rimini",
		State:     "Emilia-Romagna",
		Country:   "IT",
		Elevation: 40,
		Latitude:  44.0203018188,
		Longitude: 12.611700058,
		Timezone:  "Europe/Rome",
	},
	"LIPS": {
		ICAO:      "LIPS",
		Name:      "Istrana Airport",
		City:      "Istrana",
		State:     "Veneto",
		Country:   "IT",
		Elevation: 137,
		Latitude:  45.6848983765,
		Longitude: 12.0829000473,
		Timezone:  "Europe/Rome",
	},
	"LIPT": {
		ICAO:      "LIPT",
		IATA:      "VIC",
		Name:      "Vicenza Airport",
		City:      "Vicenza",
		State:     "Veneto",
		Country:   "IT",
		Elevation: 128,
		Latitude:  45.5733985901,
		Longitude: 11.5295000076,
		Timezone:  "Europe/Rome",
	},
	"LIPU": {
		ICAO:      "LIPU",
		IATA:      "QPA",
		Name:      "Padova Airport",
		City:      "Padova",
		State:     "Veneto",
		Country:   "IT",
		Elevation: 44,
		Latitude:  45.3958015442,
		Longitude: 11.8479003906,
		Timezone:  "Europe/Rome",
	},
	"LIPV": {
		ICAO:      "LIPV",
		Name:      "Venezia / Lido - San Nicolo Airport",
		City:      "Venezia",
		State:     "Veneto",
		Country:   "IT",
		Elevation: 13,
		Latitude:  45.4282989502,
		Longitude: 12.3880996704,
		Timezone:  "Europe/Rome",
	},
	"LIPX": {
		ICAO:      "LIPX",
		IATA:      "VRN",
		Name:      "Verona / Villafranca Airport",
		City:      "Verona",
		State:     "Veneto",
		Country:   "IT",
		Elevation: 239,
		Latitude:  45.3956985474,
		Longitude: 10.8885002136,
		Timezone:  "Europe/Rome",
	},
	"LIPY": {
		ICAO:      "LIPY",
		IATA:      "AOI",
		Name:      "Ancona / Falconara Airport",
		City:      "Ancona",
		State:     "The-Marches",
		Country:   "IT",
		Elevation: 49,
		Latitude:  43.6162986755,
		Longitude: 13.3622999191,
		Timezone:  "Europe/Rome",
	},
	"LIPZ": {
		ICAO:      "LIPZ",
		IATA:      "VCE",
		Name:      "Venezia / Tessera -  Marco Polo Airport",
		City:      "Venezia",
		State:     "Veneto",
		Country:   "IT",
		Elevation: 7,
		Latitude:  45.5052986145,
		Longitude: 12.3519001007,
		Timezone:  "Europe/Rome",
	},
	"LIQB": {
		ICAO:      "LIQB",
		IATA:      "QZO",
		Name:      "Arezzo Airport",
		City:      "Arezzo",
		State:     "Tuscany",
		Country:   "IT",
		Elevation: 814,
		Latitude:  43.4558982849,
		Longitude: 11.8470001221,
		Timezone:  "Europe/Rome",
	},
	"LIQL": {
		ICAO:      "LIQL",
		IATA:      "LCV",
		Name:      "Lucca / Tassignano Airport",
		City:      "Lucca",
		State:     "Tuscany",
		Country:   "IT",
		Elevation: 39,
		Latitude:  43.8282012939,
		Longitude: 10.5781002045,
		Timezone:  "Europe/Rome",
	},
	"LIQN": {
		ICAO:      "LIQN",
		IATA:      "QRT",
		Name:      "Rieti Airport",
		City:      "Rieti",
		State:     "Latium",
		Country:   "IT",
		Elevation: 1278,
		Latitude:  42.4272003174,
		Longitude: 12.8516998291,
		Timezone:  "Europe/Rome",
	},
	"LIQS": {
		ICAO:      "LIQS",
		IATA:      "SAY",
		Name:      "Siena / Ampugnano Airport",
		City:      "Siena",
		State:     "Tuscany",
		Country:   "IT",
		Elevation: 634,
		Latitude:  43.2563018799,
		Longitude: 11.2550001144,
		Timezone:  "Europe/Rome",
	},
	"LIQW": {
		ICAO:      "LIQW",
		Name:      "Sarzana / Luni Military Airport",
		City:      "Sarzana",
		State:     "Liguria",
		Country:   "IT",
		Elevation: 45,
		Latitude:  44.0880012512,
		Longitude: 9.987950325,
		Timezone:  "Europe/Rome",
	},
	"LIRA": {
		ICAO:      "LIRA",
		IATA:      "CIA",
		Name:      "Ciampino Airport",
		City:      "Roma",
		State:     "Latium",
		Country:   "IT",
		Elevation: 427,
		Latitude:  41.7994003296,
		Longitude: 12.5949001312,
		Timezone:  "Europe/Rome",
	},
	"LIRE": {
		ICAO:      "LIRE",
		Name:      "Pratica Di Mare Airport",
		City:      "Pomezia",
		State:     "Latium",
		Country:   "IT",
		Elevation: 41,
		Latitude:  41.654499054,
		Longitude: 12.4451999664,
		Timezone:  "Europe/Rome",
	},
	"LIRF": {
		ICAO:      "LIRF",
		IATA:      "FCO",
		Name:      "Leonardo Da Vinci (Fiumicino) International Airport",
		City:      "Rome",
		State:     "Latium",
		Country:   "IT",
		Elevation: 15,
		Latitude:  41.8045005798,
		Longitude: 12.2508001328,
		Timezone:  "Europe/Rome",
	},
	"LIRG": {
		ICAO:      "LIRG",
		Name:      "Guidonia Airport",
		City:      "Guidonia",
		State:     "Latium",
		Country:   "IT",
		Elevation: 289,
		Latitude:  41.9902992249,
		Longitude: 12.7407999039,
		Timezone:  "Europe/Rome",
	},
	"LIRH": {
		ICAO:      "LIRH",
		IATA:      "QFR",
		Name:      "Frosinone Military Airport",
		City:      "Frosinone",
		State:     "Latium",
		Country:   "IT",
		Latitude:  41.6464004517,
		Longitude: 13.2997999191,
		Timezone:  "Europe/Rome",
	},
	"LIRI": {
		ICAO:      "LIRI",
		IATA:      "QSR",
		Name:      "Salerno / Pontecagnano Airport",
		City:      "Salerno",
		State:     "Campania",
		Country:   "IT",
		Elevation: 119,
		Latitude:  40.6203994751,
		Longitude: 14.9112997055,
		Timezone:  "Europe/Rome",
	},
	"LIRJ": {
		ICAO:      "LIRJ",
		IATA:      "EBA",
		Name:      "Marina Di Campo Airport",
		City:      "Marina  Di Campo",
		State:     "Tuscany",
		Country:   "IT",
		Elevation: 31,
		Latitude:  42.7602996826,
		Longitude: 10.23939991,
		Timezone:  "Europe/Rome",
	},
	"LIRL": {
		ICAO:      "LIRL",
		IATA:      "QLT",
		Name:      "Latina Airport",
		City:      "Latina",
		State:     "Latium",
		Country:   "IT",
		Elevation: 94,
		Latitude:  41.5424003601,
		Longitude: 12.9090003967,
		Timezone:  "Europe/Rome",
	},
	"LIRM": {
		ICAO:      "LIRM",
		Name:      "Grazzanise Airport",
		City:      "Caserta",
		State:     "Campania",
		Country:   "IT",
		Elevation: 29,
		Latitude:  41.060798645,
		Longitude: 14.0818996429,
		Timezone:  "Europe/Rome",
	},
	"LIRN": {
		ICAO:      "LIRN",
		IATA:      "NAP",
		Name:      "Napoli / Capodichino International Airport",
		City:      "Napoli",
		State:     "Campania",
		Country:   "IT",
		Elevation: 294,
		Latitude:  40.8860015869,
		Longitude: 14.2908000946,
		Timezone:  "Europe/Rome",
	},
	"LIRP": {
		ICAO:      "LIRP",
		IATA:      "PSA",
		Name:      "Pisa / San Giusto - Galileo Galilei International Airport",
		City:      "Pisa",
		State:     "Tuscany",
		Country:   "IT",
		Elevation: 6,
		Latitude:  43.6838989258,
		Longitude: 10.3927001953,
		Timezone:  "Europe/Rome",
	},
	"LIRQ": {
		ICAO:      "LIRQ",
		IATA:      "FLR",
		Name:      "Firenze / Peretola Airport",
		City:      "Firenze",
		State:     "Tuscany",
		Country:   "IT",
		Elevation: 142,
		Latitude:  43.8100013733,
		Longitude: 11.2051000595,
		Timezone:  "Europe/Rome",
	},
	"LIRS": {
		ICAO:      "LIRS",
		IATA:      "GRS",
		Name:      "Grosseto Airport",
		City:      "Grosetto",
		State:     "Tuscany",
		Country:   "IT",
		Elevation: 17,
		Latitude:  42.7597007751,
		Longitude: 11.0719003677,
		Timezone:  "Europe/Rome",
	},
	"LIRU": {
		ICAO:      "LIRU",
		Name:      "Urbe Airport",
		City:      "Roma",
		State:     "Latium",
		Country:   "IT",
		Elevation: 55,
		Latitude:  41.9519004822,
		Longitude: 12.4989004135,
		Timezone:  "Europe/Rome",
	},
	"LIRV": {
		ICAO:      "LIRV",
		Name:      "Viterbo Airport",
		City:      "Viterbo",
		State:     "Latium",
		Country:   "IT",
		Elevation: 992,
		Latitude:  42.4301986694,
		Longitude: 12.0642004013,
		Timezone:  "Europe/Rome",
	},
	"LIRZ": {
		ICAO:      "LIRZ",
		IATA:      "PEG",
		Name:      "Perugia / San Egidio Airport",
		City:      "Perugia",
		State:     "Umbria",
		Country:   "IT",
		Elevation: 697,
		Latitude:  43.0959014893,
		Longitude: 12.5131998062,
		Timezone:  "Europe/Rome",
	},
	"LJAJ": {
		ICAO:      "LJAJ",
		Name:      "Ajdovscina Airport",
		City:      "Ajdovscina",
		State:     "Ajdovscina",
		Country:   "SI",
		Elevation: 381,
		Latitude:  45.8891983032,
		Longitude: 13.8868999481,
		Timezone:  "Europe/Ljubljana",
	},
	"LJBL": {
		ICAO:      "LJBL",
		Name:      "Lesce Bled Glider Airport",
		City:      "Bled",
		State:     "Radovljica",
		Country:   "SI",
		Elevation: 1654,
		Latitude:  46.3572006226,
		Longitude: 14.1738996506,
		Timezone:  "Europe/Ljubljana",
	},
	"LJBO": {
		ICAO:      "LJBO",
		Name:      "Bovec Letalisce Airport",
		City:      "Bovec",
		State:     "Bovec",
		Country:   "SI",
		Elevation: 1417,
		Latitude:  46.3286018372,
		Longitude: 13.5486001968,
		Timezone:  "Europe/Ljubljana",
	},
	"LJCE": {
		ICAO:      "LJCE",
		Name:      "Cerklje Airport",
		State:     "Brezice",
		Country:   "SI",
		Elevation: 510,
		Latitude:  45.9000015259,
		Longitude: 15.5302000046,
		Timezone:  "Europe/Ljubljana",
	},
	"LJCL": {
		ICAO:      "LJCL",
		Name:      "Celje Glider Airport",
		City:      "Celje",
		State:     "Celje",
		Country:   "SI",
		Elevation: 801,
		Latitude:  46.2456016541,
		Longitude: 15.2230997086,
		Timezone:  "Europe/Ljubljana",
	},
	"LJDI": {
		ICAO:      "LJDI",
		Name:      "Divaca Airport",
		City:      "Divaca",
		State:     "Divaca",
		Country:   "SI",
		Elevation: 1420,
		Latitude:  45.6819000244,
		Longitude: 14.0027999878,
		Timezone:  "Europe/Ljubljana",
	},
	"LJLJ": {
		ICAO:      "LJLJ",
		IATA:      "LJU",
		Name:      "Ljubljana Joze Pucnik Airport",
		City:      "Ljubljana",
		State:     "Cerklje-na-Gorenjskem",
		Country:   "SI",
		Elevation: 1273,
		Latitude:  46.2237014771,
		Longitude: 14.4575996399,
		Timezone:  "Europe/Ljubljana",
	},
	"LJMB": {
		ICAO:      "LJMB",
		IATA:      "MBX",
		Name:      "Maribor Airport",
		State:     "Hoce-Slivnica",
		Country:   "SI",
		Elevation: 876,
		Latitude:  46.4799003601,
		Longitude: 15.6861000061,
		Timezone:  "Europe/Ljubljana",
	},
	"LJMS": {
		ICAO:      "LJMS",
		Name:      "Murska Sobota Airport",
		City:      "Murska Sobota",
		State:     "Murska-Sobota",
		Country:   "SI",
		Elevation: 600,
		Latitude:  46.6293983459,
		Longitude: 16.1749992371,
		Timezone:  "Europe/Ljubljana",
	},
	"LJNM": {
		ICAO:      "LJNM",
		Name:      "Novo Mesto Airport",
		City:      "Novo mesto",
		State:     "Novo-Mesto",
		Country:   "SI",
		Elevation: 572,
		Latitude:  45.8102989197,
		Longitude: 15.1127004623,
		Timezone:  "Europe/Ljubljana",
	},
	"LJPO": {
		ICAO:      "LJPO",
		Name:      "Postojna Airfield",
		City:      "Postojna",
		State:     "Postojna",
		Country:   "SI",
		Latitude:  45.7522010803,
		Longitude: 14.1947002411,
		Timezone:  "Europe/Ljubljana",
	},
	"LJPT": {
		ICAO:      "LJPT",
		Name:      "Ptuj Airport",
		State:     "Gorisnica",
		Country:   "SI",
		Elevation: 689,
		Latitude:  46.4253005981,
		Longitude: 15.9885997772,
		Timezone:  "Europe/Ljubljana",
	},
	"LJPZ": {
		ICAO:      "LJPZ",
		IATA:      "POW",
		Name:      "Portoroz Airport",
		City:      "Portoroz",
		State:     "Piran-Pirano",
		Country:   "SI",
		Elevation: 7,
		Latitude:  45.473400116,
		Longitude: 13.6149997711,
		Timezone:  "Europe/Ljubljana",
	},
	"LJSG": {
		ICAO:      "LJSG",
		Name:      "Slovenj Gradec Airport",
		City:      "Slovenc Gradec",
		State:     "Slovenj-Gradec",
		Country:   "SI",
		Elevation: 1645,
		Latitude:  46.4720001221,
		Longitude: 15.1169996262,
		Timezone:  "Europe/Ljubljana",
	},
	"LJVE": {
		ICAO:      "LJVE",
		Name:      "Velenje Airport",
		City:      "Velenje",
		State:     "Sostanj",
		Country:   "SI",
		Elevation: 1283,
		Latitude:  46.3974990845,
		Longitude: 15.04529953,
		Timezone:  "Europe/Ljubljana",
	},
	"LKBA": {
		ICAO:      "LKBA",
		Name:      "Breclav Airport",
		City:      "Breclav",
		State:     "South-Moravian",
		Country:   "CZ",
		Elevation: 531,
		Latitude:  48.7900009155,
		Longitude: 16.8924999237,
		Timezone:  "Europe/Prague",
	},
	"LKBE": {
		ICAO:      "LKBE",
		Name:      "Benesov Airport",
		City:      "Benesov",
		State:     "Central-Bohemia",
		Country:   "CZ",
		Elevation: 1319,
		Latitude:  49.7407989502,
		Longitude: 14.6447000504,
		Timezone:  "Europe/Prague",
	},
	"LKBO": {
		ICAO:      "LKBO",
		Name:      "Bohunovice Airport",
		City:      "Bohunovice",
		State:     "Olomoucky",
		Country:   "CZ",
		Elevation: 771,
		Latitude:  49.6706008911,
		Longitude: 17.2950000763,
		Timezone:  "Europe/Prague",
	},
	"LKBR": {
		ICAO:      "LKBR",
		Name:      "Broumov Airport",
		City:      "Broumov",
		State:     "Kralovehradecky",
		Country:   "CZ",
		Elevation: 1342,
		Latitude:  50.5619010925,
		Longitude: 16.3428001404,
		Timezone:  "Europe/Prague",
	},
	"LKBU": {
		ICAO:      "LKBU",
		Name:      "Bubovice Airport",
		City:      "Bubovice",
		State:     "Central-Bohemia",
		Country:   "CZ",
		Elevation: 1401,
		Latitude:  49.9743995667,
		Longitude: 14.1780996323,
		Timezone:  "Europe/Prague",
	},
	"LKCB": {
		ICAO:      "LKCB",
		Name:      "Cheb Airport",
		City:      "Cheb",
		State:     "Karlovarsky",
		Country:   "CZ",
		Elevation: 1585,
		Latitude:  50.0661010742,
		Longitude: 12.4117002487,
		Timezone:  "Europe/Prague",
	},
	"LKCE": {
		ICAO:      "LKCE",
		Name:      "Ceska Lipa Airport",
		City:      "Ceska Lipa",
		State:     "Liberecky",
		Country:   "CZ",
		Elevation: 932,
		Latitude:  50.709400177,
		Longitude: 14.5666999817,
		Timezone:  "Europe/Prague",
	},
	"LKCH": {
		ICAO:      "LKCH",
		Name:      "Chomutov Airport",
		City:      "Chotumov",
		State:     "Ustecky",
		Country:   "CZ",
		Elevation: 1132,
		Latitude:  50.4688987732,
		Longitude: 13.4680995941,
		Timezone:  "Europe/Prague",
	},
	"LKCL": {
		ICAO:      "LKCL",
		Name:      "Cernovice Air Base",
		City:      "Cernovice",
		State:     "South-Moravian",
		Country:   "CZ",
		Elevation: 778,
		Latitude:  49.1775016785,
		Longitude: 16.6608009338,
		Timezone:  "Europe/Prague",
	},
	"LKCM": {
		ICAO:      "LKCM",
		Name:      "Medlanky Airport",
		City:      "Medlanky",
		State:     "South-Moravian",
		Country:   "CZ",
		Elevation: 853,
		Latitude:  49.2366981506,
		Longitude: 16.5550003052,
		Timezone:  "Europe/Prague",
	},
	"LKCO": {
		ICAO:      "LKCO",
		Name:      "Chocen Airport",
		City:      "Chocen",
		State:     "Pardubicky",
		Country:   "CZ",
		Elevation: 873,
		Latitude:  49.9772796631,
		Longitude: 16.1849212646,
		Timezone:  "Europe/Prague",
	},
	"LKCR": {
		ICAO:      "LKCR",
		Name:      "Chrudim Airport",
		City:      "Chrudim",
		State:     "Pardubicky",
		Country:   "CZ",
		Elevation: 981,
		Latitude:  49.9364013672,
		Longitude: 15.7805995941,
		Timezone:  "Europe/Prague",
	},
	"LKCS": {
		ICAO:      "LKCS",
		Name:      "Ceske Budejovice Airport",
		City:      "Ceske Budejovice",
		State:     "Jihocesky",
		Country:   "CZ",
		Elevation: 1417,
		Latitude:  48.9463996887,
		Longitude: 14.4274997711,
		Timezone:  "Europe/Prague",
	},
	"LKCT": {
		ICAO:      "LKCT",
		Name:      "Chotebor Airport",
		City:      "Chotebor",
		State:     "Vysocina",
		Country:   "CZ",
		Elevation: 1949,
		Latitude:  49.685798645,
		Longitude: 15.6760997772,
		Timezone:  "Europe/Prague",
	},
	"LKCV": {
		ICAO:      "LKCV",
		Name:      "Caslav Air Base",
		City:      "Caslav",
		State:     "Central-Bohemia",
		Country:   "CZ",
		Elevation: 794,
		Latitude:  49.9397010803,
		Longitude: 15.3817996979,
		Timezone:  "Europe/Prague",
	},
	"LKDK": {
		ICAO:      "LKDK",
		Name:      "Dvur Kralove Nad Labem Airport",
		City:      "Dvur Kralove Nad Labem",
		State:     "Kralovehradecky",
		Country:   "CZ",
		Elevation: 932,
		Latitude:  50.4141998291,
		Longitude: 15.8368997574,
		Timezone:  "Europe/Prague",
	},
	"LKER": {
		ICAO:      "LKER",
		Name:      "Erpuzice Airport",
		City:      "Erpuzice",
		State:     "Plzensky",
		Country:   "CZ",
		Elevation: 1572,
		Latitude:  49.8027992249,
		Longitude: 13.0381002426,
		Timezone:  "Europe/Prague",
	},
	"LKFR": {
		ICAO:      "LKFR",
		Name:      "Frydlant Airport",
		City:      "Frydlant",
		State:     "Moravskoslezsky",
		Country:   "CZ",
		Elevation: 1440,
		Latitude:  49.5894012451,
		Longitude: 18.3791999817,
		Timezone:  "Europe/Prague",
	},
	"LKHB": {
		ICAO:      "LKHB",
		Name:      "Havlickuv Brod Airport",
		City:      "Havlickuv Brod",
		State:     "Vysocina",
		Country:   "CZ",
		Elevation: 1519,
		Latitude:  49.5971984863,
		Longitude: 15.549200058,
		Timezone:  "Europe/Prague",
	},
	"LKHC": {
		ICAO:      "LKHC",
		Name:      "Horice Airport",
		City:      "Horice",
		State:     "Kralovehradecky",
		Country:   "CZ",
		Elevation: 922,
		Latitude:  50.3572006226,
		Longitude: 15.5767002106,
		Timezone:  "Europe/Prague",
	},
	"LKHD": {
		ICAO:      "LKHD",
		Name:      "Hodkovice Nad Mohelkou Airport",
		City:      "Hodkovice Nad Mohelkou",
		State:     "Liberecky",
		Country:   "CZ",
		Elevation: 1480,
		Latitude:  50.6571998596,
		Longitude: 15.0777997971,
		Timezone:  "Europe/Prague",
	},
	"LKHK": {
		ICAO:      "LKHK",
		Name:      "Hradec Kralove Airport",
		City:      "Hradec Kralove",
		State:     "Kralovehradecky",
		Country:   "CZ",
		Elevation: 791,
		Latitude:  50.253200531,
		Longitude: 15.845199585,
		Timezone:  "Europe/Prague",
	},
	"LKHN": {
		ICAO:      "LKHN",
		Name:      "Hranice Airport",
		City:      "Hranice",
		State:     "Olomoucky",
		Country:   "CZ",
		Elevation: 801,
		Latitude:  49.5461006165,
		Longitude: 17.7043991089,
		Timezone:  "Europe/Prague",
	},
	"LKHO": {
		ICAO:      "LKHO",
		IATA:      "GTW",
		Name:      "Holesov Airport",
		City:      "Holesov",
		State:     "Zlin",
		Country:   "CZ",
		Elevation: 761,
		Latitude:  49.3143997192,
		Longitude: 17.5688991547,
		Timezone:  "Europe/Prague",
	},
	"LKHR": {
		ICAO:      "LKHR",
		Name:      "Hradcany Airport",
		City:      "Ralsko",
		State:     "Liberecky",
		Country:   "CZ",
		Elevation: 912,
		Latitude:  50.6205558777,
		Longitude: 14.7380561829,
		Timezone:  "Europe/Prague",
	},
	"LKHS": {
		ICAO:      "LKHS",
		Name:      "Hosin Airport",
		City:      "Hosin",
		State:     "Jihocesky",
		Country:   "CZ",
		Elevation: 1621,
		Latitude:  49.0400009155,
		Longitude: 14.4949998856,
		Timezone:  "Europe/Prague",
	},
	"LKHV": {
		ICAO:      "LKHV",
		Name:      "Horovice Airport",
		City:      "Horovice",
		State:     "Central-Bohemia",
		Country:   "CZ",
		Elevation: 1214,
		Latitude:  49.8480987549,
		Longitude: 13.8935003281,
		Timezone:  "Europe/Prague",
	},
	"LKJA": {
		ICAO:      "LKJA",
		Name:      "Jaromer Airport",
		City:      "Jaromer",
		State:     "Kralovehradecky",
		Country:   "CZ",
		Elevation: 889,
		Latitude:  50.331401825,
		Longitude: 15.9539003372,
		Timezone:  "Europe/Prague",
	},
	"LKJC": {
		ICAO:      "LKJC",
		Name:      "Jicin Airport",
		City:      "Jicin",
		State:     "Kralovehradecky",
		Country:   "CZ",
		Elevation: 863,
		Latitude:  50.4300003052,
		Longitude: 15.3331003189,
		Timezone:  "Europe/Prague",
	},
	"LKJH": {
		ICAO:      "LKJH",
		Name:      "Jindrichuv Hradec Airport",
		City:      "Jindrichuv Hradec",
		State:     "Jihocesky",
		Country:   "CZ",
		Elevation: 1667,
		Latitude:  49.1502990723,
		Longitude: 14.9710998535,
		Timezone:  "Europe/Prague",
	},
	"LKJI": {
		ICAO:      "LKJI",
		Name:      "Jihlava Airport",
		City:      "Jihlava",
		State:     "Vysocina",
		Country:   "CZ",
		Elevation: 1821,
		Latitude:  49.4193992615,
		Longitude: 15.6352996826,
		Timezone:  "Europe/Prague",
	},
	"LKKA": {
		ICAO:      "LKKA",
		Name:      "Krizanov Airport",
		City:      "Krizanov",
		State:     "Vysocina",
		Country:   "CZ",
		Elevation: 1831,
		Latitude:  49.3683013916,
		Longitude: 16.1161003113,
		Timezone:  "Europe/Prague",
	},
	"LKKB": {
		ICAO:      "LKKB",
		Name:      "Kbely Air Base",
		City:      "Prague",
		State:     "Praha",
		Country:   "CZ",
		Elevation: 939,
		Latitude:  50.1213989258,
		Longitude: 14.5436000824,
		Timezone:  "Europe/Prague",
	},
	"LKKC": {
		ICAO:      "LKKC",
		Name:      "Krizenec Plana Airport",
		City:      "Plana",
		State:     "Plzensky",
		Country:   "CZ",
		Elevation: 2080,
		Latitude:  49.8706016541,
		Longitude: 12.7725000381,
		Timezone:  "Europe/Prague",
	},
	"LKKL": {
		ICAO:      "LKKL",
		Name:      "Kladno Airport",
		City:      "Kladno",
		State:     "Central-Bohemia",
		Country:   "CZ",
		Elevation: 1421,
		Latitude:  50.1128005981,
		Longitude: 14.0896997452,
		Timezone:  "Europe/Prague",
	},
	"LKKM": {
		ICAO:      "LKKM",
		Name:      "Kromeriz Airport",
		City:      "Kromeriz",
		State:     "Zlin",
		Country:   "CZ",
		Elevation: 620,
		Latitude:  49.2855987549,
		Longitude: 17.4158000946,
		Timezone:  "Europe/Prague",
	},
	"LKKO": {
		ICAO:      "LKKO",
		Name:      "Kolin Airport",
		City:      "Kolin",
		State:     "Central-Bohemia",
		Country:   "CZ",
		Elevation: 932,
		Latitude:  50.0018997192,
		Longitude: 15.1732997894,
		Timezone:  "Europe/Prague",
	},
	"LKKR": {
		ICAO:      "LKKR",
		Name:      "Krnov Airport",
		City:      "Krnov",
		State:     "Moravskoslezsky",
		Country:   "CZ",
		Elevation: 1230,
		Latitude:  50.0741996765,
		Longitude: 17.6947002411,
		Timezone:  "Europe/Prague",
	},
	"LKKT": {
		ICAO:      "LKKT",
		Name:      "Klatovy Airport",
		City:      "Klatovy",
		State:     "Plzensky",
		Country:   "CZ",
		Elevation: 1299,
		Latitude:  49.4183006287,
		Longitude: 13.3219003677,
		Timezone:  "Europe/Prague",
	},
	"LKKU": {
		ICAO:      "LKKU",
		IATA:      "UHE",
		Name:      "Kunovice Airport",
		City:      "Uherske Hradiste",
		State:     "Zlin",
		Country:   "CZ",
		Elevation: 581,
		Latitude:  49.0293998718,
		Longitude: 17.439699173,
		Timezone:  "Europe/Prague",
	},
	"LKKV": {
		ICAO:      "LKKV",
		IATA:      "KLV",
		Name:      "Karlovy Vary International Airport",
		City:      "Karlovy Vary",
		State:     "Karlovarsky",
		Country:   "CZ",
		Elevation: 1989,
		Latitude:  50.202999115,
		Longitude: 12.9149999619,
		Timezone:  "Europe/Prague",
	},
	"LKKY": {
		ICAO:      "LKKY",
		Name:      "Kyjov Airport",
		City:      "Kyjov",
		State:     "South-Moravian",
		Country:   "CZ",
		Elevation: 689,
		Latitude:  48.9799995422,
		Longitude: 17.1247005463,
		Timezone:  "Europe/Prague",
	},
	"LKLB": {
		ICAO:      "LKLB",
		Name:      "Liberec Airport",
		City:      "Liberec",
		State:     "Liberecky",
		Country:   "CZ",
		Elevation: 1329,
		Latitude:  50.7682991028,
		Longitude: 15.0249996185,
		Timezone:  "Europe/Prague",
	},
	"LKLN": {
		ICAO:      "LKLN",
		Name:      "Plzen Line Airport",
		City:      "Plzen",
		State:     "Plzensky",
		Country:   "CZ",
		Elevation: 1188,
		Latitude:  49.675201416,
		Longitude: 13.274600029,
		Timezone:  "Europe/Prague",
	},
	"LKLT": {
		ICAO:      "LKLT",
		Name:      "Letnany Airport",
		City:      "Letnany",
		State:     "Praha",
		Country:   "CZ",
		Elevation: 909,
		Latitude:  50.131401062,
		Longitude: 14.5256004333,
		Timezone:  "Europe/Prague",
	},
	"LKMB": {
		ICAO:      "LKMB",
		Name:      "Mlada Boleslav Airport",
		City:      "Mlada Boleslav",
		State:     "Central-Bohemia",
		Country:   "CZ",
		Elevation: 781,
		Latitude:  50.3983001709,
		Longitude: 14.8983001709,
		Timezone:  "Europe/Prague",
	},
	"LKMH": {
		ICAO:      "LKMH",
		Name:      "Mnichovo Hradiste Airport",
		City:      "Mnichovo Hradiste",
		State:     "Central-Bohemia",
		Country:   "CZ",
		Elevation: 800,
		Latitude:  50.5401992798,
		Longitude: 15.0066003799,
		Timezone:  "Europe/Prague",
	},
	"LKMI": {
		ICAO:      "LKMI",
		Name:      "Mikulovice Airport",
		City:      "Mikulovice",
		State:     "Olomoucky",
		Country:   "CZ",
		Elevation: 1381,
		Latitude:  50.301700592,
		Longitude: 17.2975006104,
		Timezone:  "Europe/Prague",
	},
	"LKMK": {
		ICAO:      "LKMK",
		Name:      "Moravska Trebova Airport",
		City:      "Moravska Trebova",
		State:     "Pardubicky",
		Country:   "CZ",
		Elevation: 1319,
		Latitude:  49.7983016968,
		Longitude: 16.6877994537,
		Timezone:  "Europe/Prague",
	},
	"LKMO": {
		ICAO:      "LKMO",
		Name:      "Most Airport",
		City:      "Most",
		State:     "Ustecky",
		Country:   "CZ",
		Elevation: 1089,
		Latitude:  50.5250015259,
		Longitude: 13.6830997467,
		Timezone:  "Europe/Prague",
	},
	"LKMR": {
		ICAO:      "LKMR",
		IATA:      "MKA",
		Name:      "Marianske Lazne Airport",
		City:      "Marianske Lazne",
		State:     "Karlovarsky",
		Country:   "CZ",
		Elevation: 1772,
		Latitude:  49.9227981567,
		Longitude: 12.7246999741,
		Timezone:  "Europe/Prague",
	},
	"LKMT": {
		ICAO:      "LKMT",
		IATA:      "OSR",
		Name:      "Ostrava Leos Janacek Airport",
		City:      "Ostrava",
		State:     "Moravskoslezsky",
		Country:   "CZ",
		Elevation: 844,
		Latitude:  49.6963005066,
		Longitude: 18.1110992432,
		Timezone:  "Europe/Prague",
	},
	"LKNA": {
		ICAO:      "LKNA",
		Name:      "Namest Air Base",
		City:      "Namest",
		State:     "Vysocina",
		Country:   "CZ",
		Elevation: 1548,
		Latitude:  49.1657981873,
		Longitude: 16.1249008179,
		Timezone:  "Europe/Prague",
	},
	"LKNM": {
		ICAO:      "LKNM",
		Name:      "Nove Mesto Airport",
		City:      "Nove Mesto Nad Metuji Morave",
		State:     "Kralovehradecky",
		Country:   "CZ",
		Elevation: 1001,
		Latitude:  50.364200592,
		Longitude: 16.1135997772,
		Timezone:  "Europe/Prague",
	},
	"LKNY": {
		ICAO:      "LKNY",
		Name:      "Nymburk Ul Ploch Airport",
		City:      "Nymburk",
		State:     "Central-Bohemia",
		Country:   "CZ",
		Elevation: 607,
		Latitude:  50.168598175,
		Longitude: 15.0524997711,
		Timezone:  "Europe/Prague",
	},
	"LKOL": {
		ICAO:      "LKOL",
		IATA:      "OLO",
		Name:      "Olomouc Glider Airport",
		City:      "Olomouc",
		State:     "Olomoucky",
		Country:   "CZ",
		Elevation: 869,
		Latitude:  49.5877990723,
		Longitude: 17.2108001709,
		Timezone:  "Europe/Prague",
	},
	"LKOT": {
		ICAO:      "LKOT",
		Name:      "Otrokovice Zlin Airport",
		City:      "Otrokovice",
		State:     "Zlin",
		Country:   "CZ",
		Elevation: 600,
		Latitude:  49.198299408,
		Longitude: 17.5177993774,
		Timezone:  "Europe/Prague",
	},
	"LKPA": {
		ICAO:      "LKPA",
		Name:      "Policka Airport",
		City:      "Policka",
		State:     "Pardubicky",
		Country:   "CZ",
		Elevation: 1982,
		Latitude:  49.7393989563,
		Longitude: 16.2588996887,
		Timezone:  "Europe/Prague",
	},
	"LKPC": {
		ICAO:      "LKPC",
		Name:      "Panensky Tynec Airport",
		City:      "Panensky Tynec",
		State:     "Ustecky",
		Country:   "CZ",
		Elevation: 1207,
		Latitude:  50.306098938,
		Longitude: 13.9342002869,
		Timezone:  "Europe/Prague",
	},
	"LKPD": {
		ICAO:      "LKPD",
		IATA:      "PED",
		Name:      "Pardubice Airport",
		City:      "Pardubice",
		State:     "Pardubicky",
		Country:   "CZ",
		Elevation: 741,
		Latitude:  50.0134010315,
		Longitude: 15.7385997772,
		Timezone:  "Europe/Prague",
	},
	"LKPI": {
		ICAO:      "LKPI",
		Name:      "Pribyslav Airport",
		City:      "Pribyslav",
		State:     "Vysocina",
		Country:   "CZ",
		Elevation: 1739,
		Latitude:  49.5807991028,
		Longitude: 15.7628002167,
		Timezone:  "Europe/Prague",
	},
	"LKPJ": {
		ICAO:      "LKPJ",
		Name:      "Prostejov Airport",
		City:      "Prostejov",
		State:     "Olomoucky",
		Country:   "CZ",
		Elevation: 699,
		Latitude:  49.4477996826,
		Longitude: 17.1338996887,
		Timezone:  "Europe/Prague",
	},
	"LKPL": {
		ICAO:      "LKPL",
		Name:      "Letkov Airport",
		City:      "Letkov",
		State:     "Plzensky",
		Country:   "CZ",
		Elevation: 1371,
		Latitude:  49.7230987549,
		Longitude: 13.4521999359,
		Timezone:  "Europe/Prague",
	},
	"LKPM": {
		ICAO:      "LKPM",
		Name:      "Pribram Airport",
		City:      "Pribram",
		State:     "Central-Bohemia",
		Country:   "CZ",
		Elevation: 1529,
		Latitude:  49.7201004028,
		Longitude: 14.1006002426,
		Timezone:  "Europe/Prague",
	},
	"LKPN": {
		ICAO:      "LKPN",
		Name:      "Podhorany Glider Airport",
		City:      "Podhoramy",
		State:     "Pardubicky",
		Country:   "CZ",
		Elevation: 1250,
		Latitude:  49.939201355,
		Longitude: 15.5496997833,
		Timezone:  "Europe/Prague",
	},
	"LKPO": {
		ICAO:      "LKPO",
		IATA:      "PRV",
		Name:      "Prerov Air Base",
		City:      "Prerov",
		State:     "Olomoucky",
		Country:   "CZ",
		Elevation: 676,
		Latitude:  49.4258003235,
		Longitude: 17.4046993256,
		Timezone:  "Europe/Prague",
	},
	"LKPR": {
		ICAO:      "LKPR",
		IATA:      "PRG",
		Name:      "Ruzyne International Airport",
		City:      "Prague",
		State:     "Praha",
		Country:   "CZ",
		Elevation: 1247,
		Latitude:  50.1007995605,
		Longitude: 14.2600002289,
		Timezone:  "Europe/Prague",
	},
	"LKPS": {
		ICAO:      "LKPS",
		Name:      "Plasy Rybnice Airport",
		City:      "Rybnice",
		State:     "Plzensky",
		Country:   "CZ",
		Elevation: 1430,
		Latitude:  49.92029953,
		Longitude: 13.3768997192,
		Timezone:  "Europe/Prague",
	},
	"LKRA": {
		ICAO:      "LKRA",
		Name:      "Rana Loumy Airport",
		City:      "Rana",
		State:     "Ustecky",
		Country:   "CZ",
		Elevation: 879,
		Latitude:  50.4039001465,
		Longitude: 13.7518997192,
		Timezone:  "Europe/Prague",
	},
	"LKRK": {
		ICAO:      "LKRK",
		Name:      "Rakovnik Glider Airport",
		City:      "Rakovnik",
		State:     "Central-Bohemia",
		Country:   "CZ",
		Elevation: 1270,
		Latitude:  50.0942001343,
		Longitude: 13.6888999939,
		Timezone:  "Europe/Prague",
	},
	"LKRO": {
		ICAO:      "LKRO",
		Name:      "Roudnice Mad Airport",
		City:      "Roudnice Nad Labem",
		State:     "Ustecky",
		Country:   "CZ",
		Elevation: 732,
		Latitude:  50.4105987549,
		Longitude: 14.226099968,
		Timezone:  "Europe/Prague",
	},
	"LKSA": {
		ICAO:      "LKSA",
		Name:      "Stankov Glider Airport",
		City:      "Stankov",
		State:     "Plzensky",
		Country:   "CZ",
		Elevation: 1401,
		Latitude:  49.5666999817,
		Longitude: 13.0486001968,
		Timezone:  "Europe/Prague",
	},
	"LKSB": {
		ICAO:      "LKSB",
		Name:      "Stichovice Pluml Airport",
		City:      "Mostkovice",
		State:     "Olomoucky",
		Country:   "CZ",
		Elevation: 840,
		Latitude:  49.4860992432,
		Longitude: 17.0555992126,
		Timezone:  "Europe/Prague",
	},
	"LKSK": {
		ICAO:      "LKSK",
		Name:      "Skutec Airport",
		City:      "Skutec",
		State:     "Pardubicky",
		Country:   "CZ",
		Elevation: 1601,
		Latitude:  49.8278007507,
		Longitude: 16.0058002472,
		Timezone:  "Europe/Prague",
	},
	"LKSN": {
		ICAO:      "LKSN",
		Name:      "Slany Airport",
		City:      "Slany",
		State:     "Central-Bohemia",
		Country:   "CZ",
		Elevation: 1079,
		Latitude:  50.2167015076,
		Longitude: 14.0886001587,
		Timezone:  "Europe/Prague",
	},
	"LKSO": {
		ICAO:      "LKSO",
		Name:      "Sobeslav Airport",
		City:      "Sobeslav",
		State:     "Jihocesky",
		Country:   "CZ",
		Elevation: 1342,
		Latitude:  49.2467002869,
		Longitude: 14.7136001587,
		Timezone:  "Europe/Prague",
	},
	"LKSR": {
		ICAO:      "LKSR",
		Name:      "Strunkovice Airport",
		City:      "Strunkovice",
		State:     "Jihocesky",
		Country:   "CZ",
		Elevation: 1572,
		Latitude:  49.0825004578,
		Longitude: 14.075799942,
		Timezone:  "Europe/Prague",
	},
	"LKST": {
		ICAO:      "LKST",
		Name:      "Strakonice Airport",
		City:      "Strakonice",
		State:     "Jihocesky",
		Country:   "CZ",
		Elevation: 1381,
		Latitude:  49.251701355,
		Longitude: 13.8928003311,
		Timezone:  "Europe/Prague",
	},
	"LKSU": {
		ICAO:      "LKSU",
		Name:      "Sumperk Airport",
		City:      "Sumperk",
		State:     "Olomoucky",
		Country:   "CZ",
		Elevation: 1099,
		Latitude:  49.9606018066,
		Longitude: 17.0177993774,
		Timezone:  "Europe/Prague",
	},
	"LKSZ": {
		ICAO:      "LKSZ",
		Name:      "Sazena Kralupy Airport",
		City:      "Sazena",
		State:     "Central-Bohemia",
		Country:   "CZ",
		Elevation: 761,
		Latitude:  50.3246994019,
		Longitude: 14.2588996887,
		Timezone:  "Europe/Prague",
	},
	"LKTA": {
		ICAO:      "LKTA",
		Name:      "Tabor Airport",
		City:      "Tabor",
		State:     "Jihocesky",
		Country:   "CZ",
		Elevation: 1440,
		Latitude:  49.3911018372,
		Longitude: 14.7082996368,
		Timezone:  "Europe/Prague",
	},
	"LKTB": {
		ICAO:      "LKTB",
		IATA:      "BRQ",
		Name:      "Brno-Turany Airport",
		City:      "Brno",
		State:     "South-Moravian",
		Country:   "CZ",
		Elevation: 778,
		Latitude:  49.1512985229,
		Longitude: 16.6944007874,
		Timezone:  "Europe/Prague",
	},
	"LKTC": {
		ICAO:      "LKTC",
		Name:      "Tocna Praha Glider Airport",
		City:      "Tocna",
		State:     "Praha",
		Country:   "CZ",
		Elevation: 1030,
		Latitude:  49.9852981567,
		Longitude: 14.4260997772,
		Timezone:  "Europe/Prague",
	},
	"LKTD": {
		ICAO:      "LKTD",
		Name:      "Tachov Oldrichov Airport",
		City:      "Tachov",
		State:     "Plzensky",
		Country:   "CZ",
		Elevation: 1640,
		Latitude:  49.7977981567,
		Longitude: 12.7068996429,
		Timezone:  "Europe/Prague",
	},
	"LKTO": {
		ICAO:      "LKTO",
		Name:      "Touzim Airport",
		City:      "Touzim",
		State:     "Karlovarsky",
		Country:   "CZ",
		Elevation: 2139,
		Latitude:  50.0863990784,
		Longitude: 12.9527997971,
		Timezone:  "Europe/Prague",
	},
	"LKTV": {
		ICAO:      "LKTV",
		Name:      "Tabor Vsechov Ultralight Airport",
		City:      "Tabor",
		State:     "Jihocesky",
		Country:   "CZ",
		Elevation: 1604,
		Latitude:  49.4378013611,
		Longitude: 14.6217002869,
		Timezone:  "Europe/Prague",
	},
	"LKUL": {
		ICAO:      "LKUL",
		Name:      "Usti Nad Labem Airport",
		City:      "Usti Nad Labem",
		State:     "Ustecky",
		Country:   "CZ",
		Elevation: 791,
		Latitude:  50.6996994019,
		Longitude: 13.9696998596,
		Timezone:  "Europe/Prague",
	},
	"LKUO": {
		ICAO:      "LKUO",
		Name:      "Usti Mad Orlici Airport",
		City:      "Usti Mad Orlici",
		State:     "Pardubicky",
		Country:   "CZ",
		Elevation: 1342,
		Latitude:  49.9785995483,
		Longitude: 16.426399231,
		Timezone:  "Europe/Prague",
	},
	"LKVL": {
		ICAO:      "LKVL",
		Name:      "Vlasin Glider Airport",
		City:      "Vlasin",
		State:     "Central-Bohemia",
		Country:   "CZ",
		Elevation: 1421,
		Latitude:  49.7289009094,
		Longitude: 14.8788995743,
		Timezone:  "Europe/Prague",
	},
	"LKVM": {
		ICAO:      "LKVM",
		Name:      "Vysoke Myto Glider Airport",
		City:      "Vysoke Myto",
		State:     "Pardubicky",
		Country:   "CZ",
		Elevation: 991,
		Latitude:  49.9268989563,
		Longitude: 16.1858005524,
		Timezone:  "Europe/Prague",
	},
	"LKVO": {
		ICAO:      "LKVO",
		IATA:      "VOD",
		Name:      "Vodochody Airport",
		City:      "Vodochoky",
		State:     "Central-Bohemia",
		Country:   "CZ",
		Elevation: 919,
		Latitude:  50.2165985107,
		Longitude: 14.3957996368,
		Timezone:  "Europe/Prague",
	},
	"LKVP": {
		ICAO:      "LKVP",
		Name:      "Velke Porici Glider Airport",
		City:      "Velke Porici",
		State:     "Kralovehradecky",
		Country:   "CZ",
		Elevation: 1322,
		Latitude:  50.4678001404,
		Longitude: 16.2056007385,
		Timezone:  "Europe/Prague",
	},
	"LKVR": {
		ICAO:      "LKVR",
		Name:      "Vrchlabi Airport",
		City:      "Vrchlabi",
		State:     "Kralovehradecky",
		Country:   "CZ",
		Elevation: 1611,
		Latitude:  50.6241989136,
		Longitude: 15.6464004517,
		Timezone:  "Europe/Prague",
	},
	"LKVY": {
		ICAO:      "LKVY",
		Name:      "Vyskov Airport",
		City:      "Vyskov",
		State:     "South-Moravian",
		Country:   "CZ",
		Elevation: 922,
		Latitude:  49.3003005981,
		Longitude: 17.0252990723,
		Timezone:  "Europe/Prague",
	},
	"LKZA": {
		ICAO:      "LKZA",
		IATA:      "ZBE",
		Name:      "Zabreh Ostrava Airport",
		City:      "Zabreh",
		State:     "Moravskoslezsky",
		Country:   "CZ",
		Elevation: 791,
		Latitude:  49.9282989502,
		Longitude: 18.0783004761,
		Timezone:  "Europe/Prague",
	},
	"LKZB": {
		ICAO:      "LKZB",
		Name:      "Zbraslavice Glider Airport",
		City:      "Zbraslavice",
		State:     "Central-Bohemia",
		Country:   "CZ",
		Elevation: 1621,
		Latitude:  49.814201355,
		Longitude: 15.2017002106,
		Timezone:  "Europe/Prague",
	},
	"LKZD": {
		ICAO:      "LKZD",
		Name:      "Zatec Macerka Ar Airport",
		City:      "Zatek",
		State:     "Ustecky",
		Country:   "CZ",
		Elevation: 879,
		Latitude:  50.3175010681,
		Longitude: 13.5128002167,
		Timezone:  "Europe/Prague",
	},
	"LKZM": {
		ICAO:      "LKZM",
		Name:      "Zamberk Airport",
		City:      "Zamberk",
		State:     "Pardubicky",
		Country:   "CZ",
		Elevation: 1411,
		Latitude:  50.0839004517,
		Longitude: 16.4438991547,
		Timezone:  "Europe/Prague",
	},
	"LKZN": {
		ICAO:      "LKZN",
		Name:      "Znojmo Airport",
		City:      "Znojmo",
		State:     "South-Moravian",
		Country:   "CZ",
		Elevation: 830,
		Latitude:  48.8177986145,
		Longitude: 16.0657997131,
		Timezone:  "Europe/Prague",
	},
	"LL07": {
		ICAO:      "LL07",
		Name:      "Tautz Airport",
		City:      "Mount Carroll",
		State:     "Illinois",
		Country:   "US",
		Elevation: 853,
		Latitude:  42.0638999939,
		Longitude: -89.9695968628,
		Timezone:  "America/Chicago",
	},
	"LL09": {
		ICAO:      "LL09",
		Name:      "Air Estates Inc Airport",
		City:      "Mundelein",
		State:     "Illinois",
		Country:   "US",
		Elevation: 800,
		Latitude:  42.2806015015,
		Longitude: -88.0944976807,
		Timezone:  "America/Chicago",
	},
	"LL10": {
		ICAO:      "LL10",
		Name:      "Naper Aero Club Airport",
		City:      "Naperville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 708,
		Latitude:  41.7347984314,
		Longitude: -88.2033996582,
		Timezone:  "America/Chicago",
	},
	"LL13": {
		ICAO:      "LL13",
		Name:      "Stoney Real Estate Airport",
		City:      "Enfield",
		State:     "Illinois",
		Country:   "US",
		Elevation: 460,
		Latitude:  38.0485992432,
		Longitude: -88.3005981445,
		Timezone:  "America/Chicago",
	},
	"LL18": {
		ICAO:      "LL18",
		Name:      "Ernest E Orwig Airport",
		City:      "North Henderson",
		State:     "Illinois",
		Country:   "US",
		Elevation: 790,
		Latitude:  41.0973014832,
		Longitude: -90.4529037476,
		Timezone:  "America/Chicago",
	},
	"LL1B": {
		ICAO:      "LL1B",
		Name:      "Arad Airport",
		City:      "Arad",
		Country:   "IL",
		Elevation: 1952,
		Latitude:  31.2291698456,
		Longitude: 35.189037323,
		Timezone:  "Asia/Jerusalem",
	},
	"LL20": {
		ICAO:      "LL20",
		Name:      "Smith Restricted Landing Area",
		City:      "Greenfield",
		State:     "Illinois",
		Country:   "US",
		Elevation: 582,
		Latitude:  39.3130989075,
		Longitude: -90.2778015137,
		Timezone:  "America/Chicago",
	},
	"LL22": {
		ICAO:      "LL22",
		Name:      "Brookeridge Air Park",
		City:      "Downers Grove",
		State:     "Illinois",
		Country:   "US",
		Elevation: 750,
		Latitude:  41.7327003479,
		Longitude: -87.9992980957,
		Timezone:  "America/Chicago",
	},
	"LL23": {
		ICAO:      "LL23",
		Name:      "Meyer Airport",
		City:      "Oquawka",
		State:     "Illinois",
		Country:   "US",
		Elevation: 580,
		Latitude:  41.01720047,
		Longitude: -90.8777999878,
		Timezone:  "America/Chicago",
	},
	"LL24": {
		ICAO:      "LL24",
		Name:      "Sunset Acres Airport",
		City:      "Manteno",
		State:     "Illinois",
		Country:   "US",
		Elevation: 665,
		Latitude:  41.2070007324,
		Longitude: -87.7853012085,
		Timezone:  "America/Chicago",
	},
	"LL26": {
		ICAO:      "LL26",
		Name:      "Schaller Airport",
		City:      "New Athens",
		State:     "Illinois",
		Country:   "US",
		Elevation: 434,
		Latitude:  38.2750015259,
		Longitude: -89.8709030151,
		Timezone:  "America/Chicago",
	},
	"LL27": {
		ICAO:      "LL27",
		Name:      "Smith Airport",
		City:      "Macomb",
		State:     "Illinois",
		Country:   "US",
		Elevation: 702,
		Latitude:  40.4542007446,
		Longitude: -90.6234970093,
		Timezone:  "America/Chicago",
	},
	"LL28": {
		ICAO:      "LL28",
		Name:      "Sd Aero Airport",
		City:      "Capron",
		State:     "Illinois",
		Country:   "US",
		Elevation: 930,
		Latitude:  42.432800293,
		Longitude: -88.7172012329,
		Timezone:  "America/Chicago",
	},
	"LL32": {
		ICAO:      "LL32",
		Name:      "C D Maulding Airport",
		City:      "Paxton",
		State:     "Illinois",
		Country:   "US",
		Elevation: 800,
		Latitude:  40.4500007629,
		Longitude: -88.0749969482,
		Timezone:  "America/Chicago",
	},
	"LL33": {
		ICAO:      "LL33",
		Name:      "Rdh Farms Airport",
		City:      "Good Hope",
		State:     "Illinois",
		Country:   "US",
		Elevation: 745,
		Latitude:  40.57970047,
		Longitude: -90.6797027588,
		Timezone:  "America/Chicago",
	},
	"LL34": {
		ICAO:      "LL34",
		Name:      "Thorp Airport",
		City:      "Wapella",
		State:     "Illinois",
		Country:   "US",
		Elevation: 750,
		Latitude:  40.2167015076,
		Longitude: -88.900100708,
		Timezone:  "America/Chicago",
	},
	"LL38": {
		ICAO:      "LL38",
		Name:      "Stutzke Airport",
		City:      "Sterling",
		State:     "Illinois",
		Country:   "US",
		Elevation: 740,
		Latitude:  41.8102989197,
		Longitude: -89.8383026123,
		Timezone:  "America/Chicago",
	},
	"LL39": {
		ICAO:      "LL39",
		Name:      "Curanda Airport",
		City:      "Morris",
		State:     "Illinois",
		Country:   "US",
		Elevation: 620,
		Latitude:  41.2574996948,
		Longitude: -88.4777984619,
		Timezone:  "America/Chicago",
	},
	"LL40": {
		ICAO:      "LL40",
		Name:      "Frankfort Airport",
		City:      "Frankfort",
		State:     "Illinois",
		Country:   "US",
		Elevation: 778,
		Latitude:  41.4775009155,
		Longitude: -87.8404998779,
		Timezone:  "America/Chicago",
	},
	"LL43": {
		ICAO:      "LL43",
		Name:      "Gerald H Hamer Airport",
		City:      "Peru",
		State:     "Illinois",
		Country:   "US",
		Elevation: 690,
		Latitude:  41.4500007629,
		Longitude: -89.1667022705,
		Timezone:  "America/Chicago",
	},
	"LL44": {
		ICAO:      "LL44",
		Name:      "Trovero Airport",
		City:      "Peru",
		State:     "Illinois",
		Country:   "US",
		Elevation: 625,
		Latitude:  41.2985992432,
		Longitude: -89.1092987061,
		Timezone:  "America/Chicago",
	},
	"LL45": {
		ICAO:      "LL45",
		Name:      "Lindell Loveless Airport",
		City:      "Gillespie",
		State:     "Illinois",
		Country:   "US",
		Elevation: 657,
		Latitude:  39.1823005676,
		Longitude: -89.8218002319,
		Timezone:  "America/Chicago",
	},
	"LL46": {
		ICAO:      "LL46",
		Name:      "Read Airport",
		City:      "Piper City",
		State:     "Illinois",
		Country:   "US",
		Elevation: 663,
		Latitude:  40.8353004456,
		Longitude: -88.1528015137,
		Timezone:  "America/Chicago",
	},
	"LL48": {
		ICAO:      "LL48",
		Name:      "Wilcoxon Airport",
		City:      "Athens",
		State:     "Illinois",
		Country:   "US",
		Elevation: 600,
		Latitude:  39.9491996765,
		Longitude: -89.7425003052,
		Timezone:  "America/Chicago",
	},
	"LL49": {
		ICAO:      "LL49",
		Name:      "Raymond Restricted Landing Area",
		City:      "Jerseyville",
		State:     "Illinois",
		Country:   "US",
		Elevation: 658,
		Latitude:  39.0621986389,
		Longitude: -90.2722015381,
		Timezone:  "America/Chicago",
	},
	"LL51": {
		ICAO:      "LL51",
		Name:      "Riley's Field",
		City:      "Plainfield",
		State:     "Illinois",
		Country:   "US",
		Elevation: 663,
		Latitude:  41.6041984558,
		Longitude: -88.2736968994,
		Timezone:  "America/Chicago",
	},
	"LL52": {
		ICAO:      "LL52",
		Name:      "Wilts Airport",
		City:      "Streator",
		State:     "Illinois",
		Country:   "US",
		Elevation: 650,
		Latitude:  41.1610984802,
		Longitude: -88.9391021729,
		Timezone:  "America/Chicago",
	},
	"LL53": {
		ICAO:      "LL53",
		Name:      "Olson Airport",
		City:      "Plato Center",
		State:     "Illinois",
		Country:   "US",
		Elevation: 955,
		Latitude:  42.0088996887,
		Longitude: -88.4576034546,
		Timezone:  "America/Chicago",
	},
	"LL54": {
		ICAO:      "LL54",
		Name:      "Dunn Airport",
		City:      "Paw Paw",
		State:     "Illinois",
		Country:   "US",
		Elevation: 850,
		Latitude:  41.7193984985,
		Longitude: -89.0299987793,
		Timezone:  "America/Chicago",
	},
	"LL55": {
		ICAO:      "LL55",
		Name:      "Gentry Airport",
		City:      "Polo",
		State:     "Illinois",
		Country:   "US",
		Elevation: 865,
		Latitude:  41.9908981323,
		Longitude: -89.5597991943,
		Timezone:  "America/Chicago",
	},
	"LL56": {
		ICAO:      "LL56",
		Name:      "Schott Airport",
		City:      "Pontiac",
		State:     "Illinois",
		Country:   "US",
		Elevation: 659,
		Latitude:  40.8842010498,
		Longitude: -88.5908966064,
		Timezone:  "America/Chicago",
	},
	"LL59": {
		ICAO:      "LL59",
		Name:      "Palmahim Air Base",
		City:      "Rishon LeZion",
		Country:   "IL",
		Elevation: 32,
		Latitude:  31.8978996277,
		Longitude: 34.6907997131,
		Timezone:  "Asia/Jerusalem",
	},
	"LL61": {
		ICAO:      "LL61",
		Name:      "Doering's Port Airport",
		City:      "Prophetstown",
		State:     "Illinois",
		Country:   "US",
		Elevation: 606,
		Latitude:  41.6856002808,
		Longitude: -89.9026031494,
		Timezone:  "America/Chicago",
	},
	"LL64": {
		ICAO:      "LL64",
		Name:      "Ralph E. Daniels Airport",
		City:      "Putnam",
		State:     "Illinois",
		Country:   "US",
		Elevation: 457,
		Latitude:  41.1995010376,
		Longitude: -89.3945007324,
		Timezone:  "America/Chicago",
	},
	"LL65": {
		ICAO:      "LL65",
		Name:      "Edgar Read Airport",
		City:      "Putnam",
		State:     "Illinois",
		Country:   "US",
		Elevation: 690,
		Latitude:  41.1944999695,
		Longitude: -89.4501037598,
		Timezone:  "America/Chicago",
	},
	"LL66": {
		ICAO:      "LL66",
		Name:      "Egland Field",
		City:      "Ransom",
		State:     "Illinois",
		Country:   "US",
		Elevation: 750,
		Latitude:  41.2122993469,
		Longitude: -88.6930999756,
		Timezone:  "America/Chicago",
	},
	"LL69": {
		ICAO:      "LL69",
		Name:      "Brewer Airport",
		City:      "Ramsey",
		State:     "Illinois",
		Country:   "US",
		Elevation: 640,
		Latitude:  39.2108421986,
		Longitude: -89.1182398796,
		Timezone:  "America/Chicago",
	},
	"LL74": {
		ICAO:      "LL74",
		Name:      "Maple Hurst Farms Airport",
		City:      "Rochelle",
		State:     "Illinois",
		Country:   "US",
		Elevation: 823,
		Latitude:  42.0083999634,
		Longitude: -89.0486984253,
		Timezone:  "America/Chicago",
	},
	"LL75": {
		ICAO:      "LL75",
		Name:      "Chester Wyss Airport",
		City:      "Rock City",
		State:     "Illinois",
		Country:   "US",
		Elevation: 920,
		Latitude:  42.4249992371,
		Longitude: -89.4501037598,
		Timezone:  "America/Chicago",
	},
	"LL78": {
		ICAO:      "LL78",
		Name:      "Lz Fairwinds Airport",
		City:      "Rockford",
		State:     "Illinois",
		Country:   "US",
		Elevation: 752,
		Latitude:  42.3116989136,
		Longitude: -89.1509017944,
		Timezone:  "America/Chicago",
	},
	"LL79": {
		ICAO:      "LL79",
		Name:      "Severson Airport",
		City:      "Rockford",
		State:     "Illinois",
		Country:   "US",
		Elevation: 813,
		Latitude:  42.3931007385,
		Longitude: -89.1306991577,
		Timezone:  "America/Chicago",
	},
	"LL80": {
		ICAO:      "LL80",
		Name:      "Darrington Airport",
		City:      "Rockford",
		State:     "Illinois",
		Country:   "US",
		Elevation: 862,
		Latitude:  42.3819999695,
		Longitude: -89.1612014771,
		Timezone:  "America/Chicago",
	},
	"LL81": {
		ICAO:      "LL81",
		Name:      "Spickard-Marshall Airport",
		City:      "Rockford",
		State:     "Illinois",
		Country:   "US",
		Elevation: 778,
		Latitude:  42.375,
		Longitude: -89.070098877,
		Timezone:  "America/Chicago",
	},
	"LL87": {
		ICAO:      "LL87",
		Name:      "Compass Rose Airport",
		City:      "Rockton",
		State:     "Illinois",
		Country:   "US",
		Elevation: 970,
		Latitude:  42.4556007385,
		Longitude: -88.9029006958,
		Timezone:  "America/Chicago",
	},
	"LL88": {
		ICAO:      "LL88",
		Name:      "Harry D Fenton Airport",
		City:      "Rockton",
		State:     "Illinois",
		Country:   "US",
		Elevation: 770,
		Latitude:  42.4749984741,
		Longitude: -89.0889968872,
		Timezone:  "America/Chicago",
	},
	"LL89": {
		ICAO:      "LL89",
		Name:      "Blackhawk Farms Inc Airport",
		City:      "Rockton",
		State:     "Illinois",
		Country:   "US",
		Elevation: 759,
		Latitude:  42.4872016907,
		Longitude: -89.1079025269,
		Timezone:  "America/Chicago",
	},
	"LL90": {
		ICAO:      "LL90",
		Name:      "Mc Curdy Strip",
		City:      "Rockton/Belvidere",
		State:     "Illinois",
		Country:   "US",
		Elevation: 960,
		Latitude:  42.4458999634,
		Longitude: -88.900100708,
		Timezone:  "America/Chicago",
	},
	"LL91": {
		ICAO:      "LL91",
		Name:      "Hillman Airport",
		City:      "Rock City",
		State:     "Illinois",
		Country:   "US",
		Elevation: 870,
		Latitude:  42.4157981873,
		Longitude: -89.4529037476,
		Timezone:  "America/Chicago",
	},
	"LL94": {
		ICAO:      "LL94",
		Name:      "Mc Curdy Airport",
		City:      "Roscoe",
		State:     "Illinois",
		Country:   "US",
		Elevation: 755,
		Latitude:  42.4345016479,
		Longitude: -88.99949646,
		Timezone:  "America/Chicago",
	},
	"LL97": {
		ICAO:      "LL97",
		Name:      "Rockenbach Airport",
		City:      "Round Lake",
		State:     "Illinois",
		Country:   "US",
		Elevation: 800,
		Latitude:  42.3389015198,
		Longitude: -88.0848007202,
		Timezone:  "America/Chicago",
	},
	"LL98": {
		ICAO:      "LL98",
		Name:      "John W Meils Restricted Landing Area",
		City:      "Rutland",
		State:     "Illinois",
		Country:   "US",
		Elevation: 705,
		Latitude:  40.9435997009,
		Longitude: -89.1082992554,
		Timezone:  "America/Chicago",
	},
	"LLBG": {
		ICAO:      "LLBG",
		IATA:      "TLV",
		Name:      "Ben Gurion International Airport",
		City:      "Tel Aviv",
		State:     "Central-District",
		Country:   "IL",
		Elevation: 135,
		Latitude:  32.0113983154,
		Longitude: 34.8866996765,
		Timezone:  "Asia/Jerusalem",
	},
	"LLBS": {
		ICAO:      "LLBS",
		IATA:      "BEV",
		Name:      "Be'er Sheva (Teyman) Airport",
		City:      "Beersheva",
		State:     "Southern-District",
		Country:   "IL",
		Elevation: 656,
		Latitude:  31.2870006561,
		Longitude: 34.7229995728,
		Timezone:  "Asia/Jerusalem",
	},
	"LLEK": {
		ICAO:      "LLEK",
		Name:      "Tel Nov Air Base",
		City:      "Rehovot",
		Country:   "IL",
		Elevation: 193,
		Latitude:  31.8395004272,
		Longitude: 34.8218002319,
		Timezone:  "Asia/Jerusalem",
	},
	"LLER": {
		ICAO:      "LLER",
		IATA:      "ETM",
		Name:      "Ramon Airport",
		City:      "Eilat",
		State:     "Southern-District",
		Country:   "IL",
		Elevation: 288,
		Latitude:  29.723694,
		Longitude: 35.011417,
		Timezone:  "Asia/Jerusalem",
	},
	"LLES": {
		ICAO:      "LLES",
		Name:      "Ein Shemer Airfield",
		City:      "Ein Shemer",
		Country:   "IL",
		Elevation: 95,
		Latitude:  32.4407997131,
		Longitude: 35.0077018738,
		Timezone:  "Asia/Jerusalem",
	},
	"LLET": {
		ICAO:      "LLET",
		IATA:      "ETH",
		Name:      "Eilat Airport",
		City:      "Eilat",
		State:     "Southern-District",
		Country:   "IL",
		Elevation: 42,
		Latitude:  29.5613002777,
		Longitude: 34.9600982666,
		Timezone:  "Asia/Jerusalem",
	},
	"LLEY": {
		ICAO:      "LLEY",
		IATA:      "EIY",
		Name:      "Ein Yahav Airfield",
		City:      "Sapir",
		State:     "Southern-District",
		Country:   "IL",
		Elevation: -164,
		Latitude:  30.6217002869,
		Longitude: 35.2033004761,
		Timezone:  "Asia/Jerusalem",
	},
	"LLFK": {
		ICAO:      "LLFK",
		Name:      "Fiq Airfield",
		City:      "Katzrin",
		State:     "Quneitra",
		Country:   "IL",
		Elevation: 1218,
		Latitude:  32.786388888,
		Longitude: 35.7174987793,
		Timezone:  "Asia/Jerusalem",
	},
	"LLHA": {
		ICAO:      "LLHA",
		IATA:      "HFA",
		Name:      "Haifa International Airport",
		City:      "Haifa",
		State:     "Haifa",
		Country:   "IL",
		Elevation: 28,
		Latitude:  32.8093986511,
		Longitude: 35.0430984497,
		Timezone:  "Asia/Jerusalem",
	},
	"LLHB": {
		ICAO:      "LLHB",
		Name:      "Hatzerim Air Base",
		City:      "Beersheba",
		Country:   "IL",
		Elevation: 725,
		Latitude:  31.2334003448,
		Longitude: 34.6626014709,
		Timezone:  "Asia/Jerusalem",
	},
	"LLHS": {
		ICAO:      "LLHS",
		Name:      "Hatzor Air Base",
		City:      "Hatzor",
		Country:   "IL",
		Elevation: 148,
		Latitude:  31.7625007629,
		Longitude: 34.7271995544,
		Timezone:  "Asia/Jerusalem",
	},
	"LLHZ": {
		ICAO:      "LLHZ",
		Name:      "Herzliya Airport",
		City:      "Herzliya",
		State:     "Tel-Aviv",
		Country:   "IL",
		Elevation: 121,
		Latitude:  32.1805992126,
		Longitude: 34.8347015381,
		Timezone:  "Asia/Jerusalem",
	},
	"LLIB": {
		ICAO:      "LLIB",
		IATA:      "RPN",
		Name:      "Ben Ya'akov Airport",
		City:      "Rosh Pina",
		State:     "Northern-District",
		Country:   "IL",
		Elevation: 922,
		Latitude:  32.9809989929,
		Longitude: 35.5718994141,
		Timezone:  "Asia/Jerusalem",
	},
	"LLKS": {
		ICAO:      "LLKS",
		IATA:      "KSW",
		Name:      "Kiryat Shmona Airport",
		City:      "Kiryat Shmona",
		State:     "Northern-District",
		Country:   "IL",
		Elevation: 381,
		Latitude:  33.2167015076,
		Longitude: 35.5999984741,
		Timezone:  "Asia/Jerusalem",
	},
	"LLMG": {
		ICAO:      "LLMG",
		Name:      "Megiddo Airport",
		City:      "Afula",
		State:     "Northern-District",
		Country:   "IL",
		Elevation: 200,
		Latitude:  32.5973014832,
		Longitude: 35.2288017273,
		Timezone:  "Asia/Jerusalem",
	},
	"LLMR": {
		ICAO:      "LLMR",
		IATA:      "MIP",
		Name:      "Mitzpe Ramon Airfield",
		City:      "Mitzpe Ramon",
		State:     "Southern-District",
		Country:   "IL",
		Elevation: 2556,
		Latitude:  30.6506004333,
		Longitude: 34.8069000244,
		Timezone:  "Asia/Jerusalem",
	},
	"LLMZ": {
		ICAO:      "LLMZ",
		IATA:      "MTZ",
		Name:      "Bar Yehuda Airfield",
		City:      "Masada",
		State:     "Southern-District",
		Country:   "IL",
		Elevation: -1266,
		Latitude:  31.3281993866,
		Longitude: 35.3885993958,
		Timezone:  "Asia/Amman",
	},
	"LLNV": {
		ICAO:      "LLNV",
		IATA:      "VTM",
		Name:      "Nevatim Air Base",
		City:      "Beersheba",
		State:     "Southern-District",
		Country:   "IL",
		Elevation: 1330,
		Latitude:  31.2082996368,
		Longitude: 35.012298584,
		Timezone:  "Asia/Jerusalem",
	},
	"LLOV": {
		ICAO:      "LLOV",
		IATA:      "VDA",
		Name:      "Ovda International Airport",
		City:      "Eilat",
		State:     "Southern-District",
		Country:   "IL",
		Elevation: 1492,
		Latitude:  29.9402999878,
		Longitude: 34.935798645,
		Timezone:  "Asia/Jerusalem",
	},
	"LLRD": {
		ICAO:      "LLRD",
		Name:      "Ramat David Air Base",
		City:      "Haifa",
		Country:   "IL",
		Elevation: 185,
		Latitude:  32.6651000977,
		Longitude: 35.1795005798,
		Timezone:  "Asia/Jerusalem",
	},
	"LLRM": {
		ICAO:      "LLRM",
		IATA:      "MIP",
		Name:      "Ramon Air Base",
		City:      "Beersheba",
		State:     "Southern-District",
		Country:   "IL",
		Elevation: 2126,
		Latitude:  30.7761001587,
		Longitude: 34.6666984558,
		Timezone:  "Asia/Jerusalem",
	},
	"LLSD": {
		ICAO:      "LLSD",
		IATA:      "SDV",
		Name:      "Sde Dov Airport",
		City:      "Tel Aviv",
		State:     "Tel-Aviv",
		Country:   "IL",
		Elevation: 43,
		Latitude:  32.1147003174,
		Longitude: 34.7821998596,
		Timezone:  "Asia/Jerusalem",
	},
	"LLYT": {
		ICAO:      "LLYT",
		IATA:      "YOT",
		Name:      "Yotvata Airfield",
		City:      "Yotvata",
		State:     "Southern-District",
		Country:   "IL",
		Elevation: 300,
		Latitude:  29.9062004089,
		Longitude: 35.0666999817,
		Timezone:  "Asia/Jerusalem",
	},
	"LMML": {
		ICAO:      "LMML",
		IATA:      "MLA",
		Name:      "Luqa Airport",
		City:      "Luqa",
		State:     "Luqa",
		Country:   "MT",
		Elevation: 300,
		Latitude:  35.857498,
		Longitude: 14.4775,
		Timezone:  "Europe/Malta",
	},
	"LOAA": {
		ICAO:      "LOAA",
		Name:      "Ottenschlag Airport",
		City:      "Ottenschlag",
		State:     "Lower-Austria",
		Country:   "AT",
		Elevation: 2867,
		Latitude:  48.418598175,
		Longitude: 15.2166996002,
		Timezone:  "Europe/Vienna",
	},
	"LOAB": {
		ICAO:      "LOAB",
		Name:      "Dobersberg Airport",
		City:      "Dobersberg",
		State:     "Lower-Austria",
		Country:   "AT",
		Elevation: 1719,
		Latitude:  48.9233016968,
		Longitude: 15.2966995239,
		Timezone:  "Europe/Vienna",
	},
	"LOAD": {
		ICAO:      "LOAD",
		Name:      "Voeltendorf Airport",
		City:      "Voeltendorf",
		State:     "Lower-Austria",
		Country:   "AT",
		Elevation: 1063,
		Latitude:  48.1617012024,
		Longitude: 15.5882997513,
		Timezone:  "Europe/Vienna",
	},
	"LOAG": {
		ICAO:      "LOAG",
		Name:      "Krems Airport",
		City:      "Krems",
		State:     "Lower-Austria",
		Country:   "AT",
		Elevation: 1022,
		Latitude:  48.4468994141,
		Longitude: 15.6350002289,
		Timezone:  "Europe/Vienna",
	},
	"LOAL": {
		ICAO:      "LOAL",
		Name:      "Leopoldsdorf Airport",
		City:      "Leopoldsdorf",
		State:     "Lower-Austria",
		Country:   "AT",
		Elevation: 486,
		Latitude:  48.2266998291,
		Longitude: 16.6721992493,
		Timezone:  "Europe/Vienna",
	},
	"LOAN": {
		ICAO:      "LOAN",
		Name:      "Wiener Neustadt East Airport",
		City:      "Wiener Neustadt",
		State:     "Lower-Austria",
		Country:   "AT",
		Elevation: 896,
		Latitude:  47.8432998657,
		Longitude: 16.260099411,
		Timezone:  "Europe/Vienna",
	},
	"LOAR": {
		ICAO:      "LOAR",
		Name:      "Altlichtenwarth Airport",
		City:      "Altlichtenwarth",
		State:     "Lower-Austria",
		Country:   "AT",
		Elevation: 604,
		Latitude:  48.6660995483,
		Longitude: 16.8250007629,
		Timezone:  "Europe/Vienna",
	},
	"LOAS": {
		ICAO:      "LOAS",
		Name:      "Spitzerberg Airport",
		City:      "Spitzerberg",
		State:     "Lower-Austria",
		Country:   "AT",
		Elevation: 600,
		Latitude:  48.1021995544,
		Longitude: 16.935300827,
		Timezone:  "Europe/Vienna",
	},
	"LOAT": {
		ICAO:      "LOAT",
		Name:      "Trausdorf Airport",
		City:      "Eisenstadt",
		State:     "Burgenland",
		Country:   "AT",
		Elevation: 525,
		Latitude:  47.7999992371,
		Longitude: 16.5669994354,
		Timezone:  "Europe/Vienna",
	},
	"LOAU": {
		ICAO:      "LOAU",
		Name:      "Stockerau Airport",
		City:      "Stockerau",
		State:     "Lower-Austria",
		Country:   "AT",
		Elevation: 686,
		Latitude:  48.4099998474,
		Longitude: 16.1916999817,
		Timezone:  "Europe/Vienna",
	},
	"LOAV": {
		ICAO:      "LOAV",
		Name:      "Voslau Airport",
		City:      "Bad Voslau",
		State:     "Lower-Austria",
		Country:   "AT",
		Elevation: 765,
		Latitude:  47.9650001526,
		Longitude: 16.2600002289,
		Timezone:  "Europe/Vienna",
	},
	"LOGF": {
		ICAO:      "LOGF",
		Name:      "Fuerstenfeld Airport",
		City:      "Fuerstenfeld",
		State:     "Styria",
		Country:   "AT",
		Elevation: 830,
		Latitude:  47.0625,
		Longitude: 16.0855998993,
		Timezone:  "Europe/Vienna",
	},
	"LOGG": {
		ICAO:      "LOGG",
		Name:      "Punitz-Gussing Airport",
		City:      "Punitz",
		State:     "Burgenland",
		Country:   "AT",
		Elevation: 950,
		Latitude:  47.1465301514,
		Longitude: 16.3168430328,
		Timezone:  "Europe/Vienna",
	},
	"LOGI": {
		ICAO:      "LOGI",
		Name:      "Trieben Airport",
		City:      "Trieben",
		State:     "Styria",
		Country:   "AT",
		Elevation: 2283,
		Latitude:  47.4942016602,
		Longitude: 14.4975004196,
		Timezone:  "Europe/Vienna",
	},
	"LOGK": {
		ICAO:      "LOGK",
		Name:      "Kapfenberg Airport",
		City:      "Kapfenberg",
		State:     "Styria",
		Country:   "AT",
		Elevation: 1690,
		Latitude:  47.4583015442,
		Longitude: 15.3299999237,
		Timezone:  "Europe/Vienna",
	},
	"LOGL": {
		ICAO:      "LOGL",
		Name:      "Turnau Airport",
		City:      "Lanzen",
		State:     "Styria",
		Country:   "AT",
		Elevation: 2575,
		Latitude:  47.5567016602,
		Longitude: 15.3249998093,
		Timezone:  "Europe/Vienna",
	},
	"LOGM": {
		ICAO:      "LOGM",
		Name:      "Mariazell Airport",
		City:      "Mariazell",
		State:     "Styria",
		Country:   "AT",
		Elevation: 2822,
		Latitude:  47.7882995605,
		Longitude: 15.3016996384,
		Timezone:  "Europe/Vienna",
	},
	"LOGO": {
		ICAO:      "LOGO",
		Name:      "Niederoeblarn Airport",
		City:      "Niederoeblarn",
		State:     "Styria",
		Country:   "AT",
		Elevation: 2139,
		Latitude:  47.4782981873,
		Longitude: 14.0082998276,
		Timezone:  "Europe/Vienna",
	},
	"LOGP": {
		ICAO:      "LOGP",
		Name:      "Pinkafeld Airport",
		City:      "Pinkafeld",
		State:     "Burgenland",
		Country:   "AT",
		Elevation: 1339,
		Latitude:  47.3866844177,
		Longitude: 16.1126728058,
		Timezone:  "Europe/Vienna",
	},
	"LOGT": {
		ICAO:      "LOGT",
		Name:      "Timmersdorf Airport",
		City:      "Timmersdorf",
		State:     "Styria",
		Country:   "AT",
		Elevation: 2060,
		Latitude:  47.3800010681,
		Longitude: 14.9682998657,
		Timezone:  "Europe/Vienna",
	},
	"LOGW": {
		ICAO:      "LOGW",
		Name:      "Unterfladnitz Airport",
		City:      "Weiz",
		State:     "Styria",
		Country:   "AT",
		Elevation: 1296,
		Latitude:  47.1707992554,
		Longitude: 15.6653003693,
		Timezone:  "Europe/Vienna",
	},
	"LOIH": {
		ICAO:      "LOIH",
		IATA:      "HOH",
		Name:      "Hohenems-Dornbirn Airport",
		City:      "Hohenems / Dornbirn",
		State:     "Vorarlberg",
		Country:   "AT",
		Elevation: 1352,
		Latitude:  47.3849983215,
		Longitude: 9.6999998093,
		Timezone:  "Europe/Vienna",
	},
	"LOIJ": {
		ICAO:      "LOIJ",
		Name:      "St. Johann In Tirol Airport",
		City:      "St. Johann In Tirol",
		State:     "Tyrol",
		Country:   "AT",
		Elevation: 2198,
		Latitude:  47.5200996399,
		Longitude: 12.4497003555,
		Timezone:  "Europe/Vienna",
	},
	"LOIK": {
		ICAO:      "LOIK",
		Name:      "Langkampfen Airport",
		City:      "Kufstein",
		State:     "Tyrol",
		Country:   "AT",
		Elevation: 1588,
		Latitude:  47.5651168823,
		Longitude: 12.1280698776,
		Timezone:  "Europe/Vienna",
	},
	"LOIR": {
		ICAO:      "LOIR",
		Name:      "Hoefen Airport",
		City:      "Reutte",
		State:     "Tyrol",
		Country:   "AT",
		Elevation: 2805,
		Latitude:  47.4700012207,
		Longitude: 10.6916999817,
		Timezone:  "Europe/Vienna",
	},
	"LOKF": {
		ICAO:      "LOKF",
		Name:      "Ossiacher See Airport",
		City:      "Feldkirchen",
		State:     "Carinthia",
		Country:   "AT",
		Elevation: 1706,
		Latitude:  46.7089004517,
		Longitude: 14.0771999359,
		Timezone:  "Europe/Vienna",
	},
	"LOKG": {
		ICAO:      "LOKG",
		Name:      "Glainach Airport",
		City:      "Ferlach",
		State:     "Carinthia",
		Country:   "AT",
		Elevation: 1499,
		Latitude:  46.5332984924,
		Longitude: 14.3332996368,
		Timezone:  "Europe/Vienna",
	},
	"LOKH": {
		ICAO:      "LOKH",
		Name:      "Hirt Airport",
		City:      "Friesach",
		State:     "Carinthia",
		Country:   "AT",
		Elevation: 2018,
		Latitude:  46.9282989502,
		Longitude: 14.4333000183,
		Timezone:  "Europe/Vienna",
	},
	"LOKL": {
		ICAO:      "LOKL",
		Name:      "Nikolsdorf Airport",
		City:      "Lienz",
		State:     "Tyrol",
		Country:   "AT",
		Elevation: 2087,
		Latitude:  46.7988014221,
		Longitude: 12.8767995834,
		Timezone:  "Europe/Vienna",
	},
	"LOKM": {
		ICAO:      "LOKM",
		Name:      "Mayerhofen Airport",
		City:      "Mayerhofen",
		State:     "Carinthia",
		Country:   "AT",
		Elevation: 2119,
		Latitude:  46.9757995605,
		Longitude: 14.3717002869,
		Timezone:  "Europe/Vienna",
	},
	"LOKN": {
		ICAO:      "LOKN",
		Name:      "Notsch Im Gailtal Airport",
		City:      "Notsch Im Gailtal",
		State:     "Carinthia",
		Country:   "AT",
		Elevation: 1801,
		Latitude:  46.5811004639,
		Longitude: 13.6291999817,
		Timezone:  "Europe/Rome",
	},
	"LOKW": {
		ICAO:      "LOKW",
		Name:      "Wolfsberg Airport",
		City:      "Wolfsberg",
		State:     "Carinthia",
		Country:   "AT",
		Elevation: 1460,
		Latitude:  46.8182983398,
		Longitude: 14.8263998032,
		Timezone:  "Europe/Vienna",
	},
	"LOLC": {
		ICAO:      "LOLC",
		Name:      "Scharnstein Airport",
		City:      "Scharnstein",
		State:     "Upper-Austria",
		Country:   "AT",
		Elevation: 1742,
		Latitude:  47.898601532,
		Longitude: 13.9393997192,
		Timezone:  "Europe/Vienna",
	},
	"LOLE": {
		ICAO:      "LOLE",
		Name:      "Eferding Airport",
		City:      "Eferding",
		State:     "Upper-Austria",
		Country:   "AT",
		Elevation: 886,
		Latitude:  48.3382987976,
		Longitude: 13.9849996567,
		Timezone:  "Europe/Vienna",
	},
	"LOLF": {
		ICAO:      "LOLF",
		Name:      "Freistadt Airport",
		City:      "Freistadt",
		State:     "Upper-Austria",
		Country:   "AT",
		Elevation: 2238,
		Latitude:  48.5149993896,
		Longitude: 14.4083003998,
		Timezone:  "Europe/Vienna",
	},
	"LOLG": {
		ICAO:      "LOLG",
		Name:      "St. Georgen Am Ybbsfelde Airport",
		City:      "St. Georgen Am Ybbsfelde",
		State:     "Lower-Austria",
		Country:   "AT",
		Elevation: 827,
		Latitude:  48.1035995483,
		Longitude: 14.950799942,
		Timezone:  "Europe/Vienna",
	},
	"LOLH": {
		ICAO:      "LOLH",
		Name:      "Hb Hofkirchen Airport",
		City:      "Hofkirchen",
		State:     "Upper-Austria",
		Country:   "AT",
		Elevation: 1175,
		Latitude:  48.1399993896,
		Longitude: 14.3382997513,
		Timezone:  "Europe/Vienna",
	},
	"LOLK": {
		ICAO:      "LOLK",
		Name:      "Kirchheim Airport",
		City:      "Ried",
		State:     "Upper-Austria",
		Country:   "AT",
		Elevation: 1350,
		Latitude:  48.2122001648,
		Longitude: 13.345000267,
		Timezone:  "Europe/Vienna",
	},
	"LOLM": {
		ICAO:      "LOLM",
		Name:      "Micheldorf Airport",
		City:      "Micheldorf",
		State:     "Upper-Austria",
		Country:   "AT",
		Elevation: 1509,
		Latitude:  47.8708000183,
		Longitude: 14.125,
		Timezone:  "Europe/Vienna",
	},
	"LOLO": {
		ICAO:      "LOLO",
		Name:      "Linz-Ost Airport",
		City:      "Linz",
		State:     "Upper-Austria",
		Country:   "AT",
		Elevation: 827,
		Latitude:  48.2999992371,
		Longitude: 14.3332996368,
		Timezone:  "Europe/Vienna",
	},
	"LOLS": {
		ICAO:      "LOLS",
		Name:      "Schaerding-Suben Airport",
		City:      "Schaerding / Suben",
		State:     "Upper-Austria",
		Country:   "AT",
		Elevation: 1070,
		Latitude:  48.403301239,
		Longitude: 13.4483003616,
		Timezone:  "Europe/Vienna",
	},
	"LOLT": {
		ICAO:      "LOLT",
		Name:      "Seitenstetten Airport",
		City:      "Seitenstetten",
		State:     "Lower-Austria",
		Country:   "AT",
		Elevation: 1050,
		Latitude:  48.0499992371,
		Longitude: 14.6519002914,
		Timezone:  "Europe/Vienna",
	},
	"LOLU": {
		ICAO:      "LOLU",
		Name:      "Gmunden-Laakirchen Airport",
		City:      "Gmunden",
		State:     "Upper-Austria",
		Country:   "AT",
		Elevation: 1670,
		Latitude:  47.9516983032,
		Longitude: 13.8667001724,
		Timezone:  "Europe/Vienna",
	},
	"LOLW": {
		ICAO:      "LOLW",
		Name:      "Wels Airport",
		City:      "Wels",
		State:     "Upper-Austria",
		Country:   "AT",
		Elevation: 1043,
		Latitude:  48.1833000183,
		Longitude: 14.0409002304,
		Timezone:  "Europe/Vienna",
	},
	"LOSM": {
		ICAO:      "LOSM",
		Name:      "Mauterndorf Airport",
		City:      "Mauterndorf",
		State:     "Salzburg",
		Country:   "AT",
		Elevation: 3642,
		Latitude:  47.1324996948,
		Longitude: 13.6969003677,
		Timezone:  "Europe/Vienna",
	},
	"LOWG": {
		ICAO:      "LOWG",
		IATA:      "GRZ",
		Name:      "Graz Airport",
		City:      "Graz",
		State:     "Styria",
		Country:   "AT",
		Elevation: 1115,
		Latitude:  46.9911003113,
		Longitude: 15.4395999908,
		Timezone:  "Europe/Vienna",
	},
	"LOWI": {
		ICAO:      "LOWI",
		IATA:      "INN",
		Name:      "Innsbruck Airport",
		City:      "Innsbruck",
		State:     "Tyrol",
		Country:   "AT",
		Elevation: 1906,
		Latitude:  47.2602005005,
		Longitude: 11.3439998627,
		Timezone:  "Europe/Vienna",
	},
	"LOWK": {
		ICAO:      "LOWK",
		IATA:      "KLU",
		Name:      "Klagenfurt Airport",
		City:      "Klagenfurt am Worthersee",
		State:     "Carinthia",
		Country:   "AT",
		Elevation: 1470,
		Latitude:  46.6425018311,
		Longitude: 14.3376998901,
		Timezone:  "Europe/Vienna",
	},
	"LOWL": {
		ICAO:      "LOWL",
		IATA:      "LNZ",
		Name:      "Linz Airport",
		City:      "Linz",
		State:     "Upper-Austria",
		Country:   "AT",
		Elevation: 978,
		Latitude:  48.2332000732,
		Longitude: 14.1875,
		Timezone:  "Europe/Vienna",
	},
	"LOWS": {
		ICAO:      "LOWS",
		IATA:      "SZG",
		Name:      "Salzburg Airport",
		City:      "Salzburg",
		State:     "Salzburg",
		Country:   "AT",
		Elevation: 1411,
		Latitude:  47.7933006287,
		Longitude: 13.0043001175,
		Timezone:  "Europe/Berlin",
	},
	"LOWW": {
		ICAO:      "LOWW",
		IATA:      "VIE",
		Name:      "Vienna International Airport",
		City:      "Vienna",
		State:     "Lower-Austria",
		Country:   "AT",
		Elevation: 600,
		Latitude:  48.1102981567,
		Longitude: 16.5697002411,
		Timezone:  "Europe/Vienna",
	},
	"LOWZ": {
		ICAO:      "LOWZ",
		Name:      "Zell Am See Airport",
		City:      "Zell Am See",
		State:     "Salzburg",
		Country:   "AT",
		Elevation: 2470,
		Latitude:  47.2933006287,
		Longitude: 12.7882995605,
		Timezone:  "Europe/Vienna",
	},
	"LOXN": {
		ICAO:      "LOXN",
		Name:      "Wiener Neustadt West Airport",
		City:      "Wiener Neustadt",
		State:     "Lower-Austria",
		Country:   "AT",
		Elevation: 935,
		Latitude:  47.8400001526,
		Longitude: 16.2217006683,
		Timezone:  "Europe/Vienna",
	},
	"LOXT": {
		ICAO:      "LOXT",
		Name:      "Brumowski  Air Base",
		City:      "Langenlebarn",
		State:     "Lower-Austria",
		Country:   "AT",
		Elevation: 574,
		Latitude:  48.3166999817,
		Longitude: 16.1166992188,
		Timezone:  "Europe/Vienna",
	},
	"LOXZ": {
		ICAO:      "LOXZ",
		Name:      "Hinterstoisser Air Base",
		City:      "Zeltweg",
		State:     "Styria",
		Country:   "AT",
		Elevation: 2220,
		Latitude:  47.2028007507,
		Longitude: 14.7441997528,
		Timezone:  "Europe/Vienna",
	},
	"LP77": {
		ICAO:      "LP77",
		Name:      "Santa Margarida Airport",
		State:     "Santarem",
		Country:   "PT",
		Elevation: 541,
		Latitude:  39.4024009705,
		Longitude: -8.289059639,
		Timezone:  "Europe/Lisbon",
	},
	"LPAR": {
		ICAO:      "LPAR",
		Name:      "Alverca Airport",
		State:     "Lisbon",
		Country:   "PT",
		Elevation: 11,
		Latitude:  38.8833007813,
		Longitude: -9.0300998688,
		Timezone:  "Europe/Lisbon",
	},
	"LPAV": {
		ICAO:      "LPAV",
		Name:      "Aveiro Airport",
		State:     "Aveiro",
		Country:   "PT",
		Elevation: 26,
		Latitude:  40.6542015076,
		Longitude: -8.7435503006,
		Timezone:  "Europe/Lisbon",
	},
	"LPAZ": {
		ICAO:      "LPAZ",
		IATA:      "SMA",
		Name:      "Santa Maria Airport",
		City:      "Vila do Porto",
		State:     "Azores",
		Country:   "PT",
		Elevation: 308,
		Latitude:  36.9714012146,
		Longitude: -25.1706008911,
		Timezone:  "Atlantic/Azores",
	},
	"LPBG": {
		ICAO:      "LPBG",
		IATA:      "BGC",
		Name:      "Braganca Airport",
		State:     "Braganca",
		Country:   "PT",
		Elevation: 2241,
		Latitude:  41.85779953,
		Longitude: -6.7071299553,
		Timezone:  "Europe/Lisbon",
	},
	"LPBJ": {
		ICAO:      "LPBJ",
		IATA:      "BYJ",
		Name:      "Beja International Airport",
		City:      "Beja",
		State:     "Beja",
		Country:   "PT",
		Elevation: 636,
		Latitude:  38.0788993835,
		Longitude: -7.9324002266,
		Timezone:  "Europe/Lisbon",
	},
	"LPBR": {
		ICAO:      "LPBR",
		IATA:      "BGZ",
		Name:      "Braga Municipal Aerodrome",
		City:      "Braga",
		State:     "Braga",
		Country:   "PT",
		Elevation: 247,
		Latitude:  41.5871009827,
		Longitude: -8.445139885,
		Timezone:  "Europe/Lisbon",
	},
	"LPCH": {
		ICAO:      "LPCH",
		IATA:      "CHV",
		Name:      "Chaves Airport",
		City:      "Chaves",
		State:     "Vila-Real",
		Country:   "PT",
		Latitude:  41.7223243713,
		Longitude: -7.4631500244,
		Timezone:  "Europe/Lisbon",
	},
	"LPCO": {
		ICAO:      "LPCO",
		IATA:      "CBP",
		Name:      "Coimbra Airport",
		State:     "Coimbra",
		Country:   "PT",
		Elevation: 587,
		Latitude:  40.1571998596,
		Longitude: -8.470000267,
		Timezone:  "Europe/Lisbon",
	},
	"LPCR": {
		ICAO:      "LPCR",
		IATA:      "CVU",
		Name:      "Corvo Airport",
		City:      "Corvo",
		State:     "Azores",
		Country:   "PT",
		Latitude:  39.6715011597,
		Longitude: -31.1135997772,
		Timezone:  "Atlantic/Azores",
	},
	"LPCS": {
		ICAO:      "LPCS",
		IATA:      "CAT",
		Name:      "Cascais Airport",
		State:     "Lisbon",
		Country:   "PT",
		Elevation: 325,
		Latitude:  38.7249984741,
		Longitude: -9.3552303314,
		Timezone:  "Europe/Lisbon",
	},
	"LPCV": {
		ICAO:      "LPCV",
		IATA:      "COV",
		Name:      "Covilha Airport",
		State:     "Castelo-Branco",
		Country:   "PT",
		Elevation: 1572,
		Latitude:  40.2648010254,
		Longitude: -7.4799599648,
		Timezone:  "Europe/Lisbon",
	},
	"LPEV": {
		ICAO:      "LPEV",
		Name:      "Evora Airport",
		State:     "Evora",
		Country:   "PT",
		Elevation: 807,
		Latitude:  38.5335006714,
		Longitude: -7.8896398544,
		Timezone:  "Europe/Lisbon",
	},
	"LPFL": {
		ICAO:      "LPFL",
		IATA:      "FLW",
		Name:      "Flores Airport",
		City:      "Santa Cruz das Flores",
		State:     "Azores",
		Country:   "PT",
		Elevation: 112,
		Latitude:  39.4552993774,
		Longitude: -31.1313991547,
		Timezone:  "Atlantic/Azores",
	},
	"LPFR": {
		ICAO:      "LPFR",
		IATA:      "FAO",
		Name:      "Faro Airport",
		City:      "Faro",
		State:     "Faro",
		Country:   "PT",
		Elevation: 24,
		Latitude:  37.0144004822,
		Longitude: -7.9659099579,
		Timezone:  "Europe/Lisbon",
	},
	"LPGR": {
		ICAO:      "LPGR",
		IATA:      "GRW",
		Name:      "Graciosa Airport",
		City:      "Santa Cruz da Graciosa",
		State:     "Azores",
		Country:   "PT",
		Elevation: 86,
		Latitude:  39.0922012329,
		Longitude: -28.029800415,
		Timezone:  "Atlantic/Azores",
	},
	"LPHR": {
		ICAO:      "LPHR",
		IATA:      "HOR",
		Name:      "Horta Airport",
		City:      "Horta",
		State:     "Azores",
		Country:   "PT",
		Elevation: 118,
		Latitude:  38.5199012756,
		Longitude: -28.7159004211,
		Timezone:  "Atlantic/Azores",
	},
	"LPIN": {
		ICAO:      "LPIN",
		Name:      "Espinho Airport",
		State:     "Porto",
		Country:   "PT",
		Elevation: 10,
		Latitude:  40.9742012024,
		Longitude: -8.6444396973,
		Timezone:  "Europe/Lisbon",
	},
	"LPJF": {
		ICAO:      "LPJF",
		Name:      "Leiria Gandara Airport",
		City:      "Leiria",
		State:     "Leiria",
		Country:   "PT",
		Elevation: 151,
		Latitude:  39.7785987854,
		Longitude: -8.8197202682,
		Timezone:  "Europe/Lisbon",
	},
	"LPJO": {
		ICAO:      "LPJO",
		Name:      "Alijo Airport",
		City:      "Alijo",
		State:     "Vila-Real",
		Country:   "PT",
		Elevation: 2562,
		Latitude:  41.3252983093,
		Longitude: -7.4461097717,
		Timezone:  "Europe/Lisbon",
	},
	"LPLA": {
		ICAO:      "LPLA",
		IATA:      "TER",
		Name:      "Lajes Field",
		City:      "Lajes",
		State:     "Azores",
		Country:   "PT",
		Elevation: 180,
		Latitude:  38.7617988586,
		Longitude: -27.0907993317,
		Timezone:  "Atlantic/Azores",
	},
	"LPLZ": {
		ICAO:      "LPLZ",
		Name:      "Lousa Private Airport",
		City:      "Lousa",
		State:     "Coimbra",
		Country:   "PT",
		Elevation: 669,
		Latitude:  40.1342010498,
		Longitude: -8.2325000763,
		Timezone:  "Europe/Lisbon",
	},
	"LPMA": {
		ICAO:      "LPMA",
		IATA:      "FNC",
		Name:      "Madeira Airport",
		City:      "Funchal",
		State:     "Madeira",
		Country:   "PT",
		Elevation: 192,
		Latitude:  32.6978988647,
		Longitude: -16.7744998932,
		Timezone:  "Atlantic/Madeira",
	},
	"LPMF": {
		ICAO:      "LPMF",
		Name:      "Monfortinho Airport",
		City:      "Monfortinho",
		State:     "Castelo-Branco",
		Country:   "PT",
		Latitude:  39.976398468,
		Longitude: -6.9066238403,
		Timezone:  "Europe/Lisbon",
	},
	"LPMI": {
		ICAO:      "LPMI",
		Name:      "Mirandela Airport",
		City:      "Mirandela",
		State:     "Braganca",
		Country:   "PT",
		Latitude:  41.468006134,
		Longitude: -7.2262573242,
		Timezone:  "Europe/Lisbon",
	},
	"LPMO": {
		ICAO:      "LPMO",
		Name:      "Montargil Airport",
		City:      "Montargil",
		State:     "Evora",
		Country:   "PT",
		Elevation: 341,
		Latitude:  38.9958000183,
		Longitude: -8.14083004,
		Timezone:  "Europe/Lisbon",
	},
	"LPMR": {
		ICAO:      "LPMR",
		Name:      "Monte Real Airport",
		State:     "Leiria",
		Country:   "PT",
		Elevation: 187,
		Latitude:  39.831199646,
		Longitude: -8.887260437,
		Timezone:  "Europe/Lisbon",
	},
	"LPMT": {
		ICAO:      "LPMT",
		Name:      "Montijo Airport",
		City:      "Montijo",
		State:     "Setubal",
		Country:   "PT",
		Elevation: 46,
		Latitude:  38.7038993835,
		Longitude: -9.0359201431,
		Timezone:  "Europe/Lisbon",
	},
	"LPMU": {
		ICAO:      "LPMU",
		Name:      "Mogadouro Airport",
		City:      "Mogadouro",
		State:     "Braganca",
		Country:   "PT",
		Elevation: 2349,
		Latitude:  41.396900177,
		Longitude: -6.6816701889,
		Timezone:  "Europe/Lisbon",
	},
	"LPOT": {
		ICAO:      "LPOT",
		Name:      "Ota Airport",
		City:      "Ota",
		State:     "Lisbon",
		Country:   "PT",
		Elevation: 140,
		Latitude:  39.0875015259,
		Longitude: -8.9627799988,
		Timezone:  "Europe/Lisbon",
	},
	"LPOV": {
		ICAO:      "LPOV",
		Name:      "Ovar Airport",
		State:     "Aveiro",
		Country:   "PT",
		Elevation: 56,
		Latitude:  40.9159011841,
		Longitude: -8.6459197998,
		Timezone:  "Europe/Lisbon",
	},
	"LPPD": {
		ICAO:      "LPPD",
		IATA:      "PDL",
		Name:      "Joao Paulo II Airport",
		City:      "Ponta Delgada",
		State:     "Azores",
		Country:   "PT",
		Elevation: 259,
		Latitude:  37.7411994934,
		Longitude: -25.6979007721,
		Timezone:  "Atlantic/Azores",
	},
	"LPPI": {
		ICAO:      "LPPI",
		IATA:      "PIX",
		Name:      "Pico Airport",
		City:      "Pico Island",
		State:     "Azores",
		Country:   "PT",
		Elevation: 109,
		Latitude:  38.5542984009,
		Longitude: -28.4412994385,
		Timezone:  "Atlantic/Azores",
	},
	"LPPM": {
		ICAO:      "LPPM",
		IATA:      "PRM",
		Name:      "Portimao Airport",
		State:     "Faro",
		Country:   "PT",
		Elevation: 5,
		Latitude:  37.1492996216,
		Longitude: -8.5839595795,
		Timezone:  "Europe/Lisbon",
	},
	"LPPR": {
		ICAO:      "LPPR",
		IATA:      "OPO",
		Name:      "Francisco de Sa Carneiro Airport",
		City:      "Porto",
		State:     "Porto",
		Country:   "PT",
		Elevation: 228,
		Latitude:  41.2481002808,
		Longitude: -8.6813898087,
		Timezone:  "Europe/Lisbon",
	},
	"LPPS": {
		ICAO:      "LPPS",
		IATA:      "PXO",
		Name:      "Porto Santo Airport",
		City:      "Vila Baleira",
		State:     "Madeira",
		Country:   "PT",
		Elevation: 341,
		Latitude:  33.0733985901,
		Longitude: -16.3500003815,
		Timezone:  "Atlantic/Madeira",
	},
	"LPPT": {
		ICAO:      "LPPT",
		IATA:      "LIS",
		Name:      "Lisbon Portela Airport",
		City:      "Lisbon",
		State:     "Lisbon",
		Country:   "PT",
		Elevation: 374,
		Latitude:  38.7812995911,
		Longitude: -9.1359195709,
		Timezone:  "Europe/Lisbon",
	},
	"LPPV": {
		ICAO:      "LPPV",
		Name:      "Praia Verde 23M Airport",
		State:     "Faro",
		Country:   "PT",
		Elevation: 10,
		Latitude:  37.1925010681,
		Longitude: -7.4730601311,
		Timezone:  "Europe/Lisbon",
	},
	"LPSC": {
		ICAO:      "LPSC",
		Name:      "Santa Cruz Airport",
		State:     "Lisbon",
		Country:   "PT",
		Latitude:  39.1246986389,
		Longitude: -9.3811302185,
		Timezone:  "Europe/Lisbon",
	},
	"LPSI": {
		ICAO:      "LPSI",
		IATA:      "SIE",
		Name:      "Sines Airport",
		State:     "Setubal",
		Country:   "PT",
		Latitude:  37.9416923523,
		Longitude: -8.8173007965,
		Timezone:  "Europe/Lisbon",
	},
	"LPSJ": {
		ICAO:      "LPSJ",
		IATA:      "SJZ",
		Name:      "Sao Jorge Airport",
		City:      "Velas",
		State:     "Azores",
		Country:   "PT",
		Elevation: 311,
		Latitude:  38.6655006409,
		Longitude: -28.1758003235,
		Timezone:  "Atlantic/Azores",
	},
	"LPSO": {
		ICAO:      "LPSO",
		Name:      "Ponte De Sor",
		City:      "Ponte De Sor",
		State:     "Portalegre",
		Country:   "PT",
		Elevation: 390,
		Latitude:  39.208089,
		Longitude: -8.057534,
		Timezone:  "Europe/Lisbon",
	},
	"LPSR": {
		ICAO:      "LPSR",
		Name:      "Santarem Airport",
		City:      "Santarem",
		State:     "Santarem",
		Country:   "PT",
		Latitude:  39.2094993591,
		Longitude: -8.6883001328,
		Timezone:  "Europe/Lisbon",
	},
	"LPST": {
		ICAO:      "LPST",
		Name:      "Sintra Airport",
		State:     "Lisbon",
		Country:   "PT",
		Elevation: 440,
		Latitude:  38.8311004639,
		Longitude: -9.3395500183,
		Timezone:  "Europe/Lisbon",
	},
	"LPTN": {
		ICAO:      "LPTN",
		Name:      "Tancos Airport",
		State:     "Santarem",
		Country:   "PT",
		Elevation: 266,
		Latitude:  39.4751014709,
		Longitude: -8.3645801544,
		Timezone:  "Europe/Lisbon",
	},
	"LPVL": {
		ICAO:      "LPVL",
		Name:      "Vilar Da Luz Airport",
		City:      "Maia",
		State:     "Porto",
		Country:   "PT",
		Latitude:  41.2772903442,
		Longitude: -8.5162067413,
		Timezone:  "Europe/Lisbon",
	},
	"LPVR": {
		ICAO:      "LPVR",
		IATA:      "VRL",
		Name:      "Vila Real Airport",
		State:     "Vila-Real",
		Country:   "PT",
		Elevation: 1805,
		Latitude:  41.2742996216,
		Longitude: -7.7204699516,
		Timezone:  "Europe/Lisbon",
	},
	"LPVZ": {
		ICAO:      "LPVZ",
		IATA:      "VSE",
		Name:      "Viseu Airport",
		State:     "Viseu",
		Country:   "PT",
		Elevation: 2060,
		Latitude:  40.7254981995,
		Longitude: -7.8889899254,
		Timezone:  "Europe/Lisbon",
	},
	"LQBI": {
		ICAO:      "LQBI",
		Name:      "Bihac Golubic Airport",
		City:      "Bihac",
		State:     "Federation-of-B&H",
		Country:   "BA",
		Elevation: 761,
		Latitude:  44.7959367906,
		Longitude: 15.9035682678,
		Timezone:  "Europe/Sarajevo",
	},
	"LQBK": {
		ICAO:      "LQBK",
		IATA:      "BNX",
		Name:      "Banja Luka International Airport",
		City:      "Banja Luka",
		State:     "Srspka",
		Country:   "BA",
		Elevation: 400,
		Latitude:  44.9413986206,
		Longitude: 17.2975006104,
		Timezone:  "Europe/Sarajevo",
	},
	"LQBZ": {
		ICAO:      "LQBZ",
		Name:      "Zaluzani Airport",
		City:      "Banja Luka",
		State:     "Srspka",
		Country:   "BA",
		Elevation: 541,
		Latitude:  44.7913856506,
		Longitude: 17.2146568298,
		Timezone:  "Europe/Sarajevo",
	},
	"LQJL": {
		ICAO:      "LQJL",
		Name:      "Tuzla Jegin Lug Sport Airfield",
		City:      "Tuzla",
		State:     "Federation-of-B&H",
		Country:   "BA",
		Elevation: 869,
		Latitude:  44.4578018188,
		Longitude: 18.8083000183,
		Timezone:  "Europe/Sarajevo",
	},
	"LQLV": {
		ICAO:      "LQLV",
		Name:      "Livno Brda Bosni Airport",
		City:      "Livno",
		State:     "Federation-of-B&H",
		Country:   "BA",
		Elevation: 2349,
		Latitude:  43.7930984497,
		Longitude: 16.9069004059,
		Timezone:  "Europe/Sarajevo",
	},
	"LQMO": {
		ICAO:      "LQMO",
		IATA:      "OMO",
		Name:      "Mostar International Airport",
		City:      "Mostar",
		State:     "Federation-of-B&H",
		Country:   "BA",
		Elevation: 156,
		Latitude:  43.2829017639,
		Longitude: 17.8458995819,
		Timezone:  "Europe/Sarajevo",
	},
	"LQPD": {
		ICAO:      "LQPD",
		Name:      "Prijedor Urije Airport",
		City:      "Prijedor",
		State:     "Srspka",
		Country:   "BA",
		Elevation: 591,
		Latitude:  44.9880981445,
		Longitude: 16.7357997894,
		Timezone:  "Europe/Sarajevo",
	},
	"LQSA": {
		ICAO:      "LQSA",
		IATA:      "SJJ",
		Name:      "Sarajevo International Airport",
		City:      "Sarajevo",
		State:     "Federation-of-B&H",
		Country:   "BA",
		Elevation: 1708,
		Latitude:  43.8246002197,
		Longitude: 18.3314990997,
		Timezone:  "Europe/Sarajevo",
	},
	"LQTZ": {
		ICAO:      "LQTZ",
		IATA:      "TZL",
		Name:      "Tuzla International Airport",
		City:      "Tuzla",
		State:     "Federation-of-B&H",
		Country:   "BA",
		Elevation: 784,
		Latitude:  44.4586982727,
		Longitude: 18.7248001099,
		Timezone:  "Europe/Sarajevo",
	},
	"LQVI": {
		ICAO:      "LQVI",
		Name:      "Visoko Sport Airfield",
		City:      "Visoko",
		State:     "Federation-of-B&H",
		Country:   "BA",
		Elevation: 1470,
		Latitude:  44.0249769612,
		Longitude: 18.0970573425,
		Timezone:  "Europe/Sarajevo",
	},
	"LR79": {
		ICAO:      "LR79",
		Name:      "Ianca Air Base",
		City:      "Ianca",
		State:     "Braila",
		Country:   "RO",
		Elevation: 131,
		Latitude:  45.1586990356,
		Longitude: 27.4309005737,
		Timezone:  "Europe/Bucharest",
	},
	"LR80": {
		ICAO:      "LR80",
		Name:      "Fetesti Air Base",
		City:      "Fetesti",
		State:     "Calaraşi",
		Country:   "RO",
		Elevation: 180,
		Latitude:  44.3922996521,
		Longitude: 27.7266998291,
		Timezone:  "Europe/Bucharest",
	},
	"LR82": {
		ICAO:      "LR82",
		Name:      "Boboc Air Base",
		City:      "Boboc",
		State:     "Buzau",
		Country:   "RO",
		Elevation: 344,
		Latitude:  45.2164001465,
		Longitude: 26.9785995483,
		Timezone:  "Europe/Bucharest",
	},
	"LRAR": {
		ICAO:      "LRAR",
		IATA:      "ARW",
		Name:      "Arad International Airport",
		City:      "Arad",
		State:     "Arad",
		Country:   "RO",
		Elevation: 352,
		Latitude:  46.1766014099,
		Longitude: 21.2619991302,
		Timezone:  "Europe/Bucharest",
	},
	"LRBC": {
		ICAO:      "LRBC",
		IATA:      "BCM",
		Name:      "Bacau Airport",
		City:      "Bacau",
		State:     "Bacau",
		Country:   "RO",
		Elevation: 607,
		Latitude:  46.521900177,
		Longitude: 26.9102993011,
		Timezone:  "Europe/Bucharest",
	},
	"LRBM": {
		ICAO:      "LRBM",
		IATA:      "BAY",
		Name:      "Tautii Magheraus Airport",
		City:      "Baia Mare",
		State:     "Maramureş",
		Country:   "RO",
		Elevation: 605,
		Latitude:  47.6584014893,
		Longitude: 23.4699993134,
		Timezone:  "Europe/Bucharest",
	},
	"LRBS": {
		ICAO:      "LRBS",
		IATA:      "BBU",
		Name:      "Baneasa International Airport",
		City:      "Bucharest",
		State:     "Bucureşti",
		Country:   "RO",
		Elevation: 297,
		Latitude:  44.503200531,
		Longitude: 26.1021003723,
		Timezone:  "Europe/Bucharest",
	},
	"LRCK": {
		ICAO:      "LRCK",
		IATA:      "CND",
		Name:      "Mihail Kogalniceanu International Airport",
		City:      "Constanta",
		State:     "Constanța",
		Country:   "RO",
		Elevation: 353,
		Latitude:  44.3622016907,
		Longitude: 28.4883003235,
		Timezone:  "Europe/Bucharest",
	},
	"LRCL": {
		ICAO:      "LRCL",
		IATA:      "CLJ",
		Name:      "Cluj-Napoca International Airport",
		City:      "Cluj-Napoca",
		State:     "Cluj",
		Country:   "RO",
		Elevation: 1036,
		Latitude:  46.7851982117,
		Longitude: 23.6861991882,
		Timezone:  "Europe/Bucharest",
	},
	"LRCS": {
		ICAO:      "LRCS",
		IATA:      "CSB",
		Name:      "Caransebes Airport",
		City:      "Caransebes",
		State:     "Caraş-Severin",
		Country:   "RO",
		Elevation: 866,
		Latitude:  45.4199981689,
		Longitude: 22.2532997131,
		Timezone:  "Europe/Bucharest",
	},
	"LRCT": {
		ICAO:      "LRCT",
		Name:      "Campia Turzii Air Base",
		City:      "Campia Turzii",
		State:     "Cluj",
		Country:   "RO",
		Elevation: 1083,
		Latitude:  46.5023002625,
		Longitude: 23.8859004974,
		Timezone:  "Europe/Bucharest",
	},
	"LRCV": {
		ICAO:      "LRCV",
		IATA:      "CRA",
		Name:      "Craiova Airport",
		City:      "Craiova",
		State:     "Dolj",
		Country:   "RO",
		Elevation: 626,
		Latitude:  44.3180999756,
		Longitude: 23.8885993958,
		Timezone:  "Europe/Bucharest",
	},
	"LRIA": {
		ICAO:      "LRIA",
		IATA:      "IAS",
		Name:      "Iasi Airport",
		City:      "Iasi",
		State:     "Iaşi",
		Country:   "RO",
		Elevation: 397,
		Latitude:  47.1785011292,
		Longitude: 27.6205997467,
		Timezone:  "Europe/Bucharest",
	},
	"LROD": {
		ICAO:      "LROD",
		IATA:      "OMR",
		Name:      "Oradea International Airport",
		City:      "Oradea",
		State:     "Bihor",
		Country:   "RO",
		Elevation: 465,
		Latitude:  47.0252990723,
		Longitude: 21.9025001526,
		Timezone:  "Europe/Bucharest",
	},
	"LROP": {
		ICAO:      "LROP",
		IATA:      "OTP",
		Name:      "Henri Coanda International Airport",
		City:      "Bucharest",
		State:     "Ilfov",
		Country:   "RO",
		Elevation: 314,
		Latitude:  44.5722007751,
		Longitude: 26.1021995544,
		Timezone:  "Europe/Bucharest",
	},
	"LRSB": {
		ICAO:      "LRSB",
		IATA:      "SBZ",
		Name:      "Sibiu International Airport",
		City:      "Sibiu",
		State:     "Sibiu",
		Country:   "RO",
		Elevation: 1496,
		Latitude:  45.7855987549,
		Longitude: 24.091299057,
		Timezone:  "Europe/Bucharest",
	},
	"LRSM": {
		ICAO:      "LRSM",
		IATA:      "SUJ",
		Name:      "Satu Mare Airport",
		City:      "Satu Mare",
		State:     "Satu-Mare",
		Country:   "RO",
		Elevation: 405,
		Latitude:  47.7033004761,
		Longitude: 22.8857002258,
		Timezone:  "Europe/Bucharest",
	},
	"LRSV": {
		ICAO:      "LRSV",
		IATA:      "SCV",
		Name:      "Suceava Stefan cel Mare Airport",
		City:      "Suceava",
		State:     "Suceava",
		Country:   "RO",
		Elevation: 1375,
		Latitude:  47.6875,
		Longitude: 26.3540992737,
		Timezone:  "Europe/Bucharest",
	},
	"LRTC": {
		ICAO:      "LRTC",
		IATA:      "TCE",
		Name:      "Tulcea Airport",
		City:      "Tulcea",
		State:     "Tulcea",
		Country:   "RO",
		Elevation: 200,
		Latitude:  45.0625,
		Longitude: 28.7143001556,
		Timezone:  "Europe/Bucharest",
	},
	"LRTM": {
		ICAO:      "LRTM",
		IATA:      "TGM",
		Name:      "Transilvania Targu Mures International Airport",
		City:      "Targu Mures",
		State:     "Mureş",
		Country:   "RO",
		Elevation: 963,
		Latitude:  46.4677009583,
		Longitude: 24.4125003815,
		Timezone:  "Europe/Bucharest",
	},
	"LRTR": {
		ICAO:      "LRTR",
		IATA:      "TSR",
		Name:      "Timisoara Traian Vuia Airport",
		City:      "Timisoara",
		State:     "Timiş",
		Country:   "RO",
		Elevation: 348,
		Latitude:  45.8098983765,
		Longitude: 21.3379001617,
		Timezone:  "Europe/Bucharest",
	},
	"LRTZ": {
		ICAO:      "LRTZ",
		Name:      "Tuzla Romania Airport",
		City:      "Tuzla",
		State:     "Constanța",
		Country:   "RO",
		Elevation: 161,
		Latitude:  43.9841995239,
		Longitude: 28.6096992493,
		Timezone:  "Europe/Bucharest",
	},
	"LS00": {
		ICAO:      "LS00",
		Name:      "Whitehead Ultralightport",
		City:      "Zachary",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 105,
		Latitude:  30.6755008698,
		Longitude: -91.1143035889,
		Timezone:  "America/Chicago",
	},
	"LS03": {
		ICAO:      "LS03",
		Name:      "Stephen's Flying Service Airport",
		City:      "Winnsboro",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 89,
		Latitude:  32.2933006287,
		Longitude: -91.5932998657,
		Timezone:  "America/Chicago",
	},
	"LS04": {
		ICAO:      "LS04",
		Name:      "Bobby Jones Flying Service Inc Airport",
		City:      "Mangham",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 69,
		Latitude:  32.3166999817,
		Longitude: -91.8395004272,
		Timezone:  "America/Chicago",
	},
	"LS09": {
		ICAO:      "LS09",
		Name:      "Sylvester's Airport",
		City:      "Ville Platte",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 70,
		Latitude:  30.6958007813,
		Longitude: -92.3175964355,
		Timezone:  "America/Chicago",
	},
	"LS10": {
		ICAO:      "LS10",
		Name:      "Reynolds Airport",
		City:      "Westlake",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 20,
		Latitude:  30.2721004486,
		Longitude: -93.3117980957,
		Timezone:  "America/Chicago",
	},
	"LS11": {
		ICAO:      "LS11",
		Name:      "Liddieville Airport",
		City:      "Winnsboro",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 73,
		Latitude:  32.1534996033,
		Longitude: -91.8436965942,
		Timezone:  "America/Chicago",
	},
	"LS14": {
		ICAO:      "LS14",
		Name:      "Lyon Airport",
		City:      "Thornwell",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 6,
		Latitude:  30.0709991455,
		Longitude: -92.8251037598,
		Timezone:  "America/Chicago",
	},
	"LS16": {
		ICAO:      "LS16",
		Name:      "Circle G Airport",
		City:      "Jigger",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 75,
		Latitude:  32.0112991333,
		Longitude: -91.7550964355,
		Timezone:  "America/Chicago",
	},
	"LS18": {
		ICAO:      "LS18",
		Name:      "Ag Aviation Airport",
		City:      "Welsh",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 21,
		Latitude:  30.2238998413,
		Longitude: -92.7786026001,
		Timezone:  "America/Chicago",
	},
	"LS21": {
		ICAO:      "LS21",
		Name:      "Southland Strip Ultralightport",
		City:      "Houma",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 10,
		Latitude:  29.6704998016,
		Longitude: -90.7705993652,
		Timezone:  "America/Chicago",
	},
	"LS25": {
		ICAO:      "LS25",
		Name:      "Annison Private Airport",
		City:      "Slaughter",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 120,
		Latitude:  30.7045993805,
		Longitude: -91.0606994629,
		Timezone:  "America/Chicago",
	},
	"LS34": {
		ICAO:      "LS34",
		Name:      "Bordelon Airpark",
		City:      "Breaux Bridge",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 22,
		Latitude:  30.3066997528,
		Longitude: -91.8764038086,
		Timezone:  "America/Chicago",
	},
	"LS35": {
		ICAO:      "LS35",
		Name:      "Nauga Field",
		City:      "St Francisville",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 38,
		Latitude:  30.7625007629,
		Longitude: -91.2772979736,
		Timezone:  "America/Chicago",
	},
	"LS36": {
		ICAO:      "LS36",
		Name:      "Light Plane Flyers Airfield",
		City:      "Iowa",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 25,
		Latitude:  30.2502002716,
		Longitude: -92.9720001221,
		Timezone:  "America/Chicago",
	},
	"LS38": {
		ICAO:      "LS38",
		Name:      "Richard's Airport",
		City:      "New Iberia",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 15,
		Latitude:  30.0130996704,
		Longitude: -91.8692016602,
		Timezone:  "America/Chicago",
	},
	"LS39": {
		ICAO:      "LS39",
		Name:      "Country Bend Airport",
		City:      "Watson",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 49,
		Latitude:  30.5893993378,
		Longitude: -90.9845962524,
		Timezone:  "America/Chicago",
	},
	"LS40": {
		ICAO:      "LS40",
		Name:      "St Charles Airport",
		City:      "Ama",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 13,
		Latitude:  29.9519004822,
		Longitude: -90.2861022949,
		Timezone:  "America/Chicago",
	},
	"LS41": {
		ICAO:      "LS41",
		Name:      "Tater Patch Ultralightport",
		City:      "Blanchard",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 240,
		Latitude:  32.5671005249,
		Longitude: -93.8788986206,
		Timezone:  "America/Chicago",
	},
	"LS43": {
		ICAO:      "LS43",
		Name:      "Southland Strip Nr 2 Ultralightport",
		City:      "Gray",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 10,
		Latitude:  29.6718997955,
		Longitude: -90.7676010132,
		Timezone:  "America/Chicago",
	},
	"LS44": {
		ICAO:      "LS44",
		Name:      "Ken Guidry No 3 Airport",
		City:      "Kaplan",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 15,
		Latitude:  30.0898990631,
		Longitude: -92.3436965942,
		Timezone:  "America/Chicago",
	},
	"LS46": {
		ICAO:      "LS46",
		Name:      "Blount Airport",
		City:      "Monterey",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 50,
		Latitude:  31.4584999084,
		Longitude: -91.8228988647,
		Timezone:  "America/Chicago",
	},
	"LS49": {
		ICAO:      "LS49",
		Name:      "Simpson Airport",
		City:      "Plaquemine",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 13,
		Latitude:  30.3027000427,
		Longitude: -91.2714996338,
		Timezone:  "America/Chicago",
	},
	"LS54": {
		ICAO:      "LS54",
		Name:      "Brian's Ultralightport",
		City:      "Zachary",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 65,
		Latitude:  30.6919002533,
		Longitude: -91.1875991821,
		Timezone:  "America/Chicago",
	},
	"LS59": {
		ICAO:      "LS59",
		Name:      "La Petite Airdrome Ultralightport",
		City:      "Opelousas",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 63,
		Latitude:  30.6294002533,
		Longitude: -92.1878967285,
		Timezone:  "America/Chicago",
	},
	"LS68": {
		ICAO:      "LS68",
		Name:      "Ken Guidry Nr 1 Airport",
		City:      "Abbeville",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 10,
		Latitude:  29.9347000122,
		Longitude: -92.2135009766,
		Timezone:  "America/Chicago",
	},
	"LS69": {
		ICAO:      "LS69",
		Name:      "L J Earnest Airport",
		City:      "Benton",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 165,
		Latitude:  32.6435012817,
		Longitude: -93.662399292,
		Timezone:  "America/Chicago",
	},
	"LS72": {
		ICAO:      "LS72",
		Name:      "Aerolite Aero Park Ultralightport",
		City:      "Welsh",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 13,
		Latitude:  30.202999115,
		Longitude: -92.8121032715,
		Timezone:  "America/Chicago",
	},
	"LS76": {
		ICAO:      "LS76",
		Name:      "Hickham Field",
		City:      "Covington",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 25,
		Latitude:  30.4626998901,
		Longitude: -90.1700973511,
		Timezone:  "America/Chicago",
	},
	"LS77": {
		ICAO:      "LS77",
		Name:      "A & P Airpark",
		City:      "Slaughter",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 140,
		Latitude:  30.7271003723,
		Longitude: -91.1485977173,
		Timezone:  "America/Chicago",
	},
	"LS83": {
		ICAO:      "LS83",
		Name:      "Kifer Airport",
		City:      "Newellton",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 77,
		Latitude:  32.0626564026,
		Longitude: -91.2542800903,
		Timezone:  "America/Chicago",
	},
	"LS86": {
		ICAO:      "LS86",
		Name:      "Le Blanc Field",
		City:      "Hammond",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 158,
		Latitude:  30.6819000244,
		Longitude: -90.4192962646,
		Timezone:  "America/Chicago",
	},
	"LS89": {
		ICAO:      "LS89",
		Name:      "Sara Field",
		City:      "St Francisville",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 333,
		Latitude:  30.9466991425,
		Longitude: -91.4749984741,
		Timezone:  "America/Chicago",
	},
	"LS90": {
		ICAO:      "LS90",
		Name:      "Caddo Detention Center Airport",
		City:      "Shreveport",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 294,
		Latitude:  32.2624015808,
		Longitude: -93.9356994629,
		Timezone:  "America/Chicago",
	},
	"LS91": {
		ICAO:      "LS91",
		Name:      "Tim Bullard Memorial Airport",
		City:      "Arnaudville",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 32,
		Latitude:  30.4172000885,
		Longitude: -91.9736022949,
		Timezone:  "America/Chicago",
	},
	"LS92": {
		ICAO:      "LS92",
		Name:      "Koenig Airpark",
		City:      "Robert",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 100,
		Latitude:  30.5974006653,
		Longitude: -90.31590271,
		Timezone:  "America/Chicago",
	},
	"LS93": {
		ICAO:      "LS93",
		Name:      "Greenwood Plantation Ultralightport",
		City:      "Gibson",
		State:     "Louisiana",
		Country:   "US",
		Elevation: 6,
		Latitude:  29.6107997894,
		Longitude: -90.898399353,
		Timezone:  "America/Chicago",
	},
	"LSGB": {
		ICAO:      "LSGB",
		Name:      "Bex Airport",
		City:      "Bex",
		State:     "Vaud",
		Country:   "CH",
		Elevation: 1312,
		Latitude:  46.2583007813,
		Longitude: 6.9863901138,
		Timezone:  "Europe/Zurich",
	},
	"LSGC": {
		ICAO:      "LSGC",
		Name:      "Les Eplatures Airport",
		City:      "La Chaux-de-Fonds",
		State:     "Neuchatel",
		Country:   "CH",
		Elevation: 3368,
		Latitude:  47.0839004517,
		Longitude: 6.792840004,
		Timezone:  "Europe/Zurich",
	},
	"LSGE": {
		ICAO:      "LSGE",
		Name:      "Ecuvillens Airport",
		State:     "Fribourg",
		Country:   "CH",
		Elevation: 2293,
		Latitude:  46.7550010681,
		Longitude: 7.0761098862,
		Timezone:  "Europe/Zurich",
	},
	"LSGG": {
		ICAO:      "LSGG",
		IATA:      "GVA",
		Name:      "Geneva Cointrin International Airport",
		City:      "Geneva",
		State:     "Geneva",
		Country:   "CH",
		Elevation: 1411,
		Latitude:  46.2380981445,
		Longitude: 6.1089501381,
		Timezone:  "Europe/Paris",
	},
	"LSGK": {
		ICAO:      "LSGK",
		Name:      "Saanen Airport",
		City:      "Saanen",
		State:     "Bern",
		Country:   "CH",
		Elevation: 3307,
		Latitude:  46.4874992371,
		Longitude: 7.2508301735,
		Timezone:  "Europe/Zurich",
	},
	"LSGL": {
		ICAO:      "LSGL",
		IATA:      "QLS",
		Name:      "Lausanne-la Blecherette Airport",
		City:      "Lausanne",
		State:     "Vaud",
		Country:   "CH",
		Elevation: 2041,
		Latitude:  46.54529953,
		Longitude: 6.6166701317,
		Timezone:  "Europe/Zurich",
	},
	"LSGN": {
		ICAO:      "LSGN",
		IATA:      "QNC",
		Name:      "Neuchatel Airport",
		State:     "Neuchatel",
		Country:   "CH",
		Elevation: 1427,
		Latitude:  46.9575004578,
		Longitude: 6.8647198677,
		Timezone:  "Europe/Zurich",
	},
	"LSGP": {
		ICAO:      "LSGP",
		Name:      "La Cote Airport",
		City:      "La Cote",
		State:     "Vaud",
		Country:   "CH",
		Elevation: 1352,
		Latitude:  46.4063987732,
		Longitude: 6.2580599785,
		Timezone:  "Europe/Zurich",
	},
	"LSGR": {
		ICAO:      "LSGR",
		Name:      "Reichenbach Air Base",
		State:     "Bern",
		Country:   "CH",
		Elevation: 2385,
		Latitude:  46.6136016846,
		Longitude: 7.6777801514,
		Timezone:  "Europe/Zurich",
	},
	"LSGS": {
		ICAO:      "LSGS",
		IATA:      "SIR",
		Name:      "Sion Airport",
		City:      "Sion",
		State:     "Valais",
		Country:   "CH",
		Elevation: 1585,
		Latitude:  46.2196006775,
		Longitude: 7.3267598152,
		Timezone:  "Europe/Zurich",
	},
	"LSGT": {
		ICAO:      "LSGT",
		Name:      "Gruyeres Airport",
		State:     "Fribourg",
		Country:   "CH",
		Elevation: 2257,
		Latitude:  46.5942001343,
		Longitude: 7.0944399834,
		Timezone:  "Europe/Zurich",
	},
	"LSGY": {
		ICAO:      "LSGY",
		Name:      "Yverdon-les-Bains Airport",
		City:      "Yverdon-les-Bains",
		State:     "Vaud",
		Country:   "CH",
		Elevation: 1421,
		Latitude:  46.7619018555,
		Longitude: 6.6133298874,
		Timezone:  "Europe/Zurich",
	},
	"LSMA": {
		ICAO:      "LSMA",
		Name:      "Alpnach Air Base",
		City:      "Alpnach",
		State:     "Obwalden",
		Country:   "CH",
		Elevation: 1460,
		Latitude:  46.943901062,
		Longitude: 8.2841701508,
		Timezone:  "Europe/Zurich",
	},
	"LSMD": {
		ICAO:      "LSMD",
		Name:      "Dubendorf Airport",
		City:      "Zurich",
		State:     "Zurich",
		Country:   "CH",
		Elevation: 1470,
		Latitude:  47.398601532,
		Longitude: 8.648229599,
		Timezone:  "Europe/Zurich",
	},
	"LSME": {
		ICAO:      "LSME",
		IATA:      "EML",
		Name:      "Emmen Airport",
		State:     "Lucerne",
		Country:   "CH",
		Elevation: 1400,
		Latitude:  47.0923995972,
		Longitude: 8.3051204681,
		Timezone:  "Europe/Zurich",
	},
	"LSMF": {
		ICAO:      "LSMF",
		Name:      "Mollis Airport",
		State:     "Glarus",
		Country:   "CH",
		Elevation: 1485,
		Latitude:  47.0788993835,
		Longitude: 9.0648298264,
		Timezone:  "Europe/Zurich",
	},
	"LSMI": {
		ICAO:      "LSMI",
		IATA:      "ZIN",
		Name:      "Interlaken Air Base",
		State:     "Bern",
		Country:   "CH",
		Latitude:  46.6766014099,
		Longitude: 7.8790798187,
		Timezone:  "Europe/Zurich",
	},
	"LSMJ": {
		ICAO:      "LSMJ",
		Name:      "Turtman Air Base",
		City:      "Turtman",
		State:     "Valais",
		Country:   "CH",
		Latitude:  46.3026008606,
		Longitude: 7.7139501572,
		Timezone:  "Europe/Zurich",
	},
	"LSML": {
		ICAO:      "LSML",
		Name:      "Lodrino Air Base",
		City:      "Lodrino",
		State:     "Ticino",
		Country:   "CH",
		Latitude:  46.2958984375,
		Longitude: 8.9921302795,
		Timezone:  "Europe/Zurich",
	},
	"LSMM": {
		ICAO:      "LSMM",
		Name:      "Meiringen Airport",
		State:     "Bern",
		Country:   "CH",
		Elevation: 1895,
		Latitude:  46.7433013916,
		Longitude: 8.1099996567,
		Timezone:  "Europe/Zurich",
	},
	"LSMP": {
		ICAO:      "LSMP",
		Name:      "Payerne Airport",
		State:     "Vaud",
		Country:   "CH",
		Elevation: 1465,
		Latitude:  46.8432006836,
		Longitude: 6.9150600433,
		Timezone:  "Europe/Zurich",
	},
	"LSPA": {
		ICAO:      "LSPA",
		Name:      "Amlikon Glider Airport",
		State:     "Thurgau",
		Country:   "CH",
		Elevation: 1371,
		Latitude:  47.5741996765,
		Longitude: 9.0474996567,
		Timezone:  "Europe/Zurich",
	},
	"LSPD": {
		ICAO:      "LSPD",
		Name:      "Dittingen Airport",
		State:     "Basel-Landschaft",
		Country:   "CH",
		Elevation: 1758,
		Latitude:  47.4385986328,
		Longitude: 7.4913902283,
		Timezone:  "Europe/Zurich",
	},
	"LSPF": {
		ICAO:      "LSPF",
		Name:      "Schaffhausen Airport",
		State:     "Schaffhausen",
		Country:   "CH",
		Elevation: 1519,
		Latitude:  47.6906013489,
		Longitude: 8.5269403458,
		Timezone:  "Europe/Zurich",
	},
	"LSPG": {
		ICAO:      "LSPG",
		Name:      "Kagiswil Airport",
		State:     "Obwalden",
		Country:   "CH",
		Elevation: 1525,
		Latitude:  46.9082984924,
		Longitude: 8.2541704178,
		Timezone:  "Europe/Zurich",
	},
	"LSPH": {
		ICAO:      "LSPH",
		Name:      "Winterthur Airport",
		City:      "Winterthur",
		State:     "Zurich",
		Country:   "CH",
		Elevation: 1506,
		Latitude:  47.5149993896,
		Longitude: 8.7719402313,
		Timezone:  "Europe/Zurich",
	},
	"LSPK": {
		ICAO:      "LSPK",
		Name:      "Hasenstrick Airport",
		State:     "Zurich",
		Country:   "CH",
		Latitude:  47.2799987793,
		Longitude: 8.881939888,
		Timezone:  "Europe/Zurich",
	},
	"LSPL": {
		ICAO:      "LSPL",
		Name:      "Langenthal Airport",
		State:     "Bern",
		Country:   "CH",
		Elevation: 1575,
		Latitude:  47.182800293,
		Longitude: 7.7413902283,
		Timezone:  "Europe/Zurich",
	},
	"LSPM": {
		ICAO:      "LSPM",
		Name:      "Ambri Airport",
		City:      "Quinto",
		State:     "Ticino",
		Country:   "CH",
		Elevation: 3241,
		Latitude:  46.5125007629,
		Longitude: 8.6897802353,
		Timezone:  "Europe/Zurich",
	},
	"LSPN": {
		ICAO:      "LSPN",
		Name:      "Triengen Airport",
		State:     "Lucerne",
		Country:   "CH",
		Elevation: 1594,
		Latitude:  47.2266998291,
		Longitude: 8.0780601501,
		Timezone:  "Europe/Zurich",
	},
	"LSPO": {
		ICAO:      "LSPO",
		Name:      "Olten Airport",
		State:     "Solothurn",
		Country:   "CH",
		Latitude:  47.3443984985,
		Longitude: 7.8894400597,
		Timezone:  "Europe/Zurich",
	},
	"LSPU": {
		ICAO:      "LSPU",
		Name:      "Muenster Aero Airport",
		State:     "Valais",
		Country:   "CH",
		Elevation: 4380,
		Latitude:  46.4803009033,
		Longitude: 8.2633304596,
		Timezone:  "Europe/Zurich",
	},
	"LSPV": {
		ICAO:      "LSPV",
		Name:      "Wangen-Lachen Airport",
		State:     "Schwyz",
		Country:   "CH",
		Elevation: 1335,
		Latitude:  47.20470047,
		Longitude: 8.8675003052,
		Timezone:  "Europe/Zurich",
	},
	"LSTA": {
		ICAO:      "LSTA",
		Name:      "Raron Airport",
		City:      "Raron",
		State:     "Valais",
		Country:   "CH",
		Elevation: 2029,
		Latitude:  46.3036003113,
		Longitude: 7.8233299255,
		Timezone:  "Europe/Zurich",
	},
	"LSTB": {
		ICAO:      "LSTB",
		Name:      "Bellechasse Airport",
		State:     "Fribourg",
		Country:   "CH",
		Elevation: 1421,
		Latitude:  46.9794006348,
		Longitude: 7.1322197914,
		Timezone:  "Europe/Zurich",
	},
	"LSTO": {
		ICAO:      "LSTO",
		Name:      "Motiers Airport",
		State:     "Neuchatel",
		Country:   "CH",
		Elevation: 2402,
		Latitude:  46.9166984558,
		Longitude: 6.6149997711,
		Timezone:  "Europe/Zurich",
	},
	"LSTR": {
		ICAO:      "LSTR",
		Name:      "Montricher Airport",
		State:     "Vaud",
		Country:   "CH",
		Elevation: 2178,
		Latitude:  46.5903015137,
		Longitude: 6.4005599022,
		Timezone:  "Europe/Zurich",
	},
	"LSTS": {
		ICAO:      "LSTS",
		Name:      "St Stephan Airport",
		State:     "Bern",
		Country:   "CH",
		Elevation: 3304,
		Latitude:  46.4973983765,
		Longitude: 7.4125699997,
		Timezone:  "Europe/Zurich",
	},
	"LSTZ": {
		ICAO:      "LSTZ",
		Name:      "Zweisimmen Airport",
		State:     "Bern",
		Country:   "CH",
		Elevation: 3068,
		Latitude:  46.5525016785,
		Longitude: 7.3805599213,
		Timezone:  "Europe/Zurich",
	},
	"LSXV": {
		ICAO:      "LSXV",
		Name:      "San Vittore Airport",
		State:     "Grisons",
		Country:   "CH",
		Elevation: 869,
		Latitude:  46.2341995239,
		Longitude: 9.0941696167,
		Timezone:  "Europe/Zurich",
	},
	"LSZA": {
		ICAO:      "LSZA",
		IATA:      "LUG",
		Name:      "Lugano Airport",
		City:      "Lugano",
		State:     "Ticino",
		Country:   "CH",
		Elevation: 915,
		Latitude:  46.0042991638,
		Longitude: 8.9105796814,
		Timezone:  "Europe/Zurich",
	},
	"LSZB": {
		ICAO:      "LSZB",
		IATA:      "BRN",
		Name:      "Bern Belp Airport",
		City:      "Bern",
		State:     "Bern",
		Country:   "CH",
		Elevation: 1674,
		Latitude:  46.914100647,
		Longitude: 7.4971499443,
		Timezone:  "Europe/Zurich",
	},
	"LSZC": {
		ICAO:      "LSZC",
		IATA:      "BXO",
		Name:      "Buochs Airport",
		City:      "Buochs",
		State:     "Nidwalden",
		Country:   "CH",
		Elevation: 1475,
		Latitude:  46.9744444444,
		Longitude: 8.3969444444,
		Timezone:  "Europe/Zurich",
	},
	"LSZE": {
		ICAO:      "LSZE",
		Name:      "Bad Ragaz Airport",
		State:     "Saint-Gallen",
		Country:   "CH",
		Elevation: 1617,
		Latitude:  47.0149993896,
		Longitude: 9.4819402695,
		Timezone:  "Europe/Zurich",
	},
	"LSZF": {
		ICAO:      "LSZF",
		Name:      "Birrfeld Airport",
		State:     "Aargau",
		Country:   "CH",
		Elevation: 1300,
		Latitude:  47.4435997009,
		Longitude: 8.2336101532,
		Timezone:  "Europe/Zurich",
	},
	"LSZG": {
		ICAO:      "LSZG",
		IATA:      "ZHI",
		Name:      "Grenchen Airport",
		State:     "Solothurn",
		Country:   "CH",
		Elevation: 1411,
		Latitude:  47.1815986633,
		Longitude: 7.4171900749,
		Timezone:  "Europe/Zurich",
	},
	"LSZH": {
		ICAO:      "LSZH",
		IATA:      "ZRH",
		Name:      "Zurich Airport",
		City:      "Zurich",
		State:     "Zurich",
		Country:   "CH",
		Elevation: 1416,
		Latitude:  47.4646987915,
		Longitude: 8.5491695404,
		Timezone:  "Europe/Zurich",
	},
	"LSZI": {
		ICAO:      "LSZI",
		Name:      "Fricktal-Schupfart Airport",
		State:     "Aargau",
		Country:   "CH",
		Elevation: 1788,
		Latitude:  47.5088996887,
		Longitude: 7.9499998093,
		Timezone:  "Europe/Berlin",
	},
	"LSZJ": {
		ICAO:      "LSZJ",
		Name:      "Courtelary Airport",
		State:     "Bern",
		Country:   "CH",
		Elevation: 2247,
		Latitude:  47.1836013794,
		Longitude: 7.0908298492,
		Timezone:  "Europe/Zurich",
	},
	"LSZK": {
		ICAO:      "LSZK",
		Name:      "Speck-Fehraltorf Airport",
		State:     "Zurich",
		Country:   "CH",
		Elevation: 1748,
		Latitude:  47.3763999939,
		Longitude: 8.7574996948,
		Timezone:  "Europe/Zurich",
	},
	"LSZL": {
		ICAO:      "LSZL",
		IATA:      "ZJI",
		Name:      "Locarno Airport",
		State:     "Ticino",
		Country:   "CH",
		Elevation: 650,
		Latitude:  46.1608009338,
		Longitude: 8.8786096573,
		Timezone:  "Europe/Zurich",
	},
	"LSZM": {
		ICAO:      "LSZM",
		IATA:      "MLH",
		Name:      "EuroAirport Basel-Mulhouse-Freiburg Airport",
		City:      "Bale/Mulhouse",
		State:     "Grand-Est",
		Country:   "CH",
		Elevation: 885,
		Latitude:  47.5895996094,
		Longitude: 7.5299100876,
		Timezone:  "Europe/Paris",
	},
	"LSZN": {
		ICAO:      "LSZN",
		Name:      "Hausen am Albis Airport",
		State:     "Zurich",
		Country:   "CH",
		Elevation: 1928,
		Latitude:  47.2386016846,
		Longitude: 8.5155601501,
		Timezone:  "Europe/Zurich",
	},
	"LSZO": {
		ICAO:      "LSZO",
		Name:      "Luzern-Beromunster Airport",
		City:      "Lucerne",
		State:     "Lucerne",
		Country:   "CH",
		Elevation: 2146,
		Latitude:  47.1899986267,
		Longitude: 8.2047195435,
		Timezone:  "Europe/Zurich",
	},
	"LSZP": {
		ICAO:      "LSZP",
		Name:      "Biel-Kappelen Airport",
		State:     "Bern",
		Country:   "CH",
		Elevation: 1437,
		Latitude:  47.0891990662,
		Longitude: 7.2899999619,
		Timezone:  "Europe/Zurich",
	},
	"LSZQ": {
		ICAO:      "LSZQ",
		Name:      "Bressaucourt Airport",
		City:      "Bressaucourt",
		State:     "Jura",
		Country:   "CH",
		Elevation: 1866,
		Latitude:  47.392677,
		Longitude: 7.028648,
		Timezone:  "Europe/Zurich",
	},
	"LSZR": {
		ICAO:      "LSZR",
		IATA:      "ACH",
		Name:      "St Gallen Altenrhein Airport",
		City:      "Altenrhein",
		State:     "Saint-Gallen",
		Country:   "CH",
		Elevation: 1306,
		Latitude:  47.4850006104,
		Longitude: 9.5607700348,
		Timezone:  "Europe/Vienna",
	},
	"LSZS": {
		ICAO:      "LSZS",
		IATA:      "SMV",
		Name:      "Samedan Airport",
		State:     "Grisons",
		Country:   "CH",
		Elevation: 5600,
		Latitude:  46.5340995789,
		Longitude: 9.8841104507,
		Timezone:  "Europe/Zurich",
	},
	"LSZT": {
		ICAO:      "LSZT",
		Name:      "Lommis Airfield",
		City:      "Lommis",
		State:     "Thurgau",
		Country:   "CH",
		Elevation: 1539,
		Latitude:  47.5243988037,
		Longitude: 9.0030603409,
		Timezone:  "Europe/Zurich",
	},
	"LSZU": {
		ICAO:      "LSZU",
		Name:      "Buttwil Airport",
		State:     "Aargau",
		Country:   "CH",
		Elevation: 2372,
		Latitude:  47.2647018433,
		Longitude: 8.3024997711,
		Timezone:  "Europe/Zurich",
	},
	"LSZV": {
		ICAO:      "LSZV",
		Name:      "Sitterdorf Airport",
		State:     "Thurgau",
		Country:   "CH",
		Elevation: 1660,
		Latitude:  47.5088996887,
		Longitude: 9.2627801895,
		Timezone:  "Europe/Zurich",
	},
	"LSZW": {
		ICAO:      "LSZW",
		Name:      "Thun Airport",
		City:      "Thun",
		State:     "Bern",
		Country:   "CH",
		Elevation: 1837,
		Latitude:  46.756401062,
		Longitude: 7.6005601883,
		Timezone:  "Europe/Zurich",
	},
	"LSZX": {
		ICAO:      "LSZX",
		Name:      "Schanis Airport",
		State:     "Saint-Gallen",
		Country:   "CH",
		Elevation: 1365,
		Latitude:  47.1716995239,
		Longitude: 9.039440155,
		Timezone:  "Europe/Zurich",
	},
	"LTAB": {
		ICAO:      "LTAB",
		Name:      "Guvercinlik Airport",
		City:      "Ankara",
		State:     "Ankara",
		Country:   "TR",
		Elevation: 2687,
		Latitude:  39.9350013733,
		Longitude: 32.7407989502,
		Timezone:  "Europe/Istanbul",
	},
	"LTAC": {
		ICAO:      "LTAC",
		IATA:      "ESB",
		Name:      "Esenboga International Airport",
		City:      "Ankara",
		State:     "Ankara",
		Country:   "TR",
		Elevation: 3125,
		Latitude:  40.1281013489,
		Longitude: 32.995098114,
		Timezone:  "Europe/Istanbul",
	},
	"LTAD": {
		ICAO:      "LTAD",
		IATA:      "ANK",
		Name:      "Etimesgut Air Base",
		City:      "Ankara",
		State:     "Ankara",
		Country:   "TR",
		Elevation: 2653,
		Latitude:  39.949798584,
		Longitude: 32.6885986328,
		Timezone:  "Europe/Istanbul",
	},
	"LTAE": {
		ICAO:      "LTAE",
		Name:      "Akinci Air Base",
		City:      "Ankara",
		State:     "Ankara",
		Country:   "TR",
		Elevation: 2767,
		Latitude:  40.0788993835,
		Longitude: 32.5656013489,
		Timezone:  "Europe/Istanbul",
	},
	"LTAF": {
		ICAO:      "LTAF",
		IATA:      "ADA",
		Name:      "Adana Airport",
		City:      "Adana",
		State:     "Adana",
		Country:   "TR",
		Elevation: 65,
		Latitude:  36.9822006226,
		Longitude: 35.2803993225,
		Timezone:  "Europe/Istanbul",
	},
	"LTAG": {
		ICAO:      "LTAG",
		IATA:      "UAB",
		Name:      "Incirlik Air Base",
		City:      "Adana",
		State:     "Adana",
		Country:   "TR",
		Elevation: 238,
		Latitude:  37.0021018982,
		Longitude: 35.4258995056,
		Timezone:  "Europe/Istanbul",
	},
	"LTAH": {
		ICAO:      "LTAH",
		IATA:      "AFY",
		Name:      "Afyon Airport",
		City:      "Afyonkarahisar",
		State:     "Afyonkarahisar",
		Country:   "TR",
		Elevation: 3310,
		Latitude:  38.726398468,
		Longitude: 30.6011009216,
		Timezone:  "Europe/Istanbul",
	},
	"LTAI": {
		ICAO:      "LTAI",
		IATA:      "AYT",
		Name:      "Antalya International Airport",
		City:      "Antalya",
		State:     "Antalya",
		Country:   "TR",
		Elevation: 177,
		Latitude:  36.8987007141,
		Longitude: 30.8005008698,
		Timezone:  "Europe/Istanbul",
	},
	"LTAJ": {
		ICAO:      "LTAJ",
		IATA:      "GZT",
		Name:      "Gaziantep International Airport",
		City:      "Gaziantep",
		State:     "Gaziantep",
		Country:   "TR",
		Elevation: 2315,
		Latitude:  36.9472007751,
		Longitude: 37.4786987305,
		Timezone:  "Europe/Istanbul",
	},
	"LTAK": {
		ICAO:      "LTAK",
		Name:      "Iskenderun Airport",
		City:      "Iskenderun",
		State:     "Hatay",
		Country:   "TR",
		Elevation: 25,
		Latitude:  36.5744552612,
		Longitude: 36.1534194946,
		Timezone:  "Europe/Istanbul",
	},
	"LTAL": {
		ICAO:      "LTAL",
		IATA:      "KFS",
		Name:      "Kastamonu Airport",
		City:      "Kastamonu",
		State:     "Kastamonu",
		Country:   "TR",
		Elevation: 3520,
		Latitude:  41.314201355,
		Longitude: 33.7957992554,
		Timezone:  "Europe/Istanbul",
	},
	"LTAN": {
		ICAO:      "LTAN",
		IATA:      "KYA",
		Name:      "Konya Airport",
		City:      "Konya",
		State:     "Konya",
		Country:   "TR",
		Elevation: 3381,
		Latitude:  37.9790000916,
		Longitude: 32.5619010925,
		Timezone:  "Europe/Istanbul",
	},
	"LTAO": {
		ICAO:      "LTAO",
		Name:      "Malatya Tulga Airport",
		City:      "Malatya",
		State:     "Malatya",
		Country:   "TR",
		Elevation: 3016,
		Latitude:  38.3536987305,
		Longitude: 38.2538986206,
		Timezone:  "Europe/Istanbul",
	},
	"LTAP": {
		ICAO:      "LTAP",
		IATA:      "MZH",
		Name:      "Amasya Merzifon Airport",
		City:      "Amasya",
		State:     "Amasya",
		Country:   "TR",
		Elevation: 1758,
		Latitude:  40.8293991089,
		Longitude: 35.5219993591,
		Timezone:  "Europe/Istanbul",
	},
	"LTAQ": {
		ICAO:      "LTAQ",
		IATA:      "SSX",
		Name:      "Samsun Samair Airport",
		City:      "Samsun",
		State:     "Samsun",
		Country:   "TR",
		Elevation: 521,
		Latitude:  41.2786445618,
		Longitude: 36.3051795959,
		Timezone:  "Europe/Istanbul",
	},
	"LTAR": {
		ICAO:      "LTAR",
		IATA:      "VAS",
		Name:      "Sivas Airport",
		City:      "Sivas",
		State:     "Sivas",
		Country:   "TR",
		Elevation: 5236,
		Latitude:  39.8138008118,
		Longitude: 36.9034996033,
		Timezone:  "Europe/Istanbul",
	},
	"LTAS": {
		ICAO:      "LTAS",
		IATA:      "ONQ",
		Name:      "Zonguldak Airport",
		City:      "Zonguldak",
		State:     "Zonguldak",
		Country:   "TR",
		Elevation: 39,
		Latitude:  41.506401062,
		Longitude: 32.0886001587,
		Timezone:  "Europe/Istanbul",
	},
	"LTAT": {
		ICAO:      "LTAT",
		IATA:      "MLX",
		Name:      "Malatya Erhac Airport",
		City:      "Malatya",
		State:     "Malatya",
		Country:   "TR",
		Elevation: 2828,
		Latitude:  38.4352989197,
		Longitude: 38.0909996033,
		Timezone:  "Europe/Istanbul",
	},
	"LTAU": {
		ICAO:      "LTAU",
		IATA:      "ASR",
		Name:      "Kayseri Erkilet Airport",
		City:      "Kayseri",
		State:     "Kayseri",
		Country:   "TR",
		Elevation: 3463,
		Latitude:  38.770401001,
		Longitude: 35.4953994751,
		Timezone:  "Europe/Istanbul",
	},
	"LTAV": {
		ICAO:      "LTAV",
		Name:      "Sivrihisar Airport",
		City:      "Sivrihisar",
		State:     "Eskişehir",
		Country:   "TR",
		Elevation: 3185,
		Latitude:  39.451499939,
		Longitude: 31.3652992249,
		Timezone:  "Europe/Istanbul",
	},
	"LTAW": {
		ICAO:      "LTAW",
		IATA:      "TJK",
		Name:      "Tokat Airport",
		City:      "Tokat",
		State:     "Tokat",
		Country:   "TR",
		Elevation: 1831,
		Latitude:  40.3074302673,
		Longitude: 36.3674087524,
		Timezone:  "Europe/Istanbul",
	},
	"LTAY": {
		ICAO:      "LTAY",
		IATA:      "DNZ",
		Name:      "Cardak Airport",
		City:      "Denizli",
		State:     "Denizli",
		Country:   "TR",
		Elevation: 2795,
		Latitude:  37.7855987549,
		Longitude: 29.7012996674,
		Timezone:  "Europe/Istanbul",
	},
	"LTAZ": {
		ICAO:      "LTAZ",
		IATA:      "NAV",
		Name:      "Nevsehir Kapadokya International Airport",
		City:      "Nevsehir",
		State:     "Nevşehir",
		Country:   "TR",
		Elevation: 3100,
		Latitude:  38.771900177,
		Longitude: 34.5345001221,
		Timezone:  "Europe/Istanbul",
	},
	"LTBA": {
		ICAO:      "LTBA",
		IATA:      "ISL",
		Name:      "Ataturk International Airport",
		City:      "Istanbul",
		State:     "Istanbul",
		Country:   "TR",
		Elevation: 163,
		Latitude:  40.9768981934,
		Longitude: 28.8145999908,
		Timezone:  "Europe/Istanbul",
	},
	"LTBC": {
		ICAO:      "LTBC",
		Name:      "Alasehir Airport",
		City:      "Alasehir",
		State:     "Manisa",
		Country:   "TR",
		Latitude:  38.3699989319,
		Longitude: 28.557100296,
		Timezone:  "Europe/Istanbul",
	},
	"LTBD": {
		ICAO:      "LTBD",
		IATA:      "CII",
		Name:      "Cildir Airport",
		City:      "Aydin",
		State:     "Aydin",
		Country:   "TR",
		Elevation: 102,
		Latitude:  37.8149986267,
		Longitude: 27.8952999115,
		Timezone:  "Europe/Istanbul",
	},
	"LTBE": {
		ICAO:      "LTBE",
		IATA:      "BTZ",
		Name:      "Bursa Airport",
		City:      "Bursa",
		State:     "Bursa",
		Country:   "TR",
		Elevation: 331,
		Latitude:  40.2332992554,
		Longitude: 29.0091991425,
		Timezone:  "Europe/Istanbul",
	},
	"LTBF": {
		ICAO:      "LTBF",
		IATA:      "BZI",
		Name:      "Balikesir Merkez Airport",
		City:      "Balikesir",
		State:     "Balikesir",
		Country:   "TR",
		Elevation: 340,
		Latitude:  39.6193008423,
		Longitude: 27.9260005951,
		Timezone:  "Europe/Istanbul",
	},
	"LTBG": {
		ICAO:      "LTBG",
		IATA:      "BDM",
		Name:      "Bandirma Airport",
		City:      "Bandirma",
		State:     "Balikesir",
		Country:   "TR",
		Elevation: 170,
		Latitude:  40.3180007935,
		Longitude: 27.9776992798,
		Timezone:  "Europe/Istanbul",
	},
	"LTBH": {
		ICAO:      "LTBH",
		IATA:      "CKZ",
		Name:      "Canakkale Airport",
		City:      "Canakkale",
		State:     "Canakkale",
		Country:   "TR",
		Elevation: 23,
		Latitude:  40.1376991272,
		Longitude: 26.4267997742,
		Timezone:  "Europe/Istanbul",
	},
	"LTBI": {
		ICAO:      "LTBI",
		IATA:      "ESK",
		Name:      "Eskisehir Air Base",
		City:      "Eskisehir",
		State:     "Eskişehir",
		Country:   "TR",
		Elevation: 2581,
		Latitude:  39.7840995789,
		Longitude: 30.5820999146,
		Timezone:  "Europe/Istanbul",
	},
	"LTBJ": {
		ICAO:      "LTBJ",
		IATA:      "ADB",
		Name:      "Adnan Menderes International Airport",
		City:      "Izmir",
		State:     "Izmir",
		Country:   "TR",
		Elevation: 412,
		Latitude:  38.2924003601,
		Longitude: 27.156999588,
		Timezone:  "Europe/Istanbul",
	},
	"LTBK": {
		ICAO:      "LTBK",
		Name:      "Gaziemir Airport",
		State:     "Izmir",
		Country:   "TR",
		Elevation: 433,
		Latitude:  38.3190994263,
		Longitude: 27.1594009399,
		Timezone:  "Europe/Istanbul",
	},
	"LTBL": {
		ICAO:      "LTBL",
		IATA:      "IGL",
		Name:      "Cigli Airport",
		City:      "Izmir",
		State:     "Izmir",
		Country:   "TR",
		Elevation: 16,
		Latitude:  38.5130004883,
		Longitude: 27.010099411,
		Timezone:  "Europe/Istanbul",
	},
	"LTBM": {
		ICAO:      "LTBM",
		Name:      "Isparta Airport",
		State:     "Isparta",
		Country:   "TR",
		Elevation: 3250,
		Latitude:  37.7851219177,
		Longitude: 30.5900192261,
		Timezone:  "Europe/Istanbul",
	},
	"LTBN": {
		ICAO:      "LTBN",
		Name:      "Kutahya Airport",
		City:      "Kutahya",
		State:     "Kuetahya",
		Country:   "TR",
		Elevation: 3026,
		Latitude:  39.426700592,
		Longitude: 30.0168991089,
		Timezone:  "Europe/Istanbul",
	},
	"LTBO": {
		ICAO:      "LTBO",
		IATA:      "USQ",
		Name:      "Usak Airport",
		City:      "Usak",
		State:     "Uşak",
		Country:   "TR",
		Elevation: 2897,
		Latitude:  38.6814994812,
		Longitude: 29.4717006683,
		Timezone:  "Europe/Istanbul",
	},
	"LTBP": {
		ICAO:      "LTBP",
		Name:      "Yalova Airport",
		City:      "Ciftlikkoy",
		State:     "Yalova",
		Country:   "TR",
		Elevation: 6,
		Latitude:  40.6843986511,
		Longitude: 29.3756999969,
		Timezone:  "Europe/Istanbul",
	},
	"LTBQ": {
		ICAO:      "LTBQ",
		IATA:      "KCO",
		Name:      "Cengiz Topel Airport",
		State:     "Kocaeli",
		Country:   "TR",
		Elevation: 182,
		Latitude:  40.7350006104,
		Longitude: 30.0832996368,
		Timezone:  "Europe/Istanbul",
	},
	"LTBR": {
		ICAO:      "LTBR",
		IATA:      "YEI",
		Name:      "Bursa Yenisehir Airport",
		City:      "Bursa",
		State:     "Bursa",
		Country:   "TR",
		Elevation: 764,
		Latitude:  40.2551994324,
		Longitude: 29.5625991821,
		Timezone:  "Europe/Istanbul",
	},
	"LTBS": {
		ICAO:      "LTBS",
		IATA:      "DLM",
		Name:      "Dalaman International Airport",
		City:      "Dalaman",
		State:     "Mugla",
		Country:   "TR",
		Elevation: 20,
		Latitude:  36.7131004333,
		Longitude: 28.7924995422,
		Timezone:  "Europe/Istanbul",
	},
	"LTBT": {
		ICAO:      "LTBT",
		Name:      "Akhisar Airport",
		State:     "Manisa",
		Country:   "TR",
		Elevation: 263,
		Latitude:  38.8088989258,
		Longitude: 27.8339004517,
		Timezone:  "Europe/Istanbul",
	},
	"LTBU": {
		ICAO:      "LTBU",
		IATA:      "TEQ",
		Name:      "Tekirdag Corlu Airport",
		City:      "Corlu",
		State:     "Tekirdag",
		Country:   "TR",
		Elevation: 574,
		Latitude:  41.1381988525,
		Longitude: 27.9190998077,
		Timezone:  "Europe/Istanbul",
	},
	"LTBV": {
		ICAO:      "LTBV",
		IATA:      "BXN",
		Name:      "Imsik Airport",
		City:      "Bodrum",
		State:     "Mugla",
		Country:   "TR",
		Elevation: 202,
		Latitude:  37.1400985718,
		Longitude: 27.6697006226,
		Timezone:  "Europe/Istanbul",
	},
	"LTBW": {
		ICAO:      "LTBW",
		Name:      "Istanbul Hezarfen Airfield",
		City:      "Istanbul",
		State:     "Istanbul",
		Country:   "TR",
		Elevation: 57,
		Latitude:  41.1035995483,
		Longitude: 28.547700882,
		Timezone:  "Europe/Istanbul",
	},
	"LTBX": {
		ICAO:      "LTBX",
		Name:      "Samandira Air Base",
		City:      "Umraniye",
		State:     "Istanbul",
		Country:   "TR",
		Elevation: 400,
		Latitude:  40.9930000305,
		Longitude: 29.2164993286,
		Timezone:  "Europe/Istanbul",
	},
	"LTBY": {
		ICAO:      "LTBY",
		IATA:      "AOE",
		Name:      "Anadolu University Airport",
		City:      "Eskisehir",
		State:     "Eskişehir",
		Country:   "TR",
		Elevation: 2588,
		Latitude:  39.8098983765,
		Longitude: 30.5193996429,
		Timezone:  "Europe/Istanbul",
	},
	"LTBZ": {
		ICAO:      "LTBZ",
		IATA:      "KZR",
		Name:      "Zafer Airport",
		City:      "Altintas",
		State:     "Kutahya",
		Country:   "TR",
		Elevation: 3327,
		Latitude:  39.1090247,
		Longitude: 30.1371297052,
		Timezone:  "Europe/Istanbul",
	},
	"LTCA": {
		ICAO:      "LTCA",
		IATA:      "EZS",
		Name:      "Elazig Airport",
		City:      "Elazig",
		State:     "Elazig",
		Country:   "TR",
		Elevation: 2927,
		Latitude:  38.6068992615,
		Longitude: 39.2914009094,
		Timezone:  "Europe/Istanbul",
	},
	"LTCB": {
		ICAO:      "LTCB",
		IATA:      "OGU",
		Name:      "Ordu–Giresun Airport",
		City:      "Ordu",
		State:     "Ordu Province",
		Country:   "TR",
		Elevation: 18,
		Latitude:  40.966667,
		Longitude: 38.08,
		Timezone:  "Europe/Istanbul",
	},
	"LTCC": {
		ICAO:      "LTCC",
		IATA:      "DIY",
		Name:      "Diyarbakir Airport",
		City:      "Diyarbakir",
		State:     "Diyarbakir",
		Country:   "TR",
		Elevation: 2251,
		Latitude:  37.893901825,
		Longitude: 40.2010002136,
		Timezone:  "Europe/Istanbul",
	},
	"LTCD": {
		ICAO:      "LTCD",
		IATA:      "ERC",
		Name:      "Erzincan Airport",
		City:      "Erzincan",
		State:     "Erzincan",
		Country:   "TR",
		Elevation: 3783,
		Latitude:  39.7102012634,
		Longitude: 39.5270004272,
		Timezone:  "Europe/Istanbul",
	},
	"LTCE": {
		ICAO:      "LTCE",
		IATA:      "ERZ",
		Name:      "Erzurum International Airport",
		City:      "Erzurum",
		State:     "Erzurum",
		Country:   "TR",
		Elevation: 5763,
		Latitude:  39.9565010071,
		Longitude: 41.1702003479,
		Timezone:  "Europe/Istanbul",
	},
	"LTCF": {
		ICAO:      "LTCF",
		IATA:      "KSY",
		Name:      "Kars Airport",
		City:      "Kars",
		State:     "Kars",
		Country:   "TR",
		Elevation: 5889,
		Latitude:  40.5621986389,
		Longitude: 43.1150016785,
		Timezone:  "Europe/Istanbul",
	},
	"LTCG": {
		ICAO:      "LTCG",
		IATA:      "TZX",
		Name:      "Trabzon International Airport",
		City:      "Trabzon",
		State:     "Trabzon",
		Country:   "TR",
		Elevation: 104,
		Latitude:  40.995098114,
		Longitude: 39.7896995544,
		Timezone:  "Europe/Istanbul",
	},
	"LTCH": {
		ICAO:      "LTCH",
		IATA:      "SFQ",
		Name:      "Sanliurfa Airport",
		City:      "Sanliurfa",
		State:     "Sanliurfa",
		Country:   "TR",
		Elevation: 1483,
		Latitude:  37.0942993164,
		Longitude: 38.8470993042,
		Timezone:  "Europe/Istanbul",
	},
	"LTCI": {
		ICAO:      "LTCI",
		IATA:      "VAN",
		Name:      "Van Ferit Melen Airport",
		City:      "Van",
		State:     "Van",
		Country:   "TR",
		Elevation: 5480,
		Latitude:  38.4682006836,
		Longitude: 43.3322982788,
		Timezone:  "Europe/Istanbul",
	},
	"LTCJ": {
		ICAO:      "LTCJ",
		IATA:      "BAL",
		Name:      "Batman Airport",
		City:      "Batman",
		State:     "Batman",
		Country:   "TR",
		Elevation: 1822,
		Latitude:  37.9290008545,
		Longitude: 41.1166000366,
		Timezone:  "Europe/Istanbul",
	},
	"LTCK": {
		ICAO:      "LTCK",
		IATA:      "MSR",
		Name:      "Mus Airport",
		City:      "Mus",
		State:     "Muş",
		Country:   "TR",
		Elevation: 4157,
		Latitude:  38.7477989197,
		Longitude: 41.6612014771,
		Timezone:  "Europe/Istanbul",
	},
	"LTCL": {
		ICAO:      "LTCL",
		IATA:      "SXZ",
		Name:      "Siirt Airport",
		City:      "Siirt",
		State:     "Siirt",
		Country:   "TR",
		Elevation: 2001,
		Latitude:  37.9789009094,
		Longitude: 41.8404006958,
		Timezone:  "Europe/Istanbul",
	},
	"LTCM": {
		ICAO:      "LTCM",
		IATA:      "NOP",
		Name:      "Sinop Airport",
		City:      "Sinop",
		State:     "Sinop",
		Country:   "TR",
		Elevation: 20,
		Latitude:  42.0158004761,
		Longitude: 35.0663986206,
		Timezone:  "Europe/Istanbul",
	},
	"LTCN": {
		ICAO:      "LTCN",
		IATA:      "KCM",
		Name:      "Kahramanmaras Airport",
		City:      "Kahramanmaras",
		State:     "Kahramanmaraş",
		Country:   "TR",
		Elevation: 1723,
		Latitude:  37.5388259888,
		Longitude: 36.9535217285,
		Timezone:  "Europe/Istanbul",
	},
	"LTCO": {
		ICAO:      "LTCO",
		IATA:      "AJI",
		Name:      "Agri Airport",
		City:      "Agri",
		State:     "Agri",
		Country:   "TR",
		Elevation: 5462,
		Latitude:  39.6545410156,
		Longitude: 43.0259780884,
		Timezone:  "Europe/Istanbul",
	},
	"LTCP": {
		ICAO:      "LTCP",
		IATA:      "ADF",
		Name:      "Adiyaman Airport",
		City:      "Adiyaman",
		State:     "Adiyaman",
		Country:   "TR",
		Elevation: 2216,
		Latitude:  37.7313995361,
		Longitude: 38.4688987732,
		Timezone:  "Europe/Istanbul",
	},
	"LTCR": {
		ICAO:      "LTCR",
		IATA:      "MQM",
		Name:      "Mardin Airport",
		City:      "Mardin",
		State:     "Mardin",
		Country:   "TR",
		Elevation: 1729,
		Latitude:  37.2233009338,
		Longitude: 40.6316986084,
		Timezone:  "Europe/Istanbul",
	},
	"LTCS": {
		ICAO:      "LTCS",
		IATA:      "GNY",
		Name:      "Sanliurfa GAP Airport",
		City:      "Sanliurfa",
		State:     "Sanliurfa",
		Country:   "TR",
		Elevation: 2708,
		Latitude:  37.4456634521,
		Longitude: 38.8955917358,
		Timezone:  "Europe/Istanbul",
	},
	"LTCT": {
		ICAO:      "LTCT",
		IATA:      "IGD",
		Name:      "Igdir Airport",
		City:      "Igdir",
		State:     "Kars",
		Country:   "TR",
		Elevation: 3101,
		Latitude:  39.9766273499,
		Longitude: 43.8766479492,
		Timezone:  "Europe/Istanbul",
	},
	"LTCW": {
		ICAO:      "LTCW",
		IATA:      "YKO",
		Name:      "Hakkari Yuksekova Airport",
		City:      "Yuksekova",
		State:     "Hakkari",
		Country:   "TR",
		Elevation: 6400,
		Latitude:  37.551667,
		Longitude: 44.233611,
		Timezone:  "Europe/Istanbul",
	},
	"LTDA": {
		ICAO:      "LTDA",
		IATA:      "HTY",
		Name:      "Hatay Airport",
		City:      "Hatay",
		State:     "Hatay",
		Country:   "TR",
		Elevation: 269,
		Latitude:  36.36277771,
		Longitude: 36.2822227478,
		Timezone:  "Europe/Istanbul",
	},
	"LTFA": {
		ICAO:      "LTFA",
		Name:      "Kaklic Airport",
		City:      "Izmir",
		State:     "Izmir",
		Country:   "TR",
		Elevation: 13,
		Latitude:  38.5176010132,
		Longitude: 26.977399826,
		Timezone:  "Europe/Istanbul",
	},
	"LTFB": {
		ICAO:      "LTFB",
		Name:      "Selcuk Efes Airport",
		City:      "Selcuk",
		State:     "Izmir",
		Country:   "TR",
		Elevation: 10,
		Latitude:  37.9506988525,
		Longitude: 27.329000473,
		Timezone:  "Europe/Istanbul",
	},
	"LTFC": {
		ICAO:      "LTFC",
		IATA:      "ISE",
		Name:      "Suleyman Demirel International Airport",
		City:      "Isparta",
		State:     "Isparta",
		Country:   "TR",
		Elevation: 2835,
		Latitude:  37.8554000854,
		Longitude: 30.3684005737,
		Timezone:  "Europe/Istanbul",
	},
	"LTFD": {
		ICAO:      "LTFD",
		IATA:      "EDO",
		Name:      "Balikesir Korfez Airport",
		City:      "Edremit",
		State:     "Balikesir",
		Country:   "TR",
		Elevation: 50,
		Latitude:  39.554599762,
		Longitude: 27.0137996674,
		Timezone:  "Europe/Istanbul",
	},
	"LTFE": {
		ICAO:      "LTFE",
		IATA:      "BJV",
		Name:      "Milas Bodrum International Airport",
		City:      "Bodrum",
		State:     "Mugla",
		Country:   "TR",
		Elevation: 21,
		Latitude:  37.2505989075,
		Longitude: 27.6643009186,
		Timezone:  "Europe/Istanbul",
	},
	"LTFG": {
		ICAO:      "LTFG",
		IATA:      "GZP",
		Name:      "Gazipasa Airport",
		City:      "Gazipasa",
		State:     "Antalya",
		Country:   "TR",
		Elevation: 86,
		Latitude:  36.2992172241,
		Longitude: 32.3005981445,
		Timezone:  "Europe/Istanbul",
	},
	"LTFH": {
		ICAO:      "LTFH",
		IATA:      "SZF",
		Name:      "Samsun Carsamba Airport",
		City:      "Samsun",
		State:     "Samsun",
		Country:   "TR",
		Elevation: 18,
		Latitude:  41.2545013428,
		Longitude: 36.5671005249,
		Timezone:  "Europe/Istanbul",
	},
	"LTFJ": {
		ICAO:      "LTFJ",
		IATA:      "SAW",
		Name:      "Sabiha Gokcen International Airport",
		City:      "Istanbul",
		State:     "Istanbul",
		Country:   "TR",
		Elevation: 312,
		Latitude:  40.898601532,
		Longitude: 29.3092002869,
		Timezone:  "Europe/Istanbul",
	},
	"LTFK": {
		ICAO:      "LTFK",
		IATA:      "GKD",
		Name:      "Imroz Airport",
		City:      "Gokceada",
		State:     "Canakkale",
		Country:   "TR",
		Elevation: 73,
		Latitude:  40.204498291,
		Longitude: 25.8833007812,
		Timezone:  "Europe/Istanbul",
	},
	"LTFM": {
		ICAO:      "LTFM",
		IATA:      "IST",
		Name:      "Istanbul Airport",
		City:      "Arnavutkoy",
		State:     "Istanbul",
		Country:   "TR",
		Elevation: 325,
		Latitude:  41.262222,
		Longitude: 28.727778,
		Timezone:  "Europe/Istanbul",
	},
	"LTXE": {
		ICAO:      "LTXE",
		Name:      "Karain Airport",
		City:      "Antalya",
		State:     "Hakkari",
		Country:   "TR",
		Elevation: 1015,
		Latitude:  37.0964431763,
		Longitude: 30.6477355957,
		Timezone:  "Europe/Istanbul",
	},
	"LUBL": {
		ICAO:      "LUBL",
		IATA:      "BZY",
		Name:      "Balti International Airport",
		City:      "Strymba",
		State:     "Balţi",
		Country:   "MD",
		Elevation: 758,
		Latitude:  47.8381004333,
		Longitude: 27.7814998627,
		Timezone:  "Europe/Chisinau",
	},
	"LUBM": {
		ICAO:      "LUBM",
		Name:      "Marculesti International Airport",
		City:      "Marculesti",
		State:     "Floreşti",
		Country:   "MD",
		Elevation: 312,
		Latitude:  47.862701416,
		Longitude: 28.2127990723,
		Timezone:  "Europe/Chisinau",
	},
	"LUCH": {
		ICAO:      "LUCH",
		Name:      "Cahul International Airport",
		City:      "Cahul",
		State:     "Cahul",
		Country:   "MD",
		Elevation: 652,
		Latitude:  45.8437995911,
		Longitude: 28.2637004852,
		Timezone:  "Europe/Chisinau",
	},
	"LUCL": {
		ICAO:      "LUCL",
		Name:      "Chadyr Lunga Airport",
		State:     "Gagauzia",
		Country:   "MD",
		Elevation: 587,
		Latitude:  46.034994,
		Longitude: 28.851844,
		Timezone:  "Europe/Chisinau",
	},
	"LUKK": {
		ICAO:      "LUKK",
		IATA:      "KIV",
		Name:      "Chisinau International Airport",
		City:      "Chisinau",
		State:     "Chișinau-Municipality",
		Country:   "MD",
		Elevation: 399,
		Latitude:  46.9277000427,
		Longitude: 28.9309997559,
		Timezone:  "Europe/Chisinau",
	},
	"LUTR": {
		ICAO:      "LUTR",
		Name:      "Tiraspol Airport",
		City:      "Tiraspol",
		State:     "Stinga-Nistrului",
		Country:   "MD",
		Elevation: 141,
		Latitude:  46.8680992126,
		Longitude: 29.5905990601,
		Timezone:  "Europe/Chisinau",
	},
	"LW75": {
		ICAO:      "LW75",
		Name:      "Stenkovec Brazda Airport",
		City:      "Brazda",
		Country:   "MK",
		Elevation: 1150,
		Latitude:  42.0591011047,
		Longitude: 21.3910007477,
		Timezone:  "Europe/Skopje",
	},
	"LWOH": {
		ICAO:      "LWOH",
		IATA:      "OHD",
		Name:      "Ohrid St. Paul the Apostle Airport",
		City:      "Ohrid",
		State:     "Debarca",
		Country:   "MK",
		Elevation: 2313,
		Latitude:  41.1800003052,
		Longitude: 20.7423000336,
		Timezone:  "Europe/Skopje",
	},
	"LWSK": {
		ICAO:      "LWSK",
		IATA:      "SKP",
		Name:      "Skopje Alexander the Great Airport",
		City:      "Skopje",
		State:     "Ilinden",
		Country:   "MK",
		Elevation: 781,
		Latitude:  41.9616012573,
		Longitude: 21.6214008331,
		Timezone:  "Europe/Skopje",
	},
	"LXGB": {
		ICAO:      "LXGB",
		IATA:      "GIB",
		Name:      "Gibraltar Airport",
		City:      "Gibraltar",
		Country:   "GI",
		Elevation: 15,
		Latitude:  36.1511993408,
		Longitude: -5.3496599197,
		Timezone:  "Europe/Gibraltar",
	},
	"LY87": {
		ICAO:      "LY87",
		Name:      "Kovin Airport",
		Country:   "RS",
		Elevation: 280,
		Latitude:  44.7743,
		Longitude: 20.9613,
		Timezone:  "Europe/Belgrade",
	},
	"LYBE": {
		ICAO:      "LYBE",
		IATA:      "BEG",
		Name:      "Belgrade Nikola Tesla Airport",
		City:      "Belgrad",
		Country:   "RS",
		Elevation: 335,
		Latitude:  44.8184013367,
		Longitude: 20.3090991974,
		Timezone:  "Europe/Belgrade",
	},
	"LYBJ": {
		ICAO:      "LYBJ",
		Name:      "Airport Lisicji Jarak",
		City:      "Belgrade",
		Country:   "RS",
		Elevation: 214,
		Latitude:  44.9369583333,
		Longitude: 20.4425555556,
		Timezone:  "Europe/Belgrade",
	},
	"LYBO": {
		ICAO:      "LYBO",
		Name:      "Bor Airport",
		City:      "Bor",
		Country:   "RS",
		Elevation: 1266,
		Latitude:  44.0181999207,
		Longitude: 22.1371002197,
		Timezone:  "Europe/Belgrade",
	},
	"LYBR": {
		ICAO:      "LYBR",
		IATA:      "IVG",
		Name:      "Berane Airport",
		City:      "Berane",
		Country:   "ME",
		Elevation: 2287,
		Latitude:  42.8390007019,
		Longitude: 19.8619995117,
		Timezone:  "Europe/Podgorica",
	},
	"LYBT": {
		ICAO:      "LYBT",
		IATA:      "BJY",
		Name:      "Batajnica Air Base",
		City:      "Batajnica",
		Country:   "RS",
		Elevation: 265,
		Latitude:  44.935299,
		Longitude: 20.2575,
		Timezone:  "Europe/Belgrade",
	},
	"LYKI": {
		ICAO:      "LYKI",
		Name:      "Kikinda Airport",
		City:      "Kikinda",
		Country:   "RS",
		Elevation: 266,
		Latitude:  45.768122,
		Longitude: 20.417261,
		Timezone:  "Europe/Belgrade",
	},
	"LYKV": {
		ICAO:      "LYKV",
		Name:      "Ladevci Airport",
		City:      "Kraljevo",
		Country:   "RS",
		Elevation: 686,
		Latitude:  43.818298,
		Longitude: 20.5872,
		Timezone:  "Europe/Belgrade",
	},
	"LYNI": {
		ICAO:      "LYNI",
		IATA:      "INI",
		Name:      "Nis Airport",
		City:      "Nis",
		State:     "Central-Serbia",
		Country:   "RS",
		Elevation: 648,
		Latitude:  43.337299,
		Longitude: 21.853701,
		Timezone:  "Europe/Belgrade",
	},
	"LYNK": {
		ICAO:      "LYNK",
		Name:      "Niksic Airport",
		City:      "Niksic",
		Country:   "ME",
		Elevation: 2008,
		Latitude:  42.7737007141,
		Longitude: 18.9148006439,
		Timezone:  "Europe/Podgorica",
	},
	"LYNS": {
		ICAO:      "LYNS",
		IATA:      "QND",
		Name:      "Cenej Airport",
		City:      "Novi Sad",
		Country:   "RS",
		Elevation: 266,
		Latitude:  45.385799,
		Longitude: 19.839199,
		Timezone:  "Europe/Belgrade",
	},
	"LYPG": {
		ICAO:      "LYPG",
		IATA:      "TGD",
		Name:      "Podgorica Airport",
		City:      "Podgorica",
		State:     "Podgorica",
		Country:   "ME",
		Elevation: 141,
		Latitude:  42.3594017029,
		Longitude: 19.2518997192,
		Timezone:  "Europe/Podgorica",
	},
	"LYPO": {
		ICAO:      "LYPO",
		Name:      "Cemovsko Polje Airport",
		City:      "Podgorica",
		Country:   "ME",
		Elevation: 197,
		Latitude:  42.4223175049,
		Longitude: 19.2907772064,
		Timezone:  "Europe/Podgorica",
	},
	"LYPT": {
		ICAO:      "LYPT",
		Name:      "Batlava-Donja Penduha Airfield",
		City:      "Batlava",
		State:     "Pristina",
		Country:   "KS",
		Elevation: 1978,
		Latitude:  42.845500946,
		Longitude: 21.2199993134,
		Timezone:  "Europe/Belgrade",
	},
	"LYSM": {
		ICAO:      "LYSM",
		Name:      "Veliki Radinci Airfield",
		City:      "Sremska Mitrovica",
		Country:   "RS",
		Elevation: 320,
		Latitude:  45.037453,
		Longitude: 19.660661,
		Timezone:  "Europe/Belgrade",
	},
	"LYSP": {
		ICAO:      "LYSP",
		Name:      "Rudine airfield",
		City:      "Smederevska palanka",
		Country:   "RS",
		Latitude:  44.3513888889,
		Longitude: 20.96,
		Timezone:  "Europe/Belgrade",
	},
	"LYSU": {
		ICAO:      "LYSU",
		Name:      "Subotica Airport",
		City:      "Subotica",
		Country:   "RS",
		Elevation: 340,
		Latitude:  46.022837,
		Longitude: 19.706454,
		Timezone:  "Europe/Belgrade",
	},
	"LYTR": {
		ICAO:      "LYTR",
		Name:      "Trstenik Airport",
		Country:   "RS",
		Latitude:  43.614189,
		Longitude: 21.030217,
		Timezone:  "Europe/Belgrade",
	},
	"LYTV": {
		ICAO:      "LYTV",
		IATA:      "TIV",
		Name:      "Tivat Airport",
		City:      "Tivat",
		Country:   "ME",
		Elevation: 20,
		Latitude:  42.4047012329,
		Longitude: 18.7233009338,
		Timezone:  "Europe/Podgorica",
	},
	"LYUZ": {
		ICAO:      "LYUZ",
		IATA:      "UZC",
		Name:      "Ponikve Airport",
		City:      "Uzice",
		Country:   "RS",
		Elevation: 2966,
		Latitude:  43.898899,
		Longitude: 19.697701,
		Timezone:  "Europe/Belgrade",
	},
	"LYVA": {
		ICAO:      "LYVA",
		IATA:      "QWV",
		Name:      "Divci Airport",
		City:      "Valjevo",
		Country:   "RS",
		Elevation: 469,
		Latitude:  44.2978166667,
		Longitude: 20.0219111111,
		Timezone:  "Europe/Belgrade",
	},
	"LYVR": {
		ICAO:      "LYVR",
		Name:      "Vrsac International Airport",
		Country:   "RS",
		Elevation: 276,
		Latitude:  45.1469,
		Longitude: 21.3099,
		Timezone:  "Europe/Belgrade",
	},
	"LYZR": {
		ICAO:      "LYZR",
		IATA:      "ZRE",
		Name:      "Zrenjanin Airport",
		City:      "Zrenjanin",
		State:     "Vojvodina",
		Country:   "RS",
		Elevation: 246,
		Latitude:  45.339756,
		Longitude: 20.454078,
		Timezone:  "Europe/Belgrade",
	},
	"LZDB": {
		ICAO:      "LZDB",
		Name:      "Dubnica Glider Airport",
		City:      "Dubnica nad Vahom",
		State:     "Trenciansky",
		Country:   "SK",
		Elevation: 771,
		Latitude:  48.9968986511,
		Longitude: 18.192199707,
		Timezone:  "Europe/Bratislava",
	},
	"LZHL": {
		ICAO:      "LZHL",
		Name:      "Holic Airport",
		City:      "Holic",
		State:     "Trnavsky",
		Country:   "SK",
		Elevation: 520,
		Latitude:  48.8103,
		Longitude: 17.1338,
		Timezone:  "Europe/Bratislava",
	},
	"LZIB": {
		ICAO:      "LZIB",
		IATA:      "BTS",
		Name:      "M. R. Stefanik Airport",
		City:      "Bratislava",
		State:     "Bratislavsky",
		Country:   "SK",
		Elevation: 436,
		Latitude:  48.1702003479,
		Longitude: 17.2126998901,
		Timezone:  "Europe/Bratislava",
	},
	"LZKC": {
		ICAO:      "LZKC",
		Name:      "Kamenica nad Cirochou Airport",
		City:      "Kamenica nad Cirochou",
		State:     "Presovsky",
		Country:   "SK",
		Elevation: 560,
		Latitude:  48.937,
		Longitude: 21.9952,
		Timezone:  "Europe/Bratislava",
	},
	"LZKS": {
		ICAO:      "LZKS",
		Name:      "Kralova pri Senci Airport",
		City:      "Senec",
		State:     "Bratislavsky",
		Country:   "SK",
		Elevation: 400,
		Latitude:  48.2010993958,
		Longitude: 17.4719009399,
		Timezone:  "Europe/Bratislava",
	},
	"LZKZ": {
		ICAO:      "LZKZ",
		IATA:      "KSC",
		Name:      "Kosice Airport",
		City:      "Kosice",
		State:     "Kosicky",
		Country:   "SK",
		Elevation: 755,
		Latitude:  48.6631011963,
		Longitude: 21.2411003113,
		Timezone:  "Europe/Bratislava",
	},
	"LZLU": {
		ICAO:      "LZLU",
		IATA:      "LUE",
		Name:      "Lucenec Airport",
		City:      "Lucenec",
		State:     "Banskobystricky",
		Country:   "SK",
		Latitude:  48.3394012451,
		Longitude: 19.7357997894,
		Timezone:  "Europe/Bratislava",
	},
	"LZMA": {
		ICAO:      "LZMA",
		Name:      "Martin Glider Airport",
		City:      "Martin",
		State:     "Zilinsky",
		Country:   "SK",
		Elevation: 1381,
		Latitude:  49.0652999878,
		Longitude: 18.950799942,
		Timezone:  "Europe/Bratislava",
	},
	"LZMC": {
		ICAO:      "LZMC",
		Name:      "Malacky Air Base",
		City:      "Malacky",
		State:     "Bratislavsky",
		Country:   "SK",
		Elevation: 679,
		Latitude:  48.4020004272,
		Longitude: 17.1184005737,
		Timezone:  "Europe/Bratislava",
	},
	"LZNI": {
		ICAO:      "LZNI",
		Name:      "Nitra Glider Airport",
		City:      "Nitra",
		State:     "Nitriansky",
		Country:   "SK",
		Elevation: 440,
		Latitude:  48.2793998718,
		Longitude: 18.1327991486,
		Timezone:  "Europe/Bratislava",
	},
	"LZPE": {
		ICAO:      "LZPE",
		Name:      "Prievidza Glider Airport",
		City:      "Prievidza",
		State:     "Nitriansky",
		Country:   "SK",
		Elevation: 850,
		Latitude:  48.7661018372,
		Longitude: 18.5867004395,
		Timezone:  "Europe/Bratislava",
	},
	"LZPP": {
		ICAO:      "LZPP",
		IATA:      "PZY",
		Name:      "Piestany Airport",
		City:      "Piestany",
		Country:   "SK",
		Elevation: 545,
		Latitude:  48.6251983643,
		Longitude: 17.8283996582,
		Timezone:  "Europe/Bratislava",
	},
	"LZPT": {
		ICAO:      "LZPT",
		Name:      "Male Bielice Glider Airport",
		City:      "Partizanske",
		State:     "Trenciansky",
		Country:   "SK",
		Elevation: 600,
		Latitude:  48.6192016602,
		Longitude: 18.32970047,
		Timezone:  "Europe/Bratislava",
	},
	"LZPW": {
		ICAO:      "LZPW",
		IATA:      "POV",
		Name:      "Presov Air Base",
		City:      "Presov",
		State:     "Presovsky",
		Country:   "SK",
		Elevation: 1060,
		Latitude:  49.0297012329,
		Longitude: 21.3155994415,
		Timezone:  "Europe/Bratislava",
	},
	"LZSE": {
		ICAO:      "LZSE",
		Name:      "Senica Airport",
		City:      "Senica",
		Country:   "SK",
		Elevation: 610,
		Latitude:  48.6568984985,
		Longitude: 17.3321990967,
		Timezone:  "Europe/Bratislava",
	},
	"LZSK": {
		ICAO:      "LZSK",
		Name:      "Svidnik Airport",
		City:      "Svidnik",
		State:     "Presovsky",
		Country:   "SK",
		Elevation: 1161,
		Latitude:  49.3339004517,
		Longitude: 21.5702991486,
		Timezone:  "Europe/Bratislava",
	},
	"LZSL": {
		ICAO:      "LZSL",
		IATA:      "SLD",
		Name:      "Sliac Airport",
		City:      "Sliac",
		State:     "Banskobystricky",
		Country:   "SK",
		Elevation: 1043,
		Latitude:  48.6377983093,
		Longitude: 19.1340999603,
		Timezone:  "Europe/Bratislava",
	},
	"LZSV": {
		ICAO:      "LZSV",
		Name:      "Spisska Nova Glider Airport",
		City:      "Spisska Nova Ves",
		State:     "Kosicky",
		Country:   "SK",
		Elevation: 1621,
		Latitude:  48.9407997131,
		Longitude: 20.5338993073,
		Timezone:  "Europe/Bratislava",
	},
	"LZTN": {
		ICAO:      "LZTN",
		Name:      "Trencin Air Base",
		City:      "Trencin",
		State:     "Nitriansky",
		Country:   "SK",
		Elevation: 676,
		Latitude:  48.8650016785,
		Longitude: 17.9923000336,
		Timezone:  "Europe/Bratislava",
	},
	"LZTR": {
		ICAO:      "LZTR",
		Name:      "Letisko Airport",
		City:      "Boleraz",
		State:     "Trnavsky",
		Country:   "SK",
		Elevation: 666,
		Latitude:  48.4557991028,
		Longitude: 17.5263996124,
		Timezone:  "Europe/Bratislava",
	},
	"LZTT": {
		ICAO:      "LZTT",
		IATA:      "TAT",
		Name:      "Poprad-Tatry Airport",
		City:      "Poprad",
		State:     "Presovsky",
		Country:   "SK",
		Elevation: 2356,
		Latitude:  49.073600769,
		Longitude: 20.2411003113,
		Timezone:  "Europe/Bratislava",
	},
	"LZZI": {
		ICAO:      "LZZI",
		IATA:      "ILZ",
		Name:      "Zilina Airport",
		City:      "Zilina",
		State:     "Zilinsky",
		Country:   "SK",
		Elevation: 1020,
		Latitude:  49.2314987183,
		Longitude: 18.6135005951,
		Timezone:  "Europe/Bratislava",
	},
	"MA07": {
		ICAO:      "MA07",
		Name:      "Friends Ultralightport",
		City:      "Westport",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 131,
		Latitude:  41.5676002502,
		Longitude: -71.1192016602,
		Timezone:  "America/New_York",
	},
	"MA16": {
		ICAO:      "MA16",
		Name:      "Marshall's Airport",
		City:      "Holden",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 630,
		Latitude:  42.3633995056,
		Longitude: -71.8217010498,
		Timezone:  "America/New_York",
	},
	"MA18": {
		ICAO:      "MA18",
		Name:      "Cmelak Field",
		City:      "Sheffield",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 980,
		Latitude:  42.0791015625,
		Longitude: -73.3071975708,
		Timezone:  "America/New_York",
	},
	"MA19": {
		ICAO:      "MA19",
		Name:      "Berlin Lndg Area",
		City:      "Berlin",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 270,
		Latitude:  42.3675994873,
		Longitude: -71.6417007446,
		Timezone:  "America/New_York",
	},
	"MA30": {
		ICAO:      "MA30",
		Name:      "Dresser Hill Airport",
		City:      "Charlton",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 810,
		Latitude:  42.1001014709,
		Longitude: -71.9662017822,
		Timezone:  "America/New_York",
	},
	"MA36": {
		ICAO:      "MA36",
		Name:      "Snow Airport",
		City:      "Ipswich",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 61,
		Latitude:  42.6665000916,
		Longitude: -70.8529968262,
		Timezone:  "America/New_York",
	},
	"MA43": {
		ICAO:      "MA43",
		Name:      "Morehaven Airport",
		City:      "Granville",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 1120,
		Latitude:  42.0722999573,
		Longitude: -72.900100708,
		Timezone:  "America/New_York",
	},
	"MA44": {
		ICAO:      "MA44",
		Name:      "Trade Wind Airport",
		City:      "Oak Bluffs",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 43,
		Latitude:  41.4417991638,
		Longitude: -70.5702972412,
		Timezone:  "America/New_York",
	},
	"MA52": {
		ICAO:      "MA52",
		Name:      "Sids Airport",
		City:      "Maynard",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 220,
		Latitude:  42.4381713867,
		Longitude: -71.4702453613,
		Timezone:  "America/New_York",
	},
	"MA53": {
		ICAO:      "MA53",
		Name:      "Ware Airport",
		City:      "Ware",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 483,
		Latitude:  42.2820014954,
		Longitude: -72.2147979736,
		Timezone:  "America/New_York",
	},
	"MA55": {
		ICAO:      "MA55",
		Name:      "Muskeget Island Airport",
		City:      "Muskeget Island",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 90,
		Latitude:  41.3348007202,
		Longitude: -70.2994995117,
		Timezone:  "America/New_York",
	},
	"MA63": {
		ICAO:      "MA63",
		Name:      "Sherman-Private Airport",
		City:      "Pembroke",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 34,
		Latitude:  42.0740013123,
		Longitude: -70.7981033325,
		Timezone:  "America/New_York",
	},
	"MA64": {
		ICAO:      "MA64",
		Name:      "Pheasant Field",
		City:      "Pembroke",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 40,
		Latitude:  42.0754013062,
		Longitude: -70.8152999878,
		Timezone:  "America/New_York",
	},
	"MA66": {
		ICAO:      "MA66",
		Name:      "Wolomolopoag STOLport",
		City:      "Sharon",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 276,
		Latitude:  42.1007003784,
		Longitude: -71.2078018188,
		Timezone:  "America/New_York",
	},
	"MA70": {
		ICAO:      "MA70",
		Name:      "Sudbury Airport",
		City:      "Sudbury",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 135,
		Latitude:  42.3508987427,
		Longitude: -71.4298019409,
		Timezone:  "America/New_York",
	},
	"MA72": {
		ICAO:      "MA72",
		Name:      "Tuckernuck Airport",
		City:      "Tuckernuck Island",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 23,
		Latitude:  41.2975006104,
		Longitude: -70.2603988647,
		Timezone:  "America/New_York",
	},
	"MA75": {
		ICAO:      "MA75",
		Name:      "Sky Glen Airport",
		City:      "Uxbridge",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 260,
		Latitude:  42.0409011841,
		Longitude: -71.6162033081,
		Timezone:  "America/New_York",
	},
	"MA77": {
		ICAO:      "MA77",
		Name:      "Blueberry Hill Airport",
		City:      "Washington",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 2000,
		Latitude:  42.3750991821,
		Longitude: -73.1475982666,
		Timezone:  "America/New_York",
	},
	"MA80": {
		ICAO:      "MA80",
		Name:      "Mundale Airport",
		City:      "Westfield",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 280,
		Latitude:  42.1198005676,
		Longitude: -72.8143005371,
		Timezone:  "America/New_York",
	},
	"MA86": {
		ICAO:      "MA86",
		Name:      "Kendalls Lndg Area Airport",
		City:      "Windsor",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 2102,
		Latitude:  42.5055999756,
		Longitude: -73.0597991943,
		Timezone:  "America/New_York",
	},
	"MA88": {
		ICAO:      "MA88",
		Name:      "Albert Farms Airport",
		City:      "Worthington",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 1425,
		Latitude:  42.3900985718,
		Longitude: -72.9309005737,
		Timezone:  "America/New_York",
	},
	"MA89": {
		ICAO:      "MA89",
		Name:      "B&B Farm Airport",
		City:      "New Braintree",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 980,
		Latitude:  42.3460998535,
		Longitude: -72.1038970947,
		Timezone:  "America/New_York",
	},
	"MA97": {
		ICAO:      "MA97",
		Name:      "Waters Airport",
		City:      "Sutton",
		State:     "Massachusetts",
		Country:   "US",
		Elevation: 700,
		Latitude:  42.1250991821,
		Longitude: -71.7397994995,
		Timezone:  "America/New_York",
	},
	"MBAC": {
		ICAO:      "MBAC",
		Name:      "Ambergris Cay International Airport",
		City:      "Big Ambergris Cay",
		State:     "Big Ambergris Cay",
		Country:   "GB",
		Elevation: 9,
		Latitude:  21.3006333,
		Longitude: -71.64115,
		Timezone:  "America/Grand_Turk",
	},
	"MBGT": {
		ICAO:      "MBGT",
		IATA:      "GDT",
		Name:      "JAGS McCartney International Airport",
		City:      "Cockburn Town",
		Country:   "TC",
		Elevation: 13,
		Latitude:  21.4444999695,
		Longitude: -71.1423034668,
		Timezone:  "America/Grand_Turk",
	},
	"MBMC": {
		ICAO:      "MBMC",
		IATA:      "MDS",
		Name:      "Middle Caicos Airport",
		City:      "Middle Caicos",
		Country:   "TC",
		Elevation: 9,
		Latitude:  21.82602,
		Longitude: -71.8025,
		Timezone:  "America/Grand_Turk",
	},
	"MBNC": {
		ICAO:      "MBNC",
		IATA:      "NCA",
		Name:      "North Caicos Airport",
		Country:   "TC",
		Elevation: 10,
		Latitude:  21.9174995422,
		Longitude: -71.9395980835,
		Timezone:  "America/Grand_Turk",
	},
	"MBPI": {
		ICAO:      "MBPI",
		IATA:      "PIC",
		Name:      "Pine Cay Airport",
		City:      "Pine Cay",
		Country:   "TC",
		Elevation: 3,
		Latitude:  21.8999996185,
		Longitude: -72.0999984741,
		Timezone:  "America/Grand_Turk",
	},
	"MBPV": {
		ICAO:      "MBPV",
		IATA:      "PLS",
		Name:      "Providenciales Airport",
		City:      "Providenciales Island",
		Country:   "TC",
		Elevation: 15,
		Latitude:  21.7735996246,
		Longitude: -72.2658996582,
		Timezone:  "America/Grand_Turk",
	},
	"MBSC": {
		ICAO:      "MBSC",
		IATA:      "XSC",
		Name:      "South Caicos Airport",
		Country:   "TC",
		Elevation: 6,
		Latitude:  21.5156993866,
		Longitude: -71.528503418,
		Timezone:  "America/Grand_Turk",
	},
	"MBSY": {
		ICAO:      "MBSY",
		IATA:      "SLX",
		Name:      "Salt Cay Airport",
		City:      "Salt Cay",
		Country:   "TC",
		Elevation: 3,
		Latitude:  21.3330001831,
		Longitude: -71.1999969482,
		Timezone:  "America/Grand_Turk",
	},
	"MD00": {
		ICAO:      "MD00",
		Name:      "Fair's Airport",
		City:      "Pocomoke City",
		State:     "Maryland",
		Country:   "US",
		Elevation: 22,
		Latitude:  38.0461997986,
		Longitude: -75.5532989502,
		Timezone:  "America/New_York",
	},
	"MD01": {
		ICAO:      "MD01",
		Name:      "Wingfield Airport",
		City:      "Ridge",
		State:     "Maryland",
		Country:   "US",
		Elevation: 20,
		Latitude:  38.1435012817,
		Longitude: -76.3565979004,
		Timezone:  "America/New_York",
	},
	"MD03": {
		ICAO:      "MD03",
		Name:      "Woodstock Airport",
		City:      "Chesapeake City",
		State:     "Maryland",
		Country:   "US",
		Elevation: 81,
		Latitude:  39.4883995056,
		Longitude: -75.8110961914,
		Timezone:  "America/New_York",
	},
	"MD04": {
		ICAO:      "MD04",
		Name:      "Rossneck Airport",
		City:      "Cambridge",
		State:     "Maryland",
		Country:   "US",
		Elevation: 5,
		Latitude:  38.568901062,
		Longitude: -76.2358016968,
		Timezone:  "America/New_York",
	},
	"MD05": {
		ICAO:      "MD05",
		Name:      "Finagin Airfield",
		City:      "Welcome",
		State:     "Maryland",
		Country:   "US",
		Elevation: 185,
		Latitude:  38.5074996948,
		Longitude: -77.12159729,
		Timezone:  "America/New_York",
	},
	"MD06": {
		ICAO:      "MD06",
		Name:      "Pilots Cove Airport",
		City:      "Tompkinsville",
		State:     "Maryland",
		Country:   "US",
		Elevation: 20,
		Latitude:  38.3353996277,
		Longitude: -76.8786010742,
		Timezone:  "America/New_York",
	},
	"MD09": {
		ICAO:      "MD09",
		Name:      "Meadow Brook Airport",
		City:      "Queen Anne",
		State:     "Maryland",
		Country:   "US",
		Elevation: 50,
		Latitude:  38.9154014587,
		Longitude: -75.9710006714,
		Timezone:  "America/New_York",
	},
	"MD11": {
		ICAO:      "MD11",
		Name:      "Wright Field",
		City:      "Chestertown",
		State:     "Maryland",
		Country:   "US",
		Elevation: 80,
		Latitude:  39.2770996094,
		Longitude: -76.0626983643,
		Timezone:  "America/New_York",
	},
	"MD12": {
		ICAO:      "MD12",
		Name:      "Bell Airport",
		City:      "Emmittsburg",
		State:     "Maryland",
		Country:   "US",
		Elevation: 370,
		Latitude:  39.6363983154,
		Longitude: -77.2928009033,
		Timezone:  "America/New_York",
	},
	"MD14": {
		ICAO:      "MD14",
		Name:      "Robinson Private Airport",
		City:      "Benedict",
		State:     "Maryland",
		Country:   "US",
		Elevation: 20,
		Latitude:  38.5242996216,
		Longitude: -76.6836013794,
		Timezone:  "America/New_York",
	},
	"MD17": {
		ICAO:      "MD17",
		Name:      "Tinsley Airstrip",
		City:      "Butler",
		State:     "Maryland",
		Country:   "US",
		Elevation: 400,
		Latitude:  39.5167999268,
		Longitude: -76.733001709,
		Timezone:  "America/New_York",
	},
	"MD18": {
		ICAO:      "MD18",
		Name:      "Horn Point Airport",
		City:      "Cambridge",
		State:     "Maryland",
		Country:   "US",
		Elevation: 12,
		Latitude:  38.5895996094,
		Longitude: -76.1384963989,
		Timezone:  "America/New_York",
	},
	"MD19": {
		ICAO:      "MD19",
		Name:      "Hybarc Farm Airport",
		City:      "Chestertown",
		State:     "Maryland",
		Country:   "US",
		Elevation: 50,
		Latitude:  39.2032012939,
		Longitude: -76.0249023438,
		Timezone:  "America/New_York",
	},
	"MD20": {
		ICAO:      "MD20",
		Name:      "Greer Airport",
		City:      "Taneytown",
		State:     "Maryland",
		Country:   "US",
		Elevation: 520,
		Latitude:  39.6361999512,
		Longitude: -77.1228027344,
		Timezone:  "America/New_York",
	},
	"MD21": {
		ICAO:      "MD21",
		Name:      "Ashland Landing Farm Airport",
		City:      "Centreville",
		State:     "Maryland",
		Country:   "US",
		Elevation: 10,
		Latitude:  39.1180992126,
		Longitude: -76.0963973999,
		Timezone:  "America/New_York",
	},
	"MD22": {
		ICAO:      "MD22",
		Name:      "Deale Airport",
		City:      "Deale",
		State:     "Maryland",
		Country:   "US",
		Elevation: 15,
		Latitude:  38.7834014893,
		Longitude: -76.5646972656,
		Timezone:  "America/New_York",
	},
	"MD23": {
		ICAO:      "MD23",
		Name:      "Kennersley Airport",
		City:      "Church Hill",
		State:     "Maryland",
		Country:   "US",
		Elevation: 30,
		Latitude:  39.1447982788,
		Longitude: -76.0404968262,
		Timezone:  "America/New_York",
	},
	"MD24": {
		ICAO:      "MD24",
		Name:      "Haysfield Airport",
		City:      "Clarksville",
		State:     "Maryland",
		Country:   "US",
		Elevation: 420,
		Latitude:  39.2382011414,
		Longitude: -76.9466018677,
		Timezone:  "America/New_York",
	},
	"MD28": {
		ICAO:      "MD28",
		Name:      "Ewing Airport",
		City:      "Easton",
		State:     "Maryland",
		Country:   "US",
		Elevation: 50,
		Latitude:  38.809299469,
		Longitude: -75.9922027588,
		Timezone:  "America/New_York",
	},
	"MD31": {
		ICAO:      "MD31",
		Name:      "Forest Hill Airport",
		City:      "Forest Hill",
		State:     "Maryland",
		Country:   "US",
		Elevation: 476,
		Latitude:  39.5801010132,
		Longitude: -76.3747024536,
		Timezone:  "America/New_York",
	},
	"MD35": {
		ICAO:      "MD35",
		Name:      "Spring Hill Airport",
		City:      "Hebron",
		State:     "Maryland",
		Country:   "US",
		Elevation: 42,
		Latitude:  38.4347991943,
		Longitude: -75.6512985229,
		Timezone:  "America/New_York",
	},
	"MD39": {
		ICAO:      "MD39",
		Name:      "Buds Ferry Airport",
		City:      "Indian Head",
		State:     "Maryland",
		Country:   "US",
		Elevation: 50,
		Latitude:  38.5209999084,
		Longitude: -77.2518997192,
		Timezone:  "America/New_York",
	},
	"MD41": {
		ICAO:      "MD41",
		Name:      "Gary Field",
		City:      "Denton",
		State:     "Maryland",
		Country:   "US",
		Elevation: 50,
		Latitude:  38.8983001709,
		Longitude: -75.9105987549,
		Timezone:  "America/New_York",
	},
	"MD42": {
		ICAO:      "MD42",
		Name:      "Keymar Airpark",
		City:      "Keymar",
		State:     "Maryland",
		Country:   "US",
		Elevation: 437,
		Latitude:  39.6194992065,
		Longitude: -77.2275009155,
		Timezone:  "America/New_York",
	},
	"MD43": {
		ICAO:      "MD43",
		Name:      "Mountain Road Airport",
		City:      "Lakeshore",
		State:     "Maryland",
		Country:   "US",
		Elevation: 70,
		Latitude:  39.1184005737,
		Longitude: -76.49659729,
		Timezone:  "America/New_York",
	},
	"MD45": {
		ICAO:      "MD45",
		Name:      "Hampton Airport",
		City:      "Leonardtown",
		State:     "Maryland",
		Country:   "US",
		Elevation: 18,
		Latitude:  38.2307014465,
		Longitude: -76.6287994385,
		Timezone:  "America/New_York",
	},
	"MD46": {
		ICAO:      "MD46",
		Name:      "Glenair Airport",
		City:      "Dayton",
		State:     "Maryland",
		Country:   "US",
		Elevation: 590,
		Latitude:  39.2695999146,
		Longitude: -77.0053024292,
		Timezone:  "America/New_York",
	},
	"MD47": {
		ICAO:      "MD47",
		Name:      "Barnes Airport",
		City:      "Lisbon",
		State:     "Maryland",
		Country:   "US",
		Elevation: 780,
		Latitude:  39.332901001,
		Longitude: -77.0969009399,
		Timezone:  "America/New_York",
	},
	"MD48": {
		ICAO:      "MD48",
		Name:      "Albrecht Airstrip",
		City:      "Long Green",
		State:     "Maryland",
		Country:   "US",
		Elevation: 325,
		Latitude:  39.473400116,
		Longitude: -76.5001983643,
		Timezone:  "America/New_York",
	},
	"MD50": {
		ICAO:      "MD50",
		Name:      "Chesapeake Ranch Airstrip",
		City:      "Lusby",
		State:     "Maryland",
		Country:   "US",
		Elevation: 116,
		Latitude:  38.3611984253,
		Longitude: -76.4051971436,
		Timezone:  "America/New_York",
	},
	"MD53": {
		ICAO:      "MD53",
		Name:      "Mac Kinnis Airport",
		City:      "Salisbury",
		State:     "Maryland",
		Country:   "US",
		Elevation: 10,
		Latitude:  38.2887001038,
		Longitude: -75.739402771,
		Timezone:  "America/New_York",
	},
	"MD55": {
		ICAO:      "MD55",
		Name:      "Holly Springs Farm Airport",
		City:      "Nanjemoy",
		State:     "Maryland",
		Country:   "US",
		Elevation: 90,
		Latitude:  38.3975982666,
		Longitude: -77.1860961914,
		Timezone:  "America/New_York",
	},
	"MD56": {
		ICAO:      "MD56",
		Name:      "Three J Airport",
		City:      "New Windsor",
		State:     "Maryland",
		Country:   "US",
		Elevation: 510,
		Latitude:  39.5690002441,
		Longitude: -77.1213989258,
		Timezone:  "America/New_York",
	},
	"MD57": {
		ICAO:      "MD57",
		Name:      "Beverly Airport",
		City:      "Pocomoke City",
		State:     "Maryland",
		Country:   "US",
		Elevation: 10,
		Latitude:  38.0070991516,
		Longitude: -75.617401123,
		Timezone:  "America/New_York",
	},
	"MD59": {
		ICAO:      "MD59",
		Name:      "Silver Hill Airport",
		City:      "Chestertown",
		State:     "Maryland",
		Country:   "US",
		Elevation: 30,
		Latitude:  39.1398010254,
		Longitude: -76.0935974121,
		Timezone:  "America/New_York",
	},
	"MD61": {
		ICAO:      "MD61",
		Name:      "Lynch's Landing Airport",
		City:      "Rising Sun",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 510,
		Latitude:  39.7246017456,
		Longitude: -76.0705032349,
		Timezone:  "America/New_York",
	},
	"MD62": {
		ICAO:      "MD62",
		Name:      "Church Hill Airport",
		City:      "Church Hill",
		State:     "Maryland",
		Country:   "US",
		Elevation: 70,
		Latitude:  39.125,
		Longitude: -76.0016021729,
		Timezone:  "America/New_York",
	},
	"MD63": {
		ICAO:      "MD63",
		Name:      "Cromwell Farm Airport",
		City:      "Chestertown",
		State:     "Maryland",
		Country:   "US",
		Elevation: 65,
		Latitude:  39.2747993469,
		Longitude: -76.0099029541,
		Timezone:  "America/New_York",
	},
	"MD64": {
		ICAO:      "MD64",
		Name:      "Stewart Airport",
		City:      "St Michaels",
		State:     "Maryland",
		Country:   "US",
		Elevation: 10,
		Latitude:  38.7995986938,
		Longitude: -76.2723999023,
		Timezone:  "America/New_York",
	},
	"MD66": {
		ICAO:      "MD66",
		Name:      "Smith Field",
		City:      "Chestertown",
		State:     "Maryland",
		Country:   "US",
		Elevation: 60,
		Latitude:  39.3378982544,
		Longitude: -76.0979995728,
		Timezone:  "America/New_York",
	},
	"MD69": {
		ICAO:      "MD69",
		Name:      "Harris Airport",
		City:      "Still Pond",
		State:     "Maryland",
		Country:   "US",
		Elevation: 80,
		Latitude:  39.3334007263,
		Longitude: -76.0830001831,
		Timezone:  "America/New_York",
	},
	"MD70": {
		ICAO:      "MD70",
		Name:      "Flying Acres Airport",
		City:      "Sudlersville",
		State:     "Maryland",
		Country:   "US",
		Elevation: 70,
		Latitude:  39.1811981201,
		Longitude: -75.9087982178,
		Timezone:  "America/New_York",
	},
	"MD73": {
		ICAO:      "MD73",
		Name:      "Happy Landings Farm Airport",
		City:      "Unionville",
		State:     "Maryland",
		Country:   "US",
		Elevation: 693,
		Latitude:  39.4584007263,
		Longitude: -77.1480026245,
		Timezone:  "America/New_York",
	},
	"MD74": {
		ICAO:      "MD74",
		Name:      "Good Neighbor Farm Airport",
		City:      "Unionville",
		State:     "Maryland",
		Country:   "US",
		Elevation: 580,
		Latitude:  39.4743003845,
		Longitude: -77.1952972412,
		Timezone:  "America/New_York",
	},
	"MD75": {
		ICAO:      "MD75",
		Name:      "Stolcrest STOLport",
		City:      "Urbana",
		State:     "Maryland",
		Country:   "US",
		Elevation: 530,
		Latitude:  39.3418006897,
		Longitude: -77.3035964966,
		Timezone:  "America/New_York",
	},
	"MD77": {
		ICAO:      "MD77",
		Name:      "Flying M Farms Airport",
		City:      "Comus",
		State:     "Maryland",
		Country:   "US",
		Elevation: 590,
		Latitude:  39.2597999573,
		Longitude: -77.3414001465,
		Timezone:  "America/New_York",
	},
	"MD78": {
		ICAO:      "MD78",
		Name:      "Woodbine Airport",
		City:      "Woodbine",
		State:     "Maryland",
		Country:   "US",
		Elevation: 720,
		Latitude:  39.3833999634,
		Longitude: -77.0702972412,
		Timezone:  "America/New_York",
	},
	"MD82": {
		ICAO:      "MD82",
		Name:      "Ragged Island Airport",
		City:      "Cambridge",
		State:     "Maryland",
		Country:   "US",
		Elevation: 5,
		Latitude:  38.5397987366,
		Longitude: -76.2754974365,
		Timezone:  "America/New_York",
	},
	"MD83": {
		ICAO:      "MD83",
		Name:      "Ty-Ti-To Airport",
		City:      "La Plata",
		State:     "Maryland",
		Country:   "US",
		Elevation: 170,
		Latitude:  38.5208015442,
		Longitude: -77.0680999756,
		Timezone:  "America/New_York",
	},
	"MD85": {
		ICAO:      "MD85",
		Name:      "Carmean Airport",
		City:      "Ridgely",
		State:     "Maryland",
		Country:   "US",
		Elevation: 58,
		Latitude:  38.9328994751,
		Longitude: -75.8901977539,
		Timezone:  "America/New_York",
	},
	"MD91": {
		ICAO:      "MD91",
		Name:      "Saxon Farms Airport",
		City:      "Church Hill",
		State:     "Maryland",
		Country:   "US",
		Elevation: 70,
		Latitude:  39.1523017883,
		Longitude: -76.0186004639,
		Timezone:  "America/New_York",
	},
	"MD93": {
		ICAO:      "MD93",
		Name:      "Hexton Farms Airport",
		City:      "Cecilton",
		State:     "Maryland",
		Country:   "US",
		Elevation: 65,
		Latitude:  39.3750991821,
		Longitude: -75.9066009521,
		Timezone:  "America/New_York",
	},
	"MD94": {
		ICAO:      "MD94",
		Name:      "Farmington Airport",
		City:      "Farmington",
		State:     "Maryland",
		Country:   "US",
		Elevation: 445,
		Latitude:  39.6764984131,
		Longitude: -76.0369033813,
		Timezone:  "America/New_York",
	},
	"MD95": {
		ICAO:      "MD95",
		Name:      "Reservoir Airport",
		City:      "Finksburg",
		State:     "Maryland",
		Country:   "US",
		Elevation: 550,
		Latitude:  39.4819984436,
		Longitude: -76.8815994263,
		Timezone:  "America/New_York",
	},
	"MD97": {
		ICAO:      "MD97",
		Name:      "Lanseair Farms Airport",
		City:      "La Plata",
		State:     "Maryland",
		Country:   "US",
		Elevation: 40,
		Latitude:  38.4614982605,
		Longitude: -77.0464019775,
		Timezone:  "America/New_York",
	},
	"MD99": {
		ICAO:      "MD99",
		Name:      "Moran Field",
		City:      "Westernport",
		State:     "Maryland",
		Country:   "US",
		Elevation: 2080,
		Latitude:  39.5005989075,
		Longitude: -79.072303772,
		Timezone:  "America/New_York",
	},
	"MDAB": {
		ICAO:      "MDAB",
		IATA:      "EPS",
		Name:      "Arroyo Barril Airport",
		City:      "Arroyo Barril",
		State:     "Samana",
		Country:   "DO",
		Elevation: 57,
		Latitude:  19.198600769,
		Longitude: -69.4298019409,
		Timezone:  "America/Santo_Domingo",
	},
	"MDAD": {
		ICAO:      "MDAD",
		Name:      "Azua Airport",
		City:      "Azua",
		State:     "Azua",
		Country:   "DO",
		Elevation: 262,
		Latitude:  18.4255008698,
		Longitude: -70.7313995361,
		Timezone:  "America/Santo_Domingo",
	},
	"MDAN": {
		ICAO:      "MDAN",
		Name:      "Cotui Angelina Airport",
		City:      "Cotui Angelina",
		State:     "Sanchez-Ramirez",
		Country:   "DO",
		Elevation: 200,
		Latitude:  19.1315994263,
		Longitude: -70.2216033936,
		Timezone:  "America/Santo_Domingo",
	},
	"MDBA": {
		ICAO:      "MDBA",
		Name:      "Consuelo Batey Anita airport",
		City:      "Consuelo",
		State:     "Hato-Mayor",
		Country:   "DO",
		Elevation: 150,
		Latitude:  18.6833324432,
		Longitude: -69.3833312988,
		Timezone:  "America/Santo_Domingo",
	},
	"MDBC": {
		ICAO:      "MDBC",
		Name:      "La Romana Batey Cacata airport",
		City:      "La Romana",
		State:     "La-Romana",
		Country:   "DO",
		Elevation: 223,
		Latitude:  18.4797000885,
		Longitude: -68.9159011841,
		Timezone:  "America/Santo_Domingo",
	},
	"MDBE": {
		ICAO:      "MDBE",
		Name:      "La Romana Batey Lechuga Airport",
		City:      "La Romana",
		State:     "El-Seibo",
		Country:   "DO",
		Elevation: 252,
		Latitude:  18.5867996216,
		Longitude: -69.0553970337,
		Timezone:  "America/Santo_Domingo",
	},
	"MDBG": {
		ICAO:      "MDBG",
		Name:      "Higuey Airport",
		City:      "Higuey",
		State:     "La-Altagracia",
		Country:   "DO",
		Elevation: 429,
		Latitude:  18.6315994263,
		Longitude: -68.7378005981,
		Timezone:  "America/Santo_Domingo",
	},
	"MDBH": {
		ICAO:      "MDBH",
		IATA:      "BRX",
		Name:      "Maria Montez International Airport",
		City:      "Barahona",
		State:     "Barahona",
		Country:   "DO",
		Elevation: 10,
		Latitude:  18.251499176,
		Longitude: -71.1203994751,
		Timezone:  "America/Santo_Domingo",
	},
	"MDBL": {
		ICAO:      "MDBL",
		Name:      "Boca Chica Airport",
		City:      "Boca Chica",
		State:     "Santo-Domingo",
		Country:   "DO",
		Elevation: 1,
		Latitude:  18.450799942,
		Longitude: -69.5925979614,
		Timezone:  "America/Santo_Domingo",
	},
	"MDBM": {
		ICAO:      "MDBM",
		Name:      "San Pedro De Macoris Airport",
		City:      "San Pedro de Macoris",
		State:     "San-Pedro-de-Macoris",
		Country:   "DO",
		Elevation: 17,
		Latitude:  18.4480991364,
		Longitude: -69.2811965942,
		Timezone:  "America/Santo_Domingo",
	},
	"MDCR": {
		ICAO:      "MDCR",
		IATA:      "CBJ",
		Name:      "Cabo Rojo Airport",
		City:      "Cabo Rojo",
		State:     "Pedernales",
		Country:   "DO",
		Elevation: 262,
		Latitude:  17.9290008545,
		Longitude: -71.6447982788,
		Timezone:  "America/Santo_Domingo",
	},
	"MDCY": {
		ICAO:      "MDCY",
		IATA:      "AZS",
		Name:      "Samana El Catey International Airport",
		City:      "Samana",
		State:     "Maria-Trinidad-Sanchez",
		Country:   "DO",
		Elevation: 30,
		Latitude:  19.2670001984,
		Longitude: -69.7419967651,
		Timezone:  "America/Santo_Domingo",
	},
	"MDCZ": {
		ICAO:      "MDCZ",
		IATA:      "COZ",
		Name:      "Constanza Dom Re Airport",
		City:      "Costanza",
		State:     "La-Vega",
		Country:   "DO",
		Elevation: 3950,
		Latitude:  18.9074993134,
		Longitude: -70.7219009399,
		Timezone:  "America/Santo_Domingo",
	},
	"MDDJ": {
		ICAO:      "MDDJ",
		Name:      "Dajabon Airport",
		City:      "Dajabon",
		State:     "Dajabon",
		Country:   "DO",
		Elevation: 69,
		Latitude:  19.5636005402,
		Longitude: -71.6808013916,
		Timezone:  "America/Port-au-Prince",
	},
	"MDER": {
		ICAO:      "MDER",
		Name:      "El Ranchito Airport",
		City:      "La Vega",
		State:     "La-Vega",
		Country:   "DO",
		Elevation: 240,
		Latitude:  19.1848132376,
		Longitude: -70.3649640083,
		Timezone:  "America/Santo_Domingo",
	},
	"MDES": {
		ICAO:      "MDES",
		Name:      "Penuela / Esperanza Field",
		City:      "Valverde",
		State:     "Valverde",
		Country:   "DO",
		Elevation: 328,
		Latitude:  19.5841122474,
		Longitude: -70.95672369,
		Timezone:  "America/Santo_Domingo",
	},
	"MDHN": {
		ICAO:      "MDHN",
		Name:      "Juancho Enriquillo Airport",
		City:      "Barahona",
		State:     "Barahona",
		Country:   "DO",
		Elevation: 115,
		Latitude:  17.8725186193,
		Longitude: -71.2687182426,
		Timezone:  "America/Santo_Domingo",
	},
	"MDJB": {
		ICAO:      "MDJB",
		IATA:      "JBQ",
		Name:      "La Isabela International Airport",
		City:      "La Isabela",
		State:     "Santo-Domingo",
		Country:   "DO",
		Elevation: 98,
		Latitude:  18.5725002289,
		Longitude: -69.9856033325,
		Timezone:  "America/Santo_Domingo",
	},
	"MDJI": {
		ICAO:      "MDJI",
		Name:      "Jimani Airport",
		State:     "Independencia",
		Country:   "DO",
		Latitude:  18.49285,
		Longitude: -71.869999,
		Timezone:  "America/Port-au-Prince",
	},
	"MDLL": {
		ICAO:      "MDLL",
		Name:      "Los Llanos de Sabanatosa Airport",
		State:     "San-Pedro-de-Macoris",
		Country:   "DO",
		Elevation: 262,
		Latitude:  18.5964259778,
		Longitude: -69.5257759094,
		Timezone:  "America/Santo_Domingo",
	},
	"MDLM": {
		ICAO:      "MDLM",
		Name:      "Los Montones Airport",
		City:      "San Cristobal",
		State:     "San-Cristobal",
		Country:   "DO",
		Elevation: 305,
		Latitude:  18.521233,
		Longitude: -70.099132,
		Timezone:  "America/Santo_Domingo",
	},
	"MDLR": {
		ICAO:      "MDLR",
		IATA:      "LRM",
		Name:      "Casa De Campo International Airport",
		City:      "La Romana",
		State:     "La-Romana",
		Country:   "DO",
		Elevation: 240,
		Latitude:  18.4507007599,
		Longitude: -68.9117965698,
		Timezone:  "America/Santo_Domingo",
	},
	"MDMA": {
		ICAO:      "MDMA",
		Name:      "Magdalena Cuya Airport",
		City:      "La Romana",
		State:     "La-Altagracia",
		Country:   "DO",
		Elevation: 325,
		Latitude:  18.501933,
		Longitude: -68.799099,
		Timezone:  "America/Santo_Domingo",
	},
	"MDMC": {
		ICAO:      "MDMC",
		Name:      "Monte Cristi Airport",
		City:      "Monte Cristi",
		State:     "Monte-Cristi",
		Country:   "DO",
		Elevation: 121,
		Latitude:  19.8652992249,
		Longitude: -71.6453018188,
		Timezone:  "America/Santo_Domingo",
	},
	"MDPC": {
		ICAO:      "MDPC",
		IATA:      "PUJ",
		Name:      "Punta Cana International Airport",
		City:      "Punta Cana",
		State:     "La-Altagracia",
		Country:   "DO",
		Elevation: 47,
		Latitude:  18.5673999786,
		Longitude: -68.3634033203,
		Timezone:  "America/Santo_Domingo",
	},
	"MDPM": {
		ICAO:      "MDPM",
		Name:      "Piloto Airport",
		City:      "Monte Cristi",
		State:     "Monte-Cristi",
		Country:   "DO",
		Elevation: 131,
		Latitude:  19.5977273869,
		Longitude: -71.205970645,
		Timezone:  "America/Santo_Domingo",
	},
	"MDPO": {
		ICAO:      "MDPO",
		IATA:      "EPS",
		Name:      "Samana El Portillo Airport",
		City:      "Samana",
		State:     "Samana",
		Country:   "DO",
		Elevation: 30,
		Latitude:  19.3214150435,
		Longitude: -69.4959121943,
		Timezone:  "America/Santo_Domingo",
	},
	"MDPP": {
		ICAO:      "MDPP",
		IATA:      "POP",
		Name:      "Gregorio Luperon International Airport",
		City:      "Puerto Plata",
		State:     "Puerto-Plata",
		Country:   "DO",
		Elevation: 15,
		Latitude:  19.757900238,
		Longitude: -70.5699996948,
		Timezone:  "America/Santo_Domingo",
	},
	"MDSA": {
		ICAO:      "MDSA",
		Name:      "San Juan De La Maguana Airport",
		City:      "San Juan de la Maguana",
		State:     "San-Juan",
		Country:   "DO",
		Elevation: 1309,
		Latitude:  18.8255004883,
		Longitude: -71.2313995361,
		Timezone:  "America/Santo_Domingo",
	},
	"MDSB": {
		ICAO:      "MDSB",
		IATA:      "SNX",
		Name:      "Sabana de Mar Airport",
		City:      "Sabana de Mar",
		State:     "Hato-Mayor",
		Country:   "DO",
		Elevation: 15,
		Latitude:  19.0618991852,
		Longitude: -69.3852005005,
		Timezone:  "America/Santo_Domingo",
	},
	"MDSD": {
		ICAO:      "MDSD",
		IATA:      "SDQ",
		Name:      "Las Americas International Airport",
		City:      "Santo Domingo",
		State:     "Nacional",
		Country:   "DO",
		Elevation: 59,
		Latitude:  18.4297008514,
		Longitude: -69.6688995361,
		Timezone:  "America/Santo_Domingo",
	},
	"MDSI": {
		ICAO:      "MDSI",
		Name:      "San Isidro Air Base",
		City:      "San Isidro",
		State:     "Santo-Domingo",
		Country:   "DO",
		Elevation: 111,
		Latitude:  18.5037002563,
		Longitude: -69.7617034912,
		Timezone:  "America/Santo_Domingo",
	},
	"MDST": {
		ICAO:      "MDST",
		IATA:      "STI",
		Name:      "Cibao International Airport",
		City:      "Santiago",
		State:     "Dajabon",
		Country:   "DO",
		Elevation: 565,
		Latitude:  19.4060993195,
		Longitude: -70.6046981812,
		Timezone:  "America/Santo_Domingo",
	},
	"MDWO": {
		ICAO:      "MDWO",
		Name:      "Walterio Airport",
		City:      "Monte Cristi",
		State:     "Monte-Cristi",
		Country:   "DO",
		Elevation: 33,
		Latitude:  19.757867,
		Longitude: -71.623482,
		Timezone:  "America/Santo_Domingo",
	},
	"ME00": {
		ICAO:      "ME00",
		Name:      "Fort Fairfield Airport",
		City:      "Fort Fairfield",
		State:     "Maine",
		Country:   "US",
		Elevation: 471,
		Latitude:  46.765706,
		Longitude: -67.847392,
		Timezone:  "America/New_York",
	},
	"ME03": {
		ICAO:      "ME03",
		Name:      "Webber Jones Airport",
		City:      "Brownville",
		State:     "Maine",
		Country:   "US",
		Elevation: 400,
		Latitude:  45.3362007141,
		Longitude: -69.0509033203,
		Timezone:  "America/New_York",
	},
	"ME05": {
		ICAO:      "ME05",
		Name:      "Allen St Peter Memorial Airport",
		City:      "Portage",
		State:     "Maine",
		Country:   "US",
		Elevation: 800,
		Latitude:  46.75,
		Longitude: -68.4711990356,
		Timezone:  "America/New_York",
	},
	"ME06": {
		ICAO:      "ME06",
		Name:      "Flying Ed Airport",
		City:      "Charlotte",
		State:     "Maine",
		Country:   "US",
		Elevation: 440,
		Latitude:  45.0279006958,
		Longitude: -67.3094024658,
		Timezone:  "America/New_York",
	},
	"ME08": {
		ICAO:      "ME08",
		Name:      "Gadabout Gaddis Airport",
		City:      "Bingham",
		State:     "Maine",
		Country:   "US",
		Elevation: 342,
		Latitude:  45.0402984619,
		Longitude: -69.8688964844,
		Timezone:  "America/New_York",
	},
	"ME14": {
		ICAO:      "ME14",
		Name:      "Bald Mountain Airport",
		City:      "Camden",
		State:     "Maine",
		Country:   "US",
		Elevation: 570,
		Latitude:  44.2305984497,
		Longitude: -69.1231002808,
		Timezone:  "America/New_York",
	},
	"ME16": {
		ICAO:      "ME16",
		Name:      "Loring International Airport",
		City:      "Limestone",
		State:     "Maine",
		Country:   "US",
		Elevation: 746,
		Latitude:  46.9504013062,
		Longitude: -67.8859024048,
		Timezone:  "America/New_York",
	},
	"ME17": {
		ICAO:      "ME17",
		Name:      "Thomas Airport",
		City:      "Roxbury",
		State:     "Maine",
		Country:   "US",
		Elevation: 670,
		Latitude:  44.6601982117,
		Longitude: -70.5879974365,
		Timezone:  "America/New_York",
	},
	"ME19": {
		ICAO:      "ME19",
		Name:      "Clayton Lake Strip",
		City:      "Clayton Lake",
		State:     "Maine",
		Country:   "US",
		Elevation: 1150,
		Latitude:  46.6161003113,
		Longitude: -69.5223007202,
		Timezone:  "America/New_York",
	},
	"ME20": {
		ICAO:      "ME20",
		Name:      "Bob-Mar Airport",
		City:      "Fairfield",
		State:     "Maine",
		Country:   "US",
		Elevation: 310,
		Latitude:  44.5992012024,
		Longitude: -69.6239013672,
		Timezone:  "America/New_York",
	},
	"ME22": {
		ICAO:      "ME22",
		Name:      "Avery Field",
		City:      "Rockwood",
		State:     "Maine",
		Country:   "US",
		Elevation: 1070,
		Latitude:  45.6711997986,
		Longitude: -69.8068008423,
		Timezone:  "America/New_York",
	},
	"ME25": {
		ICAO:      "ME25",
		Name:      "Sunrise Farms Airport",
		City:      "Fort Kent",
		State:     "Maine",
		Country:   "US",
		Elevation: 640,
		Latitude:  47.1660995483,
		Longitude: -68.391998291,
		Timezone:  "America/New_York",
	},
	"ME32": {
		ICAO:      "ME32",
		Name:      "Morrel Field",
		City:      "Sangerville",
		State:     "Maine",
		Country:   "US",
		Elevation: 500,
		Latitude:  45.0903015137,
		Longitude: -69.2586975098,
		Timezone:  "America/New_York",
	},
	"ME33": {
		ICAO:      "ME33",
		Name:      "Farr Field",
		City:      "Harpswell",
		State:     "Maine",
		Country:   "US",
		Elevation: 70,
		Latitude:  43.7787017822,
		Longitude: -70.0130996704,
		Timezone:  "America/New_York",
	},
	"ME34": {
		ICAO:      "ME34",
		Name:      "Robinson Ridge Field",
		City:      "Jefferson",
		State:     "Maine",
		Country:   "US",
		Elevation: 250,
		Latitude:  44.1736984253,
		Longitude: -69.4287033081,
		Timezone:  "America/New_York",
	},
	"ME35": {
		ICAO:      "ME35",
		Name:      "Terra B & B Airport",
		City:      "Levant",
		State:     "Maine",
		Country:   "US",
		Elevation: 382,
		Latitude:  44.9314994812,
		Longitude: -69.0073013306,
		Timezone:  "America/New_York",
	},
	"ME36": {
		ICAO:      "ME36",
		Name:      "Mount Vernon Airport",
		City:      "Mount Vernon",
		State:     "Maine",
		Country:   "US",
		Elevation: 478,
		Latitude:  44.5092010498,
		Longitude: -69.9525985718,
		Timezone:  "America/New_York",
	},
	"ME38": {
		ICAO:      "ME38",
		Name:      "Mars Hill Airport",
		City:      "Mars Hill",
		State:     "Maine",
		Country:   "US",
		Elevation: 432,
		Latitude:  46.5278015137,
		Longitude: -67.87449646,
		Timezone:  "America/New_York",
	},
	"ME39": {
		ICAO:      "ME39",
		Name:      "Rocky Ridge Airport",
		City:      "Greene",
		State:     "Maine",
		Country:   "US",
		Elevation: 480,
		Latitude:  44.2354011536,
		Longitude: -70.132598877,
		Timezone:  "America/New_York",
	},
	"ME41": {
		ICAO:      "ME41",
		Name:      "Witherspoons Airport",
		City:      "North Haven",
		State:     "Maine",
		Country:   "US",
		Elevation: 80,
		Latitude:  44.1525993347,
		Longitude: -68.8764038086,
		Timezone:  "America/New_York",
	},
	"ME44": {
		ICAO:      "ME44",
		Name:      "Cummings Airport",
		City:      "Paris Hill",
		State:     "Maine",
		Country:   "US",
		Elevation: 960,
		Latitude:  44.2890014648,
		Longitude: -70.4461975098,
		Timezone:  "America/New_York",
	},
	"ME45": {
		ICAO:      "ME45",
		Name:      "Goosefair Airport",
		City:      "Kennebunkport",
		State:     "Maine",
		Country:   "US",
		Elevation: 40,
		Latitude:  43.4056015015,
		Longitude: -70.4322967529,
		Timezone:  "America/New_York",
	},
	"ME46": {
		ICAO:      "ME46",
		Name:      "Back Acres Airport",
		City:      "Kennebunkport",
		State:     "Maine",
		Country:   "US",
		Elevation: 60,
		Latitude:  43.4123001099,
		Longitude: -70.4467010498,
		Timezone:  "America/New_York",
	},
	"ME47": {
		ICAO:      "ME47",
		Name:      "Payne Field",
		City:      "Hartland",
		State:     "Maine",
		Country:   "US",
		Elevation: 350,
		Latitude:  44.8544998169,
		Longitude: -69.4563980103,
		Timezone:  "America/New_York",
	},
	"ME55": {
		ICAO:      "ME55",
		Name:      "Vinalhaven Airport",
		City:      "Vinalhaven",
		State:     "Maine",
		Country:   "US",
		Elevation: 72,
		Latitude:  44.0736999512,
		Longitude: -68.818901062,
		Timezone:  "America/New_York",
	},
	"ME56": {
		ICAO:      "ME56",
		Name:      "Kimberly Airport",
		City:      "Waldoboro",
		State:     "Maine",
		Country:   "US",
		Elevation: 255,
		Latitude:  44.0691986084,
		Longitude: -69.3052978516,
		Timezone:  "America/New_York",
	},
	"ME60": {
		ICAO:      "ME60",
		Name:      "Sunny Hill Airport",
		City:      "Belgrade",
		State:     "Maine",
		Country:   "US",
		Elevation: 390,
		Latitude:  44.4683990479,
		Longitude: -69.8178024292,
		Timezone:  "America/New_York",
	},
	"ME61": {
		ICAO:      "ME61",
		Name:      "Salmon Falls Airport",
		City:      "Berwick",
		State:     "Maine",
		Country:   "US",
		Elevation: 190,
		Latitude:  43.2944984436,
		Longitude: -70.8994979858,
		Timezone:  "America/New_York",
	},
	"ME62": {
		ICAO:      "ME62",
		Name:      "Grignons Private Lndg Area Airport",
		City:      "West Pittsfield",
		State:     "Maine",
		Country:   "US",
		Elevation: 390,
		Latitude:  44.8097991943,
		Longitude: -69.4730987549,
		Timezone:  "America/New_York",
	},
	"ME64": {
		ICAO:      "ME64",
		Name:      "Skydive Lebanon Airport",
		City:      "West Lebanon",
		State:     "Maine",
		Country:   "US",
		Elevation: 365,
		Latitude:  43.375,
		Longitude: -70.9291992188,
		Timezone:  "America/New_York",
	},
	"ME67": {
		ICAO:      "ME67",
		Name:      "Morrill Airpark",
		City:      "Van Buren",
		State:     "Maine",
		Country:   "US",
		Elevation: 860,
		Latitude:  47.1274986267,
		Longitude: -67.9658966064,
		Timezone:  "America/New_York",
	},
	"ME68": {
		ICAO:      "ME68",
		Name:      "Newport Sky Park Airport",
		City:      "Newport",
		State:     "Maine",
		Country:   "US",
		Elevation: 300,
		Latitude:  44.820098877,
		Longitude: -69.2384033203,
		Timezone:  "America/New_York",
	},
	"ME69": {
		ICAO:      "ME69",
		Name:      "Cliffords Airport",
		City:      "Sabattus",
		State:     "Maine",
		Country:   "US",
		Elevation: 280,
		Latitude:  44.0690002441,
		Longitude: -70.0644989014,
		Timezone:  "America/New_York",
	},
	"ME70": {
		ICAO:      "ME70",
		Name:      "Greenfield Hill Airport",
		City:      "Cambridge",
		State:     "Maine",
		Country:   "US",
		Elevation: 672,
		Latitude:  45.0376014709,
		Longitude: -69.4792022705,
		Timezone:  "America/New_York",
	},
	"ME74": {
		ICAO:      "ME74",
		Name:      "Miller's Field",
		City:      "Newburgh",
		State:     "Maine",
		Country:   "US",
		Elevation: 160,
		Latitude:  44.7495002747,
		Longitude: -68.9813995361,
		Timezone:  "America/New_York",
	},
	"ME75": {
		ICAO:      "ME75",
		Name:      "Margerison Airport",
		City:      "Lisbon Falls",
		State:     "Maine",
		Country:   "US",
		Elevation: 220,
		Latitude:  43.9539985657,
		Longitude: -70.1414031982,
		Timezone:  "America/New_York",
	},
	"ME79": {
		ICAO:      "ME79",
		Name:      "Two Falls Airport",
		City:      "Willimantic",
		State:     "Maine",
		Country:   "US",
		Elevation: 366,
		Latitude:  45.3055992126,
		Longitude: -69.3467025757,
		Timezone:  "America/New_York",
	},
	"ME88": {
		ICAO:      "ME88",
		Name:      "Morrison's Airport",
		City:      "Perry",
		State:     "Maine",
		Country:   "US",
		Elevation: 200,
		Latitude:  44.9916992188,
		Longitude: -67.0744018555,
		Timezone:  "America/New_York",
	},
	"ME89": {
		ICAO:      "ME89",
		Name:      "Buzzy's Field",
		City:      "Acton",
		State:     "Maine",
		Country:   "US",
		Elevation: 650,
		Latitude:  43.6083984375,
		Longitude: -70.9681015015,
		Timezone:  "America/New_York",
	},
	"ME91": {
		ICAO:      "ME91",
		Name:      "Webster Field",
		City:      "Gorham",
		State:     "Maine",
		Country:   "US",
		Elevation: 200,
		Latitude:  43.6750984192,
		Longitude: -70.4955978394,
		Timezone:  "America/New_York",
	},
	"ME92": {
		ICAO:      "ME92",
		Name:      "Hilltop Airport",
		City:      "Jay",
		State:     "Maine",
		Country:   "US",
		Elevation: 600,
		Latitude:  44.5167007446,
		Longitude: -70.2222976685,
		Timezone:  "America/New_York",
	},
	"ME94": {
		ICAO:      "ME94",
		Name:      "Clark Field",
		City:      "Union",
		State:     "Maine",
		Country:   "US",
		Elevation: 54,
		Latitude:  44.2167015076,
		Longitude: -69.2911987305,
		Timezone:  "America/New_York",
	},
	"ME96": {
		ICAO:      "ME96",
		Name:      "Clark Field",
		City:      "Gorham",
		State:     "Maine",
		Country:   "US",
		Elevation: 250,
		Latitude:  43.7336006165,
		Longitude: -70.4863967896,
		Timezone:  "America/New_York",
	},
	"MECT": {
		ICAO:      "MECT",
		Name:      "Catsa Airport",
		City:      "Liberia",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 66,
		Latitude:  10.5169439316,
		Longitude: -85.5655593872,
		Timezone:  "America/Costa_Rica",
	},
	"MG13": {
		ICAO:      "MG13",
		Name:      "Inta Northeast Airport",
		City:      "El Achiotal",
		State:     "Alta-Verapaz",
		Country:   "GT",
		Elevation: 510,
		Latitude:  15.8042001724,
		Longitude: -89.8507003784,
		Timezone:  "America/Guatemala",
	},
	"MGBN": {
		ICAO:      "MGBN",
		Name:      "Bananera Airport",
		City:      "Morales",
		State:     "Izabal",
		Country:   "GT",
		Elevation: 130,
		Latitude:  15.4735002518,
		Longitude: -88.8371963501,
		Timezone:  "America/Guatemala",
	},
	"MGCB": {
		ICAO:      "MGCB",
		IATA:      "CBV",
		Name:      "Coban Airport",
		City:      "Coban",
		State:     "Alta-Verapaz",
		Country:   "GT",
		Elevation: 4339,
		Latitude:  15.4689998627,
		Longitude: -90.4067001343,
		Timezone:  "America/Guatemala",
	},
	"MGCQ": {
		ICAO:      "MGCQ",
		IATA:      "CIQ",
		Name:      "Chiquimula Airport",
		City:      "Chiquimula",
		State:     "Chiquimula",
		Country:   "GT",
		Elevation: 1484,
		Latitude:  14.83092,
		Longitude: -89.520938,
		Timezone:  "America/Guatemala",
	},
	"MGCR": {
		ICAO:      "MGCR",
		IATA:      "CMM",
		Name:      "Carmelita Airport",
		City:      "Carmelita",
		State:     "Peten",
		Country:   "GT",
		Elevation: 753,
		Latitude:  17.4612007141,
		Longitude: -90.0537033081,
		Timezone:  "America/Guatemala",
	},
	"MGCT": {
		ICAO:      "MGCT",
		IATA:      "CTF",
		Name:      "Coatepeque Airport",
		City:      "Coatepeque",
		State:     "Totonicapan",
		Country:   "GT",
		Elevation: 1486,
		Latitude:  14.6941995621,
		Longitude: -91.8824996948,
		Timezone:  "America/Guatemala",
	},
	"MGDL": {
		ICAO:      "MGDL",
		IATA:      "DON",
		Name:      "Dos Lagunas Airport",
		City:      "Dos Lagunas",
		State:     "Peten",
		Country:   "GT",
		Elevation: 1057,
		Latitude:  17.6124000549,
		Longitude: -89.6884002686,
		Timezone:  "America/Guatemala",
	},
	"MGES": {
		ICAO:      "MGES",
		Name:      "Esquipulas Airport",
		City:      "Esquipulas",
		State:     "Chiquimula",
		Country:   "GT",
		Elevation: 3090,
		Latitude:  14.5666704178,
		Longitude: -89.3499984741,
		Timezone:  "America/Guatemala",
	},
	"MGGT": {
		ICAO:      "MGGT",
		IATA:      "GUA",
		Name:      "La Aurora Airport",
		City:      "Guatemala City",
		State:     "Guatemala",
		Country:   "GT",
		Elevation: 4952,
		Latitude:  14.5832996368,
		Longitude: -90.5274963379,
		Timezone:  "America/Guatemala",
	},
	"MGHT": {
		ICAO:      "MGHT",
		IATA:      "HUG",
		Name:      "Huehuetenango Airport",
		City:      "Huehuetenango",
		State:     "Huehuetenango",
		Country:   "GT",
		Elevation: 6375,
		Latitude:  15.3274002075,
		Longitude: -91.4624023438,
		Timezone:  "America/Guatemala",
	},
	"MGLL": {
		ICAO:      "MGLL",
		Name:      "La Libertad Airport",
		City:      "La Libertad",
		State:     "Peten",
		Country:   "GT",
		Elevation: 623,
		Latitude:  16.7502994537,
		Longitude: -90.1399993896,
		Timezone:  "America/Guatemala",
	},
	"MGML": {
		ICAO:      "MGML",
		Name:      "Malacatan Airport",
		City:      "San Marcos",
		State:     "San-Marcos",
		Country:   "GT",
		Elevation: 1194,
		Latitude:  14.9074640274,
		Longitude: -92.0879211426,
		Timezone:  "America/Guatemala",
	},
	"MGMM": {
		ICAO:      "MGMM",
		IATA:      "MCR",
		Name:      "Melchor de Mencos Airport",
		City:      "Melchor de Mencos",
		State:     "Peten",
		Country:   "GT",
		Elevation: 4670,
		Latitude:  17.0686092377,
		Longitude: -89.1522216797,
		Timezone:  "America/Belize",
	},
	"MGPB": {
		ICAO:      "MGPB",
		IATA:      "PBR",
		Name:      "Puerto Barrios Airport",
		City:      "Puerto Barrios",
		State:     "Izabal",
		Country:   "GT",
		Elevation: 33,
		Latitude:  15.7308998108,
		Longitude: -88.5838012695,
		Timezone:  "America/Guatemala",
	},
	"MGPC": {
		ICAO:      "MGPC",
		IATA:      "PCG",
		Name:      "Paso Caballos Airport",
		City:      "Paso Caballos",
		State:     "Peten",
		Country:   "GT",
		Elevation: 213,
		Latitude:  17.2639007568,
		Longitude: -90.2563018799,
		Timezone:  "America/Guatemala",
	},
	"MGPG": {
		ICAO:      "MGPG",
		IATA:      "PKJ",
		Name:      "Playa Grande Airport",
		City:      "Playa Grande",
		State:     "Quiche",
		Country:   "GT",
		Elevation: 577,
		Latitude:  15.9975004196,
		Longitude: -90.7416992188,
		Timezone:  "America/Guatemala",
	},
	"MGPP": {
		ICAO:      "MGPP",
		IATA:      "PON",
		Name:      "Poptun Airport",
		City:      "Poptun",
		State:     "Peten",
		Country:   "GT",
		Elevation: 1801,
		Latitude:  16.325799942,
		Longitude: -89.4160995483,
		Timezone:  "America/Guatemala",
	},
	"MGQC": {
		ICAO:      "MGQC",
		IATA:      "AQB",
		Name:      "Santa Cruz del Quiche Airport",
		City:      "Santa Cruz del Quiche",
		State:     "Quiche",
		Country:   "GT",
		Elevation: 6631,
		Latitude:  15.0122003555,
		Longitude: -91.1505966187,
		Timezone:  "America/Guatemala",
	},
	"MGQZ": {
		ICAO:      "MGQZ",
		IATA:      "AAZ",
		Name:      "Quezaltenango Airport",
		City:      "Quezaltenango",
		State:     "Quetzaltenango",
		Country:   "GT",
		Elevation: 7779,
		Latitude:  14.8655996323,
		Longitude: -91.5019989014,
		Timezone:  "America/Guatemala",
	},
	"MGRA": {
		ICAO:      "MGRA",
		Name:      "Rabinal Airport",
		City:      "Rabinal",
		State:     "Baja-Verapaz",
		Country:   "GT",
		Elevation: 3180,
		Latitude:  15.0935,
		Longitude: -90.5063888889,
		Timezone:  "America/Guatemala",
	},
	"MGRB": {
		ICAO:      "MGRB",
		IATA:      "RUV",
		Name:      "Rubelsanto Airport",
		City:      "Rubelsanto",
		State:     "Alta-Verapaz",
		Country:   "GT",
		Elevation: 426,
		Latitude:  15.9919996262,
		Longitude: -90.4452972412,
		Timezone:  "America/Guatemala",
	},
	"MGRD": {
		ICAO:      "MGRD",
		IATA:      "LCF",
		Name:      "Las Vegas Airport",
		City:      "Rio Dulce",
		State:     "Izabal",
		Country:   "GT",
		Elevation: 28,
		Latitude:  15.6688995361,
		Longitude: -88.957901001,
		Timezone:  "America/Guatemala",
	},
	"MGRT": {
		ICAO:      "MGRT",
		IATA:      "RER",
		Name:      "Retalhuleu Airport",
		City:      "Retalhuleu",
		State:     "Retalhuleu",
		Country:   "GT",
		Elevation: 656,
		Latitude:  14.5209999084,
		Longitude: -91.697303772,
		Timezone:  "America/Guatemala",
	},
	"MGSJ": {
		ICAO:      "MGSJ",
		IATA:      "GSJ",
		Name:      "San Jose Airport",
		City:      "Puerto San Jose",
		State:     "Escuintla",
		Country:   "GT",
		Elevation: 29,
		Latitude:  13.9362001419,
		Longitude: -90.8358001709,
		Timezone:  "America/Guatemala",
	},
	"MGSM": {
		ICAO:      "MGSM",
		Name:      "San Marcos Airport",
		City:      "San Marcos",
		State:     "San-Marcos",
		Country:   "GT",
		Elevation: 7933,
		Latitude:  14.9562997818,
		Longitude: -91.8062973022,
		Timezone:  "America/Guatemala",
	},
	"MGTK": {
		ICAO:      "MGTK",
		IATA:      "FRS",
		Name:      "Mundo Maya International Airport",
		City:      "San Benito",
		State:     "Peten",
		Country:   "GT",
		Elevation: 427,
		Latitude:  16.9137992859,
		Longitude: -89.8664016724,
		Timezone:  "America/Guatemala",
	},
	"MGUX": {
		ICAO:      "MGUX",
		IATA:      "UAX",
		Name:      "Uaxactun Airport",
		City:      "Uaxactun",
		State:     "Peten",
		Country:   "GT",
		Elevation: 573,
		Latitude:  17.3938884735,
		Longitude: -89.6327438354,
		Timezone:  "America/Guatemala",
	},
	"MGZA": {
		ICAO:      "MGZA",
		Name:      "Zacapa Airport",
		City:      "Zacapa",
		State:     "Zacapa",
		Country:   "GT",
		Elevation: 626,
		Latitude:  14.9603004456,
		Longitude: -89.5391998291,
		Timezone:  "America/Guatemala",
	},
	"MHAC": {
		ICAO:      "MHAC",
		Name:      "Acensa Airport",
		City:      "Acensa",
		State:     "Choluteca",
		Country:   "HN",
		Elevation: 42,
		Latitude:  13.2436113358,
		Longitude: -87.3413925171,
		Timezone:  "America/Tegucigalpa",
	},
	"MHAG": {
		ICAO:      "MHAG",
		Name:      "Sur Agropecuaria Airport",
		City:      "Buena Vista",
		State:     "Choluteca",
		Country:   "HN",
		Elevation: 12,
		Latitude:  13.25,
		Longitude: -87.3580551147,
		Timezone:  "America/Tegucigalpa",
	},
	"MHAH": {
		ICAO:      "MHAH",
		IATA:      "AHS",
		Name:      "Ahuas Airport",
		City:      "Ahuas",
		State:     "Gracias-a-Dios",
		Country:   "HN",
		Elevation: 98,
		Latitude:  15.4722003937,
		Longitude: -84.3522033691,
		Timezone:  "America/Tegucigalpa",
	},
	"MHAL": {
		ICAO:      "MHAL",
		Name:      "El Alto Airport",
		State:     "Francisco-Morazan",
		Country:   "HN",
		Elevation: 760,
		Latitude:  14.4933328629,
		Longitude: -87.1063919067,
		Timezone:  "America/Tegucigalpa",
	},
	"MHAM": {
		ICAO:      "MHAM",
		Name:      "Amapala Airport",
		City:      "Amapala Island",
		State:     "Valle",
		Country:   "HN",
		Elevation: 98,
		Latitude:  13.2683000565,
		Longitude: -87.6603012085,
		Timezone:  "America/El_Salvador",
	},
	"MHAN": {
		ICAO:      "MHAN",
		Name:      "San Luis Airport",
		City:      "San Luis",
		State:     "Comayagua",
		Country:   "HN",
		Elevation: 690,
		Latitude:  14.7458333969,
		Longitude: -87.4052810669,
		Timezone:  "America/Tegucigalpa",
	},
	"MHAP": {
		ICAO:      "MHAP",
		Name:      "Apala Airport",
		City:      "Apala",
		State:     "El-Paraiso",
		Country:   "HN",
		Elevation: 470,
		Latitude:  14.0108327866,
		Longitude: -86.3891677856,
		Timezone:  "America/Tegucigalpa",
	},
	"MHAR": {
		ICAO:      "MHAR",
		Name:      "Archaga Airport",
		City:      "Archaga",
		State:     "Francisco-Morazan",
		Country:   "HN",
		Elevation: 875,
		Latitude:  14.2852783203,
		Longitude: -87.2291641235,
		Timezone:  "America/Tegucigalpa",
	},
	"MHAS": {
		ICAO:      "MHAS",
		Name:      "Alas Del Socorro Airport",
		City:      "Siguatepeque",
		State:     "Comayagua",
		Country:   "HN",
		Elevation: 1068,
		Latitude:  14.5930557251,
		Longitude: -87.8405532837,
		Timezone:  "America/Tegucigalpa",
	},
	"MHAU": {
		ICAO:      "MHAU",
		Name:      "Hacienda Ulua Airport",
		City:      "Tasajeras",
		State:     "Olancho",
		Country:   "HN",
		Elevation: 710,
		Latitude:  14.7177782059,
		Longitude: -86.5708312988,
		Timezone:  "America/Tegucigalpa",
	},
	"MHAY": {
		ICAO:      "MHAY",
		Name:      "Araslaya Airport",
		City:      "Araslaya",
		State:     "Gracias-a-Dios",
		Country:   "HN",
		Elevation: 3,
		Latitude:  15.7352781296,
		Longitude: -84.5433349609,
		Timezone:  "America/Tegucigalpa",
	},
	"MHAZ": {
		ICAO:      "MHAZ",
		Name:      "Aserradero Azacualpa Airport",
		City:      "Cacao",
		State:     "Olancho",
		Country:   "HN",
		Elevation: 370,
		Latitude:  14.4333333969,
		Longitude: -86.0891647339,
		Timezone:  "America/Tegucigalpa",
	},
	"MHBA": {
		ICAO:      "MHBA",
		Name:      "Barbareta Airport",
		City:      "Barbareta Island",
		State:     "Bay-Islands",
		Country:   "HN",
		Elevation: 7,
		Latitude:  16.4330558777,
		Longitude: -86.1394424438,
		Timezone:  "America/Tegucigalpa",
	},
	"MHBE": {
		ICAO:      "MHBE",
		Name:      "San Bernardo Airport",
		City:      "San Bernardo",
		State:     "Choluteca",
		Country:   "HN",
		Elevation: 14,
		Latitude:  13.1019439697,
		Longitude: -87.1419448853,
		Timezone:  "America/Tegucigalpa",
	},
	"MHBL": {
		ICAO:      "MHBL",
		IATA:      "BHG",
		Name:      "Brus Laguna Airport",
		City:      "Brus Laguna",
		State:     "Gracias-a-Dios",
		Country:   "HN",
		Elevation: 2,
		Latitude:  15.7630996704,
		Longitude: -84.5436019897,
		Timezone:  "America/Tegucigalpa",
	},
	"MHCA": {
		ICAO:      "MHCA",
		IATA:      "CAA",
		Name:      "Catacamas Airport",
		City:      "Catacamas",
		State:     "Olancho",
		Country:   "HN",
		Elevation: 1489,
		Latitude:  14.9169998169,
		Longitude: -85.9000015259,
		Timezone:  "America/Tegucigalpa",
	},
	"MHCB": {
		ICAO:      "MHCB",
		Name:      "Cocobila Airport",
		City:      "Cocobila",
		State:     "Gracias-a-Dios",
		Country:   "HN",
		Elevation: 2,
		Latitude:  15.8883333206,
		Longitude: -84.7555541992,
		Timezone:  "America/Tegucigalpa",
	},
	"MHCC": {
		ICAO:      "MHCC",
		Name:      "Concepcion Airport",
		City:      "Concepcion",
		State:     "Intibuca",
		Country:   "HN",
		Elevation: 560,
		Latitude:  14.0266666412,
		Longitude: -88.3441696167,
		Timezone:  "America/Tegucigalpa",
	},
	"MHCG": {
		ICAO:      "MHCG",
		Name:      "Shaoguan Airport",
		City:      "Comayagua",
		State:     "Comayagua",
		Country:   "HN",
		Elevation: 1980,
		Latitude:  14.432299614,
		Longitude: -87.6246032715,
		Timezone:  "America/Tegucigalpa",
	},
	"MHCI": {
		ICAO:      "MHCI",
		Name:      "Chiquerito Airport",
		City:      "Chiquerito",
		State:     "Colon",
		Country:   "HN",
		Elevation: 30,
		Latitude:  15.8319444656,
		Longitude: -85.0269470215,
		Timezone:  "America/Tegucigalpa",
	},
	"MHCL": {
		ICAO:      "MHCL",
		Name:      "Colon Airport",
		City:      "Marcala",
		State:     "La-Paz",
		Country:   "HN",
		Elevation: 1240,
		Latitude:  14.1619443893,
		Longitude: -88.0344467163,
		Timezone:  "America/Tegucigalpa",
	},
	"MHCM": {
		ICAO:      "MHCM",
		Name:      "Choloma Airport",
		City:      "Choloma",
		State:     "Santa-Barbara",
		Country:   "HN",
		Elevation: 1100,
		Latitude:  14.8883333206,
		Longitude: -88.4044418335,
		Timezone:  "America/Tegucigalpa",
	},
	"MHCO": {
		ICAO:      "MHCO",
		Name:      "Coco Airport",
		State:     "Gracias-a-Dios",
		Country:   "HN",
		Elevation: 61,
		Latitude:  15.2527780533,
		Longitude: -84.2333297729,
		Timezone:  "America/Tegucigalpa",
	},
	"MHCP": {
		ICAO:      "MHCP",
		Name:      "Cochino Pequeno Airport",
		City:      "Cochino Pequeno Island",
		State:     "Bay-Islands",
		Country:   "HN",
		Elevation: 1,
		Latitude:  15.9544439316,
		Longitude: -86.4986114502,
		Timezone:  "America/Tegucigalpa",
	},
	"MHCR": {
		ICAO:      "MHCR",
		Name:      "Carta Airport",
		City:      "La Union",
		State:     "Olancho",
		Country:   "HN",
		Elevation: 800,
		Latitude:  15.0293998718,
		Longitude: -86.6923980713,
		Timezone:  "America/Tegucigalpa",
	},
	"MHCS": {
		ICAO:      "MHCS",
		IATA:      "CYL",
		Name:      "Coyoles Airport",
		City:      "Coyoles",
		State:     "Yoro",
		Country:   "HN",
		Elevation: 150,
		Latitude:  15.445555687,
		Longitude: -86.67527771,
		Timezone:  "America/Tegucigalpa",
	},
	"MHCU": {
		ICAO:      "MHCU",
		IATA:      "CDD",
		Name:      "Cauquira Airport",
		City:      "Cauquira",
		State:     "Gracias-a-Dios",
		Country:   "HN",
		Elevation: 21,
		Latitude:  15.3166666031,
		Longitude: -83.5916671753,
		Timezone:  "America/Tegucigalpa",
	},
	"MHDU": {
		ICAO:      "MHDU",
		Name:      "Durzona Airport",
		City:      "Mocoron",
		State:     "Gracias-a-Dios",
		Country:   "HN",
		Elevation: 250,
		Latitude:  14.9891004562,
		Longitude: -84.2208023071,
		Timezone:  "America/Tegucigalpa",
	},
	"MHEA": {
		ICAO:      "MHEA",
		IATA:      "OAN",
		Name:      "El Arrayan Airport",
		City:      "Olanchito",
		State:     "Yoro",
		Country:   "HN",
		Elevation: 220,
		Latitude:  15.5055561066,
		Longitude: -86.57472229,
		Timezone:  "America/Tegucigalpa",
	},
	"MHEC": {
		ICAO:      "MHEC",
		Name:      "El Cubo Airport",
		State:     "Francisco-Morazan",
		Country:   "HN",
		Elevation: 780,
		Latitude:  14.457777977,
		Longitude: -87.0586090088,
		Timezone:  "America/Tegucigalpa",
	},
	"MHEI": {
		ICAO:      "MHEI",
		Name:      "La America Airport",
		City:      "El Guanacaste",
		State:     "Yoro",
		Country:   "HN",
		Elevation: 53,
		Latitude:  15.6486110687,
		Longitude: -87.7883300781,
		Timezone:  "America/Tegucigalpa",
	},
	"MHEL": {
		ICAO:      "MHEL",
		Name:      "Barrio El Aterrizaje Airport",
		City:      "El Paraiso",
		State:     "El-Paraiso",
		Country:   "HN",
		Elevation: 760,
		Latitude:  13.8730564117,
		Longitude: -86.5605545044,
		Timezone:  "America/Tegucigalpa",
	},
	"MHEN": {
		ICAO:      "MHEN",
		Name:      "Aserradero El Encino Airport",
		City:      "La Canoa",
		State:     "Francisco-Morazan",
		Country:   "HN",
		Elevation: 970,
		Latitude:  14.6619443893,
		Longitude: -86.9150009155,
		Timezone:  "America/Tegucigalpa",
	},
	"MHEZ": {
		ICAO:      "MHEZ",
		Name:      "La Esperanza Airport",
		City:      "Lerida",
		State:     "Colon",
		Country:   "HN",
		Elevation: 60,
		Latitude:  15.6097221375,
		Longitude: -86.1394424438,
		Timezone:  "America/Tegucigalpa",
	},
	"MHFA": {
		ICAO:      "MHFA",
		Name:      "Finca San Antonio Airport",
		City:      "La Lima",
		State:     "Olancho",
		Country:   "HN",
		Elevation: 646,
		Latitude:  14.5980558395,
		Longitude: -86.4838867188,
		Timezone:  "America/Tegucigalpa",
	},
	"MHFC": {
		ICAO:      "MHFC",
		Name:      "Fort Cay Airport",
		City:      "Fort Cay",
		State:     "Bay-Islands",
		Country:   "HN",
		Elevation: 5,
		Latitude:  16.403055191,
		Longitude: -86.2844467163,
		Timezone:  "America/Tegucigalpa",
	},
	"MHFD": {
		ICAO:      "MHFD",
		Name:      "Finca 12 Airport",
		City:      "Agua Blanca",
		State:     "Yoro",
		Country:   "HN",
		Elevation: 116,
		Latitude:  15.2697219849,
		Longitude: -87.9027786255,
		Timezone:  "America/Tegucigalpa",
	},
	"MHFN": {
		ICAO:      "MHFN",
		Name:      "San Fernando Airport",
		City:      "San Fernando",
		State:     "El-Paraiso",
		Country:   "HN",
		Elevation: 648,
		Latitude:  14.1133327484,
		Longitude: -86.9527816772,
		Timezone:  "America/Tegucigalpa",
	},
	"MHGA": {
		ICAO:      "MHGA",
		Name:      "Chumbagua Airport",
		City:      "Chumbagua",
		State:     "Santa-Barbara",
		Country:   "HN",
		Elevation: 300,
		Latitude:  15.1766672134,
		Longitude: -88.4899978638,
		Timezone:  "America/Tegucigalpa",
	},
	"MHGB": {
		ICAO:      "MHGB",
		Name:      "Guayabillas Airport",
		City:      "Esquias",
		State:     "Comayagua",
		Country:   "HN",
		Elevation: 700,
		Latitude:  14.7466669083,
		Longitude: -87.3752746582,
		Timezone:  "America/Tegucigalpa",
	},
	"MHGC": {
		ICAO:      "MHGC",
		Name:      "Grupo Carnol Airport",
		City:      "Catacamas",
		State:     "Olancho",
		Country:   "HN",
		Elevation: 378,
		Latitude:  14.8386106491,
		Longitude: -85.8683319092,
		Timezone:  "America/Tegucigalpa",
	},
	"MHGE": {
		ICAO:      "MHGE",
		Name:      "El Aguacate Airport",
		City:      "El Aguacate",
		State:     "Olancho",
		Country:   "HN",
		Elevation: 384,
		Latitude:  14.875,
		Longitude: -85.7761077881,
		Timezone:  "America/Tegucigalpa",
	},
	"MHGG": {
		ICAO:      "MHGG",
		Name:      "Santiago De Puringla Airport",
		City:      "Santiago de Puringla",
		State:     "Intibuca",
		Country:   "HN",
		Elevation: 960,
		Latitude:  14.3675003052,
		Longitude: -87.9113922119,
		Timezone:  "America/Tegucigalpa",
	},
	"MHGR": {
		ICAO:      "MHGR",
		Name:      "Sangrelaya Airport",
		City:      "Donel",
		State:     "Colon",
		Country:   "HN",
		Elevation: 3,
		Latitude:  15.9699001312,
		Longitude: -85.0892028809,
		Timezone:  "America/Tegucigalpa",
	},
	"MHGS": {
		ICAO:      "MHGS",
		IATA:      "GAC",
		Name:      "Gracias Airport",
		City:      "El Molino",
		State:     "Lempira",
		Country:   "HN",
		Elevation: 828,
		Latitude:  14.5966672897,
		Longitude: -88.5941696167,
		Timezone:  "America/Tegucigalpa",
	},
	"MHGY": {
		ICAO:      "MHGY",
		Name:      "Guayape Airport",
		City:      "El Zapato",
		State:     "Olancho",
		Country:   "HN",
		Elevation: 765,
		Latitude:  14.7847223282,
		Longitude: -86.8633346558,
		Timezone:  "America/Tegucigalpa",
	},
	"MHHE": {
		ICAO:      "MHHE",
		Name:      "La Herradura Airport",
		City:      "Punuare",
		State:     "Olancho",
		Country:   "HN",
		Elevation: 342,
		Latitude:  14.7550001144,
		Longitude: -86.0041656494,
		Timezone:  "America/Tegucigalpa",
	},
	"MHHG": {
		ICAO:      "MHHG",
		Name:      "Hacienda Galeras Airport",
		City:      "La Lima",
		State:     "Olancho",
		Country:   "HN",
		Elevation: 618,
		Latitude:  14.5988893509,
		Longitude: -86.4608306885,
		Timezone:  "America/Tegucigalpa",
	},
	"MHHO": {
		ICAO:      "MHHO",
		Name:      "Agua Caliente Airport",
		City:      "Horcones",
		State:     "Yoro",
		Country:   "HN",
		Elevation: 250,
		Latitude:  15.427778244,
		Longitude: -86.8949966431,
		Timezone:  "America/Tegucigalpa",
	},
	"MHIC": {
		ICAO:      "MHIC",
		Name:      "Islas Del Cisne Airport",
		City:      "Del Cisne Island",
		State:     "Bay-Islands",
		Country:   "HN",
		Elevation: 7,
		Latitude:  17.4073009491,
		Longitude: -83.9327011108,
		Timezone:  "America/Tegucigalpa",
	},
	"MHIN": {
		ICAO:      "MHIN",
		Name:      "Minas De Oro Airport",
		City:      "Minas de Oro",
		State:     "Comayagua",
		Country:   "HN",
		Elevation: 1000,
		Latitude:  14.788611412,
		Longitude: -87.3497238159,
		Timezone:  "America/Tegucigalpa",
	},
	"MHIR": {
		ICAO:      "MHIR",
		IATA:      "IRN",
		Name:      "Iriona Airport",
		City:      "Iriona",
		State:     "Colon",
		Country:   "HN",
		Elevation: 8,
		Latitude:  15.9391670227,
		Longitude: -85.13722229,
		Timezone:  "America/Tegucigalpa",
	},
	"MHIZ": {
		ICAO:      "MHIZ",
		Name:      "Izapan Airport",
		City:      "La Puerta",
		State:     "Colon",
		Country:   "HN",
		Elevation: 8,
		Latitude:  15.9102783203,
		Longitude: -85.1711120605,
		Timezone:  "America/Tegucigalpa",
	},
	"MHJC": {
		ICAO:      "MHJC",
		Name:      "Jocon Airport",
		City:      "Jocon",
		State:     "Yoro",
		Country:   "HN",
		Elevation: 490,
		Latitude:  15.290555954,
		Longitude: -86.9122238159,
		Timezone:  "America/Tegucigalpa",
	},
	"MHJE": {
		ICAO:      "MHJE",
		Name:      "Nueva Jerusalen Airport",
		City:      "Rio Platano",
		State:     "Gracias-a-Dios",
		Country:   "HN",
		Elevation: 5,
		Latitude:  15.8780555725,
		Longitude: -84.7263870239,
		Timezone:  "America/Tegucigalpa",
	},
	"MHJI": {
		ICAO:      "MHJI",
		Name:      "Jicalapa Airport",
		City:      "Jicalapa",
		State:     "Olancho",
		Country:   "HN",
		Elevation: 650,
		Latitude:  15.006111145,
		Longitude: -86.0491638184,
		Timezone:  "America/Tegucigalpa",
	},
	"MHJO": {
		ICAO:      "MHJO",
		Name:      "La Joya Airport",
		State:     "Francisco-Morazan",
		Country:   "HN",
		Elevation: 780,
		Latitude:  14.8047218323,
		Longitude: -86.9191665649,
		Timezone:  "America/Tegucigalpa",
	},
	"MHJQ": {
		ICAO:      "MHJQ",
		Name:      "Joya Del Quebracho Airport",
		City:      "Caleras/El Triunfo",
		State:     "Francisco-Morazan",
		Country:   "HN",
		Elevation: 890,
		Latitude:  14.838889122,
		Longitude: -86.9622192383,
		Timezone:  "America/Tegucigalpa",
	},
	"MHJU": {
		ICAO:      "MHJU",
		IATA:      "JUT",
		Name:      "Jutigalpa airport",
		City:      "Jutigalpa",
		State:     "Olancho",
		Country:   "HN",
		Elevation: 1314,
		Latitude:  14.6526002884,
		Longitude: -86.2202987671,
		Timezone:  "America/Tegucigalpa",
	},
	"MHLA": {
		ICAO:      "MHLA",
		Name:      "La Alondra Airport",
		City:      "Candelaria",
		State:     "Lempira",
		Country:   "HN",
		Elevation: 1000,
		Latitude:  14.1088886261,
		Longitude: -88.5338897705,
		Timezone:  "America/Tegucigalpa",
	},
	"MHLC": {
		ICAO:      "MHLC",
		IATA:      "LCE",
		Name:      "Goloson International Airport",
		City:      "La Ceiba",
		State:     "Atlantida",
		Country:   "HN",
		Elevation: 49,
		Latitude:  15.7425003052,
		Longitude: -86.8529968262,
		Timezone:  "America/Tegucigalpa",
	},
	"MHLE": {
		ICAO:      "MHLE",
		IATA:      "LEZ",
		Name:      "La Esperanza Airport",
		City:      "La Esperanza",
		State:     "Intibuca",
		Country:   "HN",
		Elevation: 5475,
		Latitude:  14.2911109924,
		Longitude: -88.1750030518,
		Timezone:  "America/Tegucigalpa",
	},
	"MHLF": {
		ICAO:      "MHLF",
		Name:      "Flefil Airport",
		State:     "Francisco-Morazan",
		Country:   "HN",
		Elevation: 810,
		Latitude:  14.423333168,
		Longitude: -87.119720459,
		Timezone:  "America/Tegucigalpa",
	},
	"MHLG": {
		ICAO:      "MHLG",
		Name:      "La Grecia Airport",
		City:      "Monjaras",
		State:     "Choluteca",
		Country:   "HN",
		Elevation: 29,
		Latitude:  13.2147216797,
		Longitude: -87.363609314,
		Timezone:  "America/Tegucigalpa",
	},
	"MHLI": {
		ICAO:      "MHLI",
		Name:      "Talanguita Airport",
		State:     "Francisco-Morazan",
		Country:   "HN",
		Elevation: 760,
		Latitude:  14.4875001907,
		Longitude: -87.179725647,
		Timezone:  "America/Tegucigalpa",
	},
	"MHLJ": {
		ICAO:      "MHLJ",
		Name:      "Las Lajas Airport",
		City:      "Coyoles",
		State:     "Olancho",
		Country:   "HN",
		Elevation: 1050,
		Latitude:  14.8947219849,
		Longitude: -86.57472229,
		Timezone:  "America/Tegucigalpa",
	},
	"MHLM": {
		ICAO:      "MHLM",
		IATA:      "SAP",
		Name:      "Ramon Villeda Morales International Airport",
		City:      "La Mesa",
		State:     "Cortes",
		Country:   "HN",
		Elevation: 91,
		Latitude:  15.4525995255,
		Longitude: -87.9235992432,
		Timezone:  "America/Tegucigalpa",
	},
	"MHLN": {
		ICAO:      "MHLN",
		Name:      "Limon Airport",
		City:      "Limon",
		State:     "Colon",
		Country:   "HN",
		Elevation: 3,
		Latitude:  15.8633327484,
		Longitude: -85.4986114502,
		Timezone:  "America/Tegucigalpa",
	},
	"MHLP": {
		ICAO:      "MHLP",
		Name:      "Mapulaca Airport",
		City:      "Mapulaca",
		State:     "Lempira",
		Country:   "HN",
		Elevation: 170,
		Latitude:  14.0341672897,
		Longitude: -88.6283340454,
		Timezone:  "America/El_Salvador",
	},
	"MHLS": {
		ICAO:      "MHLS",
		Name:      "Laguna Seca Airport",
		City:      "Laguna Seca",
		State:     "Olancho",
		Country:   "HN",
		Elevation: 353,
		Latitude:  14.6447000504,
		Longitude: -86.0698013306,
		Timezone:  "America/Tegucigalpa",
	},
	"MHLT": {
		ICAO:      "MHLT",
		Name:      "Llanos Del Tigre Airport",
		City:      "Moroceli",
		State:     "El-Paraiso",
		Country:   "HN",
		Elevation: 625,
		Latitude:  14.076666832,
		Longitude: -86.8416671753,
		Timezone:  "America/Tegucigalpa",
	},
	"MHLU": {
		ICAO:      "MHLU",
		Name:      "Luz Y Vida Airport",
		City:      "San Luis Pajon",
		State:     "Santa-Barbara",
		Country:   "HN",
		Elevation: 764,
		Latitude:  15.1463890076,
		Longitude: -88.444442749,
		Timezone:  "America/Tegucigalpa",
	},
	"MHLV": {
		ICAO:      "MHLV",
		Name:      "Las Vegas Airport",
		City:      "Las Vegas",
		State:     "Santa-Barbara",
		Country:   "HN",
		Elevation: 870,
		Latitude:  14.8736114502,
		Longitude: -88.0744476318,
		Timezone:  "America/Tegucigalpa",
	},
	"MHLZ": {
		ICAO:      "MHLZ",
		Name:      "La Estanzuela Airport",
		City:      "el agua",
		State:     "Copan",
		Country:   "HN",
		Elevation: 2227,
		Latitude:  15.016667366,
		Longitude: -88.8000030518,
		Timezone:  "America/Tegucigalpa",
	},
	"MHMI": {
		ICAO:      "MHMI",
		Name:      "Las Minitas Airport",
		City:      "Vallecillo",
		State:     "Yoro",
		Country:   "HN",
		Elevation: 2500,
		Latitude:  15.0511112213,
		Longitude: -87.2466659546,
		Timezone:  "America/Tegucigalpa",
	},
	"MHML": {
		ICAO:      "MHML",
		Name:      "Monte Libano Airport",
		City:      "Monte Libano",
		State:     "Choluteca",
		Country:   "HN",
		Elevation: 46,
		Latitude:  13.179444313,
		Longitude: -87.1855545044,
		Timezone:  "America/Tegucigalpa",
	},
	"MHMM": {
		ICAO:      "MHMM",
		Name:      "San Antonio Malera Airport",
		City:      "Malera",
		State:     "Santa-Barbara",
		Country:   "HN",
		Elevation: 670,
		Latitude:  14.6719436646,
		Longitude: -88.2549972534,
		Timezone:  "America/Tegucigalpa",
	},
	"MHMN": {
		ICAO:      "MHMN",
		Name:      "Monica Airport",
		City:      "Monica",
		State:     "Atlantida",
		Country:   "HN",
		Elevation: 59,
		Latitude:  15.6529998779,
		Longitude: -87.0876998901,
		Timezone:  "America/Tegucigalpa",
	},
	"MHMO": {
		ICAO:      "MHMO",
		Name:      "San Marcos De Ocotepeque Airport",
		City:      "San Marcos",
		State:     "Ocotepeque",
		Country:   "HN",
		Elevation: 961,
		Latitude:  14.4130563736,
		Longitude: -88.9519424438,
		Timezone:  "America/Tegucigalpa",
	},
	"MHMP": {
		ICAO:      "MHMP",
		Name:      "Marcos Perez Airport",
		City:      "Agua Blanca",
		State:     "Yoro",
		Country:   "HN",
		Elevation: 115,
		Latitude:  15.286110878,
		Longitude: -87.8852767944,
		Timezone:  "America/Tegucigalpa",
	},
	"MHMS": {
		ICAO:      "MHMS",
		Name:      "Las Marias Airport",
		City:      "Siksatara",
		State:     "Gracias-a-Dios",
		Country:   "HN",
		Elevation: 30,
		Latitude:  15.6727781296,
		Longitude: -84.8447189331,
		Timezone:  "America/Tegucigalpa",
	},
	"MHMT": {
		ICAO:      "MHMT",
		Name:      "Manto Airport",
		City:      "Manto",
		State:     "Olancho",
		Country:   "HN",
		Elevation: 650,
		Latitude:  14.915555954,
		Longitude: -86.3816680908,
		Timezone:  "America/Tegucigalpa",
	},
	"MHNA": {
		ICAO:      "MHNA",
		Name:      "Ciudad Nacaome Airport",
		City:      "Nacaome",
		State:     "Choluteca",
		Country:   "HN",
		Elevation: 35,
		Latitude:  13.5294437408,
		Longitude: -87.314163208,
		Timezone:  "America/Tegucigalpa",
	},
	"MHNB": {
		ICAO:      "MHNB",
		Name:      "Noveno Batallon Airport",
		City:      "Apala",
		State:     "El-Paraiso",
		Country:   "HN",
		Elevation: 426,
		Latitude:  14.043299675,
		Longitude: -86.4207992554,
		Timezone:  "America/Tegucigalpa",
	},
	"MHNC": {
		ICAO:      "MHNC",
		Name:      "Nueva Choluteca Airport",
		City:      "Choluteca",
		State:     "Choluteca",
		Country:   "HN",
		Elevation: 52,
		Latitude:  13.3233327866,
		Longitude: -87.1425018311,
		Timezone:  "America/Tegucigalpa",
	},
	"MHNJ": {
		ICAO:      "MHNJ",
		IATA:      "GJA",
		Name:      "La Laguna Airport",
		City:      "Guanaja",
		State:     "Bay-Islands",
		Country:   "HN",
		Elevation: 49,
		Latitude:  16.445400238,
		Longitude: -85.9066009521,
		Timezone:  "America/Tegucigalpa",
	},
	"MHNS": {
		ICAO:      "MHNS",
		Name:      "Los Llanos Airport",
		State:     "La-Paz",
		Country:   "HN",
		Elevation: 1180,
		Latitude:  14.2105560303,
		Longitude: -88.0216674805,
		Timezone:  "America/Tegucigalpa",
	},
	"MHNV": {
		ICAO:      "MHNV",
		Name:      "Nueva Ocotepeque Airport",
		City:      "Nueva Ocotepeque",
		State:     "Ocotepeque",
		Country:   "HN",
		Elevation: 850,
		Latitude:  14.4288892746,
		Longitude: -89.1938858032,
		Timezone:  "America/Tegucigalpa",
	},
	"MHOT": {
		ICAO:      "MHOT",
		Name:      "Ocotales Airport",
		City:      "Copon",
		State:     "Colon",
		Country:   "HN",
		Elevation: 130,
		Latitude:  15.6691665649,
		Longitude: -85.194442749,
		Timezone:  "America/Tegucigalpa",
	},
	"MHPA": {
		ICAO:      "MHPA",
		Name:      "Barra Del Patuca Airport",
		City:      "Barra del Patuca",
		State:     "Gracias-a-Dios",
		Country:   "HN",
		Elevation: 4,
		Latitude:  15.8013887405,
		Longitude: -84.2966690063,
		Timezone:  "America/Tegucigalpa",
	},
	"MHPC": {
		ICAO:      "MHPC",
		IATA:      "PCH",
		Name:      "Palacios Airport",
		City:      "Palacios",
		State:     "Gracias-a-Dios",
		Country:   "HN",
		Elevation: 30,
		Latitude:  15.9549999237,
		Longitude: -84.9413909912,
		Timezone:  "America/Tegucigalpa",
	},
	"MHPI": {
		ICAO:      "MHPI",
		Name:      "Agropecuaria Piedra De Agua Azul Airport",
		City:      "Buena Vista",
		State:     "Choluteca",
		Country:   "HN",
		Elevation: 56,
		Latitude:  13.2961111069,
		Longitude: -87.3455581665,
		Timezone:  "America/Tegucigalpa",
	},
	"MHPL": {
		ICAO:      "MHPL",
		IATA:      "PEU",
		Name:      "Puerto Lempira Airport",
		City:      "Puerto Lempira",
		State:     "Gracias-a-Dios",
		Country:   "HN",
		Elevation: 25,
		Latitude:  15.2622003555,
		Longitude: -83.7811965942,
		Timezone:  "America/Tegucigalpa",
	},
	"MHPS": {
		ICAO:      "MHPS",
		Name:      "Desvio Potrerillos Airport",
		City:      "El Encinal",
		State:     "Olancho",
		Country:   "HN",
		Elevation: 395,
		Latitude:  14.5958328247,
		Longitude: -86.1500015259,
		Timezone:  "America/Tegucigalpa",
	},
	"MHPV": {
		ICAO:      "MHPV",
		Name:      "El Porvenir Airport",
		City:      "Manga",
		State:     "Colon",
		Country:   "HN",
		Elevation: 69,
		Latitude:  15.5302782059,
		Longitude: -86.2736129761,
		Timezone:  "America/Tegucigalpa",
	},
	"MHPY": {
		ICAO:      "MHPY",
		Name:      "Payasito Airport",
		City:      "Farallones",
		State:     "Colon",
		Country:   "HN",
		Elevation: 5,
		Latitude:  15.8758325577,
		Longitude: -85.3969421387,
		Timezone:  "America/Tegucigalpa",
	},
	"MHRA": {
		ICAO:      "MHRA",
		Name:      "Rapaco II Airport",
		State:     "El-Paraiso",
		Country:   "HN",
		Elevation: 645,
		Latitude:  14.0736112595,
		Longitude: -86.9100036621,
		Timezone:  "America/Tegucigalpa",
	},
	"MHRD": {
		ICAO:      "MHRD",
		Name:      "Rus Rus II Airport",
		City:      "Rus Rus",
		State:     "Gracias-a-Dios",
		Country:   "HN",
		Elevation: 173,
		Latitude:  14.7872219086,
		Longitude: -84.369720459,
		Timezone:  "America/Tegucigalpa",
	},
	"MHRH": {
		ICAO:      "MHRH",
		Name:      "Regional Hamer Airport",
		State:     "Colon",
		Country:   "HN",
		Elevation: 202,
		Latitude:  15.5633325577,
		Longitude: -86.1913909912,
		Timezone:  "America/Tegucigalpa",
	},
	"MHRJ": {
		ICAO:      "MHRJ",
		Name:      "Rancho Jamastran Airport",
		City:      "Apala",
		State:     "El-Paraiso",
		Country:   "HN",
		Elevation: 482,
		Latitude:  14.0609998703,
		Longitude: -86.4075012207,
		Timezone:  "America/Tegucigalpa",
	},
	"MHRO": {
		ICAO:      "MHRO",
		IATA:      "RTB",
		Name:      "Juan Manuel Galvez International Airport",
		City:      "Roatan Island",
		State:     "Bay-Islands",
		Country:   "HN",
		Elevation: 18,
		Latitude:  16.3167991638,
		Longitude: -86.5230026245,
		Timezone:  "America/Tegucigalpa",
	},
	"MHRR": {
		ICAO:      "MHRR",
		Name:      "Rus Rus I Airport",
		City:      "Rus Rus",
		State:     "Gracias-a-Dios",
		Country:   "HN",
		Elevation: 90,
		Latitude:  14.7130556107,
		Longitude: -84.4524993896,
		Timezone:  "America/Tegucigalpa",
	},
	"MHRS": {
		ICAO:      "MHRS",
		IATA:      "SDH",
		Name:      "Santa Rosa Copan Airport",
		City:      "Santa Rosa de Copan",
		State:     "Copan",
		Country:   "HN",
		Elevation: 2818,
		Latitude:  14.8149003983,
		Longitude: -88.7515029907,
		Timezone:  "America/Tegucigalpa",
	},
	"MHRU": {
		ICAO:      "MHRU",
		IATA:      "RUY",
		Name:      "Copan Ruinas Airport",
		City:      "Ruinas de Copan",
		State:     "Copan",
		Country:   "HN",
		Elevation: 1990,
		Latitude:  14.8402996063,
		Longitude: -89.1439971924,
		Timezone:  "America/Tegucigalpa",
	},
	"MHRY": {
		ICAO:      "MHRY",
		Name:      "Raya Airport",
		City:      "Raya",
		State:     "Gracias-a-Dios",
		Country:   "HN",
		Elevation: 30,
		Latitude:  15.0663890839,
		Longitude: -83.2972183228,
		Timezone:  "America/Tegucigalpa",
	},
	"MHSC": {
		ICAO:      "MHSC",
		IATA:      "XPL",
		Name:      "Coronel Enrique Soto Cano Air Base",
		City:      "Comayagua",
		State:     "Comayagua",
		Country:   "HN",
		Elevation: 2061,
		Latitude:  14.382399559,
		Longitude: -87.6212005615,
		Timezone:  "America/Tegucigalpa",
	},
	"MHSI": {
		ICAO:      "MHSI",
		Name:      "Sico Airport",
		City:      "Sico",
		State:     "Colon",
		Country:   "HN",
		Elevation: 20,
		Latitude:  15.818611145,
		Longitude: -85.114440918,
		Timezone:  "America/Tegucigalpa",
	},
	"MHSJ": {
		ICAO:      "MHSJ",
		Name:      "San Jose Airport",
		City:      "Oropoli",
		State:     "El-Paraiso",
		Country:   "HN",
		Elevation: 454,
		Latitude:  13.8277778625,
		Longitude: -86.8102798462,
		Timezone:  "America/Tegucigalpa",
	},
	"MHSL": {
		ICAO:      "MHSL",
		Name:      "San Lorenzo Airport",
		City:      "El Hato",
		State:     "Valle",
		Country:   "HN",
		Elevation: 8,
		Latitude:  13.4422216415,
		Longitude: -87.4597244263,
		Timezone:  "America/Tegucigalpa",
	},
	"MHSN": {
		ICAO:      "MHSN",
		Name:      "Sinaloa Airport",
		City:      "Sinaloa",
		State:     "Colon",
		Country:   "HN",
		Elevation: 20,
		Latitude:  15.6933326721,
		Longitude: -85.9552764893,
		Timezone:  "America/Tegucigalpa",
	},
	"MHSX": {
		ICAO:      "MHSX",
		Name:      "Sixatigni Airport",
		State:     "Gracias-a-Dios",
		Country:   "HN",
		Elevation: 70,
		Latitude:  15.1836109161,
		Longitude: -84.369720459,
		Timezone:  "America/Tegucigalpa",
	},
	"MHSZ": {
		ICAO:      "MHSZ",
		Name:      "Santa Barbara Airport",
		City:      "Choluteca",
		State:     "Choluteca",
		Country:   "HN",
		Elevation: 28,
		Latitude:  13.3305559158,
		Longitude: -87.1544418335,
		Timezone:  "America/Tegucigalpa",
	},
	"MHTA": {
		ICAO:      "MHTA",
		Name:      "Tamara Airport",
		City:      "Tamara",
		State:     "Francisco-Morazan",
		Country:   "HN",
		Elevation: 1080,
		Latitude:  14.1800003052,
		Longitude: -87.3499984741,
		Timezone:  "America/Tegucigalpa",
	},
	"MHTB": {
		ICAO:      "MHTB",
		Name:      "La Katabila Airport",
		City:      "Tuntuntara",
		State:     "Gracias-a-Dios",
		Country:   "HN",
		Elevation: 14,
		Latitude:  15.1750001907,
		Longitude: -83.682220459,
		Timezone:  "America/Tegucigalpa",
	},
	"MHTE": {
		ICAO:      "MHTE",
		IATA:      "TEA",
		Name:      "Tela Airport",
		City:      "Tela",
		State:     "Atlantida",
		Country:   "HN",
		Elevation: 7,
		Latitude:  15.7758998871,
		Longitude: -87.4757995605,
		Timezone:  "America/Tegucigalpa",
	},
	"MHTG": {
		ICAO:      "MHTG",
		IATA:      "TGU",
		Name:      "Toncontin International Airport",
		City:      "Tegucigalpa",
		State:     "Francisco-Morazan",
		Country:   "HN",
		Elevation: 3294,
		Latitude:  14.0608997345,
		Longitude: -87.2172012329,
		Timezone:  "America/Tegucigalpa",
	},
	"MHTI": {
		ICAO:      "MHTI",
		Name:      "Tipimuratara Airport",
		City:      "Tipimunatara",
		State:     "Gracias-a-Dios",
		Country:   "HN",
		Elevation: 43,
		Latitude:  15.0011110306,
		Longitude: -83.7222213745,
		Timezone:  "America/Tegucigalpa",
	},
	"MHTJ": {
		ICAO:      "MHTJ",
		IATA:      "TJI",
		Name:      "Trujillo Airport",
		City:      "Trujillo",
		State:     "Colon",
		Country:   "HN",
		Elevation: 3,
		Latitude:  15.9267997742,
		Longitude: -85.9382019043,
		Timezone:  "America/Tegucigalpa",
	},
	"MHTM": {
		ICAO:      "MHTM",
		Name:      "Santa Maria Airport",
		City:      "Santa Maria",
		State:     "La-Paz",
		Country:   "HN",
		Elevation: 1151,
		Latitude:  14.2772216797,
		Longitude: -87.936668396,
		Timezone:  "America/Tegucigalpa",
	},
	"MHTO": {
		ICAO:      "MHTO",
		Name:      "Tocoa Airport",
		City:      "Tocoa",
		State:     "Colon",
		Country:   "HN",
		Elevation: 37,
		Latitude:  15.6597223282,
		Longitude: -85.9941635132,
		Timezone:  "America/Tegucigalpa",
	},
	"MHUC": {
		ICAO:      "MHUC",
		Name:      "Auca Airport",
		City:      "Auca",
		State:     "Gracias-a-Dios",
		Country:   "HN",
		Elevation: 190,
		Latitude:  14.9372215271,
		Longitude: -83.8441696167,
		Timezone:  "America/Tegucigalpa",
	},
	"MHUG": {
		ICAO:      "MHUG",
		Name:      "Guanacastal Airport",
		City:      "Canchias",
		State:     "Comayagua",
		Country:   "HN",
		Elevation: 1563,
		Latitude:  14.915555954,
		Longitude: -87.8333358765,
		Timezone:  "America/Tegucigalpa",
	},
	"MHUH": {
		ICAO:      "MHUH",
		Name:      "Uhi Airport",
		City:      "Uhi",
		State:     "Gracias-a-Dios",
		Country:   "HN",
		Elevation: 3,
		Latitude:  15.476099968,
		Longitude: -83.9113006592,
		Timezone:  "America/Tegucigalpa",
	},
	"MHUL": {
		ICAO:      "MHUL",
		IATA:      "SCD",
		Name:      "Sulaco Airport",
		City:      "Sulaco",
		State:     "Comayagua",
		Country:   "HN",
		Elevation: 400,
		Latitude:  14.9071998596,
		Longitude: -87.2633972168,
		Timezone:  "America/Tegucigalpa",
	},
	"MHUO": {
		ICAO:      "MHUO",
		Name:      "La Union Airport",
		City:      "La Union",
		State:     "Lempira",
		Country:   "HN",
		Elevation: 1000,
		Latitude:  14.8080558777,
		Longitude: -88.4199981689,
		Timezone:  "America/Tegucigalpa",
	},
	"MHUT": {
		ICAO:      "MHUT",
		IATA:      "UII",
		Name:      "Utila Airport",
		City:      "Utila Island",
		State:     "Bay-Islands",
		Country:   "HN",
		Elevation: 29,
		Latitude:  16.1131000519,
		Longitude: -86.8803024292,
		Timezone:  "America/Tegucigalpa",
	},
	"MHUY": {
		ICAO:      "MHUY",
		Name:      "Cucuyagua Airport",
		City:      "Cucuyagua",
		State:     "Copan",
		Country:   "HN",
		Elevation: 805,
		Latitude:  14.6269435883,
		Longitude: -88.8763885498,
		Timezone:  "America/Tegucigalpa",
	},
	"MHVE": {
		ICAO:      "MHVE",
		Name:      "Villa Hermosa Airport",
		City:      "Coyolillo",
		State:     "Lempira",
		Country:   "HN",
		Elevation: 1280,
		Latitude:  14.2302780151,
		Longitude: -88.799446106,
		Timezone:  "America/Tegucigalpa",
	},
	"MHVG": {
		ICAO:      "MHVG",
		Name:      "Villa Guadalupe Airport",
		City:      "Villa Guadalupe",
		State:     "Santa-Barbara",
		Country:   "HN",
		Elevation: 400,
		Latitude:  15.0455560684,
		Longitude: -88.3088912964,
		Timezone:  "America/Tegucigalpa",
	},
	"MHWA": {
		ICAO:      "MHWA",
		Name:      "Wampusirpi I Airport",
		City:      "Wampusirpi",
		State:     "Gracias-a-Dios",
		Country:   "HN",
		Elevation: 250,
		Latitude:  15.1602783203,
		Longitude: -84.617225647,
		Timezone:  "America/Tegucigalpa",
	},
	"MHWD": {
		ICAO:      "MHWD",
		Name:      "Wampusirpi II Airport",
		City:      "Wampusirpi",
		State:     "Gracias-a-Dios",
		Country:   "HN",
		Elevation: 250,
		Latitude:  15.1591672897,
		Longitude: -84.5983352661,
		Timezone:  "America/Tegucigalpa",
	},
	"MHWP": {
		ICAO:      "MHWP",
		Name:      "Waplaya Airport",
		City:      "Waplaya",
		State:     "Gracias-a-Dios",
		Country:   "HN",
		Elevation: 43,
		Latitude:  15.2130556107,
		Longitude: -84.068611145,
		Timezone:  "America/Tegucigalpa",
	},
	"MHWR": {
		ICAO:      "MHWR",
		Name:      "Warunta Airport",
		City:      "Warunta",
		State:     "Gracias-a-Dios",
		Country:   "HN",
		Elevation: 30,
		Latitude:  15.348610878,
		Longitude: -84.2344436646,
		Timezone:  "America/Tegucigalpa",
	},
	"MHWS": {
		ICAO:      "MHWS",
		Name:      "Wasma Airport",
		City:      "Wasma",
		State:     "Gracias-a-Dios",
		Country:   "HN",
		Elevation: 15,
		Latitude:  15.4819002151,
		Longitude: -84.4160995483,
		Timezone:  "America/Tegucigalpa",
	},
	"MHWW": {
		ICAO:      "MHWW",
		Name:      "Wawina Airport",
		City:      "Wawina",
		State:     "Gracias-a-Dios",
		Country:   "HN",
		Elevation: 19,
		Latitude:  15.4161109924,
		Longitude: -84.4736099243,
		Timezone:  "America/Tegucigalpa",
	},
	"MHYX": {
		ICAO:      "MHYX",
		Name:      "Yaxu Airport",
		City:      "Puerto Lempira",
		State:     "Gracias-a-Dios",
		Country:   "HN",
		Elevation: 8,
		Latitude:  15.2775001526,
		Longitude: -83.820274353,
		Timezone:  "America/Tegucigalpa",
	},
	"MHZA": {
		ICAO:      "MHZA",
		Name:      "Azacualpa Airport",
		City:      "Azacualpa",
		State:     "Lempira",
		Country:   "HN",
		Elevation: 1200,
		Latitude:  14.2569437027,
		Longitude: -88.3925018311,
		Timezone:  "America/Tegucigalpa",
	},
	"MI00": {
		ICAO:      "MI00",
		Name:      "Bonnie Field",
		City:      "Rock",
		State:     "Michigan",
		Country:   "US",
		Elevation: 970,
		Latitude:  46.0522003174,
		Longitude: -87.2582015991,
		Timezone:  "America/Detroit",
	},
	"MI01": {
		ICAO:      "MI01",
		Name:      "Fasel Field",
		City:      "Avoca",
		State:     "Michigan",
		Country:   "US",
		Elevation: 735,
		Latitude:  43.057800293,
		Longitude: -82.676902771,
		Timezone:  "America/Detroit",
	},
	"MI02": {
		ICAO:      "MI02",
		Name:      "Brablec Farms Airport",
		City:      "Britton",
		State:     "Michigan",
		Country:   "US",
		Elevation: 691,
		Latitude:  41.9869995117,
		Longitude: -83.8047027588,
		Timezone:  "America/Detroit",
	},
	"MI04": {
		ICAO:      "MI04",
		Name:      "Airborn Paraflite Ultralightport",
		City:      "Ithaca",
		State:     "Michigan",
		Country:   "US",
		Elevation: 758,
		Latitude:  43.3069000244,
		Longitude: -84.6457977295,
		Timezone:  "America/Detroit",
	},
	"MI05": {
		ICAO:      "MI05",
		Name:      "Crompton's Private Strip",
		City:      "Clio",
		State:     "Michigan",
		Country:   "US",
		Elevation: 680,
		Latitude:  43.1666984558,
		Longitude: -83.7874984741,
		Timezone:  "America/Detroit",
	},
	"MI06": {
		ICAO:      "MI06",
		Name:      "Lee Field",
		City:      "Bennington",
		State:     "Michigan",
		Country:   "US",
		Elevation: 810,
		Latitude:  42.9230995178,
		Longitude: -84.2833023071,
		Timezone:  "America/Detroit",
	},
	"MI09": {
		ICAO:      "MI09",
		Name:      "Milan Airport",
		City:      "Milan",
		State:     "Michigan",
		Country:   "US",
		Elevation: 706,
		Latitude:  42.0499992371,
		Longitude: -83.740196228,
		Timezone:  "America/Detroit",
	},
	"MI10": {
		ICAO:      "MI10",
		Name:      "Hoerners Corners Airport",
		City:      "De Witt",
		State:     "Michigan",
		Country:   "US",
		Elevation: 800,
		Latitude:  42.8652992249,
		Longitude: -84.7054977417,
		Timezone:  "America/Detroit",
	},
	"MI11": {
		ICAO:      "MI11",
		Name:      "Crippen Field",
		City:      "Charlotte",
		State:     "Michigan",
		Country:   "US",
		Elevation: 890,
		Latitude:  42.534198761,
		Longitude: -84.801399231,
		Timezone:  "America/Detroit",
	},
	"MI13": {
		ICAO:      "MI13",
		Name:      "Minikey Airport",
		City:      "Sheridan",
		State:     "Michigan",
		Country:   "US",
		Elevation: 900,
		Latitude:  43.1617012024,
		Longitude: -85.0616989136,
		Timezone:  "America/Detroit",
	},
	"MI15": {
		ICAO:      "MI15",
		Name:      "Wards Long Acres Airport",
		City:      "Fair Haven",
		State:     "Michigan",
		Country:   "US",
		Elevation: 618,
		Latitude:  42.7291984558,
		Longitude: -82.6651992798,
		Timezone:  "America/Detroit",
	},
	"MI18": {
		ICAO:      "MI18",
		Name:      "Cedarville Airport",
		City:      "Flushing",
		State:     "Michigan",
		Country:   "US",
		Elevation: 725,
		Latitude:  43.0931015015,
		Longitude: -83.8360977173,
		Timezone:  "America/Detroit",
	},
	"MI19": {
		ICAO:      "MI19",
		Name:      "Thorn Airport",
		City:      "Custer",
		State:     "Michigan",
		Country:   "US",
		Elevation: 780,
		Latitude:  43.9300003052,
		Longitude: -86.1656036377,
		Timezone:  "America/Detroit",
	},
	"MI21": {
		ICAO:      "MI21",
		Name:      "A.C. Miller Airport",
		City:      "Frontier",
		State:     "Michigan",
		Country:   "US",
		Elevation: 944,
		Latitude:  41.7374992371,
		Longitude: -84.5649032593,
		Timezone:  "America/Detroit",
	},
	"MI22": {
		ICAO:      "MI22",
		Name:      "Crump Airport",
		City:      "Niles",
		State:     "Michigan",
		Country:   "US",
		Elevation: 745,
		Latitude:  41.8736991882,
		Longitude: -86.202796936,
		Timezone:  "America/Detroit",
	},
	"MI23": {
		ICAO:      "MI23",
		Name:      "Mckimmy Field",
		City:      "Bridgeport",
		State:     "Michigan",
		Country:   "US",
		Elevation: 600,
		Latitude:  43.3469009399,
		Longitude: -83.8666992188,
		Timezone:  "America/Detroit",
	},
	"MI24": {
		ICAO:      "MI24",
		Name:      "Wilson Township Airport",
		City:      "East Jordan",
		State:     "Michigan",
		Country:   "US",
		Elevation: 730,
		Latitude:  45.1417007446,
		Longitude: -85.0066986084,
		Timezone:  "America/Detroit",
	},
	"MI25": {
		ICAO:      "MI25",
		Name:      "Myers Airport",
		City:      "Bridgman",
		State:     "Michigan",
		Country:   "US",
		Elevation: 670,
		Latitude:  41.9222984314,
		Longitude: -86.5456008911,
		Timezone:  "America/Detroit",
	},
	"MI26": {
		ICAO:      "MI26",
		Name:      "Hynes Field",
		City:      "Hartland",
		State:     "Michigan",
		Country:   "US",
		Elevation: 978,
		Latitude:  42.6109008789,
		Longitude: -83.7326965332,
		Timezone:  "America/Detroit",
	},
	"MI27": {
		ICAO:      "MI27",
		Name:      "Haigh Airport",
		City:      "Howell",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1000,
		Latitude:  42.6124992371,
		Longitude: -83.8541030884,
		Timezone:  "America/Detroit",
	},
	"MI29": {
		ICAO:      "MI29",
		Name:      "Gooding Airport",
		City:      "Whitmore Lake",
		State:     "Michigan",
		Country:   "US",
		Elevation: 920,
		Latitude:  42.3763999939,
		Longitude: -83.7927017212,
		Timezone:  "America/Detroit",
	},
	"MI31": {
		ICAO:      "MI31",
		Name:      "Cornish Field",
		City:      "Lawton",
		State:     "Michigan",
		Country:   "US",
		Elevation: 910,
		Latitude:  42.1015014648,
		Longitude: -85.8378982544,
		Timezone:  "America/Detroit",
	},
	"MI33": {
		ICAO:      "MI33",
		Name:      "Adair Airstrip",
		City:      "Richmond",
		State:     "Michigan",
		Country:   "US",
		Elevation: 631,
		Latitude:  42.792301178,
		Longitude: -82.6393966675,
		Timezone:  "America/Detroit",
	},
	"MI35": {
		ICAO:      "MI35",
		Name:      "Claucherty Airport",
		City:      "Litchfield",
		State:     "Michigan",
		Country:   "US",
		Elevation: 988,
		Latitude:  42.0778007507,
		Longitude: -84.8291015625,
		Timezone:  "America/Detroit",
	},
	"MI37": {
		ICAO:      "MI37",
		Name:      "Wenning Landing Area Airport",
		City:      "Marine City",
		State:     "Michigan",
		Country:   "US",
		Elevation: 620,
		Latitude:  42.7555999756,
		Longitude: -82.5582962036,
		Timezone:  "America/Detroit",
	},
	"MI38": {
		ICAO:      "MI38",
		Name:      "Eichmeier Field",
		City:      "Mason",
		State:     "Michigan",
		Country:   "US",
		Elevation: 900,
		Latitude:  42.6022987366,
		Longitude: -84.426399231,
		Timezone:  "America/Detroit",
	},
	"MI39": {
		ICAO:      "MI39",
		Name:      "Laszlo Airport",
		City:      "Milan",
		State:     "Michigan",
		Country:   "US",
		Elevation: 675,
		Latitude:  42.0750007629,
		Longitude: -83.6082992554,
		Timezone:  "America/Detroit",
	},
	"MI41": {
		ICAO:      "MI41",
		Name:      "Crystal Airport",
		City:      "St. Clair",
		State:     "Michigan",
		Country:   "US",
		Elevation: 610,
		Latitude:  42.8316993713,
		Longitude: -82.5345993042,
		Timezone:  "America/Detroit",
	},
	"MI43": {
		ICAO:      "MI43",
		Name:      "Pewanogowink-Banks Airport",
		City:      "Montrose",
		State:     "Michigan",
		Country:   "US",
		Elevation: 660,
		Latitude:  43.1861000061,
		Longitude: -83.9013977051,
		Timezone:  "America/Detroit",
	},
	"MI44": {
		ICAO:      "MI44",
		Name:      "D J Airport",
		City:      "Mount Pleasant",
		State:     "Michigan",
		Country:   "US",
		Elevation: 755,
		Latitude:  43.6722984314,
		Longitude: -84.7367019653,
		Timezone:  "America/Detroit",
	},
	"MI45": {
		ICAO:      "MI45",
		Name:      "Ed Schulte's Place STOLport",
		City:      "Ortonville",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1050,
		Latitude:  42.8081016541,
		Longitude: -83.4518966675,
		Timezone:  "America/Detroit",
	},
	"MI47": {
		ICAO:      "MI47",
		Name:      "Dysinger Airport",
		City:      "Perry",
		State:     "Michigan",
		Country:   "US",
		Elevation: 897,
		Latitude:  42.8083992004,
		Longitude: -84.1902008057,
		Timezone:  "America/Detroit",
	},
	"MI48": {
		ICAO:      "MI48",
		Name:      "Ham-A-Lot Field",
		City:      "Petersburg",
		State:     "Michigan",
		Country:   "US",
		Elevation: 680,
		Latitude:  41.8334007263,
		Longitude: -83.6832962036,
		Timezone:  "America/Detroit",
	},
	"MI50": {
		ICAO:      "MI50",
		Name:      "Dodge Airport",
		City:      "Romeo",
		State:     "Michigan",
		Country:   "US",
		Elevation: 705,
		Latitude:  42.7667007446,
		Longitude: -82.9498977661,
		Timezone:  "America/Detroit",
	},
	"MI51": {
		ICAO:      "MI51",
		Name:      "Loars Field",
		City:      "Onsted",
		State:     "Michigan",
		Country:   "US",
		Elevation: 995,
		Latitude:  41.9968986511,
		Longitude: -84.1885986328,
		Timezone:  "America/Detroit",
	},
	"MI52": {
		ICAO:      "MI52",
		Name:      "Markham Airport",
		City:      "Clinton",
		State:     "Michigan",
		Country:   "US",
		Elevation: 825,
		Latitude:  42.0905990601,
		Longitude: -83.9680023193,
		Timezone:  "America/Detroit",
	},
	"MI53": {
		ICAO:      "MI53",
		Name:      "Mills Field",
		City:      "Newport",
		State:     "Michigan",
		Country:   "US",
		Elevation: 590,
		Latitude:  42.0092010498,
		Longitude: -83.2583007813,
		Timezone:  "America/Detroit",
	},
	"MI55": {
		ICAO:      "MI55",
		Name:      "Zeitler Airport",
		City:      "Merril",
		State:     "Michigan",
		Country:   "US",
		Elevation: 678,
		Latitude:  43.4900016785,
		Longitude: -84.3653030396,
		Timezone:  "America/Detroit",
	},
	"MI58": {
		ICAO:      "MI58",
		Name:      "Stony Acres Airport",
		City:      "Sunfield",
		State:     "Michigan",
		Country:   "US",
		Elevation: 849,
		Latitude:  42.7545013428,
		Longitude: -85.0286026001,
		Timezone:  "America/Detroit",
	},
	"MI60": {
		ICAO:      "MI60",
		Name:      "Tannehill Airfield",
		City:      "Fife Lake",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1050,
		Latitude:  44.5914001465,
		Longitude: -85.310798645,
		Timezone:  "America/Detroit",
	},
	"MI67": {
		ICAO:      "MI67",
		Name:      "Tyrone Airport",
		City:      "Bailey",
		State:     "Michigan",
		Country:   "US",
		Elevation: 810,
		Latitude:  43.2891998291,
		Longitude: -85.7811965942,
		Timezone:  "America/Detroit",
	},
	"MI68": {
		ICAO:      "MI68",
		Name:      "Del Hickcox Airport",
		City:      "Baroda",
		State:     "Michigan",
		Country:   "US",
		Elevation: 645,
		Latitude:  41.9375,
		Longitude: -86.5014038086,
		Timezone:  "America/Detroit",
	},
	"MI70": {
		ICAO:      "MI70",
		Name:      "Carl's Airport",
		City:      "Macon",
		State:     "Michigan",
		Country:   "US",
		Elevation: 850,
		Latitude:  42.0712013245,
		Longitude: -83.8835983276,
		Timezone:  "America/Detroit",
	},
	"MI71": {
		ICAO:      "MI71",
		Name:      "Silver Lake Airport",
		City:      "Mears",
		State:     "Michigan",
		Country:   "US",
		Elevation: 772,
		Latitude:  43.6445007324,
		Longitude: -86.4529037476,
		Timezone:  "America/Detroit",
	},
	"MI72": {
		ICAO:      "MI72",
		Name:      "Nelson Airport",
		City:      "Buchanan",
		State:     "Michigan",
		Country:   "US",
		Elevation: 665,
		Latitude:  41.8499984741,
		Longitude: -86.3499984741,
		Timezone:  "America/Detroit",
	},
	"MI73": {
		ICAO:      "MI73",
		Name:      "North Cedar Airport",
		City:      "Cedar Springs",
		State:     "Michigan",
		Country:   "US",
		Elevation: 893,
		Latitude:  43.2583999634,
		Longitude: -85.5458984375,
		Timezone:  "America/Detroit",
	},
	"MI76": {
		ICAO:      "MI76",
		Name:      "Reading Airport",
		City:      "Fennville",
		State:     "Michigan",
		Country:   "US",
		Elevation: 700,
		Latitude:  42.5499992371,
		Longitude: -86.2166976929,
		Timezone:  "America/Detroit",
	},
	"MI77": {
		ICAO:      "MI77",
		Name:      "Shafter Airport",
		City:      "Galesburg",
		State:     "Michigan",
		Country:   "US",
		Elevation: 805,
		Latitude:  42.2999992371,
		Longitude: -85.4167022705,
		Timezone:  "America/Detroit",
	},
	"MI78": {
		ICAO:      "MI78",
		Name:      "Weller Airport",
		City:      "Grandville",
		State:     "Michigan",
		Country:   "US",
		Elevation: 600,
		Latitude:  42.9166984558,
		Longitude: -85.7834014893,
		Timezone:  "America/Detroit",
	},
	"MI80": {
		ICAO:      "MI80",
		Name:      "Wabasis Lake Airport",
		City:      "Greenville",
		State:     "Michigan",
		Country:   "US",
		Elevation: 892,
		Latitude:  43.1283988953,
		Longitude: -85.3992004395,
		Timezone:  "America/Detroit",
	},
	"MI82": {
		ICAO:      "MI82",
		Name:      "Harry's Field",
		City:      "Hudsonville",
		State:     "Michigan",
		Country:   "US",
		Elevation: 662,
		Latitude:  42.8763999939,
		Longitude: -85.9555969238,
		Timezone:  "America/Detroit",
	},
	"MI84": {
		ICAO:      "MI84",
		Name:      "Kerby Field",
		City:      "Mattawan",
		State:     "Michigan",
		Country:   "US",
		Elevation: 780,
		Latitude:  42.1983985901,
		Longitude: -85.8067016602,
		Timezone:  "America/Detroit",
	},
	"MI86": {
		ICAO:      "MI86",
		Name:      "Midget Airport",
		City:      "North Muskegon",
		State:     "Michigan",
		Country:   "US",
		Elevation: 669,
		Latitude:  43.3424987793,
		Longitude: -86.1868972778,
		Timezone:  "America/Detroit",
	},
	"MI88": {
		ICAO:      "MI88",
		Name:      "Bakers Field",
		City:      "Burnips",
		State:     "Michigan",
		Country:   "US",
		Elevation: 680,
		Latitude:  42.7322998047,
		Longitude: -85.9052963257,
		Timezone:  "America/Detroit",
	},
	"MI89": {
		ICAO:      "MI89",
		Name:      "Perry Airport",
		City:      "Schoolcraft",
		State:     "Michigan",
		Country:   "US",
		Elevation: 894,
		Latitude:  42.1487007141,
		Longitude: -85.6528015137,
		Timezone:  "America/Detroit",
	},
	"MI92": {
		ICAO:      "MI92",
		Name:      "Lilienthal Airport",
		City:      "Iron Mountain",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1250,
		Latitude:  45.9327011108,
		Longitude: -88.098197937,
		Timezone:  "America/Menominee",
	},
	"MI93": {
		ICAO:      "MI93",
		Name:      "Circle T Ranch Airport",
		City:      "Curran",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1010,
		Latitude:  44.6878013611,
		Longitude: -83.7438964844,
		Timezone:  "America/Detroit",
	},
	"MI94": {
		ICAO:      "MI94",
		Name:      "Fontecchio Airport",
		City:      "E Kingsford",
		State:     "Michigan",
		Country:   "US",
		Elevation: 1110,
		Latitude:  45.7846984863,
		Longitude: -88.0681991577,
		Timezone:  "America/Chicago",
	},
	"MI95": {
		ICAO:      "MI95",
		Name:      "Taylors Flight Park Airport",
		City:      "Cassopolis",
		State:     "Michigan",
		Country:   "US",
		Elevation: 890,
		Latitude:  41.846698761,
		Longitude: -85.8949966431,
		Timezone:  "America/Detroit",
	},
	"MI98": {
		ICAO:      "MI98",
		Name:      "Dowd Field",
		City:      "Fibre",
		State:     "Michigan",
		Country:   "US",
		Elevation: 705,
		Latitude:  46.2014007568,
		Longitude: -84.7416992188,
		Timezone:  "America/Detroit",
	},
	"MI99": {
		ICAO:      "MI99",
		Name:      "Robertson Field",
		City:      "Richmond",
		State:     "Michigan",
		Country:   "US",
		Elevation: 620,
		Latitude:  42.7989006042,
		Longitude: -82.6587982178,
		Timezone:  "America/Detroit",
	},
	"MKBS": {
		ICAO:      "MKBS",
		IATA:      "OCJ",
		Name:      "Boscobel Aerodrome",
		City:      "Ocho Rios",
		State:     "St-Mary",
		Country:   "JM",
		Elevation: 90,
		Latitude:  18.4041996002,
		Longitude: -76.96900177,
		Timezone:  "America/Jamaica",
	},
	"MKJP": {
		ICAO:      "MKJP",
		IATA:      "KIN",
		Name:      "Norman Manley International Airport",
		City:      "Kingston",
		State:     "Kingston",
		Country:   "JM",
		Elevation: 10,
		Latitude:  17.9356994629,
		Longitude: -76.7874984741,
		Timezone:  "America/Jamaica",
	},
	"MKJS": {
		ICAO:      "MKJS",
		IATA:      "MBJ",
		Name:      "Sangster International Airport",
		City:      "Montego Bay",
		State:     "St-James",
		Country:   "JM",
		Elevation: 4,
		Latitude:  18.5037002563,
		Longitude: -77.9133987427,
		Timezone:  "America/Jamaica",
	},
	"MKKJ": {
		ICAO:      "MKKJ",
		IATA:      "POT",
		Name:      "Ken Jones Airport",
		City:      "Ken Jones",
		State:     "Portland",
		Country:   "JM",
		Elevation: 20,
		Latitude:  18.1987991333,
		Longitude: -76.5345001221,
		Timezone:  "America/Jamaica",
	},
	"MKNG": {
		ICAO:      "MKNG",
		IATA:      "NEG",
		Name:      "Negril Airport",
		City:      "Negril",
		State:     "Hanover",
		Country:   "JM",
		Elevation: 9,
		Latitude:  18.3428001404,
		Longitude: -78.3320999146,
		Timezone:  "America/Jamaica",
	},
	"MKTP": {
		ICAO:      "MKTP",
		IATA:      "KTP",
		Name:      "Tinson Pen Airport",
		City:      "Tinson Pen",
		State:     "St-Andrew",
		Country:   "JM",
		Elevation: 16,
		Latitude:  17.9885997772,
		Longitude: -76.8237991333,
		Timezone:  "America/Jamaica",
	},
	"MLIP": {
		ICAO:      "MLIP",
		IATA:      "MIJ",
		Name:      "Mili Island Airport",
		City:      "Mili Island",
		State:     "Mili-Atoll",
		Country:   "MH",
		Elevation: 4,
		Latitude:  6.0833301544,
		Longitude: 171.733001709,
		Timezone:  "Pacific/Majuro",
	},
	"MLLJ": {
		ICAO:      "MLLJ",
		Name:      "La Javilla Airport",
		City:      "Guanacaste",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 43,
		Latitude:  9.8111200333,
		Longitude: -85.295501709,
		Timezone:  "America/Costa_Rica",
	},
	"MM10": {
		ICAO:      "MM10",
		Name:      "Isla Cedros Airport",
		State:     "Baja-California",
		Country:   "MX",
		Elevation: 98,
		Latitude:  28.0375995636,
		Longitude: -115.1900024414,
		Timezone:  "America/Tijuana",
	},
	"MM11": {
		ICAO:      "MM11",
		Name:      "Rancho El Durangueno Airport",
		State:     "Durango",
		Country:   "MX",
		Elevation: 8202,
		Latitude:  24.4183006287,
		Longitude: -104.8860015869,
		Timezone:  "America/Monterrey",
	},
	"MM12": {
		ICAO:      "MM12",
		Name:      "Bacubirito Airport",
		State:     "Sinaloa",
		Country:   "MX",
		Elevation: 986,
		Latitude:  25.815700531,
		Longitude: -107.9079971313,
		Timezone:  "America/Mazatlan",
	},
	"MM13": {
		ICAO:      "MM13",
		Name:      "Ciudad Camargo Southeast Airport",
		State:     "Chihuahua",
		Country:   "MX",
		Elevation: 4265,
		Latitude:  27.5988998413,
		Longitude: -105.1039962769,
		Timezone:  "America/Chihuahua",
	},
	"MM14": {
		ICAO:      "MM14",
		Name:      "Rio Verde Airport",
		State:     "San-Luis-Potosi",
		Country:   "MX",
		Elevation: 3281,
		Latitude:  21.9629001617,
		Longitude: -100.0080032349,
		Timezone:  "America/Mexico_City",
	},
	"MM16": {
		ICAO:      "MM16",
		Name:      "J Agustin Castro Airport",
		State:     "Durango",
		Country:   "MX",
		Elevation: 3720,
		Latitude:  25.5265007019,
		Longitude: -103.5179977417,
		Timezone:  "America/Monterrey",
	},
	"MM17": {
		ICAO:      "MM17",
		Name:      "La Retama Southwest Airport",
		State:     "Tamaulipas",
		Country:   "MX",
		Elevation: 2000,
		Latitude:  23.966299057,
		Longitude: -98.8348007202,
		Timezone:  "America/Monterrey",
	},
	"MM20": {
		ICAO:      "MM20",
		Name:      "Cordoba Airport",
		State:     "Veracruz",
		Country:   "MX",
		Elevation: 3000,
		Latitude:  18.8742008209,
		Longitude: -96.9581985474,
		Timezone:  "America/Mexico_City",
	},
	"MM21": {
		ICAO:      "MM21",
		Name:      "La Encarnacion Airport",
		State:     "Nuevo-Leon",
		Country:   "MX",
		Elevation: 1560,
		Latitude:  25.7360992432,
		Longitude: -100.2249984741,
		Timezone:  "America/Monterrey",
	},
	"MM22": {
		ICAO:      "MM22",
		Name:      "La Pesca Airport",
		State:     "Tamaulipas",
		Country:   "MX",
		Elevation: 15,
		Latitude:  23.8022003174,
		Longitude: -97.7736968994,
		Timezone:  "America/Monterrey",
	},
	"MM23": {
		ICAO:      "MM23",
		Name:      "Rancho Santa Ynes Airport",
		State:     "Baja-California",
		Country:   "MX",
		Elevation: 1843,
		Latitude:  29.72590065,
		Longitude: -114.6999969482,
		Timezone:  "America/Tijuana",
	},
	"MM25": {
		ICAO:      "MM25",
		Name:      "Ciudad Guzman Airport",
		State:     "Jalisco",
		Country:   "MX",
		Elevation: 4987,
		Latitude:  19.7084999084,
		Longitude: -103.4899978638,
		Timezone:  "America/Mexico_City",
	},
	"MM27": {
		ICAO:      "MM27",
		Name:      "Cementos Mexicanos Airport",
		State:     "San-Luis-Potosi",
		Country:   "MX",
		Elevation: 302,
		Latitude:  21.9839992523,
		Longitude: -98.9618988037,
		Timezone:  "America/Mexico_City",
	},
	"MM28": {
		ICAO:      "MM28",
		Name:      "Tizayuca Airport",
		State:     "Morelos",
		Country:   "MX",
		Elevation: 7600,
		Latitude:  19.8740005493,
		Longitude: -98.9436035156,
		Timezone:  "America/Mexico_City",
	},
	"MM29": {
		ICAO:      "MM29",
		Name:      "Ta Lo De Soto Airport",
		State:     "Guerrero",
		Country:   "MX",
		Elevation: 490,
		Latitude:  16.4634990692,
		Longitude: -98.3929977417,
		Timezone:  "America/Mexico_City",
	},
	"MM30": {
		ICAO:      "MM30",
		Name:      "Campo Gobierno Airport",
		State:     "Sinaloa",
		Country:   "MX",
		Elevation: 43,
		Latitude:  24.6585006714,
		Longitude: -107.5520019531,
		Timezone:  "America/Mazatlan",
	},
	"MM31": {
		ICAO:      "MM31",
		Name:      "Fausto Vega Santander Airport",
		State:     "Veracruz",
		Country:   "MX",
		Elevation: 10,
		Latitude:  20.9468002319,
		Longitude: -97.3750991821,
		Timezone:  "America/Mexico_City",
	},
	"MM32": {
		ICAO:      "MM32",
		Name:      "Puerto Libertad North Airport",
		State:     "Sonora",
		Country:   "MX",
		Elevation: 148,
		Latitude:  29.929599762,
		Longitude: -112.65599823,
		Timezone:  "America/Hermosillo",
	},
	"MM36": {
		ICAO:      "MM36",
		Name:      "Rancho San Salvador Northeast Airport",
		State:     "Nuevo-Leon",
		Country:   "MX",
		Elevation: 900,
		Latitude:  26.6784992218,
		Longitude: -100.1709976196,
		Timezone:  "America/Monterrey",
	},
	"MM37": {
		ICAO:      "MM37",
		Name:      "Casa Madero Airport",
		State:     "Coahuila",
		Country:   "MX",
		Elevation: 4498,
		Latitude:  25.5062007904,
		Longitude: -102.202003479,
		Timezone:  "America/Monterrey",
	},
	"MM38": {
		ICAO:      "MM38",
		Name:      "Falcon Dam Airport",
		State:     "Tamaulipas",
		Country:   "MX",
		Elevation: 150,
		Latitude:  26.5210990906,
		Longitude: -99.1485977173,
		Timezone:  "America/Chicago",
	},
	"MM39": {
		ICAO:      "MM39",
		Name:      "Guamuchil Airport",
		State:     "Sinaloa",
		Country:   "MX",
		Elevation: 80,
		Latitude:  25.4386997223,
		Longitude: -108.0920028687,
		Timezone:  "America/Mazatlan",
	},
	"MM41": {
		ICAO:      "MM41",
		Name:      "Leon Gonzales Pie De La Cuesta Airport",
		State:     "Guerrero",
		Country:   "MX",
		Elevation: 20,
		Latitude:  16.910200119,
		Longitude: -99.9887008667,
		Timezone:  "America/Mexico_City",
	},
	"MM42": {
		ICAO:      "MM42",
		Name:      "Muzquiz New Airport",
		State:     "Coahuila",
		Country:   "MX",
		Elevation: 1750,
		Latitude:  27.856300354,
		Longitude: -101.5260009766,
		Timezone:  "America/Monterrey",
	},
	"MM44": {
		ICAO:      "MM44",
		Name:      "Agualeguas Old Airport",
		State:     "Nuevo-Leon",
		Country:   "MX",
		Elevation: 676,
		Latitude:  26.3043994904,
		Longitude: -99.550201416,
		Timezone:  "America/Monterrey",
	},
	"MM47": {
		ICAO:      "MM47",
		Name:      "Jaujilla Airport",
		State:     "Michoacan",
		Country:   "MX",
		Elevation: 6398,
		Latitude:  19.8514995575,
		Longitude: -101.7519989014,
		Timezone:  "America/Mexico_City",
	},
	"MM48": {
		ICAO:      "MM48",
		Name:      "Ciudad Pemex Airport",
		State:     "Tabasco",
		Country:   "MX",
		Elevation: 20,
		Latitude:  17.8801002502,
		Longitude: -92.4757003784,
		Timezone:  "America/Mexico_City",
	},
	"MM50": {
		ICAO:      "MM50",
		Name:      "Cosala Airport",
		State:     "Sinaloa",
		Country:   "MX",
		Elevation: 250,
		Latitude:  24.4206008911,
		Longitude: -106.702003479,
		Timezone:  "America/Mazatlan",
	},
	"MM51": {
		ICAO:      "MM51",
		Name:      "Rancho Guadalupe South Airport",
		State:     "Coahuila",
		Country:   "MX",
		Elevation: 2200,
		Latitude:  26.6355991364,
		Longitude: -100.8899993896,
		Timezone:  "America/Monterrey",
	},
	"MM53": {
		ICAO:      "MM53",
		Name:      "Nuevo Dolores Airport",
		State:     "Tamaulipas",
		Country:   "MX",
		Elevation: 480,
		Latitude:  24.0615005493,
		Longitude: -98.4156036377,
		Timezone:  "America/Monterrey",
	},
	"MM54": {
		ICAO:      "MM54",
		Name:      "Rancho Marina Vieja Airport",
		State:     "Tamaulipas",
		Country:   "MX",
		Elevation: 100,
		Latitude:  23.7381000519,
		Longitude: -98.0612030029,
		Timezone:  "America/Monterrey",
	},
	"MM56": {
		ICAO:      "MM56",
		Name:      "Xicotencatl Airport",
		State:     "Tamaulipas",
		Country:   "MX",
		Elevation: 377,
		Latitude:  22.9538993835,
		Longitude: -98.959602356,
		Timezone:  "America/Monterrey",
	},
	"MM61": {
		ICAO:      "MM61",
		Name:      "Ixmiquilpan Airport",
		State:     "Hidalgo",
		Country:   "MX",
		Elevation: 5577,
		Latitude:  20.486000061,
		Longitude: -99.2587966919,
		Timezone:  "America/Mexico_City",
	},
	"MM63": {
		ICAO:      "MM63",
		Name:      "Autlan Airport",
		State:     "Jalisco",
		Country:   "MX",
		Elevation: 2900,
		Latitude:  19.745300293,
		Longitude: -104.3369979858,
		Timezone:  "America/Mexico_City",
	},
	"MM64": {
		ICAO:      "MM64",
		Name:      "Cuatro Cienegas New Airport",
		State:     "Coahuila",
		Country:   "MX",
		Elevation: 2450,
		Latitude:  26.9904003143,
		Longitude: -102.0329971313,
		Timezone:  "America/Monterrey",
	},
	"MM65": {
		ICAO:      "MM65",
		Name:      "Agua Prieta South Airport",
		State:     "Sonora",
		Country:   "MX",
		Elevation: 4165,
		Latitude:  31.2523002625,
		Longitude: -109.608001709,
		Timezone:  "America/Hermosillo",
	},
	"MM66": {
		ICAO:      "MM66",
		Name:      "Coahuayana Airport",
		State:     "Michoacan",
		Country:   "MX",
		Elevation: 200,
		Latitude:  18.6846008301,
		Longitude: -103.6760025024,
		Timezone:  "America/Mexico_City",
	},
	"MM67": {
		ICAO:      "MM67",
		Name:      "Matehuala Airport",
		State:     "San-Luis-Potosi",
		Country:   "MX",
		Elevation: 5118,
		Latitude:  23.6765995026,
		Longitude: -100.6230010986,
		Timezone:  "America/Mexico_City",
	},
	"MM68": {
		ICAO:      "MM68",
		Name:      "Mina Hercules Airport",
		State:     "Coahuila",
		Country:   "MX",
		Elevation: 4310,
		Latitude:  28.0366001129,
		Longitude: -103.7710037231,
		Timezone:  "America/Monterrey",
	},
	"MM69": {
		ICAO:      "MM69",
		Name:      "Tacicuri Airport",
		State:     "Sonora",
		Country:   "MX",
		Elevation: 2536,
		Latitude:  30.6744995117,
		Longitude: -110.9309997559,
		Timezone:  "America/Hermosillo",
	},
	"MM72": {
		ICAO:      "MM72",
		Name:      "Cupul Airport",
		State:     "Yucatan",
		Country:   "MX",
		Elevation: 56,
		Latitude:  21.1557006836,
		Longitude: -88.1728973389,
		Timezone:  "America/Merida",
	},
	"MM73": {
		ICAO:      "MM73",
		Name:      "Huetamo Airport",
		State:     "Michoacan",
		Country:   "MX",
		Elevation: 984,
		Latitude:  18.5273990631,
		Longitude: -100.8509979248,
		Timezone:  "America/Mexico_City",
	},
	"MM75": {
		ICAO:      "MM75",
		Name:      "Agua Prieta Southwest Airport",
		State:     "Sonora",
		Country:   "MX",
		Elevation: 4000,
		Latitude:  31.2429008484,
		Longitude: -109.625,
		Timezone:  "America/Hermosillo",
	},
	"MM77": {
		ICAO:      "MM77",
		Name:      "Las Delicias Airport",
		State:     "Chihuahua",
		Country:   "MX",
		Elevation: 3888,
		Latitude:  28.2131004333,
		Longitude: -105.4479980469,
		Timezone:  "America/Chihuahua",
	},
	"MM79": {
		ICAO:      "MM79",
		Name:      "El Fuerte Airport",
		State:     "Sinaloa",
		Country:   "MX",
		Elevation: 246,
		Latitude:  26.3981990814,
		Longitude: -108.6119995117,
		Timezone:  "America/Mazatlan",
	},
	"MM80": {
		ICAO:      "MM80",
		Name:      "Rancho La Milpita Airport",
		State:     "Sonora",
		Country:   "MX",
		Elevation: 4707,
		Latitude:  30.4808998108,
		Longitude: -109.6439971924,
		Timezone:  "America/Hermosillo",
	},
	"MM81": {
		ICAO:      "MM81",
		Name:      "Isla Socorro Airport",
		State:     "Colima",
		Country:   "MX",
		Elevation: 1085,
		Latitude:  18.7742004395,
		Longitude: -110.9300003052,
		Timezone:  "America/Mexico_City",
	},
	"MM82": {
		ICAO:      "MM82",
		Name:      "Laguna Del Rey Airport",
		State:     "Coahuila",
		Country:   "MX",
		Elevation: 3450,
		Latitude:  27.0237998962,
		Longitude: -103.375,
		Timezone:  "America/Matamoros",
	},
	"MM84": {
		ICAO:      "MM84",
		Name:      "Parral Airport",
		State:     "Chihuahua",
		Country:   "MX",
		Elevation: 5965,
		Latitude:  26.9221000671,
		Longitude: -105.7819976807,
		Timezone:  "America/Chihuahua",
	},
	"MMAA": {
		ICAO:      "MMAA",
		IATA:      "ACA",
		Name:      "General Juan N Alvarez International Airport",
		City:      "Acapulco",
		State:     "Guerrero",
		Country:   "MX",
		Elevation: 16,
		Latitude:  16.7570991516,
		Longitude: -99.7539978027,
		Timezone:  "America/Mexico_City",
	},
	"MMAL": {
		ICAO:      "MMAL",
		Name:      "Agualeguas Airport",
		State:     "Nuevo-Leon",
		Country:   "MX",
		Elevation: 676,
		Latitude:  26.3339996338,
		Longitude: -99.5423965454,
		Timezone:  "America/Monterrey",
	},
	"MMAN": {
		ICAO:      "MMAN",
		IATA:      "NTR",
		Name:      "Del Norte International Airport",
		State:     "Nuevo-Leon",
		Country:   "MX",
		Elevation: 1476,
		Latitude:  25.8656005859,
		Longitude: -100.2369995117,
		Timezone:  "America/Monterrey",
	},
	"MMAS": {
		ICAO:      "MMAS",
		IATA:      "AGU",
		Name:      "Jesus Teran International Airport",
		City:      "Aguascalientes",
		State:     "Aguascalientes",
		Country:   "MX",
		Elevation: 6112,
		Latitude:  21.7056007385,
		Longitude: -102.318000793,
		Timezone:  "America/Mexico_City",
	},
	"MMBT": {
		ICAO:      "MMBT",
		IATA:      "HUX",
		Name:      "Bahias de Huatulco International Airport",
		City:      "Huatulco",
		State:     "Oaxaca",
		Country:   "MX",
		Elevation: 464,
		Latitude:  15.7753000259,
		Longitude: -96.2626037598,
		Timezone:  "America/Mexico_City",
	},
	"MMCA": {
		ICAO:      "MMCA",
		IATA:      "CNA",
		Name:      "Cananea Airport",
		State:     "Sonora",
		Country:   "MX",
		Elevation: 5200,
		Latitude:  31.0661501549,
		Longitude: -110.097877979,
		Timezone:  "America/Hermosillo",
	},
	"MMCB": {
		ICAO:      "MMCB",
		IATA:      "CVJ",
		Name:      "General Mariano Matamoros Airport",
		State:     "Morelos",
		Country:   "MX",
		Elevation: 4277,
		Latitude:  18.8348007202,
		Longitude: -99.2612991333,
		Timezone:  "America/Mexico_City",
	},
	"MMCC": {
		ICAO:      "MMCC",
		IATA:      "ACN",
		Name:      "Ciudad Acuna New International Airport",
		City:      "Ciudad Acuna",
		State:     "Coahuila",
		Country:   "MX",
		Elevation: 1410,
		Latitude:  29.3328990936,
		Longitude: -101.0989990234,
		Timezone:  "America/Matamoros",
	},
	"MMCE": {
		ICAO:      "MMCE",
		IATA:      "CME",
		Name:      "Ciudad del Carmen International Airport",
		City:      "Ciudad del Carmen",
		State:     "Campeche",
		Country:   "MX",
		Elevation: 10,
		Latitude:  18.6536998749,
		Longitude: -91.7990036011,
		Timezone:  "America/Merida",
	},
	"MMCG": {
		ICAO:      "MMCG",
		IATA:      "NCG",
		Name:      "Nuevo Casas Grandes Airport",
		State:     "Chihuahua",
		Country:   "MX",
		Elevation: 4850,
		Latitude:  30.3973999023,
		Longitude: -107.875,
		Timezone:  "America/Chihuahua",
	},
	"MMCH": {
		ICAO:      "MMCH",
		Name:      "Chilpancingo Airport",
		State:     "Guerrero",
		Country:   "MX",
		Elevation: 4199,
		Latitude:  17.5737991333,
		Longitude: -99.5142974854,
		Timezone:  "America/Mexico_City",
	},
	"MMCL": {
		ICAO:      "MMCL",
		IATA:      "CUL",
		Name:      "Federal de Bachigualato International Airport",
		City:      "Culiacan",
		State:     "Sinaloa",
		Country:   "MX",
		Elevation: 108,
		Latitude:  24.7644996643,
		Longitude: -107.474998474,
		Timezone:  "America/Mazatlan",
	},
	"MMCM": {
		ICAO:      "MMCM",
		IATA:      "CTM",
		Name:      "Chetumal International Airport",
		City:      "Chetumal",
		State:     "Quintana-Roo",
		Country:   "MX",
		Elevation: 39,
		Latitude:  18.504699707,
		Longitude: -88.3267974854,
		Timezone:  "America/Cancun",
	},
	"MMCN": {
		ICAO:      "MMCN",
		IATA:      "CEN",
		Name:      "Ciudad Obregon International Airport",
		City:      "Ciudad Obregon",
		State:     "Sonora",
		Country:   "MX",
		Elevation: 243,
		Latitude:  27.3925991058,
		Longitude: -109.8330001831,
		Timezone:  "America/Hermosillo",
	},
	"MMCO": {
		ICAO:      "MMCO",
		IATA:      "CJT",
		Name:      "Comitan Airport",
		State:     "Chiapas",
		Country:   "MX",
		Elevation: 5161,
		Latitude:  16.176700592,
		Longitude: -92.0505981445,
		Timezone:  "America/Mexico_City",
	},
	"MMCP": {
		ICAO:      "MMCP",
		IATA:      "CPE",
		Name:      "Ingeniero Alberto Acuna Ongay International Airport",
		City:      "Campeche",
		State:     "Campeche",
		Country:   "MX",
		Elevation: 34,
		Latitude:  19.8167991638,
		Longitude: -90.5002975464,
		Timezone:  "America/Merida",
	},
	"MMCS": {
		ICAO:      "MMCS",
		IATA:      "CJS",
		Name:      "Abraham Gonzalez International Airport",
		City:      "Ciudad Juarez",
		State:     "Chihuahua",
		Country:   "MX",
		Elevation: 3904,
		Latitude:  31.636100769,
		Longitude: -106.4290008545,
		Timezone:  "America/Ojinaga",
	},
	"MMCT": {
		ICAO:      "MMCT",
		IATA:      "CZA",
		Name:      "Chichen Itza International Airport",
		State:     "Yucatan",
		Country:   "MX",
		Elevation: 102,
		Latitude:  20.6413002014,
		Longitude: -88.4461975098,
		Timezone:  "America/Merida",
	},
	"MMCU": {
		ICAO:      "MMCU",
		IATA:      "CUU",
		Name:      "General Roberto Fierro Villalobos International Airport",
		City:      "Chihuahua",
		State:     "Chihuahua",
		Country:   "MX",
		Elevation: 4462,
		Latitude:  28.7028999329,
		Longitude: -105.964996338,
		Timezone:  "America/Chihuahua",
	},
	"MMCV": {
		ICAO:      "MMCV",
		IATA:      "CVM",
		Name:      "General Pedro Jose Mendez International Airport",
		City:      "Ciudad Victoria",
		State:     "Tamaulipas",
		Country:   "MX",
		Elevation: 761,
		Latitude:  23.7033004761,
		Longitude: -98.9564971924,
		Timezone:  "America/Monterrey",
	},
	"MMCY": {
		ICAO:      "MMCY",
		IATA:      "CYW",
		Name:      "Captain Rogelio Castillo National Airport",
		City:      "Celaya",
		State:     "Guanajuato",
		Country:   "MX",
		Elevation: 5709,
		Latitude:  20.545999527,
		Longitude: -100.8870010376,
		Timezone:  "America/Mexico_City",
	},
	"MMCZ": {
		ICAO:      "MMCZ",
		IATA:      "CZM",
		Name:      "Cozumel International Airport",
		City:      "Cozumel",
		State:     "Quintana-Roo",
		Country:   "MX",
		Elevation: 15,
		Latitude:  20.5223999023,
		Longitude: -86.9255981445,
		Timezone:  "America/Cancun",
	},
	"MMDA": {
		ICAO:      "MMDA",
		IATA:      "CUA",
		Name:      "Ciudad Constitucion Airport",
		City:      "Ciudad Constitucion",
		State:     "Baja-California-Sur",
		Country:   "MX",
		Elevation: 213,
		Latitude:  25.0538005829,
		Longitude: -111.6149978638,
		Timezone:  "America/Mazatlan",
	},
	"MMDM": {
		ICAO:      "MMDM",
		IATA:      "MMC",
		Name:      "Ciudad Mante National Airport",
		City:      "Ciudad Mante",
		State:     "Tamaulipas",
		Country:   "MX",
		Elevation: 341,
		Latitude:  22.7431774139,
		Longitude: -99.0173721313,
		Timezone:  "America/Monterrey",
	},
	"MMDO": {
		ICAO:      "MMDO",
		IATA:      "DGO",
		Name:      "General Guadalupe Victoria International Airport",
		City:      "Durango",
		State:     "Durango",
		Country:   "MX",
		Elevation: 6104,
		Latitude:  24.1242008209,
		Longitude: -104.527999878,
		Timezone:  "America/Monterrey",
	},
	"MMEP": {
		ICAO:      "MMEP",
		IATA:      "TPQ",
		Name:      "Amado Nervo National Airport",
		City:      "Tepic",
		State:     "Nayarit",
		Country:   "MX",
		Elevation: 3020,
		Latitude:  21.419500351,
		Longitude: -104.8430023193,
		Timezone:  "America/Mazatlan",
	},
	"MMES": {
		ICAO:      "MMES",
		IATA:      "ESE",
		Name:      "Ensenada Airport",
		State:     "Baja-California",
		Country:   "MX",
		Elevation: 66,
		Latitude:  31.79529953,
		Longitude: -116.6029968262,
		Timezone:  "America/Tijuana",
	},
	"MMGL": {
		ICAO:      "MMGL",
		IATA:      "GDL",
		Name:      "Don Miguel Hidalgo Y Costilla International Airport",
		City:      "Guadalajara",
		State:     "Jalisco",
		Country:   "MX",
		Elevation: 5016,
		Latitude:  20.5217990875,
		Longitude: -103.3109970093,
		Timezone:  "America/Mexico_City",
	},
	"MMGM": {
		ICAO:      "MMGM",
		IATA:      "GYM",
		Name:      "General Jose Maria Yanez International Airport",
		City:      "Guaymas",
		State:     "Sonora",
		Country:   "MX",
		Elevation: 59,
		Latitude:  27.9689998627,
		Longitude: -110.9250030518,
		Timezone:  "America/Hermosillo",
	},
	"MMGR": {
		ICAO:      "MMGR",
		IATA:      "GUB",
		Name:      "Guerrero Negro Airport",
		City:      "Guerrero Negro",
		State:     "Baja-California",
		Country:   "MX",
		Elevation: 59,
		Latitude:  28.0261001587,
		Longitude: -114.0240020752,
		Timezone:  "America/Mazatlan",
	},
	"MMGT": {
		ICAO:      "MMGT",
		Name:      "Guanajuato Airport",
		State:     "Guanajuato",
		Country:   "MX",
		Elevation: 6152,
		Latitude:  20.9172000885,
		Longitude: -101.3339996338,
		Timezone:  "America/Mexico_City",
	},
	"MMHC": {
		ICAO:      "MMHC",
		IATA:      "TCN",
		Name:      "Tehuacan Airport",
		State:     "Puebla",
		Country:   "MX",
		Elevation: 5509,
		Latitude:  18.4972000122,
		Longitude: -97.4198989868,
		Timezone:  "America/Mexico_City",
	},
	"MMHO": {
		ICAO:      "MMHO",
		IATA:      "HMO",
		Name:      "General Ignacio P. Garcia International Airport",
		City:      "Hermosillo",
		State:     "Sonora",
		Country:   "MX",
		Elevation: 627,
		Latitude:  29.0958995819,
		Longitude: -111.047996521,
		Timezone:  "America/Hermosillo",
	},
	"MMIA": {
		ICAO:      "MMIA",
		IATA:      "CLQ",
		Name:      "Lic. Miguel de la Madrid Airport",
		City:      "Colima",
		State:     "Colima",
		Country:   "MX",
		Elevation: 2467,
		Latitude:  19.2770004272,
		Longitude: -103.577003479,
		Timezone:  "America/Mexico_City",
	},
	"MMIM": {
		ICAO:      "MMIM",
		IATA:      "ISJ",
		Name:      "Isla Mujeres Airport",
		State:     "Quintana-Roo",
		Country:   "MX",
		Elevation: 7,
		Latitude:  21.2450008392,
		Longitude: -86.7399978638,
		Timezone:  "America/Cancun",
	},
	"MMIO": {
		ICAO:      "MMIO",
		IATA:      "SLW",
		Name:      "Plan De Guadalupe International Airport",
		City:      "Saltillo",
		State:     "Coahuila",
		Country:   "MX",
		Elevation: 4778,
		Latitude:  25.5494995117,
		Longitude: -100.9290008545,
		Timezone:  "America/Monterrey",
	},
	"MMIT": {
		ICAO:      "MMIT",
		IATA:      "IZT",
		Name:      "Ixtepec Airport",
		State:     "Oaxaca",
		Country:   "MX",
		Elevation: 164,
		Latitude:  16.449300766,
		Longitude: -95.0936965942,
		Timezone:  "America/Mexico_City",
	},
	"MMJA": {
		ICAO:      "MMJA",
		IATA:      "JAL",
		Name:      "El Lencero Airport",
		City:      "Xalapa",
		State:     "Veracruz",
		Country:   "MX",
		Elevation: 3127,
		Latitude:  19.4750995636,
		Longitude: -96.7975006104,
		Timezone:  "America/Mexico_City",
	},
	"MMJC": {
		ICAO:      "MMJC",
		IATA:      "AZP",
		Name:      "Atizapan De Zaragoza Airport",
		State:     "Mexico",
		Country:   "MX",
		Elevation: 8120,
		Latitude:  19.5748004913,
		Longitude: -99.2888031006,
		Timezone:  "America/Mexico_City",
	},
	"MMLC": {
		ICAO:      "MMLC",
		IATA:      "LZC",
		Name:      "Lazaro Cardenas Airport",
		City:      "Lazaro Cardenas",
		State:     "Michoacan",
		Country:   "MX",
		Elevation: 39,
		Latitude:  18.0016994476,
		Longitude: -102.221000671,
		Timezone:  "America/Mexico_City",
	},
	"MMLM": {
		ICAO:      "MMLM",
		IATA:      "LMM",
		Name:      "Valle del Fuerte International Airport",
		City:      "Los Mochis",
		State:     "Sinaloa",
		Country:   "MX",
		Elevation: 16,
		Latitude:  25.6851997375,
		Longitude: -109.081001282,
		Timezone:  "America/Mazatlan",
	},
	"MMLO": {
		ICAO:      "MMLO",
		IATA:      "BJX",
		Name:      "Del Bajio International Airport",
		City:      "Silao",
		State:     "Guanajuato",
		Country:   "MX",
		Elevation: 5956,
		Latitude:  20.9934997559,
		Longitude: -101.481002808,
		Timezone:  "America/Mexico_City",
	},
	"MMLP": {
		ICAO:      "MMLP",
		IATA:      "LAP",
		Name:      "Manuel Marquez de Leon International Airport",
		City:      "La Paz",
		State:     "Baja-California-Sur",
		Country:   "MX",
		Elevation: 69,
		Latitude:  24.0727005005,
		Longitude: -110.361999512,
		Timezone:  "America/Mazatlan",
	},
	"MMLT": {
		ICAO:      "MMLT",
		IATA:      "LTO",
		Name:      "Loreto International Airport",
		City:      "Loreto",
		State:     "Baja-California-Sur",
		Country:   "MX",
		Elevation: 34,
		Latitude:  25.989200592,
		Longitude: -111.3479995728,
		Timezone:  "America/Mazatlan",
	},
	"MMMA": {
		ICAO:      "MMMA",
		IATA:      "MAM",
		Name:      "General Servando Canales International Airport",
		City:      "Matamoros",
		State:     "Tamaulipas",
		Country:   "MX",
		Elevation: 25,
		Latitude:  25.7698993683,
		Longitude: -97.5252990723,
		Timezone:  "America/Matamoros",
	},
	"MMMD": {
		ICAO:      "MMMD",
		IATA:      "MID",
		Name:      "Licenciado Manuel Crescencio Rejon Int Airport",
		City:      "Merida",
		State:     "Yucatan",
		Country:   "MX",
		Elevation: 38,
		Latitude:  20.9370002747,
		Longitude: -89.657699585,
		Timezone:  "America/Merida",
	},
	"MMMG": {
		ICAO:      "MMMG",
		IATA:      "MUG",
		Name:      "Mulege Airport",
		City:      "Mulege",
		State:     "Baja-California-Sur",
		Country:   "MX",
		Elevation: 140,
		Latitude:  26.897310257,
		Longitude: -111.9575500488,
		Timezone:  "America/Mazatlan",
	},
	"MMML": {
		ICAO:      "MMML",
		IATA:      "MXL",
		Name:      "General Rodolfo Sanchez Taboada International Airport",
		City:      "Mexicali",
		State:     "Baja-California",
		Country:   "MX",
		Elevation: 74,
		Latitude:  32.6305999756,
		Longitude: -115.241996765,
		Timezone:  "America/Tijuana",
	},
	"MMMM": {
		ICAO:      "MMMM",
		IATA:      "MLM",
		Name:      "General Francisco J. Mujica International Airport",
		City:      "Morelia",
		State:     "Michoacan",
		Country:   "MX",
		Elevation: 6033,
		Latitude:  19.849899292,
		Longitude: -101.025001526,
		Timezone:  "America/Mexico_City",
	},
	"MMMT": {
		ICAO:      "MMMT",
		IATA:      "MTT",
		Name:      "Minatitlan/Coatzacoalcos National Airport",
		City:      "Minatitlan",
		State:     "Veracruz",
		Country:   "MX",
		Elevation: 36,
		Latitude:  18.1033992767,
		Longitude: -94.5807037354,
		Timezone:  "America/Mexico_City",
	},
	"MMMV": {
		ICAO:      "MMMV",
		IATA:      "LOV",
		Name:      "Monclova International Airport",
		State:     "Coahuila",
		Country:   "MX",
		Elevation: 1864,
		Latitude:  26.9556999207,
		Longitude: -101.4700012207,
		Timezone:  "America/Monterrey",
	},
	"MMMX": {
		ICAO:      "MMMX",
		IATA:      "MEX",
		Name:      "Licenciado Benito Juarez International Airport",
		City:      "Mexico City",
		State:     "Mexico-City",
		Country:   "MX",
		Elevation: 7316,
		Latitude:  19.4363002777,
		Longitude: -99.0720977783,
		Timezone:  "America/Mexico_City",
	},
	"MMMY": {
		ICAO:      "MMMY",
		IATA:      "MTY",
		Name:      "General Mariano Escobedo International Airport",
		City:      "Monterrey",
		State:     "Nuevo-Leon",
		Country:   "MX",
		Elevation: 1278,
		Latitude:  25.7784996033,
		Longitude: -100.107002258,
		Timezone:  "America/Monterrey",
	},
	"MMMZ": {
		ICAO:      "MMMZ",
		IATA:      "MZT",
		Name:      "General Rafael Buelna International Airport",
		City:      "Mazatlan",
		State:     "Sinaloa",
		Country:   "MX",
		Elevation: 38,
		Latitude:  23.1613998413,
		Longitude: -106.26599884,
		Timezone:  "America/Mazatlan",
	},
	"MMNG": {
		ICAO:      "MMNG",
		IATA:      "NOG",
		Name:      "Nogales International Airport",
		State:     "Sonora",
		Country:   "MX",
		Elevation: 3990,
		Latitude:  31.2261009216,
		Longitude: -110.9759979248,
		Timezone:  "America/Hermosillo",
	},
	"MMNL": {
		ICAO:      "MMNL",
		IATA:      "NLD",
		Name:      "Quetzalcoatl International Airport",
		City:      "Nuevo Laredo",
		State:     "Tamaulipas",
		Country:   "MX",
		Elevation: 484,
		Latitude:  27.4438991547,
		Longitude: -99.5705032349,
		Timezone:  "America/Matamoros",
	},
	"MMOX": {
		ICAO:      "MMOX",
		IATA:      "OAX",
		Name:      "Xoxocotlan International Airport",
		City:      "Oaxaca",
		State:     "Oaxaca",
		Country:   "MX",
		Elevation: 4989,
		Latitude:  16.9999008179,
		Longitude: -96.726600647,
		Timezone:  "America/Mexico_City",
	},
	"MMPA": {
		ICAO:      "MMPA",
		IATA:      "PAZ",
		Name:      "El Tajin National Airport",
		City:      "Poza Rica",
		State:     "Veracruz",
		Country:   "MX",
		Elevation: 497,
		Latitude:  20.6026992798,
		Longitude: -97.4608001709,
		Timezone:  "America/Mexico_City",
	},
	"MMPB": {
		ICAO:      "MMPB",
		IATA:      "PBC",
		Name:      "Hermanos Serdan International Airport",
		City:      "Puebla",
		State:     "Puebla",
		Country:   "MX",
		Elevation: 7361,
		Latitude:  19.1581001282,
		Longitude: -98.3713989258,
		Timezone:  "America/Mexico_City",
	},
	"MMPC": {
		ICAO:      "MMPC",
		Name:      "Ingeniero Juan Guillermo Villasana Airport",
		State:     "Hidalgo",
		Country:   "MX",
		Elevation: 7600,
		Latitude:  20.0774002075,
		Longitude: -98.7825012207,
		Timezone:  "America/Mexico_City",
	},
	"MMPE": {
		ICAO:      "MMPE",
		IATA:      "PPE",
		Name:      "Puerto Penasco International Airport",
		City:      "Puerto Penasco",
		State:     "Sonora",
		Country:   "MX",
		Elevation: 30,
		Latitude:  31.356202302,
		Longitude: -113.525676727,
		Timezone:  "America/Hermosillo",
	},
	"MMPG": {
		ICAO:      "MMPG",
		IATA:      "PDS",
		Name:      "Piedras Negras International Airport",
		State:     "Coahuila",
		Country:   "MX",
		Elevation: 901,
		Latitude:  28.6273994446,
		Longitude: -100.5350036621,
		Timezone:  "America/Matamoros",
	},
	"MMPL": {
		ICAO:      "MMPL",
		IATA:      "PCO",
		Name:      "Punta Colorada Airport",
		City:      "La Ribera",
		State:     "Baja-California-Sur",
		Country:   "MX",
		Elevation: 55,
		Latitude:  23.575011,
		Longitude: -109.535826,
		Timezone:  "America/Mazatlan",
	},
	"MMPN": {
		ICAO:      "MMPN",
		IATA:      "UPN",
		Name:      "Licenciado y General Ignacio Lopez Rayon Airport",
		State:     "Michoacan",
		Country:   "MX",
		Elevation: 5258,
		Latitude:  19.3966999054,
		Longitude: -102.0390014648,
		Timezone:  "America/Mexico_City",
	},
	"MMPP": {
		ICAO:      "MMPP",
		Name:      "Punta Pescadero Airport",
		State:     "Baja-California-Sur",
		Country:   "MX",
		Elevation: 33,
		Latitude:  23.8034000397,
		Longitude: -109.7070007324,
		Timezone:  "America/Mazatlan",
	},
	"MMPQ": {
		ICAO:      "MMPQ",
		IATA:      "PQM",
		Name:      "Palenque International Airport",
		State:     "Chiapas",
		Country:   "MX",
		Elevation: 200,
		Latitude:  17.5333995819,
		Longitude: -91.9844970703,
		Timezone:  "America/Mexico_City",
	},
	"MMPR": {
		ICAO:      "MMPR",
		IATA:      "PVR",
		Name:      "Licenciado Gustavo Diaz Ordaz International Airport",
		City:      "Puerto Vallarta",
		State:     "Jalisco",
		Country:   "MX",
		Elevation: 23,
		Latitude:  20.6800994873,
		Longitude: -105.2539978027,
		Timezone:  "America/Bahia_Banderas",
	},
	"MMPS": {
		ICAO:      "MMPS",
		IATA:      "PXM",
		Name:      "Puerto Escondido International Airport",
		City:      "Puerto Escondido",
		State:     "Oaxaca",
		Country:   "MX",
		Elevation: 294,
		Latitude:  15.8768997192,
		Longitude: -97.0891036987,
		Timezone:  "America/Mexico_City",
	},
	"MMQT": {
		ICAO:      "MMQT",
		IATA:      "QRO",
		Name:      "Queretaro Intercontinental Airport",
		City:      "Queretaro",
		State:     "Queretaro",
		Country:   "MX",
		Elevation: 6296,
		Latitude:  20.6173000336,
		Longitude: -100.185997009,
		Timezone:  "America/Mexico_City",
	},
	"MMRX": {
		ICAO:      "MMRX",
		IATA:      "REX",
		Name:      "General Lucio Blanco International Airport",
		City:      "Reynosa",
		State:     "Tamaulipas",
		Country:   "MX",
		Elevation: 139,
		Latitude:  26.0088996887,
		Longitude: -98.2285003662,
		Timezone:  "America/Matamoros",
	},
	"MMSC": {
		ICAO:      "MMSC",
		IATA:      "SZT",
		Name:      "San Cristobal De Las Casas Airport",
		State:     "Chiapas",
		Country:   "MX",
		Elevation: 7707,
		Latitude:  16.6902999878,
		Longitude: -92.5300979614,
		Timezone:  "America/Mexico_City",
	},
	"MMSD": {
		ICAO:      "MMSD",
		IATA:      "SJD",
		Name:      "Los Cabos International Airport",
		City:      "San Jose del Cabo",
		State:     "Baja-California-Sur",
		Country:   "MX",
		Elevation: 374,
		Latitude:  23.1518001556,
		Longitude: -109.7210006714,
		Timezone:  "America/Mazatlan",
	},
	"MMSF": {
		ICAO:      "MMSF",
		IATA:      "SFH",
		Name:      "San Felipe International Airport",
		State:     "Baja-California",
		Country:   "MX",
		Elevation: 148,
		Latitude:  30.9302005768,
		Longitude: -114.8089981079,
		Timezone:  "America/Tijuana",
	},
	"MMSL": {
		ICAO:      "MMSL",
		Name:      "Cabo San Lucas International Airport",
		City:      "Cabo San Lucas",
		State:     "Baja-California-Sur",
		Country:   "MX",
		Elevation: 459,
		Latitude:  22.9477005005,
		Longitude: -109.93699646,
		Timezone:  "America/Mazatlan",
	},
	"MMSM": {
		ICAO:      "MMSM",
		IATA:      "NLU",
		Name:      "Felipe Angeles International Airport",
		City:      "Santa Lucia",
		State:     "Mexico",
		Country:   "MX",
		Elevation: 7369,
		Latitude:  19.756667,
		Longitude: -99.015278,
		Timezone:  "America/Mexico_City",
	},
	"MMSP": {
		ICAO:      "MMSP",
		IATA:      "SLP",
		Name:      "Ponciano Arriaga International Airport",
		City:      "San Luis Potosi",
		State:     "San-Luis-Potosi",
		Country:   "MX",
		Elevation: 6035,
		Latitude:  22.2542991638,
		Longitude: -100.930999756,
		Timezone:  "America/Mexico_City",
	},
	"MMTA": {
		ICAO:      "MMTA",
		Name:      "Tlaxcala Airport",
		State:     "Tlaxcala",
		Country:   "MX",
		Elevation: 8229,
		Latitude:  19.5380001068,
		Longitude: -98.173500061,
		Timezone:  "America/Mexico_City",
	},
	"MMTB": {
		ICAO:      "MMTB",
		Name:      "Teran Air Base",
		City:      "Tuxtla Gutierrez",
		State:     "Chiapas",
		Country:   "MX",
		Elevation: 1909,
		Latitude:  16.739900589,
		Longitude: -93.1733016968,
		Timezone:  "America/Mexico_City",
	},
	"MMTC": {
		ICAO:      "MMTC",
		IATA:      "TRC",
		Name:      "Francisco Sarabia International Airport",
		City:      "Torreon",
		State:     "Coahuila",
		Country:   "MX",
		Elevation: 3688,
		Latitude:  25.5683002472,
		Longitude: -103.411003113,
		Timezone:  "America/Monterrey",
	},
	"MMTG": {
		ICAO:      "MMTG",
		IATA:      "TGZ",
		Name:      "Angel Albino Corzo International Airport",
		City:      "Tuxtla Gutierrez",
		State:     "Chiapas",
		Country:   "MX",
		Elevation: 1499,
		Latitude:  16.5636005402,
		Longitude: -93.0224990845,
		Timezone:  "America/Mexico_City",
	},
	"MMTJ": {
		ICAO:      "MMTJ",
		IATA:      "TIJ",
		Name:      "General Abelardo L. Rodriguez International Airport",
		City:      "Tijuana",
		State:     "Baja-California-Sur",
		Country:   "MX",
		Elevation: 489,
		Latitude:  32.5410995483,
		Longitude: -116.9700012207,
		Timezone:  "America/Los_Angeles",
	},
	"MMTM": {
		ICAO:      "MMTM",
		IATA:      "TAM",
		Name:      "General Francisco Javier Mina International Airport",
		City:      "Tampico",
		State:     "Tamaulipas",
		Country:   "MX",
		Elevation: 80,
		Latitude:  22.2964000702,
		Longitude: -97.8658981323,
		Timezone:  "America/Monterrey",
	},
	"MMTN": {
		ICAO:      "MMTN",
		IATA:      "TSL",
		Name:      "Tamuin Airport",
		State:     "San-Luis-Potosi",
		Country:   "MX",
		Elevation: 164,
		Latitude:  22.0382995605,
		Longitude: -98.8065032959,
		Timezone:  "America/Mexico_City",
	},
	"MMTO": {
		ICAO:      "MMTO",
		IATA:      "TLC",
		Name:      "Licenciado Adolfo Lopez Mateos International Airport",
		City:      "Toluca",
		State:     "Mexico",
		Country:   "MX",
		Elevation: 8466,
		Latitude:  19.3370990753,
		Longitude: -99.5660018921,
		Timezone:  "America/Mexico_City",
	},
	"MMTP": {
		ICAO:      "MMTP",
		IATA:      "TAP",
		Name:      "Tapachula International Airport",
		City:      "Tapachula",
		State:     "Chiapas",
		Country:   "MX",
		Elevation: 97,
		Latitude:  14.7943000793,
		Longitude: -92.3700027466,
		Timezone:  "America/Mexico_City",
	},
	"MMTX": {
		ICAO:      "MMTX",
		IATA:      "WIX",
		Name:      "Tuxpan Airport",
		State:     "Jalisco",
		Country:   "MX",
		Elevation: 4019,
		Latitude:  19.5839004517,
		Longitude: -103.3840026855,
		Timezone:  "America/Mexico_City",
	},
	"MMUN": {
		ICAO:      "MMUN",
		IATA:      "CUN",
		Name:      "Cancun International Airport",
		City:      "Cancun",
		State:     "Quintana-Roo",
		Country:   "MX",
		Elevation: 22,
		Latitude:  21.0365009308,
		Longitude: -86.8770980835,
		Timezone:  "America/Cancun",
	},
	"MMVA": {
		ICAO:      "MMVA",
		IATA:      "VSA",
		Name:      "Carlos Rovirosa Perez International Airport",
		City:      "Villahermosa",
		State:     "Tabasco",
		Country:   "MX",
		Elevation: 46,
		Latitude:  17.9969997406,
		Longitude: -92.8173980713,
		Timezone:  "America/Mexico_City",
	},
	"MMVR": {
		ICAO:      "MMVR",
		IATA:      "VER",
		Name:      "General Heriberto Jara International Airport",
		City:      "Veracruz",
		State:     "Veracruz",
		Country:   "MX",
		Elevation: 90,
		Latitude:  19.1459007263,
		Longitude: -96.1873016357,
		Timezone:  "America/Mexico_City",
	},
	"MMZC": {
		ICAO:      "MMZC",
		IATA:      "ZCL",
		Name:      "General Leobardo C. Ruiz International Airport",
		City:      "Zacatecas",
		State:     "Zacatecas",
		Country:   "MX",
		Elevation: 7141,
		Latitude:  22.8971004486,
		Longitude: -102.68699646,
		Timezone:  "America/Mexico_City",
	},
	"MMZH": {
		ICAO:      "MMZH",
		IATA:      "ZIH",
		Name:      "Ixtapa Zihuatanejo International Airport",
		City:      "Ixtapa",
		State:     "Guerrero",
		Country:   "MX",
		Elevation: 26,
		Latitude:  17.601600647,
		Longitude: -101.460998535,
		Timezone:  "America/Mexico_City",
	},
	"MMZM": {
		ICAO:      "MMZM",
		IATA:      "ZMM",
		Name:      "Zamora Airport",
		State:     "Michoacan",
		Country:   "MX",
		Elevation: 5141,
		Latitude:  20.0450000763,
		Longitude: -102.2760009766,
		Timezone:  "America/Mexico_City",
	},
	"MMZO": {
		ICAO:      "MMZO",
		IATA:      "ZLO",
		Name:      "Playa De Oro International Airport",
		City:      "Manzanillo",
		State:     "Colima",
		Country:   "MX",
		Elevation: 30,
		Latitude:  19.1448001862,
		Longitude: -104.558998108,
		Timezone:  "America/Mexico_City",
	},
	"MMZP": {
		ICAO:      "MMZP",
		Name:      "Zapopan Airport",
		State:     "Jalisco",
		Country:   "MX",
		Elevation: 5333,
		Latitude:  20.7558002472,
		Longitude: -103.4649963379,
		Timezone:  "America/Mexico_City",
	},
	"MN00": {
		ICAO:      "MN00",
		Name:      "Webb Lake Airport",
		City:      "Hackensack",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1359,
		Latitude:  46.95980072,
		Longitude: -94.42469788,
		Timezone:  "America/Chicago",
	},
	"MN01": {
		ICAO:      "MN01",
		Name:      "Hammars Farm Airport",
		City:      "Ada",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 886,
		Latitude:  47.2832984924,
		Longitude: -96.6169967651,
		Timezone:  "America/Chicago",
	},
	"MN04": {
		ICAO:      "MN04",
		Name:      "Aggies Landing Airport",
		City:      "Alexandria",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1387,
		Latitude:  45.773601532,
		Longitude: -95.262802124,
		Timezone:  "America/Chicago",
	},
	"MN06": {
		ICAO:      "MN06",
		Name:      "Pulkrabek Private Landing Field",
		City:      "Angus",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 850,
		Latitude:  48.0499992371,
		Longitude: -96.7836990356,
		Timezone:  "America/Chicago",
	},
	"MN08": {
		ICAO:      "MN08",
		Name:      "Eagles Nest Aerodrome",
		City:      "Eagle Lake",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1000,
		Latitude:  44.126499176,
		Longitude: -93.8718032837,
		Timezone:  "America/Chicago",
	},
	"MN11": {
		ICAO:      "MN11",
		Name:      "Lorenz Airport",
		City:      "Benson",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1050,
		Latitude:  45.2999992371,
		Longitude: -95.5449981689,
		Timezone:  "America/Chicago",
	},
	"MN12": {
		ICAO:      "MN12",
		Name:      "Robco Airport",
		City:      "Laporte",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1380,
		Latitude:  47.1988983154,
		Longitude: -94.7521972656,
		Timezone:  "America/Chicago",
	},
	"MN13": {
		ICAO:      "MN13",
		Name:      "Moberg Air Base",
		City:      "Bemidji",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1373,
		Latitude:  47.4961013794,
		Longitude: -94.952796936,
		Timezone:  "America/Chicago",
	},
	"MN15": {
		ICAO:      "MN15",
		Name:      "Empire Farm Strip",
		City:      "Bongards",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1000,
		Latitude:  44.7949981689,
		Longitude: -93.8538970947,
		Timezone:  "America/Chicago",
	},
	"MN17": {
		ICAO:      "MN17",
		Name:      "Jackson Field",
		City:      "Brainerd",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1225,
		Latitude:  46.27220154,
		Longitude: -94.23940277,
		Timezone:  "America/Chicago",
	},
	"MN18": {
		ICAO:      "MN18",
		Name:      "Barrett Airport",
		City:      "Brainerd",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1275,
		Latitude:  46.1781997681,
		Longitude: -94.0886001587,
		Timezone:  "America/Chicago",
	},
	"MN19": {
		ICAO:      "MN19",
		Name:      "Brandt Airport",
		City:      "Delano",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 930,
		Latitude:  45.0472984314,
		Longitude: -93.8364028931,
		Timezone:  "America/Chicago",
	},
	"MN20": {
		ICAO:      "MN20",
		Name:      "Runke's Field",
		City:      "Cedar Mills",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1100,
		Latitude:  44.9104995728,
		Longitude: -94.5714035034,
		Timezone:  "America/Chicago",
	},
	"MN23": {
		ICAO:      "MN23",
		Name:      "Dupre's Airport",
		City:      "Chisago",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 936,
		Latitude:  45.3157997131,
		Longitude: -92.8858032227,
		Timezone:  "America/Chicago",
	},
	"MN24": {
		ICAO:      "MN24",
		Name:      "Surfside Airport",
		City:      "Circle Pines",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 890,
		Latitude:  45.1500015259,
		Longitude: -93.116897583,
		Timezone:  "America/Chicago",
	},
	"MN28": {
		ICAO:      "MN28",
		Name:      "Lux Strip",
		City:      "Cosmos",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1120,
		Latitude:  44.983001709,
		Longitude: -94.7481002808,
		Timezone:  "America/Chicago",
	},
	"MN31": {
		ICAO:      "MN31",
		Name:      "L & M Aerodrome",
		City:      "Litchfield",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1205,
		Latitude:  45.0621986389,
		Longitude: -94.5792007446,
		Timezone:  "America/Chicago",
	},
	"MN32": {
		ICAO:      "MN32",
		Name:      "Nietz Airstrip",
		City:      "Oronoco",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1130,
		Latitude:  44.1913986206,
		Longitude: -92.4526977539,
		Timezone:  "America/Chicago",
	},
	"MN37": {
		ICAO:      "MN37",
		Name:      "Shelton's Private Airport",
		City:      "Duluth",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1320,
		Latitude:  46.9449005127,
		Longitude: -92.3571014404,
		Timezone:  "America/Chicago",
	},
	"MN38": {
		ICAO:      "MN38",
		Name:      "Spud Field",
		City:      "East Grand Forks",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 840,
		Latitude:  47.9249992371,
		Longitude: -96.9837036133,
		Timezone:  "America/Chicago",
	},
	"MN40": {
		ICAO:      "MN40",
		Name:      "Meadowvale Airport",
		City:      "Elk River",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 950,
		Latitude:  45.3624992371,
		Longitude: -93.5886001587,
		Timezone:  "America/Chicago",
	},
	"MN41": {
		ICAO:      "MN41",
		Name:      "Timmers Landing Field",
		City:      "Silver Lake",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1070,
		Latitude:  44.973428,
		Longitude: -94.214186,
		Timezone:  "America/Chicago",
	},
	"MN42": {
		ICAO:      "MN42",
		Name:      "Oak Lake Air Strip",
		City:      "Erskine",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1204,
		Latitude:  47.6585998535,
		Longitude: -95.9460983276,
		Timezone:  "America/Chicago",
	},
	"MN44": {
		ICAO:      "MN44",
		Name:      "Angen Field",
		City:      "Garfield",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1450,
		Latitude:  45.9364013672,
		Longitude: -95.5261993408,
		Timezone:  "America/Chicago",
	},
	"MN45": {
		ICAO:      "MN45",
		Name:      "Jennrich Field",
		City:      "Farmington",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 951,
		Latitude:  44.5661010742,
		Longitude: -93.1322021484,
		Timezone:  "America/Chicago",
	},
	"MN46": {
		ICAO:      "MN46",
		Name:      "Lucht Field",
		City:      "Farmington",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 900,
		Latitude:  44.6040992737,
		Longitude: -93.0919036865,
		Timezone:  "America/Chicago",
	},
	"MN47": {
		ICAO:      "MN47",
		Name:      "Cameron's Airport",
		City:      "Princeton",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 930,
		Latitude:  45.54970169,
		Longitude: -93.46980286,
		Timezone:  "America/Chicago",
	},
	"MN49": {
		ICAO:      "MN49",
		Name:      "Tuma Private Airport",
		City:      "Montgomery",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1100,
		Latitude:  44.4239006042,
		Longitude: -93.4843978882,
		Timezone:  "America/Chicago",
	},
	"MN50": {
		ICAO:      "MN50",
		Name:      "Arthur Field",
		City:      "Gatzke",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1150,
		Latitude:  48.3702011108,
		Longitude: -95.8069000244,
		Timezone:  "America/Chicago",
	},
	"MN51": {
		ICAO:      "MN51",
		Name:      "Bowers Airport",
		City:      "Stacy",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 909,
		Latitude:  45.4536018372,
		Longitude: -92.9747009277,
		Timezone:  "America/Chicago",
	},
	"MN52": {
		ICAO:      "MN52",
		Name:      "Gospel Ranch Airport",
		City:      "Hibbing",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1360,
		Latitude:  47.22909927,
		Longitude: -93.09210205,
		Timezone:  "America/Chicago",
	},
	"MN53": {
		ICAO:      "MN53",
		Name:      "Stanley Field",
		City:      "Graceville",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1112,
		Latitude:  45.5821990967,
		Longitude: -96.576499939,
		Timezone:  "America/Chicago",
	},
	"MN55": {
		ICAO:      "MN55",
		Name:      "Sandy Flats Airport",
		City:      "Hastings",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 835,
		Latitude:  44.7052001953,
		Longitude: -92.8118972778,
		Timezone:  "America/Chicago",
	},
	"MN57": {
		ICAO:      "MN57",
		Name:      "Bush Field",
		City:      "Hinckley",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1050,
		Latitude:  46.036072,
		Longitude: -92.988253,
		Timezone:  "America/Chicago",
	},
	"MN58": {
		ICAO:      "MN58",
		Name:      "Sky Meadow Airport",
		City:      "Hastings",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 820,
		Latitude:  44.6977005005,
		Longitude: -92.8640975952,
		Timezone:  "America/Chicago",
	},
	"MN60": {
		ICAO:      "MN60",
		Name:      "Itzen Air Strip",
		City:      "Herman",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1053,
		Latitude:  45.7633018494,
		Longitude: -96.2155990601,
		Timezone:  "America/Chicago",
	},
	"MN61": {
		ICAO:      "MN61",
		Name:      "Ward Airport",
		City:      "Hollandale",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1205,
		Latitude:  43.7602005005,
		Longitude: -93.1780014038,
		Timezone:  "America/Chicago",
	},
	"MN62": {
		ICAO:      "MN62",
		Name:      "Quast Airport",
		City:      "Hutchinson",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1050,
		Latitude:  44.8499984741,
		Longitude: -94.3503036499,
		Timezone:  "America/Chicago",
	},
	"MN63": {
		ICAO:      "MN63",
		Name:      "Stocker Private Airport",
		City:      "Jordan",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 855,
		Latitude:  44.6833000183,
		Longitude: -93.6369018555,
		Timezone:  "America/Chicago",
	},
	"MN64": {
		ICAO:      "MN64",
		Name:      "H Reder Field",
		City:      "Lakefield",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1504,
		Latitude:  43.6958007813,
		Longitude: -95.1725006104,
		Timezone:  "America/Chicago",
	},
	"MN66": {
		ICAO:      "MN66",
		Name:      "Ingleside Airport",
		City:      "Loretto",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1021,
		Latitude:  45.0630989075,
		Longitude: -93.6522979736,
		Timezone:  "America/Chicago",
	},
	"MN67": {
		ICAO:      "MN67",
		Name:      "Sky Park Airport",
		City:      "Lydia",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 940,
		Latitude:  44.6596984863,
		Longitude: -93.5183029175,
		Timezone:  "America/Chicago",
	},
	"MN68": {
		ICAO:      "MN68",
		Name:      "Pangerl Airport",
		City:      "Braham",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 990,
		Latitude:  45.7008018494,
		Longitude: -93.37159729,
		Timezone:  "America/Chicago",
	},
	"MN69": {
		ICAO:      "MN69",
		Name:      "Maple Airport",
		City:      "Maple Plain",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1026,
		Latitude:  45.0027008057,
		Longitude: -93.6718978882,
		Timezone:  "America/Chicago",
	},
	"MN71": {
		ICAO:      "MN71",
		Name:      "Ziermann Airport",
		City:      "Mayer",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 988,
		Latitude:  44.875,
		Longitude: -93.8918991089,
		Timezone:  "America/Chicago",
	},
	"MN74": {
		ICAO:      "MN74",
		Name:      "B & D Flyers International Airport",
		City:      "Minnesota Lake",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1055,
		Latitude:  43.8305015564,
		Longitude: -93.8404998779,
		Timezone:  "America/Chicago",
	},
	"MN77": {
		ICAO:      "MN77",
		Name:      "Jerger's Field",
		City:      "Moorhead",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 900,
		Latitude:  46.9235992432,
		Longitude: -96.7155990601,
		Timezone:  "America/Chicago",
	},
	"MN79": {
		ICAO:      "MN79",
		Name:      "Schroeder Airport",
		City:      "Becker",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 970,
		Latitude:  45.4314002991,
		Longitude: -93.8576965332,
		Timezone:  "America/Chicago",
	},
	"MN81": {
		ICAO:      "MN81",
		Name:      "J J and T Airport",
		City:      "East Grand Forks",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 828,
		Latitude:  47.988899231,
		Longitude: -97.0311965942,
		Timezone:  "America/Chicago",
	},
	"MN84": {
		ICAO:      "MN84",
		Name:      "Bachand Airport",
		City:      "Oklee",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1100,
		Latitude:  47.8138999939,
		Longitude: -95.891998291,
		Timezone:  "America/Chicago",
	},
	"MN85": {
		ICAO:      "MN85",
		Name:      "Swiderski Field",
		City:      "Onamia",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1250,
		Latitude:  46.0332984924,
		Longitude: -93.6613998413,
		Timezone:  "America/Chicago",
	},
	"MN86": {
		ICAO:      "MN86",
		Name:      "Sky Manor Aero Estates Airport",
		City:      "Park Rapids",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1492,
		Latitude:  47.0457992554,
		Longitude: -95.1224975586,
		Timezone:  "America/Chicago",
	},
	"MN87": {
		ICAO:      "MN87",
		Name:      "Cooks Landing Airport",
		City:      "Ham Lake",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 895,
		Latitude:  45.243598938,
		Longitude: -93.2217025757,
		Timezone:  "America/Chicago",
	},
	"MN88": {
		ICAO:      "MN88",
		Name:      "Empire Valley Airport",
		City:      "Paynesville",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1160,
		Latitude:  45.2868995667,
		Longitude: -94.6753005981,
		Timezone:  "America/Chicago",
	},
	"MN89": {
		ICAO:      "MN89",
		Name:      "Aerovilla Airport",
		City:      "Perham",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1374,
		Latitude:  46.6166000366,
		Longitude: -95.6502990723,
		Timezone:  "America/Chicago",
	},
	"MN91": {
		ICAO:      "MN91",
		Name:      "Reynolds Field",
		City:      "Princeton",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 985,
		Latitude:  45.5374984741,
		Longitude: -93.5140991211,
		Timezone:  "America/Chicago",
	},
	"MN92": {
		ICAO:      "MN92",
		Name:      "Lennartson Airport",
		City:      "Proctor",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1250,
		Latitude:  46.7499008179,
		Longitude: -92.300201416,
		Timezone:  "America/Chicago",
	},
	"MN93": {
		ICAO:      "MN93",
		Name:      "Chandler Field",
		City:      "Randall",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1200,
		Latitude:  46.1054992676,
		Longitude: -94.516998291,
		Timezone:  "America/Chicago",
	},
	"MN95": {
		ICAO:      "MN95",
		Name:      "Velo Airstrip",
		City:      "Rothsay",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1386,
		Latitude:  46.511100769,
		Longitude: -96.1920013428,
		Timezone:  "America/Chicago",
	},
	"MN96": {
		ICAO:      "MN96",
		Name:      "Rohwer Airport",
		City:      "Rushmore",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1640,
		Latitude:  43.5983009338,
		Longitude: -95.8170013428,
		Timezone:  "America/Chicago",
	},
	"MN97": {
		ICAO:      "MN97",
		Name:      "Thens Private Airstrip",
		City:      "St. Cloud",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1042,
		Latitude:  45.5901985168,
		Longitude: -94.2043991089,
		Timezone:  "America/Chicago",
	},
	"MN98": {
		ICAO:      "MN98",
		Name:      "Aero-Plain Airport",
		City:      "St Francis",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 931,
		Latitude:  45.4113998413,
		Longitude: -93.3846969604,
		Timezone:  "America/Chicago",
	},
	"MN99": {
		ICAO:      "MN99",
		Name:      "Serenity Airport",
		City:      "Lester Prairie",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1050,
		Latitude:  44.915599823,
		Longitude: -94.0888977051,
		Timezone:  "America/Chicago",
	},
	"MNAL": {
		ICAO:      "MNAL",
		Name:      "Alamicamba Airport",
		City:      "Alamicamba",
		State:     "Atlantico-Norte-(RAAN)",
		Country:   "NI",
		Elevation: 65,
		Latitude:  13.5122995377,
		Longitude: -84.2471008301,
		Timezone:  "America/Managua",
	},
	"MNAM": {
		ICAO:      "MNAM",
		Name:      "Altamira Airport",
		City:      "Altamira",
		State:     "Boaco",
		Country:   "NI",
		Elevation: 121,
		Latitude:  12.1106004715,
		Longitude: -85.7604980469,
		Timezone:  "America/Managua",
	},
	"MNBC": {
		ICAO:      "MNBC",
		Name:      "Boaco Airport",
		City:      "Boaco",
		State:     "Boaco",
		Country:   "NI",
		Elevation: 1184,
		Latitude:  12.4690999985,
		Longitude: -85.6625976563,
		Timezone:  "America/Managua",
	},
	"MNBL": {
		ICAO:      "MNBL",
		IATA:      "BEF",
		Name:      "Bluefields Airport",
		City:      "Bluefileds",
		State:     "Atlantico-Sur",
		Country:   "NI",
		Elevation: 20,
		Latitude:  11.9910001755,
		Longitude: -83.7741012573,
		Timezone:  "America/Managua",
	},
	"MNBR": {
		ICAO:      "MNBR",
		Name:      "Los Brasiles Airport",
		City:      "Los Brasiles",
		State:     "Managua",
		Country:   "NI",
		Elevation: 262,
		Latitude:  12.1899995804,
		Longitude: -86.3538970947,
		Timezone:  "America/Managua",
	},
	"MNBZ": {
		ICAO:      "MNBZ",
		IATA:      "BZA",
		Name:      "San Pedro Airport",
		City:      "Bonanza",
		Country:   "NI",
		Elevation: 600,
		Latitude:  13.9499998093,
		Longitude: -84.5999984741,
		Timezone:  "America/Managua",
	},
	"MNCH": {
		ICAO:      "MNCH",
		Name:      "Chinandega",
		City:      "Chinandega",
		State:     "Chinandega",
		Country:   "NI",
		Elevation: 174,
		Latitude:  12.6400003433,
		Longitude: -87.1373977661,
		Timezone:  "America/Managua",
	},
	"MNCI": {
		ICAO:      "MNCI",
		IATA:      "RNI",
		Name:      "Corn Island",
		City:      "Corn Island",
		Country:   "NI",
		Elevation: 1,
		Latitude:  12.162899971,
		Longitude: -83.0637969971,
		Timezone:  "America/Managua",
	},
	"MNCT": {
		ICAO:      "MNCT",
		Name:      "Corinto Airport",
		City:      "Chinandega",
		State:     "Chinandega",
		Country:   "NI",
		Elevation: 1,
		Latitude:  12.4788999557,
		Longitude: -87.1772003174,
		Timezone:  "America/Managua",
	},
	"MNDM": {
		ICAO:      "MNDM",
		Name:      "Dos Montes Airport",
		City:      "Dos Montes",
		State:     "Matagalpa",
		Country:   "NI",
		Elevation: 1469,
		Latitude:  12.8004999161,
		Longitude: -86.1483001709,
		Timezone:  "America/Managua",
	},
	"MNEP": {
		ICAO:      "MNEP",
		Name:      "La Esperanza Airport",
		City:      "La Esperanza",
		State:     "Atlantico-Sur",
		Country:   "NI",
		Elevation: 88,
		Latitude:  12.1999998093,
		Longitude: -84.2833328247,
		Timezone:  "America/Managua",
	},
	"MNES": {
		ICAO:      "MNES",
		Name:      "Esteli Airport",
		City:      "Esteli",
		State:     "Esteli",
		Country:   "NI",
		Elevation: 2811,
		Latitude:  13.111700058,
		Longitude: -86.3632965088,
		Timezone:  "America/Managua",
	},
	"MNFC": {
		ICAO:      "MNFC",
		Name:      "Punta Huete (Panchito) Airport",
		City:      "Punta Huete",
		State:     "Managua",
		Country:   "NI",
		Elevation: 217,
		Latitude:  12.3534002304,
		Longitude: -86.1828994751,
		Timezone:  "America/Managua",
	},
	"MNFF": {
		ICAO:      "MNFF",
		Name:      "El Bluff Airport",
		City:      "El Bluff",
		State:     "Atlantico-Sur",
		Country:   "NI",
		Elevation: 1,
		Latitude:  11.9942998886,
		Longitude: -83.6878967285,
		Timezone:  "America/Managua",
	},
	"MNHG": {
		ICAO:      "MNHG",
		Name:      "Hato Grande Airport",
		City:      "Hato Grande",
		State:     "Chontales",
		Country:   "NI",
		Elevation: 177,
		Latitude:  11.9813995361,
		Longitude: -85.4327011108,
		Timezone:  "America/Managua",
	},
	"MNJG": {
		ICAO:      "MNJG",
		Name:      "Jinotega",
		City:      "Jinotega",
		State:     "Jinotega",
		Country:   "NI",
		Elevation: 3232,
		Latitude:  13.0857000351,
		Longitude: -85.987701416,
		Timezone:  "America/Managua",
	},
	"MNJU": {
		ICAO:      "MNJU",
		Name:      "Juigalpa",
		City:      "Juigalpa",
		State:     "Chontales",
		Country:   "NI",
		Elevation: 295,
		Latitude:  12.1000003815,
		Longitude: -85.3666687012,
		Timezone:  "America/Managua",
	},
	"MNKW": {
		ICAO:      "MNKW",
		Name:      "Karawala Airport",
		City:      "Karawala",
		State:     "Atlantico-Sur",
		Country:   "NI",
		Elevation: 3,
		Latitude:  12.8921003342,
		Longitude: -83.582901001,
		Timezone:  "America/Managua",
	},
	"MNLN": {
		ICAO:      "MNLN",
		Name:      "Leon (Fanor Urroz) Airport",
		City:      "Leon",
		State:     "Leon",
		Country:   "NI",
		Elevation: 328,
		Latitude:  12.4292001724,
		Longitude: -86.9028015137,
		Timezone:  "America/Managua",
	},
	"MNMA": {
		ICAO:      "MNMA",
		Name:      "Macantaca Airport",
		City:      "Macantaca",
		State:     "Atlantico-Norte-(RAAN)",
		Country:   "NI",
		Elevation: 55,
		Latitude:  13.2451000214,
		Longitude: -84.0839004517,
		Timezone:  "America/Managua",
	},
	"MNMG": {
		ICAO:      "MNMG",
		IATA:      "MGA",
		Name:      "Augusto C. Sandino (Managua) International Airport",
		City:      "Managua",
		State:     "Managua",
		Country:   "NI",
		Elevation: 194,
		Latitude:  12.141500473,
		Longitude: -86.1681976318,
		Timezone:  "America/Managua",
	},
	"MNMR": {
		ICAO:      "MNMR",
		Name:      "Montelimar Airport",
		City:      "Montelimar",
		State:     "Managua",
		Country:   "NI",
		Elevation: 94,
		Latitude:  11.8051996231,
		Longitude: -86.5112991333,
		Timezone:  "America/Managua",
	},
	"MNNG": {
		ICAO:      "MNNG",
		IATA:      "NVG",
		Name:      "Nueva Guinea Airport",
		City:      "Nueva Guinea",
		Country:   "NI",
		Elevation: 606,
		Latitude:  11.6666669846,
		Longitude: -84.4499969482,
		Timezone:  "America/Managua",
	},
	"MNPC": {
		ICAO:      "MNPC",
		IATA:      "PUZ",
		Name:      "Puerto Cabezas Airport",
		City:      "Puerto Cabezas",
		State:     "Atlantico-Norte-(RAAN)",
		Country:   "NI",
		Elevation: 52,
		Latitude:  14.0472002029,
		Longitude: -83.3867034912,
		Timezone:  "America/Managua",
	},
	"MNPP": {
		ICAO:      "MNPP",
		Name:      "El Papalonal Airport",
		City:      "El Papalonal",
		State:     "Leon",
		Country:   "NI",
		Elevation: 141,
		Latitude:  12.4657001495,
		Longitude: -86.4679031372,
		Timezone:  "America/Managua",
	},
	"MNRS": {
		ICAO:      "MNRS",
		Name:      "Rivas",
		City:      "Rivas",
		State:     "Rivas",
		Country:   "NI",
		Elevation: 174,
		Latitude:  11.4277000427,
		Longitude: -85.8385009766,
		Timezone:  "America/Managua",
	},
	"MNRT": {
		ICAO:      "MNRT",
		IATA:      "RFS",
		Name:      "Rosita Airport",
		City:      "La Rosita",
		State:     "Atlantico-Norte-(RAAN)",
		Country:   "NI",
		Elevation: 193,
		Latitude:  13.8896999359,
		Longitude: -84.4088973999,
		Timezone:  "America/Managua",
	},
	"MNSC": {
		ICAO:      "MNSC",
		IATA:      "NCR",
		Name:      "San Carlos",
		City:      "San Carlos",
		State:     "Granada",
		Country:   "NI",
		Elevation: 91,
		Latitude:  11.1333999634,
		Longitude: -84.7699966431,
		Timezone:  "America/Managua",
	},
	"MNSI": {
		ICAO:      "MNSI",
		IATA:      "SIU",
		Name:      "Siuna",
		City:      "Siuna",
		Country:   "NI",
		Elevation: 606,
		Latitude:  13.7272224426,
		Longitude: -84.7777786255,
		Timezone:  "America/Managua",
	},
	"MNWP": {
		ICAO:      "MNWP",
		IATA:      "WSP",
		Name:      "Waspam Airport",
		City:      "Waspam",
		Country:   "NI",
		Elevation: 98,
		Latitude:  14.7391996384,
		Longitude: -83.9693984985,
		Timezone:  "America/Managua",
	},
	"MO00": {
		ICAO:      "MO00",
		Name:      "Turkey Mountain Estates Airport",
		City:      "Shell Knob",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1000,
		Latitude:  36.5917015076,
		Longitude: -93.6669006348,
		Timezone:  "America/Chicago",
	},
	"MO01": {
		ICAO:      "MO01",
		Name:      "Cuinche Airport",
		City:      "Strafford",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1475,
		Latitude:  37.25,
		Longitude: -93.1060028076,
		Timezone:  "America/Chicago",
	},
	"MO02": {
		ICAO:      "MO02",
		Name:      "Morgan Airport",
		City:      "Rolla",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1018,
		Latitude:  37.9166984558,
		Longitude: -91.6835021973,
		Timezone:  "America/Chicago",
	},
	"MO03": {
		ICAO:      "MO03",
		Name:      "Moders Airport",
		City:      "House Springs",
		State:     "Missouri",
		Country:   "US",
		Elevation: 465,
		Latitude:  38.4241981506,
		Longitude: -90.582901001,
		Timezone:  "America/Chicago",
	},
	"MO04": {
		ICAO:      "MO04",
		Name:      "Lyell Airport",
		City:      "Hunnewell",
		State:     "Missouri",
		Country:   "US",
		Elevation: 769,
		Latitude:  39.7167015076,
		Longitude: -91.8501968384,
		Timezone:  "America/Chicago",
	},
	"MO07": {
		ICAO:      "MO07",
		Name:      "Adventures Aloft Airport",
		City:      "Jonesburg",
		State:     "Missouri",
		Country:   "US",
		Elevation: 871,
		Latitude:  38.8725013733,
		Longitude: -91.300201416,
		Timezone:  "America/Chicago",
	},
	"MO08": {
		ICAO:      "MO08",
		Name:      "Homan Field",
		City:      "Sedalia",
		State:     "Missouri",
		Country:   "US",
		Elevation: 885,
		Latitude:  38.6627998352,
		Longitude: -93.3327026367,
		Timezone:  "America/Chicago",
	},
	"MO09": {
		ICAO:      "MO09",
		Name:      "Sharpe Farms Airport",
		City:      "Lewistown",
		State:     "Missouri",
		Country:   "US",
		Elevation: 702,
		Latitude:  40.0237007141,
		Longitude: -91.8738021851,
		Timezone:  "America/Chicago",
	},
	"MO10": {
		ICAO:      "MO10",
		Name:      "Hawk Air Airport",
		City:      "Sturgeon",
		State:     "Missouri",
		Country:   "US",
		Elevation: 870,
		Latitude:  39.2042999268,
		Longitude: -92.2623977661,
		Timezone:  "America/Chicago",
	},
	"MO11": {
		ICAO:      "MO11",
		Name:      "Barron Aviation Airport",
		City:      "Perry",
		State:     "Missouri",
		Country:   "US",
		Elevation: 745,
		Latitude:  39.4005012512,
		Longitude: -91.582901001,
		Timezone:  "America/Chicago",
	},
	"MO13": {
		ICAO:      "MO13",
		Name:      "Blue Hollow Airpark",
		City:      "Eldridge",
		State:     "Missouri",
		Country:   "US",
		Elevation: 820,
		Latitude:  37.7975997925,
		Longitude: -92.8292007446,
		Timezone:  "America/Chicago",
	},
	"MO14": {
		ICAO:      "MO14",
		Name:      "Marshall Field",
		City:      "Fair Grove",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1166,
		Latitude:  37.422778,
		Longitude: -93.208611,
		Timezone:  "America/Chicago",
	},
	"MO15": {
		ICAO:      "MO15",
		Name:      "Beckner Field",
		City:      "Marshfield",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1285,
		Latitude:  37.3875007629,
		Longitude: -92.9573974609,
		Timezone:  "America/Chicago",
	},
	"MO16": {
		ICAO:      "MO16",
		Name:      "Baugh Flight Park Ultralightport",
		City:      "Avilla",
		State:     "Missouri",
		Country:   "US",
		Elevation: 984,
		Latitude:  37.163898468,
		Longitude: -94.168296814,
		Timezone:  "America/Chicago",
	},
	"MO17": {
		ICAO:      "MO17",
		Name:      "Garst Airport",
		City:      "Watson",
		State:     "Missouri",
		Country:   "US",
		Elevation: 905,
		Latitude:  40.4791984558,
		Longitude: -95.6461029053,
		Timezone:  "America/Chicago",
	},
	"MO18": {
		ICAO:      "MO18",
		Name:      "Clevenger Airport",
		City:      "Blue Springs",
		State:     "Missouri",
		Country:   "US",
		Elevation: 990,
		Latitude:  38.9472007751,
		Longitude: -94.2490997314,
		Timezone:  "America/Chicago",
	},
	"MO22": {
		ICAO:      "MO22",
		Name:      "Jta Asbell Field",
		City:      "Carl Junction",
		State:     "Missouri",
		Country:   "US",
		Elevation: 940,
		Latitude:  37.1916999817,
		Longitude: -94.549697876,
		Timezone:  "America/Chicago",
	},
	"MO23": {
		ICAO:      "MO23",
		Name:      "Phillips Field",
		City:      "California",
		State:     "Missouri",
		Country:   "US",
		Elevation: 820,
		Latitude:  38.6333999634,
		Longitude: -92.5335006714,
		Timezone:  "America/Chicago",
	},
	"MO24": {
		ICAO:      "MO24",
		Name:      "Lonesome Sky Airport",
		City:      "Craig",
		State:     "Missouri",
		Country:   "US",
		Elevation: 868,
		Latitude:  40.1427993774,
		Longitude: -95.3882980347,
		Timezone:  "America/Chicago",
	},
	"MO25": {
		ICAO:      "MO25",
		Name:      "Show Me The Sky Airport",
		City:      "Barnard",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1050,
		Latitude:  40.1375007629,
		Longitude: -94.9255981445,
		Timezone:  "America/Chicago",
	},
	"MO26": {
		ICAO:      "MO26",
		Name:      "Peterson Farm Airport",
		City:      "Kearney",
		State:     "Missouri",
		Country:   "US",
		Elevation: 860,
		Latitude:  39.393901825,
		Longitude: -94.3529968262,
		Timezone:  "America/Chicago",
	},
	"MO27": {
		ICAO:      "MO27",
		Name:      "Ccc Airport",
		City:      "Cameron",
		State:     "Missouri",
		Country:   "US",
		Elevation: 984,
		Latitude:  39.7352981567,
		Longitude: -94.1927032471,
		Timezone:  "America/Chicago",
	},
	"MO29": {
		ICAO:      "MO29",
		Name:      "Inter-State Airport",
		City:      "Pleasant Hill",
		State:     "Missouri",
		Country:   "US",
		Elevation: 960,
		Latitude:  38.7722015381,
		Longitude: -94.3348999023,
		Timezone:  "America/Chicago",
	},
	"MO30": {
		ICAO:      "MO30",
		Name:      "Harbour Airport",
		City:      "Gravois Mills",
		State:     "Missouri",
		Country:   "US",
		Elevation: 823,
		Latitude:  38.2699012756,
		Longitude: -92.8115005493,
		Timezone:  "America/Chicago",
	},
	"MO31": {
		ICAO:      "MO31",
		Name:      "Malina Airport",
		City:      "Rogersville",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1350,
		Latitude:  37.0494995117,
		Longitude: -93.0724029541,
		Timezone:  "America/Chicago",
	},
	"MO32": {
		ICAO:      "MO32",
		Name:      "Table Rock Airport",
		City:      "Golden",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1053,
		Latitude:  36.5287017822,
		Longitude: -93.676902771,
		Timezone:  "America/Chicago",
	},
	"MO33": {
		ICAO:      "MO33",
		Name:      "Barlet's Base Airport",
		City:      "Carthage",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1068,
		Latitude:  37.1291999817,
		Longitude: -94.2537994385,
		Timezone:  "America/Chicago",
	},
	"MO34": {
		ICAO:      "MO34",
		Name:      "Ellingsen Field",
		City:      "Everton",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1115,
		Latitude:  37.3596992493,
		Longitude: -93.6624984741,
		Timezone:  "America/Chicago",
	},
	"MO35": {
		ICAO:      "MO35",
		Name:      "Harvey Airport",
		City:      "Eolia",
		State:     "Missouri",
		Country:   "US",
		Elevation: 710,
		Latitude:  39.2127990723,
		Longitude: -90.9173965454,
		Timezone:  "America/Chicago",
	},
	"MO36": {
		ICAO:      "MO36",
		Name:      "Riddle's Roost Airport",
		City:      "Foristell",
		State:     "Missouri",
		Country:   "US",
		Elevation: 650,
		Latitude:  38.8320007324,
		Longitude: -90.9251022339,
		Timezone:  "America/Chicago",
	},
	"MO37": {
		ICAO:      "MO37",
		Name:      "Ridgeview Ranch Airport",
		City:      "Harrisonville",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1000,
		Latitude:  38.7230987549,
		Longitude: -94.3818969727,
		Timezone:  "America/Chicago",
	},
	"MO39": {
		ICAO:      "MO39",
		Name:      "Baldwin Airport",
		City:      "Silex",
		State:     "Missouri",
		Country:   "US",
		Elevation: 505,
		Latitude:  39.1133995056,
		Longitude: -91.0418014526,
		Timezone:  "America/Chicago",
	},
	"MO42": {
		ICAO:      "MO42",
		Name:      "Wileys Air Strip",
		City:      "Gorin",
		State:     "Missouri",
		Country:   "US",
		Elevation: 662,
		Latitude:  40.3763999939,
		Longitude: -92.0724029541,
		Timezone:  "America/Chicago",
	},
	"MO43": {
		ICAO:      "MO43",
		Name:      "Wilkins Airport",
		City:      "Montgomery City",
		State:     "Missouri",
		Country:   "US",
		Elevation: 835,
		Latitude:  38.96849823,
		Longitude: -91.4434967041,
		Timezone:  "America/Chicago",
	},
	"MO45": {
		ICAO:      "MO45",
		Name:      "Sky-Vu Airport",
		City:      "Montgomery City",
		State:     "Missouri",
		Country:   "US",
		Elevation: 800,
		Latitude:  38.89220047,
		Longitude: -91.5660018921,
		Timezone:  "America/Chicago",
	},
	"MO46": {
		ICAO:      "MO46",
		Name:      "Walker Airport",
		City:      "Louisana",
		State:     "Missouri",
		Country:   "US",
		Elevation: 510,
		Latitude:  39.5203018188,
		Longitude: -91.1717987061,
		Timezone:  "America/Chicago",
	},
	"MO49": {
		ICAO:      "MO49",
		Name:      "Five Mile Airport",
		City:      "Joplin",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1100,
		Latitude:  36.9702987671,
		Longitude: -94.5766983032,
		Timezone:  "America/Chicago",
	},
	"MO51": {
		ICAO:      "MO51",
		Name:      "Schaback Strip",
		City:      "Weston",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1000,
		Latitude:  39.4235992432,
		Longitude: -94.8477020264,
		Timezone:  "America/Chicago",
	},
	"MO52": {
		ICAO:      "MO52",
		Name:      "Skyriders Airport",
		City:      "Wheatland",
		State:     "Missouri",
		Country:   "US",
		Elevation: 900,
		Latitude:  37.8666992188,
		Longitude: -93.3834991455,
		Timezone:  "America/Chicago",
	},
	"MO53": {
		ICAO:      "MO53",
		Name:      "Meadows Airport",
		City:      "Bethany",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1010,
		Latitude:  40.2332992554,
		Longitude: -93.9336013794,
		Timezone:  "America/Chicago",
	},
	"MO54": {
		ICAO:      "MO54",
		Name:      "Dudley Airport",
		City:      "Dudley",
		State:     "Missouri",
		Country:   "US",
		Elevation: 310,
		Latitude:  36.7820014954,
		Longitude: -90.0889968872,
		Timezone:  "America/Chicago",
	},
	"MO56": {
		ICAO:      "MO56",
		Name:      "Lost Mine Airport",
		City:      "Theodosia",
		State:     "Missouri",
		Country:   "US",
		Elevation: 820,
		Latitude:  36.5141983032,
		Longitude: -92.6401977539,
		Timezone:  "America/Chicago",
	},
	"MO57": {
		ICAO:      "MO57",
		Name:      "Henderson Mounds E B G Airport",
		City:      "East Prairie",
		State:     "Missouri",
		Country:   "US",
		Elevation: 295,
		Latitude:  36.7070007324,
		Longitude: -89.4666976929,
		Timezone:  "America/Chicago",
	},
	"MO58": {
		ICAO:      "MO58",
		Name:      "Waldemer Flying W Ranch Airport",
		City:      "Fairdealing",
		State:     "Missouri",
		Country:   "US",
		Elevation: 460,
		Latitude:  36.6584014893,
		Longitude: -90.6404037476,
		Timezone:  "America/Chicago",
	},
	"MO59": {
		ICAO:      "MO59",
		Name:      "K & N Field",
		City:      "Moscow Mills",
		State:     "Missouri",
		Country:   "US",
		Elevation: 500,
		Latitude:  38.9145011902,
		Longitude: -90.900100708,
		Timezone:  "America/Chicago",
	},
	"MO61": {
		ICAO:      "MO61",
		Name:      "Bel-Voir Acres Airport",
		City:      "Golden",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1139,
		Latitude:  36.5583992004,
		Longitude: -93.6418991089,
		Timezone:  "America/Chicago",
	},
	"MO62": {
		ICAO:      "MO62",
		Name:      "Hibbs Farm Airport",
		City:      "Sheridan",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1150,
		Latitude:  40.5102996826,
		Longitude: -94.5815963745,
		Timezone:  "America/Chicago",
	},
	"MO64": {
		ICAO:      "MO64",
		Name:      "Kimberling Airport",
		City:      "Kimberling City",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1050,
		Latitude:  36.6097984314,
		Longitude: -93.4446029663,
		Timezone:  "America/Chicago",
	},
	"MO65": {
		ICAO:      "MO65",
		Name:      "Misty Meadows Airport",
		City:      "Kirbyville",
		State:     "Missouri",
		Country:   "US",
		Elevation: 940,
		Latitude:  36.5834007263,
		Longitude: -93.1029968262,
		Timezone:  "America/Chicago",
	},
	"MO66": {
		ICAO:      "MO66",
		Name:      "Linden Air Airport",
		City:      "Ozark",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1280,
		Latitude:  37.059167,
		Longitude: -93.145556,
		Timezone:  "America/Chicago",
	},
	"MO67": {
		ICAO:      "MO67",
		Name:      "Hayes Field",
		City:      "Poplar Bluff",
		State:     "Missouri",
		Country:   "US",
		Elevation: 323,
		Latitude:  36.7112007141,
		Longitude: -90.3140029907,
		Timezone:  "America/Chicago",
	},
	"MO68": {
		ICAO:      "MO68",
		Name:      "Eads Ridge Airport",
		City:      "Fair Grove",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1310,
		Latitude:  37.4053001404,
		Longitude: -93.0996017456,
		Timezone:  "America/Chicago",
	},
	"MO71": {
		ICAO:      "MO71",
		Name:      "Block Air Village Airport",
		City:      "Holt",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1030,
		Latitude:  39.4631004333,
		Longitude: -94.418296814,
		Timezone:  "America/Chicago",
	},
	"MO72": {
		ICAO:      "MO72",
		Name:      "Flying 'J' Ranch Airport",
		City:      "Van Buren",
		State:     "Missouri",
		Country:   "US",
		Elevation: 690,
		Latitude:  37.0402984619,
		Longitude: -90.8667984009,
		Timezone:  "America/Chicago",
	},
	"MO73": {
		ICAO:      "MO73",
		Name:      "Rolling Shoals Farm Airport",
		City:      "Williamsville",
		State:     "Missouri",
		Country:   "US",
		Elevation: 450,
		Latitude:  36.9875984192,
		Longitude: -90.5584030151,
		Timezone:  "America/Chicago",
	},
	"MO74": {
		ICAO:      "MO74",
		Name:      "Cedar Creek Airport",
		City:      "Columbia",
		State:     "Missouri",
		Country:   "US",
		Elevation: 820,
		Latitude:  38.9222984314,
		Longitude: -92.1696014404,
		Timezone:  "America/Chicago",
	},
	"MO75": {
		ICAO:      "MO75",
		Name:      "Brownsberger Airport",
		City:      "Montrose",
		State:     "Missouri",
		Country:   "US",
		Elevation: 794,
		Latitude:  38.3050003052,
		Longitude: -94.0532989502,
		Timezone:  "America/Chicago",
	},
	"MO77": {
		ICAO:      "MO77",
		Name:      "Arnika Ranch Airport",
		City:      "Ava",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1320,
		Latitude:  36.9751014709,
		Longitude: -92.5427017212,
		Timezone:  "America/Chicago",
	},
	"MO78": {
		ICAO:      "MO78",
		Name:      "Stickle Cattle Farms Airport",
		City:      "Belle",
		State:     "Missouri",
		Country:   "US",
		Elevation: 700,
		Latitude:  38.3166999817,
		Longitude: -91.7835006714,
		Timezone:  "America/Chicago",
	},
	"MO79": {
		ICAO:      "MO79",
		Name:      "Sky Ranch Ii Airport",
		City:      "Belleview",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1060,
		Latitude:  37.6791992188,
		Longitude: -90.7154006958,
		Timezone:  "America/Chicago",
	},
	"MO80": {
		ICAO:      "MO80",
		Name:      "Johnston's Nest Aerodrome",
		City:      "Mountain Grove",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1482,
		Latitude:  37.1605987549,
		Longitude: -92.3235015869,
		Timezone:  "America/Chicago",
	},
	"MO81": {
		ICAO:      "MO81",
		Name:      "Dove Airstrip",
		City:      "Bismarck",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1080,
		Latitude:  37.6708984375,
		Longitude: -90.6147994995,
		Timezone:  "America/Chicago",
	},
	"MO82": {
		ICAO:      "MO82",
		Name:      "Howard Airport",
		City:      "Gorin",
		State:     "Missouri",
		Country:   "US",
		Elevation: 740,
		Latitude:  40.4089012146,
		Longitude: -92.0568008423,
		Timezone:  "America/Chicago",
	},
	"MO83": {
		ICAO:      "MO83",
		Name:      "Widmark Airport",
		City:      "Green City",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1045,
		Latitude:  40.2531013489,
		Longitude: -92.9549026489,
		Timezone:  "America/Chicago",
	},
	"MO84": {
		ICAO:      "MO84",
		Name:      "Viburnum Airport",
		City:      "Viburnum",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1272,
		Latitude:  37.7167015076,
		Longitude: -91.1334991455,
		Timezone:  "America/Chicago",
	},
	"MO85": {
		ICAO:      "MO85",
		Name:      "Wells Airport",
		City:      "Marshfield",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1530,
		Latitude:  37.251701355,
		Longitude: -92.8338012695,
		Timezone:  "America/Chicago",
	},
	"MO86": {
		ICAO:      "MO86",
		Name:      "Sanctuary Airport",
		City:      "Bellflower",
		State:     "Missouri",
		Country:   "US",
		Elevation: 750,
		Latitude:  38.9597015381,
		Longitude: -91.3039016724,
		Timezone:  "America/Chicago",
	},
	"MO87": {
		ICAO:      "MO87",
		Name:      "Powis Airport",
		City:      "Oak Grove",
		State:     "Missouri",
		Country:   "US",
		Elevation: 879,
		Latitude:  38.9458007813,
		Longitude: -94.1619033813,
		Timezone:  "America/Chicago",
	},
	"MO88": {
		ICAO:      "MO88",
		Name:      "Feutz Airport",
		City:      "Hatton",
		State:     "Missouri",
		Country:   "US",
		Elevation: 890,
		Latitude:  39.0999984741,
		Longitude: -92.0093002319,
		Timezone:  "America/Chicago",
	},
	"MO89": {
		ICAO:      "MO89",
		Name:      "Bar-Vik Airport",
		City:      "Laurie",
		State:     "Missouri",
		Country:   "US",
		Elevation: 930,
		Latitude:  38.1741981506,
		Longitude: -92.8584976196,
		Timezone:  "America/Chicago",
	},
	"MO93": {
		ICAO:      "MO93",
		Name:      "Keeven Air Ranch Airport",
		City:      "Hawk Point",
		State:     "Missouri",
		Country:   "US",
		Elevation: 700,
		Latitude:  39.044819,
		Longitude: -91.148094,
		Timezone:  "America/Chicago",
	},
	"MO94": {
		ICAO:      "MO94",
		Name:      "Starr Airport",
		City:      "Trimble",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1000,
		Latitude:  39.5,
		Longitude: -94.5802001953,
		Timezone:  "America/Chicago",
	},
	"MO95": {
		ICAO:      "MO95",
		Name:      "Hawkins Airport",
		City:      "Carrollton",
		State:     "Missouri",
		Country:   "US",
		Elevation: 720,
		Latitude:  39.4416999817,
		Longitude: -93.6622009277,
		Timezone:  "America/Chicago",
	},
	"MO96": {
		ICAO:      "MO96",
		Name:      "Ray Johnson Inc Airport",
		City:      "Centerville",
		State:     "Missouri",
		Country:   "US",
		Elevation: 950,
		Latitude:  37.4161987305,
		Longitude: -90.9635009766,
		Timezone:  "America/Chicago",
	},
	"MO98": {
		ICAO:      "MO98",
		Name:      "Williams Airport",
		City:      "Cleveland",
		State:     "Missouri",
		Country:   "US",
		Elevation: 985,
		Latitude:  38.7131004333,
		Longitude: -94.5849990845,
		Timezone:  "America/Chicago",
	},
	"MO99": {
		ICAO:      "MO99",
		Name:      "Miller Airport",
		City:      "Climax Springs",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1000,
		Latitude:  38.1164016724,
		Longitude: -93.0771026611,
		Timezone:  "America/Chicago",
	},
	"MP00": {
		ICAO:      "MP00",
		Name:      "Cap Justiniano Montenegro Airport",
		City:      "Pedasi",
		State:     "Los-Santos",
		Country:   "PA",
		Elevation: 16,
		Latitude:  7.5568799973,
		Longitude: -80.0233001709,
		Timezone:  "America/Panama",
	},
	"MP01": {
		ICAO:      "MP01",
		Name:      "Finca Ceiba Airport",
		City:      "Finca Jagua",
		State:     "Chiriqui",
		Country:   "PA",
		Elevation: 52,
		Latitude:  8.3549995422,
		Longitude: -82.8364028931,
		Timezone:  "America/Panama",
	},
	"MP02": {
		ICAO:      "MP02",
		Name:      "Finca 45 Airport",
		City:      "Dos Canos",
		State:     "Bocas-del-Toro",
		Country:   "PA",
		Elevation: 56,
		Latitude:  9.5433301926,
		Longitude: -82.7338027954,
		Timezone:  "America/Costa_Rica",
	},
	"MP03": {
		ICAO:      "MP03",
		Name:      "La Cabezona Airport",
		City:      "Guarumal",
		State:     "Chiriqui",
		Country:   "PA",
		Elevation: 31,
		Latitude:  8.3383903503,
		Longitude: -82.5035018921,
		Timezone:  "America/Panama",
	},
	"MP17": {
		ICAO:      "MP17",
		Name:      "Finca 67 Airport",
		City:      "Changuinola",
		State:     "Bocas-del-Toro",
		Country:   "PA",
		Elevation: 30,
		Latitude:  9.4344100952,
		Longitude: -82.4990997314,
		Timezone:  "America/Panama",
	},
	"MP18": {
		ICAO:      "MP18",
		Name:      "Penonome Airport",
		City:      "Penonome",
		State:     "Cocle",
		Country:   "PA",
		Elevation: 282,
		Latitude:  8.5038299561,
		Longitude: -80.3602981567,
		Timezone:  "America/Panama",
	},
	"MP21": {
		ICAO:      "MP21",
		Name:      "Alvaro Berroa Airport",
		City:      "Nueva California",
		State:     "Chiriqui",
		Country:   "PA",
		Elevation: 5000,
		Latitude:  8.7703895569,
		Longitude: -82.6643981934,
		Timezone:  "America/Panama",
	},
	"MP22": {
		ICAO:      "MP22",
		Name:      "Ingenio Santa Rosa Airport",
		City:      "Ingenio Santa Rosa",
		State:     "Cocle",
		Country:   "PA",
		Elevation: 109,
		Latitude:  8.1952199936,
		Longitude: -80.6586990356,
		Timezone:  "America/Panama",
	},
	"MP23": {
		ICAO:      "MP23",
		Name:      "Calzada Larga Airport",
		City:      "Calzada Larga",
		State:     "Panama",
		Country:   "PA",
		Elevation: 394,
		Latitude:  9.1662797928,
		Longitude: -79.5450973511,
		Timezone:  "America/Panama",
	},
	"MP24": {
		ICAO:      "MP24",
		Name:      "Chame Airport",
		City:      "La Paz",
		State:     "Panama-Oeste",
		Country:   "PA",
		Elevation: 141,
		Latitude:  8.5884599686,
		Longitude: -79.8897018433,
		Timezone:  "America/Panama",
	},
	"MP26": {
		ICAO:      "MP26",
		Name:      "Punta Cocos Airport",
		City:      "Punta Cocos",
		State:     "Panama",
		Country:   "PA",
		Elevation: 66,
		Latitude:  8.2248497009,
		Longitude: -78.9044036865,
		Timezone:  "America/Panama",
	},
	"MP27": {
		ICAO:      "MP27",
		Name:      "Deborah Airport",
		City:      "Guabito",
		State:     "Bocas-del-Toro",
		Country:   "PA",
		Elevation: 20,
		Latitude:  9.5161399841,
		Longitude: -82.5954971313,
		Timezone:  "America/Costa_Rica",
	},
	"MPBO": {
		ICAO:      "MPBO",
		IATA:      "BOC",
		Name:      "Bocas Del Toro International Airport",
		City:      "Isla Colon",
		State:     "Bocas-del-Toro",
		Country:   "PA",
		Elevation: 10,
		Latitude:  9.3408498764,
		Longitude: -82.2508010864,
		Timezone:  "America/Panama",
	},
	"MPCE": {
		ICAO:      "MPCE",
		IATA:      "CTD",
		Name:      "Alonso Valderrama Airport",
		City:      "Chitre",
		State:     "Herrera",
		Country:   "PA",
		Elevation: 33,
		Latitude:  7.9878401756,
		Longitude: -80.4096984863,
		Timezone:  "America/Panama",
	},
	"MPCH": {
		ICAO:      "MPCH",
		IATA:      "CHX",
		Name:      "Cap Manuel Nino International Airport",
		City:      "Changuinola",
		State:     "Bocas-del-Toro",
		Country:   "PA",
		Elevation: 19,
		Latitude:  9.4586400986,
		Longitude: -82.5167999268,
		Timezone:  "America/Panama",
	},
	"MPDA": {
		ICAO:      "MPDA",
		IATA:      "DAV",
		Name:      "Enrique Malek International Airport",
		City:      "David",
		State:     "Chiriqui",
		Country:   "PA",
		Elevation: 89,
		Latitude:  8.390999794,
		Longitude: -82.4349975586,
		Timezone:  "America/Panama",
	},
	"MPEJ": {
		ICAO:      "MPEJ",
		IATA:      "ONX",
		Name:      "Enrique Adolfo Jimenez Airport",
		City:      "Colon",
		State:     "Colon",
		Country:   "PA",
		Elevation: 25,
		Latitude:  9.3566398621,
		Longitude: -79.867401123,
		Timezone:  "America/Panama",
	},
	"MPFS": {
		ICAO:      "MPFS",
		Name:      "Fort Sherman Airport",
		City:      "Fort Sherman",
		State:     "Colon",
		Country:   "PA",
		Elevation: 10,
		Latitude:  9.3650903702,
		Longitude: -79.949798584,
		Timezone:  "America/Panama",
	},
	"MPHO": {
		ICAO:      "MPHO",
		IATA:      "BLB",
		Name:      "Howard Airport",
		City:      "Panama City",
		State:     "Panama",
		Country:   "PA",
		Elevation: 52,
		Latitude:  8.9147901535,
		Longitude: -79.5996017456,
		Timezone:  "America/Panama",
	},
	"MPJE": {
		ICAO:      "MPJE",
		IATA:      "JQE",
		Name:      "Jaque Airport",
		City:      "Jaque",
		State:     "Darien",
		Country:   "PA",
		Elevation: 29,
		Latitude:  7.5177798271,
		Longitude: -78.1572036743,
		Timezone:  "America/Panama",
	},
	"MPLP": {
		ICAO:      "MPLP",
		IATA:      "PLP",
		Name:      "Captain Ramon Xatruch Airport",
		City:      "La Palma",
		State:     "Darien",
		Country:   "PA",
		Elevation: 30,
		Latitude:  8.4066696167,
		Longitude: -78.1417007446,
		Timezone:  "America/Panama",
	},
	"MPMG": {
		ICAO:      "MPMG",
		IATA:      "PAC",
		Name:      "Marcos A. Gelabert International Airport",
		City:      "Albrook",
		State:     "Panama",
		Country:   "PA",
		Elevation: 31,
		Latitude:  8.9733400345,
		Longitude: -79.5556030273,
		Timezone:  "America/Panama",
	},
	"MPNU": {
		ICAO:      "MPNU",
		Name:      "Augusto Vergara Airport",
		City:      "Los Santos",
		State:     "Los-Santos",
		Country:   "PA",
		Elevation: 69,
		Latitude:  7.8573799133,
		Longitude: -80.2761001587,
		Timezone:  "America/Panama",
	},
	"MPOA": {
		ICAO:      "MPOA",
		IATA:      "PUE",
		Name:      "Puerto Obaldia Airport",
		City:      "Puerto Obaldia",
		State:     "Guna-Yala",
		Country:   "PA",
		Elevation: 223,
		Latitude:  8.6862602234,
		Longitude: -77.5243988037,
		Timezone:  "America/Panama",
	},
	"MPPA": {
		ICAO:      "MPPA",
		IATA:      "BLB",
		Name:      "Howard/Panama Pacifico International Airport",
		City:      "Panama City",
		State:     "Panama",
		Country:   "PA",
		Elevation: 52,
		Latitude:  8.9147901535,
		Longitude: -79.5996017456,
		Timezone:  "America/Panama",
	},
	"MPSA": {
		ICAO:      "MPSA",
		IATA:      "SYP",
		Name:      "Ruben Cantu Airport",
		City:      "Santiago",
		State:     "Veraguas",
		Country:   "PA",
		Elevation: 272,
		Latitude:  8.0855998993,
		Longitude: -80.9452972412,
		Timezone:  "America/Panama",
	},
	"MPSM": {
		ICAO:      "MPSM",
		IATA:      "RIH",
		Name:      "Scarlett Martinez International Airport",
		City:      "Rio Hato",
		State:     "Cocle",
		Country:   "PA",
		Elevation: 121,
		Latitude:  8.375833,
		Longitude: -80.127778,
		Timezone:  "America/Panama",
	},
	"MPTO": {
		ICAO:      "MPTO",
		IATA:      "PTY",
		Name:      "Tocumen International Airport",
		City:      "Tocumen",
		State:     "Panama",
		Country:   "PA",
		Elevation: 135,
		Latitude:  9.0713596344,
		Longitude: -79.3834991455,
		Timezone:  "America/Panama",
	},
	"MPVR": {
		ICAO:      "MPVR",
		IATA:      "PVE",
		Name:      "El Porvenir Airport",
		City:      "El Porvenir",
		State:     "Guna-Yala",
		Country:   "PA",
		Elevation: 17,
		Latitude:  9.5634202957,
		Longitude: -79.0040969849,
		Timezone:  "America/Panama",
	},
	"MPWN": {
		ICAO:      "MPWN",
		IATA:      "NBL",
		Name:      "San Blas Airport",
		City:      "Wannukandi",
		State:     "Guna-Yala",
		Country:   "PA",
		Elevation: 17,
		Latitude:  9.4496002197,
		Longitude: -78.9794998169,
		Timezone:  "America/Panama",
	},
	"MPZL": {
		ICAO:      "MPZL",
		Name:      "Finca 32 Airport",
		City:      "La Dalia",
		State:     "Bocas-del-Toro",
		Country:   "PA",
		Elevation: 23,
		Latitude:  9.4270896912,
		Longitude: -82.5626983643,
		Timezone:  "America/Panama",
	},
	"MRAD": {
		ICAO:      "MRAD",
		Name:      "Aerodamas Airport",
		City:      "Quepos",
		State:     "Puntarenas",
		Country:   "CR",
		Elevation: 18,
		Latitude:  9.4577770233,
		Longitude: -84.2089996338,
		Timezone:  "America/Costa_Rica",
	},
	"MRAJ": {
		ICAO:      "MRAJ",
		Name:      "Aranjuez Airport",
		City:      "Miramar",
		State:     "Puntarenas",
		Country:   "CR",
		Elevation: 52,
		Latitude:  10.0500001907,
		Longitude: -84.8000030518,
		Timezone:  "America/Costa_Rica",
	},
	"MRAM": {
		ICAO:      "MRAM",
		Name:      "Amubri Airport",
		City:      "Amubri",
		State:     "Limon",
		Country:   "CR",
		Elevation: 301,
		Latitude:  9.5216302872,
		Longitude: -82.9546966553,
		Timezone:  "America/Costa_Rica",
	},
	"MRAN": {
		ICAO:      "MRAN",
		IATA:      "FON",
		Name:      "Arenal Airport",
		City:      "La Fortuna/San Carlos",
		State:     "Alajuela",
		Country:   "CR",
		Elevation: 342,
		Latitude:  10.4779996872,
		Longitude: -84.6344985962,
		Timezone:  "America/Costa_Rica",
	},
	"MRAO": {
		ICAO:      "MRAO",
		IATA:      "TTQ",
		Name:      "Aerotortuguero Airport",
		City:      "Roxana",
		State:     "Limon",
		Country:   "CR",
		Elevation: 82,
		Latitude:  10.5690002441,
		Longitude: -83.5148010254,
		Timezone:  "America/Costa_Rica",
	},
	"MRAR": {
		ICAO:      "MRAR",
		Name:      "Atirro Airport",
		City:      "Turrialba",
		State:     "Cartago",
		Country:   "CR",
		Elevation: 1936,
		Latitude:  9.8500003815,
		Longitude: -83.6500015259,
		Timezone:  "America/Costa_Rica",
	},
	"MRAT": {
		ICAO:      "MRAT",
		Name:      "Altamira de San Carlos Airport",
		City:      "San Carlos",
		State:     "Alajuela",
		Country:   "CR",
		Elevation: 226,
		Latitude:  10.516699791,
		Longitude: -84.3666992188,
		Timezone:  "America/Costa_Rica",
	},
	"MRBA": {
		ICAO:      "MRBA",
		IATA:      "BAI",
		Name:      "Buenos Aires Airport",
		City:      "Punta Arenas",
		State:     "Puntarenas",
		Country:   "CR",
		Elevation: 1214,
		Latitude:  9.1636104584,
		Longitude: -83.3299026489,
		Timezone:  "America/Costa_Rica",
	},
	"MRBB": {
		ICAO:      "MRBB",
		Name:      "Babilonia Airport",
		City:      "Siquirres",
		State:     "Limon",
		Country:   "CR",
		Elevation: 591,
		Latitude:  10.1333332062,
		Longitude: -83.5833358765,
		Timezone:  "America/Costa_Rica",
	},
	"MRBC": {
		ICAO:      "MRBC",
		IATA:      "BCL",
		Name:      "Barra del Colorado Airport",
		City:      "Pococi",
		State:     "Limon",
		Country:   "CR",
		Elevation: 3,
		Latitude:  10.768699646,
		Longitude: -83.5856018066,
		Timezone:  "America/Costa_Rica",
	},
	"MRBM": {
		ICAO:      "MRBM",
		Name:      "Bremen Airport",
		City:      "Guacimo",
		State:     "Limon",
		Country:   "CR",
		Elevation: 102,
		Latitude:  10.219499588,
		Longitude: -83.5845031738,
		Timezone:  "America/Costa_Rica",
	},
	"MRBN": {
		ICAO:      "MRBN",
		Name:      "Bataan (Monte Libano) Airport",
		City:      "Limon",
		State:     "Limon",
		Country:   "CR",
		Elevation: 39,
		Latitude:  10.0833330154,
		Longitude: -83.3166656494,
		Timezone:  "America/Costa_Rica",
	},
	"MRBO": {
		ICAO:      "MRBO",
		Name:      "Boca Naranjo Airport",
		City:      "Quepos",
		State:     "Puntarenas",
		Country:   "CR",
		Elevation: 15,
		Latitude:  9.391960144,
		Longitude: -84.1259002686,
		Timezone:  "America/Costa_Rica",
	},
	"MRBP": {
		ICAO:      "MRBP",
		Name:      "Barra de Parismina Airport",
		City:      "Pococi",
		State:     "Limon",
		Country:   "CR",
		Elevation: 7,
		Latitude:  10.3000001907,
		Longitude: -83.3499984741,
		Timezone:  "America/Costa_Rica",
	},
	"MRCA": {
		ICAO:      "MRCA",
		IATA:      "CSC",
		Name:      "Codela Airport",
		City:      "Guapiles",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 328,
		Latitude:  10.4139995575,
		Longitude: -85.0916976929,
		Timezone:  "America/Costa_Rica",
	},
	"MRCC": {
		ICAO:      "MRCC",
		IATA:      "OTR",
		Name:      "Coto 47 Airport",
		City:      "Corredores",
		State:     "Puntarenas",
		Country:   "CR",
		Elevation: 26,
		Latitude:  8.601559639,
		Longitude: -82.9685974121,
		Timezone:  "America/Costa_Rica",
	},
	"MRCE": {
		ICAO:      "MRCE",
		Name:      "Carate Airport",
		City:      "Golfito",
		State:     "Puntarenas",
		Country:   "CR",
		Elevation: 16,
		Latitude:  8.445110321,
		Longitude: -83.4649963379,
		Timezone:  "America/Costa_Rica",
	},
	"MRCH": {
		ICAO:      "MRCH",
		IATA:      "JAP",
		Name:      "Chacarita Airport",
		City:      "Puntarenas",
		State:     "Puntarenas",
		Country:   "CR",
		Elevation: 7,
		Latitude:  9.9814100266,
		Longitude: -84.7726974487,
		Timezone:  "America/Costa_Rica",
	},
	"MRCI": {
		ICAO:      "MRCI",
		Name:      "Ciruelas Airport",
		City:      "Bagaces",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 361,
		Latitude:  10.516667366,
		Longitude: -85.3499984741,
		Timezone:  "America/Costa_Rica",
	},
	"MRCL": {
		ICAO:      "MRCL",
		Name:      "Coyolar Airport",
		City:      "Liberia",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 262,
		Latitude:  10.667222023,
		Longitude: -85.511390686,
		Timezone:  "America/Costa_Rica",
	},
	"MRCO": {
		ICAO:      "MRCO",
		Name:      "El Cerrito Airport",
		City:      "Filadelfia",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 59,
		Latitude:  10.4535999298,
		Longitude: -85.5581970215,
		Timezone:  "America/Costa_Rica",
	},
	"MRCR": {
		ICAO:      "MRCR",
		IATA:      "RIK",
		Name:      "Carrillo Airport",
		City:      "Nicoya",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 6,
		Latitude:  9.8705101013,
		Longitude: -85.4813995361,
		Timezone:  "America/Costa_Rica",
	},
	"MRCV": {
		ICAO:      "MRCV",
		Name:      "Cabo Velas Airport",
		City:      "Nicoya",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 33,
		Latitude:  10.3556995392,
		Longitude: -85.852897644,
		Timezone:  "America/Costa_Rica",
	},
	"MRCZ": {
		ICAO:      "MRCZ",
		Name:      "Carrizal Airport",
		City:      "Carrizal",
		State:     "Alajuela",
		Country:   "CR",
		Elevation: 4650,
		Latitude:  10.0875997543,
		Longitude: -84.1695022583,
		Timezone:  "America/Costa_Rica",
	},
	"MRDC": {
		ICAO:      "MRDC",
		Name:      "Duacari 2 Airport",
		City:      "Guapiles",
		State:     "Limon",
		Country:   "CR",
		Elevation: 79,
		Latitude:  10.3514995575,
		Longitude: -83.6305007935,
		Timezone:  "America/Costa_Rica",
	},
	"MRDD": {
		ICAO:      "MRDD",
		Name:      "Don Diego Airport",
		City:      "Limon",
		State:     "Limon",
		Country:   "CR",
		Elevation: 161,
		Latitude:  9.5500001907,
		Longitude: -82.8833312988,
		Timezone:  "America/Costa_Rica",
	},
	"MRDK": {
		ICAO:      "MRDK",
		IATA:      "DRK",
		Name:      "Drake Bay Airport",
		City:      "Puntarenas",
		State:     "Puntarenas",
		Country:   "CR",
		Elevation: 12,
		Latitude:  8.7188901901,
		Longitude: -83.6417007446,
		Timezone:  "America/Costa_Rica",
	},
	"MRDM": {
		ICAO:      "MRDM",
		Name:      "Dos Marias Airport",
		City:      "Chorotega",
		State:     "Puntarenas",
		Country:   "CR",
		Elevation: 16,
		Latitude:  9.9279098511,
		Longitude: -84.9899978638,
		Timezone:  "America/Costa_Rica",
	},
	"MRDO": {
		ICAO:      "MRDO",
		Name:      "Dieciocho Airport",
		City:      "Puntarenas",
		State:     "Puntarenas",
		Country:   "CR",
		Elevation: 19,
		Latitude:  8.9036998749,
		Longitude: -83.4298019409,
		Timezone:  "America/Costa_Rica",
	},
	"MREC": {
		ICAO:      "MREC",
		Name:      "El Carmen de Siquirres Airport",
		City:      "Siquirres",
		State:     "Limon",
		Country:   "CR",
		Elevation: 56,
		Latitude:  10.2019996643,
		Longitude: -83.4721984863,
		Timezone:  "America/Costa_Rica",
	},
	"MRED": {
		ICAO:      "MRED",
		Name:      "El Descanso de Poco Sol Airport",
		City:      "San Carlos",
		State:     "Alajuela",
		Country:   "CR",
		Elevation: 23,
		Latitude:  10.8667001724,
		Longitude: -84.5333023071,
		Timezone:  "America/Costa_Rica",
	},
	"MREO": {
		ICAO:      "MREO",
		Name:      "El Ceibo Airport",
		City:      "Ticaban",
		State:     "Heredia",
		Country:   "CR",
		Elevation: 33,
		Latitude:  10.5333328247,
		Longitude: -83.8499984741,
		Timezone:  "America/Costa_Rica",
	},
	"MRER": {
		ICAO:      "MRER",
		Name:      "El Ron Ron Airport",
		City:      "Ron Ron",
		State:     "Alajuela",
		Country:   "CR",
		Elevation: 2109,
		Latitude:  10.3002996445,
		Longitude: -84.4501037598,
		Timezone:  "America/Costa_Rica",
	},
	"MRET": {
		ICAO:      "MRET",
		Name:      "Esterillos / Finca Airport",
		City:      "Parrita",
		State:     "Puntarenas",
		Country:   "CR",
		Elevation: 20,
		Latitude:  9.5333328247,
		Longitude: -84.4666671753,
		Timezone:  "America/Costa_Rica",
	},
	"MRFD": {
		ICAO:      "MRFD",
		Name:      "Finca Delicias Airport",
		City:      "Puntarenas",
		State:     "Puntarenas",
		Country:   "CR",
		Elevation: 26,
		Latitude:  8.9448299408,
		Longitude: -83.5589981079,
		Timezone:  "America/Costa_Rica",
	},
	"MRFI": {
		ICAO:      "MRFI",
		Name:      "Finca 10 / Nuevo Palmar Sur Airport",
		City:      "Puntarenas",
		State:     "Puntarenas",
		Country:   "CR",
		Elevation: 25,
		Latitude:  8.9163503647,
		Longitude: -83.5073013306,
		Timezone:  "America/Costa_Rica",
	},
	"MRFL": {
		ICAO:      "MRFL",
		IATA:      "FMG",
		Name:      "Flamingo Airport",
		City:      "Brasilito",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 270,
		Latitude:  10.4186000824,
		Longitude: -85.782699585,
		Timezone:  "America/Costa_Rica",
	},
	"MRFP": {
		ICAO:      "MRFP",
		Name:      "Frutez - Pital Airport",
		City:      "San Carlos",
		State:     "Alajuela",
		Country:   "CR",
		Elevation: 617,
		Latitude:  10.4474000931,
		Longitude: -84.3690032959,
		Timezone:  "America/Costa_Rica",
	},
	"MRFS": {
		ICAO:      "MRFS",
		Name:      "Finca 63 (Coto 63) Airport",
		City:      "Puntarenas",
		State:     "Puntarenas",
		Country:   "CR",
		Elevation: 36,
		Latitude:  8.6525001526,
		Longitude: -83.0652999878,
		Timezone:  "America/Costa_Rica",
	},
	"MRGA": {
		ICAO:      "MRGA",
		Name:      "Garza Airport",
		City:      "Garza",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 3,
		Latitude:  9.9167003632,
		Longitude: -85.6333007813,
		Timezone:  "America/Costa_Rica",
	},
	"MRGF": {
		ICAO:      "MRGF",
		IATA:      "GLF",
		Name:      "Golfito Airport",
		City:      "Golfito",
		State:     "Puntarenas",
		Country:   "CR",
		Elevation: 49,
		Latitude:  8.654009819,
		Longitude: -83.1821975708,
		Timezone:  "America/Costa_Rica",
	},
	"MRGP": {
		ICAO:      "MRGP",
		IATA:      "GPL",
		Name:      "Guapiles Airport",
		City:      "Pococi",
		State:     "Limon",
		Country:   "CR",
		Elevation: 883,
		Latitude:  10.2172002792,
		Longitude: -83.7969970703,
		Timezone:  "America/Costa_Rica",
	},
	"MRGT": {
		ICAO:      "MRGT",
		Name:      "Guatuso Airport",
		City:      "Alajuela",
		State:     "Alajuela",
		Country:   "CR",
		Elevation: 164,
		Latitude:  10.6833333969,
		Longitude: -84.8333358765,
		Timezone:  "America/Costa_Rica",
	},
	"MRHG": {
		ICAO:      "MRHG",
		Name:      "Hacienda Rancho Grande Airport",
		City:      "Sarapiqui",
		State:     "Heredia",
		Country:   "CR",
		Elevation: 98,
		Latitude:  10.4499998093,
		Longitude: -84.0833358765,
		Timezone:  "America/Costa_Rica",
	},
	"MRHH": {
		ICAO:      "MRHH",
		Name:      "Hacienda Homuha Airport",
		City:      "San Carlos",
		State:     "Alajuela",
		Country:   "CR",
		Elevation: 230,
		Latitude:  10.8000001907,
		Longitude: -84.3666687012,
		Timezone:  "America/Costa_Rica",
	},
	"MRHO": {
		ICAO:      "MRHO",
		Name:      "Hacienda Rio Cuarto Airport",
		City:      "Sarapiqui",
		State:     "Alajuela",
		Country:   "CR",
		Elevation: 197,
		Latitude:  10.5666666031,
		Longitude: -84.1500015259,
		Timezone:  "America/Costa_Rica",
	},
	"MRHP": {
		ICAO:      "MRHP",
		Name:      "Hacienda La Pacifica Airport",
		City:      "Canas",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 164,
		Latitude:  10.4499998093,
		Longitude: -85.1500015259,
		Timezone:  "America/Costa_Rica",
	},
	"MRHS": {
		ICAO:      "MRHS",
		Name:      "Hacienda La Suerte Airport",
		City:      "Pococi",
		State:     "Limon",
		Country:   "CR",
		Elevation: 492,
		Latitude:  10.4499998093,
		Longitude: -83.6666641235,
		Timezone:  "America/Costa_Rica",
	},
	"MRIA": {
		ICAO:      "MRIA",
		IATA:      "PBP",
		Name:      "Islita Airport",
		City:      "Nandayure",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 7,
		Latitude:  9.8561096191,
		Longitude: -85.3707962036,
		Timezone:  "America/Costa_Rica",
	},
	"MRIS": {
		ICAO:      "MRIS",
		Name:      "Las Islas Airport",
		City:      "San Carlos",
		State:     "Heredia",
		Country:   "CR",
		Elevation: 98,
		Latitude:  10.5583000183,
		Longitude: -83.9740982056,
		Timezone:  "America/Costa_Rica",
	},
	"MRJO": {
		ICAO:      "MRJO",
		Name:      "Jaco Airport",
		City:      "Jaco",
		State:     "Puntarenas",
		Country:   "CR",
		Elevation: 20,
		Latitude:  9.555270195,
		Longitude: -84.5563964844,
		Timezone:  "America/Costa_Rica",
	},
	"MRLA": {
		ICAO:      "MRLA",
		Name:      "La Zampona Airport",
		City:      "Guanacaste",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 328,
		Latitude:  10.266667366,
		Longitude: -85.4333343506,
		Timezone:  "America/Costa_Rica",
	},
	"MRLB": {
		ICAO:      "MRLB",
		IATA:      "LIR",
		Name:      "Daniel Oduber Quiros International Airport",
		City:      "Liberia",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 270,
		Latitude:  10.5932998657,
		Longitude: -85.5444030762,
		Timezone:  "America/Costa_Rica",
	},
	"MRLC": {
		ICAO:      "MRLC",
		IATA:      "LSL",
		Name:      "Los Chiles Airport",
		City:      "Los Chiles",
		State:     "Alajuela",
		Country:   "CR",
		Elevation: 131,
		Latitude:  11.0353002548,
		Longitude: -84.7061004639,
		Timezone:  "America/Costa_Rica",
	},
	"MRLE": {
		ICAO:      "MRLE",
		Name:      "Laurel Airport",
		City:      "Puntarenas",
		State:     "Puntarenas",
		Country:   "CR",
		Elevation: 65,
		Latitude:  8.4404096603,
		Longitude: -82.90650177,
		Timezone:  "America/Costa_Rica",
	},
	"MRLF": {
		ICAO:      "MRLF",
		Name:      "La Flor Airport",
		City:      "Liberia",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 177,
		Latitude:  10.6499996185,
		Longitude: -85.5329971313,
		Timezone:  "America/Costa_Rica",
	},
	"MRLG": {
		ICAO:      "MRLG",
		Name:      "La Garroba Airport",
		City:      "Upala",
		State:     "Alajuela",
		Country:   "CR",
		Elevation: 295,
		Latitude:  10.8000001907,
		Longitude: -84.9166641235,
		Timezone:  "America/Costa_Rica",
	},
	"MRLI": {
		ICAO:      "MRLI",
		Name:      "La Ligia Airport",
		City:      "Parrita",
		State:     "Puntarenas",
		Country:   "CR",
		Elevation: 16,
		Latitude:  9.5152702332,
		Longitude: -84.3014984131,
		Timezone:  "America/Costa_Rica",
	},
	"MRLL": {
		ICAO:      "MRLL",
		Name:      "Las Lomas Airport",
		City:      "Siquirres",
		State:     "Limon",
		Country:   "CR",
		Elevation: 1325,
		Latitude:  10.0666666031,
		Longitude: -83.5833358765,
		Timezone:  "America/Costa_Rica",
	},
	"MRLM": {
		ICAO:      "MRLM",
		IATA:      "LIO",
		Name:      "Limon International Airport",
		City:      "Puerto Limon",
		State:     "Limon",
		Country:   "CR",
		Elevation: 7,
		Latitude:  9.9579601288,
		Longitude: -83.0220031738,
		Timezone:  "America/Costa_Rica",
	},
	"MRLN": {
		ICAO:      "MRLN",
		Name:      "La Guinea Airport",
		City:      "Nicoya",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 33,
		Latitude:  10.4207000732,
		Longitude: -85.4720001221,
		Timezone:  "America/Costa_Rica",
	},
	"MRLP": {
		ICAO:      "MRLP",
		Name:      "Las Piedras Airport",
		City:      "Canas",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 82,
		Latitude:  10.3666667938,
		Longitude: -85.1999969482,
		Timezone:  "America/Costa_Rica",
	},
	"MRLT": {
		ICAO:      "MRLT",
		Name:      "Las Trancas Airport",
		City:      "Las Trancas",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 68,
		Latitude:  10.5670003891,
		Longitude: -85.5999984741,
		Timezone:  "America/Costa_Rica",
	},
	"MRLV": {
		ICAO:      "MRLV",
		Name:      "La Cueva Airport",
		City:      "Liberia",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 230,
		Latitude:  10.6812000275,
		Longitude: -85.5280990601,
		Timezone:  "America/Costa_Rica",
	},
	"MRLY": {
		ICAO:      "MRLY",
		Name:      "La Yolanda Airport",
		City:      "Parrita",
		State:     "Puntarenas",
		Country:   "CR",
		Elevation: 13,
		Latitude:  9.5500001907,
		Longitude: -84.5666656494,
		Timezone:  "America/Costa_Rica",
	},
	"MRLZ": {
		ICAO:      "MRLZ",
		Name:      "La Zopilota Airport",
		City:      "Guanacaste",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 39,
		Latitude:  10.4835996628,
		Longitude: -85.4619979858,
		Timezone:  "America/Costa_Rica",
	},
	"MRMA": {
		ICAO:      "MRMA",
		Name:      "Monte Alto Airport",
		City:      "Abangares",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 66,
		Latitude:  10.0833330154,
		Longitude: -85.2666702271,
		Timezone:  "America/Costa_Rica",
	},
	"MRMC": {
		ICAO:      "MRMC",
		Name:      "Murcielago Airport",
		City:      "La Cruz",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 66,
		Latitude:  10.9166669846,
		Longitude: -85.7166671753,
		Timezone:  "America/Costa_Rica",
	},
	"MRMJ": {
		ICAO:      "MRMJ",
		IATA:      "CSC",
		Name:      "Mojica Airport",
		City:      "Canas",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 230,
		Latitude:  10.4307003021,
		Longitude: -85.1745986938,
		Timezone:  "America/Costa_Rica",
	},
	"MRNC": {
		ICAO:      "MRNC",
		IATA:      "NCT",
		Name:      "Guanacaste Airport",
		City:      "Nicoya/Guanacate",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 365,
		Latitude:  10.1393995285,
		Longitude: -85.4458007813,
		Timezone:  "America/Costa_Rica",
	},
	"MRNS": {
		ICAO:      "MRNS",
		IATA:      "NOB",
		Name:      "Nosara Airport",
		City:      "Nicoya",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 33,
		Latitude:  9.9764900208,
		Longitude: -85.6529998779,
		Timezone:  "America/Costa_Rica",
	},
	"MROC": {
		ICAO:      "MROC",
		IATA:      "SJO",
		Name:      "Juan Santamaria International Airport",
		City:      "San Jose",
		State:     "Alajuela",
		Country:   "CR",
		Elevation: 3021,
		Latitude:  9.9938602448,
		Longitude: -84.2088012695,
		Timezone:  "America/Costa_Rica",
	},
	"MRPA": {
		ICAO:      "MRPA",
		Name:      "Palo Arco Airport",
		City:      "Nandayure",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 324,
		Latitude:  9.8514099121,
		Longitude: -85.2378997803,
		Timezone:  "America/Costa_Rica",
	},
	"MRPB": {
		ICAO:      "MRPB",
		Name:      "Playa Blanca (J. W. Berteus) Airport",
		City:      "Puntarenas",
		State:     "Puntarenas",
		Country:   "CR",
		Elevation: 6,
		Latitude:  8.6499996185,
		Longitude: -83.4329986572,
		Timezone:  "America/Costa_Rica",
	},
	"MRPC": {
		ICAO:      "MRPC",
		Name:      "Paso Canoas Airport",
		City:      "Paso Canoas",
		State:     "Puntarenas",
		Country:   "CR",
		Elevation: 324,
		Latitude:  8.9577798843,
		Longitude: -83.5503005981,
		Timezone:  "America/Costa_Rica",
	},
	"MRPD": {
		ICAO:      "MRPD",
		Name:      "Pandora Airport",
		City:      "Limon",
		State:     "Limon",
		Country:   "CR",
		Elevation: 98,
		Latitude:  9.732170105,
		Longitude: -82.9832000732,
		Timezone:  "America/Costa_Rica",
	},
	"MRPJ": {
		ICAO:      "MRPJ",
		IATA:      "PJM",
		Name:      "Puerto Jimenez Airport",
		City:      "Puerto Jimenez",
		State:     "Puntarenas",
		Country:   "CR",
		Elevation: 7,
		Latitude:  8.5333299637,
		Longitude: -83.3000030518,
		Timezone:  "America/Costa_Rica",
	},
	"MRPL": {
		ICAO:      "MRPL",
		Name:      "Portalon Airport",
		City:      "Portalon",
		State:     "Puntarenas",
		Country:   "CR",
		Elevation: 206,
		Latitude:  9.3579797745,
		Longitude: -83.9794998169,
		Timezone:  "America/Costa_Rica",
	},
	"MRPM": {
		ICAO:      "MRPM",
		IATA:      "PMZ",
		Name:      "Palmar Sur Airport",
		City:      "Palmar Sur",
		State:     "Puntarenas",
		Country:   "CR",
		Elevation: 49,
		Latitude:  8.9510297775,
		Longitude: -83.4685974121,
		Timezone:  "America/Costa_Rica",
	},
	"MRPN": {
		ICAO:      "MRPN",
		Name:      "Pelon Nuevo Airport",
		City:      "Liberia",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 49,
		Latitude:  10.486700058,
		Longitude: -85.4141998291,
		Timezone:  "America/Costa_Rica",
	},
	"MRPO": {
		ICAO:      "MRPO",
		Name:      "Punta Banco Airport",
		City:      "Puntarenas",
		State:     "Puntarenas",
		Country:   "CR",
		Elevation: 16,
		Latitude:  8.3573102951,
		Longitude: -83.1369018555,
		Timezone:  "America/Costa_Rica",
	},
	"MRPP": {
		ICAO:      "MRPP",
		Name:      "Playa Pajaros Airport",
		City:      "Paquera",
		State:     "Puntarenas",
		Country:   "CR",
		Elevation: 30,
		Latitude:  9.8583097458,
		Longitude: -84.6943969727,
		Timezone:  "America/Costa_Rica",
	},
	"MRPS": {
		ICAO:      "MRPS",
		Name:      "Penas Blancas Airport",
		City:      "Santa Cruz",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 98,
		Latitude:  10.3156003952,
		Longitude: -85.6780014038,
		Timezone:  "America/Costa_Rica",
	},
	"MRPT": {
		ICAO:      "MRPT",
		Name:      "Agropecuaria Playa Caletas Airport",
		City:      "Nicoya/Guanacate",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 7,
		Latitude:  9.733332634,
		Longitude: -85.2333297729,
		Timezone:  "America/Costa_Rica",
	},
	"MRPV": {
		ICAO:      "MRPV",
		IATA:      "SYQ",
		Name:      "Tobias Bolanos International Airport",
		City:      "San Jose",
		State:     "San-Jose",
		Country:   "CR",
		Elevation: 3287,
		Latitude:  9.9570503235,
		Longitude: -84.1398010254,
		Timezone:  "America/Costa_Rica",
	},
	"MRPY": {
		ICAO:      "MRPY",
		Name:      "Playa Ballena Airport",
		City:      "Puntarenas",
		State:     "Puntarenas",
		Country:   "CR",
		Elevation: 7,
		Latitude:  9.122420311,
		Longitude: -83.7063980103,
		Timezone:  "America/Costa_Rica",
	},
	"MRQA": {
		ICAO:      "MRQA",
		Name:      "Quebrada Azul Airport",
		City:      "Florencia",
		State:     "Alajuela",
		Country:   "CR",
		Elevation: 197,
		Latitude:  10.3999996185,
		Longitude: -84.4833297729,
		Timezone:  "America/Costa_Rica",
	},
	"MRQP": {
		ICAO:      "MRQP",
		IATA:      "XQP",
		Name:      "Quepos Managua Airport",
		City:      "Quepos",
		State:     "Puntarenas",
		Country:   "CR",
		Elevation: 85,
		Latitude:  9.4431600571,
		Longitude: -84.1297988892,
		Timezone:  "America/Costa_Rica",
	},
	"MRRF": {
		ICAO:      "MRRF",
		IATA:      "RFR",
		Name:      "Rio Frio / Progreso Airport",
		City:      "Rio Frio / Progreso",
		State:     "Heredia",
		Country:   "CR",
		Elevation: 350,
		Latitude:  10.3274002075,
		Longitude: -83.8876037598,
		Timezone:  "America/Costa_Rica",
	},
	"MRRH": {
		ICAO:      "MRRH",
		Name:      "Rancho Humo Airport",
		City:      "Santa Cruz",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 33,
		Latitude:  10.3166666031,
		Longitude: -85.3333358765,
		Timezone:  "America/Costa_Rica",
	},
	"MRRN": {
		ICAO:      "MRRN",
		Name:      "Rancho Nuevo Airport",
		City:      "Puntarenas",
		State:     "Puntarenas",
		Country:   "CR",
		Elevation: 33,
		Latitude:  9.5666666031,
		Longitude: -84.5166702271,
		Timezone:  "America/Costa_Rica",
	},
	"MRRX": {
		ICAO:      "MRRX",
		Name:      "Roxana Farms Airport",
		City:      "Limon",
		State:     "Limon",
		Country:   "CR",
		Elevation: 312,
		Latitude:  10.3081998825,
		Longitude: -83.7543029785,
		Timezone:  "America/Costa_Rica",
	},
	"MRSA": {
		ICAO:      "MRSA",
		Name:      "San Alberto Airport",
		City:      "Limon",
		State:     "Limon",
		Country:   "CR",
		Elevation: 88,
		Latitude:  10.1465997696,
		Longitude: -83.4881973267,
		Timezone:  "America/Costa_Rica",
	},
	"MRSB": {
		ICAO:      "MRSB",
		Name:      "San Cristobal Airport",
		City:      "San Carlos",
		State:     "Alajuela",
		Country:   "CR",
		Elevation: 259,
		Latitude:  10.5,
		Longitude: -84.5666656494,
		Timezone:  "America/Costa_Rica",
	},
	"MRSC": {
		ICAO:      "MRSC",
		Name:      "Santa Cruz Airport",
		City:      "Santa Cruz",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 206,
		Latitude:  10.2737998962,
		Longitude: -85.5914993286,
		Timezone:  "America/Costa_Rica",
	},
	"MRSF": {
		ICAO:      "MRSF",
		Name:      "Santa Fe Airport",
		City:      "Perez Zeledon",
		State:     "San-Jose",
		Country:   "CR",
		Elevation: 1995,
		Latitude:  9.3000001907,
		Longitude: -83.6166687012,
		Timezone:  "America/Costa_Rica",
	},
	"MRSG": {
		ICAO:      "MRSG",
		Name:      "Santa Clara De Guapiles Airport",
		City:      "Limon",
		State:     "Limon",
		Country:   "CR",
		Elevation: 262,
		Latitude:  10.2882995605,
		Longitude: -83.7135009766,
		Timezone:  "America/Costa_Rica",
	},
	"MRSH": {
		ICAO:      "MRSH",
		Name:      "Shiroles Airport",
		City:      "Talamanca",
		State:     "Limon",
		Country:   "CR",
		Elevation: 20,
		Latitude:  9.5807695389,
		Longitude: -82.9598999023,
		Timezone:  "America/Costa_Rica",
	},
	"MRSI": {
		ICAO:      "MRSI",
		IATA:      "IPZ",
		Name:      "San Isidro del General Airport",
		City:      "Perez Zeledon",
		State:     "San-Jose",
		Country:   "CR",
		Elevation: 2100,
		Latitude:  9.3526201248,
		Longitude: -83.7130966187,
		Timezone:  "America/Costa_Rica",
	},
	"MRSL": {
		ICAO:      "MRSL",
		Name:      "Salama Airport",
		City:      "Salama",
		State:     "Puntarenas",
		Country:   "CR",
		Elevation: 32,
		Latitude:  8.8330001831,
		Longitude: -83.25,
		Timezone:  "America/Costa_Rica",
	},
	"MRSM": {
		ICAO:      "MRSM",
		Name:      "Santa Marta Airport",
		City:      "Orotina",
		State:     "Alajuela",
		Country:   "CR",
		Elevation: 417,
		Latitude:  9.9166669846,
		Longitude: -84.6166687012,
		Timezone:  "America/Costa_Rica",
	},
	"MRSO": {
		ICAO:      "MRSO",
		Name:      "Santa Maria De Guacimo Airport",
		City:      "Guapiles",
		State:     "Limon",
		Country:   "CR",
		Elevation: 32,
		Latitude:  10.2670001984,
		Longitude: -83.6169967651,
		Timezone:  "America/Costa_Rica",
	},
	"MRSP": {
		ICAO:      "MRSP",
		Name:      "San Pedro Airport",
		City:      "San Pedro",
		State:     "Limon",
		Country:   "CR",
		Elevation: 230,
		Latitude:  10.4179000854,
		Longitude: -83.7529983521,
		Timezone:  "America/Costa_Rica",
	},
	"MRSQ": {
		ICAO:      "MRSQ",
		Name:      "Sarapigui Airport",
		City:      "San Carlos",
		State:     "Heredia",
		Country:   "CR",
		Elevation: 164,
		Latitude:  10.4718999863,
		Longitude: -83.9069976807,
		Timezone:  "America/Costa_Rica",
	},
	"MRSR": {
		ICAO:      "MRSR",
		IATA:      "PLD",
		Name:      "Playa Samara Airport",
		City:      "Playa Samara",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 10,
		Latitude:  10.25,
		Longitude: -85.4169998169,
		Timezone:  "America/Costa_Rica",
	},
	"MRST": {
		ICAO:      "MRST",
		Name:      "San Agustin Airport",
		City:      "Puntarenas",
		State:     "Puntarenas",
		Country:   "CR",
		Elevation: 66,
		Latitude:  10.0666666031,
		Longitude: -84.8833312988,
		Timezone:  "America/Costa_Rica",
	},
	"MRSV": {
		ICAO:      "MRSV",
		IATA:      "TOO",
		Name:      "San Vito De Java Airport",
		City:      "Coto Brus",
		State:     "Puntarenas",
		Country:   "CR",
		Elevation: 3228,
		Latitude:  8.8261098862,
		Longitude: -82.9589004517,
		Timezone:  "America/Costa_Rica",
	},
	"MRSX": {
		ICAO:      "MRSX",
		Name:      "Sixaola Airport",
		City:      "Sixaola",
		State:     "Limon",
		Country:   "CR",
		Elevation: 26,
		Latitude:  9.5032196045,
		Longitude: -82.6327972412,
		Timezone:  "America/Costa_Rica",
	},
	"MRTB": {
		ICAO:      "MRTB",
		Name:      "Ticaban Airport",
		City:      "Pococi",
		State:     "Limon",
		Country:   "CR",
		Elevation: 246,
		Latitude:  10.3666667938,
		Longitude: -83.8499984741,
		Timezone:  "America/Costa_Rica",
	},
	"MRTG": {
		ICAO:      "MRTG",
		Name:      "Taboga Airport",
		City:      "Canas",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 108,
		Latitude:  10.3500003815,
		Longitude: -85.1999969482,
		Timezone:  "America/Costa_Rica",
	},
	"MRTL": {
		ICAO:      "MRTL",
		Name:      "Talolinga Airport",
		City:      "Nicoya",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 33,
		Latitude:  10.3084001541,
		Longitude: -85.4683990479,
		Timezone:  "America/Costa_Rica",
	},
	"MRTM": {
		ICAO:      "MRTM",
		IATA:      "TNO",
		Name:      "Tamarindo De Santa Cruz Airport",
		City:      "Santa Cruz",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 41,
		Latitude:  10.3135004044,
		Longitude: -85.8154983521,
		Timezone:  "America/Costa_Rica",
	},
	"MRTR": {
		ICAO:      "MRTR",
		IATA:      "TMU",
		Name:      "Tambor Airport",
		City:      "Nicoya",
		State:     "Puntarenas",
		Country:   "CR",
		Elevation: 33,
		Latitude:  9.7385196686,
		Longitude: -85.0138015747,
		Timezone:  "America/Costa_Rica",
	},
	"MRUP": {
		ICAO:      "MRUP",
		IATA:      "UPL",
		Name:      "Upala Airport",
		City:      "Upala",
		State:     "Alajuela",
		Country:   "CR",
		Elevation: 184,
		Latitude:  10.89220047,
		Longitude: -85.0161972046,
		Timezone:  "America/Costa_Rica",
	},
	"MRYT": {
		ICAO:      "MRYT",
		Name:      "Yucatica Airport",
		City:      "Guacimo",
		State:     "Limon",
		Country:   "CR",
		Elevation: 151,
		Latitude:  10.3333330154,
		Longitude: -83.6666641235,
		Timezone:  "America/Costa_Rica",
	},
	"MRZP": {
		ICAO:      "MRZP",
		Name:      "Zapotal De Guanacaste Airport",
		City:      "Zapotal",
		State:     "Guanacaste",
		Country:   "CR",
		Elevation: 1305,
		Latitude:  9.9940099716,
		Longitude: -85.3102035522,
		Timezone:  "America/Costa_Rica",
	},
	"MS00": {
		ICAO:      "MS00",
		Name:      "Double O Ranch Airport",
		City:      "Olive Branch",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 385,
		Latitude:  34.988899231,
		Longitude: -89.9152984619,
		Timezone:  "America/Chicago",
	},
	"MS03": {
		ICAO:      "MS03",
		Name:      "Christmas Airport",
		City:      "Shelby",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 153,
		Latitude:  33.9243011475,
		Longitude: -90.7870025635,
		Timezone:  "America/Chicago",
	},
	"MS04": {
		ICAO:      "MS04",
		Name:      "Hopper Field",
		City:      "Steens",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 300,
		Latitude:  33.6225013733,
		Longitude: -88.2861022949,
		Timezone:  "America/Chicago",
	},
	"MS06": {
		ICAO:      "MS06",
		Name:      "Linwood Airport",
		City:      "Linwood",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 285,
		Latitude:  32.7904014587,
		Longitude: -90.1380996704,
		Timezone:  "America/Chicago",
	},
	"MS08": {
		ICAO:      "MS08",
		Name:      "Harrell Field",
		City:      "Pisgah",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 385,
		Latitude:  32.4738998413,
		Longitude: -89.8517990112,
		Timezone:  "America/Chicago",
	},
	"MS09": {
		ICAO:      "MS09",
		Name:      "Glidwell Flying Service Airport",
		City:      "Alligator",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 160,
		Latitude:  34.1017990112,
		Longitude: -90.7012023926,
		Timezone:  "America/Chicago",
	},
	"MS10": {
		ICAO:      "MS10",
		Name:      "Turner Field",
		City:      "Belzoni",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 109,
		Latitude:  33.1585006714,
		Longitude: -90.5043029785,
		Timezone:  "America/Chicago",
	},
	"MS11": {
		ICAO:      "MS11",
		Name:      "Mohr Farm Airport",
		City:      "Brandon",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 360,
		Latitude:  32.2084999084,
		Longitude: -89.9417037964,
		Timezone:  "America/Chicago",
	},
	"MS13": {
		ICAO:      "MS13",
		Name:      "Eifling Farms Airport",
		City:      "Glen Allan",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 110,
		Latitude:  33.0756988525,
		Longitude: -90.9533996582,
		Timezone:  "America/Chicago",
	},
	"MS14": {
		ICAO:      "MS14",
		Name:      "John Farese Airpark",
		City:      "Ashland",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 565,
		Latitude:  34.813889,
		Longitude: -89.199617,
		Timezone:  "America/Chicago",
	},
	"MS15": {
		ICAO:      "MS15",
		Name:      "Country Club Airport",
		City:      "Greenville",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 117,
		Latitude:  33.331199646,
		Longitude: -91.0370025635,
		Timezone:  "America/Chicago",
	},
	"MS18": {
		ICAO:      "MS18",
		Name:      "Dilworth Airport",
		City:      "Corinth",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 465,
		Latitude:  34.7700004578,
		Longitude: -88.6055984497,
		Timezone:  "America/Chicago",
	},
	"MS20": {
		ICAO:      "MS20",
		Name:      "Newell Flying Service Airport",
		City:      "Inverness",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 126,
		Latitude:  33.354801178,
		Longitude: -90.5790023804,
		Timezone:  "America/Chicago",
	},
	"MS21": {
		ICAO:      "MS21",
		Name:      "Lester Field",
		City:      "Inverness",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 115,
		Latitude:  33.3320999146,
		Longitude: -90.5820007324,
		Timezone:  "America/Chicago",
	},
	"MS22": {
		ICAO:      "MS22",
		Name:      "Murphey Flying Service Airport",
		City:      "Itta Bena",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 120,
		Latitude:  33.4889984131,
		Longitude: -90.3570022583,
		Timezone:  "America/Chicago",
	},
	"MS23": {
		ICAO:      "MS23",
		Name:      "Forest Home Airport",
		City:      "Woodville",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 240,
		Latitude:  31.0067005157,
		Longitude: -91.2031021118,
		Timezone:  "America/Chicago",
	},
	"MS24": {
		ICAO:      "MS24",
		Name:      "Joe Sanford Field",
		City:      "Edwards",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 180,
		Latitude:  32.246799469,
		Longitude: -90.5822982788,
		Timezone:  "America/Chicago",
	},
	"MS25": {
		ICAO:      "MS25",
		Name:      "Huntley Airport",
		City:      "Leland",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 117,
		Latitude:  33.3987007141,
		Longitude: -90.9675979614,
		Timezone:  "America/Chicago",
	},
	"MS26": {
		ICAO:      "MS26",
		Name:      "Cedar Creek Air Ranch Airport",
		City:      "Lucedale",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 165,
		Latitude:  30.8022003174,
		Longitude: -88.5475006104,
		Timezone:  "America/Chicago",
	},
	"MS27": {
		ICAO:      "MS27",
		Name:      "Aerohead Airport",
		City:      "Hurley",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 130,
		Latitude:  30.7068996429,
		Longitude: -88.4291992188,
		Timezone:  "America/Chicago",
	},
	"MS28": {
		ICAO:      "MS28",
		Name:      "Alcorn State University Airport",
		City:      "Lorman",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 269,
		Latitude:  31.8687992096,
		Longitude: -91.134803772,
		Timezone:  "America/Chicago",
	},
	"MS29": {
		ICAO:      "MS29",
		Name:      "Pace Airstrip",
		City:      "Canton",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 350,
		Latitude:  32.601944,
		Longitude: -89.863889,
		Timezone:  "America/Chicago",
	},
	"MS30": {
		ICAO:      "MS30",
		Name:      "Abide Airpark",
		City:      "Greenville",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 115,
		Latitude:  33.3056983948,
		Longitude: -90.9904022217,
		Timezone:  "America/Chicago",
	},
	"MS33": {
		ICAO:      "MS33",
		Name:      "Hobbs Airport",
		City:      "Moorhead",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 115,
		Latitude:  33.4168014526,
		Longitude: -90.4944992065,
		Timezone:  "America/Chicago",
	},
	"MS34": {
		ICAO:      "MS34",
		Name:      "Reality Plantation Airport",
		City:      "Onward",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 98,
		Latitude:  32.6982002258,
		Longitude: -90.9465026855,
		Timezone:  "America/Chicago",
	},
	"MS35": {
		ICAO:      "MS35",
		Name:      "Wade Airport",
		City:      "Rolling Fork",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 106,
		Latitude:  32.9812011719,
		Longitude: -90.9654006958,
		Timezone:  "America/Chicago",
	},
	"MS36": {
		ICAO:      "MS36",
		Name:      "Supplejack Airport",
		City:      "Canton",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 340,
		Latitude:  32.6500015259,
		Longitude: -89.9708023071,
		Timezone:  "America/Chicago",
	},
	"MS37": {
		ICAO:      "MS37",
		Name:      "West Bolivar Flying Service Airport",
		City:      "Benoit",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 136,
		Latitude:  33.6534004211,
		Longitude: -90.9889984131,
		Timezone:  "America/Chicago",
	},
	"MS38": {
		ICAO:      "MS38",
		Name:      "Flautt Airport",
		City:      "Swan Lake",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 145,
		Latitude:  33.8722991943,
		Longitude: -90.2834014893,
		Timezone:  "America/Chicago",
	},
	"MS39": {
		ICAO:      "MS39",
		Name:      "Smith County Airport",
		City:      "Taylorsville",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 332,
		Latitude:  31.8209991455,
		Longitude: -89.4583969116,
		Timezone:  "America/Chicago",
	},
	"MS40": {
		ICAO:      "MS40",
		Name:      "Southland Flying Service Inc. Airport",
		City:      "Tchula",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 115,
		Latitude:  33.2256011963,
		Longitude: -90.2466964722,
		Timezone:  "America/Chicago",
	},
	"MS41": {
		ICAO:      "MS41",
		Name:      "Flying Y Service Airport",
		City:      "Tutwiler",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 149,
		Latitude:  34.0000991821,
		Longitude: -90.4251022339,
		Timezone:  "America/Chicago",
	},
	"MS42": {
		ICAO:      "MS42",
		Name:      "Mississippi Petrified Forest Airport",
		City:      "Flora",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 265,
		Latitude:  32.520401001,
		Longitude: -90.3236999512,
		Timezone:  "America/Chicago",
	},
	"MS43": {
		ICAO:      "MS43",
		Name:      "Whitaker Airport",
		City:      "Vicksburg",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 98,
		Latitude:  32.5517997742,
		Longitude: -90.8303985596,
		Timezone:  "America/Chicago",
	},
	"MS45": {
		ICAO:      "MS45",
		Name:      "Dale Landing Strip",
		City:      "Yokena",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 85,
		Latitude:  32.1562995911,
		Longitude: -91.1246032715,
		Timezone:  "America/Chicago",
	},
	"MS48": {
		ICAO:      "MS48",
		Name:      "George Ford Airport",
		City:      "Picayune",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 60,
		Latitude:  30.6219005585,
		Longitude: -89.6965026855,
		Timezone:  "America/Chicago",
	},
	"MS49": {
		ICAO:      "MS49",
		Name:      "Mc Gehee Air Park",
		City:      "Liberty",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 315,
		Latitude:  31.1630001068,
		Longitude: -90.8172988892,
		Timezone:  "America/Chicago",
	},
	"MS50": {
		ICAO:      "MS50",
		Name:      "Clay Airport",
		City:      "Monticello",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 201,
		Latitude:  31.5792999268,
		Longitude: -90.1111984253,
		Timezone:  "America/Chicago",
	},
	"MS52": {
		ICAO:      "MS52",
		Name:      "Leuth Flying Service Airport",
		City:      "Okolona",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 310,
		Latitude:  33.9236984253,
		Longitude: -88.7305984497,
		Timezone:  "America/Chicago",
	},
	"MS53": {
		ICAO:      "MS53",
		Name:      "Wolf River Ranch Airport",
		City:      "Poplarville",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 365,
		Latitude:  30.9146003723,
		Longitude: -89.4445037842,
		Timezone:  "America/Chicago",
	},
	"MS54": {
		ICAO:      "MS54",
		Name:      "Burney Farms Airport",
		City:      "Coffeeville",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 285,
		Latitude:  34.0097999573,
		Longitude: -89.7361984253,
		Timezone:  "America/Chicago",
	},
	"MS55": {
		ICAO:      "MS55",
		Name:      "Schloss Adlers Airport",
		City:      "Red Banks",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 478,
		Latitude:  34.8109016418,
		Longitude: -89.5783996582,
		Timezone:  "America/Chicago",
	},
	"MS57": {
		ICAO:      "MS57",
		Name:      "Fred Netterville Lbr Co Wilkinson Commnty Airport",
		City:      "Woodville",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 120,
		Latitude:  31.220199585,
		Longitude: -91.2404022217,
		Timezone:  "America/Chicago",
	},
	"MS58": {
		ICAO:      "MS58",
		Name:      "Tullos Field",
		City:      "Florence",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 300,
		Latitude:  32.1654014587,
		Longitude: -90.0439987183,
		Timezone:  "America/Chicago",
	},
	"MS59": {
		ICAO:      "MS59",
		Name:      "Valley of The Moon Airport",
		City:      "Hermanville",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 125,
		Latitude:  31.9969997406,
		Longitude: -90.8422012329,
		Timezone:  "America/Chicago",
	},
	"MS61": {
		ICAO:      "MS61",
		Name:      "Simpson Field",
		City:      "Picayune",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 175,
		Latitude:  30.6252002716,
		Longitude: -89.7333984375,
		Timezone:  "America/Chicago",
	},
	"MS62": {
		ICAO:      "MS62",
		Name:      "Davis Field",
		City:      "Olive Branch",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 410,
		Latitude:  34.9911994934,
		Longitude: -89.8155975342,
		Timezone:  "America/Chicago",
	},
	"MS63": {
		ICAO:      "MS63",
		Name:      "Rye Field",
		City:      "Smithville",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 330,
		Latitude:  34.0542984009,
		Longitude: -88.3712005615,
		Timezone:  "America/Chicago",
	},
	"MS64": {
		ICAO:      "MS64",
		Name:      "Vaiden Landing Airport",
		City:      "Hernando",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 250,
		Latitude:  34.848342,
		Longitude: -89.877992,
		Timezone:  "America/Chicago",
	},
	"MS65": {
		ICAO:      "MS65",
		Name:      "Delta Flying Service Inc Airport",
		City:      "Walls",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 210,
		Latitude:  34.9500999451,
		Longitude: -90.2001037598,
		Timezone:  "America/Chicago",
	},
	"MS66": {
		ICAO:      "MS66",
		Name:      "Kennedy Executive Airport",
		City:      "Biloxi",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 42,
		Latitude:  30.4459991455,
		Longitude: -88.9259033203,
		Timezone:  "America/Chicago",
	},
	"MS68": {
		ICAO:      "MS68",
		Name:      "Lamb's Field",
		City:      "Mantachie",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 458,
		Latitude:  34.326499939,
		Longitude: -88.5725021362,
		Timezone:  "America/Chicago",
	},
	"MS69": {
		ICAO:      "MS69",
		Name:      "Falcon Field",
		City:      "Star",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 415,
		Latitude:  32.1087989807,
		Longitude: -90.0465011597,
		Timezone:  "America/Chicago",
	},
	"MS70": {
		ICAO:      "MS70",
		Name:      "Mc Lain  -Calico Airport",
		City:      "Philadelphia",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 560,
		Latitude:  32.7354011536,
		Longitude: -88.9259033203,
		Timezone:  "America/Chicago",
	},
	"MS71": {
		ICAO:      "MS71",
		Name:      "Slobovia Outernational Airport",
		City:      "Pocahontas",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 250,
		Latitude:  32.495098114,
		Longitude: -90.2929000854,
		Timezone:  "America/Chicago",
	},
	"MS74": {
		ICAO:      "MS74",
		Name:      "Lee's Field",
		City:      "Picayune",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 100,
		Latitude:  30.5363006592,
		Longitude: -89.5111999512,
		Timezone:  "America/Chicago",
	},
	"MS75": {
		ICAO:      "MS75",
		Name:      "Lewis Airport",
		City:      "Glen Allan",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 112,
		Latitude:  33.013999939,
		Longitude: -91.0390014648,
		Timezone:  "America/Chicago",
	},
	"MS76": {
		ICAO:      "MS76",
		Name:      "Wade Field",
		City:      "Calhoun City",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 300,
		Latitude:  33.7728996277,
		Longitude: -89.2425994873,
		Timezone:  "America/Chicago",
	},
	"MS77": {
		ICAO:      "MS77",
		Name:      "Lang Flying Service Airport",
		City:      "Inverness",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 114,
		Latitude:  33.3317985535,
		Longitude: -90.6432037354,
		Timezone:  "America/Chicago",
	},
	"MS80": {
		ICAO:      "MS80",
		Name:      "Hasting Airpark",
		City:      "Bruce",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 275,
		Latitude:  33.9945983887,
		Longitude: -89.3125991821,
		Timezone:  "America/Chicago",
	},
	"MS82": {
		ICAO:      "MS82",
		Name:      "Shade Tree Field",
		City:      "Lyman",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 113,
		Latitude:  30.5104999542,
		Longitude: -89.1473007202,
		Timezone:  "America/Chicago",
	},
	"MS83": {
		ICAO:      "MS83",
		Name:      "Alton Field",
		City:      "Harrisville",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 850,
		Latitude:  32.0295982361,
		Longitude: -90.1015014648,
		Timezone:  "America/Chicago",
	},
	"MS85": {
		ICAO:      "MS85",
		Name:      "Skelton Airport",
		City:      "Shaw",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 130,
		Latitude:  33.5979003906,
		Longitude: -90.8029022217,
		Timezone:  "America/Chicago",
	},
	"MS86": {
		ICAO:      "MS86",
		Name:      "Oglesby Farms Inc. Airport",
		City:      "Glen Allan",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 110,
		Latitude:  33.0500984192,
		Longitude: -91.0833969116,
		Timezone:  "America/Chicago",
	},
	"MS88": {
		ICAO:      "MS88",
		Name:      "Oreck Airport",
		City:      "Poplarville",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 180,
		Latitude:  30.7773990631,
		Longitude: -89.7250976563,
		Timezone:  "America/Chicago",
	},
	"MS90": {
		ICAO:      "MS90",
		Name:      "Flowood Industrial STOLport",
		City:      "Flowood",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 270,
		Latitude:  32.3156013489,
		Longitude: -90.1436004639,
		Timezone:  "America/Chicago",
	},
	"MS92": {
		ICAO:      "MS92",
		Name:      "Heigle Field",
		City:      "Mayersville",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 105,
		Latitude:  32.9201011658,
		Longitude: -91.0257034302,
		Timezone:  "America/Chicago",
	},
	"MS93": {
		ICAO:      "MS93",
		Name:      "Hooper Skylark Field",
		City:      "Mc Neil",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 150,
		Latitude:  30.726600647,
		Longitude: -89.7316970825,
		Timezone:  "America/Chicago",
	},
	"MS95": {
		ICAO:      "MS95",
		Name:      "Dorr Field",
		City:      "Merigold",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 142,
		Latitude:  33.8473014832,
		Longitude: -90.7250976563,
		Timezone:  "America/Chicago",
	},
	"MS96": {
		ICAO:      "MS96",
		Name:      "Barrett Field",
		City:      "Philadelphia",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 435,
		Latitude:  32.7667999268,
		Longitude: -89.1000976563,
		Timezone:  "America/Chicago",
	},
	"MS98": {
		ICAO:      "MS98",
		Name:      "Farmco Field",
		City:      "Lucedale",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 130,
		Latitude:  30.7668991089,
		Longitude: -88.5999984741,
		Timezone:  "America/Chicago",
	},
	"MS99": {
		ICAO:      "MS99",
		Name:      "Rocking T Ranch Airport",
		City:      "Bay St. Louis",
		State:     "Mississippi",
		Country:   "US",
		Elevation: 14,
		Latitude:  30.2558002472,
		Longitude: -89.5417022705,
		Timezone:  "America/Chicago",
	},
	"MSAC": {
		ICAO:      "MSAC",
		Name:      "La Aramuaca Airport",
		City:      "San Miguel",
		State:     "San-Miguel",
		Country:   "SV",
		Elevation: 285,
		Latitude:  13.4479904175,
		Longitude: -88.1201095581,
		Timezone:  "America/El_Salvador",
	},
	"MSBS": {
		ICAO:      "MSBS",
		Name:      "Barillas Airport",
		City:      "Barrillas",
		State:     "Usulutan",
		Country:   "SV",
		Elevation: 32,
		Latitude:  13.2652626038,
		Longitude: -88.499130249,
		Timezone:  "America/El_Salvador",
	},
	"MSCD": {
		ICAO:      "MSCD",
		Name:      "Ceiba Doblada Airport",
		City:      "Ceiba Doblada",
		State:     "Usulutan",
		Country:   "SV",
		Elevation: 10,
		Latitude:  13.213010788,
		Longitude: -88.6028137207,
		Timezone:  "America/El_Salvador",
	},
	"MSCH": {
		ICAO:      "MSCH",
		Name:      "La Chepona Airport",
		City:      "La Chepona",
		State:     "Usulutan",
		Country:   "SV",
		Elevation: 13,
		Latitude:  13.1880998611,
		Longitude: -88.4115982056,
		Timezone:  "America/El_Salvador",
	},
	"MSCM": {
		ICAO:      "MSCM",
		Name:      "Corral De Mulas Airport",
		City:      "Corral de Mulas",
		State:     "Usulutan",
		Country:   "SV",
		Elevation: 26,
		Latitude:  13.2055883408,
		Longitude: -88.5473403931,
		Timezone:  "America/El_Salvador",
	},
	"MSCN": {
		ICAO:      "MSCN",
		Name:      "Casas Nuevas Airport",
		City:      "Jiquilisco",
		State:     "Usulutan",
		Country:   "SV",
		Elevation: 157,
		Latitude:  13.3099727631,
		Longitude: -88.5058670044,
		Timezone:  "America/El_Salvador",
	},
	"MSCR": {
		ICAO:      "MSCR",
		Name:      "La Carrera Airport",
		City:      "Jiquilisco",
		State:     "Usulutan",
		Country:   "SV",
		Elevation: 254,
		Latitude:  13.3287029266,
		Longitude: -88.5209197998,
		Timezone:  "America/El_Salvador",
	},
	"MSES": {
		ICAO:      "MSES",
		Name:      "Espiritu Santo Airport",
		City:      "Espiritu Santo",
		State:     "Usulutan",
		Country:   "SV",
		Elevation: 1,
		Latitude:  13.2193241119,
		Longitude: -88.5540466309,
		Timezone:  "America/El_Salvador",
	},
	"MSET": {
		ICAO:      "MSET",
		Name:      "El Tamarindo Airport",
		City:      "Canoguero",
		State:     "La-Union",
		Country:   "SV",
		Elevation: 10,
		Latitude:  13.1625995636,
		Longitude: -87.904800415,
		Timezone:  "America/El_Salvador",
	},
	"MSJC": {
		ICAO:      "MSJC",
		Name:      "El Jocotillo Airport",
		City:      "El Jocotillo",
		State:     "Sonsonate",
		Country:   "SV",
		Elevation: 127,
		Latitude:  13.5861539841,
		Longitude: -89.7465209961,
		Timezone:  "America/El_Salvador",
	},
	"MSLC": {
		ICAO:      "MSLC",
		Name:      "La Cabana Airport",
		City:      "La Cabana",
		State:     "San-Salvador",
		Country:   "SV",
		Elevation: 885,
		Latitude:  14.0098104477,
		Longitude: -89.1847610474,
		Timezone:  "America/El_Salvador",
	},
	"MSLD": {
		ICAO:      "MSLD",
		Name:      "Los Comandos Airport",
		City:      "San Francisco Gotera",
		State:     "Morazan",
		Country:   "SV",
		Elevation: 919,
		Latitude:  13.7262001038,
		Longitude: -88.1066970825,
		Timezone:  "America/El_Salvador",
	},
	"MSLG": {
		ICAO:      "MSLG",
		Name:      "La Gloria Airport",
		City:      "San Salvador",
		State:     "San-Salvador",
		Country:   "SV",
		Elevation: 2163,
		Latitude:  13.7147397995,
		Longitude: -89.1913452148,
		Timezone:  "America/El_Salvador",
	},
	"MSLM": {
		ICAO:      "MSLM",
		Name:      "Las Mesas Airport",
		City:      "Las Mesas",
		State:     "La-Libertad",
		Country:   "SV",
		Elevation: 187,
		Latitude:  13.4870700836,
		Longitude: -89.1926574707,
		Timezone:  "America/El_Salvador",
	},
	"MSLP": {
		ICAO:      "MSLP",
		IATA:      "SAL",
		Name:      "El Salvador International Airport",
		City:      "Santa Clara",
		State:     "La-Paz",
		Country:   "SV",
		Elevation: 101,
		Latitude:  13.4408998489,
		Longitude: -89.0557022095,
		Timezone:  "America/El_Salvador",
	},
	"MSPT": {
		ICAO:      "MSPT",
		Name:      "El Platanar Airport",
		City:      "Suchitoto",
		State:     "Cuscatlan",
		Country:   "SV",
		Elevation: 1354,
		Latitude:  13.9453001022,
		Longitude: -89.0627975464,
		Timezone:  "America/El_Salvador",
	},
	"MSRC": {
		ICAO:      "MSRC",
		Name:      "El Ronco Airport",
		City:      "Metapan",
		State:     "Santa-Ana",
		Country:   "SV",
		Elevation: 1460,
		Latitude:  14.3187150955,
		Longitude: -89.5068969727,
		Timezone:  "America/El_Salvador",
	},
	"MSSA": {
		ICAO:      "MSSA",
		Name:      "El Palmer Airport",
		City:      "Santa Ana",
		State:     "Santa-Ana",
		Country:   "SV",
		Elevation: 2229,
		Latitude:  14.0672222222,
		Longitude: -89.6322222222,
		Timezone:  "America/El_Salvador",
	},
	"MSSJ": {
		ICAO:      "MSSJ",
		Name:      "Punta San Juan Airport",
		City:      "Corral de Mulas",
		State:     "Usulutan",
		Country:   "SV",
		Elevation: 3,
		Latitude:  13.1787347794,
		Longitude: -88.47315979,
		Timezone:  "America/El_Salvador",
	},
	"MSSM": {
		ICAO:      "MSSM",
		Name:      "El Papalon Airport",
		City:      "San Miguel",
		State:     "San-Miguel",
		Country:   "SV",
		Elevation: 283,
		Latitude:  13.4441003799,
		Longitude: -88.1269989014,
		Timezone:  "America/El_Salvador",
	},
	"MSSN": {
		ICAO:      "MSSN",
		Name:      "San Ramon Airport",
		City:      "San Ramon",
		State:     "La-Union",
		Country:   "SV",
		Elevation: 37,
		Latitude:  13.1974658966,
		Longitude: -87.9609146118,
		Timezone:  "America/El_Salvador",
	},
	"MSSS": {
		ICAO:      "MSSS",
		IATA:      "ILS",
		Name:      "Ilopango International Airport",
		City:      "San Salvador",
		State:     "San-Salvador",
		Country:   "SV",
		Elevation: 2021,
		Latitude:  13.6995000839,
		Longitude: -89.1199035645,
		Timezone:  "America/El_Salvador",
	},
	"MSZT": {
		ICAO:      "MSZT",
		Name:      "El Zapote Airport",
		City:      "Barra de Santiago",
		State:     "Ahuachapan",
		Country:   "SV",
		Elevation: 6,
		Latitude:  13.7079544067,
		Longitude: -90.0262680054,
		Timezone:  "America/El_Salvador",
	},
	"MT01": {
		ICAO:      "MT01",
		Name:      "Owen Bros Airport",
		City:      "Geraldine",
		State:     "Montana",
		Country:   "US",
		Elevation: 3106,
		Latitude:  47.6180000305,
		Longitude: -110.0920028687,
		Timezone:  "America/Denver",
	},
	"MT02": {
		ICAO:      "MT02",
		Name:      "Hellinger Airport",
		City:      "Devon",
		State:     "Montana",
		Country:   "US",
		Elevation: 3215,
		Latitude:  48.5382995605,
		Longitude: -111.4970016479,
		Timezone:  "America/Denver",
	},
	"MT03": {
		ICAO:      "MT03",
		Name:      "Lakeside Airport",
		City:      "Lakeside",
		State:     "Montana",
		Country:   "US",
		Elevation: 3440,
		Latitude:  47.993408,
		Longitude: -114.224306,
		Timezone:  "America/Denver",
	},
	"MT04": {
		ICAO:      "MT04",
		Name:      "Olfert Airport",
		City:      "Lustre",
		State:     "Montana",
		Country:   "US",
		Elevation: 2750,
		Latitude:  48.4042015076,
		Longitude: -106.0090026855,
		Timezone:  "America/Denver",
	},
	"MT05": {
		ICAO:      "MT05",
		Name:      "Laird Ranch Airport",
		City:      "Ekalaka",
		State:     "Montana",
		Country:   "US",
		Elevation: 3462,
		Latitude:  45.6680984497,
		Longitude: -104.7030029297,
		Timezone:  "America/Denver",
	},
	"MT06": {
		ICAO:      "MT06",
		Name:      "Duncan Airport",
		City:      "Helena",
		State:     "Montana",
		Country:   "US",
		Elevation: 4260,
		Latitude:  46.5702018738,
		Longitude: -112.1750030518,
		Timezone:  "America/Denver",
	},
	"MT07": {
		ICAO:      "MT07",
		Name:      "Langhus Airstrip",
		City:      "Melville",
		State:     "Montana",
		Country:   "US",
		Elevation: 5785,
		Latitude:  46.0415992737,
		Longitude: -110.158996582,
		Timezone:  "America/Denver",
	},
	"MT08": {
		ICAO:      "MT08",
		Name:      "Zortman Airport",
		City:      "Zortman",
		State:     "Montana",
		Country:   "US",
		Elevation: 3900,
		Latitude:  47.9166984558,
		Longitude: -108.483001709,
		Timezone:  "America/Denver",
	},
	"MT09": {
		ICAO:      "MT09",
		Name:      "Floyd Ranch Airport",
		City:      "Opheim",
		State:     "Montana",
		Country:   "US",
		Elevation: 2948,
		Latitude:  48.8459014893,
		Longitude: -106.6090011597,
		Timezone:  "America/Denver",
	},
	"MT11": {
		ICAO:      "MT11",
		Name:      "Etchart Field",
		City:      "Tampico",
		State:     "Montana",
		Country:   "US",
		Elevation: 2157,
		Latitude:  48.3166999817,
		Longitude: -106.8339996338,
		Timezone:  "America/Denver",
	},
	"MT12": {
		ICAO:      "MT12",
		Name:      "Smith Field",
		City:      "Butte",
		State:     "Montana",
		Country:   "US",
		Elevation: 6000,
		Latitude:  45.9384994507,
		Longitude: -112.5640029907,
		Timezone:  "America/Denver",
	},
	"MT14": {
		ICAO:      "MT14",
		Name:      "Jerry Creek Airport",
		City:      "Wise River",
		State:     "Montana",
		Country:   "US",
		Elevation: 5940,
		Latitude:  45.8235015869,
		Longitude: -112.8860015869,
		Timezone:  "America/Denver",
	},
	"MT15": {
		ICAO:      "MT15",
		Name:      "Fort Harrison Army Airfield",
		City:      "Helena",
		State:     "Montana",
		Country:   "US",
		Elevation: 4050,
		Latitude:  46.6258010864,
		Longitude: -112.11000061,
		Timezone:  "America/Denver",
	},
	"MT16": {
		ICAO:      "MT16",
		Name:      "Buffalo Trail Ranch Strip",
		City:      "Ovando",
		State:     "Montana",
		Country:   "US",
		Elevation: 3906,
		Latitude:  47.0138015747,
		Longitude: -113.2829971313,
		Timezone:  "America/Denver",
	},
	"MT17": {
		ICAO:      "MT17",
		Name:      "Wurtz Airport",
		City:      "Kalispell",
		State:     "Montana",
		Country:   "US",
		Elevation: 3800,
		Latitude:  48.9049987793,
		Longitude: -114.3820037842,
		Timezone:  "America/Denver",
	},
	"MT18": {
		ICAO:      "MT18",
		Name:      "Hoerner Airport",
		City:      "Columbia Falls",
		State:     "Montana",
		Country:   "US",
		Elevation: 3078,
		Latitude:  48.28995,
		Longitude: -114.182867,
		Timezone:  "America/Denver",
	},
	"MT19": {
		ICAO:      "MT19",
		Name:      "Olson Airport",
		City:      "Ronan",
		State:     "Montana",
		Country:   "US",
		Elevation: 3330,
		Latitude:  47.5909996033,
		Longitude: -114.0520019531,
		Timezone:  "America/Denver",
	},
	"MT20": {
		ICAO:      "MT20",
		Name:      "Hollstein Ranch Airport",
		City:      "Wibaux",
		State:     "Montana",
		Country:   "US",
		Elevation: 2778,
		Latitude:  46.8582992554,
		Longitude: -104.0989990234,
		Timezone:  "America/Denver",
	},
	"MT21": {
		ICAO:      "MT21",
		Name:      "Flying Arrow Ranch Airport",
		City:      "Butte",
		State:     "Montana",
		Country:   "US",
		Elevation: 5950,
		Latitude:  45.901599884,
		Longitude: -112.5490036011,
		Timezone:  "America/Denver",
	},
	"MT22": {
		ICAO:      "MT22",
		Name:      "Tezak's-Colterville-Spur Airport",
		City:      "Sheridan",
		State:     "Montana",
		Country:   "US",
		Elevation: 5000,
		Latitude:  45.3998985291,
		Longitude: -112.1669998169,
		Timezone:  "America/Denver",
	},
	"MT24": {
		ICAO:      "MT24",
		Name:      "Beacon Star Antique Airfield",
		City:      "Moore",
		State:     "Montana",
		Country:   "US",
		Elevation: 4320,
		Latitude:  46.9721984863,
		Longitude: -109.6139984131,
		Timezone:  "America/Denver",
	},
	"MT26": {
		ICAO:      "MT26",
		Name:      "Ranch Strip",
		City:      "Great Falls",
		State:     "Montana",
		Country:   "US",
		Elevation: 3355,
		Latitude:  47.470500946,
		Longitude: -111.2369995117,
		Timezone:  "America/Denver",
	},
	"MT29": {
		ICAO:      "MT29",
		Name:      "Sunday Creek Airpark",
		City:      "Miles City",
		State:     "Montana",
		Country:   "US",
		Elevation: 2490,
		Latitude:  46.4789009094,
		Longitude: -105.858001709,
		Timezone:  "America/Denver",
	},
	"MT30": {
		ICAO:      "MT30",
		Name:      "Trapper Creek Strip",
		City:      "Conner",
		State:     "Montana",
		Country:   "US",
		Elevation: 4040,
		Latitude:  45.9402008057,
		Longitude: -114.1370010376,
		Timezone:  "America/Denver",
	},
	"MT32": {
		ICAO:      "MT32",
		Name:      "Ckye Field",
		City:      "Stevensville",
		State:     "Montana",
		Country:   "US",
		Elevation: 3600,
		Latitude:  46.5387992859,
		Longitude: -114.0400009155,
		Timezone:  "America/Denver",
	},
	"MT33": {
		ICAO:      "MT33",
		Name:      "Bair Airport",
		City:      "Ulm",
		State:     "Montana",
		Country:   "US",
		Elevation: 3340,
		Latitude:  47.4094009399,
		Longitude: -111.4869995117,
		Timezone:  "America/Denver",
	},
	"MT34": {
		ICAO:      "MT34",
		Name:      "Ruff Airport",
		City:      "Custer",
		State:     "Montana",
		Country:   "US",
		Elevation: 2740,
		Latitude:  46.1333007813,
		Longitude: -107.5510025024,
		Timezone:  "America/Denver",
	},
	"MT36": {
		ICAO:      "MT36",
		Name:      "Cottontail Ranch Airport",
		City:      "Melville",
		State:     "Montana",
		Country:   "US",
		Elevation: 5340,
		Latitude:  46.1321983337,
		Longitude: -110.0469970703,
		Timezone:  "America/Denver",
	},
	"MT37": {
		ICAO:      "MT37",
		Name:      "Sanders Airport",
		City:      "Kalispell",
		State:     "Montana",
		Country:   "US",
		Elevation: 2910,
		Latitude:  48.1246986389,
		Longitude: -114.2399978638,
		Timezone:  "America/Denver",
	},
	"MT38": {
		ICAO:      "MT38",
		Name:      "Swank Airport",
		City:      "Poplar",
		State:     "Montana",
		Country:   "US",
		Elevation: 2205,
		Latitude:  48.4034004211,
		Longitude: -105.1569976807,
		Timezone:  "America/Denver",
	},
	"MT39": {
		ICAO:      "MT39",
		Name:      "Black Canyon Ranch Airport",
		City:      "Seeley Lake",
		State:     "Montana",
		Country:   "US",
		Elevation: 4260,
		Latitude:  47.1059989929,
		Longitude: -113.3170013428,
		Timezone:  "America/Denver",
	},
	"MT40": {
		ICAO:      "MT40",
		Name:      "Horner Field",
		City:      "Great Falls",
		State:     "Montana",
		Country:   "US",
		Elevation: 3430,
		Latitude:  47.5913009644,
		Longitude: -111.2720031738,
		Timezone:  "America/Denver",
	},
	"MT41": {
		ICAO:      "MT41",
		Name:      "Jefco Skypark Airport",
		City:      "Whitehall",
		State:     "Montana",
		Country:   "US",
		Elevation: 4510,
		Latitude:  45.8782997131,
		Longitude: -112.1169967651,
		Timezone:  "America/Denver",
	},
	"MT42": {
		ICAO:      "MT42",
		Name:      "Hasskamp STOLport",
		City:      "Three Forks",
		State:     "Montana",
		Country:   "US",
		Elevation: 4052,
		Latitude:  45.9104995728,
		Longitude: -111.513999939,
		Timezone:  "America/Denver",
	},
	"MT43": {
		ICAO:      "MT43",
		Name:      "Klies Air Strip",
		City:      "Basin",
		State:     "Montana",
		Country:   "US",
		Elevation: 6300,
		Latitude:  46.3462982178,
		Longitude: -112.34400177,
		Timezone:  "America/Denver",
	},
	"MT45": {
		ICAO:      "MT45",
		Name:      "Castleberry Airport",
		City:      "Ekalaka",
		State:     "Montana",
		Country:   "US",
		Elevation: 3373,
		Latitude:  45.9089012146,
		Longitude: -104.5759963989,
		Timezone:  "America/Denver",
	},
	"MT46": {
		ICAO:      "MT46",
		Name:      "Lakeview Airport",
		City:      "Lakeview",
		State:     "Montana",
		Country:   "US",
		Elevation: 6630,
		Latitude:  44.617099762,
		Longitude: -111.797996521,
		Timezone:  "America/Denver",
	},
	"MT47": {
		ICAO:      "MT47",
		Name:      "Metzel Creek Airport",
		City:      "Lakeview",
		State:     "Montana",
		Country:   "US",
		Elevation: 6691,
		Latitude:  44.6869010925,
		Longitude: -111.8939971924,
		Timezone:  "America/Denver",
	},
	"MT48": {
		ICAO:      "MT48",
		Name:      "Flying Y Ranch Airport",
		City:      "Livingston",
		State:     "Montana",
		Country:   "US",
		Elevation: 4857,
		Latitude:  45.4347000122,
		Longitude: -110.6439971924,
		Timezone:  "America/Denver",
	},
	"MT49": {
		ICAO:      "MT49",
		Name:      "Ford's South Airport",
		City:      "Lolo",
		State:     "Montana",
		Country:   "US",
		Elevation: 3200,
		Latitude:  46.923500061,
		Longitude: -114.0879974365,
		Timezone:  "America/Denver",
	},
	"MT50": {
		ICAO:      "MT50",
		Name:      "Lanning Ranch Airport",
		City:      "Alzada",
		State:     "Montana",
		Country:   "US",
		Elevation: 3995,
		Latitude:  45.0877990723,
		Longitude: -104.8190002441,
		Timezone:  "America/Denver",
	},
	"MT51": {
		ICAO:      "MT51",
		Name:      "Zerbe Airport",
		City:      "Frazer",
		State:     "Montana",
		Country:   "US",
		Elevation: 2720,
		Latitude:  48.2770004272,
		Longitude: -105.9619979858,
		Timezone:  "America/Denver",
	},
	"MT52": {
		ICAO:      "MT52",
		Name:      "Nine Mile Airport",
		City:      "Huson",
		State:     "Montana",
		Country:   "US",
		Elevation: 3364,
		Latitude:  47.0777015686,
		Longitude: -114.4130020142,
		Timezone:  "America/Denver",
	},
	"MT53": {
		ICAO:      "MT53",
		Name:      "Carson Field",
		City:      "Kalispell",
		State:     "Montana",
		Country:   "US",
		Elevation: 3550,
		Latitude:  48.0946998596,
		Longitude: -114.8519973755,
		Timezone:  "America/Denver",
	},
	"MT54": {
		ICAO:      "MT54",
		Name:      "Weaver Airport",
		City:      "Kalispell",
		State:     "Montana",
		Country:   "US",
		Elevation: 2929,
		Latitude:  48.2439002991,
		Longitude: -114.2440032959,
		Timezone:  "America/Denver",
	},
	"MT58": {
		ICAO:      "MT58",
		Name:      "Nelson Airport",
		City:      "Haxby / Fort Peck",
		State:     "Montana",
		Country:   "US",
		Elevation: 2425,
		Latitude:  47.7667007446,
		Longitude: -106.18800354,
		Timezone:  "America/Denver",
	},
	"MT59": {
		ICAO:      "MT59",
		Name:      "Gardner Airport",
		City:      "Hammond",
		State:     "Montana",
		Country:   "US",
		Elevation: 3560,
		Latitude:  45.26720047,
		Longitude: -104.8570022583,
		Timezone:  "America/Denver",
	},
	"MT60": {
		ICAO:      "MT60",
		Name:      "Langton Airstrip",
		City:      "Polebridge",
		State:     "Montana",
		Country:   "US",
		Elevation: 3900,
		Latitude:  48.8143997192,
		Longitude: -114.3539962769,
		Timezone:  "America/Denver",
	},
	"MT62": {
		ICAO:      "MT62",
		Name:      "Ted Luark Private STOLport",
		City:      "Huson",
		State:     "Montana",
		Country:   "US",
		Elevation: 3500,
		Latitude:  47.0666007996,
		Longitude: -114.4840011597,
		Timezone:  "America/Denver",
	},
	"MT63": {
		ICAO:      "MT63",
		Name:      "Lee Metcalf National Wildlife Refuge Airport",
		City:      "Stevensville",
		State:     "Montana",
		Country:   "US",
		Elevation: 3315,
		Latitude:  46.5555000305,
		Longitude: -114.0729980469,
		Timezone:  "America/Denver",
	},
	"MT68": {
		ICAO:      "MT68",
		Name:      "Wood Strip",
		City:      "Helena",
		State:     "Montana",
		Country:   "US",
		Elevation: 3850,
		Latitude:  46.6208000183,
		Longitude: -111.93800354,
		Timezone:  "America/Denver",
	},
	"MT71": {
		ICAO:      "MT71",
		Name:      "Edsall Field",
		City:      "Bozeman",
		State:     "Montana",
		Country:   "US",
		Elevation: 4659,
		Latitude:  45.7240982056,
		Longitude: -111.0429992676,
		Timezone:  "America/Denver",
	},
	"MT72": {
		ICAO:      "MT72",
		Name:      "Hedditch Airport",
		City:      "Corvallis",
		State:     "Montana",
		Country:   "US",
		Elevation: 3663,
		Latitude:  46.3460006714,
		Longitude: -114.1760025024,
		Timezone:  "America/Denver",
	},
	"MT73": {
		ICAO:      "MT73",
		Name:      "Camas Airport",
		City:      "Missoula",
		State:     "Montana",
		Country:   "US",
		Elevation: 3870,
		Latitude:  46.8507995605,
		Longitude: -113.5380020142,
		Timezone:  "America/Denver",
	},
	"MT74": {
		ICAO:      "MT74",
		Name:      "Sikorski Ranch Airport",
		City:      "Ekalaka",
		State:     "Montana",
		Country:   "US",
		Elevation: 3330,
		Latitude:  46.1068992615,
		Longitude: -104.4150009155,
		Timezone:  "America/Denver",
	},
	"MT75": {
		ICAO:      "MT75",
		Name:      "Buchanan Ranch Airport",
		City:      "Great Falls",
		State:     "Montana",
		Country:   "US",
		Elevation: 3345,
		Latitude:  47.4301986694,
		Longitude: -111.4599990845,
		Timezone:  "America/Denver",
	},
	"MT76": {
		ICAO:      "MT76",
		Name:      "Smith Farms Airport",
		City:      "Great Falls",
		State:     "Montana",
		Country:   "US",
		Elevation: 3340,
		Latitude:  47.3997001648,
		Longitude: -111.4700012207,
		Timezone:  "America/Denver",
	},
	"MT77": {
		ICAO:      "MT77",
		Name:      "Peterson Ranch Airport",
		City:      "Highwood",
		State:     "Montana",
		Country:   "US",
		Elevation: 3340,
		Latitude:  47.6082992554,
		Longitude: -110.90599823,
		Timezone:  "America/Denver",
	},
	"MT79": {
		ICAO:      "MT79",
		Name:      "Sand Creek Wildlife Station Airport",
		City:      "Roy",
		State:     "Montana",
		Country:   "US",
		Elevation: 2940,
		Latitude:  47.5842018127,
		Longitude: -108.7089996338,
		Timezone:  "America/Denver",
	},
	"MT80": {
		ICAO:      "MT80",
		Name:      "Vine Airport",
		City:      "Wolf Point",
		State:     "Montana",
		Country:   "US",
		Elevation: 2075,
		Latitude:  48.0430984497,
		Longitude: -105.6090011597,
		Timezone:  "America/Denver",
	},
	"MT81": {
		ICAO:      "MT81",
		Name:      "Thompson Field",
		City:      "Belgrade",
		State:     "Montana",
		Country:   "US",
		Elevation: 4400,
		Latitude:  45.8035011292,
		Longitude: -111.1380004883,
		Timezone:  "America/Denver",
	},
	"MT82": {
		ICAO:      "MT82",
		Name:      "Baxter Strip",
		City:      "Harlowton",
		State:     "Montana",
		Country:   "US",
		Elevation: 4520,
		Latitude:  46.3083000183,
		Longitude: -109.8720016479,
		Timezone:  "America/Denver",
	},
	"MT83": {
		ICAO:      "MT83",
		Name:      "Ten Mile Airport",
		City:      "Helena",
		State:     "Montana",
		Country:   "US",
		Elevation: 4294,
		Latitude:  46.5716018677,
		Longitude: -112.1809997559,
		Timezone:  "America/Denver",
	},
	"MT84": {
		ICAO:      "MT84",
		Name:      "Mower Field",
		City:      "Kalispell",
		State:     "Montana",
		Country:   "US",
		Elevation: 4000,
		Latitude:  48.2207984924,
		Longitude: -114.3759994507,
		Timezone:  "America/Denver",
	},
	"MT85": {
		ICAO:      "MT85",
		Name:      "Redfield Ag Strip",
		City:      "Opheim",
		State:     "Montana",
		Country:   "US",
		Elevation: 3100,
		Latitude:  48.7588996887,
		Longitude: -106.364997864,
		Timezone:  "America/Denver",
	},
	"MT86": {
		ICAO:      "MT86",
		Name:      "Bar E Airport",
		City:      "Helena",
		State:     "Montana",
		Country:   "US",
		Elevation: 3875,
		Latitude:  47.1689987183,
		Longitude: -112.0849990845,
		Timezone:  "America/Denver",
	},
	"MT87": {
		ICAO:      "MT87",
		Name:      "Oglesby Farms Inc Airport",
		City:      "Peerless",
		State:     "Montana",
		Country:   "US",
		Elevation: 2850,
		Latitude:  48.6666984558,
		Longitude: -105.9339981079,
		Timezone:  "America/Denver",
	},
	"MT88": {
		ICAO:      "MT88",
		Name:      "Campbell Ranch Airport",
		City:      "Marion",
		State:     "Montana",
		Country:   "US",
		Elevation: 3360,
		Latitude:  48.0043983459,
		Longitude: -114.9990005493,
		Timezone:  "America/Denver",
	},
	"MT89": {
		ICAO:      "MT89",
		Name:      "Foster Ranches Airport",
		City:      "Livingston",
		State:     "Montana",
		Country:   "US",
		Elevation: 4680,
		Latitude:  45.7999000549,
		Longitude: -110.533996582,
		Timezone:  "America/Denver",
	},
	"MT90": {
		ICAO:      "MT90",
		Name:      "J B Airport",
		City:      "Wolf Point",
		State:     "Montana",
		Country:   "US",
		Elevation: 2710,
		Latitude:  48.3877983093,
		Longitude: -105.3349990845,
		Timezone:  "America/Denver",
	},
	"MT93": {
		ICAO:      "MT93",
		Name:      "Larner Field",
		City:      "Deer Lodge",
		State:     "Montana",
		Country:   "US",
		Elevation: 4710,
		Latitude:  46.2871017456,
		Longitude: -112.7580032349,
		Timezone:  "America/Denver",
	},
	"MT94": {
		ICAO:      "MT94",
		Name:      "Ousel Falls Airport",
		City:      "Big Sky",
		State:     "Montana",
		Country:   "US",
		Elevation: 6600,
		Latitude:  45.2466011047,
		Longitude: -111.3290023804,
		Timezone:  "America/Denver",
	},
	"MT95": {
		ICAO:      "MT95",
		Name:      "Flathead Lake Sky Ranch Airport",
		City:      "Kalispell",
		State:     "Montana",
		Country:   "US",
		Elevation: 2905,
		Latitude:  48.1169013977,
		Longitude: -114.1859970093,
		Timezone:  "America/Denver",
	},
	"MT96": {
		ICAO:      "MT96",
		Name:      "Wilhelm Airstrip",
		City:      "Helena",
		State:     "Montana",
		Country:   "US",
		Elevation: 4000,
		Latitude:  46.7402000427,
		Longitude: -112.0350036621,
		Timezone:  "America/Denver",
	},
	"MT97": {
		ICAO:      "MT97",
		Name:      "Frampton Airport",
		City:      "Noxon",
		State:     "Montana",
		Country:   "US",
		Elevation: 2400,
		Latitude:  47.9785995483,
		Longitude: -115.7679977417,
		Timezone:  "America/Denver",
	},
	"MT98": {
		ICAO:      "MT98",
		Name:      "Big Beaver Creek Ranch Airport",
		City:      "Trout Creek",
		State:     "Montana",
		Country:   "US",
		Elevation: 2600,
		Latitude:  47.6980018616,
		Longitude: -115.5510025024,
		Timezone:  "America/Denver",
	},
	"MTCA": {
		ICAO:      "MTCA",
		IATA:      "CYA",
		Name:      "Les Cayes Airport",
		City:      "Les Cayes",
		State:     "Sud",
		Country:   "HT",
		Elevation: 203,
		Latitude:  18.2710990906,
		Longitude: -73.7882995605,
		Timezone:  "America/Port-au-Prince",
	},
	"MTCH": {
		ICAO:      "MTCH",
		IATA:      "CAP",
		Name:      "Cap Haitien International Airport",
		City:      "Cap Haitien",
		State:     "Nord",
		Country:   "HT",
		Elevation: 10,
		Latitude:  19.7329998016,
		Longitude: -72.1947021484,
		Timezone:  "America/Port-au-Prince",
	},
	"MTJA": {
		ICAO:      "MTJA",
		IATA:      "JAK",
		Name:      "Jacmel Airport",
		City:      "Jacmel",
		State:     "Sud-Est",
		Country:   "HT",
		Elevation: 167,
		Latitude:  18.2411003113,
		Longitude: -72.5185012817,
		Timezone:  "America/Port-au-Prince",
	},
	"MTJE": {
		ICAO:      "MTJE",
		IATA:      "JEE",
		Name:      "Jeremie Airport",
		City:      "Jeremie",
		State:     "GrandʼAnse",
		Country:   "HT",
		Elevation: 147,
		Latitude:  18.6630992889,
		Longitude: -74.1703033447,
		Timezone:  "America/Port-au-Prince",
	},
	"MTPP": {
		ICAO:      "MTPP",
		IATA:      "PAP",
		Name:      "Toussaint Louverture International Airport",
		City:      "Port-au-Prince",
		State:     "Ouest",
		Country:   "HT",
		Elevation: 122,
		Latitude:  18.5799999237,
		Longitude: -72.2925033569,
		Timezone:  "America/Port-au-Prince",
	},
	"MTPX": {
		ICAO:      "MTPX",
		IATA:      "PAX",
		Name:      "Port-de-Paix Airport",
		City:      "Port-de-Paix",
		State:     "Nord-Ouest",
		Country:   "HT",
		Elevation: 9,
		Latitude:  19.933599472,
		Longitude: -72.8486022949,
		Timezone:  "America/Port-au-Prince",
	},
	"MTRH": {
		ICAO:      "MTRH",
		Name:      "Trojes Airport",
		City:      "Trojes",
		State:     "Yoro",
		Country:   "HN",
		Elevation: 170,
		Latitude:  15.4044437408,
		Longitude: -86.6827774048,
		Timezone:  "America/Tegucigalpa",
	},
	"MU00": {
		ICAO:      "MU00",
		Name:      "Samuel L. Clemens Memorial Airport",
		City:      "Paris",
		State:     "Missouri",
		Country:   "US",
		Elevation: 730,
		Latitude:  39.4444999695,
		Longitude: -91.9517974854,
		Timezone:  "America/Chicago",
	},
	"MU02": {
		ICAO:      "MU02",
		Name:      "C.E.F. Airport",
		City:      "Warrenton",
		State:     "Missouri",
		Country:   "US",
		Elevation: 638,
		Latitude:  38.8097991943,
		Longitude: -91.1035003662,
		Timezone:  "America/Chicago",
	},
	"MU03": {
		ICAO:      "MU03",
		Name:      "Leo's Angus Ranch Airport",
		City:      "Dayton",
		State:     "Missouri",
		Country:   "US",
		Elevation: 900,
		Latitude:  38.4799995422,
		Longitude: -94.1974029541,
		Timezone:  "America/Chicago",
	},
	"MU04": {
		ICAO:      "MU04",
		Name:      "Broadwater Airport",
		City:      "Malden",
		State:     "Missouri",
		Country:   "US",
		Elevation: 280,
		Latitude:  36.5898017883,
		Longitude: -89.9200973511,
		Timezone:  "America/Chicago",
	},
	"MU07": {
		ICAO:      "MU07",
		Name:      "Angle Bar M Airport",
		City:      "Adrian",
		State:     "Missouri",
		Country:   "US",
		Elevation: 900,
		Latitude:  38.3946990967,
		Longitude: -94.3102035522,
		Timezone:  "America/Chicago",
	},
	"MU09": {
		ICAO:      "MU09",
		Name:      "Hester Airport",
		City:      "Burlington Junction",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1000,
		Latitude:  40.4667015076,
		Longitude: -95.0211029053,
		Timezone:  "America/Chicago",
	},
	"MU11": {
		ICAO:      "MU11",
		Name:      "Eagle's Landing Airport",
		City:      "Climax Springs",
		State:     "Missouri",
		Country:   "US",
		Elevation: 822,
		Latitude:  38.2209014893,
		Longitude: -93.0155029297,
		Timezone:  "America/Chicago",
	},
	"MU12": {
		ICAO:      "MU12",
		Name:      "Childress Airstrip",
		City:      "Springfield",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1400,
		Latitude:  37.1403007507,
		Longitude: -93.2154998779,
		Timezone:  "America/Chicago",
	},
	"MU13": {
		ICAO:      "MU13",
		Name:      "Bakers Landing Airport",
		City:      "Billings",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1340,
		Latitude:  37.0597991943,
		Longitude: -93.6009979248,
		Timezone:  "America/Chicago",
	},
	"MU16": {
		ICAO:      "MU16",
		Name:      "Spillman Field",
		City:      "Jamesport",
		State:     "Missouri",
		Country:   "US",
		Elevation: 981,
		Latitude:  40.0003013611,
		Longitude: -93.7899017334,
		Timezone:  "America/Chicago",
	},
	"MU18": {
		ICAO:      "MU18",
		Name:      "Stark Airport",
		City:      "Warrenton",
		State:     "Missouri",
		Country:   "US",
		Elevation: 840,
		Latitude:  38.7806015015,
		Longitude: -91.1473999023,
		Timezone:  "America/Chicago",
	},
	"MU19": {
		ICAO:      "MU19",
		Name:      "Cherokee Airpark",
		City:      "Bolivar",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1150,
		Latitude:  37.551700592,
		Longitude: -93.4001998901,
		Timezone:  "America/Chicago",
	},
	"MU20": {
		ICAO:      "MU20",
		Name:      "Robbins Airport",
		City:      "Odessa",
		State:     "Missouri",
		Country:   "US",
		Elevation: 910,
		Latitude:  38.9455986023,
		Longitude: -93.9646987915,
		Timezone:  "America/Chicago",
	},
	"MU21": {
		ICAO:      "MU21",
		Name:      "Miller Farm Airport",
		City:      "Bay",
		State:     "Missouri",
		Country:   "US",
		Elevation: 630,
		Latitude:  38.5237007141,
		Longitude: -91.5177001953,
		Timezone:  "America/Chicago",
	},
	"MU23": {
		ICAO:      "MU23",
		Name:      "White Cloud Flying Field",
		City:      "Columbia",
		State:     "Missouri",
		Country:   "US",
		Elevation: 600,
		Latitude:  39.0306015015,
		Longitude: -92.3917999268,
		Timezone:  "America/Chicago",
	},
	"MU24": {
		ICAO:      "MU24",
		Name:      "Someday Ranch Airport",
		City:      "Cedarcreek",
		State:     "Missouri",
		Country:   "US",
		Elevation: 812,
		Latitude:  36.5717010498,
		Longitude: -93.0367965698,
		Timezone:  "America/Chicago",
	},
	"MU25": {
		ICAO:      "MU25",
		Name:      "Piney Bend Airport",
		City:      "Houston",
		State:     "Missouri",
		Country:   "US",
		Elevation: 950,
		Latitude:  37.4378013611,
		Longitude: -91.9493026733,
		Timezone:  "America/Chicago",
	},
	"MU26": {
		ICAO:      "MU26",
		Name:      "Robinson Airport",
		City:      "Belton",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1080,
		Latitude:  38.7731018066,
		Longitude: -94.5652999878,
		Timezone:  "America/Chicago",
	},
	"MU27": {
		ICAO:      "MU27",
		Name:      "Woodfield Airpark Inc Airport",
		City:      "Lockwood",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1050,
		Latitude:  37.4681015015,
		Longitude: -93.929397583,
		Timezone:  "America/Chicago",
	},
	"MU32": {
		ICAO:      "MU32",
		Name:      "Sugar Branch Airport",
		City:      "Columbia",
		State:     "Missouri",
		Country:   "US",
		Elevation: 700,
		Latitude:  38.9510993958,
		Longitude: -92.4474029541,
		Timezone:  "America/Chicago",
	},
	"MU33": {
		ICAO:      "MU33",
		Name:      "Harrison Airport",
		City:      "Hurdland",
		State:     "Missouri",
		Country:   "US",
		Elevation: 860,
		Latitude:  40.0769996643,
		Longitude: -92.3199005127,
		Timezone:  "America/Chicago",
	},
	"MU34": {
		ICAO:      "MU34",
		Name:      "Haven Wood Airport",
		City:      "Round Spring",
		State:     "Missouri",
		Country:   "US",
		Elevation: 740,
		Latitude:  37.3362007141,
		Longitude: -91.3945999146,
		Timezone:  "America/Chicago",
	},
	"MU35": {
		ICAO:      "MU35",
		Name:      "Tallen Airport",
		City:      "Arkoe",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1123,
		Latitude:  40.2675018311,
		Longitude: -94.7865982056,
		Timezone:  "America/Chicago",
	},
	"MU38": {
		ICAO:      "MU38",
		Name:      "Cowgill-Roemer Airport",
		City:      "Milan",
		State:     "Missouri",
		Country:   "US",
		Elevation: 900,
		Latitude:  40.1783981323,
		Longitude: -93.1313018799,
		Timezone:  "America/Chicago",
	},
	"MU40": {
		ICAO:      "MU40",
		Name:      "Lake Village Airport",
		City:      "Paris",
		State:     "Missouri",
		Country:   "US",
		Elevation: 750,
		Latitude:  39.4606018066,
		Longitude: -92.0103988647,
		Timezone:  "America/Chicago",
	},
	"MU41": {
		ICAO:      "MU41",
		Name:      "Ingram Private Airport",
		City:      "Purdy",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1485,
		Latitude:  36.7867012024,
		Longitude: -93.887802124,
		Timezone:  "America/Chicago",
	},
	"MU42": {
		ICAO:      "MU42",
		Name:      "Strutman Field",
		City:      "Wentzville",
		State:     "Missouri",
		Country:   "US",
		Elevation: 620,
		Latitude:  38.7527999878,
		Longitude: -90.7871017456,
		Timezone:  "America/Chicago",
	},
	"MU48": {
		ICAO:      "MU48",
		Name:      "Sunburst Ranch Airport",
		City:      "Gainesville",
		State:     "Missouri",
		Country:   "US",
		Elevation: 820,
		Latitude:  36.648399353,
		Longitude: -92.2203979492,
		Timezone:  "America/Chicago",
	},
	"MU55": {
		ICAO:      "MU55",
		Name:      "Bauer Pgi Airport",
		City:      "Appleton City",
		State:     "Missouri",
		Country:   "US",
		Elevation: 900,
		Latitude:  38.1366996765,
		Longitude: -94.1737976074,
		Timezone:  "America/Chicago",
	},
	"MU56": {
		ICAO:      "MU56",
		Name:      "Brown Field",
		City:      "Broseley",
		State:     "Missouri",
		Country:   "US",
		Elevation: 323,
		Latitude:  36.7088012695,
		Longitude: -90.2682037354,
		Timezone:  "America/Chicago",
	},
	"MU62": {
		ICAO:      "MU62",
		Name:      "Hilltop Airport",
		City:      "Drexel",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1000,
		Latitude:  38.5633010864,
		Longitude: -94.5911026001,
		Timezone:  "America/Chicago",
	},
	"MU63": {
		ICAO:      "MU63",
		Name:      "Blumenstetter Airport",
		City:      "Elkland",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1327,
		Latitude:  37.4350013733,
		Longitude: -93.0668029785,
		Timezone:  "America/Chicago",
	},
	"MU65": {
		ICAO:      "MU65",
		Name:      "Lakeside Airport",
		City:      "Farmington",
		State:     "Missouri",
		Country:   "US",
		Elevation: 910,
		Latitude:  37.7695007324,
		Longitude: -90.4656982422,
		Timezone:  "America/Chicago",
	},
	"MU68": {
		ICAO:      "MU68",
		Name:      "Eu-Wish Airport",
		City:      "Hermann",
		State:     "Missouri",
		Country:   "US",
		Elevation: 920,
		Latitude:  38.6688995361,
		Longitude: -91.5360031128,
		Timezone:  "America/Chicago",
	},
	"MU69": {
		ICAO:      "MU69",
		Name:      "Peterman Airport",
		City:      "Collins",
		State:     "Missouri",
		Country:   "US",
		Elevation: 888,
		Latitude:  37.8694992065,
		Longitude: -93.6259994507,
		Timezone:  "America/Chicago",
	},
	"MU70": {
		ICAO:      "MU70",
		Name:      "Skyview Airport",
		City:      "Ozark",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1360,
		Latitude:  36.995098114,
		Longitude: -93.1388015747,
		Timezone:  "America/Chicago",
	},
	"MU71": {
		ICAO:      "MU71",
		Name:      "Vandahl Airport",
		City:      "Weston",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1020,
		Latitude:  39.3474998474,
		Longitude: -94.858001709,
		Timezone:  "America/Chicago",
	},
	"MU77": {
		ICAO:      "MU77",
		Name:      "Aero Britton Airport",
		City:      "Fayette",
		State:     "Missouri",
		Country:   "US",
		Elevation: 820,
		Latitude:  39.1291999817,
		Longitude: -92.7306976318,
		Timezone:  "America/Chicago",
	},
	"MU78": {
		ICAO:      "MU78",
		Name:      "Taylor Field",
		City:      "Waynesville",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1022,
		Latitude:  37.847964,
		Longitude: -92.188933,
		Timezone:  "America/Chicago",
	},
	"MU84": {
		ICAO:      "MU84",
		Name:      "Michael Farm Airport",
		City:      "Lone Jack",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1050,
		Latitude:  38.8995018005,
		Longitude: -94.2294006348,
		Timezone:  "America/Chicago",
	},
	"MU85": {
		ICAO:      "MU85",
		Name:      "Thomas Airport",
		City:      "Oak Grove",
		State:     "Missouri",
		Country:   "US",
		Elevation: 890,
		Latitude:  38.9550018311,
		Longitude: -94.1274032593,
		Timezone:  "America/Chicago",
	},
	"MU86": {
		ICAO:      "MU86",
		Name:      "Taus River Ranch Airport",
		City:      "Houston",
		State:     "Missouri",
		Country:   "US",
		Elevation: 956,
		Latitude:  37.4199981689,
		Longitude: -91.9577026367,
		Timezone:  "America/Chicago",
	},
	"MU89": {
		ICAO:      "MU89",
		Name:      "Woodland Airstrip",
		City:      "Troy",
		State:     "Missouri",
		Country:   "US",
		Elevation: 625,
		Latitude:  38.9119987488,
		Longitude: -91.035697937,
		Timezone:  "America/Chicago",
	},
	"MU97": {
		ICAO:      "MU97",
		Name:      "Platte Valley Airport",
		City:      "Platte City",
		State:     "Missouri",
		Country:   "US",
		Elevation: 920,
		Latitude:  39.3675003052,
		Longitude: -94.7617034912,
		Timezone:  "America/Chicago",
	},
	"MU98": {
		ICAO:      "MU98",
		Name:      "Eagles Nest Airport",
		City:      "Golden",
		State:     "Missouri",
		Country:   "US",
		Elevation: 1000,
		Latitude:  36.5181007385,
		Longitude: -93.6836013794,
		Timezone:  "America/Chicago",
	},
	"MUAT": {
		ICAO:      "MUAT",
		Name:      "Antilla Airport",
		City:      "Antilla",
		State:     "Holguin",
		Country:   "CU",
		Elevation: 6,
		Latitude:  20.8388004303,
		Longitude: -75.7328033447,
		Timezone:  "America/Havana",
	},
	"MUBA": {
		ICAO:      "MUBA",
		IATA:      "BCA",
		Name:      "Gustavo Rizo Airport",
		City:      "Baracoa",
		State:     "Guantanamo",
		Country:   "CU",
		Elevation: 26,
		Latitude:  20.3652992249,
		Longitude: -74.5062026978,
		Timezone:  "America/Havana",
	},
	"MUBE": {
		ICAO:      "MUBE",
		Name:      "El Caribe Airport",
		City:      "El Caribe",
		State:     "Pinar-del-Rio",
		Country:   "CU",
		Elevation: 29,
		Latitude:  22.3512001038,
		Longitude: -83.3990020752,
		Timezone:  "America/Havana",
	},
	"MUBO": {
		ICAO:      "MUBO",
		Name:      "Batabano Airport",
		City:      "Batabano",
		State:     "Mayabeque",
		Country:   "CU",
		Latitude:  22.7185993195,
		Longitude: -82.2686004639,
		Timezone:  "America/Havana",
	},
	"MUBR": {
		ICAO:      "MUBR",
		IATA:      "BWW",
		Name:      "Las Brujas Airport",
		City:      "Cayo Santa Maria",
		State:     "Villa-Clara",
		Country:   "CU",
		Elevation: 13,
		Latitude:  22.6212997437,
		Longitude: -79.1472015381,
		Timezone:  "America/Havana",
	},
	"MUBY": {
		ICAO:      "MUBY",
		IATA:      "BYM",
		Name:      "Carlos Manuel de Cespedes Airport",
		City:      "Bayamo",
		State:     "Granma",
		Country:   "CU",
		Elevation: 203,
		Latitude:  20.3964004517,
		Longitude: -76.6213989258,
		Timezone:  "America/Havana",
	},
	"MUCA": {
		ICAO:      "MUCA",
		IATA:      "AVI",
		Name:      "Maximo Gomez Airport",
		City:      "Ciego de Avila",
		State:     "Ciego-de-Avila",
		Country:   "CU",
		Elevation: 335,
		Latitude:  22.0270996094,
		Longitude: -78.7895965576,
		Timezone:  "America/Havana",
	},
	"MUCB": {
		ICAO:      "MUCB",
		Name:      "Caibarien Airport",
		City:      "Caibarien",
		State:     "Villa-Clara",
		Country:   "CU",
		Elevation: 50,
		Latitude:  22.5063991547,
		Longitude: -79.4698028564,
		Timezone:  "America/Havana",
	},
	"MUCC": {
		ICAO:      "MUCC",
		IATA:      "CCC",
		Name:      "Jardines Del Rey Airport",
		City:      "Cayo Coco",
		State:     "Ciego-de-Avila",
		Country:   "CU",
		Elevation: 13,
		Latitude:  22.4610004425,
		Longitude: -78.3283996582,
		Timezone:  "America/Havana",
	},
	"MUCD": {
		ICAO:      "MUCD",
		Name:      "Ciego De Avila Sur Airport",
		City:      "Ciego de Avila",
		State:     "Ciego-de-Avila",
		Country:   "CU",
		Elevation: 131,
		Latitude:  21.8211994171,
		Longitude: -78.7566986084,
		Timezone:  "America/Havana",
	},
	"MUCF": {
		ICAO:      "MUCF",
		IATA:      "CFG",
		Name:      "Jaime Gonzalez Airport",
		City:      "Cienfuegos",
		State:     "Cienfuegos",
		Country:   "CU",
		Elevation: 102,
		Latitude:  22.1499996185,
		Longitude: -80.4141998291,
		Timezone:  "America/Havana",
	},
	"MUCL": {
		ICAO:      "MUCL",
		IATA:      "CYO",
		Name:      "Vilo Acuna International Airport",
		City:      "Cayo Largo del Sur",
		State:     "Isla-de-la-Juventud",
		Country:   "CU",
		Elevation: 10,
		Latitude:  21.6165008545,
		Longitude: -81.5459976196,
		Timezone:  "America/Havana",
	},
	"MUCM": {
		ICAO:      "MUCM",
		IATA:      "CMW",
		Name:      "Ignacio Agramonte International Airport",
		City:      "Camaguey",
		State:     "Camagueey",
		Country:   "CU",
		Elevation: 413,
		Latitude:  21.42029953,
		Longitude: -77.8475036621,
		Timezone:  "America/Havana",
	},
	"MUCO": {
		ICAO:      "MUCO",
		IATA:      "QCO",
		Name:      "Colon Airport",
		City:      "Colon",
		State:     "Matanzas",
		Country:   "CU",
		Latitude:  22.7110996246,
		Longitude: -80.9227981567,
		Timezone:  "America/Havana",
	},
	"MUCU": {
		ICAO:      "MUCU",
		IATA:      "SCU",
		Name:      "Antonio Maceo International Airport",
		City:      "Santiago",
		State:     "Santiago-de-Cuba",
		Country:   "CU",
		Elevation: 249,
		Latitude:  19.9698009491,
		Longitude: -75.8354034424,
		Timezone:  "America/Havana",
	},
	"MUCV": {
		ICAO:      "MUCV",
		Name:      "Las Clavellinas Airport",
		City:      "Las Clavellinas",
		State:     "Pinar-del-Rio",
		Country:   "CU",
		Elevation: 19,
		Latitude:  22.2287006378,
		Longitude: -84.3720016479,
		Timezone:  "America/Havana",
	},
	"MUCY": {
		ICAO:      "MUCY",
		Name:      "Cayajabo Airport",
		City:      "Cayajabo",
		State:     "Artemisa",
		Country:   "CU",
		Elevation: 259,
		Latitude:  22.8644447327,
		Longitude: -82.8558349609,
		Timezone:  "America/Havana",
	},
	"MUFL": {
		ICAO:      "MUFL",
		Name:      "Florida Airport",
		City:      "Florida",
		State:     "Camagueey",
		Country:   "CU",
		Elevation: 197,
		Latitude:  21.4997005463,
		Longitude: -78.202796936,
		Timezone:  "America/Havana",
	},
	"MUGA": {
		ICAO:      "MUGA",
		Name:      "Central Agramonte Airport",
		City:      "Florida",
		State:     "Camagueey",
		Country:   "CU",
		Elevation: 196,
		Latitude:  21.5499992371,
		Longitude: -78.2333297729,
		Timezone:  "America/Havana",
	},
	"MUGM": {
		ICAO:      "MUGM",
		IATA:      "NBW",
		Name:      "Leeward Point Field",
		City:      "Guantanamo Bay Naval Station",
		State:     "Guantanamo",
		Country:   "CU",
		Elevation: 56,
		Latitude:  19.9064998627,
		Longitude: -75.2070999146,
		Timezone:  "America/Havana",
	},
	"MUGT": {
		ICAO:      "MUGT",
		IATA:      "GAO",
		Name:      "Mariana Grajales Airport",
		City:      "Guantanamo",
		State:     "Guantanamo",
		Country:   "CU",
		Elevation: 56,
		Latitude:  20.0853004456,
		Longitude: -75.1583023071,
		Timezone:  "America/Havana",
	},
	"MUGV": {
		ICAO:      "MUGV",
		Name:      "Guardalavaca Airport",
		City:      "Yaguajay",
		State:     "Holguin",
		Country:   "CU",
		Elevation: 318,
		Latitude:  21.1116008759,
		Longitude: -75.8198013306,
		Timezone:  "America/Havana",
	},
	"MUHA": {
		ICAO:      "MUHA",
		IATA:      "HAV",
		Name:      "Jose Marti International Airport",
		City:      "Havana",
		State:     "La-Habana",
		Country:   "CU",
		Elevation: 210,
		Latitude:  22.989200592,
		Longitude: -82.4091033936,
		Timezone:  "America/Havana",
	},
	"MUHG": {
		ICAO:      "MUHG",
		IATA:      "HOG",
		Name:      "Frank Pais International Airport",
		City:      "Holguin",
		State:     "Holguin",
		Country:   "CU",
		Elevation: 361,
		Latitude:  20.7856006622,
		Longitude: -76.3151016235,
		Timezone:  "America/Havana",
	},
	"MUIV": {
		ICAO:      "MUIV",
		Name:      "Nuevitas Airport",
		City:      "Nuevitas",
		State:     "Camagueey",
		Country:   "CU",
		Elevation: 3,
		Latitude:  21.5538005829,
		Longitude: -77.2585983276,
		Timezone:  "America/Havana",
	},
	"MUKW": {
		ICAO:      "MUKW",
		IATA:      "VRO",
		Name:      "Kawama Airport",
		City:      "Matanzas",
		State:     "Matanzas",
		Country:   "CU",
		Elevation: 16,
		Latitude:  23.1240005493,
		Longitude: -81.3015975952,
		Timezone:  "America/Havana",
	},
	"MULB": {
		ICAO:      "MULB",
		Name:      "Ciudad Libertad Airport",
		City:      "Havana",
		State:     "La-Habana",
		Country:   "CU",
		Elevation: 98,
		Latitude:  23.0939006805,
		Longitude: -82.4381027222,
		Timezone:  "America/Havana",
	},
	"MULM": {
		ICAO:      "MULM",
		IATA:      "LCL",
		Name:      "La Coloma Airport",
		City:      "Pinar del Rio",
		State:     "Pinar-del-Rio",
		Country:   "CU",
		Elevation: 131,
		Latitude:  22.3360996246,
		Longitude: -83.6418991089,
		Timezone:  "America/Havana",
	},
	"MUMA": {
		ICAO:      "MUMA",
		IATA:      "UMA",
		Name:      "Punta de Maisi Airport",
		City:      "Maisi",
		State:     "Guantanamo",
		Country:   "CU",
		Elevation: 3,
		Latitude:  20.2506008148,
		Longitude: -74.1504974365,
		Timezone:  "America/Havana",
	},
	"MUMG": {
		ICAO:      "MUMG",
		Name:      "Managua Airport",
		City:      "Managua",
		State:     "Mayabeque",
		Country:   "CU",
		Elevation: 377,
		Latitude:  22.9699001312,
		Longitude: -82.2748031616,
		Timezone:  "America/Havana",
	},
	"MUMH": {
		ICAO:      "MUMH",
		Name:      "Matahambre Airport",
		City:      "Matahambre",
		State:     "Pinar-del-Rio",
		Country:   "CU",
		Elevation: 396,
		Latitude:  22.5874996185,
		Longitude: -83.9449996948,
		Timezone:  "America/Havana",
	},
	"MUMJ": {
		ICAO:      "MUMJ",
		IATA:      "MJG",
		Name:      "Mayajigua Airport",
		City:      "Mayajigua",
		State:     "Sancti-Spiritus",
		Country:   "CU",
		Elevation: 173,
		Latitude:  22.2308006287,
		Longitude: -79.0622024536,
		Timezone:  "America/Havana",
	},
	"MUMO": {
		ICAO:      "MUMO",
		IATA:      "MOA",
		Name:      "Orestes Acosta Airport",
		City:      "Moa",
		State:     "Holguin",
		Country:   "CU",
		Elevation: 16,
		Latitude:  20.6539001465,
		Longitude: -74.922203064,
		Timezone:  "America/Havana",
	},
	"MUMT": {
		ICAO:      "MUMT",
		Name:      "Matanzas Airport",
		City:      "Matanzas",
		State:     "Matanzas",
		Country:   "CU",
		Elevation: 9,
		Latitude:  23.0100002289,
		Longitude: -81.5100021362,
		Timezone:  "America/Havana",
	},
	"MUMZ": {
		ICAO:      "MUMZ",
		IATA:      "MZO",
		Name:      "Sierra Maestra Airport",
		City:      "Manzanillo",
		State:     "Granma",
		Country:   "CU",
		Elevation: 112,
		Latitude:  20.2880992889,
		Longitude: -77.0892028809,
		Timezone:  "America/Havana",
	},
	"MUNB": {
		ICAO:      "MUNB",
		IATA:      "QSN",
		Name:      "San Nicolas De Bari Airport",
		City:      "San Nicolas",
		State:     "Mayabeque",
		Country:   "CU",
		Elevation: 49,
		Latitude:  22.7560997009,
		Longitude: -81.9208984375,
		Timezone:  "America/Havana",
	},
	"MUNC": {
		ICAO:      "MUNC",
		IATA:      "ICR",
		Name:      "Nicaro Airport",
		City:      "Nicaro",
		State:     "Holguin",
		Country:   "CU",
		Elevation: 26,
		Latitude:  20.6886005402,
		Longitude: -75.53150177,
		Timezone:  "America/Havana",
	},
	"MUNG": {
		ICAO:      "MUNG",
		IATA:      "GER",
		Name:      "Rafael Cabrera Airport",
		City:      "Nueva Gerona",
		State:     "Isla-de-la-Juventud",
		Country:   "CU",
		Elevation: 79,
		Latitude:  21.8346996307,
		Longitude: -82.7837982178,
		Timezone:  "America/Havana",
	},
	"MUPA": {
		ICAO:      "MUPA",
		Name:      "Punta Alegre Airport",
		City:      "Punta Alegre",
		State:     "Ciego-de-Avila",
		Country:   "CU",
		Elevation: 82,
		Latitude:  22.3799991608,
		Longitude: -78.8174972534,
		Timezone:  "America/Havana",
	},
	"MUPB": {
		ICAO:      "MUPB",
		IATA:      "UPB",
		Name:      "Playa Baracoa Airport",
		City:      "Havana",
		State:     "Artemisa",
		Country:   "CU",
		Elevation: 102,
		Latitude:  23.0328006744,
		Longitude: -82.5793991089,
		Timezone:  "America/Havana",
	},
	"MUPR": {
		ICAO:      "MUPR",
		IATA:      "QPD",
		Name:      "Pinar Del Rio Airport",
		City:      "Pinar del Rio",
		State:     "Pinar-del-Rio",
		Country:   "CU",
		Elevation: 131,
		Latitude:  22.4214000702,
		Longitude: -83.6783981323,
		Timezone:  "America/Havana",
	},
	"MUSA": {
		ICAO:      "MUSA",
		Name:      "San Antonio De Los Banos Airport",
		City:      "San Antonio de los Banos",
		State:     "Artemisa",
		Country:   "CU",
		Elevation: 164,
		Latitude:  22.8715000153,
		Longitude: -82.5093002319,
		Timezone:  "America/Havana",
	},
	"MUSC": {
		ICAO:      "MUSC",
		IATA:      "SNU",
		Name:      "Abel Santamaria Airport",
		City:      "Santa Clara",
		State:     "Villa-Clara",
		Country:   "CU",
		Elevation: 338,
		Latitude:  22.4922008514,
		Longitude: -79.9436035156,
		Timezone:  "America/Havana",
	},
	"MUSG": {
		ICAO:      "MUSG",
		Name:      "Sagua La Grande Airport",
		City:      "Sagua la Grande",
		State:     "Villa-Clara",
		Country:   "CU",
		Elevation: 36,
		Latitude:  22.8015995026,
		Longitude: -80.0548019409,
		Timezone:  "America/Havana",
	},
	"MUSJ": {
		ICAO:      "MUSJ",
		IATA:      "SNJ",
		Name:      "San Julian Air Base",
		City:      "Pinar Del Rio",
		State:     "Pinar-del-Rio",
		Country:   "CU",
		Elevation: 98,
		Latitude:  22.0953006744,
		Longitude: -84.1520004272,
		Timezone:  "America/Havana",
	},
	"MUSL": {
		ICAO:      "MUSL",
		Name:      "Joaquin de Aguero Airport",
		City:      "Playa Santa Lucia",
		State:     "Camagueey",
		Country:   "CU",
		Elevation: 13,
		Latitude:  21.5094559067,
		Longitude: -77.0205974579,
		Timezone:  "America/Havana",
	},
	"MUSN": {
		ICAO:      "MUSN",
		IATA:      "SZJ",
		Name:      "Siguanea Airport",
		City:      "Isla de la Juventud",
		State:     "Isla-de-la-Juventud",
		Country:   "CU",
		Elevation: 39,
		Latitude:  21.6424999237,
		Longitude: -82.9551010132,
		Timezone:  "America/Havana",
	},
	"MUSS": {
		ICAO:      "MUSS",
		IATA:      "USS",
		Name:      "Sancti Spiritus Airport",
		City:      "Sancti Spiritus",
		State:     "Sancti-Spiritus",
		Country:   "CU",
		Elevation: 295,
		Latitude:  21.9703998566,
		Longitude: -79.4427032471,
		Timezone:  "America/Havana",
	},
	"MUTD": {
		ICAO:      "MUTD",
		IATA:      "TND",
		Name:      "Alberto Delgado Airport",
		City:      "Trinidad",
		State:     "Sancti-Spiritus",
		Country:   "CU",
		Elevation: 125,
		Latitude:  21.7882995605,
		Longitude: -79.9972000122,
		Timezone:  "America/Havana",
	},
	"MUTI": {
		ICAO:      "MUTI",
		Name:      "Manati Airport",
		City:      "Manati",
		State:     "Las-Tunas",
		Country:   "CU",
		Elevation: 6,
		Latitude:  21.3106994629,
		Longitude: -76.9201965332,
		Timezone:  "America/Havana",
	},
	"MUVR": {
		ICAO:      "MUVR",
		IATA:      "VRA",
		Name:      "Juan Gualberto Gomez International Airport",
		City:      "Varadero",
		State:     "Matanzas",
		Country:   "CU",
		Elevation: 210,
		Latitude:  23.0344009399,
		Longitude: -81.4353027344,
		Timezone:  "America/Havana",
	},
	"MUVT": {
		ICAO:      "MUVT",
		IATA:      "VTU",
		Name:      "Hermanos Ameijeiras Airport",
		City:      "Las Tunas",
		State:     "Las-Tunas",
		Country:   "CU",
		Elevation: 328,
		Latitude:  20.9876003265,
		Longitude: -76.935798645,
		Timezone:  "America/Havana",
	},
	"MWCB": {
		ICAO:      "MWCB",
		IATA:      "CYB",
		Name:      "Gerrard Smith International Airport",
		City:      "Cayman Brac",
		Country:   "KY",
		Elevation: 8,
		Latitude:  19.6870002747,
		Longitude: -79.8827972412,
		Timezone:  "America/Cayman",
	},
	"MWCL": {
		ICAO:      "MWCL",
		IATA:      "LYB",
		Name:      "Edward Bodden Airfield",
		City:      "Little Cayman",
		Country:   "KY",
		Elevation: 3,
		Latitude:  19.6669998169,
		Longitude: -80.0999984741,
		Timezone:  "America/Cayman",
	},
	"MWCR": {
		ICAO:      "MWCR",
		IATA:      "GCM",
		Name:      "Owen Roberts International Airport",
		City:      "Georgetown",
		Country:   "KY",
		Elevation: 8,
		Latitude:  19.2928009033,
		Longitude: -81.3576965332,
		Timezone:  "America/Cayman",
	},
	"MY00": {
		ICAO:      "MY00",
		Name:      "Sieg's Farm Airport",
		City:      "Truman",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1060,
		Latitude:  43.8941001892,
		Longitude: -94.4938964844,
		Timezone:  "America/Chicago",
	},
	"MY01": {
		ICAO:      "MY01",
		Name:      "Roan Airport",
		City:      "Warren",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 860,
		Latitude:  48.1458015442,
		Longitude: -96.7526016235,
		Timezone:  "America/Chicago",
	},
	"MY03": {
		ICAO:      "MY03",
		Name:      "Braun's Airport",
		City:      "Sleepy Eye",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1009,
		Latitude:  44.2816009521,
		Longitude: -94.6405029297,
		Timezone:  "America/Chicago",
	},
	"MY04": {
		ICAO:      "MY04",
		Name:      "Koch's Personal Field",
		City:      "Taunton",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1130,
		Latitude:  44.6599998474,
		Longitude: -96.0373001099,
		Timezone:  "America/Chicago",
	},
	"MY06": {
		ICAO:      "MY06",
		Name:      "Ramerth Airport",
		City:      "Fulda",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1500,
		Latitude:  43.831401825,
		Longitude: -95.5552978516,
		Timezone:  "America/Chicago",
	},
	"MY07": {
		ICAO:      "MY07",
		Name:      "Nord Field",
		City:      "Wolverton",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 948,
		Latitude:  46.5833015442,
		Longitude: -96.6064987183,
		Timezone:  "America/Chicago",
	},
	"MY10": {
		ICAO:      "MY10",
		Name:      "Deters Farms Airport",
		City:      "Spring Grove",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1220,
		Latitude:  43.5611000061,
		Longitude: -91.6815032959,
		Timezone:  "America/Chicago",
	},
	"MY11": {
		ICAO:      "MY11",
		Name:      "Gilgenbach's Airport",
		City:      "Taopi",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1320,
		Latitude:  43.5214004517,
		Longitude: -92.6585006714,
		Timezone:  "America/Chicago",
	},
	"MY12": {
		ICAO:      "MY12",
		Name:      "Dykstra Acreage Airport",
		City:      "Trosky",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1690,
		Latitude:  43.8807983398,
		Longitude: -96.2530975342,
		Timezone:  "America/Chicago",
	},
	"MY13": {
		ICAO:      "MY13",
		Name:      "Swan Field",
		City:      "Saginaw",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1345,
		Latitude:  46.86930084,
		Longitude: -92.45709991,
		Timezone:  "America/Chicago",
	},
	"MY15": {
		ICAO:      "MY15",
		Name:      "Kastanek Airport",
		City:      "Pierz",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1320,
		Latitude:  45.9557991028,
		Longitude: -93.989402771,
		Timezone:  "America/Chicago",
	},
	"MY17": {
		ICAO:      "MY17",
		Name:      "Swanson Private Airport",
		City:      "Goodridge",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1170,
		Latitude:  48.0889015198,
		Longitude: -95.7746963501,
		Timezone:  "America/Chicago",
	},
	"MY18": {
		ICAO:      "MY18",
		Name:      "Lino Air Park",
		City:      "Lino Lakes",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 907,
		Latitude:  45.1871986389,
		Longitude: -93.1305007935,
		Timezone:  "America/Chicago",
	},
	"MY19": {
		ICAO:      "MY19",
		Name:      "Schwenk Airport",
		City:      "Murdock",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1082,
		Latitude:  45.1707992554,
		Longitude: -95.3925018311,
		Timezone:  "America/Chicago",
	},
	"MY24": {
		ICAO:      "MY24",
		Name:      "Falk Private Airport",
		City:      "Park Rapids",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1443,
		Latitude:  46.8622016907,
		Longitude: -95.0522003174,
		Timezone:  "America/Chicago",
	},
	"MY25": {
		ICAO:      "MY25",
		Name:      "Knapp Personal Use Airport",
		City:      "Foxhome",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1018,
		Latitude:  46.2799987793,
		Longitude: -96.3314971924,
		Timezone:  "America/Chicago",
	},
	"MY28": {
		ICAO:      "MY28",
		Name:      "Pinetree Airpark",
		City:      "Harris",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 930,
		Latitude:  45.5797996521,
		Longitude: -92.9630966187,
		Timezone:  "America/Chicago",
	},
	"MY29": {
		ICAO:      "MY29",
		Name:      "Kaiser's Airstrip",
		City:      "Lake Park",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1400,
		Latitude:  46.7969017029,
		Longitude: -96.0996017456,
		Timezone:  "America/Chicago",
	},
	"MY35": {
		ICAO:      "MY35",
		Name:      "Walker Field",
		City:      "Afton",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 902,
		Latitude:  44.9334983826,
		Longitude: -92.8177032471,
		Timezone:  "America/Chicago",
	},
	"MY37": {
		ICAO:      "MY37",
		Name:      "Thomas Field",
		City:      "Eyota",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1285,
		Latitude:  43.981076,
		Longitude: -92.246554,
		Timezone:  "America/Chicago",
	},
	"MY42": {
		ICAO:      "MY42",
		Name:      "Mathis Airport",
		City:      "Gonvick",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1170,
		Latitude:  47.8638000488,
		Longitude: -95.5270004272,
		Timezone:  "America/Chicago",
	},
	"MY43": {
		ICAO:      "MY43",
		Name:      "Becker Personal Airport",
		City:      "St Francis",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 920,
		Latitude:  45.4026985168,
		Longitude: -93.3893966675,
		Timezone:  "America/Chicago",
	},
	"MY46": {
		ICAO:      "MY46",
		Name:      "Hay Shakers Airport",
		City:      "Nicollet",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 995,
		Latitude:  44.2792015076,
		Longitude: -94.2133026123,
		Timezone:  "America/Chicago",
	},
	"MY47": {
		ICAO:      "MY47",
		Name:      "Guggenberger Airport",
		City:      "Sartell",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1050,
		Latitude:  45.6046981812,
		Longitude: -94.2108001709,
		Timezone:  "America/Chicago",
	},
	"MY49": {
		ICAO:      "MY49",
		Name:      "Anderson Airport",
		City:      "Bear River",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1310,
		Latitude:  47.787399292,
		Longitude: -93.0605010986,
		Timezone:  "America/Chicago",
	},
	"MY50": {
		ICAO:      "MY50",
		Name:      "Frontenac Airport",
		City:      "Frontenac",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 793,
		Latitude:  44.5027008057,
		Longitude: -92.3612976074,
		Timezone:  "America/Chicago",
	},
	"MY52": {
		ICAO:      "MY52",
		Name:      "Turkey Track Airport",
		City:      "Coates",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 923,
		Latitude:  44.7186012268,
		Longitude: -93.0440979004,
		Timezone:  "America/Chicago",
	},
	"MY54": {
		ICAO:      "MY54",
		Name:      "Cloverleaf-East Bemidji Airport",
		City:      "Bemidji",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1445,
		Latitude:  47.4383010864,
		Longitude: -94.8122024536,
		Timezone:  "America/Chicago",
	},
	"MY55": {
		ICAO:      "MY55",
		Name:      "B I R Airport",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1210,
		Latitude:  46.4197006226,
		Longitude: -94.2752990723,
		Timezone:  "America/Chicago",
	},
	"MY56": {
		ICAO:      "MY56",
		Name:      "Robertson Field",
		City:      "Hibbing",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1290,
		Latitude:  47.2523994446,
		Longitude: -92.8593978882,
		Timezone:  "America/Chicago",
	},
	"MY57": {
		ICAO:      "MY57",
		Name:      "Fedor Airport",
		City:      "Holdingford",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1175,
		Latitude:  45.7633018494,
		Longitude: -94.4574966431,
		Timezone:  "America/Chicago",
	},
	"MY58": {
		ICAO:      "MY58",
		Name:      "Home-Base Airport",
		City:      "Holloway",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1020,
		Latitude:  45.2388000488,
		Longitude: -95.9253005981,
		Timezone:  "America/Chicago",
	},
	"MY59": {
		ICAO:      "MY59",
		Name:      "Gasper Airport",
		City:      "Euclid",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 910,
		Latitude:  47.901599884,
		Longitude: -96.6122970581,
		Timezone:  "America/Chicago",
	},
	"MY60": {
		ICAO:      "MY60",
		Name:      "Hagen Airport",
		City:      "Richville",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1370,
		Latitude:  46.5027008057,
		Longitude: -95.6197967529,
		Timezone:  "America/Chicago",
	},
	"MY62": {
		ICAO:      "MY62",
		Name:      "Ultraflyte Inc Ultralightport",
		City:      "Jenkins",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1270,
		Latitude:  46.6651992798,
		Longitude: -94.3432998657,
		Timezone:  "America/Chicago",
	},
	"MY64": {
		ICAO:      "MY64",
		Name:      "Schjeldrup Airport",
		City:      "Felton",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 903,
		Latitude:  47.0311012268,
		Longitude: -96.6023025513,
		Timezone:  "America/Chicago",
	},
	"MY66": {
		ICAO:      "MY66",
		Name:      "North Star Ranch Airport",
		City:      "Onamia",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1330,
		Latitude:  46.0704994202,
		Longitude: -93.8471984863,
		Timezone:  "America/Chicago",
	},
	"MY68": {
		ICAO:      "MY68",
		Name:      "Johnsons Aero Repair Airport",
		City:      "Wheaton",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1028,
		Latitude:  45.7771987915,
		Longitude: -96.3914031982,
		Timezone:  "America/Chicago",
	},
	"MY71": {
		ICAO:      "MY71",
		Name:      "Mueller Airport",
		City:      "Georgetown",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 880,
		Latitude:  47.1263999939,
		Longitude: -96.7806015015,
		Timezone:  "America/Chicago",
	},
	"MY72": {
		ICAO:      "MY72",
		Name:      "Isle Private Airport",
		City:      "Isle",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1271,
		Latitude:  46.1596984863,
		Longitude: -93.4613037109,
		Timezone:  "America/Chicago",
	},
	"MY78": {
		ICAO:      "MY78",
		Name:      "Sunrise Airport",
		City:      "Stacy",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 910,
		Latitude:  45.4291000366,
		Longitude: -92.9862976074,
		Timezone:  "America/Chicago",
	},
	"MY80": {
		ICAO:      "MY80",
		Name:      "Rosenberg Airport",
		City:      "Ceylon",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1255,
		Latitude:  43.5222015381,
		Longitude: -94.5824966431,
		Timezone:  "America/Chicago",
	},
	"MY82": {
		ICAO:      "MY82",
		Name:      "Du Fresne Airport",
		City:      "Forest Lake",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 900,
		Latitude:  45.2130012512,
		Longitude: -93.0746994019,
		Timezone:  "America/Chicago",
	},
	"MY88": {
		ICAO:      "MY88",
		Name:      "Cary Airport",
		City:      "Park Rapids",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1413,
		Latitude:  46.8574981689,
		Longitude: -94.9831008911,
		Timezone:  "America/Chicago",
	},
	"MY89": {
		ICAO:      "MY89",
		Name:      "Pavek Personal Airport",
		City:      "Pine City",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 980,
		Latitude:  45.9082984924,
		Longitude: -92.9057998657,
		Timezone:  "America/Chicago",
	},
	"MY90": {
		ICAO:      "MY90",
		Name:      "Al's Due North Airport",
		City:      "North Branch",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 915,
		Latitude:  45.4822006226,
		Longitude: -92.9885025024,
		Timezone:  "America/Chicago",
	},
	"MY92": {
		ICAO:      "MY92",
		Name:      "Grohnke Field",
		City:      "Braham",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 960,
		Latitude:  45.7285995483,
		Longitude: -93.2882995605,
		Timezone:  "America/Chicago",
	},
	"MY94": {
		ICAO:      "MY94",
		Name:      "Country Haven Airport",
		City:      "Cloquet",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1309,
		Latitude:  46.7524986267,
		Longitude: -92.4191970825,
		Timezone:  "America/Chicago",
	},
	"MY95": {
		ICAO:      "MY95",
		Name:      "Swanson Field",
		City:      "Crown",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 935,
		Latitude:  45.4463996887,
		Longitude: -93.3827972412,
		Timezone:  "America/Chicago",
	},
	"MY97": {
		ICAO:      "MY97",
		Name:      "Nielsen's Airport",
		City:      "Farmington",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 990,
		Latitude:  44.5979995728,
		Longitude: -93.1268997192,
		Timezone:  "America/Chicago",
	},
	"MY98": {
		ICAO:      "MY98",
		Name:      "Pake Airport",
		City:      "Felton",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 898,
		Latitude:  47.045501709,
		Longitude: -96.6656036377,
		Timezone:  "America/Chicago",
	},
	"MY99": {
		ICAO:      "MY99",
		Name:      "Johnston Airport",
		City:      "Hawley",
		State:     "Minnesota",
		Country:   "US",
		Elevation: 1360,
		Latitude:  46.8574981689,
		Longitude: -96.2351989746,
		Timezone:  "America/Chicago",
	},
	"MYA0": {
		ICAO:      "MYA0",
		Name:      "Mores Island Airport",
		City:      "Mores Island",
		State:     "Moore’s-Island",
		Country:   "BS",
		Elevation: 10,
		Latitude:  26.3176994324,
		Longitude: -77.5635986328,
		Timezone:  "America/Nassau",
	},
	"MYAB": {
		ICAO:      "MYAB",
		IATA:      "MAY",
		Name:      "Clarence A. Bain Airport",
		City:      "Mangrove Cay",
		State:     "Mangrove-Cay",
		Country:   "BS",
		Elevation: 19,
		Latitude:  24.2877006531,
		Longitude: -77.6846008301,
		Timezone:  "America/Nassau",
	},
	"MYAF": {
		ICAO:      "MYAF",
		IATA:      "ASD",
		Name:      "Andros Town Airport",
		State:     "North-Andros",
		Country:   "BS",
		Elevation: 5,
		Latitude:  24.6979007721,
		Longitude: -77.7956008911,
		Timezone:  "America/Nassau",
	},
	"MYAK": {
		ICAO:      "MYAK",
		IATA:      "COX",
		Name:      "Congo Town Airport",
		City:      "Andros",
		State:     "South-Andros",
		Country:   "BS",
		Elevation: 15,
		Latitude:  24.158700943,
		Longitude: -77.5897979736,
		Timezone:  "America/Nassau",
	},
	"MYAM": {
		ICAO:      "MYAM",
		IATA:      "MHH",
		Name:      "Marsh Harbour International Airport",
		City:      "Marsh Harbour",
		Country:   "BS",
		Elevation: 6,
		Latitude:  26.5114002228,
		Longitude: -77.0835037231,
		Timezone:  "America/Nassau",
	},
	"MYAN": {
		ICAO:      "MYAN",
		IATA:      "SAQ",
		Name:      "San Andros Airport",
		City:      "Andros Island",
		State:     "North-Andros",
		Country:   "BS",
		Elevation: 5,
		Latitude:  25.0538005829,
		Longitude: -78.0490036011,
		Timezone:  "America/Nassau",
	},
	"MYAP": {
		ICAO:      "MYAP",
		IATA:      "AXP",
		Name:      "Spring Point Airport",
		City:      "Spring Point",
		State:     "Acklins",
		Country:   "BS",
		Elevation: 11,
		Latitude:  22.4417991638,
		Longitude: -73.9709014893,
		Timezone:  "America/Nassau",
	},
	"MYAS": {
		ICAO:      "MYAS",
		Name:      "Sandy Point Airport",
		State:     "South-Abaco",
		Country:   "BS",
		Elevation: 8,
		Latitude:  26.0046005249,
		Longitude: -77.3955001831,
		Timezone:  "America/Nassau",
	},
	"MYAT": {
		ICAO:      "MYAT",
		IATA:      "TCB",
		Name:      "Treasure Cay Airport",
		City:      "Treasure Cay",
		Country:   "BS",
		Elevation: 8,
		Latitude:  26.745300293,
		Longitude: -77.3912963867,
		Timezone:  "America/Nassau",
	},
	"MYAW": {
		ICAO:      "MYAW",
		IATA:      "WKR",
		Name:      "Abaco I Walker C Airport",
		State:     "Grand-Cay",
		Country:   "BS",
		Elevation: 10,
		Latitude:  27.2667007446,
		Longitude: -78.3996963501,
		Timezone:  "America/Nassau",
	},
	"MYAX": {
		ICAO:      "MYAX",
		Name:      "Spanish Cay Airport",
		State:     "North-Abaco",
		Country:   "BS",
		Elevation: 10,
		Latitude:  26.9503002167,
		Longitude: -77.543800354,
		Timezone:  "America/Nassau",
	},
	"MYBC": {
		ICAO:      "MYBC",
		IATA:      "CCZ",
		Name:      "Chub Cay Airport",
		State:     "Berry-Islands",
		Country:   "BS",
		Elevation: 5,
		Latitude:  25.4171009064,
		Longitude: -77.880897522,
		Timezone:  "America/Nassau",
	},
	"MYBG": {
		ICAO:      "MYBG",
		IATA:      "GHC",
		Name:      "Great Harbour Cay Airport",
		State:     "Berry-Islands",
		Country:   "BS",
		Elevation: 18,
		Latitude:  25.7383003235,
		Longitude: -77.8401031494,
		Timezone:  "America/Nassau",
	},
	"MYBS": {
		ICAO:      "MYBS",
		IATA:      "BIM",
		Name:      "South Bimini Airport",
		City:      "South Bimini",
		State:     "Bimini",
		Country:   "BS",
		Elevation: 10,
		Latitude:  25.6998996735,
		Longitude: -79.2647018433,
		Timezone:  "America/Nassau",
	},
	"MYCA": {
		ICAO:      "MYCA",
		IATA:      "ATC",
		Name:      "Arthur's Town Airport",
		City:      "Arthur's Town",
		State:     "Cat-Island",
		Country:   "BS",
		Elevation: 18,
		Latitude:  24.6294002533,
		Longitude: -75.6737976074,
		Timezone:  "America/Nassau",
	},
	"MYCB": {
		ICAO:      "MYCB",
		IATA:      "CAT",
		Name:      "New Bight Airport",
		City:      "Cat Island",
		State:     "Cat-Island",
		Country:   "BS",
		Elevation: 5,
		Latitude:  24.3152999878,
		Longitude: -75.4523010254,
		Timezone:  "America/Nassau",
	},
	"MYCC": {
		ICAO:      "MYCC",
		IATA:      "CXY",
		Name:      "Cat Cay Airport",
		City:      "Cat Cay",
		State:     "Bimini",
		Country:   "BS",
		Latitude:  25.6000003815,
		Longitude: -79.266998291,
		Timezone:  "America/Nassau",
	},
	"MYCH": {
		ICAO:      "MYCH",
		Name:      "Hawks News Airport",
		City:      "Hawks News",
		State:     "Cat-Island",
		Country:   "BS",
		Elevation: 3,
		Latitude:  24.1536006927,
		Longitude: -75.5199966431,
		Timezone:  "America/Nassau",
	},
	"MYCI": {
		ICAO:      "MYCI",
		IATA:      "CRI",
		Name:      "Colonel Hill Airport",
		City:      "Colonel Hill",
		State:     "Crooked-Island-and-Long-Cay",
		Country:   "BS",
		Elevation: 5,
		Latitude:  22.7455997467,
		Longitude: -74.1824035645,
		Timezone:  "America/Nassau",
	},
	"MYCP": {
		ICAO:      "MYCP",
		IATA:      "PWN",
		Name:      "Pitts Town Airport",
		City:      "Pitts Town",
		State:     "Crooked-Island-and-Long-Cay",
		Country:   "BS",
		Latitude:  22.82970047,
		Longitude: -74.3460998535,
		Timezone:  "America/Nassau",
	},
	"MYCS": {
		ICAO:      "MYCS",
		Name:      "Cay Sal Airport",
		City:      "Cay Sal",
		State:     "Bimini",
		Country:   "BS",
		Elevation: 3,
		Latitude:  23.6963996887,
		Longitude: -80.3212966919,
		Timezone:  "America/Nassau",
	},
	"MYCX": {
		ICAO:      "MYCX",
		Name:      "Cutlass Bay Airport",
		City:      "Cutlass Bay",
		State:     "Cat-Island",
		Country:   "BS",
		Elevation: 3,
		Latitude:  24.1488990784,
		Longitude: -75.3981018066,
		Timezone:  "America/Nassau",
	},
	"MYEB": {
		ICAO:      "MYEB",
		Name:      "Black Point Airstrip",
		City:      "Black Point",
		State:     "Black-Point",
		Country:   "BS",
		Elevation: 10,
		Latitude:  24.0894884488,
		Longitude: -76.3979172707,
		Timezone:  "America/Nassau",
	},
	"MYEF": {
		ICAO:      "MYEF",
		IATA:      "GGT",
		Name:      "Exuma International Airport",
		City:      "George Town",
		State:     "Exuma",
		Country:   "BS",
		Elevation: 9,
		Latitude:  23.5625991821,
		Longitude: -75.8779983521,
		Timezone:  "America/Nassau",
	},
	"MYEG": {
		ICAO:      "MYEG",
		Name:      "George Town Airport",
		City:      "George Town",
		State:     "Exuma",
		Country:   "BS",
		Elevation: 5,
		Latitude:  23.4666996002,
		Longitude: -75.7817001343,
		Timezone:  "America/Nassau",
	},
	"MYEH": {
		ICAO:      "MYEH",
		IATA:      "ELH",
		Name:      "North Eleuthera Airport",
		City:      "North Eleuthera",
		State:     "North-Eleuthera",
		Country:   "BS",
		Elevation: 13,
		Latitude:  25.474899292,
		Longitude: -76.6835021973,
		Timezone:  "America/Nassau",
	},
	"MYEL": {
		ICAO:      "MYEL",
		Name:      "Lee Stocking Airport",
		City:      "Lee Stocking",
		State:     "Exuma",
		Country:   "BS",
		Latitude:  23.7756004333,
		Longitude: -76.1035995483,
		Timezone:  "America/Nassau",
	},
	"MYEM": {
		ICAO:      "MYEM",
		IATA:      "GHB",
		Name:      "Governor's Harbour Airport",
		City:      "Governor's Harbour",
		State:     "North-Eleuthera",
		Country:   "BS",
		Elevation: 26,
		Latitude:  25.2847003937,
		Longitude: -76.3310012817,
		Timezone:  "America/Nassau",
	},
	"MYEN": {
		ICAO:      "MYEN",
		IATA:      "NMC",
		Name:      "Normans Cay Airport",
		State:     "Black-Point",
		Country:   "BS",
		Elevation: 8,
		Latitude:  24.5942993164,
		Longitude: -76.8201980591,
		Timezone:  "America/Nassau",
	},
	"MYER": {
		ICAO:      "MYER",
		IATA:      "RSD",
		Name:      "Rock Sound Airport",
		City:      "Rock Sound",
		State:     "South-Eleuthera",
		Country:   "BS",
		Elevation: 10,
		Latitude:  24.8950787333,
		Longitude: -76.1768817902,
		Timezone:  "America/Nassau",
	},
	"MYES": {
		ICAO:      "MYES",
		IATA:      "TYM",
		Name:      "Staniel Cay Airport",
		State:     "Black-Point",
		Country:   "BS",
		Elevation: 5,
		Latitude:  24.1690998077,
		Longitude: -76.4391021729,
		Timezone:  "America/Nassau",
	},
	"MYEY": {
		ICAO:      "MYEY",
		Name:      "Hog Cay Airport",
		City:      "Hog Cay",
		State:     "Exuma",
		Country:   "BS",
		Latitude:  23.3971996307,
		Longitude: -75.4969024658,
		Timezone:  "America/Nassau",
	},
	"MYGD": {
		ICAO:      "MYGD",
		Name:      "Deep Water Cay Airport",
		City:      "Deep Water Cay",
		State:     "East-Grand-Bahama",
		Country:   "BS",
		Latitude:  26.6317005157,
		Longitude: -77.9216995239,
		Timezone:  "America/Nassau",
	},
	"MYGF": {
		ICAO:      "MYGF",
		IATA:      "FPO",
		Name:      "Grand Bahama International Airport",
		City:      "Freeport",
		State:     "Freeport",
		Country:   "BS",
		Elevation: 7,
		Latitude:  26.5587005615,
		Longitude: -78.695602417,
		Timezone:  "America/Nassau",
	},
	"MYGM": {
		ICAO:      "MYGM",
		IATA:      "GBI",
		Name:      "Auxiliary Airfield",
		City:      "Grand Bahama",
		State:     "East-Grand-Bahama",
		Country:   "BS",
		Elevation: 8,
		Latitude:  26.6319007874,
		Longitude: -78.3591995239,
		Timezone:  "America/Nassau",
	},
	"MYGW": {
		ICAO:      "MYGW",
		IATA:      "WTD",
		Name:      "West End Airport",
		City:      "West End",
		State:     "West-Grand-Bahama",
		Country:   "BS",
		Elevation: 5,
		Latitude:  26.685300827,
		Longitude: -78.9749984741,
		Timezone:  "America/Nassau",
	},
	"MYIG": {
		ICAO:      "MYIG",
		IATA:      "IGA",
		Name:      "Inagua Airport",
		City:      "Matthew Town",
		State:     "Inagua",
		Country:   "BS",
		Elevation: 8,
		Latitude:  20.9750003815,
		Longitude: -73.6669006348,
		Timezone:  "America/Nassau",
	},
	"MYLD": {
		ICAO:      "MYLD",
		IATA:      "LGI",
		Name:      "Deadman's Cay Airport",
		City:      "Deadman's Cay",
		State:     "Long-Island",
		Country:   "BS",
		Elevation: 9,
		Latitude:  23.1790008545,
		Longitude: -75.0935974121,
		Timezone:  "America/Nassau",
	},
	"MYLS": {
		ICAO:      "MYLS",
		IATA:      "SML",
		Name:      "Stella Maris Airport",
		City:      "Stella Maris",
		State:     "Long-Island",
		Country:   "BS",
		Elevation: 10,
		Latitude:  23.5823168043,
		Longitude: -75.2686214447,
		Timezone:  "America/Nassau",
	},
	"MYMM": {
		ICAO:      "MYMM",
		IATA:      "MYG",
		Name:      "Mayaguana Airport",
		City:      "Mayaguana",
		State:     "Mayaguana",
		Country:   "BS",
		Elevation: 11,
		Latitude:  22.3794994354,
		Longitude: -73.0134963989,
		Timezone:  "America/Nassau",
	},
	"MYNN": {
		ICAO:      "MYNN",
		IATA:      "NAS",
		Name:      "Lynden Pindling International Airport",
		City:      "Nassau",
		State:     "New-Providence",
		Country:   "BS",
		Elevation: 16,
		Latitude:  25.0389995575,
		Longitude: -77.4662017822,
		Timezone:  "America/Nassau",
	},
	"MYPI": {
		ICAO:      "MYPI",
		IATA:      "PID",
		Name:      "Nassau Paradise Island Airport",
		City:      "Nassau",
		State:     "New-Providence",
		Country:   "BS",
		Latitude:  25.0830001831,
		Longitude: -77.3000030518,
		Timezone:  "America/Nassau",
	},
	"MYRD": {
		ICAO:      "MYRD",
		IATA:      "DCT",
		Name:      "Duncan Town Airport",
		State:     "Ragged-Island",
		Country:   "BS",
		Elevation: 6,
		Latitude:  22.1818008423,
		Longitude: -75.7294998169,
		Timezone:  "America/Nassau",
	},
	"MYRP": {
		ICAO:      "MYRP",
		IATA:      "RCY",
		Name:      "Rum Cay Airport",
		State:     "Rum-Cay",
		Country:   "BS",
		Elevation: 15,
		Latitude:  23.6844005585,
		Longitude: -74.8361968994,
		Timezone:  "America/Nassau",
	},
	"MYSM": {
		ICAO:      "MYSM",
		IATA:      "ZSA",
		Name:      "San Salvador Airport",
		City:      "San Salvador",
		State:     "San-Salvador",
		Country:   "BS",
		Elevation: 24,
		Latitude:  24.0632991791,
		Longitude: -74.5240020752,
		Timezone:  "America/Nassau",
	},
	"MYX4": {
		ICAO:      "MYX4",
		Name:      "Big Whale Cay Airport",
		City:      "Big Whale Cay",
		State:     "Berry-Islands",
		Country:   "BS",
		Elevation: 16,
		Latitude:  25.3995990753,
		Longitude: -77.7910003662,
		Timezone:  "America/Nassau",
	},
	"MYX5": {
		ICAO:      "MYX5",
		Name:      "Cistern Field",
		City:      "Cistern Field",
		State:     "Berry-Islands",
		Country:   "BS",
		Elevation: 18,
		Latitude:  25.7786998749,
		Longitude: -77.8852005005,
		Timezone:  "America/Nassau",
	},
	"MYX7": {
		ICAO:      "MYX7",
		Name:      "Rudder Cut Cay Airport",
		City:      "Rudder Cut Cay",
		State:     "Black-Point",
		Country:   "BS",
		Elevation: 10,
		Latitude:  23.8864002228,
		Longitude: -76.2537002563,
		Timezone:  "America/Nassau",
	},
	"MYX8": {
		ICAO:      "MYX8",
		Name:      "Darby Island Airport",
		City:      "Darby Island",
		State:     "Black-Point",
		Country:   "BS",
		Elevation: 5,
		Latitude:  23.8502998352,
		Longitude: -76.2282028198,
		Timezone:  "America/Nassau",
	},
	"MYXC": {
		ICAO:      "MYXC",
		Name:      "Hog Key Airport",
		City:      "Long Island",
		State:     "Long-Island",
		Country:   "BS",
		Elevation: 5,
		Latitude:  23.6007003784,
		Longitude: -75.339302063,
		Timezone:  "America/Nassau",
	},
	"MYXD": {
		ICAO:      "MYXD",
		Name:      "Leaf Cay Airport",
		City:      "Leaf Cay",
		State:     "Black-Point",
		Country:   "BS",
		Latitude:  24.1492004395,
		Longitude: -76.4664001465,
		Timezone:  "America/Nassau",
	},
	"MYXH": {
		ICAO:      "MYXH",
		Name:      "Sampson Cay Airport",
		City:      "Sampson Cay",
		State:     "Black-Point",
		Country:   "BS",
		Elevation: 8,
		Latitude:  24.2159996033,
		Longitude: -76.4785003662,
		Timezone:  "America/Nassau",
	},
	"MYZ2": {
		ICAO:      "MYZ2",
		Name:      "Cape Eleuthera Airport",
		City:      "Cape Eleuthera",
		State:     "South-Eleuthera",
		Country:   "BS",
		Elevation: 9,
		Latitude:  24.789899826,
		Longitude: -76.2939987183,
		Timezone:  "America/Nassau",
	},
	"MYZ3": {
		ICAO:      "MYZ3",
		Name:      "Little Whale Cay Berry Islands Airport",
		City:      "Little Whale Cay",
		State:     "Berry-Islands",
		Country:   "BS",
		Elevation: 5,
		Latitude:  25.4417991638,
		Longitude: -77.7617034912,
		Timezone:  "America/Nassau",
	},
	"MZBZ": {
		ICAO:      "MZBZ",
		IATA:      "BZE",
		Name:      "Philip S. W. Goldson International Airport",
		City:      "Belize City",
		State:     "Belize",
		Country:   "BZ",
		Elevation: 15,
		Latitude:  17.539100647,
		Longitude: -88.3081970215,
		Timezone:  "America/Belize",
	},
	"NA01": {
		ICAO:      "NA01",
		Name:      "Jenson Airport",
		City:      "Reynolds",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 867,
		Latitude:  47.6683006287,
		Longitude: -97.0054016113,
		Timezone:  "America/Chicago",
	},
	"NA02": {
		ICAO:      "NA02",
		Name:      "Saure Airport",
		City:      "Reynolds",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 870,
		Latitude:  47.7050018311,
		Longitude: -97.0548019409,
		Timezone:  "America/Chicago",
	},
	"NA03": {
		ICAO:      "NA03",
		Name:      "Boll Brothers Airstrip",
		City:      "Russell",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1453,
		Latitude:  48.6725006104,
		Longitude: -100.858001709,
		Timezone:  "America/Chicago",
	},
	"NA05": {
		ICAO:      "NA05",
		Name:      "Kraig Farms Airport",
		City:      "Sheldon",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1060,
		Latitude:  46.6122016907,
		Longitude: -97.4356002808,
		Timezone:  "America/Chicago",
	},
	"NA06": {
		ICAO:      "NA06",
		Name:      "Bouret Ranch Airport",
		City:      "Sheyenne",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1505,
		Latitude:  47.8628005981,
		Longitude: -98.9726028442,
		Timezone:  "America/Chicago",
	},
	"NA07": {
		ICAO:      "NA07",
		Name:      "Kornkven Airstrip",
		City:      "Souris",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1505,
		Latitude:  48.8863983154,
		Longitude: -100.6949996948,
		Timezone:  "America/Chicago",
	},
	"NA10": {
		ICAO:      "NA10",
		Name:      "Anderson Private Airport",
		City:      "Starkweather",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1460,
		Latitude:  48.3805999756,
		Longitude: -99.0336990356,
		Timezone:  "America/Chicago",
	},
	"NA12": {
		ICAO:      "NA12",
		Name:      "Johnson Private Airport",
		City:      "Turtle Lake",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1920,
		Latitude:  47.4317016602,
		Longitude: -100.8420028687,
		Timezone:  "America/Chicago",
	},
	"NA13": {
		ICAO:      "NA13",
		Name:      "Linrud Airstrip",
		City:      "Velva",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1605,
		Latitude:  48.1222000122,
		Longitude: -100.93699646,
		Timezone:  "America/Chicago",
	},
	"NA17": {
		ICAO:      "NA17",
		Name:      "Jacob Gust Airport",
		City:      "West Fargo",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 895,
		Latitude:  46.9426994324,
		Longitude: -96.9290008545,
		Timezone:  "America/Chicago",
	},
	"NA18": {
		ICAO:      "NA18",
		Name:      "Hought Airstrip",
		City:      "Williston",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 2400,
		Latitude:  48.4133987427,
		Longitude: -103.8359985352,
		Timezone:  "America/Chicago",
	},
	"NA20": {
		ICAO:      "NA20",
		Name:      "Hudson's Strip",
		City:      "Wyndmere",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1080,
		Latitude:  46.2480010986,
		Longitude: -97.2573013306,
		Timezone:  "America/Chicago",
	},
	"NA21": {
		ICAO:      "NA21",
		Name:      "Thompson Private Airport",
		City:      "Wyndmere",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1075,
		Latitude:  46.2282981873,
		Longitude: -97.2361984253,
		Timezone:  "America/Chicago",
	},
	"NA22": {
		ICAO:      "NA22",
		Name:      "Vernon Miller Private Airport",
		City:      "Rhame",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 3102,
		Latitude:  46.0125007629,
		Longitude: -103.7959976196,
		Timezone:  "America/Denver",
	},
	"NA25": {
		ICAO:      "NA25",
		Name:      "Holen Aerial Spray Airstrip",
		City:      "Bantry",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1460,
		Latitude:  48.4986000061,
		Longitude: -100.6529998779,
		Timezone:  "America/Chicago",
	},
	"NA30": {
		ICAO:      "NA30",
		Name:      "Behrens Airstrip",
		City:      "Burlington",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1740,
		Latitude:  48.2722015381,
		Longitude: -101.3960037231,
		Timezone:  "America/Chicago",
	},
	"NA32": {
		ICAO:      "NA32",
		Name:      "Johnson Airstrip",
		City:      "Eldridge",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1548,
		Latitude:  46.898601532,
		Longitude: -98.8587036133,
		Timezone:  "America/Chicago",
	},
	"NA33": {
		ICAO:      "NA33",
		Name:      "Ripplinger Strip",
		City:      "Esmond",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1600,
		Latitude:  48.0010986328,
		Longitude: -99.9260025024,
		Timezone:  "America/Chicago",
	},
	"NA38": {
		ICAO:      "NA38",
		Name:      "Sabbe Brothers Landing Strip",
		City:      "Maddock",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1572,
		Latitude:  47.9505996704,
		Longitude: -99.4296035767,
		Timezone:  "America/Chicago",
	},
	"NA39": {
		ICAO:      "NA39",
		Name:      "Dorbrinski Airport",
		City:      "Makoti",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 2133,
		Latitude:  47.8978004456,
		Longitude: -101.8550033569,
		Timezone:  "America/Chicago",
	},
	"NA44": {
		ICAO:      "NA44",
		Name:      "Georgeson Farm Strip",
		City:      "New Rockford",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1535,
		Latitude:  47.6542015076,
		Longitude: -99.1595993042,
		Timezone:  "America/Chicago",
	},
	"NA49": {
		ICAO:      "NA49",
		Name:      "Paul Airstrip",
		City:      "Rugby",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1530,
		Latitude:  48.4178009033,
		Longitude: -100.0550003052,
		Timezone:  "America/Chicago",
	},
	"NA54": {
		ICAO:      "NA54",
		Name:      "Moser Airstrip",
		City:      "Westfield",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1750,
		Latitude:  45.9550018311,
		Longitude: -100.4420013428,
		Timezone:  "America/Chicago",
	},
	"NA55": {
		ICAO:      "NA55",
		Name:      "Hager Strip",
		City:      "Barney",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1037,
		Latitude:  46.2661018372,
		Longitude: -97.0161972046,
		Timezone:  "America/Chicago",
	},
	"NA56": {
		ICAO:      "NA56",
		Name:      "Moffet Airstrip",
		City:      "Barney",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1028,
		Latitude:  46.3022003174,
		Longitude: -96.9915008545,
		Timezone:  "America/Chicago",
	},
	"NA60": {
		ICAO:      "NA60",
		Name:      "Dakota Airport",
		City:      "Grandin",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 903,
		Latitude:  47.2350006104,
		Longitude: -97.0383987427,
		Timezone:  "America/Chicago",
	},
	"NA61": {
		ICAO:      "NA61",
		Name:      "Tachenko Strip",
		City:      "Grassy Butte",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 2686,
		Latitude:  47.2956008911,
		Longitude: -103.1890029907,
		Timezone:  "America/Denver",
	},
	"NA63": {
		ICAO:      "NA63",
		Name:      "Storseth Airstrip",
		City:      "Grenora",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 2200,
		Latitude:  48.6666984558,
		Longitude: -103.8560028076,
		Timezone:  "America/Chicago",
	},
	"NA65": {
		ICAO:      "NA65",
		Name:      "Anderson Strip",
		City:      "Hoople",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 890,
		Latitude:  48.5313987732,
		Longitude: -97.6280975342,
		Timezone:  "America/Chicago",
	},
	"NA67": {
		ICAO:      "NA67",
		Name:      "Horsley Airstrip",
		City:      "Neche",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 815,
		Latitude:  48.9838981628,
		Longitude: -97.4645004272,
		Timezone:  "America/Chicago",
	},
	"NA70": {
		ICAO:      "NA70",
		Name:      "Smith Airstrip",
		City:      "Sheyenne",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1535,
		Latitude:  47.7738990784,
		Longitude: -98.9884033203,
		Timezone:  "America/Chicago",
	},
	"NA71": {
		ICAO:      "NA71",
		Name:      "M. Bodvig Airstrip",
		City:      "Tappen",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1750,
		Latitude:  46.9958000183,
		Longitude: -99.6712036133,
		Timezone:  "America/Chicago",
	},
	"NA73": {
		ICAO:      "NA73",
		Name:      "Welstad Farms Airstrip",
		City:      "Upham",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1480,
		Latitude:  48.5638999939,
		Longitude: -101,
		Timezone:  "America/Chicago",
	},
	"NA75": {
		ICAO:      "NA75",
		Name:      "Lill Strip",
		City:      "Bremen",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1560,
		Latitude:  47.676399231,
		Longitude: -99.3904037476,
		Timezone:  "America/Chicago",
	},
	"NA77": {
		ICAO:      "NA77",
		Name:      "Mc Gee Strip",
		City:      "Rhame",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 3014,
		Latitude:  45.9866981506,
		Longitude: -103.846000671,
		Timezone:  "America/Denver",
	},
	"NA79": {
		ICAO:      "NA79",
		Name:      "Philbrick Private Airstrip",
		City:      "Turtle Lake",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1915,
		Latitude:  47.4042015076,
		Longitude: -100.8820037842,
		Timezone:  "America/Chicago",
	},
	"NA81": {
		ICAO:      "NA81",
		Name:      "Central Valley Aviation Airport",
		City:      "Buxton",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 920,
		Latitude:  47.6282997131,
		Longitude: -97.0865020752,
		Timezone:  "America/Chicago",
	},
	"NA84": {
		ICAO:      "NA84",
		Name:      "L. Seckerson Airstrip",
		City:      "Eldridge",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1740,
		Latitude:  46.8004989624,
		Longitude: -99.0050964355,
		Timezone:  "America/Chicago",
	},
	"NA86": {
		ICAO:      "NA86",
		Name:      "Johnson Private Airstrip",
		City:      "Luverne",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1492,
		Latitude:  47.3333015442,
		Longitude: -97.9009017944,
		Timezone:  "America/Chicago",
	},
	"NA88": {
		ICAO:      "NA88",
		Name:      "Regan Airstrip",
		City:      "Regan",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 2060,
		Latitude:  47.2238998413,
		Longitude: -100.422996521,
		Timezone:  "America/Chicago",
	},
	"NA89": {
		ICAO:      "NA89",
		Name:      "Holzman Airstrip",
		City:      "Scranton",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 2840,
		Latitude:  46.2221984863,
		Longitude: -103.0210037231,
		Timezone:  "America/Denver",
	},
	"NA90": {
		ICAO:      "NA90",
		Name:      "Circle Z Landing Strip",
		City:      "Underwood",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 2000,
		Latitude:  47.4667015076,
		Longitude: -101.2229995728,
		Timezone:  "America/Chicago",
	},
	"NA92": {
		ICAO:      "NA92",
		Name:      "Roy Lohse Airport",
		City:      "Alamo",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 2124,
		Latitude:  48.5786018372,
		Longitude: -103.466003418,
		Timezone:  "America/Chicago",
	},
	"NA98": {
		ICAO:      "NA98",
		Name:      "Dilse Private Airstrip",
		City:      "Scranton",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 2878,
		Latitude:  46.3025016785,
		Longitude: -103.1699981689,
		Timezone:  "America/Denver",
	},
	"NA99": {
		ICAO:      "NA99",
		Name:      "Bakko Airstrip",
		City:      "Walcott",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1040,
		Latitude:  46.5504989624,
		Longitude: -97.0319976807,
		Timezone:  "America/Chicago",
	},
	"NC00": {
		ICAO:      "NC00",
		Name:      "Moretz Riverside Landing Airport",
		City:      "Sanford",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 326,
		Latitude:  35.5600013733,
		Longitude: -79.1936035156,
		Timezone:  "America/New_York",
	},
	"NC01": {
		ICAO:      "NC01",
		Name:      "Harvey Point Defense Testing Activity Airport",
		City:      "Hertford",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 10,
		Latitude:  36.0961990356,
		Longitude: -76.3271026611,
		Timezone:  "America/New_York",
	},
	"NC02": {
		ICAO:      "NC02",
		Name:      "Boomerang Airport",
		City:      "Harris",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 850,
		Latitude:  35.212600708,
		Longitude: -81.8665008545,
		Timezone:  "America/New_York",
	},
	"NC03": {
		ICAO:      "NC03",
		Name:      "Darr Field",
		City:      "High Point",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 874,
		Latitude:  35.9085006714,
		Longitude: -79.9872970581,
		Timezone:  "America/New_York",
	},
	"NC05": {
		ICAO:      "NC05",
		Name:      "Bradford Field Airport",
		City:      "Huntersville",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 649,
		Latitude:  35.4086990356,
		Longitude: -80.7941970825,
		Timezone:  "America/New_York",
	},
	"NC06": {
		ICAO:      "NC06",
		Name:      "Elk River Airport",
		City:      "Banner Elk",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 3468,
		Latitude:  36.1604003906,
		Longitude: -81.8967971802,
		Timezone:  "America/New_York",
	},
	"NC08": {
		ICAO:      "NC08",
		Name:      "Tusquittee Landing Airport",
		City:      "Hayesville",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 2000,
		Latitude:  35.0853996277,
		Longitude: -83.7362976074,
		Timezone:  "America/New_York",
	},
	"NC09": {
		ICAO:      "NC09",
		Name:      "Stoneriver Airport",
		City:      "Statesville",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 820,
		Latitude:  35.8838996887,
		Longitude: -80.7767028809,
		Timezone:  "America/New_York",
	},
	"NC10": {
		ICAO:      "NC10",
		Name:      "Nocarva Airport",
		City:      "Macon",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 275,
		Latitude:  36.5418014526,
		Longitude: -78.0579986572,
		Timezone:  "America/New_York",
	},
	"NC11": {
		ICAO:      "NC11",
		Name:      "Deck Airpark",
		City:      "Apex",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 338,
		Latitude:  35.7415008545,
		Longitude: -78.924697876,
		Timezone:  "America/New_York",
	},
	"NC12": {
		ICAO:      "NC12",
		Name:      "Lee Creek Airport",
		City:      "Aurora",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 3,
		Latitude:  35.3895988464,
		Longitude: -76.7846984863,
		Timezone:  "America/New_York",
	},
	"NC13": {
		ICAO:      "NC13",
		Name:      "Yorks Field",
		City:      "Ramseur",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 571,
		Latitude:  35.751499176,
		Longitude: -79.6706008911,
		Timezone:  "America/New_York",
	},
	"NC14": {
		ICAO:      "NC14",
		Name:      "Boone Inc Airport",
		City:      "Boone",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 3120,
		Latitude:  36.1968002319,
		Longitude: -81.639503479,
		Timezone:  "America/New_York",
	},
	"NC15": {
		ICAO:      "NC15",
		Name:      "Raeford Rhyne Airpark",
		City:      "Statesville",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 1140,
		Latitude:  35.9570999146,
		Longitude: -80.9789962769,
		Timezone:  "America/New_York",
	},
	"NC16": {
		ICAO:      "NC16",
		Name:      "Brevard Airport",
		City:      "Brevard",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 2125,
		Latitude:  35.2225990295,
		Longitude: -82.7267990112,
		Timezone:  "America/New_York",
	},
	"NC17": {
		ICAO:      "NC17",
		Name:      "Wolf's Den Airport",
		City:      "Cedar Island",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 10,
		Latitude:  34.9802017212,
		Longitude: -76.3079986572,
		Timezone:  "America/New_York",
	},
	"NC18": {
		ICAO:      "NC18",
		Name:      "Brickhouse Field",
		City:      "Burgaw",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 31,
		Latitude:  34.414100647,
		Longitude: -77.9496994019,
		Timezone:  "America/New_York",
	},
	"NC19": {
		ICAO:      "NC19",
		Name:      "Propst Airport",
		City:      "Concord",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 621,
		Latitude:  35.3917999268,
		Longitude: -80.5755996704,
		Timezone:  "America/New_York",
	},
	"NC20": {
		ICAO:      "NC20",
		Name:      "Canaan Air Base",
		City:      "New Bern",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 33,
		Latitude:  35.08727264,
		Longitude: -77.16246796,
		Timezone:  "America/New_York",
	},
	"NC21": {
		ICAO:      "NC21",
		Name:      "Aero Plantation Airport",
		City:      "Weddington",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 634,
		Latitude:  34.9917984009,
		Longitude: -80.7483978271,
		Timezone:  "America/New_York",
	},
	"NC22": {
		ICAO:      "NC22",
		Name:      "Charles Field",
		City:      "Dunn",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 185,
		Latitude:  35.2275009155,
		Longitude: -78.5551986694,
		Timezone:  "America/New_York",
	},
	"NC25": {
		ICAO:      "NC25",
		Name:      "Gold Hill Airport",
		City:      "Gold Hill",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 720,
		Latitude:  35.5089988708,
		Longitude: -80.3112030029,
		Timezone:  "America/New_York",
	},
	"NC26": {
		ICAO:      "NC26",
		Name:      "Long Island Airport",
		City:      "Long Island",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 864,
		Latitude:  35.6609992981,
		Longitude: -80.9705963135,
		Timezone:  "America/New_York",
	},
	"NC27": {
		ICAO:      "NC27",
		Name:      "Lower Creek Airport",
		City:      "Lenoir",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 1195,
		Latitude:  35.9445991516,
		Longitude: -81.4900970459,
		Timezone:  "America/New_York",
	},
	"NC28": {
		ICAO:      "NC28",
		Name:      "Meadstown Airstrip",
		City:      "Elizabeth City",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 7,
		Latitude:  36.2001991272,
		Longitude: -76.2358016968,
		Timezone:  "America/New_York",
	},
	"NC29": {
		ICAO:      "NC29",
		Name:      "Bradley Field",
		City:      "China Grove",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 795,
		Latitude:  35.620098114,
		Longitude: -80.5976028442,
		Timezone:  "America/New_York",
	},
	"NC30": {
		ICAO:      "NC30",
		Name:      "Miller Air Park",
		City:      "Mooresville",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 864,
		Latitude:  35.6139984131,
		Longitude: -80.7415008545,
		Timezone:  "America/New_York",
	},
	"NC31": {
		ICAO:      "NC31",
		Name:      "Quiet Acres Airport",
		City:      "Thomasville",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 740,
		Latitude:  35.7649002075,
		Longitude: -80.0242004395,
		Timezone:  "America/New_York",
	},
	"NC32": {
		ICAO:      "NC32",
		Name:      "Berts Airport",
		City:      "Clemmons",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 690,
		Latitude:  36.0257987976,
		Longitude: -80.4364013672,
		Timezone:  "America/New_York",
	},
	"NC34": {
		ICAO:      "NC34",
		Name:      "Miles Airport",
		City:      "Chapel Hill",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 580,
		Latitude:  35.9704017639,
		Longitude: -79.1247024536,
		Timezone:  "America/New_York",
	},
	"NC35": {
		ICAO:      "NC35",
		Name:      "Spencer Airport",
		City:      "Concord",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 755,
		Latitude:  35.4029006958,
		Longitude: -80.6220016479,
		Timezone:  "America/New_York",
	},
	"NC36": {
		ICAO:      "NC36",
		Name:      "Benton's Airfield",
		City:      "Pikeville",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 145,
		Latitude:  35.5028991699,
		Longitude: -78.0302963257,
		Timezone:  "America/New_York",
	},
	"NC37": {
		ICAO:      "NC37",
		Name:      "Mountain View Aerodrome",
		City:      "Reidsville",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 800,
		Latitude:  36.3400993347,
		Longitude: -79.8411026001,
		Timezone:  "America/New_York",
	},
	"NC39": {
		ICAO:      "NC39",
		Name:      "Enochville Airport",
		City:      "Kannapolis",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 789,
		Latitude:  35.5242996216,
		Longitude: -80.6477966309,
		Timezone:  "America/New_York",
	},
	"NC40": {
		ICAO:      "NC40",
		Name:      "Holeman Field",
		City:      "Roxboro",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 578,
		Latitude:  36.4996986389,
		Longitude: -79.118598938,
		Timezone:  "America/New_York",
	},
	"NC42": {
		ICAO:      "NC42",
		Name:      "Piney Ridge Airport",
		City:      "Yadkinville",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 960,
		Latitude:  36.072101593,
		Longitude: -80.699798584,
		Timezone:  "America/New_York",
	},
	"NC43": {
		ICAO:      "NC43",
		Name:      "Bear Pen Airport",
		City:      "Supply",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 63,
		Latitude:  34.1251983643,
		Longitude: -78.3380966187,
		Timezone:  "America/New_York",
	},
	"NC44": {
		ICAO:      "NC44",
		Name:      "Barclaysville Field",
		City:      "Angier",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 300,
		Latitude:  35.4668998718,
		Longitude: -78.6838989258,
		Timezone:  "America/New_York",
	},
	"NC45": {
		ICAO:      "NC45",
		Name:      "Enfield-Shearin Airport",
		City:      "Enfield",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 125,
		Latitude:  36.2172012329,
		Longitude: -77.6374969482,
		Timezone:  "America/New_York",
	},
	"NC47": {
		ICAO:      "NC47",
		Name:      "South Oak Aerodrome",
		City:      "Winterville",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 50,
		Latitude:  35.48279953,
		Longitude: -77.353302002,
		Timezone:  "America/New_York",
	},
	"NC48": {
		ICAO:      "NC48",
		Name:      "Safe Field",
		City:      "Wallace",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 50,
		Latitude:  34.7501983643,
		Longitude: -78.0599975586,
		Timezone:  "America/New_York",
	},
	"NC49": {
		ICAO:      "NC49",
		Name:      "Boyd Field",
		City:      "Grimesland",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 50,
		Latitude:  35.5278015137,
		Longitude: -77.1967010498,
		Timezone:  "America/New_York",
	},
	"NC50": {
		ICAO:      "NC50",
		Name:      "Skyland Airport",
		City:      "Jefferson",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 2860,
		Latitude:  36.4600982666,
		Longitude: -81.3615036011,
		Timezone:  "America/New_York",
	},
	"NC52": {
		ICAO:      "NC52",
		Name:      "Silver Creek Airport",
		City:      "Morganton",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 1020,
		Latitude:  35.7285003662,
		Longitude: -81.7289962769,
		Timezone:  "America/New_York",
	},
	"NC54": {
		ICAO:      "NC54",
		Name:      "Jackson Private Airport",
		City:      "Atlantic",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 10,
		Latitude:  34.8777008057,
		Longitude: -76.3565979004,
		Timezone:  "America/New_York",
	},
	"NC56": {
		ICAO:      "NC56",
		Name:      "Deer Run Airport",
		City:      "Kittrell",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 405,
		Latitude:  36.20470047,
		Longitude: -78.4689025879,
		Timezone:  "America/New_York",
	},
	"NC58": {
		ICAO:      "NC58",
		Name:      "Gryder-Teague Airport",
		City:      "Taylorsville",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 1190,
		Latitude:  35.9210014343,
		Longitude: -81.1201019287,
		Timezone:  "America/New_York",
	},
	"NC59": {
		ICAO:      "NC59",
		Name:      "Alexander County Airport",
		City:      "Taylorsville",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 1110,
		Latitude:  35.8507003784,
		Longitude: -81.1439971924,
		Timezone:  "America/New_York",
	},
	"NC61": {
		ICAO:      "NC61",
		Name:      "Blackwater Airstrip",
		City:      "Moyock",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 12,
		Latitude:  36.459400177,
		Longitude: -76.2108001709,
		Timezone:  "America/New_York",
	},
	"NC63": {
		ICAO:      "NC63",
		Name:      "Robertson Field",
		City:      "Winston Salem",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 848,
		Latitude:  36.1351013184,
		Longitude: -80.1847991943,
		Timezone:  "America/New_York",
	},
	"NC64": {
		ICAO:      "NC64",
		Name:      "Barringer Field",
		City:      "Kannapolis",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 800,
		Latitude:  35.5278015137,
		Longitude: -80.6967010498,
		Timezone:  "America/New_York",
	},
	"NC65": {
		ICAO:      "NC65",
		Name:      "Yonder Airport",
		City:      "Atkinson",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 35,
		Latitude:  34.5596008301,
		Longitude: -78.2680969238,
		Timezone:  "America/New_York",
	},
	"NC66": {
		ICAO:      "NC66",
		Name:      "Ventosa Plantation Airport",
		City:      "Scotland Neck",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 30,
		Latitude:  36.1313018799,
		Longitude: -77.2957992554,
		Timezone:  "America/New_York",
	},
	"NC67": {
		ICAO:      "NC67",
		Name:      "Six Oaks Airport",
		City:      "Asheville",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 2015,
		Latitude:  35.467778,
		Longitude: -82.4525,
		Timezone:  "America/New_York",
	},
	"NC69": {
		ICAO:      "NC69",
		Name:      "Twin Oak Airport",
		City:      "Clinton",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 125,
		Latitude:  34.8709983826,
		Longitude: -78.4741973877,
		Timezone:  "America/New_York",
	},
	"NC70": {
		ICAO:      "NC70",
		Name:      "Mckee Airport",
		City:      "Fairmont",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 110,
		Latitude:  34.504901886,
		Longitude: -79.1025009155,
		Timezone:  "America/New_York",
	},
	"NC71": {
		ICAO:      "NC71",
		Name:      "E T Field",
		City:      "Parkton",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 187,
		Latitude:  34.9179000854,
		Longitude: -79.0524978638,
		Timezone:  "America/New_York",
	},
	"NC72": {
		ICAO:      "NC72",
		Name:      "Hinton Field",
		City:      "Princeton",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 152,
		Latitude:  35.451499939,
		Longitude: -78.1600036621,
		Timezone:  "America/New_York",
	},
	"NC73": {
		ICAO:      "NC73",
		Name:      "Hodges Farm Airport",
		City:      "Sladesville",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 6,
		Latitude:  35.4388008118,
		Longitude: -76.4494018555,
		Timezone:  "America/New_York",
	},
	"NC75": {
		ICAO:      "NC75",
		Name:      "Schneider Haven Airstrip",
		City:      "Harmony",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 865,
		Latitude:  35.8983001709,
		Longitude: -80.7202987671,
		Timezone:  "America/New_York",
	},
	"NC76": {
		ICAO:      "NC76",
		Name:      "Massengill Airport",
		City:      "Newton Grove",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 190,
		Latitude:  35.3179016113,
		Longitude: -78.387802124,
		Timezone:  "America/New_York",
	},
	"NC77": {
		ICAO:      "NC77",
		Name:      "Chalfant Airport",
		City:      "Concord",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 587,
		Latitude:  35.4556999207,
		Longitude: -80.5755996704,
		Timezone:  "America/New_York",
	},
	"NC78": {
		ICAO:      "NC78",
		Name:      "Turbeville Airport",
		City:      "Tabor City",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 105,
		Latitude:  34.2102012634,
		Longitude: -78.9552993774,
		Timezone:  "America/New_York",
	},
	"NC79": {
		ICAO:      "NC79",
		Name:      "Bear Creek Airport",
		City:      "Mount Pleasant",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 795,
		Latitude:  35.4071006775,
		Longitude: -80.3575973511,
		Timezone:  "America/New_York",
	},
	"NC81": {
		ICAO:      "NC81",
		Name:      "Cox Airport",
		City:      "Apex",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 455,
		Latitude:  35.7518005371,
		Longitude: -78.859703064,
		Timezone:  "America/New_York",
	},
	"NC82": {
		ICAO:      "NC82",
		Name:      "Dublin Field",
		City:      "Dublin",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 126,
		Latitude:  34.6696014404,
		Longitude: -78.7024993896,
		Timezone:  "America/New_York",
	},
	"NC86": {
		ICAO:      "NC86",
		Name:      "Rocking A Farm Airport",
		City:      "White Oak",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 58,
		Latitude:  34.7336006165,
		Longitude: -78.7530975342,
		Timezone:  "America/New_York",
	},
	"NC87": {
		ICAO:      "NC87",
		Name:      "Dogwood Farm Airport",
		City:      "Havelock",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 26,
		Latitude:  34.8818016052,
		Longitude: -76.9427032471,
		Timezone:  "America/New_York",
	},
	"NC93": {
		ICAO:      "NC93",
		Name:      "South River Airport",
		City:      "Salisbury",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 650,
		Latitude:  35.7778015137,
		Longitude: -80.5014038086,
		Timezone:  "America/New_York",
	},
	"NC97": {
		ICAO:      "NC97",
		Name:      "Dirt Dobber's Grass Strip",
		City:      "Shelby",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 930,
		Latitude:  35.3689002991,
		Longitude: -81.5224990845,
		Timezone:  "America/New_York",
	},
	"NC99": {
		ICAO:      "NC99",
		Name:      "Bagwell Airport",
		City:      "Garner",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 300,
		Latitude:  35.6743011475,
		Longitude: -78.6072006226,
		Timezone:  "America/New_York",
	},
	"NCAI": {
		ICAO:      "NCAI",
		IATA:      "AIT",
		Name:      "Aitutaki Airport",
		City:      "Aitutaki",
		Country:   "CK",
		Elevation: 14,
		Latitude:  -18.8309001923,
		Longitude: -159.7640075684,
		Timezone:  "Pacific/Rarotonga",
	},
	"NCAT": {
		ICAO:      "NCAT",
		IATA:      "AIU",
		Name:      "Enua Airport",
		City:      "Atiu Island",
		Country:   "CK",
		Elevation: 36,
		Latitude:  -19.9678001404,
		Longitude: -158.1190032959,
		Timezone:  "Pacific/Rarotonga",
	},
	"NCMG": {
		ICAO:      "NCMG",
		IATA:      "MGS",
		Name:      "Mangaia Island Airport",
		City:      "Mangaia Island",
		Country:   "CK",
		Elevation: 45,
		Latitude:  -21.895986557,
		Longitude: -157.9066619873,
		Timezone:  "Pacific/Rarotonga",
	},
	"NCMH": {
		ICAO:      "NCMH",
		IATA:      "MHX",
		Name:      "Manihiki Island Airport",
		City:      "Manihiki Island",
		Country:   "CK",
		Latitude:  -10.3767004013,
		Longitude: -161.0019989014,
		Timezone:  "Pacific/Rarotonga",
	},
	"NCMK": {
		ICAO:      "NCMK",
		IATA:      "MUK",
		Name:      "Mauke Airport",
		City:      "Mauke Island",
		Country:   "CK",
		Elevation: 26,
		Latitude:  -20.136100769,
		Longitude: -157.3450012207,
		Timezone:  "Pacific/Rarotonga",
	},
	"NCMN": {
		ICAO:      "NCMN",
		Name:      "Manuae Airport",
		City:      "Manuae",
		Country:   "CK",
		Elevation: 15,
		Latitude:  -19.2667007446,
		Longitude: -158.9600067139,
		Timezone:  "Pacific/Rarotonga",
	},
	"NCMR": {
		ICAO:      "NCMR",
		IATA:      "MOI",
		Name:      "Mitiaro Island Airport",
		City:      "Mitiaro Island",
		Country:   "CK",
		Elevation: 25,
		Latitude:  -19.8425006866,
		Longitude: -157.7030029297,
		Timezone:  "Pacific/Rarotonga",
	},
	"NCPY": {
		ICAO:      "NCPY",
		IATA:      "PYE",
		Name:      "Tongareva Airport",
		City:      "Penrhyn Island",
		Country:   "CK",
		Elevation: 8,
		Latitude:  -9.0143699646,
		Longitude: -158.032409668,
		Timezone:  "Pacific/Rarotonga",
	},
	"NCRG": {
		ICAO:      "NCRG",
		IATA:      "RAR",
		Name:      "Rarotonga International Airport",
		City:      "Avarua",
		Country:   "CK",
		Elevation: 19,
		Latitude:  -21.2026996613,
		Longitude: -159.8059997559,
		Timezone:  "Pacific/Rarotonga",
	},
	"ND01": {
		ICAO:      "ND01",
		Name:      "Nelson Airport",
		City:      "Amenia",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 950,
		Latitude:  47.0457992554,
		Longitude: -97.1865005493,
		Timezone:  "America/Chicago",
	},
	"ND02": {
		ICAO:      "ND02",
		Name:      "Ricketyback Field",
		City:      "Aneta",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1510,
		Latitude:  47.8111000061,
		Longitude: -97.9600982666,
		Timezone:  "America/Chicago",
	},
	"ND07": {
		ICAO:      "ND07",
		Name:      "Punton Private Airport",
		City:      "Absaraka",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1105,
		Latitude:  47.0027999878,
		Longitude: -97.4072036743,
		Timezone:  "America/Chicago",
	},
	"ND09": {
		ICAO:      "ND09",
		Name:      "Bryn Airport",
		City:      "Dazey",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1431,
		Latitude:  47.1911010742,
		Longitude: -98.1689987183,
		Timezone:  "America/Chicago",
	},
	"ND10": {
		ICAO:      "ND10",
		Name:      "Flying-N-Ranch Airport",
		City:      "Christine",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 930,
		Latitude:  46.5489006042,
		Longitude: -96.7864990234,
		Timezone:  "America/Chicago",
	},
	"ND14": {
		ICAO:      "ND14",
		Name:      "Fordville Airport",
		City:      "Fordville",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1154,
		Latitude:  48.2346992493,
		Longitude: -97.8018035889,
		Timezone:  "America/Chicago",
	},
	"ND16": {
		ICAO:      "ND16",
		Name:      "Frei Private Airport",
		City:      "Halliday",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 2100,
		Latitude:  47.4416999817,
		Longitude: -102.2419967651,
		Timezone:  "America/Denver",
	},
	"ND18": {
		ICAO:      "ND18",
		Name:      "Judy Strip",
		City:      "Glenburn",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1570,
		Latitude:  48.506401062,
		Longitude: -101.2170028687,
		Timezone:  "America/Chicago",
	},
	"ND20": {
		ICAO:      "ND20",
		Name:      "Gensrich Airport",
		City:      "Hatton",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1021,
		Latitude:  47.7042007446,
		Longitude: -97.3500976563,
		Timezone:  "America/Chicago",
	},
	"ND22": {
		ICAO:      "ND22",
		Name:      "Fischer Private Airport",
		City:      "Garrison",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 2065,
		Latitude:  47.6833000183,
		Longitude: -101.5999984741,
		Timezone:  "America/Chicago",
	},
	"ND23": {
		ICAO:      "ND23",
		Name:      "Wright Field",
		City:      "Williston",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1885,
		Latitude:  48.1880989075,
		Longitude: -103.6200027466,
		Timezone:  "America/Chicago",
	},
	"ND24": {
		ICAO:      "ND24",
		Name:      "Inkster Airport",
		City:      "Inkster",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1092,
		Latitude:  48.1403007507,
		Longitude: -97.6725997925,
		Timezone:  "America/Chicago",
	},
	"ND26": {
		ICAO:      "ND26",
		Name:      "Gajewski Field",
		City:      "Alexander",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 2100,
		Latitude:  47.9000015259,
		Longitude: -103.658996582,
		Timezone:  "America/Chicago",
	},
	"ND29": {
		ICAO:      "ND29",
		Name:      "Swenson Airport",
		City:      "Belfield",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 2900,
		Latitude:  46.6666984558,
		Longitude: -103.2340011597,
		Timezone:  "America/Denver",
	},
	"ND32": {
		ICAO:      "ND32",
		Name:      "Kalainov Private Airport",
		City:      "Steele",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1845,
		Latitude:  46.9155006409,
		Longitude: -99.8945007324,
		Timezone:  "America/Chicago",
	},
	"ND34": {
		ICAO:      "ND34",
		Name:      "Moen Airport",
		City:      "Epping",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 2200,
		Latitude:  48.2750015259,
		Longitude: -103.3499984741,
		Timezone:  "America/Chicago",
	},
	"ND35": {
		ICAO:      "ND35",
		Name:      "Lindemann Airport",
		City:      "Lucca",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1199,
		Latitude:  46.7000007629,
		Longitude: -97.7004013062,
		Timezone:  "America/Chicago",
	},
	"ND36": {
		ICAO:      "ND36",
		Name:      "Don's Airport",
		City:      "St Thomas",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 840,
		Latitude:  48.6380004883,
		Longitude: -97.4848022461,
		Timezone:  "America/Chicago",
	},
	"ND38": {
		ICAO:      "ND38",
		Name:      "Rices Airpark",
		City:      "Maddock",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1575,
		Latitude:  47.9583015442,
		Longitude: -99.5364990234,
		Timezone:  "America/Chicago",
	},
	"ND40": {
		ICAO:      "ND40",
		Name:      "Rau Field",
		City:      "Medina",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1855,
		Latitude:  46.8166999817,
		Longitude: -99.2503967285,
		Timezone:  "America/Chicago",
	},
	"ND41": {
		ICAO:      "ND41",
		Name:      "Voller Airport",
		City:      "Strasburg",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1900,
		Latitude:  46.1347007751,
		Longitude: -100.1129989624,
		Timezone:  "America/Chicago",
	},
	"ND42": {
		ICAO:      "ND42",
		Name:      "Warren Pietsch Airport",
		City:      "Sawyer",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1550,
		Latitude:  48.103099823,
		Longitude: -101.0780029297,
		Timezone:  "America/Chicago",
	},
	"ND43": {
		ICAO:      "ND43",
		Name:      "Flying S Ranch Airport",
		City:      "Minot",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1840,
		Latitude:  48.2000007629,
		Longitude: -101.4039993286,
		Timezone:  "America/Chicago",
	},
	"ND44": {
		ICAO:      "ND44",
		Name:      "Underwood Airport",
		City:      "Underwood",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 2035,
		Latitude:  47.46670151,
		Longitude: -101.1340027,
		Timezone:  "America/Chicago",
	},
	"ND47": {
		ICAO:      "ND47",
		Name:      "Geske Airfield",
		City:      "Enderlin",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1103,
		Latitude:  46.6408004761,
		Longitude: -97.5511016846,
		Timezone:  "America/Chicago",
	},
	"ND49": {
		ICAO:      "ND49",
		Name:      "Krause Private Airport",
		City:      "Wyndmere",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1060,
		Latitude:  46.2235984802,
		Longitude: -97.1304016113,
		Timezone:  "America/Chicago",
	},
	"ND52": {
		ICAO:      "ND52",
		Name:      "True North Airpark",
		City:      "West Fargo",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 913,
		Latitude:  46.8027992249,
		Longitude: -97.0121994019,
		Timezone:  "America/Chicago",
	},
	"ND54": {
		ICAO:      "ND54",
		Name:      "Whitman Field",
		City:      "Robinson",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1783,
		Latitude:  47.1291999817,
		Longitude: -99.7304000854,
		Timezone:  "America/Chicago",
	},
	"ND59": {
		ICAO:      "ND59",
		Name:      "Grieve Airport",
		City:      "Buffalo",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1208,
		Latitude:  46.9874992371,
		Longitude: -97.5295028687,
		Timezone:  "America/Chicago",
	},
	"ND60": {
		ICAO:      "ND60",
		Name:      "Frokjer Airport",
		City:      "Arvilla",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 953,
		Latitude:  47.9589004517,
		Longitude: -97.4797973633,
		Timezone:  "America/Chicago",
	},
	"ND62": {
		ICAO:      "ND62",
		Name:      "Brands Airport",
		City:      "Taylor",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 2423,
		Latitude:  46.8483009338,
		Longitude: -102.4580001831,
		Timezone:  "America/Denver",
	},
	"ND65": {
		ICAO:      "ND65",
		Name:      "Letzring Airport",
		City:      "Center",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 2230,
		Latitude:  46.988899231,
		Longitude: -101.3539962769,
		Timezone:  "America/North_Dakota/Center",
	},
	"ND68": {
		ICAO:      "ND68",
		Name:      "Vining Airport",
		City:      "Wheatland",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 980,
		Latitude:  46.9557991028,
		Longitude: -97.3012008667,
		Timezone:  "America/Chicago",
	},
	"ND72": {
		ICAO:      "ND72",
		Name:      "Lonetree Airstrip",
		City:      "Harvey",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1647,
		Latitude:  47.6843986511,
		Longitude: -100.0849990845,
		Timezone:  "America/Chicago",
	},
	"ND73": {
		ICAO:      "ND73",
		Name:      "Peterson Airport",
		City:      "Amenia",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 999,
		Latitude:  47.0483016968,
		Longitude: -97.2555999756,
		Timezone:  "America/Chicago",
	},
	"ND74": {
		ICAO:      "ND74",
		Name:      "Smith Private Airport",
		City:      "Amenia",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 940,
		Latitude:  47.0166015625,
		Longitude: -97.162902832,
		Timezone:  "America/Chicago",
	},
	"ND75": {
		ICAO:      "ND75",
		Name:      "Pete's Tractor Salvage Airport",
		City:      "Anamoose",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1655,
		Latitude:  47.7200012207,
		Longitude: -100.2689971924,
		Timezone:  "America/Chicago",
	},
	"ND76": {
		ICAO:      "ND76",
		Name:      "Turner Field",
		City:      "Arthur",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 942,
		Latitude:  47.0597000122,
		Longitude: -97.1567993164,
		Timezone:  "America/Chicago",
	},
	"ND78": {
		ICAO:      "ND78",
		Name:      "Wilcox Farm Airport",
		City:      "Ayr",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1210,
		Latitude:  47.0400009155,
		Longitude: -97.5370025635,
		Timezone:  "America/Chicago",
	},
	"ND80": {
		ICAO:      "ND80",
		Name:      "Spitzer Airport",
		City:      "Baldwin",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 2050,
		Latitude:  47.0816993713,
		Longitude: -100.7249984741,
		Timezone:  "America/Chicago",
	},
	"ND85": {
		ICAO:      "ND85",
		Name:      "Humann Private Airstrip",
		City:      "Hazelton",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1980,
		Latitude:  46.4930992126,
		Longitude: -100.1790008545,
		Timezone:  "America/Chicago",
	},
	"ND86": {
		ICAO:      "ND86",
		Name:      "Peterson Airport",
		City:      "Grendra",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 2190,
		Latitude:  48.5638999939,
		Longitude: -103.9629974365,
		Timezone:  "America/Chicago",
	},
	"ND87": {
		ICAO:      "ND87",
		Name:      "Indian Hill Resort Airport",
		City:      "Garrison",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 2010,
		Latitude:  47.6055984497,
		Longitude: -102.1050033569,
		Timezone:  "America/Chicago",
	},
	"ND89": {
		ICAO:      "ND89",
		Name:      "Mutschler Field",
		City:      "Clementsville",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1490,
		Latitude:  47.0558013916,
		Longitude: -98.5070037842,
		Timezone:  "America/Chicago",
	},
	"ND90": {
		ICAO:      "ND90",
		Name:      "Dahl Private Airport",
		City:      "Cogswell/Brampton/",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1285,
		Latitude:  45.9665985107,
		Longitude: -97.7337036133,
		Timezone:  "America/Chicago",
	},
	"ND92": {
		ICAO:      "ND92",
		Name:      "Schroeder Airport",
		City:      "Davenport",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 920,
		Latitude:  46.7083015442,
		Longitude: -97.1169967651,
		Timezone:  "America/Chicago",
	},
	"ND96": {
		ICAO:      "ND96",
		Name:      "Sauter Airport",
		City:      "Bismark",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1730,
		Latitude:  46.801700592,
		Longitude: -100.6790008545,
		Timezone:  "America/Chicago",
	},
	"ND98": {
		ICAO:      "ND98",
		Name:      "Cloud Nine Airport",
		City:      "Lincoln",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1830,
		Latitude:  46.760799408,
		Longitude: -100.6549987793,
		Timezone:  "America/Chicago",
	},
	"ND99": {
		ICAO:      "ND99",
		Name:      "Ellig Field",
		City:      "Hickson",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 910,
		Latitude:  46.720199585,
		Longitude: -96.8095016479,
		Timezone:  "America/Chicago",
	},
	"NE01": {
		ICAO:      "NE01",
		Name:      "Schutte Airport",
		City:      "Guide Rock",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1876,
		Latitude:  40.1946983337,
		Longitude: -98.3519973755,
		Timezone:  "America/Chicago",
	},
	"NE04": {
		ICAO:      "NE04",
		Name:      "Ely Airport",
		City:      "Guide Rock",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1743,
		Latitude:  40.0945014954,
		Longitude: -98.3416976929,
		Timezone:  "America/Chicago",
	},
	"NE06": {
		ICAO:      "NE06",
		Name:      "Woolf Brothers Airport",
		City:      "Lynch",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1395,
		Latitude:  42.8250007629,
		Longitude: -98.4253997803,
		Timezone:  "America/Chicago",
	},
	"NE08": {
		ICAO:      "NE08",
		Name:      "Larrabee Airport",
		City:      "Lamar",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3518,
		Latitude:  40.5477981567,
		Longitude: -101.9779968262,
		Timezone:  "America/Denver",
	},
	"NE09": {
		ICAO:      "NE09",
		Name:      "Simpson Airport",
		City:      "Norfolk",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1650,
		Latitude:  41.9632987976,
		Longitude: -97.467300415,
		Timezone:  "America/Chicago",
	},
	"NE11": {
		ICAO:      "NE11",
		Name:      "Durands Sky Ranch Airport",
		City:      "Omaha",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1285,
		Latitude:  41.3600006104,
		Longitude: -96.0402984619,
		Timezone:  "America/Chicago",
	},
	"NE12": {
		ICAO:      "NE12",
		Name:      "Sullivan Airstrip",
		City:      "Palmer",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1745,
		Latitude:  41.238899231,
		Longitude: -98.2281036377,
		Timezone:  "America/Chicago",
	},
	"NE15": {
		ICAO:      "NE15",
		Name:      "Starns Auxiliary Airport",
		City:      "Ashland",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1080,
		Latitude:  41.0499992371,
		Longitude: -96.3827972412,
		Timezone:  "America/Chicago",
	},
	"NE17": {
		ICAO:      "NE17",
		Name:      "Walts Aerial Service Airport",
		City:      "St. Paul",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1795,
		Latitude:  41.2486000061,
		Longitude: -98.4559020996,
		Timezone:  "America/Chicago",
	},
	"NE18": {
		ICAO:      "NE18",
		Name:      "Krutz Airport",
		City:      "Seward",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1450,
		Latitude:  40.9141998291,
		Longitude: -97.0805969238,
		Timezone:  "America/Chicago",
	},
	"NE19": {
		ICAO:      "NE19",
		Name:      "Ag Air Airport",
		City:      "Broadwater",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3651,
		Latitude:  41.5957984924,
		Longitude: -102.8170013428,
		Timezone:  "America/Denver",
	},
	"NE20": {
		ICAO:      "NE20",
		Name:      "Bernadt Airport",
		City:      "Steinauer",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1290,
		Latitude:  40.1861000061,
		Longitude: -96.2114028931,
		Timezone:  "America/Chicago",
	},
	"NE24": {
		ICAO:      "NE24",
		Name:      "Polaks Sky Ranch Airport",
		City:      "Valparaiso",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1268,
		Latitude:  41.0332984924,
		Longitude: -96.8170013428,
		Timezone:  "America/Chicago",
	},
	"NE25": {
		ICAO:      "NE25",
		Name:      "Orr Field",
		City:      "North Platte",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2860,
		Latitude:  41.1819000244,
		Longitude: -100.8600006104,
		Timezone:  "America/Chicago",
	},
	"NE27": {
		ICAO:      "NE27",
		Name:      "Abie Sky Ranch Airport",
		City:      "Abie",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1440,
		Latitude:  41.3666992188,
		Longitude: -96.950302124,
		Timezone:  "America/Chicago",
	},
	"NE29": {
		ICAO:      "NE29",
		Name:      "Cavanaugh Airport",
		City:      "Minden",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2175,
		Latitude:  40.5139007568,
		Longitude: -99.0120010376,
		Timezone:  "America/Chicago",
	},
	"NE30": {
		ICAO:      "NE30",
		Name:      "Olson Field",
		City:      "Holdrege",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2330,
		Latitude:  40.573600769,
		Longitude: -99.4204025269,
		Timezone:  "America/Chicago",
	},
	"NE31": {
		ICAO:      "NE31",
		Name:      "B.C. Air Airport",
		City:      "Battle Creek",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1600,
		Latitude:  42.0144996643,
		Longitude: -97.5864028931,
		Timezone:  "America/Chicago",
	},
	"NE33": {
		ICAO:      "NE33",
		Name:      "Clearidge Airport",
		City:      "Bennet",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1320,
		Latitude:  40.7097015381,
		Longitude: -96.4766998291,
		Timezone:  "America/Chicago",
	},
	"NE34": {
		ICAO:      "NE34",
		Name:      "Fehringer Aerodrome",
		City:      "Sidney",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 4300,
		Latitude:  41.038898468,
		Longitude: -102.9530029297,
		Timezone:  "America/Denver",
	},
	"NE35": {
		ICAO:      "NE35",
		Name:      "Sutton Airport",
		City:      "Chester",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1583,
		Latitude:  40.0583000183,
		Longitude: -97.5411987305,
		Timezone:  "America/Chicago",
	},
	"NE37": {
		ICAO:      "NE37",
		Name:      "Loseke Airstrip",
		City:      "Columbus",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1490,
		Latitude:  41.4724998474,
		Longitude: -97.2714004517,
		Timezone:  "America/Chicago",
	},
	"NE38": {
		ICAO:      "NE38",
		Name:      "Aknux Airport",
		City:      "Prosser",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2025,
		Latitude:  40.7094993591,
		Longitude: -98.5600967407,
		Timezone:  "America/Chicago",
	},
	"NE39": {
		ICAO:      "NE39",
		Name:      "Warbonnet Ag Strip",
		City:      "Davey",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1325,
		Latitude:  40.9500007629,
		Longitude: -96.6502990723,
		Timezone:  "America/Chicago",
	},
	"NE40": {
		ICAO:      "NE40",
		Name:      "Denton Airfield",
		City:      "Denton",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1270,
		Latitude:  40.715801239,
		Longitude: -96.8677978516,
		Timezone:  "America/Chicago",
	},
	"NE42": {
		ICAO:      "NE42",
		Name:      "Dream Field",
		City:      "Lincoln",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1194,
		Latitude:  40.9333000183,
		Longitude: -96.7005996704,
		Timezone:  "America/Chicago",
	},
	"NE43": {
		ICAO:      "NE43",
		Name:      "Musiel Airport",
		City:      "Morse Bluff",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1230,
		Latitude:  41.4397010803,
		Longitude: -96.7542037964,
		Timezone:  "America/Chicago",
	},
	"NE44": {
		ICAO:      "NE44",
		Name:      "Koinzan Airport",
		City:      "Elgin",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1919,
		Latitude:  41.9832992554,
		Longitude: -98.0837020874,
		Timezone:  "America/Chicago",
	},
	"NE45": {
		ICAO:      "NE45",
		Name:      "Stewart Field",
		City:      "Bennet",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1380,
		Latitude:  40.6944007874,
		Longitude: -96.577796936,
		Timezone:  "America/Chicago",
	},
	"NE49": {
		ICAO:      "NE49",
		Name:      "Koke Airport",
		City:      "Gretna",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1265,
		Latitude:  41.0666999817,
		Longitude: -96.2502975464,
		Timezone:  "America/Chicago",
	},
	"NE50": {
		ICAO:      "NE50",
		Name:      "Sudbeck Field",
		City:      "Hartington",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1480,
		Latitude:  42.6152992249,
		Longitude: -97.4103012085,
		Timezone:  "America/Chicago",
	},
	"NE51": {
		ICAO:      "NE51",
		Name:      "Kohles Airport",
		City:      "Martell",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1320,
		Latitude:  40.6571998596,
		Longitude: -96.7577972412,
		Timezone:  "America/Chicago",
	},
	"NE52": {
		ICAO:      "NE52",
		Name:      "Easton Field",
		City:      "Hebron",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1600,
		Latitude:  40.0778007507,
		Longitude: -97.6149978638,
		Timezone:  "America/Chicago",
	},
	"NE53": {
		ICAO:      "NE53",
		Name:      "Liesveld Airport",
		City:      "Holland",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1385,
		Latitude:  40.5910987854,
		Longitude: -96.5888977051,
		Timezone:  "America/Chicago",
	},
	"NE54": {
		ICAO:      "NE54",
		Name:      "Dostal-Bradley Airport",
		City:      "Howells",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1500,
		Latitude:  41.7083015442,
		Longitude: -96.9794998169,
		Timezone:  "America/Chicago",
	},
	"NE56": {
		ICAO:      "NE56",
		Name:      "Werner Airport",
		City:      "Valley",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1157,
		Latitude:  41.3414001465,
		Longitude: -96.3949966431,
		Timezone:  "America/Chicago",
	},
	"NE57": {
		ICAO:      "NE57",
		Name:      "Thomas Airport",
		City:      "Liberty",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1350,
		Latitude:  40.0922012329,
		Longitude: -96.4781036377,
		Timezone:  "America/Chicago",
	},
	"NE59": {
		ICAO:      "NE59",
		Name:      "Pester Airport",
		City:      "Lincoln",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1150,
		Latitude:  40.8333015442,
		Longitude: -96.5670013428,
		Timezone:  "America/Chicago",
	},
	"NE64": {
		ICAO:      "NE64",
		Name:      "Luetkenhaus Airport",
		City:      "Memphis",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1120,
		Latitude:  41.1053009033,
		Longitude: -96.4095001221,
		Timezone:  "America/Chicago",
	},
	"NE65": {
		ICAO:      "NE65",
		Name:      "Roth Airport",
		City:      "Milford",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1485,
		Latitude:  40.766998291,
		Longitude: -97.077796936,
		Timezone:  "America/Chicago",
	},
	"NE69": {
		ICAO:      "NE69",
		Name:      "Browns Airport",
		City:      "Weeping Water",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1183,
		Latitude:  40.8675003052,
		Longitude: -96.1100006104,
		Timezone:  "America/Chicago",
	},
	"NE73": {
		ICAO:      "NE73",
		Name:      "Stinking Water Creek Airport",
		City:      "Elsie",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3165,
		Latitude:  40.7233009338,
		Longitude: -101.4570007324,
		Timezone:  "America/Denver",
	},
	"NE79": {
		ICAO:      "NE79",
		Name:      "Mueller Field",
		City:      "Roca",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1405,
		Latitude:  40.6528015137,
		Longitude: -96.6211013794,
		Timezone:  "America/Chicago",
	},
	"NE80": {
		ICAO:      "NE80",
		Name:      "Weaver Airport",
		City:      "Roca /Lincoln/",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1330,
		Latitude:  40.676399231,
		Longitude: -96.6391983032,
		Timezone:  "America/Chicago",
	},
	"NE82": {
		ICAO:      "NE82",
		Name:      "Nolte Farms Airport",
		City:      "Murray",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1245,
		Latitude:  40.9324989319,
		Longitude: -95.99949646,
		Timezone:  "America/Chicago",
	},
	"NE85": {
		ICAO:      "NE85",
		Name:      "Glaser Airport",
		City:      "Spalding",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 2035,
		Latitude:  41.6871986389,
		Longitude: -98.3025970459,
		Timezone:  "America/Chicago",
	},
	"NE86": {
		ICAO:      "NE86",
		Name:      "Traudt Airport",
		City:      "Sutton",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1725,
		Latitude:  40.665599823,
		Longitude: -97.8697967529,
		Timezone:  "America/Chicago",
	},
	"NE87": {
		ICAO:      "NE87",
		Name:      "Rempe Private Airport",
		City:      "Superior",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1760,
		Latitude:  40.0971984863,
		Longitude: -98.0647964478,
		Timezone:  "America/Chicago",
	},
	"NE88": {
		ICAO:      "NE88",
		Name:      "Novotny/Tonar Farms Airport",
		City:      "Valparaiso",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1426,
		Latitude:  41.0208015442,
		Longitude: -96.8641967773,
		Timezone:  "America/Chicago",
	},
	"NE89": {
		ICAO:      "NE89",
		Name:      "Herberts Farm Airport",
		City:      "Winnetoon",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1658,
		Latitude:  42.5433006287,
		Longitude: -97.9686965942,
		Timezone:  "America/Chicago",
	},
	"NE92": {
		ICAO:      "NE92",
		Name:      "Thomsen Airport",
		City:      "Wymore",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 1405,
		Latitude:  40.0494003296,
		Longitude: -96.6660995483,
		Timezone:  "America/Chicago",
	},
	"NE99": {
		ICAO:      "NE99",
		Name:      "Hawkins Ranch Airport",
		City:      "Arthur",
		State:     "Nebraska",
		Country:   "US",
		Elevation: 3680,
		Latitude:  41.5750007629,
		Longitude: -101.8180007935,
		Timezone:  "America/Denver",
	},
	"NFCI": {
		ICAO:      "NFCI",
		IATA:      "ICI",
		Name:      "Cicia Airport",
		City:      "Cicia",
		State:     "Eastern",
		Country:   "FJ",
		Elevation: 13,
		Latitude:  -17.7432994843,
		Longitude: -179.341995239,
		Timezone:  "Pacific/Fiji",
	},
	"NFFA": {
		ICAO:      "NFFA",
		IATA:      "BFJ",
		Name:      "Ba Airport",
		State:     "Western",
		Country:   "FJ",
		Elevation: 20,
		Latitude:  -17.5350555556,
		Longitude: 177.680916667,
		Timezone:  "Pacific/Fiji",
	},
	"NFFN": {
		ICAO:      "NFFN",
		IATA:      "NAN",
		Name:      "Nadi International Airport",
		City:      "Nadi",
		State:     "Western",
		Country:   "FJ",
		Elevation: 59,
		Latitude:  -17.755399704,
		Longitude: 177.4429931641,
		Timezone:  "Pacific/Fiji",
	},
	"NFFO": {
		ICAO:      "NFFO",
		IATA:      "PTF",
		Name:      "Malolo Lailai Island Airport",
		City:      "Malolo Lailai Island",
		State:     "Western",
		Country:   "FJ",
		Elevation: 10,
		Latitude:  -17.7779006958,
		Longitude: 177.197006226,
		Timezone:  "Pacific/Fiji",
	},
	"NFKB": {
		ICAO:      "NFKB",
		Name:      "Kaibu Island Airport",
		City:      "Kaibu Island",
		State:     "Eastern",
		Country:   "FJ",
		Latitude:  -17.2541999817,
		Longitude: -179.4889984131,
		Timezone:  "Pacific/Fiji",
	},
	"NFKD": {
		ICAO:      "NFKD",
		IATA:      "KDV",
		Name:      "Vunisea Airport",
		City:      "Vunisea",
		Country:   "FJ",
		Elevation: 6,
		Latitude:  -19.0580997467,
		Longitude: 178.156997681,
		Timezone:  "Pacific/Fiji",
	},
	"NFMA": {
		ICAO:      "NFMA",
		IATA:      "MNF",
		Name:      "Mana Island Airport",
		City:      "Mana Island",
		State:     "Western",
		Country:   "FJ",
		Latitude:  -17.6730995178,
		Longitude: 177.098007202,
		Timezone:  "Pacific/Fiji",
	},
	"NFMO": {
		ICAO:      "NFMO",
		IATA:      "MFJ",
		Name:      "Moala Airport",
		City:      "Moala",
		State:     "Eastern",
		Country:   "FJ",
		Elevation: 13,
		Latitude:  -18.5666999817,
		Longitude: 179.951004028,
		Timezone:  "Pacific/Fiji",
	},
	"NFNA": {
		ICAO:      "NFNA",
		IATA:      "SUV",
		Name:      "Nausori International Airport",
		City:      "Nausori",
		State:     "Central",
		Country:   "FJ",
		Elevation: 17,
		Latitude:  -18.0433006287,
		Longitude: 178.5590057373,
		Timezone:  "Pacific/Fiji",
	},
	"NFNB": {
		ICAO:      "NFNB",
		IATA:      "LEV",
		Name:      "Levuka Airfield",
		City:      "Bureta",
		State:     "Eastern",
		Country:   "FJ",
		Elevation: 11,
		Latitude:  -17.7110996246,
		Longitude: 178.759002686,
		Timezone:  "Pacific/Fiji",
	},
	"NFNG": {
		ICAO:      "NFNG",
		IATA:      "NGI",
		Name:      "Ngau Airport",
		City:      "Ngau",
		State:     "Eastern",
		Country:   "FJ",
		Elevation: 50,
		Latitude:  -18.1156005859,
		Longitude: 179.339996338,
		Timezone:  "Pacific/Fiji",
	},
	"NFNH": {
		ICAO:      "NFNH",
		IATA:      "LUC",
		Name:      "Laucala Island Airport",
		City:      "Laucala Island",
		State:     "Northern",
		Country:   "FJ",
		Elevation: 10,
		Latitude:  -16.7481002808,
		Longitude: -179.6670074463,
		Timezone:  "Pacific/Fiji",
	},
	"NFNK": {
		ICAO:      "NFNK",
		IATA:      "LKB",
		Name:      "Lakeba Island Airport",
		City:      "Lakeba Island",
		State:     "Eastern",
		Country:   "FJ",
		Elevation: 280,
		Latitude:  -18.1991996765,
		Longitude: -178.817001343,
		Timezone:  "Pacific/Fiji",
	},
	"NFNL": {
		ICAO:      "NFNL",
		IATA:      "LBS",
		Name:      "Labasa Airport",
		State:     "Northern",
		Country:   "FJ",
		Elevation: 44,
		Latitude:  -16.4666996002,
		Longitude: 179.3399963379,
		Timezone:  "Pacific/Fiji",
	},
	"NFNM": {
		ICAO:      "NFNM",
		IATA:      "TVU",
		Name:      "Matei Airport",
		City:      "Matei",
		State:     "Northern",
		Country:   "FJ",
		Elevation: 60,
		Latitude:  -16.6905994415,
		Longitude: -179.876998901,
		Timezone:  "Pacific/Fiji",
	},
	"NFNO": {
		ICAO:      "NFNO",
		IATA:      "KXF",
		Name:      "Koro Island Airport",
		City:      "Koro Island",
		State:     "Eastern",
		Country:   "FJ",
		Elevation: 358,
		Latitude:  -17.3458003998,
		Longitude: 179.42199707,
		Timezone:  "Pacific/Fiji",
	},
	"NFNR": {
		ICAO:      "NFNR",
		IATA:      "RTA",
		Name:      "Rotuma Airport",
		City:      "Rotuma",
		State:     "Rotuma",
		Country:   "FJ",
		Elevation: 22,
		Latitude:  -12.4825000763,
		Longitude: 177.0709991455,
		Timezone:  "Pacific/Fiji",
	},
	"NFNS": {
		ICAO:      "NFNS",
		IATA:      "SVU",
		Name:      "Savusavu Airport",
		City:      "Savusavu",
		State:     "Northern",
		Country:   "FJ",
		Elevation: 17,
		Latitude:  -16.8027992249,
		Longitude: 179.341003418,
		Timezone:  "Pacific/Fiji",
	},
	"NFNV": {
		ICAO:      "NFNV",
		IATA:      "VAU",
		Name:      "Vatukoula Airport",
		City:      "Vatukoula",
		State:     "Western",
		Country:   "FJ",
		Elevation: 156,
		Latitude:  -17.5,
		Longitude: 177.8419952393,
		Timezone:  "Pacific/Fiji",
	},
	"NFNW": {
		ICAO:      "NFNW",
		IATA:      "KAY",
		Name:      "Wakaya Island Airport",
		City:      "Wakaya Island",
		Country:   "FJ",
		Elevation: 130,
		Latitude:  -17.6170005798,
		Longitude: 179.016998291,
		Timezone:  "Pacific/Fiji",
	},
	"NFOL": {
		ICAO:      "NFOL",
		IATA:      "ONU",
		Name:      "Ono-I-Lau Airport",
		City:      "Ono-I-Lau",
		Country:   "FJ",
		Elevation: 110,
		Latitude:  -20.6499996185,
		Longitude: -178.6999969482,
		Timezone:  "Pacific/Fiji",
	},
	"NFSW": {
		ICAO:      "NFSW",
		IATA:      "YAS",
		Name:      "Yasawa Island Airport",
		City:      "Yasawa Island",
		State:     "Western",
		Country:   "FJ",
		Elevation: 37,
		Latitude:  -16.7588996887,
		Longitude: 177.5449981689,
		Timezone:  "Pacific/Fiji",
	},
	"NFTE": {
		ICAO:      "NFTE",
		IATA:      "EUA",
		Name:      "Kaufana Airport",
		City:      "Eua Island",
		State:     "ʻEua",
		Country:   "TO",
		Elevation: 325,
		Latitude:  -21.3782997131,
		Longitude: -174.957992554,
		Timezone:  "Pacific/Tongatapu",
	},
	"NFTF": {
		ICAO:      "NFTF",
		IATA:      "TBU",
		Name:      "Fua'amotu International Airport",
		City:      "Nuku'alofa",
		State:     "Tongatapu",
		Country:   "TO",
		Elevation: 126,
		Latitude:  -21.2411994934,
		Longitude: -175.1499938965,
		Timezone:  "Pacific/Tongatapu",
	},
	"NFTL": {
		ICAO:      "NFTL",
		IATA:      "HPA",
		Name:      "Lifuka Island Airport",
		City:      "Lifuka",
		State:     "Ha‘apai",
		Country:   "TO",
		Elevation: 31,
		Latitude:  -19.7770004272,
		Longitude: -174.341003418,
		Timezone:  "Pacific/Tongatapu",
	},
	"NFTO": {
		ICAO:      "NFTO",
		IATA:      "Niu",
		Name:      "Mata'aho Airport",
		City:      "Angaha",
		State:     "Vava‘u",
		Country:   "TO",
		Elevation: 160,
		Latitude:  -15.5707998276,
		Longitude: -175.632995605,
		Timezone:  "Pacific/Tongatapu",
	},
	"NFTP": {
		ICAO:      "NFTP",
		IATA:      "NTT",
		Name:      "Kuini Lavenia Airport",
		City:      "Niuatoputapu",
		State:     "Niuas",
		Country:   "TO",
		Elevation: 30,
		Latitude:  -15.9773376519,
		Longitude: -173.791029453,
		Timezone:  "Pacific/Tongatapu",
	},
	"NFTV": {
		ICAO:      "NFTV",
		IATA:      "VAV",
		Name:      "Vava'u International Airport",
		City:      "Vava'u Island",
		State:     "Vava‘u",
		Country:   "TO",
		Elevation: 236,
		Latitude:  -18.5853004456,
		Longitude: -173.9620056152,
		Timezone:  "Pacific/Tongatapu",
	},
	"NFVB": {
		ICAO:      "NFVB",
		IATA:      "VBV",
		Name:      "Vanua Balavu Airport",
		City:      "Vanua Balavu",
		State:     "Eastern",
		Country:   "FJ",
		Elevation: 76,
		Latitude:  -17.2689990997,
		Longitude: -178.9759979248,
		Timezone:  "Pacific/Fiji",
	},
	"NFVL": {
		ICAO:      "NFVL",
		IATA:      "VTF",
		Name:      "Vatulele Airport",
		City:      "Vatulele",
		State:     "Western",
		Country:   "FJ",
		Elevation: 36,
		Latitude:  -18.5125007629,
		Longitude: 177.6390075684,
		Timezone:  "Pacific/Fiji",
	},
	"NGAB": {
		ICAO:      "NGAB",
		IATA:      "ABF",
		Name:      "Abaiang Airport",
		City:      "Abaiang",
		State:     "Gilbert-Islands",
		Country:   "KI",
		Latitude:  1.798609972,
		Longitude: 173.0410003662,
		Timezone:  "Pacific/Tarawa",
	},
	"NGBR": {
		ICAO:      "NGBR",
		IATA:      "BEZ",
		Name:      "Beru Airport",
		City:      "Beru",
		State:     "Gilbert-Islands",
		Country:   "KI",
		Elevation: 6,
		Latitude:  -1.3547199965,
		Longitude: 176.0070037842,
		Timezone:  "Pacific/Tarawa",
	},
	"NGFU": {
		ICAO:      "NGFU",
		IATA:      "FUN",
		Name:      "Funafuti International Airport",
		City:      "Funafuti",
		State:     "Funafuti",
		Country:   "TV",
		Latitude:  -8.525,
		Longitude: 179.195999,
		Timezone:  "Pacific/Funafuti",
	},
	"NGKT": {
		ICAO:      "NGKT",
		IATA:      "KUC",
		Name:      "Kuria Airport",
		City:      "Kuria",
		State:     "Gilbert-Islands",
		Country:   "KI",
		Latitude:  0.218611002,
		Longitude: 173.4420013428,
		Timezone:  "Pacific/Tarawa",
	},
	"NGMA": {
		ICAO:      "NGMA",
		IATA:      "MNK",
		Name:      "Maiana Airport",
		City:      "Maiana",
		State:     "Gilbert-Islands",
		Country:   "KI",
		Elevation: 8,
		Latitude:  1.0036100149,
		Longitude: 173.0310058594,
		Timezone:  "Pacific/Tarawa",
	},
	"NGMK": {
		ICAO:      "NGMK",
		IATA:      "MZK",
		Name:      "Marakei Airport",
		City:      "Marakei",
		State:     "Gilbert-Islands",
		Country:   "KI",
		Elevation: 10,
		Latitude:  2.0586099625,
		Longitude: 173.2709960938,
		Timezone:  "Pacific/Tarawa",
	},
	"NGMN": {
		ICAO:      "NGMN",
		IATA:      "MTK",
		Name:      "Makin Island Airport",
		City:      "Makin Island",
		State:     "Gilbert-Islands",
		Country:   "KI",
		Latitude:  3.3744399548,
		Longitude: 172.9920043945,
		Timezone:  "Pacific/Tarawa",
	},
	"NGNU": {
		ICAO:      "NGNU",
		IATA:      "NIG",
		Name:      "Nikunau Airport",
		City:      "Nikunau",
		State:     "Gilbert-Islands",
		Country:   "KI",
		Elevation: 6,
		Latitude:  -1.314440012,
		Longitude: 176.410003662,
		Timezone:  "Pacific/Tarawa",
	},
	"NGON": {
		ICAO:      "NGON",
		IATA:      "OOT",
		Name:      "Onotoa Airport",
		City:      "Onotoa",
		State:     "Gilbert-Islands",
		Country:   "KI",
		Elevation: 7,
		Latitude:  -1.796110034,
		Longitude: 175.5260009766,
		Timezone:  "Pacific/Tarawa",
	},
	"NGTA": {
		ICAO:      "NGTA",
		IATA:      "TRW",
		Name:      "Bonriki International Airport",
		City:      "Tarawa",
		State:     "Gilbert-Islands",
		Country:   "KI",
		Elevation: 9,
		Latitude:  1.3816399574,
		Longitude: 173.1470031738,
		Timezone:  "Pacific/Tarawa",
	},
	"NGTB": {
		ICAO:      "NGTB",
		IATA:      "AEA",
		Name:      "Abemama Atoll Airport",
		City:      "Abemama Atoll",
		State:     "Gilbert-Islands",
		Country:   "KI",
		Elevation: 8,
		Latitude:  0.4908330142,
		Longitude: 173.828994751,
		Timezone:  "Pacific/Tarawa",
	},
	"NGTE": {
		ICAO:      "NGTE",
		IATA:      "TBF",
		Name:      "Tabiteuea North Airport",
		State:     "Gilbert-Islands",
		Country:   "KI",
		Elevation: 7,
		Latitude:  -1.2244700193,
		Longitude: 174.7760009766,
		Timezone:  "Pacific/Tarawa",
	},
	"NGTM": {
		ICAO:      "NGTM",
		IATA:      "TMN",
		Name:      "Tamana Island Airport",
		City:      "Tamana Island",
		State:     "Gilbert-Islands",
		Country:   "KI",
		Latitude:  -2.4858300686,
		Longitude: 175.9700012207,
		Timezone:  "Pacific/Tarawa",
	},
	"NGTO": {
		ICAO:      "NGTO",
		IATA:      "NON",
		Name:      "Nonouti Airport",
		City:      "Nonouti",
		State:     "Gilbert-Islands",
		Country:   "KI",
		Elevation: 10,
		Latitude:  -0.6397219896,
		Longitude: 174.4279937744,
		Timezone:  "Pacific/Tarawa",
	},
	"NGTR": {
		ICAO:      "NGTR",
		IATA:      "AIS",
		Name:      "Arorae Island Airport",
		City:      "Arorae Island",
		State:     "Gilbert-Islands",
		Country:   "KI",
		Latitude:  -2.6161100864,
		Longitude: 176.8029937744,
		Timezone:  "Pacific/Tarawa",
	},
	"NGTS": {
		ICAO:      "NGTS",
		IATA:      "TSU",
		Name:      "Tabiteuea South Airport",
		City:      "Tabiteuea South",
		State:     "Gilbert-Islands",
		Country:   "KI",
		Latitude:  -1.4744399786,
		Longitude: 175.0639953613,
		Timezone:  "Pacific/Tarawa",
	},
	"NGTU": {
		ICAO:      "NGTU",
		IATA:      "BBG",
		Name:      "Butaritari Atoll Airport",
		City:      "Butaritari Atoll",
		State:     "Gilbert-Islands",
		Country:   "KI",
		Elevation: 5,
		Latitude:  3.0858299732,
		Longitude: 172.8110046387,
		Timezone:  "Pacific/Tarawa",
	},
	"NGUK": {
		ICAO:      "NGUK",
		IATA:      "AAK",
		Name:      "Buariki Airport",
		City:      "Buariki",
		State:     "Gilbert-Islands",
		Country:   "KI",
		Latitude:  0.1852779984,
		Longitude: 173.6369934082,
		Timezone:  "Pacific/Tarawa",
	},
	"NH07": {
		ICAO:      "NH07",
		Name:      "Cooper Farm Airport",
		City:      "Loudon",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 485,
		Latitude:  43.2803993225,
		Longitude: -71.4565963745,
		Timezone:  "America/New_York",
	},
	"NH15": {
		ICAO:      "NH15",
		Name:      "Locke Lake Airport",
		City:      "Barnstead",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 702,
		Latitude:  43.3866996765,
		Longitude: -71.2333984375,
		Timezone:  "America/New_York",
	},
	"NH16": {
		ICAO:      "NH16",
		Name:      "Brookline Airport",
		City:      "Brookline",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 415,
		Latitude:  42.7417984009,
		Longitude: -71.7076034546,
		Timezone:  "America/New_York",
	},
	"NH17": {
		ICAO:      "NH17",
		Name:      "Zim Airport",
		City:      "Mason",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 520,
		Latitude:  42.7276000977,
		Longitude: -71.7175979614,
		Timezone:  "America/New_York",
	},
	"NH18": {
		ICAO:      "NH18",
		Name:      "Chickville Airport",
		City:      "Center Ossipee",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 560,
		Latitude:  43.73339844,
		Longitude: -71.15119934,
		Timezone:  "America/New_York",
	},
	"NH20": {
		ICAO:      "NH20",
		Name:      "Ward Field",
		City:      "Sanbornton",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 440,
		Latitude:  43.4922981262,
		Longitude: -71.6458969116,
		Timezone:  "America/New_York",
	},
	"NH25": {
		ICAO:      "NH25",
		Name:      "Flying H Skyport Airport",
		City:      "Melvin Village",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 620,
		Latitude:  43.6866989136,
		Longitude: -71.2891998291,
		Timezone:  "America/New_York",
	},
	"NH31": {
		ICAO:      "NH31",
		Name:      "Mountain View Field",
		City:      "Wolfeboro",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 965,
		Latitude:  43.650100708,
		Longitude: -71.1494979858,
		Timezone:  "America/New_York",
	},
	"NH38": {
		ICAO:      "NH38",
		Name:      "Leavitt Airport",
		City:      "Albany",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 860,
		Latitude:  43.9725990295,
		Longitude: -71.1697998047,
		Timezone:  "America/New_York",
	},
	"NH39": {
		ICAO:      "NH39",
		Name:      "Frank D. Comerford Airport",
		City:      "Walpole",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 640,
		Latitude:  43.0334014893,
		Longitude: -72.4298019409,
		Timezone:  "America/New_York",
	},
	"NH40": {
		ICAO:      "NH40",
		Name:      "Eagles Nest Airport",
		City:      "New London",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 998,
		Latitude:  43.4047012329,
		Longitude: -71.9504013062,
		Timezone:  "America/New_York",
	},
	"NH43": {
		ICAO:      "NH43",
		Name:      "Murphy-Sherwood Park Airport",
		City:      "Nottingham",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 200,
		Latitude:  43.1795005798,
		Longitude: -71.1370010376,
		Timezone:  "America/New_York",
	},
	"NH49": {
		ICAO:      "NH49",
		Name:      "Bradley Field",
		City:      "Woodstock",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 650,
		Latitude:  43.9702987671,
		Longitude: -71.6794967651,
		Timezone:  "America/New_York",
	},
	"NH60": {
		ICAO:      "NH60",
		Name:      "Huff Memorial Airport",
		City:      "New Boston",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 840,
		Latitude:  42.9606018066,
		Longitude: -71.6533966064,
		Timezone:  "America/New_York",
	},
	"NH61": {
		ICAO:      "NH61",
		Name:      "Heaton Airport",
		City:      "Chester",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 188,
		Latitude:  42.9681015015,
		Longitude: -71.1811981201,
		Timezone:  "America/New_York",
	},
	"NH67": {
		ICAO:      "NH67",
		Name:      "Winterwood Airport",
		City:      "Brentwood",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 130,
		Latitude:  42.9784011841,
		Longitude: -71.0094985962,
		Timezone:  "America/New_York",
	},
	"NH69": {
		ICAO:      "NH69",
		Name:      "Windsock Village Airport",
		City:      "West Ossipee",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 460,
		Latitude:  43.8306007385,
		Longitude: -71.18699646,
		Timezone:  "America/New_York",
	},
	"NH76": {
		ICAO:      "NH76",
		Name:      "Mason Airfield",
		City:      "Mason",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 740,
		Latitude:  42.7290000916,
		Longitude: -71.7842025757,
		Timezone:  "America/New_York",
	},
	"NH78": {
		ICAO:      "NH78",
		Name:      "Steck Farm Airport",
		City:      "Pelham",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 460,
		Latitude:  42.7472000122,
		Longitude: -71.3692016602,
		Timezone:  "America/New_York",
	},
	"NH84": {
		ICAO:      "NH84",
		Name:      "Northwood Airport",
		City:      "Northwood",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 640,
		Latitude:  43.2075996399,
		Longitude: -71.2211990356,
		Timezone:  "America/New_York",
	},
	"NH86": {
		ICAO:      "NH86",
		Name:      "Intervale Airport",
		City:      "Henniker",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 425,
		Latitude:  43.1750984192,
		Longitude: -71.7873001099,
		Timezone:  "America/New_York",
	},
	"NH88": {
		ICAO:      "NH88",
		Name:      "Country Club Air Park",
		City:      "Goffstown",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 780,
		Latitude:  43.0672988892,
		Longitude: -71.6111984253,
		Timezone:  "America/New_York",
	},
	"NH96": {
		ICAO:      "NH96",
		Name:      "Wentworth Aerodrome Airport",
		City:      "Wentworth",
		State:     "New-Hampshire",
		Country:   "US",
		Elevation: 610,
		Latitude:  43.873444,
		Longitude: -71.904722,
		Timezone:  "America/New_York",
	},
	"NIUE": {
		ICAO:      "NIUE",
		IATA:      "IUE",
		Name:      "Niue International Airport",
		City:      "Alofi",
		Country:   "NU",
		Elevation: 209,
		Latitude:  -19.0790309906,
		Longitude: -169.9255981445,
		Timezone:  "Pacific/Niue",
	},
	"NJ02": {
		ICAO:      "NJ02",
		Name:      "Alloway Airfield",
		City:      "Alloway",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 140,
		Latitude:  39.5418014526,
		Longitude: -75.304397583,
		Timezone:  "America/New_York",
	},
	"NJ06": {
		ICAO:      "NJ06",
		Name:      "B J Farms Airport",
		City:      "Shiloh",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 100,
		Latitude:  39.4575996399,
		Longitude: -75.2732009888,
		Timezone:  "America/New_York",
	},
	"NJ09": {
		ICAO:      "NJ09",
		Name:      "Matthews Airport",
		City:      "Belvidere",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 750,
		Latitude:  40.7918014526,
		Longitude: -75.0703964233,
		Timezone:  "America/New_York",
	},
	"NJ16": {
		ICAO:      "NJ16",
		Name:      "Sliker Strip",
		City:      "Califon",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 965,
		Latitude:  40.749066667,
		Longitude: -74.862786111,
		Timezone:  "America/New_York",
	},
	"NJ20": {
		ICAO:      "NJ20",
		Name:      "Coyle Field",
		City:      "Chatsworth",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 190,
		Latitude:  39.8125991821,
		Longitude: -74.4245986938,
		Timezone:  "America/New_York",
	},
	"NJ24": {
		ICAO:      "NJ24",
		Name:      "Warren Grove Range Airport",
		City:      "Burlington County",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 105,
		Latitude:  39.6967010498,
		Longitude: -74.4000015259,
		Timezone:  "America/New_York",
	},
	"NJ25": {
		ICAO:      "NJ25",
		Name:      "Peaslees Airstrip",
		City:      "Clarksboro",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 99,
		Latitude:  39.7892990112,
		Longitude: -75.2291030884,
		Timezone:  "America/New_York",
	},
	"NJ30": {
		ICAO:      "NJ30",
		Name:      "Paruszewski Farm Strip",
		City:      "Salem",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 12,
		Latitude:  39.5307998657,
		Longitude: -75.4772033691,
		Timezone:  "America/New_York",
	},
	"NJ46": {
		ICAO:      "NJ46",
		Name:      "Mar Bar L Farms Airport",
		City:      "Englishtown",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 80,
		Latitude:  40.2765007019,
		Longitude: -74.3884963989,
		Timezone:  "America/New_York",
	},
	"NJ47": {
		ICAO:      "NJ47",
		Name:      "Teeny Weeny Acres Airport",
		City:      "Flemington",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 565,
		Latitude:  40.5167999268,
		Longitude: -74.8828964233,
		Timezone:  "America/New_York",
	},
	"NJ49": {
		ICAO:      "NJ49",
		Name:      "Bradford Field",
		City:      "Flemington",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 530,
		Latitude:  40.5007019043,
		Longitude: -74.9560012817,
		Timezone:  "America/New_York",
	},
	"NJ50": {
		ICAO:      "NJ50",
		Name:      "The Landing Airport",
		City:      "Flemington",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 470,
		Latitude:  40.4807014465,
		Longitude: -74.9617996216,
		Timezone:  "America/New_York",
	},
	"NJ52": {
		ICAO:      "NJ52",
		Name:      "Folsom Airport",
		City:      "Folsom",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 90,
		Latitude:  39.5918006897,
		Longitude: -74.8246002197,
		Timezone:  "America/New_York",
	},
	"NJ58": {
		ICAO:      "NJ58",
		Name:      "Nordheim Flying K Airpark",
		City:      "Bargaintown",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 53,
		Latitude:  39.3926010132,
		Longitude: -74.606300354,
		Timezone:  "America/New_York",
	},
	"NJ59": {
		ICAO:      "NJ59",
		Name:      "Ekdahl Airport",
		City:      "Freehold",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 118,
		Latitude:  40.1828994751,
		Longitude: -74.2788009644,
		Timezone:  "America/New_York",
	},
	"NJ60": {
		ICAO:      "NJ60",
		Name:      "Cuddihy Landing Strip",
		City:      "Freehold",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 105,
		Latitude:  40.1870994568,
		Longitude: -74.2670974731,
		Timezone:  "America/New_York",
	},
	"NJ61": {
		ICAO:      "NJ61",
		Name:      "Malone Airport",
		City:      "Frenchtown",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 480,
		Latitude:  40.5500984192,
		Longitude: -75.0162963867,
		Timezone:  "America/New_York",
	},
	"NJ63": {
		ICAO:      "NJ63",
		Name:      "Eagles Lair Airport",
		City:      "Frenchtown",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 460,
		Latitude:  40.4678993225,
		Longitude: -75.0503997803,
		Timezone:  "America/New_York",
	},
	"NJ64": {
		ICAO:      "NJ64",
		Name:      "Reeder Airport",
		City:      "Glassboro",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 145,
		Latitude:  39.7056999207,
		Longitude: -75.0802001953,
		Timezone:  "America/New_York",
	},
	"NJ65": {
		ICAO:      "NJ65",
		Name:      "John E. Rogers Airport",
		City:      "Great Meadows",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 500,
		Latitude:  40.8534011841,
		Longitude: -74.9296035767,
		Timezone:  "America/New_York",
	},
	"NJ69": {
		ICAO:      "NJ69",
		Name:      "Ideal Mfg Corp Airport",
		City:      "Hammonton",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 80,
		Latitude:  39.6585006714,
		Longitude: -74.8023986816,
		Timezone:  "America/New_York",
	},
	"NJ72": {
		ICAO:      "NJ72",
		Name:      "Hop Brook Farm Airport",
		City:      "Holmdel",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 140,
		Latitude:  40.3512001038,
		Longitude: -74.1812973022,
		Timezone:  "America/New_York",
	},
	"NJ74": {
		ICAO:      "NJ74",
		Name:      "Salem Airfield",
		City:      "Salem",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 15,
		Latitude:  39.5625991821,
		Longitude: -75.4496002197,
		Timezone:  "America/New_York",
	},
	"NJ75": {
		ICAO:      "NJ75",
		Name:      "Werner Private Airport",
		City:      "Jamesburg",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 50,
		Latitude:  40.3501014709,
		Longitude: -74.3663024902,
		Timezone:  "America/New_York",
	},
	"NJ79": {
		ICAO:      "NJ79",
		Name:      "Goat Hill Airport",
		City:      "Lambertville",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 350,
		Latitude:  40.3515014648,
		Longitude: -74.9093017578,
		Timezone:  "America/New_York",
	},
	"NJ81": {
		ICAO:      "NJ81",
		Name:      "Free Spirit Airport",
		City:      "Vineland",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 85,
		Latitude:  39.4422988892,
		Longitude: -74.9427032471,
		Timezone:  "America/New_York",
	},
	"NJ84": {
		ICAO:      "NJ84",
		Name:      "Hidden Acres Farm Airport",
		City:      "Bridgeton",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 60,
		Latitude:  39.4331016541,
		Longitude: -75.396697998,
		Timezone:  "America/New_York",
	},
	"NJ95": {
		ICAO:      "NJ95",
		Name:      "Herr Brothers Airport",
		City:      "Milford",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 330,
		Latitude:  40.5556983948,
		Longitude: -75.0724029541,
		Timezone:  "America/New_York",
	},
	"NJ96": {
		ICAO:      "NJ96",
		Name:      "Als Landing Strip",
		City:      "Minotola",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 115,
		Latitude:  39.5261993408,
		Longitude: -74.9565963745,
		Timezone:  "America/New_York",
	},
	"NJ98": {
		ICAO:      "NJ98",
		Name:      "Zitone Airport",
		City:      "Montague",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 587,
		Latitude:  41.3167991638,
		Longitude: -74.7496032715,
		Timezone:  "America/New_York",
	},
	"NK01": {
		ICAO:      "NK01",
		Name:      "Countryman's Landing Strip",
		City:      "Carthage",
		State:     "New-York",
		Country:   "US",
		Elevation: 790,
		Latitude:  43.9995002747,
		Longitude: -75.6048965454,
		Timezone:  "America/New_York",
	},
	"NK04": {
		ICAO:      "NK04",
		Name:      "Shepard Airport",
		City:      "Constantia",
		State:     "New-York",
		Country:   "US",
		Elevation: 463,
		Latitude:  43.2500991821,
		Longitude: -75.9996032715,
		Timezone:  "America/New_York",
	},
	"NK05": {
		ICAO:      "NK05",
		Name:      "Tom N' Jerry Airport",
		City:      "Danby",
		State:     "New-York",
		Country:   "US",
		Elevation: 1490,
		Latitude:  42.354801178,
		Longitude: -76.5160980225,
		Timezone:  "America/New_York",
	},
	"NK06": {
		ICAO:      "NK06",
		Name:      "S.O.P. Airport",
		City:      "Germantown",
		State:     "New-York",
		Country:   "US",
		Elevation: 190,
		Latitude:  42.0961990356,
		Longitude: -73.8554000854,
		Timezone:  "America/New_York",
	},
	"NK07": {
		ICAO:      "NK07",
		Name:      "Big Island Airport",
		City:      "Goshen",
		State:     "New-York",
		Country:   "US",
		Elevation: 410,
		Latitude:  41.3278999329,
		Longitude: -74.4098968506,
		Timezone:  "America/New_York",
	},
	"NK08": {
		ICAO:      "NK08",
		Name:      "Sheeley's Farm Airport",
		City:      "High Falls",
		State:     "New-York",
		Country:   "US",
		Elevation: 260,
		Latitude:  41.8317985535,
		Longitude: -74.1298980713,
		Timezone:  "America/New_York",
	},
	"NK09": {
		ICAO:      "NK09",
		Name:      "Eagle Ridge Airport",
		City:      "Burdett",
		State:     "New-York",
		Country:   "US",
		Elevation: 1480,
		Latitude:  42.4056015015,
		Longitude: -76.8205032349,
		Timezone:  "America/New_York",
	},
	"NK12": {
		ICAO:      "NK12",
		Name:      "Donnelly's Airport",
		City:      "Wales",
		State:     "New-York",
		Country:   "US",
		Elevation: 1030,
		Latitude:  42.7695007324,
		Longitude: -78.5469970703,
		Timezone:  "America/New_York",
	},
	"NK16": {
		ICAO:      "NK16",
		Name:      "Hendricks Field",
		City:      "Gouverneur",
		State:     "New-York",
		Country:   "US",
		Elevation: 480,
		Latitude:  44.3680992126,
		Longitude: -75.4023971558,
		Timezone:  "America/New_York",
	},
	"NK17": {
		ICAO:      "NK17",
		Name:      "August Field",
		City:      "Gansevoort",
		State:     "New-York",
		Country:   "US",
		Elevation: 131,
		Latitude:  43.209361,
		Longitude: -73.581711,
		Timezone:  "America/New_York",
	},
	"NK19": {
		ICAO:      "NK19",
		Name:      "Potoczak Airport",
		City:      "Clarence Center",
		State:     "New-York",
		Country:   "US",
		Elevation: 582,
		Latitude:  43.0611991882,
		Longitude: -78.6613998413,
		Timezone:  "America/New_York",
	},
	"NK24": {
		ICAO:      "NK24",
		Name:      "Tilden Airport",
		City:      "Montour Falls",
		State:     "New-York",
		Country:   "US",
		Elevation: 900,
		Latitude:  42.3473014832,
		Longitude: -76.8787994385,
		Timezone:  "America/New_York",
	},
	"NK25": {
		ICAO:      "NK25",
		Name:      "Cambria Airport",
		City:      "Lockport",
		State:     "New-York",
		Country:   "US",
		Elevation: 380,
		Latitude:  43.2159004211,
		Longitude: -78.8084030151,
		Timezone:  "America/New_York",
	},
	"NK26": {
		ICAO:      "NK26",
		Name:      "Old Forge Airport",
		City:      "Old Forge",
		State:     "New-York",
		Country:   "US",
		Elevation: 1753,
		Latitude:  43.725183,
		Longitude: -74.976217,
		Timezone:  "America/New_York",
	},
	"NK27": {
		ICAO:      "NK27",
		Name:      "Hedge Hop Field",
		City:      "Springville",
		State:     "New-York",
		Country:   "US",
		Elevation: 1400,
		Latitude:  42.4889984131,
		Longitude: -78.634803772,
		Timezone:  "America/New_York",
	},
	"NK31": {
		ICAO:      "NK31",
		Name:      "Dawn Patrol Aviation Airport",
		City:      "Albion",
		State:     "New-York",
		Country:   "US",
		Elevation: 480,
		Latitude:  43.2695007324,
		Longitude: -78.1453018188,
		Timezone:  "America/New_York",
	},
	"NK43": {
		ICAO:      "NK43",
		Name:      "Jerry Phibbs Airport",
		City:      "Albany",
		State:     "New-York",
		Country:   "US",
		Elevation: 340,
		Latitude:  42.7583999634,
		Longitude: -73.9245986938,
		Timezone:  "America/New_York",
	},
	"NK47": {
		ICAO:      "NK47",
		Name:      "Dalrymples Airport",
		City:      "Elmira",
		State:     "New-York",
		Country:   "US",
		Elevation: 900,
		Latitude:  42.0416984558,
		Longitude: -76.8330001831,
		Timezone:  "America/New_York",
	},
	"NK52": {
		ICAO:      "NK52",
		Name:      "Connelly Field",
		City:      "Elmira",
		State:     "New-York",
		Country:   "US",
		Elevation: 1525,
		Latitude:  42.0850982666,
		Longitude: -76.9213027954,
		Timezone:  "America/New_York",
	},
	"NK53": {
		ICAO:      "NK53",
		Name:      "Dodge/Coppola/Wheeler Airport",
		City:      "Sullivanville",
		State:     "New-York",
		Country:   "US",
		Elevation: 1030,
		Latitude:  42.2056007385,
		Longitude: -76.7705001831,
		Timezone:  "America/New_York",
	},
	"NK54": {
		ICAO:      "NK54",
		Name:      "Matejka Field",
		City:      "Erin",
		State:     "New-York",
		Country:   "US",
		Elevation: 1690,
		Latitude:  42.176700592,
		Longitude: -76.6410980225,
		Timezone:  "America/New_York",
	},
	"NK68": {
		ICAO:      "NK68",
		Name:      "White Birch Field",
		City:      "Hancock",
		State:     "New-York",
		Country:   "US",
		Elevation: 1860,
		Latitude:  42.0500984192,
		Longitude: -75.3010025024,
		Timezone:  "America/New_York",
	},
	"NK71": {
		ICAO:      "NK71",
		Name:      "Marcellus Airport",
		City:      "Marcellus",
		State:     "New-York",
		Country:   "US",
		Elevation: 850,
		Latitude:  43.0111999512,
		Longitude: -76.3497009277,
		Timezone:  "America/New_York",
	},
	"NK72": {
		ICAO:      "NK72",
		Name:      "James Henion Private Field",
		City:      "Ithaca",
		State:     "New-York",
		Country:   "US",
		Elevation: 1300,
		Latitude:  42.3762016296,
		Longitude: -76.5324020386,
		Timezone:  "America/New_York",
	},
	"NK74": {
		ICAO:      "NK74",
		Name:      "Match Mate Airport",
		City:      "Aurora",
		State:     "New-York",
		Country:   "US",
		Elevation: 840,
		Latitude:  42.7444992065,
		Longitude: -76.6635971069,
		Timezone:  "America/New_York",
	},
	"NK76": {
		ICAO:      "NK76",
		Name:      "Grammar Airport",
		City:      "Geneva",
		State:     "New-York",
		Country:   "US",
		Elevation: 755,
		Latitude:  42.8824996948,
		Longitude: -77.0516967773,
		Timezone:  "America/New_York",
	},
	"NK79": {
		ICAO:      "NK79",
		Name:      "Lewis Landing Airport",
		City:      "Slate Hill",
		State:     "New-York",
		Country:   "US",
		Elevation: 880,
		Latitude:  41.4183998108,
		Longitude: -74.5324020386,
		Timezone:  "America/New_York",
	},
	"NK81": {
		ICAO:      "NK81",
		Name:      "Roberts Roost Airport",
		City:      "Arcade",
		State:     "New-York",
		Country:   "US",
		Elevation: 1751,
		Latitude:  42.6333999634,
		Longitude: -78.3553009033,
		Timezone:  "America/New_York",
	},
	"NK82": {
		ICAO:      "NK82",
		Name:      "Engineers Airport",
		City:      "Constantia",
		State:     "New-York",
		Country:   "US",
		Elevation: 480,
		Latitude:  43.3158988953,
		Longitude: -76.0149002075,
		Timezone:  "America/New_York",
	},
	"NK83": {
		ICAO:      "NK83",
		Name:      "Nettie's Place Airport",
		City:      "Quaker Street",
		State:     "New-York",
		Country:   "US",
		Elevation: 1330,
		Latitude:  42.7100982666,
		Longitude: -74.2237014771,
		Timezone:  "America/New_York",
	},
	"NK87": {
		ICAO:      "NK87",
		Name:      "Merkle Airport",
		City:      "Clarence Center",
		State:     "New-York",
		Country:   "US",
		Elevation: 590,
		Latitude:  43.0666999817,
		Longitude: -78.5997009277,
		Timezone:  "America/New_York",
	},
	"NK89": {
		ICAO:      "NK89",
		Name:      "Ultralight Flight Farm Ultralightport",
		City:      "Monterey",
		State:     "New-York",
		Country:   "US",
		Elevation: 1200,
		Latitude:  42.2916984558,
		Longitude: -77.0579986572,
		Timezone:  "America/New_York",
	},
	"NK91": {
		ICAO:      "NK91",
		Name:      "Boyle's Landing Airport",
		City:      "Sharon Springs",
		State:     "New-York",
		Country:   "US",
		Elevation: 1300,
		Latitude:  42.8162002563,
		Longitude: -74.6246032715,
		Timezone:  "America/New_York",
	},
	"NLWF": {
		ICAO:      "NLWF",
		IATA:      "FUT",
		Name:      "Pointe Vele Airport",
		City:      "Futuna Island",
		State:     "Alo",
		Country:   "WF",
		Elevation: 20,
		Latitude:  -14.3114004135,
		Longitude: -178.065994263,
		Timezone:  "Pacific/Wallis",
	},
	"NLWW": {
		ICAO:      "NLWW",
		IATA:      "WLS",
		Name:      "Hihifo Airport",
		City:      "Wallis Island",
		State:     "Uvea",
		Country:   "WF",
		Elevation: 79,
		Latitude:  -13.2383003235,
		Longitude: -176.199005127,
		Timezone:  "Pacific/Wallis",
	},
	"NM01": {
		ICAO:      "NM01",
		Name:      "Rancho Magdalena Airport",
		City:      "Magdalena",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 6676,
		Latitude:  34.0974998474,
		Longitude: -107.297996521,
		Timezone:  "America/Denver",
	},
	"NM02": {
		ICAO:      "NM02",
		Name:      "Biplane Ranch Airport",
		City:      "Moriarty",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 6500,
		Latitude:  34.92229843,
		Longitude: -106.1669998,
		Timezone:  "America/Denver",
	},
	"NM03": {
		ICAO:      "NM03",
		Name:      "K-D Field",
		City:      "Dexter",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 3465,
		Latitude:  33.275100708,
		Longitude: -104.3690032959,
		Timezone:  "America/Denver",
	},
	"NM04": {
		ICAO:      "NM04",
		Name:      "Keller Field",
		City:      "Farmington",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 5345,
		Latitude:  36.6777992249,
		Longitude: -108.0839996338,
		Timezone:  "America/Denver",
	},
	"NM05": {
		ICAO:      "NM05",
		Name:      "Williams Airport",
		City:      "Farmington",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 5500,
		Latitude:  36.7447013855,
		Longitude: -108.1500015259,
		Timezone:  "America/Denver",
	},
	"NM08": {
		ICAO:      "NM08",
		Name:      "Solo Ranch Airport",
		City:      "Deming",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4200,
		Latitude:  32.2625999451,
		Longitude: -107.4039993286,
		Timezone:  "America/Denver",
	},
	"NM09": {
		ICAO:      "NM09",
		Name:      "First Aero Squadron Airpark",
		City:      "Columbus",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4140,
		Latitude:  31.8673000336,
		Longitude: -107.6360015869,
		Timezone:  "America/Denver",
	},
	"NM10": {
		ICAO:      "NM10",
		Name:      "Tequesquite Ranch Airport",
		City:      "Albert",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4500,
		Latitude:  35.8791999817,
		Longitude: -103.81300354,
		Timezone:  "America/Denver",
	},
	"NM13": {
		ICAO:      "NM13",
		Name:      "Burris E Station Airport",
		City:      "Las Cruces",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4300,
		Latitude:  32.2472991943,
		Longitude: -107.1920013428,
		Timezone:  "America/Denver",
	},
	"NM14": {
		ICAO:      "NM14",
		Name:      "Burris 'E' Ranch Airport",
		City:      "Las Cruces",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4400,
		Latitude:  32.3111991882,
		Longitude: -107.2949981689,
		Timezone:  "America/Denver",
	},
	"NM15": {
		ICAO:      "NM15",
		Name:      "Clavel Ranch Airport",
		City:      "Roy",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 5534,
		Latitude:  36.0461006165,
		Longitude: -103.8870010376,
		Timezone:  "America/Denver",
	},
	"NM16": {
		ICAO:      "NM16",
		Name:      "Chama Land & Cattle Co. Airport",
		City:      "Chama",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 8025,
		Latitude:  36.8666992188,
		Longitude: -106.5670013428,
		Timezone:  "America/Denver",
	},
	"NM17": {
		ICAO:      "NM17",
		Name:      "Ray Ranch Airport",
		City:      "Roy",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 5850,
		Latitude:  35.9305992126,
		Longitude: -104.1839981079,
		Timezone:  "America/Denver",
	},
	"NM20": {
		ICAO:      "NM20",
		Name:      "Benedict Airpark",
		City:      "Roswell",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 3556,
		Latitude:  33.2519989014,
		Longitude: -104.4420013428,
		Timezone:  "America/Denver",
	},
	"NM21": {
		ICAO:      "NM21",
		Name:      "Patterson Ranch Airport",
		City:      "Stanley",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 6696,
		Latitude:  35.1749992371,
		Longitude: -105.8700027466,
		Timezone:  "America/Denver",
	},
	"NM24": {
		ICAO:      "NM24",
		Name:      "Eastside Airport",
		City:      "Chama",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 8030,
		Latitude:  36.9216995239,
		Longitude: -106.5599975586,
		Timezone:  "America/Denver",
	},
	"NM25": {
		ICAO:      "NM25",
		Name:      "Lincoln Station Airport",
		City:      "Corona",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 6655,
		Latitude:  34.1241989136,
		Longitude: -105.6709976196,
		Timezone:  "America/Denver",
	},
	"NM26": {
		ICAO:      "NM26",
		Name:      "Luna Landing Airport",
		City:      "Deming",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4250,
		Latitude:  32.1001014709,
		Longitude: -107.8229980469,
		Timezone:  "America/Denver",
	},
	"NM27": {
		ICAO:      "NM27",
		Name:      "Sanostee Airport",
		City:      "Gallup",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 5950,
		Latitude:  36.4225006104,
		Longitude: -108.861000061,
		Timezone:  "America/Denver",
	},
	"NM28": {
		ICAO:      "NM28",
		Name:      "Beckett Farm Airport",
		City:      "Tularosa",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4430,
		Latitude:  33.0569992065,
		Longitude: -106.0250015259,
		Timezone:  "America/Denver",
	},
	"NM29": {
		ICAO:      "NM29",
		Name:      "Rosebud Airport",
		City:      "Rosebud",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4820,
		Latitude:  35.8342018127,
		Longitude: -103.43699646,
		Timezone:  "America/Denver",
	},
	"NM30": {
		ICAO:      "NM30",
		Name:      "Mitchell Farms Airport",
		City:      "Bellview",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4475,
		Latitude:  34.8750991821,
		Longitude: -103.0670013428,
		Timezone:  "America/Denver",
	},
	"NM31": {
		ICAO:      "NM31",
		Name:      "Elk Valley Airstrip",
		City:      "Pie Town",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 7600,
		Latitude:  34.496208,
		Longitude: -108.029722,
		Timezone:  "America/Denver",
	},
	"NM32": {
		ICAO:      "NM32",
		Name:      "Bell Ranch Waggoner Airport",
		City:      "Bell Ranch",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4725,
		Latitude:  35.7078018188,
		Longitude: -104.0609970093,
		Timezone:  "America/Denver",
	},
	"NM33": {
		ICAO:      "NM33",
		Name:      "Bell Ranch Headquarters Airport",
		City:      "Bell Ranch",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4400,
		Latitude:  35.5475006104,
		Longitude: -104.0839996338,
		Timezone:  "America/Denver",
	},
	"NM36": {
		ICAO:      "NM36",
		Name:      "Candy Kitchen Ranch Airport",
		City:      "Ramah",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 7220,
		Latitude:  34.9028015137,
		Longitude: -108.5149993896,
		Timezone:  "America/Denver",
	},
	"NM37": {
		ICAO:      "NM37",
		Name:      "Adobe Ranch Private Airport",
		City:      "Truth Or Consequences",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 7354,
		Latitude:  33.576499939,
		Longitude: -107.9000015259,
		Timezone:  "America/Denver",
	},
	"NM38": {
		ICAO:      "NM38",
		Name:      "Double V Ranch Airport",
		City:      "Fort Sumner",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4114,
		Latitude:  34.1172981262,
		Longitude: -104.4270019531,
		Timezone:  "America/Denver",
	},
	"NM39": {
		ICAO:      "NM39",
		Name:      "Davenport Airport",
		City:      "Farmington",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 5720,
		Latitude:  36.6328010559,
		Longitude: -108.3109970093,
		Timezone:  "America/Denver",
	},
	"NM40": {
		ICAO:      "NM40",
		Name:      "Gorby Ranch Airport",
		City:      "Alamogordo",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4400,
		Latitude:  32.9634017944,
		Longitude: -105.9990005493,
		Timezone:  "America/Denver",
	},
	"NM41": {
		ICAO:      "NM41",
		Name:      "Happy Mountain Airport",
		City:      "Pie Town",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 7250,
		Latitude:  34.0611991882,
		Longitude: -108.0960006714,
		Timezone:  "America/Denver",
	},
	"NM42": {
		ICAO:      "NM42",
		Name:      "Aero Tech Inc Airport",
		City:      "Clovis",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4250,
		Latitude:  34.4157981873,
		Longitude: -103.1480026245,
		Timezone:  "America/Denver",
	},
	"NM44": {
		ICAO:      "NM44",
		Name:      "Bojax Ranch Airport",
		City:      "Kenna",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 3920,
		Latitude:  34.0172996521,
		Longitude: -104.1159973145,
		Timezone:  "America/Denver",
	},
	"NM47": {
		ICAO:      "NM47",
		Name:      "2 X 4 Ranch Airport",
		City:      "Artesia",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 3360,
		Latitude:  32.7801017761,
		Longitude: -104.3809967041,
		Timezone:  "America/Denver",
	},
	"NM48": {
		ICAO:      "NM48",
		Name:      "Esenada Airport",
		City:      "Ensenada",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 7640,
		Latitude:  36.7305984497,
		Longitude: -106.4950027466,
		Timezone:  "America/Denver",
	},
	"NM49": {
		ICAO:      "NM49",
		Name:      "J & M Farms Airport",
		City:      "Willard",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 6135,
		Latitude:  34.5815010071,
		Longitude: -106.0749969482,
		Timezone:  "America/Denver",
	},
	"NM51": {
		ICAO:      "NM51",
		Name:      "Chloride Airport",
		City:      "Chloride",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 6202,
		Latitude:  33.3283996582,
		Longitude: -107.6620025635,
		Timezone:  "America/Denver",
	},
	"NM52": {
		ICAO:      "NM52",
		Name:      "Camco Ranch Airport",
		City:      "Nara Visa",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4255,
		Latitude:  35.5746994019,
		Longitude: -103.2870025635,
		Timezone:  "America/Denver",
	},
	"NM53": {
		ICAO:      "NM53",
		Name:      "San Miguel Ranch Airport",
		City:      "Las Vegas",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 6300,
		Latitude:  35.5099983215,
		Longitude: -104.5839996338,
		Timezone:  "America/Denver",
	},
	"NM55": {
		ICAO:      "NM55",
		Name:      "Whitewater Mesa Ranch Airport",
		City:      "Glenwood",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 5460,
		Latitude:  33.3628005981,
		Longitude: -108.8639984131,
		Timezone:  "America/Denver",
	},
	"NM56": {
		ICAO:      "NM56",
		Name:      "Mystic Bluffs Airport",
		City:      "Ramah",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 6980,
		Latitude:  35.1652984619,
		Longitude: -108.458999634,
		Timezone:  "America/Denver",
	},
	"NM59": {
		ICAO:      "NM59",
		Name:      "Price's Dairy Airport",
		City:      "Albuquerque",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 5100,
		Latitude:  35.324798584,
		Longitude: -106.6669998169,
		Timezone:  "America/Denver",
	},
	"NM61": {
		ICAO:      "NM61",
		Name:      "New Mexico Soaring Ranch Airport",
		City:      "Albuquerque",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 6530,
		Latitude:  35.1166992188,
		Longitude: -106.1470031738,
		Timezone:  "America/Denver",
	},
	"NM63": {
		ICAO:      "NM63",
		Name:      "Big Sky Airport",
		City:      "Stanley",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 6200,
		Latitude:  35.1657981873,
		Longitude: -106.0120010376,
		Timezone:  "America/Denver",
	},
	"NM64": {
		ICAO:      "NM64",
		Name:      "Gray Ranch Airport",
		City:      "Animas",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 5155,
		Latitude:  31.4587001801,
		Longitude: -108.8629989624,
		Timezone:  "America/Denver",
	},
	"NM66": {
		ICAO:      "NM66",
		Name:      "Poco Loco Airport",
		City:      "Pie Town",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 7420,
		Latitude:  34.4151992798,
		Longitude: -108.077003479,
		Timezone:  "America/Denver",
	},
	"NM67": {
		ICAO:      "NM67",
		Name:      "La Mesa Park Airport",
		City:      "Raton",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 6578,
		Latitude:  36.8569984436,
		Longitude: -104.4479980469,
		Timezone:  "America/Denver",
	},
	"NM68": {
		ICAO:      "NM68",
		Name:      "Rael Ranch Horse Pasture Airport",
		City:      "Horse Springs",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 6950,
		Latitude:  33.8050994873,
		Longitude: -108.3430023193,
		Timezone:  "America/Denver",
	},
	"NM69": {
		ICAO:      "NM69",
		Name:      "Casas Adobes Airpark",
		City:      "Mimbres",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 5800,
		Latitude:  32.814201355,
		Longitude: -107.9400024414,
		Timezone:  "America/Denver",
	},
	"NM70": {
		ICAO:      "NM70",
		Name:      "Rodeo Airport",
		City:      "Rodeo",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4157,
		Latitude:  31.9500007629,
		Longitude: -109.0419998169,
		Timezone:  "America/Denver",
	},
	"NM71": {
		ICAO:      "NM71",
		Name:      "Three Rivers Ranch Airport",
		City:      "Three Rivers",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4807,
		Latitude:  33.332901001,
		Longitude: -106.0260009766,
		Timezone:  "America/Denver",
	},
	"NM72": {
		ICAO:      "NM72",
		Name:      "Turner Ridgeport Airport",
		City:      "Silver City",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 6037,
		Latitude:  32.7419013977,
		Longitude: -108.2880020142,
		Timezone:  "America/Denver",
	},
	"NM73": {
		ICAO:      "NM73",
		Name:      "Akin and Akin Airport",
		City:      "Mc Alister",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4858,
		Latitude:  34.7723007202,
		Longitude: -103.8560028076,
		Timezone:  "America/Denver",
	},
	"NM74": {
		ICAO:      "NM74",
		Name:      "Cubero Airport",
		City:      "Cubero",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 6192,
		Latitude:  35.0875015259,
		Longitude: -107.56300354,
		Timezone:  "America/Denver",
	},
	"NM75": {
		ICAO:      "NM75",
		Name:      "Doolittle Ranch Airport",
		City:      "Watrous",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 6550,
		Latitude:  35.8135986328,
		Longitude: -104.9700012207,
		Timezone:  "America/Denver",
	},
	"NM76": {
		ICAO:      "NM76",
		Name:      "Park Springs Airport",
		City:      "Anton Chico",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 5115,
		Latitude:  35.260799408,
		Longitude: -104.9189987183,
		Timezone:  "America/Denver",
	},
	"NM78": {
		ICAO:      "NM78",
		Name:      "Hacienda Sur Luna Airport",
		City:      "Columbus",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4150,
		Latitude:  31.8778991699,
		Longitude: -107.6460037231,
		Timezone:  "America/Denver",
	},
	"NM79": {
		ICAO:      "NM79",
		Name:      "Shoestring Ranch Airport",
		City:      "Organ",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4440,
		Latitude:  32.4430999756,
		Longitude: -106.6800003052,
		Timezone:  "America/Denver",
	},
	"NM80": {
		ICAO:      "NM80",
		Name:      "Quemado Airport",
		City:      "Quemado",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 6917,
		Latitude:  34.3403015137,
		Longitude: -108.5149993896,
		Timezone:  "America/Denver",
	},
	"NM83": {
		ICAO:      "NM83",
		Name:      "Industrial Airpark",
		City:      "Hobbs",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 3707,
		Latitude:  32.7667999268,
		Longitude: -103.2089996338,
		Timezone:  "America/Denver",
	},
	"NM84": {
		ICAO:      "NM84",
		Name:      "G Bar F Ranch Airport",
		City:      "Capitan",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 6505,
		Latitude:  33.5444984436,
		Longitude: -105.3759994507,
		Timezone:  "America/Denver",
	},
	"NM86": {
		ICAO:      "NM86",
		Name:      "Playas Air Strip",
		City:      "Playas",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4519,
		Latitude:  31.9368000031,
		Longitude: -108.5370025635,
		Timezone:  "America/Denver",
	},
	"NM87": {
		ICAO:      "NM87",
		Name:      "Jenkins Airport",
		City:      "Roswell",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 3600,
		Latitude:  33.2383003235,
		Longitude: -104.4710006714,
		Timezone:  "America/Denver",
	},
	"NM88": {
		ICAO:      "NM88",
		Name:      "Skywagon Farm Airport",
		City:      "Bosque",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4770,
		Latitude:  34.54330063,
		Longitude: -106.7600021,
		Timezone:  "America/Denver",
	},
	"NM89": {
		ICAO:      "NM89",
		Name:      "Manzano-Mountain Air Ranch Airport",
		City:      "Los Lunas",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 5630,
		Latitude:  34.7756996155,
		Longitude: -106.5210037231,
		Timezone:  "America/Denver",
	},
	"NM90": {
		ICAO:      "NM90",
		Name:      "Amigos Del Cielo Airport",
		City:      "Rodeo",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 4107,
		Latitude:  31.9029998779,
		Longitude: -109.0250015259,
		Timezone:  "America/Denver",
	},
	"NM91": {
		ICAO:      "NM91",
		Name:      "High Lonesome Airport",
		City:      "Fence Lake",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 7474,
		Latitude:  34.62229919,
		Longitude: -108.7900009,
		Timezone:  "America/Denver",
	},
	"NM92": {
		ICAO:      "NM92",
		Name:      "Tierra De Dios Airport",
		City:      "Rowe",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 7050,
		Latitude:  35.295767,
		Longitude: -105.6115,
		Timezone:  "America/Denver",
	},
	"NR02": {
		ICAO:      "NR02",
		Name:      "Jordan Field",
		City:      "Mount Gilead",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 600,
		Latitude:  35.237778,
		Longitude: -79.995556,
		Timezone:  "America/New_York",
	},
	"NR04": {
		ICAO:      "NR04",
		Name:      "Southern Skies Airstrip",
		City:      "Taylorsville",
		State:     "North-Carolina",
		Country:   "US",
		Elevation: 1180,
		Latitude:  35.960278,
		Longitude: -81.173056,
		Timezone:  "America/New_York",
	},
	"NSAS": {
		ICAO:      "NSAS",
		IATA:      "OFU",
		Name:      "Ofu Village Airport",
		City:      "Ofu Village",
		State:     "Manu'a",
		Country:   "AS",
		Elevation: 9,
		Latitude:  -14.1843996048,
		Longitude: -169.6699981689,
		Timezone:  "Pacific/Pago_Pago",
	},
	"NSAU": {
		ICAO:      "NSAU",
		IATA:      "AAU",
		Name:      "Asau Airport",
		City:      "Asau",
		Country:   "WS",
		Latitude:  -13.5058002472,
		Longitude: -172.628005981,
		Timezone:  "Pacific/Apia",
	},
	"NSFA": {
		ICAO:      "NSFA",
		IATA:      "APW",
		Name:      "Faleolo International Airport",
		City:      "Apia",
		State:     "A'ana",
		Country:   "WS",
		Elevation: 58,
		Latitude:  -13.8299999237,
		Longitude: -172.0079956055,
		Timezone:  "Pacific/Apia",
	},
	"NSFI": {
		ICAO:      "NSFI",
		IATA:      "FGI",
		Name:      "Fagali'i Airport",
		City:      "Apia",
		State:     "Tuamasaga",
		Country:   "WS",
		Elevation: 131,
		Latitude:  -13.8486995697,
		Longitude: -171.740005493,
		Timezone:  "Pacific/Apia",
	},
	"NSFQ": {
		ICAO:      "NSFQ",
		IATA:      "FTI",
		Name:      "Fitiuta Airport",
		City:      "Fitiuta Village",
		State:     "Manu'a",
		Country:   "AS",
		Elevation: 110,
		Latitude:  -14.2172002792,
		Longitude: -169.425003052,
		Timezone:  "Pacific/Pago_Pago",
	},
	"NSMA": {
		ICAO:      "NSMA",
		IATA:      "MXS",
		Name:      "Maota Airport",
		City:      "Maota",
		Country:   "WS",
		Latitude:  -13.7423000336,
		Longitude: -172.2579956055,
		Timezone:  "Pacific/Apia",
	},
	"NSTU": {
		ICAO:      "NSTU",
		IATA:      "PPG",
		Name:      "Pago Pago International Airport",
		City:      "Pago Pago",
		State:     "Eastern-District",
		Country:   "AS",
		Elevation: 32,
		Latitude:  -14.3310003281,
		Longitude: -170.710006714,
		Timezone:  "Pacific/Pago_Pago",
	},
	"NTAA": {
		ICAO:      "NTAA",
		IATA:      "PPT",
		Name:      "Faa'a International Airport",
		City:      "Papeete",
		State:     "Iles-du-Vent",
		Country:   "PF",
		Elevation: 5,
		Latitude:  -17.5536994934,
		Longitude: -149.606994629,
		Timezone:  "Pacific/Tahiti",
	},
	"NTAR": {
		ICAO:      "NTAR",
		IATA:      "RUR",
		Name:      "Rurutu Airport",
		State:     "Iles-Australes",
		Country:   "PF",
		Elevation: 18,
		Latitude:  -22.4340991974,
		Longitude: -151.3609924316,
		Timezone:  "Pacific/Tahiti",
	},
	"NTAT": {
		ICAO:      "NTAT",
		IATA:      "TUB",
		Name:      "Tubuai Airport",
		State:     "Iles-Australes",
		Country:   "PF",
		Elevation: 7,
		Latitude:  -23.3654003143,
		Longitude: -149.5240020752,
		Timezone:  "Pacific/Tahiti",
	},
	"NTAV": {
		ICAO:      "NTAV",
		IATA:      "RVV",
		Name:      "Raivavae Airport",
		State:     "Iles-Australes",
		Country:   "PF",
		Elevation: 7,
		Latitude:  -23.8852005005,
		Longitude: -147.662002563,
		Timezone:  "Pacific/Tahiti",
	},
	"NTGA": {
		ICAO:      "NTGA",
		IATA:      "AAA",
		Name:      "Anaa Airport",
		State:     "Iles-Tuamotu-Gambier",
		Country:   "PF",
		Elevation: 10,
		Latitude:  -17.3526000977,
		Longitude: -145.5099945068,
		Timezone:  "Pacific/Tahiti",
	},
	"NTGB": {
		ICAO:      "NTGB",
		IATA:      "FGU",
		Name:      "Fangatau Airport",
		State:     "Iles-Tuamotu-Gambier",
		Country:   "PF",
		Elevation: 9,
		Latitude:  -15.819899559,
		Longitude: -140.8869934082,
		Timezone:  "Pacific/Tahiti",
	},
	"NTGC": {
		ICAO:      "NTGC",
		IATA:      "TIH",
		Name:      "Tikehau Airport",
		Country:   "PF",
		Elevation: 6,
		Latitude:  -15.119600296,
		Longitude: -148.2310028076,
		Timezone:  "Pacific/Tahiti",
	},
	"NTGD": {
		ICAO:      "NTGD",
		IATA:      "APK",
		Name:      "Apataki Airport",
		City:      "Apataki",
		State:     "Iles-Tuamotu-Gambier",
		Country:   "PF",
		Elevation: 8,
		Latitude:  -15.5735998154,
		Longitude: -146.4149932861,
		Timezone:  "Pacific/Tahiti",
	},
	"NTGE": {
		ICAO:      "NTGE",
		IATA:      "REA",
		Name:      "Reao Airport",
		State:     "Iles-Tuamotu-Gambier",
		Country:   "PF",
		Elevation: 12,
		Latitude:  -18.4659004211,
		Longitude: -136.4400024414,
		Timezone:  "Pacific/Tahiti",
	},
	"NTGF": {
		ICAO:      "NTGF",
		IATA:      "FAV",
		Name:      "Fakarava Airport",
		State:     "Iles-Tuamotu-Gambier",
		Country:   "PF",
		Elevation: 13,
		Latitude:  -16.0541000366,
		Longitude: -145.6569976807,
		Timezone:  "Pacific/Tahiti",
	},
	"NTGH": {
		ICAO:      "NTGH",
		IATA:      "HHZ",
		Name:      "Hikueru Atoll Airport",
		City:      "Hikueru Atoll",
		State:     "Iles-Tuamotu-Gambier",
		Country:   "PF",
		Elevation: 5,
		Latitude:  -17.5447006226,
		Longitude: -142.6139984131,
		Timezone:  "Pacific/Tahiti",
	},
	"NTGI": {
		ICAO:      "NTGI",
		IATA:      "XMH",
		Name:      "Manihi Airport",
		State:     "Iles-Tuamotu-Gambier",
		Country:   "PF",
		Elevation: 14,
		Latitude:  -14.4368000031,
		Longitude: -146.0700073242,
		Timezone:  "Pacific/Tahiti",
	},
	"NTGJ": {
		ICAO:      "NTGJ",
		IATA:      "GMR",
		Name:      "Totegegie Airport",
		State:     "Iles-Tuamotu-Gambier",
		Country:   "PF",
		Elevation: 7,
		Latitude:  -23.0799007416,
		Longitude: -134.8899993896,
		Timezone:  "Pacific/Gambier",
	},
	"NTGK": {
		ICAO:      "NTGK",
		IATA:      "KKR",
		Name:      "Kaukura Airport",
		State:     "Iles-Tuamotu-Gambier",
		Country:   "PF",
		Elevation: 11,
		Latitude:  -15.6632995605,
		Longitude: -146.8849945068,
		Timezone:  "Pacific/Tahiti",
	},
	"NTGM": {
		ICAO:      "NTGM",
		IATA:      "MKP",
		Name:      "Makemo Airport",
		State:     "Iles-Tuamotu-Gambier",
		Country:   "PF",
		Elevation: 3,
		Latitude:  -16.5839004517,
		Longitude: -143.6580047607,
		Timezone:  "Pacific/Tahiti",
	},
	"NTGN": {
		ICAO:      "NTGN",
		IATA:      "NAU",
		Name:      "Napuka Island Airport",
		City:      "Napuka Island",
		State:     "Iles-Tuamotu-Gambier",
		Country:   "PF",
		Elevation: 7,
		Latitude:  -14.1767997742,
		Longitude: -141.266998291,
		Timezone:  "Pacific/Tahiti",
	},
	"NTGO": {
		ICAO:      "NTGO",
		IATA:      "TKV",
		Name:      "Tatakoto Airport",
		City:      "Tatakoto",
		State:     "Iles-Tuamotu-Gambier",
		Country:   "PF",
		Elevation: 12,
		Latitude:  -17.3553009033,
		Longitude: -138.4450073242,
		Timezone:  "Pacific/Tahiti",
	},
	"NTGP": {
		ICAO:      "NTGP",
		IATA:      "PKP",
		Name:      "Puka Puka Airport",
		Country:   "PF",
		Elevation: 5,
		Latitude:  -14.8094997406,
		Longitude: -138.81300354,
		Timezone:  "Pacific/Tahiti",
	},
	"NTGQ": {
		ICAO:      "NTGQ",
		IATA:      "PUK",
		Name:      "Pukarua Airport",
		City:      "Pukarua",
		State:     "Iles-Tuamotu-Gambier",
		Country:   "PF",
		Elevation: 5,
		Latitude:  -18.2956008911,
		Longitude: -137.016998291,
		Timezone:  "Pacific/Tahiti",
	},
	"NTGR": {
		ICAO:      "NTGR",
		Name:      "Aratika Airport",
		City:      "Aratika",
		Country:   "PF",
		Elevation: 7,
		Latitude:  -15.6332998276,
		Longitude: -145.5,
		Timezone:  "Pacific/Tahiti",
	},
	"NTGS": {
		ICAO:      "NTGS",
		Name:      "Marutea Airport",
		City:      "Auorotini",
		Country:   "PF",
		Elevation: 10,
		Latitude:  -21.4821949005,
		Longitude: -135.6370544434,
		Timezone:  "Pacific/Gambier",
	},
	"NTGT": {
		ICAO:      "NTGT",
		IATA:      "TKP",
		Name:      "Takapoto Airport",
		Country:   "PF",
		Elevation: 12,
		Latitude:  -14.7095003128,
		Longitude: -145.2460021973,
		Timezone:  "Pacific/Tahiti",
	},
	"NTGU": {
		ICAO:      "NTGU",
		IATA:      "AXR",
		Name:      "Arutua Airport",
		State:     "Iles-Tuamotu-Gambier",
		Country:   "PF",
		Elevation: 9,
		Latitude:  -15.2482995987,
		Longitude: -146.6170043945,
		Timezone:  "Pacific/Tahiti",
	},
	"NTGV": {
		ICAO:      "NTGV",
		IATA:      "MVT",
		Name:      "Mataiva Airport",
		Country:   "PF",
		Elevation: 11,
		Latitude:  -14.8681001663,
		Longitude: -148.7169952393,
		Timezone:  "Pacific/Tahiti",
	},
	"NTGW": {
		ICAO:      "NTGW",
		IATA:      "NUK",
		Name:      "Nukutavake Airport",
		City:      "Nukutavake",
		State:     "Iles-Tuamotu-Gambier",
		Country:   "PF",
		Elevation: 17,
		Latitude:  -19.2849998474,
		Longitude: -138.7720031738,
		Timezone:  "Pacific/Tahiti",
	},
	"NTGY": {
		ICAO:      "NTGY",
		IATA:      "ZTA",
		Name:      "Tureia Airport",
		Country:   "PF",
		Elevation: 12,
		Latitude:  -20.7896995544,
		Longitude: -138.5700073242,
		Timezone:  "Pacific/Tahiti",
	},
	"NTHE": {
		ICAO:      "NTHE",
		IATA:      "AHE",
		Name:      "Ahe Airport",
		City:      "Ahe Atoll",
		State:     "Iles-Tuamotu-Gambier",
		Country:   "PF",
		Elevation: 11,
		Latitude:  -14.4280996323,
		Longitude: -146.2570037842,
		Timezone:  "Pacific/Tahiti",
	},
	"NTKA": {
		ICAO:      "NTKA",
		IATA:      "KHZ",
		Name:      "Kauehi Airport",
		City:      "Kauehi",
		Country:   "PF",
		Elevation: 13,
		Latitude:  -15.7807998657,
		Longitude: -145.1239929199,
		Timezone:  "Pacific/Tahiti",
	},
	"NTKF": {
		ICAO:      "NTKF",
		IATA:      "FAC",
		Name:      "Faaite Airport",
		State:     "Iles-Tuamotu-Gambier",
		Country:   "PF",
		Elevation: 7,
		Latitude:  -16.6867008209,
		Longitude: -145.328994751,
		Timezone:  "Pacific/Tahiti",
	},
	"NTKH": {
		ICAO:      "NTKH",
		IATA:      "FHZ",
		Name:      "Fakahina Airport",
		City:      "Fakahina",
		Country:   "PF",
		Elevation: 3,
		Latitude:  -15.9921998978,
		Longitude: -140.1649932861,
		Timezone:  "Pacific/Tahiti",
	},
	"NTKK": {
		ICAO:      "NTKK",
		IATA:      "RKA",
		Name:      "Aratika Nord Airport",
		Country:   "PF",
		Elevation: 10,
		Latitude:  -15.4853000641,
		Longitude: -145.470001221,
		Timezone:  "Pacific/Tahiti",
	},
	"NTKM": {
		ICAO:      "NTKM",
		IATA:      "TJN",
		Name:      "Takume Airport",
		City:      "Takume",
		Country:   "PF",
		Elevation: 7,
		Latitude:  -15.8547000885,
		Longitude: -142.2680053711,
		Timezone:  "Pacific/Tahiti",
	},
	"NTKO": {
		ICAO:      "NTKO",
		IATA:      "RRR",
		Name:      "Raroia Airport",
		State:     "Iles-Tuamotu-Gambier",
		Country:   "PF",
		Elevation: 18,
		Latitude:  -16.045,
		Longitude: -142.476944,
		Timezone:  "Pacific/Tahiti",
	},
	"NTKR": {
		ICAO:      "NTKR",
		IATA:      "TKX",
		Name:      "Takaroa Airport",
		State:     "Iles-Tuamotu-Gambier",
		Country:   "PF",
		Elevation: 13,
		Latitude:  -14.4558000565,
		Longitude: -145.0249938965,
		Timezone:  "Pacific/Tahiti",
	},
	"NTKT": {
		ICAO:      "NTKT",
		IATA:      "KXU",
		Name:      "Katiu Airport",
		City:      "Katiu",
		State:     "Iles-Tuamotu-Gambier",
		Country:   "PF",
		Elevation: 7,
		Latitude:  -16.3393993378,
		Longitude: -144.4029998779,
		Timezone:  "Pacific/Tahiti",
	},
	"NTMD": {
		ICAO:      "NTMD",
		IATA:      "NHV",
		Name:      "Nuku Hiva Airport",
		State:     "Iles-Marquises",
		Country:   "PF",
		Elevation: 220,
		Latitude:  -8.7955999374,
		Longitude: -140.2290039063,
		Timezone:  "Pacific/Marquesas",
	},
	"NTMN": {
		ICAO:      "NTMN",
		IATA:      "AUQ",
		Name:      "Hiva Oa-Atuona Airport",
		State:     "Iles-Marquises",
		Country:   "PF",
		Elevation: 1481,
		Latitude:  -9.7687902451,
		Longitude: -139.0110015869,
		Timezone:  "Pacific/Marquesas",
	},
	"NTMP": {
		ICAO:      "NTMP",
		IATA:      "UAP",
		Name:      "Ua Pou Airport",
		City:      "Ua Pou",
		State:     "Iles-Marquises",
		Country:   "PF",
		Elevation: 16,
		Latitude:  -9.3516702652,
		Longitude: -140.0780029297,
		Timezone:  "Pacific/Marquesas",
	},
	"NTMU": {
		ICAO:      "NTMU",
		IATA:      "UAH",
		Name:      "Ua Huka Airport",
		City:      "Ua Huka",
		State:     "Iles-Marquises",
		Country:   "PF",
		Elevation: 160,
		Latitude:  -8.9361095428,
		Longitude: -139.5520019531,
		Timezone:  "Pacific/Marquesas",
	},
	"NTTB": {
		ICAO:      "NTTB",
		IATA:      "BOB",
		Name:      "Bora Bora Airport",
		City:      "Motu Mute",
		State:     "Leeward-Islands",
		Country:   "PF",
		Elevation: 10,
		Latitude:  -16.4444007874,
		Longitude: -151.7510070801,
		Timezone:  "Pacific/Tahiti",
	},
	"NTTE": {
		ICAO:      "NTTE",
		IATA:      "TTI",
		Name:      "Tetiaroa Airport",
		City:      "Tetiaroa",
		Country:   "PF",
		Elevation: 7,
		Latitude:  -17.013299942,
		Longitude: -149.5870056152,
		Timezone:  "Pacific/Tahiti",
	},
	"NTTG": {
		ICAO:      "NTTG",
		IATA:      "RGI",
		Name:      "Rangiroa Airport",
		State:     "Iles-Tuamotu-Gambier",
		Country:   "PF",
		Elevation: 10,
		Latitude:  -14.9542999268,
		Longitude: -147.6609954834,
		Timezone:  "Pacific/Tahiti",
	},
	"NTTH": {
		ICAO:      "NTTH",
		IATA:      "HUH",
		Name:      "Huahine-Fare Airport",
		City:      "Fare",
		State:     "Leeward-Islands",
		Country:   "PF",
		Elevation: 7,
		Latitude:  -16.6872005463,
		Longitude: -151.0220031738,
		Timezone:  "Pacific/Tahiti",
	},
	"NTTM": {
		ICAO:      "NTTM",
		IATA:      "MOZ",
		Name:      "Moorea Airport",
		State:     "Iles-du-Vent",
		Country:   "PF",
		Elevation: 9,
		Latitude:  -17.4899997711,
		Longitude: -149.7619934082,
		Timezone:  "Pacific/Tahiti",
	},
	"NTTO": {
		ICAO:      "NTTO",
		IATA:      "HOI",
		Name:      "Hao Airport",
		State:     "Iles-Tuamotu-Gambier",
		Country:   "PF",
		Elevation: 10,
		Latitude:  -18.0748004913,
		Longitude: -140.9459991455,
		Timezone:  "Pacific/Tahiti",
	},
	"NTTP": {
		ICAO:      "NTTP",
		IATA:      "MAU",
		Name:      "Maupiti Airport",
		State:     "Leeward-Islands",
		Country:   "PF",
		Elevation: 15,
		Latitude:  -16.4265003204,
		Longitude: -152.2440032959,
		Timezone:  "Pacific/Tahiti",
	},
	"NTTR": {
		ICAO:      "NTTR",
		IATA:      "RFP",
		Name:      "Raiatea Airport",
		City:      "Uturoa",
		State:     "Leeward-Islands",
		Country:   "PF",
		Elevation: 3,
		Latitude:  -16.7229003906,
		Longitude: -151.466003418,
		Timezone:  "Pacific/Tahiti",
	},
	"NTTX": {
		ICAO:      "NTTX",
		IATA:      "UOA",
		Name:      "Mururoa Atoll Airport",
		City:      "Mururoa Atoll",
		State:     "Iles-Tuamotu-Gambier",
		Country:   "PF",
		Elevation: 7,
		Latitude:  -21.8670005798,
		Longitude: -138.9170074463,
		Timezone:  "Pacific/Tahiti",
	},
	"NTUV": {
		ICAO:      "NTUV",
		IATA:      "VHZ",
		Name:      "Vahitahi Airport",
		City:      "Vahitahi",
		State:     "Iles-Tuamotu-Gambier",
		Country:   "PF",
		Elevation: 9,
		Latitude:  -18.7800006866,
		Longitude: -138.8529968262,
		Timezone:  "Pacific/Tahiti",
	},
	"NV00": {
		ICAO:      "NV00",
		Name:      "Valley View Airport",
		City:      "Pahrump",
		State:     "Nevada",
		Country:   "US",
		Elevation: 2740,
		Latitude:  36.2555007935,
		Longitude: -115.9929962158,
		Timezone:  "America/Los_Angeles",
	},
	"NV02": {
		ICAO:      "NV02",
		Name:      "O'Toole Ranch Airport",
		City:      "Austin",
		State:     "Nevada",
		Country:   "US",
		Elevation: 6520,
		Latitude:  39.0666007996,
		Longitude: -117.4219970703,
		Timezone:  "America/Los_Angeles",
	},
	"NV04": {
		ICAO:      "NV04",
		Name:      "Marys River Ranch Airport",
		City:      "Deeth",
		State:     "Nevada",
		Country:   "US",
		Elevation: 5600,
		Latitude:  41.3162994385,
		Longitude: -115.25,
		Timezone:  "America/Los_Angeles",
	},
	"NV05": {
		ICAO:      "NV05",
		Name:      "Soldier Meadow Nr 2 Airport",
		City:      "Gerlach",
		State:     "Nevada",
		Country:   "US",
		Elevation: 4150,
		Latitude:  41.123500824,
		Longitude: -119.138999939,
		Timezone:  "America/Los_Angeles",
	},
	"NV06": {
		ICAO:      "NV06",
		Name:      "Soldier Meadow Nr 1 Airport",
		City:      "Gerlach",
		State:     "Nevada",
		Country:   "US",
		Elevation: 4494,
		Latitude:  41.3827018738,
		Longitude: -119.1689987183,
		Timezone:  "America/Los_Angeles",
	},
	"NV07": {
		ICAO:      "NV07",
		Name:      "Sunnyside/Kirch Wildlife Mgmt Area Airport",
		City:      "Lund",
		State:     "Nevada",
		Country:   "US",
		Elevation: 5288,
		Latitude:  38.4146995544,
		Longitude: -115.0370025635,
		Timezone:  "America/Los_Angeles",
	},
	"NV08": {
		ICAO:      "NV08",
		Name:      "Petan Ranch Airport",
		City:      "Mountain City",
		State:     "Nevada",
		Country:   "US",
		Elevation: 5616,
		Latitude:  41.7379989624,
		Longitude: -116.216003418,
		Timezone:  "America/Los_Angeles",
	},
	"NV09": {
		ICAO:      "NV09",
		Name:      "H Bar H Airport",
		City:      "Reno",
		State:     "Nevada",
		Country:   "US",
		Elevation: 5220,
		Latitude:  39.7243995667,
		Longitude: -119.8990020752,
		Timezone:  "America/Los_Angeles",
	},
	"NV11": {
		ICAO:      "NV11",
		Name:      "Yucca Airstrip",
		City:      "Mercury",
		State:     "Nevada",
		Country:   "US",
		Elevation: 3919,
		Latitude:  36.9458007813,
		Longitude: -116.0380020142,
		Timezone:  "America/Los_Angeles",
	},
	"NV12": {
		ICAO:      "NV12",
		Name:      "I-L Ranch Airport",
		City:      "Tuscarora",
		State:     "Nevada",
		Country:   "US",
		Elevation: 5368,
		Latitude:  41.5638008118,
		Longitude: -116.408996582,
		Timezone:  "America/Los_Angeles",
	},
	"NV13": {
		ICAO:      "NV13",
		Name:      "Bailey Ranch Airport",
		City:      "Carson City",
		State:     "Nevada",
		Country:   "US",
		Elevation: 5060,
		Latitude:  39.2827987671,
		Longitude: -119.8280029297,
		Timezone:  "America/Los_Angeles",
	},
	"NV14": {
		ICAO:      "NV14",
		Name:      "Juniper Airport",
		City:      "Montello",
		State:     "Nevada",
		Country:   "US",
		Elevation: 5400,
		Latitude:  41.3554992676,
		Longitude: -114.2300033569,
		Timezone:  "America/Los_Angeles",
	},
	"NV16": {
		ICAO:      "NV16",
		Name:      "Lackerman Ranch Airport",
		City:      "Gerlach",
		State:     "Nevada",
		Country:   "US",
		Elevation: 4706,
		Latitude:  40.9992980957,
		Longitude: -119.93699646,
		Timezone:  "America/Los_Angeles",
	},
	"NV17": {
		ICAO:      "NV17",
		Name:      "Youngberg Ranch Airport",
		City:      "Lemmon Valley",
		State:     "Nevada",
		Country:   "US",
		Elevation: 4960,
		Latitude:  39.6731987,
		Longitude: -119.81300354,
		Timezone:  "America/Los_Angeles",
	},
	"NV22": {
		ICAO:      "NV22",
		Name:      "Red Rock Ranch Airport",
		City:      "Elko",
		State:     "Nevada",
		Country:   "US",
		Elevation: 6158,
		Latitude:  40.3126983643,
		Longitude: -115.8970031738,
		Timezone:  "America/Los_Angeles",
	},
	"NV23": {
		ICAO:      "NV23",
		Name:      "Air Sailing Gliderport",
		City:      "Sparks",
		State:     "Nevada",
		Country:   "US",
		Elevation: 4300,
		Latitude:  39.8721008301,
		Longitude: -119.7009963989,
		Timezone:  "America/Los_Angeles",
	},
	"NV24": {
		ICAO:      "NV24",
		Name:      "Fort Ruby Ranch Airstrip",
		City:      "Ruby Valley",
		State:     "Nevada",
		Country:   "US",
		Elevation: 6006,
		Latitude:  40.0579986572,
		Longitude: -115.5059967041,
		Timezone:  "America/Los_Angeles",
	},
	"NV26": {
		ICAO:      "NV26",
		Name:      "Voc Tech Airport",
		City:      "Henderson",
		State:     "Nevada",
		Country:   "US",
		Elevation: 1913,
		Latitude:  36.0791015625,
		Longitude: -115.0690002441,
		Timezone:  "America/Los_Angeles",
	},
	"NV27": {
		ICAO:      "NV27",
		Name:      "Circle L Ranch Airport",
		City:      "Dyer",
		State:     "Nevada",
		Country:   "US",
		Elevation: 4835,
		Latitude:  37.7165985107,
		Longitude: -118.0839996338,
		Timezone:  "America/Los_Angeles",
	},
	"NV30": {
		ICAO:      "NV30",
		Name:      "Dixie Valley Airport",
		City:      "Fallon",
		State:     "Nevada",
		Country:   "US",
		Elevation: 3443,
		Latitude:  39.9665985107,
		Longitude: -117.8280029297,
		Timezone:  "America/Los_Angeles",
	},
	"NV31": {
		ICAO:      "NV31",
		Name:      "Barker Creek Ranch Airstrip",
		City:      "Round Mountain",
		State:     "Nevada",
		Country:   "US",
		Elevation: 6620,
		Latitude:  38.8058013916,
		Longitude: -117.03099823,
		Timezone:  "America/Los_Angeles",
	},
	"NV33": {
		ICAO:      "NV33",
		Name:      "Farias Wheel Airport",
		City:      "Smith Valley",
		State:     "Nevada",
		Country:   "US",
		Elevation: 4848,
		Latitude:  38.8331985474,
		Longitude: -119.3960037231,
		Timezone:  "America/Los_Angeles",
	},
	"NV35": {
		ICAO:      "NV35",
		Name:      "Hudson Airport",
		City:      "Austin",
		State:     "Nevada",
		Country:   "US",
		Elevation: 5180,
		Latitude:  39.5443992615,
		Longitude: -117.7509994507,
		Timezone:  "America/Los_Angeles",
	},
	"NV36": {
		ICAO:      "NV36",
		Name:      "Imvite Airport",
		City:      "Amargosa Valley",
		State:     "Nevada",
		Country:   "US",
		Elevation: 2250,
		Latitude:  36.4477005005,
		Longitude: -116.4540023804,
		Timezone:  "America/Los_Angeles",
	},
	"NV41": {
		ICAO:      "NV41",
		Name:      "Sulfur Airport",
		City:      "Gerlach",
		State:     "Nevada",
		Country:   "US",
		Elevation: 4068,
		Latitude:  40.8638000488,
		Longitude: -118.7350006104,
		Timezone:  "America/Los_Angeles",
	},
	"NV42": {
		ICAO:      "NV42",
		Name:      "Heritage Airport",
		City:      "Jean",
		State:     "California",
		Country:   "US",
		Elevation: 2575,
		Latitude:  35.7757987976,
		Longitude: -115.6309967041,
		Timezone:  "America/Los_Angeles",
	},
	"NV44": {
		ICAO:      "NV44",
		Name:      "Justover Field",
		City:      "Sparks",
		State:     "Nevada",
		Country:   "US",
		Elevation: 4480,
		Latitude:  39.7551994324,
		Longitude: -119.5989990234,
		Timezone:  "America/Los_Angeles",
	},
	"NV47": {
		ICAO:      "NV47",
		Name:      "Palomino Airport",
		City:      "Sparks",
		State:     "Nevada",
		Country:   "US",
		Elevation: 4240,
		Latitude:  39.8331985474,
		Longitude: -119.6819992065,
		Timezone:  "America/Los_Angeles",
	},
	"NV54": {
		ICAO:      "NV54",
		Name:      "Flying S Ranch Ultralightport",
		City:      "Pahrump",
		State:     "Nevada",
		Country:   "US",
		Elevation: 2800,
		Latitude:  36.3069992065,
		Longitude: -115.9889984131,
		Timezone:  "America/Los_Angeles",
	},
	"NV55": {
		ICAO:      "NV55",
		Name:      "Pinenut Airport",
		City:      "Gardnerville",
		State:     "Nevada",
		Country:   "US",
		Elevation: 5260,
		Latitude:  38.8988990784,
		Longitude: -119.6490020752,
		Timezone:  "America/Los_Angeles",
	},
	"NV56": {
		ICAO:      "NV56",
		Name:      "Wine Glass Ranch Airport",
		City:      "Round Mountain",
		State:     "Nevada",
		Country:   "US",
		Elevation: 5568,
		Latitude:  38.7994003296,
		Longitude: -117.1750030518,
		Timezone:  "America/Los_Angeles",
	},
	"NV64": {
		ICAO:      "NV64",
		Name:      "Swanson Ranch 3 Airport",
		City:      "Battle Mountain",
		State:     "Nevada",
		Country:   "US",
		Elevation: 4940,
		Latitude:  40.0083007813,
		Longitude: -117.391998291,
		Timezone:  "America/Los_Angeles",
	},
	"NV67": {
		ICAO:      "NV67",
		Name:      "Pilot Creek Ranches Airport",
		City:      "Montello",
		State:     "Nevada",
		Country:   "US",
		Elevation: 4980,
		Latitude:  41.1157989502,
		Longitude: -114.1139984131,
		Timezone:  "America/Los_Angeles",
	},
	"NV68": {
		ICAO:      "NV68",
		Name:      "Pine Grove Airport",
		City:      "Orovada",
		State:     "Nevada",
		Country:   "US",
		Elevation: 4250,
		Latitude:  41.565700531,
		Longitude: -117.8089981079,
		Timezone:  "America/Los_Angeles",
	},
	"NV72": {
		ICAO:      "NV72",
		Name:      "Sweetwater (USMC) Airport",
		City:      "Wellington",
		State:     "Nevada",
		Country:   "US",
		Elevation: 6837,
		Latitude:  38.5097999573,
		Longitude: -119.217002869,
		Timezone:  "America/Los_Angeles",
	},
	"NV74": {
		ICAO:      "NV74",
		Name:      "Calvada Meadows Airport",
		City:      "Pahrump",
		State:     "Nevada",
		Country:   "US",
		Elevation: 2726,
		Latitude:  36.2710990906,
		Longitude: -115.9950027466,
		Timezone:  "America/Los_Angeles",
	},
	"NV77": {
		ICAO:      "NV77",
		Name:      "Empire Farms Airport",
		City:      "Empire",
		State:     "Nevada",
		Country:   "US",
		Elevation: 4085,
		Latitude:  40.4356994629,
		Longitude: -119.4619979858,
		Timezone:  "America/Los_Angeles",
	},
	"NV83": {
		ICAO:      "NV83",
		Name:      "Hadley Airport",
		City:      "Round Mountain",
		State:     "Nevada",
		Country:   "US",
		Elevation: 5744,
		Latitude:  38.694098,
		Longitude: -117.146752,
		Timezone:  "America/Los_Angeles",
	},
	"NV96": {
		ICAO:      "NV96",
		Name:      "Rolling Thunder Airport",
		City:      "Sparks",
		State:     "Nevada",
		Country:   "US",
		Elevation: 4240,
		Latitude:  39.8252983093,
		Longitude: -119.6539993286,
		Timezone:  "America/Los_Angeles",
	},
	"NV97": {
		ICAO:      "NV97",
		Name:      "Desert Creek Airport",
		City:      "Wellington",
		State:     "Nevada",
		Country:   "US",
		Elevation: 5358,
		Latitude:  38.6842002869,
		Longitude: -119.3219985962,
		Timezone:  "America/Los_Angeles",
	},
	"NV98": {
		ICAO:      "NV98",
		Name:      "Caas Airport",
		City:      "Pahrump",
		State:     "Nevada",
		Country:   "US",
		Elevation: 2800,
		Latitude:  36.1502990723,
		Longitude: -115.9000015259,
		Timezone:  "America/Los_Angeles",
	},
	"NVSA": {
		ICAO:      "NVSA",
		IATA:      "MTV",
		Name:      "Mota Lava Airport",
		City:      "Ablow",
		Country:   "VU",
		Elevation: 63,
		Latitude:  -13.6660003662,
		Longitude: 167.7120056152,
		Timezone:  "Pacific/Efate",
	},
	"NVSC": {
		ICAO:      "NVSC",
		IATA:      "SLH",
		Name:      "Sola Airport",
		City:      "Sola",
		State:     "Torba",
		Country:   "VU",
		Elevation: 7,
		Latitude:  -13.8516998291,
		Longitude: 167.5370025635,
		Timezone:  "Pacific/Efate",
	},
	"NVSD": {
		ICAO:      "NVSD",
		IATA:      "TOH",
		Name:      "Torres Airstrip",
		City:      "Loh/Linua",
		State:     "Torba",
		Country:   "VU",
		Latitude:  -13.3280000687,
		Longitude: 166.6380004883,
		Timezone:  "Pacific/Efate",
	},
	"NVSE": {
		ICAO:      "NVSE",
		IATA:      "EAE",
		Name:      "Sangafa Airport",
		City:      "Sangafa",
		Country:   "VU",
		Elevation: 7,
		Latitude:  -17.0902996063,
		Longitude: 168.3430023193,
		Timezone:  "Pacific/Efate",
	},
	"NVSF": {
		ICAO:      "NVSF",
		IATA:      "CCV",
		Name:      "Craig Cove Airport",
		City:      "Craig Cove",
		State:     "Malampa",
		Country:   "VU",
		Elevation: 69,
		Latitude:  -16.2649993896,
		Longitude: 167.9239959717,
		Timezone:  "Pacific/Efate",
	},
	"NVSG": {
		ICAO:      "NVSG",
		IATA:      "LOD",
		Name:      "Longana Airport",
		City:      "Longana",
		State:     "Penama",
		Country:   "VU",
		Elevation: 167,
		Latitude:  -15.3066997528,
		Longitude: 167.9669952393,
		Timezone:  "Pacific/Efate",
	},
	"NVSH": {
		ICAO:      "NVSH",
		IATA:      "SSR",
		Name:      "Sara Airport",
		City:      "Pentecost Island",
		State:     "Penama",
		Country:   "VU",
		Latitude:  -15.4708003998,
		Longitude: 168.1519927979,
		Timezone:  "Pacific/Efate",
	},
	"NVSI": {
		ICAO:      "NVSI",
		IATA:      "PBJ",
		Name:      "Tavie Airport",
		City:      "Paama Island",
		State:     "Malampa",
		Country:   "VU",
		Latitude:  -16.438999176,
		Longitude: 168.2570037842,
		Timezone:  "Pacific/Efate",
	},
	"NVSL": {
		ICAO:      "NVSL",
		IATA:      "LPM",
		Name:      "Lamap Airport",
		City:      "Lamap",
		State:     "Malampa",
		Country:   "VU",
		Elevation: 7,
		Latitude:  -16.454000473,
		Longitude: 167.8229980469,
		Timezone:  "Pacific/Efate",
	},
	"NVSM": {
		ICAO:      "NVSM",
		IATA:      "LNB",
		Name:      "Lamen Bay Airport",
		City:      "Lamen Bay",
		State:     "Shefa",
		Country:   "VU",
		Elevation: 7,
		Latitude:  -16.5841999054,
		Longitude: 168.158996582,
		Timezone:  "Pacific/Efate",
	},
	"NVSN": {
		ICAO:      "NVSN",
		IATA:      "MWF",
		Name:      "Naone Airport",
		City:      "Maewo Island",
		State:     "Penama",
		Country:   "VU",
		Latitude:  -15,
		Longitude: 168.0829925537,
		Timezone:  "Pacific/Efate",
	},
	"NVSO": {
		ICAO:      "NVSO",
		IATA:      "LNE",
		Name:      "Lonorore Airport",
		City:      "Lonorore",
		State:     "Penama",
		Country:   "VU",
		Elevation: 43,
		Latitude:  -15.8655996323,
		Longitude: 168.1719970703,
		Timezone:  "Pacific/Efate",
	},
	"NVSP": {
		ICAO:      "NVSP",
		IATA:      "NUS",
		Name:      "Norsup Airport",
		City:      "Norsup",
		State:     "Malampa",
		Country:   "VU",
		Elevation: 23,
		Latitude:  -16.07970047,
		Longitude: 167.4010009766,
		Timezone:  "Pacific/Efate",
	},
	"NVSQ": {
		ICAO:      "NVSQ",
		IATA:      "ZGU",
		Name:      "Gaua Island Airport",
		City:      "Gaua Island",
		State:     "Torba",
		Country:   "VU",
		Latitude:  -14.2180995941,
		Longitude: 167.587005615,
		Timezone:  "Pacific/Efate",
	},
	"NVSR": {
		ICAO:      "NVSR",
		IATA:      "RCL",
		Name:      "Redcliffe Airport",
		City:      "Redcliffe",
		State:     "Penama",
		Country:   "VU",
		Elevation: 36,
		Latitude:  -15.4720001221,
		Longitude: 167.8350067139,
		Timezone:  "Pacific/Efate",
	},
	"NVSS": {
		ICAO:      "NVSS",
		IATA:      "SON",
		Name:      "Santo Pekoa International Airport",
		City:      "Luganville",
		State:     "Sanma",
		Country:   "VU",
		Elevation: 184,
		Latitude:  -15.5050001144,
		Longitude: 167.220001221,
		Timezone:  "Pacific/Efate",
	},
	"NVST": {
		ICAO:      "NVST",
		IATA:      "TGH",
		Name:      "Tongoa Island Airport",
		City:      "Tongoa Island",
		State:     "Shefa",
		Country:   "VU",
		Elevation: 443,
		Latitude:  -16.8910999298,
		Longitude: 168.550994873,
		Timezone:  "Pacific/Efate",
	},
	"NVSU": {
		ICAO:      "NVSU",
		IATA:      "ULB",
		Name:      "Ulei Airport",
		City:      "Ambryn Island",
		State:     "Malampa",
		Country:   "VU",
		Latitude:  -16.3297,
		Longitude: 168.3011,
		Timezone:  "Pacific/Efate",
	},
	"NVSV": {
		ICAO:      "NVSV",
		IATA:      "VLS",
		Name:      "Valesdir Airport",
		City:      "Valesdir",
		State:     "Shefa",
		Country:   "VU",
		Elevation: 10,
		Latitude:  -16.7961006165,
		Longitude: 168.1770019531,
		Timezone:  "Pacific/Efate",
	},
	"NVSW": {
		ICAO:      "NVSW",
		IATA:      "WLH",
		Name:      "Walaha Airport",
		City:      "Walaha",
		State:     "Penama",
		Country:   "VU",
		Elevation: 151,
		Latitude:  -15.4119997025,
		Longitude: 167.6909942627,
		Timezone:  "Pacific/Efate",
	},
	"NVSX": {
		ICAO:      "NVSX",
		IATA:      "SWJ",
		Name:      "Southwest Bay Airport",
		City:      "Malekula Island",
		State:     "Malampa",
		Country:   "VU",
		Latitude:  -16.4864,
		Longitude: 167.4472,
		Timezone:  "Pacific/Efate",
	},
	"NVSZ": {
		ICAO:      "NVSZ",
		IATA:      "OLJ",
		Name:      "North West Santo Airport",
		City:      "Olpoi",
		State:     "Sanma",
		Country:   "VU",
		Latitude:  -14.8816995621,
		Longitude: 166.557998657,
		Timezone:  "Pacific/Efate",
	},
	"NVVA": {
		ICAO:      "NVVA",
		IATA:      "AUY",
		Name:      "Anelghowhat Airport",
		City:      "Anelghowhat",
		State:     "Tafea",
		Country:   "VU",
		Elevation: 7,
		Latitude:  -20.2492008209,
		Longitude: 169.7709960938,
		Timezone:  "Pacific/Efate",
	},
	"NVVB": {
		ICAO:      "NVVB",
		IATA:      "AWD",
		Name:      "Aniwa Airport",
		City:      "Aniwa",
		State:     "Tafea",
		Country:   "VU",
		Elevation: 69,
		Latitude:  -19.2399997711,
		Longitude: 169.6049957275,
		Timezone:  "Pacific/Efate",
	},
	"NVVD": {
		ICAO:      "NVVD",
		IATA:      "DLY",
		Name:      "Dillon's Bay Airport",
		City:      "Dillon's Bay",
		State:     "Tafea",
		Country:   "VU",
		Elevation: 538,
		Latitude:  -18.7693996429,
		Longitude: 169.0010070801,
		Timezone:  "Pacific/Efate",
	},
	"NVVF": {
		ICAO:      "NVVF",
		IATA:      "FTA",
		Name:      "Futuna Airport",
		City:      "Futuna Island",
		State:     "Tafea",
		Country:   "VU",
		Elevation: 95,
		Latitude:  -19.5163993835,
		Longitude: 170.231994629,
		Timezone:  "Pacific/Efate",
	},
	"NVVI": {
		ICAO:      "NVVI",
		IATA:      "IPA",
		Name:      "Ipota Airport",
		City:      "Ipota",
		State:     "Tafea",
		Country:   "VU",
		Elevation: 23,
		Latitude:  -18.856389,
		Longitude: 169.283333,
		Timezone:  "Pacific/Efate",
	},
	"NVVJ": {
		ICAO:      "NVVJ",
		Name:      "Forari Airport",
		City:      "Forari",
		State:     "Shefa",
		Country:   "VU",
		Elevation: 351,
		Latitude:  -17.7010993958,
		Longitude: 168.5269927979,
		Timezone:  "Pacific/Efate",
	},
	"NVVK": {
		ICAO:      "NVVK",
		Name:      "Lenakel Airport",
		City:      "Lenakel",
		State:     "Tafea",
		Country:   "VU",
		Elevation: 230,
		Latitude:  -19.5130004883,
		Longitude: 169.2599945068,
		Timezone:  "Pacific/Efate",
	},
	"NVVQ": {
		ICAO:      "NVVQ",
		IATA:      "UIQ",
		Name:      "Quion Hill Airport",
		City:      "Quion Hill",
		State:     "Shefa",
		Country:   "VU",
		Elevation: 16,
		Latitude:  -17.5400009155,
		Longitude: 168.4420013428,
		Timezone:  "Pacific/Efate",
	},
	"NVVV": {
		ICAO:      "NVVV",
		IATA:      "VLI",
		Name:      "Port Vila Bauerfield Airport",
		City:      "Port Vila",
		State:     "Shefa",
		Country:   "VU",
		Elevation: 70,
		Latitude:  -17.699300766,
		Longitude: 168.3200073242,
		Timezone:  "Pacific/Efate",
	},
	"NVVW": {
		ICAO:      "NVVW",
		IATA:      "TAH",
		Name:      "Tanna Airport",
		State:     "Tafea",
		Country:   "VU",
		Elevation: 19,
		Latitude:  -19.4550991058,
		Longitude: 169.2239990234,
		Timezone:  "Pacific/Efate",
	},
	"NWWA": {
		ICAO:      "NWWA",
		IATA:      "TGJ",
		Name:      "Tiga Airport",
		City:      "Tiga",
		Country:   "NC",
		Elevation: 128,
		Latitude:  -21.0960998535,
		Longitude: 167.8040008545,
		Timezone:  "Pacific/Noumea",
	},
	"NWWB": {
		ICAO:      "NWWB",
		Name:      "Bourail - Poe Airport",
		City:      "Bourail / Poe",
		Country:   "NC",
		Elevation: 7,
		Latitude:  -21.6096992493,
		Longitude: 165.3970031738,
		Timezone:  "Pacific/Noumea",
	},
	"NWWC": {
		ICAO:      "NWWC",
		IATA:      "BMY",
		Name:      "Ile Art - Waala Airport",
		City:      "Waala",
		State:     "North-Province",
		Country:   "NC",
		Elevation: 306,
		Latitude:  -19.7206001282,
		Longitude: 163.6609954834,
		Timezone:  "Pacific/Noumea",
	},
	"NWWD": {
		ICAO:      "NWWD",
		IATA:      "KNQ",
		Name:      "Kone Airport",
		City:      "Kone",
		Country:   "NC",
		Elevation: 23,
		Latitude:  -21.0543003082,
		Longitude: 164.8370056152,
		Timezone:  "Pacific/Noumea",
	},
	"NWWE": {
		ICAO:      "NWWE",
		IATA:      "ILP",
		Name:      "Ile des Pins Airport",
		City:      "Ile des Pins",
		Country:   "NC",
		Elevation: 315,
		Latitude:  -22.5888996124,
		Longitude: 167.4559936523,
		Timezone:  "Pacific/Noumea",
	},
	"NWWF": {
		ICAO:      "NWWF",
		Name:      "Voh Airport",
		City:      "Voh",
		State:     "North-Province",
		Country:   "NC",
		Elevation: 33,
		Latitude:  -20.966999054,
		Longitude: 164.6999969482,
		Timezone:  "Pacific/Noumea",
	},
	"NWWH": {
		ICAO:      "NWWH",
		IATA:      "HLU",
		Name:      "Nesson Airport",
		City:      "Houailou",
		Country:   "NC",
		Elevation: 10,
		Latitude:  -21.2563991547,
		Longitude: 165.6170043945,
		Timezone:  "Pacific/Noumea",
	},
	"NWWK": {
		ICAO:      "NWWK",
		IATA:      "KOC",
		Name:      "Koumac Airport",
		City:      "Koumac",
		Country:   "NC",
		Elevation: 42,
		Latitude:  -20.5463008881,
		Longitude: 164.2559967041,
		Timezone:  "Pacific/Noumea",
	},
	"NWWL": {
		ICAO:      "NWWL",
		IATA:      "LIF",
		Name:      "Lifou Airport",
		City:      "Lifou",
		State:     "Loyalty-Islands",
		Country:   "NC",
		Elevation: 92,
		Latitude:  -20.7747993469,
		Longitude: 167.2400054932,
		Timezone:  "Pacific/Noumea",
	},
	"NWWM": {
		ICAO:      "NWWM",
		IATA:      "GEA",
		Name:      "Noumea Magenta Airport",
		City:      "Noumea",
		State:     "South-Province",
		Country:   "NC",
		Elevation: 10,
		Latitude:  -22.2583007813,
		Longitude: 166.4730072021,
		Timezone:  "Pacific/Noumea",
	},
	"NWWO": {
		ICAO:      "NWWO",
		IATA:      "IOU",
		Name:      "Edmond Cane Airport",
		City:      "Ile Ouen",
		Country:   "NC",
		Elevation: 7,
		Latitude:  -22.4599990845,
		Longitude: 166.7830047607,
		Timezone:  "Pacific/Noumea",
	},
	"NWWP": {
		ICAO:      "NWWP",
		IATA:      "PUV",
		Name:      "Poum Airport",
		City:      "Poum",
		Country:   "NC",
		Elevation: 6,
		Latitude:  -20.2891998291,
		Longitude: 164.0989990234,
		Timezone:  "Pacific/Noumea",
	},
	"NWWQ": {
		ICAO:      "NWWQ",
		IATA:      "PDC",
		Name:      "Mueo Airport",
		City:      "Mueo",
		Country:   "NC",
		Latitude:  -21.316400528,
		Longitude: 164.9989929199,
		Timezone:  "Pacific/Noumea",
	},
	"NWWR": {
		ICAO:      "NWWR",
		IATA:      "MEE",
		Name:      "Mare Airport",
		City:      "Mare",
		State:     "Loyalty-Islands",
		Country:   "NC",
		Elevation: 141,
		Latitude:  -21.4817008972,
		Longitude: 168.0379943848,
		Timezone:  "Pacific/Noumea",
	},
	"NWWS": {
		ICAO:      "NWWS",
		Name:      "Plaine des Lacs Airport",
		City:      "Plaine des Lacs",
		Country:   "NC",
		Elevation: 902,
		Latitude:  -22.2670001984,
		Longitude: 166.9170074463,
		Timezone:  "Pacific/Noumea",
	},
	"NWWT": {
		ICAO:      "NWWT",
		Name:      "Oua Tom Airport",
		City:      "La Foa",
		Country:   "NC",
		Elevation: 98,
		Latitude:  -21.8197002411,
		Longitude: 165.8609924316,
		Timezone:  "Pacific/Noumea",
	},
	"NWWU": {
		ICAO:      "NWWU",
		IATA:      "TOU",
		Name:      "Touho Airport",
		City:      "Touho",
		Country:   "NC",
		Elevation: 10,
		Latitude:  -20.7900009155,
		Longitude: 165.2590026855,
		Timezone:  "Pacific/Noumea",
	},
	"NWWV": {
		ICAO:      "NWWV",
		IATA:      "UVE",
		Name:      "Ouvea Airport",
		City:      "Ouvea",
		State:     "Loyalty-Islands",
		Country:   "NC",
		Elevation: 23,
		Latitude:  -20.6406002045,
		Longitude: 166.5729980469,
		Timezone:  "Pacific/Noumea",
	},
	"NWWW": {
		ICAO:      "NWWW",
		IATA:      "NOU",
		Name:      "La Tontouta International Airport",
		City:      "Noumea",
		State:     "South-Province",
		Country:   "NC",
		Elevation: 52,
		Latitude:  -22.0146007538,
		Longitude: 166.2129974365,
		Timezone:  "Pacific/Noumea",
	},
	"NWWX": {
		ICAO:      "NWWX",
		Name:      "Canala Airport",
		City:      "Canala",
		State:     "North-Province",
		Country:   "NC",
		Elevation: 7,
		Latitude:  -21.5202007294,
		Longitude: 165.9720001221,
		Timezone:  "Pacific/Noumea",
	},
	"NWWY": {
		ICAO:      "NWWY",
		Name:      "Ouaco Airport",
		City:      "Ouaco",
		Country:   "NC",
		Elevation: 69,
		Latitude:  -20.8500003815,
		Longitude: 164.5330047607,
		Timezone:  "Pacific/Noumea",
	},
	"NX01": {
		ICAO:      "NX01",
		Name:      "Cole Landing Zone Airport",
		State:     "Arkansas",
		Country:   "US",
		Elevation: 811,
		Latitude:  35.2779006958,
		Longitude: -94.2242965698,
		Timezone:  "America/Chicago",
	},
	"NY00": {
		ICAO:      "NY00",
		Name:      "Basher Field",
		City:      "Sardinia",
		State:     "New-York",
		Country:   "US",
		Elevation: 1855,
		Latitude:  42.5666999817,
		Longitude: -78.556098938,
		Timezone:  "America/New_York",
	},
	"NY01": {
		ICAO:      "NY01",
		Name:      "Knowlesville Airport",
		City:      "Knowlesville",
		State:     "New-York",
		Country:   "US",
		Elevation: 555,
		Latitude:  43.2209014893,
		Longitude: -78.3150024414,
		Timezone:  "America/New_York",
	},
	"NY02": {
		ICAO:      "NY02",
		Name:      "Mountain Top Airport",
		City:      "Davenport",
		State:     "New-York",
		Country:   "US",
		Elevation: 1960,
		Latitude:  42.4873008728,
		Longitude: -74.7801971436,
		Timezone:  "America/New_York",
	},
	"NY03": {
		ICAO:      "NY03",
		Name:      "Skytop Airport",
		City:      "Roscoe",
		State:     "New-York",
		Country:   "US",
		Elevation: 2094,
		Latitude:  41.8750991821,
		Longitude: -74.9802017212,
		Timezone:  "America/New_York",
	},
	"NY04": {
		ICAO:      "NY04",
		Name:      "East Arcade Airport",
		City:      "Arcade",
		State:     "New-York",
		Country:   "US",
		Elevation: 1830,
		Latitude:  42.5856018066,
		Longitude: -78.3491973877,
		Timezone:  "America/New_York",
	},
	"NY05": {
		ICAO:      "NY05",
		Name:      "Hogan Airport",
		City:      "Esperance",
		State:     "New-York",
		Country:   "US",
		Elevation: 1260,
		Latitude:  42.78150177,
		Longitude: -74.3256988525,
		Timezone:  "America/New_York",
	},
	"NY06": {
		ICAO:      "NY06",
		Name:      "Gaines Valley Aviation Airport",
		City:      "Albion",
		State:     "New-York",
		Country:   "US",
		Elevation: 380,
		Latitude:  43.2963981628,
		Longitude: -78.2138977051,
		Timezone:  "America/New_York",
	},
	"NY07": {
		ICAO:      "NY07",
		Name:      "Hurlbut Field",
		City:      "Canton",
		State:     "New-York",
		Country:   "US",
		Elevation: 500,
		Latitude:  44.5457992554,
		Longitude: -75.1611022949,
		Timezone:  "America/New_York",
	},
	"NY09": {
		ICAO:      "NY09",
		Name:      "Middle Hope Airport",
		City:      "Newburgh",
		State:     "New-York",
		Country:   "US",
		Elevation: 450,
		Latitude:  41.5611991882,
		Longitude: -74.0162963867,
		Timezone:  "America/New_York",
	},
	"NY10": {
		ICAO:      "NY10",
		Name:      "Duflo Airport",
		City:      "New Bremen",
		State:     "New-York",
		Country:   "US",
		Elevation: 789,
		Latitude:  43.8426017761,
		Longitude: -75.4307022095,
		Timezone:  "America/New_York",
	},
	"NY11": {
		ICAO:      "NY11",
		Name:      "Lakestone Farm Airport",
		City:      "Macedon",
		State:     "New-York",
		Country:   "US",
		Elevation: 520,
		Latitude:  43.0353012085,
		Longitude: -77.3079986572,
		Timezone:  "America/New_York",
	},
	"NY13": {
		ICAO:      "NY13",
		Name:      "D'Amico Airport",
		City:      "Lyons",
		State:     "New-York",
		Country:   "US",
		Elevation: 515,
		Latitude:  43.0917015076,
		Longitude: -76.9905014038,
		Timezone:  "America/New_York",
	},
	"NY15": {
		ICAO:      "NY15",
		Name:      "Lakeville Airport",
		City:      "Lakeville",
		State:     "New-York",
		Country:   "US",
		Elevation: 935,
		Latitude:  42.826499939,
		Longitude: -77.7144012451,
		Timezone:  "America/New_York",
	},
	"NY16": {
		ICAO:      "NY16",
		Name:      "Oak Ridge Airport",
		City:      "East Palmyra",
		State:     "New-York",
		Country:   "US",
		Elevation: 540,
		Latitude:  43.098400116,
		Longitude: -77.1819000244,
		Timezone:  "America/New_York",
	},
	"NY17": {
		ICAO:      "NY17",
		Name:      "Adirondack Airpark Estates Airport",
		City:      "Plattsburgh",
		State:     "New-York",
		Country:   "US",
		Elevation: 1352,
		Latitude:  44.6091995239,
		Longitude: -73.8579025269,
		Timezone:  "America/New_York",
	},
	"NY18": {
		ICAO:      "NY18",
		Name:      "Neno International Airport",
		City:      "Ithaca",
		State:     "New-York",
		Country:   "US",
		Elevation: 1283,
		Latitude:  42.4486999512,
		Longitude: -76.616897583,
		Timezone:  "America/New_York",
	},
	"NY19": {
		ICAO:      "NY19",
		Name:      "Walls Airport",
		City:      "Elbridge",
		State:     "New-York",
		Country:   "US",
		Elevation: 610,
		Latitude:  43.0439987183,
		Longitude: -76.4618988037,
		Timezone:  "America/New_York",
	},
	"NY20": {
		ICAO:      "NY20",
		Name:      "Nellis Field",
		City:      "Fort Plain",
		State:     "New-York",
		Country:   "US",
		Elevation: 700,
		Latitude:  42.9584007263,
		Longitude: -74.6788024902,
		Timezone:  "America/New_York",
	},
	"NY23": {
		ICAO:      "NY23",
		Name:      "Mohawk Aviation Center Airport",
		City:      "Mohawk",
		State:     "New-York",
		Country:   "US",
		Elevation: 1210,
		Latitude:  42.9875984192,
		Longitude: -74.9664993286,
		Timezone:  "America/New_York",
	},
	"NY24": {
		ICAO:      "NY24",
		Name:      "Taylor Johnson Airport",
		City:      "Niagara Falls",
		State:     "New-York",
		Country:   "US",
		Elevation: 633,
		Latitude:  43.1277999878,
		Longitude: -78.9700012207,
		Timezone:  "America/New_York",
	},
	"NY25": {
		ICAO:      "NY25",
		Name:      "F&F Airpark Airport",
		City:      "Oneonta",
		State:     "New-York",
		Country:   "US",
		Elevation: 1110,
		Latitude:  42.4500999451,
		Longitude: -75.0162963867,
		Timezone:  "America/New_York",
	},
	"NY26": {
		ICAO:      "NY26",
		Name:      "Sky-Ranch Airport",
		City:      "Herkimer",
		State:     "New-York",
		Country:   "US",
		Elevation: 1330,
		Latitude:  42.9833984375,
		Longitude: -74.899597168,
		Timezone:  "America/New_York",
	},
	"NY27": {
		ICAO:      "NY27",
		Name:      "South Dayton Airport",
		City:      "South Dayton",
		State:     "New-York",
		Country:   "US",
		Elevation: 1300,
		Latitude:  42.3708992004,
		Longitude: -79.0094985962,
		Timezone:  "America/New_York",
	},
	"NY28": {
		ICAO:      "NY28",
		Name:      "Anthonson Airport",
		City:      "Jordan",
		State:     "New-York",
		Country:   "US",
		Elevation: 430,
		Latitude:  43.0708999634,
		Longitude: -76.4079971313,
		Timezone:  "America/New_York",
	},
	"NY29": {
		ICAO:      "NY29",
		Name:      "Schuyler Airport",
		City:      "Watkins Glen",
		State:     "New-York",
		Country:   "US",
		Elevation: 870,
		Latitude:  42.4519996643,
		Longitude: -76.9115982056,
		Timezone:  "America/New_York",
	},
	"NY31": {
		ICAO:      "NY31",
		Name:      "Rabbit Lane Airport",
		City:      "Phoenix",
		State:     "New-York",
		Country:   "US",
		Elevation: 405,
		Latitude:  43.2237014771,
		Longitude: -76.3687973022,
		Timezone:  "America/New_York",
	},
	"NY32": {
		ICAO:      "NY32",
		Name:      "Fort Hill Airport",
		City:      "West Bloomfield",
		State:     "New-York",
		Country:   "US",
		Elevation: 915,
		Latitude:  42.9020004272,
		Longitude: -77.5477981567,
		Timezone:  "America/New_York",
	},
	"NY33": {
		ICAO:      "NY33",
		Name:      "Westerlo Airport",
		City:      "Westerlo",
		State:     "New-York",
		Country:   "US",
		Elevation: 1400,
		Latitude:  42.5223007202,
		Longitude: -74.0287017822,
		Timezone:  "America/New_York",
	},
	"NY34": {
		ICAO:      "NY34",
		Name:      "Randall's Roost Airport",
		City:      "Cameron",
		State:     "New-York",
		Country:   "US",
		Elevation: 1825,
		Latitude:  42.1622009277,
		Longitude: -77.4274978638,
		Timezone:  "America/New_York",
	},
	"NY35": {
		ICAO:      "NY35",
		Name:      "Stanton Airport",
		City:      "New Paltz",
		State:     "New-York",
		Country:   "US",
		Elevation: 303,
		Latitude:  41.7500991821,
		Longitude: -74.1163024902,
		Timezone:  "America/New_York",
	},
	"NY37": {
		ICAO:      "NY37",
		Name:      "Galway Airport",
		City:      "Galway",
		State:     "New-York",
		Country:   "US",
		Elevation: 950,
		Latitude:  43.0092010498,
		Longitude: -74.0912017822,
		Timezone:  "America/New_York",
	},
	"NY38": {
		ICAO:      "NY38",
		Name:      "Mc Bride's Airport",
		City:      "Gorham",
		State:     "New-York",
		Country:   "US",
		Elevation: 915,
		Latitude:  42.853099823,
		Longitude: -77.1247024536,
		Timezone:  "America/New_York",
	},
	"NY40": {
		ICAO:      "NY40",
		Name:      "Gentzke Aeronautical Park Airport",
		City:      "Holland",
		State:     "New-York",
		Country:   "US",
		Elevation: 1750,
		Latitude:  42.6105995178,
		Longitude: -78.5955963135,
		Timezone:  "America/New_York",
	},
	"NY42": {
		ICAO:      "NY42",
		Name:      "Paradise Airport",
		City:      "Red Creek",
		State:     "New-York",
		Country:   "US",
		Elevation: 420,
		Latitude:  43.2201004028,
		Longitude: -76.6899032593,
		Timezone:  "America/New_York",
	},
	"NY48": {
		ICAO:      "NY48",
		Name:      "Grace's Landing Airport",
		City:      "South Kortright",
		State:     "New-York",
		Country:   "US",
		Elevation: 1500,
		Latitude:  42.3381004333,
		Longitude: -74.7490005493,
		Timezone:  "America/New_York",
	},
	"NY49": {
		ICAO:      "NY49",
		Name:      "Mesmer Airport",
		City:      "Niagara Falls",
		State:     "New-York",
		Country:   "US",
		Elevation: 600,
		Latitude:  42.9928016663,
		Longitude: -78.9863967896,
		Timezone:  "America/New_York",
	},
	"NY50": {
		ICAO:      "NY50",
		Name:      "Johnson Airport",
		City:      "Perth",
		State:     "New-York",
		Country:   "US",
		Elevation: 760,
		Latitude:  42.9916992188,
		Longitude: -74.136100769,
		Timezone:  "America/New_York",
	},
	"NY51": {
		ICAO:      "NY51",
		Name:      "Russell Airport",
		City:      "Canajoharie",
		State:     "New-York",
		Country:   "US",
		Elevation: 700,
		Latitude:  42.870098114,
		Longitude: -74.4992980957,
		Timezone:  "America/New_York",
	},
	"NY53": {
		ICAO:      "NY53",
		Name:      "Tri County Airways Airport",
		City:      "West Edmeston",
		State:     "New-York",
		Country:   "US",
		Elevation: 1150,
		Latitude:  42.7389984131,
		Longitude: -75.300201416,
		Timezone:  "America/New_York",
	},
	"NY55": {
		ICAO:      "NY55",
		Name:      "Grund Field",
		City:      "Ithaca",
		State:     "New-York",
		Country:   "US",
		Elevation: 1453,
		Latitude:  42.4472999573,
		Longitude: -76.5969009399,
		Timezone:  "America/New_York",
	},
	"NY57": {
		ICAO:      "NY57",
		Name:      "Remsen City Airport",
		City:      "Remsen",
		State:     "New-York",
		Country:   "US",
		Elevation: 1220,
		Latitude:  43.3600997925,
		Longitude: -75.1846008301,
		Timezone:  "America/New_York",
	},
	"NY58": {
		ICAO:      "NY58",
		Name:      "Snow Field",
		City:      "Amsterdam",
		State:     "New-York",
		Country:   "US",
		Elevation: 775,
		Latitude:  42.9566993713,
		Longitude: -74.0814971924,
		Timezone:  "America/New_York",
	},
	"NY59": {
		ICAO:      "NY59",
		Name:      "Valenty Mierek Airport",
		City:      "Ava",
		State:     "New-York",
		Country:   "US",
		Elevation: 1220,
		Latitude:  43.387298584,
		Longitude: -75.4862976074,
		Timezone:  "America/New_York",
	},
	"NY60": {
		ICAO:      "NY60",
		Name:      "Vasile Field",
		City:      "Beekmanton",
		State:     "New-York",
		Country:   "US",
		Elevation: 580,
		Latitude:  44.7625007629,
		Longitude: -73.5593032837,
		Timezone:  "America/New_York",
	},
	"NY62": {
		ICAO:      "NY62",
		Name:      "Curtis Airport",
		City:      "Verona",
		State:     "New-York",
		Country:   "US",
		Elevation: 535,
		Latitude:  43.1458015442,
		Longitude: -75.5419006348,
		Timezone:  "America/New_York",
	},
	"NY63": {
		ICAO:      "NY63",
		Name:      "Archdale Meadows Airport",
		City:      "Greenwich",
		State:     "New-York",
		Country:   "US",
		Elevation: 584,
		Latitude:  43.0532989502,
		Longitude: -73.478302002,
		Timezone:  "America/New_York",
	},
	"NY64": {
		ICAO:      "NY64",
		Name:      "The Pines Airport",
		City:      "Broadalbin",
		State:     "New-York",
		Country:   "US",
		Elevation: 850,
		Latitude:  43.1029014587,
		Longitude: -74.1550979614,
		Timezone:  "America/New_York",
	},
	"NY65": {
		ICAO:      "NY65",
		Name:      "Circle K Ranch Airport",
		City:      "Broadalbin",
		State:     "New-York",
		Country:   "US",
		Elevation: 920,
		Latitude:  43.137298584,
		Longitude: -74.1162033081,
		Timezone:  "America/New_York",
	},
	"NY66": {
		ICAO:      "NY66",
		Name:      "Lewis Field",
		City:      "Broadalbin",
		State:     "New-York",
		Country:   "US",
		Elevation: 900,
		Latitude:  43.1333999634,
		Longitude: -74.1162033081,
		Timezone:  "America/New_York",
	},
	"NY67": {
		ICAO:      "NY67",
		Name:      "Carter Flight Park Ultralightport",
		City:      "Manlius",
		State:     "New-York",
		Country:   "US",
		Elevation: 1301,
		Latitude:  42.9930992126,
		Longitude: -75.9116973877,
		Timezone:  "America/New_York",
	},
	"NY68": {
		ICAO:      "NY68",
		Name:      "Richfield Airport",
		City:      "Richfield Springs",
		State:     "New-York",
		Country:   "US",
		Elevation: 1450,
		Latitude:  42.8611984253,
		Longitude: -74.9453964233,
		Timezone:  "America/New_York",
	},
	"NY69": {
		ICAO:      "NY69",
		Name:      "John Gonzales Field",
		City:      "Cape Vincent",
		State:     "New-York",
		Country:   "US",
		Elevation: 260,
		Latitude:  44.120098114,
		Longitude: -76.3013000488,
		Timezone:  "America/New_York",
	},
	"NY70": {
		ICAO:      "NY70",
		Name:      "Scott'S Sky Ranch Airport",
		City:      "Mount Morris",
		State:     "New-York",
		Country:   "US",
		Elevation: 1221,
		Latitude:  42.648889,
		Longitude: -77.920278,
		Timezone:  "America/New_York",
	},
	"NY71": {
		ICAO:      "NY71",
		Name:      "O'Riley Airport",
		City:      "Fort Plain",
		State:     "New-York",
		Country:   "US",
		Elevation: 700,
		Latitude:  42.9756011963,
		Longitude: -74.7298965454,
		Timezone:  "America/New_York",
	},
	"NY72": {
		ICAO:      "NY72",
		Name:      "Poolsbrook Aerodrome",
		City:      "Manlius",
		State:     "New-York",
		Country:   "US",
		Elevation: 500,
		Latitude:  43.0722999573,
		Longitude: -75.9384994507,
		Timezone:  "America/New_York",
	},
	"NY73": {
		ICAO:      "NY73",
		Name:      "Miller Field",
		City:      "Coventryville",
		State:     "New-York",
		Country:   "US",
		Elevation: 1580,
		Latitude:  42.3180999756,
		Longitude: -75.5990982056,
		Timezone:  "America/New_York",
	},
	"NY74": {
		ICAO:      "NY74",
		Name:      "Deer Run Airport",
		City:      "Coxsackie",
		State:     "New-York",
		Country:   "US",
		Elevation: 320,
		Latitude:  42.3236999512,
		Longitude: -73.8731994629,
		Timezone:  "America/New_York",
	},
	"NY76": {
		ICAO:      "NY76",
		Name:      "Neverland Airport",
		City:      "Cattaraugus",
		State:     "New-York",
		Country:   "US",
		Elevation: 1901,
		Latitude:  42.3156661987,
		Longitude: -78.8953323364,
		Timezone:  "America/New_York",
	},
	"NY77": {
		ICAO:      "NY77",
		Name:      "Wandervogel Gliderport",
		City:      "Delanson",
		State:     "New-York",
		Country:   "US",
		Elevation: 1260,
		Latitude:  42.7181015015,
		Longitude: -74.1634979248,
		Timezone:  "America/New_York",
	},
	"NY78": {
		ICAO:      "NY78",
		Name:      "Downsville Airport",
		City:      "Downsville",
		State:     "New-York",
		Country:   "US",
		Elevation: 1087,
		Latitude:  42.0583992004,
		Longitude: -75.020401001,
		Timezone:  "America/New_York",
	},
	"NY79": {
		ICAO:      "NY79",
		Name:      "Keysa Airport",
		City:      "Bliss",
		State:     "New-York",
		Country:   "US",
		Elevation: 1620,
		Latitude:  42.596698761,
		Longitude: -78.3403015137,
		Timezone:  "America/New_York",
	},
	"NY81": {
		ICAO:      "NY81",
		Name:      "Heldeberg Airstrip",
		City:      "East Berne",
		State:     "New-York",
		Country:   "US",
		Elevation: 1237,
		Latitude:  42.6111984253,
		Longitude: -74.0647964478,
		Timezone:  "America/New_York",
	},
	"NY82": {
		ICAO:      "NY82",
		Name:      "Alexander Farm Airport",
		City:      "East Schodack",
		State:     "New-York",
		Country:   "US",
		Elevation: 525,
		Latitude:  42.5806007385,
		Longitude: -73.6162033081,
		Timezone:  "America/New_York",
	},
	"NY84": {
		ICAO:      "NY84",
		Name:      "Richter Aero Airport",
		City:      "Essex",
		State:     "New-York",
		Country:   "US",
		Elevation: 360,
		Latitude:  44.2792015076,
		Longitude: -73.3787002563,
		Timezone:  "America/New_York",
	},
	"NY85": {
		ICAO:      "NY85",
		Name:      "Hickory Hollow Airport",
		City:      "Floyd",
		State:     "New-York",
		Country:   "US",
		Elevation: 930,
		Latitude:  43.2495002747,
		Longitude: -75.3137969971,
		Timezone:  "America/New_York",
	},
	"NY86": {
		ICAO:      "NY86",
		Name:      "Waxwing Airport",
		City:      "Rensselaerville",
		State:     "New-York",
		Country:   "US",
		Elevation: 1680,
		Latitude:  42.5125999451,
		Longitude: -74.1148986816,
		Timezone:  "America/New_York",
	},
	"NY87": {
		ICAO:      "NY87",
		Name:      "Amsterdam Airfield",
		City:      "Fort Johnson",
		State:     "New-York",
		Country:   "US",
		Elevation: 340,
		Latitude:  42.962600708,
		Longitude: -74.2537002563,
		Timezone:  "America/New_York",
	},
	"NY88": {
		ICAO:      "NY88",
		Name:      "Hickory Acres Airport",
		City:      "Fort Plain",
		State:     "New-York",
		Country:   "US",
		Elevation: 970,
		Latitude:  42.8833999634,
		Longitude: -74.6912994385,
		Timezone:  "America/New_York",
	},
	"NY89": {
		ICAO:      "NY89",
		Name:      "Fairbank Farms Airport",
		City:      "Jamestown",
		State:     "New-York",
		Country:   "US",
		Elevation: 1350,
		Latitude:  42.0801010132,
		Longitude: -79.3914031982,
		Timezone:  "America/New_York",
	},
	"NY91": {
		ICAO:      "NY91",
		Name:      "Cloverdale Farm Airport",
		City:      "Peterboro",
		State:     "New-York",
		Country:   "US",
		Elevation: 1465,
		Latitude:  42.9940261841,
		Longitude: -75.7027740479,
		Timezone:  "America/New_York",
	},
	"NY92": {
		ICAO:      "NY92",
		Name:      "Owasco Airport",
		City:      "Moravia",
		State:     "New-York",
		Country:   "US",
		Elevation: 1378,
		Latitude:  42.7358016968,
		Longitude: -76.4858016968,
		Timezone:  "America/New_York",
	},
	"NY94": {
		ICAO:      "NY94",
		Name:      "Old Rhinebeck Airport",
		City:      "Rhinebeck",
		State:     "New-York",
		Country:   "US",
		Elevation: 323,
		Latitude:  41.9715004,
		Longitude: -73.86289978,
		Timezone:  "America/New_York",
	},
	"NY96": {
		ICAO:      "NY96",
		Name:      "Mexico Airdrome Airport",
		City:      "Mexico",
		State:     "New-York",
		Country:   "US",
		Elevation: 470,
		Latitude:  43.426700592,
		Longitude: -76.1938018799,
		Timezone:  "America/New_York",
	},
	"NY97": {
		ICAO:      "NY97",
		Name:      "Northway Airport",
		City:      "Morrisonville",
		State:     "New-York",
		Country:   "US",
		Elevation: 420,
		Latitude:  44.6666984558,
		Longitude: -73.5662002563,
		Timezone:  "America/New_York",
	},
	"NZ12": {
		ICAO:      "NZ12",
		Name:      "Palmer Station Airport",
		Country:   "AQ",
		Elevation: 149,
		Latitude:  -64.7750015259,
		Longitude: -64.054397583,
		Timezone:  "Antarctica/Palmer",
	},
	"NZAA": {
		ICAO:      "NZAA",
		IATA:      "AKL",
		Name:      "Auckland International Airport",
		City:      "Auckland",
		State:     "Auckland",
		Country:   "NZ",
		Elevation: 23,
		Latitude:  -37.0080986023,
		Longitude: 174.7920074463,
		Timezone:  "Pacific/Auckland",
	},
	"NZAE": {
		ICAO:      "NZAE",
		Name:      "Mount Tarawera Airport",
		City:      "Mount Tarawera",
		State:     "Auckland",
		Country:   "NZ",
		Elevation: 3100,
		Latitude:  -38.233001709,
		Longitude: 176.516998291,
		Timezone:  "Pacific/Auckland",
	},
	"NZAG": {
		ICAO:      "NZAG",
		Name:      "Matarangi Airport",
		State:     "Auckland",
		Country:   "NZ",
		Elevation: 10,
		Latitude:  -36.7317008972,
		Longitude: 175.6499938965,
		Timezone:  "Pacific/Auckland",
	},
	"NZAN": {
		ICAO:      "NZAN",
		Name:      "Alfredton Airport",
		City:      "Alfredton",
		State:     "Manawatu-Wanganui",
		Country:   "NZ",
		Elevation: 500,
		Latitude:  -40.6981010437,
		Longitude: 175.8560028076,
		Timezone:  "Pacific/Auckland",
	},
	"NZAP": {
		ICAO:      "NZAP",
		IATA:      "TUO",
		Name:      "Taupo Airport",
		City:      "Taupo",
		State:     "Auckland",
		Country:   "NZ",
		Elevation: 1335,
		Latitude:  -38.7397003174,
		Longitude: 176.0839996338,
		Timezone:  "Pacific/Auckland",
	},
	"NZAR": {
		ICAO:      "NZAR",
		IATA:      "AMZ",
		Name:      "Ardmore Airport",
		City:      "Manurewa",
		State:     "Auckland",
		Country:   "NZ",
		Elevation: 111,
		Latitude:  -37.0297012329,
		Longitude: 174.9730072021,
		Timezone:  "Pacific/Auckland",
	},
	"NZAS": {
		ICAO:      "NZAS",
		IATA:      "ASG",
		Name:      "Ashburton Aerodrome",
		State:     "Canterbury",
		Country:   "NZ",
		Elevation: 302,
		Latitude:  -43.903301239,
		Longitude: 171.7969970703,
		Timezone:  "Pacific/Auckland",
	},
	"NZBA": {
		ICAO:      "NZBA",
		Name:      "Balclutha Aerodrome",
		City:      "Balclutha",
		State:     "Otago",
		Country:   "NZ",
		Elevation: 22,
		Latitude:  -46.2433013916,
		Longitude: 169.75,
		Timezone:  "Pacific/Auckland",
	},
	"NZCB": {
		ICAO:      "NZCB",
		Name:      "Centre Bush Aerodrome",
		City:      "Centre Bush",
		State:     "Southland",
		Country:   "NZ",
		Elevation: 255,
		Latitude:  -46.0400009155,
		Longitude: 168.31199646,
		Timezone:  "Pacific/Auckland",
	},
	"NZCH": {
		ICAO:      "NZCH",
		IATA:      "CHC",
		Name:      "Christchurch International Airport",
		City:      "Christchurch",
		State:     "Canterbury",
		Country:   "NZ",
		Elevation: 123,
		Latitude:  -43.4893989563,
		Longitude: 172.5319976807,
		Timezone:  "Pacific/Auckland",
	},
	"NZCI": {
		ICAO:      "NZCI",
		IATA:      "CHT",
		Name:      "Chatham Islands-Tuuta Airport",
		City:      "Waitangi",
		State:     "Chatham-Islands",
		Country:   "NZ",
		Elevation: 43,
		Latitude:  -43.8100013733,
		Longitude: -176.4570007324,
		Timezone:  "Pacific/Chatham",
	},
	"NZCS": {
		ICAO:      "NZCS",
		Name:      "Cromwell Racecourse Aerodrome",
		Country:   "NZ",
		Elevation: 856,
		Latitude:  -45.0485992432,
		Longitude: 169.171005249,
		Timezone:  "Pacific/Auckland",
	},
	"NZCX": {
		ICAO:      "NZCX",
		IATA:      "CMV",
		Name:      "Coromandel Aerodrome",
		State:     "Waikato",
		Country:   "NZ",
		Elevation: 10,
		Latitude:  -36.7916984558,
		Longitude: 175.5090026855,
		Timezone:  "Pacific/Auckland",
	},
	"NZDA": {
		ICAO:      "NZDA",
		IATA:      "DGR",
		Name:      "Dargaville Aerodrome",
		State:     "Northland",
		Country:   "NZ",
		Elevation: 10,
		Latitude:  -35.9397010803,
		Longitude: 173.8939971924,
		Timezone:  "Pacific/Auckland",
	},
	"NZDN": {
		ICAO:      "NZDN",
		IATA:      "DUD",
		Name:      "Dunedin Airport",
		City:      "Dunedin",
		State:     "Otago",
		Country:   "NZ",
		Elevation: 4,
		Latitude:  -45.9281005859,
		Longitude: 170.1979980469,
		Timezone:  "Pacific/Auckland",
	},
	"NZDV": {
		ICAO:      "NZDV",
		Name:      "Dannevirke Aerodrome",
		State:     "Manawatu-Wanganui",
		Country:   "NZ",
		Elevation: 640,
		Latitude:  -40.2282981873,
		Longitude: 176.078994751,
		Timezone:  "Pacific/Auckland",
	},
	"NZES": {
		ICAO:      "NZES",
		Name:      "Wharepapa South Airport",
		State:     "Waikato",
		Country:   "NZ",
		Elevation: 443,
		Latitude:  -38.1483001709,
		Longitude: 175.5500030518,
		Timezone:  "Pacific/Auckland",
	},
	"NZFF": {
		ICAO:      "NZFF",
		Name:      "Forest Field Aerodrome",
		State:     "Canterbury",
		Country:   "NZ",
		Elevation: 381,
		Latitude:  -43.385799408,
		Longitude: 172.3609924316,
		Timezone:  "Pacific/Auckland",
	},
	"NZFI": {
		ICAO:      "NZFI",
		Name:      "Feilding Airport",
		City:      "Feilding",
		State:     "Wellington",
		Country:   "NZ",
		Elevation: 214,
		Latitude:  -40.2557983398,
		Longitude: 175.6049957275,
		Timezone:  "Pacific/Auckland",
	},
	"NZFJ": {
		ICAO:      "NZFJ",
		IATA:      "WHO",
		Name:      "Franz Josef Aerodrome",
		State:     "West-Coast",
		Country:   "NZ",
		Elevation: 240,
		Latitude:  -43.3630981445,
		Longitude: 170.1340026855,
		Timezone:  "Pacific/Auckland",
	},
	"NZFP": {
		ICAO:      "NZFP",
		Name:      "Foxpine Aerodrome",
		Country:   "NZ",
		Elevation: 39,
		Latitude:  -40.4566993713,
		Longitude: 175.2700042725,
		Timezone:  "Pacific/Auckland",
	},
	"NZFT": {
		ICAO:      "NZFT",
		Name:      "Flat Point Aerodrome",
		Country:   "NZ",
		Elevation: 46,
		Latitude:  -41.2433013916,
		Longitude: 175.9600067139,
		Timezone:  "Pacific/Auckland",
	},
	"NZGA": {
		ICAO:      "NZGA",
		Name:      "Galatea Airfield",
		State:     "Bay-of-Plenty",
		Country:   "NZ",
		Elevation: 594,
		Latitude:  -38.4039001465,
		Longitude: 176.7440032959,
		Timezone:  "Pacific/Auckland",
	},
	"NZGB": {
		ICAO:      "NZGB",
		IATA:      "GBZ",
		Name:      "Great Barrier Aerodrome",
		City:      "Claris",
		State:     "Auckland",
		Country:   "NZ",
		Elevation: 20,
		Latitude:  -36.2414016724,
		Longitude: 175.4720001221,
		Timezone:  "Pacific/Auckland",
	},
	"NZGC": {
		ICAO:      "NZGC",
		Name:      "Gore3 Airport",
		State:     "Southland",
		Country:   "NZ",
		Elevation: 200,
		Latitude:  -46.1567001343,
		Longitude: 168.8979949951,
		Timezone:  "Pacific/Auckland",
	},
	"NZGM": {
		ICAO:      "NZGM",
		IATA:      "GMN",
		Name:      "Greymouth Airport",
		State:     "West-Coast",
		Country:   "NZ",
		Elevation: 10,
		Latitude:  -42.4617004395,
		Longitude: 171.1900024414,
		Timezone:  "Pacific/Auckland",
	},
	"NZGR": {
		ICAO:      "NZGR",
		Name:      "Great Mercury Airport",
		State:     "Auckland",
		Country:   "NZ",
		Elevation: 180,
		Latitude:  -36.5974998474,
		Longitude: 175.766998291,
		Timezone:  "Pacific/Auckland",
	},
	"NZGS": {
		ICAO:      "NZGS",
		IATA:      "GIS",
		Name:      "Gisborne Airport",
		City:      "Gisborne",
		State:     "Gisborne",
		Country:   "NZ",
		Elevation: 15,
		Latitude:  -38.6632995605,
		Longitude: 177.9779968262,
		Timezone:  "Pacific/Auckland",
	},
	"NZGT": {
		ICAO:      "NZGT",
		IATA:      "GTN",
		Name:      "Glentanner Airport",
		City:      "Glentanner Station",
		State:     "Canterbury",
		Country:   "NZ",
		Elevation: 1824,
		Latitude:  -43.9067001343,
		Longitude: 170.1280059814,
		Timezone:  "Pacific/Auckland",
	},
	"NZGY": {
		ICAO:      "NZGY",
		Name:      "Glenorchy Airport",
		State:     "Otago",
		Country:   "NZ",
		Elevation: 1260,
		Latitude:  -44.8717002869,
		Longitude: 168.3979949951,
		Timezone:  "Pacific/Auckland",
	},
	"NZHA": {
		ICAO:      "NZHA",
		Name:      "Hawera Airport",
		City:      "Hawera",
		State:     "Taranaki",
		Country:   "NZ",
		Elevation: 371,
		Latitude:  -39.5532989502,
		Longitude: 174.266998291,
		Timezone:  "Pacific/Auckland",
	},
	"NZHB": {
		ICAO:      "NZHB",
		Name:      "Auckland Hobsonville Airport",
		State:     "Auckland",
		Country:   "NZ",
		Elevation: 59,
		Latitude:  -36.7916984558,
		Longitude: 174.6640014648,
		Timezone:  "Pacific/Auckland",
	},
	"NZHK": {
		ICAO:      "NZHK",
		IATA:      "HKK",
		Name:      "Hokitika Airfield",
		State:     "West-Coast",
		Country:   "NZ",
		Elevation: 146,
		Latitude:  -42.7136001587,
		Longitude: 170.9850006104,
		Timezone:  "Pacific/Auckland",
	},
	"NZHN": {
		ICAO:      "NZHN",
		IATA:      "HLZ",
		Name:      "Hamilton International Airport",
		City:      "Hamilton",
		State:     "Waikato",
		Country:   "NZ",
		Elevation: 172,
		Latitude:  -37.8666992188,
		Longitude: 175.332000732,
		Timezone:  "Pacific/Auckland",
	},
	"NZHP": {
		ICAO:      "NZHP",
		Name:      "Lake Haupiri/Gloriavale Christian Community Airport",
		State:     "West-Coast",
		Country:   "NZ",
		Elevation: 600,
		Latitude:  -42.6012001038,
		Longitude: 171.699005127,
		Timezone:  "Pacific/Auckland",
	},
	"NZHR": {
		ICAO:      "NZHR",
		Name:      "Hanmer Springs Airport",
		State:     "Canterbury",
		Country:   "NZ",
		Elevation: 1109,
		Latitude:  -42.5508003235,
		Longitude: 172.8269958496,
		Timezone:  "Pacific/Auckland",
	},
	"NZHS": {
		ICAO:      "NZHS",
		Name:      "Hastings Airport",
		State:     "Hawke's-Bay",
		Country:   "NZ",
		Elevation: 72,
		Latitude:  -39.6467018127,
		Longitude: 176.766998291,
		Timezone:  "Pacific/Auckland",
	},
	"NZHT": {
		ICAO:      "NZHT",
		Name:      "Haast Aerodrome",
		State:     "West-Coast",
		Country:   "NZ",
		Elevation: 13,
		Latitude:  -43.8652992249,
		Longitude: 169.0410003662,
		Timezone:  "Pacific/Auckland",
	},
	"NZIR": {
		ICAO:      "NZIR",
		Name:      "McMurdo Station Ice Runway",
		City:      "McMurdo Station",
		Country:   "AQ",
		Elevation: 1,
		Latitude:  -77.8539962769,
		Longitude: 166.4689941406,
		Timezone:  "Antarctica/McMurdo",
	},
	"NZKC": {
		ICAO:      "NZKC",
		Name:      "Kelly Field",
		State:     "Auckland",
		Country:   "NZ",
		Elevation: 52,
		Latitude:  -37.2560997009,
		Longitude: 175.0720062256,
		Timezone:  "Pacific/Auckland",
	},
	"NZKD": {
		ICAO:      "NZKD",
		Name:      "Motu Kaikoura Island Aerodrome",
		City:      "Kaikoura Island",
		State:     "Auckland",
		Country:   "NZ",
		Elevation: 400,
		Latitude:  -36.1781005859,
		Longitude: 175.324005127,
		Timezone:  "Pacific/Auckland",
	},
	"NZKE": {
		ICAO:      "NZKE",
		IATA:      "WIK",
		Name:      "Waiheke Reeve Airport",
		State:     "Auckland",
		Country:   "NZ",
		Elevation: 226,
		Latitude:  -36.8088989258,
		Longitude: 175.0859985352,
		Timezone:  "Pacific/Auckland",
	},
	"NZKF": {
		ICAO:      "NZKF",
		Name:      "Kaipara Flats Airport",
		State:     "Auckland",
		Country:   "NZ",
		Elevation: 112,
		Latitude:  -36.4063987732,
		Longitude: 174.5870056152,
		Timezone:  "Pacific/Auckland",
	},
	"NZKI": {
		ICAO:      "NZKI",
		IATA:      "KBZ",
		Name:      "Kaikoura Airport",
		State:     "Marlborough",
		Country:   "NZ",
		Elevation: 20,
		Latitude:  -42.4249992371,
		Longitude: 173.6049957275,
		Timezone:  "Pacific/Auckland",
	},
	"NZKK": {
		ICAO:      "NZKK",
		IATA:      "KKE",
		Name:      "Kerikeri Airport",
		City:      "Kerikeri",
		State:     "Northland",
		Country:   "NZ",
		Elevation: 492,
		Latitude:  -35.2627983093,
		Longitude: 173.9120025635,
		Timezone:  "Pacific/Auckland",
	},
	"NZKM": {
		ICAO:      "NZKM",
		Name:      "Karamea Airport",
		Country:   "NZ",
		Elevation: 30,
		Latitude:  -41.2366981506,
		Longitude: 172.1049957275,
		Timezone:  "Pacific/Auckland",
	},
	"NZKO": {
		ICAO:      "NZKO",
		IATA:      "KKO",
		Name:      "Kaikohe Airport",
		State:     "Auckland",
		Country:   "NZ",
		Elevation: 571,
		Latitude:  -35.4510993958,
		Longitude: 173.8170013428,
		Timezone:  "Pacific/Auckland",
	},
	"NZKT": {
		ICAO:      "NZKT",
		IATA:      "KAT",
		Name:      "Kaitaia Airport",
		City:      "Kaitaia",
		State:     "Auckland",
		Country:   "NZ",
		Elevation: 270,
		Latitude:  -35.0699996948,
		Longitude: 173.2850036621,
		Timezone:  "Pacific/Auckland",
	},
	"NZKY": {
		ICAO:      "NZKY",
		Name:      "Kowhai Aerodrome",
		State:     "Hawke's-Bay",
		Country:   "NZ",
		Elevation: 840,
		Latitude:  -39.8460998535,
		Longitude: 176.4230041504,
		Timezone:  "Pacific/Auckland",
	},
	"NZLD": {
		ICAO:      "NZLD",
		Name:      "Limestone Downs Airport",
		City:      "Limestone Downs",
		State:     "Auckland",
		Country:   "NZ",
		Elevation: 596,
		Latitude:  -37.4799995422,
		Longitude: 174.7449951172,
		Timezone:  "Pacific/Auckland",
	},
	"NZLE": {
		ICAO:      "NZLE",
		Name:      "Lake Station Airport",
		State:     "Nelson",
		Country:   "NZ",
		Elevation: 1709,
		Latitude:  -41.7592010498,
		Longitude: 172.7460021973,
		Timezone:  "Pacific/Auckland",
	},
	"NZLX": {
		ICAO:      "NZLX",
		IATA:      "ALR",
		Name:      "Alexandra Airport",
		City:      "Alexandra",
		State:     "Otago",
		Country:   "NZ",
		Elevation: 752,
		Latitude:  -45.2117004395,
		Longitude: 169.3730010986,
		Timezone:  "Pacific/Auckland",
	},
	"NZMA": {
		ICAO:      "NZMA",
		IATA:      "MTA",
		Name:      "Matamata Glider Airport",
		State:     "Auckland",
		Country:   "NZ",
		Elevation: 180,
		Latitude:  -37.7344017029,
		Longitude: 175.7420043945,
		Timezone:  "Pacific/Auckland",
	},
	"NZMC": {
		ICAO:      "NZMC",
		IATA:      "MON",
		Name:      "Mount Cook Airport",
		State:     "Canterbury",
		Country:   "NZ",
		Elevation: 2153,
		Latitude:  -43.7649993896,
		Longitude: 170.1329956055,
		Timezone:  "Pacific/Auckland",
	},
	"NZME": {
		ICAO:      "NZME",
		Name:      "Mercer1 PDZ Airport",
		State:     "Auckland",
		Country:   "NZ",
		Elevation: 30,
		Latitude:  -37.2574996948,
		Longitude: 175.1150054932,
		Timezone:  "Pacific/Auckland",
	},
	"NZMF": {
		ICAO:      "NZMF",
		IATA:      "MFN",
		Name:      "Milford Sound Airport",
		State:     "Southland",
		Country:   "NZ",
		Elevation: 10,
		Latitude:  -44.6733016968,
		Longitude: 167.9230041504,
		Timezone:  "Pacific/Auckland",
	},
	"NZMJ": {
		ICAO:      "NZMJ",
		Name:      "Martins Bay Aerodrome",
		State:     "Otago",
		Country:   "NZ",
		Elevation: 348,
		Latitude:  -44.3657989502,
		Longitude: 168.0180053711,
		Timezone:  "Pacific/Auckland",
	},
	"NZMK": {
		ICAO:      "NZMK",
		IATA:      "MZP",
		Name:      "Motueka Airport",
		State:     "Nelson",
		Country:   "NZ",
		Elevation: 39,
		Latitude:  -41.123298645,
		Longitude: 172.9889984131,
		Timezone:  "Pacific/Auckland",
	},
	"NZML": {
		ICAO:      "NZML",
		Name:      "Molesworth Airport",
		City:      "Molesworth",
		State:     "Marlborough",
		Country:   "NZ",
		Elevation: 2813,
		Latitude:  -42.0830001831,
		Longitude: 173.2830047607,
		Timezone:  "Pacific/Auckland",
	},
	"NZMO": {
		ICAO:      "NZMO",
		IATA:      "TEU",
		Name:      "Manapouri Airport",
		State:     "Southland",
		Country:   "NZ",
		Elevation: 687,
		Latitude:  -45.5331001282,
		Longitude: 167.6499938965,
		Timezone:  "Pacific/Auckland",
	},
	"NZMR": {
		ICAO:      "NZMR",
		Name:      "Murchison Airport",
		City:      "Murchison",
		Country:   "NZ",
		Elevation: 531,
		Latitude:  -41.7966995239,
		Longitude: 172.3150024414,
		Timezone:  "Pacific/Auckland",
	},
	"NZMS": {
		ICAO:      "NZMS",
		IATA:      "MRO",
		Name:      "Hood Airport",
		City:      "Masterton",
		State:     "Wellington",
		Country:   "NZ",
		Elevation: 364,
		Latitude:  -40.9733009338,
		Longitude: 175.6340026855,
		Timezone:  "Pacific/Auckland",
	},
	"NZMT": {
		ICAO:      "NZMT",
		Name:      "Martinborough Airport",
		City:      "Martinborough",
		State:     "Wellington",
		Country:   "NZ",
		Latitude:  -41.2141990662,
		Longitude: 175.4859924316,
		Timezone:  "Pacific/Auckland",
	},
	"NZMW": {
		ICAO:      "NZMW",
		Name:      "Makarora Airstrip",
		Country:   "NZ",
		Elevation: 1142,
		Latitude:  -44.2317008972,
		Longitude: 169.2299957275,
		Timezone:  "Pacific/Auckland",
	},
	"NZMZ": {
		ICAO:      "NZMZ",
		Name:      "Matakana Island Airport",
		City:      "Matakana Island",
		State:     "Auckland",
		Country:   "NZ",
		Elevation: 95,
		Latitude:  -37.5999984741,
		Longitude: 176.0829925537,
		Timezone:  "Pacific/Auckland",
	},
	"NZNE": {
		ICAO:      "NZNE",
		Name:      "North Shore Aerodrome",
		City:      "Dairy Flat",
		State:     "Auckland",
		Country:   "NZ",
		Elevation: 210,
		Latitude:  -36.6567001343,
		Longitude: 174.6549987793,
		Timezone:  "Pacific/Auckland",
	},
	"NZNP": {
		ICAO:      "NZNP",
		IATA:      "NPL",
		Name:      "New Plymouth Airport",
		City:      "New Plymouth",
		State:     "Taranaki",
		Country:   "NZ",
		Elevation: 97,
		Latitude:  -39.0085983276,
		Longitude: 174.1790008545,
		Timezone:  "Pacific/Auckland",
	},
	"NZNR": {
		ICAO:      "NZNR",
		IATA:      "NPE",
		Name:      "Napier Airport",
		State:     "Hawke's-Bay",
		Country:   "NZ",
		Elevation: 6,
		Latitude:  -39.465801239,
		Longitude: 176.8699951172,
		Timezone:  "Pacific/Auckland",
	},
	"NZNS": {
		ICAO:      "NZNS",
		IATA:      "NSN",
		Name:      "Nelson Airport",
		City:      "Nelson",
		State:     "Nelson",
		Country:   "NZ",
		Elevation: 17,
		Latitude:  -41.2983016968,
		Longitude: 173.220993042,
		Timezone:  "Pacific/Auckland",
	},
	"NZNV": {
		ICAO:      "NZNV",
		IATA:      "IVC",
		Name:      "Invercargill Airport",
		City:      "Invercargill",
		State:     "Southland",
		Country:   "NZ",
		Elevation: 5,
		Latitude:  -46.412399292,
		Longitude: 168.31300354,
		Timezone:  "Pacific/Auckland",
	},
	"NZOA": {
		ICAO:      "NZOA",
		Name:      "Omarama Glider Airport",
		State:     "Canterbury",
		Country:   "NZ",
		Elevation: 1381,
		Latitude:  -44.4866981506,
		Longitude: 169.9859924316,
		Timezone:  "Pacific/Auckland",
	},
	"NZOH": {
		ICAO:      "NZOH",
		IATA:      "OHA",
		Name:      "RNZAF Base Ohakea",
		State:     "Wellington",
		Country:   "NZ",
		Elevation: 164,
		Latitude:  -40.2060012817,
		Longitude: 175.3880004883,
		Timezone:  "Pacific/Auckland",
	},
	"NZOI": {
		ICAO:      "NZOI",
		Name:      "Motiti Island Airport",
		City:      "Motiti Island",
		State:     "Auckland",
		Country:   "NZ",
		Elevation: 135,
		Latitude:  -37.6329994202,
		Longitude: 176.4170074463,
		Timezone:  "Pacific/Auckland",
	},
	"NZOM": {
		ICAO:      "NZOM",
		Name:      "Omaka Blenheim Airport",
		City:      "Blenheim",
		State:     "Marlborough",
		Country:   "NZ",
		Elevation: 102,
		Latitude:  -41.5400009155,
		Longitude: 173.9219970703,
		Timezone:  "Pacific/Auckland",
	},
	"NZOP": {
		ICAO:      "NZOP",
		Name:      "Opotiki Airport",
		State:     "Gisborne",
		Country:   "NZ",
		Elevation: 30,
		Latitude:  -38.0224990845,
		Longitude: 177.3070068359,
		Timezone:  "Pacific/Auckland",
	},
	"NZOU": {
		ICAO:      "NZOU",
		IATA:      "OAM",
		Name:      "Oamaru Airport",
		State:     "Otago",
		Country:   "NZ",
		Elevation: 99,
		Latitude:  -44.9700012207,
		Longitude: 171.0820007324,
		Timezone:  "Pacific/Auckland",
	},
	"NZOX": {
		ICAO:      "NZOX",
		Name:      "Okiwi Station Airport",
		State:     "Auckland",
		Country:   "NZ",
		Elevation: 10,
		Latitude:  -36.1464004517,
		Longitude: 175.4190063477,
		Timezone:  "Pacific/Auckland",
	},
	"NZPA": {
		ICAO:      "NZPA",
		Name:      "Paihia Private Airport",
		State:     "Northland",
		Country:   "NZ",
		Elevation: 20,
		Latitude:  -35.2821998596,
		Longitude: 174.091003418,
		Timezone:  "Pacific/Auckland",
	},
	"NZPG": {
		ICAO:      "NZPG",
		Name:      "McMurdo Station Pegasus Field",
		City:      "McMurdo Station",
		Country:   "AQ",
		Elevation: 18,
		Latitude:  -77.9634017944,
		Longitude: 166.5249938965,
		Timezone:  "Antarctica/McMurdo",
	},
	"NZPH": {
		ICAO:      "NZPH",
		Name:      "Pudding Hill Aerodrome",
		State:     "Canterbury",
		Country:   "NZ",
		Elevation: 1558,
		Latitude:  -43.5942001343,
		Longitude: 171.5310058594,
		Timezone:  "Pacific/Auckland",
	},
	"NZPI": {
		ICAO:      "NZPI",
		Name:      "Parakai Aerodrome",
		State:     "Auckland",
		Country:   "NZ",
		Elevation: 49,
		Latitude:  -36.6519012451,
		Longitude: 174.4360046387,
		Timezone:  "Pacific/Auckland",
	},
	"NZPK": {
		ICAO:      "NZPK",
		Name:      "Pikes Point Airport",
		City:      "Pikes Point",
		State:     "Auckland",
		Country:   "NZ",
		Latitude:  -36.9282989502,
		Longitude: 174.81199646,
		Timezone:  "Pacific/Auckland",
	},
	"NZPM": {
		ICAO:      "NZPM",
		IATA:      "PMR",
		Name:      "Palmerston North Airport",
		State:     "Wellington",
		Country:   "NZ",
		Elevation: 151,
		Latitude:  -40.3205986023,
		Longitude: 175.6170043945,
		Timezone:  "Pacific/Auckland",
	},
	"NZPN": {
		ICAO:      "NZPN",
		IATA:      "PCN",
		Name:      "Picton Aerodrome",
		City:      "Picton",
		State:     "Marlborough",
		Country:   "NZ",
		Elevation: 161,
		Latitude:  -41.3460998535,
		Longitude: 173.9559936523,
		Timezone:  "Pacific/Auckland",
	},
	"NZPO": {
		ICAO:      "NZPO",
		Name:      "Porangahau Aerodrome",
		State:     "Hawke's-Bay",
		Country:   "NZ",
		Elevation: 200,
		Latitude:  -40.2807998657,
		Longitude: 176.6519927979,
		Timezone:  "Pacific/Auckland",
	},
	"NZPP": {
		ICAO:      "NZPP",
		IATA:      "PPQ",
		Name:      "Paraparaumu Airport",
		State:     "Wellington",
		Country:   "NZ",
		Elevation: 22,
		Latitude:  -40.9047012329,
		Longitude: 174.9889984131,
		Timezone:  "Pacific/Auckland",
	},
	"NZQN": {
		ICAO:      "NZQN",
		IATA:      "ZQN",
		Name:      "Queenstown International Airport",
		City:      "Queenstown",
		State:     "Otago",
		Country:   "NZ",
		Elevation: 1171,
		Latitude:  -45.0210990906,
		Longitude: 168.738998413,
		Timezone:  "Pacific/Auckland",
	},
	"NZRA": {
		ICAO:      "NZRA",
		IATA:      "RAG",
		Name:      "Raglan Airfield",
		State:     "Waikato",
		Country:   "NZ",
		Elevation: 10,
		Latitude:  -37.8046989441,
		Longitude: 174.8600006104,
		Timezone:  "Pacific/Auckland",
	},
	"NZRC": {
		ICAO:      "NZRC",
		IATA:      "SZS",
		Name:      "Ryans Creek Aerodrome",
		City:      "Oban",
		State:     "Southland",
		Country:   "NZ",
		Elevation: 62,
		Latitude:  -46.8997001648,
		Longitude: 168.100997925,
		Timezone:  "Pacific/Auckland",
	},
	"NZRI": {
		ICAO:      "NZRI",
		Name:      "Rangitata Island Airport",
		City:      "Rangitata Island",
		State:     "Canterbury",
		Country:   "NZ",
		Elevation: 288,
		Latitude:  -44.0849990845,
		Longitude: 171.4160003662,
		Timezone:  "Pacific/Auckland",
	},
	"NZRK": {
		ICAO:      "NZRK",
		Name:      "Rangitaiki Airfield",
		State:     "Auckland",
		Country:   "NZ",
		Elevation: 2323,
		Latitude:  -38.8863983154,
		Longitude: 176.3639984131,
		Timezone:  "Pacific/Auckland",
	},
	"NZRO": {
		ICAO:      "NZRO",
		IATA:      "ROT",
		Name:      "Rotorua Regional Airport",
		City:      "Rotorua",
		State:     "Auckland",
		Country:   "NZ",
		Elevation: 935,
		Latitude:  -38.1091995239,
		Longitude: 176.3170013428,
		Timezone:  "Pacific/Auckland",
	},
	"NZRT": {
		ICAO:      "NZRT",
		Name:      "Rangiora Airfield",
		City:      "Rangiora",
		State:     "Canterbury",
		Country:   "NZ",
		Elevation: 180,
		Latitude:  -43.2900009155,
		Longitude: 172.5420074463,
		Timezone:  "Pacific/Auckland",
	},
	"NZRU": {
		ICAO:      "NZRU",
		Name:      "Waiouru Airport",
		State:     "Wellington",
		Country:   "NZ",
		Elevation: 2686,
		Latitude:  -39.4463996887,
		Longitude: 175.6580047607,
		Timezone:  "Pacific/Auckland",
	},
	"NZRW": {
		ICAO:      "NZRW",
		Name:      "Ruawai Aerodrome",
		State:     "Auckland",
		Country:   "NZ",
		Elevation: 62,
		Latitude:  -36.0971984863,
		Longitude: 173.9770050049,
		Timezone:  "Pacific/Auckland",
	},
	"NZRX": {
		ICAO:      "NZRX",
		Name:      "Roxburgh Aerodrome",
		State:     "Otago",
		Country:   "NZ",
		Elevation: 463,
		Latitude:  -45.5116996765,
		Longitude: 169.3170013428,
		Timezone:  "Pacific/Auckland",
	},
	"NZSD": {
		ICAO:      "NZSD",
		Name:      "Stratford Airport",
		City:      "Stratford",
		State:     "Taranaki",
		Country:   "NZ",
		Elevation: 951,
		Latitude:  -39.318901062,
		Longitude: 174.3099975586,
		Timezone:  "Pacific/Auckland",
	},
	"NZSL": {
		ICAO:      "NZSL",
		Name:      "Springhill Airport",
		State:     "Auckland",
		Country:   "NZ",
		Elevation: 102,
		Latitude:  -36.323600769,
		Longitude: 174.5559997559,
		Timezone:  "Pacific/Auckland",
	},
	"NZSP": {
		ICAO:      "NZSP",
		Name:      "South Pole Station Airport",
		Country:   "AQ",
		Elevation: 9300,
		Latitude:  -90,
		Timezone:  "Antarctica/McMurdo",
	},
	"NZTA": {
		ICAO:      "NZTA",
		Name:      "Te Aroha Airfield",
		State:     "Auckland",
		Country:   "NZ",
		Elevation: 30,
		Latitude:  -37.4747009277,
		Longitude: 175.6179962158,
		Timezone:  "Pacific/Auckland",
	},
	"NZTE": {
		ICAO:      "NZTE",
		Name:      "Te Kowhai Airfield",
		State:     "Auckland",
		Country:   "NZ",
		Elevation: 49,
		Latitude:  -37.7442016602,
		Longitude: 175.1609954834,
		Timezone:  "Pacific/Auckland",
	},
	"NZTG": {
		ICAO:      "NZTG",
		IATA:      "TRG",
		Name:      "Tauranga Airport",
		City:      "Tauranga",
		State:     "Bay-of-Plenty",
		Country:   "NZ",
		Elevation: 13,
		Latitude:  -37.6719017029,
		Longitude: 176.1959991455,
		Timezone:  "Pacific/Auckland",
	},
	"NZTH": {
		ICAO:      "NZTH",
		IATA:      "TMZ",
		Name:      "Thames Aerodrome",
		State:     "Waikato",
		Country:   "NZ",
		Elevation: 10,
		Latitude:  -37.1567001343,
		Longitude: 175.5500030518,
		Timezone:  "Pacific/Auckland",
	},
	"NZTI": {
		ICAO:      "NZTI",
		Name:      "Taieri Airport",
		State:     "Otago",
		Country:   "NZ",
		Elevation: 92,
		Latitude:  -45.8600006104,
		Longitude: 170.358001709,
		Timezone:  "Pacific/Auckland",
	},
	"NZTK": {
		ICAO:      "NZTK",
		IATA:      "KTF",
		Name:      "Takaka Airport",
		State:     "Tasman",
		Country:   "NZ",
		Elevation: 102,
		Latitude:  -40.8133010864,
		Longitude: 172.7749938965,
		Timezone:  "Pacific/Auckland",
	},
	"NZTL": {
		ICAO:      "NZTL",
		Name:      "Tekapo Aerodrome",
		State:     "Canterbury",
		Country:   "NZ",
		Elevation: 2496,
		Latitude:  -44.0052986145,
		Longitude: 170.444000244,
		Timezone:  "Pacific/Auckland",
	},
	"NZTM": {
		ICAO:      "NZTM",
		Name:      "Taumarunui Airport",
		State:     "Auckland",
		Country:   "NZ",
		Elevation: 902,
		Latitude:  -38.8394012451,
		Longitude: 175.2619934082,
		Timezone:  "Pacific/Auckland",
	},
	"NZTN": {
		ICAO:      "NZTN",
		Name:      "Turangi Airport",
		State:     "Wellington",
		Country:   "NZ",
		Elevation: 1220,
		Latitude:  -38.9682998657,
		Longitude: 175.8139953613,
		Timezone:  "Pacific/Auckland",
	},
	"NZTO": {
		ICAO:      "NZTO",
		IATA:      "TKZ",
		Name:      "Tokoroa Airfield",
		City:      "Tokoroa",
		State:     "Waikato",
		Country:   "NZ",
		Elevation: 1220,
		Latitude:  -38.2366981506,
		Longitude: 175.891998291,
		Timezone:  "Pacific/Auckland",
	},
	"NZTS": {
		ICAO:      "NZTS",
		IATA:      "THH",
		Name:      "Taharoa Aerodrome",
		City:      "Taharoa",
		State:     "Waikato",
		Country:   "NZ",
		Elevation: 69,
		Latitude:  -38.181098938,
		Longitude: 174.7079925537,
		Timezone:  "Pacific/Auckland",
	},
	"NZTT": {
		ICAO:      "NZTT",
		Name:      "Te Kuiti Airfield",
		City:      "Te Kuiti",
		State:     "Waikato",
		Country:   "NZ",
		Elevation: 161,
		Latitude:  -38.3032989502,
		Longitude: 175.1470031738,
		Timezone:  "Pacific/Auckland",
	},
	"NZTU": {
		ICAO:      "NZTU",
		IATA:      "TIU",
		Name:      "Timaru Airport",
		State:     "Canterbury",
		Country:   "NZ",
		Elevation: 89,
		Latitude:  -44.3027992249,
		Longitude: 171.2250061035,
		Timezone:  "Pacific/Auckland",
	},
	"NZTZ": {
		ICAO:      "NZTZ",
		Name:      "Te Anau Airport",
		State:     "Southland",
		Country:   "NZ",
		Elevation: 791,
		Latitude:  -45.4627990723,
		Longitude: 167.7010040283,
		Timezone:  "Pacific/Auckland",
	},
	"NZUK": {
		ICAO:      "NZUK",
		IATA:      "TWZ",
		Name:      "Pukaki Airport",
		City:      "Twitzel",
		State:     "Canterbury",
		Country:   "NZ",
		Elevation: 1575,
		Latitude:  -44.2350006104,
		Longitude: 170.117996216,
		Timezone:  "Pacific/Auckland",
	},
	"NZUN": {
		ICAO:      "NZUN",
		Name:      "Pauanui Beach Glider Airport",
		State:     "Waikato",
		Country:   "NZ",
		Elevation: 20,
		Latitude:  -37.0217018127,
		Longitude: 175.8639984131,
		Timezone:  "Pacific/Auckland",
	},
	"NZVL": {
		ICAO:      "NZVL",
		Name:      "Mandeville Aerodrome",
		Country:   "NZ",
		Elevation: 351,
		Latitude:  -45.9902992249,
		Longitude: 168.81199646,
		Timezone:  "Pacific/Auckland",
	},
	"NZVR": {
		ICAO:      "NZVR",
		Name:      "Taihape Airport",
		City:      "Taihape",
		Country:   "NZ",
		Elevation: 1550,
		Latitude:  -39.6850013733,
		Longitude: 175.7890014648,
		Timezone:  "Pacific/Auckland",
	},
	"NZWB": {
		ICAO:      "NZWB",
		IATA:      "BHE",
		Name:      "Woodbourne Airport",
		City:      "Blenheim",
		State:     "Marlborough",
		Country:   "NZ",
		Elevation: 109,
		Latitude:  -41.5182991028,
		Longitude: 173.8699951172,
		Timezone:  "Pacific/Auckland",
	},
	"NZWD": {
		ICAO:      "NZWD",
		Name:      "Williams Field",
		City:      "McMurdo Station",
		Country:   "AQ",
		Elevation: 68,
		Latitude:  -77.867401123,
		Longitude: 167.0570068359,
		Timezone:  "Antarctica/McMurdo",
	},
	"NZWF": {
		ICAO:      "NZWF",
		IATA:      "WKA",
		Name:      "Wanaka Airport",
		State:     "Otago",
		Country:   "NZ",
		Elevation: 1142,
		Latitude:  -44.7221984863,
		Longitude: 169.2460021973,
		Timezone:  "Pacific/Auckland",
	},
	"NZWJ": {
		ICAO:      "NZWJ",
		Name:      "Wellsford Airport",
		State:     "Auckland",
		Country:   "NZ",
		Elevation: 194,
		Latitude:  -36.2969017029,
		Longitude: 174.5229949951,
		Timezone:  "Pacific/Auckland",
	},
	"NZWK": {
		ICAO:      "NZWK",
		IATA:      "WHK",
		Name:      "Whakatane Airport",
		State:     "Auckland",
		Country:   "NZ",
		Elevation: 20,
		Latitude:  -37.9206008911,
		Longitude: 176.9140014648,
		Timezone:  "Pacific/Auckland",
	},
	"NZWL": {
		ICAO:      "NZWL",
		Name:      "West Melton Aerodrome",
		State:     "Canterbury",
		Country:   "NZ",
		Elevation: 312,
		Latitude:  -43.4766998291,
		Longitude: 172.3970031738,
		Timezone:  "Pacific/Auckland",
	},
	"NZWM": {
		ICAO:      "NZWM",
		Name:      "Waimate Airport",
		State:     "Canterbury",
		Country:   "NZ",
		Elevation: 79,
		Latitude:  -44.7900009155,
		Longitude: 171.0919952393,
		Timezone:  "Pacific/Auckland",
	},
	"NZWN": {
		ICAO:      "NZWN",
		IATA:      "WLG",
		Name:      "Wellington International Airport",
		City:      "Wellington",
		State:     "Wellington",
		Country:   "NZ",
		Elevation: 41,
		Latitude:  -41.3272018433,
		Longitude: 174.804992676,
		Timezone:  "Pacific/Auckland",
	},
	"NZWO": {
		ICAO:      "NZWO",
		IATA:      "WIR",
		Name:      "Wairoa Airport",
		City:      "Wairoa",
		State:     "Hawke's-Bay",
		Country:   "NZ",
		Elevation: 42,
		Latitude:  -39.0069007874,
		Longitude: 177.4069976807,
		Timezone:  "Pacific/Auckland",
	},
	"NZWP": {
		ICAO:      "NZWP",
		Name:      "RNZAF Base Auckland-Whenuapai",
		State:     "Auckland",
		Country:   "NZ",
		Elevation: 100,
		Latitude:  -36.7877998352,
		Longitude: 174.6300048828,
		Timezone:  "Pacific/Auckland",
	},
	"NZWR": {
		ICAO:      "NZWR",
		IATA:      "WRE",
		Name:      "Whangarei Airport",
		State:     "Northland",
		Country:   "NZ",
		Elevation: 133,
		Latitude:  -35.7682991028,
		Longitude: 174.3650054932,
		Timezone:  "Pacific/Auckland",
	},
	"NZWS": {
		ICAO:      "NZWS",
		IATA:      "WSZ",
		Name:      "Westport Airport",
		State:     "West-Coast",
		Country:   "NZ",
		Elevation: 13,
		Latitude:  -41.7380981445,
		Longitude: 171.5809936523,
		Timezone:  "Pacific/Auckland",
	},
	"NZWT": {
		ICAO:      "NZWT",
		IATA:      "WTZ",
		Name:      "Whitianga Airport",
		State:     "Auckland",
		Country:   "NZ",
		Elevation: 10,
		Latitude:  -36.8316993713,
		Longitude: 175.6790008545,
		Timezone:  "Pacific/Auckland",
	},
	"NZWU": {
		ICAO:      "NZWU",
		IATA:      "WAG",
		Name:      "Wanganui Airport",
		City:      "Wanganui",
		State:     "Manawatu-Wanganui",
		Country:   "NZ",
		Elevation: 27,
		Latitude:  -39.9622001648,
		Longitude: 175.0249938965,
		Timezone:  "Pacific/Auckland",
	},
	"NZWV": {
		ICAO:      "NZWV",
		Name:      "Waihi Beach Airport",
		State:     "Bay-of-Plenty",
		Country:   "NZ",
		Elevation: 3,
		Latitude:  -37.4300003052,
		Longitude: 175.9519958496,
		Timezone:  "Pacific/Auckland",
	},
	"NZYP": {
		ICAO:      "NZYP",
		Name:      "Waipukurau Airport",
		State:     "Hawke's-Bay",
		Country:   "NZ",
		Elevation: 430,
		Latitude:  -39.9967002869,
		Longitude: 176.5370025635,
		Timezone:  "Pacific/Auckland",
	},
	"OA00": {
		ICAO:      "OA00",
		Name:      "Taildragger Airport",
		City:      "Cambridge",
		State:     "Ohio",
		Country:   "US",
		Elevation: 800,
		Latitude:  40.1170005798,
		Longitude: -81.5711975098,
		Timezone:  "America/New_York",
	},
	"OA04": {
		ICAO:      "OA04",
		Name:      "Rutter Airport",
		City:      "Arcadia",
		State:     "Ohio",
		Country:   "US",
		Elevation: 815,
		Latitude:  41.0975990295,
		Longitude: -83.5027008057,
		Timezone:  "America/New_York",
	},
	"OA05": {
		ICAO:      "OA05",
		Name:      "Hogan Airport",
		City:      "Mc Gonigle",
		State:     "Ohio",
		Country:   "US",
		Elevation: 865,
		Latitude:  39.4241981506,
		Longitude: -84.6591033936,
		Timezone:  "America/New_York",
	},
	"OA07": {
		ICAO:      "OA07",
		Name:      "Hemlock Field",
		City:      "Rock Creek",
		State:     "Ohio",
		Country:   "US",
		Elevation: 815,
		Latitude:  41.7014007568,
		Longitude: -80.8602981567,
		Timezone:  "America/New_York",
	},
	"OA10": {
		ICAO:      "OA10",
		Name:      "Bernie's Airport",
		City:      "Mount Orab",
		State:     "Ohio",
		Country:   "US",
		Elevation: 950,
		Latitude:  39.0085983276,
		Longitude: -83.9092025757,
		Timezone:  "America/New_York",
	},
	"OA11": {
		ICAO:      "OA11",
		Name:      "Heitman Field",
		City:      "Anna",
		State:     "Ohio",
		Country:   "US",
		Elevation: 998,
		Latitude:  40.3760986328,
		Longitude: -84.2549972534,
		Timezone:  "America/New_York",
	},
	"OA12": {
		ICAO:      "OA12",
		Name:      "Buena Vista Farm Airport",
		City:      "Morrow",
		State:     "Ohio",
		Country:   "US",
		Elevation: 855,
		Latitude:  39.334400177,
		Longitude: -84.0699996948,
		Timezone:  "America/New_York",
	},
	"OA14": {
		ICAO:      "OA14",
		Name:      "Roxford Airport",
		City:      "Dennison",
		State:     "Ohio",
		Country:   "US",
		Elevation: 872,
		Latitude:  40.422222,
		Longitude: -81.318056,
		Timezone:  "America/New_York",
	},
	"OA16": {
		ICAO:      "OA16",
		Name:      "Lost Bridge Airport",
		City:      "North Bend",
		State:     "Ohio",
		Country:   "US",
		Elevation: 818,
		Latitude:  39.156111,
		Longitude: -84.781667,
		Timezone:  "America/New_York",
	},
	"OA19": {
		ICAO:      "OA19",
		Name:      "Kiko Farm Airport",
		City:      "North Canton",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1170,
		Latitude:  40.898889,
		Longitude: -81.3,
		Timezone:  "America/New_York",
	},
	"OA23": {
		ICAO:      "OA23",
		Name:      "Heins Field",
		City:      "Arcanum",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1040,
		Latitude:  39.968667,
		Longitude: -84.539,
		Timezone:  "America/New_York",
	},
	"OA51": {
		ICAO:      "OA51",
		Name:      "Fighter Field Airport",
		City:      "Watkins",
		State:     "Ohio",
		Country:   "US",
		Elevation: 980,
		Latitude:  40.206111,
		Longitude: -83.228611,
		Timezone:  "America/New_York",
	},
	"OA85": {
		ICAO:      "OA85",
		Name:      "Riverview Airport",
		City:      "Dresden",
		State:     "Ohio",
		Country:   "US",
		Elevation: 730,
		Latitude:  40.1365013123,
		Longitude: -81.9936981201,
		Timezone:  "America/New_York",
	},
	"OABN": {
		ICAO:      "OABN",
		IATA:      "BIN",
		Name:      "Bamiyan Airport",
		City:      "Bamiyan",
		State:     "Bamyan",
		Country:   "AF",
		Elevation: 8367,
		Latitude:  34.8170013428,
		Longitude: 67.8170013428,
		Timezone:  "Asia/Kabul",
	},
	"OABT": {
		ICAO:      "OABT",
		IATA:      "BST",
		Name:      "Bost Airport",
		City:      "Bost",
		State:     "Helmand",
		Country:   "AF",
		Elevation: 2464,
		Latitude:  31.5597000122,
		Longitude: 64.3649978638,
		Timezone:  "Asia/Kabul",
	},
	"OACC": {
		ICAO:      "OACC",
		IATA:      "CCN",
		Name:      "Chakcharan Airport",
		City:      "Chakcharan",
		State:     "Ghowr",
		Country:   "AF",
		Elevation: 7383,
		Latitude:  34.533000946,
		Longitude: 65.266998291,
		Timezone:  "Asia/Kabul",
	},
	"OADZ": {
		ICAO:      "OADZ",
		IATA:      "DAZ",
		Name:      "Darwaz Airport",
		City:      "Darwaz",
		State:     "Badakhshan",
		Country:   "AF",
		Elevation: 5250,
		Latitude:  38.461101532,
		Longitude: 70.8824996948,
		Timezone:  "Asia/Dushanbe",
	},
	"OAFR": {
		ICAO:      "OAFR",
		IATA:      "FAH",
		Name:      "Farah Airport",
		City:      "Farah",
		State:     "Farah",
		Country:   "AF",
		Elevation: 3083,
		Latitude:  32.3670005798,
		Longitude: 62.1829986572,
		Timezone:  "Asia/Kabul",
	},
	"OAFZ": {
		ICAO:      "OAFZ",
		IATA:      "FBD",
		Name:      "Faizabad Airport",
		City:      "Faizabad",
		State:     "Badakhshan",
		Country:   "AF",
		Elevation: 3872,
		Latitude:  37.1211013794,
		Longitude: 70.5180969238,
		Timezone:  "Asia/Kabul",
	},
	"OAHN": {
		ICAO:      "OAHN",
		IATA:      "KWH",
		Name:      "Khwahan Airport",
		City:      "Khwahan",
		State:     "Badakhshan",
		Country:   "AF",
		Elevation: 3593,
		Latitude:  37.8829994202,
		Longitude: 70.2170028687,
		Timezone:  "Asia/Dushanbe",
	},
	"OAHR": {
		ICAO:      "OAHR",
		IATA:      "HEA",
		Name:      "Herat Airport",
		State:     "Herat",
		Country:   "AF",
		Elevation: 3206,
		Latitude:  34.2099990845,
		Longitude: 62.2282981873,
		Timezone:  "Asia/Kabul",
	},
	"OAIX": {
		ICAO:      "OAIX",
		IATA:      "OAI",
		Name:      "Bagram Air Base",
		City:      "Bagram",
		State:     "Parwan",
		Country:   "AF",
		Elevation: 4895,
		Latitude:  34.9460983276,
		Longitude: 69.2649993896,
		Timezone:  "Asia/Kabul",
	},
	"OAJL": {
		ICAO:      "OAJL",
		IATA:      "JAA",
		Name:      "Jalalabad Airport",
		State:     "Nangarhar",
		Country:   "AF",
		Elevation: 1814,
		Latitude:  34.3997993469,
		Longitude: 70.4985961914,
		Timezone:  "Asia/Kabul",
	},
	"OAKB": {
		ICAO:      "OAKB",
		IATA:      "KBL",
		Name:      "Kabul International Airport",
		City:      "Kabul",
		State:     "Kabul",
		Country:   "AF",
		Elevation: 5877,
		Latitude:  34.5658988953,
		Longitude: 69.2123031616,
		Timezone:  "Asia/Kabul",
	},
	"OAKG": {
		ICAO:      "OAKG",
		Name:      "Khojaghar Airport",
		City:      "Khojaghar",
		State:     "Takhar",
		Country:   "AF",
		Elevation: 1607,
		Latitude:  37.0830001831,
		Longitude: 69.3669967651,
		Timezone:  "Asia/Kabul",
	},
	"OAKN": {
		ICAO:      "OAKN",
		IATA:      "KDH",
		Name:      "Kandahar Airport",
		State:     "Kandahar",
		Country:   "AF",
		Elevation: 3337,
		Latitude:  31.5058002472,
		Longitude: 65.8478012085,
		Timezone:  "Asia/Kabul",
	},
	"OAKS": {
		ICAO:      "OAKS",
		IATA:      "KHT",
		Name:      "Khost Airport",
		City:      "Khost",
		State:     "Khowst",
		Country:   "AF",
		Elevation: 3756,
		Latitude:  33.3334007263,
		Longitude: 69.952003479,
		Timezone:  "Asia/Kabul",
	},
	"OAMN": {
		ICAO:      "OAMN",
		IATA:      "MMZ",
		Name:      "Maimana Airport",
		State:     "Faryab",
		Country:   "AF",
		Elevation: 2743,
		Latitude:  35.9308013916,
		Longitude: 64.7609024048,
		Timezone:  "Asia/Kabul",
	},
	"OAMS": {
		ICAO:      "OAMS",
		IATA:      "MZR",
		Name:      "Mazar I Sharif Airport",
		State:     "Balkh",
		Country:   "AF",
		Elevation: 1284,
		Latitude:  36.7069015503,
		Longitude: 67.2097015381,
		Timezone:  "Asia/Kabul",
	},
	"OAQM": {
		ICAO:      "OAQM",
		Name:      "Kron Monajn Airport",
		City:      "Kron Monajn",
		State:     "Badakhshan",
		Country:   "AF",
		Elevation: 8420,
		Latitude:  36.1669998169,
		Longitude: 70.7170028687,
		Timezone:  "Asia/Kabul",
	},
	"OAQN": {
		ICAO:      "OAQN",
		IATA:      "LQN",
		Name:      "Qala-I-Naw Airport",
		City:      "Qala-I-Naw",
		State:     "Badghis",
		Country:   "AF",
		Elevation: 2999,
		Latitude:  34.9850006104,
		Longitude: 63.1178016663,
		Timezone:  "Asia/Kabul",
	},
	"OASA": {
		ICAO:      "OASA",
		IATA:      "OAS",
		Name:      "Sharana Airstrip",
		City:      "Sharana",
		State:     "Paktika",
		Country:   "AF",
		Elevation: 7340,
		Latitude:  33.12575,
		Longitude: 68.838517,
		Timezone:  "Asia/Kabul",
	},
	"OASD": {
		ICAO:      "OASD",
		IATA:      "OAH",
		Name:      "Shindand Airport",
		State:     "Herat",
		Country:   "AF",
		Elevation: 3773,
		Latitude:  33.3913002014,
		Longitude: 62.2610015869,
		Timezone:  "Asia/Kabul",
	},
	"OASG": {
		ICAO:      "OASG",
		Name:      "Sheberghan Airport",
		State:     "Jowzjan",
		Country:   "AF",
		Elevation: 1053,
		Latitude:  36.7508010864,
		Longitude: 65.9132003784,
		Timezone:  "Asia/Kabul",
	},
	"OASN": {
		ICAO:      "OASN",
		IATA:      "SGA",
		Name:      "Sheghnan Airport",
		City:      "Sheghnan",
		State:     "Badakhshan",
		Country:   "AF",
		Elevation: 6700,
		Latitude:  37.5670013428,
		Longitude: 71.5,
		Timezone:  "Asia/Dushanbe",
	},
	"OATN": {
		ICAO:      "OATN",
		IATA:      "TII",
		Name:      "Tarin Kowt Airport",
		City:      "Tarin Kowt",
		State:     "Oruzgan",
		Country:   "AF",
		Elevation: 4429,
		Latitude:  32.6041984558,
		Longitude: 65.8657989502,
		Timezone:  "Asia/Kabul",
	},
	"OATQ": {
		ICAO:      "OATQ",
		IATA:      "TQN",
		Name:      "Talolqan Airport",
		City:      "Taloqan",
		State:     "Takhar",
		Country:   "AF",
		Elevation: 2606,
		Latitude:  36.7706985474,
		Longitude: 69.5319976807,
		Timezone:  "Asia/Kabul",
	},
	"OAUZ": {
		ICAO:      "OAUZ",
		IATA:      "UND",
		Name:      "Konduz Airport",
		State:     "Kunduz",
		Country:   "AF",
		Elevation: 1457,
		Latitude:  36.6651000977,
		Longitude: 68.9107971191,
		Timezone:  "Asia/Kabul",
	},
	"OAYQ": {
		ICAO:      "OAYQ",
		Name:      "Yangi Qala Airport",
		State:     "Takhar",
		Country:   "AF",
		Elevation: 1600,
		Latitude:  37.4589996338,
		Longitude: 69.6465988159,
		Timezone:  "Asia/Kabul",
	},
	"OAYW": {
		ICAO:      "OAYW",
		Name:      "Yawan Airport",
		State:     "Badakhshan",
		Country:   "AF",
		Elevation: 5645,
		Latitude:  37.5616989136,
		Longitude: 70.4417037964,
		Timezone:  "Asia/Kabul",
	},
	"OAZI": {
		ICAO:      "OAZI",
		IATA:      "OAZ",
		Name:      "Camp Bastion Airport",
		State:     "Helmand",
		Country:   "AF",
		Elevation: 2943,
		Latitude:  31.8638000488,
		Longitude: 64.2246017456,
		Timezone:  "Asia/Kabul",
	},
	"OAZJ": {
		ICAO:      "OAZJ",
		IATA:      "ZAJ",
		Name:      "Zaranj Airport",
		City:      "Zaranj",
		Country:   "AF",
		Elevation: 1572,
		Latitude:  30.972222,
		Longitude: 61.865833,
		Timezone:  "Asia/Kabul",
	},
	"OBBI": {
		ICAO:      "OBBI",
		IATA:      "BAH",
		Name:      "Bahrain International Airport",
		City:      "Manama",
		State:     "Muharraq",
		Country:   "BH",
		Elevation: 6,
		Latitude:  26.2707996368,
		Longitude: 50.6335983276,
		Timezone:  "Asia/Bahrain",
	},
	"OBBS": {
		ICAO:      "OBBS",
		Name:      "Sheik Isa Air Base",
		City:      "Sitrah",
		Country:   "BH",
		Elevation: 136,
		Latitude:  25.9183998108,
		Longitude: 50.5905990601,
		Timezone:  "Asia/Bahrain",
	},
	"OE43": {
		ICAO:      "OE43",
		Name:      "Arfa Airport",
		State:     "Makkah",
		Country:   "SA",
		Elevation: 4155,
		Latitude:  21.323600769,
		Longitude: 41.198299408,
		Timezone:  "Asia/Riyadh",
	},
	"OE45": {
		ICAO:      "OE45",
		Name:      "Ras Khafji Airport",
		State:     "Eastern-Province",
		Country:   "SA",
		Elevation: 50,
		Latitude:  28.3899002075,
		Longitude: 48.5164985657,
		Timezone:  "Asia/Riyadh",
	},
	"OE46": {
		ICAO:      "OE46",
		Name:      "Al Lidem Airport",
		State:     "Ar-Riyaḑ",
		Country:   "SA",
		Elevation: 2225,
		Latitude:  20.47590065,
		Longitude: 44.7574996948,
		Timezone:  "Asia/Riyadh",
	},
	"OE47": {
		ICAO:      "OE47",
		Name:      "Al Kharj",
		State:     "Ar-Riyaḑ",
		Country:   "SA",
		Latitude:  24.0607223511,
		Longitude: 47.4108047485,
		Timezone:  "Asia/Riyadh",
	},
	"OE48": {
		ICAO:      "OE48",
		Name:      "Quz South Airport",
		State:     "Makkah",
		Country:   "SA",
		Elevation: 200,
		Latitude:  18.8931999207,
		Longitude: 41.3549995422,
		Timezone:  "Asia/Riyadh",
	},
	"OE49": {
		ICAO:      "OE49",
		Name:      "Mecca East Airport",
		State:     "Makkah",
		Country:   "SA",
		Elevation: 1210,
		Latitude:  21.4375,
		Longitude: 39.9942016602,
		Timezone:  "Asia/Riyadh",
	},
	"OE51": {
		ICAO:      "OE51",
		Name:      "Al Kharj East Airport",
		State:     "Ar-Riyaḑ",
		Country:   "SA",
		Elevation: 1650,
		Latitude:  24.0515003204,
		Longitude: 47.6542015076,
		Timezone:  "Asia/Riyadh",
	},
	"OE52": {
		ICAO:      "OE52",
		Name:      "Al Artawiyah South Airport",
		State:     "Ar-Riyaḑ",
		Country:   "SA",
		Elevation: 1900,
		Latitude:  26.6116008759,
		Longitude: 45.3219985962,
		Timezone:  "Asia/Riyadh",
	},
	"OE53": {
		ICAO:      "OE53",
		Name:      "King Abdul Aziz Military Academy Airport",
		State:     "Ar-Riyaḑ",
		Country:   "SA",
		Elevation: 2420,
		Latitude:  24.9421005249,
		Longitude: 46.3912010193,
		Timezone:  "Asia/Riyadh",
	},
	"OE54": {
		ICAO:      "OE54",
		Name:      "Hafar Al Atk Airport",
		State:     "Ar-Riyaḑ",
		Country:   "SA",
		Elevation: 1970,
		Latitude:  25.9934005737,
		Longitude: 46.5367012024,
		Timezone:  "Asia/Riyadh",
	},
	"OE55": {
		ICAO:      "OE55",
		Name:      "Ghafah Airport",
		Country:   "SA",
		Elevation: 699,
		Latitude:  21.2269992828,
		Longitude: 55.2672996521,
		Timezone:  "Asia/Riyadh",
	},
	"OE56": {
		ICAO:      "OE56",
		Name:      "Khalfan Airport",
		State:     "Eastern-Province",
		Country:   "SA",
		Elevation: 305,
		Latitude:  22.710100174,
		Longitude: 53.2812004089,
		Timezone:  "Asia/Riyadh",
	},
	"OEAA": {
		ICAO:      "OEAA",
		Name:      "Abu Ali Airport",
		State:     "Eastern-Province",
		Country:   "SA",
		Elevation: 69,
		Latitude:  27.3190994263,
		Longitude: 49.5871009827,
		Timezone:  "Asia/Riyadh",
	},
	"OEAB": {
		ICAO:      "OEAB",
		IATA:      "AHB",
		Name:      "Abha Regional Airport",
		City:      "Abha",
		Country:   "SA",
		Elevation: 6858,
		Latitude:  18.2404003143,
		Longitude: 42.6566009521,
		Timezone:  "Asia/Riyadh",
	},
	"OEAH": {
		ICAO:      "OEAH",
		IATA:      "HOF",
		Name:      "Al Ahsa Airport",
		State:     "Eastern-Province",
		Country:   "SA",
		Elevation: 588,
		Latitude:  25.2852993011,
		Longitude: 49.4851989746,
		Timezone:  "Asia/Riyadh",
	},
	"OEAO": {
		ICAO:      "OEAO",
		IATA:      "ULH",
		Name:      "Prince Abdul Majeed bin Abdulaziz International Airport",
		City:      "Al-'Ula",
		State:     "Al Madinah",
		Country:   "SA",
		Elevation: 2047,
		Latitude:  26.483333,
		Longitude: 38.116944,
		Timezone:  "Asia/Riyadh",
	},
	"OEBA": {
		ICAO:      "OEBA",
		IATA:      "ABT",
		Name:      "Al Baha Airport",
		Country:   "SA",
		Elevation: 5486,
		Latitude:  20.2961006165,
		Longitude: 41.6343002319,
		Timezone:  "Asia/Riyadh",
	},
	"OEBH": {
		ICAO:      "OEBH",
		IATA:      "BHH",
		Name:      "Bisha Airport",
		Country:   "SA",
		Elevation: 3887,
		Latitude:  19.9843997955,
		Longitude: 42.6208992004,
		Timezone:  "Asia/Riyadh",
	},
	"OEBQ": {
		ICAO:      "OEBQ",
		Name:      "Abqaiq Airport",
		State:     "Eastern-Province",
		Country:   "SA",
		Elevation: 229,
		Latitude:  25.9113006592,
		Longitude: 49.5912017822,
		Timezone:  "Asia/Riyadh",
	},
	"OEDF": {
		ICAO:      "OEDF",
		IATA:      "DMM",
		Name:      "King Fahd International Airport",
		City:      "Ad Dammam",
		State:     "Eastern-Province",
		Country:   "SA",
		Elevation: 72,
		Latitude:  26.471200943,
		Longitude: 49.7979011536,
		Timezone:  "Asia/Riyadh",
	},
	"OEDM": {
		ICAO:      "OEDM",
		Name:      "Prince Salman Bin Abdulaziz Airport",
		State:     "Ar-Riyaḑ",
		Country:   "SA",
		Elevation: 3026,
		Latitude:  24.4498996735,
		Longitude: 44.1212005615,
		Timezone:  "Asia/Riyadh",
	},
	"OEDR": {
		ICAO:      "OEDR",
		IATA:      "DHA",
		Name:      "King Abdulaziz Air Base",
		State:     "Eastern-Province",
		Country:   "SA",
		Elevation: 84,
		Latitude:  26.2653999329,
		Longitude: 50.1520004272,
		Timezone:  "Asia/Riyadh",
	},
	"OEDW": {
		ICAO:      "OEDW",
		IATA:      "DWD",
		Name:      "Dawadmi Domestic Airport",
		City:      "Dawadmi",
		State:     "Ar-Riyaḑ",
		Country:   "SA",
		Elevation: 3429,
		Latitude:  24.5,
		Longitude: 44.4000015259,
		Timezone:  "Asia/Riyadh",
	},
	"OEGN": {
		ICAO:      "OEGN",
		IATA:      "GIZ",
		Name:      "Jizan Regional Airport",
		City:      "Jizan",
		State:     "Jizan",
		Country:   "SA",
		Elevation: 20,
		Latitude:  16.9011001587,
		Longitude: 42.5858001709,
		Timezone:  "Asia/Riyadh",
	},
	"OEGS": {
		ICAO:      "OEGS",
		IATA:      "ELQ",
		Name:      "Gassim Airport",
		Country:   "SA",
		Elevation: 2126,
		Latitude:  26.3027992249,
		Longitude: 43.7743988037,
		Timezone:  "Asia/Riyadh",
	},
	"OEGT": {
		ICAO:      "OEGT",
		IATA:      "URY",
		Name:      "Guriat Domestic Airport",
		State:     "Al-Jawf",
		Country:   "SA",
		Elevation: 1672,
		Latitude:  31.4118995667,
		Longitude: 37.279499054,
		Timezone:  "Asia/Riyadh",
	},
	"OEHL": {
		ICAO:      "OEHL",
		IATA:      "HAS",
		Name:      "Hail Airport",
		State:     "Ḩaʼil",
		Country:   "SA",
		Elevation: 3331,
		Latitude:  27.4379005432,
		Longitude: 41.6862983704,
		Timezone:  "Asia/Riyadh",
	},
	"OEHR": {
		ICAO:      "OEHR",
		Name:      "Harad Airport",
		City:      "Harad",
		State:     "Eastern-Province",
		Country:   "SA",
		Elevation: 110,
		Latitude:  24.1329994202,
		Longitude: 49.0670013428,
		Timezone:  "Asia/Riyadh",
	},
	"OEHW": {
		ICAO:      "OEHW",
		Name:      "Hawtah Airport",
		State:     "Ar-Riyaḑ",
		Country:   "SA",
		Elevation: 2091,
		Latitude:  22.9678001404,
		Longitude: 46.8995018005,
		Timezone:  "Asia/Riyadh",
	},
	"OEJB": {
		ICAO:      "OEJB",
		IATA:      "QJB",
		Name:      "Jubail Airport",
		City:      "Jubail",
		State:     "Eastern-Province",
		Country:   "SA",
		Elevation: 26,
		Latitude:  27.0389995575,
		Longitude: 49.4051017761,
		Timezone:  "Asia/Riyadh",
	},
	"OEJF": {
		ICAO:      "OEJF",
		Name:      "King Faisal Naval Base",
		City:      "Jeddah",
		State:     "Makkah",
		Country:   "SA",
		Elevation: 7,
		Latitude:  21.3481006622,
		Longitude: 39.1730003357,
		Timezone:  "Asia/Riyadh",
	},
	"OEJN": {
		ICAO:      "OEJN",
		IATA:      "JED",
		Name:      "King Abdulaziz International Airport",
		City:      "Jeddah",
		State:     "Makkah",
		Country:   "SA",
		Elevation: 48,
		Latitude:  21.679599762,
		Longitude: 39.15650177,
		Timezone:  "Asia/Riyadh",
	},
	"OEKK": {
		ICAO:      "OEKK",
		IATA:      "HBT",
		Name:      "King Khaled Military City Airport",
		City:      "King Khaled Military City",
		Country:   "SA",
		Elevation: 1352,
		Latitude:  27.9008998871,
		Longitude: 45.5281982422,
		Timezone:  "Asia/Riyadh",
	},
	"OEKM": {
		ICAO:      "OEKM",
		IATA:      "KMX",
		Name:      "King Khaled Air Base",
		Country:   "SA",
		Elevation: 6778,
		Latitude:  18.2973003387,
		Longitude: 42.8035011292,
		Timezone:  "Asia/Riyadh",
	},
	"OEKR": {
		ICAO:      "OEKR",
		Name:      "Khurais Airport",
		State:     "Eastern-Province",
		Country:   "SA",
		Elevation: 1300,
		Latitude:  25.0695991516,
		Longitude: 48.1949005127,
		Timezone:  "Asia/Riyadh",
	},
	"OEMA": {
		ICAO:      "OEMA",
		IATA:      "MED",
		Name:      "Prince Mohammad Bin Abdulaziz Airport",
		City:      "Medina",
		State:     "Al-Madinah-al-Munawwarah",
		Country:   "SA",
		Elevation: 2151,
		Latitude:  24.5534000397,
		Longitude: 39.7051010132,
		Timezone:  "Asia/Riyadh",
	},
	"OENG": {
		ICAO:      "OENG",
		IATA:      "EAM",
		Name:      "Nejran Airport",
		Country:   "SA",
		Elevation: 3982,
		Latitude:  17.6114006042,
		Longitude: 44.4192008972,
		Timezone:  "Asia/Riyadh",
	},
	"OENR": {
		ICAO:      "OENR",
		Name:      "Nariya Airport",
		City:      "Nariya",
		State:     "Eastern-Province",
		Country:   "SA",
		Elevation: 180,
		Latitude:  27.5,
		Longitude: 48.5,
		Timezone:  "Asia/Riyadh",
	},
	"OEPA": {
		ICAO:      "OEPA",
		IATA:      "AQI",
		Name:      "Hafr Al Batin Airport",
		City:      "Qaisumah",
		State:     "Eastern-Province",
		Country:   "SA",
		Elevation: 1174,
		Latitude:  28.3351993561,
		Longitude: 46.1250991821,
		Timezone:  "Asia/Riyadh",
	},
	"OEPC": {
		ICAO:      "OEPC",
		Name:      "Pump Station 3 Airport",
		State:     "Ar-Riyaḑ",
		Country:   "SA",
		Elevation: 1740,
		Latitude:  25.1744995117,
		Longitude: 47.4883995056,
		Timezone:  "Asia/Riyadh",
	},
	"OEPF": {
		ICAO:      "OEPF",
		Name:      "Pump Station 6 Airport",
		State:     "Ar-Riyaḑ",
		Country:   "SA",
		Elevation: 2530,
		Latitude:  24.7103004456,
		Longitude: 44.9645004272,
		Timezone:  "Asia/Riyadh",
	},
	"OEPI": {
		ICAO:      "OEPI",
		Name:      "Pump Station 9 Airport",
		State:     "Ar-Riyaḑ",
		Country:   "SA",
		Elevation: 3000,
		Latitude:  24.2761001587,
		Longitude: 42.143699646,
		Timezone:  "Asia/Riyadh",
	},
	"OEPJ": {
		ICAO:      "OEPJ",
		Name:      "Pump Station 10 Airport",
		Country:   "SA",
		Elevation: 2840,
		Latitude:  24.1072998047,
		Longitude: 41.0359992981,
		Timezone:  "Asia/Riyadh",
	},
	"OEPK": {
		ICAO:      "OEPK",
		Name:      "Ipsa 3 Airport",
		State:     "Eastern-Province",
		Country:   "SA",
		Elevation: 1000,
		Latitude:  27.9542007446,
		Longitude: 46.7425994873,
		Timezone:  "Asia/Riyadh",
	},
	"OEPS": {
		ICAO:      "OEPS",
		IATA:      "AKH",
		Name:      "Prince Sultan Air Base",
		State:     "Ar-Riyaḑ",
		Country:   "SA",
		Elevation: 1651,
		Latitude:  24.0627002716,
		Longitude: 47.5805015564,
		Timezone:  "Asia/Riyadh",
	},
	"OERB": {
		ICAO:      "OERB",
		Name:      "Rabigh Airport",
		State:     "Makkah",
		Country:   "SA",
		Elevation: 22,
		Latitude:  22.7026004791,
		Longitude: 39.0698013306,
		Timezone:  "Asia/Riyadh",
	},
	"OERF": {
		ICAO:      "OERF",
		IATA:      "RAH",
		Name:      "Rafha Domestic Airport",
		City:      "Rafha",
		State:     "Northern-Borders",
		Country:   "SA",
		Elevation: 1474,
		Latitude:  29.6263999939,
		Longitude: 43.4906005859,
		Timezone:  "Asia/Riyadh",
	},
	"OERK": {
		ICAO:      "OERK",
		IATA:      "RUH",
		Name:      "King Khaled International Airport",
		City:      "Riyadh",
		Country:   "SA",
		Elevation: 2049,
		Latitude:  24.9575996399,
		Longitude: 46.6987991333,
		Timezone:  "Asia/Riyadh",
	},
	"OERM": {
		ICAO:      "OERM",
		Name:      "Ras Mishab Airport",
		State:     "Eastern-Province",
		Country:   "SA",
		Elevation: 13,
		Latitude:  28.0795993805,
		Longitude: 48.611000061,
		Timezone:  "Asia/Riyadh",
	},
	"OERR": {
		ICAO:      "OERR",
		IATA:      "RAE",
		Name:      "Arar Domestic Airport",
		City:      "Arar",
		State:     "Northern-Borders",
		Country:   "SA",
		Elevation: 1813,
		Latitude:  30.9066009521,
		Longitude: 41.1381988525,
		Timezone:  "Asia/Riyadh",
	},
	"OERT": {
		ICAO:      "OERT",
		Name:      "Ras Tanura Airport",
		State:     "Eastern-Province",
		Country:   "SA",
		Elevation: 6,
		Latitude:  26.7231006622,
		Longitude: 50.0307998657,
		Timezone:  "Asia/Riyadh",
	},
	"OERY": {
		ICAO:      "OERY",
		IATA:      "XXN",
		Name:      "Riyadh Air Base",
		City:      "Riyadh",
		State:     "Ar-Riyaḑ",
		Country:   "SA",
		Elevation: 2082,
		Latitude:  24.7098007202,
		Longitude: 46.7252006531,
		Timezone:  "Asia/Riyadh",
	},
	"OESB": {
		ICAO:      "OESB",
		Name:      "Shaibah Airport",
		State:     "Eastern-Province",
		Country:   "SA",
		Elevation: 300,
		Latitude:  22.5144996643,
		Longitude: 53.964099884,
		Timezone:  "Asia/Riyadh",
	},
	"OESH": {
		ICAO:      "OESH",
		IATA:      "SHW",
		Name:      "Sharurah Airport",
		Country:   "SA",
		Elevation: 2363,
		Latitude:  17.4668998718,
		Longitude: 47.1213989258,
		Timezone:  "Asia/Riyadh",
	},
	"OESK": {
		ICAO:      "OESK",
		IATA:      "AJF",
		Name:      "Al-Jawf Domestic Airport",
		City:      "Al-Jawf",
		State:     "Al-Jawf",
		Country:   "SA",
		Elevation: 2261,
		Latitude:  29.7851009369,
		Longitude: 40.0999984741,
		Timezone:  "Asia/Riyadh",
	},
	"OESL": {
		ICAO:      "OESL",
		IATA:      "SLF",
		Name:      "Sulayel Airport",
		State:     "Ar-Riyaḑ",
		Country:   "SA",
		Elevation: 2021,
		Latitude:  20.4647006989,
		Longitude: 45.6195983887,
		Timezone:  "Asia/Riyadh",
	},
	"OETB": {
		ICAO:      "OETB",
		IATA:      "TUU",
		Name:      "Tabuk Airport",
		State:     "Tabuk",
		Country:   "SA",
		Elevation: 2551,
		Latitude:  28.3654003143,
		Longitude: 36.6189002991,
		Timezone:  "Asia/Riyadh",
	},
	"OETF": {
		ICAO:      "OETF",
		IATA:      "TIF",
		Name:      "Taif Airport",
		State:     "Makkah",
		Country:   "SA",
		Elevation: 4848,
		Latitude:  21.4834003448,
		Longitude: 40.5443000793,
		Timezone:  "Asia/Riyadh",
	},
	"OETH": {
		ICAO:      "OETH",
		Name:      "Thumamah Airport",
		State:     "Ar-Riyaḑ",
		Country:   "SA",
		Elevation: 1870,
		Latitude:  25.2129993439,
		Longitude: 46.6409988403,
		Timezone:  "Asia/Riyadh",
	},
	"OETN": {
		ICAO:      "OETN",
		Name:      "Ras Tanajib Airport",
		State:     "Eastern-Province",
		Country:   "SA",
		Elevation: 30,
		Latitude:  27.8677997589,
		Longitude: 48.7691001892,
		Timezone:  "Asia/Riyadh",
	},
	"OETR": {
		ICAO:      "OETR",
		IATA:      "TUI",
		Name:      "Turaif Domestic Airport",
		State:     "Northern-Borders",
		Country:   "SA",
		Elevation: 2803,
		Latitude:  31.6926994324,
		Longitude: 38.7312011719,
		Timezone:  "Asia/Riyadh",
	},
	"OEUD": {
		ICAO:      "OEUD",
		Name:      "Udhailiyah Airport",
		State:     "Eastern-Province",
		Country:   "SA",
		Elevation: 759,
		Latitude:  25.1511001587,
		Longitude: 49.3288002014,
		Timezone:  "Asia/Riyadh",
	},
	"OEUM": {
		ICAO:      "OEUM",
		Name:      "Umm Lajj Airport",
		City:      "Umm Lejj",
		State:     "Tabuk",
		Country:   "SA",
		Elevation: 50,
		Latitude:  25.1669998169,
		Longitude: 37.3330001831,
		Timezone:  "Asia/Riyadh",
	},
	"OEWD": {
		ICAO:      "OEWD",
		IATA:      "WAE",
		Name:      "Wadi Al Dawasir Airport",
		State:     "Ar-Riyaḑ",
		Country:   "SA",
		Elevation: 2062,
		Latitude:  20.5042991638,
		Longitude: 45.1996002197,
		Timezone:  "Asia/Riyadh",
	},
	"OEWJ": {
		ICAO:      "OEWJ",
		IATA:      "EJH",
		Name:      "Al Wajh Domestic Airport",
		City:      "Al Wajh",
		State:     "Tabuk",
		Country:   "SA",
		Elevation: 66,
		Latitude:  26.198600769,
		Longitude: 36.476398468,
		Timezone:  "Asia/Riyadh",
	},
	"OEYN": {
		ICAO:      "OEYN",
		IATA:      "YNB",
		Name:      "Yenbo Airport",
		State:     "Al-Madinah-al-Munawwarah",
		Country:   "SA",
		Elevation: 26,
		Latitude:  24.1441993713,
		Longitude: 38.0634002686,
		Timezone:  "Asia/Riyadh",
	},
	"OEZL": {
		ICAO:      "OEZL",
		IATA:      "ZUL",
		Name:      "Zilfi Airport",
		City:      "Zilfi",
		State:     "Ar-Riyaḑ",
		Country:   "SA",
		Elevation: 1900,
		Latitude:  26.3500003815,
		Longitude: 44.8330001831,
		Timezone:  "Asia/Riyadh",
	},
	"OG00": {
		ICAO:      "OG00",
		Name:      "3 Rivers Recreation Area Airport",
		City:      "Culver",
		State:     "Oregon",
		Country:   "US",
		Elevation: 2695,
		Latitude:  44.5578994751,
		Longitude: -121.4010009766,
		Timezone:  "America/Los_Angeles",
	},
	"OG01": {
		ICAO:      "OG01",
		Name:      "Dunning Vineyards Airport",
		City:      "Corvallis",
		State:     "Oregon",
		Country:   "US",
		Elevation: 240,
		Latitude:  44.5974998474,
		Longitude: -123.233001709,
		Timezone:  "America/Los_Angeles",
	},
	"OG02": {
		ICAO:      "OG02",
		Name:      "Rieben Airport",
		City:      "Banks",
		State:     "Oregon",
		Country:   "US",
		Elevation: 200,
		Latitude:  45.6102981567,
		Longitude: -123.0800018311,
		Timezone:  "America/Los_Angeles",
	},
	"OG04": {
		ICAO:      "OG04",
		Name:      "Rothrock Field",
		City:      "Adams",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1450,
		Latitude:  45.7486000061,
		Longitude: -118.6210021973,
		Timezone:  "America/Los_Angeles",
	},
	"OG05": {
		ICAO:      "OG05",
		Name:      "Sundance Meadows Airport",
		City:      "Bend",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3850,
		Latitude:  43.9900016785,
		Longitude: -121.2060012817,
		Timezone:  "America/Los_Angeles",
	},
	"OG06": {
		ICAO:      "OG06",
		Name:      "Table Rock Airport",
		City:      "Christmas Valley",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4305,
		Latitude:  43.2114982605,
		Longitude: -120.8430023193,
		Timezone:  "America/Los_Angeles",
	},
	"OG07": {
		ICAO:      "OG07",
		Name:      "Cove Side Ranch Port Airport",
		City:      "Cove",
		State:     "Oregon",
		Country:   "US",
		Elevation: 2770,
		Latitude:  45.3335990906,
		Longitude: -117.8199996948,
		Timezone:  "America/Los_Angeles",
	},
	"OG08": {
		ICAO:      "OG08",
		Name:      "Curtis Airfield",
		City:      "Helix",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1805,
		Latitude:  45.8086013794,
		Longitude: -118.65599823,
		Timezone:  "America/Los_Angeles",
	},
	"OG10": {
		ICAO:      "OG10",
		Name:      "Mt Hope Airport",
		City:      "Lebanon",
		State:     "Oregon",
		Country:   "US",
		Elevation: 400,
		Latitude:  44.5572013855,
		Longitude: -122.8259963989,
		Timezone:  "America/Los_Angeles",
	},
	"OG12": {
		ICAO:      "OG12",
		Name:      "Wilson Ranch Airport",
		City:      "Prineville",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4220,
		Latitude:  44.2122001648,
		Longitude: -120.5240020752,
		Timezone:  "America/Los_Angeles",
	},
	"OG13": {
		ICAO:      "OG13",
		Name:      "Fly By Night Airport",
		City:      "Ruch",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1660,
		Latitude:  42.2461013794,
		Longitude: -123.0699996948,
		Timezone:  "America/Los_Angeles",
	},
	"OG14": {
		ICAO:      "OG14",
		Name:      "Ponderosa Ranch Airport",
		City:      "Seneca",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4584,
		Latitude:  44.0242996216,
		Longitude: -118.8489990234,
		Timezone:  "America/Los_Angeles",
	},
	"OG15": {
		ICAO:      "OG15",
		Name:      "Sage Ranch Airport",
		City:      "Sisters",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3170,
		Latitude:  44.2332000732,
		Longitude: -121.3909988403,
		Timezone:  "America/Los_Angeles",
	},
	"OG16": {
		ICAO:      "OG16",
		Name:      "Jim's Airstrip",
		City:      "Halsey",
		State:     "Oregon",
		Country:   "US",
		Elevation: 275,
		Latitude:  44.382598877,
		Longitude: -123.2129974365,
		Timezone:  "America/Los_Angeles",
	},
	"OG17": {
		ICAO:      "OG17",
		Name:      "Teed's Airport",
		City:      "Lakeview",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4930,
		Latitude:  42.1848983765,
		Longitude: -120.466003418,
		Timezone:  "America/Los_Angeles",
	},
	"OG18": {
		ICAO:      "OG18",
		Name:      "Lawen Strip",
		City:      "Lawen",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4130,
		Latitude:  43.479598999,
		Longitude: -118.8310012817,
		Timezone:  "America/Los_Angeles",
	},
	"OG19": {
		ICAO:      "OG19",
		Name:      "Bombay Farms Airport",
		City:      "Madras",
		State:     "Oregon",
		Country:   "US",
		Elevation: 2385,
		Latitude:  44.650100708,
		Longitude: -121.2379989624,
		Timezone:  "America/Los_Angeles",
	},
	"OG20": {
		ICAO:      "OG20",
		Name:      "Fairways Airport",
		City:      "Oregon City",
		State:     "Oregon",
		Country:   "US",
		Elevation: 525,
		Latitude:  45.3207015991,
		Longitude: -122.5510025024,
		Timezone:  "America/Los_Angeles",
	},
	"OG21": {
		ICAO:      "OG21",
		Name:      "Dry Creek Airpark",
		City:      "Prineville",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3625,
		Latitude:  44.195098877,
		Longitude: -120.8229980469,
		Timezone:  "America/Los_Angeles",
	},
	"OG23": {
		ICAO:      "OG23",
		Name:      "Poverty Hollow Airport",
		City:      "Sheridan",
		State:     "Oregon",
		Country:   "US",
		Elevation: 330,
		Latitude:  45.1571006775,
		Longitude: -123.3909988403,
		Timezone:  "America/Los_Angeles",
	},
	"OG24": {
		ICAO:      "OG24",
		Name:      "Flying D Ranch Airport",
		City:      "Sutherlin",
		State:     "Oregon",
		Country:   "US",
		Elevation: 730,
		Latitude:  43.3886985779,
		Longitude: -123.2220001221,
		Timezone:  "America/Los_Angeles",
	},
	"OG25": {
		ICAO:      "OG25",
		Name:      "Firefly Ranch Airfield",
		City:      "White City",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1310,
		Latitude:  42.5111999512,
		Longitude: -122.9240036011,
		Timezone:  "America/Los_Angeles",
	},
	"OG26": {
		ICAO:      "OG26",
		Name:      "Roscoes Airport",
		City:      "Willamina",
		State:     "Oregon",
		Country:   "US",
		Elevation: 250,
		Latitude:  45.0619010925,
		Longitude: -123.486000061,
		Timezone:  "America/Los_Angeles",
	},
	"OG27": {
		ICAO:      "OG27",
		Name:      "Muddy Creek Airport",
		City:      "Haines",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3500,
		Latitude:  44.9401016235,
		Longitude: -118.0120010376,
		Timezone:  "America/Los_Angeles",
	},
	"OG28": {
		ICAO:      "OG28",
		Name:      "The Green Trees Ranch Airport",
		City:      "Scio",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1139,
		Latitude:  44.6832008362,
		Longitude: -122.7829971313,
		Timezone:  "America/Los_Angeles",
	},
	"OG29": {
		ICAO:      "OG29",
		Name:      "Mc Kinnon Airpark",
		City:      "Sandy",
		State:     "Oregon",
		Country:   "US",
		Elevation: 658,
		Latitude:  45.4306983948,
		Longitude: -122.241996765,
		Timezone:  "America/Los_Angeles",
	},
	"OG30": {
		ICAO:      "OG30",
		Name:      "Aeroacres Airport",
		City:      "Oregon City",
		State:     "Oregon",
		Country:   "US",
		Elevation: 420,
		Latitude:  45.3165016174,
		Longitude: -122.6050033569,
		Timezone:  "America/Los_Angeles",
	},
	"OG31": {
		ICAO:      "OG31",
		Name:      "Shady Cove Airpark",
		City:      "Shady Cove",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1504,
		Latitude:  42.6082000732,
		Longitude: -122.8259963989,
		Timezone:  "America/Los_Angeles",
	},
	"OG33": {
		ICAO:      "OG33",
		Name:      "Oregon Sky Ranch Airport",
		City:      "Milton/Freewater",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1357,
		Latitude:  45.8790016174,
		Longitude: -118.4049987793,
		Timezone:  "America/Los_Angeles",
	},
	"OG34": {
		ICAO:      "OG34",
		Name:      "Meyer Riverside Airpark",
		City:      "Tigard",
		State:     "Oregon",
		Country:   "US",
		Elevation: 252,
		Latitude:  45.3997993469,
		Longitude: -122.8290023804,
		Timezone:  "America/Los_Angeles",
	},
	"OG36": {
		ICAO:      "OG36",
		Name:      "Munson Airport",
		City:      "Junction City",
		State:     "Oregon",
		Country:   "US",
		Elevation: 302,
		Latitude:  44.2504005432,
		Longitude: -123.2279968262,
		Timezone:  "America/Los_Angeles",
	},
	"OG39": {
		ICAO:      "OG39",
		Name:      "Longview Ranch Airport",
		City:      "Kimberly",
		State:     "Oregon",
		Country:   "US",
		Elevation: 2080,
		Latitude:  44.6617012024,
		Longitude: -119.6520004272,
		Timezone:  "America/Los_Angeles",
	},
	"OG40": {
		ICAO:      "OG40",
		Name:      "Napier Ranch Airport",
		City:      "Roseburg",
		State:     "Oregon",
		Country:   "US",
		Elevation: 485,
		Latitude:  43.2373008728,
		Longitude: -123.4929962158,
		Timezone:  "America/Los_Angeles",
	},
	"OG41": {
		ICAO:      "OG41",
		Name:      "Nace Family Airstrip",
		City:      "Glendale",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1550,
		Latitude:  42.7711982727,
		Longitude: -123.3150024414,
		Timezone:  "America/Los_Angeles",
	},
	"OG42": {
		ICAO:      "OG42",
		Name:      "Quail Field",
		City:      "Pendleton",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1610,
		Latitude:  45.6548995972,
		Longitude: -118.5899963379,
		Timezone:  "America/Los_Angeles",
	},
	"OG44": {
		ICAO:      "OG44",
		Name:      "Watts Landing Airport",
		City:      "Amity",
		State:     "Oregon",
		Country:   "US",
		Elevation: 155,
		Latitude:  45.1534004211,
		Longitude: -123.2369995117,
		Timezone:  "America/Los_Angeles",
	},
	"OG45": {
		ICAO:      "OG45",
		Name:      "Bald Mountain Airport",
		City:      "Baker City",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4640,
		Latitude:  44.6169013977,
		Longitude: -117.8720016479,
		Timezone:  "America/Los_Angeles",
	},
	"OG48": {
		ICAO:      "OG48",
		Name:      "Greer Airport",
		City:      "Coburg",
		State:     "Oregon",
		Country:   "US",
		Elevation: 360,
		Latitude:  44.1958999634,
		Longitude: -123.0729980469,
		Timezone:  "America/Los_Angeles",
	},
	"OG49": {
		ICAO:      "OG49",
		Name:      "Coca Cola Airport",
		City:      "Corvalis",
		State:     "Oregon",
		Country:   "US",
		Elevation: 300,
		Latitude:  44.4219017029,
		Longitude: -123.2590026855,
		Timezone:  "America/Los_Angeles",
	},
	"OG50": {
		ICAO:      "OG50",
		Name:      "Cottonwood Creek Ranch Airport",
		City:      "Harper",
		State:     "Oregon",
		Country:   "US",
		Elevation: 2555,
		Latitude:  43.8471984863,
		Longitude: -117.5739974976,
		Timezone:  "America/Boise",
	},
	"OG51": {
		ICAO:      "OG51",
		Name:      "Six Springs Ranch Airport",
		City:      "Madras",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3780,
		Latitude:  44.4835014343,
		Longitude: -120.9160003662,
		Timezone:  "America/Los_Angeles",
	},
	"OG52": {
		ICAO:      "OG52",
		Name:      "Jpm Airport",
		City:      "Monmouth",
		State:     "Oregon",
		Country:   "US",
		Elevation: 260,
		Latitude:  44.8391990662,
		Longitude: -123.2590026855,
		Timezone:  "America/Los_Angeles",
	},
	"OG53": {
		ICAO:      "OG53",
		Name:      "Wildhorse Valley Airport",
		City:      "Andrews",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4225,
		Latitude:  42.4878997803,
		Longitude: -118.608001709,
		Timezone:  "America/Los_Angeles",
	},
	"OG54": {
		ICAO:      "OG54",
		Name:      "Shaniko Cattle Airport",
		City:      "Shaniko",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3300,
		Latitude:  44.9600982666,
		Longitude: -120.8850021362,
		Timezone:  "America/Los_Angeles",
	},
	"OG63": {
		ICAO:      "OG63",
		Name:      "Gilbert Airport",
		City:      "North Plains",
		State:     "Oregon",
		Country:   "US",
		Elevation: 211,
		Latitude:  45.648889,
		Longitude: -123.039444,
		Timezone:  "America/Los_Angeles",
	},
	"OH02": {
		ICAO:      "OH02",
		Name:      "Eickmeier Airport",
		City:      "Mc Clure",
		State:     "Ohio",
		Country:   "US",
		Elevation: 694,
		Latitude:  41.2944984436,
		Longitude: -83.9546966553,
		Timezone:  "America/New_York",
	},
	"OH03": {
		ICAO:      "OH03",
		Name:      "Woodworth Airport",
		City:      "Madison",
		State:     "Ohio",
		Country:   "US",
		Elevation: 678,
		Latitude:  41.792301178,
		Longitude: -81.0581970215,
		Timezone:  "America/New_York",
	},
	"OH07": {
		ICAO:      "OH07",
		Name:      "Sunset Strip",
		City:      "Marlboro",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1190,
		Latitude:  40.943901062,
		Longitude: -81.2389984131,
		Timezone:  "America/New_York",
	},
	"OH11": {
		ICAO:      "OH11",
		Name:      "Arnold Airport",
		City:      "Harrisonville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 930,
		Latitude:  39.1105995178,
		Longitude: -82.0922012329,
		Timezone:  "America/New_York",
	},
	"OH13": {
		ICAO:      "OH13",
		Name:      "Bulick Field",
		City:      "Minerva",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1285,
		Latitude:  40.7681007385,
		Longitude: -81.0581970215,
		Timezone:  "America/New_York",
	},
	"OH14": {
		ICAO:      "OH14",
		Name:      "Brown's Lakeside Landings Airport",
		City:      "Sunbury",
		State:     "Ohio",
		Country:   "US",
		Elevation: 970,
		Latitude:  40.2792015076,
		Longitude: -82.9168014526,
		Timezone:  "America/New_York",
	},
	"OH15": {
		ICAO:      "OH15",
		Name:      "Minerva Airport",
		City:      "Minerva",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1080,
		Latitude:  40.7458992004,
		Longitude: -81.1443023682,
		Timezone:  "America/New_York",
	},
	"OH17": {
		ICAO:      "OH17",
		Name:      "August Acres Airport",
		City:      "Center Village",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1100,
		Latitude:  40.1626014709,
		Longitude: -82.7817993164,
		Timezone:  "America/New_York",
	},
	"OH18": {
		ICAO:      "OH18",
		Name:      "Freefall Field",
		City:      "Sycamore",
		State:     "Ohio",
		Country:   "US",
		Elevation: 885,
		Latitude:  40.9324989319,
		Longitude: -83.140296936,
		Timezone:  "America/New_York",
	},
	"OH19": {
		ICAO:      "OH19",
		Name:      "Hiltner Airport",
		City:      "North Creek",
		State:     "Ohio",
		Country:   "US",
		Elevation: 725,
		Latitude:  41.1549987793,
		Longitude: -84.2148971558,
		Timezone:  "America/New_York",
	},
	"OH20": {
		ICAO:      "OH20",
		Name:      "Spring Valley Farm Airport",
		City:      "Marshallville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1000,
		Latitude:  40.9197998047,
		Longitude: -81.7012023926,
		Timezone:  "America/New_York",
	},
	"OH21": {
		ICAO:      "OH21",
		Name:      "Horning Airport",
		City:      "Galion",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1190,
		Latitude:  40.7361984253,
		Longitude: -82.7527008057,
		Timezone:  "America/New_York",
	},
	"OH22": {
		ICAO:      "OH22",
		Name:      "Stoltzfus Airfield",
		City:      "Kidron",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1130,
		Latitude:  40.7611999512,
		Longitude: -81.7789993286,
		Timezone:  "America/New_York",
	},
	"OH24": {
		ICAO:      "OH24",
		Name:      "Brookfield Airpark",
		City:      "Brookfield",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1040,
		Latitude:  41.2248001099,
		Longitude: -80.5790023804,
		Timezone:  "America/New_York",
	},
	"OH26": {
		ICAO:      "OH26",
		Name:      "Birdland Airport",
		City:      "Thompson",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1160,
		Latitude:  41.6563987732,
		Longitude: -81.1062011719,
		Timezone:  "America/New_York",
	},
	"OH27": {
		ICAO:      "OH27",
		Name:      "Salem Lakefront Airport",
		City:      "Salem",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1170,
		Latitude:  40.807800293,
		Longitude: -80.8453979492,
		Timezone:  "America/New_York",
	},
	"OH28": {
		ICAO:      "OH28",
		Name:      "Donner Field",
		City:      "Leesburg",
		State:     "Ohio",
		Country:   "US",
		Elevation: 990,
		Latitude:  39.3666992188,
		Longitude: -83.5999984741,
		Timezone:  "America/New_York",
	},
	"OH29": {
		ICAO:      "OH29",
		Name:      "Petersburg Airport",
		City:      "Petersburg",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1174,
		Latitude:  40.9006004333,
		Longitude: -80.5553970337,
		Timezone:  "America/New_York",
	},
	"OH32": {
		ICAO:      "OH32",
		Name:      "Stone Airport",
		City:      "Alvada",
		State:     "Ohio",
		Country:   "US",
		Elevation: 835,
		Latitude:  41.006401062,
		Longitude: -83.4273986816,
		Timezone:  "America/New_York",
	},
	"OH33": {
		ICAO:      "OH33",
		Name:      "Humphries Rotordrome Airport",
		City:      "Amelia",
		State:     "Ohio",
		Country:   "US",
		Elevation: 840,
		Latitude:  39.0567016602,
		Longitude: -84.2041015625,
		Timezone:  "America/New_York",
	},
	"OH34": {
		ICAO:      "OH34",
		Name:      "Leis Airport",
		City:      "Ansonia",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1015,
		Latitude:  40.2058982849,
		Longitude: -84.6615982056,
		Timezone:  "America/New_York",
	},
	"OH35": {
		ICAO:      "OH35",
		Name:      "Mission Field",
		City:      "Hallsville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 900,
		Latitude:  39.4283981323,
		Longitude: -82.8227005005,
		Timezone:  "America/New_York",
	},
	"OH36": {
		ICAO:      "OH36",
		Name:      "Riverside Airport",
		City:      "Zanesville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 700,
		Latitude:  39.9861984253,
		Longitude: -81.9837036133,
		Timezone:  "America/New_York",
	},
	"OH37": {
		ICAO:      "OH37",
		Name:      "Glade STOLport",
		City:      "Beaver",
		State:     "Ohio",
		Country:   "US",
		Elevation: 690,
		Latitude:  39.0116996765,
		Longitude: -82.7884979248,
		Timezone:  "America/New_York",
	},
	"OH38": {
		ICAO:      "OH38",
		Name:      "Blue Bird Airport",
		City:      "Belpre",
		State:     "Ohio",
		Country:   "US",
		Elevation: 780,
		Latitude:  39.3144989014,
		Longitude: -81.6165008545,
		Timezone:  "America/New_York",
	},
	"OH39": {
		ICAO:      "OH39",
		Name:      "Pine Lake Airport",
		City:      "Berkshire",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1000,
		Latitude:  40.2347984314,
		Longitude: -82.9192962646,
		Timezone:  "America/New_York",
	},
	"OH41": {
		ICAO:      "OH41",
		Name:      "Cutler Field",
		City:      "Powell",
		State:     "Ohio",
		Country:   "US",
		Elevation: 945,
		Latitude:  40.1930999756,
		Longitude: -83.1119003296,
		Timezone:  "America/New_York",
	},
	"OH42": {
		ICAO:      "OH42",
		Name:      "Hawk's Nest Airport",
		City:      "Butler",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1260,
		Latitude:  40.5401000977,
		Longitude: -82.3937988281,
		Timezone:  "America/New_York",
	},
	"OH43": {
		ICAO:      "OH43",
		Name:      "Anderson Airport",
		City:      "Canal Winchester",
		State:     "Ohio",
		Country:   "US",
		Elevation: 800,
		Latitude:  39.8334007263,
		Longitude: -82.7332000732,
		Timezone:  "America/New_York",
	},
	"OH44": {
		ICAO:      "OH44",
		Name:      "Grover Airport",
		City:      "Cheshire",
		State:     "Ohio",
		Country:   "US",
		Elevation: 945,
		Latitude:  40.2206001282,
		Longitude: -82.9462966919,
		Timezone:  "America/New_York",
	},
	"OH45": {
		ICAO:      "OH45",
		Name:      "Drake Airport",
		City:      "Bowling Green",
		State:     "Ohio",
		Country:   "US",
		Elevation: 655,
		Latitude:  41.4542007446,
		Longitude: -83.5863037109,
		Timezone:  "America/New_York",
	},
	"OH48": {
		ICAO:      "OH48",
		Name:      "Alliance Airport",
		City:      "Alliance",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1077,
		Latitude:  40.9056015015,
		Longitude: -81.0651016235,
		Timezone:  "America/New_York",
	},
	"OH50": {
		ICAO:      "OH50",
		Name:      "Lindsey L.S. Airport",
		City:      "Circleville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 780,
		Latitude:  39.6875991821,
		Longitude: -83.0969009399,
		Timezone:  "America/New_York",
	},
	"OH51": {
		ICAO:      "OH51",
		Name:      "Miami Valley Career Technology Center Airport",
		City:      "Clayton",
		State:     "Ohio",
		Country:   "US",
		Elevation: 930,
		Latitude:  39.8433990479,
		Longitude: -84.3312988281,
		Timezone:  "America/New_York",
	},
	"OH57": {
		ICAO:      "OH57",
		Name:      "Berlin Station Lndg Strip",
		City:      "Delaware",
		State:     "Ohio",
		Country:   "US",
		Elevation: 955,
		Latitude:  40.2588996887,
		Longitude: -83.0118026733,
		Timezone:  "America/New_York",
	},
	"OH59": {
		ICAO:      "OH59",
		Name:      "Mc Intosh Airport",
		City:      "Farmersville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 914,
		Latitude:  39.6958999634,
		Longitude: -84.448600769,
		Timezone:  "America/New_York",
	},
	"OH60": {
		ICAO:      "OH60",
		Name:      "Gerker Airfield",
		City:      "Fort Jennings",
		State:     "Ohio",
		Country:   "US",
		Elevation: 780,
		Latitude:  40.9537010193,
		Longitude: -84.2735977173,
		Timezone:  "America/New_York",
	},
	"OH61": {
		ICAO:      "OH61",
		Name:      "Amanda Airport",
		City:      "Amanda",
		State:     "Ohio",
		Country:   "US",
		Elevation: 900,
		Latitude:  39.6352996826,
		Longitude: -82.7378997803,
		Timezone:  "America/New_York",
	},
	"OH63": {
		ICAO:      "OH63",
		Name:      "Jims Airport",
		City:      "Germantown",
		State:     "Ohio",
		Country:   "US",
		Elevation: 955,
		Latitude:  39.6001014709,
		Longitude: -84.4166030884,
		Timezone:  "America/New_York",
	},
	"OH64": {
		ICAO:      "OH64",
		Name:      "Hummel Airport",
		City:      "Germantown",
		State:     "Ohio",
		Country:   "US",
		Elevation: 695,
		Latitude:  39.5931015015,
		Longitude: -84.3438034058,
		Timezone:  "America/New_York",
	},
	"OH65": {
		ICAO:      "OH65",
		Name:      "Antique Acres Airpark",
		City:      "Goshen",
		State:     "Ohio",
		Country:   "US",
		Elevation: 895,
		Latitude:  39.2393989563,
		Longitude: -84.1003036499,
		Timezone:  "America/New_York",
	},
	"OH66": {
		ICAO:      "OH66",
		Name:      "Obannon Creek Aerodrome",
		City:      "Goshen",
		State:     "Ohio",
		Country:   "US",
		Elevation: 835,
		Latitude:  39.2394981384,
		Longitude: -84.1763000488,
		Timezone:  "America/New_York",
	},
	"OH67": {
		ICAO:      "OH67",
		Name:      "Sell Field",
		City:      "Bradford",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1018,
		Latitude:  40.1206016541,
		Longitude: -84.4535980225,
		Timezone:  "America/New_York",
	},
	"OH68": {
		ICAO:      "OH68",
		Name:      "Raylene Airport",
		City:      "Harrison",
		State:     "Ohio",
		Country:   "US",
		Elevation: 590,
		Latitude:  39.3000984192,
		Longitude: -84.7499008179,
		Timezone:  "America/New_York",
	},
	"OH71": {
		ICAO:      "OH71",
		Name:      "Chapman Memorial Field",
		City:      "Centerburg",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1180,
		Latitude:  40.2947998047,
		Longitude: -82.731300354,
		Timezone:  "America/New_York",
	},
	"OH74": {
		ICAO:      "OH74",
		Name:      "Kearns Airport",
		City:      "Johnstown",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1210,
		Latitude:  40.0937004089,
		Longitude: -82.716796875,
		Timezone:  "America/New_York",
	},
	"OH77": {
		ICAO:      "OH77",
		Name:      "Lumberton Airport",
		City:      "Wilmington",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1008,
		Latitude:  39.5531005859,
		Longitude: -83.8548965454,
		Timezone:  "America/New_York",
	},
	"OH78": {
		ICAO:      "OH78",
		Name:      "Swigart Airport",
		City:      "Laura",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1005,
		Latitude:  39.9676017761,
		Longitude: -84.4087982178,
		Timezone:  "America/New_York",
	},
	"OH81": {
		ICAO:      "OH81",
		Name:      "Graham's Landing Airport",
		City:      "Valley City",
		State:     "Ohio",
		Country:   "US",
		Elevation: 890,
		Latitude:  41.2256011963,
		Longitude: -81.9414978027,
		Timezone:  "America/New_York",
	},
	"OH82": {
		ICAO:      "OH82",
		Name:      "Graham Farm Airport",
		City:      "Adams Mills",
		State:     "Ohio",
		Country:   "US",
		Elevation: 725,
		Latitude:  40.1534004211,
		Longitude: -81.9735031128,
		Timezone:  "America/New_York",
	},
	"OH86": {
		ICAO:      "OH86",
		Name:      "Deeds Field",
		City:      "Sugar Grove",
		State:     "Ohio",
		Country:   "US",
		Elevation: 800,
		Latitude:  39.648399353,
		Longitude: -82.5399017334,
		Timezone:  "America/New_York",
	},
	"OH88": {
		ICAO:      "OH88",
		Name:      "Roberts Airport",
		City:      "London",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1066,
		Latitude:  39.8527984619,
		Longitude: -83.4704971313,
		Timezone:  "America/New_York",
	},
	"OH90": {
		ICAO:      "OH90",
		Name:      "Hochstetler Airport",
		City:      "Lyons",
		State:     "Ohio",
		Country:   "US",
		Elevation: 763,
		Latitude:  41.7072982788,
		Longitude: -84.234703064,
		Timezone:  "America/Detroit",
	},
	"OH91": {
		ICAO:      "OH91",
		Name:      "Dave Rice Ultralightport",
		City:      "Lima",
		State:     "Ohio",
		Country:   "US",
		Elevation: 775,
		Latitude:  40.70009995,
		Longitude: -84.23439789,
		Timezone:  "America/New_York",
	},
	"OH92": {
		ICAO:      "OH92",
		Name:      "Porter Airport",
		City:      "Hopewell",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1070,
		Latitude:  39.9542007446,
		Longitude: -82.1831970215,
		Timezone:  "America/New_York",
	},
	"OH93": {
		ICAO:      "OH93",
		Name:      "Darby Airport",
		City:      "Milford Center",
		State:     "Ohio",
		Country:   "US",
		Elevation: 972,
		Latitude:  40.1403007507,
		Longitude: -83.3901977539,
		Timezone:  "America/New_York",
	},
	"OH94": {
		ICAO:      "OH94",
		Name:      "Knore Airport",
		City:      "Minford",
		State:     "Ohio",
		Country:   "US",
		Elevation: 674,
		Latitude:  38.837600708,
		Longitude: -82.8348999023,
		Timezone:  "America/New_York",
	},
	"OH95": {
		ICAO:      "OH95",
		Name:      "Maplewood Orchard Airport",
		City:      "Morrow",
		State:     "Ohio",
		Country:   "US",
		Elevation: 880,
		Latitude:  39.3842010498,
		Longitude: -84.168296814,
		Timezone:  "America/New_York",
	},
	"OH96": {
		ICAO:      "OH96",
		Name:      "Neals Airport",
		City:      "Mount Orab",
		State:     "Ohio",
		Country:   "US",
		Elevation: 953,
		Latitude:  39.0335998535,
		Longitude: -83.886100769,
		Timezone:  "America/New_York",
	},
	"OH97": {
		ICAO:      "OH97",
		Name:      "Carcioppolo Field",
		City:      "Mantua",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1165,
		Latitude:  41.3591995239,
		Longitude: -81.2177963257,
		Timezone:  "America/New_York",
	},
	"OH98": {
		ICAO:      "OH98",
		Name:      "Lyttle Airport",
		City:      "New Albany",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1050,
		Latitude:  40.0494995117,
		Longitude: -82.7882003784,
		Timezone:  "America/New_York",
	},
	"OH99": {
		ICAO:      "OH99",
		Name:      "Planevue Airport",
		City:      "Newark",
		State:     "Ohio",
		Country:   "US",
		Elevation: 935,
		Latitude:  40.1139984131,
		Longitude: -82.4581985474,
		Timezone:  "America/New_York",
	},
	"OI01": {
		ICAO:      "OI01",
		Name:      "Lazy-W Airport",
		City:      "Logan",
		State:     "Ohio",
		Country:   "US",
		Elevation: 750,
		Latitude:  39.5136985779,
		Longitude: -82.4489974976,
		Timezone:  "America/New_York",
	},
	"OI03": {
		ICAO:      "OI03",
		Name:      "Garmcar Sw Airport",
		State:     "Semnan",
		Country:   "IR",
		Elevation: 2717,
		Latitude:  35.1741981506,
		Longitude: 52.323299408,
		Timezone:  "Asia/Tehran",
	},
	"OI06": {
		ICAO:      "OI06",
		Name:      "Wagner International Airport",
		City:      "West Milton",
		State:     "Ohio",
		Country:   "US",
		Elevation: 935,
		Latitude:  39.9914016724,
		Longitude: -84.2916030884,
		Timezone:  "America/New_York",
	},
	"OI07": {
		ICAO:      "OI07",
		Name:      "Varns Farms Airport",
		City:      "Wooster",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1110,
		Latitude:  40.827999115,
		Longitude: -81.9796981812,
		Timezone:  "America/New_York",
	},
	"OI08": {
		ICAO:      "OI08",
		Name:      "Davies Air Field",
		City:      "Wooster",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1200,
		Latitude:  40.738899231,
		Longitude: -81.8716964722,
		Timezone:  "America/New_York",
	},
	"OI11": {
		ICAO:      "OI11",
		Name:      "Hibbetts Airport",
		City:      "Carrollton",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1060,
		Latitude:  40.637298584,
		Longitude: -81.1126022339,
		Timezone:  "America/New_York",
	},
	"OI12": {
		ICAO:      "OI12",
		Name:      "Fly-A-Way Farm Airport",
		City:      "Delaware",
		State:     "Ohio",
		Country:   "US",
		Elevation: 940,
		Latitude:  40.2503013611,
		Longitude: -82.9943008423,
		Timezone:  "America/New_York",
	},
	"OI13": {
		ICAO:      "OI13",
		Name:      "Mahmudabad Airport",
		State:     "Fars",
		Country:   "IR",
		Elevation: 6970,
		Latitude:  30,
		Longitude: 53.7114982605,
		Timezone:  "Asia/Tehran",
	},
	"OI19": {
		ICAO:      "OI19",
		Name:      "Firuzabad Airport",
		State:     "Tehran",
		Country:   "IR",
		Elevation: 3258,
		Latitude:  35.5298995972,
		Longitude: 51.5057983398,
		Timezone:  "Asia/Tehran",
	},
	"OI1C": {
		ICAO:      "OI1C",
		Name:      "Mehtar Kalateh Airport",
		State:     "Golestan",
		Country:   "IR",
		Elevation: -64,
		Latitude:  36.8568992615,
		Longitude: 54.2022018433,
		Timezone:  "Asia/Tehran",
	},
	"OI20": {
		ICAO:      "OI20",
		Name:      "Bahregan Airport",
		State:     "Bushehr",
		Country:   "IR",
		Elevation: 20,
		Latitude:  29.8400993347,
		Longitude: 50.2727012634,
		Timezone:  "Asia/Tehran",
	},
	"OI21": {
		ICAO:      "OI21",
		Name:      "Semnan New Airport",
		State:     "Semnan",
		Country:   "IR",
		Elevation: 3659,
		Latitude:  35.3894004822,
		Longitude: 53.6713981628,
		Timezone:  "Asia/Tehran",
	},
	"OI22": {
		ICAO:      "OI22",
		Name:      "Carroll's Airport",
		City:      "Elyria",
		State:     "Ohio",
		Country:   "US",
		Elevation: 755,
		Latitude:  41.3339004517,
		Longitude: -82.1026000977,
		Timezone:  "America/New_York",
	},
	"OI24": {
		ICAO:      "OI24",
		Name:      "Gonbad Ghabus Airport",
		State:     "Golestan",
		Country:   "IR",
		Elevation: 107,
		Latitude:  37.2461013794,
		Longitude: 55.0959014893,
		Timezone:  "Asia/Tehran",
	},
	"OI25": {
		ICAO:      "OI25",
		Name:      "Ross Field",
		City:      "Greenfield",
		State:     "Ohio",
		Country:   "US",
		Elevation: 960,
		Latitude:  39.3652992249,
		Longitude: -83.3544006348,
		Timezone:  "America/New_York",
	},
	"OI26": {
		ICAO:      "OI26",
		Name:      "Hillcrest Airport",
		City:      "Oxford",
		State:     "Ohio",
		Country:   "US",
		Elevation: 890,
		Latitude:  39.5055999756,
		Longitude: -84.6987991333,
		Timezone:  "America/New_York",
	},
	"OI27": {
		ICAO:      "OI27",
		Name:      "Checkpoint Charlie Airport",
		City:      "Lowell",
		State:     "Ohio",
		Country:   "US",
		Elevation: 640,
		Latitude:  39.5567016602,
		Longitude: -81.5696029663,
		Timezone:  "America/New_York",
	},
	"OI28": {
		ICAO:      "OI28",
		Name:      "Lanker Airport",
		City:      "Mount Gilead",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1440,
		Latitude:  40.5839004517,
		Longitude: -82.6792984009,
		Timezone:  "America/New_York",
	},
	"OI29": {
		ICAO:      "OI29",
		Name:      "Yellowbird Farm Airport",
		City:      "Otway",
		State:     "Ohio",
		Country:   "US",
		Elevation: 610,
		Latitude:  38.8541984558,
		Longitude: -83.2304992676,
		Timezone:  "America/New_York",
	},
	"OI31": {
		ICAO:      "OI31",
		Name:      "Pheasant Run Airport",
		City:      "Leroy",
		State:     "Ohio",
		Country:   "US",
		Elevation: 960,
		Latitude:  41.7098007202,
		Longitude: -81.1248016357,
		Timezone:  "America/New_York",
	},
	"OI32": {
		ICAO:      "OI32",
		Name:      "Stoney's Airport",
		City:      "Ravenna",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1105,
		Latitude:  41.1680984497,
		Longitude: -81.287902832,
		Timezone:  "America/New_York",
	},
	"OI34": {
		ICAO:      "OI34",
		Name:      "Fricke Airport",
		City:      "Spring Valley",
		State:     "Ohio",
		Country:   "US",
		Elevation: 905,
		Latitude:  39.6130981445,
		Longitude: -83.992401123,
		Timezone:  "America/New_York",
	},
	"OI36": {
		ICAO:      "OI36",
		Name:      "Farpoint Airfield",
		City:      "Gratis",
		State:     "Ohio",
		Country:   "US",
		Elevation: 990,
		Latitude:  39.6225013733,
		Longitude: -84.4831008911,
		Timezone:  "America/New_York",
	},
	"OI38": {
		ICAO:      "OI38",
		Name:      "Hillview Airstrip",
		City:      "East Liberty",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1400,
		Latitude:  40.3087005615,
		Longitude: -83.615196228,
		Timezone:  "America/New_York",
	},
	"OI39": {
		ICAO:      "OI39",
		Name:      "Vensil Farms Airport",
		City:      "Frazeysburg",
		State:     "Ohio",
		Country:   "US",
		Elevation: 760,
		Latitude:  40.1067008972,
		Longitude: -82.1321029663,
		Timezone:  "America/New_York",
	},
	"OI40": {
		ICAO:      "OI40",
		Name:      "Hallelujah Field",
		City:      "Goshen",
		State:     "Ohio",
		Country:   "US",
		Elevation: 890,
		Latitude:  39.2025985718,
		Longitude: -84.1046981812,
		Timezone:  "America/New_York",
	},
	"OI41": {
		ICAO:      "OI41",
		Name:      "Far View Airport",
		City:      "Hiram",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1300,
		Latitude:  41.295728,
		Longitude: -81.158165,
		Timezone:  "America/New_York",
	},
	"OI42": {
		ICAO:      "OI42",
		Name:      "D. A. Chandler Airport",
		City:      "New London",
		State:     "Ohio",
		Country:   "US",
		Elevation: 950,
		Latitude:  41.1328010559,
		Longitude: -82.4607009888,
		Timezone:  "America/New_York",
	},
	"OI45": {
		ICAO:      "OI45",
		Name:      "Shootz Field",
		City:      "Richfield",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1250,
		Latitude:  41.2650985718,
		Longitude: -81.6586990356,
		Timezone:  "America/New_York",
	},
	"OI46": {
		ICAO:      "OI46",
		Name:      "Sharondale Field",
		City:      "Sharon Center",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1180,
		Latitude:  41.1151008606,
		Longitude: -81.7151031494,
		Timezone:  "America/New_York",
	},
	"OI49": {
		ICAO:      "OI49",
		Name:      "Flying Acres Airport",
		City:      "Delaware",
		State:     "Ohio",
		Country:   "US",
		Elevation: 975,
		Latitude:  40.3838996887,
		Longitude: -83.1069030762,
		Timezone:  "America/New_York",
	},
	"OI52": {
		ICAO:      "OI52",
		Name:      "Lyons Field",
		City:      "Jamestown",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1045,
		Latitude:  39.6250991821,
		Longitude: -83.7833023071,
		Timezone:  "America/New_York",
	},
	"OI55": {
		ICAO:      "OI55",
		Name:      "Scheibe Field",
		City:      "Wooster",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1010,
		Latitude:  40.7509002686,
		Longitude: -82.0043029785,
		Timezone:  "America/New_York",
	},
	"OI56": {
		ICAO:      "OI56",
		Name:      "Lee's Dogpatch Airport",
		City:      "Kirkersville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1055,
		Latitude:  39.95320129,
		Longitude: -82.64070129,
		Timezone:  "America/New_York",
	},
	"OI57": {
		ICAO:      "OI57",
		Name:      "Mc Kinney Field",
		City:      "Hamersville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 970,
		Latitude:  38.9269981384,
		Longitude: -83.9885025024,
		Timezone:  "America/New_York",
	},
	"OI58": {
		ICAO:      "OI58",
		Name:      "Lockeridge Airport",
		City:      "Canton",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1090,
		Latitude:  40.8417015076,
		Longitude: -81.4378967285,
		Timezone:  "America/New_York",
	},
	"OI59": {
		ICAO:      "OI59",
		Name:      "Miller Airport",
		City:      "Chester",
		State:     "Ohio",
		Country:   "US",
		Elevation: 690,
		Latitude:  39.1316986084,
		Longitude: -81.9318008423,
		Timezone:  "America/New_York",
	},
	"OI60": {
		ICAO:      "OI60",
		Name:      "Gorman Airport",
		City:      "Mansfield",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1230,
		Latitude:  40.7547988892,
		Longitude: -82.3916015625,
		Timezone:  "America/New_York",
	},
	"OI62": {
		ICAO:      "OI62",
		Name:      "Rataiczak Airport",
		City:      "Russell",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1200,
		Latitude:  41.476398468,
		Longitude: -81.3171005249,
		Timezone:  "America/New_York",
	},
	"OI64": {
		ICAO:      "OI64",
		Name:      "Bush Field",
		City:      "Thompson",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1145,
		Latitude:  41.6805992126,
		Longitude: -81.0301971436,
		Timezone:  "America/New_York",
	},
	"OI66": {
		ICAO:      "OI66",
		Name:      "Eibling Circle E Airport",
		City:      "Dola",
		State:     "Ohio",
		Country:   "US",
		Elevation: 980,
		Latitude:  40.7408981323,
		Longitude: -83.6709976196,
		Timezone:  "America/New_York",
	},
	"OI67": {
		ICAO:      "OI67",
		Name:      "Mc Clain Field",
		City:      "Mechanicstown",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1250,
		Latitude:  40.6083984375,
		Longitude: -80.9330978394,
		Timezone:  "America/New_York",
	},
	"OI68": {
		ICAO:      "OI68",
		Name:      "Hilty Field",
		City:      "Rittman",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1000,
		Latitude:  40.9413986206,
		Longitude: -81.7977981567,
		Timezone:  "America/New_York",
	},
	"OI69": {
		ICAO:      "OI69",
		Name:      "Blackacre Farm Airport",
		City:      "Painesville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 640,
		Latitude:  41.7784004211,
		Longitude: -81.1669998169,
		Timezone:  "America/New_York",
	},
	"OI72": {
		ICAO:      "OI72",
		Name:      "Marvin Thiel Field",
		City:      "Blakeslee",
		State:     "Ohio",
		Country:   "US",
		Elevation: 859,
		Latitude:  41.5222015381,
		Longitude: -84.7208023071,
		Timezone:  "America/New_York",
	},
	"OI73": {
		ICAO:      "OI73",
		Name:      "Williamson Airport",
		City:      "Cedarville",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1060,
		Latitude:  39.712600708,
		Longitude: -83.8038024902,
		Timezone:  "America/New_York",
	},
	"OI77": {
		ICAO:      "OI77",
		Name:      "Mc Ardle Airport",
		City:      "Fostoria",
		State:     "Ohio",
		Country:   "US",
		Elevation: 715,
		Latitude:  41.2159004211,
		Longitude: -83.5579986572,
		Timezone:  "America/New_York",
	},
	"OI78": {
		ICAO:      "OI78",
		Name:      "Diebleys Airport",
		City:      "Fostoria",
		State:     "Ohio",
		Country:   "US",
		Elevation: 823,
		Latitude:  41.0848007202,
		Longitude: -83.4363021851,
		Timezone:  "America/New_York",
	},
	"OI81": {
		ICAO:      "OI81",
		Name:      "Mollica Airport",
		City:      "Austintown",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1170,
		Latitude:  41.0713996887,
		Longitude: -80.7585983276,
		Timezone:  "America/New_York",
	},
	"OI82": {
		ICAO:      "OI82",
		Name:      "Russ Airport",
		City:      "Jackson",
		State:     "Ohio",
		Country:   "US",
		Elevation: 800,
		Latitude:  38.9122009277,
		Longitude: -82.724899292,
		Timezone:  "America/New_York",
	},
	"OI84": {
		ICAO:      "OI84",
		Name:      "King Knoll Airport",
		City:      "Okeana",
		State:     "Ohio",
		Country:   "US",
		Elevation: 855,
		Latitude:  39.3871002197,
		Longitude: -84.7978973389,
		Timezone:  "America/New_York",
	},
	"OI85": {
		ICAO:      "OI85",
		Name:      "Mill Creek Airport",
		City:      "Ostrander",
		State:     "Ohio",
		Country:   "US",
		Elevation: 960,
		Latitude:  40.2683982849,
		Longitude: -83.2216033936,
		Timezone:  "America/New_York",
	},
	"OI88": {
		ICAO:      "OI88",
		Name:      "Mindzak Airfield",
		City:      "Greenwich",
		State:     "Ohio",
		Country:   "US",
		Elevation: 1109,
		Latitude:  41.0111999512,
		Longitude: -82.4359970093,
		Timezone:  "America/New_York",
	},
	"OI89": {
		ICAO:      "OI89",
		Name:      "Mite Airport",
		City:      "Camden",
		State:     "Ohio",
		Country:   "US",
		Elevation: 866,
		Latitude:  39.6495018005,
		Longitude: -84.6299972534,
		Timezone:  "America/New_York",
	},
	"OI91": {
		ICAO:      "OI91",
		Name:      "Hammond Airport",
		City:      "Yellow Springs",
		State:     "Ohio",
		Country:   "US",
		Elevation: 990,
		Latitude:  39.7797012329,
		Longitude: -83.9092025757,
		Timezone:  "America/New_York",
	},
	"OI92": {
		ICAO:      "OI92",
		Name:      "Conklin Airport",
		City:      "Bowling Green",
		State:     "Ohio",
		Country:   "US",
		Elevation: 675,
		Latitude:  41.3917007446,
		Longitude: -83.7033004761,
		Timezone:  "America/New_York",
	},
	"OI99": {
		ICAO:      "OI99",
		Name:      "Mulholland Airport",
		City:      "Celina",
		State:     "Ohio",
		Country:   "US",
		Elevation: 875,
		Latitude:  40.5733985901,
		Longitude: -84.4899978638,
		Timezone:  "America/New_York",
	},
	"OIAA": {
		ICAO:      "OIAA",
		IATA:      "ABD",
		Name:      "Abadan Airport",
		City:      "Abadan",
		State:     "Khuzestan",
		Country:   "IR",
		Elevation: 10,
		Latitude:  30.371099472,
		Longitude: 48.2282981873,
		Timezone:  "Asia/Tehran",
	},
	"OIAD": {
		ICAO:      "OIAD",
		IATA:      "DEF",
		Name:      "Dezful Airport",
		State:     "Khuzestan",
		Country:   "IR",
		Elevation: 474,
		Latitude:  32.4343986511,
		Longitude: 48.3975982666,
		Timezone:  "Asia/Tehran",
	},
	"OIAG": {
		ICAO:      "OIAG",
		IATA:      "AKW",
		Name:      "Aghajari Airport",
		State:     "Khuzestan",
		Country:   "IR",
		Elevation: 88,
		Latitude:  30.7444000244,
		Longitude: 49.6772003174,
		Timezone:  "Asia/Tehran",
	},
	"OIAH": {
		ICAO:      "OIAH",
		IATA:      "GCH",
		Name:      "Gachsaran Airport",
		State:     "Kohgiluyeh-va-Buyer-Aḩmad",
		Country:   "IR",
		Elevation: 2414,
		Latitude:  30.337600708,
		Longitude: 50.827999115,
		Timezone:  "Asia/Tehran",
	},
	"OIAI": {
		ICAO:      "OIAI",
		Name:      "Shahid Asyaee Airport",
		State:     "Khuzestan",
		Country:   "IR",
		Elevation: 1206,
		Latitude:  32.0023994446,
		Longitude: 49.270401001,
		Timezone:  "Asia/Tehran",
	},
	"OIAJ": {
		ICAO:      "OIAJ",
		IATA:      "OMI",
		Name:      "Omidiyeh Airport",
		City:      "Omidiyeh",
		State:     "Khuzestan",
		Country:   "IR",
		Elevation: 85,
		Latitude:  30.8351993561,
		Longitude: 49.5349006653,
		Timezone:  "Asia/Tehran",
	},
	"OIAM": {
		ICAO:      "OIAM",
		IATA:      "MRX",
		Name:      "Mahshahr Airport",
		State:     "Khuzestan",
		Country:   "IR",
		Elevation: 8,
		Latitude:  30.5562000275,
		Longitude: 49.1519012451,
		Timezone:  "Asia/Tehran",
	},
	"OIAW": {
		ICAO:      "OIAW",
		IATA:      "AWZ",
		Name:      "Ahwaz Airport",
		City:      "Ahwaz",
		State:     "Khuzestan",
		Country:   "IR",
		Elevation: 66,
		Latitude:  31.3374004364,
		Longitude: 48.7620010376,
		Timezone:  "Asia/Tehran",
	},
	"OIBA": {
		ICAO:      "OIBA",
		IATA:      "AEU",
		Name:      "Abumusa Island Airport",
		State:     "Hormozgan",
		Country:   "IR",
		Elevation: 23,
		Latitude:  25.8756999969,
		Longitude: 55.033000946,
		Timezone:  "Asia/Tehran",
	},
	"OIBB": {
		ICAO:      "OIBB",
		IATA:      "BUZ",
		Name:      "Bushehr Airport",
		City:      "Bushehr",
		State:     "Bushehr",
		Country:   "IR",
		Elevation: 68,
		Latitude:  28.9447994232,
		Longitude: 50.8345985413,
		Timezone:  "Asia/Tehran",
	},
	"OIBH": {
		ICAO:      "OIBH",
		IATA:      "IAQ",
		Name:      "Bastak Airport",
		State:     "Hormozgan",
		Country:   "IR",
		Elevation: 1350,
		Latitude:  27.2126998901,
		Longitude: 54.3185997009,
		Timezone:  "Asia/Tehran",
	},
	"OIBI": {
		ICAO:      "OIBI",
		IATA:      "AOY",
		Name:      "Asaloyeh Airport",
		City:      "Asaloyeh",
		Country:   "IR",
		Elevation: 15,
		Latitude:  27.4813995361,
		Longitude: 52.6155014038,
		Timezone:  "Asia/Tehran",
	},
	"OIBJ": {
		ICAO:      "OIBJ",
		IATA:      "KNR",
		Name:      "Jam Airport",
		City:      "Kangan",
		State:     "Bushehr",
		Country:   "IR",
		Elevation: 2173,
		Latitude:  27.8204994202,
		Longitude: 52.3521995544,
		Timezone:  "Asia/Tehran",
	},
	"OIBK": {
		ICAO:      "OIBK",
		IATA:      "KIH",
		Name:      "Kish International Airport",
		City:      "Kish Island",
		State:     "Hormozgan",
		Country:   "IR",
		Elevation: 101,
		Latitude:  26.5261993408,
		Longitude: 53.9802017212,
		Timezone:  "Asia/Tehran",
	},
	"OIBL": {
		ICAO:      "OIBL",
		IATA:      "BDH",
		Name:      "Bandar Lengeh Airport",
		City:      "Bandar Lengeh",
		State:     "Hormozgan",
		Country:   "IR",
		Elevation: 67,
		Latitude:  26.531999588,
		Longitude: 54.824798584,
		Timezone:  "Asia/Tehran",
	},
	"OIBN": {
		ICAO:      "OIBN",
		Name:      "Borazjan Airport",
		City:      "Borazjan",
		State:     "Bushehr",
		Country:   "IR",
		Elevation: 265,
		Latitude:  29.2666664124,
		Longitude: 51.2000007629,
		Timezone:  "Asia/Tehran",
	},
	"OIBP": {
		ICAO:      "OIBP",
		IATA:      "YEH",
		Name:      "Persian Gulf International Airport",
		City:      "Asalouyeh",
		State:     "Bushehr",
		Country:   "IR",
		Elevation: 27,
		Latitude:  27.3796005249,
		Longitude: 52.737701416,
		Timezone:  "Asia/Tehran",
	},
	"OIBQ": {
		ICAO:      "OIBQ",
		IATA:      "KHK",
		Name:      "Khark Island Airport",
		State:     "Bushehr",
		Country:   "IR",
		Elevation: 17,
		Latitude:  29.2602996826,
		Longitude: 50.3238983154,
		Timezone:  "Asia/Tehran",
	},
	"OIBS": {
		ICAO:      "OIBS",
		IATA:      "SXI",
		Name:      "Sirri Island Airport",
		State:     "Hormozgan",
		Country:   "IR",
		Elevation: 43,
		Latitude:  25.9088993073,
		Longitude: 54.5393981934,
		Timezone:  "Asia/Tehran",
	},
	"OIBV": {
		ICAO:      "OIBV",
		IATA:      "LVP",
		Name:      "Lavan Island Airport",
		State:     "Hormozgan",
		Country:   "IR",
		Elevation: 76,
		Latitude:  26.810300827,
		Longitude: 53.356300354,
		Timezone:  "Asia/Tehran",
	},
	"OICB": {
		ICAO:      "OICB",
		Name:      "Baneh Airport",
		City:      "Baneh",
		State:     "Kordestan",
		Country:   "IR",
		Elevation: 4911,
		Latitude:  35.987701416,
		Longitude: 45.8470993042,
		Timezone:  "Asia/Tehran",
	},
	"OICC": {
		ICAO:      "OICC",
		IATA:      "KSH",
		Name:      "Shahid Ashrafi Esfahani Airport",
		City:      "Kermanshah",
		State:     "Kermanshah",
		Country:   "IR",
		Elevation: 4307,
		Latitude:  34.3459014893,
		Longitude: 47.1581001282,
		Timezone:  "Asia/Tehran",
	},
	"OICD": {
		ICAO:      "OICD",
		Name:      "Abdanan Airport",
		City:      "Abdanan",
		State:     "Ilam",
		Country:   "IR",
		Elevation: 2600,
		Latitude:  32.9345016479,
		Longitude: 47.4833984375,
		Timezone:  "Asia/Tehran",
	},
	"OICG": {
		ICAO:      "OICG",
		Name:      "Ghasre-Shirin Airport",
		City:      "Ghasre-Shirin",
		State:     "Kermanshah",
		Country:   "IR",
		Latitude:  34.5166664124,
		Longitude: 45.5666656494,
		Timezone:  "Asia/Tehran",
	},
	"OICI": {
		ICAO:      "OICI",
		IATA:      "IIL",
		Name:      "Ilam Airport",
		City:      "Ilam",
		State:     "Ilam",
		Country:   "IR",
		Elevation: 4404,
		Latitude:  33.5866012573,
		Longitude: 46.404800415,
		Timezone:  "Asia/Tehran",
	},
	"OICK": {
		ICAO:      "OICK",
		IATA:      "KHD",
		Name:      "Khoram Abad Airport",
		State:     "Lorestan",
		Country:   "IR",
		Elevation: 3782,
		Latitude:  33.4353981018,
		Longitude: 48.2829017639,
		Timezone:  "Asia/Tehran",
	},
	"OICS": {
		ICAO:      "OICS",
		IATA:      "SDG",
		Name:      "Sanandaj Airport",
		State:     "Kordestan",
		Country:   "IR",
		Elevation: 4522,
		Latitude:  35.2458992004,
		Longitude: 47.0092010498,
		Timezone:  "Asia/Tehran",
	},
	"OICZ": {
		ICAO:      "OICZ",
		Name:      "Aligoodarz Airport",
		State:     "Lorestan",
		Country:   "IR",
		Elevation: 6505,
		Latitude:  33.3835983276,
		Longitude: 49.6963996887,
		Timezone:  "Asia/Tehran",
	},
	"OIFE": {
		ICAO:      "OIFE",
		IATA:      "IFH",
		Name:      "Hesa Airport",
		City:      "Hesa",
		State:     "Isfahan",
		Country:   "IR",
		Elevation: 5256,
		Latitude:  32.9289016724,
		Longitude: 51.5611000061,
		Timezone:  "Asia/Tehran",
	},
	"OIFH": {
		ICAO:      "OIFH",
		Name:      "Shahid Vatan Pour Air Base",
		State:     "Isfahan",
		Country:   "IR",
		Elevation: 5310,
		Latitude:  32.5670013428,
		Longitude: 51.6916007996,
		Timezone:  "Asia/Tehran",
	},
	"OIFK": {
		ICAO:      "OIFK",
		IATA:      "KKS",
		Name:      "Kashan Airport",
		State:     "Isfahan",
		Country:   "IR",
		Elevation: 3465,
		Latitude:  33.8953018188,
		Longitude: 51.5769996643,
		Timezone:  "Asia/Tehran",
	},
	"OIFM": {
		ICAO:      "OIFM",
		IATA:      "IFN",
		Name:      "Esfahan Shahid Beheshti International Airport",
		City:      "Isfahan",
		State:     "Isfahan",
		Country:   "IR",
		Elevation: 5059,
		Latitude:  32.7508010864,
		Longitude: 51.8613014221,
		Timezone:  "Asia/Tehran",
	},
	"OIFP": {
		ICAO:      "OIFP",
		Name:      "Badr Air Base",
		City:      "Esfahan",
		State:     "Isfahan",
		Country:   "IR",
		Elevation: 5242,
		Latitude:  32.6211013794,
		Longitude: 51.6969985962,
		Timezone:  "Asia/Tehran",
	},
	"OIFS": {
		ICAO:      "OIFS",
		IATA:      "CQD",
		Name:      "Shahrekord Airport",
		City:      "Shahrekord",
		State:     "Chaharmahal-and-Bakhtiari",
		Country:   "IR",
		Elevation: 6723,
		Latitude:  32.2971992493,
		Longitude: 50.8422012329,
		Timezone:  "Asia/Tehran",
	},
	"OIFV": {
		ICAO:      "OIFV",
		Name:      "Zarrinshahr Airport",
		City:      "Zarrinshahr",
		State:     "Isfahan",
		Country:   "IR",
		Elevation: 5650,
		Latitude:  32.3274002075,
		Longitude: 51.3773002625,
		Timezone:  "Asia/Tehran",
	},
	"OIGG": {
		ICAO:      "OIGG",
		IATA:      "RAS",
		Name:      "Sardar-e-Jangal Airport",
		City:      "Rasht",
		State:     "Gilan",
		Country:   "IR",
		Elevation: -40,
		Latitude:  37.323333,
		Longitude: 49.617778,
		Timezone:  "Asia/Tehran",
	},
	"OIHH": {
		ICAO:      "OIHH",
		IATA:      "HDM",
		Name:      "Hamadan Airport",
		City:      "Hamadan",
		State:     "Hamadan",
		Country:   "IR",
		Elevation: 5755,
		Latitude:  34.8692016602,
		Longitude: 48.5525016785,
		Timezone:  "Asia/Tehran",
	},
	"OIHM": {
		ICAO:      "OIHM",
		Name:      "Malayer Airport",
		City:      "Malayer",
		State:     "Hamadan",
		Country:   "IR",
		Elevation: 5662,
		Latitude:  34.2833328247,
		Longitude: 48.8166656494,
		Timezone:  "Asia/Tehran",
	},
	"OIHR": {
		ICAO:      "OIHR",
		IATA:      "AJK",
		Name:      "Arak Airport",
		City:      "Araak",
		State:     "Markazi",
		Country:   "IR",
		Elevation: 5440,
		Latitude:  34.1380996704,
		Longitude: 49.8473014832,
		Timezone:  "Asia/Tehran",
	},
	"OIHS": {
		ICAO:      "OIHS",
		IATA:      "NUJ",
		Name:      "Hamadan Air Base",
		City:      "Hamadan",
		State:     "Hamadan",
		Country:   "IR",
		Elevation: 5609,
		Latitude:  35.2116012573,
		Longitude: 48.6534004211,
		Timezone:  "Asia/Tehran",
	},
	"OIIA": {
		ICAO:      "OIIA",
		Name:      "Ghazvin Azadi Airport",
		State:     "Alborz",
		Country:   "IR",
		Elevation: 3769,
		Latitude:  35.9520988464,
		Longitude: 50.4508018494,
		Timezone:  "Asia/Tehran",
	},
	"OIIC": {
		ICAO:      "OIIC",
		Name:      "Kushke Nosrat Airport",
		State:     "Qom",
		Country:   "IR",
		Elevation: 3008,
		Latitude:  34.9840011597,
		Longitude: 50.8061981201,
		Timezone:  "Asia/Tehran",
	},
	"OIID": {
		ICAO:      "OIID",
		Name:      "Doshan Tappeh Air Base",
		State:     "Tehran",
		Country:   "IR",
		Elevation: 4046,
		Latitude:  35.702999115,
		Longitude: 51.4751014709,
		Timezone:  "Asia/Tehran",
	},
	"OIIE": {
		ICAO:      "OIIE",
		IATA:      "IKA",
		Name:      "Imam Khomeini International Airport",
		City:      "Tehran",
		State:     "Tehran",
		Country:   "IR",
		Elevation: 3305,
		Latitude:  35.4160995483,
		Longitude: 51.1521987915,
		Timezone:  "Asia/Tehran",
	},
	"OIIG": {
		ICAO:      "OIIG",
		Name:      "Ghale Morghi Airport",
		State:     "Tehran",
		Country:   "IR",
		Elevation: 3627,
		Latitude:  35.6447982788,
		Longitude: 51.3806991577,
		Timezone:  "Asia/Tehran",
	},
	"OIII": {
		ICAO:      "OIII",
		IATA:      "THR",
		Name:      "Mehrabad International Airport",
		City:      "Tehran",
		State:     "Tehran",
		Country:   "IR",
		Elevation: 3962,
		Latitude:  35.689201355,
		Longitude: 51.3134002686,
		Timezone:  "Asia/Tehran",
	},
	"OIIK": {
		ICAO:      "OIIK",
		IATA:      "GZW",
		Name:      "Qazvin Airport",
		City:      "Qazvin",
		State:     "Qazvin",
		Country:   "IR",
		Elevation: 4184,
		Latitude:  36.2401008606,
		Longitude: 50.0471000671,
		Timezone:  "Asia/Tehran",
	},
	"OIIM": {
		ICAO:      "OIIM",
		Name:      "Naja Airport",
		State:     "Alborz",
		Country:   "IR",
		Elevation: 4040,
		Latitude:  35.7762985229,
		Longitude: 50.8810005188,
		Timezone:  "Asia/Tehran",
	},
	"OIIP": {
		ICAO:      "OIIP",
		IATA:      "PYK",
		Name:      "Payam Airport",
		State:     "Alborz",
		Country:   "IR",
		Elevation: 4170,
		Latitude:  35.7761001587,
		Longitude: 50.8266983032,
		Timezone:  "Asia/Tehran",
	},
	"OIIQ": {
		ICAO:      "OIIQ",
		Name:      "Ghom Airport",
		City:      "Ghom",
		State:     "Qom",
		Country:   "IR",
		Latitude:  34.6333312988,
		Longitude: 50.8833312988,
		Timezone:  "Asia/Tehran",
	},
	"OIIS": {
		ICAO:      "OIIS",
		IATA:      "SNX",
		Name:      "Semnan Airport",
		City:      "Semnan",
		State:     "Semnan",
		Country:   "IR",
		Elevation: 3665,
		Latitude:  35.5910987854,
		Longitude: 53.495098114,
		Timezone:  "Asia/Tehran",
	},
	"OIIU": {
		ICAO:      "OIIU",
		Name:      "Damghan Airport",
		State:     "Semnan",
		Country:   "IR",
		Elevation: 3887,
		Latitude:  36.1666984558,
		Longitude: 54.3438987732,
		Timezone:  "Asia/Tehran",
	},
	"OIKB": {
		ICAO:      "OIKB",
		IATA:      "BND",
		Name:      "Bandar Abbas International Airport",
		City:      "Bandar Abbas",
		State:     "Hormozgan",
		Country:   "IR",
		Elevation: 22,
		Latitude:  27.2182998657,
		Longitude: 56.3777999878,
		Timezone:  "Asia/Tehran",
	},
	"OIKF": {
		ICAO:      "OIKF",
		Name:      "Baft Airport",
		State:     "Kerman",
		Country:   "IR",
		Elevation: 7385,
		Latitude:  29.2341995239,
		Longitude: 56.6068992615,
		Timezone:  "Asia/Tehran",
	},
	"OIKJ": {
		ICAO:      "OIKJ",
		IATA:      "JYR",
		Name:      "Jiroft Airport",
		State:     "Kerman",
		Country:   "IR",
		Elevation: 2663,
		Latitude:  28.7269001007,
		Longitude: 57.67029953,
		Timezone:  "Asia/Tehran",
	},
	"OIKK": {
		ICAO:      "OIKK",
		IATA:      "KER",
		Name:      "Kerman Airport",
		City:      "Kerman",
		State:     "Kerman",
		Country:   "IR",
		Elevation: 5741,
		Latitude:  30.2744007111,
		Longitude: 56.9510993958,
		Timezone:  "Asia/Tehran",
	},
	"OIKM": {
		ICAO:      "OIKM",
		IATA:      "BXR",
		Name:      "Bam Airport",
		State:     "Kerman",
		Country:   "IR",
		Elevation: 3231,
		Latitude:  29.0841999054,
		Longitude: 58.4500007629,
		Timezone:  "Asia/Tehran",
	},
	"OIKP": {
		ICAO:      "OIKP",
		IATA:      "HDR",
		Name:      "Havadarya Airport",
		City:      "Havadarya",
		State:     "Hormozgan",
		Country:   "IR",
		Elevation: 19,
		Latitude:  27.1583003998,
		Longitude: 56.1725006104,
		Timezone:  "Asia/Tehran",
	},
	"OIKQ": {
		ICAO:      "OIKQ",
		IATA:      "GSM",
		Name:      "Dayrestan Airport",
		State:     "Hormozgan",
		Country:   "IR",
		Elevation: 45,
		Latitude:  26.7546005249,
		Longitude: 55.9024009705,
		Timezone:  "Asia/Tehran",
	},
	"OIKR": {
		ICAO:      "OIKR",
		IATA:      "RJN",
		Name:      "Rafsanjan Airport",
		State:     "Kerman",
		Country:   "IR",
		Elevation: 5298,
		Latitude:  30.297700882,
		Longitude: 56.0511016846,
		Timezone:  "Asia/Tehran",
	},
	"OIKY": {
		ICAO:      "OIKY",
		IATA:      "SYJ",
		Name:      "Sirjan Airport",
		State:     "Kerman",
		Country:   "IR",
		Elevation: 5846,
		Latitude:  29.5508995056,
		Longitude: 55.6726989746,
		Timezone:  "Asia/Tehran",
	},
	"OIMB": {
		ICAO:      "OIMB",
		IATA:      "XBJ",
		Name:      "Birjand Airport",
		City:      "Birjand",
		State:     "Khorasan-e-Jonubi",
		Country:   "IR",
		Elevation: 4952,
		Latitude:  32.8981018066,
		Longitude: 59.2661018372,
		Timezone:  "Asia/Tehran",
	},
	"OIMC": {
		ICAO:      "OIMC",
		IATA:      "CKT",
		Name:      "Sarakhs Airport",
		City:      "Sarakhs",
		State:     "Razavi-Khorasan",
		Country:   "IR",
		Elevation: 945,
		Latitude:  36.5012016296,
		Longitude: 61.0648994446,
		Timezone:  "Asia/Tehran",
	},
	"OIMD": {
		ICAO:      "OIMD",
		Name:      "Gonabad Airport",
		City:      "Gonabad",
		State:     "Razavi-Khorasan",
		Country:   "IR",
		Latitude:  34.3499984741,
		Longitude: 58.6800003052,
		Timezone:  "Asia/Tehran",
	},
	"OIMH": {
		ICAO:      "OIMH",
		Name:      "Torbat-e Heydarieh Airport",
		City:      "Torbat-E-Heidarieh",
		State:     "Razavi-Khorasan",
		Country:   "IR",
		Elevation: 4363,
		Latitude:  35.2667007446,
		Longitude: 59.2167015076,
		Timezone:  "Asia/Tehran",
	},
	"OIMJ": {
		ICAO:      "OIMJ",
		IATA:      "RUD",
		Name:      "Shahroud Airport",
		State:     "Semnan",
		Country:   "IR",
		Elevation: 4215,
		Latitude:  36.4253005981,
		Longitude: 55.1041984558,
		Timezone:  "Asia/Tehran",
	},
	"OIMM": {
		ICAO:      "OIMM",
		IATA:      "MHD",
		Name:      "Mashhad International Airport",
		City:      "Mashhad",
		State:     "Razavi-Khorasan",
		Country:   "IR",
		Elevation: 3263,
		Latitude:  36.2351989746,
		Longitude: 59.6409988403,
		Timezone:  "Asia/Tehran",
	},
	"OIMN": {
		ICAO:      "OIMN",
		IATA:      "BJB",
		Name:      "Bojnord Airport",
		City:      "Bojnord",
		State:     "Khorasan-e-Shomali",
		Country:   "IR",
		Elevation: 3499,
		Latitude:  37.4930000305,
		Longitude: 57.3082008362,
		Timezone:  "Asia/Tehran",
	},
	"OIMS": {
		ICAO:      "OIMS",
		IATA:      "AFZ",
		Name:      "Sabzevar National Airport",
		City:      "Sabzevar",
		State:     "Razavi-Khorasan",
		Country:   "IR",
		Elevation: 3010,
		Latitude:  36.1680984497,
		Longitude: 57.595199585,
		Timezone:  "Asia/Tehran",
	},
	"OIMT": {
		ICAO:      "OIMT",
		IATA:      "TCX",
		Name:      "Tabas Airport",
		City:      "Tabas",
		State:     "Yazd",
		Country:   "IR",
		Elevation: 2312,
		Latitude:  33.6678009033,
		Longitude: 56.8927001953,
		Timezone:  "Asia/Tehran",
	},
	"OIMX": {
		ICAO:      "OIMX",
		Name:      "Soga Airport",
		City:      "Soga",
		State:     "Khorasan-e-Shomali",
		Country:   "IR",
		Elevation: 2460,
		Latitude:  37.6278991699,
		Longitude: 56.1730995178,
		Timezone:  "Asia/Tehran",
	},
	"OINA": {
		ICAO:      "OINA",
		Name:      "Amol Airport",
		State:     "Mazandaran",
		Country:   "IR",
		Elevation: 318,
		Latitude:  36.4707984924,
		Longitude: 52.3633003235,
		Timezone:  "Asia/Tehran",
	},
	"OINB": {
		ICAO:      "OINB",
		IATA:      "BBL",
		Name:      "Babolsar Airport",
		City:      "Babolsar",
		State:     "Mazandaran",
		Country:   "IR",
		Elevation: -49,
		Latitude:  36.7200012207,
		Longitude: 52.6500015259,
		Timezone:  "Asia/Tehran",
	},
	"OINE": {
		ICAO:      "OINE",
		IATA:      "KLM",
		Name:      "Kalaleh Airport",
		State:     "Mazandaran",
		Country:   "IR",
		Elevation: 425,
		Latitude:  37.3833007813,
		Longitude: 55.4519996643,
		Timezone:  "Asia/Tehran",
	},
	"OING": {
		ICAO:      "OING",
		IATA:      "GBT",
		Name:      "Gorgan Airport",
		City:      "Gorgan",
		State:     "Mazandaran",
		Country:   "IR",
		Elevation: -24,
		Latitude:  36.9094009399,
		Longitude: 54.4012985229,
		Timezone:  "Asia/Tehran",
	},
	"OINH": {
		ICAO:      "OINH",
		Name:      "Behshahr Airport",
		State:     "Mazandaran",
		Country:   "IR",
		Elevation: 137,
		Latitude:  36.6944007874,
		Longitude: 53.5424995422,
		Timezone:  "Asia/Tehran",
	},
	"OINI": {
		ICAO:      "OINI",
		Name:      "Ghaem Shahr Airport",
		City:      "Ghaem Shahr",
		State:     "Mazandaran",
		Country:   "IR",
		Latitude:  36.4666671753,
		Longitude: 52.8666687012,
		Timezone:  "Asia/Tehran",
	},
	"OINJ": {
		ICAO:      "OINJ",
		IATA:      "BSM",
		Name:      "Bishe Kola Air Base",
		City:      "Amol",
		State:     "Mazandaran",
		Country:   "IR",
		Elevation: -79,
		Latitude:  36.6551017761,
		Longitude: 52.3496017456,
		Timezone:  "Asia/Tehran",
	},
	"OINM": {
		ICAO:      "OINM",
		Name:      "Mahmood Abad Airport",
		City:      "Mahmood Abad",
		State:     "Isfahan",
		Country:   "IR",
		Latitude:  34.1691665649,
		Longitude: 51.3175010681,
		Timezone:  "Asia/Tehran",
	},
	"OINN": {
		ICAO:      "OINN",
		IATA:      "NSH",
		Name:      "Noshahr Airport",
		State:     "Mazandaran",
		Country:   "IR",
		Elevation: -61,
		Latitude:  36.6632995605,
		Longitude: 51.4646987915,
		Timezone:  "Asia/Tehran",
	},
	"OINR": {
		ICAO:      "OINR",
		IATA:      "RZR",
		Name:      "Ramsar Airport",
		State:     "Mazandaran",
		Country:   "IR",
		Elevation: -70,
		Latitude:  36.9099006653,
		Longitude: 50.679599762,
		Timezone:  "Asia/Tehran",
	},
	"OINZ": {
		ICAO:      "OINZ",
		IATA:      "SRY",
		Name:      "Dasht-e Naz Airport",
		City:      "Sari",
		State:     "Mazandaran",
		Country:   "IR",
		Elevation: 35,
		Latitude:  36.635799408,
		Longitude: 53.1935997009,
		Timezone:  "Asia/Tehran",
	},
	"OISA": {
		ICAO:      "OISA",
		Name:      "Abadeh Airport",
		City:      "Abadeh",
		State:     "Fars",
		Country:   "IR",
		Elevation: 5320,
		Latitude:  31.1666679382,
		Longitude: 52.6666679382,
		Timezone:  "Asia/Tehran",
	},
	"OISD": {
		ICAO:      "OISD",
		Name:      "Darab Airport",
		City:      "Darab",
		State:     "Fars",
		Country:   "IR",
		Elevation: 3600,
		Latitude:  28.7215995789,
		Longitude: 54.4412994385,
		Timezone:  "Asia/Tehran",
	},
	"OISF": {
		ICAO:      "OISF",
		IATA:      "FAZ",
		Name:      "Fasa Airport",
		City:      "Fasa",
		State:     "Fars",
		Country:   "IR",
		Elevation: 4261,
		Latitude:  28.8917999268,
		Longitude: 53.7233009338,
		Timezone:  "Asia/Tehran",
	},
	"OISJ": {
		ICAO:      "OISJ",
		IATA:      "JAR",
		Name:      "Jahrom Airport",
		State:     "Fars",
		Country:   "IR",
		Elevation: 3358,
		Latitude:  28.5867004395,
		Longitude: 53.5791015625,
		Timezone:  "Asia/Tehran",
	},
	"OISL": {
		ICAO:      "OISL",
		IATA:      "LRR",
		Name:      "Lar Airport",
		City:      "Lar",
		State:     "Fars",
		Country:   "IR",
		Elevation: 2641,
		Latitude:  27.6746997833,
		Longitude: 54.3833007812,
		Timezone:  "Asia/Tehran",
	},
	"OISO": {
		ICAO:      "OISO",
		Name:      "Zargan Airport",
		City:      "Zargan",
		State:     "Fars",
		Country:   "IR",
		Elevation: 5500,
		Latitude:  29.7541999817,
		Longitude: 52.6943016052,
		Timezone:  "Asia/Tehran",
	},
	"OISR": {
		ICAO:      "OISR",
		IATA:      "LFM",
		Name:      "Lamerd Airport",
		City:      "Lamerd",
		State:     "Fars",
		Country:   "IR",
		Elevation: 1337,
		Latitude:  27.3726997375,
		Longitude: 53.1888008118,
		Timezone:  "Asia/Tehran",
	},
	"OISS": {
		ICAO:      "OISS",
		IATA:      "SYZ",
		Name:      "Shiraz Shahid Dastghaib International Airport",
		City:      "Shiraz",
		State:     "Fars",
		Country:   "IR",
		Elevation: 4920,
		Latitude:  29.5391998291,
		Longitude: 52.5898017883,
		Timezone:  "Asia/Tehran",
	},
	"OISY": {
		ICAO:      "OISY",
		IATA:      "YES",
		Name:      "Yasouj Airport",
		State:     "Kohgiluyeh-va-Buyer-Aḩmad",
		Country:   "IR",
		Elevation: 5939,
		Latitude:  30.7005004883,
		Longitude: 51.5451011658,
		Timezone:  "Asia/Tehran",
	},
	"OITK": {
		ICAO:      "OITK",
		IATA:      "KHY",
		Name:      "Khoy Airport",
		City:      "Khoy",
		State:     "Az̄arbayjan-e-Gharbi",
		Country:   "IR",
		Elevation: 3981,
		Latitude:  38.4275016785,
		Longitude: 44.9735984802,
		Timezone:  "Asia/Tehran",
	},
	"OITL": {
		ICAO:      "OITL",
		IATA:      "ADU",
		Name:      "Ardabil Airport",
		City:      "Ardabil",
		State:     "Ardabil",
		Country:   "IR",
		Elevation: 4315,
		Latitude:  38.3256988525,
		Longitude: 48.4244003296,
		Timezone:  "Asia/Tehran",
	},
	"OITM": {
		ICAO:      "OITM",
		IATA:      "ACP",
		Name:      "Sahand Airport",
		City:      "Maragheh",
		State:     "East-Azerbaijan",
		Country:   "IR",
		Elevation: 4396,
		Latitude:  37.3479995728,
		Longitude: 46.1278991699,
		Timezone:  "Asia/Tehran",
	},
	"OITP": {
		ICAO:      "OITP",
		IATA:      "PFQ",
		Name:      "Parsabade Moghan Airport",
		City:      "Parsabad",
		State:     "Ardabil",
		Country:   "IR",
		Elevation: 251,
		Latitude:  39.6035995483,
		Longitude: 47.8815002441,
		Timezone:  "Asia/Tehran",
	},
	"OITR": {
		ICAO:      "OITR",
		IATA:      "OMH",
		Name:      "Urmia Airport",
		City:      "Urmia",
		State:     "Az̄arbayjan-e-Gharbi",
		Country:   "IR",
		Elevation: 4343,
		Latitude:  37.6680984497,
		Longitude: 45.0686988831,
		Timezone:  "Asia/Tehran",
	},
	"OITS": {
		ICAO:      "OITS",
		Name:      "Saghez Airport",
		City:      "Saghez",
		State:     "Kordestan",
		Country:   "IR",
		Elevation: 4880,
		Latitude:  36.25,
		Longitude: 46.2666664124,
		Timezone:  "Asia/Tehran",
	},
	"OITT": {
		ICAO:      "OITT",
		IATA:      "TBZ",
		Name:      "Tabriz International Airport",
		City:      "Tabriz",
		State:     "East-Azerbaijan",
		Country:   "IR",
		Elevation: 4459,
		Latitude:  38.1338996887,
		Longitude: 46.2350006104,
		Timezone:  "Asia/Tehran",
	},
	"OITU": {
		ICAO:      "OITU",
		IATA:      "IMQ",
		Name:      "Makou Airport",
		City:      "Makou",
		State:     "Az̄arbayjan-e-Gharbi",
		Country:   "IR",
		Latitude:  39.3300018311,
		Longitude: 44.4300003052,
		Timezone:  "Asia/Tehran",
	},
	"OITZ": {
		ICAO:      "OITZ",
		IATA:      "JWN",
		Name:      "Zanjan Airport",
		State:     "Zanjan",
		Country:   "IR",
		Elevation: 5382,
		Latitude:  36.7737007141,
		Longitude: 48.3594017029,
		Timezone:  "Asia/Tehran",
	},
	"OIYB": {
		ICAO:      "OIYB",
		Name:      "Bafgh Airport",
		City:      "Bafgh",
		State:     "Yazd",
		Country:   "IR",
		Latitude:  31.5833320618,
		Longitude: 55.5666656494,
		Timezone:  "Asia/Tehran",
	},
	"OIYY": {
		ICAO:      "OIYY",
		IATA:      "AZD",
		Name:      "Shahid Sadooghi Airport",
		City:      "Yazd",
		State:     "Yazd",
		Country:   "IR",
		Elevation: 4054,
		Latitude:  31.9048995972,
		Longitude: 54.2765007019,
		Timezone:  "Asia/Tehran",
	},
	"OIZB": {
		ICAO:      "OIZB",
		IATA:      "ACZ",
		Name:      "Zabol Airport",
		State:     "Sistan-and-Baluchestan",
		Country:   "IR",
		Elevation: 1628,
		Latitude:  31.0983009338,
		Longitude: 61.5438995361,
		Timezone:  "Asia/Tehran",
	},
	"OIZC": {
		ICAO:      "OIZC",
		IATA:      "ZBR",
		Name:      "Konarak Airport",
		City:      "Chabahar",
		State:     "Sistan-and-Baluchestan",
		Country:   "IR",
		Elevation: 43,
		Latitude:  25.4433002472,
		Longitude: 60.3820991516,
		Timezone:  "Asia/Tehran",
	},
	"OIZH": {
		ICAO:      "OIZH",
		IATA:      "ZAH",
		Name:      "Zahedan International Airport",
		City:      "Zahedan",
		State:     "Sistan-and-Baluchestan",
		Country:   "IR",
		Elevation: 4564,
		Latitude:  29.4757003784,
		Longitude: 60.9062004089,
		Timezone:  "Asia/Tehran",
	},
	"OIZI": {
		ICAO:      "OIZI",
		IATA:      "IHR",
		Name:      "Iran Shahr Airport",
		State:     "Sistan-and-Baluchestan",
		Country:   "IR",
		Elevation: 2040,
		Latitude:  27.2360992432,
		Longitude: 60.7200012207,
		Timezone:  "Asia/Tehran",
	},
	"OIZJ": {
		ICAO:      "OIZJ",
		Name:      "Jask Airport",
		City:      "Jask",
		State:     "Hormozgan",
		Country:   "IR",
		Elevation: 19,
		Latitude:  25.6536006927,
		Longitude: 57.7993011475,
		Timezone:  "Asia/Tehran",
	},
	"OIZS": {
		ICAO:      "OIZS",
		Name:      "Saravan Airport",
		State:     "Sistan-and-Baluchestan",
		Country:   "IR",
		Elevation: 3930,
		Latitude:  27.4193000793,
		Longitude: 62.3157997131,
		Timezone:  "Asia/Tehran",
	},
	"OJ38": {
		ICAO:      "OJ38",
		Name:      "Zarqa Airport",
		City:      "Zarqa",
		State:     "Zarqa",
		Country:   "JO",
		Elevation: 2400,
		Latitude:  32.0252990723,
		Longitude: 36.1447982788,
		Timezone:  "Asia/Amman",
	},
	"OJ40": {
		ICAO:      "OJ40",
		Name:      "Al Azraq / Al Shaheed Muwaffaq Salti Air Base",
		Country:   "JO",
		Elevation: 1706,
		Latitude:  31.8255996704,
		Longitude: 36.7820014954,
		Timezone:  "Asia/Amman",
	},
	"OJAI": {
		ICAO:      "OJAI",
		IATA:      "AMM",
		Name:      "Queen Alia International Airport",
		City:      "Amman",
		State:     "Amman",
		Country:   "JO",
		Elevation: 2395,
		Latitude:  31.7226009369,
		Longitude: 35.9931983948,
		Timezone:  "Asia/Amman",
	},
	"OJAM": {
		ICAO:      "OJAM",
		IATA:      "ADJ",
		Name:      "Amman-Marka International Airport",
		City:      "Amman",
		State:     "Amman",
		Country:   "JO",
		Elevation: 2555,
		Latitude:  31.972700119,
		Longitude: 35.9916000366,
		Timezone:  "Asia/Amman",
	},
	"OJAQ": {
		ICAO:      "OJAQ",
		IATA:      "AQJ",
		Name:      "Aqaba King Hussein International Airport",
		City:      "Aqaba",
		State:     "Aqaba",
		Country:   "JO",
		Elevation: 175,
		Latitude:  29.6116008759,
		Longitude: 35.0181007385,
		Timezone:  "Asia/Amman",
	},
	"OJHF": {
		ICAO:      "OJHF",
		Name:      "Prince Hasan Air Base",
		State:     "Mafraq",
		Country:   "JO",
		Elevation: 2220,
		Latitude:  32.1607017517,
		Longitude: 37.1493988037,
		Timezone:  "Asia/Amman",
	},
	"OJMF": {
		ICAO:      "OJMF",
		IATA:      "OMF",
		Name:      "King Hussein Air College",
		City:      "Mafraq",
		State:     "Mafraq",
		Country:   "JO",
		Elevation: 2240,
		Latitude:  32.3563995361,
		Longitude: 36.2592010498,
		Timezone:  "Asia/Amman",
	},
	"OK00": {
		ICAO:      "OK00",
		Name:      "Jacktown Airport",
		City:      "Jacktown",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 954,
		Latitude:  35.5163002014,
		Longitude: -97.022102356,
		Timezone:  "America/Chicago",
	},
	"OK01": {
		ICAO:      "OK01",
		Name:      "Sky Haven Airpark/Sellmeyer Field",
		City:      "Vera",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 675,
		Latitude:  36.4290008545,
		Longitude: -95.9032974243,
		Timezone:  "America/Chicago",
	},
	"OK02": {
		ICAO:      "OK02",
		Name:      "Dick's Airport",
		City:      "Goldsby",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1160,
		Latitude:  35.1108016968,
		Longitude: -97.4816970825,
		Timezone:  "America/Chicago",
	},
	"OK03": {
		ICAO:      "OK03",
		IATA:      "DWN",
		Name:      "Downtown Airpark",
		City:      "Oklahoma City",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1180,
		Latitude:  35.4491996765,
		Longitude: -97.5330963135,
		Timezone:  "America/Chicago",
	},
	"OK04": {
		ICAO:      "OK04",
		Name:      "Canadian River Ranch Airport",
		City:      "Eufaula",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 850,
		Latitude:  35.3093986511,
		Longitude: -95.8082962036,
		Timezone:  "America/Chicago",
	},
	"OK05": {
		ICAO:      "OK05",
		Name:      "Ray Preston Airport",
		City:      "Hollis",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1660,
		Latitude:  34.7000999451,
		Longitude: -99.9253997803,
		Timezone:  "America/Chicago",
	},
	"OK06": {
		ICAO:      "OK06",
		Name:      "Snake Creek Wilderness Airport",
		City:      "Cookson",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1130,
		Latitude:  35.6478004456,
		Longitude: -94.9499969482,
		Timezone:  "America/Chicago",
	},
	"OK07": {
		ICAO:      "OK07",
		Name:      "Djs Airport",
		City:      "Lawton",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1115,
		Latitude:  34.5181999207,
		Longitude: -98.3255996704,
		Timezone:  "America/Chicago",
	},
	"OK08": {
		ICAO:      "OK08",
		Name:      "Hill Top Private Airport",
		City:      "Lawton",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1100,
		Latitude:  34.5334014893,
		Longitude: -98.3503036499,
		Timezone:  "America/Chicago",
	},
	"OK10": {
		ICAO:      "OK10",
		Name:      "Entropy Airport",
		City:      "Elmore City",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1020,
		Latitude:  34.5819015503,
		Longitude: -97.4096984863,
		Timezone:  "America/Chicago",
	},
	"OK11": {
		ICAO:      "OK11",
		Name:      "Ksa Orchards Airport",
		City:      "Comanche",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1080,
		Latitude:  34.3175010681,
		Longitude: -98.0149993896,
		Timezone:  "America/Chicago",
	},
	"OK12": {
		ICAO:      "OK12",
		Name:      "Jones Farm Field",
		City:      "Walters",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1042,
		Latitude:  34.399167,
		Longitude: -98.266389,
		Timezone:  "America/Chicago",
	},
	"OK13": {
		ICAO:      "OK13",
		Name:      "Erroport Airport",
		City:      "Mounds",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 660,
		Latitude:  35.8457984924,
		Longitude: -95.9710998535,
		Timezone:  "America/Chicago",
	},
	"OK14": {
		ICAO:      "OK14",
		Name:      "Woodlake Airport",
		City:      "Alva",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1420,
		Latitude:  36.8352012634,
		Longitude: -98.657699585,
		Timezone:  "America/Chicago",
	},
	"OK15": {
		ICAO:      "OK15",
		Name:      "Avian Country Estates Airport",
		City:      "Oolagah",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 750,
		Latitude:  36.3760986328,
		Longitude: -95.7052993774,
		Timezone:  "America/Chicago",
	},
	"OK16": {
		ICAO:      "OK16",
		Name:      "Fairmont Field Airport",
		City:      "Fairmont",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1168,
		Latitude:  36.3647003174,
		Longitude: -97.6678009033,
		Timezone:  "America/Chicago",
	},
	"OK17": {
		ICAO:      "OK17",
		Name:      "Bass Aero Airport",
		City:      "Ardmore",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 840,
		Latitude:  34.21825,
		Longitude: -97.052567,
		Timezone:  "America/Chicago",
	},
	"OK18": {
		ICAO:      "OK18",
		Name:      "Grand Isle Airport",
		City:      "Adair",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 636,
		Latitude:  36.4234008789,
		Longitude: -95.1710968018,
		Timezone:  "America/Chicago",
	},
	"OK20": {
		ICAO:      "OK20",
		Name:      "Sageeyah Airfield",
		City:      "Claremore",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 640,
		Latitude:  36.3837013245,
		Longitude: -95.6480026245,
		Timezone:  "America/Chicago",
	},
	"OK21": {
		ICAO:      "OK21",
		Name:      "Longs Airport North Airport",
		City:      "Pryor",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 650,
		Latitude:  36.3334007263,
		Longitude: -95.304397583,
		Timezone:  "America/Chicago",
	},
	"OK22": {
		ICAO:      "OK22",
		Name:      "Bluebird Airpark",
		City:      "Shawnee",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1030,
		Latitude:  35.4029998779,
		Longitude: -96.8321990967,
		Timezone:  "America/Chicago",
	},
	"OK23": {
		ICAO:      "OK23",
		Name:      "Taliaferro Field",
		City:      "Ardmore",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 930,
		Latitude:  34.19829941,
		Longitude: -97.23120117,
		Timezone:  "America/Chicago",
	},
	"OK24": {
		ICAO:      "OK24",
		Name:      "Colby Field",
		City:      "Lone Grove",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 930,
		Latitude:  34.206199646,
		Longitude: -97.2311019897,
		Timezone:  "America/Chicago",
	},
	"OK25": {
		ICAO:      "OK25",
		Name:      "Cherokee Ranch Airport",
		City:      "Haskell",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 710,
		Latitude:  35.8133010864,
		Longitude: -95.7533035278,
		Timezone:  "America/Chicago",
	},
	"OK27": {
		ICAO:      "OK27",
		Name:      "Venture Aerodrome Airpark Nr 2 Ultralightport",
		City:      "Roland",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 450,
		Latitude:  35.4356002808,
		Longitude: -94.474899292,
		Timezone:  "America/Chicago",
	},
	"OK28": {
		ICAO:      "OK28",
		Name:      "Mcdaniel Aviation Airport",
		City:      "Ravia",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 700,
		Latitude:  34.229167,
		Longitude: -96.749167,
		Timezone:  "America/Chicago",
	},
	"OK29": {
		ICAO:      "OK29",
		Name:      "Travis Airport",
		City:      "Marietta",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 862,
		Latitude:  33.9359016418,
		Longitude: -97.0544967651,
		Timezone:  "America/Chicago",
	},
	"OK30": {
		ICAO:      "OK30",
		Name:      "Grandcraft Landing Strip",
		City:      "Langley",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 827,
		Latitude:  36.4917984009,
		Longitude: -95.050201416,
		Timezone:  "America/Chicago",
	},
	"OK31": {
		ICAO:      "OK31",
		Name:      "Whittington Ranch Airport",
		City:      "Thackerville",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 845,
		Latitude:  33.778542,
		Longitude: -97.131458,
		Timezone:  "America/Chicago",
	},
	"OK34": {
		ICAO:      "OK34",
		Name:      "Gustafson Airport",
		City:      "Sallisaw",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 855,
		Latitude:  35.4842987061,
		Longitude: -94.8419036865,
		Timezone:  "America/Chicago",
	},
	"OK36": {
		ICAO:      "OK36",
		Name:      "W C Ranch STOLport",
		City:      "Crowder",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 725,
		Latitude:  35.1197013855,
		Longitude: -95.7039031982,
		Timezone:  "America/Chicago",
	},
	"OK37": {
		ICAO:      "OK37",
		Name:      "Dog Iron Ranch Airport",
		City:      "Oologah",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 760,
		Latitude:  36.4695015,
		Longitude: -95.6678009,
		Timezone:  "America/Chicago",
	},
	"OK39": {
		ICAO:      "OK39",
		Name:      "Judy Ranch Airport",
		City:      "Forgan",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 2240,
		Latitude:  36.9667015076,
		Longitude: -100.3170013428,
		Timezone:  "America/Chicago",
	},
	"OK40": {
		ICAO:      "OK40",
		Name:      "Eden Ranch Airport",
		City:      "Freedom",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1641,
		Latitude:  36.7583999634,
		Longitude: -99.0336990356,
		Timezone:  "America/Chicago",
	},
	"OK42": {
		ICAO:      "OK42",
		Name:      "Siegmanns Airport",
		City:      "Hennessey",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1053,
		Latitude:  36.0834007263,
		Longitude: -97.7170028687,
		Timezone:  "America/Chicago",
	},
	"OK43": {
		ICAO:      "OK43",
		Name:      "Logsdon Ranch Airport",
		City:      "Alva",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1475,
		Latitude:  36.5306015015,
		Longitude: -98.6367034912,
		Timezone:  "America/Chicago",
	},
	"OK44": {
		ICAO:      "OK44",
		Name:      "Canyon Springs Ranch Airport",
		City:      "Bokchito",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 625,
		Latitude:  34.1461982727,
		Longitude: -96.0768966675,
		Timezone:  "America/Chicago",
	},
	"OK46": {
		ICAO:      "OK46",
		Name:      "Mc Crays Airport",
		City:      "Manchester",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1260,
		Latitude:  36.9589004517,
		Longitude: -98.075302124,
		Timezone:  "America/Chicago",
	},
	"OK47": {
		ICAO:      "OK47",
		Name:      "Miller Brothers Airport",
		City:      "Manchester",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1250,
		Latitude:  36.9416999817,
		Longitude: -98.016998291,
		Timezone:  "America/Chicago",
	},
	"OK48": {
		ICAO:      "OK48",
		Name:      "Grass Roots Airport",
		City:      "Newkirk",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1113,
		Latitude:  36.875,
		Longitude: -97.0961990356,
		Timezone:  "America/Chicago",
	},
	"OK49": {
		ICAO:      "OK49",
		Name:      "Secrest Ranch Airport",
		City:      "Ponca City",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1050,
		Latitude:  36.6500015259,
		Longitude: -97.016998291,
		Timezone:  "America/Chicago",
	},
	"OK50": {
		ICAO:      "OK50",
		Name:      "Traynor Ranch Airport",
		City:      "Waukomis",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1220,
		Latitude:  36.3325004578,
		Longitude: -97.8930969238,
		Timezone:  "America/Chicago",
	},
	"OK51": {
		ICAO:      "OK51",
		Name:      "Enix Boys Airport",
		City:      "Hennessey",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1100,
		Latitude:  36.1041984558,
		Longitude: -97.93699646,
		Timezone:  "America/Chicago",
	},
	"OK54": {
		ICAO:      "OK54",
		Name:      "May Ranch Airport",
		City:      "Alva",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1833,
		Latitude:  36.9967002869,
		Longitude: -99.0047988892,
		Timezone:  "America/Chicago",
	},
	"OK55": {
		ICAO:      "OK55",
		Name:      "Bost Ranch Airport",
		City:      "Luther",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1070,
		Latitude:  35.7145004272,
		Longitude: -97.2258987427,
		Timezone:  "America/Chicago",
	},
	"OK58": {
		ICAO:      "OK58",
		Name:      "Bluestem Airport",
		City:      "Collinsville",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 660,
		Latitude:  36.3973007202,
		Longitude: -95.9049987793,
		Timezone:  "America/Chicago",
	},
	"OK59": {
		ICAO:      "OK59",
		Name:      "Wolf Mountain Airport",
		City:      "Poteau",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 860,
		Latitude:  35.0223007202,
		Longitude: -94.7043991089,
		Timezone:  "America/Chicago",
	},
	"OK62": {
		ICAO:      "OK62",
		Name:      "Strader Ranch Airport",
		City:      "Ardmore",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 850,
		Latitude:  34.2486991882,
		Longitude: -97.209197998,
		Timezone:  "America/Chicago",
	},
	"OK64": {
		ICAO:      "OK64",
		Name:      "Weedpatch International Airport",
		City:      "Wakita",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1113,
		Latitude:  36.8138999939,
		Longitude: -97.9865036011,
		Timezone:  "America/Chicago",
	},
	"OK66": {
		ICAO:      "OK66",
		Name:      "F.W. Zaloudek Airport",
		City:      "Kremlin",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1118,
		Latitude:  36.5567016602,
		Longitude: -97.8337020874,
		Timezone:  "America/Chicago",
	},
	"OK69": {
		ICAO:      "OK69",
		Name:      "Monarch Field",
		City:      "Chelsea",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 755,
		Latitude:  36.525100708,
		Longitude: -95.4669036865,
		Timezone:  "America/Chicago",
	},
	"OK74": {
		ICAO:      "OK74",
		Name:      "Flying H Airport",
		City:      "Ashland",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 850,
		Latitude:  34.829498291,
		Longitude: -96.0342025757,
		Timezone:  "America/Chicago",
	},
	"OK79": {
		ICAO:      "OK79",
		Name:      "Temple Airport Inc Airport",
		City:      "Temple",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1005,
		Latitude:  34.2568016052,
		Longitude: -98.2412033081,
		Timezone:  "America/Chicago",
	},
	"OK80": {
		ICAO:      "OK80",
		Name:      "Thomas Ranch Airport",
		City:      "Ada",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 811,
		Latitude:  34.7286987305,
		Longitude: -96.6010971069,
		Timezone:  "America/Chicago",
	},
	"OK81": {
		ICAO:      "OK81",
		Name:      "Homer Ranch Airport",
		City:      "Albion",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 570,
		Latitude:  34.6584014893,
		Longitude: -95.0710983276,
		Timezone:  "America/Chicago",
	},
	"OK82": {
		ICAO:      "OK82",
		Name:      "Scottys Field",
		City:      "Altus",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1425,
		Latitude:  34.6916999817,
		Longitude: -99.3448028564,
		Timezone:  "America/Chicago",
	},
	"OK83": {
		ICAO:      "OK83",
		Name:      "Sheffield-Smith Airstrip",
		City:      "Altus",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1355,
		Latitude:  34.5682983398,
		Longitude: -99.3516998291,
		Timezone:  "America/Chicago",
	},
	"OK85": {
		ICAO:      "OK85",
		Name:      "Goddard Ranch Airport",
		City:      "Ardmore",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 858,
		Latitude:  34.3000984192,
		Longitude: -96.9169998169,
		Timezone:  "America/Chicago",
	},
	"OK87": {
		ICAO:      "OK87",
		Name:      "Renavair Field",
		City:      "Wynnewood",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 905,
		Latitude:  34.6549987793,
		Longitude: -97.2146987915,
		Timezone:  "America/Chicago",
	},
	"OK88": {
		ICAO:      "OK88",
		Name:      "Silverwood Ultralightport",
		City:      "Broken Arrow",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 632,
		Latitude:  36.0101013184,
		Longitude: -95.8435974121,
		Timezone:  "America/Chicago",
	},
	"OK89": {
		ICAO:      "OK89",
		Name:      "Stuart Ranch Airport",
		City:      "Caddo",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 624,
		Latitude:  34.1556015015,
		Longitude: -96.1557998657,
		Timezone:  "America/Chicago",
	},
	"OK90": {
		ICAO:      "OK90",
		Name:      "Boatner Field",
		City:      "Calera",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 734,
		Latitude:  33.9020004272,
		Longitude: -96.4721984863,
		Timezone:  "America/Chicago",
	},
	"OK91": {
		ICAO:      "OK91",
		Name:      "Stidham-Private Airport",
		City:      "Chickasha",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1177,
		Latitude:  34.9833984375,
		Longitude: -97.9337005615,
		Timezone:  "America/Chicago",
	},
	"OK93": {
		ICAO:      "OK93",
		Name:      "Airman Acres Airport",
		City:      "Collinsville",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 695,
		Latitude:  36.3334007263,
		Longitude: -95.8835983276,
		Timezone:  "America/Chicago",
	},
	"OK94": {
		ICAO:      "OK94",
		Name:      "Sand Ridge Airpark Inc Airport",
		City:      "Collinsville",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 675,
		Latitude:  36.354801178,
		Longitude: -95.8032989502,
		Timezone:  "America/Chicago",
	},
	"OK95": {
		ICAO:      "OK95",
		Name:      "Disney Airport",
		City:      "Disney",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 955,
		Latitude:  36.4889984131,
		Longitude: -94.9582977295,
		Timezone:  "America/Chicago",
	},
	"OK96": {
		ICAO:      "OK96",
		Name:      "Ives Airport",
		City:      "Wellston",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1035,
		Latitude:  35.756401062,
		Longitude: -97.0848007202,
		Timezone:  "America/Chicago",
	},
	"OK97": {
		ICAO:      "OK97",
		Name:      "Ketchum Ranch Airport",
		City:      "Duncan",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1100,
		Latitude:  34.5334014893,
		Longitude: -97.766998291,
		Timezone:  "America/Chicago",
	},
	"OK98": {
		ICAO:      "OK98",
		Name:      "King Airport",
		City:      "Eakly",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1535,
		Latitude:  35.3000984192,
		Longitude: -98.4837036133,
		Timezone:  "America/Chicago",
	},
	"OKAJ": {
		ICAO:      "OKAJ",
		IATA:      "XIJ",
		Name:      "Ahmed Al Jaber Air Base",
		City:      "Ahmed Al Jaber AB",
		Country:   "KW",
		Elevation: 409,
		Latitude:  28.9347991943,
		Longitude: 47.7919006348,
		Timezone:  "Asia/Kuwait",
	},
	"OKAS": {
		ICAO:      "OKAS",
		Name:      "Ali Al Salem Air Base",
		Country:   "KW",
		Elevation: 472,
		Latitude:  29.3467006683,
		Longitude: 47.5208015442,
		Timezone:  "Asia/Kuwait",
	},
	"OKDI": {
		ICAO:      "OKDI",
		Name:      "Udairi Army Air Field",
		City:      "Camp Buehring",
		State:     "Al-Jahraʼ",
		Country:   "KW",
		Elevation: 430,
		Latitude:  29.697599411,
		Longitude: 47.436000824,
		Timezone:  "Asia/Kuwait",
	},
	"OKKK": {
		ICAO:      "OKKK",
		IATA:      "KWI",
		Name:      "Kuwait International Airport",
		City:      "Kuwait City",
		Country:   "KW",
		Elevation: 206,
		Latitude:  29.226600647,
		Longitude: 47.9688987732,
		Timezone:  "Asia/Kuwait",
	},
	"OL02": {
		ICAO:      "OL02",
		Name:      "West Buttercreek Airport",
		City:      "Echo",
		State:     "Oregon",
		Country:   "US",
		Elevation: 917,
		Latitude:  45.6652984619,
		Longitude: -119.3860015869,
		Timezone:  "America/Los_Angeles",
	},
	"OL03": {
		ICAO:      "OL03",
		Name:      "Happy Valley Airport",
		City:      "Happy Valley",
		State:     "Oregon",
		Country:   "US",
		Elevation: 797,
		Latitude:  45.4482002258,
		Longitude: -122.5,
		Timezone:  "America/Los_Angeles",
	},
	"OL04": {
		ICAO:      "OL04",
		Name:      "Decker Ranch Airport",
		City:      "Kent",
		State:     "Oregon",
		Country:   "US",
		Elevation: 2621,
		Latitude:  45.1665000916,
		Longitude: -120.6679992676,
		Timezone:  "America/Los_Angeles",
	},
	"OL05": {
		ICAO:      "OL05",
		Name:      "Skydive Oregon Airport",
		City:      "Molalla",
		State:     "Oregon",
		Country:   "US",
		Elevation: 360,
		Latitude:  45.1461982727,
		Longitude: -122.6179962158,
		Timezone:  "America/Los_Angeles",
	},
	"OL09": {
		ICAO:      "OL09",
		Name:      "Jennings Ranch Airport",
		City:      "Cooperton",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1620,
		Latitude:  34.8833999634,
		Longitude: -98.7502975464,
		Timezone:  "America/Chicago",
	},
	"OL12": {
		ICAO:      "OL12",
		Name:      "Northwest Edmond Airport",
		City:      "Edmond",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 1070,
		Latitude:  35.7075996399,
		Longitude: -97.5406036377,
		Timezone:  "America/Chicago",
	},
	"OL19": {
		ICAO:      "OL19",
		Name:      "Stuart Mountain Airpark",
		City:      "Stuart",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 990,
		Latitude:  34.9154014587,
		Longitude: -96.1294021606,
		Timezone:  "America/Chicago",
	},
	"OL20": {
		ICAO:      "OL20",
		Name:      "Whitehorn Cove Airport",
		City:      "Wagoner",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 568,
		Latitude:  35.9972991943,
		Longitude: -95.2589035034,
		Timezone:  "America/Chicago",
	},
	"OL23": {
		ICAO:      "OL23",
		Name:      "Morris Airport",
		City:      "Sapulpa",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 730,
		Latitude:  35.9500999451,
		Longitude: -96.1044006348,
		Timezone:  "America/Chicago",
	},
	"OL92": {
		ICAO:      "OL92",
		Name:      "Hi-Way Airport",
		City:      "Bartlesville",
		State:     "Oklahoma",
		Country:   "US",
		Elevation: 680,
		Latitude:  36.7834014893,
		Longitude: -95.950302124,
		Timezone:  "America/Chicago",
	},
	"OLBA": {
		ICAO:      "OLBA",
		IATA:      "BEY",
		Name:      "Beirut Rafic Hariri International Airport",
		City:      "Beirut",
		State:     "Mont-Liban",
		Country:   "LB",
		Elevation: 87,
		Latitude:  33.8208999634,
		Longitude: 35.4883995056,
		Timezone:  "Asia/Beirut",
	},
	"OLKA": {
		ICAO:      "OLKA",
		IATA:      "KYE",
		Name:      "Rene Mouawad Air Base",
		City:      "Tripoli",
		State:     "Liban-Nord",
		Country:   "LB",
		Elevation: 75,
		Latitude:  34.5892982483,
		Longitude: 36.0112991333,
		Timezone:  "Asia/Beirut",
	},
	"OLRA": {
		ICAO:      "OLRA",
		Name:      "Rayak Air Base",
		City:      "Rayak",
		State:     "Beqaa",
		Country:   "LB",
		Elevation: 3018,
		Latitude:  33.8507995605,
		Longitude: 35.987701416,
		Timezone:  "Asia/Beirut",
	},
	"OMAA": {
		ICAO:      "OMAA",
		IATA:      "AUH",
		Name:      "Abu Dhabi International Airport",
		City:      "Abu Dhabi",
		State:     "Abu-Dhabi",
		Country:   "AE",
		Elevation: 88,
		Latitude:  24.4330005646,
		Longitude: 54.6511001587,
		Timezone:  "Asia/Dubai",
	},
	"OMAB": {
		ICAO:      "OMAB",
		Name:      "Buhasa Airport",
		State:     "Abu-Dhabi",
		Country:   "AE",
		Latitude:  23.5997009277,
		Longitude: 53.3793983459,
		Timezone:  "Asia/Dubai",
	},
	"OMAD": {
		ICAO:      "OMAD",
		IATA:      "AZI",
		Name:      "Al Bateen Executive Airport",
		State:     "Abu-Dhabi",
		Country:   "AE",
		Elevation: 16,
		Latitude:  24.4283008575,
		Longitude: 54.4580993652,
		Timezone:  "Asia/Dubai",
	},
	"OMAF": {
		ICAO:      "OMAF",
		Name:      "Futaysi",
		State:     "Abu-Dhabi",
		Country:   "AE",
		Latitude:  24.3788890839,
		Longitude: 54.3161125183,
		Timezone:  "Asia/Dubai",
	},
	"OMAH": {
		ICAO:      "OMAH",
		Name:      "Al Hamra Aux Airport",
		State:     "Abu-Dhabi",
		Country:   "AE",
		Elevation: 50,
		Latitude:  24.0739994049,
		Longitude: 52.4636001587,
		Timezone:  "Asia/Dubai",
	},
	"OMAJ": {
		ICAO:      "OMAJ",
		Name:      "Jebel Dhana Airport",
		State:     "Abu-Dhabi",
		Country:   "AE",
		Elevation: 43,
		Latitude:  24.1874008179,
		Longitude: 52.6139984131,
		Timezone:  "Asia/Dubai",
	},
	"OMAL": {
		ICAO:      "OMAL",
		IATA:      "AAN",
		Name:      "Al Ain International Airport",
		City:      "Al Ain",
		State:     "Abu-Dhabi",
		Country:   "AE",
		Elevation: 869,
		Latitude:  24.2616996765,
		Longitude: 55.6091995239,
		Timezone:  "Asia/Dubai",
	},
	"OMAM": {
		ICAO:      "OMAM",
		IATA:      "DHF",
		Name:      "Al Dhafra Air Base",
		State:     "Abu-Dhabi",
		Country:   "AE",
		Elevation: 77,
		Latitude:  24.2481994629,
		Longitude: 54.5476989746,
		Timezone:  "Asia/Dubai",
	},
	"OMAQ": {
		ICAO:      "OMAQ",
		Name:      "Qarnayn Airport",
		City:      "Qarnayn Island",
		State:     "Abu-Dhabi",
		Country:   "AE",
		Latitude:  24.9305992126,
		Longitude: 52.8550987244,
		Timezone:  "Asia/Dubai",
	},
	"OMAR": {
		ICAO:      "OMAR",
		Name:      "Arzanah Airport",
		City:      "Arzanah Island",
		State:     "Abu-Dhabi",
		Country:   "AE",
		Elevation: 15,
		Latitude:  24.780500412,
		Longitude: 52.5598983765,
		Timezone:  "Asia/Dubai",
	},
	"OMAS": {
		ICAO:      "OMAS",
		Name:      "Das Island Airport",
		City:      "Das Island",
		State:     "Abu-Dhabi",
		Country:   "AE",
		Elevation: 12,
		Latitude:  25.1462001801,
		Longitude: 52.8736991882,
		Timezone:  "Asia/Dubai",
	},
	"OMAZ": {
		ICAO:      "OMAZ",
		Name:      "Zirku Airport",
		City:      "Zirku Island",
		State:     "Abu-Dhabi",
		Country:   "AE",
		Elevation: 14,
		Latitude:  24.8624916077,
		Longitude: 53.0771408081,
		Timezone:  "Asia/Dubai",
	},
	"OMDB": {
		ICAO:      "OMDB",
		IATA:      "DXB",
		Name:      "Dubai International Airport",
		City:      "Dubai",
		State:     "Dubai",
		Country:   "AE",
		Elevation: 34,
		Latitude:  25.2527999878,
		Longitude: 55.3643989563,
		Timezone:  "Asia/Dubai",
	},
	"OMDM": {
		ICAO:      "OMDM",
		IATA:      "NHD",
		Name:      "Al Minhad Air Base",
		City:      "Dubai",
		State:     "Dubai",
		Country:   "AE",
		Elevation: 165,
		Latitude:  25.0268001556,
		Longitude: 55.3661994934,
		Timezone:  "Asia/Dubai",
	},
	"OMDW": {
		ICAO:      "OMDW",
		IATA:      "DWC",
		Name:      "Al Maktoum International Airport",
		City:      "Jebel Ali",
		State:     "Dubai",
		Country:   "AE",
		Elevation: 114,
		Latitude:  24.8966666667,
		Longitude: 55.1613888889,
		Timezone:  "Asia/Dubai",
	},
	"OMFJ": {
		ICAO:      "OMFJ",
		IATA:      "FJR",
		Name:      "Fujairah International Airport",
		State:     "Al-Fujayrah",
		Country:   "AE",
		Elevation: 152,
		Latitude:  25.1121997833,
		Longitude: 56.3240013123,
		Timezone:  "Asia/Dubai",
	},
	"OMRK": {
		ICAO:      "OMRK",
		IATA:      "RKT",
		Name:      "Ras Al Khaimah International Airport",
		City:      "Ras Al Khaimah",
		State:     "Raʼs-al-Khaymah",
		Country:   "AE",
		Elevation: 102,
		Latitude:  25.6135005951,
		Longitude: 55.9388008118,
		Timezone:  "Asia/Dubai",
	},
	"OMRS": {
		ICAO:      "OMRS",
		Name:      "Al Saqr Field",
		State:     "Raʼs-al-Khaymah",
		Country:   "AE",
		Latitude:  25.7527770996,
		Longitude: 55.9594459534,
		Timezone:  "Asia/Dubai",
	},
	"OMSJ": {
		ICAO:      "OMSJ",
		IATA:      "SHJ",
		Name:      "Sharjah International Airport",
		City:      "Sharjah",
		State:     "Ash-Shariqah",
		Country:   "AE",
		Elevation: 111,
		Latitude:  25.3285999298,
		Longitude: 55.51720047,
		Timezone:  "Asia/Dubai",
	},
	"OMU9": {
		ICAO:      "OMU9",
		Name:      "Kollmeyer Airport",
		City:      "Pilot Grove",
		State:     "Missouri",
		Country:   "US",
		Elevation: 810,
		Latitude:  38.8842010498,
		Longitude: -92.985496521,
		Timezone:  "America/Chicago",
	},
	"OMUQ": {
		ICAO:      "OMUQ",
		Name:      "Umm Al Quwain",
		State:     "Umm-al-Qaywayn",
		Country:   "AE",
		Latitude:  25.5772228241,
		Longitude: 55.6511116028,
		Timezone:  "Asia/Dubai",
	},
	"OOBR": {
		ICAO:      "OOBR",
		IATA:      "RMB",
		Name:      "Buraimi Airport",
		City:      "Buraimi",
		State:     "Al-Buraimi",
		Country:   "OM",
		Elevation: 970,
		Latitude:  24.2329998016,
		Longitude: 55.783000946,
		Timezone:  "Asia/Dubai",
	},
	"OODQ": {
		ICAO:      "OODQ",
		IATA:      "DQM",
		Name:      "Duqm International Airport",
		City:      "Duqm",
		State:     "Duqm",
		Country:   "OM",
		Elevation: 383,
		Latitude:  19.5019,
		Longitude: 57.6342,
		Timezone:  "Asia/Muscat",
	},
	"OOFD": {
		ICAO:      "OOFD",
		IATA:      "FAU",
		Name:      "Fahud Airport",
		City:      "Fahud",
		State:     "Az̧-Z̧ahirah",
		Country:   "OM",
		Elevation: 552,
		Latitude:  22.354759318,
		Longitude: 56.4841461182,
		Timezone:  "Asia/Muscat",
	},
	"OOFQ": {
		ICAO:      "OOFQ",
		Name:      "Firq Air Base",
		City:      "Firq",
		State:     "Ad-Dakhiliyah",
		Country:   "OM",
		Elevation: 1560,
		Latitude:  22.8691997528,
		Longitude: 57.5433006287,
		Timezone:  "Asia/Muscat",
	},
	"OOGB": {
		ICAO:      "OOGB",
		IATA:      "RNM",
		Name:      "Qarn Alam Airport",
		City:      "Ghaba",
		State:     "Al-Wusţa",
		Country:   "OM",
		Elevation: 443,
		Latitude:  21.3829994202,
		Longitude: 57.0499992371,
		Timezone:  "Asia/Muscat",
	},
	"OOHA": {
		ICAO:      "OOHA",
		Name:      "Haima Airport",
		City:      "Haima",
		State:     "Al-Wusţa",
		Country:   "OM",
		Elevation: 400,
		Latitude:  19.966999054,
		Longitude: 56.283000946,
		Timezone:  "Asia/Muscat",
	},
	"OOIA": {
		ICAO:      "OOIA",
		Name:      "Ibra Airport",
		City:      "Ibra",
		State:     "Northeastern-Governorate",
		Country:   "OM",
		Elevation: 1500,
		Latitude:  22.7327777778,
		Longitude: 58.5136111111,
		Timezone:  "Asia/Muscat",
	},
	"OOIZ": {
		ICAO:      "OOIZ",
		Name:      "Izki Air Base",
		City:      "Izki",
		State:     "Ad-Dakhiliyah",
		Country:   "OM",
		Elevation: 1700,
		Latitude:  22.8917007446,
		Longitude: 57.7583007813,
		Timezone:  "Asia/Muscat",
	},
	"OOKB": {
		ICAO:      "OOKB",
		IATA:      "KHS",
		Name:      "Khasab Air Base",
		City:      "Khasab",
		State:     "Musandam",
		Country:   "OM",
		Elevation: 100,
		Latitude:  26.170999527,
		Longitude: 56.2406005859,
		Timezone:  "Asia/Muscat",
	},
	"OOLK": {
		ICAO:      "OOLK",
		IATA:      "LKW",
		Name:      "Lekhwair Airport",
		State:     "Az̧-Z̧ahirah",
		Country:   "OM",
		Elevation: 354,
		Latitude:  22.8049846844,
		Longitude: 55.3733682632,
		Timezone:  "Asia/Muscat",
	},
	"OOMA": {
		ICAO:      "OOMA",
		IATA:      "MSH",
		Name:      "Masirah Air Base",
		City:      "Masirah",
		State:     "Southeastern-Governorate",
		Country:   "OM",
		Elevation: 64,
		Latitude:  20.6753997803,
		Longitude: 58.890499115,
		Timezone:  "Asia/Muscat",
	},
	"OOMS": {
		ICAO:      "OOMS",
		IATA:      "MCT",
		Name:      "Muscat International Airport",
		City:      "Muscat",
		State:     "Muscat",
		Country:   "OM",
		Elevation: 48,
		Latitude:  23.5932998657,
		Longitude: 58.2844009399,
		Timezone:  "Asia/Muscat",
	},
	"OOMX": {
		ICAO:      "OOMX",
		IATA:      "OMM",
		Name:      "Marmul Airport",
		City:      "Marmul",
		State:     "Dhofar",
		Country:   "OM",
		Elevation: 925,
		Latitude:  18.1359996796,
		Longitude: 55.1820983887,
		Timezone:  "Asia/Muscat",
	},
	"OORQ": {
		ICAO:      "OORQ",
		IATA:      "MNH",
		Name:      "Rustaq Airport",
		City:      "Al Muladdah",
		State:     "Al-Batinah-South",
		Country:   "OM",
		Elevation: 349,
		Latitude:  23.6405555556,
		Longitude: 57.4875,
		Timezone:  "Asia/Muscat",
	},
	"OOSA": {
		ICAO:      "OOSA",
		IATA:      "SLL",
		Name:      "Salalah Airport",
		City:      "Salalah",
		State:     "Dhofar",
		Country:   "OM",
		Elevation: 73,
		Latitude:  17.0387001038,
		Longitude: 54.0913009644,
		Timezone:  "Asia/Muscat",
	},
	"OOSH": {
		ICAO:      "OOSH",
		IATA:      "OHS",
		Name:      "Sohar Airport",
		City:      "Sohar",
		State:     "Al-Batinah-North",
		Country:   "OM",
		Elevation: 105,
		Latitude:  24.4641666667,
		Longitude: 56.6283333333,
		Timezone:  "Asia/Muscat",
	},
	"OOSQ": {
		ICAO:      "OOSQ",
		Name:      "Saiq Airport",
		City:      "Saiq",
		State:     "Ad-Dakhiliyah",
		Country:   "OM",
		Elevation: 6500,
		Latitude:  23.0669994354,
		Longitude: 57.6500015259,
		Timezone:  "Asia/Muscat",
	},
	"OOSR": {
		ICAO:      "OOSR",
		IATA:      "SUH",
		Name:      "Sur Airport",
		City:      "Sur",
		State:     "Southeastern-Governorate",
		Country:   "OM",
		Elevation: 20,
		Latitude:  22.533000946,
		Longitude: 59.483001709,
		Timezone:  "Asia/Muscat",
	},
	"OOTH": {
		ICAO:      "OOTH",
		IATA:      "TTH",
		Name:      "Thumrait Air Base",
		City:      "Thumrait",
		State:     "Dhofar",
		Country:   "OM",
		Elevation: 1570,
		Latitude:  17.6660003662,
		Longitude: 54.0246009827,
		Timezone:  "Asia/Muscat",
	},
	"OOYB": {
		ICAO:      "OOYB",
		Name:      "Yibal Airport",
		City:      "Yibal Oilfield",
		State:     "Az̧-Z̧ahirah",
		Country:   "OM",
		Latitude:  22.201499939,
		Longitude: 56.0344009399,
		Timezone:  "Asia/Muscat",
	},
	"OP10": {
		ICAO:      "OP10",
		Name:      "Thar Airport",
		City:      "Chuchra",
		State:     "Sindh",
		Country:   "PK",
		Elevation: 167,
		Latitude:  27.2005004883,
		Longitude: 69.1538009644,
		Timezone:  "Asia/Karachi",
	},
	"OP11": {
		ICAO:      "OP11",
		Name:      "Mir Baz Airport",
		City:      "Mir Baz",
		State:     "Balochistan",
		Country:   "PK",
		Elevation: 2350,
		Latitude:  31.1877994537,
		Longitude: 70.1845016479,
		Timezone:  "Asia/Karachi",
	},
	"OP13": {
		ICAO:      "OP13",
		Name:      "Mad Jamu Airport",
		City:      "Mad Jamu Kholelan",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 256,
		Latitude:  28.3561992645,
		Longitude: 70.6001968384,
		Timezone:  "Asia/Karachi",
	},
	"OP14": {
		ICAO:      "OP14",
		Name:      "Belab Airport",
		City:      "Belab",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 377,
		Latitude:  30.3400001526,
		Longitude: 70.5567016602,
		Timezone:  "Asia/Karachi",
	},
	"OP15": {
		ICAO:      "OP15",
		Name:      "Ouzkani Airport",
		City:      "Khandor",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 2331,
		Latitude:  30.2528991699,
		Longitude: 70.2128982544,
		Timezone:  "Asia/Karachi",
	},
	"OP16": {
		ICAO:      "OP16",
		Name:      "Khorewah Airport",
		City:      "Khorewah",
		State:     "Sindh",
		Country:   "PK",
		Elevation: 16,
		Latitude:  24.7872009277,
		Longitude: 68.5571975708,
		Timezone:  "Asia/Karachi",
	},
	"OP17": {
		ICAO:      "OP17",
		Name:      "Dadu West Airport",
		City:      "Dadu",
		State:     "Sindh",
		Country:   "PK",
		Elevation: 128,
		Latitude:  26.7408008575,
		Longitude: 67.6666030884,
		Timezone:  "Asia/Karachi",
	},
	"OP18": {
		ICAO:      "OP18",
		Name:      "Kot Addu Airport",
		City:      "Kot  Addu",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 423,
		Latitude:  30.4948997498,
		Longitude: 70.9710998535,
		Timezone:  "Asia/Karachi",
	},
	"OP19": {
		ICAO:      "OP19",
		Name:      "Chashma Airport",
		City:      "Kundian",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 645,
		Latitude:  32.4244995117,
		Longitude: 71.4585037231,
		Timezone:  "Asia/Karachi",
	},
	"OP1A": {
		ICAO:      "OP1A",
		Name:      "Jam Nida Northwest Airport",
		City:      "Jam Nida",
		State:     "Sindh",
		Country:   "PK",
		Elevation: 420,
		Latitude:  26.1980991364,
		Longitude: 67.5037002563,
		Timezone:  "Asia/Karachi",
	},
	"OP1Y": {
		ICAO:      "OP1Y",
		Name:      "Chandhar Airport",
		City:      "Hafizabad",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 620,
		Latitude:  32.0778007507,
		Longitude: 73.7901992798,
		Timezone:  "Asia/Karachi",
	},
	"OP21": {
		ICAO:      "OP21",
		Name:      "Velhari Airport",
		City:      "Velhari",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 430,
		Latitude:  30.0914001465,
		Longitude: 72.1538009644,
		Timezone:  "Asia/Karachi",
	},
	"OP22": {
		ICAO:      "OP22",
		Name:      "Rajanpur Airport",
		City:      "Rajanpur",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 401,
		Latitude:  29.263999939,
		Longitude: 70.1864013672,
		Timezone:  "Asia/Karachi",
	},
	"OP23": {
		ICAO:      "OP23",
		Name:      "Thal Airport",
		City:      "Thal",
		State:     "Khyber-Pakhtunkhwa",
		Country:   "PK",
		Elevation: 2650,
		Latitude:  33.3875999451,
		Longitude: 70.5887985229,
		Timezone:  "Asia/Karachi",
	},
	"OP25": {
		ICAO:      "OP25",
		Name:      "Gurha Salim Airport",
		City:      "Gurha Salim",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 800,
		Latitude:  32.8785018921,
		Longitude: 73.6053009033,
		Timezone:  "Asia/Karachi",
	},
	"OP26": {
		ICAO:      "OP26",
		Name:      "Khewra Airport",
		City:      "Khewra",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 767,
		Latitude:  32.6291007996,
		Longitude: 73.0225982666,
		Timezone:  "Asia/Karachi",
	},
	"OP27": {
		ICAO:      "OP27",
		Name:      "Rahwali Airport",
		City:      "Rahwali",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 745,
		Latitude:  32.2389984131,
		Longitude: 74.1309967041,
		Timezone:  "Asia/Karachi",
	},
	"OP28": {
		ICAO:      "OP28",
		Name:      "Dhingar Airport",
		City:      "Dhingar",
		State:     "Balochistan",
		Country:   "PK",
		Elevation: 5215,
		Latitude:  29.9232997894,
		Longitude: 66.7461013794,
		Timezone:  "Asia/Karachi",
	},
	"OP31": {
		ICAO:      "OP31",
		Name:      "Kashmor Airport",
		City:      "Kashmor",
		State:     "Sindh",
		Country:   "PK",
		Elevation: 260,
		Latitude:  28.4706001282,
		Longitude: 69.5990982056,
		Timezone:  "Asia/Karachi",
	},
	"OP32": {
		ICAO:      "OP32",
		Name:      "Khairpur Airport",
		City:      "Khairpur",
		State:     "Sindh",
		Country:   "PK",
		Elevation: 300,
		Latitude:  28.0361003876,
		Longitude: 69.6748962402,
		Timezone:  "Asia/Karachi",
	},
	"OP33": {
		ICAO:      "OP33",
		Name:      "Kandhkot Airport",
		City:      "Kandhkot",
		State:     "Sindh",
		Country:   "PK",
		Elevation: 250,
		Latitude:  28.2740001678,
		Longitude: 69.2760009766,
		Timezone:  "Asia/Karachi",
	},
	"OP34": {
		ICAO:      "OP34",
		Name:      "Nok Kundi Airport",
		City:      "Nok Kundi",
		State:     "Balochistan",
		Country:   "PK",
		Elevation: 2227,
		Latitude:  28.8194007874,
		Longitude: 62.7307014465,
		Timezone:  "Asia/Karachi",
	},
	"OP35": {
		ICAO:      "OP35",
		Name:      "Juzzak Airport",
		City:      "Juzzak",
		State:     "Balochistan",
		Country:   "PK",
		Elevation: 2861,
		Latitude:  29.0405006409,
		Longitude: 61.6473999023,
		Timezone:  "Asia/Karachi",
	},
	"OP36": {
		ICAO:      "OP36",
		Name:      "Robray Airport",
		State:     "Balochistan",
		Country:   "PK",
		Elevation: 1840,
		Latitude:  26.2518997192,
		Longitude: 63.1302986145,
		Timezone:  "Asia/Karachi",
	},
	"OP37": {
		ICAO:      "OP37",
		Name:      "Pano Aqil Southeast Airport",
		City:      "Pano Aqil",
		State:     "Sindh",
		Country:   "PK",
		Elevation: 200,
		Latitude:  27.8131008148,
		Longitude: 69.167098999,
		Timezone:  "Asia/Karachi",
	},
	"OPAB": {
		ICAO:      "OPAB",
		IATA:      "AAW",
		Name:      "Abbottabad Airport",
		City:      "Abbottabad",
		State:     "Khyber-Pakhtunkhwa",
		Country:   "PK",
		Elevation: 4072,
		Latitude:  34.2000007629,
		Longitude: 73.25,
		Timezone:  "Asia/Karachi",
	},
	"OPBG": {
		ICAO:      "OPBG",
		IATA:      "BHW",
		Name:      "Bhagatanwala Airport",
		City:      "Bhagatanwala",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 600,
		Latitude:  32.056098938,
		Longitude: 72.9484024048,
		Timezone:  "Asia/Karachi",
	},
	"OPBL": {
		ICAO:      "OPBL",
		Name:      "Bela Airport",
		City:      "Khan Bela",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 347,
		Latitude:  29.4922008514,
		Longitude: 71.1184005737,
		Timezone:  "Asia/Karachi",
	},
	"OPBN": {
		ICAO:      "OPBN",
		IATA:      "BNP",
		Name:      "Bannu Airport",
		City:      "Bannu",
		State:     "Khyber-Pakhtunkhwa",
		Country:   "PK",
		Elevation: 1325,
		Latitude:  32.9729003906,
		Longitude: 70.5279006958,
		Timezone:  "Asia/Karachi",
	},
	"OPBR": {
		ICAO:      "OPBR",
		IATA:      "WGB",
		Name:      "Bahawalnagar Airport",
		City:      "Bahawalnagar",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 500,
		Latitude:  29.9463005066,
		Longitude: 73.2490997314,
		Timezone:  "Asia/Karachi",
	},
	"OPBW": {
		ICAO:      "OPBW",
		IATA:      "BHV",
		Name:      "Bahawalpur Airport",
		City:      "Bahawalpur",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 392,
		Latitude:  29.3481006622,
		Longitude: 71.7180023193,
		Timezone:  "Asia/Karachi",
	},
	"OPCH": {
		ICAO:      "OPCH",
		IATA:      "CJL",
		Name:      "Chitral Airport",
		City:      "Chitral",
		State:     "Khyber-Pakhtunkhwa",
		Country:   "PK",
		Elevation: 4920,
		Latitude:  35.8866004944,
		Longitude: 71.8005981445,
		Timezone:  "Asia/Karachi",
	},
	"OPCL": {
		ICAO:      "OPCL",
		IATA:      "CHB",
		Name:      "Chilas Airport",
		City:      "Chilas",
		State:     "Gilgit-Baltistan",
		Country:   "PK",
		Elevation: 4146,
		Latitude:  35.426700592,
		Longitude: 74.081703186,
		Timezone:  "Asia/Karachi",
	},
	"OPCT": {
		ICAO:      "OPCT",
		Name:      "Chirat Airport",
		City:      "Nowshera",
		State:     "Khyber-Pakhtunkhwa",
		Country:   "PK",
		Elevation: 1192,
		Latitude:  33.9681015015,
		Longitude: 71.9674987793,
		Timezone:  "Asia/Karachi",
	},
	"OPDB": {
		ICAO:      "OPDB",
		IATA:      "DBA",
		Name:      "Dalbandin Airport",
		City:      "Dalbandin",
		State:     "Balochistan",
		Country:   "PK",
		Elevation: 2800,
		Latitude:  28.8782997131,
		Longitude: 64.3998031616,
		Timezone:  "Asia/Karachi",
	},
	"OPDD": {
		ICAO:      "OPDD",
		IATA:      "DDU",
		Name:      "Dadu Airport",
		City:      "Dadu",
		State:     "Sindh",
		Country:   "PK",
		Elevation: 121,
		Latitude:  26.5545005798,
		Longitude: 67.6745986938,
		Timezone:  "Asia/Karachi",
	},
	"OPDG": {
		ICAO:      "OPDG",
		IATA:      "DEA",
		Name:      "Dera Ghazi Khan Airport",
		City:      "Dera Ghazi Khan",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 492,
		Latitude:  29.9610004425,
		Longitude: 70.4859008789,
		Timezone:  "Asia/Karachi",
	},
	"OPDI": {
		ICAO:      "OPDI",
		IATA:      "DSK",
		Name:      "Dera Ismael Khan Airport",
		City:      "Dera Ismael Khan",
		State:     "Khyber-Pakhtunkhwa",
		Country:   "PK",
		Elevation: 594,
		Latitude:  31.9094009399,
		Longitude: 70.8965988159,
		Timezone:  "Asia/Karachi",
	},
	"OPDK": {
		ICAO:      "OPDK",
		Name:      "Daharki Airport",
		City:      "Daharki",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 274,
		Latitude:  28.5,
		Longitude: 69.6999969482,
		Timezone:  "Asia/Karachi",
	},
	"OPFA": {
		ICAO:      "OPFA",
		IATA:      "LYP",
		Name:      "Faisalabad International Airport",
		City:      "Faisalabad",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 591,
		Latitude:  31.3649997711,
		Longitude: 72.9947967529,
		Timezone:  "Asia/Karachi",
	},
	"OPGD": {
		ICAO:      "OPGD",
		IATA:      "GWD",
		Name:      "Gwadar International Airport",
		City:      "Gwadar",
		State:     "Balochistan",
		Country:   "PK",
		Elevation: 36,
		Latitude:  25.2332992554,
		Longitude: 62.329498291,
		Timezone:  "Asia/Karachi",
	},
	"OPGT": {
		ICAO:      "OPGT",
		IATA:      "GIL",
		Name:      "Gilgit Airport",
		City:      "Gilgit",
		State:     "Gilgit-Baltistan",
		Country:   "PK",
		Elevation: 4796,
		Latitude:  35.918800354,
		Longitude: 74.3336029053,
		Timezone:  "Asia/Karachi",
	},
	"OPIS": {
		ICAO:      "OPIS",
		IATA:      "ISB",
		Name:      "Islamabad International Airport",
		City:      "Islamabad",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 1761,
		Latitude:  33.5490833333333,
		Longitude: 72.82565,
		Timezone:  "Asia/Karachi",
	},
	"OPJA": {
		ICAO:      "OPJA",
		IATA:      "JAG",
		Name:      "Shahbaz Air Base",
		City:      "Jacobabad",
		State:     "Sindh",
		Country:   "PK",
		Elevation: 185,
		Latitude:  28.2842006683,
		Longitude: 68.4496994019,
		Timezone:  "Asia/Karachi",
	},
	"OPJI": {
		ICAO:      "OPJI",
		IATA:      "JIW",
		Name:      "Jiwani Airport",
		City:      "Jiwani",
		State:     "Balochistan",
		Country:   "PK",
		Elevation: 186,
		Latitude:  25.0678005219,
		Longitude: 61.8054008484,
		Timezone:  "Asia/Karachi",
	},
	"OPKA": {
		ICAO:      "OPKA",
		Name:      "Cape Monz Airport",
		City:      "Cape Monz",
		State:     "Sindh",
		Country:   "PK",
		Elevation: 40,
		Latitude:  24.8279323578,
		Longitude: 66.6644287109,
		Timezone:  "Asia/Karachi",
	},
	"OPKC": {
		ICAO:      "OPKC",
		IATA:      "KHI",
		Name:      "Jinnah International Airport",
		City:      "Karachi",
		State:     "Sindh",
		Country:   "PK",
		Elevation: 100,
		Latitude:  24.9064998627,
		Longitude: 67.1607971191,
		Timezone:  "Asia/Karachi",
	},
	"OPKD": {
		ICAO:      "OPKD",
		IATA:      "HDD",
		Name:      "Hyderabad Airport",
		City:      "Hyderabad",
		State:     "Sindh",
		Country:   "PK",
		Elevation: 130,
		Latitude:  25.3180999756,
		Longitude: 68.3660964966,
		Timezone:  "Asia/Karachi",
	},
	"OPKE": {
		ICAO:      "OPKE",
		Name:      "Chore Airport",
		City:      "Chore",
		State:     "Sindh",
		Country:   "PK",
		Elevation: 200,
		Latitude:  25.5259990692,
		Longitude: 69.7717971802,
		Timezone:  "Asia/Karachi",
	},
	"OPKF": {
		ICAO:      "OPKF",
		Name:      "Gharo Airport",
		City:      "Gharo",
		State:     "Sindh",
		Country:   "PK",
		Elevation: 20,
		Latitude:  24.7290000916,
		Longitude: 67.587600708,
		Timezone:  "Asia/Karachi",
	},
	"OPKH": {
		ICAO:      "OPKH",
		IATA:      "KDD",
		Name:      "Khuzdar Airport",
		City:      "Khuzdar",
		State:     "Balochistan",
		Country:   "PK",
		Elevation: 4012,
		Latitude:  27.790599823,
		Longitude: 66.6473007202,
		Timezone:  "Asia/Karachi",
	},
	"OPKK": {
		ICAO:      "OPKK",
		Name:      "Korangi Creek Airport",
		City:      "Karachi",
		State:     "Sindh",
		Country:   "PK",
		Elevation: 3,
		Latitude:  24.7842998505,
		Longitude: 67.138999939,
		Timezone:  "Asia/Karachi",
	},
	"OPKL": {
		ICAO:      "OPKL",
		IATA:      "KBH",
		Name:      "Kalat Airport",
		City:      "Kalat",
		State:     "Balochistan",
		Country:   "PK",
		Elevation: 6100,
		Latitude:  29.1333332062,
		Longitude: 66.5166702271,
		Timezone:  "Asia/Karachi",
	},
	"OPKN": {
		ICAO:      "OPKN",
		Name:      "Kharan Airport",
		City:      "Kharan",
		State:     "Balochistan",
		Country:   "PK",
		Elevation: 2408,
		Latitude:  28.5972003937,
		Longitude: 65.4225006104,
		Timezone:  "Asia/Karachi",
	},
	"OPKT": {
		ICAO:      "OPKT",
		IATA:      "OHT",
		Name:      "Kohat Airport",
		City:      "Kohat",
		State:     "Khyber-Pakhtunkhwa",
		Country:   "PK",
		Elevation: 1650,
		Latitude:  33.5699996948,
		Longitude: 71.4400024414,
		Timezone:  "Asia/Karachi",
	},
	"OPLA": {
		ICAO:      "OPLA",
		IATA:      "LHE",
		Name:      "Alama Iqbal International Airport",
		City:      "Lahore",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 712,
		Latitude:  31.5216007233,
		Longitude: 74.4036026001,
		Timezone:  "Asia/Karachi",
	},
	"OPLH": {
		ICAO:      "OPLH",
		Name:      "Walton Airport",
		City:      "Walton",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 679,
		Latitude:  31.4948005676,
		Longitude: 74.3461990356,
		Timezone:  "Asia/Karachi",
	},
	"OPLL": {
		ICAO:      "OPLL",
		IATA:      "LRG",
		Name:      "Loralai Airport",
		City:      "Loralai",
		State:     "Balochistan",
		Country:   "PK",
		Elevation: 4631,
		Latitude:  30.3554992676,
		Longitude: 68.6135025024,
		Timezone:  "Asia/Karachi",
	},
	"OPMA": {
		ICAO:      "OPMA",
		IATA:      "XJM",
		Name:      "Mangla Airport",
		City:      "Mangla",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 902,
		Latitude:  33.0500984192,
		Longitude: 73.6383972168,
		Timezone:  "Asia/Karachi",
	},
	"OPMF": {
		ICAO:      "OPMF",
		IATA:      "MFG",
		Name:      "Muzaffarabad Airport",
		City:      "Muzaffarabad",
		State:     "Azad-Kashmir",
		Country:   "PK",
		Elevation: 2691,
		Latitude:  34.3390007019,
		Longitude: 73.5085983276,
		Timezone:  "Asia/Karachi",
	},
	"OPMI": {
		ICAO:      "OPMI",
		IATA:      "MWD",
		Name:      "Mianwali Air Base",
		City:      "Mianwali",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 690,
		Latitude:  32.5630989075,
		Longitude: 71.5707015991,
		Timezone:  "Asia/Karachi",
	},
	"OPMJ": {
		ICAO:      "OPMJ",
		IATA:      "MJD",
		Name:      "Moenjodaro Airport",
		City:      "Moenjodaro",
		State:     "Sindh",
		Country:   "PK",
		Elevation: 154,
		Latitude:  27.3351993561,
		Longitude: 68.1430969238,
		Timezone:  "Asia/Karachi",
	},
	"OPMK": {
		ICAO:      "OPMK",
		Name:      "Mirpur Khas Air Base",
		City:      "Mirpur Khas",
		State:     "Sindh",
		Country:   "PK",
		Elevation: 60,
		Latitude:  25.6825008392,
		Longitude: 69.0727996826,
		Timezone:  "Asia/Karachi",
	},
	"OPMN": {
		ICAO:      "OPMN",
		Name:      "Miram Shah Airport",
		City:      "Miram Shah",
		State:     "FATA",
		Country:   "PK",
		Elevation: 3022,
		Latitude:  33.0147018433,
		Longitude: 70.0643997192,
		Timezone:  "Asia/Karachi",
	},
	"OPMP": {
		ICAO:      "OPMP",
		IATA:      "MPD",
		Name:      "Sindhri Tharparkar Airport",
		City:      "Sindhri",
		State:     "Sindh",
		Country:   "PK",
		Elevation: 51,
		Latitude:  25.682800293,
		Longitude: 69.0727996826,
		Timezone:  "Asia/Karachi",
	},
	"OPMR": {
		ICAO:      "OPMR",
		Name:      "Masroor Air Base",
		City:      "Karachi",
		State:     "Sindh",
		Country:   "PK",
		Elevation: 35,
		Latitude:  24.8936004639,
		Longitude: 66.9387969971,
		Timezone:  "Asia/Karachi",
	},
	"OPMS": {
		ICAO:      "OPMS",
		Name:      "Minhas Air Base",
		City:      "Kamra",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 1023,
		Latitude:  33.8690986633,
		Longitude: 72.4009017944,
		Timezone:  "Asia/Karachi",
	},
	"OPMT": {
		ICAO:      "OPMT",
		IATA:      "MUX",
		Name:      "Multan International Airport",
		City:      "Multan",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 403,
		Latitude:  30.2031993866,
		Longitude: 71.4190979004,
		Timezone:  "Asia/Karachi",
	},
	"OPNH": {
		ICAO:      "OPNH",
		IATA:      "WNS",
		Name:      "Nawabshah Airport",
		City:      "Nawabash",
		State:     "Sindh",
		Country:   "PK",
		Elevation: 95,
		Latitude:  26.2194004059,
		Longitude: 68.3900985718,
		Timezone:  "Asia/Karachi",
	},
	"OPNK": {
		ICAO:      "OPNK",
		IATA:      "NHS",
		Name:      "Nushki Airport",
		City:      "Nushki",
		State:     "Balochistan",
		Country:   "PK",
		Elevation: 3200,
		Latitude:  29.5389995575,
		Longitude: 66.0233001709,
		Timezone:  "Asia/Karachi",
	},
	"OPOK": {
		ICAO:      "OPOK",
		Name:      "Okara Cantonment Airstrip",
		City:      "Okara",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 568,
		Latitude:  30.7409992218,
		Longitude: 73.3576965332,
		Timezone:  "Asia/Karachi",
	},
	"OPOR": {
		ICAO:      "OPOR",
		IATA:      "ORW",
		Name:      "Ormara Airport",
		City:      "Ormara Raik",
		State:     "Balochistan",
		Country:   "PK",
		Elevation: 10,
		Latitude:  25.2747001648,
		Longitude: 64.5859985352,
		Timezone:  "Asia/Karachi",
	},
	"OPPC": {
		ICAO:      "OPPC",
		IATA:      "PAJ",
		Name:      "Parachinar Airport",
		City:      "Parachinar",
		State:     "FATA",
		Country:   "PK",
		Elevation: 5800,
		Latitude:  33.9020996094,
		Longitude: 70.0716018677,
		Timezone:  "Asia/Karachi",
	},
	"OPPG": {
		ICAO:      "OPPG",
		IATA:      "PJG",
		Name:      "Panjgur Airport",
		City:      "Panjgur",
		State:     "Balochistan",
		Country:   "PK",
		Elevation: 3289,
		Latitude:  26.9545001984,
		Longitude: 64.1324996948,
		Timezone:  "Asia/Karachi",
	},
	"OPPI": {
		ICAO:      "OPPI",
		IATA:      "PSI",
		Name:      "Pasni Airport",
		City:      "Pasni",
		State:     "Balochistan",
		Country:   "PK",
		Elevation: 33,
		Latitude:  25.2905006409,
		Longitude: 63.3451004028,
		Timezone:  "Asia/Karachi",
	},
	"OPPN": {
		ICAO:      "OPPN",
		Name:      "Pishin Airport",
		City:      "Pishin",
		State:     "Balochistan",
		Country:   "PK",
		Elevation: 5075,
		Latitude:  30.5562000275,
		Longitude: 66.985496521,
		Timezone:  "Asia/Karachi",
	},
	"OPPS": {
		ICAO:      "OPPS",
		IATA:      "PEW",
		Name:      "Peshawar International Airport",
		City:      "Peshawar",
		State:     "Khyber-Pakhtunkhwa",
		Country:   "PK",
		Elevation: 1158,
		Latitude:  33.9939002991,
		Longitude: 71.5146026611,
		Timezone:  "Asia/Karachi",
	},
	"OPQS": {
		ICAO:      "OPQS",
		Name:      "Qasim Airport",
		City:      "Qasim",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 1581,
		Latitude:  33.5601997375,
		Longitude: 73.033203125,
		Timezone:  "Asia/Karachi",
	},
	"OPQT": {
		ICAO:      "OPQT",
		IATA:      "UET",
		Name:      "Quetta International Airport",
		City:      "Quetta",
		State:     "Balochistan",
		Country:   "PK",
		Elevation: 5267,
		Latitude:  30.2513999939,
		Longitude: 66.9377975464,
		Timezone:  "Asia/Karachi",
	},
	"OPRK": {
		ICAO:      "OPRK",
		IATA:      "RYK",
		Name:      "Shaikh Zaid Airport",
		City:      "Rahim Yar Khan",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 271,
		Latitude:  28.3838996887,
		Longitude: 70.2796020508,
		Timezone:  "Asia/Karachi",
	},
	"OPRN": {
		ICAO:      "OPRN",
		IATA:      "ISB",
		Name:      "Benazir Bhutto International Airport",
		City:      "Islamabad",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 1668,
		Latitude:  33.6166992188,
		Longitude: 73.0991973877,
		Timezone:  "Asia/Karachi",
	},
	"OPRQ": {
		ICAO:      "OPRQ",
		Name:      "Rafiqui Air Base",
		City:      "Shorkot",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 492,
		Latitude:  30.7581005096,
		Longitude: 72.2825012207,
		Timezone:  "Asia/Karachi",
	},
	"OPRS": {
		ICAO:      "OPRS",
		Name:      "Risalpur Air Base",
		City:      "Risalpur",
		State:     "Khyber-Pakhtunkhwa",
		Country:   "PK",
		Elevation: 1050,
		Latitude:  34.0811004639,
		Longitude: 71.9726028442,
		Timezone:  "Asia/Karachi",
	},
	"OPRT": {
		ICAO:      "OPRT",
		IATA:      "RAZ",
		Name:      "Rawalakot Airport",
		City:      "Rawalakot",
		State:     "Azad-Kashmir",
		Country:   "PK",
		Elevation: 5479,
		Latitude:  33.8497009277,
		Longitude: 73.7981033325,
		Timezone:  "Asia/Karachi",
	},
	"OPSB": {
		ICAO:      "OPSB",
		IATA:      "SBQ",
		Name:      "Sibi Airport",
		City:      "Sibi",
		State:     "Balochistan",
		Country:   "PK",
		Elevation: 436,
		Latitude:  29.5711994171,
		Longitude: 67.8479003906,
		Timezone:  "Asia/Karachi",
	},
	"OPSD": {
		ICAO:      "OPSD",
		IATA:      "KDU",
		Name:      "Skardu Airport",
		City:      "Skardu",
		State:     "Gilgit-Baltistan",
		Country:   "PK",
		Elevation: 7316,
		Latitude:  35.3354988098,
		Longitude: 75.5360031128,
		Timezone:  "Asia/Karachi",
	},
	"OPSF": {
		ICAO:      "OPSF",
		Name:      "Faisal Air Base",
		City:      "Karachi",
		State:     "Sindh",
		Country:   "PK",
		Elevation: 29,
		Latitude:  24.8742008209,
		Longitude: 67.1184997559,
		Timezone:  "Asia/Karachi",
	},
	"OPSK": {
		ICAO:      "OPSK",
		IATA:      "SKZ",
		Name:      "Sukkur Airport",
		City:      "Mirpur Khas",
		State:     "Sindh",
		Country:   "PK",
		Elevation: 196,
		Latitude:  27.7220001221,
		Longitude: 68.7917022705,
		Timezone:  "Asia/Karachi",
	},
	"OPSN": {
		ICAO:      "OPSN",
		IATA:      "SYW",
		Name:      "Sehwan Sharif Airport",
		City:      "Sehwan Sharif",
		State:     "Sindh",
		Country:   "PK",
		Elevation: 121,
		Latitude:  26.4731006622,
		Longitude: 67.7172012329,
		Timezone:  "Asia/Karachi",
	},
	"OPSP": {
		ICAO:      "OPSP",
		Name:      "Shekhupura Airport",
		City:      "Shekhupura",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 680,
		Latitude:  31.7084999084,
		Longitude: 74.0102005005,
		Timezone:  "Asia/Karachi",
	},
	"OPSR": {
		ICAO:      "OPSR",
		IATA:      "SGI",
		Name:      "Mushaf Air Base",
		City:      "Sargodha",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 614,
		Latitude:  32.0485992432,
		Longitude: 72.6650009155,
		Timezone:  "Asia/Karachi",
	},
	"OPSS": {
		ICAO:      "OPSS",
		IATA:      "SDT",
		Name:      "Saidu Sharif Airport",
		City:      "Saidu Sharif",
		State:     "Khyber-Pakhtunkhwa",
		Country:   "PK",
		Elevation: 3183,
		Latitude:  34.8135986328,
		Longitude: 72.3527984619,
		Timezone:  "Asia/Karachi",
	},
	"OPST": {
		ICAO:      "OPST",
		IATA:      "SKT",
		Name:      "Sialkot Airport",
		City:      "Sialkot",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 837,
		Latitude:  32.5355567932,
		Longitude: 74.3638916016,
		Timezone:  "Asia/Karachi",
	},
	"OPSU": {
		ICAO:      "OPSU",
		IATA:      "SUL",
		Name:      "Sui Airport",
		City:      "Sui",
		State:     "Balochistan",
		Country:   "PK",
		Elevation: 763,
		Latitude:  28.6450996399,
		Longitude: 69.176902771,
		Timezone:  "Asia/Karachi",
	},
	"OPSW": {
		ICAO:      "OPSW",
		IATA:      "SWN",
		Name:      "Sahiwal Airport",
		City:      "Sahiwal",
		State:     "Punjab",
		Country:   "PK",
		Elevation: 570,
		Latitude:  31.8894443512,
		Longitude: 72.3916702271,
		Timezone:  "Asia/Karachi",
	},
	"OPTA": {
		ICAO:      "OPTA",
		IATA:      "TLB",
		Name:      "Tarbela Dam Airport",
		City:      "Tarbela",
		State:     "Khyber-Pakhtunkhwa",
		Country:   "PK",
		Elevation: 1114,
		Latitude:  33.9860992432,
		Longitude: 72.6113967896,
		Timezone:  "Asia/Karachi",
	},
	"OPTH": {
		ICAO:      "OPTH",
		IATA:      "BDN",
		Name:      "Talhar Airport",
		City:      "Badin",
		State:     "Sindh",
		Country:   "PK",
		Elevation: 28,
		Latitude:  24.8414993286,
		Longitude: 68.8384017944,
		Timezone:  "Asia/Karachi",
	},
	"OPTT": {
		ICAO:      "OPTT",
		IATA:      "TFT",
		Name:      "Taftan Airport",
		City:      "Taftan",
		State:     "Balochistan",
		Country:   "PK",
		Elevation: 2742,
		Latitude:  28.9641990662,
		Longitude: 61.5954017639,
		Timezone:  "Asia/Karachi",
	},
	"OPTU": {
		ICAO:      "OPTU",
		IATA:      "TUK",
		Name:      "Turbat International Airport",
		City:      "Turbat",
		State:     "Balochistan",
		Country:   "PK",
		Elevation: 498,
		Latitude:  25.9864006042,
		Longitude: 63.0302009583,
		Timezone:  "Asia/Karachi",
	},
	"OPWN": {
		ICAO:      "OPWN",
		IATA:      "WAF",
		Name:      "Wana Airport",
		City:      "Waana",
		State:     "FATA",
		Country:   "PK",
		Elevation: 4550,
		Latitude:  32.3046989441,
		Longitude: 69.5703964233,
		Timezone:  "Asia/Karachi",
	},
	"OPZB": {
		ICAO:      "OPZB",
		IATA:      "PZH",
		Name:      "Zhob Airport",
		City:      "Fort Sandeman",
		State:     "Balochistan",
		Country:   "PK",
		Elevation: 4728,
		Latitude:  31.3584003448,
		Longitude: 69.4636001587,
		Timezone:  "Asia/Karachi",
	},
	"OR00": {
		ICAO:      "OR00",
		Name:      "Flying K Ranch Airport",
		City:      "Beaverton",
		State:     "Oregon",
		Country:   "US",
		Elevation: 350,
		Latitude:  45.4345016479,
		Longitude: -122.8799972534,
		Timezone:  "America/Los_Angeles",
	},
	"OR02": {
		ICAO:      "OR02",
		Name:      "River Run Ranch Airport",
		City:      "Redmond",
		State:     "Oregon",
		Country:   "US",
		Elevation: 2600,
		Latitude:  44.3357009888,
		Longitude: -121.0199966431,
		Timezone:  "America/Los_Angeles",
	},
	"OR04": {
		ICAO:      "OR04",
		Name:      "Kennel Airstrip",
		City:      "Bend",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3595,
		Latitude:  44.0295982361,
		Longitude: -121.1650009155,
		Timezone:  "America/Los_Angeles",
	},
	"OR05": {
		ICAO:      "OR05",
		Name:      "Flying M Airport",
		City:      "Yamhill",
		State:     "Oregon",
		Country:   "US",
		Elevation: 448,
		Latitude:  45.3620986938,
		Longitude: -123.3560028076,
		Timezone:  "America/Los_Angeles",
	},
	"OR06": {
		ICAO:      "OR06",
		Name:      "Snider Creek Airport",
		City:      "Medford",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1430,
		Latitude:  42.5390014648,
		Longitude: -122.922996521,
		Timezone:  "America/Los_Angeles",
	},
	"OR07": {
		ICAO:      "OR07",
		Name:      "Miranda'S Skyranch Airport",
		City:      "Long Creek",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3823,
		Latitude:  44.76240158,
		Longitude: -119.1019974,
		Timezone:  "America/Los_Angeles",
	},
	"OR08": {
		ICAO:      "OR08",
		Name:      "Barton Lake Ranch Airport",
		City:      "Diamond",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4159,
		Latitude:  43.1287994385,
		Longitude: -118.6770019531,
		Timezone:  "America/Los_Angeles",
	},
	"OR09": {
		ICAO:      "OR09",
		Name:      "Whitehorse Ranch Airport",
		City:      "Fields",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4447,
		Latitude:  42.3356018066,
		Longitude: -118.2580032349,
		Timezone:  "America/Los_Angeles",
	},
	"OR10": {
		ICAO:      "OR10",
		Name:      "Roaring Springs Ranch Airport",
		City:      "Frenchglen",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4576,
		Latitude:  42.6496009827,
		Longitude: -118.9919967651,
		Timezone:  "America/Los_Angeles",
	},
	"OR11": {
		ICAO:      "OR11",
		Name:      "Jensens Strip",
		City:      "Haines",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3350,
		Latitude:  44.912399292,
		Longitude: -117.9580001831,
		Timezone:  "America/Los_Angeles",
	},
	"OR12": {
		ICAO:      "OR12",
		Name:      "Oxbow Airport",
		City:      "Homestead",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1806,
		Latitude:  45.0042991638,
		Longitude: -116.8489990234,
		Timezone:  "America/Boise",
	},
	"OR13": {
		ICAO:      "OR13",
		Name:      "Wiley Creek Airport",
		City:      "Dayville",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4130,
		Latitude:  44.5023994446,
		Longitude: -119.3720016479,
		Timezone:  "America/Los_Angeles",
	},
	"OR14": {
		ICAO:      "OR14",
		Name:      "Juntura Airport",
		City:      "Juntura",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3034,
		Latitude:  43.7415008545,
		Longitude: -118.0640029907,
		Timezone:  "America/Boise",
	},
	"OR16": {
		ICAO:      "OR16",
		Name:      "Umpleby Ranch Airport",
		City:      "North Powder",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3412,
		Latitude:  45.0573997498,
		Longitude: -117.9680023193,
		Timezone:  "America/Los_Angeles",
	},
	"OR17": {
		ICAO:      "OR17",
		Name:      "Oxbow Ranch Airport",
		City:      "Prairie City",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3972,
		Latitude:  44.4118003845,
		Longitude: -118.6910018921,
		Timezone:  "America/Los_Angeles",
	},
	"OR19": {
		ICAO:      "OR19",
		Name:      "Propst Airport",
		City:      "Albany",
		State:     "Oregon",
		Country:   "US",
		Elevation: 239,
		Latitude:  44.6386985779,
		Longitude: -123.0230026245,
		Timezone:  "America/Los_Angeles",
	},
	"OR20": {
		ICAO:      "OR20",
		Name:      "Hemmingson Airport",
		City:      "Albany",
		State:     "Oregon",
		Country:   "US",
		Elevation: 250,
		Latitude:  44.6581993103,
		Longitude: -123.1429977417,
		Timezone:  "America/Los_Angeles",
	},
	"OR21": {
		ICAO:      "OR21",
		Name:      "Miller Airstrip",
		City:      "Albany",
		State:     "Oregon",
		Country:   "US",
		Elevation: 279,
		Latitude:  44.5932006836,
		Longitude: -122.9899978638,
		Timezone:  "America/Los_Angeles",
	},
	"OR22": {
		ICAO:      "OR22",
		Name:      "Roppair Airport",
		City:      "Albany",
		State:     "Oregon",
		Country:   "US",
		Elevation: 247,
		Latitude:  44.6193008423,
		Longitude: -123.0319976807,
		Timezone:  "America/Los_Angeles",
	},
	"OR23": {
		ICAO:      "OR23",
		Name:      "Karpens Airport",
		City:      "Astoria",
		State:     "Oregon",
		Country:   "US",
		Elevation: 125,
		Latitude:  46.1613998413,
		Longitude: -123.6399993896,
		Timezone:  "America/Los_Angeles",
	},
	"OR24": {
		ICAO:      "OR24",
		Name:      "Barrett Field",
		City:      "Athena",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1738,
		Latitude:  45.8212013245,
		Longitude: -118.4980010986,
		Timezone:  "America/Los_Angeles",
	},
	"OR25": {
		ICAO:      "OR25",
		Name:      "Flying E Airport",
		City:      "Aumsville",
		State:     "Oregon",
		Country:   "US",
		Elevation: 325,
		Latitude:  44.8447990417,
		Longitude: -122.9010009766,
		Timezone:  "America/Los_Angeles",
	},
	"OR26": {
		ICAO:      "OR26",
		Name:      "Farr Airport",
		City:      "Lakeview",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4775,
		Latitude:  42.1277008057,
		Longitude: -120.4840011597,
		Timezone:  "America/Los_Angeles",
	},
	"OR27": {
		ICAO:      "OR27",
		Name:      "Chadwick Airport",
		City:      "Banks",
		State:     "Oregon",
		Country:   "US",
		Elevation: 670,
		Latitude:  45.6332015991,
		Longitude: -123.1679992676,
		Timezone:  "America/Los_Angeles",
	},
	"OR28": {
		ICAO:      "OR28",
		Name:      "Harvey'S Acres Airport",
		City:      "Beaverton",
		State:     "Oregon",
		Country:   "US",
		Elevation: 250,
		Latitude:  45.44150162,
		Longitude: -122.8929977,
		Timezone:  "America/Los_Angeles",
	},
	"OR29": {
		ICAO:      "OR29",
		Name:      "Gopher Gulch Airport",
		City:      "Bend",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3480,
		Latitude:  44.1111984253,
		Longitude: -121.3339996338,
		Timezone:  "America/Los_Angeles",
	},
	"OR30": {
		ICAO:      "OR30",
		Name:      "D M Stevenson Ranch Airport",
		City:      "Bend",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3800,
		Latitude:  44.003200531,
		Longitude: -121.2720031738,
		Timezone:  "America/Los_Angeles",
	},
	"OR32": {
		ICAO:      "OR32",
		Name:      "Simtag Farms Airport",
		City:      "Boardman",
		State:     "Oregon",
		Country:   "US",
		Elevation: 550,
		Latitude:  45.7521018982,
		Longitude: -119.9459991455,
		Timezone:  "America/Los_Angeles",
	},
	"OR34": {
		ICAO:      "OR34",
		Name:      "Whippet Field",
		City:      "Sisters",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3040,
		Latitude:  44.3222999573,
		Longitude: -121.4329986572,
		Timezone:  "America/Los_Angeles",
	},
	"OR35": {
		ICAO:      "OR35",
		Name:      "Flying K Bar J Ranch Airport",
		City:      "Boring",
		State:     "Oregon",
		Country:   "US",
		Elevation: 635,
		Latitude:  45.4426002502,
		Longitude: -122.3209991455,
		Timezone:  "America/Los_Angeles",
	},
	"OR38": {
		ICAO:      "OR38",
		Name:      "Harchenko Industrial Airport",
		City:      "Brooks",
		State:     "Oregon",
		Country:   "US",
		Elevation: 183,
		Latitude:  45.0775985718,
		Longitude: -122.9290008545,
		Timezone:  "America/Los_Angeles",
	},
	"OR39": {
		ICAO:      "OR39",
		Name:      "Flying Tom Airport",
		City:      "Corvallis",
		State:     "Oregon",
		Country:   "US",
		Elevation: 300,
		Latitude:  44.6576004028,
		Longitude: -123.2119979858,
		Timezone:  "America/Los_Angeles",
	},
	"OR40": {
		ICAO:      "OR40",
		Name:      "Dietz Airpark",
		City:      "Canby",
		State:     "Oregon",
		Country:   "US",
		Elevation: 180,
		Latitude:  45.2556991577,
		Longitude: -122.6510009766,
		Timezone:  "America/Los_Angeles",
	},
	"OR41": {
		ICAO:      "OR41",
		Name:      "Workman Airpark",
		City:      "Canby",
		State:     "Oregon",
		Country:   "US",
		Elevation: 210,
		Latitude:  45.2075996399,
		Longitude: -122.6689987183,
		Timezone:  "America/Los_Angeles",
	},
	"OR42": {
		ICAO:      "OR42",
		Name:      "Norway Airport",
		City:      "Norway",
		State:     "Oregon",
		Country:   "US",
		Elevation: 23,
		Latitude:  43.1026000977,
		Longitude: -124.1610031128,
		Timezone:  "America/Los_Angeles",
	},
	"OR44": {
		ICAO:      "OR44",
		Name:      "Briggs Airport",
		City:      "Coburg",
		State:     "Oregon",
		Country:   "US",
		Elevation: 387,
		Latitude:  44.153301239,
		Longitude: -123.0650024414,
		Timezone:  "America/Los_Angeles",
	},
	"OR45": {
		ICAO:      "OR45",
		Name:      "West Point Airport",
		City:      "Coburg",
		State:     "Oregon",
		Country:   "US",
		Elevation: 378,
		Latitude:  44.197101593,
		Longitude: -123.0559997559,
		Timezone:  "America/Los_Angeles",
	},
	"OR46": {
		ICAO:      "OR46",
		Name:      "Ajax Airport",
		City:      "Condon",
		State:     "Oregon",
		Country:   "US",
		Elevation: 2165,
		Latitude:  45.3832015991,
		Longitude: -120.4680023193,
		Timezone:  "America/Los_Angeles",
	},
	"OR47": {
		ICAO:      "OR47",
		Name:      "Strauch Field",
		City:      "Junction City",
		State:     "Oregon",
		Country:   "US",
		Elevation: 335,
		Latitude:  44.1883010864,
		Longitude: -123.1890029907,
		Timezone:  "America/Los_Angeles",
	},
	"OR48": {
		ICAO:      "OR48",
		Name:      "Snyder Ranch Airport",
		City:      "Condon",
		State:     "Oregon",
		Country:   "US",
		Elevation: 2655,
		Latitude:  45.1943016052,
		Longitude: -120.2170028687,
		Timezone:  "America/Los_Angeles",
	},
	"OR50": {
		ICAO:      "OR50",
		Name:      "Lehman Field",
		City:      "Corbett",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1003,
		Latitude:  45.4856987,
		Longitude: -122.2340011597,
		Timezone:  "America/Los_Angeles",
	},
	"OR51": {
		ICAO:      "OR51",
		Name:      "Gillette Field",
		City:      "Scio",
		State:     "Oregon",
		Country:   "US",
		Elevation: 735,
		Latitude:  44.7196006775,
		Longitude: -122.6999969482,
		Timezone:  "America/Los_Angeles",
	},
	"OR52": {
		ICAO:      "OR52",
		Name:      "Venell Airport",
		City:      "Corvallis",
		State:     "Oregon",
		Country:   "US",
		Elevation: 246,
		Latitude:  44.4734992981,
		Longitude: -123.3030014038,
		Timezone:  "America/Los_Angeles",
	},
	"OR53": {
		ICAO:      "OR53",
		Name:      "Wapinitia Airport",
		City:      "Wapinitia",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1910,
		Latitude:  45.1456985474,
		Longitude: -121.2289962769,
		Timezone:  "America/Los_Angeles",
	},
	"OR54": {
		ICAO:      "OR54",
		Name:      "Winn Airport",
		City:      "Corvallis",
		State:     "Oregon",
		Country:   "US",
		Elevation: 205,
		Latitude:  44.5984992981,
		Longitude: -123.2070007324,
		Timezone:  "America/Los_Angeles",
	},
	"OR56": {
		ICAO:      "OR56",
		Name:      "Holiday Airport",
		City:      "Corvallis",
		State:     "Oregon",
		Country:   "US",
		Elevation: 225,
		Latitude:  44.5698013306,
		Longitude: -123.1709976196,
		Timezone:  "America/Los_Angeles",
	},
	"OR57": {
		ICAO:      "OR57",
		Name:      "Walker Airport",
		City:      "Creswell",
		State:     "Oregon",
		Country:   "US",
		Elevation: 580,
		Latitude:  43.8686981201,
		Longitude: -123.033996582,
		Timezone:  "America/Los_Angeles",
	},
	"OR59": {
		ICAO:      "OR59",
		Name:      "Trivelpiece Airport",
		City:      "Yamhill",
		State:     "Oregon",
		Country:   "US",
		Elevation: 165,
		Latitude:  45.3315010071,
		Longitude: -123.2070007324,
		Timezone:  "America/Los_Angeles",
	},
	"OR60": {
		ICAO:      "OR60",
		Name:      "Fargher Airport",
		City:      "Dufur",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1440,
		Latitude:  45.4365005493,
		Longitude: -121.1660003662,
		Timezone:  "America/Los_Angeles",
	},
	"OR61": {
		ICAO:      "OR61",
		Name:      "Apple Valley Airport",
		City:      "Buxton",
		State:     "Oregon",
		Country:   "US",
		Elevation: 247,
		Latitude:  45.6783981323,
		Longitude: -123.1859970093,
		Timezone:  "America/Los_Angeles",
	},
	"OR64": {
		ICAO:      "OR64",
		Name:      "Beach Ranch Airport",
		City:      "Enterprise",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4000,
		Latitude:  45.9637985229,
		Longitude: -117.2350006104,
		Timezone:  "America/Los_Angeles",
	},
	"OR65": {
		ICAO:      "OR65",
		Name:      "Eagle Nest Ranch Airport",
		City:      "Estacada",
		State:     "Oregon",
		Country:   "US",
		Elevation: 425,
		Latitude:  45.354801178,
		Longitude: -122.3460006714,
		Timezone:  "America/Los_Angeles",
	},
	"OR66": {
		ICAO:      "OR66",
		Name:      "Beaver Oaks Airport",
		City:      "Estacada",
		State:     "Oregon",
		Country:   "US",
		Elevation: 400,
		Latitude:  45.3040008545,
		Longitude: -122.361000061,
		Timezone:  "America/Los_Angeles",
	},
	"OR67": {
		ICAO:      "OR67",
		Name:      "Mc Gill Airport",
		City:      "Estacada",
		State:     "Oregon",
		Country:   "US",
		Elevation: 370,
		Latitude:  45.3372993469,
		Longitude: -122.398002625,
		Timezone:  "America/Los_Angeles",
	},
	"OR70": {
		ICAO:      "OR70",
		Name:      "Pine Valley Airport",
		City:      "Halfway",
		State:     "Oregon",
		Country:   "US",
		Elevation: 2577,
		Latitude:  44.8596000671,
		Longitude: -117.0879974365,
		Timezone:  "America/Los_Angeles",
	},
	"OR72": {
		ICAO:      "OR72",
		Name:      "Krueger Airport",
		City:      "Boring",
		State:     "Oregon",
		Country:   "US",
		Elevation: 620,
		Latitude:  45.4421005249,
		Longitude: -122.3229980469,
		Timezone:  "America/Los_Angeles",
	},
	"OR73": {
		ICAO:      "OR73",
		Name:      "Calvert Peak STOLport",
		City:      "Glendale",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3808,
		Latitude:  42.7787017822,
		Longitude: -123.7340011597,
		Timezone:  "America/Los_Angeles",
	},
	"OR74": {
		ICAO:      "OR74",
		Name:      "Winkle Bar Airport",
		City:      "Grants Pass",
		State:     "Oregon",
		Country:   "US",
		Elevation: 450,
		Latitude:  42.701499939,
		Longitude: -123.8050003052,
		Timezone:  "America/Los_Angeles",
	},
	"OR76": {
		ICAO:      "OR76",
		Name:      "Waynes Air Service Airport",
		City:      "Halsey",
		State:     "Oregon",
		Country:   "US",
		Elevation: 285,
		Latitude:  44.3632011414,
		Longitude: -123.1399993896,
		Timezone:  "America/Los_Angeles",
	},
	"OR77": {
		ICAO:      "OR77",
		Name:      "Faust Field Airport",
		City:      "Independence",
		State:     "Oregon",
		Country:   "US",
		Elevation: 175,
		Latitude:  44.8003997803,
		Longitude: -123.15599823,
		Timezone:  "America/Los_Angeles",
	},
	"OR78": {
		ICAO:      "OR78",
		Name:      "Daniels Field",
		City:      "Harrisburg",
		State:     "Oregon",
		Country:   "US",
		Elevation: 320,
		Latitude:  44.2720985413,
		Longitude: -123.0609970093,
		Timezone:  "America/Los_Angeles",
	},
	"OR79": {
		ICAO:      "OR79",
		Name:      "Knox's Private Airstrip",
		City:      "Harrisburg",
		State:     "Oregon",
		Country:   "US",
		Elevation: 313,
		Latitude:  44.2892990112,
		Longitude: -123.1309967041,
		Timezone:  "America/Los_Angeles",
	},
	"OR81": {
		ICAO:      "OR81",
		Name:      "Olinger Airpark",
		City:      "Hillsboro",
		State:     "Oregon",
		Country:   "US",
		Elevation: 185,
		Latitude:  45.5597991943,
		Longitude: -123.019996643,
		Timezone:  "America/Los_Angeles",
	},
	"OR85": {
		ICAO:      "OR85",
		Name:      "Wigrich Airport",
		City:      "Independence",
		State:     "Oregon",
		Country:   "US",
		Elevation: 165,
		Latitude:  44.8123016357,
		Longitude: -123.1500015259,
		Timezone:  "America/Los_Angeles",
	},
	"OR86": {
		ICAO:      "OR86",
		Name:      "Gilmour Ag Air Airport",
		City:      "Jefferson",
		State:     "Oregon",
		Country:   "US",
		Elevation: 244,
		Latitude:  44.7554016113,
		Longitude: -123.04699707,
		Timezone:  "America/Los_Angeles",
	},
	"OR87": {
		ICAO:      "OR87",
		Name:      "Blue Skies Farm Airport",
		City:      "Salem",
		State:     "Oregon",
		Country:   "US",
		Elevation: 200,
		Latitude:  44.966111,
		Longitude: -122.9225,
		Timezone:  "America/Los_Angeles",
	},
	"OR89": {
		ICAO:      "OR89",
		Name:      "Kinzua Airport",
		City:      "Kinzua",
		State:     "Oregon",
		Country:   "US",
		Elevation: 3966,
		Latitude:  44.9999008179,
		Longitude: -120.0530014038,
		Timezone:  "America/Los_Angeles",
	},
	"OR90": {
		ICAO:      "OR90",
		Name:      "Lafayette Airstrip",
		City:      "Lafayette",
		State:     "Oregon",
		Country:   "US",
		Elevation: 160,
		Latitude:  45.2443008423,
		Longitude: -123.1289978027,
		Timezone:  "America/Los_Angeles",
	},
	"OR92": {
		ICAO:      "OR92",
		Name:      "Billiebob Ultralightport",
		City:      "Grants Pass",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1000,
		Latitude:  42.3914985657,
		Longitude: -123.43699646,
		Timezone:  "America/Los_Angeles",
	},
	"OR94": {
		ICAO:      "OR94",
		Name:      "Showa Airport",
		City:      "Brownsville",
		State:     "Oregon",
		Country:   "US",
		Elevation: 298,
		Latitude:  44.3917007446,
		Longitude: -123.0299987793,
		Timezone:  "America/Los_Angeles",
	},
	"OR95": {
		ICAO:      "OR95",
		Name:      "Vineyard Airport",
		City:      "Mc Coy",
		State:     "Oregon",
		Country:   "US",
		Elevation: 190,
		Latitude:  45.0498008728,
		Longitude: -123.2350006104,
		Timezone:  "America/Los_Angeles",
	},
	"OR96": {
		ICAO:      "OR96",
		Name:      "Beagle Sky Ranch Airport",
		City:      "Medford",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1436,
		Latitude:  42.5390014648,
		Longitude: -122.9039993286,
		Timezone:  "America/Los_Angeles",
	},
	"OR97": {
		ICAO:      "OR97",
		Name:      "Burrill Airport",
		City:      "Medford",
		State:     "Oregon",
		Country:   "US",
		Elevation: 1310,
		Latitude:  42.4387016296,
		Longitude: -122.8639984131,
		Timezone:  "America/Los_Angeles",
	},
	"OR98": {
		ICAO:      "OR98",
		Name:      "Seneca Emergency Airstrip",
		City:      "Seneca",
		State:     "Oregon",
		Country:   "US",
		Elevation: 4666,
		Latitude:  44.1381988525,
		Longitude: -118.9820022583,
		Timezone:  "America/Los_Angeles",
	},
	"ORAA": {
		ICAO:      "ORAA",
		IATA:      "IQA",
		Name:      "Al Asad Air Base",
		City:      "Hit",
		State:     "Anbar",
		Country:   "IQ",
		Elevation: 618,
		Latitude:  33.7855987549,
		Longitude: 42.4412002563,
		Timezone:  "Asia/Baghdad",
	},
	"ORAT": {
		ICAO:      "ORAT",
		IATA:      "TQD",
		Name:      "Al Taqaddum Air Base",
		City:      "Al Habbaniyah",
		State:     "Anbar",
		Country:   "IQ",
		Elevation: 275,
		Latitude:  33.3381004333,
		Longitude: 43.5970993042,
		Timezone:  "Asia/Baghdad",
	},
	"ORBB": {
		ICAO:      "ORBB",
		IATA:      "BMN",
		Name:      "Bamarni Airport",
		City:      "Bamarni",
		State:     "Dahuk",
		Country:   "IQ",
		Elevation: 3455,
		Latitude:  37.0988006592,
		Longitude: 43.2666015625,
		Timezone:  "Asia/Baghdad",
	},
	"ORBD": {
		ICAO:      "ORBD",
		Name:      "Balad Southeast Air Base",
		City:      "Balad",
		State:     "Salah-ad-Din",
		Country:   "IQ",
		Elevation: 161,
		Latitude:  33.9402008057,
		Longitude: 44.3615989685,
		Timezone:  "Asia/Baghdad",
	},
	"ORBI": {
		ICAO:      "ORBI",
		IATA:      "BGW",
		Name:      "Baghdad International Airport",
		City:      "Baghdad",
		State:     "Baghdad",
		Country:   "IQ",
		Elevation: 114,
		Latitude:  33.2625007629,
		Longitude: 44.2346000671,
		Timezone:  "Asia/Baghdad",
	},
	"ORBM": {
		ICAO:      "ORBM",
		IATA:      "OSB",
		Name:      "Mosul International Airport",
		City:      "Mosul",
		State:     "Ninawa",
		Country:   "IQ",
		Elevation: 719,
		Latitude:  36.3058013916,
		Longitude: 43.1473999023,
		Timezone:  "Asia/Baghdad",
	},
	"ORBR": {
		ICAO:      "ORBR",
		Name:      "Bashur Airfield",
		City:      "Bashur",
		State:     "Arbil",
		Country:   "IQ",
		Elevation: 2079,
		Latitude:  36.5336990356,
		Longitude: 44.3399009705,
		Timezone:  "Asia/Baghdad",
	},
	"ORER": {
		ICAO:      "ORER",
		IATA:      "EBL",
		Name:      "Erbil International Airport",
		City:      "Arbil",
		State:     "Arbil",
		Country:   "IQ",
		Elevation: 1341,
		Latitude:  36.2375984192,
		Longitude: 43.9631996155,
		Timezone:  "Asia/Baghdad",
	},
	"ORKK": {
		ICAO:      "ORKK",
		IATA:      "KIK",
		Name:      "Kirkuk Air Base",
		City:      "Kirkuk",
		State:     "Kirkuk",
		Country:   "IQ",
		Elevation: 1061,
		Latitude:  35.4695014954,
		Longitude: 44.3488998413,
		Timezone:  "Asia/Baghdad",
	},
	"ORMM": {
		ICAO:      "ORMM",
		IATA:      "BSR",
		Name:      "Basrah International Airport",
		City:      "Basrah",
		State:     "Basra",
		Country:   "IQ",
		Elevation: 11,
		Latitude:  30.5491008759,
		Longitude: 47.6621017456,
		Timezone:  "Asia/Baghdad",
	},
	"ORNI": {
		ICAO:      "ORNI",
		IATA:      "NJF",
		Name:      "Al Najaf International Airport",
		City:      "Najaf",
		State:     "Najaf Governorate",
		Country:   "IQ",
		Elevation: 103,
		Latitude:  31.989722,
		Longitude: 44.404167,
		Timezone:  "Asia/Baghdad",
	},
	"ORQW": {
		ICAO:      "ORQW",
		IATA:      "RQW",
		Name:      "Qayyarah West Airport",
		City:      "Qayyarah",
		State:     "Ninawa",
		Country:   "IQ",
		Elevation: 749,
		Latitude:  35.76720047,
		Longitude: 43.1250991821,
		Timezone:  "Asia/Baghdad",
	},
	"ORSH": {
		ICAO:      "ORSH",
		Name:      "Al Sahra Army Air Field",
		City:      "Tikrit",
		State:     "Salah-ad-Din",
		Country:   "IQ",
		Elevation: 451,
		Latitude:  34.6731987,
		Longitude: 43.5429992676,
		Timezone:  "Asia/Baghdad",
	},
	"ORSU": {
		ICAO:      "ORSU",
		IATA:      "ISU",
		Name:      "Sulaymaniyah International Airport",
		City:      "Sulaymaniyah",
		State:     "As-Sulaymaniyah",
		Country:   "IQ",
		Elevation: 2494,
		Latitude:  35.5617485046,
		Longitude: 45.3167381287,
		Timezone:  "Asia/Baghdad",
	},
	"ORTF": {
		ICAO:      "ORTF",
		Name:      "Tall Afar Army Air Field",
		City:      "Tall Afar",
		State:     "Ninawa",
		Country:   "IQ",
		Elevation: 996,
		Latitude:  36.2831001282,
		Longitude: 42.4029998779,
		Timezone:  "Asia/Baghdad",
	},
	"ORTI": {
		ICAO:      "ORTI",
		Name:      "Al Taji Army Air Field",
		City:      "Taji",
		State:     "Baghdad",
		Country:   "IQ",
		Elevation: 121,
		Latitude:  33.5238990784,
		Longitude: 44.2566986084,
		Timezone:  "Asia/Baghdad",
	},
	"ORTL": {
		ICAO:      "ORTL",
		Name:      "Ali Air Base",
		City:      "Nasiriyah",
		State:     "Dhi-Qar",
		Country:   "IQ",
		Elevation: 20,
		Latitude:  30.9358005524,
		Longitude: 46.0900993347,
		Timezone:  "Asia/Baghdad",
	},
	"ORUB": {
		ICAO:      "ORUB",
		Name:      "Ubaydah Bin Al Jarrah Airport",
		City:      "Al Cut",
		State:     "Wasiţ",
		Country:   "IQ",
		Elevation: 68,
		Latitude:  32.4819984436,
		Longitude: 45.7570991516,
		Timezone:  "Asia/Baghdad",
	},
	"ORUQ": {
		ICAO:      "ORUQ",
		Name:      "Umm Qasr Airport",
		State:     "Basra",
		Country:   "IQ",
		Latitude:  30.0181435212,
		Longitude: 47.9246163368,
		Timezone:  "Asia/Baghdad",
	},
	"OS58": {
		ICAO:      "OS58",
		Name:      "Hamah Air Base",
		State:     "Hama",
		Country:   "SY",
		Elevation: 1014,
		Latitude:  35.1181983948,
		Longitude: 36.7112007141,
		Timezone:  "Asia/Damascus",
	},
	"OS59": {
		ICAO:      "OS59",
		Name:      "Tabqa Airport",
		State:     "Ar-Raqqah",
		Country:   "SY",
		Elevation: 1050,
		Latitude:  35.754699707,
		Longitude: 38.5667991638,
		Timezone:  "Asia/Damascus",
	},
	"OS61": {
		ICAO:      "OS61",
		Name:      "Dumayr Air Base",
		State:     "Rif-dimashq",
		Country:   "SY",
		Elevation: 2060,
		Latitude:  33.6096000671,
		Longitude: 36.7490005493,
		Timezone:  "Asia/Damascus",
	},
	"OS62": {
		ICAO:      "OS62",
		Name:      "Jirah Air Base",
		State:     "Aleppo",
		Country:   "SY",
		Elevation: 1145,
		Latitude:  36.096698761,
		Longitude: 37.9365005493,
		Timezone:  "Asia/Damascus",
	},
	"OS63": {
		ICAO:      "OS63",
		Name:      "Marj Ruhayyil Air Base",
		City:      "Marj Ruhayyil",
		State:     "Rif-dimashq",
		Country:   "SY",
		Elevation: 2190,
		Latitude:  33.2864990234,
		Longitude: 36.4574012756,
		Timezone:  "Asia/Damascus",
	},
	"OS65": {
		ICAO:      "OS65",
		Name:      "Shayrat Air Base",
		City:      "Shayrat",
		State:     "Homs",
		Country:   "SY",
		Elevation: 2644,
		Latitude:  34.4902992249,
		Longitude: 36.9090003967,
		Timezone:  "Asia/Damascus",
	},
	"OS66": {
		ICAO:      "OS66",
		Name:      "Rasin El Aboud Airport",
		State:     "Aleppo",
		Country:   "SY",
		Elevation: 1207,
		Latitude:  36.1870994568,
		Longitude: 37.5831985474,
		Timezone:  "Asia/Damascus",
	},
	"OS67": {
		ICAO:      "OS67",
		Name:      "Mezze Airport",
		State:     "Dimashq",
		Country:   "SY",
		Elevation: 2407,
		Latitude:  33.4775009155,
		Longitude: 36.223400116,
		Timezone:  "Asia/Damascus",
	},
	"OS68": {
		ICAO:      "OS68",
		Name:      "Saiqal Air Base",
		City:      "Saiqal",
		State:     "Rif-dimashq",
		Country:   "SY",
		Elevation: 2300,
		Latitude:  33.6823997498,
		Longitude: 37.2139015198,
		Timezone:  "Asia/Damascus",
	},
	"OS73": {
		ICAO:      "OS73",
		Name:      "Al Al Airport",
		State:     "Quneitra",
		Country:   "SY",
		Elevation: 1214,
		Latitude:  32.7867012024,
		Longitude: 35.7179985046,
		Timezone:  "Asia/Jerusalem",
	},
	"OSAP": {
		ICAO:      "OSAP",
		IATA:      "ALP",
		Name:      "Aleppo International Airport",
		City:      "Aleppo",
		State:     "Aleppo",
		Country:   "SY",
		Elevation: 1276,
		Latitude:  36.1806983948,
		Longitude: 37.2243995667,
		Timezone:  "Asia/Damascus",
	},
	"OSDI": {
		ICAO:      "OSDI",
		IATA:      "DAM",
		Name:      "Damascus International Airport",
		City:      "Damascus",
		State:     "Rif-dimashq",
		Country:   "SY",
		Elevation: 2020,
		Latitude:  33.4114990234,
		Longitude: 36.5155982971,
		Timezone:  "Asia/Damascus",
	},
	"OSDZ": {
		ICAO:      "OSDZ",
		IATA:      "DEZ",
		Name:      "Deir ez-Zor Airport",
		City:      "Deir ez-Zor",
		State:     "Deir-ez-Zor",
		Country:   "SY",
		Elevation: 700,
		Latitude:  35.2854003906,
		Longitude: 40.1759986877,
		Timezone:  "Asia/Damascus",
	},
	"OSKL": {
		ICAO:      "OSKL",
		IATA:      "KAC",
		Name:      "Kamishly Airport",
		City:      "Kamishly",
		State:     "Al-Hasakah",
		Country:   "SY",
		Elevation: 1480,
		Latitude:  37.0205993652,
		Longitude: 41.1913986206,
		Timezone:  "Asia/Damascus",
	},
	"OSLK": {
		ICAO:      "OSLK",
		IATA:      "LTK",
		Name:      "Bassel Al-Assad International Airport",
		City:      "Latakia",
		State:     "Latakia",
		Country:   "SY",
		Elevation: 157,
		Latitude:  35.4011001587,
		Longitude: 35.9486999512,
		Timezone:  "Asia/Damascus",
	},
	"OSPR": {
		ICAO:      "OSPR",
		IATA:      "PMS",
		Name:      "Palmyra Airport",
		State:     "Homs",
		Country:   "SY",
		Elevation: 1322,
		Latitude:  34.5573997498,
		Longitude: 38.3168983459,
		Timezone:  "Asia/Damascus",
	},
	"OTBD": {
		ICAO:      "OTBD",
		IATA:      "DIA",
		Name:      "Doha International Airport",
		City:      "Doha",
		State:     "Baladiyat-ad-Dawḩah",
		Country:   "QA",
		Elevation: 35,
		Latitude:  25.261100769,
		Longitude: 51.5651016235,
		Timezone:  "Asia/Qatar",
	},
	"OTBH": {
		ICAO:      "OTBH",
		IATA:      "IUD",
		Name:      "Al Udeid Air Base",
		City:      "Ar Rayyan",
		State:     "Baladiyat-ar-Rayyan",
		Country:   "QA",
		Elevation: 130,
		Latitude:  25.1173000336,
		Longitude: 51.3149986267,
		Timezone:  "Asia/Qatar",
	},
	"OTBK": {
		ICAO:      "OTBK",
		Name:      "Al Khawr Airport",
		City:      "Al Khawr",
		State:     "Al-Khawr",
		Country:   "QA",
		Elevation: 10,
		Latitude:  25.6296005249,
		Longitude: 51.5066986084,
		Timezone:  "Asia/Qatar",
	},
	"OTHH": {
		ICAO:      "OTHH",
		IATA:      "DOH",
		Name:      "Hamad International Airport",
		City:      "Doha",
		State:     "Baladiyat-ad-Dawḩah",
		Country:   "QA",
		Elevation: 13,
		Latitude:  25.2605946,
		Longitude: 51.6137665,
		Timezone:  "Asia/Qatar",
	},
	"OY75": {
		ICAO:      "OY75",
		Name:      "Canoxy Airport",
		State:     "Muḩafaz̧at-Ḩaḑramawt",
		Country:   "YE",
		Elevation: 3300,
		Latitude:  15.6017999649,
		Longitude: 49.0806999207,
		Timezone:  "Asia/Aden",
	},
	"OYAA": {
		ICAO:      "OYAA",
		IATA:      "ADE",
		Name:      "Aden International Airport",
		City:      "Aden",
		State:     "Aden",
		Country:   "YE",
		Elevation: 7,
		Latitude:  12.8295001984,
		Longitude: 45.0288009644,
		Timezone:  "Asia/Aden",
	},
	"OYAB": {
		ICAO:      "OYAB",
		Name:      "Ibb Airport",
		State:     "Sanaa",
		Country:   "YE",
		Latitude:  16.010753477,
		Longitude: 43.1781148911,
		Timezone:  "Asia/Aden",
	},
	"OYAT": {
		ICAO:      "OYAT",
		IATA:      "AXK",
		Name:      "Ataq Airport",
		State:     "Shabwah",
		Country:   "YE",
		Elevation: 3735,
		Latitude:  14.5513000488,
		Longitude: 46.8261985779,
		Timezone:  "Asia/Aden",
	},
	"OYBA": {
		ICAO:      "OYBA",
		Name:      "Al Badie Airport",
		Country:   "YE",
		Elevation: 908,
		Latitude:  18.7192993164,
		Longitude: 50.8368988037,
		Timezone:  "Asia/Aden",
	},
	"OYBI": {
		ICAO:      "OYBI",
		IATA:      "BYD",
		Name:      "Al-Bayda Airport",
		City:      "Al-Bayda",
		State:     "Al-Bayda-Governorate",
		Country:   "YE",
		Elevation: 6120,
		Latitude:  13.9670000076,
		Longitude: 45.5830001831,
		Timezone:  "Asia/Aden",
	},
	"OYBN": {
		ICAO:      "OYBN",
		IATA:      "BHN",
		Name:      "Beihan Airport",
		State:     "Shabwah",
		Country:   "YE",
		Elevation: 3800,
		Latitude:  14.781999588,
		Longitude: 45.7201004028,
		Timezone:  "Asia/Aden",
	},
	"OYBQ": {
		ICAO:      "OYBQ",
		Name:      "Al-Bough Airport",
		City:      "Al-Bough",
		State:     "Omran",
		Country:   "YE",
		Elevation: 3800,
		Latitude:  17.3467006683,
		Longitude: 44.6217002869,
		Timezone:  "Asia/Aden",
	},
	"OYGD": {
		ICAO:      "OYGD",
		IATA:      "AAY",
		Name:      "Al Ghaidah International Airport",
		State:     "Al-Mahrah",
		Country:   "YE",
		Elevation: 134,
		Latitude:  16.1916999817,
		Longitude: 52.1749992371,
		Timezone:  "Asia/Aden",
	},
	"OYHD": {
		ICAO:      "OYHD",
		IATA:      "HOD",
		Name:      "Hodeidah International Airport",
		City:      "Hodeida",
		State:     "Al-Hudaydah",
		Country:   "YE",
		Elevation: 41,
		Latitude:  14.7530002594,
		Longitude: 42.9762992859,
		Timezone:  "Asia/Aden",
	},
	"OYKM": {
		ICAO:      "OYKM",
		Name:      "Kamaran Airport",
		City:      "Kamaran",
		State:     "Al-Hudaydah",
		Country:   "YE",
		Elevation: 51,
		Latitude:  15.3633003235,
		Longitude: 42.6049995422,
		Timezone:  "Asia/Aden",
	},
	"OYMB": {
		ICAO:      "OYMB",
		IATA:      "MYN",
		Name:      "Mareb Airport",
		City:      "Mareb",
		State:     "Ma’rib",
		Country:   "YE",
		Elevation: 3300,
		Latitude:  15.4692001343,
		Longitude: 45.3269004822,
		Timezone:  "Asia/Aden",
	},
	"OYQN": {
		ICAO:      "OYQN",
		IATA:      "IHN",
		Name:      "Qishn Airport",
		City:      "Qishn",
		State:     "Al-Mahrah",
		Country:   "YE",
		Elevation: 98,
		Latitude:  15.4169998169,
		Longitude: 51.6829986572,
		Timezone:  "Asia/Aden",
	},
	"OYRN": {
		ICAO:      "OYRN",
		IATA:      "RIY",
		Name:      "Mukalla International Airport",
		City:      "Riyan",
		State:     "Muḩafaz̧at-Ḩaḑramawt",
		Country:   "YE",
		Elevation: 54,
		Latitude:  14.6625995636,
		Longitude: 49.375,
		Timezone:  "Asia/Aden",
	},
	"OYRT": {
		ICAO:      "OYRT",
		Name:      "Barat Airport",
		City:      "Barat",
		State:     "Al-Jawf",
		Country:   "YE",
		Elevation: 6800,
		Latitude:  16.7329998016,
		Longitude: 44.3499984741,
		Timezone:  "Asia/Aden",
	},
	"OYSF": {
		ICAO:      "OYSF",
		Name:      "As Salif Airport",
		City:      "As Salif",
		State:     "Al-Hudaydah",
		Country:   "YE",
		Latitude:  15.3000001907,
		Longitude: 42.6829986572,
		Timezone:  "Asia/Aden",
	},
	"OYSH": {
		ICAO:      "OYSH",
		IATA:      "SYE",
		Name:      "Sadah Airport",
		City:      "Sadah",
		State:     "Sa‘dah",
		Country:   "YE",
		Elevation: 5938,
		Latitude:  16.966999054,
		Longitude: 43.733001709,
		Timezone:  "Asia/Aden",
	},
	"OYSN": {
		ICAO:      "OYSN",
		IATA:      "SAH",
		Name:      "Sana'a International Airport",
		City:      "Sana'a",
		State:     "Amanat-Al-Asimah",
		Country:   "YE",
		Elevation: 7216,
		Latitude:  15.4763002396,
		Longitude: 44.2196998596,
		Timezone:  "Asia/Aden",
	},
	"OYSQ": {
		ICAO:      "OYSQ",
		IATA:      "SCT",
		Name:      "Socotra International Airport",
		City:      "Socotra Islands",
		State:     "Soqatra",
		Country:   "YE",
		Elevation: 146,
		Latitude:  12.6307001114,
		Longitude: 53.9057998657,
		Timezone:  "Asia/Aden",
	},
	"OYSY": {
		ICAO:      "OYSY",
		IATA:      "GXF",
		Name:      "Sayun International Airport",
		City:      "Sayun",
		State:     "Muḩafaz̧at-Ḩaḑramawt",
		Country:   "YE",
		Elevation: 2097,
		Latitude:  15.9660997391,
		Longitude: 48.7882995605,
		Timezone:  "Asia/Aden",
	},
	"OYTZ": {
		ICAO:      "OYTZ",
		IATA:      "TAI",
		Name:      "Ta'izz International Airport",
		City:      "Ta'izz",
		State:     "Ta‘izz",
		Country:   "YE",
		Elevation: 4838,
		Latitude:  13.6859998703,
		Longitude: 44.1390991211,
		Timezone:  "Asia/Aden",
	},
	"OYZM": {
		ICAO:      "OYZM",
		Name:      "Al-Hazm Airport",
		City:      "Al-Hazm",
		State:     "Al-Jawf",
		Country:   "YE",
		Elevation: 3200,
		Latitude:  16.2000007629,
		Longitude: 44.783000946,
		Timezone:  "Asia/Aden",
	},
	"PA01": {
		ICAO:      "PA01",
		Name:      "Merrys Pymatuning Airport",
		City:      "Linesville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1203,
		Latitude:  41.6750984192,
		Longitude: -80.4345016479,
		Timezone:  "America/New_York",
	},
	"PA02": {
		ICAO:      "PA02",
		Name:      "Dillen Personal Airport",
		City:      "Mc Kean",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1028,
		Latitude:  41.9833984375,
		Longitude: -80.1830978394,
		Timezone:  "America/New_York",
	},
	"PA05": {
		ICAO:      "PA05",
		Name:      "Monesmith Airport",
		City:      "Point Pleasant",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 390,
		Latitude:  40.4412002563,
		Longitude: -75.0748977661,
		Timezone:  "America/New_York",
	},
	"PA07": {
		ICAO:      "PA07",
		Name:      "Bilinski Airport",
		City:      "Middleboro",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1080,
		Latitude:  42.0144996643,
		Longitude: -80.1201019287,
		Timezone:  "America/New_York",
	},
	"PA08": {
		ICAO:      "PA08",
		Name:      "Jj & Pk Airport",
		City:      "Greencastle",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 542,
		Latitude:  39.7933998108,
		Longitude: -77.7780990601,
		Timezone:  "America/New_York",
	},
	"PA17": {
		ICAO:      "PA17",
		Name:      "Canaan's Field",
		City:      "Lawrenceville",
		State:     "New-York",
		Country:   "US",
		Elevation: 1600,
		Latitude:  42.0037002563,
		Longitude: -77.0999984741,
		Timezone:  "America/New_York",
	},
	"PA18": {
		ICAO:      "PA18",
		Name:      "Buzzards Field",
		City:      "Mill Hall",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 760,
		Latitude:  41.0942001343,
		Longitude: -77.4143981934,
		Timezone:  "America/New_York",
	},
	"PA19": {
		ICAO:      "PA19",
		Name:      "Strizki Ultralightport",
		City:      "Elkdale",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1260,
		Latitude:  41.6972999573,
		Longitude: -75.5949020386,
		Timezone:  "America/New_York",
	},
	"PA20": {
		ICAO:      "PA20",
		Name:      "Thermal G. Ranch Gliderport",
		City:      "Erie",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1430,
		Latitude:  42.0205993652,
		Longitude: -80.070602417,
		Timezone:  "America/New_York",
	},
	"PA21": {
		ICAO:      "PA21",
		Name:      "West Middlesex Airport",
		City:      "West Middlesex",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1100,
		Latitude:  41.1652984619,
		Longitude: -80.5026016235,
		Timezone:  "America/New_York",
	},
	"PA22": {
		ICAO:      "PA22",
		Name:      "Hermitage Airport",
		City:      "Sharon",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1100,
		Latitude:  41.2014007568,
		Longitude: -80.4223022461,
		Timezone:  "America/New_York",
	},
	"PA23": {
		ICAO:      "PA23",
		Name:      "Kingsdale Air Park",
		City:      "Littlestown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 595,
		Latitude:  39.7270011902,
		Longitude: -77.1119003296,
		Timezone:  "America/New_York",
	},
	"PA25": {
		ICAO:      "PA25",
		Name:      "Cuatros Vientos Airport",
		City:      "Quaker City",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 500,
		Latitude:  40.6593017578,
		Longitude: -75.8738021851,
		Timezone:  "America/New_York",
	},
	"PA30": {
		ICAO:      "PA30",
		Name:      "East Penn Airport",
		City:      "Andreas",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 885,
		Latitude:  40.7667999268,
		Longitude: -75.774597168,
		Timezone:  "America/New_York",
	},
	"PA31": {
		ICAO:      "PA31",
		Name:      "Rover Airport",
		City:      "Annville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 500,
		Latitude:  40.2834014893,
		Longitude: -76.5393981934,
		Timezone:  "America/New_York",
	},
	"PA33": {
		ICAO:      "PA33",
		Name:      "Lars/Private Airport",
		City:      "Athens",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1300,
		Latitude:  41.9500999451,
		Longitude: -76.483001709,
		Timezone:  "America/New_York",
	},
	"PA35": {
		ICAO:      "PA35",
		Name:      "Bally Spring Farm Airport",
		City:      "Bally",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 660,
		Latitude:  40.4319992065,
		Longitude: -75.6259994507,
		Timezone:  "America/New_York",
	},
	"PA36": {
		ICAO:      "PA36",
		Name:      "Stefanik Airport",
		City:      "Bedminster",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 520,
		Latitude:  40.4528999329,
		Longitude: -75.2037963867,
		Timezone:  "America/New_York",
	},
	"PA39": {
		ICAO:      "PA39",
		Name:      "Beaver Springs Airport",
		City:      "Benfer",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 642,
		Latitude:  40.7765007019,
		Longitude: -77.2164001465,
		Timezone:  "America/New_York",
	},
	"PA40": {
		ICAO:      "PA40",
		Name:      "Benton Airport",
		City:      "Benton",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 780,
		Latitude:  41.2000999451,
		Longitude: -76.3871994019,
		Timezone:  "America/New_York",
	},
	"PA43": {
		ICAO:      "PA43",
		Name:      "Arnold Airport",
		City:      "Birdsboro",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 680,
		Latitude:  40.2667999268,
		Longitude: -75.8830032349,
		Timezone:  "America/New_York",
	},
	"PA44": {
		ICAO:      "PA44",
		Name:      "Stone Castle Motel Airport",
		City:      "Bloomsburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 585,
		Latitude:  40.9735984802,
		Longitude: -76.5186004639,
		Timezone:  "America/New_York",
	},
	"PA45": {
		ICAO:      "PA45",
		Name:      "Logue Airport",
		City:      "Bodines",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1320,
		Latitude:  41.4166984558,
		Longitude: -76.9329986572,
		Timezone:  "America/New_York",
	},
	"PA49": {
		ICAO:      "PA49",
		Name:      "Mountain Bay Air Park Inc Airport",
		City:      "Hawley",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1500,
		Latitude:  41.3801002502,
		Longitude: -75.2323989868,
		Timezone:  "America/New_York",
	},
	"PA51": {
		ICAO:      "PA51",
		Name:      "Bowtie Airport",
		City:      "Biglerville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 560,
		Latitude:  39.9085006714,
		Longitude: -77.2052001953,
		Timezone:  "America/New_York",
	},
	"PA52": {
		ICAO:      "PA52",
		Name:      "Oberlander Airport",
		City:      "Union City",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1626,
		Latitude:  41.859333,
		Longitude: -79.907167,
		Timezone:  "America/New_York",
	},
	"PA53": {
		ICAO:      "PA53",
		Name:      "Cosklos Elkview Airport",
		City:      "Carbondale",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1710,
		Latitude:  41.6269989014,
		Longitude: -75.5281982422,
		Timezone:  "America/New_York",
	},
	"PA54": {
		ICAO:      "PA54",
		Name:      "Kellachows Airport",
		City:      "Carbondale",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1430,
		Latitude:  41.5223007202,
		Longitude: -75.3960037231,
		Timezone:  "America/New_York",
	},
	"PA55": {
		ICAO:      "PA55",
		Name:      "Neiderer Airport",
		City:      "Carlisle",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 440,
		Latitude:  40.2168006897,
		Longitude: -77.1164016724,
		Timezone:  "America/New_York",
	},
	"PA58": {
		ICAO:      "PA58",
		Name:      "Rocktop Airport",
		City:      "Chambersburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 770,
		Latitude:  39.9749984741,
		Longitude: -77.557800293,
		Timezone:  "America/New_York",
	},
	"PA61": {
		ICAO:      "PA61",
		Name:      "Kunda Airport",
		City:      "Collegeville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 252,
		Latitude:  40.1668014526,
		Longitude: -75.4496002197,
		Timezone:  "America/New_York",
	},
	"PA65": {
		ICAO:      "PA65",
		Name:      "Hi-Vu Airport",
		City:      "Coplay",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 760,
		Latitude:  40.7000999451,
		Longitude: -75.5495986938,
		Timezone:  "America/New_York",
	},
	"PA66": {
		ICAO:      "PA66",
		Name:      "Cumberland Valley Airstrip",
		City:      "Greencastle",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 550,
		Latitude:  39.8167991638,
		Longitude: -77.7260971069,
		Timezone:  "America/New_York",
	},
	"PA68": {
		ICAO:      "PA68",
		Name:      "Bugs Airport",
		City:      "Nazareth",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 430,
		Latitude:  40.7142982483,
		Longitude: -75.3042984009,
		Timezone:  "America/New_York",
	},
	"PA69": {
		ICAO:      "PA69",
		Name:      "Vogelsong Airport",
		City:      "Dillsburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 590,
		Latitude:  40.1287002563,
		Longitude: -76.9693984985,
		Timezone:  "America/New_York",
	},
	"PA70": {
		ICAO:      "PA70",
		Name:      "Fetters Construction Airport",
		City:      "Downingtown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 500,
		Latitude:  40.0918006897,
		Longitude: -75.7108001709,
		Timezone:  "America/New_York",
	},
	"PA71": {
		ICAO:      "PA71",
		Name:      "Dwight's Delight Airport",
		City:      "Dushore",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1717,
		Latitude:  41.5209007263,
		Longitude: -76.4621963501,
		Timezone:  "America/New_York",
	},
	"PA72": {
		ICAO:      "PA72",
		Name:      "Circle W Airfield",
		City:      "East Berlin",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 460,
		Latitude:  39.9586982727,
		Longitude: -76.9708023071,
		Timezone:  "America/New_York",
	},
	"PA73": {
		ICAO:      "PA73",
		Name:      "D.Evans Farm Airport",
		City:      "East Prospect",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 525,
		Latitude:  39.9794998169,
		Longitude: -76.5361022949,
		Timezone:  "America/New_York",
	},
	"PA74": {
		ICAO:      "PA74",
		Name:      "Gusler Airport",
		City:      "Elizabethville/Halifax",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 700,
		Latitude:  40.4654006958,
		Longitude: -76.8399963379,
		Timezone:  "America/New_York",
	},
	"PA75": {
		ICAO:      "PA75",
		Name:      "Baker Airport",
		City:      "Elkland",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1800,
		Latitude:  41.9650993347,
		Longitude: -77.3158035278,
		Timezone:  "America/New_York",
	},
	"PA76": {
		ICAO:      "PA76",
		Name:      "Rosenzweig Airport",
		City:      "Wilkes Barre",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1930,
		Latitude:  41.1370010376,
		Longitude: -75.8610992432,
		Timezone:  "America/New_York",
	},
	"PA77": {
		ICAO:      "PA77",
		Name:      "Erwinna Private Airport",
		City:      "Erwinna",
		State:     "New-Jersey",
		Country:   "US",
		Elevation: 350,
		Latitude:  40.5000991821,
		Longitude: -75.0662994385,
		Timezone:  "America/New_York",
	},
	"PA78": {
		ICAO:      "PA78",
		Name:      "Tintinhull Airport",
		City:      "Erwinna",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 350,
		Latitude:  40.5000991821,
		Longitude: -75.082901001,
		Timezone:  "America/New_York",
	},
	"PA81": {
		ICAO:      "PA81",
		Name:      "5 Lakes Airport",
		City:      "Fayetteville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 819,
		Latitude:  39.9127998352,
		Longitude: -77.5327987671,
		Timezone:  "America/New_York",
	},
	"PA82": {
		ICAO:      "PA82",
		Name:      "Grayce Farms Airport",
		City:      "Fleetville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1300,
		Latitude:  41.6001014709,
		Longitude: -75.6829986572,
		Timezone:  "America/New_York",
	},
	"PA84": {
		ICAO:      "PA84",
		Name:      "Level Acres Farm Airport",
		City:      "Fordville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 523,
		Latitude:  39.9542999268,
		Longitude: -76.9094009399,
		Timezone:  "America/New_York",
	},
	"PA85": {
		ICAO:      "PA85",
		Name:      "Drewniany-Springmeadow Airport",
		City:      "Frederick",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 330,
		Latitude:  40.3047981262,
		Longitude: -75.5335006714,
		Timezone:  "America/New_York",
	},
	"PA86": {
		ICAO:      "PA86",
		Name:      "Krill Personal Use Airport",
		City:      "Frystown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 520,
		Latitude:  40.4476013184,
		Longitude: -76.3411026001,
		Timezone:  "America/New_York",
	},
	"PA88": {
		ICAO:      "PA88",
		Name:      "Nemacolin Airport",
		City:      "Farmington",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 2010,
		Latitude:  39.8055992126,
		Longitude: -79.5488967896,
		Timezone:  "America/New_York",
	},
	"PA89": {
		ICAO:      "PA89",
		Name:      "Ben's Landing Ultralightport",
		City:      "East Berlin",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 550,
		Latitude:  39.9589996338,
		Longitude: -77.049697876,
		Timezone:  "America/New_York",
	},
	"PA92": {
		ICAO:      "PA92",
		Name:      "Blue Mountain Academy (Private) Airport",
		City:      "Hamburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 531,
		Latitude:  40.552898407,
		Longitude: -76.0288009644,
		Timezone:  "America/New_York",
	},
	"PAAK": {
		ICAO:      "PAAK",
		IATA:      "AKB",
		Name:      "Atka Airport",
		City:      "Atka",
		State:     "Alaska",
		Country:   "US",
		Elevation: 57,
		Latitude:  52.22029877,
		Longitude: -174.2059937,
		Timezone:  "America/Adak",
	},
	"PAAL": {
		ICAO:      "PAAL",
		IATA:      "PML",
		Name:      "Port Moller Airport",
		City:      "Cold Bay",
		State:     "Alaska",
		Country:   "US",
		Elevation: 20,
		Latitude:  56.0060005188,
		Longitude: -160.561004639,
		Timezone:  "America/Anchorage",
	},
	"PAAM": {
		ICAO:      "PAAM",
		Name:      "Driftwood Bay Air Force Station Airport",
		City:      "Dutch Harbor",
		State:     "Alaska",
		Country:   "US",
		Elevation: 24,
		Latitude:  53.9729499817,
		Longitude: -166.858459473,
		Timezone:  "America/Nome",
	},
	"PAAN": {
		ICAO:      "PAAN",
		Name:      "Gold King Creek Airport",
		City:      "Fairbanks",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1720,
		Latitude:  64.1989974976,
		Longitude: -147.9279937744,
		Timezone:  "America/Anchorage",
	},
	"PAAQ": {
		ICAO:      "PAAQ",
		IATA:      "PAQ",
		Name:      "Palmer Municipal Airport",
		City:      "Palmer",
		State:     "Alaska",
		Country:   "US",
		Elevation: 242,
		Latitude:  61.5948982239,
		Longitude: -149.0890045166,
		Timezone:  "America/Anchorage",
	},
	"PABA": {
		ICAO:      "PABA",
		IATA:      "BTI",
		Name:      "Barter Island LRRS Airport",
		City:      "Barter Island Lrrs",
		State:     "Alaska",
		Country:   "US",
		Elevation: 2,
		Latitude:  70.1340026855,
		Longitude: -143.582000732,
		Timezone:  "America/Anchorage",
	},
	"PABE": {
		ICAO:      "PABE",
		IATA:      "BET",
		Name:      "Bethel Airport",
		City:      "Bethel",
		State:     "Alaska",
		Country:   "US",
		Elevation: 126,
		Latitude:  60.77980042,
		Longitude: -161.8379974,
		Timezone:  "America/Anchorage",
	},
	"PABG": {
		ICAO:      "PABG",
		IATA:      "BVU",
		Name:      "Beluga Airport",
		City:      "Beluga",
		State:     "Alaska",
		Country:   "US",
		Elevation: 130,
		Latitude:  61.1721992493,
		Longitude: -151.044006348,
		Timezone:  "America/Anchorage",
	},
	"PABI": {
		ICAO:      "PABI",
		IATA:      "BIG",
		Name:      "Allen Army Airfield",
		City:      "Delta Junction Ft Greely",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1291,
		Latitude:  63.9944992065,
		Longitude: -145.722000122,
		Timezone:  "America/Anchorage",
	},
	"PABL": {
		ICAO:      "PABL",
		IATA:      "BKC",
		Name:      "Buckland Airport",
		City:      "Buckland",
		State:     "Alaska",
		Country:   "US",
		Elevation: 31,
		Latitude:  65.9815979004,
		Longitude: -161.149002075,
		Timezone:  "America/Anchorage",
	},
	"PABM": {
		ICAO:      "PABM",
		IATA:      "BMX",
		Name:      "Big Mountain Airport",
		City:      "Big Mountain",
		State:     "Alaska",
		Country:   "US",
		Elevation: 663,
		Latitude:  59.3611984253,
		Longitude: -155.259002686,
		Timezone:  "America/Anchorage",
	},
	"PABN": {
		ICAO:      "PABN",
		Name:      "Devils Mountain Lodge Airport",
		City:      "Nabesna",
		State:     "Alaska",
		Country:   "US",
		Elevation: 2880,
		Latitude:  62.4017982483,
		Longitude: -142.994995117,
		Timezone:  "America/Anchorage",
	},
	"PABP": {
		ICAO:      "PABP",
		Name:      "Badami Airport",
		City:      "Deadhorse",
		State:     "Alaska",
		Country:   "US",
		Elevation: 26,
		Latitude:  70.1374969482,
		Longitude: -147.0299987793,
		Timezone:  "America/Anchorage",
	},
	"PABR": {
		ICAO:      "PABR",
		IATA:      "BRW",
		Name:      "Wiley Post Will Rogers Memorial Airport",
		City:      "Barrow",
		State:     "Alaska",
		Country:   "US",
		Elevation: 44,
		Latitude:  71.2854003906,
		Longitude: -156.7660064697,
		Timezone:  "America/Anchorage",
	},
	"PABT": {
		ICAO:      "PABT",
		IATA:      "BTT",
		Name:      "Bettles Airport",
		City:      "Bettles",
		State:     "Alaska",
		Country:   "US",
		Elevation: 647,
		Latitude:  66.91390228,
		Longitude: -151.529007,
		Timezone:  "America/Anchorage",
	},
	"PABU": {
		ICAO:      "PABU",
		Name:      "Bullen Point Air Force Station Airport",
		City:      "Kaktovik",
		State:     "Alaska",
		Country:   "US",
		Elevation: 18,
		Latitude:  70.1727981567,
		Longitude: -146.8359985352,
		Timezone:  "America/Anchorage",
	},
	"PABV": {
		ICAO:      "PABV",
		Name:      "Birchwood Airport",
		City:      "Birchwood",
		State:     "Alaska",
		Country:   "US",
		Elevation: 83,
		Latitude:  61.41650009,
		Longitude: -149.5070038,
		Timezone:  "America/Anchorage",
	},
	"PACD": {
		ICAO:      "PACD",
		IATA:      "CDB",
		Name:      "Cold Bay Airport",
		City:      "Cold Bay",
		State:     "Alaska",
		Country:   "US",
		Elevation: 96,
		Latitude:  55.2061004639,
		Longitude: -162.7250061035,
		Timezone:  "America/Nome",
	},
	"PACE": {
		ICAO:      "PACE",
		IATA:      "CEM",
		Name:      "Central Airport",
		City:      "Central",
		State:     "Alaska",
		Country:   "US",
		Elevation: 937,
		Latitude:  65.57379913,
		Longitude: -144.7830048,
		Timezone:  "America/Anchorage",
	},
	"PACH": {
		ICAO:      "PACH",
		IATA:      "CHU",
		Name:      "Chuathbaluk Airport",
		City:      "Chuathbaluk",
		State:     "Alaska",
		Country:   "US",
		Elevation: 244,
		Latitude:  61.57910156,
		Longitude: -159.2160034,
		Timezone:  "America/Anchorage",
	},
	"PACI": {
		ICAO:      "PACI",
		IATA:      "CIK",
		Name:      "Chalkyitsik Airport",
		City:      "Chalkyitsik",
		State:     "Alaska",
		Country:   "US",
		Elevation: 544,
		Latitude:  66.6449966431,
		Longitude: -143.740005493,
		Timezone:  "America/Anchorage",
	},
	"PACK": {
		ICAO:      "PACK",
		IATA:      "CYF",
		Name:      "Chefornak Airport",
		City:      "Chefornak",
		State:     "Alaska",
		Country:   "US",
		Elevation: 40,
		Latitude:  60.1492004395,
		Longitude: -164.2859954834,
		Timezone:  "America/Nome",
	},
	"PACL": {
		ICAO:      "PACL",
		Name:      "Clear Airport",
		City:      "Clear",
		State:     "Alaska",
		Country:   "US",
		Elevation: 552,
		Latitude:  64.3012008667,
		Longitude: -149.119995117,
		Timezone:  "America/Anchorage",
	},
	"PACM": {
		ICAO:      "PACM",
		IATA:      "SCM",
		Name:      "Scammon Bay Airport",
		City:      "Scammon Bay",
		State:     "Alaska",
		Country:   "US",
		Elevation: 14,
		Latitude:  61.8452987671,
		Longitude: -165.570999146,
		Timezone:  "America/Nome",
	},
	"PACR": {
		ICAO:      "PACR",
		IATA:      "IRC",
		Name:      "Circle City /New/ Airport",
		City:      "Circle",
		State:     "Alaska",
		Country:   "US",
		Elevation: 613,
		Latitude:  65.83049774,
		Longitude: -144.076004,
		Timezone:  "America/Anchorage",
	},
	"PACS": {
		ICAO:      "PACS",
		IATA:      "WSF",
		Name:      "Cape Sarichef Airport",
		City:      "Cape Sarichef",
		State:     "Alaska",
		Country:   "US",
		Elevation: 291,
		Latitude:  54.5824012756,
		Longitude: -164.9140014648,
		Timezone:  "America/Nome",
	},
	"PACV": {
		ICAO:      "PACV",
		IATA:      "CDV",
		Name:      "Merle K (Mudhole) Smith Airport",
		City:      "Cordova",
		State:     "Alaska",
		Country:   "US",
		Elevation: 54,
		Latitude:  60.4917984,
		Longitude: -145.4779968,
		Timezone:  "America/Anchorage",
	},
	"PACX": {
		ICAO:      "PACX",
		IATA:      "CXF",
		Name:      "Coldfoot Airport",
		City:      "Coldfoot",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1042,
		Latitude:  67.2521972656,
		Longitude: -150.203994751,
		Timezone:  "America/Anchorage",
	},
	"PACY": {
		ICAO:      "PACY",
		IATA:      "CYT",
		Name:      "Yakataga Airport",
		City:      "Yakataga",
		State:     "Alaska",
		Country:   "US",
		Elevation: 12,
		Latitude:  60.0820007324,
		Longitude: -142.4929962158,
		Timezone:  "America/Anchorage",
	},
	"PACZ": {
		ICAO:      "PACZ",
		IATA:      "CZF",
		Name:      "Cape Romanzof LRRS Airport",
		City:      "Cape Romanzof",
		State:     "Alaska",
		Country:   "US",
		Elevation: 464,
		Latitude:  61.78030014,
		Longitude: -166.0390015,
		Timezone:  "America/Nome",
	},
	"PADE": {
		ICAO:      "PADE",
		IATA:      "DRG",
		Name:      "Deering Airport",
		City:      "Deering",
		State:     "Alaska",
		Country:   "US",
		Elevation: 21,
		Latitude:  66.0696029663,
		Longitude: -162.76600647,
		Timezone:  "America/Nome",
	},
	"PADG": {
		ICAO:      "PADG",
		IATA:      "RDB",
		Name:      "Red Dog Airport",
		City:      "Red Dog",
		State:     "Alaska",
		Country:   "US",
		Elevation: 969,
		Latitude:  68.0320968628,
		Longitude: -162.899002075,
		Timezone:  "America/Nome",
	},
	"PADK": {
		ICAO:      "PADK",
		IATA:      "ADK",
		Name:      "Adak Airport",
		City:      "Adak Island",
		State:     "Alaska",
		Country:   "US",
		Elevation: 18,
		Latitude:  51.8779983521,
		Longitude: -176.6459960938,
		Timezone:  "America/Adak",
	},
	"PADL": {
		ICAO:      "PADL",
		IATA:      "DLG",
		Name:      "Dillingham Airport",
		City:      "Dillingham",
		State:     "Alaska",
		Country:   "US",
		Elevation: 81,
		Latitude:  59.04470062,
		Longitude: -158.5050049,
		Timezone:  "America/Anchorage",
	},
	"PADM": {
		ICAO:      "PADM",
		IATA:      "MLL",
		Name:      "Marshall Don Hunter Sr Airport",
		City:      "Marshall",
		State:     "Alaska",
		Country:   "US",
		Elevation: 103,
		Latitude:  61.8642997742,
		Longitude: -162.0260009766,
		Timezone:  "America/Anchorage",
	},
	"PADQ": {
		ICAO:      "PADQ",
		IATA:      "ADQ",
		Name:      "Kodiak Airport",
		City:      "Kodiak",
		State:     "Alaska",
		Country:   "US",
		Elevation: 78,
		Latitude:  57.75,
		Longitude: -152.4940033,
		Timezone:  "America/Anchorage",
	},
	"PADU": {
		ICAO:      "PADU",
		IATA:      "DUT",
		Name:      "Unalaska Airport",
		City:      "Unalaska",
		State:     "Alaska",
		Country:   "US",
		Elevation: 22,
		Latitude:  53.900100708,
		Longitude: -166.5440063477,
		Timezone:  "America/Nome",
	},
	"PADY": {
		ICAO:      "PADY",
		IATA:      "KKH",
		Name:      "Kongiganak Airport",
		City:      "Kongiganak",
		State:     "Alaska",
		Country:   "US",
		Elevation: 30,
		Latitude:  59.9608001709,
		Longitude: -162.8809967041,
		Timezone:  "America/Nome",
	},
	"PAED": {
		ICAO:      "PAED",
		IATA:      "EDF",
		Name:      "Elmendorf Air Force Base",
		City:      "Anchorage",
		State:     "Alaska",
		Country:   "US",
		Elevation: 212,
		Latitude:  61.2509994507,
		Longitude: -149.8070068359,
		Timezone:  "America/Anchorage",
	},
	"PAEE": {
		ICAO:      "PAEE",
		IATA:      "EEK",
		Name:      "Eek Airport",
		City:      "Eek",
		State:     "Alaska",
		Country:   "US",
		Elevation: 12,
		Latitude:  60.21367264,
		Longitude: -162.0438843,
		Timezone:  "America/Nome",
	},
	"PAEG": {
		ICAO:      "PAEG",
		IATA:      "EAA",
		Name:      "Eagle Airport",
		City:      "Eagle",
		State:     "Alaska",
		Country:   "US",
		Elevation: 908,
		Latitude:  64.77639771,
		Longitude: -141.151001,
		Timezone:  "America/Anchorage",
	},
	"PAEH": {
		ICAO:      "PAEH",
		IATA:      "EHM",
		Name:      "Cape Newenham LRRS Airport",
		City:      "Cape Newenham",
		State:     "Alaska",
		Country:   "US",
		Elevation: 541,
		Latitude:  58.6464004517,
		Longitude: -162.06300354,
		Timezone:  "America/Nome",
	},
	"PAEI": {
		ICAO:      "PAEI",
		IATA:      "EIL",
		Name:      "Eielson Air Force Base",
		City:      "Fairbanks",
		State:     "Alaska",
		Country:   "US",
		Elevation: 547,
		Latitude:  64.66570282,
		Longitude: -147.102005,
		Timezone:  "America/Anchorage",
	},
	"PAEM": {
		ICAO:      "PAEM",
		IATA:      "EMK",
		Name:      "Emmonak Airport",
		City:      "Emmonak",
		State:     "Alaska",
		Country:   "US",
		Elevation: 13,
		Latitude:  62.78609848,
		Longitude: -164.4909973,
		Timezone:  "America/Nome",
	},
	"PAEN": {
		ICAO:      "PAEN",
		IATA:      "ENA",
		Name:      "Kenai Municipal Airport",
		City:      "Kenai",
		State:     "Alaska",
		Country:   "US",
		Elevation: 99,
		Latitude:  60.5731010437,
		Longitude: -151.2449951172,
		Timezone:  "America/Anchorage",
	},
	"PAEW": {
		ICAO:      "PAEW",
		IATA:      "WWT",
		Name:      "Newtok Airport",
		City:      "Newtok",
		State:     "Alaska",
		Country:   "US",
		Elevation: 25,
		Latitude:  60.9390983582,
		Longitude: -164.6410064697,
		Timezone:  "America/Nome",
	},
	"PAFA": {
		ICAO:      "PAFA",
		IATA:      "FAI",
		Name:      "Fairbanks International Airport",
		City:      "Fairbanks",
		State:     "Alaska",
		Country:   "US",
		Elevation: 439,
		Latitude:  64.81510162,
		Longitude: -147.8560028,
		Timezone:  "America/Anchorage",
	},
	"PAFB": {
		ICAO:      "PAFB",
		IATA:      "FBK",
		Name:      "Ladd AAF Airfield",
		City:      "Fairbanks/Ft Wainwright",
		State:     "Alaska",
		Country:   "US",
		Elevation: 454,
		Latitude:  64.83750153,
		Longitude: -147.6139984,
		Timezone:  "America/Anchorage",
	},
	"PAFE": {
		ICAO:      "PAFE",
		IATA:      "AFE",
		Name:      "Kake Airport",
		City:      "Kake",
		State:     "Alaska",
		Country:   "US",
		Elevation: 172,
		Latitude:  56.9613990784,
		Longitude: -133.9100036621,
		Timezone:  "America/Sitka",
	},
	"PAFL": {
		ICAO:      "PAFL",
		Name:      "Tin Creek Airport",
		City:      "Farewell Lake",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1151,
		Latitude:  62.5316009521,
		Longitude: -153.6109924316,
		Timezone:  "America/Anchorage",
	},
	"PAFM": {
		ICAO:      "PAFM",
		IATA:      "ABL",
		Name:      "Ambler Airport",
		City:      "Ambler",
		State:     "Alaska",
		Country:   "US",
		Elevation: 334,
		Latitude:  67.106300354,
		Longitude: -157.856994629,
		Timezone:  "America/Anchorage",
	},
	"PAFS": {
		ICAO:      "PAFS",
		IATA:      "NIB",
		Name:      "Nikolai Airport",
		City:      "Nikolai",
		State:     "Alaska",
		Country:   "US",
		Elevation: 441,
		Latitude:  63.0186004639,
		Longitude: -154.358001709,
		Timezone:  "America/Anchorage",
	},
	"PAFV": {
		ICAO:      "PAFV",
		IATA:      "FMC",
		Name:      "Five Mile Airport",
		City:      "Five Mile",
		State:     "Alaska",
		Country:   "US",
		Elevation: 510,
		Latitude:  65.9270019531,
		Longitude: -149.839996338,
		Timezone:  "America/Anchorage",
	},
	"PAFW": {
		ICAO:      "PAFW",
		IATA:      "FWL",
		Name:      "Farewell Airport",
		City:      "Farewell",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1535,
		Latitude:  62.5065994263,
		Longitude: -153.8939971924,
		Timezone:  "America/Anchorage",
	},
	"PAGA": {
		ICAO:      "PAGA",
		IATA:      "GAL",
		Name:      "Edward G. Pitka Sr Airport",
		City:      "Galena",
		State:     "Alaska",
		Country:   "US",
		Elevation: 153,
		Latitude:  64.73619843,
		Longitude: -156.9369965,
		Timezone:  "America/Anchorage",
	},
	"PAGB": {
		ICAO:      "PAGB",
		IATA:      "GBH",
		Name:      "Galbraith Lake Airport",
		City:      "Galbraith Lake",
		State:     "Alaska",
		Country:   "US",
		Elevation: 2663,
		Latitude:  68.4796981812,
		Longitude: -149.4900054932,
		Timezone:  "America/Anchorage",
	},
	"PAGG": {
		ICAO:      "PAGG",
		IATA:      "KWK",
		Name:      "Kwigillingok Airport",
		City:      "Kwigillingok",
		State:     "Alaska",
		Country:   "US",
		Elevation: 18,
		Latitude:  59.876499176,
		Longitude: -163.1690063477,
		Timezone:  "America/Nome",
	},
	"PAGH": {
		ICAO:      "PAGH",
		Name:      "Shungnak Airport",
		City:      "Shungnak",
		State:     "Alaska",
		Country:   "US",
		Elevation: 197,
		Latitude:  66.8880996704,
		Longitude: -157.1620025635,
		Timezone:  "America/Anchorage",
	},
	"PAGK": {
		ICAO:      "PAGK",
		IATA:      "GKN",
		Name:      "Gulkana Airport",
		City:      "Gulkana",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1586,
		Latitude:  62.1548996,
		Longitude: -145.4570007,
		Timezone:  "America/Anchorage",
	},
	"PAGL": {
		ICAO:      "PAGL",
		IATA:      "GLV",
		Name:      "Golovin Airport",
		City:      "Golovin",
		State:     "Alaska",
		Country:   "US",
		Elevation: 59,
		Latitude:  64.5504989624,
		Longitude: -163.007003784,
		Timezone:  "America/Nome",
	},
	"PAGM": {
		ICAO:      "PAGM",
		IATA:      "GAM",
		Name:      "Gambell Airport",
		City:      "Gambell",
		State:     "Alaska",
		Country:   "US",
		Elevation: 27,
		Latitude:  63.7667999268,
		Longitude: -171.733001709,
		Timezone:  "America/Nome",
	},
	"PAGQ": {
		ICAO:      "PAGQ",
		IATA:      "BGQ",
		Name:      "Big Lake Airport",
		City:      "Big Lake",
		State:     "Alaska",
		Country:   "US",
		Elevation: 158,
		Latitude:  61.5360984802,
		Longitude: -149.8139953613,
		Timezone:  "America/Anchorage",
	},
	"PAGS": {
		ICAO:      "PAGS",
		IATA:      "GST",
		Name:      "Gustavus Airport",
		City:      "Gustavus",
		State:     "Alaska",
		Country:   "US",
		Elevation: 35,
		Latitude:  58.4253006,
		Longitude: -135.7070007,
		Timezone:  "America/Juneau",
	},
	"PAGT": {
		ICAO:      "PAGT",
		IATA:      "NME",
		Name:      "Nightmute Airport",
		City:      "Nightmute",
		State:     "Alaska",
		Country:   "US",
		Elevation: 4,
		Latitude:  60.4710006714,
		Longitude: -164.7010040283,
		Timezone:  "America/Nome",
	},
	"PAGY": {
		ICAO:      "PAGY",
		IATA:      "SGY",
		Name:      "Skagway Airport",
		City:      "Skagway",
		State:     "Alaska",
		Country:   "US",
		Elevation: 44,
		Latitude:  59.4600982666,
		Longitude: -135.3159942627,
		Timezone:  "America/Juneau",
	},
	"PAGZ": {
		ICAO:      "PAGZ",
		IATA:      "GMT",
		Name:      "Granite Mountain Air Station",
		City:      "Granite Mountain",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1313,
		Latitude:  65.40209961,
		Longitude: -161.2810059,
		Timezone:  "America/Anchorage",
	},
	"PAHC": {
		ICAO:      "PAHC",
		IATA:      "HCR",
		Name:      "Holy Cross Airport",
		City:      "Holy Cross",
		State:     "Alaska",
		Country:   "US",
		Elevation: 70,
		Latitude:  62.1883010864,
		Longitude: -159.7749938965,
		Timezone:  "America/Anchorage",
	},
	"PAHE": {
		ICAO:      "PAHE",
		IATA:      "HNE",
		Name:      "Tahneta Pass Airport",
		City:      "Tahneta Pass Lodge",
		State:     "Alaska",
		Country:   "US",
		Elevation: 2960,
		Latitude:  61.9014015198,
		Longitude: -147.304000854,
		Timezone:  "America/Anchorage",
	},
	"PAHL": {
		ICAO:      "PAHL",
		IATA:      "HSL",
		Name:      "Huslia Airport",
		City:      "Huslia",
		State:     "Alaska",
		Country:   "US",
		Elevation: 220,
		Latitude:  65.69789886,
		Longitude: -156.3509979,
		Timezone:  "America/Anchorage",
	},
	"PAHN": {
		ICAO:      "PAHN",
		IATA:      "HNS",
		Name:      "Haines Airport",
		City:      "Haines",
		State:     "Alaska",
		Country:   "US",
		Elevation: 15,
		Latitude:  59.2438011169,
		Longitude: -135.5240020752,
		Timezone:  "America/Juneau",
	},
	"PAHO": {
		ICAO:      "PAHO",
		IATA:      "HOM",
		Name:      "Homer Airport",
		City:      "Homer",
		State:     "Alaska",
		Country:   "US",
		Elevation: 84,
		Latitude:  59.6455993652,
		Longitude: -151.4770050049,
		Timezone:  "America/Anchorage",
	},
	"PAHP": {
		ICAO:      "PAHP",
		IATA:      "HPB",
		Name:      "Hooper Bay Airport",
		City:      "Hooper Bay",
		State:     "Alaska",
		Country:   "US",
		Elevation: 13,
		Latitude:  61.52389908,
		Longitude: -166.1470032,
		Timezone:  "America/Nome",
	},
	"PAHU": {
		ICAO:      "PAHU",
		IATA:      "HUS",
		Name:      "Hughes Airport",
		City:      "Hughes",
		State:     "Alaska",
		Country:   "US",
		Elevation: 299,
		Latitude:  66.04109955,
		Longitude: -154.2630005,
		Timezone:  "America/Anchorage",
	},
	"PAHV": {
		ICAO:      "PAHV",
		Name:      "Healy River Airport",
		City:      "Healy",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1263,
		Latitude:  63.8661994934,
		Longitude: -148.9689941406,
		Timezone:  "America/Anchorage",
	},
	"PAHX": {
		ICAO:      "PAHX",
		IATA:      "SHX",
		Name:      "Shageluk Airport",
		City:      "Shageluk",
		State:     "Alaska",
		Country:   "US",
		Elevation: 79,
		Latitude:  62.6922988892,
		Longitude: -159.569000244,
		Timezone:  "America/Anchorage",
	},
	"PAIG": {
		ICAO:      "PAIG",
		IATA:      "IGG",
		Name:      "Igiugig Airport",
		City:      "Igiugig",
		State:     "Alaska",
		Country:   "US",
		Elevation: 90,
		Latitude:  59.3240013123,
		Longitude: -155.9019927979,
		Timezone:  "America/Anchorage",
	},
	"PAII": {
		ICAO:      "PAII",
		IATA:      "EGX",
		Name:      "Egegik Airport",
		City:      "Egegik",
		State:     "Alaska",
		Country:   "US",
		Elevation: 92,
		Latitude:  58.1855010986,
		Longitude: -157.375,
		Timezone:  "America/Anchorage",
	},
	"PAIK": {
		ICAO:      "PAIK",
		IATA:      "IAN",
		Name:      "Bob Baker Memorial Airport",
		City:      "Kiana",
		State:     "Alaska",
		Country:   "US",
		Elevation: 166,
		Latitude:  66.9759979248,
		Longitude: -160.43699646,
		Timezone:  "America/Anchorage",
	},
	"PAIL": {
		ICAO:      "PAIL",
		IATA:      "ILI",
		Name:      "Iliamna Airport",
		City:      "Iliamna",
		State:     "Alaska",
		Country:   "US",
		Elevation: 192,
		Latitude:  59.75439835,
		Longitude: -154.9109955,
		Timezone:  "America/Anchorage",
	},
	"PAIM": {
		ICAO:      "PAIM",
		IATA:      "UTO",
		Name:      "Indian Mountain LRRS Airport",
		City:      "Utopia Creek",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1273,
		Latitude:  65.99279785,
		Longitude: -153.7039948,
		Timezone:  "America/Anchorage",
	},
	"PAIN": {
		ICAO:      "PAIN",
		IATA:      "MCL",
		Name:      "Mc Kinley National Park Airport",
		City:      "Mckinley Park",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1720,
		Latitude:  63.7326011658,
		Longitude: -148.910995483,
		Timezone:  "America/Anchorage",
	},
	"PAIW": {
		ICAO:      "PAIW",
		IATA:      "WAA",
		Name:      "Wales Airport",
		City:      "Wales",
		State:     "Alaska",
		Country:   "US",
		Elevation: 22,
		Latitude:  65.6225967407,
		Longitude: -168.095001221,
		Timezone:  "America/Nome",
	},
	"PAJC": {
		ICAO:      "PAJC",
		Name:      "Chignik Airport",
		City:      "Chignik",
		State:     "Alaska",
		Country:   "US",
		Elevation: 18,
		Latitude:  56.3115005493,
		Longitude: -158.3730010986,
		Timezone:  "America/Anchorage",
	},
	"PAJN": {
		ICAO:      "PAJN",
		IATA:      "JNU",
		Name:      "Juneau International Airport",
		City:      "Juneau",
		State:     "Alaska",
		Country:   "US",
		Elevation: 21,
		Latitude:  58.3549995422,
		Longitude: -134.5760040283,
		Timezone:  "America/Juneau",
	},
	"PAJO": {
		ICAO:      "PAJO",
		Name:      "Johnstone Point Airport",
		City:      "Hinchinbrook",
		State:     "Alaska",
		Country:   "US",
		Elevation: 15,
		Latitude:  60.4817008972,
		Longitude: -146.583999634,
		Timezone:  "America/Anchorage",
	},
	"PAJV": {
		ICAO:      "PAJV",
		Name:      "Jonesville Mine Airport",
		City:      "Sutton",
		State:     "Alaska",
		Country:   "US",
		Elevation: 870,
		Latitude:  61.7313995361,
		Longitude: -148.9279937744,
		Timezone:  "America/Anchorage",
	},
	"PAJZ": {
		ICAO:      "PAJZ",
		IATA:      "KGK",
		Name:      "Koliganek Airport",
		City:      "Koliganek",
		State:     "Alaska",
		Country:   "US",
		Elevation: 269,
		Latitude:  59.726600647,
		Longitude: -157.2590026855,
		Timezone:  "America/Anchorage",
	},
	"PAKD": {
		ICAO:      "PAKD",
		IATA:      "KDK",
		Name:      "Kodiak Municipal Airport",
		City:      "Kodiak",
		State:     "Alaska",
		Country:   "US",
		Elevation: 139,
		Latitude:  57.8059005737,
		Longitude: -152.3739929199,
		Timezone:  "America/Anchorage",
	},
	"PAKF": {
		ICAO:      "PAKF",
		IATA:      "KFP",
		Name:      "False Pass Airport",
		City:      "False Pass",
		State:     "Alaska",
		Country:   "US",
		Elevation: 20,
		Latitude:  54.8474006653,
		Longitude: -163.4100036621,
		Timezone:  "America/Nome",
	},
	"PAKH": {
		ICAO:      "PAKH",
		IATA:      "AKK",
		Name:      "Akhiok Airport",
		City:      "Akhiok",
		State:     "Alaska",
		Country:   "US",
		Elevation: 44,
		Latitude:  56.9387016296,
		Longitude: -154.182998657,
		Timezone:  "America/Anchorage",
	},
	"PAKI": {
		ICAO:      "PAKI",
		IATA:      "KPN",
		Name:      "Kipnuk Airport",
		City:      "Kipnuk",
		State:     "Alaska",
		Country:   "US",
		Elevation: 11,
		Latitude:  59.9329986572,
		Longitude: -164.0310058594,
		Timezone:  "America/Nome",
	},
	"PAKK": {
		ICAO:      "PAKK",
		IATA:      "KKA",
		Name:      "Koyuk Alfred Adams Airport",
		City:      "Koyuk",
		State:     "Alaska",
		Country:   "US",
		Elevation: 154,
		Latitude:  64.9394989014,
		Longitude: -161.154006958,
		Timezone:  "America/Anchorage",
	},
	"PAKL": {
		ICAO:      "PAKL",
		IATA:      "LKK",
		Name:      "Kulik Lake Airport",
		City:      "Kulik Lake",
		State:     "Alaska",
		Country:   "US",
		Elevation: 717,
		Latitude:  58.9821014404,
		Longitude: -155.1210021973,
		Timezone:  "America/Anchorage",
	},
	"PAKN": {
		ICAO:      "PAKN",
		IATA:      "AKN",
		Name:      "King Salmon Airport",
		City:      "King Salmon",
		State:     "Alaska",
		Country:   "US",
		Elevation: 73,
		Latitude:  58.67679977,
		Longitude: -156.6490021,
		Timezone:  "America/Anchorage",
	},
	"PAKO": {
		ICAO:      "PAKO",
		IATA:      "IKO",
		Name:      "Nikolski Air Station",
		City:      "Nikolski",
		State:     "Alaska",
		Country:   "US",
		Elevation: 77,
		Latitude:  52.9416007996,
		Longitude: -168.8489990234,
		Timezone:  "America/Nome",
	},
	"PAKP": {
		ICAO:      "PAKP",
		IATA:      "AKP",
		Name:      "Anaktuvuk Pass Airport",
		City:      "Anaktuvuk Pass",
		State:     "Alaska",
		Country:   "US",
		Elevation: 2102,
		Latitude:  68.13359833,
		Longitude: -151.7429962,
		Timezone:  "America/Anchorage",
	},
	"PAKT": {
		ICAO:      "PAKT",
		IATA:      "KTN",
		Name:      "Ketchikan International Airport",
		City:      "Ketchikan",
		State:     "Alaska",
		Country:   "US",
		Elevation: 89,
		Latitude:  55.35559845,
		Longitude: -131.7140045,
		Timezone:  "America/Sitka",
	},
	"PAKU": {
		ICAO:      "PAKU",
		IATA:      "UUK",
		Name:      "Ugnu-Kuparuk Airport",
		City:      "Kuparuk",
		State:     "Alaska",
		Country:   "US",
		Elevation: 67,
		Latitude:  70.3308029175,
		Longitude: -149.598007202,
		Timezone:  "America/Anchorage",
	},
	"PAKV": {
		ICAO:      "PAKV",
		IATA:      "KAL",
		Name:      "Kaltag Airport",
		City:      "Kaltag",
		State:     "Alaska",
		Country:   "US",
		Elevation: 181,
		Latitude:  64.31909943,
		Longitude: -158.7409973,
		Timezone:  "America/Anchorage",
	},
	"PAKW": {
		ICAO:      "PAKW",
		IATA:      "AKW",
		Name:      "Klawock Airport",
		City:      "Klawock",
		State:     "Alaska",
		Country:   "US",
		Elevation: 80,
		Latitude:  55.5792007446,
		Longitude: -133.0760040283,
		Timezone:  "America/Sitka",
	},
	"PAKY": {
		ICAO:      "PAKY",
		IATA:      "KYK",
		Name:      "Karluk Airport",
		City:      "Karluk",
		State:     "Alaska",
		Country:   "US",
		Elevation: 137,
		Latitude:  57.5671005249,
		Longitude: -154.449996948,
		Timezone:  "America/Anchorage",
	},
	"PALB": {
		ICAO:      "PALB",
		IATA:      "KLN",
		Name:      "Larsen Bay Airport",
		City:      "Larsen Bay",
		State:     "Alaska",
		Country:   "US",
		Elevation: 87,
		Latitude:  57.5350990295,
		Longitude: -153.9779968262,
		Timezone:  "America/Anchorage",
	},
	"PALG": {
		ICAO:      "PALG",
		IATA:      "KLG",
		Name:      "Kalskag Airport",
		City:      "Kalskag",
		State:     "Alaska",
		Country:   "US",
		Elevation: 55,
		Latitude:  61.5363006592,
		Longitude: -160.341003418,
		Timezone:  "America/Anchorage",
	},
	"PALJ": {
		ICAO:      "PALJ",
		IATA:      "PTA",
		Name:      "Port Alsworth Airport",
		City:      "Port Alsworth",
		State:     "Alaska",
		Country:   "US",
		Elevation: 280,
		Latitude:  60.2042999268,
		Longitude: -154.319000244,
		Timezone:  "America/Anchorage",
	},
	"PALN": {
		ICAO:      "PALN",
		IATA:      "LNI",
		Name:      "Lonely Air Station",
		City:      "Lonely",
		State:     "Alaska",
		Country:   "US",
		Elevation: 17,
		Latitude:  70.91069794,
		Longitude: -153.2420044,
		Timezone:  "America/Anchorage",
	},
	"PALP": {
		ICAO:      "PALP",
		IATA:      "DQH",
		Name:      "Alpine Airstrip",
		City:      "Deadhorse",
		State:     "Alaska",
		Country:   "US",
		Elevation: 18,
		Latitude:  70.3442993164,
		Longitude: -150.945007324,
		Timezone:  "America/Anchorage",
	},
	"PALR": {
		ICAO:      "PALR",
		IATA:      "WCR",
		Name:      "Chandalar Lake Airport",
		City:      "Chandalar Lake",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1920,
		Latitude:  67.5045013428,
		Longitude: -148.483001709,
		Timezone:  "America/Anchorage",
	},
	"PALU": {
		ICAO:      "PALU",
		IATA:      "LUR",
		Name:      "Cape Lisburne LRRS Airport",
		City:      "Cape Lisburne",
		State:     "Alaska",
		Country:   "US",
		Elevation: 16,
		Latitude:  68.87509918,
		Longitude: -166.1100006,
		Timezone:  "America/Nome",
	},
	"PAMB": {
		ICAO:      "PAMB",
		IATA:      "KMO",
		Name:      "Manokotak Airport",
		City:      "Manokotak",
		State:     "Alaska",
		Country:   "US",
		Elevation: 100,
		Latitude:  58.9902000427,
		Longitude: -159.050003052,
		Timezone:  "America/Anchorage",
	},
	"PAMC": {
		ICAO:      "PAMC",
		IATA:      "MCG",
		Name:      "McGrath Airport",
		City:      "McGrath",
		State:     "Alaska",
		Country:   "US",
		Elevation: 341,
		Latitude:  62.95289993,
		Longitude: -155.6060028,
		Timezone:  "America/Anchorage",
	},
	"PAMD": {
		ICAO:      "PAMD",
		IATA:      "MDO",
		Name:      "Middleton Island Airport",
		City:      "Middleton Island",
		State:     "Alaska",
		Country:   "US",
		Elevation: 100,
		Latitude:  59.4499015808,
		Longitude: -146.307006836,
		Timezone:  "America/Anchorage",
	},
	"PAMH": {
		ICAO:      "PAMH",
		IATA:      "LMA",
		Name:      "Minchumina Airport",
		City:      "Minchumina",
		State:     "Alaska",
		Country:   "US",
		Elevation: 678,
		Latitude:  63.8860015869,
		Longitude: -152.3020019531,
		Timezone:  "America/Anchorage",
	},
	"PAMK": {
		ICAO:      "PAMK",
		IATA:      "SMK",
		Name:      "St Michael Airport",
		City:      "St Michael",
		State:     "Alaska",
		Country:   "US",
		Elevation: 98,
		Latitude:  63.49010086,
		Longitude: -162.1100006,
		Timezone:  "America/Nome",
	},
	"PAML": {
		ICAO:      "PAML",
		IATA:      "MLY",
		Name:      "Manley Hot Springs Airport",
		City:      "Manley Hot Springs",
		State:     "Alaska",
		Country:   "US",
		Elevation: 270,
		Latitude:  64.9975967407,
		Longitude: -150.643997192,
		Timezone:  "America/Anchorage",
	},
	"PAMO": {
		ICAO:      "PAMO",
		IATA:      "MOU",
		Name:      "Mountain Village Airport",
		City:      "Mountain Village",
		State:     "Alaska",
		Country:   "US",
		Elevation: 337,
		Latitude:  62.0954017639,
		Longitude: -163.6820068359,
		Timezone:  "America/Nome",
	},
	"PAMR": {
		ICAO:      "PAMR",
		IATA:      "MRI",
		Name:      "Merrill Field",
		City:      "Anchorage",
		State:     "Iowa",
		Country:   "US",
		Elevation: 137,
		Latitude:  61.2135009766,
		Longitude: -149.8439941406,
		Timezone:  "America/Anchorage",
	},
	"PAMX": {
		ICAO:      "PAMX",
		IATA:      "MXY",
		Name:      "Mc Carthy Airport",
		City:      "Mccarthy",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1531,
		Latitude:  61.4370994568,
		Longitude: -142.904006958,
		Timezone:  "America/Anchorage",
	},
	"PAMY": {
		ICAO:      "PAMY",
		IATA:      "MYU",
		Name:      "Mekoryuk Airport",
		City:      "Mekoryuk",
		State:     "Alaska",
		Country:   "US",
		Elevation: 48,
		Latitude:  60.3713989258,
		Longitude: -166.2709960938,
		Timezone:  "America/Nome",
	},
	"PANA": {
		ICAO:      "PANA",
		IATA:      "WNA",
		Name:      "Napakiak Airport",
		City:      "Napakiak",
		State:     "Alaska",
		Country:   "US",
		Elevation: 17,
		Latitude:  60.6902999878,
		Longitude: -161.9790039063,
		Timezone:  "America/Anchorage",
	},
	"PANC": {
		ICAO:      "PANC",
		IATA:      "ANC",
		Name:      "Ted Stevens Anchorage International Airport",
		City:      "Anchorage",
		State:     "Alaska",
		Country:   "US",
		Elevation: 152,
		Latitude:  61.1744003296,
		Longitude: -149.9960021973,
		Timezone:  "America/Anchorage",
	},
	"PANI": {
		ICAO:      "PANI",
		IATA:      "ANI",
		Name:      "Aniak Airport",
		City:      "Aniak",
		State:     "Alaska",
		Country:   "US",
		Elevation: 88,
		Latitude:  61.5816001892,
		Longitude: -159.5429992676,
		Timezone:  "America/Anchorage",
	},
	"PANN": {
		ICAO:      "PANN",
		IATA:      "ENN",
		Name:      "Nenana Municipal Airport",
		City:      "Nenana",
		State:     "Alaska",
		Country:   "US",
		Elevation: 362,
		Latitude:  64.5473022461,
		Longitude: -149.074005127,
		Timezone:  "America/Anchorage",
	},
	"PANO": {
		ICAO:      "PANO",
		IATA:      "NNL",
		Name:      "Nondalton Airport",
		City:      "Nondalton",
		State:     "Alaska",
		Country:   "US",
		Elevation: 314,
		Latitude:  59.9802017212,
		Longitude: -154.8390045166,
		Timezone:  "America/Anchorage",
	},
	"PANT": {
		ICAO:      "PANT",
		IATA:      "ANN",
		Name:      "Annette Island Airport",
		City:      "Annette",
		State:     "Alaska",
		Country:   "US",
		Elevation: 119,
		Latitude:  55.0424003601,
		Longitude: -131.5720062256,
		Timezone:  "America/Metlakatla",
	},
	"PANU": {
		ICAO:      "PANU",
		IATA:      "NUL",
		Name:      "Nulato Airport",
		City:      "Nulato",
		State:     "Alaska",
		Country:   "US",
		Elevation: 399,
		Latitude:  64.7293014526,
		Longitude: -158.074005127,
		Timezone:  "America/Anchorage",
	},
	"PANV": {
		ICAO:      "PANV",
		IATA:      "ANV",
		Name:      "Anvik Airport",
		City:      "Anvik",
		State:     "Alaska",
		Country:   "US",
		Elevation: 291,
		Latitude:  62.64670181,
		Longitude: -160.1909943,
		Timezone:  "America/Anchorage",
	},
	"PANW": {
		ICAO:      "PANW",
		IATA:      "KNW",
		Name:      "New Stuyahok Airport",
		City:      "New Stuyahok",
		State:     "Alaska",
		Country:   "US",
		Elevation: 364,
		Latitude:  59.4499015808,
		Longitude: -157.32800293,
		Timezone:  "America/Anchorage",
	},
	"PAOB": {
		ICAO:      "PAOB",
		IATA:      "OBU",
		Name:      "Kobuk Airport",
		City:      "Kobuk",
		State:     "Alaska",
		Country:   "US",
		Elevation: 137,
		Latitude:  66.9123001099,
		Longitude: -156.897003174,
		Timezone:  "America/Anchorage",
	},
	"PAOC": {
		ICAO:      "PAOC",
		IATA:      "PCA",
		Name:      "Portage Creek Airport",
		City:      "Portage Creek",
		State:     "Alaska",
		Country:   "US",
		Elevation: 137,
		Latitude:  58.90650177,
		Longitude: -157.714004517,
		Timezone:  "America/Anchorage",
	},
	"PAOH": {
		ICAO:      "PAOH",
		IATA:      "HNH",
		Name:      "Hoonah Airport",
		City:      "Hoonah",
		State:     "Alaska",
		Country:   "US",
		Elevation: 19,
		Latitude:  58.0960998535,
		Longitude: -135.4100036621,
		Timezone:  "America/Juneau",
	},
	"PAOM": {
		ICAO:      "PAOM",
		IATA:      "OME",
		Name:      "Nome Airport",
		City:      "Nome",
		State:     "Alaska",
		Country:   "US",
		Elevation: 37,
		Latitude:  64.5121994019,
		Longitude: -165.4450073242,
		Timezone:  "America/Nome",
	},
	"PAOO": {
		ICAO:      "PAOO",
		IATA:      "OOK",
		Name:      "Toksook Bay Airport",
		City:      "Toksook Bay",
		State:     "Alaska",
		Country:   "US",
		Elevation: 59,
		Latitude:  60.54140091,
		Longitude: -165.0870056,
		Timezone:  "America/Nome",
	},
	"PAOR": {
		ICAO:      "PAOR",
		IATA:      "ORT",
		Name:      "Northway Airport",
		City:      "Northway",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1715,
		Latitude:  62.9612999,
		Longitude: -141.9290009,
		Timezone:  "America/Anchorage",
	},
	"PAOT": {
		ICAO:      "PAOT",
		IATA:      "OTZ",
		Name:      "Ralph Wien Memorial Airport",
		City:      "Kotzebue",
		State:     "Alaska",
		Country:   "US",
		Elevation: 14,
		Latitude:  66.88469696,
		Longitude: -162.598999,
		Timezone:  "America/Nome",
	},
	"PAOU": {
		ICAO:      "PAOU",
		IATA:      "NLG",
		Name:      "Nelson Lagoon Airport",
		City:      "Nelson Lagoon",
		State:     "Alaska",
		Country:   "US",
		Elevation: 14,
		Latitude:  56.0074996948,
		Longitude: -161.1600036621,
		Timezone:  "America/Anchorage",
	},
	"PAPB": {
		ICAO:      "PAPB",
		IATA:      "PBV",
		Name:      "St George Airport",
		City:      "St George",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 125,
		Latitude:  56.5783004761,
		Longitude: -169.6620025635,
		Timezone:  "America/Nome",
	},
	"PAPC": {
		ICAO:      "PAPC",
		IATA:      "KPC",
		Name:      "Port Clarence Coast Guard Station",
		City:      "Port Clarence",
		State:     "Alaska",
		Country:   "US",
		Elevation: 10,
		Latitude:  65.2537002563,
		Longitude: -166.85899353,
		Timezone:  "America/Nome",
	},
	"PAPE": {
		ICAO:      "PAPE",
		IATA:      "KPV",
		Name:      "Perryville Airport",
		City:      "Perryville",
		State:     "Alaska",
		Country:   "US",
		Elevation: 29,
		Latitude:  55.90599823,
		Longitude: -159.1629944,
		Timezone:  "America/Anchorage",
	},
	"PAPG": {
		ICAO:      "PAPG",
		IATA:      "PSG",
		Name:      "Petersburg James A Johnson Airport",
		City:      "Petersburg",
		State:     "Alaska",
		Country:   "US",
		Elevation: 111,
		Latitude:  56.80170059,
		Longitude: -132.9450073,
		Timezone:  "America/Sitka",
	},
	"PAPH": {
		ICAO:      "PAPH",
		IATA:      "PTH",
		Name:      "Port Heiden Airport",
		City:      "Port Heiden",
		State:     "Alaska",
		Country:   "US",
		Elevation: 95,
		Latitude:  56.9590988159,
		Longitude: -158.6329956055,
		Timezone:  "America/Anchorage",
	},
	"PAPK": {
		ICAO:      "PAPK",
		IATA:      "PKA",
		Name:      "Napaskiak Airport",
		City:      "Napaskiak",
		State:     "Alaska",
		Country:   "US",
		Elevation: 24,
		Latitude:  60.70289993,
		Longitude: -161.7779999,
		Timezone:  "America/Anchorage",
	},
	"PAPM": {
		ICAO:      "PAPM",
		IATA:      "PTU",
		Name:      "Platinum Airport",
		City:      "Platinum",
		State:     "Alaska",
		Country:   "US",
		Elevation: 15,
		Latitude:  59.0113983154,
		Longitude: -161.8200073242,
		Timezone:  "America/Anchorage",
	},
	"PAPN": {
		ICAO:      "PAPN",
		IATA:      "PIP",
		Name:      "Pilot Point Airport",
		City:      "Pilot Point",
		State:     "Alaska",
		Country:   "US",
		Elevation: 57,
		Latitude:  57.5803985596,
		Longitude: -157.572006226,
		Timezone:  "America/Anchorage",
	},
	"PAPO": {
		ICAO:      "PAPO",
		IATA:      "PHO",
		Name:      "Point Hope Airport",
		City:      "Point Hope",
		State:     "Alaska",
		Country:   "US",
		Elevation: 12,
		Latitude:  68.3488006592,
		Longitude: -166.7989959717,
		Timezone:  "America/Nome",
	},
	"PAPR": {
		ICAO:      "PAPR",
		IATA:      "PPC",
		Name:      "Prospect Creek Airport",
		City:      "Prospect Creek",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1095,
		Latitude:  66.8141021729,
		Longitude: -150.6439971924,
		Timezone:  "America/Anchorage",
	},
	"PAQH": {
		ICAO:      "PAQH",
		IATA:      "KWN",
		Name:      "Quinhagak Airport",
		City:      "Quinhagak",
		State:     "Alaska",
		Country:   "US",
		Elevation: 42,
		Latitude:  59.75510025,
		Longitude: -161.8450012,
		Timezone:  "America/Anchorage",
	},
	"PAQT": {
		ICAO:      "PAQT",
		IATA:      "NUI",
		Name:      "Nuiqsut Airport",
		City:      "Nuiqsut",
		State:     "Alaska",
		Country:   "US",
		Elevation: 38,
		Latitude:  70.2099990845,
		Longitude: -151.005996704,
		Timezone:  "America/Anchorage",
	},
	"PARC": {
		ICAO:      "PARC",
		IATA:      "ARC",
		Name:      "Arctic Village Airport",
		City:      "Arctic Village",
		State:     "Alaska",
		Country:   "US",
		Elevation: 2092,
		Latitude:  68.1147003174,
		Longitude: -145.578994751,
		Timezone:  "America/Anchorage",
	},
	"PARS": {
		ICAO:      "PARS",
		IATA:      "RSH",
		Name:      "Russian Mission Airport",
		City:      "Russian Mission",
		State:     "Alaska",
		Country:   "US",
		Elevation: 51,
		Latitude:  61.7788848877,
		Longitude: -161.3194580078,
		Timezone:  "America/Anchorage",
	},
	"PARY": {
		ICAO:      "PARY",
		IATA:      "RBY",
		Name:      "Ruby Airport",
		City:      "Ruby",
		State:     "Alaska",
		Country:   "US",
		Elevation: 658,
		Latitude:  64.72720337,
		Longitude: -155.4700012,
		Timezone:  "America/Anchorage",
	},
	"PASA": {
		ICAO:      "PASA",
		IATA:      "SVA",
		Name:      "Savoonga Airport",
		City:      "Savoonga",
		State:     "Alaska",
		Country:   "US",
		Elevation: 53,
		Latitude:  63.6864013672,
		Longitude: -170.4929962158,
		Timezone:  "America/Nome",
	},
	"PASC": {
		ICAO:      "PASC",
		IATA:      "SCC",
		Name:      "Deadhorse Airport",
		City:      "Deadhorse",
		State:     "Alaska",
		Country:   "US",
		Elevation: 65,
		Latitude:  70.19470215,
		Longitude: -148.4649963,
		Timezone:  "America/Anchorage",
	},
	"PASD": {
		ICAO:      "PASD",
		IATA:      "SDP",
		Name:      "Sand Point Airport",
		City:      "Sand Point",
		State:     "Alaska",
		Country:   "US",
		Elevation: 21,
		Latitude:  55.3149986267,
		Longitude: -160.5229949951,
		Timezone:  "America/Anchorage",
	},
	"PASH": {
		ICAO:      "PASH",
		IATA:      "SHH",
		Name:      "Shishmaref Airport",
		City:      "Shishmaref",
		State:     "Alaska",
		Country:   "US",
		Elevation: 12,
		Latitude:  66.2496032715,
		Longitude: -166.0890045166,
		Timezone:  "America/Nome",
	},
	"PASI": {
		ICAO:      "PASI",
		IATA:      "SIT",
		Name:      "Sitka Rocky Gutierrez Airport",
		City:      "Sitka",
		State:     "Alaska",
		Country:   "US",
		Elevation: 21,
		Latitude:  57.0471000671,
		Longitude: -135.3619995117,
		Timezone:  "America/Sitka",
	},
	"PASK": {
		ICAO:      "PASK",
		IATA:      "WLK",
		Name:      "Selawik Airport",
		City:      "Selawik",
		State:     "Alaska",
		Country:   "US",
		Elevation: 17,
		Latitude:  66.60009766,
		Longitude: -159.9859924,
		Timezone:  "America/Anchorage",
	},
	"PASL": {
		ICAO:      "PASL",
		IATA:      "SQL",
		Name:      "Sleetmute Airport",
		City:      "Sleetmute",
		State:     "Alaska",
		Country:   "US",
		Elevation: 190,
		Latitude:  61.7005004883,
		Longitude: -157.166000366,
		Timezone:  "America/Anchorage",
	},
	"PASM": {
		ICAO:      "PASM",
		IATA:      "KSM",
		Name:      "St Mary's Airport",
		City:      "St Mary's",
		State:     "Alaska",
		Country:   "US",
		Elevation: 312,
		Latitude:  62.0605011,
		Longitude: -163.302002,
		Timezone:  "America/Nome",
	},
	"PASN": {
		ICAO:      "PASN",
		IATA:      "SNP",
		Name:      "St Paul Island Airport",
		City:      "St Paul Island",
		State:     "Alaska",
		Country:   "US",
		Elevation: 63,
		Latitude:  57.167301178,
		Longitude: -170.2200012207,
		Timezone:  "America/Nome",
	},
	"PASO": {
		ICAO:      "PASO",
		IATA:      "SOV",
		Name:      "Seldovia Airport",
		City:      "Seldovia",
		State:     "Alaska",
		Country:   "US",
		Elevation: 29,
		Latitude:  59.442401886,
		Longitude: -151.703994751,
		Timezone:  "America/Anchorage",
	},
	"PASP": {
		ICAO:      "PASP",
		IATA:      "SMU",
		Name:      "Sheep Mountain Airport",
		City:      "Sheep Mountain",
		State:     "Alaska",
		Country:   "US",
		Elevation: 2750,
		Latitude:  61.8120002747,
		Longitude: -147.507003784,
		Timezone:  "America/Anchorage",
	},
	"PAST": {
		ICAO:      "PAST",
		IATA:      "UMM",
		Name:      "Summit Airport",
		City:      "Summit",
		State:     "Alaska",
		Country:   "US",
		Elevation: 2409,
		Latitude:  63.3315010071,
		Longitude: -149.126998901,
		Timezone:  "America/Anchorage",
	},
	"PASV": {
		ICAO:      "PASV",
		IATA:      "SVW",
		Name:      "Sparrevohn LRRS Airport",
		City:      "Sparrevohn",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1585,
		Latitude:  61.09740067,
		Longitude: -155.5740051,
		Timezone:  "America/Anchorage",
	},
	"PASW": {
		ICAO:      "PASW",
		IATA:      "SKW",
		Name:      "Skwentna Airport",
		City:      "Skwentna",
		State:     "Alaska",
		Country:   "US",
		Elevation: 148,
		Latitude:  61.9653015137,
		Longitude: -151.1909942627,
		Timezone:  "America/Anchorage",
	},
	"PASX": {
		ICAO:      "PASX",
		IATA:      "SXQ",
		Name:      "Soldotna Airport",
		City:      "Soldotna",
		State:     "Alaska",
		Country:   "US",
		Elevation: 113,
		Latitude:  60.4757003784,
		Longitude: -151.033996582,
		Timezone:  "America/Anchorage",
	},
	"PASY": {
		ICAO:      "PASY",
		IATA:      "SYA",
		Name:      "Eareckson Air Station",
		City:      "Shemya",
		State:     "Alaska",
		Country:   "US",
		Elevation: 95,
		Latitude:  52.71229935,
		Longitude: 174.1139984,
		Timezone:  "America/Adak",
	},
	"PATA": {
		ICAO:      "PATA",
		IATA:      "TAL",
		Name:      "Ralph M Calhoun Memorial Airport",
		City:      "Tanana",
		State:     "Alaska",
		Country:   "US",
		Elevation: 236,
		Latitude:  65.1744003296,
		Longitude: -152.10899353,
		Timezone:  "America/Anchorage",
	},
	"PATC": {
		ICAO:      "PATC",
		IATA:      "TNC",
		Name:      "Tin City Long Range Radar Station Airport",
		City:      "Tin City",
		State:     "Alaska",
		Country:   "US",
		Elevation: 271,
		Latitude:  65.56310272,
		Longitude: -167.9219971,
		Timezone:  "America/Nome",
	},
	"PATE": {
		ICAO:      "PATE",
		IATA:      "TLA",
		Name:      "Teller Airport",
		City:      "Teller",
		State:     "Alaska",
		Country:   "US",
		Elevation: 294,
		Latitude:  65.2404022217,
		Longitude: -166.3390045166,
		Timezone:  "America/Nome",
	},
	"PATG": {
		ICAO:      "PATG",
		IATA:      "TOG",
		Name:      "Togiak Airport",
		City:      "Togiak Village",
		State:     "Alaska",
		Country:   "US",
		Elevation: 21,
		Latitude:  59.0527992249,
		Longitude: -160.3970031738,
		Timezone:  "America/Anchorage",
	},
	"PATJ": {
		ICAO:      "PATJ",
		IATA:      "TKJ",
		Name:      "Tok Airport",
		City:      "Tok",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1670,
		Latitude:  63.303333,
		Longitude: -143.001111,
		Timezone:  "America/Anchorage",
	},
	"PATK": {
		ICAO:      "PATK",
		IATA:      "TKA",
		Name:      "Talkeetna Airport",
		City:      "Talkeetna",
		State:     "Alaska",
		Country:   "US",
		Elevation: 358,
		Latitude:  62.3204994202,
		Longitude: -150.0939941406,
		Timezone:  "America/Anchorage",
	},
	"PATL": {
		ICAO:      "PATL",
		IATA:      "TLJ",
		Name:      "Tatalina LRRS Airport",
		City:      "Takotna",
		State:     "Alaska",
		Country:   "US",
		Elevation: 964,
		Latitude:  62.8944015503,
		Longitude: -155.977005005,
		Timezone:  "America/Anchorage",
	},
	"PATQ": {
		ICAO:      "PATQ",
		IATA:      "ATK",
		Name:      "Atqasuk Edward Burnell Sr Memorial Airport",
		City:      "Atqasuk",
		State:     "Alaska",
		Country:   "US",
		Elevation: 96,
		Latitude:  70.467300415,
		Longitude: -157.4360046387,
		Timezone:  "America/Anchorage",
	},
	"PATW": {
		ICAO:      "PATW",
		Name:      "Cantwell Airport",
		City:      "Cantwell",
		State:     "Alaska",
		Country:   "US",
		Elevation: 2190,
		Latitude:  63.3912010193,
		Longitude: -148.9559936523,
		Timezone:  "America/Anchorage",
	},
	"PAUK": {
		ICAO:      "PAUK",
		IATA:      "AUK",
		Name:      "Alakanuk Airport",
		City:      "Alakanuk",
		State:     "Alaska",
		Country:   "US",
		Elevation: 10,
		Latitude:  62.6800422668,
		Longitude: -164.659927368,
		Timezone:  "America/Nome",
	},
	"PAUM": {
		ICAO:      "PAUM",
		IATA:      "UMT",
		Name:      "Umiat Airport",
		City:      "Umiat",
		State:     "Alaska",
		Country:   "US",
		Elevation: 267,
		Latitude:  69.37110138,
		Longitude: -152.1360016,
		Timezone:  "America/Anchorage",
	},
	"PAUN": {
		ICAO:      "PAUN",
		IATA:      "UNK",
		Name:      "Unalakleet Airport",
		City:      "Unalakleet",
		State:     "Alaska",
		Country:   "US",
		Elevation: 27,
		Latitude:  63.88840103,
		Longitude: -160.798996,
		Timezone:  "America/Anchorage",
	},
	"PAUO": {
		ICAO:      "PAUO",
		IATA:      "WOW",
		Name:      "Willow Airport",
		City:      "Willow",
		State:     "Alaska",
		Country:   "US",
		Elevation: 221,
		Latitude:  61.7541999817,
		Longitude: -150.0520019531,
		Timezone:  "America/Anchorage",
	},
	"PAUT": {
		ICAO:      "PAUT",
		Name:      "Akun Airport",
		City:      "Akutan",
		State:     "Alaska",
		Country:   "US",
		Latitude:  54.1449925194,
		Longitude: -165.607566833,
		Timezone:  "America/Nome",
	},
	"PAVA": {
		ICAO:      "PAVA",
		IATA:      "VAK",
		Name:      "Chevak Airport",
		City:      "Chevak",
		State:     "Alaska",
		Country:   "US",
		Elevation: 75,
		Latitude:  61.5409,
		Longitude: -165.6005,
		Timezone:  "America/Nome",
	},
	"PAVC": {
		ICAO:      "PAVC",
		IATA:      "KVC",
		Name:      "King Cove Airport",
		City:      "King Cove",
		State:     "Alaska",
		Country:   "US",
		Elevation: 155,
		Latitude:  55.1162986755,
		Longitude: -162.2660064697,
		Timezone:  "America/Nome",
	},
	"PAVD": {
		ICAO:      "PAVD",
		IATA:      "VDZ",
		Name:      "Valdez Pioneer Field",
		City:      "Valdez",
		State:     "Alaska",
		Country:   "US",
		Elevation: 121,
		Latitude:  61.13389969,
		Longitude: -146.2480011,
		Timezone:  "America/Anchorage",
	},
	"PAVE": {
		ICAO:      "PAVE",
		IATA:      "VEE",
		Name:      "Venetie Airport",
		City:      "Venetie",
		State:     "Alaska",
		Country:   "US",
		Elevation: 574,
		Latitude:  67.0086975098,
		Longitude: -146.365997314,
		Timezone:  "America/Anchorage",
	},
	"PAVL": {
		ICAO:      "PAVL",
		IATA:      "KVL",
		Name:      "Kivalina Airport",
		City:      "Kivalina",
		State:     "Alaska",
		Country:   "US",
		Elevation: 13,
		Latitude:  67.7361984253,
		Longitude: -164.56300354,
		Timezone:  "America/Nome",
	},
	"PAWB": {
		ICAO:      "PAWB",
		IATA:      "WBQ",
		Name:      "Beaver Airport",
		City:      "Beaver",
		State:     "Alaska",
		Country:   "US",
		Elevation: 359,
		Latitude:  66.362197876,
		Longitude: -147.406997681,
		Timezone:  "America/Anchorage",
	},
	"PAWD": {
		ICAO:      "PAWD",
		IATA:      "SWD",
		Name:      "Seward Airport",
		City:      "Seward",
		State:     "Alaska",
		Country:   "US",
		Elevation: 22,
		Latitude:  60.1268997192,
		Longitude: -149.4190063477,
		Timezone:  "America/Anchorage",
	},
	"PAWG": {
		ICAO:      "PAWG",
		IATA:      "WRG",
		Name:      "Wrangell Airport",
		City:      "Wrangell",
		State:     "Alaska",
		Country:   "US",
		Elevation: 49,
		Latitude:  56.48429871,
		Longitude: -132.3699951,
		Timezone:  "America/Sitka",
	},
	"PAWI": {
		ICAO:      "PAWI",
		IATA:      "AIN",
		Name:      "Wainwright Airport",
		City:      "Wainwright",
		State:     "Alaska",
		Country:   "US",
		Elevation: 41,
		Latitude:  70.6380004883,
		Longitude: -159.994995117,
		Timezone:  "America/Anchorage",
	},
	"PAWM": {
		ICAO:      "PAWM",
		IATA:      "WMO",
		Name:      "White Mountain Airport",
		City:      "White Mountain",
		State:     "Alaska",
		Country:   "US",
		Elevation: 267,
		Latitude:  64.689201355,
		Longitude: -163.4129943848,
		Timezone:  "America/Nome",
	},
	"PAWN": {
		ICAO:      "PAWN",
		IATA:      "WTK",
		Name:      "Noatak Airport",
		City:      "Noatak",
		State:     "Alaska",
		Country:   "US",
		Elevation: 88,
		Latitude:  67.5661010742,
		Longitude: -162.9750061035,
		Timezone:  "America/Nome",
	},
	"PAWR": {
		ICAO:      "PAWR",
		Name:      "Whittier Airport",
		City:      "Whittier",
		State:     "Alaska",
		Country:   "US",
		Elevation: 30,
		Latitude:  60.7771987915,
		Longitude: -148.7220001221,
		Timezone:  "America/Anchorage",
	},
	"PAWS": {
		ICAO:      "PAWS",
		IATA:      "IYS",
		Name:      "Wasilla Airport",
		City:      "Wasilla",
		State:     "Alaska",
		Country:   "US",
		Elevation: 354,
		Latitude:  61.5717010498,
		Longitude: -149.5399932861,
		Timezone:  "America/Anchorage",
	},
	"PAWT": {
		ICAO:      "PAWT",
		Name:      "Wainwright Air Station",
		City:      "Wainwright",
		State:     "Alaska",
		Country:   "US",
		Elevation: 35,
		Latitude:  70.61340332,
		Longitude: -159.8600006,
		Timezone:  "America/Anchorage",
	},
	"PAXK": {
		ICAO:      "PAXK",
		Name:      "Paxson Airport",
		City:      "Paxson",
		State:     "Alaska",
		Country:   "US",
		Elevation: 2653,
		Latitude:  63.0246009827,
		Longitude: -145.50100708,
		Timezone:  "America/Anchorage",
	},
	"PAYA": {
		ICAO:      "PAYA",
		IATA:      "YAK",
		Name:      "Yakutat Airport",
		City:      "Yakutat",
		State:     "Alaska",
		Country:   "US",
		Elevation: 33,
		Latitude:  59.5032997131,
		Longitude: -139.660003662,
		Timezone:  "America/Yakutat",
	},
	"PAZK": {
		ICAO:      "PAZK",
		Name:      "Skelton Airport",
		City:      "Eureka",
		State:     "Alaska",
		Country:   "US",
		Elevation: 3289,
		Latitude:  61.9370002747,
		Longitude: -147.169006348,
		Timezone:  "America/Anchorage",
	},
	"PCIS": {
		ICAO:      "PCIS",
		IATA:      "CIS",
		Name:      "Canton Airport",
		City:      "Abariringa",
		State:     "Phoenix-Islands",
		Country:   "KI",
		Elevation: 9,
		Latitude:  -2.7681200504,
		Longitude: -171.7100067139,
		Timezone:  "Pacific/Enderbury",
	},
	"PFAK": {
		ICAO:      "PFAK",
		IATA:      "AKI",
		Name:      "Akiak Airport",
		City:      "Akiak",
		State:     "Alaska",
		Country:   "US",
		Elevation: 30,
		Latitude:  60.9029006958,
		Longitude: -161.231002808,
		Timezone:  "America/Anchorage",
	},
	"PFAL": {
		ICAO:      "PFAL",
		IATA:      "AET",
		Name:      "Allakaket Airport",
		City:      "Allakaket",
		State:     "Alaska",
		Country:   "US",
		Elevation: 441,
		Latitude:  66.5518035889,
		Longitude: -152.621994019,
		Timezone:  "America/Anchorage",
	},
	"PFCB": {
		ICAO:      "PFCB",
		IATA:      "NCN",
		Name:      "Chenega Bay Airport",
		City:      "Chenega",
		State:     "Alaska",
		Country:   "US",
		Elevation: 72,
		Latitude:  60.0773010254,
		Longitude: -147.992004395,
		Timezone:  "America/Anchorage",
	},
	"PFCL": {
		ICAO:      "PFCL",
		IATA:      "CLP",
		Name:      "Clarks Point Airport",
		City:      "Clarks Point",
		State:     "Alaska",
		Country:   "US",
		Elevation: 80,
		Latitude:  58.83369827,
		Longitude: -158.529007,
		Timezone:  "America/Anchorage",
	},
	"PFEL": {
		ICAO:      "PFEL",
		IATA:      "ELI",
		Name:      "Elim Airport",
		City:      "Elim",
		State:     "Alaska",
		Country:   "US",
		Elevation: 162,
		Latitude:  64.61470032,
		Longitude: -162.2720032,
		Timezone:  "America/Nome",
	},
	"PFKA": {
		ICAO:      "PFKA",
		IATA:      "KUK",
		Name:      "Kasigluk Airport",
		City:      "Kasigluk",
		State:     "Alaska",
		Country:   "US",
		Elevation: 48,
		Latitude:  60.87440109,
		Longitude: -162.5240021,
		Timezone:  "America/Nome",
	},
	"PFKK": {
		ICAO:      "PFKK",
		IATA:      "KNK",
		Name:      "Kokhanok Airport",
		City:      "Kokhanok",
		State:     "Alaska",
		Country:   "US",
		Elevation: 115,
		Latitude:  59.4332008362,
		Longitude: -154.804000854,
		Timezone:  "America/Anchorage",
	},
	"PFKO": {
		ICAO:      "PFKO",
		IATA:      "KOT",
		Name:      "Kotlik Airport",
		City:      "Kotlik",
		State:     "Alaska",
		Country:   "US",
		Elevation: 15,
		Latitude:  63.0306015015,
		Longitude: -163.533004761,
		Timezone:  "America/Nome",
	},
	"PFKT": {
		ICAO:      "PFKT",
		IATA:      "KTS",
		Name:      "Brevig Mission Airport",
		City:      "Brevig Mission",
		State:     "Alaska",
		Country:   "US",
		Elevation: 38,
		Latitude:  65.3312988281,
		Longitude: -166.466003418,
		Timezone:  "America/Nome",
	},
	"PFKU": {
		ICAO:      "PFKU",
		IATA:      "KYU",
		Name:      "Koyukuk Airport",
		City:      "Koyukuk",
		State:     "Alaska",
		Country:   "US",
		Elevation: 149,
		Latitude:  64.8760986328,
		Longitude: -157.727005005,
		Timezone:  "America/Anchorage",
	},
	"PFKW": {
		ICAO:      "PFKW",
		IATA:      "KWT",
		Name:      "Kwethluk Airport",
		City:      "Kwethluk",
		State:     "Alaska",
		Country:   "US",
		Elevation: 25,
		Latitude:  60.7902984619,
		Longitude: -161.444000244,
		Timezone:  "America/Anchorage",
	},
	"PFNO": {
		ICAO:      "PFNO",
		IATA:      "ORV",
		Name:      "Robert (Bob) Curtis Memorial Airport",
		City:      "Noorvik",
		State:     "Alaska",
		Country:   "US",
		Elevation: 55,
		Latitude:  66.81790161,
		Longitude: -161.0189972,
		Timezone:  "America/Anchorage",
	},
	"PFSH": {
		ICAO:      "PFSH",
		IATA:      "SKK",
		Name:      "Shaktoolik Airport",
		City:      "Shaktoolik",
		State:     "Alaska",
		Country:   "US",
		Elevation: 24,
		Latitude:  64.37110138,
		Longitude: -161.223999,
		Timezone:  "America/Anchorage",
	},
	"PFTO": {
		ICAO:      "PFTO",
		IATA:      "TKJ",
		Name:      "Tok Junction Airport",
		City:      "Tok",
		State:     "Alaska",
		Country:   "US",
		Elevation: 1639,
		Latitude:  63.32949829,
		Longitude: -142.9539948,
		Timezone:  "America/Anchorage",
	},
	"PFWS": {
		ICAO:      "PFWS",
		IATA:      "WSN",
		Name:      "South Naknek Nr 2 Airport",
		City:      "South Naknek",
		State:     "Alaska",
		Country:   "US",
		Elevation: 162,
		Latitude:  58.7033996582,
		Longitude: -157.007995605,
		Timezone:  "America/Anchorage",
	},
	"PFYU": {
		ICAO:      "PFYU",
		IATA:      "FYU",
		Name:      "Fort Yukon Airport",
		City:      "Fort Yukon",
		State:     "Alaska",
		Country:   "US",
		Elevation: 433,
		Latitude:  66.5715026855,
		Longitude: -145.25,
		Timezone:  "America/Anchorage",
	},
	"PGRO": {
		ICAO:      "PGRO",
		IATA:      "ROP",
		Name:      "Rota International Airport",
		City:      "Rota Island",
		State:     "Rota",
		Country:   "MP",
		Elevation: 607,
		Latitude:  14.1743001938,
		Longitude: 145.2429962158,
		Timezone:  "Pacific/Saipan",
	},
	"PGSN": {
		ICAO:      "PGSN",
		IATA:      "SPN",
		Name:      "Francisco C. Ada Saipan International Airport",
		City:      "Saipan Island",
		State:     "Saipan",
		Country:   "MP",
		Elevation: 215,
		Latitude:  15.1190004349,
		Longitude: 145.7290039063,
		Timezone:  "Pacific/Saipan",
	},
	"PGUA": {
		ICAO:      "PGUA",
		IATA:      "UAM",
		Name:      "Andersen Air Force Base",
		City:      "Andersen",
		State:     "Yigo",
		Country:   "GU",
		Elevation: 627,
		Latitude:  13.5839996338,
		Longitude: 144.9299926758,
		Timezone:  "Pacific/Guam",
	},
	"PGUM": {
		ICAO:      "PGUM",
		IATA:      "GUM",
		Name:      "Antonio B. Won Pat International Airport",
		City:      "Hagatna",
		State:     "Barrigada",
		Country:   "GU",
		Elevation: 298,
		Latitude:  13.4834003448,
		Longitude: 144.796005249,
		Timezone:  "Pacific/Guam",
	},
	"PGWT": {
		ICAO:      "PGWT",
		IATA:      "TIQ",
		Name:      "Tinian International Airport",
		City:      "Tinian Island",
		State:     "Tinian",
		Country:   "MP",
		Elevation: 271,
		Latitude:  14.9991998672,
		Longitude: 145.6190032959,
		Timezone:  "Pacific/Saipan",
	},
	"PHBK": {
		ICAO:      "PHBK",
		IATA:      "BKH",
		Name:      "Barking Sands Airport",
		City:      "Kekaha",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 23,
		Latitude:  22.0228004456,
		Longitude: -159.785003662,
		Timezone:  "Pacific/Honolulu",
	},
	"PHDH": {
		ICAO:      "PHDH",
		IATA:      "HDH",
		Name:      "Dillingham Airfield",
		City:      "Mokuleia",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 14,
		Latitude:  21.5795001984,
		Longitude: -158.197006226,
		Timezone:  "Pacific/Honolulu",
	},
	"PHHF": {
		ICAO:      "PHHF",
		Name:      "French Frigate Shoals Airport",
		City:      "Tern Island",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 6,
		Latitude:  23.8694000244,
		Longitude: -166.2850036621,
		Timezone:  "Pacific/Honolulu",
	},
	"PHHI": {
		ICAO:      "PHHI",
		IATA:      "HHI",
		Name:      "Wheeler Army Airfield",
		City:      "Wahiawa",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 837,
		Latitude:  21.48349953,
		Longitude: -158.0399933,
		Timezone:  "Pacific/Honolulu",
	},
	"PHHN": {
		ICAO:      "PHHN",
		IATA:      "HNM",
		Name:      "Hana Airport",
		City:      "Hana",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 78,
		Latitude:  20.7956008911,
		Longitude: -156.0140075684,
		Timezone:  "Pacific/Honolulu",
	},
	"PHJH": {
		ICAO:      "PHJH",
		IATA:      "JHM",
		Name:      "Kapalua Airport",
		City:      "Lahaina",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 256,
		Latitude:  20.9629001617,
		Longitude: -156.6730041504,
		Timezone:  "Pacific/Honolulu",
	},
	"PHJR": {
		ICAO:      "PHJR",
		IATA:      "JRF",
		Name:      "Kalaeloa (John Rodgers Field) Airport",
		City:      "Kapolei",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 30,
		Latitude:  21.30739975,
		Longitude: -158.0700073,
		Timezone:  "Pacific/Honolulu",
	},
	"PHKO": {
		ICAO:      "PHKO",
		IATA:      "KOA",
		Name:      "Kona International At Keahole Airport",
		City:      "Kailua/Kona",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 47,
		Latitude:  19.7388000488,
		Longitude: -156.046005249,
		Timezone:  "Pacific/Honolulu",
	},
	"PHLI": {
		ICAO:      "PHLI",
		IATA:      "LIH",
		Name:      "Lihue Airport",
		City:      "Lihue",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 153,
		Latitude:  21.9759998322,
		Longitude: -159.3390045166,
		Timezone:  "Pacific/Honolulu",
	},
	"PHLU": {
		ICAO:      "PHLU",
		IATA:      "LUP",
		Name:      "Kalaupapa Airport",
		City:      "Kalaupapa",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 24,
		Latitude:  21.21100044,
		Longitude: -156.973999,
		Timezone:  "Pacific/Honolulu",
	},
	"PHMK": {
		ICAO:      "PHMK",
		IATA:      "MKK",
		Name:      "Molokai Airport",
		City:      "Kaunakakai",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 454,
		Latitude:  21.1529006958,
		Longitude: -157.095993042,
		Timezone:  "Pacific/Honolulu",
	},
	"PHMU": {
		ICAO:      "PHMU",
		IATA:      "MUE",
		Name:      "Waimea Kohala Airport",
		City:      "Kamuela",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 2671,
		Latitude:  20.0013008118,
		Longitude: -155.6679992676,
		Timezone:  "Pacific/Honolulu",
	},
	"PHNG": {
		ICAO:      "PHNG",
		IATA:      "NGF",
		Name:      "Kaneohe Bay MCAS (Marion E. Carl Field) Airport",
		City:      "Kaneohe",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 24,
		Latitude:  21.4505004883,
		Longitude: -157.768005371,
		Timezone:  "Pacific/Honolulu",
	},
	"PHNL": {
		ICAO:      "PHNL",
		IATA:      "HNL",
		Name:      "Honolulu International Airport",
		City:      "Honolulu",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 13,
		Latitude:  21.3187007904,
		Longitude: -157.9219970703,
		Timezone:  "Pacific/Honolulu",
	},
	"PHNP": {
		ICAO:      "PHNP",
		Name:      "Ford Island Naval Auxiliary Landing Field",
		City:      "Honolulu",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 18,
		Latitude:  21.364900589,
		Longitude: -157.9600067139,
		Timezone:  "Pacific/Honolulu",
	},
	"PHNY": {
		ICAO:      "PHNY",
		IATA:      "LNY",
		Name:      "Lanai Airport",
		City:      "Lanai City",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 1308,
		Latitude:  20.7856006622,
		Longitude: -156.9510040283,
		Timezone:  "Pacific/Honolulu",
	},
	"PHOG": {
		ICAO:      "PHOG",
		IATA:      "OGG",
		Name:      "Kahului Airport",
		City:      "Kahului",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 54,
		Latitude:  20.8985996246,
		Longitude: -156.4299926758,
		Timezone:  "Pacific/Honolulu",
	},
	"PHPA": {
		ICAO:      "PHPA",
		IATA:      "PAK",
		Name:      "Port Allen Airport",
		City:      "Hanapepe",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 24,
		Latitude:  21.896900177,
		Longitude: -159.602996826,
		Timezone:  "Pacific/Honolulu",
	},
	"PHSF": {
		ICAO:      "PHSF",
		IATA:      "BSF",
		Name:      "Bradshaw Army Airfield",
		City:      "Camp Pohakuloa",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 6190,
		Latitude:  19.760099411,
		Longitude: -155.554000854,
		Timezone:  "Pacific/Honolulu",
	},
	"PHTO": {
		ICAO:      "PHTO",
		IATA:      "ITO",
		Name:      "Hilo International Airport",
		City:      "Hilo",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 38,
		Latitude:  19.7213993073,
		Longitude: -155.0480041504,
		Timezone:  "Pacific/Honolulu",
	},
	"PHUP": {
		ICAO:      "PHUP",
		IATA:      "UPP",
		Name:      "Upolu Airport",
		City:      "Hawi",
		State:     "Hawaii",
		Country:   "US",
		Elevation: 96,
		Latitude:  20.2653007507,
		Longitude: -155.8600006104,
		Timezone:  "Pacific/Honolulu",
	},
	"PKMA": {
		ICAO:      "PKMA",
		IATA:      "ENT",
		Name:      "Eniwetok Airport",
		City:      "Eniwetok Atoll",
		State:     "Enewetak-Atoll",
		Country:   "MH",
		Elevation: 13,
		Latitude:  11.3407001495,
		Longitude: 162.3280029297,
		Timezone:  "Pacific/Majuro",
	},
	"PKMJ": {
		ICAO:      "PKMJ",
		IATA:      "MAJ",
		Name:      "Marshall Islands International Airport",
		City:      "Majuro Atoll",
		State:     "Majuro-Atoll",
		Country:   "MH",
		Elevation: 6,
		Latitude:  7.0647602081,
		Longitude: 171.2720031738,
		Timezone:  "Pacific/Majuro",
	},
	"PKRO": {
		ICAO:      "PKRO",
		Name:      "Dyess Army Air Field",
		City:      "Roi-Namur",
		State:     "Kwajalein-Atoll",
		Country:   "MH",
		Elevation: 9,
		Latitude:  9.3968896866,
		Longitude: 167.470993042,
		Timezone:  "Pacific/Kwajalein",
	},
	"PKSA": {
		ICAO:      "PKSA",
		IATA:      "KIA",
		Name:      "Kaieteur International Airport",
		City:      "Kaieteur Falls",
		State:     "Potaro-Siparuni",
		Country:   "GY",
		Latitude:  5.1727547646,
		Longitude: -59.491481781,
		Timezone:  "America/Guyana",
	},
	"PKWA": {
		ICAO:      "PKWA",
		IATA:      "KWA",
		Name:      "Bucholz Army Air Field",
		City:      "Kwajalein",
		State:     "Kwajalein-Atoll",
		Country:   "MH",
		Elevation: 9,
		Latitude:  8.72012043,
		Longitude: 167.7319946289,
		Timezone:  "Pacific/Kwajalein",
	},
	"PLCH": {
		ICAO:      "PLCH",
		IATA:      "CXI",
		Name:      "Cassidy International Airport",
		City:      "Banana",
		State:     "Line-Islands",
		Country:   "KI",
		Elevation: 5,
		Latitude:  1.9861600399,
		Longitude: -157.3500061035,
		Timezone:  "Pacific/Kiritimati",
	},
	"PLFA": {
		ICAO:      "PLFA",
		IATA:      "TNV",
		Name:      "Tabuaeran Island Airport",
		City:      "Tabuaeran Island",
		State:     "Line-Islands",
		Country:   "KI",
		Latitude:  3.8994400501,
		Longitude: -159.3890075684,
		Timezone:  "Pacific/Kiritimati",
	},
	"PLPA": {
		ICAO:      "PLPA",
		Name:      "Palmyra (Cooper) Airport",
		City:      "Palmyra Island Atoll",
		State:     "Palmyra-Atoll",
		Country:   "UM",
		Elevation: 5,
		Latitude:  5.8887481689,
		Longitude: -162.0778656006,
		Timezone:  "Etc/GMT+11",
	},
	"PLWN": {
		ICAO:      "PLWN",
		Name:      "Washington Island Airstrip",
		City:      "Teraina",
		State:     "Line-Islands",
		Country:   "KI",
		Elevation: 30,
		Latitude:  4.698359,
		Longitude: -160.394376,
		Timezone:  "Pacific/Kiritimati",
	},
	"PMDY": {
		ICAO:      "PMDY",
		IATA:      "MDY",
		Name:      "Henderson Field",
		City:      "Sand Island",
		State:     "Midway-Islands",
		Country:   "UM",
		Elevation: 13,
		Latitude:  28.2017002106,
		Longitude: -177.3809967041,
		Timezone:  "Pacific/Midway",
	},
	"PN00": {
		ICAO:      "PN00",
		Name:      "Crosswinds Airfield",
		City:      "Sassamansville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 300,
		Latitude:  40.3432006836,
		Longitude: -75.5727005005,
		Timezone:  "America/New_York",
	},
	"PN01": {
		ICAO:      "PN01",
		Name:      "Cedar Run Airport",
		City:      "Dorseyville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1180,
		Latitude:  40.5881004333,
		Longitude: -79.9058990479,
		Timezone:  "America/New_York",
	},
	"PN02": {
		ICAO:      "PN02",
		Name:      "Offutt Acres Airport",
		City:      "East Brady",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 865,
		Latitude:  40.9847984314,
		Longitude: -79.5663986206,
		Timezone:  "America/New_York",
	},
	"PN04": {
		ICAO:      "PN04",
		Name:      "Strittmatter Airport",
		City:      "Ebensburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 2080,
		Latitude:  40.547000885,
		Longitude: -78.6947021484,
		Timezone:  "America/New_York",
	},
	"PN06": {
		ICAO:      "PN06",
		Name:      "Carlson Airport",
		City:      "Edinboro",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1490,
		Latitude:  41.9042015076,
		Longitude: -80.1026000977,
		Timezone:  "America/New_York",
	},
	"PN08": {
		ICAO:      "PN08",
		Name:      "Davis Airport",
		City:      "Chambersville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1373,
		Latitude:  40.7346000671,
		Longitude: -79.1598968506,
		Timezone:  "America/New_York",
	},
	"PN10": {
		ICAO:      "PN10",
		Name:      "Cash Creek Airport",
		City:      "Ulster",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1250,
		Latitude:  41.8651008606,
		Longitude: -76.5099029541,
		Timezone:  "America/New_York",
	},
	"PN11": {
		ICAO:      "PN11",
		Name:      "Reno Airport",
		City:      "Enon Valley",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1160,
		Latitude:  40.8898010254,
		Longitude: -80.4865036011,
		Timezone:  "America/New_York",
	},
	"PN12": {
		ICAO:      "PN12",
		Name:      "Sweet Valley Airport",
		City:      "Sweet Valley",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1200,
		Latitude:  41.2806015015,
		Longitude: -78.1430969238,
		Timezone:  "America/New_York",
	},
	"PN13": {
		ICAO:      "PN13",
		Name:      "C & W Milliron Flying Field",
		City:      "Sigel",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1700,
		Latitude:  41.2873001099,
		Longitude: -79.1663970947,
		Timezone:  "America/New_York",
	},
	"PN14": {
		ICAO:      "PN14",
		Name:      "Misty Hill Farm Airport",
		City:      "Ren Frew",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1243,
		Latitude:  40.7893981934,
		Longitude: -79.9974975586,
		Timezone:  "America/New_York",
	},
	"PN15": {
		ICAO:      "PN15",
		Name:      "Greeley Airport",
		City:      "Coudersport",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 2300,
		Latitude:  41.8791999817,
		Longitude: -77.9499969482,
		Timezone:  "America/New_York",
	},
	"PN16": {
		ICAO:      "PN16",
		Name:      "Pabst Blue Ribbon Airport",
		City:      "Evans City",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1250,
		Latitude:  40.7583999634,
		Longitude: -80.0137023926,
		Timezone:  "America/New_York",
	},
	"PN17": {
		ICAO:      "PN17",
		Name:      "Marsteller Airport",
		City:      "Stewartstown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 780,
		Latitude:  39.7333984375,
		Longitude: -76.5329971313,
		Timezone:  "America/New_York",
	},
	"PN18": {
		ICAO:      "PN18",
		Name:      "Fairview Evergreen Airport",
		City:      "Fairview",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 805,
		Latitude:  42.0273017883,
		Longitude: -80.2436981201,
		Timezone:  "America/New_York",
	},
	"PN21": {
		ICAO:      "PN21",
		Name:      "Mc Clure Airport",
		City:      "Troy",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1470,
		Latitude:  41.8278999329,
		Longitude: -76.8565979004,
		Timezone:  "America/New_York",
	},
	"PN25": {
		ICAO:      "PN25",
		Name:      "Lindsay Airport",
		City:      "Freeport",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1080,
		Latitude:  40.7355995178,
		Longitude: -79.6886978149,
		Timezone:  "America/New_York",
	},
	"PN30": {
		ICAO:      "PN30",
		Name:      "Schrenkel Airport",
		City:      "Cassville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1200,
		Latitude:  40.2627983093,
		Longitude: -78.1025009155,
		Timezone:  "America/New_York",
	},
	"PN31": {
		ICAO:      "PN31",
		Name:      "Sagulla Airport",
		City:      "Grove City",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1378,
		Latitude:  41.2430992126,
		Longitude: -80.1141967773,
		Timezone:  "America/New_York",
	},
	"PN33": {
		ICAO:      "PN33",
		Name:      "Branning Field",
		City:      "Honesdale",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1420,
		Latitude:  41.552898407,
		Longitude: -75.2820968628,
		Timezone:  "America/New_York",
	},
	"PN35": {
		ICAO:      "PN35",
		Name:      "Flying R Airport",
		City:      "Harrisonville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 960,
		Latitude:  40.0000991821,
		Longitude: -78.0830993652,
		Timezone:  "America/New_York",
	},
	"PN36": {
		ICAO:      "PN36",
		Name:      "Rayne Airport",
		City:      "Home",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1420,
		Latitude:  40.7333984375,
		Longitude: -79.0672988892,
		Timezone:  "America/New_York",
	},
	"PN37": {
		ICAO:      "PN37",
		Name:      "Fino Airport",
		City:      "Hookstown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1150,
		Latitude:  40.6050987244,
		Longitude: -80.4580993652,
		Timezone:  "America/New_York",
	},
	"PN38": {
		ICAO:      "PN38",
		Name:      "Culmerville Airport",
		City:      "Culmerville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1220,
		Latitude:  40.6626014709,
		Longitude: -79.8455963135,
		Timezone:  "America/New_York",
	},
	"PN40": {
		ICAO:      "PN40",
		Name:      "Moorhead Airpark LLC",
		City:      "North East",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 805,
		Latitude:  42.1842002869,
		Longitude: -79.8975982666,
		Timezone:  "America/New_York",
	},
	"PN43": {
		ICAO:      "PN43",
		Name:      "Sainovich Airport",
		City:      "Industry",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1275,
		Latitude:  40.7000999451,
		Longitude: -80.4330978394,
		Timezone:  "America/New_York",
	},
	"PN46": {
		ICAO:      "PN46",
		Name:      "River Hill Aviation Airport",
		City:      "Apollo",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1250,
		Latitude:  40.5643997192,
		Longitude: -79.5457992554,
		Timezone:  "America/New_York",
	},
	"PN47": {
		ICAO:      "PN47",
		Name:      "Lohr's Landing Airport",
		City:      "Jennerstown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1970,
		Latitude:  40.2083015442,
		Longitude: -79.0558013916,
		Timezone:  "America/New_York",
	},
	"PN48": {
		ICAO:      "PN48",
		Name:      "Bullfly Ultralightport",
		City:      "Germansville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 812,
		Latitude:  40.7083015442,
		Longitude: -75.6860961914,
		Timezone:  "America/New_York",
	},
	"PN49": {
		ICAO:      "PN49",
		Name:      "Dunbar Airport",
		City:      "Kittanning",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1500,
		Latitude:  40.7719993591,
		Longitude: -79.4289016724,
		Timezone:  "America/New_York",
	},
	"PN50": {
		ICAO:      "PN50",
		Name:      "Skyline Airstrip",
		City:      "Shoemakersville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 580,
		Latitude:  40.5065002441,
		Longitude: -75.9432983398,
		Timezone:  "America/New_York",
	},
	"PN53": {
		ICAO:      "PN53",
		Name:      "Strohmier Airport",
		City:      "Loretto",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1962,
		Latitude:  40.5514984131,
		Longitude: -78.6233978271,
		Timezone:  "America/New_York",
	},
	"PN54": {
		ICAO:      "PN54",
		Name:      "Akm Airfield",
		City:      "Columia Cross Roads",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1393,
		Latitude:  41.8417015076,
		Longitude: -76.7457962036,
		Timezone:  "America/New_York",
	},
	"PN55": {
		ICAO:      "PN55",
		Name:      "Skala Airport",
		City:      "Mc Clellandtown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1270,
		Latitude:  39.8583984375,
		Longitude: -79.8234024048,
		Timezone:  "America/New_York",
	},
	"PN56": {
		ICAO:      "PN56",
		Name:      "Miller Airport",
		City:      "Mahaffey",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1540,
		Latitude:  40.8852996826,
		Longitude: -78.7391967773,
		Timezone:  "America/New_York",
	},
	"PN57": {
		ICAO:      "PN57",
		Name:      "Marion Center Speedway Airport",
		City:      "Marion Center",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1500,
		Latitude:  40.8120002747,
		Longitude: -79.0373001099,
		Timezone:  "America/New_York",
	},
	"PN58": {
		ICAO:      "PN58",
		Name:      "Bittner/Whitsel Airport",
		City:      "Waynesboro",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 760,
		Latitude:  39.7426986694,
		Longitude: -77.526802063,
		Timezone:  "America/New_York",
	},
	"PN59": {
		ICAO:      "PN59",
		Name:      "Stitt Airport",
		City:      "Kittanning",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1200,
		Latitude:  40.8311004639,
		Longitude: -79.4869003296,
		Timezone:  "America/New_York",
	},
	"PN62": {
		ICAO:      "PN62",
		Name:      "Cataney Airport",
		City:      "Midway",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1200,
		Latitude:  40.3750991821,
		Longitude: -80.2997970581,
		Timezone:  "America/New_York",
	},
	"PN64": {
		ICAO:      "PN64",
		Name:      "Hilling International Airport",
		City:      "Huntingdon",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1400,
		Latitude:  40.5023994446,
		Longitude: -77.9393997192,
		Timezone:  "America/New_York",
	},
	"PN66": {
		ICAO:      "PN66",
		Name:      "Dunlea Airpark",
		City:      "New Alexandria",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1220,
		Latitude:  40.4255981445,
		Longitude: -79.3636016846,
		Timezone:  "America/New_York",
	},
	"PN68": {
		ICAO:      "PN68",
		Name:      "Jackson Airport",
		City:      "New Galilee",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 930,
		Latitude:  40.9009017944,
		Longitude: -80.3681030273,
		Timezone:  "America/New_York",
	},
	"PN69": {
		ICAO:      "PN69",
		Name:      "Altemose Ultralightport",
		City:      "Malvern",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 345,
		Latitude:  40.0924987793,
		Longitude: -75.5158004761,
		Timezone:  "America/New_York",
	},
	"PN71": {
		ICAO:      "PN71",
		Name:      "Jansen Vineyards Airport",
		City:      "Richfield",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 860,
		Latitude:  40.6500015259,
		Longitude: -77.0575027466,
		Timezone:  "America/New_York",
	},
	"PN72": {
		ICAO:      "PN72",
		Name:      "Rocky Hollow Field",
		City:      "Strongstown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1806,
		Latitude:  40.5691986084,
		Longitude: -78.8824996948,
		Timezone:  "America/New_York",
	},
	"PN73": {
		ICAO:      "PN73",
		Name:      "Beers Farm Airport",
		City:      "Orbisona/Rockhill",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 630,
		Latitude:  40.2300987244,
		Longitude: -77.9206008911,
		Timezone:  "America/New_York",
	},
	"PN83": {
		ICAO:      "PN83",
		Name:      "Skunk Hollow Airport",
		City:      "Plumville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1380,
		Latitude:  40.8027992249,
		Longitude: -79.2024993896,
		Timezone:  "America/New_York",
	},
	"PN84": {
		ICAO:      "PN84",
		Name:      "Lake Arthur Field",
		City:      "Portersville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1240,
		Latitude:  40.984500885,
		Longitude: -80.1620025635,
		Timezone:  "America/New_York",
	},
	"PN85": {
		ICAO:      "PN85",
		Name:      "Brandon Airport",
		City:      "Prospect",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1290,
		Latitude:  40.876499176,
		Longitude: -80.0330963135,
		Timezone:  "America/New_York",
	},
	"PN90": {
		ICAO:      "PN90",
		Name:      "Ranch-Aero Airport",
		City:      "Roulette",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1575,
		Latitude:  41.7486991882,
		Longitude: -78.0997009277,
		Timezone:  "America/New_York",
	},
	"PN91": {
		ICAO:      "PN91",
		Name:      "Sharretts Airport",
		City:      "Sabinsville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 2360,
		Latitude:  41.8334007263,
		Longitude: -77.4997024536,
		Timezone:  "America/New_York",
	},
	"PN95": {
		ICAO:      "PN95",
		Name:      "Horne Airport",
		City:      "Scenery Hill",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1040,
		Latitude:  40.1001014709,
		Longitude: -80.0330963135,
		Timezone:  "America/New_York",
	},
	"PN99": {
		ICAO:      "PN99",
		Name:      "Lackawannock Airport",
		City:      "Sharon",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1300,
		Latitude:  41.2058982849,
		Longitude: -80.3581008911,
		Timezone:  "America/New_York",
	},
	"POLI": {
		ICAO:      "POLI",
		Name:      "Oliktok LRRS Airport",
		City:      "Oliktok Point",
		State:     "Alaska",
		Country:   "US",
		Elevation: 16,
		Latitude:  70.49971,
		Longitude: -149.879535,
		Timezone:  "America/Anchorage",
	},
	"PPIZ": {
		ICAO:      "PPIZ",
		IATA:      "PIZ",
		Name:      "Point Lay Lrrs Airport",
		City:      "Point Lay",
		State:     "Alaska",
		Country:   "US",
		Elevation: 22,
		Latitude:  69.73290253,
		Longitude: -163.0050049,
		Timezone:  "America/Nome",
	},
	"PR07": {
		ICAO:      "PR07",
		Name:      "Boqueron Airport",
		City:      "Las Piedras",
		State:     "Las-Piedras",
		Country:   "PR",
		Elevation: 373,
		Latitude:  18.2019004822,
		Longitude: -65.8391036987,
		Timezone:  "America/Puerto_Rico",
	},
	"PR10": {
		ICAO:      "PR10",
		Name:      "Boqueron Airport",
		City:      "Cabo Rojo",
		State:     "Cabo-Rojo",
		Country:   "PR",
		Elevation: 3,
		Latitude:  18.0107002258,
		Longitude: -67.1412963867,
		Timezone:  "America/Puerto_Rico",
	},
	"PR20": {
		ICAO:      "PR20",
		Name:      "Michael Gonzalez Airport",
		City:      "Adjuntas",
		State:     "Adjuntas",
		Country:   "PR",
		Elevation: 2340,
		Latitude:  18.1802005768,
		Longitude: -66.7568969727,
		Timezone:  "America/Puerto_Rico",
	},
	"PR24": {
		ICAO:      "PR24",
		Name:      "Cullingford Field",
		City:      "Boqueron",
		State:     "Cabo-Rojo",
		Country:   "PR",
		Elevation: 14,
		Latitude:  17.9764003754,
		Longitude: -67.1707992554,
		Timezone:  "America/Puerto_Rico",
	},
	"PR27": {
		ICAO:      "PR27",
		Name:      "Santa Isabel Airport",
		City:      "Santa Isabel",
		State:     "Santa-Isabel-Municipio",
		Country:   "PR",
		Elevation: 27,
		Latitude:  17.9631996155,
		Longitude: -66.3911972046,
		Timezone:  "America/Puerto_Rico",
	},
	"PS00": {
		ICAO:      "PS00",
		Name:      "Tallman West Airport",
		City:      "Dauphin",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 480,
		Latitude:  40.4056015015,
		Longitude: -76.9400024414,
		Timezone:  "America/New_York",
	},
	"PS02": {
		ICAO:      "PS02",
		Name:      "Heberlig Airport",
		City:      "Newville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 560,
		Latitude:  40.1642990112,
		Longitude: -77.3704986572,
		Timezone:  "America/New_York",
	},
	"PS03": {
		ICAO:      "PS03",
		Name:      "Elephant Path Airport",
		City:      "Perkasie",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 370,
		Latitude:  40.4104003906,
		Longitude: -75.2293014526,
		Timezone:  "America/New_York",
	},
	"PS05": {
		ICAO:      "PS05",
		Name:      "G & N Airport",
		City:      "Albion",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 985,
		Latitude:  41.8753013611,
		Longitude: -80.3748016357,
		Timezone:  "America/New_York",
	},
	"PS06": {
		ICAO:      "PS06",
		Name:      "Snook Airport",
		City:      "Beaver Springs",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 710,
		Latitude:  40.7333984375,
		Longitude: -77.2664031982,
		Timezone:  "America/New_York",
	},
	"PS08": {
		ICAO:      "PS08",
		Name:      "Lazy B Ranch Airport",
		City:      "Dover",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 450,
		Latitude:  40.0225982666,
		Longitude: -76.8165969849,
		Timezone:  "America/New_York",
	},
	"PS11": {
		ICAO:      "PS11",
		Name:      "Mc Cardle Farm Airport",
		City:      "Burnham",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 600,
		Latitude:  40.6453018188,
		Longitude: -77.5552978516,
		Timezone:  "America/New_York",
	},
	"PS12": {
		ICAO:      "PS12",
		Name:      "Grover Airport",
		City:      "Weatherly",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1223,
		Latitude:  40.9168014526,
		Longitude: -75.8663024902,
		Timezone:  "America/New_York",
	},
	"PS15": {
		ICAO:      "PS15",
		Name:      "Vicars Private Airport",
		City:      "Altoona",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1370,
		Latitude:  40.5475997925,
		Longitude: -78.3917007446,
		Timezone:  "America/New_York",
	},
	"PS18": {
		ICAO:      "PS18",
		Name:      "Pecora Field",
		City:      "Bradford",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1500,
		Latitude:  41.9037017822,
		Longitude: -78.6275024414,
		Timezone:  "America/New_York",
	},
	"PS20": {
		ICAO:      "PS20",
		Name:      "Fairview Farm Airfield",
		City:      "Pine Grove",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 600,
		Latitude:  40.5334014893,
		Longitude: -76.4621963501,
		Timezone:  "America/New_York",
	},
	"PS21": {
		ICAO:      "PS21",
		Name:      "Flying Eagle Airport",
		City:      "Dalmatia",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 660,
		Latitude:  40.6459007263,
		Longitude: -76.8122024536,
		Timezone:  "America/New_York",
	},
	"PS23": {
		ICAO:      "PS23",
		Name:      "Windy Hill Airport",
		City:      "Curwensville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1622,
		Latitude:  40.0850982666,
		Longitude: -78.5475006104,
		Timezone:  "America/New_York",
	},
	"PS25": {
		ICAO:      "PS25",
		Name:      "Captain's Folly Airport",
		City:      "Gettysburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 641,
		Latitude:  39.8433990479,
		Longitude: -77.1647033691,
		Timezone:  "America/New_York",
	},
	"PS27": {
		ICAO:      "PS27",
		Name:      "Juergensen Airpark and Maritime Facility Airport",
		City:      "Addison",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1890,
		Latitude:  39.7313995361,
		Longitude: -79.375,
		Timezone:  "America/New_York",
	},
	"PS35": {
		ICAO:      "PS35",
		Name:      "Dutch Country Egg Farms Airport",
		City:      "Fredericksburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 550,
		Latitude:  40.4580993652,
		Longitude: -76.4580001831,
		Timezone:  "America/New_York",
	},
	"PS38": {
		ICAO:      "PS38",
		Name:      "Bert's Airport",
		City:      "Birdsboro - Pottstown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 240,
		Latitude:  40.2855987549,
		Longitude: -75.7481994629,
		Timezone:  "America/New_York",
	},
	"PS39": {
		ICAO:      "PS39",
		Name:      "Downes Airport",
		City:      "Clymer",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1590,
		Latitude:  40.6282997131,
		Longitude: -78.9602966309,
		Timezone:  "America/New_York",
	},
	"PS43": {
		ICAO:      "PS43",
		Name:      "Barnes Farmland Airport",
		City:      "Volant",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1280,
		Latitude:  41.0680999756,
		Longitude: -80.255897522,
		Timezone:  "America/New_York",
	},
	"PS46": {
		ICAO:      "PS46",
		Name:      "Graystrip Airport",
		City:      "Dublin",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 642,
		Latitude:  40.3623008728,
		Longitude: -75.1801986694,
		Timezone:  "America/New_York",
	},
	"PS47": {
		ICAO:      "PS47",
		Name:      "Hamilton Hill Airport",
		City:      "Ebensburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 2245,
		Latitude:  40.427898407,
		Longitude: -78.7414016724,
		Timezone:  "America/New_York",
	},
	"PS49": {
		ICAO:      "PS49",
		Name:      "Hartman Airport",
		City:      "Garrett",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 2350,
		Latitude:  39.8604011536,
		Longitude: -78.9886016846,
		Timezone:  "America/New_York",
	},
	"PS50": {
		ICAO:      "PS50",
		Name:      "Huf Airport",
		City:      "Harford",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1600,
		Latitude:  41.7678985596,
		Longitude: -75.723197937,
		Timezone:  "America/New_York",
	},
	"PS52": {
		ICAO:      "PS52",
		Name:      "Alberter Farms Airport",
		City:      "Johnstown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 2070,
		Latitude:  40.195098877,
		Longitude: -78.8328018188,
		Timezone:  "America/New_York",
	},
	"PS53": {
		ICAO:      "PS53",
		Name:      "7-H Skeet Club Inc. Field",
		City:      "Patton",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1605,
		Latitude:  40.638999939,
		Longitude: -78.5813980103,
		Timezone:  "America/New_York",
	},
	"PS54": {
		ICAO:      "PS54",
		Name:      "Gunden Airport",
		City:      "Perkasie",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 530,
		Latitude:  40.3292999268,
		Longitude: -75.2816009521,
		Timezone:  "America/New_York",
	},
	"PS61": {
		ICAO:      "PS61",
		Name:      "Tidioute Airport",
		City:      "Tidioute",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1100,
		Latitude:  41.6908988953,
		Longitude: -79.3658981323,
		Timezone:  "America/New_York",
	},
	"PS64": {
		ICAO:      "PS64",
		Name:      "Morton's Airport",
		City:      "Cambridge Springs",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1200,
		Latitude:  41.8250999451,
		Longitude: -80.0789031982,
		Timezone:  "America/New_York",
	},
	"PS66": {
		ICAO:      "PS66",
		Name:      "Parker-Cramer Airport",
		City:      "Clarion",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1480,
		Latitude:  41.2114982605,
		Longitude: -79.3609008789,
		Timezone:  "America/New_York",
	},
	"PS67": {
		ICAO:      "PS67",
		Name:      "Uphill Airport",
		City:      "Claysville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1360,
		Latitude:  40.1062011719,
		Longitude: -80.4309005737,
		Timezone:  "America/New_York",
	},
	"PS68": {
		ICAO:      "PS68",
		Name:      "Buckingham Airport",
		City:      "Doylestown",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 250,
		Latitude:  40.3322982788,
		Longitude: -75.0304031372,
		Timezone:  "America/New_York",
	},
	"PS69": {
		ICAO:      "PS69",
		Name:      "Barnhart Airport",
		City:      "Flinton",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1640,
		Latitude:  40.7270011902,
		Longitude: -78.5394973755,
		Timezone:  "America/New_York",
	},
	"PS70": {
		ICAO:      "PS70",
		Name:      "Cider Field",
		City:      "Friedensburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 770,
		Latitude:  40.6058998108,
		Longitude: -76.2829971313,
		Timezone:  "America/New_York",
	},
	"PS73": {
		ICAO:      "PS73",
		Name:      "Poverty Airport",
		City:      "Mackeyville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 860,
		Latitude:  41.0634002686,
		Longitude: -77.4336013794,
		Timezone:  "America/New_York",
	},
	"PS82": {
		ICAO:      "PS82",
		Name:      "Lazy J. Ranch Airport",
		City:      "Springbrook",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1650,
		Latitude:  41.2958984375,
		Longitude: -75.5977020264,
		Timezone:  "America/New_York",
	},
	"PS87": {
		ICAO:      "PS87",
		Name:      "Home Safe Airport",
		City:      "Ellsworth/Bentleyville",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1140,
		Latitude:  40.1058998108,
		Longitude: -80.0338973999,
		Timezone:  "America/New_York",
	},
	"PS98": {
		ICAO:      "PS98",
		Name:      "Travis Airport",
		City:      "Smicksburg",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1320,
		Latitude:  40.8828010559,
		Longitude: -79.1965026855,
		Timezone:  "America/New_York",
	},
	"PS99": {
		ICAO:      "PS99",
		Name:      "West Penn Township Airport",
		City:      "Tamaqua",
		State:     "Pennsylvania",
		Country:   "US",
		Elevation: 1200,
		Latitude:  40.7478981018,
		Longitude: -75.9329986572,
		Timezone:  "America/New_York",
	},
	"PTKK": {
		ICAO:      "PTKK",
		IATA:      "TKK",
		Name:      "Chuuk International Airport",
		City:      "Weno Island",
		State:     "Chuuk",
		Country:   "FM",
		Elevation: 11,
		Latitude:  7.4618701935,
		Longitude: 151.8430023193,
		Timezone:  "Pacific/Chuuk",
	},
	"PTPN": {
		ICAO:      "PTPN",
		IATA:      "PNI",
		Name:      "Pohnpei International Airport",
		City:      "Pohnpei Island",
		State:     "Pohnpei",
		Country:   "FM",
		Elevation: 10,
		Latitude:  6.9850997925,
		Longitude: 158.2089996338,
		Timezone:  "Pacific/Pohnpei",
	},
	"PTRO": {
		ICAO:      "PTRO",
		IATA:      "ROR",
		Name:      "Babelthuap Airport",
		City:      "Babelthuap Island",
		State:     "Airai",
		Country:   "PW",
		Elevation: 176,
		Latitude:  7.367650032,
		Longitude: 134.5440063477,
		Timezone:  "Pacific/Palau",
	},
	"PTSA": {
		ICAO:      "PTSA",
		IATA:      "KSA",
		Name:      "Kosrae International Airport",
		City:      "Okat",
		State:     "Kosrae",
		Country:   "FM",
		Elevation: 11,
		Latitude:  5.356979847,
		Longitude: 162.957992554,
		Timezone:  "Pacific/Kosrae",
	},
	"PTYA": {
		ICAO:      "PTYA",
		IATA:      "YAP",
		Name:      "Yap International Airport",
		City:      "Yap Island",
		State:     "Yap",
		Country:   "FM",
		Elevation: 91,
		Latitude:  9.4989099503,
		Longitude: 138.0829925537,
		Timezone:  "Pacific/Chuuk",
	},
	"PWAK": {
		ICAO:      "PWAK",
		IATA:      "AWK",
		Name:      "Wake Island Airfield",
		City:      "Wake Island",
		State:     "Wake-Island",
		Country:   "UM",
		Elevation: 14,
		Latitude:  19.2821006775,
		Longitude: 166.6360015869,
		Timezone:  "Pacific/Wake",
	},
	"RBLA": {
		ICAO:      "RBLA",
		Name:      "Sierra Blanca Airport",
		City:      "Sierra Blanca",
		State:     "Texas",
		Country:   "US",
		Elevation: 4512,
		Latitude:  31.2583007812,
		Longitude: -105.297996521,
		Timezone:  "America/Denver",
	},
	"RCAY": {
		ICAO:      "RCAY",
		Name:      "Gangshan Air Force Base",
		City:      "Gangshan",
		State:     "Takao",
		Country:   "TW",
		Elevation: 34,
		Latitude:  22.7824993134,
		Longitude: 120.2630004883,
		Timezone:  "Asia/Taipei",
	},
	"RCBS": {
		ICAO:      "RCBS",
		IATA:      "KNH",
		Name:      "Kinmen Airport",
		City:      "Shang-I",
		State:     "Fukien",
		Country:   "TW",
		Elevation: 93,
		Latitude:  24.4279003143,
		Longitude: 118.3590011597,
		Timezone:  "Asia/Taipei",
	},
	"RCDC": {
		ICAO:      "RCDC",
		IATA:      "PIF",
		Name:      "Pingtung South Airport",
		City:      "Pingtung",
		State:     "Taiwan",
		Country:   "TW",
		Elevation: 78,
		Latitude:  22.6723995209,
		Longitude: 120.4619979858,
		Timezone:  "Asia/Taipei",
	},
	"RCDI": {
		ICAO:      "RCDI",
		Name:      "Longtan Air Base",
		City:      "Longtan Township",
		State:     "Taiwan",
		Country:   "TW",
		Elevation: 790,
		Latitude:  24.8551006317,
		Longitude: 121.2379989624,
		Timezone:  "Asia/Taipei",
	},
	"RCFG": {
		ICAO:      "RCFG",
		IATA:      "LZN",
		Name:      "Matsu Nangan Airport",
		City:      "Nangang Island",
		State:     "Fukien",
		Country:   "TW",
		Elevation: 232,
		Latitude:  26.1597995758,
		Longitude: 119.9580001831,
		Timezone:  "Asia/Taipei",
	},
	"RCFN": {
		ICAO:      "RCFN",
		IATA:      "TTT",
		Name:      "Taitung Airport",
		City:      "Taitung City",
		State:     "Taiwan",
		Country:   "TW",
		Elevation: 143,
		Latitude:  22.7549991608,
		Longitude: 121.1019973755,
		Timezone:  "Asia/Taipei",
	},
	"RCGI": {
		ICAO:      "RCGI",
		IATA:      "GNI",
		Name:      "Lyudao Airport",
		City:      "Lyudao",
		State:     "Taiwan",
		Country:   "TW",
		Elevation: 28,
		Latitude:  22.6739006042,
		Longitude: 121.466003418,
		Timezone:  "Asia/Taipei",
	},
	"RCGM": {
		ICAO:      "RCGM",
		Name:      "Taoyuan Air Base",
		City:      "Taoyuan City",
		State:     "Taiwan",
		Country:   "TW",
		Elevation: 144,
		Latitude:  25.0566005707,
		Longitude: 121.2442626953,
		Timezone:  "Asia/Taipei",
	},
	"RCKH": {
		ICAO:      "RCKH",
		IATA:      "KHH",
		Name:      "Kaohsiung International Airport",
		City:      "Kaohsiung City",
		State:     "Takao",
		Country:   "TW",
		Elevation: 31,
		Latitude:  22.5771007538,
		Longitude: 120.3499984741,
		Timezone:  "Asia/Taipei",
	},
	"RCKU": {
		ICAO:      "RCKU",
		IATA:      "CYI",
		Name:      "Chiayi Airport",
		City:      "Chiayi City",
		State:     "Taiwan",
		Country:   "TW",
		Elevation: 85,
		Latitude:  23.4617996216,
		Longitude: 120.3929977417,
		Timezone:  "Asia/Taipei",
	},
	"RCKW": {
		ICAO:      "RCKW",
		IATA:      "HCN",
		Name:      "Hengchun Airport",
		City:      "Hengchung",
		State:     "Taiwan",
		Country:   "TW",
		Elevation: 46,
		Latitude:  22.0410995483,
		Longitude: 120.7300033569,
		Timezone:  "Asia/Taipei",
	},
	"RCLG": {
		ICAO:      "RCLG",
		IATA:      "TXG",
		Name:      "Taichung Airport",
		City:      "Taichung City",
		State:     "Taiwan",
		Country:   "TW",
		Elevation: 369,
		Latitude:  24.1863002777,
		Longitude: 120.6539993286,
		Timezone:  "Asia/Taipei",
	},
	"RCLY": {
		ICAO:      "RCLY",
		IATA:      "KYD",
		Name:      "Lanyu Airport",
		City:      "Orchid Island",
		State:     "Taiwan",
		Country:   "TW",
		Elevation: 44,
		Latitude:  22.0270004272,
		Longitude: 121.5350036621,
		Timezone:  "Asia/Taipei",
	},
	"RCMQ": {
		ICAO:      "RCMQ",
		IATA:      "RMQ",
		Name:      "Taichung Ching Chuang Kang Airport",
		City:      "Taichung City",
		State:     "Taiwan",
		Country:   "TW",
		Elevation: 663,
		Latitude:  24.2646999359,
		Longitude: 120.6210021973,
		Timezone:  "Asia/Taipei",
	},
	"RCMT": {
		ICAO:      "RCMT",
		IATA:      "MFK",
		Name:      "Matsu Beigan Airport",
		City:      "Beigan Island",
		State:     "Fukien",
		Country:   "TW",
		Elevation: 41,
		Latitude:  26.224199295,
		Longitude: 120.0029983521,
		Timezone:  "Asia/Taipei",
	},
	"RCNN": {
		ICAO:      "RCNN",
		IATA:      "TNN",
		Name:      "Tainan Airport",
		City:      "Tainan City",
		State:     "Taiwan",
		Country:   "TW",
		Elevation: 63,
		Latitude:  22.9503993988,
		Longitude: 120.2060012817,
		Timezone:  "Asia/Taipei",
	},
	"RCPO": {
		ICAO:      "RCPO",
		IATA:      "HSZ",
		Name:      "Hsinchu Air Base",
		City:      "Hsinchu City",
		State:     "Taiwan",
		Country:   "TW",
		Elevation: 26,
		Latitude:  24.8180007935,
		Longitude: 120.9390029907,
		Timezone:  "Asia/Taipei",
	},
	"RCQC": {
		ICAO:      "RCQC",
		IATA:      "MZG",
		Name:      "Makung Airport",
		City:      "Makung City",
		State:     "Taiwan",
		Country:   "TW",
		Elevation: 103,
		Latitude:  23.5687007904,
		Longitude: 119.6279983521,
		Timezone:  "Asia/Taipei",
	},
	"RCQS": {
		ICAO:      "RCQS",
		Name:      "Chihhang Air Base",
		City:      "Taitung City",
		State:     "Taiwan",
		Country:   "TW",
		Elevation: 121,
		Latitude:  22.7931003571,
		Longitude: 121.1819992065,
		Timezone:  "Asia/Taipei",
	},
	"RCSQ": {
		ICAO:      "RCSQ",
		IATA:      "PIF",
		Name:      "Pingtung North Airport",
		City:      "Pingtung",
		State:     "Taiwan",
		Country:   "TW",
		Elevation: 97,
		Latitude:  22.7001991272,
		Longitude: 120.4820022583,
		Timezone:  "Asia/Taipei",
	},
	"RCSS": {
		ICAO:      "RCSS",
		IATA:      "TSA",
		Name:      "Taipei Songshan Airport",
		City:      "Taipei City",
		State:     "Taiwan",
		Country:   "TW",
		Elevation: 18,
		Latitude:  25.0694007874,
		Longitude: 121.5520019531,
		Timezone:  "Asia/Taipei",
	},
	"RCTP": {
		ICAO:      "RCTP",
		IATA:      "TPE",
		Name:      "Taiwan Taoyuan International Airport",
		City:      "Taipei",
		State:     "Taiwan",
		Country:   "TW",
		Elevation: 106,
		Latitude:  25.0776996613,
		Longitude: 121.233001709,
		Timezone:  "Asia/Taipei",
	},
	"RCWA": {
		ICAO:      "RCWA",
		IATA:      "WOT",
		Name:      "Wang-an Airport",
		City:      "Wang-an",
		State:     "Taiwan",
		Country:   "TW",
		Elevation: 115,
		Latitude:  23.3673725128,
		Longitude: 119.5027770996,
		Timezone:  "Asia/Taipei",
	},
	"RCYU": {
		ICAO:      "RCYU",
		IATA:      "HUN",
		Name:      "Hualien Airport",
		City:      "Hualien City",
		State:     "Taiwan",
		Country:   "TW",
		Elevation: 52,
		Latitude:  24.0230998993,
		Longitude: 121.6179962158,
		Timezone:  "Asia/Taipei",
	},
	"RHST": {
		ICAO:      "RHST",
		Name:      "Rheinstetten Airport",
		State:     "Baden-Wuerttemberg",
		Country:   "DE",
		Latitude:  48.9777777778,
		Longitude: 8.3425,
		Timezone:  "Europe/Berlin",
	},
	"RI07": {
		ICAO:      "RI07",
		Name:      "Wing-Over Farm Airport",
		City:      "Tiverton",
		State:     "Rhode-Island",
		Country:   "US",
		Elevation: 113,
		Latitude:  41.5789985657,
		Longitude: -71.1417007446,
		Timezone:  "America/New_York",
	},
	"RI11": {
		ICAO:      "RI11",
		Name:      "Riconn Airport",
		City:      "Greene",
		State:     "Rhode-Island",
		Country:   "US",
		Elevation: 385,
		Latitude:  41.6968002319,
		Longitude: -71.7836990356,
		Timezone:  "America/New_York",
	},
	"RI20": {
		ICAO:      "RI20",
		Name:      "Mystery Farm Airport",
		City:      "Cranston",
		State:     "Rhode-Island",
		Country:   "US",
		Elevation: 310,
		Latitude:  41.7394981384,
		Longitude: -71.5244979858,
		Timezone:  "America/New_York",
	},
	"RJ04": {
		ICAO:      "RJ04",
		Name:      "Shikabe Airport",
		State:     "Hokkaido",
		Country:   "JP",
		Elevation: 63,
		Latitude:  42.0449981689,
		Longitude: 140.7929992676,
		Timezone:  "Asia/Tokyo",
	},
	"RJAA": {
		ICAO:      "RJAA",
		IATA:      "NRT",
		Name:      "Narita International Airport",
		City:      "Tokyo",
		State:     "Chiba",
		Country:   "JP",
		Elevation: 141,
		Latitude:  35.7647018433,
		Longitude: 140.3860015869,
		Timezone:  "Asia/Tokyo",
	},
	"RJAF": {
		ICAO:      "RJAF",
		IATA:      "MMJ",
		Name:      "Matsumoto Airport",
		City:      "Matsumoto",
		State:     "Nagano",
		Country:   "JP",
		Elevation: 2182,
		Latitude:  36.1668014526,
		Longitude: 137.9230041504,
		Timezone:  "Asia/Tokyo",
	},
	"RJAH": {
		ICAO:      "RJAH",
		IATA:      "IBR",
		Name:      "Hyakuri Airport",
		City:      "Omitama",
		State:     "Ibaraki",
		Country:   "JP",
		Elevation: 105,
		Latitude:  36.181098938,
		Longitude: 140.414993286,
		Timezone:  "Asia/Tokyo",
	},
	"RJAK": {
		ICAO:      "RJAK",
		Name:      "Kasumigaura Airport",
		State:     "Ibaraki",
		Country:   "JP",
		Elevation: 92,
		Latitude:  36.0346984863,
		Longitude: 140.1929931641,
		Timezone:  "Asia/Tokyo",
	},
	"RJAM": {
		ICAO:      "RJAM",
		IATA:      "MUS",
		Name:      "Minami Torishima Airport",
		State:     "Tokyo",
		Country:   "JP",
		Elevation: 22,
		Latitude:  24.2896995544,
		Longitude: 153.979003906,
		Timezone:  "Asia/Tokyo",
	},
	"RJAN": {
		ICAO:      "RJAN",
		Name:      "Nijima Airport",
		State:     "Tokyo",
		Country:   "JP",
		Elevation: 92,
		Latitude:  34.3694000244,
		Longitude: 139.2689971924,
		Timezone:  "Asia/Tokyo",
	},
	"RJAW": {
		ICAO:      "RJAW",
		IATA:      "IWO",
		Name:      "Iwo Jima Airport",
		State:     "Tokyo",
		Country:   "JP",
		Elevation: 384,
		Latitude:  24.7840003967,
		Longitude: 141.3229980469,
		Timezone:  "Asia/Tokyo",
	},
	"RJAZ": {
		ICAO:      "RJAZ",
		Name:      "Kozushima Airport",
		City:      "Kozushima",
		State:     "Tokyo",
		Country:   "JP",
		Elevation: 452,
		Latitude:  34.1899986267,
		Longitude: 139.1340026855,
		Timezone:  "Asia/Tokyo",
	},
	"RJBB": {
		ICAO:      "RJBB",
		IATA:      "KIX",
		Name:      "Kansai International Airport",
		City:      "Osaka",
		State:     "Osaka",
		Country:   "JP",
		Elevation: 26,
		Latitude:  34.4272994995,
		Longitude: 135.2440032959,
		Timezone:  "Asia/Tokyo",
	},
	"RJBD": {
		ICAO:      "RJBD",
		IATA:      "SHM",
		Name:      "Nanki Shirahama Airport",
		City:      "Shirahama",
		State:     "Wakayama",
		Country:   "JP",
		Elevation: 298,
		Latitude:  33.6622009277,
		Longitude: 135.363998413,
		Timezone:  "Asia/Tokyo",
	},
	"RJBE": {
		ICAO:      "RJBE",
		IATA:      "UKB",
		Name:      "Kobe Airport",
		City:      "Kobe",
		State:     "Hyogo",
		Country:   "JP",
		Elevation: 22,
		Latitude:  34.6328010559,
		Longitude: 135.2239990234,
		Timezone:  "Asia/Tokyo",
	},
	"RJBH": {
		ICAO:      "RJBH",
		IATA:      "HIW",
		Name:      "Hiroshimanishi Airport",
		State:     "Hiroshima",
		Country:   "JP",
		Elevation: 15,
		Latitude:  34.3669013977,
		Longitude: 132.4140014648,
		Timezone:  "Asia/Tokyo",
	},
	"RJBK": {
		ICAO:      "RJBK",
		Name:      "Kohnan Airport",
		State:     "Okayama",
		Country:   "JP",
		Elevation: 3,
		Latitude:  34.590801239,
		Longitude: 133.9329986572,
		Timezone:  "Asia/Tokyo",
	},
	"RJBT": {
		ICAO:      "RJBT",
		IATA:      "TJH",
		Name:      "Tajima Airport",
		City:      "Tajima",
		State:     "Hyogo",
		Country:   "JP",
		Elevation: 584,
		Latitude:  35.5127983093,
		Longitude: 134.7870025635,
		Timezone:  "Asia/Tokyo",
	},
	"RJCA": {
		ICAO:      "RJCA",
		Name:      "Asahikawa Airport",
		State:     "Hokkaido",
		Country:   "JP",
		Elevation: 377,
		Latitude:  43.7952003479,
		Longitude: 142.3630065918,
		Timezone:  "Asia/Tokyo",
	},
	"RJCB": {
		ICAO:      "RJCB",
		IATA:      "OBO",
		Name:      "Tokachi-Obihiro Airport",
		City:      "Obihiro",
		State:     "Hokkaido",
		Country:   "JP",
		Elevation: 505,
		Latitude:  42.7332992554,
		Longitude: 143.216995239,
		Timezone:  "Asia/Tokyo",
	},
	"RJCC": {
		ICAO:      "RJCC",
		IATA:      "CTS",
		Name:      "New Chitose Airport",
		City:      "Chitose / Tomakomai",
		State:     "Hokkaido",
		Country:   "JP",
		Elevation: 82,
		Latitude:  42.7751998901,
		Longitude: 141.6920013428,
		Timezone:  "Asia/Tokyo",
	},
	"RJCH": {
		ICAO:      "RJCH",
		IATA:      "HKD",
		Name:      "Hakodate Airport",
		City:      "Hakodate",
		State:     "Hokkaido",
		Country:   "JP",
		Elevation: 151,
		Latitude:  41.7700004578,
		Longitude: 140.822006226,
		Timezone:  "Asia/Tokyo",
	},
	"RJCJ": {
		ICAO:      "RJCJ",
		Name:      "Chitose Air Base",
		State:     "Hokkaido",
		Country:   "JP",
		Elevation: 87,
		Latitude:  42.7944984436,
		Longitude: 141.666000366,
		Timezone:  "Asia/Tokyo",
	},
	"RJCK": {
		ICAO:      "RJCK",
		IATA:      "KUH",
		Name:      "Kushiro Airport",
		City:      "Kushiro",
		State:     "Hokkaido",
		Country:   "JP",
		Elevation: 327,
		Latitude:  43.0410003662,
		Longitude: 144.192993164,
		Timezone:  "Asia/Tokyo",
	},
	"RJCM": {
		ICAO:      "RJCM",
		IATA:      "MMB",
		Name:      "Memanbetsu Airport",
		City:      "Ozora",
		State:     "Hokkaido",
		Country:   "JP",
		Elevation: 135,
		Latitude:  43.8805999756,
		Longitude: 144.164001465,
		Timezone:  "Asia/Tokyo",
	},
	"RJCN": {
		ICAO:      "RJCN",
		IATA:      "SHB",
		Name:      "Nakashibetsu Airport",
		City:      "Nakashibetsu",
		State:     "Hokkaido",
		Country:   "JP",
		Elevation: 234,
		Latitude:  43.5774993896,
		Longitude: 144.960006714,
		Timezone:  "Asia/Tokyo",
	},
	"RJCO": {
		ICAO:      "RJCO",
		IATA:      "OKD",
		Name:      "Okadama Airport",
		City:      "Sapporo",
		State:     "Hokkaido",
		Country:   "JP",
		Elevation: 25,
		Latitude:  43.1161003113,
		Longitude: 141.3800048828,
		Timezone:  "Asia/Tokyo",
	},
	"RJCR": {
		ICAO:      "RJCR",
		IATA:      "RBJ",
		Name:      "Rebun Airport Airport",
		State:     "Hokkaido",
		Country:   "JP",
		Elevation: 92,
		Latitude:  45.4550018311,
		Longitude: 141.0390014648,
		Timezone:  "Asia/Tokyo",
	},
	"RJCT": {
		ICAO:      "RJCT",
		Name:      "Tokachi Airport",
		State:     "Hokkaido",
		Country:   "JP",
		Elevation: 281,
		Latitude:  42.890499115,
		Longitude: 143.1580047607,
		Timezone:  "Asia/Tokyo",
	},
	"RJCW": {
		ICAO:      "RJCW",
		IATA:      "WKJ",
		Name:      "Wakkanai Airport",
		City:      "Wakkanai",
		State:     "Hokkaido",
		Country:   "JP",
		Elevation: 30,
		Latitude:  45.4042015076,
		Longitude: 141.800994873,
		Timezone:  "Asia/Tokyo",
	},
	"RJDA": {
		ICAO:      "RJDA",
		IATA:      "AXJ",
		Name:      "Amakusa Airport",
		State:     "Kumamoto",
		Country:   "JP",
		Elevation: 340,
		Latitude:  32.4824981689,
		Longitude: 130.158996582,
		Timezone:  "Asia/Tokyo",
	},
	"RJDB": {
		ICAO:      "RJDB",
		IATA:      "IKI",
		Name:      "Iki Airport",
		City:      "Iki",
		State:     "Nagasaki",
		Country:   "JP",
		Elevation: 41,
		Latitude:  33.7490005493,
		Longitude: 129.785003662,
		Timezone:  "Asia/Tokyo",
	},
	"RJDC": {
		ICAO:      "RJDC",
		IATA:      "UBJ",
		Name:      "Yamaguchi Ube Airport",
		City:      "Ube",
		State:     "Yamaguchi",
		Country:   "JP",
		Elevation: 23,
		Latitude:  33.9300003052,
		Longitude: 131.279006958,
		Timezone:  "Asia/Tokyo",
	},
	"RJDK": {
		ICAO:      "RJDK",
		Name:      "Kamigoto Airport",
		City:      "Kamigoto",
		State:     "Nagasaki",
		Country:   "JP",
		Elevation: 263,
		Latitude:  33.0130996704,
		Longitude: 129.1920013428,
		Timezone:  "Asia/Tokyo",
	},
	"RJDM": {
		ICAO:      "RJDM",
		Name:      "Metabaru Air Base",
		State:     "Saga",
		Country:   "JP",
		Elevation: 49,
		Latitude:  33.3252983093,
		Longitude: 130.4140014648,
		Timezone:  "Asia/Tokyo",
	},
	"RJDO": {
		ICAO:      "RJDO",
		Name:      "Ojika Airport",
		State:     "Nagasaki",
		Country:   "JP",
		Elevation: 30,
		Latitude:  33.1907997131,
		Longitude: 129.0899963379,
		Timezone:  "Asia/Tokyo",
	},
	"RJDT": {
		ICAO:      "RJDT",
		IATA:      "TSJ",
		Name:      "Tsushima Airport",
		City:      "Tsushima",
		State:     "Nagasaki",
		Country:   "JP",
		Elevation: 213,
		Latitude:  34.2849006653,
		Longitude: 129.330993652,
		Timezone:  "Asia/Tokyo",
	},
	"RJEB": {
		ICAO:      "RJEB",
		IATA:      "MBE",
		Name:      "Monbetsu Airport",
		City:      "Monbetsu",
		State:     "Hokkaido",
		Country:   "JP",
		Elevation: 80,
		Latitude:  44.3039016724,
		Longitude: 143.404006958,
		Timezone:  "Asia/Tokyo",
	},
	"RJEC": {
		ICAO:      "RJEC",
		IATA:      "AKJ",
		Name:      "Asahikawa Airport",
		City:      "Asahikawa",
		State:     "Hokkaido",
		Country:   "JP",
		Elevation: 721,
		Latitude:  43.6707992554,
		Longitude: 142.4470062256,
		Timezone:  "Asia/Tokyo",
	},
	"RJEO": {
		ICAO:      "RJEO",
		IATA:      "OIR",
		Name:      "Okushiri Airport",
		State:     "Hokkaido",
		Country:   "JP",
		Elevation: 161,
		Latitude:  42.0717010498,
		Longitude: 139.4329986572,
		Timezone:  "Asia/Tokyo",
	},
	"RJER": {
		ICAO:      "RJER",
		IATA:      "RIS",
		Name:      "Rishiri Airport",
		City:      "Rishiri",
		State:     "Hokkaido",
		Country:   "JP",
		Elevation: 112,
		Latitude:  45.2420005798,
		Longitude: 141.186004639,
		Timezone:  "Asia/Tokyo",
	},
	"RJFA": {
		ICAO:      "RJFA",
		Name:      "Ashiya Airport",
		State:     "Fukuoka",
		Country:   "JP",
		Elevation: 98,
		Latitude:  33.8830986023,
		Longitude: 130.6529998779,
		Timezone:  "Asia/Tokyo",
	},
	"RJFC": {
		ICAO:      "RJFC",
		IATA:      "KUM",
		Name:      "Yakushima Airport",
		State:     "Kagoshima",
		Country:   "JP",
		Elevation: 124,
		Latitude:  30.3855991364,
		Longitude: 130.658996582,
		Timezone:  "Asia/Tokyo",
	},
	"RJFE": {
		ICAO:      "RJFE",
		IATA:      "FUJ",
		Name:      "Fukue Airport",
		City:      "Goto",
		State:     "Nagasaki",
		Country:   "JP",
		Elevation: 273,
		Latitude:  32.6663017273,
		Longitude: 128.8329925537,
		Timezone:  "Asia/Tokyo",
	},
	"RJFF": {
		ICAO:      "RJFF",
		IATA:      "FUK",
		Name:      "Fukuoka Airport",
		City:      "Fukuoka",
		State:     "Fukuoka",
		Country:   "JP",
		Elevation: 32,
		Latitude:  33.585899353,
		Longitude: 130.4510040283,
		Timezone:  "Asia/Tokyo",
	},
	"RJFG": {
		ICAO:      "RJFG",
		IATA:      "TNE",
		Name:      "New Tanegashima Airport",
		State:     "Kagoshima",
		Country:   "JP",
		Elevation: 768,
		Latitude:  30.6051006317,
		Longitude: 130.9909973145,
		Timezone:  "Asia/Tokyo",
	},
	"RJFK": {
		ICAO:      "RJFK",
		IATA:      "KOJ",
		Name:      "Kagoshima Airport",
		City:      "Kagoshima",
		State:     "Kagoshima",
		Country:   "JP",
		Elevation: 906,
		Latitude:  31.8034000397,
		Longitude: 130.7189941406,
		Timezone:  "Asia/Tokyo",
	},
	"RJFM": {
		ICAO:      "RJFM",
		IATA:      "KMI",
		Name:      "Miyazaki Airport",
		City:      "Miyazaki",
		State:     "Miyazaki",
		Country:   "JP",
		Elevation: 20,
		Latitude:  31.877199173,
		Longitude: 131.449005127,
		Timezone:  "Asia/Tokyo",
	},
	"RJFN": {
		ICAO:      "RJFN",
		Name:      "Nyutabaru Airport",
		State:     "Miyazaki",
		Country:   "JP",
		Elevation: 259,
		Latitude:  32.0835990906,
		Longitude: 131.4510040283,
		Timezone:  "Asia/Tokyo",
	},
	"RJFO": {
		ICAO:      "RJFO",
		IATA:      "OIT",
		Name:      "Oita Airport",
		City:      "Oita",
		State:     "Oita",
		Country:   "JP",
		Elevation: 19,
		Latitude:  33.4794006348,
		Longitude: 131.736999512,
		Timezone:  "Asia/Tokyo",
	},
	"RJFR": {
		ICAO:      "RJFR",
		IATA:      "KKJ",
		Name:      "Kitakyushu Airport",
		City:      "Kitakyushu",
		State:     "Fukuoka",
		Country:   "JP",
		Elevation: 21,
		Latitude:  33.8459014893,
		Longitude: 131.035003662,
		Timezone:  "Asia/Tokyo",
	},
	"RJFS": {
		ICAO:      "RJFS",
		IATA:      "HSG",
		Name:      "Saga Airport",
		City:      "Saga",
		State:     "Saga",
		Country:   "JP",
		Elevation: 6,
		Latitude:  33.1497001648,
		Longitude: 130.302001953,
		Timezone:  "Asia/Tokyo",
	},
	"RJFT": {
		ICAO:      "RJFT",
		IATA:      "KMJ",
		Name:      "Kumamoto Airport",
		City:      "Kumamoto",
		State:     "Kumamoto",
		Country:   "JP",
		Elevation: 642,
		Latitude:  32.8372993469,
		Longitude: 130.8549957275,
		Timezone:  "Asia/Tokyo",
	},
	"RJFU": {
		ICAO:      "RJFU",
		IATA:      "NGS",
		Name:      "Nagasaki Airport",
		City:      "Nagasaki",
		State:     "Nagasaki",
		Country:   "JP",
		Elevation: 15,
		Latitude:  32.9169006348,
		Longitude: 129.914001465,
		Timezone:  "Asia/Tokyo",
	},
	"RJFY": {
		ICAO:      "RJFY",
		Name:      "Kanoya Airport",
		State:     "Kagoshima",
		Country:   "JP",
		Elevation: 214,
		Latitude:  31.3675994873,
		Longitude: 130.8450012207,
		Timezone:  "Asia/Tokyo",
	},
	"RJFZ": {
		ICAO:      "RJFZ",
		Name:      "Tsuiki Airport",
		State:     "Fukuoka",
		Country:   "JP",
		Elevation: 55,
		Latitude:  33.6850013733,
		Longitude: 131.0399932861,
		Timezone:  "Asia/Tokyo",
	},
	"RJGG": {
		ICAO:      "RJGG",
		IATA:      "NGO",
		Name:      "Chubu Centrair International Airport",
		City:      "Tokoname",
		State:     "Aichi",
		Country:   "JP",
		Elevation: 15,
		Latitude:  34.8583984375,
		Longitude: 136.8049926758,
		Timezone:  "Asia/Tokyo",
	},
	"RJKA": {
		ICAO:      "RJKA",
		IATA:      "ASJ",
		Name:      "Amami Airport",
		City:      "Amami",
		State:     "Kagoshima",
		Country:   "JP",
		Elevation: 27,
		Latitude:  28.4305992126,
		Longitude: 129.7129974365,
		Timezone:  "Asia/Tokyo",
	},
	"RJKB": {
		ICAO:      "RJKB",
		IATA:      "OKE",
		Name:      "Okierabu Airport",
		State:     "Kagoshima",
		Country:   "JP",
		Elevation: 101,
		Latitude:  27.4255008698,
		Longitude: 128.7010040283,
		Timezone:  "Asia/Tokyo",
	},
	"RJKI": {
		ICAO:      "RJKI",
		IATA:      "KKX",
		Name:      "Kikai Airport",
		State:     "Kagoshima",
		Country:   "JP",
		Elevation: 21,
		Latitude:  28.3213005066,
		Longitude: 129.9279937744,
		Timezone:  "Asia/Tokyo",
	},
	"RJKN": {
		ICAO:      "RJKN",
		IATA:      "TKN",
		Name:      "Tokunoshima Airport",
		City:      "Tokunoshima",
		State:     "Kagoshima",
		Country:   "JP",
		Elevation: 17,
		Latitude:  27.8363990784,
		Longitude: 128.8809967041,
		Timezone:  "Asia/Tokyo",
	},
	"RJNA": {
		ICAO:      "RJNA",
		IATA:      "NKM",
		Name:      "Nagoya Airport",
		City:      "Nagoya",
		State:     "Aichi",
		Country:   "JP",
		Elevation: 52,
		Latitude:  35.2550010681,
		Longitude: 136.9239959717,
		Timezone:  "Asia/Tokyo",
	},
	"RJNF": {
		ICAO:      "RJNF",
		IATA:      "FKJ",
		Name:      "Fukui Airport",
		State:     "Fukui",
		Country:   "JP",
		Elevation: 19,
		Latitude:  36.1427993774,
		Longitude: 136.2239990234,
		Timezone:  "Asia/Tokyo",
	},
	"RJNG": {
		ICAO:      "RJNG",
		IATA:      "QGU",
		Name:      "Gifu Airport",
		City:      "Gifu",
		State:     "Gifu",
		Country:   "JP",
		Elevation: 128,
		Latitude:  35.3941001892,
		Longitude: 136.8699951172,
		Timezone:  "Asia/Tokyo",
	},
	"RJNH": {
		ICAO:      "RJNH",
		Name:      "Hamamatsu Airport",
		State:     "Shizuoka",
		Country:   "JP",
		Elevation: 150,
		Latitude:  34.7501983643,
		Longitude: 137.7030029297,
		Timezone:  "Asia/Tokyo",
	},
	"RJNK": {
		ICAO:      "RJNK",
		IATA:      "KMQ",
		Name:      "Komatsu Airport",
		City:      "Kanazawa",
		State:     "Ishikawa",
		Country:   "JP",
		Elevation: 36,
		Latitude:  36.3945999146,
		Longitude: 136.4069976807,
		Timezone:  "Asia/Tokyo",
	},
	"RJNO": {
		ICAO:      "RJNO",
		IATA:      "OKI",
		Name:      "Oki Airport",
		City:      "Okinoshima",
		State:     "Shimane",
		Country:   "JP",
		Elevation: 311,
		Latitude:  36.181098938,
		Longitude: 133.3249969482,
		Timezone:  "Asia/Tokyo",
	},
	"RJNS": {
		ICAO:      "RJNS",
		IATA:      "FSZ",
		Name:      "Mt. Fuji Shizuoka Airport",
		State:     "Shizuoka",
		Country:   "JP",
		Elevation: 433,
		Latitude:  34.7960434679,
		Longitude: 138.18775177,
		Timezone:  "Asia/Tokyo",
	},
	"RJNT": {
		ICAO:      "RJNT",
		IATA:      "TOY",
		Name:      "Toyama Airport",
		City:      "Toyama",
		State:     "Toyama",
		Country:   "JP",
		Elevation: 95,
		Latitude:  36.6483001709,
		Longitude: 137.18800354,
		Timezone:  "Asia/Tokyo",
	},
	"RJNW": {
		ICAO:      "RJNW",
		IATA:      "NTQ",
		Name:      "Noto Airport",
		City:      "Wajima",
		State:     "Ishikawa",
		Country:   "JP",
		Elevation: 718,
		Latitude:  37.2930984497,
		Longitude: 136.962005615,
		Timezone:  "Asia/Tokyo",
	},
	"RJNY": {
		ICAO:      "RJNY",
		Name:      "Shizuhama Airport",
		State:     "Shizuoka",
		Country:   "JP",
		Elevation: 23,
		Latitude:  34.8128013611,
		Longitude: 138.2980041504,
		Timezone:  "Asia/Tokyo",
	},
	"RJOA": {
		ICAO:      "RJOA",
		IATA:      "HIJ",
		Name:      "Hiroshima Airport",
		City:      "Hiroshima",
		State:     "Hiroshima",
		Country:   "JP",
		Elevation: 1088,
		Latitude:  34.4361000061,
		Longitude: 132.919006348,
		Timezone:  "Asia/Tokyo",
	},
	"RJOB": {
		ICAO:      "RJOB",
		IATA:      "OKJ",
		Name:      "Okayama Airport",
		City:      "Okayama City",
		State:     "Okayama",
		Country:   "JP",
		Elevation: 806,
		Latitude:  34.7569007874,
		Longitude: 133.854995728,
		Timezone:  "Asia/Tokyo",
	},
	"RJOC": {
		ICAO:      "RJOC",
		IATA:      "IZO",
		Name:      "Izumo Airport",
		City:      "Izumo",
		State:     "Shimane",
		Country:   "JP",
		Elevation: 15,
		Latitude:  35.4136009216,
		Longitude: 132.88999939,
		Timezone:  "Asia/Tokyo",
	},
	"RJOE": {
		ICAO:      "RJOE",
		Name:      "Akeno Airport",
		State:     "Mie",
		Country:   "JP",
		Elevation: 20,
		Latitude:  34.5332984924,
		Longitude: 136.6719970703,
		Timezone:  "Asia/Tokyo",
	},
	"RJOF": {
		ICAO:      "RJOF",
		Name:      "Hofu Airport",
		State:     "Yamaguchi",
		Country:   "JP",
		Elevation: 7,
		Latitude:  34.0346984863,
		Longitude: 131.5489959717,
		Timezone:  "Asia/Tokyo",
	},
	"RJOH": {
		ICAO:      "RJOH",
		IATA:      "YGJ",
		Name:      "Miho Yonago Airport",
		City:      "Yonago",
		State:     "Tottori",
		Country:   "JP",
		Elevation: 20,
		Latitude:  35.4921989441,
		Longitude: 133.2359924316,
		Timezone:  "Asia/Tokyo",
	},
	"RJOI": {
		ICAO:      "RJOI",
		IATA:      "IWK",
		Name:      "Iwakuni Marine Corps Air Station",
		City:      "Iwakuni",
		State:     "Yamaguchi",
		Country:   "JP",
		Elevation: 7,
		Latitude:  34.143901825,
		Longitude: 132.2359924316,
		Timezone:  "Asia/Tokyo",
	},
	"RJOK": {
		ICAO:      "RJOK",
		IATA:      "KCZ",
		Name:      "Kochi Ryoma Airport",
		City:      "Nankoku",
		State:     "Kochi",
		Country:   "JP",
		Elevation: 42,
		Latitude:  33.5461006165,
		Longitude: 133.669006348,
		Timezone:  "Asia/Tokyo",
	},
	"RJOM": {
		ICAO:      "RJOM",
		IATA:      "MYJ",
		Name:      "Matsuyama Airport",
		City:      "Matsuyama",
		State:     "Ehime",
		Country:   "JP",
		Elevation: 25,
		Latitude:  33.8272018433,
		Longitude: 132.6999969482,
		Timezone:  "Asia/Tokyo",
	},
	"RJOO": {
		ICAO:      "RJOO",
		IATA:      "ITM",
		Name:      "Osaka International Airport",
		City:      "Osaka",
		State:     "Hyogo",
		Country:   "JP",
		Elevation: 50,
		Latitude:  34.7854995728,
		Longitude: 135.43800354,
		Timezone:  "Asia/Tokyo",
	},
	"RJOR": {
		ICAO:      "RJOR",
		IATA:      "TTJ",
		Name:      "Tottori Airport",
		City:      "Tottori",
		State:     "Tottori",
		Country:   "JP",
		Elevation: 65,
		Latitude:  35.5301017761,
		Longitude: 134.167007446,
		Timezone:  "Asia/Tokyo",
	},
	"RJOS": {
		ICAO:      "RJOS",
		IATA:      "TKS",
		Name:      "Tokushima Airport",
		City:      "Tokushima",
		State:     "Tokushima",
		Country:   "JP",
		Elevation: 26,
		Latitude:  34.1328010559,
		Longitude: 134.606994629,
		Timezone:  "Asia/Tokyo",
	},
	"RJOT": {
		ICAO:      "RJOT",
		IATA:      "TAK",
		Name:      "Takamatsu Airport",
		City:      "Takamatsu",
		State:     "Kagawa",
		Country:   "JP",
		Elevation: 607,
		Latitude:  34.2141990662,
		Longitude: 134.01600647,
		Timezone:  "Asia/Tokyo",
	},
	"RJOW": {
		ICAO:      "RJOW",
		IATA:      "IWJ",
		Name:      "Iwami Airport",
		City:      "Masuda",
		State:     "Shimane",
		Country:   "JP",
		Elevation: 184,
		Latitude:  34.676399231,
		Longitude: 131.789993286,
		Timezone:  "Asia/Tokyo",
	},
	"RJOY": {
		ICAO:      "RJOY",
		Name:      "Yao Airport",
		City:      "Yao",
		State:     "Osaka",
		Country:   "JP",
		Elevation: 39,
		Latitude:  34.5962982178,
		Longitude: 135.6029968262,
		Timezone:  "Asia/Tokyo",
	},
	"RJOZ": {
		ICAO:      "RJOZ",
		Name:      "Ozuki Airport",
		State:     "Yamaguchi",
		Country:   "JP",
		Elevation: 13,
		Latitude:  34.04529953,
		Longitude: 131.0520019531,
		Timezone:  "Asia/Tokyo",
	},
	"RJSA": {
		ICAO:      "RJSA",
		IATA:      "AOJ",
		Name:      "Aomori Airport",
		City:      "Aomori",
		State:     "Aomori",
		Country:   "JP",
		Elevation: 664,
		Latitude:  40.7346992493,
		Longitude: 140.6909942627,
		Timezone:  "Asia/Tokyo",
	},
	"RJSC": {
		ICAO:      "RJSC",
		IATA:      "GAJ",
		Name:      "Yamagata Airport",
		City:      "Yamagata",
		State:     "Yamagata",
		Country:   "JP",
		Elevation: 353,
		Latitude:  38.4118995667,
		Longitude: 140.371002197,
		Timezone:  "Asia/Tokyo",
	},
	"RJSD": {
		ICAO:      "RJSD",
		IATA:      "SDS",
		Name:      "Sado Airport",
		State:     "Niigata",
		Country:   "JP",
		Elevation: 88,
		Latitude:  38.0601997375,
		Longitude: 138.4140014648,
		Timezone:  "Asia/Tokyo",
	},
	"RJSF": {
		ICAO:      "RJSF",
		IATA:      "FKS",
		Name:      "Fukushima Airport",
		City:      "Sukagawa",
		State:     "Fukushima",
		Country:   "JP",
		Elevation: 1221,
		Latitude:  37.2274017334,
		Longitude: 140.4309997559,
		Timezone:  "Asia/Tokyo",
	},
	"RJSH": {
		ICAO:      "RJSH",
		IATA:      "HHE",
		Name:      "Hachinohe Airport",
		State:     "Aomori",
		Country:   "JP",
		Elevation: 152,
		Latitude:  40.5564002991,
		Longitude: 141.466003418,
		Timezone:  "Asia/Tokyo",
	},
	"RJSI": {
		ICAO:      "RJSI",
		IATA:      "HNA",
		Name:      "Hanamaki Airport",
		State:     "Iwate",
		Country:   "JP",
		Elevation: 297,
		Latitude:  39.4286003113,
		Longitude: 141.1349945068,
		Timezone:  "Asia/Tokyo",
	},
	"RJSK": {
		ICAO:      "RJSK",
		IATA:      "AXT",
		Name:      "Akita Airport",
		City:      "Akita",
		State:     "Akita",
		Country:   "JP",
		Elevation: 313,
		Latitude:  39.6156005859,
		Longitude: 140.2189941406,
		Timezone:  "Asia/Tokyo",
	},
	"RJSM": {
		ICAO:      "RJSM",
		IATA:      "MSJ",
		Name:      "Misawa Air Base",
		City:      "Misawa",
		State:     "Aomori",
		Country:   "JP",
		Elevation: 119,
		Latitude:  40.7032012939,
		Longitude: 141.367996216,
		Timezone:  "Asia/Tokyo",
	},
	"RJSN": {
		ICAO:      "RJSN",
		IATA:      "KIJ",
		Name:      "Niigata Airport",
		City:      "Niigata",
		State:     "Niigata",
		Country:   "JP",
		Elevation: 29,
		Latitude:  37.9558982849,
		Longitude: 139.121002197,
		Timezone:  "Asia/Tokyo",
	},
	"RJSO": {
		ICAO:      "RJSO",
		Name:      "JASDF Airport",
		City:      "Ominato",
		State:     "Aomori",
		Country:   "JP",
		Elevation: 24,
		Latitude:  41.233001709,
		Longitude: 141.1329956055,
		Timezone:  "Asia/Tokyo",
	},
	"RJSR": {
		ICAO:      "RJSR",
		IATA:      "ONJ",
		Name:      "Odate Noshiro Airport",
		City:      "Odate",
		State:     "Akita",
		Country:   "JP",
		Elevation: 292,
		Latitude:  40.1918983459,
		Longitude: 140.371002197,
		Timezone:  "Asia/Tokyo",
	},
	"RJSS": {
		ICAO:      "RJSS",
		IATA:      "SDJ",
		Name:      "Sendai Airport",
		City:      "Sendai",
		State:     "Miyagi",
		Country:   "JP",
		Elevation: 15,
		Latitude:  38.1397018433,
		Longitude: 140.917007446,
		Timezone:  "Asia/Tokyo",
	},
	"RJST": {
		ICAO:      "RJST",
		Name:      "Matsushima Air Base",
		City:      "Ishinomaki",
		State:     "Miyagi",
		Country:   "JP",
		Elevation: 7,
		Latitude:  38.4048995972,
		Longitude: 141.220001221,
		Timezone:  "Asia/Tokyo",
	},
	"RJSU": {
		ICAO:      "RJSU",
		Name:      "Sendai Kasuminom Airport",
		State:     "Miyagi",
		Country:   "JP",
		Elevation: 20,
		Latitude:  38.2355995178,
		Longitude: 140.9230041504,
		Timezone:  "Asia/Tokyo",
	},
	"RJSY": {
		ICAO:      "RJSY",
		IATA:      "SYO",
		Name:      "Shonai Airport",
		City:      "Shonai",
		State:     "Yamagata",
		Country:   "JP",
		Elevation: 86,
		Latitude:  38.8121986389,
		Longitude: 139.787002563,
		Timezone:  "Asia/Tokyo",
	},
	"RJTA": {
		ICAO:      "RJTA",
		IATA:      "NJA",
		Name:      "Atsugi Naval Air Facility",
		State:     "Kanagawa",
		Country:   "JP",
		Elevation: 205,
		Latitude:  35.4546012878,
		Longitude: 139.4499969482,
		Timezone:  "Asia/Tokyo",
	},
	"RJTC": {
		ICAO:      "RJTC",
		Name:      "Tachikawa Airfield",
		City:      "Tokyo",
		State:     "Tokyo",
		Country:   "JP",
		Elevation: 312,
		Latitude:  35.7108001709,
		Longitude: 139.4029998779,
		Timezone:  "Asia/Tokyo",
	},
	"RJTE": {
		ICAO:      "RJTE",
		Name:      "Tateyama Airport",
		State:     "Chiba",
		Country:   "JP",
		Elevation: 10,
		Latitude:  34.9870986938,
		Longitude: 139.828994751,
		Timezone:  "Asia/Tokyo",
	},
	"RJTF": {
		ICAO:      "RJTF",
		Name:      "Chofu Airport",
		City:      "Tokyo",
		State:     "Tokyo",
		Country:   "JP",
		Elevation: 141,
		Latitude:  35.6716995239,
		Longitude: 139.5279998779,
		Timezone:  "Asia/Tokyo",
	},
	"RJTH": {
		ICAO:      "RJTH",
		IATA:      "HAC",
		Name:      "Hachijojima Airport",
		City:      "Hachijojima",
		State:     "Tokyo",
		Country:   "JP",
		Elevation: 303,
		Latitude:  33.1150016785,
		Longitude: 139.785995483,
		Timezone:  "Asia/Tokyo",
	},
	"RJTJ": {
		ICAO:      "RJTJ",
		Name:      "Iruma Air Base",
		City:      "Tokyo",
		State:     "Saitama",
		Country:   "JP",
		Elevation: 295,
		Latitude:  35.8418998718,
		Longitude: 139.4109954834,
		Timezone:  "Asia/Tokyo",
	},
	"RJTK": {
		ICAO:      "RJTK",
		Name:      "Kisarazu Airport",
		State:     "Chiba",
		Country:   "JP",
		Elevation: 10,
		Latitude:  35.3983001709,
		Longitude: 139.9100036621,
		Timezone:  "Asia/Tokyo",
	},
	"RJTL": {
		ICAO:      "RJTL",
		Name:      "Shimofusa Airport",
		State:     "Chiba",
		Country:   "JP",
		Elevation: 98,
		Latitude:  35.7989006042,
		Longitude: 140.0110015869,
		Timezone:  "Asia/Tokyo",
	},
	"RJTO": {
		ICAO:      "RJTO",
		IATA:      "OIM",
		Name:      "Oshima Airport",
		City:      "Izu Oshima",
		State:     "Tokyo",
		Country:   "JP",
		Elevation: 130,
		Latitude:  34.7820014954,
		Longitude: 139.36000061,
		Timezone:  "Asia/Tokyo",
	},
	"RJTQ": {
		ICAO:      "RJTQ",
		IATA:      "MYE",
		Name:      "Miyakejima Airport",
		City:      "Miyakejima",
		State:     "Tokyo",
		Country:   "JP",
		Elevation: 67,
		Latitude:  34.073600769,
		Longitude: 139.559997559,
		Timezone:  "Asia/Tokyo",
	},
	"RJTR": {
		ICAO:      "RJTR",
		Name:      "Kastner Army Air Field",
		State:     "Kanagawa",
		Country:   "JP",
		Elevation: 360,
		Latitude:  35.5138015747,
		Longitude: 139.3939971924,
		Timezone:  "Asia/Tokyo",
	},
	"RJTT": {
		ICAO:      "RJTT",
		IATA:      "HND",
		Name:      "Tokyo International Airport",
		City:      "Tokyo",
		State:     "Tokyo",
		Country:   "JP",
		Elevation: 35,
		Latitude:  35.5522994995,
		Longitude: 139.7799987793,
		Timezone:  "Asia/Tokyo",
	},
	"RJTU": {
		ICAO:      "RJTU",
		IATA:      "QUT",
		Name:      "Utsunomiya Airport",
		State:     "Tochigi",
		Country:   "JP",
		Elevation: 334,
		Latitude:  36.5144996643,
		Longitude: 139.8710021973,
		Timezone:  "Asia/Tokyo",
	},
	"RJTY": {
		ICAO:      "RJTY",
		IATA:      "OKO",
		Name:      "Yokota Air Base",
		City:      "Fussa",
		State:     "Tokyo",
		Country:   "JP",
		Elevation: 463,
		Latitude:  35.748500824,
		Longitude: 139.3480072021,
		Timezone:  "Asia/Tokyo",
	},
	"RK13": {
		ICAO:      "RK13",
		Name:      "G 404 Airport",
		Country:   "KR",
		Elevation: 655,
		Latitude:  38.0875015259,
		Longitude: 127.9869995117,
		Timezone:  "Asia/Seoul",
	},
	"RK14": {
		ICAO:      "RK14",
		Name:      "G 231 Airport",
		Country:   "KR",
		Elevation: 464,
		Latitude:  38.026599884,
		Longitude: 127.3669967651,
		Timezone:  "Asia/Seoul",
	},
	"RK15": {
		ICAO:      "RK15",
		Name:      "G 237 Airport",
		Country:   "KR",
		Elevation: 510,
		Latitude:  38.1547012329,
		Longitude: 127.31300354,
		Timezone:  "Asia/Seoul",
	},
	"RK16": {
		ICAO:      "RK16",
		Name:      "G 312 Airport",
		State:     "Gangwon-do",
		Country:   "KR",
		Elevation: 855,
		Latitude:  38.0731010437,
		Longitude: 127.5230026245,
		Timezone:  "Asia/Seoul",
	},
	"RK17": {
		ICAO:      "RK17",
		Name:      "G 406 Airport",
		Country:   "KR",
		Elevation: 656,
		Latitude:  38.1398010254,
		Longitude: 128.0090026855,
		Timezone:  "Asia/Seoul",
	},
	"RK18": {
		ICAO:      "RK18",
		Name:      "G 233 Airport",
		Country:   "KR",
		Elevation: 340,
		Latitude:  38.0792007446,
		Longitude: 127.2720031738,
		Timezone:  "Asia/Seoul",
	},
	"RK19": {
		ICAO:      "RK19",
		Name:      "G 314 Airport",
		Country:   "KR",
		Elevation: 600,
		Latitude:  38.1375007629,
		Longitude: 127.7419967651,
		Timezone:  "Asia/Seoul",
	},
	"RK20": {
		ICAO:      "RK20",
		Name:      "G 500 Airport",
		State:     "Chungcheongnam-do",
		Country:   "KR",
		Elevation: 66,
		Latitude:  36.1110992432,
		Longitude: 127.1039962769,
		Timezone:  "Asia/Seoul",
	},
	"RK21": {
		ICAO:      "RK21",
		Name:      "G 413 Airport",
		Country:   "KR",
		Elevation: 23,
		Latitude:  38.3827018738,
		Longitude: 128.4579925537,
		Timezone:  "Asia/Seoul",
	},
	"RK22": {
		ICAO:      "RK22",
		Name:      "G 313 Airport",
		Country:   "KR",
		Elevation: 460,
		Latitude:  38.1190986633,
		Longitude: 127.6849975586,
		Timezone:  "Asia/Seoul",
	},
	"RK24": {
		ICAO:      "RK24",
		Name:      "G 522 Airport",
		State:     "North-Chungcheong",
		Country:   "KR",
		Elevation: 546,
		Latitude:  36.1801986694,
		Longitude: 127.8330001831,
		Timezone:  "Asia/Seoul",
	},
	"RK25": {
		ICAO:      "RK25",
		Name:      "G 107 Airport",
		Country:   "KR",
		Elevation: 45,
		Latitude:  37.7153015137,
		Longitude: 126.5589981079,
		Timezone:  "Asia/Seoul",
	},
	"RK27": {
		ICAO:      "RK27",
		Name:      "G 218 Airport",
		Country:   "KR",
		Elevation: 381,
		Latitude:  37.8955001831,
		Longitude: 126.9720001221,
		Timezone:  "Asia/Seoul",
	},
	"RK28": {
		ICAO:      "RK28",
		Name:      "G 219 Airport",
		State:     "Gyeonggi-do",
		Country:   "KR",
		Elevation: 317,
		Latitude:  37.9094009399,
		Longitude: 127.0080032349,
		Timezone:  "Asia/Seoul",
	},
	"RK29": {
		ICAO:      "RK29",
		Name:      "G 203 Airport",
		State:     "Gyeonggi-do",
		Country:   "KR",
		Elevation: 80,
		Latitude:  37.643699646,
		Longitude: 127.1490020752,
		Timezone:  "Asia/Seoul",
	},
	"RK31": {
		ICAO:      "RK31",
		Name:      "G 307 Airport",
		Country:   "KR",
		Elevation: 270,
		Latitude:  37.9295005798,
		Longitude: 127.7570037842,
		Timezone:  "Asia/Seoul",
	},
	"RK32": {
		ICAO:      "RK32",
		Name:      "G 420 Airport",
		Country:   "KR",
		Elevation: 951,
		Latitude:  37.956199646,
		Longitude: 128.3159942627,
		Timezone:  "Asia/Seoul",
	},
	"RK33": {
		ICAO:      "RK33",
		Name:      "G 418 Airport",
		Country:   "KR",
		Elevation: 933,
		Latitude:  37.341999054,
		Longitude: 128.3860015869,
		Timezone:  "Asia/Seoul",
	},
	"RK34": {
		ICAO:      "RK34",
		Name:      "G 417 Airport",
		Country:   "KR",
		Elevation: 1778,
		Latitude:  37.6502990723,
		Longitude: 128.5700073242,
		Timezone:  "Asia/Seoul",
	},
	"RK35": {
		ICAO:      "RK35",
		Name:      "G 225 Airport",
		State:     "Gyeonggi-do",
		Country:   "KR",
		Elevation: 138,
		Latitude:  38.018901825,
		Longitude: 126.9769973755,
		Timezone:  "Asia/Seoul",
	},
	"RK36": {
		ICAO:      "RK36",
		Name:      "G 238 Airport",
		Country:   "KR",
		Elevation: 262,
		Latitude:  38.1766014099,
		Longitude: 127.1029968262,
		Timezone:  "Asia/Seoul",
	},
	"RK37": {
		ICAO:      "RK37",
		Name:      "G 227 Airport",
		Country:   "KR",
		Elevation: 220,
		Latitude:  38.0255012512,
		Longitude: 127.1119995117,
		Timezone:  "Asia/Seoul",
	},
	"RK38": {
		ICAO:      "RK38",
		Name:      "G 228 Airport",
		Country:   "KR",
		Elevation: 228,
		Latitude:  38.029800415,
		Longitude: 127.1409988403,
		Timezone:  "Asia/Seoul",
	},
	"RK39": {
		ICAO:      "RK39",
		Name:      "G 239 Airport",
		State:     "Gangwon-do",
		Country:   "KR",
		Elevation: 630,
		Latitude:  38.1991996765,
		Longitude: 127.2220001221,
		Timezone:  "Asia/Seoul",
	},
	"RK40": {
		ICAO:      "RK40",
		Name:      "G 240 Airport",
		Country:   "KR",
		Elevation: 630,
		Latitude:  38.2489013672,
		Longitude: 127.3769989014,
		Timezone:  "Asia/Seoul",
	},
	"RK41": {
		ICAO:      "RK41",
		Name:      "G 317 Airport",
		Country:   "KR",
		Elevation: 750,
		Latitude:  38.215801239,
		Longitude: 127.6539993286,
		Timezone:  "Asia/Seoul",
	},
	"RK42": {
		ICAO:      "RK42",
		Name:      "G 311 Airport",
		Country:   "KR",
		Elevation: 590,
		Latitude:  38.0567016602,
		Longitude: 127.7949981689,
		Timezone:  "Asia/Seoul",
	},
	"RK43": {
		ICAO:      "RK43",
		Name:      "G 414 Airport",
		Country:   "KR",
		Elevation: 688,
		Latitude:  38.1054992676,
		Longitude: 128.1999969482,
		Timezone:  "Asia/Seoul",
	},
	"RK44": {
		ICAO:      "RK44",
		Name:      "G 412 Airport",
		State:     "Gangwon-do",
		Country:   "KR",
		Elevation: 950,
		Latitude:  38.2416992188,
		Longitude: 128.2079925537,
		Timezone:  "Asia/Seoul",
	},
	"RK48": {
		ICAO:      "RK48",
		Name:      "G 419 Airport",
		Country:   "KR",
		Elevation: 450,
		Latitude:  37.7032012939,
		Longitude: 127.9049987793,
		Timezone:  "Asia/Seoul",
	},
	"RK49": {
		ICAO:      "RK49",
		Name:      "G 530 Airport",
		Country:   "KR",
		Elevation: 91,
		Latitude:  36.7560005188,
		Longitude: 126.3300018311,
		Timezone:  "Asia/Seoul",
	},
	"RK50": {
		ICAO:      "RK50",
		Name:      "G 526 Airport",
		State:     "Chungcheongnam-do",
		Country:   "KR",
		Elevation: 197,
		Latitude:  36.5838012695,
		Longitude: 126.6600036621,
		Timezone:  "Asia/Seoul",
	},
	"RK52": {
		ICAO:      "RK52",
		Name:      "G 501 Airport",
		Country:   "KR",
		Elevation: 243,
		Latitude:  37.2868003845,
		Longitude: 127.2269973755,
		Timezone:  "Asia/Seoul",
	},
	"RK58": {
		ICAO:      "RK58",
		Name:      "G 806 Airport",
		State:     "Gyeongsangbuk-do",
		Country:   "KR",
		Elevation: 86,
		Latitude:  35.8566017151,
		Longitude: 129.2120056152,
		Timezone:  "Asia/Seoul",
	},
	"RK5M": {
		ICAO:      "RK5M",
		Name:      "C 719 Heliport",
		State:     "Jeollabuk-do",
		Country:   "KR",
		Elevation: 132,
		Latitude:  35.876108,
		Longitude: 127.132562,
		Timezone:  "Asia/Seoul",
	},
	"RK60": {
		ICAO:      "RK60",
		Name:      "G 712 Airport",
		Country:   "KR",
		Elevation: 67,
		Latitude:  35.3067016602,
		Longitude: 126.4950027466,
		Timezone:  "Asia/Seoul",
	},
	"RK6D": {
		ICAO:      "RK6D",
		Name:      "G 710 Airport",
		Country:   "KR",
		Elevation: 173,
		Latitude:  35.3420982361,
		Longitude: 127.0299987793,
		Timezone:  "Asia/Seoul",
	},
	"RK6O": {
		ICAO:      "RK6O",
		Name:      "G 605 Airport",
		Country:   "KR",
		Elevation: 925,
		Latitude:  37.1628990173,
		Longitude: 128.2189941406,
		Timezone:  "Asia/Seoul",
	},
	"RK82": {
		ICAO:      "RK82",
		Name:      "G 405 Airport",
		Country:   "KR",
		Elevation: 721,
		Latitude:  38.1186981201,
		Longitude: 128.0390014648,
		Timezone:  "Asia/Seoul",
	},
	"RKD1": {
		ICAO:      "RKD1",
		Name:      "G 802 Airport",
		City:      "Masan",
		Country:   "KR",
		Elevation: 110,
		Latitude:  35.2582015991,
		Longitude: 128.6260070801,
		Timezone:  "Asia/Seoul",
	},
	"RKJB": {
		ICAO:      "RKJB",
		IATA:      "MWX",
		Name:      "Muan International Airport",
		State:     "Jeollanam-do",
		Country:   "KR",
		Latitude:  34.991406,
		Longitude: 126.382814,
		Timezone:  "Asia/Seoul",
	},
	"RKJJ": {
		ICAO:      "RKJJ",
		IATA:      "KWJ",
		Name:      "Gwangju Airport",
		City:      "Gwangju",
		State:     "Gwangju",
		Country:   "KR",
		Elevation: 39,
		Latitude:  35.1263999939,
		Longitude: 126.808998108,
		Timezone:  "Asia/Seoul",
	},
	"RKJK": {
		ICAO:      "RKJK",
		IATA:      "KUV",
		Name:      "Kunsan Air Base",
		City:      "Kunsan",
		State:     "Jeollabuk-do",
		Country:   "KR",
		Elevation: 29,
		Latitude:  35.9038009644,
		Longitude: 126.6159973145,
		Timezone:  "Asia/Seoul",
	},
	"RKJU": {
		ICAO:      "RKJU",
		IATA:      "CHN",
		Name:      "Jeon Ju Airport",
		City:      "Jeon Ju",
		State:     "Jeollabuk-do",
		Country:   "KR",
		Elevation: 96,
		Latitude:  35.8783988953,
		Longitude: 127.1200027466,
		Timezone:  "Asia/Seoul",
	},
	"RKJY": {
		ICAO:      "RKJY",
		IATA:      "RSU",
		Name:      "Yeosu Airport",
		City:      "Yeosu",
		State:     "Jeollanam-do",
		Country:   "KR",
		Elevation: 53,
		Latitude:  34.842300415,
		Longitude: 127.6169967651,
		Timezone:  "Asia/Seoul",
	},
	"RKNC": {
		ICAO:      "RKNC",
		IATA:      "QUN",
		Name:      "A-306 Airport",
		City:      "Chun Chon City",
		State:     "Gangwon-do",
		Country:   "KR",
		Elevation: 245,
		Latitude:  37.8838005066,
		Longitude: 127.7180023193,
		Timezone:  "Asia/Seoul",
	},
	"RKND": {
		ICAO:      "RKND",
		IATA:      "SHO",
		Name:      "Sokcho Airport",
		State:     "Gangwon-do",
		Country:   "KR",
		Elevation: 92,
		Latitude:  38.1426010132,
		Longitude: 128.5989990234,
		Timezone:  "Asia/Seoul",
	},
	"RKNN": {
		ICAO:      "RKNN",
		IATA:      "KAG",
		Name:      "Gangneung Airport",
		State:     "Gangwon-do",
		Country:   "KR",
		Elevation: 35,
		Latitude:  37.7536010742,
		Longitude: 128.9440002441,
		Timezone:  "Asia/Seoul",
	},
	"RKNW": {
		ICAO:      "RKNW",
		IATA:      "WJU",
		Name:      "Wonju Airport",
		City:      "Wonju",
		State:     "Gangwon-do",
		Country:   "KR",
		Elevation: 329,
		Latitude:  37.4380989075,
		Longitude: 127.959999084,
		Timezone:  "Asia/Seoul",
	},
	"RKNY": {
		ICAO:      "RKNY",
		IATA:      "YNY",
		Name:      "Yangyang International Airport",
		City:      "Sokcho / Gangneung",
		State:     "Gangwon-do",
		Country:   "KR",
		Elevation: 241,
		Latitude:  38.0612983704,
		Longitude: 128.6690063477,
		Timezone:  "Asia/Seoul",
	},
	"RKPC": {
		ICAO:      "RKPC",
		IATA:      "CJU",
		Name:      "Jeju International Airport",
		City:      "Jeju City",
		State:     "Jeju-do",
		Country:   "KR",
		Elevation: 118,
		Latitude:  33.5112991333,
		Longitude: 126.4929962158,
		Timezone:  "Asia/Seoul",
	},
	"RKPD": {
		ICAO:      "RKPD",
		IATA:      "JDG",
		Name:      "Jeongseok Airport",
		State:     "Jeju-do",
		Country:   "KR",
		Elevation: 1171,
		Latitude:  33.3996009827,
		Longitude: 126.7119979858,
		Timezone:  "Asia/Seoul",
	},
	"RKPE": {
		ICAO:      "RKPE",
		IATA:      "CHF",
		Name:      "Jinhae Airport",
		City:      "Jinhae",
		State:     "Gyeongsangnam-do",
		Country:   "KR",
		Elevation: 8,
		Latitude:  35.1412010193,
		Longitude: 128.6959991455,
		Timezone:  "Asia/Seoul",
	},
	"RKPK": {
		ICAO:      "RKPK",
		IATA:      "PUS",
		Name:      "Gimhae International Airport",
		City:      "Busan",
		State:     "Busan",
		Country:   "KR",
		Elevation: 6,
		Latitude:  35.1795005798,
		Longitude: 128.93800354,
		Timezone:  "Asia/Seoul",
	},
	"RKPS": {
		ICAO:      "RKPS",
		IATA:      "HIN",
		Name:      "Sacheon Air Base",
		City:      "Sacheon",
		State:     "Gyeongsangnam-do",
		Country:   "KR",
		Elevation: 25,
		Latitude:  35.0885009766,
		Longitude: 128.0700073242,
		Timezone:  "Asia/Seoul",
	},
	"RKPU": {
		ICAO:      "RKPU",
		IATA:      "USN",
		Name:      "Ulsan Airport",
		City:      "Ulsan",
		State:     "Ulsan",
		Country:   "KR",
		Elevation: 45,
		Latitude:  35.59349823,
		Longitude: 129.352005005,
		Timezone:  "Asia/Seoul",
	},
	"RKRA": {
		ICAO:      "RKRA",
		Name:      "G 222 Airport",
		Country:   "KR",
		Elevation: 287,
		Latitude:  37.8303985596,
		Longitude: 126.9899978638,
		Timezone:  "Asia/Seoul",
	},
	"RKRB": {
		ICAO:      "RKRB",
		Name:      "G 103 Airport",
		Country:   "KR",
		Elevation: 92,
		Latitude:  37.474899292,
		Longitude: 126.7470016479,
		Timezone:  "Asia/Seoul",
	},
	"RKRG": {
		ICAO:      "RKRG",
		Name:      "G 301 Airport",
		State:     "Gyeonggi-do",
		Country:   "KR",
		Elevation: 230,
		Latitude:  37.5003013611,
		Longitude: 127.6190032959,
		Timezone:  "Asia/Seoul",
	},
	"RKRK": {
		ICAO:      "RKRK",
		Name:      "G 213 Airport",
		Country:   "KR",
		Elevation: 400,
		Latitude:  37.8116989136,
		Longitude: 127.3570022583,
		Timezone:  "Asia/Seoul",
	},
	"RKRN": {
		ICAO:      "RKRN",
		Name:      "G 510 Airport",
		Country:   "KR",
		Elevation: 255,
		Latitude:  37.1968994141,
		Longitude: 127.4749984741,
		Timezone:  "Asia/Seoul",
	},
	"RKRO": {
		ICAO:      "RKRO",
		Name:      "G 217 Airport",
		State:     "Gyeonggi-do",
		Country:   "KR",
		Elevation: 328,
		Latitude:  37.8665008545,
		Longitude: 127.1750030518,
		Timezone:  "Asia/Seoul",
	},
	"RKRP": {
		ICAO:      "RKRP",
		Name:      "G 110 Airport",
		Country:   "KR",
		Elevation: 65,
		Latitude:  37.7625999451,
		Longitude: 126.7929992676,
		Timezone:  "Asia/Seoul",
	},
	"RKSG": {
		ICAO:      "RKSG",
		Name:      "A 511 Airport",
		State:     "Gyeonggi-do",
		Country:   "KR",
		Elevation: 51,
		Latitude:  36.9622001648,
		Longitude: 127.03099823,
		Timezone:  "Asia/Seoul",
	},
	"RKSI": {
		ICAO:      "RKSI",
		IATA:      "ICN",
		Name:      "Incheon International Airport",
		City:      "Seoul",
		State:     "Incheon",
		Country:   "KR",
		Elevation: 23,
		Latitude:  37.4691009521,
		Longitude: 126.4509963989,
		Timezone:  "Asia/Seoul",
	},
	"RKSM": {
		ICAO:      "RKSM",
		IATA:      "SSN",
		Name:      "Seoul Air Base",
		State:     "Gyeonggi-do",
		Country:   "KR",
		Elevation: 92,
		Latitude:  37.4458007813,
		Longitude: 127.1139984131,
		Timezone:  "Asia/Seoul",
	},
	"RKSO": {
		ICAO:      "RKSO",
		IATA:      "OSN",
		Name:      "Osan Air Base",
		State:     "Gyeonggi-do",
		Country:   "KR",
		Elevation: 38,
		Latitude:  37.0905990601,
		Longitude: 127.0299987793,
		Timezone:  "Asia/Seoul",
	},
	"RKSS": {
		ICAO:      "RKSS",
		IATA:      "GMP",
		Name:      "Gimpo International Airport",
		City:      "Seoul",
		State:     "Seoul",
		Country:   "KR",
		Elevation: 58,
		Latitude:  37.5583000183,
		Longitude: 126.791000366,
		Timezone:  "Asia/Seoul",
	},
	"RKSW": {
		ICAO:      "RKSW",
		IATA:      "SWU",
		Name:      "Suwon Airport",
		State:     "Gyeonggi-do",
		Country:   "KR",
		Elevation: 88,
		Latitude:  37.2393989563,
		Longitude: 127.0070037842,
		Timezone:  "Asia/Seoul",
	},
	"RKTE": {
		ICAO:      "RKTE",
		Name:      "Seongmu Airport",
		State:     "North-Chungcheong",
		Country:   "KR",
		Elevation: 258,
		Latitude:  36.5681991577,
		Longitude: 127.5,
		Timezone:  "Asia/Seoul",
	},
	"RKTH": {
		ICAO:      "RKTH",
		IATA:      "KPO",
		Name:      "Pohang Airport",
		City:      "Pohang",
		State:     "Gyeongsangbuk-do",
		Country:   "KR",
		Elevation: 70,
		Latitude:  35.9878997803,
		Longitude: 129.419998169,
		Timezone:  "Asia/Seoul",
	},
	"RKTI": {
		ICAO:      "RKTI",
		IATA:      "JWO",
		Name:      "Jungwon Air Base",
		State:     "North-Chungcheong",
		Country:   "KR",
		Elevation: 281,
		Latitude:  37.0299987793,
		Longitude: 127.8850021362,
		Timezone:  "Asia/Seoul",
	},
	"RKTN": {
		ICAO:      "RKTN",
		IATA:      "TAE",
		Name:      "Daegu Airport",
		City:      "Daegu",
		State:     "Daegu",
		Country:   "KR",
		Elevation: 116,
		Latitude:  35.8941001892,
		Longitude: 128.658996582,
		Timezone:  "Asia/Seoul",
	},
	"RKTP": {
		ICAO:      "RKTP",
		IATA:      "HMY",
		Name:      "Seosan Air Base",
		City:      "Seosan",
		State:     "Chungcheongnam-do",
		Country:   "KR",
		Elevation: 39,
		Latitude:  36.7039985657,
		Longitude: 126.486000061,
		Timezone:  "Asia/Seoul",
	},
	"RKTU": {
		ICAO:      "RKTU",
		IATA:      "CJJ",
		Name:      "Cheongju International Airport",
		City:      "Cheongju",
		State:     "North-Chungcheong",
		Country:   "KR",
		Elevation: 191,
		Latitude:  36.7165985107,
		Longitude: 127.499000549,
		Timezone:  "Asia/Seoul",
	},
	"RKTY": {
		ICAO:      "RKTY",
		IATA:      "YEC",
		Name:      "Yecheon Airport",
		State:     "Gyeongsangbuk-do",
		Country:   "KR",
		Elevation: 354,
		Latitude:  36.6319007874,
		Longitude: 128.3549957275,
		Timezone:  "Asia/Seoul",
	},
	"RKUC": {
		ICAO:      "RKUC",
		Name:      "G 505 Airport",
		Country:   "KR",
		Elevation: 82,
		Latitude:  36.5716018677,
		Longitude: 127.2949981689,
		Timezone:  "Asia/Seoul",
	},
	"RKUL": {
		ICAO:      "RKUL",
		Name:      "G 536 Airport",
		Country:   "KR",
		Elevation: 104,
		Latitude:  36.2694015503,
		Longitude: 127.1139984131,
		Timezone:  "Asia/Seoul",
	},
	"ROAH": {
		ICAO:      "ROAH",
		IATA:      "OKA",
		Name:      "Naha Airport",
		City:      "Naha",
		State:     "Okinawa",
		Country:   "JP",
		Elevation: 12,
		Latitude:  26.1958007812,
		Longitude: 127.646003723,
		Timezone:  "Asia/Tokyo",
	},
	"RODE": {
		ICAO:      "RODE",
		Name:      "Ie Shima Auxiliary Air Base",
		State:     "Okinawa",
		Country:   "JP",
		Elevation: 184,
		Latitude:  26.7257575989,
		Longitude: 127.7649002075,
		Timezone:  "Asia/Tokyo",
	},
	"RODN": {
		ICAO:      "RODN",
		IATA:      "DNA",
		Name:      "Kadena Air Base",
		State:     "Okinawa",
		Country:   "JP",
		Elevation: 143,
		Latitude:  26.3556003571,
		Longitude: 127.7679977417,
		Timezone:  "Asia/Tokyo",
	},
	"ROIG": {
		ICAO:      "ROIG",
		IATA:      "ISG",
		Name:      "Ishigaki Airport",
		City:      "Ishigaki",
		State:     "Okinawa",
		Country:   "JP",
		Elevation: 93,
		Latitude:  24.344499588,
		Longitude: 124.18699646,
		Timezone:  "Asia/Tokyo",
	},
	"ROKJ": {
		ICAO:      "ROKJ",
		IATA:      "UEO",
		Name:      "Kumejima Airport",
		State:     "Okinawa",
		Country:   "JP",
		Elevation: 23,
		Latitude:  26.3635005951,
		Longitude: 126.7139968872,
		Timezone:  "Asia/Tokyo",
	},
	"ROKR": {
		ICAO:      "ROKR",
		IATA:      "KJP",
		Name:      "Kerama Airport",
		City:      "Kerama",
		State:     "Okinawa",
		Country:   "JP",
		Elevation: 156,
		Latitude:  26.1683006287,
		Longitude: 127.2929992676,
		Timezone:  "Asia/Tokyo",
	},
	"ROMD": {
		ICAO:      "ROMD",
		IATA:      "MMD",
		Name:      "Minami Daito Airport",
		State:     "Okinawa",
		Country:   "JP",
		Elevation: 167,
		Latitude:  25.8465003967,
		Longitude: 131.2630004883,
		Timezone:  "Asia/Tokyo",
	},
	"ROMY": {
		ICAO:      "ROMY",
		IATA:      "MMY",
		Name:      "Miyako Airport",
		City:      "Miyako City",
		State:     "Okinawa",
		Country:   "JP",
		Elevation: 150,
		Latitude:  24.7828006744,
		Longitude: 125.294998169,
		Timezone:  "Asia/Tokyo",
	},
	"RORA": {
		ICAO:      "RORA",
		IATA:      "AGJ",
		Name:      "Aguni Airport",
		City:      "Aguni",
		State:     "Okinawa",
		Country:   "JP",
		Elevation: 38,
		Latitude:  26.5925006866,
		Longitude: 127.2409973145,
		Timezone:  "Asia/Tokyo",
	},
	"RORE": {
		ICAO:      "RORE",
		IATA:      "IEJ",
		Name:      "Ie Jima Airport",
		State:     "Okinawa",
		Country:   "JP",
		Elevation: 246,
		Latitude:  26.7220001221,
		Longitude: 127.7850036621,
		Timezone:  "Asia/Tokyo",
	},
	"RORH": {
		ICAO:      "RORH",
		IATA:      "HTR",
		Name:      "Hateruma Airport",
		City:      "Hateruma",
		State:     "Okinawa",
		Country:   "JP",
		Elevation: 43,
		Latitude:  24.0589008331,
		Longitude: 123.8059997559,
		Timezone:  "Asia/Tokyo",
	},
	"RORK": {
		ICAO:      "RORK",
		IATA:      "KTD",
		Name:      "Kitadaito Airport",
		State:     "Okinawa",
		Country:   "JP",
		Elevation: 80,
		Latitude:  25.9447002411,
		Longitude: 131.3269958496,
		Timezone:  "Asia/Tokyo",
	},
	"RORS": {
		ICAO:      "RORS",
		IATA:      "SHI",
		Name:      "Shimojishima Airport",
		State:     "Okinawa",
		Country:   "JP",
		Elevation: 54,
		Latitude:  24.8267002106,
		Longitude: 125.1449966431,
		Timezone:  "Asia/Tokyo",
	},
	"RORT": {
		ICAO:      "RORT",
		IATA:      "TRA",
		Name:      "Tarama Airport",
		State:     "Okinawa",
		Country:   "JP",
		Elevation: 36,
		Latitude:  24.6539001465,
		Longitude: 124.6750030518,
		Timezone:  "Asia/Tokyo",
	},
	"RORY": {
		ICAO:      "RORY",
		IATA:      "RNJ",
		Name:      "Yoron Airport",
		State:     "Kagoshima",
		Country:   "JP",
		Elevation: 52,
		Latitude:  27.0440006256,
		Longitude: 128.4019927979,
		Timezone:  "Asia/Tokyo",
	},
	"ROTM": {
		ICAO:      "ROTM",
		Name:      "Futenma Marine Corps Air Station",
		State:     "Okinawa",
		Country:   "JP",
		Elevation: 247,
		Latitude:  26.2742996216,
		Longitude: 127.7559967041,
		Timezone:  "Asia/Tokyo",
	},
	"ROYN": {
		ICAO:      "ROYN",
		IATA:      "OGN",
		Name:      "Yonaguni Airport",
		State:     "Okinawa",
		Country:   "JP",
		Elevation: 70,
		Latitude:  24.4668998718,
		Longitude: 122.9779968262,
		Timezone:  "Asia/Tokyo",
	},
	"RP12": {
		ICAO:      "RP12",
		Name:      "Pagbilao Grande Island Airport",
		State:     "Calabarzon",
		Country:   "PH",
		Elevation: 13,
		Latitude:  13.9001998901,
		Longitude: 121.7440032959,
		Timezone:  "Asia/Manila",
	},
	"RP13": {
		ICAO:      "RP13",
		Name:      "Nonoc Airport",
		State:     "Caraga",
		Country:   "PH",
		Elevation: 10,
		Latitude:  9.8092603683,
		Longitude: 125.5930023193,
		Timezone:  "Asia/Manila",
	},
	"RP15": {
		ICAO:      "RP15",
		Name:      "Pasar Airport",
		State:     "Eastern-Visayas",
		Country:   "PH",
		Elevation: 16,
		Latitude:  10.9120998383,
		Longitude: 124.43699646,
		Timezone:  "Asia/Manila",
	},
	"RP16": {
		ICAO:      "RP16",
		Name:      "Seahawk Lz Airport",
		State:     "ARMM",
		Country:   "PH",
		Elevation: 1011,
		Latitude:  6.6541700363,
		Longitude: 121.9800033569,
		Timezone:  "Asia/Manila",
	},
	"RP17": {
		ICAO:      "RP17",
		Name:      "Malita Airport",
		State:     "Davao",
		Country:   "PH",
		Elevation: 16,
		Latitude:  6.4022798538,
		Longitude: 125.6190032959,
		Timezone:  "Asia/Manila",
	},
	"RPLA": {
		ICAO:      "RPLA",
		Name:      "Pinamalayan Airport",
		State:     "Mimaropa",
		Country:   "PH",
		Elevation: 92,
		Latitude:  12.9857997894,
		Longitude: 121.4260025024,
		Timezone:  "Asia/Manila",
	},
	"RPLB": {
		ICAO:      "RPLB",
		IATA:      "SFS",
		Name:      "Subic Bay International Airport",
		City:      "Olongapo City",
		State:     "Central-Luzon",
		Country:   "PH",
		Elevation: 64,
		Latitude:  14.7944002151,
		Longitude: 120.2710037231,
		Timezone:  "Asia/Manila",
	},
	"RPLC": {
		ICAO:      "RPLC",
		IATA:      "CRK",
		Name:      "Diosdado Macapagal International Airport",
		City:      "Angeles City",
		State:     "Central-Luzon",
		Country:   "PH",
		Elevation: 484,
		Latitude:  15.1859998703,
		Longitude: 120.559997559,
		Timezone:  "Asia/Manila",
	},
	"RPLI": {
		ICAO:      "RPLI",
		IATA:      "LAO",
		Name:      "Laoag International Airport",
		City:      "Laoag City",
		State:     "Ilocos",
		Country:   "PH",
		Elevation: 25,
		Latitude:  18.1781005859,
		Longitude: 120.5319976807,
		Timezone:  "Asia/Manila",
	},
	"RPLJ": {
		ICAO:      "RPLJ",
		Name:      "Jomalig Island Airport",
		City:      "Jomalig",
		State:     "Calabarzon",
		Country:   "PH",
		Elevation: 36,
		Latitude:  14.704000473,
		Longitude: 122.3310012817,
		Timezone:  "Asia/Manila",
	},
	"RPLL": {
		ICAO:      "RPLL",
		IATA:      "MNL",
		Name:      "Ninoy Aquino International Airport",
		City:      "Manila",
		Country:   "PH",
		Elevation: 75,
		Latitude:  14.508600235,
		Longitude: 121.019996643,
		Timezone:  "Asia/Manila",
	},
	"RPLN": {
		ICAO:      "RPLN",
		Name:      "Palanan Municipal Airport",
		City:      "Palanan",
		State:     "Cagayan-Valley",
		Country:   "PH",
		Elevation: 52,
		Latitude:  17.0656668941,
		Longitude: 122.427670956,
		Timezone:  "Asia/Manila",
	},
	"RPLO": {
		ICAO:      "RPLO",
		IATA:      "CYU",
		Name:      "Cuyo Airport",
		City:      "Cuyo",
		State:     "Mimaropa",
		Country:   "PH",
		Latitude:  10.8580999374,
		Longitude: 121.0690002441,
		Timezone:  "Asia/Manila",
	},
	"RPLP": {
		ICAO:      "RPLP",
		IATA:      "LGP",
		Name:      "Legazpi City International Airport",
		City:      "Legazpi City",
		State:     "Bicol",
		Country:   "PH",
		Elevation: 66,
		Latitude:  13.1575,
		Longitude: 123.735,
		Timezone:  "Asia/Manila",
	},
	"RPLR": {
		ICAO:      "RPLR",
		Name:      "Carmen Rosales Airstrip",
		City:      "Carmen",
		State:     "Ilocos",
		Country:   "PH",
		Latitude:  15.8843421975,
		Longitude: 120.601944923,
		Timezone:  "Asia/Manila",
	},
	"RPLS": {
		ICAO:      "RPLS",
		IATA:      "NSP",
		Name:      "Sangley Point Air Base",
		City:      "Cavite City",
		State:     "Calabarzon",
		Country:   "PH",
		Elevation: 8,
		Latitude:  14.4954004288,
		Longitude: 120.9039993286,
		Timezone:  "Asia/Manila",
	},
	"RPLT": {
		ICAO:      "RPLT",
		Name:      "Itbayat Airport",
		City:      "Itbayat",
		State:     "ARMM",
		Country:   "PH",
		Elevation: 328,
		Latitude:  20.716999054,
		Longitude: 121.8170013428,
		Timezone:  "Asia/Manila",
	},
	"RPLU": {
		ICAO:      "RPLU",
		IATA:      "LBX",
		Name:      "Lubang Airport",
		State:     "Mimaropa",
		Country:   "PH",
		Elevation: 43,
		Latitude:  13.8554000854,
		Longitude: 120.1050033569,
		Timezone:  "Asia/Manila",
	},
	"RPLV": {
		ICAO:      "RPLV",
		Name:      "Fort Magsaysay Airport",
		State:     "Central-Luzon",
		Country:   "PH",
		Elevation: 200,
		Latitude:  15.4347000122,
		Longitude: 121.091003418,
		Timezone:  "Asia/Manila",
	},
	"RPLX": {
		ICAO:      "RPLX",
		Name:      "Corregidor Airport",
		City:      "Corregidor",
		State:     "Central-Luzon",
		Country:   "PH",
		Elevation: 102,
		Latitude:  14.3913002014,
		Longitude: 120.6070022583,
		Timezone:  "Asia/Manila",
	},
	"RPLY": {
		ICAO:      "RPLY",
		Name:      "Alabat Island Airport",
		City:      "Alabat Island",
		State:     "Calabarzon",
		Country:   "PH",
		Elevation: 6,
		Latitude:  14.232000351,
		Longitude: 121.9290008545,
		Timezone:  "Asia/Manila",
	},
	"RPLZ": {
		ICAO:      "RPLZ",
		Name:      "Sorsogon",
		State:     "Bicol",
		Country:   "PH",
		Latitude:  12.998588562,
		Longitude: 124.018081665,
		Timezone:  "Asia/Manila",
	},
	"RPMA": {
		ICAO:      "RPMA",
		IATA:      "AAV",
		Name:      "Allah Valley Airport",
		City:      "Surallah",
		State:     "Soccsksargen",
		Country:   "PH",
		Elevation: 659,
		Latitude:  6.3668198586,
		Longitude: 124.7509994507,
		Timezone:  "Asia/Manila",
	},
	"RPMB": {
		ICAO:      "RPMB",
		IATA:      "GES",
		Name:      "General Santos International Airport",
		City:      "General Santos City",
		State:     "Soccsksargen",
		Country:   "PH",
		Elevation: 28,
		Latitude:  6.1064400673,
		Longitude: 125.2350006104,
		Timezone:  "Asia/Manila",
	},
	"RPMC": {
		ICAO:      "RPMC",
		IATA:      "CBO",
		Name:      "Awang Airport",
		City:      "Cotabato City",
		State:     "ARMM",
		Country:   "PH",
		Elevation: 189,
		Latitude:  7.1652398109,
		Longitude: 124.2099990845,
		Timezone:  "Asia/Manila",
	},
	"RPMD": {
		ICAO:      "RPMD",
		IATA:      "DVO",
		Name:      "Francisco Bangoy International Airport",
		City:      "Davao City",
		State:     "Davao",
		Country:   "PH",
		Elevation: 96,
		Latitude:  7.1255202293,
		Longitude: 125.6460037231,
		Timezone:  "Asia/Manila",
	},
	"RPME": {
		ICAO:      "RPME",
		IATA:      "BXU",
		Name:      "Bancasi Airport",
		City:      "Butuan City",
		State:     "Caraga",
		Country:   "PH",
		Elevation: 141,
		Latitude:  8.9513196945,
		Longitude: 125.4779968262,
		Timezone:  "Asia/Manila",
	},
	"RPMF": {
		ICAO:      "RPMF",
		IATA:      "BPH",
		Name:      "Bislig Airport",
		State:     "Caraga",
		Country:   "PH",
		Elevation: 12,
		Latitude:  8.1959495544,
		Longitude: 126.3219985962,
		Timezone:  "Asia/Manila",
	},
	"RPMG": {
		ICAO:      "RPMG",
		IATA:      "DPL",
		Name:      "Dipolog Airport",
		City:      "Dipolog City",
		State:     "Zamboanga-Peninsula",
		Country:   "PH",
		Elevation: 12,
		Latitude:  8.6019834988,
		Longitude: 123.341875076,
		Timezone:  "Asia/Manila",
	},
	"RPMH": {
		ICAO:      "RPMH",
		IATA:      "CGM",
		Name:      "Camiguin Airport",
		State:     "Northern-Mindanao",
		Country:   "PH",
		Elevation: 53,
		Latitude:  9.2535200119,
		Longitude: 124.7070007324,
		Timezone:  "Asia/Manila",
	},
	"RPMI": {
		ICAO:      "RPMI",
		IATA:      "IGN",
		Name:      "Iligan Airport",
		Country:   "PH",
		Elevation: 1300,
		Latitude:  8.130490303,
		Longitude: 124.2149963379,
		Timezone:  "Asia/Manila",
	},
	"RPMJ": {
		ICAO:      "RPMJ",
		IATA:      "JOL",
		Name:      "Jolo Airport",
		State:     "ARMM",
		Country:   "PH",
		Elevation: 118,
		Latitude:  6.0536699295,
		Longitude: 121.0110015869,
		Timezone:  "Asia/Manila",
	},
	"RPML": {
		ICAO:      "RPML",
		IATA:      "CGY",
		Name:      "Cagayan De Oro Airport",
		City:      "Cagayan De Oro City",
		State:     "Northern-Mindanao",
		Country:   "PH",
		Elevation: 601,
		Latitude:  8.4156198502,
		Longitude: 124.611000061,
		Timezone:  "Asia/Manila",
	},
	"RPMM": {
		ICAO:      "RPMM",
		IATA:      "MLP",
		Name:      "Malabang Airport",
		City:      "Malabang",
		State:     "ARMM",
		Country:   "PH",
		Elevation: 27,
		Latitude:  7.6172199249,
		Longitude: 124.0589981079,
		Timezone:  "Asia/Manila",
	},
	"RPMN": {
		ICAO:      "RPMN",
		IATA:      "SGS",
		Name:      "Sanga Sanga Airport",
		State:     "ARMM",
		Country:   "PH",
		Elevation: 15,
		Latitude:  5.0469899178,
		Longitude: 119.7429962158,
		Timezone:  "Asia/Manila",
	},
	"RPMO": {
		ICAO:      "RPMO",
		IATA:      "OZC",
		Name:      "Labo Airport",
		City:      "Ozamiz City",
		State:     "Northern-Mindanao",
		Country:   "PH",
		Elevation: 75,
		Latitude:  8.1785097122,
		Longitude: 123.8420028687,
		Timezone:  "Asia/Manila",
	},
	"RPMP": {
		ICAO:      "RPMP",
		IATA:      "PAG",
		Name:      "Pagadian Airport",
		City:      "Pagadian City",
		State:     "Zamboanga-Peninsula",
		Country:   "PH",
		Elevation: 5,
		Latitude:  7.8307314479,
		Longitude: 123.461179733,
		Timezone:  "Asia/Manila",
	},
	"RPMQ": {
		ICAO:      "RPMQ",
		IATA:      "MXI",
		Name:      "Mati National Airport",
		State:     "Davao",
		Country:   "PH",
		Elevation: 156,
		Latitude:  6.9493699074,
		Longitude: 126.2730026245,
		Timezone:  "Asia/Manila",
	},
	"RPMR": {
		ICAO:      "RPMR",
		IATA:      "GES",
		Name:      "Tambler Airport",
		State:     "Soccsksargen",
		Country:   "PH",
		Elevation: 505,
		Latitude:  6.0580000877,
		Longitude: 125.0960006714,
		Timezone:  "Asia/Manila",
	},
	"RPMS": {
		ICAO:      "RPMS",
		IATA:      "SUG",
		Name:      "Surigao Airport",
		City:      "Surigao City",
		State:     "Caraga",
		Country:   "PH",
		Elevation: 20,
		Latitude:  9.7558383256,
		Longitude: 125.480947495,
		Timezone:  "Asia/Manila",
	},
	"RPMU": {
		ICAO:      "RPMU",
		IATA:      "CDY",
		Name:      "Cagayan de Sulu Airport",
		City:      "Mapun",
		State:     "ARMM",
		Country:   "PH",
		Latitude:  7.013999939,
		Longitude: 118.4960021973,
		Timezone:  "Asia/Manila",
	},
	"RPMV": {
		ICAO:      "RPMV",
		IATA:      "IPE",
		Name:      "Ipil Airport",
		City:      "Ipil",
		State:     "Zamboanga-Peninsula",
		Country:   "PH",
		Latitude:  7.7855601311,
		Longitude: 122.6019973755,
		Timezone:  "Asia/Manila",
	},
	"RPMW": {
		ICAO:      "RPMW",
		IATA:      "TDG",
		Name:      "Tandag Airport",
		State:     "Caraga",
		Country:   "PH",
		Elevation: 16,
		Latitude:  9.0721101761,
		Longitude: 126.1709976196,
		Timezone:  "Asia/Manila",
	},
	"RPMY": {
		ICAO:      "RPMY",
		IATA:      "CGY",
		Name:      "Malaybalay Airport",
		City:      "Malaybalay City",
		State:     "Northern-Mindanao",
		Country:   "PH",
		Elevation: 2051,
		Latitude:  8.1416597366,
		Longitude: 125.1169967651,
		Timezone:  "Asia/Manila",
	},
	"RPMZ": {
		ICAO:      "RPMZ",
		IATA:      "ZAM",
		Name:      "Zamboanga International Airport",
		City:      "Zamboanga City",
		State:     "Zamboanga-Peninsula",
		Country:   "PH",
		Elevation: 33,
		Latitude:  6.9224200249,
		Longitude: 122.0599975586,
		Timezone:  "Asia/Manila",
	},
	"RPNS": {
		ICAO:      "RPNS",
		IATA:      "IAO",
		Name:      "Siargao Airport",
		City:      "Del Carmen",
		State:     "Caraga",
		Country:   "PH",
		Elevation: 10,
		Latitude:  9.8591003418,
		Longitude: 126.013999939,
		Timezone:  "Asia/Manila",
	},
	"RPSB": {
		ICAO:      "RPSB",
		Name:      "Bantayan Airport",
		State:     "Central-Visayas",
		Country:   "PH",
		Latitude:  11.1624,
		Longitude: 123.7848,
		Timezone:  "Asia/Manila",
	},
	"RPSD": {
		ICAO:      "RPSD",
		IATA:      "RZP",
		Name:      "Cesar Lim Rodriguez Airport",
		City:      "Taytay Airport",
		State:     "Mimaropa",
		Country:   "PH",
		Elevation: 80,
		Latitude:  10.81874,
		Longitude: 119.507697,
		Timezone:  "Asia/Manila",
	},
	"RPSM": {
		ICAO:      "RPSM",
		Name:      "Maasin Airport",
		State:     "Eastern-Visayas",
		Country:   "PH",
		Elevation: 328,
		Latitude:  10.1872997284,
		Longitude: 124.7819976807,
		Timezone:  "Asia/Manila",
	},
	"RPUB": {
		ICAO:      "RPUB",
		IATA:      "BAG",
		Name:      "Loakan Airport",
		City:      "Baguio City",
		State:     "Cordillera",
		Country:   "PH",
		Elevation: 4251,
		Latitude:  16.3750991821,
		Longitude: 120.6200027466,
		Timezone:  "Asia/Manila",
	},
	"RPUD": {
		ICAO:      "RPUD",
		IATA:      "DTE",
		Name:      "Daet Airport",
		City:      "Daet",
		State:     "Bicol",
		Country:   "PH",
		Elevation: 10,
		Latitude:  14.1291999817,
		Longitude: 122.9800033569,
		Timezone:  "Asia/Manila",
	},
	"RPUE": {
		ICAO:      "RPUE",
		Name:      "Lucena Airport",
		City:      "Lucena",
		State:     "Calabarzon",
		Country:   "PH",
		Elevation: 39,
		Latitude:  13.9289999008,
		Longitude: 121.6019973755,
		Timezone:  "Asia/Manila",
	},
	"RPUF": {
		ICAO:      "RPUF",
		Name:      "Basa Air Base",
		State:     "Central-Luzon",
		Country:   "PH",
		Elevation: 151,
		Latitude:  14.9864997864,
		Longitude: 120.4929962158,
		Timezone:  "Asia/Manila",
	},
	"RPUG": {
		ICAO:      "RPUG",
		Name:      "Lingayen Airport",
		State:     "Ilocos",
		Country:   "PH",
		Elevation: 7,
		Latitude:  16.0347995758,
		Longitude: 120.2409973145,
		Timezone:  "Asia/Manila",
	},
	"RPUH": {
		ICAO:      "RPUH",
		IATA:      "SJI",
		Name:      "San Jose Airport",
		City:      "San Jose",
		State:     "Mimaropa",
		Country:   "PH",
		Elevation: 14,
		Latitude:  12.3614997864,
		Longitude: 121.04699707,
		Timezone:  "Asia/Manila",
	},
	"RPUI": {
		ICAO:      "RPUI",
		Name:      "Iba Airport",
		City:      "Iba",
		State:     "Central-Luzon",
		Country:   "PH",
		Elevation: 11,
		Latitude:  15.3247556686,
		Longitude: 119.969497681,
		Timezone:  "Asia/Manila",
	},
	"RPUJ": {
		ICAO:      "RPUJ",
		Name:      "Castillejos Airport",
		City:      "Castillejos",
		State:     "Central-Luzon",
		Country:   "PH",
		Elevation: 200,
		Latitude:  14.9670000076,
		Longitude: 120.1500015259,
		Timezone:  "Asia/Manila",
	},
	"RPUK": {
		ICAO:      "RPUK",
		Name:      "Calapan Airport",
		City:      "Calapan City",
		State:     "Mimaropa",
		Country:   "PH",
		Elevation: 11,
		Latitude:  13.4169998169,
		Longitude: 121.1999969482,
		Timezone:  "Asia/Manila",
	},
	"RPUL": {
		ICAO:      "RPUL",
		Name:      "Fernando Air Base",
		State:     "Calabarzon",
		Country:   "PH",
		Elevation: 1220,
		Latitude:  13.9549999237,
		Longitude: 121.125,
		Timezone:  "Asia/Manila",
	},
	"RPUM": {
		ICAO:      "RPUM",
		IATA:      "MBO",
		Name:      "Mamburao Airport",
		State:     "Mimaropa",
		Country:   "PH",
		Elevation: 13,
		Latitude:  13.2081003189,
		Longitude: 120.6050033569,
		Timezone:  "Asia/Manila",
	},
	"RPUN": {
		ICAO:      "RPUN",
		IATA:      "WNP",
		Name:      "Naga Airport",
		City:      "Naga",
		State:     "Bicol",
		Country:   "PH",
		Elevation: 142,
		Latitude:  13.5848999023,
		Longitude: 123.2699966431,
		Timezone:  "Asia/Manila",
	},
	"RPUO": {
		ICAO:      "RPUO",
		IATA:      "BSO",
		Name:      "Basco Airport",
		City:      "Basco",
		State:     "Cagayan-Valley",
		Country:   "PH",
		Elevation: 291,
		Latitude:  20.4512996674,
		Longitude: 121.980003357,
		Timezone:  "Asia/Manila",
	},
	"RPUQ": {
		ICAO:      "RPUQ",
		Name:      "Vigan Airport",
		City:      "Vigan City",
		State:     "Ilocos",
		Country:   "PH",
		Elevation: 16,
		Latitude:  17.5552997589,
		Longitude: 120.3560028076,
		Timezone:  "Asia/Manila",
	},
	"RPUR": {
		ICAO:      "RPUR",
		IATA:      "BQA",
		Name:      "Dr.Juan C. Angara Airport",
		City:      "Baler",
		State:     "Central-Luzon",
		Country:   "PH",
		Elevation: 108,
		Latitude:  15.7298002243,
		Longitude: 121.5,
		Timezone:  "Asia/Manila",
	},
	"RPUS": {
		ICAO:      "RPUS",
		IATA:      "SFE",
		Name:      "San Fernando Airport",
		State:     "Ilocos",
		Country:   "PH",
		Elevation: 13,
		Latitude:  16.5956001282,
		Longitude: 120.3030014038,
		Timezone:  "Asia/Manila",
	},
	"RPUT": {
		ICAO:      "RPUT",
		IATA:      "TUG",
		Name:      "Tuguegarao Airport",
		City:      "Tuguegarao City",
		State:     "Cagayan-Valley",
		Country:   "PH",
		Elevation: 70,
		Latitude:  17.6433676823,
		Longitude: 121.733150482,
		Timezone:  "Asia/Manila",
	},
	"RPUU": {
		ICAO:      "RPUU",
		Name:      "Bulan Airport",
		City:      "Bulan",
		State:     "Bicol",
		Country:   "PH",
		Elevation: 3937,
		Latitude:  12.6842002869,
		Longitude: 123.8779983521,
		Timezone:  "Asia/Manila",
	},
	"RPUV": {
		ICAO:      "RPUV",
		IATA:      "VRC",
		Name:      "Virac Airport",
		City:      "Virac",
		State:     "Bicol",
		Country:   "PH",
		Elevation: 121,
		Latitude:  13.5763998032,
		Longitude: 124.2060012817,
		Timezone:  "Asia/Manila",
	},
	"RPUW": {
		ICAO:      "RPUW",
		IATA:      "MRQ",
		Name:      "Marinduque Airport",
		City:      "Gasan",
		State:     "Mimaropa",
		Country:   "PH",
		Elevation: 32,
		Latitude:  13.361000061,
		Longitude: 121.8259963989,
		Timezone:  "Asia/Manila",
	},
	"RPUX": {
		ICAO:      "RPUX",
		Name:      "Plaridel Airport",
		City:      "Plaridel",
		State:     "Central-Luzon",
		Country:   "PH",
		Elevation: 20,
		Latitude:  14.8930997849,
		Longitude: 120.8519973755,
		Timezone:  "Asia/Manila",
	},
	"RPUY": {
		ICAO:      "RPUY",
		IATA:      "CYZ",
		Name:      "Cauayan Airport",
		City:      "Cauayan City",
		State:     "Cagayan-Valley",
		Country:   "PH",
		Elevation: 200,
		Latitude:  16.9298992157,
		Longitude: 121.752998352,
		Timezone:  "Asia/Manila",
	},
	"RPUZ": {
		ICAO:      "RPUZ",
		Name:      "Bagabag Airport",
		State:     "Cagayan-Valley",
		Country:   "PH",
		Elevation: 820,
		Latitude:  16.6191997528,
		Longitude: 121.2519989014,
		Timezone:  "Asia/Manila",
	},
	"RPVA": {
		ICAO:      "RPVA",
		IATA:      "TAC",
		Name:      "Daniel Z. Romualdez Airport",
		City:      "Tacloban City",
		State:     "Eastern-Visayas",
		Country:   "PH",
		Elevation: 10,
		Latitude:  11.2276000977,
		Longitude: 125.027999878,
		Timezone:  "Asia/Manila",
	},
	"RPVB": {
		ICAO:      "RPVB",
		IATA:      "BCD",
		Name:      "Bacolod-Silay City International Airport",
		City:      "Bacolod City",
		State:     "Negros-Island-Region",
		Country:   "PH",
		Latitude:  10.7763996124,
		Longitude: 123.0149993896,
		Timezone:  "Asia/Manila",
	},
	"RPVC": {
		ICAO:      "RPVC",
		IATA:      "CYP",
		Name:      "Calbayog Airport",
		City:      "Calbayog City",
		State:     "Eastern-Visayas",
		Country:   "PH",
		Elevation: 12,
		Latitude:  12.0726995468,
		Longitude: 124.5449981689,
		Timezone:  "Asia/Manila",
	},
	"RPVD": {
		ICAO:      "RPVD",
		IATA:      "DGT",
		Name:      "Sibulan Airport",
		City:      "Dumaguete City",
		State:     "Central-Visayas",
		Country:   "PH",
		Elevation: 15,
		Latitude:  9.3337097168,
		Longitude: 123.300003052,
		Timezone:  "Asia/Manila",
	},
	"RPVE": {
		ICAO:      "RPVE",
		IATA:      "MPH",
		Name:      "Godofredo P. Ramos Airport",
		City:      "Malay",
		State:     "Western-Visayas",
		Country:   "PH",
		Elevation: 7,
		Latitude:  11.9245004654,
		Longitude: 121.95400238,
		Timezone:  "Asia/Manila",
	},
	"RPVF": {
		ICAO:      "RPVF",
		IATA:      "CRM",
		Name:      "Catarman National Airport",
		City:      "Catarman",
		State:     "Eastern-Visayas",
		Country:   "PH",
		Elevation: 6,
		Latitude:  12.5024003983,
		Longitude: 124.6360015869,
		Timezone:  "Asia/Manila",
	},
	"RPVG": {
		ICAO:      "RPVG",
		Name:      "Guiuan Airport",
		City:      "Guiuan",
		State:     "Eastern-Visayas",
		Country:   "PH",
		Elevation: 7,
		Latitude:  11.0354995728,
		Longitude: 125.741996765,
		Timezone:  "Asia/Manila",
	},
	"RPVH": {
		ICAO:      "RPVH",
		Name:      "Hilongos Airport",
		City:      "Hilongos",
		State:     "Eastern-Visayas",
		Country:   "PH",
		Elevation: 12,
		Latitude:  10.3800001144,
		Longitude: 124.7429962158,
		Timezone:  "Asia/Manila",
	},
	"RPVI": {
		ICAO:      "RPVI",
		IATA:      "ILO",
		Name:      "Iloilo International Airport",
		City:      "Iloilo City",
		State:     "Western-Visayas",
		Country:   "PH",
		Elevation: 27,
		Latitude:  10.833017,
		Longitude: 122.493358,
		Timezone:  "Asia/Manila",
	},
	"RPVJ": {
		ICAO:      "RPVJ",
		IATA:      "MBT",
		Name:      "Moises R. Espinosa Airport",
		City:      "Masbate",
		State:     "Bicol",
		Country:   "PH",
		Elevation: 26,
		Latitude:  12.3694000244,
		Longitude: 123.628997803,
		Timezone:  "Asia/Manila",
	},
	"RPVK": {
		ICAO:      "RPVK",
		IATA:      "KLO",
		Name:      "Kalibo International Airport",
		City:      "Kalibo",
		State:     "Western-Visayas",
		Country:   "PH",
		Elevation: 14,
		Latitude:  11.679400444,
		Longitude: 122.375999451,
		Timezone:  "Asia/Manila",
	},
	"RPVL": {
		ICAO:      "RPVL",
		Name:      "Wasig Airport",
		City:      "Mansalay",
		State:     "Mimaropa",
		Country:   "PH",
		Elevation: 13,
		Latitude:  12.5361003876,
		Longitude: 121.4810028076,
		Timezone:  "Asia/Manila",
	},
	"RPVM": {
		ICAO:      "RPVM",
		IATA:      "CEB",
		Name:      "Mactan Cebu International Airport",
		City:      "Lapu-Lapu City",
		State:     "Central-Visayas",
		Country:   "PH",
		Elevation: 31,
		Latitude:  10.3074998856,
		Longitude: 123.9789962769,
		Timezone:  "Asia/Manila",
	},
	"RPVO": {
		ICAO:      "RPVO",
		IATA:      "OMC",
		Name:      "Ormoc Airport",
		City:      "Ormoc City",
		State:     "Eastern-Visayas",
		Country:   "PH",
		Elevation: 83,
		Latitude:  11.0579996109,
		Longitude: 124.5650024414,
		Timezone:  "Asia/Manila",
	},
	"RPVP": {
		ICAO:      "RPVP",
		IATA:      "PPS",
		Name:      "Puerto Princesa Airport",
		City:      "Puerto Princesa City",
		State:     "Mimaropa",
		Country:   "PH",
		Elevation: 71,
		Latitude:  9.7421197891,
		Longitude: 118.7590026855,
		Timezone:  "Asia/Manila",
	},
	"RPVR": {
		ICAO:      "RPVR",
		IATA:      "RXS",
		Name:      "Roxas Airport",
		City:      "Roxas City",
		State:     "Western-Visayas",
		Country:   "PH",
		Elevation: 10,
		Latitude:  11.597700119,
		Longitude: 122.7519989014,
		Timezone:  "Asia/Manila",
	},
	"RPVS": {
		ICAO:      "RPVS",
		IATA:      "EUQ",
		Name:      "Evelio Javier Airport",
		City:      "San Jose",
		State:     "Western-Visayas",
		Country:   "PH",
		Elevation: 23,
		Latitude:  10.765999794,
		Longitude: 121.932998657,
		Timezone:  "Asia/Manila",
	},
	"RPVT": {
		ICAO:      "RPVT",
		IATA:      "TAG",
		Name:      "Tagbilaran Airport",
		City:      "Tagbilaran City",
		State:     "Central-Visayas",
		Country:   "PH",
		Elevation: 38,
		Latitude:  9.6640796661,
		Longitude: 123.8529968262,
		Timezone:  "Asia/Manila",
	},
	"RPVU": {
		ICAO:      "RPVU",
		IATA:      "TBH",
		Name:      "Romblon Airport",
		City:      "Romblon",
		State:     "Mimaropa",
		Country:   "PH",
		Elevation: 10,
		Latitude:  12.3109998703,
		Longitude: 122.084999084,
		Timezone:  "Asia/Manila",
	},
	"RPVV": {
		ICAO:      "RPVV",
		IATA:      "USU",
		Name:      "Francisco B. Reyes Airport",
		City:      "Coron",
		State:     "Mimaropa",
		Country:   "PH",
		Elevation: 148,
		Latitude:  12.1215000153,
		Longitude: 120.099998474,
		Timezone:  "Asia/Manila",
	},
	"RPVW": {
		ICAO:      "RPVW",
		IATA:      "BPR",
		Name:      "Borongan Airport",
		City:      "Borongan City",
		State:     "Eastern-Visayas",
		Country:   "PH",
		Elevation: 7,
		Latitude:  11.6743001938,
		Longitude: 125.478996277,
		Timezone:  "Asia/Manila",
	},
	"RPVY": {
		ICAO:      "RPVY",
		Name:      "Catbalogan Airport",
		State:     "Eastern-Visayas",
		Country:   "PH",
		Latitude:  11.81,
		Longitude: 124.83,
		Timezone:  "Asia/Manila",
	},
	"RPXX": {
		ICAO:      "RPXX",
		IATA:      "ENI",
		Name:      "El Nido Airport",
		City:      "El Nido",
		State:     "Palawan",
		Country:   "PH",
		Elevation: 17,
		Latitude:  11.2025,
		Longitude: 119.416,
		Timezone:  "Asia/Manila",
	},
	"SA01": {
		ICAO:      "SA01",
		Name:      "Cachi Airport",
		City:      "Cachi",
		State:     "Salta",
		Country:   "AR",
		Elevation: 8232,
		Latitude:  -25.1049995422,
		Longitude: -66.1572036743,
		Timezone:  "America/Argentina/Salta",
	},
	"SA02": {
		ICAO:      "SA02",
		Name:      "Cafayate Airport",
		City:      "Cafayate",
		State:     "Salta",
		Country:   "AR",
		Elevation: 5375,
		Latitude:  -26.0561008453,
		Longitude: -65.9368972778,
		Timezone:  "America/Argentina/Salta",
	},
	"SA03": {
		ICAO:      "SA03",
		Name:      "Villa Minetti Airport",
		City:      "Villa Minetti",
		State:     "Santa-Fe",
		Country:   "AR",
		Elevation: 246,
		Latitude:  -28.6201,
		Longitude: -61.6038,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SA05": {
		ICAO:      "SA05",
		Name:      "Bell Ville Airport",
		City:      "Bell Ville",
		State:     "Cordoba",
		Country:   "AR",
		Elevation: 429,
		Latitude:  -32.6599,
		Longitude: -62.702,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SA06": {
		ICAO:      "SA06",
		Name:      "Santa Rita Airport",
		City:      "Saladillo",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 177,
		Latitude:  -35.8283,
		Longitude: -60.1467,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SA07": {
		ICAO:      "SA07",
		Name:      "El Porton Airport",
		City:      "Buta Ranquil",
		State:     "Neuquen",
		Country:   "AR",
		Elevation: 2853,
		Latitude:  -37.1949996948,
		Longitude: -69.6094970703,
		Timezone:  "America/Argentina/Salta",
	},
	"SA10": {
		ICAO:      "SA10",
		Name:      "Estancia Don Panos Airport",
		City:      "Presidencia Roca",
		State:     "Chaco",
		Country:   "AR",
		Elevation: 262,
		Latitude:  -26.2959,
		Longitude: -59.5353,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SA11": {
		ICAO:      "SA11",
		Name:      "Campo Arenal Airport",
		City:      "Campo Arenal",
		State:     "Catamarca",
		Country:   "AR",
		Elevation: 7622,
		Latitude:  -27.0722999573,
		Longitude: -66.5860977173,
		Timezone:  "America/Argentina/Catamarca",
	},
	"SA12": {
		ICAO:      "SA12",
		Name:      "Quemu Quemu Airport",
		City:      "Quemu Quemu",
		State:     "La-Pampa",
		Country:   "AR",
		Elevation: 396,
		Latitude:  -36.0587,
		Longitude: -63.6313,
		Timezone:  "America/Argentina/Salta",
	},
	"SA13": {
		ICAO:      "SA13",
		Name:      "Estancia La Estrella Airport",
		City:      "Tandil",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 456,
		Latitude:  -37.5593986511,
		Longitude: -58.6660003662,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SA15": {
		ICAO:      "SA15",
		Name:      "Lago Fagnano North Airport",
		City:      "Kaiken",
		State:     "Tierra-del-Fuego",
		Country:   "AR",
		Elevation: 600,
		Latitude:  -54.4996986389,
		Longitude: -67.1731033325,
		Timezone:  "America/Argentina/Ushuaia",
	},
	"SA16": {
		ICAO:      "SA16",
		Name:      "La Puntilla Airport",
		City:      "La Puntilla",
		State:     "Mendoza",
		Country:   "AR",
		Elevation: 2952,
		Latitude:  -32.9627990723,
		Longitude: -68.8737030029,
		Timezone:  "America/Argentina/Mendoza",
	},
	"SA17": {
		ICAO:      "SA17",
		Name:      "Rio Cuarto Aeroclub Airport",
		City:      "Rio Cuarto",
		State:     "Cordoba",
		Country:   "AR",
		Elevation: 1423,
		Latitude:  -33.1605987549,
		Longitude: -64.3382034302,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SA19": {
		ICAO:      "SA19",
		Name:      "Puerto Deseado West Airport",
		City:      "Puerto Deseado",
		State:     "Santa-Cruz",
		Country:   "AR",
		Elevation: 271,
		Latitude:  -47.7201004028,
		Longitude: -65.9324035645,
		Timezone:  "America/Argentina/Rio_Gallegos",
	},
	"SA20": {
		ICAO:      "SA20",
		Name:      "Loma La Lata Airport",
		City:      "Neuquen",
		State:     "Neuquen",
		Country:   "AR",
		Elevation: 1328,
		Latitude:  -38.4137992859,
		Longitude: -68.7372970581,
		Timezone:  "America/Argentina/Salta",
	},
	"SA21": {
		ICAO:      "SA21",
		Name:      "Veinticinco De Mayo Airport",
		City:      "Colonia Veinticinco de Mayo",
		State:     "La-Pampa",
		Country:   "AR",
		Elevation: 1138,
		Latitude:  -37.8097991943,
		Longitude: -67.6593017578,
		Timezone:  "America/Argentina/Salta",
	},
	"SA22": {
		ICAO:      "SA22",
		Name:      "Santa Victoria Airport",
		City:      "Salta",
		State:     "Salta",
		Country:   "AR",
		Elevation: 918,
		Latitude:  -22.2856998444,
		Longitude: -62.7136993408,
		Timezone:  "America/Argentina/Salta",
	},
	"SA23": {
		ICAO:      "SA23",
		Name:      "Apostoles Airport",
		City:      "Apostoles",
		State:     "Misiones",
		Country:   "AR",
		Elevation: 581,
		Latitude:  -27.9032001495,
		Longitude: -55.765499115,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SA24": {
		ICAO:      "SA24",
		Name:      "Calilegua Airport",
		City:      "Calilegua",
		State:     "Jujuy",
		Country:   "AR",
		Elevation: 1449,
		Latitude:  -23.7819004059,
		Longitude: -64.74949646,
		Timezone:  "America/Argentina/Jujuy",
	},
	"SA25": {
		ICAO:      "SA25",
		Name:      "Canadon Seco Airport",
		City:      "Canadon Seco",
		State:     "Santa-Cruz",
		Country:   "AR",
		Elevation: 295,
		Latitude:  -46.539,
		Longitude: -67.5639,
		Timezone:  "America/Argentina/Rio_Gallegos",
	},
	"SA26": {
		ICAO:      "SA26",
		Name:      "Bella Vista Airport",
		City:      "Bella Vista",
		State:     "Santa-Fe",
		Country:   "AR",
		Elevation: 180,
		Latitude:  -28.5262,
		Longitude: -59.0385,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SA27": {
		ICAO:      "SA27",
		Name:      "Puerto Rosales Airport",
		City:      "Puerto Rosales",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 16,
		Latitude:  -38.8970985413,
		Longitude: -62.0102996826,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SA28": {
		ICAO:      "SA28",
		Name:      "Nanco Lauquen Airport",
		City:      "Trenque Lauquen",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 301,
		Latitude:  -35.9715,
		Longitude: -62.7724,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SA29": {
		ICAO:      "SA29",
		Name:      "Choele Choel Airport",
		City:      "Choele Choel",
		State:     "Rio-Negro",
		Country:   "AR",
		Elevation: 642,
		Latitude:  -39.2863998413,
		Longitude: -65.6102981567,
		Timezone:  "America/Argentina/Salta",
	},
	"SA30": {
		ICAO:      "SA30",
		IATA:      "CCT",
		Name:      "Colonia Catriel Airport",
		City:      "Colonia Catriel",
		State:     "Rio-Negro",
		Country:   "AR",
		Elevation: 1026,
		Latitude:  -37.9101982117,
		Longitude: -67.8349990845,
		Timezone:  "America/Argentina/Salta",
	},
	"SA31": {
		ICAO:      "SA31",
		Name:      "San Nicolas De Los Arroyos Airport",
		City:      "San Nicolas",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 98,
		Latitude:  -33.3907,
		Longitude: -60.1957,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SA33": {
		ICAO:      "SA33",
		Name:      "Comandante Luis Piedrabuena Airport",
		City:      "Comandante Luis Piedrabuena",
		State:     "Santa-Cruz",
		Country:   "AR",
		Elevation: 78,
		Latitude:  -49.9951,
		Longitude: -68.9531,
		Timezone:  "America/Argentina/Rio_Gallegos",
	},
	"SA34": {
		ICAO:      "SA34",
		Name:      "Trelew Aeroclub Airport",
		City:      "Trelew",
		State:     "Chubut",
		Country:   "AR",
		Elevation: 127,
		Latitude:  -43.2356,
		Longitude: -65.3241,
		Timezone:  "America/Argentina/Catamarca",
	},
	"SA35": {
		ICAO:      "SA35",
		Name:      "Andalgala Airport",
		City:      "Andalgala",
		State:     "Catamarca",
		Country:   "AR",
		Elevation: 2952,
		Latitude:  -27.6317005157,
		Longitude: -66.3507995605,
		Timezone:  "America/Argentina/Catamarca",
	},
	"SA36": {
		ICAO:      "SA36",
		Name:      "Saladillo Airport",
		City:      "Saladillo",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 147,
		Latitude:  -35.6081,
		Longitude: -59.8164,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SA38": {
		ICAO:      "SA38",
		Name:      "Las Lajas Airport",
		City:      "Las Lajas",
		State:     "Neuquen",
		Country:   "AR",
		Elevation: 2788,
		Latitude:  -38.538898468,
		Longitude: -70.337097168,
		Timezone:  "America/Argentina/Salta",
	},
	"SA39": {
		ICAO:      "SA39",
		Name:      "Jujuy Aeroclub Airport",
		City:      "San Salvador de Jujuy",
		State:     "Jujuy",
		Country:   "AR",
		Elevation: 4047,
		Latitude:  -24.2364997864,
		Longitude: -65.2690963745,
		Timezone:  "America/Argentina/Jujuy",
	},
	"SA40": {
		ICAO:      "SA40",
		Name:      "San Juan Aeroclub Airport",
		City:      "San Juan",
		State:     "San-Juan",
		Country:   "AR",
		Elevation: 2079,
		Latitude:  -31.6028003693,
		Longitude: -68.5473022461,
		Timezone:  "America/Argentina/San_Juan",
	},
	"SA41": {
		ICAO:      "SA41",
		Name:      "El Colorado Airport",
		City:      "El Colorado",
		State:     "Formosa",
		Country:   "AR",
		Elevation: 269,
		Latitude:  -26.3166866302,
		Longitude: -59.3397636414,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SA42": {
		ICAO:      "SA42",
		Name:      "Santa Maria Airport",
		City:      "Santa Maria",
		State:     "Catamarca",
		Country:   "AR",
		Elevation: 5904,
		Latitude:  -26.6734008789,
		Longitude: -66.0240020752,
		Timezone:  "America/Argentina/Catamarca",
	},
	"SA44": {
		ICAO:      "SA44",
		Name:      "Londres/Belen Airport",
		City:      "Londres/Belen",
		State:     "Catamarca",
		Country:   "AR",
		Elevation: 3936,
		Latitude:  -27.7078990936,
		Longitude: -67.0951004028,
		Timezone:  "America/Argentina/Catamarca",
	},
	"SA45": {
		ICAO:      "SA45",
		Name:      "Frias Airport",
		City:      "Frias",
		State:     "Santiago-del-Estero",
		Country:   "AR",
		Elevation: 1082,
		Latitude:  -28.6343994141,
		Longitude: -65.1091995239,
		Timezone:  "America/Argentina/Catamarca",
	},
	"SA46": {
		ICAO:      "SA46",
		Name:      "Fiambala Airport",
		City:      "Fiambala",
		State:     "Catamarca",
		Country:   "AR",
		Elevation: 5029,
		Latitude:  -27.64220047,
		Longitude: -67.6247024536,
		Timezone:  "America/Argentina/Catamarca",
	},
	"SAAA": {
		ICAO:      "SAAA",
		Name:      "San Antonio De Areco Airport",
		City:      "San Antonio de Areco",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 98,
		Latitude:  -34.2214,
		Longitude: -59.4422,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SAAC": {
		ICAO:      "SAAC",
		IATA:      "COC",
		Name:      "Comodoro Pierrestegui Airport",
		City:      "Concordia",
		State:     "Entre-Rios",
		Country:   "AR",
		Elevation: 112,
		Latitude:  -31.2969,
		Longitude: -57.9966,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SAAG": {
		ICAO:      "SAAG",
		IATA:      "GHU",
		Name:      "Gualeguaychu Airport",
		City:      "Gualeguaychu",
		State:     "Entre-Rios",
		Country:   "AR",
		Elevation: 75,
		Latitude:  -33.0103,
		Longitude: -58.6131,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SAAI": {
		ICAO:      "SAAI",
		Name:      "Punta Indio Airport",
		City:      "Veronica",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 49,
		Latitude:  -35.3532981873,
		Longitude: -57.2900009155,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SAAJ": {
		ICAO:      "SAAJ",
		IATA:      "JNI",
		Name:      "Junin Airport",
		City:      "Junin",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 262,
		Latitude:  -34.5459,
		Longitude: -60.9306,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SAAK": {
		ICAO:      "SAAK",
		Name:      "Isla Martin Garcia Airport",
		City:      "Isla Martin Garcia",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 6,
		Latitude:  -34.1821,
		Longitude: -58.2469,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SAAM": {
		ICAO:      "SAAM",
		Name:      "Mazaruca Airport",
		City:      "Puerto Ibicuy/Mazuraca",
		State:     "Entre-Rios",
		Country:   "AR",
		Elevation: 20,
		Latitude:  -33.5960006714,
		Longitude: -59.266998291,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SAAN": {
		ICAO:      "SAAN",
		Name:      "Pergamino Airport",
		City:      "Pergamino",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 236,
		Latitude:  -33.919,
		Longitude: -60.648,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SAAP": {
		ICAO:      "SAAP",
		IATA:      "PRA",
		Name:      "General Urquiza Airport",
		City:      "Parana",
		State:     "Entre-Rios",
		Country:   "AR",
		Elevation: 242,
		Latitude:  -31.7948,
		Longitude: -60.4804,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SAAR": {
		ICAO:      "SAAR",
		IATA:      "ROS",
		Name:      "Islas Malvinas Airport",
		City:      "Rosario",
		State:     "Santa-Fe",
		Country:   "AR",
		Elevation: 85,
		Latitude:  -32.9036,
		Longitude: -60.785,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SAAU": {
		ICAO:      "SAAU",
		Name:      "Villaguay Airport",
		City:      "Villaguay",
		State:     "Entre-Rios",
		Country:   "AR",
		Elevation: 131,
		Latitude:  -31.855,
		Longitude: -59.0756,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SAAV": {
		ICAO:      "SAAV",
		IATA:      "SFN",
		Name:      "Sauce Viejo Airport",
		City:      "Santa Fe",
		State:     "Santa-Fe",
		Country:   "AR",
		Elevation: 55,
		Latitude:  -31.7117,
		Longitude: -60.8117,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SABE": {
		ICAO:      "SABE",
		IATA:      "AEP",
		Name:      "Jorge Newbery Airpark",
		City:      "Buenos Aires",
		State:     "Buenos-Aires-F-D-",
		Country:   "AR",
		Elevation: 18,
		Latitude:  -34.5592,
		Longitude: -58.4156,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SABP": {
		ICAO:      "SABP",
		Name:      "Coronel Pringles Airport",
		City:      "Coronel Pringles",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 810,
		Latitude:  -38.0066680908,
		Longitude: -61.331943512,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SACA": {
		ICAO:      "SACA",
		Name:      "Cap D. Omar Dario Gelardi Airport",
		City:      "Santa Isabel - Cordoba",
		State:     "Cordoba",
		Country:   "AR",
		Elevation: 1594,
		Latitude:  -31.4419002533,
		Longitude: -64.2582015991,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SACC": {
		ICAO:      "SACC",
		IATA:      "LCM",
		Name:      "La Cumbre Airport",
		City:      "La Cumbre",
		State:     "Cordoba",
		Country:   "AR",
		Elevation: 3733,
		Latitude:  -31.0058002472,
		Longitude: -64.5318984985,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SACD": {
		ICAO:      "SACD",
		Name:      "Coronel Olmedo Airport",
		City:      "Cordoba",
		State:     "Cordoba",
		Country:   "AR",
		Elevation: 1416,
		Latitude:  -31.4874992371,
		Longitude: -64.1414031982,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SACE": {
		ICAO:      "SACE",
		Name:      "Escuela De Aviacion Militar Airport",
		City:      "Santa Isabel - Cordoba",
		State:     "Cordoba",
		Country:   "AR",
		Elevation: 1646,
		Latitude:  -31.4449996948,
		Longitude: -64.2838973999,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SACI": {
		ICAO:      "SACI",
		Name:      "Pilar Airport",
		City:      "Pilar",
		State:     "Cordoba",
		Country:   "AR",
		Elevation: 1072,
		Latitude:  -31.6819992065,
		Longitude: -63.8638000488,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SACL": {
		ICAO:      "SACL",
		Name:      "Laguna Larga Airport",
		City:      "Laguna Larga",
		State:     "Cordoba",
		Country:   "AR",
		Elevation: 1013,
		Latitude:  -31,
		Longitude: -63,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SACM": {
		ICAO:      "SACM",
		Name:      "Villa General Mitre Airport",
		City:      "Villa General Mitre",
		State:     "Cordoba",
		Country:   "AR",
		Elevation: 1760,
		Latitude:  -30.7005996704,
		Longitude: -64.0430984497,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SACN": {
		ICAO:      "SACN",
		Name:      "Ascochinga Airport",
		City:      "Ascochinga",
		State:     "Cordoba",
		Country:   "AR",
		Elevation: 677,
		Latitude:  -30.9679203033,
		Longitude: -64.2777404785,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SACO": {
		ICAO:      "SACO",
		IATA:      "COR",
		Name:      "Ingeniero Ambrosio Taravella Airport",
		City:      "Cordoba",
		State:     "Cordoba",
		Country:   "AR",
		Elevation: 1604,
		Latitude:  -31.323600769,
		Longitude: -64.2080001831,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SACP": {
		ICAO:      "SACP",
		Name:      "Chepes Airport",
		City:      "Chepes",
		State:     "La-Rioja",
		Country:   "AR",
		Elevation: 714,
		Latitude:  -31.3561420441,
		Longitude: -66.5918884277,
		Timezone:  "America/Argentina/La_Rioja",
	},
	"SACQ": {
		ICAO:      "SACQ",
		Name:      "Monte Quemado Airport",
		City:      "Monte Quemado",
		State:     "Santiago-del-Estero",
		Country:   "AR",
		Elevation: 1242,
		Latitude:  -26.1597995758,
		Longitude: -64.4091033936,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SACS": {
		ICAO:      "SACS",
		Name:      "Villa De Soto Airport",
		City:      "Villa de Soto",
		State:     "Cordoba",
		Country:   "AR",
		Elevation: 2013,
		Latitude:  -30.8652000427,
		Longitude: -64.9789962769,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SACT": {
		ICAO:      "SACT",
		Name:      "Chamical Airport",
		City:      "Gobernador Gordillo",
		State:     "La-Rioja",
		Country:   "AR",
		Elevation: 1502,
		Latitude:  -30.3453006744,
		Longitude: -66.2936019897,
		Timezone:  "America/Argentina/La_Rioja",
	},
	"SACV": {
		ICAO:      "SACV",
		Name:      "Villa de Maria de Rio Seco Airport",
		City:      "Villa de Maria de Rio Seco",
		State:     "Cordoba",
		Country:   "AR",
		Latitude:  -29.9,
		Longitude: -63.683,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SADF": {
		ICAO:      "SADF",
		IATA:      "FDO",
		Name:      "San Fernando Airport",
		City:      "San Fernando",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 10,
		Latitude:  -34.4532,
		Longitude: -58.5896,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SADG": {
		ICAO:      "SADG",
		Name:      "Monte Grande Airport",
		City:      "Monte Grande",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 30,
		Latitude:  -34.8055992126,
		Longitude: -58.4920005798,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SADJ": {
		ICAO:      "SADJ",
		Name:      "Mariano Moreno Airport",
		City:      "General Sarmiento",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 105,
		Latitude:  -34.5606,
		Longitude: -58.7896,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SADL": {
		ICAO:      "SADL",
		IATA:      "LPG",
		Name:      "La Plata Airport",
		City:      "La Plata",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 72,
		Latitude:  -34.9722,
		Longitude: -57.8947,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SADM": {
		ICAO:      "SADM",
		Name:      "Moron Airport",
		City:      "Moron",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 95,
		Latitude:  -34.6763,
		Longitude: -58.6428,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SADO": {
		ICAO:      "SADO",
		Name:      "Campo De Mayo Airport",
		City:      "Campo de Mayo",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 78,
		Latitude:  -34.5344,
		Longitude: -58.6717,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SADP": {
		ICAO:      "SADP",
		IATA:      "EPA",
		Name:      "El Palomar Airport",
		City:      "El Palomar",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 59,
		Latitude:  -34.6099,
		Longitude: -58.6126,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SADQ": {
		ICAO:      "SADQ",
		Name:      "Quilmes Airport",
		City:      "Quilmes",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 6,
		Latitude:  -34.7064,
		Longitude: -58.2444,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SADR": {
		ICAO:      "SADR",
		Name:      "Merlo Airport",
		City:      "Merlo",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 49,
		Latitude:  -34.6848,
		Longitude: -58.6891,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SADS": {
		ICAO:      "SADS",
		Name:      "Aeroclub Argentino Airport",
		City:      "San Justo",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 62,
		Latitude:  -34.7311,
		Longitude: -58.5975,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SADZ": {
		ICAO:      "SADZ",
		Name:      "Matanza Airport",
		City:      "La Matanza",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 9,
		Latitude:  -34.7261,
		Longitude: -58.5011,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SAEA": {
		ICAO:      "SAEA",
		Name:      "General Acha Airport",
		City:      "General Acha",
		State:     "La-Pampa",
		Country:   "AR",
		Elevation: 908,
		Latitude:  -37.4007987976,
		Longitude: -64.6128005981,
		Timezone:  "America/Argentina/Salta",
	},
	"SAEK": {
		ICAO:      "SAEK",
		Name:      "Laprida Airport",
		City:      "Laprida",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 705,
		Latitude:  -37.518901825,
		Longitude: -60.7769012451,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SAEL": {
		ICAO:      "SAEL",
		Name:      "Las Flores Airport",
		City:      "Las Flores",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 111,
		Latitude:  -36.0672,
		Longitude: -59.1042,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SAEM": {
		ICAO:      "SAEM",
		IATA:      "MJR",
		Name:      "Miramar Airport",
		City:      "Miramar",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 42,
		Latitude:  -38.2271,
		Longitude: -57.8697,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SAET": {
		ICAO:      "SAET",
		Name:      "Trenque Lauquen Airport",
		City:      "Trenque Lauquen",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 324,
		Latitude:  -36.0158,
		Longitude: -62.6939,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SAEZ": {
		ICAO:      "SAEZ",
		IATA:      "EZE",
		Name:      "Ministro Pistarini International Airport",
		City:      "Ezeiza",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 67,
		Latitude:  -34.8222,
		Longitude: -58.5358,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SAFR": {
		ICAO:      "SAFR",
		IATA:      "RAF",
		Name:      "Rafaela Airport",
		City:      "Rafaela",
		State:     "Santa-Fe",
		Country:   "AR",
		Elevation: 325,
		Latitude:  -31.2825,
		Longitude: -61.5017,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SAFS": {
		ICAO:      "SAFS",
		IATA:      "NCJ",
		Name:      "Sunchales Aeroclub Airport",
		City:      "Sunchales",
		State:     "Santa-Fe",
		Country:   "AR",
		Elevation: 311,
		Latitude:  -30.9575,
		Longitude: -61.5283,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SAFT": {
		ICAO:      "SAFT",
		Name:      "El Trebol Airport",
		City:      "El Trebol",
		State:     "Santa-Fe",
		Country:   "AR",
		Elevation: 301,
		Latitude:  -32.2153,
		Longitude: -61.7161,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SAFV": {
		ICAO:      "SAFV",
		Name:      "Venado Tuerto Airport",
		City:      "Venado Tuerto",
		State:     "Santa-Fe",
		Country:   "AR",
		Elevation: 367,
		Latitude:  -33.6818,
		Longitude: -61.9564,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SAHC": {
		ICAO:      "SAHC",
		IATA:      "HOS",
		Name:      "Chos Malal Airport",
		City:      "Chos Malal",
		State:     "Neuquen",
		Country:   "AR",
		Elevation: 2788,
		Latitude:  -37.4446983337,
		Longitude: -70.2225036621,
		Timezone:  "America/Argentina/Salta",
	},
	"SAHE": {
		ICAO:      "SAHE",
		IATA:      "CVH",
		Name:      "Caviahue Airport",
		City:      "Lafontaine",
		State:     "Neuquen",
		Country:   "AR",
		Elevation: 5435,
		Latitude:  -37.851398468,
		Longitude: -71.0092010498,
		Timezone:  "America/Argentina/Salta",
	},
	"SAHI": {
		ICAO:      "SAHI",
		Name:      "Cipoletti Aeroclub Airport",
		City:      "Cipoletti",
		State:     "Rio-Negro",
		Country:   "AR",
		Elevation: 885,
		Latitude:  -38.8931007385,
		Longitude: -67.9886016846,
		Timezone:  "America/Argentina/Salta",
	},
	"SAHR": {
		ICAO:      "SAHR",
		IATA:      "GNR",
		Name:      "Dr. Arturo H. Illia Airport",
		City:      "General Roca",
		State:     "Rio-Negro",
		Country:   "AR",
		Elevation: 852,
		Latitude:  -39.0007019043,
		Longitude: -67.6204986572,
		Timezone:  "America/Argentina/Salta",
	},
	"SAHS": {
		ICAO:      "SAHS",
		IATA:      "RDS",
		Name:      "Rincon De Los Sauces Airport",
		City:      "Rincon de los Sauces",
		State:     "Neuquen",
		Country:   "AR",
		Elevation: 1968,
		Latitude:  -37.3905982971,
		Longitude: -68.9041976929,
		Timezone:  "America/Argentina/Mendoza",
	},
	"SAHV": {
		ICAO:      "SAHV",
		Name:      "Victorica Airport",
		City:      "Victorica",
		State:     "La-Pampa",
		Country:   "AR",
		Elevation: 1020,
		Latitude:  -36.2374992371,
		Longitude: -65.450553894,
		Timezone:  "America/Argentina/Salta",
	},
	"SAHZ": {
		ICAO:      "SAHZ",
		IATA:      "APZ",
		Name:      "Zapala Airport",
		City:      "Zapala",
		State:     "Neuquen",
		Country:   "AR",
		Elevation: 3330,
		Latitude:  -38.9754981995,
		Longitude: -70.1136016846,
		Timezone:  "America/Argentina/Salta",
	},
	"SAMA": {
		ICAO:      "SAMA",
		Name:      "General Alvear Airport",
		City:      "General Alvear",
		State:     "Mendoza",
		Country:   "AR",
		Elevation: 1489,
		Latitude:  -35.0588989258,
		Longitude: -67.6928024292,
		Timezone:  "America/Argentina/Mendoza",
	},
	"SAMC": {
		ICAO:      "SAMC",
		Name:      "Cristo Redentor Airport",
		City:      "Cristo Redentor",
		Country:   "AR",
		Elevation: 10477,
		Latitude:  -32.8073005676,
		Longitude: -70.0681991577,
		Timezone:  "America/Argentina/Mendoza",
	},
	"SAME": {
		ICAO:      "SAME",
		IATA:      "MDZ",
		Name:      "El Plumerillo Airport",
		City:      "Mendoza",
		State:     "Mendoza",
		Country:   "AR",
		Elevation: 2310,
		Latitude:  -32.8316993713,
		Longitude: -68.7929000854,
		Timezone:  "America/Argentina/Mendoza",
	},
	"SAMH": {
		ICAO:      "SAMH",
		Name:      "Valle Hermoso Airport",
		City:      "Valle Hermoso",
		State:     "Mendoza",
		Country:   "AR",
		Elevation: 2500,
		Latitude:  -35.1461982727,
		Longitude: -70.2053985596,
		Timezone:  "America/Argentina/Mendoza",
	},
	"SAMI": {
		ICAO:      "SAMI",
		Name:      "San Martin Airport",
		City:      "San Martin",
		State:     "Mendoza",
		Country:   "AR",
		Elevation: 2154,
		Latitude:  -33.0625,
		Longitude: -68.5083007812,
		Timezone:  "America/Argentina/Mendoza",
	},
	"SAMJ": {
		ICAO:      "SAMJ",
		Name:      "Jachal Airport",
		City:      "Jachal",
		State:     "San-Juan",
		Country:   "AR",
		Elevation: 3790,
		Latitude:  -30.2495002747,
		Longitude: -68.740196228,
		Timezone:  "America/Argentina/San_Juan",
	},
	"SAML": {
		ICAO:      "SAML",
		Name:      "Punta De Vacas Airport",
		City:      "Punta de Vacas",
		State:     "Mendoza",
		Country:   "AR",
		Elevation: 7906,
		Latitude:  -32.854598999,
		Longitude: -69.7604980469,
		Timezone:  "America/Argentina/Mendoza",
	},
	"SAMM": {
		ICAO:      "SAMM",
		IATA:      "LGS",
		Name:      "Comodoro D.R. Salomon Airport",
		City:      "Malargue",
		State:     "Mendoza",
		Country:   "AR",
		Elevation: 4685,
		Latitude:  -35.493598938,
		Longitude: -69.5743026733,
		Timezone:  "America/Argentina/Mendoza",
	},
	"SAMP": {
		ICAO:      "SAMP",
		Name:      "La Paz Airport",
		City:      "La Paz",
		State:     "Mendoza",
		Country:   "AR",
		Latitude:  -33.467,
		Longitude: -67.55,
		Timezone:  "America/Argentina/Mendoza",
	},
	"SAMQ": {
		ICAO:      "SAMQ",
		Name:      "Mendoza Airpark",
		City:      "Mendoza",
		State:     "Mendoza",
		Country:   "AR",
		Elevation: 2694,
		Latitude:  -32.8652992249,
		Longitude: -68.8719024658,
		Timezone:  "America/Argentina/Mendoza",
	},
	"SAMR": {
		ICAO:      "SAMR",
		IATA:      "AFA",
		Name:      "Suboficial Ay Santiago Germano Airport",
		City:      "San Rafael",
		State:     "Mendoza",
		Country:   "AR",
		Elevation: 2470,
		Latitude:  -34.5882987976,
		Longitude: -68.4039001465,
		Timezone:  "America/Argentina/Mendoza",
	},
	"SAMS": {
		ICAO:      "SAMS",
		Name:      "San Carlos Airport",
		City:      "San Carlos",
		State:     "Mendoza",
		Country:   "AR",
		Elevation: 3050,
		Latitude:  -33.7756996155,
		Longitude: -69.0537033081,
		Timezone:  "America/Argentina/Mendoza",
	},
	"SAMU": {
		ICAO:      "SAMU",
		Name:      "Uspallata Airport",
		City:      "Uspallata",
		State:     "Mendoza",
		Country:   "AR",
		Elevation: 6525,
		Latitude:  -32.538898468,
		Longitude: -69.3458023071,
		Timezone:  "America/Argentina/Mendoza",
	},
	"SANC": {
		ICAO:      "SANC",
		IATA:      "CTC",
		Name:      "Catamarca Airport",
		City:      "Catamarca",
		State:     "Catamarca",
		Country:   "AR",
		Elevation: 1522,
		Latitude:  -28.5956001282,
		Longitude: -65.751701355,
		Timezone:  "America/Argentina/Catamarca",
	},
	"SANE": {
		ICAO:      "SANE",
		IATA:      "SDE",
		Name:      "Vicecomodoro Angel D. La Paz Aragones Airport",
		City:      "Santiago del Estero",
		State:     "Santiago-del-Estero",
		Country:   "AR",
		Elevation: 656,
		Latitude:  -27.7655563354,
		Longitude: -64.3099975586,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SANH": {
		ICAO:      "SANH",
		IATA:      "RHD",
		Name:      "Las Termas Airport",
		City:      "Rio Hondo",
		State:     "Santiago-del-Estero",
		Country:   "AR",
		Elevation: 270,
		Latitude:  -27.4736995697,
		Longitude: -64.9055023193,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SANI": {
		ICAO:      "SANI",
		Name:      "Tinogasta Airport",
		City:      "Tinogasta",
		State:     "Catamarca",
		Country:   "AR",
		Elevation: 3968,
		Latitude:  -28.0377998352,
		Longitude: -67.5802993774,
		Timezone:  "America/Argentina/Catamarca",
	},
	"SANL": {
		ICAO:      "SANL",
		IATA:      "IRJ",
		Name:      "Capitan V A Almonacid Airport",
		City:      "La Rioja",
		State:     "La-Rioja",
		Country:   "AR",
		Elevation: 1437,
		Latitude:  -29.3815994263,
		Longitude: -66.7957992554,
		Timezone:  "America/Argentina/La_Rioja",
	},
	"SANO": {
		ICAO:      "SANO",
		Name:      "Chilecito Airport",
		City:      "Chilecito",
		State:     "La-Rioja",
		Country:   "AR",
		Elevation: 3099,
		Latitude:  -29.2238998413,
		Longitude: -67.4389038086,
		Timezone:  "America/Argentina/La_Rioja",
	},
	"SANT": {
		ICAO:      "SANT",
		IATA:      "TUC",
		Name:      "Teniente Benjamin Matienzo Airport",
		City:      "San Miguel de Tucuman",
		State:     "Tucuman",
		Country:   "AR",
		Elevation: 1493,
		Latitude:  -26.8409004211,
		Longitude: -65.1048965454,
		Timezone:  "America/Argentina/Tucuman",
	},
	"SANU": {
		ICAO:      "SANU",
		IATA:      "UAQ",
		Name:      "Domingo Faustino Sarmiento Airport",
		City:      "San Juan",
		State:     "San-Juan",
		Country:   "AR",
		Elevation: 1958,
		Latitude:  -31.5715007782,
		Longitude: -68.4181976318,
		Timezone:  "America/Argentina/San_Juan",
	},
	"SANW": {
		ICAO:      "SANW",
		IATA:      "CRR",
		Name:      "Ceres Airport",
		City:      "Ceres",
		State:     "Santa-Fe",
		Country:   "AR",
		Elevation: 285,
		Latitude:  -29.8631,
		Longitude: -61.8728,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SAOC": {
		ICAO:      "SAOC",
		IATA:      "RCU",
		Name:      "Area De Material Airport",
		City:      "Rio Cuarto",
		State:     "Cordoba",
		Country:   "AR",
		Elevation: 1380,
		Latitude:  -33.0850982666,
		Longitude: -64.2612991333,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SAOD": {
		ICAO:      "SAOD",
		IATA:      "VDR",
		Name:      "Villa Dolores Airport",
		City:      "Villa Dolores",
		State:     "Cordoba",
		Country:   "AR",
		Elevation: 1847,
		Latitude:  -31.9451999664,
		Longitude: -65.1463012695,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SAOE": {
		ICAO:      "SAOE",
		Name:      "Rio Tercero Airport",
		City:      "Rio Tercero",
		State:     "Cordoba",
		Country:   "AR",
		Elevation: 1246,
		Latitude:  -32.1733016968,
		Longitude: -64.0883026123,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SAOI": {
		ICAO:      "SAOI",
		Name:      "Villa Maria Airport",
		City:      "Villa Maria del Rio Seco",
		State:     "Cordoba",
		Country:   "AR",
		Elevation: 672,
		Latitude:  -32.3894004822,
		Longitude: -63.2585983276,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SAOL": {
		ICAO:      "SAOL",
		Name:      "Laboulaye Airport",
		City:      "Laboulaye",
		State:     "Cordoba",
		Country:   "AR",
		Elevation: 449,
		Latitude:  -34.1353988647,
		Longitude: -63.3623008728,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SAOM": {
		ICAO:      "SAOM",
		Name:      "Marcos Juarez Airport",
		City:      "Marcos Juarez",
		State:     "Cordoba",
		Country:   "AR",
		Elevation: 360,
		Latitude:  -32.6836,
		Longitude: -62.1578,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SAOR": {
		ICAO:      "SAOR",
		IATA:      "VME",
		Name:      "Villa Reynolds Airport",
		City:      "Villa Reynolds",
		State:     "San-Luis",
		Country:   "AR",
		Elevation: 1591,
		Latitude:  -33.7299003601,
		Longitude: -65.3873977661,
		Timezone:  "America/Argentina/San_Luis",
	},
	"SAOS": {
		ICAO:      "SAOS",
		IATA:      "RLO",
		Name:      "Valle Del Conlara International Airport",
		City:      "Merlo",
		State:     "San-Luis",
		Country:   "AR",
		Elevation: 2021,
		Latitude:  -32.3847007751,
		Longitude: -65.1865005493,
		Timezone:  "America/Argentina/San_Luis",
	},
	"SAOU": {
		ICAO:      "SAOU",
		IATA:      "LUQ",
		Name:      "Brigadier Mayor D Cesar Raul Ojeda Airport",
		City:      "San Luis",
		State:     "San-Luis",
		Country:   "AR",
		Elevation: 2328,
		Latitude:  -33.2732009888,
		Longitude: -66.3563995361,
		Timezone:  "America/Argentina/San_Luis",
	},
	"SARC": {
		ICAO:      "SARC",
		IATA:      "CNQ",
		Name:      "Corrientes Airport",
		City:      "Corrientes",
		State:     "Corrientes",
		Country:   "AR",
		Elevation: 202,
		Latitude:  -27.4455,
		Longitude: -58.7619,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SARD": {
		ICAO:      "SARD",
		Name:      "Resistencia (City) Airport",
		City:      "Resistencia",
		State:     "Chaco",
		Country:   "AR",
		Elevation: 175,
		Latitude:  -27.4657001495,
		Longitude: -58.9790000916,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SARE": {
		ICAO:      "SARE",
		IATA:      "RES",
		Name:      "Resistencia International Airport",
		City:      "Resistencia",
		State:     "Chaco",
		Country:   "AR",
		Elevation: 173,
		Latitude:  -27.45,
		Longitude: -59.0561,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SARF": {
		ICAO:      "SARF",
		IATA:      "FMA",
		Name:      "Formosa Airport",
		City:      "Formosa",
		State:     "Formosa",
		Country:   "AR",
		Elevation: 193,
		Latitude:  -26.2127,
		Longitude: -58.2281,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SARG": {
		ICAO:      "SARG",
		Name:      "Caa Cati Airport",
		City:      "General Paz",
		State:     "Corrientes",
		Country:   "AR",
		Elevation: 249,
		Latitude:  -27.7553005219,
		Longitude: -57.6329994202,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SARI": {
		ICAO:      "SARI",
		IATA:      "IGR",
		Name:      "Cataratas Del Iguazu International Airport",
		City:      "Puerto Iguazu",
		State:     "Misiones",
		Country:   "AR",
		Elevation: 916,
		Latitude:  -25.7373008728,
		Longitude: -54.473400116,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SARL": {
		ICAO:      "SARL",
		IATA:      "AOL",
		Name:      "Paso De Los Libres Airport",
		City:      "Paso de los Libres",
		State:     "Corrientes",
		Country:   "AR",
		Elevation: 230,
		Latitude:  -29.6894,
		Longitude: -57.1521,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SARM": {
		ICAO:      "SARM",
		IATA:      "MCS",
		Name:      "Monte Caseros Airport",
		City:      "Monte Caseros",
		State:     "Corrientes",
		Country:   "AR",
		Elevation: 170,
		Latitude:  -30.2719,
		Longitude: -57.6402,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SARP": {
		ICAO:      "SARP",
		IATA:      "PSS",
		Name:      "Libertador Gral D Jose De San Martin Airport",
		City:      "Posadas",
		State:     "Misiones",
		Country:   "AR",
		Elevation: 430,
		Latitude:  -27.3858,
		Longitude: -55.9707,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SARS": {
		ICAO:      "SARS",
		IATA:      "PRQ",
		Name:      "Termal Airport",
		City:      "Presidencia Roque Saenz Pena",
		State:     "Chaco",
		Country:   "AR",
		Elevation: 308,
		Latitude:  -26.753610611,
		Longitude: -60.4922218323,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SARV": {
		ICAO:      "SARV",
		Name:      "Villa Angela Airport",
		City:      "Villa Angela",
		State:     "Chaco",
		Country:   "AR",
		Elevation: 246,
		Latitude:  -27.5915,
		Longitude: -60.6864,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SASA": {
		ICAO:      "SASA",
		IATA:      "SLA",
		Name:      "Martin Miguel De Guemes International Airport",
		City:      "Salta",
		State:     "Salta",
		Country:   "AR",
		Elevation: 4088,
		Latitude:  -24.8560009003,
		Longitude: -65.4861984253,
		Timezone:  "America/Argentina/Salta",
	},
	"SASJ": {
		ICAO:      "SASJ",
		IATA:      "JUJ",
		Name:      "Gobernador Horacio Guzman International Airport",
		City:      "San Salvador de Jujuy",
		State:     "Jujuy",
		Country:   "AR",
		Elevation: 3019,
		Latitude:  -24.3927993774,
		Longitude: -65.0978012085,
		Timezone:  "America/Argentina/Jujuy",
	},
	"SASL": {
		ICAO:      "SASL",
		Name:      "Salar De Cauchari Airport",
		City:      "Salar de Cauchari",
		State:     "Salta",
		Country:   "AR",
		Elevation: 13000,
		Latitude:  -24.0191993713,
		Longitude: -66.797203064,
		Timezone:  "America/Argentina/Jujuy",
	},
	"SASM": {
		ICAO:      "SASM",
		Name:      "Metan Airport",
		City:      "Metan",
		State:     "Salta",
		Country:   "AR",
		Elevation: 2765,
		Latitude:  -25.5144004822,
		Longitude: -64.9655990601,
		Timezone:  "America/Argentina/Salta",
	},
	"SASO": {
		ICAO:      "SASO",
		IATA:      "ORA",
		Name:      "Oran Airport",
		City:      "Oran",
		State:     "Salta",
		Country:   "AR",
		Elevation: 1171,
		Latitude:  -23.1527996063,
		Longitude: -64.3292007446,
		Timezone:  "America/Argentina/Salta",
	},
	"SASQ": {
		ICAO:      "SASQ",
		Name:      "La Quiaca Airport",
		City:      "La Quiaca",
		State:     "Jujuy",
		Country:   "AR",
		Elevation: 11414,
		Latitude:  -22.1506004333,
		Longitude: -65.5774993896,
		Timezone:  "America/Argentina/Jujuy",
	},
	"SASR": {
		ICAO:      "SASR",
		Name:      "Rivadavia Airport",
		City:      "Rivadavia",
		State:     "Salta",
		Country:   "AR",
		Elevation: 679,
		Latitude:  -24.1886005402,
		Longitude: -62.8833999634,
		Timezone:  "America/Argentina/Salta",
	},
	"SAST": {
		ICAO:      "SAST",
		IATA:      "TTG",
		Name:      "General Enrique Mosconi Airport",
		City:      "Tartagal",
		State:     "Salta",
		Country:   "AR",
		Elevation: 1472,
		Latitude:  -22.619600296,
		Longitude: -63.7937011719,
		Timezone:  "America/Argentina/Salta",
	},
	"SATC": {
		ICAO:      "SATC",
		IATA:      "CLX",
		Name:      "Clorinda Airport",
		City:      "Clorinda",
		State:     "Formosa",
		Country:   "AR",
		Elevation: 206,
		Latitude:  -25.3036,
		Longitude: -57.7344,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SATD": {
		ICAO:      "SATD",
		IATA:      "ELO",
		Name:      "El Dorado Airport",
		City:      "El Dorado",
		State:     "Misiones",
		Country:   "AR",
		Elevation: 685,
		Latitude:  -26.3974990845,
		Longitude: -54.5746994019,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SATG": {
		ICAO:      "SATG",
		IATA:      "OYA",
		Name:      "Goya Airport",
		City:      "Goya",
		State:     "Corrientes",
		Country:   "AR",
		Elevation: 128,
		Latitude:  -29.1058,
		Longitude: -59.2189,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SATI": {
		ICAO:      "SATI",
		Name:      "Bernardo De Irigoyen Airport",
		City:      "Bernardo de Irigoyen",
		State:     "Misiones",
		Country:   "AR",
		Elevation: 2673,
		Latitude:  -26.2813892365,
		Longitude: -53.6733322144,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SATK": {
		ICAO:      "SATK",
		IATA:      "LLS",
		Name:      "Alferez Armando Rodriguez Airport",
		City:      "Las Lomitas",
		State:     "Formosa",
		Country:   "AR",
		Elevation: 426,
		Latitude:  -24.7213993073,
		Longitude: -60.5488014221,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SATM": {
		ICAO:      "SATM",
		IATA:      "MDX",
		Name:      "Mercedes Airport",
		City:      "Mercedes",
		State:     "Corrientes",
		Country:   "AR",
		Elevation: 358,
		Latitude:  -29.2213993073,
		Longitude: -58.0877990723,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SATO": {
		ICAO:      "SATO",
		Name:      "Obera Airport",
		City:      "Obera",
		State:     "Misiones",
		Country:   "AR",
		Elevation: 1125,
		Latitude:  -27.5181999207,
		Longitude: -55.1241989136,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SATR": {
		ICAO:      "SATR",
		IATA:      "RCQ",
		Name:      "Reconquista Airport",
		City:      "Reconquista",
		State:     "Santa-Fe",
		Country:   "AR",
		Elevation: 160,
		Latitude:  -29.2103,
		Longitude: -59.68,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SATU": {
		ICAO:      "SATU",
		IATA:      "UZU",
		Name:      "Curuzu Cuatia Airport",
		City:      "Curuzu Cuatia",
		State:     "Corrientes",
		Country:   "AR",
		Elevation: 229,
		Latitude:  -29.7706,
		Longitude: -57.9789,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SAVA": {
		ICAO:      "SAVA",
		Name:      "Piedra Del Aguila Airport",
		City:      "Piedra del Aguila",
		State:     "Rio-Negro",
		Country:   "AR",
		Elevation: 2128,
		Latitude:  -40.192199707,
		Longitude: -70.0100021362,
		Timezone:  "America/Argentina/Salta",
	},
	"SAVB": {
		ICAO:      "SAVB",
		IATA:      "EHL",
		Name:      "El Bolson Airport",
		City:      "El Bolson",
		State:     "Rio-Negro",
		Country:   "AR",
		Elevation: 1141,
		Latitude:  -41.9431991577,
		Longitude: -71.5323028564,
		Timezone:  "America/Argentina/Salta",
	},
	"SAVC": {
		ICAO:      "SAVC",
		IATA:      "CRD",
		Name:      "General E. Mosconi Airport",
		City:      "Comodoro Rivadavia",
		State:     "Chubut",
		Country:   "AR",
		Elevation: 189,
		Latitude:  -45.7853,
		Longitude: -67.4655,
		Timezone:  "America/Argentina/Catamarca",
	},
	"SAVD": {
		ICAO:      "SAVD",
		IATA:      "EMX",
		Name:      "El Maiten Airport",
		City:      "El Maiten",
		State:     "Chubut",
		Country:   "AR",
		Elevation: 2355,
		Latitude:  -42.0292015076,
		Longitude: -71.1725006104,
		Timezone:  "America/Argentina/Catamarca",
	},
	"SAVE": {
		ICAO:      "SAVE",
		IATA:      "EQS",
		Name:      "Brigadier Antonio Parodi Airport",
		City:      "Esquel",
		State:     "Chubut",
		Country:   "AR",
		Elevation: 2621,
		Latitude:  -42.908000946,
		Longitude: -71.139503479,
		Timezone:  "America/Argentina/Catamarca",
	},
	"SAVH": {
		ICAO:      "SAVH",
		IATA:      "LHS",
		Name:      "Las Heras Airport",
		City:      "Las Heras",
		State:     "Santa-Cruz",
		Country:   "AR",
		Elevation: 1082,
		Latitude:  -46.5382995605,
		Longitude: -68.9653015137,
		Timezone:  "America/Argentina/Rio_Gallegos",
	},
	"SAVJ": {
		ICAO:      "SAVJ",
		IATA:      "IGB",
		Name:      "Cabo F.A.A. H. R. Bordon Airport",
		City:      "Ingeniero Jacobacci",
		State:     "Rio-Negro",
		Country:   "AR",
		Elevation: 2925,
		Latitude:  -41.3208999634,
		Longitude: -69.5748977661,
		Timezone:  "America/Argentina/Salta",
	},
	"SAVM": {
		ICAO:      "SAVM",
		IATA:      "OLN",
		Name:      "Lago Musters Airport",
		City:      "Sarmiento",
		State:     "Chubut",
		Country:   "AR",
		Elevation: 883,
		Latitude:  -45.5777777778,
		Longitude: -69.0727777778,
		Timezone:  "America/Argentina/Catamarca",
	},
	"SAVN": {
		ICAO:      "SAVN",
		IATA:      "OES",
		Name:      "Antoine De St Exupery Airport",
		City:      "San Antonio Oeste",
		State:     "Rio-Negro",
		Country:   "AR",
		Elevation: 85,
		Latitude:  -40.7512,
		Longitude: -65.0343,
		Timezone:  "America/Argentina/Salta",
	},
	"SAVP": {
		ICAO:      "SAVP",
		Name:      "Paso De Los Indios Airport",
		City:      "Paso de los Indios",
		State:     "Chubut",
		Country:   "AR",
		Elevation: 575,
		Latitude:  -43.8666992188,
		Longitude: -69.0500030518,
		Timezone:  "America/Argentina/Catamarca",
	},
	"SAVQ": {
		ICAO:      "SAVQ",
		IATA:      "MQD",
		Name:      "Maquinchao Airport",
		City:      "Maquinchao",
		State:     "Rio-Negro",
		Country:   "AR",
		Elevation: 2912,
		Latitude:  -41.2430992126,
		Longitude: -68.7078018188,
		Timezone:  "America/Argentina/Salta",
	},
	"SAVR": {
		ICAO:      "SAVR",
		IATA:      "ARR",
		Name:      "D. Casimiro Szlapelis Airport",
		City:      "Alto Rio Senguerr",
		State:     "Chubut",
		Country:   "AR",
		Elevation: 2286,
		Latitude:  -45.0135993958,
		Longitude: -70.8122024536,
		Timezone:  "America/Argentina/Catamarca",
	},
	"SAVS": {
		ICAO:      "SAVS",
		IATA:      "SGV",
		Name:      "Sierra Grande Airport",
		City:      "Sierra Grande",
		State:     "Rio-Negro",
		Country:   "AR",
		Elevation: 688,
		Latitude:  -41.5917015076,
		Longitude: -65.3394012451,
		Timezone:  "America/Argentina/Salta",
	},
	"SAVT": {
		ICAO:      "SAVT",
		IATA:      "REL",
		Name:      "Almirante Marco Andres Zar Airport",
		City:      "Rawson",
		State:     "Chubut",
		Country:   "AR",
		Elevation: 141,
		Latitude:  -43.2105,
		Longitude: -65.2703,
		Timezone:  "America/Argentina/Catamarca",
	},
	"SAVV": {
		ICAO:      "SAVV",
		IATA:      "VDM",
		Name:      "Gobernador Castello Airport",
		City:      "Viedma / Carmen de Patagones",
		State:     "Rio-Negro",
		Country:   "AR",
		Elevation: 20,
		Latitude:  -40.8692,
		Longitude: -63.0004,
		Timezone:  "America/Argentina/Salta",
	},
	"SAVY": {
		ICAO:      "SAVY",
		IATA:      "PMY",
		Name:      "El Tehuelche Airport",
		City:      "Puerto Madryn",
		State:     "Chubut",
		Country:   "AR",
		Elevation: 427,
		Latitude:  -42.7592,
		Longitude: -65.1027,
		Timezone:  "America/Argentina/Catamarca",
	},
	"SAWA": {
		ICAO:      "SAWA",
		IATA:      "ING",
		Name:      "Lago Argentino Airport",
		City:      "El Calafate",
		State:     "Santa-Cruz",
		Country:   "AR",
		Elevation: 732,
		Latitude:  -50.336101532,
		Longitude: -72.2485961914,
		Timezone:  "America/Argentina/Rio_Gallegos",
	},
	"SAWB": {
		ICAO:      "SAWB",
		Name:      "Marambio Base",
		City:      "Seymour Island",
		Country:   "AQ",
		Elevation: 760,
		Latitude:  -64.2382965088,
		Longitude: -56.6307983398,
		Timezone:  "America/Argentina/Ushuaia",
	},
	"SAWC": {
		ICAO:      "SAWC",
		IATA:      "FTE",
		Name:      "El Calafate Airport",
		City:      "El Calafate",
		State:     "Santa-Cruz",
		Country:   "AR",
		Elevation: 669,
		Latitude:  -50.2803001404,
		Longitude: -72.0531005859,
		Timezone:  "America/Argentina/Rio_Gallegos",
	},
	"SAWD": {
		ICAO:      "SAWD",
		IATA:      "PUD",
		Name:      "Puerto Deseado Airport",
		City:      "Puerto Deseado",
		State:     "Santa-Cruz",
		Country:   "AR",
		Elevation: 268,
		Latitude:  -47.7353,
		Longitude: -65.9041,
		Timezone:  "America/Argentina/Rio_Gallegos",
	},
	"SAWE": {
		ICAO:      "SAWE",
		IATA:      "RGA",
		Name:      "Hermes Quijada International Airport",
		City:      "Rio Grande",
		State:     "Tierra-del-Fuego",
		Country:   "AR",
		Elevation: 65,
		Latitude:  -53.7777,
		Longitude: -67.7494,
		Timezone:  "America/Argentina/Ushuaia",
	},
	"SAWG": {
		ICAO:      "SAWG",
		IATA:      "RGL",
		Name:      "Piloto Civil N. Fernandez Airport",
		City:      "Rio Gallegos",
		State:     "Santa-Cruz",
		Country:   "AR",
		Elevation: 61,
		Latitude:  -51.6089,
		Longitude: -69.3126,
		Timezone:  "America/Argentina/Rio_Gallegos",
	},
	"SAWH": {
		ICAO:      "SAWH",
		IATA:      "USH",
		Name:      "Malvinas Argentinas Airport",
		City:      "Ushuahia",
		State:     "Tierra-del-Fuego",
		Country:   "AR",
		Elevation: 102,
		Latitude:  -54.8433,
		Longitude: -68.2958,
		Timezone:  "America/Argentina/Ushuaia",
	},
	"SAWJ": {
		ICAO:      "SAWJ",
		IATA:      "ULA",
		Name:      "Capitan D Daniel Vazquez Airport",
		City:      "San Julian",
		State:     "Santa-Cruz",
		Country:   "AR",
		Elevation: 203,
		Latitude:  -49.3068,
		Longitude: -67.8026,
		Timezone:  "America/Argentina/Rio_Gallegos",
	},
	"SAWL": {
		ICAO:      "SAWL",
		Name:      "Tolwin Observatory Airport",
		City:      "Tolwin (Lago Fagnano)/observatorio MET(TAIS)",
		State:     "Tierra-del-Fuego",
		Country:   "AR",
		Elevation: 90,
		Latitude:  -54.521900177,
		Longitude: -67.1989974976,
		Timezone:  "America/Argentina/Ushuaia",
	},
	"SAWM": {
		ICAO:      "SAWM",
		IATA:      "ROY",
		Name:      "Rio Mayo Airport",
		City:      "Rio Mayo",
		State:     "Chubut",
		Country:   "AR",
		Elevation: 1784,
		Latitude:  -45.7038993835,
		Longitude: -70.2455978394,
		Timezone:  "America/Argentina/Catamarca",
	},
	"SAWO": {
		ICAO:      "SAWO",
		Name:      "Estacion Aeronaval Airport",
		City:      "Ushuahia",
		Country:   "AR",
		Elevation: 19,
		Latitude:  -54.8227,
		Longitude: -68.3043,
		Timezone:  "America/Argentina/Ushuaia",
	},
	"SAWP": {
		ICAO:      "SAWP",
		IATA:      "PMQ",
		Name:      "Perito Moreno Airport",
		City:      "Perito Moreno",
		State:     "Santa-Cruz",
		Country:   "AR",
		Elevation: 1410,
		Latitude:  -46.5378990173,
		Longitude: -70.9786987305,
		Timezone:  "America/Argentina/Rio_Gallegos",
	},
	"SAWR": {
		ICAO:      "SAWR",
		IATA:      "GGS",
		Name:      "Gobernador Gregores Airport",
		City:      "Gobernador Gregores",
		State:     "Santa-Cruz",
		Country:   "AR",
		Elevation: 356,
		Latitude:  -48.7831001282,
		Longitude: -70.1500015259,
		Timezone:  "America/Argentina/Rio_Gallegos",
	},
	"SAWS": {
		ICAO:      "SAWS",
		IATA:      "JSM",
		Name:      "Jose De San Martin Airport",
		City:      "Chubut",
		State:     "Chubut",
		Country:   "AR",
		Elevation: 2407,
		Latitude:  -44.0485992432,
		Longitude: -70.4589004517,
		Timezone:  "America/Argentina/Catamarca",
	},
	"SAWT": {
		ICAO:      "SAWT",
		IATA:      "RYO",
		Name:      "28 De Noviembre Airport",
		City:      "El Turbio",
		State:     "Santa-Cruz",
		Country:   "AR",
		Elevation: 895,
		Latitude:  -51.6091995239,
		Longitude: -72.2217025757,
		Timezone:  "America/Argentina/Rio_Gallegos",
	},
	"SAWU": {
		ICAO:      "SAWU",
		IATA:      "RZA",
		Name:      "Santa Cruz Airport",
		City:      "Santa Cruz",
		State:     "Santa-Cruz",
		Country:   "AR",
		Elevation: 364,
		Latitude:  -50.0165,
		Longitude: -68.5792,
		Timezone:  "America/Argentina/Rio_Gallegos",
	},
	"SAWZ": {
		ICAO:      "SAWZ",
		Name:      "Matienzo Base",
		Country:   "AQ",
		Elevation: 75,
		Latitude:  -64,
		Longitude: -60,
		Timezone:  "America/Argentina/Ushuaia",
	},
	"SAYB": {
		ICAO:      "SAYB",
		Name:      "Base Belgrano II Airport",
		Country:   "AQ",
		Elevation: 820,
		Latitude:  -77.8740548037,
		Longitude: -34.626159668,
		Timezone:  "Antarctica/Rothera",
	},
	"SAYE": {
		ICAO:      "SAYE",
		Name:      "Esperanza Base",
		City:      "Hope Bay",
		Country:   "AQ",
		Elevation: 75,
		Latitude:  -63.4000015259,
		Longitude: -56.9832992554,
		Timezone:  "America/Argentina/Ushuaia",
	},
	"SAYJ": {
		ICAO:      "SAYJ",
		Name:      "Jubany Airbase",
		City:      "South Shetland Archipielago- King George Island",
		Country:   "AQ",
		Elevation: 12,
		Latitude:  -62.2383003235,
		Longitude: -58.6665992737,
		Timezone:  "America/Argentina/Ushuaia",
	},
	"SAYO": {
		ICAO:      "SAYO",
		Name:      "Orcadas Base",
		City:      "Laurie Island",
		Country:   "AQ",
		Elevation: 75,
		Latitude:  -60.7402992249,
		Longitude: -44.7425003052,
		Timezone:  "America/Argentina/Ushuaia",
	},
	"SAYS": {
		ICAO:      "SAYS",
		Name:      "San Martin Base",
		Country:   "AQ",
		Elevation: 75,
		Latitude:  -68.1166992188,
		Longitude: -67.0999984741,
		Timezone:  "America/Argentina/Ushuaia",
	},
	"SAZA": {
		ICAO:      "SAZA",
		Name:      "Azul Airport",
		City:      "Azul",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 478,
		Latitude:  -36.8368988037,
		Longitude: -59.8802986145,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SAZB": {
		ICAO:      "SAZB",
		IATA:      "BHI",
		Name:      "Comandante Espora Airport",
		City:      "Bahia Blanca",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 246,
		Latitude:  -38.725,
		Longitude: -62.1693,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SAZC": {
		ICAO:      "SAZC",
		IATA:      "CSZ",
		Name:      "Brigadier D.H.E. Ruiz Airport",
		City:      "Coronel Suarez",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 767,
		Latitude:  -37.4460983276,
		Longitude: -61.8893013,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SAZD": {
		ICAO:      "SAZD",
		Name:      "Dolores Airport",
		City:      "Dolores",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 32,
		Latitude:  -36.3225,
		Longitude: -57.7214,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SAZE": {
		ICAO:      "SAZE",
		Name:      "Pigue Airport",
		City:      "Pigue",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 997,
		Latitude:  -37.6016998291,
		Longitude: -62.3781013489,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SAZF": {
		ICAO:      "SAZF",
		IATA:      "OVR",
		Name:      "Olavarria Airport",
		City:      "Olavarria",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 551,
		Latitude:  -36.8899993896,
		Longitude: -60.2165985107,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SAZG": {
		ICAO:      "SAZG",
		IATA:      "GPO",
		Name:      "General Pico Airport",
		City:      "General Pico",
		State:     "La-Pampa",
		Country:   "AR",
		Elevation: 459,
		Latitude:  -35.6962013245,
		Longitude: -63.7583007812,
		Timezone:  "America/Argentina/Salta",
	},
	"SAZH": {
		ICAO:      "SAZH",
		IATA:      "OYO",
		Name:      "Tres Arroyos Airport",
		City:      "Tres Arroyos",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 400,
		Latitude:  -38.3869,
		Longitude: -60.3297,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SAZI": {
		ICAO:      "SAZI",
		Name:      "Bolivar Airport",
		City:      "Bolivar",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 308,
		Latitude:  -36.1866,
		Longitude: -61.0764,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SAZJ": {
		ICAO:      "SAZJ",
		Name:      "Benito Juarez Airport",
		City:      "Benito Juarez",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 682,
		Latitude:  -37.7061004639,
		Longitude: -59.7919006348,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SAZK": {
		ICAO:      "SAZK",
		Name:      "Cerro Catedral Airport",
		City:      "Cerro Catedral",
		State:     "Rio-Negro",
		Country:   "AR",
		Elevation: 3450,
		Latitude:  -41.1632995605,
		Longitude: -71.4427032471,
		Timezone:  "America/Argentina/Salta",
	},
	"SAZL": {
		ICAO:      "SAZL",
		IATA:      "SST",
		Name:      "Santa Teresita Airport",
		City:      "Santa Teresita",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 9,
		Latitude:  -36.5423,
		Longitude: -56.7218,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SAZM": {
		ICAO:      "SAZM",
		IATA:      "MDQ",
		Name:      "Astor Piazzola International Airport",
		City:      "Mar del Plata",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 72,
		Latitude:  -37.9342,
		Longitude: -57.5733,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SAZN": {
		ICAO:      "SAZN",
		IATA:      "NQN",
		Name:      "Presidente Peron Airport",
		City:      "Neuquen",
		State:     "Neuquen",
		Country:   "AR",
		Elevation: 895,
		Latitude:  -38.9490013123,
		Longitude: -68.1557006836,
		Timezone:  "America/Argentina/Salta",
	},
	"SAZO": {
		ICAO:      "SAZO",
		IATA:      "NEC",
		Name:      "Necochea Airport",
		City:      "Necochea",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 72,
		Latitude:  -38.4831,
		Longitude: -58.8172,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SAZP": {
		ICAO:      "SAZP",
		IATA:      "PEH",
		Name:      "Comodoro Pedro Zanni Airport",
		City:      "Pehuajo",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 278,
		Latitude:  -35.8446,
		Longitude: -61.8576,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SAZQ": {
		ICAO:      "SAZQ",
		Name:      "Rio Colorado Airport",
		City:      "Rio Colorado",
		State:     "Rio-Negro",
		Country:   "AR",
		Elevation: 275,
		Latitude:  -38.995,
		Longitude: -64.1408,
		Timezone:  "America/Argentina/Salta",
	},
	"SAZR": {
		ICAO:      "SAZR",
		IATA:      "RSA",
		Name:      "Santa Rosa Airport",
		City:      "Santa Rosa",
		State:     "La-Pampa",
		Country:   "AR",
		Elevation: 630,
		Latitude:  -36.5882987976,
		Longitude: -64.2757034302,
		Timezone:  "America/Argentina/Salta",
	},
	"SAZS": {
		ICAO:      "SAZS",
		IATA:      "BRC",
		Name:      "San Carlos De Bariloche Airport",
		City:      "San Carlos de Bariloche",
		State:     "Rio-Negro",
		Country:   "AR",
		Elevation: 2774,
		Latitude:  -41.1511993408,
		Longitude: -71.1575012207,
		Timezone:  "America/Argentina/Salta",
	},
	"SAZT": {
		ICAO:      "SAZT",
		IATA:      "TDL",
		Name:      "Heroes De Malvinas Airport",
		City:      "Tandil",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 574,
		Latitude:  -37.2374000549,
		Longitude: -59.2279014587,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SAZU": {
		ICAO:      "SAZU",
		Name:      "Puelches Airport",
		City:      "Puelches",
		State:     "La-Pampa",
		Country:   "AR",
		Elevation: 180,
		Latitude:  -38.1451034546,
		Longitude: -65.9229431152,
		Timezone:  "America/Argentina/Salta",
	},
	"SAZV": {
		ICAO:      "SAZV",
		IATA:      "VLG",
		Name:      "Villa Gesell Airport",
		City:      "Villa Gesell",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 32,
		Latitude:  -37.2354,
		Longitude: -57.0292,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SAZW": {
		ICAO:      "SAZW",
		IATA:      "CUT",
		Name:      "Cutral-Co Airport",
		City:      "Cutral-Co",
		State:     "Neuquen",
		Country:   "AR",
		Elevation: 2132,
		Latitude:  -38.9397010803,
		Longitude: -69.2646026611,
		Timezone:  "America/Argentina/Salta",
	},
	"SAZX": {
		ICAO:      "SAZX",
		Name:      "Nueve De Julio Airport",
		City:      "Nueve de Julio",
		State:     "Buenos-Aires",
		Country:   "AR",
		Elevation: 246,
		Latitude:  -35.3997,
		Longitude: -60.9358,
		Timezone:  "America/Argentina/Buenos_Aires",
	},
	"SAZY": {
		ICAO:      "SAZY",
		IATA:      "CPC",
		Name:      "Aviador C. Campos Airport",
		City:      "Chapelco/San Martin de los Andes",
		State:     "Neuquen",
		Country:   "AR",
		Elevation: 2569,
		Latitude:  -40.0754013062,
		Longitude: -71.137298584,
		Timezone:  "America/Argentina/Salta",
	},
	"SBAA": {
		ICAO:      "SBAA",
		IATA:      "CDJ",
		Name:      "Conceicao do Araguaia Airport",
		City:      "Conceicao Do Araguaia",
		State:     "Para",
		Country:   "BR",
		Elevation: 653,
		Latitude:  -8.3483495712,
		Longitude: -49.3014984131,
		Timezone:  "America/Araguaina",
	},
	"SBAE": {
		ICAO:      "SBAE",
		IATA:      "JTC",
		Name:      "Bauru-Arealva Airport",
		City:      "Bauru",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1949,
		Latitude:  -22.15778,
		Longitude: -49.06833,
		Timezone:  "America/Sao_Paulo",
	},
	"SBAF": {
		ICAO:      "SBAF",
		IATA:      "0",
		Name:      "Campo Delio Jardim de Mattos Airport",
		City:      "Rio De Janeiro",
		State:     "Rio-de-Janeiro",
		Country:   "BR",
		Elevation: 110,
		Latitude:  -22.8750991821,
		Longitude: -43.3847007751,
		Timezone:  "America/Sao_Paulo",
	},
	"SBAM": {
		ICAO:      "SBAM",
		IATA:      "0",
		Name:      "Amapa Airport",
		City:      "Amapa",
		State:     "Amapa",
		Country:   "BR",
		Elevation: 45,
		Latitude:  2.0775101185,
		Longitude: -50.8582000732,
		Timezone:  "America/Belem",
	},
	"SBAN": {
		ICAO:      "SBAN",
		Name:      "Base Aerea Airport",
		City:      "Anapolis",
		State:     "Goias",
		Country:   "BR",
		Elevation: 3731,
		Latitude:  -16.2292003632,
		Longitude: -48.9642982483,
		Timezone:  "America/Sao_Paulo",
	},
	"SBAQ": {
		ICAO:      "SBAQ",
		IATA:      "AQA",
		Name:      "Araraquara Airport",
		City:      "Araraquara",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2334,
		Latitude:  -21.8120002747,
		Longitude: -48.1329994202,
		Timezone:  "America/Sao_Paulo",
	},
	"SBAR": {
		ICAO:      "SBAR",
		IATA:      "AJU",
		Name:      "Santa Maria Airport",
		City:      "Aracaju",
		State:     "Sergipe",
		Country:   "BR",
		Elevation: 23,
		Latitude:  -10.984000206,
		Longitude: -37.0703010559,
		Timezone:  "America/Maceio",
	},
	"SBAS": {
		ICAO:      "SBAS",
		IATA:      "AIF",
		Name:      "Marcelo Pires Halzhausen Airport",
		City:      "Assis",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1850,
		Latitude:  -22.64,
		Longitude: -50.453056,
		Timezone:  "America/Sao_Paulo",
	},
	"SBAT": {
		ICAO:      "SBAT",
		IATA:      "AFL",
		Name:      "Alta Floresta Airport",
		City:      "Alta Floresta",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 948,
		Latitude:  -9.8663892746,
		Longitude: -56.1049995422,
		Timezone:  "America/Cuiaba",
	},
	"SBAU": {
		ICAO:      "SBAU",
		IATA:      "ARU",
		Name:      "Aracatuba Airport",
		City:      "Aracatuba",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1361,
		Latitude:  -21.1413002014,
		Longitude: -50.4247016907,
		Timezone:  "America/Sao_Paulo",
	},
	"SBAX": {
		ICAO:      "SBAX",
		IATA:      "AAX",
		Name:      "Araxa Airport",
		City:      "Araxa",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3276,
		Latitude:  -19.5631999969,
		Longitude: -46.9603996277,
		Timezone:  "America/Sao_Paulo",
	},
	"SBBE": {
		ICAO:      "SBBE",
		IATA:      "BEL",
		Name:      "Val de Cans/Julio Cezar Ribeiro International Airport",
		City:      "Belem",
		State:     "Para",
		Country:   "BR",
		Elevation: 54,
		Latitude:  -1.3792500496,
		Longitude: -48.4762992859,
		Timezone:  "America/Belem",
	},
	"SBBG": {
		ICAO:      "SBBG",
		IATA:      "BGX",
		Name:      "Comandante Gustavo Kraemer Airport",
		City:      "Bage",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 600,
		Latitude:  -31.390499115,
		Longitude: -54.1122016907,
		Timezone:  "America/Sao_Paulo",
	},
	"SBBH": {
		ICAO:      "SBBH",
		IATA:      "PLU",
		Name:      "Pampulha - Carlos Drummond de Andrade Airport",
		City:      "Belo Horizonte",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2589,
		Latitude:  -19.8512001038,
		Longitude: -43.9505996704,
		Timezone:  "America/Sao_Paulo",
	},
	"SBBI": {
		ICAO:      "SBBI",
		IATA:      "BFH",
		Name:      "Bacacheri Airport",
		City:      "Curitiba",
		State:     "Parana",
		Country:   "BR",
		Elevation: 3057,
		Latitude:  -25.4050998688,
		Longitude: -49.2319984436,
		Timezone:  "America/Sao_Paulo",
	},
	"SBBP": {
		ICAO:      "SBBP",
		IATA:      "BJP",
		Name:      "Aeroporto Estadual Arthur Siqueira Airport",
		City:      "Braganca Paulista",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2887,
		Latitude:  -22.979167,
		Longitude: -46.5375,
		Timezone:  "America/Sao_Paulo",
	},
	"SBBQ": {
		ICAO:      "SBBQ",
		IATA:      "QAK",
		Name:      "Major Brigadeiro Doorgal Borges Airport",
		City:      "Barbacena",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3657,
		Latitude:  -21.26720047,
		Longitude: -43.761100769,
		Timezone:  "America/Sao_Paulo",
	},
	"SBBR": {
		ICAO:      "SBBR",
		IATA:      "BSB",
		Name:      "Presidente Juscelino Kubistschek International Airport",
		City:      "Brasilia",
		State:     "Federal-District",
		Country:   "BR",
		Elevation: 3497,
		Latitude:  -15.8691673279,
		Longitude: -47.9208335876,
		Timezone:  "America/Sao_Paulo",
	},
	"SBBT": {
		ICAO:      "SBBT",
		IATA:      "BAT",
		Name:      "Chafei Amsei Airport",
		City:      "Barretos",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1898,
		Latitude:  -20.5844993591,
		Longitude: -48.5941009521,
		Timezone:  "America/Sao_Paulo",
	},
	"SBBU": {
		ICAO:      "SBBU",
		IATA:      "BAU",
		Name:      "Bauru Airport",
		City:      "Bauru",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2025,
		Latitude:  -22.3449993134,
		Longitude: -49.0537986755,
		Timezone:  "America/Sao_Paulo",
	},
	"SBBV": {
		ICAO:      "SBBV",
		IATA:      "BVB",
		Name:      "Atlas Brasil Cantanhede Airport",
		City:      "Boa Vista",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 276,
		Latitude:  2.8413889408,
		Longitude: -60.6922225952,
		Timezone:  "America/Boa_Vista",
	},
	"SBBW": {
		ICAO:      "SBBW",
		IATA:      "BPG",
		Name:      "Barra do Garcas Airport",
		City:      "Barra Do Garcas",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1147,
		Latitude:  -15.8613004684,
		Longitude: -52.3889007568,
		Timezone:  "America/Cuiaba",
	},
	"SBBZ": {
		ICAO:      "SBBZ",
		IATA:      "BZC",
		Name:      "Umberto Modiano Airport",
		City:      "Cabo Frio",
		State:     "Rio-de-Janeiro",
		Country:   "BR",
		Elevation: 10,
		Latitude:  -22.7709999084,
		Longitude: -41.9628982544,
		Timezone:  "America/Sao_Paulo",
	},
	"SBCA": {
		ICAO:      "SBCA",
		IATA:      "CAC",
		Name:      "Cascavel Airport",
		City:      "Cascavel",
		State:     "Parana",
		Country:   "BR",
		Elevation: 2473,
		Latitude:  -25.0002994537,
		Longitude: -53.5008010864,
		Timezone:  "America/Sao_Paulo",
	},
	"SBCB": {
		ICAO:      "SBCB",
		IATA:      "CFB",
		Name:      "Cabo Frio Airport",
		City:      "Cabo Frio",
		State:     "Rio-de-Janeiro",
		Country:   "BR",
		Elevation: 23,
		Latitude:  -22.9216995239,
		Longitude: -42.0742988586,
		Timezone:  "America/Sao_Paulo",
	},
	"SBCC": {
		ICAO:      "SBCC",
		Name:      "Cachimbo Airport",
		City:      "Novo Progresso",
		State:     "Para",
		Country:   "BR",
		Elevation: 1762,
		Latitude:  -9.3339395523,
		Longitude: -54.9654006958,
		Timezone:  "America/Santarem",
	},
	"SBCD": {
		ICAO:      "SBCD",
		IATA:      "0",
		Name:      "Cacador Airport",
		City:      "Cacador",
		State:     "Santa-Catarina",
		Country:   "BR",
		Elevation: 3376,
		Latitude:  -26.78840065,
		Longitude: -50.9398002625,
		Timezone:  "America/Sao_Paulo",
	},
	"SBCF": {
		ICAO:      "SBCF",
		IATA:      "CNF",
		Name:      "Tancredo Neves International Airport",
		City:      "Belo Horizonte",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2715,
		Latitude:  -19.6244430542,
		Longitude: -43.9719429016,
		Timezone:  "America/Sao_Paulo",
	},
	"SBCG": {
		ICAO:      "SBCG",
		IATA:      "CGR",
		Name:      "Campo Grande Airport",
		City:      "Campo Grande",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1833,
		Latitude:  -20.4687004089,
		Longitude: -54.6725006104,
		Timezone:  "America/Campo_Grande",
	},
	"SBCH": {
		ICAO:      "SBCH",
		IATA:      "XAP",
		Name:      "Chapeco Airport",
		City:      "Chapeco",
		State:     "Santa-Catarina",
		Country:   "BR",
		Elevation: 2146,
		Latitude:  -27.1341991425,
		Longitude: -52.6566009521,
		Timezone:  "America/Sao_Paulo",
	},
	"SBCI": {
		ICAO:      "SBCI",
		IATA:      "CLN",
		Name:      "Brig. Lysias Augusto Rodrigues Airport",
		City:      "Carolina",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 565,
		Latitude:  -7.3204398155,
		Longitude: -47.4586982727,
		Timezone:  "America/Fortaleza",
	},
	"SBCJ": {
		ICAO:      "SBCJ",
		IATA:      "CKS",
		Name:      "Carajas Airport",
		City:      "Carajas",
		State:     "Para",
		Country:   "BR",
		Elevation: 2064,
		Latitude:  -6.1152777672,
		Longitude: -50.0013885498,
		Timezone:  "America/Belem",
	},
	"SBCM": {
		ICAO:      "SBCM",
		IATA:      "CCM",
		Name:      "Forquilhinha - Criciuma Airport",
		City:      "Criciuma",
		State:     "Santa-Catarina",
		Country:   "BR",
		Elevation: 93,
		Latitude:  -28.7244434357,
		Longitude: -49.4213905334,
		Timezone:  "America/Sao_Paulo",
	},
	"SBCN": {
		ICAO:      "SBCN",
		IATA:      "CLV",
		Name:      "Caldas Novas Airport",
		City:      "Caldas Novas",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2247,
		Latitude:  -17.7252998352,
		Longitude: -48.6074981689,
		Timezone:  "America/Sao_Paulo",
	},
	"SBCO": {
		ICAO:      "SBCO",
		IATA:      "QNS",
		Name:      "Canoas Airport",
		City:      "Porto Alegre",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 26,
		Latitude:  -29.9458999634,
		Longitude: -51.1444015503,
		Timezone:  "America/Sao_Paulo",
	},
	"SBCP": {
		ICAO:      "SBCP",
		IATA:      "CAW",
		Name:      "Bartolomeu Lisandro Airport",
		City:      "Campos Dos Goytacazes",
		State:     "Rio-de-Janeiro",
		Country:   "BR",
		Elevation: 57,
		Latitude:  -21.698299408,
		Longitude: -41.301700592,
		Timezone:  "America/Sao_Paulo",
	},
	"SBCR": {
		ICAO:      "SBCR",
		IATA:      "CMG",
		Name:      "Corumba International Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 461,
		Latitude:  -19.0119438171,
		Longitude: -57.6713905334,
		Timezone:  "America/Campo_Grande",
	},
	"SBCT": {
		ICAO:      "SBCT",
		IATA:      "CWB",
		Name:      "Afonso Pena Airport",
		City:      "Curitiba",
		State:     "Parana",
		Country:   "BR",
		Elevation: 2988,
		Latitude:  -25.5284996033,
		Longitude: -49.1758003235,
		Timezone:  "America/Sao_Paulo",
	},
	"SBCV": {
		ICAO:      "SBCV",
		IATA:      "CRQ",
		Name:      "Caravelas Airport",
		City:      "Caravelas",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 36,
		Latitude:  -17.652299881,
		Longitude: -39.2531013489,
		Timezone:  "America/Bahia",
	},
	"SBCX": {
		ICAO:      "SBCX",
		IATA:      "CXJ",
		Name:      "Campo dos Bugres Airport",
		City:      "Caxias Do Sul",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 2472,
		Latitude:  -29.1970996857,
		Longitude: -51.1875,
		Timezone:  "America/Sao_Paulo",
	},
	"SBCY": {
		ICAO:      "SBCY",
		IATA:      "CGB",
		Name:      "Marechal Rondon Airport",
		City:      "Cuiaba",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 617,
		Latitude:  -15.6528997421,
		Longitude: -56.1166992188,
		Timezone:  "America/Cuiaba",
	},
	"SBCZ": {
		ICAO:      "SBCZ",
		IATA:      "CZS",
		Name:      "Cruzeiro do Sul Airport",
		City:      "Cruzeiro Do Sul",
		State:     "Acre",
		Country:   "BR",
		Elevation: 637,
		Latitude:  -7.5999097824,
		Longitude: -72.7695007324,
		Timezone:  "America/Rio_Branco",
	},
	"SBDB": {
		ICAO:      "SBDB",
		IATA:      "BYO",
		Name:      "Bonito Airport",
		City:      "Bonito",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1180,
		Latitude:  -21.2294445038,
		Longitude: -56.456111908,
		Timezone:  "America/Campo_Grande",
	},
	"SBDN": {
		ICAO:      "SBDN",
		IATA:      "PPB",
		Name:      "Presidente Prudente Airport",
		City:      "Presidente Prudente",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1477,
		Latitude:  -22.1751003265,
		Longitude: -51.4245986938,
		Timezone:  "America/Sao_Paulo",
	},
	"SBEG": {
		ICAO:      "SBEG",
		IATA:      "MAO",
		Name:      "Eduardo Gomes International Airport",
		City:      "Manaus",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 264,
		Latitude:  -3.0386099815,
		Longitude: -60.0497016907,
		Timezone:  "America/Manaus",
	},
	"SBEK": {
		ICAO:      "SBEK",
		IATA:      "JCR",
		Name:      "Jacareacanga Airport",
		City:      "Jacareacanga",
		State:     "Para",
		Country:   "BR",
		Elevation: 323,
		Latitude:  -6.2331600189,
		Longitude: -57.7769012451,
		Timezone:  "America/Santarem",
	},
	"SBEP": {
		ICAO:      "SBEP",
		IATA:      "ESI",
		Name:      "Espinosa Airport",
		City:      "Espinosa",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1900,
		Latitude:  -14.9336944444,
		Longitude: -42.81,
		Timezone:  "America/Sao_Paulo",
	},
	"SBES": {
		ICAO:      "SBES",
		IATA:      "0",
		Name:      "Sao Pedro da Aldeia Airport",
		City:      "Sao Pedro Da Aldeia",
		State:     "Rio-de-Janeiro",
		Country:   "BR",
		Elevation: 61,
		Latitude:  -22.8129005432,
		Longitude: -42.0926017761,
		Timezone:  "America/Sao_Paulo",
	},
	"SBFI": {
		ICAO:      "SBFI",
		IATA:      "IGU",
		Name:      "Cataratas International Airport",
		City:      "Foz Do Iguacu",
		State:     "Parana",
		Country:   "BR",
		Elevation: 786,
		Latitude:  -25.6002788544,
		Longitude: -54.4850006104,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SBFL": {
		ICAO:      "SBFL",
		IATA:      "FLN",
		Name:      "Hercilio Luz International Airport",
		City:      "Florianopolis",
		State:     "Santa-Catarina",
		Country:   "BR",
		Elevation: 16,
		Latitude:  -27.6702785492,
		Longitude: -48.5525016785,
		Timezone:  "America/Sao_Paulo",
	},
	"SBFN": {
		ICAO:      "SBFN",
		IATA:      "FEN",
		Name:      "Fernando de Noronha Airport",
		City:      "Fernando De Noronha",
		State:     "Pernambuco",
		Country:   "BR",
		Elevation: 193,
		Latitude:  -3.854929924,
		Longitude: -32.4233016968,
		Timezone:  "America/Noronha",
	},
	"SBFU": {
		ICAO:      "SBFU",
		IATA:      "0",
		Name:      "Furnas Airport",
		City:      "Sao Jose Da Barra",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2413,
		Latitude:  -20.7028007507,
		Longitude: -46.3353004456,
		Timezone:  "America/Sao_Paulo",
	},
	"SBFZ": {
		ICAO:      "SBFZ",
		IATA:      "FOR",
		Name:      "Pinto Martins International Airport",
		City:      "Fortaleza",
		State:     "Ceara",
		Country:   "BR",
		Elevation: 82,
		Latitude:  -3.7762799263,
		Longitude: -38.5326004028,
		Timezone:  "America/Fortaleza",
	},
	"SBGL": {
		ICAO:      "SBGL",
		IATA:      "GIG",
		Name:      "Galeao - Antonio Carlos Jobim International Airport",
		City:      "Rio De Janeiro",
		State:     "Rio-de-Janeiro",
		Country:   "BR",
		Elevation: 28,
		Latitude:  -22.8099994659,
		Longitude: -43.2505569458,
		Timezone:  "America/Sao_Paulo",
	},
	"SBGM": {
		ICAO:      "SBGM",
		IATA:      "GJM",
		Name:      "Guajara-Mirim Airport",
		City:      "Guajara-Mirim",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 478,
		Latitude:  -10.7863998413,
		Longitude: -65.2847976685,
		Timezone:  "America/Porto_Velho",
	},
	"SBGO": {
		ICAO:      "SBGO",
		IATA:      "GYN",
		Name:      "Santa Genoveva Airport",
		City:      "Goiania",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2450,
		Latitude:  -16.6319999695,
		Longitude: -49.2206993103,
		Timezone:  "America/Sao_Paulo",
	},
	"SBGP": {
		ICAO:      "SBGP",
		IATA:      "0",
		Name:      "EMBRAER - Unidade Gaviao Peixoto Airport",
		City:      "Gaviao Peixoto",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1998,
		Latitude:  -21.7737007141,
		Longitude: -48.4051017761,
		Timezone:  "America/Sao_Paulo",
	},
	"SBGR": {
		ICAO:      "SBGR",
		IATA:      "GRU",
		Name:      "Guarulhos - Governador Andre Franco Montoro International Airport",
		City:      "Sao Paulo",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2459,
		Latitude:  -23.4355564117,
		Longitude: -46.4730567932,
		Timezone:  "America/Sao_Paulo",
	},
	"SBGU": {
		ICAO:      "SBGU",
		IATA:      "GPB",
		Name:      "Tancredo Thomas de Faria Airport",
		City:      "Guarapuava",
		State:     "Parana",
		Country:   "BR",
		Elevation: 3494,
		Latitude:  -25.3875007629,
		Longitude: -51.520198822,
		Timezone:  "America/Sao_Paulo",
	},
	"SBGV": {
		ICAO:      "SBGV",
		IATA:      "GVR",
		Name:      "Governador Valadares Airport",
		City:      "Governador Valadares",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 561,
		Latitude:  -18.8952007294,
		Longitude: -41.9822006226,
		Timezone:  "America/Sao_Paulo",
	},
	"SBGW": {
		ICAO:      "SBGW",
		IATA:      "GUJ",
		Name:      "Guaratingueta Airport",
		City:      "Guaratingueta",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1761,
		Latitude:  -22.7915992737,
		Longitude: -45.2047996521,
		Timezone:  "America/Sao_Paulo",
	},
	"SBHT": {
		ICAO:      "SBHT",
		IATA:      "ATM",
		Name:      "Altamira Airport",
		City:      "Altamira",
		State:     "Para",
		Country:   "BR",
		Elevation: 369,
		Latitude:  -3.2539100647,
		Longitude: -52.2540016174,
		Timezone:  "America/Santarem",
	},
	"SBIC": {
		ICAO:      "SBIC",
		IATA:      "ITA",
		Name:      "Itacoatiara Airport",
		City:      "Itacoatiara",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 142,
		Latitude:  -3.1272599697,
		Longitude: -58.4812011719,
		Timezone:  "America/Manaus",
	},
	"SBIH": {
		ICAO:      "SBIH",
		IATA:      "ITB",
		Name:      "Itaituba Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 110,
		Latitude:  -4.2423400879,
		Longitude: -56.0007019043,
		Timezone:  "America/Santarem",
	},
	"SBIL": {
		ICAO:      "SBIL",
		IATA:      "IOS",
		Name:      "Bahia - Jorge Amado Airport",
		City:      "Ilheus",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 15,
		Latitude:  -14.8159999847,
		Longitude: -39.0331993103,
		Timezone:  "America/Bahia",
	},
	"SBIP": {
		ICAO:      "SBIP",
		IATA:      "IPN",
		Name:      "Usiminas Airport",
		City:      "Ipatinga",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 784,
		Latitude:  -19.4706993103,
		Longitude: -42.4875984192,
		Timezone:  "America/Sao_Paulo",
	},
	"SBIT": {
		ICAO:      "SBIT",
		IATA:      "0",
		Name:      "Hidroeletrica Airport",
		City:      "Itumbiara",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1630,
		Latitude:  -18.4447002411,
		Longitude: -49.2134017944,
		Timezone:  "America/Sao_Paulo",
	},
	"SBIZ": {
		ICAO:      "SBIZ",
		IATA:      "IMP",
		Name:      "Prefeito Renato Moreira Airport",
		City:      "Imperatriz",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 432,
		Latitude:  -5.5312900543,
		Longitude: -47.4599990845,
		Timezone:  "America/Fortaleza",
	},
	"SBJC": {
		ICAO:      "SBJC",
		IATA:      "0",
		Name:      "Julio Cesar Airport",
		City:      "Belem",
		State:     "Para",
		Country:   "BR",
		Elevation: 52,
		Latitude:  -1.4141600132,
		Longitude: -48.4607009888,
		Timezone:  "America/Belem",
	},
	"SBJD": {
		ICAO:      "SBJD",
		Name:      "Jundiai Airport",
		City:      "Jundiai",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2484,
		Latitude:  -23.1803690757,
		Longitude: -46.9444084167,
		Timezone:  "America/Sao_Paulo",
	},
	"SBJE": {
		ICAO:      "SBJE",
		IATA:      "JJD",
		Name:      "Comte. Ariston Pessoa Regional Airport",
		City:      "Jijoca de Jericoacoara",
		State:     "Ceara",
		Country:   "BR",
		Elevation: 89,
		Latitude:  -2.906667,
		Longitude: -40.358056,
		Timezone:  "America/Fortaleza",
	},
	"SBJF": {
		ICAO:      "SBJF",
		IATA:      "JDF",
		Name:      "Francisco de Assis Airport",
		City:      "Juiz De Fora",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2989,
		Latitude:  -21.7915000916,
		Longitude: -43.3867988586,
		Timezone:  "America/Sao_Paulo",
	},
	"SBJP": {
		ICAO:      "SBJP",
		IATA:      "JPA",
		Name:      "Presidente Castro Pinto Airport",
		City:      "Joao Pessoa",
		State:     "Paraiba",
		Country:   "BR",
		Elevation: 217,
		Latitude:  -7.1458330154,
		Longitude: -34.9486122131,
		Timezone:  "America/Fortaleza",
	},
	"SBJR": {
		ICAO:      "SBJR",
		Name:      "Jacarepagua - Roberto Marinho Airport",
		City:      "Rio De Janeiro",
		State:     "Rio-de-Janeiro",
		Country:   "BR",
		Elevation: 10,
		Latitude:  -22.9874992371,
		Longitude: -43.3699989319,
		Timezone:  "America/Sao_Paulo",
	},
	"SBJU": {
		ICAO:      "SBJU",
		IATA:      "JDO",
		Name:      "Orlando Bezerra de Menezes Airport",
		City:      "Juazeiro Do Norte",
		State:     "Ceara",
		Country:   "BR",
		Elevation: 1392,
		Latitude:  -7.2189598084,
		Longitude: -39.2700996399,
		Timezone:  "America/Fortaleza",
	},
	"SBJV": {
		ICAO:      "SBJV",
		IATA:      "JOI",
		Name:      "Lauro Carneiro de Loyola Airport",
		City:      "Joinville",
		State:     "Santa-Catarina",
		Country:   "BR",
		Elevation: 15,
		Latitude:  -26.2245006561,
		Longitude: -48.7974014282,
		Timezone:  "America/Sao_Paulo",
	},
	"SBKG": {
		ICAO:      "SBKG",
		IATA:      "CPV",
		Name:      "Presidente Joao Suassuna Airport",
		City:      "Campina Grande",
		State:     "Paraiba",
		Country:   "BR",
		Elevation: 1646,
		Latitude:  -7.2699198723,
		Longitude: -35.8964004517,
		Timezone:  "America/Fortaleza",
	},
	"SBKP": {
		ICAO:      "SBKP",
		IATA:      "VCP",
		Name:      "Viracopos International Airport",
		City:      "Campinas",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2170,
		Latitude:  -23.0074005127,
		Longitude: -47.1344985962,
		Timezone:  "America/Sao_Paulo",
	},
	"SBLE": {
		ICAO:      "SBLE",
		IATA:      "LEC",
		Name:      "Chapada Diamantina Airport",
		City:      "Lencois",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1676,
		Latitude:  -12.4822998047,
		Longitude: -41.2770004272,
		Timezone:  "America/Bahia",
	},
	"SBLJ": {
		ICAO:      "SBLJ",
		IATA:      "LAJ",
		Name:      "Lages Airport",
		City:      "Lages",
		State:     "Santa-Catarina",
		Country:   "BR",
		Elevation: 3065,
		Latitude:  -27.7821006775,
		Longitude: -50.28150177,
		Timezone:  "America/Sao_Paulo",
	},
	"SBLN": {
		ICAO:      "SBLN",
		IATA:      "LIP",
		Name:      "Lins Airport",
		City:      "Lins",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1559,
		Latitude:  -21.6639995575,
		Longitude: -49.7304992676,
		Timezone:  "America/Sao_Paulo",
	},
	"SBLO": {
		ICAO:      "SBLO",
		IATA:      "LDB",
		Name:      "Governador Jose Richa Airport",
		City:      "Londrina",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1867,
		Latitude:  -23.3335990906,
		Longitude: -51.1301002502,
		Timezone:  "America/Sao_Paulo",
	},
	"SBLP": {
		ICAO:      "SBLP",
		IATA:      "LAZ",
		Name:      "Bom Jesus da Lapa Airport",
		City:      "Bom Jesus Da Lapa",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1454,
		Latitude:  -13.2621002197,
		Longitude: -43.4081001282,
		Timezone:  "America/Bahia",
	},
	"SBLS": {
		ICAO:      "SBLS",
		IATA:      "0",
		Name:      "Lagoa Santa Airport",
		City:      "Lagoa Santa",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2795,
		Latitude:  -19.6616001129,
		Longitude: -43.8964004517,
		Timezone:  "America/Sao_Paulo",
	},
	"SBMA": {
		ICAO:      "SBMA",
		IATA:      "MAB",
		Name:      "Maraba Airport",
		City:      "Maraba",
		State:     "Para",
		Country:   "BR",
		Elevation: 357,
		Latitude:  -5.3685898781,
		Longitude: -49.1380004883,
		Timezone:  "America/Belem",
	},
	"SBMC": {
		ICAO:      "SBMC",
		IATA:      "MQH",
		Name:      "Minacu Airport",
		City:      "Minacu",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1401,
		Latitude:  -13.5490999222,
		Longitude: -48.1953010559,
		Timezone:  "America/Sao_Paulo",
	},
	"SBMD": {
		ICAO:      "SBMD",
		IATA:      "MEU",
		Name:      "Monte Dourado Airport",
		City:      "Almeirim",
		State:     "Para",
		Country:   "BR",
		Elevation: 677,
		Latitude:  -0.8898389935,
		Longitude: -52.6021995544,
		Timezone:  "America/Santarem",
	},
	"SBME": {
		ICAO:      "SBME",
		IATA:      "MEA",
		Name:      "Macae Airport",
		City:      "Macae",
		State:     "Rio-de-Janeiro",
		Country:   "BR",
		Elevation: 8,
		Latitude:  -22.343000412,
		Longitude: -41.7659988403,
		Timezone:  "America/Sao_Paulo",
	},
	"SBMG": {
		ICAO:      "SBMG",
		IATA:      "MGF",
		Name:      "Regional de Maringa - Silvio Nane Junior Airport",
		City:      "Maringa",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1788,
		Latitude:  -23.4794445038,
		Longitude: -52.01222229,
		Timezone:  "America/Sao_Paulo",
	},
	"SBMK": {
		ICAO:      "SBMK",
		IATA:      "MOC",
		Name:      "Mario Ribeiro Airport",
		City:      "Montes Claros",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2191,
		Latitude:  -16.7068996429,
		Longitude: -43.818901062,
		Timezone:  "America/Sao_Paulo",
	},
	"SBML": {
		ICAO:      "SBML",
		IATA:      "MII",
		Name:      "Marilia Airport",
		City:      "Marilia",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2122,
		Latitude:  -22.1968994141,
		Longitude: -49.926399231,
		Timezone:  "America/Sao_Paulo",
	},
	"SBMN": {
		ICAO:      "SBMN",
		IATA:      "PLL",
		Name:      "Ponta Pelada Airport",
		City:      "Manaus",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 267,
		Latitude:  -3.1460399628,
		Longitude: -59.9863014221,
		Timezone:  "America/Manaus",
	},
	"SBMO": {
		ICAO:      "SBMO",
		IATA:      "MCZ",
		Name:      "Zumbi dos Palmares Airport",
		City:      "Maceio",
		State:     "Alagoas",
		Country:   "BR",
		Elevation: 387,
		Latitude:  -9.5108098984,
		Longitude: -35.7916984558,
		Timezone:  "America/Maceio",
	},
	"SBMQ": {
		ICAO:      "SBMQ",
		IATA:      "MCP",
		Name:      "Alberto Alcolumbre Airport",
		City:      "Macapa",
		State:     "Amapa",
		Country:   "BR",
		Elevation: 56,
		Latitude:  0.0506640002,
		Longitude: -51.0722007751,
		Timezone:  "America/Belem",
	},
	"SBMS": {
		ICAO:      "SBMS",
		IATA:      "MVF",
		Name:      "Dix-Sept Rosado Airport",
		City:      "Mossoro",
		State:     "Rio-Grande-do-Norte",
		Country:   "BR",
		Elevation: 76,
		Latitude:  -5.2019200325,
		Longitude: -37.3642997742,
		Timezone:  "America/Fortaleza",
	},
	"SBMT": {
		ICAO:      "SBMT",
		IATA:      "SAO",
		Name:      "Campo de Marte Airport",
		City:      "Sao Paulo",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2368,
		Latitude:  -23.5090999603,
		Longitude: -46.6377983093,
		Timezone:  "America/Sao_Paulo",
	},
	"SBMY": {
		ICAO:      "SBMY",
		IATA:      "MNX",
		Name:      "Manicore Airport",
		City:      "Manicore",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 174,
		Latitude:  -5.8113799095,
		Longitude: -61.278301239,
		Timezone:  "America/Manaus",
	},
	"SBNF": {
		ICAO:      "SBNF",
		IATA:      "NVT",
		Name:      "Ministro Victor Konder International Airport",
		City:      "Navegantes",
		State:     "Santa-Catarina",
		Country:   "BR",
		Elevation: 18,
		Latitude:  -26.8799991608,
		Longitude: -48.6514015198,
		Timezone:  "America/Sao_Paulo",
	},
	"SBNM": {
		ICAO:      "SBNM",
		IATA:      "GEL",
		Name:      "Santo Angelo Airport",
		City:      "Santo Angelo",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 1056,
		Latitude:  -28.2817001343,
		Longitude: -54.1691017151,
		Timezone:  "America/Sao_Paulo",
	},
	"SBNT": {
		ICAO:      "SBNT",
		IATA:      "NAT",
		Name:      "Augusto Severo Airport",
		City:      "Natal",
		State:     "Rio-Grande-do-Norte",
		Country:   "BR",
		Elevation: 169,
		Latitude:  -5.9114198685,
		Longitude: -35.2476997375,
		Timezone:  "America/Fortaleza",
	},
	"SBOI": {
		ICAO:      "SBOI",
		IATA:      "OYK",
		Name:      "Oiapoque Airport",
		City:      "Oiapoque",
		Country:   "BR",
		Elevation: 63,
		Latitude:  3.8554899693,
		Longitude: -51.7969017029,
		Timezone:  "America/Belem",
	},
	"SBPA": {
		ICAO:      "SBPA",
		IATA:      "POA",
		Name:      "Salgado Filho Airport",
		City:      "Porto Alegre",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 11,
		Latitude:  -29.9944000244,
		Longitude: -51.1713981628,
		Timezone:  "America/Sao_Paulo",
	},
	"SBPB": {
		ICAO:      "SBPB",
		IATA:      "PHB",
		Name:      "Prefeito Doutor Joao Silva Filho Airport",
		City:      "Parnaiba",
		State:     "Piaui",
		Country:   "BR",
		Elevation: 16,
		Latitude:  -2.8937499523,
		Longitude: -41.7319984436,
		Timezone:  "America/Fortaleza",
	},
	"SBPC": {
		ICAO:      "SBPC",
		IATA:      "POO",
		Name:      "Pocos de Caldas Airport",
		City:      "Pocos De Caldas",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 4135,
		Latitude:  -21.843000412,
		Longitude: -46.5679016113,
		Timezone:  "America/Sao_Paulo",
	},
	"SBPF": {
		ICAO:      "SBPF",
		IATA:      "PFB",
		Name:      "Lauro Kurtz Airport",
		City:      "Passo Fundo",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 2376,
		Latitude:  -28.2439994812,
		Longitude: -52.3265991211,
		Timezone:  "America/Sao_Paulo",
	},
	"SBPJ": {
		ICAO:      "SBPJ",
		IATA:      "PMW",
		Name:      "Brigadeiro Lysias Rodrigues Airport",
		City:      "Palmas",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 774,
		Latitude:  -10.2915000916,
		Longitude: -48.3569984436,
		Timezone:  "America/Araguaina",
	},
	"SBPK": {
		ICAO:      "SBPK",
		IATA:      "PET",
		Name:      "Pelotas Airport",
		City:      "Pelotas",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 59,
		Latitude:  -31.7183990479,
		Longitude: -52.3277015686,
		Timezone:  "America/Sao_Paulo",
	},
	"SBPL": {
		ICAO:      "SBPL",
		IATA:      "PNZ",
		Name:      "Senador Nilo Coelho Airport",
		City:      "Petrolina",
		State:     "Pernambuco",
		Country:   "BR",
		Elevation: 1263,
		Latitude:  -9.3624095917,
		Longitude: -40.5690994263,
		Timezone:  "America/Recife",
	},
	"SBPN": {
		ICAO:      "SBPN",
		IATA:      "PNB",
		Name:      "Porto Nacional Airport",
		City:      "Porto Nacional",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 870,
		Latitude:  -10.7194004059,
		Longitude: -48.3997001648,
		Timezone:  "America/Araguaina",
	},
	"SBPP": {
		ICAO:      "SBPP",
		IATA:      "PMG",
		Name:      "Ponta Pora Airport",
		City:      "Ponta Pora",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 2156,
		Latitude:  -22.5496006012,
		Longitude: -55.7025985718,
		Timezone:  "America/Asuncion",
	},
	"SBPR": {
		ICAO:      "SBPR",
		IATA:      "0",
		Name:      "Carlos Prates Airport",
		City:      "Belo Horizonte",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3044,
		Latitude:  -19.9092998505,
		Longitude: -43.9906997681,
		Timezone:  "America/Sao_Paulo",
	},
	"SBPS": {
		ICAO:      "SBPS",
		IATA:      "BPS",
		Name:      "Porto Seguro Airport",
		City:      "Porto Seguro",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 168,
		Latitude:  -16.4386005402,
		Longitude: -39.0808982849,
		Timezone:  "America/Bahia",
	},
	"SBPV": {
		ICAO:      "SBPV",
		IATA:      "PVH",
		Name:      "Governador Jorge Teixeira de Oliveira Airport",
		City:      "Porto Velho",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 290,
		Latitude:  -8.7092895508,
		Longitude: -63.9023017883,
		Timezone:  "America/Porto_Velho",
	},
	"SBQV": {
		ICAO:      "SBQV",
		IATA:      "VDC",
		Name:      "Vitoria da Conquista Airport",
		City:      "Vitoria Da Conquista",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 3002,
		Latitude:  -14.8627996445,
		Longitude: -40.8630981445,
		Timezone:  "America/Bahia",
	},
	"SBRB": {
		ICAO:      "SBRB",
		IATA:      "RBR",
		Name:      "Placido de Castro Airport",
		City:      "Rio Branco",
		State:     "Acre",
		Country:   "BR",
		Elevation: 633,
		Latitude:  -9.868888855,
		Longitude: -67.8980560303,
		Timezone:  "America/Rio_Branco",
	},
	"SBRF": {
		ICAO:      "SBRF",
		IATA:      "REC",
		Name:      "Guararapes - Gilberto Freyre International Airport",
		City:      "Recife",
		State:     "Pernambuco",
		Country:   "BR",
		Elevation: 33,
		Latitude:  -8.1264896393,
		Longitude: -34.9235992432,
		Timezone:  "America/Recife",
	},
	"SBRJ": {
		ICAO:      "SBRJ",
		IATA:      "SDU",
		Name:      "Santos Dumont Airport",
		City:      "Rio De Janeiro",
		State:     "Rio-de-Janeiro",
		Country:   "BR",
		Elevation: 11,
		Latitude:  -22.9104995728,
		Longitude: -43.1631011963,
		Timezone:  "America/Sao_Paulo",
	},
	"SBRP": {
		ICAO:      "SBRP",
		IATA:      "RAO",
		Name:      "Leite Lopes Airport",
		City:      "Ribeirao Preto",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1806,
		Latitude:  -21.1363887787,
		Longitude: -47.7766685486,
		Timezone:  "America/Sao_Paulo",
	},
	"SBRR": {
		ICAO:      "SBRR",
		IATA:      "BRB",
		Name:      "Barreirinhas Airport",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 40,
		Latitude:  -2.7555555556,
		Longitude: -42.81,
		Timezone:  "America/Fortaleza",
	},
	"SBSC": {
		ICAO:      "SBSC",
		IATA:      "SNZ",
		Name:      "Santa Cruz Airport",
		City:      "Rio De Janeiro",
		State:     "Rio-de-Janeiro",
		Country:   "BR",
		Elevation: 10,
		Latitude:  -22.9323997498,
		Longitude: -43.7191009521,
		Timezone:  "America/Sao_Paulo",
	},
	"SBSJ": {
		ICAO:      "SBSJ",
		IATA:      "SJK",
		Name:      "Professor Urbano Ernesto Stumpf Airport",
		City:      "Sao Jose Dos Campos",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2120,
		Latitude:  -23.2292003632,
		Longitude: -45.8614997864,
		Timezone:  "America/Sao_Paulo",
	},
	"SBSL": {
		ICAO:      "SBSL",
		IATA:      "SLZ",
		Name:      "Marechal Cunha Machado International Airport",
		City:      "Sao Luis",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 178,
		Latitude:  -2.5853600502,
		Longitude: -44.2341003418,
		Timezone:  "America/Fortaleza",
	},
	"SBSM": {
		ICAO:      "SBSM",
		IATA:      "RIA",
		Name:      "Santa Maria Airport",
		City:      "Santa Maria",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 287,
		Latitude:  -29.7113990784,
		Longitude: -53.6882019043,
		Timezone:  "America/Sao_Paulo",
	},
	"SBSN": {
		ICAO:      "SBSN",
		IATA:      "STM",
		Name:      "Maestro Wilson Fonseca Airport",
		City:      "Santarem",
		State:     "Para",
		Country:   "BR",
		Elevation: 198,
		Latitude:  -2.4247219563,
		Longitude: -54.7858314514,
		Timezone:  "America/Santarem",
	},
	"SBSP": {
		ICAO:      "SBSP",
		IATA:      "CGH",
		Name:      "Congonhas Airport",
		City:      "Sao Paulo",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2631,
		Latitude:  -23.6261100769,
		Longitude: -46.6563873291,
		Timezone:  "America/Sao_Paulo",
	},
	"SBSR": {
		ICAO:      "SBSR",
		IATA:      "SJP",
		Name:      "Sao Jose do Rio Preto Airport",
		City:      "Sao Jose Do Rio Preto",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1784,
		Latitude:  -20.8166007996,
		Longitude: -49.40650177,
		Timezone:  "America/Sao_Paulo",
	},
	"SBST": {
		ICAO:      "SBST",
		IATA:      "SSZ",
		Name:      "Base Aerea de Santos Airport",
		City:      "Guaruja",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 10,
		Latitude:  -23.9280567169,
		Longitude: -46.2997207642,
		Timezone:  "America/Sao_Paulo",
	},
	"SBSV": {
		ICAO:      "SBSV",
		IATA:      "SSA",
		Name:      "Deputado Luiz Eduardo Magalhaes International Airport",
		City:      "Salvador",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 64,
		Latitude:  -12.9086112976,
		Longitude: -38.3224983215,
		Timezone:  "America/Bahia",
	},
	"SBTA": {
		ICAO:      "SBTA",
		IATA:      "QHP",
		Name:      "Base de Aviacao de Taubate Airport",
		City:      "Taubate",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1908,
		Latitude:  -23.0401000977,
		Longitude: -45.5159988403,
		Timezone:  "America/Sao_Paulo",
	},
	"SBTB": {
		ICAO:      "SBTB",
		IATA:      "TMT",
		Name:      "Trombetas Airport",
		City:      "Oriximina",
		State:     "Para",
		Country:   "BR",
		Elevation: 287,
		Latitude:  -1.4895999432,
		Longitude: -56.3968009949,
		Timezone:  "America/Santarem",
	},
	"SBTC": {
		ICAO:      "SBTC",
		IATA:      "UNA",
		Name:      "Hotel Transamerica Airport",
		City:      "Una",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 20,
		Latitude:  -15.3551998138,
		Longitude: -38.9990005493,
		Timezone:  "America/Bahia",
	},
	"SBTD": {
		ICAO:      "SBTD",
		IATA:      "TOW",
		Name:      "Toledo Airport",
		City:      "Toledo",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1843,
		Latitude:  -24.6863002777,
		Longitude: -53.6974983215,
		Timezone:  "America/Sao_Paulo",
	},
	"SBTE": {
		ICAO:      "SBTE",
		IATA:      "THE",
		Name:      "Senador Petronio Portela Airport",
		City:      "Teresina",
		State:     "Piaui",
		Country:   "BR",
		Elevation: 219,
		Latitude:  -5.0599398613,
		Longitude: -42.8235015869,
		Timezone:  "America/Fortaleza",
	},
	"SBTF": {
		ICAO:      "SBTF",
		IATA:      "TFF",
		Name:      "Tefe Airport",
		City:      "Tefe",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 188,
		Latitude:  -3.3829400539,
		Longitude: -64.7240982056,
		Timezone:  "America/Manaus",
	},
	"SBTG": {
		ICAO:      "SBTG",
		IATA:      "TJL",
		Name:      "Plinio Alarcom Airport",
		City:      "Tres Lagoas",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1075,
		Latitude:  -20.754444,
		Longitude: -51.683889,
		Timezone:  "America/Campo_Grande",
	},
	"SBTK": {
		ICAO:      "SBTK",
		IATA:      "TRQ",
		Name:      "Tarauaca Airport",
		City:      "Tarauaca",
		State:     "Acre",
		Country:   "BR",
		Elevation: 646,
		Latitude:  -8.1552600861,
		Longitude: -70.7833023071,
		Timezone:  "America/Rio_Branco",
	},
	"SBTL": {
		ICAO:      "SBTL",
		IATA:      "TEC",
		Name:      "Telemaco Borba Airport",
		City:      "Telemaco Borba",
		State:     "Parana",
		Country:   "BR",
		Elevation: 2610,
		Latitude:  -24.3178005219,
		Longitude: -50.651599884,
		Timezone:  "America/Sao_Paulo",
	},
	"SBTR": {
		ICAO:      "SBTR",
		IATA:      "0",
		Name:      "Torres Airport",
		City:      "Torres",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 26,
		Latitude:  -29.414899826,
		Longitude: -49.8100013733,
		Timezone:  "America/Sao_Paulo",
	},
	"SBTS": {
		ICAO:      "SBTS",
		Name:      "Tirios Airport",
		City:      "Obidos",
		State:     "Para",
		Country:   "BR",
		Elevation: 1127,
		Latitude:  2.2234699726,
		Longitude: -55.9460983276,
		Timezone:  "America/Santarem",
	},
	"SBTT": {
		ICAO:      "SBTT",
		IATA:      "TBT",
		Name:      "Tabatinga Airport",
		City:      "Tabatinga",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 279,
		Latitude:  -4.2556700706,
		Longitude: -69.935798645,
		Timezone:  "America/Bogota",
	},
	"SBTU": {
		ICAO:      "SBTU",
		IATA:      "TUR",
		Name:      "Tucurui Airport",
		City:      "Tucurui",
		State:     "Para",
		Country:   "BR",
		Elevation: 830,
		Latitude:  -3.7860100269,
		Longitude: -49.7202987671,
		Timezone:  "America/Belem",
	},
	"SBTV": {
		ICAO:      "SBTV",
		Name:      "Terravista Airport",
		City:      "Porto Seguro",
		Country:   "BR",
		Elevation: 174,
		Latitude:  -16.5408325195,
		Longitude: -39.1077766418,
		Timezone:  "America/Bahia",
	},
	"SBUA": {
		ICAO:      "SBUA",
		IATA:      "SJL",
		Name:      "Sao Gabriel da Cachoeira Airport",
		City:      "Sao Gabriel Da Cachoeira",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 251,
		Latitude:  -0.14835,
		Longitude: -66.9855,
		Timezone:  "America/Manaus",
	},
	"SBUF": {
		ICAO:      "SBUF",
		IATA:      "PAV",
		Name:      "Paulo Afonso Airport",
		City:      "Paulo Afonso",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 883,
		Latitude:  -9.4008798599,
		Longitude: -38.2505989075,
		Timezone:  "America/Bahia",
	},
	"SBUG": {
		ICAO:      "SBUG",
		IATA:      "URG",
		Name:      "Rubem Berta Airport",
		City:      "Uruguaiana",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 256,
		Latitude:  -29.7821998596,
		Longitude: -57.0382003784,
		Timezone:  "America/Sao_Paulo",
	},
	"SBUL": {
		ICAO:      "SBUL",
		IATA:      "UDI",
		Name:      "Ten. Cel. Aviador Cesar Bombonato Airport",
		City:      "Uberlandia",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3094,
		Latitude:  -18.8836116791,
		Longitude: -48.225276947,
		Timezone:  "America/Sao_Paulo",
	},
	"SBUR": {
		ICAO:      "SBUR",
		IATA:      "UBA",
		Name:      "Mario de Almeida Franco Airport",
		City:      "Uberaba",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2655,
		Latitude:  -19.7647228241,
		Longitude: -47.9661102295,
		Timezone:  "America/Sao_Paulo",
	},
	"SBVG": {
		ICAO:      "SBVG",
		IATA:      "VAG",
		Name:      "Major Brigadeiro Trompowsky Airport",
		City:      "Varginha",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3025,
		Latitude:  -21.5900993347,
		Longitude: -45.4733009338,
		Timezone:  "America/Sao_Paulo",
	},
	"SBVH": {
		ICAO:      "SBVH",
		IATA:      "BVH",
		Name:      "Vilhena Airport",
		City:      "Vilhena",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 2018,
		Latitude:  -12.6943998337,
		Longitude: -60.0983009338,
		Timezone:  "America/Cuiaba",
	},
	"SBVT": {
		ICAO:      "SBVT",
		IATA:      "VIX",
		Name:      "Eurico de Aguiar Salles Airport",
		City:      "Vitoria",
		State:     "Espirito-Santo",
		Country:   "BR",
		Elevation: 11,
		Latitude:  -20.2580566406,
		Longitude: -40.2863883972,
		Timezone:  "America/Sao_Paulo",
	},
	"SBYA": {
		ICAO:      "SBYA",
		IATA:      "0",
		Name:      "Iauarete Airport",
		City:      "Sao Gabriel Da Cachoeira",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 345,
		Latitude:  0.6075000167,
		Longitude: -69.185798645,
		Timezone:  "America/Bogota",
	},
	"SBYS": {
		ICAO:      "SBYS",
		IATA:      "QPS",
		Name:      "Campo Fontenelle Airport",
		City:      "Pirassununga",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1968,
		Latitude:  -21.9846000671,
		Longitude: -47.3348007202,
		Timezone:  "America/Sao_Paulo",
	},
	"SBZM": {
		ICAO:      "SBZM",
		IATA:      "IZA",
		Name:      "Zona da Mata Regional Airport",
		City:      "Juiz De Fora",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1348,
		Latitude:  -21.5130558014,
		Longitude: -43.1730575562,
		Timezone:  "America/Sao_Paulo",
	},
	"SC00": {
		ICAO:      "SC00",
		Name:      "Triple Tree Airport",
		City:      "Woodruff",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 680,
		Latitude:  34.6744003296,
		Longitude: -82.0014038086,
		Timezone:  "America/New_York",
	},
	"SC01": {
		ICAO:      "SC01",
		Name:      "Sugar Hill Airport",
		City:      "Allendale",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 203,
		Latitude:  33.0390014648,
		Longitude: -81.2942962646,
		Timezone:  "America/New_York",
	},
	"SC03": {
		ICAO:      "SC03",
		Name:      "Mc Neil Airport",
		City:      "Cameron",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 170,
		Latitude:  33.5570983887,
		Longitude: -80.648399353,
		Timezone:  "America/New_York",
	},
	"SC05": {
		ICAO:      "SC05",
		Name:      "Laurel Hill Plantation Airport",
		City:      "Beaufort",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 10,
		Latitude:  32.4912986755,
		Longitude: -80.6150970459,
		Timezone:  "America/New_York",
	},
	"SC06": {
		ICAO:      "SC06",
		Name:      "Pluff Mud Field",
		City:      "Charleston",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 15,
		Latitude:  32.6320991516,
		Longitude: -80.092300415,
		Timezone:  "America/New_York",
	},
	"SC07": {
		ICAO:      "SC07",
		Name:      "Alan's Airport",
		City:      "St Matthews",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 369,
		Latitude:  33.6758003235,
		Longitude: -80.9496994019,
		Timezone:  "America/New_York",
	},
	"SC08": {
		ICAO:      "SC08",
		Name:      "Bethel-Lake Wylie Airport",
		City:      "York",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 670,
		Latitude:  35.0500984192,
		Longitude: -81.137298584,
		Timezone:  "America/New_York",
	},
	"SC10": {
		ICAO:      "SC10",
		Name:      "State Line Ultraport/Flightpark Ultralightport",
		City:      "Chesnee",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 870,
		Latitude:  35.1785011292,
		Longitude: -81.8179016113,
		Timezone:  "America/New_York",
	},
	"SC12": {
		ICAO:      "SC12",
		Name:      "Davis Airport",
		City:      "Garnett",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 82,
		Latitude:  32.6721000671,
		Longitude: -81.3112030029,
		Timezone:  "America/New_York",
	},
	"SC13": {
		ICAO:      "SC13",
		Name:      "Darden Airport",
		City:      "Gaston",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 450,
		Latitude:  33.8406982422,
		Longitude: -81.1837005615,
		Timezone:  "America/New_York",
	},
	"SC14": {
		ICAO:      "SC14",
		Name:      "Shealy Airport",
		City:      "Gilbert",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 380,
		Latitude:  34.0625991821,
		Longitude: -81.4097976685,
		Timezone:  "America/New_York",
	},
	"SC15": {
		ICAO:      "SC15",
		Name:      "Airy Hall Airport",
		City:      "Green Pond",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 20,
		Latitude:  32.6320991516,
		Longitude: -80.5039978027,
		Timezone:  "America/New_York",
	},
	"SC16": {
		ICAO:      "SC16",
		Name:      "Curry Airport",
		City:      "Hartsville",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 199,
		Latitude:  34.3778991699,
		Longitude: -80.0053024292,
		Timezone:  "America/New_York",
	},
	"SC17": {
		ICAO:      "SC17",
		Name:      "Russell Airport",
		City:      "Holly Hill",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 110,
		Latitude:  33.3578987122,
		Longitude: -80.4598007202,
		Timezone:  "America/New_York",
	},
	"SC19": {
		ICAO:      "SC19",
		Name:      "Lamar Airport",
		City:      "Lamar",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 175,
		Latitude:  34.1677017212,
		Longitude: -80.0775985718,
		Timezone:  "America/New_York",
	},
	"SC1A": {
		ICAO:      "SC1A",
		Name:      "Canchones West Airport",
		City:      "Canchones",
		State:     "Tarapaca",
		Country:   "CL",
		Elevation: 3251,
		Latitude:  -20.4270992279,
		Longitude: -69.6415023804,
		Timezone:  "America/Santiago",
	},
	"SC20": {
		ICAO:      "SC20",
		Name:      "Harman Airport",
		City:      "Langley",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 458,
		Latitude:  33.4842987061,
		Longitude: -81.8258972168,
		Timezone:  "America/New_York",
	},
	"SC21": {
		ICAO:      "SC21",
		Name:      "Myrtle Beach Hardee Airpark",
		City:      "Loris",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 45,
		Latitude:  33.9417991638,
		Longitude: -78.8330993652,
		Timezone:  "America/New_York",
	},
	"SC23": {
		ICAO:      "SC23",
		Name:      "Eagles Nest-Fairview Airpark",
		City:      "Pelion",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 460,
		Latitude:  33.763999939,
		Longitude: -81.3587036133,
		Timezone:  "America/New_York",
	},
	"SC24": {
		ICAO:      "SC24",
		Name:      "Eagle Ridge Airport",
		City:      "Seneca",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 850,
		Latitude:  34.6716995239,
		Longitude: -82.9135971069,
		Timezone:  "America/New_York",
	},
	"SC25": {
		ICAO:      "SC25",
		Name:      "Ross Strip",
		City:      "W Society Hill",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 190,
		Latitude:  34.4226989746,
		Longitude: -79.884803772,
		Timezone:  "America/New_York",
	},
	"SC26": {
		ICAO:      "SC26",
		Name:      "Hawks Nest Farm Airport",
		City:      "West Union",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 1050,
		Latitude:  34.8081016541,
		Longitude: -82.9985961914,
		Timezone:  "America/New_York",
	},
	"SC27": {
		ICAO:      "SC27",
		Name:      "Tallon Field",
		City:      "Lynchburg",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 190,
		Latitude:  34.1010017395,
		Longitude: -80.1256027222,
		Timezone:  "America/New_York",
	},
	"SC28": {
		ICAO:      "SC28",
		Name:      "Javika Airport",
		City:      "Surfside Beach",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 26,
		Latitude:  33.6520996094,
		Longitude: -78.9942016602,
		Timezone:  "America/New_York",
	},
	"SC29": {
		ICAO:      "SC29",
		Name:      "Mc Kay Airport",
		City:      "Timmonsville",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 135,
		Latitude:  34.1106987,
		Longitude: -79.9014968872,
		Timezone:  "America/New_York",
	},
	"SC32": {
		ICAO:      "SC32",
		Name:      "Sky Valley Airpark",
		City:      "Pelzer",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 900,
		Latitude:  34.6503982544,
		Longitude: -82.5035018921,
		Timezone:  "America/New_York",
	},
	"SC33": {
		ICAO:      "SC33",
		Name:      "Cox Airport",
		City:      "Simpsonville",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 1000,
		Latitude:  34.8242988586,
		Longitude: -82.259803772,
		Timezone:  "America/New_York",
	},
	"SC34": {
		ICAO:      "SC34",
		Name:      "Iva Field",
		City:      "Iva",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 660,
		Latitude:  34.2971000671,
		Longitude: -82.6837005615,
		Timezone:  "America/New_York",
	},
	"SC35": {
		ICAO:      "SC35",
		Name:      "Ehrhardt Airport",
		City:      "Ehrhardt",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 146,
		Latitude:  33.0895996094,
		Longitude: -81.0082015991,
		Timezone:  "America/New_York",
	},
	"SC36": {
		ICAO:      "SC36",
		Name:      "Emery Airport",
		City:      "Greer",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 1220,
		Latitude:  35.1246986389,
		Longitude: -82.3264007568,
		Timezone:  "America/New_York",
	},
	"SC37": {
		ICAO:      "SC37",
		Name:      "Crosswinds-Wilson Private Airport",
		City:      "Cross",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 90,
		Latitude:  33.2718009949,
		Longitude: -80.2543029785,
		Timezone:  "America/New_York",
	},
	"SC38": {
		ICAO:      "SC38",
		Name:      "Pocotaligo Airport",
		City:      "Manning",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 112,
		Latitude:  33.7075004578,
		Longitude: -80.0805969238,
		Timezone:  "America/New_York",
	},
	"SC39": {
		ICAO:      "SC39",
		Name:      "Green Pond Airport",
		City:      "Woodruff",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 788,
		Latitude:  34.808553284,
		Longitude: -82.0773768425,
		Timezone:  "America/New_York",
	},
	"SC40": {
		ICAO:      "SC40",
		Name:      "Pearson's Farm Airport",
		City:      "Spartanburg",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 850,
		Latitude:  35.0573005676,
		Longitude: -81.9448013306,
		Timezone:  "America/New_York",
	},
	"SC41": {
		ICAO:      "SC41",
		Name:      "Palmetto Air Plantation Airport",
		City:      "Manning",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 140,
		Latitude:  33.66609955,
		Longitude: -80.24389648,
		Timezone:  "America/New_York",
	},
	"SC42": {
		ICAO:      "SC42",
		Name:      "Hondarosa Airport",
		City:      "Charleston",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 13,
		Latitude:  32.6869010925,
		Longitude: -80.1063995361,
		Timezone:  "America/New_York",
	},
	"SC43": {
		ICAO:      "SC43",
		Name:      "Moore's Field",
		City:      "Williamston",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 871,
		Latitude:  34.5937004089,
		Longitude: -82.3990020752,
		Timezone:  "America/New_York",
	},
	"SC44": {
		ICAO:      "SC44",
		Name:      "El Porvenir Airpark",
		City:      "Boykin",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 250,
		Latitude:  34.12879944,
		Longitude: -80.54730225,
		Timezone:  "America/New_York",
	},
	"SC45": {
		ICAO:      "SC45",
		Name:      "Gilbert International Airport",
		City:      "Gilbert",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 551,
		Latitude:  33.8839988708,
		Longitude: -81.3817977905,
		Timezone:  "America/New_York",
	},
	"SC46": {
		ICAO:      "SC46",
		Name:      "House Movers Field",
		City:      "Batesburg",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 540,
		Latitude:  33.8510017395,
		Longitude: -81.5484008789,
		Timezone:  "America/New_York",
	},
	"SC47": {
		ICAO:      "SC47",
		Name:      "Parker Field",
		City:      "Simpsonville",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 860,
		Latitude:  34.7613983154,
		Longitude: -82.1613998413,
		Timezone:  "America/New_York",
	},
	"SC48": {
		ICAO:      "SC48",
		Name:      "Byrd Field",
		City:      "Turbeville",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 110,
		Latitude:  33.8843002319,
		Longitude: -79.9998016357,
		Timezone:  "America/New_York",
	},
	"SC49": {
		ICAO:      "SC49",
		Name:      "Oficina Victoria Airport",
		City:      "Oficina Victoria",
		State:     "Tarapaca",
		Country:   "CL",
		Elevation: 3280,
		Latitude:  -20.7346992493,
		Longitude: -69.6257019043,
		Timezone:  "America/Santiago",
	},
	"SC50": {
		ICAO:      "SC50",
		Name:      "Yonges Island Airport",
		City:      "Charleston",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 10,
		Latitude:  32.6973991394,
		Longitude: -80.2258987427,
		Timezone:  "America/New_York",
	},
	"SC51": {
		ICAO:      "SC51",
		Name:      "Too Goo Doo Farms Airport",
		City:      "Meggett",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 15,
		Latitude:  32.7046012878,
		Longitude: -80.2531967163,
		Timezone:  "America/New_York",
	},
	"SC52": {
		ICAO:      "SC52",
		Name:      "Oakview Airport",
		City:      "Six Mile",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 1040,
		Latitude:  34.8123016357,
		Longitude: -82.8188018799,
		Timezone:  "America/New_York",
	},
	"SC55": {
		ICAO:      "SC55",
		Name:      "Broxton Bridge Plantation Airport",
		City:      "Ehrhardt",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 120,
		Latitude:  33.0056991577,
		Longitude: -81.0484008789,
		Timezone:  "America/New_York",
	},
	"SC56": {
		ICAO:      "SC56",
		Name:      "Thomason Airfield",
		City:      "Mountville",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 642,
		Latitude:  34.3634986877,
		Longitude: -81.9932022095,
		Timezone:  "America/New_York",
	},
	"SC57": {
		ICAO:      "SC57",
		Name:      "Creech Aviation Facility Airport",
		City:      "Wedgefield",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 178,
		Latitude:  33.8843002319,
		Longitude: -80.4947967529,
		Timezone:  "America/New_York",
	},
	"SC58": {
		ICAO:      "SC58",
		Name:      "Hartness Airport",
		City:      "Greenville",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 850,
		Latitude:  34.8470993042,
		Longitude: -82.2518005371,
		Timezone:  "America/New_York",
	},
	"SC60": {
		ICAO:      "SC60",
		Name:      "Cockfield Aerodrome",
		City:      "Scranton",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 82,
		Latitude:  33.9040985107,
		Longitude: -79.6839981079,
		Timezone:  "America/New_York",
	},
	"SC61": {
		ICAO:      "SC61",
		Name:      "Wild Irish Rose Airport",
		City:      "Cheraw",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 376,
		Latitude:  34.6179008484,
		Longitude: -80.1517028809,
		Timezone:  "America/New_York",
	},
	"SC65": {
		ICAO:      "SC65",
		Name:      "Raven's Run Airport",
		City:      "Mount Pleasant",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 8,
		Latitude:  32.8237991333,
		Longitude: -79.8067016602,
		Timezone:  "America/New_York",
	},
	"SC67": {
		ICAO:      "SC67",
		Name:      "Country Squire Airport",
		City:      "Rock Hill",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 587,
		Latitude:  34.9099006653,
		Longitude: -81.074798584,
		Timezone:  "America/New_York",
	},
	"SC69": {
		ICAO:      "SC69",
		Name:      "Shiloh Plantation Airport",
		City:      "Ehrhardt",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 131,
		Latitude:  33.1206016541,
		Longitude: -80.9507980347,
		Timezone:  "America/New_York",
	},
	"SC70": {
		ICAO:      "SC70",
		Name:      "Anna's Airport",
		City:      "Walhalla",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 930,
		Latitude:  34.7249984741,
		Longitude: -83.0849990845,
		Timezone:  "America/New_York",
	},
	"SC72": {
		ICAO:      "SC72",
		Name:      "Chandelle Airport",
		City:      "Greer",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 850,
		Latitude:  34.8228988647,
		Longitude: -82.1482009888,
		Timezone:  "America/New_York",
	},
	"SC74": {
		ICAO:      "SC74",
		Name:      "Marsh Point Airport",
		City:      "Beaufort",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 12,
		Latitude:  32.4618988037,
		Longitude: -80.6008987427,
		Timezone:  "America/New_York",
	},
	"SC75": {
		ICAO:      "SC75",
		Name:      "Oolenoy Valley Airport",
		City:      "Pickens",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 1000,
		Latitude:  34.9981994629,
		Longitude: -82.7349014282,
		Timezone:  "America/New_York",
	},
	"SC76": {
		ICAO:      "SC76",
		Name:      "Unity Aerodrome",
		City:      "Lancaster",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 640,
		Latitude:  34.8027000427,
		Longitude: -80.6800994873,
		Timezone:  "America/New_York",
	},
	"SC78": {
		ICAO:      "SC78",
		Name:      "King Field",
		City:      "Cleveland",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 1250,
		Latitude:  35.0906982422,
		Longitude: -82.4953994751,
		Timezone:  "America/New_York",
	},
	"SC79": {
		ICAO:      "SC79",
		Name:      "Bermuda High Gliderport",
		City:      "Lancaster",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 550,
		Latitude:  34.6113014221,
		Longitude: -80.449798584,
		Timezone:  "America/New_York",
	},
	"SC80": {
		ICAO:      "SC80",
		Name:      "Anderson Airport",
		City:      "Pendleton",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 817,
		Latitude:  34.6678009033,
		Longitude: -82.7099990845,
		Timezone:  "America/New_York",
	},
	"SC81": {
		ICAO:      "SC81",
		Name:      "Abbeville Airport",
		City:      "Abbeville",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 550,
		Latitude:  34.1515007019,
		Longitude: -82.3536987305,
		Timezone:  "America/New_York",
	},
	"SC82": {
		ICAO:      "SC82",
		Name:      "Oakhill Airpark",
		City:      "Williamson",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 837,
		Latitude:  34.5847015381,
		Longitude: -82.3516998291,
		Timezone:  "America/New_York",
	},
	"SC86": {
		ICAO:      "SC86",
		Name:      "Williamsport Airpark",
		City:      "Easley",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 1100,
		Latitude:  34.8646011353,
		Longitude: -82.5675964355,
		Timezone:  "America/New_York",
	},
	"SC87": {
		ICAO:      "SC87",
		Name:      "Avinger Field",
		City:      "Vance",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 138,
		Latitude:  33.4427986145,
		Longitude: -80.4555969238,
		Timezone:  "America/New_York",
	},
	"SC88": {
		ICAO:      "SC88",
		Name:      "Davis Field",
		City:      "Chesnee",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 935,
		Latitude:  35.1473007202,
		Longitude: -81.7942962646,
		Timezone:  "America/New_York",
	},
	"SC89": {
		ICAO:      "SC89",
		Name:      "Price Airport",
		City:      "Dillon",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 138,
		Latitude:  34.4749984741,
		Longitude: -79.3936004639,
		Timezone:  "America/New_York",
	},
	"SC90": {
		ICAO:      "SC90",
		Name:      "Do-Little Field",
		City:      "St. Matthews",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 369,
		Latitude:  33.6761016846,
		Longitude: -80.9483032227,
		Timezone:  "America/New_York",
	},
	"SC91": {
		ICAO:      "SC91",
		Name:      "Bell's Branch Airport",
		City:      "Branchville",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 95,
		Latitude:  33.3064994812,
		Longitude: -80.848197937,
		Timezone:  "America/New_York",
	},
	"SC92": {
		ICAO:      "SC92",
		Name:      "Rambos Field",
		City:      "Abbeville",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 550,
		Latitude:  34.1031990051,
		Longitude: -82.2817993164,
		Timezone:  "America/New_York",
	},
	"SC93": {
		ICAO:      "SC93",
		Name:      "Paul's Plantation Airport",
		City:      "Darlington",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 180,
		Latitude:  34.3660011292,
		Longitude: -79.8242034912,
		Timezone:  "America/New_York",
	},
	"SC94": {
		ICAO:      "SC94",
		Name:      "Weaver Field",
		City:      "Johnsonville",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 75,
		Latitude:  33.8894004822,
		Longitude: -79.506401062,
		Timezone:  "America/New_York",
	},
	"SC95": {
		ICAO:      "SC95",
		Name:      "Perry International Airport",
		City:      "Perry",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 438,
		Latitude:  33.6292991638,
		Longitude: -81.3296966553,
		Timezone:  "America/New_York",
	},
	"SC96": {
		ICAO:      "SC96",
		Name:      "Tokeena Air Park",
		City:      "Seneca",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 800,
		Latitude:  34.5602989197,
		Longitude: -82.9353027344,
		Timezone:  "America/New_York",
	},
	"SC97": {
		ICAO:      "SC97",
		Name:      "Riverbend Airpark",
		City:      "Mauldin",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 920,
		Latitude:  34.7266998291,
		Longitude: -82.3396987915,
		Timezone:  "America/New_York",
	},
	"SC98": {
		ICAO:      "SC98",
		Name:      "Mount Holly Airport",
		City:      "Goose Creek",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 55,
		Latitude:  33.0542984009,
		Longitude: -80.0833969116,
		Timezone:  "America/New_York",
	},
	"SC99": {
		ICAO:      "SC99",
		Name:      "Whiteplains Airport",
		City:      "Lexington",
		State:     "South-Carolina",
		Country:   "US",
		Elevation: 520,
		Latitude:  33.9639015198,
		Longitude: -81.3591995239,
		Timezone:  "America/New_York",
	},
	"SCAA": {
		ICAO:      "SCAA",
		Name:      "Anorada Airport",
		City:      "Frutillar",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 459,
		Latitude:  -41.0427780151,
		Longitude: -73.0475006104,
		Timezone:  "America/Santiago",
	},
	"SCAB": {
		ICAO:      "SCAB",
		Name:      "El Alba Airport",
		City:      "La Lumbrera",
		State:     "Santiago-Metropolitan",
		Country:   "CL",
		Elevation: 450,
		Latitude:  -33.6588897705,
		Longitude: -71.2891693115,
		Timezone:  "America/Santiago",
	},
	"SCAC": {
		ICAO:      "SCAC",
		IATA:      "ZUD",
		Name:      "Pupelde Airport",
		City:      "Ancud",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 315,
		Latitude:  -41.9039001465,
		Longitude: -73.7966995239,
		Timezone:  "America/Santiago",
	},
	"SCAD": {
		ICAO:      "SCAD",
		Name:      "Santa Adriana Airport",
		City:      "Ovalle",
		State:     "Coquimbo",
		Country:   "CL",
		Elevation: 558,
		Latitude:  -30.628610611,
		Longitude: -71.2730560303,
		Timezone:  "America/Santiago",
	},
	"SCAE": {
		ICAO:      "SCAE",
		Name:      "El Buitre Airport",
		City:      "Arica",
		State:     "Arica-y-Parinacota",
		Country:   "CL",
		Elevation: 328,
		Latitude:  -18.5100002289,
		Longitude: -70.2891998291,
		Timezone:  "America/Santiago",
	},
	"SCAF": {
		ICAO:      "SCAF",
		Name:      "San Alfonso Airport",
		City:      "Alhue",
		State:     "Santiago-Metropolitan",
		Country:   "CL",
		Elevation: 407,
		Latitude:  -34.0555992126,
		Longitude: -71.2369003296,
		Timezone:  "America/Santiago",
	},
	"SCAG": {
		ICAO:      "SCAG",
		Name:      "Agua Santa Airport",
		City:      "Palmilla",
		State:     "O'Higgins",
		Country:   "CL",
		Elevation: 476,
		Latitude:  -34.5514755847,
		Longitude: -71.3358914852,
		Timezone:  "America/Santiago",
	},
	"SCAH": {
		ICAO:      "SCAH",
		Name:      "Tolquien Airport",
		City:      "Achao",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 551,
		Latitude:  -42.4494018555,
		Longitude: -73.5246963501,
		Timezone:  "America/Santiago",
	},
	"SCAJ": {
		ICAO:      "SCAJ",
		Name:      "Las Alpacas Airport",
		City:      "Retiro",
		State:     "Maule",
		Country:   "CL",
		Elevation: 550,
		Latitude:  -36.1452789307,
		Longitude: -71.763053894,
		Timezone:  "America/Santiago",
	},
	"SCAK": {
		ICAO:      "SCAK",
		Name:      "Atacalco Airport",
		City:      "Recinto",
		State:     "Biobio",
		Country:   "CL",
		Elevation: 2362,
		Latitude:  -36.9194450378,
		Longitude: -71.5774993896,
		Timezone:  "America/Santiago",
	},
	"SCAL": {
		ICAO:      "SCAL",
		Name:      "Valchac Airport",
		City:      "Cochrane",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 1116,
		Latitude:  -47.115556,
		Longitude: -72.480278,
		Timezone:  "America/Santiago",
	},
	"SCAM": {
		ICAO:      "SCAM",
		Name:      "Alempue Airport",
		City:      "Teno",
		State:     "Maule",
		Country:   "CL",
		Elevation: 1083,
		Latitude:  -34.8650016785,
		Longitude: -71.1019439697,
		Timezone:  "America/Santiago",
	},
	"SCAN": {
		ICAO:      "SCAN",
		IATA:      "LOB",
		Name:      "San Rafael Airport",
		City:      "Los Andes",
		State:     "Valparaiso",
		Country:   "CL",
		Elevation: 2461,
		Latitude:  -32.814201355,
		Longitude: -70.646697998,
		Timezone:  "America/Santiago",
	},
	"SCAO": {
		ICAO:      "SCAO",
		Name:      "Palo Alto Airport",
		City:      "Lolol",
		State:     "O'Higgins",
		Country:   "CL",
		Elevation: 279,
		Latitude:  -34.7330551147,
		Longitude: -71.7408370972,
		Timezone:  "America/Santiago",
	},
	"SCAP": {
		ICAO:      "SCAP",
		IATA:      "WAP",
		Name:      "Alto Palena Airport",
		City:      "Alto Palena",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 794,
		Latitude:  -43.6119003296,
		Longitude: -71.806098938,
		Timezone:  "America/Santiago",
	},
	"SCAQ": {
		ICAO:      "SCAQ",
		Name:      "Arquilhue Airport",
		City:      "Lago Ranco",
		State:     "Los-Rios",
		Country:   "CL",
		Elevation: 590,
		Latitude:  -40.198299408,
		Longitude: -72.0291976929,
		Timezone:  "America/Santiago",
	},
	"SCAR": {
		ICAO:      "SCAR",
		IATA:      "ARI",
		Name:      "Chacalluta Airport",
		City:      "Arica",
		State:     "Arica-y-Parinacota",
		Country:   "CL",
		Elevation: 167,
		Latitude:  -18.3484992981,
		Longitude: -70.3386993408,
		Timezone:  "America/Lima",
	},
	"SCAS": {
		ICAO:      "SCAS",
		IATA:      "WPA",
		Name:      "Cabo 1° Juan Roman Airport",
		City:      "Puerto Aysen",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 30,
		Latitude:  -45.3992004395,
		Longitude: -72.6703033447,
		Timezone:  "America/Santiago",
	},
	"SCAT": {
		ICAO:      "SCAT",
		IATA:      "CPO",
		Name:      "Desierto de Atacama Airport",
		City:      "Copiapo",
		State:     "Atacama",
		Country:   "CL",
		Elevation: 670,
		Latitude:  -27.2611999512,
		Longitude: -70.7791976929,
		Timezone:  "America/Santiago",
	},
	"SCAU": {
		ICAO:      "SCAU",
		Name:      "Juan Enrique Airport",
		City:      "Paine",
		State:     "Santiago-Metropolitan",
		Country:   "CL",
		Elevation: 1263,
		Latitude:  -33.8936004639,
		Longitude: -70.885597229,
		Timezone:  "America/Santiago",
	},
	"SCAV": {
		ICAO:      "SCAV",
		Name:      "La Vertiente Airport",
		City:      "Chillan",
		State:     "Biobio",
		Country:   "CL",
		Elevation: 459,
		Latitude:  -36.6102790833,
		Longitude: -72.0383300781,
		Timezone:  "America/Santiago",
	},
	"SCAY": {
		ICAO:      "SCAY",
		Name:      "Ayacara Airport",
		City:      "Ayacara",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 80,
		Latitude:  -42.3108329773,
		Longitude: -72.7902755737,
		Timezone:  "America/Santiago",
	},
	"SCAZ": {
		ICAO:      "SCAZ",
		Name:      "Azopardo Airport",
		City:      "Timaukel",
		State:     "Magallanes",
		Country:   "CL",
		Elevation: 59,
		Latitude:  -54.4797210693,
		Longitude: -68.936668396,
		Timezone:  "America/Punta_Arenas",
	},
	"SCBA": {
		ICAO:      "SCBA",
		IATA:      "BBA",
		Name:      "Balmaceda Airport",
		City:      "Balmaceda",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 1722,
		Latitude:  -45.9160995483,
		Longitude: -71.6894989014,
		Timezone:  "America/Santiago",
	},
	"SCBB": {
		ICAO:      "SCBB",
		Name:      "Del Bio Bio Airport",
		City:      "Negrete",
		State:     "Biobio",
		Country:   "CL",
		Elevation: 249,
		Latitude:  -37.587222,
		Longitude: -72.503611,
		Timezone:  "America/Santiago",
	},
	"SCBC": {
		ICAO:      "SCBC",
		Name:      "Cacique Blanco Airport",
		City:      "Lago Verde",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 750,
		Latitude:  -44.2311096191,
		Longitude: -71.8641662598,
		Timezone:  "America/Santiago",
	},
	"SCBD": {
		ICAO:      "SCBD",
		Name:      "El Boldal Airport",
		City:      "Santa Cruz",
		State:     "O'Higgins",
		Country:   "CL",
		Elevation: 574,
		Latitude:  -34.6844444275,
		Longitude: -71.2061080933,
		Timezone:  "America/Santiago",
	},
	"SCBE": {
		ICAO:      "SCBE",
		IATA:      "TOQ",
		Name:      "Barriles Airport",
		City:      "Tocopilla",
		State:     "Antofagasta",
		Country:   "CL",
		Elevation: 3475,
		Latitude:  -22.1410999298,
		Longitude: -70.0628967285,
		Timezone:  "America/Santiago",
	},
	"SCBI": {
		ICAO:      "SCBI",
		Name:      "Pampa Guanaco Airport",
		City:      "Bahia Inutil",
		State:     "Magallanes",
		Country:   "CL",
		Elevation: 591,
		Latitude:  -54.0449981689,
		Longitude: -68.8316650391,
		Timezone:  "America/Punta_Arenas",
	},
	"SCBN": {
		ICAO:      "SCBN",
		Name:      "Cotreumo Airport",
		City:      "Rio Bueno",
		State:     "Los-Rios",
		Country:   "CL",
		Elevation: 607,
		Latitude:  -40.4006004333,
		Longitude: -72.6613998413,
		Timezone:  "America/Santiago",
	},
	"SCBO": {
		ICAO:      "SCBO",
		Name:      "General Bernardo O'Higgins Base Skyway",
		City:      "General Bernardo O'Higgins Base",
		Country:   "AQ",
		Elevation: 900,
		Latitude:  -63.344,
		Longitude: -57.842,
		Timezone:  "America/Punta_Arenas",
	},
	"SCBQ": {
		ICAO:      "SCBQ",
		Name:      "El Bosque Airport",
		City:      "Santiago",
		State:     "Santiago-Metropolitan",
		Country:   "CL",
		Elevation: 1844,
		Latitude:  -33.5617980957,
		Longitude: -70.6884002686,
		Timezone:  "America/Santiago",
	},
	"SCBR": {
		ICAO:      "SCBR",
		Name:      "Lago Brown Airport",
		City:      "Lago Brown",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 540,
		Latitude:  -47.3919448853,
		Longitude: -72.3174972534,
		Timezone:  "America/Santiago",
	},
	"SCBS": {
		ICAO:      "SCBS",
		Name:      "Posesion Airport",
		City:      "Bahia Posesion",
		State:     "Magallanes",
		Country:   "CL",
		Elevation: 100,
		Latitude:  -52.2999992371,
		Longitude: -68.9333343506,
		Timezone:  "America/Punta_Arenas",
	},
	"SCBT": {
		ICAO:      "SCBT",
		Name:      "Rincon Bonito Airport",
		City:      "Cochamo",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 705,
		Latitude:  -42.0288887024,
		Longitude: -72.0961074829,
		Timezone:  "America/Santiago",
	},
	"SCBU": {
		ICAO:      "SCBU",
		Name:      "El Baul Airstrip",
		City:      "Molina",
		State:     "Maule",
		Country:   "CL",
		Elevation: 3070,
		Latitude:  -35.341435,
		Longitude: -70.9108,
		Timezone:  "America/Santiago",
	},
	"SCBV": {
		ICAO:      "SCBV",
		Name:      "Bellavista Airport",
		City:      "Rio Claro",
		State:     "Maule",
		Country:   "CL",
		Elevation: 885,
		Latitude:  -35.1897239685,
		Longitude: -71.2966690063,
		Timezone:  "America/Santiago",
	},
	"SCCA": {
		ICAO:      "SCCA",
		Name:      "El Boldo Airport",
		City:      "Cauquenes",
		State:     "Maule",
		Country:   "CL",
		Elevation: 525,
		Latitude:  -35.9713897705,
		Longitude: -72.2249984741,
		Timezone:  "America/Santiago",
	},
	"SCCB": {
		ICAO:      "SCCB",
		Name:      "Pedro Villarroel C. Airport",
		City:      "Combarbala",
		State:     "Coquimbo",
		Country:   "CL",
		Elevation: 3002,
		Latitude:  -31.221944809,
		Longitude: -71.070274353,
		Timezone:  "America/Santiago",
	},
	"SCCC": {
		ICAO:      "SCCC",
		IATA:      "CCH",
		Name:      "Chile Chico Airport",
		City:      "Chile Chico",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 1070,
		Latitude:  -46.5833015442,
		Longitude: -71.6874008179,
		Timezone:  "America/Santiago",
	},
	"SCCE": {
		ICAO:      "SCCE",
		Name:      "El Arenal Airport",
		City:      "Cauquenes",
		State:     "Maule",
		Country:   "CL",
		Elevation: 397,
		Latitude:  -35.896389,
		Longitude: -72.055833,
		Timezone:  "America/Santiago",
	},
	"SCCF": {
		ICAO:      "SCCF",
		IATA:      "CJC",
		Name:      "El Loa Airport",
		City:      "Calama",
		State:     "Antofagasta",
		Country:   "CL",
		Elevation: 7543,
		Latitude:  -22.4981994629,
		Longitude: -68.9036026001,
		Timezone:  "America/Santiago",
	},
	"SCCG": {
		ICAO:      "SCCG",
		Name:      "La Pelicana Airport",
		City:      "Combarbala",
		State:     "Coquimbo",
		Country:   "CL",
		Elevation: 2790,
		Latitude:  -31.0638885498,
		Longitude: -70.9544448853,
		Timezone:  "America/Santiago",
	},
	"SCCH": {
		ICAO:      "SCCH",
		IATA:      "YAI",
		Name:      "Gral. Bernardo O´Higgins Airport",
		City:      "Chillan",
		State:     "Biobio",
		Country:   "CL",
		Elevation: 495,
		Latitude:  -36.5825004578,
		Longitude: -72.0314025879,
		Timezone:  "America/Santiago",
	},
	"SCCI": {
		ICAO:      "SCCI",
		IATA:      "PUQ",
		Name:      "Pdte. Carlos Ibanez del Campo Airport",
		City:      "Punta Arenas",
		State:     "Magallanes",
		Country:   "CL",
		Elevation: 139,
		Latitude:  -53.0026016235,
		Longitude: -70.854598999,
		Timezone:  "America/Punta_Arenas",
	},
	"SCCK": {
		ICAO:      "SCCK",
		Name:      "Contao Airport",
		City:      "Contao",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 10,
		Latitude:  -41.8003044128,
		Longitude: -72.7222366333,
		Timezone:  "America/Santiago",
	},
	"SCCL": {
		ICAO:      "SCCL",
		Name:      "Caldera Airport",
		City:      "Caldera",
		State:     "Atacama",
		Country:   "CL",
		Elevation: 180,
		Latitude:  -27.0781002045,
		Longitude: -70.7953033447,
		Timezone:  "America/Santiago",
	},
	"SCCM": {
		ICAO:      "SCCM",
		Name:      "Molco Airport",
		City:      "Choshuenco",
		State:     "Los-Rios",
		Country:   "CL",
		Elevation: 479,
		Latitude:  -39.8250007629,
		Longitude: -72.0824966431,
		Timezone:  "America/Santiago",
	},
	"SCCN": {
		ICAO:      "SCCN",
		Name:      "Alto Cauquenes Airport",
		City:      "Cauquenes",
		State:     "Maule",
		Country:   "CL",
		Elevation: 568,
		Latitude:  -35.880279541,
		Longitude: -72.3272247314,
		Timezone:  "America/Santiago",
	},
	"SCCP": {
		ICAO:      "SCCP",
		Name:      "Callipulli Airport",
		City:      "Osorno",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 377,
		Latitude:  -40.6202774048,
		Longitude: -72.8463897705,
		Timezone:  "America/Santiago",
	},
	"SCCQ": {
		ICAO:      "SCCQ",
		IATA:      "COW",
		Name:      "Tambillos Airport",
		City:      "Coquimbo",
		State:     "Coquimbo",
		Country:   "CL",
		Elevation: 656,
		Latitude:  -30.198889,
		Longitude: -71.246944,
		Timezone:  "America/Santiago",
	},
	"SCCR": {
		ICAO:      "SCCR",
		Name:      "Enrique Mayer Soto Airport",
		City:      "Caleta Tortel",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 4,
		Latitude:  -47.787776947,
		Longitude: -73.5202789307,
		Timezone:  "America/Santiago",
	},
	"SCCS": {
		ICAO:      "SCCS",
		Name:      "Santa Rita Airport",
		City:      "Casablanca",
		State:     "Valparaiso",
		Country:   "CL",
		Elevation: 738,
		Latitude:  -33.2775001526,
		Longitude: -71.4563903809,
		Timezone:  "America/Santiago",
	},
	"SCCT": {
		ICAO:      "SCCT",
		Name:      "Quivolgo Airport",
		City:      "Constitucion",
		State:     "Maule",
		Country:   "CL",
		Elevation: 36,
		Latitude:  -35.3086013794,
		Longitude: -72.3927993774,
		Timezone:  "America/Santiago",
	},
	"SCCU": {
		ICAO:      "SCCU",
		Name:      "Lolco Airport",
		City:      "Lonquimay",
		State:     "Araucania",
		Country:   "CL",
		Elevation: 2132,
		Latitude:  -38.1580543518,
		Longitude: -71.4238891602,
		Timezone:  "America/Santiago",
	},
	"SCCV": {
		ICAO:      "SCCV",
		Name:      "Curacavi Airport",
		City:      "Curacavi",
		State:     "Santiago-Metropolitan",
		Country:   "CL",
		Elevation: 686,
		Latitude:  -33.4127998352,
		Longitude: -71.1656036377,
		Timezone:  "America/Santiago",
	},
	"SCCY": {
		ICAO:      "SCCY",
		IATA:      "GXQ",
		Name:      "Teniente Vidal Airport",
		City:      "Coyhaique",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 1020,
		Latitude:  -45.5942001343,
		Longitude: -72.1061019897,
		Timezone:  "America/Santiago",
	},
	"SCDA": {
		ICAO:      "SCDA",
		IATA:      "IQQ",
		Name:      "Diego Aracena Airport",
		City:      "Iquique",
		State:     "Tarapaca",
		Country:   "CL",
		Elevation: 155,
		Latitude:  -20.535200119,
		Longitude: -70.1812973022,
		Timezone:  "America/Santiago",
	},
	"SCDD": {
		ICAO:      "SCDD",
		Name:      "Don Dobri Airport",
		City:      "Puerto Varas",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 226,
		Latitude:  -41.2347221375,
		Longitude: -72.5149993896,
		Timezone:  "America/Santiago",
	},
	"SCDH": {
		ICAO:      "SCDH",
		Name:      "Vodudahue Airport",
		City:      "Chaiten",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 98,
		Latitude:  -42.4888877869,
		Longitude: -72.3505554199,
		Timezone:  "America/Santiago",
	},
	"SCDI": {
		ICAO:      "SCDI",
		Name:      "Pichidangui Airport",
		City:      "Pichidangui",
		State:     "Coquimbo",
		Country:   "CL",
		Elevation: 98,
		Latitude:  -32.1430549622,
		Longitude: -71.505279541,
		Timezone:  "America/Santiago",
	},
	"SCDL": {
		ICAO:      "SCDL",
		Name:      "El Algarrobo Airport",
		City:      "Cabildo",
		State:     "Valparaiso",
		Country:   "CL",
		Elevation: 951,
		Latitude:  -32.4719429016,
		Longitude: -70.993057251,
		Timezone:  "America/Santiago",
	},
	"SCDM": {
		ICAO:      "SCDM",
		Name:      "San Damian Airport",
		City:      "Duao",
		State:     "Maule",
		Country:   "CL",
		Elevation: 220,
		Latitude:  -34.8780555725,
		Longitude: -72.1455535889,
		Timezone:  "America/Santiago",
	},
	"SCDQ": {
		ICAO:      "SCDQ",
		Name:      "San Lorenzo Airport",
		City:      "Duqueco",
		State:     "Biobio",
		Country:   "CL",
		Elevation: 1902,
		Latitude:  -37.5233345032,
		Longitude: -71.7261123657,
		Timezone:  "America/Santiago",
	},
	"SCDS": {
		ICAO:      "SCDS",
		Name:      "San Andres Airport",
		City:      "Retiro",
		State:     "Maule",
		Country:   "CL",
		Elevation: 574,
		Latitude:  -36.0055541992,
		Longitude: -71.7666702271,
		Timezone:  "America/Santiago",
	},
	"SCDW": {
		ICAO:      "SCDW",
		Name:      "Almirante Schroeders Airport",
		City:      "Isla Dawson",
		State:     "Magallanes",
		Country:   "CL",
		Elevation: 160,
		Latitude:  -53.6106987,
		Longitude: -70.4704971313,
		Timezone:  "America/Punta_Arenas",
	},
	"SCEA": {
		ICAO:      "SCEA",
		Name:      "El Amarillo Airport",
		City:      "Chaiten",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 1083,
		Latitude:  -43.006111145,
		Longitude: -72.4783325195,
		Timezone:  "America/Santiago",
	},
	"SCEB": {
		ICAO:      "SCEB",
		Name:      "Entrada Baker Airport",
		City:      "Entrada Baker",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 1640,
		Latitude:  -47.1841659546,
		Longitude: -71.9761123657,
		Timezone:  "America/Santiago",
	},
	"SCEC": {
		ICAO:      "SCEC",
		Name:      "Pelicano Airport",
		City:      "Estacion Chanaral",
		State:     "Coquimbo",
		Country:   "CL",
		Elevation: 3678,
		Latitude:  -29.1448993683,
		Longitude: -70.887802124,
		Timezone:  "America/Santiago",
	},
	"SCED": {
		ICAO:      "SCED",
		Name:      "Los Cedros Airport",
		City:      "Sagrada Familia",
		State:     "Maule",
		Country:   "CL",
		Elevation: 305,
		Latitude:  -35.1549987793,
		Longitude: -71.5852813721,
		Timezone:  "America/Santiago",
	},
	"SCEG": {
		ICAO:      "SCEG",
		Name:      "El Corte Airport",
		City:      "Talagante",
		State:     "Santiago-Metropolitan",
		Country:   "CL",
		Elevation: 1024,
		Latitude:  -33.7099990845,
		Longitude: -70.9130554199,
		Timezone:  "America/Santiago",
	},
	"SCEH": {
		ICAO:      "SCEH",
		Name:      "El Huachi Airport",
		City:      "Santa Barbara",
		State:     "Biobio",
		Country:   "CL",
		Elevation: 1558,
		Latitude:  -37.6472206116,
		Longitude: -71.7538909912,
		Timezone:  "America/Santiago",
	},
	"SCEK": {
		ICAO:      "SCEK",
		Name:      "Chepica Airport",
		City:      "Chepica",
		State:     "O'Higgins",
		Country:   "CL",
		Elevation: 610,
		Latitude:  -34.7397232056,
		Longitude: -71.32472229,
		Timezone:  "America/Santiago",
	},
	"SCEL": {
		ICAO:      "SCEL",
		IATA:      "SCL",
		Name:      "Comodoro Arturo Merino Benitez International Airport",
		City:      "Santiago",
		State:     "Santiago-Metropolitan",
		Country:   "CL",
		Elevation: 1555,
		Latitude:  -33.3930015564,
		Longitude: -70.7857971191,
		Timezone:  "America/Santiago",
	},
	"SCEO": {
		ICAO:      "SCEO",
		Name:      "El Salto Airport",
		City:      "Parral",
		State:     "Maule",
		Country:   "CL",
		Elevation: 564,
		Latitude:  -36.130279541,
		Longitude: -71.8574981689,
		Timezone:  "America/Santiago",
	},
	"SCEP": {
		ICAO:      "SCEP",
		Name:      "El Principal Airport",
		City:      "Pirque",
		State:     "Santiago-Metropolitan",
		Country:   "CL",
		Elevation: 2592,
		Latitude:  -33.7249984741,
		Longitude: -70.5102996826,
		Timezone:  "America/Santiago",
	},
	"SCER": {
		ICAO:      "SCER",
		Name:      "Quintero Airport",
		City:      "Quintero",
		State:     "Valparaiso",
		Country:   "CL",
		Elevation: 12,
		Latitude:  -32.7901992798,
		Longitude: -71.521697998,
		Timezone:  "America/Santiago",
	},
	"SCES": {
		ICAO:      "SCES",
		IATA:      "ESR",
		Name:      "Ricardo Garcia Posada Airport",
		City:      "El Salvador",
		State:     "Atacama",
		Country:   "CL",
		Elevation: 5240,
		Latitude:  -26.3111000061,
		Longitude: -69.7651977539,
		Timezone:  "America/Santiago",
	},
	"SCET": {
		ICAO:      "SCET",
		Name:      "El Tambo Airport",
		City:      "San Vicente De Tagua Tagua",
		State:     "O'Higgins",
		Country:   "CL",
		Elevation: 748,
		Latitude:  -34.4755554199,
		Longitude: -71.0141677856,
		Timezone:  "America/Santiago",
	},
	"SCEV": {
		ICAO:      "SCEV",
		IATA:      "FRT",
		Name:      "El Avellano Airport",
		City:      "Frutillar",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 489,
		Latitude:  -41.1170005798,
		Longitude: -73.0500030518,
		Timezone:  "America/Santiago",
	},
	"SCEX": {
		ICAO:      "SCEX",
		Name:      "Rio Exploradores Airport",
		City:      "Aysen",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 59,
		Latitude:  -46.353611,
		Longitude: -73.327778,
		Timezone:  "America/Santiago",
	},
	"SCEY": {
		ICAO:      "SCEY",
		Name:      "Entrada Mayer Airport",
		City:      "Entrada Mayer",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 1550,
		Latitude:  -48.2014517431,
		Longitude: -72.3275899887,
		Timezone:  "America/Argentina/Rio_Gallegos",
	},
	"SCFA": {
		ICAO:      "SCFA",
		IATA:      "ANF",
		Name:      "Cerro Moreno Airport",
		City:      "Antofagasta",
		State:     "Antofagasta",
		Country:   "CL",
		Elevation: 455,
		Latitude:  -23.4444999695,
		Longitude: -70.445098877,
		Timezone:  "America/Santiago",
	},
	"SCFC": {
		ICAO:      "SCFC",
		Name:      "Fachinal Airport",
		City:      "Fachinal",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 693,
		Latitude:  -46.551109314,
		Longitude: -72.2180557251,
		Timezone:  "America/Santiago",
	},
	"SCFF": {
		ICAO:      "SCFF",
		Name:      "Freirina Airport",
		City:      "Freirina",
		State:     "Atacama",
		Country:   "CL",
		Elevation: 738,
		Latitude:  -28.5263881683,
		Longitude: -71.0619430542,
		Timezone:  "America/Santiago",
	},
	"SCFI": {
		ICAO:      "SCFI",
		IATA:      "FRT",
		Name:      "Fundo Tehuen Airport",
		City:      "Frutillar",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 377,
		Latitude:  -41.0950012207,
		Longitude: -73.126663208,
		Timezone:  "America/Santiago",
	},
	"SCFJ": {
		ICAO:      "SCFJ",
		Name:      "Fray Jorge Airport",
		City:      "Ovalle",
		State:     "Coquimbo",
		Country:   "CL",
		Elevation: 492,
		Latitude:  -30.6800003052,
		Longitude: -71.5786132813,
		Timezone:  "America/Santiago",
	},
	"SCFK": {
		ICAO:      "SCFK",
		Name:      "Fundo El Carmen Airport",
		City:      "Chillan",
		State:     "Biobio",
		Country:   "CL",
		Elevation: 557,
		Latitude:  -36.6413879395,
		Longitude: -72.0166702271,
		Timezone:  "America/Santiago",
	},
	"SCFL": {
		ICAO:      "SCFL",
		Name:      "Fundo Loma Larga Airport",
		City:      "Casablanca",
		State:     "Valparaiso",
		Country:   "CL",
		Elevation: 869,
		Latitude:  -33.2799987793,
		Longitude: -71.3792037964,
		Timezone:  "America/Santiago",
	},
	"SCFM": {
		ICAO:      "SCFM",
		IATA:      "WPR",
		Name:      "Capitan Fuentes Martinez Airport Airport",
		City:      "Porvenir",
		State:     "Magallanes",
		Country:   "CL",
		Elevation: 104,
		Latitude:  -53.2537002563,
		Longitude: -70.3191986084,
		Timezone:  "America/Punta_Arenas",
	},
	"SCFN": {
		ICAO:      "SCFN",
		Name:      "Russfin Airport",
		City:      "Russfin",
		State:     "Magallanes",
		Country:   "CL",
		Elevation: 600,
		Latitude:  -53.7999992371,
		Longitude: -69.266998291,
		Timezone:  "America/Punta_Arenas",
	},
	"SCFO": {
		ICAO:      "SCFO",
		Name:      "La Reforma Airport",
		City:      "Pelarco",
		State:     "Maule",
		Country:   "CL",
		Elevation: 853,
		Latitude:  -35.4580574036,
		Longitude: -71.2811126709,
		Timezone:  "America/Santiago",
	},
	"SCFR": {
		ICAO:      "SCFR",
		Name:      "Frutillar Airport",
		City:      "Frutillar",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 469,
		Latitude:  -41.1308326721,
		Longitude: -73.0647201538,
		Timezone:  "America/Santiago",
	},
	"SCFS": {
		ICAO:      "SCFS",
		Name:      "Los Calafates Airport",
		City:      "Chonchi",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 82,
		Latitude:  -42.6788902283,
		Longitude: -73.8966674805,
		Timezone:  "America/Santiago",
	},
	"SCFT": {
		ICAO:      "SCFT",
		IATA:      "FFU",
		Name:      "Futaleufu Airport",
		City:      "Futaleufu",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 1148,
		Latitude:  -43.189201355,
		Longitude: -71.8510971069,
		Timezone:  "America/Santiago",
	},
	"SCFU": {
		ICAO:      "SCFU",
		Name:      "Loncopan Airport",
		City:      "Futrono",
		State:     "Los-Rios",
		Country:   "CL",
		Elevation: 886,
		Latitude:  -40.0663986206,
		Longitude: -72.5378036499,
		Timezone:  "America/Santiago",
	},
	"SCFX": {
		ICAO:      "SCFX",
		Name:      "Isla San Felix Airport",
		City:      "Isla San Felix",
		State:     "Atacama",
		Country:   "CL",
		Elevation: 165,
		Latitude:  -26.2938995361,
		Longitude: -80.0961990356,
		Timezone:  "America/Santiago",
	},
	"SCGA": {
		ICAO:      "SCGA",
		Name:      "Punta Galera Airport",
		City:      "La Union",
		State:     "Los-Rios",
		Country:   "CL",
		Elevation: 118,
		Latitude:  -40.0018997192,
		Longitude: -73.6964035034,
		Timezone:  "America/Santiago",
	},
	"SCGB": {
		ICAO:      "SCGB",
		Name:      "Guadaba Airport",
		City:      "Los Sauces",
		State:     "Araucania",
		Country:   "CL",
		Elevation: 272,
		Latitude:  -37.9777793884,
		Longitude: -72.9294433594,
		Timezone:  "America/Santiago",
	},
	"SCGC": {
		ICAO:      "SCGC",
		Name:      "Union Glaciar Airport",
		City:      "Antartica",
		Country:   "AQ",
		Elevation: 2461,
		Latitude:  -79.777778,
		Longitude: -83.320833,
		Timezone:  "Etc/GMT+6",
	},
	"SCGE": {
		ICAO:      "SCGE",
		IATA:      "LSQ",
		Name:      "Maria Dolores Airport",
		City:      "Los Angeles",
		State:     "Biobio",
		Country:   "CL",
		Elevation: 374,
		Latitude:  -37.4016990662,
		Longitude: -72.4253997803,
		Timezone:  "America/Santiago",
	},
	"SCGF": {
		ICAO:      "SCGF",
		Name:      "Golfo Azul Airport",
		City:      "Futrono",
		State:     "Los-Rios",
		Country:   "CL",
		Elevation: 486,
		Latitude:  -40.140833,
		Longitude: -72.283611,
		Timezone:  "America/Santiago",
	},
	"SCGH": {
		ICAO:      "SCGH",
		Name:      "Cholguahue Airport",
		City:      "Los Angeles",
		State:     "Biobio",
		Country:   "CL",
		Elevation: 732,
		Latitude:  -37.4697227478,
		Longitude: -72.1386108398,
		Timezone:  "America/Santiago",
	},
	"SCGI": {
		ICAO:      "SCGI",
		Name:      "San Guillermo Airport",
		City:      "Retiro",
		State:     "Maule",
		Country:   "CL",
		Elevation: 495,
		Latitude:  -36.0011100769,
		Longitude: -71.8294448853,
		Timezone:  "America/Santiago",
	},
	"SCGL": {
		ICAO:      "SCGL",
		Name:      "Las Aguilas Airport",
		City:      "Rapel",
		State:     "O'Higgins",
		Country:   "CL",
		Elevation: 450,
		Latitude:  -34.1691665649,
		Longitude: -71.5311126709,
		Timezone:  "America/Santiago",
	},
	"SCGM": {
		ICAO:      "SCGM",
		Name:      "Los Gomeros Airport",
		City:      "Rengo",
		State:     "O'Higgins",
		Country:   "CL",
		Elevation: 1034,
		Latitude:  -34.3611106873,
		Longitude: -70.881942749,
		Timezone:  "America/Santiago",
	},
	"SCGN": {
		ICAO:      "SCGN",
		Name:      "Caleta Gonzalo Airport",
		City:      "Chaiten",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 33,
		Latitude:  -42.5636100769,
		Longitude: -72.5997238159,
		Timezone:  "America/Santiago",
	},
	"SCGO": {
		ICAO:      "SCGO",
		Name:      "Los Confines Airport",
		City:      "Angol",
		State:     "Araucania",
		Country:   "CL",
		Elevation: 240,
		Latitude:  -37.7947006226,
		Longitude: -72.6872024536,
		Timezone:  "America/Santiago",
	},
	"SCGP": {
		ICAO:      "SCGP",
		Name:      "Curaco Airport",
		City:      "Panguipulli",
		State:     "Los-Rios",
		Country:   "CL",
		Elevation: 820,
		Latitude:  -39.5306015015,
		Longitude: -72.3499984741,
		Timezone:  "America/Santiago",
	},
	"SCGS": {
		ICAO:      "SCGS",
		Name:      "Siberia Airport",
		City:      "Cholguan",
		State:     "Biobio",
		Country:   "CL",
		Elevation: 722,
		Latitude:  -37.1772232056,
		Longitude: -72.0797195435,
		Timezone:  "America/Santiago",
	},
	"SCGU": {
		ICAO:      "SCGU",
		Name:      "Aguas Blancas Airport",
		City:      "Antofagasta",
		State:     "Antofagasta",
		Country:   "CL",
		Elevation: 3346,
		Latitude:  -24.1364002228,
		Longitude: -69.8285980225,
		Timezone:  "America/Santiago",
	},
	"SCGV": {
		ICAO:      "SCGV",
		Name:      "Punta Gaviota Airport",
		City:      "Caleta Chanaral De Aceituno",
		State:     "Atacama",
		Country:   "CL",
		Elevation: 99,
		Latitude:  -29.084444,
		Longitude: -71.467222,
		Timezone:  "America/Santiago",
	},
	"SCGY": {
		ICAO:      "SCGY",
		Name:      "Los Guayes Airport",
		City:      "Cunco",
		State:     "Araucania",
		Country:   "CL",
		Elevation: 1312,
		Latitude:  -39.0530548096,
		Longitude: -71.9936141968,
		Timezone:  "America/Santiago",
	},
	"SCGZ": {
		ICAO:      "SCGZ",
		IATA:      "WPU",
		Name:      "Guardiamarina Zanartu Airport",
		City:      "Puerto Williams",
		State:     "Magallanes",
		Country:   "CL",
		Elevation: 88,
		Latitude:  -54.931098938,
		Longitude: -67.6262969971,
		Timezone:  "America/Argentina/Ushuaia",
	},
	"SCHA": {
		ICAO:      "SCHA",
		Name:      "Chamonate Airport",
		City:      "Copiapo",
		State:     "Atacama",
		Country:   "CL",
		Elevation: 984,
		Latitude:  -27.2968997955,
		Longitude: -70.4131011963,
		Timezone:  "America/Santiago",
	},
	"SCHC": {
		ICAO:      "SCHC",
		Name:      "Chicureo Airport",
		City:      "Santiago",
		State:     "Santiago-Metropolitan",
		Country:   "CL",
		Elevation: 1886,
		Latitude:  -33.2677764893,
		Longitude: -70.6472244263,
		Timezone:  "America/Santiago",
	},
	"SCHD": {
		ICAO:      "SCHD",
		Name:      "Chumilden Airport",
		City:      "Chaiten",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 66,
		Latitude:  -42.5244445801,
		Longitude: -72.816947937,
		Timezone:  "America/Santiago",
	},
	"SCHE": {
		ICAO:      "SCHE",
		Name:      "Rucamanqui Airport",
		City:      "Huepil",
		State:     "Biobio",
		Country:   "CL",
		Elevation: 1132,
		Latitude:  -37.19972229,
		Longitude: -71.8966674805,
		Timezone:  "America/Santiago",
	},
	"SCHG": {
		ICAO:      "SCHG",
		Name:      "Almahue Airport",
		City:      "Pichidegua",
		State:     "O'Higgins",
		Country:   "CL",
		Elevation: 420,
		Latitude:  -34.3986129761,
		Longitude: -71.3711090088,
		Timezone:  "America/Santiago",
	},
	"SCHH": {
		ICAO:      "SCHH",
		Name:      "Punta Baja Airport",
		City:      "Chile Chico",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 721,
		Latitude:  -46.7936096191,
		Longitude: -72.7952804565,
		Timezone:  "America/Santiago",
	},
	"SCHK": {
		ICAO:      "SCHK",
		Name:      "Hueicolla Airport",
		City:      "La Union",
		State:     "Los-Rios",
		Country:   "CL",
		Elevation: 59,
		Latitude:  -40.1585998535,
		Longitude: -73.6544036865,
		Timezone:  "America/Santiago",
	},
	"SCHL": {
		ICAO:      "SCHL",
		Name:      "Hacienda Lipangue Airport",
		City:      "Lampa",
		State:     "Santiago-Metropolitan",
		Country:   "CL",
		Elevation: 1745,
		Latitude:  -33.3352775574,
		Longitude: -70.9111099243,
		Timezone:  "America/Santiago",
	},
	"SCHM": {
		ICAO:      "SCHM",
		Name:      "Punta El Saco Airport",
		City:      "Isla Mocha",
		State:     "Biobio",
		Country:   "CL",
		Elevation: 23,
		Latitude:  -38.4099998474,
		Longitude: -73.9008331299,
		Timezone:  "America/Santiago",
	},
	"SCHN": {
		ICAO:      "SCHN",
		Name:      "Chan Chan Airport",
		City:      "Choshuenco",
		State:     "Los-Rios",
		Country:   "CL",
		Elevation: 457,
		Latitude:  -39.8657989502,
		Longitude: -72.131401062,
		Timezone:  "America/Santiago",
	},
	"SCHO": {
		ICAO:      "SCHO",
		Name:      "Punta Chungo Airport",
		City:      "Los Vilos",
		State:     "Coquimbo",
		Country:   "CL",
		Elevation: 249,
		Latitude:  -31.8827781677,
		Longitude: -71.4763870239,
		Timezone:  "America/Santiago",
	},
	"SCHP": {
		ICAO:      "SCHP",
		Name:      "Copihue Airport",
		City:      "Retiro",
		State:     "Maule",
		Country:   "CL",
		Elevation: 551,
		Latitude:  -36.0774993896,
		Longitude: -71.7797241211,
		Timezone:  "America/Santiago",
	},
	"SCHR": {
		ICAO:      "SCHR",
		IATA:      "LGR",
		Name:      "Cochrane Airport",
		City:      "Cochrane",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 643,
		Latitude:  -47.2438011169,
		Longitude: -72.5884017944,
		Timezone:  "America/Santiago",
	},
	"SCHT": {
		ICAO:      "SCHT",
		Name:      "Tic Toc Airport",
		City:      "Chaiten",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 13,
		Latitude:  -43.6130561829,
		Longitude: -72.8974990845,
		Timezone:  "America/Santiago",
	},
	"SCHU": {
		ICAO:      "SCHU",
		Name:      "Gran Canon Airport",
		City:      "Huasco",
		State:     "Atacama",
		Country:   "CL",
		Elevation: 230,
		Latitude:  -28.0988883972,
		Longitude: -71.1466674805,
		Timezone:  "America/Santiago",
	},
	"SCHW": {
		ICAO:      "SCHW",
		Name:      "Hualaihue Airport",
		City:      "Hualaihue",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 30,
		Latitude:  -42.0286102295,
		Longitude: -72.6894454956,
		Timezone:  "America/Santiago",
	},
	"SCIA": {
		ICAO:      "SCIA",
		Name:      "Isla Apiao Airport",
		City:      "Isla Apiao",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 250,
		Latitude:  -42.5997238159,
		Longitude: -73.2122192383,
		Timezone:  "America/Santiago",
	},
	"SCIB": {
		ICAO:      "SCIB",
		Name:      "Butachauques Airport",
		City:      "Isla Butachauques",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 150,
		Latitude:  -42.3083343506,
		Longitude: -73.1422195435,
		Timezone:  "America/Santiago",
	},
	"SCIC": {
		ICAO:      "SCIC",
		Name:      "General Freire Airport",
		City:      "Curico",
		State:     "Maule",
		Country:   "CL",
		Elevation: 722,
		Latitude:  -34.9666671753,
		Longitude: -71.2163925171,
		Timezone:  "America/Santiago",
	},
	"SCID": {
		ICAO:      "SCID",
		Name:      "Marco Davison Bascur Airport",
		City:      "Punta Arenas",
		State:     "Magallanes",
		Country:   "CL",
		Elevation: 30,
		Latitude:  -52.8736114502,
		Longitude: -70.75,
		Timezone:  "America/Punta_Arenas",
	},
	"SCIE": {
		ICAO:      "SCIE",
		IATA:      "CCP",
		Name:      "Carriel Sur Airport",
		City:      "Concepcion",
		State:     "Biobio",
		Country:   "CL",
		Elevation: 26,
		Latitude:  -36.7727012634,
		Longitude: -73.0631027222,
		Timezone:  "America/Santiago",
	},
	"SCIF": {
		ICAO:      "SCIF",
		Name:      "Chollinco Airport",
		City:      "Llifen",
		State:     "Los-Rios",
		Country:   "CL",
		Elevation: 330,
		Latitude:  -40.2057991028,
		Longitude: -72.2313995361,
		Timezone:  "America/Santiago",
	},
	"SCIH": {
		ICAO:      "SCIH",
		Name:      "Caleta Blanco Airport",
		City:      "Isla Las Huichas",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 115,
		Latitude:  -45.1477775574,
		Longitude: -73.5086135864,
		Timezone:  "America/Santiago",
	},
	"SCII": {
		ICAO:      "SCII",
		Name:      "Puerto Ingeniero Ibanez Airport",
		City:      "Puerto Ingeniero Ibanez",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 820,
		Latitude:  -46.2858314514,
		Longitude: -71.9477767944,
		Timezone:  "America/Santiago",
	},
	"SCIK": {
		ICAO:      "SCIK",
		Name:      "Isla Talcan Airport",
		City:      "Isla Talcan",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 45,
		Latitude:  -42.7466659546,
		Longitude: -72.9705581665,
		Timezone:  "America/Santiago",
	},
	"SCIL": {
		ICAO:      "SCIL",
		Name:      "Auco Airport",
		City:      "Illapel",
		State:     "Coquimbo",
		Country:   "CL",
		Elevation: 1420,
		Latitude:  -31.5778007507,
		Longitude: -71.1108016968,
		Timezone:  "America/Santiago",
	},
	"SCIM": {
		ICAO:      "SCIM",
		Name:      "Isla Mocha Airport",
		City:      "Isla Mocha",
		State:     "Biobio",
		Country:   "CL",
		Elevation: 52,
		Latitude:  -38.3849983215,
		Longitude: -73.8688964844,
		Timezone:  "America/Santiago",
	},
	"SCIO": {
		ICAO:      "SCIO",
		Name:      "Laguna Redonda Airport",
		City:      "Villa O'Higgins",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 1970,
		Latitude:  -48.9335784912,
		Longitude: -72.7996139526,
		Timezone:  "America/Argentina/Rio_Gallegos",
	},
	"SCIP": {
		ICAO:      "SCIP",
		IATA:      "IPC",
		Name:      "Mataveri Airport",
		City:      "Isla De Pascua",
		State:     "Valparaiso",
		Country:   "CL",
		Elevation: 227,
		Latitude:  -27.1648006439,
		Longitude: -109.42199707,
		Timezone:  "Pacific/Easter",
	},
	"SCIR": {
		ICAO:      "SCIR",
		Name:      "Robinson Crusoe Airport",
		City:      "Isla Robinson Crusoe",
		State:     "Valparaiso",
		Country:   "CL",
		Elevation: 433,
		Latitude:  -33.6650009155,
		Longitude: -78.9297027588,
		Timezone:  "America/Santiago",
	},
	"SCIS": {
		ICAO:      "SCIS",
		Name:      "Puerto Sur Airport",
		City:      "Isla Santa Maria",
		State:     "Biobio",
		Country:   "CL",
		Elevation: 20,
		Latitude:  -37.0261116028,
		Longitude: -73.506942749,
		Timezone:  "America/Santiago",
	},
	"SCIT": {
		ICAO:      "SCIT",
		Name:      "Ivan Martinez Airport",
		City:      "Isla Tierra Del Fuego",
		State:     "Magallanes",
		Country:   "CL",
		Elevation: 144,
		Latitude:  -52.9550018311,
		Longitude: -70.0327987671,
		Timezone:  "America/Punta_Arenas",
	},
	"SCIV": {
		ICAO:      "SCIV",
		Name:      "Panimavida Airport",
		City:      "Panimavida",
		State:     "Maule",
		Country:   "CL",
		Elevation: 591,
		Latitude:  -35.7569007874,
		Longitude: -71.4041976929,
		Timezone:  "America/Santiago",
	},
	"SCJC": {
		ICAO:      "SCJC",
		Name:      "James Conrad Airport",
		City:      "Ranguelmo",
		State:     "Biobio",
		Country:   "CL",
		Elevation: 528,
		Latitude:  -36.581111908,
		Longitude: -72.7733306885,
		Timezone:  "America/Santiago",
	},
	"SCJK": {
		ICAO:      "SCJK",
		Name:      "Juan Kemp Airport",
		City:      "Osorno",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 459,
		Latitude:  -40.8844451904,
		Longitude: -72.368888855,
		Timezone:  "America/Santiago",
	},
	"SCJO": {
		ICAO:      "SCJO",
		IATA:      "ZOS",
		Name:      "Canal Bajo Carlos - Hott Siebert Airport",
		City:      "Osorno",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 187,
		Latitude:  -40.6111984253,
		Longitude: -73.0609970093,
		Timezone:  "America/Santiago",
	},
	"SCJV": {
		ICAO:      "SCJV",
		Name:      "El Parron Airport",
		City:      "San Javier",
		State:     "Maule",
		Country:   "CL",
		Elevation: 200,
		Latitude:  -35.5480537415,
		Longitude: -71.7761077881,
		Timezone:  "America/Santiago",
	},
	"SCKA": {
		ICAO:      "SCKA",
		Name:      "Santa Marta Airport",
		City:      "San Carlos",
		State:     "Biobio",
		Country:   "CL",
		Elevation: 577,
		Latitude:  -36.493057251,
		Longitude: -71.8969421387,
		Timezone:  "America/Santiago",
	},
	"SCKB": {
		ICAO:      "SCKB",
		Name:      "Llollenorte Airport",
		City:      "Lago Caburga",
		State:     "Araucania",
		Country:   "CL",
		Elevation: 1641,
		Latitude:  -39.0574989319,
		Longitude: -71.6897201538,
		Timezone:  "America/Santiago",
	},
	"SCKC": {
		ICAO:      "SCKC",
		Name:      "Roberto Chavez Airport",
		City:      "Cunco",
		State:     "Araucania",
		Country:   "CL",
		Elevation: 895,
		Latitude:  -38.9069442749,
		Longitude: -72.2152786255,
		Timezone:  "America/Santiago",
	},
	"SCKD": {
		ICAO:      "SCKD",
		Name:      "El Cardal Airport",
		City:      "Rio Bueno",
		State:     "Los-Rios",
		Country:   "CL",
		Elevation: 525,
		Latitude:  -40.4678001404,
		Longitude: -72.6849975586,
		Timezone:  "America/Santiago",
	},
	"SCKE": {
		ICAO:      "SCKE",
		Name:      "Piedra Negra Airport",
		City:      "Pelluhue",
		State:     "Maule",
		Country:   "CL",
		Elevation: 164,
		Latitude:  -35.8544425964,
		Longitude: -72.6458358765,
		Timezone:  "America/Santiago",
	},
	"SCKI": {
		ICAO:      "SCKI",
		Name:      "Los Lirios Airport",
		City:      "Curico",
		State:     "Maule",
		Country:   "CL",
		Elevation: 820,
		Latitude:  -34.9038887024,
		Longitude: -71.201385498,
		Timezone:  "America/Santiago",
	},
	"SCKK": {
		ICAO:      "SCKK",
		Name:      "La Cascada Airport",
		City:      "Molina",
		State:     "Maule",
		Country:   "CL",
		Elevation: 1804,
		Latitude:  -35.3849983215,
		Longitude: -71.1066665649,
		Timezone:  "America/Santiago",
	},
	"SCKL": {
		ICAO:      "SCKL",
		Name:      "Lipangui Airport",
		City:      "Lampa",
		State:     "Santiago-Metropolitan",
		Country:   "CL",
		Elevation: 1558,
		Latitude:  -33.331111908,
		Longitude: -70.8499984741,
		Timezone:  "America/Santiago",
	},
	"SCKM": {
		ICAO:      "SCKM",
		Name:      "Cochamo Airport",
		City:      "Cochamo",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 130,
		Latitude:  -41.4833335876,
		Longitude: -72.3000030518,
		Timezone:  "America/Santiago",
	},
	"SCKN": {
		ICAO:      "SCKN",
		Name:      "Licancel Airport",
		City:      "Licanten",
		State:     "Maule",
		Country:   "CL",
		Elevation: 59,
		Latitude:  -34.9891662598,
		Longitude: -72.0188903809,
		Timezone:  "America/Santiago",
	},
	"SCKO": {
		ICAO:      "SCKO",
		Name:      "Agua Buena Airport",
		City:      "Collipulli",
		State:     "Araucania",
		Country:   "CL",
		Elevation: 1420,
		Latitude:  -37.993888855,
		Longitude: -72.249168396,
		Timezone:  "America/Santiago",
	},
	"SCKP": {
		ICAO:      "SCKP",
		IATA:      "CPP",
		Name:      "Coposa Airport",
		City:      "Pica",
		State:     "Tarapaca",
		Country:   "CL",
		Elevation: 12468,
		Latitude:  -20.7504997253,
		Longitude: -68.6835021973,
		Timezone:  "America/Santiago",
	},
	"SCKQ": {
		ICAO:      "SCKQ",
		Name:      "Curimanque Airport",
		City:      "Pucon",
		State:     "Araucania",
		Country:   "CL",
		Elevation: 1476,
		Latitude:  -39.3269462585,
		Longitude: -72.0272216797,
		Timezone:  "America/Santiago",
	},
	"SCKT": {
		ICAO:      "SCKT",
		Name:      "Torreon Airport",
		City:      "Coelemu",
		State:     "Biobio",
		Country:   "CL",
		Elevation: 216,
		Latitude:  -36.4988899231,
		Longitude: -72.6800003052,
		Timezone:  "America/Santiago",
	},
	"SCLA": {
		ICAO:      "SCLA",
		Name:      "General Tovarias Airport",
		City:      "Lautaro",
		State:     "Araucania",
		Country:   "CL",
		Elevation: 699,
		Latitude:  -38.5427780151,
		Longitude: -72.4547195435,
		Timezone:  "America/Santiago",
	},
	"SCLB": {
		ICAO:      "SCLB",
		Name:      "Los Pehuenches Airport",
		City:      "Lebu",
		State:     "Biobio",
		Country:   "CL",
		Elevation: 587,
		Latitude:  -37.6580543518,
		Longitude: -73.629447937,
		Timezone:  "America/Santiago",
	},
	"SCLC": {
		ICAO:      "SCLC",
		Name:      "Municipal de Vitacura Airport",
		City:      "Santiago",
		State:     "Santiago-Metropolitan",
		Country:   "CL",
		Elevation: 2274,
		Latitude:  -33.3807983398,
		Longitude: -70.5821990967,
		Timezone:  "America/Santiago",
	},
	"SCLD": {
		ICAO:      "SCLD",
		Name:      "Llanada Grande Airport",
		City:      "Llanada Grande",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 1001,
		Latitude:  -41.8666992188,
		Longitude: -71.9344024658,
		Timezone:  "America/Santiago",
	},
	"SCLE": {
		ICAO:      "SCLE",
		Name:      "La Escondida Airport",
		City:      "Antofagasta",
		State:     "Antofagasta",
		Country:   "CL",
		Elevation: 10289,
		Latitude:  -24.2947006226,
		Longitude: -69.1316986084,
		Timezone:  "America/Santiago",
	},
	"SCLF": {
		ICAO:      "SCLF",
		Name:      "Calcurrupe Airport",
		City:      "Llifen",
		State:     "Los-Rios",
		Country:   "CL",
		Elevation: 335,
		Latitude:  -40.2442016602,
		Longitude: -72.2369003296,
		Timezone:  "America/Santiago",
	},
	"SCLG": {
		ICAO:      "SCLG",
		Name:      "La Aguada Airport",
		City:      "Pencahue",
		State:     "Maule",
		Country:   "CL",
		Elevation: 246,
		Latitude:  -35.3427772522,
		Longitude: -71.7769470215,
		Timezone:  "America/Santiago",
	},
	"SCLI": {
		ICAO:      "SCLI",
		Name:      "Torca Airport",
		City:      "Llico",
		State:     "Maule",
		Country:   "CL",
		Elevation: 66,
		Latitude:  -34.7833328247,
		Longitude: -72.0508346558,
		Timezone:  "America/Santiago",
	},
	"SCLJ": {
		ICAO:      "SCLJ",
		Name:      "La Junta Airport",
		City:      "La Junta",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 120,
		Latitude:  -43.975276947,
		Longitude: -72.4130554199,
		Timezone:  "America/Santiago",
	},
	"SCLK": {
		ICAO:      "SCLK",
		Name:      "Lago Colico Airport",
		City:      "Cunco",
		State:     "Araucania",
		Country:   "CL",
		Elevation: 1148,
		Latitude:  -39.0574989319,
		Longitude: -72.0741653442,
		Timezone:  "America/Santiago",
	},
	"SCLL": {
		ICAO:      "SCLL",
		IATA:      "VLR",
		Name:      "Vallenar Airport",
		City:      "Vallenar",
		State:     "Atacama",
		Country:   "CL",
		Elevation: 1725,
		Latitude:  -28.5963993073,
		Longitude: -70.7559967041,
		Timezone:  "America/Santiago",
	},
	"SCLM": {
		ICAO:      "SCLM",
		Name:      "Las Mercedes Airport",
		City:      "San Javier",
		State:     "Maule",
		Country:   "CL",
		Elevation: 246,
		Latitude:  -35.5,
		Longitude: -71.8808364868,
		Timezone:  "America/Santiago",
	},
	"SCLN": {
		ICAO:      "SCLN",
		IATA:      "ZLR",
		Name:      "Municipal de Linares Airport",
		City:      "Linares",
		State:     "Maule",
		Country:   "CL",
		Elevation: 591,
		Latitude:  -35.8616981506,
		Longitude: -71.5485992432,
		Timezone:  "America/Santiago",
	},
	"SCLO": {
		ICAO:      "SCLO",
		Name:      "Leones Airport",
		City:      "Chile Chico",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 705,
		Latitude:  -46.7655563354,
		Longitude: -72.8183364868,
		Timezone:  "America/Santiago",
	},
	"SCLP": {
		ICAO:      "SCLP",
		Name:      "Los Petiles Airport",
		City:      "Molina",
		State:     "Maule",
		Country:   "CL",
		Elevation: 1575,
		Latitude:  -35.2155570984,
		Longitude: -71.1086120605,
		Timezone:  "America/Santiago",
	},
	"SCLQ": {
		ICAO:      "SCLQ",
		Name:      "Diego Portales Airport",
		City:      "La Ligua",
		State:     "Valparaiso",
		Country:   "CL",
		Elevation: 250,
		Latitude:  -32.4524993896,
		Longitude: -71.2588882446,
		Timezone:  "America/Santiago",
	},
	"SCLR": {
		ICAO:      "SCLR",
		Name:      "Los Alerces Airport",
		City:      "Chaiten",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 345,
		Latitude:  -42.7805557251,
		Longitude: -72.6447219849,
		Timezone:  "America/Santiago",
	},
	"SCLS": {
		ICAO:      "SCLS",
		Name:      "Esperanza Airport",
		City:      "Lautaro",
		State:     "Araucania",
		Country:   "CL",
		Elevation: 1230,
		Latitude:  -38.5488891602,
		Longitude: -72.1458358765,
		Timezone:  "America/Santiago",
	},
	"SCLU": {
		ICAO:      "SCLU",
		Name:      "La Laguna Airport",
		City:      "Marchigue",
		State:     "O'Higgins",
		Country:   "CL",
		Elevation: 528,
		Latitude:  -34.3525009155,
		Longitude: -71.6638870239,
		Timezone:  "America/Santiago",
	},
	"SCLV": {
		ICAO:      "SCLV",
		Name:      "La Vina Airport",
		City:      "Guangali",
		State:     "Valparaiso",
		Country:   "CL",
		Elevation: 870,
		Latitude:  -32.138053894,
		Longitude: -71.3980560303,
		Timezone:  "America/Santiago",
	},
	"SCLY": {
		ICAO:      "SCLY",
		Name:      "La Playa Airport",
		City:      "Arauco",
		State:     "Biobio",
		Country:   "CL",
		Elevation: 32,
		Latitude:  -37.2163887024,
		Longitude: -73.2330551147,
		Timezone:  "America/Santiago",
	},
	"SCMA": {
		ICAO:      "SCMA",
		Name:      "Puerto Marin Balmaceda Airport",
		City:      "Puerto Marin Balmaceda",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 10,
		Latitude:  -43.7863883972,
		Longitude: -72.9502792358,
		Timezone:  "America/Santiago",
	},
	"SCMC": {
		ICAO:      "SCMC",
		Name:      "Meseta Cosmelli Airport",
		City:      "Puerto Guadal",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 950,
		Latitude:  -46.7455558777,
		Longitude: -72.5261077881,
		Timezone:  "America/Santiago",
	},
	"SCME": {
		ICAO:      "SCME",
		Name:      "Los Cuatro Diablos Airport",
		City:      "Melipilla",
		State:     "Santiago-Metropolitan",
		Country:   "CL",
		Elevation: 660,
		Latitude:  -33.676109314,
		Longitude: -71.1433334351,
		Timezone:  "America/Santiago",
	},
	"SCMF": {
		ICAO:      "SCMF",
		Name:      "Malloco Airport",
		City:      "Villarrica",
		State:     "Araucania",
		Country:   "CL",
		Elevation: 935,
		Latitude:  -39.2558326721,
		Longitude: -72.3433303833,
		Timezone:  "America/Santiago",
	},
	"SCMG": {
		ICAO:      "SCMG",
		Name:      "Santa Maria de Mingre Airport",
		City:      "San Javier",
		State:     "Maule",
		Country:   "CL",
		Elevation: 574,
		Latitude:  -35.5705566406,
		Longitude: -72.0038909912,
		Timezone:  "America/Santiago",
	},
	"SCMH": {
		ICAO:      "SCMH",
		Name:      "La Esperanza Airport",
		City:      "Marchigue",
		State:     "O'Higgins",
		Country:   "CL",
		Elevation: 540,
		Latitude:  -34.2872238159,
		Longitude: -71.5583343506,
		Timezone:  "America/Santiago",
	},
	"SCMI": {
		ICAO:      "SCMI",
		Name:      "Los Tricahues Airport",
		City:      "Mialqui",
		State:     "Coquimbo",
		Country:   "CL",
		Elevation: 1650,
		Latitude:  -30.7374992371,
		Longitude: -70.8597259521,
		Timezone:  "America/Santiago",
	},
	"SCMK": {
		ICAO:      "SCMK",
		Name:      "Melinka Airport",
		City:      "Melinka",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 35,
		Latitude:  -43.8950004578,
		Longitude: -73.7388916016,
		Timezone:  "America/Santiago",
	},
	"SCML": {
		ICAO:      "SCML",
		Name:      "Melipeuco Airport",
		City:      "Santa Barbara",
		State:     "Araucania",
		Country:   "CL",
		Elevation: 1388,
		Latitude:  -38.856111,
		Longitude: -71.8125,
		Timezone:  "America/Santiago",
	},
	"SCMO": {
		ICAO:      "SCMO",
		Name:      "Los Monos Airport",
		City:      "Molina",
		State:     "Maule",
		Country:   "CL",
		Elevation: 574,
		Latitude:  -35.1902770996,
		Longitude: -71.4186096191,
		Timezone:  "America/Santiago",
	},
	"SCMP": {
		ICAO:      "SCMP",
		Name:      "Melipilla Airport",
		City:      "Melipilla",
		State:     "Santiago-Metropolitan",
		Country:   "CL",
		Elevation: 574,
		Latitude:  -33.6738891602,
		Longitude: -71.193611145,
		Timezone:  "America/Santiago",
	},
	"SCMR": {
		ICAO:      "SCMR",
		Name:      "Las Aguilas Oriente Airport",
		City:      "Rapel",
		State:     "O'Higgins",
		Country:   "CL",
		Elevation: 450,
		Latitude:  -34.169167,
		Longitude: -71.531111,
		Timezone:  "America/Santiago",
	},
	"SCMS": {
		ICAO:      "SCMS",
		Name:      "Las Moras Airport",
		City:      "Longavi",
		State:     "Maule",
		Country:   "CL",
		Elevation: 512,
		Latitude:  -36.010278,
		Longitude: -71.594444,
		Timezone:  "America/Santiago",
	},
	"SCMU": {
		ICAO:      "SCMU",
		Name:      "Panilonco Airport",
		City:      "Pichilemu",
		State:     "O'Higgins",
		Country:   "CL",
		Elevation: 574,
		Latitude:  -34.2811126709,
		Longitude: -71.9469451904,
		Timezone:  "America/Santiago",
	},
	"SCMV": {
		ICAO:      "SCMV",
		Name:      "Vina San Pedro Airport",
		City:      "Molina",
		State:     "Maule",
		Country:   "CL",
		Elevation: 738,
		Latitude:  -35.1058349609,
		Longitude: -71.3277740479,
		Timezone:  "America/Santiago",
	},
	"SCMY": {
		ICAO:      "SCMY",
		Name:      "Carolina Airport",
		City:      "Michilla",
		State:     "Antofagasta",
		Country:   "CL",
		Elevation: 128,
		Latitude:  -22.7036113739,
		Longitude: -70.2825012207,
		Timezone:  "America/Santiago",
	},
	"SCMZ": {
		ICAO:      "SCMZ",
		Name:      "Marina de Rapel Airport",
		City:      "El Manzano",
		State:     "O'Higgins",
		Country:   "CL",
		Elevation: 344,
		Latitude:  -34.1516685486,
		Longitude: -71.4458312988,
		Timezone:  "America/Santiago",
	},
	"SCNA": {
		ICAO:      "SCNA",
		Name:      "Fundo La Cana Airport",
		City:      "Linares",
		State:     "Maule",
		Country:   "CL",
		Elevation: 525,
		Latitude:  -35.976389,
		Longitude: -71.597222,
		Timezone:  "America/Santiago",
	},
	"SCND": {
		ICAO:      "SCND",
		Name:      "Nadis Airport",
		City:      "Nadis",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 262,
		Latitude:  -47.4730567932,
		Longitude: -72.9519424438,
		Timezone:  "America/Santiago",
	},
	"SCNE": {
		ICAO:      "SCNE",
		Name:      "Maria Elena Airport",
		City:      "Maria Elena",
		State:     "Antofagasta",
		Country:   "CL",
		Elevation: 4003,
		Latitude:  -22.3072223663,
		Longitude: -69.7050018311,
		Timezone:  "America/Santiago",
	},
	"SCNG": {
		ICAO:      "SCNG",
		Name:      "Papageno Airport",
		City:      "Panguipulli",
		State:     "Araucania",
		Country:   "CL",
		Elevation: 853,
		Latitude:  -39.5505981445,
		Longitude: -72.2902984619,
		Timezone:  "America/Santiago",
	},
	"SCNH": {
		ICAO:      "SCNH",
		Name:      "Ainhoa Airport",
		City:      "Vilcun",
		State:     "Araucania",
		Country:   "CL",
		Elevation: 992,
		Latitude:  -38.683333,
		Longitude: -72.243889,
		Timezone:  "America/Santiago",
	},
	"SCNI": {
		ICAO:      "SCNI",
		Name:      "Santa Eugenia Airport",
		City:      "San Nicolas",
		State:     "Biobio",
		Country:   "CL",
		Elevation: 410,
		Latitude:  -36.4344444275,
		Longitude: -72.1569442749,
		Timezone:  "America/Santiago",
	},
	"SCNK": {
		ICAO:      "SCNK",
		Name:      "Los Pelambres Airport",
		City:      "Cuncumen",
		State:     "Coquimbo",
		Country:   "CL",
		Elevation: 4003,
		Latitude:  -31.8730564117,
		Longitude: -70.6183319092,
		Timezone:  "America/Santiago",
	},
	"SCNM": {
		ICAO:      "SCNM",
		Name:      "Las Misiones Airport",
		City:      "Canete",
		State:     "Biobio",
		Country:   "CL",
		Elevation: 252,
		Latitude:  -37.7761116028,
		Longitude: -73.3836135864,
		Timezone:  "America/Santiago",
	},
	"SCNO": {
		ICAO:      "SCNO",
		Name:      "Nochaco Airport",
		City:      "Nochaco",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 328,
		Latitude:  -40.892868042,
		Longitude: -72.8799972534,
		Timezone:  "America/Santiago",
	},
	"SCNR": {
		ICAO:      "SCNR",
		Name:      "Fundo Naicura Airport",
		City:      "Rengo",
		State:     "O'Higgins",
		Country:   "CL",
		Elevation: 1050,
		Latitude:  -34.3202781677,
		Longitude: -70.9202804565,
		Timezone:  "America/Santiago",
	},
	"SCNS": {
		ICAO:      "SCNS",
		Name:      "Sandra Scabini Airport",
		City:      "Punta Arenas",
		State:     "Magallanes",
		Country:   "CL",
		Elevation: 236,
		Latitude:  -52.3961105347,
		Longitude: -69.756942749,
		Timezone:  "America/Punta_Arenas",
	},
	"SCNT": {
		ICAO:      "SCNT",
		IATA:      "PNT",
		Name:      "Tte. Julio Gallardo Airport",
		City:      "Puerto Natales",
		State:     "Magallanes",
		Country:   "CL",
		Elevation: 217,
		Latitude:  -51.6715011597,
		Longitude: -72.5283966064,
		Timezone:  "America/Punta_Arenas",
	},
	"SCNY": {
		ICAO:      "SCNY",
		Name:      "Yendegaia Airport",
		City:      "Navarino",
		State:     "Magallanes",
		Country:   "CL",
		Elevation: 164,
		Latitude:  -54.8313903809,
		Longitude: -68.8316650391,
		Timezone:  "America/Punta_Arenas",
	},
	"SCOA": {
		ICAO:      "SCOA",
		Name:      "Estancia Los Loros Airport",
		City:      "Ovalle",
		State:     "Coquimbo",
		Country:   "CL",
		Elevation: 820,
		Latitude:  -30.7894439697,
		Longitude: -71.6183319092,
		Timezone:  "America/Santiago",
	},
	"SCOB": {
		ICAO:      "SCOB",
		Name:      "Tabali Bajo Airport",
		City:      "Ovalle",
		State:     "Coquimbo",
		Country:   "CL",
		Elevation: 732,
		Latitude:  -30.643333,
		Longitude: -71.401944,
		Timezone:  "America/Santiago",
	},
	"SCOC": {
		ICAO:      "SCOC",
		Name:      "Las Araucarias Airport",
		City:      "Puerto Octay",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 600,
		Latitude:  -40.993057251,
		Longitude: -72.6597213745,
		Timezone:  "America/Santiago",
	},
	"SCOE": {
		ICAO:      "SCOE",
		Name:      "San Miguel Airport",
		City:      "Romeral",
		State:     "Maule",
		Country:   "CL",
		Elevation: 1509,
		Latitude:  -34.962223053,
		Longitude: -71.0163879395,
		Timezone:  "America/Santiago",
	},
	"SCOH": {
		ICAO:      "SCOH",
		Name:      "Villa O'Higgins Airport",
		City:      "Villa O'Higgins",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 898,
		Latitude:  -48.4692115784,
		Longitude: -72.5643157959,
		Timezone:  "America/Santiago",
	},
	"SCOI": {
		ICAO:      "SCOI",
		Name:      "Los Coipos Airport",
		City:      "Hualanne",
		State:     "Maule",
		Country:   "CL",
		Elevation: 318,
		Latitude:  -34.912778,
		Longitude: -71.660833,
		Timezone:  "America/Santiago",
	},
	"SCOL": {
		ICAO:      "SCOL",
		Name:      "Refugio del Lago Airport",
		City:      "Puyehue",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 676,
		Latitude:  -40.6894454956,
		Longitude: -72.3455581665,
		Timezone:  "America/Santiago",
	},
	"SCOM": {
		ICAO:      "SCOM",
		Name:      "Olmue Airport",
		City:      "Olmue",
		State:     "Valparaiso",
		Country:   "CL",
		Elevation: 396,
		Latitude:  -32.9975013733,
		Longitude: -71.1722259521,
		Timezone:  "America/Santiago",
	},
	"SCON": {
		ICAO:      "SCON",
		Name:      "Quellon Airport",
		City:      "Quellon",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 16,
		Latitude:  -43.1366996765,
		Longitude: -73.6350021362,
		Timezone:  "America/Santiago",
	},
	"SCOO": {
		ICAO:      "SCOO",
		Name:      "Melimoyu Airport",
		City:      "Cisnes",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 20,
		Latitude:  -44.0975,
		Longitude: -73.096944,
		Timezone:  "America/Santiago",
	},
	"SCOP": {
		ICAO:      "SCOP",
		Name:      "Pilauco Airport",
		City:      "Osorno",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 213,
		Latitude:  -40.5691680908,
		Longitude: -73.1299972534,
		Timezone:  "America/Santiago",
	},
	"SCOS": {
		ICAO:      "SCOS",
		Name:      "Osorno Pampa Ale Airport",
		City:      "Osorno",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 79,
		Latitude:  -40.5471992493,
		Longitude: -73.1556015015,
		Timezone:  "America/Santiago",
	},
	"SCOT": {
		ICAO:      "SCOT",
		Name:      "Santa Rosa de Tabali Airport",
		City:      "Ovalle",
		State:     "Coquimbo",
		Country:   "CL",
		Elevation: 787,
		Latitude:  -30.6765003204,
		Longitude: -71.4010009766,
		Timezone:  "America/Santiago",
	},
	"SCOV": {
		ICAO:      "SCOV",
		IATA:      "OVL",
		Name:      "El Tuqui Airport",
		City:      "Ovalle",
		State:     "Coquimbo",
		Country:   "CL",
		Elevation: 1100,
		Latitude:  -30.5592002869,
		Longitude: -71.1755981445,
		Timezone:  "America/Santiago",
	},
	"SCOY": {
		ICAO:      "SCOY",
		Name:      "Huayanay Airport",
		City:      "Ovalle",
		State:     "Coquimbo",
		Country:   "CL",
		Elevation: 164,
		Latitude:  -30.4305553436,
		Longitude: -71.5322189331,
		Timezone:  "America/Santiago",
	},
	"SCPA": {
		ICAO:      "SCPA",
		Name:      "Paranal Airport",
		City:      "Antofagasta",
		State:     "Antofagasta",
		Country:   "CL",
		Elevation: 6834,
		Latitude:  -24.6394004822,
		Longitude: -70.3488998413,
		Timezone:  "America/Santiago",
	},
	"SCPB": {
		ICAO:      "SCPB",
		Name:      "Puelo Bajo Airport",
		City:      "Puelo Bajo",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 120,
		Latitude:  -41.6622238159,
		Longitude: -72.3022232056,
		Timezone:  "America/Santiago",
	},
	"SCPC": {
		ICAO:      "SCPC",
		IATA:      "ZPC",
		Name:      "Pucon Airport",
		City:      "Pucon",
		State:     "Araucania",
		Country:   "CL",
		Elevation: 853,
		Latitude:  -39.2928009033,
		Longitude: -71.9159011841,
		Timezone:  "America/Santiago",
	},
	"SCPE": {
		ICAO:      "SCPE",
		Name:      "San Pedro de Atacama Airport",
		City:      "San Pedro De Atacama",
		State:     "Antofagasta",
		Country:   "CL",
		Elevation: 7960,
		Latitude:  -22.9216995239,
		Longitude: -68.1584014893,
		Timezone:  "America/Santiago",
	},
	"SCPF": {
		ICAO:      "SCPF",
		Name:      "Marcel Marchant Airport",
		City:      "Puerto Montt",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 367,
		Latitude:  -41.4577789307,
		Longitude: -72.9186096191,
		Timezone:  "America/Santiago",
	},
	"SCPG": {
		ICAO:      "SCPG",
		Name:      "Municipal de Panguipulli Airport",
		City:      "Panguipulli",
		State:     "Los-Rios",
		Country:   "CL",
		Elevation: 940,
		Latitude:  -39.653301239,
		Longitude: -72.3602981567,
		Timezone:  "America/Santiago",
	},
	"SCPH": {
		ICAO:      "SCPH",
		Name:      "Puyuhuapi Airport",
		City:      "Puyuhuapi",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 43,
		Latitude:  -44.3856010437,
		Longitude: -72.5942001343,
		Timezone:  "America/Santiago",
	},
	"SCPI": {
		ICAO:      "SCPI",
		Name:      "Pullami Airport",
		City:      "Coihueco",
		State:     "Biobio",
		Country:   "CL",
		Elevation: 804,
		Latitude:  -36.586667,
		Longitude: -71.807778,
		Timezone:  "America/Santiago",
	},
	"SCPK": {
		ICAO:      "SCPK",
		Name:      "Puerto Cisnes Airport",
		City:      "Puerto Cisnes",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 45,
		Latitude:  -44.7458343506,
		Longitude: -72.700553894,
		Timezone:  "America/Santiago",
	},
	"SCPL": {
		ICAO:      "SCPL",
		Name:      "Calpulli Airport",
		City:      "Paillaco",
		State:     "Los-Rios",
		Country:   "CL",
		Elevation: 755,
		Latitude:  -40.1288986206,
		Longitude: -72.6616973877,
		Timezone:  "America/Santiago",
	},
	"SCPM": {
		ICAO:      "SCPM",
		Name:      "Pichilemu Airport",
		City:      "Pichilemu",
		State:     "O'Higgins",
		Country:   "CL",
		Elevation: 82,
		Latitude:  -34.3941650391,
		Longitude: -72.0191650391,
		Timezone:  "America/Santiago",
	},
	"SCPN": {
		ICAO:      "SCPN",
		Name:      "Pillan Airport",
		City:      "Chaiten",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 33,
		Latitude:  -42.5461120605,
		Longitude: -72.4941635132,
		Timezone:  "America/Santiago",
	},
	"SCPO": {
		ICAO:      "SCPO",
		Name:      "Los Paltos Airport",
		City:      "Quinta Tilcoco",
		State:     "O'Higgins",
		Country:   "CL",
		Elevation: 869,
		Latitude:  -34.3280563354,
		Longitude: -70.9641647339,
		Timezone:  "America/Santiago",
	},
	"SCPP": {
		ICAO:      "SCPP",
		Name:      "Poco a Poco Airport",
		City:      "Mulchen",
		State:     "Biobio",
		Country:   "CL",
		Elevation: 1680,
		Latitude:  -37.8488883972,
		Longitude: -72.0083312988,
		Timezone:  "America/Santiago",
	},
	"SCPQ": {
		ICAO:      "SCPQ",
		IATA:      "MHC",
		Name:      "Mocopulli Airport",
		City:      "Dalcahue",
		State:     "Los Lagos Region",
		Country:   "CL",
		Elevation: 528,
		Latitude:  -42.340278,
		Longitude: -73.715556,
		Timezone:  "America/Santiago",
	},
	"SCPR": {
		ICAO:      "SCPR",
		Name:      "Corte Alto Airport",
		City:      "Purranque",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 416,
		Latitude:  -40.9433326721,
		Longitude: -73.1699981689,
		Timezone:  "America/Santiago",
	},
	"SCPS": {
		ICAO:      "SCPS",
		Name:      "Perales Airport",
		City:      "Copiapo",
		State:     "Atacama",
		Country:   "CL",
		Elevation: 633,
		Latitude:  -27.3516674042,
		Longitude: -70.5894470215,
		Timezone:  "America/Santiago",
	},
	"SCPT": {
		ICAO:      "SCPT",
		Name:      "La Puerta Airport",
		City:      "Santa Cruz",
		State:     "O'Higgins",
		Country:   "CL",
		Elevation: 509,
		Latitude:  -34.613056,
		Longitude: -71.375833,
		Timezone:  "America/Santiago",
	},
	"SCPU": {
		ICAO:      "SCPU",
		Name:      "Peulla Airport",
		City:      "Peulla",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 900,
		Latitude:  -41.0541648865,
		Longitude: -72.0083312988,
		Timezone:  "America/Santiago",
	},
	"SCPV": {
		ICAO:      "SCPV",
		IATA:      "PUX",
		Name:      "El Mirador Airport",
		City:      "Puerto Varas",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 430,
		Latitude:  -41.3493995667,
		Longitude: -72.9467010498,
		Timezone:  "America/Santiago",
	},
	"SCPW": {
		ICAO:      "SCPW",
		Name:      "Peumo Airport",
		City:      "Peumo",
		State:     "O'Higgins",
		Country:   "CL",
		Elevation: 557,
		Latitude:  -34.4088897705,
		Longitude: -71.1688919067,
		Timezone:  "America/Santiago",
	},
	"SCPX": {
		ICAO:      "SCPX",
		Name:      "Punta Catalina Airport",
		City:      "Punta Catalina",
		State:     "Magallanes",
		Country:   "CL",
		Elevation: 90,
		Latitude:  -52.5916671753,
		Longitude: -68.75,
		Timezone:  "America/Punta_Arenas",
	},
	"SCPY": {
		ICAO:      "SCPY",
		Name:      "Cerro Castillo Airport",
		City:      "Cerro Castillo",
		State:     "Magallanes",
		Country:   "CL",
		Elevation: 427,
		Latitude:  -51.26222229,
		Longitude: -72.3375015259,
		Timezone:  "America/Punta_Arenas",
	},
	"SCPZ": {
		ICAO:      "SCPZ",
		Name:      "Patriot Hills Airport",
		City:      "Antartica",
		Country:   "AQ",
		Elevation: 2900,
		Latitude:  -80.3182983398,
		Longitude: -81.3080978394,
		Timezone:  "Etc/GMT+5",
	},
	"SCQC": {
		ICAO:      "SCQC",
		Name:      "La Colmena Airport",
		City:      "Quino",
		State:     "Araucania",
		Country:   "CL",
		Elevation: 823,
		Latitude:  -38.3330574036,
		Longitude: -72.5869445801,
		Timezone:  "America/Santiago",
	},
	"SCQE": {
		ICAO:      "SCQE",
		Name:      "Quenac Airport",
		City:      "Isla Quenac",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 164,
		Latitude:  -42.4672241211,
		Longitude: -73.3358306885,
		Timezone:  "America/Santiago",
	},
	"SCQI": {
		ICAO:      "SCQI",
		Name:      "Icalma Airport",
		City:      "Lonquimay",
		State:     "Araucania",
		Country:   "CL",
		Elevation: 3806,
		Latitude:  -38.7497215271,
		Longitude: -71.2177810669,
		Timezone:  "America/Santiago",
	},
	"SCQK": {
		ICAO:      "SCQK",
		Name:      "Lequecahue Airport",
		City:      "Tirua",
		State:     "Biobio",
		Country:   "CL",
		Elevation: 220,
		Latitude:  -38.3516654968,
		Longitude: -73.488609314,
		Timezone:  "America/Santiago",
	},
	"SCQL": {
		ICAO:      "SCQL",
		Name:      "El Boco Airport",
		City:      "Quillota",
		State:     "Valparaiso",
		Country:   "CL",
		Elevation: 459,
		Latitude:  -32.8511123657,
		Longitude: -71.2463912964,
		Timezone:  "America/Santiago",
	},
	"SCQM": {
		ICAO:      "SCQM",
		Name:      "Las Quemas Airport",
		City:      "Osorno",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 250,
		Latitude:  -40.6383323669,
		Longitude: -73.1275024414,
		Timezone:  "America/Santiago",
	},
	"SCQO": {
		ICAO:      "SCQO",
		Name:      "Quitralco Airport",
		City:      "Aysen",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 46,
		Latitude:  -45.580556,
		Longitude: -73.225556,
		Timezone:  "America/Santiago",
	},
	"SCQR": {
		ICAO:      "SCQR",
		Name:      "Los Morros Airport",
		City:      "Cobquecura",
		State:     "Biobio",
		Country:   "CL",
		Elevation: 7,
		Latitude:  -36.1344451904,
		Longitude: -72.8044433594,
		Timezone:  "America/Santiago",
	},
	"SCQT": {
		ICAO:      "SCQT",
		Name:      "LasTacas Airport",
		City:      "Quebrada Las Tacas",
		State:     "Coquimbo",
		Country:   "CL",
		Elevation: 140,
		Latitude:  -30.0963993073,
		Longitude: -71.3640975952,
		Timezone:  "America/Santiago",
	},
	"SCQW": {
		ICAO:      "SCQW",
		Name:      "Quemchi Airport",
		City:      "Quemchi",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 393,
		Latitude:  -42.1527786255,
		Longitude: -73.5169448853,
		Timezone:  "America/Santiago",
	},
	"SCQX": {
		ICAO:      "SCQX",
		Name:      "Queilen Airport",
		City:      "Queilen",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 60,
		Latitude:  -42.8930549622,
		Longitude: -73.4763870239,
		Timezone:  "America/Santiago",
	},
	"SCQY": {
		ICAO:      "SCQY",
		Name:      "Villa Portales Airport",
		City:      "Lonquimay",
		State:     "Araucania",
		Country:   "CL",
		Elevation: 3182,
		Latitude:  -38.4499015808,
		Longitude: -71.3547973633,
		Timezone:  "America/Santiago",
	},
	"SCRA": {
		ICAO:      "SCRA",
		IATA:      "CNR",
		Name:      "Chanaral Airport",
		City:      "Chanaral",
		State:     "Atacama",
		Country:   "CL",
		Elevation: 97,
		Latitude:  -26.3325004578,
		Longitude: -70.6072998047,
		Timezone:  "America/Santiago",
	},
	"SCRB": {
		ICAO:      "SCRB",
		Name:      "Rio Bravo Airport",
		City:      "Tortel",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 37,
		Latitude:  -47.9858322144,
		Longitude: -73.138885498,
		Timezone:  "America/Santiago",
	},
	"SCRC": {
		ICAO:      "SCRC",
		Name:      "Villa Tapera Airport",
		City:      "Rio Cisnes",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 1640,
		Latitude:  -44.6263885498,
		Longitude: -71.638885498,
		Timezone:  "America/Santiago",
	},
	"SCRD": {
		ICAO:      "SCRD",
		IATA:      "VAP",
		Name:      "Rodelillo Airport",
		City:      "Vina Del Mar",
		State:     "Valparaiso",
		Country:   "CL",
		Elevation: 1100,
		Latitude:  -33.0680999756,
		Longitude: -71.5575027466,
		Timezone:  "America/Santiago",
	},
	"SCRE": {
		ICAO:      "SCRE",
		Name:      "Estancia Rio Cisnes Airport",
		City:      "Rio Cisnes",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 2150,
		Latitude:  -44.5,
		Longitude: -71.3216629028,
		Timezone:  "America/Santiago",
	},
	"SCRF": {
		ICAO:      "SCRF",
		Name:      "Laguna San Rafael Airport",
		City:      "Laguna San Rafael",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 20,
		Latitude:  -46.6388893127,
		Longitude: -73.8499984741,
		Timezone:  "America/Santiago",
	},
	"SCRG": {
		ICAO:      "SCRG",
		IATA:      "QRC",
		Name:      "De La Independencia Airport",
		City:      "Rancagua",
		State:     "O'Higgins",
		Country:   "CL",
		Elevation: 1446,
		Latitude:  -34.1736984253,
		Longitude: -70.7757034302,
		Timezone:  "America/Santiago",
	},
	"SCRH": {
		ICAO:      "SCRH",
		Name:      "Renihue Airport",
		City:      "Renihue",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 16,
		Latitude:  -42.5855560303,
		Longitude: -72.4944458008,
		Timezone:  "America/Santiago",
	},
	"SCRI": {
		ICAO:      "SCRI",
		Name:      "Rio Frio Airport",
		City:      "Rio Frio",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 1000,
		Latitude:  -41.7333335876,
		Longitude: -71.9666671753,
		Timezone:  "America/Santiago",
	},
	"SCRL": {
		ICAO:      "SCRL",
		Name:      "La Estrella Airport",
		City:      "Rapel",
		State:     "O'Higgins",
		Country:   "CL",
		Elevation: 397,
		Latitude:  -34.2011108398,
		Longitude: -71.4816665649,
		Timezone:  "America/Santiago",
	},
	"SCRM": {
		ICAO:      "SCRM",
		IATA:      "TNM",
		Name:      "Teniente Rodolfo Marsh Martin Base",
		City:      "Isla Rey Jorge",
		Country:   "AQ",
		Elevation: 147,
		Latitude:  -62.1907997131,
		Longitude: -58.9866981506,
		Timezone:  "America/Punta_Arenas",
	},
	"SCRN": {
		ICAO:      "SCRN",
		Name:      "Rio Negro Airport",
		City:      "Rio Negro",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 10,
		Latitude:  -41.962223053,
		Longitude: -72.4538879395,
		Timezone:  "America/Santiago",
	},
	"SCRO": {
		ICAO:      "SCRO",
		Name:      "Santa Barbara Airport",
		City:      "Romeral",
		State:     "Maule",
		Country:   "CL",
		Elevation: 1420,
		Latitude:  -34.9588890076,
		Longitude: -71.1827774048,
		Timezone:  "America/Santiago",
	},
	"SCRP": {
		ICAO:      "SCRP",
		Name:      "Rapelhuapi Airport",
		City:      "Rapel",
		State:     "O'Higgins",
		Country:   "CL",
		Elevation: 568,
		Latitude:  -34.1058006287,
		Longitude: -71.512802124,
		Timezone:  "America/Santiago",
	},
	"SCRQ": {
		ICAO:      "SCRQ",
		Name:      "Rucananco Airport",
		City:      "Rio Bueno",
		State:     "Los-Rios",
		Country:   "CL",
		Elevation: 502,
		Latitude:  -40.521900177,
		Longitude: -72.7727966309,
		Timezone:  "America/Santiago",
	},
	"SCRR": {
		ICAO:      "SCRR",
		Name:      "Purrahuin Airport",
		City:      "Rio Bueno",
		State:     "Los-Rios",
		Country:   "CL",
		Elevation: 345,
		Latitude:  -40.365278,
		Longitude: -72.776667,
		Timezone:  "America/Santiago",
	},
	"SCRS": {
		ICAO:      "SCRS",
		Name:      "El Rosario Airport",
		City:      "Cartagena",
		State:     "Santiago-Metropolitan",
		Country:   "CL",
		Elevation: 804,
		Latitude:  -33.488889,
		Longitude: -71.405556,
		Timezone:  "America/Santiago",
	},
	"SCRT": {
		ICAO:      "SCRT",
		Name:      "El Almendro Airport",
		City:      "Retiro",
		State:     "Maule",
		Country:   "CL",
		Elevation: 486,
		Latitude:  -35.9605560303,
		Longitude: -71.7933349609,
		Timezone:  "America/Santiago",
	},
	"SCRU": {
		ICAO:      "SCRU",
		Name:      "Rio Murta Airport",
		City:      "Rio Murta",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 600,
		Latitude:  -46.4533348083,
		Longitude: -72.688331604,
		Timezone:  "America/Santiago",
	},
	"SCRV": {
		ICAO:      "SCRV",
		Name:      "La Veronica Airport",
		City:      "Vilcun",
		State:     "Araucania",
		Country:   "CL",
		Elevation: 1280,
		Latitude:  -38.611389,
		Longitude: -72.128889,
		Timezone:  "America/Santiago",
	},
	"SCRW": {
		ICAO:      "SCRW",
		Name:      "Rucalonco Airport",
		City:      "Paredones",
		State:     "O'Higgins",
		Country:   "CL",
		Elevation: 240,
		Latitude:  -34.5486106873,
		Longitude: -72.0452804565,
		Timezone:  "America/Santiago",
	},
	"SCRZ": {
		ICAO:      "SCRZ",
		Name:      "El Carrizal Airport",
		City:      "Marchigue",
		State:     "O'Higgins",
		Country:   "CL",
		Elevation: 492,
		Latitude:  -34.443333,
		Longitude: -71.706111,
		Timezone:  "America/Santiago",
	},
	"SCSA": {
		ICAO:      "SCSA",
		Name:      "Alberto Santos Dumont Airport",
		City:      "Til Til",
		State:     "Santiago-Metropolitan",
		Country:   "CL",
		Elevation: 2346,
		Latitude:  -33.0286102295,
		Longitude: -70.8799972534,
		Timezone:  "America/Santiago",
	},
	"SCSB": {
		ICAO:      "SCSB",
		IATA:      "SMB",
		Name:      "Franco Bianco Airport",
		City:      "Cerro Sombrero",
		State:     "Magallanes",
		Country:   "CL",
		Elevation: 104,
		Latitude:  -52.7366981506,
		Longitude: -69.3336029053,
		Timezone:  "America/Punta_Arenas",
	},
	"SCSD": {
		ICAO:      "SCSD",
		Name:      "San Fernando Airport",
		City:      "San Fernando",
		State:     "O'Higgins",
		Country:   "CL",
		Elevation: 1079,
		Latitude:  -34.5657997131,
		Longitude: -70.9682998657,
		Timezone:  "America/Santiago",
	},
	"SCSE": {
		ICAO:      "SCSE",
		IATA:      "LSC",
		Name:      "La Florida Airport",
		City:      "La Serena-Coquimbo",
		State:     "Coquimbo",
		Country:   "CL",
		Elevation: 481,
		Latitude:  -29.9162006378,
		Longitude: -71.1995010376,
		Timezone:  "America/Santiago",
	},
	"SCSF": {
		ICAO:      "SCSF",
		IATA:      "SSD",
		Name:      "Victor Lafon Airport",
		City:      "San Felipe",
		State:     "Valparaiso",
		Country:   "CL",
		Elevation: 2188,
		Latitude:  -32.7458000183,
		Longitude: -70.7050018311,
		Timezone:  "America/Santiago",
	},
	"SCSG": {
		ICAO:      "SCSG",
		Name:      "San Geronimo Airport",
		City:      "Algarrobo",
		State:     "Valparaiso",
		Country:   "CL",
		Elevation: 341,
		Latitude:  -33.3532981873,
		Longitude: -71.6260986328,
		Timezone:  "America/Santiago",
	},
	"SCSH": {
		ICAO:      "SCSH",
		Name:      "El Budi Airport",
		City:      "Teodoro Schmidt",
		State:     "Araucania",
		Country:   "CL",
		Elevation: 82,
		Latitude:  -39.0627784729,
		Longitude: -73.1699981689,
		Timezone:  "America/Santiago",
	},
	"SCSJ": {
		ICAO:      "SCSJ",
		Name:      "San Javier Airport",
		City:      "San Javier",
		State:     "Maule",
		Country:   "CL",
		Elevation: 361,
		Latitude:  -35.6297225952,
		Longitude: -71.69972229,
		Timezone:  "America/Santiago",
	},
	"SCSK": {
		ICAO:      "SCSK",
		Name:      "Colorado Airport",
		City:      "San Clemente",
		State:     "Maule",
		Country:   "CL",
		Elevation: 1410,
		Latitude:  -35.6630554199,
		Longitude: -71.3036117554,
		Timezone:  "America/Santiago",
	},
	"SCSL": {
		ICAO:      "SCSL",
		Name:      "El Salar Airport",
		City:      "Salar\u00a0De Atacama",
		State:     "Antofagasta",
		Country:   "CL",
		Elevation: 7556,
		Latitude:  -23.6488990784,
		Longitude: -68.3050003052,
		Timezone:  "America/Santiago",
	},
	"SCSM": {
		ICAO:      "SCSM",
		Name:      "Minsal Airport",
		City:      "Salar\u00a0De Atacama",
		State:     "Antofagasta",
		Country:   "CL",
		Elevation: 7776,
		Latitude:  -23.5852775574,
		Longitude: -68.382774353,
		Timezone:  "America/Santiago",
	},
	"SCSN": {
		ICAO:      "SCSN",
		Name:      "Santo Domingo Airport",
		City:      "Santo Domingo",
		State:     "Valparaiso",
		Country:   "CL",
		Elevation: 246,
		Latitude:  -33.6563987732,
		Longitude: -71.614402771,
		Timezone:  "America/Santiago",
	},
	"SCSO": {
		ICAO:      "SCSO",
		Name:      "Costa del Sol Airport",
		City:      "Lago Rapel",
		State:     "O'Higgins",
		Country:   "CL",
		Elevation: 390,
		Latitude:  -34.1208343506,
		Longitude: -71.5272216797,
		Timezone:  "America/Santiago",
	},
	"SCSP": {
		ICAO:      "SCSP",
		Name:      "El Sobrante Airport",
		City:      "Petorca",
		State:     "Valparaiso",
		Country:   "CL",
		Elevation: 2559,
		Latitude:  -32.223611,
		Longitude: -70.8,
		Timezone:  "America/Santiago",
	},
	"SCSQ": {
		ICAO:      "SCSQ",
		Name:      "Quilpe Airport",
		City:      "San Pablo",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 157,
		Latitude:  -40.3411102295,
		Longitude: -73.3172225952,
		Timezone:  "America/Santiago",
	},
	"SCSR": {
		ICAO:      "SCSR",
		Name:      "Segundo Corral Alto Airport",
		City:      "Segundo Corral",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 1200,
		Latitude:  -42.0833320618,
		Longitude: -71.8638916016,
		Timezone:  "America/Santiago",
	},
	"SCSS": {
		ICAO:      "SCSS",
		Name:      "San Sebastian Airport",
		City:      "San Sebastian",
		State:     "Magallanes",
		Country:   "CL",
		Elevation: 50,
		Latitude:  -53.3170013428,
		Longitude: -68.6500015259,
		Timezone:  "America/Punta_Arenas",
	},
	"SCST": {
		ICAO:      "SCST",
		IATA:      "WCA",
		Name:      "Gamboa Airport",
		City:      "Castro",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 151,
		Latitude:  -42.4902992249,
		Longitude: -73.7727966309,
		Timezone:  "America/Santiago",
	},
	"SCSU": {
		ICAO:      "SCSU",
		Name:      "Santa Lucia Airport",
		City:      "Freire",
		State:     "Araucania",
		Country:   "CL",
		Elevation: 705,
		Latitude:  -38.9191665649,
		Longitude: -72.3666687012,
		Timezone:  "America/Santiago",
	},
	"SCSV": {
		ICAO:      "SCSV",
		Name:      "Vinasutil Airport",
		City:      "Peralillo",
		State:     "O'Higgins",
		Country:   "CL",
		Elevation: 427,
		Latitude:  -34.448890686,
		Longitude: -71.3858337402,
		Timezone:  "America/Santiago",
	},
	"SCSZ": {
		ICAO:      "SCSZ",
		Name:      "Puerto Sanchez Airport",
		City:      "Puerto Sanchez",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 680,
		Latitude:  -46.5999984741,
		Longitude: -72.6166687012,
		Timezone:  "America/Santiago",
	},
	"SCTA": {
		ICAO:      "SCTA",
		Name:      "Santa Luisa Airport",
		City:      "Santa Barbara",
		State:     "Biobio",
		Country:   "CL",
		Elevation: 1818,
		Latitude:  -37.743889,
		Longitude: -71.71,
		Timezone:  "America/Santiago",
	},
	"SCTB": {
		ICAO:      "SCTB",
		Name:      "Eulogio Sanchez Airport",
		City:      "Santiago",
		State:     "Santiago-Metropolitan",
		Country:   "CL",
		Elevation: 2129,
		Latitude:  -33.4562988281,
		Longitude: -70.5466995239,
		Timezone:  "America/Santiago",
	},
	"SCTC": {
		ICAO:      "SCTC",
		IATA:      "ZCO",
		Name:      "Maquehue Airport",
		City:      "Temuco",
		State:     "Araucania",
		Country:   "CL",
		Elevation: 304,
		Latitude:  -38.7667999268,
		Longitude: -72.6371002197,
		Timezone:  "America/Santiago",
	},
	"SCTE": {
		ICAO:      "SCTE",
		IATA:      "PMC",
		Name:      "El Tepual Airport",
		City:      "Puerto Montt",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 294,
		Latitude:  -41.4388999939,
		Longitude: -73.09400177,
		Timezone:  "America/Santiago",
	},
	"SCTG": {
		ICAO:      "SCTG",
		Name:      "Tongoy Airport",
		City:      "Tongoy",
		State:     "Coquimbo",
		Country:   "CL",
		Elevation: 49,
		Latitude:  -30.2663879395,
		Longitude: -71.4836120605,
		Timezone:  "America/Santiago",
	},
	"SCTH": {
		ICAO:      "SCTH",
		Name:      "Tres Chorrillos Airport",
		City:      "San Gregorio",
		State:     "Magallanes",
		Country:   "CL",
		Elevation: 322,
		Latitude:  -52.525278,
		Longitude: -70.723889,
		Timezone:  "America/Punta_Arenas",
	},
	"SCTL": {
		ICAO:      "SCTL",
		IATA:      "TLX",
		Name:      "Panguilemo Airport",
		City:      "Talca",
		State:     "Maule",
		Country:   "CL",
		Elevation: 371,
		Latitude:  -35.3777999878,
		Longitude: -71.6016998291,
		Timezone:  "America/Santiago",
	},
	"SCTM": {
		ICAO:      "SCTM",
		Name:      "La Montana Airport",
		City:      "Curico",
		State:     "Maule",
		Country:   "CL",
		Elevation: 1969,
		Latitude:  -34.9702758789,
		Longitude: -70.8463897705,
		Timezone:  "America/Santiago",
	},
	"SCTN": {
		ICAO:      "SCTN",
		IATA:      "WCH",
		Name:      "Chaiten Airport",
		City:      "Chaiten",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 13,
		Latitude:  -42.932800293,
		Longitude: -72.6990966797,
		Timezone:  "America/Santiago",
	},
	"SCTO": {
		ICAO:      "SCTO",
		IATA:      "ZIC",
		Name:      "Victoria Airport",
		City:      "Victoria",
		State:     "Araucania",
		Country:   "CL",
		Elevation: 1148,
		Latitude:  -38.2456016541,
		Longitude: -72.3486022949,
		Timezone:  "America/Santiago",
	},
	"SCTP": {
		ICAO:      "SCTP",
		Name:      "Rio Pascua Airport",
		City:      "Tortel",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 197,
		Latitude:  -48.2277793884,
		Longitude: -73.2977752686,
		Timezone:  "America/Santiago",
	},
	"SCTQ": {
		ICAO:      "SCTQ",
		Name:      "Tres Quebradas Airport",
		City:      "Alto Del Carmen",
		State:     "Atacama",
		Country:   "CL",
		Elevation: 11690,
		Latitude:  -29.2527999878,
		Longitude: -70.0888977051,
		Timezone:  "America/Santiago",
	},
	"SCTR": {
		ICAO:      "SCTR",
		Name:      "Traiguen Airport",
		City:      "Traiguen",
		State:     "Araucania",
		Country:   "CL",
		Elevation: 812,
		Latitude:  -38.2713890076,
		Longitude: -72.6624984741,
		Timezone:  "America/Santiago",
	},
	"SCTS": {
		ICAO:      "SCTS",
		Name:      "Santa Teresa del Almendral Airport",
		City:      "Melipilla",
		State:     "Santiago-Metropolitan",
		Country:   "CL",
		Elevation: 574,
		Latitude:  -33.575553894,
		Longitude: -71.2608337402,
		Timezone:  "America/Santiago",
	},
	"SCTT": {
		ICAO:      "SCTT",
		IATA:      "TTC",
		Name:      "Las Breas Airport",
		City:      "Taltal",
		State:     "Antofagasta",
		Country:   "CL",
		Elevation: 2580,
		Latitude:  -25.5643005371,
		Longitude: -70.3759002686,
		Timezone:  "America/Santiago",
	},
	"SCTU": {
		ICAO:      "SCTU",
		Name:      "Litueche Airport",
		City:      "Litueche",
		State:     "O'Higgins",
		Country:   "CL",
		Elevation: 787,
		Latitude:  -34.1105537415,
		Longitude: -71.7158355713,
		Timezone:  "America/Santiago",
	},
	"SCTW": {
		ICAO:      "SCTW",
		Name:      "El Tapihue Airport",
		City:      "Casablanca",
		State:     "Valparaiso",
		Country:   "CL",
		Elevation: 935,
		Latitude:  -33.3202781677,
		Longitude: -71.3361129761,
		Timezone:  "America/Santiago",
	},
	"SCUI": {
		ICAO:      "SCUI",
		Name:      "Pumalin Airport",
		City:      "Chaiten",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 18,
		Latitude:  -42.6958351135,
		Longitude: -72.8333358765,
		Timezone:  "America/Santiago",
	},
	"SCUL": {
		ICAO:      "SCUL",
		Name:      "El Litral Airport",
		City:      "Bulnes",
		State:     "Biobio",
		Country:   "CL",
		Elevation: 180,
		Latitude:  -36.7936096191,
		Longitude: -72.4208297729,
		Timezone:  "America/Santiago",
	},
	"SCUM": {
		ICAO:      "SCUM",
		Name:      "La Obra Airport",
		City:      "Cumpeo",
		State:     "Maule",
		Country:   "CL",
		Elevation: 673,
		Latitude:  -35.3005561829,
		Longitude: -71.3302764893,
		Timezone:  "America/Santiago",
	},
	"SCUN": {
		ICAO:      "SCUN",
		Name:      "Uni Frutti Airport",
		City:      "Teno",
		State:     "O'Higgins",
		Country:   "CL",
		Elevation: 950,
		Latitude:  -34.8130569458,
		Longitude: -71.0461120605,
		Timezone:  "America/Santiago",
	},
	"SCUP": {
		ICAO:      "SCUP",
		Name:      "Lontuecito Airport",
		City:      "Cumpeo",
		State:     "Maule",
		Country:   "CL",
		Elevation: 1066,
		Latitude:  -35.2822227478,
		Longitude: -71.2363891602,
		Timezone:  "America/Santiago",
	},
	"SCUR": {
		ICAO:      "SCUR",
		Name:      "Rucamalen Airport",
		City:      "Bulnes",
		State:     "Biobio",
		Country:   "CL",
		Elevation: 475,
		Latitude:  -36.8119430542,
		Longitude: -72.1672210693,
		Timezone:  "America/Santiago",
	},
	"SCUT": {
		ICAO:      "SCUT",
		Name:      "Verfrut Sur Airport",
		City:      "Longavi",
		State:     "Maule",
		Country:   "CL",
		Elevation: 1132,
		Latitude:  -36.2125015259,
		Longitude: -71.5444412231,
		Timezone:  "America/Santiago",
	},
	"SCUZ": {
		ICAO:      "SCUZ",
		Name:      "Aerosanta Cruz Airport",
		City:      "Santa Cruz",
		State:     "O'Higgins",
		Country:   "CL",
		Elevation: 502,
		Latitude:  -34.6491661072,
		Longitude: -71.3861083984,
		Timezone:  "America/Santiago",
	},
	"SCVA": {
		ICAO:      "SCVA",
		Name:      "Vinamar Airport",
		City:      "Casablanca",
		State:     "Valparaiso",
		Country:   "CL",
		Elevation: 935,
		Latitude:  -33.3569450378,
		Longitude: -71.3541641235,
		Timezone:  "America/Santiago",
	},
	"SCVB": {
		ICAO:      "SCVB",
		Name:      "Hospital Villa Baviera Airport",
		City:      "Parral",
		State:     "Maule",
		Country:   "CL",
		Elevation: 1040,
		Latitude:  -36.4002761841,
		Longitude: -71.563331604,
		Timezone:  "America/Santiago",
	},
	"SCVC": {
		ICAO:      "SCVC",
		Name:      "El Indio Airport",
		City:      "Vicuna",
		State:     "Coquimbo",
		Country:   "CL",
		Elevation: 11975,
		Latitude:  -29.834444046,
		Longitude: -70.0461120605,
		Timezone:  "America/Santiago",
	},
	"SCVD": {
		ICAO:      "SCVD",
		IATA:      "ZAL",
		Name:      "Pichoy Airport",
		City:      "Valdivia",
		State:     "Los-Rios",
		Country:   "CL",
		Elevation: 59,
		Latitude:  -39.6500015259,
		Longitude: -73.0860977173,
		Timezone:  "America/Santiago",
	},
	"SCVE": {
		ICAO:      "SCVE",
		Name:      "Lago Verde Airport",
		City:      "Lago Verde",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 1020,
		Latitude:  -44.2708320618,
		Longitude: -71.9166641235,
		Timezone:  "America/Santiago",
	},
	"SCVF": {
		ICAO:      "SCVF",
		Name:      "Verfrut Airport",
		City:      "San Pedro",
		State:     "Santiago-Metropolitan",
		Country:   "CL",
		Elevation: 443,
		Latitude:  -34.0072212219,
		Longitude: -71.3919448853,
		Timezone:  "America/Santiago",
	},
	"SCVG": {
		ICAO:      "SCVG",
		Name:      "El Vergel Airport",
		City:      "Rinihue",
		State:     "Los-Rios",
		Country:   "CL",
		Elevation: 820,
		Latitude:  -39.8293991089,
		Longitude: -72.4882965088,
		Timezone:  "America/Santiago",
	},
	"SCVH": {
		ICAO:      "SCVH",
		Name:      "La Victoria de Chacabuco Airport",
		City:      "Santiago",
		State:     "Santiago-Metropolitan",
		Country:   "CL",
		Elevation: 2139,
		Latitude:  -33.0508346558,
		Longitude: -70.7088851929,
		Timezone:  "America/Santiago",
	},
	"SCVI": {
		ICAO:      "SCVI",
		Name:      "Villarrica Airport",
		City:      "Villarrica",
		State:     "Araucania",
		Country:   "CL",
		Elevation: 945,
		Latitude:  -39.3125,
		Longitude: -72.0622253418,
		Timezone:  "America/Santiago",
	},
	"SCVJ": {
		ICAO:      "SCVJ",
		Name:      "Paredes Viejas Airport",
		City:      "Marchigue",
		State:     "O'Higgins",
		Country:   "CL",
		Elevation: 449,
		Latitude:  -34.3958320618,
		Longitude: -71.5452804565,
		Timezone:  "America/Santiago",
	},
	"SCVK": {
		ICAO:      "SCVK",
		Name:      "El Alamo Airport",
		City:      "Vichuquen",
		State:     "Maule",
		Country:   "CL",
		Elevation: 33,
		Latitude:  -34.8477783203,
		Longitude: -72.0222244263,
		Timezone:  "America/Santiago",
	},
	"SCVL": {
		ICAO:      "SCVL",
		Name:      "Las Marias Airport",
		City:      "Valdivia",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 13,
		Latitude:  -39.7969017029,
		Longitude: -73.2416992188,
		Timezone:  "America/Santiago",
	},
	"SCVM": {
		ICAO:      "SCVM",
		IATA:      "KNA",
		Name:      "Vina del mar Airport",
		City:      "Vina Del Mar",
		State:     "Valparaiso",
		Country:   "CL",
		Elevation: 461,
		Latitude:  -32.9496002197,
		Longitude: -71.4785995483,
		Timezone:  "America/Santiago",
	},
	"SCVN": {
		ICAO:      "SCVN",
		Name:      "Huancara Airport",
		City:      "Vicuna",
		State:     "Coquimbo",
		Country:   "CL",
		Elevation: 1968,
		Latitude:  -30.0268993378,
		Longitude: -70.7516021729,
		Timezone:  "America/Santiago",
	},
	"SCVO": {
		ICAO:      "SCVO",
		Name:      "Maria Ester Airport",
		City:      "Victoria",
		State:     "Araucania",
		Country:   "CL",
		Elevation: 978,
		Latitude:  -38.2319450378,
		Longitude: -72.4833297729,
		Timezone:  "America/Santiago",
	},
	"SCVQ": {
		ICAO:      "SCVQ",
		Name:      "Cuatro Pantanos Airport",
		City:      "Vichuquen",
		State:     "Maule",
		Country:   "CL",
		Elevation: 82,
		Latitude:  -34.837223053,
		Longitude: -72.0650024414,
		Timezone:  "America/Santiago",
	},
	"SCVS": {
		ICAO:      "SCVS",
		Name:      "Lago Vargas Airport",
		City:      "Cochrane",
		State:     "Aisen",
		Country:   "CL",
		Elevation: 56,
		Latitude:  -47.6772232056,
		Longitude: -73.067779541,
		Timezone:  "America/Santiago",
	},
	"SCVT": {
		ICAO:      "SCVT",
		Name:      "Vina Tarapaca Airport",
		City:      "Isla De Maipo",
		State:     "Santiago-Metropolitan",
		Country:   "CL",
		Elevation: 1066,
		Latitude:  -33.7661094666,
		Longitude: -70.9238891602,
		Timezone:  "America/Santiago",
	},
	"SCVU": {
		ICAO:      "SCVU",
		Name:      "Agromanzun Airport",
		City:      "Vilcun",
		State:     "Araucania",
		Country:   "CL",
		Elevation: 738,
		Latitude:  -38.6958351135,
		Longitude: -72.3405532837,
		Timezone:  "America/Santiago",
	},
	"SCVV": {
		ICAO:      "SCVV",
		Name:      "Los Maitenes de Villa Vieja Airport",
		City:      "La Union",
		State:     "Los-Rios",
		Country:   "CL",
		Elevation: 147,
		Latitude:  -40.3163986206,
		Longitude: -72.9772033691,
		Timezone:  "America/Santiago",
	},
	"SCVY": {
		ICAO:      "SCVY",
		Name:      "Malla Airport",
		City:      "Vilcun",
		State:     "Araucania",
		Country:   "CL",
		Elevation: 820,
		Latitude:  -38.6913871765,
		Longitude: -72.2855529785,
		Timezone:  "America/Santiago",
	},
	"SCXA": {
		ICAO:      "SCXA",
		Name:      "Alupenhue Airport",
		City:      "Molina",
		State:     "Maule",
		Country:   "CL",
		Elevation: 1600,
		Latitude:  -35.2394447327,
		Longitude: -71.0708312988,
		Timezone:  "America/Santiago",
	},
	"SCXB": {
		ICAO:      "SCXB",
		Name:      "Las Brujas Airport",
		City:      "Salamanca",
		State:     "Coquimbo",
		Country:   "CL",
		Elevation: 1483,
		Latitude:  -31.7866668701,
		Longitude: -71.0111083984,
		Timezone:  "America/Santiago",
	},
	"SCXR": {
		ICAO:      "SCXR",
		Name:      "Las Bandurrias Airport",
		City:      "Lago Ranco",
		State:     "Los-Rios",
		Country:   "CL",
		Elevation: 328,
		Latitude:  -40.3199996948,
		Longitude: -72.2249984741,
		Timezone:  "America/Santiago",
	},
	"SCYB": {
		ICAO:      "SCYB",
		Name:      "Trilahue Airport",
		City:      "Yumbel",
		State:     "Biobio",
		Country:   "CL",
		Elevation: 423,
		Latitude:  -37.1172218323,
		Longitude: -72.4111099243,
		Timezone:  "America/Santiago",
	},
	"SCYC": {
		ICAO:      "SCYC",
		Name:      "La Capilla Airport",
		City:      "Puyehue",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 485,
		Latitude:  -40.6277770996,
		Longitude: -72.76222229,
		Timezone:  "America/Santiago",
	},
	"SCYL": {
		ICAO:      "SCYL",
		Name:      "Lican Airport",
		City:      "Puyehue",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 625,
		Latitude:  -40.6413993835,
		Longitude: -72.4111022949,
		Timezone:  "America/Santiago",
	},
	"SCYO": {
		ICAO:      "SCYO",
		Name:      "Poyo Airport",
		City:      "El Chaiten",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 13,
		Latitude:  -42.226667,
		Longitude: -72.693611,
		Timezone:  "America/Santiago",
	},
	"SCYR": {
		ICAO:      "SCYR",
		Name:      "Los Maitenes Airport",
		City:      "Retiro",
		State:     "Maule",
		Country:   "CL",
		Elevation: 538,
		Latitude:  -36.031665802,
		Longitude: -71.7411117554,
		Timezone:  "America/Santiago",
	},
	"SCYU": {
		ICAO:      "SCYU",
		Name:      "Cuyumaique Airport",
		City:      "Purranque",
		State:     "Los-Lagos",
		Country:   "CL",
		Elevation: 443,
		Latitude:  -40.940556,
		Longitude: -73.011667,
		Timezone:  "America/Santiago",
	},
	"SCZB": {
		ICAO:      "SCZB",
		Name:      "Pozo Brujo Airport",
		City:      "La Union",
		State:     "Los-Rios",
		Country:   "CL",
		Elevation: 394,
		Latitude:  -40.205278,
		Longitude: -72.568056,
		Timezone:  "America/Santiago",
	},
	"SCZC": {
		ICAO:      "SCZC",
		Name:      "Casas Viejas Airport",
		City:      "Zapallar",
		State:     "Valparaiso",
		Country:   "CL",
		Elevation: 262,
		Latitude:  -32.5952758789,
		Longitude: -71.3472213745,
		Timezone:  "America/Santiago",
	},
	"SCZE": {
		ICAO:      "SCZE",
		Name:      "Estero Seco Airport",
		City:      "Pirque",
		State:     "Santiago-Metropolitan",
		Country:   "CL",
		Elevation: 2461,
		Latitude:  -33.7449989319,
		Longitude: -70.549446106,
		Timezone:  "America/Santiago",
	},
	"SD00": {
		ICAO:      "SD00",
		Name:      "Mj Aviation Ii Airport",
		City:      "Elk Point",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1115,
		Latitude:  42.6786994934,
		Longitude: -96.6988983154,
		Timezone:  "America/Chicago",
	},
	"SD01": {
		ICAO:      "SD01",
		Name:      "Mj Aviation I Airport",
		City:      "Letcher",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1305,
		Latitude:  43.8997001648,
		Longitude: -98.1076965332,
		Timezone:  "America/Chicago",
	},
	"SD03": {
		ICAO:      "SD03",
		Name:      "Calico Field",
		City:      "Winfred",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1750,
		Latitude:  44.0504989624,
		Longitude: -97.3350982666,
		Timezone:  "America/Chicago",
	},
	"SD05": {
		ICAO:      "SD05",
		Name:      "Thorson Airfield",
		City:      "Aberdeen",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1300,
		Latitude:  45.3716011047,
		Longitude: -98.4786987305,
		Timezone:  "America/Chicago",
	},
	"SD13": {
		ICAO:      "SD13",
		Name:      "Hofer Private Airport",
		City:      "Doland",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1368,
		Latitude:  44.8666000366,
		Longitude: -98.0719985962,
		Timezone:  "America/Chicago",
	},
	"SD14": {
		ICAO:      "SD14",
		Name:      "Valburg Ranch Airport",
		City:      "Draper",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1688,
		Latitude:  43.7332992554,
		Longitude: -100.3980026245,
		Timezone:  "America/Chicago",
	},
	"SD19": {
		ICAO:      "SD19",
		Name:      "Andersen Farms Airport",
		City:      "Badger",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1750,
		Latitude:  44.4653015137,
		Longitude: -97.2225036621,
		Timezone:  "America/Chicago",
	},
	"SD21": {
		ICAO:      "SD21",
		Name:      "Hayes Emergency Airstrip",
		City:      "Hayes",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1986,
		Latitude:  44.3708000183,
		Longitude: -101.0090026855,
		Timezone:  "America/Denver",
	},
	"SD24": {
		ICAO:      "SD24",
		Name:      "Bruch Ranch Airport",
		City:      "Sturgis",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 3070,
		Latitude:  44.4472007751,
		Longitude: -103.3949966431,
		Timezone:  "America/Denver",
	},
	"SD25": {
		ICAO:      "SD25",
		Name:      "Rappe Field",
		City:      "Ipswich",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1521,
		Latitude:  45.4522018433,
		Longitude: -99.012298584,
		Timezone:  "America/Chicago",
	},
	"SD29": {
		ICAO:      "SD29",
		Name:      "Menno Airport",
		City:      "Menno",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1350,
		Latitude:  43.2444000244,
		Longitude: -97.5531005859,
		Timezone:  "America/Chicago",
	},
	"SD33": {
		ICAO:      "SD33",
		Name:      "Sky Ranch For Boys Airport",
		City:      "Camp Crook",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 3200,
		Latitude:  45.5,
		Longitude: -104,
		Timezone:  "America/Denver",
	},
	"SD35": {
		ICAO:      "SD35",
		Name:      "Bruch Airfield",
		City:      "Sturgis",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 2980,
		Latitude:  44.494167,
		Longitude: -103.394444,
		Timezone:  "America/Denver",
	},
	"SD36": {
		ICAO:      "SD36",
		Name:      "Booth Ranch Airport",
		City:      "Ridgeview",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 2250,
		Latitude:  45.1291999817,
		Longitude: -100.7559967041,
		Timezone:  "America/Denver",
	},
	"SD37": {
		ICAO:      "SD37",
		Name:      "Fiedler Airport",
		City:      "Selby",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1950,
		Latitude:  45.4961013794,
		Longitude: -100.0039978027,
		Timezone:  "America/Chicago",
	},
	"SD41": {
		ICAO:      "SD41",
		Name:      "Shambo Ranch Airport",
		City:      "Lemmon",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 2500,
		Latitude:  45.5332984924,
		Longitude: -101.8339996338,
		Timezone:  "America/Denver",
	},
	"SD42": {
		ICAO:      "SD42",
		Name:      "Waltner & Richards Airport",
		City:      "Bridgewater",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1403,
		Latitude:  43.5666007996,
		Longitude: -97.5336990356,
		Timezone:  "America/Chicago",
	},
	"SD43": {
		ICAO:      "SD43",
		Name:      "Ingle Airport",
		City:      "Cavour",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1305,
		Latitude:  44.2397003174,
		Longitude: -98.049697876,
		Timezone:  "America/Chicago",
	},
	"SD44": {
		ICAO:      "SD44",
		Name:      "Cook Field",
		City:      "Chamberlain",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1678,
		Latitude:  43.7832984924,
		Longitude: -99.257598877,
		Timezone:  "America/Chicago",
	},
	"SD46": {
		ICAO:      "SD46",
		Name:      "Jensen Airport",
		City:      "Dell Rapids",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1520,
		Latitude:  43.8333015442,
		Longitude: -96.8503036499,
		Timezone:  "America/Chicago",
	},
	"SD47": {
		ICAO:      "SD47",
		Name:      "Hunt Field",
		City:      "Eagle Butte",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 2440,
		Latitude:  45.0027999878,
		Longitude: -101.2129974365,
		Timezone:  "America/Denver",
	},
	"SD48": {
		ICAO:      "SD48",
		Name:      "Blomberg 42 Ranch Private Airport",
		City:      "Faith",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 2610,
		Latitude:  45.2000007629,
		Longitude: -102.6500015259,
		Timezone:  "America/Denver",
	},
	"SD49": {
		ICAO:      "SD49",
		Name:      "Hite Private Airport",
		City:      "Ferney",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1320,
		Latitude:  45.2872009277,
		Longitude: -98.0542984009,
		Timezone:  "America/Chicago",
	},
	"SD50": {
		ICAO:      "SD50",
		Name:      "Harrold Municipal Airport",
		City:      "Harrold",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1787,
		Latitude:  44.5308990479,
		Longitude: -99.7474975586,
		Timezone:  "America/Chicago",
	},
	"SD53": {
		ICAO:      "SD53",
		Name:      "Howard Field",
		City:      "Beresford",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1250,
		Latitude:  42.9063987732,
		Longitude: -96.810798645,
		Timezone:  "America/Chicago",
	},
	"SD55": {
		ICAO:      "SD55",
		Name:      "Winter Airfield",
		City:      "Huron",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1323,
		Latitude:  44.3157997131,
		Longitude: -98.2733001709,
		Timezone:  "America/Chicago",
	},
	"SD56": {
		ICAO:      "SD56",
		Name:      "Ralph Myers Airport",
		City:      "Onida",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1900,
		Latitude:  44.8666000366,
		Longitude: -99.6171035767,
		Timezone:  "America/Chicago",
	},
	"SD57": {
		ICAO:      "SD57",
		Name:      "W L Thompson Airport",
		City:      "Pierre",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 2113,
		Latitude:  44.5666007996,
		Longitude: -100.75,
		Timezone:  "America/Denver",
	},
	"SD59": {
		ICAO:      "SD59",
		Name:      "Carr Airport",
		City:      "Prairie City",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 2886,
		Latitude:  45.5332984924,
		Longitude: -102.8140029907,
		Timezone:  "America/Denver",
	},
	"SD60": {
		ICAO:      "SD60",
		Name:      "Ike John Private Airport",
		City:      "Strool/Prairie City",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 2756,
		Latitude:  45.4000015259,
		Longitude: -102.7959976196,
		Timezone:  "America/Denver",
	},
	"SD61": {
		ICAO:      "SD61",
		Name:      "Plihal Farms Airport",
		City:      "Tyndall",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1435,
		Latitude:  43.0167007446,
		Longitude: -97.8669967651,
		Timezone:  "America/Chicago",
	},
	"SD62": {
		ICAO:      "SD62",
		Name:      "Schaller Airport",
		City:      "Verdon",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1320,
		Latitude:  45.25,
		Longitude: -98.1669998169,
		Timezone:  "America/Chicago",
	},
	"SD65": {
		ICAO:      "SD65",
		Name:      "Whipple Ranch Airport",
		City:      "Wilmot",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1462,
		Latitude:  45.4376983643,
		Longitude: -96.9998016357,
		Timezone:  "America/Chicago",
	},
	"SD66": {
		ICAO:      "SD66",
		Name:      "Cooks Airport",
		City:      "Cottonwood",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 2456,
		Latitude:  43.9733009338,
		Longitude: -101.952003479,
		Timezone:  "America/Denver",
	},
	"SD68": {
		ICAO:      "SD68",
		Name:      "Flying T Airport",
		City:      "Hot Springs",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 3675,
		Latitude:  43.3847007751,
		Longitude: -103.4309997559,
		Timezone:  "America/Denver",
	},
	"SD71": {
		ICAO:      "SD71",
		Name:      "Bogner Field",
		City:      "Oelrichs",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 3660,
		Latitude:  43.1166000366,
		Longitude: -103.3669967651,
		Timezone:  "America/Denver",
	},
	"SD72": {
		ICAO:      "SD72",
		Name:      "Vig Ranch Airfield",
		City:      "Opal",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 2600,
		Latitude:  44.8499984741,
		Longitude: -102.5839996338,
		Timezone:  "America/Denver",
	},
	"SD73": {
		ICAO:      "SD73",
		Name:      "Staben Strip",
		City:      "Philip",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 2227,
		Latitude:  44.5208015442,
		Longitude: -101.5930023193,
		Timezone:  "America/Denver",
	},
	"SD74": {
		ICAO:      "SD74",
		Name:      "Vander Wal Private Airport",
		City:      "Pollock",
		State:     "North-Dakota",
		Country:   "US",
		Elevation: 1955,
		Latitude:  45.9954986572,
		Longitude: -100.3759994507,
		Timezone:  "America/Chicago",
	},
	"SD75": {
		ICAO:      "SD75",
		Name:      "Livingston Airport",
		City:      "Stickney",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1641,
		Latitude:  43.5910987854,
		Longitude: -98.452003479,
		Timezone:  "America/Chicago",
	},
	"SD76": {
		ICAO:      "SD76",
		Name:      "Tennant Ranch Airport",
		City:      "Camp Crook",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 3090,
		Latitude:  45.8293991089,
		Longitude: -103.9449996948,
		Timezone:  "America/Denver",
	},
	"SD77": {
		ICAO:      "SD77",
		Name:      "Barber Strip",
		City:      "Rapid City",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 3200,
		Latitude:  44.0003013611,
		Longitude: -103.0179977417,
		Timezone:  "America/Denver",
	},
	"SD78": {
		ICAO:      "SD78",
		Name:      "Anderson Aerial Spraying Airport",
		City:      "Kennebec",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1750,
		Latitude:  43.9015007019,
		Longitude: -99.8788986206,
		Timezone:  "America/Chicago",
	},
	"SD79": {
		ICAO:      "SD79",
		Name:      "Monty Harer Airstrip",
		City:      "Hoven",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1860,
		Latitude:  45.2014007568,
		Longitude: -99.8361968994,
		Timezone:  "America/Chicago",
	},
	"SD81": {
		ICAO:      "SD81",
		Name:      "Vivian Airport",
		City:      "Vivian",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1920,
		Latitude:  43.9314002991,
		Longitude: -100.3150024414,
		Timezone:  "America/Chicago",
	},
	"SD82": {
		ICAO:      "SD82",
		Name:      "Webster-Eneboe Airstrip",
		City:      "Wilmot",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1160,
		Latitude:  45.4249992371,
		Longitude: -96.8669967651,
		Timezone:  "America/Chicago",
	},
	"SD83": {
		ICAO:      "SD83",
		Name:      "Lee Airport",
		City:      "De Smet",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1555,
		Latitude:  44.2155990601,
		Longitude: -97.6074981689,
		Timezone:  "America/Chicago",
	},
	"SD84": {
		ICAO:      "SD84",
		Name:      "Marone Airport",
		City:      "Huron",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1285,
		Latitude:  44.2658004761,
		Longitude: -98.1203994751,
		Timezone:  "America/Chicago",
	},
	"SD88": {
		ICAO:      "SD88",
		Name:      "Dunn Airport",
		City:      "Bison",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 2467,
		Latitude:  45.3890991211,
		Longitude: -102.0029983521,
		Timezone:  "America/Denver",
	},
	"SD89": {
		ICAO:      "SD89",
		Name:      "Priebe Landing Strip",
		City:      "Pukwana",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1628,
		Latitude:  43.7705993652,
		Longitude: -99.2074966431,
		Timezone:  "America/Chicago",
	},
	"SD90": {
		ICAO:      "SD90",
		Name:      "Mitchell's Strip",
		City:      "Spearfish",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 3920,
		Latitude:  44.4672012329,
		Longitude: -103.7870025635,
		Timezone:  "America/Denver",
	},
	"SD93": {
		ICAO:      "SD93",
		Name:      "Gary Myers Airport",
		City:      "Gettysburg",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1930,
		Latitude:  44.864200592,
		Longitude: -99.6063995361,
		Timezone:  "America/Chicago",
	},
	"SD94": {
		ICAO:      "SD94",
		Name:      "Lundin Airport",
		City:      "Revillo",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1186,
		Latitude:  44.9930000305,
		Longitude: -96.4769973755,
		Timezone:  "America/Chicago",
	},
	"SD97": {
		ICAO:      "SD97",
		Name:      "Oller Airport",
		City:      "Vivian",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 2100,
		Latitude:  43.8777999878,
		Longitude: -100.3619995117,
		Timezone:  "America/Chicago",
	},
	"SD98": {
		ICAO:      "SD98",
		Name:      "Barber Airport",
		City:      "Enning",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 2655,
		Latitude:  44.4625015259,
		Longitude: -102.5520019531,
		Timezone:  "America/Denver",
	},
	"SD99": {
		ICAO:      "SD99",
		Name:      "Glawe's Airport",
		City:      "Sioux Falls",
		State:     "South-Dakota",
		Country:   "US",
		Elevation: 1495,
		Latitude:  43.625,
		Longitude: -96.7502975464,
		Timezone:  "America/Chicago",
	},
	"SDAA": {
		ICAO:      "SDAA",
		Name:      "Araras Airport",
		City:      "Araras",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2247,
		Latitude:  -22.339099884,
		Longitude: -47.3592987061,
		Timezone:  "America/Sao_Paulo",
	},
	"SDAD": {
		ICAO:      "SDAD",
		Name:      "Everaldo Moras Barreto Airport",
		City:      "Adamantina",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1394,
		Latitude:  -21.6947994232,
		Longitude: -51.0973014832,
		Timezone:  "America/Sao_Paulo",
	},
	"SDAE": {
		ICAO:      "SDAE",
		Name:      "Sao Pedro Airport",
		City:      "Sao Pedro",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1858,
		Latitude:  -22.5836105347,
		Longitude: -47.8961105347,
		Timezone:  "America/Sao_Paulo",
	},
	"SDAF": {
		ICAO:      "SDAF",
		Name:      "Fazenda Palmeiras Airport",
		City:      "Analandia",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2762,
		Latitude:  -22.1567001343,
		Longitude: -47.7192993164,
		Timezone:  "America/Sao_Paulo",
	},
	"SDAG": {
		ICAO:      "SDAG",
		Name:      "Angra dos Reis Airport",
		City:      "Angra Dos Reis",
		State:     "Rio-de-Janeiro",
		Country:   "BR",
		Elevation: 10,
		Latitude:  -22.9752998352,
		Longitude: -44.3070983887,
		Timezone:  "America/Sao_Paulo",
	},
	"SDAI": {
		ICAO:      "SDAI",
		Name:      "Americana Airport",
		City:      "Americana",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2085,
		Latitude:  -22.7558002472,
		Longitude: -47.2694015503,
		Timezone:  "America/Sao_Paulo",
	},
	"SDAJ": {
		ICAO:      "SDAJ",
		Name:      "Fazenda Santa Marina Airport",
		City:      "Santo Antonio Do Aracangua",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1126,
		Latitude:  -20.8470993042,
		Longitude: -50.7879981995,
		Timezone:  "America/Sao_Paulo",
	},
	"SDAM": {
		ICAO:      "SDAM",
		IATA:      "CPQ",
		Name:      "Amarais Airport",
		City:      "Campinas",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2008,
		Latitude:  -22.8591995239,
		Longitude: -47.1082000732,
		Timezone:  "America/Sao_Paulo",
	},
	"SDAN": {
		ICAO:      "SDAN",
		Name:      "Fazenda Santo Angelo Airport",
		City:      "Brotas",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1680,
		Latitude:  -22.2455558777,
		Longitude: -48.2577781677,
		Timezone:  "America/Sao_Paulo",
	},
	"SDAS": {
		ICAO:      "SDAS",
		Name:      "Fazenda Santo Antonio Airport",
		City:      "Mirandopolis",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1141,
		Latitude:  -21.2756996155,
		Longitude: -51.2831993103,
		Timezone:  "America/Sao_Paulo",
	},
	"SDAY": {
		ICAO:      "SDAY",
		IATA:      "0",
		Name:      "Fazenda Altair Airport",
		City:      "Santa Ernestina",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2034,
		Latitude:  -21.4671993256,
		Longitude: -48.3671989441,
		Timezone:  "America/Sao_Paulo",
	},
	"SDBA": {
		ICAO:      "SDBA",
		Name:      "Batatais Airport",
		City:      "Batatais",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2890,
		Latitude:  -20.8772220612,
		Longitude: -47.5855560303,
		Timezone:  "America/Sao_Paulo",
	},
	"SDBB": {
		ICAO:      "SDBB",
		Name:      "Bebedouro Airport",
		City:      "Bebedouro",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1942,
		Latitude:  -20.9004001617,
		Longitude: -48.4733009338,
		Timezone:  "America/Sao_Paulo",
	},
	"SDBI": {
		ICAO:      "SDBI",
		Name:      "Aeroclube Airport",
		City:      "Birigui",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1394,
		Latitude:  -21.2819442749,
		Longitude: -50.3269424438,
		Timezone:  "America/Sao_Paulo",
	},
	"SDBK": {
		ICAO:      "SDBK",
		IATA:      "QCJ",
		Name:      "Botucatu Airport",
		City:      "Botucatu",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 3012,
		Latitude:  -22.9395008087,
		Longitude: -48.4679985046,
		Timezone:  "America/Sao_Paulo",
	},
	"SDBN": {
		ICAO:      "SDBN",
		Name:      "Fazenda Bonanza Airport",
		City:      "Salto De Pirapora",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2316,
		Latitude:  -23.6861114502,
		Longitude: -47.5574989319,
		Timezone:  "America/Sao_Paulo",
	},
	"SDBV": {
		ICAO:      "SDBV",
		Name:      "Balneario Sao Januario Airport",
		City:      "Ilha Comprida",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 26,
		Latitude:  -24.8708324432,
		Longitude: -47.7583312988,
		Timezone:  "America/Sao_Paulo",
	},
	"SDBY": {
		ICAO:      "SDBY",
		IATA:      "0",
		Name:      "Bariri Airport",
		City:      "Bariri",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1594,
		Latitude:  -22.0692005157,
		Longitude: -48.706401825,
		Timezone:  "America/Sao_Paulo",
	},
	"SDCA": {
		ICAO:      "SDCA",
		Name:      "Capao Bonito Airport",
		City:      "Capao Bonito",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2402,
		Latitude:  -24.0361995697,
		Longitude: -48.3567008972,
		Timezone:  "America/Sao_Paulo",
	},
	"SDCD": {
		ICAO:      "SDCD",
		Name:      "Catanduva Airport",
		City:      "Catanduva",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1841,
		Latitude:  -21.1492004395,
		Longitude: -48.9887008667,
		Timezone:  "America/Sao_Paulo",
	},
	"SDCE": {
		ICAO:      "SDCE",
		Name:      "Fazenda Marambaia Airport",
		City:      "Pirapora",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1766,
		Latitude:  -17.4383335114,
		Longitude: -44.9369430542,
		Timezone:  "America/Sao_Paulo",
	},
	"SDCG": {
		ICAO:      "SDCG",
		IATA:      "OLC",
		Name:      "Senadora Eunice Micheles Airport",
		City:      "Sao Paulo De Olivenca",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 335,
		Latitude:  -3.4679295077,
		Longitude: -68.9204120636,
		Timezone:  "America/Manaus",
	},
	"SDCL": {
		ICAO:      "SDCL",
		Name:      "Fazenda Campo Alegre Airport",
		City:      "Cerqueira Cesar",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2182,
		Latitude:  -23.1541671753,
		Longitude: -49.2088890076,
		Timezone:  "America/Sao_Paulo",
	},
	"SDCM": {
		ICAO:      "SDCM",
		Name:      "Sete Povos Airport",
		City:      "Sao Desiderio",
		Country:   "BR",
		Elevation: 2957,
		Latitude:  -12.8633327484,
		Longitude: -46.1624984741,
		Timezone:  "America/Bahia",
	},
	"SDCO": {
		ICAO:      "SDCO",
		IATA:      "SOD",
		Name:      "Sorocaba Airport",
		City:      "Sorocaba",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2077,
		Latitude:  -23.4780006409,
		Longitude: -47.4900016785,
		Timezone:  "America/Sao_Paulo",
	},
	"SDCR": {
		ICAO:      "SDCR",
		Name:      "Fazenda Caicara Airport",
		City:      "Itanhaem",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 36,
		Latitude:  -24.0711116791,
		Longitude: -46.8294448853,
		Timezone:  "America/Sao_Paulo",
	},
	"SDCT": {
		ICAO:      "SDCT",
		Name:      "Fazenda Santo Antonio do Oricanga Airport",
		City:      "Estiva Gerbi",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2237,
		Latitude:  -22.2286109924,
		Longitude: -46.9700012207,
		Timezone:  "America/Sao_Paulo",
	},
	"SDCU": {
		ICAO:      "SDCU",
		Name:      "Brigadeiro Francisco Pinto de Moura Airport",
		City:      "Casemiro De Abreu",
		State:     "Rio-de-Janeiro",
		Country:   "BR",
		Elevation: 18,
		Latitude:  -22.5541667938,
		Longitude: -42.1150016785,
		Timezone:  "America/Sao_Paulo",
	},
	"SDCW": {
		ICAO:      "SDCW",
		Name:      "Fazenda Paloma Airport",
		City:      "Colombia",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1706,
		Latitude:  -20.3261108398,
		Longitude: -48.8283348083,
		Timezone:  "America/Sao_Paulo",
	},
	"SDCZ": {
		ICAO:      "SDCZ",
		Name:      "Fazenda Bom Jardim Airport",
		City:      "Mangaratiba",
		State:     "Rio-de-Janeiro",
		Country:   "BR",
		Elevation: 16,
		Latitude:  -22.9361114502,
		Longitude: -44.0986099243,
		Timezone:  "America/Sao_Paulo",
	},
	"SDDB": {
		ICAO:      "SDDB",
		Name:      "Maggi Agropecuaria Airport",
		City:      "Campos Novos Paulista",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1703,
		Latitude:  -22.6124992371,
		Longitude: -50.0955543518,
		Timezone:  "America/Sao_Paulo",
	},
	"SDDJ": {
		ICAO:      "SDDJ",
		IATA:      "0",
		Name:      "Fazenda Santa Maria Airport",
		City:      "Moncoes",
		Country:   "BR",
		Elevation: 1525,
		Latitude:  -20.8488998413,
		Longitude: -50.0793991089,
		Timezone:  "America/Sao_Paulo",
	},
	"SDDK": {
		ICAO:      "SDDK",
		Name:      "Saint-Exupery Airport",
		City:      "Ocaucu",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1886,
		Latitude:  -22.4827785492,
		Longitude: -49.9838905334,
		Timezone:  "America/Sao_Paulo",
	},
	"SDDL": {
		ICAO:      "SDDL",
		Name:      "Fazenda Dois Lagos Airport",
		City:      "Tatui",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1916,
		Latitude:  -23.3075008392,
		Longitude: -47.9338874817,
		Timezone:  "America/Sao_Paulo",
	},
	"SDDN": {
		ICAO:      "SDDN",
		Name:      "Andradina Airport",
		City:      "Andradina",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1247,
		Latitude:  -20.9249992371,
		Longitude: -51.3819999695,
		Timezone:  "America/Sao_Paulo",
	},
	"SDDR": {
		ICAO:      "SDDR",
		IATA:      "QDC",
		Name:      "Dracena Airport",
		City:      "Dracena",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1220,
		Latitude:  -21.4605007172,
		Longitude: -51.6068992615,
		Timezone:  "America/Sao_Paulo",
	},
	"SDDV": {
		ICAO:      "SDDV",
		IATA:      "0",
		Name:      "Usina Catanduva Airport",
		City:      "Palmares Paulista",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1860,
		Latitude:  -21.1268997192,
		Longitude: -48.8446998596,
		Timezone:  "America/Sao_Paulo",
	},
	"SDDX": {
		ICAO:      "SDDX",
		Name:      "Fazenda Paiquere Airport",
		City:      "Populina",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1483,
		Latitude:  -19.8880558014,
		Longitude: -50.5141677856,
		Timezone:  "America/Sao_Paulo",
	},
	"SDEC": {
		ICAO:      "SDEC",
		Name:      "Rancho Enagri Airport",
		City:      "Alto Garcas",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2572,
		Latitude:  -16.95611,
		Longitude: -53.57,
		Timezone:  "America/Cuiaba",
	},
	"SDED": {
		ICAO:      "SDED",
		Name:      "Edra do Brasil Airport",
		City:      "Ipeuna",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2008,
		Latitude:  -22.4402770996,
		Longitude: -47.7016677856,
		Timezone:  "America/Sao_Paulo",
	},
	"SDEE": {
		ICAO:      "SDEE",
		Name:      "Rancho Villela Airport",
		City:      "Itai",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1801,
		Latitude:  -23.5702781677,
		Longitude: -48.8633346558,
		Timezone:  "America/Sao_Paulo",
	},
	"SDEG": {
		ICAO:      "SDEG",
		Name:      "Fazenda Nova California Airport",
		City:      "Martinopolis",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1450,
		Latitude:  -22.4666996002,
		Longitude: -51.2008018494,
		Timezone:  "America/Sao_Paulo",
	},
	"SDEJ": {
		ICAO:      "SDEJ",
		Name:      "Fazenda Guara do Pinhal Airport",
		City:      "Itapetininga",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2036,
		Latitude:  -23.6658325195,
		Longitude: -48.0736122131,
		Timezone:  "America/Sao_Paulo",
	},
	"SDEM": {
		ICAO:      "SDEM",
		Name:      "Estancia Machado Airport",
		City:      "Alvares Machado",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1388,
		Latitude:  -22.1011104584,
		Longitude: -51.4516677856,
		Timezone:  "America/Sao_Paulo",
	},
	"SDEO": {
		ICAO:      "SDEO",
		Name:      "Fazenda Entre Rios Airport",
		City:      "Ibitinga",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1709,
		Latitude:  -21.8785991669,
		Longitude: -48.6594009399,
		Timezone:  "America/Sao_Paulo",
	},
	"SDEP": {
		ICAO:      "SDEP",
		Name:      "Presidente Epitacio Airport",
		City:      "Presidente Epitacio",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 974,
		Latitude:  -21.7742996216,
		Longitude: -52.1437988281,
		Timezone:  "America/Campo_Grande",
	},
	"SDER": {
		ICAO:      "SDER",
		Name:      "Fazenda Entre Rios Airport",
		City:      "Sud Menucci",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1207,
		Latitude:  -20.7649993896,
		Longitude: -50.8569450378,
		Timezone:  "America/Sao_Paulo",
	},
	"SDES": {
		ICAO:      "SDES",
		Name:      "Estancia Colorado Airport",
		City:      "Sales De Oliveira",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2372,
		Latitude:  -20.7602787018,
		Longitude: -47.8619422913,
		Timezone:  "America/Sao_Paulo",
	},
	"SDET": {
		ICAO:      "SDET",
		Name:      "Tiete Airport",
		City:      "Tiete",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1644,
		Latitude:  -23.1047210693,
		Longitude: -47.724445343,
		Timezone:  "America/Sao_Paulo",
	},
	"SDFA": {
		ICAO:      "SDFA",
		Name:      "Fazenda Bonanca Airport",
		City:      "Pereira Barreto",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1200,
		Latitude:  -20.676399231,
		Longitude: -51.0307998657,
		Timezone:  "America/Sao_Paulo",
	},
	"SDFB": {
		ICAO:      "SDFB",
		IATA:      "0",
		Name:      "Fazenda Brumado Airport",
		City:      "Barretos",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1755,
		Latitude:  -20.4230995178,
		Longitude: -48.567199707,
		Timezone:  "America/Sao_Paulo",
	},
	"SDFC": {
		ICAO:      "SDFC",
		IATA:      "0",
		Name:      "Fazenda Constancia Airport",
		City:      "Altair",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1673,
		Latitude:  -20.4885997772,
		Longitude: -49.1880989075,
		Timezone:  "America/Sao_Paulo",
	},
	"SDFD": {
		ICAO:      "SDFD",
		Name:      "Fernandopolis Airport",
		City:      "Fernandopolis",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1640,
		Latitude:  -20.276599884,
		Longitude: -50.2155990601,
		Timezone:  "America/Sao_Paulo",
	},
	"SDFE": {
		ICAO:      "SDFE",
		Name:      "Fazenda Santo Antonio Airport",
		City:      "Bela Vista",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 633,
		Latitude:  -22.0680561066,
		Longitude: -56.7024993896,
		Timezone:  "America/Campo_Grande",
	},
	"SDFF": {
		ICAO:      "SDFF",
		Name:      "Fazenda Fittipaldi Citrus Airport",
		City:      "Araraquara",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1673,
		Latitude:  -21.8544445038,
		Longitude: -48.2958335876,
		Timezone:  "America/Sao_Paulo",
	},
	"SDFH": {
		ICAO:      "SDFH",
		Name:      "Fazenda Amalia Airport",
		City:      "Santa Rosa Do Viterbo",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2318,
		Latitude:  -21.4440002441,
		Longitude: -47.3708992004,
		Timezone:  "America/Sao_Paulo",
	},
	"SDFK": {
		ICAO:      "SDFK",
		Name:      "Fazenda Figueira Airport",
		City:      "Maraba Paulista",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1234,
		Latitude:  -22.0688896179,
		Longitude: -52.1588897705,
		Timezone:  "America/Sao_Paulo",
	},
	"SDFL": {
		ICAO:      "SDFL",
		Name:      "Fazenda Santa Albana Airport",
		City:      "Itapetininga",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2051,
		Latitude:  -23.5711116791,
		Longitude: -48.2833328247,
		Timezone:  "America/Sao_Paulo",
	},
	"SDFM": {
		ICAO:      "SDFM",
		Name:      "Fazenda Santa Maria Airport",
		City:      "Teodoro Sampaio",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1083,
		Latitude:  -22.2888889313,
		Longitude: -52.2558326721,
		Timezone:  "America/Sao_Paulo",
	},
	"SDFN": {
		ICAO:      "SDFN",
		Name:      "Fazenda Sao Francisco Airport",
		City:      "Fartura",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1703,
		Latitude:  -23.4750003815,
		Longitude: -49.543888092,
		Timezone:  "America/Sao_Paulo",
	},
	"SDFT": {
		ICAO:      "SDFT",
		Name:      "Virgulino de Oliveira Airport",
		City:      "Itapira",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2152,
		Latitude:  -22.4144439697,
		Longitude: -46.8088874817,
		Timezone:  "America/Sao_Paulo",
	},
	"SDFU": {
		ICAO:      "SDFU",
		Name:      "Fazenda Ponte Funda Airport",
		City:      "Presidente Eptacio",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 984,
		Latitude:  -21.8705558777,
		Longitude: -52.1630554199,
		Timezone:  "America/Sao_Paulo",
	},
	"SDFX": {
		ICAO:      "SDFX",
		Name:      "Casa Nova Airport",
		City:      "Casa Nova",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1247,
		Latitude:  -9.1570901871,
		Longitude: -40.9376983643,
		Timezone:  "America/Bahia",
	},
	"SDFY": {
		ICAO:      "SDFY",
		Name:      "Fazenda Ipameri Airport",
		City:      "Osvaldo Cruz",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1280,
		Latitude:  -21.6313896179,
		Longitude: -50.8088874817,
		Timezone:  "America/Sao_Paulo",
	},
	"SDGB": {
		ICAO:      "SDGB",
		Name:      "Usina Colorado Airport",
		City:      "Guaira",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1988,
		Latitude:  -20.2769432068,
		Longitude: -48.1819458008,
		Timezone:  "America/Sao_Paulo",
	},
	"SDGC": {
		ICAO:      "SDGC",
		IATA:      "0",
		Name:      "Garca Airport",
		City:      "Garca",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2182,
		Latitude:  -22.183599472,
		Longitude: -49.6561012268,
		Timezone:  "America/Sao_Paulo",
	},
	"SDGF": {
		ICAO:      "SDGF",
		Name:      "Fazenda Fartura Airport",
		City:      "Ocaucu",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1647,
		Latitude:  -22.3672218323,
		Longitude: -49.9469451904,
		Timezone:  "America/Sao_Paulo",
	},
	"SDGJ": {
		ICAO:      "SDGJ",
		Name:      "Fazenda Suissa Airport",
		City:      "Guaimbe",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1748,
		Latitude:  -21.9122219086,
		Longitude: -49.8574981689,
		Timezone:  "America/Sao_Paulo",
	},
	"SDGK": {
		ICAO:      "SDGK",
		Name:      "Fazenda Gurucaia Airport",
		City:      "Guararapes",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1345,
		Latitude:  -21.4947223663,
		Longitude: -50.7338905334,
		Timezone:  "America/Sao_Paulo",
	},
	"SDGP": {
		ICAO:      "SDGP",
		Name:      "Hotel Fazenda Capricho Airport",
		City:      "Avare",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2099,
		Latitude:  -23.2244434357,
		Longitude: -48.7961120605,
		Timezone:  "America/Sao_Paulo",
	},
	"SDGR": {
		ICAO:      "SDGR",
		Name:      "Guararapes Airport",
		City:      "Guararapes",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1312,
		Latitude:  -21.2630558014,
		Longitude: -50.6530570984,
		Timezone:  "America/Sao_Paulo",
	},
	"SDGT": {
		ICAO:      "SDGT",
		Name:      "Fazenda Alvorada Airport",
		City:      "Campos Novos Paulista",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1895,
		Latitude:  -22.5249996185,
		Longitude: -50.0458335876,
		Timezone:  "America/Sao_Paulo",
	},
	"SDGU": {
		ICAO:      "SDGU",
		Name:      "Usina Bonfim Airport",
		City:      "Guariba",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2116,
		Latitude:  -21.4044437408,
		Longitude: -48.3033332825,
		Timezone:  "America/Sao_Paulo",
	},
	"SDGY": {
		ICAO:      "SDGY",
		Name:      "Fazenda Anhangai Airport",
		City:      "Aracatuba",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1181,
		Latitude:  -20.9274997711,
		Longitude: -50.7433319092,
		Timezone:  "America/Sao_Paulo",
	},
	"SDHC": {
		ICAO:      "SDHC",
		Name:      "Fazenda Koga Airport",
		City:      "Getulina",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1292,
		Latitude:  -21.8719444275,
		Longitude: -49.9802780151,
		Timezone:  "America/Sao_Paulo",
	},
	"SDHE": {
		ICAO:      "SDHE",
		Name:      "Fazenda Herdade Airport",
		City:      "Itapira",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2494,
		Latitude:  -22.4925003052,
		Longitude: -46.8436126709,
		Timezone:  "America/Sao_Paulo",
	},
	"SDIB": {
		ICAO:      "SDIB",
		Name:      "Irmaos Ribeiro Airport",
		City:      "Espirito Santo Do Pinhal",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 3018,
		Latitude:  -22.1581001282,
		Longitude: -46.7752990723,
		Timezone:  "America/Sao_Paulo",
	},
	"SDID": {
		ICAO:      "SDID",
		Name:      "Itarare Airport",
		City:      "Itarare",
		State:     "Parana",
		Country:   "BR",
		Elevation: 2598,
		Latitude:  -24.1352787018,
		Longitude: -49.3283348083,
		Timezone:  "America/Sao_Paulo",
	},
	"SDIG": {
		ICAO:      "SDIG",
		Name:      "Ibitinga Airport",
		City:      "Ibitinga",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1780,
		Latitude:  -21.7472991943,
		Longitude: -48.8557014465,
		Timezone:  "America/Sao_Paulo",
	},
	"SDIH": {
		ICAO:      "SDIH",
		Name:      "Fazenda Irohy Airport",
		City:      "Biritiba-Mirim",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2467,
		Latitude:  -23.5655555725,
		Longitude: -46.069442749,
		Timezone:  "America/Sao_Paulo",
	},
	"SDIL": {
		ICAO:      "SDIL",
		Name:      "Fazenda Pedra Branca Airport",
		City:      "Angra Dos Reis",
		State:     "Rio-de-Janeiro",
		Country:   "BR",
		Elevation: 10,
		Latitude:  -22.9034004211,
		Longitude: -44.2890014648,
		Timezone:  "America/Sao_Paulo",
	},
	"SDIM": {
		ICAO:      "SDIM",
		Name:      "Itanhaem Airport",
		City:      "Itanhaem",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 13,
		Latitude:  -24.1647224426,
		Longitude: -46.7855567932,
		Timezone:  "America/Sao_Paulo",
	},
	"SDIN": {
		ICAO:      "SDIN",
		Name:      "Clube do Ceu Airport",
		City:      "Rio De Janeiro",
		State:     "Rio-de-Janeiro",
		Country:   "BR",
		Elevation: 3,
		Latitude:  -22.9974994659,
		Longitude: -43.4013900757,
		Timezone:  "America/Sao_Paulo",
	},
	"SDIO": {
		ICAO:      "SDIO",
		Name:      "Aeroclube de Itapolis Airport",
		City:      "Itapolis",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1739,
		Latitude:  -21.5997219086,
		Longitude: -48.8327789307,
		Timezone:  "America/Sao_Paulo",
	},
	"SDIP": {
		ICAO:      "SDIP",
		Name:      "Fazenda Centro de Voo a Vela Ipua Airport",
		City:      "Cacapava",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1913,
		Latitude:  -23.0433330536,
		Longitude: -45.775554657,
		Timezone:  "America/Sao_Paulo",
	},
	"SDIU": {
		ICAO:      "SDIU",
		Name:      "Itu Airport",
		City:      "Itu",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2231,
		Latitude:  -23.3497219086,
		Longitude: -47.3277778625,
		Timezone:  "America/Sao_Paulo",
	},
	"SDIV": {
		ICAO:      "SDIV",
		Name:      "Ituverava Airport",
		City:      "Ituverava",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2044,
		Latitude:  -20.3754997253,
		Longitude: -47.768699646,
		Timezone:  "America/Sao_Paulo",
	},
	"SDIZ": {
		ICAO:      "SDIZ",
		Name:      "Fazenda Sao Paulo Airport",
		City:      "Cardoso",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1656,
		Latitude:  -20.1555557251,
		Longitude: -49.9347229004,
		Timezone:  "America/Sao_Paulo",
	},
	"SDJA": {
		ICAO:      "SDJA",
		Name:      "Doutor Jose Augusto de Arruda Botelho Airport",
		City:      "Itirapina",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2569,
		Latitude:  -22.19472,
		Longitude: -47.86167,
		Timezone:  "America/Sao_Paulo",
	},
	"SDJC": {
		ICAO:      "SDJC",
		IATA:      "0",
		Name:      "Jaboticabal Airport",
		City:      "Jaboticabal",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2024,
		Latitude:  -21.2299995422,
		Longitude: -48.2846984863,
		Timezone:  "America/Sao_Paulo",
	},
	"SDJF": {
		ICAO:      "SDJF",
		Name:      "Fazenda Jequitiba Airport",
		City:      "Buri",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2310,
		Latitude:  -23.8452777863,
		Longitude: -48.4480552673,
		Timezone:  "America/Sao_Paulo",
	},
	"SDJG": {
		ICAO:      "SDJG",
		Name:      "Fazenda Ponte Preta Airport",
		City:      "Santa Cruz Do Rio Pardo",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1689,
		Latitude:  -22.7333335876,
		Longitude: -49.4833335876,
		Timezone:  "America/Sao_Paulo",
	},
	"SDJL": {
		ICAO:      "SDJL",
		IATA:      "JLS",
		Name:      "Jales Airport",
		City:      "Jales",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1496,
		Latitude:  -20.2929992676,
		Longitude: -50.5463981628,
		Timezone:  "America/Sao_Paulo",
	},
	"SDJN": {
		ICAO:      "SDJN",
		Name:      "Fazenda Sao Luiz Airport",
		City:      "Jardinopolis",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1772,
		Latitude:  -21.1005554199,
		Longitude: -47.7505569458,
		Timezone:  "America/Sao_Paulo",
	},
	"SDJO": {
		ICAO:      "SDJO",
		IATA:      "0",
		Name:      "Sao Joaquim da Barra Airport",
		City:      "Sao Joaquim Da Barra",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2136,
		Latitude:  -20.5932998657,
		Longitude: -47.8422012329,
		Timezone:  "America/Sao_Paulo",
	},
	"SDJQ": {
		ICAO:      "SDJQ",
		Name:      "Fazenda Sao Joaquim O.B. Airport",
		City:      "Pereira Barreto",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1276,
		Latitude:  -20.8283996582,
		Longitude: -51.0079994202,
		Timezone:  "America/Sao_Paulo",
	},
	"SDJR": {
		ICAO:      "SDJR",
		Name:      "Fazenda Santa Ines Airport",
		City:      "Sao Jose Do Rio Preto",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1804,
		Latitude:  -20.8824996948,
		Longitude: -49.3925018311,
		Timezone:  "America/Sao_Paulo",
	},
	"SDJV": {
		ICAO:      "SDJV",
		Name:      "Sao Joao da Boa Vista Airport",
		City:      "Sao Joao Da Boa Vista",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2500,
		Latitude:  -22.0172214508,
		Longitude: -46.8408317566,
		Timezone:  "America/Sao_Paulo",
	},
	"SDJZ": {
		ICAO:      "SDJZ",
		IATA:      "0",
		Name:      "Fazenda Barra do Agudo Airport",
		City:      "Morro Agudo",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1608,
		Latitude:  -20.8083000183,
		Longitude: -48.2219009399,
		Timezone:  "America/Sao_Paulo",
	},
	"SDKB": {
		ICAO:      "SDKB",
		Name:      "Casa Branca Airport",
		City:      "Casa Branca",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2346,
		Latitude:  -21.7880554199,
		Longitude: -47.055557251,
		Timezone:  "America/Sao_Paulo",
	},
	"SDKC": {
		ICAO:      "SDKC",
		Name:      "Fazenda Eldorado Airport",
		City:      "Galia",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1771,
		Latitude:  -22.3036117554,
		Longitude: -49.462223053,
		Timezone:  "America/Sao_Paulo",
	},
	"SDKE": {
		ICAO:      "SDKE",
		Name:      "Coribe Airport",
		City:      "Coribe",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 2198,
		Latitude:  -13.7772216797,
		Longitude: -44.4644432068,
		Timezone:  "America/Bahia",
	},
	"SDKF": {
		ICAO:      "SDKF",
		Name:      "Juazeiro Airport",
		City:      "Curaca",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1220,
		Latitude:  -9.1555404663,
		Longitude: -40.0914001465,
		Timezone:  "America/Bahia",
	},
	"SDKG": {
		ICAO:      "SDKG",
		Name:      "Fazenda Gavea Airport",
		City:      "Cerqueira Cesar",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2031,
		Latitude:  -23.2250003815,
		Longitude: -49.1791687012,
		Timezone:  "America/Sao_Paulo",
	},
	"SDKJ": {
		ICAO:      "SDKJ",
		Name:      "Formosa do Rio Preto Airport",
		City:      "Formosa Do Rio Preto",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1598,
		Latitude:  -11.0215997696,
		Longitude: -45.1866989136,
		Timezone:  "America/Bahia",
	},
	"SDKK": {
		ICAO:      "SDKK",
		IATA:      "QOA",
		Name:      "Mococa Airport",
		City:      "Mococa",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2116,
		Latitude:  -21.4878005981,
		Longitude: -47.0344009399,
		Timezone:  "America/Sao_Paulo",
	},
	"SDKN": {
		ICAO:      "SDKN",
		Name:      "Fazenda e Haras Bela Vista Airport",
		City:      "Mineiros Do Tiete",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2339,
		Latitude:  -22.4380569458,
		Longitude: -48.4055557251,
		Timezone:  "America/Sao_Paulo",
	},
	"SDKO": {
		ICAO:      "SDKO",
		Name:      "Fazenda Campo Grande Airport",
		City:      "Colombia",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1830,
		Latitude:  -20.2576999664,
		Longitude: -48.7886009216,
		Timezone:  "America/Sao_Paulo",
	},
	"SDKP": {
		ICAO:      "SDKP",
		Name:      "Fazenda Jamaica Airport",
		City:      "Pompeia",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1611,
		Latitude:  -22.1897220612,
		Longitude: -50.2788887024,
		Timezone:  "America/Sao_Paulo",
	},
	"SDKQ": {
		ICAO:      "SDKQ",
		Name:      "Fazenda Sao Joao Airport",
		City:      "Guaraci",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1673,
		Latitude:  -20.2166671753,
		Longitude: -49.0666656494,
		Timezone:  "America/Sao_Paulo",
	},
	"SDKR": {
		ICAO:      "SDKR",
		Name:      "Fazenda Triangulo Airport",
		City:      "Pontes E Lacerda",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1116,
		Latitude:  -15.1750001907,
		Longitude: -59.1711120605,
		Timezone:  "America/Cuiaba",
	},
	"SDKW": {
		ICAO:      "SDKW",
		Name:      "FazendaSodema Airport",
		City:      "Vera",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1181,
		Latitude:  -12.295278,
		Longitude: -55.536944,
		Timezone:  "America/Cuiaba",
	},
	"SDLC": {
		ICAO:      "SDLC",
		IATA:      "0",
		Name:      "Lucelia Airport",
		City:      "Lucelia",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1476,
		Latitude:  -21.7493991852,
		Longitude: -51.0182991028,
		Timezone:  "America/Sao_Paulo",
	},
	"SDLE": {
		ICAO:      "SDLE",
		Name:      "Rio de Contas Airport",
		City:      "Rio De Contas",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 3576,
		Latitude:  -13.5874996185,
		Longitude: -41.7891654968,
		Timezone:  "America/Bahia",
	},
	"SDLG": {
		ICAO:      "SDLG",
		Name:      "Salviano Inacio Rocha Airport",
		City:      "Ourolandia",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1890,
		Latitude:  -10.9744443893,
		Longitude: -41.069442749,
		Timezone:  "America/Bahia",
	},
	"SDLH": {
		ICAO:      "SDLH",
		Name:      "Caldas do Jorro Airport",
		City:      "Tucano",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 696,
		Latitude:  -11.0441665649,
		Longitude: -38.7841682434,
		Timezone:  "America/Bahia",
	},
	"SDLI": {
		ICAO:      "SDLI",
		Name:      "Abare Airport",
		City:      "Abare",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 955,
		Latitude:  -8.736989975,
		Longitude: -39.115398407,
		Timezone:  "America/Bahia",
	},
	"SDLJ": {
		ICAO:      "SDLJ",
		Name:      "Igarape Bahia Airport",
		City:      "Parauapebas",
		State:     "Para",
		Country:   "BR",
		Elevation: 2257,
		Latitude:  -6.0477800369,
		Longitude: -50.57970047,
		Timezone:  "America/Belem",
	},
	"SDLK": {
		ICAO:      "SDLK",
		Name:      "Cacule Airport",
		City:      "Cacule",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 2064,
		Latitude:  -14.4812002182,
		Longitude: -42.2647018433,
		Timezone:  "America/Bahia",
	},
	"SDLL": {
		ICAO:      "SDLL",
		IATA:      "0",
		Name:      "Leme Airport",
		City:      "Leme",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2024,
		Latitude:  -22.2269001007,
		Longitude: -47.3821983337,
		Timezone:  "America/Sao_Paulo",
	},
	"SDLN": {
		ICAO:      "SDLN",
		Name:      "Fazenda Tosana Airport",
		City:      "Cabo Frio",
		State:     "Rio-de-Janeiro",
		Country:   "BR",
		Elevation: 26,
		Latitude:  -22.5997219086,
		Longitude: -42.0288887024,
		Timezone:  "America/Sao_Paulo",
	},
	"SDLO": {
		ICAO:      "SDLO",
		Name:      "Fazenda Pontal Airport",
		City:      "Cairu",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 12,
		Latitude:  -13.5648002625,
		Longitude: -38.9399986267,
		Timezone:  "America/Bahia",
	},
	"SDLP": {
		ICAO:      "SDLP",
		IATA:      "QGC",
		Name:      "Lencois Paulista Airport",
		City:      "Lencois Paulista",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2039,
		Latitude:  -22.5783996582,
		Longitude: -48.7746009827,
		Timezone:  "America/Sao_Paulo",
	},
	"SDLQ": {
		ICAO:      "SDLQ",
		Name:      "Fazenda Perobal Airport",
		City:      "Guararapes",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1339,
		Latitude:  -21.503610611,
		Longitude: -50.7780570984,
		Timezone:  "America/Sao_Paulo",
	},
	"SDLR": {
		ICAO:      "SDLR",
		Name:      "Fazenda Sonho Dourado Airport",
		City:      "Valentim Gentil",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1722,
		Latitude:  -20.4672222137,
		Longitude: -50.1422233582,
		Timezone:  "America/Sao_Paulo",
	},
	"SDLU": {
		ICAO:      "SDLU",
		Name:      "Fazenda Santa Luiza Airport",
		City:      "Barra Bonita",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1782,
		Latitude:  -22.4831008911,
		Longitude: -48.5245018005,
		Timezone:  "America/Sao_Paulo",
	},
	"SDLV": {
		ICAO:      "SDLV",
		Name:      "Fazenda Sao Francisco Airport",
		City:      "Florestopolis",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1581,
		Latitude:  -22.9208335876,
		Longitude: -51.3519439697,
		Timezone:  "America/Sao_Paulo",
	},
	"SDLW": {
		ICAO:      "SDLW",
		Name:      "Fazenda Campo Real Airport",
		City:      "Campo Novo Do Parecis",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1804,
		Latitude:  -13.4469442368,
		Longitude: -57.7811126709,
		Timezone:  "America/Cuiaba",
	},
	"SDLX": {
		ICAO:      "SDLX",
		Name:      "Fazenda Jacareuna Airport",
		City:      "Sao Felix Do Araguaia",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1417,
		Latitude:  -11.441110611,
		Longitude: -52.2047233582,
		Timezone:  "America/Cuiaba",
	},
	"SDLY": {
		ICAO:      "SDLY",
		IATA:      "0",
		Name:      "Matao Airport",
		City:      "Matao",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2083,
		Latitude:  -21.6231002808,
		Longitude: -48.3525009155,
		Timezone:  "America/Sao_Paulo",
	},
	"SDMA": {
		ICAO:      "SDMA",
		Name:      "Fazenda Maristela Airport",
		City:      "Tremembe",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1804,
		Latitude:  -22.9541664124,
		Longitude: -45.6016654968,
		Timezone:  "America/Sao_Paulo",
	},
	"SDMC": {
		ICAO:      "SDMC",
		Name:      "Marica Airport",
		City:      "Marica",
		State:     "Rio-de-Janeiro",
		Country:   "BR",
		Elevation: 13,
		Latitude:  -22.919500351,
		Longitude: -42.8308982849,
		Timezone:  "America/Sao_Paulo",
	},
	"SDMD": {
		ICAO:      "SDMD",
		Name:      "Fazenda Santa Izabel Airport",
		City:      "Coxim",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1033,
		Latitude:  -18,
		Longitude: -54.6958333333,
		Timezone:  "America/Campo_Grande",
	},
	"SDME": {
		ICAO:      "SDME",
		Name:      "Marchesan S.A. Airport",
		City:      "Matao",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1942,
		Latitude:  -21.6305999756,
		Longitude: -48.3928985596,
		Timezone:  "America/Sao_Paulo",
	},
	"SDMH": {
		ICAO:      "SDMH",
		IATA:      "0",
		Name:      "Mirassol Airport",
		City:      "Mirassol",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1916,
		Latitude:  -20.8071994781,
		Longitude: -49.4860992432,
		Timezone:  "America/Sao_Paulo",
	},
	"SDMJ": {
		ICAO:      "SDMJ",
		IATA:      "0",
		Name:      "Mogi Mirim Airport",
		City:      "Mogi Mirim",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2274,
		Latitude:  -22.4099998474,
		Longitude: -46.9053001404,
		Timezone:  "America/Sao_Paulo",
	},
	"SDMO": {
		ICAO:      "SDMO",
		Name:      "Monte Alto Airport",
		City:      "Monte Alto",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2461,
		Latitude:  -21.2589812941,
		Longitude: -48.523607254,
		Timezone:  "America/Sao_Paulo",
	},
	"SDMQ": {
		ICAO:      "SDMQ",
		Name:      "Fazenda Quebracho Brasil Airport",
		City:      "Porto Murtinho",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 299,
		Latitude:  -21.8480555556,
		Longitude: -57.8991666667,
		Timezone:  "America/Asuncion",
	},
	"SDMR": {
		ICAO:      "SDMR",
		Name:      "Marambaia Airport",
		City:      "Rio De Janeiro",
		State:     "Rio-de-Janeiro",
		Country:   "BR",
		Elevation: 16,
		Latitude:  -23.0663890839,
		Longitude: -43.8797225952,
		Timezone:  "America/Sao_Paulo",
	},
	"SDMV": {
		ICAO:      "SDMV",
		Name:      "Fazenda Morro Vermelho Airport",
		City:      "Jau",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1736,
		Latitude:  -22.2772006989,
		Longitude: -48.6044998169,
		Timezone:  "America/Sao_Paulo",
	},
	"SDMW": {
		ICAO:      "SDMW",
		Name:      "Fazenda Karaja Airport",
		City:      "Nova Crixas",
		State:     "Goias",
		Country:   "BR",
		Elevation: 833,
		Latitude:  -14.1125,
		Longitude: -50.6808333333,
		Timezone:  "America/Sao_Paulo",
	},
	"SDMX": {
		ICAO:      "SDMX",
		Name:      "Fazenda Fontana Airport",
		City:      "Campo Verde",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2267,
		Latitude:  -15.5,
		Longitude: -55.3791666667,
		Timezone:  "America/Cuiaba",
	},
	"SDMY": {
		ICAO:      "SDMY",
		Name:      "Fazenda Cambuhy Airport",
		City:      "Matao",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2008,
		Latitude:  -21.632900238,
		Longitude: -48.4790992737,
		Timezone:  "America/Sao_Paulo",
	},
	"SDNA": {
		ICAO:      "SDNA",
		Name:      "Fazenda Vale Verde Airport",
		City:      "Comendador Gomes",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2336,
		Latitude:  -19.671944,
		Longitude: -48.994722,
		Timezone:  "America/Sao_Paulo",
	},
	"SDND": {
		ICAO:      "SDND",
		Name:      "Fazenda Nova Damasco Airport",
		City:      "Narandiba",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1411,
		Latitude:  -22.4827785492,
		Longitude: -51.5302772522,
		Timezone:  "America/Sao_Paulo",
	},
	"SDNE": {
		ICAO:      "SDNE",
		Name:      "Fazenda Sao Joaquim Airport",
		City:      "Maraba Paulista",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1585,
		Latitude:  -22.1994438171,
		Longitude: -51.9736099243,
		Timezone:  "America/Sao_Paulo",
	},
	"SDNG": {
		ICAO:      "SDNG",
		Name:      "Fazenda Sao Gabriel Airport",
		City:      "Nova Maringa",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1197,
		Latitude:  -13.1000003815,
		Longitude: -56.9527778625,
		Timezone:  "America/Cuiaba",
	},
	"SDNH": {
		ICAO:      "SDNH",
		IATA:      "0",
		Name:      "Novo Horizonte Airport",
		City:      "Novo Horizonte",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1526,
		Latitude:  -21.497800827,
		Longitude: -49.2344017029,
		Timezone:  "America/Sao_Paulo",
	},
	"SDNI": {
		ICAO:      "SDNI",
		Name:      "Nascimento I Airport",
		City:      "Vargem Grande Paulista",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 3090,
		Latitude:  -23.6197223663,
		Longitude: -46.9836120605,
		Timezone:  "America/Sao_Paulo",
	},
	"SDNJ": {
		ICAO:      "SDNJ",
		Name:      "Nascimento II Airport",
		City:      "Capao Bonito",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2604,
		Latitude:  -24.07472229,
		Longitude: -48.2247238159,
		Timezone:  "America/Sao_Paulo",
	},
	"SDNK": {
		ICAO:      "SDNK",
		Name:      "Fazenda Vitoria Airport",
		City:      "Primavera Do Leste",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2513,
		Latitude:  -14.9650001526,
		Longitude: -54.0508346558,
		Timezone:  "America/Cuiaba",
	},
	"SDNL": {
		ICAO:      "SDNL",
		IATA:      "0",
		Name:      "Fazenda Sao Francisco do Itaquere Airport",
		City:      "Nova Europa",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1877,
		Latitude:  -21.7597007751,
		Longitude: -48.586101532,
		Timezone:  "America/Sao_Paulo",
	},
	"SDNO": {
		ICAO:      "SDNO",
		Name:      "Sao Manuel Airport",
		City:      "Sao Manuel",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2395,
		Latitude:  -22.6958007813,
		Longitude: -48.5765991211,
		Timezone:  "America/Sao_Paulo",
	},
	"SDNV": {
		ICAO:      "SDNV",
		Name:      "Aldeia Nacepoti Airport",
		City:      "Altamira",
		State:     "Para",
		Country:   "BR",
		Elevation: 1001,
		Latitude:  -9.503889,
		Longitude: -54.011944,
		Timezone:  "America/Santarem",
	},
	"SDNW": {
		ICAO:      "SDNW",
		Name:      "Aldeia Piaracu Airport",
		City:      "Sao Jose Do Xingu",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1148,
		Latitude:  -10.788333,
		Longitude: -53.071667,
		Timezone:  "America/Cuiaba",
	},
	"SDNY": {
		ICAO:      "SDNY",
		IATA:      "QNV",
		Name:      "Aeroclube Airport",
		City:      "Nova Iguacu",
		State:     "Rio-de-Janeiro",
		Country:   "BR",
		Elevation: 164,
		Latitude:  -22.745300293,
		Longitude: -43.4603004456,
		Timezone:  "America/Sao_Paulo",
	},
	"SDNZ": {
		ICAO:      "SDNZ",
		Name:      "Fazenda Alto do Piriqui Airport",
		City:      "Sonora",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1640,
		Latitude:  -17.8449993134,
		Longitude: -54.7688903809,
		Timezone:  "America/Campo_Grande",
	},
	"SDOA": {
		ICAO:      "SDOA",
		Name:      "Lagoa da Floresta Airport",
		City:      "Barra Do Corda",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 591,
		Latitude:  -5.4886111111,
		Longitude: -45.4886111111,
		Timezone:  "America/Fortaleza",
	},
	"SDOB": {
		ICAO:      "SDOB",
		Name:      "Fazenda Sao Jose OB Airport",
		City:      "Tapiratiba",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2585,
		Latitude:  -21.4251995087,
		Longitude: -46.7542991638,
		Timezone:  "America/Sao_Paulo",
	},
	"SDOD": {
		ICAO:      "SDOD",
		Name:      "Fazenda Vida Airport",
		City:      "Barra Do Corda",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 820,
		Latitude:  -5.84,
		Longitude: -45.5419444444,
		Timezone:  "America/Fortaleza",
	},
	"SDOE": {
		ICAO:      "SDOE",
		Name:      "Fazenda Siberia Airport",
		City:      "Grajau",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 741,
		Latitude:  -5.7222222222,
		Longitude: -45.7913888889,
		Timezone:  "America/Fortaleza",
	},
	"SDOI": {
		ICAO:      "SDOI",
		Name:      "Centro Nacional de Para-quedismo Airport",
		City:      "Boituva",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2051,
		Latitude:  -23.2980556488,
		Longitude: -47.6919441223,
		Timezone:  "America/Sao_Paulo",
	},
	"SDOL": {
		ICAO:      "SDOL",
		Name:      "Agropesp Airport",
		City:      "Tapura",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1001,
		Latitude:  -12.441389,
		Longitude: -56.406944,
		Timezone:  "America/Cuiaba",
	},
	"SDOR": {
		ICAO:      "SDOR",
		Name:      "Fazenda Mosquito Airport",
		City:      "Orlandia",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2415,
		Latitude:  -20.7472229004,
		Longitude: -47.8897209167,
		Timezone:  "America/Sao_Paulo",
	},
	"SDOU": {
		ICAO:      "SDOU",
		IATA:      "OUS",
		Name:      "Ourinhos Airport",
		City:      "Ourinhos",
		Country:   "BR",
		Elevation: 1532,
		Latitude:  -22.9664993286,
		Longitude: -49.9132995605,
		Timezone:  "America/Sao_Paulo",
	},
	"SDOV": {
		ICAO:      "SDOV",
		Name:      "Mozarlandia Airport",
		City:      "Mozarlandia",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1076,
		Latitude:  -14.7704000473,
		Longitude: -50.5640983582,
		Timezone:  "America/Sao_Paulo",
	},
	"SDOW": {
		ICAO:      "SDOW",
		IATA:      "OIA",
		Name:      "Ourilandia do Norte Airport",
		City:      "Ourilandia Do Norte",
		State:     "Para",
		Country:   "BR",
		Elevation: 751,
		Latitude:  -6.7631001472,
		Longitude: -51.0499000549,
		Timezone:  "America/Belem",
	},
	"SDOX": {
		ICAO:      "SDOX",
		Name:      "Fazenda Pixoxo Airport",
		City:      "Sao Carlos",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2133,
		Latitude:  -21.7811107635,
		Longitude: -47.8844451904,
		Timezone:  "America/Sao_Paulo",
	},
	"SDOZ": {
		ICAO:      "SDOZ",
		Name:      "Aldeia Pontal Airport",
		City:      "Apiacas",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 869,
		Latitude:  -8.100833,
		Longitude: -58.284444,
		Timezone:  "America/Cuiaba",
	},
	"SDPA": {
		ICAO:      "SDPA",
		Name:      "Fazenda Portobello Airport",
		City:      "Mangaratiba",
		State:     "Rio-de-Janeiro",
		Country:   "BR",
		Elevation: 20,
		Latitude:  -22.9274997711,
		Longitude: -44.0800018311,
		Timezone:  "America/Sao_Paulo",
	},
	"SDPC": {
		ICAO:      "SDPC",
		Name:      "Atena Airport",
		City:      "Rancharia",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1585,
		Latitude:  -22.15278,
		Longitude: -51.02861,
		Timezone:  "America/Sao_Paulo",
	},
	"SDPD": {
		ICAO:      "SDPD",
		Name:      "Pindamonhangaba Airport",
		City:      "Pindamonhangaba",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1919,
		Latitude:  -22.943611145,
		Longitude: -45.431388855,
		Timezone:  "America/Sao_Paulo",
	},
	"SDPG": {
		ICAO:      "SDPG",
		Name:      "Fazenda Progresso Airport",
		City:      "Andradina",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1198,
		Latitude:  -21.0544433594,
		Longitude: -51.4061126709,
		Timezone:  "America/Sao_Paulo",
	},
	"SDPN": {
		ICAO:      "SDPN",
		Name:      "Penapolis Airport",
		City:      "Penapolis",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1371,
		Latitude:  -21.4099998474,
		Longitude: -50.0335998535,
		Timezone:  "America/Sao_Paulo",
	},
	"SDPP": {
		ICAO:      "SDPP",
		Name:      "Fazenda Fortaleza Airport",
		City:      "Paranapanema",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1952,
		Latitude:  -23.2929992676,
		Longitude: -48.8143997192,
		Timezone:  "America/Sao_Paulo",
	},
	"SDPV": {
		ICAO:      "SDPV",
		IATA:      "0",
		Name:      "Presidente Venceslau Airport",
		City:      "Presidente Venceslau",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1460,
		Latitude:  -21.8932991028,
		Longitude: -51.8843994141,
		Timezone:  "America/Sao_Paulo",
	},
	"SDPW": {
		ICAO:      "SDPW",
		IATA:      "QHB",
		Name:      "Piracicaba Airport",
		City:      "Piracicaba",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1887,
		Latitude:  -22.7115001678,
		Longitude: -47.6181983948,
		Timezone:  "America/Sao_Paulo",
	},
	"SDPY": {
		ICAO:      "SDPY",
		Name:      "Pirassununga Airport",
		City:      "Pirassununga",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2241,
		Latitude:  -22.0263881683,
		Longitude: -47.4194450378,
		Timezone:  "America/Sao_Paulo",
	},
	"SDQE": {
		ICAO:      "SDQE",
		Name:      "Fazenda Carambola Airport",
		City:      "Ponta Pora",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 2041,
		Latitude:  -22.5694446564,
		Longitude: -55.6361122131,
		Timezone:  "America/Campo_Grande",
	},
	"SDQG": {
		ICAO:      "SDQG",
		Name:      "Fazenda Nova Floresta Airport",
		City:      "Caiabu",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1460,
		Latitude:  -21.8988895416,
		Longitude: -51.1863899231,
		Timezone:  "America/Sao_Paulo",
	},
	"SDQH": {
		ICAO:      "SDQH",
		Name:      "Fazenda da Ilha Airport",
		City:      "Tibagi",
		State:     "Parana",
		Country:   "BR",
		Elevation: 2812,
		Latitude:  -24.6499996185,
		Longitude: -50.3624992371,
		Timezone:  "America/Sao_Paulo",
	},
	"SDQL": {
		ICAO:      "SDQL",
		Name:      "Fazenda Cerro Azul Airport",
		City:      "Nioaque",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1191,
		Latitude:  -21.18861,
		Longitude: -55.7344,
		Timezone:  "America/Campo_Grande",
	},
	"SDQN": {
		ICAO:      "SDQN",
		Name:      "Fazenda Tangara Airport",
		City:      "Nioaque",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 919,
		Latitude:  -20.8986110687,
		Longitude: -55.7099990845,
		Timezone:  "America/Campo_Grande",
	},
	"SDQQ": {
		ICAO:      "SDQQ",
		Name:      "Companhia Agricola de Quata Airport",
		City:      "Quata",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1919,
		Latitude:  -22.2861003876,
		Longitude: -50.6389007568,
		Timezone:  "America/Sao_Paulo",
	},
	"SDQV": {
		ICAO:      "SDQV",
		Name:      "Fazenda Santa Fe Airport",
		City:      "Caceres",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 676,
		Latitude:  -16.1399993896,
		Longitude: -58.5149993896,
		Timezone:  "America/Cuiaba",
	},
	"SDQW": {
		ICAO:      "SDQW",
		Name:      "Fazenda Criciuma Airport",
		City:      "Sorriso",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1575,
		Latitude:  -13.41,
		Longitude: -55.233611,
		Timezone:  "America/Cuiaba",
	},
	"SDQX": {
		ICAO:      "SDQX",
		Name:      "Fazenda Esperanca Airport",
		City:      "Itapeva",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2267,
		Latitude:  -23.867778,
		Longitude: -48.805278,
		Timezone:  "America/Sao_Paulo",
	},
	"SDQZ": {
		ICAO:      "SDQZ",
		Name:      "Fazenda Espinhaco II Airport",
		City:      "Barra Do Garcas",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 991,
		Latitude:  -15.3972215652,
		Longitude: -52.07472229,
		Timezone:  "America/Cuiaba",
	},
	"SDRA": {
		ICAO:      "SDRA",
		IATA:      "0",
		Name:      "Fazenda das Represas Airport",
		City:      "Sao Miguel Arcanjo",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2271,
		Latitude:  -23.8756008148,
		Longitude: -48.0517997742,
		Timezone:  "America/Sao_Paulo",
	},
	"SDRC": {
		ICAO:      "SDRC",
		Name:      "Fazenda Santana Airport",
		City:      "Rancharia",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1608,
		Latitude:  -22.1819438934,
		Longitude: -50.8658332825,
		Timezone:  "America/Sao_Paulo",
	},
	"SDRD": {
		ICAO:      "SDRD",
		Name:      "Adhemar Ribeiro Airport",
		City:      "Santa Rita Do Passa Quatro",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2776,
		Latitude:  -21.6424999237,
		Longitude: -47.4711112976,
		Timezone:  "America/Sao_Paulo",
	},
	"SDRF": {
		ICAO:      "SDRF",
		Name:      "Fazenda Recanto Feliz Airport",
		City:      "Rio Claro",
		State:     "Rio-de-Janeiro",
		Country:   "BR",
		Elevation: 1750,
		Latitude:  -22.7461109161,
		Longitude: -44.0508346558,
		Timezone:  "America/Sao_Paulo",
	},
	"SDRH": {
		ICAO:      "SDRH",
		Name:      "Fazenda Chaparral Airport",
		City:      "Campo Grande",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1509,
		Latitude:  -20.8444442749,
		Longitude: -54.5261116028,
		Timezone:  "America/Campo_Grande",
	},
	"SDRK": {
		ICAO:      "SDRK",
		IATA:      "QIQ",
		Name:      "Rio Claro Airport",
		City:      "Rio Claro",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1991,
		Latitude:  -22.4311008453,
		Longitude: -47.564201355,
		Timezone:  "America/Sao_Paulo",
	},
	"SDRO": {
		ICAO:      "SDRO",
		Name:      "Agropastoril bom Pastor Airport",
		City:      "Rondonopolis",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1034,
		Latitude:  -16.465555191,
		Longitude: -54.7119445801,
		Timezone:  "America/Cuiaba",
	},
	"SDRR": {
		ICAO:      "SDRR",
		IATA:      "QVP",
		Name:      "Avare-Arandu Airport",
		City:      "Avare",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2657,
		Latitude:  -23.0925006866,
		Longitude: -48.9874000549,
		Timezone:  "America/Sao_Paulo",
	},
	"SDRS": {
		ICAO:      "SDRS",
		IATA:      "QRZ",
		Name:      "Resende Airport",
		City:      "Resende",
		State:     "Rio-de-Janeiro",
		Country:   "BR",
		Elevation: 1320,
		Latitude:  -22.4785003662,
		Longitude: -44.4803009033,
		Timezone:  "America/Sao_Paulo",
	},
	"SDRU": {
		ICAO:      "SDRU",
		Name:      "Fazenda Caramuru Airport",
		City:      "Inubia Paulista",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1285,
		Latitude:  -21.6938896179,
		Longitude: -50.9361114502,
		Timezone:  "America/Sao_Paulo",
	},
	"SDRW": {
		ICAO:      "SDRW",
		Name:      "Fazenda Pirapitinga Airport",
		City:      "Canapolis",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2172,
		Latitude:  -18.7775,
		Longitude: -49.231667,
		Timezone:  "America/Sao_Paulo",
	},
	"SDRX": {
		ICAO:      "SDRX",
		Name:      "Fazenda Palmeiras Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 499,
		Latitude:  -17.9669437408,
		Longitude: -57.0330543518,
		Timezone:  "America/Campo_Grande",
	},
	"SDRY": {
		ICAO:      "SDRY",
		Name:      "Fazenda Sao Paulo Airport",
		City:      "Amambai",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1493,
		Latitude:  -22.9527778625,
		Longitude: -54.8452796936,
		Timezone:  "America/Campo_Grande",
	},
	"SDRZ": {
		ICAO:      "SDRZ",
		Name:      "Fazenda Maraba Airport",
		City:      "Porto Murtinho",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 499,
		Latitude:  -21.6872215271,
		Longitude: -57.3580551147,
		Timezone:  "America/Campo_Grande",
	},
	"SDSB": {
		ICAO:      "SDSB",
		Name:      "Fazenda Sao Sebastiao Airport",
		City:      "Rifaina",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1705,
		Latitude:  -19.9958324432,
		Longitude: -47.5238876343,
		Timezone:  "America/Sao_Paulo",
	},
	"SDSC": {
		ICAO:      "SDSC",
		IATA:      "QSC",
		Name:      "Sao Carlos Airport",
		City:      "Sao Carlos",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2649,
		Latitude:  -21.8754005432,
		Longitude: -47.9037017822,
		Timezone:  "America/Sao_Paulo",
	},
	"SDSE": {
		ICAO:      "SDSE",
		Name:      "Sitio Santa Helena Airport",
		City:      "Gabriel Monteiro",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1414,
		Latitude:  -21.4733333588,
		Longitude: -50.6155548096,
		Timezone:  "America/Sao_Paulo",
	},
	"SDSH": {
		ICAO:      "SDSH",
		Name:      "Fazenda Sangrila Airport",
		City:      "Bauru",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1755,
		Latitude:  -22.2955551147,
		Longitude: -49.212223053,
		Timezone:  "America/Sao_Paulo",
	},
	"SDSJ": {
		ICAO:      "SDSJ",
		IATA:      "0",
		Name:      "Fazenda Sao Joao Airport",
		City:      "Orlandia",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2172,
		Latitude:  -20.7455997467,
		Longitude: -47.9336013794,
		Timezone:  "America/Sao_Paulo",
	},
	"SDSK": {
		ICAO:      "SDSK",
		Name:      "Saquarema Airport",
		City:      "Saquarema",
		State:     "Rio-de-Janeiro",
		Country:   "BR",
		Elevation: 26,
		Latitude:  -22.9297218323,
		Longitude: -42.506942749,
		Timezone:  "America/Sao_Paulo",
	},
	"SDSM": {
		ICAO:      "SDSM",
		Name:      "Fazenda Nossa Senhora da Conceicao Airport",
		City:      "Sao Manuel",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2482,
		Latitude:  -22.7647228241,
		Longitude: -48.5424995422,
		Timezone:  "America/Sao_Paulo",
	},
	"SDSO": {
		ICAO:      "SDSO",
		Name:      "Fazenda Sao Geraldo Airport",
		City:      "Piquerobi",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1476,
		Latitude:  -22.0038890839,
		Longitude: -51.8111114502,
		Timezone:  "America/Sao_Paulo",
	},
	"SDSQ": {
		ICAO:      "SDSQ",
		Name:      "Usina Santa Rita Airport",
		City:      "Santa Rita Do Passa Quatro",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2067,
		Latitude:  -21.7236118317,
		Longitude: -47.6569442749,
		Timezone:  "America/Sao_Paulo",
	},
	"SDST": {
		ICAO:      "SDST",
		Name:      "Fazenda Santa Terezinha da Barra Airport",
		City:      "Sao Carlos",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2385,
		Latitude:  -21.8819446564,
		Longitude: -47.7761116028,
		Timezone:  "America/Sao_Paulo",
	},
	"SDSV": {
		ICAO:      "SDSV",
		Name:      "Fazenda Santa Candida Airport",
		City:      "Guariba",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2080,
		Latitude:  -21.3277778625,
		Longitude: -48.1711120605,
		Timezone:  "America/Sao_Paulo",
	},
	"SDTB": {
		ICAO:      "SDTB",
		Name:      "Atibaia Airport",
		City:      "Atibaia",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2592,
		Latitude:  -23.1277770996,
		Longitude: -46.57472229,
		Timezone:  "America/Sao_Paulo",
	},
	"SDTE": {
		ICAO:      "SDTE",
		Name:      "Fazenda Tapijara Airport",
		City:      "Arandu",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1929,
		Latitude:  -23.2199993134,
		Longitude: -49.0759010315,
		Timezone:  "America/Sao_Paulo",
	},
	"SDTF": {
		ICAO:      "SDTF",
		Name:      "Tatui Airport",
		City:      "Tatui",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2083,
		Latitude:  -23.3318996429,
		Longitude: -47.8782997131,
		Timezone:  "America/Sao_Paulo",
	},
	"SDTI": {
		ICAO:      "SDTI",
		Name:      "Tupi Paulista Airport",
		City:      "Tupi Paulista",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1198,
		Latitude:  -21.3930568695,
		Longitude: -51.5988883972,
		Timezone:  "America/Sao_Paulo",
	},
	"SDTK": {
		ICAO:      "SDTK",
		Name:      "Parati Airport",
		City:      "Parati",
		State:     "Rio-de-Janeiro",
		Country:   "BR",
		Elevation: 10,
		Latitude:  -23.2243995667,
		Longitude: -44.7202987671,
		Timezone:  "America/Sao_Paulo",
	},
	"SDTN": {
		ICAO:      "SDTN",
		Name:      "Usina Acucareira Santo Antonio Airport",
		City:      "Sertaozinho",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1969,
		Latitude:  -21.1219444275,
		Longitude: -47.9522209167,
		Timezone:  "America/Sao_Paulo",
	},
	"SDTO": {
		ICAO:      "SDTO",
		IATA:      "0",
		Name:      "Fazenda Cataco Airport",
		City:      "Ubarana",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1447,
		Latitude:  -21.2635993958,
		Longitude: -49.7930984497,
		Timezone:  "America/Sao_Paulo",
	},
	"SDTP": {
		ICAO:      "SDTP",
		Name:      "Tupa Airport",
		City:      "Tupa",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1805,
		Latitude:  -21.8890991211,
		Longitude: -50.5051002502,
		Timezone:  "America/Sao_Paulo",
	},
	"SDTQ": {
		ICAO:      "SDTQ",
		Name:      "Fazenda Santa Thereza Airport",
		City:      "Aparecida Do Rio Doce",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2149,
		Latitude:  -18.2327785492,
		Longitude: -51.2022209167,
		Timezone:  "America/Sao_Paulo",
	},
	"SDTR": {
		ICAO:      "SDTR",
		Name:      "Fazenda Bandeirantes Airport",
		City:      "Porto Murtinho",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 499,
		Latitude:  -20.8833332062,
		Longitude: -57.4177780151,
		Timezone:  "America/Campo_Grande",
	},
	"SDTS": {
		ICAO:      "SDTS",
		Name:      "Fazenda Planalto Airport",
		City:      "Jaraguari",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 2329,
		Latitude:  -20.3413887024,
		Longitude: -54.4158325195,
		Timezone:  "America/Campo_Grande",
	},
	"SDTT": {
		ICAO:      "SDTT",
		Name:      "Fazenda Barra do Traitu Airport",
		City:      "Aracatuba",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1345,
		Latitude:  -21.2469444275,
		Longitude: -50.4902763367,
		Timezone:  "America/Sao_Paulo",
	},
	"SDTW": {
		ICAO:      "SDTW",
		Name:      "Fazenda Sao Joao Airport",
		City:      "Terenos",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 919,
		Latitude:  -20.3261108398,
		Longitude: -55.1005554199,
		Timezone:  "America/Campo_Grande",
	},
	"SDTX": {
		ICAO:      "SDTX",
		Name:      "Fazenda Sao Jose Airport",
		City:      "Ilha Solteira",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1236,
		Latitude:  -20.5419445038,
		Longitude: -51.3574981689,
		Timezone:  "America/Sao_Paulo",
	},
	"SDTY": {
		ICAO:      "SDTY",
		Name:      "Usina Sao Martinho Airport",
		City:      "Pradopolis",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1713,
		Latitude:  -21.339799881,
		Longitude: -48.1148986816,
		Timezone:  "America/Sao_Paulo",
	},
	"SDTZ": {
		ICAO:      "SDTZ",
		Name:      "Fazenda Santa Julia Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 492,
		Latitude:  -20.1191673279,
		Longitude: -57.1119422913,
		Timezone:  "America/Campo_Grande",
	},
	"SDUA": {
		ICAO:      "SDUA",
		Name:      "Agrorural Sol Nascente Airport",
		City:      "Aveiro",
		State:     "Para",
		Country:   "BR",
		Elevation: 108,
		Latitude:  -3.5086109638,
		Longitude: -56.3158340454,
		Timezone:  "America/Santarem",
	},
	"SDUB": {
		ICAO:      "SDUB",
		IATA:      "UBT",
		Name:      "Ubatuba Airport",
		City:      "Ubatuba",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 13,
		Latitude:  -23.4410991669,
		Longitude: -45.0755996704,
		Timezone:  "America/Sao_Paulo",
	},
	"SDUD": {
		ICAO:      "SDUD",
		Name:      "Usina Santa Adelia Airport",
		City:      "Jaboticabal",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1991,
		Latitude:  -21.3323001862,
		Longitude: -48.3162002563,
		Timezone:  "America/Sao_Paulo",
	},
	"SDUE": {
		ICAO:      "SDUE",
		Name:      "Fazenda Figueira Airport",
		City:      "Penapolis",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1526,
		Latitude:  -21.5119438171,
		Longitude: -50.1211128235,
		Timezone:  "America/Sao_Paulo",
	},
	"SDUF": {
		ICAO:      "SDUF",
		Name:      "Fazenda Touro Peru Airport",
		City:      "Porto Murtinho",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 492,
		Latitude:  -21.080556,
		Longitude: -57.497222,
		Timezone:  "America/Campo_Grande",
	},
	"SDUH": {
		ICAO:      "SDUH",
		Name:      "Fazenda Sumare Airport",
		City:      "Ribas Do Rio Pardo",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1001,
		Latitude:  -21.5424995422,
		Longitude: -53.3430557251,
		Timezone:  "America/Campo_Grande",
	},
	"SDUK": {
		ICAO:      "SDUK",
		Name:      "Fazenda Xaimite Airport",
		City:      "Agua Boa",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1601,
		Latitude:  -14.0966672897,
		Longitude: -52.9019432068,
		Timezone:  "America/Cuiaba",
	},
	"SDUL": {
		ICAO:      "SDUL",
		Name:      "Usina Santa Lydia Airport",
		City:      "Ribeirao Preto",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2270,
		Latitude:  -21.2016677856,
		Longitude: -47.9205551147,
		Timezone:  "America/Sao_Paulo",
	},
	"SDUN": {
		ICAO:      "SDUN",
		IATA:      "ITP",
		Name:      "Itaperuna Airport",
		City:      "Itaperuna",
		State:     "Rio-de-Janeiro",
		Country:   "BR",
		Elevation: 410,
		Latitude:  -21.2192993164,
		Longitude: -41.8759002686,
		Timezone:  "America/Sao_Paulo",
	},
	"SDUO": {
		ICAO:      "SDUO",
		IATA:      "QGS",
		Name:      "Alagoinhas Airport",
		City:      "Alagoinhas",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 568,
		Latitude:  -12.1751003265,
		Longitude: -38.3801994324,
		Timezone:  "America/Bahia",
	},
	"SDUQ": {
		ICAO:      "SDUQ",
		Name:      "Paraguacu Paulista Airport",
		City:      "Paraguacu Paulista",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1525,
		Latitude:  -22.4269008636,
		Longitude: -50.6068992615,
		Timezone:  "America/Sao_Paulo",
	},
	"SDUR": {
		ICAO:      "SDUR",
		Name:      "Ibera Airport",
		City:      "Ponta Grossa",
		State:     "Parana",
		Country:   "BR",
		Elevation: 2937,
		Latitude:  -25.0519447327,
		Longitude: -50.1908340454,
		Timezone:  "America/Sao_Paulo",
	},
	"SDUS": {
		ICAO:      "SDUS",
		Name:      "Mayrowy Airport",
		City:      "Apiacas",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 689,
		Latitude:  -7.9561109543,
		Longitude: -57.8388900757,
		Timezone:  "America/Cuiaba",
	},
	"SDUU": {
		ICAO:      "SDUU",
		Name:      "Usina California Airport",
		City:      "Parapua",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1476,
		Latitude:  -21.91111,
		Longitude: -50.86833,
		Timezone:  "America/Sao_Paulo",
	},
	"SDUZ": {
		ICAO:      "SDUZ",
		Name:      "Usina Sao Luiz Airport",
		City:      "Ourinhos",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1608,
		Latitude:  -22.9483337402,
		Longitude: -49.7661094666,
		Timezone:  "America/Sao_Paulo",
	},
	"SDVA": {
		ICAO:      "SDVA",
		Name:      "Fazenda Campo Vitoria Airport",
		City:      "Vargem Grande Do Sul",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2329,
		Latitude:  -21.8322219849,
		Longitude: -46.9555549622,
		Timezone:  "America/Sao_Paulo",
	},
	"SDVE": {
		ICAO:      "SDVE",
		Name:      "Vera Cruz Airport",
		City:      "Vera Cruz",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2100,
		Latitude:  -22.2301998138,
		Longitude: -49.817199707,
		Timezone:  "America/Sao_Paulo",
	},
	"SDVG": {
		ICAO:      "SDVG",
		IATA:      "VOT",
		Name:      "Votuporanga Airport",
		City:      "Votuporanga",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1667,
		Latitude:  -20.4631996155,
		Longitude: -50.0045013428,
		Timezone:  "America/Sao_Paulo",
	},
	"SDVH": {
		ICAO:      "SDVH",
		Name:      "Fazenda Vale Eldorado Airport",
		City:      "Atibaia",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2727,
		Latitude:  -23.0041675568,
		Longitude: -46.6347236633,
		Timezone:  "America/Sao_Paulo",
	},
	"SDVI": {
		ICAO:      "SDVI",
		Name:      "Comandante Gastao Airport",
		City:      "Tangara Da Serra",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1391,
		Latitude:  -14.65,
		Longitude: -57.5,
		Timezone:  "America/Cuiaba",
	},
	"SDVJ": {
		ICAO:      "SDVJ",
		Name:      "Fazenda Santa Marta Airport",
		City:      "Santa Maria Das Barreiras",
		State:     "Para",
		Country:   "BR",
		Elevation: 807,
		Latitude:  -8.7225,
		Longitude: -50.455,
		Timezone:  "America/Belem",
	},
	"SDVL": {
		ICAO:      "SDVL",
		Name:      "Valenca Airport",
		City:      "Valenca",
		State:     "Rio-de-Janeiro",
		Country:   "BR",
		Elevation: 1903,
		Latitude:  -22.2427787781,
		Longitude: -43.7136116028,
		Timezone:  "America/Sao_Paulo",
	},
	"SDVN": {
		ICAO:      "SDVN",
		Name:      "Fazenda Santissima Trindade Airport",
		City:      "Amambai",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1299,
		Latitude:  -23.286944,
		Longitude: -54.841111,
		Timezone:  "America/Campo_Grande",
	},
	"SDVQ": {
		ICAO:      "SDVQ",
		Name:      "Fazenda Baia das Conchas Airport",
		City:      "Porto Murtinho",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 351,
		Latitude:  -21.19,
		Longitude: -57.64,
		Timezone:  "America/Campo_Grande",
	},
	"SDVS": {
		ICAO:      "SDVS",
		IATA:      "0",
		Name:      "Fazenda Vassoural Airport",
		City:      "Pontal",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1801,
		Latitude:  -21.0727996826,
		Longitude: -48.034198761,
		Timezone:  "America/Sao_Paulo",
	},
	"SDVU": {
		ICAO:      "SDVU",
		Name:      "Aldeia Kremoro Airport",
		City:      "Peixoto De Azevedo",
		State:     "Para",
		Country:   "BR",
		Elevation: 3281,
		Latitude:  -7.2097220421,
		Longitude: -52.9052772522,
		Timezone:  "America/Santarem",
	},
	"SDWC": {
		ICAO:      "SDWC",
		Name:      "Aldeia Bau Airport",
		City:      "Altamira",
		State:     "Para",
		Country:   "BR",
		Elevation: 699,
		Latitude:  -7.3602781296,
		Longitude: -54.8297233582,
		Timezone:  "America/Santarem",
	},
	"SDWD": {
		ICAO:      "SDWD",
		Name:      "Fazenda California Airport",
		City:      "Barretos",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1856,
		Latitude:  -20.4308338165,
		Longitude: -48.6049995422,
		Timezone:  "America/Sao_Paulo",
	},
	"SDWE": {
		ICAO:      "SDWE",
		Name:      "Usina Sao Carlos Airport",
		City:      "Jaboticabal",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1919,
		Latitude:  -21.2841663361,
		Longitude: -48.162776947,
		Timezone:  "America/Sao_Paulo",
	},
	"SDWG": {
		ICAO:      "SDWG",
		Name:      "Fazenda Itapiranga Airport",
		City:      "Maraba Paulista",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1394,
		Latitude:  -22.0630569458,
		Longitude: -51.9247207642,
		Timezone:  "America/Sao_Paulo",
	},
	"SDWH": {
		ICAO:      "SDWH",
		Name:      "Fazenda Avanhandava Airport",
		City:      "Jose Bonifacio",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1444,
		Latitude:  -21.1886005402,
		Longitude: -49.9361000061,
		Timezone:  "America/Sao_Paulo",
	},
	"SDWI": {
		ICAO:      "SDWI",
		Name:      "Fazenda Lagoa do Cavalo Airport",
		City:      "Gravata",
		State:     "Pernambuco",
		Country:   "BR",
		Elevation: 1709,
		Latitude:  -8.17,
		Longitude: -35.63,
		Timezone:  "America/Recife",
	},
	"SDWJ": {
		ICAO:      "SDWJ",
		Name:      "Aldeia Kubenkroke Airport",
		City:      "Altamira",
		Country:   "BR",
		Elevation: 1099,
		Latitude:  -8.7252779007,
		Longitude: -53.3902778625,
		Timezone:  "America/Santarem",
	},
	"SDWK": {
		ICAO:      "SDWK",
		Name:      "Junqueira Airport",
		City:      "Igarapava",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1850,
		Latitude:  -20.0016670227,
		Longitude: -47.7675018311,
		Timezone:  "America/Sao_Paulo",
	},
	"SDWL": {
		ICAO:      "SDWL",
		Name:      "Tres Marias Airport",
		City:      "Itupeva",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2566,
		Latitude:  -23.1916675568,
		Longitude: -47.0750007629,
		Timezone:  "America/Sao_Paulo",
	},
	"SDWM": {
		ICAO:      "SDWM",
		Name:      "Aldeia de Metuktire Airport",
		City:      "Peixoto De Azevedo",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2953,
		Latitude:  -10.0636110306,
		Longitude: -52.9966659546,
		Timezone:  "America/Cuiaba",
	},
	"SDWN": {
		ICAO:      "SDWN",
		Name:      "Fazenda Barreiro Grande Airport",
		City:      "Colombia",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1857,
		Latitude:  -20.3474998474,
		Longitude: -48.7724990845,
		Timezone:  "America/Sao_Paulo",
	},
	"SDWR": {
		ICAO:      "SDWR",
		Name:      "Fazenda Primavera Airport",
		City:      "Mirandopolis",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1627,
		Latitude:  -21.0777778625,
		Longitude: -51.144443512,
		Timezone:  "America/Sao_Paulo",
	},
	"SDWU": {
		ICAO:      "SDWU",
		Name:      "Fazenda Anahi Airport",
		City:      "Porto Murtinho",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 279,
		Latitude:  -21.7083320618,
		Longitude: -57.7405548096,
		Timezone:  "America/Campo_Grande",
	},
	"SDWX": {
		ICAO:      "SDWX",
		Name:      "Pista Sol Nascente Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 541,
		Latitude:  -5.4908328056,
		Longitude: -57.2158317566,
		Timezone:  "America/Santarem",
	},
	"SDXB": {
		ICAO:      "SDXB",
		Name:      "Cristalina Airport",
		City:      "Cristalina",
		State:     "Goias",
		Country:   "BR",
		Elevation: 3937,
		Latitude:  -16.7910003662,
		Longitude: -47.645198822,
		Timezone:  "America/Sao_Paulo",
	},
	"SDXE": {
		ICAO:      "SDXE",
		Name:      "Chacara Serradinho Airport",
		City:      "Jaboticabal",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2037,
		Latitude:  -21.264799118,
		Longitude: -48.356098175,
		Timezone:  "America/Sao_Paulo",
	},
	"SDXF": {
		ICAO:      "SDXF",
		Name:      "Alto Paraiso Airport",
		City:      "Alto Paraiso",
		State:     "Goias",
		Country:   "BR",
		Elevation: 4413,
		Latitude:  -14.1210002899,
		Longitude: -47.5308990479,
		Timezone:  "America/Sao_Paulo",
	},
	"SDXI": {
		ICAO:      "SDXI",
		Name:      "Fazenda Cajueiro Airport",
		City:      "Sao Domingos Do Capim",
		State:     "Para",
		Country:   "BR",
		Elevation: 577,
		Latitude:  -4.2386112213,
		Longitude: -48.4766654968,
		Timezone:  "America/Belem",
	},
	"SDXJ": {
		ICAO:      "SDXJ",
		Name:      "Costa Rica Airport",
		City:      "Costa Rica",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 2428,
		Latitude:  -18.48667,
		Longitude: -53.15333,
		Timezone:  "America/Campo_Grande",
	},
	"SDXO": {
		ICAO:      "SDXO",
		Name:      "Fazenda Santa Maria da Mata Airport",
		City:      "Sud Menucci",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1142,
		Latitude:  -20.7275009155,
		Longitude: -50.9716682434,
		Timezone:  "America/Sao_Paulo",
	},
	"SDXQ": {
		ICAO:      "SDXQ",
		Name:      "Fazenda Flamboyant Airport",
		City:      "Castilho",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1112,
		Latitude:  -20.9069442749,
		Longitude: -51.5755577087,
		Timezone:  "America/Sao_Paulo",
	},
	"SDXR": {
		ICAO:      "SDXR",
		Name:      "Fazenda Vaca Mocha Airport",
		City:      "Caracol",
		State:     "Concepcion",
		Country:   "BR",
		Elevation: 463,
		Latitude:  -22.2255554199,
		Longitude: -57.1547203064,
		Timezone:  "America/Asuncion",
	},
	"SDXT": {
		ICAO:      "SDXT",
		Name:      "Fazenda Boa Vista Airport",
		City:      "Rio Negro",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1100,
		Latitude:  -19.4355564117,
		Longitude: -55.0627784729,
		Timezone:  "America/Campo_Grande",
	},
	"SDXU": {
		ICAO:      "SDXU",
		Name:      "Fazenda Aracatuba Airport",
		City:      "Pacaembu",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1401,
		Latitude:  -21.3433322906,
		Longitude: -51.2516670227,
		Timezone:  "America/Sao_Paulo",
	},
	"SDYB": {
		ICAO:      "SDYB",
		Name:      "Fazenda Sao Jose do Barreiro Airport",
		City:      "Itapetininga",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2234,
		Latitude:  -23.7852783203,
		Longitude: -48.3400001526,
		Timezone:  "America/Sao_Paulo",
	},
	"SDYF": {
		ICAO:      "SDYF",
		Name:      "Fazenda Irmaos Munaretto Airport",
		City:      "Lucas Do Rio Verde",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1408,
		Latitude:  -13.1066665649,
		Longitude: -55.993057251,
		Timezone:  "America/Cuiaba",
	},
	"SDYI": {
		ICAO:      "SDYI",
		Name:      "Fazenda Buritiz Airport",
		City:      "Buritizeiro",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3045,
		Latitude:  -17.9174995422,
		Longitude: -45.4025001526,
		Timezone:  "America/Sao_Paulo",
	},
	"SDYJ": {
		ICAO:      "SDYJ",
		Name:      "Jose Martins da Silva Airport",
		City:      "Regente Feijo",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1663,
		Latitude:  -22.2255554199,
		Longitude: -51.3480567932,
		Timezone:  "America/Sao_Paulo",
	},
	"SDYM": {
		ICAO:      "SDYM",
		IATA:      "QGB",
		Name:      "Limeira Airport",
		City:      "Limeira",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2172,
		Latitude:  -22.603889,
		Longitude: -47.411944,
		Timezone:  "America/Sao_Paulo",
	},
	"SDYN": {
		ICAO:      "SDYN",
		Name:      "Fazenda Esmeralda Airport",
		City:      "Taciba",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1365,
		Latitude:  -22.5888881683,
		Longitude: -51.2988891602,
		Timezone:  "America/Sao_Paulo",
	},
	"SDYS": {
		ICAO:      "SDYS",
		Name:      "Aeroportobelo Airport",
		City:      "Porto Belo",
		State:     "Santa-Catarina",
		Country:   "BR",
		Elevation: 33,
		Latitude:  -27.173611,
		Longitude: -48.631111,
		Timezone:  "America/Sao_Paulo",
	},
	"SDYW": {
		ICAO:      "SDYW",
		Name:      "Itapeva Airport",
		City:      "Itapeva",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2320,
		Latitude:  -23.941400528,
		Longitude: -48.8818016052,
		Timezone:  "America/Sao_Paulo",
	},
	"SDYX": {
		ICAO:      "SDYX",
		Name:      "Campo Maior Airport",
		City:      "Campo Maior",
		State:     "Piaui",
		Country:   "BR",
		Elevation: 490,
		Latitude:  -4.8541669846,
		Longitude: -42.2605552673,
		Timezone:  "America/Fortaleza",
	},
	"SDZA": {
		ICAO:      "SDZA",
		Name:      "Fazenda Santa Maria Airport",
		City:      "Pirajui",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1804,
		Latitude:  -22.0055561066,
		Longitude: -49.4111099243,
		Timezone:  "America/Sao_Paulo",
	},
	"SDZH": {
		ICAO:      "SDZH",
		Name:      "Fazenda Santa Helena Airport",
		City:      "Pindamonhangaba",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1804,
		Latitude:  -22.8903999329,
		Longitude: -45.4933013916,
		Timezone:  "America/Sao_Paulo",
	},
	"SDZM": {
		ICAO:      "SDZM",
		Name:      "Fazenda Sao Marcos Airport",
		City:      "Itapeva",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2254,
		Latitude:  -23.9223995209,
		Longitude: -48.8182983398,
		Timezone:  "America/Sao_Paulo",
	},
	"SDZR": {
		ICAO:      "SDZR",
		Name:      "Fazenda Cruzeiro Airport",
		City:      "Guariba",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2116,
		Latitude:  -21.4122219086,
		Longitude: -48.3286094666,
		Timezone:  "America/Sao_Paulo",
	},
	"SDZS": {
		ICAO:      "SDZS",
		Name:      "Fazenda Bananeira Airport",
		City:      "Altair",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1673,
		Latitude:  -20.5799999237,
		Longitude: -49.07970047,
		Timezone:  "America/Sao_Paulo",
	},
	"SDZT": {
		ICAO:      "SDZT",
		Name:      "Fazenda Posses do Rio Grande Airport",
		City:      "Guaraci",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1581,
		Latitude:  -20.3108329773,
		Longitude: -48.9769439697,
		Timezone:  "America/Sao_Paulo",
	},
	"SDZU": {
		ICAO:      "SDZU",
		Name:      "Fazenda Anhumas Airport",
		City:      "Maraba Paulista",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1001,
		Latitude:  -22.1224994659,
		Longitude: -52.1977767944,
		Timezone:  "America/Sao_Paulo",
	},
	"SDZV": {
		ICAO:      "SDZV",
		Name:      "Fazenda Gandu Airport",
		City:      "Duartina",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1841,
		Latitude:  -22.4761104584,
		Longitude: -49.4700012207,
		Timezone:  "America/Sao_Paulo",
	},
	"SDZW": {
		ICAO:      "SDZW",
		Name:      "Fazenda Ipanema Airport",
		City:      "Andradina",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1181,
		Latitude:  -20.8116664886,
		Longitude: -51.2941665649,
		Timezone:  "America/Sao_Paulo",
	},
	"SDZX": {
		ICAO:      "SDZX",
		Name:      "Fazenda Suri Airport",
		City:      "Campinapolis",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1050,
		Latitude:  -14.815556,
		Longitude: -53.133056,
		Timezone:  "America/Cuiaba",
	},
	"SDZZ": {
		ICAO:      "SDZZ",
		Name:      "Sitio Sao Jose Airport",
		City:      "Nova Europa",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1790,
		Latitude:  -21.7977771759,
		Longitude: -48.543888092,
		Timezone:  "America/Sao_Paulo",
	},
	"SE01": {
		ICAO:      "SE01",
		Name:      "Alba Elena Airport",
		City:      "Rocafuerte",
		State:     "Guayas",
		Country:   "EC",
		Elevation: 150,
		Latitude:  -2.166670084,
		Longitude: -79.2833023071,
		Timezone:  "America/Guayaquil",
	},
	"SE02": {
		ICAO:      "SE02",
		Name:      "Guabital Airport",
		City:      "Soledad",
		State:     "Guayas",
		Country:   "EC",
		Elevation: 45,
		Latitude:  -2.7733900547,
		Longitude: -79.7024993896,
		Timezone:  "America/Guayaquil",
	},
	"SE03": {
		ICAO:      "SE03",
		Name:      "Las Penas Airport",
		City:      "Isla Puna",
		State:     "Guayas",
		Country:   "EC",
		Elevation: 3,
		Latitude:  -2.9471600056,
		Longitude: -80.1007003784,
		Timezone:  "America/Guayaquil",
	},
	"SE08": {
		ICAO:      "SE08",
		Name:      "San Carlos Costa Airport",
		City:      "Lorenzo de Garaicoa",
		State:     "Guayas",
		Country:   "EC",
		Elevation: 21,
		Latitude:  -2.0547199249,
		Longitude: -79.4330978394,
		Timezone:  "America/Guayaquil",
	},
	"SE09": {
		ICAO:      "SE09",
		Name:      "San Rafael Airport",
		City:      "San Rafael",
		State:     "Guayas",
		Country:   "EC",
		Elevation: 190,
		Latitude:  -2.1816699505,
		Longitude: -79.2367019653,
		Timezone:  "America/Guayaquil",
	},
	"SE10": {
		ICAO:      "SE10",
		Name:      "Vinces Airport",
		City:      "Vinces",
		State:     "Los-Rios",
		Country:   "EC",
		Elevation: 32,
		Latitude:  -1.5961099863,
		Longitude: -79.7260971069,
		Timezone:  "America/Guayaquil",
	},
	"SE11": {
		ICAO:      "SE11",
		Name:      "Km 60 Airport",
		City:      "Consumulo",
		State:     "Santo-Domingo-de-los-Tsachilas",
		Country:   "EC",
		Elevation: 1640,
		Latitude:  -0.5366669893,
		Longitude: -79.3688964844,
		Timezone:  "America/Guayaquil",
	},
	"SE13": {
		ICAO:      "SE13",
		Name:      "Babahoyo North Airport",
		City:      "Babahoyo",
		State:     "Los-Rios",
		Country:   "EC",
		Elevation: 29,
		Latitude:  -1.7033530474,
		Longitude: -79.5380477905,
		Timezone:  "America/Guayaquil",
	},
	"SE14": {
		ICAO:      "SE14",
		Name:      "San Carlos Airport",
		City:      "Jose Real",
		State:     "Guayas",
		Country:   "EC",
		Elevation: 126,
		Latitude:  -2.1665520668,
		Longitude: -79.4337005615,
		Timezone:  "America/Guayaquil",
	},
	"SE15": {
		ICAO:      "SE15",
		Name:      "Quevedo North Airport",
		City:      "Quevedo",
		State:     "Los-Rios",
		Country:   "EC",
		Elevation: 314,
		Latitude:  -0.999826014,
		Longitude: -79.4153594971,
		Timezone:  "America/Guayaquil",
	},
	"SE16": {
		ICAO:      "SE16",
		Name:      "Loja Airport",
		City:      "Loja",
		State:     "Loja",
		Country:   "EC",
		Elevation: 7684,
		Latitude:  -3.8814239502,
		Longitude: -79.2347488403,
		Timezone:  "America/Guayaquil",
	},
	"SE22": {
		ICAO:      "SE22",
		Name:      "Celia Maria Airport",
		City:      "Pasaje",
		State:     "El-Oro",
		Country:   "EC",
		Elevation: 110,
		Latitude:  -3.2793300152,
		Longitude: -79.8009033203,
		Timezone:  "America/Guayaquil",
	},
	"SE58": {
		ICAO:      "SE58",
		Name:      "Banasur Airport",
		City:      "Buenavista",
		State:     "El-Oro",
		Country:   "EC",
		Elevation: 30,
		Latitude:  -3.3409399986,
		Longitude: -79.8638000488,
		Timezone:  "America/Guayaquil",
	},
	"SE59": {
		ICAO:      "SE59",
		Name:      "La Mina Airport",
		City:      "El Guabo",
		State:     "El-Oro",
		Country:   "EC",
		Elevation: 13,
		Latitude:  -3.20291996,
		Longitude: -79.7982025146,
		Timezone:  "America/Guayaquil",
	},
	"SE60": {
		ICAO:      "SE60",
		Name:      "La Puntilla Airport",
		City:      "La Puntilla",
		State:     "Canar",
		Country:   "EC",
		Elevation: 45,
		Latitude:  -2.4428501129,
		Longitude: -79.3989028931,
		Timezone:  "America/Guayaquil",
	},
	"SEAM": {
		ICAO:      "SEAM",
		IATA:      "ATF",
		Name:      "Chachoan Airport",
		City:      "Ambato",
		State:     "Tungurahua",
		Country:   "EC",
		Elevation: 8502,
		Latitude:  -1.2120699883,
		Longitude: -78.5746002197,
		Timezone:  "America/Guayaquil",
	},
	"SEAP": {
		ICAO:      "SEAP",
		Name:      "Arapicos Airport",
		City:      "Sangay",
		State:     "Morona-Santiago",
		Country:   "EC",
		Elevation: 2920,
		Latitude:  -1.8576200008,
		Longitude: -77.9413986206,
		Timezone:  "America/Guayaquil",
	},
	"SEAR": {
		ICAO:      "SEAR",
		Name:      "Arajuno Airport",
		City:      "Arajuno",
		State:     "Napo",
		Country:   "EC",
		Elevation: 1700,
		Latitude:  -1.235730052,
		Longitude: -77.6854019165,
		Timezone:  "America/Guayaquil",
	},
	"SEAS": {
		ICAO:      "SEAS",
		Name:      "Ascazubi Airport",
		City:      "Ascazubi",
		State:     "Pichincha",
		Country:   "EC",
		Elevation: 9517,
		Latitude:  -0.0833332986,
		Longitude: -78.2833328247,
		Timezone:  "America/Guayaquil",
	},
	"SEAY": {
		ICAO:      "SEAY",
		Name:      "Ayangue Airport",
		City:      "San Pedro",
		State:     "Santa-Elena",
		Country:   "EC",
		Elevation: 18,
		Latitude:  -1.9777779579,
		Longitude: -80.739440918,
		Timezone:  "America/Guayaquil",
	},
	"SEBF": {
		ICAO:      "SEBF",
		Name:      "Buena Fe Airport",
		City:      "Buena Fe",
		State:     "Los-Rios",
		Country:   "EC",
		Elevation: 320,
		Latitude:  -0.8981379867,
		Longitude: -79.4855422974,
		Timezone:  "America/Guayaquil",
	},
	"SECH": {
		ICAO:      "SECH",
		Name:      "Chone Airport",
		City:      "Chone",
		State:     "Manabi",
		Country:   "EC",
		Elevation: 51,
		Latitude:  -0.6951010227,
		Longitude: -80.0886001587,
		Timezone:  "America/Guayaquil",
	},
	"SECM": {
		ICAO:      "SECM",
		Name:      "Hacienda Clementina Airport",
		City:      "Pozuelos",
		State:     "Los-Rios",
		Country:   "EC",
		Elevation: 328,
		Latitude:  -1.7062699795,
		Longitude: -79.3788986206,
		Timezone:  "America/Guayaquil",
	},
	"SECO": {
		ICAO:      "SECO",
		IATA:      "OCC",
		Name:      "Francisco De Orellana Airport",
		City:      "Coca",
		State:     "Napo",
		Country:   "EC",
		Elevation: 834,
		Latitude:  -0.4628860056,
		Longitude: -76.9868011475,
		Timezone:  "America/Guayaquil",
	},
	"SECQ": {
		ICAO:      "SECQ",
		Name:      "Coaque Airport",
		City:      "Coaque",
		Country:   "EC",
		Elevation: 131,
		Latitude:  9.999e-07,
		Longitude: -80.0999984741,
		Timezone:  "America/Guayaquil",
	},
	"SECR": {
		ICAO:      "SECR",
		Name:      "Curaray Airport",
		City:      "Curacay",
		State:     "Pastaza",
		Country:   "EC",
		Elevation: 832,
		Latitude:  -1.3780299425,
		Longitude: -76.939201355,
		Timezone:  "America/Guayaquil",
	},
	"SECU": {
		ICAO:      "SECU",
		IATA:      "CUE",
		Name:      "Mariscal Lamar Airport",
		City:      "Cuenca",
		State:     "Azuay",
		Country:   "EC",
		Elevation: 8306,
		Latitude:  -2.8894701004,
		Longitude: -78.9843978882,
		Timezone:  "America/Guayaquil",
	},
	"SEGE": {
		ICAO:      "SEGE",
		Name:      "Guale Airport",
		City:      "Guale",
		State:     "Manabi",
		Country:   "EC",
		Elevation: 222,
		Latitude:  -1.6166700125,
		Longitude: -80.2332992554,
		Timezone:  "America/Guayaquil",
	},
	"SEGS": {
		ICAO:      "SEGS",
		IATA:      "GPS",
		Name:      "Seymour Airport",
		City:      "Baltra",
		State:     "Galapagos",
		Country:   "EC",
		Elevation: 207,
		Latitude:  -0.4537580013,
		Longitude: -90.2658996582,
		Timezone:  "Pacific/Galapagos",
	},
	"SEGU": {
		ICAO:      "SEGU",
		IATA:      "GYE",
		Name:      "Simon Bolivar International Airport",
		City:      "Guayaquil",
		State:     "Guayas",
		Country:   "EC",
		Elevation: 19,
		Latitude:  -2.15741992,
		Longitude: -79.8835983276,
		Timezone:  "America/Guayaquil",
	},
	"SEGZ": {
		ICAO:      "SEGZ",
		Name:      "Gualaquiza Airport",
		City:      "Santiago",
		State:     "Morona-Santiago",
		Country:   "EC",
		Elevation: 2640,
		Latitude:  -3.4232099056,
		Longitude: -78.5670013428,
		Timezone:  "America/Guayaquil",
	},
	"SEHI": {
		ICAO:      "SEHI",
		Name:      "Cotachachi Airport",
		City:      "Cotacachi",
		State:     "Pichincha",
		Country:   "EC",
		Elevation: 7824,
		Latitude:  -0.302940011,
		Longitude: -78.2683029175,
		Timezone:  "America/Guayaquil",
	},
	"SEIB": {
		ICAO:      "SEIB",
		Name:      "Atahualpa Airport",
		City:      "Ibarra",
		State:     "Imbabura",
		Country:   "EC",
		Elevation: 7304,
		Latitude:  0.3384189904,
		Longitude: -78.1363983154,
		Timezone:  "America/Guayaquil",
	},
	"SEII": {
		ICAO:      "SEII",
		IATA:      "IBB",
		Name:      "General Villamil Airport",
		City:      "Isabela",
		State:     "Galapagos",
		Country:   "EC",
		Elevation: 35,
		Latitude:  -0.942628026,
		Longitude: -90.9530029297,
		Timezone:  "Pacific/Galapagos",
	},
	"SEIS": {
		ICAO:      "SEIS",
		Name:      "Isabel Maria Airport",
		City:      "America",
		State:     "Guayas",
		Country:   "EC",
		Elevation: 20,
		Latitude:  -1.8666700125,
		Longitude: -79.7332992554,
		Timezone:  "America/Guayaquil",
	},
	"SEJI": {
		ICAO:      "SEJI",
		IATA:      "JIP",
		Name:      "Jipijapa Airport",
		City:      "Jipijapa",
		State:     "Manabi",
		Country:   "EC",
		Elevation: 223,
		Latitude:  -1,
		Longitude: -80.6666641235,
		Timezone:  "America/Guayaquil",
	},
	"SEJM": {
		ICAO:      "SEJM",
		Name:      "Jama Airport",
		City:      "Jama",
		State:     "Manabi",
		Country:   "EC",
		Elevation: 37,
		Latitude:  -0.2014970034,
		Longitude: -80.2654037476,
		Timezone:  "America/Guayaquil",
	},
	"SEKK": {
		ICAO:      "SEKK",
		Name:      "Km 192 Airport",
		City:      "Lagartera",
		State:     "Esmeraldas",
		Country:   "EC",
		Elevation: 1247,
		Latitude:  0.1842029989,
		Longitude: -79.391998291,
		Timezone:  "America/Guayaquil",
	},
	"SELI": {
		ICAO:      "SELI",
		Name:      "Limoncocha Airport",
		City:      "Limoncocha",
		Country:   "EC",
		Elevation: 820,
		Latitude:  -0.4063630104,
		Longitude: -76.6238021851,
		Timezone:  "America/Guayaquil",
	},
	"SELJ": {
		ICAO:      "SELJ",
		Name:      "Hacienda La Julia Airport",
		City:      "Julia",
		State:     "Los-Rios",
		Country:   "EC",
		Elevation: 50,
		Latitude:  -1.7043800354,
		Longitude: -79.5522994995,
		Timezone:  "America/Guayaquil",
	},
	"SELM": {
		ICAO:      "SELM",
		Name:      "Loma Larga Airport",
		City:      "Loma Larga",
		Country:   "EC",
		Elevation: 210,
		Latitude:  -1.5015799999,
		Longitude: -79.4809036255,
		Timezone:  "America/Guayaquil",
	},
	"SELT": {
		ICAO:      "SELT",
		IATA:      "LTX",
		Name:      "Cotopaxi International Airport",
		City:      "Latacunga",
		State:     "Cotopaxi",
		Country:   "EC",
		Elevation: 9205,
		Latitude:  -0.906832993,
		Longitude: -78.6157989502,
		Timezone:  "America/Guayaquil",
	},
	"SEMA": {
		ICAO:      "SEMA",
		IATA:      "MRR",
		Name:      "Jose Maria Velasco Ibarra Airport",
		City:      "Macara",
		State:     "Loja",
		Country:   "EC",
		Elevation: 1508,
		Latitude:  -4.3782300949,
		Longitude: -79.9410018921,
		Timezone:  "America/Guayaquil",
	},
	"SEMC": {
		ICAO:      "SEMC",
		IATA:      "XMS",
		Name:      "Coronel E Carvajal Airport",
		City:      "Macas",
		Country:   "EC",
		Elevation: 3452,
		Latitude:  -2.2991700172,
		Longitude: -78.1207962036,
		Timezone:  "America/Guayaquil",
	},
	"SEMH": {
		ICAO:      "SEMH",
		IATA:      "MCH",
		Name:      "General Manuel Serrano Airport",
		City:      "Machala",
		State:     "El-Oro",
		Country:   "EC",
		Elevation: 11,
		Latitude:  -3.2688999176,
		Longitude: -79.9616012573,
		Timezone:  "America/Guayaquil",
	},
	"SEMO": {
		ICAO:      "SEMO",
		Name:      "El Carmen Airport",
		City:      "Montalvo",
		State:     "Pastaza",
		Country:   "EC",
		Elevation: 960,
		Latitude:  -2.0670099258,
		Longitude: -76.9757003784,
		Timezone:  "America/Guayaquil",
	},
	"SEMT": {
		ICAO:      "SEMT",
		IATA:      "MEC",
		Name:      "Eloy Alfaro International Airport",
		City:      "Manta",
		State:     "Manabi",
		Country:   "EC",
		Elevation: 48,
		Latitude:  -0.9460780025,
		Longitude: -80.6788024902,
		Timezone:  "America/Guayaquil",
	},
	"SEMX": {
		ICAO:      "SEMX",
		Name:      "Maragrosa Airport",
		City:      "Puerto Balao",
		State:     "Guayas",
		Country:   "EC",
		Elevation: 18,
		Latitude:  -2.851099968,
		Longitude: -79.8035964966,
		Timezone:  "America/Guayaquil",
	},
	"SEMY": {
		ICAO:      "SEMY",
		Name:      "Martinica Airport",
		City:      "Martinica",
		State:     "Los-Rios",
		Country:   "EC",
		Elevation: 90,
		Latitude:  -1.7397199869,
		Longitude: -79.6214981079,
		Timezone:  "America/Guayaquil",
	},
	"SENL": {
		ICAO:      "SENL",
		IATA:      "LGQ",
		Name:      "Nueva Loja Airport",
		City:      "Lago Agrio",
		State:     "Sucumbios",
		Country:   "EC",
		Elevation: 980,
		Latitude:  0.0930560008,
		Longitude: -76.8675003052,
		Timezone:  "America/Guayaquil",
	},
	"SEPB": {
		ICAO:      "SEPB",
		Name:      "Pedro Carbo Airport",
		City:      "Pecro Carbo",
		State:     "Guayas",
		Country:   "EC",
		Elevation: 321,
		Latitude:  -1.8115930557,
		Longitude: -80.232460022,
		Timezone:  "America/Guayaquil",
	},
	"SEPC": {
		ICAO:      "SEPC",
		Name:      "Patuca Airport",
		City:      "Patuca",
		State:     "Morona-Santiago",
		Country:   "EC",
		Elevation: 1781,
		Latitude:  -2.7517199516,
		Longitude: -78.2636032104,
		Timezone:  "America/Guayaquil",
	},
	"SEPD": {
		ICAO:      "SEPD",
		Name:      "Pedernales Airport",
		City:      "Pedernales",
		State:     "Manabi",
		Country:   "EC",
		Elevation: 60,
		Latitude:  0.0676190034,
		Longitude: -80.0504989624,
		Timezone:  "America/Guayaquil",
	},
	"SEPO": {
		ICAO:      "SEPO",
		Name:      "Posorja Airport",
		City:      "Posorja",
		State:     "Guayas",
		Country:   "EC",
		Elevation: 30,
		Latitude:  -2.6820299625,
		Longitude: -80.2755966187,
		Timezone:  "America/Guayaquil",
	},
	"SEPS": {
		ICAO:      "SEPS",
		Name:      "Amable Calle Gutierrez Airport",
		City:      "Pasaje",
		State:     "El-Oro",
		Country:   "EC",
		Elevation: 22,
		Latitude:  -3.3196699619,
		Longitude: -79.769203186,
		Timezone:  "America/Guayaquil",
	},
	"SEPT": {
		ICAO:      "SEPT",
		IATA:      "PYO",
		Name:      "Putumayo Airport",
		City:      "Puerto Putumayo",
		State:     "Sucumbios",
		Country:   "EC",
		Elevation: 733,
		Latitude:  0.0833330005,
		Longitude: -75.9000015259,
		Timezone:  "America/Guayaquil",
	},
	"SEPU": {
		ICAO:      "SEPU",
		Name:      "Puna Airport",
		City:      "Puna",
		State:     "Guayas",
		Country:   "EC",
		Elevation: 3,
		Latitude:  -2.7336800098,
		Longitude: -79.9152984619,
		Timezone:  "America/Guayaquil",
	},
	"SEPV": {
		ICAO:      "SEPV",
		IATA:      "PVO",
		Name:      "Reales Tamarindos Airport",
		City:      "Portoviejo",
		State:     "Manabi",
		Country:   "EC",
		Elevation: 130,
		Latitude:  -1.0416500568,
		Longitude: -80.4721984863,
		Timezone:  "America/Guayaquil",
	},
	"SEPX": {
		ICAO:      "SEPX",
		Name:      "Payo Airport",
		City:      "Hacienda Payo",
		State:     "Guayas",
		Country:   "EC",
		Elevation: 101,
		Latitude:  -2.3099999428,
		Longitude: -79.4599990845,
		Timezone:  "America/Guayaquil",
	},
	"SEPZ": {
		ICAO:      "SEPZ",
		Name:      "Los Perez Airport",
		City:      "Los Perez",
		State:     "Los-Rios",
		Country:   "EC",
		Elevation: 75,
		Latitude:  -0.9537220001,
		Longitude: -79.3751983643,
		Timezone:  "America/Guayaquil",
	},
	"SEQE": {
		ICAO:      "SEQE",
		Name:      "Quevedo Airport",
		City:      "Quevedo",
		State:     "Los-Rios",
		Country:   "EC",
		Elevation: 350,
		Latitude:  -0.9894000292,
		Longitude: -79.4651031494,
		Timezone:  "America/Guayaquil",
	},
	"SEQM": {
		ICAO:      "SEQM",
		IATA:      "UIO",
		Name:      "Nuevo Aeropuerto Internacional Mariscal Sucre",
		City:      "Quito",
		State:     "Pichincha",
		Country:   "EC",
		Elevation: 9200,
		Latitude:  -0.1291666667,
		Longitude: -78.3575,
		Timezone:  "America/Guayaquil",
	},
	"SERB": {
		ICAO:      "SERB",
		Name:      "Chimborazo Airport",
		City:      "Riobamba",
		State:     "Chimborazo",
		Country:   "EC",
		Elevation: 9151,
		Latitude:  -1.653429985,
		Longitude: -78.6560974121,
		Timezone:  "America/Guayaquil",
	},
	"SERO": {
		ICAO:      "SERO",
		IATA:      "ETR",
		Name:      "Coronel Artilleria Victor Larrea Airport",
		City:      "Santa Rosa",
		State:     "El-Oro",
		Country:   "EC",
		Elevation: 170,
		Latitude:  -3.4351599217,
		Longitude: -79.9777984619,
		Timezone:  "America/Guayaquil",
	},
	"SESA": {
		ICAO:      "SESA",
		IATA:      "SNC",
		Name:      "General Ulpiano Paez Airport",
		City:      "Salinas",
		State:     "Santa-Elena",
		Country:   "EC",
		Elevation: 18,
		Latitude:  -2.2049899101,
		Longitude: -80.988899231,
		Timezone:  "America/Guayaquil",
	},
	"SESC": {
		ICAO:      "SESC",
		IATA:      "SUQ",
		Name:      "Sucua Airport",
		City:      "Sucua",
		State:     "Morona-Santiago",
		Country:   "EC",
		Elevation: 3116,
		Latitude:  -2.4830000401,
		Longitude: -78.1669998169,
		Timezone:  "America/Guayaquil",
	},
	"SESD": {
		ICAO:      "SESD",
		Name:      "Santo Domingo de Los Colorados Airport",
		City:      "Santo Domingo de Los Colorades",
		State:     "Santo-Domingo-de-los-Tsachilas",
		Country:   "EC",
		Elevation: 1714,
		Latitude:  -0.2482219934,
		Longitude: -79.2144012451,
		Timezone:  "America/Guayaquil",
	},
	"SESM": {
		ICAO:      "SESM",
		IATA:      "PTZ",
		Name:      "Rio Amazonas Airport",
		City:      "Shell Mera",
		State:     "Pastaza",
		Country:   "EC",
		Elevation: 3465,
		Latitude:  -1.5052399635,
		Longitude: -78.0626983643,
		Timezone:  "America/Guayaquil",
	},
	"SEST": {
		ICAO:      "SEST",
		IATA:      "SCY",
		Name:      "San Cristobal Airport",
		City:      "San Cristobal",
		State:     "Galapagos",
		Country:   "EC",
		Elevation: 62,
		Latitude:  -0.9102060199,
		Longitude: -89.617401123,
		Timezone:  "Pacific/Galapagos",
	},
	"SESV": {
		ICAO:      "SESV",
		IATA:      "BHA",
		Name:      "Los Perales Airport",
		City:      "Bahia de Caraquez",
		State:     "Manabi",
		Country:   "EC",
		Elevation: 10,
		Latitude:  -0.6081110239,
		Longitude: -80.4027023315,
		Timezone:  "America/Guayaquil",
	},
	"SETA": {
		ICAO:      "SETA",
		Name:      "Taura Airport",
		City:      "Taura",
		State:     "Guayas",
		Country:   "EC",
		Elevation: 56,
		Latitude:  -2.2610399723,
		Longitude: -79.6801986694,
		Timezone:  "America/Guayaquil",
	},
	"SETE": {
		ICAO:      "SETE",
		Name:      "Mayor Galo Torres Airport",
		City:      "Tena",
		State:     "Napo",
		Country:   "EC",
		Elevation: 1708,
		Latitude:  -0.986766994,
		Longitude: -77.8195037842,
		Timezone:  "America/Guayaquil",
	},
	"SETG": {
		ICAO:      "SETG",
		Name:      "Tenguel Airport",
		City:      "Boca de Tenguel",
		State:     "Guayas",
		Country:   "EC",
		Elevation: 33,
		Latitude:  -2.9986000061,
		Longitude: -79.7947006226,
		Timezone:  "America/Guayaquil",
	},
	"SETH": {
		ICAO:      "SETH",
		IATA:      "TSC",
		Name:      "Taisha Airport",
		City:      "Taisha",
		State:     "Morona-Santiago",
		Country:   "EC",
		Elevation: 1669,
		Latitude:  -2.3816699982,
		Longitude: -77.5027999878,
		Timezone:  "America/Guayaquil",
	},
	"SETI": {
		ICAO:      "SETI",
		IATA:      "TPN",
		Name:      "Tiputini Airport",
		City:      "Tiputini",
		State:     "Orellana",
		Country:   "EC",
		Elevation: 997,
		Latitude:  -0.7761110067,
		Longitude: -75.5263977051,
		Timezone:  "America/Guayaquil",
	},
	"SETM": {
		ICAO:      "SETM",
		IATA:      "LOH",
		Name:      "Camilo Ponce Enriquez Airport",
		City:      "La Toma (Catamayo)",
		State:     "Loja",
		Country:   "EC",
		Elevation: 4056,
		Latitude:  -3.9958899021,
		Longitude: -79.3719024658,
		Timezone:  "America/Guayaquil",
	},
	"SETN": {
		ICAO:      "SETN",
		IATA:      "ESM",
		Name:      "General Rivadeneira Airport",
		City:      "Tachina",
		State:     "Esmeraldas",
		Country:   "EC",
		Elevation: 32,
		Latitude:  0.9785190225,
		Longitude: -79.6266021729,
		Timezone:  "America/Guayaquil",
	},
	"SETR": {
		ICAO:      "SETR",
		IATA:      "TPC",
		Name:      "Tarapoa Airport",
		City:      "Tarapoa",
		State:     "Sucumbios",
		Country:   "EC",
		Elevation: 814,
		Latitude:  -0.1229560003,
		Longitude: -76.3377990723,
		Timezone:  "America/Guayaquil",
	},
	"SETU": {
		ICAO:      "SETU",
		IATA:      "TUA",
		Name:      "Teniente Coronel Luis a Mantilla Airport",
		City:      "Tulcan",
		State:     "Carchi",
		Country:   "EC",
		Elevation: 9649,
		Latitude:  0.8095059991,
		Longitude: -77.7080993652,
		Timezone:  "America/Guayaquil",
	},
	"SEVR": {
		ICAO:      "SEVR",
		Name:      "El Vergel Airport",
		City:      "El Vergel",
		State:     "Chimborazo",
		Country:   "EC",
		Elevation: 9002,
		Latitude:  -1.75,
		Longitude: -78.6166687012,
		Timezone:  "America/Guayaquil",
	},
	"SFAL": {
		ICAO:      "SFAL",
		IATA:      "PSY",
		Name:      "Stanley Airport",
		City:      "Stanley",
		Country:   "FK",
		Elevation: 75,
		Latitude:  -51.6856994629,
		Longitude: -57.7775993347,
		Timezone:  "Atlantic/Stanley",
	},
	"SG67": {
		ICAO:      "SG67",
		Name:      "Estancia Pai Quara Airport",
		City:      "Estancia Pai Quara",
		State:     "Amambay",
		Country:   "PY",
		Elevation: 712,
		Latitude:  -23.2199993134,
		Longitude: -55.9500007629,
		Timezone:  "America/Asuncion",
	},
	"SGAS": {
		ICAO:      "SGAS",
		IATA:      "ASU",
		Name:      "Silvio Pettirossi International Airport",
		City:      "Asuncion",
		State:     "Central",
		Country:   "PY",
		Elevation: 292,
		Latitude:  -25.2399997711,
		Longitude: -57.5200004578,
		Timezone:  "America/Asuncion",
	},
	"SGAY": {
		ICAO:      "SGAY",
		IATA:      "AYO",
		Name:      "Juan De Ayolas Airport",
		City:      "Ayolas",
		State:     "Misiones",
		Country:   "PY",
		Elevation: 223,
		Latitude:  -27.3700008392,
		Longitude: -56.8499984741,
		Timezone:  "America/Asuncion",
	},
	"SGBA": {
		ICAO:      "SGBA",
		Name:      "Bella Vista Sur Airport",
		City:      "Colonia Doctor Pastor Obligado",
		State:     "Itapua",
		Country:   "PY",
		Elevation: 650,
		Latitude:  -27.052393,
		Longitude: -55.578026,
		Timezone:  "America/Asuncion",
	},
	"SGBN": {
		ICAO:      "SGBN",
		IATA:      "BFA",
		Name:      "Bahia Negra Airport",
		City:      "Bahia Negra",
		State:     "Alto-Paraguay",
		Country:   "PY",
		Elevation: 277,
		Latitude:  -20.2245998383,
		Longitude: -58.1791992188,
		Timezone:  "America/Asuncion",
	},
	"SGCO": {
		ICAO:      "SGCO",
		IATA:      "CIO",
		Name:      "Teniente Col Carmelo Peralta Airport",
		City:      "Concepcion",
		State:     "Concepcion",
		Country:   "PY",
		Elevation: 253,
		Latitude:  -23.4400005341,
		Longitude: -57.4300003052,
		Timezone:  "America/Asuncion",
	},
	"SGEN": {
		ICAO:      "SGEN",
		IATA:      "ENO",
		Name:      "Encarnacion Airport",
		City:      "Encarnacion",
		State:     "Itapua",
		Country:   "PY",
		Elevation: 269,
		Latitude:  -27.2999992371,
		Longitude: -55.9099998474,
		Timezone:  "America/Asuncion",
	},
	"SGES": {
		ICAO:      "SGES",
		IATA:      "AGT",
		Name:      "Guarani International Airport",
		City:      "Ciudad del Este",
		State:     "Alto-Parana",
		Country:   "PY",
		Elevation: 846,
		Latitude:  -25.4599990845,
		Longitude: -54.8400001526,
		Timezone:  "America/Asuncion",
	},
	"SGFI": {
		ICAO:      "SGFI",
		IATA:      "FLM",
		Name:      "Filadelfia Airport",
		City:      "Filadelfia",
		State:     "Boqueron",
		Country:   "PY",
		Elevation: 423,
		Latitude:  -22.3600006104,
		Longitude: -60.0499992371,
		Timezone:  "America/Asuncion",
	},
	"SGGR": {
		ICAO:      "SGGR",
		Name:      "Salto del Guaira Airport",
		City:      "Salto del Guaira",
		State:     "Canindeyu",
		Country:   "PY",
		Elevation: 990,
		Latitude:  -24.032269,
		Longitude: -54.350829,
		Timezone:  "America/Asuncion",
	},
	"SGIB": {
		ICAO:      "SGIB",
		Name:      "Itaipu Airport",
		City:      "Itaipu",
		State:     "Alto-Parana",
		Country:   "PY",
		Elevation: 762,
		Latitude:  -25.4099998474,
		Longitude: -54.6199989319,
		Timezone:  "America/Asuncion",
	},
	"SGME": {
		ICAO:      "SGME",
		IATA:      "ESG",
		Name:      "Dr. Luis Maria Argana International Airport",
		City:      "Mariscal Estigarribia",
		State:     "Boqueron",
		Country:   "PY",
		Elevation: 553,
		Latitude:  -22.0499992371,
		Longitude: -60.6199989319,
		Timezone:  "America/Asuncion",
	},
	"SGNB": {
		ICAO:      "SGNB",
		Name:      "Villa Hayes Sarg Airport",
		City:      "Villa Hayes",
		State:     "Presidente-Hayes",
		Country:   "PY",
		Elevation: 210,
		Latitude:  -25.1599998474,
		Longitude: -57.5600013733,
		Timezone:  "America/Asuncion",
	},
	"SGOL": {
		ICAO:      "SGOL",
		IATA:      "OLK",
		Name:      "Fuerte Olimpo Airport",
		City:      "Fuerte Olimpo",
		State:     "Alto-Paraguay",
		Country:   "PY",
		Elevation: 275,
		Latitude:  -21.0452003479,
		Longitude: -57.8824996948,
		Timezone:  "America/Asuncion",
	},
	"SGOV": {
		ICAO:      "SGOV",
		Name:      "Coronel Oviedo Airport",
		City:      "Coronel Oviedo",
		State:     "Caaguazu",
		Country:   "PY",
		Elevation: 487,
		Latitude:  -25.5200004578,
		Longitude: -56.4099998474,
		Timezone:  "America/Asuncion",
	},
	"SGPC": {
		ICAO:      "SGPC",
		Name:      "Pozo Colorado Airport",
		City:      "Pozo Colorado",
		State:     "Presidente-Hayes",
		Country:   "PY",
		Elevation: 325,
		Latitude:  -23.499127,
		Longitude: -58.785914,
		Timezone:  "America/Asuncion",
	},
	"SGPG": {
		ICAO:      "SGPG",
		Name:      "Pelayo Prats Gill Airstrip",
		State:     "Boqueron",
		Country:   "PY",
		Elevation: 690,
		Latitude:  -22.555819,
		Longitude: -61.613992,
		Timezone:  "America/Asuncion",
	},
	"SGPI": {
		ICAO:      "SGPI",
		IATA:      "PIL",
		Name:      "Carlos Miguel Gimenez Airport",
		City:      "Pilar",
		State:     "Neembucu",
		Country:   "PY",
		Elevation: 249,
		Latitude:  -26.8799991608,
		Longitude: -58.3199996948,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SGPJ": {
		ICAO:      "SGPJ",
		IATA:      "PJC",
		Name:      "Dr Augusto Roberto Fuster International Airport",
		City:      "Pedro Juan Caballero",
		State:     "Amambay",
		Country:   "PY",
		Elevation: 1873,
		Latitude:  -22.6399993896,
		Longitude: -55.8300018311,
		Timezone:  "America/Asuncion",
	},
	"SGPO": {
		ICAO:      "SGPO",
		Name:      "Puerto Pinasco Airport",
		City:      "Pinasco",
		State:     "Presidente-Hayes",
		Country:   "PY",
		Elevation: 239,
		Latitude:  -22.6299991608,
		Longitude: -57.8400001526,
		Timezone:  "America/Asuncion",
	},
	"SGRO": {
		ICAO:      "SGRO",
		Name:      "Rosario Airport",
		City:      "Rosario",
		State:     "San-Pedro",
		Country:   "PY",
		Elevation: 236,
		Latitude:  -24.4300003052,
		Longitude: -57.1199989319,
		Timezone:  "America/Asuncion",
	},
	"SGSP": {
		ICAO:      "SGSP",
		Name:      "San Pedro Airport",
		City:      "San Pedro del Ycuamandiyu",
		State:     "San-Pedro",
		Country:   "PY",
		Elevation: 270,
		Latitude:  -24.082883,
		Longitude: -57.088112,
		Timezone:  "America/Asuncion",
	},
	"SGST": {
		ICAO:      "SGST",
		Name:      "Santa Teresa Airport",
		City:      "Santa Teresa",
		State:     "Amambay",
		Country:   "PY",
		Elevation: 582,
		Latitude:  -22.6200008392,
		Longitude: -56.6300010681,
		Timezone:  "America/Asuncion",
	},
	"SIAB": {
		ICAO:      "SIAB",
		Name:      "Leda Mello Resende Airport",
		City:      "Tres Pontas",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3050,
		Latitude:  -21.3719005585,
		Longitude: -45.4948997498,
		Timezone:  "America/Sao_Paulo",
	},
	"SIAD": {
		ICAO:      "SIAD",
		Name:      "Estancia Regina Airport",
		City:      "Porto Murtinho",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 394,
		Latitude:  -21.0088882446,
		Longitude: -57.3480567932,
		Timezone:  "America/Campo_Grande",
	},
	"SIAF": {
		ICAO:      "SIAF",
		Name:      "Fazenda Sao Jose Airport",
		City:      "Sapezal",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1903,
		Latitude:  -13.7411108017,
		Longitude: -58.8855552673,
		Timezone:  "America/Cuiaba",
	},
	"SIAI": {
		ICAO:      "SIAI",
		Name:      "Fazenda Gaivota Airport",
		City:      "Pocone",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 541,
		Latitude:  -16.2361106873,
		Longitude: -56.918888092,
		Timezone:  "America/Cuiaba",
	},
	"SIAM": {
		ICAO:      "SIAM",
		IATA:      "0",
		Name:      "Fazenda Palmital Airport",
		City:      "Morro Agudo",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1709,
		Latitude:  -20.6963996887,
		Longitude: -48.2863998413,
		Timezone:  "America/Sao_Paulo",
	},
	"SIAO": {
		ICAO:      "SIAO",
		Name:      "Fazenda Araras Airport",
		City:      "Corguinho",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1493,
		Latitude:  -19.6297225952,
		Longitude: -55.2519454956,
		Timezone:  "America/Campo_Grande",
	},
	"SIAP": {
		ICAO:      "SIAP",
		Name:      "Ana Paula Airport",
		City:      "Acegua",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 584,
		Latitude:  -31.73777771,
		Longitude: -54.0894432068,
		Timezone:  "America/Sao_Paulo",
	},
	"SIAR": {
		ICAO:      "SIAR",
		Name:      "Fazenda Bom Retiro Airport",
		City:      "Ibirarema",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1617,
		Latitude:  -22.7991676331,
		Longitude: -50.0461120605,
		Timezone:  "America/Sao_Paulo",
	},
	"SIAU": {
		ICAO:      "SIAU",
		Name:      "Fazenda Jatoba Airport",
		City:      "Barao De Melgaco",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 404,
		Latitude:  -16.4833335876,
		Longitude: -56.2700004578,
		Timezone:  "America/Cuiaba",
	},
	"SIAX": {
		ICAO:      "SIAX",
		Name:      "Fazenda Beira Rio Airport",
		City:      "Novo Progresso",
		State:     "Para",
		Country:   "BR",
		Elevation: 870,
		Latitude:  -7.5502781868,
		Longitude: -56.1725006104,
		Timezone:  "America/Santarem",
	},
	"SIAY": {
		ICAO:      "SIAY",
		Name:      "Auriflama Airport",
		City:      "Auriflama",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1277,
		Latitude:  -20.7419433594,
		Longitude: -50.531665802,
		Timezone:  "America/Sao_Paulo",
	},
	"SIBA": {
		ICAO:      "SIBA",
		Name:      "Fazenda Jatoba Airport",
		City:      "Aquidauana",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 400,
		Latitude:  -20.0991668701,
		Longitude: -55.9269447327,
		Timezone:  "America/Campo_Grande",
	},
	"SIBC": {
		ICAO:      "SIBC",
		Name:      "Vila de Tocos Airport",
		City:      "Campos Dos Goytacazes",
		State:     "Rio-de-Janeiro",
		Country:   "BR",
		Elevation: 26,
		Latitude:  -21.8777770996,
		Longitude: -41.2947235107,
		Timezone:  "America/Sao_Paulo",
	},
	"SIBD": {
		ICAO:      "SIBD",
		Name:      "Benedito Mutran Airport",
		City:      "Maraba",
		State:     "Para",
		Country:   "BR",
		Elevation: 551,
		Latitude:  -5.7585701942,
		Longitude: -49.1762008667,
		Timezone:  "America/Belem",
	},
	"SIBF": {
		ICAO:      "SIBF",
		Name:      "Bunge Fertilizantes Airport",
		City:      "Cajati",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 249,
		Latitude:  -24.7191677094,
		Longitude: -48.1186103821,
		Timezone:  "America/Sao_Paulo",
	},
	"SIBK": {
		ICAO:      "SIBK",
		Name:      "Sada Siderurgia Ltda Airport",
		City:      "Varzea Da Palma",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1759,
		Latitude:  -17.519443512,
		Longitude: -44.7611122131,
		Timezone:  "America/Sao_Paulo",
	},
	"SIBN": {
		ICAO:      "SIBN",
		Name:      "Fazenda Canaa Airport",
		City:      "Piraju",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2021,
		Latitude:  -23.0713882446,
		Longitude: -49.4324989319,
		Timezone:  "America/Sao_Paulo",
	},
	"SIBO": {
		ICAO:      "SIBO",
		Name:      "Sao Luiz Airport",
		City:      "Boa Esperanca",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1919,
		Latitude:  -24.2625007629,
		Longitude: -52.7922210693,
		Timezone:  "America/Sao_Paulo",
	},
	"SIBU": {
		ICAO:      "SIBU",
		Name:      "Catole da Rocha Airport",
		City:      "Catole Da Rocha",
		State:     "Paraiba",
		Country:   "BR",
		Elevation: 892,
		Latitude:  -6.3625302315,
		Longitude: -37.7561988831,
		Timezone:  "America/Fortaleza",
	},
	"SIBV": {
		ICAO:      "SIBV",
		Name:      "Fazenda Bela Vista Airport",
		City:      "Nova Alvorada Do Sul",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1083,
		Latitude:  -21.4486103058,
		Longitude: -54.488609314,
		Timezone:  "America/Campo_Grande",
	},
	"SIBW": {
		ICAO:      "SIBW",
		Name:      "Conceicao Airport",
		City:      "Conceicao",
		State:     "Paraiba",
		Country:   "BR",
		Elevation: 1214,
		Latitude:  -7.5582299232,
		Longitude: -38.4964981079,
		Timezone:  "America/Fortaleza",
	},
	"SIBX": {
		ICAO:      "SIBX",
		Name:      "Rosana Camargo Airport",
		City:      "Buritama",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1345,
		Latitude:  -21.0911102295,
		Longitude: -50.2433319092,
		Timezone:  "America/Sao_Paulo",
	},
	"SIBY": {
		ICAO:      "SIBY",
		Name:      "Monteiro Airport",
		City:      "Monteiro",
		State:     "Paraiba",
		Country:   "BR",
		Elevation: 2031,
		Latitude:  -7.8782401085,
		Longitude: -37.1399993896,
		Timezone:  "America/Fortaleza",
	},
	"SIBZ": {
		ICAO:      "SIBZ",
		Name:      "Itaporanga Airport",
		City:      "Itaporanga",
		State:     "Paraiba",
		Country:   "BR",
		Elevation: 958,
		Latitude:  -7.3116598129,
		Longitude: -38.112701416,
		Timezone:  "America/Fortaleza",
	},
	"SICA": {
		ICAO:      "SICA",
		Name:      "Brejo das Freiras Airport",
		City:      "Sao Joao Do Rio Do Peixe",
		State:     "Paraiba",
		Country:   "BR",
		Elevation: 1017,
		Latitude:  -6.7041668892,
		Longitude: -38.5027770996,
		Timezone:  "America/Fortaleza",
	},
	"SICB": {
		ICAO:      "SICB",
		Name:      "Pedro Simoes Pimenta Airport",
		City:      "Cuite",
		State:     "Paraiba",
		Country:   "BR",
		Elevation: 2139,
		Latitude:  -6.4875001907,
		Longitude: -36.1399993896,
		Timezone:  "America/Fortaleza",
	},
	"SICH": {
		ICAO:      "SICH",
		Name:      "Fazenda Serrinha Airport",
		City:      "Tres Lagoas",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1188,
		Latitude:  -20.6608333588,
		Longitude: -52.3400001526,
		Timezone:  "America/Campo_Grande",
	},
	"SICJ": {
		ICAO:      "SICJ",
		Name:      "Gaivota Aviacao Agricola Airport",
		City:      "Sapezal",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1706,
		Latitude:  -13.4600000381,
		Longitude: -58.7750015259,
		Timezone:  "America/Cuiaba",
	},
	"SICK": {
		ICAO:      "SICK",
		Name:      "Cidade Capelinha Airport",
		City:      "Capelinha",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3113,
		Latitude:  -17.6827774048,
		Longitude: -42.5311126709,
		Timezone:  "America/Sao_Paulo",
	},
	"SICM": {
		ICAO:      "SICM",
		Name:      "Fazenda Nossa Senhora de Fatima Airport",
		City:      "Brasilandia",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 984,
		Latitude:  -21.5388889313,
		Longitude: -52.1105575562,
		Timezone:  "America/Campo_Grande",
	},
	"SICN": {
		ICAO:      "SICN",
		Name:      "Fazenda Vista Bonita Airport",
		City:      "Sandovalina",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1348,
		Latitude:  -22.5033321381,
		Longitude: -51.8127784729,
		Timezone:  "America/Sao_Paulo",
	},
	"SICO": {
		ICAO:      "SICO",
		Name:      "SOLAG - Sol e Lua Aviacao Agricola Airport",
		City:      "Lucas Do Rio Verde",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1453,
		Latitude:  -13.3197221756,
		Longitude: -56.0469436646,
		Timezone:  "America/Cuiaba",
	},
	"SICP": {
		ICAO:      "SICP",
		Name:      "Doutor Walter Beckert Airport",
		City:      "Faxinal",
		State:     "Parana",
		Country:   "BR",
		Elevation: 2779,
		Latitude:  -23.9716663361,
		Longitude: -51.3236122131,
		Timezone:  "America/Sao_Paulo",
	},
	"SICW": {
		ICAO:      "SICW",
		Name:      "Fazenda Santa Fe Airport",
		City:      "Campo Alegre De Goias",
		State:     "Goias",
		Country:   "BR",
		Elevation: 3094,
		Latitude:  -17.4638881683,
		Longitude: -47.8316688538,
		Timezone:  "America/Sao_Paulo",
	},
	"SICY": {
		ICAO:      "SICY",
		Name:      "Fazenda do Cedro Airport",
		City:      "Chapada",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 2143,
		Latitude:  -28.1083335876,
		Longitude: -53.1963882446,
		Timezone:  "America/Sao_Paulo",
	},
	"SIDB": {
		ICAO:      "SIDB",
		Name:      "Doutor Almir Lopes de Oliveira Melo Airport",
		City:      "Coelho Neto",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 174,
		Latitude:  -4.2688889503,
		Longitude: -43.0005569458,
		Timezone:  "America/Fortaleza",
	},
	"SIDD": {
		ICAO:      "SIDD",
		Name:      "Fazenda Sao Joao do Monte Alto Airport",
		City:      "Maracaju",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1476,
		Latitude:  -21.4972229004,
		Longitude: -55.2638893127,
		Timezone:  "America/Campo_Grande",
	},
	"SIDG": {
		ICAO:      "SIDG",
		Name:      "Fazenda Jatobasso Airport",
		City:      "Ponta Pora",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 2152,
		Latitude:  -22.4291667938,
		Longitude: -55.5333328247,
		Timezone:  "America/Campo_Grande",
	},
	"SIDM": {
		ICAO:      "SIDM",
		Name:      "Fazenda Cibrapa Airport",
		City:      "Barra Do Garcas",
		Country:   "BR",
		Elevation: 821,
		Latitude:  -15.0383329391,
		Longitude: -52.0983314514,
		Timezone:  "America/Cuiaba",
	},
	"SIDN": {
		ICAO:      "SIDN",
		Name:      "Fazenda Ipanema Airport",
		City:      "Ipero",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1985,
		Latitude:  -23.4255561829,
		Longitude: -47.5813903809,
		Timezone:  "America/Sao_Paulo",
	},
	"SIDR": {
		ICAO:      "SIDR",
		Name:      "Retiro Piuva Airport",
		City:      "Miranda",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 328,
		Latitude:  -19.8355560303,
		Longitude: -56.7036094666,
		Timezone:  "America/Campo_Grande",
	},
	"SIDS": {
		ICAO:      "SIDS",
		Name:      "Fazenda Santa Lucinha Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 300,
		Latitude:  -18.0233325958,
		Longitude: -56.7322235107,
		Timezone:  "America/Campo_Grande",
	},
	"SIDY": {
		ICAO:      "SIDY",
		Name:      "Fazenda Yanduy Airport",
		City:      "Lunardelle",
		State:     "Parana",
		Country:   "BR",
		Elevation: 985,
		Latitude:  -24.0774993896,
		Longitude: -51.819442749,
		Timezone:  "America/Sao_Paulo",
	},
	"SIDZ": {
		ICAO:      "SIDZ",
		Name:      "Estancia Ayrton Senna Airport",
		City:      "Caarapo",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1509,
		Latitude:  -22.6561107635,
		Longitude: -54.8222236633,
		Timezone:  "America/Campo_Grande",
	},
	"SIEA": {
		ICAO:      "SIEA",
		Name:      "Fazenda Ibipora Airport",
		City:      "Guararapes",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1181,
		Latitude:  -21.4633331299,
		Longitude: -50.7966651917,
		Timezone:  "America/Sao_Paulo",
	},
	"SIEB": {
		ICAO:      "SIEB",
		Name:      "Fazenda Carrapicho Airport",
		City:      "Dois Irmaos Do Buriti",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 932,
		Latitude:  -20.7469444275,
		Longitude: -55.3572235107,
		Timezone:  "America/Campo_Grande",
	},
	"SIEC": {
		ICAO:      "SIEC",
		Name:      "Fazenda Anhumas II Airport",
		City:      "Presidente Eptacio",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 955,
		Latitude:  -22.0424995422,
		Longitude: -52.2913894653,
		Timezone:  "America/Sao_Paulo",
	},
	"SIED": {
		ICAO:      "SIED",
		Name:      "Colonial Airport",
		City:      "Verdelandia",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1621,
		Latitude:  -15.4513893127,
		Longitude: -43.4052772522,
		Timezone:  "America/Sao_Paulo",
	},
	"SIEF": {
		ICAO:      "SIEF",
		Name:      "Fazenda Sao Jose Airport",
		City:      "Campo Grande",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1722,
		Latitude:  -20.5733337402,
		Longitude: -54.7169456482,
		Timezone:  "America/Campo_Grande",
	},
	"SIEG": {
		ICAO:      "SIEG",
		Name:      "Fazenda Santa Edwiges Airport",
		City:      "Taquarussu",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 951,
		Latitude:  -22.653055191,
		Longitude: -53.3930549622,
		Timezone:  "America/Campo_Grande",
	},
	"SIEJ": {
		ICAO:      "SIEJ",
		Name:      "Monte Verde Airport",
		City:      "Nova Monte Verde",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 821,
		Latitude:  -9.947221756,
		Longitude: -57.4997215271,
		Timezone:  "America/Cuiaba",
	},
	"SIEL": {
		ICAO:      "SIEL",
		Name:      "Fazenda Varzea Funda Airport",
		City:      "Caceres",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 709,
		Latitude:  -16.5836105347,
		Longitude: -57.7322235107,
		Timezone:  "America/Cuiaba",
	},
	"SIEN": {
		ICAO:      "SIEN",
		Name:      "Fazenda Sao Marcos Airport",
		City:      "General Carneiro",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1503,
		Latitude:  -15.404999733,
		Longitude: -53.3422203064,
		Timezone:  "America/Cuiaba",
	},
	"SIET": {
		ICAO:      "SIET",
		Name:      "Estancia Tara Airport",
		City:      "Miranda",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 607,
		Latitude:  -20.0883331299,
		Longitude: -56.7247238159,
		Timezone:  "America/Campo_Grande",
	},
	"SIEV": {
		ICAO:      "SIEV",
		Name:      "Fazenda Boca da Onca Airport",
		City:      "Bodoquena",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1050,
		Latitude:  -20.7669448853,
		Longitude: -56.706111908,
		Timezone:  "America/Campo_Grande",
	},
	"SIEZ": {
		ICAO:      "SIEZ",
		Name:      "Fazenda Pedra Branca Airport",
		City:      "Tibagi",
		State:     "Parana",
		Country:   "BR",
		Elevation: 2953,
		Latitude:  -24.6241664886,
		Longitude: -50.4591674805,
		Timezone:  "America/Sao_Paulo",
	},
	"SIFA": {
		ICAO:      "SIFA",
		Name:      "Fazenda Sete Airport",
		City:      "Miranda",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 328,
		Latitude:  -19.7855567932,
		Longitude: -56.4738883972,
		Timezone:  "America/Campo_Grande",
	},
	"SIFC": {
		ICAO:      "SIFC",
		Name:      "Fazenda Cachoeirinha Airport",
		City:      "Miranda",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 722,
		Latitude:  -20.3822212219,
		Longitude: -56.2688903809,
		Timezone:  "America/Campo_Grande",
	},
	"SIFD": {
		ICAO:      "SIFD",
		Name:      "Fazenda Dois Irmaos Airport",
		City:      "Caceres",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 591,
		Latitude:  -16.38722229,
		Longitude: -57.2886123657,
		Timezone:  "America/Cuiaba",
	},
	"SIFE": {
		ICAO:      "SIFE",
		Name:      "Fazenda Dinora Airport",
		City:      "Nova Fatima",
		State:     "Parana",
		Country:   "BR",
		Elevation: 2100,
		Latitude:  -23.4163894653,
		Longitude: -50.6008338928,
		Timezone:  "America/Sao_Paulo",
	},
	"SIFF": {
		ICAO:      "SIFF",
		Name:      "Fazenda Firme Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 233,
		Latitude:  -19.2594432831,
		Longitude: -57.0116653442,
		Timezone:  "America/Campo_Grande",
	},
	"SIFH": {
		ICAO:      "SIFH",
		Name:      "Fazenda Santa Rita do Araguaia Airport",
		City:      "Santa Rita Do Araguaia",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2464,
		Latitude:  -17.3252773285,
		Longitude: -53.1969451904,
		Timezone:  "America/Cuiaba",
	},
	"SIFJ": {
		ICAO:      "SIFJ",
		Name:      "Fazenda Jurema Airport",
		City:      "Rio Brilhante",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 709,
		Latitude:  -21.8052787781,
		Longitude: -54.2463874817,
		Timezone:  "America/Campo_Grande",
	},
	"SIFM": {
		ICAO:      "SIFM",
		Name:      "Fazenda Mudanca Airport",
		City:      "Aripuana",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 499,
		Latitude:  -9.9122219086,
		Longitude: -60.8619422913,
		Timezone:  "America/Cuiaba",
	},
	"SIFN": {
		ICAO:      "SIFN",
		Name:      "Fazenda Planura Airport",
		City:      "Autazes",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 188,
		Latitude:  -3.593611002,
		Longitude: -59.6522216797,
		Timezone:  "America/Manaus",
	},
	"SIFQ": {
		ICAO:      "SIFQ",
		Name:      "Condominio Menega Airport",
		City:      "Flores Da Cunha",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 2566,
		Latitude:  -29.0444431305,
		Longitude: -51.1388893127,
		Timezone:  "America/Sao_Paulo",
	},
	"SIFR": {
		ICAO:      "SIFR",
		Name:      "Fazenda Romaria Airport",
		City:      "Romaria",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3297,
		Latitude:  -18.8705997467,
		Longitude: -47.5593986511,
		Timezone:  "America/Sao_Paulo",
	},
	"SIFT": {
		ICAO:      "SIFT",
		Name:      "Fazenda Travessao Airport",
		City:      "Vila Bela Da Santissima Trindade",
		State:     "Santa-Cruz",
		Country:   "BR",
		Elevation: 906,
		Latitude:  -16.1477775574,
		Longitude: -60.1166687012,
		Timezone:  "America/Cuiaba",
	},
	"SIFU": {
		ICAO:      "SIFU",
		Name:      "Fazenda Ribeirao Airport",
		City:      "Barra Do Pirai",
		State:     "Rio-de-Janeiro",
		Country:   "BR",
		Elevation: 1319,
		Latitude:  -22.415555954,
		Longitude: -43.7752761841,
		Timezone:  "America/Sao_Paulo",
	},
	"SIFV": {
		ICAO:      "SIFV",
		Name:      "Primo Bitti Airport",
		City:      "Aracruz",
		State:     "Espirito-Santo",
		Country:   "BR",
		Elevation: 88,
		Latitude:  -19.8264007568,
		Longitude: -40.1007995605,
		Timezone:  "America/Sao_Paulo",
	},
	"SIFW": {
		ICAO:      "SIFW",
		Name:      "Fazenda Sao Lourenco Airport",
		City:      "Corumba",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 400,
		Latitude:  -17.7183322906,
		Longitude: -57.0013885498,
		Timezone:  "America/Campo_Grande",
	},
	"SIFY": {
		ICAO:      "SIFY",
		Name:      "Fazenda Yndiana Airport",
		City:      "Porto Murtinho",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 607,
		Latitude:  -21.7783336639,
		Longitude: -57.4958343506,
		Timezone:  "America/Campo_Grande",
	},
	"SIGF": {
		ICAO:      "SIGF",
		Name:      "Fazenda Jopejo Airport",
		City:      "Chapada Dos Guimaraes",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1020,
		Latitude:  -15,
		Longitude: -55.8250007629,
		Timezone:  "America/Cuiaba",
	},
	"SIGL": {
		ICAO:      "SIGL",
		Name:      "Fazenda Boa Sorte Airport",
		City:      "Nova Andradina",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1099,
		Latitude:  -21.9152774811,
		Longitude: -53.5175018311,
		Timezone:  "America/Campo_Grande",
	},
	"SIGM": {
		ICAO:      "SIGM",
		Name:      "Fazenda Sao Benedito Airport",
		City:      "Caceres",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 469,
		Latitude:  -16.2705554962,
		Longitude: -57.86277771,
		Timezone:  "America/Cuiaba",
	},
	"SIGN": {
		ICAO:      "SIGN",
		Name:      "Asa Delta Airport",
		City:      "General Carneiro",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1585,
		Latitude:  -15.5713891983,
		Longitude: -53.2336120605,
		Timezone:  "America/Cuiaba",
	},
	"SIGO": {
		ICAO:      "SIGO",
		Name:      "Fazenda Araguari Airport",
		City:      "Gaucha Do Norte",
		Country:   "BR",
		Elevation: 2126,
		Latitude:  -13.3366670609,
		Longitude: -53.9577789307,
		Timezone:  "America/Cuiaba",
	},
	"SIGS": {
		ICAO:      "SIGS",
		Name:      "Fazenda Granja Santiago Airport",
		City:      "Paracatu",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1946,
		Latitude:  -17.1622219086,
		Longitude: -46.6599998474,
		Timezone:  "America/Sao_Paulo",
	},
	"SIGT": {
		ICAO:      "SIGT",
		Name:      "Fazenda Goiabeira Airport",
		City:      "Itabera",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2260,
		Latitude:  -23.9894447327,
		Longitude: -49.1791687012,
		Timezone:  "America/Sao_Paulo",
	},
	"SIGU": {
		ICAO:      "SIGU",
		Name:      "Fazenda Cafe no Bule Airport",
		City:      "Ribas Do Rio Pardo",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 2199,
		Latitude:  -19.8152770996,
		Longitude: -53.8258323669,
		Timezone:  "America/Campo_Grande",
	},
	"SIGW": {
		ICAO:      "SIGW",
		Name:      "Estancia Miranda Airport",
		City:      "Caracol",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 640,
		Latitude:  -21.9144439697,
		Longitude: -57.1438903809,
		Timezone:  "America/Campo_Grande",
	},
	"SIGX": {
		ICAO:      "SIGX",
		Name:      "Fazenda Gairova Airport",
		City:      "Juara",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1083,
		Latitude:  -11.1977777481,
		Longitude: -58.2366676331,
		Timezone:  "America/Cuiaba",
	},
	"SIGY": {
		ICAO:      "SIGY",
		Name:      "Fazenda Progresso Airport",
		City:      "Sebastiao Leal",
		State:     "Piaui",
		Country:   "BR",
		Elevation: 1476,
		Latitude:  -7.5025000572,
		Longitude: -44.2033348083,
		Timezone:  "America/Fortaleza",
	},
	"SIHB": {
		ICAO:      "SIHB",
		Name:      "Fazenda Soberana Airport",
		City:      "Grajau",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 417,
		Latitude:  -5.4558329582,
		Longitude: -46.0625,
		Timezone:  "America/Fortaleza",
	},
	"SIHG": {
		ICAO:      "SIHG",
		Name:      "Fazenda Ponderosa Airport",
		City:      "Pium",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 656,
		Latitude:  -10.0716667175,
		Longitude: -49.9955558777,
		Timezone:  "America/Araguaina",
	},
	"SIHI": {
		ICAO:      "SIHI",
		Name:      "Fazenda Itaipavas Airport",
		City:      "Picarras",
		State:     "Para",
		Country:   "BR",
		Elevation: 633,
		Latitude:  -6.6163887978,
		Longitude: -48.9958343506,
		Timezone:  "America/Belem",
	},
	"SIHJ": {
		ICAO:      "SIHJ",
		Name:      "Sitio Flyer Airport",
		City:      "Palmas",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 958,
		Latitude:  -10.1080560684,
		Longitude: -48.3263893127,
		Timezone:  "America/Araguaina",
	},
	"SIHN": {
		ICAO:      "SIHN",
		Name:      "Estancia Vacadiga Airport",
		City:      "Bela Vista",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 936,
		Latitude:  -21.7986106873,
		Longitude: -56.5713882446,
		Timezone:  "America/Campo_Grande",
	},
	"SIHV": {
		ICAO:      "SIHV",
		Name:      "Fazenda Boa Sorte Airport",
		City:      "Paracatu",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2002,
		Latitude:  -17.2111110687,
		Longitude: -46.6144447327,
		Timezone:  "America/Sao_Paulo",
	},
	"SIHW": {
		ICAO:      "SIHW",
		Name:      "Agrishow Airport",
		City:      "Ribeirao Preto",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1781,
		Latitude:  -21.2188892365,
		Longitude: -47.8763885498,
		Timezone:  "America/Sao_Paulo",
	},
	"SIIA": {
		ICAO:      "SIIA",
		Name:      "Fazenda Parana Airport",
		City:      "Brasnorte",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 653,
		Latitude:  -11.6999998093,
		Longitude: -58.2180557251,
		Timezone:  "America/Cuiaba",
	},
	"SIID": {
		ICAO:      "SIID",
		Name:      "Crepurizao Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 695,
		Latitude:  -6.8200001717,
		Longitude: -56.8422203064,
		Timezone:  "America/Santarem",
	},
	"SIIE": {
		ICAO:      "SIIE",
		Name:      "Fazenda Cachoeira Branca Airport",
		City:      "Ribas Do Rio Pardo",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1733,
		Latitude:  -20.8919448853,
		Longitude: -53.4041671753,
		Timezone:  "America/Campo_Grande",
	},
	"SIIG": {
		ICAO:      "SIIG",
		Name:      "Eliza Camargo de Arruda Botelho Airport",
		City:      "Rancharia",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1756,
		Latitude:  -22.3575000763,
		Longitude: -51.1008338928,
		Timezone:  "America/Sao_Paulo",
	},
	"SIII": {
		ICAO:      "SIII",
		IATA:      "0",
		Name:      "Doutor Heraclito da Motta Luiz Airport",
		City:      "Guaira",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1798,
		Latitude:  -20.4330997467,
		Longitude: -48.2243995667,
		Timezone:  "America/Sao_Paulo",
	},
	"SIIK": {
		ICAO:      "SIIK",
		Name:      "Fazenda Santa Lidia Airport",
		City:      "Ribas Do Rio Pardo",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1811,
		Latitude:  -20.7316665649,
		Longitude: -53.6852798462,
		Timezone:  "America/Campo_Grande",
	},
	"SIIM": {
		ICAO:      "SIIM",
		Name:      "Fazenda Girua Airport",
		City:      "Rio Verde Do Mato Grosso",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1408,
		Latitude:  -18.8024997711,
		Longitude: -54.8477783203,
		Timezone:  "America/Campo_Grande",
	},
	"SIIN": {
		ICAO:      "SIIN",
		Name:      "Independencia Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 951,
		Latitude:  -7.5949997902,
		Longitude: -56.7027778625,
		Timezone:  "America/Santarem",
	},
	"SIIQ": {
		ICAO:      "SIIQ",
		Name:      "Fazenda Aracoxim Airport",
		City:      "Sonora",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2198,
		Latitude:  -17.3974990845,
		Longitude: -53.7186126709,
		Timezone:  "America/Campo_Grande",
	},
	"SIIS": {
		ICAO:      "SIIS",
		Name:      "Fazenda Uberaba Airport",
		City:      "Vila Alta",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1100,
		Latitude:  -23.6499996185,
		Longitude: -53.8302764893,
		Timezone:  "America/Sao_Paulo",
	},
	"SIIU": {
		ICAO:      "SIIU",
		Name:      "Santa Marta Airport",
		City:      "Campo Florido",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2218,
		Latitude:  -19.643611908,
		Longitude: -48.7008323669,
		Timezone:  "America/Sao_Paulo",
	},
	"SIIX": {
		ICAO:      "SIIX",
		Name:      "Fazenda Corrego Fundo Airport",
		City:      "Agua Boa",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1207,
		Latitude:  -13.7369441986,
		Longitude: -52.5894432068,
		Timezone:  "America/Cuiaba",
	},
	"SIIZ": {
		ICAO:      "SIIZ",
		Name:      "Fazenda Chalana Airport",
		City:      "Diamantino",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2001,
		Latitude:  -14.1644439697,
		Longitude: -57.162776947,
		Timezone:  "America/Cuiaba",
	},
	"SIJE": {
		ICAO:      "SIJE",
		Name:      "Fazenda Joana Estancia Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 476,
		Latitude:  -20.0069446564,
		Longitude: -57.3916664124,
		Timezone:  "America/Campo_Grande",
	},
	"SIJG": {
		ICAO:      "SIJG",
		Name:      "Fazenda Jaguarete Airport",
		City:      "Guararapes",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1290,
		Latitude:  -21.4433326721,
		Longitude: -50.7233314514,
		Timezone:  "America/Sao_Paulo",
	},
	"SIJJ": {
		ICAO:      "SIJJ",
		Name:      "Fazenda Salamanca Airport",
		City:      "Aral Moreira",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1181,
		Latitude:  -22.8086109161,
		Longitude: -55.5333328247,
		Timezone:  "America/Campo_Grande",
	},
	"SIJK": {
		ICAO:      "SIJK",
		Name:      "Vazante Airport",
		City:      "Vazante",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2457,
		Latitude:  -17.9549999237,
		Longitude: -46.8616676331,
		Timezone:  "America/Sao_Paulo",
	},
	"SIJM": {
		ICAO:      "SIJM",
		Name:      "Fazenda Santa Maria do Porto da Capivara Airport",
		City:      "Barao De Melgaco",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 328,
		Latitude:  -17.144443512,
		Longitude: -56.5977783203,
		Timezone:  "America/Cuiaba",
	},
	"SIJO": {
		ICAO:      "SIJO",
		Name:      "Fazenda Sao Joao do Jatobazinho Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 381,
		Latitude:  -18.577222,
		Longitude: -57.511111,
		Timezone:  "America/Campo_Grande",
	},
	"SIJR": {
		ICAO:      "SIJR",
		Name:      "Ely Rego Airport",
		City:      "Balsa Nova",
		State:     "Parana",
		Country:   "BR",
		Elevation: 2592,
		Latitude:  -25.5394439697,
		Longitude: -49.5999984741,
		Timezone:  "America/Sao_Paulo",
	},
	"SIJU": {
		ICAO:      "SIJU",
		Name:      "Fazenda Areia Branca Airport",
		City:      "Juina",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1073,
		Latitude:  -11.7838888168,
		Longitude: -58.4533348083,
		Timezone:  "America/Cuiaba",
	},
	"SIJZ": {
		ICAO:      "SIJZ",
		Name:      "Fazenda Vera Paz Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 801,
		Latitude:  -7.3947219849,
		Longitude: -56.7647209167,
		Timezone:  "America/Santarem",
	},
	"SIKB": {
		ICAO:      "SIKB",
		Name:      "Agua Boa Airport",
		City:      "Agua Boa",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2815,
		Latitude:  -14.009721756,
		Longitude: -52.1502761841,
		Timezone:  "America/Cuiaba",
	},
	"SIKC": {
		ICAO:      "SIKC",
		Name:      "Colider Airport",
		City:      "Colider",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1266,
		Latitude:  -10.764166832,
		Longitude: -55.4611091614,
		Timezone:  "America/Cuiaba",
	},
	"SIKJ": {
		ICAO:      "SIKJ",
		Name:      "Fazenda Cristalino Airport",
		City:      "Alta Floresta",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 919,
		Latitude:  -9.6677780151,
		Longitude: -55.9766654968,
		Timezone:  "America/Cuiaba",
	},
	"SIKK": {
		ICAO:      "SIKK",
		Name:      "Calciolandia Airport",
		City:      "Arcos",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2180,
		Latitude:  -20.2505569458,
		Longitude: -45.6450004578,
		Timezone:  "America/Sao_Paulo",
	},
	"SIKM": {
		ICAO:      "SIKM",
		Name:      "Fazenda Viamao Airport",
		City:      "Arame",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 863,
		Latitude:  -5.2891669273,
		Longitude: -46.0016670227,
		Timezone:  "America/Fortaleza",
	},
	"SIKN": {
		ICAO:      "SIKN",
		Name:      "Fazenda Bonanza Airport",
		City:      "Arame",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 928,
		Latitude:  -5.1572217941,
		Longitude: -45.9155578613,
		Timezone:  "America/Fortaleza",
	},
	"SIKO": {
		ICAO:      "SIKO",
		Name:      "Fazenda Pison Airport",
		City:      "Maues",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 541,
		Latitude:  -5.0199999809,
		Longitude: -57.7802772522,
		Timezone:  "America/Manaus",
	},
	"SIKP": {
		ICAO:      "SIKP",
		Name:      "Fazenda Santa Teresa Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 404,
		Latitude:  -18.303611,
		Longitude: -57.506944,
		Timezone:  "America/Campo_Grande",
	},
	"SIKQ": {
		ICAO:      "SIKQ",
		Name:      "Fazenda Samauma Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 633,
		Latitude:  -7.2361111641,
		Longitude: -57.1058311462,
		Timezone:  "America/Santarem",
	},
	"SIKX": {
		ICAO:      "SIKX",
		Name:      "Fazenda Continental Airport",
		City:      "Colombia",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1476,
		Latitude:  -20.2411117554,
		Longitude: -48.6583328247,
		Timezone:  "America/Sao_Paulo",
	},
	"SIKY": {
		ICAO:      "SIKY",
		Name:      "Fazenda Beira Rio Airport",
		City:      "Costa Rica",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 2100,
		Latitude:  -18.7194442749,
		Longitude: -53.3375015259,
		Timezone:  "America/Campo_Grande",
	},
	"SIKZ": {
		ICAO:      "SIKZ",
		Name:      "Fazenda Olho D`Agua Airport",
		City:      "Parnarama",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 390,
		Latitude:  -5.6666669846,
		Longitude: -43.5349998474,
		Timezone:  "America/Fortaleza",
	},
	"SILA": {
		ICAO:      "SILA",
		Name:      "Fazenda Santa Ada Airport",
		City:      "Jatei",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1247,
		Latitude:  -22.7583332062,
		Longitude: -53.844165802,
		Timezone:  "America/Campo_Grande",
	},
	"SILB": {
		ICAO:      "SILB",
		Name:      "Aviador Mario Luiz Spinelli Airport",
		City:      "Chapada Dos Guimaraes",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1247,
		Latitude:  -15.0033330917,
		Longitude: -55.5011100769,
		Timezone:  "America/Cuiaba",
	},
	"SILC": {
		ICAO:      "SILC",
		IATA:      "LVR",
		Name:      "Municipal Bom Futuro Airport",
		City:      "Lucas do Rio Verde",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1345,
		Latitude:  -13.0378611111,
		Longitude: -55.95025,
		Timezone:  "America/Cuiaba",
	},
	"SILD": {
		ICAO:      "SILD",
		Name:      "Fazenda Colorado Airport",
		City:      "Bodoquena",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 512,
		Latitude:  -20.6305561066,
		Longitude: -56.7719421387,
		Timezone:  "America/Campo_Grande",
	},
	"SILG": {
		ICAO:      "SILG",
		Name:      "Fazenda Panama Airport",
		City:      "Sapezal",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1690,
		Latitude:  -13.76,
		Longitude: -58.74722,
		Timezone:  "America/Cuiaba",
	},
	"SILI": {
		ICAO:      "SILI",
		Name:      "Ilha dos Porcos Grandes Airport",
		City:      "Angra Dos Reis",
		State:     "Rio-de-Janeiro",
		Country:   "BR",
		Elevation: 197,
		Latitude:  -23.058889389,
		Longitude: -44.3144454956,
		Timezone:  "America/Sao_Paulo",
	},
	"SILJ": {
		ICAO:      "SILJ",
		Name:      "Do Gaucho Airport",
		City:      "Laranjal Do Jari",
		State:     "Amapa",
		Country:   "BR",
		Elevation: 135,
		Latitude:  -0.8333330154,
		Longitude: -52.0525016785,
		Timezone:  "America/Belem",
	},
	"SILM": {
		ICAO:      "SILM",
		Name:      "Fazenda Regalito Airport",
		City:      "Flores De Goias",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1598,
		Latitude:  -14.4705562592,
		Longitude: -46.8725013733,
		Timezone:  "America/Sao_Paulo",
	},
	"SILN": {
		ICAO:      "SILN",
		Name:      "Fazenda Rio Capim Airport",
		City:      "Paragominas",
		State:     "Para",
		Country:   "BR",
		Elevation: 325,
		Latitude:  -3.5586109161,
		Longitude: -48.6563873291,
		Timezone:  "America/Belem",
	},
	"SILQ": {
		ICAO:      "SILQ",
		Name:      "Clube de Ultraleves Aeroleve Airport",
		City:      "Cascavel",
		State:     "Parana",
		Country:   "BR",
		Elevation: 2132,
		Latitude:  -24.9266662598,
		Longitude: -53.4983329773,
		Timezone:  "America/Sao_Paulo",
	},
	"SILS": {
		ICAO:      "SILS",
		Name:      "Agua Branca Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 667,
		Latitude:  -6.390832901,
		Longitude: -56.2849998474,
		Timezone:  "America/Santarem",
	},
	"SILT": {
		ICAO:      "SILT",
		Name:      "Fazenda Trez Irmaos Airport",
		City:      "Santa Fe Do Sul",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1460,
		Latitude:  -20.2277774811,
		Longitude: -50.968334198,
		Timezone:  "America/Sao_Paulo",
	},
	"SILU": {
		ICAO:      "SILU",
		Name:      "Fazenda Cristo Airport",
		City:      "Miranda",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 427,
		Latitude:  -19.8658332825,
		Longitude: -56.9872207642,
		Timezone:  "America/Campo_Grande",
	},
	"SILW": {
		ICAO:      "SILW",
		Name:      "Fazenda Mata Assombrada Airport",
		City:      "Bela Vista",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 912,
		Latitude:  -21.727222,
		Longitude: -56.509167,
		Timezone:  "America/Campo_Grande",
	},
	"SILX": {
		ICAO:      "SILX",
		Name:      "Fazenda Jandaia Airport",
		City:      "Peixoto De Azevedo",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1070,
		Latitude:  -10.51,
		Longitude: -53.65,
		Timezone:  "America/Cuiaba",
	},
	"SILY": {
		ICAO:      "SILY",
		Name:      "Fazenda Tres Marias Airport",
		City:      "Vila Alta",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1300,
		Latitude:  -23.5119438171,
		Longitude: -53.8041687012,
		Timezone:  "America/Sao_Paulo",
	},
	"SILZ": {
		ICAO:      "SILZ",
		Name:      "Fazenda Mariad Airport",
		City:      "Juazeiro",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1197,
		Latitude:  -9.3922224045,
		Longitude: -40.3574981689,
		Timezone:  "America/Bahia",
	},
	"SIMA": {
		ICAO:      "SIMA",
		Name:      "Fazenda Mandiore Airport",
		City:      "Corumba",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 299,
		Latitude:  -17.927778244,
		Longitude: -57.624168396,
		Timezone:  "America/Campo_Grande",
	},
	"SIME": {
		ICAO:      "SIME",
		Name:      "Comandante Carlos Inacio Agnes Airport",
		City:      "Davinopolis",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 443,
		Latitude:  -5.576666832,
		Longitude: -47.4227790833,
		Timezone:  "America/Fortaleza",
	},
	"SIMF": {
		ICAO:      "SIMF",
		Name:      "Fazenda Sao Miguel Airport",
		City:      "Unai",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3287,
		Latitude:  -15.9730558395,
		Longitude: -46.7094421387,
		Timezone:  "America/Sao_Paulo",
	},
	"SIMG": {
		ICAO:      "SIMG",
		Name:      "Fazenda Seis Palmas Airport",
		City:      "Porto Murtinho",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 312,
		Latitude:  -21.0994434357,
		Longitude: -57.7158317566,
		Timezone:  "America/Campo_Grande",
	},
	"SIMI": {
		ICAO:      "SIMI",
		Name:      "Fazenda Boa Vista Airport",
		City:      "Tacuru",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1231,
		Latitude:  -23.6369438171,
		Longitude: -55.1844444275,
		Timezone:  "America/Campo_Grande",
	},
	"SIMJ": {
		ICAO:      "SIMJ",
		Name:      "Capinopolis - Aviacao Agricola Buttarello Ltda. Airport",
		City:      "Capinopolis",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2005,
		Latitude:  -18.6592006683,
		Longitude: -49.5475997925,
		Timezone:  "America/Sao_Paulo",
	},
	"SIMK": {
		ICAO:      "SIMK",
		IATA:      "FRC",
		Name:      "Franca Airport",
		City:      "Franca",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 3292,
		Latitude:  -20.5921993256,
		Longitude: -47.382900238,
		Timezone:  "America/Sao_Paulo",
	},
	"SIML": {
		ICAO:      "SIML",
		Name:      "Codenorte Airport",
		City:      "Moju",
		State:     "Para",
		Country:   "BR",
		Elevation: 220,
		Latitude:  -3.285556078,
		Longitude: -49.206111908,
		Timezone:  "America/Belem",
	},
	"SIMO": {
		ICAO:      "SIMO",
		Name:      "Posto de Protecao Ambiental Santo Andre Airport",
		City:      "Barao De Melgaco",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 365,
		Latitude:  -16.7708320618,
		Longitude: -56.1725006104,
		Timezone:  "America/Cuiaba",
	},
	"SIMQ": {
		ICAO:      "SIMQ",
		Name:      "Dois de Maio Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 600,
		Latitude:  -18.2088890076,
		Longitude: -55.7530555725,
		Timezone:  "America/Campo_Grande",
	},
	"SIMR": {
		ICAO:      "SIMR",
		Name:      "Fazenda Reunidas Airport",
		City:      "Rio Verde",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2362,
		Latitude:  -17.7525005341,
		Longitude: -51.1519432068,
		Timezone:  "America/Sao_Paulo",
	},
	"SIMU": {
		ICAO:      "SIMU",
		Name:      "Fazenda Mutum Airport",
		City:      "Anaurilandia",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 978,
		Latitude:  -22.2824993134,
		Longitude: -52.8427772522,
		Timezone:  "America/Campo_Grande",
	},
	"SIMV": {
		ICAO:      "SIMV",
		Name:      "Fazenda Mata Velha Airport",
		City:      "Uberaba",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2733,
		Latitude:  -19.8383331299,
		Longitude: -47.8497238159,
		Timezone:  "America/Sao_Paulo",
	},
	"SIMW": {
		ICAO:      "SIMW",
		Name:      "Fazenda Retiro do Cervo I Airport",
		City:      "Jaraguari",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1562,
		Latitude:  -20.3544445038,
		Longitude: -53.9677772522,
		Timezone:  "America/Campo_Grande",
	},
	"SIMX": {
		ICAO:      "SIMX",
		Name:      "Fazenda Jangada Airport",
		City:      "Jangada",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 722,
		Latitude:  -15.2408332825,
		Longitude: -56.5780563354,
		Timezone:  "America/Cuiaba",
	},
	"SIMY": {
		ICAO:      "SIMY",
		Name:      "Fazenda Porto Oculto Airport",
		City:      "Itaquirai",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 972,
		Latitude:  -23.2124996185,
		Longitude: -54.0952796936,
		Timezone:  "America/Campo_Grande",
	},
	"SIMZ": {
		ICAO:      "SIMZ",
		Name:      "Fazenda Primeiro de Maio Airport",
		City:      "Sonora",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 584,
		Latitude:  -17.8755569458,
		Longitude: -54.9411125183,
		Timezone:  "America/Campo_Grande",
	},
	"SINA": {
		ICAO:      "SINA",
		Name:      "Sao Francisco Airport",
		City:      "Sao Pedro Do Parana",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1122,
		Latitude:  -22.7888889313,
		Longitude: -53.1977767944,
		Timezone:  "America/Sao_Paulo",
	},
	"SINC": {
		ICAO:      "SINC",
		Name:      "Nova Conceicao Agroindustrial Airport",
		City:      "Baiao",
		State:     "Para",
		Country:   "BR",
		Elevation: 49,
		Latitude:  -2.9230558872,
		Longitude: -49.7852783203,
		Timezone:  "America/Belem",
	},
	"SING": {
		ICAO:      "SING",
		Name:      "Fazenda Sao Joao Airport",
		City:      "Nova Granada",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1739,
		Latitude:  -20.5566673279,
		Longitude: -49.2883338928,
		Timezone:  "America/Sao_Paulo",
	},
	"SINH": {
		ICAO:      "SINH",
		Name:      "Fazenda Esperanca Airport",
		City:      "Novo Horizonte Do Sul",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 804,
		Latitude:  -22.6494445801,
		Longitude: -53.5644454956,
		Timezone:  "America/Campo_Grande",
	},
	"SINJ": {
		ICAO:      "SINJ",
		Name:      "Fazenda Novo Horizonte - TATTERSAL Airport",
		City:      "Coxim",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1205,
		Latitude:  -18.3680553436,
		Longitude: -54.7769432068,
		Timezone:  "America/Campo_Grande",
	},
	"SINK": {
		ICAO:      "SINK",
		Name:      "Fazenda Negrinha Airport",
		City:      "Parapua",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1582,
		Latitude:  -21.8758335114,
		Longitude: -50.8716659546,
		Timezone:  "America/Sao_Paulo",
	},
	"SINM": {
		ICAO:      "SINM",
		Name:      "Fazenda Itanhanga Airport",
		City:      "Nova Mutum",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 938,
		Latitude:  -13.101111412,
		Longitude: -56.5299987793,
		Timezone:  "America/Cuiaba",
	},
	"SINN": {
		ICAO:      "SINN",
		Name:      "Fazenda Nova Holanda Airport",
		City:      "Balsas",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 1657,
		Latitude:  -8.8069438934,
		Longitude: -46.3588905334,
		Timezone:  "America/Fortaleza",
	},
	"SINO": {
		ICAO:      "SINO",
		Name:      "Fazenda Mapisa Airport",
		City:      "Bom Jesus Das Selvas",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 634,
		Latitude:  -4.4130558968,
		Longitude: -46.5180549622,
		Timezone:  "America/Fortaleza",
	},
	"SINP": {
		ICAO:      "SINP",
		Name:      "Fazenda Santa Maria Airport",
		City:      "Jardim",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1083,
		Latitude:  -21.5080566406,
		Longitude: -56.4247207642,
		Timezone:  "America/Campo_Grande",
	},
	"SINQ": {
		ICAO:      "SINQ",
		Name:      "Laranjal Airport",
		City:      "Laranjal Do Jari",
		State:     "Amapa",
		Country:   "BR",
		Elevation: 250,
		Latitude:  -0.7947220206,
		Longitude: -52.488609314,
		Timezone:  "America/Belem",
	},
	"SINR": {
		ICAO:      "SINR",
		Name:      "Fazenda Recanto Airport",
		City:      "Alcinopolis",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1231,
		Latitude:  -18.0113887787,
		Longitude: -54.0161094666,
		Timezone:  "America/Campo_Grande",
	},
	"SINS": {
		ICAO:      "SINS",
		Name:      "Fazenda Cachoeira do Lontra Airport",
		City:      "Campo Grande",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 1211,
		Latitude:  -21.2883338928,
		Longitude: -53.7105560303,
		Timezone:  "America/Campo_Grande",
	},
	"SINT": {
		ICAO:      "SINT",
		Name:      "Fazenda Santa Adelia Airport",
		City:      "Santa Isabel",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2251,
		Latitude:  -15.1947221756,
		Longitude: -49.3794441223,
		Timezone:  "America/Sao_Paulo",
	},
	"SINU": {
		ICAO:      "SINU",
		Name:      "Fazenda Xerez Airport",
		City:      "Bela Vista",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 936,
		Latitude:  -21.9244441986,
		Longitude: -56.7086105347,
		Timezone:  "America/Campo_Grande",
	},
	"SINV": {
		ICAO:      "SINV",
		Name:      "Fazenda Santa Rita Airport",
		City:      "Custodia",
		State:     "Pernambuco",
		Country:   "BR",
		Elevation: 1634,
		Latitude:  -8.2283334732,
		Longitude: -37.7563896179,
		Timezone:  "America/Recife",
	},
	"SINZ": {
		ICAO:      "SINZ",
		Name:      "Xanadu Airport",
		City:      "Fernandes Pinheiro",
		State:     "Parana",
		Country:   "BR",
		Elevation: 2796,
		Latitude:  -25.4380569458,
		Longitude: -50.5277786255,
		Timezone:  "America/Sao_Paulo",
	},
	"SIOA": {
		ICAO:      "SIOA",
		Name:      "Fazenda Perdizes Airport",
		City:      "Lassance",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1969,
		Latitude:  -17.6725006104,
		Longitude: -44.5930557251,
		Timezone:  "America/Sao_Paulo",
	},
	"SIOB": {
		ICAO:      "SIOB",
		Name:      "Fazenda Guanabara Airport",
		City:      "Andradina",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1362,
		Latitude:  -20.8211116791,
		Longitude: -51.2705574036,
		Timezone:  "America/Sao_Paulo",
	},
	"SIOC": {
		ICAO:      "SIOC",
		Name:      "Santo Expedito Airport",
		City:      "Nova Independencia",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1300,
		Latitude:  -21.1422214508,
		Longitude: -51.5463905334,
		Timezone:  "America/Sao_Paulo",
	},
	"SIOD": {
		ICAO:      "SIOD",
		Name:      "Fazenda Santa Helena Airport",
		City:      "Nova Andradina",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 952,
		Latitude:  -21.9972229004,
		Longitude: -53.4247207642,
		Timezone:  "America/Campo_Grande",
	},
	"SIOE": {
		ICAO:      "SIOE",
		Name:      "Mocelin Airport",
		City:      "Dois Vizinhos",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1850,
		Latitude:  -25.793333,
		Longitude: -53.034722,
		Timezone:  "America/Sao_Paulo",
	},
	"SIOF": {
		ICAO:      "SIOF",
		Name:      "Fazenda Cachoeira Airport",
		City:      "Nova Alvorada Do Sul",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1247,
		Latitude:  -21.4311103821,
		Longitude: -54.3769454956,
		Timezone:  "America/Campo_Grande",
	},
	"SIOI": {
		ICAO:      "SIOI",
		Name:      "Rio Arraias Airport",
		City:      "Uniao Do Sul",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 3773,
		Latitude:  -11.76222229,
		Longitude: -54.3452796936,
		Timezone:  "America/Cuiaba",
	},
	"SIOJ": {
		ICAO:      "SIOJ",
		Name:      "Fazenda Soroteca Airport",
		City:      "Sao Jose Dos Quatro Marcos",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 749,
		Latitude:  -15.6313886642,
		Longitude: -58.3083343506,
		Timezone:  "America/Cuiaba",
	},
	"SIOK": {
		ICAO:      "SIOK",
		Name:      "Fazenda Agua Doce Airport",
		City:      "Nobres",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1313,
		Latitude:  -14.6422224045,
		Longitude: -56.2388877869,
		Timezone:  "America/Cuiaba",
	},
	"SIOL": {
		ICAO:      "SIOL",
		Name:      "Fazenda Pontal Airport",
		City:      "Corumbaiba",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1706,
		Latitude:  -18.3225002289,
		Longitude: -48.844165802,
		Timezone:  "America/Sao_Paulo",
	},
	"SIOM": {
		ICAO:      "SIOM",
		Name:      "Rio Ouro Airport",
		City:      "Uniao Do Sul",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 3741,
		Latitude:  -11.5388889313,
		Longitude: -54.1938896179,
		Timezone:  "America/Cuiaba",
	},
	"SIOO": {
		ICAO:      "SIOO",
		Name:      "Fazenda Piquiri Airport",
		City:      "Porto Murtinho",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 273,
		Latitude:  -22.0238895416,
		Longitude: -57.9027786255,
		Timezone:  "America/Campo_Grande",
	},
	"SIOP": {
		ICAO:      "SIOP",
		Name:      "Fazenda Londrina II Airport",
		City:      "Porto Murtinho",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 272,
		Latitude:  -21.9722213745,
		Longitude: -57.850276947,
		Timezone:  "America/Campo_Grande",
	},
	"SIOQ": {
		ICAO:      "SIOQ",
		Name:      "Fazenda Mato Alto Airport",
		City:      "Iguatemi",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1313,
		Latitude:  -23.5038890839,
		Longitude: -54.3141670227,
		Timezone:  "America/Campo_Grande",
	},
	"SIOR": {
		ICAO:      "SIOR",
		Name:      "Fazenda Sao Miguel Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 358,
		Latitude:  -18.0980567932,
		Longitude: -57.1533317566,
		Timezone:  "America/Campo_Grande",
	},
	"SIOS": {
		ICAO:      "SIOS",
		Name:      "Fazenda Santa Josefa Airport",
		City:      "Jatei",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1313,
		Latitude:  -22.769443512,
		Longitude: -53.963054657,
		Timezone:  "America/Campo_Grande",
	},
	"SIOT": {
		ICAO:      "SIOT",
		Name:      "Fazenda Santa Verginia Airport",
		City:      "Santa Rita Do Pardo",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1092,
		Latitude:  -21.4894447327,
		Longitude: -52.3838882446,
		Timezone:  "America/Campo_Grande",
	},
	"SIOU": {
		ICAO:      "SIOU",
		Name:      "Fazenda Santa Helena Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 378,
		Latitude:  -18.2980556488,
		Longitude: -57.0147209167,
		Timezone:  "America/Campo_Grande",
	},
	"SIOV": {
		ICAO:      "SIOV",
		Name:      "Taua Airport",
		City:      "Prado",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 44,
		Latitude:  -16.9013881683,
		Longitude: -39.1588897705,
		Timezone:  "America/Bahia",
	},
	"SIOZ": {
		ICAO:      "SIOZ",
		Name:      "Fazenda Santa Fe Airport",
		City:      "Santana Do Araguaia",
		Country:   "BR",
		Elevation: 591,
		Latitude:  -9.5880556107,
		Longitude: -50.3572235107,
		Timezone:  "America/Belem",
	},
	"SIPA": {
		ICAO:      "SIPA",
		Name:      "Aeroclube de Sao Jose do Rio Pardo Airport",
		City:      "Sao Jose Do Rio Pardo",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2849,
		Latitude:  -21.6447219849,
		Longitude: -46.9333343506,
		Timezone:  "America/Sao_Paulo",
	},
	"SIPB": {
		ICAO:      "SIPB",
		Name:      "CAVU - Clube de Aviacao Ultraleve Airport",
		City:      "Raposa",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 109,
		Latitude:  -2.4416670799,
		Longitude: -44.1247215271,
		Timezone:  "America/Fortaleza",
	},
	"SIPC": {
		ICAO:      "SIPC",
		Name:      "Portal do Ceu Airport",
		City:      "Belem",
		State:     "Para",
		Country:   "BR",
		Elevation: 16,
		Latitude:  -1.3147219419,
		Longitude: -48.4294433594,
		Timezone:  "America/Belem",
	},
	"SIPE": {
		ICAO:      "SIPE",
		Name:      "Itapessoca Airport",
		City:      "Goiana",
		State:     "Pernambuco",
		Country:   "BR",
		Elevation: 16,
		Latitude:  -7.6560101509,
		Longitude: -34.8564987183,
		Timezone:  "America/Recife",
	},
	"SIPG": {
		ICAO:      "SIPG",
		Name:      "Fazenda Agua Preta Airport",
		City:      "Cocalinho",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 794,
		Latitude:  -14.1380996704,
		Longitude: -51.4664001465,
		Timezone:  "America/Cuiaba",
	},
	"SIPH": {
		ICAO:      "SIPH",
		Name:      "Fazenda Palmeiras Airport",
		City:      "Caseara",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 722,
		Latitude:  -9.6019439697,
		Longitude: -49.7738876343,
		Timezone:  "America/Araguaina",
	},
	"SIPJ": {
		ICAO:      "SIPJ",
		Name:      "Fazenda Marreco Airport",
		City:      "Sao Goncalo Do Abaete",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1969,
		Latitude:  -18.1086101532,
		Longitude: -45.2155570984,
		Timezone:  "America/Sao_Paulo",
	},
	"SIPK": {
		ICAO:      "SIPK",
		Name:      "SESC Pantanal Airport",
		City:      "Pocone",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 348,
		Latitude:  -16.4974002838,
		Longitude: -56.4221992493,
		Timezone:  "America/Cuiaba",
	},
	"SIPL": {
		ICAO:      "SIPL",
		Name:      "Fazenda Entre Serras Airport",
		City:      "Planalto Da Serra",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1824,
		Latitude:  -14.6222219467,
		Longitude: -54.4766654968,
		Timezone:  "America/Cuiaba",
	},
	"SIPN": {
		ICAO:      "SIPN",
		Name:      "Fazenda Progresso Airport",
		City:      "Porto Murtinho",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 328,
		Latitude:  -21.5100002289,
		Longitude: -57.6905555725,
		Timezone:  "America/Campo_Grande",
	},
	"SIPO": {
		ICAO:      "SIPO",
		Name:      "Sebastiao Jose Pereira Airport",
		City:      "Condeuba",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 2260,
		Latitude:  -14.9241666794,
		Longitude: -41.9566688538,
		Timezone:  "America/Bahia",
	},
	"SIPQ": {
		ICAO:      "SIPQ",
		Name:      "Fazenda Rancho Estrela Airport",
		City:      "Sao Miguel Do Passa Quatro",
		State:     "Goias",
		Country:   "BR",
		Elevation: 3002,
		Latitude:  -16.9983329773,
		Longitude: -48.7380561829,
		Timezone:  "America/Sao_Paulo",
	},
	"SIPR": {
		ICAO:      "SIPR",
		Name:      "Fazenda Sao Jose Airport",
		City:      "Parauna",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2451,
		Latitude:  -17.0011100769,
		Longitude: -50.7427787781,
		Timezone:  "America/Sao_Paulo",
	},
	"SIPU": {
		ICAO:      "SIPU",
		Name:      "Fazenda Lagoa da Capa Airport",
		City:      "Perdizes",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3567,
		Latitude:  -19.4483337402,
		Longitude: -47.36277771,
		Timezone:  "America/Sao_Paulo",
	},
	"SIPY": {
		ICAO:      "SIPY",
		Name:      "Ambrosio Airport",
		City:      "Portel",
		State:     "Para",
		Country:   "BR",
		Elevation: 96,
		Latitude:  -1.9813890457,
		Longitude: -50.8113899231,
		Timezone:  "America/Belem",
	},
	"SIQA": {
		ICAO:      "SIQA",
		Name:      "Senador Antonio Farias Airport",
		City:      "Baia Formosa",
		State:     "Rio-Grande-do-Norte",
		Country:   "BR",
		Elevation: 171,
		Latitude:  -6.4074997902,
		Longitude: -35.0447235107,
		Timezone:  "America/Fortaleza",
	},
	"SIQB": {
		ICAO:      "SIQB",
		Name:      "Monte Verde Airport",
		City:      "Portel",
		State:     "Para",
		Country:   "BR",
		Elevation: 249,
		Latitude:  -3.2741670609,
		Longitude: -50.3213882446,
		Timezone:  "America/Belem",
	},
	"SIQC": {
		ICAO:      "SIQC",
		Name:      "Fazenda Paraiso Airport",
		City:      "Costa Rica",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1592,
		Latitude:  -19.1572227478,
		Longitude: -52.9958343506,
		Timezone:  "America/Campo_Grande",
	},
	"SIQD": {
		ICAO:      "SIQD",
		Name:      "Fazenda Estancia Regina Airport",
		City:      "Terenos",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1968,
		Latitude:  -20.640832901,
		Longitude: -54.7905578613,
		Timezone:  "America/Campo_Grande",
	},
	"SIQE": {
		ICAO:      "SIQE",
		Name:      "Botelho Airport",
		City:      "Brasilia",
		State:     "Goias",
		Country:   "BR",
		Elevation: 3399,
		Latitude:  -15.9416666031,
		Longitude: -47.7202796936,
		Timezone:  "America/Sao_Paulo",
	},
	"SIQG": {
		ICAO:      "SIQG",
		Name:      "Fazenda Morro Alto Airport",
		City:      "Nova Canaa Do Norte",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 958,
		Latitude:  -10.506111145,
		Longitude: -55.7105560303,
		Timezone:  "America/Cuiaba",
	},
	"SIQI": {
		ICAO:      "SIQI",
		Name:      "Aerolis Airport",
		City:      "Palmares Do Sul",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 20,
		Latitude:  -30.5422210693,
		Longitude: -50.4708328247,
		Timezone:  "America/Sao_Paulo",
	},
	"SIQJ": {
		ICAO:      "SIQJ",
		Name:      "Morro do Chapeu II Airport",
		City:      "Chapada Dos Guimaraes",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 989,
		Latitude:  -14.9582996368,
		Longitude: -55.8007011414,
		Timezone:  "America/Cuiaba",
	},
	"SIQK": {
		ICAO:      "SIQK",
		Name:      "Rodrigues Airport",
		City:      "Nova Odessa",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1909,
		Latitude:  -22.7813892365,
		Longitude: -47.3341674805,
		Timezone:  "America/Sao_Paulo",
	},
	"SIQM": {
		ICAO:      "SIQM",
		Name:      "Fazenda Bom Jardim Airport",
		City:      "Porteirao",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1693,
		Latitude:  -17.9002780914,
		Longitude: -50.1236114502,
		Timezone:  "America/Sao_Paulo",
	},
	"SIQN": {
		ICAO:      "SIQN",
		Name:      "Fazenda Flor da Mata Airport",
		City:      "Sao Jose Do Xingu",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1264,
		Latitude:  -10.7166671753,
		Longitude: -52.8019447327,
		Timezone:  "America/Cuiaba",
	},
	"SIQO": {
		ICAO:      "SIQO",
		Name:      "Fazenda Mercedes Estancia Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 499,
		Latitude:  -18.3341674805,
		Longitude: -55.4850006104,
		Timezone:  "America/Campo_Grande",
	},
	"SIQQ": {
		ICAO:      "SIQQ",
		Name:      "Aero Resende Airport",
		City:      "Bom Jesus",
		State:     "Goias",
		Country:   "BR",
		Elevation: 6825,
		Latitude:  -18.1336116791,
		Longitude: -49.7961120605,
		Timezone:  "America/Sao_Paulo",
	},
	"SIQR": {
		ICAO:      "SIQR",
		Name:      "Fazenda Triangulo Airport",
		City:      "Bonopolis",
		State:     "Goias",
		Country:   "BR",
		Elevation: 804,
		Latitude:  -13.5819444656,
		Longitude: -49.8205566406,
		Timezone:  "America/Sao_Paulo",
	},
	"SIQS": {
		ICAO:      "SIQS",
		Name:      "Fazenda Terra do Sol Airport",
		City:      "Porto Estrela",
		Country:   "BR",
		Elevation: 722,
		Latitude:  -15.3763885498,
		Longitude: -57.1297225952,
		Timezone:  "America/Cuiaba",
	},
	"SIQU": {
		ICAO:      "SIQU",
		Name:      "Alcoolvare Airport",
		City:      "Aparecida Do Taboado",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1280,
		Latitude:  -20.0863895416,
		Longitude: -51.3088874817,
		Timezone:  "America/Campo_Grande",
	},
	"SIQV": {
		ICAO:      "SIQV",
		Name:      "Fazenda Sol Aberto Airport",
		City:      "Sapezal",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1687,
		Latitude:  -12.8950004578,
		Longitude: -58.5800018311,
		Timezone:  "America/Cuiaba",
	},
	"SIQW": {
		ICAO:      "SIQW",
		Name:      "Fazenda Santa Maria do Jauru Airport",
		City:      "Porto Esperidiao",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 755,
		Latitude:  -15.8163890839,
		Longitude: -58.5530548096,
		Timezone:  "America/Cuiaba",
	},
	"SIQX": {
		ICAO:      "SIQX",
		Name:      "Fazenda Barranquinho Airport",
		City:      "Porto Estrela",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 558,
		Latitude:  -15.6366672516,
		Longitude: -57.4680557251,
		Timezone:  "America/Cuiaba",
	},
	"SIQY": {
		ICAO:      "SIQY",
		Name:      "Fazenda Amizade Airport",
		City:      "Sorriso",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1264,
		Latitude:  -12.6280555725,
		Longitude: -55.9449996948,
		Timezone:  "America/Cuiaba",
	},
	"SIQZ": {
		ICAO:      "SIQZ",
		Name:      "Fazenda Carolina Airport",
		City:      "Buritis",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1871,
		Latitude:  -15.6491670609,
		Longitude: -46.4430541992,
		Timezone:  "America/Sao_Paulo",
	},
	"SIRA": {
		ICAO:      "SIRA",
		Name:      "Fazenda Magiana Airport",
		City:      "Sorriso",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1575,
		Latitude:  -12.3886108398,
		Longitude: -55.2427787781,
		Timezone:  "America/Cuiaba",
	},
	"SIRB": {
		ICAO:      "SIRB",
		Name:      "Fazenda Mimoso Airport",
		City:      "Rio Brilhante",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1476,
		Latitude:  -21.7711105347,
		Longitude: -54.3816680908,
		Timezone:  "America/Campo_Grande",
	},
	"SIRC": {
		ICAO:      "SIRC",
		Name:      "Fazenda Jarina Airport",
		City:      "Peixoto De Azevedo",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1083,
		Latitude:  -10.3252782822,
		Longitude: -53.5852775574,
		Timezone:  "America/Cuiaba",
	},
	"SIRD": {
		ICAO:      "SIRD",
		Name:      "Rancho Jatoba Airport",
		City:      "Novo Santo Antonio",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 821,
		Latitude:  -12.2344436646,
		Longitude: -50.9183311462,
		Timezone:  "America/Cuiaba",
	},
	"SIRE": {
		ICAO:      "SIRE",
		Name:      "Fazenda Rancho Estrela IV Airport",
		City:      "Sao Domingos",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2549,
		Latitude:  -13.6519441605,
		Longitude: -46.5413894653,
		Timezone:  "America/Sao_Paulo",
	},
	"SIRF": {
		ICAO:      "SIRF",
		Name:      "Jose Mucio Monteiro Airport",
		City:      "Rio Formoso",
		State:     "Pernambuco",
		Country:   "BR",
		Elevation: 479,
		Latitude:  -8.631111145,
		Longitude: -35.2716674805,
		Timezone:  "America/Recife",
	},
	"SIRG": {
		ICAO:      "SIRG",
		Name:      "Fazenda Esperanca Airport",
		City:      "Alto Da Boa Vista",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1214,
		Latitude:  -12.0336112976,
		Longitude: -51.6177787781,
		Timezone:  "America/Cuiaba",
	},
	"SIRH": {
		ICAO:      "SIRH",
		Name:      "Fazenda Morada da Lua Airport",
		City:      "Coxim",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1450,
		Latitude:  -18.2672214508,
		Longitude: -54.3136100769,
		Timezone:  "America/Campo_Grande",
	},
	"SIRI": {
		ICAO:      "SIRI",
		Name:      "Barra Grande Airport",
		City:      "Marau",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 23,
		Latitude:  -13.9036111832,
		Longitude: -38.9358329773,
		Timezone:  "America/Bahia",
	},
	"SIRJ": {
		ICAO:      "SIRJ",
		Name:      "Caima Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 115,
		Latitude:  -4.3035898209,
		Longitude: -56.2181015015,
		Timezone:  "America/Santarem",
	},
	"SIRN": {
		ICAO:      "SIRN",
		Name:      "Fazenda Jao Airport",
		City:      "Nova Xavantina",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 854,
		Latitude:  -14.75833,
		Longitude: -51.98972,
		Timezone:  "America/Cuiaba",
	},
	"SIRQ": {
		ICAO:      "SIRQ",
		Name:      "Fazenda Maranata Airport",
		City:      "Chupinguaia",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 1000,
		Latitude:  -12.773888588,
		Longitude: -60.8694458008,
		Timezone:  "America/Porto_Velho",
	},
	"SIRR": {
		ICAO:      "SIRR",
		Name:      "Lagoa dos Coqueiros Airport",
		City:      "Paranaita",
		Country:   "BR",
		Elevation: 1280,
		Latitude:  -9.2355556488,
		Longitude: -56.98777771,
		Timezone:  "America/Cuiaba",
	},
	"SIRS": {
		ICAO:      "SIRS",
		Name:      "Fazenda Sao Sebastiao Airport",
		City:      "Campo Florido",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2461,
		Latitude:  -19.6424999237,
		Longitude: -48.7894439697,
		Timezone:  "America/Sao_Paulo",
	},
	"SIRT": {
		ICAO:      "SIRT",
		Name:      "Marca de Casco Airport",
		City:      "Sao Jose Do Rio Claro",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1158,
		Latitude:  -13.7274999619,
		Longitude: -57.0216674805,
		Timezone:  "America/Cuiaba",
	},
	"SIRU": {
		ICAO:      "SIRU",
		Name:      "Fazenda Bahia Airport",
		City:      "Pedra Preta",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 6234,
		Latitude:  -16.9213886261,
		Longitude: -54.0741653442,
		Timezone:  "America/Cuiaba",
	},
	"SIRV": {
		ICAO:      "SIRV",
		Name:      "Fazenda Morro Branco Airport",
		City:      "Porto Esperidiao",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 400,
		Latitude:  -16.1180553436,
		Longitude: -58.7658348083,
		Timezone:  "America/Cuiaba",
	},
	"SIRW": {
		ICAO:      "SIRW",
		Name:      "Fazenda Serra Vermelha Airport",
		City:      "Sambaiba",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 1687,
		Latitude:  -6.8727779388,
		Longitude: -45.3027763367,
		Timezone:  "America/Fortaleza",
	},
	"SISB": {
		ICAO:      "SISB",
		Name:      "Fazenda Eldorado - Agropecuaria Paleana S/A Airport",
		City:      "Bom Jesus Do Araguaia",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1108,
		Latitude:  -12.1541671753,
		Longitude: -51.6869430542,
		Timezone:  "America/Cuiaba",
	},
	"SISD": {
		ICAO:      "SISD",
		Name:      "Fazenda Sao Sebastiao Airport",
		City:      "Aparecida Do Rio Doce",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1982,
		Latitude:  -18.1480560303,
		Longitude: -51.2716674805,
		Timezone:  "America/Sao_Paulo",
	},
	"SISE": {
		ICAO:      "SISE",
		Name:      "Fazenda Kanaxue Airport",
		City:      "Sao Felix Do Araguaia",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 650,
		Latitude:  -12.015832901,
		Longitude: -50.8666687012,
		Timezone:  "America/Cuiaba",
	},
	"SISF": {
		ICAO:      "SISF",
		Name:      "Fazenda Santa Feliciana Airport",
		City:      "Bonito",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 2198,
		Latitude:  -20.941110611,
		Longitude: -56.6733322144,
		Timezone:  "America/Campo_Grande",
	},
	"SISI": {
		ICAO:      "SISI",
		Name:      "Fazenda Sete Ilhas Airport",
		City:      "Acreuna",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1720,
		Latitude:  -17.6341667175,
		Longitude: -50.4705543518,
		Timezone:  "America/Sao_Paulo",
	},
	"SISJ": {
		ICAO:      "SISJ",
		Name:      "Fazenda Sao Jose do Piquiri Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 427,
		Latitude:  -17.4541664124,
		Longitude: -55.5591659546,
		Timezone:  "America/Campo_Grande",
	},
	"SISL": {
		ICAO:      "SISL",
		Name:      "Fazenda Santa Lucia Airport",
		City:      "Barao De Melgaco",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 509,
		Latitude:  -16.8941669464,
		Longitude: -55.9072227478,
		Timezone:  "America/Cuiaba",
	},
	"SISN": {
		ICAO:      "SISN",
		Name:      "Fazenda Nossa Senhora Aparecida Airport",
		City:      "Sonora",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 2001,
		Latitude:  -17.5450000763,
		Longitude: -53.8938903809,
		Timezone:  "America/Campo_Grande",
	},
	"SISO": {
		ICAO:      "SISO",
		Name:      "Fazenda Santo Ambrosio Airport",
		City:      "Chaves",
		State:     "Para",
		Country:   "BR",
		Elevation: 20,
		Latitude:  -0.0916669965,
		Longitude: -49.5805549622,
		Timezone:  "America/Belem",
	},
	"SISP": {
		ICAO:      "SISP",
		IATA:      "0",
		Name:      "Santa Paula Airport",
		City:      "Ipigua",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1860,
		Latitude:  -20.7038993835,
		Longitude: -49.3913993835,
		Timezone:  "America/Sao_Paulo",
	},
	"SISS": {
		ICAO:      "SISS",
		Name:      "Fazenda Lagoa do Campo Airport",
		City:      "Ribas Do Rio Pardo",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1542,
		Latitude:  -20.384721756,
		Longitude: -53.4011116028,
		Timezone:  "America/Campo_Grande",
	},
	"SISU": {
		ICAO:      "SISU",
		Name:      "Fazenda Pontal do Lagoa Airport",
		City:      "Santa Rita Do Pardo",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1264,
		Latitude:  -20.8700008392,
		Longitude: -53.1549987793,
		Timezone:  "America/Campo_Grande",
	},
	"SISW": {
		ICAO:      "SISW",
		Name:      "Fazenda Mato Grosso Airport",
		City:      "Campos de Julio",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1690,
		Latitude:  -13.82278,
		Longitude: -59.16278,
		Timezone:  "America/Cuiaba",
	},
	"SISX": {
		ICAO:      "SISX",
		Name:      "Santa Terezinha Airport",
		City:      "Aquidauana",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 361,
		Latitude:  -19.3772220612,
		Longitude: -56.0561103821,
		Timezone:  "America/Campo_Grande",
	},
	"SISY": {
		ICAO:      "SISY",
		Name:      "Piraquara Airport",
		City:      "Piraquara",
		State:     "Parana",
		Country:   "BR",
		Elevation: 3212,
		Latitude:  -25.4513893127,
		Longitude: -49.1438903809,
		Timezone:  "America/Sao_Paulo",
	},
	"SISZ": {
		ICAO:      "SISZ",
		Name:      "Santa Felicidade Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 821,
		Latitude:  -5.5758328438,
		Longitude: -56.6397209167,
		Timezone:  "America/Santarem",
	},
	"SITA": {
		ICAO:      "SITA",
		Name:      "Nossa Senhora da Conceicao Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 958,
		Latitude:  -7.2202777863,
		Longitude: -56.1666679382,
		Timezone:  "America/Santarem",
	},
	"SITB": {
		ICAO:      "SITB",
		Name:      "Fazenda Touro Branco Airport",
		City:      "Navirai",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 951,
		Latitude:  -23.0141677856,
		Longitude: -53.7874984741,
		Timezone:  "America/Campo_Grande",
	},
	"SITC": {
		ICAO:      "SITC",
		Name:      "Carmine Castaldo Airport",
		City:      "Tabapora",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1889,
		Latitude:  -11.3022222519,
		Longitude: -56.8366661072,
		Timezone:  "America/Cuiaba",
	},
	"SITD": {
		ICAO:      "SITD",
		Name:      "Cara Preta Airport",
		City:      "Itaituba",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 155,
		Latitude:  -5.1061110497,
		Longitude: -57.4824981689,
		Timezone:  "America/Santarem",
	},
	"SITF": {
		ICAO:      "SITF",
		Name:      "Fazenda Deposito Airport",
		City:      "Pacaraima",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 365,
		Latitude:  4.1083297729,
		Longitude: -60.7332992554,
		Timezone:  "America/Boa_Vista",
	},
	"SITH": {
		ICAO:      "SITH",
		Name:      "Fazenda Itaipu Airport",
		City:      "Turvelandia",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1644,
		Latitude:  -17.6736106873,
		Longitude: -50.3880577087,
		Timezone:  "America/Sao_Paulo",
	},
	"SITJ": {
		ICAO:      "SITJ",
		Name:      "Fazenda Sao Gabriel Airport",
		City:      "Balsas",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 998,
		Latitude:  -7.4905557632,
		Longitude: -46.144443512,
		Timezone:  "America/Fortaleza",
	},
	"SITL": {
		ICAO:      "SITL",
		Name:      "Fazenda Flores Airport",
		City:      "Tamboril",
		State:     "Ceara",
		Country:   "BR",
		Elevation: 1148,
		Latitude:  -5.049167,
		Longitude: -40.234167,
		Timezone:  "America/Fortaleza",
	},
	"SITN": {
		ICAO:      "SITN",
		Name:      "Fazenda Santo Antonio Diamandore Airport",
		City:      "Corumba",
		State:     "Santa-Cruz",
		Country:   "BR",
		Elevation: 493,
		Latitude:  -18.2430553436,
		Longitude: -57.5561103821,
		Timezone:  "America/Campo_Grande",
	},
	"SITO": {
		ICAO:      "SITO",
		Name:      "Muricoca Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 361,
		Latitude:  -7.1541671753,
		Longitude: -56.6361122131,
		Timezone:  "America/Santarem",
	},
	"SITR": {
		ICAO:      "SITR",
		Name:      "Fazenda Corrego Limpo Airport",
		City:      "Terenos",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1509,
		Latitude:  -20.173333,
		Longitude: -55.069444,
		Timezone:  "America/Campo_Grande",
	},
	"SITT": {
		ICAO:      "SITT",
		Name:      "Fazenda Guaira Airport",
		City:      "Balsas",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 1900,
		Latitude:  -7.0908331871,
		Longitude: -45.9541664124,
		Timezone:  "America/Fortaleza",
	},
	"SITU": {
		ICAO:      "SITU",
		Name:      "Fazenda Santa Genoveva Airport",
		City:      "Turvelandia",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1526,
		Latitude:  -17.9400005341,
		Longitude: -50.3422203064,
		Timezone:  "America/Sao_Paulo",
	},
	"SITV": {
		ICAO:      "SITV",
		Name:      "Fazenda Chapadao Airport",
		City:      "Santa Rita Do Trivelato",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1450,
		Latitude:  -13.4108333588,
		Longitude: -55.6738891602,
		Timezone:  "America/Cuiaba",
	},
	"SITW": {
		ICAO:      "SITW",
		Name:      "Fazenda Santa Luzia Airport",
		City:      "Sao Raimundo Das Mangabeiras",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 1720,
		Latitude:  -6.8233327866,
		Longitude: -45.411945343,
		Timezone:  "America/Fortaleza",
	},
	"SITX": {
		ICAO:      "SITX",
		Name:      "Fazenda Chapeu de Pano Airport",
		City:      "Aquidauana",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 519,
		Latitude:  -19.7744445801,
		Longitude: -55.5216674805,
		Timezone:  "America/Campo_Grande",
	},
	"SITY": {
		ICAO:      "SITY",
		Name:      "Fazenda Sao Geraldo Airport",
		City:      "Aquidauana",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 486,
		Latitude:  -19.4038887024,
		Longitude: -55.86277771,
		Timezone:  "America/Campo_Grande",
	},
	"SITZ": {
		ICAO:      "SITZ",
		Name:      "Fazenda MRV Airport",
		City:      "Monte Azul",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1713,
		Latitude:  -14.9177780151,
		Longitude: -43.3597221375,
		Timezone:  "America/Sao_Paulo",
	},
	"SIUA": {
		ICAO:      "SIUA",
		Name:      "Fazenda Barra do Lagoa Airport",
		City:      "Santa Rita Do Pardo",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1083,
		Latitude:  -20.8727779388,
		Longitude: -53.2108345032,
		Timezone:  "America/Campo_Grande",
	},
	"SIUB": {
		ICAO:      "SIUB",
		Name:      "Aerojaco Airport",
		City:      "Sorriso",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1510,
		Latitude:  -13.5050001144,
		Longitude: -55.1508331299,
		Timezone:  "America/Cuiaba",
	},
	"SIUC": {
		ICAO:      "SIUC",
		Name:      "Selva de Pedra Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 391,
		Latitude:  -7.2263889313,
		Longitude: -56.6708335876,
		Timezone:  "America/Santarem",
	},
	"SIUF": {
		ICAO:      "SIUF",
		Name:      "Fazenda Sao Francisco Airport",
		City:      "Peixoto De Azevedo",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1231,
		Latitude:  -10.1569442749,
		Longitude: -53.7016677856,
		Timezone:  "America/Cuiaba",
	},
	"SIUG": {
		ICAO:      "SIUG",
		Name:      "Fazenda Santa Irene Airport",
		City:      "Sandovalina",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1116,
		Latitude:  -22.4950008392,
		Longitude: -51.9094429016,
		Timezone:  "America/Sao_Paulo",
	},
	"SIUH": {
		ICAO:      "SIUH",
		Name:      "Fazenda Santa Ana Airport",
		City:      "Itauba",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1038,
		Latitude:  -11.1397218704,
		Longitude: -55.8797225952,
		Timezone:  "America/Cuiaba",
	},
	"SIUJ": {
		ICAO:      "SIUJ",
		Name:      "Pista Santa Tereza Airport",
		City:      "Itaituba",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 401,
		Latitude:  -5.1858329773,
		Longitude: -57.5844421387,
		Timezone:  "America/Santarem",
	},
	"SIUK": {
		ICAO:      "SIUK",
		Name:      "Serra Verde Airport",
		City:      "Jacareacanga",
		State:     "Para",
		Country:   "BR",
		Elevation: 840,
		Latitude:  -6.9250001907,
		Longitude: -56.9913902283,
		Timezone:  "America/Santarem",
	},
	"SIUL": {
		ICAO:      "SIUL",
		Name:      "Agrosserra Airport",
		City:      "Sao Raimundo Das Mangabeiras",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 1926,
		Latitude:  -6.7755560875,
		Longitude: -45.8916664124,
		Timezone:  "America/Fortaleza",
	},
	"SIUM": {
		ICAO:      "SIUM",
		Name:      "Fazenda Planeste Airport",
		City:      "Balsas",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 1874,
		Latitude:  -8.5561113358,
		Longitude: -46.8499984741,
		Timezone:  "America/Araguaina",
	},
	"SIUO": {
		ICAO:      "SIUO",
		Name:      "Fazenda Santa Adelia Airport",
		City:      "Sao Jose Do Xingu",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1034,
		Latitude:  -10.7219438553,
		Longitude: -52.2858314514,
		Timezone:  "America/Cuiaba",
	},
	"SIUQ": {
		ICAO:      "SIUQ",
		Name:      "Fazenda Primavera Airport",
		City:      "Campo Grande",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1660,
		Latitude:  -20.9775009155,
		Longitude: -54.3341674805,
		Timezone:  "America/Campo_Grande",
	},
	"SIUR": {
		ICAO:      "SIUR",
		Name:      "Fazenda Ribeirao Airport",
		City:      "Camapua",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1549,
		Latitude:  -19.6075000763,
		Longitude: -53.4997215271,
		Timezone:  "America/Campo_Grande",
	},
	"SIUS": {
		ICAO:      "SIUS",
		Name:      "Fazenda Dona Rosa S. Rezek Airport",
		City:      "Jatei",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1240,
		Latitude:  -22.7147216797,
		Longitude: -54.0447235107,
		Timezone:  "America/Campo_Grande",
	},
	"SIUT": {
		ICAO:      "SIUT",
		Name:      "Sao Paulo do Piaui Airport",
		City:      "Prado",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 82,
		Latitude:  -17.012778,
		Longitude: -39.179722,
		Timezone:  "America/Bahia",
	},
	"SIUV": {
		ICAO:      "SIUV",
		Name:      "Fazenda Sao Matheus Airport",
		City:      "Selviria",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1365,
		Latitude:  -20.3177776337,
		Longitude: -51.7836112976,
		Timezone:  "America/Campo_Grande",
	},
	"SIUW": {
		ICAO:      "SIUW",
		Name:      "Candote Airport",
		City:      "Tres Lagoas",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1093,
		Latitude:  -20.8391990662,
		Longitude: -52.3016014099,
		Timezone:  "America/Campo_Grande",
	},
	"SIUX": {
		ICAO:      "SIUX",
		Name:      "Pista Comunidade Sao Domingos Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 985,
		Latitude:  -6.4222221375,
		Longitude: -56.0505561829,
		Timezone:  "America/Santarem",
	},
	"SIUY": {
		ICAO:      "SIUY",
		Name:      "Santos Dumont Airport",
		City:      "Novo Progresso",
		State:     "Para",
		Country:   "BR",
		Elevation: 870,
		Latitude:  -7.0127778053,
		Longitude: -55.875,
		Timezone:  "America/Santarem",
	},
	"SIUZ": {
		ICAO:      "SIUZ",
		Name:      "Boa Esperanca Airport",
		City:      "Novo Progresso",
		State:     "Para",
		Country:   "BR",
		Elevation: 788,
		Latitude:  -7.6613888741,
		Longitude: -55.9602775574,
		Timezone:  "America/Santarem",
	},
	"SIVC": {
		ICAO:      "SIVC",
		Name:      "Fazenda Iporanga Airport",
		City:      "Mutunopolis",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1231,
		Latitude:  -13.7588891983,
		Longitude: -49.4772224426,
		Timezone:  "America/Sao_Paulo",
	},
	"SIVD": {
		ICAO:      "SIVD",
		Name:      "Fazenda dos Tres Rios Airport",
		City:      "Unai",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1723,
		Latitude:  -16.9449996948,
		Longitude: -46.2737998962,
		Timezone:  "America/Sao_Paulo",
	},
	"SIVE": {
		ICAO:      "SIVE",
		Name:      "Fazenda Royal Airport",
		City:      "Itapetininga",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2447,
		Latitude:  -23.506944,
		Longitude: -48,
		Timezone:  "America/Sao_Paulo",
	},
	"SIVG": {
		ICAO:      "SIVG",
		Name:      "Clube Aerodesportivo Selva Airport",
		City:      "Sinop",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1280,
		Latitude:  -11.8755559921,
		Longitude: -55.444442749,
		Timezone:  "America/Cuiaba",
	},
	"SIVJ": {
		ICAO:      "SIVJ",
		Name:      "Cirrus Sociedade Aerodesportiva Airport",
		City:      "Jaboticatubas",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2402,
		Latitude:  -19.4527778625,
		Longitude: -43.899723053,
		Timezone:  "America/Sao_Paulo",
	},
	"SIVK": {
		ICAO:      "SIVK",
		Name:      "Crispiano Airport",
		City:      "Monte Alegre De Goias",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1657,
		Latitude:  -13.2227783203,
		Longitude: -46.9530563354,
		Timezone:  "America/Sao_Paulo",
	},
	"SIVL": {
		ICAO:      "SIVL",
		Name:      "Sitio Flor de Lys Airport",
		City:      "Junco Do Serido",
		State:     "Paraiba",
		Country:   "BR",
		Elevation: 1641,
		Latitude:  -6.9452781677,
		Longitude: -36.6725006104,
		Timezone:  "America/Fortaleza",
	},
	"SIVN": {
		ICAO:      "SIVN",
		Name:      "Fazenda Sansara Airport",
		City:      "Montes Claros",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2149,
		Latitude:  -16.599993,
		Longitude: -43.768225,
		Timezone:  "America/Sao_Paulo",
	},
	"SIVO": {
		ICAO:      "SIVO",
		Name:      "Fazenda Malu Airport",
		City:      "Bom Jesus Do Araguaia",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1378,
		Latitude:  -12.448056221,
		Longitude: -51.7741661072,
		Timezone:  "America/Cuiaba",
	},
	"SIVP": {
		ICAO:      "SIVP",
		Name:      "Fazenda Bandeirante Airport",
		City:      "Nova Crixas",
		State:     "Goias",
		Country:   "BR",
		Elevation: 899,
		Latitude:  -14.4463891983,
		Longitude: -50.5361099243,
		Timezone:  "America/Sao_Paulo",
	},
	"SIVQ": {
		ICAO:      "SIVQ",
		Name:      "Fazenda Annalu Airport",
		City:      "Deodapolis",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1083,
		Latitude:  -22.0822219849,
		Longitude: -54.2011108398,
		Timezone:  "America/Campo_Grande",
	},
	"SIVR": {
		ICAO:      "SIVR",
		Name:      "Fazenda Sao Raimundo Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 909,
		Latitude:  -7.625,
		Longitude: -56.7400016785,
		Timezone:  "America/Santarem",
	},
	"SIVU": {
		ICAO:      "SIVU",
		Name:      "Joao Moteiro Airport",
		City:      "Vila Velha",
		State:     "Espirito-Santo",
		Country:   "BR",
		Elevation: 16,
		Latitude:  -20.4227771759,
		Longitude: -40.3333320618,
		Timezone:  "America/Sao_Paulo",
	},
	"SIVV": {
		ICAO:      "SIVV",
		Name:      "Fazenda Bororeo Airport",
		City:      "Barao De Melgaco",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 489,
		Latitude:  -16.9274997711,
		Longitude: -56.2330551147,
		Timezone:  "America/Cuiaba",
	},
	"SIVW": {
		ICAO:      "SIVW",
		Name:      "Banaer de Jacupiranga Airport",
		City:      "Jacupiranga",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 89,
		Latitude:  -24.7152786255,
		Longitude: -48.0263900757,
		Timezone:  "America/Sao_Paulo",
	},
	"SIVY": {
		ICAO:      "SIVY",
		Name:      "Fazenda Sapucai Airport",
		City:      "Tasso Fragoso",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 1575,
		Latitude:  -8.2988891602,
		Longitude: -45.7633323669,
		Timezone:  "America/Fortaleza",
	},
	"SIVZ": {
		ICAO:      "SIVZ",
		Name:      "Mata-Fome Airport",
		City:      "Itaqui",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 220,
		Latitude:  -29.1908321381,
		Longitude: -56.5486106873,
		Timezone:  "America/Sao_Paulo",
	},
	"SIWA": {
		ICAO:      "SIWA",
		Name:      "Fazenda Estrela do Aripuana Airport",
		City:      "Aripuana",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1060,
		Latitude:  -10.7488889694,
		Longitude: -59.3686103821,
		Timezone:  "America/Cuiaba",
	},
	"SIWB": {
		ICAO:      "SIWB",
		Name:      "Fazenda Sperafico Airport",
		City:      "Amambai",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1965,
		Latitude:  -23.1902770996,
		Longitude: -55.2866668701,
		Timezone:  "America/Campo_Grande",
	},
	"SIWC": {
		ICAO:      "SIWC",
		Name:      "Fazenda Santa Rita Airport",
		City:      "Diamantino",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2054,
		Latitude:  -14.1338891983,
		Longitude: -57.3661117554,
		Timezone:  "America/Cuiaba",
	},
	"SIWE": {
		ICAO:      "SIWE",
		Name:      "Nelson Pizzani Airport",
		City:      "Monte Carlo",
		State:     "Santa-Catarina",
		Country:   "BR",
		Elevation: 3223,
		Latitude:  -27.2126998901,
		Longitude: -50.9565010071,
		Timezone:  "America/Sao_Paulo",
	},
	"SIWF": {
		ICAO:      "SIWF",
		Name:      "Algodoeira Airport",
		City:      "Diamantino",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2035,
		Latitude:  -14.1855564117,
		Longitude: -57.5505561829,
		Timezone:  "America/Cuiaba",
	},
	"SIWG": {
		ICAO:      "SIWG",
		Name:      "Fazenda Sao Jose Airport",
		City:      "Rosario Oeste",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 735,
		Latitude:  -14.556388855,
		Longitude: -55.8199996948,
		Timezone:  "America/Cuiaba",
	},
	"SIWH": {
		ICAO:      "SIWH",
		Name:      "Fazenda Frigorifico Atlas Airport",
		City:      "Santana Do Araguaia",
		State:     "Para",
		Country:   "BR",
		Elevation: 2533,
		Latitude:  -9.4125003815,
		Longitude: -50.4255561829,
		Timezone:  "America/Belem",
	},
	"SIWI": {
		ICAO:      "SIWI",
		Name:      "Guapirama Airport",
		City:      "Diamantino",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1700,
		Latitude:  -13.9341669083,
		Longitude: -57.2427787781,
		Timezone:  "America/Cuiaba",
	},
	"SIWJ": {
		ICAO:      "SIWJ",
		Name:      "Luiz Eduardo Magalhaes Airport",
		City:      "Campo Verde",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2533,
		Latitude:  -15.5319442749,
		Longitude: -55.1377792358,
		Timezone:  "America/Cuiaba",
	},
	"SIWK": {
		ICAO:      "SIWK",
		Name:      "Fazenda Santa Fe Airport",
		City:      "Santa Rita Do Trivelato",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1575,
		Latitude:  -14.0513887405,
		Longitude: -55.3836097717,
		Timezone:  "America/Cuiaba",
	},
	"SIWL": {
		ICAO:      "SIWL",
		Name:      "Fazenda Espigao Airport",
		City:      "Diamantino",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1690,
		Latitude:  -14.125,
		Longitude: -56.9361114502,
		Timezone:  "America/Cuiaba",
	},
	"SIWM": {
		ICAO:      "SIWM",
		Name:      "Fazenda Bandeirantes Airport",
		City:      "Goiatuba",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1969,
		Latitude:  -17.9913883209,
		Longitude: -49.5352783203,
		Timezone:  "America/Sao_Paulo",
	},
	"SIWN": {
		ICAO:      "SIWN",
		Name:      "Fazenda Vargem das Flores Airport",
		City:      "Joviania",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1477,
		Latitude:  -17.8427772522,
		Longitude: -49.581943512,
		Timezone:  "America/Sao_Paulo",
	},
	"SIWP": {
		ICAO:      "SIWP",
		Name:      "Fazenda Agua Doce Airport",
		City:      "Laguna Carapa",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1201,
		Latitude:  -22.8672218323,
		Longitude: -54.4408340454,
		Timezone:  "America/Campo_Grande",
	},
	"SIWQ": {
		ICAO:      "SIWQ",
		Name:      "Fazenda Mateira Airport",
		City:      "Santa Rita Do Pardo",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1234,
		Latitude:  -20.9661102295,
		Longitude: -53.0783348083,
		Timezone:  "America/Campo_Grande",
	},
	"SIWR": {
		ICAO:      "SIWR",
		Name:      "Fazenda Amazonas Airport",
		City:      "Nova Crixas",
		State:     "Goias",
		Country:   "BR",
		Elevation: 985,
		Latitude:  -13.7997217178,
		Longitude: -50.4891662598,
		Timezone:  "America/Sao_Paulo",
	},
	"SIWT": {
		ICAO:      "SIWT",
		Name:      "Santa Helena Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 699,
		Latitude:  -7.8402781487,
		Longitude: -56.6594429016,
		Timezone:  "America/Santarem",
	},
	"SIWU": {
		ICAO:      "SIWU",
		Name:      "Fazenda Nossa Senhora da Candelaria Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 365,
		Latitude:  -18.518611908,
		Longitude: -57.5983314514,
		Timezone:  "America/Campo_Grande",
	},
	"SIWV": {
		ICAO:      "SIWV",
		Name:      "Boa Vista Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 699,
		Latitude:  -7.8561110497,
		Longitude: -56.7144432068,
		Timezone:  "America/Santarem",
	},
	"SIWY": {
		ICAO:      "SIWY",
		Name:      "Fazenda Campo Augusta Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 483,
		Latitude:  -17.509721756,
		Longitude: -55.7688903809,
		Timezone:  "America/Campo_Grande",
	},
	"SIWZ": {
		ICAO:      "SIWZ",
		Name:      "Fazenda Fazendinha Airport",
		City:      "Aquidauana",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 312,
		Latitude:  -19.4855556488,
		Longitude: -56.4866676331,
		Timezone:  "America/Campo_Grande",
	},
	"SIXA": {
		ICAO:      "SIXA",
		Name:      "Fazenda Uniao Suzanopolis Airport",
		City:      "Suzanopolis",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1398,
		Latitude:  -20.5316677094,
		Longitude: -51.0916671753,
		Timezone:  "America/Sao_Paulo",
	},
	"SIXB": {
		ICAO:      "SIXB",
		Name:      "Fazenda Ilha Verde Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 312,
		Latitude:  -18.5055561066,
		Longitude: -57.4022216797,
		Timezone:  "America/Campo_Grande",
	},
	"SIXD": {
		ICAO:      "SIXD",
		Name:      "Fazenda da Paz Airport",
		City:      "Santana Do Livramento",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 913,
		Latitude:  -30.8394432068,
		Longitude: -55.5672225952,
		Timezone:  "America/Montevideo",
	},
	"SIXE": {
		ICAO:      "SIXE",
		Name:      "Aeroclube de Eldorado do Sul Airport",
		City:      "Eldorado Do Sul",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 33,
		Latitude:  -30.05027771,
		Longitude: -51.443611145,
		Timezone:  "America/Sao_Paulo",
	},
	"SIXF": {
		ICAO:      "SIXF",
		Name:      "Fazenda Soledade Airport",
		City:      "Campo Alegre De Goias",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2717,
		Latitude:  -17.3833332062,
		Longitude: -47.5355567932,
		Timezone:  "America/Sao_Paulo",
	},
	"SIXG": {
		ICAO:      "SIXG",
		Name:      "Fazenda Pouso Alegre Airport",
		City:      "Catalao",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2812,
		Latitude:  -18.0277786255,
		Longitude: -47.4597206116,
		Timezone:  "America/Sao_Paulo",
	},
	"SIXI": {
		ICAO:      "SIXI",
		Name:      "Fazenda Antonio Andrade Airport",
		City:      "Itororo",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 827,
		Latitude:  -15.1802778244,
		Longitude: -40.0927772522,
		Timezone:  "America/Bahia",
	},
	"SIXL": {
		ICAO:      "SIXL",
		Name:      "Fazenda Sao Francisco Airport",
		City:      "Canarana",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1152,
		Latitude:  -13.3105564117,
		Longitude: -52.8922233582,
		Timezone:  "America/Cuiaba",
	},
	"SIXM": {
		ICAO:      "SIXM",
		Name:      "Comunidade Sao Leopoldo do Tapajos Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 492,
		Latitude:  -4.9722218513,
		Longitude: -56.9208335876,
		Timezone:  "America/Santarem",
	},
	"SIXO": {
		ICAO:      "SIXO",
		Name:      "Fazenda Nossa Senhora de Fatima Airport",
		City:      "Comodoro",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1070,
		Latitude:  -13.9844999313,
		Longitude: -59.6584014893,
		Timezone:  "America/Cuiaba",
	},
	"SIXP": {
		ICAO:      "SIXP",
		Name:      "Fazenda Paiaguas Airport",
		City:      "Diamantino",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2038,
		Latitude:  -14.0722999573,
		Longitude: -57.4506988525,
		Timezone:  "America/Cuiaba",
	},
	"SIXQ": {
		ICAO:      "SIXQ",
		Name:      "Fazenda Sao Joao Airport",
		City:      "Paranhos",
		State:     "Canindeyu",
		Country:   "BR",
		Elevation: 1500,
		Latitude:  -23.8472213745,
		Longitude: -55.4358329773,
		Timezone:  "America/Asuncion",
	},
	"SIXV": {
		ICAO:      "SIXV",
		Name:      "Fazenda Rancho do Planalto Airport",
		City:      "Alcinopolis",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 2789,
		Latitude:  -18.1394443512,
		Longitude: -53.3647232056,
		Timezone:  "America/Campo_Grande",
	},
	"SIXW": {
		ICAO:      "SIXW",
		Name:      "Cimento Poty Airport",
		City:      "Caapora",
		Country:   "BR",
		Elevation: 1,
		Latitude:  -7.5316672325,
		Longitude: -34.8616676331,
		Timezone:  "America/Fortaleza",
	},
	"SIXX": {
		ICAO:      "SIXX",
		Name:      "Usinas Braco Norte Airport",
		City:      "Guaranta Do Norte",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1159,
		Latitude:  -9.6583328247,
		Longitude: -54.9544448853,
		Timezone:  "America/Cuiaba",
	},
	"SIXZ": {
		ICAO:      "SIXZ",
		Name:      "Fazenda Spartacus Airport",
		City:      "Taquarivai",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2346,
		Latitude:  -24,
		Longitude: -48.6083335876,
		Timezone:  "America/Sao_Paulo",
	},
	"SIYE": {
		ICAO:      "SIYE",
		Name:      "Fazenda Campo Helio Airport",
		City:      "Aquidauana",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 411,
		Latitude:  -19.0963897705,
		Longitude: -55.5697212219,
		Timezone:  "America/Campo_Grande",
	},
	"SIYF": {
		ICAO:      "SIYF",
		Name:      "Fazenda Araras Airport",
		City:      "Itai",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2166,
		Latitude:  -23.3083324432,
		Longitude: -49.0891685486,
		Timezone:  "America/Sao_Paulo",
	},
	"SIYH": {
		ICAO:      "SIYH",
		Name:      "Fazenda Cruzeiro Airport",
		City:      "Rio Verde Do Mato Grosso",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 508,
		Latitude:  -18.9819431305,
		Longitude: -55.2916679382,
		Timezone:  "America/Campo_Grande",
	},
	"SIYK": {
		ICAO:      "SIYK",
		Name:      "Fazenda Sorriso Metalico Airport",
		City:      "Urutai",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2605,
		Latitude:  -17.4566669464,
		Longitude: -48.2152786255,
		Timezone:  "America/Sao_Paulo",
	},
	"SIYN": {
		ICAO:      "SIYN",
		Name:      "Eletro Primavera LTDA Airport",
		City:      "Pimenta Bueno",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 749,
		Latitude:  -11.9008331299,
		Longitude: -61.230556488,
		Timezone:  "America/Porto_Velho",
	},
	"SIYO": {
		ICAO:      "SIYO",
		Name:      "Fazenda Goias Barreiro Airport",
		City:      "Montividiu",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2848,
		Latitude:  -17.2555561066,
		Longitude: -51.306388855,
		Timezone:  "America/Sao_Paulo",
	},
	"SIYP": {
		ICAO:      "SIYP",
		Name:      "Fazenda Shangri-la Airport",
		City:      "Alta Floresta",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1057,
		Latitude:  -10.282500267,
		Longitude: -56.469165802,
		Timezone:  "America/Cuiaba",
	},
	"SIYQ": {
		ICAO:      "SIYQ",
		Name:      "Tapura Airport",
		City:      "Tapura",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1310,
		Latitude:  -12.7916669846,
		Longitude: -56.4986114502,
		Timezone:  "America/Cuiaba",
	},
	"SIYR": {
		ICAO:      "SIYR",
		Name:      "Santa Terezinha Airport",
		City:      "Tapajara",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 2199,
		Latitude:  -28.0522212982,
		Longitude: -51.9608345032,
		Timezone:  "America/Sao_Paulo",
	},
	"SIYS": {
		ICAO:      "SIYS",
		Name:      "Helibras Airport",
		City:      "Itajuba",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2759,
		Latitude:  -22.4308338165,
		Longitude: -45.4791679382,
		Timezone:  "America/Sao_Paulo",
	},
	"SIYT": {
		ICAO:      "SIYT",
		Name:      "Mamuru Airport",
		City:      "Juruti",
		State:     "Para",
		Country:   "BR",
		Elevation: 99,
		Latitude:  -3.1169440746,
		Longitude: -56.5861091614,
		Timezone:  "America/Santarem",
	},
	"SIYW": {
		ICAO:      "SIYW",
		Name:      "Fazenda Salvacao Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 601,
		Latitude:  -5.9522218704,
		Longitude: -56.5433311462,
		Timezone:  "America/Santarem",
	},
	"SIYX": {
		ICAO:      "SIYX",
		Name:      "Fazenda Varginha Airport",
		City:      "Andradas",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 4200,
		Latitude:  -21.9500007629,
		Longitude: -46.5499992371,
		Timezone:  "America/Sao_Paulo",
	},
	"SIYY": {
		ICAO:      "SIYY",
		Name:      "Ilha do Caracara Airport",
		City:      "Pocone",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 610,
		Latitude:  -17.4585990906,
		Longitude: -56.8405990601,
		Timezone:  "America/Campo_Grande",
	},
	"SIZA": {
		ICAO:      "SIZA",
		Name:      "Parque Manoelito Argolo Airport",
		City:      "Entre Rios",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 558,
		Latitude:  -11.9602775574,
		Longitude: -38.0027770996,
		Timezone:  "America/Bahia",
	},
	"SIZC": {
		ICAO:      "SIZC",
		Name:      "Fazenda Fenix Airport",
		City:      "Faria Lemos",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1542,
		Latitude:  -20.7794437408,
		Longitude: -41.9955558777,
		Timezone:  "America/Sao_Paulo",
	},
	"SIZF": {
		ICAO:      "SIZF",
		Name:      "Nossa Senhora das Gracas Airport",
		City:      "Altonia",
		State:     "Parana",
		Country:   "BR",
		Elevation: 873,
		Latitude:  -23.9950008392,
		Longitude: -53.956111908,
		Timezone:  "America/Sao_Paulo",
	},
	"SIZG": {
		ICAO:      "SIZG",
		Name:      "Fazenda Encontro das Aguas Airport",
		City:      "Sao Manuel",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1631,
		Latitude:  -22.6079006195,
		Longitude: -48.3879013062,
		Timezone:  "America/Sao_Paulo",
	},
	"SIZH": {
		ICAO:      "SIZH",
		Name:      "Fazenda Vista Alegre Airport",
		City:      "Aparecida Do Taboado",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1483,
		Latitude:  -19.9930553436,
		Longitude: -51.5141677856,
		Timezone:  "America/Campo_Grande",
	},
	"SIZI": {
		ICAO:      "SIZI",
		Name:      "Fazenda Vitoria Airport",
		City:      "Itaruma",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1854,
		Latitude:  -18.7527999878,
		Longitude: -51.4730987549,
		Timezone:  "America/Sao_Paulo",
	},
	"SIZK": {
		ICAO:      "SIZK",
		Name:      "Fazenda Vera Cruz Airport",
		City:      "Barra Do Garcas",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 995,
		Latitude:  -14.875,
		Longitude: -52.1155548096,
		Timezone:  "America/Cuiaba",
	},
	"SIZM": {
		ICAO:      "SIZM",
		Name:      "Fazenda Douradinho Airport",
		City:      "Nova Andradina",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1201,
		Latitude:  -21.640832901,
		Longitude: -53.3005561829,
		Timezone:  "America/Campo_Grande",
	},
	"SIZN": {
		ICAO:      "SIZN",
		Name:      "Fazenda Santa Ana Airport",
		City:      "Corumbiara",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 847,
		Latitude:  -13.0041666031,
		Longitude: -61.2400016785,
		Timezone:  "America/Porto_Velho",
	},
	"SIZO": {
		ICAO:      "SIZO",
		Name:      "Calcario Vale do Araguaia Airport",
		City:      "Cocalinho",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 860,
		Latitude:  -14.1902999878,
		Longitude: -51.643901825,
		Timezone:  "America/Cuiaba",
	},
	"SIZQ": {
		ICAO:      "SIZQ",
		Name:      "Fazenda Real Airport",
		City:      "Botucatu",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 3078,
		Latitude:  -23.0361118317,
		Longitude: -48.4405555725,
		Timezone:  "America/Sao_Paulo",
	},
	"SIZR": {
		ICAO:      "SIZR",
		Name:      "Fazenda Carolina Airport",
		City:      "Corumbiara",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 853,
		Latitude:  -12.9005556107,
		Longitude: -61.1300010681,
		Timezone:  "America/Porto_Velho",
	},
	"SIZS": {
		ICAO:      "SIZS",
		Name:      "Agropecuaria Equus Airport",
		City:      "Gaucha",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1201,
		Latitude:  -13.5375003815,
		Longitude: -53.4391670227,
		Timezone:  "America/Cuiaba",
	},
	"SIZT": {
		ICAO:      "SIZT",
		Name:      "Fazenda Buriti Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 299,
		Latitude:  -19.3544445038,
		Longitude: -56.793056488,
		Timezone:  "America/Campo_Grande",
	},
	"SIZX": {
		ICAO:      "SIZX",
		IATA:      "JUA",
		Name:      "Juara Sul Airport",
		City:      "Juara",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 991,
		Latitude:  -11.2866668701,
		Longitude: -57.5388908386,
		Timezone:  "America/Cuiaba",
	},
	"SJAA": {
		ICAO:      "SJAA",
		Name:      "Fazenda Sao Jose do Rio Branco Airport",
		City:      "Pontes E Lacerda",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1142,
		Latitude:  -15.0713891983,
		Longitude: -59.1533317566,
		Timezone:  "America/Cuiaba",
	},
	"SJAC": {
		ICAO:      "SJAC",
		Name:      "Fazenda Rio Preto Airport",
		City:      "Sao Felix Do Araguaia",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 886,
		Latitude:  -11.4755563736,
		Longitude: -51.6872215271,
		Timezone:  "America/Cuiaba",
	},
	"SJAE": {
		ICAO:      "SJAE",
		Name:      "Fazenda Furnas Airport",
		City:      "Ribeirao Cascalheira",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1618,
		Latitude:  -13.1402778625,
		Longitude: -51.801109314,
		Timezone:  "America/Cuiaba",
	},
	"SJAF": {
		ICAO:      "SJAF",
		Name:      "Fazenda Agua Limpa Airport",
		City:      "Campo Novo Do Parecis",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1804,
		Latitude:  -13.670832634,
		Longitude: -57.8955574036,
		Timezone:  "America/Cuiaba",
	},
	"SJAG": {
		ICAO:      "SJAG",
		Name:      "Fazenda Paredao Airport",
		City:      "Vila Bela Da Santissima Trindade",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 804,
		Latitude:  -13.628610611,
		Longitude: -60.2883338928,
		Timezone:  "America/Cuiaba",
	},
	"SJAI": {
		ICAO:      "SJAI",
		Name:      "Fazenda Davilandia Airport",
		City:      "Montividiu Do Norte",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1359,
		Latitude:  -13.1875,
		Longitude: -48.75,
		Timezone:  "America/Sao_Paulo",
	},
	"SJAK": {
		ICAO:      "SJAK",
		Name:      "Fazenda F5 Airport",
		City:      "Tupaciguara",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1624,
		Latitude:  -18.3641662598,
		Longitude: -48.8719444275,
		Timezone:  "America/Sao_Paulo",
	},
	"SJAL": {
		ICAO:      "SJAL",
		Name:      "Fazenda Santa Maria Airport",
		City:      "Acailandia",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 1077,
		Latitude:  -4.6463890076,
		Longitude: -47.2333335876,
		Timezone:  "America/Fortaleza",
	},
	"SJAQ": {
		ICAO:      "SJAQ",
		Name:      "Fazenda Renascenca Airport",
		City:      "Unai",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2999,
		Latitude:  -16.1202774048,
		Longitude: -46.3600006104,
		Timezone:  "America/Sao_Paulo",
	},
	"SJAR": {
		ICAO:      "SJAR",
		Name:      "Fazenda Bela Manha Airport",
		City:      "Santa Clara D`Oeste",
		Country:   "BR",
		Elevation: 1198,
		Latitude:  -20.0186004639,
		Longitude: -50.8992004395,
		Timezone:  "America/Sao_Paulo",
	},
	"SJAS": {
		ICAO:      "SJAS",
		Name:      "Fazenda Sao Roque Airport",
		City:      "Reginopolis",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1450,
		Latitude:  -21.6691665649,
		Longitude: -49.1730575562,
		Timezone:  "America/Sao_Paulo",
	},
	"SJAU": {
		ICAO:      "SJAU",
		Name:      "Araguacema Airport",
		City:      "Araguacema",
		Country:   "BR",
		Elevation: 680,
		Latitude:  -8.8377199173,
		Longitude: -49.5574989319,
		Timezone:  "America/Araguaina",
	},
	"SJAW": {
		ICAO:      "SJAW",
		Name:      "Fazenda Citricola Airport",
		City:      "Gaviao Peixoto",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2057,
		Latitude:  -21.7252788544,
		Longitude: -48.4519424438,
		Timezone:  "America/Sao_Paulo",
	},
	"SJAX": {
		ICAO:      "SJAX",
		Name:      "Fazenda Acalanto Airport",
		City:      "Sao Desiderio",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 2657,
		Latitude:  -12.9650001526,
		Longitude: -45.9761123657,
		Timezone:  "America/Bahia",
	},
	"SJAY": {
		ICAO:      "SJAY",
		Name:      "Fazenda Ventura Airport",
		City:      "Sao Desiderio",
		Country:   "BR",
		Elevation: 3127,
		Latitude:  -13.1608333588,
		Longitude: -46.2008323669,
		Timezone:  "America/Bahia",
	},
	"SJAZ": {
		ICAO:      "SJAZ",
		Name:      "Fazenda Querubim Airport",
		City:      "Sao Desiderio",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 2733,
		Latitude:  -12.7688894272,
		Longitude: -46.0213890076,
		Timezone:  "America/Bahia",
	},
	"SJBF": {
		ICAO:      "SJBF",
		Name:      "Fazenda J.K. Airport",
		City:      "Mimoso De Goias",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2136,
		Latitude:  -15.1588888168,
		Longitude: -48.2838897705,
		Timezone:  "America/Sao_Paulo",
	},
	"SJBK": {
		ICAO:      "SJBK",
		Name:      "Iriri Airport",
		City:      "Altamira",
		State:     "Para",
		Country:   "BR",
		Elevation: 581,
		Latitude:  -5.6791667938,
		Longitude: -54.2449989319,
		Timezone:  "America/Santarem",
	},
	"SJBN": {
		ICAO:      "SJBN",
		Name:      "Aldeia Pykany Airport",
		City:      "Altamira",
		State:     "Para",
		Country:   "BR",
		Elevation: 1100,
		Latitude:  -8.1666669846,
		Longitude: -54.8297233582,
		Timezone:  "America/Santarem",
	},
	"SJBR": {
		ICAO:      "SJBR",
		Name:      "Fazenda Santa Luzia Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 401,
		Latitude:  -17.8766670227,
		Longitude: -56.3888893127,
		Timezone:  "America/Campo_Grande",
	},
	"SJBS": {
		ICAO:      "SJBS",
		Name:      "Fazenda Livramento Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 443,
		Latitude:  -19.268611908,
		Longitude: -56.0311126709,
		Timezone:  "America/Campo_Grande",
	},
	"SJBT": {
		ICAO:      "SJBT",
		Name:      "Fazenda Sao Joao Airport",
		City:      "Aquidauana",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 417,
		Latitude:  -19.5744438171,
		Longitude: -56.099445343,
		Timezone:  "America/Campo_Grande",
	},
	"SJBU": {
		ICAO:      "SJBU",
		Name:      "Fazenda Baluarte Airport",
		City:      "Lagoa Dos Patos",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1720,
		Latitude:  -17.0127773285,
		Longitude: -44.7902793884,
		Timezone:  "America/Sao_Paulo",
	},
	"SJBW": {
		ICAO:      "SJBW",
		Name:      "Fazenda Barbacena Airport",
		City:      "Sao Pedro Do Ivai",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1247,
		Latitude:  -23.8394432068,
		Longitude: -51.9027786255,
		Timezone:  "America/Sao_Paulo",
	},
	"SJBX": {
		ICAO:      "SJBX",
		Name:      "Severino Lopes Airport",
		City:      "Sao Jose Do Mipibu",
		State:     "Rio-Grande-do-Norte",
		Country:   "BR",
		Elevation: 329,
		Latitude:  -6.0199999809,
		Longitude: -35.2397232056,
		Timezone:  "America/Fortaleza",
	},
	"SJBY": {
		ICAO:      "SJBY",
		Name:      "Joao Silva Airport",
		City:      "Santa Ines",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 82,
		Latitude:  -3.6542799473,
		Longitude: -45.345199585,
		Timezone:  "America/Fortaleza",
	},
	"SJBZ": {
		ICAO:      "SJBZ",
		Name:      "Fazenda Tres Barras Airport",
		City:      "Itaruma",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1411,
		Latitude:  -18.9572219849,
		Longitude: -51.1086120605,
		Timezone:  "America/Sao_Paulo",
	},
	"SJCA": {
		ICAO:      "SJCA",
		Name:      "Comandante Vittorio Bonomi Airport",
		City:      "Mococa",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2133,
		Latitude:  -21.5303993225,
		Longitude: -47.0465011597,
		Timezone:  "America/Sao_Paulo",
	},
	"SJCD": {
		ICAO:      "SJCD",
		Name:      "Fazenda Pingo D`Agua Airport",
		City:      "Pedro Gomes",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1303,
		Latitude:  -17.7211112976,
		Longitude: -53.9375,
		Timezone:  "America/Campo_Grande",
	},
	"SJCE": {
		ICAO:      "SJCE",
		Name:      "Usina Eldorado Airport",
		City:      "Rio Brilhante",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1690,
		Latitude:  -21.8522224426,
		Longitude: -54.0188903809,
		Timezone:  "America/Campo_Grande",
	},
	"SJCF": {
		ICAO:      "SJCF",
		Name:      "Fazenda Estrela D`Alva Airport",
		City:      "Canarana",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1250,
		Latitude:  -13.1891670227,
		Longitude: -52.3069458008,
		Timezone:  "America/Cuiaba",
	},
	"SJCL": {
		ICAO:      "SJCL",
		Name:      "Fazenda Aracatuba Airport",
		City:      "Nova Crixas",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1100,
		Latitude:  -14.5313892365,
		Longitude: -50.4086112976,
		Timezone:  "America/Sao_Paulo",
	},
	"SJCM": {
		ICAO:      "SJCM",
		Name:      "Fazenda Sao Marcus Airport",
		City:      "Pontal Do Araguaia",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 4361,
		Latitude:  -16.1216678619,
		Longitude: -52.9508323669,
		Timezone:  "America/Cuiaba",
	},
	"SJCN": {
		ICAO:      "SJCN",
		Name:      "Aero Agricola Cristalina Airport",
		City:      "Cristalina",
		State:     "Goias",
		Country:   "BR",
		Elevation: 3544,
		Latitude:  -16.9069442749,
		Longitude: -47.7083320618,
		Timezone:  "America/Sao_Paulo",
	},
	"SJCO": {
		ICAO:      "SJCO",
		Name:      "Fazenda Aparecida Airport",
		City:      "Penapolis",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1510,
		Latitude:  -21.4980564117,
		Longitude: -50.1011123657,
		Timezone:  "America/Sao_Paulo",
	},
	"SJCP": {
		ICAO:      "SJCP",
		Name:      "Fazenda Sao Roberto Airport",
		City:      "Parauna",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2497,
		Latitude:  -17.0727787018,
		Longitude: -50.8474998474,
		Timezone:  "America/Sao_Paulo",
	},
	"SJCQ": {
		ICAO:      "SJCQ",
		Name:      "Fazenda Serra Azul Airport",
		City:      "Baixa Grande",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1123,
		Latitude:  -11.935500145,
		Longitude: -40.0507011414,
		Timezone:  "America/Bahia",
	},
	"SJCW": {
		ICAO:      "SJCW",
		Name:      "Jatapu Airport",
		City:      "Urucara",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 132,
		Latitude:  -1.7120499611,
		Longitude: -58.5111999512,
		Timezone:  "America/Manaus",
	},
	"SJCY": {
		ICAO:      "SJCY",
		Name:      "Estancia Santa Rita Airport",
		City:      "Cuiaba",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 617,
		Latitude:  -15.679444313,
		Longitude: -55.9555549622,
		Timezone:  "America/Cuiaba",
	},
	"SJCZ": {
		ICAO:      "SJCZ",
		Name:      "Fazenda Paraiso Airport",
		City:      "Nova Ubirata",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1101,
		Latitude:  -12.4291667938,
		Longitude: -54.2563896179,
		Timezone:  "America/Cuiaba",
	},
	"SJDA": {
		ICAO:      "SJDA",
		Name:      "Fazenda Cruzeiro Airport",
		City:      "Uberaba",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3160,
		Latitude:  -19.1922225952,
		Longitude: -48.1172218323,
		Timezone:  "America/Sao_Paulo",
	},
	"SJDD": {
		ICAO:      "SJDD",
		Name:      "Fazenda Jumbo Airport",
		City:      "Taquarussu",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 854,
		Latitude:  -22.5138893127,
		Longitude: -53.3419456482,
		Timezone:  "America/Campo_Grande",
	},
	"SJDG": {
		ICAO:      "SJDG",
		Name:      "Projeto Iriri Airport",
		City:      "Altamira",
		State:     "Para",
		Country:   "BR",
		Elevation: 657,
		Latitude:  -6.1880559921,
		Longitude: -54.0722236633,
		Timezone:  "America/Santarem",
	},
	"SJDH": {
		ICAO:      "SJDH",
		Name:      "Fazenda S.M. Airport",
		City:      "Jaborandi",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 2891,
		Latitude:  -14.5500001907,
		Longitude: -45.7666664124,
		Timezone:  "America/Bahia",
	},
	"SJDI": {
		ICAO:      "SJDI",
		Name:      "Fazenda Concordia Airport",
		City:      "Navirai",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1247,
		Latitude:  -22.8491668701,
		Longitude: -54.206111908,
		Timezone:  "America/Campo_Grande",
	},
	"SJDJ": {
		ICAO:      "SJDJ",
		Name:      "Arenhart Airport",
		City:      "Uruguaiana",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 210,
		Latitude:  -29.8886108398,
		Longitude: -57.1399993896,
		Timezone:  "America/Sao_Paulo",
	},
	"SJDL": {
		ICAO:      "SJDL",
		Name:      "Fazenda Paraiso Airport",
		City:      "Corumba",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1313,
		Latitude:  -17.3077774048,
		Longitude: -55.8452796936,
		Timezone:  "America/Campo_Grande",
	},
	"SJDN": {
		ICAO:      "SJDN",
		Name:      "Fazenda Campo Alegre Airport",
		City:      "Aquidauana",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 558,
		Latitude:  -19.456111908,
		Longitude: -55.7408332825,
		Timezone:  "America/Campo_Grande",
	},
	"SJDS": {
		ICAO:      "SJDS",
		Name:      "Fabrica Fortaleza Airport",
		City:      "Eusebio",
		State:     "Ceara",
		Country:   "BR",
		Elevation: 66,
		Latitude:  -3.9272220135,
		Longitude: -38.5005569458,
		Timezone:  "America/Fortaleza",
	},
	"SJDT": {
		ICAO:      "SJDT",
		Name:      "Fazenda Sao Roque Airport",
		City:      "Aquidauana",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 453,
		Latitude:  -19.5183334351,
		Longitude: -55.3119430542,
		Timezone:  "America/Campo_Grande",
	},
	"SJDU": {
		ICAO:      "SJDU",
		Name:      "Nossa Senhora do Loreto Airport",
		City:      "Uruguaiana",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 329,
		Latitude:  -29.728055954,
		Longitude: -56.962223053,
		Timezone:  "America/Sao_Paulo",
	},
	"SJDV": {
		ICAO:      "SJDV",
		Name:      "Fazenda Redencao Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 312,
		Latitude:  -19.6186103821,
		Longitude: -56.743057251,
		Timezone:  "America/Campo_Grande",
	},
	"SJDW": {
		ICAO:      "SJDW",
		Name:      "Fazenda Canada Airport",
		City:      "Caracol",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 548,
		Latitude:  -22.1269435883,
		Longitude: -57.2894439697,
		Timezone:  "America/Campo_Grande",
	},
	"SJDX": {
		ICAO:      "SJDX",
		Name:      "Fazenda Bandeirantes Airport",
		City:      "Aquidauana",
		Country:   "BR",
		Elevation: 375,
		Latitude:  -19.4122219086,
		Longitude: -55.5497207642,
		Timezone:  "America/Campo_Grande",
	},
	"SJDY": {
		ICAO:      "SJDY",
		Name:      "Fazenda da Barra Airport",
		City:      "Camapua",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1533,
		Latitude:  -18.6630554199,
		Longitude: -54.068611145,
		Timezone:  "America/Campo_Grande",
	},
	"SJEA": {
		ICAO:      "SJEA",
		Name:      "Cadete 56-147 Santana Airport",
		City:      "Palmeiropolis",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 985,
		Latitude:  -12.8500003815,
		Longitude: -48.225276947,
		Timezone:  "America/Araguaina",
	},
	"SJED": {
		ICAO:      "SJED",
		Name:      "Fazenda Abath Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 348,
		Latitude:  -18.6800003052,
		Longitude: -57.2249984741,
		Timezone:  "America/Campo_Grande",
	},
	"SJEF": {
		ICAO:      "SJEF",
		Name:      "Fazenda Costa Rica Airport",
		City:      "Aquidauana",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 342,
		Latitude:  -19.7469444275,
		Longitude: -56.0422210693,
		Timezone:  "America/Campo_Grande",
	},
	"SJEG": {
		ICAO:      "SJEG",
		Name:      "Estancia Esmeralda Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 525,
		Latitude:  -20.1605567932,
		Longitude: -57.1580543518,
		Timezone:  "America/Campo_Grande",
	},
	"SJEH": {
		ICAO:      "SJEH",
		Name:      "Fazenda Santa Eliza Airport",
		City:      "Ribeirao Bonito",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2395,
		Latitude:  -22.1019439697,
		Longitude: -48.2269439697,
		Timezone:  "America/Sao_Paulo",
	},
	"SJEJ": {
		ICAO:      "SJEJ",
		Name:      "Fazenda Bonanza Airport",
		City:      "Avai",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1713,
		Latitude:  -22.1550006866,
		Longitude: -49.287776947,
		Timezone:  "America/Sao_Paulo",
	},
	"SJEK": {
		ICAO:      "SJEK",
		Name:      "Satelite Aviacao Agricola Airport",
		City:      "Canarana",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 4594,
		Latitude:  -13.5455560684,
		Longitude: -52.2638893127,
		Timezone:  "America/Cuiaba",
	},
	"SJEL": {
		ICAO:      "SJEL",
		Name:      "Eldorado Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 837,
		Latitude:  -6.9555559158,
		Longitude: -56.4711112976,
		Timezone:  "America/Santarem",
	},
	"SJEM": {
		ICAO:      "SJEM",
		Name:      "Estancia Pousada dos Passaros Airport",
		City:      "Vera",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1231,
		Latitude:  -12.3191671371,
		Longitude: -55.3338890076,
		Timezone:  "America/Cuiaba",
	},
	"SJEN": {
		ICAO:      "SJEN",
		Name:      "Pista PLA e Silva Airport",
		City:      "Santa Vitoria Do Palmar",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 10,
		Latitude:  -33.1438903809,
		Longitude: -52.9763908386,
		Timezone:  "America/Sao_Paulo",
	},
	"SJEQ": {
		ICAO:      "SJEQ",
		Name:      "Fazenda Bom Jesus Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 404,
		Latitude:  -18.1827774048,
		Longitude: -56.7825012207,
		Timezone:  "America/Campo_Grande",
	},
	"SJER": {
		ICAO:      "SJER",
		Name:      "Fazenda Canada Airport",
		City:      "Acreuna",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1841,
		Latitude:  -17.31888889,
		Longitude: -50.32027778,
		Timezone:  "America/Sao_Paulo",
	},
	"SJES": {
		ICAO:      "SJES",
		Name:      "Fazenda Agua Pe Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 407,
		Latitude:  -18.1177787781,
		Longitude: -56.6411094666,
		Timezone:  "America/Campo_Grande",
	},
	"SJET": {
		ICAO:      "SJET",
		Name:      "Fazenda Recreio Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 404,
		Latitude:  -17.5647220612,
		Longitude: -56.7866668701,
		Timezone:  "America/Campo_Grande",
	},
	"SJEU": {
		ICAO:      "SJEU",
		Name:      "Fazenda Sao Cristovao Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 404,
		Latitude:  -17.8691673279,
		Longitude: -56.1938896179,
		Timezone:  "America/Campo_Grande",
	},
	"SJEV": {
		ICAO:      "SJEV",
		Name:      "Fazenda Sao Francisco Airport",
		City:      "Corumba",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 401,
		Latitude:  -17.6483325958,
		Longitude: -56.8861122131,
		Timezone:  "America/Campo_Grande",
	},
	"SJEW": {
		ICAO:      "SJEW",
		Name:      "Fazenda Horita Airport",
		City:      "Sao Desiderio",
		Country:   "BR",
		Elevation: 3163,
		Latitude:  -13.0111112595,
		Longitude: -46.2380561829,
		Timezone:  "America/Bahia",
	},
	"SJEX": {
		ICAO:      "SJEX",
		Name:      "Fazenda Vitoria Airport",
		City:      "Sao Desiderio",
		Country:   "BR",
		Elevation: 2966,
		Latitude:  -13.010556221,
		Longitude: -46.1077766418,
		Timezone:  "America/Bahia",
	},
	"SJEY": {
		ICAO:      "SJEY",
		Name:      "Fazenda Agronet Airport",
		City:      "Indiapora",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1562,
		Latitude:  -19.9905567169,
		Longitude: -50.2661094666,
		Timezone:  "America/Sao_Paulo",
	},
	"SJFB": {
		ICAO:      "SJFB",
		Name:      "Fazenda Bacuri Airport",
		City:      "Riachao",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 1815,
		Latitude:  -8.3827781677,
		Longitude: -46.4422225952,
		Timezone:  "America/Fortaleza",
	},
	"SJFF": {
		ICAO:      "SJFF",
		Name:      "Frigo Estrela Airport",
		City:      "Estrela D`Oeste",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1595,
		Latitude:  -20.2947216034,
		Longitude: -50.4105567932,
		Timezone:  "America/Sao_Paulo",
	},
	"SJFG": {
		ICAO:      "SJFG",
		Name:      "Fazenda Penedo Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 250,
		Latitude:  -5.5211110115,
		Longitude: -57.1074981689,
		Timezone:  "America/Santarem",
	},
	"SJFI": {
		ICAO:      "SJFI",
		Name:      "CIFI Airport",
		City:      "Cuiaba",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 644,
		Latitude:  -15.634721756,
		Longitude: -55.9905548096,
		Timezone:  "America/Cuiaba",
	},
	"SJFJ": {
		ICAO:      "SJFJ",
		Name:      "Tres Marias da Pitangueiras Airport",
		City:      "Santo Antonio Do Leverger",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 522,
		Latitude:  -17.2350006104,
		Longitude: -55.2266654968,
		Timezone:  "America/Cuiaba",
	},
	"SJFK": {
		ICAO:      "SJFK",
		Name:      "Fazenda Cachoeira Airport",
		City:      "Araguacu",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 752,
		Latitude:  -12.5966672897,
		Longitude: -49.5180549622,
		Timezone:  "America/Araguaina",
	},
	"SJFL": {
		ICAO:      "SJFL",
		Name:      "Fazenda Campo Formoso Airport",
		City:      "Sandolandia",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 1083,
		Latitude:  -12.5616674423,
		Longitude: -49.8633346558,
		Timezone:  "America/Araguaina",
	},
	"SJFM": {
		ICAO:      "SJFM",
		Name:      "Fazenda Macuco Airport",
		City:      "Eldorado",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1149,
		Latitude:  -23.8358325958,
		Longitude: -54.231388092,
		Timezone:  "America/Campo_Grande",
	},
	"SJFN": {
		ICAO:      "SJFN",
		Name:      "Fazenda Brasil Novo Airport",
		City:      "Coxim",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 752,
		Latitude:  -18.3716678619,
		Longitude: -54.9288902283,
		Timezone:  "America/Campo_Grande",
	},
	"SJFO": {
		ICAO:      "SJFO",
		Name:      "Fazenda California Airport",
		City:      "Porteirao",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1549,
		Latitude:  -18.0366668701,
		Longitude: -50.1994438171,
		Timezone:  "America/Sao_Paulo",
	},
	"SJFP": {
		ICAO:      "SJFP",
		Name:      "Fazenda Perobal Airport",
		City:      "Corumbiara",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 984,
		Latitude:  -12.9108333588,
		Longitude: -61.368888855,
		Timezone:  "America/Porto_Velho",
	},
	"SJFQ": {
		ICAO:      "SJFQ",
		Name:      "Fazenda Quatrirmas Airport",
		City:      "Itatinga",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2441,
		Latitude:  -22.959444046,
		Longitude: -48.6747207642,
		Timezone:  "America/Sao_Paulo",
	},
	"SJFR": {
		ICAO:      "SJFR",
		Name:      "Fronteira Airport",
		City:      "Brasnorte",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1492,
		Latitude:  -12.523888588,
		Longitude: -58.2355575562,
		Timezone:  "America/Cuiaba",
	},
	"SJFT": {
		ICAO:      "SJFT",
		Name:      "Fazenda Triunfo Rio Turvo Airport",
		City:      "Nova Bandeirantes",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1018,
		Latitude:  -10.0930557251,
		Longitude: -57.7269439697,
		Timezone:  "America/Cuiaba",
	},
	"SJFU": {
		ICAO:      "SJFU",
		Name:      "Tabuleiro Airport",
		City:      "Bananeiras",
		State:     "Paraiba",
		Country:   "BR",
		Elevation: 1716,
		Latitude:  -6.6941671371,
		Longitude: -35.630279541,
		Timezone:  "America/Fortaleza",
	},
	"SJFV": {
		ICAO:      "SJFV",
		Name:      "Fazenda Gaia Airport",
		City:      "Monte Alegre De Minas",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2762,
		Latitude:  -18.7600002289,
		Longitude: -48.755279541,
		Timezone:  "America/Sao_Paulo",
	},
	"SJFW": {
		ICAO:      "SJFW",
		Name:      "Fazenda W. Egidio III Airport",
		City:      "Unai",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2648,
		Latitude:  -15.9380559921,
		Longitude: -46.4775009155,
		Timezone:  "America/Sao_Paulo",
	},
	"SJFX": {
		ICAO:      "SJFX",
		Name:      "Fazenda Santo Angelo Airport",
		City:      "Sao Desiderio",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 2799,
		Latitude:  -12.798333168,
		Longitude: -46.0958328247,
		Timezone:  "America/Bahia",
	},
	"SJGA": {
		ICAO:      "SJGA",
		Name:      "Aerorural Floramazon Airport",
		City:      "Aveiro",
		State:     "Para",
		Country:   "BR",
		Elevation: 131,
		Latitude:  -3.7022221088,
		Longitude: -56.0538902283,
		Timezone:  "America/Santarem",
	},
	"SJGB": {
		ICAO:      "SJGB",
		Name:      "Fazenda Ponte de Pedra Airport",
		City:      "Campo Novo Do Parecis",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1690,
		Latitude:  -13.5841665268,
		Longitude: -57.463054657,
		Timezone:  "America/Cuiaba",
	},
	"SJGC": {
		ICAO:      "SJGC",
		Name:      "Fazenda Busato I Airport",
		City:      "Sao Desiderio",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1201,
		Latitude:  -12.9016666412,
		Longitude: -45.5,
		Timezone:  "America/Bahia",
	},
	"SJGD": {
		ICAO:      "SJGD",
		Name:      "Fazenda Rio Brilhante Airport",
		City:      "Sao Desiderio",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 2500,
		Latitude:  -12.8033332825,
		Longitude: -45.7236099243,
		Timezone:  "America/Bahia",
	},
	"SJGE": {
		ICAO:      "SJGE",
		Name:      "Pista Nacoes Unidas Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 591,
		Latitude:  -6.0813889503,
		Longitude: -56.2894439697,
		Timezone:  "America/Santarem",
	},
	"SJGF": {
		ICAO:      "SJGF",
		Name:      "Aerodromo Nova Era Airport",
		City:      "Pantano Grande",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 131,
		Latitude:  -30.2127780914,
		Longitude: -52.4413871765,
		Timezone:  "America/Sao_Paulo",
	},
	"SJGH": {
		ICAO:      "SJGH",
		Name:      "Aeropel Airport",
		City:      "Sao Borja",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 300,
		Latitude:  -28.7163887024,
		Longitude: -55.9925003052,
		Timezone:  "America/Sao_Paulo",
	},
	"SJGI": {
		ICAO:      "SJGI",
		Name:      "Fazenda Cypi Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 328,
		Latitude:  -19.3380565643,
		Longitude: -56.9077796936,
		Timezone:  "America/Campo_Grande",
	},
	"SJGJ": {
		ICAO:      "SJGJ",
		Name:      "Namocoli Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 282,
		Latitude:  -20.6088886261,
		Longitude: -57.5541687012,
		Timezone:  "America/Campo_Grande",
	},
	"SJGK": {
		ICAO:      "SJGK",
		Name:      "Fazenda Rio liberdade Airport",
		City:      "Santana Do Araguaia",
		State:     "Para",
		Country:   "BR",
		Elevation: 404,
		Latitude:  -9.4286108017,
		Longitude: -51.0269432068,
		Timezone:  "America/Belem",
	},
	"SJGN": {
		ICAO:      "SJGN",
		Name:      "Fazenda Agromar Airport",
		City:      "Sao Jose Do Rio Claro",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1201,
		Latitude:  -13.6625003815,
		Longitude: -56.4827766418,
		Timezone:  "America/Cuiaba",
	},
	"SJGO": {
		ICAO:      "SJGO",
		Name:      "Eldorado Airport",
		City:      "Eldorado",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1079,
		Latitude:  -23.818611145,
		Longitude: -54.2524986267,
		Timezone:  "America/Campo_Grande",
	},
	"SJGP": {
		ICAO:      "SJGP",
		Name:      "Condominio Fly-In Community Airport",
		City:      "Rondonopolis",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1047,
		Latitude:  -16.4461994171,
		Longitude: -54.6641998291,
		Timezone:  "America/Cuiaba",
	},
	"SJGR": {
		ICAO:      "SJGR",
		Name:      "Fazenda Cavalgada Grande Airport",
		City:      "Crixas",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1050,
		Latitude:  -14.604722023,
		Longitude: -50.1527786255,
		Timezone:  "America/Sao_Paulo",
	},
	"SJGT": {
		ICAO:      "SJGT",
		Name:      "Salto Augusto Airport",
		City:      "Apiacas",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1559,
		Latitude:  -8.8927783966,
		Longitude: -58.5497207642,
		Timezone:  "America/Cuiaba",
	},
	"SJGU": {
		ICAO:      "SJGU",
		Name:      "Araguatins Airport",
		City:      "Araguatins",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 411,
		Latitude:  -5.684440136,
		Longitude: -48.1156005859,
		Timezone:  "America/Araguaina",
	},
	"SJGW": {
		ICAO:      "SJGW",
		Name:      "Estancia Floresta Airport",
		City:      "Sinop",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1280,
		Latitude:  -11.7827777863,
		Longitude: -55.5391654968,
		Timezone:  "America/Cuiaba",
	},
	"SJGX": {
		ICAO:      "SJGX",
		Name:      "Da Praia Velha Airport",
		City:      "Camaqua",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 30,
		Latitude:  -31.1266670227,
		Longitude: -51.5758323669,
		Timezone:  "America/Sao_Paulo",
	},
	"SJGY": {
		ICAO:      "SJGY",
		Name:      "Fazenda Barriguda Airport",
		City:      "Sao Domingos",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1965,
		Latitude:  -13.707777977,
		Longitude: -46.7358322144,
		Timezone:  "America/Sao_Paulo",
	},
	"SJGZ": {
		ICAO:      "SJGZ",
		Name:      "Fazenda Aracatuba Airport",
		City:      "Anaurilandia",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1231,
		Latitude:  -22.0900001526,
		Longitude: -52.6749992371,
		Timezone:  "America/Campo_Grande",
	},
	"SJHA": {
		ICAO:      "SJHA",
		Name:      "Fazenda Janaina Airport",
		City:      "Amambai",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1296,
		Latitude:  -22.9830551147,
		Longitude: -55.0200004578,
		Timezone:  "America/Campo_Grande",
	},
	"SJHD": {
		ICAO:      "SJHD",
		Name:      "Fazenda Modelo Airport",
		City:      "Brasilandia",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1375,
		Latitude:  -21.3822212219,
		Longitude: -52.198890686,
		Timezone:  "America/Campo_Grande",
	},
	"SJHE": {
		ICAO:      "SJHE",
		Name:      "Fazenda Palmeira Airport",
		City:      "Ipiranga Do Norte",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1201,
		Latitude:  -12.0938892365,
		Longitude: -55.9977798462,
		Timezone:  "America/Cuiaba",
	},
	"SJHF": {
		ICAO:      "SJHF",
		Name:      "Bunge Airport",
		City:      "Urucui",
		State:     "Piaui",
		Country:   "BR",
		Elevation: 1837,
		Latitude:  -7.3722219467,
		Longitude: -44.3961105347,
		Timezone:  "America/Fortaleza",
	},
	"SJHG": {
		ICAO:      "SJHG",
		IATA:      "CFO",
		Name:      "Confresa Airport",
		City:      "Confresa",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 781,
		Latitude:  -10.6344003677,
		Longitude: -51.5635986328,
		Timezone:  "America/Cuiaba",
	},
	"SJHH": {
		ICAO:      "SJHH",
		Name:      "Fazenda Poruina Airport",
		City:      "Serranopolis",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2287,
		Latitude:  -18.1200008392,
		Longitude: -52.2177772522,
		Timezone:  "America/Sao_Paulo",
	},
	"SJHJ": {
		ICAO:      "SJHJ",
		Name:      "Fazenda Remanso Airport",
		City:      "Cacoal",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 831,
		Latitude:  -11.1822223663,
		Longitude: -61.0888900757,
		Timezone:  "America/Porto_Velho",
	},
	"SJHK": {
		ICAO:      "SJHK",
		Name:      "Musica Airport",
		City:      "Dom Pedrito",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 394,
		Latitude:  -30.9361114502,
		Longitude: -55.0808334351,
		Timezone:  "America/Sao_Paulo",
	},
	"SJHL": {
		ICAO:      "SJHL",
		Name:      "Frisama Eldorado Airport",
		City:      "Eldorado Do Carajas",
		State:     "Para",
		Country:   "BR",
		Elevation: 384,
		Latitude:  -6.0947217941,
		Longitude: -49.3613891602,
		Timezone:  "America/Belem",
	},
	"SJHM": {
		ICAO:      "SJHM",
		Name:      "Fazenda Globo Airport",
		City:      "Sapezal",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1588,
		Latitude:  -12.9536113739,
		Longitude: -58.5902786255,
		Timezone:  "America/Cuiaba",
	},
	"SJHN": {
		ICAO:      "SJHN",
		Name:      "Fazenda Belluno Airport",
		City:      "Alfredo Wagner",
		State:     "Santa-Catarina",
		Country:   "BR",
		Elevation: 3445,
		Latitude:  -27.8172225952,
		Longitude: -49.2338905334,
		Timezone:  "America/Sao_Paulo",
	},
	"SJHO": {
		ICAO:      "SJHO",
		Name:      "Fazenda Colorado Airport",
		City:      "Diamantino",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1699,
		Latitude:  -13.9513893127,
		Longitude: -57.105556488,
		Timezone:  "America/Cuiaba",
	},
	"SJHP": {
		ICAO:      "SJHP",
		Name:      "Monte Belo Airport",
		City:      "Ribas Do Rio Pardo",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1624,
		Latitude:  -20.2763881683,
		Longitude: -53.6616668701,
		Timezone:  "America/Campo_Grande",
	},
	"SJHQ": {
		ICAO:      "SJHQ",
		Name:      "Fazenda Sao Jose da Reunidas Airport",
		City:      "Sao Jose Do Xingu",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1155,
		Latitude:  -11.0336112976,
		Longitude: -52.3338890076,
		Timezone:  "America/Cuiaba",
	},
	"SJHS": {
		ICAO:      "SJHS",
		Name:      "Fazenda Sao Vicente Airport",
		City:      "Uberlandia",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2477,
		Latitude:  -19.3174991608,
		Longitude: -48.3486099243,
		Timezone:  "America/Sao_Paulo",
	},
	"SJHT": {
		ICAO:      "SJHT",
		Name:      "Fazenda ANB Airport",
		City:      "Corguinho",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1194,
		Latitude:  -19.8316669464,
		Longitude: -54.8294448853,
		Timezone:  "America/Campo_Grande",
	},
	"SJHU": {
		ICAO:      "SJHU",
		Name:      "Agropecuaria Vale do Guapore Airport",
		City:      "Vila Bela Da Santissima Trindade",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1149,
		Latitude:  -13.878610611,
		Longitude: -60.0963897705,
		Timezone:  "America/Cuiaba",
	},
	"SJHW": {
		ICAO:      "SJHW",
		Name:      "Professor Mauricio Joppert Airport",
		City:      "Engenheiro Navarro",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2221,
		Latitude:  -17.2936115265,
		Longitude: -43.9586105347,
		Timezone:  "America/Sao_Paulo",
	},
	"SJHX": {
		ICAO:      "SJHX",
		Name:      "Frisama Acailandia Airport",
		City:      "Acailandia",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 926,
		Latitude:  -4.9455561638,
		Longitude: -47.4647216797,
		Timezone:  "America/Fortaleza",
	},
	"SJHZ": {
		ICAO:      "SJHZ",
		Name:      "Fazenda Dois Netos Airport",
		City:      "Paranatinga",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1165,
		Latitude:  -13.2455558777,
		Longitude: -54.0338897705,
		Timezone:  "America/Cuiaba",
	},
	"SJIC": {
		ICAO:      "SJIC",
		Name:      "Pedro da Costa Lima Airport",
		City:      "Santa Rita Do Pardo",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1050,
		Latitude:  -21.2444438934,
		Longitude: -52.7861099243,
		Timezone:  "America/Campo_Grande",
	},
	"SJID": {
		ICAO:      "SJID",
		Name:      "Fazenda Goiaba Airport",
		City:      "Tres Lagoas",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1549,
		Latitude:  -19.9172210693,
		Longitude: -52.6313896179,
		Timezone:  "America/Campo_Grande",
	},
	"SJIE": {
		ICAO:      "SJIE",
		Name:      "Aero Agricola Sapezal Airport",
		City:      "Sapezal",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1739,
		Latitude:  -13.55027771,
		Longitude: -58.8600006104,
		Timezone:  "America/Cuiaba",
	},
	"SJIF": {
		ICAO:      "SJIF",
		Name:      "Fazenda Nova Era Airport",
		City:      "Iguatemi",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1247,
		Latitude:  -23.3941669464,
		Longitude: -54.7524986267,
		Timezone:  "America/Campo_Grande",
	},
	"SJII": {
		ICAO:      "SJII",
		Name:      "Fazenda Sao Marcos Airport",
		City:      "Dourados",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1395,
		Latitude:  -22.296667099,
		Longitude: -54.9413871765,
		Timezone:  "America/Campo_Grande",
	},
	"SJIJ": {
		ICAO:      "SJIJ",
		Name:      "Fazenda Amparo Airport",
		City:      "Alto Taquari",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2766,
		Latitude:  -17.7819442749,
		Longitude: -53.4611091614,
		Timezone:  "America/Cuiaba",
	},
	"SJIL": {
		ICAO:      "SJIL",
		Name:      "Fazenda Santa Barbara Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 427,
		Latitude:  -19.0986118317,
		Longitude: -55.8477783203,
		Timezone:  "America/Campo_Grande",
	},
	"SJIO": {
		ICAO:      "SJIO",
		Name:      "Guararapes Airport",
		City:      "Santa Cecilia",
		State:     "Santa-Catarina",
		Country:   "BR",
		Elevation: 3609,
		Latitude:  -26.9919433594,
		Longitude: -50.3830566406,
		Timezone:  "America/Sao_Paulo",
	},
	"SJIQ": {
		ICAO:      "SJIQ",
		Name:      "Usina Coruripe - Filial Limeira do Oeste Airport",
		City:      "Limeira Do Oeste",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1384,
		Latitude:  -19.5233325958,
		Longitude: -50.6555557251,
		Timezone:  "America/Sao_Paulo",
	},
	"SJIR": {
		ICAO:      "SJIR",
		Name:      "Engenheiro Joaquim Martins Airport",
		City:      "Iramaia",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1733,
		Latitude:  -13.3077783585,
		Longitude: -40.8827781677,
		Timezone:  "America/Bahia",
	},
	"SJIS": {
		ICAO:      "SJIS",
		Name:      "Fazenda Futuro Airport",
		City:      "Ribas Do Rio Pardo",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1598,
		Latitude:  -20.3466663361,
		Longitude: -53.456943512,
		Timezone:  "America/Campo_Grande",
	},
	"SJIT": {
		ICAO:      "SJIT",
		Name:      "Itau de Minas Airport",
		City:      "Itau De Minas",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2520,
		Latitude:  -20.7553005219,
		Longitude: -46.7501983643,
		Timezone:  "America/Sao_Paulo",
	},
	"SJIU": {
		ICAO:      "SJIU",
		Name:      "Fazenda Beija-Flor Airport",
		City:      "Esplanada",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 555,
		Latitude:  -11.7941665649,
		Longitude: -38.0908317566,
		Timezone:  "America/Bahia",
	},
	"SJIV": {
		ICAO:      "SJIV",
		Name:      "Sao Joao Airport",
		City:      "Macambara",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 269,
		Latitude:  -29.0858325958,
		Longitude: -56.2147216797,
		Timezone:  "America/Sao_Paulo",
	},
	"SJIY": {
		ICAO:      "SJIY",
		Name:      "Sao Leopoldo Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 814,
		Latitude:  -4.9791669846,
		Longitude: -56.9272232056,
		Timezone:  "America/Santarem",
	},
	"SJIZ": {
		ICAO:      "SJIZ",
		Name:      "Modulo Aguapei Airport",
		City:      "Porto Esperidiao",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 869,
		Latitude:  -15.868888855,
		Longitude: -59.2188873291,
		Timezone:  "America/Cuiaba",
	},
	"SJJA": {
		ICAO:      "SJJA",
		Name:      "Fazenda Jamanxin Airport",
		City:      "Novo Progresso",
		State:     "Para",
		Country:   "BR",
		Elevation: 800,
		Latitude:  -8.375,
		Longitude: -55.3061103821,
		Timezone:  "America/Santarem",
	},
	"SJJB": {
		ICAO:      "SJJB",
		Name:      "Fazenda Cacique Airport",
		City:      "Brasilandia",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1348,
		Latitude:  -20.7766666412,
		Longitude: -52.7772216797,
		Timezone:  "America/Campo_Grande",
	},
	"SJJF": {
		ICAO:      "SJJF",
		Name:      "Pousada do Rio Roosevelt Airport",
		City:      "Novo Aripuana",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 292,
		Latitude:  -8.4944438934,
		Longitude: -60.9625015259,
		Timezone:  "America/Manaus",
	},
	"SJJG": {
		ICAO:      "SJJG",
		Name:      "Fazenda Tanguru Airport",
		City:      "Querencia",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1050,
		Latitude:  -12.7216672897,
		Longitude: -52.3786125183,
		Timezone:  "America/Cuiaba",
	},
	"SJJH": {
		ICAO:      "SJJH",
		Name:      "Pousada Rio Xingu Airport",
		City:      "Vitoria Do Xingu",
		State:     "Para",
		Country:   "BR",
		Elevation: 233,
		Latitude:  -3.2494440079,
		Longitude: -51.7224998474,
		Timezone:  "America/Belem",
	},
	"SJJJ": {
		ICAO:      "SJJJ",
		Name:      "Fazenda Juliana Airport",
		City:      "Chupinguaia",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 1132,
		Latitude:  -12.6824998856,
		Longitude: -61.0888900757,
		Timezone:  "America/Porto_Velho",
	},
	"SJJK": {
		ICAO:      "SJJK",
		Name:      "Fazenda Taquaral Airport",
		City:      "Alta Floresta",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 980,
		Latitude:  -10.0288887024,
		Longitude: -56.7383003235,
		Timezone:  "America/Cuiaba",
	},
	"SJJL": {
		ICAO:      "SJJL",
		Name:      "Fazenda Campo Aberto Airport",
		City:      "Barreiras",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 2418,
		Latitude:  -11.7882995605,
		Longitude: -45.9646987915,
		Timezone:  "America/Bahia",
	},
	"SJJM": {
		ICAO:      "SJJM",
		Name:      "Michels Aviacao Agricola LTDA Airport",
		City:      "Sapezal",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1884,
		Latitude:  -13.5322217941,
		Longitude: -58.7652778625,
		Timezone:  "America/Cuiaba",
	},
	"SJJN": {
		ICAO:      "SJJN",
		Name:      "Fazenda Ponto de Apoio Airport",
		City:      "Agua Clara",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1320,
		Latitude:  -20.4663887024,
		Longitude: -52.6875,
		Timezone:  "America/Campo_Grande",
	},
	"SJJO": {
		ICAO:      "SJJO",
		Name:      "Joao de Oliveira Bueno Airport",
		City:      "Monte Alegre De Goias",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1431,
		Latitude:  -13.1925001144,
		Longitude: -47.1155548096,
		Timezone:  "America/Araguaina",
	},
	"SJJP": {
		ICAO:      "SJJP",
		Name:      "Sao Francisco Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 778,
		Latitude:  -7.8352780342,
		Longitude: -56.5383338928,
		Timezone:  "America/Santarem",
	},
	"SJJR": {
		ICAO:      "SJJR",
		Name:      "Fazenda Recreio Airport",
		City:      "Andrelandia",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3810,
		Latitude:  -21.7194442749,
		Longitude: -44.2094421387,
		Timezone:  "America/Sao_Paulo",
	},
	"SJJT": {
		ICAO:      "SJJT",
		Name:      "Fazenda Vale Verde Airport",
		City:      "Pium",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 581,
		Latitude:  -9.9611110687,
		Longitude: -49.7519454956,
		Timezone:  "America/Araguaina",
	},
	"SJJU": {
		ICAO:      "SJJU",
		Name:      "Fazenda Barra Mansa Airport",
		City:      "Nova Lacerda",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 725,
		Latitude:  -14.0494441986,
		Longitude: -60.1519432068,
		Timezone:  "America/Cuiaba",
	},
	"SJJV": {
		ICAO:      "SJJV",
		Name:      "Fazenda Pilar Airport",
		City:      "Aveiro",
		State:     "Para",
		Country:   "BR",
		Elevation: 499,
		Latitude:  -3.2933330536,
		Longitude: -56.1847229004,
		Timezone:  "America/Santarem",
	},
	"SJJW": {
		ICAO:      "SJJW",
		Name:      "Fazenda Santa Emilia Airport",
		City:      "Aquidauana",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 443,
		Latitude:  -19.5055561066,
		Longitude: -55.6130561829,
		Timezone:  "America/Campo_Grande",
	},
	"SJJX": {
		ICAO:      "SJJX",
		Name:      "Maca Aero Airport",
		City:      "Tailandia",
		State:     "Para",
		Country:   "BR",
		Elevation: 179,
		Latitude:  -2.8980560303,
		Longitude: -48.9586105347,
		Timezone:  "America/Belem",
	},
	"SJKA": {
		ICAO:      "SJKA",
		Name:      "Aerocampo Airport",
		City:      "Campo Novo Do Parecis",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1798,
		Latitude:  -13.6352777481,
		Longitude: -57.8994445801,
		Timezone:  "America/Cuiaba",
	},
	"SJKB": {
		ICAO:      "SJKB",
		Name:      "Cotriguacu Airport",
		City:      "Cotriguacu",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1506,
		Latitude:  -9.8994436264,
		Longitude: -58.581943512,
		Timezone:  "America/Cuiaba",
	},
	"SJKC": {
		ICAO:      "SJKC",
		Name:      "Fazenda Costa Norte II Airport",
		City:      "Apiacas",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 787,
		Latitude:  -9.2913894653,
		Longitude: -58.0330543518,
		Timezone:  "America/Cuiaba",
	},
	"SJKG": {
		ICAO:      "SJKG",
		Name:      "Fazenda Jatoba Airport",
		City:      "Uruara",
		State:     "Para",
		Country:   "BR",
		Elevation: 499,
		Latitude:  -3.5133330822,
		Longitude: -54.0955543518,
		Timezone:  "America/Santarem",
	},
	"SJKH": {
		ICAO:      "SJKH",
		Name:      "Boa Fe Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 722,
		Latitude:  -7.4280557632,
		Longitude: -56.3408317566,
		Timezone:  "America/Santarem",
	},
	"SJKI": {
		ICAO:      "SJKI",
		Name:      "Curiau Airport",
		City:      "Macapa",
		State:     "Amapa",
		Country:   "BR",
		Elevation: 66,
		Latitude:  0.2088889927,
		Longitude: -51.0092010498,
		Timezone:  "America/Belem",
	},
	"SJKJ": {
		ICAO:      "SJKJ",
		Name:      "Fazenda Cangaia Airport",
		City:      "Sonora",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2549,
		Latitude:  -17.4427776337,
		Longitude: -53.8088874817,
		Timezone:  "America/Campo_Grande",
	},
	"SJKL": {
		ICAO:      "SJKL",
		Name:      "Cana Airport",
		City:      "Normandia",
		State:     "Potaro-Siparuni",
		Country:   "BR",
		Elevation: 994,
		Latitude:  4.6188898087,
		Longitude: -60.0717010498,
		Timezone:  "America/Boa_Vista",
	},
	"SJKM": {
		ICAO:      "SJKM",
		Name:      "Canawapai Airport",
		City:      "Uiramuta",
		State:     "Potaro-Siparuni",
		Country:   "BR",
		Elevation: 1886,
		Latitude:  4.8080601692,
		Longitude: -60.0222015381,
		Timezone:  "America/Boa_Vista",
	},
	"SJKN": {
		ICAO:      "SJKN",
		Name:      "Caracana Airport",
		City:      "Uiramuta",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 1988,
		Latitude:  4.7047200203,
		Longitude: -60.2653007507,
		Timezone:  "America/Boa_Vista",
	},
	"SJKO": {
		ICAO:      "SJKO",
		Name:      "Cararuau Airport",
		City:      "Normandia",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 594,
		Latitude:  4.0630598068,
		Longitude: -60.1650009155,
		Timezone:  "America/Boa_Vista",
	},
	"SJKP": {
		ICAO:      "SJKP",
		Name:      "Caraparu Airport",
		City:      "Uiramuta",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 1194,
		Latitude:  4.5411100388,
		Longitude: -60.4631004333,
		Timezone:  "America/Boa_Vista",
	},
	"SJKQ": {
		ICAO:      "SJKQ",
		Name:      "Caramambatai (Mapae) Airport",
		City:      "Uiramuta",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 1989,
		Latitude:  5.1230602264,
		Longitude: -60.5858001709,
		Timezone:  "America/Boa_Vista",
	},
	"SJKR": {
		ICAO:      "SJKR",
		Name:      "Pista KRB Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 706,
		Latitude:  -6.0805559158,
		Longitude: -56.3891677856,
		Timezone:  "America/Santarem",
	},
	"SJKS": {
		ICAO:      "SJKS",
		Name:      "Catrimani I Airport",
		City:      "Mucajai",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 1092,
		Latitude:  2.3755600452,
		Longitude: -63.0083007813,
		Timezone:  "America/Boa_Vista",
	},
	"SJKT": {
		ICAO:      "SJKT",
		Name:      "Contao Airport",
		City:      "Normandia",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 577,
		Latitude:  4.17166996,
		Longitude: -60.543598175,
		Timezone:  "America/Boa_Vista",
	},
	"SJKU": {
		ICAO:      "SJKU",
		Name:      "Cumaipa Airport",
		City:      "Uiramuta",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 2188,
		Latitude:  4.8138899803,
		Longitude: -60.4735984802,
		Timezone:  "America/Boa_Vista",
	},
	"SJKV": {
		ICAO:      "SJKV",
		Name:      "Cumana I Airport",
		City:      "Pacaraima",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 1191,
		Latitude:  4.4702801704,
		Longitude: -60.7780990601,
		Timezone:  "America/Boa_Vista",
	},
	"SJKW": {
		ICAO:      "SJKW",
		Name:      "Cumana II Airport",
		City:      "Pacaraima",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 1191,
		Latitude:  4.4269399643,
		Longitude: -60.8238983154,
		Timezone:  "America/Boa_Vista",
	},
	"SJKX": {
		ICAO:      "SJKX",
		Name:      "Cutia Airport",
		City:      "Normandia",
		State:     "Upper-Takutu-Upper-Essequibo",
		Country:   "BR",
		Elevation: 496,
		Latitude:  4.1700000763,
		Longitude: -59.7486000061,
		Timezone:  "America/Boa_Vista",
	},
	"SJKZ": {
		ICAO:      "SJKZ",
		Name:      "Estevao Airport",
		City:      "Uiramuta",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 994,
		Latitude:  4.4497199059,
		Longitude: -60.439201355,
		Timezone:  "America/Boa_Vista",
	},
	"SJLA": {
		ICAO:      "SJLA",
		Name:      "Flechal Airport",
		City:      "Uiramuta",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 1988,
		Latitude:  4.6613898277,
		Longitude: -60.2919006348,
		Timezone:  "America/Boa_Vista",
	},
	"SJLB": {
		ICAO:      "SJLB",
		Name:      "Fazenda Chaparral Airport",
		City:      "Lambari D`Oeste",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 949,
		Latitude:  -15.5019435883,
		Longitude: -57.7672233582,
		Timezone:  "America/Cuiaba",
	},
	"SJLC": {
		ICAO:      "SJLC",
		Name:      "Hakoma Airport",
		City:      "Alto Alegre",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 1191,
		Latitude:  2.7200000286,
		Longitude: -63.5675010681,
		Timezone:  "America/Boa_Vista",
	},
	"SJLD": {
		ICAO:      "SJLD",
		Name:      "Fazenda Porto do Campo Airport",
		City:      "Lambari D`Oeste",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 594,
		Latitude:  -15.7155561447,
		Longitude: -57.7113876343,
		Timezone:  "America/Cuiaba",
	},
	"SJLE": {
		ICAO:      "SJLE",
		Name:      "Halikato-U Airport",
		City:      "Alto Alegre",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 994,
		Latitude:  3.2249999046,
		Longitude: -63.1994018555,
		Timezone:  "America/Boa_Vista",
	},
	"SJLF": {
		ICAO:      "SJLF",
		Name:      "Homoxi Airport",
		City:      "Iracema",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 2284,
		Latitude:  2.4977800846,
		Longitude: -63.7296981812,
		Timezone:  "America/Boa_Vista",
	},
	"SJLH": {
		ICAO:      "SJLH",
		Name:      "Jacamim Airport",
		City:      "Bonfim",
		State:     "Upper-Takutu-Upper-Essequibo",
		Country:   "BR",
		Elevation: 495,
		Latitude:  2.1761100292,
		Longitude: -59.7839012146,
		Timezone:  "America/Boa_Vista",
	},
	"SJLI": {
		ICAO:      "SJLI",
		Name:      "Fazenda Lilliani Airport",
		City:      "Bom Jardim",
		State:     "Para",
		Country:   "BR",
		Elevation: 958,
		Latitude:  -4.2136111259,
		Longitude: -46.9749984741,
		Timezone:  "America/Fortaleza",
	},
	"SJLJ": {
		ICAO:      "SJLJ",
		Name:      "Jatapuzinho Airport",
		City:      "Caroebe",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 397,
		Latitude:  0.5972219706,
		Longitude: -59.221698761,
		Timezone:  "America/Boa_Vista",
	},
	"SJLK": {
		ICAO:      "SJLK",
		Name:      "Lago Grande I Airport",
		City:      "Bonfim",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 298,
		Latitude:  3.4427800179,
		Longitude: -60.3525009155,
		Timezone:  "America/Boa_Vista",
	},
	"SJLL": {
		ICAO:      "SJLL",
		Name:      "Maturuca Airport",
		City:      "Normandia",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 1493,
		Latitude:  4.4608302116,
		Longitude: -60.1002998352,
		Timezone:  "America/Boa_Vista",
	},
	"SJLM": {
		ICAO:      "SJLM",
		Name:      "Leao de Ouro Airport",
		City:      "Pacaraima",
		Country:   "BR",
		Elevation: 298,
		Latitude:  4.1516699791,
		Longitude: -61.4277992249,
		Timezone:  "America/Boa_Vista",
	},
	"SJLN": {
		ICAO:      "SJLN",
		Name:      "Maloquinha Airport",
		City:      "Pacaraima",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 1890,
		Latitude:  4.726940155,
		Longitude: -60.5905990601,
		Timezone:  "America/Boa_Vista",
	},
	"SJLO": {
		ICAO:      "SJLO",
		Name:      "Manalai Airport",
		City:      "Uiramuta",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 2189,
		Latitude:  5.0841698647,
		Longitude: -60.3788986206,
		Timezone:  "America/Boa_Vista",
	},
	"SJLP": {
		ICAO:      "SJLP",
		Name:      "Manoa-Pium Airport",
		City:      "Bonfim",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 348,
		Latitude:  2.9800000191,
		Longitude: -60.0952987671,
		Timezone:  "America/Boa_Vista",
	},
	"SJLQ": {
		ICAO:      "SJLQ",
		Name:      "Maracana Airport",
		City:      "Normandia",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 1791,
		Latitude:  4.3649997711,
		Longitude: -60.0167007446,
		Timezone:  "America/Boa_Vista",
	},
	"SJLS": {
		ICAO:      "SJLS",
		Name:      "Marupa Airport",
		City:      "Bonfim",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 495,
		Latitude:  2.1252799034,
		Longitude: -59.8213996887,
		Timezone:  "America/Boa_Vista",
	},
	"SJLT": {
		ICAO:      "SJLT",
		Name:      "Milho Airport",
		City:      "Boa Vista",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 358,
		Latitude:  3.3855600357,
		Longitude: -60.4077987671,
		Timezone:  "America/Boa_Vista",
	},
	"SJLU": {
		ICAO:      "SJLU",
		Name:      "Missao Catrinami Airport",
		City:      "Caracarai",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 397,
		Latitude:  1.7408299446,
		Longitude: -62.2863998413,
		Timezone:  "America/Boa_Vista",
	},
	"SJLV": {
		ICAO:      "SJLV",
		Name:      "Morro Airport",
		City:      "Uiramuta",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 1792,
		Latitude:  4.3572201729,
		Longitude: -59.9700012207,
		Timezone:  "America/Boa_Vista",
	},
	"SJLW": {
		ICAO:      "SJLW",
		Name:      "Mudubim I Airport",
		City:      "Pacaraima",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 1493,
		Latitude:  4.42166996,
		Longitude: -60.4897003174,
		Timezone:  "America/Boa_Vista",
	},
	"SJLX": {
		ICAO:      "SJLX",
		Name:      "Mutum Airport",
		City:      "Uiramuta",
		Country:   "BR",
		Elevation: 1792,
		Latitude:  4.4530601501,
		Longitude: -59.8539009094,
		Timezone:  "America/Boa_Vista",
	},
	"SJLZ": {
		ICAO:      "SJLZ",
		Name:      "Orinduque Airport",
		City:      "Uiramuta",
		State:     "Potaro-Siparuni",
		Country:   "BR",
		Elevation: 1896,
		Latitude:  4.7344398499,
		Longitude: -60.0346984863,
		Timezone:  "America/Boa_Vista",
	},
	"SJMA": {
		ICAO:      "SJMA",
		Name:      "Fazenda Mundo Acabado Airport",
		City:      "Sonora",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 525,
		Latitude:  -17.9355564117,
		Longitude: -54.926109314,
		Timezone:  "America/Campo_Grande",
	},
	"SJMD": {
		ICAO:      "SJMD",
		Name:      "Paapiu Novo Airport",
		City:      "Iracema",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 696,
		Latitude:  2.7227799892,
		Longitude: -62.9502983093,
		Timezone:  "America/Boa_Vista",
	},
	"SJME": {
		ICAO:      "SJME",
		Name:      "Mato Grosso Airport",
		City:      "Pacaraima",
		State:     "Bolivar",
		Country:   "BR",
		Elevation: 2783,
		Latitude:  4.5705599785,
		Longitude: -60.9030990601,
		Timezone:  "America/Boa_Vista",
	},
	"SJMF": {
		ICAO:      "SJMF",
		Name:      "Pacu Airport",
		City:      "Normandia",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 994,
		Latitude:  4.1638898849,
		Longitude: -60.2221984863,
		Timezone:  "America/Boa_Vista",
	},
	"SJMH": {
		ICAO:      "SJMH",
		Name:      "Palimiu Airport",
		City:      "Alto Alegre",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 896,
		Latitude:  3.3180599213,
		Longitude: -62.9710998535,
		Timezone:  "America/Boa_Vista",
	},
	"SJMI": {
		ICAO:      "SJMI",
		Name:      "Parafuri Airport",
		City:      "Alto Alegre",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 1988,
		Latitude:  3.2880599499,
		Longitude: -63.8431015015,
		Timezone:  "America/Boa_Vista",
	},
	"SJMJ": {
		ICAO:      "SJMJ",
		Name:      "Pedra Branca Airport",
		City:      "Uiramuta",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 1489,
		Latitude:  4.4572200775,
		Longitude: -60.2677993774,
		Timezone:  "America/Boa_Vista",
	},
	"SJMK": {
		ICAO:      "SJMK",
		Name:      "Pedra Preta Airport",
		City:      "Uiramuta",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 2188,
		Latitude:  4.7130599022,
		Longitude: -60.476398468,
		Timezone:  "America/Boa_Vista",
	},
	"SJML": {
		ICAO:      "SJML",
		Name:      "Piolho Airport",
		City:      "Pacaraima",
		State:     "Bolivar",
		Country:   "BR",
		Elevation: 2287,
		Latitude:  4.7013897896,
		Longitude: -60.7336006165,
		Timezone:  "America/Boa_Vista",
	},
	"SJMM": {
		ICAO:      "SJMM",
		Name:      "Pipi Airport",
		City:      "Uiramuta",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 2585,
		Latitude:  4.9299998283,
		Longitude: -60.3218994141,
		Timezone:  "America/Boa_Vista",
	},
	"SJMN": {
		ICAO:      "SJMN",
		Name:      "Raposa Airport",
		City:      "Normandia",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 495,
		Latitude:  3.8099999428,
		Longitude: -60.0903015137,
		Timezone:  "America/Boa_Vista",
	},
	"SJMO": {
		ICAO:      "SJMO",
		Name:      "Santa Isabel Airport",
		City:      "Pacaraima",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 1194,
		Latitude:  4.4699997902,
		Longitude: -60.8624992371,
		Timezone:  "America/Boa_Vista",
	},
	"SJMP": {
		ICAO:      "SJMP",
		Name:      "Santa Liberdade Airport",
		City:      "Pacaraima",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 1890,
		Latitude:  4.2874999046,
		Longitude: -60.2569007874,
		Timezone:  "America/Boa_Vista",
	},
	"SJMQ": {
		ICAO:      "SJMQ",
		Name:      "Santa Maria de Normandia Airport",
		City:      "Normandia",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 847,
		Latitude:  4.1702799797,
		Longitude: -60.1377983093,
		Timezone:  "America/Boa_Vista",
	},
	"SJMR": {
		ICAO:      "SJMR",
		Name:      "Maragogipe Airport",
		City:      "Itaquirai",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 998,
		Latitude:  -23.4611110687,
		Longitude: -54.2988891602,
		Timezone:  "America/Campo_Grande",
	},
	"SJMT": {
		ICAO:      "SJMT",
		Name:      "Sao Miguel Cachoeirinha Airport",
		City:      "Pacaraima",
		State:     "Bolivar",
		Country:   "BR",
		Elevation: 1194,
		Latitude:  4.4902801514,
		Longitude: -60.9906005859,
		Timezone:  "America/Boa_Vista",
	},
	"SJMU": {
		ICAO:      "SJMU",
		Name:      "Sauparu Airport",
		City:      "Uiramuta",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 2287,
		Latitude:  4.9897198677,
		Longitude: -60.4356002808,
		Timezone:  "America/Boa_Vista",
	},
	"SJMV": {
		ICAO:      "SJMV",
		Name:      "Serra do Sol Airport",
		City:      "Uiramuta",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 2480,
		Latitude:  4.9425001144,
		Longitude: -60.4752998352,
		Timezone:  "America/Boa_Vista",
	},
	"SJMW": {
		ICAO:      "SJMW",
		Name:      "Soco Airport",
		City:      "Uiramuta",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 1690,
		Latitude:  4.4699997902,
		Longitude: -60.1768989563,
		Timezone:  "America/Boa_Vista",
	},
	"SJMX": {
		ICAO:      "SJMX",
		Name:      "Suapi Airport",
		City:      "Pacaraima",
		State:     "Bolivar",
		Country:   "BR",
		Elevation: 2487,
		Latitude:  4.5752801895,
		Longitude: -60.8582992554,
		Timezone:  "America/Boa_Vista",
	},
	"SJMZ": {
		ICAO:      "SJMZ",
		Name:      "Fazenda Mizote Airport",
		City:      "Sao Desiderio",
		Country:   "BR",
		Elevation: 2701,
		Latitude:  -12.9575004578,
		Longitude: -45.9847221375,
		Timezone:  "America/Bahia",
	},
	"SJNA": {
		ICAO:      "SJNA",
		Name:      "Clube de Esportes Aereos e Nauticos do Para Airport",
		City:      "Belem",
		State:     "Para",
		Country:   "BR",
		Elevation: 17,
		Latitude:  -1.2672220469,
		Longitude: -48.4300003052,
		Timezone:  "America/Belem",
	},
	"SJNB": {
		ICAO:      "SJNB",
		Name:      "Ubaru Airport",
		City:      "Pacaraima",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 1194,
		Latitude:  4.5016698837,
		Longitude: -60.8022003174,
		Timezone:  "America/Boa_Vista",
	},
	"SJNC": {
		ICAO:      "SJNC",
		Name:      "Uraricoera Airport",
		City:      "Alto Alegre",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 794,
		Latitude:  3.1461100578,
		Longitude: -62.2303009033,
		Timezone:  "America/Boa_Vista",
	},
	"SJND": {
		ICAO:      "SJND",
		Name:      "Nova Descoberta Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 821,
		Latitude:  -6.618888855,
		Longitude: -56.6925010681,
		Timezone:  "America/Santarem",
	},
	"SJNE": {
		ICAO:      "SJNE",
		Name:      "Wapum Airport",
		City:      "Bonfim",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 495,
		Latitude:  1.9900000095,
		Longitude: -59.9571990967,
		Timezone:  "America/Boa_Vista",
	},
	"SJNF": {
		ICAO:      "SJNF",
		Name:      "Wilimon Airport",
		City:      "Uiramuta",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 1739,
		Latitude:  4.6350002289,
		Longitude: -60.1761016846,
		Timezone:  "America/Boa_Vista",
	},
	"SJNG": {
		ICAO:      "SJNG",
		Name:      "Xidea Airport",
		City:      "Alto Alegre",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 1988,
		Latitude:  2.6069400311,
		Longitude: -63.8694000244,
		Timezone:  "America/Boa_Vista",
	},
	"SJNH": {
		ICAO:      "SJNH",
		Name:      "Fazenda Nhu-Vera Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 492,
		Latitude:  -18.0272216797,
		Longitude: -55.4602775574,
		Timezone:  "America/Campo_Grande",
	},
	"SJNI": {
		ICAO:      "SJNI",
		Name:      "Fazenda Mutum Airport",
		City:      "Nova Mutum",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1575,
		Latitude:  -13.9197216034,
		Longitude: -56.0844421387,
		Timezone:  "America/Cuiaba",
	},
	"SJNJ": {
		ICAO:      "SJNJ",
		Name:      "Fazenda Lakanka Airport",
		City:      "Jussara",
		State:     "Goias",
		Country:   "BR",
		Elevation: 892,
		Latitude:  -15.4324998856,
		Longitude: -51.5905570984,
		Timezone:  "America/Sao_Paulo",
	},
	"SJNK": {
		ICAO:      "SJNK",
		Name:      "Fazenda Tres Perobas Airport",
		City:      "Santa Luzia D`Oeste",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 1119,
		Latitude:  -12.0816669464,
		Longitude: -61.6399993896,
		Timezone:  "America/Porto_Velho",
	},
	"SJNL": {
		ICAO:      "SJNL",
		Name:      "Fazenda Reata Airport",
		City:      "Campo Novo Do Parecis",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1791,
		Latitude:  -13.6202783585,
		Longitude: -57.5808334351,
		Timezone:  "America/Cuiaba",
	},
	"SJNM": {
		ICAO:      "SJNM",
		Name:      "Fazenda Santa Luzia Airport",
		City:      "Nova Mutum",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1520,
		Latitude:  -13.9997215271,
		Longitude: -55.9452781677,
		Timezone:  "America/Cuiaba",
	},
	"SJNN": {
		ICAO:      "SJNN",
		Name:      "Fazenda Botelho Airport",
		City:      "Nova Mamore",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 535,
		Latitude:  -10.2222223282,
		Longitude: -64.3433303833,
		Timezone:  "America/Porto_Velho",
	},
	"SJNO": {
		ICAO:      "SJNO",
		Name:      "Fazenda Nazare Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 348,
		Latitude:  -18.6736106873,
		Longitude: -57.1233329773,
		Timezone:  "America/Campo_Grande",
	},
	"SJNP": {
		ICAO:      "SJNP",
		IATA:      "NPR",
		Name:      "Novo Progresso Airport",
		City:      "Novo Progresso",
		State:     "Para",
		Country:   "BR",
		Elevation: 794,
		Latitude:  -7.1258330345,
		Longitude: -55.4008331299,
		Timezone:  "America/Santarem",
	},
	"SJNQ": {
		ICAO:      "SJNQ",
		Name:      "Destilaria Medasa Airport",
		City:      "Medeiros Neto",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 656,
		Latitude:  -17.4850006104,
		Longitude: -40.1577796936,
		Timezone:  "America/Bahia",
	},
	"SJNR": {
		ICAO:      "SJNR",
		Name:      "Norte Jet Airport",
		City:      "Benevides",
		State:     "Para",
		Country:   "BR",
		Elevation: 88,
		Latitude:  -1.3072220087,
		Longitude: -48.2574996948,
		Timezone:  "America/Belem",
	},
	"SJNS": {
		ICAO:      "SJNS",
		Name:      "Ilha dos Macacos Airport",
		City:      "Paranaita",
		Country:   "BR",
		Elevation: 1266,
		Latitude:  -9.2255563736,
		Longitude: -57.0224990845,
		Timezone:  "America/Cuiaba",
	},
	"SJNU": {
		ICAO:      "SJNU",
		Name:      "Fazenda Campo Alto Airport",
		City:      "Ribeirao Cascalheira",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1099,
		Latitude:  -13.0372219086,
		Longitude: -51.7138900757,
		Timezone:  "America/Cuiaba",
	},
	"SJNV": {
		ICAO:      "SJNV",
		Name:      "Fazenda Sao Francisco Airport",
		City:      "Gouvelandia",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1857,
		Latitude:  -18.431388855,
		Longitude: -50.256111145,
		Timezone:  "America/Sao_Paulo",
	},
	"SJNW": {
		ICAO:      "SJNW",
		Name:      "Fazenda Nhandu Airport",
		City:      "Novo Mundo",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1103,
		Latitude:  -9.6338891983,
		Longitude: -55.2269439697,
		Timezone:  "America/Cuiaba",
	},
	"SJNX": {
		ICAO:      "SJNX",
		Name:      "Cosmos Aviacao Agricola Airport",
		City:      "Chapadao Do Ceu",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2730,
		Latitude:  -18.4080562592,
		Longitude: -52.630279541,
		Timezone:  "America/Sao_Paulo",
	},
	"SJNZ": {
		ICAO:      "SJNZ",
		Name:      "Fazenda Tres Lagoas Airport",
		City:      "Sapezal",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1746,
		Latitude:  -13.2527780533,
		Longitude: -58.7291679382,
		Timezone:  "America/Cuiaba",
	},
	"SJOG": {
		ICAO:      "SJOG",
		Name:      "Ariquemes Airport",
		City:      "Ariquemes",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 530,
		Latitude:  -9.884721756,
		Longitude: -63.0488891602,
		Timezone:  "America/Porto_Velho",
	},
	"SJOH": {
		ICAO:      "SJOH",
		Name:      "Juruti Airport",
		City:      "Juruti",
		State:     "Para",
		Country:   "BR",
		Elevation: 108,
		Latitude:  -2.1866669655,
		Longitude: -56.0902786255,
		Timezone:  "America/Santarem",
	},
	"SJOI": {
		ICAO:      "SJOI",
		Name:      "Fazenda Jaguatirica Airport",
		City:      "Porto Murtinho",
		State:     "Alto-Paraguay",
		Country:   "BR",
		Elevation: 289,
		Latitude:  -22.0322227478,
		Longitude: -57.9819450378,
		Timezone:  "America/Asuncion",
	},
	"SJOJ": {
		ICAO:      "SJOJ",
		Name:      "Simasa Airport",
		City:      "Acailandia",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 656,
		Latitude:  -4.869166851,
		Longitude: -47.3891677856,
		Timezone:  "America/Fortaleza",
	},
	"SJOK": {
		ICAO:      "SJOK",
		Name:      "Fazenda Modelo Airport",
		City:      "Agua Clara",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 2041,
		Latitude:  -19.3155555725,
		Longitude: -52.9766654968,
		Timezone:  "America/Campo_Grande",
	},
	"SJOM": {
		ICAO:      "SJOM",
		Name:      "Fazenda Karla Renata Airport",
		City:      "Castanheira",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 991,
		Latitude:  -10.6324996948,
		Longitude: -58.4449996948,
		Timezone:  "America/Cuiaba",
	},
	"SJON": {
		ICAO:      "SJON",
		Name:      "Fazenda Sao Jose do Generoso Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 328,
		Latitude:  -18.8724994659,
		Longitude: -56.7891654968,
		Timezone:  "America/Campo_Grande",
	},
	"SJOO": {
		ICAO:      "SJOO",
		Name:      "Doutor Joao Osmar de Oliveira Airport",
		City:      "Comodoro",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 864,
		Latitude:  -14.1341667175,
		Longitude: -59.8197212219,
		Timezone:  "America/Cuiaba",
	},
	"SJOR": {
		ICAO:      "SJOR",
		Name:      "Fazenda Parabufalos Airport",
		City:      "Macapa",
		State:     "Amapa",
		Country:   "BR",
		Elevation: 30,
		Latitude:  1.0194400549,
		Longitude: -50.2139015198,
		Timezone:  "America/Belem",
	},
	"SJOU": {
		ICAO:      "SJOU",
		Name:      "Marfrig Frigorifico Airport",
		City:      "Paranatinga",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1628,
		Latitude:  -14.453332901,
		Longitude: -54.0336112976,
		Timezone:  "America/Cuiaba",
	},
	"SJOW": {
		ICAO:      "SJOW",
		Name:      "Fazenda Guadalupe Airport",
		City:      "Santo Antonio Do Aracangua",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1188,
		Latitude:  -20.9816665649,
		Longitude: -50.5269432068,
		Timezone:  "America/Sao_Paulo",
	},
	"SJOX": {
		ICAO:      "SJOX",
		Name:      "Fazenda Brinquinho Airport",
		City:      "Sao Desiderio",
		Country:   "BR",
		Elevation: 3145,
		Latitude:  -12.8249998093,
		Longitude: -46.2449989319,
		Timezone:  "America/Araguaina",
	},
	"SJOY": {
		ICAO:      "SJOY",
		Name:      "Fazenda Sao Miguel Airport",
		City:      "Sao Desiderio",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 2999,
		Latitude:  -12.6072216034,
		Longitude: -46.2394447327,
		Timezone:  "America/Araguaina",
	},
	"SJPA": {
		ICAO:      "SJPA",
		Name:      "EMAL - Pimenta Bueno Airport",
		City:      "Pimenta Bueno",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 1001,
		Latitude:  -11.6861114502,
		Longitude: -60.6297225952,
		Timezone:  "America/Porto_Velho",
	},
	"SJPG": {
		ICAO:      "SJPG",
		Name:      "Pista Branca de Neve Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 545,
		Latitude:  -6.125,
		Longitude: -56.3988876343,
		Timezone:  "America/Santarem",
	},
	"SJPH": {
		ICAO:      "SJPH",
		Name:      "Fazenda Palma Airport",
		City:      "Luziania",
		State:     "Goias",
		Country:   "BR",
		Elevation: 3297,
		Latitude:  -16.1894435883,
		Longitude: -48.0780563354,
		Timezone:  "America/Sao_Paulo",
	},
	"SJPI": {
		ICAO:      "SJPI",
		Name:      "Fazenda Pirapitinga Airport",
		City:      "Cocalzinho De Goias",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2510,
		Latitude:  -15.4127779007,
		Longitude: -48.6341667175,
		Timezone:  "America/Sao_Paulo",
	},
	"SJPK": {
		ICAO:      "SJPK",
		Name:      "Marcos da Cunha Airport",
		City:      "Luis Eduardo Magalhaes",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 2815,
		Latitude:  -12.3613891602,
		Longitude: -46.1341667175,
		Timezone:  "America/Bahia",
	},
	"SJPL": {
		ICAO:      "SJPL",
		Name:      "Fazenda Australia Airport",
		City:      "Deodapolis",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1116,
		Latitude:  -21.9741668701,
		Longitude: -54.1477775574,
		Timezone:  "America/Campo_Grande",
	},
	"SJPO": {
		ICAO:      "SJPO",
		Name:      "Xingo Airport",
		City:      "Piranhas",
		State:     "Alagoas",
		Country:   "BR",
		Elevation: 791,
		Latitude:  -9.5877780914,
		Longitude: -37.7783317566,
		Timezone:  "America/Maceio",
	},
	"SJPP": {
		ICAO:      "SJPP",
		Name:      "Aero Rural Airport",
		City:      "Alta Floresta",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 879,
		Latitude:  -9.8569440842,
		Longitude: -56.0044441223,
		Timezone:  "America/Cuiaba",
	},
	"SJPQ": {
		ICAO:      "SJPQ",
		Name:      "Avstal Airport",
		City:      "Unai",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3205,
		Latitude:  -16.3197212219,
		Longitude: -47.225276947,
		Timezone:  "America/Sao_Paulo",
	},
	"SJPS": {
		ICAO:      "SJPS",
		Name:      "Fazenda Monte Belo Airport",
		City:      "Barra Do Bugres",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 636,
		Latitude:  -14.9799995422,
		Longitude: -57.9916687012,
		Timezone:  "America/Cuiaba",
	},
	"SJPU": {
		ICAO:      "SJPU",
		Name:      "Pau D`Arco Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 866,
		Latitude:  -7.1199998856,
		Longitude: -56.8144454956,
		Timezone:  "America/Santarem",
	},
	"SJPV": {
		ICAO:      "SJPV",
		Name:      "Fazenda Guimaraes Airport",
		City:      "Lucas Do Rio Verde",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1378,
		Latitude:  -12.9763889313,
		Longitude: -56.1836128235,
		Timezone:  "America/Cuiaba",
	},
	"SJPW": {
		ICAO:      "SJPW",
		Name:      "Fazenda Guimaraes II Airport",
		City:      "Lucas Do Rio Verde",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1312,
		Latitude:  -12.9019441605,
		Longitude: -56.3719444275,
		Timezone:  "America/Cuiaba",
	},
	"SJPX": {
		ICAO:      "SJPX",
		Name:      "Fazenda Cidade Verde Airport",
		City:      "Poxoreo",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2028,
		Latitude:  -15.4152784348,
		Longitude: -54.3722229004,
		Timezone:  "America/Cuiaba",
	},
	"SJQA": {
		ICAO:      "SJQA",
		Name:      "Fazenda Clarim Airport",
		City:      "Sao Desiderio",
		State:     "Goias",
		Country:   "BR",
		Elevation: 3121,
		Latitude:  -13.2122220993,
		Longitude: -46.2388877869,
		Timezone:  "America/Bahia",
	},
	"SJQB": {
		ICAO:      "SJQB",
		Name:      "Fazenda Symalu Airport",
		City:      "Cumaru Do Norte",
		State:     "Para",
		Country:   "BR",
		Elevation: 601,
		Latitude:  -8.3275003433,
		Longitude: -51.1049995422,
		Timezone:  "America/Belem",
	},
	"SJQD": {
		ICAO:      "SJQD",
		Name:      "Fazenda Sao Roque do Vale do Rio Negro Airport",
		City:      "Aquidauana",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 296,
		Latitude:  -19.3691673279,
		Longitude: -56.3488883972,
		Timezone:  "America/Campo_Grande",
	},
	"SJQE": {
		ICAO:      "SJQE",
		Name:      "Bonacin Airport",
		City:      "Piraquara",
		State:     "Parana",
		Country:   "BR",
		Elevation: 2999,
		Latitude:  -25.4077134595,
		Longitude: -49.0504789352,
		Timezone:  "America/Sao_Paulo",
	},
	"SJQF": {
		ICAO:      "SJQF",
		Name:      "Fazenda Londrina Airport",
		City:      "Campo Novo Do Parecis",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1690,
		Latitude:  -13.3555555344,
		Longitude: -57.7550010681,
		Timezone:  "America/Cuiaba",
	},
	"SJQH": {
		ICAO:      "SJQH",
		Name:      "Fazenda Tempero Seco Airport",
		City:      "Labrea",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 375,
		Latitude:  -9.3963890076,
		Longitude: -66.679725647,
		Timezone:  "America/Manaus",
	},
	"SJQI": {
		ICAO:      "SJQI",
		Name:      "Porto Jofre Airport",
		City:      "Pocone",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 361,
		Latitude:  -17.3613891602,
		Longitude: -56.7791671753,
		Timezone:  "America/Campo_Grande",
	},
	"SJQK": {
		ICAO:      "SJQK",
		Name:      "Barra do Vento Airport",
		City:      "Boa Vista",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 279,
		Latitude:  2.6955599785,
		Longitude: -60.814201355,
		Timezone:  "America/Boa_Vista",
	},
	"SJQN": {
		ICAO:      "SJQN",
		Name:      "Quirinopolis Airport",
		City:      "Quirinopolis",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1922,
		Latitude:  -18.446100235,
		Longitude: -50.409198761,
		Timezone:  "America/Sao_Paulo",
	},
	"SJQQ": {
		ICAO:      "SJQQ",
		Name:      "Fazenda Terra Alta Airport",
		City:      "Portel",
		State:     "Para",
		Country:   "BR",
		Elevation: 66,
		Latitude:  -2.8650000095,
		Longitude: -50.9927787781,
		Timezone:  "America/Belem",
	},
	"SJQR": {
		ICAO:      "SJQR",
		Name:      "Jacarezinho Airport",
		City:      "Valparaiso",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1312,
		Latitude:  -21.1480560303,
		Longitude: -50.9011116028,
		Timezone:  "America/Sao_Paulo",
	},
	"SJQS": {
		ICAO:      "SJQS",
		Name:      "Paredao da Serra Airport",
		City:      "Chapada Dos Guimaraes",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2117,
		Latitude:  -15.4719438553,
		Longitude: -55.7200012207,
		Timezone:  "America/Cuiaba",
	},
	"SJQT": {
		ICAO:      "SJQT",
		Name:      "Fazenda Haras RPC Airport",
		City:      "Nova Lima",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 4209,
		Latitude:  -20.1686115265,
		Longitude: -43.9269447327,
		Timezone:  "America/Sao_Paulo",
	},
	"SJQV": {
		ICAO:      "SJQV",
		Name:      "Usina Santa Luiza Airport",
		City:      "Motuca",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1978,
		Latitude:  -21.5091667175,
		Longitude: -48.1972236633,
		Timezone:  "America/Sao_Paulo",
	},
	"SJQX": {
		ICAO:      "SJQX",
		Name:      "Fazenda Uruara Airport",
		City:      "Uruara",
		State:     "Para",
		Country:   "BR",
		Elevation: 797,
		Latitude:  -3.5247220993,
		Longitude: -53.3861122131,
		Timezone:  "America/Santarem",
	},
	"SJQZ": {
		ICAO:      "SJQZ",
		Name:      "Dona Iracema Airport",
		City:      "Porto Nacional",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 919,
		Latitude:  -10.6063890457,
		Longitude: -48.3486099243,
		Timezone:  "America/Araguaina",
	},
	"SJRA": {
		ICAO:      "SJRA",
		Name:      "Pesqueiro Xingu Airport",
		City:      "Sao Felix Do Xingu",
		State:     "Para",
		Country:   "BR",
		Elevation: 525,
		Latitude:  -5.7750000954,
		Longitude: -52.6222229004,
		Timezone:  "America/Belem",
	},
	"SJRB": {
		ICAO:      "SJRB",
		Name:      "Fazenda VR Airport",
		City:      "Santa Cruz Do Xingu",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1191,
		Latitude:  -10.1130561829,
		Longitude: -52.5050010681,
		Timezone:  "America/Cuiaba",
	},
	"SJRC": {
		ICAO:      "SJRC",
		Name:      "Serra da Borda Airport",
		City:      "Vila Bela Da Santissima Trindade",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1575,
		Latitude:  -14.8272218704,
		Longitude: -59.6880569458,
		Timezone:  "America/Cuiaba",
	},
	"SJRG": {
		ICAO:      "SJRG",
		IATA:      "RIG",
		Name:      "Rio Grande Airport",
		City:      "Rio Grande",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 27,
		Latitude:  -32.081699,
		Longitude: -52.163299,
		Timezone:  "America/Sao_Paulo",
	},
	"SJRH": {
		ICAO:      "SJRH",
		Name:      "Fazenda Galheiro Airport",
		City:      "Campo Verde",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1788,
		Latitude:  -15.3080558777,
		Longitude: -54.5997238159,
		Timezone:  "America/Cuiaba",
	},
	"SJRI": {
		ICAO:      "SJRI",
		Name:      "Condominio Boa Esperanca Airport",
		City:      "Baixa Grande Do Ribeiro",
		State:     "Piaui",
		Country:   "BR",
		Elevation: 1825,
		Latitude:  -8.4063892365,
		Longitude: -45.4413871765,
		Timezone:  "America/Fortaleza",
	},
	"SJRJ": {
		ICAO:      "SJRJ",
		Name:      "Agromineral Mutum Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 752,
		Latitude:  -7.6402778625,
		Longitude: -56.787776947,
		Timezone:  "America/Santarem",
	},
	"SJRK": {
		ICAO:      "SJRK",
		Name:      "Agropecuaria Sao Roberto Airport",
		City:      "Santana Do Araguaia",
		State:     "Para",
		Country:   "BR",
		Elevation: 860,
		Latitude:  -9.2322216034,
		Longitude: -50.9302787781,
		Timezone:  "America/Belem",
	},
	"SJRL": {
		ICAO:      "SJRL",
		Name:      "Fazenda do Acucar Airport",
		City:      "Nobres",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1001,
		Latitude:  -14.4586105347,
		Longitude: -56.0527763367,
		Timezone:  "America/Cuiaba",
	},
	"SJRO": {
		ICAO:      "SJRO",
		Name:      "Fazenda Santa Elisa Airport",
		City:      "Auriflama",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1312,
		Latitude:  -20.5305557251,
		Longitude: -50.6349983215,
		Timezone:  "America/Sao_Paulo",
	},
	"SJRP": {
		ICAO:      "SJRP",
		Name:      "Fazenda Santangelo Agropecuaria Airport",
		City:      "Miranda",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 450,
		Latitude:  -20.1324996948,
		Longitude: -56.3711128235,
		Timezone:  "America/Campo_Grande",
	},
	"SJRR": {
		ICAO:      "SJRR",
		Name:      "Fazenda Barreiro Airport",
		City:      "Parauna",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2461,
		Latitude:  -17.0541667938,
		Longitude: -50.9283332825,
		Timezone:  "America/Sao_Paulo",
	},
	"SJRS": {
		ICAO:      "SJRS",
		Name:      "Fazenda Guanabara Airport",
		City:      "Junqueiropolis",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1017,
		Latitude:  -21.3436107635,
		Longitude: -51.4033317566,
		Timezone:  "America/Sao_Paulo",
	},
	"SJRT": {
		ICAO:      "SJRT",
		Name:      "Fazenda Arrendamento Airport",
		City:      "Paranatinga",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1499,
		Latitude:  -14.3694438934,
		Longitude: -54.0024986267,
		Timezone:  "America/Cuiaba",
	},
	"SJRV": {
		ICAO:      "SJRV",
		Name:      "PCH Garganta da Jararaca Airport",
		City:      "Nova Maringa",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1394,
		Latitude:  -13.3861112595,
		Longitude: -57.6155548096,
		Timezone:  "America/Cuiaba",
	},
	"SJRW": {
		ICAO:      "SJRW",
		Name:      "PCH Buriti Airport",
		City:      "Agua Clara",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1581,
		Latitude:  -19.3411102295,
		Longitude: -52.6894454956,
		Timezone:  "America/Campo_Grande",
	},
	"SJSA": {
		ICAO:      "SJSA",
		Name:      "Fazenda Aimores Airport",
		City:      "Juti",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1263,
		Latitude:  -22.7897224426,
		Longitude: -54.5977783203,
		Timezone:  "America/Campo_Grande",
	},
	"SJSC": {
		ICAO:      "SJSC",
		Name:      "Fazenda Sao Sebastiao do Castelo Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 358,
		Latitude:  -18.6611118317,
		Longitude: -57.6238899231,
		Timezone:  "America/Campo_Grande",
	},
	"SJSD": {
		ICAO:      "SJSD",
		Name:      "Fazenda Sao Bento Airport",
		City:      "Sidrolandia",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1772,
		Latitude:  -21.0727787018,
		Longitude: -55.1283340454,
		Timezone:  "America/Campo_Grande",
	},
	"SJSE": {
		ICAO:      "SJSE",
		Name:      "Serabi Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 1017,
		Latitude:  -6.3197221756,
		Longitude: -55.7949981689,
		Timezone:  "America/Santarem",
	},
	"SJSF": {
		ICAO:      "SJSF",
		Name:      "Agropecuaria Barrinha Airport",
		City:      "Sao Francisco Sales",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1310,
		Latitude:  -19.8413887024,
		Longitude: -49.688331604,
		Timezone:  "America/Sao_Paulo",
	},
	"SJSH": {
		ICAO:      "SJSH",
		Name:      "Sonhar Airport",
		City:      "Governador Celso Ramos",
		State:     "Santa-Catarina",
		Country:   "BR",
		Elevation: 33,
		Latitude:  -27.3705558777,
		Longitude: -48.6122207642,
		Timezone:  "America/Sao_Paulo",
	},
	"SJSJ": {
		ICAO:      "SJSJ",
		Name:      "Fazenda Sao Jose do Cangalha Airport",
		City:      "Agua Clara",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1201,
		Latitude:  -19.5155563354,
		Longitude: -52.6047210693,
		Timezone:  "America/Campo_Grande",
	},
	"SJSK": {
		ICAO:      "SJSK",
		Name:      "Mario Spada Airport",
		City:      "Minas Novas",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3271,
		Latitude:  -17.5283336639,
		Longitude: -42.3836097717,
		Timezone:  "America/Sao_Paulo",
	},
	"SJSL": {
		ICAO:      "SJSL",
		Name:      "Fazenda Santa Luiza Airport",
		City:      "Icaraima",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1162,
		Latitude:  -23.3477783203,
		Longitude: -53.5,
		Timezone:  "America/Sao_Paulo",
	},
	"SJSM": {
		ICAO:      "SJSM",
		Name:      "Fazenda Santa Rita Airport",
		City:      "Itaruma",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1451,
		Latitude:  -19.006111145,
		Longitude: -50.9522209167,
		Timezone:  "America/Sao_Paulo",
	},
	"SJSN": {
		ICAO:      "SJSN",
		Name:      "Fazenda Carvalho Airport",
		City:      "Sao Luiz Do Norte",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1871,
		Latitude:  -14.8366670609,
		Longitude: -49.3613891602,
		Timezone:  "America/Sao_Paulo",
	},
	"SJSO": {
		ICAO:      "SJSO",
		Name:      "Fazenda Divisao II Airport",
		City:      "Lucas Do Rio Verde",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1358,
		Latitude:  -12.9536113739,
		Longitude: -56.3183326721,
		Timezone:  "America/Cuiaba",
	},
	"SJSR": {
		ICAO:      "SJSR",
		Name:      "Fazenda Santa Rosa Airport",
		City:      "Mara Rosa",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1044,
		Latitude:  -13.9922218323,
		Longitude: -49.7630577087,
		Timezone:  "America/Sao_Paulo",
	},
	"SJSS": {
		ICAO:      "SJSS",
		Name:      "Agrifor Airport",
		City:      "Sorriso",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1100,
		Latitude:  -12.5083332062,
		Longitude: -55.6952781677,
		Timezone:  "America/Cuiaba",
	},
	"SJSU": {
		ICAO:      "SJSU",
		Name:      "Fazenda Sao Marcus Airport",
		City:      "Vila Alta",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1083,
		Latitude:  -23.5641670227,
		Longitude: -53.7972221375,
		Timezone:  "America/Sao_Paulo",
	},
	"SJTA": {
		ICAO:      "SJTA",
		Name:      "Cristal Airport",
		City:      "Bora",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1355,
		Latitude:  -22.1816673279,
		Longitude: -50.4925003052,
		Timezone:  "America/Sao_Paulo",
	},
	"SJTB": {
		ICAO:      "SJTB",
		Name:      "Fazenda Buriti Airport",
		City:      "Chapada Dos Guimaraes",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2451,
		Latitude:  -15.4650001526,
		Longitude: -55.6225013733,
		Timezone:  "America/Cuiaba",
	},
	"SJTF": {
		ICAO:      "SJTF",
		Name:      "Fazenda Mequens Airport",
		City:      "Alta Floresta D`Oeste",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 552,
		Latitude:  -13.0619440079,
		Longitude: -62.2574996948,
		Timezone:  "America/Porto_Velho",
	},
	"SJTG": {
		ICAO:      "SJTG",
		Name:      "Fazenda Calcario Tangara Airport",
		City:      "Tangara Da Serra",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1831,
		Latitude:  -14.4502782822,
		Longitude: -57.8005561829,
		Timezone:  "America/Cuiaba",
	},
	"SJTI": {
		ICAO:      "SJTI",
		Name:      "Fazenda Iapo Airport",
		City:      "Tibagi",
		State:     "Parana",
		Country:   "BR",
		Elevation: 2700,
		Latitude:  -24.5277786255,
		Longitude: -50.3658332825,
		Timezone:  "America/Sao_Paulo",
	},
	"SJTJ": {
		ICAO:      "SJTJ",
		Name:      "Fazenda Guanabara Airport",
		City:      "Nova Alvorada Do Sul",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1450,
		Latitude:  -21.1166667938,
		Longitude: -54.44972229,
		Timezone:  "America/Campo_Grande",
	},
	"SJTK": {
		ICAO:      "SJTK",
		Name:      "Ecofly Airport",
		City:      "Brotas",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2129,
		Latitude:  -22.2969436646,
		Longitude: -48.1202774048,
		Timezone:  "America/Sao_Paulo",
	},
	"SJTL": {
		ICAO:      "SJTL",
		Name:      "Aero Agricola Gaivota Airport",
		City:      "Jaguapita",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1821,
		Latitude:  -22.9786109924,
		Longitude: -51.5486106873,
		Timezone:  "America/Sao_Paulo",
	},
	"SJTM": {
		ICAO:      "SJTM",
		Name:      "Santa Cecilia Airport",
		City:      "Mirandopolis",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1260,
		Latitude:  -20.8877773285,
		Longitude: -50.9883346558,
		Timezone:  "America/Sao_Paulo",
	},
	"SJTN": {
		ICAO:      "SJTN",
		Name:      "Fazenda Alvorada do Caxingo Airport",
		City:      "Monte Alegre De Goias",
		Country:   "BR",
		Elevation: 1742,
		Latitude:  -13.375,
		Longitude: -46.8974990845,
		Timezone:  "America/Sao_Paulo",
	},
	"SJTO": {
		ICAO:      "SJTO",
		Name:      "Sao Jose Airport",
		City:      "Campo Novo Do Parecis",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2083,
		Latitude:  -14.1438894272,
		Longitude: -57.7805557251,
		Timezone:  "America/Cuiaba",
	},
	"SJTP": {
		ICAO:      "SJTP",
		Name:      "Boa Safra Airport",
		City:      "Tapura",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1321,
		Latitude:  -12.728055954,
		Longitude: -56.5027770996,
		Timezone:  "America/Cuiaba",
	},
	"SJTQ": {
		ICAO:      "SJTQ",
		Name:      "Fazenda Santa tereza Airport",
		City:      "Petropolis",
		State:     "Rio-de-Janeiro",
		Country:   "BR",
		Elevation: 2087,
		Latitude:  -22.368888855,
		Longitude: -43.2513885498,
		Timezone:  "America/Sao_Paulo",
	},
	"SJTS": {
		ICAO:      "SJTS",
		Name:      "Terra Santa Airport",
		City:      "Terra Santa",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 83,
		Latitude:  -2.0727779865,
		Longitude: -56.4888877869,
		Timezone:  "America/Santarem",
	},
	"SJTT": {
		ICAO:      "SJTT",
		Name:      "Fazenda Lagoa do Barro Airport",
		City:      "Novo Planalto",
		State:     "Goias",
		Country:   "BR",
		Elevation: 926,
		Latitude:  -13.2386112213,
		Longitude: -49.7094421387,
		Timezone:  "America/Sao_Paulo",
	},
	"SJTU": {
		ICAO:      "SJTU",
		Name:      "Fazenda Tucano Airport",
		City:      "Campo Novo Do Parecis",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1765,
		Latitude:  -13.582777977,
		Longitude: -57.5594444275,
		Timezone:  "America/Cuiaba",
	},
	"SJTV": {
		ICAO:      "SJTV",
		Name:      "Fazenda Guerra Airport",
		City:      "Juara",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 984,
		Latitude:  -11.4741668701,
		Longitude: -57.9211120605,
		Timezone:  "America/Cuiaba",
	},
	"SJTZ": {
		ICAO:      "SJTZ",
		Name:      "Dom Eliseu Airport",
		City:      "Dom Eliseu",
		Country:   "BR",
		Elevation: 800,
		Latitude:  -4.3027777672,
		Longitude: -47.5816688538,
		Timezone:  "America/Belem",
	},
	"SJUA": {
		ICAO:      "SJUA",
		Name:      "Fazenda Ipanema Airport",
		City:      "Correntina",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 3101,
		Latitude:  -13.6258325577,
		Longitude: -46.1397209167,
		Timezone:  "America/Bahia",
	},
	"SJUB": {
		ICAO:      "SJUB",
		Name:      "Fazenda Rio Azul Airport",
		City:      "Vila Bela Da Santissima Trindade",
		Country:   "BR",
		Elevation: 896,
		Latitude:  -15.1833333969,
		Longitude: -60.2216682434,
		Timezone:  "America/Cuiaba",
	},
	"SJUC": {
		ICAO:      "SJUC",
		Name:      "Jardim Paraiso Airport",
		City:      "Luis Eduardo Magalhaes",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 2395,
		Latitude:  -12.108332634,
		Longitude: -45.7775001526,
		Timezone:  "America/Bahia",
	},
	"SJUD": {
		ICAO:      "SJUD",
		Name:      "Usina Dourados Airport",
		City:      "Dourados",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1420,
		Latitude:  -21.990278244,
		Longitude: -55.1130561829,
		Timezone:  "America/Campo_Grande",
	},
	"SJUH": {
		ICAO:      "SJUH",
		Name:      "Fazenda Lagoinha Airport",
		City:      "Pedrinopolis",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3346,
		Latitude:  -19.2386112213,
		Longitude: -47.4977798462,
		Timezone:  "America/Sao_Paulo",
	},
	"SJUJ": {
		ICAO:      "SJUJ",
		Name:      "Fazenda Juina Airport",
		City:      "Campos De Julio",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2215,
		Latitude:  -13.4536113739,
		Longitude: -59.2602767944,
		Timezone:  "America/Cuiaba",
	},
	"SJUK": {
		ICAO:      "SJUK",
		Name:      "Fazenda Capitao Verdi Airport",
		City:      "Tangara Da Serra",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2435,
		Latitude:  -14.3599996567,
		Longitude: -57.9177780151,
		Timezone:  "America/Cuiaba",
	},
	"SJUL": {
		ICAO:      "SJUL",
		Name:      "Fazenda Rio Pardo Airport",
		City:      "Iaras",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2257,
		Latitude:  -22.8777770996,
		Longitude: -49.037776947,
		Timezone:  "America/Sao_Paulo",
	},
	"SJUQ": {
		ICAO:      "SJUQ",
		Name:      "Fazenda Vista Alegre Airport",
		City:      "Tres Lagoas",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1349,
		Latitude:  -19.7391662598,
		Longitude: -52.6061096191,
		Timezone:  "America/Campo_Grande",
	},
	"SJUS": {
		ICAO:      "SJUS",
		Name:      "Fazenda Acara Airport",
		City:      "Britania",
		State:     "Goias",
		Country:   "BR",
		Elevation: 804,
		Latitude:  -15.0744438171,
		Longitude: -51.1588897705,
		Timezone:  "America/Sao_Paulo",
	},
	"SJUU": {
		ICAO:      "SJUU",
		Name:      "Fazenda Parana Airport",
		City:      "Nova Roma",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1450,
		Latitude:  -14.0280561447,
		Longitude: -46.8711128235,
		Timezone:  "America/Sao_Paulo",
	},
	"SJVB": {
		ICAO:      "SJVB",
		Name:      "Fazenda Sagrado Coracao de Jesus Airport",
		City:      "Vila Bela Da Santissima Trindade",
		State:     "Santa-Cruz",
		Country:   "BR",
		Elevation: 499,
		Latitude:  -15.0916671753,
		Longitude: -60.1216659546,
		Timezone:  "America/Cuiaba",
	},
	"SJVC": {
		ICAO:      "SJVC",
		Name:      "Ibiporanga da Felicidade Airport",
		City:      "Tanabi",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1779,
		Latitude:  -20.4591674805,
		Longitude: -49.5336112976,
		Timezone:  "America/Sao_Paulo",
	},
	"SJVD": {
		ICAO:      "SJVD",
		Name:      "Nazare da Felicidade Airport",
		City:      "Buritizeiro",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2550,
		Latitude:  -17.8547210693,
		Longitude: -45.2405548096,
		Timezone:  "America/Sao_Paulo",
	},
	"SJVF": {
		ICAO:      "SJVF",
		Name:      "Fazenda Colorado Airport",
		City:      "Figueirao",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1785,
		Latitude:  -18.8177776337,
		Longitude: -53.8650016785,
		Timezone:  "America/Campo_Grande",
	},
	"SJVL": {
		ICAO:      "SJVL",
		Name:      "Fazenda Cachoeirinha Airport",
		City:      "Perdizes",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3281,
		Latitude:  -19.3702774048,
		Longitude: -47.1288871765,
		Timezone:  "America/Sao_Paulo",
	},
	"SJVM": {
		ICAO:      "SJVM",
		Name:      "Estancia California Airport",
		City:      "Alta Floresta",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 945,
		Latitude:  -9.8919439316,
		Longitude: -56.1294441223,
		Timezone:  "America/Cuiaba",
	},
	"SJVN": {
		ICAO:      "SJVN",
		Name:      "Fazenda do Limao Airport",
		City:      "Almeirim",
		State:     "Para",
		Country:   "BR",
		Elevation: 984,
		Latitude:  0.2891669869,
		Longitude: -53.8699989319,
		Timezone:  "America/Santarem",
	},
	"SJVO": {
		ICAO:      "SJVO",
		Name:      "Aragarcas Airport",
		City:      "Aragarcas",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1061,
		Latitude:  -15.8993997574,
		Longitude: -52.2411003113,
		Timezone:  "America/Cuiaba",
	},
	"SJVP": {
		ICAO:      "SJVP",
		Name:      "Vila Pitinga Airport",
		City:      "Presidente Figueiredo",
		Country:   "BR",
		Elevation: 876,
		Latitude:  -0.7991669774,
		Longitude: -60.0838890076,
		Timezone:  "America/Manaus",
	},
	"SJVQ": {
		ICAO:      "SJVQ",
		Name:      "Trialcool Airport",
		City:      "Canapolis",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2231,
		Latitude:  -18.8894443512,
		Longitude: -49.2861099243,
		Timezone:  "America/Sao_Paulo",
	},
	"SJVR": {
		ICAO:      "SJVR",
		Name:      "Fazenda Sao Joaquim Airport",
		City:      "Jardim",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1181,
		Latitude:  -21.4919433594,
		Longitude: -56.4872207642,
		Timezone:  "America/Campo_Grande",
	},
	"SJVS": {
		ICAO:      "SJVS",
		Name:      "Vale do Paranaiba Airport",
		City:      "Capinopolis",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1739,
		Latitude:  -18.7030563354,
		Longitude: -49.6869430542,
		Timezone:  "America/Sao_Paulo",
	},
	"SJVV": {
		ICAO:      "SJVV",
		Name:      "Fazenda Catelani Airport",
		City:      "Minacu",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2461,
		Latitude:  -13.4733333588,
		Longitude: -48.4591674805,
		Timezone:  "America/Sao_Paulo",
	},
	"SJVW": {
		ICAO:      "SJVW",
		Name:      "Campos Gerais Airport",
		City:      "Balsas",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 1706,
		Latitude:  -8.5169439316,
		Longitude: -46.7469444275,
		Timezone:  "America/Fortaleza",
	},
	"SJVX": {
		ICAO:      "SJVX",
		Name:      "Pagrisa Airport",
		City:      "Ulianopolis",
		State:     "Para",
		Country:   "BR",
		Elevation: 453,
		Latitude:  -3.7005560398,
		Longitude: -47.7633323669,
		Timezone:  "America/Belem",
	},
	"SJVY": {
		ICAO:      "SJVY",
		Name:      "Haras Bela Vista Airport",
		City:      "Sao Miguel Do Tocantins",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 335,
		Latitude:  -5.5416669846,
		Longitude: -47.5024986267,
		Timezone:  "America/Araguaina",
	},
	"SJVZ": {
		ICAO:      "SJVZ",
		Name:      "Fazenda Sao Paulo Airport",
		City:      "Sapezal",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2012,
		Latitude:  -13.6102781296,
		Longitude: -58.5691680908,
		Timezone:  "America/Cuiaba",
	},
	"SJWA": {
		ICAO:      "SJWA",
		Name:      "Usina Maracaju Airport",
		City:      "Maracaju",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1509,
		Latitude:  -21.3452777863,
		Longitude: -55.4333343506,
		Timezone:  "America/Campo_Grande",
	},
	"SJWB": {
		ICAO:      "SJWB",
		Name:      "Pouso Alto Airport",
		City:      "Piracanjuba",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2428,
		Latitude:  -17.4577789307,
		Longitude: -48.9830551147,
		Timezone:  "America/Sao_Paulo",
	},
	"SJWC": {
		ICAO:      "SJWC",
		Name:      "Fazenda Nossa Senhora da Muxima Airport",
		City:      "Agua Clara",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1532,
		Latitude:  -19.5872211456,
		Longitude: -52.9519424438,
		Timezone:  "America/Campo_Grande",
	},
	"SJWD": {
		ICAO:      "SJWD",
		Name:      "Usina Passa Tempo Airport",
		City:      "Rio Brilhante",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 984,
		Latitude:  -21.5072212219,
		Longitude: -54.7158317566,
		Timezone:  "America/Campo_Grande",
	},
	"SJWE": {
		ICAO:      "SJWE",
		Name:      "Fazenda Arariba Airport",
		City:      "Canarana",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1279,
		Latitude:  -13.4047222137,
		Longitude: -52.8483314514,
		Timezone:  "America/Cuiaba",
	},
	"SJWG": {
		ICAO:      "SJWG",
		Name:      "Itagro Airport",
		City:      "Alegrete",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 384,
		Latitude:  -29.7802772522,
		Longitude: -55.6972236633,
		Timezone:  "America/Sao_Paulo",
	},
	"SJWH": {
		ICAO:      "SJWH",
		Name:      "Fazenda Santa Ines Airport",
		City:      "Nova Brasilandia",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1330,
		Latitude:  -14.6511106491,
		Longitude: -55.3838882446,
		Timezone:  "America/Cuiaba",
	},
	"SJWI": {
		ICAO:      "SJWI",
		Name:      "Fazenda Flor Airport",
		City:      "Campo Grande",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1420,
		Latitude:  -21.4969444275,
		Longitude: -53.7497215271,
		Timezone:  "America/Campo_Grande",
	},
	"SJWL": {
		ICAO:      "SJWL",
		Name:      "Boca da Mata Airport",
		City:      "Gaucha Do Norte",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1804,
		Latitude:  -13.5744438171,
		Longitude: -53.9830551147,
		Timezone:  "America/Cuiaba",
	},
	"SJWM": {
		ICAO:      "SJWM",
		Name:      "Fazenda Sanga Funda Airport",
		City:      "Sapezal",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1949,
		Latitude:  -13.0666666031,
		Longitude: -58.7058334351,
		Timezone:  "America/Cuiaba",
	},
	"SJWN": {
		ICAO:      "SJWN",
		Name:      "Fazenda Santa Dulcina Airport",
		City:      "Taquarituba",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2096,
		Latitude:  -23.5499992371,
		Longitude: -49.25,
		Timezone:  "America/Sao_Paulo",
	},
	"SJWO": {
		ICAO:      "SJWO",
		Name:      "Terra Santa Airport",
		City:      "Barra Do Bugres",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 699,
		Latitude:  -14.8069438934,
		Longitude: -57.9736099243,
		Timezone:  "America/Cuiaba",
	},
	"SJWP": {
		ICAO:      "SJWP",
		Name:      "Warpol Airport",
		City:      "Sao Desiderio",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 2650,
		Latitude:  -12.6194438934,
		Longitude: -45.9236106873,
		Timezone:  "America/Bahia",
	},
	"SJWQ": {
		ICAO:      "SJWQ",
		Name:      "Jose Dona Airport",
		City:      "Birigui",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1338,
		Latitude:  -21.2175006866,
		Longitude: -50.3008346558,
		Timezone:  "America/Sao_Paulo",
	},
	"SJWT": {
		ICAO:      "SJWT",
		Name:      "Fazenda Sucuri Airport",
		City:      "Itiquira",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1788,
		Latitude:  -17.3958320618,
		Longitude: -54.7597236633,
		Timezone:  "America/Cuiaba",
	},
	"SJWU": {
		ICAO:      "SJWU",
		Name:      "Fazenda Ideal Airport",
		City:      "Nova Venecia",
		State:     "Espirito-Santo",
		Country:   "BR",
		Elevation: 471,
		Latitude:  -18.6672210693,
		Longitude: -40.3327789307,
		Timezone:  "America/Sao_Paulo",
	},
	"SJWV": {
		ICAO:      "SJWV",
		Name:      "Fazenda Sao Judas Airport",
		City:      "Sao Miguel Do Araguaia",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1030,
		Latitude:  -13.3780555725,
		Longitude: -50.4244422913,
		Timezone:  "America/Sao_Paulo",
	},
	"SJWW": {
		ICAO:      "SJWW",
		Name:      "Fazenda Cachoeira Preta Airport",
		City:      "Agua Clara",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1083,
		Latitude:  -20.0213890076,
		Longitude: -53.2161102295,
		Timezone:  "America/Campo_Grande",
	},
	"SJWX": {
		ICAO:      "SJWX",
		Name:      "Rio Alegre Airport",
		City:      "Euclides Da Cunha Paulista",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 981,
		Latitude:  -22.5750007629,
		Longitude: -52.5030555725,
		Timezone:  "America/Sao_Paulo",
	},
	"SJXA": {
		ICAO:      "SJXA",
		Name:      "Aeromis Airport",
		City:      "Bage",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 886,
		Latitude:  -31.2922210693,
		Longitude: -54.1561126709,
		Timezone:  "America/Sao_Paulo",
	},
	"SJXB": {
		ICAO:      "SJXB",
		Name:      "Aquapey Airport",
		City:      "Porto Esperdiao",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 600,
		Latitude:  -15.7563886642,
		Longitude: -58.8019447327,
		Timezone:  "America/Cuiaba",
	},
	"SJXC": {
		ICAO:      "SJXC",
		Name:      "Usina Santa Cruz Airport",
		City:      "Santa Cruz Cabralia",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 410,
		Latitude:  -16.1830558777,
		Longitude: -39.3580551147,
		Timezone:  "America/Bahia",
	},
	"SJXG": {
		ICAO:      "SJXG",
		Name:      "Fazenda Agroplan Airport",
		City:      "Sao Felix Do Araguaia",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1083,
		Latitude:  -11.6633329391,
		Longitude: -52.8499984741,
		Timezone:  "America/Cuiaba",
	},
	"SJXH": {
		ICAO:      "SJXH",
		Name:      "Aguas Frias Airport",
		City:      "Irai",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 1135,
		Latitude:  -27.2169437408,
		Longitude: -53.2724990845,
		Timezone:  "America/Sao_Paulo",
	},
	"SJXM": {
		ICAO:      "SJXM",
		Name:      "Carolina de Assis Repetto Airport",
		City:      "Lima Duarte",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3452,
		Latitude:  -21.7322216034,
		Longitude: -43.894443512,
		Timezone:  "America/Sao_Paulo",
	},
	"SJXN": {
		ICAO:      "SJXN",
		Name:      "Parque Fazenda Bom Futuro Airport",
		City:      "Pocone",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 361,
		Latitude:  -17.1913890839,
		Longitude: -57.0338897705,
		Timezone:  "America/Cuiaba",
	},
	"SJXS": {
		ICAO:      "SJXS",
		Name:      "Veracel Airport",
		City:      "Belmonte",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 434,
		Latitude:  -16.1044445038,
		Longitude: -39.3638877869,
		Timezone:  "America/Bahia",
	},
	"SJXV": {
		ICAO:      "SJXV",
		Name:      "Fazenda Bela Vista Airport",
		City:      "Amambai",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1256,
		Latitude:  -22.9311103821,
		Longitude: -55.0544433594,
		Timezone:  "America/Campo_Grande",
	},
	"SJXW": {
		ICAO:      "SJXW",
		Name:      "Fazenda 3M Airport",
		City:      "Nova Alvorada Do Sul",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1155,
		Latitude:  -21.6736106873,
		Longitude: -54.0358314514,
		Timezone:  "America/Campo_Grande",
	},
	"SJXX": {
		ICAO:      "SJXX",
		Name:      "Recreio Airport",
		City:      "Ponta Pora",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1968,
		Latitude:  -22.6794433594,
		Longitude: -55.5405578613,
		Timezone:  "America/Campo_Grande",
	},
	"SJYD": {
		ICAO:      "SJYD",
		Name:      "Fazenda Kajussol Airport",
		City:      "Santa Luzia D`Oeste",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 636,
		Latitude:  -11.9647216797,
		Longitude: -61.686668396,
		Timezone:  "America/Porto_Velho",
	},
	"SJYE": {
		ICAO:      "SJYE",
		Name:      "Agua Fria Airport",
		City:      "Pacaraima",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 994,
		Latitude:  4.6205601692,
		Longitude: -60.2771987915,
		Timezone:  "America/Boa_Vista",
	},
	"SJYF": {
		ICAO:      "SJYF",
		Name:      "Ajarani Airport",
		City:      "Caracarai",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 594,
		Latitude:  2.0077800751,
		Longitude: -61.4757995605,
		Timezone:  "America/Boa_Vista",
	},
	"SJYG": {
		ICAO:      "SJYG",
		Name:      "Alto Mucajai Airport",
		City:      "Alto Alegre",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 237,
		Latitude:  2.7650001049,
		Longitude: -62.2214012146,
		Timezone:  "America/Boa_Vista",
	},
	"SJYI": {
		ICAO:      "SJYI",
		Name:      "Arai Airport",
		City:      "Pacaraima",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 1500,
		Latitude:  4.4438900948,
		Longitude: -60.8471984863,
		Timezone:  "America/Boa_Vista",
	},
	"SJYJ": {
		ICAO:      "SJYJ",
		Name:      "Aratha-U Airport",
		City:      "Alto Alegre",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 236,
		Latitude:  3.1608300209,
		Longitude: -63.7813987732,
		Timezone:  "America/Boa_Vista",
	},
	"SJYK": {
		ICAO:      "SJYK",
		Name:      "Baixo Catrimani Airport",
		City:      "Caracarai",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 171,
		Latitude:  1.0711100101,
		Longitude: -62.2182998657,
		Timezone:  "America/Boa_Vista",
	},
	"SJYL": {
		ICAO:      "SJYL",
		Name:      "Baixo Mucajai Airport",
		City:      "Alto Alegre",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 499,
		Latitude:  2.7369399071,
		Longitude: -62.0194015503,
		Timezone:  "America/Boa_Vista",
	},
	"SJYM": {
		ICAO:      "SJYM",
		Name:      "Bala Airport",
		City:      "Pacaraima",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 348,
		Latitude:  3.8372199535,
		Longitude: -60.6032981873,
		Timezone:  "America/Boa_Vista",
	},
	"SJYO": {
		ICAO:      "SJYO",
		Name:      "Bananal Airport",
		City:      "Pacaraima",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 892,
		Latitude:  4.6327800751,
		Longitude: -60.5856018066,
		Timezone:  "America/Boa_Vista",
	},
	"SJYP": {
		ICAO:      "SJYP",
		Name:      "Bananeira Airport",
		City:      "Uiramuta",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 892,
		Latitude:  4.3211097717,
		Longitude: -60.193901062,
		Timezone:  "America/Boa_Vista",
	},
	"SJYR": {
		ICAO:      "SJYR",
		Name:      "Caju Airport",
		City:      "Pacaraima",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 794,
		Latitude:  4.7058300972,
		Longitude: -60.5153007507,
		Timezone:  "America/Boa_Vista",
	},
	"SJYS": {
		ICAO:      "SJYS",
		Name:      "Camara Airport",
		City:      "Normandia",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 594,
		Latitude:  3.9908299446,
		Longitude: -60.1817016602,
		Timezone:  "America/Boa_Vista",
	},
	"SJYT": {
		ICAO:      "SJYT",
		Name:      "Campo Formoso Airport",
		City:      "Pacaraima",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 1194,
		Latitude:  4.7199997902,
		Longitude: -60.7696990967,
		Timezone:  "America/Boa_Vista",
	},
	"SJYU": {
		ICAO:      "SJYU",
		Name:      "Campo Grande Airport",
		City:      "Pacaraima",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 1194,
		Latitude:  4.6855602264,
		Longitude: -60.8027992249,
		Timezone:  "America/Boa_Vista",
	},
	"SJYW": {
		ICAO:      "SJYW",
		Name:      "Fazenda Santa Terezinha Airport",
		City:      "Santa Terezinha",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 748,
		Latitude:  -10.3313894272,
		Longitude: -50.9402770996,
		Timezone:  "America/Cuiaba",
	},
	"SJYY": {
		ICAO:      "SJYY",
		Name:      "Rio do Sangue Airport",
		City:      "Brasnorte",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 935,
		Latitude:  -11.5222215652,
		Longitude: -58.1772232056,
		Timezone:  "America/Cuiaba",
	},
	"SJYZ": {
		ICAO:      "SJYZ",
		Name:      "Fazenda Lagoa Encantada Airport",
		City:      "Primavera Do Leste",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1968,
		Latitude:  -15.270277977,
		Longitude: -54.2574996948,
		Timezone:  "America/Cuiaba",
	},
	"SJZA": {
		ICAO:      "SJZA",
		Name:      "Capao Alto Airport",
		City:      "Tapes",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 107,
		Latitude:  -30.6763896942,
		Longitude: -51.5252761841,
		Timezone:  "America/Sao_Paulo",
	},
	"SJZB": {
		ICAO:      "SJZB",
		Name:      "Vale do Tucana Airport",
		City:      "Castanheira",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 879,
		Latitude:  -10.6894435883,
		Longitude: -58.649723053,
		Timezone:  "America/Cuiaba",
	},
	"SJZC": {
		ICAO:      "SJZC",
		Name:      "Destilaria Sao Luiz Airport",
		City:      "Maraial",
		State:     "Pernambuco",
		Country:   "BR",
		Elevation: 1371,
		Latitude:  -8.7788887024,
		Longitude: -35.7849998474,
		Timezone:  "America/Recife",
	},
	"SJZD": {
		ICAO:      "SJZD",
		Name:      "Fazenda Bandeiras Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 361,
		Latitude:  -17.8619441986,
		Longitude: -56.7030563354,
		Timezone:  "America/Campo_Grande",
	},
	"SJZE": {
		ICAO:      "SJZE",
		Name:      "Fazenda Brioso Airport",
		City:      "Tres Lagoas",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1270,
		Latitude:  -20.2405567169,
		Longitude: -52.3866653442,
		Timezone:  "America/Campo_Grande",
	},
	"SJZF": {
		ICAO:      "SJZF",
		Name:      "Fazenda Campo Triste Airport",
		City:      "Tres Lagoas",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1197,
		Latitude:  -20.6186103821,
		Longitude: -52.0180549622,
		Timezone:  "America/Campo_Grande",
	},
	"SJZG": {
		ICAO:      "SJZG",
		Name:      "Leonel de Moura Brizola Airport",
		City:      "Confresa",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 843,
		Latitude:  -10.6583328247,
		Longitude: -51.4361114502,
		Timezone:  "America/Cuiaba",
	},
	"SJZH": {
		ICAO:      "SJZH",
		Name:      "Fazenda Rio do Fogo Airport",
		City:      "Nova Santa Helena",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 981,
		Latitude:  -10.8816671371,
		Longitude: -54.9977798462,
		Timezone:  "America/Cuiaba",
	},
	"SJZM": {
		ICAO:      "SJZM",
		Name:      "Carlos Alberto Pinto Airport",
		City:      "Arraias",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 1338,
		Latitude:  -12.6069440842,
		Longitude: -46.7472229004,
		Timezone:  "America/Araguaina",
	},
	"SJZO": {
		ICAO:      "SJZO",
		Name:      "Fazenda Boi Branco Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 394,
		Latitude:  -18.465555191,
		Longitude: -56.7633323669,
		Timezone:  "America/Campo_Grande",
	},
	"SJZP": {
		ICAO:      "SJZP",
		Name:      "Mathovi Airport",
		City:      "Novo Mundo",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1070,
		Latitude:  -9.8330564499,
		Longitude: -55.4647216797,
		Timezone:  "America/Cuiaba",
	},
	"SJZQ": {
		ICAO:      "SJZQ",
		Name:      "Fazenda Monte Alegre Airport",
		City:      "Corrente",
		State:     "Piaui",
		Country:   "BR",
		Elevation: 1444,
		Latitude:  -10.5641670227,
		Longitude: -44.9469451904,
		Timezone:  "America/Fortaleza",
	},
	"SJZR": {
		ICAO:      "SJZR",
		Name:      "Fazenda Italia Airport",
		City:      "Urucui",
		State:     "Piaui",
		Country:   "BR",
		Elevation: 1280,
		Latitude:  -7.4447221756,
		Longitude: -44.3533325195,
		Timezone:  "America/Fortaleza",
	},
	"SJZS": {
		ICAO:      "SJZS",
		Name:      "Fazenda Canel Airport",
		City:      "Urucui",
		State:     "Piaui",
		Country:   "BR",
		Elevation: 1280,
		Latitude:  -7.5399999619,
		Longitude: -44.8224983215,
		Timezone:  "America/Fortaleza",
	},
	"SJZT": {
		ICAO:      "SJZT",
		Name:      "Fazenda Busato II Airport",
		City:      "Serra Ramalho",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1500,
		Latitude:  -13.2386112213,
		Longitude: -43.7361106873,
		Timezone:  "America/Bahia",
	},
	"SJZW": {
		ICAO:      "SJZW",
		Name:      "Fazenda Regina Airport",
		City:      "Itatinga",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2044,
		Latitude:  -23.2866668701,
		Longitude: -48.7380561829,
		Timezone:  "America/Sao_Paulo",
	},
	"SJZX": {
		ICAO:      "SJZX",
		Name:      "Fazenda Rebojo Airport",
		City:      "Corumba",
		State:     "Santa-Cruz",
		Country:   "BR",
		Elevation: 397,
		Latitude:  -20.0263881683,
		Longitude: -57.918888092,
		Timezone:  "America/Campo_Grande",
	},
	"SJZZ": {
		ICAO:      "SJZZ",
		Name:      "Fazenda Passo Fundo Airport",
		City:      "Primavera Do Leste",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2199,
		Latitude:  -15.0174999237,
		Longitude: -53.9358329773,
		Timezone:  "America/Cuiaba",
	},
	"SK53": {
		ICAO:      "SK53",
		Name:      "Yaguarito Airport",
		City:      "Surimena",
		State:     "Meta",
		Country:   "CO",
		Elevation: 836,
		Latitude:  3.88,
		Longitude: -73.339996,
		Timezone:  "America/Bogota",
	},
	"SKAC": {
		ICAO:      "SKAC",
		IATA:      "ACR",
		Name:      "Araracuara Airport",
		City:      "Araracuara",
		State:     "Amazonas",
		Country:   "CO",
		Elevation: 1250,
		Latitude:  -0.5833,
		Longitude: -72.4083,
		Timezone:  "America/Bogota",
	},
	"SKAD": {
		ICAO:      "SKAD",
		IATA:      "ACD",
		Name:      "Alcides Fernandez Airport",
		City:      "Acandi",
		State:     "Choco",
		Country:   "CO",
		Elevation: 50,
		Latitude:  8.51667,
		Longitude: -77.3,
		Timezone:  "America/Bogota",
	},
	"SKAG": {
		ICAO:      "SKAG",
		IATA:      "HAY",
		Name:      "Aguachica Airport",
		City:      "Aguachica",
		State:     "Cesar",
		Country:   "CO",
		Elevation: 700,
		Latitude:  8.3,
		Longitude: -73.6306,
		Timezone:  "America/Bogota",
	},
	"SKAM": {
		ICAO:      "SKAM",
		IATA:      "AFI",
		Name:      "Amalfi Airport",
		City:      "Amalfi",
		State:     "Antioquia",
		Country:   "CO",
		Elevation: 5507,
		Latitude:  6.91667,
		Longitude: -75.0667,
		Timezone:  "America/Bogota",
	},
	"SKAN": {
		ICAO:      "SKAN",
		IATA:      "ADN",
		Name:      "Andes Airport",
		City:      "Andes",
		State:     "Antioquia",
		Country:   "CO",
		Elevation: 3900,
		Latitude:  5.66667,
		Longitude: -75.8833,
		Timezone:  "America/Bogota",
	},
	"SKAP": {
		ICAO:      "SKAP",
		IATA:      "API",
		Name:      "Gomez Nino Apiay Air Base",
		City:      "Apiay",
		State:     "Meta",
		Country:   "CO",
		Elevation: 1207,
		Latitude:  4.07607,
		Longitude: -73.5627,
		Timezone:  "America/Bogota",
	},
	"SKAR": {
		ICAO:      "SKAR",
		IATA:      "AXM",
		Name:      "El Eden Airport",
		City:      "Armenia",
		State:     "Quindio",
		Country:   "CO",
		Elevation: 3990,
		Latitude:  4.45278,
		Longitude: -75.7664,
		Timezone:  "America/Bogota",
	},
	"SKAS": {
		ICAO:      "SKAS",
		IATA:      "PUU",
		Name:      "Tres De Mayo Airport",
		City:      "Puerto Asis",
		State:     "Putumayo",
		Country:   "CO",
		Elevation: 815,
		Latitude:  0.505228,
		Longitude: -76.5008,
		Timezone:  "America/Bogota",
	},
	"SKBC": {
		ICAO:      "SKBC",
		IATA:      "ELB",
		Name:      "Las Flores Airport",
		City:      "El Banco",
		State:     "Magdalena",
		Country:   "CO",
		Elevation: 111,
		Latitude:  9.04554,
		Longitude: -73.9749,
		Timezone:  "America/Bogota",
	},
	"SKBE": {
		ICAO:      "SKBE",
		Name:      "Becerril Airport",
		City:      "Becerril",
		State:     "Cesar",
		Country:   "CO",
		Elevation: 433,
		Latitude:  9.66278,
		Longitude: -73.2744,
		Timezone:  "America/Bogota",
	},
	"SKBG": {
		ICAO:      "SKBG",
		IATA:      "BGA",
		Name:      "Palonegro Airport",
		City:      "Bucaramanga",
		State:     "Santander",
		Country:   "CO",
		Elevation: 3897,
		Latitude:  7.1265,
		Longitude: -73.1848,
		Timezone:  "America/Bogota",
	},
	"SKBN": {
		ICAO:      "SKBN",
		Name:      "Buenavista Airport",
		City:      "Fonseca",
		State:     "La-Guajira",
		Country:   "CO",
		Elevation: 850,
		Latitude:  10.8879,
		Longitude: -72.9004,
		Timezone:  "America/Bogota",
	},
	"SKBO": {
		ICAO:      "SKBO",
		IATA:      "BOG",
		Name:      "El Dorado International Airport",
		City:      "Bogota",
		State:     "Bogota-D-C-",
		Country:   "CO",
		Elevation: 8361,
		Latitude:  4.70159,
		Longitude: -74.1469,
		Timezone:  "America/Bogota",
	},
	"SKBQ": {
		ICAO:      "SKBQ",
		IATA:      "BAQ",
		Name:      "Ernesto Cortissoz International Airport",
		City:      "Barranquilla",
		State:     "Atlantico",
		Country:   "CO",
		Elevation: 98,
		Latitude:  10.8896,
		Longitude: -74.7808,
		Timezone:  "America/Bogota",
	},
	"SKBR": {
		ICAO:      "SKBR",
		Name:      "Berastegui Airport",
		City:      "Berastegui",
		State:     "Cordoba",
		Country:   "CO",
		Elevation: 38,
		Latitude:  8.88331,
		Longitude: -75.6832,
		Timezone:  "America/Bogota",
	},
	"SKBS": {
		ICAO:      "SKBS",
		IATA:      "BSC",
		Name:      "Jose Celestino Mutis Airport",
		City:      "Bahia Solano",
		State:     "Choco",
		Country:   "CO",
		Elevation: 80,
		Latitude:  6.20292,
		Longitude: -77.3947,
		Timezone:  "America/Bogota",
	},
	"SKBU": {
		ICAO:      "SKBU",
		IATA:      "BUN",
		Name:      "Gerardo Tobar Lopez Airport",
		City:      "Buenaventura",
		State:     "Valle-del-Cauca",
		Country:   "CO",
		Elevation: 48,
		Latitude:  3.81963,
		Longitude: -76.9898,
		Timezone:  "America/Bogota",
	},
	"SKCA": {
		ICAO:      "SKCA",
		IATA:      "CPB",
		Name:      "Capurgana Airport",
		City:      "Capurgana",
		State:     "Choco",
		Country:   "CO",
		Elevation: 49,
		Latitude:  8.63333,
		Longitude: -77.35,
		Timezone:  "America/Bogota",
	},
	"SKCB": {
		ICAO:      "SKCB",
		Name:      "Carmen De Bolivar Airport",
		City:      "Carmen De Bolivar",
		State:     "Bolivar",
		Country:   "CO",
		Elevation: 590,
		Latitude:  9.68417,
		Longitude: -75.1261,
		Timezone:  "America/Bogota",
	},
	"SKCC": {
		ICAO:      "SKCC",
		IATA:      "CUC",
		Name:      "Camilo Daza International Airport",
		City:      "Cucuta",
		State:     "Norte-de-Santander",
		Country:   "CO",
		Elevation: 1096,
		Latitude:  7.92757,
		Longitude: -72.5115,
		Timezone:  "America/Bogota",
	},
	"SKCD": {
		ICAO:      "SKCD",
		IATA:      "COG",
		Name:      "Mandinga Airport",
		City:      "Condoto",
		State:     "Choco",
		Country:   "CO",
		Elevation: 213,
		Latitude:  5.08333,
		Longitude: -76.7,
		Timezone:  "America/Bogota",
	},
	"SKCG": {
		ICAO:      "SKCG",
		IATA:      "CTG",
		Name:      "Rafael Nunez International Airport",
		City:      "Cartagena",
		State:     "Bolivar",
		Country:   "CO",
		Elevation: 4,
		Latitude:  10.4424,
		Longitude: -75.513,
		Timezone:  "America/Bogota",
	},
	"SKCH": {
		ICAO:      "SKCH",
		Name:      "El Pacifico Airport",
		City:      "Cartagena Del Chiara",
		State:     "Caqueta",
		Country:   "CO",
		Elevation: 754,
		Latitude:  1.35,
		Longitude: -74.84,
		Timezone:  "America/Bogota",
	},
	"SKCI": {
		ICAO:      "SKCI",
		IATA:      "CCO",
		Name:      "Carimagua Airport",
		City:      "Puerto Lopez",
		State:     "Meta",
		Country:   "CO",
		Elevation: 700,
		Latitude:  4.56417,
		Longitude: -71.3364,
		Timezone:  "America/Bogota",
	},
	"SKCL": {
		ICAO:      "SKCL",
		IATA:      "CLO",
		Name:      "Alfonso Bonilla Aragon International Airport",
		City:      "Cali",
		State:     "Valle-del-Cauca",
		Country:   "CO",
		Elevation: 3162,
		Latitude:  3.54322,
		Longitude: -76.3816,
		Timezone:  "America/Bogota",
	},
	"SKCM": {
		ICAO:      "SKCM",
		IATA:      "CIM",
		Name:      "Cimitarra Airport",
		City:      "Cimitarra",
		State:     "Santander",
		Country:   "CO",
		Elevation: 740,
		Latitude:  6.367,
		Longitude: -73.967,
		Timezone:  "America/Bogota",
	},
	"SKCN": {
		ICAO:      "SKCN",
		IATA:      "RAV",
		Name:      "Cravo Norte Airport",
		City:      "Cravo Norte",
		State:     "Arauca",
		Country:   "CO",
		Elevation: 341,
		Latitude:  6.31684,
		Longitude: -70.2107,
		Timezone:  "America/Bogota",
	},
	"SKCO": {
		ICAO:      "SKCO",
		IATA:      "TCO",
		Name:      "La Florida Airport",
		City:      "Tumaco",
		State:     "Narino",
		Country:   "CO",
		Elevation: 8,
		Latitude:  1.81442,
		Longitude: -78.7492,
		Timezone:  "America/Bogota",
	},
	"SKCP": {
		ICAO:      "SKCP",
		IATA:      "BHF",
		Name:      "Cupica Airport",
		City:      "Bahia Solano",
		State:     "Choco",
		Country:   "CO",
		Latitude:  6.691944,
		Longitude: -77.477222,
		Timezone:  "America/Bogota",
	},
	"SKCR": {
		ICAO:      "SKCR",
		IATA:      "CUO",
		Name:      "Caruru Airport",
		City:      "Caruru",
		State:     "Vaupes",
		Country:   "CO",
		Elevation: 820,
		Latitude:  1.0136,
		Longitude: -71.2961,
		Timezone:  "America/Bogota",
	},
	"SKCU": {
		ICAO:      "SKCU",
		IATA:      "CAQ",
		Name:      "Juan H White Airport",
		City:      "Caucasia",
		State:     "Antioquia",
		Country:   "CO",
		Elevation: 174,
		Latitude:  7.96847,
		Longitude: -75.1985,
		Timezone:  "America/Bogota",
	},
	"SKCV": {
		ICAO:      "SKCV",
		IATA:      "CVE",
		Name:      "Covenas Airport",
		City:      "Covenas",
		Country:   "CO",
		Elevation: 31,
		Latitude:  9.40092,
		Longitude: -75.6913,
		Timezone:  "America/Bogota",
	},
	"SKCZ": {
		ICAO:      "SKCZ",
		IATA:      "CZU",
		Name:      "Las Brujas Airport",
		City:      "Corozal",
		State:     "Sucre",
		Country:   "CO",
		Elevation: 528,
		Latitude:  9.33274,
		Longitude: -75.2856,
		Timezone:  "America/Bogota",
	},
	"SKEB": {
		ICAO:      "SKEB",
		IATA:      "EBG",
		Name:      "El Bagre Airport",
		City:      "El Bagre",
		State:     "Antioquia",
		Country:   "CO",
		Elevation: 180,
		Latitude:  7.59647,
		Longitude: -74.8089,
		Timezone:  "America/Bogota",
	},
	"SKEJ": {
		ICAO:      "SKEJ",
		IATA:      "EJA",
		Name:      "Yariguies Airport",
		City:      "Barrancabermeja",
		State:     "Santander",
		Country:   "CO",
		Elevation: 412,
		Latitude:  7.02433,
		Longitude: -73.8068,
		Timezone:  "America/Bogota",
	},
	"SKFL": {
		ICAO:      "SKFL",
		IATA:      "FLA",
		Name:      "Gustavo Artunduaga Paredes Airport",
		City:      "Florencia",
		State:     "Caqueta",
		Country:   "CO",
		Elevation: 803,
		Latitude:  1.58919,
		Longitude: -75.5644,
		Timezone:  "America/Bogota",
	},
	"SKFR": {
		ICAO:      "SKFR",
		Name:      "Furatena Airport",
		City:      "Quipama",
		State:     "Boyaca",
		Country:   "CO",
		Elevation: 3850,
		Latitude:  5.5275,
		Longitude: -74.1869,
		Timezone:  "America/Bogota",
	},
	"SKFU": {
		ICAO:      "SKFU",
		IATA:      "FDA",
		Name:      "Fundacion Airport",
		City:      "Fundacion",
		State:     "Magdalena",
		Country:   "CO",
		Elevation: 150,
		Latitude:  10.5333,
		Longitude: -74.2,
		Timezone:  "America/Bogota",
	},
	"SKGA": {
		ICAO:      "SKGA",
		IATA:      "LGT",
		Name:      "La Gaviota Airport",
		State:     "Vichada",
		Country:   "CO",
		Latitude:  4.549722,
		Longitude: -70.925,
		Timezone:  "America/Bogota",
	},
	"SKGB": {
		ICAO:      "SKGB",
		Name:      "Marco Fidel Suarez Air Base",
		City:      "Cali",
		State:     "Valle-del-Cauca",
		Country:   "CO",
		Elevation: 3165,
		Latitude:  3.45899,
		Longitude: -76.4966,
		Timezone:  "America/Bogota",
	},
	"SKGI": {
		ICAO:      "SKGI",
		IATA:      "GIR",
		Name:      "Santiago Vila Airport",
		City:      "Girardot",
		State:     "Tolima",
		Country:   "CO",
		Elevation: 900,
		Latitude:  4.27624,
		Longitude: -74.7967,
		Timezone:  "America/Bogota",
	},
	"SKGO": {
		ICAO:      "SKGO",
		IATA:      "CRC",
		Name:      "Santa Ana Airport",
		City:      "Cartago",
		State:     "Valle-del-Cauca",
		Country:   "CO",
		Elevation: 2979,
		Latitude:  4.75818,
		Longitude: -75.9557,
		Timezone:  "America/Bogota",
	},
	"SKGP": {
		ICAO:      "SKGP",
		IATA:      "GPI",
		Name:      "Juan Casiano Airport",
		City:      "Guapi",
		State:     "Cauca",
		Country:   "CO",
		Elevation: 164,
		Latitude:  2.57013,
		Longitude: -77.8986,
		Timezone:  "America/Bogota",
	},
	"SKGY": {
		ICAO:      "SKGY",
		Name:      "Guaymaral Airport",
		City:      "Chia",
		State:     "Cundinamarca",
		Country:   "CO",
		Elevation: 8390,
		Latitude:  4.81233,
		Longitude: -74.0649,
		Timezone:  "America/Bogota",
	},
	"SKGZ": {
		ICAO:      "SKGZ",
		IATA:      "GLJ",
		Name:      "Garzon La Jagua Airport",
		City:      "Garzon",
		State:     "Huila",
		Country:   "CO",
		Elevation: 2620,
		Latitude:  2.1464,
		Longitude: -75.6944,
		Timezone:  "America/Bogota",
	},
	"SKHA": {
		ICAO:      "SKHA",
		IATA:      "CPL",
		Name:      "Chaparral Airport",
		City:      "Chaparral",
		State:     "Tolima",
		Country:   "CO",
		Elevation: 2730,
		Latitude:  3.717,
		Longitude: -75.467,
		Timezone:  "America/Bogota",
	},
	"SKHC": {
		ICAO:      "SKHC",
		IATA:      "HTZ",
		Name:      "Hato Corozal Airport",
		City:      "Hato Corozal",
		State:     "Casanare",
		Country:   "CO",
		Elevation: 820,
		Latitude:  6.15,
		Longitude: -71.75,
		Timezone:  "America/Bogota",
	},
	"SKIB": {
		ICAO:      "SKIB",
		IATA:      "IBE",
		Name:      "Perales Airport",
		City:      "Ibague",
		State:     "Tolima",
		Country:   "CO",
		Elevation: 2999,
		Latitude:  4.42161,
		Longitude: -75.1333,
		Timezone:  "America/Bogota",
	},
	"SKIG": {
		ICAO:      "SKIG",
		IATA:      "IGO",
		Name:      "Chigorodo Airport",
		City:      "Chigorodo",
		State:     "Antioquia",
		Country:   "CO",
		Elevation: 110,
		Latitude:  7.68038,
		Longitude: -76.6865,
		Timezone:  "America/Bogota",
	},
	"SKIO": {
		ICAO:      "SKIO",
		IATA:      "MMP",
		Name:      "Cicuco Airport",
		City:      "Limon",
		State:     "Bolivar",
		Country:   "CO",
		Elevation: 65,
		Latitude:  9.26878,
		Longitude: -74.6523,
		Timezone:  "America/Bogota",
	},
	"SKIP": {
		ICAO:      "SKIP",
		IATA:      "IPI",
		Name:      "San Luis Airport",
		City:      "Ipiales",
		State:     "Narino",
		Country:   "CO",
		Elevation: 9765,
		Latitude:  0.861925,
		Longitude: -77.6718,
		Timezone:  "America/Bogota",
	},
	"SKLA": {
		ICAO:      "SKLA",
		Name:      "Malaga Airport",
		City:      "Malaga",
		State:     "Santander",
		Country:   "CO",
		Elevation: 6710,
		Latitude:  6.70472,
		Longitude: -72.7303,
		Timezone:  "America/Bogota",
	},
	"SKLC": {
		ICAO:      "SKLC",
		IATA:      "APO",
		Name:      "Antonio Roldan Betancourt Airport",
		City:      "Carepa",
		State:     "Antioquia",
		Country:   "CO",
		Elevation: 46,
		Latitude:  7.81196,
		Longitude: -76.7164,
		Timezone:  "America/Bogota",
	},
	"SKLG": {
		ICAO:      "SKLG",
		IATA:      "LQM",
		Name:      "Caucaya Airport",
		City:      "Puerto Leguizamo",
		State:     "Putumayo",
		Country:   "CO",
		Elevation: 573,
		Latitude:  -0.182278,
		Longitude: -74.7708,
		Timezone:  "America/Bogota",
	},
	"SKLM": {
		ICAO:      "SKLM",
		IATA:      "MCJ",
		Name:      "Jorge Isaac Airport",
		City:      "La Mina-Maicao",
		State:     "La-Guajira",
		Country:   "CO",
		Elevation: 281,
		Latitude:  11.2325,
		Longitude: -72.4901,
		Timezone:  "America/Bogota",
	},
	"SKLP": {
		ICAO:      "SKLP",
		IATA:      "LPD",
		Name:      "La Pedrera Airport",
		City:      "La Pedrera",
		State:     "Amazonas",
		Country:   "CO",
		Elevation: 590,
		Latitude:  -1.32861,
		Longitude: -69.5797,
		Timezone:  "America/Bogota",
	},
	"SKLT": {
		ICAO:      "SKLT",
		IATA:      "LET",
		Name:      "Alfredo Vasquez Cobo International Airport",
		City:      "Leticia",
		State:     "Amazonas",
		Country:   "CO",
		Elevation: 277,
		Latitude:  -4.19355,
		Longitude: -69.9432,
		Timezone:  "America/Bogota",
	},
	"SKMA": {
		ICAO:      "SKMA",
		Name:      "Madrid Air Base",
		City:      "Madrid",
		State:     "Cundinamarca",
		Country:   "CO",
		Elevation: 8325,
		Latitude:  4.7278,
		Longitude: -74.2754,
		Timezone:  "America/Bogota",
	},
	"SKMD": {
		ICAO:      "SKMD",
		IATA:      "EOH",
		Name:      "Enrique Olaya Herrera Airport",
		City:      "Medellin",
		State:     "Antioquia",
		Country:   "CO",
		Elevation: 4949,
		Latitude:  6.220549,
		Longitude: -75.590582,
		Timezone:  "America/Bogota",
	},
	"SKME": {
		ICAO:      "SKME",
		Name:      "Melgar Air Base",
		City:      "Melgar",
		State:     "Cundinamarca",
		Country:   "CO",
		Elevation: 1028,
		Latitude:  4.21644,
		Longitude: -74.635,
		Timezone:  "America/Bogota",
	},
	"SKMF": {
		ICAO:      "SKMF",
		IATA:      "MFS",
		Name:      "Miraflores Airport",
		City:      "Miraflores",
		State:     "Guaviare",
		Country:   "CO",
		Elevation: 730,
		Latitude:  1.35,
		Longitude: -71.9444,
		Timezone:  "America/Bogota",
	},
	"SKMG": {
		ICAO:      "SKMG",
		IATA:      "MGN",
		Name:      "Baracoa Airport",
		City:      "Magangue",
		State:     "Bolivar",
		Country:   "CO",
		Elevation: 178,
		Latitude:  9.28474,
		Longitude: -74.8461,
		Timezone:  "America/Bogota",
	},
	"SKMJ": {
		ICAO:      "SKMJ",
		Name:      "Maicao Airport",
		City:      "Maicao",
		State:     "La-Guajira",
		Country:   "CO",
		Elevation: 160,
		Latitude:  11.3899,
		Longitude: -72.239197,
		Timezone:  "America/Bogota",
	},
	"SKML": {
		ICAO:      "SKML",
		IATA:      "MTB",
		Name:      "Montelibano Airport",
		City:      "Montelibano",
		State:     "Cordoba",
		Country:   "CO",
		Elevation: 160,
		Latitude:  7.97174,
		Longitude: -75.4325,
		Timezone:  "America/Bogota",
	},
	"SKMN": {
		ICAO:      "SKMN",
		Name:      "Casanare Airport",
		City:      "Mani",
		State:     "Casanare",
		Country:   "CO",
		Elevation: 600,
		Latitude:  4.83333,
		Longitude: -72.2667,
		Timezone:  "America/Bogota",
	},
	"SKMP": {
		ICAO:      "SKMP",
		IATA:      "MMP",
		Name:      "San Bernardo Airport",
		City:      "Mompos",
		State:     "Magdalena",
		Country:   "CO",
		Elevation: 60,
		Latitude:  9.25872,
		Longitude: -74.438,
		Timezone:  "America/Bogota",
	},
	"SKMR": {
		ICAO:      "SKMR",
		IATA:      "MTR",
		Name:      "Los Garzones Airport",
		City:      "Monteria",
		State:     "Cordoba",
		Country:   "CO",
		Elevation: 36,
		Latitude:  8.82374,
		Longitude: -75.8258,
		Timezone:  "America/Bogota",
	},
	"SKMU": {
		ICAO:      "SKMU",
		IATA:      "MVP",
		Name:      "Fabio Alberto Leon Bentley Airport",
		City:      "Mitu",
		State:     "Vaupes",
		Country:   "CO",
		Elevation: 680,
		Latitude:  1.25366,
		Longitude: -70.2339,
		Timezone:  "America/Bogota",
	},
	"SKMZ": {
		ICAO:      "SKMZ",
		IATA:      "MZL",
		Name:      "La Nubia Airport",
		City:      "Manizales",
		State:     "Caldas",
		Country:   "CO",
		Elevation: 6871,
		Latitude:  5.0296,
		Longitude: -75.4647,
		Timezone:  "America/Bogota",
	},
	"SKNC": {
		ICAO:      "SKNC",
		IATA:      "NCI",
		Name:      "Necocli Airport",
		City:      "Necocli",
		State:     "Antioquia",
		Country:   "CO",
		Elevation: 19,
		Latitude:  8.45,
		Longitude: -76.7833,
		Timezone:  "America/Bogota",
	},
	"SKNQ": {
		ICAO:      "SKNQ",
		IATA:      "NQU",
		Name:      "Reyes Murillo Airport",
		City:      "Nuqui",
		State:     "Choco",
		Country:   "CO",
		Elevation: 12,
		Latitude:  5.6964,
		Longitude: -77.2806,
		Timezone:  "America/Bogota",
	},
	"SKNV": {
		ICAO:      "SKNV",
		IATA:      "NVA",
		Name:      "Benito Salas Airport",
		City:      "Neiva",
		State:     "Huila",
		Country:   "CO",
		Elevation: 1464,
		Latitude:  2.95015,
		Longitude: -75.294,
		Timezone:  "America/Bogota",
	},
	"SKOC": {
		ICAO:      "SKOC",
		IATA:      "OCV",
		Name:      "Aguas Claras Airport",
		City:      "Ocana",
		State:     "Cesar",
		Country:   "CO",
		Elevation: 3850,
		Latitude:  8.31506,
		Longitude: -73.3583,
		Timezone:  "America/Bogota",
	},
	"SKOE": {
		ICAO:      "SKOE",
		IATA:      "ORC",
		Name:      "Orocue Airport",
		City:      "Orocue",
		State:     "Casanare",
		Country:   "CO",
		Elevation: 434,
		Latitude:  4.79222,
		Longitude: -71.3564,
		Timezone:  "America/Bogota",
	},
	"SKOL": {
		ICAO:      "SKOL",
		Name:      "Puerto Lopez Airport",
		City:      "Puerto Lopez",
		State:     "Meta",
		Country:   "CO",
		Elevation: 599,
		Latitude:  4.089869,
		Longitude: -72.976483,
		Timezone:  "America/Bogota",
	},
	"SKOR": {
		ICAO:      "SKOR",
		Name:      "Orito Airport",
		City:      "Orito",
		State:     "Putumayo",
		Country:   "CO",
		Elevation: 1132,
		Latitude:  0.669444,
		Longitude: -76.88,
		Timezone:  "America/Bogota",
	},
	"SKOT": {
		ICAO:      "SKOT",
		IATA:      "OTU",
		Name:      "Otu Airport",
		City:      "El Rhin",
		State:     "Antioquia",
		Country:   "CO",
		Elevation: 2060,
		Latitude:  7.01037,
		Longitude: -74.7155,
		Timezone:  "America/Bogota",
	},
	"SKPA": {
		ICAO:      "SKPA",
		IATA:      "RON",
		Name:      "Juan Jose Rondon Airport",
		City:      "Paipa",
		State:     "Boyaca",
		Country:   "CO",
		Elevation: 8205,
		Latitude:  5.76454,
		Longitude: -73.1054,
		Timezone:  "America/Bogota",
	},
	"SKPB": {
		ICAO:      "SKPB",
		Name:      "Puerto Bolivar Airport",
		City:      "Portete",
		State:     "La-Guajira",
		Country:   "CO",
		Elevation: 90,
		Latitude:  12.2215,
		Longitude: -71.9848,
		Timezone:  "America/Bogota",
	},
	"SKPC": {
		ICAO:      "SKPC",
		IATA:      "PCR",
		Name:      "German Olano Airport",
		City:      "Puerto Carreno",
		State:     "Vichada",
		Country:   "CO",
		Elevation: 177,
		Latitude:  6.18472,
		Longitude: -67.4932,
		Timezone:  "America/Bogota",
	},
	"SKPD": {
		ICAO:      "SKPD",
		IATA:      "PDA",
		Name:      "Obando Airport",
		City:      "Puerto Inirida",
		State:     "Guainia",
		Country:   "CO",
		Elevation: 460,
		Latitude:  3.85353,
		Longitude: -67.9062,
		Timezone:  "America/Bogota",
	},
	"SKPE": {
		ICAO:      "SKPE",
		IATA:      "PEI",
		Name:      "Matecana International Airport",
		City:      "Pereira",
		State:     "Risaralda",
		Country:   "CO",
		Elevation: 4416,
		Latitude:  4.81267,
		Longitude: -75.7395,
		Timezone:  "America/Bogota",
	},
	"SKPG": {
		ICAO:      "SKPG",
		Name:      "Puerto Gaitan Airport",
		City:      "Puerto Gaitan",
		State:     "Meta",
		Country:   "CO",
		Elevation: 510,
		Latitude:  4.30444,
		Longitude: -72.0844,
		Timezone:  "America/Bogota",
	},
	"SKPI": {
		ICAO:      "SKPI",
		IATA:      "PTX",
		Name:      "Pitalito Airport",
		City:      "Pitalito",
		State:     "Huila",
		Country:   "CO",
		Elevation: 4212,
		Latitude:  1.85777,
		Longitude: -76.0857,
		Timezone:  "America/Bogota",
	},
	"SKPL": {
		ICAO:      "SKPL",
		IATA:      "PLT",
		Name:      "Plato Airport",
		City:      "Plato",
		State:     "Magdalena",
		Country:   "CO",
		Elevation: 69,
		Latitude:  9.8,
		Longitude: -74.7833,
		Timezone:  "America/Bogota",
	},
	"SKPN": {
		ICAO:      "SKPN",
		IATA:      "NAR",
		Name:      "Puerto Nare Airport",
		City:      "Armenia",
		State:     "Antioquia",
		Country:   "CO",
		Elevation: 476,
		Latitude:  6.21002,
		Longitude: -74.5906,
		Timezone:  "America/Bogota",
	},
	"SKPP": {
		ICAO:      "SKPP",
		IATA:      "PPN",
		Name:      "Guillermo Leon Valencia Airport",
		City:      "Popayan",
		State:     "Cauca",
		Country:   "CO",
		Elevation: 5687,
		Latitude:  2.4544,
		Longitude: -76.6093,
		Timezone:  "America/Bogota",
	},
	"SKPQ": {
		ICAO:      "SKPQ",
		IATA:      "PQE",
		Name:      "German Olano Air Base",
		City:      "La Dorada",
		State:     "Cundinamarca",
		Country:   "CO",
		Elevation: 566,
		Latitude:  5.48361,
		Longitude: -74.6574,
		Timezone:  "America/Bogota",
	},
	"SKPR": {
		ICAO:      "SKPR",
		IATA:      "PBE",
		Name:      "Puerto Berrio Airport",
		City:      "Puerto Berrio",
		State:     "Antioquia",
		Country:   "CO",
		Elevation: 445,
		Latitude:  6.46034,
		Longitude: -74.4105,
		Timezone:  "America/Bogota",
	},
	"SKPS": {
		ICAO:      "SKPS",
		IATA:      "PSO",
		Name:      "Antonio Narino Airport",
		City:      "Pasto",
		State:     "Narino",
		Country:   "CO",
		Elevation: 5951,
		Latitude:  1.39625,
		Longitude: -77.2915,
		Timezone:  "America/Bogota",
	},
	"SKPV": {
		ICAO:      "SKPV",
		IATA:      "PVA",
		Name:      "El Embrujo Airport",
		City:      "Providencia",
		State:     "San-Andres-y-Providencia",
		Country:   "CO",
		Elevation: 10,
		Latitude:  13.3569,
		Longitude: -81.3583,
		Timezone:  "America/Bogota",
	},
	"SKPZ": {
		ICAO:      "SKPZ",
		IATA:      "PZA",
		Name:      "Paz De Ariporo Airport",
		City:      "Paz De Ariporo",
		State:     "Casanare",
		Country:   "CO",
		Elevation: 900,
		Latitude:  5.87615,
		Longitude: -71.8866,
		Timezone:  "America/Bogota",
	},
	"SKQU": {
		ICAO:      "SKQU",
		IATA:      "MQU",
		Name:      "Jose Celestino Mutis Airport",
		City:      "Mariquita",
		State:     "Tolima",
		Country:   "CO",
		Elevation: 1531,
		Latitude:  5.21256,
		Longitude: -74.8836,
		Timezone:  "America/Bogota",
	},
	"SKRG": {
		ICAO:      "SKRG",
		IATA:      "MDE",
		Name:      "Jose Maria Cordova International Airport",
		City:      "Rionegro",
		State:     "Antioquia",
		Country:   "CO",
		Elevation: 6955,
		Latitude:  6.16454,
		Longitude: -75.4231,
		Timezone:  "America/Bogota",
	},
	"SKRH": {
		ICAO:      "SKRH",
		IATA:      "RCH",
		Name:      "Almirante Padilla Airport",
		City:      "Riohacha",
		State:     "La-Guajira",
		Country:   "CO",
		Elevation: 43,
		Latitude:  11.5262,
		Longitude: -72.926,
		Timezone:  "America/Bogota",
	},
	"SKSA": {
		ICAO:      "SKSA",
		IATA:      "RVE",
		Name:      "Los Colonizadores Airport",
		City:      "Saravena",
		State:     "Arauca",
		Country:   "CO",
		Elevation: 672,
		Latitude:  6.951389,
		Longitude: -71.856944,
		Timezone:  "America/Bogota",
	},
	"SKSF": {
		ICAO:      "SKSF",
		Name:      "Santa Fe De Antioquia Airport",
		State:     "Antioquia",
		Country:   "CO",
		Latitude:  6.500556,
		Longitude: -75.8225,
		Timezone:  "America/Bogota",
	},
	"SKSG": {
		ICAO:      "SKSG",
		Name:      "San Gil Airport",
		City:      "San Gil",
		State:     "Santander",
		Country:   "CO",
		Elevation: 5498,
		Latitude:  6.58444,
		Longitude: -73.1283,
		Timezone:  "America/Bogota",
	},
	"SKSI": {
		ICAO:      "SKSI",
		Name:      "Santiago I Airport",
		City:      "Mani",
		State:     "Casanare",
		Country:   "CO",
		Elevation: 573,
		Latitude:  4.73214,
		Longitude: -72.323196,
		Timezone:  "America/Bogota",
	},
	"SKSJ": {
		ICAO:      "SKSJ",
		IATA:      "SJE",
		Name:      "Jorge E. Gonzalez Torres Airport",
		City:      "San Jose Del Guaviare",
		State:     "Guaviare",
		Country:   "CO",
		Elevation: 605,
		Latitude:  2.57969,
		Longitude: -72.6394,
		Timezone:  "America/Bogota",
	},
	"SKSM": {
		ICAO:      "SKSM",
		IATA:      "SMR",
		Name:      "Simon Bolivar International Airport",
		City:      "Santa Marta",
		State:     "Magdalena",
		Country:   "CO",
		Elevation: 22,
		Latitude:  11.1196,
		Longitude: -74.2306,
		Timezone:  "America/Bogota",
	},
	"SKSO": {
		ICAO:      "SKSO",
		IATA:      "SOX",
		Name:      "Alberto Lleras Camargo Airport",
		City:      "Sogamoso",
		State:     "Boyaca",
		Country:   "CO",
		Elevation: 8155,
		Latitude:  5.67732,
		Longitude: -72.9703,
		Timezone:  "America/Bogota",
	},
	"SKSP": {
		ICAO:      "SKSP",
		IATA:      "ADZ",
		Name:      "Gustavo Rojas Pinilla International Airport",
		City:      "San Andres",
		State:     "San-Andres-y-Providencia",
		Country:   "CO",
		Elevation: 19,
		Latitude:  12.5836,
		Longitude: -81.7112,
		Timezone:  "America/Bogota",
	},
	"SKSV": {
		ICAO:      "SKSV",
		IATA:      "SVI",
		Name:      "Eduardo Falla Solano Airport",
		City:      "San Vicente Del Caguan",
		State:     "Caqueta",
		Country:   "CO",
		Elevation: 920,
		Latitude:  2.15217,
		Longitude: -74.7663,
		Timezone:  "America/Bogota",
	},
	"SKTA": {
		ICAO:      "SKTA",
		IATA:      "TAU",
		Name:      "Tauramena Airport",
		City:      "Tauramena",
		State:     "Casanare",
		Country:   "CO",
		Elevation: 1462,
		Latitude:  5.01281,
		Longitude: -72.7424,
		Timezone:  "America/Bogota",
	},
	"SKTB": {
		ICAO:      "SKTB",
		IATA:      "TIB",
		Name:      "Tibu Airport",
		City:      "Tibu",
		State:     "Norte-de-Santander",
		Country:   "CO",
		Elevation: 194,
		Latitude:  8.63152,
		Longitude: -72.7304,
		Timezone:  "America/Bogota",
	},
	"SKTD": {
		ICAO:      "SKTD",
		IATA:      "TDA",
		Name:      "Trinidad Airport",
		City:      "Trinidad",
		State:     "Casanare",
		Country:   "CO",
		Elevation: 649,
		Latitude:  5.43278,
		Longitude: -71.6625,
		Timezone:  "America/Bogota",
	},
	"SKTI": {
		ICAO:      "SKTI",
		Name:      "Tolemaida Air Base",
		City:      "Las Ceibas",
		State:     "Cundinamarca",
		Country:   "CO",
		Elevation: 1617,
		Latitude:  4.2448,
		Longitude: -74.6499,
		Timezone:  "America/Bogota",
	},
	"SKTJ": {
		ICAO:      "SKTJ",
		Name:      "Tunja Airport",
		City:      "Tunja",
		State:     "Boyaca",
		Country:   "CO",
		Elevation: 8940,
		Latitude:  5.54138,
		Longitude: -73.3445,
		Timezone:  "America/Bogota",
	},
	"SKTL": {
		ICAO:      "SKTL",
		IATA:      "TLU",
		Name:      "Tolu Airport",
		City:      "Tolu",
		State:     "Sucre",
		Country:   "CO",
		Elevation: 16,
		Latitude:  9.50945,
		Longitude: -75.5854,
		Timezone:  "America/Bogota",
	},
	"SKTM": {
		ICAO:      "SKTM",
		IATA:      "TME",
		Name:      "Gustavo Vargas Airport",
		City:      "Tame",
		State:     "Arauca",
		Country:   "CO",
		Elevation: 1050,
		Latitude:  6.45108,
		Longitude: -71.7603,
		Timezone:  "America/Bogota",
	},
	"SKTQ": {
		ICAO:      "SKTQ",
		IATA:      "TQS",
		Name:      "Tres Esquinas Air Base",
		City:      "Tres Esquinas",
		State:     "Caqueta",
		Country:   "CO",
		Elevation: 585,
		Latitude:  0.7459,
		Longitude: -75.234,
		Timezone:  "America/Bogota",
	},
	"SKTU": {
		ICAO:      "SKTU",
		IATA:      "TRB",
		Name:      "Gonzalo Mejia Airport",
		City:      "Turbo",
		State:     "Antioquia",
		Country:   "CO",
		Elevation: 6,
		Latitude:  8.07453,
		Longitude: -76.7415,
		Timezone:  "America/Bogota",
	},
	"SKUA": {
		ICAO:      "SKUA",
		Name:      "Marandua Air Base",
		City:      "Marandua",
		State:     "Vichada",
		Country:   "CO",
		Elevation: 285,
		Latitude:  5.52448,
		Longitude: -68.6856,
		Timezone:  "America/Bogota",
	},
	"SKUC": {
		ICAO:      "SKUC",
		IATA:      "AUC",
		Name:      "Santiago Perez Airport",
		City:      "Arauca",
		State:     "Arauca",
		Country:   "CO",
		Elevation: 420,
		Latitude:  7.06888,
		Longitude: -70.7369,
		Timezone:  "America/Bogota",
	},
	"SKUI": {
		ICAO:      "SKUI",
		IATA:      "UIB",
		Name:      "El Carano Airport",
		City:      "Quibdo",
		State:     "Choco",
		Country:   "CO",
		Elevation: 204,
		Latitude:  5.69076,
		Longitude: -76.6412,
		Timezone:  "America/Bogota",
	},
	"SKUL": {
		ICAO:      "SKUL",
		IATA:      "ULQ",
		Name:      "Farfan Airport",
		City:      "Tulua",
		State:     "Valle-del-Cauca",
		Country:   "CO",
		Elevation: 3132,
		Latitude:  4.08836,
		Longitude: -76.2351,
		Timezone:  "America/Bogota",
	},
	"SKUR": {
		ICAO:      "SKUR",
		IATA:      "URR",
		Name:      "Urrao Airport",
		City:      "Urrao",
		State:     "Antioquia",
		Country:   "CO",
		Elevation: 6090,
		Latitude:  6.32883,
		Longitude: -76.1425,
		Timezone:  "America/Bogota",
	},
	"SKVG": {
		ICAO:      "SKVG",
		IATA:      "VGZ",
		Name:      "Villa Garzon Airport",
		City:      "Villa Garzon",
		State:     "Putumayo",
		Country:   "CO",
		Elevation: 1248,
		Latitude:  0.981944,
		Longitude: -76.604167,
		Timezone:  "America/Bogota",
	},
	"SKVL": {
		ICAO:      "SKVL",
		IATA:      "PYA",
		Name:      "Velasquez Airport",
		City:      "Velasquez",
		State:     "Boyaca",
		Country:   "CO",
		Elevation: 566,
		Latitude:  5.93904,
		Longitude: -74.457,
		Timezone:  "America/Bogota",
	},
	"SKVN": {
		ICAO:      "SKVN",
		Name:      "Villanueva Airport",
		City:      "Villanueva",
		State:     "Casanare",
		Country:   "CO",
		Elevation: 1002,
		Latitude:  4.62472,
		Longitude: -72.9469,
		Timezone:  "America/Bogota",
	},
	"SKVP": {
		ICAO:      "SKVP",
		IATA:      "VUP",
		Name:      "Alfonso Lopez Pumarejo Airport",
		City:      "Valledupar",
		State:     "Cesar",
		Country:   "CO",
		Elevation: 483,
		Latitude:  10.435,
		Longitude: -73.2495,
		Timezone:  "America/Bogota",
	},
	"SKVV": {
		ICAO:      "SKVV",
		IATA:      "VVC",
		Name:      "Vanguardia Airport",
		City:      "Villavicencio",
		State:     "Meta",
		Country:   "CO",
		Elevation: 1394,
		Latitude:  4.16787,
		Longitude: -73.6138,
		Timezone:  "America/Bogota",
	},
	"SKYA": {
		ICAO:      "SKYA",
		IATA:      "AYG",
		Name:      "Yaguara Airport",
		City:      "San Vicente Del Caguan",
		State:     "Caqueta",
		Country:   "CO",
		Elevation: 812,
		Latitude:  1.54417,
		Longitude: -73.9333,
		Timezone:  "America/Bogota",
	},
	"SKYP": {
		ICAO:      "SKYP",
		IATA:      "EYP",
		Name:      "El Yopal Airport",
		City:      "El Yopal",
		State:     "Casanare",
		Country:   "CO",
		Elevation: 1028,
		Latitude:  5.31911,
		Longitude: -72.384,
		Timezone:  "America/Bogota",
	},
	"SLAB": {
		ICAO:      "SLAB",
		Name:      "Abopo Airport",
		City:      "Cordillera",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 2752,
		Latitude:  -18.6319999695,
		Longitude: -62.939201355,
		Timezone:  "America/La_Paz",
	},
	"SLAG": {
		ICAO:      "SLAG",
		IATA:      "MHW",
		Name:      "Monteagudo Airport",
		City:      "El Banado",
		State:     "Chuquisaca",
		Country:   "BO",
		Elevation: 3674,
		Latitude:  -19.8269996643,
		Longitude: -63.9609985352,
		Timezone:  "America/La_Paz",
	},
	"SLAL": {
		ICAO:      "SLAL",
		IATA:      "SRE",
		Name:      "AlcantarÃ\u00ad International Airport",
		City:      "Sucre",
		State:     "Chuquisaca",
		Country:   "BO",
		Elevation: 10184,
		Latitude:  -19.24627,
		Longitude: -65.15104,
		Timezone:  "America/La_Paz",
	},
	"SLAM": {
		ICAO:      "SLAM",
		Name:      "Arampampa Airport",
		City:      "Gral. Ballivian",
		State:     "Potosi",
		Country:   "BO",
		Elevation: 10250,
		Latitude:  -17.8833332062,
		Longitude: -66.0999984741,
		Timezone:  "America/La_Paz",
	},
	"SLAN": {
		ICAO:      "SLAN",
		Name:      "Angora Airport",
		City:      "Ballivian",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 731,
		Latitude:  -14.432299614,
		Longitude: -65.7687988281,
		Timezone:  "America/La_Paz",
	},
	"SLAP": {
		ICAO:      "SLAP",
		IATA:      "APB",
		Name:      "Apolo Airport",
		City:      "Apolo",
		State:     "La-Paz",
		Country:   "BO",
		Elevation: 4639,
		Latitude:  -14.7356004715,
		Longitude: -68.4119033813,
		Timezone:  "America/La_Paz",
	},
	"SLAQ": {
		ICAO:      "SLAQ",
		Name:      "Aiquile Airport",
		City:      "Campero",
		State:     "Cochabamba",
		Country:   "BO",
		Elevation: 7208,
		Latitude:  -18.2154006958,
		Longitude: -65.1877975464,
		Timezone:  "America/La_Paz",
	},
	"SLAS": {
		ICAO:      "SLAS",
		IATA:      "ASC",
		Name:      "Ascencion De Guarayos Airport",
		City:      "Ascension de Guarayos",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 807,
		Latitude:  -15.9302997589,
		Longitude: -63.1567001343,
		Timezone:  "America/La_Paz",
	},
	"SLBA": {
		ICAO:      "SLBA",
		IATA:      "BVL",
		Name:      "Baures Airport",
		City:      "Baures",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 470,
		Latitude:  -13.5833330154,
		Longitude: -63.5833320618,
		Timezone:  "America/La_Paz",
	},
	"SLBB": {
		ICAO:      "SLBB",
		Name:      "Bulo Bulo Airport",
		City:      "Carrasco",
		State:     "Cochabamba",
		Country:   "BO",
		Elevation: 886,
		Latitude:  -17.2573337555,
		Longitude: -64.3666687012,
		Timezone:  "America/La_Paz",
	},
	"SLBF": {
		ICAO:      "SLBF",
		Name:      "Blanca Flor Airport",
		City:      "Madre De Dios",
		State:     "Pando",
		Country:   "BO",
		Elevation: 656,
		Latitude:  -11.7833337784,
		Longitude: -66.8833312988,
		Timezone:  "America/La_Paz",
	},
	"SLBH": {
		ICAO:      "SLBH",
		Name:      "Buena Hora Airport",
		City:      "Ballivian",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 656,
		Latitude:  -12.1333332062,
		Longitude: -66.5333328247,
		Timezone:  "America/La_Paz",
	},
	"SLBJ": {
		ICAO:      "SLBJ",
		IATA:      "BJO",
		Name:      "Bermejo Airport",
		City:      "Bermejo",
		State:     "Tarija",
		Country:   "BO",
		Elevation: 1249,
		Latitude:  -22.7733001709,
		Longitude: -64.3128967285,
		Timezone:  "America/Argentina/Salta",
	},
	"SLBN": {
		ICAO:      "SLBN",
		Name:      "Bella Union Airport",
		City:      "Mamore",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 722,
		Latitude:  -13.6333332062,
		Longitude: -65.2833328247,
		Timezone:  "America/La_Paz",
	},
	"SLBR": {
		ICAO:      "SLBR",
		Name:      "Buen Retiro Itenez Airport",
		City:      "Itenes",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 774,
		Latitude:  -13.4166669846,
		Longitude: -63.4833335876,
		Timezone:  "America/La_Paz",
	},
	"SLBS": {
		ICAO:      "SLBS",
		Name:      "Bella Vista Itenez Airport",
		City:      "Itenes",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 774,
		Latitude:  -14.2677001953,
		Longitude: -64.2990036011,
		Timezone:  "America/La_Paz",
	},
	"SLBT": {
		ICAO:      "SLBT",
		Name:      "Buen Retiro Ballivian Airport",
		City:      "Ballivian",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 886,
		Latitude:  -13.8999996185,
		Longitude: -66.5500030518,
		Timezone:  "America/La_Paz",
	},
	"SLBV": {
		ICAO:      "SLBV",
		Name:      "Villa Vista Montano Airport",
		City:      "De Montano",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 519,
		Latitude:  -13.8500003815,
		Longitude: -66.3666687012,
		Timezone:  "America/La_Paz",
	},
	"SLCA": {
		ICAO:      "SLCA",
		IATA:      "CAM",
		Name:      "Camiri Airport",
		City:      "Camiri",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 2614,
		Latitude:  -20.0063991547,
		Longitude: -63.5278015137,
		Timezone:  "America/La_Paz",
	},
	"SLCB": {
		ICAO:      "SLCB",
		IATA:      "CBB",
		Name:      "Jorge Wilsterman International Airport",
		City:      "Cochabamba",
		State:     "Cochabamba",
		Country:   "BO",
		Elevation: 8360,
		Latitude:  -17.4211006165,
		Longitude: -66.1771011353,
		Timezone:  "America/La_Paz",
	},
	"SLCC": {
		ICAO:      "SLCC",
		Name:      "Copacabana Airport",
		City:      "Copacabana",
		State:     "La-Paz",
		Country:   "BO",
		Elevation: 12591,
		Latitude:  -16.1910991669,
		Longitude: -69.0955963135,
		Timezone:  "America/La_Paz",
	},
	"SLCD": {
		ICAO:      "SLCD",
		Name:      "Canada Airport",
		City:      "Gutierrez",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 1954,
		Latitude:  -20.0254001617,
		Longitude: -63.089099884,
		Timezone:  "America/La_Paz",
	},
	"SLCE": {
		ICAO:      "SLCE",
		Name:      "Cabezas Airport",
		City:      "Cordillera",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 1328,
		Latitude:  -18.7954006195,
		Longitude: -63.3003005981,
		Timezone:  "America/La_Paz",
	},
	"SLCG": {
		ICAO:      "SLCG",
		Name:      "Charagua Airport",
		City:      "Cordillera",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 2040,
		Latitude:  -19.7856006622,
		Longitude: -63.1884002686,
		Timezone:  "America/La_Paz",
	},
	"SLCH": {
		ICAO:      "SLCH",
		Name:      "Chimore Airport",
		City:      "Chimore",
		State:     "Cochabamba",
		Country:   "BO",
		Elevation: 875,
		Latitude:  -16.9900188446,
		Longitude: -65.1415023804,
		Timezone:  "America/La_Paz",
	},
	"SLCI": {
		ICAO:      "SLCI",
		Name:      "Caigua Airport",
		City:      "Gran Chaco",
		State:     "Tarija",
		Country:   "BO",
		Elevation: 1440,
		Latitude:  -21.1666660309,
		Longitude: -63.6333351135,
		Timezone:  "America/La_Paz",
	},
	"SLCL": {
		ICAO:      "SLCL",
		Name:      "Collpani Airport",
		City:      "Abaroa",
		State:     "Oruro",
		Country:   "BO",
		Elevation: 12185,
		Latitude:  -18.8916664124,
		Longitude: -66.7833328247,
		Timezone:  "America/La_Paz",
	},
	"SLCM": {
		ICAO:      "SLCM",
		Name:      "Camiare Airport",
		City:      "Yacuma",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 550,
		Latitude:  -14.0743331909,
		Longitude: -65.6503295898,
		Timezone:  "America/La_Paz",
	},
	"SLCN": {
		ICAO:      "SLCN",
		Name:      "Charana Airport",
		City:      "Charana",
		Country:   "BO",
		Elevation: 13339,
		Latitude:  -17.5946998596,
		Longitude: -69.4330978394,
		Timezone:  "America/La_Paz",
	},
	"SLCO": {
		ICAO:      "SLCO",
		IATA:      "CIJ",
		Name:      "Capitan Anibal Arab Airport",
		City:      "Cobija",
		State:     "Pando",
		Country:   "BO",
		Elevation: 889,
		Latitude:  -11.0403995514,
		Longitude: -68.7829971313,
		Timezone:  "America/La_Paz",
	},
	"SLCP": {
		ICAO:      "SLCP",
		IATA:      "CEP",
		Name:      "Concepcion Airport",
		City:      "Concepcion",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 1620,
		Latitude:  -16.138299942,
		Longitude: -62.0285987854,
		Timezone:  "America/La_Paz",
	},
	"SLCR": {
		ICAO:      "SLCR",
		Name:      "Comarapa Airport",
		City:      "M. Caballero",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 6186,
		Latitude:  -17.9136009216,
		Longitude: -64.5177993774,
		Timezone:  "America/La_Paz",
	},
	"SLCS": {
		ICAO:      "SLCS",
		Name:      "Cerdas Airport",
		City:      "Nor Lipez",
		State:     "Potosi",
		Country:   "BO",
		Elevation: 12640,
		Latitude:  -20.8083333333,
		Longitude: -66.4011111111,
		Timezone:  "America/La_Paz",
	},
	"SLCT": {
		ICAO:      "SLCT",
		Name:      "Choreti Bolivia South America Airport",
		City:      "Mamore",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 656,
		Latitude:  -13.5144996643,
		Longitude: -64.9652023315,
		Timezone:  "America/La_Paz",
	},
	"SLCU": {
		ICAO:      "SLCU",
		Name:      "Culpina Airport",
		City:      "Sud Cinti",
		State:     "Chuquisaca",
		Country:   "BO",
		Elevation: 9676,
		Latitude:  -20.8333339691,
		Longitude: -64.9666671753,
		Timezone:  "America/La_Paz",
	},
	"SLCV": {
		ICAO:      "SLCV",
		Name:      "Cavinas Airport",
		City:      "Ballivian",
		Country:   "BO",
		Elevation: 656,
		Latitude:  -12.5497999191,
		Longitude: -66.9124984741,
		Timezone:  "America/La_Paz",
	},
	"SLCX": {
		ICAO:      "SLCX",
		Name:      "Chive Airport",
		City:      "Iturralde",
		State:     "Pando",
		Country:   "BO",
		Elevation: 618,
		Latitude:  -12.3778333664,
		Longitude: -68.5676651001,
		Timezone:  "America/La_Paz",
	},
	"SLCY": {
		ICAO:      "SLCY",
		Name:      "Collpa Airport",
		City:      "A. Ibanez",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 1170,
		Latitude:  -17.5650005341,
		Longitude: -63.1747016907,
		Timezone:  "America/La_Paz",
	},
	"SLDA": {
		ICAO:      "SLDA",
		Name:      "Caranda Airport",
		City:      "Ichilo",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 1082,
		Latitude:  -17.5166664124,
		Longitude: -63.5166664124,
		Timezone:  "America/La_Paz",
	},
	"SLDN": {
		ICAO:      "SLDN",
		Name:      "El Desengano Airport",
		City:      "Yacuma",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 730,
		Latitude:  -14.2735996246,
		Longitude: -65.4400024414,
		Timezone:  "America/La_Paz",
	},
	"SLEA": {
		ICAO:      "SLEA",
		Name:      "El Cocal Airport",
		City:      "Yacuma",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 722,
		Latitude:  -13.5593996048,
		Longitude: -66.0790023804,
		Timezone:  "America/La_Paz",
	},
	"SLEC": {
		ICAO:      "SLEC",
		Name:      "El Cairo Airport",
		City:      "Yacuma",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 722,
		Latitude:  -13.7833337784,
		Longitude: -66.4000015259,
		Timezone:  "America/La_Paz",
	},
	"SLEE": {
		ICAO:      "SLEE",
		Name:      "El Escondido Airport",
		City:      "Tesoro",
		Country:   "BO",
		Elevation: 1246,
		Latitude:  -21.466999054,
		Longitude: -63.3527984619,
		Timezone:  "America/La_Paz",
	},
	"SLEF": {
		ICAO:      "SLEF",
		Name:      "El Triunfo Airport",
		City:      "Yacuma",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 486,
		Latitude:  -13.8833332062,
		Longitude: -65.7666702271,
		Timezone:  "America/La_Paz",
	},
	"SLEH": {
		ICAO:      "SLEH",
		Name:      "El Rancho Airport",
		City:      "Mamore",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 722,
		Latitude:  -13.0500001907,
		Longitude: -65.0333328247,
		Timezone:  "America/La_Paz",
	},
	"SLEI": {
		ICAO:      "SLEI",
		Name:      "Espino Airport",
		City:      "Cordillera",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 2313,
		Latitude:  -19.2999000549,
		Longitude: -63.2089996338,
		Timezone:  "America/La_Paz",
	},
	"SLEK": {
		ICAO:      "SLEK",
		Name:      "El Condor Airport",
		City:      "Tesoro",
		State:     "Tarija",
		Country:   "BO",
		Elevation: 1734,
		Latitude:  -21.7008342743,
		Longitude: -63.4568328857,
		Timezone:  "America/La_Paz",
	},
	"SLEO": {
		ICAO:      "SLEO",
		Name:      "El Paraiso Airport",
		City:      "Yacuma",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 722,
		Latitude:  -14.3833332062,
		Longitude: -65.8499984741,
		Timezone:  "America/La_Paz",
	},
	"SLEP": {
		ICAO:      "SLEP",
		Name:      "El Peru Airport",
		City:      "Yacuma",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 528,
		Latitude:  -14.6046667099,
		Longitude: -65.6183319092,
		Timezone:  "America/La_Paz",
	},
	"SLER": {
		ICAO:      "SLER",
		Name:      "Cerrillos Airport",
		City:      "Hernando Siles",
		State:     "Chuquisaca",
		Country:   "BO",
		Elevation: 3300,
		Latitude:  -20.0166664124,
		Longitude: -64.0333328247,
		Timezone:  "America/La_Paz",
	},
	"SLES": {
		ICAO:      "SLES",
		Name:      "Estalsa Airport",
		City:      "Poopo",
		State:     "Oruro",
		Country:   "BO",
		Elevation: 12460,
		Latitude:  -18.4500007629,
		Longitude: -66.9166641235,
		Timezone:  "America/La_Paz",
	},
	"SLET": {
		ICAO:      "SLET",
		IATA:      "SRZ",
		Name:      "El Trompillo Airport",
		City:      "Santa Cruz",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 1371,
		Latitude:  -17.8115997314,
		Longitude: -63.1715011597,
		Timezone:  "America/La_Paz",
	},
	"SLEV": {
		ICAO:      "SLEV",
		Name:      "El Salvador Airport",
		City:      "Ballivian",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 574,
		Latitude:  -14.4982995987,
		Longitude: -66.7629013062,
		Timezone:  "America/La_Paz",
	},
	"SLEZ": {
		ICAO:      "SLEZ",
		Name:      "La Esperanza Airport",
		City:      "Moxos",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 1100,
		Latitude:  -14.7666664124,
		Longitude: -65.5333328247,
		Timezone:  "America/La_Paz",
	},
	"SLFA": {
		ICAO:      "SLFA",
		Name:      "Fatima Airport",
		City:      "Ballivian",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 1000,
		Latitude:  -15.3333330154,
		Longitude: -66.8000030518,
		Timezone:  "America/La_Paz",
	},
	"SLFL": {
		ICAO:      "SLFL",
		Name:      "Florida Airport",
		City:      "Moira",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 600,
		Latitude:  -14.6141662598,
		Longitude: -61.1974983215,
		Timezone:  "America/La_Paz",
	},
	"SLFO": {
		ICAO:      "SLFO",
		Name:      "Flor De Oro Airport",
		City:      "Moxos",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 560,
		Latitude:  -13.5548000336,
		Longitude: -61.0041007996,
		Timezone:  "America/La_Paz",
	},
	"SLFR": {
		ICAO:      "SLFR",
		Name:      "Florencia Airport",
		City:      "Moxos",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 574,
		Latitude:  -15.6833333969,
		Longitude: -65.5666656494,
		Timezone:  "America/La_Paz",
	},
	"SLGP": {
		ICAO:      "SLGP",
		Name:      "Guirapembi Airport",
		City:      "Cordillera",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 1493,
		Latitude:  -18.0559997559,
		Longitude: -63.1631660461,
		Timezone:  "America/La_Paz",
	},
	"SLGY": {
		ICAO:      "SLGY",
		IATA:      "GYA",
		Name:      "Capitan de Av. Emilio Beltran Airport",
		City:      "Guayaramerin",
		State:     "Rondonia",
		Country:   "BO",
		Elevation: 557,
		Latitude:  -10.820599556,
		Longitude: -65.3455963135,
		Timezone:  "America/La_Paz",
	},
	"SLHA": {
		ICAO:      "SLHA",
		Name:      "Cachascani Airport",
		City:      "Itenes",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 722,
		Latitude:  -13.5,
		Longitude: -64.1999969482,
		Timezone:  "America/La_Paz",
	},
	"SLHC": {
		ICAO:      "SLHC",
		Name:      "Huacareta Airport",
		City:      "H. Silez",
		State:     "Chuquisaca",
		Country:   "BO",
		Elevation: 4668,
		Latitude:  -20.3908004761,
		Longitude: -64.0154037476,
		Timezone:  "America/La_Paz",
	},
	"SLHJ": {
		ICAO:      "SLHJ",
		IATA:      "BVK",
		Name:      "Huacaraje Airport",
		City:      "Itenes",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 846,
		Latitude:  -13.5500001907,
		Longitude: -63.7478981018,
		Timezone:  "America/La_Paz",
	},
	"SLHS": {
		ICAO:      "SLHS",
		Name:      "Chacobos Airport",
		City:      "Parapeti",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 1437,
		Latitude:  -19.5948009491,
		Longitude: -62.565700531,
		Timezone:  "America/La_Paz",
	},
	"SLIG": {
		ICAO:      "SLIG",
		Name:      "Inglaterra Airport",
		City:      "Ballivian",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 780,
		Latitude:  -13.4437999725,
		Longitude: -66.5059967041,
		Timezone:  "America/La_Paz",
	},
	"SLIN": {
		ICAO:      "SLIN",
		Name:      "Santa Catalina Airport",
		City:      "Moxos",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 722,
		Latitude:  -15.5,
		Longitude: -65.9166641235,
		Timezone:  "America/La_Paz",
	},
	"SLIP": {
		ICAO:      "SLIP",
		Name:      "Samaipata Airport",
		City:      "Florida",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 5600,
		Latitude:  -18.1833324432,
		Longitude: -63.8833351135,
		Timezone:  "America/La_Paz",
	},
	"SLIR": {
		ICAO:      "SLIR",
		Name:      "Ibori Airport",
		City:      "Itenes",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 775,
		Latitude:  -13.0500001907,
		Longitude: -63.6666679382,
		Timezone:  "America/La_Paz",
	},
	"SLIT": {
		ICAO:      "SLIT",
		Name:      "Itaguazurenda Airport",
		City:      "Cordillera",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 1598,
		Latitude:  -19.7775001526,
		Longitude: -63.0877990723,
		Timezone:  "America/La_Paz",
	},
	"SLIX": {
		ICAO:      "SLIX",
		Name:      "Ixiamas Airport",
		City:      "Iturralde",
		State:     "La-Paz",
		Country:   "BO",
		Elevation: 725,
		Latitude:  -13.7600002289,
		Longitude: -68.1368331909,
		Timezone:  "America/La_Paz",
	},
	"SLIY": {
		ICAO:      "SLIY",
		Name:      "Intiraymi Airport",
		City:      "La Joya",
		State:     "Oruro",
		Country:   "BO",
		Elevation: 12467,
		Latitude:  -17.8083324432,
		Longitude: -67.4433364868,
		Timezone:  "America/La_Paz",
	},
	"SLIZ": {
		ICAO:      "SLIZ",
		Name:      "Izozog Airport",
		City:      "Cordillera",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 1082,
		Latitude:  -18.6638336182,
		Longitude: -63.0023345947,
		Timezone:  "America/La_Paz",
	},
	"SLJA": {
		ICAO:      "SLJA",
		Name:      "Jatata Airport",
		City:      "Madre De Dios",
		State:     "Pando",
		Country:   "BO",
		Elevation: 575,
		Latitude:  -11.7728328705,
		Longitude: -67.2559967041,
		Timezone:  "America/La_Paz",
	},
	"SLJE": {
		ICAO:      "SLJE",
		IATA:      "SJS",
		Name:      "San Jose De Chiquitos Airport",
		City:      "San Jose de Chiquitos",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 974,
		Latitude:  -17.8307991028,
		Longitude: -60.7430992126,
		Timezone:  "America/La_Paz",
	},
	"SLJN": {
		ICAO:      "SLJN",
		Name:      "San Juan Estancias Airport",
		City:      "Cercado",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 820,
		Latitude:  -14.1666669846,
		Longitude: -64.7666702271,
		Timezone:  "America/La_Paz",
	},
	"SLJO": {
		ICAO:      "SLJO",
		IATA:      "SJB",
		Name:      "San Joaquin Airport",
		City:      "San Joaquin",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 456,
		Latitude:  -13.0528001785,
		Longitude: -64.6616973877,
		Timezone:  "America/La_Paz",
	},
	"SLJS": {
		ICAO:      "SLJS",
		Name:      "Josuani Airport",
		City:      "Yacuma",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 498,
		Latitude:  -13.0833330154,
		Longitude: -66.1666641235,
		Timezone:  "America/La_Paz",
	},
	"SLJV": {
		ICAO:      "SLJV",
		IATA:      "SJV",
		Name:      "San Javier Airport",
		City:      "San Javier",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 1702,
		Latitude:  -16.2707996368,
		Longitude: -62.4702987671,
		Timezone:  "America/La_Paz",
	},
	"SLLB": {
		ICAO:      "SLLB",
		Name:      "Las Brizas Airport",
		City:      "Yacuma",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 722,
		Latitude:  -14.5166664124,
		Longitude: -65.7666702271,
		Timezone:  "America/La_Paz",
	},
	"SLLH": {
		ICAO:      "SLLH",
		Name:      "Lago Huachi Airport",
		City:      "Itenes",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 700,
		Latitude:  -14.0166664124,
		Longitude: -63.5333328247,
		Timezone:  "America/La_Paz",
	},
	"SLLI": {
		ICAO:      "SLLI",
		Name:      "La India Airport",
		City:      "Ballivian",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 540,
		Latitude:  -13.5738334656,
		Longitude: -66.8356704712,
		Timezone:  "America/La_Paz",
	},
	"SLLJ": {
		ICAO:      "SLLJ",
		Name:      "Laja Airport",
		City:      "Los Andes",
		State:     "La-Paz",
		Country:   "BO",
		Elevation: 12103,
		Latitude:  -16.5338993073,
		Longitude: -68.3005981445,
		Timezone:  "America/La_Paz",
	},
	"SLLL": {
		ICAO:      "SLLL",
		Name:      "Laguna Loa Airport",
		City:      "Ballivian",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 720,
		Latitude:  -14.3000001907,
		Longitude: -66.7833328247,
		Timezone:  "America/La_Paz",
	},
	"SLLP": {
		ICAO:      "SLLP",
		IATA:      "LPB",
		Name:      "El Alto International Airport",
		City:      "La Paz / El Alto",
		State:     "La-Paz",
		Country:   "BO",
		Elevation: 13355,
		Latitude:  -16.513299942,
		Longitude: -68.1922988892,
		Timezone:  "America/La_Paz",
	},
	"SLLQ": {
		ICAO:      "SLLQ",
		Name:      "Copaquilla Airport",
		City:      "L. Cabrera",
		State:     "Oruro",
		Country:   "BO",
		Elevation: 12150,
		Latitude:  -19.3333339691,
		Longitude: -67.0833358765,
		Timezone:  "America/La_Paz",
	},
	"SLLS": {
		ICAO:      "SLLS",
		Name:      "Lagunillas Airport",
		City:      "Caraparicito",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 2950,
		Latitude:  -19.6000003815,
		Longitude: -63.6500015259,
		Timezone:  "America/La_Paz",
	},
	"SLLZ": {
		ICAO:      "SLLZ",
		Name:      "San Lorenzo Airport",
		City:      "Moxos",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 825,
		Latitude:  -15.3833332062,
		Longitude: -65.75,
		Timezone:  "America/La_Paz",
	},
	"SLMB": {
		ICAO:      "SLMB",
		Name:      "Mangabalito Airport",
		City:      "Velasco",
		State:     "La-Paz",
		Country:   "BO",
		Elevation: 1190,
		Latitude:  -13.7878332138,
		Longitude: -68.3546676636,
		Timezone:  "America/La_Paz",
	},
	"SLMD": {
		ICAO:      "SLMD",
		Name:      "Madidi Airport",
		City:      "Iturralde",
		State:     "La-Paz",
		Country:   "BO",
		Elevation: 500,
		Latitude:  -13.3833332062,
		Longitude: -68.6500015259,
		Timezone:  "America/La_Paz",
	},
	"SLME": {
		ICAO:      "SLME",
		Name:      "Mandeyepecua Airport",
		City:      "Cordillera",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 1338,
		Latitude:  -20.3980007172,
		Longitude: -63.140499115,
		Timezone:  "America/La_Paz",
	},
	"SLMG": {
		ICAO:      "SLMG",
		IATA:      "MGD",
		Name:      "Magdalena Airport",
		City:      "Magdalena",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 462,
		Latitude:  -13.2607483767,
		Longitude: -64.0607643127,
		Timezone:  "America/La_Paz",
	},
	"SLMM": {
		ICAO:      "SLMM",
		Name:      "Mora Airport",
		City:      "Velasco",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 600,
		Latitude:  -14.608499527,
		Longitude: -61.1918334961,
		Timezone:  "America/La_Paz",
	},
	"SLMN": {
		ICAO:      "SLMN",
		Name:      "Manuripi Airport",
		City:      "Manuripi",
		State:     "Pando",
		Country:   "BO",
		Elevation: 705,
		Latitude:  -11.7384996414,
		Longitude: -68.3546676636,
		Timezone:  "America/La_Paz",
	},
	"SLMX": {
		ICAO:      "SLMX",
		Name:      "Monos Arana Airport",
		City:      "Valle Grande",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 1220,
		Latitude:  -17.3333339691,
		Longitude: -64.1666641235,
		Timezone:  "America/La_Paz",
	},
	"SLMZ": {
		ICAO:      "SLMZ",
		Name:      "Mizque Airport",
		City:      "Mizque",
		State:     "Cochabamba",
		Country:   "BO",
		Elevation: 7030,
		Latitude:  -17.9309997559,
		Longitude: -65.3802032471,
		Timezone:  "America/La_Paz",
	},
	"SLNL": {
		ICAO:      "SLNL",
		Name:      "Canada Larga Airport",
		City:      "Chiquitos",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 1197,
		Latitude:  -17.7000007629,
		Longitude: -62.4166679382,
		Timezone:  "America/La_Paz",
	},
	"SLNO": {
		ICAO:      "SLNO",
		Name:      "Nuevo Mundo Airport",
		City:      "Yacuma",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 548,
		Latitude:  -14.4333333969,
		Longitude: -66.0666656494,
		Timezone:  "America/La_Paz",
	},
	"SLNV": {
		ICAO:      "SLNV",
		Name:      "Nieve Airport",
		City:      "Yacuma",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 780,
		Latitude:  -14.0166664124,
		Longitude: -65.8000030518,
		Timezone:  "America/La_Paz",
	},
	"SLOR": {
		ICAO:      "SLOR",
		IATA:      "ORU",
		Name:      "Juan Mendoza Airport",
		City:      "Oruro",
		State:     "Oruro",
		Country:   "BO",
		Elevation: 12152,
		Latitude:  -17.962600708,
		Longitude: -67.0762023926,
		Timezone:  "America/La_Paz",
	},
	"SLPA": {
		ICAO:      "SLPA",
		Name:      "Palmar Airport",
		City:      "Guanacos",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 1342,
		Latitude:  -18.9716663361,
		Longitude: -62.9753341675,
		Timezone:  "America/La_Paz",
	},
	"SLPF": {
		ICAO:      "SLPF",
		Name:      "Piso Firme Airport",
		City:      "Velasco",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 575,
		Latitude:  -13.6288337708,
		Longitude: -61.7386665344,
		Timezone:  "America/La_Paz",
	},
	"SLPG": {
		ICAO:      "SLPG",
		Name:      "Progreso Airport",
		City:      "Moxos",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 754,
		Latitude:  -14.1784000397,
		Longitude: -65.3756027222,
		Timezone:  "America/La_Paz",
	},
	"SLPI": {
		ICAO:      "SLPI",
		Name:      "Pitai Airport",
		City:      "Moxos",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 605,
		Latitude:  -15.8000001907,
		Longitude: -65.6166687012,
		Timezone:  "America/La_Paz",
	},
	"SLPL": {
		ICAO:      "SLPL",
		Name:      "Padilla Airport",
		City:      "J. Mendoza",
		State:     "Chuquisaca",
		Country:   "BO",
		Elevation: 6770,
		Latitude:  -19.2994003296,
		Longitude: -64.2940979004,
		Timezone:  "America/La_Paz",
	},
	"SLPM": {
		ICAO:      "SLPM",
		Name:      "Palmira Airport",
		City:      "Yacuma",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 530,
		Latitude:  -13.7314996719,
		Longitude: -66.4783325195,
		Timezone:  "America/La_Paz",
	},
	"SLPN": {
		ICAO:      "SLPN",
		Name:      "Porvenir Norte Airport",
		City:      "Velasco",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 595,
		Latitude:  -13.9873332977,
		Longitude: -61.545665741,
		Timezone:  "America/La_Paz",
	},
	"SLPO": {
		ICAO:      "SLPO",
		IATA:      "POI",
		Name:      "Capitan Nicolas Rojas Airport",
		City:      "Potosi",
		State:     "Potosi",
		Country:   "BO",
		Elevation: 12913,
		Latitude:  -19.5431003571,
		Longitude: -65.7237014771,
		Timezone:  "America/La_Paz",
	},
	"SLPR": {
		ICAO:      "SLPR",
		IATA:      "PUR",
		Name:      "Puerto Rico Airport",
		City:      "Puerto Rico/Manuripi",
		State:     "Pando",
		Country:   "BO",
		Elevation: 597,
		Latitude:  -11.1076631546,
		Longitude: -67.5511550903,
		Timezone:  "America/La_Paz",
	},
	"SLPS": {
		ICAO:      "SLPS",
		IATA:      "PSZ",
		Name:      "Capitan Av. Salvador Ogaya G. airport",
		City:      "Puerto Suarez",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 505,
		Latitude:  -18.9752998352,
		Longitude: -57.8205986023,
		Timezone:  "America/La_Paz",
	},
	"SLPW": {
		ICAO:      "SLPW",
		Name:      "El Porvenir Sur Airport",
		City:      "Luis Calvo",
		State:     "Chuquisaca",
		Country:   "BO",
		Elevation: 2238,
		Latitude:  -20.754699707,
		Longitude: -63.209400177,
		Timezone:  "America/La_Paz",
	},
	"SLPY": {
		ICAO:      "SLPY",
		Name:      "Piray Airport",
		City:      "Sanja Honda",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 1730,
		Latitude:  -18.3334999084,
		Longitude: -63.1925010681,
		Timezone:  "America/La_Paz",
	},
	"SLQN": {
		ICAO:      "SLQN",
		Name:      "Coquinal Airport",
		City:      "Ballivian",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 535,
		Latitude:  -13.1833333969,
		Longitude: -67,
		Timezone:  "America/La_Paz",
	},
	"SLRA": {
		ICAO:      "SLRA",
		IATA:      "SRD",
		Name:      "San Ramon Airport",
		City:      "San Ramon / Mamore",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 698,
		Latitude:  -13.2638998032,
		Longitude: -64.6038970947,
		Timezone:  "America/La_Paz",
	},
	"SLRB": {
		ICAO:      "SLRB",
		IATA:      "RBO",
		Name:      "Robore Airport",
		City:      "Robore",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 905,
		Latitude:  -18.3292007446,
		Longitude: -59.7649993896,
		Timezone:  "America/La_Paz",
	},
	"SLRF": {
		ICAO:      "SLRF",
		Name:      "Refugio Airport",
		City:      "Velasco",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 610,
		Latitude:  -14.7627000809,
		Longitude: -61.0320014954,
		Timezone:  "America/La_Paz",
	},
	"SLRH": {
		ICAO:      "SLRH",
		Name:      "Rancho Alegre Airport",
		City:      "Yacuma",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 754,
		Latitude:  -12.1666669846,
		Longitude: -65.7833328247,
		Timezone:  "America/La_Paz",
	},
	"SLRI": {
		ICAO:      "SLRI",
		IATA:      "RIB",
		Name:      "Capitan Av. Selin Zeitun Lopez Airport",
		City:      "Riberalta",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 462,
		Latitude:  -11,
		Longitude: -66,
		Timezone:  "America/La_Paz",
	},
	"SLRL": {
		ICAO:      "SLRL",
		Name:      "Rosal Airport",
		City:      "Yacuma",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 482,
		Latitude:  -13.9750003815,
		Longitude: -65.625,
		Timezone:  "America/La_Paz",
	},
	"SLRP": {
		ICAO:      "SLRP",
		Name:      "Rosapata Airport",
		City:      "Carangas",
		State:     "Oruro",
		Country:   "BO",
		Elevation: 12240,
		Latitude:  -18.6416664124,
		Longitude: -67.5583343506,
		Timezone:  "America/La_Paz",
	},
	"SLRQ": {
		ICAO:      "SLRQ",
		IATA:      "RBQ",
		Name:      "Rurenabaque Airport",
		City:      "Rurenabaque",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 676,
		Latitude:  -14.4279003143,
		Longitude: -67.4968032837,
		Timezone:  "America/La_Paz",
	},
	"SLRR": {
		ICAO:      "SLRR",
		Name:      "Retiro Airport",
		City:      "Yacuma",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 556,
		Latitude:  -14.4833335876,
		Longitude: -66.3499984741,
		Timezone:  "America/La_Paz",
	},
	"SLRS": {
		ICAO:      "SLRS",
		Name:      "Rio Seco Airport",
		City:      "Campo Ingelmeco",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 1836,
		Latitude:  -18.2008342743,
		Longitude: -63.2560005188,
		Timezone:  "America/La_Paz",
	},
	"SLRT": {
		ICAO:      "SLRT",
		Name:      "Santa Rita Airport",
		City:      "Yacuma",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 750,
		Latitude:  -13.9333333969,
		Longitude: -66.3333358765,
		Timezone:  "America/La_Paz",
	},
	"SLRX": {
		ICAO:      "SLRX",
		Name:      "Rincon Del Tigre Airport",
		City:      "German Busch",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 664,
		Latitude:  -18.2008342743,
		Longitude: -58.1699981689,
		Timezone:  "America/La_Paz",
	},
	"SLRY": {
		ICAO:      "SLRY",
		IATA:      "REY",
		Name:      "Reyes Airport",
		City:      "Reyes",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 935,
		Latitude:  -14.304400444,
		Longitude: -67.3534011841,
		Timezone:  "America/La_Paz",
	},
	"SLSA": {
		ICAO:      "SLSA",
		IATA:      "SBL",
		Name:      "Santa Ana Del Yacuma Airport",
		City:      "Santa Ana del Yacuma",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 472,
		Latitude:  -13.7622003555,
		Longitude: -65.4352035522,
		Timezone:  "America/La_Paz",
	},
	"SLSB": {
		ICAO:      "SLSB",
		IATA:      "SRJ",
		Name:      "Capitan Av. German Quiroga G. Airport",
		City:      "San Borja",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 633,
		Latitude:  -14.8591995239,
		Longitude: -66.7375030518,
		Timezone:  "America/La_Paz",
	},
	"SLSD": {
		ICAO:      "SLSD",
		Name:      "San Carlos Gutierrez Airport",
		City:      "Yacuma",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 1820,
		Latitude:  -14.5611000061,
		Longitude: -65.5271987915,
		Timezone:  "America/La_Paz",
	},
	"SLSE": {
		ICAO:      "SLSE",
		Name:      "Santa Teresita Airport",
		City:      "Angel Sandoval",
		State:     "Mato-Grosso",
		Country:   "BO",
		Elevation: 756,
		Latitude:  -16.2849998474,
		Longitude: -59.4951667786,
		Timezone:  "America/Cuiaba",
	},
	"SLSF": {
		ICAO:      "SLSF",
		Name:      "San Francisco Moxos Airport",
		City:      "Beni",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 722,
		Latitude:  -14.9666662216,
		Longitude: -65.6333312988,
		Timezone:  "America/La_Paz",
	},
	"SLSG": {
		ICAO:      "SLSG",
		Name:      "San Miguel De Gaser Airport",
		City:      "Yacuma",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 700,
		Latitude:  -13.9020996094,
		Longitude: -66.1567001343,
		Timezone:  "America/La_Paz",
	},
	"SLSH": {
		ICAO:      "SLSH",
		Name:      "Santa Ana De Huachi Airport",
		City:      "Sud Yungas",
		State:     "La-Paz",
		Country:   "BO",
		Elevation: 1640,
		Latitude:  -15.4833335876,
		Longitude: -67.4833297729,
		Timezone:  "America/La_Paz",
	},
	"SLSI": {
		ICAO:      "SLSI",
		IATA:      "SNG",
		Name:      "Capitan Av. Juan Cochamanidis S. Airport",
		City:      "San Ignacio de Velasco",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 1354,
		Latitude:  -16.383600235,
		Longitude: -60.9627990723,
		Timezone:  "America/La_Paz",
	},
	"SLSJ": {
		ICAO:      "SLSJ",
		Name:      "Salinas Airport",
		City:      "Ladislao Cabrera",
		State:     "Oruro",
		Country:   "BO",
		Elevation: 12027,
		Latitude:  -19.5,
		Longitude: -67.3333358765,
		Timezone:  "America/La_Paz",
	},
	"SLSK": {
		ICAO:      "SLSK",
		Name:      "Santa Rosa Del Sara Airport",
		City:      "A. Ibanez",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 850,
		Latitude:  -17.1000003815,
		Longitude: -63.6333351135,
		Timezone:  "America/La_Paz",
	},
	"SLSM": {
		ICAO:      "SLSM",
		IATA:      "SNM",
		Name:      "San Ignacio de Moxos Airport",
		City:      "San Ignacio de Moxos",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 790,
		Latitude:  -14,
		Longitude: -65.6339035034,
		Timezone:  "America/La_Paz",
	},
	"SLSN": {
		ICAO:      "SLSN",
		Name:      "Sanandita Airport",
		City:      "Gran Chaco",
		State:     "Tarija",
		Country:   "BO",
		Elevation: 3200,
		Latitude:  -21.6499996185,
		Longitude: -63.5999984741,
		Timezone:  "America/La_Paz",
	},
	"SLSP": {
		ICAO:      "SLSP",
		Name:      "Sipuati Airport",
		City:      "Gran Chaco",
		State:     "Tarija",
		Country:   "BO",
		Elevation: 1400,
		Latitude:  -21.0734996796,
		Longitude: -63.109500885,
		Timezone:  "America/La_Paz",
	},
	"SLSQ": {
		ICAO:      "SLSQ",
		Name:      "Santiago De Chiquitos Airport",
		City:      "Santiago de Chiquitos",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 1990,
		Latitude:  -18.3458003998,
		Longitude: -59.604801178,
		Timezone:  "America/La_Paz",
	},
	"SLSR": {
		ICAO:      "SLSR",
		IATA:      "SRB",
		Name:      "Santa Rosa De Yacuma Airport",
		City:      "Santa Rosa",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 738,
		Latitude:  -14.0662002563,
		Longitude: -66.7867965698,
		Timezone:  "America/La_Paz",
	},
	"SLSS": {
		ICAO:      "SLSS",
		Name:      "Saipiru Airport",
		City:      "Cordillera",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 2039,
		Latitude:  -19.3911991119,
		Longitude: -63.072101593,
		Timezone:  "America/La_Paz",
	},
	"SLSU": {
		ICAO:      "SLSU",
		IATA:      "SRE",
		Name:      "Juana Azurduy De Padilla Airport",
		City:      "Sucre",
		State:     "Chuquisaca",
		Country:   "BO",
		Elevation: 9540,
		Latitude:  -19.0070991516,
		Longitude: -65.2886962891,
		Timezone:  "America/La_Paz",
	},
	"SLTA": {
		ICAO:      "SLTA",
		Name:      "Taquipirenda Airport",
		City:      "Cordillera",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 2340,
		Latitude:  -20.3432998657,
		Longitude: -63.1394004822,
		Timezone:  "America/La_Paz",
	},
	"SLTC": {
		ICAO:      "SLTC",
		Name:      "Tres Cruces Airport",
		City:      "Chiquitos",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 880,
		Latitude:  -17.4316673279,
		Longitude: -62.1051673889,
		Timezone:  "America/La_Paz",
	},
	"SLTE": {
		ICAO:      "SLTE",
		Name:      "Teoponte Airport",
		City:      "Larecaja",
		State:     "La-Paz",
		Country:   "BO",
		Elevation: 1400,
		Latitude:  -15.4983329773,
		Longitude: -67.8166656494,
		Timezone:  "America/La_Paz",
	},
	"SLTG": {
		ICAO:      "SLTG",
		Name:      "Santiago Airport",
		City:      "Moxos",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 1000,
		Latitude:  -15.2278003693,
		Longitude: -65.4501037598,
		Timezone:  "America/La_Paz",
	},
	"SLTI": {
		ICAO:      "SLTI",
		IATA:      "MQK",
		Name:      "San Matias Airport",
		City:      "San Matias",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 403,
		Latitude:  -16.3391990662,
		Longitude: -58.4019012451,
		Timezone:  "America/Cuiaba",
	},
	"SLTJ": {
		ICAO:      "SLTJ",
		IATA:      "TJA",
		Name:      "Capitan Oriel Lea Plaza Airport",
		City:      "Tarija",
		State:     "Tarija",
		Country:   "BO",
		Elevation: 6079,
		Latitude:  -21.5557003021,
		Longitude: -64.7013015747,
		Timezone:  "America/La_Paz",
	},
	"SLTL": {
		ICAO:      "SLTL",
		Name:      "Toldos Airport",
		City:      "Nor Lipez",
		State:     "Potosi",
		Country:   "BO",
		Elevation: 11800,
		Latitude:  -21.1686000824,
		Longitude: -67.1789016724,
		Timezone:  "America/La_Paz",
	},
	"SLTO": {
		ICAO:      "SLTO",
		Name:      "Totora Airport",
		City:      "Oruro",
		State:     "La-Paz",
		Country:   "BO",
		Elevation: 12530,
		Latitude:  -17.798166275,
		Longitude: -68.09349823,
		Timezone:  "America/La_Paz",
	},
	"SLTR": {
		ICAO:      "SLTR",
		IATA:      "TDD",
		Name:      "Teniente Av. Jorge Henrich Arauz Airport",
		City:      "Trinidad",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 509,
		Latitude:  -14.8186998367,
		Longitude: -64.9179992676,
		Timezone:  "America/La_Paz",
	},
	"SLTU": {
		ICAO:      "SLTU",
		Name:      "Turco Airport",
		City:      "Sajama",
		State:     "Oruro",
		Country:   "BO",
		Elevation: 12500,
		Latitude:  -18.1958332062,
		Longitude: -68.141998291,
		Timezone:  "America/La_Paz",
	},
	"SLTW": {
		ICAO:      "SLTW",
		Name:      "Tita Airport",
		City:      "San Rafael",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 1312,
		Latitude:  -18.4664001465,
		Longitude: -62.1119995117,
		Timezone:  "America/La_Paz",
	},
	"SLTX": {
		ICAO:      "SLTX",
		Name:      "Tuichi Airport",
		City:      "Franz Tamayo",
		State:     "La-Paz",
		Country:   "BO",
		Elevation: 2360,
		Latitude:  -14.3166666031,
		Longitude: -68.5833358765,
		Timezone:  "America/La_Paz",
	},
	"SLTY": {
		ICAO:      "SLTY",
		Name:      "Tiguipa Airport",
		City:      "Yacuma",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 722,
		Latitude:  -13.8000001907,
		Longitude: -66.2333297729,
		Timezone:  "America/La_Paz",
	},
	"SLTZ": {
		ICAO:      "SLTZ",
		Name:      "Tupiza Airport",
		City:      "Sud Chichas",
		Country:   "BO",
		Elevation: 11300,
		Latitude:  -21.3446998596,
		Longitude: -65.6156005859,
		Timezone:  "America/La_Paz",
	},
	"SLUC": {
		ICAO:      "SLUC",
		Name:      "Cuevo Airport",
		City:      "Cordillera",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 3440,
		Latitude:  -20.4804992676,
		Longitude: -63.5281677246,
		Timezone:  "America/La_Paz",
	},
	"SLUK": {
		ICAO:      "SLUK",
		Name:      "Curahuara de Carangas Airport",
		City:      "Oruro",
		State:     "Oruro",
		Country:   "BO",
		Elevation: 12920,
		Latitude:  -17.9857997894,
		Longitude: -68.3943023682,
		Timezone:  "America/La_Paz",
	},
	"SLUS": {
		ICAO:      "SLUS",
		Name:      "Urusi Airport",
		City:      "Ballivian",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 805,
		Latitude:  -14.3166666031,
		Longitude: -66.1666641235,
		Timezone:  "America/La_Paz",
	},
	"SLUU": {
		ICAO:      "SLUU",
		Name:      "Ulla Ulla Airport",
		City:      "Franz Tamayo",
		State:     "La-Paz",
		Country:   "BO",
		Elevation: 14360,
		Latitude:  -15.0333337784,
		Longitude: -69.25,
		Timezone:  "America/La_Paz",
	},
	"SLUY": {
		ICAO:      "SLUY",
		IATA:      "UYU",
		Name:      "Uyuni Airport",
		City:      "Quijarro",
		State:     "Potosi",
		Country:   "BO",
		Elevation: 11136,
		Latitude:  -20.4463005066,
		Longitude: -66.8483963013,
		Timezone:  "America/La_Paz",
	},
	"SLVA": {
		ICAO:      "SLVA",
		Name:      "Villa Aroma Airport",
		City:      "Aroma",
		State:     "La-Paz",
		Country:   "BO",
		Elevation: 12533,
		Latitude:  -17.3666667938,
		Longitude: -67.75,
		Timezone:  "America/La_Paz",
	},
	"SLVD": {
		ICAO:      "SLVD",
		Name:      "Covendo Airport",
		City:      "Sud Yungas",
		State:     "La-Paz",
		Country:   "BO",
		Elevation: 4887,
		Latitude:  -15.8333330154,
		Longitude: -67.0833358765,
		Timezone:  "America/La_Paz",
	},
	"SLVE": {
		ICAO:      "SLVE",
		Name:      "Venecia Airport",
		City:      "Yacuma",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 722,
		Latitude:  -14.6359996796,
		Longitude: -66.7502975464,
		Timezone:  "America/La_Paz",
	},
	"SLVG": {
		ICAO:      "SLVG",
		IATA:      "VAH",
		Name:      "Capitan Av. Vidal Villagomez Toledo Airport",
		City:      "Vallegrande",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 6551,
		Latitude:  -18.4825000763,
		Longitude: -64.0994033813,
		Timezone:  "America/La_Paz",
	},
	"SLVI": {
		ICAO:      "SLVI",
		Name:      "Caranavi Airport",
		City:      "Nor Yungas",
		State:     "La-Paz",
		Country:   "BO",
		Elevation: 2100,
		Latitude:  -15.75,
		Longitude: -67.5666656494,
		Timezone:  "America/La_Paz",
	},
	"SLVM": {
		ICAO:      "SLVM",
		IATA:      "VLM",
		Name:      "Teniente Coronel Rafael Pabon Airport",
		City:      "Villamontes",
		State:     "Tarija",
		Country:   "BO",
		Elevation: 1305,
		Latitude:  -21.2551994324,
		Longitude: -63.4056015015,
		Timezone:  "America/La_Paz",
	},
	"SLVN": {
		ICAO:      "SLVN",
		Name:      "Villa Negrita Airport",
		City:      "Yacuma",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 872,
		Latitude:  -13.2658004761,
		Longitude: -65.8758010864,
		Timezone:  "America/La_Paz",
	},
	"SLVR": {
		ICAO:      "SLVR",
		IATA:      "VVI",
		Name:      "Viru Viru International Airport",
		City:      "Santa Cruz",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 1224,
		Latitude:  -17.6448001862,
		Longitude: -63.1353988647,
		Timezone:  "America/La_Paz",
	},
	"SLVT": {
		ICAO:      "SLVT",
		Name:      "Tesoro/La Vertiente Airport",
		City:      "Tesoro",
		State:     "Tarija",
		Country:   "BO",
		Elevation: 1300,
		Latitude:  -21.2666664124,
		Longitude: -63.3583335876,
		Timezone:  "America/La_Paz",
	},
	"SLVU": {
		ICAO:      "SLVU",
		Name:      "Vuelta Grande Airport",
		City:      "Luis Calvo",
		State:     "Chuquisaca",
		Country:   "BO",
		Elevation: 1900,
		Latitude:  -20.90483284,
		Longitude: -63.1921653748,
		Timezone:  "America/La_Paz",
	},
	"SLVV": {
		ICAO:      "SLVV",
		Name:      "Villa Elvira Airport",
		City:      "Mamore",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 722,
		Latitude:  -13.6443004608,
		Longitude: -64.4408035278,
		Timezone:  "America/La_Paz",
	},
	"SLVZ": {
		ICAO:      "SLVZ",
		Name:      "Villazon Airport",
		City:      "M.Omiste",
		State:     "Potosi",
		Country:   "BO",
		Elevation: 1100,
		Latitude:  -22.0776996613,
		Longitude: -65.5888977051,
		Timezone:  "America/Argentina/Jujuy",
	},
	"SLYA": {
		ICAO:      "SLYA",
		IATA:      "BYC",
		Name:      "Yacuiba Airport",
		City:      "Yacuiba",
		State:     "Tarija",
		Country:   "BO",
		Elevation: 2112,
		Latitude:  -21.960899353,
		Longitude: -63.6516990662,
		Timezone:  "America/La_Paz",
	},
	"SLYG": {
		ICAO:      "SLYG",
		Name:      "Yabog Airport",
		City:      "Las Juntas",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 1412,
		Latitude:  -18.654499054,
		Longitude: -63.1328010559,
		Timezone:  "America/La_Paz",
	},
	"SLYI": {
		ICAO:      "SLYI",
		Name:      "Yapacani Airport",
		City:      "Ichilo",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 816,
		Latitude:  -17.0156669617,
		Longitude: -64.0563354492,
		Timezone:  "America/La_Paz",
	},
	"SLZB": {
		ICAO:      "SLZB",
		Name:      "San Pedro Rb Airport",
		City:      "Moxos",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 720,
		Latitude:  -14.8666667938,
		Longitude: -65.6500015259,
		Timezone:  "America/La_Paz",
	},
	"SLZC": {
		ICAO:      "SLZC",
		Name:      "San Carlos Airport",
		City:      "Ichilo",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 1150,
		Latitude:  -17.4333324432,
		Longitude: -63.6833343506,
		Timezone:  "America/La_Paz",
	},
	"SLZF": {
		ICAO:      "SLZF",
		Name:      "San Francisco Naciff Airport",
		City:      "Nacif",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 482,
		Latitude:  -13.7486000061,
		Longitude: -64.4536972046,
		Timezone:  "America/La_Paz",
	},
	"SLZJ": {
		ICAO:      "SLZJ",
		Name:      "San Pedro Richard Airport",
		City:      "Itenes",
		State:     "El-Beni",
		Country:   "BO",
		Elevation: 700,
		Latitude:  -14.2166662216,
		Longitude: -63.6500015259,
		Timezone:  "America/La_Paz",
	},
	"SLZV": {
		ICAO:      "SLZV",
		Name:      "San Vicente Airport",
		City:      "Velasco",
		State:     "Santa-Cruz",
		Country:   "BO",
		Elevation: 1000,
		Latitude:  -16.2737007141,
		Longitude: -60.0906982422,
		Timezone:  "America/Cuiaba",
	},
	"SMAF": {
		ICAO:      "SMAF",
		Name:      "Afobakka Airstrip",
		City:      "Afobakka",
		State:     "Brokopondo",
		Country:   "SR",
		Elevation: 80,
		Latitude:  4.998505,
		Longitude: -54.992033,
		Timezone:  "America/Paramaribo",
	},
	"SMBN": {
		ICAO:      "SMBN",
		IATA:      "ABN",
		Name:      "Albina Airport",
		City:      "Albina",
		State:     "Marowijne",
		Country:   "SR",
		Elevation: 19,
		Latitude:  5.512720108,
		Longitude: -54.0500984192,
		Timezone:  "America/Cayenne",
	},
	"SMBO": {
		ICAO:      "SMBO",
		IATA:      "BTO",
		Name:      "Botopasi Airport",
		City:      "Botopasi",
		State:     "Sipaliwini",
		Country:   "SR",
		Elevation: 242,
		Latitude:  4.2175110935,
		Longitude: -55.4470539093,
		Timezone:  "America/Paramaribo",
	},
	"SMCA": {
		ICAO:      "SMCA",
		IATA:      "AAJ",
		Name:      "Cayana Airstrip",
		City:      "Awaradam",
		State:     "Sipaliwini",
		Country:   "SR",
		Elevation: 360,
		Latitude:  3.898681,
		Longitude: -55.577907,
		Timezone:  "America/Paramaribo",
	},
	"SMCI": {
		ICAO:      "SMCI",
		Name:      "Coeroeni Airport",
		City:      "Coeroeni",
		State:     "Sipaliwini",
		Country:   "SR",
		Elevation: 479,
		Latitude:  3.367000103,
		Longitude: -57.3330001831,
		Timezone:  "America/Guyana",
	},
	"SMCO": {
		ICAO:      "SMCO",
		IATA:      "TOT",
		Name:      "Totness Airport",
		City:      "Totness",
		State:     "Coronie",
		Country:   "SR",
		Elevation: 10,
		Latitude:  5.8658299446,
		Longitude: -56.3274993896,
		Timezone:  "America/Paramaribo",
	},
	"SMCT": {
		ICAO:      "SMCT",
		Name:      "Cottica Airstrip",
		City:      "Lawa Cottica",
		State:     "Sipaliwini",
		Country:   "SR",
		Elevation: 330,
		Latitude:  3.85216,
		Longitude: -54.228201,
		Timezone:  "America/Cayenne",
	},
	"SMDA": {
		ICAO:      "SMDA",
		IATA:      "DRJ",
		Name:      "Drietabbetje Airport",
		City:      "Drietabbetje",
		State:     "Sipaliwini",
		Country:   "SR",
		Elevation: 236,
		Latitude:  4.1166667938,
		Longitude: -54.6666717529,
		Timezone:  "America/Paramaribo",
	},
	"SMDJ": {
		ICAO:      "SMDJ",
		IATA:      "DOE",
		Name:      "Djumu-Djomoe Airport",
		City:      "Djumu-Djomoe",
		State:     "Sipaliwini",
		Country:   "SR",
		Elevation: 290,
		Latitude:  4.0057126106,
		Longitude: -55.4816436768,
		Timezone:  "America/Paramaribo",
	},
	"SMDO": {
		ICAO:      "SMDO",
		IATA:      "LDO",
		Name:      "Ladouanie Airport",
		City:      "Aurora",
		State:     "Sipaliwini",
		Country:   "SR",
		Elevation: 236,
		Latitude:  4.3761082835,
		Longitude: -55.4070568085,
		Timezone:  "America/Paramaribo",
	},
	"SMDU": {
		ICAO:      "SMDU",
		Name:      "Alalapadu Airstrip",
		City:      "Alalapadi",
		State:     "Sipaliwini",
		Country:   "SR",
		Elevation: 880,
		Latitude:  2.5235,
		Longitude: -56.3247,
		Timezone:  "America/Paramaribo",
	},
	"SMGH": {
		ICAO:      "SMGH",
		Name:      "Godo Holo Airstrip",
		City:      "Pikienkondre of Miranda",
		State:     "Sipaliwini",
		Country:   "SR",
		Elevation: 280,
		Latitude:  4.0583,
		Longitude: -54.7861,
		Timezone:  "America/Paramaribo",
	},
	"SMJP": {
		ICAO:      "SMJP",
		IATA:      "PBM",
		Name:      "Johan Adolf Pengel International Airport",
		City:      "Zandery",
		State:     "Para",
		Country:   "SR",
		Elevation: 59,
		Latitude:  5.4528298378,
		Longitude: -55.1878013611,
		Timezone:  "America/Paramaribo",
	},
	"SMKA": {
		ICAO:      "SMKA",
		Name:      "Kabalebo Airport",
		City:      "Kabalebo",
		State:     "Sipaliwini",
		Country:   "SR",
		Elevation: 535,
		Latitude:  4.4060001373,
		Longitude: -57.2229995728,
		Timezone:  "America/Paramaribo",
	},
	"SMKE": {
		ICAO:      "SMKE",
		Name:      "Kayser Airport",
		City:      "Kayserberg",
		State:     "Sipaliwini",
		Country:   "SR",
		Elevation: 849,
		Latitude:  3.0999999046,
		Longitude: -56.483001709,
		Timezone:  "America/Paramaribo",
	},
	"SMMO": {
		ICAO:      "SMMO",
		IATA:      "MOJ",
		Name:      "Moengo Airstrip",
		City:      "Moengo",
		State:     "Marowijne",
		Country:   "SR",
		Elevation: 49,
		Latitude:  5.6076,
		Longitude: -54.4003,
		Timezone:  "America/Paramaribo",
	},
	"SMNI": {
		ICAO:      "SMNI",
		IATA:      "ICK",
		Name:      "Nieuw Nickerie Airport",
		City:      "Nieuw Nickerie",
		State:     "Nickerie",
		Country:   "SR",
		Elevation: 9,
		Latitude:  5.9555602074,
		Longitude: -57.0393981934,
		Timezone:  "America/Paramaribo",
	},
	"SMPA": {
		ICAO:      "SMPA",
		IATA:      "OEM",
		Name:      "Vincent Fayks Airport",
		City:      "Paloemeu",
		State:     "Sipaliwini",
		Country:   "SR",
		Elevation: 714,
		Latitude:  3.345279932,
		Longitude: -55.4425010681,
		Timezone:  "America/Paramaribo",
	},
	"SMPG": {
		ICAO:      "SMPG",
		Name:      "Poesoegroenoe(Pusugrunu) Airstrip",
		City:      "Poesoegroenoe",
		State:     "Sipaliwini",
		Country:   "SR",
		Elevation: 390,
		Latitude:  4.397914,
		Longitude: -55.793374,
		Timezone:  "America/Paramaribo",
	},
	"SMSI": {
		ICAO:      "SMSI",
		Name:      "Sipaliwini Airport",
		City:      "Sipaliwini",
		State:     "Sipaliwini",
		Country:   "SR",
		Elevation: 744,
		Latitude:  1.9660500288,
		Longitude: -56.0035018921,
		Timezone:  "America/Paramaribo",
	},
	"SMSM": {
		ICAO:      "SMSM",
		Name:      "Kwamalasoemoetoe Airport Airport",
		City:      "Kwamalasoemoetoe Airport",
		State:     "Sipaliwini",
		Country:   "SR",
		Elevation: 905,
		Latitude:  2.333329916,
		Longitude: -56.7833328247,
		Timezone:  "America/Paramaribo",
	},
	"SMST": {
		ICAO:      "SMST",
		IATA:      "SMZ",
		Name:      "Stoelmanseiland Airport",
		City:      "Stoelmanseiland",
		State:     "Sipaliwini",
		Country:   "SR",
		Elevation: 187,
		Latitude:  4.3499999046,
		Longitude: -54.4166679382,
		Timezone:  "America/Paramaribo",
	},
	"SMTA": {
		ICAO:      "SMTA",
		Name:      "Tabiki Airstrip",
		City:      "Benzdorp",
		State:     "Sipaliwini",
		Country:   "SR",
		Elevation: 740,
		Latitude:  3.619932,
		Longitude: -54.142714,
		Timezone:  "America/Paramaribo",
	},
	"SMTB": {
		ICAO:      "SMTB",
		Name:      "Rudi Kappel Airport",
		City:      "Tafelberg",
		State:     "Sipaliwini",
		Country:   "SR",
		Elevation: 1112,
		Latitude:  3.7829999924,
		Longitude: -56.1500015259,
		Timezone:  "America/Paramaribo",
	},
	"SMTP": {
		ICAO:      "SMTP",
		IATA:      "KCB",
		Name:      "Tepoe Airstrip",
		City:      "Kasikasima",
		State:     "Sipaliwini",
		Country:   "SR",
		Elevation: 596,
		Latitude:  3.1500000954,
		Longitude: -55.716999054,
		Timezone:  "America/Paramaribo",
	},
	"SMWA": {
		ICAO:      "SMWA",
		IATA:      "AGI",
		Name:      "Wageningen Airport Airport",
		City:      "Wageningen Airport",
		State:     "Nickerie",
		Country:   "SR",
		Elevation: 6,
		Latitude:  5.7666668892,
		Longitude: -56.6333312988,
		Timezone:  "America/Paramaribo",
	},
	"SMWS": {
		ICAO:      "SMWS",
		IATA:      "WSO",
		Name:      "Washabo Airport",
		City:      "Washabo",
		State:     "Sipaliwini",
		Country:   "SR",
		Elevation: 68,
		Latitude:  5.2501502037,
		Longitude: -57.1738014221,
		Timezone:  "America/Guyana",
	},
	"SMZO": {
		ICAO:      "SMZO",
		IATA:      "ORG",
		Name:      "Zorg en Hoop Airport",
		City:      "Paramaribo",
		State:     "Paramaribo",
		Country:   "SR",
		Elevation: 10,
		Latitude:  5.8110799789,
		Longitude: -55.190700531,
		Timezone:  "America/Paramaribo",
	},
	"SN00": {
		ICAO:      "SN00",
		Name:      "J Roesner Airport",
		City:      "Salina",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1211,
		Latitude:  38.8469009399,
		Longitude: -97.5503005981,
		Timezone:  "America/Chicago",
	},
	"SN02": {
		ICAO:      "SN02",
		Name:      "Wright International Airport",
		City:      "Kanorado",
		State:     "Kansas",
		Country:   "US",
		Elevation: 3883,
		Latitude:  39.4082984924,
		Longitude: -102.0210037231,
		Timezone:  "America/Denver",
	},
	"SN03": {
		ICAO:      "SN03",
		Name:      "Lenora Municipal Airport",
		City:      "Lenora",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2373,
		Latitude:  39.6293983459,
		Longitude: -100.0149993896,
		Timezone:  "America/Chicago",
	},
	"SN04": {
		ICAO:      "SN04",
		Name:      "Roberts Memorial Airport",
		City:      "Emporia",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1185,
		Latitude:  38.4230995178,
		Longitude: -96.3719024658,
		Timezone:  "America/Chicago",
	},
	"SN05": {
		ICAO:      "SN05",
		Name:      "Halstead Airport",
		City:      "Halstead",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1412,
		Latitude:  38.0313987732,
		Longitude: -97.5092010498,
		Timezone:  "America/Chicago",
	},
	"SN07": {
		ICAO:      "SN07",
		Name:      "Beaumont Hotel Airport",
		City:      "Beaumont",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1617,
		Latitude:  37.659198761,
		Longitude: -96.5271987915,
		Timezone:  "America/Chicago",
	},
	"SN08": {
		ICAO:      "SN08",
		Name:      "Fisher Airport",
		City:      "Burrton",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1430,
		Latitude:  37.9916992188,
		Longitude: -97.6564025879,
		Timezone:  "America/Chicago",
	},
	"SN09": {
		ICAO:      "SN09",
		Name:      "Solomon Valley Airpark",
		City:      "Minneapolis",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1205,
		Latitude:  39.113899231,
		Longitude: -97.6830978394,
		Timezone:  "America/Chicago",
	},
	"SN10": {
		ICAO:      "SN10",
		Name:      "Belle Plaine Farms Airport",
		City:      "Belle Plaine",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1190,
		Latitude:  37.3958015442,
		Longitude: -97.2099990845,
		Timezone:  "America/Chicago",
	},
	"SN12": {
		ICAO:      "SN12",
		Name:      "Jenkinson Airport",
		City:      "Meade",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2737,
		Latitude:  37.363899231,
		Longitude: -100.4710006714,
		Timezone:  "America/Chicago",
	},
	"SN13": {
		ICAO:      "SN13",
		Name:      "Albers Airport",
		City:      "Cheney",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1380,
		Latitude:  37.6582984924,
		Longitude: -97.7744979858,
		Timezone:  "America/Chicago",
	},
	"SN14": {
		ICAO:      "SN14",
		Name:      "Schoolcraft Airport",
		City:      "Agricola",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1165,
		Latitude:  38.4166984558,
		Longitude: -95.5168991089,
		Timezone:  "America/Chicago",
	},
	"SN16": {
		ICAO:      "SN16",
		Name:      "Curtis Airport",
		City:      "Kechi",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1380,
		Latitude:  37.8563995361,
		Longitude: -97.2697982788,
		Timezone:  "America/Chicago",
	},
	"SN17": {
		ICAO:      "SN17",
		Name:      "Shupe Airport",
		City:      "Ashland",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1769,
		Latitude:  37.0719985962,
		Longitude: -99.6051025391,
		Timezone:  "America/Chicago",
	},
	"SN18": {
		ICAO:      "SN18",
		Name:      "Sills Air Park",
		City:      "Augusta",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1295,
		Latitude:  37.6666984558,
		Longitude: -97.016998291,
		Timezone:  "America/Chicago",
	},
	"SN19": {
		ICAO:      "SN19",
		Name:      "Flying H Ranch Airport",
		City:      "Augusta",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1400,
		Latitude:  37.625,
		Longitude: -97.0961990356,
		Timezone:  "America/Chicago",
	},
	"SN20": {
		ICAO:      "SN20",
		Name:      "Brady-Pippin Airport",
		City:      "Augusta",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1300,
		Latitude:  37.7195014954,
		Longitude: -97.0488967896,
		Timezone:  "America/Chicago",
	},
	"SN21": {
		ICAO:      "SN21",
		Name:      "Flory /Private/ Airport",
		City:      "Baldwin City",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1100,
		Latitude:  38.8227996826,
		Longitude: -95.2878036499,
		Timezone:  "America/Chicago",
	},
	"SN22": {
		ICAO:      "SN22",
		Name:      "Hoelting Airport",
		City:      "Basehor",
		State:     "Kansas",
		Country:   "US",
		Elevation: 960,
		Latitude:  39.1122016907,
		Longitude: -94.9561004639,
		Timezone:  "America/Chicago",
	},
	"SN23": {
		ICAO:      "SN23",
		Name:      "Overmiller Airport",
		City:      "Smith Center",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2010,
		Latitude:  39.9119987488,
		Longitude: -98.6656036377,
		Timezone:  "America/Chicago",
	},
	"SN25": {
		ICAO:      "SN25",
		Name:      "Lucas Airport",
		City:      "Big Bow",
		State:     "Kansas",
		Country:   "US",
		Elevation: 3172,
		Latitude:  37.418598175,
		Longitude: -101.5579986572,
		Timezone:  "America/Chicago",
	},
	"SN26": {
		ICAO:      "SN26",
		Name:      "Horttor Airport",
		City:      "Blue Mound",
		State:     "Kansas",
		Country:   "US",
		Elevation: 970,
		Latitude:  38.1777992249,
		Longitude: -95.0029983521,
		Timezone:  "America/Chicago",
	},
	"SN27": {
		ICAO:      "SN27",
		Name:      "Bovinair Airport",
		City:      "Bonner Springs",
		State:     "Kansas",
		Country:   "US",
		Elevation: 920,
		Latitude:  39.0625,
		Longitude: -94.9447021484,
		Timezone:  "America/Chicago",
	},
	"SN28": {
		ICAO:      "SN28",
		Name:      "Belcher Airport",
		City:      "Brookville",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1420,
		Latitude:  38.86859894,
		Longitude: -97.93779755,
		Timezone:  "America/Chicago",
	},
	"SN29": {
		ICAO:      "SN29",
		Name:      "Rucker Burdett Airport",
		City:      "Burdett",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2151,
		Latitude:  38.1875,
		Longitude: -99.533996582,
		Timezone:  "America/Chicago",
	},
	"SN32": {
		ICAO:      "SN32",
		Name:      "Rands Airport",
		City:      "Belle Plaine",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1210,
		Latitude:  37.426399231,
		Longitude: -97.2695007324,
		Timezone:  "America/Chicago",
	},
	"SN33": {
		ICAO:      "SN33",
		Name:      "Callaway Airpark",
		City:      "Clay Center",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1275,
		Latitude:  39.4333000183,
		Longitude: -97.1211013794,
		Timezone:  "America/Chicago",
	},
	"SN34": {
		ICAO:      "SN34",
		Name:      "Rucker Airport",
		City:      "Clearwater",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1300,
		Latitude:  37.5750007629,
		Longitude: -97.4919967651,
		Timezone:  "America/Chicago",
	},
	"SN36": {
		ICAO:      "SN36",
		Name:      "Wamsley Field",
		City:      "Conway Springs",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1335,
		Latitude:  37.426399231,
		Longitude: -97.637802124,
		Timezone:  "America/Chicago",
	},
	"SN37": {
		ICAO:      "SN37",
		Name:      "Harold K. Wells Airport",
		City:      "Elmdale",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1100,
		Latitude:  38.3870010376,
		Longitude: -96.7296981812,
		Timezone:  "America/Chicago",
	},
	"SN39": {
		ICAO:      "SN39",
		Name:      "Baldock Farm Airport",
		City:      "Delphos",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1350,
		Latitude:  39.25,
		Longitude: -97.6502990723,
		Timezone:  "America/Chicago",
	},
	"SN40": {
		ICAO:      "SN40",
		Name:      "Olson Aerodrome",
		City:      "Derby",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1325,
		Latitude:  37.523601532,
		Longitude: -97.1984024048,
		Timezone:  "America/Chicago",
	},
	"SN41": {
		ICAO:      "SN41",
		Name:      "Ziggy Carline Airport",
		City:      "Girard",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1300,
		Latitude:  37.5041999817,
		Longitude: -94.8266983032,
		Timezone:  "America/Chicago",
	},
	"SN42": {
		ICAO:      "SN42",
		Name:      "Harrod Airport",
		City:      "Ottawa",
		State:     "Kansas",
		Country:   "US",
		Elevation: 990,
		Latitude:  38.5614013672,
		Longitude: -95.2081985474,
		Timezone:  "America/Chicago",
	},
	"SN43": {
		ICAO:      "SN43",
		Name:      "Strafuss Airport",
		City:      "Effingham",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1050,
		Latitude:  39.5041999817,
		Longitude: -95.4710998535,
		Timezone:  "America/Chicago",
	},
	"SN44": {
		ICAO:      "SN44",
		Name:      "Bob Faler Airport",
		City:      "Elk City",
		State:     "Kansas",
		Country:   "US",
		Elevation: 850,
		Latitude:  37.3000984192,
		Longitude: -95.9002990723,
		Timezone:  "America/Chicago",
	},
	"SN45": {
		ICAO:      "SN45",
		Name:      "Michael's Airport",
		City:      "Lawrence",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1105,
		Latitude:  38.831401825,
		Longitude: -95.2664031982,
		Timezone:  "America/Chicago",
	},
	"SN46": {
		ICAO:      "SN46",
		Name:      "Taylor Airport",
		City:      "Benton",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1340,
		Latitude:  37.8718986511,
		Longitude: -97.0721969604,
		Timezone:  "America/Chicago",
	},
	"SN47": {
		ICAO:      "SN47",
		Name:      "Converse Farm Airport",
		City:      "Eskridge",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1231,
		Latitude:  38.7667007446,
		Longitude: -96.1072006226,
		Timezone:  "America/Chicago",
	},
	"SN48": {
		ICAO:      "SN48",
		Name:      "Moore Field",
		City:      "Byers",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1992,
		Latitude:  37.7882995605,
		Longitude: -98.8498001099,
		Timezone:  "America/Chicago",
	},
	"SN49": {
		ICAO:      "SN49",
		Name:      "Mc Collough Airfield",
		City:      "Randall",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1465,
		Latitude:  39.6852989197,
		Longitude: -97.9711990356,
		Timezone:  "America/Chicago",
	},
	"SN51": {
		ICAO:      "SN51",
		Name:      "Meitl Airport",
		City:      "Dresden",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2725,
		Latitude:  39.5896987915,
		Longitude: -100.4280014038,
		Timezone:  "America/Chicago",
	},
	"SN52": {
		ICAO:      "SN52",
		Name:      "Pilot Pointe Estates Airport",
		City:      "Douglass",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1237,
		Latitude:  37.48139954,
		Longitude: -97.01200104,
		Timezone:  "America/Chicago",
	},
	"SN53": {
		ICAO:      "SN53",
		Name:      "Bonner Field",
		City:      "Fredonia",
		State:     "Kansas",
		Country:   "US",
		Elevation: 950,
		Latitude:  37.4333992004,
		Longitude: -95.9169006348,
		Timezone:  "America/Chicago",
	},
	"SN55": {
		ICAO:      "SN55",
		Name:      "R J C Farms Inc Airport",
		City:      "Friend",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2900,
		Latitude:  38.2153015137,
		Longitude: -101.0029983521,
		Timezone:  "America/Chicago",
	},
	"SN57": {
		ICAO:      "SN57",
		Name:      "Witham Airport",
		City:      "Ottawa",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1110,
		Latitude:  38.698600769,
		Longitude: -95.3003005981,
		Timezone:  "America/Chicago",
	},
	"SN58": {
		ICAO:      "SN58",
		Name:      "Foster Field",
		City:      "El Dorado",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1440,
		Latitude:  37.8699989319,
		Longitude: -96.646697998,
		Timezone:  "America/Chicago",
	},
	"SN59": {
		ICAO:      "SN59",
		Name:      "Hermon Farm Airport",
		City:      "Gardner",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1060,
		Latitude:  38.8527984619,
		Longitude: -94.9100036621,
		Timezone:  "America/Chicago",
	},
	"SN61": {
		ICAO:      "SN61",
		Name:      "Yoder Airpark",
		City:      "Goddard",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1475,
		Latitude:  37.6596984863,
		Longitude: -97.6295013428,
		Timezone:  "America/Chicago",
	},
	"SN62": {
		ICAO:      "SN62",
		Name:      "Roberts Field",
		City:      "Bentley",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1384,
		Latitude:  37.8741989136,
		Longitude: -97.5124969482,
		Timezone:  "America/Chicago",
	},
	"SN63": {
		ICAO:      "SN63",
		Name:      "Gail Ballard Airport",
		City:      "Haviland",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2165,
		Latitude:  37.6083984375,
		Longitude: -99.1156997681,
		Timezone:  "America/Chicago",
	},
	"SN64": {
		ICAO:      "SN64",
		Name:      "Eck Field",
		City:      "Goddard",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1330,
		Latitude:  37.6958007813,
		Longitude: -97.5391998291,
		Timezone:  "America/Chicago",
	},
	"SN65": {
		ICAO:      "SN65",
		Name:      "Lake Waltanna Airport",
		City:      "Goddard",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1440,
		Latitude:  37.5917015076,
		Longitude: -97.6169967651,
		Timezone:  "America/Chicago",
	},
	"SN66": {
		ICAO:      "SN66",
		Name:      "Beesley Farms Airport",
		City:      "Gove",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2635,
		Latitude:  38.8182983398,
		Longitude: -100.4550018311,
		Timezone:  "America/Chicago",
	},
	"SN67": {
		ICAO:      "SN67",
		Name:      "Button Airport",
		City:      "Great Bend",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1845,
		Latitude:  38.3833007813,
		Longitude: -98.7503967285,
		Timezone:  "America/Chicago",
	},
	"SN68": {
		ICAO:      "SN68",
		Name:      "Lil Bird Airport",
		City:      "Valley Center",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1456,
		Latitude:  37.8791999817,
		Longitude: -97.3103027344,
		Timezone:  "America/Chicago",
	},
	"SN69": {
		ICAO:      "SN69",
		Name:      "Emmerson Airport",
		City:      "Hammond",
		State:     "Kansas",
		Country:   "US",
		Elevation: 860,
		Latitude:  37.9314002991,
		Longitude: -94.6427001953,
		Timezone:  "America/Chicago",
	},
	"SN70": {
		ICAO:      "SN70",
		Name:      "Kaypod Airport",
		City:      "Harper",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1379,
		Latitude:  37.2513999939,
		Longitude: -97.954498291,
		Timezone:  "America/Chicago",
	},
	"SN71": {
		ICAO:      "SN71",
		Name:      "Bob Park Airport",
		City:      "Harper",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1396,
		Latitude:  37.3027992249,
		Longitude: -97.9642028809,
		Timezone:  "America/Chicago",
	},
	"SN72": {
		ICAO:      "SN72",
		Name:      "Graham Farms Airport",
		City:      "Harris",
		State:     "Kansas",
		Country:   "US",
		Elevation: 985,
		Latitude:  38.2999992371,
		Longitude: -95.4400024414,
		Timezone:  "America/Chicago",
	},
	"SN75": {
		ICAO:      "SN75",
		Name:      "Sommers Airport",
		City:      "Highland",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1070,
		Latitude:  39.8666992188,
		Longitude: -95.3336029053,
		Timezone:  "America/Chicago",
	},
	"SN76": {
		ICAO:      "SN76",
		Name:      "Sunflower Aerodrome",
		City:      "Hutchinson",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1582,
		Latitude:  37.926399231,
		Longitude: -97.9061965942,
		Timezone:  "America/Chicago",
	},
	"SN77": {
		ICAO:      "SN77",
		Name:      "Johns Airport",
		City:      "Johnson",
		State:     "Kansas",
		Country:   "US",
		Elevation: 3390,
		Latitude:  37.4000015259,
		Longitude: -101.766998291,
		Timezone:  "America/Chicago",
	},
	"SN78": {
		ICAO:      "SN78",
		Name:      "Albright Airport",
		City:      "Bucyrus",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1000,
		Latitude:  38.6981010437,
		Longitude: -94.7188034058,
		Timezone:  "America/Chicago",
	},
	"SN81": {
		ICAO:      "SN81",
		Name:      "Hancock Airport",
		City:      "Piper",
		State:     "Kansas",
		Country:   "US",
		Elevation: 940,
		Latitude:  39.1819000244,
		Longitude: -94.8700027466,
		Timezone:  "America/Chicago",
	},
	"SN83": {
		ICAO:      "SN83",
		Name:      "Highcrest Air Park",
		City:      "Lansing",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1000,
		Latitude:  39.2271995544,
		Longitude: -94.9514007568,
		Timezone:  "America/Chicago",
	},
	"SN84": {
		ICAO:      "SN84",
		Name:      "Blaser's Airport",
		City:      "Leavenworth",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1000,
		Latitude:  39.2708015442,
		Longitude: -94.987701416,
		Timezone:  "America/Chicago",
	},
	"SN85": {
		ICAO:      "SN85",
		Name:      "Fox Fire Airport",
		City:      "Wellsville",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1058,
		Latitude:  38.7336006165,
		Longitude: -95.1022033691,
		Timezone:  "America/Chicago",
	},
	"SN86": {
		ICAO:      "SN86",
		Name:      "Arrow B Ranch Airport",
		City:      "Leoti",
		State:     "Kansas",
		Country:   "US",
		Elevation: 3430,
		Latitude:  38.7042007446,
		Longitude: -101.4209976196,
		Timezone:  "America/Chicago",
	},
	"SN87": {
		ICAO:      "SN87",
		Name:      "Supreme Feeders Airport",
		City:      "Liberal",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2830,
		Latitude:  37.2599983215,
		Longitude: -100.9069976807,
		Timezone:  "America/Chicago",
	},
	"SN88": {
		ICAO:      "SN88",
		Name:      "Crosswind Airfield",
		City:      "Louisburg",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1100,
		Latitude:  38.5713996887,
		Longitude: -94.7126998901,
		Timezone:  "America/Chicago",
	},
	"SN89": {
		ICAO:      "SN89",
		Name:      "Somerset Airport",
		City:      "Louisburg",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1100,
		Latitude:  38.5377998352,
		Longitude: -94.7268981934,
		Timezone:  "America/Chicago",
	},
	"SN90": {
		ICAO:      "SN90",
		Name:      "D'Field Airport",
		City:      "Mc Louth",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1100,
		Latitude:  39.226398468,
		Longitude: -95.1919021606,
		Timezone:  "America/Chicago",
	},
	"SN91": {
		ICAO:      "SN91",
		Name:      "Godfrey Airport",
		City:      "Madison",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1100,
		Latitude:  38.1096992493,
		Longitude: -96.1156005859,
		Timezone:  "America/Chicago",
	},
	"SN92": {
		ICAO:      "SN92",
		Name:      "Sunshine Ranch Airport",
		City:      "Manhattan",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1300,
		Latitude:  39.3333015442,
		Longitude: -96.700302124,
		Timezone:  "America/Chicago",
	},
	"SN93": {
		ICAO:      "SN93",
		Name:      "Dickson /Private/ Airport",
		City:      "Miller",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1140,
		Latitude:  38.6458015442,
		Longitude: -95.9891967773,
		Timezone:  "America/Chicago",
	},
	"SN94": {
		ICAO:      "SN94",
		Name:      "Amy Airport",
		City:      "Minneola",
		State:     "Kansas",
		Country:   "US",
		Elevation: 2560,
		Latitude:  37.4667015076,
		Longitude: -100,
		Timezone:  "America/Chicago",
	},
	"SN95": {
		ICAO:      "SN95",
		Name:      "Roberts Air Field",
		City:      "Sylvia",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1765,
		Latitude:  37.9775009155,
		Longitude: -98.4041976929,
		Timezone:  "America/Chicago",
	},
	"SN97": {
		ICAO:      "SN97",
		Name:      "Brollier Airport",
		City:      "Moscow",
		State:     "Kansas",
		Country:   "US",
		Elevation: 3056,
		Latitude:  37.3264007568,
		Longitude: -101.1979980469,
		Timezone:  "America/Chicago",
	},
	"SN98": {
		ICAO:      "SN98",
		Name:      "Anton Flying Uv Airport",
		City:      "Satanta",
		Country:   "US",
		Elevation: 2954,
		Latitude:  37.4207992554,
		Longitude: -100.9420013428,
		Timezone:  "America/Chicago",
	},
	"SN99": {
		ICAO:      "SN99",
		Name:      "Laflin Ranch Airport",
		City:      "Olsburg",
		State:     "Kansas",
		Country:   "US",
		Elevation: 1190,
		Latitude:  39.4000015259,
		Longitude: -96.6169967651,
		Timezone:  "America/Chicago",
	},
	"SNAB": {
		ICAO:      "SNAB",
		Name:      "Araripina Airport",
		City:      "Araripina",
		State:     "Pernambuco",
		Country:   "BR",
		Elevation: 2194,
		Latitude:  -7.5867600441,
		Longitude: -40.5354995728,
		Timezone:  "America/Recife",
	},
	"SNAC": {
		ICAO:      "SNAC",
		Name:      "Fazenda Santa Rosa Airport",
		City:      "Xinguara",
		State:     "Para",
		Country:   "BR",
		Elevation: 699,
		Latitude:  -7.036670208,
		Longitude: -50.0889015198,
		Timezone:  "America/Belem",
	},
	"SNAD": {
		ICAO:      "SNAD",
		Name:      "Fazenda Aguia Branca Airport",
		City:      "Gurupi",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 410,
		Latitude:  -3.327778101,
		Longitude: -46.7797203064,
		Timezone:  "America/Fortaleza",
	},
	"SNAE": {
		ICAO:      "SNAE",
		Name:      "Arcoverde Airport",
		City:      "Arcoverde",
		State:     "Pernambuco",
		Country:   "BR",
		Elevation: 2080,
		Latitude:  -8.4078798294,
		Longitude: -37.0881996155,
		Timezone:  "America/Recife",
	},
	"SNAF": {
		ICAO:      "SNAF",
		Name:      "Fazenda Agua Comprida Airport",
		City:      "Agua Comprida",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2211,
		Latitude:  -19.9469432831,
		Longitude: -48.0336112976,
		Timezone:  "America/Sao_Paulo",
	},
	"SNAG": {
		ICAO:      "SNAG",
		Name:      "Araguari Airport",
		City:      "Araguari",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3107,
		Latitude:  -18.6683006287,
		Longitude: -48.1904983521,
		Timezone:  "America/Sao_Paulo",
	},
	"SNAH": {
		ICAO:      "SNAH",
		Name:      "Adustina Airport",
		City:      "Adustina",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 892,
		Latitude:  -10.5809001923,
		Longitude: -38.0887985229,
		Timezone:  "America/Bahia",
	},
	"SNAI": {
		ICAO:      "SNAI",
		IATA:      "APY",
		Name:      "Alto Parnaiba Airport",
		City:      "Alto Parnaiba",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 968,
		Latitude:  -9.0836105347,
		Longitude: -45.9505577087,
		Timezone:  "America/Fortaleza",
	},
	"SNAJ": {
		ICAO:      "SNAJ",
		Name:      "Fazenda Jaguare Airport",
		City:      "Paragominas",
		State:     "Para",
		Country:   "BR",
		Elevation: 436,
		Latitude:  -3.421667099,
		Longitude: -48.3355560303,
		Timezone:  "America/Belem",
	},
	"SNAL": {
		ICAO:      "SNAL",
		IATA:      "APQ",
		Name:      "Arapiraca Airport",
		City:      "Arapiraca",
		State:     "Alagoas",
		Country:   "BR",
		Elevation: 886,
		Latitude:  -9.7753601074,
		Longitude: -36.6291999817,
		Timezone:  "America/Maceio",
	},
	"SNAM": {
		ICAO:      "SNAM",
		Name:      "Santo Antonio do Amparo Airport",
		City:      "Santo Antonio Do Amparo",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3601,
		Latitude:  -20.9134998322,
		Longitude: -44.8928985596,
		Timezone:  "America/Sao_Paulo",
	},
	"SNAP": {
		ICAO:      "SNAP",
		Name:      "Janauba Airport",
		City:      "Janauba",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1732,
		Latitude:  -15.732000351,
		Longitude: -43.3231010437,
		Timezone:  "America/Sao_Paulo",
	},
	"SNAR": {
		ICAO:      "SNAR",
		IATA:      "AMJ",
		Name:      "Cirilo Queiroz Airport",
		City:      "Almenara",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 640,
		Latitude:  -16.18389,
		Longitude: -40.66722,
		Timezone:  "America/Sao_Paulo",
	},
	"SNAS": {
		ICAO:      "SNAS",
		Name:      "Tres Marias Airport",
		City:      "Tres Marias",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2579,
		Latitude:  -18.2224006653,
		Longitude: -45.188999176,
		Timezone:  "America/Sao_Paulo",
	},
	"SNAT": {
		ICAO:      "SNAT",
		Name:      "Aracati Airport",
		City:      "Aracati",
		State:     "Ceara",
		Country:   "BR",
		Elevation: 128,
		Latitude:  -4.5671401024,
		Longitude: -37.7999992371,
		Timezone:  "America/Fortaleza",
	},
	"SNAU": {
		ICAO:      "SNAU",
		IATA:      "0",
		Name:      "Aeroclube Airport",
		City:      "Aracaju",
		State:     "Sergipe",
		Country:   "BR",
		Elevation: 23,
		Latitude:  -10.9033002853,
		Longitude: -37.0830993652,
		Timezone:  "America/Maceio",
	},
	"SNAV": {
		ICAO:      "SNAV",
		Name:      "Agrovale Airport",
		City:      "Juazeiro",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1289,
		Latitude:  -9.50417,
		Longitude: -40.36944,
		Timezone:  "America/Bahia",
	},
	"SNAW": {
		ICAO:      "SNAW",
		Name:      "Fazenda Boa Fe Airport",
		City:      "Conquista",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2461,
		Latitude:  -19.8769435883,
		Longitude: -47.6797218323,
		Timezone:  "America/Sao_Paulo",
	},
	"SNAY": {
		ICAO:      "SNAY",
		Name:      "Fazenda Rio Lages Airport",
		City:      "Sao Felix Do Xingu",
		State:     "Para",
		Country:   "BR",
		Elevation: 853,
		Latitude:  -6.2811107635,
		Longitude: -51.7461128235,
		Timezone:  "America/Belem",
	},
	"SNAZ": {
		ICAO:      "SNAZ",
		Name:      "Amargosa Airport",
		City:      "Amargosa",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1394,
		Latitude:  -12.9935998917,
		Longitude: -39.6399002075,
		Timezone:  "America/Bahia",
	},
	"SNBB": {
		ICAO:      "SNBB",
		Name:      "Fazenda Agronol Airport",
		City:      "Luis Eduardo Magalhaes",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 2464,
		Latitude:  -11.947221756,
		Longitude: -45.725276947,
		Timezone:  "America/Bahia",
	},
	"SNBC": {
		ICAO:      "SNBC",
		IATA:      "BDC",
		Name:      "Barra do Corda Airport",
		City:      "Barra Do Corda",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 509,
		Latitude:  -5.5025,
		Longitude: -45.215833,
		Timezone:  "America/Fortaleza",
	},
	"SNBD": {
		ICAO:      "SNBD",
		Name:      "Sobradinho Airport",
		City:      "Juazeiro",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1240,
		Latitude:  -9.4620103836,
		Longitude: -40.8242988586,
		Timezone:  "America/Bahia",
	},
	"SNBE": {
		ICAO:      "SNBE",
		Name:      "Fazenda Botuvera Airport",
		City:      "Paranatinga",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1001,
		Latitude:  -14.4702777778,
		Longitude: -53.6677777778,
		Timezone:  "America/Cuiaba",
	},
	"SNBG": {
		ICAO:      "SNBG",
		Name:      "Baixo Guandu - Aimores Airport",
		City:      "Baixo Guandu",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 279,
		Latitude:  -19.4990005493,
		Longitude: -41.0418014526,
		Timezone:  "America/Sao_Paulo",
	},
	"SNBI": {
		ICAO:      "SNBI",
		Name:      "Bacabal Airport",
		City:      "Bacabal",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 64,
		Latitude:  -4.2277097702,
		Longitude: -44.8199996948,
		Timezone:  "America/Fortaleza",
	},
	"SNBJ": {
		ICAO:      "SNBJ",
		Name:      "Belo Jardim Airport",
		City:      "Belo Jardim",
		State:     "Pernambuco",
		Country:   "BR",
		Elevation: 2100,
		Latitude:  -8.3452701569,
		Longitude: -36.4412002563,
		Timezone:  "America/Recife",
	},
	"SNBL": {
		ICAO:      "SNBL",
		IATA:      "BVM",
		Name:      "Belmonte Airport",
		City:      "Belmonte",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 33,
		Latitude:  -15.8717002869,
		Longitude: -38.8718986511,
		Timezone:  "America/Bahia",
	},
	"SNBM": {
		ICAO:      "SNBM",
		Name:      "Cristiano Ferreira Varella Airport",
		City:      "Muriae",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 886,
		Latitude:  -21.1261005402,
		Longitude: -42.3944015503,
		Timezone:  "America/Sao_Paulo",
	},
	"SNBN": {
		ICAO:      "SNBN",
		Name:      "Balbinot Airport",
		City:      "Portel",
		State:     "Para",
		Country:   "BR",
		Elevation: 203,
		Latitude:  -2.8758330345,
		Longitude: -50.4916687012,
		Timezone:  "America/Belem",
	},
	"SNBO": {
		ICAO:      "SNBO",
		IATA:      "0",
		Name:      "Boquira Airport",
		City:      "Boquira",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1886,
		Latitude:  -12.790599823,
		Longitude: -42.7289009094,
		Timezone:  "America/Bahia",
	},
	"SNBR": {
		ICAO:      "SNBR",
		IATA:      "BRA",
		Name:      "Barreiras Airport",
		City:      "Barreiras",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 2447,
		Latitude:  -12.0789003372,
		Longitude: -45.0089988708,
		Timezone:  "America/Bahia",
	},
	"SNBS": {
		ICAO:      "SNBS",
		IATA:      "BSS",
		Name:      "Balsas Airport",
		City:      "Balsas",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 930,
		Latitude:  -7.5260300636,
		Longitude: -46.0532989502,
		Timezone:  "America/Fortaleza",
	},
	"SNBT": {
		ICAO:      "SNBT",
		Name:      "Benedito Leite Airport",
		City:      "Benedito Leite",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 984,
		Latitude:  -7.2169442177,
		Longitude: -44.5505561829,
		Timezone:  "America/Fortaleza",
	},
	"SNBU": {
		ICAO:      "SNBU",
		IATA:      "BMS",
		Name:      "Socrates Mariani Bittencourt Airport",
		City:      "Brumado",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1673,
		Latitude:  -14.255399704,
		Longitude: -41.8175010681,
		Timezone:  "America/Bahia",
	},
	"SNBV": {
		ICAO:      "SNBV",
		Name:      "Campo de Boi Airport",
		City:      "Ipixuna Do Para",
		State:     "Para",
		Country:   "BR",
		Elevation: 186,
		Latitude:  -2.5233099461,
		Longitude: -47.5173988342,
		Timezone:  "America/Belem",
	},
	"SNBW": {
		ICAO:      "SNBW",
		Name:      "Baiao Airport",
		City:      "Baiao",
		State:     "Para",
		Country:   "BR",
		Elevation: 105,
		Latitude:  -2.8002779484,
		Longitude: -49.6672210693,
		Timezone:  "America/Belem",
	},
	"SNBX": {
		ICAO:      "SNBX",
		IATA:      "BQQ",
		Name:      "Barra Airport",
		City:      "Barra",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1345,
		Latitude:  -11.0808000565,
		Longitude: -43.1474990845,
		Timezone:  "America/Bahia",
	},
	"SNBY": {
		ICAO:      "SNBY",
		Name:      "Bambui Airport",
		City:      "Bambui",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2523,
		Latitude:  -20.0363998413,
		Longitude: -45.9723014832,
		Timezone:  "America/Sao_Paulo",
	},
	"SNBZ": {
		ICAO:      "SNBZ",
		Name:      "Paramirim Airport",
		City:      "Paramirim",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1788,
		Latitude:  -13.4461107254,
		Longitude: -42.249168396,
		Timezone:  "America/Bahia",
	},
	"SNCA": {
		ICAO:      "SNCA",
		Name:      "Campo Belo Airport",
		City:      "Campo Belo",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3182,
		Latitude:  -20.8922996521,
		Longitude: -45.3350982666,
		Timezone:  "America/Sao_Paulo",
	},
	"SNCB": {
		ICAO:      "SNCB",
		Name:      "Castanhal Airport",
		City:      "Prainha",
		State:     "Para",
		Country:   "BR",
		Elevation: 502,
		Latitude:  -2.5236110687,
		Longitude: -53.9397201538,
		Timezone:  "America/Santarem",
	},
	"SNCC": {
		ICAO:      "SNCC",
		Name:      "Calcoene Airport",
		City:      "Calcoene",
		State:     "Amapa",
		Country:   "BR",
		Elevation: 59,
		Latitude:  2.5147199631,
		Longitude: -50.9522018433,
		Timezone:  "America/Belem",
	},
	"SNCE": {
		ICAO:      "SNCE",
		IATA:      "0",
		Name:      "Campo do Meio Airport",
		City:      "Campo Do Meio",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2559,
		Latitude:  -21.1061000824,
		Longitude: -45.8046989441,
		Timezone:  "America/Sao_Paulo",
	},
	"SNCF": {
		ICAO:      "SNCF",
		Name:      "Continental Airport",
		City:      "Altamira",
		State:     "Para",
		Country:   "BR",
		Elevation: 485,
		Latitude:  -3.2952780724,
		Longitude: -52.2591667175,
		Timezone:  "America/Santarem",
	},
	"SNCG": {
		ICAO:      "SNCG",
		Name:      "Campos Gerais Airport",
		City:      "Campos Gerais",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2838,
		Latitude:  -21.2261104584,
		Longitude: -45.7791671753,
		Timezone:  "America/Sao_Paulo",
	},
	"SNCI": {
		ICAO:      "SNCI",
		Name:      "Cibrasa Airport",
		City:      "Capanema",
		State:     "Para",
		Country:   "BR",
		Elevation: 174,
		Latitude:  -1.2309999466,
		Longitude: -47.2005996704,
		Timezone:  "America/Belem",
	},
	"SNCJ": {
		ICAO:      "SNCJ",
		Name:      "Piquiatuba Airport",
		City:      "Santarem",
		State:     "Para",
		Country:   "BR",
		Elevation: 407,
		Latitude:  -2.546667099,
		Longitude: -54.7050018311,
		Timezone:  "America/Santarem",
	},
	"SNCL": {
		ICAO:      "SNCL",
		Name:      "Lorenzo Airport",
		City:      "Cairu",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 3,
		Latitude:  -13.3894443512,
		Longitude: -38.9099998474,
		Timezone:  "America/Bahia",
	},
	"SNCP": {
		ICAO:      "SNCP",
		IATA:      "CTP",
		Name:      "Carutapera Airport",
		City:      "Carutapera",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 85,
		Latitude:  -1.2502779961,
		Longitude: -46.0172233582,
		Timezone:  "America/Fortaleza",
	},
	"SNCQ": {
		ICAO:      "SNCQ",
		Name:      "Cocorobo Airport",
		City:      "Euclides Da Cunha",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1509,
		Latitude:  -10.5272216797,
		Longitude: -39.0330543518,
		Timezone:  "America/Bahia",
	},
	"SNCS": {
		ICAO:      "SNCS",
		Name:      "Campos Sales Airport",
		City:      "Campos Sales",
		State:     "Ceara",
		Country:   "BR",
		Elevation: 2001,
		Latitude:  -7.0530200005,
		Longitude: -40.3585014343,
		Timezone:  "America/Fortaleza",
	},
	"SNCT": {
		ICAO:      "SNCT",
		Name:      "Ubaporanga Airport",
		City:      "Caratinga",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2067,
		Latitude:  -19.7252998352,
		Longitude: -42.1120986938,
		Timezone:  "America/Sao_Paulo",
	},
	"SNCU": {
		ICAO:      "SNCU",
		IATA:      "CPU",
		Name:      "Cururupu Airport",
		City:      "Cururupu",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 39,
		Latitude:  -1.821111,
		Longitude: -44.866944,
		Timezone:  "America/Fortaleza",
	},
	"SNCV": {
		ICAO:      "SNCV",
		IATA:      "0",
		Name:      "Campina Verde Airport",
		City:      "Campina Verde",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1805,
		Latitude:  -19.5380992889,
		Longitude: -49.4949989319,
		Timezone:  "America/Sao_Paulo",
	},
	"SNCW": {
		ICAO:      "SNCW",
		Name:      "Centro de Lancamento de Alcantara Airport",
		City:      "Alcantara",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 148,
		Latitude:  -2.3729999065,
		Longitude: -44.3964004517,
		Timezone:  "America/Fortaleza",
	},
	"SNCX": {
		ICAO:      "SNCX",
		IATA:      "QCH",
		Name:      "Colatina Airport",
		City:      "Colatina",
		State:     "Espirito-Santo",
		Country:   "BR",
		Elevation: 492,
		Latitude:  -19.4869995117,
		Longitude: -40.5793991089,
		Timezone:  "America/Sao_Paulo",
	},
	"SNCY": {
		ICAO:      "SNCY",
		Name:      "Campo da Praia Airport",
		City:      "Coruripe",
		State:     "Alagoas",
		Country:   "BR",
		Elevation: 138,
		Latitude:  -10.2208328247,
		Longitude: -36.2861099243,
		Timezone:  "America/Maceio",
	},
	"SNCZ": {
		ICAO:      "SNCZ",
		Name:      "Ponte Nova Airport",
		City:      "Ponte Nova",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1877,
		Latitude:  -20.4032993317,
		Longitude: -42.9165000916,
		Timezone:  "America/Sao_Paulo",
	},
	"SNDB": {
		ICAO:      "SNDB",
		Name:      "Ruropolis Airport",
		City:      "Ruropolis",
		State:     "Para",
		Country:   "BR",
		Elevation: 325,
		Latitude:  -4.0855560303,
		Longitude: -54.9108314514,
		Timezone:  "America/Santarem",
	},
	"SNDC": {
		ICAO:      "SNDC",
		IATA:      "RDC",
		Name:      "Redencao Airport",
		City:      "Redencao",
		State:     "Para",
		Country:   "BR",
		Elevation: 670,
		Latitude:  -8.0332899094,
		Longitude: -49.9799003601,
		Timezone:  "America/Belem",
	},
	"SNDD": {
		ICAO:      "SNDD",
		Name:      "Fazenda Eldorado Airport",
		City:      "Igarape Do Meio",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 118,
		Latitude:  -3.6922221184,
		Longitude: -45.2700004578,
		Timezone:  "America/Fortaleza",
	},
	"SNDF": {
		ICAO:      "SNDF",
		Name:      "Fazenda Santa Lucia Airport",
		City:      "Santa Ines",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 272,
		Latitude:  -3.6549999714,
		Longitude: -45.4511108398,
		Timezone:  "America/Fortaleza",
	},
	"SNDH": {
		ICAO:      "SNDH",
		Name:      "Aba Airport",
		City:      "Barreiras",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1682,
		Latitude:  -12.1602783203,
		Longitude: -45.0224990845,
		Timezone:  "America/Bahia",
	},
	"SNDI": {
		ICAO:      "SNDI",
		Name:      "Pista Castanheirinho Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 623,
		Latitude:  -7.0997219086,
		Longitude: -56.8522224426,
		Timezone:  "America/Santarem",
	},
	"SNDJ": {
		ICAO:      "SNDJ",
		Name:      "Santana dos Brejos Airport",
		City:      "Santana Dos Brejos",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1759,
		Latitude:  -12.976111412,
		Longitude: -44.0391654968,
		Timezone:  "America/Bahia",
	},
	"SNDN": {
		ICAO:      "SNDN",
		IATA:      "LEP",
		Name:      "Leopoldina Airport",
		City:      "Leopoldina",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 919,
		Latitude:  -21.4661006927,
		Longitude: -42.7270011902,
		Timezone:  "America/Sao_Paulo",
	},
	"SNDO": {
		ICAO:      "SNDO",
		Name:      "Ouro Branco Airport",
		City:      "Paracatu",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3066,
		Latitude:  -16.7163887024,
		Longitude: -47.0177764893,
		Timezone:  "America/Sao_Paulo",
	},
	"SNDP": {
		ICAO:      "SNDP",
		Name:      "Palmares Airport",
		City:      "Acara",
		State:     "Para",
		Country:   "BR",
		Elevation: 218,
		Latitude:  -2.2602779865,
		Longitude: -48.5983314514,
		Timezone:  "America/Belem",
	},
	"SNDQ": {
		ICAO:      "SNDQ",
		Name:      "Fazenda Soya Airport",
		City:      "Sao Desiderio",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 2549,
		Latitude:  -12.9211111069,
		Longitude: -45.569442749,
		Timezone:  "America/Bahia",
	},
	"SNDR": {
		ICAO:      "SNDR",
		Name:      "Domingos Rego Airport",
		City:      "Timon",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 373,
		Latitude:  -5.0797901154,
		Longitude: -42.8736991882,
		Timezone:  "America/Fortaleza",
	},
	"SNDT": {
		ICAO:      "SNDT",
		IATA:      "DTI",
		Name:      "Diamantina Airport",
		City:      "Diamantina",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 4446,
		Latitude:  -18.232000351,
		Longitude: -43.6503982544,
		Timezone:  "America/Sao_Paulo",
	},
	"SNDU": {
		ICAO:      "SNDU",
		Name:      "Ponta Do Sol Airport",
		City:      "Capitolio",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2854,
		Latitude:  -20.638333,
		Longitude: -45.998056,
		Timezone:  "America/Sao_Paulo",
	},
	"SNDV": {
		ICAO:      "SNDV",
		IATA:      "DIQ",
		Name:      "Divinopolis Airport",
		City:      "Divinopolis",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2608,
		Latitude:  -20.1807003021,
		Longitude: -44.8708992004,
		Timezone:  "America/Sao_Paulo",
	},
	"SNDW": {
		ICAO:      "SNDW",
		Name:      "Divisa Airport",
		City:      "Encruzilhada",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3051,
		Latitude:  -15.7172222137,
		Longitude: -41.0172233582,
		Timezone:  "America/Bahia",
	},
	"SNDX": {
		ICAO:      "SNDX",
		Name:      "Cachoeira Airport",
		City:      "Coruripe",
		State:     "Alagoas",
		Country:   "BR",
		Elevation: 345,
		Latitude:  -10.0399999619,
		Longitude: -36.3263893127,
		Timezone:  "America/Maceio",
	},
	"SNEA": {
		ICAO:      "SNEA",
		Name:      "Picarrao Airport",
		City:      "Estrela Do Sul",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3166,
		Latitude:  -18.7213897705,
		Longitude: -47.8655548096,
		Timezone:  "America/Sao_Paulo",
	},
	"SNEB": {
		ICAO:      "SNEB",
		Name:      "Paragominas Airport",
		City:      "Paragominas",
		State:     "Para",
		Country:   "BR",
		Elevation: 508,
		Latitude:  -3.0252780914,
		Longitude: -47.3169441223,
		Timezone:  "America/Belem",
	},
	"SNEC": {
		ICAO:      "SNEC",
		Name:      "Outeiros da Brisas Airport",
		City:      "Porto Seguro",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 130,
		Latitude:  -16.7130565643,
		Longitude: -39.1430549622,
		Timezone:  "America/Bahia",
	},
	"SNED": {
		ICAO:      "SNED",
		IATA:      "CNV",
		Name:      "Canavieiras Airport",
		City:      "Canavieiras",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 19,
		Latitude:  -15.6669998169,
		Longitude: -38.95470047,
		Timezone:  "America/Bahia",
	},
	"SNEE": {
		ICAO:      "SNEE",
		Name:      "Jaua Airport",
		City:      "Camacari",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 76,
		Latitude:  -12.8058586121,
		Longitude: -38.2437515259,
		Timezone:  "America/Bahia",
	},
	"SNEH": {
		ICAO:      "SNEH",
		Name:      "Mineracao Porquinho Airport",
		City:      "Itaituba",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 298,
		Latitude:  -5.167222023,
		Longitude: -57.625831604,
		Timezone:  "America/Manaus",
	},
	"SNEJ": {
		ICAO:      "SNEJ",
		Name:      "Monte Verde Airport",
		City:      "Camanducaia",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 5102,
		Latitude:  -22.8597221375,
		Longitude: -46.0374984741,
		Timezone:  "America/Sao_Paulo",
	},
	"SNEK": {
		ICAO:      "SNEK",
		Name:      "Moria Airport",
		City:      "Dom Bosco",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1201,
		Latitude:  -16.8838882446,
		Longitude: -46.2363891602,
		Timezone:  "America/Sao_Paulo",
	},
	"SNEL": {
		ICAO:      "SNEL",
		Name:      "Belterra Airport",
		City:      "Belterra",
		State:     "Para",
		Country:   "BR",
		Elevation: 541,
		Latitude:  -2.6669440269,
		Longitude: -54.900554657,
		Timezone:  "America/Santarem",
	},
	"SNEM": {
		ICAO:      "SNEM",
		IATA:      "0",
		Name:      "Encanta Moca Airport",
		City:      "Recife",
		State:     "Pernambuco",
		Country:   "BR",
		Elevation: 10,
		Latitude:  -8.0941696167,
		Longitude: -34.8927993774,
		Timezone:  "America/Recife",
	},
	"SNEN": {
		ICAO:      "SNEN",
		Name:      "Fazenda Calumbi Airport",
		City:      "Fortuna",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 689,
		Latitude:  -5.8133330345,
		Longitude: -44.1377792358,
		Timezone:  "America/Fortaleza",
	},
	"SNEO": {
		ICAO:      "SNEO",
		Name:      "Dezoito Airport",
		City:      "Neropolis",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2700,
		Latitude:  -16.441110611,
		Longitude: -49.19972229,
		Timezone:  "America/Sao_Paulo",
	},
	"SNES": {
		ICAO:      "SNES",
		Name:      "Esplanada Airport",
		City:      "Esplanada",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 164,
		Latitude:  -11.7505559921,
		Longitude: -37.9502792358,
		Timezone:  "America/Bahia",
	},
	"SNEU": {
		ICAO:      "SNEU",
		Name:      "Euclides da Cunha Airport",
		City:      "Euclides Da Cunha",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1499,
		Latitude:  -10.527299881,
		Longitude: -39.032699585,
		Timezone:  "America/Bahia",
	},
	"SNEW": {
		ICAO:      "SNEW",
		Name:      "Fazenda Congonhas Airport",
		City:      "Carneirinho",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1375,
		Latitude:  -19.6625995636,
		Longitude: -50.9513015747,
		Timezone:  "America/Sao_Paulo",
	},
	"SNEY": {
		ICAO:      "SNEY",
		Name:      "Fazenda Lagoa do Morro Airport",
		City:      "Brejoes",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 2666,
		Latitude:  -13.1024999619,
		Longitude: -39.9036102295,
		Timezone:  "America/Bahia",
	},
	"SNEZ": {
		ICAO:      "SNEZ",
		Name:      "Fazenda Palmeira do Capim Airport",
		City:      "Novo Brasil",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1220,
		Latitude:  -15.959444046,
		Longitude: -50.7138900757,
		Timezone:  "America/Sao_Paulo",
	},
	"SNFA": {
		ICAO:      "SNFA",
		Name:      "Fazenda Santo Antonio Airport",
		City:      "Unai",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1795,
		Latitude:  -16.7161998749,
		Longitude: -46.5211982727,
		Timezone:  "America/Sao_Paulo",
	},
	"SNFB": {
		ICAO:      "SNFB",
		Name:      "Fazenda Minas Gerais Airport",
		City:      "Pontes E Lacerda",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2395,
		Latitude:  -14.777222,
		Longitude: -59.046944,
		Timezone:  "America/Cuiaba",
	},
	"SNFD": {
		ICAO:      "SNFD",
		Name:      "Fazenda Santo Antonio Airport",
		City:      "Buritizeiro",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1739,
		Latitude:  -17.0802783966,
		Longitude: -45.4472236633,
		Timezone:  "America/Sao_Paulo",
	},
	"SNFE": {
		ICAO:      "SNFE",
		Name:      "Alfenas Airport",
		City:      "Alfenas",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2871,
		Latitude:  -21.4314002991,
		Longitude: -45.9320983887,
		Timezone:  "America/Sao_Paulo",
	},
	"SNFF": {
		ICAO:      "SNFF",
		Name:      "Feijo Airport",
		City:      "Fortaleza",
		State:     "Ceara",
		Country:   "BR",
		Elevation: 82,
		Latitude:  -3.8094439507,
		Longitude: -38.6169433594,
		Timezone:  "America/Fortaleza",
	},
	"SNFG": {
		ICAO:      "SNFG",
		Name:      "Fazenda Citropar Airport",
		City:      "Capitao Poco",
		State:     "Para",
		Country:   "BR",
		Elevation: 220,
		Latitude:  -1.8416670561,
		Longitude: -47.1841659546,
		Timezone:  "America/Belem",
	},
	"SNFH": {
		ICAO:      "SNFH",
		Name:      "Fazenda Dom Felipe Airport",
		City:      "Ponta Pora",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1857,
		Latitude:  -22.55917,
		Longitude: -55.37583,
		Timezone:  "America/Campo_Grande",
	},
	"SNFI": {
		ICAO:      "SNFI",
		Name:      "Fazenda Sao Jose do Parnaiba Airport",
		City:      "Tupaciguara",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1676,
		Latitude:  -18.3875007629,
		Longitude: -48.8616676331,
		Timezone:  "America/Sao_Paulo",
	},
	"SNFJ": {
		ICAO:      "SNFJ",
		Name:      "Pousada Thaimacu Airport",
		City:      "Jacareacanga",
		State:     "Para",
		Country:   "BR",
		Elevation: 751,
		Latitude:  -9.0627784729,
		Longitude: -56.593334198,
		Timezone:  "America/Santarem",
	},
	"SNFK": {
		ICAO:      "SNFK",
		Name:      "Francisco Sa Airport",
		City:      "Francisco Sa",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2484,
		Latitude:  -16.4383335114,
		Longitude: -43.4694442749,
		Timezone:  "America/Sao_Paulo",
	},
	"SNFL": {
		ICAO:      "SNFL",
		Name:      "Garimbo Tocantinzinho Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 590,
		Latitude:  -6.0505561829,
		Longitude: -56.3022232056,
		Timezone:  "America/Santarem",
	},
	"SNFN": {
		ICAO:      "SNFN",
		Name:      "Fazenda Santa Fe Airport",
		City:      "Jardinopolis",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1739,
		Latitude:  -20.94,
		Longitude: -47.87,
		Timezone:  "America/Sao_Paulo",
	},
	"SNFO": {
		ICAO:      "SNFO",
		IATA:      "0",
		Name:      "Formiga Airport",
		City:      "Formiga",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3337,
		Latitude:  -20.3952999115,
		Longitude: -45.4836006165,
		Timezone:  "America/Sao_Paulo",
	},
	"SNFP": {
		ICAO:      "SNFP",
		Name:      "Fazenda Pirapo Airport",
		City:      "Sorriso",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1273,
		Latitude:  -12.181388855,
		Longitude: -55.7886123657,
		Timezone:  "America/Cuiaba",
	},
	"SNFQ": {
		ICAO:      "SNFQ",
		Name:      "Fazenda Paineira Airport",
		City:      "Sao Sebastiao Da Bela Vista",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3001,
		Latitude:  -22.1494445801,
		Longitude: -45.7625007629,
		Timezone:  "America/Sao_Paulo",
	},
	"SNFR": {
		ICAO:      "SNFR",
		Name:      "Belem de Sao Francisco Airport",
		City:      "Belem De Sao Francisco",
		State:     "Pernambuco",
		Country:   "BR",
		Elevation: 1034,
		Latitude:  -8.7695903778,
		Longitude: -38.9496002197,
		Timezone:  "America/Recife",
	},
	"SNFS": {
		ICAO:      "SNFS",
		Name:      "Fazenda Mimoso S/A Airport",
		City:      "Barreiras",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 2536,
		Latitude:  -11.871389389,
		Longitude: -45.7372207642,
		Timezone:  "America/Bahia",
	},
	"SNFT": {
		ICAO:      "SNFT",
		Name:      "Fazenda Serinhaem Airport",
		City:      "Itubera",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 9,
		Latitude:  -13.8366670609,
		Longitude: -38.9936103821,
		Timezone:  "America/Bahia",
	},
	"SNFU": {
		ICAO:      "SNFU",
		Name:      "Frutal Airport",
		City:      "Frutal",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1808,
		Latitude:  -20.0027999878,
		Longitude: -48.9584007263,
		Timezone:  "America/Sao_Paulo",
	},
	"SNFV": {
		ICAO:      "SNFV",
		Name:      "Lago Joanes Airport",
		City:      "Simoes Filho",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 239,
		Latitude:  -12.6666669846,
		Longitude: -38.3958320618,
		Timezone:  "America/Bahia",
	},
	"SNFW": {
		ICAO:      "SNFW",
		Name:      "Sao Joao do Piaui Airport",
		City:      "Sao Joao Do Piaui",
		State:     "Piaui",
		Country:   "BR",
		Elevation: 1004,
		Latitude:  -8.391111,
		Longitude: -42.250556,
		Timezone:  "America/Fortaleza",
	},
	"SNFX": {
		ICAO:      "SNFX",
		IATA:      "SXX",
		Name:      "Sao Felix do Xingu Airport",
		City:      "Sao Felix Do Xingu",
		State:     "Para",
		Country:   "BR",
		Elevation: 656,
		Latitude:  -6.6402778625,
		Longitude: -51.9900016785,
		Timezone:  "America/Belem",
	},
	"SNFZ": {
		ICAO:      "SNFZ",
		Name:      "Fazenda Villa Terezinha Airport",
		City:      "Bocaiuva",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2765,
		Latitude:  -17.405500412,
		Longitude: -43.9403991699,
		Timezone:  "America/Sao_Paulo",
	},
	"SNGA": {
		ICAO:      "SNGA",
		IATA:      "GUZ",
		Name:      "Guarapari Airport",
		City:      "Guarapari",
		State:     "Espirito-Santo",
		Country:   "BR",
		Elevation: 28,
		Latitude:  -20.6464996338,
		Longitude: -40.4919013977,
		Timezone:  "America/Sao_Paulo",
	},
	"SNGB": {
		ICAO:      "SNGB",
		Name:      "Gilbues Airport",
		City:      "Gilbues",
		State:     "Piaui",
		Country:   "BR",
		Elevation: 1476,
		Latitude:  -9.8336105347,
		Longitude: -45.3672218323,
		Timezone:  "America/Fortaleza",
	},
	"SNGC": {
		ICAO:      "SNGC",
		Name:      "Pista Canaa Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 741,
		Latitude:  -7.193611145,
		Longitude: -57.0488891602,
		Timezone:  "America/Santarem",
	},
	"SNGD": {
		ICAO:      "SNGD",
		IATA:      "GDP",
		Name:      "Guadalupe Airport",
		City:      "Guadalupe",
		State:     "Piaui",
		Country:   "BR",
		Elevation: 564,
		Latitude:  -6.7822198868,
		Longitude: -43.5821990967,
		Timezone:  "America/Fortaleza",
	},
	"SNGG": {
		ICAO:      "SNGG",
		Name:      "Bom Jesus do Gurgueia Airport",
		City:      "Bom Jesus Do Gurgueia",
		State:     "Piaui",
		Country:   "BR",
		Elevation: 1066,
		Latitude:  -9.0572500229,
		Longitude: -44.3711013794,
		Timezone:  "America/Fortaleza",
	},
	"SNGH": {
		ICAO:      "SNGH",
		Name:      "Guanhaes Airport",
		City:      "Guanhaes",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2618,
		Latitude:  -18.7077007294,
		Longitude: -42.8389015198,
		Timezone:  "America/Sao_Paulo",
	},
	"SNGI": {
		ICAO:      "SNGI",
		IATA:      "GNM",
		Name:      "Guanambi Airport",
		City:      "Guanambi",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1815,
		Latitude:  -14.2082004547,
		Longitude: -42.7461013794,
		Timezone:  "America/Bahia",
	},
	"SNGJ": {
		ICAO:      "SNGJ",
		IATA:      "0",
		Name:      "Grajau Airport",
		City:      "Grajau",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 773,
		Latitude:  -5.8002800941,
		Longitude: -46.1338996887,
		Timezone:  "America/Fortaleza",
	},
	"SNGK": {
		ICAO:      "SNGK",
		Name:      "Fazenda Bela Vista Airport",
		City:      "Monte Alegre De Minas",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1984,
		Latitude:  -19.0483322144,
		Longitude: -48.9722213745,
		Timezone:  "America/Sao_Paulo",
	},
	"SNGM": {
		ICAO:      "SNGM",
		IATA:      "GMS",
		Name:      "Fazenda Canada Airport",
		City:      "Uberlandia",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2624,
		Latitude:  -18.8624992371,
		Longitude: -48.4152793884,
		Timezone:  "America/Sao_Paulo",
	},
	"SNGN": {
		ICAO:      "SNGN",
		IATA:      "QGP",
		Name:      "Garanhuns Airport",
		City:      "Garanhuns",
		State:     "Pernambuco",
		Country:   "BR",
		Elevation: 2533,
		Latitude:  -8.834280014,
		Longitude: -36.4715995789,
		Timezone:  "America/Recife",
	},
	"SNGO": {
		ICAO:      "SNGO",
		Name:      "Fazenda Nova Airport",
		City:      "Uberlandia",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2689,
		Latitude:  -19.0477771759,
		Longitude: -48.5822219849,
		Timezone:  "America/Sao_Paulo",
	},
	"SNGP": {
		ICAO:      "SNGP",
		Name:      "Fazenda Santa Maria Airport",
		City:      "Tupaciguara",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2624,
		Latitude:  -18.5869445801,
		Longitude: -48.8691673279,
		Timezone:  "America/Sao_Paulo",
	},
	"SNGQ": {
		ICAO:      "SNGQ",
		Name:      "Fazenda Estrela Dalva Airport",
		City:      "Rondon Do Para",
		State:     "Para",
		Country:   "BR",
		Elevation: 446,
		Latitude:  -4.2508330345,
		Longitude: -48.7680549622,
		Timezone:  "America/Belem",
	},
	"SNGR": {
		ICAO:      "SNGR",
		Name:      "Gorotire Airport",
		City:      "Gorotire",
		State:     "Para",
		Country:   "BR",
		Elevation: 656,
		Latitude:  -7.7836108208,
		Longitude: -51.1338882446,
		Timezone:  "America/Belem",
	},
	"SNGS": {
		ICAO:      "SNGS",
		Name:      "Aeroclube de Alagoas Airport",
		City:      "Maceio",
		State:     "Alagoas",
		Country:   "BR",
		Elevation: 285,
		Latitude:  -9.5874996185,
		Longitude: -35.7577781677,
		Timezone:  "America/Maceio",
	},
	"SNGT": {
		ICAO:      "SNGT",
		Name:      "Gentio de Ouro Airport",
		City:      "Gentio De Ouro",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 3501,
		Latitude:  -11.4416999817,
		Longitude: -42.5181999207,
		Timezone:  "America/Bahia",
	},
	"SNGU": {
		ICAO:      "SNGU",
		Name:      "Gurupa Airport",
		City:      "Gurupa",
		State:     "Para",
		Country:   "BR",
		Elevation: 66,
		Latitude:  -1.4169440269,
		Longitude: -51.6338882446,
		Timezone:  "America/Belem",
	},
	"SNGW": {
		ICAO:      "SNGW",
		Name:      "Fazenda Cauaxi Airport",
		City:      "Paragominas",
		State:     "Para",
		Country:   "BR",
		Elevation: 754,
		Latitude:  -3.7633330822,
		Longitude: -48.1738891602,
		Timezone:  "America/Belem",
	},
	"SNGX": {
		ICAO:      "SNGX",
		Name:      "Guaxupe Airport",
		City:      "Guaxupe",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2786,
		Latitude:  -21.3264007568,
		Longitude: -46.7312011719,
		Timezone:  "America/Sao_Paulo",
	},
	"SNHD": {
		ICAO:      "SNHD",
		Name:      "Fazenda Floresta do Lobo Airport",
		City:      "Uberlandia",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3159,
		Latitude:  -19.0775,
		Longitude: -48.130278,
		Timezone:  "America/Sao_Paulo",
	},
	"SNHE": {
		ICAO:      "SNHE",
		Name:      "Fazenda Inhumas do Chapadao Airport",
		City:      "Uberaba",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2792,
		Latitude:  -19.3022212982,
		Longitude: -48.2041664124,
		Timezone:  "America/Sao_Paulo",
	},
	"SNHG": {
		ICAO:      "SNHG",
		Name:      "Fazenda Varjadao Airport",
		City:      "Sandolandia",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 794,
		Latitude:  -12.341389,
		Longitude: -49.621111,
		Timezone:  "America/Araguaina",
	},
	"SNHJ": {
		ICAO:      "SNHJ",
		Name:      "Usina de Jose Bonifacio Airport",
		City:      "Jose Bonifacio",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1404,
		Latitude:  -21.095,
		Longitude: -49.914722,
		Timezone:  "America/Sao_Paulo",
	},
	"SNHK": {
		ICAO:      "SNHK",
		Name:      "Nacional Grafite I Airport",
		City:      "Itapecerica",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3051,
		Latitude:  -20.4344444275,
		Longitude: -45.1383323669,
		Timezone:  "America/Sao_Paulo",
	},
	"SNHL": {
		ICAO:      "SNHL",
		Name:      "Lagoa da Taboca Airport",
		City:      "Joao Pinheiro",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1969,
		Latitude:  -17.1933326721,
		Longitude: -45.6655578613,
		Timezone:  "America/Sao_Paulo",
	},
	"SNHO": {
		ICAO:      "SNHO",
		Name:      "Nacional de Grafite II Airport",
		City:      "Pedra Azul",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1968,
		Latitude:  -15.8361110687,
		Longitude: -41.0763893127,
		Timezone:  "America/Sao_Paulo",
	},
	"SNHP": {
		ICAO:      "SNHP",
		Name:      "Herbert Mata Pires Airport",
		City:      "Itaberaba",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1188,
		Latitude:  -12.402463913,
		Longitude: -40.4746551514,
		Timezone:  "America/Bahia",
	},
	"SNHQ": {
		ICAO:      "SNHQ",
		Name:      "Nacional de Grafite III Airport",
		City:      "Salto Da Divisa",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 984,
		Latitude:  -16.1811103821,
		Longitude: -39.9319458008,
		Timezone:  "America/Bahia",
	},
	"SNHR": {
		ICAO:      "SNHR",
		Name:      "Fazenda Agua Santa Airport",
		City:      "Perdizes",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3548,
		Latitude:  -19.3630561829,
		Longitude: -47.3594436646,
		Timezone:  "America/Sao_Paulo",
	},
	"SNHS": {
		ICAO:      "SNHS",
		Name:      "Santa Magalhaes Airport",
		City:      "Serra Talhada",
		State:     "Pernambuco",
		Country:   "BR",
		Elevation: 1571,
		Latitude:  -8.0623998642,
		Longitude: -38.3288002014,
		Timezone:  "America/Recife",
	},
	"SNHU": {
		ICAO:      "SNHU",
		Name:      "FazendaBola Sete Airport",
		City:      "Itinga Do Maranhao",
		State:     "Para",
		Country:   "BR",
		Elevation: 699,
		Latitude:  -4.57,
		Longitude: -47.47,
		Timezone:  "America/Fortaleza",
	},
	"SNHW": {
		ICAO:      "SNHW",
		Name:      "Fazenda Agro-Marata Airport",
		City:      "Santa Luzia",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 1001,
		Latitude:  -4.3222222328,
		Longitude: -46.2283325195,
		Timezone:  "America/Fortaleza",
	},
	"SNHX": {
		ICAO:      "SNHX",
		Name:      "Fazenda Samello Airport",
		City:      "Paracatu",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2591,
		Latitude:  -17.3624992371,
		Longitude: -47.4408340454,
		Timezone:  "America/Sao_Paulo",
	},
	"SNIA": {
		ICAO:      "SNIA",
		Name:      "Igarape-Acu Airport",
		City:      "Igarape-Acu",
		State:     "Para",
		Country:   "BR",
		Elevation: 180,
		Latitude:  -1.1272219419,
		Longitude: -47.5991668701,
		Timezone:  "America/Belem",
	},
	"SNIB": {
		ICAO:      "SNIB",
		Name:      "Itaberaba Airport",
		City:      "Itaberaba",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 929,
		Latitude:  -12.5,
		Longitude: -40.2699012756,
		Timezone:  "America/Bahia",
	},
	"SNIC": {
		ICAO:      "SNIC",
		IATA:      "IRE",
		Name:      "Irece Airport",
		City:      "Irece",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 2561,
		Latitude:  -11.3399000168,
		Longitude: -41.8470001221,
		Timezone:  "America/Bahia",
	},
	"SNIE": {
		ICAO:      "SNIE",
		Name:      "Caetite Airport",
		City:      "Caetite",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 2854,
		Latitude:  -14.0121002197,
		Longitude: -42.4945983887,
		Timezone:  "America/Bahia",
	},
	"SNIF": {
		ICAO:      "SNIF",
		Name:      "Fazenda Serra Dourada Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 443,
		Latitude:  -5.7172222137,
		Longitude: -56.4361114502,
		Timezone:  "America/Santarem",
	},
	"SNIG": {
		ICAO:      "SNIG",
		IATA:      "QIG",
		Name:      "Iguatu Airport",
		City:      "Iguatu",
		State:     "Ceara",
		Country:   "BR",
		Elevation: 699,
		Latitude:  -6.34664011,
		Longitude: -39.293800354,
		Timezone:  "America/Fortaleza",
	},
	"SNII": {
		ICAO:      "SNII",
		Name:      "Ouroland Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 699,
		Latitude:  -6.8169441223,
		Longitude: -56.5505561829,
		Timezone:  "America/Santarem",
	},
	"SNIK": {
		ICAO:      "SNIK",
		Name:      "Itamarandiba Airport",
		City:      "Itamarandiba",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3196,
		Latitude:  -17.8505554199,
		Longitude: -42.8505554199,
		Timezone:  "America/Sao_Paulo",
	},
	"SNIN": {
		ICAO:      "SNIN",
		Name:      "Prainha Airport",
		City:      "Prainha",
		State:     "Para",
		Country:   "BR",
		Elevation: 279,
		Latitude:  -1.8002779484,
		Longitude: -53.4838905334,
		Timezone:  "America/Santarem",
	},
	"SNIO": {
		ICAO:      "SNIO",
		Name:      "Cipo Airport",
		City:      "Cipo",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 499,
		Latitude:  -11.1172218323,
		Longitude: -38.5169448853,
		Timezone:  "America/Bahia",
	},
	"SNIP": {
		ICAO:      "SNIP",
		IATA:      "QIT",
		Name:      "Itapetinga Airport",
		City:      "Itapetinga",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 915,
		Latitude:  -15.2445001602,
		Longitude: -40.2771987915,
		Timezone:  "America/Bahia",
	},
	"SNIR": {
		ICAO:      "SNIR",
		Name:      "Fazenda Mata Escura Airport",
		City:      "Parnarama",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 712,
		Latitude:  -5.7427778244,
		Longitude: -43.5988883972,
		Timezone:  "America/Fortaleza",
	},
	"SNIS": {
		ICAO:      "SNIS",
		Name:      "Fazenda Rio Largo Airport",
		City:      "Caxias",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 459,
		Latitude:  -5.1969442368,
		Longitude: -43.5977783203,
		Timezone:  "America/Fortaleza",
	},
	"SNIT": {
		ICAO:      "SNIT",
		Name:      "Ibotirama Airport",
		City:      "Ibotirama",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1398,
		Latitude:  -12.1686000824,
		Longitude: -43.2220993042,
		Timezone:  "America/Bahia",
	},
	"SNIU": {
		ICAO:      "SNIU",
		IATA:      "IPU",
		Name:      "Ipiau Airport",
		City:      "Ipiau",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 492,
		Latitude:  -14.1338891983,
		Longitude: -39.7338905334,
		Timezone:  "America/Bahia",
	},
	"SNIV": {
		ICAO:      "SNIV",
		Name:      "Fazenda Jumari Airport",
		City:      "Ipiacu",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1424,
		Latitude:  -18.646389,
		Longitude: -49.8825,
		Timezone:  "America/Sao_Paulo",
	},
	"SNIX": {
		ICAO:      "SNIX",
		Name:      "Fazenda Masutti Airport",
		City:      "Campos De Julio",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1617,
		Latitude:  -13.553333,
		Longitude: -59.100278,
		Timezone:  "America/Cuiaba",
	},
	"SNIY": {
		ICAO:      "SNIY",
		Name:      "Ibimirim Airport",
		City:      "Ibimirim",
		State:     "Pernambuco",
		Country:   "BR",
		Elevation: 1329,
		Latitude:  -8.5002784729,
		Longitude: -37.6669425964,
		Timezone:  "America/Recife",
	},
	"SNIZ": {
		ICAO:      "SNIZ",
		Name:      "Fazenda Jaqueline Airport",
		City:      "Vilhena",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 1772,
		Latitude:  -12.8825,
		Longitude: -60.119444,
		Timezone:  "America/Porto_Velho",
	},
	"SNJA": {
		ICAO:      "SNJA",
		Name:      "Jardim de Angicos Airport",
		City:      "Jardim De Angicos",
		State:     "Rio-Grande-do-Norte",
		Country:   "BR",
		Elevation: 492,
		Latitude:  -5.6386108398,
		Longitude: -35.9566688538,
		Timezone:  "America/Fortaleza",
	},
	"SNJB": {
		ICAO:      "SNJB",
		IATA:      "JCM",
		Name:      "Jacobina Airport",
		City:      "Jacobina",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1640,
		Latitude:  -11.1632003784,
		Longitude: -40.5531005859,
		Timezone:  "America/Bahia",
	},
	"SNJC": {
		ICAO:      "SNJC",
		Name:      "Fazenda Agua Boa Airport",
		City:      "Ribamar Fiquene",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 495,
		Latitude:  -5.9611110687,
		Longitude: -47.3922233582,
		Timezone:  "America/Fortaleza",
	},
	"SNJD": {
		ICAO:      "SNJD",
		IATA:      "FEC",
		Name:      "Joao Durval Carneiro Airport",
		City:      "Feira De Santana",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 768,
		Latitude:  -12.2003002167,
		Longitude: -38.9067993164,
		Timezone:  "America/Bahia",
	},
	"SNJE": {
		ICAO:      "SNJE",
		Name:      "Fazenda Sertao do Formoso Airport",
		City:      "Jaborandi",
		State:     "Goias",
		Country:   "BR",
		Elevation: 3028,
		Latitude:  -14.7936105728,
		Longitude: -46.0030555725,
		Timezone:  "America/Bahia",
	},
	"SNJG": {
		ICAO:      "SNJG",
		Name:      "Fazenda Sao Jose da Lagoa Airport",
		City:      "Nova Crixas",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1286,
		Latitude:  -14.5766666667,
		Longitude: -50.7055555556,
		Timezone:  "America/Sao_Paulo",
	},
	"SNJH": {
		ICAO:      "SNJH",
		Name:      "Sao Jose do Jacuipe Airport",
		City:      "Sao Jose Do Jacuipe",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1391,
		Latitude:  -11.4527778625,
		Longitude: -40.0422210693,
		Timezone:  "America/Bahia",
	},
	"SNJI": {
		ICAO:      "SNJI",
		Name:      "Fazenda Fortaleza de Santa Terezinha Airport",
		City:      "Jequitai",
		Country:   "BR",
		Elevation: 2077,
		Latitude:  -17.1868000031,
		Longitude: -44.6495018005,
		Timezone:  "America/Sao_Paulo",
	},
	"SNJJ": {
		ICAO:      "SNJJ",
		Name:      "Fazenda Vo Anizio Airport",
		City:      "Dois Irmaos Do Buriti",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 837,
		Latitude:  -20.715278,
		Longitude: -55.379722,
		Timezone:  "America/Campo_Grande",
	},
	"SNJK": {
		ICAO:      "SNJK",
		IATA:      "JEQ",
		Name:      "Jequie Airport",
		City:      "Jequie",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 646,
		Latitude:  -13.877699852,
		Longitude: -40.0716018677,
		Timezone:  "America/Bahia",
	},
	"SNJL": {
		ICAO:      "SNJL",
		Name:      "Fazenda do Rochedo Airport",
		City:      "Juiz De Fora",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2300,
		Latitude:  -21.8455562592,
		Longitude: -43.1875,
		Timezone:  "America/Sao_Paulo",
	},
	"SNJM": {
		ICAO:      "SNJM",
		Name:      "Aeroporto Elias Breder Airport",
		City:      "Manhuacu",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2723,
		Latitude:  -20.259721756,
		Longitude: -42.1838874817,
		Timezone:  "America/Sao_Paulo",
	},
	"SNJN": {
		ICAO:      "SNJN",
		IATA:      "JNA",
		Name:      "Januaria Airport",
		City:      "Januaria",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1575,
		Latitude:  -15.4737997055,
		Longitude: -44.3855018616,
		Timezone:  "America/Sao_Paulo",
	},
	"SNJO": {
		ICAO:      "SNJO",
		Name:      "Aeroclube Airport",
		City:      "Joao Pessoa",
		State:     "Paraiba",
		Country:   "BR",
		Elevation: 16,
		Latitude:  -7.091989994,
		Longitude: -34.8415985107,
		Timezone:  "America/Fortaleza",
	},
	"SNJP": {
		ICAO:      "SNJP",
		Name:      "Joao Pinheiro Airport",
		City:      "Joao Pinheiro",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2990,
		Latitude:  -17.7875003815,
		Longitude: -46.1199989319,
		Timezone:  "America/Sao_Paulo",
	},
	"SNJQ": {
		ICAO:      "SNJQ",
		Name:      "Jequitinhonha Airport",
		City:      "Jequitinhonha",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 853,
		Latitude:  -16.4419002533,
		Longitude: -41.0368995667,
		Timezone:  "America/Sao_Paulo",
	},
	"SNJR": {
		ICAO:      "SNJR",
		IATA:      "JDR",
		Name:      "Prefeito Octavio de Almeida Neves Airport",
		City:      "Sao Joao Del Rei",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3117,
		Latitude:  -21.0849990845,
		Longitude: -44.2247238159,
		Timezone:  "America/Sao_Paulo",
	},
	"SNJS": {
		ICAO:      "SNJS",
		Name:      "Jardim do Serido Airport",
		City:      "Jardim Do Serido",
		State:     "Rio-Grande-do-Norte",
		Country:   "BR",
		Elevation: 837,
		Latitude:  -6.5411109924,
		Longitude: -36.7316665649,
		Timezone:  "America/Fortaleza",
	},
	"SNJT": {
		ICAO:      "SNJT",
		Name:      "Fazenda Caraibas Airport",
		City:      "Janauba",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1811,
		Latitude:  -15.6541671753,
		Longitude: -43.4016685486,
		Timezone:  "America/Sao_Paulo",
	},
	"SNJV": {
		ICAO:      "SNJV",
		Name:      "Fazenda Santa Monica Airport",
		City:      "Sao Joao Da Ponte",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2077,
		Latitude:  -16.002737,
		Longitude: -43.746648,
		Timezone:  "America/Sao_Paulo",
	},
	"SNJW": {
		ICAO:      "SNJW",
		Name:      "Jaragua Airport",
		City:      "Sacramento",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1909,
		Latitude:  -20.0403003693,
		Longitude: -47.4215011597,
		Timezone:  "America/Sao_Paulo",
	},
	"SNJZ": {
		ICAO:      "SNJZ",
		Name:      "Fazenda Santa Izabel do Porto Seguro da Jau Airport",
		City:      "Sao Felix Do Xingu",
		State:     "Para",
		Country:   "BR",
		Elevation: 804,
		Latitude:  -7.0763888359,
		Longitude: -52.9347229004,
		Timezone:  "America/Santarem",
	},
	"SNKB": {
		ICAO:      "SNKB",
		Name:      "Aeroclube Airport",
		City:      "Campina Grande",
		State:     "Paraiba",
		Country:   "BR",
		Elevation: 1926,
		Latitude:  -7.1783328056,
		Longitude: -35.9902763367,
		Timezone:  "America/Fortaleza",
	},
	"SNKC": {
		ICAO:      "SNKC",
		Name:      "Cocos Airport",
		City:      "Cocos",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 2018,
		Latitude:  -14.17277778,
		Longitude: -44.54666667,
		Timezone:  "America/Bahia",
	},
	"SNKD": {
		ICAO:      "SNKD",
		Name:      "Conceicao do Mato Dentro Airport",
		City:      "Conceicao Do Mato Dentro",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2196,
		Latitude:  -19.0202999115,
		Longitude: -43.4338989258,
		Timezone:  "America/Sao_Paulo",
	},
	"SNKE": {
		ICAO:      "SNKE",
		IATA:      "CMP",
		Name:      "Santana do Araguaia Airport",
		City:      "Santana Do Araguaia",
		State:     "Para",
		Country:   "BR",
		Elevation: 597,
		Latitude:  -9.3199701309,
		Longitude: -50.3284988403,
		Timezone:  "America/Belem",
	},
	"SNKF": {
		ICAO:      "SNKF",
		IATA:      "QDF",
		Name:      "Conselheiro Lafaiete Airport",
		City:      "Conselheiro Lafaiete",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3478,
		Latitude:  -20.7385997772,
		Longitude: -43.7974014282,
		Timezone:  "America/Sao_Paulo",
	},
	"SNKH": {
		ICAO:      "SNKH",
		Name:      "Creputia Airport",
		City:      "Creputia",
		State:     "Para",
		Country:   "BR",
		Elevation: 984,
		Latitude:  -8.116944313,
		Longitude: -57.1172218323,
		Timezone:  "America/Santarem",
	},
	"SNKI": {
		ICAO:      "SNKI",
		IATA:      "CDI",
		Name:      "Cachoeiro do Itapemirim Airport",
		City:      "Cachoeiro Do Itapemirim",
		State:     "Espirito-Santo",
		Country:   "BR",
		Elevation: 335,
		Latitude:  -20.8342990875,
		Longitude: -41.1856002808,
		Timezone:  "America/Sao_Paulo",
	},
	"SNKJ": {
		ICAO:      "SNKJ",
		Name:      "Fazenda Rio Dourado Airport",
		City:      "Cumaru Do Norte",
		State:     "Para",
		Country:   "BR",
		Elevation: 814,
		Latitude:  -8.3441667557,
		Longitude: -51.4513893127,
		Timezone:  "America/Belem",
	},
	"SNKK": {
		ICAO:      "SNKK",
		Name:      "Caico Airport",
		City:      "Caico",
		State:     "Rio-Grande-do-Norte",
		Country:   "BR",
		Elevation: 600,
		Latitude:  -6.4391698837,
		Longitude: -37.0783004761,
		Timezone:  "America/Fortaleza",
	},
	"SNKL": {
		ICAO:      "SNKL",
		IATA:      "0",
		Name:      "Colinas Airport",
		City:      "Colinas",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 394,
		Latitude:  -5.9669399261,
		Longitude: -44.2338981628,
		Timezone:  "America/Fortaleza",
	},
	"SNKN": {
		ICAO:      "SNKN",
		IATA:      "QCP",
		Name:      "Currais Novos Airport",
		City:      "Currais Novos",
		State:     "Rio-Grande-do-Norte",
		Country:   "BR",
		Elevation: 1148,
		Latitude:  -6.2808327675,
		Longitude: -36.5402793884,
		Timezone:  "America/Fortaleza",
	},
	"SNKO": {
		ICAO:      "SNKO",
		Name:      "Brotas de Macaubas Airport",
		City:      "Brotas De Macaubas",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 2667,
		Latitude:  -11.9994001389,
		Longitude: -42.6348991394,
		Timezone:  "America/Bahia",
	},
	"SNKQ": {
		ICAO:      "SNKQ",
		Name:      "Klaus Mangold Airport",
		City:      "Ipixuna Do Para",
		State:     "Para",
		Country:   "BR",
		Elevation: 200,
		Latitude:  -2.3919439316,
		Longitude: -47.8450012207,
		Timezone:  "America/Belem",
	},
	"SNKR": {
		ICAO:      "SNKR",
		Name:      "Corrente Airport",
		City:      "Corrente",
		State:     "Piaui",
		Country:   "BR",
		Elevation: 1552,
		Latitude:  -10.4519443512,
		Longitude: -45.136390686,
		Timezone:  "America/Fortaleza",
	},
	"SNKS": {
		ICAO:      "SNKS",
		Name:      "Santa Rita de Cassia Airport",
		City:      "Santa Rita De Cassia",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1476,
		Latitude:  -10.9502782822,
		Longitude: -44.4672203064,
		Timezone:  "America/Bahia",
	},
	"SNKT": {
		ICAO:      "SNKT",
		Name:      "Crepurizinho Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 1092,
		Latitude:  -6.8444437981,
		Longitude: -56.5894432068,
		Timezone:  "America/Santarem",
	},
	"SNKU": {
		ICAO:      "SNKU",
		Name:      "Canudos Airport",
		City:      "Canudos",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1247,
		Latitude:  -9.9005556107,
		Longitude: -39.1336097717,
		Timezone:  "America/Bahia",
	},
	"SNKV": {
		ICAO:      "SNKV",
		Name:      "Fazenda Campo Verde Airport",
		City:      "Tartarugalzinho",
		State:     "Amapa",
		Country:   "BR",
		Elevation: 49,
		Latitude:  1.0427800417,
		Longitude: -50.5167007446,
		Timezone:  "America/Belem",
	},
	"SNKW": {
		ICAO:      "SNKW",
		Name:      "Fazenda Boa Sorte Airport",
		City:      "Iacri",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1398,
		Latitude:  -21.918889,
		Longitude: -50.636389,
		Timezone:  "America/Sao_Paulo",
	},
	"SNKX": {
		ICAO:      "SNKX",
		Name:      "Marborges Airport",
		City:      "Moju",
		State:     "Para",
		Country:   "BR",
		Elevation: 114,
		Latitude:  -1.9716670513,
		Longitude: -48.6280555725,
		Timezone:  "America/Belem",
	},
	"SNKY": {
		ICAO:      "SNKY",
		Name:      "Fazenda Cikel Airport",
		City:      "Paragominas",
		State:     "Para",
		Country:   "BR",
		Elevation: 299,
		Latitude:  -3.664444,
		Longitude: -48.834444,
		Timezone:  "America/Belem",
	},
	"SNKZ": {
		ICAO:      "SNKZ",
		Name:      "Fazenda Remanso do Pito Airport",
		City:      "Navirai",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 912,
		Latitude:  -23.2013893127,
		Longitude: -53.855556488,
		Timezone:  "America/Campo_Grande",
	},
	"SNLA": {
		ICAO:      "SNLA",
		Name:      "Fazenda Lagoa das Antas Airport",
		City:      "Canaa Dos Carajas",
		State:     "Para",
		Country:   "BR",
		Elevation: 899,
		Latitude:  -6.5675001144,
		Longitude: -49.7194442749,
		Timezone:  "America/Belem",
	},
	"SNLB": {
		ICAO:      "SNLB",
		IATA:      "LVB",
		Name:      "Livramento do Brumado Airport",
		City:      "Livramento Do Brumado",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1559,
		Latitude:  -13.6506004333,
		Longitude: -41.8339004517,
		Timezone:  "America/Bahia",
	},
	"SNLC": {
		ICAO:      "SNLC",
		Name:      "Las Casas Airport",
		City:      "Las Casas",
		State:     "Para",
		Country:   "BR",
		Elevation: 1969,
		Latitude:  -7.9669442177,
		Longitude: -50.0172233582,
		Timezone:  "America/Belem",
	},
	"SNLD": {
		ICAO:      "SNLD",
		Name:      "Fazenda Serra Azul Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 472,
		Latitude:  -5.6966671944,
		Longitude: -56.494720459,
		Timezone:  "America/Santarem",
	},
	"SNLF": {
		ICAO:      "SNLF",
		Name:      "Filial Campo Florido Airport",
		City:      "Campo Florido",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1959,
		Latitude:  -19.8138885498,
		Longitude: -48.7388877869,
		Timezone:  "America/Sao_Paulo",
	},
	"SNLG": {
		ICAO:      "SNLG",
		Name:      "Serra do Cipo Airport",
		City:      "Jaboticatubas",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2621,
		Latitude:  -19.3986110687,
		Longitude: -43.7463874817,
		Timezone:  "America/Sao_Paulo",
	},
	"SNLH": {
		ICAO:      "SNLH",
		Name:      "Lajinha Airport",
		City:      "Lajinha",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2051,
		Latitude:  -20.1402778625,
		Longitude: -41.6091651917,
		Timezone:  "America/Sao_Paulo",
	},
	"SNLI": {
		ICAO:      "SNLI",
		Name:      "Abaete Airport",
		City:      "Abaete",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2178,
		Latitude:  -19.1555995941,
		Longitude: -45.4948005676,
		Timezone:  "America/Sao_Paulo",
	},
	"SNLJ": {
		ICAO:      "SNLJ",
		Name:      "Agropecuaria Ceu Aberto Ltda Airport",
		City:      "Aripuana",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 3428,
		Latitude:  -9.516667366,
		Longitude: -61.25,
		Timezone:  "America/Cuiaba",
	},
	"SNLK": {
		ICAO:      "SNLK",
		Name:      "Agropecuaria Ferreira Penco Ltda Airport",
		City:      "Aripuana",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 3297,
		Latitude:  -9.7158327103,
		Longitude: -61.0466651917,
		Timezone:  "America/Cuiaba",
	},
	"SNLM": {
		ICAO:      "SNLM",
		Name:      "Fazenda Agropecuaria Rancho 11 Airport",
		City:      "Porto Esperidiao",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1001,
		Latitude:  -15.6141672134,
		Longitude: -58.9002761841,
		Timezone:  "America/Cuiaba",
	},
	"SNLN": {
		ICAO:      "SNLN",
		Name:      "Linhares Airport",
		City:      "Linhares",
		State:     "Espirito-Santo",
		Country:   "BR",
		Elevation: 131,
		Latitude:  -19.3551998138,
		Longitude: -40.0694999695,
		Timezone:  "America/Sao_Paulo",
	},
	"SNLO": {
		ICAO:      "SNLO",
		IATA:      "SSO",
		Name:      "Sao Lourenco Airport",
		City:      "Sao Lourenco",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2871,
		Latitude:  -22.0909004211,
		Longitude: -45.0444984436,
		Timezone:  "America/Sao_Paulo",
	},
	"SNLQ": {
		ICAO:      "SNLQ",
		Name:      "Fazenda Conforto Airport",
		City:      "Nova Crixas",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1001,
		Latitude:  -14.0657997131,
		Longitude: -50.4159011841,
		Timezone:  "America/Sao_Paulo",
	},
	"SNLT": {
		ICAO:      "SNLT",
		Name:      "Paulistana Airport",
		City:      "Paulistana",
		State:     "Piaui",
		Country:   "BR",
		Elevation: 1181,
		Latitude:  -8.1663885117,
		Longitude: -41.1538887024,
		Timezone:  "America/Fortaleza",
	},
	"SNLU": {
		ICAO:      "SNLU",
		Name:      "Fazenda Boa Luz Airport",
		City:      "Laranjeiras",
		State:     "Sergipe",
		Country:   "BR",
		Elevation: 167,
		Latitude:  -10.8422222137,
		Longitude: -37.2038879395,
		Timezone:  "America/Maceio",
	},
	"SNLV": {
		ICAO:      "SNLV",
		Name:      "Lavrinhas Airport",
		City:      "Lavrinhas",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3478,
		Latitude:  -17.7358322144,
		Longitude: -43.4594421387,
		Timezone:  "America/Sao_Paulo",
	},
	"SNLX": {
		ICAO:      "SNLX",
		Name:      "Fazenda Primavera I Airport",
		City:      "Rondon Do Para",
		State:     "Para",
		Country:   "BR",
		Elevation: 1050,
		Latitude:  -4.8005561829,
		Longitude: -48.3933334351,
		Timezone:  "America/Belem",
	},
	"SNLY": {
		ICAO:      "SNLY",
		Name:      "Lagoa da Prata Airport",
		City:      "Lagoa Da Prata",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2159,
		Latitude:  -20.0613994598,
		Longitude: -45.554901123,
		Timezone:  "America/Sao_Paulo",
	},
	"SNLZ": {
		ICAO:      "SNLZ",
		Name:      "Fazenda Boca da Mata Airport",
		City:      "Divinopolis Do Tocantins",
		State:     "Para",
		Country:   "BR",
		Elevation: 669,
		Latitude:  -9.4319438934,
		Longitude: -49.5911102295,
		Timezone:  "America/Araguaina",
	},
	"SNMA": {
		ICAO:      "SNMA",
		IATA:      "MTE",
		Name:      "Monte Alegre Airport",
		City:      "Monte Alegre",
		State:     "Para",
		Country:   "BR",
		Elevation: 325,
		Latitude:  -1.9958000183,
		Longitude: -54.0741996765,
		Timezone:  "America/Santarem",
	},
	"SNMB": {
		ICAO:      "SNMB",
		IATA:      "0",
		Name:      "Mombaca Airport",
		City:      "Mombaca",
		State:     "Ceara",
		Country:   "BR",
		Elevation: 748,
		Latitude:  -5.7441701889,
		Longitude: -39.6213989258,
		Timezone:  "America/Fortaleza",
	},
	"SNMC": {
		ICAO:      "SNMC",
		Name:      "Macaubas Airport",
		City:      "Macaubas",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 2133,
		Latitude:  -13.0249996185,
		Longitude: -42.6722984314,
		Timezone:  "America/Bahia",
	},
	"SNMD": {
		ICAO:      "SNMD",
		Name:      "Mundico Coelho Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 471,
		Latitude:  -6.8205561638,
		Longitude: -56.8447227478,
		Timezone:  "America/Santarem",
	},
	"SNME": {
		ICAO:      "SNME",
		Name:      "Costa dos Coqueiros Airport",
		City:      "Mata De Sao Joao",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 233,
		Latitude:  -12.4897222519,
		Longitude: -38.0236091614,
		Timezone:  "America/Bahia",
	},
	"SNMF": {
		ICAO:      "SNMF",
		Name:      "Agropastoril Monte Alegre Airport",
		City:      "Monte Belo",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2841,
		Latitude:  -21.3505554199,
		Longitude: -46.2505569458,
		Timezone:  "America/Sao_Paulo",
	},
	"SNMI": {
		ICAO:      "SNMI",
		Name:      "Mina Caraiba Airport",
		City:      "Jaguarari",
		State:     "Ceara",
		Country:   "BR",
		Elevation: 1476,
		Latitude:  -9.8347215652,
		Longitude: -39.8833312988,
		Timezone:  "America/Bahia",
	},
	"SNMJ": {
		ICAO:      "SNMJ",
		Name:      "Maracas Airport",
		City:      "Maracas",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 3280,
		Latitude:  -13.4333000183,
		Longitude: -40.4333000183,
		Timezone:  "America/Bahia",
	},
	"SNMK": {
		ICAO:      "SNMK",
		Name:      "Mocambinho Airport",
		City:      "Jaiba",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1519,
		Latitude:  -15.093000412,
		Longitude: -43.9790992737,
		Timezone:  "America/Sao_Paulo",
	},
	"SNML": {
		ICAO:      "SNML",
		Name:      "Manduca Leao Airport",
		City:      "Rio Largo",
		State:     "Alagoas",
		Country:   "BR",
		Elevation: 387,
		Latitude:  -9.5444440842,
		Longitude: -35.8294448853,
		Timezone:  "America/Maceio",
	},
	"SNMM": {
		ICAO:      "SNMM",
		Name:      "Morada Nova de Minas Airport",
		City:      "Morada Nova De Minas",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1988,
		Latitude:  -18.5938892365,
		Longitude: -45.3530578613,
		Timezone:  "America/Sao_Paulo",
	},
	"SNMN": {
		ICAO:      "SNMN",
		Name:      "Minas Novas Airport",
		City:      "Minas Novas",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2461,
		Latitude:  -17.2338886261,
		Longitude: -42.5838890076,
		Timezone:  "America/Sao_Paulo",
	},
	"SNMO": {
		ICAO:      "SNMO",
		Name:      "Morada Nova Airport",
		City:      "Morada Nova",
		State:     "Ceara",
		Country:   "BR",
		Elevation: 509,
		Latitude:  -5.0836110115,
		Longitude: -38.3836097717,
		Timezone:  "America/Fortaleza",
	},
	"SNMP": {
		ICAO:      "SNMP",
		Name:      "Fazenda Marocopa Airport",
		City:      "Paracatu",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1752,
		Latitude:  -17.3230552673,
		Longitude: -46.5380554199,
		Timezone:  "America/Sao_Paulo",
	},
	"SNMR": {
		ICAO:      "SNMR",
		Name:      "Marau Airport",
		City:      "Marau",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 66,
		Latitude:  -14.1172218323,
		Longitude: -38.9838905334,
		Timezone:  "America/Bahia",
	},
	"SNMS": {
		ICAO:      "SNMS",
		IATA:      "0",
		Name:      "Monte Santo de Minas Airport",
		City:      "Monte Santo De Minas",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2936,
		Latitude:  -21.1758003235,
		Longitude: -46.9589004517,
		Timezone:  "America/Sao_Paulo",
	},
	"SNMU": {
		ICAO:      "SNMU",
		IATA:      "MVS",
		Name:      "Mucuri Airport",
		City:      "Mucuri",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 276,
		Latitude:  -18.0489006042,
		Longitude: -39.864200592,
		Timezone:  "America/Bahia",
	},
	"SNMV": {
		ICAO:      "SNMV",
		Name:      "Fazenda Flavia Airport",
		City:      "Alcinopolis",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1250,
		Latitude:  -18.3500003815,
		Longitude: -53.75,
		Timezone:  "America/Campo_Grande",
	},
	"SNMW": {
		ICAO:      "SNMW",
		Name:      "Agropecuaria Paralelo Dez Ltda Airport",
		City:      "Aripuana",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 3035,
		Latitude:  -9.9994440079,
		Longitude: -61.0522232056,
		Timezone:  "America/Cuiaba",
	},
	"SNMX": {
		ICAO:      "SNMX",
		IATA:      "SBJ",
		Name:      "Sao Mateus Airport",
		City:      "Sao Mateus",
		State:     "Espirito-Santo",
		Country:   "BR",
		Elevation: 98,
		Latitude:  -18.7213001251,
		Longitude: -39.8336982727,
		Timezone:  "America/Sao_Paulo",
	},
	"SNMZ": {
		ICAO:      "SNMZ",
		IATA:      "PTQ",
		Name:      "Porto de Moz Airport",
		City:      "Porto De Moz",
		State:     "Para",
		Country:   "BR",
		Elevation: 53,
		Latitude:  -1.7414499521,
		Longitude: -52.2360992432,
		Timezone:  "America/Belem",
	},
	"SNNB": {
		ICAO:      "SNNB",
		Name:      "Fazenda Cana Brava Airport",
		City:      "Paracatu",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1772,
		Latitude:  -17.1769447327,
		Longitude: -46.2975006104,
		Timezone:  "America/Sao_Paulo",
	},
	"SNNC": {
		ICAO:      "SNNC",
		Name:      "Rio Capim Airport",
		City:      "Ipixuna Do Para",
		State:     "Para",
		Country:   "BR",
		Elevation: 236,
		Latitude:  -2.8202779293,
		Longitude: -47.8930549622,
		Timezone:  "America/Belem",
	},
	"SNND": {
		ICAO:      "SNND",
		Name:      "Berlin Ltda Airport",
		City:      "Ituiutaba",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1815,
		Latitude:  -18.935300827,
		Longitude: -49.4726982117,
		Timezone:  "America/Sao_Paulo",
	},
	"SNNE": {
		ICAO:      "SNNE",
		Name:      "Sao Joao Nepomuceno Airport",
		City:      "Sao Joao Nepomuceno",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1322,
		Latitude:  -21.5422210693,
		Longitude: -43.0211105347,
		Timezone:  "America/Sao_Paulo",
	},
	"SNNF": {
		ICAO:      "SNNF",
		Name:      "Nossa Senhora de Fatima Airport",
		City:      "Teresina",
		State:     "Piaui",
		Country:   "BR",
		Elevation: 607,
		Latitude:  -5.0336108208,
		Longitude: -42.7055549622,
		Timezone:  "America/Fortaleza",
	},
	"SNNH": {
		ICAO:      "SNNH",
		Name:      "Carinhanha Airport",
		City:      "Carinhanha",
		Country:   "BR",
		Elevation: 1483,
		Latitude:  -14.298500061,
		Longitude: -43.7972984314,
		Timezone:  "America/Bahia",
	},
	"SNNI": {
		ICAO:      "SNNI",
		Name:      "Praia do Forte Airport",
		City:      "Mata De Sao Joao",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 49,
		Latitude:  -12.5649995804,
		Longitude: -38.0216674805,
		Timezone:  "America/Bahia",
	},
	"SNNJ": {
		ICAO:      "SNNJ",
		Name:      "Fazenda Santa Cruz Airport",
		City:      "Iaciara",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1529,
		Latitude:  -14.2177782059,
		Longitude: -46.8386116028,
		Timezone:  "America/Sao_Paulo",
	},
	"SNNK": {
		ICAO:      "SNNK",
		Name:      "Fazenda Teragro Airport",
		City:      "Matias Cardoso",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1512,
		Latitude:  -14.7694444656,
		Longitude: -43.4152793884,
		Timezone:  "America/Sao_Paulo",
	},
	"SNNM": {
		ICAO:      "SNNM",
		Name:      "Entre Rios - Vitoria Airport",
		City:      "Guarapuava",
		State:     "Parana",
		Country:   "BR",
		Elevation: 3602,
		Latitude:  -25.566110611,
		Longitude: -51.4966659546,
		Timezone:  "America/Sao_Paulo",
	},
	"SNNO": {
		ICAO:      "SNNO",
		Name:      "Fazenda Sao Domingos Airport",
		City:      "Comodoro",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1001,
		Latitude:  -14.0638885498,
		Longitude: -59.8663902283,
		Timezone:  "America/Cuiaba",
	},
	"SNNP": {
		ICAO:      "SNNP",
		Name:      "Nilo Pecanha Airport",
		City:      "Nilo Pecanha",
		State:     "Para",
		Country:   "BR",
		Elevation: 984,
		Latitude:  -8.0669441223,
		Longitude: -52.1672210693,
		Timezone:  "America/Belem",
	},
	"SNNQ": {
		ICAO:      "SNNQ",
		Name:      "Fazenda Santa Lucia Airport",
		City:      "Porto Murtinho",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1640,
		Latitude:  -23.03888889,
		Longitude: -54.7475,
		Timezone:  "America/Campo_Grande",
	},
	"SNNS": {
		ICAO:      "SNNS",
		Name:      "Fazenda Jaibense Airport",
		City:      "Matias Cardoso",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1644,
		Latitude:  -14.9461107254,
		Longitude: -43.6811103821,
		Timezone:  "America/Sao_Paulo",
	},
	"SNNT": {
		ICAO:      "SNNT",
		Name:      "Nova Ponte Airport",
		City:      "Nova Ponte",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3261,
		Latitude:  -19.1984004974,
		Longitude: -47.7286987305,
		Timezone:  "America/Sao_Paulo",
	},
	"SNNU": {
		ICAO:      "SNNU",
		IATA:      "NNU",
		Name:      "Nanuque Airport",
		City:      "Nanuque",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 591,
		Latitude:  -17.823299408,
		Longitude: -40.3298988342,
		Timezone:  "America/Sao_Paulo",
	},
	"SNNV": {
		ICAO:      "SNNV",
		Name:      "Fazenda Estrela do Sul Airport",
		City:      "Pedra Preta",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1099,
		Latitude:  -16.5705566406,
		Longitude: -54.1708335876,
		Timezone:  "America/Cuiaba",
	},
	"SNNX": {
		ICAO:      "SNNX",
		Name:      "Fazenda Boa Vista Airport",
		City:      "Corumbiara",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 866,
		Latitude:  -12.8333330154,
		Longitude: -61.3499984741,
		Timezone:  "America/Porto_Velho",
	},
	"SNNY": {
		ICAO:      "SNNY",
		Name:      "Fazenda Espirito Santo Airport",
		City:      "Comodoro",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 1001,
		Latitude:  -13.2805557251,
		Longitude: -60.331111908,
		Timezone:  "America/Cuiaba",
	},
	"SNNZ": {
		ICAO:      "SNNZ",
		Name:      "Fazenda Agua Turva Airport",
		City:      "Caracol",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 745,
		Latitude:  -22.1236114502,
		Longitude: -57.1038894653,
		Timezone:  "America/Campo_Grande",
	},
	"SNOB": {
		ICAO:      "SNOB",
		IATA:      "QBX",
		Name:      "Sobral Airport",
		City:      "Sobral",
		State:     "Ceara",
		Country:   "BR",
		Elevation: 210,
		Latitude:  -3.6788899899,
		Longitude: -40.3367996216,
		Timezone:  "America/Fortaleza",
	},
	"SNOC": {
		ICAO:      "SNOC",
		IATA:      "0",
		Name:      "Morro do Chapeu Airport",
		City:      "Morro Do Chapeu",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 3609,
		Latitude:  -11.5343999863,
		Longitude: -41.1796989441,
		Timezone:  "America/Bahia",
	},
	"SNOD": {
		ICAO:      "SNOD",
		Name:      "Praia do Saco Airport",
		City:      "Estancia",
		State:     "Sergipe",
		Country:   "BR",
		Elevation: 20,
		Latitude:  -11.4219436646,
		Longitude: -37.3224983215,
		Timezone:  "America/Maceio",
	},
	"SNOE": {
		ICAO:      "SNOE",
		IATA:      "0",
		Name:      "Oeiras Airport",
		City:      "Oeiras",
		State:     "Piaui",
		Country:   "BR",
		Elevation: 820,
		Latitude:  -7.018889904,
		Longitude: -42.1671981812,
		Timezone:  "America/Fortaleza",
	},
	"SNOF": {
		ICAO:      "SNOF",
		IATA:      "0",
		Name:      "Ouro Fino Airport",
		City:      "Ouro Fino",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2818,
		Latitude:  -22.2975006104,
		Longitude: -46.3932991028,
		Timezone:  "America/Sao_Paulo",
	},
	"SNOG": {
		ICAO:      "SNOG",
		Name:      "Ceara-Mirim Airport",
		City:      "Ceara-Mirim",
		State:     "Rio-Grande-do-Norte",
		Country:   "BR",
		Elevation: 171,
		Latitude:  -5.6612100601,
		Longitude: -35.413898468,
		Timezone:  "America/Fortaleza",
	},
	"SNOI": {
		ICAO:      "SNOI",
		Name:      "Fazenda Rio Mutuca Airport",
		City:      "Juara",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 951,
		Latitude:  -10.4263887405,
		Longitude: -58.1613883972,
		Timezone:  "America/Cuiaba",
	},
	"SNOJ": {
		ICAO:      "SNOJ",
		Name:      "Minalpa - Mineracao Alto Parnaiba Ltda. Airport",
		City:      "Alta Floresta",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 984,
		Latitude:  -10.4166669846,
		Longitude: -56.3886108398,
		Timezone:  "America/Cuiaba",
	},
	"SNOK": {
		ICAO:      "SNOK",
		Name:      "Fazenda Chao de Estrelas Airport",
		City:      "Aurora Do Para",
		State:     "Para",
		Country:   "BR",
		Elevation: 126,
		Latitude:  -2.3272221088,
		Longitude: -47.6511116028,
		Timezone:  "America/Belem",
	},
	"SNOM": {
		ICAO:      "SNOM",
		Name:      "Fazenda Guaporei Airport",
		City:      "Vila Bela Da Santissima Trindade",
		State:     "Santa-Cruz",
		Country:   "BR",
		Elevation: 1001,
		Latitude:  -15.0066995621,
		Longitude: -60.2206993103,
		Timezone:  "America/Cuiaba",
	},
	"SNOO": {
		ICAO:      "SNOO",
		Name:      "Fazenda Quatrilho Airport",
		City:      "Sorriso",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1457,
		Latitude:  -13.35778,
		Longitude: -55.61611,
		Timezone:  "America/Cuiaba",
	},
	"SNOP": {
		ICAO:      "SNOP",
		IATA:      "0",
		Name:      "Propria Airport",
		City:      "Propria",
		State:     "Sergipe",
		Country:   "BR",
		Elevation: 164,
		Latitude:  -10.26720047,
		Longitude: -36.8335990906,
		Timezone:  "America/Maceio",
	},
	"SNOQ": {
		ICAO:      "SNOQ",
		Name:      "Fazenda Agropecuaria Crioulo Airport",
		City:      "Tacuru",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1263,
		Latitude:  -23.5988883972,
		Longitude: -55.1425018311,
		Timezone:  "America/Campo_Grande",
	},
	"SNOS": {
		ICAO:      "SNOS",
		IATA:      "PSW",
		Name:      "Municipal Jose Figueiredo Airport",
		City:      "Passos",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2697,
		Latitude:  -20.7322006226,
		Longitude: -46.6618003845,
		Timezone:  "America/Sao_Paulo",
	},
	"SNOT": {
		ICAO:      "SNOT",
		Name:      "Fazenda Santa Fe do Guapore Airport",
		City:      "Comodoro",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 951,
		Latitude:  -14.0116672516,
		Longitude: -59.8866653442,
		Timezone:  "America/Cuiaba",
	},
	"SNOU": {
		ICAO:      "SNOU",
		IATA:      "FEJ",
		Name:      "Novo Aerodromo de Feijo Airport",
		City:      "Feijo",
		State:     "Acre",
		Country:   "BR",
		Elevation: 394,
		Latitude:  -8.140832901,
		Longitude: -70.3472213745,
		Timezone:  "America/Rio_Branco",
	},
	"SNOV": {
		ICAO:      "SNOV",
		Name:      "Povoado de Camacari Airport",
		City:      "Coruripe",
		State:     "Alagoas",
		Country:   "BR",
		Elevation: 279,
		Latitude:  -10.1225004196,
		Longitude: -36.2922210693,
		Timezone:  "America/Maceio",
	},
	"SNOW": {
		ICAO:      "SNOW",
		Name:      "Fazenda Turmalina Airport",
		City:      "Martinopolis",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1378,
		Latitude:  -21.9822216034,
		Longitude: -50.9722213745,
		Timezone:  "America/Sao_Paulo",
	},
	"SNOX": {
		ICAO:      "SNOX",
		IATA:      "ORX",
		Name:      "Oriximina Airport",
		City:      "Oriximina",
		State:     "Para",
		Country:   "BR",
		Elevation: 262,
		Latitude:  -1.7140799761,
		Longitude: -55.8362007141,
		Timezone:  "America/Santarem",
	},
	"SNOY": {
		ICAO:      "SNOY",
		Name:      "Ouricuri Airport",
		City:      "Ouricuri",
		State:     "Pernambuco",
		Country:   "BR",
		Elevation: 1529,
		Latitude:  -7.8765201569,
		Longitude: -40.0918006897,
		Timezone:  "America/Recife",
	},
	"SNOZ": {
		ICAO:      "SNOZ",
		Name:      "Coronel Alexandre Raposo Airport",
		City:      "Paco Do Lumiar",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 154,
		Latitude:  -2.5209701061,
		Longitude: -44.1212997437,
		Timezone:  "America/Fortaleza",
	},
	"SNPA": {
		ICAO:      "SNPA",
		Name:      "Para de Minas Airport",
		City:      "Para De Minas",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2598,
		Latitude:  -19.8425998688,
		Longitude: -44.6012001038,
		Timezone:  "America/Sao_Paulo",
	},
	"SNPB": {
		ICAO:      "SNPB",
		IATA:      "0",
		Name:      "Pastos Bons Airport",
		City:      "Pastos Bons",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 197,
		Latitude:  -6.6669402122,
		Longitude: -44.0839004517,
		Timezone:  "America/Fortaleza",
	},
	"SNPC": {
		ICAO:      "SNPC",
		IATA:      "PCS",
		Name:      "Picos Airport",
		City:      "Picos",
		State:     "Piaui",
		Country:   "BR",
		Elevation: 1050,
		Latitude:  -7.0620598793,
		Longitude: -41.5237007141,
		Timezone:  "America/Fortaleza",
	},
	"SNPD": {
		ICAO:      "SNPD",
		IATA:      "POJ",
		Name:      "Patos de Minas Airport",
		City:      "Patos De Minas",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2793,
		Latitude:  -18.6728000641,
		Longitude: -46.4911994934,
		Timezone:  "America/Sao_Paulo",
	},
	"SNPE": {
		ICAO:      "SNPE",
		Name:      "Penedo Airport",
		City:      "Penedo",
		State:     "Alagoas",
		Country:   "BR",
		Elevation: 197,
		Latitude:  -10.2658996582,
		Longitude: -36.5514984131,
		Timezone:  "America/Maceio",
	},
	"SNPF": {
		ICAO:      "SNPF",
		Name:      "Fazenda Canabrava Airport",
		City:      "Parnarama",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 689,
		Latitude:  -5.5911111832,
		Longitude: -43.317779541,
		Timezone:  "America/Fortaleza",
	},
	"SNPI": {
		ICAO:      "SNPI",
		IATA:      "0",
		Name:      "Piata Airport",
		City:      "Piata",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 4118,
		Latitude:  -13.0005998611,
		Longitude: -41.7750015259,
		Timezone:  "America/Bahia",
	},
	"SNPJ": {
		ICAO:      "SNPJ",
		IATA:      "0",
		Name:      "Patrocinio Airport",
		City:      "Patrocinio",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3229,
		Latitude:  -18.9094009399,
		Longitude: -46.9827003479,
		Timezone:  "America/Sao_Paulo",
	},
	"SNPL": {
		ICAO:      "SNPL",
		Name:      "Fazenda Pacuruxu Airport",
		City:      "Santa Mercedes",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1181,
		Latitude:  -21.2563896179,
		Longitude: -51.7288894653,
		Timezone:  "America/Sao_Paulo",
	},
	"SNPM": {
		ICAO:      "SNPM",
		Name:      "Palmeiras Airport",
		City:      "Palmeiras",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 2395,
		Latitude:  -12.50055556,
		Longitude: -41.58388889,
		Timezone:  "America/Bahia",
	},
	"SNPN": {
		ICAO:      "SNPN",
		Name:      "Fazenda Agua Fria Airport",
		City:      "Lavinia",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1378,
		Latitude:  -21.0302772522,
		Longitude: -50.9258346558,
		Timezone:  "America/Sao_Paulo",
	},
	"SNPO": {
		ICAO:      "SNPO",
		IATA:      "0",
		Name:      "Pompeu Airport",
		City:      "Pompeu",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2313,
		Latitude:  -19.2033004761,
		Longitude: -45.0233001709,
		Timezone:  "America/Sao_Paulo",
	},
	"SNPP": {
		ICAO:      "SNPP",
		Name:      "Morro de Sao Paulo Airport",
		City:      "Cairu",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 66,
		Latitude:  -13.4369440079,
		Longitude: -38.9225006104,
		Timezone:  "America/Bahia",
	},
	"SNPQ": {
		ICAO:      "SNPQ",
		Name:      "Pesqueira Airport",
		City:      "Pesqueira",
		State:     "Pernambuco",
		Country:   "BR",
		Elevation: 2067,
		Latitude:  -8.3741674423,
		Longitude: -36.6377792358,
		Timezone:  "America/Recife",
	},
	"SNPS": {
		ICAO:      "SNPS",
		Name:      "Pontezinha Airport",
		City:      "Buritis",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3277,
		Latitude:  -15.2601003647,
		Longitude: -46.7624015808,
		Timezone:  "America/Sao_Paulo",
	},
	"SNPT": {
		ICAO:      "SNPT",
		Name:      "Passa Tempo Airport",
		City:      "Passa Tempo",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3143,
		Latitude:  -20.6505565643,
		Longitude: -44.5005569458,
		Timezone:  "America/Sao_Paulo",
	},
	"SNPU": {
		ICAO:      "SNPU",
		IATA:      "0",
		Name:      "Paraguacu Airport",
		City:      "Paraguacu",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2838,
		Latitude:  -21.5592002869,
		Longitude: -45.751701355,
		Timezone:  "America/Sao_Paulo",
	},
	"SNPW": {
		ICAO:      "SNPW",
		Name:      "Piaus Airport",
		City:      "Piaus",
		State:     "Para",
		Country:   "BR",
		Elevation: 591,
		Latitude:  -8.8336105347,
		Longitude: -50.0005569458,
		Timezone:  "America/Belem",
	},
	"SNPX": {
		ICAO:      "SNPX",
		IATA:      "PIV",
		Name:      "Pirapora Airport",
		City:      "Pirapora",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1808,
		Latitude:  -17.3169002533,
		Longitude: -44.8602981567,
		Timezone:  "America/Sao_Paulo",
	},
	"SNPY": {
		ICAO:      "SNPY",
		Name:      "Sao Sebastiao do Paraiso Airport",
		City:      "Sao Sebastiao Do Paraiso",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3110,
		Latitude:  -20.9482002258,
		Longitude: -46.9832000732,
		Timezone:  "America/Sao_Paulo",
	},
	"SNPZ": {
		ICAO:      "SNPZ",
		Name:      "Pedra Azul Airport",
		City:      "Pedra Azul",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2625,
		Latitude:  -16.0672225952,
		Longitude: -41.1672210693,
		Timezone:  "America/Sao_Paulo",
	},
	"SNQA": {
		ICAO:      "SNQA",
		Name:      "Fazenda Tracaja Airport",
		City:      "Paragominas",
		State:     "Para",
		Country:   "BR",
		Elevation: 138,
		Latitude:  -2.640556097,
		Longitude: -47.8622207642,
		Timezone:  "America/Belem",
	},
	"SNQB": {
		ICAO:      "SNQB",
		Name:      "Fazenda Val Paraiso Airport",
		City:      "Paragominas",
		State:     "Para",
		Country:   "BR",
		Elevation: 256,
		Latitude:  -3.1683330536,
		Longitude: -48.0713882446,
		Timezone:  "America/Belem",
	},
	"SNQD": {
		ICAO:      "SNQD",
		Name:      "Sousa Airport",
		City:      "Sousa",
		State:     "Paraiba",
		Country:   "BR",
		Elevation: 745,
		Latitude:  -6.7854399681,
		Longitude: -38.2333984375,
		Timezone:  "America/Fortaleza",
	},
	"SNQE": {
		ICAO:      "SNQE",
		Name:      "Fazenda Serra Grande Airport",
		City:      "Parauapebas",
		State:     "Para",
		Country:   "BR",
		Elevation: 682,
		Latitude:  -6.0688891411,
		Longitude: -49.8400001526,
		Timezone:  "America/Belem",
	},
	"SNQG": {
		ICAO:      "SNQG",
		IATA:      "FLB",
		Name:      "Cangapara Airport",
		City:      "Floriano",
		State:     "Piaui",
		Country:   "BR",
		Elevation: 689,
		Latitude:  -6.8463897705,
		Longitude: -43.0773010254,
		Timezone:  "America/Fortaleza",
	},
	"SNQI": {
		ICAO:      "SNQI",
		Name:      "Fazenda Camagril Agropecuaria Airport",
		City:      "Diamantino",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2756,
		Latitude:  -14.1180562973,
		Longitude: -57.3411102295,
		Timezone:  "America/Cuiaba",
	},
	"SNQJ": {
		ICAO:      "SNQJ",
		Name:      "Rancho Paraiso Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 499,
		Latitude:  -17.5227775574,
		Longitude: -55.400554657,
		Timezone:  "America/Campo_Grande",
	},
	"SNQL": {
		ICAO:      "SNQL",
		Name:      "Bom Jesus Airport",
		City:      "Caceres",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 482,
		Latitude:  -15.7349996567,
		Longitude: -57.3463897705,
		Timezone:  "America/Cuiaba",
	},
	"SNQM": {
		ICAO:      "SNQM",
		Name:      "Queimadas Airport",
		City:      "Queimadas",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1050,
		Latitude:  -10.9883327484,
		Longitude: -39.6205558777,
		Timezone:  "America/Bahia",
	},
	"SNQO": {
		ICAO:      "SNQO",
		Name:      "Fazenda Campanario Airport",
		City:      "Bodoquena",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 558,
		Latitude:  -20.375,
		Longitude: -56.5419425964,
		Timezone:  "America/Campo_Grande",
	},
	"SNQP": {
		ICAO:      "SNQP",
		IATA:      "0",
		Name:      "Fazenda Sao Francisco Airport",
		City:      "Magda",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1437,
		Latitude:  -20.5674991608,
		Longitude: -50.2303009033,
		Timezone:  "America/Sao_Paulo",
	},
	"SNQR": {
		ICAO:      "SNQR",
		Name:      "Fazenda Bahia - Don Bosco Airport",
		City:      "Pocone",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 3,
		Latitude:  -16.7277774811,
		Longitude: -57.2333335876,
		Timezone:  "America/Cuiaba",
	},
	"SNQS": {
		ICAO:      "SNQS",
		Name:      "Fazenda Nossa Senhora Auxiliadora Airport",
		City:      "Ivinhema",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1001,
		Latitude:  -22.3044433594,
		Longitude: -54.0305557251,
		Timezone:  "America/Campo_Grande",
	},
	"SNQT": {
		ICAO:      "SNQT",
		Name:      "Fazenda Estrela do Sul Airport",
		City:      "Umuarama",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1542,
		Latitude:  -23.7958335876,
		Longitude: -53.4797210693,
		Timezone:  "America/Sao_Paulo",
	},
	"SNQU": {
		ICAO:      "SNQU",
		Name:      "Mucuge Airport",
		City:      "Mucuge",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 3510,
		Latitude:  -13.0312004089,
		Longitude: -41.4440994263,
		Timezone:  "America/Bahia",
	},
	"SNQV": {
		ICAO:      "SNQV",
		Name:      "Curvelo Airport",
		City:      "Curvelo",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2205,
		Latitude:  -18.7493991852,
		Longitude: -44.4570007324,
		Timezone:  "America/Sao_Paulo",
	},
	"SNQW": {
		ICAO:      "SNQW",
		Name:      "Cururu Airport",
		City:      "Jacareacanga",
		State:     "Para",
		Country:   "BR",
		Elevation: 453,
		Latitude:  -7.5644440651,
		Longitude: -57.7422218323,
		Timezone:  "America/Santarem",
	},
	"SNQX": {
		ICAO:      "SNQX",
		Name:      "Quixada Airport",
		City:      "Quixada",
		State:     "Ceara",
		Country:   "BR",
		Elevation: 653,
		Latitude:  -4.9790701866,
		Longitude: -38.9875984192,
		Timezone:  "America/Fortaleza",
	},
	"SNQZ": {
		ICAO:      "SNQZ",
		Name:      "Fazenda Bagagem Airport",
		City:      "Aral Moreira",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1837,
		Latitude:  -22.8427772522,
		Longitude: -55.5705566406,
		Timezone:  "America/Campo_Grande",
	},
	"SNRA": {
		ICAO:      "SNRA",
		Name:      "Fazenda Santo Andre Airport",
		City:      "Pratinha",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 4134,
		Latitude:  -19.7261104584,
		Longitude: -46.3527793884,
		Timezone:  "America/Sao_Paulo",
	},
	"SNRB": {
		ICAO:      "SNRB",
		Name:      "Fazenda Rosa de Maio Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 102,
		Latitude:  -4.3152098656,
		Longitude: -56.1074981689,
		Timezone:  "America/Santarem",
	},
	"SNRD": {
		ICAO:      "SNRD",
		IATA:      "PDF",
		Name:      "Prado Airport",
		City:      "Prado",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 66,
		Latitude:  -17.2966995239,
		Longitude: -39.2711982727,
		Timezone:  "America/Bahia",
	},
	"SNRE": {
		ICAO:      "SNRE",
		Name:      "Fazenda Rebeca Airport",
		City:      "Serra Dourada",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1575,
		Latitude:  -12.6827001572,
		Longitude: -43.8714981079,
		Timezone:  "America/Bahia",
	},
	"SNRH": {
		ICAO:      "SNRH",
		Name:      "Rachid Saliba Airport",
		City:      "Boa Esperanca",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2625,
		Latitude:  -21.0394001007,
		Longitude: -45.8722000122,
		Timezone:  "America/Sao_Paulo",
	},
	"SNRJ": {
		ICAO:      "SNRJ",
		Name:      "Brejo Airport",
		City:      "Brejo",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 328,
		Latitude:  -3.6669440269,
		Longitude: -42.8338890076,
		Timezone:  "America/Fortaleza",
	},
	"SNRM": {
		ICAO:      "SNRM",
		Name:      "Remanso Airport",
		City:      "Remanso",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1381,
		Latitude:  -9.6502780914,
		Longitude: -42.1172218323,
		Timezone:  "America/Bahia",
	},
	"SNRN": {
		ICAO:      "SNRN",
		Name:      "Fazenda Vinte de Maio Airport",
		City:      "Getulina",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1509,
		Latitude:  -21.808889389,
		Longitude: -49.9711112976,
		Timezone:  "America/Sao_Paulo",
	},
	"SNRP": {
		ICAO:      "SNRP",
		IATA:      "0",
		Name:      "Rio Paranaiba Airport",
		City:      "Rio Paranaiba",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3757,
		Latitude:  -19.2124996185,
		Longitude: -46.2406005859,
		Timezone:  "America/Sao_Paulo",
	},
	"SNRQ": {
		ICAO:      "SNRQ",
		Name:      "Porto Rico Airport",
		City:      "Jacareacanga",
		State:     "Para",
		Country:   "BR",
		Elevation: 331,
		Latitude:  -6.1041669846,
		Longitude: -57.3794441223,
		Timezone:  "America/Santarem",
	},
	"SNRR": {
		ICAO:      "SNRR",
		Name:      "Fazenda Aratau Airport",
		City:      "Novo Repartimento",
		State:     "Para",
		Country:   "BR",
		Elevation: 604,
		Latitude:  -4.151309967,
		Longitude: -50.1632995605,
		Timezone:  "America/Belem",
	},
	"SNRS": {
		ICAO:      "SNRS",
		Name:      "Russas Airport",
		City:      "Russas",
		State:     "Ceara",
		Country:   "BR",
		Elevation: 115,
		Latitude:  -4.9472699165,
		Longitude: -38.0082015991,
		Timezone:  "America/Fortaleza",
	},
	"SNRU": {
		ICAO:      "SNRU",
		IATA:      "CAU",
		Name:      "Caruaru Airport",
		City:      "Caruaru",
		State:     "Pernambuco",
		Country:   "BR",
		Elevation: 1891,
		Latitude:  -8.2823896408,
		Longitude: -36.0135002136,
		Timezone:  "America/Recife",
	},
	"SNRV": {
		ICAO:      "SNRV",
		Name:      "Fazenda Rio Vermelho Airport",
		City:      "Sapucaia",
		State:     "Para",
		Country:   "BR",
		Elevation: 450,
		Latitude:  -6.8827781677,
		Longitude: -49.4638900757,
		Timezone:  "America/Belem",
	},
	"SNRW": {
		ICAO:      "SNRW",
		Name:      "Fazenda Barreiro Airport",
		City:      "Miracema Do Tocantins",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 1033,
		Latitude:  -9.9072217941,
		Longitude: -48.6349983215,
		Timezone:  "America/Araguaina",
	},
	"SNRX": {
		ICAO:      "SNRX",
		IATA:      "0",
		Name:      "Riachao Airport",
		City:      "Riachao",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 1312,
		Latitude:  -7.350279808,
		Longitude: -46.6338996887,
		Timezone:  "America/Fortaleza",
	},
	"SNSA": {
		ICAO:      "SNSA",
		Name:      "Fazenda Bom Sucesso Airport",
		City:      "Vazante",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2454,
		Latitude:  -17.6086101532,
		Longitude: -46.7038879395,
		Timezone:  "America/Sao_Paulo",
	},
	"SNSB": {
		ICAO:      "SNSB",
		Name:      "Sao Bento Airport",
		City:      "Sao Bento",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 230,
		Latitude:  -2.7044439316,
		Longitude: -44.8447227478,
		Timezone:  "America/Fortaleza",
	},
	"SNSC": {
		ICAO:      "SNSC",
		IATA:      "0",
		Name:      "Sacramento Airport",
		City:      "Sacramento",
		Country:   "BR",
		Elevation: 3288,
		Latitude:  -19.8931007385,
		Longitude: -47.4221992493,
		Timezone:  "America/Sao_Paulo",
	},
	"SNSD": {
		ICAO:      "SNSD",
		Name:      "Sao Luiz Airport",
		City:      "Sao Desiderio",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 2664,
		Latitude:  -12.9194440842,
		Longitude: -45.7144432068,
		Timezone:  "America/Bahia",
	},
	"SNSE": {
		ICAO:      "SNSE",
		Name:      "Sento Se Airport",
		City:      "Sento Se",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1306,
		Latitude:  -9.7413892746,
		Longitude: -41.8411102295,
		Timezone:  "America/Bahia",
	},
	"SNSF": {
		ICAO:      "SNSF",
		Name:      "Fazenda Entre Rios Airport",
		City:      "Porto Murtinho",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1050,
		Latitude:  -21.3649997711,
		Longitude: -57.1091651917,
		Timezone:  "America/Campo_Grande",
	},
	"SNSG": {
		ICAO:      "SNSG",
		Name:      "Salgueiro Airport",
		City:      "Salgueiro",
		State:     "Pernambuco",
		Country:   "BR",
		Elevation: 1539,
		Latitude:  -8.0471601486,
		Longitude: -39.1363983154,
		Timezone:  "America/Recife",
	},
	"SNSH": {
		ICAO:      "SNSH",
		IATA:      "0",
		Name:      "Sao Jose Airport",
		City:      "Santarem",
		State:     "Para",
		Country:   "BR",
		Elevation: 440,
		Latitude:  -2.5744400024,
		Longitude: -54.7313995361,
		Timezone:  "America/Santarem",
	},
	"SNSI": {
		ICAO:      "SNSI",
		Name:      "Santa Maria do Suacui Airport",
		City:      "Santa Maria Do Suacui",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1575,
		Latitude:  -18.1986103058,
		Longitude: -42.4661102295,
		Timezone:  "America/Sao_Paulo",
	},
	"SNSJ": {
		ICAO:      "SNSJ",
		Name:      "Fazenda Fogliatelli Airport",
		City:      "Sapezal",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1723,
		Latitude:  -13.5297222137,
		Longitude: -58.5733337402,
		Timezone:  "America/Cuiaba",
	},
	"SNSL": {
		ICAO:      "SNSL",
		Name:      "Fazenda Ipitinga Airport",
		City:      "Almeirim",
		State:     "Para",
		Country:   "BR",
		Elevation: 1680,
		Latitude:  0.3411110044,
		Longitude: -53.8147010803,
		Timezone:  "America/Santarem",
	},
	"SNSM": {
		ICAO:      "SNSM",
		Name:      "Salinopolis Airport",
		City:      "Salinopolis",
		State:     "Para",
		Country:   "BR",
		Elevation: 88,
		Latitude:  -0.6956639886,
		Longitude: -47.3347015381,
		Timezone:  "America/Belem",
	},
	"SNSO": {
		ICAO:      "SNSO",
		Name:      "Serro Airport",
		City:      "Serro",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2428,
		Latitude:  -18.608499527,
		Longitude: -43.4240989685,
		Timezone:  "America/Sao_Paulo",
	},
	"SNSR": {
		ICAO:      "SNSR",
		Name:      "Fazenda Santos Reis Airport",
		City:      "Guarda-Mor",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2165,
		Latitude:  -17.5478992462,
		Longitude: -46.9786987305,
		Timezone:  "America/Sao_Paulo",
	},
	"SNSS": {
		ICAO:      "SNSS",
		Name:      "Salinas Airport",
		City:      "Salinas",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2504,
		Latitude:  -16.2082996368,
		Longitude: -42.3219985962,
		Timezone:  "America/Sao_Paulo",
	},
	"SNST": {
		ICAO:      "SNST",
		Name:      "Souto Soares Airport",
		City:      "Souto Soares",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 2753,
		Latitude:  -12.0972995758,
		Longitude: -41.6407012939,
		Timezone:  "America/Bahia",
	},
	"SNSU": {
		ICAO:      "SNSU",
		Name:      "Fazenda Pontao Airport",
		City:      "Xinguara",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 495,
		Latitude:  -6.881111145,
		Longitude: -49.1622238159,
		Timezone:  "America/Araguaina",
	},
	"SNSV": {
		ICAO:      "SNSV",
		Name:      "Fazenda Sao Pedro Airport",
		City:      "Eldorado",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 591,
		Latitude:  -23.6616668701,
		Longitude: -54.0774993896,
		Timezone:  "America/Campo_Grande",
	},
	"SNSW": {
		ICAO:      "SNSW",
		IATA:      "SFK",
		Name:      "Soure Airport",
		City:      "Soure",
		State:     "Para",
		Country:   "BR",
		Elevation: 43,
		Latitude:  -0.6994310021,
		Longitude: -48.5209999084,
		Timezone:  "America/Belem",
	},
	"SNSX": {
		ICAO:      "SNSX",
		Name:      "Fazenda Sao Martinho Airport",
		City:      "Ilha Solteira",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1148,
		Latitude:  -20.3572216034,
		Longitude: -51.2511100769,
		Timezone:  "America/Sao_Paulo",
	},
	"SNSY": {
		ICAO:      "SNSY",
		Name:      "Fazenda Sao Jose Airport",
		City:      "Bonito",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1247,
		Latitude:  -20.9449996948,
		Longitude: -56.4961128235,
		Timezone:  "America/Campo_Grande",
	},
	"SNSZ": {
		ICAO:      "SNSZ",
		Name:      "Fazenda Itaguassu Airport",
		City:      "Maracaju",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1401,
		Latitude:  -21.355556488,
		Longitude: -55.619720459,
		Timezone:  "America/Campo_Grande",
	},
	"SNTA": {
		ICAO:      "SNTA",
		Name:      "Fazenda Sao Miguel Airport",
		City:      "Pompeia",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1444,
		Latitude:  -21.8686103821,
		Longitude: -50.1163902283,
		Timezone:  "America/Sao_Paulo",
	},
	"SNTB": {
		ICAO:      "SNTB",
		Name:      "Afogados da Ingazeira Airport",
		City:      "Afogados Da Ingazeira",
		State:     "Pernambuco",
		Country:   "BR",
		Elevation: 1844,
		Latitude:  -7.723610878,
		Longitude: -37.618888855,
		Timezone:  "America/Recife",
	},
	"SNTC": {
		ICAO:      "SNTC",
		Name:      "Tambori Airport",
		City:      "Teresina",
		State:     "Piaui",
		Country:   "BR",
		Elevation: 66,
		Latitude:  -5.1599998474,
		Longitude: -42.7197227478,
		Timezone:  "America/Fortaleza",
	},
	"SNTD": {
		ICAO:      "SNTD",
		Name:      "Fazenda Mamoneira Airport",
		City:      "Natalandia",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1862,
		Latitude:  -16.5862007141,
		Longitude: -46.5010986328,
		Timezone:  "America/Sao_Paulo",
	},
	"SNTF": {
		ICAO:      "SNTF",
		IATA:      "TXF",
		Name:      "Teixeira de Freitas Airport",
		City:      "Teixeira De Freitas",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 344,
		Latitude:  -17.5244998932,
		Longitude: -39.6684989929,
		Timezone:  "America/Bahia",
	},
	"SNTG": {
		ICAO:      "SNTG",
		Name:      "Afranio Airport",
		City:      "Afranio",
		State:     "Pernambuco",
		Country:   "BR",
		Elevation: 1703,
		Latitude:  -8.6105556488,
		Longitude: -41.0002784729,
		Timezone:  "America/Recife",
	},
	"SNTI": {
		ICAO:      "SNTI",
		IATA:      "OBI",
		Name:      "Obidos Airport",
		City:      "Obidos",
		State:     "Para",
		Country:   "BR",
		Elevation: 328,
		Latitude:  -1.8671699762,
		Longitude: -55.5144004822,
		Timezone:  "America/Santarem",
	},
	"SNTJ": {
		ICAO:      "SNTJ",
		Name:      "Santa Teresa Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 98,
		Latitude:  -4.2913889885,
		Longitude: -56.0875015259,
		Timezone:  "America/Santarem",
	},
	"SNTK": {
		ICAO:      "SNTK",
		IATA:      "0",
		Name:      "Monte Carmelo Airport",
		City:      "Monte Carmelo",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2904,
		Latitude:  -18.7171993256,
		Longitude: -47.4831008911,
		Timezone:  "America/Sao_Paulo",
	},
	"SNTL": {
		ICAO:      "SNTL",
		Name:      "Tamboril Airport",
		City:      "Tamboril",
		State:     "Ceara",
		Country:   "BR",
		Elevation: 1116,
		Latitude:  -4.8578300476,
		Longitude: -40.3745994568,
		Timezone:  "America/Fortaleza",
	},
	"SNTM": {
		ICAO:      "SNTM",
		Name:      "Turmalina Airport",
		City:      "Turmalina",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2894,
		Latitude:  -17.2338886261,
		Longitude: -42.7338905334,
		Timezone:  "America/Sao_Paulo",
	},
	"SNTO": {
		ICAO:      "SNTO",
		IATA:      "TFL",
		Name:      "Juscelino Kubitscheck Airport",
		City:      "Teofilo Otoni",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1572,
		Latitude:  -17.8922996521,
		Longitude: -41.5135993958,
		Timezone:  "America/Sao_Paulo",
	},
	"SNTP": {
		ICAO:      "SNTP",
		Name:      "Agropalma Airport",
		City:      "Tailandia",
		State:     "Para",
		Country:   "BR",
		Elevation: 66,
		Latitude:  -2.5252780914,
		Longitude: -48.7519454956,
		Timezone:  "America/Belem",
	},
	"SNTQ": {
		ICAO:      "SNTQ",
		IATA:      "0",
		Name:      "Buritirama Airport",
		City:      "Buritirama",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1690,
		Latitude:  -10.7242002487,
		Longitude: -43.6547012329,
		Timezone:  "America/Bahia",
	},
	"SNTR": {
		ICAO:      "SNTR",
		Name:      "Piritiba Airport",
		City:      "Piritiba",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1896,
		Latitude:  -11.7381000519,
		Longitude: -40.5694999695,
		Timezone:  "America/Bahia",
	},
	"SNTS": {
		ICAO:      "SNTS",
		Name:      "Patos Airport",
		City:      "Patos",
		State:     "Paraiba",
		Country:   "BR",
		Elevation: 820,
		Latitude:  -7.0389900208,
		Longitude: -37.2515983582,
		Timezone:  "America/Fortaleza",
	},
	"SNTT": {
		ICAO:      "SNTT",
		Name:      "Tranquilo Restolin Airport",
		City:      "Felixlandia",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2009,
		Latitude:  -18.7581996918,
		Longitude: -45.1076011658,
		Timezone:  "America/Sao_Paulo",
	},
	"SNTU": {
		ICAO:      "SNTU",
		Name:      "Turiacu Airport",
		City:      "Turiacu",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 33,
		Latitude:  -1.7669440508,
		Longitude: -45.4505577087,
		Timezone:  "America/Fortaleza",
	},
	"SNTV": {
		ICAO:      "SNTV",
		Name:      "Fazenda Veredao Airport",
		City:      "Taiobeiras",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2559,
		Latitude:  -15.6553001404,
		Longitude: -41.6540985107,
		Timezone:  "America/Sao_Paulo",
	},
	"SNTW": {
		ICAO:      "SNTW",
		Name:      "Senador Teotonio Vilela Airport",
		City:      "Teotonio Vilela",
		State:     "Alagoas",
		Country:   "BR",
		Elevation: 443,
		Latitude:  -9.896944046,
		Longitude: -36.293056488,
		Timezone:  "America/Maceio",
	},
	"SNTX": {
		ICAO:      "SNTX",
		Name:      "Litoranea Airport",
		City:      "Cururupu",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 26,
		Latitude:  -1.3019440174,
		Longitude: -44.9049987793,
		Timezone:  "America/Fortaleza",
	},
	"SNTY": {
		ICAO:      "SNTY",
		Name:      "Correntina Airport",
		City:      "Correntina",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 2159,
		Latitude:  -13.3805561066,
		Longitude: -44.6172218323,
		Timezone:  "America/Bahia",
	},
	"SNUB": {
		ICAO:      "SNUB",
		Name:      "Uba Airport",
		City:      "Uba",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1115,
		Latitude:  -21.1210002899,
		Longitude: -42.8819007874,
		Timezone:  "America/Sao_Paulo",
	},
	"SNUC": {
		ICAO:      "SNUC",
		Name:      "Acu Airport",
		City:      "Acu",
		State:     "Rio-Grande-do-Norte",
		Country:   "BR",
		Elevation: 394,
		Latitude:  -5.5950498581,
		Longitude: -36.960899353,
		Timezone:  "America/Fortaleza",
	},
	"SNUD": {
		ICAO:      "SNUD",
		Name:      "Urbano Santos Airport",
		City:      "Urbano Santos",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 197,
		Latitude:  -3.1997220516,
		Longitude: -43.386390686,
		Timezone:  "America/Fortaleza",
	},
	"SNUF": {
		ICAO:      "SNUF",
		Name:      "Fazenda Treis Coringas Airport",
		City:      "Uberaba",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3264,
		Latitude:  -19.5244445801,
		Longitude: -47.8525009155,
		Timezone:  "America/Sao_Paulo",
	},
	"SNUG": {
		ICAO:      "SNUG",
		Name:      "Fazenda Eunice Airport",
		City:      "Vila Bela Da Santissima Trindade",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 787,
		Latitude:  -14.9555559158,
		Longitude: -59.6227798462,
		Timezone:  "America/Cuiaba",
	},
	"SNUH": {
		ICAO:      "SNUH",
		Name:      "Piumhi Airport",
		City:      "Piumhi",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2444,
		Latitude:  -20.4386005402,
		Longitude: -45.9928016663,
		Timezone:  "America/Sao_Paulo",
	},
	"SNUI": {
		ICAO:      "SNUI",
		Name:      "Aracuai Airport",
		City:      "Aracuai",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1181,
		Latitude:  -16.8526000977,
		Longitude: -42.0451011658,
		Timezone:  "America/Sao_Paulo",
	},
	"SNUJ": {
		ICAO:      "SNUJ",
		Name:      "Santa Maria da Boa Vista Airport",
		City:      "Santa Maria Da Boa Vista",
		State:     "Pernambuco",
		Country:   "BR",
		Elevation: 1253,
		Latitude:  -8.7819442749,
		Longitude: -39.8722229004,
		Timezone:  "America/Recife",
	},
	"SNUK": {
		ICAO:      "SNUK",
		Name:      "Fazenda Lago Vermelho Airport",
		City:      "Buritizeiro",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1690,
		Latitude:  -17.0352783203,
		Longitude: -45.4863891602,
		Timezone:  "America/Sao_Paulo",
	},
	"SNUL": {
		ICAO:      "SNUL",
		Name:      "Fazenda Aurora III Airport",
		City:      "Ribas Do Rio Pardo",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1214,
		Latitude:  -21.4524993896,
		Longitude: -53.3533325195,
		Timezone:  "America/Campo_Grande",
	},
	"SNUM": {
		ICAO:      "SNUM",
		Name:      "Sao Jose do Egito Airport",
		City:      "Sao Jose Do Egito",
		State:     "Paraiba",
		Country:   "BR",
		Elevation: 2133,
		Latitude:  -7.4869441986,
		Longitude: -37.2911109924,
		Timezone:  "America/Recife",
	},
	"SNUN": {
		ICAO:      "SNUN",
		Name:      "Unai Airport",
		City:      "Unai",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1998,
		Latitude:  -16.3565006256,
		Longitude: -46.9277000427,
		Timezone:  "America/Sao_Paulo",
	},
	"SNUP": {
		ICAO:      "SNUP",
		Name:      "Usina Porto Rico Airport",
		City:      "Campo Alegre",
		State:     "Alagoas",
		Country:   "BR",
		Elevation: 404,
		Latitude:  -9.802778244,
		Longitude: -36.2224998474,
		Timezone:  "America/Maceio",
	},
	"SNUS": {
		ICAO:      "SNUS",
		Name:      "Usina Delta I Airport",
		City:      "Delta",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2100,
		Latitude:  -19.9408321381,
		Longitude: -47.7783317566,
		Timezone:  "America/Sao_Paulo",
	},
	"SNUT": {
		ICAO:      "SNUT",
		Name:      "Utinga Airport",
		City:      "Utinga",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1988,
		Latitude:  -12.1055002213,
		Longitude: -41.0745010376,
		Timezone:  "America/Bahia",
	},
	"SNUU": {
		ICAO:      "SNUU",
		Name:      "Uaua Airport",
		City:      "Uaua",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1335,
		Latitude:  -9.8350000381,
		Longitude: -39.4944458008,
		Timezone:  "America/Bahia",
	},
	"SNUV": {
		ICAO:      "SNUV",
		Name:      "Fazenda Nova Aurora Airport",
		City:      "Anaurilandia",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 951,
		Latitude:  -22.2169437408,
		Longitude: -52.7402763367,
		Timezone:  "America/Campo_Grande",
	},
	"SNUW": {
		ICAO:      "SNUW",
		Name:      "Sertania Airport",
		City:      "Sertania",
		State:     "Pernambuco",
		Country:   "BR",
		Elevation: 1968,
		Latitude:  -8.0574998856,
		Longitude: -37.2894439697,
		Timezone:  "America/Recife",
	},
	"SNUX": {
		ICAO:      "SNUX",
		Name:      "Fazenda Santo Antonio Airport",
		City:      "Ribas Do Rio Pardo",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1142,
		Latitude:  -21.4047222137,
		Longitude: -53.3069458008,
		Timezone:  "America/Campo_Grande",
	},
	"SNUY": {
		ICAO:      "SNUY",
		Name:      "Buritis Airport",
		City:      "Buritis",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1870,
		Latitude:  -15.6316671371,
		Longitude: -46.4286117554,
		Timezone:  "America/Sao_Paulo",
	},
	"SNUZ": {
		ICAO:      "SNUZ",
		Name:      "Usina Coruripe Airport",
		City:      "Coruripe",
		State:     "Alagoas",
		Country:   "BR",
		Elevation: 210,
		Latitude:  -10.0933332443,
		Longitude: -36.1877784729,
		Timezone:  "America/Maceio",
	},
	"SNVA": {
		ICAO:      "SNVA",
		Name:      "Fazenda Bela Vista Airport",
		City:      "Cumaru Do Norte",
		State:     "Para",
		Country:   "BR",
		Elevation: 1037,
		Latitude:  -7.9875001907,
		Longitude: -50.6316680908,
		Timezone:  "America/Belem",
	},
	"SNVB": {
		ICAO:      "SNVB",
		IATA:      "VAL",
		Name:      "Valenca Airport",
		City:      "Valenca",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 21,
		Latitude:  -13.296500206,
		Longitude: -38.992401123,
		Timezone:  "America/Bahia",
	},
	"SNVC": {
		ICAO:      "SNVC",
		Name:      "Vicosa Airport",
		City:      "Vicosa",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2162,
		Latitude:  -20.7447223663,
		Longitude: -42.8419456482,
		Timezone:  "America/Sao_Paulo",
	},
	"SNVD": {
		ICAO:      "SNVD",
		IATA:      "0",
		Name:      "Santa Maria da Vitoria Airport",
		City:      "Santa Maria Da Vitoria",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1850,
		Latitude:  -13.4006004333,
		Longitude: -44.2172012329,
		Timezone:  "America/Bahia",
	},
	"SNVG": {
		ICAO:      "SNVG",
		IATA:      "0",
		Name:      "Volta Grande Airport",
		City:      "Conceicao Das Alagoas",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1831,
		Latitude:  -20.0338993073,
		Longitude: -48.2338981628,
		Timezone:  "America/Sao_Paulo",
	},
	"SNVH": {
		ICAO:      "SNVH",
		Name:      "Fazenda Santa Helena Airport",
		City:      "Caceres",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 492,
		Latitude:  -16.1686115265,
		Longitude: -58.1383323669,
		Timezone:  "America/Cuiaba",
	},
	"SNVI": {
		ICAO:      "SNVI",
		IATA:      "QID",
		Name:      "Melio Viana Airport",
		City:      "Tres Coracoes",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3232,
		Latitude:  -21.790599823,
		Longitude: -45.2691993713,
		Timezone:  "America/Sao_Paulo",
	},
	"SNVJ": {
		ICAO:      "SNVJ",
		Name:      "Fazenda Salto Grande Airport",
		City:      "Jauru",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1444,
		Latitude:  -15.2533330917,
		Longitude: -58.7933311462,
		Timezone:  "America/Cuiaba",
	},
	"SNVK": {
		ICAO:      "SNVK",
		Name:      "Fazenda Santa Terezinha Airport",
		City:      "Tres Lagoas",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1214,
		Latitude:  -20.5013885498,
		Longitude: -52.368888855,
		Timezone:  "America/Campo_Grande",
	},
	"SNVL": {
		ICAO:      "SNVL",
		Name:      "Virgem da Lapa Airport",
		City:      "Virgem Da Lapa",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2477,
		Latitude:  -16.7738895416,
		Longitude: -42.3875007629,
		Timezone:  "America/Sao_Paulo",
	},
	"SNVM": {
		ICAO:      "SNVM",
		Name:      "Fazenda Guaivira Airport",
		City:      "Brasilandia",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1112,
		Latitude:  -20.6644439697,
		Longitude: -52.7672233582,
		Timezone:  "America/Campo_Grande",
	},
	"SNVN": {
		ICAO:      "SNVN",
		Name:      "Fazenda Santa Clara Airport",
		City:      "Rubiacea",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1352,
		Latitude:  -21.3033332825,
		Longitude: -50.7522201538,
		Timezone:  "America/Sao_Paulo",
	},
	"SNVO": {
		ICAO:      "SNVO",
		Name:      "Fazenda Pontal Airport",
		City:      "Carneirinho",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1240,
		Latitude:  -19.978055954,
		Longitude: -50.968334198,
		Timezone:  "America/Sao_Paulo",
	},
	"SNVP": {
		ICAO:      "SNVP",
		Name:      "Fazenda Aracatuba Airport",
		City:      "Santa Fe",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 541,
		Latitude:  -6.9769439697,
		Longitude: -49.1994438171,
		Timezone:  "America/Araguaina",
	},
	"SNVQ": {
		ICAO:      "SNVQ",
		Name:      "Fazenda Santa Monica Airport",
		City:      "Piraque",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 600,
		Latitude:  -6.5230560303,
		Longitude: -48.2658348083,
		Timezone:  "America/Araguaina",
	},
	"SNVR": {
		ICAO:      "SNVR",
		Name:      "Aeroclube da Bahia Airport",
		City:      "Vera Cruz",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 12,
		Latitude:  -13.0247001648,
		Longitude: -38.6657981873,
		Timezone:  "America/Bahia",
	},
	"SNVS": {
		ICAO:      "SNVS",
		IATA:      "BVS",
		Name:      "Breves Airport",
		City:      "Breves",
		State:     "Para",
		Country:   "BR",
		Elevation: 98,
		Latitude:  -1.6365300417,
		Longitude: -50.4435997009,
		Timezone:  "America/Belem",
	},
	"SNVU": {
		ICAO:      "SNVU",
		Name:      "Fazenda Itatuba Airport",
		City:      "Cassia",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2395,
		Latitude:  -20.4308338165,
		Longitude: -46.8600006104,
		Timezone:  "America/Sao_Paulo",
	},
	"SNVV": {
		ICAO:      "SNVV",
		Name:      "Valente Airport",
		City:      "Valente",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1322,
		Latitude:  -11.411110878,
		Longitude: -39.4433326721,
		Timezone:  "America/Bahia",
	},
	"SNVW": {
		ICAO:      "SNVW",
		Name:      "Fazenda Sao Joao do Guapore Airport",
		City:      "Nova Lacerda",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 574,
		Latitude:  -13.9977779388,
		Longitude: -60.0741653442,
		Timezone:  "America/Cuiaba",
	},
	"SNVY": {
		ICAO:      "SNVY",
		Name:      "Fazenda Ouro Verde Airport",
		City:      "Paranhos",
		State:     "Canindeyu",
		Country:   "BR",
		Elevation: 1299,
		Latitude:  -23.9769439697,
		Longitude: -55.3652763367,
		Timezone:  "America/Asuncion",
	},
	"SNVZ": {
		ICAO:      "SNVZ",
		Name:      "Varzea da Palma Airport",
		City:      "Varzea Da Palma",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1719,
		Latitude:  -17.6450004578,
		Longitude: -44.7105560303,
		Timezone:  "America/Sao_Paulo",
	},
	"SNWA": {
		ICAO:      "SNWA",
		Name:      "Guaxuma Airport",
		City:      "Coruripe",
		State:     "Alagoas",
		Country:   "BR",
		Elevation: 322,
		Latitude:  -9.9805555344,
		Longitude: -36.2311096191,
		Timezone:  "America/Maceio",
	},
	"SNWB": {
		ICAO:      "SNWB",
		Name:      "Fazenda Sao Lucas Airport",
		City:      "Mirandopolis",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1099,
		Latitude:  -20.9322223663,
		Longitude: -50.956943512,
		Timezone:  "America/Sao_Paulo",
	},
	"SNWC": {
		ICAO:      "SNWC",
		IATA:      "CMC",
		Name:      "Camocim Airport",
		City:      "Camocim",
		State:     "Ceara",
		Country:   "BR",
		Elevation: 16,
		Latitude:  -2.8961799145,
		Longitude: -40.858001709,
		Timezone:  "America/Fortaleza",
	},
	"SNWD": {
		ICAO:      "SNWD",
		Name:      "Fazenda Guavira Airport",
		City:      "Iguatemi",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1335,
		Latitude:  -23.4302787781,
		Longitude: -54.6083335876,
		Timezone:  "America/Campo_Grande",
	},
	"SNWE": {
		ICAO:      "SNWE",
		Name:      "Fazenda Baia da Bugra Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 507,
		Latitude:  -20.278055191,
		Longitude: -57.2338905334,
		Timezone:  "America/Campo_Grande",
	},
	"SNWF": {
		ICAO:      "SNWF",
		Name:      "Fazenda Cachoeirinha Airport",
		City:      "Ponta Pora",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1476,
		Latitude:  -22.4380569458,
		Longitude: -55.2316665649,
		Timezone:  "America/Campo_Grande",
	},
	"SNWG": {
		ICAO:      "SNWG",
		Name:      "Fazenda Ipora Airport",
		City:      "Itaquirai",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1476,
		Latitude:  -23.5869445801,
		Longitude: -54.0577774048,
		Timezone:  "America/Campo_Grande",
	},
	"SNWH": {
		ICAO:      "SNWH",
		Name:      "Fazenda Sao Mateus Airport",
		City:      "Querencia Do Norte",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1050,
		Latitude:  -22.9666671753,
		Longitude: -53.5433311462,
		Timezone:  "America/Sao_Paulo",
	},
	"SNWI": {
		ICAO:      "SNWI",
		Name:      "Fazenda Sao Joao II Airport",
		City:      "Comodoro",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 594,
		Latitude:  -14.0688886642,
		Longitude: -60.0091667175,
		Timezone:  "America/Cuiaba",
	},
	"SNWJ": {
		ICAO:      "SNWJ",
		Name:      "Fazenda Rancho Alegre Airport",
		City:      "Araputanga",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1345,
		Latitude:  -15.2080564499,
		Longitude: -58.6247215271,
		Timezone:  "America/Cuiaba",
	},
	"SNWK": {
		ICAO:      "SNWK",
		Name:      "Fazenda Santa Terezinha Airport",
		City:      "Pedra Preta",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 722,
		Latitude:  -16.6744441986,
		Longitude: -54.2663879395,
		Timezone:  "America/Cuiaba",
	},
	"SNWL": {
		ICAO:      "SNWL",
		Name:      "Fazenda Ranchinho Airport",
		City:      "Apore",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1788,
		Latitude:  -18.8333320618,
		Longitude: -51.7911109924,
		Timezone:  "America/Sao_Paulo",
	},
	"SNWM": {
		ICAO:      "SNWM",
		Name:      "Fazenda Bonito Airport",
		City:      "Ilha Solteira",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1181,
		Latitude:  -20.3827781677,
		Longitude: -51.1955566406,
		Timezone:  "America/Sao_Paulo",
	},
	"SNWN": {
		ICAO:      "SNWN",
		Name:      "Fazenda Alianca Airport",
		City:      "Pedra Preta",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 942,
		Latitude:  -16.668056488,
		Longitude: -54.3858337402,
		Timezone:  "America/Cuiaba",
	},
	"SNWO": {
		ICAO:      "SNWO",
		Name:      "Fazenda Vera Cruz Airport",
		City:      "Amambai",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1001,
		Latitude:  -23.0177783966,
		Longitude: -54.6244430542,
		Timezone:  "America/Campo_Grande",
	},
	"SNWP": {
		ICAO:      "SNWP",
		Name:      "Laginha Airport",
		City:      "Uniao Dos Palmares",
		State:     "Alagoas",
		Country:   "BR",
		Elevation: 262,
		Latitude:  -9.3138885498,
		Longitude: -36.056388855,
		Timezone:  "America/Maceio",
	},
	"SNWR": {
		ICAO:      "SNWR",
		Name:      "Wilma Rabelo Airport",
		City:      "Senador Jose Porfirio",
		State:     "Para",
		Country:   "BR",
		Elevation: 201,
		Latitude:  -2.636590004,
		Longitude: -51.8269996643,
		Timezone:  "America/Belem",
	},
	"SNWS": {
		ICAO:      "SNWS",
		Name:      "Crateus Airport",
		City:      "Crateus",
		State:     "Ceara",
		Country:   "BR",
		Elevation: 1034,
		Latitude:  -5.2113499641,
		Longitude: -40.7042007446,
		Timezone:  "America/Fortaleza",
	},
	"SNWU": {
		ICAO:      "SNWU",
		Name:      "Fazenda Ponte Quinha Airport",
		City:      "Aral Moreira",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1575,
		Latitude:  -23.0086116791,
		Longitude: -55.5097236633,
		Timezone:  "America/Campo_Grande",
	},
	"SNWZ": {
		ICAO:      "SNWZ",
		Name:      "Uruba Airport",
		City:      "Atalaia",
		State:     "Alagoas",
		Country:   "BR",
		Elevation: 302,
		Latitude:  -9.4849996567,
		Longitude: -35.9588890076,
		Timezone:  "America/Maceio",
	},
	"SNXA": {
		ICAO:      "SNXA",
		IATA:      "0",
		Name:      "Machado Airport",
		City:      "Machado",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2969,
		Latitude:  -21.6781005859,
		Longitude: -45.8410987854,
		Timezone:  "America/Sao_Paulo",
	},
	"SNXB": {
		ICAO:      "SNXB",
		Name:      "Caxambu Airport",
		City:      "Caxambu",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2838,
		Latitude:  -21.9169998169,
		Longitude: -44.9682006836,
		Timezone:  "America/Sao_Paulo",
	},
	"SNXD": {
		ICAO:      "SNXD",
		Name:      "Fazenda Beradao Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 764,
		Latitude:  -6.2727780342,
		Longitude: -55.7858314514,
		Timezone:  "America/Santarem",
	},
	"SNXF": {
		ICAO:      "SNXF",
		Name:      "Retiro Santo Antonio da Fazenda Triunfo Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 394,
		Latitude:  -17.8924999237,
		Longitude: -56.9975013733,
		Timezone:  "America/Campo_Grande",
	},
	"SNXG": {
		ICAO:      "SNXG",
		Name:      "Fazenda dos Castanhais Airport",
		City:      "Sapucaia",
		State:     "Para",
		Country:   "BR",
		Elevation: 745,
		Latitude:  -6.7748699188,
		Longitude: -49.3613014221,
		Timezone:  "America/Belem",
	},
	"SNXH": {
		ICAO:      "SNXH",
		Name:      "Itapecuru Airport",
		City:      "Codo",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 184,
		Latitude:  -4.5130558014,
		Longitude: -44.0200004578,
		Timezone:  "America/Fortaleza",
	},
	"SNXI": {
		ICAO:      "SNXI",
		Name:      "Estancia Turistica de Pereira Barreto Airport",
		City:      "Pereira Barreto",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1289,
		Latitude:  -20.6247215271,
		Longitude: -51.0849990845,
		Timezone:  "America/Sao_Paulo",
	},
	"SNXK": {
		ICAO:      "SNXK",
		Name:      "Fazenda Lagoa do Triunfo Airport",
		City:      "Sao Felix Do Xingu",
		State:     "Para",
		Country:   "BR",
		Elevation: 558,
		Latitude:  -6.5369439125,
		Longitude: -52.7691688538,
		Timezone:  "America/Santarem",
	},
	"SNXL": {
		ICAO:      "SNXL",
		Name:      "Primavera Airport",
		City:      "Querencia",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1201,
		Latitude:  -12.6166667938,
		Longitude: -52.1538887024,
		Timezone:  "America/Cuiaba",
	},
	"SNXM": {
		ICAO:      "SNXM",
		Name:      "Fazenda Itumbiara Airport",
		City:      "Altamira",
		State:     "Para",
		Country:   "BR",
		Elevation: 886,
		Latitude:  -6.4202780724,
		Longitude: -53.6113891602,
		Timezone:  "America/Santarem",
	},
	"SNXN": {
		ICAO:      "SNXN",
		Name:      "Heringer Airport",
		City:      "Balsas",
		State:     "Sergipe",
		Country:   "BR",
		Elevation: 1021,
		Latitude:  -7.3752779961,
		Longitude: -45.9897232056,
		Timezone:  "America/Fortaleza",
	},
	"SNXO": {
		ICAO:      "SNXO",
		Name:      "Fazenda Bom Sucesso Airport",
		City:      "Novo Progresso",
		State:     "Para",
		Country:   "BR",
		Elevation: 600,
		Latitude:  -7.8936109543,
		Longitude: -56.3875007629,
		Timezone:  "America/Santarem",
	},
	"SNXQ": {
		ICAO:      "SNXQ",
		Name:      "Xique-Xique Airport",
		City:      "Xique-Xique",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1384,
		Latitude:  -10.8340997696,
		Longitude: -42.6833992004,
		Timezone:  "America/Bahia",
	},
	"SNXR": {
		ICAO:      "SNXR",
		Name:      "Fazenda Conforto Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 771,
		Latitude:  -7.3933329582,
		Longitude: -56.5230560303,
		Timezone:  "America/Santarem",
	},
	"SNXS": {
		ICAO:      "SNXS",
		IATA:      "QXC",
		Name:      "Fazenda Sao Braz Airport",
		City:      "Barra De Santo Antonio",
		State:     "Alagoas",
		Country:   "BR",
		Elevation: 164,
		Latitude:  -9.3752775192,
		Longitude: -35.5416679382,
		Timezone:  "America/Maceio",
	},
	"SNXV": {
		ICAO:      "SNXV",
		Name:      "Fazenda Lagoa das Conchas Airport",
		City:      "Colniza",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 348,
		Latitude:  -9.0872220993,
		Longitude: -61.1869430542,
		Timezone:  "America/Cuiaba",
	},
	"SNXW": {
		ICAO:      "SNXW",
		Name:      "Chaves Airport",
		City:      "Chaves",
		State:     "Para",
		Country:   "BR",
		Elevation: 20,
		Latitude:  -0.200277999,
		Longitude: -49.9672203064,
		Timezone:  "America/Belem",
	},
	"SNXX": {
		ICAO:      "SNXX",
		Name:      "Maxaranguape Airport",
		City:      "Maxaranguape",
		State:     "Rio-Grande-do-Norte",
		Country:   "BR",
		Elevation: 302,
		Latitude:  -5.3841400146,
		Longitude: -35.5284004211,
		Timezone:  "America/Fortaleza",
	},
	"SNXY": {
		ICAO:      "SNXY",
		Name:      "Cikel Brasil Verde Airport",
		City:      "Portel",
		State:     "Para",
		Country:   "BR",
		Elevation: 98,
		Latitude:  -2.7002780437,
		Longitude: -50.5286102295,
		Timezone:  "America/Belem",
	},
	"SNYA": {
		ICAO:      "SNYA",
		IATA:      "GGF",
		Name:      "Almeirim Airport",
		City:      "Almeirim",
		State:     "Para",
		Country:   "BR",
		Elevation: 584,
		Latitude:  -1.4919439554,
		Longitude: -52.5783348083,
		Timezone:  "America/Santarem",
	},
	"SNYB": {
		ICAO:      "SNYB",
		Name:      "Ituiutaba Airport",
		City:      "Ituiutaba",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1985,
		Latitude:  -18.9997997284,
		Longitude: -49.4866981506,
		Timezone:  "America/Sao_Paulo",
	},
	"SNYC": {
		ICAO:      "SNYC",
		Name:      "Rosa Branca Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 552,
		Latitude:  -7.0091671944,
		Longitude: -56.9044456482,
		Timezone:  "America/Santarem",
	},
	"SNYD": {
		ICAO:      "SNYD",
		Name:      "Pilao Arcado Airport",
		City:      "Pilao Arcado",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1312,
		Latitude:  -10.0669441223,
		Longitude: -42.4505577087,
		Timezone:  "America/Bahia",
	},
	"SNYE": {
		ICAO:      "SNYE",
		IATA:      "PHI",
		Name:      "Pinheiro Airport",
		City:      "Pinheiro",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 125,
		Latitude:  -2.4836111069,
		Longitude: -45.0672225952,
		Timezone:  "America/Fortaleza",
	},
	"SNYF": {
		ICAO:      "SNYF",
		Name:      "Fazenda Parnaiba Airport",
		City:      "Tasso Fragoso",
		Country:   "BR",
		Elevation: 1854,
		Latitude:  -8.5055561066,
		Longitude: -46.1219444275,
		Timezone:  "America/Fortaleza",
	},
	"SNYH": {
		ICAO:      "SNYH",
		IATA:      "ITI",
		Name:      "Agropecuaria Castanhais Airport",
		City:      "Cumaru Do Norte",
		State:     "Para",
		Country:   "BR",
		Elevation: 1096,
		Latitude:  -8.6999998093,
		Longitude: -51.1741676331,
		Timezone:  "America/Belem",
	},
	"SNYI": {
		ICAO:      "SNYI",
		Name:      "Fazenda HJ Airport",
		City:      "Unai",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3113,
		Latitude:  -16.6236114502,
		Longitude: -47.2997207642,
		Timezone:  "America/Sao_Paulo",
	},
	"SNYJ": {
		ICAO:      "SNYJ",
		Name:      "Pista Sao Jorge Airport",
		City:      "Jacareacanga",
		State:     "Para",
		Country:   "BR",
		Elevation: 810,
		Latitude:  -6.1336112022,
		Longitude: -57.4041671753,
		Timezone:  "America/Santarem",
	},
	"SNYL": {
		ICAO:      "SNYL",
		Name:      "Fazenda Sao Joao Airport",
		City:      "Varjao De Minas",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3300,
		Latitude:  -18.4269447327,
		Longitude: -46.0519447327,
		Timezone:  "America/Sao_Paulo",
	},
	"SNYM": {
		ICAO:      "SNYM",
		Name:      "Fazenda Flexas Airport",
		City:      "Sao Romao",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1594,
		Latitude:  -16.3763885498,
		Longitude: -45.1230545044,
		Timezone:  "America/Sao_Paulo",
	},
	"SNYN": {
		ICAO:      "SNYN",
		Name:      "Fazenda Rio das Pedras Airport",
		City:      "Prata",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2300,
		Latitude:  -19.5516662598,
		Longitude: -48.8375015259,
		Timezone:  "America/Sao_Paulo",
	},
	"SNYO": {
		ICAO:      "SNYO",
		Name:      "Fazenda Logradouro Vereda da Ponte Airport",
		City:      "Riachinho",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2848,
		Latitude:  -16.2991676331,
		Longitude: -46.1594429016,
		Timezone:  "America/Sao_Paulo",
	},
	"SNYP": {
		ICAO:      "SNYP",
		Name:      "Chacara Paraiso Airport",
		City:      "Benevides",
		State:     "Para",
		Country:   "BR",
		Elevation: 69,
		Latitude:  -1.3177779913,
		Longitude: -48.2988891602,
		Timezone:  "America/Belem",
	},
	"SNYQ": {
		ICAO:      "SNYQ",
		Name:      "Pista Sao Pedro Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 459,
		Latitude:  -5.3205561638,
		Longitude: -56.6647224426,
		Timezone:  "America/Santarem",
	},
	"SNYR": {
		ICAO:      "SNYR",
		Name:      "Fazenda Agua Azul Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 518,
		Latitude:  -5.947778225,
		Longitude: -56.5547218323,
		Timezone:  "America/Santarem",
	},
	"SNYS": {
		ICAO:      "SNYS",
		Name:      "Destilaria WD Airport",
		City:      "Joao Pinheiro",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3199,
		Latitude:  -18.2205562592,
		Longitude: -45.9911117554,
		Timezone:  "America/Sao_Paulo",
	},
	"SNYT": {
		ICAO:      "SNYT",
		Name:      "Ituacu Airport",
		City:      "Ituacu",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1804,
		Latitude:  -13.8283004761,
		Longitude: -41.3022994995,
		Timezone:  "America/Bahia",
	},
	"SNYU": {
		ICAO:      "SNYU",
		Name:      "Iturama Airport",
		City:      "Iturama",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1558,
		Latitude:  -19.7179898013,
		Longitude: -50.2201795578,
		Timezone:  "America/Sao_Paulo",
	},
	"SNYV": {
		ICAO:      "SNYV",
		Name:      "Fazenda Sococo Airport",
		City:      "Moju",
		State:     "Para",
		Country:   "BR",
		Elevation: 102,
		Latitude:  -2.1197218895,
		Longitude: -48.6458320618,
		Timezone:  "America/Belem",
	},
	"SNYW": {
		ICAO:      "SNYW",
		Name:      "Antonio Guerreiro Airport",
		City:      "Guimaraes",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 245,
		Latitude:  -2.109443903,
		Longitude: -44.6511116028,
		Timezone:  "America/Fortaleza",
	},
	"SNYX": {
		ICAO:      "SNYX",
		Name:      "Sitio Vodo Airport",
		City:      "Cascavel",
		State:     "Ceara",
		Country:   "BR",
		Elevation: 249,
		Latitude:  -4.0949997902,
		Longitude: -38.2475013733,
		Timezone:  "America/Fortaleza",
	},
	"SNYZ": {
		ICAO:      "SNYZ",
		Name:      "Vicente Jose Airport",
		City:      "Russas",
		State:     "Ceara",
		Country:   "BR",
		Elevation: 115,
		Latitude:  -4.8430562019,
		Longitude: -38.144443512,
		Timezone:  "America/Fortaleza",
	},
	"SNZA": {
		ICAO:      "SNZA",
		IATA:      "PPY",
		Name:      "Pouso Alegre Airport",
		City:      "Pouso Alegre",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2904,
		Latitude:  -22.2891998291,
		Longitude: -45.9191017151,
		Timezone:  "America/Sao_Paulo",
	},
	"SNZC": {
		ICAO:      "SNZC",
		Name:      "Fazenda Aguas Claras Airport",
		City:      "Iguatemi",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1220,
		Latitude:  -23.3250007629,
		Longitude: -54.5047225952,
		Timezone:  "America/Campo_Grande",
	},
	"SNZD": {
		ICAO:      "SNZD",
		Name:      "Fazenda Dona Hilda Airport",
		City:      "Montividiu Do Norte",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1424,
		Latitude:  -13.204167366,
		Longitude: -48.6008338928,
		Timezone:  "America/Araguaina",
	},
	"SNZE": {
		ICAO:      "SNZE",
		Name:      "Aeroxingu Airport",
		City:      "Altamira",
		State:     "Para",
		Country:   "BR",
		Elevation: 369,
		Latitude:  -3.171943903,
		Longitude: -52.1916656494,
		Timezone:  "America/Santarem",
	},
	"SNZF": {
		ICAO:      "SNZF",
		Name:      "Fazenda Bela Vista Airport",
		City:      "Soure",
		State:     "Para",
		Country:   "BR",
		Elevation: 66,
		Latitude:  -0.2713890076,
		Longitude: -48.5750007629,
		Timezone:  "America/Belem",
	},
	"SNZH": {
		ICAO:      "SNZH",
		Name:      "Fazenda Pequena Holanda Airport",
		City:      "Alto Parnaiba",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 2835,
		Latitude:  -9.0100002289,
		Longitude: -46.1447219849,
		Timezone:  "America/Fortaleza",
	},
	"SNZK": {
		ICAO:      "SNZK",
		Name:      "Fazenda Canada Airport",
		City:      "Itacarambi",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1510,
		Latitude:  -15.0242996216,
		Longitude: -44.039100647,
		Timezone:  "America/Sao_Paulo",
	},
	"SNZL": {
		ICAO:      "SNZL",
		Name:      "Fazenda Santa Thereza Airport",
		City:      "Novo Horizonte",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1401,
		Latitude:  -21.5713882446,
		Longitude: -49.3008346558,
		Timezone:  "America/Sao_Paulo",
	},
	"SNZO": {
		ICAO:      "SNZO",
		Name:      "Fazenda Bebida Velha Airport",
		City:      "Touros",
		State:     "Rio-Grande-do-Norte",
		Country:   "BR",
		Elevation: 131,
		Latitude:  -5.2969441414,
		Longitude: -35.5366668701,
		Timezone:  "America/Fortaleza",
	},
	"SNZP": {
		ICAO:      "SNZP",
		Name:      "Pocoes Airport",
		City:      "Pocoes",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 2772,
		Latitude:  -14.522600174,
		Longitude: -40.3450012207,
		Timezone:  "America/Bahia",
	},
	"SNZQ": {
		ICAO:      "SNZQ",
		Name:      "Pista Santa Lucia Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 1968,
		Latitude:  -7.2286109924,
		Longitude: -56.2327766418,
		Timezone:  "America/Santarem",
	},
	"SNZR": {
		ICAO:      "SNZR",
		Name:      "Paracatu Airport",
		City:      "Paracatu",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2359,
		Latitude:  -17.2425994873,
		Longitude: -46.8830986023,
		Timezone:  "America/Sao_Paulo",
	},
	"SNZT": {
		ICAO:      "SNZT",
		Name:      "Fazenda Recreio Airport",
		City:      "Paragominas",
		State:     "Para",
		Country:   "BR",
		Elevation: 499,
		Latitude:  -3.5063889027,
		Longitude: -48.3333320618,
		Timezone:  "America/Belem",
	},
	"SNZU": {
		ICAO:      "SNZU",
		Name:      "Fazenda Nova Conceicao Airport",
		City:      "Tome Acu",
		State:     "Para",
		Country:   "BR",
		Elevation: 243,
		Latitude:  -2.8555555344,
		Longitude: -48.2327766418,
		Timezone:  "America/Belem",
	},
	"SNZV": {
		ICAO:      "SNZV",
		Name:      "Fazenda Jamaica Airport",
		City:      "Ulianopolis",
		State:     "Para",
		Country:   "BR",
		Elevation: 361,
		Latitude:  -3.7255558968,
		Longitude: -47.4761123657,
		Timezone:  "America/Belem",
	},
	"SNZW": {
		ICAO:      "SNZW",
		IATA:      "ITE",
		Name:      "Itubera Airport",
		City:      "Itubera",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 13,
		Latitude:  -13.7322216034,
		Longitude: -39.1416664124,
		Timezone:  "America/Bahia",
	},
	"SNZY": {
		ICAO:      "SNZY",
		Name:      "Fazenda Bandeira Airport",
		City:      "Tupa",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1450,
		Latitude:  -22.0005569458,
		Longitude: -50.5977783203,
		Timezone:  "America/Sao_Paulo",
	},
	"SNZZ": {
		ICAO:      "SNZZ",
		Name:      "Muzzi Airport",
		City:      "Coruripe",
		State:     "Alagoas",
		Country:   "BR",
		Elevation: 331,
		Latitude:  -9.9677782059,
		Longitude: -36.198890686,
		Timezone:  "America/Maceio",
	},
	"SOCA": {
		ICAO:      "SOCA",
		IATA:      "CAY",
		Name:      "Cayenne-Rochambeau Airport",
		City:      "Cayenne / Rochambeau",
		State:     "Guyane",
		Country:   "GF",
		Elevation: 26,
		Latitude:  4.8198099136,
		Longitude: -52.3604011536,
		Timezone:  "America/Cayenne",
	},
	"SOGS": {
		ICAO:      "SOGS",
		IATA:      "GSI",
		Name:      "Grand-Santi Airport",
		City:      "Grand-Santi",
		State:     "Guyane",
		Country:   "GF",
		Elevation: 207,
		Latitude:  4.2858333333,
		Longitude: -54.3730555556,
		Timezone:  "America/Cayenne",
	},
	"SOML": {
		ICAO:      "SOML",
		Name:      "Malin Airport",
		City:      "Malin",
		State:     "La-Libertad",
		Country:   "PE",
		Elevation: 5190,
		Latitude:  -7.6333298683,
		Longitude: -78.6500015259,
		Timezone:  "America/Lima",
	},
	"SOOA": {
		ICAO:      "SOOA",
		IATA:      "MPY",
		Name:      "Maripasoula Airport",
		City:      "Maripasoula",
		State:     "Guyane",
		Country:   "GF",
		Elevation: 406,
		Latitude:  3.6575000286,
		Longitude: -54.0372009277,
		Timezone:  "America/Cayenne",
	},
	"SOOG": {
		ICAO:      "SOOG",
		IATA:      "OXP",
		Name:      "Saint-Georges-de-l'Oyapock Airport",
		City:      "Saint-Georges-de-l'Oyapock Airport",
		State:     "Guyane",
		Country:   "GF",
		Elevation: 46,
		Latitude:  3.8975999355,
		Longitude: -51.8041000366,
		Timezone:  "America/Belem",
	},
	"SOOM": {
		ICAO:      "SOOM",
		IATA:      "LDX",
		Name:      "Saint-Laurent-du-Maroni Airport",
		City:      "Saint-Laurent-du-Maroni",
		State:     "Guyane",
		Country:   "GF",
		Elevation: 16,
		Latitude:  5.4830598831,
		Longitude: -54.0344009399,
		Timezone:  "America/Cayenne",
	},
	"SOOR": {
		ICAO:      "SOOR",
		IATA:      "REI",
		Name:      "Regina Airport",
		City:      "Regina",
		State:     "Guyane",
		Country:   "GF",
		Elevation: 82,
		Latitude:  4.3147201538,
		Longitude: -52.1316986084,
		Timezone:  "America/Cayenne",
	},
	"SOOS": {
		ICAO:      "SOOS",
		IATA:      "XAU",
		Name:      "Saul Airport",
		City:      "Saul",
		State:     "Guyane",
		Country:   "GF",
		Elevation: 656,
		Latitude:  3.6136100292,
		Longitude: -53.2042007446,
		Timezone:  "America/Cayenne",
	},
	"SOOY": {
		ICAO:      "SOOY",
		Name:      "Sinnamary",
		City:      "Sinnamary",
		State:     "Guyane",
		Country:   "GF",
		Elevation: 33,
		Latitude:  5.3735799789,
		Longitude: -52.9455986023,
		Timezone:  "America/Cayenne",
	},
	"SPAA": {
		ICAO:      "SPAA",
		Name:      "Caraz Airport",
		City:      "Ancash",
		State:     "Ancash",
		Country:   "PE",
		Elevation: 7401,
		Latitude:  -9.0514287949,
		Longitude: -77.8101425171,
		Timezone:  "America/Lima",
	},
	"SPAB": {
		ICAO:      "SPAB",
		Name:      "Huancabamba Airport",
		City:      "Huancabamba",
		State:     "Piura",
		Country:   "PE",
		Elevation: 6312,
		Latitude:  -5.256770134,
		Longitude: -79.4429016113,
		Timezone:  "America/Lima",
	},
	"SPAC": {
		ICAO:      "SPAC",
		Name:      "Ciro Alegria Airport",
		City:      "Bagua",
		State:     "Amazonas",
		Country:   "PE",
		Elevation: 547,
		Latitude:  -4.5669999123,
		Longitude: -77.9169998169,
		Timezone:  "America/Lima",
	},
	"SPAG": {
		ICAO:      "SPAG",
		Name:      "Aguaytia Airport",
		City:      "Aguaytia",
		State:     "Ucayali",
		Country:   "PE",
		Elevation: 5853,
		Latitude:  -9.0374097824,
		Longitude: -75.5092010498,
		Timezone:  "America/Lima",
	},
	"SPAI": {
		ICAO:      "SPAI",
		Name:      "Urpay Airport",
		City:      "Pataz",
		State:     "La-Libertad",
		Country:   "PE",
		Elevation: 10400,
		Latitude:  -7.7833333015,
		Longitude: -78.0999984741,
		Timezone:  "America/Lima",
	},
	"SPAL": {
		ICAO:      "SPAL",
		Name:      "Alao Airport",
		City:      "El Dorado",
		State:     "San-Martin",
		Country:   "PE",
		Elevation: 1740,
		Latitude:  -6.5,
		Longitude: -76.7166976929,
		Timezone:  "America/Lima",
	},
	"SPAM": {
		ICAO:      "SPAM",
		Name:      "Camana Airport",
		City:      "Camana",
		State:     "Arequipa",
		Country:   "PE",
		Elevation: 209,
		Latitude:  -16.6329994202,
		Longitude: -72.6829986572,
		Timezone:  "America/Lima",
	},
	"SPAN": {
		ICAO:      "SPAN",
		Name:      "Sullana Airport",
		City:      "Sullana",
		State:     "Piura",
		Country:   "PE",
		Elevation: 197,
		Latitude:  -4.9038887024,
		Longitude: -80.6852798462,
		Timezone:  "America/Lima",
	},
	"SPAO": {
		ICAO:      "SPAO",
		IATA:      "APE",
		Name:      "San Juan Aposento Airport",
		City:      "San Juan Aposento",
		State:     "Ica",
		Country:   "PE",
		Elevation: 23,
		Latitude:  -15.3538999557,
		Longitude: -75.167098999,
		Timezone:  "America/Lima",
	},
	"SPAP": {
		ICAO:      "SPAP",
		Name:      "Picota Airport",
		City:      "Picota",
		State:     "San-Martin",
		Country:   "PE",
		Elevation: 948,
		Latitude:  -6.9000000954,
		Longitude: -76.3830032349,
		Timezone:  "America/Lima",
	},
	"SPAR": {
		ICAO:      "SPAR",
		IATA:      "ALD",
		Name:      "Alerta Airport",
		City:      "Fortaleza",
		State:     "Madre-de-Dios",
		Country:   "PE",
		Elevation: 797,
		Latitude:  -11.6829996109,
		Longitude: -69.3330001831,
		Timezone:  "America/Lima",
	},
	"SPAS": {
		ICAO:      "SPAS",
		IATA:      "AOP",
		Name:      "Alferez FAP Alfredo Vladimir Sara Bauer Airport",
		City:      "Andoas",
		State:     "Loreto",
		Country:   "PE",
		Elevation: 728,
		Latitude:  -2.7961299419,
		Longitude: -76.4665985107,
		Timezone:  "America/Lima",
	},
	"SPAT": {
		ICAO:      "SPAT",
		Name:      "Aguas Calientes Airport",
		City:      "Aguas Calientes",
		State:     "Huanuco",
		Country:   "PE",
		Elevation: 9840,
		Latitude:  -8.8333301544,
		Longitude: -74.6832962036,
		Timezone:  "America/Lima",
	},
	"SPAY": {
		ICAO:      "SPAY",
		IATA:      "ATG",
		Name:      "Teniente General Gerardo Perez Pinedo Airport",
		City:      "Atalaya",
		State:     "Junin",
		Country:   "PE",
		Elevation: 751,
		Latitude:  -10.7291002274,
		Longitude: -73.7665023804,
		Timezone:  "America/Lima",
	},
	"SPBA": {
		ICAO:      "SPBA",
		Name:      "Barranca Airport",
		City:      "Barranca",
		State:     "Lima-region",
		Country:   "PE",
		Elevation: 160,
		Latitude:  -10.75,
		Longitude: -77.7666702271,
		Timezone:  "America/Lima",
	},
	"SPBB": {
		ICAO:      "SPBB",
		IATA:      "MBP",
		Name:      "Moyobamba Airport",
		City:      "Moyobamba",
		State:     "San-Martin",
		Country:   "PE",
		Elevation: 2749,
		Latitude:  -6.0188889503,
		Longitude: -76.9883270264,
		Timezone:  "America/Lima",
	},
	"SPBC": {
		ICAO:      "SPBC",
		Name:      "Caballococha Airport",
		City:      "Caballococha",
		State:     "Loreto",
		Country:   "PE",
		Elevation: 328,
		Latitude:  -3.9168601036,
		Longitude: -70.5082015991,
		Timezone:  "America/Lima",
	},
	"SPBL": {
		ICAO:      "SPBL",
		IATA:      "BLP",
		Name:      "Huallaga Airport",
		City:      "Bellavista",
		State:     "San-Martin",
		Country:   "PE",
		Elevation: 980,
		Latitude:  -7.0605602264,
		Longitude: -76.5821990967,
		Timezone:  "America/Lima",
	},
	"SPBQ": {
		ICAO:      "SPBQ",
		Name:      "Nuevo Acari Airport",
		City:      "La Bella Union",
		State:     "Arequipa",
		Country:   "PE",
		Elevation: 590,
		Latitude:  -15.4833335876,
		Longitude: -74.6833343506,
		Timezone:  "America/Lima",
	},
	"SPBR": {
		ICAO:      "SPBR",
		IATA:      "IBP",
		Name:      "Iberia Airport",
		City:      "Iberia",
		State:     "Madre-de-Dios",
		Country:   "PE",
		Elevation: 750,
		Latitude:  -11.4116001129,
		Longitude: -69.4887008667,
		Timezone:  "America/Lima",
	},
	"SPBS": {
		ICAO:      "SPBS",
		Name:      "Bellavista Airport",
		City:      "Jeberos",
		State:     "Loreto",
		Country:   "PE",
		Elevation: 675,
		Latitude:  -5.2883901596,
		Longitude: -76.292098999,
		Timezone:  "America/Lima",
	},
	"SPBT": {
		ICAO:      "SPBT",
		Name:      "Obenteni Airport",
		City:      "Obenteni",
		State:     "Ucayali",
		Country:   "PE",
		Elevation: 3290,
		Latitude:  -10.7477779388,
		Longitude: -74.2205581665,
		Timezone:  "America/Lima",
	},
	"SPBY": {
		ICAO:      "SPBY",
		Name:      "Bayovar",
		City:      "Bayovar",
		State:     "Piura",
		Country:   "PE",
		Elevation: 88,
		Latitude:  -5.8307499886,
		Longitude: -81.0258026123,
		Timezone:  "America/Lima",
	},
	"SPCC": {
		ICAO:      "SPCC",
		Name:      "Ciudad Constitucion Airport",
		City:      "Ciudad Constitucion",
		State:     "Pasco",
		Country:   "PE",
		Elevation: 775,
		Latitude:  -9.865278244,
		Longitude: -75.0088882446,
		Timezone:  "America/Lima",
	},
	"SPCG": {
		ICAO:      "SPCG",
		Name:      "Casa Grande Airport",
		City:      "Ascope",
		State:     "La-Libertad",
		Country:   "PE",
		Elevation: 475,
		Latitude:  -7.743888855,
		Longitude: -79.1863861084,
		Timezone:  "America/Lima",
	},
	"SPCH": {
		ICAO:      "SPCH",
		Name:      "Tocache Airport",
		City:      "Tocache",
		State:     "San-Martin",
		Country:   "PE",
		Elevation: 1500,
		Latitude:  -8.1829996109,
		Longitude: -76.516998291,
		Timezone:  "America/Lima",
	},
	"SPCL": {
		ICAO:      "SPCL",
		IATA:      "PCL",
		Name:      "Cap FAP David Abenzur Rengifo International Airport",
		City:      "Pucallpa",
		State:     "Ucayali",
		Country:   "PE",
		Elevation: 513,
		Latitude:  -8.3779401779,
		Longitude: -74.5743026733,
		Timezone:  "America/Lima",
	},
	"SPCM": {
		ICAO:      "SPCM",
		Name:      "Contamana Airport",
		City:      "Contamana",
		State:     "Loreto",
		Country:   "PE",
		Elevation: 480,
		Latitude:  -7.3366699219,
		Longitude: -74.9914016724,
		Timezone:  "America/Lima",
	},
	"SPCN": {
		ICAO:      "SPCN",
		Name:      "Cunocuno Airport",
		City:      "Cunocuno",
		State:     "Arequipa",
		Country:   "PE",
		Elevation: 5300,
		Latitude:  -15.9772224426,
		Longitude: -73.1083297729,
		Timezone:  "America/Lima",
	},
	"SPCP": {
		ICAO:      "SPCP",
		Name:      "Pucacaca Airport",
		City:      "Pucacaca",
		State:     "San-Martin",
		Country:   "PE",
		Elevation: 977,
		Latitude:  -6.8330001831,
		Longitude: -76.3330001831,
		Timezone:  "America/Lima",
	},
	"SPCR": {
		ICAO:      "SPCR",
		Name:      "Acari Airbase Airport",
		City:      "Acari",
		State:     "Arequipa",
		Country:   "PE",
		Elevation: 540,
		Latitude:  -15.5,
		Longitude: -74.6666717529,
		Timezone:  "America/Lima",
	},
	"SPCT": {
		ICAO:      "SPCT",
		Name:      "Chota Airport",
		City:      "Chota",
		State:     "Cajamarca",
		Country:   "PE",
		Elevation: 7800,
		Latitude:  -6.5500001907,
		Longitude: -78.6500015259,
		Timezone:  "America/Lima",
	},
	"SPDO": {
		ICAO:      "SPDO",
		Name:      "Mollendo Airport",
		City:      "Mollendo",
		State:     "Arequipa",
		Country:   "PE",
		Elevation: 9,
		Latitude:  -17.0454006195,
		Longitude: -71.9837036133,
		Timezone:  "America/Lima",
	},
	"SPDR": {
		ICAO:      "SPDR",
		IATA:      "TDP",
		Name:      "Trompeteros Airport",
		City:      "Corrientes",
		State:     "Loreto",
		Country:   "PE",
		Elevation: 427,
		Latitude:  -3.8060100079,
		Longitude: -75.0392990112,
		Timezone:  "America/Lima",
	},
	"SPEB": {
		ICAO:      "SPEB",
		Name:      "Pebas Airport",
		City:      "Loreto",
		State:     "Loreto",
		Country:   "PE",
		Elevation: 383,
		Latitude:  -3.3333001137,
		Longitude: -71.8167037964,
		Timezone:  "America/Lima",
	},
	"SPEE": {
		ICAO:      "SPEE",
		Name:      "El Estrecho Airport",
		State:     "Loreto",
		Country:   "PE",
		Elevation: 421,
		Latitude:  -2.4540600777,
		Longitude: -72.6706008911,
		Timezone:  "America/Bogota",
	},
	"SPEN": {
		ICAO:      "SPEN",
		Name:      "Iscozasin Airport",
		City:      "Iscozasin",
		State:     "Pasco",
		Country:   "PE",
		Elevation: 898,
		Latitude:  -10.1829996109,
		Longitude: -75.1500015259,
		Timezone:  "America/Lima",
	},
	"SPEO": {
		ICAO:      "SPEO",
		IATA:      "CHM",
		Name:      "Teniente FAP Jaime A De Montreuil Morales Airport",
		City:      "Chimbote",
		State:     "Apurimac",
		Country:   "PE",
		Elevation: 69,
		Latitude:  -9.1496095657,
		Longitude: -78.5238037109,
		Timezone:  "America/Lima",
	},
	"SPEP": {
		ICAO:      "SPEP",
		Name:      "Puerto Esperanza Airport",
		City:      "Esperanza",
		State:     "Ucayali",
		Country:   "PE",
		Elevation: 725,
		Latitude:  -9.7681303024,
		Longitude: -70.7064971924,
		Timezone:  "America/Lima",
	},
	"SPEQ": {
		ICAO:      "SPEQ",
		Name:      "Cesar Torke Podesta Airport",
		City:      "Moquegua",
		State:     "Moquegua",
		Country:   "PE",
		Elevation: 4480,
		Latitude:  -17.1790008545,
		Longitude: -70.9308013916,
		Timezone:  "America/Lima",
	},
	"SPEZ": {
		ICAO:      "SPEZ",
		Name:      "Puerto Bermudez Airport",
		City:      "Oxapampa",
		State:     "Loreto",
		Country:   "PE",
		Elevation: 840,
		Latitude:  -10.299200058,
		Longitude: -74.9372024536,
		Timezone:  "America/Lima",
	},
	"SPFA": {
		ICAO:      "SPFA",
		Name:      "Fausa Airport",
		City:      "Fausa",
		State:     "Cusco",
		Country:   "PE",
		Elevation: 14809,
		Latitude:  -14.709400177,
		Longitude: -71.7311019897,
		Timezone:  "America/Lima",
	},
	"SPGB": {
		ICAO:      "SPGB",
		Name:      "Galilea Airport",
		State:     "Amazonas",
		Country:   "PE",
		Elevation: 597,
		Latitude:  -4.0318799019,
		Longitude: -77.7587966919,
		Timezone:  "America/Lima",
	},
	"SPGM": {
		ICAO:      "SPGM",
		IATA:      "TGI",
		Name:      "Tingo Maria Airport",
		City:      "Tingo Maria",
		State:     "Huanuco",
		Country:   "PE",
		Elevation: 2010,
		Latitude:  -9.1330003738,
		Longitude: -75.9499969482,
		Timezone:  "America/Lima",
	},
	"SPGP": {
		ICAO:      "SPGP",
		Name:      "Gueppi\u00ad Airport",
		State:     "Loreto",
		Country:   "PE",
		Elevation: 680,
		Latitude:  -0.1190560013,
		Longitude: -75.2479019165,
		Timezone:  "America/Bogota",
	},
	"SPGS": {
		ICAO:      "SPGS",
		Name:      "Lagunas Airport",
		City:      "Cajamarca",
		State:     "Cajamarca",
		Country:   "PE",
		Elevation: 12675,
		Latitude:  -6.9000000954,
		Longitude: -78.6333312988,
		Timezone:  "America/Lima",
	},
	"SPGT": {
		ICAO:      "SPGT",
		Name:      "Puerto Victoria Airport",
		City:      "Puerto Victoria",
		State:     "Pasco",
		Country:   "PE",
		Elevation: 782,
		Latitude:  -9.9026098251,
		Longitude: -74.9641036987,
		Timezone:  "America/Lima",
	},
	"SPGU": {
		ICAO:      "SPGU",
		Name:      "Bagua Airport",
		City:      "Bagua",
		State:     "Amazonas",
		Country:   "PE",
		Elevation: 1375,
		Latitude:  -5.6347222328,
		Longitude: -78.5305557251,
		Timezone:  "America/Lima",
	},
	"SPHA": {
		ICAO:      "SPHA",
		Name:      "Chincha Airport",
		City:      "Chincha",
		State:     "Ica",
		Country:   "PE",
		Elevation: 640,
		Latitude:  -13.3706998825,
		Longitude: -76.1128997803,
		Timezone:  "America/Lima",
	},
	"SPHC": {
		ICAO:      "SPHC",
		Name:      "Chala Airport",
		City:      "Puerto Chala",
		State:     "Arequipa",
		Country:   "PE",
		Elevation: 112,
		Latitude:  -15.8699998856,
		Longitude: -74.2473983765,
		Timezone:  "America/Lima",
	},
	"SPHI": {
		ICAO:      "SPHI",
		IATA:      "CIX",
		Name:      "Capitan FAP Jose A Quinones Gonzales International Airport",
		City:      "Chiclayo",
		State:     "Lambayeque",
		Country:   "PE",
		Elevation: 97,
		Latitude:  -6.7874798775,
		Longitude: -79.8281021118,
		Timezone:  "America/Lima",
	},
	"SPHL": {
		ICAO:      "SPHL",
		Name:      "Olmos Airport",
		City:      "<Olmos",
		State:     "Lambayeque",
		Country:   "PE",
		Elevation: 585,
		Latitude:  -5.9847221375,
		Longitude: -79.7452774048,
		Timezone:  "America/Lima",
	},
	"SPHO": {
		ICAO:      "SPHO",
		IATA:      "AYP",
		Name:      "Coronel FAP Alfredo Mendivil Duarte Airport",
		City:      "Ayacucho",
		State:     "Ayacucho",
		Country:   "PE",
		Elevation: 8917,
		Latitude:  -13.154800415,
		Longitude: -74.2043991089,
		Timezone:  "America/Lima",
	},
	"SPHU": {
		ICAO:      "SPHU",
		Name:      "Huancayo Airport",
		City:      "Huancayo",
		State:     "Junin",
		Country:   "PE",
		Elevation: 10800,
		Latitude:  -12.069899559,
		Longitude: -75.1985015869,
		Timezone:  "America/Lima",
	},
	"SPHV": {
		ICAO:      "SPHV",
		Name:      "Huanuco Viejo Airport",
		City:      "Huanuco Viejo",
		State:     "Huanuco",
		Country:   "PE",
		Elevation: 6320,
		Latitude:  -9.926320076,
		Longitude: -76.2339019775,
		Timezone:  "America/Lima",
	},
	"SPHY": {
		ICAO:      "SPHY",
		IATA:      "ANS",
		Name:      "Andahuaylas Airport",
		City:      "Andahuaylas",
		State:     "Apurimac",
		Country:   "PE",
		Elevation: 11300,
		Latitude:  -13.7063999176,
		Longitude: -73.350402832,
		Timezone:  "America/Lima",
	},
	"SPHZ": {
		ICAO:      "SPHZ",
		IATA:      "ATA",
		Name:      "Comandante FAP German Arias Graziani Airport",
		City:      "Anta",
		State:     "Ancash",
		Country:   "PE",
		Elevation: 9097,
		Latitude:  -9.3474397659,
		Longitude: -77.5983963013,
		Timezone:  "America/Lima",
	},
	"SPIA": {
		ICAO:      "SPIA",
		Name:      "Ica Airport",
		City:      "Ica",
		State:     "Ica",
		Country:   "PE",
		Elevation: 1500,
		Latitude:  -14.2000999451,
		Longitude: -75.5738983154,
		Timezone:  "America/Lima",
	},
	"SPIL": {
		ICAO:      "SPIL",
		IATA:      "UMI",
		Name:      "Quince Air Base",
		City:      "Quince Mil",
		State:     "Cusco",
		Country:   "PE",
		Elevation: 2047,
		Latitude:  -13.233300209,
		Longitude: -70.7533035278,
		Timezone:  "America/Lima",
	},
	"SPIN": {
		ICAO:      "SPIN",
		Name:      "Inapari Airport",
		City:      "Inapari",
		State:     "Madre-de-Dios",
		Country:   "PE",
		Elevation: 917,
		Latitude:  -10.9790000916,
		Longitude: -69.5594024658,
		Timezone:  "America/La_Paz",
	},
	"SPIP": {
		ICAO:      "SPIP",
		Name:      "Satipo Airport",
		City:      "Satipo",
		State:     "Junin",
		Country:   "PE",
		Elevation: 2099,
		Latitude:  -11.2829999924,
		Longitude: -74.6330032349,
		Timezone:  "America/Lima",
	},
	"SPIR": {
		ICAO:      "SPIR",
		Name:      "Patria Airport",
		City:      "Pillcopata",
		State:     "Cusco",
		Country:   "PE",
		Elevation: 2224,
		Latitude:  -13.0329999924,
		Longitude: -71.266998291,
		Timezone:  "America/Lima",
	},
	"SPIS": {
		ICAO:      "SPIS",
		Name:      "Pias Airport",
		City:      "Pataz",
		State:     "La-Libertad",
		Country:   "PE",
		Elevation: 8500,
		Latitude:  -7.87169981,
		Longitude: -77.547203064,
		Timezone:  "America/Lima",
	},
	"SPIT": {
		ICAO:      "SPIT",
		Name:      "Paita Airport",
		City:      "Paita Wesr",
		State:     "Piura",
		Country:   "PE",
		Elevation: 230,
		Latitude:  -5.0859498978,
		Longitude: -81.1528015137,
		Timezone:  "America/Lima",
	},
	"SPIY": {
		ICAO:      "SPIY",
		Name:      "Yauri Airport",
		City:      "Yauri",
		State:     "Cusco",
		Country:   "PE",
		Elevation: 12972,
		Latitude:  -14.8000001907,
		Longitude: -71.4329986572,
		Timezone:  "America/Lima",
	},
	"SPIZ": {
		ICAO:      "SPIZ",
		IATA:      "UCZ",
		Name:      "Uchiza Airport",
		City:      "Uchiza",
		State:     "Huanuco",
		Country:   "PE",
		Elevation: 1965,
		Latitude:  -8.4670000076,
		Longitude: -76.3499984741,
		Timezone:  "America/Lima",
	},
	"SPJA": {
		ICAO:      "SPJA",
		IATA:      "RIJ",
		Name:      "Juan Simons Vela Airport",
		City:      "Rioja",
		State:     "San-Martin",
		Country:   "PE",
		Elevation: 2707,
		Latitude:  -6.0678601265,
		Longitude: -77.1600036621,
		Timezone:  "America/Lima",
	},
	"SPJB": {
		ICAO:      "SPJB",
		Name:      "Pampa Grande Airport",
		City:      "Pampa Grande",
		State:     "Cajamarca",
		Country:   "PE",
		Elevation: 8727,
		Latitude:  -7.5999999046,
		Longitude: -78.0500030518,
		Timezone:  "America/Lima",
	},
	"SPJC": {
		ICAO:      "SPJC",
		IATA:      "LIM",
		Name:      "Jorge Chavez International Airport",
		City:      "Lima",
		State:     "Callao",
		Country:   "PE",
		Elevation: 113,
		Latitude:  -12.021900177,
		Longitude: -77.1143035889,
		Timezone:  "America/Lima",
	},
	"SPJE": {
		ICAO:      "SPJE",
		IATA:      "JAE",
		Name:      "Shumba Airport",
		City:      "Jaen",
		State:     "Cajamarca",
		Country:   "PE",
		Elevation: 2477,
		Latitude:  -5.5924801826,
		Longitude: -78.7740020752,
		Timezone:  "America/Lima",
	},
	"SPJI": {
		ICAO:      "SPJI",
		IATA:      "JJI",
		Name:      "Juanjui Airport",
		City:      "Juanjui",
		State:     "San-Martin",
		Country:   "PE",
		Elevation: 1148,
		Latitude:  -7.1690998077,
		Longitude: -76.7285995483,
		Timezone:  "America/Lima",
	},
	"SPJJ": {
		ICAO:      "SPJJ",
		IATA:      "JAU",
		Name:      "Francisco Carle Airport",
		City:      "Jauja",
		State:     "Junin",
		Country:   "PE",
		Elevation: 11034,
		Latitude:  -11.7831001282,
		Longitude: -75.4733963013,
		Timezone:  "America/Lima",
	},
	"SPJL": {
		ICAO:      "SPJL",
		IATA:      "JUL",
		Name:      "Inca Manco Capac International Airport",
		City:      "Juliaca",
		State:     "Puno",
		Country:   "PE",
		Elevation: 12552,
		Latitude:  -15.4671001434,
		Longitude: -70.158203125,
		Timezone:  "America/Lima",
	},
	"SPJN": {
		ICAO:      "SPJN",
		IATA:      "SJA",
		Name:      "San Juan de Marcona Airport",
		City:      "San Juan de Marcona",
		State:     "Ica",
		Country:   "PE",
		Elevation: 144,
		Latitude:  -15.3524999619,
		Longitude: -75.1371994019,
		Timezone:  "America/Lima",
	},
	"SPJR": {
		ICAO:      "SPJR",
		IATA:      "CJA",
		Name:      "Mayor General FAP Armando Revoredo Iglesias Airport",
		City:      "Cajamarca",
		State:     "Cajamarca",
		Country:   "PE",
		Elevation: 8781,
		Latitude:  -7.1391801834,
		Longitude: -78.489402771,
		Timezone:  "America/Lima",
	},
	"SPLA": {
		ICAO:      "SPLA",
		Name:      "Luisiana Airport",
		City:      "Luisiana",
		State:     "Cusco",
		Country:   "PE",
		Latitude:  -12.6667003632,
		Longitude: -73.7332992554,
		Timezone:  "America/Lima",
	},
	"SPLC": {
		ICAO:      "SPLC",
		Name:      "Mariano Melgar",
		City:      "La Joya",
		State:     "Arequipa",
		Country:   "PE",
		Elevation: 3890,
		Latitude:  -16.7915000916,
		Longitude: -71.8865966797,
		Timezone:  "America/Lima",
	},
	"SPLD": {
		ICAO:      "SPLD",
		Name:      "Celendin Airport",
		City:      "Celendin",
		State:     "Cajamarca",
		Country:   "PE",
		Elevation: 8600,
		Latitude:  -6.8675999641,
		Longitude: -78.1499023438,
		Timezone:  "America/Lima",
	},
	"SPLG": {
		ICAO:      "SPLG",
		Name:      "Lagarto Nuevo Airport",
		City:      "Lagarto",
		State:     "Madre-de-Dios",
		Country:   "PE",
		Elevation: 763,
		Latitude:  -12.6415996552,
		Longitude: -69.8375015259,
		Timezone:  "America/Lima",
	},
	"SPLN": {
		ICAO:      "SPLN",
		IATA:      "RIM",
		Name:      "San Nicolas Airport",
		City:      "Rodriguez de Mendoza",
		State:     "Amazonas",
		Country:   "PE",
		Elevation: 5082,
		Latitude:  -6.3923101425,
		Longitude: -77.5011978149,
		Timezone:  "America/Lima",
	},
	"SPLO": {
		ICAO:      "SPLO",
		IATA:      "ILQ",
		Name:      "Ilo Airport",
		City:      "Ilo",
		State:     "Moquegua",
		Country:   "PE",
		Elevation: 72,
		Latitude:  -17.6949996948,
		Longitude: -71.34400177,
		Timezone:  "America/Lima",
	},
	"SPLP": {
		ICAO:      "SPLP",
		Name:      "Las Palmas Airport",
		City:      "Chorrillos",
		State:     "Lima-region",
		Country:   "PE",
		Elevation: 250,
		Latitude:  -12.1606998444,
		Longitude: -76.9989013672,
		Timezone:  "America/Lima",
	},
	"SPLS": {
		ICAO:      "SPLS",
		Name:      "Zorrillos Airport",
		City:      "Zorrillos",
		State:     "Ucayali",
		Country:   "PE",
		Elevation: 711,
		Latitude:  -8.4169998169,
		Longitude: -75.1330032349,
		Timezone:  "America/Lima",
	},
	"SPLT": {
		ICAO:      "SPLT",
		Name:      "Lobitos Airport",
		City:      "Lobitos",
		State:     "Piura",
		Country:   "PE",
		Elevation: 45,
		Latitude:  -4.4486112595,
		Longitude: -81.2755584717,
		Timezone:  "America/Lima",
	},
	"SPME": {
		ICAO:      "SPME",
		IATA:      "TBP",
		Name:      "Capitan FAP Pedro Canga Rodriguez Airport",
		City:      "Tumbes",
		State:     "Tumbes",
		Country:   "PE",
		Elevation: 115,
		Latitude:  -3.5525300503,
		Longitude: -80.381401062,
		Timezone:  "America/Lima",
	},
	"SPMF": {
		ICAO:      "SPMF",
		IATA:      "MZA",
		Name:      "Manuel Prado Ugarteche Airport",
		City:      "Mazamari",
		State:     "Junin",
		Country:   "PE",
		Elevation: 2247,
		Latitude:  -11.3254003525,
		Longitude: -74.5355987549,
		Timezone:  "America/Lima",
	},
	"SPMR": {
		ICAO:      "SPMR",
		IATA:      "SMG",
		Name:      "Santa Maria Airport",
		City:      "Santa Maria",
		State:     "Lima-region",
		Country:   "PE",
		Elevation: 769,
		Latitude:  -11.9833335876,
		Longitude: -77,
		Timezone:  "America/Lima",
	},
	"SPMS": {
		ICAO:      "SPMS",
		IATA:      "YMS",
		Name:      "Moises Benzaquen Rengifo Airport",
		City:      "Yurimaguas",
		State:     "Loreto",
		Country:   "PE",
		Elevation: 587,
		Latitude:  -5.8937702179,
		Longitude: -76.1182022095,
		Timezone:  "America/Lima",
	},
	"SPNC": {
		ICAO:      "SPNC",
		IATA:      "HUU",
		Name:      "Alferez Fap David Figueroa Fernandini Airport",
		City:      "Huanuco",
		State:     "Huanuco",
		Country:   "PE",
		Elevation: 6070,
		Latitude:  -9.8788099289,
		Longitude: -76.2048034668,
		Timezone:  "America/Lima",
	},
	"SPNH": {
		ICAO:      "SPNH",
		Name:      "Laguna Choclococha Airport",
		City:      "Choclococha",
		State:     "Huancavelica",
		Country:   "PE",
		Elevation: 14965,
		Latitude:  -13.1656999588,
		Longitude: -75.0719985962,
		Timezone:  "America/Lima",
	},
	"SPNM": {
		ICAO:      "SPNM",
		Name:      "Nuevo Mundo Airport",
		City:      "Nuevo Mundo",
		State:     "Cusco",
		Country:   "PE",
		Elevation: 1090,
		Latitude:  -11.5415000916,
		Longitude: -73.1421966553,
		Timezone:  "America/Lima",
	},
	"SPNO": {
		ICAO:      "SPNO",
		Name:      "Ancon Airport",
		City:      "Ancon",
		State:     "Lima-region",
		Country:   "PE",
		Elevation: 38,
		Latitude:  -11.7946996689,
		Longitude: -77.1809005737,
		Timezone:  "America/Lima",
	},
	"SPNP": {
		ICAO:      "SPNP",
		Name:      "Ventilla Airport",
		City:      "Ventilla",
		State:     "Puno",
		Country:   "PE",
		Elevation: 13123,
		Latitude:  -15.8000001907,
		Longitude: -70.0500030518,
		Timezone:  "America/Lima",
	},
	"SPNR": {
		ICAO:      "SPNR",
		Name:      "Ricran Airport",
		City:      "Ricran",
		State:     "Pasco",
		Country:   "PE",
		Elevation: 13720,
		Latitude:  -10.7779502869,
		Longitude: -76.2513122559,
		Timezone:  "America/Lima",
	},
	"SPNT": {
		ICAO:      "SPNT",
		Name:      "Intuto Airport",
		City:      "Intuto",
		State:     "Loreto",
		Country:   "PE",
		Elevation: 650,
		Latitude:  -3.5829999447,
		Longitude: -74.75,
		Timezone:  "America/Lima",
	},
	"SPNU": {
		ICAO:      "SPNU",
		Name:      "Manu Airport",
		City:      "Manu",
		State:     "Madre-de-Dios",
		Country:   "PE",
		Elevation: 1010,
		Latitude:  -12.2829999924,
		Longitude: -70.8830032349,
		Timezone:  "America/Lima",
	},
	"SPOA": {
		ICAO:      "SPOA",
		IATA:      "SQU",
		Name:      "Saposoa Airport",
		City:      "Plaza Saposoa",
		Country:   "PE",
		Elevation: 982,
		Latitude:  -6.9600300789,
		Longitude: -76.7684020996,
		Timezone:  "America/Lima",
	},
	"SPOL": {
		ICAO:      "SPOL",
		Name:      "Collique Airport",
		City:      "Collique Bajo",
		State:     "Lima-region",
		Country:   "PE",
		Elevation: 410,
		Latitude:  -11.9287004471,
		Longitude: -77.0610961914,
		Timezone:  "America/Lima",
	},
	"SPON": {
		ICAO:      "SPON",
		Name:      "Orellana Airport",
		City:      "Orellana",
		State:     "Loreto",
		Country:   "PE",
		Elevation: 424,
		Latitude:  -6.8967499733,
		Longitude: -75.1521987915,
		Timezone:  "America/Lima",
	},
	"SPOR": {
		ICAO:      "SPOR",
		Name:      "Orcopampa Airport",
		City:      "Orcopampa",
		State:     "Arequipa",
		Country:   "PE",
		Elevation: 12200,
		Latitude:  -15.3170003891,
		Longitude: -72.3499984741,
		Timezone:  "America/Lima",
	},
	"SPOS": {
		ICAO:      "SPOS",
		Name:      "Zorritos Airport",
		City:      "Contralmirante Villar",
		State:     "Tumbes",
		Country:   "PE",
		Elevation: 95,
		Latitude:  -3.6737399101,
		Longitude: -80.6523971558,
		Timezone:  "America/Lima",
	},
	"SPOV": {
		ICAO:      "SPOV",
		Name:      "Shiringayoc/Hacienda Hda Mejia Airport",
		City:      "Leon Velarde",
		State:     "Madre-de-Dios",
		Country:   "PE",
		Elevation: 748,
		Latitude:  -11.8999996185,
		Longitude: -69.1669998169,
		Timezone:  "America/Lima",
	},
	"SPOY": {
		ICAO:      "SPOY",
		Name:      "Atico Airport",
		City:      "Atico",
		State:     "Ayacucho",
		Country:   "PE",
		Elevation: 9,
		Latitude:  -16.2322216034,
		Longitude: -73.611946106,
		Timezone:  "America/Lima",
	},
	"SPPA": {
		ICAO:      "SPPA",
		Name:      "Puerto Ocopa Airport",
		City:      "Puerto Ocopa",
		State:     "Junin",
		Country:   "PE",
		Elevation: 1217,
		Latitude:  -11.1330003738,
		Longitude: -74.3499984741,
		Timezone:  "America/Lima",
	},
	"SPPB": {
		ICAO:      "SPPB",
		Name:      "Tipishsa Airport",
		City:      "Puerto Breu",
		State:     "Ucayali",
		Country:   "PE",
		Elevation: 783,
		Latitude:  -9.5330600739,
		Longitude: -72.7568969727,
		Timezone:  "America/Lima",
	},
	"SPPG": {
		ICAO:      "SPPG",
		Name:      "Paramonga Airport",
		City:      "Paramonga",
		State:     "Lima-region",
		Country:   "PE",
		Elevation: 16,
		Latitude:  -10.6669998169,
		Longitude: -77.8330001831,
		Timezone:  "America/Lima",
	},
	"SPPH": {
		ICAO:      "SPPH",
		Name:      "Pampa Hermosa Airport",
		City:      "Pampa Hermosa",
		State:     "Loreto",
		Country:   "PE",
		Elevation: 510,
		Latitude:  -7.1999998093,
		Longitude: -75.3000030518,
		Timezone:  "America/Lima",
	},
	"SPPL": {
		ICAO:      "SPPL",
		Name:      "Playa Airport",
		City:      "Trismaje",
		State:     "San-Martin",
		Country:   "PE",
		Elevation: 2070,
		Latitude:  -8.5178203583,
		Longitude: -76.4804992676,
		Timezone:  "America/Lima",
	},
	"SPPM": {
		ICAO:      "SPPM",
		Name:      "Pomacocha Airport",
		City:      "Pomacocha",
		State:     "Ayacucho",
		Country:   "PE",
		Elevation: 10524,
		Latitude:  -15.1792001724,
		Longitude: -73.2743988037,
		Timezone:  "America/Lima",
	},
	"SPPN": {
		ICAO:      "SPPN",
		Name:      "Palmas del Espino Airport",
		City:      "Palmas del Espino",
		State:     "San-Martin",
		Country:   "PE",
		Elevation: 1625,
		Latitude:  -8.2925901413,
		Longitude: -76.4395980835,
		Timezone:  "America/Lima",
	},
	"SPPO": {
		ICAO:      "SPPO",
		Name:      "Pozuzo Airport",
		City:      "Pozuzo",
		Country:   "PE",
		Elevation: 2420,
		Latitude:  -10.0700998306,
		Longitude: -75.5500030518,
		Timezone:  "America/Lima",
	},
	"SPPP": {
		ICAO:      "SPPP",
		Name:      "Huanacopampa Airport",
		City:      "Huanacopampa",
		State:     "Apurimac",
		Country:   "PE",
		Elevation: 13130,
		Latitude:  -14.1559000015,
		Longitude: -72.2837982178,
		Timezone:  "America/Lima",
	},
	"SPPY": {
		ICAO:      "SPPY",
		IATA:      "CHH",
		Name:      "Chachapoyas Airport",
		City:      "Chachapoyas",
		State:     "Amazonas",
		Country:   "PE",
		Elevation: 8333,
		Latitude:  -6.2018098831,
		Longitude: -77.8561019897,
		Timezone:  "America/Lima",
	},
	"SPPZ": {
		ICAO:      "SPPZ",
		Name:      "Palcazu Airport",
		City:      "Porvenir",
		State:     "Pasco",
		Country:   "PE",
		Elevation: 932,
		Latitude:  -9.8958330154,
		Longitude: -75.1177749634,
		Timezone:  "America/Lima",
	},
	"SPQI": {
		ICAO:      "SPQI",
		Name:      "Kiteni Airport",
		City:      "La Convencion",
		State:     "Cusco",
		Country:   "PE",
		Elevation: 2540,
		Latitude:  -12.6491003036,
		Longitude: -73.0378036499,
		Timezone:  "America/Lima",
	},
	"SPQJ": {
		ICAO:      "SPQJ",
		Name:      "Jaqui Airport",
		City:      "Jaqui",
		State:     "Arequipa",
		Country:   "PE",
		Latitude:  -15.4666671753,
		Longitude: -74.4333343506,
		Timezone:  "America/Lima",
	},
	"SPQN": {
		ICAO:      "SPQN",
		IATA:      "REQ",
		Name:      "Requena Airport",
		City:      "Requena",
		State:     "Loreto",
		Country:   "PE",
		Elevation: 393,
		Latitude:  -5.0669999123,
		Longitude: -73.8669967651,
		Timezone:  "America/Lima",
	},
	"SPQR": {
		ICAO:      "SPQR",
		Name:      "Quiruvilca Airport",
		City:      "Qiruvilca",
		State:     "La-Libertad",
		Country:   "PE",
		Elevation: 13509,
		Latitude:  -7.9666666985,
		Longitude: -78.1999969482,
		Timezone:  "America/Lima",
	},
	"SPQT": {
		ICAO:      "SPQT",
		IATA:      "IQT",
		Name:      "Coronel FAP Francisco Secada Vignetta International Airport",
		City:      "Iquitos",
		State:     "Loreto",
		Country:   "PE",
		Elevation: 306,
		Latitude:  -3.7847399712,
		Longitude: -73.3087997437,
		Timezone:  "America/Lima",
	},
	"SPQU": {
		ICAO:      "SPQU",
		IATA:      "AQP",
		Name:      "Rodriguez Ballon International Airport",
		City:      "Arequipa",
		State:     "Arequipa",
		Country:   "PE",
		Elevation: 8405,
		Latitude:  -16.3411006927,
		Longitude: -71.5830993652,
		Timezone:  "America/Lima",
	},
	"SPRF": {
		ICAO:      "SPRF",
		Name:      "San Rafael Airport",
		City:      "San Rafael",
		State:     "Puno",
		Country:   "PE",
		Elevation: 14422,
		Latitude:  -14.2670001984,
		Longitude: -70.4670028687,
		Timezone:  "America/Lima",
	},
	"SPRG": {
		ICAO:      "SPRG",
		Name:      "San Regis Airport",
		City:      "Chincha",
		State:     "Ica",
		Country:   "PE",
		Elevation: 294,
		Latitude:  -13.5166664124,
		Longitude: -76.0999984741,
		Timezone:  "America/Lima",
	},
	"SPRL": {
		ICAO:      "SPRL",
		Name:      "Imperial Airport",
		City:      "Imperial",
		State:     "Huancavelica",
		Country:   "PE",
		Latitude:  -12.3166666031,
		Longitude: -75.0666656494,
		Timezone:  "America/Lima",
	},
	"SPRM": {
		ICAO:      "SPRM",
		Name:      "Capitan FAP Leonardo Alvarino Herr Airport",
		City:      "San Ramon",
		State:     "Junin",
		Country:   "PE",
		Elevation: 2600,
		Latitude:  -11.1286001205,
		Longitude: -75.3505020142,
		Timezone:  "America/Lima",
	},
	"SPRU": {
		ICAO:      "SPRU",
		IATA:      "TRU",
		Name:      "Capitan FAP Carlos Martinez De Pinillos International Airport",
		City:      "Trujillo",
		State:     "La-Libertad",
		Country:   "PE",
		Elevation: 106,
		Latitude:  -8.081410408,
		Longitude: -79.1088027954,
		Timezone:  "America/Lima",
	},
	"SPSA": {
		ICAO:      "SPSA",
		Name:      "Casma Airport",
		City:      "Casma",
		State:     "Ancash",
		Country:   "PE",
		Elevation: 248,
		Latitude:  -9.4499998093,
		Longitude: -78.3000030518,
		Timezone:  "America/Lima",
	},
	"SPSE": {
		ICAO:      "SPSE",
		Name:      "Sepahua Airport",
		City:      "Sepahua",
		State:     "Ucayali",
		Country:   "PE",
		Elevation: 886,
		Latitude:  -11.1417999268,
		Longitude: -73.0335006714,
		Timezone:  "America/Lima",
	},
	"SPSF": {
		ICAO:      "SPSF",
		IATA:      "SQD",
		Name:      "San Francisco Airport",
		City:      "San Francisco de Yeso",
		State:     "Amazonas",
		Country:   "PE",
		Elevation: 8202,
		Latitude:  -6.6166801453,
		Longitude: -77.7667007446,
		Timezone:  "America/Lima",
	},
	"SPSI": {
		ICAO:      "SPSI",
		Name:      "Siguas Airport",
		City:      "Siguas",
		State:     "Arequipa",
		Country:   "PE",
		Elevation: 4720,
		Latitude:  -16.3670005798,
		Longitude: -72.1669998169,
		Timezone:  "America/Lima",
	},
	"SPSJ": {
		ICAO:      "SPSJ",
		Name:      "San Jose de Sisa Airport",
		City:      "El Dorado",
		State:     "San-Martin",
		Country:   "PE",
		Elevation: 1200,
		Latitude:  -6.6164898872,
		Longitude: -76.6884002686,
		Timezone:  "America/Lima",
	},
	"SPSL": {
		ICAO:      "SPSL",
		Name:      "Lamas Airport",
		City:      "Lamas",
		State:     "San-Martin",
		Country:   "PE",
		Elevation: 2600,
		Latitude:  -6.4166665077,
		Longitude: -76.5333328247,
		Timezone:  "America/Lima",
	},
	"SPSN": {
		ICAO:      "SPSN",
		Name:      "Shapaja Airport",
		City:      "Shapaja",
		State:     "San-Martin",
		Country:   "PE",
		Elevation: 700,
		Latitude:  -6.5797200203,
		Longitude: -76.262802124,
		Timezone:  "America/Lima",
	},
	"SPSO": {
		ICAO:      "SPSO",
		IATA:      "PIO",
		Name:      "Capitan FAP Renan Elias Olivera International Airport",
		City:      "Pisco",
		State:     "Ica",
		Country:   "PE",
		Elevation: 39,
		Latitude:  -13.7448997498,
		Longitude: -76.2202987671,
		Timezone:  "America/Lima",
	},
	"SPSS": {
		ICAO:      "SPSS",
		Name:      "Masisea Airport",
		City:      "Masisea",
		State:     "Ucayali",
		Country:   "PE",
		Latitude:  -8.6052780151,
		Longitude: -74.304725647,
		Timezone:  "America/Lima",
	},
	"SPST": {
		ICAO:      "SPST",
		IATA:      "TPP",
		Name:      "Cadete FAP Guillermo Del Castillo Paredes Airport",
		City:      "Tarapoto",
		State:     "San-Martin",
		Country:   "PE",
		Elevation: 869,
		Latitude:  -6.5087399483,
		Longitude: -76.3731994629,
		Timezone:  "America/Lima",
	},
	"SPSY": {
		ICAO:      "SPSY",
		IATA:      "SYC",
		Name:      "Shiringayoc Airport",
		City:      "Shiringayoc",
		State:     "Madre-de-Dios",
		Country:   "PE",
		Elevation: 856,
		Latitude:  -11.8832998276,
		Longitude: -69.1999969482,
		Timezone:  "America/Lima",
	},
	"SPTA": {
		ICAO:      "SPTA",
		Name:      "Nauta Airport",
		City:      "Nauta",
		State:     "Loreto",
		Country:   "PE",
		Elevation: 370,
		Latitude:  -4.5332999229,
		Longitude: -73.5500030518,
		Timezone:  "America/Lima",
	},
	"SPTE": {
		ICAO:      "SPTE",
		Name:      "San Francisco Airport",
		City:      "San Francisco",
		State:     "Ayacucho",
		Country:   "PE",
		Elevation: 2499,
		Latitude:  -12.5329999924,
		Longitude: -73.8000030518,
		Timezone:  "America/Lima",
	},
	"SPTI": {
		ICAO:      "SPTI",
		Name:      "Puerto Inca Airport",
		City:      "Puerto Inca",
		State:     "Huanuco",
		Country:   "PE",
		Elevation: 583,
		Latitude:  -9.3830003738,
		Longitude: -74.9670028687,
		Timezone:  "America/Lima",
	},
	"SPTN": {
		ICAO:      "SPTN",
		IATA:      "TCQ",
		Name:      "Coronel FAP Carlos Ciriani Santa Rosa International Airport",
		City:      "Tacna",
		State:     "Tacna",
		Country:   "PE",
		Elevation: 1538,
		Latitude:  -18.0533008575,
		Longitude: -70.2758026123,
		Timezone:  "America/Lima",
	},
	"SPTP": {
		ICAO:      "SPTP",
		Name:      "El Pato Air Base",
		City:      "El Pato",
		State:     "Piura",
		Country:   "PE",
		Elevation: 286,
		Latitude:  -4.5498099327,
		Longitude: -81.2240982056,
		Timezone:  "America/Lima",
	},
	"SPTQ": {
		ICAO:      "SPTQ",
		Name:      "Toquepala Airport",
		City:      "Toquepala",
		State:     "Tacna",
		Country:   "PE",
		Elevation: 8536,
		Latitude:  -17.2994995117,
		Longitude: -70.6528015137,
		Timezone:  "America/Lima",
	},
	"SPTR": {
		ICAO:      "SPTR",
		Name:      "Tournavista Airport",
		City:      "Tournavista",
		State:     "Huanuco",
		Country:   "PE",
		Elevation: 649,
		Latitude:  -8.9169998169,
		Longitude: -74.733001709,
		Timezone:  "America/Lima",
	},
	"SPTU": {
		ICAO:      "SPTU",
		IATA:      "PEM",
		Name:      "Padre Aldamiz International Airport",
		City:      "Puerto Maldonado",
		State:     "Madre-de-Dios",
		Country:   "PE",
		Elevation: 659,
		Latitude:  -12.6135997772,
		Longitude: -69.2285995483,
		Timezone:  "America/Lima",
	},
	"SPTY": {
		ICAO:      "SPTY",
		Name:      "Tintay Airport",
		City:      "Tintay",
		State:     "Apurimac",
		Country:   "PE",
		Elevation: 8809,
		Latitude:  -13.9591665268,
		Longitude: -73.185836792,
		Timezone:  "America/Lima",
	},
	"SPUC": {
		ICAO:      "SPUC",
		Name:      "Huamachuco Airport",
		City:      "Huamachuco",
		State:     "La-Libertad",
		Country:   "PE",
		Elevation: 11224,
		Latitude:  -7.8216400146,
		Longitude: -78.0417022705,
		Timezone:  "America/Lima",
	},
	"SPUR": {
		ICAO:      "SPUR",
		IATA:      "PIU",
		Name:      "Capitan FAP Guillermo Concha Iberico International Airport",
		City:      "Piura",
		State:     "Piura",
		Country:   "PE",
		Elevation: 120,
		Latitude:  -5.2057499886,
		Longitude: -80.6164016724,
		Timezone:  "America/Lima",
	},
	"SPVA": {
		ICAO:      "SPVA",
		Name:      "Hacienda El Valor Airport",
		City:      "El Valor",
		State:     "Amazonas",
		Country:   "PE",
		Latitude:  -5.6624999046,
		Longitude: -78.6447219849,
		Timezone:  "America/Lima",
	},
	"SPVI": {
		ICAO:      "SPVI",
		Name:      "Vicco Airport",
		City:      "Vicco",
		State:     "Pasco",
		Country:   "PE",
		Elevation: 13464,
		Latitude:  -10.8478441238,
		Longitude: -76.2469558716,
		Timezone:  "America/Lima",
	},
	"SPVL": {
		ICAO:      "SPVL",
		Name:      "Caraveli Airport",
		City:      "Caraveli",
		State:     "Arequipa",
		Country:   "PE",
		Elevation: 5675,
		Latitude:  -15.7670001984,
		Longitude: -73.3669967651,
		Timezone:  "America/Lima",
	},
	"SPVR": {
		ICAO:      "SPVR",
		Name:      "Vitor Airport",
		City:      "Vitor",
		State:     "Arequipa",
		Country:   "PE",
		Latitude:  -16.4291992188,
		Longitude: -71.8377990723,
		Timezone:  "America/Lima",
	},
	"SPYA": {
		ICAO:      "SPYA",
		Name:      "Luya Airport",
		City:      "Luya",
		State:     "Lambayeque",
		Country:   "PE",
		Elevation: 298,
		Latitude:  -6.6916999817,
		Longitude: -79.7016983032,
		Timezone:  "America/Lima",
	},
	"SPYL": {
		ICAO:      "SPYL",
		IATA:      "TYL",
		Name:      "Capitan Montes Airport",
		State:     "Piura",
		Country:   "PE",
		Elevation: 282,
		Latitude:  -4.5766401291,
		Longitude: -81.2540969849,
		Timezone:  "America/Lima",
	},
	"SPYO": {
		ICAO:      "SPYO",
		Name:      "Pacasmayo Airport",
		City:      "Pacasmayo",
		State:     "La-Libertad",
		Country:   "PE",
		Elevation: 95,
		Latitude:  -7.4069399834,
		Longitude: -79.568901062,
		Timezone:  "America/Lima",
	},
	"SPYU": {
		ICAO:      "SPYU",
		Name:      "Yauca Airport",
		City:      "Yauca",
		State:     "Arequipa",
		Country:   "PE",
		Elevation: 488,
		Latitude:  -15.6189002991,
		Longitude: -74.5361022949,
		Timezone:  "America/Lima",
	},
	"SPZA": {
		ICAO:      "SPZA",
		IATA:      "NZA",
		Name:      "Maria Reiche Neuman Airport",
		State:     "Ica",
		Country:   "PE",
		Elevation: 1860,
		Latitude:  -14.8540000916,
		Longitude: -74.9615020752,
		Timezone:  "America/Lima",
	},
	"SPZH": {
		ICAO:      "SPZH",
		Name:      "Pachiza Airport",
		City:      "Pachiza",
		State:     "San-Martin",
		Country:   "PE",
		Elevation: 1102,
		Latitude:  -7.2992801666,
		Longitude: -76.774597168,
		Timezone:  "America/Lima",
	},
	"SPZO": {
		ICAO:      "SPZO",
		IATA:      "CUZ",
		Name:      "Alejandro Velasco Astete International Airport",
		City:      "Cusco",
		State:     "Cusco",
		Country:   "PE",
		Elevation: 10860,
		Latitude:  -13.5356998444,
		Longitude: -71.9387969971,
		Timezone:  "America/Lima",
	},
	"SPZT": {
		ICAO:      "SPZT",
		Name:      "Chazuta Airport",
		City:      "Chazuta",
		State:     "San-Martin",
		Country:   "PE",
		Elevation: 757,
		Latitude:  -6.5741667747,
		Longitude: -76.1366653442,
		Timezone:  "America/Lima",
	},
	"SSAB": {
		ICAO:      "SSAB",
		Name:      "Ibaiti Airport",
		City:      "Ibaiti",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1968,
		Latitude:  -23.7621994019,
		Longitude: -50.2630004883,
		Timezone:  "America/Sao_Paulo",
	},
	"SSAD": {
		ICAO:      "SSAD",
		Name:      "Fazenda Ribeirao Airport",
		City:      "Chapadao Do Sul",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 2736,
		Latitude:  -18.763999939,
		Longitude: -52.9175987244,
		Timezone:  "America/Campo_Grande",
	},
	"SSAE": {
		ICAO:      "SSAE",
		Name:      "Arroio Grande Airport",
		City:      "Arroio Grande",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 161,
		Latitude:  -32.2241668701,
		Longitude: -53.0672225952,
		Timezone:  "America/Sao_Paulo",
	},
	"SSAF": {
		ICAO:      "SSAF",
		Name:      "Aeroclube de Foz do Iguacu Airport",
		City:      "Santa Terezinha De Itaipu",
		State:     "Parana",
		Country:   "BR",
		Elevation: 840,
		Latitude:  -25.3694438934,
		Longitude: -54.4705543518,
		Timezone:  "America/Sao_Paulo",
	},
	"SSAG": {
		ICAO:      "SSAG",
		Name:      "Fazenda Agropecuaria Lobo Ltda Airport",
		City:      "Agua Clara",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1410,
		Latitude:  -19.7166671753,
		Longitude: -52.7255554199,
		Timezone:  "America/Campo_Grande",
	},
	"SSAJ": {
		ICAO:      "SSAJ",
		Name:      "Fazenda Esperanca Airport",
		City:      "Bataipora",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1109,
		Latitude:  -22.2755565643,
		Longitude: -53.3119430542,
		Timezone:  "America/Campo_Grande",
	},
	"SSAK": {
		ICAO:      "SSAK",
		Name:      "Aeroclube Airport",
		City:      "Cruz Alta",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 1542,
		Latitude:  -28.6577777863,
		Longitude: -53.6105575562,
		Timezone:  "America/Sao_Paulo",
	},
	"SSAM": {
		ICAO:      "SSAM",
		Name:      "Amambai Airport",
		City:      "Amambai",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1542,
		Latitude:  -23.1427783966,
		Longitude: -55.2077789307,
		Timezone:  "America/Campo_Grande",
	},
	"SSAN": {
		ICAO:      "SSAN",
		Name:      "Andira Airport",
		City:      "Andira",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1608,
		Latitude:  -23.027299881,
		Longitude: -50.2275009155,
		Timezone:  "America/Sao_Paulo",
	},
	"SSAP": {
		ICAO:      "SSAP",
		IATA:      "APU",
		Name:      "Apucarana Airport",
		City:      "Apucarana",
		State:     "Parana",
		Country:   "BR",
		Elevation: 2641,
		Latitude:  -23.609500885,
		Longitude: -51.3844985962,
		Timezone:  "America/Sao_Paulo",
	},
	"SSAQ": {
		ICAO:      "SSAQ",
		IATA:      "0",
		Name:      "Aeroclube Airport",
		City:      "Passo Fundo",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 1969,
		Latitude:  -28.2506008148,
		Longitude: -52.5167007446,
		Timezone:  "America/Sao_Paulo",
	},
	"SSAR": {
		ICAO:      "SSAR",
		Name:      "Fazenda Jacare Airport",
		City:      "Campo Grande",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1214,
		Latitude:  -21.5030555725,
		Longitude: -53.9674987793,
		Timezone:  "America/Campo_Grande",
	},
	"SSAU": {
		ICAO:      "SSAU",
		Name:      "Centeno Airport",
		City:      "Tapes",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 14,
		Latitude:  -30.7474994659,
		Longitude: -51.4288902283,
		Timezone:  "America/Sao_Paulo",
	},
	"SSAV": {
		ICAO:      "SSAV",
		Name:      "Fazenda Amonguija Airport",
		City:      "Porto Murtinho",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 400,
		Latitude:  -21.6908321381,
		Longitude: -57.4741668701,
		Timezone:  "America/Campo_Grande",
	},
	"SSAW": {
		ICAO:      "SSAW",
		Name:      "Fazenda Japecanga Airport",
		City:      "Ribas Do Rio Pardo",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1493,
		Latitude:  -20.8052787781,
		Longitude: -54.0780563354,
		Timezone:  "America/Campo_Grande",
	},
	"SSAY": {
		ICAO:      "SSAY",
		Name:      "Sitio Pouso do Aviador Airport",
		City:      "Campo Grande",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1946,
		Latitude:  -20.4885005951,
		Longitude: -54.4828987122,
		Timezone:  "America/Campo_Grande",
	},
	"SSBA": {
		ICAO:      "SSBA",
		Name:      "Fazenda Bela Vista Airport",
		City:      "Camapua",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 2017,
		Latitude:  -19.0319442749,
		Longitude: -53.6711120605,
		Timezone:  "America/Campo_Grande",
	},
	"SSBB": {
		ICAO:      "SSBB",
		Name:      "Estrela Dalva Airport",
		City:      "Bodoquena",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1207,
		Latitude:  -20.6749992371,
		Longitude: -56.629699707,
		Timezone:  "America/Campo_Grande",
	},
	"SSBC": {
		ICAO:      "SSBC",
		Name:      "Malibu Confinamento - MC Airport",
		City:      "Castilho",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1115,
		Latitude:  -20.7261104584,
		Longitude: -51.5886116028,
		Timezone:  "America/Sao_Paulo",
	},
	"SSBD": {
		ICAO:      "SSBD",
		Name:      "Sobradinho Airport",
		City:      "Sobradinho",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 1444,
		Latitude:  -29.3963890076,
		Longitude: -52.9916687012,
		Timezone:  "America/Sao_Paulo",
	},
	"SSBE": {
		ICAO:      "SSBE",
		Name:      "Camapua Airport",
		City:      "Camapua",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1848,
		Latitude:  -19.598400116,
		Longitude: -54.0210990906,
		Timezone:  "America/Campo_Grande",
	},
	"SSBF": {
		ICAO:      "SSBF",
		Name:      "Fazenda JL Airport",
		City:      "Aparecida Do Taboado",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1400,
		Latitude:  -20.0044441223,
		Longitude: -51.3866653442,
		Timezone:  "America/Campo_Grande",
	},
	"SSBG": {
		ICAO:      "SSBG",
		IATA:      "BGV",
		Name:      "Aeroclube de Bento Goncalves Airport",
		City:      "Bento Goncalves",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 2209,
		Latitude:  -29.1483325958,
		Longitude: -51.5363883972,
		Timezone:  "America/Sao_Paulo",
	},
	"SSBH": {
		ICAO:      "SSBH",
		Name:      "Fazenda Baia do Lara Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 453,
		Latitude:  -17.5677776337,
		Longitude: -55.2883338928,
		Timezone:  "America/Campo_Grande",
	},
	"SSBJ": {
		ICAO:      "SSBJ",
		Name:      "Bom Jesus Airport",
		City:      "Bom Jesus",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 3514,
		Latitude:  -28.61277771,
		Longitude: -50.4494438171,
		Timezone:  "America/Sao_Paulo",
	},
	"SSBK": {
		ICAO:      "SSBK",
		Name:      "Fazenda Araruna Airport",
		City:      "Sonora",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1886,
		Latitude:  -17.5827789307,
		Longitude: -54.6408348083,
		Timezone:  "America/Campo_Grande",
	},
	"SSBL": {
		ICAO:      "SSBL",
		IATA:      "BNU",
		Name:      "Blumenau Airport",
		City:      "Blumenau",
		State:     "Santa-Catarina",
		Country:   "BR",
		Elevation: 60,
		Latitude:  -26.8306007385,
		Longitude: -49.0903015137,
		Timezone:  "America/Sao_Paulo",
	},
	"SSBM": {
		ICAO:      "SSBM",
		Name:      "Fazenda Lageado Airport",
		City:      "Dois Irmaos Do Buriti",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 932,
		Latitude:  -20.2975006104,
		Longitude: -55.4258346558,
		Timezone:  "America/Campo_Grande",
	},
	"SSBN": {
		ICAO:      "SSBN",
		Name:      "Belem Novo Airport",
		City:      "Porto Alegre",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 20,
		Latitude:  -30.1877784729,
		Longitude: -51.1827774048,
		Timezone:  "America/Sao_Paulo",
	},
	"SSBO": {
		ICAO:      "SSBO",
		Name:      "Palmares Airport",
		City:      "Bossoroca",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 787,
		Latitude:  -28.7222213745,
		Longitude: -54.8908348083,
		Timezone:  "America/Sao_Paulo",
	},
	"SSBQ": {
		ICAO:      "SSBQ",
		Name:      "Frigorifico Marfrig Airport",
		City:      "Bataguassu",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1230,
		Latitude:  -21.7466678619,
		Longitude: -52.4783325195,
		Timezone:  "America/Campo_Grande",
	},
	"SSBR": {
		ICAO:      "SSBR",
		IATA:      "0",
		Name:      "Bandeirantes Airport",
		City:      "Bandeirantes",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1319,
		Latitude:  -23.0727996826,
		Longitude: -50.4230995178,
		Timezone:  "America/Sao_Paulo",
	},
	"SSBS": {
		ICAO:      "SSBS",
		Name:      "Fazenda Boa Esperanca Airport",
		City:      "Brasilandia",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1155,
		Latitude:  -21.2491664886,
		Longitude: -52.2894439697,
		Timezone:  "America/Campo_Grande",
	},
	"SSBT": {
		ICAO:      "SSBT",
		Name:      "Fazenda Santa Ilidia Airport",
		City:      "Bataipora",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1033,
		Latitude:  -22.324300766,
		Longitude: -53.0950012207,
		Timezone:  "America/Campo_Grande",
	},
	"SSBU": {
		ICAO:      "SSBU",
		Name:      "Fazenda Baguacu - Miragem Airport",
		City:      "Porto Murtinho",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1191,
		Latitude:  -21.4563884735,
		Longitude: -57.1402778625,
		Timezone:  "America/Campo_Grande",
	},
	"SSBV": {
		ICAO:      "SSBV",
		Name:      "Bela Vista Airport",
		City:      "Bela Vista",
		Country:   "BR",
		Elevation: 770,
		Latitude:  -22.0816001892,
		Longitude: -56.5386009216,
		Timezone:  "America/Campo_Grande",
	},
	"SSBW": {
		ICAO:      "SSBW",
		Name:      "Fazenda Boa Sorte Airport",
		City:      "Sidrolandia",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1083,
		Latitude:  -20.9130554199,
		Longitude: -55.0136108398,
		Timezone:  "America/Campo_Grande",
	},
	"SSBX": {
		ICAO:      "SSBX",
		Name:      "Fazenda Lageado Airport",
		City:      "Coxim",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 696,
		Latitude:  -18.4416675568,
		Longitude: -54.7999992371,
		Timezone:  "America/Campo_Grande",
	},
	"SSBY": {
		ICAO:      "SSBY",
		Name:      "Orlando Chesini Ometto Airport",
		City:      "Miranda",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 443,
		Latitude:  -20.0995006561,
		Longitude: -56.7938995361,
		Timezone:  "America/Campo_Grande",
	},
	"SSBZ": {
		ICAO:      "SSBZ",
		Name:      "Aeroclube Airport",
		City:      "Sapiranga",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 56,
		Latitude:  -29.6761112213,
		Longitude: -50.981388092,
		Timezone:  "America/Sao_Paulo",
	},
	"SSCA": {
		ICAO:      "SSCA",
		Name:      "Fazenda Campo Bom Airport",
		City:      "Chapadao Do Sul",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2657,
		Latitude:  -18.7394447327,
		Longitude: -52.6199989319,
		Timezone:  "America/Campo_Grande",
	},
	"SSCB": {
		ICAO:      "SSCB",
		Name:      "Fazenda Santa Maria Airport",
		City:      "Mundo Novo",
		State:     "Goias",
		Country:   "BR",
		Elevation: 978,
		Latitude:  -13.7758331299,
		Longitude: -49.9469451904,
		Timezone:  "America/Sao_Paulo",
	},
	"SSCD": {
		ICAO:      "SSCD",
		Name:      "Chapadao do Sul Airport",
		City:      "Chapadao Do Sul",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 2644,
		Latitude:  -18.8377780914,
		Longitude: -52.4866676331,
		Timezone:  "America/Campo_Grande",
	},
	"SSCE": {
		ICAO:      "SSCE",
		Name:      "Aeroclube Airport",
		City:      "Bage",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 722,
		Latitude:  -31.3277778625,
		Longitude: -54.1347236633,
		Timezone:  "America/Sao_Paulo",
	},
	"SSCF": {
		ICAO:      "SSCF",
		Name:      "Max Fontoura Airport",
		City:      "Campo Largo",
		State:     "Parana",
		Country:   "BR",
		Elevation: 3140,
		Latitude:  -25.4322223663,
		Longitude: -49.5299987793,
		Timezone:  "America/Sao_Paulo",
	},
	"SSCH": {
		ICAO:      "SSCH",
		Name:      "Fazenda Cachoeirao Airport",
		City:      "Nova Andradina",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1053,
		Latitude:  -21.5972,
		Longitude: -53.225,
		Timezone:  "America/Campo_Grande",
	},
	"SSCI": {
		ICAO:      "SSCI",
		Name:      "Coxim Airport",
		City:      "Coxim",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 957,
		Latitude:  -18.4776992798,
		Longitude: -54.7146987915,
		Timezone:  "America/Campo_Grande",
	},
	"SSCJ": {
		ICAO:      "SSCJ",
		Name:      "Fazenda Celeiro Airport",
		City:      "Rio Brilhante",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1070,
		Latitude:  -21.5416679382,
		Longitude: -54.6616668701,
		Timezone:  "America/Campo_Grande",
	},
	"SSCK": {
		ICAO:      "SSCK",
		IATA:      "CCI",
		Name:      "Concordia Airport",
		City:      "Concordia",
		State:     "Santa-Catarina",
		Country:   "BR",
		Elevation: 2461,
		Latitude:  -27.1805992126,
		Longitude: -52.0527000427,
		Timezone:  "America/Sao_Paulo",
	},
	"SSCL": {
		ICAO:      "SSCL",
		IATA:      "CSS",
		Name:      "Cassilandia Airport",
		City:      "Cassilandia",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1568,
		Latitude:  -19.1463890076,
		Longitude: -51.6852798462,
		Timezone:  "America/Campo_Grande",
	},
	"SSCM": {
		ICAO:      "SSCM",
		Name:      "Fazenda Cacadinha Airport",
		City:      "Rio Brilhante",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1312,
		Latitude:  -21.6238899231,
		Longitude: -54.8108329773,
		Timezone:  "America/Campo_Grande",
	},
	"SSCN": {
		ICAO:      "SSCN",
		IATA:      "QCN",
		Name:      "Canela Airport",
		City:      "Canela",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 2723,
		Latitude:  -29.3701992035,
		Longitude: -50.8320007324,
		Timezone:  "America/Sao_Paulo",
	},
	"SSCO": {
		ICAO:      "SSCO",
		Name:      "Comandante Marilda Zaiden de Mesquita Airport",
		City:      "Capao Do Leao",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 33,
		Latitude:  -31.7741661072,
		Longitude: -52.4513893127,
		Timezone:  "America/Sao_Paulo",
	},
	"SSCP": {
		ICAO:      "SSCP",
		IATA:      "CKO",
		Name:      "Cornelio Procopio Airport",
		City:      "Cornelio Procopio",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1854,
		Latitude:  -23.1525001526,
		Longitude: -50.6025009155,
		Timezone:  "America/Sao_Paulo",
	},
	"SSCQ": {
		ICAO:      "SSCQ",
		Name:      "Salca Airport",
		City:      "Cacequi",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 285,
		Latitude:  -29.8997211456,
		Longitude: -54.9291687012,
		Timezone:  "America/Sao_Paulo",
	},
	"SSCR": {
		ICAO:      "SSCR",
		Name:      "Marechal Candido Rondon Airport",
		City:      "Marechal Candido Rondon",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1411,
		Latitude:  -24.5126991272,
		Longitude: -54.0546989441,
		Timezone:  "America/Sao_Paulo",
	},
	"SSCS": {
		ICAO:      "SSCS",
		Name:      "Estancia Ema Airport",
		City:      "Campo Grande",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1968,
		Latitude:  -20.6197223663,
		Longitude: -54.7219429016,
		Timezone:  "America/Campo_Grande",
	},
	"SSCT": {
		ICAO:      "SSCT",
		IATA:      "GGH",
		Name:      "Cianorte Airport",
		City:      "Cianorte",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1837,
		Latitude:  -23.6914997101,
		Longitude: -52.6418991089,
		Timezone:  "America/Sao_Paulo",
	},
	"SSCU": {
		ICAO:      "SSCU",
		Name:      "Fazenda Santa Marina Airport",
		City:      "Santa Maria Das Barreiras",
		State:     "Para",
		Country:   "BR",
		Elevation: 1007,
		Latitude:  -8.8680562973,
		Longitude: -50.4794425964,
		Timezone:  "America/Belem",
	},
	"SSCV": {
		ICAO:      "SSCV",
		Name:      "Capivari Airport",
		City:      "Capivari Do Sul",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 14,
		Latitude:  -30.134721756,
		Longitude: -50.5086097717,
		Timezone:  "America/Sao_Paulo",
	},
	"SSCW": {
		ICAO:      "SSCW",
		Name:      "Fazenda Sao Sebastiao Airport",
		City:      "Corumba",
		Country:   "BR",
		Elevation: 443,
		Latitude:  -18.0397224426,
		Longitude: -56.0055541992,
		Timezone:  "America/Campo_Grande",
	},
	"SSCX": {
		ICAO:      "SSCX",
		Name:      "Fazenda Coxilha Airport",
		City:      "Passo Fundo",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 2133,
		Latitude:  -28.1172218323,
		Longitude: -52.3172225952,
		Timezone:  "America/Sao_Paulo",
	},
	"SSDB": {
		ICAO:      "SSDB",
		Name:      "Fazenda Kurupay Airport",
		City:      "Angelica",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1280,
		Latitude:  -22.0408325195,
		Longitude: -53.7311096191,
		Timezone:  "America/Campo_Grande",
	},
	"SSDC": {
		ICAO:      "SSDC",
		Name:      "Dionisio Cerqueira Airport",
		City:      "Dionisio Cerqueira",
		State:     "Misiones",
		Country:   "BR",
		Elevation: 2723,
		Latitude:  -26.295834,
		Longitude: -53.630654,
		Timezone:  "America/Argentina/Cordoba",
	},
	"SSDE": {
		ICAO:      "SSDE",
		Name:      "Fazenda Barra Dourada Airport",
		City:      "Dourados",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1083,
		Latitude:  -21.9880561829,
		Longitude: -54.238609314,
		Timezone:  "America/Campo_Grande",
	},
	"SSDF": {
		ICAO:      "SSDF",
		Name:      "Fazenda Sao Jose Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 312,
		Latitude:  -20.0994434357,
		Longitude: -57.3461112976,
		Timezone:  "America/Campo_Grande",
	},
	"SSDG": {
		ICAO:      "SSDG",
		Name:      "Fazenda Nazare Airport",
		City:      "Santa Filomena",
		State:     "Piaui",
		Country:   "BR",
		Elevation: 1962,
		Latitude:  -9.3544435501,
		Longitude: -45.5613899231,
		Timezone:  "America/Fortaleza",
	},
	"SSDH": {
		ICAO:      "SSDH",
		Name:      "Fazenda Caranday Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 541,
		Latitude:  -20.2055568695,
		Longitude: -57.063331604,
		Timezone:  "America/Campo_Grande",
	},
	"SSDI": {
		ICAO:      "SSDI",
		Name:      "Fazenda Bacatuba Airport",
		City:      "Buriti Bravo",
		State:     "Maranhao",
		Country:   "BR",
		Elevation: 584,
		Latitude:  -5.6583328247,
		Longitude: -43.7055549622,
		Timezone:  "America/Fortaleza",
	},
	"SSDJ": {
		ICAO:      "SSDJ",
		Name:      "Fazenda Bonanza Airport",
		City:      "Camapua",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1252,
		Latitude:  -19.0508327484,
		Longitude: -54.0649986267,
		Timezone:  "America/Campo_Grande",
	},
	"SSDK": {
		ICAO:      "SSDK",
		Name:      "Fazenda Vista Alegre Airport",
		City:      "Novo Horizonte Do Sul",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1099,
		Latitude:  -22.5977783203,
		Longitude: -53.6908340454,
		Timezone:  "America/Campo_Grande",
	},
	"SSDL": {
		ICAO:      "SSDL",
		Name:      "Fazenda Ribeirao Airport",
		City:      "Ribeirao Goncalves",
		State:     "Piaui",
		Country:   "BR",
		Elevation: 1588,
		Latitude:  -8.0063886643,
		Longitude: -45.338054657,
		Timezone:  "America/Fortaleza",
	},
	"SSDM": {
		ICAO:      "SSDM",
		Name:      "Aeroclube de Santana do Livramento Airport",
		City:      "Santana Do Livramento",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 1073,
		Latitude:  -30.822221756,
		Longitude: -55.5763893127,
		Timezone:  "America/Montevideo",
	},
	"SSDN": {
		ICAO:      "SSDN",
		Name:      "Fazenda Galileia Airport",
		City:      "Baixa Grande Do Ribeiro",
		State:     "Piaui",
		Country:   "BR",
		Elevation: 1932,
		Latitude:  -9.1044,
		Longitude: -45.4166679382,
		Timezone:  "America/Fortaleza",
	},
	"SSDO": {
		ICAO:      "SSDO",
		IATA:      "DOU",
		Name:      "Dourados Airport",
		City:      "Dourados",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1503,
		Latitude:  -22.2019004822,
		Longitude: -54.9266014099,
		Timezone:  "America/Campo_Grande",
	},
	"SSDP": {
		ICAO:      "SSDP",
		Name:      "Fazenda Fortaleza do Guapore Airport",
		City:      "Vila Bela Da Santissima Trindade",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 738,
		Latitude:  -14.459444046,
		Longitude: -60.1780548096,
		Timezone:  "America/Cuiaba",
	},
	"SSDQ": {
		ICAO:      "SSDQ",
		Name:      "Fazenda Marema Airport",
		City:      "Miranda",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 446,
		Latitude:  -20.1827774048,
		Longitude: -56.6358337402,
		Timezone:  "America/Campo_Grande",
	},
	"SSDS": {
		ICAO:      "SSDS",
		Name:      "Fazenda Bandeirantes Airport",
		City:      "Nova Crixas",
		State:     "Goias",
		Country:   "BR",
		Elevation: 935,
		Latitude:  -13.7072219849,
		Longitude: -50.6477775574,
		Timezone:  "America/Sao_Paulo",
	},
	"SSDT": {
		ICAO:      "SSDT",
		Name:      "Fazenda Pau D`Arco Airport",
		City:      "Redencao",
		State:     "Para",
		Country:   "BR",
		Elevation: 794,
		Latitude:  -7.943611145,
		Longitude: -50.1763877869,
		Timezone:  "America/Belem",
	},
	"SSDU": {
		ICAO:      "SSDU",
		Name:      "Fazenda Modelo Airport",
		City:      "Pedro Gomes",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1309,
		Latitude:  -17.9916667938,
		Longitude: -54.2236099243,
		Timezone:  "America/Campo_Grande",
	},
	"SSDV": {
		ICAO:      "SSDV",
		Name:      "Fazenda Santa Maria Airport",
		City:      "Coxim",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 508,
		Latitude:  -18.2161102295,
		Longitude: -55.4652786255,
		Timezone:  "America/Campo_Grande",
	},
	"SSDW": {
		ICAO:      "SSDW",
		Name:      "Fazenda Potrich Airport",
		City:      "Sorriso",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1197,
		Latitude:  -12.9244441986,
		Longitude: -55.5191688538,
		Timezone:  "America/Cuiaba",
	},
	"SSDX": {
		ICAO:      "SSDX",
		Name:      "Fazenda da Coxilha Airport",
		City:      "Camaqua",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 33,
		Latitude:  -31,
		Longitude: -51.7005577087,
		Timezone:  "America/Sao_Paulo",
	},
	"SSDY": {
		ICAO:      "SSDY",
		Name:      "Fazenda Duas Irmas Airport",
		City:      "Guiratinga",
		Country:   "BR",
		Elevation: 1410,
		Latitude:  -16.15666667,
		Longitude: -53.00972222,
		Timezone:  "America/Cuiaba",
	},
	"SSDZ": {
		ICAO:      "SSDZ",
		Name:      "Fazenda Divisao Airport",
		City:      "Amambai",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1312,
		Latitude:  -23.1338882446,
		Longitude: -54.6338882446,
		Timezone:  "America/Campo_Grande",
	},
	"SSED": {
		ICAO:      "SSED",
		Name:      "Fazenda Escalada Airport",
		City:      "Pedro Gomes",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1132,
		Latitude:  -17.9566669464,
		Longitude: -54.4191665649,
		Timezone:  "America/Campo_Grande",
	},
	"SSEE": {
		ICAO:      "SSEE",
		Name:      "Regional do Vale do Taquari Airport",
		City:      "Estrela",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 105,
		Latitude:  -29.453332901,
		Longitude: -51.9341659546,
		Timezone:  "America/Sao_Paulo",
	},
	"SSEF": {
		ICAO:      "SSEF",
		Name:      "Fazenda Passaredo Airport",
		City:      "Cajuru",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 4632,
		Latitude:  -21.326944,
		Longitude: -47.231111,
		Timezone:  "America/Sao_Paulo",
	},
	"SSEG": {
		ICAO:      "SSEG",
		Name:      "Estancia da Gruta Airport",
		City:      "Pelotas",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 33,
		Latitude:  -31.9424991608,
		Longitude: -52.4794425964,
		Timezone:  "America/Sao_Paulo",
	},
	"SSEI": {
		ICAO:      "SSEI",
		Name:      "Fazenda Santo Andre Airport",
		City:      "Miranda",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 489,
		Latitude:  -20.3386116028,
		Longitude: -56.6983337402,
		Timezone:  "America/Campo_Grande",
	},
	"SSEK": {
		ICAO:      "SSEK",
		Name:      "Fazenda Lontra Airport",
		City:      "Ribas Do Rio Pardo",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1299,
		Latitude:  -20.0177783966,
		Longitude: -53.2524986267,
		Timezone:  "America/Campo_Grande",
	},
	"SSEL": {
		ICAO:      "SSEL",
		Name:      "Fazenda Jatiuca Airport",
		City:      "Agua Clara",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1601,
		Latitude:  -20.1697216034,
		Longitude: -52.9330558777,
		Timezone:  "America/Campo_Grande",
	},
	"SSEO": {
		ICAO:      "SSEO",
		Name:      "Fazenda Mimoso Airport",
		City:      "Ribas Do Rio Pardo",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1299,
		Latitude:  -20.7344436646,
		Longitude: -53.5766677856,
		Timezone:  "America/Campo_Grande",
	},
	"SSEP": {
		ICAO:      "SSEP",
		IATA:      "0",
		Name:      "Sao Sepe Airport",
		City:      "Sao Sepe",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 502,
		Latitude:  -30.1821994781,
		Longitude: -53.5793991089,
		Timezone:  "America/Sao_Paulo",
	},
	"SSEQ": {
		ICAO:      "SSEQ",
		Name:      "Fazenda Rio Brilhante Airport",
		City:      "Angelica",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1099,
		Latitude:  -21.9233322144,
		Longitude: -53.9399986267,
		Timezone:  "America/Campo_Grande",
	},
	"SSER": {
		ICAO:      "SSER",
		IATA:      "ERM",
		Name:      "Erechim Airport",
		City:      "Erechim",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 2498,
		Latitude:  -27.6618995667,
		Longitude: -52.2682991028,
		Timezone:  "America/Sao_Paulo",
	},
	"SSES": {
		ICAO:      "SSES",
		Name:      "Encruzilhada do Sul Airport",
		City:      "Encruzilhada Do Sul",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 1460,
		Latitude:  -30.558889389,
		Longitude: -52.5825004578,
		Timezone:  "America/Sao_Paulo",
	},
	"SSET": {
		ICAO:      "SSET",
		Name:      "Tambau Airport",
		City:      "Tambau",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2451,
		Latitude:  -21.69722222,
		Longitude: -47.31555556,
		Timezone:  "America/Sao_Paulo",
	},
	"SSEU": {
		ICAO:      "SSEU",
		Name:      "Monte Carmelo Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 915,
		Latitude:  -7.2830557823,
		Longitude: -56.5769424438,
		Timezone:  "America/Santarem",
	},
	"SSEV": {
		ICAO:      "SSEV",
		Name:      "Sao Camilo Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 289,
		Latitude:  -18.0872211456,
		Longitude: -57.1313896179,
		Timezone:  "America/Campo_Grande",
	},
	"SSEW": {
		ICAO:      "SSEW",
		Name:      "Sitio Limoeiro Airport",
		City:      "Pardinho",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 3084,
		Latitude:  -23.15,
		Longitude: -48.31,
		Timezone:  "America/Sao_Paulo",
	},
	"SSEX": {
		ICAO:      "SSEX",
		Name:      "Estancia Caiman Airport",
		City:      "Miranda",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 361,
		Latitude:  -19.9661102295,
		Longitude: -56.3108329773,
		Timezone:  "America/Campo_Grande",
	},
	"SSEY": {
		ICAO:      "SSEY",
		Name:      "Estancia Itapororo Airport",
		City:      "Alegrete",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 197,
		Latitude:  -29.433889389,
		Longitude: -56.3005561829,
		Timezone:  "America/Sao_Paulo",
	},
	"SSEZ": {
		ICAO:      "SSEZ",
		Name:      "Espumoso Airport",
		City:      "Espumoso",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 1198,
		Latitude:  -28.7374992371,
		Longitude: -52.856388092,
		Timezone:  "America/Sao_Paulo",
	},
	"SSFA": {
		ICAO:      "SSFA",
		Name:      "Foz do Areia Airport",
		City:      "Pinhao",
		State:     "Parana",
		Country:   "BR",
		Elevation: 3645,
		Latitude:  -25.9815006256,
		Longitude: -51.6407012939,
		Timezone:  "America/Sao_Paulo",
	},
	"SSFB": {
		ICAO:      "SSFB",
		IATA:      "FBE",
		Name:      "Francisco Beltrao Airport",
		City:      "Francisco Beltrao",
		State:     "Parana",
		Country:   "BR",
		Elevation: 2100,
		Latitude:  -26.0592002869,
		Longitude: -53.0634994507,
		Timezone:  "America/Sao_Paulo",
	},
	"SSFD": {
		ICAO:      "SSFD",
		Name:      "Fazenda Acaraja Airport",
		City:      "Amambai",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1224,
		Latitude:  -23.0011100769,
		Longitude: -54.6994438171,
		Timezone:  "America/Campo_Grande",
	},
	"SSFE": {
		ICAO:      "SSFE",
		Name:      "Estancia Hercules Airport",
		City:      "Foz Do Iguacu",
		State:     "Alto-Parana",
		Country:   "BR",
		Elevation: 558,
		Latitude:  -25.4606990814,
		Longitude: -54.5988006592,
		Timezone:  "America/Asuncion",
	},
	"SSFF": {
		ICAO:      "SSFF",
		Name:      "Fazenda Sao Francisco Airport",
		City:      "Nova Alvorada Do Sul",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1204,
		Latitude:  -21.1786117554,
		Longitude: -54.293056488,
		Timezone:  "America/Campo_Grande",
	},
	"SSFG": {
		ICAO:      "SSFG",
		Name:      "Fazenda Guanabara Airport",
		City:      "Paranapoema",
		State:     "Parana",
		Country:   "BR",
		Elevation: 925,
		Latitude:  -22.6583328247,
		Longitude: -52.1305541992,
		Timezone:  "America/Sao_Paulo",
	},
	"SSFH": {
		ICAO:      "SSFH",
		Name:      "Vale dos Dinassauros Airport",
		City:      "Uberaba",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2438,
		Latitude:  -19.782222,
		Longitude: -47.811944,
		Timezone:  "America/Sao_Paulo",
	},
	"SSFI": {
		ICAO:      "SSFI",
		Name:      "Fazenda Itamarati Airport",
		City:      "Ponta Pora",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1837,
		Latitude:  -22.1881999969,
		Longitude: -55.5778007507,
		Timezone:  "America/Campo_Grande",
	},
	"SSFJ": {
		ICAO:      "SSFJ",
		Name:      "Fazenda Retiro da Cachoeira Airport",
		City:      "Campo Grande",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1435,
		Latitude:  -20.8077774048,
		Longitude: -54.619720459,
		Timezone:  "America/Campo_Grande",
	},
	"SSFK": {
		ICAO:      "SSFK",
		Name:      "Forte Coimbra Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 279,
		Latitude:  -19.9061107635,
		Longitude: -57.7794456482,
		Timezone:  "America/Campo_Grande",
	},
	"SSFL": {
		ICAO:      "SSFL",
		Name:      "Joao Pereira dos Santos Filho Airport",
		City:      "Fronteiras",
		State:     "Piaui",
		Country:   "BR",
		Elevation: 1466,
		Latitude:  -7.0999999046,
		Longitude: -40.6269454956,
		Timezone:  "America/Fortaleza",
	},
	"SSFM": {
		ICAO:      "SSFM",
		Name:      "Fazenda Marimbondo Airport",
		City:      "Conselheiro Mairinck",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1526,
		Latitude:  -23.5369434357,
		Longitude: -50.0980567932,
		Timezone:  "America/Sao_Paulo",
	},
	"SSFN": {
		ICAO:      "SSFN",
		Name:      "Fazenda Novilho Airport",
		City:      "Caarapo",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1706,
		Latitude:  -22.6191673279,
		Longitude: -54.9258346558,
		Timezone:  "America/Campo_Grande",
	},
	"SSFO": {
		ICAO:      "SSFO",
		Name:      "Fazenda Novo Horizonte Airport",
		City:      "Miranda",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 354,
		Latitude:  -20.0397224426,
		Longitude: -56.3202781677,
		Timezone:  "America/Campo_Grande",
	},
	"SSFP": {
		ICAO:      "SSFP",
		Name:      "Fazenda Sao Paulo Airport",
		City:      "Tres Lagoas",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1145,
		Latitude:  -20.8641662598,
		Longitude: -52.1747207642,
		Timezone:  "America/Campo_Grande",
	},
	"SSFQ": {
		ICAO:      "SSFQ",
		Name:      "Fazenda Barra Nova Airport",
		City:      "Rio Brilhante",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1608,
		Latitude:  -21.3197212219,
		Longitude: -54.7352790833,
		Timezone:  "America/Campo_Grande",
	},
	"SSFR": {
		ICAO:      "SSFR",
		Name:      "Fronteira Airport",
		City:      "Fronteira",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1598,
		Latitude:  -20.2775001526,
		Longitude: -49.1908340454,
		Timezone:  "America/Sao_Paulo",
	},
	"SSFU": {
		ICAO:      "SSFU",
		Name:      "Fazenda Ligacao Airport",
		City:      "Sete Quedas",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1739,
		Latitude:  -23.8480567932,
		Longitude: -54.8161125183,
		Timezone:  "America/Campo_Grande",
	},
	"SSFW": {
		ICAO:      "SSFW",
		Name:      "Fazenda Grotao Airport",
		City:      "Coxim",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1063,
		Latitude:  -18.6774997711,
		Longitude: -54.3922233582,
		Timezone:  "America/Campo_Grande",
	},
	"SSFX": {
		ICAO:      "SSFX",
		Name:      "Fazenda Primavera Airport",
		City:      "Nova Fatima",
		State:     "Parana",
		Country:   "BR",
		Elevation: 2239,
		Latitude:  -23.4558334351,
		Longitude: -50.5758323669,
		Timezone:  "America/Sao_Paulo",
	},
	"SSGA": {
		ICAO:      "SSGA",
		Name:      "Garibaldi Airport",
		City:      "Garibaldi",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 2231,
		Latitude:  -29.2693996429,
		Longitude: -51.5317001343,
		Timezone:  "America/Sao_Paulo",
	},
	"SSGB": {
		ICAO:      "SSGB",
		Name:      "Guaratuba Airport",
		City:      "Guaratuba",
		State:     "Parana",
		Country:   "BR",
		Elevation: 49,
		Latitude:  -25.8815994263,
		Longitude: -48.6120986938,
		Timezone:  "America/Sao_Paulo",
	},
	"SSGC": {
		ICAO:      "SSGC",
		Name:      "General Canrobert Pereira da Costa Airport",
		City:      "Aquidauana",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 566,
		Latitude:  -20.4533004761,
		Longitude: -55.757598877,
		Timezone:  "America/Campo_Grande",
	},
	"SSGE": {
		ICAO:      "SSGE",
		Name:      "Fazenda Periquitos Airport",
		City:      "Tres Lagoas",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1180,
		Latitude:  -20.5069446564,
		Longitude: -51.8391685486,
		Timezone:  "America/Campo_Grande",
	},
	"SSGF": {
		ICAO:      "SSGF",
		Name:      "Fazenda Formosa Airport",
		City:      "Ribas Do Rio Pardo",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1575,
		Latitude:  -20.1994991302,
		Longitude: -53.224899292,
		Timezone:  "America/Campo_Grande",
	},
	"SSGG": {
		ICAO:      "SSGG",
		Name:      "Fazenda Jacare de Chifre Airport",
		City:      "Porto Murtinho",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 381,
		Latitude:  -21.0277786255,
		Longitude: -57.4972229004,
		Timezone:  "America/Campo_Grande",
	},
	"SSGH": {
		ICAO:      "SSGH",
		Name:      "Fazenda Capao Verde Airport",
		City:      "Aquidauana",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 410,
		Latitude:  -20.0736103058,
		Longitude: -56.0683326721,
		Timezone:  "America/Campo_Grande",
	},
	"SSGI": {
		ICAO:      "SSGI",
		Name:      "Astral Airport",
		City:      "Eldorado Do Sul",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 33,
		Latitude:  -30.0602779388,
		Longitude: -51.4375,
		Timezone:  "America/Sao_Paulo",
	},
	"SSGJ": {
		ICAO:      "SSGJ",
		Name:      "Fazenda Poleiro Grande Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 331,
		Latitude:  -17.2497215271,
		Longitude: -56.275554657,
		Timezone:  "America/Campo_Grande",
	},
	"SSGK": {
		ICAO:      "SSGK",
		Name:      "Fazenda Volta Grande Airport",
		City:      "Maracaju",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1699,
		Latitude:  -21.4558334351,
		Longitude: -55.456943512,
		Timezone:  "America/Campo_Grande",
	},
	"SSGM": {
		ICAO:      "SSGM",
		Name:      "Fazenda Seriema Airport",
		City:      "Bela Vista",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1151,
		Latitude:  -21.6933326721,
		Longitude: -56.5625,
		Timezone:  "America/Campo_Grande",
	},
	"SSGN": {
		ICAO:      "SSGN",
		Name:      "Fazenda Guaiba Airport",
		City:      "Iguatemi",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1401,
		Latitude:  -23.2391662598,
		Longitude: -54.5411109924,
		Timezone:  "America/Campo_Grande",
	},
	"SSGO": {
		ICAO:      "SSGO",
		Name:      "Rosada Airport",
		City:      "Sao Gabriel D`Oeste",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 2211,
		Latitude:  -19.4297218323,
		Longitude: -54.587223053,
		Timezone:  "America/Campo_Grande",
	},
	"SSGP": {
		ICAO:      "SSGP",
		Name:      "Piquet Airport",
		City:      "Brasilia",
		State:     "Federal-District",
		Country:   "BR",
		Elevation: 3281,
		Latitude:  -15.8538894653,
		Longitude: -47.806388855,
		Timezone:  "America/Sao_Paulo",
	},
	"SSGR": {
		ICAO:      "SSGR",
		IATA:      "0",
		Name:      "Guapore Airport",
		City:      "Guapore",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 1558,
		Latitude:  -28.8946990967,
		Longitude: -51.8553009033,
		Timezone:  "America/Sao_Paulo",
	},
	"SSGS": {
		ICAO:      "SSGS",
		Name:      "Fazenda Falcao Airport",
		City:      "Primavera Do Leste",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1975,
		Latitude:  -15.399444,
		Longitude: -54.181111,
		Timezone:  "America/Cuiaba",
	},
	"SSGT": {
		ICAO:      "SSGT",
		Name:      "Fazenda Capao Alto Airport",
		City:      "Rio Brilhante",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1050,
		Latitude:  -21.6988887787,
		Longitude: -54.8138885498,
		Timezone:  "America/Campo_Grande",
	},
	"SSGU": {
		ICAO:      "SSGU",
		Name:      "Fazenda Gurucaia Airport",
		City:      "Santa Isabel Do Ivai",
		State:     "Parana",
		Country:   "BR",
		Elevation: 886,
		Latitude:  -23.1933326721,
		Longitude: -53.3283348083,
		Timezone:  "America/Sao_Paulo",
	},
	"SSGW": {
		ICAO:      "SSGW",
		Name:      "Goio-Ere Airport",
		City:      "Goio-Ere",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1608,
		Latitude:  -24.2203006744,
		Longitude: -53.0444984436,
		Timezone:  "America/Sao_Paulo",
	},
	"SSGX": {
		ICAO:      "SSGX",
		Name:      "Fazenda Rio Negro Airport",
		City:      "Aquidauana",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 312,
		Latitude:  -19.5683326721,
		Longitude: -56.2455558777,
		Timezone:  "America/Campo_Grande",
	},
	"SSGY": {
		ICAO:      "SSGY",
		IATA:      "QGA",
		Name:      "Guaira Airport",
		City:      "Guaira",
		State:     "Canindeyu",
		Country:   "BR",
		Elevation: 889,
		Latitude:  -24.0811004639,
		Longitude: -54.1916999817,
		Timezone:  "America/Campo_Grande",
	},
	"SSGZ": {
		ICAO:      "SSGZ",
		Name:      "Fazenda Santa Helena do Pindo Airport",
		City:      "Navirai",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1050,
		Latitude:  -22.8950004578,
		Longitude: -53.8677787781,
		Timezone:  "America/Campo_Grande",
	},
	"SSHA": {
		ICAO:      "SSHA",
		Name:      "Aeroclube de Aquidauana Airport",
		City:      "Aquidauana",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 620,
		Latitude:  -20.4808330536,
		Longitude: -55.769443512,
		Timezone:  "America/Campo_Grande",
	},
	"SSHB": {
		ICAO:      "SSHB",
		Name:      "Fazenda Serra Dourada Airport",
		City:      "Ivate",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1007,
		Latitude:  -23.322221756,
		Longitude: -53.4686126709,
		Timezone:  "America/Sao_Paulo",
	},
	"SSHC": {
		ICAO:      "SSHC",
		Name:      "Capitao Pedro Paranhos Airport",
		City:      "Guapirama",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1447,
		Latitude:  -23.429444,
		Longitude: -50.167778,
		Timezone:  "America/Sao_Paulo",
	},
	"SSHD": {
		ICAO:      "SSHD",
		Name:      "Fazenda Bipandora II Airport",
		City:      "Itapora",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1368,
		Latitude:  -22.0027770996,
		Longitude: -54.9011116028,
		Timezone:  "America/Campo_Grande",
	},
	"SSHF": {
		ICAO:      "SSHF",
		Name:      "Sitio Serra Negra Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 328,
		Latitude:  -18.0333328247,
		Longitude: -57.4833335876,
		Timezone:  "America/Campo_Grande",
	},
	"SSHG": {
		ICAO:      "SSHG",
		Name:      "Antonio Joao Airport",
		City:      "Antonio Joao",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 2277,
		Latitude:  -22.3593997955,
		Longitude: -55.7639007568,
		Timezone:  "America/Campo_Grande",
	},
	"SSHI": {
		ICAO:      "SSHI",
		Name:      "Fazenda Dom Francisco Airport",
		City:      "Itaquirai",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1237,
		Latitude:  -23.2511100769,
		Longitude: -54.0483322144,
		Timezone:  "America/Campo_Grande",
	},
	"SSHJ": {
		ICAO:      "SSHJ",
		Name:      "Fazenda Santa Rita Airport",
		City:      "Ponta Pora",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1686,
		Latitude:  -22.2672214508,
		Longitude: -55.4880561829,
		Timezone:  "America/Campo_Grande",
	},
	"SSHK": {
		ICAO:      "SSHK",
		Name:      "Fazenda Jaguarandy Airport",
		City:      "Ponta Pora",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1968,
		Latitude:  -22.5897216797,
		Longitude: -55.569442749,
		Timezone:  "America/Campo_Grande",
	},
	"SSHL": {
		ICAO:      "SSHL",
		Name:      "Fazenda Jussara Airport",
		City:      "Jussara",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1213,
		Latitude:  -23.5750007629,
		Longitude: -52.4083328247,
		Timezone:  "America/Sao_Paulo",
	},
	"SSHN": {
		ICAO:      "SSHN",
		Name:      "Recanto das Aguias Airport",
		City:      "Iguaracu",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1637,
		Latitude:  -23.2450008392,
		Longitude: -51.8755569458,
		Timezone:  "America/Sao_Paulo",
	},
	"SSHQ": {
		ICAO:      "SSHQ",
		Name:      "Fazenda Paraiso Airport",
		City:      "Terenos",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1280,
		Latitude:  -20.5616664886,
		Longitude: -54.8641662598,
		Timezone:  "America/Campo_Grande",
	},
	"SSHT": {
		ICAO:      "SSHT",
		Name:      "Ingazeira Airport",
		City:      "Alta Floresta",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 853,
		Latitude:  -10.1304620248,
		Longitude: -56.7186355591,
		Timezone:  "America/Cuiaba",
	},
	"SSHU": {
		ICAO:      "SSHU",
		Name:      "Fazenda Centenario Airport",
		City:      "Aquidauana",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 426,
		Latitude:  -19.4580554962,
		Longitude: -56.0452766418,
		Timezone:  "America/Campo_Grande",
	},
	"SSHW": {
		ICAO:      "SSHW",
		Name:      "Fazenda Macarai Airport",
		City:      "Itaquirai",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1056,
		Latitude:  -23.4174995422,
		Longitude: -54.0125007629,
		Timezone:  "America/Campo_Grande",
	},
	"SSHX": {
		ICAO:      "SSHX",
		Name:      "Fazenda Jatoba Airport",
		City:      "Paranhos",
		State:     "Canindeyu",
		Country:   "BR",
		Elevation: 1335,
		Latitude:  -23.9237995148,
		Longitude: -55.3018989563,
		Timezone:  "America/Campo_Grande",
	},
	"SSHZ": {
		ICAO:      "SSHZ",
		IATA:      "HRZ",
		Name:      "Horizontina Airport",
		City:      "Horizontina",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 1063,
		Latitude:  -27.638299942,
		Longitude: -54.339099884,
		Timezone:  "America/Sao_Paulo",
	},
	"SSIB": {
		ICAO:      "SSIB",
		Name:      "Fazenda Suely Airport",
		City:      "Agua Clara",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1574,
		Latitude:  -19.4799995422,
		Longitude: -52.8125,
		Timezone:  "America/Campo_Grande",
	},
	"SSIC": {
		ICAO:      "SSIC",
		Name:      "APLIC Aviacao Agricola Ltda Airport",
		City:      "Dourados",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1405,
		Latitude:  -22.3066673279,
		Longitude: -54.8244438171,
		Timezone:  "America/Campo_Grande",
	},
	"SSID": {
		ICAO:      "SSID",
		Name:      "Independencia Airport",
		City:      "Nova Andradina",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1135,
		Latitude:  -22.2791996002,
		Longitude: -53.3834991455,
		Timezone:  "America/Campo_Grande",
	},
	"SSIE": {
		ICAO:      "SSIE",
		Name:      "Teruel Ipanema Estancia Airport",
		City:      "Campo Grande",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1732,
		Latitude:  -20.5994434357,
		Longitude: -54.6025009155,
		Timezone:  "America/Campo_Grande",
	},
	"SSIF": {
		ICAO:      "SSIF",
		Name:      "Fazenda Baunilha Airport",
		City:      "Itaquirai",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 970,
		Latitude:  -23.560300827,
		Longitude: -54.0339012146,
		Timezone:  "America/Campo_Grande",
	},
	"SSIG": {
		ICAO:      "SSIG",
		Name:      "Fazenda Santa Maria Airport",
		City:      "Aquidauana",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 617,
		Latitude:  -20.2700004578,
		Longitude: -55.7283325195,
		Timezone:  "America/Campo_Grande",
	},
	"SSIH": {
		ICAO:      "SSIH",
		Name:      "Fazenda Fortaleza Airport",
		City:      "Brasilandia",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1296,
		Latitude:  -20.9463882446,
		Longitude: -52.7222213745,
		Timezone:  "America/Campo_Grande",
	},
	"SSII": {
		ICAO:      "SSII",
		Name:      "Fazenda Orion Airport",
		City:      "Bela Vista",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1001,
		Latitude:  -22.0227775574,
		Longitude: -56.3241653442,
		Timezone:  "America/Campo_Grande",
	},
	"SSIJ": {
		ICAO:      "SSIJ",
		IATA:      "IJU",
		Name:      "Ijui Airport",
		City:      "Ijui",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 1197,
		Latitude:  -28.3687000275,
		Longitude: -53.8465995789,
		Timezone:  "America/Sao_Paulo",
	},
	"SSIM": {
		ICAO:      "SSIM",
		Name:      "Boqueirao Alegre Airport",
		City:      "Quarai",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 732,
		Latitude:  -30.2549991608,
		Longitude: -55.8394432068,
		Timezone:  "America/Sao_Paulo",
	},
	"SSIO": {
		ICAO:      "SSIO",
		Name:      "Candiota Airport",
		City:      "Candiota",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 741,
		Latitude:  -31.4941673279,
		Longitude: -53.6908340454,
		Timezone:  "America/Sao_Paulo",
	},
	"SSIP": {
		ICAO:      "SSIP",
		Name:      "Fazenda Santa Otilia Airport",
		City:      "Porto Murtinho",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 919,
		Latitude:  -21.1905555725,
		Longitude: -57.0416679382,
		Timezone:  "America/Campo_Grande",
	},
	"SSIQ": {
		ICAO:      "SSIQ",
		IATA:      "ITQ",
		Name:      "Itaqui Airport",
		City:      "Itaqui",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 230,
		Latitude:  -29.1730995178,
		Longitude: -56.5367012024,
		Timezone:  "America/Sao_Paulo",
	},
	"SSIR": {
		ICAO:      "SSIR",
		Name:      "Ibiruba Airport",
		City:      "Ibiruba",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 1447,
		Latitude:  -28.6041679382,
		Longitude: -53.0788879395,
		Timezone:  "America/Sao_Paulo",
	},
	"SSIS": {
		ICAO:      "SSIS",
		Name:      "Major Jose Ignacio Airport",
		City:      "Ibate",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1962,
		Latitude:  -22.047778,
		Longitude: -48.097222,
		Timezone:  "America/Sao_Paulo",
	},
	"SSIU": {
		ICAO:      "SSIU",
		Name:      "Sitio Enel Airport",
		City:      "Pantano Grande",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 32,
		Latitude:  -30.1336116791,
		Longitude: -52.3652763367,
		Timezone:  "America/Sao_Paulo",
	},
	"SSIV": {
		ICAO:      "SSIV",
		Name:      "Fazenda Experimental Airport",
		City:      "Ivinhema",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1194,
		Latitude:  -22.2972221375,
		Longitude: -53.8472222222,
		Timezone:  "America/Campo_Grande",
	},
	"SSIW": {
		ICAO:      "SSIW",
		Name:      "Fazenda Damaro Airport",
		City:      "Miranda",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 344,
		Latitude:  -19.9775009155,
		Longitude: -56.4361114502,
		Timezone:  "America/Campo_Grande",
	},
	"SSIX": {
		ICAO:      "SSIX",
		Name:      "Fazenda Campo Verde Airport",
		City:      "Taquarussu",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 984,
		Latitude:  -22.6516666412,
		Longitude: -53.5077781677,
		Timezone:  "America/Campo_Grande",
	},
	"SSIY": {
		ICAO:      "SSIY",
		Name:      "Fazenda Santa Adelina Airport",
		City:      "Guaicara",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1411,
		Latitude:  -21.528889,
		Longitude: -49.743333,
		Timezone:  "America/Sao_Paulo",
	},
	"SSIZ": {
		ICAO:      "SSIZ",
		Name:      "Condominio Fazenda da Grama Airport",
		City:      "Itupeva",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2257,
		Latitude:  -23.059167,
		Longitude: -47.0625,
		Timezone:  "America/Sao_Paulo",
	},
	"SSJA": {
		ICAO:      "SSJA",
		IATA:      "JCB",
		Name:      "Santa Terezinha Airport",
		City:      "Joacaba",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2546,
		Latitude:  -27.1714000702,
		Longitude: -51.5532989502,
		Timezone:  "America/Sao_Paulo",
	},
	"SSJB": {
		ICAO:      "SSJB",
		Name:      "Salenco Airport",
		City:      "Sao Gabriel D`Oeste",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1253,
		Latitude:  -18.8566665649,
		Longitude: -54.3474998474,
		Timezone:  "America/Campo_Grande",
	},
	"SSJC": {
		ICAO:      "SSJC",
		Name:      "Fazenda do Jacui Airport",
		City:      "Guaiba",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 26,
		Latitude:  -29.9830551147,
		Longitude: -51.3005561829,
		Timezone:  "America/Sao_Paulo",
	},
	"SSJE": {
		ICAO:      "SSJE",
		Name:      "Fazenda Sao Joaquim Airport",
		City:      "Selviria",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1325,
		Latitude:  -20.35222222,
		Longitude: -51.55583333,
		Timezone:  "America/Campo_Grande",
	},
	"SSJG": {
		ICAO:      "SSJG",
		Name:      "Fazenda Santa Ana Airport",
		City:      "Porto Murtinho",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 351,
		Latitude:  -21.2163887024,
		Longitude: -57.681388855,
		Timezone:  "America/Campo_Grande",
	},
	"SSJH": {
		ICAO:      "SSJH",
		Name:      "Fazenda Guarani Airport",
		City:      "Nova Andradina",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 787,
		Latitude:  -22.1658325195,
		Longitude: -53.5730552673,
		Timezone:  "America/Campo_Grande",
	},
	"SSJI": {
		ICAO:      "SSJI",
		Name:      "Jardim Airport",
		City:      "Jardim",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1053,
		Latitude:  -21.4930992126,
		Longitude: -56.1525993347,
		Timezone:  "America/Campo_Grande",
	},
	"SSJJ": {
		ICAO:      "SSJJ",
		Name:      "Fazenda Tupi Airport",
		City:      "Nova Andradina",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1377,
		Latitude:  -21.8072223663,
		Longitude: -53.1294441223,
		Timezone:  "America/Campo_Grande",
	},
	"SSJK": {
		ICAO:      "SSJK",
		IATA:      "0",
		Name:      "Julio de Castilho Airport",
		City:      "Julio De Castilho",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 1633,
		Latitude:  -29.1530990601,
		Longitude: -53.7433013916,
		Timezone:  "America/Sao_Paulo",
	},
	"SSJL": {
		ICAO:      "SSJL",
		Name:      "Fazenda Jamaica Airport",
		City:      "Camapua",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1706,
		Latitude:  -19.3477783203,
		Longitude: -53.7700004578,
		Timezone:  "America/Campo_Grande",
	},
	"SSJM": {
		ICAO:      "SSJM",
		Name:      "Fazenda Julio Martins Airport",
		City:      "Chapadao Do Sul",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 2621,
		Latitude:  -18.8425006866,
		Longitude: -52.4994430542,
		Timezone:  "America/Campo_Grande",
	},
	"SSJO": {
		ICAO:      "SSJO",
		Name:      "Fazenda Angico Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 511,
		Latitude:  -19.3763885498,
		Longitude: -57.4527778625,
		Timezone:  "America/Campo_Grande",
	},
	"SSJP": {
		ICAO:      "SSJP",
		Name:      "Fazenda Junqueira Airport",
		City:      "Colorado",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1427,
		Latitude:  -22.8950004578,
		Longitude: -51.9291687012,
		Timezone:  "America/Sao_Paulo",
	},
	"SSJQ": {
		ICAO:      "SSJQ",
		Name:      "Fazenda Floresta Airport",
		City:      "Ribeirao Cascalheira",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1266,
		Latitude:  -12.81,
		Longitude: -51.88,
		Timezone:  "America/Cuiaba",
	},
	"SSJR": {
		ICAO:      "SSJR",
		Name:      "Jaguarao Airport",
		City:      "Jaguarao",
		State:     "Cerro-Largo",
		Country:   "BR",
		Elevation: 66,
		Latitude:  -32.5405578613,
		Longitude: -53.3841667175,
		Timezone:  "America/Sao_Paulo",
	},
	"SSJT": {
		ICAO:      "SSJT",
		Name:      "Fazenda Rio Lagoa Airport",
		City:      "Ribas Do Rio Pardo",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1082,
		Latitude:  -20.84889,
		Longitude: -53.20667,
		Timezone:  "America/Campo_Grande",
	},
	"SSJU": {
		ICAO:      "SSJU",
		Name:      "Fazenda Santa Paula Airport",
		City:      "Inocencia",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1200,
		Latitude:  -19.3813896179,
		Longitude: -52.3097229004,
		Timezone:  "America/Campo_Grande",
	},
	"SSJV": {
		ICAO:      "SSJV",
		Name:      "Piratininga Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 150,
		Latitude:  -17.9424991608,
		Longitude: -56.4799995422,
		Timezone:  "America/Campo_Grande",
	},
	"SSJW": {
		ICAO:      "SSJW",
		Name:      "Fazenda Jauru Airport",
		City:      "Rio Brilhante",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1017,
		Latitude:  -21.7824993134,
		Longitude: -53.8922233582,
		Timezone:  "America/Campo_Grande",
	},
	"SSJX": {
		ICAO:      "SSJX",
		Name:      "Sao Bento Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 150,
		Latitude:  -19.478055954,
		Longitude: -57.0141677856,
		Timezone:  "America/Campo_Grande",
	},
	"SSJY": {
		ICAO:      "SSJY",
		Name:      "Sao Joao Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 150,
		Latitude:  -17.7075004578,
		Longitude: -56.2172203064,
		Timezone:  "America/Campo_Grande",
	},
	"SSJZ": {
		ICAO:      "SSJZ",
		Name:      "Fazenda Tres Unidos Airport",
		City:      "Terra Roxa",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1000,
		Latitude:  -24.0516662598,
		Longitude: -54.0066680908,
		Timezone:  "America/Sao_Paulo",
	},
	"SSKA": {
		ICAO:      "SSKA",
		Name:      "Fazenda Campinas Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 328,
		Latitude:  -18.9055557251,
		Longitude: -56.3597221375,
		Timezone:  "America/Campo_Grande",
	},
	"SSKB": {
		ICAO:      "SSKB",
		Name:      "Fazenda Tupanciretan Airport",
		City:      "Aquidauana",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 312,
		Latitude:  -19.4744434357,
		Longitude: -56.3772201538,
		Timezone:  "America/Campo_Grande",
	},
	"SSKC": {
		ICAO:      "SSKC",
		Name:      "Fazenda Uniao Airport",
		City:      "Inocencia",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1758,
		Latitude:  -19.2924995422,
		Longitude: -52.0608329773,
		Timezone:  "America/Campo_Grande",
	},
	"SSKD": {
		ICAO:      "SSKD",
		Name:      "Fazenda Uniao Airport",
		City:      "Aquidauana",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 410,
		Latitude:  -19.75,
		Longitude: -55.4666671753,
		Timezone:  "America/Campo_Grande",
	},
	"SSKE": {
		ICAO:      "SSKE",
		Name:      "Campo Evelina Airport",
		City:      "Santa Monica",
		State:     "Parana",
		Country:   "BR",
		Elevation: 984,
		Latitude:  -23.1211109161,
		Longitude: -53.0763893127,
		Timezone:  "America/Sao_Paulo",
	},
	"SSKF": {
		ICAO:      "SSKF",
		Name:      "Porto Bonito Airport",
		City:      "Itaquirai",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1180,
		Latitude:  -23.3374996185,
		Longitude: -53.869720459,
		Timezone:  "America/Campo_Grande",
	},
	"SSKG": {
		ICAO:      "SSKG",
		Name:      "Estancia Santa Maria Airport",
		City:      "Campo Grande",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 2028,
		Latitude:  -20.5053005219,
		Longitude: -54.5255012512,
		Timezone:  "America/Campo_Grande",
	},
	"SSKH": {
		ICAO:      "SSKH",
		Name:      "Fazenda Karl Hermann Isenberg Airport",
		City:      "Aral Moreira",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1853,
		Latitude:  -22.9116668701,
		Longitude: -55.5388908386,
		Timezone:  "America/Campo_Grande",
	},
	"SSKI": {
		ICAO:      "SSKI",
		Name:      "Fazenda Santa Helena Airport",
		City:      "Cornelio Procopio",
		Country:   "BR",
		Elevation: 1673,
		Latitude:  -23.340555191,
		Longitude: -50.5830574036,
		Timezone:  "America/Sao_Paulo",
	},
	"SSKJ": {
		ICAO:      "SSKJ",
		Name:      "Fazenda Mestica Airport",
		City:      "Rio Branco Do Ivai",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1575,
		Latitude:  -24.3072223663,
		Longitude: -51.4108314514,
		Timezone:  "America/Sao_Paulo",
	},
	"SSKK": {
		ICAO:      "SSKK",
		Name:      "Capao da Canoa Airport",
		City:      "Capao Da Canoa",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 33,
		Latitude:  -29.7636108398,
		Longitude: -50.036945343,
		Timezone:  "America/Sao_Paulo",
	},
	"SSKL": {
		ICAO:      "SSKL",
		Name:      "Fazenda Piao Airport",
		City:      "Amambai",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1542,
		Latitude:  -23.4069442749,
		Longitude: -55.2266654968,
		Timezone:  "America/Campo_Grande",
	},
	"SSKM": {
		ICAO:      "SSKM",
		IATA:      "CBW",
		Name:      "Campo Mourao Airport",
		City:      "Campo Mourao",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1854,
		Latitude:  -24.0091991425,
		Longitude: -52.3568000793,
		Timezone:  "America/Sao_Paulo",
	},
	"SSKN": {
		ICAO:      "SSKN",
		Name:      "Campo Novo Airport",
		City:      "Campo Novo",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 1460,
		Latitude:  -27.6672000885,
		Longitude: -53.8089981079,
		Timezone:  "America/Sao_Paulo",
	},
	"SSKO": {
		ICAO:      "SSKO",
		Name:      "Independencia Airport",
		City:      "Anastacio",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 544,
		Latitude:  -20.4761104584,
		Longitude: -55.8336105347,
		Timezone:  "America/Campo_Grande",
	},
	"SSKP": {
		ICAO:      "SSKP",
		Name:      "Copacel Airport",
		City:      "Palotina",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1050,
		Latitude:  -24.308889389,
		Longitude: -53.8422203064,
		Timezone:  "America/Sao_Paulo",
	},
	"SSKQ": {
		ICAO:      "SSKQ",
		Name:      "Fazenda Santa Rosa Airport",
		City:      "Nova Crixas",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1001,
		Latitude:  -14.541111,
		Longitude: -50.461944,
		Timezone:  "America/Sao_Paulo",
	},
	"SSKR": {
		ICAO:      "SSKR",
		Name:      "Fazenda Primavera Airport",
		City:      "Parauna",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2306,
		Latitude:  -17.17,
		Longitude: -50.82,
		Timezone:  "America/Sao_Paulo",
	},
	"SSKS": {
		ICAO:      "SSKS",
		IATA:      "QDB",
		Name:      "Cachoeira do Sul Airport",
		City:      "Cachoeira Do Sul",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 253,
		Latitude:  -30.0018997192,
		Longitude: -52.9407997131,
		Timezone:  "America/Sao_Paulo",
	},
	"SSKT": {
		ICAO:      "SSKT",
		Name:      "Aeroclube de Santa Catarina Airport",
		City:      "Sao Jose",
		State:     "Santa-Catarina",
		Country:   "BR",
		Elevation: 20,
		Latitude:  -27.6116676331,
		Longitude: -48.6727790833,
		Timezone:  "America/Sao_Paulo",
	},
	"SSKU": {
		ICAO:      "SSKU",
		IATA:      "QCR",
		Name:      "Curitibanos Airport",
		City:      "Curitibanos",
		State:     "Santa-Catarina",
		Country:   "BR",
		Elevation: 3133,
		Latitude:  -27.2824993134,
		Longitude: -50.6114006042,
		Timezone:  "America/Sao_Paulo",
	},
	"SSKV": {
		ICAO:      "SSKV",
		Name:      "Retiro Sao Joao Airport",
		City:      "Ponta Pora",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1738,
		Latitude:  -22.396944046,
		Longitude: -55.4855575562,
		Timezone:  "America/Campo_Grande",
	},
	"SSKW": {
		ICAO:      "SSKW",
		IATA:      "OAL",
		Name:      "Cacoal Airport",
		City:      "Cacoal",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 817,
		Latitude:  -11.493611,
		Longitude: -61.175556,
		Timezone:  "America/Porto_Velho",
	},
	"SSKX": {
		ICAO:      "SSKX",
		Name:      "Fazenda Sao Luiz do Matao Airport",
		City:      "Carneirinho",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1250,
		Latitude:  -19.8722222222,
		Longitude: -50.6547222222,
		Timezone:  "America/Sao_Paulo",
	},
	"SSKY": {
		ICAO:      "SSKY",
		Name:      "Fazenda Querencia Airport",
		City:      "Tangara Da Serra",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2362,
		Latitude:  -14.4736111111,
		Longitude: -58.3552777778,
		Timezone:  "America/Cuiaba",
	},
	"SSKZ": {
		ICAO:      "SSKZ",
		Name:      "Carazinho Airport",
		City:      "Carazinho",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 1854,
		Latitude:  -28.3225002289,
		Longitude: -52.8162002563,
		Timezone:  "America/Sao_Paulo",
	},
	"SSLA": {
		ICAO:      "SSLA",
		IATA:      "0",
		Name:      "Laguna Airport",
		City:      "Laguna",
		State:     "Santa-Catarina",
		Country:   "BR",
		Elevation: 20,
		Latitude:  -28.3766994476,
		Longitude: -48.7428016663,
		Timezone:  "America/Sao_Paulo",
	},
	"SSLB": {
		ICAO:      "SSLB",
		Name:      "Fazenda Sao Paulino Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 394,
		Latitude:  -19.0252780914,
		Longitude: -55.926109314,
		Timezone:  "America/Campo_Grande",
	},
	"SSLC": {
		ICAO:      "SSLC",
		Name:      "Fazenda Laguna Carem Airport",
		City:      "Laguna Carapa",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1755,
		Latitude:  -22.6733322144,
		Longitude: -55.2366676331,
		Timezone:  "America/Campo_Grande",
	},
	"SSLD": {
		ICAO:      "SSLD",
		Name:      "Fazenda Lourdes Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 328,
		Latitude:  -18.6438884735,
		Longitude: -56.7594451904,
		Timezone:  "America/Campo_Grande",
	},
	"SSLF": {
		ICAO:      "SSLF",
		Name:      "Fazenda Palmeira Airport",
		City:      "Taquarussu",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 885,
		Latitude:  -22.5436115265,
		Longitude: -53.4294433594,
		Timezone:  "America/Campo_Grande",
	},
	"SSLG": {
		ICAO:      "SSLG",
		Name:      "Sao Luis Gonzaga Airport",
		City:      "Sao Luis Gonzaga",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 656,
		Latitude:  -28.3763885498,
		Longitude: -55.0374984741,
		Timezone:  "America/Sao_Paulo",
	},
	"SSLI": {
		ICAO:      "SSLI",
		IATA:      "LVB",
		Name:      "Estancia Portal do Sol Airport",
		City:      "Itirapina",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2425,
		Latitude:  -22.165278,
		Longitude: -47.892778,
		Timezone:  "America/Sao_Paulo",
	},
	"SSLJ": {
		ICAO:      "SSLJ",
		Name:      "Fazenda Santa Izabel Airport",
		City:      "Flores De Goias",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1434,
		Latitude:  -14.265278,
		Longitude: -47.0275,
		Timezone:  "America/Sao_Paulo",
	},
	"SSLK": {
		ICAO:      "SSLK",
		Name:      "Fazenda Aldeia Airport",
		City:      "Coxim",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 394,
		Latitude:  -18.2163887024,
		Longitude: -55.1588897705,
		Timezone:  "America/Campo_Grande",
	},
	"SSLL": {
		ICAO:      "SSLL",
		Name:      "Fazenda Itaguassu Airport",
		City:      "Antonio Joao",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1788,
		Latitude:  -22.2436103821,
		Longitude: -55.9316673279,
		Timezone:  "America/Asuncion",
	},
	"SSLN": {
		ICAO:      "SSLN",
		IATA:      "LOI",
		Name:      "Helmuth Baungarten Airport",
		City:      "Lontras",
		State:     "Santa-Catarina",
		Country:   "BR",
		Elevation: 1095,
		Latitude:  -27.1599998474,
		Longitude: -49.5424995422,
		Timezone:  "America/Sao_Paulo",
	},
	"SSLO": {
		ICAO:      "SSLO",
		Name:      "Loanda Airport",
		City:      "Loanda",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1542,
		Latitude:  -22.9171009064,
		Longitude: -53.1489982605,
		Timezone:  "America/Sao_Paulo",
	},
	"SSLP": {
		ICAO:      "SSLP",
		Name:      "Fazenda Lucero Pora Airport",
		City:      "Porto Murtinho",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 299,
		Latitude:  -21.9108333588,
		Longitude: -57.6466674805,
		Timezone:  "America/Campo_Grande",
	},
	"SSLQ": {
		ICAO:      "SSLQ",
		Name:      "Fazenda Campanario Airport",
		City:      "Laguna Carapa",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1362,
		Latitude:  -22.7847213745,
		Longitude: -55.0741653442,
		Timezone:  "America/Campo_Grande",
	},
	"SSLR": {
		ICAO:      "SSLR",
		Name:      "Ocorema Airport",
		City:      "Ladario",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 413,
		Latitude:  -19.1049995422,
		Longitude: -57.593334198,
		Timezone:  "America/Campo_Grande",
	},
	"SSLS": {
		ICAO:      "SSLS",
		Name:      "Salto Santiago Airport",
		City:      "Laranjeiras Do Sul",
		State:     "Parana",
		Country:   "BR",
		Elevation: 2087,
		Latitude:  -25.5149993896,
		Longitude: -52.5574989319,
		Timezone:  "America/Sao_Paulo",
	},
	"SSLT": {
		ICAO:      "SSLT",
		IATA:      "ALQ",
		Name:      "Alegrete Novo Airport",
		City:      "Alegrete",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 459,
		Latitude:  -29.8127002716,
		Longitude: -55.8933982849,
		Timezone:  "America/Sao_Paulo",
	},
	"SSLU": {
		ICAO:      "SSLU",
		Name:      "Fazenda Aurora Airport",
		City:      "Jardim",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 754,
		Latitude:  -21.5233325958,
		Longitude: -56.6116676331,
		Timezone:  "America/Campo_Grande",
	},
	"SSLW": {
		ICAO:      "SSLW",
		Name:      "Fazenda Tres Cerros Airport",
		City:      "Bela Vista",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 853,
		Latitude:  -21.9733333588,
		Longitude: -56.7975006104,
		Timezone:  "America/Campo_Grande",
	},
	"SSLX": {
		ICAO:      "SSLX",
		Name:      "Fazenda Brasilia do Sul Airport",
		City:      "Juti",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1296,
		Latitude:  -22.7049999237,
		Longitude: -54.6083335876,
		Timezone:  "America/Campo_Grande",
	},
	"SSLZ": {
		ICAO:      "SSLZ",
		Name:      "Fazenda Vitoria do Ivinhema Airport",
		City:      "Ivinhema",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1296,
		Latitude:  -22.5197219849,
		Longitude: -53.6647224426,
		Timezone:  "America/Campo_Grande",
	},
	"SSMC": {
		ICAO:      "SSMC",
		Name:      "Fazenda Sao Miguel da Catequese Airport",
		City:      "Nova Andradina",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1209,
		Latitude:  -22.0877780914,
		Longitude: -53.5200004578,
		Timezone:  "America/Campo_Grande",
	},
	"SSMD": {
		ICAO:      "SSMD",
		Name:      "Medianeira Airport",
		City:      "Medianeira",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1473,
		Latitude:  -25.3108997345,
		Longitude: -54.0704994202,
		Timezone:  "America/Sao_Paulo",
	},
	"SSME": {
		ICAO:      "SSME",
		Name:      "Fazenda Ipiranga Airport",
		City:      "Anastacio",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 804,
		Latitude:  -20.980556488,
		Longitude: -55.5658340454,
		Timezone:  "America/Campo_Grande",
	},
	"SSMF": {
		ICAO:      "SSMF",
		IATA:      "QMF",
		Name:      "Mafra Airport",
		City:      "Mafra",
		State:     "Santa-Catarina",
		Country:   "BR",
		Elevation: 2690,
		Latitude:  -26.1588993073,
		Longitude: -49.8321990967,
		Timezone:  "America/Sao_Paulo",
	},
	"SSMH": {
		ICAO:      "SSMH",
		Name:      "Fazenda Santa Maria Airport",
		City:      "Capao Do Leao",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 66,
		Latitude:  -31.9294433594,
		Longitude: -52.4636116028,
		Timezone:  "America/Sao_Paulo",
	},
	"SSMI": {
		ICAO:      "SSMI",
		Name:      "Fazenda Morrinhos Airport",
		City:      "Bela Vista",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1148,
		Latitude:  -22.0838890076,
		Longitude: -56.3005561829,
		Timezone:  "America/Campo_Grande",
	},
	"SSMJ": {
		ICAO:      "SSMJ",
		Name:      "Maracaju Airport",
		City:      "Maracaju",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1283,
		Latitude:  -21.6338882446,
		Longitude: -55.1450004578,
		Timezone:  "America/Campo_Grande",
	},
	"SSMK": {
		ICAO:      "SSMK",
		Name:      "El Dorado Airport",
		City:      "Eldorado Do Sul",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 20,
		Latitude:  -30.0733337402,
		Longitude: -51.4097213745,
		Timezone:  "America/Sao_Paulo",
	},
	"SSML": {
		ICAO:      "SSML",
		Name:      "Fazenda Maria Luiza Airport",
		City:      "Iguatemi",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1083,
		Latitude:  -23.5972213745,
		Longitude: -54.6458320618,
		Timezone:  "America/Campo_Grande",
	},
	"SSMN": {
		ICAO:      "SSMN",
		Name:      "Kururuzinho Airport",
		City:      "Jacareacanga",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1640,
		Latitude:  -8.88,
		Longitude: -57.33,
		Timezone:  "America/Cuiaba",
	},
	"SSMP": {
		ICAO:      "SSMP",
		Name:      "Fazenda Tres Minas Airport",
		City:      "Itambe",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1391,
		Latitude:  -23.6727771759,
		Longitude: -52.0097236633,
		Timezone:  "America/Sao_Paulo",
	},
	"SSMR": {
		ICAO:      "SSMR",
		Name:      "Manoel Ribas Airport",
		City:      "Manoel Ribas",
		State:     "Parana",
		Country:   "BR",
		Elevation: 2822,
		Latitude:  -24.5296001434,
		Longitude: -51.6519012451,
		Timezone:  "America/Sao_Paulo",
	},
	"SSMS": {
		ICAO:      "SSMS",
		Name:      "Fazenda Rancho Esperanca Airport",
		City:      "Aral Moreira",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1522,
		Latitude:  -22.87,
		Longitude: -55.5,
		Timezone:  "America/Campo_Grande",
	},
	"SSMT": {
		ICAO:      "SSMT",
		Name:      "Mostardas Airport",
		City:      "Mostardas",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 59,
		Latitude:  -31.1035995483,
		Longitude: -50.9103012085,
		Timezone:  "America/Sao_Paulo",
	},
	"SSMV": {
		ICAO:      "SSMV",
		Name:      "Fazenda Boa Vista Airport",
		City:      "Pirassununga",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2434,
		Latitude:  -22.0425,
		Longitude: -47.56,
		Timezone:  "America/Sao_Paulo",
	},
	"SSMW": {
		ICAO:      "SSMW",
		Name:      "Ica Airport",
		City:      "Peruibe",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 49,
		Latitude:  -24.3152770996,
		Longitude: -47.0138893127,
		Timezone:  "America/Sao_Paulo",
	},
	"SSMX": {
		ICAO:      "SSMX",
		Name:      "Fazenda Planalto Airport",
		City:      "Nova Resende",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3868,
		Latitude:  -21.12,
		Longitude: -46.42,
		Timezone:  "America/Sao_Paulo",
	},
	"SSMY": {
		ICAO:      "SSMY",
		Name:      "Sao Miguel do Iguacu Airport",
		City:      "Sao Miguel Do Iguacu",
		State:     "Parana",
		Country:   "BR",
		Elevation: 951,
		Latitude:  -25.39888889,
		Longitude: -54.23472222,
		Timezone:  "America/Sao_Paulo",
	},
	"SSMZ": {
		ICAO:      "SSMZ",
		Name:      "Fazenda Sao Mateus Airport",
		City:      "Comodoro",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1070,
		Latitude:  -13.917222,
		Longitude: -59.983333,
		Timezone:  "America/Cuiaba",
	},
	"SSNA": {
		ICAO:      "SSNA",
		Name:      "Estancia Miranda Airport",
		City:      "Miranda",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 436,
		Latitude:  -19.9799995422,
		Longitude: -56.4338874817,
		Timezone:  "America/Campo_Grande",
	},
	"SSNB": {
		ICAO:      "SSNB",
		Name:      "Navirai Airport",
		City:      "Navirai",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1329,
		Latitude:  -23.0347995758,
		Longitude: -54.1801986694,
		Timezone:  "America/Campo_Grande",
	},
	"SSND": {
		ICAO:      "SSND",
		Name:      "Santa Maria Airport",
		City:      "Sao Felix Do Araguaia",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1119,
		Latitude:  -11.16,
		Longitude: -52.82,
		Timezone:  "America/Cuiaba",
	},
	"SSNF": {
		ICAO:      "SSNF",
		Name:      "Fazenda Naisa Airport",
		City:      "Colombia",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1739,
		Latitude:  -20.3297214508,
		Longitude: -48.8100013733,
		Timezone:  "America/Sao_Paulo",
	},
	"SSNG": {
		ICAO:      "SSNG",
		IATA:      "QGF",
		Name:      "Montenegro Airport",
		City:      "Montenegro",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 125,
		Latitude:  -29.7194004059,
		Longitude: -51.4893989563,
		Timezone:  "America/Sao_Paulo",
	},
	"SSNH": {
		ICAO:      "SSNH",
		IATA:      "QHV",
		Name:      "Novo Hamburgo Airport",
		City:      "Novo Hamburgo",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 66,
		Latitude:  -29.696100235,
		Longitude: -51.0816993713,
		Timezone:  "America/Sao_Paulo",
	},
	"SSNI": {
		ICAO:      "SSNI",
		Name:      "Fazenda Novo Rumo Airport",
		City:      "Navirai",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1247,
		Latitude:  -22.9852771759,
		Longitude: -53.8769454956,
		Timezone:  "America/Campo_Grande",
	},
	"SSNJ": {
		ICAO:      "SSNJ",
		Name:      "Fazenda Primavera Airport",
		City:      "Aquidauana",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 341,
		Latitude:  -19.5075,
		Longitude: -55.90528,
		Timezone:  "America/Campo_Grande",
	},
	"SSNK": {
		ICAO:      "SSNK",
		Name:      "Sementes Petrovina Airport",
		City:      "Pedra Preta",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2362,
		Latitude:  -16.8427772522,
		Longitude: -54.0758323669,
		Timezone:  "America/Cuiaba",
	},
	"SSNL": {
		ICAO:      "SSNL",
		Name:      "Fazenda Essencia Airport",
		City:      "Nobres",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1818,
		Latitude:  -14.316667,
		Longitude: -55.766944,
		Timezone:  "America/Cuiaba",
	},
	"SSNM": {
		ICAO:      "SSNM",
		Name:      "Guaranta Airport",
		City:      "Guaranta Do Norte",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 968,
		Latitude:  -9.9658327103,
		Longitude: -54.8849983215,
		Timezone:  "America/Cuiaba",
	},
	"SSNN": {
		ICAO:      "SSNN",
		Name:      "Salvaterra II Airport",
		City:      "Joao Pinheiro",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1749,
		Latitude:  -17.1975002289,
		Longitude: -45.7766685486,
		Timezone:  "America/Sao_Paulo",
	},
	"SSNO": {
		ICAO:      "SSNO",
		IATA:      "0",
		Name:      "Nonoai Airport",
		City:      "Nonoai",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 1969,
		Latitude:  -27.3672008514,
		Longitude: -52.7338981628,
		Timezone:  "America/Sao_Paulo",
	},
	"SSNP": {
		ICAO:      "SSNP",
		IATA:      "0",
		Name:      "Nova Prata Airport",
		City:      "Nova Prata",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 2264,
		Latitude:  -28.8024997711,
		Longitude: -51.6041984558,
		Timezone:  "America/Sao_Paulo",
	},
	"SSNQ": {
		ICAO:      "SSNQ",
		Name:      "Nioaque Airport",
		City:      "Nioaque",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 984,
		Latitude:  -21.1847229004,
		Longitude: -55.8302764893,
		Timezone:  "America/Campo_Grande",
	},
	"SSNR": {
		ICAO:      "SSNR",
		Name:      "Fazenda Novo Horizonte Airport",
		City:      "Coxim",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 754,
		Latitude:  -18.318611145,
		Longitude: -54.9355545044,
		Timezone:  "America/Campo_Grande",
	},
	"SSNT": {
		ICAO:      "SSNT",
		Name:      "Fazenda Rosa Mistica Airport",
		City:      "Amambai",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1289,
		Latitude:  -23.2924995422,
		Longitude: -55.0558319092,
		Timezone:  "America/Campo_Grande",
	},
	"SSNU": {
		ICAO:      "SSNU",
		Name:      "Santa Isabel Airport",
		City:      "Brotas",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2582,
		Latitude:  -22.2408332825,
		Longitude: -48.0250015259,
		Timezone:  "America/Sao_Paulo",
	},
	"SSNV": {
		ICAO:      "SSNV",
		Name:      "Fazenda Novo Hamburgo Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 285,
		Latitude:  -19.2330551147,
		Longitude: -57.0297203064,
		Timezone:  "America/Campo_Grande",
	},
	"SSNW": {
		ICAO:      "SSNW",
		Name:      "Fazenda Buriti Airport",
		City:      "Barreiras",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 2408,
		Latitude:  -12.3024997711,
		Longitude: -45.5139007568,
		Timezone:  "America/Bahia",
	},
	"SSNY": {
		ICAO:      "SSNY",
		Name:      "Fazenda Florida Airport",
		City:      "Caceres",
		Country:   "BR",
		Elevation: 541,
		Latitude:  -17.183611,
		Longitude: -58.216667,
		Timezone:  "America/Cuiaba",
	},
	"SSOC": {
		ICAO:      "SSOC",
		Name:      "Fazenda Sans Souci - Sede Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 305,
		Latitude:  -19.0333328247,
		Longitude: -56.9500007629,
		Timezone:  "America/Campo_Grande",
	},
	"SSOD": {
		ICAO:      "SSOD",
		Name:      "Fazenda Pouso Alegre Airport",
		City:      "Miranda",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 374,
		Latitude:  -20.2477779388,
		Longitude: -56.4152793884,
		Timezone:  "America/Campo_Grande",
	},
	"SSOE": {
		ICAO:      "SSOE",
		IATA:      "SQX",
		Name:      "Sao Miguel do Oeste Airport",
		City:      "Sao Miguel Do Oeste",
		State:     "Santa-Catarina",
		Country:   "BR",
		Elevation: 2180,
		Latitude:  -26.7816009521,
		Longitude: -53.5035018921,
		Timezone:  "America/Sao_Paulo",
	},
	"SSOF": {
		ICAO:      "SSOF",
		Name:      "Fazenda Vale da Serra Airport",
		City:      "Monte Azul",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1575,
		Latitude:  -15.1888885498,
		Longitude: -43.5311126709,
		Timezone:  "America/Sao_Paulo",
	},
	"SSOG": {
		ICAO:      "SSOG",
		IATA:      "APX",
		Name:      "Arapongas Airport",
		City:      "Arapongas",
		State:     "Parana",
		Country:   "BR",
		Elevation: 2599,
		Latitude:  -23.3528995514,
		Longitude: -51.4916992188,
		Timezone:  "America/Sao_Paulo",
	},
	"SSOI": {
		ICAO:      "SSOI",
		Name:      "Pederneiras Airport",
		City:      "Pederneiras",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1788,
		Latitude:  -22.3072223663,
		Longitude: -48.7719421387,
		Timezone:  "America/Sao_Paulo",
	},
	"SSOJ": {
		ICAO:      "SSOJ",
		Name:      "Fazenda Estrela Airport",
		City:      "Inocencia",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1509,
		Latitude:  -19.979167,
		Longitude: -51.735,
		Timezone:  "America/Campo_Grande",
	},
	"SSOK": {
		ICAO:      "SSOK",
		Name:      "14 Bis Airport",
		City:      "Londrina",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1932,
		Latitude:  -23.2141666412,
		Longitude: -51.1858329773,
		Timezone:  "America/Sao_Paulo",
	},
	"SSOL": {
		ICAO:      "SSOL",
		Name:      "Lavras Airport",
		City:      "Lavras",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3146,
		Latitude:  -21.2406005859,
		Longitude: -44.9639015198,
		Timezone:  "America/Sao_Paulo",
	},
	"SSON": {
		ICAO:      "SSON",
		Name:      "Fazenda Joalice Airport",
		City:      "Tacuru",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1476,
		Latitude:  -23.7877788544,
		Longitude: -55.0291671753,
		Timezone:  "America/Campo_Grande",
	},
	"SSOO": {
		ICAO:      "SSOO",
		Name:      "Estancia J.D.S Airport",
		City:      "Fernao",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1903,
		Latitude:  -22.3372211456,
		Longitude: -49.543056488,
		Timezone:  "America/Sao_Paulo",
	},
	"SSOP": {
		ICAO:      "SSOP",
		Name:      "Fazenda Cachoeirinha Airport",
		City:      "Aparecida Do Taboado",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1539,
		Latitude:  -20.0433330536,
		Longitude: -51.4747238159,
		Timezone:  "America/Campo_Grande",
	},
	"SSOQ": {
		ICAO:      "SSOQ",
		Name:      "Fazenda Barranco Alto Airport",
		City:      "Aquidauana",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 502,
		Latitude:  -19.5777778625,
		Longitude: -56.1522216797,
		Timezone:  "America/Campo_Grande",
	},
	"SSOR": {
		ICAO:      "SSOR",
		Name:      "Chico Ledur Airport",
		City:      "Alegrete",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 354,
		Latitude:  -29.8180561066,
		Longitude: -55.7141685486,
		Timezone:  "America/Sao_Paulo",
	},
	"SSOS": {
		ICAO:      "SSOS",
		Name:      "Osorio Airport",
		City:      "Osorio",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 39,
		Latitude:  -29.9022216797,
		Longitude: -50.2505569458,
		Timezone:  "America/Sao_Paulo",
	},
	"SSOT": {
		ICAO:      "SSOT",
		Name:      "Fazenda Saco da Tapera Airport",
		City:      "Sao Romao",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1946,
		Latitude:  -16.346944809,
		Longitude: -45.3491668701,
		Timezone:  "America/Sao_Paulo",
	},
	"SSOU": {
		ICAO:      "SSOU",
		IATA:      "AIR",
		Name:      "Fazenda Inhumas Airport",
		City:      "Paracatu",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1772,
		Latitude:  -17.0369434357,
		Longitude: -46.2602767944,
		Timezone:  "America/Sao_Paulo",
	},
	"SSOV": {
		ICAO:      "SSOV",
		Name:      "Fazenda Hora Airport",
		City:      "Urucuia",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1837,
		Latitude:  -16.1466674805,
		Longitude: -45.8250007629,
		Timezone:  "America/Sao_Paulo",
	},
	"SSOW": {
		ICAO:      "SSOW",
		Name:      "Comandante Carmelo Airport",
		City:      "Itaituba",
		State:     "Para",
		Country:   "BR",
		Elevation: 974,
		Latitude:  -6.84,
		Longitude: -56.586667,
		Timezone:  "America/Santarem",
	},
	"SSOY": {
		ICAO:      "SSOY",
		Name:      "FazendaMorada do Sol Airport",
		City:      "Nova Maringa",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 971,
		Latitude:  -12.290556,
		Longitude: -57.019444,
		Timezone:  "America/Cuiaba",
	},
	"SSPB": {
		ICAO:      "SSPB",
		IATA:      "PTO",
		Name:      "Pato Branco Airport",
		City:      "Pato Branco",
		State:     "Parana",
		Country:   "BR",
		Elevation: 2697,
		Latitude:  -26.2178001404,
		Longitude: -52.6943016052,
		Timezone:  "America/Sao_Paulo",
	},
	"SSPC": {
		ICAO:      "SSPC",
		Name:      "Fazenda Santa Julia Airport",
		City:      "Cocalinho",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 866,
		Latitude:  -13.731667,
		Longitude: -51.34,
		Timezone:  "America/Cuiaba",
	},
	"SSPD": {
		ICAO:      "SSPD",
		Name:      "Fazenda Paragua Airport",
		City:      "Comodoro",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 922,
		Latitude:  -14.766667,
		Longitude: -60.033333,
		Timezone:  "America/Cuiaba",
	},
	"SSPE": {
		ICAO:      "SSPE",
		Name:      "Essencia Retiro do Rio Cuiaba Airport",
		City:      "Nobres",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 837,
		Latitude:  -14.435556,
		Longitude: -55.609444,
		Timezone:  "America/Cuiaba",
	},
	"SSPF": {
		ICAO:      "SSPF",
		Name:      "Fazenda Sao Rafael Airport",
		City:      "Sao Jose Das Palmeiras",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1420,
		Latitude:  -24.7695007324,
		Longitude: -54.1186981201,
		Timezone:  "America/Sao_Paulo",
	},
	"SSPG": {
		ICAO:      "SSPG",
		IATA:      "PNG",
		Name:      "Paranagua Airport",
		City:      "Paranagua",
		State:     "Parana",
		Country:   "BR",
		Elevation: 16,
		Latitude:  -25.5401000977,
		Longitude: -48.5312004089,
		Timezone:  "America/Sao_Paulo",
	},
	"SSPI": {
		ICAO:      "SSPI",
		IATA:      "PVI",
		Name:      "Paranavai Airport",
		City:      "Paranavai",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1526,
		Latitude:  -23.0898990631,
		Longitude: -52.4884986877,
		Timezone:  "America/Sao_Paulo",
	},
	"SSPJ": {
		ICAO:      "SSPJ",
		Name:      "Fazenda Reserva Airport",
		City:      "Reserva Do Iguacu",
		State:     "Parana",
		Country:   "BR",
		Elevation: 3438,
		Latitude:  -25.8505554199,
		Longitude: -52.0172233582,
		Timezone:  "America/Sao_Paulo",
	},
	"SSPK": {
		ICAO:      "SSPK",
		Name:      "Porecatu Airport",
		City:      "Porecatu",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1424,
		Latitude:  -22.778799057,
		Longitude: -51.3614006042,
		Timezone:  "America/Sao_Paulo",
	},
	"SSPL": {
		ICAO:      "SSPL",
		Name:      "Palmeira das Missoes Airport",
		City:      "Palmeira Da Missoes",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 2083,
		Latitude:  -27.9018001556,
		Longitude: -53.32970047,
		Timezone:  "America/Sao_Paulo",
	},
	"SSPM": {
		ICAO:      "SSPM",
		Name:      "Porto Murtinho Airport",
		City:      "Porto Murtinho",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 260,
		Latitude:  -21.7096004486,
		Longitude: -57.8801994324,
		Timezone:  "America/Asuncion",
	},
	"SSPN": {
		ICAO:      "SSPN",
		IATA:      "PBB",
		Name:      "Paranaiba Airport",
		City:      "Paranaiba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1446,
		Latitude:  -19.6511993408,
		Longitude: -51.1994018555,
		Timezone:  "America/Campo_Grande",
	},
	"SSPO": {
		ICAO:      "SSPO",
		Name:      "Aplicacao Aviacao Agricola II Airport",
		City:      "Itapora",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1165,
		Latitude:  -22.0091667175,
		Longitude: -54.7897224426,
		Timezone:  "America/Campo_Grande",
	},
	"SSPP": {
		ICAO:      "SSPP",
		Name:      "Fazenda Palma Airport",
		City:      "Pelotas",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 39,
		Latitude:  -31.6569442749,
		Longitude: -52.2077789307,
		Timezone:  "America/Sao_Paulo",
	},
	"SSPQ": {
		ICAO:      "SSPQ",
		Name:      "Santa Rosa do Purus Airport",
		City:      "Santa Rosa Do Purus",
		Country:   "BR",
		Elevation: 751,
		Latitude:  -9.446388889,
		Longitude: -70.48277778,
		Timezone:  "America/Rio_Branco",
	},
	"SSPR": {
		ICAO:      "SSPR",
		Name:      "Porto Walter Airport",
		City:      "Porto Walter",
		State:     "Acre",
		Country:   "BR",
		Elevation: 794,
		Latitude:  -8.271667,
		Longitude: -72.745278,
		Timezone:  "America/Rio_Branco",
	},
	"SSPS": {
		ICAO:      "SSPS",
		Name:      "Palmas Airport",
		City:      "Palmas",
		State:     "Parana",
		Country:   "BR",
		Elevation: 3609,
		Latitude:  -26.4720001221,
		Longitude: -51.9758987427,
		Timezone:  "America/Sao_Paulo",
	},
	"SSPT": {
		ICAO:      "SSPT",
		Name:      "Palotina Airport",
		City:      "Palotina",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1220,
		Latitude:  -24.3432998657,
		Longitude: -53.8287010193,
		Timezone:  "America/Sao_Paulo",
	},
	"SSPV": {
		ICAO:      "SSPV",
		Name:      "Marechal Thaumaturgo Airport",
		City:      "Marechal Thaumaturgo",
		State:     "Acre",
		Country:   "BR",
		Elevation: 751,
		Latitude:  -8.959444,
		Longitude: -72.779722,
		Timezone:  "America/Rio_Branco",
	},
	"SSPX": {
		ICAO:      "SSPX",
		Name:      "Manoel Urbano Airport",
		City:      "Manoel Urbano",
		State:     "Acre",
		Country:   "BR",
		Elevation: 531,
		Latitude:  -8.849167,
		Longitude: -69.266389,
		Timezone:  "America/Rio_Branco",
	},
	"SSPY": {
		ICAO:      "SSPY",
		Name:      "Porto Indio Airport",
		City:      "Corumba",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 390,
		Latitude:  -17.6697216034,
		Longitude: -57.7530555725,
		Timezone:  "America/Campo_Grande",
	},
	"SSQA": {
		ICAO:      "SSQA",
		Name:      "Fazenda Guapore Airport",
		City:      "Comodoro",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 830,
		Latitude:  -13.7230558395,
		Longitude: -60.3152770996,
		Timezone:  "America/Cuiaba",
	},
	"SSQB": {
		ICAO:      "SSQB",
		Name:      "Jordao Airport",
		City:      "Jordao",
		State:     "Acre",
		Country:   "BR",
		Elevation: 1197,
		Latitude:  -9.189444,
		Longitude: -71.946944,
		Timezone:  "America/Rio_Branco",
	},
	"SSQC": {
		ICAO:      "SSQC",
		Name:      "Siqueira Campos Airport",
		City:      "Siqueira Campos",
		State:     "Parana",
		Country:   "BR",
		Elevation: 2313,
		Latitude:  -23.6744995117,
		Longitude: -49.8153991699,
		Timezone:  "America/Sao_Paulo",
	},
	"SSQF": {
		ICAO:      "SSQF",
		Name:      "Fazenda Floresta Negra Airport",
		City:      "Sete Quedas",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1263,
		Latitude:  -23.8141002655,
		Longitude: -54.6987991333,
		Timezone:  "America/Asuncion",
	},
	"SSQI": {
		ICAO:      "SSQI",
		Name:      "Santa Anastacia Airport",
		City:      "Sorriso",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1250,
		Latitude:  -12.5091666667,
		Longitude: -55.6847222222,
		Timezone:  "America/Cuiaba",
	},
	"SSQL": {
		ICAO:      "SSQL",
		Name:      "Fazenda Matricha Airport",
		City:      "Uirapuru",
		State:     "Goias",
		Country:   "BR",
		Elevation: 3609,
		Latitude:  -14.1141672134,
		Longitude: -50.1222229004,
		Timezone:  "America/Sao_Paulo",
	},
	"SSQM": {
		ICAO:      "SSQM",
		IATA:      "0",
		Name:      "Tanque Novo Airport",
		City:      "Tanque Novo",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 2464,
		Latitude:  -13.545800209,
		Longitude: -42.4799995422,
		Timezone:  "America/Bahia",
	},
	"SSQN": {
		ICAO:      "SSQN",
		IATA:      "0",
		Name:      "Mundo Novo Airport",
		City:      "Mundo Novo",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1788,
		Latitude:  -11.7721996307,
		Longitude: -40.4193992615,
		Timezone:  "America/Bahia",
	},
	"SSQO": {
		ICAO:      "SSQO",
		Name:      "Fazenda Itapora Airport",
		City:      "Ribas Do Rio Pardo",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1590,
		Latitude:  -20.0827789307,
		Longitude: -53.8886108398,
		Timezone:  "America/Campo_Grande",
	},
	"SSQP": {
		ICAO:      "SSQP",
		Name:      "Monte Santo Airport",
		City:      "Monte Santo",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1362,
		Latitude:  -10.4783334732,
		Longitude: -39.2874984741,
		Timezone:  "America/Bahia",
	},
	"SSQR": {
		ICAO:      "SSQR",
		Name:      "Fazenda Primavera Airport",
		City:      "Ribas Do Rio Pardo",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1286,
		Latitude:  -21.388611,
		Longitude: -53.600556,
		Timezone:  "America/Campo_Grande",
	},
	"SSQT": {
		ICAO:      "SSQT",
		IATA:      "QAC",
		Name:      "Castro Airport",
		City:      "Castro",
		State:     "Parana",
		Country:   "BR",
		Elevation: 3314,
		Latitude:  -24.8075008392,
		Longitude: -49.9603004456,
		Timezone:  "America/Sao_Paulo",
	},
	"SSQU": {
		ICAO:      "SSQU",
		Name:      "Fazenda Alegrete - Retiro Carabda Airport",
		City:      "Aquidauana",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 410,
		Latitude:  -20.0647220612,
		Longitude: -55.7016677856,
		Timezone:  "America/Campo_Grande",
	},
	"SSQZ": {
		ICAO:      "SSQZ",
		Name:      "Luiz Eduardo Magalhaes Airport",
		City:      "Mimoso Do Oeste",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 2408,
		Latitude:  -12.1066999435,
		Longitude: -45.8945999146,
		Timezone:  "America/Bahia",
	},
	"SSRA": {
		ICAO:      "SSRA",
		Name:      "Fazenda Rancho Belo Airport",
		City:      "Nova Andradina",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1384,
		Latitude:  -21.7083320618,
		Longitude: -53.099445343,
		Timezone:  "America/Campo_Grande",
	},
	"SSRB": {
		ICAO:      "SSRB",
		Name:      "Rio Brilhante Airport",
		City:      "Rio Brilhante",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1024,
		Latitude:  -21.8477783203,
		Longitude: -54.5277786255,
		Timezone:  "America/Campo_Grande",
	},
	"SSRC": {
		ICAO:      "SSRC",
		Name:      "Fazenda Santa Ines Airport",
		City:      "Rio Brilhante",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1084,
		Latitude:  -21.7030563354,
		Longitude: -54.3683319092,
		Timezone:  "America/Campo_Grande",
	},
	"SSRD": {
		ICAO:      "SSRD",
		Name:      "Fazenda Sol Nascente Airport",
		City:      "Tacuru",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1443,
		Latitude:  -23.3974990845,
		Longitude: -55.2483329773,
		Timezone:  "America/Campo_Grande",
	},
	"SSRE": {
		ICAO:      "SSRE",
		Name:      "Realeza Airport",
		City:      "Realeza",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1693,
		Latitude:  -25.8012008667,
		Longitude: -53.5126991272,
		Timezone:  "America/Sao_Paulo",
	},
	"SSRF": {
		ICAO:      "SSRF",
		Name:      "Castro Alves Airport",
		City:      "Castro Alves",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 869,
		Latitude:  -12.7657003403,
		Longitude: -39.4467010498,
		Timezone:  "America/Bahia",
	},
	"SSRG": {
		ICAO:      "SSRG",
		Name:      "Fazenda Rancho Grande Airport",
		City:      "Laguna Carapa",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1319,
		Latitude:  -22.7766666412,
		Longitude: -54.9766654968,
		Timezone:  "America/Campo_Grande",
	},
	"SSRH": {
		ICAO:      "SSRH",
		Name:      "Fazenda Retiro do Marimbondo Airport",
		City:      "Anastacio",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 577,
		Latitude:  -20.778055191,
		Longitude: -55.9827766418,
		Timezone:  "America/Campo_Grande",
	},
	"SSRI": {
		ICAO:      "SSRI",
		Name:      "Fazenda Dos Sonhos Airport",
		City:      "Anastacio",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 551,
		Latitude:  -20.7588882446,
		Longitude: -56.0325012207,
		Timezone:  "America/Campo_Grande",
	},
	"SSRJ": {
		ICAO:      "SSRJ",
		Name:      "Fazenda Jequitiba Airport",
		City:      "Nova Alvorada Do Sul",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1181,
		Latitude:  -21.8733329773,
		Longitude: -53.8127784729,
		Timezone:  "America/Campo_Grande",
	},
	"SSRK": {
		ICAO:      "SSRK",
		Name:      "Campo Alegre de Lourdes Airport",
		City:      "Campo Alegre De Lourdes",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 1519,
		Latitude:  -9.5139102936,
		Longitude: -42.995098114,
		Timezone:  "America/Bahia",
	},
	"SSRL": {
		ICAO:      "SSRL",
		Name:      "Fazenda Santa Ana Airport",
		City:      "Deodapolis",
		Country:   "BR",
		Elevation: 1001,
		Latitude:  -22.0372219086,
		Longitude: -54.1888885498,
		Timezone:  "America/Campo_Grande",
	},
	"SSRM": {
		ICAO:      "SSRM",
		Name:      "Fazenda Sao Sebastiao Airport",
		City:      "Bonito",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 791,
		Latitude:  -21.019443512,
		Longitude: -56.2761116028,
		Timezone:  "America/Campo_Grande",
	},
	"SSRN": {
		ICAO:      "SSRN",
		Name:      "Aeroclube de Rio Negrinho Airport",
		City:      "Rio Negrinho",
		State:     "Parana",
		Country:   "BR",
		Elevation: 3025,
		Latitude:  -26.3199996948,
		Longitude: -49.5205574036,
		Timezone:  "America/Sao_Paulo",
	},
	"SSRO": {
		ICAO:      "SSRO",
		Name:      "Fazenda Santa Rosa Airport",
		City:      "Navirai",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1217,
		Latitude:  -22.8916664124,
		Longitude: -54.1169433594,
		Timezone:  "America/Campo_Grande",
	},
	"SSRQ": {
		ICAO:      "SSRQ",
		Name:      "Refugio Airport",
		City:      "Dom Pedrito",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 709,
		Latitude:  -30.9191665649,
		Longitude: -54.4677772522,
		Timezone:  "America/Sao_Paulo",
	},
	"SSRS": {
		ICAO:      "SSRS",
		Name:      "Rancho Sumidor Airport",
		City:      "Rio Do Oeste",
		State:     "Santa-Catarina",
		Country:   "BR",
		Elevation: 1476,
		Latitude:  -27.1919441223,
		Longitude: -49.8313903809,
		Timezone:  "America/Sao_Paulo",
	},
	"SSRT": {
		ICAO:      "SSRT",
		Name:      "Fazenda Mangal Airport",
		City:      "Curua",
		State:     "Para",
		Country:   "BR",
		Elevation: 289,
		Latitude:  -1.5413889885,
		Longitude: -55.180557251,
		Timezone:  "America/Santarem",
	},
	"SSRU": {
		ICAO:      "SSRU",
		IATA:      "SQY",
		Name:      "Sao Lourenco do Sul Airport",
		City:      "Sao Lourenco Do Sul",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 28,
		Latitude:  -31.3833007813,
		Longitude: -52.0327987671,
		Timezone:  "America/Sao_Paulo",
	},
	"SSRX": {
		ICAO:      "SSRX",
		Name:      "Fazenda Marrua Airport",
		City:      "Ribeirao Cascalheira",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 591,
		Latitude:  -12.779999733,
		Longitude: -51.11277771,
		Timezone:  "America/Cuiaba",
	},
	"SSRY": {
		ICAO:      "SSRY",
		Name:      "Rio Pardo Airport",
		City:      "Rio Pardo",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 230,
		Latitude:  -29.9727783203,
		Longitude: -52.3611106873,
		Timezone:  "America/Sao_Paulo",
	},
	"SSRZ": {
		ICAO:      "SSRZ",
		Name:      "Rosario do Sul Airport",
		City:      "Rosario Do Sul",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 348,
		Latitude:  -30.2775001526,
		Longitude: -54.92527771,
		Timezone:  "America/Sao_Paulo",
	},
	"SSSA": {
		ICAO:      "SSSA",
		Name:      "Fazenda Sant`anna do Apa Airport",
		City:      "Bela Vista",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1047,
		Latitude:  -22.0266666412,
		Longitude: -56.193611145,
		Timezone:  "America/Campo_Grande",
	},
	"SSSB": {
		ICAO:      "SSSB",
		IATA:      "QOJ",
		Name:      "Sao Borja Airport",
		City:      "Sao Borja",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 246,
		Latitude:  -28.6548995972,
		Longitude: -56.0345993042,
		Timezone:  "America/Sao_Paulo",
	},
	"SSSC": {
		ICAO:      "SSSC",
		IATA:      "CSU",
		Name:      "Santa Cruz do Sul Airport",
		City:      "Santa Cruz Do Sul",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 646,
		Latitude:  -29.6840991974,
		Longitude: -52.4122009277,
		Timezone:  "America/Sao_Paulo",
	},
	"SSSD": {
		ICAO:      "SSSD",
		IATA:      "0",
		Name:      "Soledade Airport",
		City:      "Soledade",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 2165,
		Latitude:  -28.8631000519,
		Longitude: -52.540599823,
		Timezone:  "America/Sao_Paulo",
	},
	"SSSG": {
		ICAO:      "SSSG",
		IATA:      "0",
		Name:      "Sao Gabriel Airport",
		City:      "Sao Gabriel",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 472,
		Latitude:  -30.3449993134,
		Longitude: -54.2677993774,
		Timezone:  "America/Sao_Paulo",
	},
	"SSSH": {
		ICAO:      "SSSH",
		Name:      "Fazenda Santo Inacio Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 275,
		Latitude:  -19.1991672516,
		Longitude: -57.0111122131,
		Timezone:  "America/Campo_Grande",
	},
	"SSSO": {
		ICAO:      "SSSO",
		Name:      "Fazenda Santa Odila Airport",
		City:      "Brasilandia",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 984,
		Latitude:  -21.4111118317,
		Longitude: -52.0325012207,
		Timezone:  "America/Campo_Grande",
	},
	"SSSP": {
		ICAO:      "SSSP",
		Name:      "Fazenda Santa Delfina Airport",
		City:      "Miranda",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 328,
		Latitude:  -20.0402774811,
		Longitude: -56.3100013733,
		Timezone:  "America/Campo_Grande",
	},
	"SSSQ": {
		ICAO:      "SSSQ",
		IATA:      "0",
		Name:      "Sao Joaquim Airport",
		City:      "Sao Joaquim",
		State:     "Santa-Catarina",
		Country:   "BR",
		Elevation: 4462,
		Latitude:  -28.2588996887,
		Longitude: -49.9157981873,
		Timezone:  "America/Sao_Paulo",
	},
	"SSSR": {
		ICAO:      "SSSR",
		Name:      "Fazenda Santa Rosa Airport",
		City:      "Agua Clara",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1476,
		Latitude:  -20.3163890839,
		Longitude: -52.7333335876,
		Timezone:  "America/Campo_Grande",
	},
	"SSSS": {
		ICAO:      "SSSS",
		Name:      "Sao Francisco do Sul Airport",
		City:      "Sao Francisco Do Sul",
		State:     "Santa-Catarina",
		Country:   "BR",
		Elevation: 49,
		Latitude:  -26.2211112976,
		Longitude: -48.5644454956,
		Timezone:  "America/Sao_Paulo",
	},
	"SSST": {
		ICAO:      "SSST",
		Name:      "Santiago Airport",
		City:      "Santiago",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 1460,
		Latitude:  -29.2150001526,
		Longitude: -54.8422203064,
		Timezone:  "America/Sao_Paulo",
	},
	"SSSW": {
		ICAO:      "SSSW",
		Name:      "Fazenda Santa Maria de Inocencia Airport",
		City:      "Inocencia",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1575,
		Latitude:  -19.7211112976,
		Longitude: -51.9502792358,
		Timezone:  "America/Campo_Grande",
	},
	"SSSX": {
		ICAO:      "SSSX",
		Name:      "Fazenda Sans Souci - Retiro Mutum Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 300,
		Latitude:  -18.9333324432,
		Longitude: -57.0166664124,
		Timezone:  "America/Campo_Grande",
	},
	"SSSY": {
		ICAO:      "SSSY",
		Name:      "Fazenda Silvana Airport",
		City:      "Nova Andradina",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 781,
		Latitude:  -22.5166664124,
		Longitude: -53.5116653442,
		Timezone:  "America/Campo_Grande",
	},
	"SSSZ": {
		ICAO:      "SSSZ",
		IATA:      "0",
		Name:      "Sertanopolis Airport",
		City:      "Sertanopolis",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1181,
		Latitude:  -23.0622005463,
		Longitude: -51.0139007568,
		Timezone:  "America/Sao_Paulo",
	},
	"SSTA": {
		ICAO:      "SSTA",
		Name:      "Santa Maria Airport",
		City:      "Sao Gabriel",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 709,
		Latitude:  -30.2458324432,
		Longitude: -54.8008346558,
		Timezone:  "America/Sao_Paulo",
	},
	"SSTB": {
		ICAO:      "SSTB",
		Name:      "Tres Barras Airport",
		City:      "Tres Barras",
		State:     "Santa-Catarina",
		Country:   "BR",
		Elevation: 2559,
		Latitude:  -26.135799408,
		Longitude: -50.3109016418,
		Timezone:  "America/Sao_Paulo",
	},
	"SSTD": {
		ICAO:      "SSTD",
		Name:      "Fazenda Arco de Canto Airport",
		City:      "Andre Da Rocha",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 2311,
		Latitude:  -28.5508327484,
		Longitude: -51.5961112976,
		Timezone:  "America/Sao_Paulo",
	},
	"SSTF": {
		ICAO:      "SSTF",
		Name:      "Fazenda Triunfo Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 328,
		Latitude:  -17.8724994659,
		Longitude: -57.1300010681,
		Timezone:  "America/Campo_Grande",
	},
	"SSTG": {
		ICAO:      "SSTG",
		Name:      "Fazenda Santa Terezinha Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 525,
		Latitude:  -18.4694442749,
		Longitude: -57.0550003052,
		Timezone:  "America/Campo_Grande",
	},
	"SSTH": {
		ICAO:      "SSTH",
		Name:      "Fazenda Thalia Airport",
		City:      "Balsa Nova",
		State:     "Parana",
		Country:   "BR",
		Elevation: 3346,
		Latitude:  -25.5205001831,
		Longitude: -49.6819992065,
		Timezone:  "America/Sao_Paulo",
	},
	"SSTI": {
		ICAO:      "SSTI",
		Name:      "Arraial d`Ajuda Airport",
		City:      "Porto Seguro",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 279,
		Latitude:  -16.457778,
		Longitude: -39.165833,
		Timezone:  "America/Bahia",
	},
	"SSTJ": {
		ICAO:      "SSTJ",
		Name:      "Walter Ewaldo Siegel Airport",
		City:      "Trombudo Central",
		State:     "Santa-Catarina",
		Country:   "BR",
		Elevation: 1322,
		Latitude:  -27.3491668701,
		Longitude: -49.818611145,
		Timezone:  "America/Sao_Paulo",
	},
	"SSTK": {
		ICAO:      "SSTK",
		Name:      "Fazenda Uniao de Iguatemi Airport",
		City:      "Iguatemi",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 794,
		Latitude:  -23.6827774048,
		Longitude: -54.5397224426,
		Timezone:  "America/Campo_Grande",
	},
	"SSTL": {
		ICAO:      "SSTL",
		Name:      "Tres Lagoas Airport",
		City:      "Tres Lagoas",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1050,
		Latitude:  -20.7541999817,
		Longitude: -51.6842002869,
		Timezone:  "America/Campo_Grande",
	},
	"SSTN": {
		ICAO:      "SSTN",
		Name:      "Alberto Bertelli Airport",
		City:      "Araraquara",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2283,
		Latitude:  -21.6786117554,
		Longitude: -48.2119445801,
		Timezone:  "America/Sao_Paulo",
	},
	"SSTO": {
		ICAO:      "SSTO",
		IATA:      "0",
		Name:      "Tres Passos Airport",
		City:      "Tres Passos",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 1503,
		Latitude:  -27.5125007629,
		Longitude: -53.8983001709,
		Timezone:  "America/Sao_Paulo",
	},
	"SSTP": {
		ICAO:      "SSTP",
		Name:      "Fazenda Baessa Airport",
		City:      "Turvelandia",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1591,
		Latitude:  -17.970833,
		Longitude: -50.320556,
		Timezone:  "America/Sao_Paulo",
	},
	"SSTQ": {
		ICAO:      "SSTQ",
		Name:      "Silvestre Airport",
		City:      "Pedra Branca Do Amapari",
		State:     "Amapa",
		Country:   "BR",
		Elevation: 459,
		Latitude:  0.886389,
		Longitude: -51.879444,
		Timezone:  "America/Belem",
	},
	"SSTS": {
		ICAO:      "SSTS",
		Name:      "Fazenda Bandeirantes Airport",
		City:      "Taquarussu",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 813,
		Latitude:  -22.6088886261,
		Longitude: -53.4472236633,
		Timezone:  "America/Campo_Grande",
	},
	"SSTU": {
		ICAO:      "SSTU",
		Name:      "Fazenda Jauru Airport",
		City:      "Camapua",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1250,
		Latitude:  -18.7299995422,
		Longitude: -54.1049995422,
		Timezone:  "America/Campo_Grande",
	},
	"SSTY": {
		ICAO:      "SSTY",
		Name:      "Usina Foz do Chopim Airport",
		City:      "Cruzeiro Do Iguacu",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1640,
		Latitude:  -25.5625,
		Longitude: -53.1769447327,
		Timezone:  "America/Sao_Paulo",
	},
	"SSUA": {
		ICAO:      "SSUA",
		Name:      "Fazenda Sonora Estancia Airport",
		City:      "Sonora",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1575,
		Latitude:  -17.631111145,
		Longitude: -54.7700004578,
		Timezone:  "America/Campo_Grande",
	},
	"SSUC": {
		ICAO:      "SSUC",
		Name:      "Fazenda Aurora II Airport",
		City:      "Ribas Do Rio Pardo",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1194,
		Latitude:  -21.4911117554,
		Longitude: -53.2038879395,
		Timezone:  "America/Campo_Grande",
	},
	"SSUL": {
		ICAO:      "SSUL",
		Name:      "Ultraleve Clube Curitiba Airport",
		City:      "Sao Jose Dos Pinhais",
		State:     "Parana",
		Country:   "BR",
		Elevation: 3215,
		Latitude:  -25.4769439697,
		Longitude: -49.1541671753,
		Timezone:  "America/Sao_Paulo",
	},
	"SSUM": {
		ICAO:      "SSUM",
		IATA:      "UMU",
		Name:      "Umuarama Airport",
		City:      "Umuarama",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1558,
		Latitude:  -23.7987003326,
		Longitude: -53.3138008118,
		Timezone:  "America/Sao_Paulo",
	},
	"SSUQ": {
		ICAO:      "SSUQ",
		Name:      "Fazenda Santa Olinda Airport",
		City:      "Sidrolandia",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 915,
		Latitude:  -20.7549991608,
		Longitude: -55.0433311462,
		Timezone:  "America/Campo_Grande",
	},
	"SSUR": {
		ICAO:      "SSUR",
		Name:      "Fazenda Agua Branca Airport",
		City:      "Santa Rita Do Pardo",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1264,
		Latitude:  -21.5836105347,
		Longitude: -52.8722229004,
		Timezone:  "America/Campo_Grande",
	},
	"SSUT": {
		ICAO:      "SSUT",
		Name:      "Fazenda Agrofel Airport",
		City:      "Tangara Da Serra",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 705,
		Latitude:  -14.4444437027,
		Longitude: -58.2977790833,
		Timezone:  "America/Cuiaba",
	},
	"SSUU": {
		ICAO:      "SSUU",
		Name:      "Fazenda Colorado Airport",
		City:      "Lagoa Da Confusao",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 761,
		Latitude:  -10.7416667938,
		Longitude: -49.4799995422,
		Timezone:  "America/Araguaina",
	},
	"SSUV": {
		ICAO:      "SSUV",
		IATA:      "QVB",
		Name:      "Uniao da Vitoria Airport",
		City:      "Uniao Da Vitoria",
		State:     "Santa-Catarina",
		Country:   "BR",
		Elevation: 2467,
		Latitude:  -26.2334003448,
		Longitude: -51.0677986145,
		Timezone:  "America/Sao_Paulo",
	},
	"SSUX": {
		ICAO:      "SSUX",
		IATA:      "0",
		Name:      "Usina Mandu Airport",
		City:      "Guaira",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1597,
		Latitude:  -20.4822006226,
		Longitude: -48.3727989197,
		Timezone:  "America/Sao_Paulo",
	},
	"SSUY": {
		ICAO:      "SSUY",
		Name:      "Fazenda Menina Airport",
		City:      "Itapura",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1115,
		Latitude:  -20.640499115,
		Longitude: -51.4768981934,
		Timezone:  "America/Sao_Paulo",
	},
	"SSUZ": {
		ICAO:      "SSUZ",
		Name:      "Fazenda Vista Alegre Airport",
		City:      "Sud Menucci",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1322,
		Latitude:  -20.6616668701,
		Longitude: -50.8416671753,
		Timezone:  "America/Sao_Paulo",
	},
	"SSVA": {
		ICAO:      "SSVA",
		Name:      "Fazenda Visa Estancia Airport",
		City:      "Ladario",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 805,
		Latitude:  -19.0625,
		Longitude: -57.6291656494,
		Timezone:  "America/Campo_Grande",
	},
	"SSVB": {
		ICAO:      "SSVB",
		Name:      "Fazenda Vaca Branca Airport",
		City:      "Navirai",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1086,
		Latitude:  -23.0672225952,
		Longitude: -53.8199996948,
		Timezone:  "America/Campo_Grande",
	},
	"SSVC": {
		ICAO:      "SSVC",
		Name:      "Vacaria Airport",
		City:      "Vacaria",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 3215,
		Latitude:  -28.509721756,
		Longitude: -50.9183311462,
		Timezone:  "America/Sao_Paulo",
	},
	"SSVD": {
		ICAO:      "SSVD",
		Name:      "Fazenda Alvorada Airport",
		City:      "Quarto Centenario",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1312,
		Latitude:  -24.2947216034,
		Longitude: -53.25,
		Timezone:  "America/Sao_Paulo",
	},
	"SSVE": {
		ICAO:      "SSVE",
		Name:      "Fazenda Cerro Verde Airport",
		City:      "Tacuru",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1250,
		Latitude:  -23.8127784729,
		Longitude: -55.0577774048,
		Timezone:  "America/Campo_Grande",
	},
	"SSVF": {
		ICAO:      "SSVF",
		Name:      "Rancho Donana Airport",
		City:      "Cerqueira Cesar",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2257,
		Latitude:  -22.9619445801,
		Longitude: -49.1483345032,
		Timezone:  "America/Sao_Paulo",
	},
	"SSVG": {
		ICAO:      "SSVG",
		Name:      "Fazenda Santa Virginia Airport",
		City:      "Ponta Pora",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 2016,
		Latitude:  -22.2838897705,
		Longitude: -55.6533317566,
		Timezone:  "America/Campo_Grande",
	},
	"SSVH": {
		ICAO:      "SSVH",
		Name:      "Fazenda Paqueta Airport",
		City:      "Ponta Pora",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1398,
		Latitude:  -22.3669433594,
		Longitude: -55.1277770996,
		Timezone:  "America/Campo_Grande",
	},
	"SSVI": {
		ICAO:      "SSVI",
		IATA:      "VIA",
		Name:      "Videira Airport",
		City:      "Videira",
		State:     "Santa-Catarina",
		Country:   "BR",
		Elevation: 2756,
		Latitude:  -26.9997005463,
		Longitude: -51.1418991089,
		Timezone:  "America/Sao_Paulo",
	},
	"SSVJ": {
		ICAO:      "SSVJ",
		Name:      "Fazenda Cedro Airport",
		City:      "Ponta Pora",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 2280,
		Latitude:  -21.8708324432,
		Longitude: -55.7475013733,
		Timezone:  "America/Campo_Grande",
	},
	"SSVK": {
		ICAO:      "SSVK",
		Name:      "Fazenda Dom Arlindo Airport",
		City:      "Navirai",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1299,
		Latitude:  -22.7944431305,
		Longitude: -54.1569442749,
		Timezone:  "America/Campo_Grande",
	},
	"SSVL": {
		ICAO:      "SSVL",
		Name:      "Fazenda Santa Claudina Airport",
		City:      "Caarapo",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1476,
		Latitude:  -22.5216674805,
		Longitude: -54.6777763367,
		Timezone:  "America/Campo_Grande",
	},
	"SSVM": {
		ICAO:      "SSVM",
		Name:      "Fazenda Santa Alaide Airport",
		City:      "Itapora",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1499,
		Latitude:  -22.0025005341,
		Longitude: -54.9741668701,
		Timezone:  "America/Campo_Grande",
	},
	"SSVN": {
		ICAO:      "SSVN",
		IATA:      "0",
		Name:      "Veranopolis Airport",
		City:      "Veranopolis",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 2133,
		Latitude:  -28.9349994659,
		Longitude: -51.5682983398,
		Timezone:  "America/Sao_Paulo",
	},
	"SSVO": {
		ICAO:      "SSVO",
		Name:      "Usina de Moncoes Airport",
		City:      "Moncoes",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1437,
		Latitude:  -20.8716678619,
		Longitude: -50.1269454956,
		Timezone:  "America/Sao_Paulo",
	},
	"SSVP": {
		ICAO:      "SSVP",
		IATA:      "CTQ",
		Name:      "Santa Vitoria do Palmar Airport",
		City:      "Santa Vitoria Do Palmar",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 82,
		Latitude:  -33.5022239685,
		Longitude: -53.344165802,
		Timezone:  "America/Sao_Paulo",
	},
	"SSVQ": {
		ICAO:      "SSVQ",
		Name:      "Fazenda Serrito Airport",
		City:      "Laguna Carapa",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1510,
		Latitude:  -22.4102783203,
		Longitude: -54.988609314,
		Timezone:  "America/Campo_Grande",
	},
	"SSVU": {
		ICAO:      "SSVU",
		Name:      "Fazenda Serra do Cabral Airport",
		City:      "Lassance",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3773,
		Latitude:  -17.698889,
		Longitude: -44.350278,
		Timezone:  "America/Sao_Paulo",
	},
	"SSVW": {
		ICAO:      "SSVW",
		Name:      "Fazenda Galera Airport",
		City:      "Conquista D`Oeste",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 975,
		Latitude:  -14.5130558014,
		Longitude: -59.5569458008,
		Timezone:  "America/Cuiaba",
	},
	"SSVX": {
		ICAO:      "SSVX",
		Name:      "Fazenda Manduri Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 394,
		Latitude:  -18.5100002289,
		Longitude: -56.4288902283,
		Timezone:  "America/Campo_Grande",
	},
	"SSVY": {
		ICAO:      "SSVY",
		IATA:      "0",
		Name:      "Fazenda Tres Barras Airport",
		City:      "Pitangueiras",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1575,
		Latitude:  -20.9377994537,
		Longitude: -48.1733016968,
		Timezone:  "America/Sao_Paulo",
	},
	"SSWA": {
		ICAO:      "SSWA",
		IATA:      "0",
		Name:      "Venancio Aires Airport",
		City:      "Venancio Aires",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 226,
		Latitude:  -29.6131000519,
		Longitude: -52.2172012329,
		Timezone:  "America/Sao_Paulo",
	},
	"SSWC": {
		ICAO:      "SSWC",
		Name:      "Fazenda Campo Verde Airport",
		City:      "Costa Rica",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 2608,
		Latitude:  -18.9330558777,
		Longitude: -53.069442749,
		Timezone:  "America/Campo_Grande",
	},
	"SSWE": {
		ICAO:      "SSWE",
		Name:      "Fazenda California Airport",
		City:      "Inubia Paulista",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1450,
		Latitude:  -21.743888855,
		Longitude: -50.9474983215,
		Timezone:  "America/Sao_Paulo",
	},
	"SSWF": {
		ICAO:      "SSWF",
		Name:      "Frederico Westphalen Airport",
		City:      "Frederico Westphalen",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 1857,
		Latitude:  -27.3477783203,
		Longitude: -53.4025001526,
		Timezone:  "America/Sao_Paulo",
	},
	"SSWH": {
		ICAO:      "SSWH",
		Name:      "Fazenda Agropecuaria Rossignolo Airport",
		City:      "Sao Carlos",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2300,
		Latitude:  -21.912222,
		Longitude: -47.823611,
		Timezone:  "America/Sao_Paulo",
	},
	"SSWK": {
		ICAO:      "SSWK",
		Name:      "Guaiacuruzinho Airport",
		City:      "Santa Mariana",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1148,
		Latitude:  -23.0319442749,
		Longitude: -50.4958343506,
		Timezone:  "America/Sao_Paulo",
	},
	"SSWL": {
		ICAO:      "SSWL",
		Name:      "Haras Fazenda Reunidas Sobral Airport",
		City:      "Santa Isabel Do Para",
		State:     "Para",
		Country:   "BR",
		Elevation: 115,
		Latitude:  -1.2847219706,
		Longitude: -48.213054657,
		Timezone:  "America/Belem",
	},
	"SSWM": {
		ICAO:      "SSWM",
		Name:      "Fazenda Santa Maria Airport",
		City:      "Londrina",
		State:     "Parana",
		Country:   "BR",
		Elevation: 2461,
		Latitude:  -23.5,
		Longitude: -51.2999992371,
		Timezone:  "America/Sao_Paulo",
	},
	"SSWN": {
		ICAO:      "SSWN",
		Name:      "Fazenda Figueira Branca Airport",
		City:      "Caceres",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 820,
		Latitude:  -16.2283325195,
		Longitude: -57.5922203064,
		Timezone:  "America/Cuiaba",
	},
	"SSWO": {
		ICAO:      "SSWO",
		Name:      "Fazenda Santa Marta Airport",
		City:      "Navirai",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1083,
		Latitude:  -22.9663887024,
		Longitude: -54.0758323669,
		Timezone:  "America/Campo_Grande",
	},
	"SSWP": {
		ICAO:      "SSWP",
		Name:      "Fazenda Araguaia Airport",
		City:      "Santa Maria Das Barreiras",
		State:     "Para",
		Country:   "BR",
		Elevation: 797,
		Latitude:  -8.7425003052,
		Longitude: -50.2075004578,
		Timezone:  "America/Belem",
	},
	"SSWQ": {
		ICAO:      "SSWQ",
		Name:      "Santa Anastacia II Airport",
		City:      "Sorriso",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1283,
		Latitude:  -12.5275,
		Longitude: -55.6247222222,
		Timezone:  "America/Cuiaba",
	},
	"SSWR": {
		ICAO:      "SSWR",
		Name:      "Fazenda Campana Airport",
		City:      "Rio Brilhante",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1113,
		Latitude:  -21.8880558014,
		Longitude: -54.2952766418,
		Timezone:  "America/Campo_Grande",
	},
	"SSWS": {
		ICAO:      "SSWS",
		IATA:      "0",
		Name:      "Cacapava do Sul Airport",
		City:      "Cacapava Do Sul",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 1312,
		Latitude:  -30.5494003296,
		Longitude: -53.4592018127,
		Timezone:  "America/Sao_Paulo",
	},
	"SSWT": {
		ICAO:      "SSWT",
		Name:      "Cambara Airport",
		City:      "Cambara Do Sul",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 3527,
		Latitude:  -28.9997215271,
		Longitude: -50.0761108398,
		Timezone:  "America/Sao_Paulo",
	},
	"SSWV": {
		ICAO:      "SSWV",
		Name:      "Fazenda Barra Grande Airport",
		City:      "Tomazinha",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1772,
		Latitude:  -23.590555191,
		Longitude: -50.0411109924,
		Timezone:  "America/Sao_Paulo",
	},
	"SSWW": {
		ICAO:      "SSWW",
		Name:      "Fazenda Belem Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 262,
		Latitude:  -18.6741676331,
		Longitude: -55.8947219849,
		Timezone:  "America/Campo_Grande",
	},
	"SSWX": {
		ICAO:      "SSWX",
		Name:      "Fazenda Retiro Campanha Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 262,
		Latitude:  -19.028055191,
		Longitude: -56.2905578613,
		Timezone:  "America/Campo_Grande",
	},
	"SSWZ": {
		ICAO:      "SSWZ",
		Name:      "Fazenda Dona Nenem Airport",
		City:      "Presidente Olegario",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 3458,
		Latitude:  -18.5094432831,
		Longitude: -46.2738876343,
		Timezone:  "America/Sao_Paulo",
	},
	"SSXA": {
		ICAO:      "SSXA",
		Name:      "Fazenda Congonhas Airport",
		City:      "Rancho Alegre",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1526,
		Latitude:  -23.0386104584,
		Longitude: -50.9344444275,
		Timezone:  "America/Sao_Paulo",
	},
	"SSXB": {
		ICAO:      "SSXB",
		Name:      "Fazenda Alaska II Airport",
		City:      "Barao De Melgaco",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 551,
		Latitude:  -17.1602993011,
		Longitude: -56.5696983337,
		Timezone:  "America/Cuiaba",
	},
	"SSXC": {
		ICAO:      "SSXC",
		Name:      "Fazenda Alaska I Airport",
		City:      "Barao De Melgaco",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 571,
		Latitude:  -17.0499992371,
		Longitude: -56.0833320618,
		Timezone:  "America/Cuiaba",
	},
	"SSXD": {
		ICAO:      "SSXD",
		IATA:      "0",
		Name:      "Sarandi Airport",
		City:      "Sarandi",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 2034,
		Latitude:  -27.9811000824,
		Longitude: -52.8636016846,
		Timezone:  "America/Sao_Paulo",
	},
	"SSXH": {
		ICAO:      "SSXH",
		Name:      "Fazenda Jacamin Airport",
		City:      "Nova Mutum",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1099,
		Latitude:  -13.698056221,
		Longitude: -56.2908325195,
		Timezone:  "America/Cuiaba",
	},
	"SSXI": {
		ICAO:      "SSXI",
		Name:      "Fazenda Macisa Airport",
		City:      "Tabapora",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1201,
		Latitude:  -11.179444313,
		Longitude: -56.7911109924,
		Timezone:  "America/Cuiaba",
	},
	"SSXK": {
		ICAO:      "SSXK",
		Name:      "Fazenda Alvorada do Marape Airport",
		City:      "Tapura",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1001,
		Latitude:  -12.8719444275,
		Longitude: -56.7419433594,
		Timezone:  "America/Cuiaba",
	},
	"SSXL": {
		ICAO:      "SSXL",
		Name:      "Laguna Airport",
		City:      "Miranda",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 554,
		Latitude:  -20.4347229004,
		Longitude: -56.3358345032,
		Timezone:  "America/Campo_Grande",
	},
	"SSXM": {
		ICAO:      "SSXM",
		Name:      "Fazenda Guaciara Airport",
		City:      "Jatei",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1443,
		Latitude:  -22.7866668701,
		Longitude: -53.8827781677,
		Timezone:  "America/Campo_Grande",
	},
	"SSXN": {
		ICAO:      "SSXN",
		Name:      "Aereo Amazonia Airport",
		City:      "Altamira",
		State:     "Para",
		Country:   "BR",
		Elevation: 820,
		Latitude:  -8.298333333,
		Longitude: -55.11472222,
		Timezone:  "America/Santarem",
	},
	"SSXO": {
		ICAO:      "SSXO",
		Name:      "Pousada das Aguias Airport",
		City:      "Presidente Castelo Branco",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1932,
		Latitude:  -23.2888889313,
		Longitude: -52.1291656494,
		Timezone:  "America/Sao_Paulo",
	},
	"SSXP": {
		ICAO:      "SSXP",
		Name:      "Fazenda Vitoria Airport",
		City:      "Centro Novo",
		State:     "Para",
		Country:   "BR",
		Elevation: 523,
		Latitude:  -3.6202778816,
		Longitude: -48.756111145,
		Timezone:  "America/Belem",
	},
	"SSXQ": {
		ICAO:      "SSXQ",
		Name:      "Fazenda Jota III Airport",
		City:      "Inocencia",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1466,
		Latitude:  -19.3286113739,
		Longitude: -51.9738883972,
		Timezone:  "America/Campo_Grande",
	},
	"SSXR": {
		ICAO:      "SSXR",
		Name:      "Galletti II Airport",
		City:      "Portel",
		State:     "Para",
		Country:   "BR",
		Elevation: 974,
		Latitude:  -2.9513890743,
		Longitude: -51.1461105347,
		Timezone:  "America/Belem",
	},
	"SSXU": {
		ICAO:      "SSXU",
		Name:      "Fazenda Fartura Airport",
		City:      "Tome Acu",
		State:     "Para",
		Country:   "BR",
		Elevation: 276,
		Latitude:  -3.1577780247,
		Longitude: -48.5400009155,
		Timezone:  "America/Belem",
	},
	"SSXV": {
		ICAO:      "SSXV",
		Name:      "Fazenda Santa Rita Airport",
		City:      "Navirai",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1201,
		Latitude:  -22.9500007629,
		Longitude: -54.1122207642,
		Timezone:  "America/Campo_Grande",
	},
	"SSXW": {
		ICAO:      "SSXW",
		Name:      "Aero Porto Dal Lago Airport",
		City:      "Chapadao Do Sul",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2625,
		Latitude:  -18.8077774048,
		Longitude: -52.600276947,
		Timezone:  "America/Campo_Grande",
	},
	"SSXX": {
		ICAO:      "SSXX",
		IATA:      "AXE",
		Name:      "Xanxere Airport",
		City:      "Xanxere",
		State:     "Santa-Catarina",
		Country:   "BR",
		Elevation: 2986,
		Latitude:  -26.8755569458,
		Longitude: -52.3730545044,
		Timezone:  "America/Sao_Paulo",
	},
	"SSXY": {
		ICAO:      "SSXY",
		Name:      "Barradao Airport",
		City:      "Colniza",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 459,
		Latitude:  -9.6197223663,
		Longitude: -60.6674995422,
		Timezone:  "America/Cuiaba",
	},
	"SSXZ": {
		ICAO:      "SSXZ",
		Name:      "Fazenda Santa Rosa Airport",
		City:      "Iturama",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1348,
		Latitude:  -19.7474994659,
		Longitude: -50.1291656494,
		Timezone:  "America/Sao_Paulo",
	},
	"SSYA": {
		ICAO:      "SSYA",
		IATA:      "AAG",
		Name:      "Arapoti Airport",
		City:      "Arapoti",
		State:     "Parana",
		Country:   "BR",
		Elevation: 2641,
		Latitude:  -24.1039009094,
		Longitude: -49.789100647,
		Timezone:  "America/Sao_Paulo",
	},
	"SSYB": {
		ICAO:      "SSYB",
		Name:      "Fazenda Santa Elza Airport",
		City:      "Costa Rica",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1762,
		Latitude:  -18.6833324432,
		Longitude: -53.2833328247,
		Timezone:  "America/Campo_Grande",
	},
	"SSYC": {
		ICAO:      "SSYC",
		Name:      "Fazenda Mimoso Airport",
		City:      "Chapadao Do Sul",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1959,
		Latitude:  -19.0494441986,
		Longitude: -52.9638900757,
		Timezone:  "America/Campo_Grande",
	},
	"SSYD": {
		ICAO:      "SSYD",
		Name:      "Fazenda Andorinha I Airport",
		City:      "Pau D`Arco",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 643,
		Latitude:  -7.5480561256,
		Longitude: -49.2080574036,
		Timezone:  "America/Araguaina",
	},
	"SSYE": {
		ICAO:      "SSYE",
		Name:      "Fazenda Roberta Airport",
		City:      "Araputanga",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1033,
		Latitude:  -15.3208332062,
		Longitude: -58.5016670227,
		Timezone:  "America/Cuiaba",
	},
	"SSYF": {
		ICAO:      "SSYF",
		Name:      "Fazenda do Cafe Airport",
		City:      "Monte Alegre De Minas",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2762,
		Latitude:  -18.8716678619,
		Longitude: -48.7205543518,
		Timezone:  "America/Sao_Paulo",
	},
	"SSYL": {
		ICAO:      "SSYL",
		Name:      "Fazenda Tuju Puitan Airport",
		City:      "Iguatemi",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1142,
		Latitude:  -23.3411102295,
		Longitude: -54.5688896179,
		Timezone:  "America/Campo_Grande",
	},
	"SSYM": {
		ICAO:      "SSYM",
		Name:      "Tuiuiu Airport",
		City:      "Miranda",
		Country:   "BR",
		Elevation: 548,
		Latitude:  -20.0869445801,
		Longitude: -56.6163902283,
		Timezone:  "America/Campo_Grande",
	},
	"SSYN": {
		ICAO:      "SSYN",
		Name:      "Arrozeira Airport",
		City:      "Miranda",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 427,
		Latitude:  -20.0813884735,
		Longitude: -56.6677780151,
		Timezone:  "America/Campo_Grande",
	},
	"SSYO": {
		ICAO:      "SSYO",
		Name:      "Cachoeira Rica Airport",
		City:      "Chapada Dos Guimaraes",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1155,
		Latitude:  -15.2624998093,
		Longitude: -55.6147232056,
		Timezone:  "America/Cuiaba",
	},
	"SSYQ": {
		ICAO:      "SSYQ",
		Name:      "Agropecuaria Monte Alverne Airport",
		City:      "Carazinho",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 1752,
		Latitude:  -28.3788890839,
		Longitude: -52.8794441223,
		Timezone:  "America/Sao_Paulo",
	},
	"SSYR": {
		ICAO:      "SSYR",
		Name:      "Fazenda Santa Maria Airport",
		City:      "Tres Lagoas",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1214,
		Latitude:  -20.3477783203,
		Longitude: -52.3233337402,
		Timezone:  "America/Campo_Grande",
	},
	"SSYS": {
		ICAO:      "SSYS",
		Name:      "Fazenda Imbauba Airport",
		City:      "Tres Lagoas",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1552,
		Latitude:  -19.9088897705,
		Longitude: -52.5363883972,
		Timezone:  "America/Campo_Grande",
	},
	"SSYT": {
		ICAO:      "SSYT",
		Name:      "Itapiranga Airport",
		City:      "Itapiranga",
		State:     "Santa-Catarina",
		Country:   "BR",
		Elevation: 1247,
		Latitude:  -27.1424999237,
		Longitude: -53.685798645,
		Timezone:  "America/Sao_Paulo",
	},
	"SSYU": {
		ICAO:      "SSYU",
		Name:      "Fazenda Tres Pontes Airport",
		City:      "Agua Clara",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1621,
		Latitude:  -19.8269443512,
		Longitude: -52.5866661072,
		Timezone:  "America/Campo_Grande",
	},
	"SSYV": {
		ICAO:      "SSYV",
		Name:      "Fazenda Progresso Airport",
		City:      "Agua Clara",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1650,
		Latitude:  -19.7263889313,
		Longitude: -52.8513908386,
		Timezone:  "America/Campo_Grande",
	},
	"SSYW": {
		ICAO:      "SSYW",
		Name:      "Fazenda Terra Forte Airport",
		City:      "Coxim",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1460,
		Latitude:  -18.2683334351,
		Longitude: -54.7825012207,
		Timezone:  "America/Campo_Grande",
	},
	"SSYY": {
		ICAO:      "SSYY",
		Name:      "Fazenda Fundaozinho Airport",
		City:      "Costa Rica",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1923,
		Latitude:  -18.9511108398,
		Longitude: -53.1494445801,
		Timezone:  "America/Campo_Grande",
	},
	"SSYZ": {
		ICAO:      "SSYZ",
		Name:      "Fazenda Procomp II Airport",
		City:      "Aral Moreira",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 2080,
		Latitude:  -22.8511104584,
		Longitude: -55.3961105347,
		Timezone:  "America/Campo_Grande",
	},
	"SSZA": {
		ICAO:      "SSZA",
		Name:      "Fazenda Sao Sebastiao da Formosa Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 331,
		Latitude:  -17.7222213745,
		Longitude: -56.7533340454,
		Timezone:  "America/Campo_Grande",
	},
	"SSZB": {
		ICAO:      "SSZB",
		Name:      "Fazenda Bela Vista Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 331,
		Latitude:  -18.2066669464,
		Longitude: -56.8094444275,
		Timezone:  "America/Campo_Grande",
	},
	"SSZC": {
		ICAO:      "SSZC",
		Name:      "Fazenda Lambari Airport",
		City:      "Coxim",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 590,
		Latitude:  -18.0197219849,
		Longitude: -55.000831604,
		Timezone:  "America/Campo_Grande",
	},
	"SSZF": {
		ICAO:      "SSZF",
		Name:      "Fazenda Santa Gertrudes Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 150,
		Latitude:  -18.4777774811,
		Longitude: -56.0216674805,
		Timezone:  "America/Campo_Grande",
	},
	"SSZG": {
		ICAO:      "SSZG",
		Name:      "Fazenda Sao Joao Airport",
		City:      "Agua Clara",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1575,
		Latitude:  -19.2572212219,
		Longitude: -52.8963890076,
		Timezone:  "America/Campo_Grande",
	},
	"SSZH": {
		ICAO:      "SSZH",
		Name:      "Fazenda Procomp I Airport",
		City:      "Tacuru",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 928,
		Latitude:  -23.6461105347,
		Longitude: -54.7147216797,
		Timezone:  "America/Campo_Grande",
	},
	"SSZJ": {
		ICAO:      "SSZJ",
		Name:      "Fazenda Torrao de Ouro Airport",
		City:      "Pedro Gomes",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 899,
		Latitude:  -18.1807003021,
		Longitude: -54.5018005371,
		Timezone:  "America/Campo_Grande",
	},
	"SSZK": {
		ICAO:      "SSZK",
		Name:      "Sao Carlos Airport",
		City:      "Caracol",
		State:     "Concepcion",
		Country:   "BR",
		Elevation: 410,
		Latitude:  -22.2147216797,
		Longitude: -57.306388855,
		Timezone:  "America/Asuncion",
	},
	"SSZL": {
		ICAO:      "SSZL",
		Name:      "Fazenda Corrego Azul Airport",
		City:      "Brasilandia",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 994,
		Latitude:  -21.4544448853,
		Longitude: -52.1686096191,
		Timezone:  "America/Campo_Grande",
	},
	"SSZM": {
		ICAO:      "SSZM",
		Name:      "Reserva Acurizal Airport",
		City:      "Corumba",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 377,
		Latitude:  -17.8308334351,
		Longitude: -57.5516662598,
		Timezone:  "America/Campo_Grande",
	},
	"SSZN": {
		ICAO:      "SSZN",
		Name:      "Prudentopolis Airport",
		City:      "Prudentopolis",
		State:     "Parana",
		Country:   "BR",
		Elevation: 2451,
		Latitude:  -25.1819438934,
		Longitude: -50.9236106873,
		Timezone:  "America/Sao_Paulo",
	},
	"SSZP": {
		ICAO:      "SSZP",
		Name:      "Fazenda Santa Adriana Airport",
		City:      "Jatei",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1499,
		Latitude:  -22.6961116791,
		Longitude: -54.2158317566,
		Timezone:  "America/Campo_Grande",
	},
	"SSZQ": {
		ICAO:      "SSZQ",
		Name:      "Fazenda Panorama Airport",
		City:      "Bela Vista",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1197,
		Latitude:  -22.0389995575,
		Longitude: -56.2694015503,
		Timezone:  "America/Campo_Grande",
	},
	"SSZR": {
		ICAO:      "SSZR",
		IATA:      "SRA",
		Name:      "Santa Rosa Airport",
		City:      "Santa Rosa",
		State:     "Rio-Grande-do-Sul",
		Country:   "BR",
		Elevation: 984,
		Latitude:  -27.9067001343,
		Longitude: -54.520401001,
		Timezone:  "America/Sao_Paulo",
	},
	"SSZS": {
		ICAO:      "SSZS",
		IATA:      "0",
		Name:      "Centenario do Sul Airport",
		City:      "Centenario Do Sul",
		State:     "Parana",
		Country:   "BR",
		Elevation: 1591,
		Latitude:  -22.8307991028,
		Longitude: -51.6035995483,
		Timezone:  "America/Sao_Paulo",
	},
	"SSZT": {
		ICAO:      "SSZT",
		Name:      "Fazenda Sao Joao Airport",
		City:      "Aguas De Santa Barbara",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 2146,
		Latitude:  -22.7941665649,
		Longitude: -49.3411102295,
		Timezone:  "America/Sao_Paulo",
	},
	"SSZU": {
		ICAO:      "SSZU",
		Name:      "Fazenda Tres Marias Airport",
		City:      "Agua Clara",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1099,
		Latitude:  -19.4950008392,
		Longitude: -53.0405578613,
		Timezone:  "America/Campo_Grande",
	},
	"SSZV": {
		ICAO:      "SSZV",
		Name:      "Fazenda Bordolandia Airport",
		City:      "Serra Nova Dourada",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1066,
		Latitude:  -12.0619440079,
		Longitude: -51.5330543518,
		Timezone:  "America/Cuiaba",
	},
	"SSZW": {
		ICAO:      "SSZW",
		IATA:      "PGZ",
		Name:      "Ponta Grossa Airport",
		City:      "Ponta Grossa",
		State:     "Parana",
		Country:   "BR",
		Elevation: 2588,
		Latitude:  -25.1847000122,
		Longitude: -50.1441001892,
		Timezone:  "America/Sao_Paulo",
	},
	"SSZX": {
		ICAO:      "SSZX",
		Name:      "Fazenda Lapa do Lobo Airport",
		City:      "Paranaiba",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1447,
		Latitude:  -19.5272216797,
		Longitude: -50.9702796936,
		Timezone:  "America/Campo_Grande",
	},
	"SSZY": {
		ICAO:      "SSZY",
		Name:      "Carneirinho Agroindustrial Airport",
		City:      "Carneirinho",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1414,
		Latitude:  -19.8099994659,
		Longitude: -50.7813873291,
		Timezone:  "America/Sao_Paulo",
	},
	"SSZZ": {
		ICAO:      "SSZZ",
		Name:      "Fazenda Thaua Airport",
		City:      "Sonora",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 2024,
		Latitude:  -17.7130565643,
		Longitude: -54.4088897705,
		Timezone:  "America/Campo_Grande",
	},
	"SUAA": {
		ICAO:      "SUAA",
		Name:      "Angel S Adami Airport",
		City:      "Montevideo",
		State:     "Montevideo",
		Country:   "UY",
		Elevation: 174,
		Latitude:  -34.7891998291,
		Longitude: -56.2647018433,
		Timezone:  "America/Montevideo",
	},
	"SUAG": {
		ICAO:      "SUAG",
		IATA:      "ATI",
		Name:      "Artigas International Airport",
		City:      "Artigas",
		State:     "Artigas",
		Country:   "UY",
		Elevation: 410,
		Latitude:  -30.4006996155,
		Longitude: -56.507900238,
		Timezone:  "America/Montevideo",
	},
	"SUCA": {
		ICAO:      "SUCA",
		IATA:      "CYR",
		Name:      "Laguna de Los Patos International Airport",
		City:      "Colonia",
		State:     "Colonia",
		Country:   "UY",
		Elevation: 66,
		Latitude:  -34.456401825,
		Longitude: -57.7705993652,
		Timezone:  "America/Montevideo",
	},
	"SUCM": {
		ICAO:      "SUCM",
		IATA:      "CAR",
		Name:      "Carmelo International Airport",
		City:      "Carmelo",
		State:     "Colonia",
		Country:   "UY",
		Elevation: 42,
		Latitude:  -33.9661102295,
		Longitude: -58.3252792358,
		Timezone:  "America/Montevideo",
	},
	"SUDU": {
		ICAO:      "SUDU",
		IATA:      "DZO",
		Name:      "Santa Bernardina International Airport",
		City:      "Durazno",
		State:     "Durazno",
		Country:   "UY",
		Elevation: 305,
		Latitude:  -33.3588981628,
		Longitude: -56.4991989136,
		Timezone:  "America/Montevideo",
	},
	"SULS": {
		ICAO:      "SULS",
		IATA:      "PDP",
		Name:      "Capitan Corbeta CA Curbelo International Airport",
		City:      "Punta del Este",
		State:     "Maldonado",
		Country:   "UY",
		Elevation: 95,
		Latitude:  -34.8550987244,
		Longitude: -55.0942993164,
		Timezone:  "America/Montevideo",
	},
	"SUME": {
		ICAO:      "SUME",
		IATA:      "MER",
		Name:      "Ricardo de Tomasi International Airport",
		City:      "Mercedes",
		State:     "Soriano",
		Country:   "UY",
		Elevation: 68,
		Latitude:  -33.25,
		Longitude: -58.0670013428,
		Timezone:  "America/Montevideo",
	},
	"SUMO": {
		ICAO:      "SUMO",
		IATA:      "MLZ",
		Name:      "Cerro Largo International Airport",
		City:      "Melo",
		State:     "Cerro-Largo",
		Country:   "UY",
		Elevation: 364,
		Latitude:  -32.3378982544,
		Longitude: -54.2167015076,
		Timezone:  "America/Montevideo",
	},
	"SUMU": {
		ICAO:      "SUMU",
		IATA:      "MVD",
		Name:      "Carrasco International /General C L Berisso Airport",
		City:      "Montevideo",
		State:     "Canelones",
		Country:   "UY",
		Elevation: 105,
		Latitude:  -34.8384017944,
		Longitude: -56.0307998657,
		Timezone:  "America/Montevideo",
	},
	"SUPE": {
		ICAO:      "SUPE",
		IATA:      "MDO",
		Name:      "El Jaguel / Punta del Este Airport",
		City:      "Maldonado",
		State:     "Maldonado",
		Country:   "UY",
		Elevation: 66,
		Latitude:  -34.9169998169,
		Longitude: -54.9169998169,
		Timezone:  "America/Montevideo",
	},
	"SUPU": {
		ICAO:      "SUPU",
		IATA:      "PDU",
		Name:      "Tydeo Larre Borges Airport",
		City:      "Paysandu",
		State:     "Paysandu",
		Country:   "UY",
		Elevation: 138,
		Latitude:  -32.3633003235,
		Longitude: -58.0619010925,
		Timezone:  "America/Montevideo",
	},
	"SURB": {
		ICAO:      "SURB",
		Name:      "Rio Branco Airport",
		City:      "Jaguarao",
		State:     "Cerro-Largo",
		Country:   "UY",
		Elevation: 55,
		Latitude:  -32.5830574036,
		Longitude: -53.4541664124,
		Timezone:  "America/Montevideo",
	},
	"SURV": {
		ICAO:      "SURV",
		IATA:      "RVY",
		Name:      "Presidente General Don Oscar D. Gestido International Airport",
		City:      "Rivera",
		State:     "Rivera",
		Country:   "UY",
		Elevation: 712,
		Latitude:  -30.9745998383,
		Longitude: -55.4762001038,
		Timezone:  "America/Montevideo",
	},
	"SUSO": {
		ICAO:      "SUSO",
		IATA:      "STY",
		Name:      "Nueva Hesperides International Airport",
		City:      "Salto",
		State:     "Salto",
		Country:   "UY",
		Elevation: 187,
		Latitude:  -31.4384994507,
		Longitude: -57.9852981567,
		Timezone:  "America/Montevideo",
	},
	"SUTB": {
		ICAO:      "SUTB",
		IATA:      "TAW",
		Name:      "Tacuarembo Airport",
		City:      "Tacuarembo",
		State:     "Tacuarembo",
		Country:   "UY",
		Elevation: 440,
		Latitude:  -31.7490005493,
		Longitude: -55.9258003235,
		Timezone:  "America/Montevideo",
	},
	"SUTR": {
		ICAO:      "SUTR",
		IATA:      "TYT",
		Name:      "Treinta y Tres Airport",
		City:      "Treinta y Tres",
		State:     "Treinta-y-Tres",
		Country:   "UY",
		Elevation: 337,
		Latitude:  -33.2000007629,
		Longitude: -54.3499984741,
		Timezone:  "America/Montevideo",
	},
	"SUVO": {
		ICAO:      "SUVO",
		IATA:      "VCH",
		Name:      "Vichadero Airport",
		City:      "Vichadero",
		State:     "Rivera",
		Country:   "UY",
		Elevation: 488,
		Latitude:  -31.7670001984,
		Longitude: -54.6170005798,
		Timezone:  "America/Montevideo",
	},
	"SV02": {
		ICAO:      "SV02",
		Name:      "San Diego De Cabrutica Airport",
		City:      "El Roble",
		State:     "Anzoategui",
		Country:   "VE",
		Elevation: 492,
		Latitude:  8.4044303894,
		Longitude: -65.0155029297,
		Timezone:  "America/Caracas",
	},
	"SV73": {
		ICAO:      "SV73",
		Name:      "Uverito Airport",
		State:     "Monagas",
		Country:   "VE",
		Elevation: 300,
		Latitude:  8.6703195572,
		Longitude: -62.6254005432,
		Timezone:  "America/Caracas",
	},
	"SV74": {
		ICAO:      "SV74",
		Name:      "Aricuaisa Airport",
		State:     "Zulia",
		Country:   "VE",
		Elevation: 165,
		Latitude:  9.581870079,
		Longitude: -72.7880020142,
		Timezone:  "America/Caracas",
	},
	"SV75": {
		ICAO:      "SV75",
		Name:      "Machiques North Airport",
		State:     "Zulia",
		Country:   "VE",
		Elevation: 361,
		Latitude:  10.0882997513,
		Longitude: -72.5548019409,
		Timezone:  "America/Caracas",
	},
	"SVAC": {
		ICAO:      "SVAC",
		IATA:      "AGV",
		Name:      "Oswaldo Guevara Mujica Airport",
		City:      "Acarigua",
		State:     "Portuguesa",
		Country:   "VE",
		Elevation: 640,
		Latitude:  9.5533752441,
		Longitude: -69.2378692627,
		Timezone:  "America/Caracas",
	},
	"SVAE": {
		ICAO:      "SVAE",
		Name:      "Montellano Airport",
		State:     "Zulia",
		Country:   "VE",
		Elevation: 656,
		Latitude:  10.3281002045,
		Longitude: -72.4810028076,
		Timezone:  "America/Caracas",
	},
	"SVAF": {
		ICAO:      "SVAF",
		Name:      "Medano Alto Airport",
		City:      "Medano Alto",
		State:     "Apure",
		Country:   "VE",
		Elevation: 459,
		Latitude:  6.9333329201,
		Longitude: -68.0166702271,
		Timezone:  "America/Caracas",
	},
	"SVAH": {
		ICAO:      "SVAH",
		Name:      "Cachimbo Airport",
		City:      "Coco de Mono",
		State:     "Yaracuy",
		Country:   "VE",
		Elevation: 1000,
		Latitude:  10.0333328247,
		Longitude: -69.1666641235,
		Timezone:  "America/Caracas",
	},
	"SVAI": {
		ICAO:      "SVAI",
		Name:      "Arichuna Airport",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 145,
		Latitude:  7.8666667938,
		Longitude: -67.0333328247,
		Timezone:  "America/Caracas",
	},
	"SVAJ": {
		ICAO:      "SVAJ",
		Name:      "Mata de Juajua Airport",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 393,
		Latitude:  8.8189296722,
		Longitude: -65.8867034912,
		Timezone:  "America/Caracas",
	},
	"SVAL": {
		ICAO:      "SVAL",
		Name:      "Algodonal Airport",
		State:     "Barinas",
		Country:   "VE",
		Elevation: 574,
		Latitude:  8.3500003815,
		Longitude: -69.5166702271,
		Timezone:  "America/Caracas",
	},
	"SVAM": {
		ICAO:      "SVAM",
		Name:      "Mata de Cana Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 344,
		Latitude:  7.3166670799,
		Longitude: -69.1999969482,
		Timezone:  "America/Caracas",
	},
	"SVAN": {
		ICAO:      "SVAN",
		IATA:      "AAO",
		Name:      "Anaco Airport",
		City:      "Anaco",
		State:     "Anzoategui",
		Country:   "VE",
		Elevation: 721,
		Latitude:  9.4302253723,
		Longitude: -64.4707260132,
		Timezone:  "America/Caracas",
	},
	"SVAO": {
		ICAO:      "SVAO",
		Name:      "Altagracia de Orituco Airport",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 1148,
		Latitude:  9.8633327484,
		Longitude: -66.2933349609,
		Timezone:  "America/Caracas",
	},
	"SVAP": {
		ICAO:      "SVAP",
		Name:      "Apurito Airport",
		City:      "Apurito",
		State:     "Apure",
		Country:   "VE",
		Elevation: 295,
		Latitude:  7.9188499451,
		Longitude: -68.483001709,
		Timezone:  "America/Caracas",
	},
	"SVAQ": {
		ICAO:      "SVAQ",
		Name:      "Mata Oscura Airport",
		State:     "Cojedes",
		Country:   "VE",
		Elevation: 196,
		Latitude:  9.25,
		Longitude: -68.4833297729,
		Timezone:  "America/Caracas",
	},
	"SVAR": {
		ICAO:      "SVAR",
		Name:      "Adicora Airport",
		State:     "Falcon",
		Country:   "VE",
		Elevation: 7,
		Latitude:  11.9368000031,
		Longitude: -69.8096008301,
		Timezone:  "America/Caracas",
	},
	"SVAS": {
		ICAO:      "SVAS",
		IATA:      "LPJ",
		Name:      "Armando Schwarck Airport",
		City:      "Guayabal",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 266,
		Latitude:  6.5780501366,
		Longitude: -66.8169021606,
		Timezone:  "America/Caracas",
	},
	"SVAT": {
		ICAO:      "SVAT",
		Name:      "San Fernando de Atabapo Airport",
		City:      "San Fernando de Atabapo",
		State:     "Amazonas",
		Country:   "VE",
		Elevation: 301,
		Latitude:  4.0510401726,
		Longitude: -67.6996994019,
		Timezone:  "America/Bogota",
	},
	"SVAU": {
		ICAO:      "SVAU",
		Name:      "Agua Clara Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 328,
		Latitude:  6.75,
		Longitude: -68.3333358765,
		Timezone:  "America/Caracas",
	},
	"SVAV": {
		ICAO:      "SVAV",
		Name:      "El Alcaravan Airport",
		State:     "Monagas",
		Country:   "VE",
		Elevation: 128,
		Latitude:  9.2733325958,
		Longitude: -63.4183311462,
		Timezone:  "America/Caracas",
	},
	"SVAW": {
		ICAO:      "SVAW",
		Name:      "Merecure Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 220,
		Latitude:  7.4833331108,
		Longitude: -67.8499984741,
		Timezone:  "America/Caracas",
	},
	"SVAX": {
		ICAO:      "SVAX",
		Name:      "Los Andes Airport",
		State:     "Zulia",
		Country:   "VE",
		Elevation: 393,
		Latitude:  10,
		Longitude: -72.5833358765,
		Timezone:  "America/Caracas",
	},
	"SVAY": {
		ICAO:      "SVAY",
		Name:      "Araya Airport",
		State:     "Sucre",
		Country:   "VE",
		Elevation: 32,
		Latitude:  10.5833330154,
		Longitude: -64.2466659546,
		Timezone:  "America/Caracas",
	},
	"SVAZ": {
		ICAO:      "SVAZ",
		Name:      "Mata Palos Airport",
		State:     "Barinas",
		Country:   "VE",
		Elevation: 224,
		Latitude:  8.4333333969,
		Longitude: -68.5500030518,
		Timezone:  "America/Caracas",
	},
	"SVBA": {
		ICAO:      "SVBA",
		Name:      "Boca Anaro Airport",
		City:      "Boca de Anaro",
		State:     "Barinas",
		Country:   "VE",
		Elevation: 298,
		Latitude:  7.8333330154,
		Longitude: -70.2666702271,
		Timezone:  "America/Caracas",
	},
	"SVBC": {
		ICAO:      "SVBC",
		IATA:      "BLA",
		Name:      "General Jose Antonio Anzoategui International Airport",
		City:      "Barcelona",
		State:     "Anzoategui",
		Country:   "VE",
		Elevation: 26,
		Latitude:  10.1070995331,
		Longitude: -64.689201355,
		Timezone:  "America/Caracas",
	},
	"SVBD": {
		ICAO:      "SVBD",
		Name:      "Los Camorucos Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 321,
		Latitude:  7.6166667938,
		Longitude: -69.0500030518,
		Timezone:  "America/Caracas",
	},
	"SVBE": {
		ICAO:      "SVBE",
		Name:      "Bernal Airport",
		State:     "Zulia",
		Country:   "VE",
		Elevation: 249,
		Latitude:  9.6666669846,
		Longitude: -72.5833358765,
		Timezone:  "America/Caracas",
	},
	"SVBF": {
		ICAO:      "SVBF",
		Name:      "Hato Uranon Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 410,
		Latitude:  6.9666671753,
		Longitude: -67.2166671753,
		Timezone:  "America/Caracas",
	},
	"SVBG": {
		ICAO:      "SVBG",
		Name:      "La Batalla Airport",
		City:      "La Cruz",
		State:     "Cojedes",
		Country:   "VE",
		Elevation: 273,
		Latitude:  9.1999998093,
		Longitude: -68.1666641235,
		Timezone:  "America/Caracas",
	},
	"SVBH": {
		ICAO:      "SVBH",
		Name:      "Los Belseres Airport",
		State:     "Falcon",
		Country:   "VE",
		Elevation: 3,
		Latitude:  10.9416666031,
		Longitude: -68.2666702271,
		Timezone:  "America/Caracas",
	},
	"SVBI": {
		ICAO:      "SVBI",
		IATA:      "BNS",
		Name:      "Barinas Airport",
		City:      "Barinas",
		State:     "Barinas",
		Country:   "VE",
		Elevation: 666,
		Latitude:  8.6195697784,
		Longitude: -70.2208023071,
		Timezone:  "America/Caracas",
	},
	"SVBJ": {
		ICAO:      "SVBJ",
		Name:      "Hato Viejo Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 524,
		Latitude:  7.3216671944,
		Longitude: -69.6699981689,
		Timezone:  "America/Caracas",
	},
	"SVBK": {
		ICAO:      "SVBK",
		Name:      "Bocono Airport",
		City:      "Bocono",
		State:     "Trujillo",
		Country:   "VE",
		Elevation: 5084,
		Latitude:  9.2671003342,
		Longitude: -70.2304992676,
		Timezone:  "America/Caracas",
	},
	"SVBL": {
		ICAO:      "SVBL",
		Name:      "El Libertador Airbase",
		State:     "Aragua",
		Country:   "VE",
		Elevation: 1450,
		Latitude:  10.1833753586,
		Longitude: -67.5573196411,
		Timezone:  "America/Caracas",
	},
	"SVBM": {
		ICAO:      "SVBM",
		IATA:      "BRM",
		Name:      "Barquisimeto International Airport",
		City:      "Barquisimeto",
		State:     "Lara",
		Country:   "VE",
		Elevation: 2042,
		Latitude:  10.0427465439,
		Longitude: -69.3586196899,
		Timezone:  "America/Caracas",
	},
	"SVBN": {
		ICAO:      "SVBN",
		Name:      "Bocon Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 1462,
		Latitude:  6.5999999046,
		Longitude: -62.7333335876,
		Timezone:  "America/Caracas",
	},
	"SVBP": {
		ICAO:      "SVBP",
		Name:      "Jobito Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 138,
		Latitude:  7.5999999046,
		Longitude: -67.1999969482,
		Timezone:  "America/Caracas",
	},
	"SVBQ": {
		ICAO:      "SVBQ",
		Name:      "Bachaquero Airport",
		State:     "Zulia",
		Country:   "VE",
		Elevation: 43,
		Latitude:  10.0015001297,
		Longitude: -71.0837020874,
		Timezone:  "America/Caracas",
	},
	"SVBR": {
		ICAO:      "SVBR",
		Name:      "Carbonero Airport",
		State:     "Yaracuy",
		Country:   "VE",
		Elevation: 328,
		Latitude:  10.4166669846,
		Longitude: -68.6666641235,
		Timezone:  "America/Caracas",
	},
	"SVBS": {
		ICAO:      "SVBS",
		IATA:      "MYC",
		Name:      "Escuela Mariscal Sucre Airport",
		City:      "Maracay",
		State:     "Aragua",
		Country:   "VE",
		Elevation: 1338,
		Latitude:  10.2499780655,
		Longitude: -67.6494216919,
		Timezone:  "America/Caracas",
	},
	"SVBT": {
		ICAO:      "SVBT",
		Name:      "Boquemonte Airport",
		State:     "Portuguesa",
		Country:   "VE",
		Elevation: 1000,
		Latitude:  9.1999998093,
		Longitude: -69.75,
		Timezone:  "America/Caracas",
	},
	"SVBU": {
		ICAO:      "SVBU",
		Name:      "Central Venezuela Airport",
		City:      "Bobures",
		State:     "Zulia",
		Country:   "VE",
		Elevation: 33,
		Latitude:  9.1666669846,
		Longitude: -71.0999984741,
		Timezone:  "America/Caracas",
	},
	"SVBV": {
		ICAO:      "SVBV",
		Name:      "Banco Verde Airport",
		State:     "Portuguesa",
		Country:   "VE",
		Elevation: 348,
		Latitude:  8.733332634,
		Longitude: -69.5333328247,
		Timezone:  "America/Caracas",
	},
	"SVBW": {
		ICAO:      "SVBW",
		Name:      "Hato El Burro Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 921,
		Latitude:  7.1316671371,
		Longitude: -63.4566688538,
		Timezone:  "America/Caracas",
	},
	"SVBX": {
		ICAO:      "SVBX",
		Name:      "La Cabana Airport",
		State:     "Zulia",
		Country:   "VE",
		Elevation: 215,
		Latitude:  9.766667366,
		Longitude: -72.6999969482,
		Timezone:  "America/Caracas",
	},
	"SVCB": {
		ICAO:      "SVCB",
		IATA:      "CBL",
		Name:      "Ciudad Bolivar Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 197,
		Latitude:  8.1221609116,
		Longitude: -63.5369567871,
		Timezone:  "America/Caracas",
	},
	"SVCD": {
		ICAO:      "SVCD",
		IATA:      "CXA",
		Name:      "Caicara del Orinoco Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 141,
		Latitude:  7.6255102158,
		Longitude: -66.1628036499,
		Timezone:  "America/Caracas",
	},
	"SVCE": {
		ICAO:      "SVCE",
		Name:      "Apruna Airport",
		City:      "Cuenca de Unare",
		State:     "Anzoategui",
		Country:   "VE",
		Elevation: 204,
		Latitude:  9.3503055573,
		Longitude: -65.2806549072,
		Timezone:  "America/Caracas",
	},
	"SVCG": {
		ICAO:      "SVCG",
		IATA:      "CUV",
		Name:      "Casigua El Cubo Airport",
		City:      "Casigua El Cubo",
		State:     "Zulia",
		Country:   "VE",
		Elevation: 99,
		Latitude:  8.7581396103,
		Longitude: -72.5363006592,
		Timezone:  "America/Caracas",
	},
	"SVCH": {
		ICAO:      "SVCH",
		Name:      "Achaguas Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 187,
		Latitude:  7.7399997711,
		Longitude: -68.2399978638,
		Timezone:  "America/Caracas",
	},
	"SVCI": {
		ICAO:      "SVCI",
		Name:      "Punta de Oro Airport",
		State:     "Trujillo",
		Country:   "VE",
		Elevation: 37,
		Latitude:  9.5016670227,
		Longitude: -70.9516677856,
		Timezone:  "America/Caracas",
	},
	"SVCJ": {
		ICAO:      "SVCJ",
		Name:      "San Carlos Airport",
		City:      "San Carlos",
		State:     "Cojedes",
		Country:   "VE",
		Elevation: 512,
		Latitude:  9.6477203369,
		Longitude: -68.5746994019,
		Timezone:  "America/Caracas",
	},
	"SVCK": {
		ICAO:      "SVCK",
		Name:      "Cano Lucas Airport",
		City:      "El Poblado",
		State:     "Portuguesa",
		Country:   "VE",
		Elevation: 301,
		Latitude:  9.4166669846,
		Longitude: -68.8833312988,
		Timezone:  "America/Caracas",
	},
	"SVCL": {
		ICAO:      "SVCL",
		IATA:      "CLZ",
		Name:      "Calabozo Airport",
		City:      "Guarico",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 328,
		Latitude:  8.9246559143,
		Longitude: -67.4170913696,
		Timezone:  "America/Caracas",
	},
	"SVCM": {
		ICAO:      "SVCM",
		Name:      "Canita Mendera Airport",
		City:      "Tinaco",
		State:     "Cojedes",
		Country:   "VE",
		Elevation: 662,
		Latitude:  9.6833333969,
		Longitude: -68.4166641235,
		Timezone:  "America/Caracas",
	},
	"SVCN": {
		ICAO:      "SVCN",
		IATA:      "CAJ",
		Name:      "Canaima Airport",
		City:      "Canaima",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 1450,
		Latitude:  6.2319889069,
		Longitude: -62.8544311523,
		Timezone:  "America/Caracas",
	},
	"SVCO": {
		ICAO:      "SVCO",
		IATA:      "VCR",
		Name:      "Carora Airport",
		City:      "Carora",
		State:     "Lara",
		Country:   "VE",
		Elevation: 1437,
		Latitude:  10.1756029129,
		Longitude: -70.0652160645,
		Timezone:  "America/Caracas",
	},
	"SVCP": {
		ICAO:      "SVCP",
		IATA:      "CUP",
		Name:      "General Francisco Bermudez Airport",
		City:      "Carupano",
		State:     "Sucre",
		Country:   "VE",
		Elevation: 33,
		Latitude:  10.6600141525,
		Longitude: -63.261680603,
		Timezone:  "America/Caracas",
	},
	"SVCQ": {
		ICAO:      "SVCQ",
		Name:      "Cazorla Airport",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 639,
		Latitude:  8.0606002808,
		Longitude: -66.9699020386,
		Timezone:  "America/Caracas",
	},
	"SVCR": {
		ICAO:      "SVCR",
		IATA:      "CZE",
		Name:      "Jose Leonardo Chirinos Airport",
		City:      "Coro",
		State:     "Falcon",
		Country:   "VE",
		Elevation: 52,
		Latitude:  11.4149436951,
		Longitude: -69.6809005737,
		Timezone:  "America/Caracas",
	},
	"SVCS": {
		ICAO:      "SVCS",
		Name:      "Oscar Machado Zuluaga Airport",
		City:      "Caracas",
		State:     "Miranda",
		Country:   "VE",
		Elevation: 2145,
		Latitude:  10.286110878,
		Longitude: -66.8161087036,
		Timezone:  "America/Caracas",
	},
	"SVCT": {
		ICAO:      "SVCT",
		Name:      "Caujarito Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 360,
		Latitude:  6.8833332062,
		Longitude: -68.3833312988,
		Timezone:  "America/Caracas",
	},
	"SVCU": {
		ICAO:      "SVCU",
		IATA:      "CUM",
		Name:      "Cumana (Antonio Jose de Sucre) Airport",
		State:     "Sucre",
		Country:   "VE",
		Elevation: 14,
		Latitude:  10.4503326416,
		Longitude: -64.1304702759,
		Timezone:  "America/Caracas",
	},
	"SVCV": {
		ICAO:      "SVCV",
		Name:      "Cunaviche Airport",
		City:      "Cunaviche",
		State:     "Apure",
		Country:   "VE",
		Elevation: 262,
		Latitude:  7.4000000954,
		Longitude: -67.4333343506,
		Timezone:  "America/Caracas",
	},
	"SVCW": {
		ICAO:      "SVCW",
		Name:      "Chinazon Airport",
		State:     "Trujillo",
		Country:   "VE",
		Elevation: 98,
		Latitude:  9.326666832,
		Longitude: -70.9333343506,
		Timezone:  "America/Caracas",
	},
	"SVCX": {
		ICAO:      "SVCX",
		Name:      "Guachara Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 656,
		Latitude:  7.3000001907,
		Longitude: -68.3666687012,
		Timezone:  "America/Caracas",
	},
	"SVCY": {
		ICAO:      "SVCY",
		Name:      "Cuartel Yaruro Airport",
		City:      "Las Campanas",
		State:     "Apure",
		Country:   "VE",
		Elevation: 442,
		Latitude:  6.9333329201,
		Longitude: -68.6333312988,
		Timezone:  "America/Caracas",
	},
	"SVCZ": {
		ICAO:      "SVCZ",
		Name:      "Capitan Manuel Rios Airbase",
		City:      "Carrizal",
		State:     "Aragua",
		Country:   "VE",
		Elevation: 595,
		Latitude:  9.3726501465,
		Longitude: -66.9227981567,
		Timezone:  "America/Caracas",
	},
	"SVDA": {
		ICAO:      "SVDA",
		IATA:      "isl",
		Name:      "La Tortuga Punta Delgada Airport",
		City:      "Isla La Tortuga",
		State:     "Dependencias-Federales",
		Country:   "VE",
		Elevation: 26,
		Latitude:  10.9366674423,
		Longitude: -65.2233352661,
		Timezone:  "America/Caracas",
	},
	"SVDB": {
		ICAO:      "SVDB",
		Name:      "Dona Bella Airport",
		State:     "Cojedes",
		Country:   "VE",
		Elevation: 651,
		Latitude:  9.3333330154,
		Longitude: -68.4333343506,
		Timezone:  "America/Caracas",
	},
	"SVDC": {
		ICAO:      "SVDC",
		Name:      "Cocuiza Airport",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 230,
		Latitude:  8.5900001526,
		Longitude: -66.2549972534,
		Timezone:  "America/Caracas",
	},
	"SVDF": {
		ICAO:      "SVDF",
		Name:      "Punto Fijo Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 360,
		Latitude:  6.8666667938,
		Longitude: -68.7833328247,
		Timezone:  "America/Caracas",
	},
	"SVDG": {
		ICAO:      "SVDG",
		Name:      "Punta Gorda Airport",
		State:     "Monagas",
		Country:   "VE",
		Elevation: 859,
		Latitude:  9.6499996185,
		Longitude: -63.7216682434,
		Timezone:  "America/Caracas",
	},
	"SVDM": {
		ICAO:      "SVDM",
		Name:      "Dos Mosquises Airport",
		City:      "Isla Los Roques",
		State:     "Dependencias-Federales",
		Country:   "VE",
		Elevation: 5,
		Latitude:  11.7967996597,
		Longitude: -66.8929977417,
		Timezone:  "America/Caracas",
	},
	"SVDP": {
		ICAO:      "SVDP",
		Name:      "La Divina Pastora Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 2581,
		Latitude:  4.7300000191,
		Longitude: -60.9650001526,
		Timezone:  "America/Caracas",
	},
	"SVDR": {
		ICAO:      "SVDR",
		Name:      "Cordereno Airport",
		City:      "Cordereno",
		State:     "Barinas",
		Country:   "VE",
		Elevation: 268,
		Latitude:  7.9333329201,
		Longitude: -69.9000015259,
		Timezone:  "America/Caracas",
	},
	"SVDS": {
		ICAO:      "SVDS",
		Name:      "Codsa Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 2850,
		Latitude:  4.7916669846,
		Longitude: -61.1199989319,
		Timezone:  "America/Caracas",
	},
	"SVDV": {
		ICAO:      "SVDV",
		Name:      "Hato Veladero Airport",
		State:     "Monagas",
		Country:   "VE",
		Elevation: 239,
		Latitude:  9.4499998093,
		Longitude: -63.1749992371,
		Timezone:  "America/Caracas",
	},
	"SVDW": {
		ICAO:      "SVDW",
		Name:      "Ciudad Piar Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 879,
		Latitude:  7.4937901497,
		Longitude: -63.2697982788,
		Timezone:  "America/Caracas",
	},
	"SVDX": {
		ICAO:      "SVDX",
		Name:      "Hato Puga Airport",
		State:     "Delta-Amacuro",
		Country:   "VE",
		Elevation: 268,
		Latitude:  8.4666671753,
		Longitude: -62.4833335876,
		Timezone:  "America/Caracas",
	},
	"SVDZ": {
		ICAO:      "SVDZ",
		Name:      "Puerto Paez",
		State:     "Apure",
		Country:   "VE",
		Elevation: 146,
		Latitude:  6.2333331108,
		Longitude: -67.4336090088,
		Timezone:  "America/Caracas",
	},
	"SVEB": {
		ICAO:      "SVEB",
		Name:      "El Guayabo del Zulia Airport",
		State:     "Zulia",
		Country:   "VE",
		Elevation: 79,
		Latitude:  8.6761598587,
		Longitude: -72.3356018066,
		Timezone:  "America/Caracas",
	},
	"SVED": {
		ICAO:      "SVED",
		IATA:      "EOR",
		Name:      "El Dorado Airport",
		City:      "Bolivar",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 318,
		Latitude:  6.7333331108,
		Longitude: -61.5833320618,
		Timezone:  "America/Caracas",
	},
	"SVEE": {
		ICAO:      "SVEE",
		Name:      "El Esterero Airport",
		City:      "El Saman de Apure",
		State:     "Apure",
		Country:   "VE",
		Elevation: 590,
		Latitude:  7.9083328247,
		Longitude: -68.6966705322,
		Timezone:  "America/Caracas",
	},
	"SVEF": {
		ICAO:      "SVEF",
		Name:      "Empujeca Airport",
		State:     "Yaracuy",
		Country:   "VE",
		Elevation: 66,
		Latitude:  10.595000267,
		Longitude: -68.4950027466,
		Timezone:  "America/Caracas",
	},
	"SVEG": {
		ICAO:      "SVEG",
		Name:      "El Guayabo de Cojedes Airport",
		State:     "Cojedes",
		Country:   "VE",
		Elevation: 23,
		Latitude:  9.6999998093,
		Longitude: -68.3333358765,
		Timezone:  "America/Caracas",
	},
	"SVEJ": {
		ICAO:      "SVEJ",
		Name:      "El Jabillal Airport",
		State:     "Trujillo",
		Country:   "VE",
		Elevation: 354,
		Latitude:  9.4499998093,
		Longitude: -70.8000030518,
		Timezone:  "America/Caracas",
	},
	"SVEK": {
		ICAO:      "SVEK",
		Name:      "El Capitan Airport",
		State:     "Zulia",
		Country:   "VE",
		Elevation: 452,
		Latitude:  10.1135997772,
		Longitude: -72.5765991211,
		Timezone:  "America/Caracas",
	},
	"SVEL": {
		ICAO:      "SVEL",
		Name:      "El Callao (El Peru) Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 747,
		Latitude:  7.3000001907,
		Longitude: -61.7999992371,
		Timezone:  "America/Caracas",
	},
	"SVEM": {
		ICAO:      "SVEM",
		Name:      "El Paraiso Airport",
		City:      "El Paraiso",
		State:     "Barinas",
		Country:   "VE",
		Elevation: 656,
		Latitude:  8.3333330154,
		Longitude: -70.375,
		Timezone:  "America/Caracas",
	},
	"SVEN": {
		ICAO:      "SVEN",
		Name:      "Encontrados Airport",
		State:     "Zulia",
		Country:   "VE",
		Elevation: 66,
		Latitude:  9.1116666794,
		Longitude: -72.2699966431,
		Timezone:  "America/Caracas",
	},
	"SVEP": {
		ICAO:      "SVEP",
		Name:      "El Palmar Airport",
		State:     "Aragua",
		Country:   "VE",
		Elevation: 1554,
		Latitude:  10.2066669464,
		Longitude: -67.4416656494,
		Timezone:  "America/Caracas",
	},
	"SVER": {
		ICAO:      "SVER",
		Name:      "El Respiro Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 387,
		Latitude:  7.2166671753,
		Longitude: -68.9833297729,
		Timezone:  "America/Caracas",
	},
	"SVES": {
		ICAO:      "SVES",
		Name:      "El Saman de Apure Airport",
		City:      "El Saman de Apure",
		State:     "Apure",
		Country:   "VE",
		Elevation: 361,
		Latitude:  7.9123501778,
		Longitude: -68.6927032471,
		Timezone:  "America/Caracas",
	},
	"SVET": {
		ICAO:      "SVET",
		Name:      "El Manteco Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 1000,
		Latitude:  7.3610701561,
		Longitude: -62.5321006775,
		Timezone:  "America/Caracas",
	},
	"SVEW": {
		ICAO:      "SVEW",
		Name:      "El Cenizo Airport",
		State:     "Trujillo",
		Country:   "VE",
		Elevation: 426,
		Latitude:  9.5500001907,
		Longitude: -70.6833343506,
		Timezone:  "America/Caracas",
	},
	"SVEX": {
		ICAO:      "SVEX",
		Name:      "La Esperanza Airport",
		State:     "Portuguesa",
		Country:   "VE",
		Elevation: 256,
		Latitude:  8.6999998093,
		Longitude: -69.0749969482,
		Timezone:  "America/Caracas",
	},
	"SVEY": {
		ICAO:      "SVEY",
		Name:      "El Centro Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 318,
		Latitude:  6.2800002098,
		Longitude: -69.1166687012,
		Timezone:  "America/Caracas",
	},
	"SVEZ": {
		ICAO:      "SVEZ",
		IATA:      "EOZ",
		Name:      "Elorza Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 295,
		Latitude:  7.0833330154,
		Longitude: -69.5333328247,
		Timezone:  "America/Caracas",
	},
	"SVFE": {
		ICAO:      "SVFE",
		Name:      "Hacienda Santa Fe Airport",
		State:     "Zulia",
		Country:   "VE",
		Elevation: 524,
		Latitude:  9.5,
		Longitude: -72.5833358765,
		Timezone:  "America/Caracas",
	},
	"SVFG": {
		ICAO:      "SVFG",
		Name:      "El Flagelo Airport",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 663,
		Latitude:  9.1883325577,
		Longitude: -67.0166702271,
		Timezone:  "America/Caracas",
	},
	"SVFK": {
		ICAO:      "SVFK",
		Name:      "La Fe de Cojedes Airport",
		City:      "La Mosca",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 412,
		Latitude:  9.483332634,
		Longitude: -67.8166656494,
		Timezone:  "America/Caracas",
	},
	"SVFL": {
		ICAO:      "SVFL",
		Name:      "Fundacion Layera Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 219,
		Latitude:  7.1999998093,
		Longitude: -68.1333312988,
		Timezone:  "America/Caracas",
	},
	"SVFM": {
		ICAO:      "SVFM",
		Name:      "Francisco de Miranda Airport",
		City:      "La Carlota",
		State:     "Miranda",
		Country:   "VE",
		Elevation: 2739,
		Latitude:  10.4850330353,
		Longitude: -66.8435134888,
		Timezone:  "America/Caracas",
	},
	"SVFR": {
		ICAO:      "SVFR",
		Name:      "El Frio Airport",
		City:      "Guarico",
		State:     "Apure",
		Country:   "VE",
		Elevation: 219,
		Latitude:  7.8233327866,
		Longitude: -68.8983306885,
		Timezone:  "America/Caracas",
	},
	"SVFS": {
		ICAO:      "SVFS",
		Name:      "Fagotrans Airport",
		State:     "Aragua",
		Country:   "VE",
		Elevation: 590,
		Latitude:  9.4016399384,
		Longitude: -66.6430969238,
		Timezone:  "America/Caracas",
	},
	"SVFT": {
		ICAO:      "SVFT",
		Name:      "El Fuentero Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 420,
		Latitude:  7.0999999046,
		Longitude: -69.5833358765,
		Timezone:  "America/Caracas",
	},
	"SVFU": {
		ICAO:      "SVFU",
		Name:      "La Fortuna Airport",
		City:      "San Antonio de Caparo",
		State:     "Tachira",
		Country:   "VE",
		Elevation: 417,
		Latitude:  7.5833330154,
		Longitude: -71.4666671753,
		Timezone:  "America/Caracas",
	},
	"SVFX": {
		ICAO:      "SVFX",
		Name:      "La Fe de Apure Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 442,
		Latitude:  7.2133331299,
		Longitude: -69.4966659546,
		Timezone:  "America/Caracas",
	},
	"SVFY": {
		ICAO:      "SVFY",
		Name:      "Finca Yacurito Airport",
		State:     "Portuguesa",
		Country:   "VE",
		Elevation: 1706,
		Latitude:  9.0333328247,
		Longitude: -69.0999984741,
		Timezone:  "America/Caracas",
	},
	"SVGA": {
		ICAO:      "SVGA",
		Name:      "Agropecuaria Gamma Airport",
		State:     "Barinas",
		Country:   "VE",
		Elevation: 495,
		Latitude:  8.75,
		Longitude: -70,
		Timezone:  "America/Caracas",
	},
	"SVGC": {
		ICAO:      "SVGC",
		Name:      "La Gran China Airport",
		State:     "Zulia",
		Country:   "VE",
		Elevation: 138,
		Latitude:  9.7770795822,
		Longitude: -72.4868011475,
		Timezone:  "America/Caracas",
	},
	"SVGD": {
		ICAO:      "SVGD",
		IATA:      "GDO",
		Name:      "Guasdalito Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 426,
		Latitude:  7.2333331108,
		Longitude: -70.8000030518,
		Timezone:  "America/Caracas",
	},
	"SVGE": {
		ICAO:      "SVGE",
		Name:      "Ganaderia Pedernales Airport",
		State:     "Portuguesa",
		Country:   "VE",
		Elevation: 467,
		Latitude:  9.2100000381,
		Longitude: -69.5816650391,
		Timezone:  "America/Caracas",
	},
	"SVGG": {
		ICAO:      "SVGG",
		Name:      "Agropecuaria Los Araguaneyes Airport",
		State:     "Monagas",
		Country:   "VE",
		Elevation: 507,
		Latitude:  9.5833330154,
		Longitude: -63.0499992371,
		Timezone:  "America/Caracas",
	},
	"SVGH": {
		ICAO:      "SVGH",
		Name:      "La Guacharaca Airport",
		State:     "Aragua",
		Country:   "VE",
		Elevation: 984,
		Latitude:  9.5349998474,
		Longitude: -66.7249984741,
		Timezone:  "America/Caracas",
	},
	"SVGI": {
		ICAO:      "SVGI",
		IATA:      "GUI",
		Name:      "Guiria Airport",
		Country:   "VE",
		Elevation: 42,
		Latitude:  10.5740776062,
		Longitude: -62.3126678467,
		Timezone:  "America/Caracas",
	},
	"SVGL": {
		ICAO:      "SVGL",
		Name:      "Agua Linda Airport",
		State:     "Barinas",
		Country:   "VE",
		Elevation: 417,
		Latitude:  7.6350002289,
		Longitude: -70.8499984741,
		Timezone:  "America/Caracas",
	},
	"SVGP": {
		ICAO:      "SVGP",
		Name:      "Guesipo Airport",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 641,
		Latitude:  9.6333332062,
		Longitude: -67.1833343506,
		Timezone:  "America/Caracas",
	},
	"SVGR": {
		ICAO:      "SVGR",
		Name:      "El Milagro Carabobo Airport",
		State:     "Carabobo",
		Country:   "VE",
		Elevation: 2013,
		Latitude:  10.1529998779,
		Longitude: -68.3751983643,
		Timezone:  "America/Caracas",
	},
	"SVGT": {
		ICAO:      "SVGT",
		Name:      "Guasipati Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 977,
		Latitude:  7.4749999046,
		Longitude: -61.9000015259,
		Timezone:  "America/Caracas",
	},
	"SVGU": {
		ICAO:      "SVGU",
		IATA:      "GUQ",
		Name:      "Guanare Airport",
		City:      "Guanare",
		State:     "Portuguesa",
		Country:   "VE",
		Elevation: 606,
		Latitude:  9.0269441605,
		Longitude: -69.7551498413,
		Timezone:  "America/Caracas",
	},
	"SVGX": {
		ICAO:      "SVGX",
		Name:      "Las Guadalupes Airport",
		State:     "Anzoategui",
		Country:   "VE",
		Elevation: 984,
		Latitude:  8.7550001144,
		Longitude: -63.5250015259,
		Timezone:  "America/Caracas",
	},
	"SVGZ": {
		ICAO:      "SVGZ",
		Name:      "La Garza Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 237,
		Latitude:  7.9499998093,
		Longitude: -69.1166687012,
		Timezone:  "America/Caracas",
	},
	"SVHA": {
		ICAO:      "SVHA",
		Name:      "Hato Altamira de Bolivar Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 1098,
		Latitude:  7.5999999046,
		Longitude: -63.2166671753,
		Timezone:  "America/Caracas",
	},
	"SVHC": {
		ICAO:      "SVHC",
		Name:      "Hacienda El Caimito Airport",
		State:     "Zulia",
		Country:   "VE",
		Elevation: 138,
		Latitude:  10.1499996185,
		Longitude: -72.1833343506,
		Timezone:  "America/Caracas",
	},
	"SVHD": {
		ICAO:      "SVHD",
		Name:      "Hacienda El Calvario Airport",
		State:     "Zulia",
		Country:   "VE",
		Elevation: 264,
		Latitude:  9.8999996185,
		Longitude: -72.5500030518,
		Timezone:  "America/Caracas",
	},
	"SVHE": {
		ICAO:      "SVHE",
		Name:      "Hato Rancho Alegre Airport",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 534,
		Latitude:  9.5666666031,
		Longitude: -67.5666656494,
		Timezone:  "America/Caracas",
	},
	"SVHG": {
		ICAO:      "SVHG",
		IATA:      "HGE",
		Name:      "Higuerote Airport",
		State:     "Miranda",
		Country:   "VE",
		Elevation: 10,
		Latitude:  10.4624528885,
		Longitude: -66.0927581787,
		Timezone:  "America/Caracas",
	},
	"SVHH": {
		ICAO:      "SVHH",
		Name:      "Churuguara Airport",
		City:      "Churuguara",
		State:     "Falcon",
		Country:   "VE",
		Elevation: 2461,
		Latitude:  10.8080997467,
		Longitude: -69.6383972168,
		Timezone:  "America/Caracas",
	},
	"SVHK": {
		ICAO:      "SVHK",
		Name:      "Hato la Guaca Airport",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 328,
		Latitude:  9.3999996185,
		Longitude: -67.7833328247,
		Timezone:  "America/Caracas",
	},
	"SVHL": {
		ICAO:      "SVHL",
		Name:      "Hacienda Carutal Airport",
		State:     "Aragua",
		Country:   "VE",
		Elevation: 1000,
		Latitude:  9.7166671753,
		Longitude: -66.7833328247,
		Timezone:  "America/Caracas",
	},
	"SVHM": {
		ICAO:      "SVHM",
		Name:      "Hato Los Mamones Airport",
		City:      "Hato Los Mamomes",
		State:     "Cojedes",
		Country:   "VE",
		Elevation: 594,
		Latitude:  9.3333330154,
		Longitude: -67.9166641235,
		Timezone:  "America/Caracas",
	},
	"SVHN": {
		ICAO:      "SVHN",
		Name:      "Hato La Chaconera Airport",
		State:     "Barinas",
		Country:   "VE",
		Elevation: 208,
		Latitude:  8.1054096222,
		Longitude: -68.7191009521,
		Timezone:  "America/Caracas",
	},
	"SVHO": {
		ICAO:      "SVHO",
		Name:      "El Oasis Airport",
		City:      "La Palmita",
		State:     "Barinas",
		Country:   "VE",
		Elevation: 396,
		Latitude:  8.3682203293,
		Longitude: -69.9570999146,
		Timezone:  "America/Caracas",
	},
	"SVHP": {
		ICAO:      "SVHP",
		Name:      "Hacienda El Paso Airport",
		City:      "Las Vegas del Tuy",
		State:     "Falcon",
		Country:   "VE",
		Elevation: 495,
		Latitude:  10.7133331299,
		Longitude: -69.2083358765,
		Timezone:  "America/Caracas",
	},
	"SVHR": {
		ICAO:      "SVHR",
		Name:      "Hato El Carito Airport",
		City:      "Los Marimares",
		State:     "Cojedes",
		Country:   "VE",
		Elevation: 256,
		Latitude:  9.5,
		Longitude: -67.9499969482,
		Timezone:  "America/Caracas",
	},
	"SVHS": {
		ICAO:      "SVHS",
		Name:      "Hato El Sesenta Airport",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 499,
		Latitude:  8.25,
		Longitude: -65.5833358765,
		Timezone:  "America/Caracas",
	},
	"SVHT": {
		ICAO:      "SVHT",
		Name:      "Hato Altamira de Monagas Airport",
		State:     "Monagas",
		Country:   "VE",
		Elevation: 154,
		Latitude:  9.3666667938,
		Longitude: -63.0833320618,
		Timezone:  "America/Caracas",
	},
	"SVHX": {
		ICAO:      "SVHX",
		Name:      "Hato La Candelaria Airport",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 254,
		Latitude:  8.6833333969,
		Longitude: -67.7166671753,
		Timezone:  "America/Caracas",
	},
	"SVHY": {
		ICAO:      "SVHY",
		Name:      "Hato Las Yeguas Airport",
		State:     "Cojedes",
		Country:   "VE",
		Elevation: 1049,
		Latitude:  8.720000267,
		Longitude: -68.4716644287,
		Timezone:  "America/Caracas",
	},
	"SVIC": {
		ICAO:      "SVIC",
		IATA:      "ICA",
		Name:      "Icabaru Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 1574,
		Latitude:  4.3363199234,
		Longitude: -61.7396011353,
		Timezone:  "America/Caracas",
	},
	"SVIE": {
		ICAO:      "SVIE",
		IATA:      "ICC",
		Name:      "Andres Miguel Salazar Marcano Airport",
		City:      "Isla de Coche",
		State:     "Nueva-Esparta",
		Country:   "VE",
		Elevation: 10,
		Latitude:  10.7944059372,
		Longitude: -63.981590271,
		Timezone:  "America/Caracas",
	},
	"SVIM": {
		ICAO:      "SVIM",
		Name:      "Hato El Caiman Airport",
		State:     "Portuguesa",
		Country:   "VE",
		Elevation: 462,
		Latitude:  9.1000003815,
		Longitude: -69.6200027466,
		Timezone:  "America/Caracas",
	},
	"SVJC": {
		ICAO:      "SVJC",
		IATA:      "LSP",
		Name:      "Josefa Camejo International Airport",
		City:      "Paraguana",
		State:     "Falcon",
		Country:   "VE",
		Elevation: 75,
		Latitude:  11.7807750702,
		Longitude: -70.1514968872,
		Timezone:  "America/Caracas",
	},
	"SVJF": {
		ICAO:      "SVJF",
		Name:      "Juan Florencio Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 187,
		Latitude:  7.4000000954,
		Longitude: -67.9833297729,
		Timezone:  "America/Caracas",
	},
	"SVJH": {
		ICAO:      "SVJH",
		Name:      "Juancho Airport",
		City:      "Soledad",
		State:     "Anzoategui",
		Country:   "VE",
		Elevation: 197,
		Latitude:  8.1797199249,
		Longitude: -63.5960006714,
		Timezone:  "America/Caracas",
	},
	"SVJI": {
		ICAO:      "SVJI",
		Name:      "Hato La Lejania Airport",
		State:     "Monagas",
		Country:   "VE",
		Elevation: 264,
		Latitude:  9.0971803665,
		Longitude: -63.118598938,
		Timezone:  "America/Caracas",
	},
	"SVJJ": {
		ICAO:      "SVJJ",
		Name:      "Curujujul Airport",
		City:      "Anzoategui",
		State:     "Apure",
		Country:   "VE",
		Elevation: 321,
		Latitude:  7.443333149,
		Longitude: -67.7966690063,
		Timezone:  "America/Caracas",
	},
	"SVJL": {
		ICAO:      "SVJL",
		Name:      "Los Lajeros Airport",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 132,
		Latitude:  8.016667366,
		Longitude: -67.3833312988,
		Timezone:  "America/Caracas",
	},
	"SVJM": {
		ICAO:      "SVJM",
		Name:      "San Juan de Los Morros Airport",
		City:      "San Juan de los Morros",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 1404,
		Latitude:  9.906952858,
		Longitude: -67.3796386719,
		Timezone:  "America/Caracas",
	},
	"SVJO": {
		ICAO:      "SVJO",
		Name:      "La Vigia Oeste Airport",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 98,
		Latitude:  9.6666669846,
		Longitude: -67.0500030518,
		Timezone:  "America/Caracas",
	},
	"SVJU": {
		ICAO:      "SVJU",
		Name:      "Hato La Laguna del Junco Airport",
		State:     "Cojedes",
		Country:   "VE",
		Elevation: 295,
		Latitude:  9.1833333969,
		Longitude: -68.0999984741,
		Timezone:  "America/Caracas",
	},
	"SVKA": {
		ICAO:      "SVKA",
		IATA:      "KAV",
		Name:      "Kavanayen Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 3900,
		Latitude:  5.632999897,
		Longitude: -61.783000946,
		Timezone:  "America/Caracas",
	},
	"SVKB": {
		ICAO:      "SVKB",
		Name:      "La Calzada Airport",
		State:     "Barinas",
		Country:   "VE",
		Elevation: 291,
		Latitude:  7.9833331108,
		Longitude: -70.0999984741,
		Timezone:  "America/Caracas",
	},
	"SVKC": {
		ICAO:      "SVKC",
		Name:      "El Carrao Airport",
		State:     "Portuguesa",
		Country:   "VE",
		Elevation: 240,
		Latitude:  8.5399999619,
		Longitude: -68.8050003052,
		Timezone:  "America/Caracas",
	},
	"SVKI": {
		ICAO:      "SVKI",
		Name:      "Piscuri Airport",
		State:     "Tachira",
		Country:   "VE",
		Elevation: 726,
		Latitude:  7.5466599464,
		Longitude: -71.7883987427,
		Timezone:  "America/Caracas",
	},
	"SVKL": {
		ICAO:      "SVKL",
		Name:      "Las Clavellinas Airport",
		City:      "Las Clavellinasmer",
		State:     "Trujillo",
		Country:   "VE",
		Elevation: 66,
		Latitude:  9.3249998093,
		Longitude: -70.9749984741,
		Timezone:  "America/Caracas",
	},
	"SVKN": {
		ICAO:      "SVKN",
		Name:      "Cano Negro Airport",
		State:     "Amazonas",
		Country:   "VE",
		Elevation: 328,
		Latitude:  5.5,
		Longitude: -65.6183319092,
		Timezone:  "America/Caracas",
	},
	"SVKV": {
		ICAO:      "SVKV",
		Name:      "Corral Viejo Airport",
		City:      "La Bombita",
		State:     "Trujillo",
		Country:   "VE",
		Elevation: 14,
		Latitude:  9.6666669846,
		Longitude: -70.9666671753,
		Timezone:  "America/Caracas",
	},
	"SVLA": {
		ICAO:      "SVLA",
		Name:      "Las Adjuntas Airport",
		City:      "Las Colmenas",
		State:     "Monagas",
		Country:   "VE",
		Elevation: 534,
		Latitude:  9.016667366,
		Longitude: -63.75,
		Timezone:  "America/Caracas",
	},
	"SVLB": {
		ICAO:      "SVLB",
		Name:      "La Blanquilla Airport",
		City:      "La Blanquilla - Isla Margarita",
		State:     "Dependencias-Federales",
		Country:   "VE",
		Elevation: 59,
		Latitude:  11.8205785751,
		Longitude: -64.5862121582,
		Timezone:  "America/Caracas",
	},
	"SVLC": {
		ICAO:      "SVLC",
		Name:      "El Lucero de Apure Airport",
		City:      "El Lucero",
		State:     "Apure",
		Country:   "VE",
		Elevation: 262,
		Latitude:  6.9833331108,
		Longitude: -68.0666656494,
		Timezone:  "America/Caracas",
	},
	"SVLD": {
		ICAO:      "SVLD",
		Name:      "Las Brisas Airport",
		City:      "Mata de Vino",
		State:     "Apure",
		Country:   "VE",
		Elevation: 328,
		Latitude:  7.203332901,
		Longitude: -69.5366668701,
		Timezone:  "America/Caracas",
	},
	"SVLE": {
		ICAO:      "SVLE",
		Name:      "El Lucero del Zulia Airport",
		State:     "Zulia",
		Country:   "VE",
		Elevation: 33,
		Latitude:  10.4499998093,
		Longitude: -71.3666687012,
		Timezone:  "America/Caracas",
	},
	"SVLF": {
		ICAO:      "SVLF",
		IATA:      "LFR",
		Name:      "La Fria Airport",
		State:     "Tachira",
		Country:   "VE",
		Elevation: 305,
		Latitude:  8.2391672134,
		Longitude: -72.2710266113,
		Timezone:  "America/Caracas",
	},
	"SVLG": {
		ICAO:      "SVLG",
		Name:      "Las Majaguas Airport",
		State:     "Portuguesa",
		Country:   "VE",
		Elevation: 793,
		Latitude:  9.6499996185,
		Longitude: -68.9833297729,
		Timezone:  "America/Caracas",
	},
	"SVLH": {
		ICAO:      "SVLH",
		Name:      "Las Delicias Airport",
		City:      "Banco del Medio",
		State:     "Apure",
		Country:   "VE",
		Elevation: 373,
		Latitude:  7.1666669846,
		Longitude: -69.3833312988,
		Timezone:  "America/Caracas",
	},
	"SVLI": {
		ICAO:      "SVLI",
		Name:      "La Alcancia Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 46,
		Latitude:  7.3000001907,
		Longitude: -67.3333358765,
		Timezone:  "America/Caracas",
	},
	"SVLJ": {
		ICAO:      "SVLJ",
		Name:      "La Vieja Airport",
		City:      "La Ceiba",
		State:     "Anzoategui",
		Country:   "VE",
		Elevation: 1262,
		Latitude:  9.6833333969,
		Longitude: -64.1833343506,
		Timezone:  "America/Caracas",
	},
	"SVLK": {
		ICAO:      "SVLK",
		Name:      "Hato Macolla Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 296,
		Latitude:  6.1666669846,
		Longitude: -69.1666641235,
		Timezone:  "America/Bogota",
	},
	"SVLL": {
		ICAO:      "SVLL",
		Name:      "Callejas Airport",
		City:      "Callejas",
		State:     "Barinas",
		Country:   "VE",
		Elevation: 209,
		Latitude:  8.0083332062,
		Longitude: -69.626663208,
		Timezone:  "America/Caracas",
	},
	"SVLM": {
		ICAO:      "SVLM",
		Name:      "Las Flores Airport",
		State:     "Monagas",
		Country:   "VE",
		Elevation: 118,
		Latitude:  9.4979400635,
		Longitude: -62.9013061523,
		Timezone:  "America/Caracas",
	},
	"SVLN": {
		ICAO:      "SVLN",
		Name:      "La Candelaria Airport",
		State:     "Falcon",
		Country:   "VE",
		Elevation: 420,
		Latitude:  10.7266998291,
		Longitude: -68.3335037231,
		Timezone:  "America/Caracas",
	},
	"SVLO": {
		ICAO:      "SVLO",
		Name:      "La Orchila Airport",
		State:     "Dependencias-Federales",
		Country:   "VE",
		Elevation: 5,
		Latitude:  11.8072004318,
		Longitude: -66.1796035767,
		Timezone:  "America/Caracas",
	},
	"SVLP": {
		ICAO:      "SVLP",
		Name:      "Las Lagunitas Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 344,
		Latitude:  6.9333329201,
		Longitude: -68.3666687012,
		Timezone:  "America/Caracas",
	},
	"SVLS": {
		ICAO:      "SVLS",
		Name:      "Las Corobas Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 295,
		Latitude:  6.7666668892,
		Longitude: -68.3499984741,
		Timezone:  "America/Caracas",
	},
	"SVLT": {
		ICAO:      "SVLT",
		Name:      "La Argentina Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 270,
		Latitude:  7.5833330154,
		Longitude: -69.1833343506,
		Timezone:  "America/Caracas",
	},
	"SVLU": {
		ICAO:      "SVLU",
		Name:      "El Lechozo Airport",
		City:      "El Lechozo",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 529,
		Latitude:  9.3000001907,
		Longitude: -66.6783294678,
		Timezone:  "America/Caracas",
	},
	"SVLW": {
		ICAO:      "SVLW",
		Name:      "Los Aguacates Airport",
		State:     "Carabobo",
		Country:   "VE",
		Elevation: 1540,
		Latitude:  10.0683326721,
		Longitude: -68.0216674805,
		Timezone:  "America/Caracas",
	},
	"SVLX": {
		ICAO:      "SVLX",
		Name:      "La Yaguita Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 295,
		Latitude:  6.9666671753,
		Longitude: -69.3499984741,
		Timezone:  "America/Caracas",
	},
	"SVLY": {
		ICAO:      "SVLY",
		Name:      "La Yagua Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 393,
		Latitude:  7.2833328247,
		Longitude: -68.5333328247,
		Timezone:  "America/Caracas",
	},
	"SVLZ": {
		ICAO:      "SVLZ",
		Name:      "Las Cruces Airport",
		State:     "Portuguesa",
		Country:   "VE",
		Elevation: 494,
		Latitude:  8.3288803101,
		Longitude: -68.7519989014,
		Timezone:  "America/Caracas",
	},
	"SVMA": {
		ICAO:      "SVMA",
		Name:      "La Mona Airport",
		State:     "Carabobo",
		Country:   "VE",
		Elevation: 2140,
		Latitude:  10.1431999207,
		Longitude: -68.1978988647,
		Timezone:  "America/Caracas",
	},
	"SVMB": {
		ICAO:      "SVMB",
		Name:      "El Embrujo Airport",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 720,
		Latitude:  8.671667099,
		Longitude: -66.5049972534,
		Timezone:  "America/Caracas",
	},
	"SVMC": {
		ICAO:      "SVMC",
		IATA:      "MAR",
		Name:      "La Chinita International Airport",
		City:      "Maracaibo",
		State:     "Zulia",
		Country:   "VE",
		Elevation: 239,
		Latitude:  10.5582084656,
		Longitude: -71.7278594971,
		Timezone:  "America/Caracas",
	},
	"SVMD": {
		ICAO:      "SVMD",
		IATA:      "MRD",
		Name:      "Alberto Carnevalli Airport",
		City:      "Merida",
		State:     "Merida",
		Country:   "VE",
		Elevation: 5007,
		Latitude:  8.58207798,
		Longitude: -71.1610412598,
		Timezone:  "America/Caracas",
	},
	"SVME": {
		ICAO:      "SVME",
		Name:      "Mapire Airport",
		State:     "Anzoategui",
		Country:   "VE",
		Elevation: 196,
		Latitude:  7.738629818,
		Longitude: -64.7194976807,
		Timezone:  "America/Caracas",
	},
	"SVMF": {
		ICAO:      "SVMF",
		Name:      "Mata Charo Airport",
		State:     "Barinas",
		Country:   "VE",
		Elevation: 377,
		Latitude:  8.5333328247,
		Longitude: -69.9499969482,
		Timezone:  "America/Caracas",
	},
	"SVMG": {
		ICAO:      "SVMG",
		IATA:      "PMV",
		Name:      "Del Caribe Santiago Marino International Airport",
		City:      "Isla Margarita",
		State:     "Nueva-Esparta",
		Country:   "VE",
		Elevation: 74,
		Latitude:  10.9126033783,
		Longitude: -63.9665985107,
		Timezone:  "America/Caracas",
	},
	"SVMH": {
		ICAO:      "SVMH",
		Name:      "Macanillal Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 426,
		Latitude:  7.3166670799,
		Longitude: -68,
		Timezone:  "America/Caracas",
	},
	"SVMI": {
		ICAO:      "SVMI",
		IATA:      "CCS",
		Name:      "Maiquetia (Simon Bolivar Internacional) Airport",
		City:      "Caracas",
		State:     "Vargas",
		Country:   "VE",
		Elevation: 235,
		Latitude:  10.6031169891,
		Longitude: -66.9905853271,
		Timezone:  "America/Caracas",
	},
	"SVMJ": {
		ICAO:      "SVMJ",
		Name:      "Hacienda Las Lomas Airport",
		State:     "Yaracuy",
		Country:   "VE",
		Elevation: 214,
		Latitude:  10.5983333588,
		Longitude: -68.5350036621,
		Timezone:  "America/Caracas",
	},
	"SVMK": {
		ICAO:      "SVMK",
		Name:      "El Milagro Cojedes Airport",
		State:     "Cojedes",
		Country:   "VE",
		Elevation: 262,
		Latitude:  9.2299995422,
		Longitude: -67.9700012207,
		Timezone:  "America/Caracas",
	},
	"SVML": {
		ICAO:      "SVML",
		Name:      "El Milagro Sureste Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 230,
		Latitude:  7.2666668892,
		Longitude: -67.2833328247,
		Timezone:  "America/Caracas",
	},
	"SVMM": {
		ICAO:      "SVMM",
		Name:      "Mamon Airport",
		State:     "Falcon",
		Country:   "VE",
		Elevation: 151,
		Latitude:  11.266667366,
		Longitude: -70.25,
		Timezone:  "America/Caracas",
	},
	"SVMN": {
		ICAO:      "SVMN",
		Name:      "Mene Grande Airport",
		City:      "Mene Grande",
		State:     "Zulia",
		Country:   "VE",
		Elevation: 90,
		Latitude:  9.8031597137,
		Longitude: -70.8921966553,
		Timezone:  "America/Caracas",
	},
	"SVMO": {
		ICAO:      "SVMO",
		Name:      "El Milagro Oeste Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 295,
		Latitude:  7.6833329201,
		Longitude: -68.1833343506,
		Timezone:  "America/Caracas",
	},
	"SVMP": {
		ICAO:      "SVMP",
		Name:      "Metropolitano Airport",
		State:     "Miranda",
		Country:   "VE",
		Elevation: 574,
		Latitude:  10.1331691742,
		Longitude: -66.7878265381,
		Timezone:  "America/Caracas",
	},
	"SVMQ": {
		ICAO:      "SVMQ",
		Name:      "Fundo la Marquesena Airport",
		State:     "Barinas",
		Country:   "VE",
		Elevation: 646,
		Latitude:  8.8416671753,
		Longitude: -70.0333328247,
		Timezone:  "America/Caracas",
	},
	"SVMS": {
		ICAO:      "SVMS",
		Name:      "Guasimal Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 328,
		Latitude:  7.7166671753,
		Longitude: -67.9833297729,
		Timezone:  "America/Caracas",
	},
	"SVMT": {
		ICAO:      "SVMT",
		IATA:      "MUN",
		Name:      "Maturin Airport",
		State:     "Anzoategui",
		Country:   "VE",
		Elevation: 224,
		Latitude:  9.754529953,
		Longitude: -63.1473999023,
		Timezone:  "America/Caracas",
	},
	"SVMV": {
		ICAO:      "SVMV",
		Name:      "Maroa Airport",
		State:     "Amazonas",
		Country:   "VE",
		Elevation: 501,
		Latitude:  2.6925001144,
		Longitude: -67.5188980103,
		Timezone:  "America/Caracas",
	},
	"SVMW": {
		ICAO:      "SVMW",
		Name:      "Los Malabares Airport",
		State:     "Tachira",
		Country:   "VE",
		Elevation: 98,
		Latitude:  7.6166667938,
		Longitude: -72.4499969482,
		Timezone:  "America/Caracas",
	},
	"SVMX": {
		ICAO:      "SVMX",
		Name:      "Mata Airport",
		City:      "Las Fernandez",
		State:     "Anzoategui",
		Country:   "VE",
		Elevation: 695,
		Latitude:  9.1999998093,
		Longitude: -64.0500030518,
		Timezone:  "America/Caracas",
	},
	"SVMY": {
		ICAO:      "SVMY",
		Name:      "Manapiare Airport",
		State:     "Amazonas",
		Country:   "VE",
		Elevation: 656,
		Latitude:  5.3083329201,
		Longitude: -66.0500030518,
		Timezone:  "America/Caracas",
	},
	"SVMZ": {
		ICAO:      "SVMZ",
		Name:      "Mantecal Airport",
		City:      "Mantecal",
		State:     "Apure",
		Country:   "VE",
		Elevation: 295,
		Latitude:  7.5575799942,
		Longitude: -69.1416015625,
		Timezone:  "America/Caracas",
	},
	"SVNA": {
		ICAO:      "SVNA",
		Name:      "Macana Airport",
		State:     "Zulia",
		Country:   "VE",
		Elevation: 200,
		Latitude:  9.9230203629,
		Longitude: -72.0804977417,
		Timezone:  "America/Caracas",
	},
	"SVNB": {
		ICAO:      "SVNB",
		Name:      "Maripa Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 200,
		Latitude:  7.4000000954,
		Longitude: -65.1833343506,
		Timezone:  "America/Caracas",
	},
	"SVNE": {
		ICAO:      "SVNE",
		Name:      "Las Mercedes Airport",
		State:     "Barinas",
		Country:   "VE",
		Elevation: 492,
		Latitude:  7.7666668892,
		Longitude: -70.2666702271,
		Timezone:  "America/Caracas",
	},
	"SVNF": {
		ICAO:      "SVNF",
		Name:      "Morichitos Airport",
		City:      "Morichito",
		State:     "Apure",
		Country:   "VE",
		Elevation: 210,
		Latitude:  7.3833332062,
		Longitude: -68.8333358765,
		Timezone:  "America/Caracas",
	},
	"SVNG": {
		ICAO:      "SVNG",
		Name:      "El Mango Airport",
		State:     "Zulia",
		Country:   "VE",
		Elevation: 393,
		Latitude:  10.2833328247,
		Longitude: -72.4666671753,
		Timezone:  "America/Caracas",
	},
	"SVNJ": {
		ICAO:      "SVNJ",
		Name:      "Morichito II Airport",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 361,
		Latitude:  8.9666671753,
		Longitude: -66.0833358765,
		Timezone:  "America/Caracas",
	},
	"SVNK": {
		ICAO:      "SVNK",
		Name:      "Hato Mikitole Airport",
		State:     "Anzoategui",
		Country:   "VE",
		Elevation: 328,
		Latitude:  8.4177780151,
		Longitude: -63.567779541,
		Timezone:  "America/Caracas",
	},
	"SVNP": {
		ICAO:      "SVNP",
		Name:      "Marisol Airport",
		State:     "Portuguesa",
		Country:   "VE",
		Elevation: 328,
		Latitude:  9.1333332062,
		Longitude: -69.0550003052,
		Timezone:  "America/Caracas",
	},
	"SVNR": {
		ICAO:      "SVNR",
		Name:      "El Morichal Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 492,
		Latitude:  6.9833331108,
		Longitude: -68.6333312988,
		Timezone:  "America/Caracas",
	},
	"SVNT": {
		ICAO:      "SVNT",
		Name:      "Mata de Turagua Airport",
		City:      "La Guarandinga",
		State:     "Apure",
		Country:   "VE",
		Elevation: 354,
		Latitude:  7.6999998093,
		Longitude: -69.3000030518,
		Timezone:  "America/Caracas",
	},
	"SVNV": {
		ICAO:      "SVNV",
		Name:      "Las Nieves Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 866,
		Latitude:  6.5833330154,
		Longitude: -66.1999969482,
		Timezone:  "America/Caracas",
	},
	"SVNX": {
		ICAO:      "SVNX",
		Name:      "Morichal Airport",
		State:     "Monagas",
		Country:   "VE",
		Elevation: 272,
		Latitude:  8.8281497955,
		Longitude: -63.0950012207,
		Timezone:  "America/Caracas",
	},
	"SVOA": {
		ICAO:      "SVOA",
		Name:      "San Lorenzo De Barinas Airport",
		State:     "Barinas",
		Country:   "VE",
		Elevation: 374,
		Latitude:  8.2847223282,
		Longitude: -69.9166641235,
		Timezone:  "America/Caracas",
	},
	"SVOB": {
		ICAO:      "SVOB",
		Name:      "San Lorenzo De Falcon Airport",
		State:     "Falcon",
		Country:   "VE",
		Elevation: 137,
		Latitude:  11.0500001907,
		Longitude: -68.4833297729,
		Timezone:  "America/Caracas",
	},
	"SVOC": {
		ICAO:      "SVOC",
		Name:      "San Luis Capanaparo Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 202,
		Latitude:  6.9666671753,
		Longitude: -68.3333358765,
		Timezone:  "America/Caracas",
	},
	"SVOD": {
		ICAO:      "SVOD",
		Name:      "Santa Ana Airport",
		State:     "Barinas",
		Country:   "VE",
		Elevation: 262,
		Latitude:  7.9833331108,
		Longitude: -67.6500015259,
		Timezone:  "America/Caracas",
	},
	"SVOE": {
		ICAO:      "SVOE",
		Name:      "Sabaneta Airport",
		City:      "Sabaneta",
		State:     "Barinas",
		Country:   "VE",
		Elevation: 530,
		Latitude:  8.7441396713,
		Longitude: -69.9138031006,
		Timezone:  "America/Caracas",
	},
	"SVOF": {
		ICAO:      "SVOF",
		Name:      "Santa Barbara de Monagas Airport",
		State:     "Monagas",
		Country:   "VE",
		Elevation: 804,
		Latitude:  9.6984996796,
		Longitude: -63.6222991943,
		Timezone:  "America/Caracas",
	},
	"SVOG": {
		ICAO:      "SVOG",
		Name:      "Santa Barbara del Amazonas Airport",
		State:     "Amazonas",
		Country:   "VE",
		Elevation: 328,
		Latitude:  3.9416670799,
		Longitude: -67.0783309937,
		Timezone:  "America/Caracas",
	},
	"SVOH": {
		ICAO:      "SVOH",
		Name:      "San Silvestre Airport",
		State:     "Barinas",
		Country:   "VE",
		Elevation: 350,
		Latitude:  8.3500003815,
		Longitude: -70.0500030518,
		Timezone:  "America/Caracas",
	},
	"SVOI": {
		ICAO:      "SVOI",
		Name:      "Los Indios Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 328,
		Latitude:  7.3000001907,
		Longitude: -68.0666656494,
		Timezone:  "America/Caracas",
	},
	"SVOJ": {
		ICAO:      "SVOJ",
		Name:      "Ojo de Agua Airport",
		State:     "Barinas",
		Country:   "VE",
		Elevation: 328,
		Latitude:  8.0666666031,
		Longitude: -70.1833343506,
		Timezone:  "America/Caracas",
	},
	"SVOK": {
		ICAO:      "SVOK",
		Name:      "Oritupano Airport",
		State:     "Monagas",
		Country:   "VE",
		Elevation: 485,
		Latitude:  9.0616674423,
		Longitude: -63.4449996948,
		Timezone:  "America/Caracas",
	},
	"SVOL": {
		ICAO:      "SVOL",
		Name:      "San Roque Airport",
		City:      "Santa Ana",
		State:     "Anzoategui",
		Country:   "VE",
		Elevation: 590,
		Latitude:  9.3166666031,
		Longitude: -64.6333312988,
		Timezone:  "America/Caracas",
	},
	"SVON": {
		ICAO:      "SVON",
		IATA:      "CBS",
		Name:      "Oro Negro Airport",
		City:      "Cabimas",
		State:     "Zulia",
		Country:   "VE",
		Elevation: 164,
		Latitude:  10.3306999207,
		Longitude: -71.3225021362,
		Timezone:  "America/Caracas",
	},
	"SVOO": {
		ICAO:      "SVOO",
		Name:      "Hato El Rosero Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 262,
		Latitude:  7.6500000954,
		Longitude: -69.4166641235,
		Timezone:  "America/Caracas",
	},
	"SVOP": {
		ICAO:      "SVOP",
		Name:      "Los Oripopos Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 278,
		Latitude:  7.5999999046,
		Longitude: -68.4333343506,
		Timezone:  "America/Caracas",
	},
	"SVOQ": {
		ICAO:      "SVOQ",
		Name:      "Santa Monica Airport",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 836,
		Latitude:  8.6666669846,
		Longitude: -67.6666641235,
		Timezone:  "America/Caracas",
	},
	"SVOR": {
		ICAO:      "SVOR",
		Name:      "San Ramon Airport",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 231,
		Latitude:  8.0916671753,
		Longitude: -66.4216690063,
		Timezone:  "America/Caracas",
	},
	"SVOS": {
		ICAO:      "SVOS",
		Name:      "Los Cocos Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 328,
		Latitude:  7.4666671753,
		Longitude: -68.6666641235,
		Timezone:  "America/Caracas",
	},
	"SVOT": {
		ICAO:      "SVOT",
		Name:      "San Pablo De Barinas Airport",
		State:     "Barinas",
		Country:   "VE",
		Elevation: 390,
		Latitude:  7.7666668892,
		Longitude: -70.6333312988,
		Timezone:  "America/Caracas",
	},
	"SVOW": {
		ICAO:      "SVOW",
		Name:      "Coco de Mono Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 380,
		Latitude:  7.1999998093,
		Longitude: -68.5166702271,
		Timezone:  "America/Caracas",
	},
	"SVOZ": {
		ICAO:      "SVOZ",
		Name:      "San Pablo Paeno Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 209,
		Latitude:  7.7833328247,
		Longitude: -68.5,
		Timezone:  "America/Caracas",
	},
	"SVPA": {
		ICAO:      "SVPA",
		IATA:      "PYH",
		Name:      "Cacique Aramare Airport",
		State:     "Amazonas",
		Country:   "VE",
		Elevation: 245,
		Latitude:  5.619989872,
		Longitude: -67.6061019897,
		Timezone:  "America/Bogota",
	},
	"SVPB": {
		ICAO:      "SVPB",
		Name:      "Punta Brava Airport",
		Country:   "VE",
		Elevation: 19,
		Latitude:  10.7839002609,
		Longitude: -68.3436965942,
		Timezone:  "America/Caracas",
	},
	"SVPC": {
		ICAO:      "SVPC",
		IATA:      "PBL",
		Name:      "General Bartolome Salom International Airport",
		State:     "Carabobo",
		Country:   "VE",
		Elevation: 32,
		Latitude:  10.4805002213,
		Longitude: -68.0729980469,
		Timezone:  "America/Caracas",
	},
	"SVPD": {
		ICAO:      "SVPD",
		Name:      "Hato La Ponderosa Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 257,
		Latitude:  7.0999999046,
		Longitude: -67.9000015259,
		Timezone:  "America/Caracas",
	},
	"SVPE": {
		ICAO:      "SVPE",
		IATA:      "PDZ",
		Name:      "Pedernales Airport",
		State:     "Delta-Amacuro",
		Country:   "VE",
		Elevation: 25,
		Latitude:  9.9792404175,
		Longitude: -62.2285995483,
		Timezone:  "America/Caracas",
	},
	"SVPF": {
		ICAO:      "SVPF",
		Name:      "Pariaguan Airport",
		City:      "Pariaguan",
		State:     "Anzoategui",
		Country:   "VE",
		Elevation: 787,
		Latitude:  8.8191204071,
		Longitude: -64.7397003174,
		Timezone:  "America/Caracas",
	},
	"SVPG": {
		ICAO:      "SVPG",
		Name:      "El Progreso Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 984,
		Latitude:  7.1333332062,
		Longitude: -68.5833358765,
		Timezone:  "America/Caracas",
	},
	"SVPH": {
		ICAO:      "SVPH",
		IATA:      "PPH",
		Name:      "Perai Tepuy Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 2797,
		Latitude:  4.5666670799,
		Longitude: -61.4833335876,
		Timezone:  "America/Caracas",
	},
	"SVPI": {
		ICAO:      "SVPI",
		Name:      "El Pauji Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 2716,
		Latitude:  4.4666671753,
		Longitude: -61.593334198,
		Timezone:  "America/Caracas",
	},
	"SVPJ": {
		ICAO:      "SVPJ",
		Name:      "Pesurca Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 726,
		Latitude:  6.2833328247,
		Longitude: -69.3166656494,
		Timezone:  "America/Caracas",
	},
	"SVPK": {
		ICAO:      "SVPK",
		Name:      "Las Palmeras Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 262,
		Latitude:  7.6333332062,
		Longitude: -69.3666687012,
		Timezone:  "America/Caracas",
	},
	"SVPL": {
		ICAO:      "SVPL",
		Name:      "El Pueblito Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 1050,
		Latitude:  7.8166670799,
		Longitude: -62.7166671753,
		Timezone:  "America/Caracas",
	},
	"SVPM": {
		ICAO:      "SVPM",
		IATA:      "SCI",
		Name:      "Paramillo Airport",
		State:     "Tachira",
		Country:   "VE",
		Elevation: 3314,
		Latitude:  7.801320076,
		Longitude: -72.2029037476,
		Timezone:  "America/Caracas",
	},
	"SVPN": {
		ICAO:      "SVPN",
		Name:      "Espino Airport",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 689,
		Latitude:  8.5635004044,
		Longitude: -66.0183029175,
		Timezone:  "America/Caracas",
	},
	"SVPO": {
		ICAO:      "SVPO",
		Name:      "Hato El Porvenir Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 295,
		Latitude:  7.5500001907,
		Longitude: -68.25,
		Timezone:  "America/Caracas",
	},
	"SVPP": {
		ICAO:      "SVPP",
		Name:      "Luepa Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 4250,
		Latitude:  5.7926797867,
		Longitude: -61.4402008057,
		Timezone:  "America/Caracas",
	},
	"SVPQ": {
		ICAO:      "SVPQ",
		Name:      "El Pardillero Airport",
		State:     "Barinas",
		Country:   "VE",
		Elevation: 2001,
		Latitude:  8.5982704163,
		Longitude: -69.7608032227,
		Timezone:  "America/Caracas",
	},
	"SVPR": {
		ICAO:      "SVPR",
		IATA:      "PZO",
		Name:      "General Manuel Carlos Piar International Airport",
		City:      "Puerto Ordaz-Ciudad Guayana",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 472,
		Latitude:  8.2885303497,
		Longitude: -62.7603988647,
		Timezone:  "America/Caracas",
	},
	"SVPS": {
		ICAO:      "SVPS",
		Name:      "Hacienda La Pastora Airport",
		City:      "San Pedro",
		State:     "Lara",
		Country:   "VE",
		Elevation: 1084,
		Latitude:  9.8900003433,
		Longitude: -70.1216659546,
		Timezone:  "America/Caracas",
	},
	"SVPT": {
		ICAO:      "SVPT",
		IATA:      "PTM",
		Name:      "Palmarito Airport",
		City:      "Palmarito",
		State:     "Apure",
		Country:   "VE",
		Elevation: 347,
		Latitude:  7.5666699409,
		Longitude: -70.1832962036,
		Timezone:  "America/Caracas",
	},
	"SVPU": {
		ICAO:      "SVPU",
		Name:      "La Paragua Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 797,
		Latitude:  6.8499999046,
		Longitude: -63.3166656494,
		Timezone:  "America/Caracas",
	},
	"SVPV": {
		ICAO:      "SVPV",
		Name:      "Pedraza La Vieja Airport",
		City:      "Pedraza La Vieja",
		State:     "Barinas",
		Country:   "VE",
		Elevation: 554,
		Latitude:  7.9166669846,
		Longitude: -71.0333328247,
		Timezone:  "America/Caracas",
	},
	"SVPW": {
		ICAO:      "SVPW",
		Name:      "Pelayo Airport",
		State:     "Anzoategui",
		Country:   "VE",
		Elevation: 574,
		Latitude:  9,
		Longitude: -63.6666679382,
		Timezone:  "America/Caracas",
	},
	"SVPX": {
		ICAO:      "SVPX",
		Name:      "El Pinal Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 268,
		Latitude:  7.3666667938,
		Longitude: -68.6833343506,
		Timezone:  "America/Caracas",
	},
	"SVPY": {
		ICAO:      "SVPY",
		Name:      "Palma Sola Airport",
		State:     "Carabobo",
		Country:   "VE",
		Elevation: 32,
		Latitude:  10.4899997711,
		Longitude: -68.1750030518,
		Timezone:  "America/Caracas",
	},
	"SVPZ": {
		ICAO:      "SVPZ",
		Name:      "La Paz Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 344,
		Latitude:  7.4883332253,
		Longitude: -68.9933319092,
		Timezone:  "America/Caracas",
	},
	"SVQA": {
		ICAO:      "SVQA",
		Name:      "Santa Rosalia Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 446,
		Latitude:  7.4666671753,
		Longitude: -65.5666656494,
		Timezone:  "America/Caracas",
	},
	"SVQB": {
		ICAO:      "SVQB",
		Name:      "Santa Elena de Rio Claro Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 219,
		Latitude:  7.0500001907,
		Longitude: -67.7666702271,
		Timezone:  "America/Caracas",
	},
	"SVQC": {
		ICAO:      "SVQC",
		Name:      "Santa Maria La Tigra Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 164,
		Latitude:  6.5,
		Longitude: -67.1333312988,
		Timezone:  "America/Caracas",
	},
	"SVQD": {
		ICAO:      "SVQD",
		Name:      "Santa Juana Airport",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 705,
		Latitude:  8.9583330154,
		Longitude: -65.1383361816,
		Timezone:  "America/Caracas",
	},
	"SVQI": {
		ICAO:      "SVQI",
		Name:      "Tucupido Airport",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 377,
		Latitude:  9.3500003815,
		Longitude: -65.8000030518,
		Timezone:  "America/Caracas",
	},
	"SVQJ": {
		ICAO:      "SVQJ",
		Name:      "La Trinidad De Apure Airport",
		City:      "La Trinidad",
		State:     "Apure",
		Country:   "VE",
		Elevation: 267,
		Latitude:  7.2333331108,
		Longitude: -69.0999984741,
		Timezone:  "America/Caracas",
	},
	"SVQK": {
		ICAO:      "SVQK",
		Name:      "Tierra Negra Airport",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 924,
		Latitude:  9.6783332825,
		Longitude: -66.1633300781,
		Timezone:  "America/Caracas",
	},
	"SVQL": {
		ICAO:      "SVQL",
		Name:      "Tocomita Airport",
		City:      "Guri",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 590,
		Latitude:  7.7577600479,
		Longitude: -63.0822982788,
		Timezone:  "America/Caracas",
	},
	"SVQN": {
		ICAO:      "SVQN",
		Name:      "Torunos Airport",
		State:     "Barinas",
		Country:   "VE",
		Elevation: 337,
		Latitude:  8.3166666031,
		Longitude: -70.0500030518,
		Timezone:  "America/Caracas",
	},
	"SVQO": {
		ICAO:      "SVQO",
		Name:      "Tocuyito Airport",
		State:     "Carabobo",
		Country:   "VE",
		Elevation: 1710,
		Latitude:  10.0823001862,
		Longitude: -68.1190032959,
		Timezone:  "America/Caracas",
	},
	"SVQP": {
		ICAO:      "SVQP",
		Name:      "Miquelon Airport",
		State:     "Zulia",
		Country:   "VE",
		Elevation: 18,
		Latitude:  9.0500001907,
		Longitude: -71.368057251,
		Timezone:  "America/Caracas",
	},
	"SVQQ": {
		ICAO:      "SVQQ",
		Name:      "Los Conucos Airport",
		State:     "Anzoategui",
		Country:   "VE",
		Elevation: 262,
		Latitude:  9.8500003815,
		Longitude: -65.3166656494,
		Timezone:  "America/Caracas",
	},
	"SVQR": {
		ICAO:      "SVQR",
		Name:      "Santa Rita Airport",
		City:      "El Diero",
		State:     "Apure",
		Country:   "VE",
		Elevation: 367,
		Latitude:  7.5166668892,
		Longitude: -68.6833343506,
		Timezone:  "America/Caracas",
	},
	"SVQS": {
		ICAO:      "SVQS",
		Name:      "Santa Rita Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 210,
		Latitude:  7.6666669846,
		Longitude: -66.8000030518,
		Timezone:  "America/Caracas",
	},
	"SVQT": {
		ICAO:      "SVQT",
		Name:      "Los Quitasoles Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 492,
		Latitude:  7.6333332062,
		Longitude: -69.9250030518,
		Timezone:  "America/Caracas",
	},
	"SVQU": {
		ICAO:      "SVQU",
		Name:      "Santa Rosa de Guanare Airport",
		City:      "San Isidro",
		State:     "Portuguesa",
		Country:   "VE",
		Elevation: 429,
		Latitude:  8.7495803833,
		Longitude: -69.7225036621,
		Timezone:  "America/Caracas",
	},
	"SVQW": {
		ICAO:      "SVQW",
		Name:      "Tamatal Airport",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 379,
		Latitude:  8.8333330154,
		Longitude: -66.3666687012,
		Timezone:  "America/Caracas",
	},
	"SVQX": {
		ICAO:      "SVQX",
		Name:      "Tamayare Airport",
		City:      "El Paso",
		State:     "Lara",
		Country:   "VE",
		Elevation: 2100,
		Latitude:  10.5666666031,
		Longitude: -70.1333312988,
		Timezone:  "America/Caracas",
	},
	"SVQY": {
		ICAO:      "SVQY",
		Name:      "Tapaquire Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 475,
		Latitude:  7.9099998474,
		Longitude: -64.0583343506,
		Timezone:  "America/Caracas",
	},
	"SVQZ": {
		ICAO:      "SVQZ",
		Name:      "Santa Rosa de Barinas Airport",
		State:     "Barinas",
		Country:   "VE",
		Elevation: 354,
		Latitude:  8.4333333969,
		Longitude: -69.7333297729,
		Timezone:  "America/Caracas",
	},
	"SVRA": {
		ICAO:      "SVRA",
		Name:      "Rio de Agua Airport",
		State:     "Sucre",
		Country:   "VE",
		Elevation: 33,
		Latitude:  10.5717000961,
		Longitude: -62.9922981262,
		Timezone:  "America/Caracas",
	},
	"SVRB": {
		ICAO:      "SVRB",
		Name:      "Cumarebo Airport",
		State:     "Falcon",
		Country:   "VE",
		Elevation: 26,
		Latitude:  11.4866666794,
		Longitude: -69.3550033569,
		Timezone:  "America/Caracas",
	},
	"SVRC": {
		ICAO:      "SVRC",
		Name:      "El Rocio Airport",
		State:     "Falcon",
		Country:   "VE",
		Elevation: 328,
		Latitude:  10.7833328247,
		Longitude: -68.5500030518,
		Timezone:  "America/Caracas",
	},
	"SVRE": {
		ICAO:      "SVRE",
		Name:      "Aracay Abajo Airport",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 420,
		Latitude:  8.3933334351,
		Longitude: -65.8666687012,
		Timezone:  "America/Caracas",
	},
	"SVRG": {
		ICAO:      "SVRG",
		Name:      "Rancho Grande de Apure Airport",
		City:      "La Leona",
		State:     "Apure",
		Country:   "VE",
		Elevation: 918,
		Latitude:  7.3499999046,
		Longitude: -69.3499984741,
		Timezone:  "America/Caracas",
	},
	"SVRH": {
		ICAO:      "SVRH",
		Name:      "Rancho Grande de Tachira Airport",
		State:     "Tachira",
		Country:   "VE",
		Elevation: 315,
		Latitude:  7.6166667938,
		Longitude: -71.9833297729,
		Timezone:  "America/Caracas",
	},
	"SVRI": {
		ICAO:      "SVRI",
		Name:      "Riecito Airport",
		City:      "Cano Riecito",
		State:     "Apure",
		Country:   "VE",
		Elevation: 442,
		Latitude:  6.7840600014,
		Longitude: -68.8650970459,
		Timezone:  "America/Caracas",
	},
	"SVRK": {
		ICAO:      "SVRK",
		Name:      "Los Arrecifes Airport",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 328,
		Latitude:  9.1666669846,
		Longitude: -67.75,
		Timezone:  "America/Caracas",
	},
	"SVRL": {
		ICAO:      "SVRL",
		Name:      "San Pancracio Airport",
		State:     "Barinas",
		Country:   "VE",
		Elevation: 2034,
		Latitude:  8.516667366,
		Longitude: -69.6416702271,
		Timezone:  "America/Caracas",
	},
	"SVRM": {
		ICAO:      "SVRM",
		Name:      "Hato La Romerena Airport",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 665,
		Latitude:  9.5,
		Longitude: -67.6666641235,
		Timezone:  "America/Caracas",
	},
	"SVRN": {
		ICAO:      "SVRN",
		Name:      "El Terron Airport",
		State:     "Monagas",
		Country:   "VE",
		Elevation: 376,
		Latitude:  9.3833332062,
		Longitude: -63.2666664124,
		Timezone:  "America/Caracas",
	},
	"SVRO": {
		ICAO:      "SVRO",
		Name:      "El Rosario Airport",
		State:     "Sucre",
		Country:   "VE",
		Elevation: 693,
		Latitude:  10.4366674423,
		Longitude: -63.9166679382,
		Timezone:  "America/Caracas",
	},
	"SVRQ": {
		ICAO:      "SVRQ",
		Name:      "Rosetta Airport",
		State:     "Amazonas",
		Country:   "VE",
		Elevation: 550,
		Latitude:  5.4333329201,
		Longitude: -66.1341705322,
		Timezone:  "America/Caracas",
	},
	"SVRR": {
		ICAO:      "SVRR",
		Name:      "Carrao Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 1542,
		Latitude:  5.9666671753,
		Longitude: -62.25,
		Timezone:  "America/Caracas",
	},
	"SVRS": {
		ICAO:      "SVRS",
		IATA:      "LRV",
		Name:      "Gran Roque Airport",
		City:      "Los Roques",
		State:     "Dependencias-Federales",
		Country:   "VE",
		Elevation: 17,
		Latitude:  11.9499998093,
		Longitude: -66.6699981689,
		Timezone:  "America/Caracas",
	},
	"SVRT": {
		ICAO:      "SVRT",
		Name:      "Corocito Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 354,
		Latitude:  7.0500001907,
		Longitude: -68.5666656494,
		Timezone:  "America/Caracas",
	},
	"SVRU": {
		ICAO:      "SVRU",
		Name:      "Roblecito Airport",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 459,
		Latitude:  9.200220108,
		Longitude: -66.3111114502,
		Timezone:  "America/Caracas",
	},
	"SVRW": {
		ICAO:      "SVRW",
		Name:      "Rosario Airport",
		State:     "Zulia",
		Country:   "VE",
		Elevation: 118,
		Latitude:  9.1999998093,
		Longitude: -72.3833312988,
		Timezone:  "America/Caracas",
	},
	"SVRX": {
		ICAO:      "SVRX",
		Name:      "Hacienda Rio Yaza Airport",
		State:     "Zulia",
		Country:   "VE",
		Elevation: 492,
		Latitude:  9.8434696198,
		Longitude: -72.547203064,
		Timezone:  "America/Caracas",
	},
	"SVRY": {
		ICAO:      "SVRY",
		Name:      "El Merey Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 242,
		Latitude:  8.1666669846,
		Longitude: -62.8333320618,
		Timezone:  "America/Caracas",
	},
	"SVRZ": {
		ICAO:      "SVRZ",
		Name:      "El Corozo Airport",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 590,
		Latitude:  8.4666671753,
		Longitude: -66.0333328247,
		Timezone:  "America/Caracas",
	},
	"SVSA": {
		ICAO:      "SVSA",
		IATA:      "SVZ",
		Name:      "San Antonio Del Tachira Airport",
		State:     "Tachira",
		Country:   "VE",
		Elevation: 1312,
		Latitude:  7.8408298492,
		Longitude: -72.4396972656,
		Timezone:  "America/Bogota",
	},
	"SVSB": {
		ICAO:      "SVSB",
		IATA:      "SBB",
		Name:      "Santa Barbara de Barinas Airport",
		City:      "Santa Barbara",
		State:     "Barinas",
		Country:   "VE",
		Elevation: 590,
		Latitude:  7.8035140038,
		Longitude: -71.1657180786,
		Timezone:  "America/Caracas",
	},
	"SVSC": {
		ICAO:      "SVSC",
		Name:      "San Carlos Rio Negro Airport",
		State:     "Amazonas",
		Country:   "VE",
		Elevation: 367,
		Latitude:  1.9166669846,
		Longitude: -67.0500030518,
		Timezone:  "America/Bogota",
	},
	"SVSD": {
		ICAO:      "SVSD",
		Name:      "Sabana Dulce Airport",
		State:     "Portuguesa",
		Country:   "VE",
		Elevation: 390,
		Latitude:  8.8166666031,
		Longitude: -69.6333312988,
		Timezone:  "America/Caracas",
	},
	"SVSE": {
		ICAO:      "SVSE",
		IATA:      "SNV",
		Name:      "Santa Elena de Uairen Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 2938,
		Latitude:  4.5549998283,
		Longitude: -61.1500015259,
		Timezone:  "America/Caracas",
	},
	"SVSF": {
		ICAO:      "SVSF",
		Name:      "San Antonio De Falcon Airport",
		State:     "Falcon",
		Country:   "VE",
		Elevation: 26,
		Latitude:  11.0500001907,
		Longitude: -68.3850021362,
		Timezone:  "America/Caracas",
	},
	"SVSG": {
		ICAO:      "SVSG",
		Name:      "San Antonio De Barinas Airport",
		City:      "Barinas",
		State:     "Barinas",
		Country:   "VE",
		Elevation: 460,
		Latitude:  8.5666666031,
		Longitude: -70.0500030518,
		Timezone:  "America/Caracas",
	},
	"SVSH": {
		ICAO:      "SVSH",
		Name:      "San Juan De Rio Claro Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 144,
		Latitude:  7.1841669083,
		Longitude: -67.1680526733,
		Timezone:  "America/Caracas",
	},
	"SVSI": {
		ICAO:      "SVSI",
		Name:      "Agropecuaria San Francisco Airport",
		State:     "Cojedes",
		Country:   "VE",
		Elevation: 249,
		Latitude:  8.9499998093,
		Longitude: -68.0833358765,
		Timezone:  "America/Caracas",
	},
	"SVSJ": {
		ICAO:      "SVSJ",
		Name:      "Central Bolivar Airport",
		State:     "Zulia",
		Country:   "VE",
		Elevation: 39,
		Latitude:  8.9166669846,
		Longitude: -71.9333343506,
		Timezone:  "America/Caracas",
	},
	"SVSK": {
		ICAO:      "SVSK",
		Name:      "Nuestra Senora del Socorro Airport",
		State:     "Cojedes",
		Country:   "VE",
		Elevation: 485,
		Latitude:  9.7783327103,
		Longitude: -68.1750030518,
		Timezone:  "America/Caracas",
	},
	"SVSL": {
		ICAO:      "SVSL",
		Name:      "Hacienda Santa Elena de Mirand Airport",
		State:     "Miranda",
		Country:   "VE",
		Elevation: 233,
		Latitude:  10.3166666031,
		Longitude: -66.3000030518,
		Timezone:  "America/Caracas",
	},
	"SVSM": {
		ICAO:      "SVSM",
		Name:      "Fundo Santa Maria Airport",
		State:     "Barinas",
		Country:   "VE",
		Elevation: 400,
		Latitude:  8.3216667175,
		Longitude: -70.2466659546,
		Timezone:  "America/Caracas",
	},
	"SVSN": {
		ICAO:      "SVSN",
		Name:      "Los Siete Samanes Airport",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 394,
		Latitude:  8.8756399155,
		Longitude: -66.8398971558,
		Timezone:  "America/Caracas",
	},
	"SVSO": {
		ICAO:      "SVSO",
		IATA:      "STD",
		Name:      "Mayor Buenaventura Vivas International Airport",
		City:      "Santo Domingo",
		State:     "Tachira",
		Country:   "VE",
		Elevation: 1083,
		Latitude:  7.5653800964,
		Longitude: -72.0351028442,
		Timezone:  "America/Caracas",
	},
	"SVSP": {
		ICAO:      "SVSP",
		IATA:      "SNF",
		Name:      "Sub Teniente Nestor Arias Airport",
		City:      "San Felipe",
		State:     "Yaracuy",
		Country:   "VE",
		Elevation: 761,
		Latitude:  10.2786998749,
		Longitude: -68.7552032471,
		Timezone:  "America/Caracas",
	},
	"SVSQ": {
		ICAO:      "SVSQ",
		Name:      "San Leonardo Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 222,
		Latitude:  7.0333328247,
		Longitude: -68.3333358765,
		Timezone:  "America/Caracas",
	},
	"SVSR": {
		ICAO:      "SVSR",
		IATA:      "SFD",
		Name:      "San Fernando De Apure Airport",
		City:      "Inglaterra",
		State:     "Apure",
		Country:   "VE",
		Elevation: 154,
		Latitude:  7.8833198547,
		Longitude: -67.4440002441,
		Timezone:  "America/Caracas",
	},
	"SVSS": {
		ICAO:      "SVSS",
		Name:      "Hacienda San Jose Airport",
		City:      "San Jose de Trujillo",
		State:     "Trujillo",
		Country:   "VE",
		Elevation: 98,
		Latitude:  9.4666671753,
		Longitude: -71.0333328247,
		Timezone:  "America/Caracas",
	},
	"SVST": {
		ICAO:      "SVST",
		IATA:      "SOM",
		Name:      "San Tome Airport",
		State:     "Anzoategui",
		Country:   "VE",
		Elevation: 861,
		Latitude:  8.9451465607,
		Longitude: -64.1510848999,
		Timezone:  "America/Caracas",
	},
	"SVSU": {
		ICAO:      "SVSU",
		Name:      "San Benito Airport",
		State:     "Zulia",
		Country:   "VE",
		Elevation: 75,
		Latitude:  10.3022222519,
		Longitude: -72.0522232056,
		Timezone:  "America/Caracas",
	},
	"SVSW": {
		ICAO:      "SVSW",
		Name:      "San Francisco De Carabobo Airport",
		State:     "Carabobo",
		Country:   "VE",
		Elevation: 1484,
		Latitude:  10.1022224426,
		Longitude: -68.0833358765,
		Timezone:  "America/Caracas",
	},
	"SVSX": {
		ICAO:      "SVSX",
		Name:      "San Juan de los Cayos Airport",
		State:     "Falcon",
		Country:   "VE",
		Elevation: 165,
		Latitude:  11.1750001907,
		Longitude: -68.4216690063,
		Timezone:  "America/Caracas",
	},
	"SVSY": {
		ICAO:      "SVSY",
		Name:      "San Francisco de Merida Airport",
		State:     "Zulia",
		Country:   "VE",
		Elevation: 99,
		Latitude:  8.8000001907,
		Longitude: -71.5833358765,
		Timezone:  "America/Caracas",
	},
	"SVSZ": {
		ICAO:      "SVSZ",
		IATA:      "STB",
		Name:      "Santa Barbara del Zulia Airport",
		State:     "Zulia",
		Country:   "VE",
		Elevation: 32,
		Latitude:  8.9745502472,
		Longitude: -71.9432525635,
		Timezone:  "America/Caracas",
	},
	"SVTB": {
		ICAO:      "SVTB",
		Name:      "Turapa Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 390,
		Latitude:  7.4000000954,
		Longitude: -64.3666687012,
		Timezone:  "America/Caracas",
	},
	"SVTC": {
		ICAO:      "SVTC",
		IATA:      "TUV",
		Name:      "Tucupita Airport",
		City:      "Tucupita",
		State:     "Delta-Amacuro",
		Country:   "VE",
		Elevation: 16,
		Latitude:  9.0889940262,
		Longitude: -62.0941734314,
		Timezone:  "America/Caracas",
	},
	"SVTD": {
		ICAO:      "SVTD",
		Name:      "La Estacada Airport",
		City:      "La Estacada",
		State:     "Apure",
		Country:   "VE",
		Elevation: 360,
		Latitude:  7.4000000954,
		Longitude: -69.0666656494,
		Timezone:  "America/Caracas",
	},
	"SVTE": {
		ICAO:      "SVTE",
		Name:      "Antabare Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 1204,
		Latitude:  6.4833331108,
		Longitude: -62.8833312988,
		Timezone:  "America/Caracas",
	},
	"SVTF": {
		ICAO:      "SVTF",
		Name:      "La Trinidad de Ferro Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 393,
		Latitude:  7.4000000954,
		Longitude: -67.9499969482,
		Timezone:  "America/Caracas",
	},
	"SVTG": {
		ICAO:      "SVTG",
		Name:      "La Tigra Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 360,
		Latitude:  7.6333332062,
		Longitude: -68.6999969482,
		Timezone:  "America/Caracas",
	},
	"SVTJ": {
		ICAO:      "SVTJ",
		Name:      "Central Matilde Airport",
		City:      "central Matilde",
		State:     "Yaracuy",
		Country:   "VE",
		Elevation: 866,
		Latitude:  10.1578998566,
		Longitude: -68.8616027832,
		Timezone:  "America/Caracas",
	},
	"SVTK": {
		ICAO:      "SVTK",
		Name:      "La Trinidad de Orichuna Airport",
		City:      "L Trinidad de Orichuna",
		State:     "Apure",
		Country:   "VE",
		Elevation: 328,
		Latitude:  7.1383328438,
		Longitude: -69.7750015259,
		Timezone:  "America/Caracas",
	},
	"SVTL": {
		ICAO:      "SVTL",
		Name:      "La Centella Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 839,
		Latitude:  6.9416670799,
		Longitude: -65.8916702271,
		Timezone:  "America/Caracas",
	},
	"SVTM": {
		ICAO:      "SVTM",
		IATA:      "TMO",
		Name:      "Tumeremo Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 345,
		Latitude:  7.25,
		Longitude: -61.4333343506,
		Timezone:  "America/Caracas",
	},
	"SVTN": {
		ICAO:      "SVTN",
		Name:      "La Trinidad de Barinas Airport",
		State:     "Barinas",
		Country:   "VE",
		Elevation: 150,
		Latitude:  8.1333332062,
		Longitude: -68.6666641235,
		Timezone:  "America/Caracas",
	},
	"SVTO": {
		ICAO:      "SVTO",
		Name:      "El Toco Airport",
		City:      "El Toco",
		State:     "Anzoategui",
		Country:   "VE",
		Elevation: 392,
		Latitude:  9.1999998093,
		Longitude: -64.8499984741,
		Timezone:  "America/Caracas",
	},
	"SVTP": {
		ICAO:      "SVTP",
		Name:      "Playa Pintada Airport",
		Country:   "VE",
		Elevation: 9,
		Latitude:  10.1333332062,
		Longitude: -65.4416656494,
		Timezone:  "America/Caracas",
	},
	"SVTR": {
		ICAO:      "SVTR",
		Name:      "Temblador Airport",
		State:     "Monagas",
		Country:   "VE",
		Elevation: 104,
		Latitude:  9.016667366,
		Longitude: -62.7333335876,
		Timezone:  "America/Caracas",
	},
	"SVTT": {
		ICAO:      "SVTT",
		Name:      "La Torta Airport",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 672,
		Latitude:  9.5112600327,
		Longitude: -66.5539016724,
		Timezone:  "America/Caracas",
	},
	"SVTU": {
		ICAO:      "SVTU",
		Name:      "Turagua Airport",
		City:      "Turagua",
		State:     "Apure",
		Country:   "VE",
		Elevation: 338,
		Latitude:  7.7833328247,
		Longitude: -69.2333297729,
		Timezone:  "America/Caracas",
	},
	"SVTV": {
		ICAO:      "SVTV",
		Name:      "Turen Airport",
		State:     "Portuguesa",
		Country:   "VE",
		Elevation: 521,
		Latitude:  9.25,
		Longitude: -69.1333312988,
		Timezone:  "America/Caracas",
	},
	"SVTY": {
		ICAO:      "SVTY",
		Name:      "Toromacho Airport",
		City:      "Las Mercedes del Llano",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 450,
		Latitude:  9.130825803,
		Longitude: -66.3911533356,
		Timezone:  "America/Caracas",
	},
	"SVUA": {
		ICAO:      "SVUA",
		Name:      "Mayupa Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 1672,
		Latitude:  6.1500000954,
		Longitude: -62.7000007629,
		Timezone:  "America/Caracas",
	},
	"SVUK": {
		ICAO:      "SVUK",
		Name:      "San Ignacio Airport",
		State:     "Cojedes",
		Country:   "VE",
		Elevation: 426,
		Latitude:  9.4499998093,
		Longitude: -68.3666687012,
		Timezone:  "America/Caracas",
	},
	"SVUM": {
		ICAO:      "SVUM",
		IATA:      "URM",
		Name:      "Uriman Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 1148,
		Latitude:  5.3333330154,
		Longitude: -62.7666664124,
		Timezone:  "America/Caracas",
	},
	"SVUP": {
		ICAO:      "SVUP",
		Name:      "Upata Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 1056,
		Latitude:  7.9799399376,
		Longitude: -62.3227996826,
		Timezone:  "America/Caracas",
	},
	"SVUQ": {
		ICAO:      "SVUQ",
		Name:      "Uon Quen Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 2820,
		Latitude:  4.9833331108,
		Longitude: -61.7283325195,
		Timezone:  "America/Caracas",
	},
	"SVUY": {
		ICAO:      "SVUY",
		Name:      "Uruyen Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 1476,
		Latitude:  5.6800498962,
		Longitude: -62.4571990967,
		Timezone:  "America/Caracas",
	},
	"SVVA": {
		ICAO:      "SVVA",
		IATA:      "VLN",
		Name:      "Arturo Michelena International Airport",
		City:      "Valencia",
		State:     "Carabobo",
		Country:   "VE",
		Elevation: 1411,
		Latitude:  10.1497325897,
		Longitude: -67.9283981323,
		Timezone:  "America/Caracas",
	},
	"SVVB": {
		ICAO:      "SVVB",
		Name:      "Bajo Verde Airport",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 492,
		Latitude:  9.0513887405,
		Longitude: -66.4674987793,
		Timezone:  "America/Caracas",
	},
	"SVVD": {
		ICAO:      "SVVD",
		Name:      "Valle Grande Airport",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 631,
		Latitude:  8.266667366,
		Longitude: -67.1666641235,
		Timezone:  "America/Caracas",
	},
	"SVVG": {
		ICAO:      "SVVG",
		IATA:      "VIG",
		Name:      "Juan Pablo Perez Alfonso Airport",
		City:      "El Vigia",
		State:     "Merida",
		Country:   "VE",
		Elevation: 250,
		Latitude:  8.6241388321,
		Longitude: -71.672668457,
		Timezone:  "America/Caracas",
	},
	"SVVH": {
		ICAO:      "SVVH",
		Name:      "Valle Hondo Airport",
		State:     "Monagas",
		Country:   "VE",
		Elevation: 301,
		Latitude:  9.3666667938,
		Longitude: -62.9166679382,
		Timezone:  "America/Caracas",
	},
	"SVVK": {
		ICAO:      "SVVK",
		Name:      "La Verdad Airport",
		City:      "Las Queseritas",
		State:     "Apure",
		Country:   "VE",
		Elevation: 236,
		Latitude:  7.1333332062,
		Longitude: -69.0500030518,
		Timezone:  "America/Caracas",
	},
	"SVVL": {
		ICAO:      "SVVL",
		IATA:      "VLV",
		Name:      "Dr. Antonio Nicolas Briceno Airport",
		City:      "Valera",
		State:     "Trujillo",
		Country:   "VE",
		Elevation: 2060,
		Latitude:  9.3404779434,
		Longitude: -70.5840606689,
		Timezone:  "America/Caracas",
	},
	"SVVM": {
		ICAO:      "SVVM",
		Name:      "Buena Vista del Cano Medio Airport",
		State:     "Cojedes",
		Country:   "VE",
		Elevation: 475,
		Latitude:  6.9166669846,
		Longitude: -68.1500015259,
		Timezone:  "America/Caracas",
	},
	"SVVP": {
		ICAO:      "SVVP",
		IATA:      "VDP",
		Name:      "Valle de La Pascua Airport",
		State:     "Guarico",
		Country:   "VE",
		Elevation: 410,
		Latitude:  9.2220277786,
		Longitude: -65.9935836792,
		Timezone:  "America/Caracas",
	},
	"SVVQ": {
		ICAO:      "SVVQ",
		Name:      "Venelac  Airport",
		City:      "Quebrada Arriba",
		State:     "Lara",
		Country:   "VE",
		Elevation: 1969,
		Latitude:  10.2427997589,
		Longitude: -70.5018005371,
		Timezone:  "America/Caracas",
	},
	"SVVR": {
		ICAO:      "SVVR",
		Name:      "Hato La Vergarena Airport",
		State:     "Monagas",
		Country:   "VE",
		Elevation: 1082,
		Latitude:  6.8333330154,
		Longitude: -63.5833320618,
		Timezone:  "America/Caracas",
	},
	"SVVS": {
		ICAO:      "SVVS",
		Name:      "Villa del Rosario Airport",
		State:     "Zulia",
		Country:   "VE",
		Elevation: 259,
		Latitude:  10.3166666031,
		Longitude: -72.3000030518,
		Timezone:  "America/Caracas",
	},
	"SVVV": {
		ICAO:      "SVVV",
		Name:      "Venepal Airport",
		State:     "Carabobo",
		Country:   "VE",
		Elevation: 59,
		Latitude:  10.5402002335,
		Longitude: -68.2453994751,
		Timezone:  "America/Caracas",
	},
	"SVVX": {
		ICAO:      "SVVX",
		Name:      "Los Venados Airport",
		State:     "Bolivar",
		Country:   "VE",
		Elevation: 621,
		Latitude:  6.5616669655,
		Longitude: -66.001663208,
		Timezone:  "America/Caracas",
	},
	"SVWA": {
		ICAO:      "SVWA",
		Name:      "Mata de Agua Airport",
		City:      "Santa Barbara",
		State:     "Apure",
		Country:   "VE",
		Elevation: 226,
		Latitude:  7.3166670799,
		Longitude: -67.9083328247,
		Timezone:  "America/Caracas",
	},
	"SVWB": {
		ICAO:      "SVWB",
		Name:      "La Bananera Airport",
		Country:   "VE",
		Elevation: 49,
		Latitude:  10.4794998169,
		Longitude: -68.4728012085,
		Timezone:  "America/Caracas",
	},
	"SVWE": {
		ICAO:      "SVWE",
		Name:      "Bella Vista Airport",
		State:     "Barinas",
		Country:   "VE",
		Elevation: 266,
		Latitude:  8.0116672516,
		Longitude: -69.8316650391,
		Timezone:  "America/Caracas",
	},
	"SVXF": {
		ICAO:      "SVXF",
		Name:      "Hato Santa Clara Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 393,
		Latitude:  7.1500000954,
		Longitude: -69.25,
		Timezone:  "America/Caracas",
	},
	"SVYA": {
		ICAO:      "SVYA",
		Name:      "Yaure Airport",
		State:     "Barinas",
		Country:   "VE",
		Elevation: 482,
		Latitude:  7.7166671753,
		Longitude: -71.2333297729,
		Timezone:  "America/Caracas",
	},
	"SVYG": {
		ICAO:      "SVYG",
		Name:      "El Yagual Airport",
		Country:   "VE",
		Elevation: 262,
		Latitude:  7.4666671753,
		Longitude: -68.4499969482,
		Timezone:  "America/Caracas",
	},
	"SVYI": {
		ICAO:      "SVYI",
		Name:      "El Yavi Airport",
		State:     "Amazonas",
		Country:   "VE",
		Elevation: 557,
		Latitude:  5.4583330154,
		Longitude: -65.8983306885,
		Timezone:  "America/Caracas",
	},
	"SVYP": {
		ICAO:      "SVYP",
		Name:      "Yopita Airport",
		State:     "Apure",
		Country:   "VE",
		Elevation: 360,
		Latitude:  7.1999998093,
		Longitude: -69.3166656494,
		Timezone:  "America/Caracas",
	},
	"SVYT": {
		ICAO:      "SVYT",
		Name:      "Yutaje Airport",
		State:     "Amazonas",
		Country:   "VE",
		Elevation: 600,
		Latitude:  5.6097798347,
		Longitude: -66.1103973389,
		Timezone:  "America/Caracas",
	},
	"SWAA": {
		ICAO:      "SWAA",
		Name:      "Fazenda Ykarai Airport",
		City:      "Nova Crixas",
		State:     "Goias",
		Country:   "BR",
		Elevation: 896,
		Latitude:  -14.5438890457,
		Longitude: -50.5927772522,
		Timezone:  "America/Sao_Paulo",
	},
	"SWAE": {
		ICAO:      "SWAE",
		Name:      "Uaicas Airport",
		City:      "Alto Alegre",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 1083,
		Latitude:  3.5450000763,
		Longitude: -63.1822013855,
		Timezone:  "America/Boa_Vista",
	},
	"SWAF": {
		ICAO:      "SWAF",
		Name:      "Agroer Aviacao Agricola Airport",
		City:      "Rondonopolis",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 827,
		Latitude:  -16.4838886261,
		Longitude: -54.6672210693,
		Timezone:  "America/Cuiaba",
	},
	"SWAH": {
		ICAO:      "SWAH",
		Name:      "Agropecuaria Floresta Sao Joaquim Ltda Airport",
		City:      "Aripuana",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 325,
		Latitude:  -10.141667366,
		Longitude: -60.6527786255,
		Timezone:  "America/Cuiaba",
	},
	"SWAK": {
		ICAO:      "SWAK",
		Name:      "Assuncao do Icana Airport",
		City:      "Sao Gabriel Da Cachoeira",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 492,
		Latitude:  1.0663889647,
		Longitude: -67.6005554199,
		Timezone:  "America/Manaus",
	},
	"SWAN": {
		ICAO:      "SWAN",
		Name:      "Arenapolis Airport",
		City:      "Arenapolis",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1329,
		Latitude:  -14.5002784729,
		Longitude: -57.0172233582,
		Timezone:  "America/Cuiaba",
	},
	"SWAO": {
		ICAO:      "SWAO",
		Name:      "Anamoim Airport",
		City:      "Sao Gabriel Da Cachoeira",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 335,
		Latitude:  1.7997200489,
		Longitude: -67.3839035034,
		Timezone:  "America/Manaus",
	},
	"SWAP": {
		ICAO:      "SWAP",
		Name:      "Fazenda Apemag Airport",
		City:      "Barra Do Bugres",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 951,
		Latitude:  -15.3336105347,
		Longitude: -57.3172225952,
		Timezone:  "America/Cuiaba",
	},
	"SWAQ": {
		ICAO:      "SWAQ",
		Name:      "Erico Airport",
		City:      "Boa Vista",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 689,
		Latitude:  3.7633299828,
		Longitude: -62.4094009399,
		Timezone:  "America/Boa_Vista",
	},
	"SWAR": {
		ICAO:      "SWAR",
		Name:      "Fazenda Aruana Airport",
		City:      "Ribeirao Cascalheira",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 968,
		Latitude:  -13.082777977,
		Longitude: -51.5863876343,
		Timezone:  "America/Cuiaba",
	},
	"SWAT": {
		ICAO:      "SWAT",
		Name:      "Fazenda Cachoeira Airport",
		City:      "Alto Araguaia",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2795,
		Latitude:  -17.0891666412,
		Longitude: -53.3316688538,
		Timezone:  "America/Cuiaba",
	},
	"SWAU": {
		ICAO:      "SWAU",
		Name:      "Fazenda Nossa Senhora Aparecida Airport",
		City:      "Pirajuba",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2592,
		Latitude:  -19.95,
		Longitude: -48.58,
		Timezone:  "America/Sao_Paulo",
	},
	"SWAV": {
		ICAO:      "SWAV",
		Name:      "Fazenda Agua Verde Airport",
		City:      "Diamantino",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2037,
		Latitude:  -14.0727777481,
		Longitude: -57.4547233582,
		Timezone:  "America/Cuiaba",
	},
	"SWAY": {
		ICAO:      "SWAY",
		IATA:      "0",
		Name:      "Araguaiana Airport",
		City:      "Araguaiana",
		State:     "Goias",
		Country:   "BR",
		Elevation: 994,
		Latitude:  -15.7238998413,
		Longitude: -51.8363990784,
		Timezone:  "America/Cuiaba",
	},
	"SWAZ": {
		ICAO:      "SWAZ",
		Name:      "Santo Atanazio Airport",
		City:      "Sao Gabriel Da Cachoeira",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 394,
		Latitude:  0.3997220099,
		Longitude: -69.3005523682,
		Timezone:  "America/Manaus",
	},
	"SWBA": {
		ICAO:      "SWBA",
		IATA:      "0",
		Name:      "Buriti Alegre Airport",
		City:      "Buriti Alegre",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2848,
		Latitude:  -18.1256008148,
		Longitude: -49.0428009033,
		Timezone:  "America/Sao_Paulo",
	},
	"SWBB": {
		ICAO:      "SWBB",
		Name:      "Barra do Bugres Airport",
		City:      "Barra Do Bugres",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 722,
		Latitude:  -15.0586109161,
		Longitude: -57.1830558777,
		Timezone:  "America/Cuiaba",
	},
	"SWBC": {
		ICAO:      "SWBC",
		IATA:      "BAZ",
		Name:      "Barcelos Airport",
		City:      "Barcelos",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 112,
		Latitude:  -0.9812920094,
		Longitude: -62.9196014404,
		Timezone:  "America/Manaus",
	},
	"SWBF": {
		ICAO:      "SWBF",
		Name:      "Fazenda Boa Fortuna Airport",
		City:      "Pium",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 741,
		Latitude:  -10.0050001144,
		Longitude: -49.4919433594,
		Timezone:  "America/Araguaina",
	},
	"SWBG": {
		ICAO:      "SWBG",
		Name:      "Uniao do Vale Airport",
		City:      "Pontes E Lacerda",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 810,
		Latitude:  -15.1999998093,
		Longitude: -59.38722229,
		Timezone:  "America/Cuiaba",
	},
	"SWBH": {
		ICAO:      "SWBH",
		Name:      "Brejinho de Nazare Airport",
		City:      "Brejinho De Nazare",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 869,
		Latitude:  -11.0169439316,
		Longitude: -48.6005554199,
		Timezone:  "America/Araguaina",
	},
	"SWBI": {
		ICAO:      "SWBI",
		Name:      "Barreirinha Airport",
		City:      "Barreirinha",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 69,
		Latitude:  -2.7924199104,
		Longitude: -57.0578994751,
		Timezone:  "America/Manaus",
	},
	"SWBJ": {
		ICAO:      "SWBJ",
		Name:      "Fazenda Itarema Airport",
		City:      "Canarana",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1339,
		Latitude:  -13.2550001144,
		Longitude: -52.0400009155,
		Timezone:  "America/Cuiaba",
	},
	"SWBL": {
		ICAO:      "SWBL",
		Name:      "Balbina Airport",
		City:      "Presidente Figueiredo",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 564,
		Latitude:  -1.9248100519,
		Longitude: -59.4123001099,
		Timezone:  "America/Manaus",
	},
	"SWBN": {
		ICAO:      "SWBN",
		Name:      "Fazenda Barreirinhos Airport",
		City:      "Barao De Melgaco",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 377,
		Latitude:  -16.9463882446,
		Longitude: -56.3927764893,
		Timezone:  "America/Cuiaba",
	},
	"SWBO": {
		ICAO:      "SWBO",
		Name:      "Fazenda Sao Joao Airport",
		City:      "Campos Verdes",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1542,
		Latitude:  -14.2747220993,
		Longitude: -49.6491661072,
		Timezone:  "America/Sao_Paulo",
	},
	"SWBQ": {
		ICAO:      "SWBQ",
		Name:      "Barracao Queimado Airport",
		City:      "Vila Bela Da Santissima Trindade",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2067,
		Latitude:  -13.7336111069,
		Longitude: -59.7172203064,
		Timezone:  "America/Cuiaba",
	},
	"SWBR": {
		ICAO:      "SWBR",
		IATA:      "RBB",
		Name:      "Borba Airport",
		City:      "Borba",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 293,
		Latitude:  -4.4063401222,
		Longitude: -59.6024017334,
		Timezone:  "America/Manaus",
	},
	"SWBT": {
		ICAO:      "SWBT",
		Name:      "Fazenda Beira Rio Airport",
		City:      "Nova Canaa Do Norte",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 820,
		Latitude:  -10.3711109161,
		Longitude: -55.818611145,
		Timezone:  "America/Cuiaba",
	},
	"SWBU": {
		ICAO:      "SWBU",
		Name:      "Fazenda Fartura Baixao Novo Airport",
		City:      "Peixoto De Azevedo",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1214,
		Latitude:  -10.2155561447,
		Longitude: -55.0830574036,
		Timezone:  "America/Cuiaba",
	},
	"SWBV": {
		ICAO:      "SWBV",
		Name:      "Auaris Airport",
		City:      "Boa Vista",
		State:     "Bolivar",
		Country:   "BR",
		Elevation: 2336,
		Latitude:  4.016389,
		Longitude: -64.518611,
		Timezone:  "America/Boa_Vista",
	},
	"SWBW": {
		ICAO:      "SWBW",
		Name:      "Agro Pousadas do Guapore Airport",
		City:      "Vila Bela Da Santissima Trindade",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 886,
		Latitude:  -14.7683334351,
		Longitude: -60.0344429016,
		Timezone:  "America/Cuiaba",
	},
	"SWBX": {
		ICAO:      "SWBX",
		Name:      "Fazenda Santo Antonio do Paraiso Airport",
		City:      "Itiquira",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 650,
		Latitude:  -17.4911117554,
		Longitude: -55.2283325195,
		Timezone:  "America/Cuiaba",
	},
	"SWBY": {
		ICAO:      "SWBY",
		Name:      "Fazenda Goiasa Airport",
		City:      "Bom Jesus De Goias",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1820,
		Latitude:  -18.0505561829,
		Longitude: -49.7005577087,
		Timezone:  "America/Sao_Paulo",
	},
	"SWBZ": {
		ICAO:      "SWBZ",
		Name:      "Jpo Airport",
		City:      "Sorriso",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1155,
		Latitude:  -12.6052780151,
		Longitude: -55.7647209167,
		Timezone:  "America/Cuiaba",
	},
	"SWCA": {
		ICAO:      "SWCA",
		IATA:      "CAF",
		Name:      "Carauari Airport",
		City:      "Carauari",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 355,
		Latitude:  -4.8715200424,
		Longitude: -66.8974990845,
		Timezone:  "America/Manaus",
	},
	"SWCB": {
		ICAO:      "SWCB",
		Name:      "Campos Belos Airport",
		City:      "Campos Belos",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 2198,
		Latitude:  -13.0088891983,
		Longitude: -46.774723053,
		Timezone:  "America/Sao_Paulo",
	},
	"SWCC": {
		ICAO:      "SWCC",
		Name:      "Fazenda Curitiba Airport",
		City:      "Comodoro",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 705,
		Latitude:  -13.6222,
		Longitude: -60.60139,
		Timezone:  "America/Cuiaba",
	},
	"SWCD": {
		ICAO:      "SWCD",
		Name:      "Fazenda Santa Monica Airport",
		City:      "Alexania",
		State:     "Goias",
		Country:   "BR",
		Elevation: 3888,
		Latitude:  -15.9352998734,
		Longitude: -48.6465988159,
		Timezone:  "America/Sao_Paulo",
	},
	"SWCE": {
		ICAO:      "SWCE",
		Name:      "Fazenda Cedro Airport",
		City:      "Santo Antonio Do Leverger",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2310,
		Latitude:  -16.2042999268,
		Longitude: -55.3987007141,
		Timezone:  "America/Cuiaba",
	},
	"SWCF": {
		ICAO:      "SWCF",
		Name:      "Fazenda Confap Airport",
		City:      "Comodoro",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 673,
		Latitude:  -13.7302780151,
		Longitude: -60.4786109924,
		Timezone:  "America/Cuiaba",
	},
	"SWCG": {
		ICAO:      "SWCG",
		Name:      "Fazenda Varjao Grande Airport",
		City:      "Cacu",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2060,
		Latitude:  -18.4915008545,
		Longitude: -51.2988014221,
		Timezone:  "America/Sao_Paulo",
	},
	"SWCH": {
		ICAO:      "SWCH",
		Name:      "Fazenda Charqueada Norte Airport",
		City:      "Alta Floresta",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 820,
		Latitude:  -10.0002784729,
		Longitude: -57.0672225952,
		Timezone:  "America/Cuiaba",
	},
	"SWCJ": {
		ICAO:      "SWCJ",
		Name:      "Coperjava Airport",
		City:      "Formoso Do Araguaia",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 699,
		Latitude:  -11.8169441223,
		Longitude: -49.7505569458,
		Timezone:  "America/Araguaina",
	},
	"SWCK": {
		ICAO:      "SWCK",
		Name:      "Fazenda Cocal Airport",
		City:      "Canarana",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 984,
		Latitude:  -12.6836109161,
		Longitude: -52.7172203064,
		Timezone:  "America/Cuiaba",
	},
	"SWCL": {
		ICAO:      "SWCL",
		Name:      "Chacara Santa Eulalia Airport",
		City:      "Alta Floresta",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 669,
		Latitude:  -9.546667099,
		Longitude: -57.4158325195,
		Timezone:  "America/Cuiaba",
	},
	"SWCM": {
		ICAO:      "SWCM",
		Name:      "Fazenda Couto Magalhaes Airport",
		City:      "Agua Boa",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1378,
		Latitude:  -13.990278244,
		Longitude: -52.9269447327,
		Timezone:  "America/Cuiaba",
	},
	"SWCN": {
		ICAO:      "SWCN",
		Name:      "Fazenda Saudade Airport",
		City:      "Cocalinho",
		State:     "Goias",
		Country:   "BR",
		Elevation: 784,
		Latitude:  -14.1433334351,
		Longitude: -51.0149993896,
		Timezone:  "America/Cuiaba",
	},
	"SWCO": {
		ICAO:      "SWCO",
		Name:      "Fazenda Continental Airport",
		City:      "Cocalinho",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 919,
		Latitude:  -14.7155561447,
		Longitude: -51.2097206116,
		Timezone:  "America/Cuiaba",
	},
	"SWCP": {
		ICAO:      "SWCP",
		Name:      "Campo Novo do Parecis Airport",
		City:      "Campo Novo Do Parecis",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1818,
		Latitude:  -13.628610611,
		Longitude: -57.9263877869,
		Timezone:  "America/Cuiaba",
	},
	"SWCQ": {
		ICAO:      "SWCQ",
		IATA:      "CQS",
		Name:      "Costa Marques Airport",
		City:      "Costa Marques",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 555,
		Latitude:  -12.4210996628,
		Longitude: -64.2516021729,
		Timezone:  "America/Porto_Velho",
	},
	"SWCR": {
		ICAO:      "SWCR",
		Name:      "Porto Caracara Airport",
		City:      "Pocone",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 328,
		Latitude:  -17.8338890076,
		Longitude: -57.1672210693,
		Timezone:  "America/Campo_Grande",
	},
	"SWCT": {
		ICAO:      "SWCT",
		Name:      "Complexo Turistico S. R. Pantanal Hoteis LTDA Airport",
		City:      "Pocone",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 334,
		Latitude:  -17.3680553436,
		Longitude: -56.7847213745,
		Timezone:  "America/Campo_Grande",
	},
	"SWCU": {
		ICAO:      "SWCU",
		Name:      "Fazenda Canuana Airport",
		City:      "Formoso Do Araguaia",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 787,
		Latitude:  -11.9774999619,
		Longitude: -49.9080543518,
		Timezone:  "America/Araguaina",
	},
	"SWCV": {
		ICAO:      "SWCV",
		Name:      "Casalvasco Airport",
		City:      "Vila Bela Da Santissima Trindade",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 676,
		Latitude:  -15.489443779,
		Longitude: -60.0855560303,
		Timezone:  "America/Cuiaba",
	},
	"SWCW": {
		ICAO:      "SWCW",
		IATA:      "0",
		Name:      "Cavalcante Airport",
		City:      "Cavalcante",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2625,
		Latitude:  -13.7660999298,
		Longitude: -47.4286003113,
		Timezone:  "America/Sao_Paulo",
	},
	"SWCX": {
		ICAO:      "SWCX",
		Name:      "Cabixi Airport",
		City:      "Cabixi",
		Country:   "BR",
		Elevation: 1247,
		Latitude:  -13.7002782822,
		Longitude: -60.7005577087,
		Timezone:  "America/Cuiaba",
	},
	"SWCY": {
		ICAO:      "SWCY",
		Name:      "Fazenda Aparecida do Norte Airport",
		City:      "Nova Maringa",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1004,
		Latitude:  -12.8447217941,
		Longitude: -56.9769439697,
		Timezone:  "America/Cuiaba",
	},
	"SWCZ": {
		ICAO:      "SWCZ",
		IATA:      "0",
		Name:      "Ceres Airport",
		City:      "Ceres",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1962,
		Latitude:  -15.3446998596,
		Longitude: -49.6046981812,
		Timezone:  "America/Sao_Paulo",
	},
	"SWDA": {
		ICAO:      "SWDA",
		Name:      "Fazenda Duas Ancoras - Secao Sede Airport",
		City:      "Barra Do Garcas",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 925,
		Latitude:  -14.9358329773,
		Longitude: -52.1730575562,
		Timezone:  "America/Cuiaba",
	},
	"SWDB": {
		ICAO:      "SWDB",
		Name:      "Fazenda Eldorado Airport",
		City:      "Barra Do Garcas",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1086,
		Latitude:  -15.5769443512,
		Longitude: -52.2188873291,
		Timezone:  "America/Cuiaba",
	},
	"SWDC": {
		ICAO:      "SWDC",
		Name:      "Fazenda Arco-Iris Airport",
		City:      "Alto Garcas",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2516,
		Latitude:  -16.9383335114,
		Longitude: -53.5783348083,
		Timezone:  "America/Cuiaba",
	},
	"SWDE": {
		ICAO:      "SWDE",
		Name:      "Fazenda Piratininga Airport",
		City:      "Sao Miguel Do Araguaia",
		State:     "Goias",
		Country:   "BR",
		Elevation: 722,
		Latitude:  -12.8219442368,
		Longitude: -50.3322219849,
		Timezone:  "America/Sao_Paulo",
	},
	"SWDF": {
		ICAO:      "SWDF",
		Name:      "Fazenda Braco Forte Airport",
		City:      "Novo Mundo",
		State:     "Para",
		Country:   "BR",
		Elevation: 1417,
		Latitude:  -9.5249996185,
		Longitude: -55.0422210693,
		Timezone:  "America/Cuiaba",
	},
	"SWDG": {
		ICAO:      "SWDG",
		Name:      "Fazenda Sao Jose Airport",
		City:      "Reginopolis",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1509,
		Latitude:  -21.744444,
		Longitude: -49.164722,
		Timezone:  "America/Sao_Paulo",
	},
	"SWDI": {
		ICAO:      "SWDI",
		Name:      "Dois Irmaos Airport",
		City:      "Peixe",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 771,
		Latitude:  -11.7336111069,
		Longitude: -49.3338890076,
		Timezone:  "America/Araguaina",
	},
	"SWDJ": {
		ICAO:      "SWDJ",
		Name:      "Fazenda Duas Ancoras - Secao JAO Airport",
		City:      "Barra Do Garcas",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 886,
		Latitude:  -14.8575000763,
		Longitude: -52.0366668701,
		Timezone:  "America/Cuiaba",
	},
	"SWDK": {
		ICAO:      "SWDK",
		Name:      "Fazenda Dona Olga Airport",
		City:      "Colorado D`Oeste",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 680,
		Latitude:  -13.0197219849,
		Longitude: -61.1686096191,
		Timezone:  "America/Porto_Velho",
	},
	"SWDM": {
		ICAO:      "SWDM",
		IATA:      "DMT",
		Name:      "Diamantino Airport",
		City:      "Diamantino",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1476,
		Latitude:  -14.3768997192,
		Longitude: -56.4003982544,
		Timezone:  "America/Cuiaba",
	},
	"SWDN": {
		ICAO:      "SWDN",
		IATA:      "DNO",
		Name:      "Dianopolis Airport",
		City:      "Dianopolis",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 2001,
		Latitude:  -11.5953998566,
		Longitude: -46.846698761,
		Timezone:  "America/Araguaina",
	},
	"SWDO": {
		ICAO:      "SWDO",
		Name:      "Rosario do Leste Airport",
		City:      "Rosario Do Leste",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 699,
		Latitude:  -14.8169441223,
		Longitude: -56.4172210693,
		Timezone:  "America/Cuiaba",
	},
	"SWDQ": {
		ICAO:      "SWDQ",
		Name:      "Fazenda Saltos do Poente Airport",
		City:      "Santo Antonio Do Leverger",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1352,
		Latitude:  -16.4358329773,
		Longitude: -55.2711105347,
		Timezone:  "America/Cuiaba",
	},
	"SWDS": {
		ICAO:      "SWDS",
		Name:      "Fazenda Favo de Mel Airport",
		City:      "Pontes E Lacerda",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 846,
		Latitude:  -15.6608333588,
		Longitude: -59.305557251,
		Timezone:  "America/Cuiaba",
	},
	"SWDT": {
		ICAO:      "SWDT",
		Name:      "Fazenda Pioneira Airport",
		City:      "Diamantino",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2178,
		Latitude:  -14.2836112976,
		Longitude: -57.7005577087,
		Timezone:  "America/Cuiaba",
	},
	"SWDU": {
		ICAO:      "SWDU",
		Name:      "Diauarum Airport",
		City:      "Barra Do Garcas",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 820,
		Latitude:  -11.0336112976,
		Longitude: -53.1172218323,
		Timezone:  "America/Cuiaba",
	},
	"SWDV": {
		ICAO:      "SWDV",
		Name:      "Descalvados Airport",
		City:      "Caceres",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 361,
		Latitude:  -16.7347221375,
		Longitude: -57.7638893127,
		Timezone:  "America/Cuiaba",
	},
	"SWDW": {
		ICAO:      "SWDW",
		Name:      "Divisao Airport",
		City:      "Diamantino",
		Country:   "BR",
		Elevation: 1345,
		Latitude:  -13.116944313,
		Longitude: -56.1672210693,
		Timezone:  "America/Cuiaba",
	},
	"SWDY": {
		ICAO:      "SWDY",
		Name:      "Fazenda Agua Fria Airport",
		City:      "Nova Crixas",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1175,
		Latitude:  -14.7547216415,
		Longitude: -50.3774986267,
		Timezone:  "America/Sao_Paulo",
	},
	"SWDZ": {
		ICAO:      "SWDZ",
		Name:      "Fazenda Agua Limpa Airport",
		City:      "Indiavai",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1181,
		Latitude:  -15.2894439697,
		Longitude: -58.5302772522,
		Timezone:  "America/Cuiaba",
	},
	"SWEB": {
		ICAO:      "SWEB",
		Name:      "Fazenda Arauna Airport",
		City:      "Juara",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 951,
		Latitude:  -11.5675001144,
		Longitude: -58.0261116028,
		Timezone:  "America/Cuiaba",
	},
	"SWEC": {
		ICAO:      "SWEC",
		IATA:      "ARS",
		Name:      "Estancia das Cascatas Airport",
		City:      "Aragarcas",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1017,
		Latitude:  -15.8966999054,
		Longitude: -52.0956001282,
		Timezone:  "America/Cuiaba",
	},
	"SWED": {
		ICAO:      "SWED",
		Name:      "Fazenda Eldorado Airport",
		City:      "Arapoema",
		Country:   "BR",
		Elevation: 686,
		Latitude:  -7.7572221756,
		Longitude: -49.1216659546,
		Timezone:  "America/Araguaina",
	},
	"SWEE": {
		ICAO:      "SWEE",
		Name:      "Estirao do Equador Airport",
		City:      "Atalaia Do Norte",
		State:     "Loreto",
		Country:   "BR",
		Elevation: 98,
		Latitude:  -4.5237798691,
		Longitude: -71.560798645,
		Timezone:  "America/Eirunepe",
	},
	"SWEF": {
		ICAO:      "SWEF",
		Name:      "Fazenda Europa Airport",
		City:      "Cocalinho",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 899,
		Latitude:  -14.80166667,
		Longitude: -51.34,
		Timezone:  "America/Cuiaba",
	},
	"SWEG": {
		ICAO:      "SWEG",
		Name:      "Fazenda Duas Marias Airport",
		City:      "Barao De Melgaco",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 446,
		Latitude:  -16.8333320618,
		Longitude: -55.6691665649,
		Timezone:  "America/Cuiaba",
	},
	"SWEI": {
		ICAO:      "SWEI",
		IATA:      "ERN",
		Name:      "Eirunepe Airport",
		City:      "Eirunepe",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 412,
		Latitude:  -6.6395301819,
		Longitude: -69.8797988892,
		Timezone:  "America/Eirunepe",
	},
	"SWEK": {
		ICAO:      "SWEK",
		IATA:      "CQA",
		Name:      "Canarana Airport",
		City:      "Canarana",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1314,
		Latitude:  -13.5744438171,
		Longitude: -52.2705574036,
		Timezone:  "America/Cuiaba",
	},
	"SWEL": {
		ICAO:      "SWEL",
		Name:      "Fazenda Santa Eulalia Airport",
		City:      "Araguaiana",
		Country:   "BR",
		Elevation: 853,
		Latitude:  -15.0583333969,
		Longitude: -51.9166679382,
		Timezone:  "America/Cuiaba",
	},
	"SWEM": {
		ICAO:      "SWEM",
		Name:      "Emal - Empresa de Mineiracao Aripuana Ltda. Airport",
		City:      "Primavera Do Leste",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1837,
		Latitude:  -14.618888855,
		Longitude: -54.0033340454,
		Timezone:  "America/Cuiaba",
	},
	"SWEN": {
		ICAO:      "SWEN",
		Name:      "Estancia Recanto Airport",
		City:      "Alta Floresta",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1017,
		Latitude:  -9.9419441223,
		Longitude: -56.1005554199,
		Timezone:  "America/Cuiaba",
	},
	"SWEO": {
		ICAO:      "SWEO",
		Name:      "Itapara Sport Fishing Airport",
		City:      "Rorainopolis",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 262,
		Latitude:  0.2097219974,
		Longitude: -61.5499992371,
		Timezone:  "America/Boa_Vista",
	},
	"SWER": {
		ICAO:      "SWER",
		Name:      "Estancia Santana Airport",
		City:      "Cuiaba",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 801,
		Latitude:  -15.4697217941,
		Longitude: -56.063331604,
		Timezone:  "America/Cuiaba",
	},
	"SWET": {
		ICAO:      "SWET",
		Name:      "Fazenda Encantado Airport",
		City:      "Sapezal",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 558,
		Latitude:  -13.3166666031,
		Longitude: -58.7983322144,
		Timezone:  "America/Cuiaba",
	},
	"SWEU": {
		ICAO:      "SWEU",
		Name:      "Ecotur Univini Park Airport",
		City:      "Caracarai",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 266,
		Latitude:  0.801388979,
		Longitude: -61.6605987549,
		Timezone:  "America/Boa_Vista",
	},
	"SWEW": {
		ICAO:      "SWEW",
		Name:      "Fazenda Felicidade Airport",
		City:      "Porto Esperidiao",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 656,
		Latitude:  -15.8422222137,
		Longitude: -58.6316680908,
		Timezone:  "America/Cuiaba",
	},
	"SWEX": {
		ICAO:      "SWEX",
		Name:      "Fazenda Santa Escolastica Airport",
		City:      "Pedra Preta",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1145,
		Latitude:  -16.8466663361,
		Longitude: -54.4588890076,
		Timezone:  "America/Cuiaba",
	},
	"SWEY": {
		ICAO:      "SWEY",
		Name:      "Fazenda Santa Fe Airport",
		City:      "Santa Tereza De Goias",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1197,
		Latitude:  -13.5349998474,
		Longitude: -48.9016685486,
		Timezone:  "America/Sao_Paulo",
	},
	"SWEZ": {
		ICAO:      "SWEZ",
		Name:      "Fazenda Santa Rita de Cassia Airport",
		City:      "Jangada",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 666,
		Latitude:  -15.2502784729,
		Longitude: -56.4811096191,
		Timezone:  "America/Cuiaba",
	},
	"SWFA": {
		ICAO:      "SWFA",
		Name:      "Fazenda Pamplona Airport",
		City:      "Cristalina",
		State:     "Goias",
		Country:   "BR",
		Elevation: 3281,
		Latitude:  -16.2505569458,
		Longitude: -47.6172218323,
		Timezone:  "America/Sao_Paulo",
	},
	"SWFC": {
		ICAO:      "SWFC",
		Name:      "Fazenda Colorado Airport",
		City:      "Juscimeira",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1191,
		Latitude:  -16.1672210693,
		Longitude: -54.8427772522,
		Timezone:  "America/Cuiaba",
	},
	"SWFD": {
		ICAO:      "SWFD",
		Name:      "Fazenda Tombador Airport",
		City:      "Brasnorte",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 863,
		Latitude:  -11.7166671753,
		Longitude: -58.0480575562,
		Timezone:  "America/Cuiaba",
	},
	"SWFE": {
		ICAO:      "SWFE",
		Name:      "Fazenda Colen Airport",
		City:      "Lucas Do Rio Verde",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1345,
		Latitude:  -13.3144435883,
		Longitude: -56.11277771,
		Timezone:  "America/Cuiaba",
	},
	"SWFF": {
		ICAO:      "SWFF",
		Name:      "Fazenda Figueira Branca Airport",
		City:      "Sinop",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1001,
		Latitude:  -10.723610878,
		Longitude: -54.8383331299,
		Timezone:  "America/Cuiaba",
	},
	"SWFG": {
		ICAO:      "SWFG",
		Name:      "Fazenda Guanabara Airport",
		City:      "Barra Do Garcas",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1116,
		Latitude:  -12.029999733,
		Longitude: -51.6775016785,
		Timezone:  "America/Cuiaba",
	},
	"SWFH": {
		ICAO:      "SWFH",
		Name:      "Fazenda Manchete Airport",
		City:      "Araguacema",
		Country:   "BR",
		Elevation: 640,
		Latitude:  -9.7469444275,
		Longitude: -49.9808311462,
		Timezone:  "America/Araguaina",
	},
	"SWFJ": {
		ICAO:      "SWFJ",
		IATA:      "FEJ",
		Name:      "Feijo Airport",
		City:      "Feijo",
		Country:   "BR",
		Elevation: 394,
		Latitude:  -8.1655797958,
		Longitude: -70.3529968262,
		Timezone:  "America/Rio_Branco",
	},
	"SWFK": {
		ICAO:      "SWFK",
		Name:      "Fazenda Juquei Agropecuaria Airport",
		City:      "Padre Bernardo",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2297,
		Latitude:  -15.07472229,
		Longitude: -48.238609314,
		Timezone:  "America/Sao_Paulo",
	},
	"SWFL": {
		ICAO:      "SWFL",
		Name:      "Fazenda Peixe Bravo Airport",
		City:      "Santa Terezinha",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 656,
		Latitude:  -10.116944313,
		Longitude: -50.4172210693,
		Timezone:  "America/Cuiaba",
	},
	"SWFN": {
		ICAO:      "SWFN",
		Name:      "Flores Airport",
		City:      "Manaus",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 203,
		Latitude:  -3.0727779865,
		Longitude: -60.0211105347,
		Timezone:  "America/Manaus",
	},
	"SWFO": {
		ICAO:      "SWFO",
		Name:      "Fazenda Bocaina Airport",
		City:      "Jangada",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1037,
		Latitude:  -15.3463888168,
		Longitude: -56.7111091614,
		Timezone:  "America/Cuiaba",
	},
	"SWFP": {
		ICAO:      "SWFP",
		Name:      "Fazenda Paraiso Airport",
		City:      "Porto Alegre Do Norte",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 984,
		Latitude:  -10.6388893127,
		Longitude: -52.0325012207,
		Timezone:  "America/Cuiaba",
	},
	"SWFQ": {
		ICAO:      "SWFQ",
		Name:      "Fazenda Alvorada Airport",
		City:      "Porto Nacional",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 943,
		Latitude:  -10.3920001984,
		Longitude: -48.6791992188,
		Timezone:  "America/Araguaina",
	},
	"SWFR": {
		ICAO:      "SWFR",
		Name:      "Formosa Airport",
		City:      "Formosa",
		State:     "Goias",
		Country:   "BR",
		Elevation: 3166,
		Latitude:  -15.555100441,
		Longitude: -47.3456993103,
		Timezone:  "America/Sao_Paulo",
	},
	"SWFS": {
		ICAO:      "SWFS",
		Name:      "Fazenda Sucuri Airport",
		City:      "Sinop",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1575,
		Latitude:  -13.5313892365,
		Longitude: -54.9338874817,
		Timezone:  "America/Cuiaba",
	},
	"SWFT": {
		ICAO:      "SWFT",
		Name:      "Fazenda Santa Marcia Airport",
		City:      "Caiaponia",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2822,
		Latitude:  -16.5808334351,
		Longitude: -52.1425018311,
		Timezone:  "America/Sao_Paulo",
	},
	"SWFU": {
		ICAO:      "SWFU",
		Name:      "Fortuna Airport",
		City:      "Fortuna",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 866,
		Latitude:  -16.1669445038,
		Longitude: -59.5005569458,
		Timezone:  "America/Cuiaba",
	},
	"SWFW": {
		ICAO:      "SWFW",
		Name:      "Fazenda Jangada Airport",
		City:      "Barra Do Garcas",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 919,
		Latitude:  -14.7172222137,
		Longitude: -51.9088897705,
		Timezone:  "America/Cuiaba",
	},
	"SWFX": {
		ICAO:      "SWFX",
		IATA:      "SXO",
		Name:      "Sao Felix do Araguaia Airport",
		City:      "Sao Felix Do Araguaia",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 650,
		Latitude:  -11.632399559,
		Longitude: -50.6896018982,
		Timezone:  "America/Cuiaba",
	},
	"SWFY": {
		ICAO:      "SWFY",
		Name:      "Fazenda Ipe Airport",
		City:      "Vila Rica",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 718,
		Latitude:  -9.9669437408,
		Longitude: -50.9505577087,
		Timezone:  "America/Cuiaba",
	},
	"SWFZ": {
		ICAO:      "SWFZ",
		Name:      "Serafino Ferruzzi Airport",
		City:      "Alta Floresta",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 820,
		Latitude:  -9.9755563736,
		Longitude: -56.6730575562,
		Timezone:  "America/Cuiaba",
	},
	"SWGA": {
		ICAO:      "SWGA",
		Name:      "Fazenda Guara Airport",
		City:      "Vila Bela Da Santissima Trindade",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1378,
		Latitude:  -14.8225002289,
		Longitude: -60.0172233582,
		Timezone:  "America/Cuiaba",
	},
	"SWGE": {
		ICAO:      "SWGE",
		Name:      "Fazenda Pirigara Airport",
		City:      "Barao De Melgaco",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 344,
		Latitude:  -17.0072212219,
		Longitude: -56.4780578613,
		Timezone:  "America/Cuiaba",
	},
	"SWGF": {
		ICAO:      "SWGF",
		Name:      "Hidreletrica de Jirau Airport",
		City:      "Porto Velho",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 295,
		Latitude:  -9.288611,
		Longitude: -64.623611,
		Timezone:  "America/Porto_Velho",
	},
	"SWGH": {
		ICAO:      "SWGH",
		Name:      "Fazenda Primavera Airport",
		City:      "Caceres",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1299,
		Latitude:  -16.2380561829,
		Longitude: -57.6199989319,
		Timezone:  "America/Cuiaba",
	},
	"SWGI": {
		ICAO:      "SWGI",
		IATA:      "GRP",
		Name:      "Gurupi Airport",
		City:      "Gurupi",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 1148,
		Latitude:  -11.7396001816,
		Longitude: -49.1321983337,
		Timezone:  "America/Araguaina",
	},
	"SWGK": {
		ICAO:      "SWGK",
		Name:      "Fazenda Sete Barras Airport",
		City:      "Ribeirao Cascalheira",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 801,
		Latitude:  -12.5597219467,
		Longitude: -51.6886100769,
		Timezone:  "America/Cuiaba",
	},
	"SWGL": {
		ICAO:      "SWGL",
		Name:      "Goalcool Airport",
		City:      "Serranopolis",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2067,
		Latitude:  -18.3913898468,
		Longitude: -52.1044425964,
		Timezone:  "America/Sao_Paulo",
	},
	"SWGM": {
		ICAO:      "SWGM",
		Name:      "Ninho das Aguias Airport",
		City:      "Cuiaba",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2464,
		Latitude:  -15.4899997711,
		Longitude: -55.8147201538,
		Timezone:  "America/Cuiaba",
	},
	"SWGN": {
		ICAO:      "SWGN",
		IATA:      "AUX",
		Name:      "Araguaina Airport",
		City:      "Araguaina",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 771,
		Latitude:  -7.2278699875,
		Longitude: -48.2405014038,
		Timezone:  "America/Araguaina",
	},
	"SWGO": {
		ICAO:      "SWGO",
		Name:      "Goianesia Airport",
		City:      "Goianesia",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2303,
		Latitude:  -15.3400001526,
		Longitude: -49.1361122131,
		Timezone:  "America/Sao_Paulo",
	},
	"SWGP": {
		ICAO:      "SWGP",
		Name:      "Palmeiras de Goias Airport",
		City:      "Palmeiras De Goias",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1995,
		Latitude:  -16.8269004822,
		Longitude: -49.8788986206,
		Timezone:  "America/Sao_Paulo",
	},
	"SWGQ": {
		ICAO:      "SWGQ",
		Name:      "Fazenda Monte Alegre Airport",
		City:      "Nova Olimpia",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 755,
		Latitude:  -14.7808332443,
		Longitude: -57.3541679382,
		Timezone:  "America/Cuiaba",
	},
	"SWGS": {
		ICAO:      "SWGS",
		Name:      "Fazenda Divisa Airport",
		City:      "Santana Do Araguaia",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 755,
		Latitude:  -9.7916669846,
		Longitude: -50.8199996948,
		Timezone:  "America/Belem",
	},
	"SWGT": {
		ICAO:      "SWGT",
		Name:      "Fazenda Nova Airport",
		City:      "Jandaia",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1844,
		Latitude:  -17.188056,
		Longitude: -50.213333,
		Timezone:  "America/Sao_Paulo",
	},
	"SWGU": {
		ICAO:      "SWGU",
		Name:      "Fazenda Agua Fria Airport",
		City:      "Sandolandia",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 738,
		Latitude:  -12.398888588,
		Longitude: -50.0547218323,
		Timezone:  "America/Araguaina",
	},
	"SWGV": {
		ICAO:      "SWGV",
		Name:      "Fazenda Cocal Airport",
		City:      "Agua Boa",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1004,
		Latitude:  -12.801943779,
		Longitude: -52.6077766418,
		Timezone:  "America/Cuiaba",
	},
	"SWGW": {
		ICAO:      "SWGW",
		Name:      "Fazenda Santa Rita do Araguaia Airport",
		City:      "Jussara",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1047,
		Latitude:  -15.4377784729,
		Longitude: -51.5986099243,
		Timezone:  "America/Sao_Paulo",
	},
	"SWGZ": {
		ICAO:      "SWGZ",
		Name:      "Fazenda Guanabara Airport",
		City:      "Nova Olimpia",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 988,
		Latitude:  -14.7669448853,
		Longitude: -57.2672233582,
		Timezone:  "America/Cuiaba",
	},
	"SWHA": {
		ICAO:      "SWHA",
		Name:      "Fazenda Boa Esperanca Soberbinho Airport",
		City:      "Guarinos",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1553,
		Latitude:  -14.6447219849,
		Longitude: -49.8163871765,
		Timezone:  "America/Sao_Paulo",
	},
	"SWHE": {
		ICAO:      "SWHE",
		Name:      "Agropecuaria Cajabi Airport",
		City:      "Marvelandia",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1352,
		Latitude:  -10.7344436646,
		Longitude: -54.5344429016,
		Timezone:  "America/Cuiaba",
	},
	"SWHF": {
		ICAO:      "SWHF",
		Name:      "Fazenda Frigovale I Airport",
		City:      "Porto Nacional",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 860,
		Latitude:  -10.485833168,
		Longitude: -48.3338890076,
		Timezone:  "America/Araguaina",
	},
	"SWHG": {
		ICAO:      "SWHG",
		Name:      "Santa Helena de Goias Airport",
		City:      "Santa Helena De Goias",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1591,
		Latitude:  -17.8838882446,
		Longitude: -50.5672225952,
		Timezone:  "America/Sao_Paulo",
	},
	"SWHJ": {
		ICAO:      "SWHJ",
		Name:      "Fazenda Bela Vista Airport",
		City:      "Nova Monte Verde",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 899,
		Latitude:  -10.0555562973,
		Longitude: -56.9911117554,
		Timezone:  "America/Cuiaba",
	},
	"SWHL": {
		ICAO:      "SWHL",
		Name:      "Fazenda Maraba Airport",
		City:      "Campo Verde",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2247,
		Latitude:  -15.2749996185,
		Longitude: -55.1216659546,
		Timezone:  "America/Cuiaba",
	},
	"SWHN": {
		ICAO:      "SWHN",
		Name:      "Fazenda Nossa Senhora de Fatima Airport",
		City:      "Nova Monte Verde",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 853,
		Latitude:  -10.1563892365,
		Longitude: -56.9811096191,
		Timezone:  "America/Cuiaba",
	},
	"SWHO": {
		ICAO:      "SWHO",
		Name:      "Gleba Oscar Americano I Airport",
		City:      "Porto Dos Gauchos",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1017,
		Latitude:  -11.5336112976,
		Longitude: -56.5505561829,
		Timezone:  "America/Cuiaba",
	},
	"SWHP": {
		ICAO:      "SWHP",
		IATA:      "GGB",
		Name:      "Fazenda Olhos D`agua Airport",
		City:      "Agua Boa",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1407,
		Latitude:  -14.0194444656,
		Longitude: -52.1522216797,
		Timezone:  "America/Cuiaba",
	},
	"SWHQ": {
		ICAO:      "SWHQ",
		Name:      "Fazenda Santa Cruz Airport",
		City:      "Vila Bela Da Santissima Trindade",
		Country:   "BR",
		Elevation: 1060,
		Latitude:  -15.1891670227,
		Longitude: -59.9391670227,
		Timezone:  "America/Cuiaba",
	},
	"SWHS": {
		ICAO:      "SWHS",
		Name:      "Fazenda Sao Joao Airport",
		City:      "Maurilandia",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1729,
		Latitude:  -18.0533332825,
		Longitude: -50.3869438171,
		Timezone:  "America/Sao_Paulo",
	},
	"SWHT": {
		ICAO:      "SWHT",
		IATA:      "HUW",
		Name:      "Humaita Airport",
		City:      "Humaita",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 230,
		Latitude:  -7.5321202278,
		Longitude: -63.072101593,
		Timezone:  "America/Manaus",
	},
	"SWHU": {
		ICAO:      "SWHU",
		Name:      "Fazenda Izaura - Usina Cocal Airport",
		City:      "Racharia",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1542,
		Latitude:  -22.506667,
		Longitude: -50.865278,
		Timezone:  "America/Sao_Paulo",
	},
	"SWHY": {
		ICAO:      "SWHY",
		Name:      "Fazenda Esperanca I Airport",
		City:      "Rondon Do Para",
		State:     "Para",
		Country:   "BR",
		Elevation: 285,
		Latitude:  -4.13,
		Longitude: -48.69,
		Timezone:  "America/Belem",
	},
	"SWHZ": {
		ICAO:      "SWHZ",
		Name:      "Fazenda Taquarucu Retiro Airport",
		City:      "Carlinda",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1010,
		Latitude:  -10.3386106491,
		Longitude: -55.9655570984,
		Timezone:  "America/Cuiaba",
	},
	"SWIA": {
		ICAO:      "SWIA",
		IATA:      "0",
		Name:      "Iaciara Airport",
		City:      "Iaciara",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1854,
		Latitude:  -14.1063995361,
		Longitude: -46.6693992615,
		Timezone:  "America/Sao_Paulo",
	},
	"SWIB": {
		ICAO:      "SWIB",
		Name:      "Fazenda Santa Laura Airport",
		City:      "Nova Santa Helena",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1096,
		Latitude:  -10.86277771,
		Longitude: -54.9141654968,
		Timezone:  "America/Cuiaba",
	},
	"SWID": {
		ICAO:      "SWID",
		Name:      "Fazenda Indiara Airport",
		City:      "Sao Miguel Do Araguaia",
		State:     "Goias",
		Country:   "BR",
		Elevation: 909,
		Latitude:  -13.3149003983,
		Longitude: -49.7748985291,
		Timezone:  "America/Sao_Paulo",
	},
	"SWIE": {
		ICAO:      "SWIE",
		Name:      "Fazenda Sao Francisco Airport",
		City:      "Mundo Novo",
		State:     "Goias",
		Country:   "BR",
		Elevation: 984,
		Latitude:  -13.9194440842,
		Longitude: -50.0099983215,
		Timezone:  "America/Sao_Paulo",
	},
	"SWIG": {
		ICAO:      "SWIG",
		Name:      "Fazenda Sao Jose Airport",
		City:      "Pontes E Lacerda",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 879,
		Latitude:  -15.0108327866,
		Longitude: -59.2522201538,
		Timezone:  "America/Cuiaba",
	},
	"SWII": {
		ICAO:      "SWII",
		IATA:      "IPG",
		Name:      "Ipiranga Airport",
		City:      "Santo Antonio Do Ica",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 131,
		Latitude:  -2.9390699863,
		Longitude: -69.6940002441,
		Timezone:  "America/Bogota",
	},
	"SWIJ": {
		ICAO:      "SWIJ",
		Name:      "Fazenda Sao Jorge Airport",
		City:      "Itiquira",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1683,
		Latitude:  -17.5180568695,
		Longitude: -54.8263893127,
		Timezone:  "America/Cuiaba",
	},
	"SWIK": {
		ICAO:      "SWIK",
		Name:      "Fazenda Vanguarda Airport",
		City:      "Cocalinho",
		State:     "Goias",
		Country:   "BR",
		Elevation: 935,
		Latitude:  -14.917222023,
		Longitude: -51.2838897705,
		Timezone:  "America/Cuiaba",
	},
	"SWIL": {
		ICAO:      "SWIL",
		Name:      "Fazenda Taruma Airport",
		City:      "Pedra Preta",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1499,
		Latitude:  -17.015832901,
		Longitude: -54.3497238159,
		Timezone:  "America/Cuiaba",
	},
	"SWIM": {
		ICAO:      "SWIM",
		Name:      "Fazenda Tres Ranchos Airport",
		City:      "Montes Claros De Goias",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1142,
		Latitude:  -16.1266670227,
		Longitude: -51.6697235107,
		Timezone:  "America/Sao_Paulo",
	},
	"SWIN": {
		ICAO:      "SWIN",
		Name:      "Fazenda Itamarati Norte Airport",
		City:      "Campo Novo Do Parecis",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2188,
		Latitude:  -14.2430000305,
		Longitude: -57.995300293,
		Timezone:  "America/Cuiaba",
	},
	"SWIO": {
		ICAO:      "SWIO",
		Name:      "Fazenda Triangulo Airport",
		City:      "Edeia",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1562,
		Latitude:  -17.621389389,
		Longitude: -50.1416664124,
		Timezone:  "America/Sao_Paulo",
	},
	"SWIP": {
		ICAO:      "SWIP",
		Name:      "Ipameri Airport",
		City:      "Ipameri",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2729,
		Latitude:  -17.6872215271,
		Longitude: -48.161945343,
		Timezone:  "America/Sao_Paulo",
	},
	"SWIU": {
		ICAO:      "SWIU",
		Name:      "Madeirinha Airport",
		City:      "Aripuana",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 476,
		Latitude:  -9.8463888168,
		Longitude: -61.4174995422,
		Timezone:  "America/Cuiaba",
	},
	"SWIV": {
		ICAO:      "SWIV",
		Name:      "Aguia Branca do Paru Airport",
		City:      "Almeirim",
		State:     "Para",
		Country:   "BR",
		Elevation: 479,
		Latitude:  -1.268056035,
		Longitude: -53.261390686,
		Timezone:  "America/Santarem",
	},
	"SWIW": {
		ICAO:      "SWIW",
		Name:      "Brigadeiro Eduardo Gomes Airport",
		City:      "Nova Mutum",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1398,
		Latitude:  -13.817199707,
		Longitude: -56.0360984802,
		Timezone:  "America/Cuiaba",
	},
	"SWIX": {
		ICAO:      "SWIX",
		Name:      "Fazenda Michelin Airport",
		City:      "Itiquira",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1640,
		Latitude:  -17.3741664886,
		Longitude: -54.7350006104,
		Timezone:  "America/Cuiaba",
	},
	"SWIY": {
		ICAO:      "SWIY",
		IATA:      "IDO",
		Name:      "Santa Izabel do Morro Airport",
		City:      "Cristalandia",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 647,
		Latitude:  -11.5722999573,
		Longitude: -50.6661987305,
		Timezone:  "America/Araguaina",
	},
	"SWJA": {
		ICAO:      "SWJA",
		Name:      "Fazenda Juina Airport",
		City:      "Juina",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1100,
		Latitude:  -11.6427783966,
		Longitude: -59.0875015259,
		Timezone:  "America/Cuiaba",
	},
	"SWJB": {
		ICAO:      "SWJB",
		Name:      "Fazenda Juba Airport",
		City:      "Tangara Da Serra",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1667,
		Latitude:  -14.7449998856,
		Longitude: -58.1283340454,
		Timezone:  "America/Cuiaba",
	},
	"SWJC": {
		ICAO:      "SWJC",
		Name:      "Jaciara Airport",
		City:      "Jaciara",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1198,
		Latitude:  -15.9716672897,
		Longitude: -54.9686126709,
		Timezone:  "America/Cuiaba",
	},
	"SWJD": {
		ICAO:      "SWJD",
		Name:      "Fazenda Genipapo Airport",
		City:      "Sao Domingos",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1279,
		Latitude:  -13.4522218704,
		Longitude: -46.6794433594,
		Timezone:  "America/Sao_Paulo",
	},
	"SWJE": {
		ICAO:      "SWJE",
		Name:      "Fazenda Ibia Airport",
		City:      "Baliza",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2634,
		Latitude:  -16.396944046,
		Longitude: -52.3286094666,
		Timezone:  "America/Sao_Paulo",
	},
	"SWJF": {
		ICAO:      "SWJF",
		Name:      "Fazenda Sao Marcelo Airport",
		City:      "Juruena",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 840,
		Latitude:  -10.233332634,
		Longitude: -58.4944458008,
		Timezone:  "America/Cuiaba",
	},
	"SWJG": {
		ICAO:      "SWJG",
		Name:      "Fazenda Jaragua Airport",
		City:      "Arenapolis",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1083,
		Latitude:  -14.4836111069,
		Longitude: -57.4338874817,
		Timezone:  "America/Cuiaba",
	},
	"SWJI": {
		ICAO:      "SWJI",
		IATA:      "JPR",
		Name:      "Ji-Parana Airport",
		City:      "Ji-Parana",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 598,
		Latitude:  -10.8708000183,
		Longitude: -61.8465003967,
		Timezone:  "America/Porto_Velho",
	},
	"SWJJ": {
		ICAO:      "SWJJ",
		Name:      "Fazenda Januarinho Airport",
		City:      "Salto Do Ceu",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 755,
		Latitude:  -15.05027771,
		Longitude: -57.5005569458,
		Timezone:  "America/Cuiaba",
	},
	"SWJK": {
		ICAO:      "SWJK",
		Name:      "Fazenda Sao Paulo Airport",
		City:      "Britania",
		State:     "Goias",
		Country:   "BR",
		Elevation: 984,
		Latitude:  -14.9763889313,
		Longitude: -51.3355560303,
		Timezone:  "America/Cuiaba",
	},
	"SWJL": {
		ICAO:      "SWJL",
		Name:      "Fazenda Invernada Airport",
		City:      "Itaja",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1345,
		Latitude:  -19.0411109924,
		Longitude: -51.2594451904,
		Timezone:  "America/Sao_Paulo",
	},
	"SWJM": {
		ICAO:      "SWJM",
		Name:      "Fazenda Bom Jesus Airport",
		City:      "Novo Alegre",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2067,
		Latitude:  -12.9663887024,
		Longitude: -46.5652770996,
		Timezone:  "America/Araguaina",
	},
	"SWJN": {
		ICAO:      "SWJN",
		IATA:      "JIA",
		Name:      "Juina Airport",
		City:      "Juina",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1083,
		Latitude:  -11.4194440842,
		Longitude: -58.7016677856,
		Timezone:  "America/Cuiaba",
	},
	"SWJO": {
		ICAO:      "SWJO",
		Name:      "Fazenda Sao Paulo - Retiro Santa Maria Airport",
		City:      "Britania",
		State:     "Goias",
		Country:   "BR",
		Elevation: 984,
		Latitude:  -15.1400003433,
		Longitude: -51.2544441223,
		Timezone:  "America/Sao_Paulo",
	},
	"SWJP": {
		ICAO:      "SWJP",
		Name:      "Bittencourt Airport",
		City:      "Japura",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 257,
		Latitude:  -1.4039299488,
		Longitude: -69.4237976074,
		Timezone:  "America/Bogota",
	},
	"SWJQ": {
		ICAO:      "SWJQ",
		Name:      "Fazenda Colibri Airport",
		City:      "Selviria",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1391,
		Latitude:  -20.2269439697,
		Longitude: -51.8458328247,
		Timezone:  "America/Campo_Grande",
	},
	"SWJR": {
		ICAO:      "SWJR",
		Name:      "Fazenda Uniao Airport",
		City:      "Santa Carmen",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1180,
		Latitude:  -11.9488887787,
		Longitude: -54.5155563354,
		Timezone:  "America/Cuiaba",
	},
	"SWJS": {
		ICAO:      "SWJS",
		Name:      "Fazenda Samambaia Airport",
		City:      "Jussara",
		State:     "Goias",
		Country:   "BR",
		Elevation: 692,
		Latitude:  -15.4861106873,
		Longitude: -51.269443512,
		Timezone:  "America/Sao_Paulo",
	},
	"SWJT": {
		ICAO:      "SWJT",
		Name:      "Fazenda Joatao Airport",
		City:      "Sao Felix Do Araguaia",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1165,
		Latitude:  -11.4939870834,
		Longitude: -52.6180725098,
		Timezone:  "America/Cuiaba",
	},
	"SWJU": {
		ICAO:      "SWJU",
		IATA:      "JRN",
		Name:      "Juruena Airport",
		City:      "Juruena",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 525,
		Latitude:  -10.3058328629,
		Longitude: -58.4894447327,
		Timezone:  "America/Cuiaba",
	},
	"SWJV": {
		ICAO:      "SWJV",
		Name:      "Palmeiras do Javari Airport",
		City:      "Atalaia Do Norte",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 394,
		Latitude:  -5.127240181,
		Longitude: -72.8099975586,
		Timezone:  "America/Eirunepe",
	},
	"SWJW": {
		ICAO:      "SWJW",
		IATA:      "JTI",
		Name:      "Jatai Airport",
		City:      "Jatai",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2529,
		Latitude:  -17.8299007416,
		Longitude: -51.7729988098,
		Timezone:  "America/Sao_Paulo",
	},
	"SWJX": {
		ICAO:      "SWJX",
		Name:      "Fazenda das Perobas Airport",
		City:      "Prudente De Morais",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2762,
		Latitude:  -19.5022220612,
		Longitude: -44.1577796936,
		Timezone:  "America/Sao_Paulo",
	},
	"SWJY": {
		ICAO:      "SWJY",
		Name:      "Fazenda Campo Alegre III Airport",
		City:      "Alto Garcas",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2625,
		Latitude:  -16.8447227478,
		Longitude: -53.7344436646,
		Timezone:  "America/Cuiaba",
	},
	"SWJZ": {
		ICAO:      "SWJZ",
		Name:      "Fazenda Morro Vermelho Airport",
		City:      "Juscimeira",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1654,
		Latitude:  -15.9813890457,
		Longitude: -54.5338897705,
		Timezone:  "America/Cuiaba",
	},
	"SWKA": {
		ICAO:      "SWKA",
		Name:      "Fazenda Canada Airport",
		City:      "Jussara",
		State:     "Goias",
		Country:   "BR",
		Elevation: 951,
		Latitude:  -15.5952777863,
		Longitude: -51.2572212219,
		Timezone:  "America/Sao_Paulo",
	},
	"SWKB": {
		ICAO:      "SWKB",
		Name:      "Fazenda Campina Airport",
		City:      "Boca Do Acre",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 699,
		Latitude:  -9.1424999237,
		Longitude: -67.4458312988,
		Timezone:  "America/Manaus",
	},
	"SWKC": {
		ICAO:      "SWKC",
		IATA:      "CCX",
		Name:      "Caceres Airport",
		City:      "Caceres",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 492,
		Latitude:  -16.0436000824,
		Longitude: -57.629901886,
		Timezone:  "America/Cuiaba",
	},
	"SWKE": {
		ICAO:      "SWKE",
		Name:      "Cobrape Airport",
		City:      "Formoso Do Araguaia",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 656,
		Latitude:  -11.50861,
		Longitude: -49.84222,
		Timezone:  "America/Araguaina",
	},
	"SWKG": {
		ICAO:      "SWKG",
		Name:      "Fazenda Agua Limpa Airport",
		City:      "Nova Ubirata",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1623,
		Latitude:  -13.4072217941,
		Longitude: -54.7519454956,
		Timezone:  "America/Cuiaba",
	},
	"SWKH": {
		ICAO:      "SWKH",
		Name:      "Fazenda Floresta Airport",
		City:      "Campo Verde",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2050,
		Latitude:  -15.2211112976,
		Longitude: -55.1327781677,
		Timezone:  "America/Cuiaba",
	},
	"SWKJ": {
		ICAO:      "SWKJ",
		Name:      "Fazenda Mutum Airport",
		City:      "Dom Aquino",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1821,
		Latitude:  -15.3980560303,
		Longitude: -54.6083335876,
		Timezone:  "America/Cuiaba",
	},
	"SWKL": {
		ICAO:      "SWKL",
		Name:      "Fazenda Primavera do Terebinto Airport",
		City:      "Alto Alegre Dos Parecis",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 590,
		Latitude:  -12.5577783585,
		Longitude: -61.9961128235,
		Timezone:  "America/Porto_Velho",
	},
	"SWKM": {
		ICAO:      "SWKM",
		Name:      "Aero Agricola Rondon Airport",
		City:      "Tangara Da Serra",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1390,
		Latitude:  -14.6441669464,
		Longitude: -57.5041656494,
		Timezone:  "America/Cuiaba",
	},
	"SWKO": {
		ICAO:      "SWKO",
		IATA:      "CIZ",
		Name:      "Coari Airport",
		City:      "Coari",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 131,
		Latitude:  -4.134059906,
		Longitude: -63.132598877,
		Timezone:  "America/Manaus",
	},
	"SWKP": {
		ICAO:      "SWKP",
		Name:      "Chacara Veneza Airport",
		City:      "Pontes E Lacerda",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 866,
		Latitude:  -15.2425003052,
		Longitude: -59.2852783203,
		Timezone:  "America/Cuiaba",
	},
	"SWKQ": {
		ICAO:      "SWKQ",
		Name:      "Serra da Capivara Airport",
		City:      "Sao Raimundo Nonato",
		State:     "Piaui",
		Country:   "BR",
		Elevation: 1362,
		Latitude:  -9.082778,
		Longitude: -42.644444,
		Timezone:  "America/Fortaleza",
	},
	"SWKR": {
		ICAO:      "SWKR",
		Name:      "Fazenda Mangue Che II Airport",
		City:      "Itaruma",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1752,
		Latitude:  -18.6700000763,
		Longitude: -51.5852775574,
		Timezone:  "America/Sao_Paulo",
	},
	"SWKT": {
		ICAO:      "SWKT",
		IATA:      "TLZ",
		Name:      "Catalao Airport",
		City:      "Catalao",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2612,
		Latitude:  -18.2168006897,
		Longitude: -47.8997001648,
		Timezone:  "America/Sao_Paulo",
	},
	"SWKU": {
		ICAO:      "SWKU",
		Name:      "Cucui Airport",
		City:      "Sao Gabriel Da Cachoeira",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 274,
		Latitude:  1.1206099987,
		Longitude: -66.841796875,
		Timezone:  "America/Manaus",
	},
	"SWKV": {
		ICAO:      "SWKV",
		Name:      "Fazenda Beira Rio Airport",
		City:      "Itauba",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1096,
		Latitude:  -10.9008331299,
		Longitude: -55.8583335876,
		Timezone:  "America/Cuiaba",
	},
	"SWKX": {
		ICAO:      "SWKX",
		IATA:      "0",
		Name:      "Corixa Airport",
		City:      "Caceres",
		State:     "Santa-Cruz",
		Country:   "BR",
		Elevation: 394,
		Latitude:  -16.383600235,
		Longitude: -58.317199707,
		Timezone:  "America/Cuiaba",
	},
	"SWKY": {
		ICAO:      "SWKY",
		Name:      "Fazenda Recanto da Cachoeira Airport",
		City:      "Cristais Paulista",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 3064,
		Latitude:  -20.3397216797,
		Longitude: -47.4730567932,
		Timezone:  "America/Sao_Paulo",
	},
	"SWKZ": {
		ICAO:      "SWKZ",
		Name:      "Fazenda Cerejal Airport",
		City:      "Brasnorte",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 984,
		Latitude:  -11.717499733,
		Longitude: -58.2594451904,
		Timezone:  "America/Cuiaba",
	},
	"SWLA": {
		ICAO:      "SWLA",
		Name:      "Fazenda Amalia Airport",
		City:      "Juina",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1030,
		Latitude:  -11.884721756,
		Longitude: -58.6316680908,
		Timezone:  "America/Cuiaba",
	},
	"SWLB": {
		ICAO:      "SWLB",
		IATA:      "LBR",
		Name:      "Labrea Airport",
		City:      "Labrea",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 190,
		Latitude:  -7.2789697647,
		Longitude: -64.7695007324,
		Timezone:  "America/Manaus",
	},
	"SWLC": {
		ICAO:      "SWLC",
		IATA:      "RVD",
		Name:      "General Leite de Castro Airport",
		City:      "Rio Verde",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2464,
		Latitude:  -17.8347225189,
		Longitude: -50.956111908,
		Timezone:  "America/Sao_Paulo",
	},
	"SWLD": {
		ICAO:      "SWLD",
		Name:      "Porto Cajueiro Airport",
		City:      "Januaria",
		State:     "Bahia",
		Country:   "BR",
		Elevation: 2510,
		Latitude:  -14.8858327866,
		Longitude: -45.3386116028,
		Timezone:  "America/Bahia",
	},
	"SWLE": {
		ICAO:      "SWLE",
		Name:      "Pontes e Lacerda Airport",
		City:      "Pontes E Lacerda",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1148,
		Latitude:  -15.2311105728,
		Longitude: -59.3411102295,
		Timezone:  "America/Cuiaba",
	},
	"SWLF": {
		ICAO:      "SWLF",
		Name:      "Fazenda Dona Laura Airport",
		City:      "Sao Jose Dos Quatro Marcos",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 725,
		Latitude:  -15.6202783585,
		Longitude: -58.2088890076,
		Timezone:  "America/Cuiaba",
	},
	"SWLG": {
		ICAO:      "SWLG",
		Name:      "Recanto Airport",
		City:      "Ribeirao Cascalheira",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1335,
		Latitude:  -12.999444,
		Longitude: -51.905556,
		Timezone:  "America/Cuiaba",
	},
	"SWLI": {
		ICAO:      "SWLI",
		Name:      "Fazenda Liberdade Airport",
		City:      "Barao De Melgaco",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 361,
		Latitude:  -17.0080566406,
		Longitude: -56.4661102295,
		Timezone:  "America/Cuiaba",
	},
	"SWLK": {
		ICAO:      "SWLK",
		Name:      "Fazenda Conceicao Airport",
		City:      "Juara",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1164,
		Latitude:  -11.0002784729,
		Longitude: -58.2152786255,
		Timezone:  "America/Cuiaba",
	},
	"SWLM": {
		ICAO:      "SWLM",
		Name:      "Fazenda Excalibur Airport",
		City:      "Vila Bela Da Santissima Trindade",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 971,
		Latitude:  -15.2944440842,
		Longitude: -60.3663902283,
		Timezone:  "America/Cuiaba",
	},
	"SWLN": {
		ICAO:      "SWLN",
		Name:      "Fazenda Franciosi Airport",
		City:      "Sapezal",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1837,
		Latitude:  -13.4322223663,
		Longitude: -58.4727783203,
		Timezone:  "America/Cuiaba",
	},
	"SWLP": {
		ICAO:      "SWLP",
		Name:      "Fazenda Rosa - Paralelo 10 Airport",
		City:      "Aripuana",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 535,
		Latitude:  -9.9686107635,
		Longitude: -61.2297210693,
		Timezone:  "America/Cuiaba",
	},
	"SWLR": {
		ICAO:      "SWLR",
		Name:      "Fazenda Miura Airport",
		City:      "Pontes E Lacerda",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 820,
		Latitude:  -15.4405555725,
		Longitude: -59.6572227478,
		Timezone:  "America/Cuiaba",
	},
	"SWLT": {
		ICAO:      "SWLT",
		Name:      "Fazenda Tatiana Airport",
		City:      "Marcelandia",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1197,
		Latitude:  -10.433889389,
		Longitude: -54.1672210693,
		Timezone:  "America/Cuiaba",
	},
	"SWLV": {
		ICAO:      "SWLV",
		Name:      "Santo Antonio do Lerverger Airport",
		City:      "Santo Antonio Do Leverger",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 544,
		Latitude:  -15.8534002304,
		Longitude: -56.0873985291,
		Timezone:  "America/Cuiaba",
	},
	"SWMA": {
		ICAO:      "SWMA",
		Name:      "Aero Agricola Fulanete Airport",
		City:      "Barreiras",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 2828,
		Latitude:  -11.8336105347,
		Longitude: -46.2666664124,
		Timezone:  "America/Bahia",
	},
	"SWMB": {
		ICAO:      "SWMB",
		Name:      "Fazenda Marimbondo Airport",
		City:      "Goiatuba",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2165,
		Latitude:  -17.9772224426,
		Longitude: -49.5883331299,
		Timezone:  "America/Sao_Paulo",
	},
	"SWMC": {
		ICAO:      "SWMC",
		Name:      "Capao da Cruz Airport",
		City:      "Luis Antonio",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1936,
		Latitude:  -21.4583320618,
		Longitude: -47.8877792358,
		Timezone:  "America/Sao_Paulo",
	},
	"SWME": {
		ICAO:      "SWME",
		Name:      "Mineiros Airport",
		City:      "Mineiros",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2707,
		Latitude:  -17.551399231,
		Longitude: -52.5567016602,
		Timezone:  "America/Sao_Paulo",
	},
	"SWMG": {
		ICAO:      "SWMG",
		Name:      "Fazenda Caraibas Airport",
		City:      "Monte Alegre De Goias",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1637,
		Latitude:  -13.385556221,
		Longitude: -46.7750015259,
		Timezone:  "America/Sao_Paulo",
	},
	"SWMH": {
		ICAO:      "SWMH",
		Name:      "Fazenda Bom Pastor Airport",
		City:      "Itapirapua",
		State:     "Goias",
		Country:   "BR",
		Elevation: 895,
		Latitude:  -15.542222023,
		Longitude: -51.0177764893,
		Timezone:  "America/Sao_Paulo",
	},
	"SWMJ": {
		ICAO:      "SWMJ",
		Name:      "Pousada Xaraes Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 318,
		Latitude:  -19.4958324432,
		Longitude: -56.9599990845,
		Timezone:  "America/Campo_Grande",
	},
	"SWMK": {
		ICAO:      "SWMK",
		Name:      "Maturaca Airport",
		City:      "Sao Gabriel Da Cachoeira",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 354,
		Latitude:  0.6282690167,
		Longitude: -66.1150970459,
		Timezone:  "America/Manaus",
	},
	"SWML": {
		ICAO:      "SWML",
		Name:      "Monte Alegre de Goias Airport",
		City:      "Monte Alegre De Goias",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1821,
		Latitude:  -13.2497215271,
		Longitude: -46.8713874817,
		Timezone:  "America/Sao_Paulo",
	},
	"SWMM": {
		ICAO:      "SWMM",
		Name:      "Buritizal Agropastoril Ltda Airport",
		City:      "Aripuana",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 432,
		Latitude:  -9.30027771,
		Longitude: -60.6749992371,
		Timezone:  "America/Cuiaba",
	},
	"SWMN": {
		ICAO:      "SWMN",
		Name:      "Fazenda Forquilha Airport",
		City:      "Matrincha",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1076,
		Latitude:  -15.2163887024,
		Longitude: -50.8366661072,
		Timezone:  "America/Sao_Paulo",
	},
	"SWMO": {
		ICAO:      "SWMO",
		Name:      "Fazenda Maria Jose Airport",
		City:      "Itiquira",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1716,
		Latitude:  -17.4527778625,
		Longitude: -54.7422218323,
		Timezone:  "America/Cuiaba",
	},
	"SWMQ": {
		ICAO:      "SWMQ",
		Name:      "Fazenda Muiraquita Airport",
		City:      "Aripuana",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 367,
		Latitude:  -10.4169435501,
		Longitude: -60.5505561829,
		Timezone:  "America/Cuiaba",
	},
	"SWMR": {
		ICAO:      "SWMR",
		Name:      "Morro Redondo Airport",
		City:      "Morro Redondo",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2067,
		Latitude:  -14.2505559921,
		Longitude: -48.0838890076,
		Timezone:  "America/Sao_Paulo",
	},
	"SWMT": {
		ICAO:      "SWMT",
		Name:      "Fazenda Centro da Mata Airport",
		City:      "Nova Ubirata",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1214,
		Latitude:  -12.6577777863,
		Longitude: -54.8277778625,
		Timezone:  "America/Cuiaba",
	},
	"SWMU": {
		ICAO:      "SWMU",
		Name:      "Sumuru Airport",
		City:      "Boa Vista",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 1640,
		Latitude:  4.3163900375,
		Longitude: -60.7005996704,
		Timezone:  "America/Boa_Vista",
	},
	"SWMV": {
		ICAO:      "SWMV",
		Name:      "Paa-Piu Airport",
		City:      "Mucajai",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 984,
		Latitude:  2.7269399166,
		Longitude: -62.9505996704,
		Timezone:  "America/Boa_Vista",
	},
	"SWMW": {
		ICAO:      "SWMW",
		IATA:      "MBZ",
		Name:      "Maues Airport",
		City:      "Maues",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 69,
		Latitude:  -3.3721699715,
		Longitude: -57.7248001099,
		Timezone:  "America/Manaus",
	},
	"SWMX": {
		ICAO:      "SWMX",
		Name:      "Morrinhos Airport",
		City:      "Morrinhos",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2789,
		Latitude:  -17.7614994049,
		Longitude: -49.1221008301,
		Timezone:  "America/Sao_Paulo",
	},
	"SWMY": {
		ICAO:      "SWMY",
		Name:      "Porto Fluvial Suia Missu Airport",
		City:      "Barra Do Garcas",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 623,
		Latitude:  -11.5669441223,
		Longitude: -50.7005577087,
		Timezone:  "America/Araguaina",
	},
	"SWMZ": {
		ICAO:      "SWMZ",
		Name:      "Estancia Aeronaldo Airport",
		City:      "Rondonopolis",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1000,
		Latitude:  -16.4427776337,
		Longitude: -54.7019424438,
		Timezone:  "America/Cuiaba",
	},
	"SWNA": {
		ICAO:      "SWNA",
		IATA:      "NVP",
		Name:      "Novo Aripuana Airport",
		City:      "Novo Aripuana",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 118,
		Latitude:  -5.1180300713,
		Longitude: -60.3648986816,
		Timezone:  "America/Manaus",
	},
	"SWNB": {
		ICAO:      "SWNB",
		Name:      "Nobres Airport",
		City:      "Nobres",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 722,
		Latitude:  -14.7072219849,
		Longitude: -56.3383331299,
		Timezone:  "America/Cuiaba",
	},
	"SWNC": {
		ICAO:      "SWNC",
		Name:      "Fazenda Procomp Airport",
		City:      "Nova Bandeirantes",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 673,
		Latitude:  -9.4011106491,
		Longitude: -58.2286109924,
		Timezone:  "America/Cuiaba",
	},
	"SWNF": {
		ICAO:      "SWNF",
		Name:      "Agropecuaria Tratex Airport",
		City:      "Colider",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1033,
		Latitude:  -10.8919439316,
		Longitude: -55.5641670227,
		Timezone:  "America/Cuiaba",
	},
	"SWNG": {
		ICAO:      "SWNG",
		Name:      "Fazenda Nossa Senhora das Gracas Airport",
		City:      "Sorriso",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1279,
		Latitude:  -12.8966665268,
		Longitude: -55.8344421387,
		Timezone:  "America/Cuiaba",
	},
	"SWNH": {
		ICAO:      "SWNH",
		Name:      "Aruana Airport",
		City:      "Aruana",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 820,
		Latitude:  -14.9324998856,
		Longitude: -51.0480995178,
		Timezone:  "America/Sao_Paulo",
	},
	"SWNI": {
		ICAO:      "SWNI",
		IATA:      "AQM",
		Name:      "Nova Vida Airport",
		City:      "Ariquemes",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 410,
		Latitude:  -10.1780557632,
		Longitude: -62.8255577087,
		Timezone:  "America/Porto_Velho",
	},
	"SWNJ": {
		ICAO:      "SWNJ",
		Name:      "Fazenda Itaquere Airport",
		City:      "Novo Sao Joaquim",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1749,
		Latitude:  -15.1283330917,
		Longitude: -53.4916687012,
		Timezone:  "America/Cuiaba",
	},
	"SWNK": {
		ICAO:      "SWNK",
		IATA:      "BCR",
		Name:      "Novo Campo Airport",
		City:      "Boca Do Acre",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 394,
		Latitude:  -8.8345603943,
		Longitude: -67.3124008179,
		Timezone:  "America/Manaus",
	},
	"SWNL": {
		ICAO:      "SWNL",
		Name:      "Fazenda Aracatuba Airport",
		City:      "Lambari D`Oeste",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 493,
		Latitude:  -15.1908330917,
		Longitude: -57.7075004578,
		Timezone:  "America/Cuiaba",
	},
	"SWNM": {
		ICAO:      "SWNM",
		Name:      "Normandia Airport",
		City:      "Normandia",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 282,
		Latitude:  3.8663899899,
		Longitude: -59.6171989441,
		Timezone:  "America/Boa_Vista",
	},
	"SWNN": {
		ICAO:      "SWNN",
		Name:      "Novo Intento Airport",
		City:      "Canta",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 282,
		Latitude:  2.7799999714,
		Longitude: -60.6166687012,
		Timezone:  "America/Boa_Vista",
	},
	"SWNO": {
		ICAO:      "SWNO",
		Name:      "Nova Olinda do Norte Airport",
		City:      "Nova Olinda Do Norte",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 121,
		Latitude:  -3.8877780437,
		Longitude: -59.0783348083,
		Timezone:  "America/Manaus",
	},
	"SWNQ": {
		ICAO:      "SWNQ",
		IATA:      "NQL",
		Name:      "Niquelandia Airport",
		City:      "Niquelandia",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2756,
		Latitude:  -14.4349002838,
		Longitude: -48.4915008545,
		Timezone:  "America/Sao_Paulo",
	},
	"SWNR": {
		ICAO:      "SWNR",
		Name:      "Nortelandia Airport",
		City:      "Nortelandia",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1460,
		Latitude:  -14.4836111069,
		Longitude: -56.7672233582,
		Timezone:  "America/Cuiaba",
	},
	"SWNS": {
		ICAO:      "SWNS",
		IATA:      "APS",
		Name:      "Anapolis Airport",
		City:      "Anapolis",
		State:     "Goias",
		Country:   "BR",
		Elevation: 3648,
		Latitude:  -16.3623008728,
		Longitude: -48.9271011353,
		Timezone:  "America/Sao_Paulo",
	},
	"SWNT": {
		ICAO:      "SWNT",
		Name:      "Fazenda Santo Antonio Airport",
		City:      "Cacu",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1493,
		Latitude:  -18.9650001526,
		Longitude: -50.7986106873,
		Timezone:  "America/Sao_Paulo",
	},
	"SWNV": {
		ICAO:      "SWNV",
		Name:      "Aerodromo Nacional de Aviacao Airport",
		City:      "Goiania",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2707,
		Latitude:  -16.6254005432,
		Longitude: -49.3493003845,
		Timezone:  "America/Sao_Paulo",
	},
	"SWNW": {
		ICAO:      "SWNW",
		Name:      "Fazenda Tuiuiu Airport",
		City:      "Nobres",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1525,
		Latitude:  -13.7994441986,
		Longitude: -55.4550018311,
		Timezone:  "America/Cuiaba",
	},
	"SWNX": {
		ICAO:      "SWNX",
		Name:      "Fazenda Independencia Airport",
		City:      "Santo Antonio Do Leste",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1650,
		Latitude:  -14.8680562973,
		Longitude: -53.5011100769,
		Timezone:  "America/Cuiaba",
	},
	"SWOA": {
		ICAO:      "SWOA",
		Name:      "Arnapar Airport",
		City:      "Goianira",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2625,
		Latitude:  -16.5394439697,
		Longitude: -49.3988876343,
		Timezone:  "America/Sao_Paulo",
	},
	"SWOB": {
		ICAO:      "SWOB",
		IATA:      "FBA",
		Name:      "Fonte Boa Airport",
		City:      "Fonte Boa",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 207,
		Latitude:  -2.5326099396,
		Longitude: -66.0831985474,
		Timezone:  "America/Manaus",
	},
	"SWOC": {
		ICAO:      "SWOC",
		Name:      "Fazenda Sao Bento do Bocaja Airport",
		City:      "Caracol",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 443,
		Latitude:  -22.0252780914,
		Longitude: -57.3647232056,
		Timezone:  "America/Campo_Grande",
	},
	"SWOE": {
		ICAO:      "SWOE",
		Name:      "Fazenda Duas Lagoas Airport",
		City:      "Caceres",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 688,
		Latitude:  -16.1708335876,
		Longitude: -58.1872215271,
		Timezone:  "America/Cuiaba",
	},
	"SWOF": {
		ICAO:      "SWOF",
		Name:      "Ouro Fino Airport",
		City:      "Boa Vista",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 282,
		Latitude:  2.9380600452,
		Longitude: -60.7200012207,
		Timezone:  "America/Boa_Vista",
	},
	"SWOG": {
		ICAO:      "SWOG",
		Name:      "Fazenda Itaipu Airport",
		City:      "Sao Jose Do Rio Claro",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1470,
		Latitude:  -13.7525,
		Longitude: -56.96111,
		Timezone:  "America/Cuiaba",
	},
	"SWOI": {
		ICAO:      "SWOI",
		Name:      "Fazenda Juara Airport",
		City:      "Juara",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1200,
		Latitude:  -11.8169441223,
		Longitude: -57.5661125183,
		Timezone:  "America/Cuiaba",
	},
	"SWOJ": {
		ICAO:      "SWOJ",
		Name:      "Fazenda Nova Alvorada Airport",
		City:      "Brasnorte",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1092,
		Latitude:  -11.5733327866,
		Longitude: -58.2697219849,
		Timezone:  "America/Cuiaba",
	},
	"SWOK": {
		ICAO:      "SWOK",
		Name:      "Fazenda Pontal Airport",
		City:      "Itiquira",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2164,
		Latitude:  -17.3055553436,
		Longitude: -53.9355545044,
		Timezone:  "America/Cuiaba",
	},
	"SWOL": {
		ICAO:      "SWOL",
		Name:      "Fazenda Roncador Airport",
		City:      "Barra Do Garcas",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1351,
		Latitude:  -15.0961112976,
		Longitude: -52.4000015259,
		Timezone:  "America/Cuiaba",
	},
	"SWOM": {
		ICAO:      "SWOM",
		Name:      "Fazenda Salto Bello II Airport",
		City:      "Novo Sao Joaquim",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1889,
		Latitude:  -14.9611110687,
		Longitude: -53.4880561829,
		Timezone:  "America/Cuiaba",
	},
	"SWOO": {
		ICAO:      "SWOO",
		Name:      "Fazenda Tres Marias do Agua Limpa Airport",
		City:      "Britania",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1050,
		Latitude:  -15.0713891983,
		Longitude: -51.3927764893,
		Timezone:  "America/Sao_Paulo",
	},
	"SWOP": {
		ICAO:      "SWOP",
		Name:      "Fazenda Agropecuaria Embu Airport",
		City:      "Nova Monte Verde",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1188,
		Latitude:  -9.7875003815,
		Longitude: -57.6244430542,
		Timezone:  "America/Cuiaba",
	},
	"SWOQ": {
		ICAO:      "SWOQ",
		Name:      "Fazenda Surucucu Airport",
		City:      "Luziania",
		State:     "Goias",
		Country:   "BR",
		Elevation: 3148,
		Latitude:  -16.7433338165,
		Longitude: -47.9916687012,
		Timezone:  "America/Sao_Paulo",
	},
	"SWOR": {
		ICAO:      "SWOR",
		Name:      "Fazenda Colorado Airport",
		City:      "Britania",
		State:     "Goias",
		Country:   "BR",
		Elevation: 896,
		Latitude:  -15.4508333206,
		Longitude: -51.2233314514,
		Timezone:  "America/Sao_Paulo",
	},
	"SWOS": {
		ICAO:      "SWOS",
		Name:      "Fazenda Tapajos Airport",
		City:      "Goianesia",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2289,
		Latitude:  -15.1719436646,
		Longitude: -48.9530563354,
		Timezone:  "America/Sao_Paulo",
	},
	"SWOT": {
		ICAO:      "SWOT",
		Name:      "Fazenda Planorte Airport",
		City:      "Sapezal",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2106,
		Latitude:  -13.943333,
		Longitude: -58.895278,
		Timezone:  "America/Cuiaba",
	},
	"SWOV": {
		ICAO:      "SWOV",
		Name:      "Retiro Arrombadinho Airport",
		City:      "Itiquira",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 423,
		Latitude:  -17.5074996948,
		Longitude: -55.4080543518,
		Timezone:  "America/Campo_Grande",
	},
	"SWOW": {
		ICAO:      "SWOW",
		Name:      "Moura Airport",
		City:      "Moura",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 43,
		Latitude:  -1.4669439793,
		Longitude: -61.6172218323,
		Timezone:  "America/Manaus",
	},
	"SWOX": {
		ICAO:      "SWOX",
		Name:      "Estancia Barbosa Airport",
		City:      "Nossa Senhora Do Livramento",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 619,
		Latitude:  -15.7655563354,
		Longitude: -56.4955558777,
		Timezone:  "America/Cuiaba",
	},
	"SWOY": {
		ICAO:      "SWOY",
		Name:      "Fazenda Duas Barras Airport",
		City:      "Pontal Do Araguaia",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1239,
		Latitude:  -16.0252780914,
		Longitude: -52.8122215271,
		Timezone:  "America/Cuiaba",
	},
	"SWOZ": {
		ICAO:      "SWOZ",
		Name:      "Fazenda Santo Antonio Airport",
		City:      "Campo Verde",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2164,
		Latitude:  -15.3105564117,
		Longitude: -54.8991661072,
		Timezone:  "America/Cuiaba",
	},
	"SWPA": {
		ICAO:      "SWPA",
		Name:      "Pedro Afonso Airport",
		City:      "Pedro Afonso",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 630,
		Latitude:  -8.997618,
		Longitude: -48.155375,
		Timezone:  "America/Araguaina",
	},
	"SWPB": {
		ICAO:      "SWPB",
		Name:      "Forte Principe da Beira Airport",
		City:      "Costa Marques",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 591,
		Latitude:  -12.4186105728,
		Longitude: -64.4188919067,
		Timezone:  "America/La_Paz",
	},
	"SWPC": {
		ICAO:      "SWPC",
		Name:      "Pari Cachoeira Airport",
		City:      "Sao Gabriel Da Cachoeira",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 377,
		Latitude:  0.26305601,
		Longitude: -69.7955551147,
		Timezone:  "America/Manaus",
	},
	"SWPD": {
		ICAO:      "SWPD",
		Name:      "Pouso da Aguia Airport",
		City:      "Canta",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 282,
		Latitude:  2.7836110592,
		Longitude: -60.5877761841,
		Timezone:  "America/Boa_Vista",
	},
	"SWPE": {
		ICAO:      "SWPE",
		Name:      "Fazenda Lagoa da Mata Airport",
		City:      "Carlinda",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 866,
		Latitude:  -10.1363887787,
		Longitude: -55.5950012207,
		Timezone:  "America/Cuiaba",
	},
	"SWPG": {
		ICAO:      "SWPG",
		IATA:      "PBV",
		Name:      "Porto dos Gauchos Airport",
		City:      "Porto Dos Gauchos",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1312,
		Latitude:  -11.5169439316,
		Longitude: -57.3338890076,
		Timezone:  "America/Cuiaba",
	},
	"SWPH": {
		ICAO:      "SWPH",
		Name:      "Fazenda Paulo Abreu Airport",
		City:      "Barra Do Garcas",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 978,
		Latitude:  -15.226111412,
		Longitude: -52.1300010681,
		Timezone:  "America/Cuiaba",
	},
	"SWPI": {
		ICAO:      "SWPI",
		IATA:      "PIN",
		Name:      "Parintins Airport",
		City:      "Parintins",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 87,
		Latitude:  -2.673019886,
		Longitude: -56.7771987915,
		Timezone:  "America/Manaus",
	},
	"SWPJ": {
		ICAO:      "SWPJ",
		Name:      "Fazenda Sao Caetano Airport",
		City:      "Pontal Do Araguaia",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1587,
		Latitude:  -16.0341663361,
		Longitude: -53.0344429016,
		Timezone:  "America/Cuiaba",
	},
	"SWPK": {
		ICAO:      "SWPK",
		Name:      "Pocone Airport",
		City:      "Pocone",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 591,
		Latitude:  -16.2727775574,
		Longitude: -56.6508331299,
		Timezone:  "America/Cuiaba",
	},
	"SWPL": {
		ICAO:      "SWPL",
		Name:      "Posto Leonardo Vilas Boas Airport",
		City:      "Chapada Dos Guimaraes",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1083,
		Latitude:  -12.1983327866,
		Longitude: -53.3816680908,
		Timezone:  "America/Cuiaba",
	},
	"SWPM": {
		ICAO:      "SWPM",
		IATA:      "PBQ",
		Name:      "Pimenta Bueno Airport",
		City:      "Pimenta Bueno",
		Country:   "BR",
		Elevation: 682,
		Latitude:  -11.6415996552,
		Longitude: -61.1791000366,
		Timezone:  "America/Porto_Velho",
	},
	"SWPN": {
		ICAO:      "SWPN",
		Name:      "Parana Airport",
		City:      "Parana",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 886,
		Latitude:  -12.6197223663,
		Longitude: -47.8725013733,
		Timezone:  "America/Araguaina",
	},
	"SWPO": {
		ICAO:      "SWPO",
		Name:      "Fazenda Porta do Amazonia Airport",
		City:      "Santa Terezinha",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 787,
		Latitude:  -10.2652778625,
		Longitude: -51.2413902283,
		Timezone:  "America/Cuiaba",
	},
	"SWPP": {
		ICAO:      "SWPP",
		Name:      "Fazenda Pica-Pau Airport",
		City:      "Mozarlandia",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1001,
		Latitude:  -14.7530555725,
		Longitude: -50.6650009155,
		Timezone:  "America/Sao_Paulo",
	},
	"SWPQ": {
		ICAO:      "SWPQ",
		IATA:      "PBX",
		Name:      "Fazenda Piraguassu Airport",
		City:      "Porto Alegre Do Norte",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 705,
		Latitude:  -10.8611106873,
		Longitude: -51.6850013733,
		Timezone:  "America/Cuiaba",
	},
	"SWPR": {
		ICAO:      "SWPR",
		Name:      "Pires do Rio Airport",
		City:      "Pires Do Rio",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2428,
		Latitude:  -17.3190994263,
		Longitude: -48.3348007202,
		Timezone:  "America/Sao_Paulo",
	},
	"SWPS": {
		ICAO:      "SWPS",
		Name:      "Porto Sao Sebastiao Airport",
		City:      "Diamantino",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1106,
		Latitude:  -12.9669437408,
		Longitude: -56.1838874817,
		Timezone:  "America/Cuiaba",
	},
	"SWPT": {
		ICAO:      "SWPT",
		Name:      "Fazenda Piratinga Airport",
		City:      "Sinop",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1706,
		Latitude:  -13.6172218323,
		Longitude: -55.0133323669,
		Timezone:  "America/Cuiaba",
	},
	"SWPV": {
		ICAO:      "SWPV",
		Name:      "Planaltina Airport",
		City:      "Brasilia",
		State:     "Federal-District",
		Country:   "BR",
		Elevation: 3806,
		Latitude:  -15.6788892746,
		Longitude: -47.5441665649,
		Timezone:  "America/Sao_Paulo",
	},
	"SWPW": {
		ICAO:      "SWPW",
		Name:      "Pouso da Garca Airport",
		City:      "Barao De Melgaco",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 377,
		Latitude:  -16.9672222137,
		Longitude: -56.4172210693,
		Timezone:  "America/Cuiaba",
	},
	"SWPY": {
		ICAO:      "SWPY",
		Name:      "Primavera do Leste Airport",
		City:      "Primavera Do Leste",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2149,
		Latitude:  -15.5655555725,
		Longitude: -54.3377761841,
		Timezone:  "America/Cuiaba",
	},
	"SWPZ": {
		ICAO:      "SWPZ",
		Name:      "Oricanga de Abreu Airport",
		City:      "Posse",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2713,
		Latitude:  -14.1181001663,
		Longitude: -46.3454017639,
		Timezone:  "America/Sao_Paulo",
	},
	"SWQE": {
		ICAO:      "SWQE",
		Name:      "Querari Airport",
		City:      "Sao Gabriel Da Cachoeira",
		State:     "Vaupes",
		Country:   "BR",
		Elevation: 492,
		Latitude:  1.0913900137,
		Longitude: -69.8506011963,
		Timezone:  "America/Bogota",
	},
	"SWQG": {
		ICAO:      "SWQG",
		Name:      "Fazenda Sossego Airport",
		City:      "Maracaju",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1706,
		Latitude:  -21.4572219849,
		Longitude: -55.4211120605,
		Timezone:  "America/Campo_Grande",
	},
	"SWQH": {
		ICAO:      "SWQH",
		Name:      "Agropecuaria Nova Santana Airport",
		City:      "Aripuana",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 797,
		Latitude:  -10.1472215652,
		Longitude: -60.5386123657,
		Timezone:  "America/Cuiaba",
	},
	"SWQI": {
		ICAO:      "SWQI",
		Name:      "Caracarai Airport",
		City:      "Caracarai",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 180,
		Latitude:  1.832874,
		Longitude: -61.126896,
		Timezone:  "America/Boa_Vista",
	},
	"SWQJ": {
		ICAO:      "SWQJ",
		Name:      "Campos de Julio Airport",
		City:      "Campos De Julio",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2132,
		Latitude:  -13.4147224426,
		Longitude: -59.2675018311,
		Timezone:  "America/Cuiaba",
	},
	"SWQK": {
		ICAO:      "SWQK",
		Name:      "Pedra Airport",
		City:      "Pedra Lavrada",
		State:     "Paraiba",
		Country:   "BR",
		Elevation: 1781,
		Latitude:  -6.7666668892,
		Longitude: -36.4811096191,
		Timezone:  "America/Fortaleza",
	},
	"SWQL": {
		ICAO:      "SWQL",
		Name:      "Fazenda Tres Companheiros Airport",
		City:      "Doverlandia",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2070,
		Latitude:  -16.6825008392,
		Longitude: -52.2524986267,
		Timezone:  "America/Sao_Paulo",
	},
	"SWQM": {
		ICAO:      "SWQM",
		Name:      "Fazenda Boa Vista Airport",
		City:      "Acorizal",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 688,
		Latitude:  -15.0894441605,
		Longitude: -56.3733329773,
		Timezone:  "America/Cuiaba",
	},
	"SWQP": {
		ICAO:      "SWQP",
		Name:      "Rio Vermelho Acucar e Alcool Airport",
		City:      "Junqueiropolis",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1148,
		Latitude:  -21.3152770996,
		Longitude: -51.3699989319,
		Timezone:  "America/Sao_Paulo",
	},
	"SWQQ": {
		ICAO:      "SWQQ",
		Name:      "Fazenda Fundao Alegre Airport",
		City:      "Camapua",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1391,
		Latitude:  -18.6280555725,
		Longitude: -54.1038894653,
		Timezone:  "America/Campo_Grande",
	},
	"SWQR": {
		ICAO:      "SWQR",
		Name:      "Estancia Norteagro Airport",
		City:      "Boa Vista",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 279,
		Latitude:  2.6758329868,
		Longitude: -60.7786102295,
		Timezone:  "America/Boa_Vista",
	},
	"SWQT": {
		ICAO:      "SWQT",
		Name:      "Fazenda Cotriguacu Airport",
		City:      "Aripuana",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 738,
		Latitude:  -9.864443779,
		Longitude: -58.2291679382,
		Timezone:  "America/Cuiaba",
	},
	"SWQU": {
		ICAO:      "SWQU",
		Name:      "Fazenda Guapore Airport",
		City:      "Vila Bela Da Santissima Trindade",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 728,
		Latitude:  -14.5874996185,
		Longitude: -60.1263885498,
		Timezone:  "America/Cuiaba",
	},
	"SWQV": {
		ICAO:      "SWQV",
		Name:      "Fazenda Maristela Airport",
		City:      "Porto Esperidiao",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 833,
		Latitude:  -16.1166667938,
		Longitude: -58.969165802,
		Timezone:  "America/Cuiaba",
	},
	"SWQW": {
		ICAO:      "SWQW",
		Name:      "Fazenda Bacaeri Airport",
		City:      "Alta Floresta",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1050,
		Latitude:  -10.052778244,
		Longitude: -56.8255577087,
		Timezone:  "America/Cuiaba",
	},
	"SWQX": {
		ICAO:      "SWQX",
		Name:      "Fazenda Barro Preto Airport",
		City:      "Monte Alegre De Goias",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1578,
		Latitude:  -13.3352775574,
		Longitude: -46.90111111,
		Timezone:  "America/Sao_Paulo",
	},
	"SWQZ": {
		ICAO:      "SWQZ",
		Name:      "Fazenda Quatro Irmaos Airport",
		City:      "Ribas Do Rio Pardo",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1509,
		Latitude:  -20.8783321381,
		Longitude: -53.5533332825,
		Timezone:  "America/Campo_Grande",
	},
	"SWRA": {
		ICAO:      "SWRA",
		IATA:      "AAI",
		Name:      "Arraias Airport",
		City:      "Arraias",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 1923,
		Latitude:  -13.023888588,
		Longitude: -46.8855552673,
		Timezone:  "America/Araguaina",
	},
	"SWRB": {
		ICAO:      "SWRB",
		Name:      "Fazenda Rio dos Bugres Airport",
		City:      "Barra Do Bugres",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1148,
		Latitude:  -14.8169441223,
		Longitude: -56.9505577087,
		Timezone:  "America/Cuiaba",
	},
	"SWRC": {
		ICAO:      "SWRC",
		Name:      "Fazenda Sevilha Airport",
		City:      "Ribeirao Cascalheira",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 951,
		Latitude:  -12.6266670227,
		Longitude: -51.9341659546,
		Timezone:  "America/Cuiaba",
	},
	"SWRD": {
		ICAO:      "SWRD",
		IATA:      "ROO",
		Name:      "Rondonopolis Airport",
		City:      "Rondonopolis",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1467,
		Latitude:  -16.5860004425,
		Longitude: -54.7248001099,
		Timezone:  "America/Cuiaba",
	},
	"SWRF": {
		ICAO:      "SWRF",
		Name:      "Fazenda Toloza Airport",
		City:      "Brasnorte",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1644,
		Latitude:  -13.1836109161,
		Longitude: -57.9838905334,
		Timezone:  "America/Cuiaba",
	},
	"SWRG": {
		ICAO:      "SWRG",
		Name:      "Fazenda Rancho Maria e Tereza Airport",
		City:      "Guajara-Mirim",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 492,
		Latitude:  -10.8258333206,
		Longitude: -65.2286148071,
		Timezone:  "America/Porto_Velho",
	},
	"SWRI": {
		ICAO:      "SWRI",
		Name:      "Fazenda Roncador I Airport",
		City:      "Querencia",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 993,
		Latitude:  -12.1554002762,
		Longitude: -52.2789993286,
		Timezone:  "America/Cuiaba",
	},
	"SWRJ": {
		ICAO:      "SWRJ",
		Name:      "Fazenda Panflora Airport",
		City:      "Rosario Oeste",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 757,
		Latitude:  -15.0833330154,
		Longitude: -56.5527763367,
		Timezone:  "America/Cuiaba",
	},
	"SWRK": {
		ICAO:      "SWRK",
		Name:      "South West Rocks Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -30.9333000183,
		Longitude: 153.0330047607,
		Timezone:  "Australia/Sydney",
	},
	"SWRL": {
		ICAO:      "SWRL",
		Name:      "Fazenda Pouso Redondo Airport",
		City:      "Corumbiara",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 1086,
		Latitude:  -12.9772224426,
		Longitude: -61.0491676331,
		Timezone:  "America/Porto_Velho",
	},
	"SWRM": {
		ICAO:      "SWRM",
		Name:      "Santo Antonio do Descoberto Airport",
		City:      "Santo Antonio Do Descoberto",
		State:     "Goias",
		Country:   "BR",
		Elevation: 3674,
		Latitude:  -15.9305562973,
		Longitude: -48.3336105347,
		Timezone:  "America/Sao_Paulo",
	},
	"SWRO": {
		ICAO:      "SWRO",
		Name:      "Aeroclube de Rondonia Airport",
		City:      "Porto Velho",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 312,
		Latitude:  -8.7936105728,
		Longitude: -63.8580551147,
		Timezone:  "America/Porto_Velho",
	},
	"SWRP": {
		ICAO:      "SWRP",
		IATA:      "AIR",
		Name:      "Aripuana Airport",
		City:      "Aripuana",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 623,
		Latitude:  -10.2502775192,
		Longitude: -59.3838882446,
		Timezone:  "America/Cuiaba",
	},
	"SWRR": {
		ICAO:      "SWRR",
		Name:      "Fazenda Serra Azul Airport",
		City:      "Planalto Da Serra",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1542,
		Latitude:  -14.6677780151,
		Longitude: -54.5675010681,
		Timezone:  "America/Cuiaba",
	},
	"SWRS": {
		ICAO:      "SWRS",
		Name:      "Fazenda Santa Monica Airport",
		City:      "Rondonopolis",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1047,
		Latitude:  -16.9333324432,
		Longitude: -54.9069442749,
		Timezone:  "America/Cuiaba",
	},
	"SWRT": {
		ICAO:      "SWRT",
		Name:      "Fazenda Santa Rita Airport",
		City:      "Santa Rita Do Araguaia",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2346,
		Latitude:  -17.3477783203,
		Longitude: -53.1930541992,
		Timezone:  "America/Cuiaba",
	},
	"SWRU": {
		ICAO:      "SWRU",
		IATA:      "0",
		Name:      "Meruri Airport",
		City:      "Tesouro",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1460,
		Latitude:  -15.5564002991,
		Longitude: -53.0752983093,
		Timezone:  "America/Cuiaba",
	},
	"SWRV": {
		ICAO:      "SWRV",
		Name:      "Fazenda Vale do Rio Verde Airport",
		City:      "Diamantino",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1378,
		Latitude:  -12.1669435501,
		Longitude: -56.5338897705,
		Timezone:  "America/Cuiaba",
	},
	"SWRW": {
		ICAO:      "SWRW",
		Name:      "Fazenda Santa Alice Airport",
		City:      "Mirassol D`Oeste",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1115,
		Latitude:  -15.5874996185,
		Longitude: -58.0444450378,
		Timezone:  "America/Cuiaba",
	},
	"SWRX": {
		ICAO:      "SWRX",
		Name:      "Fazenda Santa Cruz do Pessoe Airport",
		City:      "Vila Bela Da Santissima Trindade",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 764,
		Latitude:  -15.1649999619,
		Longitude: -60.4458312988,
		Timezone:  "America/Cuiaba",
	},
	"SWRY": {
		ICAO:      "SWRY",
		Name:      "Fazenda Santa Maria Airport",
		City:      "Porto Estrela",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1853,
		Latitude:  -15.7375001907,
		Longitude: -57.4705543518,
		Timezone:  "America/Cuiaba",
	},
	"SWRZ": {
		ICAO:      "SWRZ",
		Name:      "Fazenda Arrossensal Airport",
		City:      "Nortelandia",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1306,
		Latitude:  -14.353099823,
		Longitude: -56.7137985229,
		Timezone:  "America/Cuiaba",
	},
	"SWSA": {
		ICAO:      "SWSA",
		Name:      "Fazenda Sao Joao Airport",
		City:      "Diamantino",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1476,
		Latitude:  -12.2891674042,
		Longitude: -56.5338897705,
		Timezone:  "America/Cuiaba",
	},
	"SWSB": {
		ICAO:      "SWSB",
		Name:      "Fazenda Sabia Airport",
		City:      "Araguaiana",
		Country:   "BR",
		Elevation: 951,
		Latitude:  -15.233332634,
		Longitude: -51.9772224426,
		Timezone:  "America/Cuiaba",
	},
	"SWSC": {
		ICAO:      "SWSC",
		Name:      "Fazenda Santa Cruz Airport",
		City:      "Sao Felix Do Araguaia",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1083,
		Latitude:  -11.6536111832,
		Longitude: -52.7333335876,
		Timezone:  "America/Cuiaba",
	},
	"SWSD": {
		ICAO:      "SWSD",
		Name:      "Fazenda Santo Antonio Airport",
		City:      "Araguaiana",
		Country:   "BR",
		Elevation: 1079,
		Latitude:  -15.6238889694,
		Longitude: -51.8138885498,
		Timezone:  "America/Cuiaba",
	},
	"SWSF": {
		ICAO:      "SWSF",
		Name:      "Fazenda Sao Francisco Airport",
		City:      "Vila Bela Da Santissima Trindade",
		Country:   "BR",
		Elevation: 863,
		Latitude:  -15.2241001129,
		Longitude: -60.2172012329,
		Timezone:  "America/Cuiaba",
	},
	"SWSG": {
		ICAO:      "SWSG",
		Name:      "Fazenda Simarelli Airport",
		City:      "Campos De Julio",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2136,
		Latitude:  -13.6068000793,
		Longitude: -59.3167991638,
		Timezone:  "America/Cuiaba",
	},
	"SWSH": {
		ICAO:      "SWSH",
		Name:      "Sao Raimundo II Airport",
		City:      "Alta Floresta",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 932,
		Latitude:  -9.9663887024,
		Longitude: -56.1527786255,
		Timezone:  "America/Cuiaba",
	},
	"SWSI": {
		ICAO:      "SWSI",
		IATA:      "OPS",
		Name:      "Presidente Joao Batista Figueiredo Airport",
		City:      "Sinop",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1227,
		Latitude:  -11.8850002289,
		Longitude: -55.5861091614,
		Timezone:  "America/Cuiaba",
	},
	"SWSK": {
		ICAO:      "SWSK",
		Name:      "Fazenda Sao Marco Airport",
		City:      "Boa Vista",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 295,
		Latitude:  3.0497200489,
		Longitude: -60.4838981628,
		Timezone:  "America/Boa_Vista",
	},
	"SWSM": {
		ICAO:      "SWSM",
		Name:      "Fazenda Santa Maria Airport",
		City:      "Lambari D`Oeste",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 525,
		Latitude:  -15.35833,
		Longitude: -57.69444,
		Timezone:  "America/Cuiaba",
	},
	"SWSP": {
		ICAO:      "SWSP",
		Name:      "Sopave Norte Airport",
		City:      "Paranatinga",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1690,
		Latitude:  -13.7252779007,
		Longitude: -54.7383346558,
		Timezone:  "America/Cuiaba",
	},
	"SWSQ": {
		ICAO:      "SWSQ",
		Name:      "Sao Joaquim Airport",
		City:      "Sao Gabriel Da Cachoeira",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 541,
		Latitude:  1.733055556,
		Longitude: -69.38388889,
		Timezone:  "America/Bogota",
	},
	"SWSR": {
		ICAO:      "SWSR",
		Name:      "Sertania Airport",
		City:      "Sertania",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1772,
		Latitude:  -13.7502784729,
		Longitude: -55.6338882446,
		Timezone:  "America/Cuiaba",
	},
	"SWSS": {
		ICAO:      "SWSS",
		Name:      "Fazenda Silmar Airport",
		City:      "Vila Bela Da Santissima Trindade",
		State:     "Santa-Cruz",
		Country:   "BR",
		Elevation: 863,
		Latitude:  -16.1650009155,
		Longitude: -60.1144447327,
		Timezone:  "America/Cuiaba",
	},
	"SWST": {
		ICAO:      "SWST",
		IATA:      "STZ",
		Name:      "Santa Terezinha Airport",
		City:      "Santa Terezinha",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 663,
		Latitude:  -10.4647216797,
		Longitude: -50.518611908,
		Timezone:  "America/Araguaina",
	},
	"SWSU": {
		ICAO:      "SWSU",
		Name:      "Fazenda Santa Juliana Airport",
		City:      "Ribeirao Cascalheira",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1230,
		Latitude:  -12.8522224426,
		Longitude: -52.0208320618,
		Timezone:  "America/Cuiaba",
	},
	"SWSV": {
		ICAO:      "SWSV",
		Name:      "Fazenda Santa Silvia Airport",
		City:      "Cocalinho",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 837,
		Latitude:  -14.3886108398,
		Longitude: -51.3805541992,
		Timezone:  "America/Cuiaba",
	},
	"SWSX": {
		ICAO:      "SWSX",
		Name:      "Seringal Nova Olinda Airport",
		City:      "Sena Mafureira",
		State:     "Acre",
		Country:   "BR",
		Elevation: 673,
		Latitude:  -10.1044435501,
		Longitude: -69.2119445801,
		Timezone:  "America/Rio_Branco",
	},
	"SWSY": {
		ICAO:      "SWSY",
		Name:      "Sao Simao Airport",
		City:      "Vila Bela Da Santissima Trindade",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 705,
		Latitude:  -15.9086112976,
		Longitude: -60.1547203064,
		Timezone:  "America/Cuiaba",
	},
	"SWSZ": {
		ICAO:      "SWSZ",
		Name:      "Fazenda Santa Luzia Airport",
		City:      "Nova Bandeirantes",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 741,
		Latitude:  -9.7736110687,
		Longitude: -58.0627784729,
		Timezone:  "America/Cuiaba",
	},
	"SWTA": {
		ICAO:      "SWTA",
		Name:      "Fazenda Santa Marcia Airport",
		City:      "Araputanga",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 938,
		Latitude:  -15.4152784348,
		Longitude: -58.4088897705,
		Timezone:  "America/Cuiaba",
	},
	"SWTB": {
		ICAO:      "SWTB",
		Name:      "Tabajara Airport",
		City:      "Ariquemes",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 722,
		Latitude:  -8.9169435501,
		Longitude: -62.1172218323,
		Timezone:  "America/Porto_Velho",
	},
	"SWTC": {
		ICAO:      "SWTC",
		Name:      "Tocantinia Airport",
		City:      "Tocantinia",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 656,
		Latitude:  -9.5836105347,
		Longitude: -48.4505577087,
		Timezone:  "America/Araguaina",
	},
	"SWTD": {
		ICAO:      "SWTD",
		Name:      "Fazenda Sete de Setembro Airport",
		City:      "Agua Boa",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1066,
		Latitude:  -13.881111145,
		Longitude: -52.4000015259,
		Timezone:  "America/Cuiaba",
	},
	"SWTE": {
		ICAO:      "SWTE",
		Name:      "Fazenda Trescinco Airport",
		City:      "Diamantino",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1132,
		Latitude:  -13.9169435501,
		Longitude: -56.6588897705,
		Timezone:  "America/Cuiaba",
	},
	"SWTH": {
		ICAO:      "SWTH",
		Name:      "Fazenda Trescinco Juruena Airport",
		City:      "Alta Floresta",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 883,
		Latitude:  -10.2447223663,
		Longitude: -58.2494430542,
		Timezone:  "America/Cuiaba",
	},
	"SWTI": {
		ICAO:      "SWTI",
		Name:      "Sao Vicente Airport",
		City:      "Nova Lacerda",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 791,
		Latitude:  -14.5202999115,
		Longitude: -59.7798995972,
		Timezone:  "America/Cuiaba",
	},
	"SWTJ": {
		ICAO:      "SWTJ",
		Name:      "Fazenda Cachoeira Airport",
		City:      "Tangara Da Serra",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1070,
		Latitude:  -14.6033334732,
		Longitude: -57.7972221375,
		Timezone:  "America/Cuiaba",
	},
	"SWTK": {
		ICAO:      "SWTK",
		Name:      "Fazenda Santa Maria do Guapore Airport",
		City:      "Vila Bela Da Santissima Trindade",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 797,
		Latitude:  -14.2019443512,
		Longitude: -60.280834198,
		Timezone:  "America/Cuiaba",
	},
	"SWTL": {
		ICAO:      "SWTL",
		Name:      "Fazenda Corumbiara Airport",
		City:      "Pimenta Bueno",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 1371,
		Latitude:  -12.5005559921,
		Longitude: -61.2836112976,
		Timezone:  "America/Porto_Velho",
	},
	"SWTN": {
		ICAO:      "SWTN",
		Name:      "Fazenda Arapucel Airport",
		City:      "Araputanga",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1345,
		Latitude:  -15.0413894653,
		Longitude: -58.7291679382,
		Timezone:  "America/Cuiaba",
	},
	"SWTO": {
		ICAO:      "SWTO",
		Name:      "ParaiSo do Tocantins Airport",
		City:      "Paraiso Do Tocantins",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 1083,
		Latitude:  -10.174200058,
		Longitude: -48.932800293,
		Timezone:  "America/Araguaina",
	},
	"SWTP": {
		ICAO:      "SWTP",
		IATA:      "IRZ",
		Name:      "Tapuruquara Airport",
		City:      "Santa Isabel Do Rio Negro",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 223,
		Latitude:  -0.4169439971,
		Longitude: -65.0338897705,
		Timezone:  "America/Manaus",
	},
	"SWTQ": {
		ICAO:      "SWTQ",
		Name:      "Termas Pousada do Rio Quente Airport",
		City:      "Rio Quente",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2247,
		Latitude:  -17.7686004639,
		Longitude: -48.7616996765,
		Timezone:  "America/Sao_Paulo",
	},
	"SWTR": {
		ICAO:      "SWTR",
		Name:      "Taraqua Airport",
		City:      "Sao Gabriel Da Cachoeira",
		Country:   "BR",
		Elevation: 246,
		Latitude:  0.1247219965,
		Longitude: -68.5456008911,
		Timezone:  "America/Manaus",
	},
	"SWTS": {
		ICAO:      "SWTS",
		IATA:      "TGQ",
		Name:      "Tangara da Serra Airport",
		City:      "Tangara Da Serra",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1460,
		Latitude:  -14.6619997025,
		Longitude: -57.4435005188,
		Timezone:  "America/Cuiaba",
	},
	"SWTU": {
		ICAO:      "SWTU",
		IATA:      "AZL",
		Name:      "Fazenda Tucunare Airport",
		City:      "Sapezal",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1814,
		Latitude:  -13.4655278136,
		Longitude: -58.8669347763,
		Timezone:  "America/Cuiaba",
	},
	"SWTV": {
		ICAO:      "SWTV",
		Name:      "Fazenda Rio Preto Airport",
		City:      "Salto Do Ceu",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1354,
		Latitude:  -15.0622215271,
		Longitude: -57.9686126709,
		Timezone:  "America/Cuiaba",
	},
	"SWTW": {
		ICAO:      "SWTW",
		Name:      "Fazenda Sao Lucas Airport",
		City:      "Pontes E Lacerda",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 833,
		Latitude:  -15.7063894272,
		Longitude: -59.2416687012,
		Timezone:  "America/Cuiaba",
	},
	"SWTX": {
		ICAO:      "SWTX",
		Name:      "Fazenda Olho D`Agua Airport",
		City:      "Torixoreu",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1201,
		Latitude:  -16.1452789307,
		Longitude: -52.9713897705,
		Timezone:  "America/Cuiaba",
	},
	"SWTY": {
		ICAO:      "SWTY",
		IATA:      "QHN",
		Name:      "Taguatinga Airport",
		City:      "Taguatinga",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 1959,
		Latitude:  -12.433889389,
		Longitude: -46.400554657,
		Timezone:  "America/Araguaina",
	},
	"SWUA": {
		ICAO:      "SWUA",
		IATA:      "SQM",
		Name:      "Sao Miguel do Araguaia Airport",
		City:      "Sao Miguel Do Araguaia",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1249,
		Latitude:  -13.3312997818,
		Longitude: -50.1976013184,
		Timezone:  "America/Sao_Paulo",
	},
	"SWUB": {
		ICAO:      "SWUB",
		Name:      "Fazenda Serra Alegre Airport",
		City:      "Pontes E Lacerda",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1115,
		Latitude:  -15.6741666794,
		Longitude: -59.1916656494,
		Timezone:  "America/Cuiaba",
	},
	"SWUC": {
		ICAO:      "SWUC",
		Name:      "Luciara Airport",
		City:      "Luciara",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 689,
		Latitude:  -11.2002782822,
		Longitude: -50.6838874817,
		Timezone:  "America/Cuiaba",
	},
	"SWUE": {
		ICAO:      "SWUE",
		Name:      "Fazenda Tapayuna Airport",
		City:      "Nova Canaa Do Norte",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1197,
		Latitude:  -10.703332901,
		Longitude: -55.936668396,
		Timezone:  "America/Cuiaba",
	},
	"SWUI": {
		ICAO:      "SWUI",
		Name:      "Pauini Airport",
		City:      "Pauini",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 397,
		Latitude:  -7.7224998474,
		Longitude: -67.0283355713,
		Timezone:  "America/Manaus",
	},
	"SWUJ": {
		ICAO:      "SWUJ",
		Name:      "Tagua Agropecuaria Airport",
		City:      "Vera",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1230,
		Latitude:  -12.2049999237,
		Longitude: -55.3777770996,
		Timezone:  "America/Cuiaba",
	},
	"SWUK": {
		ICAO:      "SWUK",
		Name:      "Uapui Cachoeira Airport",
		City:      "Sao Gabriel Da Cachoeira",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 249,
		Latitude:  1.2163889408,
		Longitude: -69.200553894,
		Timezone:  "America/Manaus",
	},
	"SWUL": {
		ICAO:      "SWUL",
		Name:      "Fazenda Furna Azul Airport",
		City:      "Ananas",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 407,
		Latitude:  -6.2180562019,
		Longitude: -48.4213905334,
		Timezone:  "America/Araguaina",
	},
	"SWUM": {
		ICAO:      "SWUM",
		Name:      "Fazenda Rancho Novo Airport",
		City:      "Comodoro",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 984,
		Latitude:  -13.4425001144,
		Longitude: -60.2727775574,
		Timezone:  "America/Cuiaba",
	},
	"SWUN": {
		ICAO:      "SWUN",
		Name:      "Fazenda Santa Eunice Airport",
		City:      "Nova Monte Verde",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 886,
		Latitude:  -10.1719436646,
		Longitude: -57.567779541,
		Timezone:  "America/Cuiaba",
	},
	"SWUP": {
		ICAO:      "SWUP",
		Name:      "Uirapuru Airport",
		City:      "Uirapuru",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1575,
		Latitude:  -14.0169439316,
		Longitude: -59.3672218323,
		Timezone:  "America/Cuiaba",
	},
	"SWUQ": {
		ICAO:      "SWUQ",
		Name:      "Surucucu Airport",
		City:      "Alto Alegre",
		State:     "Roraima",
		Country:   "BR",
		Elevation: 2854,
		Latitude:  2.8352799416,
		Longitude: -63.6469993591,
		Timezone:  "America/Boa_Vista",
	},
	"SWUR": {
		ICAO:      "SWUR",
		Name:      "Fazenda Vista Verde Airport",
		City:      "Palmeira Do Piaui",
		State:     "Piaui",
		Country:   "BR",
		Elevation: 1945,
		Latitude:  -8.4916667938,
		Longitude: -44.3894424438,
		Timezone:  "America/Fortaleza",
	},
	"SWUS": {
		ICAO:      "SWUS",
		Name:      "Sede Fazenda Sao Benedito Airport",
		City:      "Campo Novo Do Parecis",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1798,
		Latitude:  -14.2349996567,
		Longitude: -57.7091674805,
		Timezone:  "America/Cuiaba",
	},
	"SWUT": {
		ICAO:      "SWUT",
		Name:      "Lavoura Fazenda Sao Benedito Airport",
		City:      "Campo Novo Do Parecis",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2001,
		Latitude:  -14.220000267,
		Longitude: -57.7788887024,
		Timezone:  "America/Cuiaba",
	},
	"SWUU": {
		ICAO:      "SWUU",
		Name:      "Fazenda Uirapuru Airport",
		City:      "Sinop",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1148,
		Latitude:  -12.1336107254,
		Longitude: -54.2005577087,
		Timezone:  "America/Cuiaba",
	},
	"SWUW": {
		ICAO:      "SWUW",
		Name:      "Fazenda Santa Barbara II Airport",
		City:      "Alto Paraguai",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 810,
		Latitude:  -15.1138887405,
		Longitude: -56.8369445801,
		Timezone:  "America/Cuiaba",
	},
	"SWUY": {
		ICAO:      "SWUY",
		Name:      "Urucu Airport",
		City:      "Coari",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 210,
		Latitude:  -4.8842201233,
		Longitude: -65.3554000854,
		Timezone:  "America/Manaus",
	},
	"SWUZ": {
		ICAO:      "SWUZ",
		Name:      "Brigadeiro Araripe Macedo Airport",
		City:      "Luziania",
		State:     "Goias",
		Country:   "BR",
		Elevation: 3269,
		Latitude:  -16.2616996765,
		Longitude: -47.9687004089,
		Timezone:  "America/Sao_Paulo",
	},
	"SWVB": {
		ICAO:      "SWVB",
		IATA:      "MTG",
		Name:      "Vila Bela da Santissima Trindade Airport",
		City:      "Vila Bela Da Santissima Trindade",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 886,
		Latitude:  -15.0152778625,
		Longitude: -59.9672203064,
		Timezone:  "America/Cuiaba",
	},
	"SWVC": {
		ICAO:      "SWVC",
		IATA:      "VLP",
		Name:      "Vila Rica Airport",
		City:      "Vila Rica",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 892,
		Latitude:  -9.9794435501,
		Longitude: -51.1422233582,
		Timezone:  "America/Cuiaba",
	},
	"SWVD": {
		ICAO:      "SWVD",
		Name:      "Fazenda Rio Verde Airport",
		City:      "Serranopolis",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2592,
		Latitude:  -17.9691677094,
		Longitude: -52.5777778625,
		Timezone:  "America/Sao_Paulo",
	},
	"SWVE": {
		ICAO:      "SWVE",
		Name:      "Fazenda Vista Alegre Airport",
		City:      "Bujari",
		State:     "Acre",
		Country:   "BR",
		Elevation: 666,
		Latitude:  -9.788611412,
		Longitude: -68.0072250366,
		Timezone:  "America/Rio_Branco",
	},
	"SWVF": {
		ICAO:      "SWVF",
		Name:      "Fazenda Sto Antonio Guapore - Sede Nova Airport",
		City:      "Vila Bela Da Santissima Trindade",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 709,
		Latitude:  -16.0094432831,
		Longitude: -59.9280548096,
		Timezone:  "America/Cuiaba",
	},
	"SWVH": {
		ICAO:      "SWVH",
		Name:      "Fazenda Vaca Branca Airport",
		City:      "Alta Floresta",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1149,
		Latitude:  -9.5244436264,
		Longitude: -56.2397232056,
		Timezone:  "America/Cuiaba",
	},
	"SWVI": {
		ICAO:      "SWVI",
		Name:      "Fazenda Pirassununga Airport",
		City:      "Sao Valerio",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 787,
		Latitude:  -11.5177783966,
		Longitude: -48.3111114502,
		Timezone:  "America/Araguaina",
	},
	"SWVJ": {
		ICAO:      "SWVJ",
		Name:      "Fazenda Uiapuru Airport",
		City:      "Nova Mutum",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1519,
		Latitude:  -13.6638889313,
		Longitude: -56.0022201538,
		Timezone:  "America/Cuiaba",
	},
	"SWVK": {
		ICAO:      "SWVK",
		Name:      "Fazenda Vale do Rio Celeste Airport",
		City:      "Nova Ubirata",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1411,
		Latitude:  -13.0369443893,
		Longitude: -55.3261108398,
		Timezone:  "America/Cuiaba",
	},
	"SWVL": {
		ICAO:      "SWVL",
		Name:      "Fazenda Vertente Airport",
		City:      "Santo Antonio Do Leverger",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2444,
		Latitude:  -15.8966665268,
		Longitude: -55.3155555725,
		Timezone:  "America/Cuiaba",
	},
	"SWVN": {
		ICAO:      "SWVN",
		Name:      "Valdir Masutti Airport",
		City:      "Comodoro",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2028,
		Latitude:  -13.6327781677,
		Longitude: -59.7958335876,
		Timezone:  "America/Cuiaba",
	},
	"SWVO": {
		ICAO:      "SWVO",
		Name:      "Panorama Airport",
		City:      "Claudia",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1017,
		Latitude:  -11.3702783585,
		Longitude: -55.3805541992,
		Timezone:  "America/Cuiaba",
	},
	"SWVP": {
		ICAO:      "SWVP",
		Name:      "Fazenda Santa Isabel Airport",
		City:      "Cutias",
		State:     "Amapa",
		Country:   "BR",
		Elevation: 33,
		Latitude:  1.1566669941,
		Longitude: -50.3302764893,
		Timezone:  "America/Belem",
	},
	"SWVR": {
		ICAO:      "SWVR",
		Name:      "Estancia Rosalia Airport",
		City:      "Sinop",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1460,
		Latitude:  -11.8999996185,
		Longitude: -55.4591674805,
		Timezone:  "America/Cuiaba",
	},
	"SWVS": {
		ICAO:      "SWVS",
		Name:      "Fazenda Sto Antonio do Guapore - Baia do Padre Airport",
		City:      "Vila Bela Da Santissima Trindade",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 675,
		Latitude:  -15.859167099,
		Longitude: -60.0447235107,
		Timezone:  "America/Cuiaba",
	},
	"SWVT": {
		ICAO:      "SWVT",
		Name:      "Fazenda Estrela D`Alva Airport",
		City:      "Querencia",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1050,
		Latitude:  -12.1233329773,
		Longitude: -52.6458320618,
		Timezone:  "America/Cuiaba",
	},
	"SWVU": {
		ICAO:      "SWVU",
		Name:      "Fazenda Ramalhete Airport",
		City:      "Rio Brilhante",
		Country:   "BR",
		Elevation: 1099,
		Latitude:  -21.7332992554,
		Longitude: -54.5766983032,
		Timezone:  "America/Campo_Grande",
	},
	"SWVV": {
		ICAO:      "SWVV",
		Name:      "Fazenda Esperanca Airport",
		City:      "Vila Bela Da Santissima Trindade",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 692,
		Latitude:  -15.7519435883,
		Longitude: -60.1047210693,
		Timezone:  "America/Cuiaba",
	},
	"SWVY": {
		ICAO:      "SWVY",
		Name:      "Fazenda Carvalho Airport",
		City:      "Corumba",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 361,
		Latitude:  -17.5680561066,
		Longitude: -56.4880561829,
		Timezone:  "America/Campo_Grande",
	},
	"SWVZ": {
		ICAO:      "SWVZ",
		Name:      "EMAL - Empresa de Mineracao Aripuana Ltda Airport",
		City:      "Nobres",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2362,
		Latitude:  -14.75,
		Longitude: -56.3527793884,
		Timezone:  "America/Cuiaba",
	},
	"SWWA": {
		ICAO:      "SWWA",
		Name:      "Porangatu Airport",
		City:      "Porangatu",
		Country:   "BR",
		Elevation: 1201,
		Latitude:  -13.404800415,
		Longitude: -49.1582984924,
		Timezone:  "America/Sao_Paulo",
	},
	"SWWB": {
		ICAO:      "SWWB",
		Name:      "Fazenda Rio Azul Airport",
		City:      "Claudia",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1083,
		Latitude:  -11.2577781677,
		Longitude: -54.9269447327,
		Timezone:  "America/Cuiaba",
	},
	"SWWD": {
		ICAO:      "SWWD",
		Name:      "C.F. de Souza Bais Airport",
		City:      "Barcelos",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 207,
		Latitude:  -1.691388889,
		Longitude: -63.785,
		Timezone:  "America/Manaus",
	},
	"SWWF": {
		ICAO:      "SWWF",
		Name:      "Hotel Baiazinha Airport",
		City:      "Caceres",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 676,
		Latitude:  -16.5697212219,
		Longitude: -57.8313903809,
		Timezone:  "America/Cuiaba",
	},
	"SWWG": {
		ICAO:      "SWWG",
		Name:      "Fazenda Canamari Airport",
		City:      "Quirinopolis",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1699,
		Latitude:  -18.54806,
		Longitude: -50.38111,
		Timezone:  "America/Sao_Paulo",
	},
	"SWWI": {
		ICAO:      "SWWI",
		Name:      "Fazenda Fortaleza Airport",
		City:      "General Carneiro",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1995,
		Latitude:  -15.673333168,
		Longitude: -53.5783348083,
		Timezone:  "America/Cuiaba",
	},
	"SWWJ": {
		ICAO:      "SWWJ",
		Name:      "Posto de Protecao Ambiental Nossa Senhora do Carmo Airport",
		City:      "Barao De Melgaco",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 348,
		Latitude:  -16.6808338165,
		Longitude: -56.2991676331,
		Timezone:  "America/Cuiaba",
	},
	"SWWK": {
		ICAO:      "SWWK",
		Name:      "Urucara Airport",
		City:      "Urucara",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 98,
		Latitude:  -2.5288898945,
		Longitude: -57.7557983398,
		Timezone:  "America/Manaus",
	},
	"SWWL": {
		ICAO:      "SWWL",
		Name:      "Fazenda Agrocentro Airport",
		City:      "Barra Do Garcas",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1119,
		Latitude:  -15.6967000961,
		Longitude: -52.2639007568,
		Timezone:  "America/Cuiaba",
	},
	"SWWM": {
		ICAO:      "SWWM",
		Name:      "Fazenda Pupila Airport",
		City:      "Mantena",
		State:     "Espirito-Santo",
		Country:   "BR",
		Elevation: 663,
		Latitude:  -18.771944046,
		Longitude: -40.9494438171,
		Timezone:  "America/Sao_Paulo",
	},
	"SWWN": {
		ICAO:      "SWWN",
		Name:      "Fazenda Santa Amalia Airport",
		City:      "Alta Floresta",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1201,
		Latitude:  -10.3130559921,
		Longitude: -56.5788879395,
		Timezone:  "America/Cuiaba",
	},
	"SWWT": {
		ICAO:      "SWWT",
		Name:      "Silvio Goncalves de Mello Airport",
		City:      "Morada Nova De Minas",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1988,
		Latitude:  -18.7055568695,
		Longitude: -45.3355560303,
		Timezone:  "America/Sao_Paulo",
	},
	"SWWU": {
		ICAO:      "SWWU",
		Name:      "Uruacu Airport",
		City:      "Uruacu",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1706,
		Latitude:  -14.5255556107,
		Longitude: -49.1377792358,
		Timezone:  "America/Sao_Paulo",
	},
	"SWWV": {
		ICAO:      "SWWV",
		Name:      "Posto de Protecao Ambiental Santa Maria Airport",
		City:      "Barao De Melgaco",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 381,
		Latitude:  -16.7063884735,
		Longitude: -56.0291671753,
		Timezone:  "America/Cuiaba",
	},
	"SWWW": {
		ICAO:      "SWWW",
		Name:      "Posto de Protecao Ambiental Sao Luiz Airport",
		City:      "Barao De Melgaco",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 367,
		Latitude:  -16.6883335114,
		Longitude: -56.1763877869,
		Timezone:  "America/Cuiaba",
	},
	"SWWX": {
		ICAO:      "SWWX",
		Name:      "Posto de Protecao Ambiental Sao Joaquim Airport",
		City:      "Barao De Melgaco",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 338,
		Latitude:  -16.7461109161,
		Longitude: -56.3699989319,
		Timezone:  "America/Cuiaba",
	},
	"SWWY": {
		ICAO:      "SWWY",
		Name:      "Posto de Protecao Ambiental Espirito Santo Airport",
		City:      "Barao De Melgaco",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 354,
		Latitude:  -16.5872211456,
		Longitude: -56.2799987793,
		Timezone:  "America/Cuiaba",
	},
	"SWWZ": {
		ICAO:      "SWWZ",
		Name:      "Darinha Airport",
		City:      "Cascavel",
		State:     "Ceara",
		Country:   "BR",
		Elevation: 13,
		Latitude:  -4.1163887978,
		Longitude: -38.250831604,
		Timezone:  "America/Fortaleza",
	},
	"SWXA": {
		ICAO:      "SWXA",
		Name:      "Fazenda Canadazinho Airport",
		City:      "Jussara",
		State:     "Goias",
		Country:   "BR",
		Elevation: 889,
		Latitude:  -15.042222023,
		Longitude: -51.400554657,
		Timezone:  "America/Sao_Paulo",
	},
	"SWXC": {
		ICAO:      "SWXC",
		Name:      "Fazenda Tropical Airport",
		City:      "Cerejeiras",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 981,
		Latitude:  -12.8860998154,
		Longitude: -61.2983016968,
		Timezone:  "America/Porto_Velho",
	},
	"SWXD": {
		ICAO:      "SWXD",
		Name:      "Aerogardi Airport",
		City:      "Itapolis",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1762,
		Latitude:  -21.5611114502,
		Longitude: -48.7986106873,
		Timezone:  "America/Sao_Paulo",
	},
	"SWXG": {
		ICAO:      "SWXG",
		Name:      "Xingu Airport",
		City:      "Barra Do Garcas",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1000,
		Latitude:  -12.0055561066,
		Longitude: -53.4022216797,
		Timezone:  "America/Cuiaba",
	},
	"SWXH": {
		ICAO:      "SWXH",
		Name:      "Fazenda Sao Sebastiao Airport",
		City:      "Sao Desiderio",
		Country:   "BR",
		Elevation: 2999,
		Latitude:  -13.0633325577,
		Longitude: -46.2572212219,
		Timezone:  "America/Bahia",
	},
	"SWXJ": {
		ICAO:      "SWXJ",
		Name:      "Chacara Sao Jose Airport",
		City:      "Cuiaba",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 568,
		Latitude:  -15.5419435501,
		Longitude: -55.9838905334,
		Timezone:  "America/Cuiaba",
	},
	"SWXK": {
		ICAO:      "SWXK",
		Name:      "Fazenda Rio Alegre Airport",
		City:      "Sao Jose Do Rio Claro",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1240,
		Latitude:  -13.2742004395,
		Longitude: -56.8922996521,
		Timezone:  "America/Cuiaba",
	},
	"SWXL": {
		ICAO:      "SWXL",
		Name:      "Fazenda Lago Grande Airport",
		City:      "Sandolandia",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 600,
		Latitude:  -12.1013889313,
		Longitude: -49.9511108398,
		Timezone:  "America/Araguaina",
	},
	"SWXM": {
		ICAO:      "SWXM",
		IATA:      "MBK",
		Name:      "Regional Orlando Villas Boas Airport",
		City:      "Matupa",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 886,
		Latitude:  -10.1702775955,
		Longitude: -54.9527778625,
		Timezone:  "America/Cuiaba",
	},
	"SWXO": {
		ICAO:      "SWXO",
		Name:      "Fazenda Porto Seguro Airport",
		City:      "Sao Felix Do Xingu",
		State:     "Para",
		Country:   "BR",
		Elevation: 800,
		Latitude:  -7.1316671371,
		Longitude: -52.7272224426,
		Timezone:  "America/Santarem",
	},
	"SWXP": {
		ICAO:      "SWXP",
		Name:      "Fazenda Santa Luzia Airport",
		City:      "Sapezal",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1949,
		Latitude:  -13.7327775955,
		Longitude: -58.8880577087,
		Timezone:  "America/Cuiaba",
	},
	"SWXR": {
		ICAO:      "SWXR",
		Name:      "Faria Airport",
		City:      "Porto Esperidiao",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 787,
		Latitude:  -15.6458330154,
		Longitude: -58.7727775574,
		Timezone:  "America/Cuiaba",
	},
	"SWXS": {
		ICAO:      "SWXS",
		Name:      "Fazenda Sao Joao do Ibipora Airport",
		City:      "Pontes E Lacerda",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 951,
		Latitude:  -15.7899999619,
		Longitude: -59.636390686,
		Timezone:  "America/Cuiaba",
	},
	"SWXT": {
		ICAO:      "SWXT",
		Name:      "Fazenda Medalha Milagrosa Airport",
		City:      "Vila Bela Da Santissima Trindade",
		State:     "Santa-Cruz",
		Country:   "BR",
		Elevation: 853,
		Latitude:  -16.1016674042,
		Longitude: -60.0649986267,
		Timezone:  "America/Cuiaba",
	},
	"SWXU": {
		ICAO:      "SWXU",
		Name:      "Xapuri Airport",
		City:      "Xapuri",
		State:     "Acre",
		Country:   "BR",
		Elevation: 492,
		Latitude:  -10.6336107254,
		Longitude: -68.5505523682,
		Timezone:  "America/Rio_Branco",
	},
	"SWXV": {
		ICAO:      "SWXV",
		IATA:      "NOK",
		Name:      "Xavantina Airport",
		City:      "Nova Xavantina",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1035,
		Latitude:  -14.6983003616,
		Longitude: -52.3464012146,
		Timezone:  "America/Cuiaba",
	},
	"SWXW": {
		ICAO:      "SWXW",
		Name:      "Fazenda Sao Joao do Guapore Airport",
		City:      "Pontes E Lacerda",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 722,
		Latitude:  -15.9708328247,
		Longitude: -59.8911094666,
		Timezone:  "America/Cuiaba",
	},
	"SWXY": {
		ICAO:      "SWXY",
		Name:      "Fazenda Transoeste Airport",
		City:      "Santa Terezinha",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 869,
		Latitude:  -10.3508329391,
		Longitude: -50.6708335876,
		Timezone:  "America/Cuiaba",
	},
	"SWXZ": {
		ICAO:      "SWXZ",
		Name:      "Fazenda Taquarussu Airport",
		City:      "Comodoro",
		State:     "Rondonia",
		Country:   "BR",
		Elevation: 899,
		Latitude:  -13.3125,
		Longitude: -60.2933311462,
		Timezone:  "America/Cuiaba",
	},
	"SWYA": {
		ICAO:      "SWYA",
		Name:      "Fazenda Nossa Senhora Aparecida Airport",
		City:      "Jaciara",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1082,
		Latitude:  -16.0441665649,
		Longitude: -54.9516677856,
		Timezone:  "America/Cuiaba",
	},
	"SWYC": {
		ICAO:      "SWYC",
		Name:      "Jair Feliciano de Deus Airport",
		City:      "Mirassol D`Oeste",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 849,
		Latitude:  -15.6872215271,
		Longitude: -58.1113891602,
		Timezone:  "America/Cuiaba",
	},
	"SWYG": {
		ICAO:      "SWYG",
		Name:      "Sao Jose Airport",
		City:      "Aracruz",
		State:     "Espirito-Santo",
		Country:   "BR",
		Elevation: 187,
		Latitude:  -19.6216678619,
		Longitude: -40.2011108398,
		Timezone:  "America/Sao_Paulo",
	},
	"SWYH": {
		ICAO:      "SWYH",
		Name:      "Fazenda Alianca Airport",
		City:      "Araputanga",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 984,
		Latitude:  -15.2924995422,
		Longitude: -58.3144454956,
		Timezone:  "America/Cuiaba",
	},
	"SWYI": {
		ICAO:      "SWYI",
		Name:      "Barragem Manoel Alves Airport",
		City:      "Dianopolis",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 2067,
		Latitude:  -11.5516672134,
		Longitude: -46.9838905334,
		Timezone:  "America/Araguaina",
	},
	"SWYJ": {
		ICAO:      "SWYJ",
		Name:      "Fazenda Agropecuaria FR Ltda. Airport",
		City:      "Novo Progresso",
		State:     "Para",
		Country:   "BR",
		Elevation: 860,
		Latitude:  -7.6224999428,
		Longitude: -56.0683326721,
		Timezone:  "America/Santarem",
	},
	"SWYK": {
		ICAO:      "SWYK",
		Name:      "Fazenda Ilha Camargo Airport",
		City:      "Pocone",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 358,
		Latitude:  -17.0625991821,
		Longitude: -56.5836982727,
		Timezone:  "America/Cuiaba",
	},
	"SWYM": {
		ICAO:      "SWYM",
		Name:      "Fazenda Anhanguera Airport",
		City:      "Pontes E Lacerda",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 951,
		Latitude:  -14.6424999237,
		Longitude: -59.4491004944,
		Timezone:  "America/Cuiaba",
	},
	"SWYN": {
		ICAO:      "SWYN",
		Name:      "Prainha Airport",
		City:      "Apui",
		State:     "Amazonas",
		Country:   "BR",
		Elevation: 197,
		Latitude:  -7.1728701591,
		Longitude: -59.8395996094,
		Timezone:  "America/Manaus",
	},
	"SWYO": {
		ICAO:      "SWYO",
		Name:      "Usina Branco Peres Airport",
		City:      "Adamantina",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1335,
		Latitude:  -21.4869441986,
		Longitude: -51.025554657,
		Timezone:  "America/Sao_Paulo",
	},
	"SWYP": {
		ICAO:      "SWYP",
		Name:      "Fazenda Ribeiro do Ceu Airport",
		City:      "Nova Mutum",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1493,
		Latitude:  -13.9527778625,
		Longitude: -55.7783317566,
		Timezone:  "America/Cuiaba",
	},
	"SWYQ": {
		ICAO:      "SWYQ",
		Name:      "Fazenda Nova Fronteira Airport",
		City:      "Sinop",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1017,
		Latitude:  -11.4678001404,
		Longitude: -56.4375991821,
		Timezone:  "America/Cuiaba",
	},
	"SWYR": {
		ICAO:      "SWYR",
		Name:      "Fazenda Nossa Senhora do Pillar Airport",
		City:      "Jauru",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1001,
		Latitude:  -15.3458328247,
		Longitude: -58.7952766418,
		Timezone:  "America/Cuiaba",
	},
	"SWYT": {
		ICAO:      "SWYT",
		Name:      "Fazenda Formosa Airport",
		City:      "Novo Acordo",
		State:     "Tocantins",
		Country:   "BR",
		Elevation: 1132,
		Latitude:  -10.0130558014,
		Longitude: -47.6816673279,
		Timezone:  "America/Araguaina",
	},
	"SWYU": {
		ICAO:      "SWYU",
		Name:      "Fazenda Entre Rios Airport",
		City:      "Aruana",
		State:     "Goias",
		Country:   "BR",
		Elevation: 781,
		Latitude:  -14.4433326721,
		Longitude: -50.8666687012,
		Timezone:  "America/Sao_Paulo",
	},
	"SWYV": {
		ICAO:      "SWYV",
		Name:      "Clube de Marte Ibira de Para-Quedismo Airport",
		City:      "Ibira",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1591,
		Latitude:  -21.0913887024,
		Longitude: -49.2241668701,
		Timezone:  "America/Sao_Paulo",
	},
	"SWYW": {
		ICAO:      "SWYW",
		Name:      "Fazenda Adriana Airport",
		City:      "Alto Garcas",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2241,
		Latitude:  -16.834444046,
		Longitude: -53.8352775574,
		Timezone:  "America/Cuiaba",
	},
	"SWYY": {
		ICAO:      "SWYY",
		Name:      "Fazenda Nova Viena Airport",
		City:      "Nova Xavantina",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 994,
		Latitude:  -14.8877782822,
		Longitude: -52.2769432068,
		Timezone:  "America/Cuiaba",
	},
	"SWZA": {
		ICAO:      "SWZA",
		Name:      "Agrosan Airport",
		City:      "Juara",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 984,
		Latitude:  -11.366944313,
		Longitude: -58.1172218323,
		Timezone:  "America/Cuiaba",
	},
	"SWZB": {
		ICAO:      "SWZB",
		Name:      "Fazenda Arara Azul Airport",
		City:      "Barao De Melgaco",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 459,
		Latitude:  -17.0019435883,
		Longitude: -56.4602775574,
		Timezone:  "America/Cuiaba",
	},
	"SWZC": {
		ICAO:      "SWZC",
		Name:      "Fazenda Corixo Airport",
		City:      "Porto Esperidiao",
		State:     "Santa-Cruz",
		Country:   "BR",
		Elevation: 492,
		Latitude:  -16.2458324432,
		Longitude: -58.894443512,
		Timezone:  "America/Cuiaba",
	},
	"SWZD": {
		ICAO:      "SWZD",
		Name:      "Fazenda Denusa Destilaria Nova Uniao Airport",
		City:      "Jandaia",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1699,
		Latitude:  -17.2558326721,
		Longitude: -50.1422233582,
		Timezone:  "America/Sao_Paulo",
	},
	"SWZE": {
		ICAO:      "SWZE",
		Name:      "Fazenda Tamboril Airport",
		City:      "Pontes E Lacerda",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1099,
		Latitude:  -15.0622215271,
		Longitude: -59.374168396,
		Timezone:  "America/Cuiaba",
	},
	"SWZF": {
		ICAO:      "SWZF",
		Name:      "Fazenda Araguari Airport",
		City:      "Araputanga",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 902,
		Latitude:  -15.4947223663,
		Longitude: -58.3994445801,
		Timezone:  "America/Cuiaba",
	},
	"SWZG": {
		ICAO:      "SWZG",
		Name:      "Fazenda Lagoa Encantada Airport",
		City:      "Porto Esperidiao",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 800,
		Latitude:  -16.0191669464,
		Longitude: -58.8875007629,
		Timezone:  "America/Cuiaba",
	},
	"SWZH": {
		ICAO:      "SWZH",
		Name:      "Fazenda Cacapava Airport",
		City:      "Acreuna",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1918,
		Latitude:  -17.5750007629,
		Longitude: -50.1749992371,
		Timezone:  "America/Sao_Paulo",
	},
	"SWZJ": {
		ICAO:      "SWZJ",
		Name:      "Fazenda Sao Joao Airport",
		City:      "Crixas",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1033,
		Latitude:  -14.5472221375,
		Longitude: -50.6536102295,
		Timezone:  "America/Sao_Paulo",
	},
	"SWZL": {
		ICAO:      "SWZL",
		Name:      "Fazenda Americana Airport",
		City:      "Itambacuri",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 1106,
		Latitude:  -18.146944046,
		Longitude: -41.7144432068,
		Timezone:  "America/Sao_Paulo",
	},
	"SWZM": {
		ICAO:      "SWZM",
		Name:      "Usina Sao Simao Airport",
		City:      "Sao Simao",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1716,
		Latitude:  -18.9864997864,
		Longitude: -50.5635986328,
		Timezone:  "America/Sao_Paulo",
	},
	"SWZN": {
		ICAO:      "SWZN",
		Name:      "Fazenda Frances Airport",
		City:      "Acreuna",
		State:     "Goias",
		Country:   "BR",
		Elevation: 2210,
		Latitude:  -17.322221756,
		Longitude: -50.4258346558,
		Timezone:  "America/Sao_Paulo",
	},
	"SWZO": {
		ICAO:      "SWZO",
		Name:      "Fazenda Lagoa Funda Airport",
		City:      "Campo Verde",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 2165,
		Latitude:  -15.3427782059,
		Longitude: -55.136390686,
		Timezone:  "America/Cuiaba",
	},
	"SWZP": {
		ICAO:      "SWZP",
		Name:      "Fazenda Vale Rico Airport",
		City:      "Guiratinga",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1050,
		Latitude:  -16.3858337402,
		Longitude: -54.2022209167,
		Timezone:  "America/Cuiaba",
	},
	"SWZQ": {
		ICAO:      "SWZQ",
		Name:      "Fazenda Jussara Airport",
		City:      "Jussara",
		State:     "Goias",
		Country:   "BR",
		Elevation: 1344,
		Latitude:  -15.8452777863,
		Longitude: -50.0719451904,
		Timezone:  "America/Sao_Paulo",
	},
	"SWZR": {
		ICAO:      "SWZR",
		Name:      "Fazenda Santa Rita Airport",
		City:      "Itapirapua",
		State:     "Goias",
		Country:   "BR",
		Elevation: 965,
		Latitude:  -15.3838891983,
		Longitude: -51.0338897705,
		Timezone:  "America/Sao_Paulo",
	},
	"SWZS": {
		ICAO:      "SWZS",
		Name:      "Fazenda Nova Larga Airport",
		City:      "Caceres",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 787,
		Latitude:  -16.6583328247,
		Longitude: -57.6708335876,
		Timezone:  "America/Cuiaba",
	},
	"SWZT": {
		ICAO:      "SWZT",
		Name:      "Fazenda do Brejo Airport",
		City:      "Paraopeba",
		State:     "Minas-Gerais",
		Country:   "BR",
		Elevation: 2451,
		Latitude:  -19.2549991608,
		Longitude: -44.4733314514,
		Timezone:  "America/Sao_Paulo",
	},
	"SWZU": {
		ICAO:      "SWZU",
		Name:      "Fazenda Rodoserv I Airport",
		City:      "Ivinhema",
		State:     "Sao-Paulo",
		Country:   "BR",
		Elevation: 1001,
		Latitude:  -22.4727783203,
		Longitude: -52.5958328247,
		Timezone:  "America/Sao_Paulo",
	},
	"SWZV": {
		ICAO:      "SWZV",
		Name:      "Fazenda Rodoserv III Airport",
		City:      "Bela Vista",
		State:     "Mato-Grosso-do-Sul",
		Country:   "BR",
		Elevation: 1001,
		Latitude:  -21.8622226715,
		Longitude: -56.7738876343,
		Timezone:  "America/Campo_Grande",
	},
	"SWZW": {
		ICAO:      "SWZW",
		Name:      "Fazenda Santa Fe do Xingu Airport",
		City:      "Sao Jose Do Xingu",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 1036,
		Latitude:  -10.3211107254,
		Longitude: -52.2105560303,
		Timezone:  "America/Cuiaba",
	},
	"SWZY": {
		ICAO:      "SWZY",
		Name:      "Fazenda Sete Estrelas Airport",
		City:      "Brasnorte",
		State:     "Mato-Grosso",
		Country:   "BR",
		Elevation: 984,
		Latitude:  -11.5772218704,
		Longitude: -58.2341651917,
		Timezone:  "America/Cuiaba",
	},
	"SYAH": {
		ICAO:      "SYAH",
		IATA:      "AHL",
		Name:      "Aishalton Airport",
		City:      "Aishalton",
		State:     "Upper-Takutu-Upper-Essequibo",
		Country:   "GY",
		Elevation: 587,
		Latitude:  2.4865300655,
		Longitude: -59.3134002686,
		Timezone:  "America/Guyana",
	},
	"SYAN": {
		ICAO:      "SYAN",
		IATA:      "NAI",
		Name:      "Annai Airport",
		City:      "Annai",
		State:     "Upper-Takutu-Upper-Essequibo",
		Country:   "GY",
		Elevation: 301,
		Latitude:  3.9594399929,
		Longitude: -59.1241989136,
		Timezone:  "America/Guyana",
	},
	"SYAP": {
		ICAO:      "SYAP",
		Name:      "Apoteri Airport",
		City:      "Apoteri",
		State:     "Upper-Takutu-Upper-Essequibo",
		Country:   "GY",
		Elevation: 301,
		Latitude:  4.013879776,
		Longitude: -58.6049995422,
		Timezone:  "America/Guyana",
	},
	"SYAW": {
		ICAO:      "SYAW",
		Name:      "Awaruwaunau Airport",
		City:      "Awaruwaunau",
		State:     "Upper-Takutu-Upper-Essequibo",
		Country:   "GY",
		Elevation: 797,
		Latitude:  2.6500000954,
		Longitude: -59.2000007629,
		Timezone:  "America/Guyana",
	},
	"SYBE": {
		ICAO:      "SYBE",
		IATA:      "BCG",
		Name:      "Bemichi Airport",
		City:      "Bemichi",
		State:     "Barima-Waini",
		Country:   "GY",
		Elevation: 94,
		Latitude:  7.6999998093,
		Longitude: -59.1666603088,
		Timezone:  "America/Guyana",
	},
	"SYBR": {
		ICAO:      "SYBR",
		IATA:      "BMJ",
		Name:      "Baramita Airport",
		City:      "Baramita",
		State:     "Barima-Waini",
		Country:   "GY",
		Elevation: 328,
		Latitude:  7.3701200485,
		Longitude: -60.4879989624,
		Timezone:  "America/Guyana",
	},
	"SYBT": {
		ICAO:      "SYBT",
		IATA:      "GFO",
		Name:      "Bartica A Airport",
		City:      "Bartica",
		State:     "Cuyuni-Mazaruni",
		Country:   "GY",
		Elevation: 3,
		Latitude:  6.3747701645,
		Longitude: -58.6380996704,
		Timezone:  "America/Guyana",
	},
	"SYCJ": {
		ICAO:      "SYCJ",
		IATA:      "GEO",
		Name:      "Cheddi Jagan International Airport",
		City:      "Georgetown",
		State:     "Demerara-Mahaica",
		Country:   "GY",
		Elevation: 95,
		Latitude:  6.4985499382,
		Longitude: -58.2541007996,
		Timezone:  "America/Guyana",
	},
	"SYEB": {
		ICAO:      "SYEB",
		Name:      "Ebini Airport",
		City:      "Ebini",
		State:     "Upper-Demerara-Berbice",
		Country:   "GY",
		Elevation: 147,
		Latitude:  5.617000103,
		Longitude: -57.766998291,
		Timezone:  "America/Guyana",
	},
	"SYGO": {
		ICAO:      "SYGO",
		IATA:      "OGL",
		Name:      "Ogle Airport",
		City:      "Ogle",
		State:     "Demerara-Mahaica",
		Country:   "GY",
		Elevation: 10,
		Latitude:  6.8062801361,
		Longitude: -58.1058998108,
		Timezone:  "America/Guyana",
	},
	"SYIB": {
		ICAO:      "SYIB",
		IATA:      "IMB",
		Name:      "Imbaimadai Airport",
		City:      "Imbaimadai",
		State:     "Cuyuni-Mazaruni",
		Country:   "GY",
		Elevation: 1646,
		Latitude:  5.7081098557,
		Longitude: -60.2942008972,
		Timezone:  "America/Guyana",
	},
	"SYKK": {
		ICAO:      "SYKK",
		Name:      "Kurukabaru Airport",
		City:      "Kurukabaru",
		Country:   "GY",
		Elevation: 3198,
		Latitude:  4.7170000076,
		Longitude: -59.983001709,
		Timezone:  "America/Guyana",
	},
	"SYKM": {
		ICAO:      "SYKM",
		IATA:      "KAR",
		Name:      "Kamarang Airport",
		City:      "Kamarang",
		State:     "Cuyuni-Mazaruni",
		Country:   "GY",
		Elevation: 1601,
		Latitude:  5.8653402328,
		Longitude: -60.614200592,
		Timezone:  "America/Guyana",
	},
	"SYKR": {
		ICAO:      "SYKR",
		IATA:      "KRM",
		Name:      "Karanambo Airport",
		City:      "Karanambo",
		State:     "Upper-Takutu-Upper-Essequibo",
		Country:   "GY",
		Elevation: 300,
		Latitude:  3.751940012,
		Longitude: -59.3097000122,
		Timezone:  "America/Guyana",
	},
	"SYKS": {
		ICAO:      "SYKS",
		IATA:      "KRG",
		Name:      "Karasabai Airport",
		City:      "Karasabai",
		State:     "Upper-Takutu-Upper-Essequibo",
		Country:   "GY",
		Elevation: 731,
		Latitude:  4.0333299637,
		Longitude: -59.5332984924,
		Timezone:  "America/Guyana",
	},
	"SYKT": {
		ICAO:      "SYKT",
		IATA:      "KTO",
		Name:      "Kato Airport",
		City:      "Kato",
		State:     "Potaro-Siparuni",
		Country:   "GY",
		Elevation: 2299,
		Latitude:  4.6491599083,
		Longitude: -59.8321990967,
		Timezone:  "America/Guyana",
	},
	"SYKW": {
		ICAO:      "SYKW",
		Name:      "Kwakwani Airport",
		City:      "Kwakwani",
		State:     "Upper-Demerara-Berbice",
		Country:   "GY",
		Elevation: 147,
		Latitude:  5.2833299637,
		Longitude: -58.0499992371,
		Timezone:  "America/Guyana",
	},
	"SYLD": {
		ICAO:      "SYLD",
		Name:      "Linden Airport",
		City:      "Linden",
		State:     "Upper-Demerara-Berbice",
		Country:   "GY",
		Elevation: 180,
		Latitude:  5.9659199715,
		Longitude: -58.2703018188,
		Timezone:  "America/Guyana",
	},
	"SYLP": {
		ICAO:      "SYLP",
		IATA:      "LUB",
		Name:      "Lumid Pau Airport",
		City:      "Lumid Pau",
		State:     "Upper-Takutu-Upper-Essequibo",
		Country:   "GY",
		Elevation: 656,
		Latitude:  2.3939299583,
		Longitude: -59.4410018921,
		Timezone:  "America/Guyana",
	},
	"SYLT": {
		ICAO:      "SYLT",
		IATA:      "LTM",
		Name:      "Lethem Airport",
		City:      "Lethem",
		State:     "Upper-Takutu-Upper-Essequibo",
		Country:   "GY",
		Elevation: 351,
		Latitude:  3.3727600574,
		Longitude: -59.7893981934,
		Timezone:  "America/Boa_Vista",
	},
	"SYMB": {
		ICAO:      "SYMB",
		IATA:      "USI",
		Name:      "Mabaruma Airport",
		City:      "Mabaruma",
		State:     "Barima-Waini",
		Country:   "GY",
		Elevation: 45,
		Latitude:  8.1999998093,
		Longitude: -59.7832984924,
		Timezone:  "America/Guyana",
	},
	"SYMD": {
		ICAO:      "SYMD",
		IATA:      "MHA",
		Name:      "Mahdia Airport",
		City:      "Mahdia",
		State:     "Potaro-Siparuni",
		Country:   "GY",
		Elevation: 300,
		Latitude:  5.277490139,
		Longitude: -59.1511001587,
		Timezone:  "America/Guyana",
	},
	"SYMM": {
		ICAO:      "SYMM",
		IATA:      "MYM",
		Name:      "Monkey Mountain Airport",
		City:      "Monkey Mountain",
		State:     "Potaro-Siparuni",
		Country:   "GY",
		Elevation: 1456,
		Latitude:  4.4833297729,
		Longitude: -59.6833000183,
		Timezone:  "America/Guyana",
	},
	"SYMN": {
		ICAO:      "SYMN",
		Name:      "Manari Airport",
		City:      "Manari",
		State:     "Upper-Takutu-Upper-Essequibo",
		Country:   "GY",
		Elevation: 300,
		Latitude:  3.4330000877,
		Longitude: -59.5670013428,
		Timezone:  "America/Guyana",
	},
	"SYMP": {
		ICAO:      "SYMP",
		Name:      "Mountain Point Airport",
		City:      "Mountain Point",
		State:     "Upper-Takutu-Upper-Essequibo",
		Country:   "GY",
		Elevation: 400,
		Latitude:  3,
		Longitude: -59.6329994202,
		Timezone:  "America/Guyana",
	},
	"SYMR": {
		ICAO:      "SYMR",
		IATA:      "MWJ",
		Name:      "Matthews Ridge Airport",
		City:      "Matthews Ridge",
		State:     "Barima-Waini",
		Country:   "GY",
		Elevation: 324,
		Latitude:  7.488112,
		Longitude: -60.184777,
		Timezone:  "America/Guyana",
	},
	"SYMW": {
		ICAO:      "SYMW",
		Name:      "Marurawana Airport",
		City:      "Marurawana",
		State:     "Upper-Takutu-Upper-Essequibo",
		Country:   "GY",
		Elevation: 1017,
		Latitude:  2.7297699451,
		Longitude: -59.1497001648,
		Timezone:  "America/Guyana",
	},
	"SYNA": {
		ICAO:      "SYNA",
		IATA:      "QSX",
		Name:      "New Amsterdam Airport",
		City:      "New Amsterdam",
		State:     "East-Berbice-Corentyne",
		Country:   "GY",
		Elevation: 3,
		Latitude:  6.244324,
		Longitude: -57.474172,
		Timezone:  "America/Guyana",
	},
	"SYOR": {
		ICAO:      "SYOR",
		IATA:      "ORJ",
		Name:      "Orinduik Airport",
		City:      "Orinduik",
		Country:   "GY",
		Elevation: 1797,
		Latitude:  4.7252697945,
		Longitude: -60.0349998474,
		Timezone:  "America/Boa_Vista",
	},
	"SYPR": {
		ICAO:      "SYPR",
		IATA:      "PRR",
		Name:      "Paruma Airport",
		City:      "Paruma",
		State:     "Cuyuni-Mazaruni",
		Country:   "GY",
		Elevation: 1765,
		Latitude:  5.8154501915,
		Longitude: -61.0554008484,
		Timezone:  "America/Guyana",
	},
	"SYWI": {
		ICAO:      "SYWI",
		Name:      "Wichabai Airport",
		City:      "Wichabai",
		State:     "Upper-Takutu-Upper-Essequibo",
		Country:   "GY",
		Elevation: 518,
		Latitude:  2.8666698933,
		Longitude: -59.5332984924,
		Timezone:  "America/Guyana",
	},
	"TA00": {
		ICAO:      "TA00",
		Name:      "Rafter P Airport",
		City:      "Sweetwater",
		State:     "Texas",
		Country:   "US",
		Elevation: 2190,
		Latitude:  32.3842010498,
		Longitude: -100.2789993286,
		Timezone:  "America/Chicago",
	},
	"TA01": {
		ICAO:      "TA01",
		Name:      "Phillips Farm Airport",
		City:      "Alvarado",
		Country:   "US",
		Elevation: 800,
		Latitude:  32.3418006897,
		Longitude: -97.2502975464,
		Timezone:  "America/Chicago",
	},
	"TA02": {
		ICAO:      "TA02",
		Name:      "Howard Field",
		City:      "Rhome",
		State:     "Texas",
		Country:   "US",
		Elevation: 960,
		Latitude:  33.1029014587,
		Longitude: -97.4642028809,
		Timezone:  "America/Chicago",
	},
	"TA03": {
		ICAO:      "TA03",
		Name:      "Jo Na Acres Airport",
		City:      "Brookshire",
		State:     "Texas",
		Country:   "US",
		Elevation: 160,
		Latitude:  29.758600235,
		Longitude: -95.9085998535,
		Timezone:  "America/Chicago",
	},
	"TA05": {
		ICAO:      "TA05",
		Name:      "Anxiety Aerodrome",
		City:      "Corsicana",
		State:     "Texas",
		Country:   "US",
		Elevation: 430,
		Latitude:  32.0377006531,
		Longitude: -96.5317001343,
		Timezone:  "America/Chicago",
	},
	"TA06": {
		ICAO:      "TA06",
		Name:      "Morris Manor Airport",
		City:      "Cumby",
		State:     "Texas",
		Country:   "US",
		Elevation: 610,
		Latitude:  33.1612014771,
		Longitude: -95.8058013916,
		Timezone:  "America/Chicago",
	},
	"TA07": {
		ICAO:      "TA07",
		Name:      "Jet Ag Inc Airport",
		City:      "Dayton",
		State:     "Texas",
		Country:   "US",
		Elevation: 70,
		Latitude:  30.0072002411,
		Longitude: -94.9669036865,
		Timezone:  "America/Chicago",
	},
	"TA08": {
		ICAO:      "TA08",
		Name:      "Flying M Airport",
		City:      "Floyd",
		State:     "Texas",
		Country:   "US",
		Elevation: 585,
		Latitude:  33.1496009827,
		Longitude: -96.2739028931,
		Timezone:  "America/Chicago",
	},
	"TA11": {
		ICAO:      "TA11",
		Name:      "Tsa Gliderport",
		City:      "Midlothian",
		State:     "Texas",
		Country:   "US",
		Elevation: 660,
		Latitude:  32.3834991455,
		Longitude: -97.0141983032,
		Timezone:  "America/Chicago",
	},
	"TA13": {
		ICAO:      "TA13",
		Name:      "Nebtex Land Co. Airport",
		City:      "Texline",
		State:     "Texas",
		Country:   "US",
		Elevation: 4373,
		Latitude:  36.256401062,
		Longitude: -102.8700027466,
		Timezone:  "America/Chicago",
	},
	"TA16": {
		ICAO:      "TA16",
		Name:      "Travis Field Airport",
		City:      "Spur",
		State:     "Texas",
		Country:   "US",
		Elevation: 2317,
		Latitude:  33.442864,
		Longitude: -100.854572,
		Timezone:  "America/Chicago",
	},
	"TA17": {
		ICAO:      "TA17",
		Name:      "Live Oak Ranch Airport",
		City:      "Brenham",
		State:     "Texas",
		Country:   "US",
		Elevation: 387,
		Latitude:  30.1872005463,
		Longitude: -96.4332962036,
		Timezone:  "America/Chicago",
	},
	"TA18": {
		ICAO:      "TA18",
		Name:      "Sunset Airport",
		City:      "Forney",
		State:     "Texas",
		Country:   "US",
		Elevation: 444,
		Latitude:  32.7070999146,
		Longitude: -96.4794006348,
		Timezone:  "America/Chicago",
	},
	"TA19": {
		ICAO:      "TA19",
		Name:      "Post Oak Airfield",
		City:      "Weatherford",
		State:     "Texas",
		Country:   "US",
		Elevation: 1020,
		Latitude:  32.6745986938,
		Longitude: -97.7835998535,
		Timezone:  "America/Chicago",
	},
	"TA21": {
		ICAO:      "TA21",
		Name:      "Windmill Hill Airport",
		City:      "Lewisville",
		State:     "Texas",
		Country:   "US",
		Elevation: 620,
		Latitude:  33.0181999207,
		Longitude: -97.1192016602,
		Timezone:  "America/Chicago",
	},
	"TA22": {
		ICAO:      "TA22",
		Name:      "Reward Ranch Airport",
		City:      "Bonham",
		State:     "Texas",
		Country:   "US",
		Elevation: 615,
		Latitude:  33.7672996521,
		Longitude: -96.12159729,
		Timezone:  "America/Chicago",
	},
	"TA23": {
		ICAO:      "TA23",
		Name:      "Morris Lazy K Ranch Airport",
		City:      "Marquez",
		State:     "Texas",
		Country:   "US",
		Elevation: 360,
		Latitude:  31.2612991333,
		Longitude: -96.2164001465,
		Timezone:  "America/Chicago",
	},
	"TA24": {
		ICAO:      "TA24",
		Name:      "Smoky Bend Airport",
		City:      "Mullin",
		State:     "Texas",
		Country:   "US",
		Elevation: 1320,
		Latitude:  31.4752006531,
		Longitude: -98.7114028931,
		Timezone:  "America/Chicago",
	},
	"TA25": {
		ICAO:      "TA25",
		Name:      "Cook Canyon Ranch Airport",
		City:      "Ranger",
		State:     "Texas",
		Country:   "US",
		Elevation: 1476,
		Latitude:  32.431734,
		Longitude: -98.594762,
		Timezone:  "America/Chicago",
	},
	"TA26": {
		ICAO:      "TA26",
		Name:      "Coyote Crossing Airport",
		City:      "Maypearl",
		State:     "Texas",
		Country:   "US",
		Elevation: 580,
		Latitude:  32.2963981628,
		Longitude: -97.0568008423,
		Timezone:  "America/Chicago",
	},
	"TA27": {
		ICAO:      "TA27",
		Name:      "Flying 'K' Airport",
		City:      "Briggs",
		State:     "Texas",
		Country:   "US",
		Elevation: 1132,
		Latitude:  30.8379993439,
		Longitude: -97.950302124,
		Timezone:  "America/Chicago",
	},
	"TA28": {
		ICAO:      "TA28",
		Name:      "Woods Nr 2 Airport",
		City:      "Brookshire",
		State:     "Texas",
		Country:   "US",
		Elevation: 160,
		Latitude:  29.7936000824,
		Longitude: -95.925201416,
		Timezone:  "America/Chicago",
	},
	"TA29": {
		ICAO:      "TA29",
		Name:      "Flying D Ranch Airport",
		City:      "Del Rio",
		State:     "Texas",
		Country:   "US",
		Elevation: 1630,
		Latitude:  29.9258003235,
		Longitude: -100.8399963379,
		Timezone:  "America/Chicago",
	},
	"TA30": {
		ICAO:      "TA30",
		Name:      "Old Forker Ultralightport",
		City:      "Sante Fe",
		State:     "Texas",
		Country:   "US",
		Elevation: 15,
		Latitude:  29.3187999725,
		Longitude: -95.0858001709,
		Timezone:  "America/Chicago",
	},
	"TA31": {
		ICAO:      "TA31",
		Name:      "Tularosa Airport",
		City:      "Brackettville",
		State:     "Texas",
		Country:   "US",
		Elevation: 1397,
		Latitude:  29.4305000305,
		Longitude: -100.2580032349,
		Timezone:  "America/Chicago",
	},
	"TA33": {
		ICAO:      "TA33",
		Name:      "Meyer Field",
		City:      "Rosharon",
		State:     "Texas",
		Country:   "US",
		Elevation: 65,
		Latitude:  29.3579998016,
		Longitude: -95.6352005005,
		Timezone:  "America/Chicago",
	},
	"TA34": {
		ICAO:      "TA34",
		Name:      "Flying G Airport",
		City:      "Crystal City",
		State:     "Texas",
		Country:   "US",
		Elevation: 675,
		Latitude:  28.7668991089,
		Longitude: -100.0830001831,
		Timezone:  "America/Chicago",
	},
	"TA35": {
		ICAO:      "TA35",
		Name:      "Faust Farm Airport",
		City:      "Brenham",
		State:     "Texas",
		Country:   "US",
		Elevation: 8,
		Latitude:  30.2726993561,
		Longitude: -96.3550033569,
		Timezone:  "America/Chicago",
	},
	"TA36": {
		ICAO:      "TA36",
		Name:      "Lempa Airport",
		City:      "Cuero",
		State:     "Texas",
		Country:   "US",
		Elevation: 247,
		Latitude:  29.0058002472,
		Longitude: -97.3116989136,
		Timezone:  "America/Chicago",
	},
	"TA42": {
		ICAO:      "TA42",
		Name:      "Fly-N-Fish Lodge Airport",
		City:      "Uncertain",
		State:     "Texas",
		Country:   "US",
		Elevation: 195,
		Latitude:  32.7128982544,
		Longitude: -94.1249008179,
		Timezone:  "America/Chicago",
	},
	"TA43": {
		ICAO:      "TA43",
		Name:      "Anderosa Airpark",
		City:      "Corrigan",
		State:     "Texas",
		Country:   "US",
		Elevation: 205,
		Latitude:  31.0475006104,
		Longitude: -94.8544006348,
		Timezone:  "America/Chicago",
	},
	"TA44": {
		ICAO:      "TA44",
		Name:      "Puesta del Sol Airport",
		City:      "Hebbronville",
		State:     "Texas",
		Country:   "US",
		Elevation: 448,
		Latitude:  27.07559967,
		Longitude: -98.60169983,
		Timezone:  "America/Chicago",
	},
	"TA46": {
		ICAO:      "TA46",
		Name:      "Baum Airport",
		City:      "Kaufman",
		State:     "Texas",
		Country:   "US",
		Elevation: 500,
		Latitude:  32.5625991821,
		Longitude: -96.4011001587,
		Timezone:  "America/Chicago",
	},
	"TA47": {
		ICAO:      "TA47",
		Name:      "Richards Airport",
		City:      "Krum",
		State:     "Texas",
		Country:   "US",
		Elevation: 785,
		Latitude:  33.301399231,
		Longitude: -97.2450027466,
		Timezone:  "America/Chicago",
	},
	"TA49": {
		ICAO:      "TA49",
		Name:      "Keno Field",
		City:      "Andice",
		State:     "Texas",
		Country:   "US",
		Elevation: 1000,
		Latitude:  30.7551002502,
		Longitude: -97.7998962402,
		Timezone:  "America/Chicago",
	},
	"TA50": {
		ICAO:      "TA50",
		Name:      "Cielo Dorado Estates Airport",
		City:      "Santa Teresa/El Paso",
		State:     "New-Mexico",
		Country:   "US",
		Elevation: 3750,
		Latitude:  31.9092998505,
		Longitude: -106.6409988403,
		Timezone:  "America/Denver",
	},
	"TA51": {
		ICAO:      "TA51",
		Name:      "Eagle Airport",
		City:      "Brock",
		State:     "Texas",
		Country:   "US",
		Elevation: 885,
		Latitude:  32.6804008484,
		Longitude: -97.9373016357,
		Timezone:  "America/Chicago",
	},
	"TA52": {
		ICAO:      "TA52",
		Name:      "Flying Bull Ranch Airport",
		City:      "Barksdale",
		State:     "Texas",
		Country:   "US",
		Elevation: 1614,
		Latitude:  29.8169002533,
		Longitude: -99.9962005615,
		Timezone:  "America/Chicago",
	},
	"TA53": {
		ICAO:      "TA53",
		Name:      "Rocky Top Ranch Airport",
		City:      "Blanco",
		State:     "Texas",
		Country:   "US",
		Elevation: 1500,
		Latitude:  30.1301002502,
		Longitude: -98.4091033936,
		Timezone:  "America/Chicago",
	},
	"TA55": {
		ICAO:      "TA55",
		Name:      "Del Valle Airport",
		City:      "Del Valle",
		State:     "Texas",
		Country:   "US",
		Elevation: 460,
		Latitude:  30.1676998138,
		Longitude: -97.5927963257,
		Timezone:  "America/Chicago",
	},
	"TA56": {
		ICAO:      "TA56",
		Name:      "Blue Mountain Airport",
		City:      "Fort Davis",
		State:     "Texas",
		Country:   "US",
		Elevation: 5000,
		Latitude:  30.5531997681,
		Longitude: -103.9540023804,
		Timezone:  "America/Chicago",
	},
	"TA57": {
		ICAO:      "TA57",
		Name:      "Texas Valley Air Field",
		City:      "Waco",
		State:     "Texas",
		Country:   "US",
		Elevation: 450,
		Latitude:  31.6238002777,
		Longitude: -96.9925003052,
		Timezone:  "America/Chicago",
	},
	"TA58": {
		ICAO:      "TA58",
		Name:      "Rancho Ecantado Airport",
		City:      "Cotulla",
		State:     "Texas",
		Country:   "US",
		Elevation: 550,
		Latitude:  28.3213996887,
		Longitude: -99.4499969482,
		Timezone:  "America/Chicago",
	},
	"TA59": {
		ICAO:      "TA59",
		Name:      "Flamingo Airfield",
		City:      "Jacksboro",
		State:     "Texas",
		Country:   "US",
		Elevation: 1160,
		Latitude:  33.0993003845,
		Longitude: -98.2894973755,
		Timezone:  "America/Chicago",
	},
	"TA60": {
		ICAO:      "TA60",
		Name:      "Hurn Airport",
		City:      "Waxahachie",
		State:     "Texas",
		Country:   "US",
		Elevation: 555,
		Latitude:  32.3942985535,
		Longitude: -96.7794036865,
		Timezone:  "America/Chicago",
	},
	"TA61": {
		ICAO:      "TA61",
		Name:      "Kay Ranch Airport",
		City:      "Winona",
		State:     "Texas",
		Country:   "US",
		Elevation: 315,
		Latitude:  32.4734992981,
		Longitude: -95.165802002,
		Timezone:  "America/Chicago",
	},
	"TA63": {
		ICAO:      "TA63",
		Name:      "Flattop Ridge Airport",
		City:      "Burnet",
		State:     "Texas",
		Country:   "US",
		Elevation: 1340,
		Latitude:  30.8229007721,
		Longitude: -98.1109008789,
		Timezone:  "America/Chicago",
	},
	"TA64": {
		ICAO:      "TA64",
		Name:      "Persimmon Gap Ranch Airport",
		City:      "Marathon",
		State:     "Texas",
		Country:   "US",
		Elevation: 2680,
		Latitude:  29.700917,
		Longitude: -103.15,
		Timezone:  "America/Chicago",
	},
	"TA65": {
		ICAO:      "TA65",
		Name:      "Sportsman's World Airport",
		City:      "Palo Pinto",
		State:     "Texas",
		Country:   "US",
		Elevation: 1142,
		Latitude:  32.8212013245,
		Longitude: -98.4786987305,
		Timezone:  "America/Chicago",
	},
	"TA66": {
		ICAO:      "TA66",
		Name:      "Freedom Springs Ranch Airport",
		City:      "Pipe Creek",
		State:     "Texas",
		Country:   "US",
		Elevation: 1400,
		Latitude:  29.714099884,
		Longitude: -98.8600006104,
		Timezone:  "America/Chicago",
	},
	"TA67": {
		ICAO:      "TA67",
		Name:      "Biggin Hill Airport",
		City:      "Shallowater",
		State:     "Texas",
		Country:   "US",
		Elevation: 3330,
		Latitude:  33.7484016418,
		Longitude: -102.0670013428,
		Timezone:  "America/Chicago",
	},
	"TA70": {
		ICAO:      "TA70",
		Name:      "W J E Airport",
		City:      "Tennessee Colony",
		State:     "Texas",
		Country:   "US",
		Elevation: 251,
		Latitude:  31.7835006714,
		Longitude: -95.8974990845,
		Timezone:  "America/Chicago",
	},
	"TA73": {
		ICAO:      "TA73",
		Name:      "Barclay's Roost Airport",
		City:      "Kerrville",
		State:     "Texas",
		Country:   "US",
		Elevation: 1680,
		Latitude:  29.9747009277,
		Longitude: -99.1684036255,
		Timezone:  "America/Chicago",
	},
	"TA75": {
		ICAO:      "TA75",
		Name:      "Cotton Patch Airport",
		City:      "Lexington",
		State:     "Texas",
		Country:   "US",
		Elevation: 430,
		Latitude:  30.3871994019,
		Longitude: -97.1050033569,
		Timezone:  "America/Chicago",
	},
	"TA77": {
		ICAO:      "TA77",
		Name:      "Cottonpatch Aerodrome",
		City:      "Frisco",
		State:     "Texas",
		Country:   "US",
		Elevation: 580,
		Latitude:  33.1492996216,
		Longitude: -96.8989028931,
		Timezone:  "America/Chicago",
	},
	"TA78": {
		ICAO:      "TA78",
		Name:      "Putty Ranch Airport",
		City:      "Hico",
		State:     "Texas",
		Country:   "US",
		Elevation: 1143,
		Latitude:  31.9724006653,
		Longitude: -98.0614013672,
		Timezone:  "America/Chicago",
	},
	"TA80": {
		ICAO:      "TA80",
		Name:      "U U Ranch Airport",
		City:      "Whitesboro",
		State:     "Texas",
		Country:   "US",
		Elevation: 740,
		Latitude:  33.8931999207,
		Longitude: -96.9281005859,
		Timezone:  "America/Chicago",
	},
	"TA81": {
		ICAO:      "TA81",
		Name:      "Tyra Ranch Airport",
		City:      "Del Rio",
		State:     "Texas",
		Country:   "US",
		Elevation: 1500,
		Latitude:  29.793800354,
		Longitude: -100.8789978027,
		Timezone:  "America/Chicago",
	},
	"TA82": {
		ICAO:      "TA82",
		Name:      "R & J Livestock County Airport",
		City:      "Batesville",
		State:     "Texas",
		Country:   "US",
		Elevation: 805,
		Latitude:  29.0244007111,
		Longitude: -99.5270004272,
		Timezone:  "America/Chicago",
	},
	"TA83": {
		ICAO:      "TA83",
		Name:      "Short Field",
		City:      "Mansfield",
		State:     "Texas",
		Country:   "US",
		Elevation: 650,
		Latitude:  32.5399017334,
		Longitude: -97.192199707,
		Timezone:  "America/Chicago",
	},
	"TA84": {
		ICAO:      "TA84",
		Name:      "Miller Ranch Airport",
		City:      "Carrizo Springs",
		State:     "Texas",
		Country:   "US",
		Elevation: 750,
		Latitude:  28.3549995422,
		Longitude: -99.9331970215,
		Timezone:  "America/Chicago",
	},
	"TA85": {
		ICAO:      "TA85",
		Name:      "Chambers Airport",
		City:      "Celeste",
		State:     "Texas",
		Country:   "US",
		Elevation: 692,
		Latitude:  33.3232002258,
		Longitude: -96.2322006226,
		Timezone:  "America/Chicago",
	},
	"TA86": {
		ICAO:      "TA86",
		Name:      "Dennis's Flying Farm Airport",
		City:      "Canton",
		State:     "Texas",
		Country:   "US",
		Elevation: 500,
		Latitude:  32.6059989929,
		Longitude: -95.9349975586,
		Timezone:  "America/Chicago",
	},
	"TA87": {
		ICAO:      "TA87",
		Name:      "Carter-Norman Airport",
		City:      "Boyd",
		State:     "Texas",
		Country:   "US",
		Elevation: 737,
		Latitude:  33.1073989868,
		Longitude: -97.5481033325,
		Timezone:  "America/Chicago",
	},
	"TA89": {
		ICAO:      "TA89",
		Name:      "Vaughan Ranch Airport",
		City:      "Mullin",
		Country:   "US",
		Elevation: 1310,
		Latitude:  31.5237998962,
		Longitude: -98.8003005981,
		Timezone:  "America/Chicago",
	},
	"TA90": {
		ICAO:      "TA90",
		Name:      "Green Acres Airport",
		City:      "Hockley",
		State:     "Texas",
		Country:   "US",
		Elevation: 181,
		Latitude:  29.9747009277,
		Longitude: -95.8180007935,
		Timezone:  "America/Chicago",
	},
	"TA93": {
		ICAO:      "TA93",
		Name:      "Lm Ranch Airport",
		City:      "Sherman",
		State:     "Texas",
		Country:   "US",
		Elevation: 770,
		Latitude:  33.6333007813,
		Longitude: -96.7335968018,
		Timezone:  "America/Chicago",
	},
	"TA97": {
		ICAO:      "TA97",
		Name:      "White Wing Ranch Airport",
		City:      "Orchard",
		State:     "Texas",
		Country:   "US",
		Elevation: 125,
		Latitude:  29.6375007629,
		Longitude: -95.9688034058,
		Timezone:  "America/Chicago",
	},
	"TAPA": {
		ICAO:      "TAPA",
		IATA:      "ANU",
		Name:      "V.C. Bird International Airport",
		City:      "St. George",
		State:     "Saint-George",
		Country:   "AG",
		Elevation: 62,
		Latitude:  17.1367,
		Longitude: -61.792702,
		Timezone:  "America/Antigua",
	},
	"TAPH": {
		ICAO:      "TAPH",
		IATA:      "BBQ",
		Name:      "Codrington Airport",
		City:      "Codrington",
		State:     "Barbuda",
		Country:   "AG",
		Elevation: 15,
		Latitude:  17.635799,
		Longitude: -61.828602,
		Timezone:  "America/Antigua",
	},
	"TAPT": {
		ICAO:      "TAPT",
		Name:      "Coco Point Lodge Airstrip",
		City:      "Coco Point",
		State:     "Barbuda",
		Country:   "AG",
		Elevation: 15,
		Latitude:  17.555599,
		Longitude: -61.765301,
		Timezone:  "America/Antigua",
	},
	"TBPB": {
		ICAO:      "TBPB",
		IATA:      "BGI",
		Name:      "Sir Grantley Adams International Airport",
		City:      "Bridgetown",
		State:     "Christ-Church",
		Country:   "BB",
		Elevation: 169,
		Latitude:  13.0746002197,
		Longitude: -59.4925003052,
		Timezone:  "America/Barbados",
	},
	"TCYT": {
		ICAO:      "TCYT",
		Name:      "Crystal Brook Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -17.4500007629,
		Longitude: 144.4499969482,
		Timezone:  "Australia/Brisbane",
	},
	"TDCF": {
		ICAO:      "TDCF",
		IATA:      "DCF",
		Name:      "Canefield Airport",
		City:      "Canefield",
		State:     "Saint-Paul",
		Country:   "DM",
		Elevation: 13,
		Latitude:  15.3367004395,
		Longitude: -61.39220047,
		Timezone:  "America/Dominica",
	},
	"TDPD": {
		ICAO:      "TDPD",
		IATA:      "DOM",
		Name:      "Melville Hall Airport",
		City:      "Marigot",
		State:     "Saint-Andrew",
		Country:   "DM",
		Elevation: 73,
		Latitude:  15.5469999313,
		Longitude: -61.2999992371,
		Timezone:  "America/Dominica",
	},
	"TE01": {
		ICAO:      "TE01",
		Name:      "Dillard Ranch Airport",
		City:      "Centerville",
		State:     "Texas",
		Country:   "US",
		Elevation: 225,
		Latitude:  31.3048992157,
		Longitude: -95.7966003418,
		Timezone:  "America/Chicago",
	},
	"TE02": {
		ICAO:      "TE02",
		Name:      "Aresti Aerodrome",
		City:      "Godley",
		State:     "Texas",
		Country:   "US",
		Elevation: 985,
		Latitude:  32.4706993103,
		Longitude: -97.5603027344,
		Timezone:  "America/Chicago",
	},
	"TE04": {
		ICAO:      "TE04",
		Name:      "Split B Ranch Airport",
		City:      "Canadian",
		State:     "Texas",
		Country:   "US",
		Elevation: 2580,
		Latitude:  35.7024993896,
		Longitude: -100.2850036621,
		Timezone:  "America/Chicago",
	},
	"TE06": {
		ICAO:      "TE06",
		Name:      "Casey Field",
		City:      "Leonard",
		State:     "Texas",
		Country:   "US",
		Elevation: 670,
		Latitude:  33.3792991638,
		Longitude: -96.3169021606,
		Timezone:  "America/Chicago",
	},
	"TE08": {
		ICAO:      "TE08",
		Name:      "Flying W Airport",
		City:      "Wingate",
		State:     "Texas",
		Country:   "US",
		Elevation: 2200,
		Latitude:  32.1496009827,
		Longitude: -100.1600036621,
		Timezone:  "America/Chicago",
	},
	"TE09": {
		ICAO:      "TE09",
		Name:      "Minard Pegasus Airport",
		City:      "Alvin",
		State:     "Texas",
		Country:   "US",
		Elevation: 25,
		Latitude:  29.3318996429,
		Longitude: -95.2760009766,
		Timezone:  "America/Chicago",
	},
	"TE10": {
		ICAO:      "TE10",
		Name:      "Pierce Airport",
		City:      "Henrietta",
		State:     "Texas",
		Country:   "US",
		Elevation: 885,
		Latitude:  33.8153991699,
		Longitude: -98.2281036377,
		Timezone:  "America/Chicago",
	},
	"TE12": {
		ICAO:      "TE12",
		Name:      "Cleveland Airport",
		City:      "La Vernia",
		State:     "Texas",
		Country:   "US",
		Elevation: 600,
		Latitude:  29.3001995087,
		Longitude: -98.1836013794,
		Timezone:  "America/Chicago",
	},
	"TE13": {
		ICAO:      "TE13",
		Name:      "Weiblen Airport",
		City:      "Castroville",
		State:     "Texas",
		Country:   "US",
		Elevation: 830,
		Latitude:  29.4001998901,
		Longitude: -98.8874969482,
		Timezone:  "America/Chicago",
	},
	"TE15": {
		ICAO:      "TE15",
		Name:      "M & M Land Company Airport",
		City:      "Menard",
		State:     "Texas",
		Country:   "US",
		Elevation: 1800,
		Latitude:  30.8910007477,
		Longitude: -99.632598877,
		Timezone:  "America/Chicago",
	},
	"TE16": {
		ICAO:      "TE16",
		Name:      "Cow Pasture Airport",
		City:      "Parker",
		State:     "Texas",
		Country:   "US",
		Elevation: 795,
		Latitude:  32.250425,
		Longitude: -97.297236,
		Timezone:  "America/Chicago",
	},
	"TE17": {
		ICAO:      "TE17",
		Name:      "Heathrow Airport",
		City:      "Robinson",
		State:     "Texas",
		Country:   "US",
		Elevation: 525,
		Latitude:  31.4391002655,
		Longitude: -97.1468963623,
		Timezone:  "America/Chicago",
	},
	"TE21": {
		ICAO:      "TE21",
		Name:      "Lobo Mountain Ranch Airport",
		City:      "Bandera",
		State:     "Texas",
		Country:   "US",
		Elevation: 1540,
		Latitude:  29.6513004303,
		Longitude: -99.0864028931,
		Timezone:  "America/Chicago",
	},
	"TE24": {
		ICAO:      "TE24",
		Name:      "Horseshoe Lake Airport",
		City:      "Sanger",
		State:     "Texas",
		Country:   "US",
		Elevation: 725,
		Latitude:  33.3931999207,
		Longitude: -97.2052993774,
		Timezone:  "America/Chicago",
	},
	"TE26": {
		ICAO:      "TE26",
		Name:      "Arrow 'S' Ranch Airport",
		City:      "Cambellton",
		State:     "Texas",
		Country:   "US",
		Elevation: 275,
		Latitude:  28.7835998535,
		Longitude: -98.3499984741,
		Timezone:  "America/Chicago",
	},
	"TE27": {
		ICAO:      "TE27",
		Name:      "Harrison Piloncillo Ranch Airport",
		City:      "Catarina",
		State:     "Texas",
		Country:   "US",
		Elevation: 520,
		Latitude:  28.2544002533,
		Longitude: -99.5991973877,
		Timezone:  "America/Chicago",
	},
	"TE29": {
		ICAO:      "TE29",
		Name:      "La Esperanza Ranch Airport",
		City:      "Encinal",
		State:     "Texas",
		Country:   "US",
		Elevation: 750,
		Latitude:  28.0730991364,
		Longitude: -99.5742034912,
		Timezone:  "America/Chicago",
	},
	"TE32": {
		ICAO:      "TE32",
		Name:      "Rancho Blanco Airport",
		City:      "Laredo",
		State:     "Texas",
		Country:   "US",
		Elevation: 385,
		Latitude:  27.308599472,
		Longitude: -99.4813995361,
		Timezone:  "America/Chicago",
	},
	"TE33": {
		ICAO:      "TE33",
		Name:      "Eagles Nest Gliderport",
		City:      "Midland",
		State:     "Texas",
		Country:   "US",
		Elevation: 2985,
		Latitude:  31.6835002899,
		Longitude: -102.1839981079,
		Timezone:  "America/Chicago",
	},
	"TE34": {
		ICAO:      "TE34",
		Name:      "Reb Folbre's Place Airport",
		City:      "Millsap",
		State:     "Texas",
		Country:   "US",
		Elevation: 940,
		Latitude:  32.7625999451,
		Longitude: -97.9728012085,
		Timezone:  "America/Chicago",
	},
	"TE36": {
		ICAO:      "TE36",
		Name:      "Gerum Farm Airport",
		City:      "Shiner",
		State:     "Texas",
		Country:   "US",
		Elevation: 425,
		Latitude:  29.3918991089,
		Longitude: -97.1847000122,
		Timezone:  "America/Chicago",
	},
	"TE37": {
		ICAO:      "TE37",
		Name:      "Canyon Ranch Airport",
		City:      "Sonora",
		State:     "Texas",
		Country:   "US",
		Elevation: 2306,
		Latitude:  30.2691001892,
		Longitude: -100.4509963989,
		Timezone:  "America/Chicago",
	},
	"TE38": {
		ICAO:      "TE38",
		Name:      "Loghouse /Stol/ STOLport",
		City:      "Goodrich",
		State:     "Texas",
		Country:   "US",
		Elevation: 184,
		Latitude:  30.5926990509,
		Longitude: -94.9094009399,
		Timezone:  "America/Chicago",
	},
	"TE39": {
		ICAO:      "TE39",
		Name:      "Bucker Field",
		City:      "Era",
		State:     "Texas",
		Country:   "US",
		Elevation: 877,
		Latitude:  33.4431991577,
		Longitude: -97.273399353,
		Timezone:  "America/Chicago",
	},
	"TE40": {
		ICAO:      "TE40",
		Name:      "Caselman Ranch Airport",
		City:      "Breckenridge",
		State:     "Texas",
		Country:   "US",
		Elevation: 1387,
		Latitude:  32.6748008728,
		Longitude: -98.8262023926,
		Timezone:  "America/Chicago",
	},
	"TE45": {
		ICAO:      "TE45",
		Name:      "Buffalo Chips Airpark",
		City:      "Joshua",
		State:     "Texas",
		Country:   "US",
		Elevation: 960,
		Latitude:  32.4698982239,
		Longitude: -97.4653015137,
		Timezone:  "America/Chicago",
	},
	"TE47": {
		ICAO:      "TE47",
		Name:      "Cross Wind Acres Airport",
		City:      "Rio Vista",
		State:     "Texas",
		Country:   "US",
		Elevation: 760,
		Latitude:  32.2089996338,
		Longitude: -97.3178024292,
		Timezone:  "America/Chicago",
	},
	"TE48": {
		ICAO:      "TE48",
		Name:      "King's Ranch Airport",
		City:      "Lampasas",
		State:     "Texas",
		Country:   "US",
		Elevation: 1025,
		Latitude:  31.0449008942,
		Longitude: -98.0428009033,
		Timezone:  "America/Chicago",
	},
	"TE50": {
		ICAO:      "TE50",
		Name:      "Hirok Airport",
		City:      "Milford",
		State:     "Texas",
		Country:   "US",
		Elevation: 700,
		Latitude:  32.1731987,
		Longitude: -96.9661026001,
		Timezone:  "America/Chicago",
	},
	"TE51": {
		ICAO:      "TE51",
		Name:      "Lackorn Airport",
		City:      "Seguin",
		State:     "Texas",
		Country:   "US",
		Elevation: 420,
		Latitude:  29.539100647,
		Longitude: -97.8349990845,
		Timezone:  "America/Chicago",
	},
	"TE52": {
		ICAO:      "TE52",
		Name:      "Chigger Field",
		City:      "Cresson",
		State:     "Texas",
		Country:   "US",
		Elevation: 950,
		Latitude:  32.5359992981,
		Longitude: -97.5886001587,
		Timezone:  "America/Chicago",
	},
	"TE55": {
		ICAO:      "TE55",
		Name:      "Flyin Armadillo Airport",
		City:      "Jacksboro",
		State:     "Texas",
		Country:   "US",
		Elevation: 995,
		Latitude:  33.4216995239,
		Longitude: -98.1905975342,
		Timezone:  "America/Chicago",
	},
	"TE57": {
		ICAO:      "TE57",
		Name:      "Haass Field",
		City:      "Hondo",
		State:     "Texas",
		Country:   "US",
		Elevation: 945,
		Latitude:  29.4447994232,
		Longitude: -99.1228027344,
		Timezone:  "America/Chicago",
	},
	"TE58": {
		ICAO:      "TE58",
		Name:      "Bird Dog Airfield",
		City:      "Krum",
		State:     "Texas",
		Country:   "US",
		Elevation: 960,
		Latitude:  33.3064002991,
		Longitude: -97.3416976929,
		Timezone:  "America/Chicago",
	},
	"TE62": {
		ICAO:      "TE62",
		Name:      "High Man Tower Airstrip",
		City:      "Robstown",
		State:     "Texas",
		Country:   "US",
		Elevation: 90,
		Latitude:  27.8542995453,
		Longitude: -97.6822967529,
		Timezone:  "America/Chicago",
	},
	"TE63": {
		ICAO:      "TE63",
		Name:      "Arrowhead Ranch Airport",
		City:      "Tilden",
		State:     "Texas",
		Country:   "US",
		Elevation: 296,
		Latitude:  28.246099472,
		Longitude: -98.5528030396,
		Timezone:  "America/Chicago",
	},
	"TE67": {
		ICAO:      "TE67",
		Name:      "Cibolo Sea-Willo Airpark",
		City:      "New Berlin",
		State:     "Texas",
		Country:   "US",
		Elevation: 517,
		Latitude:  29.4291000366,
		Longitude: -98.1316986084,
		Timezone:  "America/Chicago",
	},
	"TE68": {
		ICAO:      "TE68",
		Name:      "Nuggs Flying M Airport",
		City:      "Pottsboro",
		State:     "Texas",
		Country:   "US",
		Elevation: 725,
		Latitude:  33.7440986633,
		Longitude: -96.6598968506,
		Timezone:  "America/Chicago",
	},
	"TE70": {
		ICAO:      "TE70",
		Name:      "Palmer Field",
		City:      "Ponder",
		State:     "Texas",
		Country:   "US",
		Elevation: 800,
		Latitude:  33.1986999512,
		Longitude: -97.3170013428,
		Timezone:  "America/Chicago",
	},
	"TE71": {
		ICAO:      "TE71",
		Name:      "Ghsa-Wallis Glideport Gliderport",
		City:      "Wallis",
		State:     "Texas",
		Country:   "US",
		Elevation: 127,
		Latitude:  29.6110992432,
		Longitude: -96.0279006958,
		Timezone:  "America/Chicago",
	},
	"TE72": {
		ICAO:      "TE72",
		Name:      "Haven Field",
		City:      "Waxahachie",
		State:     "Texas",
		Country:   "US",
		Elevation: 595,
		Latitude:  32.3270988464,
		Longitude: -96.8402023315,
		Timezone:  "America/Chicago",
	},
	"TE73": {
		ICAO:      "TE73",
		Name:      "Griffith Ranch Airport",
		City:      "Winnie",
		State:     "Texas",
		Country:   "US",
		Elevation: 21,
		Latitude:  29.858499527,
		Longitude: -94.3541030884,
		Timezone:  "America/Chicago",
	},
	"TE74": {
		ICAO:      "TE74",
		Name:      "Bucker Field",
		City:      "Point",
		State:     "Texas",
		Country:   "US",
		Elevation: 485,
		Latitude:  32.9536018372,
		Longitude: -95.866897583,
		Timezone:  "America/Chicago",
	},
	"TE75": {
		ICAO:      "TE75",
		Name:      "Lexington Airfield",
		City:      "Lexington",
		State:     "Texas",
		Country:   "US",
		Elevation: 470,
		Latitude:  30.4125003815,
		Longitude: -96.9666976929,
		Timezone:  "America/Chicago",
	},
	"TE76": {
		ICAO:      "TE76",
		Name:      "Vance Field",
		City:      "Alvord",
		State:     "Texas",
		Country:   "US",
		Elevation: 925,
		Latitude:  33.3683013916,
		Longitude: -97.6821975708,
		Timezone:  "America/Chicago",
	},
	"TE77": {
		ICAO:      "TE77",
		Name:      "Clover Lake Farms Airport",
		City:      "Angleton",
		State:     "Texas",
		Country:   "US",
		Elevation: 38,
		Latitude:  29.2336997986,
		Longitude: -95.4766998291,
		Timezone:  "America/Chicago",
	},
	"TE78": {
		ICAO:      "TE78",
		Name:      "Fossil Creek Ranch Airport",
		City:      "Camp Wood",
		State:     "Texas",
		Country:   "US",
		Elevation: 1325,
		Latitude:  29.5687007904,
		Longitude: -99.9745025635,
		Timezone:  "America/Chicago",
	},
	"TE81": {
		ICAO:      "TE81",
		Name:      "Smither Field",
		City:      "Denton",
		State:     "Texas",
		Country:   "US",
		Elevation: 655,
		Latitude:  33.3097000122,
		Longitude: -97.0927963257,
		Timezone:  "America/Chicago",
	},
	"TE83": {
		ICAO:      "TE83",
		Name:      "Westwind Ranch Airport",
		City:      "La Pryor",
		State:     "Texas",
		Country:   "US",
		Elevation: 686,
		Latitude:  28.9207992554,
		Longitude: -99.7561035156,
		Timezone:  "America/Chicago",
	},
	"TE84": {
		ICAO:      "TE84",
		Name:      "Mockingbird Hill Airport",
		City:      "Mc Gregor",
		State:     "Texas",
		Country:   "US",
		Elevation: 810,
		Latitude:  31.4652004242,
		Longitude: -97.4753036499,
		Timezone:  "America/Chicago",
	},
	"TE85": {
		ICAO:      "TE85",
		Name:      "Marmack Airport",
		City:      "Montgomery",
		State:     "Texas",
		Country:   "US",
		Elevation: 295,
		Latitude:  30.3421993256,
		Longitude: -95.6613006592,
		Timezone:  "America/Chicago",
	},
	"TE86": {
		ICAO:      "TE86",
		Name:      "Heritage Airfield",
		City:      "New Berlin",
		State:     "Texas",
		Country:   "US",
		Elevation: 555,
		Latitude:  29.4444007874,
		Longitude: -98.1128005981,
		Timezone:  "America/Chicago",
	},
	"TE87": {
		ICAO:      "TE87",
		Name:      "Magee Airport",
		City:      "Portland",
		State:     "Texas",
		Country:   "US",
		Elevation: 35,
		Latitude:  27.9158000946,
		Longitude: -97.3202972412,
		Timezone:  "America/Chicago",
	},
	"TE88": {
		ICAO:      "TE88",
		Name:      "Bb Airpark",
		City:      "Rosharon",
		State:     "Texas",
		Country:   "US",
		Elevation: 50,
		Latitude:  29.3568992615,
		Longitude: -95.4593963623,
		Timezone:  "America/Chicago",
	},
	"TE89": {
		ICAO:      "TE89",
		Name:      "Verhalen Airport",
		City:      "Scottsville",
		State:     "Texas",
		Country:   "US",
		Elevation: 415,
		Latitude:  32.5377006531,
		Longitude: -94.2360992432,
		Timezone:  "America/Chicago",
	},
	"TE90": {
		ICAO:      "TE90",
		Name:      "Flying L Airport",
		City:      "Bandera",
		State:     "Texas",
		Country:   "US",
		Elevation: 1380,
		Latitude:  29.7588996887,
		Longitude: -99.1524963379,
		Timezone:  "America/Chicago",
	},
	"TE91": {
		ICAO:      "TE91",
		Name:      "Harrison Farm Airport",
		City:      "Troup",
		State:     "Texas",
		Country:   "US",
		Elevation: 445,
		Latitude:  32.0392990112,
		Longitude: -95.0828018188,
		Timezone:  "America/Chicago",
	},
	"TE92": {
		ICAO:      "TE92",
		Name:      "Wales Air Field",
		City:      "Waco",
		State:     "Texas",
		Country:   "US",
		Elevation: 640,
		Latitude:  31.5944004059,
		Longitude: -97.3709030151,
		Timezone:  "America/Chicago",
	},
	"TE95": {
		ICAO:      "TE95",
		Name:      "Deiterich Ranch Airport",
		City:      "Bertram",
		State:     "Texas",
		Country:   "US",
		Elevation: 1385,
		Latitude:  30.7532997131,
		Longitude: -98.146697998,
		Timezone:  "America/Chicago",
	},
	"TE96": {
		ICAO:      "TE96",
		Name:      "Crosswinds Airfield",
		City:      "Coupland",
		State:     "Texas",
		Country:   "US",
		Elevation: 543,
		Latitude:  30.4332008362,
		Longitude: -97.4124984741,
		Timezone:  "America/Chicago",
	},
	"TE97": {
		ICAO:      "TE97",
		Name:      "Cameron Ranch Airport",
		City:      "Fredericksburg",
		State:     "Texas",
		Country:   "US",
		Elevation: 1760,
		Latitude:  30.1877994537,
		Longitude: -98.9582977295,
		Timezone:  "America/Chicago",
	},
	"TE99": {
		ICAO:      "TE99",
		Name:      "Barnstormer Airport",
		City:      "Peoria",
		State:     "Texas",
		Country:   "US",
		Elevation: 625,
		Latitude:  32.0214996338,
		Longitude: -97.2537002563,
		Timezone:  "America/Chicago",
	},
	"TFFA": {
		ICAO:      "TFFA",
		IATA:      "DSD",
		Name:      "La Desirade Airport",
		City:      "Grande Anse",
		State:     "Guadeloupe",
		Country:   "GP",
		Elevation: 10,
		Latitude:  16.2968997955,
		Longitude: -61.084400177,
		Timezone:  "America/Guadeloupe",
	},
	"TFFB": {
		ICAO:      "TFFB",
		IATA:      "BBR",
		Name:      "Baillif Airport",
		City:      "Basse Terre",
		State:     "Guadeloupe",
		Country:   "GP",
		Elevation: 59,
		Latitude:  16.013299942,
		Longitude: -61.7421989441,
		Timezone:  "America/Guadeloupe",
	},
	"TFFC": {
		ICAO:      "TFFC",
		IATA:      "SFC",
		Name:      "St-Francois Airport",
		City:      "St-Francois",
		State:     "Guadeloupe",
		Country:   "GP",
		Elevation: 10,
		Latitude:  16.2577991486,
		Longitude: -61.2625007629,
		Timezone:  "America/Guadeloupe",
	},
	"TFFF": {
		ICAO:      "TFFF",
		IATA:      "FDF",
		Name:      "Martinique Aime Cesaire International Airport",
		City:      "Fort-de-France",
		State:     "Martinique",
		Country:   "MQ",
		Elevation: 16,
		Latitude:  14.5909996033,
		Longitude: -61.003200531,
		Timezone:  "America/Martinique",
	},
	"TFFG": {
		ICAO:      "TFFG",
		IATA:      "SFG",
		Name:      "L'Esperance Airport",
		City:      "Grand Case",
		Country:   "MF",
		Elevation: 7,
		Latitude:  18.099899292,
		Longitude: -63.0471992493,
		Timezone:  "America/Lower_Princes",
	},
	"TFFJ": {
		ICAO:      "TFFJ",
		IATA:      "SBH",
		Name:      "Gustaf III Airport",
		City:      "Gustavia",
		Country:   "BL",
		Elevation: 49,
		Latitude:  17.9043998718,
		Longitude: -62.8436012268,
		Timezone:  "America/St_Barthelemy",
	},
	"TFFM": {
		ICAO:      "TFFM",
		IATA:      "GBJ",
		Name:      "Les Bases Airport",
		City:      "Grand Bourg",
		State:     "Guadeloupe",
		Country:   "GP",
		Elevation: 16,
		Latitude:  15.8687000275,
		Longitude: -61.2700004578,
		Timezone:  "America/Guadeloupe",
	},
	"TFFR": {
		ICAO:      "TFFR",
		IATA:      "PTP",
		Name:      "Pointe-a-Pitre Le Raizet",
		City:      "Pointe-a-Pitre Le Raizet",
		State:     "Guadeloupe",
		Country:   "GP",
		Elevation: 36,
		Latitude:  16.2653007507,
		Longitude: -61.5317993164,
		Timezone:  "America/Guadeloupe",
	},
	"TFFS": {
		ICAO:      "TFFS",
		IATA:      "LSS",
		Name:      "Terre-de-Haut Airport",
		City:      "Les Saintes",
		State:     "Guadeloupe",
		Country:   "GP",
		Elevation: 46,
		Latitude:  15.86439991,
		Longitude: -61.5806007385,
		Timezone:  "America/Guadeloupe",
	},
	"TGPY": {
		ICAO:      "TGPY",
		IATA:      "GND",
		Name:      "Point Salines International Airport",
		City:      "Saint George's",
		State:     "Saint-George",
		Country:   "GD",
		Elevation: 41,
		Latitude:  12.0041999817,
		Longitude: -61.7862014771,
		Timezone:  "America/Grenada",
	},
	"TGPZ": {
		ICAO:      "TGPZ",
		IATA:      "CRU",
		Name:      "Lauriston Airport",
		City:      "Carriacou Island",
		Country:   "GD",
		Elevation: 5,
		Latitude:  12.476099968,
		Longitude: -61.4728012085,
		Timezone:  "America/Grenada",
	},
	"TIST": {
		ICAO:      "TIST",
		IATA:      "STT",
		Name:      "Cyril E. King Airport",
		City:      "Charlotte Amalie",
		State:     "Saint-Thomas-Island",
		Country:   "VI",
		Elevation: 23,
		Latitude:  18.3372993469,
		Longitude: -64.9733963013,
		Timezone:  "America/St_Thomas",
	},
	"TISX": {
		ICAO:      "TISX",
		IATA:      "STX",
		Name:      "Henry E Rohlsen Airport",
		City:      "Christiansted",
		State:     "Saint-Croix-Island",
		Country:   "VI",
		Elevation: 74,
		Latitude:  17.7019004822,
		Longitude: -64.7985992432,
		Timezone:  "America/St_Thomas",
	},
	"TJAB": {
		ICAO:      "TJAB",
		IATA:      "ARE",
		Name:      "Antonio Nery Juarbe Pol Airport",
		City:      "Arecibo",
		State:     "Arecibo",
		Country:   "PR",
		Elevation: 23,
		Latitude:  18.4500007629,
		Longitude: -66.6753005981,
		Timezone:  "America/Puerto_Rico",
	},
	"TJBQ": {
		ICAO:      "TJBQ",
		IATA:      "BQN",
		Name:      "Rafael Hernandez Airport",
		City:      "Aguadilla",
		State:     "Aguadilla",
		Country:   "PR",
		Elevation: 237,
		Latitude:  18.4948997498,
		Longitude: -67.1294021606,
		Timezone:  "America/Puerto_Rico",
	},
	"TJCP": {
		ICAO:      "TJCP",
		IATA:      "CPX",
		Name:      "Benjamin Rivera Noriega Airport",
		City:      "Culebra Island",
		State:     "Culebra",
		Country:   "PR",
		Elevation: 49,
		Latitude:  18.313289,
		Longitude: -65.304324,
		Timezone:  "America/Puerto_Rico",
	},
	"TJFA": {
		ICAO:      "TJFA",
		IATA:      "FAJ",
		Name:      "Diego Jimenez Torres Airport",
		City:      "Fajardo",
		State:     "Fajardo",
		Country:   "PR",
		Elevation: 64,
		Latitude:  18.3089008331,
		Longitude: -65.6619033813,
		Timezone:  "America/Puerto_Rico",
	},
	"TJIG": {
		ICAO:      "TJIG",
		IATA:      "SIG",
		Name:      "Fernando Luis Ribas Dominicci Airport",
		City:      "San Juan",
		State:     "San-Juan",
		Country:   "PR",
		Elevation: 10,
		Latitude:  18.4568004608,
		Longitude: -66.0980987549,
		Timezone:  "America/Puerto_Rico",
	},
	"TJMZ": {
		ICAO:      "TJMZ",
		IATA:      "MAZ",
		Name:      "Eugenio Maria De Hostos Airport",
		City:      "Mayaguez",
		State:     "Mayaguez",
		Country:   "PR",
		Elevation: 28,
		Latitude:  18.2556991577,
		Longitude: -67.1484985352,
		Timezone:  "America/Puerto_Rico",
	},
	"TJPS": {
		ICAO:      "TJPS",
		IATA:      "PSE",
		Name:      "Mercedita Airport",
		City:      "Ponce",
		State:     "Ponce",
		Country:   "PR",
		Elevation: 29,
		Latitude:  18.0083007813,
		Longitude: -66.56300354,
		Timezone:  "America/Puerto_Rico",
	},
	"TJRV": {
		ICAO:      "TJRV",
		IATA:      "NRR",
		Name:      "Jose Aponte de la Torre Airport",
		City:      "Ceiba",
		State:     "Ceiba",
		Country:   "PR",
		Elevation: 38,
		Latitude:  18.245300293,
		Longitude: -65.6434020996,
		Timezone:  "America/Puerto_Rico",
	},
	"TJSJ": {
		ICAO:      "TJSJ",
		IATA:      "SJU",
		Name:      "Luis Munoz Marin International Airport",
		City:      "San Juan",
		State:     "Carolina",
		Country:   "PR",
		Elevation: 9,
		Latitude:  18.4393997192,
		Longitude: -66.0018005371,
		Timezone:  "America/Puerto_Rico",
	},
	"TJVQ": {
		ICAO:      "TJVQ",
		IATA:      "VQS",
		Name:      "Antonio Rivera Rodriguez Airport",
		City:      "Vieques Island",
		State:     "Vieques",
		Country:   "PR",
		Elevation: 49,
		Latitude:  18.1347999573,
		Longitude: -65.493598938,
		Timezone:  "America/Puerto_Rico",
	},
	"TKPK": {
		ICAO:      "TKPK",
		IATA:      "SKB",
		Name:      "Robert L. Bradshaw International Airport",
		City:      "Basseterre",
		State:     "Saint-George-Basseterre",
		Country:   "KN",
		Elevation: 170,
		Latitude:  17.3111991882,
		Longitude: -62.7187004089,
		Timezone:  "America/St_Kitts",
	},
	"TKPN": {
		ICAO:      "TKPN",
		IATA:      "NEV",
		Name:      "Vance W. Amory International Airport",
		City:      "Charlestown",
		State:     "Saint-James-Windwa",
		Country:   "KN",
		Elevation: 14,
		Latitude:  17.2056999207,
		Longitude: -62.5899009705,
		Timezone:  "America/St_Kitts",
	},
	"TLPC": {
		ICAO:      "TLPC",
		IATA:      "SLU",
		Name:      "George F. L. Charles Airport",
		City:      "Castries",
		State:     "Castries",
		Country:   "LC",
		Elevation: 22,
		Latitude:  14.0202,
		Longitude: -60.992901,
		Timezone:  "America/St_Lucia",
	},
	"TLPL": {
		ICAO:      "TLPL",
		IATA:      "UVF",
		Name:      "Hewanorra International Airport",
		City:      "Vieux Fort",
		State:     "Vieux-Fort",
		Country:   "LC",
		Elevation: 14,
		Latitude:  13.7332,
		Longitude: -60.952599,
		Timezone:  "America/St_Lucia",
	},
	"TN01": {
		ICAO:      "TN01",
		Name:      "Ray Airport",
		City:      "Rosemark",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 305,
		Latitude:  35.3473014832,
		Longitude: -89.8195037842,
		Timezone:  "America/Chicago",
	},
	"TN07": {
		ICAO:      "TN07",
		Name:      "Hawk Haven Airfield",
		City:      "Eagleville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 775,
		Latitude:  35.6871986389,
		Longitude: -86.6063995361,
		Timezone:  "America/Chicago",
	},
	"TN08": {
		ICAO:      "TN08",
		Name:      "Oliver Springs Inc Airport",
		City:      "Oliver Springs",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 790,
		Latitude:  36.0376014709,
		Longitude: -84.3069000244,
		Timezone:  "America/New_York",
	},
	"TN09": {
		ICAO:      "TN09",
		Name:      "Fergusons Flying Circus Airport",
		City:      "Philadelphia",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 910,
		Latitude:  35.7140007019,
		Longitude: -84.4110031128,
		Timezone:  "America/New_York",
	},
	"TN10": {
		ICAO:      "TN10",
		Name:      "Lake View Airport",
		City:      "Sharps Chapel",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1066,
		Latitude:  36.3390007019,
		Longitude: -83.7788009644,
		Timezone:  "America/New_York",
	},
	"TN11": {
		ICAO:      "TN11",
		Name:      "Cantwell Airport",
		City:      "Sneedville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1150,
		Latitude:  36.501499176,
		Longitude: -83.2555007935,
		Timezone:  "America/New_York",
	},
	"TN12": {
		ICAO:      "TN12",
		Name:      "Hudgin Air Airport",
		City:      "Lewisburg",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 725,
		Latitude:  35.4735984802,
		Longitude: -86.6800003052,
		Timezone:  "America/Chicago",
	},
	"TN13": {
		ICAO:      "TN13",
		Name:      "Barret Airport",
		City:      "Barretville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 240,
		Latitude:  35.3750991821,
		Longitude: -89.7667007446,
		Timezone:  "America/Chicago",
	},
	"TN14": {
		ICAO:      "TN14",
		Name:      "T-Top Airfield",
		City:      "Eagleville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 770,
		Latitude:  35.6897010803,
		Longitude: -86.6358032227,
		Timezone:  "America/Chicago",
	},
	"TN16": {
		ICAO:      "TN16",
		Name:      "Huntingdon Airport",
		City:      "Huntingdon",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 400,
		Latitude:  35.9944992065,
		Longitude: -88.4611968994,
		Timezone:  "America/Chicago",
	},
	"TN17": {
		ICAO:      "TN17",
		Name:      "Vintage Field",
		City:      "Sweetwater",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 990,
		Latitude:  35.5822982788,
		Longitude: -84.4879989624,
		Timezone:  "America/New_York",
	},
	"TN18": {
		ICAO:      "TN18",
		Name:      "Richardson Strip",
		City:      "Eads",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 340,
		Latitude:  35.2042999268,
		Longitude: -89.6667022705,
		Timezone:  "America/Chicago",
	},
	"TN20": {
		ICAO:      "TN20",
		Name:      "Seymour Air Park Inc. Airport",
		City:      "Sevierville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1000,
		Latitude:  35.8117980957,
		Longitude: -83.774597168,
		Timezone:  "America/New_York",
	},
	"TN21": {
		ICAO:      "TN21",
		Name:      "Palmer Village-Napier Lake STOLport",
		City:      "Hohenwald",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 790,
		Latitude:  35.4476013184,
		Longitude: -87.5061035156,
		Timezone:  "America/Chicago",
	},
	"TN23": {
		ICAO:      "TN23",
		Name:      "Titan Field",
		City:      "Clarksville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 385,
		Latitude:  36.4944000244,
		Longitude: -87.3218994141,
		Timezone:  "America/Chicago",
	},
	"TN24": {
		ICAO:      "TN24",
		Name:      "Spencer Field",
		City:      "Manchester",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1080,
		Latitude:  35.3875999451,
		Longitude: -85.9625015259,
		Timezone:  "America/Chicago",
	},
	"TN30": {
		ICAO:      "TN30",
		Name:      "Cub Haven Airport",
		City:      "Niota",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1000,
		Latitude:  35.5471992493,
		Longitude: -84.5100021362,
		Timezone:  "America/New_York",
	},
	"TN36": {
		ICAO:      "TN36",
		Name:      "Flying I Ranch Airport",
		City:      "Ripley",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 350,
		Latitude:  35.7620010376,
		Longitude: -89.5512008667,
		Timezone:  "America/Chicago",
	},
	"TN37": {
		ICAO:      "TN37",
		Name:      "Anderson Airport",
		City:      "Toone",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 392,
		Latitude:  35.4042015076,
		Longitude: -89.0223007202,
		Timezone:  "America/Chicago",
	},
	"TN38": {
		ICAO:      "TN38",
		Name:      "Rutledge Field",
		City:      "Tullahoma",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1125,
		Latitude:  35.3718986511,
		Longitude: -86.2668991089,
		Timezone:  "America/Chicago",
	},
	"TN39": {
		ICAO:      "TN39",
		Name:      "Amacher Strip",
		City:      "Tullahoma",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1080,
		Latitude:  35.3055992126,
		Longitude: -86.2367019653,
		Timezone:  "America/Chicago",
	},
	"TN41": {
		ICAO:      "TN41",
		Name:      "100 Aker Wood Airport",
		City:      "Spring City",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 809,
		Latitude:  35.7728004456,
		Longitude: -84.765296936,
		Timezone:  "America/New_York",
	},
	"TN43": {
		ICAO:      "TN43",
		Name:      "Isle-A-Port STOLport",
		City:      "Memphis",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 215,
		Latitude:  35.0959014893,
		Longitude: -90.1139984131,
		Timezone:  "America/Chicago",
	},
	"TN44": {
		ICAO:      "TN44",
		Name:      "Deerfield Resort Airport",
		City:      "Lafollette",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1350,
		Latitude:  36.346698761,
		Longitude: -84.0121994019,
		Timezone:  "America/New_York",
	},
	"TN47": {
		ICAO:      "TN47",
		Name:      "Needham's Airport",
		City:      "Arlington",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 289,
		Latitude:  35.2890014648,
		Longitude: -89.7412033081,
		Timezone:  "America/Chicago",
	},
	"TN48": {
		ICAO:      "TN48",
		Name:      "Turner Field",
		City:      "Dresden",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 401,
		Latitude:  36.2775993347,
		Longitude: -88.6537017822,
		Timezone:  "America/Chicago",
	},
	"TN50": {
		ICAO:      "TN50",
		Name:      "Indian Hill Farm Airport",
		City:      "New Hope",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 676,
		Latitude:  34.9915008545,
		Longitude: -85.6508026123,
		Timezone:  "America/Chicago",
	},
	"TN51": {
		ICAO:      "TN51",
		Name:      "Aydelotte STOLport",
		City:      "Springfield",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 545,
		Latitude:  36.6427993774,
		Longitude: -86.9764022827,
		Timezone:  "America/Chicago",
	},
	"TN52": {
		ICAO:      "TN52",
		Name:      "King Airport",
		City:      "Winchester",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1050,
		Latitude:  35.1380996704,
		Longitude: -86.0811004639,
		Timezone:  "America/Chicago",
	},
	"TN53": {
		ICAO:      "TN53",
		Name:      "Parr Field",
		City:      "Clarksville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 590,
		Latitude:  36.5559005737,
		Longitude: -87.4483032227,
		Timezone:  "America/Chicago",
	},
	"TN56": {
		ICAO:      "TN56",
		Name:      "Carey Airport",
		City:      "Mayland",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1960,
		Latitude:  36.0722999573,
		Longitude: -85.113899231,
		Timezone:  "America/Chicago",
	},
	"TN58": {
		ICAO:      "TN58",
		Name:      "Parsons Field",
		City:      "Brighton",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 330,
		Latitude:  35.3917007446,
		Longitude: -89.65650177,
		Timezone:  "America/Chicago",
	},
	"TN62": {
		ICAO:      "TN62",
		Name:      "Mc Afee Stol Patch STOLport",
		City:      "Johnson City",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1925,
		Latitude:  36.3722991943,
		Longitude: -82.3915023804,
		Timezone:  "America/New_York",
	},
	"TN64": {
		ICAO:      "TN64",
		Name:      "West Wind Airpark",
		City:      "Sweetwater",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1000,
		Latitude:  35.5442008972,
		Longitude: -84.5327987671,
		Timezone:  "America/New_York",
	},
	"TN65": {
		ICAO:      "TN65",
		Name:      "Long Meadow Airstrip",
		City:      "Murfreesburg",
		Country:   "US",
		Elevation: 750,
		Latitude:  35.6664009094,
		Longitude: -86.4852981567,
		Timezone:  "America/Chicago",
	},
	"TN66": {
		ICAO:      "TN66",
		Name:      "Austin Field",
		City:      "Pikeville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1000,
		Latitude:  35.7444992065,
		Longitude: -85.047203064,
		Timezone:  "America/Chicago",
	},
	"TN67": {
		ICAO:      "TN67",
		Name:      "Myers-Smith Airport",
		City:      "Estill Springs",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 900,
		Latitude:  35.2478981018,
		Longitude: -86.1286010742,
		Timezone:  "America/Chicago",
	},
	"TN68": {
		ICAO:      "TN68",
		Name:      "Gibson STOLport",
		City:      "Murfreesboro",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 805,
		Latitude:  35.768699646,
		Longitude: -86.5466995239,
		Timezone:  "America/Chicago",
	},
	"TN70": {
		ICAO:      "TN70",
		Name:      "Burkeen Field",
		City:      "Collierville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 365,
		Latitude:  35.0261993408,
		Longitude: -89.7025985718,
		Timezone:  "America/Chicago",
	},
	"TN71": {
		ICAO:      "TN71",
		Name:      "Cox Farm Airport",
		City:      "Lenoir City",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1000,
		Latitude:  35.7944984436,
		Longitude: -84.3415985107,
		Timezone:  "America/New_York",
	},
	"TN72": {
		ICAO:      "TN72",
		Name:      "King Airport",
		City:      "Sevierville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1080,
		Latitude:  35.8459014893,
		Longitude: -83.4817962646,
		Timezone:  "America/New_York",
	},
	"TN73": {
		ICAO:      "TN73",
		Name:      "Doc Jones Field",
		City:      "Nashville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 770,
		Latitude:  36.3233985901,
		Longitude: -86.9593963623,
		Timezone:  "America/Chicago",
	},
	"TN74": {
		ICAO:      "TN74",
		Name:      "Will A Hildreth Farm Airport",
		City:      "Lenoir City",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 900,
		Latitude:  35.8000984192,
		Longitude: -84.3193969727,
		Timezone:  "America/New_York",
	},
	"TN77": {
		ICAO:      "TN77",
		Name:      "Whifferdill Airport",
		City:      "Chapmansboro",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 622,
		Latitude:  36.32180023,
		Longitude: -87.15070343,
		Timezone:  "America/Chicago",
	},
	"TN79": {
		ICAO:      "TN79",
		Name:      "Oakley Airport",
		City:      "Nashville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 400,
		Latitude:  36.1722984314,
		Longitude: -86.9597015381,
		Timezone:  "America/Chicago",
	},
	"TN80": {
		ICAO:      "TN80",
		Name:      "Schiff Airport",
		City:      "Cookeville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1260,
		Latitude:  36.2333984375,
		Longitude: -85.426399231,
		Timezone:  "America/Chicago",
	},
	"TN81": {
		ICAO:      "TN81",
		Name:      "Shultz Airport",
		City:      "Franklin",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 770,
		Latitude:  35.7489013672,
		Longitude: -86.8135986328,
		Timezone:  "America/Chicago",
	},
	"TN83": {
		ICAO:      "TN83",
		Name:      "Cedar Glade Aerodrome",
		City:      "Murfreesboro",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 770,
		Latitude:  35.7223014832,
		Longitude: -86.2750015259,
		Timezone:  "America/Chicago",
	},
	"TN85": {
		ICAO:      "TN85",
		Name:      "Hogue Airport",
		City:      "Spencer",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1800,
		Latitude:  35.638999939,
		Longitude: -85.3972015381,
		Timezone:  "America/Chicago",
	},
	"TN86": {
		ICAO:      "TN86",
		Name:      "Walden Ridge Airport",
		City:      "Rockwood",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 820,
		Latitude:  35.7986984253,
		Longitude: -84.7632980347,
		Timezone:  "America/New_York",
	},
	"TN87": {
		ICAO:      "TN87",
		Name:      "Montvale Airpark",
		City:      "Maryville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1030,
		Latitude:  35.6926002502,
		Longitude: -83.9988021851,
		Timezone:  "America/New_York",
	},
	"TN89": {
		ICAO:      "TN89",
		Name:      "Matthews Airport",
		City:      "Whitwell",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 700,
		Latitude:  35.2583999634,
		Longitude: -85.4526977539,
		Timezone:  "America/Chicago",
	},
	"TN93": {
		ICAO:      "TN93",
		Name:      "Cotton Patch Airport",
		City:      "Bolivar",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 465,
		Latitude:  35.2822990417,
		Longitude: -89.0291976929,
		Timezone:  "America/Chicago",
	},
	"TN94": {
		ICAO:      "TN94",
		Name:      "Robertson Farm Airport",
		City:      "Knoxville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1020,
		Latitude:  36.0010986328,
		Longitude: -84.0744018555,
		Timezone:  "America/New_York",
	},
	"TN96": {
		ICAO:      "TN96",
		Name:      "Fall Creek Field",
		City:      "Lebanon",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 675,
		Latitude:  36.0411987305,
		Longitude: -86.3582992554,
		Timezone:  "America/Chicago",
	},
	"TN97": {
		ICAO:      "TN97",
		Name:      "Triune Airfield",
		City:      "Nashville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 740,
		Latitude:  35.8069000244,
		Longitude: -86.612197876,
		Timezone:  "America/Chicago",
	},
	"TN98": {
		ICAO:      "TN98",
		Name:      "Sky Ranch Airport",
		City:      "Knoxville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 830,
		Latitude:  35.8856010437,
		Longitude: -83.9577026367,
		Timezone:  "America/New_York",
	},
	"TN99": {
		ICAO:      "TN99",
		Name:      "Pensinger Airport",
		City:      "Greeneville",
		State:     "Tennessee",
		Country:   "US",
		Elevation: 1265,
		Latitude:  36.2963981628,
		Longitude: -82.760597229,
		Timezone:  "America/New_York",
	},
	"TNCA": {
		ICAO:      "TNCA",
		IATA:      "AUA",
		Name:      "Queen Beatrix International Airport",
		City:      "Oranjestad",
		Country:   "AW",
		Elevation: 60,
		Latitude:  12.5013999939,
		Longitude: -70.0151977539,
		Timezone:  "America/Aruba",
	},
	"TNCB": {
		ICAO:      "TNCB",
		IATA:      "BON",
		Name:      "Flamingo International Airport",
		City:      "Kralendijk",
		State:     "Bonaire",
		Country:   "BQ",
		Elevation: 20,
		Latitude:  12.1309995651,
		Longitude: -68.2685012817,
		Timezone:  "America/Kralendijk",
	},
	"TNCC": {
		ICAO:      "TNCC",
		IATA:      "CUR",
		Name:      "Hato International Airport",
		City:      "Willemstad",
		Country:   "CW",
		Elevation: 29,
		Latitude:  12.1888999939,
		Longitude: -68.9598007202,
		Timezone:  "America/Curacao",
	},
	"TNCE": {
		ICAO:      "TNCE",
		IATA:      "EUX",
		Name:      "F. D. Roosevelt Airport",
		City:      "Sint Eustatius",
		State:     "Sint-Eustatius",
		Country:   "BQ",
		Elevation: 129,
		Latitude:  17.4965000153,
		Longitude: -62.9794006348,
		Timezone:  "America/Kralendijk",
	},
	"TNCM": {
		ICAO:      "TNCM",
		IATA:      "SXM",
		Name:      "Princess Juliana International Airport",
		City:      "Saint Martin",
		Country:   "SX",
		Elevation: 13,
		Latitude:  18.0410003662,
		Longitude: -63.1088981628,
		Timezone:  "America/Lower_Princes",
	},
	"TNCS": {
		ICAO:      "TNCS",
		IATA:      "SAB",
		Name:      "Juancho E. Yrausquin Airport",
		City:      "Saba",
		State:     "Saba",
		Country:   "BQ",
		Elevation: 60,
		Latitude:  17.6450004578,
		Longitude: -63.2200012207,
		Timezone:  "America/Kralendijk",
	},
	"TQPF": {
		ICAO:      "TQPF",
		IATA:      "AXA",
		Name:      "Wallblake Airport",
		City:      "The Valley",
		Country:   "AI",
		Elevation: 127,
		Latitude:  18.2047996521,
		Longitude: -63.0550994873,
		Timezone:  "America/Anguilla",
	},
	"TRPG": {
		ICAO:      "TRPG",
		IATA:      "MNI",
		Name:      "John A. Osborne Airport",
		City:      "Gerald's Park",
		State:     "Saint-Peter",
		Country:   "MS",
		Elevation: 550,
		Latitude:  16.7914009094,
		Longitude: -62.1932983398,
		Timezone:  "America/Montserrat",
	},
	"TS00": {
		ICAO:      "TS00",
		Name:      "Fuller Airport",
		City:      "Fort Worth",
		State:     "Texas",
		Country:   "US",
		Elevation: 490,
		Latitude:  32.7882003784,
		Longitude: -97.1808013916,
		Timezone:  "America/Chicago",
	},
	"TS01": {
		ICAO:      "TS01",
		Name:      "Landry STOLport",
		City:      "Geogetown",
		State:     "Texas",
		Country:   "US",
		Elevation: 780,
		Latitude:  30.7126998901,
		Longitude: -97.6502990723,
		Timezone:  "America/Chicago",
	},
	"TS02": {
		ICAO:      "TS02",
		Name:      "One Sixty Four Place Airport",
		City:      "Jacksboro",
		State:     "Texas",
		Country:   "US",
		Elevation: 1200,
		Latitude:  33.1195983887,
		Longitude: -98.2305984497,
		Timezone:  "America/Chicago",
	},
	"TS03": {
		ICAO:      "TS03",
		Name:      "Bryant's Landing Airport",
		City:      "Walnut Springs",
		State:     "Texas",
		Country:   "US",
		Elevation: 914,
		Latitude:  32.0615005493,
		Longitude: -97.7378005981,
		Timezone:  "America/Chicago",
	},
	"TS04": {
		ICAO:      "TS04",
		Name:      "Rio Vista Ranch Airport",
		City:      "Del Rio",
		State:     "Texas",
		Country:   "US",
		Elevation: 1774,
		Latitude:  29.8358001709,
		Longitude: -100.9779968262,
		Timezone:  "America/Chicago",
	},
	"TS05": {
		ICAO:      "TS05",
		Name:      "Progreso Airport",
		City:      "Progreso",
		State:     "Texas",
		Country:   "US",
		Elevation: 70,
		Latitude:  26.0837001801,
		Longitude: -97.950302124,
		Timezone:  "America/Chicago",
	},
	"TS07": {
		ICAO:      "TS07",
		Name:      "Dry Creek Airport",
		City:      "Cypress",
		State:     "Texas",
		Country:   "US",
		Elevation: 152,
		Latitude:  29.9862995148,
		Longitude: -95.6856002808,
		Timezone:  "America/Chicago",
	},
	"TS08": {
		ICAO:      "TS08",
		Name:      "Calaway Airport",
		City:      "Ganado",
		State:     "Texas",
		Country:   "US",
		Elevation: 74,
		Latitude:  29.0835990906,
		Longitude: -96.5002975464,
		Timezone:  "America/Chicago",
	},
	"TS11": {
		ICAO:      "TS11",
		Name:      "Glenmar Airport",
		City:      "Campbell",
		State:     "Texas",
		Country:   "US",
		Elevation: 531,
		Latitude:  33.1071014404,
		Longitude: -95.9300003052,
		Timezone:  "America/Chicago",
	},
	"TS14": {
		ICAO:      "TS14",
		Name:      "Smith Flying Service Airport",
		City:      "De Leon",
		State:     "Texas",
		Country:   "US",
		Elevation: 1275,
		Latitude:  32.1086997986,
		Longitude: -98.4879989624,
		Timezone:  "America/Chicago",
	},
	"TS15": {
		ICAO:      "TS15",
		Name:      "Cibolo Creek Ranch Airport",
		City:      "Marfa",
		State:     "Texas",
		Country:   "US",
		Elevation: 4400,
		Latitude:  29.89290047,
		Longitude: -104.262001,
		Timezone:  "America/Chicago",
	},
	"TS18": {
		ICAO:      "TS18",
		Name:      "Kingsland Estates Airport",
		City:      "Kingsland",
		State:     "Texas",
		Country:   "US",
		Elevation: 873,
		Latitude:  30.6543998718,
		Longitude: -98.4785995483,
		Timezone:  "America/Chicago",
	},
	"TS20": {
		ICAO:      "TS20",
		Name:      "C Lazy T Ranch Airport",
		City:      "Seymour",
		State:     "Texas",
		Country:   "US",
		Elevation: 1365,
		Latitude:  33.5322990417,
		Longitude: -99.1937026978,
		Timezone:  "America/Chicago",
	},
	"TS21": {
		ICAO:      "TS21",
		Name:      "Roy Ranch Airport",
		City:      "Justiceburg",
		State:     "Texas",
		Country:   "US",
		Elevation: 2366,
		Latitude:  33.084833,
		Longitude: -101.114833,
		Timezone:  "America/Chicago",
	},
	"TS22": {
		ICAO:      "TS22",
		Name:      "Joye Ranch Airport",
		City:      "Smiley",
		State:     "Texas",
		Country:   "US",
		Elevation: 280,
		Latitude:  29.2416992188,
		Longitude: -97.629699707,
		Timezone:  "America/Chicago",
	},
	"TS25": {
		ICAO:      "TS25",
		Name:      "Mynatt Field",
		City:      "South Sulphur",
		State:     "Texas",
		Country:   "US",
		Elevation: 557,
		Latitude:  33.2697982788,
		Longitude: -96.0224990845,
		Timezone:  "America/Chicago",
	},
	"TS27": {
		ICAO:      "TS27",
		Name:      "River Field",
		City:      "Columbus",
		State:     "Texas",
		Country:   "US",
		Elevation: 220,
		Latitude:  29.71619987,
		Longitude: -96.59140015,
		Timezone:  "America/Chicago",
	},
	"TS29": {
		ICAO:      "TS29",
		Name:      "Liberty Hill International Airport",
		City:      "Eddy",
		State:     "Texas",
		Country:   "US",
		Elevation: 790,
		Latitude:  31.2945995331,
		Longitude: -97.2969970703,
		Timezone:  "America/Chicago",
	},
	"TS35": {
		ICAO:      "TS35",
		Name:      "West Liberty Airport",
		City:      "Dayton",
		State:     "Texas",
		Country:   "US",
		Elevation: 80,
		Latitude:  30.0583000183,
		Longitude: -94.9781036377,
		Timezone:  "America/Chicago",
	},
	"TS36": {
		ICAO:      "TS36",
		Name:      "Silver Wings Airport",
		City:      "Fredricksburg",
		State:     "Texas",
		Country:   "US",
		Elevation: 2110,
		Latitude:  30.2196006775,
		Longitude: -99.1408004761,
		Timezone:  "America/Chicago",
	},
	"TS39": {
		ICAO:      "TS39",
		Name:      "Sherman Airport",
		City:      "Lockney",
		State:     "Texas",
		Country:   "US",
		Elevation: 3285,
		Latitude:  34.3120002747,
		Longitude: -101.4120025635,
		Timezone:  "America/Chicago",
	},
	"TS40": {
		ICAO:      "TS40",
		Name:      "Celina Field",
		City:      "Celina",
		State:     "Texas",
		Country:   "US",
		Elevation: 770,
		Latitude:  33.3128013611,
		Longitude: -96.7639007568,
		Timezone:  "America/Chicago",
	},
	"TS42": {
		ICAO:      "TS42",
		Name:      "Del-Tex Airport",
		City:      "Olton",
		State:     "Texas",
		Country:   "US",
		Elevation: 3600,
		Latitude:  34.1964988708,
		Longitude: -102.1279983521,
		Timezone:  "America/Chicago",
	},
	"TS44": {
		ICAO:      "TS44",
		Name:      "Dry Branch Ranch Airport",
		City:      "Pattison",
		State:     "Texas",
		Country:   "US",
		Elevation: 130,
		Latitude:  29.8400001526,
		Longitude: -96.0233001709,
		Timezone:  "America/Chicago",
	},
	"TS47": {
		ICAO:      "TS47",
		Name:      "Rock Creek Ranch Airport",
		City:      "Whitt",
		State:     "Texas",
		Country:   "US",
		Elevation: 1130,
		Latitude:  32.96849823,
		Longitude: -98.0113983154,
		Timezone:  "America/Chicago",
	},
	"TS50": {
		ICAO:      "TS50",
		Name:      "Austinia Airport",
		City:      "Texas City",
		State:     "Texas",
		Country:   "US",
		Elevation: 15,
		Latitude:  29.4174995422,
		Longitude: -94.998298645,
		Timezone:  "America/Chicago",
	},
	"TS51": {
		ICAO:      "TS51",
		Name:      "Barton Memorial Airport",
		City:      "Garrison",
		State:     "Texas",
		Country:   "US",
		Elevation: 345,
		Latitude:  31.8370990753,
		Longitude: -94.4593963623,
		Timezone:  "America/Chicago",
	},
	"TS53": {
		ICAO:      "TS53",
		Name:      "Flying Cap Valley Airport",
		City:      "Grapevine",
		State:     "Texas",
		Country:   "US",
		Elevation: 638,
		Latitude:  32.9365005493,
		Longitude: -97.1352996826,
		Timezone:  "America/Chicago",
	},
	"TS54": {
		ICAO:      "TS54",
		Name:      "Flying B Airport",
		City:      "Pickton",
		State:     "Texas",
		Country:   "US",
		Elevation: 535,
		Latitude:  33.0148010254,
		Longitude: -95.4344024658,
		Timezone:  "America/Chicago",
	},
	"TS55": {
		ICAO:      "TS55",
		Name:      "Bat Cave Field",
		City:      "Garden Ridge",
		State:     "Texas",
		Country:   "US",
		Elevation: 850,
		Latitude:  29.6718997955,
		Longitude: -98.3044967651,
		Timezone:  "America/Chicago",
	},
	"TS57": {
		ICAO:      "TS57",
		Name:      "Red Ace Ranch Airport",
		City:      "Decatur",
		State:     "Texas",
		Country:   "US",
		Elevation: 900,
		Latitude:  33.2417984009,
		Longitude: -97.6211013794,
		Timezone:  "America/Chicago",
	},
	"TS61": {
		ICAO:      "TS61",
		Name:      "Little 'L' Ranch Airport",
		City:      "Glen Rose",
		State:     "Texas",
		Country:   "US",
		Elevation: 585,
		Latitude:  32.2117004395,
		Longitude: -97.6421966553,
		Timezone:  "America/Chicago",
	},
	"TS62": {
		ICAO:      "TS62",
		Name:      "Norris Field",
		City:      "Jayton",
		State:     "Texas",
		Country:   "US",
		Elevation: 1987,
		Latitude:  33.2000999451,
		Longitude: -100.5179977417,
		Timezone:  "America/Chicago",
	},
	"TS63": {
		ICAO:      "TS63",
		Name:      "Square Air Airport",
		City:      "Mc Kinney",
		State:     "Texas",
		Country:   "US",
		Elevation: 655,
		Latitude:  33.2458992004,
		Longitude: -96.564201355,
		Timezone:  "America/Chicago",
	},
	"TS65": {
		ICAO:      "TS65",
		Name:      "Ducote Airpark",
		City:      "San Angelo",
		State:     "Texas",
		Country:   "US",
		Elevation: 1974,
		Latitude:  31.358499527,
		Longitude: -100.6129989624,
		Timezone:  "America/Chicago",
	},
	"TS67": {
		ICAO:      "TS67",
		Name:      "Kitty Hawk Flying Field",
		City:      "Garden Ridge",
		State:     "Texas",
		Country:   "US",
		Elevation: 860,
		Latitude:  29.6252002716,
		Longitude: -98.2780990601,
		Timezone:  "America/Chicago",
	},
	"TS69": {
		ICAO:      "TS69",
		Name:      "Barronena East Airport",
		City:      "Hebbronville",
		State:     "Texas",
		Country:   "US",
		Elevation: 576,
		Latitude:  27.4850006104,
		Longitude: -98.6620025635,
		Timezone:  "America/Chicago",
	},
	"TS70": {
		ICAO:      "TS70",
		Name:      "Jack Miller STOLport",
		City:      "Mansfield",
		State:     "Texas",
		Country:   "US",
		Elevation: 600,
		Latitude:  32.5750999451,
		Longitude: -97.0869979858,
		Timezone:  "America/Chicago",
	},
	"TS71": {
		ICAO:      "TS71",
		Name:      "Flying B Ranch Airport",
		City:      "Ovilla",
		State:     "Texas",
		Country:   "US",
		Elevation: 780,
		Latitude:  32.5279006958,
		Longitude: -96.943901062,
		Timezone:  "America/Chicago",
	},
	"TS73": {
		ICAO:      "TS73",
		Name:      "Stubbs Strip",
		City:      "Fort Worth",
		State:     "Texas",
		Country:   "US",
		Elevation: 755,
		Latitude:  32.775100708,
		Longitude: -97.4878005981,
		Timezone:  "America/Chicago",
	},
	"TS74": {
		ICAO:      "TS74",
		Name:      "Glass Airport",
		City:      "Sanger",
		State:     "Texas",
		Country:   "US",
		Elevation: 810,
		Latitude:  33.3625984192,
		Longitude: -97.2945022583,
		Timezone:  "America/Chicago",
	},
	"TS75": {
		ICAO:      "TS75",
		Name:      "Mitchell Nr One Airport",
		City:      "Sanderson",
		State:     "Texas",
		Country:   "US",
		Elevation: 2700,
		Latitude:  30.5030002594,
		Longitude: -102.158996582,
		Timezone:  "America/Chicago",
	},
	"TS76": {
		ICAO:      "TS76",
		Name:      "Redstone Ranch Airport",
		City:      "Stonewall",
		State:     "Texas",
		Country:   "US",
		Elevation: 1460,
		Latitude:  30.2502002716,
		Longitude: -98.5947036743,
		Timezone:  "America/Chicago",
	},
	"TS78": {
		ICAO:      "TS78",
		Name:      "Loesch Ranch Airport",
		City:      "Booker",
		State:     "Texas",
		Country:   "US",
		Elevation: 2804,
		Latitude:  36.3917007446,
		Longitude: -100.46900177,
		Timezone:  "America/Chicago",
	},
	"TS80": {
		ICAO:      "TS80",
		Name:      "Mc Alister Farm Airport",
		City:      "Electra",
		State:     "Texas",
		Country:   "US",
		Elevation: 1125,
		Latitude:  34.1305999756,
		Longitude: -98.9030990601,
		Timezone:  "America/Chicago",
	},
	"TS85": {
		ICAO:      "TS85",
		Name:      "Diamond J Airport",
		City:      "Pottsboro",
		State:     "Texas",
		Country:   "US",
		Elevation: 802,
		Latitude:  33.8106002808,
		Longitude: -96.7039031982,
		Timezone:  "America/Chicago",
	},
	"TS87": {
		ICAO:      "TS87",
		Name:      "Bridle Ridge Airport",
		City:      "Navasota",
		State:     "Texas",
		Country:   "US",
		Elevation: 328,
		Latitude:  30.4179992676,
		Longitude: -96.0793991089,
		Timezone:  "America/Chicago",
	},
	"TS89": {
		ICAO:      "TS89",
		Name:      "Parker Airport",
		City:      "Granbury",
		State:     "Texas",
		Country:   "US",
		Elevation: 925,
		Latitude:  32.3511009216,
		Longitude: -97.8087005615,
		Timezone:  "America/Chicago",
	},
	"TS90": {
		ICAO:      "TS90",
		Name:      "Bayless Airport",
		City:      "Rosharon",
		State:     "Texas",
		Country:   "US",
		Elevation: 49,
		Latitude:  29.4025001526,
		Longitude: -95.4319000244,
		Timezone:  "America/Chicago",
	},
	"TS92": {
		ICAO:      "TS92",
		Name:      "Little Peach Airport",
		City:      "Temple",
		State:     "Texas",
		Country:   "US",
		Elevation: 710,
		Latitude:  31.2145996094,
		Longitude: -97.4266967773,
		Timezone:  "America/Chicago",
	},
	"TS94": {
		ICAO:      "TS94",
		Name:      "Rancho Buena Vista Airport",
		City:      "Bay View",
		State:     "Texas",
		Country:   "US",
		Elevation: 20,
		Latitude:  26.2511997223,
		Longitude: -97.3007965088,
		Timezone:  "America/Chicago",
	},
	"TS95": {
		ICAO:      "TS95",
		Name:      "Aviasud Airpark",
		City:      "Beasley",
		State:     "Texas",
		Country:   "US",
		Elevation: 100,
		Latitude:  29.4864006042,
		Longitude: -95.9308013916,
		Timezone:  "America/Chicago",
	},
	"TS96": {
		ICAO:      "TS96",
		Name:      "El Campo Airpark",
		City:      "El Campo",
		State:     "Texas",
		Country:   "US",
		Elevation: 128,
		Latitude:  29.2668991089,
		Longitude: -96.3252029419,
		Timezone:  "America/Chicago",
	},
	"TS98": {
		ICAO:      "TS98",
		Name:      "Wings Over Texas Airport",
		City:      "Terrell",
		State:     "Texas",
		Country:   "US",
		Elevation: 532,
		Latitude:  32.7868995667,
		Longitude: -96.3069000244,
		Timezone:  "America/Chicago",
	},
	"TT01": {
		ICAO:      "TT01",
		Name:      "Pagan Airstrip",
		City:      "Shomu-Shon",
		State:     "Northern-Islands",
		Country:   "MP",
		Elevation: 34,
		Latitude:  18.1243991852,
		Longitude: 145.7689971924,
		Timezone:  "Pacific/Saipan",
	},
	"TTCP": {
		ICAO:      "TTCP",
		IATA:      "TAB",
		Name:      "Tobago-Crown Point Airport",
		City:      "Scarborough",
		State:     "Tobago",
		Country:   "TT",
		Elevation: 38,
		Latitude:  11.1497001648,
		Longitude: -60.8321990967,
		Timezone:  "America/Port_of_Spain",
	},
	"TTPP": {
		ICAO:      "TTPP",
		IATA:      "POS",
		Name:      "Piarco International Airport",
		City:      "Port of Spain",
		State:     "Tunapuna/Piarco",
		Country:   "TT",
		Elevation: 58,
		Latitude:  10.5953998566,
		Longitude: -61.3372001648,
		Timezone:  "America/Port_of_Spain",
	},
	"TUPA": {
		ICAO:      "TUPA",
		IATA:      "NGD",
		Name:      "Captain Auguste George Airport",
		City:      "Anegada",
		Country:   "VG",
		Elevation: 9,
		Latitude:  18.7271995544,
		Longitude: -64.3296966553,
		Timezone:  "America/Tortola",
	},
	"TUPJ": {
		ICAO:      "TUPJ",
		IATA:      "EIS",
		Name:      "Terrance B. Lettsome International Airport",
		City:      "Road Town",
		Country:   "VG",
		Elevation: 15,
		Latitude:  18.4447994232,
		Longitude: -64.5429992676,
		Timezone:  "America/Tortola",
	},
	"TUPW": {
		ICAO:      "TUPW",
		IATA:      "VIJ",
		Name:      "Virgin Gorda Airport",
		City:      "Spanish Town",
		Country:   "VG",
		Elevation: 9,
		Latitude:  18.4463996887,
		Longitude: -64.4274978638,
		Timezone:  "America/Tortola",
	},
	"TVSB": {
		ICAO:      "TVSB",
		IATA:      "BQU",
		Name:      "J F Mitchell Airport",
		City:      "Bequia",
		State:     "Grenadines",
		Country:   "VC",
		Elevation: 15,
		Latitude:  12.9884004593,
		Longitude: -61.2620010376,
		Timezone:  "America/St_Vincent",
	},
	"TVSC": {
		ICAO:      "TVSC",
		IATA:      "CIW",
		Name:      "Canouan Airport",
		City:      "Canouan",
		State:     "Grenadines",
		Country:   "VC",
		Elevation: 11,
		Latitude:  12.6990003586,
		Longitude: -61.3423995972,
		Timezone:  "America/St_Vincent",
	},
	"TVSM": {
		ICAO:      "TVSM",
		IATA:      "MQS",
		Name:      "Mustique Airport",
		City:      "Mustique Island",
		State:     "Grenadines",
		Country:   "VC",
		Elevation: 8,
		Latitude:  12.8879003525,
		Longitude: -61.1801986694,
		Timezone:  "America/St_Vincent",
	},
	"TVSU": {
		ICAO:      "TVSU",
		IATA:      "UNI",
		Name:      "Union Island International Airport",
		City:      "Union Island",
		State:     "Grenadines",
		Country:   "VC",
		Elevation: 16,
		Latitude:  12.6001348495,
		Longitude: -61.411945343,
		Timezone:  "America/St_Vincent",
	},
	"TVSV": {
		ICAO:      "TVSV",
		IATA:      "SVD",
		Name:      "E. T. Joshua Airport",
		City:      "Kingstown",
		State:     "Saint-George",
		Country:   "VC",
		Elevation: 66,
		Latitude:  13.1443004608,
		Longitude: -61.210899353,
		Timezone:  "America/St_Vincent",
	},
	"TX00": {
		ICAO:      "TX00",
		Name:      "Abilene Executive Airpark",
		City:      "Abilene",
		State:     "Texas",
		Country:   "US",
		Elevation: 1822,
		Latitude:  32.4488983154,
		Longitude: -99.6200027466,
		Timezone:  "America/Chicago",
	},
	"TX01": {
		ICAO:      "TX01",
		Name:      "New Home Airport",
		City:      "New Home",
		State:     "Texas",
		Country:   "US",
		Elevation: 3234,
		Latitude:  33.3083992004,
		Longitude: -101.9100036621,
		Timezone:  "America/Chicago",
	},
	"TX02": {
		ICAO:      "TX02",
		Name:      "Portlock Airfield",
		City:      "Abilene",
		State:     "Texas",
		Country:   "US",
		Elevation: 1780,
		Latitude:  32.5265007019,
		Longitude: -99.6128997803,
		Timezone:  "America/Chicago",
	},
	"TX03": {
		ICAO:      "TX03",
		Name:      "Stapleton Field",
		City:      "Abbott",
		State:     "Texas",
		Country:   "US",
		Elevation: 716,
		Latitude:  31.8752002716,
		Longitude: -97.0669021606,
		Timezone:  "America/Chicago",
	},
	"TX05": {
		ICAO:      "TX05",
		Name:      "Dryden Airport",
		City:      "Austin",
		State:     "Texas",
		Country:   "US",
		Elevation: 500,
		Latitude:  30.26049995,
		Longitude: -97.62529755,
		Timezone:  "America/Chicago",
	},
	"TX07": {
		ICAO:      "TX07",
		Name:      "Nix River Ranch Strip",
		City:      "Comstock",
		State:     "Texas",
		Country:   "US",
		Elevation: 1820,
		Latitude:  29.934299469,
		Longitude: -101.2340011597,
		Timezone:  "America/Chicago",
	},
	"TX09": {
		ICAO:      "TX09",
		Name:      "Cunningham Airpark",
		City:      "New Braunfels",
		State:     "Texas",
		Country:   "US",
		Elevation: 617,
		Latitude:  29.675556,
		Longitude: -98.012778,
		Timezone:  "America/Chicago",
	},
	"TX11": {
		ICAO:      "TX11",
		Name:      "Ross Planes Airport",
		City:      "Cross Plains",
		State:     "Texas",
		Country:   "US",
		Elevation: 1765,
		Latitude:  32.1399993896,
		Longitude: -99.1586990356,
		Timezone:  "America/Chicago",
	},
	"TX12": {
		ICAO:      "TX12",
		Name:      "Kidd-Private Airport",
		City:      "Denison",
		State:     "Texas",
		Country:   "US",
		Elevation: 705,
		Latitude:  33.8083992004,
		Longitude: -96.6085968018,
		Timezone:  "America/Chicago",
	},
	"TX13": {
		ICAO:      "TX13",
		Name:      "Mesa Vista Ranch Airport",
		City:      "Pampa",
		State:     "Texas",
		Country:   "US",
		Elevation: 2772,
		Latitude:  35.889278,
		Longitude: -101.030139,
		Timezone:  "America/Chicago",
	},
	"TX15": {
		ICAO:      "TX15",
		Name:      "Beggs Ranch/Aledo/ Airport",
		City:      "Aledo",
		State:     "Texas",
		Country:   "US",
		Elevation: 1060,
		Latitude:  32.751499176,
		Longitude: -97.6128005981,
		Timezone:  "America/Chicago",
	},
	"TX16": {
		ICAO:      "TX16",
		Name:      "Log Cabin Airport",
		City:      "Aledo",
		State:     "Texas",
		Country:   "US",
		Elevation: 1100,
		Latitude:  32.8025016785,
		Longitude: -97.6091995239,
		Timezone:  "America/Chicago",
	},
	"TX19": {
		ICAO:      "TX19",
		Name:      "Russells Ranch Airport",
		City:      "Annona",
		State:     "Texas",
		Country:   "US",
		Elevation: 400,
		Latitude:  33.5362014771,
		Longitude: -94.9072036743,
		Timezone:  "America/Chicago",
	},
	"TX20": {
		ICAO:      "TX20",
		Name:      "Steen Airport",
		City:      "Anson",
		State:     "Texas",
		Country:   "US",
		Elevation: 1670,
		Latitude:  32.7932014465,
		Longitude: -99.8778991699,
		Timezone:  "America/Chicago",
	},
	"TX21": {
		ICAO:      "TX21",
		Name:      "Hornady Ranch Airport",
		City:      "Archer City",
		State:     "Texas",
		Country:   "US",
		Elevation: 1165,
		Latitude:  33.4779014587,
		Longitude: -98.5475997925,
		Timezone:  "America/Chicago",
	},
	"TX22": {
		ICAO:      "TX22",
		Name:      "Leroux Airport",
		City:      "Argyle",
		State:     "Texas",
		Country:   "US",
		Elevation: 670,
		Latitude:  33.1017990112,
		Longitude: -97.1549987793,
		Timezone:  "America/Chicago",
	},
	"TX23": {
		ICAO:      "TX23",
		Name:      "Flf Gliderport",
		City:      "Briggs",
		State:     "Texas",
		Country:   "US",
		Elevation: 1150,
		Latitude:  30.856300354,
		Longitude: -97.9458999634,
		Timezone:  "America/Chicago",
	},
	"TX24": {
		ICAO:      "TX24",
		Name:      "Oaks Airport",
		City:      "Arp",
		State:     "Texas",
		Country:   "US",
		Elevation: 510,
		Latitude:  32.2751998901,
		Longitude: -95.0836029053,
		Timezone:  "America/Chicago",
	},
	"TX25": {
		ICAO:      "TX25",
		Name:      "Lochridge Ranch Airport",
		City:      "Athens",
		State:     "Texas",
		Country:   "US",
		Elevation: 391,
		Latitude:  31.9892997742,
		Longitude: -95.9511032104,
		Timezone:  "America/Chicago",
	},
	"TX29": {
		ICAO:      "TX29",
		Name:      "Flying O Airport",
		City:      "Bardwell",
		State:     "Texas",
		Country:   "US",
		Elevation: 490,
		Latitude:  32.2890014648,
		Longitude: -96.7043991089,
		Timezone:  "America/Chicago",
	},
	"TX31": {
		ICAO:      "TX31",
		Name:      "Edwards Lucian Wells Ranch Airport",
		City:      "Big Spring",
		State:     "Texas",
		Country:   "US",
		Elevation: 2505,
		Latitude:  32.0736999512,
		Longitude: -101.5640029907,
		Timezone:  "America/Chicago",
	},
	"TX32": {
		ICAO:      "TX32",
		Name:      "Bar V K Airport",
		City:      "Bolivar",
		State:     "Texas",
		Country:   "US",
		Elevation: 765,
		Latitude:  33.3763999939,
		Longitude: -97.2433013916,
		Timezone:  "America/Chicago",
	},
	"TX33": {
		ICAO:      "TX33",
		Name:      "Haire Airport",
		City:      "Sanger",
		State:     "Texas",
		Country:   "US",
		Elevation: 860,
		Latitude:  33.3418006897,
		Longitude: -97.3086013794,
		Timezone:  "America/Chicago",
	},
	"TX34": {
		ICAO:      "TX34",
		Name:      "Windy Tales Airport",
		City:      "Mineral Wells",
		State:     "Texas",
		Country:   "US",
		Elevation: 980,
		Latitude:  32.8484153748,
		Longitude: -97.9843063354,
		Timezone:  "America/Chicago",
	},
	"TX35": {
		ICAO:      "TX35",
		Name:      "Key's Ranch Airport",
		City:      "Boxelder/Lydia",
		State:     "Texas",
		Country:   "US",
		Elevation: 345,
		Latitude:  33.4431991577,
		Longitude: -94.8141021729,
		Timezone:  "America/Chicago",
	},
	"TX36": {
		ICAO:      "TX36",
		Name:      "Green Ranch Airport",
		City:      "Breckenridge",
		State:     "Texas",
		Country:   "US",
		Elevation: 1190,
		Latitude:  32.8334999084,
		Longitude: -99.0419998169,
		Timezone:  "America/Chicago",
	},
	"TX37": {
		ICAO:      "TX37",
		Name:      "Flying S Ranch Airport",
		City:      "Bridgeport",
		Country:   "US",
		Elevation: 850,
		Latitude:  33.1318016052,
		Longitude: -97.7686004639,
		Timezone:  "America/Chicago",
	},
	"TX38": {
		ICAO:      "TX38",
		Name:      "Duke Ranch Airport",
		City:      "Darrouzett",
		State:     "Texas",
		Country:   "US",
		Elevation: 2705,
		Latitude:  36.3795013428,
		Longitude: -100.3430023193,
		Timezone:  "America/Chicago",
	},
	"TX39": {
		ICAO:      "TX39",
		Name:      "Ruby Field",
		City:      "Era",
		State:     "Texas",
		Country:   "US",
		Elevation: 650,
		Latitude:  33.4771995544,
		Longitude: -97.2506027222,
		Timezone:  "America/Chicago",
	},
	"TX40": {
		ICAO:      "TX40",
		Name:      "Echo Lake Airport",
		City:      "Brownsboro",
		State:     "Texas",
		Country:   "US",
		Elevation: 450,
		Latitude:  32.2542991638,
		Longitude: -95.6557998657,
		Timezone:  "America/Chicago",
	},
	"TX41": {
		ICAO:      "TX41",
		Name:      "Pippen-York Ranch Airport",
		City:      "Blanco",
		State:     "Texas",
		Country:   "US",
		Elevation: 1340,
		Latitude:  30.091299057,
		Longitude: -98.3647003174,
		Timezone:  "America/Chicago",
	},
	"TX42": {
		ICAO:      "TX42",
		Name:      "Fair Weather Field",
		City:      "Monaville",
		State:     "Texas",
		Country:   "US",
		Elevation: 175,
		Latitude:  29.9375,
		Longitude: -96.0394439697,
		Timezone:  "America/Chicago",
	},
	"TX43": {
		ICAO:      "TX43",
		Name:      "Goode Field",
		City:      "Canton",
		State:     "Texas",
		Country:   "US",
		Elevation: 445,
		Latitude:  32.5098991394,
		Longitude: -96.0027008057,
		Timezone:  "America/Chicago",
	},
	"TX45": {
		ICAO:      "TX45",
		Name:      "Hawkeye Hunting Club Airport",
		City:      "Center",
		State:     "Texas",
		Country:   "US",
		Elevation: 260,
		Latitude:  31.7667999268,
		Longitude: -97.1294021606,
		Timezone:  "America/Chicago",
	},
	"TX46": {
		ICAO:      "TX46",
		Name:      "Blackwood Airpark",
		City:      "Cleburne",
		State:     "Texas",
		Country:   "US",
		Elevation: 835,
		Latitude:  32.4095993042,
		Longitude: -97.3871994019,
		Timezone:  "America/Chicago",
	},
	"TX47": {
		ICAO:      "TX47",
		Name:      "Sandy Creek Ranch Airfield",
		City:      "Revenna",
		State:     "Texas",
		Country:   "US",
		Elevation: 547,
		Latitude:  33.7076835632,
		Longitude: -96.2238464355,
		Timezone:  "America/Chicago",
	},
	"TX48": {
		ICAO:      "TX48",
		Name:      "Talley Airport",
		City:      "Clifton",
		State:     "Texas",
		Country:   "US",
		Elevation: 880,
		Latitude:  31.832222,
		Longitude: -97.718889,
		Timezone:  "America/Chicago",
	},
	"TX49": {
		ICAO:      "TX49",
		Name:      "Biggin Hill Airport",
		City:      "Hockley",
		State:     "Texas",
		Country:   "US",
		Elevation: 250,
		Latitude:  30.1182994843,
		Longitude: -95.8600006104,
		Timezone:  "America/Chicago",
	},
	"TX51": {
		ICAO:      "TX51",
		Name:      "Smith Airport",
		City:      "Corsicana",
		State:     "Texas",
		Country:   "US",
		Elevation: 405,
		Latitude:  32.2000999451,
		Longitude: -96.3293991089,
		Timezone:  "America/Chicago",
	},
	"TX52": {
		ICAO:      "TX52",
		Name:      "Parrish Airstrip",
		City:      "Cranfills Gap",
		State:     "Texas",
		Country:   "US",
		Elevation: 917,
		Latitude:  31.7945995331,
		Longitude: -97.7739028931,
		Timezone:  "America/Chicago",
	},
	"TX54": {
		ICAO:      "TX54",
		Name:      "Aero Estates Airport",
		City:      "Celina",
		State:     "Texas",
		Country:   "US",
		Elevation: 605,
		Latitude:  33.298500061,
		Longitude: -96.863899231,
		Timezone:  "America/Chicago",
	},
	"TX56": {
		ICAO:      "TX56",
		Name:      "Douglass Ranch Airport",
		City:      "Freer",
		State:     "Texas",
		Country:   "US",
		Elevation: 680,
		Latitude:  27.7154998779,
		Longitude: -98.7024993896,
		Timezone:  "America/Chicago",
	},
	"TX61": {
		ICAO:      "TX61",
		Name:      "Baker's Place Airport",
		City:      "Elgin",
		State:     "Texas",
		Country:   "US",
		Elevation: 460,
		Latitude:  30.3302001953,
		Longitude: -97.4589004517,
		Timezone:  "America/Chicago",
	},
	"TX62": {
		ICAO:      "TX62",
		Name:      "Rhodes Ranch Airport",
		City:      "Mineola",
		State:     "Texas",
		Country:   "US",
		Elevation: 405,
		Latitude:  32.6334991455,
		Longitude: -95.4391021729,
		Timezone:  "America/Chicago",
	},
	"TX64": {
		ICAO:      "TX64",
		Name:      "Lazy 9 Ranch Airport",
		City:      "Decatur",
		State:     "Texas",
		Country:   "US",
		Elevation: 1020,
		Latitude:  33.2993011475,
		Longitude: -97.5272979736,
		Timezone:  "America/Chicago",
	},
	"TX66": {
		ICAO:      "TX66",
		Name:      "Rebel Field",
		City:      "Alta Loma",
		State:     "Texas",
		Country:   "US",
		Elevation: 28,
		Latitude:  29.3987998962,
		Longitude: -95.0784988403,
		Timezone:  "America/Chicago",
	},
	"TX67": {
		ICAO:      "TX67",
		Name:      "Embry Ranch Airport",
		City:      "Keene",
		State:     "Texas",
		Country:   "US",
		Elevation: 900,
		Latitude:  32.3605995178,
		Longitude: -97.3024978638,
		Timezone:  "America/Chicago",
	},
	"TX68": {
		ICAO:      "TX68",
		Name:      "Parker Place Airport",
		City:      "Ector",
		State:     "Texas",
		Country:   "US",
		Elevation: 620,
		Latitude:  33.5918006897,
		Longitude: -96.2766036987,
		Timezone:  "America/Chicago",
	},
	"TX70": {
		ICAO:      "TX70",
		Name:      "Eberly Ranch Airport",
		City:      "Chappell Hill",
		State:     "Texas",
		Country:   "US",
		Elevation: 280,
		Latitude:  30.0518894196,
		Longitude: -96.2182388306,
		Timezone:  "America/Chicago",
	},
	"TX72": {
		ICAO:      "TX72",
		Name:      "Wood Triple D Airport",
		City:      "Elgin",
		State:     "Texas",
		Country:   "US",
		Elevation: 505,
		Latitude:  30.3791007996,
		Longitude: -97.1947021484,
		Timezone:  "America/Chicago",
	},
	"TX73": {
		ICAO:      "TX73",
		Name:      "Big Brown Creek Airstrip",
		City:      "Fairfield",
		State:     "Texas",
		Country:   "US",
		Elevation: 323,
		Latitude:  31.8309993744,
		Longitude: -96.0522003174,
		Timezone:  "America/Chicago",
	},
	"TX74": {
		ICAO:      "TX74",
		Name:      "Thomas Flying Field",
		City:      "Parker",
		State:     "Texas",
		Country:   "US",
		Elevation: 830,
		Latitude:  32.2613983154,
		Longitude: -97.2752990723,
		Timezone:  "America/Chicago",
	},
	"TX75": {
		ICAO:      "TX75",
		Name:      "Sandbur Ranches Private Airport",
		City:      "Franklin",
		State:     "Texas",
		Country:   "US",
		Elevation: 455,
		Latitude:  31.0097007751,
		Longitude: -96.540802002,
		Timezone:  "America/Chicago",
	},
	"TX78": {
		ICAO:      "TX78",
		Name:      "Block Ranch Airport",
		City:      "Alvarado",
		State:     "Texas",
		Country:   "US",
		Elevation: 755,
		Latitude:  32.3292007446,
		Longitude: -97.2319030762,
		Timezone:  "America/Chicago",
	},
	"TX79": {
		ICAO:      "TX79",
		Name:      "Crescent C Ranch Airport",
		City:      "Cotulla",
		State:     "Texas",
		Country:   "US",
		Elevation: 277,
		Latitude:  28.0888881683,
		Longitude: -99.0488891602,
		Timezone:  "America/Chicago",
	},
	"TX81": {
		ICAO:      "TX81",
		Name:      "Robotek Airport",
		City:      "Gainesville",
		State:     "Texas",
		Country:   "US",
		Elevation: 775,
		Latitude:  33.5499992371,
		Longitude: -97.0381011963,
		Timezone:  "America/Chicago",
	},
	"TX82": {
		ICAO:      "TX82",
		Name:      "Laguna-Not A-Park Airstrip",
		City:      "Laguna Park",
		State:     "Texas",
		Country:   "US",
		Elevation: 620,
		Latitude:  31.8635997772,
		Longitude: -97.380897522,
		Timezone:  "America/Chicago",
	},
	"TX89": {
		ICAO:      "TX89",
		Name:      "Ganze Ranch Airstrip",
		City:      "Garland",
		State:     "Texas",
		Country:   "US",
		Elevation: 500,
		Latitude:  32.9697990417,
		Longitude: -96.6224975586,
		Timezone:  "America/Chicago",
	},
	"TX91": {
		ICAO:      "TX91",
		Name:      "Madeira Airpark",
		City:      "Garland",
		State:     "Texas",
		Country:   "US",
		Elevation: 450,
		Latitude:  32.9076004028,
		Longitude: -96.5969009399,
		Timezone:  "America/Chicago",
	},
	"TX92": {
		ICAO:      "TX92",
		Name:      "Green Airport",
		City:      "Georgetown",
		State:     "Texas",
		Country:   "US",
		Elevation: 850,
		Latitude:  30.5960006714,
		Longitude: -97.6694030762,
		Timezone:  "America/Chicago",
	},
	"TX93": {
		ICAO:      "TX93",
		Name:      "Wright Ranch Airport",
		City:      "Glen Rose",
		State:     "Texas",
		Country:   "US",
		Elevation: 900,
		Latitude:  32.1668014526,
		Longitude: -97.8336029053,
		Timezone:  "America/Chicago",
	},
	"TX94": {
		ICAO:      "TX94",
		Name:      "Flying R Ranch Airport",
		City:      "Era",
		State:     "Texas",
		Country:   "US",
		Elevation: 880,
		Latitude:  33.4528999329,
		Longitude: -97.2767028809,
		Timezone:  "America/Chicago",
	},
	"TX95": {
		ICAO:      "TX95",
		Name:      "Coppenger Farm Airport",
		City:      "Godley",
		State:     "Texas",
		Country:   "US",
		Elevation: 1010,
		Latitude:  32.4140014648,
		Longitude: -97.5322036743,
		Timezone:  "America/Chicago",
	},
	"TX96": {
		ICAO:      "TX96",
		Name:      "Maxwell Field",
		City:      "Sanger",
		State:     "Texas",
		Country:   "US",
		Elevation: 825,
		Latitude:  33.3992996216,
		Longitude: -97.2570037842,
		Timezone:  "America/Chicago",
	},
	"TX97": {
		ICAO:      "TX97",
		Name:      "Cade Field",
		City:      "Cranfills Gap",
		State:     "Texas",
		Country:   "US",
		Elevation: 1160,
		Latitude:  31.7649002075,
		Longitude: -97.7964019775,
		Timezone:  "America/Chicago",
	},
	"TX98": {
		ICAO:      "TX98",
		Name:      "Hawkins Private Airport",
		City:      "Godley",
		State:     "Texas",
		Country:   "US",
		Elevation: 975,
		Latitude:  32.4751014709,
		Longitude: -97.5009002686,
		Timezone:  "America/Chicago",
	},
	"TX99": {
		ICAO:      "TX99",
		Name:      "Williams Field",
		City:      "Goodland",
		State:     "Texas",
		Country:   "US",
		Elevation: 4085,
		Latitude:  34.0068016052,
		Longitude: -102.9869995117,
		Timezone:  "America/Chicago",
	},
	"TXKF": {
		ICAO:      "TXKF",
		IATA:      "BDA",
		Name:      "L.F. Wade International International Airport",
		City:      "Hamilton",
		State:     "Saint-Georgeʼs",
		Country:   "BM",
		Elevation: 12,
		Latitude:  32.3639984131,
		Longitude: -64.6787033081,
		Timezone:  "Atlantic/Bermuda",
	},
	"UA34": {
		ICAO:      "UA34",
		Name:      "Chardara Airport",
		City:      "Shardara",
		State:     "Ongtuestik-Qazaqstan",
		Country:   "KZ",
		Elevation: 709,
		Latitude:  41.2765007019,
		Longitude: 67.9710006714,
		Timezone:  "Asia/Almaty",
	},
	"UA36": {
		ICAO:      "UA36",
		Name:      "Lugovoy Airport",
		City:      "Lugovoy",
		State:     "Zhambyl",
		Country:   "KZ",
		Elevation: 2130,
		Latitude:  42.9735984802,
		Longitude: 72.7264022827,
		Timezone:  "Asia/Almaty",
	},
	"UA66": {
		ICAO:      "UA66",
		Name:      "Chirchik Airport",
		City:      "Chirchik",
		State:     "Toshkent",
		Country:   "UZ",
		Elevation: 2296,
		Latitude:  41.5167999268,
		Longitude: 69.5754013062,
		Timezone:  "Asia/Tashkent",
	},
	"UAAA": {
		ICAO:      "UAAA",
		IATA:      "ALA",
		Name:      "Almaty Airport",
		City:      "Almaty",
		State:     "Almaty-Oblysy",
		Country:   "KZ",
		Elevation: 2234,
		Latitude:  43.3521003723,
		Longitude: 77.0404968262,
		Timezone:  "Asia/Almaty",
	},
	"UAAH": {
		ICAO:      "UAAH",
		IATA:      "BXH",
		Name:      "Balkhash Airport",
		City:      "Balkhash",
		State:     "Qaraghandy",
		Country:   "KZ",
		Elevation: 1446,
		Latitude:  46.8932991028,
		Longitude: 75.0049972534,
		Timezone:  "Asia/Almaty",
	},
	"UAAL": {
		ICAO:      "UAAL",
		IATA:      "USJ",
		Name:      "Usharal Airport",
		City:      "Usharal",
		State:     "Almaty-Oblysy",
		Country:   "KZ",
		Elevation: 1288,
		Latitude:  46.190278043,
		Longitude: 80.831389126,
		Timezone:  "Asia/Almaty",
	},
	"UAAR": {
		ICAO:      "UAAR",
		IATA:      "BXJ",
		Name:      "Boralday Airport",
		City:      "Aima Ata",
		State:     "Almaty-Oblysy",
		Country:   "KZ",
		Elevation: 2313,
		Latitude:  43.3526000977,
		Longitude: 76.8836975098,
		Timezone:  "Asia/Almaty",
	},
	"UAAT": {
		ICAO:      "UAAT",
		IATA:      "TDK",
		Name:      "Taldykorgan Airport",
		City:      "Taldy Kurgan",
		State:     "Almaty-Oblysy",
		Country:   "KZ",
		Elevation: 1925,
		Latitude:  45.1262016296,
		Longitude: 78.4469985962,
		Timezone:  "Asia/Almaty",
	},
	"UACC": {
		ICAO:      "UACC",
		IATA:      "NQZ",
		Name:      "Astana International Airport",
		City:      "Astana",
		State:     "Aqmola",
		Country:   "KZ",
		Elevation: 1165,
		Latitude:  51.0222015381,
		Longitude: 71.4669036865,
		Timezone:  "Asia/Almaty",
	},
	"UACK": {
		ICAO:      "UACK",
		IATA:      "KOV",
		Name:      "Kokshetau Airport",
		City:      "Kokshetau",
		State:     "Aqmola",
		Country:   "KZ",
		Elevation: 900,
		Latitude:  53.3291015625,
		Longitude: 69.5945968628,
		Timezone:  "Asia/Almaty",
	},
	"UACP": {
		ICAO:      "UACP",
		IATA:      "PPK",
		Name:      "Petropavlosk South Airport",
		City:      "Petropavlosk",
		State:     "Soltuestik-Qazaqstan",
		Country:   "KZ",
		Elevation: 453,
		Latitude:  54.7747001648,
		Longitude: 69.1838989258,
		Timezone:  "Asia/Almaty",
	},
	"UADD": {
		ICAO:      "UADD",
		IATA:      "DMB",
		Name:      "Taraz Airport",
		City:      "Taraz",
		State:     "Zhambyl",
		Country:   "KZ",
		Elevation: 2184,
		Latitude:  42.8535995483,
		Longitude: 71.3035964966,
		Timezone:  "Asia/Almaty",
	},
	"UAFA": {
		ICAO:      "UAFA",
		Name:      "Tamga Airport",
		City:      "Tamga",
		State:     "Issyk-Kul",
		Country:   "KG",
		Elevation: 5655,
		Latitude:  42.1530990601,
		Longitude: 77.5639038086,
		Timezone:  "Asia/Bishkek",
	},
	"UAFB": {
		ICAO:      "UAFB",
		Name:      "Batken Airport",
		City:      "Batken",
		State:     "Batken",
		Country:   "KG",
		Elevation: 3517,
		Latitude:  40.0427017212,
		Longitude: 70.8380966187,
		Timezone:  "Asia/Bishkek",
	},
	"UAFE": {
		ICAO:      "UAFE",
		Name:      "Kerben Airport",
		City:      "Kerben",
		State:     "Jalal-Abad",
		Country:   "KG",
		Elevation: 4199,
		Latitude:  41.484500885,
		Longitude: 71.7337036133,
		Timezone:  "Asia/Bishkek",
	},
	"UAFF": {
		ICAO:      "UAFF",
		Name:      "Tokmok Airport",
		City:      "Tokmok",
		State:     "Chuey",
		Country:   "KG",
		Elevation: 2770,
		Latitude:  42.8288993835,
		Longitude: 75.3359985352,
		Timezone:  "Asia/Bishkek",
	},
	"UAFG": {
		ICAO:      "UAFG",
		Name:      "Cholpon-Ata Airport",
		City:      "Cholpon-Ata",
		State:     "Issyk-Kul",
		Country:   "KG",
		Elevation: 5420,
		Latitude:  42.6537017822,
		Longitude: 77.0567016602,
		Timezone:  "Asia/Bishkek",
	},
	"UAFI": {
		ICAO:      "UAFI",
		Name:      "Isfana Airport",
		City:      "Isfana",
		State:     "Batken",
		Country:   "KG",
		Elevation: 4625,
		Latitude:  39.8247374413,
		Longitude: 69.5688199997,
		Timezone:  "Asia/Bishkek",
	},
	"UAFJ": {
		ICAO:      "UAFJ",
		Name:      "Jalal-Abad Airport",
		City:      "Jalal-Abad",
		State:     "Jalal-Abad",
		Country:   "KG",
		Elevation: 2591,
		Latitude:  40.9444007874,
		Longitude: 72.9777984619,
		Timezone:  "Asia/Bishkek",
	},
	"UAFN": {
		ICAO:      "UAFN",
		Name:      "Naryn Airport",
		City:      "Naryn",
		State:     "Naryn",
		Country:   "KG",
		Elevation: 6998,
		Latitude:  41.4415016174,
		Longitude: 76.1305999756,
		Timezone:  "Asia/Bishkek",
	},
	"UAFR": {
		ICAO:      "UAFR",
		Name:      "Balykchy Airport",
		City:      "Balykchy",
		State:     "Issyk-Kul",
		Country:   "KG",
		Elevation: 5326,
		Latitude:  42.4374008179,
		Longitude: 76.1307983398,
		Timezone:  "Asia/Bishkek",
	},
	"UAFS": {
		ICAO:      "UAFS",
		Name:      "Kyzyl-Kiya Airport",
		City:      "Kyzyl-Kiya",
		State:     "Fergana",
		Country:   "KG",
		Elevation: 2928,
		Latitude:  40.2717018127,
		Longitude: 72.0470962524,
		Timezone:  "Asia/Bishkek",
	},
	"UAFT": {
		ICAO:      "UAFT",
		Name:      "Talas Airport",
		City:      "Talas",
		State:     "Talas",
		Country:   "KG",
		Elevation: 4153,
		Latitude:  42.5059013367,
		Longitude: 72.2630996704,
		Timezone:  "Asia/Bishkek",
	},
	"UAFW": {
		ICAO:      "UAFW",
		Name:      "Kant Air Base",
		City:      "Kant",
		State:     "Chuey",
		Country:   "KG",
		Elevation: 2549,
		Latitude:  42.8531990051,
		Longitude: 74.846496582,
		Timezone:  "Asia/Bishkek",
	},
	"UAFZ": {
		ICAO:      "UAFZ",
		Name:      "Kazarman Airport",
		City:      "Kazarman",
		State:     "Jalal-Abad",
		Country:   "KG",
		Elevation: 4298,
		Latitude:  41.4102908129,
		Longitude: 74.044418335,
		Timezone:  "Asia/Bishkek",
	},
	"UAII": {
		ICAO:      "UAII",
		IATA:      "CIT",
		Name:      "Shymkent Airport",
		City:      "Shymkent",
		State:     "Ongtuestik-Qazaqstan",
		Country:   "KZ",
		Elevation: 1385,
		Latitude:  42.364200592,
		Longitude: 69.4788970947,
		Timezone:  "Asia/Almaty",
	},
	"UAIT": {
		ICAO:      "UAIT",
		IATA:      "HSA",
		Name:      "Hazrat Sultan International Airport",
		City:      "Turkistan",
		State:     "Ongtuestik-Qazaqstan",
		Country:   "KZ",
		Elevation: 989,
		Latitude:  43.2769012451,
		Longitude: 68.1903991699,
		Timezone:  "Asia/Almaty",
	},
	"UAKD": {
		ICAO:      "UAKD",
		IATA:      "DZN",
		Name:      "Zhezkazgan Airport",
		City:      "Zhezkazgan",
		State:     "Qaraghandy",
		Country:   "KZ",
		Elevation: 1250,
		Latitude:  47.7083015442,
		Longitude: 67.7332992554,
		Timezone:  "Asia/Almaty",
	},
	"UAKK": {
		ICAO:      "UAKK",
		IATA:      "KGF",
		Name:      "Sary-Arka Airport",
		City:      "Karaganda",
		State:     "Qaraghandy",
		Country:   "KZ",
		Elevation: 1765,
		Latitude:  49.6707992554,
		Longitude: 73.3343963623,
		Timezone:  "Asia/Almaty",
	},
	"UAOL": {
		ICAO:      "UAOL",
		IATA:      "BXY",
		Name:      "Krainiy Airport",
		City:      "Baikonur",
		State:     "Baikonur",
		Country:   "KZ",
		Elevation: 299,
		Latitude:  45.6220016,
		Longitude: 63.2150002,
		Timezone:  "Asia/Qyzylorda",
	},
	"UAON": {
		ICAO:      "UAON",
		Name:      "Yubileyniy Airfield",
		City:      "Baikonur",
		State:     "Baikonur",
		Country:   "KZ",
		Elevation: 328,
		Latitude:  46.0550003052,
		Longitude: 63.25,
		Timezone:  "Asia/Qyzylorda",
	},
	"UAOO": {
		ICAO:      "UAOO",
		IATA:      "KZO",
		Name:      "Kzyl-Orda Southwest Airport",
		City:      "Kzyl-Orda",
		State:     "Qyzylorda",
		Country:   "KZ",
		Elevation: 433,
		Latitude:  44.7069015503,
		Longitude: 65.5924987793,
		Timezone:  "Asia/Qyzylorda",
	},
	"UARR": {
		ICAO:      "UARR",
		IATA:      "URA",
		Name:      "Uralsk Airport",
		City:      "Uralsk",
		Country:   "KZ",
		Elevation: 125,
		Latitude:  51.1507987976,
		Longitude: 51.5430984497,
		Timezone:  "Asia/Oral",
	},
	"UASA": {
		ICAO:      "UASA",
		Name:      "Ayaguz Airport",
		City:      "Ayaguz",
		State:     "East-Kazakhstan",
		Country:   "KZ",
		Elevation: 2119,
		Latitude:  47.918598175,
		Longitude: 80.452796936,
		Timezone:  "Asia/Almaty",
	},
	"UASB": {
		ICAO:      "UASB",
		IATA:      "EKB",
		Name:      "Ekibastuz Airport",
		City:      "Ekibastuz",
		State:     "Pavlodar",
		Country:   "KZ",
		Elevation: 621,
		Latitude:  51.5909996033,
		Longitude: 75.2149963379,
		Timezone:  "Asia/Almaty",
	},
	"UASK": {
		ICAO:      "UASK",
		IATA:      "UKK",
		Name:      "Ust-Kamennogorsk Airport",
		City:      "Ust Kamenogorsk",
		State:     "East-Kazakhstan",
		Country:   "KZ",
		Elevation: 939,
		Latitude:  50.0365982056,
		Longitude: 82.4942016602,
		Timezone:  "Asia/Almaty",
	},
	"UASP": {
		ICAO:      "UASP",
		IATA:      "PWQ",
		Name:      "Pavlodar Airport",
		City:      "Pavlodar",
		State:     "Pavlodar",
		Country:   "KZ",
		Elevation: 410,
		Latitude:  52.1949996948,
		Longitude: 77.0738983154,
		Timezone:  "Asia/Almaty",
	},
	"UASS": {
		ICAO:      "UASS",
		IATA:      "PLX",
		Name:      "Semey Airport",
		City:      "Semey",
		State:     "East-Kazakhstan",
		Country:   "KZ",
		Elevation: 761,
		Latitude:  50.351389,
		Longitude: 80.234444,
		Timezone:  "Asia/Almaty",
	},
	"UATA": {
		ICAO:      "UATA",
		Name:      "Aralsk Airport",
		City:      "Aralsk",
		State:     "Qyzylorda",
		Country:   "KZ",
		Elevation: 223,
		Latitude:  46.8330001831,
		Longitude: 61.6170005798,
		Timezone:  "Asia/Qyzylorda",
	},
	"UATE": {
		ICAO:      "UATE",
		IATA:      "SCO",
		Name:      "Aktau Airport",
		City:      "Aktau",
		State:     "Mangghystau",
		Country:   "KZ",
		Elevation: 73,
		Latitude:  43.8600997925,
		Longitude: 51.091999054,
		Timezone:  "Asia/Aqtau",
	},
	"UATG": {
		ICAO:      "UATG",
		IATA:      "GUW",
		Name:      "Atyrau Airport",
		City:      "Atyrau",
		State:     "Atyrau",
		Country:   "KZ",
		Elevation: -72,
		Latitude:  47.1218986511,
		Longitude: 51.8213996887,
		Timezone:  "Asia/Atyrau",
	},
	"UATR": {
		ICAO:      "UATR",
		Name:      "Chelkar Airport",
		City:      "Chelkar",
		State:     "Aqtobe",
		Country:   "KZ",
		Latitude:  47.9049987793,
		Longitude: 59.6199989319,
		Timezone:  "Asia/Aqtobe",
	},
	"UATT": {
		ICAO:      "UATT",
		IATA:      "AKX",
		Name:      "Aktobe Airport",
		City:      "Aktyubinsk",
		State:     "Aqtobe",
		Country:   "KZ",
		Elevation: 738,
		Latitude:  50.2458000183,
		Longitude: 57.2066993713,
		Timezone:  "Asia/Aqtobe",
	},
	"UAUR": {
		ICAO:      "UAUR",
		IATA:      "AYK",
		Name:      "Arkalyk North Airport",
		City:      "Arkalyk",
		Country:   "KZ",
		Elevation: 1266,
		Latitude:  50.3185997009,
		Longitude: 66.952796936,
		Timezone:  "Asia/Qostanay",
	},
	"UAUU": {
		ICAO:      "UAUU",
		IATA:      "KSN",
		Name:      "Kostanay West Airport",
		City:      "Kostanay",
		State:     "Qostanay",
		Country:   "KZ",
		Elevation: 595,
		Latitude:  53.2069015503,
		Longitude: 63.5503005981,
		Timezone:  "Asia/Qostanay",
	},
	"UBBA": {
		ICAO:      "UBBA",
		Name:      "Akstafa Airport",
		City:      "Akstafa",
		State:     "Agstafa",
		Country:   "AZ",
		Elevation: 1085,
		Latitude:  41.1223983765,
		Longitude: 45.4248008728,
		Timezone:  "Asia/Baku",
	},
	"UBBB": {
		ICAO:      "UBBB",
		IATA:      "GYD",
		Name:      "Heydar Aliyev International Airport",
		City:      "Baku",
		State:     "Baki",
		Country:   "AZ",
		Elevation: 10,
		Latitude:  40.4674987793,
		Longitude: 50.0466995239,
		Timezone:  "Asia/Baku",
	},
	"UBBG": {
		ICAO:      "UBBG",
		IATA:      "GNJ",
		Name:      "Ganja Airport",
		City:      "Ganja",
		State:     "Goygol-Rayon",
		Country:   "AZ",
		Elevation: 1083,
		Latitude:  40.737701416,
		Longitude: 46.3176002502,
		Timezone:  "Asia/Baku",
	},
	"UBBL": {
		ICAO:      "UBBL",
		IATA:      "LLK",
		Name:      "Lankaran International Airport",
		City:      "Lankaran",
		State:     "Lənkəran",
		Country:   "AZ",
		Elevation: 30,
		Latitude:  38.7463989258,
		Longitude: 48.8180007935,
		Timezone:  "Asia/Baku",
	},
	"UBBN": {
		ICAO:      "UBBN",
		IATA:      "NAJ",
		Name:      "Nakhchivan Airport",
		City:      "Nakhchivan",
		State:     "Nakhichevan",
		Country:   "AZ",
		Elevation: 2863,
		Latitude:  39.1888008118,
		Longitude: 45.4584007263,
		Timezone:  "Asia/Baku",
	},
	"UBBQ": {
		ICAO:      "UBBQ",
		IATA:      "GBB",
		Name:      "Gabala International Airport",
		City:      "Gabala",
		State:     "Qǝbǝlǝ",
		Country:   "AZ",
		Elevation: 935,
		Latitude:  40.8266666667,
		Longitude: 47.7125,
		Timezone:  "Asia/Baku",
	},
	"UBBY": {
		ICAO:      "UBBY",
		IATA:      "ZTU",
		Name:      "Zaqatala International Airport",
		City:      "Zaqatala",
		State:     "Zaqatala",
		Country:   "AZ",
		Elevation: 1279,
		Latitude:  41.5622215271,
		Longitude: 46.6672210693,
		Timezone:  "Asia/Baku",
	},
	"UBEE": {
		ICAO:      "UBEE",
		IATA:      "YLV",
		Name:      "Yevlakh Airport",
		City:      "Yevlakh",
		State:     "Yevlax-City",
		Country:   "AZ",
		Elevation: 49,
		Latitude:  40.6319007874,
		Longitude: 47.1418991089,
		Timezone:  "Asia/Baku",
	},
	"UCFL": {
		ICAO:      "UCFL",
		IATA:      "IKU",
		Name:      "Issyk-Kul International Airport",
		City:      "Tamchy",
		State:     "Issyk-Kul",
		Country:   "KG",
		Elevation: 5426,
		Latitude:  42.585714,
		Longitude: 76.701811,
		Timezone:  "Asia/Bishkek",
	},
	"UCFM": {
		ICAO:      "UCFM",
		IATA:      "FRU",
		Name:      "Manas International Airport",
		City:      "Bishkek",
		State:     "Chuey",
		Country:   "KG",
		Elevation: 2090,
		Latitude:  43.061272,
		Longitude: 74.477508,
		Timezone:  "Asia/Bishkek",
	},
	"UCFO": {
		ICAO:      "UAFO",
		IATA:      "OSS",
		Name:      "Osh Airport",
		City:      "Osh",
		State:     "Osh",
		Country:   "KG",
		Elevation: 2938,
		Latitude:  40.608997,
		Longitude: 72.793214,
		Timezone:  "Asia/Bishkek",
	},
	"UCFP": {
		ICAO:      "UAFP",
		Name:      "Karakol Airport",
		City:      "Karakol",
		State:     "Issyk-Kul",
		Country:   "KG",
		Elevation: 5590,
		Latitude:  42.508333,
		Longitude: 78.407778,
		Timezone:  "Asia/Bishkek",
	},
	"UD21": {
		ICAO:      "UD21",
		Name:      "Yerevan Yegvard Airport",
		City:      "Yerevan",
		State:     "Kotayk",
		Country:   "AM",
		Elevation: 4416,
		Latitude:  40.2941017151,
		Longitude: 44.5646018982,
		Timezone:  "Asia/Yerevan",
	},
	"UDLS": {
		ICAO:      "UDLS",
		Name:      "Stepanavan Airport",
		City:      "Stepanavan",
		State:     "Lori",
		Country:   "AM",
		Elevation: 4836,
		Latitude:  41.048500061,
		Longitude: 44.3372001648,
		Timezone:  "Asia/Yerevan",
	},
	"UDSG": {
		ICAO:      "UDSG",
		IATA:      "LWN",
		Name:      "Gyumri Shirak Airport",
		City:      "Gyumri",
		State:     "Shirak",
		Country:   "AM",
		Elevation: 5000,
		Latitude:  40.7504005432,
		Longitude: 43.8592987061,
		Timezone:  "Asia/Yerevan",
	},
	"UDYE": {
		ICAO:      "UDYE",
		Name:      "Erebuni Airport",
		City:      "Yerevan",
		State:     "Yerevan",
		Country:   "AM",
		Elevation: 2948,
		Latitude:  40.1221008301,
		Longitude: 44.4650001526,
		Timezone:  "Asia/Yerevan",
	},
	"UDYZ": {
		ICAO:      "UDYZ",
		IATA:      "EVN",
		Name:      "Zvartnots International Airport",
		City:      "Yerevan",
		Country:   "AM",
		Elevation: 2838,
		Latitude:  40.1473007202,
		Longitude: 44.3959007263,
		Timezone:  "Asia/Yerevan",
	},
	"UEBB": {
		ICAO:      "UEBB",
		Name:      "Batagay Airport",
		City:      "Batagay",
		State:     "Chukot",
		Country:   "RU",
		Elevation: 696,
		Latitude:  67.6480026245,
		Longitude: 134.6950073242,
		Timezone:  "Asia/Vladivostok",
	},
	"UEBN": {
		ICAO:      "UEBN",
		Name:      "Nizhneyansk Airport",
		City:      "Nizhneyansk",
		State:     "Chukot",
		Country:   "RU",
		Elevation: 20,
		Latitude:  71.43699646,
		Longitude: 136.1900024414,
		Timezone:  "Asia/Vladivostok",
	},
	"UEBT": {
		ICAO:      "UEBT",
		IATA:      "UKG",
		Name:      "Ust-Kuyga Airport",
		City:      "Ust-Kuyga",
		State:     "Chukot",
		Country:   "RU",
		Latitude:  70.0110015869,
		Longitude: 135.6450042725,
		Timezone:  "Asia/Vladivostok",
	},
	"UEBW": {
		ICAO:      "UEBW",
		Name:      "Verkhoyansk Airport",
		State:     "Chukot",
		Country:   "RU",
		Latitude:  67.5430984497,
		Longitude: 133.3988037109,
		Timezone:  "Asia/Vladivostok",
	},
	"UECT": {
		ICAO:      "UECT",
		IATA:      "TLK",
		Name:      "Talakan Airport",
		State:     "Chukot",
		Country:   "RU",
		Elevation: 1329,
		Latitude:  59.876389,
		Longitude: 111.044444,
		Timezone:  "Asia/Yakutsk",
	},
	"UEEA": {
		ICAO:      "UEEA",
		IATA:      "ADH",
		Name:      "Aldan Airport",
		City:      "Aldan",
		State:     "Chukot",
		Country:   "RU",
		Elevation: 2241,
		Latitude:  58.6027984619,
		Longitude: 125.408996582,
		Timezone:  "Asia/Yakutsk",
	},
	"UEEE": {
		ICAO:      "UEEE",
		IATA:      "YKS",
		Name:      "Yakutsk Airport",
		City:      "Yakutsk",
		State:     "Chukot",
		Country:   "RU",
		Elevation: 325,
		Latitude:  62.0932998657,
		Longitude: 129.7709960938,
		Timezone:  "Asia/Yakutsk",
	},
	"UELL": {
		ICAO:      "UELL",
		IATA:      "CNN",
		Name:      "Chulman Airport",
		City:      "Chulman",
		State:     "Chukot",
		Country:   "RU",
		Elevation: 2812,
		Latitude:  56.913898468,
		Longitude: 124.9140014648,
		Timezone:  "Asia/Yakutsk",
	},
	"UEMA": {
		ICAO:      "UEMA",
		IATA:      "MQJ",
		Name:      "Moma Airport",
		City:      "Honuu",
		State:     "Chukot",
		Country:   "RU",
		Elevation: 656,
		Latitude:  66.4508590698,
		Longitude: 143.2615509033,
		Timezone:  "Asia/Srednekolymsk",
	},
	"UEMH": {
		ICAO:      "UEMH",
		IATA:      "KDY",
		Name:      "Tyopliy Klyuch Airport",
		City:      "Tyopliy Klyuch",
		State:     "Chukot",
		Country:   "RU",
		Latitude:  62.7890014648,
		Longitude: 136.8549957275,
		Timezone:  "Asia/Khandyga",
	},
	"UEMM": {
		ICAO:      "UEMM",
		IATA:      "GYG",
		Name:      "Magan Airport",
		City:      "Magan",
		State:     "Chukot",
		Country:   "RU",
		Elevation: 577,
		Latitude:  62.1034812927,
		Longitude: 129.5452880859,
		Timezone:  "Asia/Yakutsk",
	},
	"UEMO": {
		ICAO:      "UEMO",
		IATA:      "OLZ",
		Name:      "Olyokminsk Airport",
		City:      "Olyokminsk",
		State:     "Chukot",
		Country:   "RU",
		Elevation: 656,
		Latitude:  60.3974990845,
		Longitude: 120.471000671,
		Timezone:  "Asia/Yakutsk",
	},
	"UEMT": {
		ICAO:      "UEMT",
		IATA:      "USR",
		Name:      "Ust-Nera Airport",
		City:      "Ust-Nera",
		State:     "Chukot",
		Country:   "RU",
		Latitude:  64.5500030518,
		Longitude: 143.1150054932,
		Timezone:  "Asia/Ust-Nera",
	},
	"UEMU": {
		ICAO:      "UEMU",
		IATA:      "UMS",
		Name:      "Ust-Maya Airport",
		City:      "Ust-Maya",
		State:     "Chukot",
		Country:   "RU",
		Elevation: 561,
		Latitude:  60.3569984436,
		Longitude: 134.4349975586,
		Timezone:  "Asia/Khandyga",
	},
	"UENI": {
		ICAO:      "UENI",
		IATA:      "VHV",
		Name:      "Verkhnevilyuisk Airport",
		City:      "Verkhnevilyuisk",
		State:     "Chukot",
		Country:   "RU",
		Latitude:  63.4580574036,
		Longitude: 120.2691650391,
		Timezone:  "Asia/Yakutsk",
	},
	"UENK": {
		ICAO:      "UENK",
		Name:      "Kyzyl-Syr Airport",
		City:      "Kyzyl-Syr",
		State:     "Chukot",
		Country:   "RU",
		Elevation: 331,
		Latitude:  63.8849983215,
		Longitude: 122.7770004272,
		Timezone:  "Asia/Yakutsk",
	},
	"UENN": {
		ICAO:      "UENN",
		IATA:      "NYR",
		Name:      "Nyurba Airport",
		City:      "Nyurba",
		State:     "Chukot",
		Country:   "RU",
		Elevation: 394,
		Latitude:  63.2949981689,
		Longitude: 118.3369979858,
		Timezone:  "Asia/Yakutsk",
	},
	"UENS": {
		ICAO:      "UENS",
		IATA:      "SUY",
		Name:      "Suntar Airport",
		City:      "Suntar",
		State:     "Chukot",
		Country:   "RU",
		Latitude:  62.1850013733,
		Longitude: 117.6350021362,
		Timezone:  "Asia/Yakutsk",
	},
	"UENW": {
		ICAO:      "UENW",
		IATA:      "VYI",
		Name:      "Vilyuisk Airport",
		City:      "Vilyuisk",
		State:     "Chukot",
		Country:   "RU",
		Elevation: 361,
		Latitude:  63.7566680908,
		Longitude: 121.6933364868,
		Timezone:  "Asia/Yakutsk",
	},
	"UERA": {
		ICAO:      "UERA",
		Name:      "Aykhal Airport",
		State:     "Chukot",
		Country:   "RU",
		Latitude:  65.959197998,
		Longitude: 111.5465011597,
		Timezone:  "Asia/Yakutsk",
	},
	"UERL": {
		ICAO:      "UERL",
		IATA:      "ULK",
		Name:      "Lensk Airport",
		City:      "Lensk",
		State:     "Chukot",
		Country:   "RU",
		Elevation: 801,
		Latitude:  60.7206001282,
		Longitude: 114.825996399,
		Timezone:  "Asia/Yakutsk",
	},
	"UERO": {
		ICAO:      "UERO",
		IATA:      "ONK",
		Name:      "Olenyok Airport",
		City:      "Olenyok",
		State:     "Chukot",
		Country:   "RU",
		Elevation: 847,
		Latitude:  68.5149993896,
		Longitude: 112.4800033569,
		Timezone:  "Asia/Yakutsk",
	},
	"UERP": {
		ICAO:      "UERP",
		IATA:      "PYJ",
		Name:      "Polyarny Airport",
		City:      "Yakutia",
		State:     "Chukot",
		Country:   "RU",
		Elevation: 1660,
		Latitude:  66.4003982544,
		Longitude: 112.029998779,
		Timezone:  "Asia/Yakutsk",
	},
	"UERR": {
		ICAO:      "UERR",
		IATA:      "MJZ",
		Name:      "Mirny Airport",
		City:      "Mirny",
		State:     "Chukot",
		Country:   "RU",
		Elevation: 1156,
		Latitude:  62.5346984863,
		Longitude: 114.0390014648,
		Timezone:  "Asia/Yakutsk",
	},
	"UERS": {
		ICAO:      "UERS",
		IATA:      "SYS",
		Name:      "Saskylakh Airport",
		City:      "Saskylakh",
		State:     "Chukot",
		Country:   "RU",
		Latitude:  71.9279022217,
		Longitude: 114.0800018311,
		Timezone:  "Asia/Yakutsk",
	},
	"UERT": {
		ICAO:      "UERT",
		Name:      "Vitim Airport",
		City:      "Vitim",
		State:     "Chukot",
		Country:   "RU",
		Elevation: 610,
		Latitude:  59.4580001831,
		Longitude: 112.56300354,
		Timezone:  "Asia/Yakutsk",
	},
	"UESG": {
		ICAO:      "UESG",
		IATA:      "BGN",
		Name:      "Belaya Gora Airport",
		City:      "Belaya Gora",
		State:     "Sakha Republic",
		Country:   "RU",
		Elevation: 82,
		Latitude:  68.556944,
		Longitude: 146.234722,
		Timezone:  "Asia/Magadan",
	},
	"UESO": {
		ICAO:      "UESO",
		IATA:      "CKH",
		Name:      "Chokurdakh Airport",
		City:      "Chokurdah",
		State:     "Chukot",
		Country:   "RU",
		Elevation: 151,
		Latitude:  70.6231002808,
		Longitude: 147.9019927979,
		Timezone:  "Asia/Srednekolymsk",
	},
	"UESS": {
		ICAO:      "UESS",
		IATA:      "CYX",
		Name:      "Cherskiy Airport",
		City:      "Cherskiy",
		State:     "Chukot",
		Country:   "RU",
		Elevation: 20,
		Latitude:  68.7406005859,
		Longitude: 161.3379974365,
		Timezone:  "Asia/Srednekolymsk",
	},
	"UEST": {
		ICAO:      "UEST",
		IATA:      "IKS",
		Name:      "Tiksi Airport",
		City:      "Tiksi",
		State:     "Chukot",
		Country:   "RU",
		Elevation: 26,
		Latitude:  71.6977005005,
		Longitude: 128.9029998779,
		Timezone:  "Asia/Yakutsk",
	},
	"UESU": {
		ICAO:      "UESU",
		IATA:      "ZKP",
		Name:      "Zyryanka Airport",
		City:      "Zyryanka",
		State:     "Chukot",
		Country:   "RU",
		Elevation: 181,
		Latitude:  65.7367019653,
		Longitude: 150.705001831,
		Timezone:  "Asia/Srednekolymsk",
	},
	"UEVV": {
		ICAO:      "UEVV",
		IATA:      "ZIX",
		Name:      "Zhigansk Airport",
		City:      "Zhigansk",
		State:     "Chukot",
		Country:   "RU",
		Elevation: 292,
		Latitude:  66.7965011597,
		Longitude: 123.361000061,
		Timezone:  "Asia/Yakutsk",
	},
	"UEWD": {
		ICAO:      "UEWD",
		Name:      "Deputatskiy Airport",
		City:      "Deputatskiy",
		State:     "Chukot",
		Country:   "RU",
		Latitude:  69.3926010132,
		Longitude: 139.8903045654,
		Timezone:  "Asia/Vladivostok",
	},
	"UG0U": {
		ICAO:      "UG0U",
		Name:      "Gudauta Air Base",
		City:      "Gudauta",
		Country:   "GE",
		Elevation: 79,
		Latitude:  43.1040000916,
		Longitude: 40.5792999268,
		Timezone:  "Asia/Tbilisi",
	},
	"UGEJ": {
		ICAO:      "UGEJ",
		Name:      "Dzhermuk Airport",
		City:      "Dzhermuk",
		State:     "Vayots-Dzor",
		Country:   "AM",
		Latitude:  39.8240013123,
		Longitude: 45.6739997864,
		Timezone:  "Asia/Yerevan",
	},
	"UGKO": {
		ICAO:      "UGKO",
		IATA:      "KUT",
		Name:      "Kopitnari Airport",
		City:      "Kutaisi",
		Country:   "GE",
		Elevation: 223,
		Latitude:  42.176700592,
		Longitude: 42.4826011658,
		Timezone:  "Asia/Tbilisi",
	},
	"UGMM": {
		ICAO:      "UGMM",
		Name:      "Mukhrani Airport",
		State:     "Shida-Kartli",
		Country:   "GE",
		Elevation: 1807,
		Latitude:  41.916,
		Longitude: 44.566,
		Timezone:  "Asia/Tbilisi",
	},
	"UGSB": {
		ICAO:      "UGSB",
		IATA:      "BUS",
		Name:      "Batumi International Airport",
		City:      "Batumi",
		State:     "Ajaria",
		Country:   "GE",
		Elevation: 105,
		Latitude:  41.6102981567,
		Longitude: 41.5997009277,
		Timezone:  "Asia/Tbilisi",
	},
	"UGSP": {
		ICAO:      "UGSP",
		Name:      "Poti East Airport",
		City:      "Poti",
		State:     "Samegrelo-and-Zemo-Svaneti",
		Country:   "GE",
		Elevation: 3,
		Latitude:  42.1629024102,
		Longitude: 41.7058074474,
		Timezone:  "Asia/Tbilisi",
	},
	"UGSS": {
		ICAO:      "UGSS",
		IATA:      "SUI",
		Name:      "Sukhumi Dranda Airport",
		City:      "Sukhumi",
		State:     "Abkhazia",
		Country:   "GE",
		Elevation: 53,
		Latitude:  42.8582000732,
		Longitude: 41.1281013489,
		Timezone:  "Asia/Tbilisi",
	},
	"UGTB": {
		ICAO:      "UGTB",
		IATA:      "TBS",
		Name:      "Tbilisi International Airport",
		City:      "Tbilisi",
		Country:   "GE",
		Elevation: 1624,
		Latitude:  41.6692008972,
		Longitude: 44.95470047,
		Timezone:  "Asia/Tbilisi",
	},
	"UHB1": {
		ICAO:      "UHB1",
		Name:      "Birofeld Air Base",
		City:      "Birofeld",
		State:     "Jewish-Autonomous-Oblast",
		Country:   "RU",
		Latitude:  48.4599990845,
		Longitude: 132.6600036621,
		Timezone:  "Asia/Vladivostok",
	},
	"UHBB": {
		ICAO:      "UHBB",
		IATA:      "BQS",
		Name:      "Ignatyevo Airport",
		City:      "Blagoveschensk",
		State:     "Amur",
		Country:   "RU",
		Elevation: 638,
		Latitude:  50.4253997803,
		Longitude: 127.4120025635,
		Timezone:  "Asia/Yakutsk",
	},
	"UHBE": {
		ICAO:      "UHBE",
		Name:      "Zeya Airport",
		City:      "Zeya",
		State:     "Amur",
		Country:   "RU",
		Latitude:  53.6904983521,
		Longitude: 127.0907974243,
		Timezone:  "Asia/Yakutsk",
	},
	"UHBI": {
		ICAO:      "UHBI",
		IATA:      "GDG",
		Name:      "Magdagachi Airport",
		City:      "Magdagachi",
		State:     "Amur",
		Country:   "RU",
		Latitude:  53.4733009338,
		Longitude: 125.7949981689,
		Timezone:  "Asia/Yakutsk",
	},
	"UHBP": {
		ICAO:      "UHBP",
		Name:      "Ekimchan Airport",
		City:      "Ekimchan",
		State:     "Amur",
		Country:   "RU",
		Elevation: 1781,
		Latitude:  53.0760993958,
		Longitude: 132.9510040283,
		Timezone:  "Asia/Yakutsk",
	},
	"UHBS": {
		ICAO:      "UHBS",
		Name:      "Svobodny Airport",
		City:      "Svobodny",
		State:     "Amur",
		Country:   "RU",
		Elevation: 600,
		Latitude:  51.4550018311,
		Longitude: 128.0983276367,
		Timezone:  "Asia/Yakutsk",
	},
	"UHBW": {
		ICAO:      "UHBW",
		IATA:      "TYD",
		Name:      "Tynda Airport",
		City:      "Tynda",
		State:     "Amur",
		Country:   "RU",
		Elevation: 2001,
		Latitude:  55.284198761,
		Longitude: 124.7789993286,
		Timezone:  "Asia/Yakutsk",
	},
	"UHHB": {
		ICAO:      "UHHB",
		Name:      "Birobidzhan Yuzhniy Airfield",
		State:     "Jewish-Autonomous-Oblast",
		Country:   "RU",
		Latitude:  48.5820007324,
		Longitude: 133.0429992676,
		Timezone:  "Asia/Vladivostok",
	},
	"UHHH": {
		ICAO:      "UHHH",
		IATA:      "KHV",
		Name:      "Khabarovsk-Novy Airport",
		City:      "Khabarovsk",
		State:     "Khabarovsk",
		Country:   "RU",
		Elevation: 244,
		Latitude:  48.5279998779,
		Longitude: 135.18800354,
		Timezone:  "Asia/Vladivostok",
	},
	"UHHT": {
		ICAO:      "UHHT",
		Name:      "Khabarovsk Airport",
		City:      "Khabarovsk",
		State:     "Khabarovsk",
		Country:   "RU",
		Elevation: 262,
		Latitude:  48.5250015259,
		Longitude: 135.1529998779,
		Timezone:  "Asia/Vladivostok",
	},
	"UHHY": {
		ICAO:      "UHHY",
		Name:      "Chumikan Airport",
		City:      "Chumikan",
		State:     "Khabarovsk",
		Country:   "RU",
		Latitude:  54.6959991455,
		Longitude: 135.2859954834,
		Timezone:  "Asia/Vladivostok",
	},
	"UHK1": {
		ICAO:      "UHK1",
		Name:      "Blagodatnoye Air Base",
		City:      "Blagodatnoye",
		State:     "Khabarovsk",
		Country:   "RU",
		Latitude:  48.4129981995,
		Longitude: 135.4149932861,
		Timezone:  "Asia/Vladivostok",
	},
	"UHK2": {
		ICAO:      "UHK2",
		Name:      "Garovka Air Base",
		City:      "Khabarovsk",
		State:     "Khabarovsk",
		Country:   "RU",
		Latitude:  48.3950004578,
		Longitude: 135.220993042,
		Timezone:  "Asia/Vladivostok",
	},
	"UHKD": {
		ICAO:      "UHKD",
		Name:      "Dzemgi Airport",
		City:      "Komsomolsk-na-Amur",
		Country:   "RU",
		Elevation: 89,
		Latitude:  50.6055984497,
		Longitude: 137.0809936523,
		Timezone:  "Asia/Vladivostok",
	},
	"UHKG": {
		ICAO:      "UHKG",
		Name:      "Kamenny Ruchey Naval Air Base",
		City:      "Sovetskaya Gavan",
		State:     "Khabarovsk",
		Country:   "RU",
		Elevation: 659,
		Latitude:  49.2357025146,
		Longitude: 140.1930999756,
		Timezone:  "Asia/Vladivostok",
	},
	"UHKK": {
		ICAO:      "UHKK",
		IATA:      "KXK",
		Name:      "Komsomolsk-on-Amur Airport",
		City:      "Komsomolsk-on-Amur",
		State:     "Khabarovsk",
		Country:   "RU",
		Elevation: 92,
		Latitude:  50.4090003967,
		Longitude: 136.9340057373,
		Timezone:  "Asia/Vladivostok",
	},
	"UHKM": {
		ICAO:      "UHKM",
		IATA:      "GVN",
		Name:      "Maygatka Airport.",
		City:      "Sovetskaya Gavan",
		State:     "Khabarovsk",
		Country:   "RU",
		Elevation: 768,
		Latitude:  48.9269981384,
		Longitude: 140.033996582,
		Timezone:  "Asia/Vladivostok",
	},
	"UHMA": {
		ICAO:      "UHMA",
		IATA:      "DYR",
		Name:      "Ugolny Airport",
		City:      "Anadyr",
		State:     "Chukotka",
		Country:   "RU",
		Elevation: 194,
		Latitude:  64.7349014282,
		Longitude: 177.7409973145,
		Timezone:  "Asia/Anadyr",
	},
	"UHMD": {
		ICAO:      "UHMD",
		IATA:      "PVS",
		Name:      "Provideniya Bay Airport",
		City:      "Chukotka",
		State:     "Chukotka",
		Country:   "RU",
		Elevation: 72,
		Latitude:  64.3780975342,
		Longitude: -173.2429962158,
		Timezone:  "Asia/Anadyr",
	},
	"UHME": {
		ICAO:      "UHME",
		Name:      "Zaliv Kresta",
		City:      "Egvekinot",
		State:     "Chukotka",
		Country:   "RU",
		Latitude:  66.3583984375,
		Longitude: -179.1076965332,
		Timezone:  "Asia/Anadyr",
	},
	"UHMF": {
		ICAO:      "UHMF",
		Name:      "Omsukchan Airport",
		City:      "Omsukchan",
		State:     "Magadan",
		Country:   "RU",
		Elevation: 1732,
		Latitude:  62.4570007324,
		Longitude: 155.7449951172,
		Timezone:  "Asia/Magadan",
	},
	"UHMG": {
		ICAO:      "UHMG",
		Name:      "Chaybukha Airport",
		City:      "Chaybukha",
		State:     "Magadan",
		Country:   "RU",
		Elevation: 207,
		Latitude:  61.8349990845,
		Longitude: 160.5480041504,
		Timezone:  "Asia/Magadan",
	},
	"UHMI": {
		ICAO:      "UHMI",
		Name:      "Mys Shmidta Airport",
		City:      "Mys Shmidta",
		State:     "Chukotka",
		Country:   "RU",
		Elevation: 20,
		Latitude:  68.8683013916,
		Longitude: -179.3730010986,
		Timezone:  "Asia/Anadyr",
	},
	"UHMK": {
		ICAO:      "UHMK",
		IATA:      "KPW",
		Name:      "Keperveem Airport",
		City:      "Keperveem",
		State:     "Chukotka",
		Country:   "RU",
		Elevation: 623,
		Latitude:  67.8450012207,
		Longitude: 166.13999939,
		Timezone:  "Asia/Anadyr",
	},
	"UHML": {
		ICAO:      "UHML",
		Name:      "Lavrentiya Airport",
		City:      "Lavrentiya",
		State:     "Chukotka",
		Country:   "RU",
		Elevation: 30,
		Latitude:  65.5800018311,
		Longitude: -170.996673584,
		Timezone:  "Asia/Anadyr",
	},
	"UHMM": {
		ICAO:      "UHMM",
		IATA:      "GDX",
		Name:      "Sokol Airport",
		City:      "Magadan",
		State:     "Magadan",
		Country:   "RU",
		Elevation: 574,
		Latitude:  59.9109992981,
		Longitude: 150.7200012207,
		Timezone:  "Asia/Magadan",
	},
	"UHMN": {
		ICAO:      "UHMN",
		Name:      "Omolon Airport",
		City:      "Omolon",
		State:     "Magadan",
		Country:   "RU",
		Latitude:  65.2399978638,
		Longitude: 160.5399932861,
		Timezone:  "Asia/Anadyr",
	},
	"UHMO": {
		ICAO:      "UHMO",
		IATA:      "KVM",
		Name:      "Markovo Airport",
		City:      "Markovo",
		State:     "Chukotka",
		Country:   "RU",
		Latitude:  64.6669998169,
		Longitude: 170.4170074463,
		Timezone:  "Asia/Anadyr",
	},
	"UHMP": {
		ICAO:      "UHMP",
		IATA:      "PWE",
		Name:      "Pevek Airport",
		City:      "Pevek",
		State:     "Chukotka",
		Country:   "RU",
		Elevation: 11,
		Latitude:  69.7833023071,
		Longitude: 170.5970001221,
		Timezone:  "Asia/Anadyr",
	},
	"UHMR": {
		ICAO:      "UHMR",
		Name:      "Beringovskiy Airport",
		City:      "Beringovsky",
		State:     "Chukotka",
		Country:   "RU",
		Elevation: 3,
		Latitude:  63.0200004578,
		Longitude: 179.2929992676,
		Timezone:  "Asia/Anadyr",
	},
	"UHMS": {
		ICAO:      "UHMS",
		Name:      "Seymchan Airport",
		City:      "Seymchan",
		State:     "Magadan",
		Country:   "RU",
		Elevation: 679,
		Latitude:  62.9207801819,
		Longitude: 152.4227752686,
		Timezone:  "Asia/Magadan",
	},
	"UHMT": {
		ICAO:      "UHMT",
		Name:      "Magadan-13 Airport",
		City:      "Magadan",
		State:     "Magadan",
		Country:   "RU",
		Elevation: 164,
		Latitude:  59.623298645,
		Longitude: 150.9219970703,
		Timezone:  "Asia/Magadan",
	},
	"UHMW": {
		ICAO:      "UHMW",
		IATA:      "SWV",
		Name:      "Severo-Evensk Airport",
		City:      "Evensk",
		State:     "Magadan",
		Country:   "RU",
		Latitude:  61.9216651917,
		Longitude: 159.2299957275,
		Timezone:  "Asia/Magadan",
	},
	"UHNA": {
		ICAO:      "UHNA",
		Name:      "Munuk Airport",
		City:      "Ayan",
		State:     "Khabarovsk",
		Country:   "RU",
		Latitude:  56.4676017761,
		Longitude: 138.1779937744,
		Timezone:  "Asia/Vladivostok",
	},
	"UHNB": {
		ICAO:      "UHNB",
		IATA:      "BQG",
		Name:      "Bogorodskoye Airport",
		City:      "Bogorodskoye",
		State:     "Khabarovsk",
		Country:   "RU",
		Latitude:  52.38,
		Longitude: 140.448,
		Timezone:  "Asia/Vladivostok",
	},
	"UHNN": {
		ICAO:      "UHNN",
		IATA:      "NLI",
		Name:      "Nikolayevsk-na-Amure Airport",
		City:      "Nikolayevsk-na-Amure Airport",
		State:     "Khabarovsk",
		Country:   "RU",
		Latitude:  53.1549987793,
		Longitude: 140.6499938965,
		Timezone:  "Asia/Vladivostok",
	},
	"UHOO": {
		ICAO:      "UHOO",
		IATA:      "OHO",
		Name:      "Okhotsk Airport",
		City:      "Okhotsk",
		State:     "Khabarovsk",
		Country:   "RU",
		Latitude:  59.4100646973,
		Longitude: 143.0565032959,
		Timezone:  "Asia/Vladivostok",
	},
	"UHPA": {
		ICAO:      "UHPA",
		Name:      "Ust-Pakhachi Airport",
		City:      "Ust-Pakhachi",
		State:     "Kamtsjatka",
		Country:   "RU",
		Elevation: 10,
		Latitude:  60.5567016602,
		Longitude: 169.1100006104,
		Timezone:  "Asia/Kamchatka",
	},
	"UHPG": {
		ICAO:      "UHPG",
		Name:      "Tigil Airport",
		City:      "Tigil",
		State:     "Kamtsjatka",
		Country:   "RU",
		Elevation: 430,
		Latitude:  57.7840003967,
		Longitude: 158.7299957275,
		Timezone:  "Asia/Kamchatka",
	},
	"UHPH": {
		ICAO:      "UHPH",
		Name:      "Khalaktyrka Airport",
		City:      "Petropavlovsk-Kamchatskiy",
		State:     "Kamtsjatka",
		Country:   "RU",
		Latitude:  53.0250015259,
		Longitude: 158.720993042,
		Timezone:  "Asia/Kamchatka",
	},
	"UHPK": {
		ICAO:      "UHPK",
		Name:      "Ust-Kamchatsk Airport",
		City:      "Ust-Kamchatsk",
		State:     "Kamtsjatka",
		Country:   "RU",
		Elevation: 200,
		Latitude:  56.2386016846,
		Longitude: 162.68800354,
		Timezone:  "Asia/Kamchatka",
	},
	"UHPL": {
		ICAO:      "UHPL",
		Name:      "Palana Airport",
		City:      "Palana",
		Country:   "RU",
		Elevation: 112,
		Latitude:  59.0816993713,
		Longitude: 159.8880004883,
		Timezone:  "Asia/Kamchatka",
	},
	"UHPM": {
		ICAO:      "UHPM",
		Name:      "Milkovo Airport",
		City:      "Milkovo",
		State:     "Kamtsjatka",
		Country:   "RU",
		Elevation: 410,
		Latitude:  54.6805801392,
		Longitude: 158.5485076904,
		Timezone:  "Asia/Kamchatka",
	},
	"UHPP": {
		ICAO:      "UHPP",
		IATA:      "PKC",
		Name:      "Yelizovo Airport",
		City:      "Petropavlovsk-Kamchatsky",
		State:     "Kamtsjatka",
		Country:   "RU",
		Elevation: 131,
		Latitude:  53.1679000854,
		Longitude: 158.453994751,
		Timezone:  "Asia/Kamchatka",
	},
	"UHPT": {
		ICAO:      "UHPT",
		Name:      "Tilichiki Airport",
		City:      "Tilichiki",
		State:     "Kamtsjatka",
		Country:   "RU",
		Elevation: 7,
		Latitude:  60.3828010559,
		Longitude: 166.0260009766,
		Timezone:  "Asia/Kamchatka",
	},
	"UHPU": {
		ICAO:      "UHPU",
		Name:      "Ust-Khayryuzovo Airport",
		City:      "Ust-Khayryuzovo",
		State:     "Kamtsjatka",
		Country:   "RU",
		Latitude:  57.0999984741,
		Longitude: 156.7350006104,
		Timezone:  "Asia/Kamchatka",
	},
	"UHPX": {
		ICAO:      "UHPX",
		Name:      "Nikolskoye Airport",
		City:      "Nikolskoye",
		State:     "Kamtsjatka",
		Country:   "RU",
		Elevation: 49,
		Latitude:  55.1783332825,
		Longitude: 166.0483398438,
		Timezone:  "Asia/Kamchatka",
	},
	"UHSB": {
		ICAO:      "UHSB",
		IATA:      "BVV",
		Name:      "Burevestnik Airport",
		City:      "Iturup Island",
		State:     "Sakhalin",
		Country:   "RU",
		Elevation: 79,
		Latitude:  44.9199981689,
		Longitude: 147.6219940186,
		Timezone:  "Asia/Ust-Nera",
	},
	"UHSH": {
		ICAO:      "UHSH",
		IATA:      "OHH",
		Name:      "Okha Airport",
		City:      "Okha",
		State:     "Sakhalin",
		Country:   "RU",
		Elevation: 115,
		Latitude:  53.5200004578,
		Longitude: 142.88999939,
		Timezone:  "Asia/Sakhalin",
	},
	"UHSK": {
		ICAO:      "UHSK",
		IATA:      "EKS",
		Name:      "Shakhtyorsk Airport",
		City:      "Shakhtersk",
		State:     "Sakhalin",
		Country:   "RU",
		Elevation: 50,
		Latitude:  49.1902999878,
		Longitude: 142.082992554,
		Timezone:  "Asia/Sakhalin",
	},
	"UHSM": {
		ICAO:      "UHSM",
		IATA:      "DEE",
		Name:      "Mendeleyevo Airport",
		City:      "Kunashir Island",
		State:     "Sakhalin",
		Country:   "RU",
		Elevation: 584,
		Latitude:  43.9584007263,
		Longitude: 145.682998657,
		Timezone:  "Asia/Ust-Nera",
	},
	"UHSO": {
		ICAO:      "UHSO",
		IATA:      "ZZO",
		Name:      "Zonalnoye Airport",
		City:      "Tymovskoye",
		State:     "Sakhalin",
		Country:   "RU",
		Elevation: 479,
		Latitude:  50.6692008972,
		Longitude: 142.761001587,
		Timezone:  "Asia/Sakhalin",
	},
	"UHSS": {
		ICAO:      "UHSS",
		IATA:      "UUS",
		Name:      "Yuzhno-Sakhalinsk Airport",
		City:      "Yuzhno-Sakhalinsk",
		State:     "Sakhalin",
		Country:   "RU",
		Elevation: 59,
		Latitude:  46.8886985779,
		Longitude: 142.7180023193,
		Timezone:  "Asia/Sakhalin",
	},
	"UHU1": {
		ICAO:      "UHU1",
		Name:      "Zolotaya Dolina Air Base",
		City:      "Vladimiro-Aleksandrovskoye",
		State:     "Primorskiy",
		Country:   "RU",
		Latitude:  42.9599990845,
		Longitude: 133.1150054932,
		Timezone:  "Asia/Vladivostok",
	},
	"UHWP": {
		ICAO:      "UHWP",
		IATA:      "TLY",
		Name:      "Plastun Airport",
		City:      "Plastun",
		State:     "Primorskiy",
		Country:   "RU",
		Latitude:  44.8149986267,
		Longitude: 136.2920074463,
		Timezone:  "Asia/Vladivostok",
	},
	"UHWW": {
		ICAO:      "UHWW",
		IATA:      "VVO",
		Name:      "Vladivostok International Airport",
		City:      "Vladivostok",
		State:     "Primorskiy",
		Country:   "RU",
		Elevation: 46,
		Latitude:  43.3989982605,
		Longitude: 132.1479949951,
		Timezone:  "Asia/Vladivostok",
	},
	"UIAA": {
		ICAO:      "UIAA",
		IATA:      "HTA",
		Name:      "Chita-Kadala Airport",
		City:      "Chita",
		State:     "Transbaikal-Territory",
		Country:   "RU",
		Elevation: 2272,
		Latitude:  52.0262985229,
		Longitude: 113.3059997559,
		Timezone:  "Asia/Chita",
	},
	"UIAE": {
		ICAO:      "UIAE",
		Name:      "Krasnokamensk Airport",
		City:      "Krasnokamensk",
		State:     "Transbaikal-Territory",
		Country:   "RU",
		Elevation: 2139,
		Latitude:  50.0331077576,
		Longitude: 118.0611419678,
		Timezone:  "Asia/Chita",
	},
	"UIAO": {
		ICAO:      "UIAO",
		Name:      "Barguzin Airport",
		City:      "Barguzin",
		State:     "Respublika-Buryatiya",
		Country:   "RU",
		Latitude:  53.5870018005,
		Longitude: 109.6999969482,
		Timezone:  "Asia/Irkutsk",
	},
	"UIAR": {
		ICAO:      "UIAR",
		Name:      "Chara Airport",
		City:      "Chara",
		State:     "Transbaikal-Territory",
		Country:   "RU",
		Elevation: 2201,
		Latitude:  56.9133338928,
		Longitude: 118.2699966431,
		Timezone:  "Asia/Chita",
	},
	"UIBB": {
		ICAO:      "UIBB",
		IATA:      "BTK",
		Name:      "Bratsk Airport",
		City:      "Bratsk",
		State:     "Irkutsk",
		Country:   "RU",
		Elevation: 1610,
		Latitude:  56.3706016541,
		Longitude: 101.6979980469,
		Timezone:  "Asia/Irkutsk",
	},
	"UIBS": {
		ICAO:      "UIBS",
		IATA:      "UIK",
		Name:      "Ust-Ilimsk Airport",
		City:      "Ust-Ilimsk",
		State:     "Irkutsk",
		Country:   "RU",
		Elevation: 1339,
		Latitude:  58.136100769,
		Longitude: 102.5650024414,
		Timezone:  "Asia/Irkutsk",
	},
	"UIBV": {
		ICAO:      "UIBV",
		Name:      "Zheleznogorsk Airport",
		City:      "Zheleznogorsk-Ilimsky",
		State:     "Irkutsk",
		Country:   "RU",
		Elevation: 1946,
		Latitude:  56.487701416,
		Longitude: 104.1060028076,
		Timezone:  "Asia/Irkutsk",
	},
	"UIIB": {
		ICAO:      "UIIB",
		Name:      "Belaya Air Base",
		City:      "Usolye-Sibirskoye",
		State:     "Irkutsk",
		Country:   "RU",
		Elevation: 1503,
		Latitude:  52.9150009155,
		Longitude: 103.5749969482,
		Timezone:  "Asia/Irkutsk",
	},
	"UIII": {
		ICAO:      "UIII",
		IATA:      "IKT",
		Name:      "Irkutsk Airport",
		City:      "Irkutsk",
		State:     "Irkutsk",
		Country:   "RU",
		Elevation: 1675,
		Latitude:  52.2680015564,
		Longitude: 104.388999939,
		Timezone:  "Asia/Irkutsk",
	},
	"UIIR": {
		ICAO:      "UIIR",
		Name:      "Irkutsk Northwest Airport",
		City:      "Irkutsk",
		State:     "Irkutsk",
		Country:   "RU",
		Elevation: 1470,
		Latitude:  52.3683013916,
		Longitude: 104.1829986572,
		Timezone:  "Asia/Irkutsk",
	},
	"UIIV": {
		ICAO:      "UIIV",
		Name:      "Zhigalovo Airport",
		State:     "Irkutsk",
		Country:   "RU",
		Latitude:  54.7999992371,
		Longitude: 105.1500015259,
		Timezone:  "Asia/Irkutsk",
	},
	"UIKB": {
		ICAO:      "UIKB",
		IATA:      "ODO",
		Name:      "Bodaybo Airport",
		City:      "Bodaybo",
		State:     "Irkutsk",
		Country:   "RU",
		Elevation: 919,
		Latitude:  57.8661003113,
		Longitude: 114.242996216,
		Timezone:  "Asia/Irkutsk",
	},
	"UIKE": {
		ICAO:      "UIKE",
		IATA:      "ERG",
		Name:      "Yerbogachen Airport",
		City:      "Erbogachen",
		State:     "Irkutsk",
		Country:   "RU",
		Elevation: 400,
		Latitude:  61.2750015259,
		Longitude: 108.029998779,
		Timezone:  "Asia/Irkutsk",
	},
	"UIKK": {
		ICAO:      "UIKK",
		IATA:      "KCK",
		Name:      "Kirensk Airport",
		City:      "Kirensk",
		State:     "Irkutsk",
		Country:   "RU",
		Latitude:  57.7729988098,
		Longitude: 108.0640029907,
		Timezone:  "Asia/Irkutsk",
	},
	"UIKM": {
		ICAO:      "UIKM",
		Name:      "Mama Airport",
		City:      "Mama",
		State:     "Irkutsk",
		Country:   "RU",
		Elevation: 659,
		Latitude:  58.313331604,
		Longitude: 112.8899993896,
		Timezone:  "Asia/Irkutsk",
	},
	"UIOO": {
		ICAO:      "UIOO",
		Name:      "Ust-Ordynskiy Airport",
		City:      "Ust-Ordynskiy",
		State:     "Irkutsk",
		Country:   "RU",
		Elevation: 1775,
		Latitude:  52.8199996948,
		Longitude: 104.7399978638,
		Timezone:  "Asia/Irkutsk",
	},
	"UITK": {
		ICAO:      "UITK",
		Name:      "Kazachinskaya Airport",
		City:      "Kazachinskaya",
		State:     "Irkutsk",
		Country:   "RU",
		Elevation: 1499,
		Latitude:  56.2801513672,
		Longitude: 107.567565918,
		Timezone:  "Asia/Irkutsk",
	},
	"UITT": {
		ICAO:      "UITT",
		IATA:      "UKX",
		Name:      "Ust-Kut Airport",
		City:      "Ust-Kut",
		State:     "Irkutsk",
		Country:   "RU",
		Elevation: 2188,
		Latitude:  56.8567008972,
		Longitude: 105.7300033569,
		Timezone:  "Asia/Irkutsk",
	},
	"UIUB": {
		ICAO:      "UIUB",
		Name:      "Bagdarin Airport",
		State:     "Respublika-Buryatiya",
		Country:   "RU",
		Elevation: 3084,
		Latitude:  54.3692016602,
		Longitude: 113.4789962769,
		Timezone:  "Asia/Irkutsk",
	},
	"UIUN": {
		ICAO:      "UIUN",
		Name:      "Nizhneangarsk Airport",
		City:      "Nizhneangarsk",
		State:     "Respublika-Buryatiya",
		Country:   "RU",
		Elevation: 1545,
		Latitude:  55.8008003235,
		Longitude: 109.5950012207,
		Timezone:  "Asia/Irkutsk",
	},
	"UIUU": {
		ICAO:      "UIUU",
		IATA:      "UUD",
		Name:      "Ulan-Ude Airport (Mukhino)",
		City:      "Ulan Ude",
		State:     "Respublika-Buryatiya",
		Country:   "RU",
		Elevation: 1690,
		Latitude:  51.807800293,
		Longitude: 107.43800354,
		Timezone:  "Asia/Irkutsk",
	},
	"UJAP": {
		ICAO:      "UJAP",
		IATA:      "UJE",
		Name:      "Ujae Atoll Airport",
		City:      "Ujae Atoll",
		State:     "Ujae-Atoll",
		Country:   "MH",
		Latitude:  8.9280595779,
		Longitude: 165.7619934082,
		Timezone:  "Pacific/Majuro",
	},
	"UK59": {
		ICAO:      "UK59",
		Name:      "Chuhuiv Airport",
		City:      "Chuhuiv",
		State:     "Kharkiv",
		Country:   "UA",
		Elevation: 453,
		Latitude:  49.8381996155,
		Longitude: 36.6412010193,
		Timezone:  "Europe/Kiev",
	},
	"UK61": {
		ICAO:      "UK61",
		Name:      "Cherlyany Airport",
		City:      "Gorodok",
		State:     "Lviv",
		Country:   "UA",
		Elevation: 895,
		Latitude:  49.7378997803,
		Longitude: 23.6688995361,
		Timezone:  "Europe/Kiev",
	},
	"UKBB": {
		ICAO:      "UKBB",
		IATA:      "KBP",
		Name:      "Boryspil International Airport",
		City:      "Kiev",
		State:     "Kyiv",
		Country:   "UA",
		Elevation: 427,
		Latitude:  50.3450012207,
		Longitude: 30.8946990967,
		Timezone:  "Europe/Kiev",
	},
	"UKBF": {
		ICAO:      "UKBF",
		Name:      "Konotop Air Base",
		City:      "Konotop",
		State:     "Sumy",
		Country:   "UA",
		Elevation: 476,
		Latitude:  51.25,
		Longitude: 33.1500015259,
		Timezone:  "Europe/Kiev",
	},
	"UKCC": {
		ICAO:      "UKCC",
		IATA:      "DOK",
		Name:      "Donetsk International Airport",
		City:      "Donetsk",
		State:     "Donetsk",
		Country:   "UA",
		Elevation: 791,
		Latitude:  48.073600769,
		Longitude: 37.7397003174,
		Timezone:  "Europe/Kiev",
	},
	"UKCK": {
		ICAO:      "UKCK",
		IATA:      "KRQ",
		Name:      "Kramatorsk Airport",
		City:      "Kramatorsk",
		State:     "Donetsk",
		Country:   "UA",
		Elevation: 646,
		Latitude:  48.7056007385,
		Longitude: 37.6288986206,
		Timezone:  "Europe/Kiev",
	},
	"UKCM": {
		ICAO:      "UKCM",
		IATA:      "MPW",
		Name:      "Mariupol International Airport",
		City:      "Mariupol",
		State:     "Donetsk",
		Country:   "UA",
		Elevation: 251,
		Latitude:  47.0760993958,
		Longitude: 37.4496002197,
		Timezone:  "Europe/Kiev",
	},
	"UKCS": {
		ICAO:      "UKCS",
		IATA:      "SEV",
		Name:      "Sievierodonetsk Airport",
		City:      "Sievierodonetsk",
		State:     "Luhansk",
		Country:   "UA",
		Elevation: 236,
		Latitude:  48.9003301015,
		Longitude: 38.5416698456,
		Timezone:  "Europe/Kiev",
	},
	"UKCW": {
		ICAO:      "UKCW",
		IATA:      "VSG",
		Name:      "Luhansk International Airport",
		City:      "Luhansk",
		State:     "Luhansk",
		Country:   "UA",
		Elevation: 636,
		Latitude:  48.4174003601,
		Longitude: 39.3740997314,
		Timezone:  "Europe/Kiev",
	},
	"UKDB": {
		ICAO:      "UKDB",
		IATA:      "ERD",
		Name:      "Berdyansk Airport",
		City:      "Berdyansk",
		State:     "Zaporizhia",
		Country:   "UA",
		Elevation: 171,
		Latitude:  46.8149986267,
		Longitude: 36.7580986023,
		Timezone:  "Europe/Zaporozhye",
	},
	"UKDD": {
		ICAO:      "UKDD",
		IATA:      "DNK",
		Name:      "Dnipropetrovsk International Airport",
		City:      "Dnipropetrovsk",
		State:     "Dnipro",
		Country:   "UA",
		Elevation: 481,
		Latitude:  48.3572006226,
		Longitude: 35.1006011963,
		Timezone:  "Europe/Kiev",
	},
	"UKDE": {
		ICAO:      "UKDE",
		IATA:      "OZH",
		Name:      "Zaporizhzhia International Airport",
		City:      "Zaporizhia",
		State:     "Zaporizhia",
		Country:   "UA",
		Elevation: 373,
		Latitude:  47.8670005798,
		Longitude: 35.315700531,
		Timezone:  "Europe/Zaporozhye",
	},
	"UKDR": {
		ICAO:      "UKDR",
		IATA:      "KWG",
		Name:      "Kryvyi Rih International Airport",
		City:      "Kryvyi Rih",
		State:     "Kropyvnytskyi",
		Country:   "UA",
		Elevation: 408,
		Latitude:  48.0433006287,
		Longitude: 33.2099990845,
		Timezone:  "Europe/Kiev",
	},
	"UKFF": {
		ICAO:      "UKFF",
		IATA:      "SIP",
		Name:      "Simferopol International Airport",
		City:      "Simferopol",
		State:     "Republic-of-Crimea",
		Country:   "UA",
		Elevation: 639,
		Latitude:  45.0522003174,
		Longitude: 33.9751014709,
		Timezone:  "Europe/Simferopol",
	},
	"UKFG": {
		ICAO:      "UKFG",
		Name:      "Gvardeyskoe Air Base",
		City:      "Sarabuz",
		State:     "Republic-of-Crimea",
		Country:   "UA",
		Elevation: 721,
		Latitude:  45.1150016785,
		Longitude: 33.9770011902,
		Timezone:  "Europe/Simferopol",
	},
	"UKFI": {
		ICAO:      "UKFI",
		Name:      "Saki Air Base",
		City:      "Saki",
		State:     "Republic-of-Crimea",
		Country:   "UA",
		Elevation: 36,
		Latitude:  45.0929985046,
		Longitude: 33.5950012207,
		Timezone:  "Europe/Simferopol",
	},
	"UKFK": {
		ICAO:      "UKFK",
		IATA:      "KHC",
		Name:      "Kerch Airport",
		City:      "Kerch",
		State:     "Republic-of-Crimea",
		Country:   "UA",
		Elevation: 171,
		Latitude:  45.3725013733,
		Longitude: 36.4014015198,
		Timezone:  "Europe/Simferopol",
	},
	"UKFV": {
		ICAO:      "UKFV",
		Name:      "Yevpatoriya Airport",
		City:      "Yevpatoriya",
		State:     "Republic-of-Crimea",
		Country:   "UA",
		Elevation: 33,
		Latitude:  45.2256011963,
		Longitude: 33.376701355,
		Timezone:  "Europe/Simferopol",
	},
	"UKFY": {
		ICAO:      "UKFY",
		Name:      "Dzhankoy Airport",
		City:      "Dzhankoy",
		State:     "Republic-of-Crimea",
		Country:   "UA",
		Elevation: 75,
		Latitude:  45.7009010315,
		Longitude: 34.4188995361,
		Timezone:  "Europe/Simferopol",
	},
	"UKHD": {
		ICAO:      "UKHD",
		Name:      "Kharkiv North Airport",
		City:      "Kharkiv",
		State:     "Kharkiv",
		Country:   "UA",
		Elevation: 592,
		Latitude:  50.0250015259,
		Longitude: 36.268901825,
		Timezone:  "Europe/Kiev",
	},
	"UKHH": {
		ICAO:      "UKHH",
		IATA:      "HRK",
		Name:      "Kharkiv International Airport",
		City:      "Kharkiv",
		State:     "Kharkiv",
		Country:   "UA",
		Elevation: 508,
		Latitude:  49.9248008728,
		Longitude: 36.2900009155,
		Timezone:  "Europe/Kiev",
	},
	"UKHP": {
		ICAO:      "UKHP",
		IATA:      "PLV",
		Name:      "Suprunovka Airport",
		City:      "Poltava",
		State:     "Poltava",
		Country:   "UA",
		Elevation: 505,
		Latitude:  49.5685997009,
		Longitude: 34.3972015381,
		Timezone:  "Europe/Kiev",
	},
	"UKHS": {
		ICAO:      "UKHS",
		IATA:      "UMY",
		Name:      "Sumy Airport",
		City:      "Sumy",
		State:     "Sumy",
		Country:   "UA",
		Elevation: 594,
		Latitude:  50.8582992554,
		Longitude: 34.7625007629,
		Timezone:  "Europe/Kiev",
	},
	"UKKE": {
		ICAO:      "UKKE",
		IATA:      "CKC",
		Name:      "Cherkasy International Airport",
		City:      "Cherkasy",
		State:     "Cherkasy",
		Country:   "UA",
		Elevation: 375,
		Latitude:  49.415599823,
		Longitude: 31.995300293,
		Timezone:  "Europe/Kiev",
	},
	"UKKG": {
		ICAO:      "UKKG",
		IATA:      "KGO",
		Name:      "Kirovograd Airport",
		City:      "Kirovograd",
		State:     "Kropyvnytskyi",
		Country:   "UA",
		Elevation: 568,
		Latitude:  48.5428009033,
		Longitude: 32.2849998474,
		Timezone:  "Europe/Kiev",
	},
	"UKKH": {
		ICAO:      "UKKH",
		Name:      "Chepelevka Airport",
		City:      "Uzin",
		State:     "Kyiv",
		Country:   "UA",
		Elevation: 568,
		Latitude:  49.790599823,
		Longitude: 30.441400528,
		Timezone:  "Europe/Kiev",
	},
	"UKKK": {
		ICAO:      "UKKK",
		IATA:      "IEV",
		Name:      "Kiev Zhuliany International Airport",
		City:      "Kiev",
		State:     "Kyiv-City",
		Country:   "UA",
		Elevation: 586,
		Latitude:  50.4016990662,
		Longitude: 30.4496994019,
		Timezone:  "Europe/Kiev",
	},
	"UKKM": {
		ICAO:      "UKKM",
		IATA:      "GML",
		Name:      "Gostomel Airport",
		City:      "Kiev",
		State:     "Kyiv",
		Country:   "UA",
		Elevation: 517,
		Latitude:  50.6035003662,
		Longitude: 30.1919002533,
		Timezone:  "Europe/Kiev",
	},
	"UKKT": {
		ICAO:      "UKKT",
		Name:      "Svyatoshyn Airport",
		City:      "Svyatoshyn",
		State:     "Kyiv-City",
		Country:   "UA",
		Elevation: 582,
		Latitude:  50.4788894653,
		Longitude: 30.3858337402,
		Timezone:  "Europe/Kiev",
	},
	"UKKV": {
		ICAO:      "UKKV",
		IATA:      "ZTR",
		Name:      "Zhytomyr Airport",
		State:     "Zhytomyr",
		Country:   "UA",
		Latitude:  50.270556,
		Longitude: 28.738611,
		Timezone:  "Europe/Kiev",
	},
	"UKLC": {
		ICAO:      "UKLC",
		IATA:      "UCK",
		Name:      "Lutsk Airport",
		City:      "Lutsk",
		State:     "Volyn",
		Country:   "UA",
		Elevation: 774,
		Latitude:  50.6784041103,
		Longitude: 25.487165451,
		Timezone:  "Europe/Kiev",
	},
	"UKLH": {
		ICAO:      "UKLH",
		IATA:      "HMJ",
		Name:      "Khmelnytskyi Airport",
		City:      "Khmelnytskyi",
		State:     "Khmelnytskyi",
		Country:   "UA",
		Elevation: 1150,
		Latitude:  49.3596992493,
		Longitude: 26.9333992004,
		Timezone:  "Europe/Kiev",
	},
	"UKLI": {
		ICAO:      "UKLI",
		IATA:      "IFO",
		Name:      "Ivano-Frankivsk International Airport",
		City:      "Ivano-Frankivsk",
		State:     "Ivano-Frankivsk",
		Country:   "UA",
		Elevation: 919,
		Latitude:  48.8842010498,
		Longitude: 24.6861000061,
		Timezone:  "Europe/Kiev",
	},
	"UKLL": {
		ICAO:      "UKLL",
		IATA:      "LWO",
		Name:      "Lviv International Airport",
		City:      "Lviv",
		State:     "Lviv",
		Country:   "UA",
		Elevation: 1071,
		Latitude:  49.8125,
		Longitude: 23.9561004639,
		Timezone:  "Europe/Kiev",
	},
	"UKLN": {
		ICAO:      "UKLN",
		IATA:      "CWC",
		Name:      "Chernivtsi International Airport",
		City:      "Chernivtsi",
		State:     "Chernivtsi",
		Country:   "UA",
		Elevation: 826,
		Latitude:  48.2593002319,
		Longitude: 25.9808006287,
		Timezone:  "Europe/Kiev",
	},
	"UKLR": {
		ICAO:      "UKLR",
		IATA:      "RWN",
		Name:      "Rivne International Airport",
		City:      "Rivne",
		State:     "Rivne",
		Country:   "UA",
		Elevation: 755,
		Latitude:  50.6071014404,
		Longitude: 26.1415996552,
		Timezone:  "Europe/Kiev",
	},
	"UKLT": {
		ICAO:      "UKLT",
		IATA:      "TNL",
		Name:      "Ternopil International Airport",
		City:      "Ternopil",
		State:     "Ternopil",
		Country:   "UA",
		Elevation: 1072,
		Latitude:  49.5242004395,
		Longitude: 25.7000999451,
		Timezone:  "Europe/Kiev",
	},
	"UKLU": {
		ICAO:      "UKLU",
		IATA:      "UDJ",
		Name:      "Uzhhorod International Airport",
		City:      "Uzhhorod",
		State:     "Transcarpathia",
		Country:   "UA",
		Elevation: 383,
		Latitude:  48.6343002319,
		Longitude: 22.2633991241,
		Timezone:  "Europe/Bratislava",
	},
	"UKOH": {
		ICAO:      "UKOH",
		IATA:      "KHE",
		Name:      "Chernobayevka Airport",
		City:      "Kherson",
		State:     "Kherson",
		Country:   "UA",
		Elevation: 148,
		Latitude:  46.6758003235,
		Longitude: 32.506401062,
		Timezone:  "Europe/Kiev",
	},
	"UKOI": {
		ICAO:      "UKOI",
		Name:      "Izmail Airport",
		City:      "Izmail",
		State:     "Odessa",
		Country:   "UA",
		Elevation: 125,
		Latitude:  45.3916664124,
		Longitude: 28.8016662598,
		Timezone:  "Europe/Kiev",
	},
	"UKOM": {
		ICAO:      "UKOM",
		Name:      "Limans'Ke Airport",
		City:      "Limans'Ke",
		State:     "Odessa",
		Country:   "UA",
		Elevation: 158,
		Latitude:  46.6691665649,
		Longitude: 30.0111103058,
		Timezone:  "Europe/Kiev",
	},
	"UKON": {
		ICAO:      "UKON",
		IATA:      "NLV",
		Name:      "Mykolaiv International Airport",
		City:      "Nikolayev",
		State:     "Mykolaiv",
		Country:   "UA",
		Elevation: 184,
		Latitude:  47.0578994751,
		Longitude: 31.9197998047,
		Timezone:  "Europe/Kiev",
	},
	"UKOO": {
		ICAO:      "UKOO",
		IATA:      "ODS",
		Name:      "Odessa International Airport",
		City:      "Odessa",
		State:     "Odessa",
		Country:   "UA",
		Elevation: 172,
		Latitude:  46.4267997742,
		Longitude: 30.6765003204,
		Timezone:  "Europe/Kiev",
	},
	"UKWW": {
		ICAO:      "UKWW",
		IATA:      "VIN",
		Name:      "Vinnytsia/Gavyryshivka Airport",
		City:      "Vinnitsa",
		State:     "Vinnyts'ka",
		Country:   "UA",
		Elevation: 961,
		Latitude:  49.242531,
		Longitude: 28.613778,
		Timezone:  "Europe/Kiev",
	},
	"ULAA": {
		ICAO:      "ULAA",
		IATA:      "ARH",
		Name:      "Talagi Airport",
		City:      "Archangelsk",
		State:     "Arkhangelskaya",
		Country:   "RU",
		Elevation: 62,
		Latitude:  64.6003036499,
		Longitude: 40.7167015076,
		Timezone:  "Europe/Moscow",
	},
	"ULAE": {
		ICAO:      "ULAE",
		Name:      "Mezen Airport",
		City:      "Mezen",
		State:     "Arkhangelskaya",
		Country:   "RU",
		Elevation: 46,
		Latitude:  65.8783340454,
		Longitude: 44.2150001526,
		Timezone:  "Europe/Moscow",
	},
	"ULAH": {
		ICAO:      "ULAH",
		IATA:      "VKV",
		Name:      "Vaskovo Airport",
		City:      "Arkhangelsk",
		State:     "Arkhangelskaya",
		Country:   "RU",
		Elevation: 82,
		Latitude:  64.4417037964,
		Longitude: 40.4216995239,
		Timezone:  "Europe/Moscow",
	},
	"ULAK": {
		ICAO:      "ULAK",
		Name:      "Severomorsk-1 Naval Air Base",
		City:      "Severomorsk",
		State:     "Murmansk",
		Country:   "RU",
		Elevation: 239,
		Latitude:  69.0316696167,
		Longitude: 33.4183311462,
		Timezone:  "Europe/Moscow",
	},
	"ULAL": {
		ICAO:      "ULAL",
		IATA:      "LDG",
		Name:      "Leshukonskoye Airport",
		City:      "Leshukonskoye",
		State:     "Arkhangelskaya",
		Country:   "RU",
		Elevation: 220,
		Latitude:  64.8960037231,
		Longitude: 45.7229995728,
		Timezone:  "Europe/Moscow",
	},
	"ULAM": {
		ICAO:      "ULAM",
		IATA:      "NNM",
		Name:      "Naryan Mar Airport",
		City:      "Naryan Mar",
		State:     "Nenets",
		Country:   "RU",
		Elevation: 36,
		Latitude:  67.6399993896,
		Longitude: 53.1218986511,
		Timezone:  "Europe/Moscow",
	},
	"ULAN": {
		ICAO:      "ULAN",
		Name:      "Shenkursk Airport",
		City:      "Shenkursk",
		State:     "Arkhangelskaya",
		Country:   "RU",
		Latitude:  62.1170005798,
		Longitude: 42.8829994202,
		Timezone:  "Europe/Moscow",
	},
	"ULAO": {
		ICAO:      "ULAO",
		Name:      "Onega Airport",
		City:      "Onega",
		State:     "Arkhangelskaya",
		Country:   "RU",
		Elevation: 59,
		Latitude:  63.9111140154,
		Longitude: 38.1258201599,
		Timezone:  "Europe/Moscow",
	},
	"ULAP": {
		ICAO:      "ULAP",
		Name:      "Karpogory Airport",
		City:      "Karpogory",
		State:     "Arkhangelskaya",
		Country:   "RU",
		Latitude:  64.0130004883,
		Longitude: 44.4550018311,
		Timezone:  "Europe/Moscow",
	},
	"ULAR": {
		ICAO:      "ULAR",
		Name:      "Kargopol Airport",
		City:      "Kargopol",
		State:     "Arkhangelskaya",
		Country:   "RU",
		Elevation: 420,
		Latitude:  61.5099983215,
		Longitude: 38.9249992371,
		Timezone:  "Europe/Moscow",
	},
	"ULAS": {
		ICAO:      "ULAS",
		IATA:      "CSH",
		Name:      "Solovki Airport",
		City:      "Solovetsky Islands",
		State:     "Arkhangelskaya",
		Country:   "RU",
		Elevation: 60,
		Latitude:  65.0299987793,
		Longitude: 35.7333335876,
		Timezone:  "Europe/Moscow",
	},
	"ULAT": {
		ICAO:      "ULAT",
		Name:      "Pertominsk Airport",
		City:      "Pertominsk",
		State:     "Arkhangelskaya",
		Country:   "RU",
		Elevation: 56,
		Latitude:  64.7900009155,
		Longitude: 38.4199981689,
		Timezone:  "Europe/Moscow",
	},
	"ULAV": {
		ICAO:      "ULAV",
		Name:      "Nizhnyaya Pesha Airport",
		City:      "Nizhnyaya Pesha",
		State:     "Nenets",
		Country:   "RU",
		Elevation: 32,
		Latitude:  66.7529983521,
		Longitude: 47.7430000305,
		Timezone:  "Europe/Moscow",
	},
	"ULBC": {
		ICAO:      "ULBC",
		Name:      "Cherepovets Airport",
		City:      "Cherepovets",
		State:     "Vologda",
		Country:   "RU",
		Elevation: 377,
		Latitude:  59.273601532,
		Longitude: 38.0158004761,
		Timezone:  "Europe/Moscow",
	},
	"ULBD": {
		ICAO:      "ULBD",
		Name:      "Dolgoshchelye Airport",
		City:      "Dolgoshchelye",
		State:     "Arkhangelskaya",
		Country:   "RU",
		Latitude:  66.0437011719,
		Longitude: 43.4580993652,
		Timezone:  "Europe/Moscow",
	},
	"ULBI": {
		ICAO:      "ULBI",
		Name:      "Chizha Airport",
		City:      "Chizha",
		State:     "Nenets",
		Country:   "RU",
		Latitude:  67.0839996338,
		Longitude: 44.3670005798,
		Timezone:  "Europe/Moscow",
	},
	"ULBL": {
		ICAO:      "ULBL",
		Name:      "Lopshenga Airport",
		City:      "Lopshenga",
		State:     "Arkhangelskaya",
		Country:   "RU",
		Elevation: 49,
		Latitude:  64.9800033569,
		Longitude: 37.6930007935,
		Timezone:  "Europe/Moscow",
	},
	"ULBM": {
		ICAO:      "ULBM",
		Name:      "Purnema Airport",
		City:      "Purnema",
		State:     "Arkhangelskaya",
		Country:   "RU",
		Elevation: 131,
		Latitude:  64.3854002577,
		Longitude: 37.4412345886,
		Timezone:  "Europe/Moscow",
	},
	"ULBO": {
		ICAO:      "ULBO",
		Name:      "Shoyna Airport",
		City:      "Shoyna",
		State:     "Nenets",
		Country:   "RU",
		Latitude:  67.8690032959,
		Longitude: 44.1510009766,
		Timezone:  "Europe/Moscow",
	},
	"ULBW": {
		ICAO:      "ULBW",
		Name:      "Verkhnyaya Zolotitsa Airport",
		State:     "Arkhangelskaya",
		Country:   "RU",
		Latitude:  65.6910018921,
		Longitude: 40.3804016113,
		Timezone:  "Europe/Moscow",
	},
	"ULBZ": {
		ICAO:      "ULBZ",
		Name:      "Letnyaya Zolotitsa Airport",
		City:      "Letnyaya Zolotitsa",
		State:     "Arkhangelskaya",
		Country:   "RU",
		Elevation: 16,
		Latitude:  64.9599990845,
		Longitude: 36.8300018311,
		Timezone:  "Europe/Moscow",
	},
	"ULDA": {
		ICAO:      "ULDA",
		Name:      "Rogachyovo Air Base",
		City:      "Belaya",
		State:     "Nenets",
		Country:   "RU",
		Elevation: 272,
		Latitude:  71.6166992188,
		Longitude: 52.4782981873,
		Timezone:  "Europe/Moscow",
	},
	"ULDD": {
		ICAO:      "ULDD",
		IATA:      "AMV",
		Name:      "Amderma Airport",
		City:      "Amderma",
		State:     "Nenets",
		Country:   "RU",
		Elevation: 13,
		Latitude:  69.7632980347,
		Longitude: 61.5564002991,
		Timezone:  "Europe/Moscow",
	},
	"ULDT": {
		ICAO:      "ULDT",
		Name:      "Karatayka Airport",
		City:      "Karatayka",
		Country:   "RU",
		Elevation: 39,
		Latitude:  68.75,
		Longitude: 61.4249992371,
		Timezone:  "Europe/Moscow",
	},
	"ULDW": {
		ICAO:      "ULDW",
		IATA:      "VRI",
		Name:      "Varandey Airport",
		State:     "Nenets",
		Country:   "RU",
		Elevation: 39,
		Latitude:  68.8485031128,
		Longitude: 58.2014007568,
		Timezone:  "Europe/Moscow",
	},
	"ULEH": {
		ICAO:      "ULEH",
		Name:      "Khorey-Ver Airport",
		City:      "Khorey-Ver",
		State:     "Nenets",
		Country:   "RU",
		Latitude:  67.4150009155,
		Longitude: 58.0449981689,
		Timezone:  "Europe/Moscow",
	},
	"ULER": {
		ICAO:      "ULER",
		Name:      "Kharuta Airport",
		City:      "Kharuta",
		State:     "Komi",
		Country:   "RU",
		Latitude:  66.8470001221,
		Longitude: 59.5449981689,
		Timezone:  "Europe/Moscow",
	},
	"ULJC": {
		ICAO:      "ULJC",
		Name:      "Tsenogora Airport",
		City:      "Tsenogora",
		State:     "Arkhangelskaya",
		Country:   "RU",
		Elevation: 262,
		Latitude:  64.8870010376,
		Longitude: 46.7210006714,
		Timezone:  "Europe/Moscow",
	},
	"ULJO": {
		ICAO:      "ULJO",
		Name:      "Olema Airport",
		City:      "Olema",
		State:     "Arkhangelskaya",
		Country:   "RU",
		Elevation: 164,
		Latitude:  64.495,
		Longitude: 46.14666,
		Timezone:  "Europe/Moscow",
	},
	"ULKK": {
		ICAO:      "ULKK",
		IATA:      "KSZ",
		Name:      "Kotlas Airport",
		City:      "Kotlas",
		State:     "Arkhangelskaya",
		Country:   "RU",
		Elevation: 184,
		Latitude:  61.2358016968,
		Longitude: 46.6974983215,
		Timezone:  "Europe/Moscow",
	},
	"ULKO": {
		ICAO:      "ULKO",
		Name:      "Oktyabrskiy Airport",
		City:      "Oktyabrskiy",
		State:     "Arkhangelskaya",
		Country:   "RU",
		Latitude:  61.0789985657,
		Longitude: 43.1990013123,
		Timezone:  "Europe/Moscow",
	},
	"ULKS": {
		ICAO:      "ULKS",
		Name:      "Savvatiya Air Base",
		City:      "Kotlas",
		State:     "Arkhangelskaya",
		Country:   "RU",
		Elevation: 318,
		Latitude:  60.9949989319,
		Longitude: 46.8699989319,
		Timezone:  "Europe/Moscow",
	},
	"ULKW": {
		ICAO:      "ULKW",
		Name:      "Velsk Airport",
		City:      "Velsk",
		State:     "Arkhangelskaya",
		Country:   "RU",
		Latitude:  61.0400009155,
		Longitude: 42.0999984741,
		Timezone:  "Europe/Moscow",
	},
	"ULLI": {
		ICAO:      "ULLI",
		IATA:      "LED",
		Name:      "Pulkovo Airport",
		City:      "St. Petersburg",
		State:     "Leningradskaya-Oblast'",
		Country:   "RU",
		Elevation: 78,
		Latitude:  59.8003005981,
		Longitude: 30.2625007629,
		Timezone:  "Europe/Moscow",
	},
	"ULLK": {
		ICAO:      "ULLK",
		Name:      "Krechevitsy Air Base",
		City:      "Novgorod",
		State:     "Novgorod",
		Country:   "RU",
		Elevation: 85,
		Latitude:  58.625,
		Longitude: 31.3850002289,
		Timezone:  "Europe/Moscow",
	},
	"ULLP": {
		ICAO:      "ULLP",
		Name:      "Pushkin Airport",
		City:      "St. Petersburg",
		State:     "Leningradskaya-Oblast'",
		Country:   "RU",
		Elevation: 230,
		Latitude:  59.6850013733,
		Longitude: 30.338300705,
		Timezone:  "Europe/Moscow",
	},
	"ULLS": {
		ICAO:      "ULLS",
		Name:      "Siversky Air Base",
		City:      "Siversky",
		State:     "Leningradskaya-Oblast'",
		Country:   "RU",
		Elevation: 341,
		Latitude:  59.3567008972,
		Longitude: 30.036699295,
		Timezone:  "Europe/Moscow",
	},
	"ULMA": {
		ICAO:      "ULMA",
		Name:      "Umba Airport",
		City:      "Umba",
		State:     "Murmansk",
		Country:   "RU",
		Elevation: 122,
		Latitude:  66.6969985962,
		Longitude: 34.3870010376,
		Timezone:  "Europe/Moscow",
	},
	"ULMK": {
		ICAO:      "ULMK",
		IATA:      "KVK",
		Name:      "Kirovsk-Apatity Airport",
		City:      "Apatity",
		State:     "Murmansk",
		Country:   "RU",
		Elevation: 515,
		Latitude:  67.4633026123,
		Longitude: 33.5882987976,
		Timezone:  "Europe/Moscow",
	},
	"ULML": {
		ICAO:      "ULML",
		Name:      "Lovozero Airport",
		City:      "Lovozero",
		State:     "Murmansk",
		Country:   "RU",
		Elevation: 499,
		Latitude:  68.021697998,
		Longitude: 35,
		Timezone:  "Europe/Moscow",
	},
	"ULMM": {
		ICAO:      "ULMM",
		IATA:      "MMK",
		Name:      "Murmansk Airport",
		City:      "Murmansk",
		State:     "Murmansk",
		Country:   "RU",
		Elevation: 266,
		Latitude:  68.7817001343,
		Longitude: 32.7508010864,
		Timezone:  "Europe/Moscow",
	},
	"ULMU": {
		ICAO:      "ULMU",
		Name:      "Tetrino Airport",
		City:      "Tetrino",
		State:     "Murmansk",
		Country:   "RU",
		Elevation: 16,
		Latitude:  66.0587670784,
		Longitude: 38.2826328278,
		Timezone:  "Europe/Moscow",
	},
	"ULMX": {
		ICAO:      "ULMX",
		Name:      "Krasnoshchelye Airport",
		City:      "Krasnoshchelye",
		State:     "Murmansk",
		Country:   "RU",
		Elevation: 492,
		Latitude:  67.3479995728,
		Longitude: 37.0769996643,
		Timezone:  "Europe/Moscow",
	},
	"ULNB": {
		ICAO:      "ULNB",
		Name:      "Borovichi Airport",
		City:      "Borovichi",
		State:     "Novgorod",
		Country:   "RU",
		Latitude:  58.436000824,
		Longitude: 33.888999939,
		Timezone:  "Europe/Moscow",
	},
	"ULNR": {
		ICAO:      "ULNR",
		Name:      "Staraya Russa Airport",
		City:      "Staraya Russa",
		State:     "Novgorod",
		Country:   "RU",
		Elevation: 52,
		Latitude:  57.9613990784,
		Longitude: 31.3843994141,
		Timezone:  "Europe/Moscow",
	},
	"ULOL": {
		ICAO:      "ULOL",
		IATA:      "VLU",
		Name:      "Velikiye Luki Airport",
		City:      "Velikiye Luki",
		State:     "Pskov",
		Country:   "RU",
		Elevation: 328,
		Latitude:  56.3810997009,
		Longitude: 30.6081008911,
		Timezone:  "Europe/Moscow",
	},
	"ULOO": {
		ICAO:      "ULOO",
		IATA:      "PKV",
		Name:      "Pskov Airport",
		City:      "Pskov",
		State:     "Pskov",
		Country:   "RU",
		Elevation: 154,
		Latitude:  57.7839012146,
		Longitude: 28.3955993652,
		Timezone:  "Europe/Moscow",
	},
	"ULPA": {
		ICAO:      "ULPA",
		Name:      "Pudozh Airport",
		City:      "Pudozh",
		State:     "Karelia",
		Country:   "RU",
		Elevation: 144,
		Latitude:  61.8250007629,
		Longitude: 36.5800018311,
		Timezone:  "Europe/Moscow",
	},
	"ULPB": {
		ICAO:      "ULPB",
		IATA:      "PES",
		Name:      "Petrozavodsk Airport",
		City:      "Petrozavodsk",
		State:     "Karelia",
		Country:   "RU",
		Elevation: 151,
		Latitude:  61.8852005005,
		Longitude: 34.1547012329,
		Timezone:  "Europe/Moscow",
	},
	"ULPK": {
		ICAO:      "ULPK",
		Name:      "Kalevala Airport",
		City:      "Kalevala",
		State:     "Karelia",
		Country:   "RU",
		Elevation: 377,
		Latitude:  65.2099990845,
		Longitude: 31.1350002289,
		Timezone:  "Europe/Moscow",
	},
	"ULPM": {
		ICAO:      "ULPM",
		Name:      "Kostomuksha Airport",
		City:      "Kostomuksha",
		State:     "Karelia",
		Country:   "RU",
		Elevation: 681,
		Latitude:  64.6179962158,
		Longitude: 30.6870002747,
		Timezone:  "Europe/Moscow",
	},
	"ULPP": {
		ICAO:      "ULPP",
		Name:      "Peski Airport",
		City:      "Petrozavodsk",
		State:     "Karelia",
		Country:   "RU",
		Latitude:  61.831943512,
		Longitude: 34.2955551147,
		Timezone:  "Europe/Moscow",
	},
	"ULPS": {
		ICAO:      "ULPS",
		Name:      "Segezha Airport",
		City:      "Segezha",
		State:     "Karelia",
		Country:   "RU",
		Elevation: 377,
		Latitude:  63.7589988708,
		Longitude: 34.283000946,
		Timezone:  "Europe/Moscow",
	},
	"ULPW": {
		ICAO:      "ULPW",
		Name:      "Sortavala Airport",
		State:     "Karelia",
		Country:   "RU",
		Latitude:  61.7360992432,
		Longitude: 30.6735992432,
		Timezone:  "Europe/Moscow",
	},
	"ULSC": {
		ICAO:      "ULSC",
		Name:      "Seltso Airfield",
		City:      "Seltso",
		State:     "Leningradskaya-Oblast'",
		Country:   "RU",
		Latitude:  59.6170005798,
		Longitude: 29.5429992676,
		Timezone:  "Europe/Moscow",
	},
	"ULSG": {
		ICAO:      "ULSG",
		Name:      "Gostilitsy Airfield",
		City:      "Gostilitsy",
		State:     "Leningradskaya-Oblast'",
		Country:   "RU",
		Latitude:  59.7266998291,
		Longitude: 29.6394996643,
		Timezone:  "Europe/Moscow",
	},
	"ULSI": {
		ICAO:      "ULSI",
		Name:      "Kusino Airfield",
		City:      "Kusino",
		State:     "Leningradskaya-Oblast'",
		Country:   "RU",
		Elevation: 95,
		Latitude:  59.4021564205,
		Longitude: 31.8155479431,
		Timezone:  "Europe/Moscow",
	},
	"ULSK": {
		ICAO:      "ULSK",
		Name:      "Korpikyulya Airfield",
		City:      "Gatchina",
		State:     "Leningradskaya-Oblast'",
		Country:   "RU",
		Latitude:  59.6049995422,
		Longitude: 30.2723007202,
		Timezone:  "Europe/Moscow",
	},
	"ULSM": {
		ICAO:      "ULSM",
		Name:      "Manushkino Airfield",
		City:      "Manushkino",
		State:     "Leningradskaya-Oblast'",
		Country:   "RU",
		Latitude:  59.8718986511,
		Longitude: 30.8027992249,
		Timezone:  "Europe/Moscow",
	},
	"ULSO": {
		ICAO:      "ULSO",
		Name:      "Kuznetsovo Airfield",
		City:      "Rozhdestveno",
		State:     "Leningradskaya-Oblast'",
		Country:   "RU",
		Latitude:  59.2452011108,
		Longitude: 29.9451999664,
		Timezone:  "Europe/Moscow",
	},
	"ULWC": {
		ICAO:      "ULWC",
		IATA:      "CEE",
		Name:      "Cherepovets Airport",
		City:      "Cherepovets",
		State:     "Vologda",
		Country:   "RU",
		Elevation: 377,
		Latitude:  59.276667,
		Longitude: 38.028333,
		Timezone:  "Europe/Moscow",
	},
	"ULWK": {
		ICAO:      "ULWK",
		Name:      "Kichmengskiy Gorodok Airport",
		City:      "Kichmengskiy Gorodok",
		State:     "Vologda",
		Country:   "RU",
		Latitude:  59.9770011902,
		Longitude: 45.8250007629,
		Timezone:  "Europe/Moscow",
	},
	"ULWR": {
		ICAO:      "ULWR",
		Name:      "Vytegra Airport",
		City:      "Vytegra",
		State:     "Vologda",
		Country:   "RU",
		Elevation: 217,
		Latitude:  61.0320014954,
		Longitude: 36.4350013733,
		Timezone:  "Europe/Moscow",
	},
	"ULWU": {
		ICAO:      "ULWU",
		IATA:      "VUS",
		Name:      "Velikiy Ustyug Airport",
		City:      "Velikiy Ustyug",
		State:     "Vologda",
		Country:   "RU",
		Elevation: 331,
		Latitude:  60.7882995605,
		Longitude: 46.2599983215,
		Timezone:  "Europe/Moscow",
	},
	"ULWW": {
		ICAO:      "ULWW",
		IATA:      "VGD",
		Name:      "Vologda Airport",
		City:      "Vologda",
		State:     "Vologda",
		Country:   "RU",
		Elevation: 387,
		Latitude:  59.2825012207,
		Longitude: 39.9444007874,
		Timezone:  "Europe/Moscow",
	},
	"UMBB": {
		ICAO:      "UMBB",
		IATA:      "BQT",
		Name:      "Brest Airport",
		City:      "Brest",
		State:     "Brest",
		Country:   "BY",
		Elevation: 468,
		Latitude:  52.1082992554,
		Longitude: 23.8980998993,
		Timezone:  "Europe/Minsk",
	},
	"UMDD": {
		ICAO:      "UMDD",
		Name:      "Lida Air Base",
		City:      "Lida",
		State:     "Grodnenskaya",
		Country:   "BY",
		Latitude:  53.877,
		Longitude: 25.377,
		Timezone:  "Europe/Minsk",
	},
	"UMGG": {
		ICAO:      "UMGG",
		IATA:      "GME",
		Name:      "Gomel Airport",
		City:      "Gomel",
		State:     "Gomel",
		Country:   "BY",
		Elevation: 472,
		Latitude:  52.5270004272,
		Longitude: 31.0167007446,
		Timezone:  "Europe/Minsk",
	},
	"UMII": {
		ICAO:      "UMII",
		IATA:      "VTB",
		Name:      "Vitebsk East Airport",
		City:      "Vitebsk",
		State:     "Vitebsk",
		Country:   "BY",
		Elevation: 682,
		Latitude:  55.126499176,
		Longitude: 30.3495998383,
		Timezone:  "Europe/Minsk",
	},
	"UMKK": {
		ICAO:      "UMKK",
		IATA:      "KGD",
		Name:      "Khrabrovo Airport",
		City:      "Kaliningrad",
		State:     "Kaliningrad",
		Country:   "RU",
		Elevation: 42,
		Latitude:  54.8899993896,
		Longitude: 20.5925998688,
		Timezone:  "Europe/Kaliningrad",
	},
	"UMLI": {
		ICAO:      "UMLI",
		Name:      "Minsk Machulishchy Airport",
		City:      "Minsk",
		State:     "Minsk",
		Country:   "BY",
		Elevation: 699,
		Latitude:  53.7733001709,
		Longitude: 27.5799999237,
		Timezone:  "Europe/Minsk",
	},
	"UMMA": {
		ICAO:      "UMMA",
		Name:      "Baranavichi Air Base",
		City:      "Baranavichi",
		State:     "Brest",
		Country:   "BY",
		Latitude:  53.0999984741,
		Longitude: 26.0499992371,
		Timezone:  "Europe/Minsk",
	},
	"UMMB": {
		ICAO:      "UMMB",
		Name:      "Borovaya Airfield",
		City:      "Minsk",
		State:     "Minsk",
		Country:   "BY",
		Latitude:  53.5699996948,
		Longitude: 27.3899993896,
		Timezone:  "Europe/Minsk",
	},
	"UMMG": {
		ICAO:      "UMMG",
		IATA:      "GNA",
		Name:      "Hrodna Airport",
		City:      "Hrodna",
		State:     "Grodnenskaya",
		Country:   "BY",
		Elevation: 443,
		Latitude:  53.6020011902,
		Longitude: 24.0538005829,
		Timezone:  "Europe/Minsk",
	},
	"UMMM": {
		ICAO:      "UMMM",
		IATA:      "MHP",
		Name:      "Minsk 1 Airport",
		City:      "Minsk",
		Country:   "BY",
		Elevation: 748,
		Latitude:  53.8644981384,
		Longitude: 27.5396995544,
		Timezone:  "Europe/Minsk",
	},
	"UMMO": {
		ICAO:      "UMMO",
		Name:      "Osovtsy Air Base",
		City:      "Byaroza",
		State:     "Brest",
		Country:   "BY",
		Latitude:  52.5569992065,
		Longitude: 24.8840007782,
		Timezone:  "Europe/Minsk",
	},
	"UMMR": {
		ICAO:      "UMMR",
		Name:      "Ross Air Base",
		City:      "Ross",
		State:     "Grodnenskaya",
		Country:   "BY",
		Latitude:  53.303,
		Longitude: 24.369,
		Timezone:  "Europe/Minsk",
	},
	"UMMS": {
		ICAO:      "UMMS",
		IATA:      "MSQ",
		Name:      "Minsk International Airport",
		City:      "Minsk",
		State:     "Minsk",
		Country:   "BY",
		Elevation: 670,
		Latitude:  53.8824996948,
		Longitude: 28.0307006836,
		Timezone:  "Europe/Minsk",
	},
	"UMNB": {
		ICAO:      "UMNB",
		Name:      "Babruisk Air Base",
		City:      "Babruisk",
		State:     "Mogilev",
		Country:   "BY",
		Latitude:  53.1049995422,
		Longitude: 29.2049999237,
		Timezone:  "Europe/Minsk",
	},
	"UMNM": {
		ICAO:      "UMNM",
		Name:      "Novo-Pashkovo Airport",
		City:      "Mahiylow",
		State:     "Mogilev",
		Country:   "BY",
		Latitude:  53.945,
		Longitude: 30.244,
		Timezone:  "Europe/Minsk",
	},
	"UMNV": {
		ICAO:      "UMNV",
		Name:      "Pruzhany Air Base",
		City:      "Pruzhany",
		State:     "Brest",
		Country:   "BY",
		Latitude:  52.5822957318,
		Longitude: 24.379863739,
		Timezone:  "Europe/Minsk",
	},
	"UMOO": {
		ICAO:      "UMOO",
		IATA:      "MVQ",
		Name:      "Mogilev Airport",
		City:      "Mogilev",
		State:     "Mogilev",
		Country:   "BY",
		Elevation: 637,
		Latitude:  53.9548988342,
		Longitude: 30.0951004028,
		Timezone:  "Europe/Minsk",
	},
	"UNAA": {
		ICAO:      "UNAA",
		IATA:      "ABA",
		Name:      "Abakan Airport",
		City:      "Abakan",
		State:     "Khakasiya",
		Country:   "RU",
		Elevation: 831,
		Latitude:  53.7400016785,
		Longitude: 91.3850021362,
		Timezone:  "Asia/Krasnoyarsk",
	},
	"UNAU": {
		ICAO:      "UNAU",
		Name:      "Shushenskoye Airport",
		City:      "Kazantsevo",
		State:     "Krasnoyarskiy",
		Country:   "RU",
		Latitude:  53.3849983215,
		Longitude: 92.0449981689,
		Timezone:  "Asia/Krasnoyarsk",
	},
	"UNBB": {
		ICAO:      "UNBB",
		IATA:      "BAX",
		Name:      "Barnaul Airport",
		City:      "Barnaul",
		State:     "Altai-Krai",
		Country:   "RU",
		Elevation: 837,
		Latitude:  53.3638000488,
		Longitude: 83.5384979248,
		Timezone:  "Asia/Barnaul",
	},
	"UNBG": {
		ICAO:      "UNBG",
		IATA:      "RGK",
		Name:      "Gorno-Altaysk Airport",
		City:      "Gorno-Altaysk",
		State:     "Altai",
		Country:   "RU",
		Elevation: 965,
		Latitude:  51.9667015076,
		Longitude: 85.8332977295,
		Timezone:  "Asia/Barnaul",
	},
	"UNBI": {
		ICAO:      "UNBI",
		Name:      "Biysk Airport",
		City:      "Biysk",
		State:     "Altai-Krai",
		Country:   "RU",
		Elevation: 620,
		Latitude:  52.4799995422,
		Longitude: 85.3399963379,
		Timezone:  "Asia/Barnaul",
	},
	"UNBM": {
		ICAO:      "UNBM",
		Name:      "Volchikha Airport",
		City:      "Volchikha",
		State:     "Altai-Krai",
		Country:   "RU",
		Latitude:  52.0250889138,
		Longitude: 80.3378677368,
		Timezone:  "Asia/Barnaul",
	},
	"UNBU": {
		ICAO:      "UNBU",
		Name:      "Ust-Koksa Airport",
		City:      "Ust-Koksa Airport",
		State:     "Altai",
		Country:   "RU",
		Latitude:  50.2649993896,
		Longitude: 85.7099990845,
		Timezone:  "Asia/Barnaul",
	},
	"UNCC": {
		ICAO:      "UNCC",
		Name:      "Novosibirsk North Airport",
		City:      "Novosibirsk",
		State:     "Novosibirsk",
		Country:   "RU",
		Elevation: 558,
		Latitude:  55.0917015076,
		Longitude: 82.9067001343,
		Timezone:  "Asia/Novosibirsk",
	},
	"UNCN": {
		ICAO:      "UNCN",
		Name:      "Krasnoozyorsk Airport",
		City:      "Krasnoozyorskoye",
		State:     "Novosibirsk",
		Country:   "RU",
		Latitude:  53.992785208,
		Longitude: 79.3133926392,
		Timezone:  "Asia/Novosibirsk",
	},
	"UNEE": {
		ICAO:      "UNEE",
		IATA:      "KEJ",
		Name:      "Kemerovo Airport",
		City:      "Kemerovo",
		State:     "Kemerovo",
		Country:   "RU",
		Elevation: 863,
		Latitude:  55.2700996399,
		Longitude: 86.1072006226,
		Timezone:  "Asia/Novokuznetsk",
	},
	"UNIB": {
		ICAO:      "UNIB",
		Name:      "Baykit Airport",
		City:      "Baykit",
		Country:   "RU",
		Elevation: 853,
		Latitude:  61.676700592,
		Longitude: 96.3550033569,
		Timezone:  "Asia/Krasnoyarsk",
	},
	"UNII": {
		ICAO:      "UNII",
		IATA:      "EIE",
		Name:      "Yeniseysk Airport",
		City:      "Yeniseysk",
		State:     "Krasnoyarskiy",
		Country:   "RU",
		Elevation: 253,
		Latitude:  58.4742012024,
		Longitude: 92.1125030518,
		Timezone:  "Asia/Krasnoyarsk",
	},
	"UNIJ": {
		ICAO:      "UNIJ",
		Name:      "Yartsevo Airport",
		City:      "Yartsevo",
		State:     "Krasnoyarskiy",
		Country:   "RU",
		Elevation: 210,
		Latitude:  60.2550010681,
		Longitude: 90.1949996948,
		Timezone:  "Asia/Krasnoyarsk",
	},
	"UNIP": {
		ICAO:      "UNIP",
		IATA:      "TGP",
		Name:      "Podkamennaya Tunguska Airport",
		City:      "Bor",
		State:     "Krasnoyarskiy",
		Country:   "RU",
		Elevation: 213,
		Latitude:  61.5896987915,
		Longitude: 89.9940032959,
		Timezone:  "Asia/Krasnoyarsk",
	},
	"UNIS": {
		ICAO:      "UNIS",
		Name:      "Severo-Eniseysk Airport",
		City:      "Severo-Eniseysk",
		State:     "Krasnoyarskiy",
		Country:   "RU",
		Elevation: 1706,
		Latitude:  60.373298645,
		Longitude: 93.0117034912,
		Timezone:  "Asia/Krasnoyarsk",
	},
	"UNIT": {
		ICAO:      "UNIT",
		Name:      "Tura Airport",
		City:      "Tura",
		Country:   "RU",
		Elevation: 2044,
		Latitude:  64.3335113525,
		Longitude: 100.4328918457,
		Timezone:  "Asia/Krasnoyarsk",
	},
	"UNIW": {
		ICAO:      "UNIW",
		Name:      "Vanavara Airport",
		City:      "Vanavara",
		Country:   "RU",
		Elevation: 892,
		Latitude:  60.3596992493,
		Longitude: 102.3239974976,
		Timezone:  "Asia/Krasnoyarsk",
	},
	"UNKB": {
		ICAO:      "UNKB",
		Name:      "Boguchany Airport",
		City:      "Boguchany",
		State:     "Krasnoyarskiy",
		Country:   "RU",
		Elevation: 446,
		Latitude:  58.3806152344,
		Longitude: 97.4723358154,
		Timezone:  "Asia/Krasnoyarsk",
	},
	"UNKI": {
		ICAO:      "UNKI",
		Name:      "Kodinsk Airport",
		State:     "Krasnoyarskiy",
		Country:   "RU",
		Latitude:  58.4794006348,
		Longitude: 99.0939025879,
		Timezone:  "Asia/Krasnoyarsk",
	},
	"UNKK": {
		ICAO:      "UNKK",
		Name:      "Krasnoyarsk Severny",
		City:      "Krasnoyarsk",
		State:     "Krasnoyarskiy",
		Country:   "RU",
		Elevation: 1030,
		Latitude:  56.22647,
		Longitude: 92.95693,
		Timezone:  "Asia/Krasnoyarsk",
	},
	"UNKL": {
		ICAO:      "UNKL",
		IATA:      "KJA",
		Name:      "Yemelyanovo Airport",
		City:      "Krasnoyarsk",
		State:     "Krasnoyarskiy",
		Country:   "RU",
		Elevation: 942,
		Latitude:  56.1729011536,
		Longitude: 92.4933013916,
		Timezone:  "Asia/Krasnoyarsk",
	},
	"UNKM": {
		ICAO:      "UNKM",
		Name:      "Cheremshanka Airport",
		City:      "Krasnoyarsk",
		State:     "Krasnoyarskiy",
		Country:   "RU",
		Latitude:  56.1769981384,
		Longitude: 92.5400009155,
		Timezone:  "Asia/Krasnoyarsk",
	},
	"UNKO": {
		ICAO:      "UNKO",
		Name:      "Sharypovo Airport",
		City:      "Sharypovo",
		State:     "Krasnoyarskiy",
		Country:   "RU",
		Elevation: 1099,
		Latitude:  55.454914093,
		Longitude: 89.1738815308,
		Timezone:  "Asia/Krasnoyarsk",
	},
	"UNKS": {
		ICAO:      "UNKS",
		IATA:      "ACS",
		Name:      "Achinsk Airport",
		City:      "Achinsk",
		State:     "Krasnoyarskiy",
		Country:   "RU",
		Elevation: 1033,
		Latitude:  56.2682991028,
		Longitude: 90.5708007813,
		Timezone:  "Asia/Krasnoyarsk",
	},
	"UNKY": {
		ICAO:      "UNKY",
		IATA:      "KYZ",
		Name:      "Kyzyl Airport",
		City:      "Kyzyl",
		State:     "Republic-of-Tyva",
		Country:   "RU",
		Elevation: 2123,
		Latitude:  51.6693992615,
		Longitude: 94.4005966187,
		Timezone:  "Asia/Krasnoyarsk",
	},
	"UNLL": {
		ICAO:      "UNLL",
		Name:      "Kolpashevo Airport",
		City:      "Kolpashevo",
		State:     "Tomsk",
		Country:   "RU",
		Elevation: 243,
		Latitude:  58.3252983093,
		Longitude: 82.9325027466,
		Timezone:  "Asia/Tomsk",
	},
	"UNNE": {
		ICAO:      "UNNE",
		Name:      "Yeltsovka Airport",
		City:      "Novosibirsk",
		State:     "Novosibirsk",
		Country:   "RU",
		Elevation: 617,
		Latitude:  55.0923995972,
		Longitude: 83.0045013428,
		Timezone:  "Asia/Novosibirsk",
	},
	"UNNS": {
		ICAO:      "UNNS",
		Name:      "Severnoye Airport",
		City:      "Severnoe",
		State:     "Novosibirsk",
		Country:   "RU",
		Latitude:  56.3349990845,
		Longitude: 78.3550033569,
		Timezone:  "Asia/Novosibirsk",
	},
	"UNNT": {
		ICAO:      "UNNT",
		IATA:      "OVB",
		Name:      "Tolmachevo Airport",
		City:      "Novosibirsk",
		State:     "Novosibirsk",
		Country:   "RU",
		Elevation: 365,
		Latitude:  55.0125999451,
		Longitude: 82.6507034302,
		Timezone:  "Asia/Novosibirsk",
	},
	"UNOK": {
		ICAO:      "UNOK",
		Name:      "Kalachinsk Airport",
		City:      "Kalachinsk",
		State:     "Omsk",
		Country:   "RU",
		Elevation: 361,
		Latitude:  55.0200004578,
		Longitude: 74.6100006104,
		Timezone:  "Asia/Omsk",
	},
	"UNOO": {
		ICAO:      "UNOO",
		IATA:      "OMS",
		Name:      "Omsk Central Airport",
		City:      "Omsk",
		State:     "Omsk",
		Country:   "RU",
		Elevation: 311,
		Latitude:  54.966999054,
		Longitude: 73.3105010986,
		Timezone:  "Asia/Omsk",
	},
	"UNOS": {
		ICAO:      "UNOS",
		Name:      "Omsk Severny Airport",
		City:      "Omsk",
		State:     "Omsk",
		Country:   "RU",
		Latitude:  54.9749984741,
		Longitude: 73.5550003052,
		Timezone:  "Asia/Omsk",
	},
	"UNOT": {
		ICAO:      "UNOT",
		Name:      "Tara Airport",
		City:      "Tara",
		State:     "Omsk",
		Country:   "RU",
		Elevation: 253,
		Latitude:  56.9000015259,
		Longitude: 74.3000030518,
		Timezone:  "Asia/Omsk",
	},
	"UNOW": {
		ICAO:      "UNOW",
		Name:      "Tevriz Airport",
		City:      "Tevriz",
		State:     "Omsk",
		Country:   "RU",
		Elevation: 217,
		Latitude:  57.5040016174,
		Longitude: 72.3310012817,
		Timezone:  "Asia/Omsk",
	},
	"UNRR": {
		ICAO:      "UNRR",
		Name:      "Barabinsk Airport",
		City:      "Barabinsk",
		State:     "Novosibirsk",
		Country:   "RU",
		Latitude:  55.3600006104,
		Longitude: 78.3000030518,
		Timezone:  "Asia/Novosibirsk",
	},
	"UNSA": {
		ICAO:      "UNSA",
		Name:      "Aleksandrovskoye Airport",
		City:      "Aleksandrovskoye",
		State:     "Tomsk",
		Country:   "RU",
		Latitude:  60.4430007935,
		Longitude: 77.8389968872,
		Timezone:  "Asia/Tomsk",
	},
	"UNSS": {
		ICAO:      "UNSS",
		IATA:      "SWT",
		Name:      "Strezhevoy Airport",
		City:      "Strezhevoy",
		State:     "Tomsk",
		Country:   "RU",
		Elevation: 164,
		Latitude:  60.709400177,
		Longitude: 77.6600036621,
		Timezone:  "Asia/Tomsk",
	},
	"UNTT": {
		ICAO:      "UNTT",
		IATA:      "TOF",
		Name:      "Bogashevo Airport",
		City:      "Tomsk",
		State:     "Tomsk",
		Country:   "RU",
		Elevation: 597,
		Latitude:  56.3802986145,
		Longitude: 85.2082977295,
		Timezone:  "Asia/Tomsk",
	},
	"UNWW": {
		ICAO:      "UNWW",
		IATA:      "NOZ",
		Name:      "Spichenkovo Airport",
		City:      "Novokuznetsk",
		State:     "Kemerovo",
		Country:   "RU",
		Elevation: 1024,
		Latitude:  53.8114013672,
		Longitude: 86.8771972656,
		Timezone:  "Asia/Novokuznetsk",
	},
	"UODD": {
		ICAO:      "UODD",
		IATA:      "DKS",
		Name:      "Dikson Airport",
		City:      "Dikson",
		State:     "Krasnoyarskiy",
		Country:   "RU",
		Elevation: 47,
		Latitude:  73.5178070068,
		Longitude: 80.3796691895,
		Timezone:  "Asia/Krasnoyarsk",
	},
	"UODN": {
		ICAO:      "UODN",
		Name:      "Nagurskoye Air Base",
		City:      "Murmansk",
		State:     "Arkhangelskaya",
		Country:   "RU",
		Elevation: 59,
		Latitude:  80.8032073975,
		Longitude: 47.6635856628,
		Timezone:  "Europe/Moscow",
	},
	"UODS": {
		ICAO:      "UODS",
		Name:      "Sredniy Ostrov Air Base",
		City:      "Khatanga",
		State:     "Krasnoyarskiy",
		Country:   "RU",
		Elevation: 26,
		Latitude:  79.5282974243,
		Longitude: 91.0749969482,
		Timezone:  "Asia/Krasnoyarsk",
	},
	"UOHH": {
		ICAO:      "UOHH",
		IATA:      "HTG",
		Name:      "Khatanga Airport",
		City:      "Khatanga",
		State:     "Krasnoyarskiy",
		Country:   "RU",
		Elevation: 95,
		Latitude:  71.9781036377,
		Longitude: 102.4909973145,
		Timezone:  "Asia/Krasnoyarsk",
	},
	"UOIC": {
		ICAO:      "UOIC",
		Name:      "Snezhnogorsk Airport",
		City:      "Snezhnogorsk",
		Country:   "RU",
		Elevation: 102,
		Latitude:  68.0766677856,
		Longitude: 87.6449966431,
		Timezone:  "Asia/Krasnoyarsk",
	},
	"UOIG": {
		ICAO:      "UOIG",
		Name:      "Svetlogorsk Airport",
		City:      "Svetlogorsk",
		State:     "Krasnoyarskiy",
		Country:   "RU",
		Elevation: 394,
		Latitude:  66.8399963379,
		Longitude: 88.4033355713,
		Timezone:  "Asia/Krasnoyarsk",
	},
	"UOII": {
		ICAO:      "UOII",
		IATA:      "IAA",
		Name:      "Igarka Airport",
		City:      "Igarka",
		State:     "Krasnoyarskiy",
		Country:   "RU",
		Elevation: 82,
		Latitude:  67.4372024536,
		Longitude: 86.6219024658,
		Timezone:  "Asia/Krasnoyarsk",
	},
	"UOOO": {
		ICAO:      "UOOO",
		IATA:      "NSK",
		Name:      "Norilsk-Alykel Airport",
		City:      "Norilsk",
		State:     "Krasnoyarskiy",
		Country:   "RU",
		Elevation: 574,
		Latitude:  69.3110961914,
		Longitude: 87.3321990967,
		Timezone:  "Asia/Krasnoyarsk",
	},
	"UOOW": {
		ICAO:      "UOOW",
		Name:      "Valek Airport",
		City:      "Valek",
		Country:   "RU",
		Elevation: 108,
		Latitude:  69.3969039917,
		Longitude: 88.3536834717,
		Timezone:  "Asia/Krasnoyarsk",
	},
	"UOTT": {
		ICAO:      "UOTT",
		IATA:      "THX",
		Name:      "Turukhansk Airport",
		City:      "Turukhansk",
		State:     "Krasnoyarskiy",
		Country:   "RU",
		Elevation: 128,
		Latitude:  65.797203064,
		Longitude: 87.9353027344,
		Timezone:  "Asia/Krasnoyarsk",
	},
	"URFB": {
		ICAO:      "URFB",
		IATA:      "UKS",
		Name:      "Belbek Airport",
		City:      "Sevastopol",
		State:     "Republic-of-Crimea",
		Country:   "UA",
		Elevation: 344,
		Latitude:  44.688999176,
		Longitude: 33.5709991455,
		Timezone:  "Europe/Simferopol",
	},
	"URFF": {
		ICAO:      "URFF",
		IATA:      "SIP",
		Name:      "Simferopol International Airport",
		City:      "Simferopol",
		State:     "Republic-of-Crimea",
		Country:   "RU",
		Elevation: 639,
		Latitude:  45.0522003174,
		Longitude: 33.9751014709,
		Timezone:  "Europe/Simferopol",
	},
	"URKA": {
		ICAO:      "URKA",
		IATA:      "AAQ",
		Name:      "Anapa Airport",
		City:      "Anapa",
		State:     "Krasnodarskiy",
		Country:   "RU",
		Elevation: 174,
		Latitude:  45.0021018982,
		Longitude: 37.3473014832,
		Timezone:  "Europe/Moscow",
	},
	"URKE": {
		ICAO:      "URKE",
		IATA:      "EIK",
		Name:      "Yeysk Airport",
		City:      "Yeysk",
		State:     "Krasnodarskiy",
		Country:   "RU",
		Elevation: 60,
		Latitude:  46.68,
		Longitude: 38.21,
		Timezone:  "Europe/Moscow",
	},
	"URKG": {
		ICAO:      "URKG",
		IATA:      "GDZ",
		Name:      "Gelendzhik Airport",
		City:      "Gelendzhik",
		State:     "Krasnodarskiy",
		Country:   "RU",
		Elevation: 98,
		Latitude:  44.5820926295,
		Longitude: 38.0124807358,
		Timezone:  "Europe/Moscow",
	},
	"URKH": {
		ICAO:      "URKH",
		Name:      "Khanskaya Airport",
		City:      "Maykop",
		State:     "Adygeya",
		Country:   "RU",
		Elevation: 600,
		Latitude:  44.6800003052,
		Longitude: 40.0349998474,
		Timezone:  "Europe/Moscow",
	},
	"URKK": {
		ICAO:      "URKK",
		IATA:      "KRR",
		Name:      "Krasnodar International Airport",
		City:      "Krasnodar",
		State:     "Krasnodarskiy",
		Country:   "RU",
		Elevation: 118,
		Latitude:  45.0346984863,
		Longitude: 39.170501709,
		Timezone:  "Europe/Moscow",
	},
	"URKM": {
		ICAO:      "URKM",
		Name:      "Maykop Airport",
		City:      "Maykop",
		State:     "Adygeya",
		Country:   "RU",
		Elevation: 679,
		Latitude:  44.653301239,
		Longitude: 40.0900001526,
		Timezone:  "Europe/Moscow",
	},
	"URKR": {
		ICAO:      "URKR",
		Name:      "Armavir Air Base",
		City:      "Armavir",
		State:     "Krasnodarskiy",
		Country:   "RU",
		Elevation: 712,
		Latitude:  44.9700012207,
		Longitude: 41.1049995422,
		Timezone:  "Europe/Moscow",
	},
	"URKS": {
		ICAO:      "URKS",
		Name:      "Enem Airport",
		City:      "Enem",
		State:     "Adygeya",
		Country:   "RU",
		Elevation: 62,
		Latitude:  44.9459991455,
		Longitude: 38.9319992065,
		Timezone:  "Europe/Moscow",
	},
	"URME": {
		ICAO:      "URME",
		Name:      "Yessentuki Airport",
		City:      "Yessentuki",
		State:     "Stavropol'skiy",
		Country:   "RU",
		Latitude:  44.06,
		Longitude: 42.829,
		Timezone:  "Europe/Moscow",
	},
	"URMG": {
		ICAO:      "URMG",
		IATA:      "GRV",
		Name:      "Grozny North Airport",
		City:      "Grozny",
		State:     "Chechnya",
		Country:   "RU",
		Latitude:  43.3883018494,
		Longitude: 45.698600769,
		Timezone:  "Europe/Moscow",
	},
	"URML": {
		ICAO:      "URML",
		IATA:      "MCX",
		Name:      "Uytash Airport",
		City:      "Makhachkala",
		State:     "Dagestan",
		Country:   "RU",
		Elevation: 12,
		Latitude:  42.8167991638,
		Longitude: 47.6523017883,
		Timezone:  "Europe/Moscow",
	},
	"URMM": {
		ICAO:      "URMM",
		IATA:      "MRV",
		Name:      "Mineralnyye Vody Airport",
		City:      "Mineralnyye Vody",
		State:     "Stavropol'skiy",
		Country:   "RU",
		Elevation: 1054,
		Latitude:  44.2251014709,
		Longitude: 43.0819015503,
		Timezone:  "Europe/Moscow",
	},
	"URMN": {
		ICAO:      "URMN",
		IATA:      "NAL",
		Name:      "Nalchik Airport",
		City:      "Nalchik",
		State:     "Kabardino-Balkariya",
		Country:   "RU",
		Elevation: 1461,
		Latitude:  43.5129013062,
		Longitude: 43.6366004944,
		Timezone:  "Europe/Moscow",
	},
	"URMO": {
		ICAO:      "URMO",
		IATA:      "OGZ",
		Name:      "Beslan Airport",
		City:      "Beslan",
		State:     "North-Ossetia",
		Country:   "RU",
		Elevation: 1673,
		Latitude:  43.2051010132,
		Longitude: 44.6066017151,
		Timezone:  "Europe/Moscow",
	},
	"URMS": {
		ICAO:      "URMS",
		IATA:      "IGT",
		Name:      "Magas Airport",
		City:      "Magas",
		State:     "Ingushetiya",
		Country:   "RU",
		Elevation: 1165,
		Latitude:  43.3222999573,
		Longitude: 45.0125999451,
		Timezone:  "Europe/Moscow",
	},
	"URMT": {
		ICAO:      "URMT",
		IATA:      "STW",
		Name:      "Stavropol Shpakovskoye Airport",
		City:      "Stavropol",
		State:     "Stavropol'skiy",
		Country:   "RU",
		Elevation: 1486,
		Latitude:  45.1091995239,
		Longitude: 42.1128005981,
		Timezone:  "Europe/Moscow",
	},
	"UROD": {
		ICAO:      "UROD",
		Name:      "Dudinka Airport",
		City:      "Dudinka",
		Country:   "RU",
		Elevation: 82,
		Latitude:  69.375,
		Longitude: 86.1566696167,
		Timezone:  "Asia/Krasnoyarsk",
	},
	"URRH": {
		ICAO:      "URRH",
		Name:      "Shakhty Airport",
		City:      "Shakhty",
		State:     "Rostov",
		Country:   "RU",
		Latitude:  47.697,
		Longitude: 40.287,
		Timezone:  "Europe/Moscow",
	},
	"URRM": {
		ICAO:      "URRM",
		Name:      "Morozovsk Air Base",
		City:      "Morozovsk",
		State:     "Rostov",
		Country:   "RU",
		Elevation: 400,
		Latitude:  48.313,
		Longitude: 41.791,
		Timezone:  "Europe/Moscow",
	},
	"URRP": {
		ICAO:      "URRP",
		IATA:      "ROV",
		Name:      "Platov International Airport",
		City:      "Rostov-on-Don",
		State:     "Rostov",
		Country:   "RU",
		Elevation: 282,
		Latitude:  47.50083333,
		Longitude: 39.93361111,
		Timezone:  "Europe/Moscow",
	},
	"URRR": {
		ICAO:      "URRR",
		IATA:      "RVI",
		Name:      "Rostov-na-Donu Airport",
		City:      "Rostov-on-Don",
		State:     "Rostov",
		Country:   "RU",
		Elevation: 280,
		Latitude:  47.2582015991,
		Longitude: 39.8180999756,
		Timezone:  "Europe/Moscow",
	},
	"URRS": {
		ICAO:      "URRS",
		Name:      "Salsk Airport",
		City:      "Salsk",
		State:     "Rostov",
		Country:   "RU",
		Latitude:  46.4575500983,
		Longitude: 41.4833450317,
		Timezone:  "Europe/Moscow",
	},
	"URRT": {
		ICAO:      "URRT",
		IATA:      "TGK",
		Name:      "Taganrog Yuzhny Airport",
		City:      "Taganrog",
		State:     "Rostov",
		Country:   "RU",
		Elevation: 118,
		Latitude:  47.2000007629,
		Longitude: 38.8499984741,
		Timezone:  "Europe/Moscow",
	},
	"URRW": {
		ICAO:      "URRW",
		Name:      "Veshenskaya Airport",
		Country:   "RU",
		Elevation: 253,
		Latitude:  49.6566009521,
		Longitude: 41.7006988525,
		Timezone:  "Europe/Moscow",
	},
	"URRY": {
		ICAO:      "URRY",
		IATA:      "VLK",
		Name:      "Zavetnoe Airport",
		State:     "Rostov",
		Country:   "RU",
		Elevation: 431,
		Latitude:  46.5443000793,
		Longitude: 43.6456985474,
		Timezone:  "Europe/Moscow",
	},
	"URSS": {
		ICAO:      "URSS",
		IATA:      "AER",
		Name:      "Sochi International Airport",
		City:      "Sochi",
		State:     "Krasnodarskiy",
		Country:   "RU",
		Elevation: 89,
		Latitude:  43.4499015808,
		Longitude: 39.9566001892,
		Timezone:  "Europe/Moscow",
	},
	"URWA": {
		ICAO:      "URWA",
		IATA:      "ASF",
		Name:      "Astrakhan Airport",
		City:      "Astrakhan",
		State:     "Astrakhan",
		Country:   "RU",
		Elevation: -65,
		Latitude:  46.2832984924,
		Longitude: 48.0063018799,
		Timezone:  "Europe/Astrakhan",
	},
	"URWI": {
		ICAO:      "URWI",
		IATA:      "ESL",
		Name:      "Elista Airport",
		City:      "Elista",
		State:     "Kalmykiya",
		Country:   "RU",
		Elevation: 501,
		Latitude:  46.3739013672,
		Longitude: 44.3308982849,
		Timezone:  "Europe/Moscow",
	},
	"URWW": {
		ICAO:      "URWW",
		IATA:      "VOG",
		Name:      "Volgograd International Airport",
		City:      "Volgograd",
		State:     "Volgograd",
		Country:   "RU",
		Elevation: 482,
		Latitude:  48.7825012207,
		Longitude: 44.345500946,
		Timezone:  "Europe/Volgograd",
	},
	"USCC": {
		ICAO:      "USCC",
		IATA:      "CEK",
		Name:      "Chelyabinsk Balandino Airport",
		City:      "Chelyabinsk",
		State:     "Chelyabinsk",
		Country:   "RU",
		Elevation: 769,
		Latitude:  55.3058013916,
		Longitude: 61.5032997131,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USCG": {
		ICAO:      "USCG",
		Name:      "Chelyabinsk Shagol Airport",
		State:     "Chelyabinsk",
		Country:   "RU",
		Elevation: 830,
		Latitude:  55.2599983215,
		Longitude: 61.2999992371,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USCM": {
		ICAO:      "USCM",
		IATA:      "MQF",
		Name:      "Magnitogorsk International Airport",
		City:      "Magnitogorsk",
		State:     "Bashkortostan",
		Country:   "RU",
		Elevation: 1430,
		Latitude:  53.3931007385,
		Longitude: 58.7556991577,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USCV": {
		ICAO:      "USCV",
		Name:      "Kalachevo Airfield",
		City:      "Chelyabinsk",
		State:     "Chelyabinsk",
		Country:   "RU",
		Latitude:  54.952999115,
		Longitude: 61.5,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USDA": {
		ICAO:      "USDA",
		IATA:      "SBT",
		Name:      "Sabetta Airport",
		City:      "Sabetta",
		State:     "Yamalo-Nenets",
		Country:   "RU",
		Elevation: 46,
		Latitude:  71.215,
		Longitude: 72.038,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USDB": {
		ICAO:      "USDB",
		IATA:      "BVJ",
		Name:      "Bovanenkovo",
		City:      "Bovanenkovo",
		State:     "Yamalo-Nenets",
		Country:   "RU",
		Elevation: 23,
		Latitude:  70.315277778,
		Longitude: 68.333611111,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USDD": {
		ICAO:      "USDD",
		IATA:      "SLY",
		Name:      "Salekhard Airport",
		City:      "Salekhard",
		State:     "Yamalo-Nenets",
		Country:   "RU",
		Elevation: 218,
		Latitude:  66.5907974243,
		Longitude: 66.611000061,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USDH": {
		ICAO:      "USDH",
		Name:      "Kharasavey Airport",
		City:      "Kharasavey",
		State:     "Yamalo-Nenets",
		Country:   "RU",
		Latitude:  71.0999984741,
		Longitude: 66.7509994507,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USDK": {
		ICAO:      "USDK",
		IATA:      "YMK",
		Name:      "Mys Kamenny Airport",
		City:      "Mys Kamennyi",
		State:     "Yamalo-Nenets",
		Country:   "RU",
		Elevation: 98,
		Latitude:  68.483001709,
		Longitude: 73.5670013428,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USDO": {
		ICAO:      "USDO",
		Name:      "Tolka Airport",
		City:      "Tolka",
		State:     "Yamalo-Nenets",
		Country:   "RU",
		Elevation: 92,
		Latitude:  63.9900016785,
		Longitude: 82.0500030518,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USDP": {
		ICAO:      "USDP",
		IATA:      "KKQ",
		Name:      "Krasnoselkup Airport",
		City:      "Krasnoselkup",
		State:     "Yamalo-Nenets",
		Country:   "RU",
		Latitude:  65.717,
		Longitude: 82.455,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USDR": {
		ICAO:      "USDR",
		Name:      "Yar-Sale Airport",
		State:     "Yamalo-Nenets",
		Country:   "RU",
		Elevation: 33,
		Latitude:  66.8529968262,
		Longitude: 70.8450012207,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USDS": {
		ICAO:      "USDS",
		IATA:      "TQL",
		Name:      "Tarko-Sale Airport",
		City:      "Tarko-Sale",
		State:     "Yamalo-Nenets",
		Country:   "RU",
		Elevation: 82,
		Latitude:  64.9308013916,
		Longitude: 77.8180999756,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USDT": {
		ICAO:      "USDT",
		Name:      "Tazovskiy Airport",
		City:      "Tazovskiy",
		State:     "Yamalo-Nenets",
		Country:   "RU",
		Elevation: 92,
		Latitude:  67.4841995239,
		Longitude: 78.6444015503,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USDU": {
		ICAO:      "USDU",
		IATA:      "UEN",
		Name:      "Urengoy Airport",
		City:      "Urengoy",
		State:     "Yamalo-Nenets",
		Country:   "RU",
		Elevation: 56,
		Latitude:  65.9599990845,
		Longitude: 78.43699646,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USHB": {
		ICAO:      "USHB",
		IATA:      "EZV",
		Name:      "Berezovo Airport",
		State:     "Khanty-Mansia",
		Country:   "RU",
		Elevation: 98,
		Latitude:  63.9210014343,
		Longitude: 65.0305023193,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USHH": {
		ICAO:      "USHH",
		IATA:      "HMA",
		Name:      "Khanty Mansiysk Airport",
		City:      "Khanty-Mansiysk",
		State:     "Khanty-Mansia",
		Country:   "RU",
		Elevation: 76,
		Latitude:  61.0284996033,
		Longitude: 69.0860977173,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USHI": {
		ICAO:      "USHI",
		IATA:      "IRM",
		Name:      "Igrim Airport",
		State:     "Khanty-Mansia",
		Country:   "RU",
		Elevation: 79,
		Latitude:  63.1987991333,
		Longitude: 64.4393005371,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USHK": {
		ICAO:      "USHK",
		IATA:      "KXD",
		Name:      "Kondinskoye Airport",
		City:      "Kondinskoye",
		State:     "Khanty-Mansia",
		Country:   "RU",
		Elevation: 131,
		Latitude:  59.6550827026,
		Longitude: 67.4300460815,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USHL": {
		ICAO:      "USHL",
		Name:      "Lugovoy Airport",
		City:      "Lugovoy",
		State:     "Khanty-Mansia",
		Country:   "RU",
		Latitude:  59.7239990234,
		Longitude: 65.8330001831,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USHN": {
		ICAO:      "USHN",
		IATA:      "NYA",
		Name:      "Nyagan Airport",
		City:      "Nyagan",
		State:     "Khanty-Mansia",
		Country:   "RU",
		Elevation: 361,
		Latitude:  62.1100006104,
		Longitude: 65.6149978638,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USHS": {
		ICAO:      "USHS",
		IATA:      "OVS",
		Name:      "Sovetskiy Airport",
		City:      "Sovetskiy",
		State:     "Khanty-Mansia",
		Country:   "RU",
		Elevation: 351,
		Latitude:  61.3266220093,
		Longitude: 63.6019134521,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USHU": {
		ICAO:      "USHU",
		IATA:      "URJ",
		Name:      "Uray Airport",
		City:      "Uray",
		State:     "Khanty-Mansia",
		Country:   "RU",
		Elevation: 190,
		Latitude:  60.1032981873,
		Longitude: 64.8266983032,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USII": {
		ICAO:      "USII",
		IATA:      "IJK",
		Name:      "Izhevsk Airport",
		City:      "Izhevsk",
		State:     "Udmurtiya",
		Country:   "RU",
		Elevation: 531,
		Latitude:  56.8280982971,
		Longitude: 53.4575004578,
		Timezone:  "Europe/Samara",
	},
	"USKK": {
		ICAO:      "USKK",
		IATA:      "KVX",
		Name:      "Pobedilovo Airport",
		City:      "Kirov",
		State:     "Kirov",
		Country:   "RU",
		Elevation: 479,
		Latitude:  58.5032997131,
		Longitude: 49.3483009338,
		Timezone:  "Europe/Kirov",
	},
	"USMM": {
		ICAO:      "USMM",
		IATA:      "NYM",
		Name:      "Nadym Airport",
		City:      "Nadym",
		State:     "Yamalo-Nenets",
		Country:   "RU",
		Elevation: 49,
		Latitude:  65.4809036255,
		Longitude: 72.6988983154,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USMQ": {
		ICAO:      "USMQ",
		Name:      "Yamburg",
		City:      "Yamburg",
		State:     "Yamalo-Nenets",
		Country:   "RU",
		Elevation: 108,
		Latitude:  67.988611,
		Longitude: 75.098333,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USMU": {
		ICAO:      "USMU",
		IATA:      "NUX",
		Name:      "Novy Urengoy Airport",
		City:      "Novy Urengoy",
		State:     "Yamalo-Nenets",
		Country:   "RU",
		Elevation: 210,
		Latitude:  66.0693969727,
		Longitude: 76.5203018188,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USNN": {
		ICAO:      "USNN",
		IATA:      "NJC",
		Name:      "Nizhnevartovsk Airport",
		City:      "Nizhnevartovsk",
		State:     "Khanty-Mansia",
		Country:   "RU",
		Elevation: 177,
		Latitude:  60.9492988586,
		Longitude: 76.4835968018,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USPB": {
		ICAO:      "USPB",
		Name:      "Bakharevka Airport",
		City:      "Perm",
		Country:   "RU",
		Elevation: 564,
		Latitude:  57.9516983032,
		Longitude: 56.1949996948,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USPP": {
		ICAO:      "USPP",
		IATA:      "PEE",
		Name:      "Bolshoye Savino Airport",
		City:      "Perm",
		State:     "Perm",
		Country:   "RU",
		Elevation: 404,
		Latitude:  57.9145011902,
		Longitude: 56.0211982727,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USPT": {
		ICAO:      "USPT",
		Name:      "Berezniki Airport",
		City:      "Solikamsk",
		Country:   "RU",
		Elevation: 207,
		Latitude:  59.5795478821,
		Longitude: 56.8576812744,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USRK": {
		ICAO:      "USRK",
		IATA:      "KGP",
		Name:      "Kogalym International Airport",
		City:      "Kogalym",
		State:     "Khanty-Mansia",
		Country:   "RU",
		Elevation: 220,
		Latitude:  62.1903991699,
		Longitude: 74.5337982178,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USRN": {
		ICAO:      "USRN",
		IATA:      "NFG",
		Name:      "Nefteyugansk Airport",
		City:      "Nefteyugansk",
		State:     "Khanty-Mansia",
		Country:   "RU",
		Elevation: 115,
		Latitude:  61.1082992554,
		Longitude: 72.6500015259,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USRO": {
		ICAO:      "USRO",
		IATA:      "NOJ",
		Name:      "Noyabrsk Airport",
		City:      "Noyabrsk",
		State:     "Yamalo-Nenets",
		Country:   "RU",
		Elevation: 446,
		Latitude:  63.1833000183,
		Longitude: 75.2699966431,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USRR": {
		ICAO:      "USRR",
		IATA:      "SGC",
		Name:      "Surgut Airport",
		City:      "Surgut",
		State:     "Khanty-Mansia",
		Country:   "RU",
		Elevation: 200,
		Latitude:  61.3437004089,
		Longitude: 73.401802063,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USSK": {
		ICAO:      "USSK",
		Name:      "Uktus Airport",
		City:      "Yekaterinburg",
		State:     "Sverdlovsk",
		Country:   "RU",
		Elevation: 643,
		Latitude:  56.7016983032,
		Longitude: 60.7900009155,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USSS": {
		ICAO:      "USSS",
		IATA:      "SVX",
		Name:      "Koltsovo Airport",
		City:      "Yekaterinburg",
		State:     "Sverdlovsk",
		Country:   "RU",
		Elevation: 764,
		Latitude:  56.7430992126,
		Longitude: 60.8027000427,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USTL": {
		ICAO:      "USTL",
		Name:      "Plekhanovo Airport",
		City:      "Tyumen",
		State:     "Tyumenskaya",
		Country:   "RU",
		Elevation: 308,
		Latitude:  57.1433334351,
		Longitude: 65.4683303833,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USTO": {
		ICAO:      "USTO",
		IATA:      "TOX",
		Name:      "Tobolsk Airport",
		City:      "Tobolsk",
		State:     "Tyumenskaya",
		Country:   "RU",
		Elevation: 167,
		Latitude:  58.135799408,
		Longitude: 68.2319030762,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USTR": {
		ICAO:      "USTR",
		IATA:      "TJM",
		Name:      "Roshchino International Airport",
		City:      "Tyumen",
		State:     "Tyumenskaya",
		Country:   "RU",
		Elevation: 378,
		Latitude:  57.1896018982,
		Longitude: 65.3243026733,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USUK": {
		ICAO:      "USUK",
		Name:      "Kurtamysh Airport",
		City:      "Kurtamysh",
		State:     "Kurgan",
		Country:   "RU",
		Latitude:  54.9150009155,
		Longitude: 64.4670028687,
		Timezone:  "Asia/Yekaterinburg",
	},
	"USUU": {
		ICAO:      "USUU",
		IATA:      "KRO",
		Name:      "Kurgan Airport",
		City:      "Kurgan",
		State:     "Kurgan",
		Country:   "RU",
		Elevation: 240,
		Latitude:  55.4752998352,
		Longitude: 65.4156036377,
		Timezone:  "Asia/Yekaterinburg",
	},
	"UT00": {
		ICAO:      "UT00",
		Name:      "Swains Creek Airport",
		City:      "Alton",
		State:     "Utah",
		Country:   "US",
		Elevation: 7780,
		Latitude:  37.4721984863,
		Longitude: -112.6220016479,
		Timezone:  "America/Denver",
	},
	"UT02": {
		ICAO:      "UT02",
		Name:      "Sandy Ranch Airport",
		City:      "Torrey",
		State:     "Utah",
		Country:   "US",
		Elevation: 5468,
		Latitude:  38.0889015198,
		Longitude: -111.0670013428,
		Timezone:  "America/Denver",
	},
	"UT03": {
		ICAO:      "UT03",
		Name:      "Hite Airport",
		City:      "Hanksville",
		State:     "Utah",
		Country:   "US",
		Elevation: 3840,
		Latitude:  37.8947016926,
		Longitude: -110.379123688,
		Timezone:  "America/Denver",
	},
	"UT06": {
		ICAO:      "UT06",
		Name:      "Rogers Roost Airport",
		City:      "Thompson Springs",
		State:     "Utah",
		Country:   "US",
		Elevation: 5100,
		Latitude:  38.9665985107,
		Longitude: -109.7200012207,
		Timezone:  "America/Denver",
	},
	"UT07": {
		ICAO:      "UT07",
		Name:      "Air Village Strip",
		City:      "Vernal",
		State:     "Utah",
		Country:   "US",
		Elevation: 5800,
		Latitude:  40.4249992371,
		Longitude: -109.5920028687,
		Timezone:  "America/Denver",
	},
	"UT08": {
		ICAO:      "UT08",
		Name:      "Camp Williams Airfield",
		City:      "Camp Williams",
		State:     "Utah",
		Country:   "US",
		Elevation: 4860,
		Latitude:  40.4319000244,
		Longitude: -111.930999756,
		Timezone:  "America/Denver",
	},
	"UT09": {
		ICAO:      "UT09",
		Name:      "Tavapats Ranch Airport",
		City:      "Green River",
		State:     "Utah",
		Country:   "US",
		Elevation: 9200,
		Latitude:  39.4832992554,
		Longitude: -110.1510009766,
		Timezone:  "America/Denver",
	},
	"UT10": {
		ICAO:      "UT10",
		Name:      "Cedar Valley Airport",
		City:      "Cedar Fort",
		State:     "Utah",
		Country:   "US",
		Elevation: 5000,
		Latitude:  40.3572006226,
		Longitude: -112.0179977417,
		Timezone:  "America/Denver",
	},
	"UT13": {
		ICAO:      "UT13",
		Name:      "Number 18 Airport",
		City:      "South Jordan",
		State:     "Utah",
		Country:   "US",
		Elevation: 4820,
		Latitude:  40.5638999939,
		Longitude: -112.013999939,
		Timezone:  "America/Denver",
	},
	"UT15": {
		ICAO:      "UT15",
		Name:      "Flying Cal Ute Rancheros Airport",
		City:      "Paragonah",
		State:     "Utah",
		Country:   "US",
		Elevation: 5764,
		Latitude:  37.9286003113,
		Longitude: -112.7689971924,
		Timezone:  "America/Denver",
	},
	"UT17": {
		ICAO:      "UT17",
		Name:      "Pfeiler Ranch Airport",
		City:      "Panguitch",
		State:     "Utah",
		Country:   "US",
		Elevation: 7040,
		Latitude:  37.9322013855,
		Longitude: -112.3320007324,
		Timezone:  "America/Denver",
	},
	"UT1M": {
		ICAO:      "UT1M",
		Name:      "Kakaydy Airport",
		City:      "Goran",
		State:     "Surxondaryo",
		Country:   "UZ",
		Elevation: 1207,
		Latitude:  37.6242980957,
		Longitude: 67.5176010132,
		Timezone:  "Asia/Samarkand",
	},
	"UT1N": {
		ICAO:      "UT1N",
		Name:      "Karshi South Airport",
		City:      "Karshi",
		State:     "Qashqadaryo",
		Country:   "UZ",
		Elevation: 1194,
		Latitude:  38.8018989563,
		Longitude: 65.7746963501,
		Timezone:  "Asia/Samarkand",
	},
	"UT1O": {
		ICAO:      "UT1O",
		Name:      "Beleuli North Airport",
		City:      "Beleuli",
		State:     "Karakalpakstan",
		Country:   "UZ",
		Elevation: 413,
		Latitude:  44.0541000366,
		Longitude: 57.5477981567,
		Timezone:  "Asia/Samarkand",
	},
	"UT1P": {
		ICAO:      "UT1P",
		Name:      "Kagan South Airport",
		City:      "Kagan",
		State:     "Bukhara",
		Country:   "UZ",
		Elevation: 689,
		Latitude:  39.6893997192,
		Longitude: 64.5503997803,
		Timezone:  "Asia/Samarkand",
	},
	"UT1Q": {
		ICAO:      "UT1Q",
		Name:      "Pakhtakor Airport",
		City:      "Pakhtakor",
		State:     "Jizzax",
		Country:   "UZ",
		Elevation: 1023,
		Latitude:  40.2527008057,
		Longitude: 67.9107971191,
		Timezone:  "Asia/Tashkent",
	},
	"UT24": {
		ICAO:      "UT24",
		Name:      "Strawberry Valley Estates Airport",
		City:      "Alton",
		State:     "Utah",
		Country:   "US",
		Elevation: 8141,
		Latitude:  37.4933013916,
		Longitude: -112.6449966431,
		Timezone:  "America/Denver",
	},
	"UT25": {
		ICAO:      "UT25",
		Name:      "Monument Valley Airport",
		City:      "Monument Valley",
		State:     "Utah",
		Country:   "US",
		Elevation: 5192,
		Latitude:  37.0167007446,
		Longitude: -110.2009963989,
		Timezone:  "America/Denver",
	},
	"UT26": {
		ICAO:      "UT26",
		Name:      "Bryce Woodland Estates Landing Strip",
		City:      "Hatch",
		State:     "Utah",
		Country:   "US",
		Elevation: 7600,
		Latitude:  37.584400177,
		Longitude: -112.391998291,
		Timezone:  "America/Denver",
	},
	"UT27": {
		ICAO:      "UT27",
		Name:      "Hurricane Mesa Airport",
		City:      "Hurricane",
		State:     "Utah",
		Country:   "US",
		Elevation: 5105,
		Latitude:  37.2510986328,
		Longitude: -113.2089996338,
		Timezone:  "America/Denver",
	},
	"UT28": {
		ICAO:      "UT28",
		Name:      "Sun Valley Estates Airport",
		City:      "Lund",
		State:     "Utah",
		Country:   "US",
		Elevation: 5110,
		Latitude:  37.9749984741,
		Longitude: -113.4629974365,
		Timezone:  "America/Denver",
	},
	"UT29": {
		ICAO:      "UT29",
		Name:      "Jack's Airport",
		City:      "Tremonton",
		State:     "Utah",
		Country:   "US",
		Elevation: 4440,
		Latitude:  41.7338981628,
		Longitude: -112.1940002441,
		Timezone:  "America/Denver",
	},
	"UT30": {
		ICAO:      "UT30",
		Name:      "Deer Springs Ranch Airport",
		City:      "Kanab",
		State:     "Utah",
		Country:   "US",
		Elevation: 6485,
		Latitude:  37.3424987793,
		Longitude: -112.2239990234,
		Timezone:  "America/Denver",
	},
	"UT37": {
		ICAO:      "UT37",
		Name:      "Carmel Mountain Ranch Airport",
		City:      "Mount Carmel",
		State:     "Utah",
		Country:   "US",
		Elevation: 5590,
		Latitude:  37.2668991089,
		Longitude: -112.7009963989,
		Timezone:  "America/Denver",
	},
	"UT40": {
		ICAO:      "UT40",
		Name:      "Payne Field",
		City:      "Syracuse",
		State:     "Utah",
		Country:   "US",
		Elevation: 4228,
		Latitude:  41.0983009338,
		Longitude: -112.1159973145,
		Timezone:  "America/Denver",
	},
	"UT41": {
		ICAO:      "UT41",
		Name:      "Glenmar Ranch Airport",
		City:      "Cedar Fort",
		State:     "Utah",
		Country:   "US",
		Elevation: 5030,
		Latitude:  40.3394012451,
		Longitude: -111.9990005493,
		Timezone:  "America/Denver",
	},
	"UT42": {
		ICAO:      "UT42",
		Name:      "Westwater Airport",
		City:      "Cisco",
		State:     "Utah",
		Country:   "US",
		Elevation: 4660,
		Latitude:  39.1467018127,
		Longitude: -109.1449966431,
		Timezone:  "America/Denver",
	},
	"UT43": {
		ICAO:      "UT43",
		Name:      "Citabriair Airport",
		City:      "Kanarraville",
		State:     "Utah",
		Country:   "US",
		Elevation: 5600,
		Latitude:  37.636100769,
		Longitude: -113.2470016479,
		Timezone:  "America/Denver",
	},
	"UT47": {
		ICAO:      "UT47",
		Name:      "Grassy Meadows/Sky Ranch Landowners Assn Airport",
		City:      "Hurricane",
		State:     "Utah",
		Country:   "US",
		Elevation: 3350,
		Latitude:  37.1018981934,
		Longitude: -113.314002991,
		Timezone:  "America/Denver",
	},
	"UT48": {
		ICAO:      "UT48",
		Name:      "Lbl Farms Airport",
		City:      "Clinton",
		State:     "Utah",
		Country:   "US",
		Elevation: 4260,
		Latitude:  41.1501998901,
		Longitude: -112.0989990234,
		Timezone:  "America/Denver",
	},
	"UT49": {
		ICAO:      "UT49",
		Name:      "Desert Aviation Airport",
		City:      "Delta",
		State:     "Utah",
		Country:   "US",
		Elevation: 4589,
		Latitude:  39.459400177,
		Longitude: -112.6549987793,
		Timezone:  "America/Denver",
	},
	"UT53": {
		ICAO:      "UT53",
		Name:      "Sky Ranch Airport",
		City:      "Moab",
		State:     "Utah",
		Country:   "US",
		Elevation: 4875,
		Latitude:  38.4878005981,
		Longitude: -109.4489974976,
		Timezone:  "America/Denver",
	},
	"UT54": {
		ICAO:      "UT54",
		Name:      "Crystal Springs Ranch Airport",
		City:      "Leeds",
		State:     "Utah",
		Country:   "US",
		Elevation: 3671,
		Latitude:  37.2532997131,
		Longitude: -113.3339996338,
		Timezone:  "America/Denver",
	},
	"UT57": {
		ICAO:      "UT57",
		Name:      "Sulphurdale Airport",
		City:      "Beaver",
		State:     "Utah",
		Country:   "US",
		Elevation: 6060,
		Latitude:  38.5724983215,
		Longitude: -112.5920028687,
		Timezone:  "America/Denver",
	},
	"UT59": {
		ICAO:      "UT59",
		Name:      "Needles Outpost Airport",
		City:      "Monticello",
		State:     "Utah",
		Country:   "US",
		Elevation: 4950,
		Latitude:  38.1758003235,
		Longitude: -109.7440032959,
		Timezone:  "America/Denver",
	},
	"UT65": {
		ICAO:      "UT65",
		Name:      "Goshute Airport",
		City:      "Ibapah",
		State:     "Utah",
		Country:   "US",
		Elevation: 5460,
		Latitude:  39.9980010986,
		Longitude: -113.9769973755,
		Timezone:  "America/Denver",
	},
	"UT67": {
		ICAO:      "UT67",
		Name:      "Paradise Air Park",
		City:      "Tridell",
		State:     "Utah",
		Country:   "US",
		Elevation: 5700,
		Latitude:  40.4411010742,
		Longitude: -109.8470001221,
		Timezone:  "America/Denver",
	},
	"UT68": {
		ICAO:      "UT68",
		Name:      "Tangri-La Airport",
		City:      "Moab",
		State:     "Utah",
		Country:   "US",
		Elevation: 3940,
		Latitude:  38.4728012085,
		Longitude: -109.6600036621,
		Timezone:  "America/Denver",
	},
	"UT69": {
		ICAO:      "UT69",
		Name:      "Pelican Lake Airport",
		City:      "Randlett",
		State:     "Utah",
		Country:   "US",
		Elevation: 4815,
		Latitude:  40.1814002991,
		Longitude: -109.6719970703,
		Timezone:  "America/Denver",
	},
	"UT70": {
		ICAO:      "UT70",
		Name:      "Garrison Airport",
		City:      "Garrison",
		State:     "Utah",
		Country:   "US",
		Elevation: 5225,
		Latitude:  38.9649009705,
		Longitude: -114.0370025635,
		Timezone:  "America/Denver",
	},
	"UT73": {
		ICAO:      "UT73",
		Name:      "Maymanak Airport",
		City:      "Maynanak",
		State:     "Qashqadaryo",
		Country:   "UZ",
		Elevation: 1047,
		Latitude:  39.1377983093,
		Longitude: 65.1652984619,
		Timezone:  "Asia/Samarkand",
	},
	"UT74": {
		ICAO:      "UT74",
		Name:      "Fry Canyon Field",
		City:      "Fry Canyon",
		State:     "Utah",
		Country:   "US",
		Elevation: 5372,
		Latitude:  37.6483001709,
		Longitude: -110.1669998169,
		Timezone:  "America/Denver",
	},
	"UT75": {
		ICAO:      "UT75",
		Name:      "Mineral Canyon Strip",
		City:      "Moab",
		State:     "Utah",
		Country:   "US",
		Elevation: 3900,
		Latitude:  38.5315831779,
		Longitude: -109.995889664,
		Timezone:  "America/Denver",
	},
	"UT77": {
		ICAO:      "UT77",
		Name:      "Kungrad Airport",
		City:      "Kungrad",
		State:     "Karakalpakstan",
		Country:   "UZ",
		Elevation: 180,
		Latitude:  43.0830001831,
		Longitude: 58.8842010498,
		Timezone:  "Asia/Samarkand",
	},
	"UT79": {
		ICAO:      "UT79",
		Name:      "3-I Rocker Ranch Airport",
		City:      "Jensen",
		State:     "Utah",
		Country:   "US",
		Elevation: 4865,
		Latitude:  40.415599823,
		Longitude: -109.34400177,
		Timezone:  "America/Denver",
	},
	"UT80": {
		ICAO:      "UT80",
		Name:      "Hamilton Fort Ranch Airport",
		City:      "Cedar City",
		State:     "Utah",
		Country:   "US",
		Elevation: 5460,
		Latitude:  37.6347007751,
		Longitude: -113.158996582,
		Timezone:  "America/Denver",
	},
	"UT82": {
		ICAO:      "UT82",
		Name:      "Beryl Junction Airport",
		City:      "Beryl",
		State:     "Utah",
		Country:   "US",
		Elevation: 5181,
		Latitude:  37.7099990845,
		Longitude: -113.6460037231,
		Timezone:  "America/Denver",
	},
	"UT83": {
		ICAO:      "UT83",
		Name:      "Thunder Ridge Airpark",
		City:      "Fruitland",
		State:     "Utah",
		Country:   "US",
		Elevation: 7050,
		Latitude:  40.2578010559,
		Longitude: -110.8590011597,
		Timezone:  "America/Denver",
	},
	"UT97": {
		ICAO:      "UT97",
		Name:      "Happy Canyon Airport",
		City:      "Happy Canyon",
		State:     "Utah",
		Country:   "US",
		Elevation: 4934,
		Latitude:  38.1571998596,
		Longitude: -110.2919998169,
		Timezone:  "America/Denver",
	},
	"UT99": {
		ICAO:      "UT99",
		Name:      "West Desert Airpark",
		City:      "Cedar Valley",
		State:     "Utah",
		Country:   "US",
		Elevation: 4902,
		Latitude:  40.2425994873,
		Longitude: -112.0930023193,
		Timezone:  "America/Denver",
	},
	"UTAA": {
		ICAO:      "UTAA",
		IATA:      "ASB",
		Name:      "Ashgabat Airport",
		City:      "Ashgabat",
		State:     "Ashgabat",
		Country:   "TM",
		Elevation: 692,
		Latitude:  37.9868011475,
		Longitude: 58.361000061,
		Timezone:  "Asia/Ashgabat",
	},
	"UTAK": {
		ICAO:      "UTAK",
		IATA:      "KRW",
		Name:      "Turkmenbashi Airport",
		City:      "Krasnovodsk",
		Country:   "TM",
		Elevation: 279,
		Latitude:  40.0633010864,
		Longitude: 53.0071983337,
		Timezone:  "Asia/Ashgabat",
	},
	"UTAM": {
		ICAO:      "UTAM",
		IATA:      "MYP",
		Name:      "Mary Airport",
		City:      "Mary",
		Country:   "TM",
		Elevation: 728,
		Latitude:  37.6194000244,
		Longitude: 61.8967018127,
		Timezone:  "Asia/Ashgabat",
	},
	"UTAT": {
		ICAO:      "UTAT",
		IATA:      "TAZ",
		Name:      "Dashoguz Airport",
		City:      "Dashoguz",
		State:     "Daşoguz",
		Country:   "TM",
		Elevation: 272,
		Latitude:  41.761100769,
		Longitude: 59.8266983032,
		Timezone:  "Asia/Ashgabat",
	},
	"UTAV": {
		ICAO:      "UTAV",
		IATA:      "CRZ",
		Name:      "Turkmenabat Airport",
		City:      "Turkmenabat",
		Country:   "TM",
		Elevation: 630,
		Latitude:  39.0833015442,
		Longitude: 63.6133003235,
		Timezone:  "Asia/Ashgabat",
	},
	"UTDD": {
		ICAO:      "UTDD",
		IATA:      "DYU",
		Name:      "Dushanbe Airport",
		City:      "Dushanbe",
		State:     "Dushanbe",
		Country:   "TJ",
		Elevation: 2575,
		Latitude:  38.5433006287,
		Longitude: 68.8249969482,
		Timezone:  "Asia/Dushanbe",
	},
	"UTDK": {
		ICAO:      "UTDK",
		IATA:      "TJU",
		Name:      "Kulob Airport",
		City:      "Kulyab",
		State:     "Khatlon",
		Country:   "TJ",
		Elevation: 2293,
		Latitude:  37.9880981445,
		Longitude: 69.8050003052,
		Timezone:  "Asia/Dushanbe",
	},
	"UTDL": {
		ICAO:      "UTDL",
		IATA:      "LBD",
		Name:      "Khudzhand Airport",
		City:      "Khudzhand",
		State:     "Viloyati-Sughd",
		Country:   "TJ",
		Elevation: 1450,
		Latitude:  40.2154006958,
		Longitude: 69.6947021484,
		Timezone:  "Asia/Dushanbe",
	},
	"UTDT": {
		ICAO:      "UTDT",
		IATA:      "KQT",
		Name:      "Qurghonteppa International Airport",
		City:      "Kurgan-Tyube",
		State:     "Khatlon",
		Country:   "TJ",
		Elevation: 1473,
		Latitude:  37.8664016724,
		Longitude: 68.8647003174,
		Timezone:  "Asia/Dushanbe",
	},
	"UTKA": {
		ICAO:      "UTKA",
		IATA:      "AZN",
		Name:      "Andizhan Airport",
		City:      "Andizhan",
		State:     "Andijon",
		Country:   "UZ",
		Elevation: 1515,
		Latitude:  40.7276992798,
		Longitude: 72.2939987183,
		Timezone:  "Asia/Tashkent",
	},
	"UTKF": {
		ICAO:      "UTKF",
		IATA:      "FEG",
		Name:      "Fergana Airport",
		City:      "Fergana",
		State:     "Fergana",
		Country:   "UZ",
		Elevation: 1980,
		Latitude:  40.3587989807,
		Longitude: 71.7450027466,
		Timezone:  "Asia/Tashkent",
	},
	"UTKK": {
		ICAO:      "UTKK",
		Name:      "Kokand Airport",
		City:      "Kokand",
		State:     "Fergana",
		Country:   "UZ",
		Elevation: 1357,
		Latitude:  40.5397987366,
		Longitude: 70.9748001099,
		Timezone:  "Asia/Tashkent",
	},
	"UTKN": {
		ICAO:      "UTKN",
		IATA:      "NMA",
		Name:      "Namangan Airport",
		City:      "Namangan",
		State:     "Namangan",
		Country:   "UZ",
		Elevation: 1555,
		Latitude:  40.9846000671,
		Longitude: 71.5567016602,
		Timezone:  "Asia/Tashkent",
	},
	"UTNM": {
		ICAO:      "UTNM",
		Name:      "Muynak Airport",
		City:      "Muynak",
		State:     "Karakalpakstan",
		Country:   "UZ",
		Elevation: 177,
		Latitude:  43.7551994324,
		Longitude: 59.0308990479,
		Timezone:  "Asia/Samarkand",
	},
	"UTNN": {
		ICAO:      "UTNN",
		IATA:      "NCU",
		Name:      "Nukus Airport",
		City:      "Nukus",
		State:     "Karakalpakstan",
		Country:   "UZ",
		Elevation: 246,
		Latitude:  42.4883995056,
		Longitude: 59.623298645,
		Timezone:  "Asia/Samarkand",
	},
	"UTNT": {
		ICAO:      "UTNT",
		Name:      "Turtkul Airport",
		City:      "Turtkul",
		State:     "Karakalpakstan",
		Country:   "UZ",
		Elevation: 305,
		Latitude:  41.5750007629,
		Longitude: 60.9665985107,
		Timezone:  "Asia/Samarkand",
	},
	"UTNU": {
		ICAO:      "UTNU",
		IATA:      "UGC",
		Name:      "Urgench Airport",
		City:      "Urgench",
		State:     "Xorazm",
		Country:   "UZ",
		Elevation: 320,
		Latitude:  41.5843009949,
		Longitude: 60.6417007446,
		Timezone:  "Asia/Samarkand",
	},
	"UTOD": {
		ICAO:      "UTOD",
		Name:      "Khorog Airport",
		City:      "Khorog",
		State:     "Gorno-Badakhshan",
		Country:   "TJ",
		Elevation: 6700,
		Latitude:  37.502222,
		Longitude: 71.513333,
		Timezone:  "Asia/Dushanbe",
	},
	"UTSA": {
		ICAO:      "UTSA",
		IATA:      "NVI",
		Name:      "Navoi Airport",
		City:      "Navoi",
		State:     "Navoiy",
		Country:   "UZ",
		Latitude:  40.1171989441,
		Longitude: 65.1707992554,
		Timezone:  "Asia/Samarkand",
	},
	"UTSB": {
		ICAO:      "UTSB",
		IATA:      "BHK",
		Name:      "Bukhara Airport",
		City:      "Bukhara",
		Country:   "UZ",
		Elevation: 751,
		Latitude:  39.7750015259,
		Longitude: 64.4832992554,
		Timezone:  "Asia/Samarkand",
	},
	"UTSH": {
		ICAO:      "UTSH",
		Name:      "Shakhristabz Airport",
		City:      "Shakhristabz",
		State:     "Qashqadaryo",
		Country:   "UZ",
		Elevation: 1900,
		Latitude:  39.0690994263,
		Longitude: 66.7543029785,
		Timezone:  "Asia/Samarkand",
	},
	"UTSK": {
		ICAO:      "UTSK",
		IATA:      "KSQ",
		Name:      "Karshi Airport",
		City:      "Karshi",
		State:     "Qashqadaryo",
		Country:   "UZ",
		Elevation: 1365,
		Latitude:  38.8025,
		Longitude: 65.773056,
		Timezone:  "Asia/Samarkand",
	},
	"UTSL": {
		ICAO:      "UTSL",
		IATA:      "KSQ",
		Name:      "Karshi Khanabad Airport",
		City:      "Khanabad",
		State:     "Qashqadaryo",
		Country:   "UZ",
		Elevation: 1365,
		Latitude:  38.8335990906,
		Longitude: 65.9215011597,
		Timezone:  "Asia/Samarkand",
	},
	"UTSM": {
		ICAO:      "UTSM",
		Name:      "Tandy Bulak Airport",
		City:      "Tandy Bulak",
		State:     "Navoiy",
		Country:   "UZ",
		Elevation: 716,
		Latitude:  41.7607917786,
		Longitude: 64.6024017334,
		Timezone:  "Asia/Samarkand",
	},
	"UTSN": {
		ICAO:      "UTSN",
		IATA:      "AFS",
		Name:      "Sugraly Airport",
		City:      "Zarafshan",
		State:     "Navoiy",
		Country:   "UZ",
		Elevation: 1396,
		Latitude:  41.613899231,
		Longitude: 64.2332000732,
		Timezone:  "Asia/Samarkand",
	},
	"UTSR": {
		ICAO:      "UTSR",
		Name:      "Sariasiya Airport",
		City:      "Sariasiya",
		State:     "Surxondaryo",
		Country:   "UZ",
		Elevation: 1949,
		Latitude:  38.4105987549,
		Longitude: 67.9451980591,
		Timezone:  "Asia/Samarkand",
	},
	"UTSS": {
		ICAO:      "UTSS",
		IATA:      "SKD",
		Name:      "Samarkand Airport",
		City:      "Samarkand",
		Country:   "UZ",
		Elevation: 2224,
		Latitude:  39.7005004883,
		Longitude: 66.9838027954,
		Timezone:  "Asia/Samarkand",
	},
	"UTST": {
		ICAO:      "UTST",
		IATA:      "TMJ",
		Name:      "Termez Airport",
		City:      "Termez",
		State:     "Surxondaryo",
		Country:   "UZ",
		Elevation: 1027,
		Latitude:  37.2867012024,
		Longitude: 67.3099975586,
		Timezone:  "Asia/Samarkand",
	},
	"UTSU": {
		ICAO:      "UTSU",
		Name:      "Uchkuduk Airport",
		City:      "Uchkuduk",
		State:     "Navoiy",
		Country:   "UZ",
		Elevation: 416,
		Latitude:  42.0830993652,
		Longitude: 63.4492988586,
		Timezone:  "Asia/Samarkand",
	},
	"UTTP": {
		ICAO:      "UTTP",
		Name:      "Tashkent East Airport",
		City:      "Tashkent",
		State:     "Toshkent",
		Country:   "UZ",
		Elevation: 1574,
		Latitude:  41.3126983643,
		Longitude: 69.3914031982,
		Timezone:  "Asia/Tashkent",
	},
	"UTTT": {
		ICAO:      "UTTT",
		IATA:      "TAS",
		Name:      "Tashkent International Airport",
		City:      "Tashkent",
		State:     "Toshkent-Shahri",
		Country:   "UZ",
		Elevation: 1417,
		Latitude:  41.257900238,
		Longitude: 69.2811965942,
		Timezone:  "Asia/Tashkent",
	},
	"UUBA": {
		ICAO:      "UUBA",
		IATA:      "KMW",
		Name:      "Kostroma Sokerkino Airport",
		City:      "Kostroma",
		State:     "Kostroma",
		Country:   "RU",
		Elevation: 446,
		Latitude:  57.7969017029,
		Longitude: 41.0194015503,
		Timezone:  "Europe/Moscow",
	},
	"UUBB": {
		ICAO:      "UUBB",
		IATA:      "BKA",
		Name:      "Bykovo Airport",
		City:      "Moscow",
		State:     "Moscow-Oblast",
		Country:   "RU",
		Elevation: 427,
		Latitude:  55.6171989441,
		Longitude: 38.0600013733,
		Timezone:  "Europe/Moscow",
	},
	"UUBC": {
		ICAO:      "UUBC",
		IATA:      "KLF",
		Name:      "Grabtsevo Airport",
		City:      "Kaluga",
		State:     "Kaluga",
		Country:   "RU",
		Elevation: 656,
		Latitude:  54.5499992371,
		Longitude: 36.3666687012,
		Timezone:  "Europe/Moscow",
	},
	"UUBD": {
		ICAO:      "UUBD",
		Name:      "Dyagilevo Air Base",
		State:     "Rjazan",
		Country:   "RU",
		Elevation: 440,
		Latitude:  54.6446990967,
		Longitude: 39.570400238,
		Timezone:  "Europe/Moscow",
	},
	"UUBI": {
		ICAO:      "UUBI",
		IATA:      "IWA",
		Name:      "Ivanovo South Airport",
		City:      "Ivanovo",
		Country:   "RU",
		Elevation: 410,
		Latitude:  56.9393997192,
		Longitude: 40.9407997131,
		Timezone:  "Europe/Moscow",
	},
	"UUBK": {
		ICAO:      "UUBK",
		IATA:      "RYB",
		Name:      "Staroselye Airport",
		City:      "Rybinsk",
		State:     "Jaroslavl",
		Country:   "RU",
		Elevation: 423,
		Latitude:  58.1041984558,
		Longitude: 38.9294013977,
		Timezone:  "Europe/Moscow",
	},
	"UUBL": {
		ICAO:      "UUBL",
		Name:      "Semyazino Airport",
		City:      "Vladimir",
		State:     "Vladimir",
		Country:   "RU",
		Elevation: 554,
		Latitude:  56.126701355,
		Longitude: 40.3149986267,
		Timezone:  "Europe/Moscow",
	},
	"UUBM": {
		ICAO:      "UUBM",
		Name:      "Myachkovo Airport",
		City:      "Moscow",
		State:     "Moscow-Oblast",
		Country:   "RU",
		Elevation: 410,
		Latitude:  55.5600013733,
		Longitude: 37.9850006104,
		Timezone:  "Europe/Moscow",
	},
	"UUBN": {
		ICAO:      "UUBN",
		Name:      "Zmeyevo Airport",
		City:      "Tver",
		State:     "Tverskaya",
		Country:   "RU",
		Elevation: 472,
		Latitude:  56.9150009155,
		Longitude: 35.9350013733,
		Timezone:  "Europe/Moscow",
	},
	"UUBP": {
		ICAO:      "UUBP",
		IATA:      "BZK",
		Name:      "Bryansk Airport",
		City:      "Bryansk",
		State:     "Brjansk",
		Country:   "RU",
		Elevation: 663,
		Latitude:  53.2141990662,
		Longitude: 34.176399231,
		Timezone:  "Europe/Moscow",
	},
	"UUBS": {
		ICAO:      "UUBS",
		IATA:      "LNX",
		Name:      "Smolensk South Airport",
		City:      "Smolensk",
		State:     "Smolensk",
		Country:   "RU",
		Elevation: 820,
		Latitude:  54.745,
		Longitude: 32.065,
		Timezone:  "Europe/Moscow",
	},
	"UUBW": {
		ICAO:      "UUBW",
		IATA:      "ZIA",
		Name:      "Ramenskoye Airport",
		City:      "Zhukovsky",
		State:     "Moscow-Oblast",
		Country:   "RU",
		Elevation: 377,
		Latitude:  55.5532989502,
		Longitude: 38.1500015259,
		Timezone:  "Europe/Moscow",
	},
	"UUDD": {
		ICAO:      "UUDD",
		IATA:      "DME",
		Name:      "Domodedovo International Airport",
		City:      "Moscow",
		State:     "Moscow-Oblast",
		Country:   "RU",
		Elevation: 588,
		Latitude:  55.4087982178,
		Longitude: 37.9062995911,
		Timezone:  "Europe/Moscow",
	},
	"UUDL": {
		ICAO:      "UUDL",
		IATA:      "IAR",
		Name:      "Tunoshna Airport",
		State:     "Jaroslavl",
		Country:   "RU",
		Elevation: 287,
		Latitude:  57.5606994629,
		Longitude: 40.1573982239,
		Timezone:  "Europe/Moscow",
	},
	"UUEE": {
		ICAO:      "UUEE",
		IATA:      "SVO",
		Name:      "Sheremetyevo International Airport",
		City:      "Moscow",
		State:     "Moscow-Oblast",
		Country:   "RU",
		Elevation: 622,
		Latitude:  55.9725990295,
		Longitude: 37.4146003723,
		Timezone:  "Europe/Moscow",
	},
	"UUEI": {
		ICAO:      "UUEI",
		Name:      "Kimry Airport",
		City:      "Kimry",
		State:     "Tverskaya",
		Country:   "RU",
		Elevation: 449,
		Latitude:  56.7983016968,
		Longitude: 37.3300018311,
		Timezone:  "Europe/Moscow",
	},
	"UUEM": {
		ICAO:      "UUEM",
		IATA:      "KLD",
		Name:      "Migalovo Air Base",
		City:      "Tver",
		State:     "Tverskaya",
		Country:   "RU",
		Elevation: 469,
		Latitude:  56.8246994019,
		Longitude: 35.7577018738,
		Timezone:  "Europe/Moscow",
	},
	"UUMI": {
		ICAO:      "UUMI",
		Name:      "Stupino Airfield",
		City:      "Stupino",
		State:     "Moscow-Oblast",
		Country:   "RU",
		Elevation: 584,
		Latitude:  54.888,
		Longitude: 38.147,
		Timezone:  "Europe/Moscow",
	},
	"UUML": {
		ICAO:      "UUML",
		Name:      "Severka Airfield",
		City:      "Voskresensk",
		State:     "Moscow-Oblast",
		Country:   "RU",
		Elevation: 433,
		Latitude:  55.206,
		Longitude: 38.677,
		Timezone:  "Europe/Moscow",
	},
	"UUMO": {
		ICAO:      "UUMO",
		IATA:      "OSF",
		Name:      "Ostafyevo International Airport",
		City:      "Moscow",
		State:     "Moscow-Oblast",
		Country:   "RU",
		Elevation: 568,
		Latitude:  55.5116996765,
		Longitude: 37.5071983337,
		Timezone:  "Europe/Moscow",
	},
	"UUMT": {
		ICAO:      "UUMT",
		Name:      "Tretyakovo Airport",
		City:      "Lukhovitsy",
		State:     "Moscow-Oblast",
		Country:   "RU",
		Elevation: 515,
		Latitude:  54.9049987793,
		Longitude: 39.0266685486,
		Timezone:  "Europe/Moscow",
	},
	"UUMU": {
		ICAO:      "UUMU",
		IATA:      "CKL",
		Name:      "Chkalovskiy Airport",
		City:      "Moscow",
		State:     "Moscow-Oblast",
		Country:   "RU",
		Elevation: 499,
		Latitude:  55.8782997131,
		Longitude: 38.0616989136,
		Timezone:  "Europe/Moscow",
	},
	"UUMX": {
		ICAO:      "UUMX",
		Name:      "Blagoveshchenye Airfield",
		City:      "Sergiev Posad",
		State:     "Moscow-Oblast",
		Country:   "RU",
		Latitude:  56.3260002136,
		Longitude: 38.0999984741,
		Timezone:  "Europe/Moscow",
	},
	"UUOB": {
		ICAO:      "UUOB",
		IATA:      "EGO",
		Name:      "Belgorod International Airport",
		City:      "Belgorod",
		State:     "Belgorod",
		Country:   "RU",
		Elevation: 735,
		Latitude:  50.6437988281,
		Longitude: 36.5900993347,
		Timezone:  "Europe/Moscow",
	},
	"UUOD": {
		ICAO:      "UUOD",
		Name:      "Pridacha Airport",
		City:      "Voronezh",
		State:     "Voronezj",
		Country:   "RU",
		Elevation: 344,
		Latitude:  51.6522712708,
		Longitude: 39.2555236816,
		Timezone:  "Europe/Moscow",
	},
	"UUOK": {
		ICAO:      "UUOK",
		IATA:      "URS",
		Name:      "Kursk East Airport",
		City:      "Kursk",
		State:     "Kursk",
		Country:   "RU",
		Elevation: 686,
		Latitude:  51.7505989075,
		Longitude: 36.2956008911,
		Timezone:  "Europe/Moscow",
	},
	"UUOL": {
		ICAO:      "UUOL",
		IATA:      "LPK",
		Name:      "Lipetsk Airport",
		City:      "Lipetsk",
		State:     "Lipetsk",
		Country:   "RU",
		Elevation: 584,
		Latitude:  52.7028007507,
		Longitude: 39.5377998352,
		Timezone:  "Europe/Moscow",
	},
	"UUOO": {
		ICAO:      "UUOO",
		IATA:      "VOZ",
		Name:      "Voronezh International Airport",
		City:      "Voronezh",
		State:     "Voronezj",
		Country:   "RU",
		Elevation: 514,
		Latitude:  51.814201355,
		Longitude: 39.229598999,
		Timezone:  "Europe/Moscow",
	},
	"UUOR": {
		ICAO:      "UUOR",
		IATA:      "OEL",
		Name:      "Oryol Yuzhny Airport",
		City:      "Orel",
		State:     "Orjol",
		Country:   "RU",
		Elevation: 656,
		Latitude:  52.9347000122,
		Longitude: 36.0022010803,
		Timezone:  "Europe/Moscow",
	},
	"UUOS": {
		ICAO:      "UUOS",
		Name:      "Stary Oskol Airport",
		City:      "Stary Oskol",
		State:     "Belgorod",
		Country:   "RU",
		Elevation: 791,
		Latitude:  51.3292160034,
		Longitude: 37.7688484192,
		Timezone:  "Europe/Moscow",
	},
	"UUOT": {
		ICAO:      "UUOT",
		IATA:      "TBW",
		Name:      "Donskoye Airport",
		City:      "Tambov",
		State:     "Tambov",
		Country:   "RU",
		Elevation: 413,
		Latitude:  52.806098938,
		Longitude: 41.48279953,
		Timezone:  "Europe/Moscow",
	},
	"UUUS": {
		ICAO:      "UUUS",
		Name:      "Tushino Airport",
		City:      "Moscow",
		State:     "Moscow",
		Country:   "RU",
		Elevation: 420,
		Latitude:  55.8183326721,
		Longitude: 37.4266662598,
		Timezone:  "Europe/Moscow",
	},
	"UUWE": {
		ICAO:      "UUWE",
		Name:      "Yermolino Airport",
		City:      "Balabanovo",
		State:     "Kaluga",
		Country:   "RU",
		Elevation: 640,
		Latitude:  55.2283325195,
		Longitude: 36.6083335876,
		Timezone:  "Europe/Moscow",
	},
	"UUWR": {
		ICAO:      "UUWR",
		IATA:      "RZN",
		Name:      "Turlatovo Airport",
		City:      "Ryazan",
		State:     "Rjazan",
		Country:   "RU",
		Elevation: 502,
		Latitude:  54.5559005737,
		Longitude: 39.8552017212,
		Timezone:  "Europe/Moscow",
	},
	"UUWV": {
		ICAO:      "UUWV",
		Name:      "Klokovo Airfield",
		City:      "Tula",
		State:     "Tula",
		Country:   "RU",
		Elevation: 499,
		Latitude:  54.239,
		Longitude: 37.6,
		Timezone:  "Europe/Moscow",
	},
	"UUWW": {
		ICAO:      "UUWW",
		IATA:      "VKO",
		Name:      "Vnukovo International Airport",
		City:      "Moscow",
		State:     "Moscow",
		Country:   "RU",
		Elevation: 685,
		Latitude:  55.5914993286,
		Longitude: 37.2615013123,
		Timezone:  "Europe/Moscow",
	},
	"UUYH": {
		ICAO:      "UUYH",
		IATA:      "UCT",
		Name:      "Ukhta Airport",
		City:      "Ukhta",
		State:     "Komi",
		Country:   "RU",
		Elevation: 482,
		Latitude:  63.5668983459,
		Longitude: 53.8046989441,
		Timezone:  "Europe/Moscow",
	},
	"UUYI": {
		ICAO:      "UUYI",
		IATA:      "INA",
		Name:      "Inta Airport",
		City:      "Inta",
		State:     "Komi",
		Country:   "RU",
		Elevation: 184,
		Latitude:  66.0548324585,
		Longitude: 60.1103210449,
		Timezone:  "Europe/Moscow",
	},
	"UUYK": {
		ICAO:      "UUYK",
		Name:      "Vuktyl Airport",
		City:      "Vuktyl",
		State:     "Komi",
		Country:   "RU",
		Elevation: 358,
		Latitude:  63.823299408,
		Longitude: 57.2799987793,
		Timezone:  "Europe/Moscow",
	},
	"UUYM": {
		ICAO:      "UUYM",
		Name:      "Yemva Airport",
		City:      "Emva",
		State:     "Komi",
		Country:   "RU",
		Latitude:  62.6054000854,
		Longitude: 50.925201416,
		Timezone:  "Europe/Moscow",
	},
	"UUYP": {
		ICAO:      "UUYP",
		IATA:      "PEX",
		Name:      "Pechora Airport",
		City:      "Pechora",
		State:     "Komi",
		Country:   "RU",
		Elevation: 98,
		Latitude:  65.1211013794,
		Longitude: 57.1307983398,
		Timezone:  "Europe/Moscow",
	},
	"UUYS": {
		ICAO:      "UUYS",
		IATA:      "USK",
		Name:      "Usinsk Airport",
		City:      "Usinsk",
		State:     "Komi",
		Country:   "RU",
		Elevation: 262,
		Latitude:  66.004699707,
		Longitude: 57.3671989441,
		Timezone:  "Europe/Moscow",
	},
	"UUYT": {
		ICAO:      "UUYT",
		Name:      "Ust-Kulom Airport",
		City:      "Ust-Kulom",
		State:     "Komi",
		Country:   "RU",
		Latitude:  61.6899986267,
		Longitude: 53.7099990845,
		Timezone:  "Europe/Moscow",
	},
	"UUYV": {
		ICAO:      "UUYV",
		Name:      "Izhma Airport",
		City:      "Izhma",
		State:     "Komi",
		Country:   "RU",
		Elevation: 230,
		Latitude:  65.0317001343,
		Longitude: 53.9700012207,
		Timezone:  "Europe/Moscow",
	},
	"UUYW": {
		ICAO:      "UUYW",
		IATA:      "VKT",
		Name:      "Vorkuta Airport",
		City:      "Vorkuta",
		State:     "Komi",
		Country:   "RU",
		Elevation: 604,
		Latitude:  67.4886016846,
		Longitude: 63.9930992126,
		Timezone:  "Europe/Moscow",
	},
	"UUYX": {
		ICAO:      "UUYX",
		IATA:      "UTS",
		Name:      "Ust-Tsylma Airport",
		City:      "Ust-Tsylma",
		State:     "Komi",
		Country:   "RU",
		Elevation: 262,
		Latitude:  65.4372940063,
		Longitude: 52.2003364563,
		Timezone:  "Europe/Moscow",
	},
	"UUYY": {
		ICAO:      "UUYY",
		IATA:      "SCW",
		Name:      "Syktyvkar Airport",
		City:      "Syktyvkar",
		State:     "Komi",
		Country:   "RU",
		Elevation: 342,
		Latitude:  61.6469993591,
		Longitude: 50.8451004028,
		Timezone:  "Europe/Moscow",
	},
	"UWGB": {
		ICAO:      "UWGB",
		Name:      "Bolshoye Boldino Airport",
		City:      "Bolshoye Boldino",
		State:     "Nizjnij-Novgorod",
		Country:   "RU",
		Latitude:  55.0200004578,
		Longitude: 45.3149986267,
		Timezone:  "Europe/Moscow",
	},
	"UWGG": {
		ICAO:      "UWGG",
		IATA:      "GOJ",
		Name:      "Nizhny Novgorod International Airport",
		City:      "Nizhny Novgorod",
		State:     "Novgorod",
		Country:   "RU",
		Elevation: 256,
		Latitude:  56.2300987244,
		Longitude: 43.7840003967,
		Timezone:  "Europe/Moscow",
	},
	"UWKB": {
		ICAO:      "UWKB",
		IATA:      "UUA",
		Name:      "Bugulma Airport",
		City:      "Bugulma",
		State:     "Tatarstan",
		Country:   "RU",
		Elevation: 991,
		Latitude:  54.6399993896,
		Longitude: 52.801700592,
		Timezone:  "Europe/Moscow",
	},
	"UWKD": {
		ICAO:      "UWKD",
		IATA:      "KZN",
		Name:      "Kazan International Airport",
		City:      "Kazan",
		State:     "Tatarstan",
		Country:   "RU",
		Elevation: 411,
		Latitude:  55.6062011719,
		Longitude: 49.2787017822,
		Timezone:  "Europe/Moscow",
	},
	"UWKE": {
		ICAO:      "UWKE",
		IATA:      "NBC",
		Name:      "Begishevo Airport",
		City:      "Nizhnekamsk",
		State:     "Tatarstan",
		Country:   "RU",
		Elevation: 643,
		Latitude:  55.5647010803,
		Longitude: 52.0924987793,
		Timezone:  "Europe/Moscow",
	},
	"UWKG": {
		ICAO:      "UWKG",
		Name:      "Borisoglebskoye Airport",
		City:      "Kazan",
		State:     "Tatarstan",
		Country:   "RU",
		Elevation: 213,
		Latitude:  55.8666992188,
		Longitude: 49.1333007813,
		Timezone:  "Europe/Moscow",
	},
	"UWKI": {
		ICAO:      "UWKI",
		Name:      "Chistopol Airport",
		City:      "Chistopol",
		State:     "Tatarstan",
		Country:   "RU",
		Elevation: 604,
		Latitude:  55.3057975769,
		Longitude: 50.6159858704,
		Timezone:  "Europe/Moscow",
	},
	"UWKJ": {
		ICAO:      "UWKJ",
		IATA:      "JOK",
		Name:      "Yoshkar-Ola Airport",
		City:      "Yoshkar-Ola",
		State:     "Mariy-El",
		Country:   "RU",
		Elevation: 348,
		Latitude:  56.7005996704,
		Longitude: 47.9047012329,
		Timezone:  "Europe/Moscow",
	},
	"UWKS": {
		ICAO:      "UWKS",
		IATA:      "CSY",
		Name:      "Cheboksary Airport",
		City:      "Cheboksary",
		State:     "Chuvashia",
		Country:   "RU",
		Elevation: 558,
		Latitude:  56.0903015137,
		Longitude: 47.3473014832,
		Timezone:  "Europe/Moscow",
	},
	"UWLL": {
		ICAO:      "UWLL",
		IATA:      "ULV",
		Name:      "Ulyanovsk Baratayevka Airport",
		City:      "Ulyanovsk",
		State:     "Ulyanovsk",
		Country:   "RU",
		Elevation: 463,
		Latitude:  54.2682991028,
		Longitude: 48.2266998291,
		Timezone:  "Europe/Ulyanovsk",
	},
	"UWLS": {
		ICAO:      "UWLS",
		Name:      "Soldatskaya Tashla Airfield",
		State:     "Ulyanovsk",
		Country:   "RU",
		Elevation: 673,
		Latitude:  54.0152837323,
		Longitude: 48.3060264587,
		Timezone:  "Europe/Ulyanovsk",
	},
	"UWLW": {
		ICAO:      "UWLW",
		IATA:      "ULY",
		Name:      "Ulyanovsk East Airport",
		City:      "Ulyanovsk",
		State:     "Ulyanovsk",
		Country:   "RU",
		Elevation: 252,
		Latitude:  54.4010009766,
		Longitude: 48.8027000427,
		Timezone:  "Europe/Ulyanovsk",
	},
	"UWOF": {
		ICAO:      "UWOF",
		Name:      "Dombarovskiy Air Base",
		City:      "Dombarovskiy",
		State:     "Orenburg",
		Country:   "RU",
		Elevation: 869,
		Latitude:  50.7999992371,
		Longitude: 59.5167007446,
		Timezone:  "Asia/Yekaterinburg",
	},
	"UWOO": {
		ICAO:      "UWOO",
		IATA:      "REN",
		Name:      "Orenburg Central Airport",
		City:      "Orenburg",
		State:     "Orenburg",
		Country:   "RU",
		Elevation: 387,
		Latitude:  51.7957992554,
		Longitude: 55.4566993713,
		Timezone:  "Asia/Yekaterinburg",
	},
	"UWOR": {
		ICAO:      "UWOR",
		IATA:      "OSW",
		Name:      "Orsk Airport",
		City:      "Orsk",
		State:     "Orenburg",
		Country:   "RU",
		Elevation: 909,
		Latitude:  51.0724983215,
		Longitude: 58.5956001282,
		Timezone:  "Asia/Yekaterinburg",
	},
	"UWOX": {
		ICAO:      "UWOX",
		Name:      "Buzuluk Airport",
		City:      "Buzuluk",
		State:     "Orenburg",
		Country:   "RU",
		Latitude:  52.8129997253,
		Longitude: 52.2220001221,
		Timezone:  "Asia/Yekaterinburg",
	},
	"UWPP": {
		ICAO:      "UWPP",
		IATA:      "PEZ",
		Name:      "Penza Airport",
		City:      "Penza",
		State:     "Penza",
		Country:   "RU",
		Elevation: 614,
		Latitude:  53.1105995178,
		Longitude: 45.0210990906,
		Timezone:  "Europe/Moscow",
	},
	"UWPS": {
		ICAO:      "UWPS",
		IATA:      "SKX",
		Name:      "Saransk Airport",
		City:      "Saransk",
		State:     "Mordoviya",
		Country:   "RU",
		Elevation: 676,
		Latitude:  54.1251296997,
		Longitude: 45.2122573853,
		Timezone:  "Europe/Moscow",
	},
	"UWSB": {
		ICAO:      "UWSB",
		IATA:      "BWO",
		Name:      "Balakovo Airport",
		City:      "Balakovo",
		State:     "Saratov",
		Country:   "RU",
		Elevation: 95,
		Latitude:  51.8582992554,
		Longitude: 47.7456016541,
		Timezone:  "Europe/Saratov",
	},
	"UWSG": {
		ICAO:      "UWSG",
		IATA:      "GSV",
		Name:      "Saratov Gagarin Airport",
		City:      "Saratov",
		State:     "Saratov",
		Country:   "RU",
		Elevation: 103,
		Latitude:  51.712778,
		Longitude: 46.171111,
		Timezone:  "Europe/Saratov",
	},
	"UWSK": {
		ICAO:      "UWSK",
		Name:      "Krasny Kut Airport",
		City:      "Krasny Kut",
		State:     "Saratov",
		Country:   "RU",
		Elevation: 177,
		Latitude:  50.9580001831,
		Longitude: 46.9469985962,
		Timezone:  "Europe/Saratov",
	},
	"UWSS": {
		ICAO:      "UWSS",
		IATA:      "RTW",
		Name:      "Saratov Central Airport",
		City:      "Saratov",
		State:     "Saratov",
		Country:   "RU",
		Elevation: 499,
		Latitude:  51.5649986267,
		Longitude: 46.0466995239,
		Timezone:  "Europe/Saratov",
	},
	"UWUA": {
		ICAO:      "UWUA",
		Name:      "Sibay Airport",
		City:      "Sibay",
		State:     "Bashkortostan",
		Country:   "RU",
		Elevation: 1309,
		Latitude:  52.687412262,
		Longitude: 58.7148017883,
		Timezone:  "Asia/Yekaterinburg",
	},
	"UWUB": {
		ICAO:      "UWUB",
		IATA:      "BCX",
		Name:      "Beloretsk Airport",
		City:      "Beloretsk",
		State:     "Bashkortostan",
		Country:   "RU",
		Elevation: 1827,
		Latitude:  53.9380989075,
		Longitude: 58.3400001526,
		Timezone:  "Asia/Yekaterinburg",
	},
	"UWUF": {
		ICAO:      "UWUF",
		IATA:      "NEF",
		Name:      "Neftekamsk Airport",
		City:      "Neftekamsk",
		State:     "Bashkortostan",
		Country:   "RU",
		Elevation: 456,
		Latitude:  56.1068992615,
		Longitude: 54.3471984863,
		Timezone:  "Asia/Yekaterinburg",
	},
	"UWUK": {
		ICAO:      "UWUK",
		IATA:      "OKT",
		Name:      "Oktyabrskiy Airport",
		City:      "Kzyl-Yar",
		State:     "Bashkortostan",
		Country:   "RU",
		Elevation: 377,
		Latitude:  54.4399986267,
		Longitude: 53.3883018494,
		Timezone:  "Asia/Yekaterinburg",
	},
	"UWUM": {
		ICAO:      "UWUM",
		Name:      "Maksimovka Airport",
		City:      "Ufa",
		State:     "Bashkortostan",
		Country:   "RU",
		Elevation: 325,
		Latitude:  54.8300018311,
		Longitude: 56.1683006287,
		Timezone:  "Asia/Yekaterinburg",
	},
	"UWUU": {
		ICAO:      "UWUU",
		IATA:      "UFA",
		Name:      "Ufa International Airport",
		City:      "Ufa",
		State:     "Bashkortostan",
		Country:   "RU",
		Elevation: 449,
		Latitude:  54.5574989319,
		Longitude: 55.8744010925,
		Timezone:  "Asia/Yekaterinburg",
	},
	"UWWB": {
		ICAO:      "UWWB",
		Name:      "Buguruslan Severny Airport",
		City:      "Buguruslan",
		State:     "Orenburg",
		Country:   "RU",
		Elevation: 728,
		Latitude:  53.7183990479,
		Longitude: 52.3718986511,
		Timezone:  "Asia/Yekaterinburg",
	},
	"UWWE": {
		ICAO:      "UWWE",
		Name:      "Verkhneye Sancheleyevo Airfield",
		City:      "Tolyatti",
		State:     "Samara",
		Country:   "RU",
		Elevation: 246,
		Latitude:  53.6960966684,
		Longitude: 49.491519928,
		Timezone:  "Europe/Samara",
	},
	"UWWG": {
		ICAO:      "UWWG",
		Name:      "Bezymyanka Airfield",
		City:      "Samara",
		State:     "Samara",
		Country:   "RU",
		Elevation: 135,
		Latitude:  53.2200012207,
		Longitude: 50.3250007629,
		Timezone:  "Europe/Samara",
	},
	"UWWS": {
		ICAO:      "UWWS",
		Name:      "Smyshlyayevka Airport",
		City:      "Samara",
		State:     "Samara",
		Country:   "RU",
		Elevation: 131,
		Latitude:  53.2400016785,
		Longitude: 50.375,
		Timezone:  "Europe/Samara",
	},
	"UWWT": {
		ICAO:      "UWWT",
		Name:      "Tolyatti Airport",
		City:      "Tolyatti",
		State:     "Samara",
		Country:   "RU",
		Elevation: 230,
		Latitude:  53.5962743924,
		Longitude: 49.3781161308,
		Timezone:  "Europe/Samara",
	},
	"UWWW": {
		ICAO:      "UWWW",
		IATA:      "KUF",
		Name:      "Kurumoch International Airport",
		City:      "Samara",
		State:     "Samara",
		Country:   "RU",
		Elevation: 477,
		Latitude:  53.504901886,
		Longitude: 50.1642990112,
		Timezone:  "Europe/Samara",
	},
	"UWWZ": {
		ICAO:      "UWWZ",
		Name:      "Buguruslan Glavny Airfield",
		City:      "Buguruslan",
		State:     "Orenburg",
		Country:   "RU",
		Latitude:  53.6180000305,
		Longitude: 52.4440002441,
		Timezone:  "Asia/Yekaterinburg",
	},
	"VA00": {
		ICAO:      "VA00",
		Name:      "Brook Hill Farm Airport",
		City:      "Steeles Tavern",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1600,
		Latitude:  37.9281997681,
		Longitude: -79.1781005859,
		Timezone:  "America/New_York",
	},
	"VA02": {
		ICAO:      "VA02",
		Name:      "Pace Airport",
		City:      "Ridgeway",
		State:     "Virginia",
		Country:   "US",
		Elevation: 800,
		Latitude:  36.5853996277,
		Longitude: -79.8938980103,
		Timezone:  "America/New_York",
	},
	"VA04": {
		ICAO:      "VA04",
		Name:      "Barrows Airport",
		City:      "Fincastle",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1100,
		Latitude:  37.492099762,
		Longitude: -79.9287033081,
		Timezone:  "America/New_York",
	},
	"VA06": {
		ICAO:      "VA06",
		Name:      "Deer Run Airport",
		City:      "Floyd",
		State:     "Virginia",
		Country:   "US",
		Elevation: 2400,
		Latitude:  36.9943008423,
		Longitude: -80.454498291,
		Timezone:  "America/New_York",
	},
	"VA07": {
		ICAO:      "VA07",
		Name:      "Burkes Garden Airport",
		City:      "Tazewell",
		State:     "Virginia",
		Country:   "US",
		Elevation: 3060,
		Latitude:  37.0956993103,
		Longitude: -81.3693008423,
		Timezone:  "America/New_York",
	},
	"VA08": {
		ICAO:      "VA08",
		Name:      "Longbranch Airport",
		City:      "Kilmarnock",
		State:     "Virginia",
		Country:   "US",
		Elevation: 20,
		Latitude:  37.7170982361,
		Longitude: -76.3588027954,
		Timezone:  "America/New_York",
	},
	"VA10": {
		ICAO:      "VA10",
		Name:      "Catawba Valley Airport",
		City:      "Blacksburg",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1720,
		Latitude:  37.2801017761,
		Longitude: -80.301399231,
		Timezone:  "America/New_York",
	},
	"VA11": {
		ICAO:      "VA11",
		Name:      "White Oak Stand Airport",
		City:      "Chilhowie",
		State:     "Virginia",
		Country:   "US",
		Elevation: 2100,
		Latitude:  36.8078994751,
		Longitude: -81.7012023926,
		Timezone:  "America/New_York",
	},
	"VA12": {
		ICAO:      "VA12",
		Name:      "Gravely Airport",
		City:      "Martinsville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 750,
		Latitude:  36.6468009949,
		Longitude: -79.7969970703,
		Timezone:  "America/New_York",
	},
	"VA13": {
		ICAO:      "VA13",
		Name:      "Breezy Knoll Airport",
		City:      "Rustburg",
		State:     "Virginia",
		Country:   "US",
		Elevation: 785,
		Latitude:  37.2653999329,
		Longitude: -79.0428009033,
		Timezone:  "America/New_York",
	},
	"VA14": {
		ICAO:      "VA14",
		Name:      "Southampton Correctional Center Airport",
		City:      "Capron",
		State:     "Virginia",
		Country:   "US",
		Elevation: 50,
		Latitude:  36.7406997681,
		Longitude: -77.2627029419,
		Timezone:  "America/New_York",
	},
	"VA15": {
		ICAO:      "VA15",
		Name:      "Stokes Airport",
		City:      "Cartersville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 295,
		Latitude:  37.6562995911,
		Longitude: -78.0449981689,
		Timezone:  "America/New_York",
	},
	"VA16": {
		ICAO:      "VA16",
		Name:      "Centreville Airport",
		City:      "Centreville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 300,
		Latitude:  38.8792991638,
		Longitude: -77.4843978882,
		Timezone:  "America/New_York",
	},
	"VA17": {
		ICAO:      "VA17",
		Name:      "Mulberry Run Airport",
		City:      "Strasburg",
		State:     "Virginia",
		Country:   "US",
		Elevation: 820,
		Latitude:  39.035900116,
		Longitude: -78.3921966553,
		Timezone:  "America/New_York",
	},
	"VA18": {
		ICAO:      "VA18",
		Name:      "Bundoran Airport",
		City:      "Charlottesville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 840,
		Latitude:  37.96849823,
		Longitude: -78.6821975708,
		Timezone:  "America/New_York",
	},
	"VA19": {
		ICAO:      "VA19",
		Name:      "Snow Hill Airport",
		City:      "Charlottesville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 268,
		Latitude:  37.9539985657,
		Longitude: -78.3692016602,
		Timezone:  "America/New_York",
	},
	"VA1B": {
		ICAO:      "VA1B",
		Name:      "Chanda Airport",
		State:     "Maharashtra",
		Country:   "IN",
		Elevation: 625,
		Latitude:  19.9946994781,
		Longitude: 79.2225036621,
		Timezone:  "Asia/Kolkata",
	},
	"VA1C": {
		ICAO:      "VA1C",
		Name:      "Birlagram Airport",
		State:     "Madhya-Pradesh",
		Country:   "IN",
		Elevation: 1541,
		Latitude:  23.4468994141,
		Longitude: 75.4203033447,
		Timezone:  "Asia/Kolkata",
	},
	"VA1D": {
		ICAO:      "VA1D",
		Name:      "Muirpur Airport",
		State:     "Uttar-Pradesh",
		Country:   "IN",
		Elevation: 1122,
		Latitude:  24.1252002716,
		Longitude: 83.0406036377,
		Timezone:  "Asia/Kolkata",
	},
	"VA1E": {
		ICAO:      "VA1E",
		Name:      "Bhilai Airport",
		State:     "Chhattisgarh",
		Country:   "IN",
		Elevation: 1014,
		Latitude:  21.2942008972,
		Longitude: 81.3795013428,
		Timezone:  "Asia/Kolkata",
	},
	"VA1F": {
		ICAO:      "VA1F",
		Name:      "Sidhi Airport",
		State:     "Madhya-Pradesh",
		Country:   "IN",
		Elevation: 1093,
		Latitude:  24.402299881,
		Longitude: 81.8147964478,
		Timezone:  "Asia/Kolkata",
	},
	"VA1G": {
		ICAO:      "VA1G",
		Name:      "Chorhata Airport",
		State:     "Madhya-Pradesh",
		Country:   "IN",
		Elevation: 1000,
		Latitude:  24.5034008026,
		Longitude: 81.2202987671,
		Timezone:  "Asia/Kolkata",
	},
	"VA1H": {
		ICAO:      "VA1H",
		Name:      "Ondwa Airport",
		State:     "Rajasthan",
		Country:   "IN",
		Elevation: 1450,
		Latitude:  25.141500473,
		Longitude: 74.6115036011,
		Timezone:  "Asia/Kolkata",
	},
	"VA1J": {
		ICAO:      "VA1J",
		Name:      "Dhana Airport",
		State:     "Madhya-Pradesh",
		Country:   "IN",
		Elevation: 1706,
		Latitude:  23.7535991669,
		Longitude: 78.855796814,
		Timezone:  "Asia/Kolkata",
	},
	"VA1K": {
		ICAO:      "VA1K",
		Name:      "Raedhanpur Airport",
		State:     "Gujarat",
		Country:   "IN",
		Elevation: 135,
		Latitude:  23.2222995758,
		Longitude: 68.8912963867,
		Timezone:  "Asia/Kolkata",
	},
	"VA1L": {
		ICAO:      "VA1L",
		Name:      "Amla Airport",
		State:     "Madhya-Pradesh",
		Country:   "IN",
		Elevation: 2435,
		Latitude:  21.926399231,
		Longitude: 78.1136016846,
		Timezone:  "Asia/Kolkata",
	},
	"VA1M": {
		ICAO:      "VA1M",
		Name:      "Karad Airport",
		Country:   "IN",
		Elevation: 1890,
		Latitude:  17.285900116,
		Longitude: 74.1580963135,
		Timezone:  "Asia/Kolkata",
	},
	"VA1N": {
		ICAO:      "VA1N",
		Name:      "Nimach Airport",
		State:     "Rajasthan",
		Country:   "IN",
		Elevation: 1617,
		Latitude:  24.4307994843,
		Longitude: 74.8677978516,
		Timezone:  "Asia/Kolkata",
	},
	"VA1O": {
		ICAO:      "VA1O",
		Name:      "Burhar Airport",
		State:     "Madhya-Pradesh",
		Country:   "IN",
		Elevation: 1500,
		Latitude:  23.235200882,
		Longitude: 81.5037002563,
		Timezone:  "Asia/Kolkata",
	},
	"VA1P": {
		ICAO:      "VA1P",
		IATA:      "DIU",
		Name:      "Diu Airport",
		City:      "Diu",
		State:     "Gujarat",
		Country:   "IN",
		Elevation: 31,
		Latitude:  20.7131004333,
		Longitude: 70.9210968018,
		Timezone:  "Asia/Kolkata",
	},
	"VA22": {
		ICAO:      "VA22",
		Name:      "Big River Ranch Airport",
		City:      "Clarksville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 375,
		Latitude:  36.6750984192,
		Longitude: -78.6949996948,
		Timezone:  "America/New_York",
	},
	"VA23": {
		ICAO:      "VA23",
		Name:      "Sanford Field",
		City:      "Coles Point",
		State:     "Virginia",
		Country:   "US",
		Elevation: 15,
		Latitude:  38.1189994812,
		Longitude: -76.6136016846,
		Timezone:  "America/New_York",
	},
	"VA24": {
		ICAO:      "VA24",
		Name:      "Skovhus Airport",
		City:      "Concord",
		State:     "Virginia",
		Country:   "US",
		Elevation: 712,
		Latitude:  37.4142990112,
		Longitude: -78.9642028809,
		Timezone:  "America/New_York",
	},
	"VA25": {
		ICAO:      "VA25",
		Name:      "Twin Towers Airport",
		City:      "Chase City",
		State:     "Virginia",
		Country:   "US",
		Elevation: 470,
		Latitude:  36.7643013,
		Longitude: -78.360496521,
		Timezone:  "America/New_York",
	},
	"VA26": {
		ICAO:      "VA26",
		Name:      "Trent Farm Airstrip",
		City:      "Critz",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1130,
		Latitude:  36.650100708,
		Longitude: -80.1164016724,
		Timezone:  "America/New_York",
	},
	"VA27": {
		ICAO:      "VA27",
		Name:      "Moorefield's Airstrip",
		City:      "Critz",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1110,
		Latitude:  36.5598983765,
		Longitude: -80.137802124,
		Timezone:  "America/New_York",
	},
	"VA28": {
		ICAO:      "VA28",
		Name:      "Layne Farm Airstrip",
		City:      "Cullen",
		State:     "Virginia",
		Country:   "US",
		Elevation: 560,
		Latitude:  37.1212997437,
		Longitude: -78.6436004639,
		Timezone:  "America/New_York",
	},
	"VA2A": {
		ICAO:      "VA2A",
		Name:      "Phalodi Airport",
		State:     "Rajasthan",
		Country:   "IN",
		Elevation: 781,
		Latitude:  27.1128997803,
		Longitude: 72.388999939,
		Timezone:  "Asia/Kolkata",
	},
	"VA2B": {
		ICAO:      "VA2B",
		Name:      "Meerut Sw Airport",
		State:     "Uttar-Pradesh",
		Country:   "IN",
		Elevation: 732,
		Latitude:  28.9048995972,
		Longitude: 77.6772994995,
		Timezone:  "Asia/Kolkata",
	},
	"VA2C": {
		ICAO:      "VA2C",
		Name:      "Gondia Airport",
		State:     "Maharashtra",
		Country:   "IN",
		Elevation: 990,
		Latitude:  21.5261993408,
		Longitude: 80.290397644,
		Timezone:  "Asia/Kolkata",
	},
	"VA2D": {
		ICAO:      "VA2D",
		Name:      "Ratlam Airport",
		State:     "Madhya-Pradesh",
		Country:   "IN",
		Elevation: 1702,
		Latitude:  23.3813991547,
		Longitude: 75.0255966187,
		Timezone:  "Asia/Kolkata",
	},
	"VA30": {
		ICAO:      "VA30",
		Name:      "Berryvale Airport",
		City:      "Culpeper",
		State:     "Virginia",
		Country:   "US",
		Elevation: 470,
		Latitude:  38.5267982483,
		Longitude: -77.9563980103,
		Timezone:  "America/New_York",
	},
	"VA31": {
		ICAO:      "VA31",
		Name:      "Apple Grove Airport",
		City:      "Kilmarnock",
		State:     "Virginia",
		Country:   "US",
		Elevation: 15,
		Latitude:  37.7625999451,
		Longitude: -76.3462982178,
		Timezone:  "America/New_York",
	},
	"VA32": {
		ICAO:      "VA32",
		Name:      "Longs Airport",
		City:      "Edinburg",
		State:     "Virginia",
		Country:   "US",
		Elevation: 870,
		Latitude:  38.8042984009,
		Longitude: -78.5719985962,
		Timezone:  "America/New_York",
	},
	"VA33": {
		ICAO:      "VA33",
		Name:      "Beaver Dam Airpark",
		City:      "Surry",
		State:     "Virginia",
		Country:   "US",
		Elevation: 100,
		Latitude:  37.0443000793,
		Longitude: -76.7682037354,
		Timezone:  "America/New_York",
	},
	"VA34": {
		ICAO:      "VA34",
		Name:      "Big Buffalo Airstrip",
		City:      "Farmville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 510,
		Latitude:  37.2835006714,
		Longitude: -78.4496994019,
		Timezone:  "America/New_York",
	},
	"VA36": {
		ICAO:      "VA36",
		Name:      "F. U. M. A. Airport",
		City:      "Fork Union",
		State:     "Virginia",
		Country:   "US",
		Elevation: 400,
		Latitude:  37.7445983887,
		Longitude: -78.2703018188,
		Timezone:  "America/New_York",
	},
	"VA38": {
		ICAO:      "VA38",
		Name:      "Sirohi Airport",
		State:     "Rajasthan",
		Country:   "IN",
		Elevation: 1000,
		Latitude:  24.8950996399,
		Longitude: 72.8459014893,
		Timezone:  "Asia/Kolkata",
	},
	"VA41": {
		ICAO:      "VA41",
		Name:      "High Hopes Airport",
		City:      "Withams",
		State:     "Virginia",
		Country:   "US",
		Elevation: 9,
		Latitude:  37.9346008301,
		Longitude: -75.6099014282,
		Timezone:  "America/New_York",
	},
	"VA42": {
		ICAO:      "VA42",
		Name:      "Dogwood Airpark",
		City:      "Fredericksburg",
		State:     "Virginia",
		Country:   "US",
		Elevation: 180,
		Latitude:  38.3585014343,
		Longitude: -77.4522018433,
		Timezone:  "America/New_York",
	},
	"VA43": {
		ICAO:      "VA43",
		Name:      "Balcony Downs Airstrip",
		City:      "Glasgow",
		State:     "Virginia",
		Country:   "US",
		Elevation: 820,
		Latitude:  37.6473999023,
		Longitude: -79.4247970581,
		Timezone:  "America/New_York",
	},
	"VA44": {
		ICAO:      "VA44",
		Name:      "Rose Retreat Farm Airport",
		City:      "Goochland",
		State:     "Virginia",
		Country:   "US",
		Elevation: 250,
		Latitude:  37.7168006897,
		Longitude: -77.8839035034,
		Timezone:  "America/New_York",
	},
	"VA45": {
		ICAO:      "VA45",
		Name:      "Lous Airport",
		City:      "Gordonsville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 480,
		Latitude:  38.1278991699,
		Longitude: -78.1635971069,
		Timezone:  "America/New_York",
	},
	"VA46": {
		ICAO:      "VA46",
		Name:      "Timber Ridge Airpark",
		City:      "Gore",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1024,
		Latitude:  39.2961997986,
		Longitude: -78.362197876,
		Timezone:  "America/New_York",
	},
	"VA47": {
		ICAO:      "VA47",
		Name:      "Jalgaon Airport",
		State:     "Maharashtra",
		Country:   "IN",
		Elevation: 800,
		Latitude:  20.9622001648,
		Longitude: 75.626701355,
		Timezone:  "Asia/Kolkata",
	},
	"VA49": {
		ICAO:      "VA49",
		Name:      "Robertson Airport",
		City:      "Hanover",
		State:     "Virginia",
		Country:   "US",
		Elevation: 193,
		Latitude:  37.7053985596,
		Longitude: -77.3724975586,
		Timezone:  "America/New_York",
	},
	"VA50": {
		ICAO:      "VA50",
		Name:      "Woods Farm Airstrip",
		City:      "Hanover",
		State:     "Virginia",
		Country:   "US",
		Elevation: 57,
		Latitude:  37.7150993347,
		Longitude: -77.3024978638,
		Timezone:  "America/New_York",
	},
	"VA51": {
		ICAO:      "VA51",
		Name:      "Banswara Airport",
		State:     "Rajasthan",
		Country:   "IN",
		Elevation: 700,
		Latitude:  23.5897006989,
		Longitude: 74.3126983643,
		Timezone:  "Asia/Kolkata",
	},
	"VA52": {
		ICAO:      "VA52",
		Name:      "Frank Field",
		City:      "Harrisonburg",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1650,
		Latitude:  38.4850997925,
		Longitude: -78.9452972412,
		Timezone:  "America/New_York",
	},
	"VA53": {
		ICAO:      "VA53",
		Name:      "Dhulia Airport",
		State:     "Maharashtra",
		Country:   "IN",
		Elevation: 920,
		Latitude:  20.9265003204,
		Longitude: 74.7370986938,
		Timezone:  "Asia/Kolkata",
	},
	"VA56": {
		ICAO:      "VA56",
		Name:      "Wells Airport",
		City:      "Ivor",
		State:     "Virginia",
		Country:   "US",
		Elevation: 88,
		Latitude:  37.0035018921,
		Longitude: -76.8274993896,
		Timezone:  "America/New_York",
	},
	"VA57": {
		ICAO:      "VA57",
		Name:      "Powhatan Airport",
		City:      "King George",
		State:     "Virginia",
		Country:   "US",
		Elevation: 60,
		Latitude:  38.248500824,
		Longitude: -77.2191009521,
		Timezone:  "America/New_York",
	},
	"VA58": {
		ICAO:      "VA58",
		Name:      "River Bend Airport",
		City:      "Maurertown",
		State:     "Virginia",
		Country:   "US",
		Elevation: 606,
		Latitude:  38.9061012268,
		Longitude: -78.4443969727,
		Timezone:  "America/New_York",
	},
	"VA59": {
		ICAO:      "VA59",
		Name:      "Weatherly & Son Airport",
		City:      "Chesapeake",
		State:     "Virginia",
		Country:   "US",
		Elevation: 10,
		Latitude:  36.6357002258,
		Longitude: -76.1516036987,
		Timezone:  "America/New_York",
	},
	"VA61": {
		ICAO:      "VA61",
		Name:      "Scott Airpark",
		City:      "Lovettsville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 529,
		Latitude:  39.263999939,
		Longitude: -77.6572036743,
		Timezone:  "America/New_York",
	},
	"VA62": {
		ICAO:      "VA62",
		Name:      "The Grass Patch Airport",
		City:      "Lovettsville",
		Country:   "US",
		Elevation: 530,
		Latitude:  39.2593002319,
		Longitude: -77.6572036743,
		Timezone:  "America/New_York",
	},
	"VA63": {
		ICAO:      "VA63",
		Name:      "Twin River Airport",
		City:      "Glasgow",
		State:     "Virginia",
		Country:   "US",
		Elevation: 720,
		Latitude:  37.679599762,
		Longitude: -79.4203033447,
		Timezone:  "America/New_York",
	},
	"VA64": {
		ICAO:      "VA64",
		Name:      "Hill Top Airport",
		City:      "Amelia",
		State:     "Virginia",
		Country:   "US",
		Elevation: 254,
		Latitude:  37.4160003662,
		Longitude: -77.9539031982,
		Timezone:  "America/New_York",
	},
	"VA65": {
		ICAO:      "VA65",
		Name:      "Ivy Hill Airport",
		City:      "Lynchburg",
		State:     "Virginia",
		Country:   "US",
		Elevation: 920,
		Latitude:  37.3945999146,
		Longitude: -79.324798584,
		Timezone:  "America/New_York",
	},
	"VA66": {
		ICAO:      "VA66",
		Name:      "Breeden Airport",
		City:      "Catlett",
		State:     "Virginia",
		Country:   "US",
		Elevation: 210,
		Latitude:  38.6076011658,
		Longitude: -77.5693969727,
		Timezone:  "America/New_York",
	},
	"VA67": {
		ICAO:      "VA67",
		Name:      "Homeland Airport",
		City:      "Culpeper",
		State:     "Virginia",
		Country:   "US",
		Elevation: 404,
		Latitude:  38.570098877,
		Longitude: -77.9349975586,
		Timezone:  "America/New_York",
	},
	"VA68": {
		ICAO:      "VA68",
		Name:      "Lakeview Aerodrome",
		City:      "Moneta",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1020,
		Latitude:  37.1188011169,
		Longitude: -79.6027984619,
		Timezone:  "America/New_York",
	},
	"VA71": {
		ICAO:      "VA71",
		Name:      "Aden Field",
		City:      "Manassas",
		State:     "Virginia",
		Country:   "US",
		Elevation: 210,
		Latitude:  38.6484985352,
		Longitude: -77.5327987671,
		Timezone:  "America/New_York",
	},
	"VA72": {
		ICAO:      "VA72",
		Name:      "Covington Airport",
		City:      "Martinsville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 940,
		Latitude:  36.6001014709,
		Longitude: -80.0386962891,
		Timezone:  "America/New_York",
	},
	"VA73": {
		ICAO:      "VA73",
		Name:      "Mazza Airport",
		City:      "Matoaca",
		State:     "Virginia",
		Country:   "US",
		Elevation: 190,
		Latitude:  37.2445983887,
		Longitude: -77.4942016602,
		Timezone:  "America/New_York",
	},
	"VA74": {
		ICAO:      "VA74",
		Name:      "Abu Road Airport",
		State:     "Rajasthan",
		Country:   "IN",
		Elevation: 875,
		Latitude:  24.4941997528,
		Longitude: 72.78150177,
		Timezone:  "Asia/Kolkata",
	},
	"VA75": {
		ICAO:      "VA75",
		Name:      "Smith Field",
		City:      "Mc Dowell",
		State:     "Virginia",
		Country:   "US",
		Elevation: 2020,
		Latitude:  38.3026008606,
		Longitude: -79.508102417,
		Timezone:  "America/New_York",
	},
	"VA76": {
		ICAO:      "VA76",
		Name:      "Baskerville Airport",
		City:      "Mc Kenney",
		State:     "Virginia",
		Country:   "US",
		Elevation: 250,
		Latitude:  36.9875984192,
		Longitude: -77.771697998,
		Timezone:  "America/New_York",
	},
	"VA77": {
		ICAO:      "VA77",
		Name:      "Hanover Air Park",
		City:      "Mechanicsville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 196,
		Latitude:  37.6940002441,
		Longitude: -77.3768997192,
		Timezone:  "America/New_York",
	},
	"VA79": {
		ICAO:      "VA79",
		Name:      "Hickory Tree Farm Airport",
		City:      "Middleburg",
		State:     "Virginia",
		Country:   "US",
		Elevation: 544,
		Latitude:  38.9528999329,
		Longitude: -77.7469024658,
		Timezone:  "America/New_York",
	},
	"VA80": {
		ICAO:      "VA80",
		Name:      "Woodle Airport",
		City:      "Midlothian",
		State:     "Virginia",
		Country:   "US",
		Elevation: 315,
		Latitude:  37.4788017273,
		Longitude: -77.7018966675,
		Timezone:  "America/New_York",
	},
	"VA81": {
		ICAO:      "VA81",
		Name:      "Cub Field",
		City:      "Mineral",
		State:     "Virginia",
		Country:   "US",
		Elevation: 396,
		Latitude:  37.945400238,
		Longitude: -77.8993988037,
		Timezone:  "America/New_York",
	},
	"VA83": {
		ICAO:      "VA83",
		Name:      "Fincastle Airport",
		City:      "Fincastle",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1140,
		Latitude:  37.5223999023,
		Longitude: -79.8519973755,
		Timezone:  "America/New_York",
	},
	"VA85": {
		ICAO:      "VA85",
		Name:      "New Castle International Airport",
		City:      "New Castle",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1320,
		Latitude:  37.4835014343,
		Longitude: -80.0998001099,
		Timezone:  "America/New_York",
	},
	"VA86": {
		ICAO:      "VA86",
		Name:      "Buddy Davis Field",
		City:      "New Market",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1042,
		Latitude:  38.6893005371,
		Longitude: -78.7063980103,
		Timezone:  "America/New_York",
	},
	"VA87": {
		ICAO:      "VA87",
		Name:      "Valley View Airport",
		City:      "Nokesville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 240,
		Latitude:  38.6917991638,
		Longitude: -77.5511016846,
		Timezone:  "America/New_York",
	},
	"VA89": {
		ICAO:      "VA89",
		Name:      "Chance Airport",
		City:      "Davis Wharf",
		State:     "Virginia",
		Country:   "US",
		Elevation: 6,
		Latitude:  37.5646018982,
		Longitude: -75.9263000488,
		Timezone:  "America/New_York",
	},
	"VA90": {
		ICAO:      "VA90",
		Name:      "Hudgins Farm Ultralightport",
		City:      "Cobbs Creek",
		State:     "Virginia",
		Country:   "US",
		Elevation: 43,
		Latitude:  37.496799469,
		Longitude: -76.4135971069,
		Timezone:  "America/New_York",
	},
	"VA91": {
		ICAO:      "VA91",
		Name:      "Johnson Field",
		City:      "Onancock",
		State:     "Virginia",
		Country:   "US",
		Elevation: 38,
		Latitude:  37.7048988342,
		Longitude: -75.726600647,
		Timezone:  "America/New_York",
	},
	"VA92": {
		ICAO:      "VA92",
		Name:      "Peace and Plenty Farm Airport",
		City:      "Onancock",
		State:     "Virginia",
		Country:   "US",
		Elevation: 8,
		Latitude:  37.7224006653,
		Longitude: -75.7857971191,
		Timezone:  "America/New_York",
	},
	"VA93": {
		ICAO:      "VA93",
		Name:      "Ayers Airport",
		City:      "Columbia Furnace",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1340,
		Latitude:  38.9051017761,
		Longitude: -78.6547012329,
		Timezone:  "America/New_York",
	},
	"VA94": {
		ICAO:      "VA94",
		Name:      "Plainview Airport",
		City:      "Powhatan",
		State:     "Virginia",
		Country:   "US",
		Elevation: 400,
		Latitude:  37.5357017517,
		Longitude: -77.8914031982,
		Timezone:  "America/New_York",
	},
	"VA95": {
		ICAO:      "VA95",
		Name:      "Spring Valley Airport",
		City:      "Powhatan",
		State:     "Virginia",
		Country:   "US",
		Elevation: 275,
		Latitude:  37.5970993042,
		Longitude: -77.8003005981,
		Timezone:  "America/New_York",
	},
	"VA96": {
		ICAO:      "VA96",
		Name:      "Faber Airport",
		City:      "Lovingston",
		State:     "Virginia",
		Country:   "US",
		Elevation: 640,
		Latitude:  37.8254013062,
		Longitude: -78.7316970825,
		Timezone:  "America/New_York",
	},
	"VA97": {
		ICAO:      "VA97",
		Name:      "Harris Airport",
		City:      "Rectortown",
		State:     "Virginia",
		Country:   "US",
		Elevation: 566,
		Latitude:  38.9126014709,
		Longitude: -77.8816986084,
		Timezone:  "America/New_York",
	},
	"VA98": {
		ICAO:      "VA98",
		Name:      "Reedville Airport",
		City:      "Reedville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 12,
		Latitude:  37.8282012939,
		Longitude: -76.2733001709,
		Timezone:  "America/New_York",
	},
	"VA99": {
		ICAO:      "VA99",
		Name:      "Federhart-Ophelia STOLport",
		City:      "Reedville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 13,
		Latitude:  37.90599823,
		Longitude: -76.2904968262,
		Timezone:  "America/New_York",
	},
	"VAAH": {
		ICAO:      "VAAH",
		IATA:      "AMD",
		Name:      "Sardar Vallabhbhai Patel International Airport",
		City:      "Ahmedabad",
		State:     "Gujarat",
		Country:   "IN",
		Elevation: 189,
		Latitude:  23.0771999359,
		Longitude: 72.6346969604,
		Timezone:  "Asia/Kolkata",
	},
	"VAAK": {
		ICAO:      "VAAK",
		IATA:      "AKD",
		Name:      "Akola Airport",
		State:     "Maharashtra",
		Country:   "IN",
		Elevation: 999,
		Latitude:  20.6989994049,
		Longitude: 77.0586013794,
		Timezone:  "Asia/Kolkata",
	},
	"VAAU": {
		ICAO:      "VAAU",
		IATA:      "IXU",
		Name:      "Aurangabad Airport",
		City:      "Aurangabad",
		State:     "Maharashtra",
		Country:   "IN",
		Elevation: 1911,
		Latitude:  19.8626995087,
		Longitude: 75.3981018066,
		Timezone:  "Asia/Kolkata",
	},
	"VABB": {
		ICAO:      "VABB",
		IATA:      "BOM",
		Name:      "Chhatrapati Shivaji International Airport",
		City:      "Mumbai",
		State:     "Maharashtra",
		Country:   "IN",
		Elevation: 37,
		Latitude:  19.0886993408,
		Longitude: 72.8678970337,
		Timezone:  "Asia/Kolkata",
	},
	"VABI": {
		ICAO:      "VABI",
		IATA:      "PAB",
		Name:      "Bilaspur Airport",
		State:     "Chhattisgarh",
		Country:   "IN",
		Elevation: 899,
		Latitude:  21.9883995056,
		Longitude: 82.111000061,
		Timezone:  "Asia/Kolkata",
	},
	"VABJ": {
		ICAO:      "VABJ",
		IATA:      "BHJ",
		Name:      "Bhuj Airport",
		City:      "Bhuj",
		State:     "Gujarat",
		Country:   "IN",
		Elevation: 268,
		Latitude:  23.2877998352,
		Longitude: 69.6701965332,
		Timezone:  "Asia/Kolkata",
	},
	"VABM": {
		ICAO:      "VABM",
		IATA:      "IXG",
		Name:      "Belgaum Airport",
		State:     "Karnataka",
		Country:   "IN",
		Elevation: 2487,
		Latitude:  15.8592996597,
		Longitude: 74.6183013916,
		Timezone:  "Asia/Kolkata",
	},
	"VABO": {
		ICAO:      "VABO",
		IATA:      "BDQ",
		Name:      "Vadodara Airport",
		City:      "Vadodara",
		State:     "Gujarat",
		Country:   "IN",
		Elevation: 129,
		Latitude:  22.3362007141,
		Longitude: 73.2263031006,
		Timezone:  "Asia/Kolkata",
	},
	"VABP": {
		ICAO:      "VABP",
		IATA:      "BHO",
		Name:      "Bhopal Airport",
		City:      "Bhopal",
		State:     "Madhya-Pradesh",
		Country:   "IN",
		Elevation: 1719,
		Latitude:  23.2875003815,
		Longitude: 77.3374023438,
		Timezone:  "Asia/Kolkata",
	},
	"VABV": {
		ICAO:      "VABV",
		IATA:      "BHU",
		Name:      "Bhavnagar Airport",
		City:      "Bhavnagar",
		State:     "Gujarat",
		Country:   "IN",
		Elevation: 44,
		Latitude:  21.752199173,
		Longitude: 72.1852035522,
		Timezone:  "Asia/Kolkata",
	},
	"VADN": {
		ICAO:      "VADN",
		IATA:      "NMB",
		Name:      "Daman Airport",
		State:     "Daman-and-Diu",
		Country:   "IN",
		Elevation: 33,
		Latitude:  20.4344005585,
		Longitude: 72.8432006836,
		Timezone:  "Asia/Kolkata",
	},
	"VADS": {
		ICAO:      "VADS",
		Name:      "Deesa Airport",
		State:     "Gujarat",
		Country:   "IN",
		Elevation: 485,
		Latitude:  24.2679004669,
		Longitude: 72.2043991089,
		Timezone:  "Asia/Kolkata",
	},
	"VAGN": {
		ICAO:      "VAGN",
		IATA:      "GUX",
		Name:      "Guna Airport",
		State:     "Madhya-Pradesh",
		Country:   "IN",
		Elevation: 1600,
		Latitude:  24.6546993256,
		Longitude: 77.3472976685,
		Timezone:  "Asia/Kolkata",
	},
	"VAHB": {
		ICAO:      "VAHB",
		IATA:      "HBX",
		Name:      "Hubli Airport",
		City:      "Hubli",
		State:     "Karnataka",
		Country:   "IN",
		Elevation: 2171,
		Latitude:  15.361700058,
		Longitude: 75.0848999023,
		Timezone:  "Asia/Kolkata",
	},
	"VAID": {
		ICAO:      "VAID",
		IATA:      "IDR",
		Name:      "Devi Ahilyabai Holkar Airport",
		City:      "Indore",
		State:     "Madhya-Pradesh",
		Country:   "IN",
		Elevation: 1850,
		Latitude:  22.7217998505,
		Longitude: 75.8011016846,
		Timezone:  "Asia/Kolkata",
	},
	"VAJB": {
		ICAO:      "VAJB",
		IATA:      "JLR",
		Name:      "Jabalpur Airport",
		State:     "Madhya-Pradesh",
		Country:   "IN",
		Elevation: 1624,
		Latitude:  23.1777992249,
		Longitude: 80.0520019531,
		Timezone:  "Asia/Kolkata",
	},
	"VAJJ": {
		ICAO:      "VAJJ",
		Name:      "Juhu Aerodrome",
		City:      "Mumbai",
		State:     "Maharashtra",
		Country:   "IN",
		Elevation: 13,
		Latitude:  19.0981006622,
		Longitude: 72.834197998,
		Timezone:  "Asia/Kolkata",
	},
	"VAJM": {
		ICAO:      "VAJM",
		IATA:      "JGA",
		Name:      "Jamnagar Airport",
		City:      "Jamnagar",
		State:     "Gujarat",
		Country:   "IN",
		Elevation: 69,
		Latitude:  22.4654998779,
		Longitude: 70.0126037598,
		Timezone:  "Asia/Kolkata",
	},
	"VAKE": {
		ICAO:      "VAKE",
		IATA:      "IXY",
		Name:      "Kandla Airport",
		City:      "Kandla",
		State:     "Gujarat",
		Country:   "IN",
		Elevation: 96,
		Latitude:  23.1126995087,
		Longitude: 70.1003036499,
		Timezone:  "Asia/Kolkata",
	},
	"VAKJ": {
		ICAO:      "VAKJ",
		IATA:      "HJR",
		Name:      "Khajuraho Airport",
		City:      "Khajuraho",
		State:     "Madhya-Pradesh",
		Country:   "IN",
		Elevation: 728,
		Latitude:  24.817199707,
		Longitude: 79.9186019897,
		Timezone:  "Asia/Kolkata",
	},
	"VAKP": {
		ICAO:      "VAKP",
		IATA:      "KLH",
		Name:      "Kolhapur Airport",
		State:     "Maharashtra",
		Country:   "IN",
		Elevation: 1996,
		Latitude:  16.6646995544,
		Longitude: 74.2893981934,
		Timezone:  "Asia/Kolkata",
	},
	"VAKS": {
		ICAO:      "VAKS",
		IATA:      "IXK",
		Name:      "Keshod Airport",
		State:     "Gujarat",
		Country:   "IN",
		Elevation: 167,
		Latitude:  21.3171005249,
		Longitude: 70.270401001,
		Timezone:  "Asia/Kolkata",
	},
	"VALT": {
		ICAO:      "VALT",
		IATA:      "LTU",
		Name:      "Murod Kond Airport",
		City:      "Latur",
		State:     "Maharashtra",
		Country:   "IN",
		Elevation: 2136,
		Latitude:  18.4115009308,
		Longitude: 76.4646987915,
		Timezone:  "Asia/Kolkata",
	},
	"VAND": {
		ICAO:      "VAND",
		IATA:      "NDC",
		Name:      "Nanded Airport",
		City:      "Nanded",
		State:     "Maharashtra",
		Country:   "IN",
		Elevation: 1250,
		Latitude:  19.1833000183,
		Longitude: 77.3167037964,
		Timezone:  "Asia/Kolkata",
	},
	"VANP": {
		ICAO:      "VANP",
		IATA:      "NAG",
		Name:      "Dr. Babasaheb Ambedkar International Airport",
		City:      "Naqpur",
		State:     "Maharashtra",
		Country:   "IN",
		Elevation: 1033,
		Latitude:  21.0921993256,
		Longitude: 79.047203064,
		Timezone:  "Asia/Kolkata",
	},
	"VANR": {
		ICAO:      "VANR",
		IATA:      "ISK",
		Name:      "Gandhinagar Airport",
		City:      "Nashik",
		State:     "Maharashtra",
		Country:   "IN",
		Elevation: 1959,
		Latitude:  19.9636993408,
		Longitude: 73.8076019287,
		Timezone:  "Asia/Kolkata",
	},
	"VAOZ": {
		ICAO:      "VAOZ",
		IATA:      "ISK",
		Name:      "Ozar Air Force Station",
		City:      "Nasik",
		State:     "Maharashtra",
		Country:   "IN",
		Elevation: 1900,
		Latitude:  20.1191005707,
		Longitude: 73.912902832,
		Timezone:  "Asia/Kolkata",
	},
	"VAPO": {
		ICAO:      "VAPO",
		IATA:      "PNQ",
		Name:      "Pune Airport",
		City:      "Pune",
		State:     "Maharashtra",
		Country:   "IN",
		Elevation: 1942,
		Latitude:  18.5820999146,
		Longitude: 73.9197006226,
		Timezone:  "Asia/Kolkata",
	},
	"VAPR": {
		ICAO:      "VAPR",
		IATA:      "PBD",
		Name:      "Porbandar Airport",
		City:      "Porbandar",
		State:     "Gujarat",
		Country:   "IN",
		Elevation: 23,
		Latitude:  21.6487007141,
		Longitude: 69.6572036743,
		Timezone:  "Asia/Kolkata",
	},
	"VARG": {
		ICAO:      "VARG",
		IATA:      "RTC",
		Name:      "Ratnagiri Airport",
		State:     "Maharashtra",
		Country:   "IN",
		Elevation: 305,
		Latitude:  17.0135993958,
		Longitude: 73.327796936,
		Timezone:  "Asia/Kolkata",
	},
	"VARK": {
		ICAO:      "VARK",
		IATA:      "RAJ",
		Name:      "Rajkot Airport",
		City:      "Rajkot",
		State:     "Gujarat",
		Country:   "IN",
		Elevation: 441,
		Latitude:  22.3092002869,
		Longitude: 70.7795028687,
		Timezone:  "Asia/Kolkata",
	},
	"VARP": {
		ICAO:      "VARP",
		IATA:      "RPR",
		Name:      "Raipur Airport",
		City:      "Raipur",
		State:     "Madhya-Pradesh",
		Country:   "IN",
		Elevation: 1041,
		Latitude:  21.1804008484,
		Longitude: 81.7388000488,
		Timezone:  "Asia/Kolkata",
	},
	"VASD": {
		ICAO:      "VASD",
		IATA:      "SAG",
		Name:      "Shirdi Airport",
		City:      "Kakadi",
		State:     "Maharashtra",
		Country:   "IN",
		Elevation: 1909,
		Latitude:  19.688611,
		Longitude: 74.378889,
		Timezone:  "Asia/Kolkata",
	},
	"VASL": {
		ICAO:      "VASL",
		IATA:      "SSE",
		Name:      "Solapur Airport",
		City:      "Solapur",
		State:     "Maharashtra",
		Country:   "IN",
		Elevation: 1584,
		Latitude:  17.6280002594,
		Longitude: 75.9347991943,
		Timezone:  "Asia/Kolkata",
	},
	"VASU": {
		ICAO:      "VASU",
		IATA:      "STV",
		Name:      "Surat Airport",
		State:     "Gujarat",
		Country:   "IN",
		Elevation: 16,
		Latitude:  21.1140995026,
		Longitude: 72.7417984009,
		Timezone:  "Asia/Kolkata",
	},
	"VAUD": {
		ICAO:      "VAUD",
		IATA:      "UDR",
		Name:      "Maharana Pratap Airport",
		City:      "Udaipur",
		State:     "Rajasthan",
		Country:   "IN",
		Elevation: 1684,
		Latitude:  24.6177005768,
		Longitude: 73.8961029053,
		Timezone:  "Asia/Kolkata",
	},
	"VCBI": {
		ICAO:      "VCBI",
		IATA:      "CMB",
		Name:      "Bandaranaike International Colombo Airport",
		City:      "Colombo",
		State:     "Western",
		Country:   "LK",
		Elevation: 30,
		Latitude:  7.1807599068,
		Longitude: 79.8841018677,
		Timezone:  "Asia/Colombo",
	},
	"VCCA": {
		ICAO:      "VCCA",
		IATA:      "ACJ",
		Name:      "Anuradhapura Air Force Base",
		City:      "Anuradhapura",
		State:     "North-Central",
		Country:   "LK",
		Elevation: 324,
		Latitude:  8.30148983,
		Longitude: 80.4279022217,
		Timezone:  "Asia/Colombo",
	},
	"VCCB": {
		ICAO:      "VCCB",
		IATA:      "BTC",
		Name:      "Batticaloa Airport",
		City:      "Batticaloa",
		State:     "Eastern-Province",
		Country:   "LK",
		Elevation: 20,
		Latitude:  7.7057600021,
		Longitude: 81.6788024902,
		Timezone:  "Asia/Colombo",
	},
	"VCCC": {
		ICAO:      "VCCC",
		IATA:      "RML",
		Name:      "Colombo Ratmalana Airport",
		City:      "Colombo",
		State:     "Western",
		Country:   "LK",
		Elevation: 22,
		Latitude:  6.8219900131,
		Longitude: 79.8861999512,
		Timezone:  "Asia/Colombo",
	},
	"VCCG": {
		ICAO:      "VCCG",
		IATA:      "GOY",
		Name:      "Amparai Airport",
		City:      "Amparai",
		State:     "Eastern-Province",
		Country:   "LK",
		Elevation: 150,
		Latitude:  7.3370800018,
		Longitude: 81.6259002686,
		Timezone:  "Asia/Colombo",
	},
	"VCCH": {
		ICAO:      "VCCH",
		IATA:      "MNH",
		Name:      "Hingurakgoda Air Force Base",
		City:      "Polonnaruwa Town",
		State:     "North-Central",
		Country:   "LK",
		Elevation: 170,
		Latitude:  8.0498104095,
		Longitude: 80.9813995361,
		Timezone:  "Asia/Colombo",
	},
	"VCCJ": {
		ICAO:      "VCCJ",
		IATA:      "JAF",
		Name:      "Kankesanturai Airport",
		City:      "Jaffna",
		State:     "Northern-Province",
		Country:   "LK",
		Elevation: 33,
		Latitude:  9.7923297882,
		Longitude: 80.070098877,
		Timezone:  "Asia/Colombo",
	},
	"VCCK": {
		ICAO:      "VCCK",
		IATA:      "KCT",
		Name:      "Koggala Airport",
		City:      "Galle",
		State:     "Southern",
		Country:   "LK",
		Elevation: 10,
		Latitude:  5.9936800003,
		Longitude: 80.3202972412,
		Timezone:  "Asia/Colombo",
	},
	"VCCN": {
		ICAO:      "VCCN",
		IATA:      "KTY",
		Name:      "Katukurunda Air Force Base",
		City:      "Kalutara",
		State:     "Western",
		Country:   "LK",
		Elevation: 10,
		Latitude:  6.5521202087,
		Longitude: 79.9775009155,
		Timezone:  "Asia/Colombo",
	},
	"VCCS": {
		ICAO:      "VCCS",
		IATA:      "GIU",
		Name:      "Sigiriya Air Force Base",
		City:      "Sigiriya",
		State:     "Central",
		Country:   "LK",
		Elevation: 630,
		Latitude:  7.9566698074,
		Longitude: 80.7285003662,
		Timezone:  "Asia/Colombo",
	},
	"VCCT": {
		ICAO:      "VCCT",
		IATA:      "TRR",
		Name:      "China Bay Airport",
		City:      "Trincomalee",
		State:     "Eastern-Province",
		Country:   "LK",
		Elevation: 6,
		Latitude:  8.5385103226,
		Longitude: 81.1819000244,
		Timezone:  "Asia/Colombo",
	},
	"VCCV": {
		ICAO:      "VCCV",
		Name:      "Vavuniya Airport",
		City:      "Vavuniya",
		State:     "Northern-Province",
		Country:   "LK",
		Elevation: 299,
		Latitude:  8.7413902283,
		Longitude: 80.4973983765,
		Timezone:  "Asia/Colombo",
	},
	"VCCW": {
		ICAO:      "VCCW",
		IATA:      "WRZ",
		Name:      "Weerawila International Airport",
		City:      "Weerawila",
		State:     "Southern",
		Country:   "LK",
		Elevation: 50,
		Latitude:  6.2544898987,
		Longitude: 81.2351989746,
		Timezone:  "Asia/Colombo",
	},
	"VCRI": {
		ICAO:      "VCRI",
		IATA:      "HRI",
		Name:      "Mattala Rajapaksa International Airport",
		City:      "Mattala",
		State:     "Hambantota",
		Country:   "LK",
		Elevation: 157,
		Latitude:  6.288889,
		Longitude: 81.123611,
		Timezone:  "Asia/Colombo",
	},
	"VDBG": {
		ICAO:      "VDBG",
		IATA:      "BBM",
		Name:      "Battambang Airport",
		City:      "Battambang",
		Country:   "KH",
		Elevation: 59,
		Latitude:  13.0956001282,
		Longitude: 103.2239990234,
		Timezone:  "Asia/Phnom_Penh",
	},
	"VDKC": {
		ICAO:      "VDKC",
		Name:      "Kompong Cham Airport",
		City:      "Kompong Cham",
		State:     "Kampong-Cham",
		Country:   "KH",
		Elevation: 148,
		Latitude:  12.0329999924,
		Longitude: 105.4329986572,
		Timezone:  "Asia/Phnom_Penh",
	},
	"VDKH": {
		ICAO:      "VDKH",
		IATA:      "KZC",
		Name:      "Kampong Chhnang Airport",
		State:     "Kampong-Chhnang",
		Country:   "KH",
		Elevation: 56,
		Latitude:  12.255200386,
		Longitude: 104.5640029907,
		Timezone:  "Asia/Phnom_Penh",
	},
	"VDKK": {
		ICAO:      "VDKK",
		IATA:      "KKZ",
		Name:      "Kaoh Kong Airport",
		City:      "Kaoh Kong",
		State:     "Koh-Kong",
		Country:   "KH",
		Latitude:  11.6133971186,
		Longitude: 102.997083664,
		Timezone:  "Asia/Phnom_Penh",
	},
	"VDKT": {
		ICAO:      "VDKT",
		IATA:      "KTI",
		Name:      "Kratie Airport",
		City:      "Kratie",
		State:     "Kratie",
		Country:   "KH",
		Latitude:  12.4879999161,
		Longitude: 106.0550003052,
		Timezone:  "Asia/Phnom_Penh",
	},
	"VDMK": {
		ICAO:      "VDMK",
		IATA:      "MWV",
		Name:      "Mondulkiri Airport",
		City:      "Sen Monorom",
		State:     "Mondolkiri",
		Country:   "KH",
		Latitude:  12.4636479116,
		Longitude: 107.187252045,
		Timezone:  "Asia/Phnom_Penh",
	},
	"VDPP": {
		ICAO:      "VDPP",
		IATA:      "PNH",
		Name:      "Phnom Penh International Airport",
		City:      "Phnom Penh",
		Country:   "KH",
		Elevation: 40,
		Latitude:  11.5466003418,
		Longitude: 104.84400177,
		Timezone:  "Asia/Phnom_Penh",
	},
	"VDRK": {
		ICAO:      "VDRK",
		IATA:      "RBE",
		Name:      "Ratanakiri Airport",
		City:      "Ratanakiri",
		State:     "Ratanakiri",
		Country:   "KH",
		Latitude:  13.7299995422,
		Longitude: 106.9869995117,
		Timezone:  "Asia/Phnom_Penh",
	},
	"VDSR": {
		ICAO:      "VDSR",
		IATA:      "REP",
		Name:      "Angkor International Airport",
		City:      "Siem Reap",
		State:     "Siem-Reap",
		Country:   "KH",
		Elevation: 60,
		Latitude:  13.4106998444,
		Longitude: 103.81300354,
		Timezone:  "Asia/Phnom_Penh",
	},
	"VDST": {
		ICAO:      "VDST",
		IATA:      "TNX",
		Name:      "Stung Treng Airport",
		City:      "Stung Treng",
		State:     "Stung-Treng",
		Country:   "KH",
		Elevation: 203,
		Latitude:  13.5319004059,
		Longitude: 106.0149993896,
		Timezone:  "Asia/Phnom_Penh",
	},
	"VDSV": {
		ICAO:      "VDSV",
		IATA:      "KOS",
		Name:      "Sihanoukville International Airport",
		City:      "Sihanukville",
		State:     "Kampot",
		Country:   "KH",
		Elevation: 33,
		Latitude:  10.57970047,
		Longitude: 103.637001038,
		Timezone:  "Asia/Phnom_Penh",
	},
	"VE23": {
		ICAO:      "VE23",
		Name:      "Burnpur Airport",
		State:     "West-Bengal",
		Country:   "IN",
		Elevation: 310,
		Latitude:  23.6315002441,
		Longitude: 86.9755020142,
		Timezone:  "Asia/Kolkata",
	},
	"VE24": {
		ICAO:      "VE24",
		Name:      "Sookerating Airport",
		State:     "Assam",
		Country:   "IN",
		Elevation: 400,
		Latitude:  27.5529003143,
		Longitude: 95.570602417,
		Timezone:  "Asia/Kolkata",
	},
	"VE36": {
		ICAO:      "VE36",
		Name:      "Nuagaon Airport",
		State:     "Odisha",
		Country:   "IN",
		Elevation: 656,
		Latitude:  20.5116996765,
		Longitude: 83.448097229,
		Timezone:  "Asia/Kolkata",
	},
	"VE41": {
		ICAO:      "VE41",
		Name:      "Giridih Airport",
		State:     "Jharkhand",
		Country:   "IN",
		Elevation: 1000,
		Latitude:  24.2019004822,
		Longitude: 86.2898025513,
		Timezone:  "Asia/Kolkata",
	},
	"VE44": {
		ICAO:      "VE44",
		Name:      "Hashimara Air Force Station",
		City:      "Jalpaiguri",
		State:     "West-Bengal",
		Country:   "IN",
		Elevation: 340,
		Latitude:  26.698299408,
		Longitude: 89.369102478,
		Timezone:  "Asia/Kolkata",
	},
	"VE54": {
		ICAO:      "VE54",
		Name:      "Daltonganj Airport",
		State:     "Jharkhand",
		Country:   "IN",
		Elevation: 803,
		Latitude:  24.0193004608,
		Longitude: 84.0951004028,
		Timezone:  "Asia/Kolkata",
	},
	"VE62": {
		ICAO:      "VE62",
		Name:      "Cuttack Airport",
		State:     "Odisha",
		Country:   "IN",
		Elevation: 115,
		Latitude:  20.5496006012,
		Longitude: 85.8862991333,
		Timezone:  "Asia/Kolkata",
	},
	"VE67": {
		ICAO:      "VE67",
		Name:      "Machuka Advanced Landing Ground",
		State:     "Arunachal-Pradesh",
		Country:   "IN",
		Elevation: 7900,
		Latitude:  28.6024580064,
		Longitude: 94.1261386871,
		Timezone:  "Asia/Kolkata",
	},
	"VE91": {
		ICAO:      "VE91",
		Name:      "Vijaynagar Advanced Landing Ground",
		State:     "Sagain",
		Country:   "IN",
		Elevation: 4167,
		Latitude:  27.1935997009,
		Longitude: 97.0037002563,
		Timezone:  "Asia/Kolkata",
	},
	"VE96": {
		ICAO:      "VE96",
		Name:      "Thuniabhand Airport",
		State:     "Madhya-Pradesh",
		Country:   "IN",
		Elevation: 2182,
		Latitude:  22.0016002655,
		Longitude: 78.9171981812,
		Timezone:  "Asia/Kolkata",
	},
	"VEAL": {
		ICAO:      "VEAL",
		Name:      "Savitri Jindal Airport",
		City:      "Angul",
		State:     "Odisha",
		Country:   "IN",
		Elevation: 624,
		Latitude:  20.910556,
		Longitude: 85.035,
		Timezone:  "Asia/Kolkata",
	},
	"VEAN": {
		ICAO:      "VEAN",
		IATA:      "IXV",
		Name:      "Along Airport",
		State:     "Arunachal-Pradesh",
		Country:   "IN",
		Elevation: 900,
		Latitude:  28.1753005981,
		Longitude: 94.8020019531,
		Timezone:  "Asia/Kolkata",
	},
	"VEAT": {
		ICAO:      "VEAT",
		IATA:      "IXA",
		Name:      "Agartala Airport",
		City:      "Agartala",
		State:     "Tripura",
		Country:   "IN",
		Elevation: 46,
		Latitude:  23.8869991302,
		Longitude: 91.2404022217,
		Timezone:  "Asia/Dhaka",
	},
	"VEBA": {
		ICAO:      "VEBA",
		Name:      "Behala Airport",
		City:      "Behala",
		State:     "West-Bengal",
		Country:   "IN",
		Elevation: 10,
		Latitude:  22.5042304993,
		Longitude: 88.2934265137,
		Timezone:  "Asia/Kolkata",
	},
	"VEBD": {
		ICAO:      "VEBD",
		IATA:      "IXB",
		Name:      "Bagdogra Airport",
		City:      "Siliguri",
		State:     "West-Bengal",
		Country:   "IN",
		Elevation: 412,
		Latitude:  26.6812000275,
		Longitude: 88.3285980225,
		Timezone:  "Asia/Kolkata",
	},
	"VEBG": {
		ICAO:      "VEBG",
		IATA:      "RGH",
		Name:      "Balurghat Airport",
		City:      "Balurghat",
		State:     "West-Bengal",
		Country:   "IN",
		Elevation: 78,
		Latitude:  25.2616996765,
		Longitude: 88.7956008911,
		Timezone:  "Asia/Kolkata",
	},
	"VEBI": {
		ICAO:      "VEBI",
		IATA:      "SHL",
		Name:      "Shillong Airport",
		City:      "Shillong",
		State:     "Meghalaya",
		Country:   "IN",
		Elevation: 2910,
		Latitude:  25.7035999298,
		Longitude: 91.9786987305,
		Timezone:  "Asia/Kolkata",
	},
	"VEBK": {
		ICAO:      "VEBK",
		Name:      "Bokaro Airport",
		State:     "Jharkhand",
		Country:   "IN",
		Elevation: 715,
		Latitude:  23.6434993744,
		Longitude: 86.1489028931,
		Timezone:  "Asia/Kolkata",
	},
	"VEBN": {
		ICAO:      "VEBN",
		IATA:      "VNS",
		Name:      "Lal Bahadur Shastri Airport",
		City:      "Varanasi",
		State:     "Uttar Pradesh",
		Country:   "IN",
		Elevation: 270,
		Latitude:  25.451167,
		Longitude: 82.858667,
		Timezone:  "Asia/Kolkata",
	},
	"VEBS": {
		ICAO:      "VEBS",
		IATA:      "BBI",
		Name:      "Biju Patnaik Airport",
		City:      "Bhubaneswar",
		State:     "Odisha",
		Country:   "IN",
		Elevation: 138,
		Latitude:  20.2444000244,
		Longitude: 85.8178024292,
		Timezone:  "Asia/Kolkata",
	},
	"VECA": {
		ICAO:      "VECA",
		Name:      "Chabua Air Force Station",
		City:      "Dibrugarh",
		State:     "Assam",
		Country:   "IN",
		Elevation: 367,
		Latitude:  27.4622001648,
		Longitude: 95.1176986694,
		Timezone:  "Asia/Kolkata",
	},
	"VECC": {
		ICAO:      "VECC",
		IATA:      "CCU",
		Name:      "Netaji Subhash Chandra Bose International Airport",
		City:      "Kolkata",
		State:     "West-Bengal",
		Country:   "IN",
		Elevation: 16,
		Latitude:  22.6546993256,
		Longitude: 88.4467010498,
		Timezone:  "Asia/Kolkata",
	},
	"VECK": {
		ICAO:      "VECK",
		Name:      "Chakulia Airport",
		State:     "Jharkhand",
		Country:   "IN",
		Elevation: 430,
		Latitude:  22.4605998993,
		Longitude: 86.7078018188,
		Timezone:  "Asia/Kolkata",
	},
	"VECO": {
		ICAO:      "VECO",
		IATA:      "COH",
		Name:      "Cooch Behar Airport",
		State:     "West-Bengal",
		Country:   "IN",
		Elevation: 138,
		Latitude:  26.330499649,
		Longitude: 89.4672012329,
		Timezone:  "Asia/Kolkata",
	},
	"VEDB": {
		ICAO:      "VEDB",
		IATA:      "DBD",
		Name:      "Dhanbad Airport",
		State:     "Bihar",
		Country:   "IN",
		Elevation: 847,
		Latitude:  23.8339996338,
		Longitude: 86.4253005981,
		Timezone:  "Asia/Kolkata",
	},
	"VEDH": {
		ICAO:      "VEDH",
		IATA:      "DBR",
		Name:      "Darbhanga Airport",
		City:      "Darbhanga",
		State:     "Bihar",
		Country:   "IN",
		Elevation: 171,
		Latitude:  26.194722,
		Longitude: 85.9175,
		Timezone:  "Asia/Kolkata",
	},
	"VEDO": {
		ICAO:      "VEDO",
		IATA:      "DGH",
		Name:      "Deoghar Airport",
		City:      "Deoghar",
		State:     "Jharkhand",
		Country:   "IN",
		Elevation: 833,
		Latitude:  24.444722,
		Longitude: 86.7025,
		Timezone:  "Asia/Kolkata",
	},
	"VEDX": {
		ICAO:      "VEDX",
		Name:      "Kalaikunda Air Force Station",
		State:     "West-Bengal",
		Country:   "IN",
		Elevation: 200,
		Latitude:  22.3395004272,
		Longitude: 87.2145004272,
		Timezone:  "Asia/Kolkata",
	},
	"VEDZ": {
		ICAO:      "VEDZ",
		IATA:      "DAE",
		Name:      "Daporijo Airport",
		State:     "Arunachal-Pradesh",
		Country:   "IN",
		Elevation: 750,
		Latitude:  27.9855003357,
		Longitude: 94.2228012085,
		Timezone:  "Asia/Kolkata",
	},
	"VEGK": {
		ICAO:      "VEGK",
		IATA:      "GOP",
		Name:      "Gorakhpur Airport",
		City:      "Gorakhpur",
		State:     "Uttar-Pradesh",
		Country:   "IN",
		Elevation: 259,
		Latitude:  26.7397003174,
		Longitude: 83.4496994019,
		Timezone:  "Asia/Kolkata",
	},
	"VEGT": {
		ICAO:      "VEGT",
		IATA:      "GAU",
		Name:      "Lokpriya Gopinath Bordoloi International Airport",
		City:      "Guwahati",
		State:     "Assam",
		Country:   "IN",
		Elevation: 162,
		Latitude:  26.1061000824,
		Longitude: 91.585899353,
		Timezone:  "Asia/Kolkata",
	},
	"VEGY": {
		ICAO:      "VEGY",
		IATA:      "GAY",
		Name:      "Gaya Airport",
		State:     "Bihar",
		Country:   "IN",
		Elevation: 380,
		Latitude:  24.7443008423,
		Longitude: 84.9512023926,
		Timezone:  "Asia/Kolkata",
	},
	"VEHK": {
		ICAO:      "VEHK",
		Name:      "Hirakud Airport",
		City:      "Sambalpur",
		State:     "Odisha",
		Country:   "IN",
		Elevation: 658,
		Latitude:  21.5802001953,
		Longitude: 84.0056991577,
		Timezone:  "Asia/Kolkata",
	},
	"VEIM": {
		ICAO:      "VEIM",
		IATA:      "IMF",
		Name:      "Imphal Airport",
		City:      "Imphal",
		State:     "Manipur",
		Country:   "IN",
		Elevation: 2540,
		Latitude:  24.7600002289,
		Longitude: 93.896697998,
		Timezone:  "Asia/Kolkata",
	},
	"VEJH": {
		ICAO:      "VEJH",
		IATA:      "JRG",
		Name:      "Jharsuguda Airport",
		City:      "Jharsuguda",
		State:     "Odisha",
		Country:   "IN",
		Elevation: 757,
		Latitude:  21.9134998322,
		Longitude: 84.0503997803,
		Timezone:  "Asia/Kolkata",
	},
	"VEJP": {
		ICAO:      "VEJP",
		IATA:      "PYB",
		Name:      "Jeypore Airport",
		City:      "Jeypore",
		State:     "Odisha",
		Country:   "IN",
		Elevation: 1952,
		Latitude:  18.8799991608,
		Longitude: 82.5520019531,
		Timezone:  "Asia/Kolkata",
	},
	"VEJS": {
		ICAO:      "VEJS",
		IATA:      "IXW",
		Name:      "Jamshedpur Airport",
		State:     "Jharkhand",
		Country:   "IN",
		Elevation: 505,
		Latitude:  22.8131999969,
		Longitude: 86.168800354,
		Timezone:  "Asia/Kolkata",
	},
	"VEJT": {
		ICAO:      "VEJT",
		IATA:      "JRH",
		Name:      "Jorhat Airport",
		City:      "Jorhat",
		State:     "Assam",
		Country:   "IN",
		Elevation: 311,
		Latitude:  26.7315006256,
		Longitude: 94.1754989624,
		Timezone:  "Asia/Kolkata",
	},
	"VEKM": {
		ICAO:      "VEKM",
		IATA:      "IXQ",
		Name:      "Kamalpur Airport",
		State:     "Tripura",
		Country:   "IN",
		Elevation: 131,
		Latitude:  24.1317005157,
		Longitude: 91.814201355,
		Timezone:  "Asia/Kolkata",
	},
	"VEKR": {
		ICAO:      "VEKR",
		IATA:      "IXH",
		Name:      "Kailashahar Airport",
		State:     "Tripura",
		Country:   "IN",
		Elevation: 79,
		Latitude:  24.3082008362,
		Longitude: 92.0072021484,
		Timezone:  "Asia/Dhaka",
	},
	"VEKU": {
		ICAO:      "VEKU",
		IATA:      "IXS",
		Name:      "Silchar Airport",
		City:      "Silchar",
		State:     "Assam",
		Country:   "IN",
		Elevation: 352,
		Latitude:  24.9129009247,
		Longitude: 92.9786987305,
		Timezone:  "Asia/Kolkata",
	},
	"VEKW": {
		ICAO:      "VEKW",
		IATA:      "IXN",
		Name:      "Khowai Airport",
		City:      "Khowai",
		State:     "Tripura",
		Country:   "IN",
		Elevation: 95,
		Latitude:  24.0618991852,
		Longitude: 91.6038970947,
		Timezone:  "Asia/Dhaka",
	},
	"VELP": {
		ICAO:      "VELP",
		IATA:      "AJL",
		Name:      "Lengpui Airport",
		City:      "Aizawl",
		State:     "Mizoram",
		Country:   "IN",
		Elevation: 1398,
		Latitude:  23.8405990601,
		Longitude: 92.6196975708,
		Timezone:  "Asia/Kolkata",
	},
	"VELR": {
		ICAO:      "VELR",
		IATA:      "IXI",
		Name:      "North Lakhimpur Airport",
		City:      "Lilabari",
		State:     "Assam",
		Country:   "IN",
		Elevation: 330,
		Latitude:  27.2954998016,
		Longitude: 94.0976028442,
		Timezone:  "Asia/Kolkata",
	},
	"VEMH": {
		ICAO:      "VEMH",
		IATA:      "LDA",
		Name:      "Malda Airport",
		City:      "Malda",
		Country:   "IN",
		Elevation: 79,
		Latitude:  25.033000946,
		Longitude: 88.1330032349,
		Timezone:  "Asia/Kolkata",
	},
	"VEMN": {
		ICAO:      "VEMN",
		IATA:      "DIB",
		Name:      "Dibrugarh Airport",
		City:      "Dibrugarh",
		State:     "Assam",
		Country:   "IN",
		Elevation: 362,
		Latitude:  27.4839000702,
		Longitude: 95.0168991089,
		Timezone:  "Asia/Kolkata",
	},
	"VEMR": {
		ICAO:      "VEMR",
		IATA:      "DMU",
		Name:      "Dimapur Airport",
		City:      "Dimapur",
		State:     "Nagaland",
		Country:   "IN",
		Elevation: 487,
		Latitude:  25.8838996887,
		Longitude: 93.7711029053,
		Timezone:  "Asia/Kolkata",
	},
	"VEMZ": {
		ICAO:      "VEMZ",
		IATA:      "MZU",
		Name:      "Muzaffarpur Airport",
		State:     "Bihar",
		Country:   "IN",
		Elevation: 174,
		Latitude:  26.1191005707,
		Longitude: 85.3136978149,
		Timezone:  "Asia/Kolkata",
	},
	"VENP": {
		ICAO:      "VENP",
		Name:      "Nawapara Airport",
		City:      "Nawapara",
		State:     "Odisha",
		Country:   "IN",
		Elevation: 1058,
		Latitude:  20.8700008392,
		Longitude: 82.5195999146,
		Timezone:  "Asia/Kolkata",
	},
	"VEPG": {
		ICAO:      "VEPG",
		IATA:      "IXT",
		Name:      "Pasighat Airport",
		City:      "Pasighat",
		State:     "Arunachal-Pradesh",
		Country:   "IN",
		Elevation: 477,
		Latitude:  28.0660991669,
		Longitude: 95.3356018066,
		Timezone:  "Asia/Kolkata",
	},
	"VEPH": {
		ICAO:      "VEPH",
		Name:      "Panagarh Air Force Station",
		State:     "West-Bengal",
		Country:   "IN",
		Elevation: 240,
		Latitude:  23.4743003845,
		Longitude: 87.4274978638,
		Timezone:  "Asia/Kolkata",
	},
	"VEPI": {
		ICAO:      "VEPI",
		Name:      "Barrackpore Air Force Station",
		City:      "Barrackpore",
		State:     "West-Bengal",
		Country:   "IN",
		Elevation: 18,
		Latitude:  22.7810001373,
		Longitude: 88.3591995239,
		Timezone:  "Asia/Kolkata",
	},
	"VEPT": {
		ICAO:      "VEPT",
		IATA:      "PAT",
		Name:      "Lok Nayak Jayaprakash Airport",
		City:      "Patna",
		State:     "Bihar",
		Country:   "IN",
		Elevation: 170,
		Latitude:  25.591299057,
		Longitude: 85.0879974365,
		Timezone:  "Asia/Kolkata",
	},
	"VEPU": {
		ICAO:      "VEPU",
		Name:      "Purnea Airport",
		State:     "Bihar",
		Country:   "IN",
		Elevation: 129,
		Latitude:  25.7595996857,
		Longitude: 87.4100036621,
		Timezone:  "Asia/Kolkata",
	},
	"VERC": {
		ICAO:      "VERC",
		IATA:      "IXR",
		Name:      "Birsa Munda Airport",
		City:      "Ranchi",
		State:     "Jharkhand",
		Country:   "IN",
		Elevation: 2148,
		Latitude:  23.3143005371,
		Longitude: 85.3217010498,
		Timezone:  "Asia/Kolkata",
	},
	"VERK": {
		ICAO:      "VERK",
		IATA:      "RRK",
		Name:      "Rourkela Airport",
		State:     "Odisha",
		Country:   "IN",
		Elevation: 659,
		Latitude:  22.2567005157,
		Longitude: 84.8145980835,
		Timezone:  "Asia/Kolkata",
	},
	"VERL": {
		ICAO:      "VERL",
		Name:      "Raxaul Airport",
		Country:   "IN",
		Elevation: 259,
		Latitude:  26.9666996002,
		Longitude: 84.8332977295,
		Timezone:  "Asia/Kolkata",
	},
	"VERU": {
		ICAO:      "VERU",
		IATA:      "RUP",
		Name:      "Rupsi India Airport",
		State:     "Assam",
		Country:   "IN",
		Elevation: 131,
		Latitude:  26.1396999359,
		Longitude: 89.9100036621,
		Timezone:  "Asia/Kolkata",
	},
	"VETJ": {
		ICAO:      "VETJ",
		IATA:      "TEI",
		Name:      "Tezu Airport",
		City:      "Tezu",
		State:     "Arunachal-Pradesh",
		Country:   "IN",
		Elevation: 800,
		Latitude:  27.9412002563,
		Longitude: 96.1343994141,
		Timezone:  "Asia/Kolkata",
	},
	"VETZ": {
		ICAO:      "VETZ",
		IATA:      "TEZ",
		Name:      "Tezpur Airport",
		State:     "Assam",
		Country:   "IN",
		Elevation: 240,
		Latitude:  26.7091007233,
		Longitude: 92.7846984863,
		Timezone:  "Asia/Kolkata",
	},
	"VEUK": {
		ICAO:      "VEUK",
		Name:      "Utkela Airport",
		State:     "Odisha",
		Country:   "IN",
		Elevation: 680,
		Latitude:  20.0974006653,
		Longitude: 83.1837997437,
		Timezone:  "Asia/Kolkata",
	},
	"VEVZ": {
		ICAO:      "VEVZ",
		IATA:      "VTZ",
		Name:      "Vishakhapatnam Airport",
		City:      "Visakhapatnam",
		State:     "Andhra-Pradesh",
		Country:   "IN",
		Elevation: 15,
		Latitude:  17.721200943,
		Longitude: 83.2245025635,
		Timezone:  "Asia/Kolkata",
	},
	"VEZO": {
		ICAO:      "VEZO",
		IATA:      "ZER",
		Name:      "Zero Airport",
		State:     "Arunachal-Pradesh",
		Country:   "IN",
		Elevation: 5403,
		Latitude:  27.588300705,
		Longitude: 93.8281021118,
		Timezone:  "Asia/Kolkata",
	},
	"VG00": {
		ICAO:      "VG00",
		Name:      "Mears Field",
		City:      "Eastville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 22,
		Latitude:  37.3698997498,
		Longitude: -75.9468994141,
		Timezone:  "America/New_York",
	},
	"VG01": {
		ICAO:      "VG01",
		Name:      "Eureka Airport",
		City:      "Keysville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 420,
		Latitude:  37.0624008179,
		Longitude: -78.5616989136,
		Timezone:  "America/New_York",
	},
	"VG05": {
		ICAO:      "VG05",
		Name:      "Payne Airport",
		City:      "Tabscott",
		State:     "Virginia",
		Country:   "US",
		Elevation: 463,
		Latitude:  37.8796005249,
		Longitude: -78.0575027466,
		Timezone:  "America/New_York",
	},
	"VG06": {
		ICAO:      "VG06",
		Name:      "Fulcher Family Farms Airport",
		City:      "Amherst",
		State:     "Virginia",
		Country:   "US",
		Elevation: 980,
		Latitude:  37.7251014709,
		Longitude: -79.0830993652,
		Timezone:  "America/New_York",
	},
	"VG07": {
		ICAO:      "VG07",
		Name:      "Rular Airport",
		City:      "Remington",
		State:     "Virginia",
		Country:   "US",
		Elevation: 305,
		Latitude:  38.538898468,
		Longitude: -77.8283004761,
		Timezone:  "America/New_York",
	},
	"VG09": {
		ICAO:      "VG09",
		Name:      "Starbase Airport",
		City:      "Cape Charles",
		State:     "Virginia",
		Country:   "US",
		Elevation: 20,
		Latitude:  37.2277984619,
		Longitude: -75.9832992554,
		Timezone:  "America/New_York",
	},
	"VG10": {
		ICAO:      "VG10",
		Name:      "Cathro Airport",
		City:      "Charlottesville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 500,
		Latitude:  37.927898407,
		Longitude: -78.551399231,
		Timezone:  "America/New_York",
	},
	"VG12": {
		ICAO:      "VG12",
		Name:      "Simpsonville Airport",
		City:      "Rhoadesville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 400,
		Latitude:  38.3334999084,
		Longitude: -77.8664016724,
		Timezone:  "America/New_York",
	},
	"VG16": {
		ICAO:      "VG16",
		Name:      "Collins Airport",
		City:      "Patrick Springs",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1100,
		Latitude:  36.6250991821,
		Longitude: -80.1706008911,
		Timezone:  "America/New_York",
	},
	"VG18": {
		ICAO:      "VG18",
		Name:      "Sky Bryce Airport",
		City:      "Basye",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1263,
		Latitude:  38.8158988953,
		Longitude: -78.7703018188,
		Timezone:  "America/New_York",
	},
	"VG19": {
		ICAO:      "VG19",
		Name:      "Sawyer STOLport",
		City:      "New Church",
		State:     "Virginia",
		Country:   "US",
		Elevation: 13,
		Latitude:  37.9906997681,
		Longitude: -75.5919036865,
		Timezone:  "America/New_York",
	},
	"VG20": {
		ICAO:      "VG20",
		Name:      "Onley Airport",
		City:      "Onley",
		State:     "Virginia",
		Country:   "US",
		Elevation: 45,
		Latitude:  37.6750984192,
		Longitude: -75.7098999023,
		Timezone:  "America/New_York",
	},
	"VG22": {
		ICAO:      "VG22",
		Name:      "Rockfish Airpark",
		City:      "Wintergreen",
		State:     "Virginia",
		Country:   "US",
		Elevation: 760,
		Latitude:  37.8796005249,
		Longitude: -78.8936004639,
		Timezone:  "America/New_York",
	},
	"VG23": {
		ICAO:      "VG23",
		Name:      "Smith Airport",
		City:      "Arcola",
		State:     "Virginia",
		Country:   "US",
		Elevation: 365,
		Latitude:  38.927898407,
		Longitude: -77.5627975464,
		Timezone:  "America/New_York",
	},
	"VG24": {
		ICAO:      "VG24",
		Name:      "Mayers Airport",
		City:      "Ashland",
		State:     "Virginia",
		Country:   "US",
		Elevation: 215,
		Latitude:  37.7817993164,
		Longitude: -77.4938964844,
		Timezone:  "America/New_York",
	},
	"VG25": {
		ICAO:      "VG25",
		Name:      "Robinson Airport",
		City:      "Bedford",
		State:     "Virginia",
		Country:   "US",
		Elevation: 850,
		Latitude:  37.3484992981,
		Longitude: -79.5719985962,
		Timezone:  "America/New_York",
	},
	"VG27": {
		ICAO:      "VG27",
		Name:      "Windy Ridge Airport",
		City:      "Bedford",
		State:     "Virginia",
		Country:   "US",
		Elevation: 750,
		Latitude:  37.2692985535,
		Longitude: -79.4358978271,
		Timezone:  "America/New_York",
	},
	"VG28": {
		ICAO:      "VG28",
		Name:      "Mann Airport",
		City:      "Boykins",
		State:     "Virginia",
		Country:   "US",
		Elevation: 35,
		Latitude:  36.5489997864,
		Longitude: -77.1913986206,
		Timezone:  "America/New_York",
	},
	"VG29": {
		ICAO:      "VG29",
		Name:      "Branham Mill Airpark",
		City:      "Warsaw",
		State:     "Virginia",
		Country:   "US",
		Elevation: 100,
		Latitude:  37.9707984924,
		Longitude: -76.7097015381,
		Timezone:  "America/New_York",
	},
	"VG30": {
		ICAO:      "VG30",
		Name:      "Scott Farm Strip",
		City:      "Cape Charles",
		State:     "Virginia",
		Country:   "US",
		Elevation: 33,
		Latitude:  37.2271003723,
		Longitude: -75.9768981934,
		Timezone:  "America/New_York",
	},
	"VG31": {
		ICAO:      "VG31",
		Name:      "Sager Field",
		City:      "Broadway",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1280,
		Latitude:  38.7134017944,
		Longitude: -78.863899231,
		Timezone:  "America/New_York",
	},
	"VG32": {
		ICAO:      "VG32",
		Name:      "Eastview Airport",
		City:      "Buckingham",
		State:     "Virginia",
		Country:   "US",
		Elevation: 500,
		Latitude:  37.6142997742,
		Longitude: -78.6035995483,
		Timezone:  "America/New_York",
	},
	"VG33": {
		ICAO:      "VG33",
		Name:      "Bull Farm Airport",
		City:      "Cape Charles",
		State:     "Virginia",
		Country:   "US",
		Elevation: 18,
		Latitude:  37.1545982361,
		Longitude: -75.9569015503,
		Timezone:  "America/New_York",
	},
	"VG34": {
		ICAO:      "VG34",
		Name:      "Merifield Airport",
		City:      "Clarksville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 355,
		Latitude:  36.5845985413,
		Longitude: -78.5008010864,
		Timezone:  "America/New_York",
	},
	"VG36": {
		ICAO:      "VG36",
		Name:      "Keysville Airport",
		City:      "Keysville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 627,
		Latitude:  37.0382003784,
		Longitude: -78.4627990723,
		Timezone:  "America/New_York",
	},
	"VG37": {
		ICAO:      "VG37",
		Name:      "Umphlett Airstrip",
		City:      "Whaleyville",
		State:     "Virginia",
		Country:   "US",
		Elevation: 75,
		Latitude:  36.5892982483,
		Longitude: -76.6380004883,
		Timezone:  "America/New_York",
	},
	"VG38": {
		ICAO:      "VG38",
		Name:      "Easter Field",
		City:      "Amelia",
		State:     "Virginia",
		Country:   "US",
		Elevation: 380,
		Latitude:  37.3695983887,
		Longitude: -78.0830993652,
		Timezone:  "America/New_York",
	},
	"VG39": {
		ICAO:      "VG39",
		Name:      "Earth Airport",
		City:      "Cape Charles",
		State:     "Virginia",
		Country:   "US",
		Elevation: 9,
		Latitude:  37.2207984924,
		Longitude: -76.0074996948,
		Timezone:  "America/New_York",
	},
	"VG40": {
		ICAO:      "VG40",
		Name:      "Woody Field Airport",
		City:      "Rocky Mount",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1150,
		Latitude:  36.8835983276,
		Longitude: -79.8523025513,
		Timezone:  "America/New_York",
	},
	"VG42": {
		ICAO:      "VG42",
		Name:      "Henshaw Airport",
		City:      "Hopewell",
		State:     "Virginia",
		Country:   "US",
		Elevation: 85,
		Latitude:  37.2523994446,
		Longitude: -77.1527023315,
		Timezone:  "America/New_York",
	},
	"VG43": {
		ICAO:      "VG43",
		Name:      "Arrowpoint Airport",
		City:      "Madison",
		State:     "Virginia",
		Country:   "US",
		Elevation: 315,
		Latitude:  38.3064002991,
		Longitude: -78.0988998413,
		Timezone:  "America/New_York",
	},
	"VG52": {
		ICAO:      "VG52",
		Name:      "Woodridge Field",
		City:      "Lovingston",
		State:     "Virginia",
		Country:   "US",
		Elevation: 663,
		Latitude:  37.8391990662,
		Longitude: -78.8217010498,
		Timezone:  "America/New_York",
	},
	"VG53": {
		ICAO:      "VG53",
		Name:      "Venning's Landing Airport",
		City:      "Rocky Mount",
		State:     "Virginia",
		Country:   "US",
		Elevation: 1100,
		Latitude:  37.043598175,
		Longitude: -79.801399231,
		Timezone:  "America/New_York",
	},
	"VG55": {
		ICAO:      "VG55",
		Name:      "Woodstock Airport",
		City:      "Woodstock",
		State:     "Virginia",
		Country:   "US",
		Elevation: 965,
		Latitude:  38.8819999695,
		Longitude: -78.557800293,
		Timezone:  "America/New_York",
	},
	"VG56": {
		ICAO:      "VG56",
		Name:      "Midway Airport",
		City:      "Bloxom",
		State:     "Virginia",
		Country:   "US",
		Elevation: 50,
		Latitude:  37.8083000183,
		Longitude: -75.6027984619,
		Timezone:  "America/New_York",
	},
	"VG57": {
		ICAO:      "VG57",
		Name:      "Maples Field",
		City:      "Manassas",
		State:     "Virginia",
		Country:   "US",
		Elevation: 225,
		Latitude:  38.6057014465,
		Longitude: -77.5847015381,
		Timezone:  "America/New_York",
	},
	"VG58": {
		ICAO:      "VG58",
		Name:      "Abbott Airport",
		City:      "Rocky Mount",
		State:     "Virginia",
		Country:   "US",
		Elevation: 981,
		Latitude:  36.945333,
		Longitude: -79.796667,
		Timezone:  "America/New_York",
	},
	"VG64": {
		ICAO:      "VG64",
		Name:      "Lee Field",
		City:      "West Point",
		State:     "Virginia",
		Country:   "US",
		Elevation: 20,
		Latitude:  37.5615,
		Longitude: -76.852528,
		Timezone:  "America/New_York",
	},
	"VGBR": {
		ICAO:      "VGBR",
		IATA:      "BZL",
		Name:      "Barisal Airport",
		City:      "Barisal",
		State:     "Barisal",
		Country:   "BD",
		Elevation: 23,
		Latitude:  22.8010005951,
		Longitude: 90.3012008667,
		Timezone:  "Asia/Dhaka",
	},
	"VGCB": {
		ICAO:      "VGCB",
		IATA:      "CXB",
		Name:      "Cox's Bazar Airport",
		City:      "Cox's Bazar",
		State:     "Chittagong",
		Country:   "BD",
		Elevation: 12,
		Latitude:  21.4521999359,
		Longitude: 91.9638977051,
		Timezone:  "Asia/Dhaka",
	},
	"VGCM": {
		ICAO:      "VGCM",
		IATA:      "CLA",
		Name:      "Comilla Airport",
		City:      "Comilla",
		State:     "Chittagong",
		Country:   "BD",
		Elevation: 25,
		Latitude:  23.4372005463,
		Longitude: 91.1896972656,
		Timezone:  "Asia/Dhaka",
	},
	"VGEG": {
		ICAO:      "VGEG",
		IATA:      "CGP",
		Name:      "Shah Amanat International Airport",
		City:      "Chittagong",
		State:     "Chittagong",
		Country:   "BD",
		Elevation: 12,
		Latitude:  22.2495994568,
		Longitude: 91.8133010864,
		Timezone:  "Asia/Dhaka",
	},
	"VGHS": {
		ICAO:      "VGHS",
		IATA:      "DAC",
		Name:      "Dhaka / Hazrat Shahjalal International Airport",
		City:      "Dhaka",
		State:     "Dhaka",
		Country:   "BD",
		Elevation: 30,
		Latitude:  23.843347,
		Longitude: 90.397783,
		Timezone:  "Asia/Dhaka",
	},
	"VGIS": {
		ICAO:      "VGIS",
		IATA:      "IRD",
		Name:      "Ishurdi Airport",
		City:      "Ishurdi",
		State:     "Rajshahi-Division",
		Country:   "BD",
		Elevation: 45,
		Latitude:  24.1525001526,
		Longitude: 89.0494003296,
		Timezone:  "Asia/Dhaka",
	},
	"VGJR": {
		ICAO:      "VGJR",
		IATA:      "JSR",
		Name:      "Jessore Airport",
		City:      "Jashahor",
		State:     "Khulna",
		Country:   "BD",
		Elevation: 20,
		Latitude:  23.1837997437,
		Longitude: 89.1607971191,
		Timezone:  "Asia/Dhaka",
	},
	"VGLM": {
		ICAO:      "VGLM",
		IATA:      "LLJ",
		Name:      "Lalmonirhat Airport",
		City:      "Lalmonirhat",
		State:     "Rangpur-Division",
		Country:   "BD",
		Elevation: 106,
		Latitude:  25.8875007629,
		Longitude: 89.4330978394,
		Timezone:  "Asia/Dhaka",
	},
	"VGRJ": {
		ICAO:      "VGRJ",
		IATA:      "RJH",
		Name:      "Shah Mokhdum Airport",
		City:      "Rajshahi",
		State:     "Rajshahi-Division",
		Country:   "BD",
		Elevation: 64,
		Latitude:  24.4372005463,
		Longitude: 88.6165008545,
		Timezone:  "Asia/Dhaka",
	},
	"VGSD": {
		ICAO:      "VGSD",
		IATA:      "SPD",
		Name:      "Saidpur Airport",
		City:      "Saidpur",
		State:     "Rangpur-Division",
		Country:   "BD",
		Elevation: 125,
		Latitude:  25.7591991425,
		Longitude: 88.9088973999,
		Timezone:  "Asia/Dhaka",
	},
	"VGSG": {
		ICAO:      "VGSG",
		IATA:      "TKR",
		Name:      "Thakurgaon Airport",
		City:      "Thakurgaon",
		State:     "Rangpur-Division",
		Country:   "BD",
		Elevation: 176,
		Latitude:  26.0163993835,
		Longitude: 88.4036026001,
		Timezone:  "Asia/Dhaka",
	},
	"VGSH": {
		ICAO:      "VGSH",
		IATA:      "ZHM",
		Name:      "Shamshernagar Airport",
		City:      "Shamshernagar",
		State:     "Sylhet",
		Country:   "BD",
		Elevation: 56,
		Latitude:  24.4169998169,
		Longitude: 91.8830032349,
		Timezone:  "Asia/Dhaka",
	},
	"VGSY": {
		ICAO:      "VGSY",
		IATA:      "ZYL",
		Name:      "Osmany International Airport",
		City:      "Sylhet",
		State:     "Sylhet",
		Country:   "BD",
		Elevation: 50,
		Latitude:  24.9631996155,
		Longitude: 91.8667984009,
		Timezone:  "Asia/Dhaka",
	},
	"VGTJ": {
		ICAO:      "VGTJ",
		Name:      "Tejgaon Airport",
		City:      "Dhaka",
		State:     "Dhaka",
		Country:   "BD",
		Elevation: 24,
		Latitude:  23.778799057,
		Longitude: 90.3826980591,
		Timezone:  "Asia/Dhaka",
	},
	"VHHH": {
		ICAO:      "VHHH",
		IATA:      "HKG",
		Name:      "Chek Lap Kok International Airport",
		City:      "Hong Kong",
		Country:   "HK",
		Elevation: 28,
		Latitude:  22.3089008331,
		Longitude: 113.915000916,
		Timezone:  "Asia/Hong_Kong",
	},
	"VHSK": {
		ICAO:      "VHSK",
		Name:      "Shek Kong Air Base",
		City:      "Hong Kong",
		Country:   "HK",
		Elevation: 50,
		Latitude:  22.4365997314,
		Longitude: 114.0800018311,
		Timezone:  "Asia/Hong_Kong",
	},
	"VI20": {
		ICAO:      "VI20",
		Name:      "Bhilwara Airport",
		State:     "Rajasthan",
		Country:   "IN",
		Elevation: 955,
		Latitude:  28.0771007538,
		Longitude: 76.2050018311,
		Timezone:  "Asia/Kolkata",
	},
	"VI40": {
		ICAO:      "VI40",
		Name:      "Karnal Airport",
		State:     "Haryana",
		Country:   "IN",
		Elevation: 829,
		Latitude:  29.714099884,
		Longitude: 77.0374984741,
		Timezone:  "Asia/Kolkata",
	},
	"VI43": {
		ICAO:      "VI43",
		Name:      "Suratgarh New Airport",
		State:     "Rajasthan",
		Country:   "IN",
		Elevation: 554,
		Latitude:  29.3878002167,
		Longitude: 73.9039001465,
		Timezone:  "Asia/Kolkata",
	},
	"VI57": {
		ICAO:      "VI57",
		Name:      "Thoise Airport",
		State:     "Jammu-and-Kashmir",
		Country:   "IN",
		Elevation: 10046,
		Latitude:  34.6525993347,
		Longitude: 77.3758010864,
		Timezone:  "Asia/Kolkata",
	},
	"VI65": {
		ICAO:      "VI65",
		Name:      "Kargil Airport",
		State:     "Jammu-and-Kashmir",
		Country:   "IN",
		Elevation: 9604,
		Latitude:  34.5242996216,
		Longitude: 76.1558990479,
		Timezone:  "Asia/Kolkata",
	},
	"VI66": {
		ICAO:      "VI66",
		Name:      "Fukche Advanced Landing Ground",
		State:     "Jammu-and-Kashmir",
		Country:   "IN",
		Elevation: 13700,
		Latitude:  32.9374008179,
		Longitude: 79.2132034302,
		Timezone:  "Asia/Kolkata",
	},
	"VI69": {
		ICAO:      "VI69",
		Name:      "Jhunjhunu Airport",
		State:     "Rajasthan",
		Country:   "IN",
		Elevation: 1110,
		Latitude:  28.1067008972,
		Longitude: 75.3756027222,
		Timezone:  "Asia/Kolkata",
	},
	"VI70": {
		ICAO:      "VI70",
		Name:      "Pilani New Airport",
		State:     "Rajasthan",
		Country:   "IN",
		Elevation: 1100,
		Latitude:  28.3488006592,
		Longitude: 75.5936965942,
		Timezone:  "Asia/Kolkata",
	},
	"VI71": {
		ICAO:      "VI71",
		Name:      "Kalka Airport",
		State:     "Haryana",
		Country:   "IN",
		Elevation: 1700,
		Latitude:  30.8245697021,
		Longitude: 76.8836517334,
		Timezone:  "Asia/Kolkata",
	},
	"VI73": {
		ICAO:      "VI73",
		Name:      "Nagaur Airport",
		State:     "Rajasthan",
		Country:   "IN",
		Elevation: 950,
		Latitude:  27.2082996368,
		Longitude: 73.7114028931,
		Timezone:  "Asia/Kolkata",
	},
	"VI75": {
		ICAO:      "VI75",
		Name:      "Kalyanpur Airport",
		State:     "Uttar-Pradesh",
		Country:   "IN",
		Elevation: 400,
		Latitude:  26.5203990936,
		Longitude: 80.2329025269,
		Timezone:  "Asia/Kolkata",
	},
	"VI76": {
		ICAO:      "VI76",
		Name:      "Band Tal Airport",
		State:     "Madhya-Pradesh",
		Country:   "IN",
		Elevation: 790,
		Latitude:  26.0000991821,
		Longitude: 78.2618026733,
		Timezone:  "Asia/Kolkata",
	},
	"VI82": {
		ICAO:      "VI82",
		Name:      "Bharkot Airport",
		State:     "Uttarakhand",
		Country:   "IN",
		Elevation: 2983,
		Latitude:  30.5828990936,
		Longitude: 78.3234024048,
		Timezone:  "Asia/Kolkata",
	},
	"VI88": {
		ICAO:      "VI88",
		Name:      "Jallowal Airport",
		State:     "Punjab",
		Country:   "IN",
		Elevation: 741,
		Latitude:  31.5599994659,
		Longitude: 75.3413009644,
		Timezone:  "Asia/Kolkata",
	},
	"VI90": {
		ICAO:      "VI90",
		Name:      "Akbarpur Airport",
		State:     "Uttar-Pradesh",
		Country:   "IN",
		Elevation: 288,
		Latitude:  26.447599411,
		Longitude: 82.5681991577,
		Timezone:  "Asia/Kolkata",
	},
	"VIAG": {
		ICAO:      "VIAG",
		IATA:      "AGR",
		Name:      "Agra Airport",
		State:     "Uttar-Pradesh",
		Country:   "IN",
		Elevation: 551,
		Latitude:  27.1557998657,
		Longitude: 77.960899353,
		Timezone:  "Asia/Kolkata",
	},
	"VIAH": {
		ICAO:      "VIAH",
		Name:      "Dhanipur Airstrip",
		City:      "Aligarth",
		State:     "Uttar-Pradesh",
		Country:   "IN",
		Latitude:  27.860856311,
		Longitude: 78.1468677521,
		Timezone:  "Asia/Kolkata",
	},
	"VIAL": {
		ICAO:      "VIAL",
		IATA:      "IXD",
		Name:      "Allahabad Airport",
		City:      "Allahabad",
		State:     "Uttar-Pradesh",
		Country:   "IN",
		Elevation: 322,
		Latitude:  25.4400997162,
		Longitude: 81.7339019775,
		Timezone:  "Asia/Kolkata",
	},
	"VIAM": {
		ICAO:      "VIAM",
		Name:      "Ambala Air Force Station",
		State:     "Haryana",
		Country:   "IN",
		Elevation: 909,
		Latitude:  30.3680992126,
		Longitude: 76.8167037964,
		Timezone:  "Asia/Kolkata",
	},
	"VIAR": {
		ICAO:      "VIAR",
		IATA:      "ATQ",
		Name:      "Sri Guru Ram Dass Jee International Airport Amritsar",
		City:      "Amritsar",
		State:     "Punjab",
		Country:   "IN",
		Elevation: 756,
		Latitude:  31.7096004486,
		Longitude: 74.7973022461,
		Timezone:  "Asia/Kolkata",
	},
	"VIAW": {
		ICAO:      "VIAW",
		Name:      "Awantipur Air Force Station",
		State:     "Jammu-and-Kashmir",
		Country:   "IN",
		Elevation: 5393,
		Latitude:  33.8765983582,
		Longitude: 74.9757003784,
		Timezone:  "Asia/Kolkata",
	},
	"VIAX": {
		ICAO:      "VIAX",
		IATA:      "AIP",
		Name:      "Adampur Air Force Station",
		State:     "Punjab",
		Country:   "IN",
		Elevation: 775,
		Latitude:  31.4337997437,
		Longitude: 75.7587966919,
		Timezone:  "Asia/Kolkata",
	},
	"VIBK": {
		ICAO:      "VIBK",
		IATA:      "BKB",
		Name:      "Nal Airport",
		City:      "Bikaner",
		State:     "Rajasthan",
		Country:   "IN",
		Elevation: 750,
		Latitude:  28.0706005096,
		Longitude: 73.2071990967,
		Timezone:  "Asia/Kolkata",
	},
	"VIBL": {
		ICAO:      "VIBL",
		Name:      "Bakshi Ka Talab Air Force Station",
		City:      "Lucknow",
		State:     "Uttar-Pradesh",
		Country:   "IN",
		Elevation: 385,
		Latitude:  26.9883003235,
		Longitude: 80.8930969238,
		Timezone:  "Asia/Kolkata",
	},
	"VIBN": {
		ICAO:      "VIBN",
		IATA:      "VNS",
		Name:      "Lal Bahadur Shastri Airport",
		City:      "Varanasi",
		State:     "Uttar-Pradesh",
		Country:   "IN",
		Elevation: 266,
		Latitude:  25.4524002075,
		Longitude: 82.8592987061,
		Timezone:  "Asia/Kolkata",
	},
	"VIBR": {
		ICAO:      "VIBR",
		IATA:      "KUU",
		Name:      "Kullu Manali Airport",
		State:     "Himachal-Pradesh",
		Country:   "IN",
		Elevation: 3573,
		Latitude:  31.8766994476,
		Longitude: 77.1544036865,
		Timezone:  "Asia/Kolkata",
	},
	"VIBT": {
		ICAO:      "VIBT",
		IATA:      "BUP",
		Name:      "Bhatinda Air Force Station",
		State:     "Punjab",
		Country:   "IN",
		Elevation: 662,
		Latitude:  30.2700996399,
		Longitude: 74.7557983398,
		Timezone:  "Asia/Kolkata",
	},
	"VIBW": {
		ICAO:      "VIBW",
		Name:      "Bhiwani Airport",
		State:     "Rajasthan",
		Country:   "IN",
		Elevation: 720,
		Latitude:  28.8369998932,
		Longitude: 76.1791000366,
		Timezone:  "Asia/Kolkata",
	},
	"VIBY": {
		ICAO:      "VIBY",
		IATA:      "BEK",
		Name:      "Bareilly Air Force Station",
		State:     "Uttar-Pradesh",
		Country:   "IN",
		Elevation: 580,
		Latitude:  28.4221000671,
		Longitude: 79.4507980347,
		Timezone:  "Asia/Kolkata",
	},
	"VICG": {
		ICAO:      "VICG",
		IATA:      "IXC",
		Name:      "Chandigarh Airport",
		City:      "Chandigarh",
		State:     "Chandigarh",
		Country:   "IN",
		Elevation: 1012,
		Latitude:  30.673500061,
		Longitude: 76.7884979248,
		Timezone:  "Asia/Kolkata",
	},
	"VICX": {
		ICAO:      "VICX",
		Name:      "Chakeri Air Force Station",
		City:      "Kanpur",
		State:     "Uttar-Pradesh",
		Country:   "IN",
		Elevation: 405,
		Latitude:  26.4043006897,
		Longitude: 80.4101028442,
		Timezone:  "Asia/Kolkata",
	},
	"VIDD": {
		ICAO:      "VIDD",
		Name:      "Safdarjung Airport",
		State:     "NCT",
		Country:   "IN",
		Elevation: 705,
		Latitude:  28.5844993591,
		Longitude: 77.2058029175,
		Timezone:  "Asia/Kolkata",
	},
	"VIDF": {
		ICAO:      "VIDF",
		Name:      "Pithorgarh Airport",
		State:     "Uttarakhand",
		Country:   "IN",
		Elevation: 4910,
		Latitude:  29.5935993195,
		Longitude: 80.2391967773,
		Timezone:  "Asia/Kolkata",
	},
	"VIDN": {
		ICAO:      "VIDN",
		IATA:      "DED",
		Name:      "Dehradun Airport",
		City:      "Dehradun",
		State:     "Uttar-Pradesh",
		Country:   "IN",
		Elevation: 1831,
		Latitude:  30.189699173,
		Longitude: 78.1802978516,
		Timezone:  "Asia/Kolkata",
	},
	"VIDP": {
		ICAO:      "VIDP",
		IATA:      "DEL",
		Name:      "Indira Gandhi International Airport",
		City:      "New Delhi",
		State:     "NCT",
		Country:   "IN",
		Elevation: 777,
		Latitude:  28.5664997101,
		Longitude: 77.1031036377,
		Timezone:  "Asia/Kolkata",
	},
	"VIDX": {
		ICAO:      "VIDX",
		Name:      "Hindon Air Force Station",
		State:     "Rajasthan",
		Country:   "IN",
		Elevation: 700,
		Latitude:  28.7077007294,
		Longitude: 77.3589019775,
		Timezone:  "Asia/Kolkata",
	},
	"VIGG": {
		ICAO:      "VIGG",
		IATA:      "DHM",
		Name:      "Kangra Airport",
		State:     "Himachal-Pradesh",
		Country:   "IN",
		Elevation: 2525,
		Latitude:  32.1651000977,
		Longitude: 76.2633972168,
		Timezone:  "Asia/Kolkata",
	},
	"VIGR": {
		ICAO:      "VIGR",
		IATA:      "GWL",
		Name:      "Gwalior Airport",
		City:      "Gwalior",
		State:     "Madhya-Pradesh",
		Country:   "IN",
		Elevation: 617,
		Latitude:  26.2933006287,
		Longitude: 78.2277984619,
		Timezone:  "Asia/Kolkata",
	},
	"VIHR": {
		ICAO:      "VIHR",
		IATA:      "HSS",
		Name:      "Hissar Airport",
		State:     "Haryana",
		Country:   "IN",
		Elevation: 700,
		Latitude:  29.1793994904,
		Longitude: 75.7553024292,
		Timezone:  "Asia/Kolkata",
	},
	"VIHX": {
		ICAO:      "VIHX",
		Name:      "Halwara Air Force Station",
		State:     "Punjab",
		Country:   "IN",
		Elevation: 790,
		Latitude:  30.748500824,
		Longitude: 75.6297988892,
		Timezone:  "Asia/Kolkata",
	},
	"VIJN": {
		ICAO:      "VIJN",
		Name:      "Jhansi Airport",
		State:     "Uttar-Pradesh",
		Country:   "IN",
		Elevation: 801,
		Latitude:  25.4911994934,
		Longitude: 78.5584030151,
		Timezone:  "Asia/Kolkata",
	},
	"VIJO": {
		ICAO:      "VIJO",
		IATA:      "JDH",
		Name:      "Jodhpur Airport",
		City:      "Jodhpur",
		State:     "Rajasthan",
		Country:   "IN",
		Elevation: 717,
		Latitude:  26.2511005402,
		Longitude: 73.0488967896,
		Timezone:  "Asia/Kolkata",
	},
	"VIJP": {
		ICAO:      "VIJP",
		IATA:      "JAI",
		Name:      "Jaipur International Airport",
		City:      "Jaipur",
		State:     "Rajasthan",
		Country:   "IN",
		Elevation: 1263,
		Latitude:  26.8241996765,
		Longitude: 75.8122024536,
		Timezone:  "Asia/Kolkata",
	},
	"VIJR": {
		ICAO:      "VIJR",
		IATA:      "JSA",
		Name:      "Jaisalmer Airport",
		State:     "Rajasthan",
		Country:   "IN",
		Elevation: 751,
		Latitude:  26.8887004852,
		Longitude: 70.8649978638,
		Timezone:  "Asia/Kolkata",
	},
	"VIJU": {
		ICAO:      "VIJU",
		IATA:      "IXJ",
		Name:      "Jammu Airport",
		City:      "Jammu",
		State:     "Jammu-and-Kashmir",
		Country:   "IN",
		Elevation: 1029,
		Latitude:  32.6890983582,
		Longitude: 74.8374023438,
		Timezone:  "Asia/Kolkata",
	},
	"VIKA": {
		ICAO:      "VIKA",
		IATA:      "KNU",
		Name:      "Kanpur Airport",
		State:     "Uttar-Pradesh",
		Country:   "IN",
		Elevation: 411,
		Latitude:  26.441400528,
		Longitude: 80.3648986816,
		Timezone:  "Asia/Kolkata",
	},
	"VIKG": {
		ICAO:      "VIKG",
		IATA:      "KQH",
		Name:      "Kishangarh Airport, Ajmer",
		City:      "Kishangarh",
		State:     "Rajasthan",
		Country:   "IN",
		Elevation: 1477,
		Latitude:  26.591167,
		Longitude: 74.816167,
		Timezone:  "Asia/Kolkata",
	},
	"VIKO": {
		ICAO:      "VIKO",
		IATA:      "KTU",
		Name:      "Kota Airport",
		State:     "Rajasthan",
		Country:   "IN",
		Elevation: 896,
		Latitude:  25.160200119,
		Longitude: 75.8455963135,
		Timezone:  "Asia/Kolkata",
	},
	"VILD": {
		ICAO:      "VILD",
		IATA:      "LUH",
		Name:      "Ludhiana Airport",
		State:     "Punjab",
		Country:   "IN",
		Elevation: 834,
		Latitude:  30.8547000885,
		Longitude: 75.9525985718,
		Timezone:  "Asia/Kolkata",
	},
	"VILH": {
		ICAO:      "VILH",
		IATA:      "IXL",
		Name:      "Leh Kushok Bakula Rimpochee Airport",
		City:      "Leh",
		State:     "Jammu-and-Kashmir",
		Country:   "IN",
		Elevation: 10682,
		Latitude:  34.1358985901,
		Longitude: 77.5465011597,
		Timezone:  "Asia/Kolkata",
	},
	"VILK": {
		ICAO:      "VILK",
		IATA:      "LKO",
		Name:      "Chaudhary Charan Singh International Airport",
		City:      "Lucknow",
		State:     "Uttar-Pradesh",
		Country:   "IN",
		Elevation: 410,
		Latitude:  26.7605991364,
		Longitude: 80.8892974854,
		Timezone:  "Asia/Kolkata",
	},
	"VILP": {
		ICAO:      "VILP",
		Name:      "Lalitpur Airport",
		City:      "Lalitpur",
		State:     "Uttar-Pradesh",
		Country:   "IN",
		Elevation: 1204,
		Latitude:  24.716999054,
		Longitude: 78.4169998169,
		Timezone:  "Asia/Kolkata",
	},
	"VIPK": {
		ICAO:      "VIPK",
		IATA:      "IXP",
		Name:      "Pathankot Air Force Station",
		State:     "Punjab",
		Country:   "IN",
		Elevation: 1017,
		Latitude:  32.2337989807,
		Longitude: 75.6345977783,
		Timezone:  "Asia/Kolkata",
	},
	"VIPL": {
		ICAO:      "VIPL",
		Name:      "Patiala Airport",
		State:     "Haryana",
		Country:   "IN",
		Elevation: 820,
		Latitude:  30.3148002625,
		Longitude: 76.3645019531,
		Timezone:  "Asia/Kolkata",
	},
	"VIPT": {
		ICAO:      "VIPT",
		IATA:      "PGH",
		Name:      "Pantnagar Airport",
		City:      "Pantnagar",
		State:     "Uttar-Pradesh",
		Country:   "IN",
		Elevation: 769,
		Latitude:  29.0333995819,
		Longitude: 79.4737014771,
		Timezone:  "Asia/Kolkata",
	},
	"VIRB": {
		ICAO:      "VIRB",
		Name:      "Fursatganj Airport",
		State:     "Uttar-Pradesh",
		Country:   "IN",
		Elevation: 360,
		Latitude:  26.248500824,
		Longitude: 81.3805007935,
		Timezone:  "Asia/Kolkata",
	},
	"VISA": {
		ICAO:      "VISA",
		Name:      "Sirsa Air Force Station",
		State:     "Haryana",
		Country:   "IN",
		Elevation: 650,
		Latitude:  29.5606002808,
		Longitude: 75.0061035156,
		Timezone:  "Asia/Kolkata",
	},
	"VISM": {
		ICAO:      "VISM",
		IATA:      "SLV",
		Name:      "Shimla Airport",
		State:     "Himachal-Pradesh",
		Country:   "IN",
		Elevation: 5072,
		Latitude:  31.0818004608,
		Longitude: 77.0680007935,
		Timezone:  "Asia/Kolkata",
	},
	"VISP": {
		ICAO:      "VISP",
		Name:      "Sarsawa Air Force Station",
		State:     "Uttar-Pradesh",
		Country:   "IN",
		Elevation: 891,
		Latitude:  29.9939002991,
		Longitude: 77.4253005981,
		Timezone:  "Asia/Kolkata",
	},
	"VISR": {
		ICAO:      "VISR",
		IATA:      "SXR",
		Name:      "Sheikh ul Alam Airport",
		City:      "Srinagar",
		State:     "Jammu-and-Kashmir",
		Country:   "IN",
		Elevation: 5429,
		Latitude:  33.9870986938,
		Longitude: 74.7742004395,
		Timezone:  "Asia/Kolkata",
	},
	"VIST": {
		ICAO:      "VIST",
		IATA:      "TNI",
		Name:      "Satna Airport",
		State:     "Madhya-Pradesh",
		Country:   "IN",
		Elevation: 1060,
		Latitude:  24.5622997284,
		Longitude: 80.8548965454,
		Timezone:  "Asia/Kolkata",
	},
	"VIUT": {
		ICAO:      "VIUT",
		Name:      "Uttarlai Airport",
		State:     "Rajasthan",
		Country:   "IN",
		Elevation: 500,
		Latitude:  25.8129997253,
		Longitude: 71.4822998047,
		Timezone:  "Asia/Kolkata",
	},
	"VIUX": {
		ICAO:      "VIUX",
		Name:      "Udhampur Air Force Station",
		State:     "Jammu-and-Kashmir",
		Country:   "IN",
		Elevation: 2066,
		Latitude:  32.9021987915,
		Longitude: 75.1561965942,
		Timezone:  "Asia/Kolkata",
	},
	"VLAP": {
		ICAO:      "VLAP",
		IATA:      "AOU",
		Name:      "Attopeu Airport",
		City:      "Attopeu",
		State:     "Attapu",
		Country:   "LA",
		Elevation: 344,
		Latitude:  14.814715357,
		Longitude: 106.821699142,
		Timezone:  "Asia/Vientiane",
	},
	"VLHS": {
		ICAO:      "VLHS",
		IATA:      "OUI",
		Name:      "Ban Huoeisay Airport",
		State:     "Bokeo",
		Country:   "LA",
		Elevation: 1380,
		Latitude:  20.2572994232,
		Longitude: 100.43699646,
		Timezone:  "Asia/Bangkok",
	},
	"VLLB": {
		ICAO:      "VLLB",
		IATA:      "LPQ",
		Name:      "Luang Phabang International Airport",
		City:      "Luang Phabang",
		State:     "Louangphabang",
		Country:   "LA",
		Elevation: 955,
		Latitude:  19.8973007202,
		Longitude: 102.1610031128,
		Timezone:  "Asia/Vientiane",
	},
	"VLLN": {
		ICAO:      "VLLN",
		IATA:      "LXG",
		Name:      "Luang Namtha Airport",
		City:      "Luang Namtha",
		State:     "Loungnamtha",
		Country:   "LA",
		Elevation: 1968,
		Latitude:  20.966999054,
		Longitude: 101.4000015259,
		Timezone:  "Asia/Vientiane",
	},
	"VLOS": {
		ICAO:      "VLOS",
		IATA:      "ODY",
		Name:      "Oudomsay Airport",
		City:      "Oudomsay",
		State:     "Oudomxai",
		Country:   "LA",
		Elevation: 1804,
		Latitude:  20.6826992035,
		Longitude: 101.9940032959,
		Timezone:  "Asia/Vientiane",
	},
	"VLPS": {
		ICAO:      "VLPS",
		IATA:      "PKZ",
		Name:      "Pakse International Airport",
		City:      "Pakse",
		State:     "Champasak",
		Country:   "LA",
		Elevation: 351,
		Latitude:  15.1321001053,
		Longitude: 105.78099823,
		Timezone:  "Asia/Vientiane",
	},
	"VLPV": {
		ICAO:      "VLPV",
		Name:      "Phonesavanh Airport",
		State:     "Xiangkhoang",
		Country:   "LA",
		Elevation: 3628,
		Latitude:  19.4549007416,
		Longitude: 103.2180023193,
		Timezone:  "Asia/Vientiane",
	},
	"VLSB": {
		ICAO:      "VLSB",
		IATA:      "ZBY",
		Name:      "Sayaboury Airport",
		City:      "Sayaboury",
		State:     "Xiagnabouli",
		Country:   "LA",
		Elevation: 1247,
		Latitude:  19.2329998016,
		Longitude: 101.733001709,
		Timezone:  "Asia/Vientiane",
	},
	"VLSK": {
		ICAO:      "VLSK",
		IATA:      "ZVK",
		Name:      "Savannakhet Airport",
		State:     "Savannahkhet",
		Country:   "LA",
		Elevation: 509,
		Latitude:  16.5566005707,
		Longitude: 104.7600021362,
		Timezone:  "Asia/Bangkok",
	},
	"VLSN": {
		ICAO:      "VLSN",
		IATA:      "NEU",
		Name:      "Sam Neua Airport",
		State:     "Houaphan",
		Country:   "LA",
		Elevation: 3281,
		Latitude:  20.4183998108,
		Longitude: 104.0670013428,
		Timezone:  "Asia/Vientiane",
	},
	"VLSV": {
		ICAO:      "VLSV",
		IATA:      "VNA",
		Name:      "Saravane Airport",
		City:      "Saravane",
		State:     "Salavan",
		Country:   "LA",
		Elevation: 574,
		Latitude:  15.7094392077,
		Longitude: 106.410698891,
		Timezone:  "Asia/Vientiane",
	},
	"VLTK": {
		ICAO:      "VLTK",
		IATA:      "THK",
		Name:      "Thakhek Airport",
		City:      "Thakhek",
		State:     "Nakhon-Phanom",
		Country:   "LA",
		Elevation: 449,
		Latitude:  17.3999996185,
		Longitude: 104.8000030518,
		Timezone:  "Asia/Bangkok",
	},
	"VLVT": {
		ICAO:      "VLVT",
		IATA:      "VTE",
		Name:      "Wattay International Airport",
		City:      "Vientiane",
		State:     "Vientiane-Prefecture",
		Country:   "LA",
		Elevation: 564,
		Latitude:  17.9883003235,
		Longitude: 102.56300354,
		Timezone:  "Asia/Vientiane",
	},
	"VLXK": {
		ICAO:      "VLXK",
		IATA:      "XKH",
		Name:      "Xieng Khouang Airport",
		City:      "Xieng Khouang",
		State:     "Xiangkhoang",
		Country:   "LA",
		Elevation: 3445,
		Latitude:  19.4500007629,
		Longitude: 103.1579971313,
		Timezone:  "Asia/Vientiane",
	},
	"VMMC": {
		ICAO:      "VMMC",
		IATA:      "MFM",
		Name:      "Macau International Airport",
		City:      "Taipa",
		Country:   "MO",
		Elevation: 20,
		Latitude:  22.1495990753,
		Longitude: 113.5920028687,
		Timezone:  "Asia/Macau",
	},
	"VNBG": {
		ICAO:      "VNBG",
		IATA:      "BJH",
		Name:      "Bajhang Airport",
		City:      "Bajhang",
		State:     "Far-Western",
		Country:   "NP",
		Elevation: 4100,
		Latitude:  29.5389995575,
		Longitude: 81.1854019165,
		Timezone:  "Asia/Kathmandu",
	},
	"VNBJ": {
		ICAO:      "VNBJ",
		IATA:      "BHP",
		Name:      "Bhojpur Airport",
		City:      "Bhojpur",
		State:     "Eastern-Region",
		Country:   "NP",
		Elevation: 4000,
		Latitude:  27.1473999023,
		Longitude: 87.0507965088,
		Timezone:  "Asia/Kathmandu",
	},
	"VNBL": {
		ICAO:      "VNBL",
		IATA:      "BGL",
		Name:      "Baglung Airport",
		City:      "Baglung",
		State:     "Western-Region",
		Country:   "NP",
		Elevation: 3320,
		Latitude:  28.2127990723,
		Longitude: 83.6662979126,
		Timezone:  "Asia/Kathmandu",
	},
	"VNBP": {
		ICAO:      "VNBP",
		IATA:      "BHR",
		Name:      "Bharatpur Airport",
		City:      "Bharatpur",
		State:     "Central-Region",
		Country:   "NP",
		Elevation: 600,
		Latitude:  27.6781005859,
		Longitude: 84.429397583,
		Timezone:  "Asia/Kathmandu",
	},
	"VNBR": {
		ICAO:      "VNBR",
		IATA:      "BJU",
		Name:      "Bajura Airport",
		City:      "Bajura",
		State:     "Far-Western",
		Country:   "NP",
		Elevation: 4300,
		Latitude:  29.5020008087,
		Longitude: 81.6689987183,
		Timezone:  "Asia/Kathmandu",
	},
	"VNBT": {
		ICAO:      "VNBT",
		IATA:      "BIT",
		Name:      "Baitadi Airport",
		City:      "Baitadi",
		State:     "Far-Western",
		Country:   "NP",
		Elevation: 4200,
		Latitude:  29.4652996063,
		Longitude: 80.5492019653,
		Timezone:  "Asia/Kathmandu",
	},
	"VNBW": {
		ICAO:      "VNBW",
		IATA:      "BWA",
		Name:      "Bhairahawa Airport",
		City:      "Bhairawa",
		State:     "Western-Region",
		Country:   "NP",
		Elevation: 358,
		Latitude:  27.5056991577,
		Longitude: 83.4162979126,
		Timezone:  "Asia/Kathmandu",
	},
	"VNCG": {
		ICAO:      "VNCG",
		IATA:      "BDP",
		Name:      "Bhadrapur Airport",
		City:      "Bhadrapur",
		State:     "Eastern-Region",
		Country:   "NP",
		Elevation: 300,
		Latitude:  26.5708007812,
		Longitude: 88.0795974731,
		Timezone:  "Asia/Kathmandu",
	},
	"VNDG": {
		ICAO:      "VNDG",
		IATA:      "DNP",
		Name:      "Tulsipur Airport",
		City:      "Dang",
		State:     "Mid-Western",
		Country:   "NP",
		Elevation: 2100,
		Latitude:  28.1110992432,
		Longitude: 82.2941970825,
		Timezone:  "Asia/Kathmandu",
	},
	"VNDH": {
		ICAO:      "VNDH",
		IATA:      "DHI",
		Name:      "Dhangarhi Airport",
		City:      "Dhangarhi",
		State:     "Far-Western",
		Country:   "NP",
		Elevation: 690,
		Latitude:  28.7532997131,
		Longitude: 80.5819015503,
		Timezone:  "Asia/Kathmandu",
	},
	"VNDL": {
		ICAO:      "VNDL",
		IATA:      "DAP",
		Name:      "Darchula Airport",
		City:      "Darchula",
		State:     "Far-Western",
		Country:   "NP",
		Elevation: 2132,
		Latitude:  29.6692008972,
		Longitude: 80.5484008789,
		Timezone:  "Asia/Kathmandu",
	},
	"VNDP": {
		ICAO:      "VNDP",
		IATA:      "DOP",
		Name:      "Dolpa Airport",
		City:      "Dolpa",
		State:     "Mid-Western",
		Country:   "NP",
		Elevation: 8200,
		Latitude:  28.9857006073,
		Longitude: 82.8190994263,
		Timezone:  "Asia/Kathmandu",
	},
	"VNDR": {
		ICAO:      "VNDR",
		Name:      "Dhorpatan Airport",
		City:      "Dhorpatan",
		State:     "Western-Region",
		Country:   "NP",
		Elevation: 8950,
		Latitude:  28.5170001984,
		Longitude: 83.0329971313,
		Timezone:  "Asia/Kathmandu",
	},
	"VNDT": {
		ICAO:      "VNDT",
		IATA:      "SIH",
		Name:      "Silgadi Doti Airport",
		City:      "Silgadi Doti",
		State:     "Far-Western",
		Country:   "NP",
		Elevation: 2100,
		Latitude:  29.2630996704,
		Longitude: 80.9359970093,
		Timezone:  "Asia/Kathmandu",
	},
	"VNGK": {
		ICAO:      "VNGK",
		IATA:      "GKH",
		Name:      "Gorkha Airport",
		City:      "Gorkha",
		State:     "Western-Region",
		Country:   "NP",
		Elevation: 1500,
		Latitude:  28.0499992371,
		Longitude: 84.483001709,
		Timezone:  "Asia/Kathmandu",
	},
	"VNJI": {
		ICAO:      "VNJI",
		IATA:      "JIR",
		Name:      "Jiri Airport",
		City:      "Jiri",
		Country:   "NP",
		Elevation: 6000,
		Latitude:  27.6170005798,
		Longitude: 86.2170028687,
		Timezone:  "Asia/Kathmandu",
	},
	"VNJL": {
		ICAO:      "VNJL",
		IATA:      "JUM",
		Name:      "Jumla Airport",
		City:      "Jumla",
		State:     "Mid-Western",
		Country:   "NP",
		Elevation: 7700,
		Latitude:  29.2742004395,
		Longitude: 82.1932983398,
		Timezone:  "Asia/Kathmandu",
	},
	"VNJP": {
		ICAO:      "VNJP",
		IATA:      "JKR",
		Name:      "Janakpur Airport",
		City:      "Janakpur",
		State:     "Central-Region",
		Country:   "NP",
		Elevation: 256,
		Latitude:  26.7087993622,
		Longitude: 85.9224014282,
		Timezone:  "Asia/Kathmandu",
	},
	"VNJS": {
		ICAO:      "VNJS",
		IATA:      "JMO",
		Name:      "Jomsom Airport",
		City:      "Jomsom",
		State:     "Western-Region",
		Country:   "NP",
		Elevation: 9000,
		Latitude:  28.780426,
		Longitude: 83.723,
		Timezone:  "Asia/Kathmandu",
	},
	"VNKL": {
		ICAO:      "VNKL",
		Name:      "Kangel Danda Airport",
		State:     "Eastern-Region",
		Country:   "NP",
		Latitude:  27.4106333137,
		Longitude: 86.6465950012,
		Timezone:  "Asia/Kathmandu",
	},
	"VNKT": {
		ICAO:      "VNKT",
		IATA:      "KTM",
		Name:      "Tribhuvan International Airport",
		City:      "Kathmandu",
		State:     "Central-Region",
		Country:   "NP",
		Elevation: 4390,
		Latitude:  27.6965999603,
		Longitude: 85.3591003418,
		Timezone:  "Asia/Kathmandu",
	},
	"VNLD": {
		ICAO:      "VNLD",
		IATA:      "LDN",
		Name:      "Lamidanda Airport",
		City:      "Lamidanda",
		State:     "Eastern-Region",
		Country:   "NP",
		Elevation: 4100,
		Latitude:  27.2530994415,
		Longitude: 86.6699981689,
		Timezone:  "Asia/Kathmandu",
	},
	"VNLK": {
		ICAO:      "VNLK",
		IATA:      "LUA",
		Name:      "Lukla Airport",
		City:      "Lukla",
		State:     "Eastern-Region",
		Country:   "NP",
		Elevation: 9380,
		Latitude:  27.6868991852,
		Longitude: 86.7296981812,
		Timezone:  "Asia/Kathmandu",
	},
	"VNLT": {
		ICAO:      "VNLT",
		IATA:      "LTG",
		Name:      "Langtang Airport",
		City:      "Langtang",
		State:     "Central-Region",
		Country:   "NP",
		Elevation: 12000,
		Latitude:  28.2000007629,
		Longitude: 85.5830001831,
		Timezone:  "Asia/Kathmandu",
	},
	"VNMA": {
		ICAO:      "VNMA",
		IATA:      "NGX",
		Name:      "Manang Airport",
		City:      "Ngawal",
		State:     "Western-Region",
		Country:   "NP",
		Elevation: 11001,
		Latitude:  28.6413993835,
		Longitude: 84.0892028809,
		Timezone:  "Asia/Kathmandu",
	},
	"VNMG": {
		ICAO:      "VNMG",
		IATA:      "MEY",
		Name:      "Meghauli Airport",
		City:      "Meghauli",
		State:     "Central-Region",
		Country:   "NP",
		Elevation: 600,
		Latitude:  27.5774,
		Longitude: 84.22875,
		Timezone:  "Asia/Kathmandu",
	},
	"VNMN": {
		ICAO:      "VNMN",
		IATA:      "XMG",
		Name:      "Mahendranagar Airport",
		City:      "Mahendranagar",
		State:     "Far-Western",
		Country:   "NP",
		Elevation: 650,
		Latitude:  28.9631996155,
		Longitude: 80.1479034424,
		Timezone:  "Asia/Kathmandu",
	},
	"VNNG": {
		ICAO:      "VNNG",
		IATA:      "KEP",
		Name:      "Nepalgunj Airport",
		City:      "Nepalgunj",
		State:     "Mid-Western",
		Country:   "NP",
		Elevation: 540,
		Latitude:  28.1035995483,
		Longitude: 81.6669998169,
		Timezone:  "Asia/Kathmandu",
	},
	"VNPK": {
		ICAO:      "VNPK",
		IATA:      "PKR",
		Name:      "Pokhara Airport",
		City:      "Pokhara",
		State:     "Western-Region",
		Country:   "NP",
		Elevation: 2712,
		Latitude:  28.2008991241,
		Longitude: 83.9821014404,
		Timezone:  "Asia/Kathmandu",
	},
	"VNPL": {
		ICAO:      "VNPL",
		IATA:      "PPL",
		Name:      "Phaplu Airport",
		City:      "Phaplu",
		State:     "Eastern-Region",
		Country:   "NP",
		Elevation: 7918,
		Latitude:  27.5177868809,
		Longitude: 86.5844535828,
		Timezone:  "Asia/Kathmandu",
	},
	"VNRB": {
		ICAO:      "VNRB",
		IATA:      "RJB",
		Name:      "Rajbiraj Airport",
		City:      "Rajbiraj",
		State:     "Eastern-Region",
		Country:   "NP",
		Elevation: 250,
		Latitude:  26.5170001984,
		Longitude: 86.75,
		Timezone:  "Asia/Kathmandu",
	},
	"VNRC": {
		ICAO:      "VNRC",
		IATA:      "RHP",
		Name:      "Ramechhap Airport",
		City:      "Ramechhap",
		State:     "Central-Region",
		Country:   "NP",
		Elevation: 1555,
		Latitude:  27.3939990997,
		Longitude: 86.0614013672,
		Timezone:  "Asia/Kathmandu",
	},
	"VNRK": {
		ICAO:      "VNRK",
		IATA:      "RUK",
		Name:      "Rukumkot Airport",
		City:      "Rukumkot",
		State:     "Mid-Western",
		Country:   "NP",
		Elevation: 2500,
		Latitude:  28.6270008087,
		Longitude: 82.1949996948,
		Timezone:  "Asia/Kathmandu",
	},
	"VNRP": {
		ICAO:      "VNRP",
		IATA:      "RPA",
		Name:      "Rolpa Airport",
		City:      "Rolpa",
		State:     "Mid-Western",
		Country:   "NP",
		Elevation: 4100,
		Latitude:  28.2670001984,
		Longitude: 82.766998291,
		Timezone:  "Asia/Kathmandu",
	},
	"VNRT": {
		ICAO:      "VNRT",
		IATA:      "RUM",
		Name:      "Rumjatar Airport",
		City:      "Rumjatar",
		State:     "Eastern-Region",
		Country:   "NP",
		Elevation: 4500,
		Latitude:  27.3034992218,
		Longitude: 86.5503997803,
		Timezone:  "Asia/Kathmandu",
	},
	"VNSI": {
		ICAO:      "VNSI",
		IATA:      "SIF",
		Name:      "Simara Airport",
		City:      "Simara",
		State:     "Central-Region",
		Country:   "NP",
		Elevation: 450,
		Latitude:  27.1595001221,
		Longitude: 84.9801025391,
		Timezone:  "Asia/Kathmandu",
	},
	"VNSK": {
		ICAO:      "VNSK",
		IATA:      "SKH",
		Name:      "Surkhet Airport",
		City:      "Surkhet",
		State:     "Mid-Western",
		Country:   "NP",
		Elevation: 2400,
		Latitude:  28.5860004425,
		Longitude: 81.6360015869,
		Timezone:  "Asia/Kathmandu",
	},
	"VNSR": {
		ICAO:      "VNSR",
		IATA:      "FEB",
		Name:      "Sanfebagar Airport",
		City:      "Sanfebagar",
		State:     "Far-Western",
		Country:   "NP",
		Elevation: 2280,
		Latitude:  29.2329998016,
		Longitude: 81.2170028687,
		Timezone:  "Asia/Kathmandu",
	},
	"VNST": {
		ICAO:      "VNST",
		IATA:      "IMK",
		Name:      "Simikot Airport",
		City:      "Simikot",
		State:     "Mid-Western",
		Country:   "NP",
		Elevation: 9246,
		Latitude:  29.9710998535,
		Longitude: 81.818901062,
		Timezone:  "Asia/Kathmandu",
	},
	"VNTH": {
		ICAO:      "VNTH",
		Name:      "Thamkharka Airport",
		City:      "Khotang Bazar",
		State:     "Eastern-Region",
		Country:   "NP",
		Elevation: 5240,
		Latitude:  27.04787,
		Longitude: 86.858022,
		Timezone:  "Asia/Kathmandu",
	},
	"VNTJ": {
		ICAO:      "VNTJ",
		IATA:      "TPJ",
		Name:      "Suketar Airport",
		City:      "Taplejung",
		State:     "Eastern-Region",
		Country:   "NP",
		Elevation: 7990,
		Latitude:  27.3509,
		Longitude: 87.69525,
		Timezone:  "Asia/Kathmandu",
	},
	"VNTP": {
		ICAO:      "VNTP",
		IATA:      "TPU",
		Name:      "Tikapur Airport",
		City:      "Tikapur",
		State:     "Far-Western",
		Country:   "NP",
		Elevation: 797,
		Latitude:  28.5170001984,
		Longitude: 81.1500015259,
		Timezone:  "Asia/Kathmandu",
	},
	"VNTR": {
		ICAO:      "VNTR",
		IATA:      "TMI",
		Name:      "Tumling Tar Airport",
		City:      "Tumling Tar",
		State:     "Eastern-Region",
		Country:   "NP",
		Elevation: 1700,
		Latitude:  27.3150005341,
		Longitude: 87.1932983398,
		Timezone:  "Asia/Kathmandu",
	},
	"VNVT": {
		ICAO:      "VNVT",
		IATA:      "BIR",
		Name:      "Biratnagar Airport",
		City:      "Biratnagar",
		State:     "Eastern-Region",
		Country:   "NP",
		Elevation: 236,
		Latitude:  26.4815006256,
		Longitude: 87.263999939,
		Timezone:  "Asia/Kathmandu",
	},
	"VO26": {
		ICAO:      "VO26",
		Name:      "Kovilpatti Airport",
		State:     "Mizoram",
		Country:   "IN",
		Elevation: 325,
		Latitude:  9.1538896561,
		Longitude: 77.8211975098,
		Timezone:  "Asia/Kolkata",
	},
	"VO52": {
		ICAO:      "VO52",
		Name:      "Harihar Airport",
		State:     "Karnataka",
		Country:   "IN",
		Elevation: 1750,
		Latitude:  14.5350999832,
		Longitude: 75.787399292,
		Timezone:  "Asia/Kolkata",
	},
	"VO94": {
		ICAO:      "VO94",
		Name:      "Campbell Bay Airport",
		State:     "Andaman-and-Nicobar",
		Country:   "IN",
		Elevation: 16,
		Latitude:  7.0130400658,
		Longitude: 93.9227981567,
		Timezone:  "Asia/Kolkata",
	},
	"VO95": {
		ICAO:      "VO95",
		Name:      "Hosur Airport",
		State:     "Tamil-Nadu",
		Country:   "IN",
		Elevation: 3116,
		Latitude:  12.6612997055,
		Longitude: 77.7671966553,
		Timezone:  "Asia/Kolkata",
	},
	"VOAR": {
		ICAO:      "VOAR",
		Name:      "Arkonam Airport",
		State:     "Tamil-Nadu",
		Country:   "IN",
		Elevation: 265,
		Latitude:  13.0712003708,
		Longitude: 79.6912002563,
		Timezone:  "Asia/Kolkata",
	},
	"VOAT": {
		ICAO:      "VOAT",
		IATA:      "AGX",
		Name:      "Agatti Airport",
		State:     "Laccadives",
		Country:   "IN",
		Elevation: 14,
		Latitude:  10.8236999512,
		Longitude: 72.1760025024,
		Timezone:  "Asia/Kolkata",
	},
	"VOBG": {
		ICAO:      "VOBG",
		Name:      "HAL Airport",
		City:      "Bangalore",
		State:     "Karnataka",
		Country:   "IN",
		Elevation: 2912,
		Latitude:  12.9499998093,
		Longitude: 77.6681976318,
		Timezone:  "Asia/Kolkata",
	},
	"VOBI": {
		ICAO:      "VOBI",
		IATA:      "BEP",
		Name:      "Bellary Airport",
		City:      "Bellary",
		State:     "Karnataka",
		Country:   "IN",
		Elevation: 30,
		Latitude:  15.1627998352,
		Longitude: 76.8827972412,
		Timezone:  "Asia/Kolkata",
	},
	"VOBL": {
		ICAO:      "VOBL",
		IATA:      "BLR",
		Name:      "Bengaluru International Airport",
		City:      "Bangalore",
		State:     "Karnataka",
		Country:   "IN",
		Elevation: 3000,
		Latitude:  13.1978998184,
		Longitude: 77.7062988281,
		Timezone:  "Asia/Kolkata",
	},
	"VOBR": {
		ICAO:      "VOBR",
		Name:      "Bidar Air Force Station",
		State:     "Karnataka",
		Country:   "IN",
		Elevation: 2178,
		Latitude:  17.9081001282,
		Longitude: 77.4870986938,
		Timezone:  "Asia/Kolkata",
	},
	"VOBZ": {
		ICAO:      "VOBZ",
		IATA:      "VGA",
		Name:      "Vijayawada Airport",
		State:     "Andhra-Pradesh",
		Country:   "IN",
		Elevation: 82,
		Latitude:  16.5303993225,
		Longitude: 80.7967987061,
		Timezone:  "Asia/Kolkata",
	},
	"VOCB": {
		ICAO:      "VOCB",
		IATA:      "CJB",
		Name:      "Coimbatore International Airport",
		City:      "Coimbatore",
		State:     "Tamil-Nadu",
		Country:   "IN",
		Elevation: 1324,
		Latitude:  11.029999733,
		Longitude: 77.0434036255,
		Timezone:  "Asia/Kolkata",
	},
	"VOCC": {
		ICAO:      "VOCC",
		Name:      "Willingdon Island Air Base",
		City:      "Kochi",
		State:     "Kerala",
		Country:   "IN",
		Elevation: 8,
		Latitude:  9.9473896027,
		Longitude: 76.2731018066,
		Timezone:  "Asia/Kolkata",
	},
	"VOCI": {
		ICAO:      "VOCI",
		IATA:      "COK",
		Name:      "Cochin International Airport",
		City:      "Cochin",
		State:     "Kerala",
		Country:   "IN",
		Elevation: 30,
		Latitude:  10.1520004272,
		Longitude: 76.4019012451,
		Timezone:  "Asia/Kolkata",
	},
	"VOCL": {
		ICAO:      "VOCL",
		IATA:      "CCJ",
		Name:      "Calicut International Airport",
		City:      "Calicut",
		State:     "Kerala",
		Country:   "IN",
		Elevation: 342,
		Latitude:  11.1367998123,
		Longitude: 75.9552993774,
		Timezone:  "Asia/Kolkata",
	},
	"VOCP": {
		ICAO:      "VOCP",
		IATA:      "CDP",
		Name:      "Cuddapah Airport",
		State:     "Andhra-Pradesh",
		Country:   "IN",
		Elevation: 430,
		Latitude:  14.5100002289,
		Longitude: 78.7727966309,
		Timezone:  "Asia/Kolkata",
	},
	"VOCX": {
		ICAO:      "VOCX",
		IATA:      "CBD",
		Name:      "Car Nicobar Air Force Station",
		State:     "Andaman-and-Nicobar",
		Country:   "IN",
		Elevation: 5,
		Latitude:  9.1525096893,
		Longitude: 92.8196029663,
		Timezone:  "Asia/Kolkata",
	},
	"VODG": {
		ICAO:      "VODG",
		Name:      "Dundigul Air Force Academy",
		City:      "Hyderabad",
		State:     "Telangana",
		Country:   "IN",
		Elevation: 2013,
		Latitude:  17.627199173,
		Longitude: 78.4033966064,
		Timezone:  "Asia/Kolkata",
	},
	"VODK": {
		ICAO:      "VODK",
		Name:      "Dunakonda Airport",
		City:      "Dunakonda",
		State:     "Andhra-Pradesh",
		Country:   "IN",
		Elevation: 462,
		Latitude:  15.8330001831,
		Longitude: 79.5,
		Timezone:  "Asia/Kolkata",
	},
	"VOGO": {
		ICAO:      "VOGO",
		IATA:      "GOI",
		Name:      "Dabolim Airport",
		City:      "Dabolim",
		State:     "Dabolim",
		Country:   "IN",
		Elevation: 186,
		Latitude:  15.380833,
		Longitude: 73.827,
		Timezone:  "Asia/Kolkata",
	},
	"VOHK": {
		ICAO:      "VOHK",
		Name:      "Hakimpet Airport",
		State:     "Telangana",
		Country:   "IN",
		Elevation: 2020,
		Latitude:  17.5534992218,
		Longitude: 78.5249023438,
		Timezone:  "Asia/Kolkata",
	},
	"VOHS": {
		ICAO:      "VOHS",
		IATA:      "HYD",
		Name:      "Rajiv Gandhi International Airport Shamshabad",
		City:      "Hyderabad",
		State:     "Telangana",
		Country:   "IN",
		Elevation: 2024,
		Latitude:  17.2313175201,
		Longitude: 78.4298553467,
		Timezone:  "Asia/Kolkata",
	},
	"VOHY": {
		ICAO:      "VOHY",
		IATA:      "BPM",
		Name:      "Begumpet Airport",
		City:      "Hyderabad",
		State:     "Telangana",
		Country:   "IN",
		Elevation: 1742,
		Latitude:  17.4531002045,
		Longitude: 78.4675979614,
		Timezone:  "Asia/Kolkata",
	},
	"VOJV": {
		ICAO:      "VOJV",
		IATA:      "VDY",
		Name:      "Vijayanagar Aerodrome (JSW)",
		State:     "Karnataka",
		Country:   "IN",
		Elevation: 1670,
		Latitude:  15.1749666667,
		Longitude: 76.6349472222,
		Timezone:  "Asia/Kolkata",
	},
	"VOKP": {
		ICAO:      "VOKP",
		Name:      "Koppal Aerodrome",
		City:      "Koppal",
		State:     "Karnataka",
		Country:   "IN",
		Elevation: 1700,
		Latitude:  15.3591003418,
		Longitude: 76.2192993164,
		Timezone:  "Asia/Kolkata",
	},
	"VOMD": {
		ICAO:      "VOMD",
		IATA:      "IXM",
		Name:      "Madurai Airport",
		City:      "Madurai",
		State:     "Tamil-Nadu",
		Country:   "IN",
		Elevation: 459,
		Latitude:  9.8345098496,
		Longitude: 78.0933990479,
		Timezone:  "Asia/Kolkata",
	},
	"VOML": {
		ICAO:      "VOML",
		IATA:      "IXE",
		Name:      "Mangalore International Airport",
		City:      "Mangalore",
		State:     "Karnataka",
		Country:   "IN",
		Elevation: 337,
		Latitude:  12.9612998962,
		Longitude: 74.8900985718,
		Timezone:  "Asia/Kolkata",
	},
	"VOMM": {
		ICAO:      "VOMM",
		IATA:      "MAA",
		Name:      "Chennai International Airport",
		City:      "Chennai",
		State:     "Tamil-Nadu",
		Country:   "IN",
		Elevation: 52,
		Latitude:  12.9900054932,
		Longitude: 80.1692962646,
		Timezone:  "Asia/Kolkata",
	},
	"VOMY": {
		ICAO:      "VOMY",
		IATA:      "MYQ",
		Name:      "Mysore Airport",
		City:      "Mysore",
		State:     "Karnataka",
		Country:   "IN",
		Elevation: 2349,
		Latitude:  12.3072004318,
		Longitude: 76.6496963501,
		Timezone:  "Asia/Kolkata",
	},
	"VONS": {
		ICAO:      "VONS",
		Name:      "Nagarjuna Sagar Airport",
		State:     "Andhra-Pradesh",
		Country:   "IN",
		Elevation: 658,
		Latitude:  16.5426998138,
		Longitude: 79.3187026978,
		Timezone:  "Asia/Kolkata",
	},
	"VOPB": {
		ICAO:      "VOPB",
		IATA:      "IXZ",
		Name:      "Vir Savarkar International Airport",
		City:      "Port Blair",
		State:     "Andaman-and-Nicobar",
		Country:   "IN",
		Elevation: 14,
		Latitude:  11.6412000656,
		Longitude: 92.7296981812,
		Timezone:  "Asia/Kolkata",
	},
	"VOPC": {
		ICAO:      "VOPC",
		IATA:      "PNY",
		Name:      "Pondicherry Airport",
		State:     "Tamil-Nadu",
		Country:   "IN",
		Elevation: 118,
		Latitude:  11.9687004089,
		Longitude: 79.8100967407,
		Timezone:  "Asia/Kolkata",
	},
	"VORM": {
		ICAO:      "VORM",
		Name:      "Ramnad Naval Air Station",
		City:      "Ramnad",
		Country:   "IN",
		Elevation: 16,
		Latitude:  9.3250904083,
		Longitude: 78.971496582,
		Timezone:  "Asia/Kolkata",
	},
	"VORY": {
		ICAO:      "VORY",
		IATA:      "RJA",
		Name:      "Rajahmundry Airport",
		City:      "Rajahmundry",
		State:     "Andhra-Pradesh",
		Country:   "IN",
		Elevation: 151,
		Latitude:  17.1103992462,
		Longitude: 81.8181991577,
		Timezone:  "Asia/Kolkata",
	},
	"VOSM": {
		ICAO:      "VOSM",
		IATA:      "SXV",
		Name:      "Salem Airport",
		State:     "Tamil-Nadu",
		Country:   "IN",
		Elevation: 1008,
		Latitude:  11.7833003998,
		Longitude: 78.0655975342,
		Timezone:  "Asia/Kolkata",
	},
	"VOSX": {
		ICAO:      "VOSX",
		Name:      "Coimbatore Air Force Station",
		City:      "Sulur",
		State:     "Tamil-Nadu",
		Country:   "IN",
		Elevation: 1250,
		Latitude:  11.0136003494,
		Longitude: 77.1596984863,
		Timezone:  "Asia/Kolkata",
	},
	"VOTJ": {
		ICAO:      "VOTJ",
		IATA:      "TJV",
		Name:      "Tanjore Air Force Base",
		City:      "Thanjavur",
		State:     "Tamil-Nadu",
		Country:   "IN",
		Elevation: 253,
		Latitude:  10.7223997116,
		Longitude: 79.101600647,
		Timezone:  "Asia/Kolkata",
	},
	"VOTK": {
		ICAO:      "VOTK",
		IATA:      "TCR",
		Name:      "Tuticorin Airport",
		City:      "Thoothukkudi",
		State:     "Tamil-Nadu",
		Country:   "IN",
		Elevation: 129,
		Latitude:  8.724167,
		Longitude: 78.025833,
		Timezone:  "Asia/Kolkata",
	},
	"VOTP": {
		ICAO:      "VOTP",
		IATA:      "TIR",
		Name:      "Tirupati Airport",
		City:      "Tirupati",
		State:     "Andhra-Pradesh",
		Country:   "IN",
		Elevation: 350,
		Latitude:  13.6324996948,
		Longitude: 79.543296814,
		Timezone:  "Asia/Kolkata",
	},
	"VOTR": {
		ICAO:      "VOTR",
		IATA:      "TRZ",
		Name:      "Tiruchirapally Civil Airport Airport",
		City:      "Tiruchirappally",
		State:     "Tamil-Nadu",
		Country:   "IN",
		Elevation: 288,
		Latitude:  10.7653999329,
		Longitude: 78.7097015381,
		Timezone:  "Asia/Kolkata",
	},
	"VOTV": {
		ICAO:      "VOTV",
		IATA:      "TRV",
		Name:      "Trivandrum International Airport",
		City:      "Trivandrum",
		State:     "Kerala",
		Country:   "IN",
		Elevation: 15,
		Latitude:  8.4821195602,
		Longitude: 76.9200973511,
		Timezone:  "Asia/Kolkata",
	},
	"VOTX": {
		ICAO:      "VOTX",
		Name:      "Tambaram Air Force Station",
		City:      "Chennai",
		State:     "Tamil-Nadu",
		Country:   "IN",
		Elevation: 90,
		Latitude:  12.9071998596,
		Longitude: 80.1219024658,
		Timezone:  "Asia/Kolkata",
	},
	"VOVR": {
		ICAO:      "VOVR",
		Name:      "Vellore Airport",
		City:      "Vellore",
		State:     "Tamil-Nadu",
		Country:   "IN",
		Elevation: 764,
		Latitude:  12.9088001251,
		Longitude: 79.0668029785,
		Timezone:  "Asia/Kolkata",
	},
	"VOWA": {
		ICAO:      "VOWA",
		IATA:      "WGC",
		Name:      "Warangal Airport",
		City:      "Warrangal",
		State:     "Telangana",
		Country:   "IN",
		Elevation: 935,
		Latitude:  17.9144001007,
		Longitude: 79.6022033691,
		Timezone:  "Asia/Kolkata",
	},
	"VOYK": {
		ICAO:      "VOYK",
		Name:      "Yelahanka Air Force Station",
		City:      "Yelahanka",
		State:     "Karnataka",
		Country:   "IN",
		Elevation: 3045,
		Latitude:  13.1354999542,
		Longitude: 77.6060028076,
		Timezone:  "Asia/Kolkata",
	},
	"VQ10": {
		ICAO:      "VQ10",
		Name:      "Yongphulla Airport",
		State:     "Tashigang",
		Country:   "BT",
		Elevation: 9000,
		Latitude:  27.2563991547,
		Longitude: 91.514503479,
		Timezone:  "Asia/Thimphu",
	},
	"VQPR": {
		ICAO:      "VQPR",
		IATA:      "PBH",
		Name:      "Paro Airport",
		City:      "Paro",
		State:     "Paro",
		Country:   "BT",
		Elevation: 7332,
		Latitude:  27.4032001495,
		Longitude: 89.4245986938,
		Timezone:  "Asia/Thimphu",
	},
	"VRBK": {
		ICAO:      "VRBK",
		IATA:      "HDK",
		Name:      "Kulhudhuffushi Airport",
		City:      "Kulhudhuffushi",
		State:     "Haa Dhaalu Atoll",
		Country:   "MV",
		Elevation: 3,
		Latitude:  6.631111,
		Longitude: 73.066667,
		Timezone:  "Indian/Maldives",
	},
	"VREI": {
		ICAO:      "VREI",
		IATA:      "IFU",
		Name:      "Ifuru Airport",
		City:      "Ifuru",
		State:     "Ifuru",
		Country:   "MV",
		Elevation: 6,
		Latitude:  5.7083,
		Longitude: 73.025,
		Timezone:  "Indian/Maldives",
	},
	"VRMF": {
		ICAO:      "VRMF",
		IATA:      "FVM",
		Name:      "Fuvahmulah Airport",
		City:      "Fuvahmulah Island",
		State:     "Gnyaviyani-Atoll",
		Country:   "MV",
		Elevation: 17,
		Latitude:  -0.3097222222,
		Longitude: 73.435,
		Timezone:  "Indian/Maldives",
	},
	"VRMG": {
		ICAO:      "VRMG",
		IATA:      "GAN",
		Name:      "Gan International Airport",
		City:      "Gan",
		State:     "Seenu",
		Country:   "MV",
		Elevation: 6,
		Latitude:  -0.6933419704,
		Longitude: 73.1556015015,
		Timezone:  "Indian/Maldives",
	},
	"VRMH": {
		ICAO:      "VRMH",
		IATA:      "HAQ",
		Name:      "Hanimaadhoo Airport",
		City:      "Haa Dhaalu Atoll",
		State:     "Haa-Dhaalu-Atholhu",
		Country:   "MV",
		Elevation: 4,
		Latitude:  6.7442297935,
		Longitude: 73.170501709,
		Timezone:  "Indian/Maldives",
	},
	"VRMK": {
		ICAO:      "VRMK",
		IATA:      "KDO",
		Name:      "Kadhdhoo Airport",
		City:      "Kadhdhoo",
		State:     "Laamu",
		Country:   "MV",
		Elevation: 4,
		Latitude:  1.85916996,
		Longitude: 73.5218963623,
		Timezone:  "Indian/Maldives",
	},
	"VRMM": {
		ICAO:      "VRMM",
		IATA:      "MLE",
		Name:      "Male International Airport",
		City:      "Male",
		State:     "Kaafu-Atoll",
		Country:   "MV",
		Elevation: 6,
		Latitude:  4.1918301582,
		Longitude: 73.5290985107,
		Timezone:  "Indian/Maldives",
	},
	"VRMT": {
		ICAO:      "VRMT",
		IATA:      "KDM",
		Name:      "Kaadedhdhoo Airport",
		City:      "Huvadhu Atoll",
		State:     "Gaafu-Dhaalu-Atholhu",
		Country:   "MV",
		Elevation: 2,
		Latitude:  0.4881309867,
		Longitude: 72.9969024658,
		Timezone:  "Indian/Maldives",
	},
	"VRMV": {
		ICAO:      "VRMV",
		IATA:      "VAM",
		Name:      "Villa Airport",
		City:      "Maamigili",
		Country:   "MV",
		Elevation: 6,
		Latitude:  3.4705555556,
		Longitude: 72.8358333333,
		Timezone:  "Indian/Maldives",
	},
	"VT01": {
		ICAO:      "VT01",
		Name:      "Teal Farm Airport",
		City:      "Huntington",
		State:     "Vermont",
		Country:   "US",
		Elevation: 1440,
		Latitude:  44.2826004028,
		Longitude: -72.9373016357,
		Timezone:  "America/New_York",
	},
	"VT02": {
		ICAO:      "VT02",
		Name:      "Red Fox Airport",
		City:      "Bondville",
		State:     "Vermont",
		Country:   "US",
		Elevation: 1434,
		Latitude:  43.1473007202,
		Longitude: -72.8851013184,
		Timezone:  "America/New_York",
	},
	"VT03": {
		ICAO:      "VT03",
		Name:      "Maule's Roost Airport",
		City:      "Lincoln",
		State:     "Vermont",
		Country:   "US",
		Elevation: 1100,
		Latitude:  44.0886993408,
		Longitude: -72.9809036255,
		Timezone:  "America/New_York",
	},
	"VT06": {
		ICAO:      "VT06",
		Name:      "Santa's Airport",
		City:      "Putney",
		State:     "Vermont",
		Country:   "US",
		Elevation: 673,
		Latitude:  43.0043983459,
		Longitude: -72.4737014771,
		Timezone:  "America/New_York",
	},
	"VT09": {
		ICAO:      "VT09",
		Name:      "Spencer Airport",
		City:      "Addison",
		State:     "Vermont",
		Country:   "US",
		Elevation: 310,
		Latitude:  44.1136016846,
		Longitude: -73.2902984619,
		Timezone:  "America/New_York",
	},
	"VT10": {
		ICAO:      "VT10",
		Name:      "Manning Personal Airstrip",
		City:      "Bridport",
		State:     "Vermont",
		Country:   "US",
		Elevation: 255,
		Latitude:  43.9813995361,
		Longitude: -73.3856964111,
		Timezone:  "America/New_York",
	},
	"VT11": {
		ICAO:      "VT11",
		Name:      "Ass-Pirin Acres Airport",
		City:      "Addison",
		State:     "Vermont",
		Country:   "US",
		Elevation: 130,
		Latitude:  44.0686988831,
		Longitude: -73.3733978271,
		Timezone:  "America/New_York",
	},
	"VT12": {
		ICAO:      "VT12",
		Name:      "E.A.Deeds Farm Airport",
		City:      "Charlotte",
		State:     "Vermont",
		Country:   "US",
		Elevation: 127,
		Latitude:  44.2911987305,
		Longitude: -73.283203125,
		Timezone:  "America/New_York",
	},
	"VT13": {
		ICAO:      "VT13",
		Name:      "Holloway Airport",
		City:      "Chester",
		State:     "Vermont",
		Country:   "US",
		Elevation: 1520,
		Latitude:  43.3000984192,
		Longitude: -72.7175979614,
		Timezone:  "America/New_York",
	},
	"VT14": {
		ICAO:      "VT14",
		Name:      "Axinn Airport",
		City:      "Weybridge",
		State:     "Vermont",
		Country:   "US",
		Elevation: 450,
		Latitude:  44.0451011658,
		Longitude: -73.1895980835,
		Timezone:  "America/New_York",
	},
	"VT15": {
		ICAO:      "VT15",
		Name:      "Savage Island Airport",
		City:      "Grand Isle",
		State:     "Vermont",
		Country:   "US",
		Elevation: 125,
		Latitude:  44.7038993835,
		Longitude: -73.2460021973,
		Timezone:  "America/New_York",
	},
	"VT17": {
		ICAO:      "VT17",
		Name:      "Smith Airport",
		City:      "Ludlow",
		State:     "Vermont",
		Country:   "US",
		Elevation: 1502,
		Latitude:  43.3689994812,
		Longitude: -72.6743011475,
		Timezone:  "America/New_York",
	},
	"VT20": {
		ICAO:      "VT20",
		Name:      "Mach Personal Strip",
		City:      "Pawlet",
		State:     "Vermont",
		Country:   "US",
		Elevation: 854,
		Latitude:  43.3666992188,
		Longitude: -73.1482009888,
		Timezone:  "America/New_York",
	},
	"VT22": {
		ICAO:      "VT22",
		Name:      "Shelburne Farms Airport",
		City:      "Shelburne",
		State:     "Vermont",
		Country:   "US",
		Elevation: 130,
		Latitude:  44.4014015198,
		Longitude: -73.2670974731,
		Timezone:  "America/New_York",
	},
	"VT23": {
		ICAO:      "VT23",
		Name:      "Bostwick Farm Airport",
		City:      "Shelburne",
		State:     "Vermont",
		Country:   "US",
		Elevation: 220,
		Latitude:  44.3708992004,
		Longitude: -73.2503967285,
		Timezone:  "America/New_York",
	},
	"VT24": {
		ICAO:      "VT24",
		Name:      "Torrey Airport",
		City:      "Shoreham",
		State:     "Vermont",
		Country:   "US",
		Elevation: 280,
		Latitude:  43.9319992065,
		Longitude: -73.3701019287,
		Timezone:  "America/New_York",
	},
	"VT25": {
		ICAO:      "VT25",
		Name:      "Sky Acres Airport",
		City:      "South Burlington",
		State:     "Vermont",
		Country:   "US",
		Elevation: 330,
		Latitude:  44.4166984558,
		Longitude: -73.1661987305,
		Timezone:  "America/New_York",
	},
	"VT26": {
		ICAO:      "VT26",
		Name:      "Allenholm Airport",
		City:      "South Hero",
		State:     "Vermont",
		Country:   "US",
		Elevation: 140,
		Latitude:  44.6344985962,
		Longitude: -73.3068008423,
		Timezone:  "America/New_York",
	},
	"VT29": {
		ICAO:      "VT29",
		Name:      "Mad River Fly-In Airport",
		City:      "Waitsfield",
		State:     "Vermont",
		Country:   "US",
		Elevation: 710,
		Latitude:  44.226398468,
		Longitude: -72.7954025269,
		Timezone:  "America/New_York",
	},
	"VT30": {
		ICAO:      "VT30",
		Name:      "Ketcham Lndg Area Airport",
		City:      "Waitsfield",
		State:     "Vermont",
		Country:   "US",
		Elevation: 780,
		Latitude:  44.1626014709,
		Longitude: -72.8339996338,
		Timezone:  "America/New_York",
	},
	"VT31": {
		ICAO:      "VT31",
		Name:      "Carriers Skypark Airport",
		City:      "Washington",
		State:     "Vermont",
		Country:   "US",
		Elevation: 1600,
		Latitude:  44.1111984253,
		Longitude: -72.4440002441,
		Timezone:  "America/New_York",
	},
	"VT33": {
		ICAO:      "VT33",
		Name:      "Yankee Kingdom Airport",
		City:      "West Addison",
		State:     "Vermont",
		Country:   "US",
		Elevation: 200,
		Latitude:  44.1055984497,
		Longitude: -73.3778991699,
		Timezone:  "America/New_York",
	},
	"VT34": {
		ICAO:      "VT34",
		Name:      "Major W. Guth STOLport",
		City:      "Westminster",
		State:     "Vermont",
		Country:   "US",
		Elevation: 819,
		Latitude:  43.0601005554,
		Longitude: -72.4919967651,
		Timezone:  "America/New_York",
	},
	"VT38": {
		ICAO:      "VT38",
		Name:      "Greenwoods Airfield",
		City:      "Alburg",
		State:     "Vermont",
		Country:   "US",
		Elevation: 179,
		Latitude:  45.0092010498,
		Longitude: -73.2466964722,
		Timezone:  "America/New_York",
	},
	"VT39": {
		ICAO:      "VT39",
		Name:      "Taylor Airport",
		City:      "Wolcott",
		State:     "Vermont",
		Country:   "US",
		Elevation: 1140,
		Latitude:  44.6076011658,
		Longitude: -72.4804000854,
		Timezone:  "America/New_York",
	},
	"VT41": {
		ICAO:      "VT41",
		Name:      "Moore's Field",
		City:      "Brattleboro",
		State:     "Vermont",
		Country:   "US",
		Elevation: 310,
		Latitude:  42.9241981506,
		Longitude: -72.529800415,
		Timezone:  "America/New_York",
	},
	"VT42": {
		ICAO:      "VT42",
		Name:      "Two Tails Airport",
		City:      "Morrisville",
		State:     "Vermont",
		Country:   "US",
		Elevation: 754,
		Latitude:  44.5125999451,
		Longitude: -72.6071014404,
		Timezone:  "America/New_York",
	},
	"VT43": {
		ICAO:      "VT43",
		Name:      "Onyon Airport",
		City:      "Townshend",
		State:     "Vermont",
		Country:   "US",
		Elevation: 427,
		Latitude:  43.0191993713,
		Longitude: -72.6529006958,
		Timezone:  "America/New_York",
	},
	"VT44": {
		ICAO:      "VT44",
		Name:      "Perras Field",
		City:      "Morrisville",
		State:     "Vermont",
		Country:   "US",
		Elevation: 820,
		Latitude:  44.5266990662,
		Longitude: -72.5814971924,
		Timezone:  "America/New_York",
	},
	"VT45": {
		ICAO:      "VT45",
		Name:      "Davis Private Airport",
		City:      "Jericho",
		State:     "Vermont",
		Country:   "US",
		Elevation: 640,
		Latitude:  44.5172996521,
		Longitude: -72.9793014526,
		Timezone:  "America/New_York",
	},
	"VT46": {
		ICAO:      "VT46",
		Name:      "Northern Lights Airport",
		City:      "Alburg",
		State:     "Vermont",
		Country:   "US",
		Elevation: 130,
		Latitude:  44.8720016479,
		Longitude: -73.2864990234,
		Timezone:  "America/New_York",
	},
	"VT47": {
		ICAO:      "VT47",
		Name:      "Miller's Pleasure Airfield",
		City:      "Vernon",
		State:     "Vermont",
		Country:   "US",
		Elevation: 285,
		Latitude:  42.7785987854,
		Longitude: -72.5278015137,
		Timezone:  "America/New_York",
	},
	"VT48": {
		ICAO:      "VT48",
		Name:      "Fairholt Airport",
		City:      "Burlington",
		State:     "Vermont",
		Country:   "US",
		Elevation: 350,
		Latitude:  44.4613990784,
		Longitude: -73.1959991455,
		Timezone:  "America/New_York",
	},
	"VT52": {
		ICAO:      "VT52",
		Name:      "Shaw Meadow Airport",
		City:      "Milton",
		State:     "Vermont",
		Country:   "US",
		Elevation: 230,
		Latitude:  44.6433982849,
		Longitude: -73.1875991821,
		Timezone:  "America/New_York",
	},
	"VT53": {
		ICAO:      "VT53",
		Name:      "Staton Airport",
		City:      "Panton",
		State:     "Vermont",
		Country:   "US",
		Elevation: 140,
		Latitude:  44.1175994873,
		Longitude: -73.3787002563,
		Timezone:  "America/New_York",
	},
	"VT55": {
		ICAO:      "VT55",
		Name:      "Brandon Airport",
		City:      "Randolph",
		State:     "Vermont",
		Country:   "US",
		Elevation: 710,
		Latitude:  43.9011993408,
		Longitude: -72.582901001,
		Timezone:  "America/New_York",
	},
	"VT56": {
		ICAO:      "VT56",
		Name:      "West Burke Aerodrome",
		City:      "West Burke",
		State:     "Vermont",
		Country:   "US",
		Elevation: 1285,
		Latitude:  44.6130981445,
		Longitude: -71.9809036255,
		Timezone:  "America/New_York",
	},
	"VT59": {
		ICAO:      "VT59",
		Name:      "Ardell Flying Field",
		City:      "Manchester",
		State:     "Vermont",
		Country:   "US",
		Elevation: 1919,
		Latitude:  43.2778015137,
		Longitude: -72.9107971191,
		Timezone:  "America/New_York",
	},
	"VT60": {
		ICAO:      "VT60",
		Name:      "Hulett Landing Strip",
		City:      "West Pawlett",
		State:     "Vermont",
		Country:   "US",
		Elevation: 720,
		Latitude:  43.3301010132,
		Longitude: -73.2425994873,
		Timezone:  "America/New_York",
	},
	"VT62": {
		ICAO:      "VT62",
		Name:      "Catamount Airfield",
		City:      "Cabot",
		State:     "Vermont",
		Country:   "US",
		Elevation: 1350,
		Latitude:  44.3903999329,
		Longitude: -72.3506011963,
		Timezone:  "America/New_York",
	},
	"VT63": {
		ICAO:      "VT63",
		Name:      "Douglas Field",
		City:      "North Troy",
		State:     "Vermont",
		Country:   "US",
		Elevation: 650,
		Latitude:  44.995300293,
		Longitude: -72.3789978027,
		Timezone:  "America/New_York",
	},
	"VT64": {
		ICAO:      "VT64",
		Name:      "Mountain View Farm Airport",
		City:      "Morristown",
		State:     "Vermont",
		Country:   "US",
		Elevation: 1200,
		Latitude:  44.5153007507,
		Longitude: -72.6411972046,
		Timezone:  "America/New_York",
	},
	"VT73": {
		ICAO:      "VT73",
		Name:      "Melrose Springs Airfield",
		City:      "Bridport",
		State:     "Vermont",
		Country:   "US",
		Elevation: 300,
		Latitude:  43.944861,
		Longitude: -73.316833,
		Timezone:  "America/New_York",
	},
	"VTBD": {
		ICAO:      "VTBD",
		IATA:      "DMK",
		Name:      "Don Mueang International Airport",
		City:      "Bangkok",
		State:     "Bangkok",
		Country:   "TH",
		Elevation: 9,
		Latitude:  13.9125995636,
		Longitude: 100.6070022583,
		Timezone:  "Asia/Bangkok",
	},
	"VTBH": {
		ICAO:      "VTBH",
		IATA:      "KKM",
		Name:      "Sa Pran Nak Airport",
		State:     "Lop-Buri",
		Country:   "TH",
		Elevation: 95,
		Latitude:  14.9493999481,
		Longitude: 100.642997742,
		Timezone:  "Asia/Bangkok",
	},
	"VTBI": {
		ICAO:      "VTBI",
		Name:      "Khao E To Airport",
		State:     "Prachin-Buri",
		Country:   "TH",
		Elevation: 80,
		Latitude:  14.0777997971,
		Longitude: 101.3799972534,
		Timezone:  "Asia/Bangkok",
	},
	"VTBK": {
		ICAO:      "VTBK",
		IATA:      "KDT",
		Name:      "Kamphaeng Saen Airport",
		City:      "Nakhon Pathom",
		State:     "Nakhon-Pathom",
		Country:   "TH",
		Elevation: 30,
		Latitude:  14.1020002365,
		Longitude: 99.9171981812,
		Timezone:  "Asia/Bangkok",
	},
	"VTBL": {
		ICAO:      "VTBL",
		IATA:      "KKM",
		Name:      "Khok Kathiam Airport",
		State:     "Lop-Buri",
		Country:   "TH",
		Elevation: 123,
		Latitude:  14.8746004105,
		Longitude: 100.663002014,
		Timezone:  "Asia/Bangkok",
	},
	"VTBN": {
		ICAO:      "VTBN",
		Name:      "Prachuap Khiri Khan Airport",
		City:      "Prachuap Khiri Khan",
		State:     "Prachuap-Khiri-Khan",
		Country:   "TH",
		Latitude:  12.4172000885,
		Longitude: 99.8694000244,
		Timezone:  "Asia/Bangkok",
	},
	"VTBO": {
		ICAO:      "VTBO",
		IATA:      "TDX",
		Name:      "Trat Airport",
		State:     "Trat",
		Country:   "TH",
		Elevation: 105,
		Latitude:  12.274600029,
		Longitude: 102.319000244,
		Timezone:  "Asia/Bangkok",
	},
	"VTBP": {
		ICAO:      "VTBP",
		Name:      "Prachuap Airport",
		State:     "Prachuap-Khiri-Khan",
		Country:   "TH",
		Elevation: 10,
		Latitude:  11.7883996964,
		Longitude: 99.8047027588,
		Timezone:  "Asia/Bangkok",
	},
	"VTBS": {
		ICAO:      "VTBS",
		IATA:      "BKK",
		Name:      "Suvarnabhumi Airport",
		City:      "Bangkok",
		State:     "Samut-Prakan",
		Country:   "TH",
		Elevation: 5,
		Latitude:  13.6810998917,
		Longitude: 100.7470016479,
		Timezone:  "Asia/Bangkok",
	},
	"VTBT": {
		ICAO:      "VTBT",
		Name:      "Bang Pra Airport",
		City:      "Bang Pra",
		State:     "Chon-Buri",
		Country:   "TH",
		Elevation: 10,
		Latitude:  13.2324259496,
		Longitude: 100.956802368,
		Timezone:  "Asia/Bangkok",
	},
	"VTBU": {
		ICAO:      "VTBU",
		IATA:      "UTP",
		Name:      "U-Tapao International Airport",
		City:      "Rayong",
		State:     "Rayong",
		Country:   "TH",
		Elevation: 42,
		Latitude:  12.6799001694,
		Longitude: 101.0049972534,
		Timezone:  "Asia/Bangkok",
	},
	"VTBW": {
		ICAO:      "VTBW",
		Name:      "Watthana Nakhon Airport",
		State:     "Sa-Kaeo",
		Country:   "TH",
		Elevation: 200,
		Latitude:  13.7687997818,
		Longitude: 102.3150024414,
		Timezone:  "Asia/Bangkok",
	},
	"VTCB": {
		ICAO:      "VTCB",
		Name:      "Phayao Ban Chiang Kham Airport",
		City:      "Phayao Ban Chiang Kham",
		State:     "Phayao",
		Country:   "TH",
		Elevation: 1272,
		Latitude:  19.5,
		Longitude: 100.2829971313,
		Timezone:  "Asia/Bangkok",
	},
	"VTCC": {
		ICAO:      "VTCC",
		IATA:      "CNX",
		Name:      "Chiang Mai International Airport",
		City:      "Chiang Mai",
		State:     "Chiang-Mai",
		Country:   "TH",
		Elevation: 1036,
		Latitude:  18.7667999268,
		Longitude: 98.962600708,
		Timezone:  "Asia/Bangkok",
	},
	"VTCH": {
		ICAO:      "VTCH",
		IATA:      "HGN",
		Name:      "Mae Hong Son Airport",
		State:     "Mae-Hong-Son",
		Country:   "TH",
		Elevation: 929,
		Latitude:  19.3013000488,
		Longitude: 97.9757995605,
		Timezone:  "Asia/Bangkok",
	},
	"VTCI": {
		ICAO:      "VTCI",
		IATA:      "PYY",
		Name:      "Mae Hong Son Airport",
		City:      "Mae Hong Son",
		State:     "Mae-Hong-Son",
		Country:   "TH",
		Elevation: 1271,
		Latitude:  19.3719997406,
		Longitude: 98.43699646,
		Timezone:  "Asia/Bangkok",
	},
	"VTCL": {
		ICAO:      "VTCL",
		IATA:      "LPT",
		Name:      "Lampang Airport",
		State:     "Lampang",
		Country:   "TH",
		Elevation: 811,
		Latitude:  18.2709007263,
		Longitude: 99.5042037964,
		Timezone:  "Asia/Bangkok",
	},
	"VTCN": {
		ICAO:      "VTCN",
		IATA:      "NNT",
		Name:      "Nan Airport",
		State:     "Nan",
		Country:   "TH",
		Elevation: 685,
		Latitude:  18.8078994751,
		Longitude: 100.7829971313,
		Timezone:  "Asia/Bangkok",
	},
	"VTCO": {
		ICAO:      "VTCO",
		Name:      "Lamphun Airport",
		State:     "Lamphun",
		Country:   "TH",
		Elevation: 925,
		Latitude:  18.546816,
		Longitude: 99.012029,
		Timezone:  "Asia/Bangkok",
	},
	"VTCP": {
		ICAO:      "VTCP",
		IATA:      "PRH",
		Name:      "Phrae Airport",
		State:     "Phrae",
		Country:   "TH",
		Elevation: 538,
		Latitude:  18.1322002411,
		Longitude: 100.1650009155,
		Timezone:  "Asia/Bangkok",
	},
	"VTCR": {
		ICAO:      "VTCR",
		Name:      "Chiang Rai Airport",
		State:     "Chiang-Rai",
		Country:   "TH",
		Elevation: 1312,
		Latitude:  19.8854007721,
		Longitude: 99.8267974854,
		Timezone:  "Asia/Bangkok",
	},
	"VTCS": {
		ICAO:      "VTCS",
		Name:      "Mae Sariang Airport",
		City:      "Mae Sariang",
		State:     "Mae-Hong-Son",
		Country:   "TH",
		Elevation: 1030,
		Latitude:  18.1830005646,
		Longitude: 97.9329986572,
		Timezone:  "Asia/Bangkok",
	},
	"VTCT": {
		ICAO:      "VTCT",
		IATA:      "CEI",
		Name:      "Chiang Rai International Airport",
		City:      "Chiang Rai",
		State:     "Chiang-Rai",
		Country:   "TH",
		Elevation: 1280,
		Latitude:  19.952299118,
		Longitude: 99.8828964233,
		Timezone:  "Asia/Bangkok",
	},
	"VTED": {
		ICAO:      "VTED",
		IATA:      "BAO",
		Name:      "Udorn Air Base",
		City:      "Ban Mak Khaen",
		State:     "Changwat-Udon-Thani",
		Country:   "TH",
		Latitude:  17.3829994202,
		Longitude: 102.8000030518,
		Timezone:  "Asia/Bangkok",
	},
	"VTPB": {
		ICAO:      "VTPB",
		IATA:      "PHY",
		Name:      "Phetchabun Airport",
		State:     "Phetchabun",
		Country:   "TH",
		Elevation: 450,
		Latitude:  16.6760005951,
		Longitude: 101.194999695,
		Timezone:  "Asia/Bangkok",
	},
	"VTPH": {
		ICAO:      "VTPH",
		IATA:      "HHQ",
		Name:      "Hua Hin Airport",
		City:      "Hua Hin",
		State:     "Phetchaburi",
		Country:   "TH",
		Elevation: 62,
		Latitude:  12.6361999512,
		Longitude: 99.951499939,
		Timezone:  "Asia/Bangkok",
	},
	"VTPI": {
		ICAO:      "VTPI",
		IATA:      "TKH",
		Name:      "Takhli Airport",
		State:     "Nakhon-Sawan",
		Country:   "TH",
		Elevation: 107,
		Latitude:  15.277299881,
		Longitude: 100.29599762,
		Timezone:  "Asia/Bangkok",
	},
	"VTPL": {
		ICAO:      "VTPL",
		Name:      "Sak Long Airport",
		State:     "Phetchabun",
		Country:   "TH",
		Elevation: 500,
		Latitude:  16.824300766,
		Longitude: 101.2509994507,
		Timezone:  "Asia/Bangkok",
	},
	"VTPM": {
		ICAO:      "VTPM",
		IATA:      "MAQ",
		Name:      "Mae Sot Airport",
		State:     "Tak",
		Country:   "TH",
		Elevation: 690,
		Latitude:  16.6998996735,
		Longitude: 98.5450973511,
		Timezone:  "Asia/Bangkok",
	},
	"VTPN": {
		ICAO:      "VTPN",
		Name:      "Nakhon Sawan Airport",
		State:     "Nakhon-Sawan",
		Country:   "TH",
		Elevation: 113,
		Latitude:  15.6730003357,
		Longitude: 100.1370010376,
		Timezone:  "Asia/Bangkok",
	},
	"VTPO": {
		ICAO:      "VTPO",
		IATA:      "THS",
		Name:      "Sukhothai Airport",
		State:     "Sukhothai",
		Country:   "TH",
		Elevation: 179,
		Latitude:  17.2380008698,
		Longitude: 99.8181991577,
		Timezone:  "Asia/Bangkok",
	},
	"VTPP": {
		ICAO:      "VTPP",
		IATA:      "PHS",
		Name:      "Phitsanulok Airport",
		State:     "Phitsanulok",
		Country:   "TH",
		Elevation: 154,
		Latitude:  16.7828998566,
		Longitude: 100.2789993286,
		Timezone:  "Asia/Bangkok",
	},
	"VTPR": {
		ICAO:      "VTPR",
		Name:      "Photharam Airport",
		City:      "Photharam",
		State:     "Ratchaburi",
		Country:   "TH",
		Elevation: 88,
		Latitude:  13.6686000824,
		Longitude: 99.7313995361,
		Timezone:  "Asia/Bangkok",
	},
	"VTPT": {
		ICAO:      "VTPT",
		IATA:      "TKT",
		Name:      "Tak Airport",
		State:     "Tak",
		Country:   "TH",
		Elevation: 478,
		Latitude:  16.8959999084,
		Longitude: 99.2533035278,
		Timezone:  "Asia/Bangkok",
	},
	"VTPU": {
		ICAO:      "VTPU",
		IATA:      "UTR",
		Name:      "Uttaradit Airport",
		City:      "Uttaradit",
		State:     "Uttaradit",
		Country:   "TH",
		Elevation: 262,
		Latitude:  17.6170005798,
		Longitude: 100.0999984741,
		Timezone:  "Asia/Bangkok",
	},
	"VTPY": {
		ICAO:      "VTPY",
		Name:      "Khunan Phumipol Airport",
		State:     "Tak",
		Country:   "TH",
		Elevation: 492,
		Latitude:  17.2341995239,
		Longitude: 99.0578994751,
		Timezone:  "Asia/Bangkok",
	},
	"VTSA": {
		ICAO:      "VTSA",
		Name:      "Khoun Khan Airport",
		State:     "Satun",
		Country:   "TH",
		Elevation: 18,
		Latitude:  6.6613998413,
		Longitude: 100.0800018311,
		Timezone:  "Asia/Bangkok",
	},
	"VTSB": {
		ICAO:      "VTSB",
		IATA:      "URT",
		Name:      "Surat Thani Airport",
		City:      "Surat Thani",
		State:     "Surat-Thani",
		Country:   "TH",
		Elevation: 20,
		Latitude:  9.1325998306,
		Longitude: 99.135597229,
		Timezone:  "Asia/Bangkok",
	},
	"VTSC": {
		ICAO:      "VTSC",
		IATA:      "NAW",
		Name:      "Narathiwat Airport",
		State:     "Narathiwat",
		Country:   "TH",
		Elevation: 16,
		Latitude:  6.5199198723,
		Longitude: 101.7429962158,
		Timezone:  "Asia/Bangkok",
	},
	"VTSE": {
		ICAO:      "VTSE",
		IATA:      "CJM",
		Name:      "Chumphon Airport",
		State:     "Chumphon",
		Country:   "TH",
		Elevation: 18,
		Latitude:  10.7111997604,
		Longitude: 99.3617019653,
		Timezone:  "Asia/Bangkok",
	},
	"VTSF": {
		ICAO:      "VTSF",
		IATA:      "NST",
		Name:      "Nakhon Si Thammarat Airport",
		City:      "Nakhon Si Thammarat",
		State:     "Nakhon-Si-Thammarat",
		Country:   "TH",
		Elevation: 13,
		Latitude:  8.5396203995,
		Longitude: 99.9447021484,
		Timezone:  "Asia/Bangkok",
	},
	"VTSG": {
		ICAO:      "VTSG",
		IATA:      "KBV",
		Name:      "Krabi Airport",
		City:      "Krabi",
		State:     "Krabi",
		Country:   "TH",
		Elevation: 82,
		Latitude:  8.0991201401,
		Longitude: 98.9861984253,
		Timezone:  "Asia/Bangkok",
	},
	"VTSH": {
		ICAO:      "VTSH",
		IATA:      "SGZ",
		Name:      "Songkhla Airport",
		State:     "Songkhla",
		Country:   "TH",
		Elevation: 12,
		Latitude:  7.186560154,
		Longitude: 100.608001709,
		Timezone:  "Asia/Bangkok",
	},
	"VTSK": {
		ICAO:      "VTSK",
		IATA:      "PAN",
		Name:      "Pattani Airport",
		State:     "Pattani",
		Country:   "TH",
		Elevation: 8,
		Latitude:  6.7854599953,
		Longitude: 101.1539993286,
		Timezone:  "Asia/Bangkok",
	},
	"VTSM": {
		ICAO:      "VTSM",
		IATA:      "USM",
		Name:      "Samui Airport",
		City:      "Na Thon (Ko Samui Island)",
		State:     "Nakhon-Si-Thammarat",
		Country:   "TH",
		Elevation: 64,
		Latitude:  9.5477895737,
		Longitude: 100.06199646,
		Timezone:  "Asia/Bangkok",
	},
	"VTSN": {
		ICAO:      "VTSN",
		Name:      "Cha Ian Airport",
		State:     "Nakhon-Si-Thammarat",
		Country:   "TH",
		Elevation: 44,
		Latitude:  8.4711503983,
		Longitude: 99.9555969238,
		Timezone:  "Asia/Bangkok",
	},
	"VTSP": {
		ICAO:      "VTSP",
		IATA:      "HKT",
		Name:      "Phuket International Airport",
		City:      "Phuket",
		State:     "Phuket",
		Country:   "TH",
		Elevation: 82,
		Latitude:  8.1132001877,
		Longitude: 98.3169021606,
		Timezone:  "Asia/Bangkok",
	},
	"VTSR": {
		ICAO:      "VTSR",
		IATA:      "UNN",
		Name:      "Ranong Airport",
		State:     "Ranong",
		Country:   "TH",
		Elevation: 57,
		Latitude:  9.7776203156,
		Longitude: 98.5855026245,
		Timezone:  "Asia/Bangkok",
	},
	"VTSS": {
		ICAO:      "VTSS",
		IATA:      "HDY",
		Name:      "Hat Yai International Airport",
		City:      "Hat Yai",
		State:     "Songkhla",
		Country:   "TH",
		Elevation: 90,
		Latitude:  6.9332098961,
		Longitude: 100.392997742,
		Timezone:  "Asia/Bangkok",
	},
	"VTST": {
		ICAO:      "VTST",
		IATA:      "TST",
		Name:      "Trang Airport",
		State:     "Trang",
		Country:   "TH",
		Elevation: 67,
		Latitude:  7.5087399483,
		Longitude: 99.6166000366,
		Timezone:  "Asia/Bangkok",
	},
	"VTUD": {
		ICAO:      "VTUD",
		IATA:      "UTH",
		Name:      "Udon Thani Airport",
		City:      "Udon Thani",
		State:     "Changwat-Udon-Thani",
		Country:   "TH",
		Elevation: 579,
		Latitude:  17.3864002228,
		Longitude: 102.788002014,
		Timezone:  "Asia/Bangkok",
	},
	"VTUI": {
		ICAO:      "VTUI",
		IATA:      "SNO",
		Name:      "Sakon Nakhon Airport",
		State:     "Sakon-Nakhon",
		Country:   "TH",
		Elevation: 529,
		Latitude:  17.1951007843,
		Longitude: 104.1190032959,
		Timezone:  "Asia/Bangkok",
	},
	"VTUJ": {
		ICAO:      "VTUJ",
		IATA:      "PXR",
		Name:      "Surin Airport",
		State:     "Surin",
		Country:   "TH",
		Elevation: 478,
		Latitude:  14.8683004379,
		Longitude: 103.4980010986,
		Timezone:  "Asia/Bangkok",
	},
	"VTUK": {
		ICAO:      "VTUK",
		IATA:      "KKC",
		Name:      "Khon Kaen Airport",
		City:      "Khon Kaen",
		State:     "Khon-Kaen",
		Country:   "TH",
		Elevation: 670,
		Latitude:  16.4666004181,
		Longitude: 102.783996582,
		Timezone:  "Asia/Bangkok",
	},
	"VTUL": {
		ICAO:      "VTUL",
		IATA:      "LOE",
		Name:      "Loei Airport",
		State:     "Loei",
		Country:   "TH",
		Elevation: 860,
		Latitude:  17.4391002655,
		Longitude: 101.7220001221,
		Timezone:  "Asia/Bangkok",
	},
	"VTUN": {
		ICAO:      "VTUN",
		Name:      "Khorat Airport",
		State:     "Nakhon-Ratchasima",
		Country:   "TH",
		Elevation: 729,
		Latitude:  14.9344997406,
		Longitude: 102.07900238,
		Timezone:  "Asia/Bangkok",
	},
	"VTUO": {
		ICAO:      "VTUO",
		IATA:      "BFV",
		Name:      "Buri Ram Airport",
		State:     "Buriram",
		Country:   "TH",
		Elevation: 590,
		Latitude:  15.2294998169,
		Longitude: 103.2529983521,
		Timezone:  "Asia/Bangkok",
	},
	"VTUQ": {
		ICAO:      "VTUQ",
		IATA:      "NAK",
		Name:      "Nakhon Ratchasima Airport",
		State:     "Nakhon-Ratchasima",
		Country:   "TH",
		Elevation: 765,
		Latitude:  14.9495000839,
		Longitude: 102.31300354,
		Timezone:  "Asia/Bangkok",
	},
	"VTUR": {
		ICAO:      "VTUR",
		Name:      "Rob Muang Airport",
		State:     "Roi-Et",
		Country:   "TH",
		Elevation: 459,
		Latitude:  16.070400238,
		Longitude: 103.6460037231,
		Timezone:  "Asia/Bangkok",
	},
	"VTUU": {
		ICAO:      "VTUU",
		IATA:      "UBP",
		Name:      "Ubon Ratchathani Airport",
		City:      "Ubon Ratchathani",
		State:     "Changwat-Ubon-Ratchathani",
		Country:   "TH",
		Elevation: 406,
		Latitude:  15.2512998581,
		Longitude: 104.870002747,
		Timezone:  "Asia/Bangkok",
	},
	"VTUV": {
		ICAO:      "VTUV",
		IATA:      "ROI",
		Name:      "Roi Et Airport",
		State:     "Roi-Et",
		Country:   "TH",
		Elevation: 451,
		Latitude:  16.1168003082,
		Longitude: 103.7740020752,
		Timezone:  "Asia/Bangkok",
	},
	"VTUW": {
		ICAO:      "VTUW",
		IATA:      "KOP",
		Name:      "Nakhon Phanom Airport",
		State:     "Nakhon-Phanom",
		Country:   "TH",
		Elevation: 587,
		Latitude:  17.3838005066,
		Longitude: 104.6429977417,
		Timezone:  "Asia/Bangkok",
	},
	"VTUZ": {
		ICAO:      "VTUZ",
		Name:      "Nam Phung Dam Airport",
		City:      "Khon Kaen",
		State:     "Khon-Kaen",
		Country:   "TH",
		Latitude:  16.6522006989,
		Longitude: 102.9670028687,
		Timezone:  "Asia/Bangkok",
	},
	"VVBM": {
		ICAO:      "VVBM",
		IATA:      "BMV",
		Name:      "Buon Ma Thuot Airport",
		City:      "Buon Ma Thuot",
		State:     "Ha-Nội",
		Country:   "VN",
		Elevation: 1729,
		Latitude:  12.668299675,
		Longitude: 108.120002747,
		Timezone:  "Asia/Ho_Chi_Minh",
	},
	"VVCA": {
		ICAO:      "VVCA",
		IATA:      "VCL",
		Name:      "Chu Lai International Airport",
		City:      "Dung Quat Bay",
		Country:   "VN",
		Elevation: 10,
		Latitude:  15.4033002853,
		Longitude: 108.706001282,
		Timezone:  "Asia/Ho_Chi_Minh",
	},
	"VVCI": {
		ICAO:      "VVCI",
		IATA:      "HPH",
		Name:      "Cat Bi International Airport",
		City:      "Haiphong",
		State:     "Hải-Phong",
		Country:   "VN",
		Elevation: 6,
		Latitude:  20.8194007874,
		Longitude: 106.7249984741,
		Timezone:  "Asia/Bangkok",
	},
	"VVCL": {
		ICAO:      "VVCL",
		Name:      "Cam Ly Airport",
		State:     "Lam-Dồng",
		Country:   "VN",
		Elevation: 4937,
		Latitude:  11.9502844865,
		Longitude: 108.411004543,
		Timezone:  "Asia/Ho_Chi_Minh",
	},
	"VVCM": {
		ICAO:      "VVCM",
		IATA:      "CAH",
		Name:      "Ca Mau Airport",
		City:      "Ca Mau City",
		State:     "Cần-Thơ",
		Country:   "VN",
		Elevation: 6,
		Latitude:  9.1776666667,
		Longitude: 105.177777778,
		Timezone:  "Asia/Ho_Chi_Minh",
	},
	"VVCR": {
		ICAO:      "VVCR",
		IATA:      "CXR",
		Name:      "Cam Ranh Airport",
		City:      "Nha Trang",
		State:     "Khanh-Hoa",
		Country:   "VN",
		Elevation: 40,
		Latitude:  11.9982004166,
		Longitude: 109.21900177,
		Timezone:  "Asia/Ho_Chi_Minh",
	},
	"VVCS": {
		ICAO:      "VVCS",
		IATA:      "VCS",
		Name:      "Co Ong Airport",
		City:      "Con Ong",
		State:     "Ba-Rịa-Vung-Tau",
		Country:   "VN",
		Elevation: 20,
		Latitude:  8.7318296433,
		Longitude: 106.633003235,
		Timezone:  "Asia/Ho_Chi_Minh",
	},
	"VVCT": {
		ICAO:      "VVCT",
		IATA:      "VCA",
		Name:      "Tra Noc Airport",
		City:      "Can Tho",
		State:     "Lao-Cai",
		Country:   "VN",
		Elevation: 9,
		Latitude:  10.085100174,
		Longitude: 105.7119979858,
		Timezone:  "Asia/Ho_Chi_Minh",
	},
	"VVDB": {
		ICAO:      "VVDB",
		IATA:      "DIN",
		Name:      "Dien Bien Phu Airport",
		City:      "Dien Bien Phu",
		State:     "Lai-Chau",
		Country:   "VN",
		Elevation: 1611,
		Latitude:  21.3974990845,
		Longitude: 103.008003235,
		Timezone:  "Asia/Bangkok",
	},
	"VVDH": {
		ICAO:      "VVDH",
		IATA:      "VDH",
		Name:      "Dong Hoi Airport",
		City:      "Dong Hoi",
		State:     "Yen-Bai",
		Country:   "VN",
		Elevation: 59,
		Latitude:  17.515,
		Longitude: 106.590556,
		Timezone:  "Asia/Bangkok",
	},
	"VVDL": {
		ICAO:      "VVDL",
		IATA:      "DLI",
		Name:      "Lien Khuong Airport",
		City:      "Dalat",
		State:     "Lam-Dồng",
		Country:   "VN",
		Elevation: 3156,
		Latitude:  11.75,
		Longitude: 108.3669967651,
		Timezone:  "Asia/Ho_Chi_Minh",
	},
	"VVDN": {
		ICAO:      "VVDN",
		IATA:      "DAD",
		Name:      "Da Nang International Airport",
		City:      "Da Nang",
		State:     "Da-Nẵng",
		Country:   "VN",
		Elevation: 33,
		Latitude:  16.0438995361,
		Longitude: 108.1989974976,
		Timezone:  "Asia/Ho_Chi_Minh",
	},
	"VVGL": {
		ICAO:      "VVGL",
		Name:      "Gia Lam Air Base",
		City:      "Hanoi",
		State:     "Ha-Nội",
		Country:   "VN",
		Elevation: 50,
		Latitude:  21.0405006409,
		Longitude: 105.8860015869,
		Timezone:  "Asia/Bangkok",
	},
	"VVKP": {
		ICAO:      "VVKP",
		Name:      "Kep Air Base",
		City:      "Kep",
		State:     "Bắc-Giang",
		Country:   "VN",
		Elevation: 55,
		Latitude:  21.3945999146,
		Longitude: 106.261001587,
		Timezone:  "Asia/Bangkok",
	},
	"VVNB": {
		ICAO:      "VVNB",
		IATA:      "HAN",
		Name:      "Noi Bai International Airport",
		City:      "Hanoi",
		State:     "Ha-Nội",
		Country:   "VN",
		Elevation: 39,
		Latitude:  21.221200943,
		Longitude: 105.8069992065,
		Timezone:  "Asia/Bangkok",
	},
	"VVNS": {
		ICAO:      "VVNS",
		IATA:      "SQH",
		Name:      "Na-San Airport",
		City:      "Son-La",
		State:     "Sơn-La",
		Country:   "VN",
		Elevation: 2133,
		Latitude:  21.216999054,
		Longitude: 104.0329971313,
		Timezone:  "Asia/Bangkok",
	},
	"VVNT": {
		ICAO:      "VVNT",
		IATA:      "NHA",
		Name:      "Nha Trang Air Base",
		City:      "Nha Trang",
		State:     "Khanh-Hoa",
		Country:   "VN",
		Elevation: 20,
		Latitude:  12.2274999619,
		Longitude: 109.192001343,
		Timezone:  "Asia/Ho_Chi_Minh",
	},
	"VVPB": {
		ICAO:      "VVPB",
		IATA:      "HUI",
		Name:      "Phu Bai Airport",
		City:      "Hue",
		State:     "Bắc-Ninh",
		Country:   "VN",
		Elevation: 48,
		Latitude:  16.4015007019,
		Longitude: 107.70300293,
		Timezone:  "Asia/Ho_Chi_Minh",
	},
	"VVPC": {
		ICAO:      "VVPC",
		IATA:      "UIH",
		Name:      "Phu Cat Airport",
		City:      "Quy Nohn",
		State:     "Khanh-Hoa",
		Country:   "VN",
		Elevation: 80,
		Latitude:  13.9549999237,
		Longitude: 109.041999817,
		Timezone:  "Asia/Ho_Chi_Minh",
	},
	"VVPK": {
		ICAO:      "VVPK",
		IATA:      "PXU",
		Name:      "Pleiku Airport",
		City:      "Pleiku",
		Country:   "VN",
		Elevation: 2434,
		Latitude:  14.0045003891,
		Longitude: 108.016998291,
		Timezone:  "Asia/Ho_Chi_Minh",
	},
	"VVPQ": {
		ICAO:      "VVPQ",
		IATA:      "PQC",
		Name:      "Phu Quoc Airport",
		City:      "Duong Dong",
		State:     "Binh-Thuận",
		Country:   "VN",
		Elevation: 23,
		Latitude:  10.2270002365,
		Longitude: 103.967002869,
		Timezone:  "Asia/Ho_Chi_Minh",
	},
	"VVPR": {
		ICAO:      "VVPR",
		IATA:      "PHA",
		Name:      "Phan Rang Airport",
		City:      "Phan Rang",
		State:     "Tuyen-Quang",
		Country:   "VN",
		Elevation: 101,
		Latitude:  11.6335000992,
		Longitude: 108.952003479,
		Timezone:  "Asia/Ho_Chi_Minh",
	},
	"VVPT": {
		ICAO:      "VVPT",
		IATA:      "PHH",
		Name:      "Phan Thiet Airport",
		City:      "Phan Thiet",
		State:     "Binh-Thuận",
		Country:   "VN",
		Latitude:  10.9063997269,
		Longitude: 108.0690002441,
		Timezone:  "Asia/Ho_Chi_Minh",
	},
	"VVRG": {
		ICAO:      "VVRG",
		IATA:      "VKG",
		Name:      "Rach Gia Airport",
		City:      "Rach Gia",
		State:     "Binh-Thuận",
		Country:   "VN",
		Elevation: 7,
		Latitude:  9.9580299723,
		Longitude: 105.132379532,
		Timezone:  "Asia/Ho_Chi_Minh",
	},
	"VVTH": {
		ICAO:      "VVTH",
		IATA:      "TBB",
		Name:      "Dong Tac Airport",
		City:      "Tuy Hoa",
		State:     "Vinh-Long",
		Country:   "VN",
		Elevation: 20,
		Latitude:  13.0495996475,
		Longitude: 109.333999634,
		Timezone:  "Asia/Ho_Chi_Minh",
	},
	"VVTS": {
		ICAO:      "VVTS",
		IATA:      "SGN",
		Name:      "Tan Son Nhat International Airport",
		City:      "Ho Chi Minh City",
		State:     "Ho-Chi-Minh-City",
		Country:   "VN",
		Elevation: 33,
		Latitude:  10.8187999725,
		Longitude: 106.652000427,
		Timezone:  "Asia/Ho_Chi_Minh",
	},
	"VVTX": {
		ICAO:      "VVTX",
		IATA:      "THD",
		Name:      "Thọ Xuân Airport",
		City:      "Thanh Hóa",
		Country:   "VN",
		Elevation: 59,
		Latitude:  19.901667,
		Longitude: 105.467778,
		Timezone:  "Asia/Bangkok",
	},
	"VVVH": {
		ICAO:      "VVVH",
		IATA:      "VII",
		Name:      "Vinh Airport",
		City:      "Vinh",
		State:     "Nghệ-An",
		Country:   "VN",
		Elevation: 23,
		Latitude:  18.7376003265,
		Longitude: 105.67099762,
		Timezone:  "Asia/Bangkok",
	},
	"VVVT": {
		ICAO:      "VVVT",
		IATA:      "VTG",
		Name:      "Vung Tau Airport",
		City:      "Vung Tau",
		State:     "Ba-Rịa-Vung-Tau",
		Country:   "VN",
		Elevation: 13,
		Latitude:  10.3725004196,
		Longitude: 107.0950012207,
		Timezone:  "Asia/Ho_Chi_Minh",
	},
	"VY02": {
		ICAO:      "VY02",
		Name:      "New Magway Airport",
		City:      "Magway",
		State:     "Magway",
		Country:   "MM",
		Elevation: 279,
		Latitude:  20.1536006927,
		Longitude: 94.9686965942,
		Timezone:  "Asia/Yangon",
	},
	"VY03": {
		ICAO:      "VY03",
		Name:      "Toungkomei Airport",
		City:      "Taungkamet",
		State:     "Tanintharyi",
		Country:   "MM",
		Elevation: 92,
		Latitude:  11.1493997574,
		Longitude: 98.7359008789,
		Timezone:  "Asia/Yangon",
	},
	"VYAN": {
		ICAO:      "VYAN",
		IATA:      "VBA",
		Name:      "Ann Airport",
		City:      "Aeng",
		State:     "Rakhine",
		Country:   "MM",
		Elevation: 74,
		Latitude:  19.7691993713,
		Longitude: 94.0261001587,
		Timezone:  "Asia/Yangon",
	},
	"VYAS": {
		ICAO:      "VYAS",
		Name:      "Anisakan Airport",
		City:      "Anisakan",
		State:     "Mandalay",
		Country:   "MM",
		Elevation: 3000,
		Latitude:  21.9554004669,
		Longitude: 96.4060974121,
		Timezone:  "Asia/Yangon",
	},
	"VYBG": {
		ICAO:      "VYBG",
		IATA:      "NYU",
		Name:      "Bagan Airport",
		City:      "Nyaung U",
		State:     "Mandalay",
		Country:   "MM",
		Elevation: 312,
		Latitude:  21.1788005829,
		Longitude: 94.9301986694,
		Timezone:  "Asia/Yangon",
	},
	"VYBM": {
		ICAO:      "VYBM",
		IATA:      "BMO",
		Name:      "Banmaw Airport",
		City:      "Banmaw",
		State:     "Kachin",
		Country:   "MM",
		Elevation: 370,
		Latitude:  24.2689990997,
		Longitude: 97.2462005615,
		Timezone:  "Asia/Yangon",
	},
	"VYBP": {
		ICAO:      "VYBP",
		IATA:      "VBP",
		Name:      "Bokpyinn Airport",
		City:      "Bokpyinn",
		State:     "Tanintharyi",
		Country:   "MM",
		Elevation: 100,
		Latitude:  11.2670001984,
		Longitude: 98.766998291,
		Timezone:  "Asia/Yangon",
	},
	"VYCI": {
		ICAO:      "VYCI",
		Name:      "Coco Island Airport",
		City:      "Coco Island",
		State:     "Yangon",
		Country:   "MM",
		Elevation: 20,
		Latitude:  14.141500473,
		Longitude: 93.3684997559,
		Timezone:  "Asia/Yangon",
	},
	"VYCZ": {
		ICAO:      "VYCZ",
		IATA:      "VBC",
		Name:      "Chanmyathazi Airport",
		City:      "Mandalay",
		State:     "Mandalay",
		Country:   "MM",
		Elevation: 252,
		Latitude:  21.9405002594,
		Longitude: 96.0895996094,
		Timezone:  "Asia/Yangon",
	},
	"VYDW": {
		ICAO:      "VYDW",
		IATA:      "TVY",
		Name:      "Dawei Airport",
		City:      "Dawei",
		State:     "Tanintharyi",
		Country:   "MM",
		Elevation: 84,
		Latitude:  14.1038999557,
		Longitude: 98.2035980225,
		Timezone:  "Asia/Yangon",
	},
	"VYEL": {
		ICAO:      "VYEL",
		IATA:      "NYT",
		Name:      "Naypyidaw Airport",
		City:      "Pyinmana",
		State:     "Mandalay",
		Country:   "MM",
		Elevation: 302,
		Latitude:  19.623500824,
		Longitude: 96.2009963989,
		Timezone:  "Asia/Yangon",
	},
	"VYGG": {
		ICAO:      "VYGG",
		IATA:      "GAW",
		Name:      "Gangaw Airport",
		City:      "Gangaw",
		State:     "Magway",
		Country:   "MM",
		Elevation: 200,
		Latitude:  22.1746997833,
		Longitude: 94.1343994141,
		Timezone:  "Asia/Yangon",
	},
	"VYGW": {
		ICAO:      "VYGW",
		IATA:      "GWA",
		Name:      "Gwa Airport",
		City:      "Gwa",
		State:     "Rakhine",
		Country:   "MM",
		Elevation: 10,
		Latitude:  17.6000003815,
		Longitude: 94.5832977295,
		Timezone:  "Asia/Yangon",
	},
	"VYHB": {
		ICAO:      "VYHB",
		Name:      "Hmawbi Air Base",
		City:      "Hmawbi",
		State:     "Yangon",
		Country:   "MM",
		Elevation: 69,
		Latitude:  17.1166992188,
		Longitude: 96.0667037964,
		Timezone:  "Asia/Yangon",
	},
	"VYHH": {
		ICAO:      "VYHH",
		IATA:      "HEH",
		Name:      "Heho Airport",
		City:      "Heho",
		State:     "Shan",
		Country:   "MM",
		Elevation: 3858,
		Latitude:  20.7469997406,
		Longitude: 96.7919998169,
		Timezone:  "Asia/Yangon",
	},
	"VYHL": {
		ICAO:      "VYHL",
		IATA:      "HOX",
		Name:      "Hommalinn Airport",
		City:      "Hommalinn",
		State:     "Sagain",
		Country:   "MM",
		Elevation: 534,
		Latitude:  24.8995990753,
		Longitude: 94.9140014648,
		Timezone:  "Asia/Yangon",
	},
	"VYHN": {
		ICAO:      "VYHN",
		IATA:      "TIO",
		Name:      "Tilin Airport",
		City:      "Tilin",
		State:     "Magway",
		Country:   "MM",
		Elevation: 1385,
		Latitude:  21.7000007629,
		Longitude: 94.0999984741,
		Timezone:  "Asia/Yangon",
	},
	"VYKG": {
		ICAO:      "VYKG",
		IATA:      "KET",
		Name:      "Kengtung Airport",
		City:      "Kengtung",
		State:     "Shan",
		Country:   "MM",
		Elevation: 2798,
		Latitude:  21.3015995026,
		Longitude: 99.6360015869,
		Timezone:  "Asia/Yangon",
	},
	"VYKI": {
		ICAO:      "VYKI",
		IATA:      "KHM",
		Name:      "Kanti Airport",
		City:      "Kanti",
		State:     "Sagain",
		Country:   "MM",
		Elevation: 6000,
		Latitude:  25.9883003235,
		Longitude: 95.6744003296,
		Timezone:  "Asia/Yangon",
	},
	"VYKL": {
		ICAO:      "VYKL",
		IATA:      "KMV",
		Name:      "Kalay Airport",
		City:      "Kalemyo",
		State:     "Sagain",
		Country:   "MM",
		Elevation: 499,
		Latitude:  23.1888008118,
		Longitude: 94.0511016846,
		Timezone:  "Asia/Yangon",
	},
	"VYKP": {
		ICAO:      "VYKP",
		IATA:      "KYP",
		Name:      "Kyaukpyu Airport",
		City:      "Kyaukpyu",
		State:     "Rakhine",
		Country:   "MM",
		Elevation: 20,
		Latitude:  19.426399231,
		Longitude: 93.5347976685,
		Timezone:  "Asia/Yangon",
	},
	"VYKT": {
		ICAO:      "VYKT",
		IATA:      "KAW",
		Name:      "Kawthoung Airport",
		City:      "Kawthoung",
		State:     "Tanintharyi",
		Country:   "MM",
		Elevation: 180,
		Latitude:  10.0493001938,
		Longitude: 98.5380020142,
		Timezone:  "Asia/Yangon",
	},
	"VYKU": {
		ICAO:      "VYKU",
		IATA:      "KYT",
		Name:      "Kyauktu Airport",
		City:      "Kyauktu",
		State:     "Magway",
		Country:   "MM",
		Elevation: 1250,
		Latitude:  21.3999996185,
		Longitude: 94.1333007813,
		Timezone:  "Asia/Yangon",
	},
	"VYLK": {
		ICAO:      "VYLK",
		IATA:      "LIW",
		Name:      "Loikaw Airport",
		City:      "Loikaw",
		State:     "Kayah",
		Country:   "MM",
		Elevation: 2940,
		Latitude:  19.6914997101,
		Longitude: 97.2147979736,
		Timezone:  "Asia/Yangon",
	},
	"VYLN": {
		ICAO:      "VYLN",
		Name:      "Lonekin Airport",
		City:      "Lonekin",
		State:     "Kachin",
		Country:   "MM",
		Elevation: 1010,
		Latitude:  25.6499996185,
		Longitude: 96.3669967651,
		Timezone:  "Asia/Yangon",
	},
	"VYLS": {
		ICAO:      "VYLS",
		IATA:      "LSH",
		Name:      "Lashio Airport",
		City:      "Lashio",
		State:     "Shan",
		Country:   "MM",
		Elevation: 2450,
		Latitude:  22.9778995514,
		Longitude: 97.7521972656,
		Timezone:  "Asia/Yangon",
	},
	"VYLY": {
		ICAO:      "VYLY",
		Name:      "Lanywa Airport",
		City:      "Lanywa",
		State:     "Magway",
		Country:   "MM",
		Elevation: 175,
		Latitude:  20.9403991699,
		Longitude: 94.8226013184,
		Timezone:  "Asia/Yangon",
	},
	"VYMD": {
		ICAO:      "VYMD",
		IATA:      "MDL",
		Name:      "Mandalay International Airport",
		City:      "Mandalay",
		State:     "Mandalay",
		Country:   "MM",
		Elevation: 300,
		Latitude:  21.7021999359,
		Longitude: 95.977897644,
		Timezone:  "Asia/Yangon",
	},
	"VYME": {
		ICAO:      "VYME",
		IATA:      "MGZ",
		Name:      "Myeik Airport",
		City:      "Mkeik",
		State:     "Tanintharyi",
		Country:   "MM",
		Elevation: 75,
		Latitude:  12.4398002625,
		Longitude: 98.6214981079,
		Timezone:  "Asia/Yangon",
	},
	"VYMH": {
		ICAO:      "VYMH",
		Name:      "Mong Hpayak Airport",
		City:      "Mong Hpayak",
		State:     "Shan",
		Country:   "MM",
		Elevation: 1700,
		Latitude:  20.8330001831,
		Longitude: 99.9499969482,
		Timezone:  "Asia/Yangon",
	},
	"VYMI": {
		ICAO:      "VYMI",
		Name:      "Mongyai Airport",
		City:      "Mongyai",
		State:     "Shan",
		Country:   "MM",
		Elevation: 2900,
		Latitude:  22.4829998016,
		Longitude: 98.0329971313,
		Timezone:  "Asia/Yangon",
	},
	"VYMK": {
		ICAO:      "VYMK",
		IATA:      "MYT",
		Name:      "Myitkyina Airport",
		City:      "Myitkyina",
		State:     "Kachin",
		Country:   "MM",
		Elevation: 475,
		Latitude:  25.383600235,
		Longitude: 97.3518981934,
		Timezone:  "Asia/Yangon",
	},
	"VYML": {
		ICAO:      "VYML",
		Name:      "Meiktila Air Base",
		City:      "Meiktila",
		State:     "Mandalay",
		Country:   "MM",
		Elevation: 699,
		Latitude:  20.8864002228,
		Longitude: 95.892791748,
		Timezone:  "Asia/Yangon",
	},
	"VYMM": {
		ICAO:      "VYMM",
		IATA:      "MNU",
		Name:      "Mawlamyine Airport",
		City:      "Mawlamyine",
		State:     "Mon",
		Country:   "MM",
		Elevation: 52,
		Latitude:  16.4447002411,
		Longitude: 97.660697937,
		Timezone:  "Asia/Yangon",
	},
	"VYMN": {
		ICAO:      "VYMN",
		IATA:      "MGU",
		Name:      "Manaung Airport",
		City:      "Manaung",
		State:     "Rakhine",
		Country:   "MM",
		Elevation: 102,
		Latitude:  18.8458003998,
		Longitude: 93.6889038086,
		Timezone:  "Asia/Yangon",
	},
	"VYMO": {
		ICAO:      "VYMO",
		IATA:      "MOE",
		Name:      "Momeik Airport",
		State:     "Shan",
		Country:   "MM",
		Elevation: 600,
		Latitude:  23.0925006866,
		Longitude: 96.6453018188,
		Timezone:  "Asia/Yangon",
	},
	"VYMP": {
		ICAO:      "VYMP",
		Name:      "Mongpyin Airport",
		City:      "Mongpyin",
		State:     "Shan",
		Country:   "MM",
		Elevation: 1516,
		Latitude:  21.3500003815,
		Longitude: 99.0329971313,
		Timezone:  "Asia/Yangon",
	},
	"VYMS": {
		ICAO:      "VYMS",
		IATA:      "MOG",
		Name:      "Mong Hsat Airport",
		City:      "Mong Hsat",
		State:     "Shan",
		Country:   "MM",
		Elevation: 1875,
		Latitude:  20.5167999268,
		Longitude: 99.2567977905,
		Timezone:  "Asia/Yangon",
	},
	"VYMT": {
		ICAO:      "VYMT",
		IATA:      "MGK",
		Name:      "Mong Tong Airport",
		City:      "Mong Tong",
		State:     "Shan",
		Country:   "MM",
		Elevation: 1840,
		Latitude:  20.2966995239,
		Longitude: 98.8989028931,
		Timezone:  "Asia/Yangon",
	},
	"VYMU": {
		ICAO:      "VYMU",
		Name:      "Myauk U Airport",
		City:      "Myauk U",
		State:     "Rakhine",
		Country:   "MM",
		Elevation: 60,
		Latitude:  20.5669994354,
		Longitude: 93.1999969482,
		Timezone:  "Asia/Yangon",
	},
	"VYMW": {
		ICAO:      "VYMW",
		IATA:      "MWQ",
		Name:      "Magway Airport",
		City:      "Magway",
		State:     "Magway",
		Country:   "MM",
		Elevation: 279,
		Latitude:  20.165599823,
		Longitude: 94.9413986206,
		Timezone:  "Asia/Yangon",
	},
	"VYMY": {
		ICAO:      "VYMY",
		IATA:      "NYW",
		Name:      "Monywar Airport",
		City:      "Monywar",
		State:     "Sagain",
		Country:   "MM",
		Elevation: 298,
		Latitude:  22.2329998016,
		Longitude: 95.1169967651,
		Timezone:  "Asia/Yangon",
	},
	"VYNM": {
		ICAO:      "VYNM",
		Name:      "Naungmom Airport",
		City:      "Naungmom",
		State:     "Kachin",
		Country:   "MM",
		Elevation: 2200,
		Latitude:  27.5,
		Longitude: 97.8170013428,
		Timezone:  "Asia/Yangon",
	},
	"VYNP": {
		ICAO:      "VYNP",
		Name:      "Nampong Air Base",
		City:      "Myitkyina",
		State:     "Kachin",
		Country:   "MM",
		Elevation: 459,
		Latitude:  25.3544006348,
		Longitude: 97.2951965332,
		Timezone:  "Asia/Yangon",
	},
	"VYNS": {
		ICAO:      "VYNS",
		IATA:      "NMS",
		Name:      "Namsang Airport",
		City:      "Namsang",
		State:     "Shan",
		Country:   "MM",
		Elevation: 3100,
		Latitude:  20.890499115,
		Longitude: 97.7359008789,
		Timezone:  "Asia/Yangon",
	},
	"VYNT": {
		ICAO:      "VYNT",
		IATA:      "NMT",
		Name:      "Namtu Airport",
		City:      "Namtu",
		State:     "Shan",
		Country:   "MM",
		Elevation: 2000,
		Latitude:  23.0830001831,
		Longitude: 97.3830032349,
		Timezone:  "Asia/Yangon",
	},
	"VYPA": {
		ICAO:      "VYPA",
		IATA:      "PAA",
		Name:      "Hpa-N Airport",
		City:      "Hpa-N",
		State:     "Kayin",
		Country:   "MM",
		Elevation: 150,
		Latitude:  16.893699646,
		Longitude: 97.6745986938,
		Timezone:  "Asia/Yangon",
	},
	"VYPB": {
		ICAO:      "VYPB",
		Name:      "Phonngbyin Airport",
		City:      "Phonngbyin",
		State:     "Sagain",
		Country:   "MM",
		Elevation: 300,
		Latitude:  24.25,
		Longitude: 94.7829971313,
		Timezone:  "Asia/Yangon",
	},
	"VYPE": {
		ICAO:      "VYPE",
		Name:      "Paletwa Airport",
		City:      "Paletwa",
		State:     "Chin",
		Country:   "MM",
		Elevation: 60,
		Latitude:  21.2999992371,
		Longitude: 92.9169998169,
		Timezone:  "Asia/Yangon",
	},
	"VYPI": {
		ICAO:      "VYPI",
		Name:      "Pearl Island Airport",
		City:      "Pearl Island",
		State:     "Tanintharyi",
		Country:   "MM",
		Elevation: 5,
		Latitude:  11.2829999924,
		Longitude: 98.1999969482,
		Timezone:  "Asia/Yangon",
	},
	"VYPK": {
		ICAO:      "VYPK",
		IATA:      "PAU",
		Name:      "Pauk Airport",
		City:      "Pauk",
		State:     "Magway",
		Country:   "MM",
		Elevation: 249,
		Latitude:  21.4491996765,
		Longitude: 94.4869003296,
		Timezone:  "Asia/Yangon",
	},
	"VYPL": {
		ICAO:      "VYPL",
		Name:      "Pinlebu Airport",
		City:      "Pinlebu",
		State:     "Sagain",
		Country:   "MM",
		Elevation: 830,
		Latitude:  24.0830001831,
		Longitude: 95.3669967651,
		Timezone:  "Asia/Yangon",
	},
	"VYPN": {
		ICAO:      "VYPN",
		IATA:      "BSX",
		Name:      "Pathein Airport",
		City:      "Pathein",
		State:     "Ayeyarwady",
		Country:   "MM",
		Elevation: 20,
		Latitude:  16.8152008057,
		Longitude: 94.7798995972,
		Timezone:  "Asia/Yangon",
	},
	"VYPP": {
		ICAO:      "VYPP",
		IATA:      "PPU",
		Name:      "Hpapun Airport",
		City:      "Pa Pun",
		State:     "Kayin",
		Country:   "MM",
		Elevation: 300,
		Latitude:  18.0666999817,
		Longitude: 97.449798584,
		Timezone:  "Asia/Yangon",
	},
	"VYPT": {
		ICAO:      "VYPT",
		IATA:      "PBU",
		Name:      "Putao Airport",
		City:      "Putao",
		State:     "Kachin",
		Country:   "MM",
		Elevation: 1500,
		Latitude:  27.3299007416,
		Longitude: 97.4263000488,
		Timezone:  "Asia/Yangon",
	},
	"VYPU": {
		ICAO:      "VYPU",
		IATA:      "PKK",
		Name:      "Pakhokku Airport",
		City:      "Pakhokku",
		State:     "Magway",
		Country:   "MM",
		Elevation: 151,
		Latitude:  21.4043,
		Longitude: 95.11125,
		Timezone:  "Asia/Yangon",
	},
	"VYPW": {
		ICAO:      "VYPW",
		Name:      "Palaw Airport",
		City:      "Palaw",
		State:     "Tanintharyi",
		Country:   "MM",
		Elevation: 150,
		Latitude:  12.9499998093,
		Longitude: 98.6500015259,
		Timezone:  "Asia/Yangon",
	},
	"VYPY": {
		ICAO:      "VYPY",
		IATA:      "PRU",
		Name:      "Pyay Airport",
		City:      "Pye",
		State:     "Bago",
		Country:   "MM",
		Elevation: 120,
		Latitude:  18.8244991302,
		Longitude: 95.2659988403,
		Timezone:  "Asia/Yangon",
	},
	"VYSA": {
		ICAO:      "VYSA",
		Name:      "Saw Airport",
		City:      "Saw",
		State:     "Magway",
		Country:   "MM",
		Elevation: 1100,
		Latitude:  21.1669998169,
		Longitude: 94.1669998169,
		Timezone:  "Asia/Yangon",
	},
	"VYSL": {
		ICAO:      "VYSL",
		Name:      "Salingyi Airport",
		City:      "Salingyi",
		State:     "Sagain",
		Country:   "MM",
		Elevation: 264,
		Latitude:  22.1000003815,
		Longitude: 95.016998291,
		Timezone:  "Asia/Yangon",
	},
	"VYSO": {
		ICAO:      "VYSO",
		Name:      "Sedoktayar Airport",
		City:      "Sedoktayar",
		State:     "Magway",
		Country:   "MM",
		Elevation: 500,
		Latitude:  20.4500007629,
		Longitude: 94.233001709,
		Timezone:  "Asia/Yangon",
	},
	"VYST": {
		ICAO:      "VYST",
		Name:      "Shante Air Base",
		City:      "Meiktila",
		State:     "Mandalay",
		Country:   "MM",
		Elevation: 630,
		Latitude:  20.9416999817,
		Longitude: 95.9144973755,
		Timezone:  "Asia/Yangon",
	},
	"VYSW": {
		ICAO:      "VYSW",
		IATA:      "AKY",
		Name:      "Sittwe Airport",
		City:      "Sittwe",
		State:     "Rakhine",
		Country:   "MM",
		Elevation: 27,
		Latitude:  20.1326999664,
		Longitude: 92.8725967407,
		Timezone:  "Asia/Yangon",
	},
	"VYTD": {
		ICAO:      "VYTD",
		IATA:      "SNW",
		Name:      "Thandwe Airport",
		City:      "Thandwe",
		State:     "Rakhine",
		Country:   "MM",
		Elevation: 20,
		Latitude:  18.4606990814,
		Longitude: 94.3001022339,
		Timezone:  "Asia/Yangon",
	},
	"VYTL": {
		ICAO:      "VYTL",
		IATA:      "THL",
		Name:      "Tachileik Airport",
		City:      "Tachileik",
		State:     "Shan",
		Country:   "MM",
		Elevation: 1280,
		Latitude:  20.4838008881,
		Longitude: 99.9354019165,
		Timezone:  "Asia/Yangon",
	},
	"VYTN": {
		ICAO:      "VYTN",
		Name:      "Tanai Airport",
		City:      "Tanai",
		State:     "Kachin",
		Country:   "MM",
		Elevation: 690,
		Latitude:  26.3670005798,
		Longitude: 96.733001709,
		Timezone:  "Asia/Yangon",
	},
	"VYTO": {
		ICAO:      "VYTO",
		Name:      "Taungoo Airport",
		City:      "Taungoo",
		State:     "Bago",
		Country:   "MM",
		Elevation: 160,
		Latitude:  19.0312995911,
		Longitude: 96.4011993408,
		Timezone:  "Asia/Yangon",
	},
	"VYTY": {
		ICAO:      "VYTY",
		Name:      "Tanyang Airport",
		City:      "Tanyang",
		State:     "Shan",
		Country:   "MM",
		Elevation: 250,
		Latitude:  22.4829998016,
		Longitude: 98.4000015259,
		Timezone:  "Asia/Yangon",
	},
	"VYXG": {
		ICAO:      "VYXG",
		Name:      "Kyaukhtu South Airport",
		City:      "Kyaukhtu South",
		State:     "Magway",
		Country:   "MM",
		Elevation: 1345,
		Latitude:  21.4067001343,
		Longitude: 94.1303024292,
		Timezone:  "Asia/Yangon",
	},
	"VYYE": {
		ICAO:      "VYYE",
		IATA:      "XYE",
		Name:      "Ye Airport",
		City:      "Ye",
		State:     "Mon",
		Country:   "MM",
		Elevation: 30,
		Latitude:  15.3000001907,
		Longitude: 97.8669967651,
		Timezone:  "Asia/Yangon",
	},
	"VYYY": {
		ICAO:      "VYYY",
		IATA:      "RGN",
		Name:      "Yangon International Airport",
		City:      "Yangon",
		State:     "Yangon",
		Country:   "MM",
		Elevation: 109,
		Latitude:  16.9073009491,
		Longitude: 96.1332015991,
		Timezone:  "Asia/Yangon",
	},
	"WA00": {
		ICAO:      "WA00",
		Name:      "Mercer Ranch Airport",
		City:      "Alderdale",
		State:     "Washington",
		Country:   "US",
		Elevation: 780,
		Latitude:  45.9000015259,
		Longitude: -119.90599823,
		Timezone:  "America/Los_Angeles",
	},
	"WA01": {
		ICAO:      "WA01",
		Name:      "Wirkkala Airport",
		City:      "Naselle",
		State:     "Washington",
		Country:   "US",
		Elevation: 35,
		Latitude:  46.3562011719,
		Longitude: -123.8150024414,
		Timezone:  "America/Los_Angeles",
	},
	"WA04": {
		ICAO:      "WA04",
		Name:      "Kyles Airport",
		City:      "Snohomish",
		State:     "Washington",
		Country:   "US",
		Elevation: 470,
		Latitude:  47.8133010864,
		Longitude: -122.0429992676,
		Timezone:  "America/Los_Angeles",
	},
	"WA05": {
		ICAO:      "WA05",
		Name:      "Apex Airpark",
		City:      "Silverdale",
		State:     "Washington",
		Country:   "US",
		Elevation: 525,
		Latitude:  47.6567993164,
		Longitude: -122.733001709,
		Timezone:  "America/Los_Angeles",
	},
	"WA07": {
		ICAO:      "WA07",
		Name:      "Barker Airport",
		City:      "Mount Vernon",
		State:     "Washington",
		Country:   "US",
		Elevation: 5,
		Latitude:  48.3748016357,
		Longitude: -122.3389968872,
		Timezone:  "America/Los_Angeles",
	},
	"WA08": {
		ICAO:      "WA08",
		Name:      "Zwainz Farms Airport",
		City:      "Reardan",
		State:     "Washington",
		Country:   "US",
		Elevation: 2795,
		Latitude:  47.7206993103,
		Longitude: -117.908996582,
		Timezone:  "America/Los_Angeles",
	},
	"WA09": {
		ICAO:      "WA09",
		Name:      "Roche Harbor Airport",
		City:      "Roche Harbor",
		State:     "Washington",
		Country:   "US",
		Elevation: 100,
		Latitude:  48.6123008728,
		Longitude: -123.138999939,
		Timezone:  "America/Los_Angeles",
	},
	"WA10": {
		ICAO:      "WA10",
		Name:      "Gosney Airport",
		City:      "Kaycee",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4830,
		Latitude:  43.7221984863,
		Longitude: -106.6669998169,
		Timezone:  "America/Denver",
	},
	"WA12": {
		ICAO:      "WA12",
		Name:      "Acme Field",
		City:      "Acme",
		State:     "Washington",
		Country:   "US",
		Elevation: 301,
		Latitude:  48.7095985413,
		Longitude: -122.1780014038,
		Timezone:  "America/Los_Angeles",
	},
	"WA14": {
		ICAO:      "WA14",
		Name:      "Connell City Airport",
		City:      "Connell",
		State:     "Washington",
		Country:   "US",
		Elevation: 925,
		Latitude:  46.662399292,
		Longitude: -118.8330001831,
		Timezone:  "America/Los_Angeles",
	},
	"WA15": {
		ICAO:      "WA15",
		Name:      "Coulee City Airport",
		City:      "Coulee City",
		State:     "Washington",
		Country:   "US",
		Elevation: 1800,
		Latitude:  47.6165008545,
		Longitude: -119.2429962158,
		Timezone:  "America/Los_Angeles",
	},
	"WA16": {
		ICAO:      "WA16",
		Name:      "Banas Field",
		City:      "Brooklyn",
		State:     "Washington",
		Country:   "US",
		Elevation: 180,
		Latitude:  46.7997016907,
		Longitude: -123.5410003662,
		Timezone:  "America/Los_Angeles",
	},
	"WA17": {
		ICAO:      "WA17",
		Name:      "Majerle Strip STOLport",
		City:      "Duvall",
		State:     "Washington",
		Country:   "US",
		Elevation: 75,
		Latitude:  47.7552986145,
		Longitude: -121.9820022583,
		Timezone:  "America/Los_Angeles",
	},
	"WA18": {
		ICAO:      "WA18",
		Name:      "Decatur /Jones/ Airport",
		City:      "Decatur Island",
		State:     "Washington",
		Country:   "US",
		Elevation: 125,
		Latitude:  48.5101013184,
		Longitude: -122.8030014038,
		Timezone:  "America/Los_Angeles",
	},
	"WA1B": {
		ICAO:      "WA1B",
		Name:      "Aru Airport",
		City:      "Fekfetal / Jerdera-Trangan Island",
		Country:   "ID",
		Elevation: 165,
		Latitude:  -6.5734200478,
		Longitude: 134.1470031738,
		Timezone:  "Asia/Jayapura",
	},
	"WA20": {
		ICAO:      "WA20",
		Name:      "Gray Ranch Airport",
		City:      "Benge",
		State:     "Washington",
		Country:   "US",
		Elevation: 1625,
		Latitude:  46.8480987549,
		Longitude: -118.1770019531,
		Timezone:  "America/Los_Angeles",
	},
	"WA21": {
		ICAO:      "WA21",
		Name:      "Elk Heights Airport",
		City:      "Elk",
		State:     "Washington",
		Country:   "US",
		Elevation: 2020,
		Latitude:  48.0019989014,
		Longitude: -117.2630004883,
		Timezone:  "America/Los_Angeles",
	},
	"WA22": {
		ICAO:      "WA22",
		Name:      "Mirth Airport",
		City:      "Hansville",
		State:     "Washington",
		Country:   "US",
		Elevation: 190,
		Latitude:  47.9385986328,
		Longitude: -122.608001709,
		Timezone:  "America/Los_Angeles",
	},
	"WA23": {
		ICAO:      "WA23",
		Name:      "Pine Bluff Airport",
		City:      "Springdale",
		State:     "Washington",
		Country:   "US",
		Elevation: 2021,
		Latitude:  48.0209999084,
		Longitude: -117.7389984131,
		Timezone:  "America/Los_Angeles",
	},
	"WA24": {
		ICAO:      "WA24",
		Name:      "Pfister's Airport",
		City:      "Pasco",
		State:     "Washington",
		Country:   "US",
		Elevation: 625,
		Latitude:  46.396900177,
		Longitude: -119.021003723,
		Timezone:  "America/Los_Angeles",
	},
	"WA25": {
		ICAO:      "WA25",
		Name:      "Green Valley Airfield",
		City:      "Granite Falls",
		State:     "Washington",
		Country:   "US",
		Elevation: 300,
		Latitude:  48.0959014893,
		Longitude: -122.0159988403,
		Timezone:  "America/Los_Angeles",
	},
	"WA26": {
		ICAO:      "WA26",
		Name:      "Harrah Airport",
		City:      "Harrah",
		State:     "Washington",
		Country:   "US",
		Elevation: 845,
		Latitude:  46.4081993103,
		Longitude: -120.6350021362,
		Timezone:  "America/Los_Angeles",
	},
	"WA29": {
		ICAO:      "WA29",
		Name:      "La Center View-Air Airport",
		City:      "La Center",
		State:     "Washington",
		Country:   "US",
		Elevation: 811,
		Latitude:  45.8956985474,
		Longitude: -122.59400177,
		Timezone:  "America/Los_Angeles",
	},
	"WA30": {
		ICAO:      "WA30",
		Name:      "Lacrosse Municipal Airport",
		City:      "Lacrosse",
		State:     "Washington",
		Country:   "US",
		Elevation: 1491,
		Latitude:  46.7910003662,
		Longitude: -117.9209976196,
		Timezone:  "America/Los_Angeles",
	},
	"WA35": {
		ICAO:      "WA35",
		Name:      "Clam Harbor Airport",
		City:      "Eastsound",
		State:     "Washington",
		Country:   "US",
		Elevation: 220,
		Latitude:  48.6222991943,
		Longitude: -122.9469985962,
		Timezone:  "America/Los_Angeles",
	},
	"WA37": {
		ICAO:      "WA37",
		Name:      "Chinook Farms Airport",
		City:      "Kittitas",
		State:     "Washington",
		Country:   "US",
		Elevation: 1880,
		Latitude:  46.9103012085,
		Longitude: -120.43699646,
		Timezone:  "America/Los_Angeles",
	},
	"WA39": {
		ICAO:      "WA39",
		Name:      "Woodfield Airport",
		City:      "Kennewick",
		State:     "Washington",
		Country:   "US",
		Elevation: 445,
		Latitude:  46.1277999878,
		Longitude: -119.0179977417,
		Timezone:  "America/Los_Angeles",
	},
	"WA41": {
		ICAO:      "WA41",
		Name:      "Bear Canyon West Field",
		City:      "Morton",
		State:     "Washington",
		Country:   "US",
		Elevation: 980,
		Latitude:  46.5988998413,
		Longitude: -122.4840011597,
		Timezone:  "America/Los_Angeles",
	},
	"WA42": {
		ICAO:      "WA42",
		Name:      "Stacey's Airport",
		City:      "Port Townsend",
		State:     "Washington",
		Country:   "US",
		Elevation: 100,
		Latitude:  47.9720001221,
		Longitude: -122.7720031738,
		Timezone:  "America/Los_Angeles",
	},
	"WA44": {
		ICAO:      "WA44",
		IATA:      "TQQ",
		Name:      "Maranggo Airport",
		City:      "Waha-Tomea Island",
		State:     "Southeast-Sulawesi",
		Country:   "ID",
		Elevation: 169,
		Latitude:  -5.7645702362,
		Longitude: 123.9169998169,
		Timezone:  "Asia/Makassar",
	},
	"WA45": {
		ICAO:      "WA45",
		Name:      "Olympic Field",
		City:      "Discovery Bay/Maynard",
		State:     "Washington",
		Country:   "US",
		Elevation: 500,
		Latitude:  47.995300293,
		Longitude: -122.9000015259,
		Timezone:  "America/Los_Angeles",
	},
	"WA46": {
		ICAO:      "WA46",
		Name:      "Daybreak Airport",
		City:      "La Center",
		State:     "Washington",
		Country:   "US",
		Elevation: 25,
		Latitude:  45.8302993774,
		Longitude: -122.6370010376,
		Timezone:  "America/Los_Angeles",
	},
	"WA47": {
		ICAO:      "WA47",
		Name:      "Flying Rock Airpark",
		City:      "Ellensburg",
		State:     "Washington",
		Country:   "US",
		Elevation: 2700,
		Latitude:  47.1537017822,
		Longitude: -120.6370010376,
		Timezone:  "America/Los_Angeles",
	},
	"WA49": {
		ICAO:      "WA49",
		Name:      "Flying R Ranch Airport",
		City:      "Cheney",
		State:     "Washington",
		Country:   "US",
		Elevation: 2450,
		Latitude:  47.5027999878,
		Longitude: -117.6480026245,
		Timezone:  "America/Los_Angeles",
	},
	"WA52": {
		ICAO:      "WA52",
		Name:      "Deer Flat Airport",
		City:      "Deer Park",
		State:     "Washington",
		Country:   "US",
		Elevation: 2200,
		Latitude:  47.9569015503,
		Longitude: -117.6039962769,
		Timezone:  "America/Los_Angeles",
	},
	"WA56": {
		ICAO:      "WA56",
		Name:      "Israel's Farm Airport",
		City:      "Sedro Woolley",
		State:     "Washington",
		Country:   "US",
		Elevation: 65,
		Latitude:  48.507598877,
		Longitude: -122.0920028687,
		Timezone:  "America/Los_Angeles",
	},
	"WA57": {
		ICAO:      "WA57",
		Name:      "Bluecreek Airport",
		City:      "Addy",
		State:     "Washington",
		Country:   "US",
		Elevation: 1740,
		Latitude:  48.3138999939,
		Longitude: -117.8399963379,
		Timezone:  "America/Los_Angeles",
	},
	"WA58": {
		ICAO:      "WA58",
		Name:      "Eliza Island Airport",
		City:      "Bellingham",
		State:     "Washington",
		Country:   "US",
		Elevation: 8,
		Latitude:  48.652606,
		Longitude: -122.584617,
		Timezone:  "America/Los_Angeles",
	},
	"WA59": {
		ICAO:      "WA59",
		Name:      "Rake's Glen Airport",
		City:      "Sequim",
		State:     "Washington",
		Country:   "US",
		Elevation: 50,
		Latitude:  48.1248016357,
		Longitude: -123.1760025024,
		Timezone:  "America/Los_Angeles",
	},
	"WA61": {
		ICAO:      "WA61",
		Name:      "Thompson Airport",
		City:      "Kingston",
		State:     "Washington",
		Country:   "US",
		Elevation: 50,
		Latitude:  47.7983016968,
		Longitude: -122.5299987793,
		Timezone:  "America/Los_Angeles",
	},
	"WA62": {
		ICAO:      "WA62",
		Name:      "Paradise Air Ranch Airport",
		City:      "Spangle",
		State:     "Washington",
		Country:   "US",
		Elevation: 2352,
		Latitude:  47.4706993103,
		Longitude: -117.4189987183,
		Timezone:  "America/Los_Angeles",
	},
	"WA63": {
		ICAO:      "WA63",
		Name:      "Pleasant Farm Airport",
		City:      "Washougal",
		State:     "Washington",
		Country:   "US",
		Elevation: 540,
		Latitude:  45.562342,
		Longitude: -122.256089,
		Timezone:  "America/Los_Angeles",
	},
	"WA66": {
		ICAO:      "WA66",
		Name:      "Spring Creek Ranch Airport",
		City:      "White Salmon",
		State:     "Washington",
		Country:   "US",
		Elevation: 475,
		Latitude:  45.8054008484,
		Longitude: -121.5009994507,
		Timezone:  "America/Los_Angeles",
	},
	"WA67": {
		ICAO:      "WA67",
		Name:      "Green Mountain STOLport",
		City:      "Vancouver",
		State:     "Washington",
		Country:   "US",
		Elevation: 270,
		Latitude:  45.6631011963,
		Longitude: -122.4729995728,
		Timezone:  "America/Los_Angeles",
	},
	"WA68": {
		ICAO:      "WA68",
		Name:      "Sky Valley Airstrip",
		City:      "Port Townsend",
		State:     "Washington",
		Country:   "US",
		Elevation: 140,
		Latitude:  48.076944,
		Longitude: -122.840661,
		Timezone:  "America/Los_Angeles",
	},
	"WA69": {
		ICAO:      "WA69",
		Name:      "Wax Orchards Airport",
		City:      "Vashon",
		State:     "Washington",
		Country:   "US",
		Elevation: 389,
		Latitude:  47.4017982483,
		Longitude: -122.4980010986,
		Timezone:  "America/Los_Angeles",
	},
	"WA70": {
		ICAO:      "WA70",
		Name:      "Frosty Creek Airport",
		City:      "Tonasket",
		State:     "Washington",
		Country:   "US",
		Elevation: 3608,
		Latitude:  48.5777015686,
		Longitude: -118.9850006104,
		Timezone:  "America/Los_Angeles",
	},
	"WA72": {
		ICAO:      "WA72",
		Name:      "Zema Private Airport",
		City:      "Colville",
		State:     "Washington",
		Country:   "US",
		Elevation: 1854,
		Latitude:  48.6446990967,
		Longitude: -117.93800354,
		Timezone:  "America/Los_Angeles",
	},
	"WA73": {
		ICAO:      "WA73",
		Name:      "Smith Airport",
		City:      "Colville",
		State:     "Washington",
		Country:   "US",
		Elevation: 2240,
		Latitude:  48.5250015259,
		Longitude: -117.8529968262,
		Timezone:  "America/Los_Angeles",
	},
	"WA74": {
		ICAO:      "WA74",
		Name:      "Quincy Flying Service Airport",
		City:      "Quincy",
		State:     "Washington",
		Country:   "US",
		Elevation: 1276,
		Latitude:  47.2266998291,
		Longitude: -119.908996582,
		Timezone:  "America/Los_Angeles",
	},
	"WA76": {
		ICAO:      "WA76",
		Name:      "Columbia Crest Winery Airport",
		City:      "Paterson",
		State:     "Washington",
		Country:   "US",
		Elevation: 640,
		Latitude:  45.956199646,
		Longitude: -119.6200027466,
		Timezone:  "America/Los_Angeles",
	},
	"WA77": {
		ICAO:      "WA77",
		Name:      "Enumclaw Airport",
		City:      "Enumclaw",
		State:     "Washington",
		Country:   "US",
		Elevation: 738,
		Latitude:  47.1957015991,
		Longitude: -122.0220031738,
		Timezone:  "America/Los_Angeles",
	},
	"WA78": {
		ICAO:      "WA78",
		Name:      "Sky River Ranch Airport",
		City:      "Washougal",
		State:     "Washington",
		Country:   "US",
		Elevation: 1239,
		Latitude:  45.6278991699,
		Longitude: -122.1849975586,
		Timezone:  "America/Los_Angeles",
	},
	"WA79": {
		ICAO:      "WA79",
		Name:      "Walter Sutton's Private Strip",
		City:      "Amboy",
		State:     "Washington",
		Country:   "US",
		Elevation: 451,
		Latitude:  45.9332008362,
		Longitude: -122.4179992676,
		Timezone:  "America/Los_Angeles",
	},
	"WA80": {
		ICAO:      "WA80",
		Name:      "Mc Clellan Field",
		City:      "Amboy",
		State:     "Washington",
		Country:   "US",
		Elevation: 840,
		Latitude:  45.8942985535,
		Longitude: -122.547996521,
		Timezone:  "America/Los_Angeles",
	},
	"WA84": {
		ICAO:      "WA84",
		Name:      "Auburn Academy Airport",
		City:      "Auburn",
		State:     "Washington",
		Country:   "US",
		Elevation: 416,
		Latitude:  47.2807006836,
		Longitude: -122.1500015259,
		Timezone:  "America/Los_Angeles",
	},
	"WA87": {
		ICAO:      "WA87",
		Name:      "Parkside Airpark",
		City:      "Battle Ground",
		State:     "Washington",
		Country:   "US",
		Elevation: 275,
		Latitude:  45.8158988953,
		Longitude: -122.5530014038,
		Timezone:  "America/Los_Angeles",
	},
	"WA88": {
		ICAO:      "WA88",
		Name:      "Horse Fly Airport",
		City:      "Ferndale",
		State:     "Washington",
		Country:   "US",
		Elevation: 70,
		Latitude:  48.8928985596,
		Longitude: -122.5569992065,
		Timezone:  "America/Los_Angeles",
	},
	"WA90": {
		ICAO:      "WA90",
		Name:      "Floathaven Airstrip",
		City:      "Bellingham",
		State:     "Washington",
		Country:   "US",
		Elevation: 325,
		Latitude:  48.7332000732,
		Longitude: -122.3509979248,
		Timezone:  "America/Los_Angeles",
	},
	"WA95": {
		ICAO:      "WA95",
		Name:      "Skyqueen Airport",
		City:      "Centralia",
		State:     "Washington",
		Country:   "US",
		Elevation: 530,
		Latitude:  46.7186012268,
		Longitude: -122.9110031128,
		Timezone:  "America/Los_Angeles",
	},
	"WA96": {
		ICAO:      "WA96",
		Name:      "Leisureland Airpark",
		City:      "Bremerton",
		State:     "Washington",
		Country:   "US",
		Elevation: 430,
		Latitude:  47.6074981689,
		Longitude: -122.7740020752,
		Timezone:  "America/Los_Angeles",
	},
	"WA97": {
		ICAO:      "WA97",
		Name:      "Buena Airport",
		City:      "Buena",
		State:     "Washington",
		Country:   "US",
		Elevation: 830,
		Latitude:  46.4431991577,
		Longitude: -120.341003418,
		Timezone:  "America/Los_Angeles",
	},
	"WA98": {
		ICAO:      "WA98",
		Name:      "Kobok Airport",
		City:      "Modjid-Celebes Island",
		Country:   "ID",
		Elevation: 114,
		Latitude:  1.1018899679,
		Longitude: 127.7030029297,
		Timezone:  "Asia/Jayapura",
	},
	"WA99": {
		ICAO:      "WA99",
		Name:      "Flying K Ranch Airport",
		City:      "Castle Rock",
		State:     "Washington",
		Country:   "US",
		Elevation: 160,
		Latitude:  46.3292999268,
		Longitude: -122.8860015869,
		Timezone:  "America/Los_Angeles",
	},
	"WAAA": {
		ICAO:      "WAAA",
		IATA:      "UPG",
		Name:      "Hasanuddin International Airport",
		City:      "Ujung Pandang-Celebes Island",
		State:     "South-Sulawesi",
		Country:   "ID",
		Elevation: 47,
		Latitude:  -5.0616297722,
		Longitude: 119.5540008545,
		Timezone:  "Asia/Makassar",
	},
	"WAAG": {
		ICAO:      "WAAG",
		Name:      "Malimpung Airport",
		City:      "Malimpung-Celebes Island",
		State:     "South-Sulawesi",
		Country:   "ID",
		Elevation: 124,
		Latitude:  -3.7269439697,
		Longitude: 119.7363891602,
		Timezone:  "Asia/Makassar",
	},
	"WAAL": {
		ICAO:      "WAAL",
		Name:      "Ponggaluku Airport",
		City:      "Wawooru-Celebes Island",
		State:     "Southeast-Sulawesi",
		Country:   "ID",
		Elevation: 43,
		Latitude:  -4.3166670799,
		Longitude: 122.4666671753,
		Timezone:  "Asia/Makassar",
	},
	"WAAP": {
		ICAO:      "WAAP",
		Name:      "Kolaka Airport",
		City:      "Kolaka-Celebes Island",
		State:     "Southeast-Sulawesi",
		Country:   "ID",
		Elevation: 49,
		Latitude:  -4.0637698174,
		Longitude: 121.6240005493,
		Timezone:  "Asia/Makassar",
	},
	"WAAT": {
		ICAO:      "WAAT",
		Name:      "Makale Airport",
		City:      "Makale-Celebes Island",
		State:     "South-Sulawesi",
		Country:   "ID",
		Elevation: 2711,
		Latitude:  -3.1093599796,
		Longitude: 119.856002808,
		Timezone:  "Asia/Makassar",
	},
	"WABB": {
		ICAO:      "WABB",
		IATA:      "BIK",
		Name:      "Frans Kaisiepo Airport",
		City:      "Biak-Supiori Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 46,
		Latitude:  -1.1900199652,
		Longitude: 136.108001709,
		Timezone:  "Asia/Jayapura",
	},
	"WABD": {
		ICAO:      "WABD",
		IATA:      "ONI",
		Name:      "Moanamani Airport",
		City:      "Moanamani-Papua Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 5204,
		Latitude:  -3.9834020138,
		Longitude: 136.0832977295,
		Timezone:  "Asia/Jayapura",
	},
	"WABF": {
		ICAO:      "WABF",
		IATA:      "FOO",
		Name:      "Kornasoren Airfield",
		City:      "Kornasoren-Numfoor Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 13,
		Latitude:  -0.9363250136,
		Longitude: 134.871994019,
		Timezone:  "Asia/Jayapura",
	},
	"WABG": {
		ICAO:      "WABG",
		IATA:      "WET",
		Name:      "Wagethe Airport",
		City:      "Wagethe-Papua Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 5627,
		Latitude:  -4.0442299843,
		Longitude: 136.2779998779,
		Timezone:  "Asia/Jayapura",
	},
	"WABI": {
		ICAO:      "WABI",
		IATA:      "NBX",
		Name:      "Nabire Airport",
		City:      "Nabire-Papua Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 20,
		Latitude:  -3.3681800365,
		Longitude: 135.4960021973,
		Timezone:  "Asia/Jayapura",
	},
	"WABL": {
		ICAO:      "WABL",
		IATA:      "ILA",
		Name:      "Illaga Airport",
		City:      "Illaga-Papua Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 6307,
		Latitude:  -3.8332800865,
		Longitude: 137.5829925537,
		Timezone:  "Asia/Jayapura",
	},
	"WABN": {
		ICAO:      "WABN",
		IATA:      "KOX",
		Name:      "Kokonau Airport",
		City:      "Kokonau-Papua Island",
		Country:   "ID",
		Elevation: 9,
		Latitude:  -4.7166671753,
		Longitude: 136.4333343506,
		Timezone:  "Asia/Jayapura",
	},
	"WABO": {
		ICAO:      "WABO",
		IATA:      "ZRI",
		Name:      "Serui Airport",
		City:      "Serui-Japen Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 50,
		Latitude:  -1.8755799532,
		Longitude: 136.2409973145,
		Timezone:  "Asia/Jayapura",
	},
	"WABP": {
		ICAO:      "WABP",
		IATA:      "TIM",
		Name:      "Moses Kilangin Airport",
		City:      "Timika-Papua Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 103,
		Latitude:  -4.5282797813,
		Longitude: 136.8869934082,
		Timezone:  "Asia/Jayapura",
	},
	"WABT": {
		ICAO:      "WABT",
		IATA:      "EWI",
		Name:      "Enarotali Airport",
		City:      "Enarotali-Papua Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 5807,
		Latitude:  -3.9258999825,
		Longitude: 136.3769989014,
		Timezone:  "Asia/Jayapura",
	},
	"WABU": {
		ICAO:      "WABU",
		Name:      "Biak Manuhua Airport",
		City:      "Biak-Supiori Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 33,
		Latitude:  -1.1735899448,
		Longitude: 136.0800018311,
		Timezone:  "Asia/Jayapura",
	},
	"WABW": {
		ICAO:      "WABW",
		Name:      "Waren Airport",
		City:      "Celebes Island",
		State:     "West-Papua",
		Country:   "ID",
		Elevation: 29,
		Latitude:  -1.6285300255,
		Longitude: 134.1239929199,
		Timezone:  "Asia/Jayapura",
	},
	"WADA": {
		ICAO:      "WADA",
		IATA:      "AMI",
		Name:      "Selaparang Airport",
		City:      "Mataram-Lombok Island",
		State:     "West-Nusa-Tenggara",
		Country:   "ID",
		Elevation: 52,
		Latitude:  -8.5607099533,
		Longitude: 116.095001221,
		Timezone:  "Asia/Makassar",
	},
	"WADB": {
		ICAO:      "WADB",
		IATA:      "BMU",
		Name:      "Muhammad Salahuddin Airport",
		City:      "Bima-Sumbawa Island",
		State:     "West-Nusa-Tenggara",
		Country:   "ID",
		Elevation: 3,
		Latitude:  -8.5396499634,
		Longitude: 118.68699646,
		Timezone:  "Asia/Makassar",
	},
	"WADD": {
		ICAO:      "WADD",
		IATA:      "DPS",
		Name:      "Ngurah Rai (Bali) International Airport",
		City:      "Denpasar-Bali Island",
		State:     "Bali",
		Country:   "ID",
		Elevation: 14,
		Latitude:  -8.748169899,
		Longitude: 115.1669998169,
		Timezone:  "Asia/Makassar",
	},
	"WADL": {
		ICAO:      "WADL",
		IATA:      "LOP",
		Name:      "Bandara International Lombok Airport",
		City:      "Mataram",
		State:     "West-Nusa-Tenggara",
		Country:   "ID",
		Elevation: 319,
		Latitude:  -8.757322,
		Longitude: 116.276675,
		Timezone:  "Asia/Makassar",
	},
	"WADS": {
		ICAO:      "WADS",
		IATA:      "SWQ",
		Name:      "Sumbawa Besar Airport",
		City:      "Sumbawa Island",
		State:     "West-Nusa-Tenggara",
		Country:   "ID",
		Elevation: 16,
		Latitude:  -8.4890403748,
		Longitude: 117.4120025635,
		Timezone:  "Asia/Makassar",
	},
	"WADT": {
		ICAO:      "WADT",
		IATA:      "TMC",
		Name:      "Tambolaka Airport",
		City:      "Waikabubak-Sumba Island",
		State:     "East-Nusa-Tenggara",
		Country:   "ID",
		Elevation: 161,
		Latitude:  -9.4097204208,
		Longitude: 119.2440032959,
		Timezone:  "Asia/Makassar",
	},
	"WADW": {
		ICAO:      "WADW",
		IATA:      "WGP",
		Name:      "Waingapu Airport",
		City:      "Waingapu-Sumba Island",
		State:     "East-Nusa-Tenggara",
		Country:   "ID",
		Elevation: 33,
		Latitude:  -9.6692199707,
		Longitude: 120.302001953,
		Timezone:  "Asia/Makassar",
	},
	"WAHI": {
		ICAO:      "WAHI",
		IATA:      "YIA",
		Name:      "Yogyakarta International Airport",
		City:      "Yogyakarta",
		State:     "  Kulon Progo Regency",
		Country:   "ID",
		Elevation: 7,
		Latitude:  -7.907459,
		Longitude: 110.05448,
		Timezone:  "Asia/Jakarta",
	},
	"WAJA": {
		ICAO:      "WAJA",
		IATA:      "ARJ",
		Name:      "Arso Airport",
		City:      "Arso-Papua Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 208,
		Latitude:  -2.9333329201,
		Longitude: 140.7833404541,
		Timezone:  "Asia/Jayapura",
	},
	"WAJB": {
		ICAO:      "WAJB",
		IATA:      "BUI",
		Name:      "Bokondini Airport",
		City:      "Bokondini-Papua Island",
		Country:   "ID",
		Elevation: 9104,
		Latitude:  -3.5836451054,
		Longitude: 138.5334472656,
		Timezone:  "Asia/Jayapura",
	},
	"WAJD": {
		ICAO:      "WAJD",
		Name:      "Wakde Airport",
		City:      "Masi Masi Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 3,
		Latitude:  -1.935289979,
		Longitude: 139.0200042725,
		Timezone:  "Asia/Jayapura",
	},
	"WAJI": {
		ICAO:      "WAJI",
		IATA:      "ZRM",
		Name:      "Sarmi Airport",
		City:      "Sarmi-Papua Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 3,
		Latitude:  -1.8695499897,
		Longitude: 138.75,
		Timezone:  "Asia/Jayapura",
	},
	"WAJJ": {
		ICAO:      "WAJJ",
		IATA:      "DJJ",
		Name:      "Sentani Airport",
		City:      "Jayapura-Papua Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 289,
		Latitude:  -2.5769500732,
		Longitude: 140.5160064697,
		Timezone:  "Asia/Jayapura",
	},
	"WAJL": {
		ICAO:      "WAJL",
		IATA:      "LHI",
		Name:      "Lereh Airport",
		City:      "Lereh-Papua Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 1317,
		Latitude:  -3.1278998852,
		Longitude: 139.9429931641,
		Timezone:  "Asia/Jayapura",
	},
	"WAJM": {
		ICAO:      "WAJM",
		IATA:      "LII",
		Name:      "Mulia Airport",
		City:      "Mulia-Papua Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 6527,
		Latitude:  -3.7094700336,
		Longitude: 137.9759979248,
		Timezone:  "Asia/Jayapura",
	},
	"WAJO": {
		ICAO:      "WAJO",
		IATA:      "OKL",
		Name:      "Oksibil Airport",
		City:      "Oksibil-Papua Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 2500,
		Latitude:  -5.0910601616,
		Longitude: 140.6100006104,
		Timezone:  "Asia/Jayapura",
	},
	"WAJR": {
		ICAO:      "WAJR",
		IATA:      "WAR",
		Name:      "Waris Airport",
		City:      "Waris-Papua Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 1,
		Latitude:  -3.2224299908,
		Longitude: 140.9790039063,
		Timezone:  "Asia/Jayapura",
	},
	"WAJS": {
		ICAO:      "WAJS",
		IATA:      "SEH",
		Name:      "Senggeh Airport",
		City:      "Senggeh-Papua Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 914,
		Latitude:  -3.4333300591,
		Longitude: 140.8170013428,
		Timezone:  "Asia/Jayapura",
	},
	"WAJU": {
		ICAO:      "WAJU",
		IATA:      "UBR",
		Name:      "Ubrub Airport",
		City:      "Ubrub-Papua Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 1370,
		Latitude:  -3.666670084,
		Longitude: 140.8500061035,
		Timezone:  "Asia/Jayapura",
	},
	"WAJW": {
		ICAO:      "WAJW",
		IATA:      "WMX",
		Name:      "Wamena Airport",
		City:      "Wamena-Papua Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 5085,
		Latitude:  -4.1025099754,
		Longitude: 138.9570007324,
		Timezone:  "Asia/Jayapura",
	},
	"WAKD": {
		ICAO:      "WAKD",
		IATA:      "MDP",
		Name:      "Mindiptana Airport",
		City:      "Mindiptana-Papua Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 157,
		Latitude:  -5.75,
		Longitude: 140.3670043945,
		Timezone:  "Asia/Jayapura",
	},
	"WAKE": {
		ICAO:      "WAKE",
		IATA:      "BXD",
		Name:      "Bade Airport",
		City:      "Bade-Papua Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 59,
		Latitude:  -7.714630127,
		Longitude: 139.5749969482,
		Timezone:  "Asia/Jayapura",
	},
	"WAKG": {
		ICAO:      "WAKG",
		Name:      "Agats Airport",
		City:      "Agats-Papua Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 17,
		Latitude:  -5.5414600372,
		Longitude: 138.1333312988,
		Timezone:  "Asia/Jayapura",
	},
	"WAKK": {
		ICAO:      "WAKK",
		IATA:      "MKQ",
		Name:      "Mopah Airport",
		City:      "Merauke-Papua Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 10,
		Latitude:  -8.5202903748,
		Longitude: 140.4179992676,
		Timezone:  "Asia/Jayapura",
	},
	"WAKO": {
		ICAO:      "WAKO",
		IATA:      "OKQ",
		Name:      "Okaba Airport",
		City:      "Okaba-Papua Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 23,
		Latitude:  -8.0882196426,
		Longitude: 139.7189941406,
		Timezone:  "Asia/Jayapura",
	},
	"WAKP": {
		ICAO:      "WAKP",
		IATA:      "KEI",
		Name:      "Kepi Airport",
		City:      "Kepi-Papua Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 67,
		Latitude:  -6.5943698883,
		Longitude: 139.3569946289,
		Timezone:  "Asia/Jayapura",
	},
	"WAKT": {
		ICAO:      "WAKT",
		IATA:      "TMH",
		Name:      "Tanah Merah Airport",
		City:      "Tanah Merah-Papua Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 57,
		Latitude:  -6.099219799,
		Longitude: 140.2980041504,
		Timezone:  "Asia/Jayapura",
	},
	"WALE": {
		ICAO:      "WALE",
		Name:      "Melak Airport",
		City:      "Melak-Borneo Island",
		Country:   "ID",
		Elevation: 330,
		Latitude:  -0.2036110014,
		Longitude: 115.7600021362,
		Timezone:  "Asia/Makassar",
	},
	"WALG": {
		ICAO:      "WALG",
		IATA:      "TJS",
		Name:      "Tanjung Harapan Airport",
		City:      "Tanjung Selor-Borneo Island",
		State:     "East-Kalimantan",
		Country:   "ID",
		Elevation: 10,
		Latitude:  2.8364100456,
		Longitude: 117.3740005493,
		Timezone:  "Asia/Makassar",
	},
	"WALJ": {
		ICAO:      "WALJ",
		IATA:      "DTD",
		Name:      "Datadawai Airport",
		City:      "Datadawai-Borneo Island",
		State:     "East-Kalimantan",
		Country:   "ID",
		Elevation: 229,
		Latitude:  0.626,
		Longitude: 116.6006,
		Timezone:  "Asia/Makassar",
	},
	"WALK": {
		ICAO:      "WALK",
		IATA:      "BEJ",
		Name:      "Barau(Kalimaru) Airport",
		City:      "Tanjung Redep-Borneo Island",
		State:     "East-Kalimantan",
		Country:   "ID",
		Elevation: 59,
		Latitude:  2.1554999352,
		Longitude: 117.431999207,
		Timezone:  "Asia/Makassar",
	},
	"WALL": {
		ICAO:      "WALL",
		IATA:      "BPN",
		Name:      "Sepinggan International Airport",
		City:      "Balikpapan-Borneo Island",
		State:     "East-Kalimantan",
		Country:   "ID",
		Elevation: 12,
		Latitude:  -1.2682700157,
		Longitude: 116.8939971924,
		Timezone:  "Asia/Makassar",
	},
	"WALQ": {
		ICAO:      "WALQ",
		Name:      "Muara Badak Pujangan Airport",
		City:      "Muara Badak-Borneo Island",
		State:     "East-Kalimantan",
		Country:   "ID",
		Elevation: 49,
		Latitude:  -0.3060750067,
		Longitude: 117.4160003662,
		Timezone:  "Asia/Makassar",
	},
	"WALR": {
		ICAO:      "WALR",
		IATA:      "TRK",
		Name:      "Juwata Airport",
		City:      "Tarakan Island",
		State:     "East-Kalimantan",
		Country:   "ID",
		Elevation: 20,
		Latitude:  3.3266899586,
		Longitude: 117.5660018921,
		Timezone:  "Asia/Makassar",
	},
	"WALS": {
		ICAO:      "WALS",
		IATA:      "AAP",
		Name:      "Aji Pangeran Tumenggung Pranoto International Airport",
		City:      "Samarinda",
		State:     "East-Kalimantan",
		Country:   "ID",
		Elevation: 82,
		Latitude:  -0.373611,
		Longitude: 117.255556,
		Timezone:  "Asia/Makassar",
	},
	"WALT": {
		ICAO:      "WALT",
		IATA:      "TSX",
		Name:      "Tanjung Santan Airport",
		City:      "Santan-Borneo Island",
		State:     "East-Kalimantan",
		Country:   "ID",
		Elevation: 121,
		Latitude:  -0.0929730013,
		Longitude: 117.4530029297,
		Timezone:  "Asia/Makassar",
	},
	"WALV": {
		ICAO:      "WALV",
		IATA:      "BYQ",
		Name:      "Bunyu Airport",
		City:      "Bunju Island",
		State:     "North-Kalimantan",
		Country:   "ID",
		Elevation: 118,
		Latitude:  3.4557199478,
		Longitude: 117.866996765,
		Timezone:  "Asia/Makassar",
	},
	"WALX": {
		ICAO:      "WALX",
		Name:      "Mangkajang Airport",
		City:      "Mangkajang-Borneo Island",
		State:     "East-Kalimantan",
		Country:   "ID",
		Elevation: 47,
		Latitude:  2.0075099468,
		Longitude: 117.7490005493,
		Timezone:  "Asia/Makassar",
	},
	"WAMA": {
		ICAO:      "WAMA",
		IATA:      "GLX",
		Name:      "Gamarmalamo Airport",
		City:      "Galela-Celebes Island",
		State:     "North-Maluku",
		Country:   "ID",
		Elevation: 180,
		Latitude:  1.8383300304,
		Longitude: 127.7860031128,
		Timezone:  "Asia/Jayapura",
	},
	"WAMB": {
		ICAO:      "WAMB",
		Name:      "Kotamubagu Airport",
		City:      "Kotamubagu-Celebes Island",
		State:     "North-Sulawesi",
		Country:   "ID",
		Elevation: 752,
		Latitude:  0.7288889885,
		Longitude: 124.3088912964,
		Timezone:  "Asia/Makassar",
	},
	"WAMC": {
		ICAO:      "WAMC",
		Name:      "Tentena Airport",
		City:      "Tentena-Celebes Island",
		Country:   "ID",
		Elevation: 1900,
		Latitude:  -2,
		Longitude: 120.7666702271,
		Timezone:  "Asia/Makassar",
	},
	"WAMD": {
		ICAO:      "WAMD",
		Name:      "Jailolo/Kuripasai Airport",
		City:      "Kuripasai-Celebes Island",
		State:     "North-Maluku",
		Country:   "ID",
		Elevation: 50,
		Latitude:  1.1169999838,
		Longitude: 127.483001709,
		Timezone:  "Asia/Jayapura",
	},
	"WAMG": {
		ICAO:      "WAMG",
		IATA:      "GTO",
		Name:      "Jalaluddin Airport",
		City:      "Gorontalo-Celebes Island",
		State:     "Gorontalo",
		Country:   "ID",
		Elevation: 105,
		Latitude:  0.6371189952,
		Longitude: 122.8499984741,
		Timezone:  "Asia/Makassar",
	},
	"WAMH": {
		ICAO:      "WAMH",
		IATA:      "NAH",
		Name:      "Naha Airport",
		City:      "Tahuna-Sangihe Island",
		State:     "North-Sulawesi",
		Country:   "ID",
		Elevation: 16,
		Latitude:  3.6832098961,
		Longitude: 125.5279998779,
		Timezone:  "Asia/Makassar",
	},
	"WAMI": {
		ICAO:      "WAMI",
		IATA:      "TLI",
		Name:      "Toli Toli Airport",
		City:      "Toli Toli-Celebes Island",
		Country:   "ID",
		Elevation: 40,
		Latitude:  -1.0297700167,
		Longitude: 120.8170013428,
		Timezone:  "Asia/Makassar",
	},
	"WAMJ": {
		ICAO:      "WAMJ",
		IATA:      "GEB",
		Name:      "Gebe Airport",
		City:      "Gebe Island",
		State:     "Maluku",
		Country:   "ID",
		Elevation: 125,
		Latitude:  -0.0788889974,
		Longitude: 129.4579925537,
		Timezone:  "Asia/Jayapura",
	},
	"WAMK": {
		ICAO:      "WAMK",
		IATA:      "KAZ",
		Name:      "Kao Airport",
		City:      "Kao-Celebes Island",
		State:     "North-Maluku",
		Country:   "ID",
		Elevation: 27,
		Latitude:  1.1852799654,
		Longitude: 127.8960037231,
		Timezone:  "Asia/Jayapura",
	},
	"WAML": {
		ICAO:      "WAML",
		IATA:      "PLW",
		Name:      "Mutiara Airport",
		City:      "Palu-Celebes Island",
		State:     "Central-Sulawesi",
		Country:   "ID",
		Elevation: 284,
		Latitude:  -0.9185420275,
		Longitude: 119.9100036621,
		Timezone:  "Asia/Makassar",
	},
	"WAMM": {
		ICAO:      "WAMM",
		IATA:      "MDC",
		Name:      "Sam Ratulangi Airport",
		City:      "Manado-Celebes Island",
		State:     "North-Sulawesi",
		Country:   "ID",
		Elevation: 264,
		Latitude:  1.5492600203,
		Longitude: 124.9260025024,
		Timezone:  "Asia/Makassar",
	},
	"WAMN": {
		ICAO:      "WAMN",
		IATA:      "MNA",
		Name:      "Melangguane Airport",
		City:      "Karakelong Island",
		State:     "North-Sulawesi",
		Country:   "ID",
		Elevation: 3,
		Latitude:  4.006939888,
		Longitude: 126.672996521,
		Timezone:  "Asia/Makassar",
	},
	"WAMP": {
		ICAO:      "WAMP",
		IATA:      "PSJ",
		Name:      "Kasiguncu Airport",
		City:      "Poso-Celebes Island",
		State:     "Central-Sulawesi",
		Country:   "ID",
		Elevation: 174,
		Latitude:  -1.4167499542,
		Longitude: 120.657997131,
		Timezone:  "Asia/Makassar",
	},
	"WAMR": {
		ICAO:      "WAMR",
		IATA:      "OTI",
		Name:      "Pitu Airport",
		City:      "Gotalalamo-Morotai Island",
		State:     "Maluku",
		Country:   "ID",
		Elevation: 49,
		Latitude:  2.0459899902,
		Longitude: 128.324996948,
		Timezone:  "Asia/Jayapura",
	},
	"WAMT": {
		ICAO:      "WAMT",
		IATA:      "TTE",
		Name:      "Sultan Khairun Babullah Airport",
		City:      "Sango-Ternate Island",
		State:     "North-Maluku",
		Country:   "ID",
		Elevation: 49,
		Latitude:  0.8314139843,
		Longitude: 127.3809967041,
		Timezone:  "Asia/Jayapura",
	},
	"WAMU": {
		ICAO:      "WAMU",
		Name:      "Wuasa Airport",
		City:      "Wuasa-Celebes Island",
		State:     "Central-Sulawesi",
		Country:   "ID",
		Elevation: 1,
		Latitude:  -1.4249999523,
		Longitude: 120.3163909912,
		Timezone:  "Asia/Makassar",
	},
	"WAMW": {
		ICAO:      "WAMW",
		IATA:      "LUW",
		Name:      "Bubung Airport",
		City:      "Luwok-Celebes Island",
		State:     "Central-Sulawesi",
		Country:   "ID",
		Elevation: 56,
		Latitude:  -1.0389200449,
		Longitude: 122.7720031738,
		Timezone:  "Asia/Makassar",
	},
	"WAMY": {
		ICAO:      "WAMY",
		IATA:      "UOL",
		Name:      "Buol Airport",
		City:      "Buol-Celebes Island",
		State:     "Central-Sulawesi",
		Country:   "ID",
		Elevation: 5,
		Latitude:  1.1590499878,
		Longitude: 121.4280014038,
		Timezone:  "Asia/Makassar",
	},
	"WAOC": {
		ICAO:      "WAOC",
		IATA:      "BTW",
		Name:      "Batu Licin Airport",
		City:      "Batu Licin-Borneo Island",
		State:     "South-Kalimantan",
		Country:   "ID",
		Elevation: 3,
		Latitude:  -3.4124100208,
		Longitude: 115.995002747,
		Timezone:  "Asia/Makassar",
	},
	"WAOH": {
		ICAO:      "WAOH",
		Name:      "Mekar Putih Airport",
		City:      "Mekar Putih-Laut Island",
		State:     "South-Kalimantan",
		Country:   "ID",
		Elevation: 197,
		Latitude:  -3.9899098873,
		Longitude: 116.1019973755,
		Timezone:  "Asia/Makassar",
	},
	"WAOI": {
		ICAO:      "WAOI",
		IATA:      "PKN",
		Name:      "Iskandar Airport",
		City:      "Pangkalanbun-Borneo Island",
		State:     "Central-Kalimantan",
		Country:   "ID",
		Elevation: 75,
		Latitude:  -2.7051999569,
		Longitude: 111.672996521,
		Timezone:  "Asia/Pontianak",
	},
	"WAOK": {
		ICAO:      "WAOK",
		IATA:      "KBU",
		Name:      "Stagen Airport",
		City:      "Laut Island",
		State:     "South-Kalimantan",
		Country:   "ID",
		Elevation: 4,
		Latitude:  -3.2947199345,
		Longitude: 116.1650009155,
		Timezone:  "Asia/Makassar",
	},
	"WAOL": {
		ICAO:      "WAOL",
		Name:      "Kuala Pembuang Airport",
		City:      "Kuala Pembuang",
		Country:   "ID",
		Elevation: 13,
		Latitude:  -2.8666698933,
		Longitude: 112.5329971313,
		Timezone:  "Asia/Pontianak",
	},
	"WAOM": {
		ICAO:      "WAOM",
		IATA:      "MTW",
		Name:      "Beringin Airport",
		City:      "Murateweh-Borneo Island",
		State:     "Central-Kalimantan",
		Country:   "ID",
		Elevation: 126,
		Latitude:  -0.9403250217,
		Longitude: 114.8938751221,
		Timezone:  "Asia/Pontianak",
	},
	"WAON": {
		ICAO:      "WAON",
		IATA:      "TJG",
		Name:      "Warukin Airport",
		City:      "Tanta-Tabalong-Borneo Island",
		State:     "South-Kalimantan",
		Country:   "ID",
		Elevation: 197,
		Latitude:  -2.2165598869,
		Longitude: 115.435997009,
		Timezone:  "Asia/Makassar",
	},
	"WAOO": {
		ICAO:      "WAOO",
		IATA:      "BDJ",
		Name:      "Syamsudin Noor Airport",
		City:      "Banjarmasin-Borneo Island",
		State:     "South-Kalimantan",
		Country:   "ID",
		Elevation: 66,
		Latitude:  -3.4423599243,
		Longitude: 114.7630004883,
		Timezone:  "Asia/Makassar",
	},
	"WAOP": {
		ICAO:      "WAOP",
		IATA:      "PKY",
		Name:      "Tjilik Riwut Airport",
		City:      "Palangkaraya-Kalimantan Tengah",
		State:     "Central-Kalimantan",
		Country:   "ID",
		Elevation: 82,
		Latitude:  -2.2251300812,
		Longitude: 113.9430007935,
		Timezone:  "Asia/Pontianak",
	},
	"WAOS": {
		ICAO:      "WAOS",
		IATA:      "SMQ",
		Name:      "Sampit(Hasan) Airport",
		City:      "Sampit-Borneo Island",
		State:     "Central-Kalimantan",
		Country:   "ID",
		Elevation: 50,
		Latitude:  -2.4991900921,
		Longitude: 112.9749984741,
		Timezone:  "Asia/Pontianak",
	},
	"WAOU": {
		ICAO:      "WAOU",
		Name:      "SANGGU Airport Buntok",
		City:      "Buntok",
		Country:   "ID",
		Elevation: 23,
		Latitude:  -1.6692999601,
		Longitude: 114.8980026245,
		Timezone:  "Asia/Pontianak",
	},
	"WAPA": {
		ICAO:      "WAPA",
		IATA:      "AHI",
		Name:      "Amahai Airport",
		City:      "Amahai-Seram Island",
		State:     "Maluku",
		Country:   "ID",
		Elevation: 29,
		Latitude:  -3.3480000496,
		Longitude: 128.925994873,
		Timezone:  "Asia/Jayapura",
	},
	"WAPB": {
		ICAO:      "WAPB",
		Name:      "Bula Airport",
		City:      "Bula-Seram Island",
		Country:   "ID",
		Elevation: 10,
		Latitude:  -3.1099600792,
		Longitude: 130.5099945068,
		Timezone:  "Asia/Jayapura",
	},
	"WAPC": {
		ICAO:      "WAPC",
		IATA:      "NDA",
		Name:      "Banda Airport Kepulauan",
		City:      "Banda Island",
		State:     "Aceh",
		Country:   "ID",
		Elevation: 49,
		Latitude:  5.1166701317,
		Longitude: 97.1166992188,
		Timezone:  "Asia/Jakarta",
	},
	"WAPD": {
		ICAO:      "WAPD",
		IATA:      "DOB",
		Name:      "Dobo Airport",
		City:      "Dobo-Kobror Island",
		State:     "Maluku",
		Country:   "ID",
		Elevation: 15,
		Latitude:  -5.7722201347,
		Longitude: 134.2120056152,
		Timezone:  "Asia/Jayapura",
	},
	"WAPE": {
		ICAO:      "WAPE",
		IATA:      "MAL",
		Name:      "Mangole Airport Falabisahaya",
		City:      "Mangole Island",
		State:     "North-Maluku",
		Country:   "ID",
		Elevation: 889,
		Latitude:  -1.87579,
		Longitude: 125.8300018311,
		Timezone:  "Asia/Jayapura",
	},
	"WAPG": {
		ICAO:      "WAPG",
		IATA:      "NRE",
		Name:      "Namrole Airport",
		City:      "Namrole-Buru Island",
		State:     "Maluku",
		Country:   "ID",
		Elevation: 997,
		Latitude:  -3.8208100796,
		Longitude: 126.7180023193,
		Timezone:  "Asia/Jayapura",
	},
	"WAPH": {
		ICAO:      "WAPH",
		IATA:      "LAH",
		Name:      "Oesman Sadik Airport Labuha",
		City:      "Labuha-Halmahera Island",
		State:     "North-Maluku",
		Country:   "ID",
		Elevation: 49,
		Latitude:  -0.6352589726,
		Longitude: 127.5019989014,
		Timezone:  "Asia/Jayapura",
	},
	"WAPI": {
		ICAO:      "WAPI",
		IATA:      "SXK",
		Name:      "Saumlaki Airport",
		City:      "Saumlaki-Yamdena Island",
		State:     "Maluku",
		Country:   "ID",
		Elevation: 218,
		Latitude:  -7.9886097908,
		Longitude: 131.3059997559,
		Timezone:  "Asia/Jayapura",
	},
	"WAPK": {
		ICAO:      "WAPK",
		IATA:      "BJK",
		Name:      "Nangasuri Airport",
		City:      "Maikoor Island",
		State:     "Maluku",
		Country:   "ID",
		Elevation: 82,
		Latitude:  -6.0661997795,
		Longitude: 134.2740020752,
		Timezone:  "Asia/Jayapura",
	},
	"WAPL": {
		ICAO:      "WAPL",
		IATA:      "LUV",
		Name:      "Dumatumbun Airport",
		City:      "Langgur-Seram Island",
		State:     "Maluku",
		Country:   "ID",
		Elevation: 10,
		Latitude:  -5.6616201401,
		Longitude: 132.7310028076,
		Timezone:  "Asia/Jayapura",
	},
	"WAPN": {
		ICAO:      "WAPN",
		IATA:      "SQN",
		Name:      "Emalamo Sanana Airport",
		City:      "Sanana-Seram Island",
		State:     "North-Maluku",
		Country:   "ID",
		Elevation: 39,
		Latitude:  -2.080509901,
		Longitude: 125.9670028687,
		Timezone:  "Asia/Jayapura",
	},
	"WAPO": {
		ICAO:      "WAPO",
		Name:      "Larat Kepulauan Larat",
		City:      "Kepulauan Larat",
		State:     "Maluku",
		Country:   "ID",
		Elevation: 80,
		Latitude:  -7.1333298683,
		Longitude: 131.7830047607,
		Timezone:  "Asia/Jayapura",
	},
	"WAPP": {
		ICAO:      "WAPP",
		IATA:      "AMQ",
		Name:      "Pattimura Airport Ambon",
		City:      "Ambon",
		State:     "Maluku",
		Country:   "ID",
		Elevation: 33,
		Latitude:  -3.7102599144,
		Longitude: 128.0890045166,
		Timezone:  "Asia/Jayapura",
	},
	"WAPQ": {
		ICAO:      "WAPQ",
		Name:      "Kisar Airport",
		City:      "Kisar-Kisar Island",
		Country:   "ID",
		Elevation: 6,
		Latitude:  -8.0184898376,
		Longitude: 127.1969985962,
		Timezone:  "Asia/Jayapura",
	},
	"WAPR": {
		ICAO:      "WAPR",
		IATA:      "NAM",
		Name:      "Namlea Airport",
		City:      "Namlea-Buru Island",
		State:     "Maluku",
		Country:   "ID",
		Elevation: 41,
		Latitude:  -3.2355699539,
		Longitude: 127.0999984741,
		Timezone:  "Asia/Jayapura",
	},
	"WAPT": {
		ICAO:      "WAPT",
		IATA:      "TAX",
		Name:      "Taliabu Island Airport",
		City:      "Tikong-Taliabu Island",
		State:     "North-Maluku",
		Country:   "ID",
		Elevation: 9,
		Latitude:  -1.642629981,
		Longitude: 124.5589981079,
		Timezone:  "Asia/Jayapura",
	},
	"WAPV": {
		ICAO:      "WAPV",
		IATA:      "WBA",
		Name:      "WahaiSeram Island",
		City:      "Seram Island",
		State:     "Maluku",
		Country:   "ID",
		Elevation: 551,
		Latitude:  -2.8141698837,
		Longitude: 129.4819946289,
		Timezone:  "Asia/Jayapura",
	},
	"WARA": {
		ICAO:      "WARA",
		IATA:      "MLG",
		Name:      "Abdul Rachman Saleh Airport",
		City:      "Malang-Java Island",
		State:     "East-Java",
		Country:   "ID",
		Elevation: 1726,
		Latitude:  -7.9265599251,
		Longitude: 112.714996338,
		Timezone:  "Asia/Jakarta",
	},
	"WARC": {
		ICAO:      "WARC",
		IATA:      "CPF",
		Name:      "Cepu Airport",
		City:      "Tjepu-Java Island",
		State:     "Central-Java",
		Country:   "ID",
		Elevation: 131,
		Latitude:  -7.1948416667,
		Longitude: 111.548166667,
		Timezone:  "Asia/Jakarta",
	},
	"WARI": {
		ICAO:      "WARI",
		Name:      "Iswahyudi Airport",
		City:      "Madiun-Java Island",
		State:     "East-Java",
		Country:   "ID",
		Elevation: 361,
		Latitude:  -7.6157698631,
		Longitude: 111.4339981079,
		Timezone:  "Asia/Jakarta",
	},
	"WARJ": {
		ICAO:      "WARJ",
		IATA:      "JOG",
		Name:      "Adi Sutjipto International Airport",
		City:      "Yogyakarta-Java Island",
		State:     "Yogyakarta",
		Country:   "ID",
		Elevation: 350,
		Latitude:  -7.7881798744,
		Longitude: 110.4319992065,
		Timezone:  "Asia/Jakarta",
	},
	"WARQ": {
		ICAO:      "WARQ",
		IATA:      "SOC",
		Name:      "Adi Sumarmo Wiryokusumo Airport",
		City:      "Sukarata(Solo)-Java Island",
		State:     "Central-Java",
		Country:   "ID",
		Elevation: 421,
		Latitude:  -7.5160899162,
		Longitude: 110.7570037842,
		Timezone:  "Asia/Jakarta",
	},
	"WARR": {
		ICAO:      "WARR",
		IATA:      "SUB",
		Name:      "Juanda International Airport",
		City:      "Surabaya",
		State:     "East-Java",
		Country:   "ID",
		Elevation: 9,
		Latitude:  -7.3798298836,
		Longitude: 112.7870025635,
		Timezone:  "Asia/Jakarta",
	},
	"WARS": {
		ICAO:      "WARS",
		IATA:      "SRG",
		Name:      "Achmad Yani Airport",
		City:      "Semarang-Java Island",
		State:     "Central-Java",
		Country:   "ID",
		Elevation: 10,
		Latitude:  -6.9727301598,
		Longitude: 110.375,
		Timezone:  "Asia/Jakarta",
	},
	"WART": {
		ICAO:      "WART",
		IATA:      "SUP",
		Name:      "Trunojoyo Airport",
		City:      "Sumenep-Madura Island",
		State:     "East-Java",
		Country:   "ID",
		Elevation: 10,
		Latitude:  -7.02425,
		Longitude: 113.890205556,
		Timezone:  "Asia/Jakarta",
	},
	"WARU": {
		ICAO:      "WARU",
		Name:      "Dewadaru - Kemujan Island",
		Country:   "ID",
		Latitude:  -5.8111109734,
		Longitude: 110.4777755737,
		Timezone:  "Asia/Jakarta",
	},
	"WASB": {
		ICAO:      "WASB",
		IATA:      "NTI",
		Name:      "Stenkol Airport",
		City:      "Bintuni-Papua Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 57,
		Latitude:  -2.1041600704,
		Longitude: 133.5220031738,
		Timezone:  "Asia/Jayapura",
	},
	"WASC": {
		ICAO:      "WASC",
		IATA:      "RSK",
		Name:      "Abresso Airport",
		City:      "Ransiki-Papua Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 52,
		Latitude:  -1.5084500313,
		Longitude: 134.18699646,
		Timezone:  "Asia/Jayapura",
	},
	"WASE": {
		ICAO:      "WASE",
		IATA:      "KEQ",
		Name:      "Kebar Airport",
		City:      "Kebar-Papua Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 2050,
		Latitude:  -0.6371009946,
		Longitude: 133.1280059814,
		Timezone:  "Asia/Jayapura",
	},
	"WASF": {
		ICAO:      "WASF",
		IATA:      "FKQ",
		Name:      "Fakfak Airport",
		City:      "Fakfak-Papua Island",
		Country:   "ID",
		Elevation: 462,
		Latitude:  -2.9201900959,
		Longitude: 132.266998291,
		Timezone:  "Asia/Jayapura",
	},
	"WASI": {
		ICAO:      "WASI",
		IATA:      "INX",
		Name:      "Inanwatan Airport",
		City:      "Inanwatan Airport-Papua Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 9,
		Latitude:  -2.1280999184,
		Longitude: 132.1609954834,
		Timezone:  "Asia/Jayapura",
	},
	"WASK": {
		ICAO:      "WASK",
		IATA:      "KNG",
		Name:      "Kaimana Airport",
		City:      "Kaimana-Papua Island",
		Country:   "ID",
		Elevation: 19,
		Latitude:  -3.6445200443,
		Longitude: 133.6959991455,
		Timezone:  "Asia/Jayapura",
	},
	"WASM": {
		ICAO:      "WASM",
		IATA:      "RDE",
		Name:      "Merdei Airport",
		City:      "Merdei-Papua Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 425,
		Latitude:  -1.5833300352,
		Longitude: 133.3329925537,
		Timezone:  "Asia/Jayapura",
	},
	"WASO": {
		ICAO:      "WASO",
		IATA:      "BXB",
		Name:      "Babo Airport",
		City:      "Babo-Papua Island",
		State:     "West-Papua",
		Country:   "ID",
		Elevation: 10,
		Latitude:  -2.5322399139,
		Longitude: 133.4389953613,
		Timezone:  "Asia/Jayapura",
	},
	"WASR": {
		ICAO:      "WASR",
		IATA:      "MKW",
		Name:      "Rendani Airport",
		City:      "Manokwari-Papua Island",
		State:     "West-Papua",
		Country:   "ID",
		Elevation: 23,
		Latitude:  -0.8918330073,
		Longitude: 134.0489959717,
		Timezone:  "Asia/Jayapura",
	},
	"WASS": {
		ICAO:      "WASS",
		IATA:      "SOQ",
		Name:      "Sorong (Jefman) Airport",
		City:      "Sorong-Papua Island",
		State:     "West-Papua",
		Country:   "ID",
		Elevation: 10,
		Latitude:  -0.9263579845,
		Longitude: 131.1210021973,
		Timezone:  "Asia/Jayapura",
	},
	"WAST": {
		ICAO:      "WAST",
		IATA:      "TXM",
		Name:      "Teminabuan Airport",
		City:      "Atinjoe-Papua Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 141,
		Latitude:  -1.4447200298,
		Longitude: 132.0209960938,
		Timezone:  "Asia/Jayapura",
	},
	"WASW": {
		ICAO:      "WASW",
		IATA:      "WSR",
		Name:      "Wasior Airport",
		City:      "Wasior-Papua Island",
		Country:   "ID",
		Elevation: 565,
		Latitude:  -2.7335898876,
		Longitude: 134.5180053711,
		Timezone:  "Asia/Jayapura",
	},
	"WATB": {
		ICAO:      "WATB",
		IATA:      "BJW",
		Name:      "Bajawa Soa Airport",
		City:      "Bajawa",
		State:     "East-Nusa-Tenggara",
		Country:   "ID",
		Elevation: 4326,
		Latitude:  -8.7074349801,
		Longitude: 121.057426929,
		Timezone:  "Asia/Makassar",
	},
	"WATC": {
		ICAO:      "WATC",
		IATA:      "MOF",
		Name:      "Maumere(Wai Oti) Airport",
		City:      "Maumere-Flores Island",
		State:     "East-Nusa-Tenggara",
		Country:   "ID",
		Elevation: 115,
		Latitude:  -8.6406497955,
		Longitude: 122.236999512,
		Timezone:  "Asia/Makassar",
	},
	"WATE": {
		ICAO:      "WATE",
		IATA:      "ENE",
		Name:      "Ende (H Hasan Aroeboesman) Airport",
		City:      "Ende-Flores Island",
		State:     "East-Nusa-Tenggara",
		Country:   "ID",
		Elevation: 49,
		Latitude:  -8.8492898941,
		Longitude: 121.661003113,
		Timezone:  "Asia/Makassar",
	},
	"WATG": {
		ICAO:      "WATG",
		IATA:      "RTG",
		Name:      "Satar Tacik Airport",
		City:      "Satar Tacik-Flores Island",
		State:     "East-Nusa-Tenggara",
		Country:   "ID",
		Elevation: 3510,
		Latitude:  -8.5970096588,
		Longitude: 120.4769973755,
		Timezone:  "Asia/Makassar",
	},
	"WATM": {
		ICAO:      "WATM",
		IATA:      "ARD",
		Name:      "Mali Airport",
		City:      "Alor Island",
		State:     "East-Nusa-Tenggara",
		Country:   "ID",
		Elevation: 10,
		Latitude:  -8.1323404312,
		Longitude: 124.5970001221,
		Timezone:  "Asia/Makassar",
	},
	"WATO": {
		ICAO:      "WATO",
		IATA:      "LBJ",
		Name:      "Komodo (Mutiara II) Airport",
		City:      "Labuan Bajo-Flores Island",
		State:     "East-Nusa-Tenggara",
		Country:   "ID",
		Elevation: 66,
		Latitude:  -8.4866600037,
		Longitude: 119.888999939,
		Timezone:  "Asia/Makassar",
	},
	"WATT": {
		ICAO:      "WATT",
		IATA:      "KOE",
		Name:      "El Tari Airport",
		City:      "Kupang-Timor Island",
		State:     "East-Nusa-Tenggara",
		Country:   "ID",
		Elevation: 335,
		Latitude:  -10.1716003418,
		Longitude: 123.6709976196,
		Timezone:  "Asia/Makassar",
	},
	"WAWB": {
		ICAO:      "WAWB",
		IATA:      "BUW",
		Name:      "Betoambari Airport",
		City:      "Bau Bau-Butung Island",
		State:     "Southeast-Sulawesi",
		Country:   "ID",
		Elevation: 164,
		Latitude:  -5.4868798256,
		Longitude: 122.5690002441,
		Timezone:  "Asia/Makassar",
	},
	"WAWJ": {
		ICAO:      "WAWJ",
		IATA:      "MJU",
		Name:      "Tampa Padang Airport",
		City:      "Mamuju-Celebes Island",
		State:     "West-Sulawesi",
		Country:   "ID",
		Elevation: 49,
		Latitude:  -2.583333,
		Longitude: 119.033333,
		Timezone:  "Asia/Makassar",
	},
	"WAWM": {
		ICAO:      "WAWM",
		IATA:      "MXB",
		Name:      "Andi Jemma Airport",
		City:      "Masamba-Celebes Island",
		State:     "South-Sulawesi",
		Country:   "ID",
		Elevation: 164,
		Latitude:  -2.5580399036,
		Longitude: 120.3239974976,
		Timezone:  "Asia/Makassar",
	},
	"WAWP": {
		ICAO:      "WAWP",
		IATA:      "KXB",
		Name:      "Sangia Nibandera Airport",
		City:      "Kolaka",
		State:     "South-Sulawesi",
		Country:   "ID",
		Elevation: 40,
		Latitude:  -4.341217,
		Longitude: 121.523983,
		Timezone:  "Asia/Makassar",
	},
	"WAWS": {
		ICAO:      "WAWS",
		IATA:      "SQR",
		Name:      "Soroako Airport",
		City:      "Soroako-Celebes Island",
		State:     "South-Sulawesi",
		Country:   "ID",
		Elevation: 1388,
		Latitude:  -2.5311999321,
		Longitude: 121.358001709,
		Timezone:  "Asia/Makassar",
	},
	"WAWT": {
		ICAO:      "WAWT",
		IATA:      "TTR",
		Name:      "Pongtiku Airport",
		City:      "Tanah Toraja-Celebes Island",
		State:     "South-Sulawesi",
		Country:   "ID",
		Elevation: 2884,
		Latitude:  -3.0447399616,
		Longitude: 119.8219985962,
		Timezone:  "Asia/Makassar",
	},
	"WAWW": {
		ICAO:      "WAWW",
		IATA:      "KDI",
		Name:      "Wolter Monginsidi Airport",
		City:      "Kendari-Celebes Island",
		State:     "Southeast-Sulawesi",
		Country:   "ID",
		Elevation: 538,
		Latitude:  -4.0816102028,
		Longitude: 122.4179992676,
		Timezone:  "Asia/Makassar",
	},
	"WAXX": {
		ICAO:      "WAXX",
		Name:      "Dominique Edward Osok Airport",
		City:      "Sorong-Papua Island",
		State:     "Papua",
		Country:   "ID",
		Elevation: 10,
		Latitude:  -0.89517802,
		Longitude: 131.2850036621,
		Timezone:  "Asia/Jayapura",
	},
	"WBAK": {
		ICAO:      "WBAK",
		Name:      "Anduki Airport",
		City:      "Anduki/Seria",
		Country:   "BN",
		Elevation: 7,
		Latitude:  4.6370000839,
		Longitude: 114.382003784,
		Timezone:  "Asia/Brunei",
	},
	"WBGA": {
		ICAO:      "WBGA",
		Name:      "Long Atip Airport",
		City:      "Long Atip",
		State:     "Sarawak",
		Country:   "MY",
		Elevation: 300,
		Latitude:  3.8169999123,
		Longitude: 114.6999969482,
		Timezone:  "Asia/Kuching",
	},
	"WBGB": {
		ICAO:      "WBGB",
		IATA:      "BTU",
		Name:      "Bintulu Airport",
		City:      "Bintulu",
		State:     "Sarawak",
		Country:   "MY",
		Elevation: 74,
		Latitude:  3.1238501072,
		Longitude: 113.019996643,
		Timezone:  "Asia/Kuching",
	},
	"WBGC": {
		ICAO:      "WBGC",
		IATA:      "BLG",
		Name:      "Belaga Airport",
		City:      "Belaga",
		State:     "Sarawak",
		Country:   "MY",
		Elevation: 200,
		Latitude:  2.6500000954,
		Longitude: 113.766998291,
		Timezone:  "Asia/Kuching",
	},
	"WBGD": {
		ICAO:      "WBGD",
		IATA:      "LSM",
		Name:      "Long Semado Airport",
		City:      "Long Semado",
		State:     "Sarawak",
		Country:   "MY",
		Elevation: 2150,
		Latitude:  4.2170000076,
		Longitude: 115.599998474,
		Timezone:  "Asia/Kuching",
	},
	"WBGE": {
		ICAO:      "WBGE",
		Name:      "Long Geng Airport",
		City:      "Long Geng",
		State:     "Sarawak",
		Country:   "MY",
		Elevation: 350,
		Latitude:  2.617000103,
		Longitude: 114.1330032349,
		Timezone:  "Asia/Kuching",
	},
	"WBGF": {
		ICAO:      "WBGF",
		IATA:      "LGL",
		Name:      "Long Lellang Airport",
		City:      "Long Datih",
		State:     "Sarawak",
		Country:   "MY",
		Elevation: 1400,
		Latitude:  3.4210000038,
		Longitude: 115.1539993286,
		Timezone:  "Asia/Kuching",
	},
	"WBGG": {
		ICAO:      "WBGG",
		IATA:      "KCH",
		Name:      "Kuching International Airport",
		City:      "Kuching",
		State:     "Sarawak",
		Country:   "MY",
		Elevation: 89,
		Latitude:  1.4846999645,
		Longitude: 110.3470001221,
		Timezone:  "Asia/Kuching",
	},
	"WBGI": {
		ICAO:      "WBGI",
		IATA:      "ODN",
		Name:      "Long Seridan Airport",
		City:      "Long Seridan",
		State:     "Sarawak",
		Country:   "MY",
		Elevation: 607,
		Latitude:  3.9670000076,
		Longitude: 115.0500030518,
		Timezone:  "Asia/Kuching",
	},
	"WBGJ": {
		ICAO:      "WBGJ",
		IATA:      "LMN",
		Name:      "Limbang Airport",
		City:      "Limbang",
		State:     "Sarawak",
		Country:   "MY",
		Elevation: 14,
		Latitude:  4.8083000183,
		Longitude: 115.0100021362,
		Timezone:  "Asia/Brunei",
	},
	"WBGK": {
		ICAO:      "WBGK",
		IATA:      "MKM",
		Name:      "Mukah Airport",
		City:      "Mukah",
		State:     "Sarawak",
		Country:   "MY",
		Elevation: 13,
		Latitude:  2.9063899517,
		Longitude: 112.0800018311,
		Timezone:  "Asia/Kuching",
	},
	"WBGL": {
		ICAO:      "WBGL",
		IATA:      "LKH",
		Name:      "Long Akah Airport",
		City:      "Long Akah",
		State:     "Sarawak",
		Country:   "MY",
		Elevation: 289,
		Latitude:  3.2999999523,
		Longitude: 114.7829971313,
		Timezone:  "Asia/Kuching",
	},
	"WBGM": {
		ICAO:      "WBGM",
		IATA:      "MUR",
		Name:      "Marudi Airport",
		City:      "Marudi",
		State:     "Sarawak",
		Country:   "MY",
		Elevation: 103,
		Latitude:  4.1789798737,
		Longitude: 114.3290023804,
		Timezone:  "Asia/Kuching",
	},
	"WBGN": {
		ICAO:      "WBGN",
		IATA:      "BSE",
		Name:      "Sematan Airport",
		City:      "Sematan",
		State:     "Sarawak",
		Country:   "MY",
		Elevation: 22,
		Latitude:  1.8136099577,
		Longitude: 109.7630004883,
		Timezone:  "Asia/Kuching",
	},
	"WBGO": {
		ICAO:      "WBGO",
		Name:      "Lio Matu Airport",
		City:      "Lio Matu",
		State:     "Sarawak",
		Country:   "MY",
		Elevation: 700,
		Latitude:  3.1500000954,
		Longitude: 115.233001709,
		Timezone:  "Asia/Kuching",
	},
	"WBGP": {
		ICAO:      "WBGP",
		IATA:      "KPI",
		Name:      "Kapit Airport",
		City:      "Kapit",
		State:     "Sarawak",
		Country:   "MY",
		Elevation: 65,
		Latitude:  2.0169999599,
		Longitude: 112.9499969482,
		Timezone:  "Asia/Kuching",
	},
	"WBGQ": {
		ICAO:      "WBGQ",
		IATA:      "BKM",
		Name:      "Bakalalan Airport",
		City:      "Bakalalan",
		State:     "Sarawak",
		Country:   "MY",
		Elevation: 2900,
		Latitude:  3.9739999771,
		Longitude: 115.6179962158,
		Timezone:  "Asia/Kuching",
	},
	"WBGR": {
		ICAO:      "WBGR",
		IATA:      "MYY",
		Name:      "Miri Airport",
		City:      "Miri",
		State:     "Sarawak",
		Country:   "MY",
		Elevation: 59,
		Latitude:  4.3220100403,
		Longitude: 113.9869995117,
		Timezone:  "Asia/Kuching",
	},
	"WBGS": {
		ICAO:      "WBGS",
		IATA:      "SBW",
		Name:      "Sibu Airport",
		City:      "Sibu",
		State:     "Sarawak",
		Country:   "MY",
		Elevation: 122,
		Latitude:  2.2616000175,
		Longitude: 111.9850006104,
		Timezone:  "Asia/Kuching",
	},
	"WBGT": {
		ICAO:      "WBGT",
		IATA:      "TGC",
		Name:      "Tanjung Manis Airport",
		City:      "Tanjung Manis",
		State:     "Sarawak",
		Country:   "MY",
		Elevation: 15,
		Latitude:  2.1778399944,
		Longitude: 111.202003479,
		Timezone:  "Asia/Kuching",
	},
	"WBGU": {
		ICAO:      "WBGU",
		IATA:      "LSU",
		Name:      "Long Sukang Airport",
		City:      "Long Sukang",
		State:     "Sarawak",
		Country:   "MY",
		Elevation: 1200,
		Latitude:  4.5522198677,
		Longitude: 115.4940032959,
		Timezone:  "Asia/Kuching",
	},
	"WBGW": {
		ICAO:      "WBGW",
		IATA:      "LWY",
		Name:      "Lawas Airport",
		City:      "Lawas",
		State:     "Sarawak",
		Country:   "MY",
		Elevation: 5,
		Latitude:  4.849170208,
		Longitude: 115.4079971313,
		Timezone:  "Asia/Kuching",
	},
	"WBGY": {
		ICAO:      "WBGY",
		IATA:      "SGG",
		Name:      "Simanggang Airport",
		City:      "Simanggang",
		State:     "Sarawak",
		Country:   "MY",
		Elevation: 35,
		Latitude:  1.2170000076,
		Longitude: 111.4499969482,
		Timezone:  "Asia/Kuching",
	},
	"WBGZ": {
		ICAO:      "WBGZ",
		IATA:      "BBN",
		Name:      "Bario Airport",
		City:      "Bario",
		State:     "Sarawak",
		Country:   "MY",
		Elevation: 3350,
		Latitude:  3.7338900566,
		Longitude: 115.4789962769,
		Timezone:  "Asia/Kuching",
	},
	"WBKA": {
		ICAO:      "WBKA",
		IATA:      "SMM",
		Name:      "Semporna Airport",
		City:      "Semporna",
		State:     "Sabah",
		Country:   "MY",
		Elevation: 60,
		Latitude:  4.4499998093,
		Longitude: 118.5830001831,
		Timezone:  "Asia/Kuching",
	},
	"WBKB": {
		ICAO:      "WBKB",
		Name:      "Kota Belud Airport",
		City:      "Kota Belud",
		State:     "Sabah",
		Country:   "MY",
		Elevation: 46,
		Latitude:  6.3651599884,
		Longitude: 116.4710006714,
		Timezone:  "Asia/Kuching",
	},
	"WBKD": {
		ICAO:      "WBKD",
		IATA:      "LDU",
		Name:      "Lahad Datu Airport",
		City:      "Lahad Datu",
		State:     "Sabah",
		Country:   "MY",
		Elevation: 45,
		Latitude:  5.0322499275,
		Longitude: 118.3239974976,
		Timezone:  "Asia/Kuching",
	},
	"WBKE": {
		ICAO:      "WBKE",
		IATA:      "TEL",
		Name:      "Telupid Airport",
		City:      "Telupid",
		State:     "Sabah",
		Country:   "MY",
		Elevation: 343,
		Latitude:  5.6286101341,
		Longitude: 117.1259994507,
		Timezone:  "Asia/Kuching",
	},
	"WBKG": {
		ICAO:      "WBKG",
		IATA:      "KGU",
		Name:      "Keningau Airport",
		City:      "Keningau",
		State:     "Sabah",
		Country:   "MY",
		Elevation: 1036,
		Latitude:  5.3574900627,
		Longitude: 116.1620025635,
		Timezone:  "Asia/Kuching",
	},
	"WBKH": {
		ICAO:      "WBKH",
		IATA:      "SXS",
		Name:      "Sahabat [Sahabat 16] Airport",
		City:      "Sahabat",
		State:     "Sabah",
		Country:   "MY",
		Elevation: 40,
		Latitude:  5.0877799988,
		Longitude: 119.09400177,
		Timezone:  "Asia/Kuching",
	},
	"WBKK": {
		ICAO:      "WBKK",
		IATA:      "BKI",
		Name:      "Kota Kinabalu International Airport",
		City:      "Kota Kinabalu",
		State:     "Sabah",
		Country:   "MY",
		Elevation: 10,
		Latitude:  5.937210083,
		Longitude: 116.0510025024,
		Timezone:  "Asia/Kuching",
	},
	"WBKL": {
		ICAO:      "WBKL",
		IATA:      "LBU",
		Name:      "Labuan Airport",
		City:      "Labuan",
		State:     "Labuan",
		Country:   "MY",
		Elevation: 101,
		Latitude:  5.3006801605,
		Longitude: 115.25,
		Timezone:  "Asia/Kuching",
	},
	"WBKM": {
		ICAO:      "WBKM",
		IATA:      "TMG",
		Name:      "Tomanggong Airport",
		City:      "Tomanggong",
		State:     "Sabah",
		Country:   "MY",
		Elevation: 26,
		Latitude:  5.4000000954,
		Longitude: 118.6500015259,
		Timezone:  "Asia/Kuching",
	},
	"WBKN": {
		ICAO:      "WBKN",
		IATA:      "GSA",
		Name:      "Long Pasia Airport",
		City:      "Long Miau",
		State:     "Sabah",
		Country:   "MY",
		Elevation: 3175,
		Latitude:  4.4000000954,
		Longitude: 115.7170028687,
		Timezone:  "Asia/Kuching",
	},
	"WBKO": {
		ICAO:      "WBKO",
		IATA:      "SPE",
		Name:      "Sepulot Airport",
		City:      "Sepulot",
		State:     "Sabah",
		Country:   "MY",
		Elevation: 800,
		Latitude:  4.7329998016,
		Longitude: 116.4670028687,
		Timezone:  "Asia/Kuching",
	},
	"WBKP": {
		ICAO:      "WBKP",
		IATA:      "PAY",
		Name:      "Pamol Airport",
		City:      "Pamol",
		State:     "Sabah",
		Country:   "MY",
		Elevation: 36,
		Latitude:  5.9993000031,
		Longitude: 117.3999176025,
		Timezone:  "Asia/Kuching",
	},
	"WBKR": {
		ICAO:      "WBKR",
		IATA:      "RNU",
		Name:      "Ranau Airport",
		City:      "Ranau",
		State:     "Sabah",
		Country:   "MY",
		Elevation: 1800,
		Latitude:  5.9499998093,
		Longitude: 116.6669998169,
		Timezone:  "Asia/Kuching",
	},
	"WBKS": {
		ICAO:      "WBKS",
		IATA:      "SDK",
		Name:      "Sandakan Airport",
		City:      "Sandakan",
		State:     "Sabah",
		Country:   "MY",
		Elevation: 46,
		Latitude:  5.9008998871,
		Longitude: 118.0589981079,
		Timezone:  "Asia/Kuching",
	},
	"WBKT": {
		ICAO:      "WBKT",
		IATA:      "KUD",
		Name:      "Kudat Airport",
		City:      "Kudat",
		State:     "Sabah",
		Country:   "MY",
		Elevation: 10,
		Latitude:  6.9225001335,
		Longitude: 116.8359985352,
		Timezone:  "Asia/Kuching",
	},
	"WBKU": {
		ICAO:      "WBKU",
		Name:      "Kuala Penyu Airport",
		City:      "Kuala Penyu",
		State:     "Sabah",
		Country:   "MY",
		Elevation: 6,
		Latitude:  5.5999999046,
		Longitude: 115.5830001831,
		Timezone:  "Asia/Kuching",
	},
	"WBKW": {
		ICAO:      "WBKW",
		IATA:      "TWU",
		Name:      "Tawau Airport",
		City:      "Tawau",
		State:     "Sabah",
		Country:   "MY",
		Elevation: 57,
		Latitude:  4.3201599121,
		Longitude: 118.1279983521,
		Timezone:  "Asia/Kuching",
	},
	"WBMU": {
		ICAO:      "WBMU",
		IATA:      "MZV",
		Name:      "Mulu Airport",
		City:      "Mulu",
		State:     "Sarawak",
		Country:   "MY",
		Elevation: 80,
		Latitude:  4.0483298302,
		Longitude: 114.8050003052,
		Timezone:  "Asia/Kuching",
	},
	"WBSB": {
		ICAO:      "WBSB",
		IATA:      "BWN",
		Name:      "Brunei International Airport",
		City:      "Bandar Seri Begawan",
		Country:   "BN",
		Elevation: 73,
		Latitude:  4.9442000389,
		Longitude: 114.9280014038,
		Timezone:  "Asia/Brunei",
	},
	"WI00": {
		ICAO:      "WI00",
		Name:      "Norrie Brook Airport",
		City:      "Eland",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1240,
		Latitude:  44.8739013672,
		Longitude: -89.2306976318,
		Timezone:  "America/Chicago",
	},
	"WI02": {
		ICAO:      "WI02",
		Name:      "Turtle Airport",
		City:      "Beloit",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 911,
		Latitude:  42.5069999695,
		Longitude: -88.9148025513,
		Timezone:  "America/Chicago",
	},
	"WI03": {
		ICAO:      "WI03",
		Name:      "Horner Farms Airport",
		City:      "Union Grove",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 770,
		Latitude:  42.7980995178,
		Longitude: -88.1333999634,
		Timezone:  "America/Chicago",
	},
	"WI05": {
		ICAO:      "WI05",
		Name:      "Stocktrade Airport",
		City:      "Elk Mound",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 882,
		Latitude:  44.8083000183,
		Longitude: -91.6844024658,
		Timezone:  "America/Chicago",
	},
	"WI07": {
		ICAO:      "WI07",
		Name:      "Waupun Airport",
		City:      "Waupun",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 940,
		Latitude:  43.6222000122,
		Longitude: -88.7667999268,
		Timezone:  "America/Chicago",
	},
	"WI09": {
		ICAO:      "WI09",
		Name:      "Heitman Field Airport",
		City:      "Dousman",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 880,
		Latitude:  42.9771995544,
		Longitude: -88.4757995605,
		Timezone:  "America/Chicago",
	},
	"WI10": {
		ICAO:      "WI10",
		Name:      "Cedar Island Airport",
		City:      "Brule",
		Country:   "US",
		Elevation: 1229,
		Latitude:  46.4499015808,
		Longitude: -91.616897583,
		Timezone:  "America/Chicago",
	},
	"WI11": {
		ICAO:      "WI11",
		Name:      "Mumm Field",
		City:      "Orfordville",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 945,
		Latitude:  42.5914001465,
		Longitude: -89.2406997681,
		Timezone:  "America/Chicago",
	},
	"WI12": {
		ICAO:      "WI12",
		Name:      "Lilac Time Airport",
		City:      "Birchwood",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1300,
		Latitude:  45.7066001892,
		Longitude: -91.6249008179,
		Timezone:  "America/Chicago",
	},
	"WI13": {
		ICAO:      "WI13",
		Name:      "Jennie'S Field",
		City:      "Wisconsin Rapids",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 993,
		Latitude:  44.254833,
		Longitude: -89.8395,
		Timezone:  "America/Chicago",
	},
	"WI14": {
		ICAO:      "WI14",
		Name:      "Baldwin Airport",
		City:      "Baldwin",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1104,
		Latitude:  44.9665985107,
		Longitude: -92.3877029419,
		Timezone:  "America/Chicago",
	},
	"WI16": {
		ICAO:      "WI16",
		Name:      "Jim Benson Field",
		City:      "Mineral Point",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1100,
		Latitude:  42.853099823,
		Longitude: -90.129699707,
		Timezone:  "America/Chicago",
	},
	"WI18": {
		ICAO:      "WI18",
		Name:      "Gateway Airport",
		City:      "Bloomer",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1021,
		Latitude:  45.1333007813,
		Longitude: -91.4502029419,
		Timezone:  "America/Chicago",
	},
	"WI19": {
		ICAO:      "WI19",
		Name:      "Cacic Airport",
		City:      "Montello",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 805,
		Latitude:  43.7999992371,
		Longitude: -89.4335021973,
		Timezone:  "America/Chicago",
	},
	"WI1A": {
		ICAO:      "WI1A",
		Name:      "Nusawiru Airport",
		City:      "Pangandaran-Java Island",
		State:     "West-Java",
		Country:   "ID",
		Elevation: 16,
		Latitude:  -7.720389843,
		Longitude: 108.4899978638,
		Timezone:  "Asia/Jakarta",
	},
	"WI1B": {
		ICAO:      "WI1B",
		Name:      "Batujajar Airport",
		City:      "Bandung-Java Island",
		State:     "West-Java",
		Country:   "ID",
		Elevation: 2500,
		Latitude:  -6.9038600922,
		Longitude: 107.475997925,
		Timezone:  "Asia/Jakarta",
	},
	"WI1C": {
		ICAO:      "WI1C",
		Name:      "Rumpin Airport",
		City:      "Rumpin-Java Island",
		State:     "West-Java",
		Country:   "ID",
		Elevation: 223,
		Latitude:  -6.3732199669,
		Longitude: 106.6220016479,
		Timezone:  "Asia/Jakarta",
	},
	"WI1G": {
		ICAO:      "WI1G",
		Name:      "Gading Wonosari Airport",
		City:      "Wonosari-Java Island",
		State:     "Yogyakarta",
		Country:   "ID",
		Elevation: 702,
		Latitude:  -7.9165902138,
		Longitude: 110.5640029907,
		Timezone:  "Asia/Jakarta",
	},
	"WI1S": {
		ICAO:      "WI1S",
		Name:      "Simpangtiga Redelong Airport",
		City:      "Tingkeum-Sumatra Island",
		State:     "Aceh",
		Country:   "ID",
		Elevation: 4636,
		Latitude:  4.7208800316,
		Longitude: 96.8511962891,
		Timezone:  "Asia/Jakarta",
	},
	"WI20": {
		ICAO:      "WI20",
		Name:      "Larson Studio Airport",
		City:      "Oconto Falls",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 700,
		Latitude:  44.8829994202,
		Longitude: -88.0920028687,
		Timezone:  "America/Chicago",
	},
	"WI21": {
		ICAO:      "WI21",
		Name:      "Crane Field",
		City:      "Cadott",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 936,
		Latitude:  44.9096984863,
		Longitude: -91.2417984009,
		Timezone:  "America/Chicago",
	},
	"WI22": {
		ICAO:      "WI22",
		Name:      "Rocket City Airport",
		City:      "Hortonville",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 905,
		Latitude:  44.3231010437,
		Longitude: -88.6007995605,
		Timezone:  "America/Chicago",
	},
	"WI23": {
		ICAO:      "WI23",
		Name:      "Cornucopia Field",
		City:      "Cornucopia",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 760,
		Latitude:  46.8666000366,
		Longitude: -91.0835037231,
		Timezone:  "America/Chicago",
	},
	"WI24": {
		ICAO:      "WI24",
		Name:      "Cardinal Ridge Airport",
		City:      "Mt Horeb",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1080,
		Latitude:  42.8810997009,
		Longitude: -89.7521972656,
		Timezone:  "America/Chicago",
	},
	"WI25": {
		ICAO:      "WI25",
		Name:      "Durand Municipal Airport",
		City:      "Durand",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 780,
		Latitude:  44.538898468,
		Longitude: -92.0140991211,
		Timezone:  "America/Chicago",
	},
	"WI26": {
		ICAO:      "WI26",
		Name:      "Mertinkes Airport",
		City:      "Eau Claire",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 920,
		Latitude:  44.7050018311,
		Longitude: -91.4654006958,
		Timezone:  "America/Chicago",
	},
	"WI27": {
		ICAO:      "WI27",
		Name:      "Skyport Airport",
		City:      "Menomonie",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 878,
		Latitude:  44.9202003479,
		Longitude: -91.974899292,
		Timezone:  "America/Chicago",
	},
	"WI28": {
		ICAO:      "WI28",
		Name:      "Walter's Agri-Center Airport",
		City:      "Rio Creek",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 717,
		Latitude:  44.586101532,
		Longitude: -87.5576019287,
		Timezone:  "America/Chicago",
	},
	"WI29": {
		ICAO:      "WI29",
		Name:      "Rush River Airport",
		City:      "Baldwin",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1078,
		Latitude:  44.9054985046,
		Longitude: -92.4834976196,
		Timezone:  "America/Chicago",
	},
	"WI31": {
		ICAO:      "WI31",
		Name:      "Minnesuing Airport",
		City:      "Lake Nebagamon",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1207,
		Latitude:  46.4143981934,
		Longitude: -91.6510009766,
		Timezone:  "America/Chicago",
	},
	"WI32": {
		ICAO:      "WI32",
		Name:      "T-Bo Field Airport",
		City:      "Chippewa Falls",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 940,
		Latitude:  44.9873008728,
		Longitude: -91.3303985596,
		Timezone:  "America/Chicago",
	},
	"WI33": {
		ICAO:      "WI33",
		Name:      "Ben Sutherland Airport",
		City:      "Minong",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1056,
		Latitude:  46.1035995483,
		Longitude: -91.9023971558,
		Timezone:  "America/Chicago",
	},
	"WI36": {
		ICAO:      "WI36",
		Name:      "Dolhun Field",
		City:      "Lake Tomahawk",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1625,
		Latitude:  45.8265991211,
		Longitude: -89.6201019287,
		Timezone:  "America/Chicago",
	},
	"WI37": {
		ICAO:      "WI37",
		Name:      "Rainbow Airport",
		City:      "Ojibwa",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1321,
		Latitude:  45.8083000183,
		Longitude: -91.0962982178,
		Timezone:  "America/Chicago",
	},
	"WI40": {
		ICAO:      "WI40",
		Name:      "Spiegel Field",
		City:      "Oregon",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 960,
		Latitude:  42.9132995605,
		Longitude: -89.3523025513,
		Timezone:  "America/Chicago",
	},
	"WI42": {
		ICAO:      "WI42",
		Name:      "Pine Grove Airport",
		City:      "Rhinelander",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1630,
		Latitude:  45.6571998596,
		Longitude: -89.5298995972,
		Timezone:  "America/Chicago",
	},
	"WI43": {
		ICAO:      "WI43",
		Name:      "Atkins Ridge Airport",
		City:      "Daleyville",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1090,
		Latitude:  42.9488983154,
		Longitude: -89.825302124,
		Timezone:  "America/Chicago",
	},
	"WI46": {
		ICAO:      "WI46",
		Name:      "Fun-Air Airport",
		City:      "Van Dyne",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 805,
		Latitude:  43.8828010559,
		Longitude: -88.5399017334,
		Timezone:  "America/Chicago",
	},
	"WI47": {
		ICAO:      "WI47",
		Name:      "Timberline Airport",
		City:      "Scandinavia",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 890,
		Latitude:  44.4333000183,
		Longitude: -89.150100708,
		Timezone:  "America/Chicago",
	},
	"WI48": {
		ICAO:      "WI48",
		Name:      "Edinger Field",
		City:      "Eureka",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 770,
		Latitude:  44.004699707,
		Longitude: -88.8709030151,
		Timezone:  "America/Chicago",
	},
	"WI49": {
		ICAO:      "WI49",
		Name:      "Blair Airport",
		City:      "Blair",
		Country:   "US",
		Elevation: 864,
		Latitude:  44.2860984802,
		Longitude: -91.2265014648,
		Timezone:  "America/Chicago",
	},
	"WI50": {
		ICAO:      "WI50",
		Name:      "Albany Airport",
		City:      "Albany",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 815,
		Latitude:  42.7181015015,
		Longitude: -89.4236984253,
		Timezone:  "America/Chicago",
	},
	"WI51": {
		ICAO:      "WI51",
		Name:      "Del Monte Airport",
		City:      "Arlington",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1064,
		Latitude:  43.3213996887,
		Longitude: -89.3209991455,
		Timezone:  "America/Chicago",
	},
	"WI52": {
		ICAO:      "WI52",
		Name:      "Schewe Airport",
		City:      "Clintonville",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 810,
		Latitude:  44.6178016663,
		Longitude: -88.6668014526,
		Timezone:  "America/Chicago",
	},
	"WI53": {
		ICAO:      "WI53",
		Name:      "Desmet Airport",
		City:      "Barneveld",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1050,
		Latitude:  42.9453010559,
		Longitude: -89.8990020752,
		Timezone:  "America/Chicago",
	},
	"WI54": {
		ICAO:      "WI54",
		Name:      "Corinth Airport",
		City:      "Athens",
		Country:   "US",
		Elevation: 1425,
		Latitude:  44.9994010925,
		Longitude: -90.149597168,
		Timezone:  "America/Chicago",
	},
	"WI55": {
		ICAO:      "WI55",
		Name:      "Broken Prop Airport",
		City:      "Berlin",
		Country:   "US",
		Elevation: 767,
		Latitude:  43.9860992432,
		Longitude: -88.9834976196,
		Timezone:  "America/Chicago",
	},
	"WI56": {
		ICAO:      "WI56",
		Name:      "C Jeidy Farms Airport",
		City:      "Bloomington",
		Country:   "US",
		Elevation: 1010,
		Latitude:  42.8816986084,
		Longitude: -90.8607025146,
		Timezone:  "America/Chicago",
	},
	"WI58": {
		ICAO:      "WI58",
		Name:      "Winfield Airport",
		City:      "Bristol",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 770,
		Latitude:  42.5222015381,
		Longitude: -88.0279006958,
		Timezone:  "America/Chicago",
	},
	"WI60": {
		ICAO:      "WI60",
		Name:      "Deer Haven Ranch Airport",
		City:      "Cecil",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 811,
		Latitude:  44.8328018188,
		Longitude: -88.4632034302,
		Timezone:  "America/Chicago",
	},
	"WI61": {
		ICAO:      "WI61",
		Name:      "Ashenfelter Aerodrome",
		City:      "Grafton",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 860,
		Latitude:  43.3488998413,
		Longitude: -87.9943008423,
		Timezone:  "America/Chicago",
	},
	"WI62": {
		ICAO:      "WI62",
		Name:      "Sss Aerodrome",
		City:      "Cedarburg",
		Country:   "US",
		Elevation: 800,
		Latitude:  43.2695007324,
		Longitude: -87.9981002808,
		Timezone:  "America/Chicago",
	},
	"WI65": {
		ICAO:      "WI65",
		Name:      "Dane Airport",
		City:      "Dane",
		Country:   "US",
		Elevation: 1035,
		Latitude:  43.2221984863,
		Longitude: -89.4682006836,
		Timezone:  "America/Chicago",
	},
	"WI66": {
		ICAO:      "WI66",
		Name:      "Smilin' Sam's Airport",
		City:      "Delavan",
		Country:   "US",
		Elevation: 980,
		Latitude:  42.5872001648,
		Longitude: -88.658203125,
		Timezone:  "America/Chicago",
	},
	"WI67": {
		ICAO:      "WI67",
		Name:      "Bennet Field",
		City:      "Ripon",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 935,
		Latitude:  43.8765983582,
		Longitude: -88.9007034302,
		Timezone:  "America/Chicago",
	},
	"WI69": {
		ICAO:      "WI69",
		Name:      "Air Troy Estates - Restricted Airport",
		City:      "East Troy",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 820,
		Latitude:  42.7999992371,
		Longitude: -88.3333969116,
		Timezone:  "America/Chicago",
	},
	"WI70": {
		ICAO:      "WI70",
		Name:      "Swan Airport",
		City:      "Elkhorn",
		Country:   "US",
		Elevation: 930,
		Latitude:  42.7070007324,
		Longitude: -88.6376037598,
		Timezone:  "America/Chicago",
	},
	"WI71": {
		ICAO:      "WI71",
		Name:      "Weedhopper Meadow Airport",
		City:      "Elkhorn",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1030,
		Latitude:  42.6875,
		Longitude: -88.6019973755,
		Timezone:  "America/Chicago",
	},
	"WI72": {
		ICAO:      "WI72",
		Name:      "Martin Fierro Airport",
		City:      "Ellenboro",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1000,
		Latitude:  42.7999992371,
		Longitude: -90.5718002319,
		Timezone:  "America/Chicago",
	},
	"WI73": {
		ICAO:      "WI73",
		Name:      "Happy Jacks Air Strip",
		City:      "Eureka",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 800,
		Latitude:  44,
		Longitude: -88.8333969116,
		Timezone:  "America/Chicago",
	},
	"WI74": {
		ICAO:      "WI74",
		Name:      "Pfaffenroth Private Airport",
		City:      "Fisk",
		Country:   "US",
		Elevation: 872,
		Latitude:  43.9249992371,
		Longitude: -88.6676025391,
		Timezone:  "America/Chicago",
	},
	"WI75": {
		ICAO:      "WI75",
		Name:      "Bogus Creek Airport",
		City:      "Stockholm",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 950,
		Latitude:  44.4989013672,
		Longitude: -92.2031021118,
		Timezone:  "America/Chicago",
	},
	"WI76": {
		ICAO:      "WI76",
		Name:      "J Rock Airport",
		City:      "Fort Atkinson",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 836,
		Latitude:  42.948600769,
		Longitude: -88.7759017944,
		Timezone:  "America/Chicago",
	},
	"WI77": {
		ICAO:      "WI77",
		Name:      "Willow Creek Airport",
		City:      "Germantown",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 860,
		Latitude:  43.2070007324,
		Longitude: -88.1604003906,
		Timezone:  "America/Chicago",
	},
	"WI78": {
		ICAO:      "WI78",
		Name:      "Martins Aerodrome",
		City:      "Green Bay",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 808,
		Latitude:  44.4258003235,
		Longitude: -87.9349975586,
		Timezone:  "America/Chicago",
	},
	"WI79": {
		ICAO:      "WI79",
		Name:      "Town Line Airport",
		City:      "Greenwood",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1270,
		Latitude:  44.8526992798,
		Longitude: -90.5565032959,
		Timezone:  "America/Chicago",
	},
	"WI81": {
		ICAO:      "WI81",
		Name:      "Rigdon Private Airport",
		City:      "Hazel Green",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 880,
		Latitude:  42.5242004395,
		Longitude: -90.591003418,
		Timezone:  "America/Chicago",
	},
	"WI84": {
		ICAO:      "WI84",
		Name:      "Johnstown Center Airport",
		City:      "Janesville",
		Country:   "US",
		Elevation: 925,
		Latitude:  42.6888999939,
		Longitude: -88.8404006958,
		Timezone:  "America/Chicago",
	},
	"WI86": {
		ICAO:      "WI86",
		Name:      "Flaglor Airport",
		City:      "Kansasville",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 800,
		Latitude:  42.660900116,
		Longitude: -88.1359024048,
		Timezone:  "America/Chicago",
	},
	"WI87": {
		ICAO:      "WI87",
		Name:      "Pine River Airport",
		City:      "Merrill",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1370,
		Latitude:  45.2033004761,
		Longitude: -89.5243988037,
		Timezone:  "America/Chicago",
	},
	"WI88": {
		ICAO:      "WI88",
		Name:      "Mount Fuji Airport",
		City:      "Lake Geneva",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1000,
		Latitude:  42.625,
		Longitude: -88.4501037598,
		Timezone:  "America/Chicago",
	},
	"WI89": {
		ICAO:      "WI89",
		Name:      "Lake Geneva Aire Estates Airport",
		City:      "Lake Geneva",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 880,
		Latitude:  42.5722007751,
		Longitude: -88.3673019409,
		Timezone:  "America/Chicago",
	},
	"WI90": {
		ICAO:      "WI90",
		Name:      "Speedwing Field",
		City:      "Babcock",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 970,
		Latitude:  44.2961006165,
		Longitude: -90.0437011719,
		Timezone:  "America/Chicago",
	},
	"WI91": {
		ICAO:      "WI91",
		Name:      "Larson Airport",
		City:      "Larsen",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 800,
		Latitude:  44.2028007507,
		Longitude: -88.6396026611,
		Timezone:  "America/Chicago",
	},
	"WI92": {
		ICAO:      "WI92",
		Name:      "Wag-Aero Airport",
		City:      "Lyons",
		Country:   "US",
		Elevation: 905,
		Latitude:  42.6610984802,
		Longitude: -88.3611984253,
		Timezone:  "America/Chicago",
	},
	"WI95": {
		ICAO:      "WI95",
		Name:      "Chilcott Farms Airport",
		City:      "Paddock Lake",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 700,
		Latitude:  42.5778007507,
		Longitude: -88.0154037476,
		Timezone:  "America/Chicago",
	},
	"WI97": {
		ICAO:      "WI97",
		Name:      "Mathaire Field",
		City:      "Marshall",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 960,
		Latitude:  43.2578010559,
		Longitude: -89.0647964478,
		Timezone:  "America/Chicago",
	},
	"WI98": {
		ICAO:      "WI98",
		Name:      "Blackburn Airport",
		City:      "Cooksville",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 888,
		Latitude:  42.8582992554,
		Longitude: -89.197303772,
		Timezone:  "America/Chicago",
	},
	"WI99": {
		ICAO:      "WI99",
		Name:      "Meier Airport",
		City:      "Milton",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 880,
		Latitude:  42.8135986328,
		Longitude: -88.9971008301,
		Timezone:  "America/Chicago",
	},
	"WIAG": {
		ICAO:      "WIAG",
		IATA:      "AKQ",
		Name:      "Menggala Airport",
		City:      "Menggala-Sumatra Island",
		Country:   "ID",
		Elevation: 98,
		Latitude:  -4.4894399643,
		Longitude: 105.2170028687,
		Timezone:  "Asia/Jakarta",
	},
	"WIAJ": {
		ICAO:      "WIAJ",
		Name:      "Semplak Airport",
		Country:   "ID",
		Latitude:  -6.5402777778,
		Longitude: 106.755277778,
		Timezone:  "Asia/Jakarta",
	},
	"WIAK": {
		ICAO:      "WIAK",
		Name:      "Margahayu Airport",
		City:      "Bandung-Java Island",
		State:     "West-Java",
		Country:   "ID",
		Elevation: 2184,
		Latitude:  -6.9827799797,
		Longitude: 107.572998047,
		Timezone:  "Asia/Jakarta",
	},
	"WIAP": {
		ICAO:      "WIAP",
		Name:      "Banyumas Airport",
		City:      "Banyumas-Java Island",
		Country:   "ID",
		Elevation: 92,
		Latitude:  -6.4648499489,
		Longitude: 105.96900177,
		Timezone:  "Asia/Jakarta",
	},
	"WIBB": {
		ICAO:      "WIBB",
		IATA:      "PKU",
		Name:      "Sultan Syarif Kasim Ii (Simpang Tiga) Airport",
		City:      "Pekanbaru-Sumatra Island",
		State:     "Riau",
		Country:   "ID",
		Elevation: 102,
		Latitude:  0.4607860148,
		Longitude: 101.4449996948,
		Timezone:  "Asia/Jakarta",
	},
	"WIBD": {
		ICAO:      "WIBD",
		IATA:      "DUM",
		Name:      "Pinang Kampai Airport",
		City:      "Dumai-Sumatra Island",
		State:     "Riau",
		Country:   "ID",
		Elevation: 55,
		Latitude:  1.6091899872,
		Longitude: 101.4339981079,
		Timezone:  "Asia/Jakarta",
	},
	"WIBR": {
		ICAO:      "WIBR",
		IATA:      "RKO",
		Name:      "Rokot Airport",
		City:      "Sipora Island",
		State:     "West-Sumatra",
		Country:   "ID",
		Elevation: 1,
		Latitude:  -2.0991199017,
		Longitude: 99.7006988525,
		Timezone:  "Asia/Jakarta",
	},
	"WIBS": {
		ICAO:      "WIBS",
		IATA:      "SEQ",
		Name:      "Sungai Pakning Bengkalis Airport",
		City:      "Bengkalis-Sumatra Island",
		State:     "Riau",
		Country:   "ID",
		Elevation: 3,
		Latitude:  1.3700000048,
		Longitude: 102.1399993896,
		Timezone:  "Asia/Jakarta",
	},
	"WIBT": {
		ICAO:      "WIBT",
		IATA:      "TJB",
		Name:      "Tanjung Balai Airport",
		City:      "Karinmunbesar Island",
		State:     "Riau-Islands",
		Country:   "ID",
		Elevation: 3,
		Latitude:  1.0499999523,
		Longitude: 103.3830032349,
		Timezone:  "Asia/Jakarta",
	},
	"WICB": {
		ICAO:      "WICB",
		Name:      "Budiarto Airport",
		City:      "Tangerang-Java Island",
		State:     "West-Java",
		Country:   "ID",
		Elevation: 151,
		Latitude:  -6.2931699753,
		Longitude: 106.569999695,
		Timezone:  "Asia/Jakarta",
	},
	"WICC": {
		ICAO:      "WICC",
		IATA:      "BDO",
		Name:      "Husein Sastranegara International Airport",
		City:      "Bandung-Java Island",
		State:     "West-Java",
		Country:   "ID",
		Elevation: 2436,
		Latitude:  -6.9006299973,
		Longitude: 107.5759963989,
		Timezone:  "Asia/Jakarta",
	},
	"WICD": {
		ICAO:      "WICD",
		IATA:      "CBN",
		Name:      "Penggung Airport",
		City:      "Cirebon-Java Island",
		State:     "West-Java",
		Country:   "ID",
		Elevation: 89,
		Latitude:  -6.7561402321,
		Longitude: 108.540000916,
		Timezone:  "Asia/Jakarta",
	},
	"WICM": {
		ICAO:      "WICM",
		IATA:      "TSY",
		Name:      "Cibeureum Airport",
		City:      "Tasikmalaya-Java Island",
		State:     "West-Java",
		Country:   "ID",
		Elevation: 1148,
		Latitude:  -7.3466000557,
		Longitude: 108.246002197,
		Timezone:  "Asia/Jakarta",
	},
	"WICT": {
		ICAO:      "WICT",
		IATA:      "TKG",
		Name:      "Radin Inten II (Branti) Airport",
		City:      "Bandar Lampung-Sumatra Island",
		Country:   "ID",
		Elevation: 282,
		Latitude:  -5.240556,
		Longitude: 105.175556,
		Timezone:  "Asia/Jakarta",
	},
	"WIDD": {
		ICAO:      "WIDD",
		IATA:      "BTH",
		Name:      "Hang Nadim Airport",
		City:      "Batam Island",
		Country:   "ID",
		Elevation: 126,
		Latitude:  1.121029973,
		Longitude: 104.1190032959,
		Timezone:  "Asia/Jakarta",
	},
	"WIDE": {
		ICAO:      "WIDE",
		IATA:      "PPR",
		Name:      "Pasir Pangaraan Airport",
		City:      "Pasir Pengarayan-Sumatra Island",
		State:     "Riau",
		Country:   "ID",
		Elevation: 151,
		Latitude:  0.8454310298,
		Longitude: 100.3700027466,
		Timezone:  "Asia/Jakarta",
	},
	"WIDN": {
		ICAO:      "WIDN",
		IATA:      "TNJ",
		Name:      "Kijang Airport",
		City:      "Tanjung Pinang-Bintan Island",
		State:     "Riau-Islands",
		Country:   "ID",
		Elevation: 52,
		Latitude:  0.9226830006,
		Longitude: 104.531997681,
		Timezone:  "Asia/Jakarta",
	},
	"WIDS": {
		ICAO:      "WIDS",
		IATA:      "SIQ",
		Name:      "Dabo Airport",
		City:      "Pasirkuning-Singkep Island",
		State:     "Riau-Islands",
		Country:   "ID",
		Elevation: 95,
		Latitude:  -0.4791890085,
		Longitude: 104.5790023804,
		Timezone:  "Asia/Jakarta",
	},
	"WIEE": {
		ICAO:      "WIEE",
		IATA:      "PDG",
		Name:      "Minangkabau Airport",
		City:      "Ketaping/Padang-Sumatra Island",
		State:     "Ketaping",
		Country:   "ID",
		Elevation: 18,
		Latitude:  -0.7869169712,
		Longitude: 100.28099823,
		Timezone:  "Asia/Jakarta",
	},
	"WIHH": {
		ICAO:      "WIHH",
		IATA:      "HLP",
		Name:      "Halim Perdanakusuma International Airport",
		City:      "Jakarta",
		State:     "Jakarta",
		Country:   "ID",
		Elevation: 84,
		Latitude:  -6.2666101456,
		Longitude: 106.8909988403,
		Timezone:  "Asia/Jakarta",
	},
	"WIHL": {
		ICAO:      "WIHL",
		IATA:      "CXP",
		Name:      "Tunggul Wulung Airport",
		City:      "Cilacap-Java Island",
		Country:   "ID",
		Elevation: 69,
		Latitude:  -7.6450600624,
		Longitude: 109.033996582,
		Timezone:  "Asia/Jakarta",
	},
	"WIHP": {
		ICAO:      "WIHP",
		IATA:      "PCB",
		Name:      "Pondok Cabe Air Base",
		City:      "Jakarta",
		Country:   "ID",
		Elevation: 200,
		Latitude:  -6.3369598389,
		Longitude: 106.7649993896,
		Timezone:  "Asia/Jakarta",
	},
	"WIIG": {
		ICAO:      "WIIG",
		IATA:      "PPJ",
		Name:      "Pulau Panjang Airport",
		City:      "Tjipara-Java Island",
		State:     "Jakarta",
		Country:   "ID",
		Elevation: 155,
		Latitude:  -5.644444,
		Longitude: 106.5625,
		Timezone:  "Asia/Jakarta",
	},
	"WIII": {
		ICAO:      "WIII",
		IATA:      "CGK",
		Name:      "Soekarno-Hatta International Airport",
		City:      "Jakarta",
		Country:   "ID",
		Elevation: 34,
		Latitude:  -6.1255698204,
		Longitude: 106.65599823,
		Timezone:  "Asia/Jakarta",
	},
	"WIIK": {
		ICAO:      "WIIK",
		Name:      "Kalijati Airport",
		City:      "Kalijati-Java Island",
		State:     "West-Java",
		Country:   "ID",
		Elevation: 361,
		Latitude:  -6.5313901901,
		Longitude: 107.658996582,
		Timezone:  "Asia/Jakarta",
	},
	"WIIR": {
		ICAO:      "WIIR",
		Name:      "Pelabuhan Ratu Airport",
		City:      "Pelabuhan Ratu-Java Island",
		State:     "West-Java",
		Country:   "ID",
		Elevation: 65,
		Latitude:  -6.9766669273,
		Longitude: 106.5327758789,
		Timezone:  "Asia/Jakarta",
	},
	"WIKL": {
		ICAO:      "WIKL",
		Name:      "Silampari Airport",
		City:      "Lubuk Linggau-Sumatra Island",
		Country:   "ID",
		Elevation: 371,
		Latitude:  -3.2777500153,
		Longitude: 102.901000977,
		Timezone:  "Asia/Jakarta",
	},
	"WIMA": {
		ICAO:      "WIMA",
		Name:      "Labuhan Bilik Airport",
		City:      "Labuhan Bilik-Sumatra Island",
		Country:   "ID",
		Elevation: 18,
		Latitude:  2.5129001141,
		Longitude: 100.1790008545,
		Timezone:  "Asia/Jakarta",
	},
	"WIMB": {
		ICAO:      "WIMB",
		IATA:      "GNS",
		Name:      "Binaka Airport",
		City:      "Gunung Sitoli-Nias Island",
		State:     "North-Sumatra",
		Country:   "ID",
		Elevation: 20,
		Latitude:  1.1663800478,
		Longitude: 97.7046966553,
		Timezone:  "Asia/Jakarta",
	},
	"WIME": {
		ICAO:      "WIME",
		IATA:      "AEG",
		Name:      "Aek Godang Airport",
		City:      "Padang Sidempuan-Sumatra Island",
		Country:   "ID",
		Elevation: 922,
		Latitude:  1.4000999928,
		Longitude: 99.4304962158,
		Timezone:  "Asia/Jakarta",
	},
	"WIMG": {
		ICAO:      "WIMG",
		Name:      "Tabing Airport",
		City:      "Padang-Sumatra Island",
		Country:   "ID",
		Elevation: 9,
		Latitude:  -0.8749889731,
		Longitude: 100.351997375,
		Timezone:  "Asia/Jakarta",
	},
	"WIMH": {
		ICAO:      "WIMH",
		Name:      "Helvetia Airport",
		City:      "Helvetia-Sumatra Island",
		State:     "North-Sumatra",
		Country:   "ID",
		Elevation: 49,
		Latitude:  3.6830599308,
		Longitude: 98.644203186,
		Timezone:  "Asia/Jakarta",
	},
	"WIMK": {
		ICAO:      "WIMK",
		IATA:      "MES",
		Name:      "Soewondo Air Force Base",
		City:      "Medan",
		Country:   "ID",
		Elevation: 114,
		Latitude:  3.559167,
		Longitude: 98.671111,
		Timezone:  "Asia/Jakarta",
	},
	"WIML": {
		ICAO:      "WIML",
		Name:      "Kisaran Airport",
		City:      "Kisaran-Sumatra Island",
		State:     "Aceh",
		Country:   "ID",
		Elevation: 96,
		Latitude:  2.9737899303,
		Longitude: 99.6009979248,
		Timezone:  "Asia/Jakarta",
	},
	"WIMM": {
		ICAO:      "WIMM",
		IATA:      "KNO",
		Name:      "Polonia International Airport",
		City:      "Medan-Sumatra Island",
		Country:   "ID",
		Elevation: 114,
		Latitude:  3.5580599308,
		Longitude: 98.6716995239,
		Timezone:  "Asia/Jakarta",
	},
	"WIMN": {
		ICAO:      "WIMN",
		IATA:      "DTB",
		Name:      "Sisingamangaraja XII International Airport",
		City:      "Siborong-Borong",
		State:     "North-Sumatra",
		Country:   "ID",
		Elevation: 4660,
		Latitude:  2.259722,
		Longitude: 98.995278,
		Timezone:  "Asia/Jakarta",
	},
	"WIMO": {
		ICAO:      "WIMO",
		Name:      "Bandar Udara Lasondre",
		Country:   "ID",
		Elevation: 5,
		Latitude:  -0.01924,
		Longitude: 98.30097,
		Timezone:  "Asia/Jakarta",
	},
	"WIMP": {
		ICAO:      "WIMP",
		IATA:      "SIW",
		Name:      "Parapat Airport",
		City:      "Parapat-Sumatra Island",
		Country:   "ID",
		Elevation: 3081,
		Latitude:  2.6666669846,
		Longitude: 98.9333343506,
		Timezone:  "Asia/Jakarta",
	},
	"WIMR": {
		ICAO:      "WIMR",
		Name:      "Pematang Siantar",
		City:      "Pematang Siantar-Sumatra Island",
		State:     "North-Sumatra",
		Country:   "ID",
		Elevation: 1361,
		Latitude:  2.9500000477,
		Longitude: 99.0500030518,
		Timezone:  "Asia/Jakarta",
	},
	"WIMS": {
		ICAO:      "WIMS",
		IATA:      "FLZ",
		Name:      "Dr Ferdinand Lumban Tobing Airport",
		City:      "Sibolga-Sumatra Island",
		State:     "North-Sumatra",
		Country:   "ID",
		Elevation: 43,
		Latitude:  1.555940032,
		Longitude: 98.8889007568,
		Timezone:  "Asia/Jakarta",
	},
	"WIMT": {
		ICAO:      "WIMT",
		Name:      "Tebing Tinggi Airport",
		City:      "Tabbing Tinggi-Sumatra Island",
		State:     "North-Sumatra",
		Country:   "ID",
		Elevation: 69,
		Latitude:  3.3666670322,
		Longitude: 99.1166687012,
		Timezone:  "Asia/Jakarta",
	},
	"WIOB": {
		ICAO:      "WIOB",
		Name:      "Bengkayang Airport",
		City:      "Bengkayang-Borneo Island",
		State:     "West-Kalimantan",
		Country:   "ID",
		Elevation: 294,
		Latitude:  0.8333330154,
		Longitude: 109.4833297729,
		Timezone:  "Asia/Pontianak",
	},
	"WIOD": {
		ICAO:      "WIOD",
		IATA:      "TJQ",
		Name:      "Buluh Tumbang (H A S Hanandjoeddin) Airport",
		City:      "Tanjung Pandan-Belitung Island",
		State:     "Bangka–Belitung-Islands",
		Country:   "ID",
		Elevation: 164,
		Latitude:  -2.7457199097,
		Longitude: 107.754997253,
		Timezone:  "Asia/Jakarta",
	},
	"WIOG": {
		ICAO:      "WIOG",
		IATA:      "NPO",
		Name:      "Nanga Pinoh I Airport",
		City:      "Nanga Pinoh-Borneo Island",
		State:     "West-Kalimantan",
		Country:   "ID",
		Elevation: 123,
		Latitude:  -0.3488689959,
		Longitude: 111.7480010986,
		Timezone:  "Asia/Pontianak",
	},
	"WIOI": {
		ICAO:      "WIOI",
		Name:      "Singkawang Airport",
		City:      "Sinkawang-Borneo Island",
		Country:   "ID",
		Elevation: 276,
		Latitude:  1.0756800175,
		Longitude: 109.68800354,
		Timezone:  "Asia/Pontianak",
	},
	"WIOK": {
		ICAO:      "WIOK",
		IATA:      "KTG",
		Name:      "Ketapang(Rahadi Usman) Airport",
		City:      "Ketapang-Borneo Island",
		Country:   "ID",
		Elevation: 46,
		Latitude:  -1.8166400194,
		Longitude: 109.9629974365,
		Timezone:  "Asia/Pontianak",
	},
	"WIOM": {
		ICAO:      "WIOM",
		IATA:      "MWK",
		Name:      "Tarempa Airport",
		City:      "Matak Island",
		State:     "Riau",
		Country:   "ID",
		Elevation: 10,
		Latitude:  3.3481199741,
		Longitude: 106.2580032349,
		Timezone:  "Asia/Jakarta",
	},
	"WION": {
		ICAO:      "WION",
		IATA:      "NTX",
		Name:      "Ranai Airport",
		City:      "Ranai-Natuna Besar Island",
		State:     "Riau-Islands",
		Country:   "ID",
		Elevation: 7,
		Latitude:  3.9087100029,
		Longitude: 108.3880004883,
		Timezone:  "Asia/Jakarta",
	},
	"WIOO": {
		ICAO:      "WIOO",
		IATA:      "PNK",
		Name:      "Supadio Airport",
		City:      "Pontianak-Borneo Island",
		State:     "West-Kalimantan",
		Country:   "ID",
		Elevation: 10,
		Latitude:  -0.150711,
		Longitude: 109.4039993286,
		Timezone:  "Asia/Pontianak",
	},
	"WIOP": {
		ICAO:      "WIOP",
		IATA:      "PSU",
		Name:      "Pangsuma Airport",
		City:      "Putussibau-Borneo Island",
		State:     "West-Kalimantan",
		Country:   "ID",
		Elevation: 297,
		Latitude:  0.8355780244,
		Longitude: 112.93699646,
		Timezone:  "Asia/Pontianak",
	},
	"WIOS": {
		ICAO:      "WIOS",
		IATA:      "SQG",
		Name:      "Sintang(Susilo) Airport",
		City:      "Sintang-Borneo Island",
		Country:   "ID",
		Elevation: 98,
		Latitude:  0.0636190027,
		Longitude: 111.4729995728,
		Timezone:  "Asia/Pontianak",
	},
	"WIPA": {
		ICAO:      "WIPA",
		IATA:      "DJB",
		Name:      "Sultan Thaha Airport",
		City:      "Jambi-Sumatra Island",
		Country:   "ID",
		Elevation: 82,
		Latitude:  -1.6380200386,
		Longitude: 103.6439971924,
		Timezone:  "Asia/Jakarta",
	},
	"WIPD": {
		ICAO:      "WIPD",
		Name:      "Banding Agung Airport",
		City:      "Pasar Bandingagung-Sumatra Island",
		Country:   "ID",
		Elevation: 2100,
		Latitude:  -4.7870202065,
		Longitude: 103.9329986572,
		Timezone:  "Asia/Jakarta",
	},
	"WIPF": {
		ICAO:      "WIPF",
		Name:      "Kuala Tungkal Airport",
		City:      "Kuala Tungkal-Sumatra Island",
		State:     "Jambi",
		Country:   "ID",
		Elevation: 12,
		Latitude:  -0.8166670203,
		Longitude: 103.4666671753,
		Timezone:  "Asia/Jakarta",
	},
	"WIPK": {
		ICAO:      "WIPK",
		IATA:      "PGK",
		Name:      "Pangkal Pinang (Depati Amir) Airport",
		City:      "Pangkal Pinang-Palaubangka Island",
		State:     "South-Sumatra",
		Country:   "ID",
		Elevation: 109,
		Latitude:  -2.1621999741,
		Longitude: 106.138999939,
		Timezone:  "Asia/Jakarta",
	},
	"WIPL": {
		ICAO:      "WIPL",
		IATA:      "BKS",
		Name:      "Padang Kemiling (Fatmawati Soekarno) Airport",
		City:      "Bengkulu-Sumatra Island",
		Country:   "ID",
		Elevation: 50,
		Latitude:  -3.863699913,
		Longitude: 102.3389968872,
		Timezone:  "Asia/Jakarta",
	},
	"WIPO": {
		ICAO:      "WIPO",
		Name:      "Gatot Subrato Airport",
		City:      "Batu Raja-Sumatra Island",
		State:     "Lampung",
		Country:   "ID",
		Elevation: 450,
		Latitude:  -4.391769886,
		Longitude: 104.4010009766,
		Timezone:  "Asia/Jakarta",
	},
	"WIPP": {
		ICAO:      "WIPP",
		IATA:      "PLM",
		Name:      "Sultan Mahmud Badaruddin Ii Airport",
		City:      "Palembang-Sumatra Island",
		State:     "South-Sumatra",
		Country:   "ID",
		Elevation: 49,
		Latitude:  -2.898250103,
		Longitude: 104.6999969482,
		Timezone:  "Asia/Jakarta",
	},
	"WIPQ": {
		ICAO:      "WIPQ",
		IATA:      "PDO",
		Name:      "Pendopo Airport",
		City:      "Talang Gudang-Sumatra Island",
		State:     "South-Sumatra",
		Country:   "ID",
		Elevation: 184,
		Latitude:  -3.2860701084,
		Longitude: 103.8799972534,
		Timezone:  "Asia/Jakarta",
	},
	"WIPR": {
		ICAO:      "WIPR",
		IATA:      "RGT",
		Name:      "Japura Airport",
		City:      "Rengat-Sumatra Island",
		State:     "Riau-Islands",
		Country:   "ID",
		Elevation: 62,
		Latitude:  -0.3528079987,
		Longitude: 102.3349990845,
		Timezone:  "Asia/Jakarta",
	},
	"WIPT": {
		ICAO:      "WIPT",
		IATA:      "PDG",
		Name:      "Minangkabau Airport",
		City:      "Ketaping/Padang-Sumatra Island",
		State:     "West-Sumatra",
		Country:   "ID",
		Elevation: 18,
		Latitude:  -0.7869169712,
		Longitude: 100.28099823,
		Timezone:  "Asia/Jakarta",
	},
	"WIPU": {
		ICAO:      "WIPU",
		IATA:      "MPC",
		Name:      "Muko Muko Airport",
		City:      "Muko Muko-Sumatra Island",
		Country:   "ID",
		Elevation: 24,
		Latitude:  -2.5418601036,
		Longitude: 101.0879974365,
		Timezone:  "Asia/Jakarta",
	},
	"WIPV": {
		ICAO:      "WIPV",
		IATA:      "KLQ",
		Name:      "Keluang Airport",
		City:      "Keluang-Sumatra Island",
		Country:   "ID",
		Elevation: 76,
		Latitude:  -2.623529911,
		Longitude: 103.9550018311,
		Timezone:  "Asia/Jakarta",
	},
	"WIPY": {
		ICAO:      "WIPY",
		Name:      "Bentayan Airport",
		City:      "Bentayan-Sumatra Island",
		Country:   "ID",
		Elevation: 40,
		Latitude:  -2.434429884,
		Longitude: 104.0920028687,
		Timezone:  "Asia/Jakarta",
	},
	"WITA": {
		ICAO:      "WITA",
		IATA:      "TPK",
		Name:      "Teuku Cut Ali Airport",
		City:      "Tapak Tuan-Sumatra Island",
		State:     "Aceh",
		Country:   "ID",
		Elevation: 103,
		Latitude:  3.263420105,
		Longitude: 97.1837997437,
		Timezone:  "Asia/Jakarta",
	},
	"WITC": {
		ICAO:      "WITC",
		IATA:      "MEQ",
		Name:      "Seunagan Airport",
		City:      "Peureumeue-Sumatra Island",
		State:     "Aceh",
		Country:   "ID",
		Elevation: 10,
		Latitude:  4.25,
		Longitude: 96.2170028687,
		Timezone:  "Asia/Jakarta",
	},
	"WITG": {
		ICAO:      "WITG",
		Name:      "Lasikin Airport",
		City:      "Lubang-Simeulue Island",
		Country:   "ID",
		Elevation: 19,
		Latitude:  2.4102799892,
		Longitude: 96.3255996704,
		Timezone:  "Asia/Jakarta",
	},
	"WITL": {
		ICAO:      "WITL",
		IATA:      "LSX",
		Name:      "Lhok Sukon Airport",
		City:      "Lhok Sukon-Sumatra Island",
		State:     "Aceh",
		Country:   "ID",
		Elevation: 28,
		Latitude:  5.0695099831,
		Longitude: 97.2592010498,
		Timezone:  "Asia/Jakarta",
	},
	"WITM": {
		ICAO:      "WITM",
		IATA:      "LSW",
		Name:      "Malikus Saleh Airport",
		City:      "Lhok Seumawe-Sumatra Island",
		State:     "Aceh",
		Country:   "ID",
		Elevation: 90,
		Latitude:  5.2266798019,
		Longitude: 96.950302124,
		Timezone:  "Asia/Jakarta",
	},
	"WITN": {
		ICAO:      "WITN",
		IATA:      "SBG",
		Name:      "Maimun Saleh Airport",
		City:      "Sabang-We Island",
		State:     "Aceh",
		Country:   "ID",
		Elevation: 393,
		Latitude:  5.8741297722,
		Longitude: 95.3396987915,
		Timezone:  "Asia/Jakarta",
	},
	"WITS": {
		ICAO:      "WITS",
		Name:      "Seumayam Airport",
		City:      "Seumayam-Sumatra Island",
		State:     "Aceh",
		Country:   "ID",
		Elevation: 53,
		Latitude:  3.75,
		Longitude: 96.6333312988,
		Timezone:  "Asia/Jakarta",
	},
	"WITT": {
		ICAO:      "WITT",
		IATA:      "BTJ",
		Name:      "Sultan Iskandarmuda Airport",
		City:      "Banda Aceh-Sumatra Island",
		State:     "Aceh",
		Country:   "ID",
		Elevation: 65,
		Latitude:  5.5235199928,
		Longitude: 95.4204025269,
		Timezone:  "Asia/Jakarta",
	},
	"WMAA": {
		ICAO:      "WMAA",
		Name:      "Bahau Airport",
		City:      "Bahau",
		State:     "Negeri-Sembilan",
		Country:   "MY",
		Elevation: 214,
		Latitude:  2.8166699409,
		Longitude: 102.4169998169,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMAB": {
		ICAO:      "WMAB",
		Name:      "Batu Pahat Airport",
		City:      "Batu Pahat",
		State:     "Johor",
		Country:   "MY",
		Elevation: 30,
		Latitude:  1.8602420092,
		Longitude: 102.9010009766,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMAC": {
		ICAO:      "WMAC",
		Name:      "Benta Airport",
		City:      "Benta",
		State:     "Pahang",
		Country:   "MY",
		Elevation: 250,
		Latitude:  4.0500001907,
		Longitude: 101.983001709,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMAD": {
		ICAO:      "WMAD",
		Name:      "Bentong Airport",
		City:      "Bentong",
		State:     "Pahang",
		Country:   "MY",
		Elevation: 350,
		Latitude:  3.5499999523,
		Longitude: 101.8830032349,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMAE": {
		ICAO:      "WMAE",
		Name:      "Bidor Airport",
		City:      "Bidor",
		State:     "Perak",
		Country:   "MY",
		Elevation: 128,
		Latitude:  4.1163887978,
		Longitude: 101.2819976807,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMAG": {
		ICAO:      "WMAG",
		Name:      "Dungun Airport",
		City:      "Dungun",
		Country:   "MY",
		Elevation: 22,
		Latitude:  4.7862901688,
		Longitude: 103.3820037842,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMAH": {
		ICAO:      "WMAH",
		Name:      "Grik Airport",
		City:      "Grik",
		Country:   "MY",
		Elevation: 420,
		Latitude:  5.4327697754,
		Longitude: 101.1230010986,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMAJ": {
		ICAO:      "WMAJ",
		Name:      "Jendarata Airport",
		City:      "Jendarata",
		State:     "Perak",
		Country:   "MY",
		Elevation: 8,
		Latitude:  3.8997199535,
		Longitude: 100.9489974976,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMAL": {
		ICAO:      "WMAL",
		Name:      "Kuala Kerai Airport",
		City:      "Kuala Kerai",
		State:     "Kelantan",
		Country:   "MY",
		Elevation: 137,
		Latitude:  5.5386800766,
		Longitude: 102.1949996948,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMAN": {
		ICAO:      "WMAN",
		IATA:      "SXT",
		Name:      "Sungai Tiang Airport",
		City:      "Taman Negara",
		State:     "Pahang",
		Country:   "MY",
		Elevation: 10,
		Latitude:  4.3302779198,
		Longitude: 102.3949966431,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMAP": {
		ICAO:      "WMAP",
		Name:      "Kluang Airport",
		City:      "Kluang",
		Country:   "MY",
		Elevation: 150,
		Latitude:  2.0413899422,
		Longitude: 103.306999207,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMAQ": {
		ICAO:      "WMAQ",
		Name:      "Labis Airport",
		City:      "Labis",
		Country:   "MY",
		Elevation: 179,
		Latitude:  2.4003798962,
		Longitude: 103.0400009155,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMAU": {
		ICAO:      "WMAU",
		IATA:      "MEP",
		Name:      "Mersing Airport",
		City:      "Mersing",
		State:     "Johor",
		Country:   "MY",
		Elevation: 10,
		Latitude:  2.382999897,
		Longitude: 103.8669967651,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMAV": {
		ICAO:      "WMAV",
		Name:      "Muar / Bakri Airport",
		City:      "Muar",
		State:     "Johor",
		Country:   "MY",
		Elevation: 20,
		Latitude:  2.0499999523,
		Longitude: 102.6500015259,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMAZ": {
		ICAO:      "WMAZ",
		Name:      "Segamat Airport",
		City:      "Segamat",
		State:     "Johor",
		Country:   "MY",
		Elevation: 50,
		Latitude:  2.5,
		Longitude: 102.8330001831,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMBA": {
		ICAO:      "WMBA",
		IATA:      "SWY",
		Name:      "Sitiawan Airport",
		City:      "Sitiawan",
		State:     "Perak",
		Country:   "MY",
		Elevation: 25,
		Latitude:  4.2163882256,
		Longitude: 100.6986083984,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMBB": {
		ICAO:      "WMBB",
		Name:      "Sungei Patani Airport",
		City:      "Sungei Patani",
		State:     "Kedah",
		Country:   "MY",
		Elevation: 90,
		Latitude:  5.6176199913,
		Longitude: 100.5429992676,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMBE": {
		ICAO:      "WMBE",
		Name:      "Termeloh Airport",
		City:      "Temerloh",
		State:     "Pahang",
		Country:   "MY",
		Elevation: 100,
		Latitude:  3.4670000076,
		Longitude: 102.3830032349,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMBF": {
		ICAO:      "WMBF",
		Name:      "Ulu Bernam Airport",
		City:      "Ulu Bernam",
		Country:   "MY",
		Elevation: 10,
		Latitude:  3.75,
		Longitude: 101.1669998169,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMBH": {
		ICAO:      "WMBH",
		Name:      "Kroh Airport",
		City:      "Kroh",
		Country:   "MY",
		Elevation: 1000,
		Latitude:  5.6830000877,
		Longitude: 100.983001709,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMBI": {
		ICAO:      "WMBI",
		IATA:      "TPG",
		Name:      "Taiping (Tekah) Airport",
		City:      "Taiping",
		State:     "Perak",
		Country:   "MY",
		Elevation: 40,
		Latitude:  4.867000103,
		Longitude: 100.7170028687,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMBT": {
		ICAO:      "WMBT",
		IATA:      "TOD",
		Name:      "Pulau Tioman Airport",
		City:      "Pulau Tioman",
		State:     "Johor",
		Country:   "MY",
		Elevation: 15,
		Latitude:  2.8181800842,
		Longitude: 104.1600036621,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMGK": {
		ICAO:      "WMGK",
		Name:      "Gong Kedak Airport",
		State:     "Terengganu",
		Country:   "MY",
		Elevation: 20,
		Latitude:  5.7995700836,
		Longitude: 102.4840011597,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMKA": {
		ICAO:      "WMKA",
		IATA:      "AOR",
		Name:      "Sultan Abdul Halim Airport",
		City:      "Alor Satar",
		Country:   "MY",
		Elevation: 15,
		Latitude:  6.1896700859,
		Longitude: 100.3980026245,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMKB": {
		ICAO:      "WMKB",
		IATA:      "BWH",
		Name:      "Butterworth Airport",
		City:      "Butterworth",
		Country:   "MY",
		Elevation: 11,
		Latitude:  5.4659199715,
		Longitude: 100.3909988403,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMKC": {
		ICAO:      "WMKC",
		IATA:      "KBR",
		Name:      "Sultan Ismail Petra Airport",
		City:      "Kota Baharu",
		State:     "Kelantan",
		Country:   "MY",
		Elevation: 16,
		Latitude:  6.16685009,
		Longitude: 102.2929992676,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMKD": {
		ICAO:      "WMKD",
		IATA:      "KUA",
		Name:      "Kuantan Airport",
		City:      "Kuantan",
		State:     "Pahang",
		Country:   "MY",
		Elevation: 58,
		Latitude:  3.7753899097,
		Longitude: 103.2089996338,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMKE": {
		ICAO:      "WMKE",
		IATA:      "KTE",
		Name:      "Kerteh Airport",
		City:      "Kerteh",
		State:     "Terengganu",
		Country:   "MY",
		Elevation: 18,
		Latitude:  4.5372200012,
		Longitude: 103.4270019531,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMKF": {
		ICAO:      "WMKF",
		Name:      "Simpang Airport",
		City:      "Kuala Lumpur",
		Country:   "MY",
		Elevation: 111,
		Latitude:  3.1122500896,
		Longitude: 101.7030029297,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMKI": {
		ICAO:      "WMKI",
		IATA:      "IPH",
		Name:      "Sultan Azlan Shah Airport",
		City:      "Ipoh",
		Country:   "MY",
		Elevation: 130,
		Latitude:  4.567969799,
		Longitude: 101.0920028687,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMKJ": {
		ICAO:      "WMKJ",
		IATA:      "JHB",
		Name:      "Senai International Airport",
		City:      "Senai",
		State:     "Johor",
		Country:   "MY",
		Elevation: 135,
		Latitude:  1.6413099766,
		Longitude: 103.6699981689,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMKK": {
		ICAO:      "WMKK",
		IATA:      "KUL",
		Name:      "Kuala Lumpur International Airport",
		City:      "Kuala Lumpur",
		Country:   "MY",
		Elevation: 69,
		Latitude:  2.745579958,
		Longitude: 101.7099990845,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMKL": {
		ICAO:      "WMKL",
		IATA:      "LGK",
		Name:      "Langkawi International Airport",
		City:      "Langkawi",
		State:     "Kedah",
		Country:   "MY",
		Elevation: 29,
		Latitude:  6.3297300339,
		Longitude: 99.7286987305,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMKM": {
		ICAO:      "WMKM",
		IATA:      "MKZ",
		Name:      "Malacca Airport",
		City:      "Malacca",
		Country:   "MY",
		Elevation: 35,
		Latitude:  2.2633600235,
		Longitude: 102.251998901,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMKN": {
		ICAO:      "WMKN",
		IATA:      "TGG",
		Name:      "Sultan Mahmud Airport",
		City:      "Kuala Terengganu",
		State:     "Terengganu",
		Country:   "MY",
		Elevation: 21,
		Latitude:  5.3826398849,
		Longitude: 103.1029968262,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMKP": {
		ICAO:      "WMKP",
		IATA:      "PEN",
		Name:      "Penang International Airport",
		City:      "Penang",
		Country:   "MY",
		Elevation: 11,
		Latitude:  5.2971401215,
		Longitude: 100.2770004272,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMLU": {
		ICAO:      "WMLU",
		Name:      "Lutong Airport",
		State:     "Sarawak",
		Country:   "MY",
		Latitude:  4.4499998093,
		Longitude: 114,
		Timezone:  "Asia/Kuching",
	},
	"WMPA": {
		ICAO:      "WMPA",
		IATA:      "PKG",
		Name:      "Pulau Pangkor Airport",
		City:      "Pangkor Island",
		Country:   "MY",
		Elevation: 19,
		Latitude:  4.2447199821,
		Longitude: 100.5530014038,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMPR": {
		ICAO:      "WMPR",
		IATA:      "RDN",
		Name:      "LTS Pulau Redang Airport",
		City:      "Redang",
		Country:   "MY",
		Elevation: 36,
		Latitude:  5.7652797699,
		Longitude: 103.0070037842,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WMSA": {
		ICAO:      "WMSA",
		IATA:      "SZB",
		Name:      "Sultan Abdul Aziz Shah International Airport",
		City:      "Subang",
		Country:   "MY",
		Elevation: 90,
		Latitude:  3.1305799484,
		Longitude: 101.5490036011,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WN00": {
		ICAO:      "WN00",
		Name:      "Kimshan Ranch Airport",
		City:      "Quilcene",
		Country:   "US",
		Elevation: 80,
		Latitude:  47.8400993347,
		Longitude: -122.8600006104,
		Timezone:  "America/Los_Angeles",
	},
	"WN03": {
		ICAO:      "WN03",
		Name:      "Van Der Vaart Airport",
		City:      "Sheboygan",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 622,
		Latitude:  43.6786003113,
		Longitude: -87.7256011963,
		Timezone:  "America/Chicago",
	},
	"WN04": {
		ICAO:      "WN04",
		Name:      "Erickson Ranch Airport",
		City:      "Colville",
		State:     "Washington",
		Country:   "US",
		Elevation: 2924,
		Latitude:  48.3923988342,
		Longitude: -118.0989990234,
		Timezone:  "America/Los_Angeles",
	},
	"WN05": {
		ICAO:      "WN05",
		Name:      "Harris Airport",
		City:      "Toledo",
		State:     "Washington",
		Country:   "US",
		Elevation: 385,
		Latitude:  46.5148010254,
		Longitude: -122.7919998169,
		Timezone:  "America/Los_Angeles",
	},
	"WN07": {
		ICAO:      "WN07",
		IATA:      "DTR",
		Name:      "Decatur Shores Airport",
		City:      "Decatur",
		State:     "Washington",
		Country:   "US",
		Elevation: 38,
		Latitude:  48.4998016357,
		Longitude: -122.814002991,
		Timezone:  "America/Los_Angeles",
	},
	"WN08": {
		ICAO:      "WN08",
		Name:      "Kendall Airstrip",
		City:      "Kendall",
		Country:   "US",
		Elevation: 548,
		Latitude:  48.9147987366,
		Longitude: -122.1129989624,
		Timezone:  "America/Los_Angeles",
	},
	"WN09": {
		ICAO:      "WN09",
		Name:      "Bucky's Airpark",
		City:      "Plainfield",
		Country:   "US",
		Elevation: 1145,
		Latitude:  44.1643981934,
		Longitude: -89.508102417,
		Timezone:  "America/Chicago",
	},
	"WN10": {
		ICAO:      "WN10",
		Name:      "Mount St Helen's Aero Ranch Airport",
		City:      "Cougar",
		State:     "Washington",
		Country:   "US",
		Elevation: 450,
		Latitude:  45.9934005737,
		Longitude: -122.3799972534,
		Timezone:  "America/Los_Angeles",
	},
	"WN11": {
		ICAO:      "WN11",
		Name:      "Red Roof Airport",
		City:      "Beecher",
		Country:   "US",
		Elevation: 963,
		Latitude:  45.5858001709,
		Longitude: -88.0037002563,
		Timezone:  "America/Chicago",
	},
	"WN13": {
		ICAO:      "WN13",
		Name:      "Vaughan Ranch Airfield",
		City:      "Port Orchard",
		Country:   "US",
		Elevation: 240,
		Latitude:  47.4636993408,
		Longitude: -122.5690002441,
		Timezone:  "America/Los_Angeles",
	},
	"WN14": {
		ICAO:      "WN14",
		Name:      "Pete's Airport",
		City:      "Deer Park",
		State:     "Washington",
		Country:   "US",
		Elevation: 2160,
		Latitude:  47.8415985107,
		Longitude: -117.4469985962,
		Timezone:  "America/Los_Angeles",
	},
	"WN15": {
		ICAO:      "WN15",
		Name:      "Burnett Landing Airport",
		City:      "Wilkeson",
		State:     "Washington",
		Country:   "US",
		Elevation: 800,
		Latitude:  47.129699707,
		Longitude: -122.0559997559,
		Timezone:  "America/Los_Angeles",
	},
	"WN17": {
		ICAO:      "WN17",
		Name:      "Hoverhawk Ranch Airport",
		City:      "Wenatchee",
		Country:   "US",
		Elevation: 2450,
		Latitude:  47.3398017883,
		Longitude: -120.3160018921,
		Timezone:  "America/Los_Angeles",
	},
	"WN18": {
		ICAO:      "WN18",
		Name:      "Becker's Landing Airport",
		City:      "Anacortes",
		Country:   "US",
		Elevation: 155,
		Latitude:  48.4567985535,
		Longitude: -122.5390014648,
		Timezone:  "America/Los_Angeles",
	},
	"WN20": {
		ICAO:      "WN20",
		Name:      "Van De Plasch Airport",
		City:      "Monroe",
		State:     "Washington",
		Country:   "US",
		Elevation: 60,
		Latitude:  47.8722991943,
		Longitude: -121.922996521,
		Timezone:  "America/Los_Angeles",
	},
	"WN21": {
		ICAO:      "WN21",
		Name:      "Lawson Airpark",
		City:      "Port Angeles",
		State:     "Washington",
		Country:   "US",
		Elevation: 650,
		Latitude:  48.0815010071,
		Longitude: -123.3980026245,
		Timezone:  "America/Los_Angeles",
	},
	"WN23": {
		ICAO:      "WN23",
		Name:      "Grand View International Airport",
		City:      "Sequim",
		State:     "Washington",
		Country:   "US",
		Elevation: 150,
		Latitude:  48.1017990112,
		Longitude: -123.1800003052,
		Timezone:  "America/Los_Angeles",
	},
	"WN24": {
		ICAO:      "WN24",
		Name:      "Cougar Flat Airstrip",
		City:      "Castle Rock",
		State:     "Washington",
		Country:   "US",
		Elevation: 180,
		Latitude:  46.2925987244,
		Longitude: -122.9499969482,
		Timezone:  "America/Los_Angeles",
	},
	"WN26": {
		ICAO:      "WN26",
		Name:      "Schoepflin Airport",
		City:      "Palouse",
		State:     "Washington",
		Country:   "US",
		Elevation: 2607,
		Latitude:  46.8849983215,
		Longitude: -117.077003479,
		Timezone:  "America/Los_Angeles",
	},
	"WN27": {
		ICAO:      "WN27",
		Name:      "Redoft Airport",
		City:      "Collins",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 815,
		Latitude:  44.0792007446,
		Longitude: -87.9757003784,
		Timezone:  "America/Chicago",
	},
	"WN28": {
		ICAO:      "WN28",
		Name:      "Sunny Slope Runway Airport",
		City:      "Egg Harbor",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 730,
		Latitude:  44.98279953,
		Longitude: -87.3095016479,
		Timezone:  "America/Chicago",
	},
	"WN29": {
		ICAO:      "WN29",
		Name:      "Blue Ribbon Airport",
		City:      "Sequim",
		State:     "Washington",
		Country:   "US",
		Elevation: 100,
		Latitude:  48.1304016113,
		Longitude: -123.2070007324,
		Timezone:  "America/Los_Angeles",
	},
	"WN30": {
		ICAO:      "WN30",
		Name:      "Clinesmith Ranch Airport",
		City:      "Benge",
		State:     "Washington",
		Country:   "US",
		Elevation: 1600,
		Latitude:  46.9198989868,
		Longitude: -118.0920028687,
		Timezone:  "America/Los_Angeles",
	},
	"WN31": {
		ICAO:      "WN31",
		Name:      "Slinkard Airfield",
		City:      "Mesa",
		State:     "Washington",
		Country:   "US",
		Elevation: 890,
		Latitude:  46.6031990051,
		Longitude: -119.06300354,
		Timezone:  "America/Los_Angeles",
	},
	"WN33": {
		ICAO:      "WN33",
		Name:      "Columbia Ag 2 Airport",
		City:      "Pasco",
		State:     "Washington",
		Country:   "US",
		Elevation: 715,
		Latitude:  46.3804016113,
		Longitude: -118.987998962,
		Timezone:  "America/Los_Angeles",
	},
	"WN34": {
		ICAO:      "WN34",
		Name:      "Ethel International Airport",
		City:      "Ethel",
		State:     "Washington",
		Country:   "US",
		Elevation: 580,
		Latitude:  46.5308990479,
		Longitude: -122.6839981079,
		Timezone:  "America/Los_Angeles",
	},
	"WN35": {
		ICAO:      "WN35",
		Name:      "Meadow Mist Airport",
		City:      "Ferndale",
		State:     "Washington",
		Country:   "US",
		Elevation: 55,
		Latitude:  48.9086990356,
		Longitude: -122.5650024414,
		Timezone:  "America/Los_Angeles",
	},
	"WN36": {
		ICAO:      "WN36",
		Name:      "Kari Field",
		City:      "Olympia",
		State:     "Washington",
		Country:   "US",
		Elevation: 224,
		Latitude:  46.96900177,
		Longitude: -122.8209991455,
		Timezone:  "America/Los_Angeles",
	},
	"WN39": {
		ICAO:      "WN39",
		Name:      "Knutson Field",
		City:      "Wyoceena",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 875,
		Latitude:  43.4818992615,
		Longitude: -89.320098877,
		Timezone:  "America/Chicago",
	},
	"WN40": {
		ICAO:      "WN40",
		Name:      "Coupeville Airpark",
		City:      "Coupeville",
		State:     "Washington",
		Country:   "US",
		Elevation: 200,
		Latitude:  48.1903991699,
		Longitude: -122.6429977417,
		Timezone:  "America/Los_Angeles",
	},
	"WN41": {
		ICAO:      "WN41",
		Name:      "Redfern Aerodrome",
		City:      "Sprague",
		State:     "Washington",
		Country:   "US",
		Elevation: 2050,
		Latitude:  47.3044013977,
		Longitude: -117.9619979858,
		Timezone:  "America/Los_Angeles",
	},
	"WN42": {
		ICAO:      "WN42",
		Name:      "Flying H Ranch Airport",
		City:      "Buckley",
		State:     "Washington",
		Country:   "US",
		Elevation: 587,
		Latitude:  47.1837997437,
		Longitude: -122.1200027466,
		Timezone:  "America/Los_Angeles",
	},
	"WN44": {
		ICAO:      "WN44",
		Name:      "Podeweltz Airport",
		City:      "Merrill",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1470,
		Latitude:  45.2393989563,
		Longitude: -89.5690002441,
		Timezone:  "America/Chicago",
	},
	"WN45": {
		ICAO:      "WN45",
		Name:      "Take Five Airport",
		City:      "Yakima",
		State:     "Washington",
		Country:   "US",
		Elevation: 1543,
		Latitude:  46.77930069,
		Longitude: -120.4440002,
		Timezone:  "America/Los_Angeles",
	},
	"WN46": {
		ICAO:      "WN46",
		Name:      "Battle Creek Airport",
		City:      "Oconomowoc",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 865,
		Latitude:  43.0675010681,
		Longitude: -88.5086975098,
		Timezone:  "America/Chicago",
	},
	"WN47": {
		ICAO:      "WN47",
		Name:      "Bear Valley Skyranch Airport",
		City:      "Mc Cleary",
		State:     "Washington",
		Country:   "US",
		Elevation: 344,
		Latitude:  47.0816993713,
		Longitude: -123.2699966431,
		Timezone:  "America/Los_Angeles",
	},
	"WN48": {
		ICAO:      "WN48",
		Name:      "Kimbrel Farm Airport",
		City:      "Oakville",
		State:     "Washington",
		Country:   "US",
		Elevation: 139,
		Latitude:  46.7904014587,
		Longitude: -123.3099975586,
		Timezone:  "America/Los_Angeles",
	},
	"WN49": {
		ICAO:      "WN49",
		Name:      "Flying S Airfield",
		City:      "Sequim",
		State:     "Washington",
		Country:   "US",
		Elevation: 20,
		Latitude:  48.121799469,
		Longitude: -123.087997437,
		Timezone:  "America/Los_Angeles",
	},
	"WN51": {
		ICAO:      "WN51",
		Name:      "Bayview Farms Airport",
		City:      "Burlington",
		State:     "Washington",
		Country:   "US",
		Elevation: 6,
		Latitude:  48.5209007263,
		Longitude: -122.4649963379,
		Timezone:  "America/Los_Angeles",
	},
	"WN53": {
		ICAO:      "WN53",
		Name:      "Frontier Airpark",
		City:      "Lake Stevens",
		State:     "Washington",
		Country:   "US",
		Elevation: 480,
		Latitude:  48.1104011536,
		Longitude: -122.06199646,
		Timezone:  "America/Los_Angeles",
	},
	"WN54": {
		ICAO:      "WN54",
		Name:      "Ellerport Airport",
		City:      "Newman Lake",
		State:     "Washington",
		Country:   "US",
		Elevation: 2146,
		Latitude:  47.7349014282,
		Longitude: -117.0709991455,
		Timezone:  "America/Los_Angeles",
	},
	"WN55": {
		ICAO:      "WN55",
		Name:      "Randle-Kiona Airpark",
		City:      "Randle",
		State:     "Washington",
		Country:   "US",
		Elevation: 934,
		Latitude:  46.5115013123,
		Longitude: -122.0059967041,
		Timezone:  "America/Los_Angeles",
	},
	"WN59": {
		ICAO:      "WN59",
		Name:      "Nelsons Nitch Airport",
		City:      "Onalaska",
		State:     "Washington",
		Country:   "US",
		Elevation: 520,
		Latitude:  46.53760147,
		Longitude: -122.7180023,
		Timezone:  "America/Los_Angeles",
	},
	"WN60": {
		ICAO:      "WN60",
		Name:      "Swanton Ultralightport",
		City:      "Sequim",
		State:     "Washington",
		Country:   "US",
		Elevation: 110,
		Latitude:  48.1405982971,
		Longitude: -123.18800354,
		Timezone:  "America/Los_Angeles",
	},
	"WN61": {
		ICAO:      "WN61",
		Name:      "Tai's Landing Airport",
		City:      "Spokane",
		State:     "Washington",
		Country:   "US",
		Elevation: 2450,
		Latitude:  47.7252006531,
		Longitude: -117.2799987793,
		Timezone:  "America/Los_Angeles",
	},
	"WN63": {
		ICAO:      "WN63",
		Name:      "Bristol Airport",
		City:      "Bristol",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 735,
		Latitude:  42.5367012024,
		Longitude: -88.0039978027,
		Timezone:  "America/Chicago",
	},
	"WN64": {
		ICAO:      "WN64",
		Name:      "J K D Farms Airport",
		City:      "Ellensburg",
		State:     "Washington",
		Country:   "US",
		Elevation: 1575,
		Latitude:  46.9743003845,
		Longitude: -120.4619979858,
		Timezone:  "America/Los_Angeles",
	},
	"WN65": {
		ICAO:      "WN65",
		Name:      "My Airport",
		City:      "Elma",
		State:     "Washington",
		Country:   "US",
		Elevation: 80,
		Latitude:  46.9725990295,
		Longitude: -123.4769973755,
		Timezone:  "America/Los_Angeles",
	},
	"WN72": {
		ICAO:      "WN72",
		Name:      "Kinch Farms Airport",
		City:      "Washtucna",
		State:     "Washington",
		Country:   "US",
		Elevation: 1650,
		Latitude:  46.8648986816,
		Longitude: -118.3249969482,
		Timezone:  "America/Los_Angeles",
	},
	"WN74": {
		ICAO:      "WN74",
		Name:      "Burnt Ridge Airstrip",
		City:      "Onalaska",
		State:     "Washington",
		Country:   "US",
		Elevation: 1070,
		Latitude:  46.5853996277,
		Longitude: -122.6259994507,
		Timezone:  "America/Los_Angeles",
	},
	"WN75": {
		ICAO:      "WN75",
		Name:      "Erin Aero Airport",
		City:      "Hartford",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1000,
		Latitude:  43.2447013855,
		Longitude: -88.3722991943,
		Timezone:  "America/Chicago",
	},
	"WN76": {
		ICAO:      "WN76",
		Name:      "Bergseth Field",
		City:      "Enumclaw",
		State:     "Washington",
		Country:   "US",
		Elevation: 1100,
		Latitude:  47.243598938,
		Longitude: -121.9250030518,
		Timezone:  "America/Los_Angeles",
	},
	"WN80": {
		ICAO:      "WN80",
		Name:      "Walters Arv Ultralightport",
		City:      "Kelso",
		State:     "Washington",
		Country:   "US",
		Elevation: 475,
		Latitude:  46.0979003906,
		Longitude: -122.8649978638,
		Timezone:  "America/Los_Angeles",
	},
	"WN85": {
		ICAO:      "WN85",
		Name:      "Morrisonville International Airport",
		City:      "Morrisonville",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 960,
		Latitude:  43.2747001648,
		Longitude: -89.350402832,
		Timezone:  "America/Chicago",
	},
	"WN86": {
		ICAO:      "WN86",
		Name:      "St Croix Valley Airport",
		City:      "Osceola",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1085,
		Latitude:  45.2639007568,
		Longitude: -92.62159729,
		Timezone:  "America/Chicago",
	},
	"WN87": {
		ICAO:      "WN87",
		Name:      "Bryan Airport",
		City:      "Enumclaw",
		State:     "Washington",
		Country:   "US",
		Elevation: 630,
		Latitude:  47.2085990906,
		Longitude: -122.0930023193,
		Timezone:  "America/Los_Angeles",
	},
	"WN88": {
		ICAO:      "WN88",
		Name:      "Martin Airport",
		City:      "Ocean Park",
		State:     "Washington",
		Country:   "US",
		Elevation: 10,
		Latitude:  46.5192985535,
		Longitude: -124.0319976807,
		Timezone:  "America/Los_Angeles",
	},
	"WN89": {
		ICAO:      "WN89",
		Name:      "Lee's Airport",
		City:      "Chattaroy",
		State:     "Washington",
		Country:   "US",
		Elevation: 1890,
		Latitude:  47.8876991272,
		Longitude: -117.34400177,
		Timezone:  "America/Los_Angeles",
	},
	"WN90": {
		ICAO:      "WN90",
		Name:      "Taylorport Airport",
		City:      "Bruce",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1250,
		Latitude:  45.375,
		Longitude: -91.2918014526,
		Timezone:  "America/Chicago",
	},
	"WN92": {
		ICAO:      "WN92",
		Name:      "Sky Meadows Airpark",
		City:      "Spokane",
		State:     "Washington",
		Country:   "US",
		Elevation: 2350,
		Latitude:  47.6085014343,
		Longitude: -117.1809997559,
		Timezone:  "America/Los_Angeles",
	},
	"WN95": {
		ICAO:      "WN95",
		Name:      "Brown Boy Airport",
		City:      "Vantage",
		State:     "Washington",
		Country:   "US",
		Elevation: 1125,
		Latitude:  46.8848991394,
		Longitude: -119.8909988403,
		Timezone:  "America/Los_Angeles",
	},
	"WN98": {
		ICAO:      "WN98",
		Name:      "Florida North Airport",
		City:      "Birchwood",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1272,
		Latitude:  45.6594009399,
		Longitude: -91.5224990845,
		Timezone:  "America/Chicago",
	},
	"WN99": {
		ICAO:      "WN99",
		Name:      "Hayes Road Airport",
		City:      "Durand",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 900,
		Latitude:  44.6571998596,
		Longitude: -91.8086013794,
		Timezone:  "America/Chicago",
	},
	"WPAT": {
		ICAO:      "WPAT",
		IATA:      "AUT",
		Name:      "Atauro Airport",
		City:      "Atauro",
		State:     "Dili",
		Country:   "TL",
		Elevation: 29,
		Latitude:  -8.2105197906,
		Longitude: 125.6159973145,
		Timezone:  "Asia/Dili",
	},
	"WPDB": {
		ICAO:      "WPDB",
		IATA:      "UAI",
		Name:      "Suai Airport",
		City:      "Suai",
		State:     "Cova-Lima",
		Country:   "TL",
		Elevation: 96,
		Latitude:  -9.3033103943,
		Longitude: 125.2870025635,
		Timezone:  "Asia/Dili",
	},
	"WPDL": {
		ICAO:      "WPDL",
		IATA:      "DIL",
		Name:      "Presidente Nicolau Lobato International Airport",
		City:      "Dili",
		State:     "Dili",
		Country:   "TL",
		Elevation: 154,
		Latitude:  -8.5464000702,
		Longitude: 125.526000977,
		Timezone:  "Asia/Dili",
	},
	"WPEC": {
		ICAO:      "WPEC",
		IATA:      "BCH",
		Name:      "Cakung Airport",
		City:      "Baucau",
		State:     "Baucau",
		Country:   "TL",
		Elevation: 1771,
		Latitude:  -8.4890298843,
		Longitude: 126.401000977,
		Timezone:  "Asia/Dili",
	},
	"WPFL": {
		ICAO:      "WPFL",
		Name:      "Abisu Airfield",
		City:      "Fuiloro",
		State:     "Lautem",
		Country:   "TL",
		Latitude:  -8.4436101913,
		Longitude: 126.9889984131,
		Timezone:  "Asia/Dili",
	},
	"WPMN": {
		ICAO:      "WPMN",
		IATA:      "MPT",
		Name:      "Maliana Airport",
		City:      "Maliana",
		State:     "Bobonaro",
		Country:   "TL",
		Latitude:  -8.972240448,
		Longitude: 125.2149963379,
		Timezone:  "Asia/Dili",
	},
	"WPOC": {
		ICAO:      "WPOC",
		IATA:      "OEC",
		Name:      "Oecussi Airport",
		City:      "Oecussi-Ambeno",
		State:     "Oecusse",
		Country:   "TL",
		Latitude:  -9.1980600357,
		Longitude: 124.3430023193,
		Timezone:  "Asia/Dili",
	},
	"WPVQ": {
		ICAO:      "WPVQ",
		IATA:      "VIQ",
		Name:      "Viqueque Airport",
		City:      "Viqueque",
		State:     "Viqueque",
		Country:   "TL",
		Elevation: 500,
		Latitude:  -8.8837995529,
		Longitude: 126.3730010986,
		Timezone:  "Asia/Dili",
	},
	"WRBU": {
		ICAO:      "WRBU",
		Name:      "Buntok Airport",
		City:      "Buntok-Borneo Island",
		State:     "Central-Kalimantan",
		Country:   "ID",
		Elevation: 59,
		Latitude:  -1.7325099707,
		Longitude: 114.8389968872,
		Timezone:  "Asia/Pontianak",
	},
	"WRKA": {
		ICAO:      "WRKA",
		IATA:      "ABU",
		Name:      "Haliwen Airport",
		City:      "Atambua-Timor Island",
		State:     "East-Nusa-Tenggara",
		Country:   "ID",
		Elevation: 1027,
		Latitude:  -9.0730495453,
		Longitude: 124.9049987793,
		Timezone:  "Asia/Makassar",
	},
	"WRKB": {
		ICAO:      "WRKB",
		IATA:      "BJW",
		Name:      "Padhameleda Airport",
		City:      "Bajawa-Flores Island",
		State:     "East-Nusa-Tenggara",
		Country:   "ID",
		Elevation: 4317,
		Latitude:  -8.808139801,
		Longitude: 120.9960021973,
		Timezone:  "Asia/Makassar",
	},
	"WRKJ": {
		ICAO:      "WRKJ",
		Name:      "Mena Airport",
		City:      "Mena-Timor Island",
		State:     "East-Nusa-Tenggara",
		Country:   "ID",
		Elevation: 320,
		Latitude:  -9.183889389,
		Longitude: 124.5891647339,
		Timezone:  "Asia/Makassar",
	},
	"WRKL": {
		ICAO:      "WRKL",
		IATA:      "LKA",
		Name:      "Gewayentana Airport",
		City:      "Larantuka-Flores Island",
		State:     "East-Nusa-Tenggara",
		Country:   "ID",
		Elevation: 57,
		Latitude:  -8.3466596603,
		Longitude: 122.9810028076,
		Timezone:  "Asia/Makassar",
	},
	"WRKM": {
		ICAO:      "WRKM",
		Name:      "Kalabahi Airport",
		City:      "Kalabahi-Alor Island",
		State:     "East-Nusa-Tenggara",
		Country:   "ID",
		Elevation: 33,
		Latitude:  -8.2166671753,
		Longitude: 124.5666656494,
		Timezone:  "Asia/Makassar",
	},
	"WRKN": {
		ICAO:      "WRKN",
		Name:      "Naikliu Airport",
		City:      "Naikliu-Timor Island",
		Country:   "ID",
		Elevation: 60,
		Latitude:  -9.495470047,
		Longitude: 123.7949981689,
		Timezone:  "Asia/Makassar",
	},
	"WRKR": {
		ICAO:      "WRKR",
		Name:      "Rote Airport",
		City:      "Namodale-Rote Island",
		State:     "East-Nusa-Tenggara",
		Country:   "ID",
		Elevation: 300,
		Latitude:  -10.7166671753,
		Longitude: 123.0666656494,
		Timezone:  "Asia/Makassar",
	},
	"WRKS": {
		ICAO:      "WRKS",
		IATA:      "SAU",
		Name:      "Sawu Airport",
		City:      "Sawu-Sawu Island",
		State:     "East-Nusa-Tenggara",
		Country:   "ID",
		Elevation: 530,
		Latitude:  -10.483300209,
		Longitude: 121.9000015259,
		Timezone:  "Asia/Makassar",
	},
	"WRLA": {
		ICAO:      "WRLA",
		IATA:      "SGQ",
		Name:      "Sanggata Airport",
		City:      "Sanggata-Timor Island",
		State:     "East-Kalimantan",
		Country:   "ID",
		Elevation: 434,
		Latitude:  0.5,
		Longitude: 117.3333587646,
		Timezone:  "Asia/Makassar",
	},
	"WRLB": {
		ICAO:      "WRLB",
		IATA:      "LBW",
		Name:      "Long Bawan Airport",
		City:      "Long Bawan-Borneo Island",
		State:     "North-Kalimantan",
		Country:   "ID",
		Elevation: 3590,
		Latitude:  3.867000103,
		Longitude: 115.6829986572,
		Timezone:  "Asia/Makassar",
	},
	"WRLC": {
		ICAO:      "WRLC",
		IATA:      "BXT",
		Name:      "Bontang Airport",
		City:      "Bontang-Borneo Island",
		State:     "East-Kalimantan",
		Country:   "ID",
		Elevation: 49,
		Latitude:  0.1196909994,
		Longitude: 117.474998474,
		Timezone:  "Asia/Makassar",
	},
	"WRLF": {
		ICAO:      "WRLF",
		IATA:      "NNX",
		Name:      "Nunukan Airport",
		City:      "Nunukan-Nunukan Island",
		State:     "East-Kalimantan",
		Country:   "ID",
		Elevation: 52,
		Latitude:  4.1365299225,
		Longitude: 117.6669998169,
		Timezone:  "Asia/Kuching",
	},
	"WRLH": {
		ICAO:      "WRLH",
		IATA:      "TNB",
		Name:      "Tanah Grogot Airport",
		City:      "Tanah Grogot-Borneo Island",
		State:     "East-Kalimantan",
		Country:   "ID",
		Elevation: 17,
		Latitude:  -1.910130024,
		Longitude: 116.202003479,
		Timezone:  "Asia/Makassar",
	},
	"WRLL": {
		ICAO:      "WRLL",
		Name:      "Balikpapan Airport",
		City:      "Seppingan-Borneo Island",
		State:     "East-Kalimantan",
		Country:   "ID",
		Elevation: 8,
		Latitude:  -1.2666670084,
		Longitude: 116.8833312988,
		Timezone:  "Asia/Makassar",
	},
	"WRLM": {
		ICAO:      "WRLM",
		Name:      "Malinau Airport",
		City:      "Malinau-Borneo Island",
		State:     "South-Kalimantan",
		Country:   "ID",
		Elevation: 96,
		Latitude:  -3.5527200699,
		Longitude: 116.6259994507,
		Timezone:  "Asia/Makassar",
	},
	"WRLN": {
		ICAO:      "WRLN",
		Name:      "Long Mawang Airport",
		City:      "Long Mawang-Borneo Island",
		State:     "North-Kalimantan",
		Country:   "ID",
		Elevation: 2343,
		Latitude:  1.8141599894,
		Longitude: 114.8840026855,
		Timezone:  "Asia/Makassar",
	},
	"WRLP": {
		ICAO:      "WRLP",
		IATA:      "LPU",
		Name:      "Long Apung Airport",
		City:      "Long Apung-Borneo Island",
		State:     "East-Kalimantan",
		Country:   "ID",
		Elevation: 627,
		Latitude:  1.704486,
		Longitude: 114.970297,
		Timezone:  "Asia/Makassar",
	},
	"WRLU": {
		ICAO:      "WRLU",
		Name:      "Sangkulirang Airport",
		City:      "Sangkulirang-Borneo Island",
		State:     "East-Kalimantan",
		Country:   "ID",
		Elevation: 34,
		Latitude:  1.0008900166,
		Longitude: 117.9960021973,
		Timezone:  "Asia/Makassar",
	},
	"WRLW": {
		ICAO:      "WRLW",
		Name:      "Muara Wahau Airport",
		City:      "Muara Wahau-Borneo Island",
		State:     "East-Kalimantan",
		Country:   "ID",
		Elevation: 135,
		Latitude:  1.0328199863,
		Longitude: 116.9110031128,
		Timezone:  "Asia/Makassar",
	},
	"WRSP": {
		ICAO:      "WRSP",
		Name:      "Surabaya Airport",
		City:      "Surabaya-Java Island",
		Country:   "ID",
		Elevation: 9,
		Latitude:  -7.2166671753,
		Longitude: 112.7166671753,
		Timezone:  "Asia/Jakarta",
	},
	"WS02": {
		ICAO:      "WS02",
		Name:      "Polish Paradise Airport",
		City:      "Oxford",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1030,
		Latitude:  43.7425003052,
		Longitude: -89.716003418,
		Timezone:  "America/Chicago",
	},
	"WS04": {
		ICAO:      "WS04",
		Name:      "Zanadu Airport",
		City:      "Arkdale",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 950,
		Latitude:  44.0325012207,
		Longitude: -89.8650970459,
		Timezone:  "America/Chicago",
	},
	"WS05": {
		ICAO:      "WS05",
		Name:      "Lonely Pines Airport",
		City:      "Kennan",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1365,
		Latitude:  45.4771995544,
		Longitude: -90.6250991821,
		Timezone:  "America/Chicago",
	},
	"WS06": {
		ICAO:      "WS06",
		Name:      "Springstead Airport",
		City:      "Park Falls",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1600,
		Latitude:  45.987701416,
		Longitude: -90.1634979248,
		Timezone:  "America/Chicago",
	},
	"WS08": {
		ICAO:      "WS08",
		Name:      "Five Corners Airways Airport",
		City:      "Appleton",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 815,
		Latitude:  44.4179992676,
		Longitude: -88.3769989014,
		Timezone:  "America/Chicago",
	},
	"WS09": {
		ICAO:      "WS09",
		Name:      "Rox Airport",
		City:      "Coloma",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1017,
		Latitude:  44.0133018494,
		Longitude: -89.5248031616,
		Timezone:  "America/Chicago",
	},
	"WS10": {
		ICAO:      "WS10",
		Name:      "Casey Lake Airport",
		City:      "Waupaca",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 850,
		Latitude:  44.408000946,
		Longitude: -89.0158996582,
		Timezone:  "America/Chicago",
	},
	"WS11": {
		ICAO:      "WS11",
		Name:      "Frievalt Airport",
		City:      "Pound",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 830,
		Latitude:  45.1002998352,
		Longitude: -88.1333999634,
		Timezone:  "America/Chicago",
	},
	"WS12": {
		ICAO:      "WS12",
		Name:      "Elert Airport",
		City:      "De Forest",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 972,
		Latitude:  43.2644004822,
		Longitude: -89.3231964111,
		Timezone:  "America/Chicago",
	},
	"WS13": {
		ICAO:      "WS13",
		Name:      "Cloud Dancer Private Airport",
		City:      "Chetek",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1050,
		Latitude:  45.3119010925,
		Longitude: -91.5896987915,
		Timezone:  "America/Chicago",
	},
	"WS14": {
		ICAO:      "WS14",
		Name:      "Lake Ell Field",
		City:      "Amherst",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1113,
		Latitude:  44.464099884,
		Longitude: -89.3587036133,
		Timezone:  "America/Chicago",
	},
	"WS15": {
		ICAO:      "WS15",
		Name:      "Mill House Field",
		City:      "Wyocena",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 820,
		Latitude:  43.4952011108,
		Longitude: -89.2654037476,
		Timezone:  "America/Chicago",
	},
	"WS16": {
		ICAO:      "WS16",
		Name:      "Scherrico Meadows Airport",
		City:      "Mosinee",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1345,
		Latitude:  44.8471984863,
		Longitude: -89.8014984131,
		Timezone:  "America/Chicago",
	},
	"WS17": {
		ICAO:      "WS17",
		Name:      "Pioneer Airport",
		City:      "Oshkosh",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 826,
		Latitude:  43.9893989563,
		Longitude: -88.5783996582,
		Timezone:  "America/Chicago",
	},
	"WS19": {
		ICAO:      "WS19",
		Name:      "Petit Cache Airport",
		City:      "Bayfield",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 760,
		Latitude:  46.8935012817,
		Longitude: -90.8207015991,
		Timezone:  "America/Chicago",
	},
	"WS20": {
		ICAO:      "WS20",
		Name:      "Young Tactical Landing Site Airport",
		City:      "Fort Mc Coy",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 912,
		Latitude:  43.95299912,
		Longitude: -90.66259766,
		Timezone:  "America/Chicago",
	},
	"WS22": {
		ICAO:      "WS22",
		Name:      "Wolfgram Airport",
		City:      "New London",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 775,
		Latitude:  44.3946990967,
		Longitude: -88.6894989014,
		Timezone:  "America/Chicago",
	},
	"WS23": {
		ICAO:      "WS23",
		Name:      "R & S Landing Strip",
		City:      "Merrill",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1309,
		Latitude:  45.2160987854,
		Longitude: -89.7472000122,
		Timezone:  "America/Chicago",
	},
	"WS25": {
		ICAO:      "WS25",
		Name:      "Shangrila Airport",
		City:      "Beecher",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 835,
		Latitude:  45.5816001892,
		Longitude: -87.8453979492,
		Timezone:  "America/Chicago",
	},
	"WS26": {
		ICAO:      "WS26",
		Name:      "Ranch Side Airport",
		City:      "Kewaunee",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 710,
		Latitude:  44.3591995239,
		Longitude: -87.6070022583,
		Timezone:  "America/Chicago",
	},
	"WS28": {
		ICAO:      "WS28",
		Name:      "Coleman Airport",
		City:      "Portage",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 800,
		Latitude:  43.5536003113,
		Longitude: -89.3898010254,
		Timezone:  "America/Chicago",
	},
	"WS29": {
		ICAO:      "WS29",
		Name:      "Tachick Field",
		City:      "Pound",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 850,
		Latitude:  45.120300293,
		Longitude: -88.1434020996,
		Timezone:  "America/Chicago",
	},
	"WS30": {
		ICAO:      "WS30",
		Name:      "Hexum Flight Park Ultralightport",
		City:      "Star Prairie",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 930,
		Latitude:  45.1890983582,
		Longitude: -92.5483016968,
		Timezone:  "America/Chicago",
	},
	"WS31": {
		ICAO:      "WS31",
		Name:      "Otto-Gibbons Airport",
		City:      "Wayside",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 850,
		Latitude:  44.2778015137,
		Longitude: -87.9445037842,
		Timezone:  "America/Chicago",
	},
	"WS32": {
		ICAO:      "WS32",
		Name:      "Prescott Field",
		City:      "Wyocena",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 875,
		Latitude:  43.4836006165,
		Longitude: -89.2947998047,
		Timezone:  "America/Chicago",
	},
	"WS33": {
		ICAO:      "WS33",
		Name:      "Storytown Airfield",
		City:      "Oregon",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1000,
		Latitude:  42.909198761,
		Longitude: -89.4509963989,
		Timezone:  "America/Chicago",
	},
	"WS36": {
		ICAO:      "WS36",
		Name:      "Swan Field",
		City:      "Almond",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1140,
		Latitude:  44.2714004517,
		Longitude: -89.4371032715,
		Timezone:  "America/Chicago",
	},
	"WS39": {
		ICAO:      "WS39",
		Name:      "Pinewood Air Park",
		City:      "Goodnow",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1560,
		Latitude:  45.6994018555,
		Longitude: -89.662399292,
		Timezone:  "America/Chicago",
	},
	"WS41": {
		ICAO:      "WS41",
		Name:      "Rusmar Farms Airport",
		City:      "Roberts",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1050,
		Latitude:  45.0121994019,
		Longitude: -92.5513000488,
		Timezone:  "America/Chicago",
	},
	"WS42": {
		ICAO:      "WS42",
		Name:      "Owen-Withee Airport",
		City:      "Withee",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1286,
		Latitude:  44.9500007629,
		Longitude: -90.6000976563,
		Timezone:  "America/Chicago",
	},
	"WS43": {
		ICAO:      "WS43",
		Name:      "Birch Creek Airport",
		City:      "De Pere",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 813,
		Latitude:  44.3554992676,
		Longitude: -88.0584030151,
		Timezone:  "America/Chicago",
	},
	"WS44": {
		ICAO:      "WS44",
		Name:      "Diderrich Ranch Airport",
		City:      "Hawkins",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1403,
		Latitude:  45.5365982056,
		Longitude: -90.7049026489,
		Timezone:  "America/Chicago",
	},
	"WS45": {
		ICAO:      "WS45",
		Name:      "Circle K Airport",
		City:      "Irma",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1500,
		Latitude:  45.3415985107,
		Longitude: -89.6745986938,
		Timezone:  "America/Chicago",
	},
	"WS46": {
		ICAO:      "WS46",
		Name:      "J & L Aviation Airport",
		City:      "Ripon",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 975,
		Latitude:  43.8310012817,
		Longitude: -88.8887023926,
		Timezone:  "America/Chicago",
	},
	"WS49": {
		ICAO:      "WS49",
		Name:      "Christie Aerodrome",
		City:      "Fort Atkinson",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 795,
		Latitude:  42.9067001343,
		Longitude: -88.8776016235,
		Timezone:  "America/Chicago",
	},
	"WS51": {
		ICAO:      "WS51",
		Name:      "Leeward Farm Airport",
		City:      "Soldiers Grove",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1055,
		Latitude:  43.3527984619,
		Longitude: -90.6809997559,
		Timezone:  "America/Chicago",
	},
	"WS55": {
		ICAO:      "WS55",
		Name:      "Barten Airport",
		City:      "Sharon",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 977,
		Latitude:  42.5175018311,
		Longitude: -88.7453994751,
		Timezone:  "America/Chicago",
	},
	"WS60": {
		ICAO:      "WS60",
		Name:      "Vern Air Park",
		City:      "Thorp",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1169,
		Latitude:  44.9338989258,
		Longitude: -90.8310012817,
		Timezone:  "America/Chicago",
	},
	"WS62": {
		ICAO:      "WS62",
		Name:      "Sugar Ridge Airport",
		City:      "Verona",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1080,
		Latitude:  42.9636001587,
		Longitude: -89.5810012817,
		Timezone:  "America/Chicago",
	},
	"WS67": {
		ICAO:      "WS67",
		Name:      "Vietmeier Airport",
		City:      "Herbster",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 795,
		Latitude:  46.7999000549,
		Longitude: -91.2085037231,
		Timezone:  "America/Chicago",
	},
	"WS68": {
		ICAO:      "WS68",
		Name:      "Krist Island Airport",
		City:      "Montello",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 770,
		Latitude:  43.7816009521,
		Longitude: -89.3078994751,
		Timezone:  "America/Chicago",
	},
	"WS69": {
		ICAO:      "WS69",
		Name:      "Log Cabin Airport",
		City:      "Eleva",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 850,
		Latitude:  44.5750007629,
		Longitude: -91.5470962524,
		Timezone:  "America/Chicago",
	},
	"WS71": {
		ICAO:      "WS71",
		Name:      "Flying J Airport",
		City:      "St Croix Falls",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1205,
		Latitude:  45.4305000305,
		Longitude: -92.6155014038,
		Timezone:  "America/Chicago",
	},
	"WS72": {
		ICAO:      "WS72",
		Name:      "Cain's Field",
		City:      "Stiles",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 640,
		Latitude:  44.8527984619,
		Longitude: -88.0120010376,
		Timezone:  "America/Chicago",
	},
	"WS74": {
		ICAO:      "WS74",
		Name:      "Al's Airway Airport",
		City:      "Watertown",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 825,
		Latitude:  43.1631011963,
		Longitude: -88.8245010376,
		Timezone:  "America/Chicago",
	},
	"WS76": {
		ICAO:      "WS76",
		Name:      "Black Dog Farm Airport",
		City:      "New London",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 898,
		Latitude:  44.4706993103,
		Longitude: -88.8245010376,
		Timezone:  "America/Chicago",
	},
	"WS77": {
		ICAO:      "WS77",
		Name:      "Circle T Airport",
		City:      "Grantsburg",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 895,
		Latitude:  45.6839981079,
		Longitude: -92.6871032715,
		Timezone:  "America/Chicago",
	},
	"WS78": {
		ICAO:      "WS78",
		Name:      "River Valley Airport",
		City:      "Rhinelander",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1525,
		Latitude:  45.5503005981,
		Longitude: -89.4878005981,
		Timezone:  "America/Chicago",
	},
	"WS81": {
		ICAO:      "WS81",
		Name:      "Alpha Hotel Airport",
		City:      "Webster",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1000,
		Latitude:  45.9480018616,
		Longitude: -92.1084976196,
		Timezone:  "America/Chicago",
	},
	"WS87": {
		ICAO:      "WS87",
		Name:      "Port Field",
		City:      "Twisp",
		State:     "Washington",
		Country:   "US",
		Elevation: 1650,
		Latitude:  48.331869,
		Longitude: -120.058253,
		Timezone:  "America/Los_Angeles",
	},
	"WS91": {
		ICAO:      "WS91",
		Name:      "Sky Hollow Airport",
		City:      "Boscobel",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 707,
		Latitude:  43.2085990906,
		Longitude: -90.6636962891,
		Timezone:  "America/Chicago",
	},
	"WS96": {
		ICAO:      "WS96",
		Name:      "Hughes Airport",
		City:      "Monroe",
		State:     "Wisconsin",
		Country:   "US",
		Elevation: 1010,
		Latitude:  42.6208000183,
		Longitude: -89.7673034668,
		Timezone:  "America/Chicago",
	},
	"WSAC": {
		ICAO:      "WSAC",
		Name:      "Changi Air Base",
		City:      "Singapore",
		Country:   "SG",
		Elevation: 22,
		Latitude:  1.3441400528,
		Longitude: 104.009002686,
		Timezone:  "Asia/Singapore",
	},
	"WSAG": {
		ICAO:      "WSAG",
		Name:      "Sembawang Air Base",
		City:      "Sembawang",
		Country:   "SG",
		Elevation: 86,
		Latitude:  1.4252599478,
		Longitude: 103.81300354,
		Timezone:  "Asia/Singapore",
	},
	"WSAP": {
		ICAO:      "WSAP",
		IATA:      "QPG",
		Name:      "Paya Lebar Air Base",
		Country:   "SG",
		Elevation: 65,
		Latitude:  1.3604199886,
		Longitude: 103.9100036621,
		Timezone:  "Asia/Singapore",
	},
	"WSAT": {
		ICAO:      "WSAT",
		IATA:      "TGA",
		Name:      "Tengah Air Base",
		Country:   "SG",
		Elevation: 50,
		Latitude:  1.3872599602,
		Longitude: 103.7089996338,
		Timezone:  "Asia/Singapore",
	},
	"WSSL": {
		ICAO:      "WSSL",
		IATA:      "XSP",
		Name:      "Seletar Airport",
		City:      "Seletar",
		Country:   "SG",
		Elevation: 36,
		Latitude:  1.4169499874,
		Longitude: 103.8679962158,
		Timezone:  "Asia/Kuala_Lumpur",
	},
	"WSSS": {
		ICAO:      "WSSS",
		IATA:      "SIN",
		Name:      "Singapore Changi International Airport",
		City:      "Singapore",
		State:     "North-East",
		Country:   "SG",
		Elevation: 22,
		Latitude:  1.3501900434,
		Longitude: 103.9940032959,
		Timezone:  "Asia/Singapore",
	},
	"WT01": {
		ICAO:      "WT01",
		Name:      "Hillcrest Farms Airport",
		City:      "Ellensburg",
		State:     "Washington",
		Country:   "US",
		Elevation: 1490,
		Latitude:  46.949167,
		Longitude: -120.481111,
		Timezone:  "America/Los_Angeles",
	},
	"WT03": {
		ICAO:      "WT03",
		Name:      "River View Airpark",
		City:      "Chelan",
		State:     "Washington",
		Country:   "US",
		Elevation: 1210,
		Latitude:  47.900278,
		Longitude: -119.895833,
		Timezone:  "America/Los_Angeles",
	},
	"WT21": {
		ICAO:      "WT21",
		Name:      "Deer Park / Radial Flyer Airport",
		State:     "Washington",
		Country:   "US",
		Elevation: 2060,
		Latitude:  47.894536,
		Longitude: -117.439807,
		Timezone:  "America/Los_Angeles",
	},
	"WT44": {
		ICAO:      "WT44",
		Name:      "Michair Airport",
		City:      "Cathlamet",
		State:     "Washington",
		Country:   "US",
		Elevation: 13,
		Latitude:  46.177333,
		Longitude: -123.385,
		Timezone:  "America/Los_Angeles",
	},
	"WT77": {
		ICAO:      "WT77",
		Name:      "Rocky Bay Airport",
		City:      "Vaughn",
		State:     "Washington",
		Country:   "US",
		Elevation: 80,
		Latitude:  47.355222,
		Longitude: -122.790278,
		Timezone:  "America/Los_Angeles",
	},
	"WT88": {
		ICAO:      "WT88",
		Name:      "Hill Airport",
		City:      "Palouse",
		State:     "Washington",
		Country:   "US",
		Elevation: 2700,
		Latitude:  46.841,
		Longitude: -117.056281,
		Timezone:  "America/Los_Angeles",
	},
	"WV00": {
		ICAO:      "WV00",
		Name:      "Deer Creek Farm Airport",
		City:      "Arbovale",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 2650,
		Latitude:  38.4707984924,
		Longitude: -79.8005981445,
		Timezone:  "America/New_York",
	},
	"WV01": {
		ICAO:      "WV01",
		Name:      "Lee Massey Airport",
		City:      "Ansted",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 1665,
		Latitude:  38.1292991638,
		Longitude: -81.0537033081,
		Timezone:  "America/New_York",
	},
	"WV06": {
		ICAO:      "WV06",
		Name:      "Lost Mountain Airport",
		City:      "Romney",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 1960,
		Latitude:  39.2850990295,
		Longitude: -78.7382965088,
		Timezone:  "America/New_York",
	},
	"WV08": {
		ICAO:      "WV08",
		Name:      "Island Airport",
		City:      "Chesapeake",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 600,
		Latitude:  38.2168006897,
		Longitude: -81.533203125,
		Timezone:  "America/New_York",
	},
	"WV09": {
		ICAO:      "WV09",
		Name:      "Mike Ferrell Field",
		City:      "Corinne",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 2220,
		Latitude:  37.5587005615,
		Longitude: -81.3507003784,
		Timezone:  "America/New_York",
	},
	"WV10": {
		ICAO:      "WV10",
		Name:      "Peterstown Airport",
		City:      "Peterstown",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 1590,
		Latitude:  37.4011993408,
		Longitude: -80.8072967529,
		Timezone:  "America/New_York",
	},
	"WV11": {
		ICAO:      "WV11",
		Name:      "Swope Farm Airport",
		City:      "Fan Rock",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 1802,
		Latitude:  37.5665016174,
		Longitude: -81.6193008423,
		Timezone:  "America/New_York",
	},
	"WV12": {
		ICAO:      "WV12",
		Name:      "Mallory Airport",
		City:      "South Charleston",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 880,
		Latitude:  38.3350982666,
		Longitude: -81.7317962646,
		Timezone:  "America/New_York",
	},
	"WV15": {
		ICAO:      "WV15",
		Name:      "Gerstell Farms Airport",
		City:      "Keyser",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 700,
		Latitude:  39.4833984375,
		Longitude: -78.926399231,
		Timezone:  "America/New_York",
	},
	"WV17": {
		ICAO:      "WV17",
		Name:      "Michaels Farms Airport",
		City:      "Martinsburg",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 510,
		Latitude:  39.3933982849,
		Longitude: -78.1557998657,
		Timezone:  "America/New_York",
	},
	"WV18": {
		ICAO:      "WV18",
		Name:      "Miller Field",
		City:      "Keyser",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 1050,
		Latitude:  39.4084014893,
		Longitude: -79.0136032104,
		Timezone:  "America/New_York",
	},
	"WV19": {
		ICAO:      "WV19",
		Name:      "Moore Field",
		City:      "Kingwood",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 1860,
		Latitude:  39.4448013306,
		Longitude: -79.689201355,
		Timezone:  "America/New_York",
	},
	"WV21": {
		ICAO:      "WV21",
		Name:      "Needwood Farm Airport",
		City:      "Harpers Ferry",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 440,
		Latitude:  39.3306007385,
		Longitude: -77.7947006226,
		Timezone:  "America/New_York",
	},
	"WV22": {
		ICAO:      "WV22",
		Name:      "Green Landings Airport",
		City:      "Hedgesville",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 510,
		Latitude:  39.5746994019,
		Longitude: -77.971496582,
		Timezone:  "America/New_York",
	},
	"WV23": {
		ICAO:      "WV23",
		Name:      "Louis Bennett Field",
		City:      "Weston",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 1014,
		Latitude:  39.0914993286,
		Longitude: -80.4694976807,
		Timezone:  "America/New_York",
	},
	"WV28": {
		ICAO:      "WV28",
		Name:      "Ruth Field",
		City:      "Clarksburg",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 1200,
		Latitude:  39.3245010376,
		Longitude: -80.3967971802,
		Timezone:  "America/New_York",
	},
	"WV29": {
		ICAO:      "WV29",
		Name:      "Valley Point Airport",
		City:      "Valley Point",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 2000,
		Latitude:  39.5797996521,
		Longitude: -79.6498031616,
		Timezone:  "America/New_York",
	},
	"WV30": {
		ICAO:      "WV30",
		Name:      "Rainelle Airport",
		City:      "Rainelle",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 3446,
		Latitude:  37.9486999512,
		Longitude: -80.7164993286,
		Timezone:  "America/New_York",
	},
	"WV32": {
		ICAO:      "WV32",
		Name:      "New River Gorge Airport",
		City:      "Lansing",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 1720,
		Latitude:  38.0890007019,
		Longitude: -81.0651016235,
		Timezone:  "America/New_York",
	},
	"WV47": {
		ICAO:      "WV47",
		Name:      "Rexroad Airport",
		City:      "Clarksburg",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 1350,
		Latitude:  39.1357002258,
		Longitude: -80.2104034424,
		Timezone:  "America/New_York",
	},
	"WV52": {
		ICAO:      "WV52",
		Name:      "Nrao Green Bank Airport",
		City:      "Green Bank",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 2710,
		Latitude:  38.4306983948,
		Longitude: -79.8255996704,
		Timezone:  "America/New_York",
	},
	"WV53": {
		ICAO:      "WV53",
		Name:      "Larew Airport",
		City:      "Newburg",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 1790,
		Latitude:  39.3720016479,
		Longitude: -79.8927993774,
		Timezone:  "America/New_York",
	},
	"WV57": {
		ICAO:      "WV57",
		Name:      "Mckee Sky Ranch Airport",
		City:      "Terra Alta",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 2757,
		Latitude:  39.4631004333,
		Longitude: -79.5425033569,
		Timezone:  "America/New_York",
	},
	"WV59": {
		ICAO:      "WV59",
		Name:      "Fayette Airport",
		City:      "Fayetteville",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 1960,
		Latitude:  38.0267982483,
		Longitude: -81.1197967529,
		Timezone:  "America/New_York",
	},
	"WV61": {
		ICAO:      "WV61",
		Name:      "Buzzards Gap Ultralightport",
		City:      "Berkeley Springs",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 960,
		Latitude:  39.4695014954,
		Longitude: -78.2356033325,
		Timezone:  "America/New_York",
	},
	"WV62": {
		ICAO:      "WV62",
		Name:      "Windwood Fly-In Resort Airport",
		City:      "Davis",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 3210,
		Latitude:  39.0550994873,
		Longitude: -79.4313964844,
		Timezone:  "America/New_York",
	},
	"WV63": {
		ICAO:      "WV63",
		Name:      "Herold Airport",
		City:      "Craigsville",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 2346,
		Latitude:  38.3345985413,
		Longitude: -80.6530990601,
		Timezone:  "America/New_York",
	},
	"WV64": {
		ICAO:      "WV64",
		Name:      "Scott Field",
		City:      "Mineral Wells",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 662,
		Latitude:  39.1781005859,
		Longitude: -81.5236968994,
		Timezone:  "America/New_York",
	},
	"WV65": {
		ICAO:      "WV65",
		Name:      "Carr Airport",
		City:      "Fairmont",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 1670,
		Latitude:  39.44929886,
		Longitude: -80.01139832,
		Timezone:  "America/New_York",
	},
	"WV66": {
		ICAO:      "WV66",
		Name:      "Glendale Fokker Field",
		City:      "Glendale",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 648,
		Latitude:  39.9486999512,
		Longitude: -80.7594985962,
		Timezone:  "America/New_York",
	},
	"WV67": {
		ICAO:      "WV67",
		Name:      "Eastview Airport",
		City:      "Romney",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 1590,
		Latitude:  39.3946990967,
		Longitude: -78.679397583,
		Timezone:  "America/New_York",
	},
	"WV68": {
		ICAO:      "WV68",
		Name:      "Durrett Ranches Airport",
		City:      "Belington",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 2300,
		Latitude:  38.9500007629,
		Longitude: -80.0306015015,
		Timezone:  "America/New_York",
	},
	"WV70": {
		ICAO:      "WV70",
		Name:      "Fairview Airport",
		City:      "Elkins",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 2600,
		Latitude:  38.956199646,
		Longitude: -79.7886962891,
		Timezone:  "America/New_York",
	},
	"WV71": {
		ICAO:      "WV71",
		Name:      "Lieving Airport",
		City:      "Letart",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 635,
		Latitude:  38.8992004395,
		Longitude: -81.9297027588,
		Timezone:  "America/New_York",
	},
	"WV76": {
		ICAO:      "WV76",
		Name:      "Slate Run Airport",
		City:      "Spencer",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 1000,
		Latitude:  38.7775993347,
		Longitude: -81.3104019165,
		Timezone:  "America/New_York",
	},
	"WV77": {
		ICAO:      "WV77",
		Name:      "Hinton-Alderson Airport",
		City:      "Pence Springs",
		State:     "West-Virginia",
		Country:   "US",
		Elevation: 1520,
		Latitude:  37.679599762,
		Longitude: -80.7104034424,
		Timezone:  "America/New_York",
	},
	"WY00": {
		ICAO:      "WY00",
		Name:      "Red Reflet Ranch Airport",
		City:      "Ten Sleep",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4619,
		Latitude:  43.96979904,
		Longitude: -107.3880005,
		Timezone:  "America/Denver",
	},
	"WY01": {
		ICAO:      "WY01",
		Name:      "Dilts Ranch Airport",
		City:      "Bill",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4800,
		Latitude:  43.3958015442,
		Longitude: -105.5550003052,
		Timezone:  "America/Denver",
	},
	"WY02": {
		ICAO:      "WY02",
		Name:      "Iberlin Ranch Nr 3 Airport",
		City:      "Bill",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4400,
		Latitude:  43.1916007996,
		Longitude: -104.93800354,
		Timezone:  "America/Denver",
	},
	"WY05": {
		ICAO:      "WY05",
		Name:      "Skyview Airpark",
		City:      "Cheyenne",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 5838,
		Latitude:  41.2172012329,
		Longitude: -104.5839996338,
		Timezone:  "America/Denver",
	},
	"WY06": {
		ICAO:      "WY06",
		Name:      "Luckinbill Airstrip",
		City:      "Cody",
		State:     "Montana",
		Country:   "US",
		Elevation: 6600,
		Latitude:  44.8382987976,
		Longitude: -109.6220016479,
		Timezone:  "America/Denver",
	},
	"WY07": {
		ICAO:      "WY07",
		Name:      "Hardy Ranch Airport",
		City:      "Douglas",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 5077,
		Latitude:  43.3251991272,
		Longitude: -105.6760025024,
		Timezone:  "America/Denver",
	},
	"WY09": {
		ICAO:      "WY09",
		Name:      "Sherwin Field Nr 1 Airport",
		City:      "Dull Center/Sw Clareton/",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4190,
		Latitude:  43.5060997009,
		Longitude: -104.8779983521,
		Timezone:  "America/Denver",
	},
	"WY11": {
		ICAO:      "WY11",
		Name:      "A A Ranch Airport",
		City:      "Encampment",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 7880,
		Latitude:  41.1582984924,
		Longitude: -106.5579986572,
		Timezone:  "America/Denver",
	},
	"WY12": {
		ICAO:      "WY12",
		Name:      "Ohman Ranch Airport",
		City:      "Gillette",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4905,
		Latitude:  43.926399231,
		Longitude: -105.6529998779,
		Timezone:  "America/Denver",
	},
	"WY13": {
		ICAO:      "WY13",
		Name:      "Little Buffalo Ranch Airport",
		City:      "Gillette",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4919,
		Latitude:  43.7860984802,
		Longitude: -105.6640014648,
		Timezone:  "America/Denver",
	},
	"WY14": {
		ICAO:      "WY14",
		Name:      "Ipy Ranch Airport",
		City:      "Hulett",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 3960,
		Latitude:  44.6375007629,
		Longitude: -104.6890029907,
		Timezone:  "America/Denver",
	},
	"WY15": {
		ICAO:      "WY15",
		Name:      "Bunch Grass Intergalactic Airport",
		City:      "Powell",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4408,
		Latitude:  44.6897010803,
		Longitude: -108.6790008545,
		Timezone:  "America/Denver",
	},
	"WY16": {
		ICAO:      "WY16",
		Name:      "Robbins Airport",
		City:      "Medicine Bow",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 7240,
		Latitude:  42.2599983215,
		Longitude: -105.8590011597,
		Timezone:  "America/Denver",
	},
	"WY17": {
		ICAO:      "WY17",
		Name:      "Ellis Ranch Airport",
		City:      "Medicine Bow",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 6530,
		Latitude:  42.021900177,
		Longitude: -106.4240036011,
		Timezone:  "America/Denver",
	},
	"WY18": {
		ICAO:      "WY18",
		Name:      "Iberlin Ranch Nr 2 Airport",
		City:      "Midwest",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 5020,
		Latitude:  43.5583000183,
		Longitude: -106.0090026855,
		Timezone:  "America/Denver",
	},
	"WY19": {
		ICAO:      "WY19",
		Name:      "Butler Airport",
		City:      "Pine Bluffs",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 5295,
		Latitude:  41.1058006287,
		Longitude: -104.2070007324,
		Timezone:  "America/Denver",
	},
	"WY22": {
		ICAO:      "WY22",
		Name:      "Iberlin Ranch Nr 1 Airport",
		City:      "Savageton",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4500,
		Latitude:  43.9500007629,
		Longitude: -105.9339981079,
		Timezone:  "America/Denver",
	},
	"WY23": {
		ICAO:      "WY23",
		Name:      "Iberlin Strip",
		City:      "Sussex",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4331,
		Latitude:  44.073299408,
		Longitude: -106.2659988403,
		Timezone:  "America/Denver",
	},
	"WY25": {
		ICAO:      "WY25",
		Name:      "Snell - North Laramie River Airport",
		City:      "Wheatland",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4490,
		Latitude:  42.3069000244,
		Longitude: -104.93699646,
		Timezone:  "America/Denver",
	},
	"WY27": {
		ICAO:      "WY27",
		Name:      "Wagonhound Airport",
		City:      "Douglas",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 5470,
		Latitude:  42.5867004395,
		Longitude: -105.5380020142,
		Timezone:  "America/Denver",
	},
	"WY29": {
		ICAO:      "WY29",
		Name:      "Vowers Ranch Airport",
		City:      "Chugwater",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 5550,
		Latitude:  41.6222000122,
		Longitude: -104.7850036621,
		Timezone:  "America/Denver",
	},
	"WY30": {
		ICAO:      "WY30",
		Name:      "Kinky Creek Divide Airport",
		City:      "Dubois",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 8803,
		Latitude:  43.3754997253,
		Longitude: -110.1100006104,
		Timezone:  "America/Denver",
	},
	"WY31": {
		ICAO:      "WY31",
		Name:      "Melody Ranch Airport",
		City:      "Jackson",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 6075,
		Latitude:  43.4095993042,
		Longitude: -110.7730026245,
		Timezone:  "America/Denver",
	},
	"WY33": {
		ICAO:      "WY33",
		Name:      "Antelope Run Ranch Airport",
		City:      "Daniel",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 7470,
		Latitude:  42.8168983459,
		Longitude: -110.2779998779,
		Timezone:  "America/Denver",
	},
	"WY34": {
		ICAO:      "WY34",
		Name:      "Bridger Creek Airport",
		City:      "Lysite",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 5350,
		Latitude:  43.3404998779,
		Longitude: -107.68699646,
		Timezone:  "America/Denver",
	},
	"WY36": {
		ICAO:      "WY36",
		Name:      "Bakers Field",
		City:      "Powell",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4350,
		Latitude:  44.7901992798,
		Longitude: -108.7340011597,
		Timezone:  "America/Denver",
	},
	"WY37": {
		ICAO:      "WY37",
		Name:      "Symons Airport",
		City:      "Sheridan",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 3681,
		Latitude:  44.840801239,
		Longitude: -106.8229980469,
		Timezone:  "America/Denver",
	},
	"WY38": {
		ICAO:      "WY38",
		Name:      "Orchard Ranch Airport",
		City:      "Ten Sleep",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 5277,
		Latitude:  43.6236000061,
		Longitude: -107.4079971313,
		Timezone:  "America/Denver",
	},
	"WY39": {
		ICAO:      "WY39",
		Name:      "Star Valley Ranch Airport",
		City:      "Thayne",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 6210,
		Latitude:  42.9693984985,
		Longitude: -110.966003418,
		Timezone:  "America/Denver",
	},
	"WY41": {
		ICAO:      "WY41",
		Name:      "Red Creek Ranch Airport",
		City:      "Rock Springs",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 7050,
		Latitude:  41.0666999817,
		Longitude: -109.0889968872,
		Timezone:  "America/Denver",
	},
	"WY42": {
		ICAO:      "WY42",
		Name:      "Yu Ranch Airport",
		City:      "Meeteetse",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 5170,
		Latitude:  44.3074989319,
		Longitude: -108.7369995117,
		Timezone:  "America/Denver",
	},
	"WY45": {
		ICAO:      "WY45",
		Name:      "Sloan Airport",
		City:      "Cheyenne",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 6085,
		Latitude:  41.1568984985,
		Longitude: -104.7080001831,
		Timezone:  "America/Denver",
	},
	"WY51": {
		ICAO:      "WY51",
		Name:      "Platt Ranch Airport",
		City:      "Encampment",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 7545,
		Latitude:  41.1650009155,
		Longitude: -106.5950012207,
		Timezone:  "America/Denver",
	},
	"WY55": {
		ICAO:      "WY55",
		Name:      "Taylor Field",
		City:      "Sundance",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4950,
		Latitude:  44.418598175,
		Longitude: -104.59400177,
		Timezone:  "America/Denver",
	},
	"WY59": {
		ICAO:      "WY59",
		Name:      "Two Bar Ranch Airport",
		City:      "Wheatland",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 5110,
		Latitude:  41.9329986572,
		Longitude: -105.0849990845,
		Timezone:  "America/Denver",
	},
	"WY60": {
		ICAO:      "WY60",
		Name:      "Heiner Airport",
		City:      "Afton",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 6440,
		Latitude:  42.861000061,
		Longitude: -110.8980026245,
		Timezone:  "America/Denver",
	},
	"WY61": {
		ICAO:      "WY61",
		Name:      "Gas Hills Airstrip",
		City:      "Waltman",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 6990,
		Latitude:  42.8680992126,
		Longitude: -107.4889984131,
		Timezone:  "America/Denver",
	},
	"WY64": {
		ICAO:      "WY64",
		Name:      "Lone Pine Flying Ranch Airport",
		City:      "Thayne",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 6000,
		Latitude:  42.9574012756,
		Longitude: -110.983001709,
		Timezone:  "America/Denver",
	},
	"WY65": {
		ICAO:      "WY65",
		Name:      "Madsen Airport",
		City:      "Gillette",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 4500,
		Latitude:  44.3488998413,
		Longitude: -105.3369979858,
		Timezone:  "America/Denver",
	},
	"WY66": {
		ICAO:      "WY66",
		Name:      "Chamberlain Brothers Ranch Airport",
		City:      "Douglas",
		State:     "Wyoming",
		Country:   "US",
		Elevation: 5237,
		Latitude:  42.7522010803,
		Longitude: -105.6200027466,
		Timezone:  "America/Denver",
	},
	"XA00": {
		ICAO:      "XA00",
		Name:      "Prose Field",
		City:      "Justin",
		State:     "Texas",
		Country:   "US",
		Elevation: 685,
		Latitude:  33.1472015381,
		Longitude: -97.2802963257,
		Timezone:  "America/Chicago",
	},
	"XA02": {
		ICAO:      "XA02",
		Name:      "Danz Ranch Airport",
		City:      "Johnson City",
		State:     "Texas",
		Country:   "US",
		Elevation: 1230,
		Latitude:  30.2801990509,
		Longitude: -98.426399231,
		Timezone:  "America/Chicago",
	},
	"XA03": {
		ICAO:      "XA03",
		Name:      "Edgington Ranch Airport",
		City:      "Sanger",
		State:     "Texas",
		Country:   "US",
		Elevation: 885,
		Latitude:  33.4283981323,
		Longitude: -97.2720031738,
		Timezone:  "America/Chicago",
	},
	"XA04": {
		ICAO:      "XA04",
		Name:      "Circle Eight Ranch Airport",
		City:      "Glen Rose",
		State:     "Texas",
		Country:   "US",
		Elevation: 800,
		Latitude:  32.2375984192,
		Longitude: -97.899597168,
		Timezone:  "America/Chicago",
	},
	"XA05": {
		ICAO:      "XA05",
		Name:      "Fairview Field",
		City:      "Sudan",
		State:     "Texas",
		Country:   "US",
		Elevation: 3819,
		Latitude:  34.1011009216,
		Longitude: -102.6240005493,
		Timezone:  "America/Chicago",
	},
	"XA07": {
		ICAO:      "XA07",
		Name:      "Spectre Airport",
		City:      "Chico",
		State:     "Texas",
		Country:   "US",
		Elevation: 835,
		Latitude:  33.2750015259,
		Longitude: -97.7166976929,
		Timezone:  "America/Chicago",
	},
	"XA08": {
		ICAO:      "XA08",
		Name:      "Los Cuernos Ranch Airport",
		City:      "Cotulla",
		State:     "Texas",
		Country:   "US",
		Elevation: 340,
		Latitude:  28.1660995483,
		Longitude: -99.0915985107,
		Timezone:  "America/Chicago",
	},
	"XA09": {
		ICAO:      "XA09",
		Name:      "Menard Airport",
		City:      "Bellevue",
		State:     "Texas",
		Country:   "US",
		Elevation: 1055,
		Latitude:  33.5819015503,
		Longitude: -98.0610961914,
		Timezone:  "America/Chicago",
	},
	"XA10": {
		ICAO:      "XA10",
		Name:      "Ponderosa Field",
		City:      "Ponder",
		State:     "Texas",
		Country:   "US",
		Elevation: 840,
		Latitude:  33.2318992615,
		Longitude: -97.3387985229,
		Timezone:  "America/Chicago",
	},
	"XA12": {
		ICAO:      "XA12",
		Name:      "Mcfarlin Ranch Airport",
		City:      "Bertram",
		State:     "Texas",
		Country:   "US",
		Elevation: 1200,
		Latitude:  30.6947002411,
		Longitude: -98.0327987671,
		Timezone:  "America/Chicago",
	},
	"XA15": {
		ICAO:      "XA15",
		Name:      "Holict Private Airport",
		City:      "Riesel",
		State:     "Texas",
		Country:   "US",
		Elevation: 425,
		Latitude:  31.4269008636,
		Longitude: -96.9796981812,
		Timezone:  "America/Chicago",
	},
	"XA16": {
		ICAO:      "XA16",
		Name:      "Tightwaad Air Ranch Airport",
		City:      "Rosston",
		State:     "Texas",
		Country:   "US",
		Elevation: 875,
		Latitude:  33.4942016602,
		Longitude: -97.4655990601,
		Timezone:  "America/Chicago",
	},
	"XA17": {
		ICAO:      "XA17",
		Name:      "Chuckster Airport",
		City:      "Tira",
		State:     "Texas",
		Country:   "US",
		Elevation: 475,
		Latitude:  33.3217010498,
		Longitude: -95.5635986328,
		Timezone:  "America/Chicago",
	},
	"XA23": {
		ICAO:      "XA23",
		Name:      "Richardson Field",
		City:      "Pittsburg",
		State:     "Texas",
		Country:   "US",
		Elevation: 415,
		Latitude:  33.01779938,
		Longitude: -94.97229767,
		Timezone:  "America/Chicago",
	},
	"XA31": {
		ICAO:      "XA31",
		Name:      "Lewis Ranch Airport",
		City:      "Leakey",
		State:     "Texas",
		Country:   "US",
		Elevation: 1730,
		Latitude:  29.8169002533,
		Longitude: -99.7878036499,
		Timezone:  "America/Chicago",
	},
	"XA32": {
		ICAO:      "XA32",
		Name:      "Wolfe Field",
		City:      "Buckholts",
		State:     "Texas",
		Country:   "US",
		Elevation: 500,
		Latitude:  30.9115009308,
		Longitude: -97.1713027954,
		Timezone:  "America/Chicago",
	},
	"XA33": {
		ICAO:      "XA33",
		Name:      "Thorny Woods Airport",
		City:      "Caddo Mills",
		State:     "Texas",
		Country:   "US",
		Elevation: 540,
		Latitude:  32.9586982727,
		Longitude: -96.2643966675,
		Timezone:  "America/Chicago",
	},
	"XA34": {
		ICAO:      "XA34",
		Name:      "Allison Farm Airport",
		City:      "Decatur",
		State:     "Texas",
		Country:   "US",
		Elevation: 850,
		Latitude:  33.2416992188,
		Longitude: -97.4280014038,
		Timezone:  "America/Chicago",
	},
	"XA40": {
		ICAO:      "XA40",
		Name:      "Hound Run Airport",
		City:      "Hallettsville",
		State:     "Texas",
		Country:   "US",
		Elevation: 385,
		Latitude:  29.5214996338,
		Longitude: -96.9368972778,
		Timezone:  "America/Chicago",
	},
	"XA41": {
		ICAO:      "XA41",
		Name:      "Lone Star Flying Service Airport",
		City:      "Harper",
		State:     "Texas",
		Country:   "US",
		Elevation: 2220,
		Latitude:  30.3549995422,
		Longitude: -99.2474975586,
		Timezone:  "America/Chicago",
	},
	"XA42": {
		ICAO:      "XA42",
		Name:      "Connies Aviation Airport",
		City:      "Kaufman",
		State:     "Texas",
		Country:   "US",
		Elevation: 340,
		Latitude:  32.3755989075,
		Longitude: -96.3871002197,
		Timezone:  "America/Chicago",
	},
	"XA43": {
		ICAO:      "XA43",
		Name:      "Fall Creek Ranch Airport",
		City:      "Kerrville",
		State:     "Texas",
		Country:   "US",
		Elevation: 2030,
		Latitude:  29.9081001282,
		Longitude: -99.2069015503,
		Timezone:  "America/Chicago",
	},
	"XA44": {
		ICAO:      "XA44",
		Name:      "Birchfield Ranch Airport",
		City:      "Lometa",
		State:     "Texas",
		Country:   "US",
		Elevation: 1550,
		Latitude:  31.2660999298,
		Longitude: -98.4238967896,
		Timezone:  "America/Chicago",
	},
	"XA45": {
		ICAO:      "XA45",
		Name:      "Weedfalls Airport",
		City:      "Lone Oak",
		State:     "Texas",
		Country:   "US",
		Elevation: 560,
		Latitude:  33.0211982727,
		Longitude: -95.9308013916,
		Timezone:  "America/Chicago",
	},
	"XA46": {
		ICAO:      "XA46",
		Name:      "Creekside Air Park",
		City:      "Mineola",
		State:     "Texas",
		Country:   "US",
		Elevation: 450,
		Latitude:  32.6935997009,
		Longitude: -95.4543991089,
		Timezone:  "America/Chicago",
	},
	"XA47": {
		ICAO:      "XA47",
		Name:      "Tick Hill Airfield",
		City:      "Moody",
		State:     "Texas",
		Country:   "US",
		Elevation: 645,
		Latitude:  31.3097000122,
		Longitude: -97.4848022461,
		Timezone:  "America/Chicago",
	},
	"XA48": {
		ICAO:      "XA48",
		Name:      "Dreamland Airport",
		City:      "Myra",
		State:     "Texas",
		Country:   "US",
		Elevation: 945,
		Latitude:  33.6417007446,
		Longitude: -97.3150024414,
		Timezone:  "America/Chicago",
	},
	"XA49": {
		ICAO:      "XA49",
		Name:      "Oso Canyon Airport",
		City:      "Ozona",
		State:     "Texas",
		Country:   "US",
		Elevation: 1820,
		Latitude:  30.0249996185,
		Longitude: -101.5459976196,
		Timezone:  "America/Chicago",
	},
	"XA51": {
		ICAO:      "XA51",
		Name:      "Smith Airport",
		City:      "Robinson",
		State:     "Texas",
		Country:   "US",
		Elevation: 525,
		Latitude:  31.4183006287,
		Longitude: -97.1258010864,
		Timezone:  "America/Chicago",
	},
	"XA52": {
		ICAO:      "XA52",
		Name:      "Ehni Airport",
		City:      "Trinity",
		State:     "Texas",
		Country:   "US",
		Elevation: 225,
		Latitude:  30.923500061,
		Longitude: -95.2935028076,
		Timezone:  "America/Chicago",
	},
	"XA64": {
		ICAO:      "XA64",
		Name:      "Nash Ranch Airport",
		City:      "Batesville",
		State:     "Texas",
		Country:   "US",
		Elevation: 780,
		Latitude:  28.9617004395,
		Longitude: -99.3852996826,
		Timezone:  "America/Chicago",
	},
	"XA65": {
		ICAO:      "XA65",
		Name:      "Txaerosport Aerodrome",
		City:      "Dorchester",
		State:     "Texas",
		Country:   "US",
		Elevation: 855,
		Latitude:  33.5360984802,
		Longitude: -96.6802978516,
		Timezone:  "America/Chicago",
	},
	"XA66": {
		ICAO:      "XA66",
		Name:      "El Jardin Ranch Airport",
		City:      "Encinal",
		State:     "Texas",
		Country:   "US",
		Elevation: 482,
		Latitude:  28.0750007629,
		Longitude: -99.2863998413,
		Timezone:  "America/Chicago",
	},
	"XA68": {
		ICAO:      "XA68",
		Name:      "Akroville Airport",
		City:      "Slidell",
		State:     "Texas",
		Country:   "US",
		Elevation: 895,
		Latitude:  33.3917999268,
		Longitude: -97.3553009033,
		Timezone:  "America/Chicago",
	},
	"XA71": {
		ICAO:      "XA71",
		Name:      "North Cedar Airport",
		City:      "Apple Springs",
		State:     "Texas",
		Country:   "US",
		Elevation: 250,
		Latitude:  31.277299881,
		Longitude: -94.9739990234,
		Timezone:  "America/Chicago",
	},
	"XA72": {
		ICAO:      "XA72",
		Name:      "Stocker Airport",
		City:      "Krum",
		State:     "Texas",
		Country:   "US",
		Elevation: 770,
		Latitude:  33.2769012451,
		Longitude: -97.2893981934,
		Timezone:  "America/Chicago",
	},
	"XA75": {
		ICAO:      "XA75",
		Name:      "Double A Airport",
		City:      "Grandview",
		State:     "Texas",
		Country:   "US",
		Elevation: 635,
		Latitude:  32.3240013123,
		Longitude: -97.0997009277,
		Timezone:  "America/Chicago",
	},
	"XA77": {
		ICAO:      "XA77",
		Name:      "Benny White Flying Airport",
		City:      "Lamesa",
		State:     "Texas",
		Country:   "US",
		Elevation: 2943,
		Latitude:  32.7486000061,
		Longitude: -101.7789993286,
		Timezone:  "America/Chicago",
	},
	"XA78": {
		ICAO:      "XA78",
		Name:      "Melody Ranch Airport",
		City:      "Mountain Springs",
		State:     "Texas",
		Country:   "US",
		Elevation: 729,
		Latitude:  33.4622993469,
		Longitude: -97.0709991455,
		Timezone:  "America/Chicago",
	},
	"XA84": {
		ICAO:      "XA84",
		Name:      "Stoney Fork Landing Airport",
		City:      "Valley View",
		State:     "Texas",
		Country:   "US",
		Elevation: 680,
		Latitude:  33.4768981934,
		Longitude: -97.1194000244,
		Timezone:  "America/Chicago",
	},
	"XA85": {
		ICAO:      "XA85",
		Name:      "Cougar Landing Airport",
		City:      "Waco",
		State:     "Texas",
		Country:   "US",
		Elevation: 595,
		Latitude:  31.63330078,
		Longitude: -97.3167038,
		Timezone:  "America/Chicago",
	},
	"XA86": {
		ICAO:      "XA86",
		Name:      "Driftwood Ranch Airport",
		City:      "Weatherford",
		State:     "Texas",
		Country:   "US",
		Elevation: 710,
		Latitude:  32.5670013428,
		Longitude: -97.8048019409,
		Timezone:  "America/Chicago",
	},
	"XA88": {
		ICAO:      "XA88",
		Name:      "Dm Ranch Airport",
		City:      "Batesville",
		State:     "Texas",
		Country:   "US",
		Elevation: 750,
		Latitude:  28.9771995544,
		Longitude: -99.5665969849,
		Timezone:  "America/Chicago",
	},
	"XA89": {
		ICAO:      "XA89",
		Name:      "Faith Ranch Airport",
		City:      "Carrizo Springs",
		State:     "Texas",
		Country:   "US",
		Elevation: 773,
		Latitude:  28.20870018,
		Longitude: -100.0189972,
		Timezone:  "America/Chicago",
	},
	"XA90": {
		ICAO:      "XA90",
		Name:      "Fly 1 On Airport",
		City:      "Golden",
		State:     "Texas",
		Country:   "US",
		Elevation: 474,
		Latitude:  32.7571983337,
		Longitude: -95.5278015137,
		Timezone:  "America/Chicago",
	},
	"XA91": {
		ICAO:      "XA91",
		Name:      "Wildwood Airport",
		City:      "Village Mills",
		State:     "Texas",
		Country:   "US",
		Elevation: 118,
		Latitude:  30.533333,
		Longitude: -94.438056,
		Timezone:  "America/Chicago",
	},
	"XA92": {
		ICAO:      "XA92",
		Name:      "Herd Ranch Airport",
		City:      "Menard",
		State:     "Texas",
		Country:   "US",
		Elevation: 2185,
		Latitude:  30.8700008392,
		Longitude: -100.1029968262,
		Timezone:  "America/Chicago",
	},
	"XA93": {
		ICAO:      "XA93",
		Name:      "Tortuga Ranch Airport",
		City:      "Brundage",
		State:     "Texas",
		Country:   "US",
		Elevation: 550,
		Latitude:  28.6193008423,
		Longitude: -99.6544036865,
		Timezone:  "America/Chicago",
	},
	"XA95": {
		ICAO:      "XA95",
		Name:      "Candelaria Airport",
		City:      "Candelaria",
		State:     "Texas",
		Country:   "US",
		Elevation: 2913,
		Latitude:  30.1494007111,
		Longitude: -104.6829986572,
		Timezone:  "America/Chicago",
	},
	"XA96": {
		ICAO:      "XA96",
		Name:      "En Gedi Ranch Airport",
		City:      "Canadian",
		State:     "Texas",
		Country:   "US",
		Elevation: 2800,
		Latitude:  35.6624984741,
		Longitude: -100.3239974976,
		Timezone:  "America/Chicago",
	},
	"XA98": {
		ICAO:      "XA98",
		Name:      "Jbj Ranch Airport",
		City:      "Decatur",
		State:     "Texas",
		Country:   "US",
		Elevation: 890,
		Latitude:  33.3027000427,
		Longitude: -97.4580993652,
		Timezone:  "America/Chicago",
	},
	"XA99": {
		ICAO:      "XA99",
		Name:      "Flat Bush Airport",
		City:      "Decatur",
		State:     "Texas",
		Country:   "US",
		Elevation: 862,
		Latitude:  33.1343994141,
		Longitude: -97.6070022583,
		Timezone:  "America/Chicago",
	},
	"XBRO": {
		ICAO:      "XBRO",
		IATA:      "SKL",
		Name:      "Skye Bridge Ashaig Airport",
		City:      "Broadford",
		State:     "Scotland",
		Country:   "GB",
		Latitude:  57.2533988953,
		Longitude: -5.8278999329,
		Timezone:  "Europe/London",
	},
	"XIUW": {
		ICAO:      "XIUW",
		Name:      "Ulan-Ude East Airport",
		City:      "Ulan Ude",
		State:     "Respublika-Buryatiya",
		Country:   "RU",
		Elevation: 1670,
		Latitude:  51.8499984741,
		Longitude: 107.7379989624,
		Timezone:  "Asia/Irkutsk",
	},
	"XLAP": {
		ICAO:      "XLAP",
		Name:      "Plestsy Airport",
		City:      "Plesetsk",
		State:     "Arkhangelskaya",
		Country:   "RU",
		Elevation: 427,
		Latitude:  62.7167015076,
		Longitude: 40.4883003235,
		Timezone:  "Europe/Moscow",
	},
	"XLLL": {
		ICAO:      "XLLL",
		Name:      "Soltsy-2 Air Base",
		City:      "Soltsy",
		State:     "Novgorod",
		Country:   "RU",
		Elevation: 266,
		Latitude:  58.1395454407,
		Longitude: 30.3304195404,
		Timezone:  "Europe/Moscow",
	},
	"XLLN": {
		ICAO:      "XLLN",
		Name:      "Kasimovo Airfield",
		City:      "St. Petersburg",
		State:     "Leningradskaya-Oblast'",
		Country:   "RU",
		Elevation: 230,
		Latitude:  60.198299408,
		Longitude: 30.3349990845,
		Timezone:  "Europe/Moscow",
	},
	"XLML": {
		ICAO:      "XLML",
		Name:      "Luostari Air Base",
		City:      "Zapolyarnyj",
		State:     "Murmansk",
		Country:   "RU",
		Elevation: 249,
		Latitude:  69.4000015259,
		Longitude: 30.9883003235,
		Timezone:  "Europe/Moscow",
	},
	"XLMV": {
		ICAO:      "XLMV",
		Name:      "Severomorsk-3 Naval Air Base",
		City:      "Severomorsk",
		State:     "Murmansk",
		Country:   "RU",
		Elevation: 564,
		Latitude:  68.8666687012,
		Longitude: 33.7166671753,
		Timezone:  "Europe/Moscow",
	},
	"XLOS": {
		ICAO:      "XLOS",
		Name:      "Ostrov Naval Air Base",
		City:      "Ostrov",
		State:     "Pskov",
		Country:   "RU",
		Elevation: 220,
		Latitude:  57.2999992371,
		Longitude: 28.4333000183,
		Timezone:  "Europe/Moscow",
	},
	"XLPD": {
		ICAO:      "XLPD",
		Name:      "Derevyannoe Airport",
		City:      "Derevyannoe",
		State:     "Karelia",
		Country:   "RU",
		Elevation: 131,
		Latitude:  61.6069984436,
		Longitude: 34.6209983826,
		Timezone:  "Europe/Moscow",
	},
	"XLPO": {
		ICAO:      "XLPO",
		Name:      "Lodeynoye Pole Air Base",
		City:      "Lodeynoye Pole",
		State:     "Leningradskaya-Oblast'",
		Country:   "RU",
		Elevation: 56,
		Latitude:  60.7099990845,
		Longitude: 33.5699996948,
		Timezone:  "Europe/Moscow",
	},
	"XLWF": {
		ICAO:      "XLWF",
		Name:      "Fedotovo Naval Air Base",
		City:      "Kipelovo",
		State:     "Vologda",
		Country:   "RU",
		Elevation: 574,
		Latitude:  59.1883010864,
		Longitude: 39.123298645,
		Timezone:  "Europe/Moscow",
	},
	"XLWT": {
		ICAO:      "XLWT",
		Name:      "Trufanovo Airfield",
		City:      "Vologda",
		State:     "Vologda",
		Country:   "RU",
		Elevation: 417,
		Latitude:  59.3089981079,
		Longitude: 39.8499984741,
		Timezone:  "Europe/Moscow",
	},
	"XRMB": {
		ICAO:      "XRMB",
		Name:      "Budennovsk",
		City:      "Budennovsk",
		State:     "Stavropol'skiy",
		Country:   "RU",
		Elevation: 361,
		Latitude:  44.833333,
		Longitude: 44.016667,
		Timezone:  "Europe/Moscow",
	},
	"XRWL": {
		ICAO:      "XRWL",
		Name:      "Lebyazhye Air Base",
		City:      "Kamyshin",
		State:     "Volgograd",
		Country:   "RU",
		Elevation: 381,
		Latitude:  50.2016983032,
		Longitude: 45.2083015442,
		Timezone:  "Europe/Volgograd",
	},
	"XS00": {
		ICAO:      "XS00",
		Name:      "Flying D Airport",
		City:      "Llano",
		State:     "Texas",
		Country:   "US",
		Elevation: 1460,
		Latitude:  30.6371994019,
		Longitude: -98.748298645,
		Timezone:  "America/Chicago",
	},
	"XS01": {
		ICAO:      "XS01",
		Name:      "Tschirhart Ranch Airport",
		City:      "Fredericksburg",
		State:     "Texas",
		Country:   "US",
		Elevation: 2060,
		Latitude:  30.4501991272,
		Longitude: -98.8861999512,
		Timezone:  "America/Chicago",
	},
	"XS03": {
		ICAO:      "XS03",
		Name:      "Herbert Ranch Airport",
		City:      "Marble Falls",
		State:     "Texas",
		Country:   "US",
		Elevation: 1140,
		Latitude:  30.481300354,
		Longitude: -98.2810974121,
		Timezone:  "America/Chicago",
	},
	"XS05": {
		ICAO:      "XS05",
		Name:      "H M Ranch Airport",
		City:      "Marion",
		State:     "Texas",
		Country:   "US",
		Elevation: 751,
		Latitude:  29.5988006592,
		Longitude: -98.1710968018,
		Timezone:  "America/Chicago",
	},
	"XS06": {
		ICAO:      "XS06",
		Name:      "Flying B Ranch Airport",
		City:      "Grandview",
		State:     "Texas",
		Country:   "US",
		Elevation: 665,
		Latitude:  32.298500061,
		Longitude: -97.1494979858,
		Timezone:  "America/Chicago",
	},
	"XS07": {
		ICAO:      "XS07",
		Name:      "W D Cornilius Ranch Airport",
		City:      "Markham",
		State:     "Texas",
		Country:   "US",
		Elevation: 47,
		Latitude:  28.9503002167,
		Longitude: -96.0599975586,
		Timezone:  "America/Chicago",
	},
	"XS08": {
		ICAO:      "XS08",
		Name:      "Polk Ranch Airport",
		City:      "Mason",
		State:     "Texas",
		Country:   "US",
		Elevation: 1840,
		Latitude:  30.8768005371,
		Longitude: -99.1995010376,
		Timezone:  "America/Chicago",
	},
	"XS09": {
		ICAO:      "XS09",
		Name:      "Estates Airpark",
		City:      "New Waverly",
		State:     "Texas",
		Country:   "US",
		Elevation: 325,
		Latitude:  30.5375003815,
		Longitude: -95.4364013672,
		Timezone:  "America/Chicago",
	},
	"XS10": {
		ICAO:      "XS10",
		Name:      "Aransas National Wildlife Refuge Airport",
		City:      "Matagorda Island",
		State:     "Texas",
		Country:   "US",
		Elevation: 10,
		Latitude:  28.1170005798,
		Longitude: -96.8003005981,
		Timezone:  "America/Chicago",
	},
	"XS11": {
		ICAO:      "XS11",
		Name:      "Idlewild Airport",
		City:      "Medina",
		State:     "Texas",
		Country:   "US",
		Elevation: 1660,
		Latitude:  29.776599884,
		Longitude: -99.3591995239,
		Timezone:  "America/Chicago",
	},
	"XS12": {
		ICAO:      "XS12",
		Name:      "C F C Aviation Ranch Airport",
		City:      "Liberty Hill",
		State:     "Texas",
		Country:   "US",
		Elevation: 1161,
		Latitude:  30.6760005951,
		Longitude: -98.0175018311,
		Timezone:  "America/Chicago",
	},
	"XS13": {
		ICAO:      "XS13",
		Name:      "T-4 Ranch Airport",
		City:      "Medina",
		State:     "Texas",
		Country:   "US",
		Elevation: 1595,
		Latitude:  29.8166007996,
		Longitude: -99.1967010498,
		Timezone:  "America/Chicago",
	},
	"XS14": {
		ICAO:      "XS14",
		Name:      "Weese International Airport",
		City:      "Greenville",
		State:     "Texas",
		Country:   "US",
		Elevation: 487,
		Latitude:  32.9693984985,
		Longitude: -96.070602417,
		Timezone:  "America/Chicago",
	},
	"XS15": {
		ICAO:      "XS15",
		Name:      "Womack Ranch Airport",
		City:      "Menard",
		State:     "Texas",
		Country:   "US",
		Elevation: 2200,
		Latitude:  30.7709999084,
		Longitude: -99.9003982544,
		Timezone:  "America/Chicago",
	},
	"XS17": {
		ICAO:      "XS17",
		Name:      "Hensley Ranch Airport",
		City:      "Milano",
		State:     "Texas",
		Country:   "US",
		Elevation: 460,
		Latitude:  30.7091007233,
		Longitude: -96.9019012451,
		Timezone:  "America/Chicago",
	},
	"XS18": {
		ICAO:      "XS18",
		Name:      "Tom J Moore Farm Airport",
		City:      "Millican",
		State:     "Texas",
		Country:   "US",
		Elevation: 198,
		Latitude:  30.3847007751,
		Longitude: -96.2247009277,
		Timezone:  "America/Chicago",
	},
	"XS20": {
		ICAO:      "XS20",
		Name:      "Dos Arroyos Ranch Airport",
		City:      "Mountain Home",
		State:     "Texas",
		Country:   "US",
		Elevation: 2058,
		Latitude:  30.1541004181,
		Longitude: -99.2977981567,
		Timezone:  "America/Chicago",
	},
	"XS21": {
		ICAO:      "XS21",
		Name:      "H & S Airfield",
		City:      "Damon",
		State:     "Texas",
		Country:   "US",
		Elevation: 100,
		Latitude:  29.3311004639,
		Longitude: -95.7046966553,
		Timezone:  "America/Chicago",
	},
	"XS22": {
		ICAO:      "XS22",
		Name:      "Y O Ranch Airport",
		City:      "Mountain Home",
		State:     "Texas",
		Country:   "US",
		Elevation: 2136,
		Latitude:  30.2040996552,
		Longitude: -99.6736984253,
		Timezone:  "America/Chicago",
	},
	"XS23": {
		ICAO:      "XS23",
		Name:      "Priour Ranch Airport",
		City:      "Mountain Home",
		State:     "Texas",
		Country:   "US",
		Elevation: 2200,
		Latitude:  30.1266002655,
		Longitude: -99.4711990356,
		Timezone:  "America/Chicago",
	},
	"XS24": {
		ICAO:      "XS24",
		Name:      "Cherry Spraying Service Airport",
		City:      "Muldoon",
		State:     "Texas",
		Country:   "US",
		Elevation: 370,
		Latitude:  29.82970047,
		Longitude: -97.0991973877,
		Timezone:  "America/Chicago",
	},
	"XS25": {
		ICAO:      "XS25",
		Name:      "Flying C Ranch Airport",
		City:      "Needville",
		State:     "Texas",
		Country:   "US",
		Elevation: 70,
		Latitude:  29.3847007751,
		Longitude: -95.6829986572,
		Timezone:  "America/Chicago",
	},
	"XS27": {
		ICAO:      "XS27",
		Name:      "Farm Air Service Airport",
		City:      "Nome",
		State:     "Texas",
		Country:   "US",
		Elevation: 45,
		Latitude:  30.0377006531,
		Longitude: -94.3960037231,
		Timezone:  "America/Chicago",
	},
	"XS28": {
		ICAO:      "XS28",
		Name:      "North Willis Airport",
		City:      "Nome",
		State:     "Texas",
		Country:   "US",
		Elevation: 42,
		Latitude:  30.026599884,
		Longitude: -94.4626998901,
		Timezone:  "America/Chicago",
	},
	"XS29": {
		ICAO:      "XS29",
		Name:      "Sonny Broussard Landing Strip",
		City:      "Nome",
		State:     "Texas",
		Country:   "US",
		Elevation: 35,
		Latitude:  29.9908008575,
		Longitude: -94.4224014282,
		Timezone:  "America/Chicago",
	},
	"XS30": {
		ICAO:      "XS30",
		Name:      "Burress Airport",
		City:      "Paris",
		State:     "Texas",
		Country:   "US",
		Elevation: 484,
		Latitude:  33.6050987244,
		Longitude: -95.4819030762,
		Timezone:  "America/Chicago",
	},
	"XS31": {
		ICAO:      "XS31",
		Name:      "Graham Field",
		City:      "Sadler",
		State:     "Texas",
		Country:   "US",
		Elevation: 695,
		Latitude:  33.7018013,
		Longitude: -96.8319015503,
		Timezone:  "America/Chicago",
	},
	"XS33": {
		ICAO:      "XS33",
		Name:      "Chesson Airport",
		City:      "Orangefield",
		State:     "Texas",
		Country:   "US",
		Elevation: 10,
		Latitude:  30.0624008179,
		Longitude: -93.8646011353,
		Timezone:  "America/Chicago",
	},
	"XS34": {
		ICAO:      "XS34",
		Name:      "Skylark Airport",
		City:      "Ennis",
		State:     "Texas",
		Country:   "US",
		Elevation: 430,
		Latitude:  32.3125,
		Longitude: -96.488889,
		Timezone:  "America/Chicago",
	},
	"XS35": {
		ICAO:      "XS35",
		Name:      "Trull Airport",
		City:      "Palacios",
		State:     "Texas",
		Country:   "US",
		Elevation: 18,
		Latitude:  28.739200592,
		Longitude: -96.2191009521,
		Timezone:  "America/Chicago",
	},
	"XS36": {
		ICAO:      "XS36",
		Name:      "Tres Ninos Ranch Airport",
		City:      "Pandale",
		State:     "Texas",
		Country:   "US",
		Elevation: 1690,
		Latitude:  30.253528,
		Longitude: -101.691631,
		Timezone:  "America/Chicago",
	},
	"XS39": {
		ICAO:      "XS39",
		Name:      "A&A Flying Service Airport",
		City:      "Pearland",
		State:     "Texas",
		Country:   "US",
		Elevation: 49,
		Latitude:  29.5363006592,
		Longitude: -95.2640991211,
		Timezone:  "America/Chicago",
	},
	"XS40": {
		ICAO:      "XS40",
		Name:      "Jay Kay Ranch Airport",
		City:      "Pearsall",
		State:     "Texas",
		Country:   "US",
		Elevation: 666,
		Latitude:  28.8003005981,
		Longitude: -99.0002975464,
		Timezone:  "America/Chicago",
	},
	"XS42": {
		ICAO:      "XS42",
		Name:      "Agricultural Supplies Airport",
		City:      "Pearsall",
		State:     "Texas",
		Country:   "US",
		Elevation: 651,
		Latitude:  28.9293994904,
		Longitude: -99.0802993774,
		Timezone:  "America/Chicago",
	},
	"XS43": {
		ICAO:      "XS43",
		Name:      "Medina River Ranch Airport",
		City:      "Pipe Creek",
		State:     "Texas",
		Country:   "US",
		Elevation: 1216,
		Latitude:  29.6616001129,
		Longitude: -98.9563980103,
		Timezone:  "America/Chicago",
	},
	"XS44": {
		ICAO:      "XS44",
		Name:      "Rancho Del Cielo Airport",
		City:      "Kent",
		State:     "Texas",
		Country:   "US",
		Elevation: 4408,
		Latitude:  31.0153007507,
		Longitude: -104.2119979858,
		Timezone:  "America/Chicago",
	},
	"XS46": {
		ICAO:      "XS46",
		Name:      "Port O'Connor Private Airport",
		City:      "Port O'Connor",
		State:     "Texas",
		Country:   "US",
		Elevation: 8,
		Latitude:  28.4297008514,
		Longitude: -96.4443969727,
		Timezone:  "America/Chicago",
	},
	"XS47": {
		ICAO:      "XS47",
		Name:      "Maurice Dauwe Farm Airport",
		City:      "Poteet",
		State:     "Texas",
		Country:   "US",
		Elevation: 447,
		Latitude:  29.0002002716,
		Longitude: -98.5710983276,
		Timezone:  "America/Chicago",
	},
	"XS48": {
		ICAO:      "XS48",
		Name:      "Marlin'S Meadow Airport",
		City:      "Yantis",
		State:     "Texas",
		Country:   "US",
		Elevation: 490,
		Latitude:  32.953889,
		Longitude: -95.5775,
		Timezone:  "America/Chicago",
	},
	"XS49": {
		ICAO:      "XS49",
		Name:      "Dean Ranch Airport",
		City:      "Shiner",
		State:     "Texas",
		Country:   "US",
		Elevation: 400,
		Latitude:  29.4902000427,
		Longitude: -97.1219024658,
		Timezone:  "America/Chicago",
	},
	"XS50": {
		ICAO:      "XS50",
		Name:      "4-G Ranch Airport",
		City:      "Premont",
		State:     "Texas",
		Country:   "US",
		Elevation: 272,
		Latitude:  27.383600235,
		Longitude: -98.3099975586,
		Timezone:  "America/Chicago",
	},
	"XS51": {
		ICAO:      "XS51",
		Name:      "Seeligson Ranch Airport",
		City:      "Premont",
		State:     "Texas",
		Country:   "US",
		Elevation: 172,
		Latitude:  27.404499054,
		Longitude: -98.1446990967,
		Timezone:  "America/Chicago",
	},
	"XS53": {
		ICAO:      "XS53",
		Name:      "Price Ranch Airport",
		City:      "Big Wells",
		State:     "Texas",
		Country:   "US",
		Elevation: 532,
		Latitude:  28.5960998535,
		Longitude: -99.5714035034,
		Timezone:  "America/Chicago",
	},
	"XS55": {
		ICAO:      "XS55",
		Name:      "Gizmo Field",
		City:      "Malone",
		State:     "Texas",
		Country:   "US",
		Elevation: 551,
		Latitude:  31.9293003082,
		Longitude: -96.9389038086,
		Timezone:  "America/Chicago",
	},
	"XS56": {
		ICAO:      "XS56",
		Name:      "Bell Airfield",
		City:      "Raymondville",
		State:     "Texas",
		Country:   "US",
		Elevation: 30,
		Latitude:  26.4486999512,
		Longitude: -97.7919006348,
		Timezone:  "America/Chicago",
	},
	"XS57": {
		ICAO:      "XS57",
		Name:      "Havelka Haven Airport",
		City:      "Mc Gregor",
		State:     "Texas",
		Country:   "US",
		Elevation: 711,
		Latitude:  31.4654006958,
		Longitude: -97.404800415,
		Timezone:  "America/Chicago",
	},
	"XS58": {
		ICAO:      "XS58",
		Name:      "Tri-County Air Service Airport",
		City:      "Raywood",
		State:     "Texas",
		Country:   "US",
		Elevation: 70,
		Latitude:  30.0440998077,
		Longitude: -94.7058029175,
		Timezone:  "America/Chicago",
	},
	"XS59": {
		ICAO:      "XS59",
		Name:      "Mellon Ranch Airport",
		City:      "Refugio",
		State:     "Texas",
		Country:   "US",
		Elevation: 38,
		Latitude:  28.2807998657,
		Longitude: -97.2114028931,
		Timezone:  "America/Chicago",
	},
	"XS60": {
		ICAO:      "XS60",
		Name:      "Mustang Community Airfield",
		City:      "Aubrey",
		State:     "Texas",
		Country:   "US",
		Elevation: 605,
		Latitude:  33.3184013367,
		Longitude: -96.90599823,
		Timezone:  "America/Chicago",
	},
	"XS61": {
		ICAO:      "XS61",
		Name:      "Deep Creek Ranch Airport",
		City:      "Richland Springs",
		State:     "Texas",
		Country:   "US",
		Elevation: 1388,
		Latitude:  31.114900589,
		Longitude: -99.00340271,
		Timezone:  "America/Chicago",
	},
	"XS62": {
		ICAO:      "XS62",
		Name:      "Casey Three Ranch Airport",
		City:      "Rio Frio",
		State:     "Texas",
		Country:   "US",
		Elevation: 1586,
		Latitude:  29.6333007813,
		Longitude: -99.7286987305,
		Timezone:  "America/Chicago",
	},
	"XS63": {
		ICAO:      "XS63",
		Name:      "Texas Air Museum Airport",
		City:      "Rio Hondo",
		State:     "Texas",
		Country:   "US",
		Elevation: 30,
		Latitude:  26.2287006378,
		Longitude: -97.560798645,
		Timezone:  "America/Chicago",
	},
	"XS64": {
		ICAO:      "XS64",
		Name:      "Farm Services Inc Airport",
		City:      "Rio Hondo",
		State:     "Texas",
		Country:   "US",
		Elevation: 25,
		Latitude:  26.2490005493,
		Longitude: -97.5650024414,
		Timezone:  "America/Chicago",
	},
	"XS66": {
		ICAO:      "XS66",
		Name:      "Rabb Dusting Inc Airport",
		City:      "Robstown",
		State:     "Texas",
		Country:   "US",
		Elevation: 83,
		Latitude:  27.803899765,
		Longitude: -97.743598938,
		Timezone:  "America/Chicago",
	},
	"XS67": {
		ICAO:      "XS67",
		Name:      "San Jose Island Airport",
		City:      "Rockport",
		State:     "Texas",
		Country:   "US",
		Elevation: 10,
		Latitude:  27.9444999695,
		Longitude: -96.9850006104,
		Timezone:  "America/Chicago",
	},
	"XS68": {
		ICAO:      "XS68",
		Name:      "Sky Lane Ranch Airport",
		City:      "Brenham",
		State:     "Texas",
		Country:   "US",
		Elevation: 290,
		Latitude:  30.2569007874,
		Longitude: -96.3285980225,
		Timezone:  "America/Chicago",
	},
	"XS69": {
		ICAO:      "XS69",
		Name:      "Hackberry Ranch Airport",
		City:      "Rocksprings",
		State:     "Texas",
		Country:   "US",
		Elevation: 1829,
		Latitude:  29.9113006592,
		Longitude: -100,
		Timezone:  "America/Chicago",
	},
	"XS70": {
		ICAO:      "XS70",
		Name:      "Glover Airport",
		City:      "Mount Pleasant",
		State:     "Texas",
		Country:   "US",
		Elevation: 400,
		Latitude:  33.134601593,
		Longitude: -95.0179977417,
		Timezone:  "America/Chicago",
	},
	"XS71": {
		ICAO:      "XS71",
		Name:      "San Christoval Ranch Airport",
		City:      "Pawnee",
		State:     "Texas",
		Country:   "US",
		Elevation: 385,
		Latitude:  28.7481002808,
		Longitude: -98.0449981689,
		Timezone:  "America/Chicago",
	},
	"XS72": {
		ICAO:      "XS72",
		Name:      "F R Duke Farm Airport",
		City:      "Romayor",
		State:     "Texas",
		Country:   "US",
		Elevation: 85,
		Latitude:  30.483499527,
		Longitude: -94.8335037231,
		Timezone:  "America/Chicago",
	},
	"XS73": {
		ICAO:      "XS73",
		Name:      "Double D Ranch Airport",
		City:      "Rosanky",
		State:     "Texas",
		Country:   "US",
		Elevation: 520,
		Latitude:  29.8736000061,
		Longitude: -97.2882995605,
		Timezone:  "America/Chicago",
	},
	"XS74": {
		ICAO:      "XS74",
		Name:      "Diamondaire Airport",
		City:      "Waco",
		State:     "Texas",
		Country:   "US",
		Elevation: 424,
		Latitude:  31.6082000732,
		Longitude: -97.1628036499,
		Timezone:  "America/Chicago",
	},
	"XS75": {
		ICAO:      "XS75",
		Name:      "West Ranch Airport",
		City:      "Round Mountain",
		State:     "Texas",
		Country:   "US",
		Elevation: 1470,
		Latitude:  30.4543991089,
		Longitude: -98.4891967773,
		Timezone:  "America/Chicago",
	},
	"XS76": {
		ICAO:      "XS76",
		Name:      "Texas Menhaden Strip",
		City:      "Sabine Pass",
		State:     "Texas",
		Country:   "US",
		Elevation: 6,
		Latitude:  29.7241001129,
		Longitude: -93.8703994751,
		Timezone:  "America/Chicago",
	},
	"XS77": {
		ICAO:      "XS77",
		Name:      "Seafood Warehouse Park Airport",
		City:      "Crystal Beach",
		State:     "Texas",
		Country:   "US",
		Elevation: 9,
		Latitude:  29.4654998779,
		Longitude: -94.6274032593,
		Timezone:  "America/Chicago",
	},
	"XS78": {
		ICAO:      "XS78",
		Name:      "Las Culebras Airport",
		City:      "Brady",
		State:     "Texas",
		Country:   "US",
		Elevation: 1387,
		Latitude:  31.385417,
		Longitude: -99.099833,
		Timezone:  "America/Chicago",
	},
	"XS79": {
		ICAO:      "XS79",
		Name:      "A W Ranch Airport",
		City:      "San Antonio",
		State:     "Texas",
		Country:   "US",
		Elevation: 540,
		Latitude:  29.258600235,
		Longitude: -98.4583969116,
		Timezone:  "America/Chicago",
	},
	"XS80": {
		ICAO:      "XS80",
		Name:      "Santiago Cattle Co. Airport",
		City:      "Comfort",
		State:     "Texas",
		Country:   "US",
		Elevation: 1497,
		Latitude:  29.972556,
		Longitude: -98.934639,
		Timezone:  "America/Chicago",
	},
	"XS86": {
		ICAO:      "XS86",
		Name:      "T-Ranch Airport",
		City:      "Castroville",
		State:     "Texas",
		Country:   "US",
		Elevation: 880,
		Latitude:  29.2668991089,
		Longitude: -98.9253005981,
		Timezone:  "America/Chicago",
	},
	"XS88": {
		ICAO:      "XS88",
		Name:      "Parson Field",
		City:      "Paris",
		State:     "Texas",
		Country:   "US",
		Elevation: 475,
		Latitude:  33.8573989868,
		Longitude: -95.7048034668,
		Timezone:  "America/Chicago",
	},
	"XS89": {
		ICAO:      "XS89",
		Name:      "Yates Airport",
		City:      "San Antonio",
		State:     "Texas",
		Country:   "US",
		Elevation: 971,
		Latitude:  29.621099472,
		Longitude: -98.4558029175,
		Timezone:  "America/Chicago",
	},
	"XS90": {
		ICAO:      "XS90",
		Name:      "Fentress Airpark",
		City:      "Fentress",
		State:     "Texas",
		Country:   "US",
		Elevation: 460,
		Latitude:  29.7682991028,
		Longitude: -97.7752990723,
		Timezone:  "America/Chicago",
	},
	"XS91": {
		ICAO:      "XS91",
		Name:      "Pickle Plantation Airport",
		City:      "Poynor",
		State:     "Texas",
		Country:   "US",
		Elevation: 580,
		Latitude:  32.0864982605,
		Longitude: -95.5992965698,
		Timezone:  "America/Chicago",
	},
	"XS92": {
		ICAO:      "XS92",
		Name:      "Jackson /Bill/ Airport",
		City:      "Paradise",
		State:     "Texas",
		Country:   "US",
		Elevation: 800,
		Latitude:  33.1459999084,
		Longitude: -97.7125015259,
		Timezone:  "America/Chicago",
	},
	"XS93": {
		ICAO:      "XS93",
		Name:      "Diamond O Ranch Airport",
		City:      "Santa Elena",
		State:     "Texas",
		Country:   "US",
		Elevation: 420,
		Latitude:  26.716999054,
		Longitude: -98.5586013794,
		Timezone:  "America/Chicago",
	},
	"XS94": {
		ICAO:      "XS94",
		Name:      "Corralitos Airport",
		City:      "San Ygnacio",
		State:     "Texas",
		Country:   "US",
		Elevation: 360,
		Latitude:  27.1114006042,
		Longitude: -99.4225006104,
		Timezone:  "America/Chicago",
	},
	"XS95": {
		ICAO:      "XS95",
		Name:      "Marty Ranch Airport",
		City:      "Schulenburg",
		State:     "Texas",
		Country:   "US",
		Elevation: 365,
		Latitude:  29.6769008636,
		Longitude: -96.9704971313,
		Timezone:  "America/Chicago",
	},
	"XS99": {
		ICAO:      "XS99",
		Name:      "Lake Water Wheel Airport",
		City:      "Shepherd",
		State:     "Texas",
		Country:   "US",
		Elevation: 80,
		Latitude:  30.4710006714,
		Longitude: -94.9126968384,
		Timezone:  "America/Chicago",
	},
	"XUBS": {
		ICAO:      "XUBS",
		Name:      "Smolensk North Airport",
		City:      "Smolensk",
		State:     "Smolensk",
		Country:   "RU",
		Elevation: 820,
		Latitude:  54.824,
		Longitude: 32.025,
		Timezone:  "Europe/Moscow",
	},
	"XUMR": {
		ICAO:      "XUMR",
		Name:      "Vyazma Airport",
		City:      "Vyazma",
		State:     "Smolensk",
		Country:   "RU",
		Elevation: 794,
		Latitude:  55.1483001709,
		Longitude: 34.3833007812,
		Timezone:  "Europe/Moscow",
	},
	"XWPD": {
		ICAO:      "XWPD",
		Name:      "Sosnovka Airfield",
		City:      "Penza",
		State:     "Penza",
		Country:   "RU",
		Latitude:  53.2610015869,
		Longitude: 45.2770004272,
		Timezone:  "Europe/Moscow",
	},
	"XWSG": {
		ICAO:      "XWSG",
		Name:      "Engels Airbase",
		City:      "Engels",
		State:     "Saratov",
		Country:   "RU",
		Elevation: 128,
		Latitude:  51.466667,
		Longitude: 46.2,
		Timezone:  "Europe/Saratov",
	},
	"YAAL": {
		ICAO:      "YAAL",
		Name:      "Yarralin Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -16.4447002411,
		Longitude: 130.8809967041,
		Timezone:  "Australia/Darwin",
	},
	"YABA": {
		ICAO:      "YABA",
		IATA:      "ALH",
		Name:      "Albany Airport",
		City:      "Albany",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 233,
		Latitude:  -34.9432983398,
		Longitude: 117.8089981079,
		Timezone:  "Australia/Perth",
	},
	"YABI": {
		ICAO:      "YABI",
		IATA:      "ABG",
		Name:      "Abingdon Downs Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -17.6166992188,
		Longitude: 143.167007446,
		Timezone:  "Australia/Brisbane",
	},
	"YABS": {
		ICAO:      "YABS",
		Name:      "Albion Downs Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -27.2833003998,
		Longitude: 120.3830032349,
		Timezone:  "Australia/Perth",
	},
	"YABU": {
		ICAO:      "YABU",
		Name:      "Amburla Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -23.3332996368,
		Longitude: 133.1829986572,
		Timezone:  "Australia/Darwin",
	},
	"YACI": {
		ICAO:      "YACI",
		Name:      "Arcadia Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -25.2000007629,
		Longitude: 148.6829986572,
		Timezone:  "Australia/Brisbane",
	},
	"YACS": {
		ICAO:      "YACS",
		Name:      "Acacia Downs Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -31.4167003632,
		Longitude: 141.8999938965,
		Timezone:  "Australia/Sydney",
	},
	"YADD": {
		ICAO:      "YADD",
		Name:      "Arubiddy Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -31.7999992371,
		Longitude: 125.9329986572,
		Timezone:  "Australia/Eucla",
	},
	"YADG": {
		ICAO:      "YADG",
		Name:      "Aldinga Airport",
		City:      "Aldinga",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 110,
		Latitude:  -35.2900009155,
		Longitude: 138.490005493,
		Timezone:  "Australia/Adelaide",
	},
	"YADI": {
		ICAO:      "YADI",
		Name:      "Adaminaby Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 3346,
		Latitude:  -35.9977989197,
		Longitude: 148.796005249,
		Timezone:  "Australia/Sydney",
	},
	"YADM": {
		ICAO:      "YADM",
		Name:      "Yandan Mine Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -21.27333333,
		Longitude: 146.99,
		Timezone:  "Australia/Brisbane",
	},
	"YADO": {
		ICAO:      "YADO",
		Name:      "Andado Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -25.411699295,
		Longitude: 135.2899932861,
		Timezone:  "Australia/Darwin",
	},
	"YADS": {
		ICAO:      "YADS",
		IATA:      "AWN",
		Name:      "Alton Downs Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -26.5333003998,
		Longitude: 139.266998291,
		Timezone:  "Australia/Adelaide",
	},
	"YADY": {
		ICAO:      "YADY",
		Name:      "Adaminaby Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -35.9832992554,
		Longitude: 148.8000030518,
		Timezone:  "Australia/Sydney",
	},
	"YAGD": {
		ICAO:      "YAGD",
		IATA:      "AUD",
		Name:      "Augustus Downs Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 46,
		Latitude:  -18.5149993896,
		Longitude: 139.8780059814,
		Timezone:  "Australia/Brisbane",
	},
	"YAHD": {
		ICAO:      "YAHD",
		Name:      "Ashburton Downs Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -23.3833007813,
		Longitude: 117.0329971313,
		Timezone:  "Australia/Perth",
	},
	"YAKG": {
		ICAO:      "YAKG",
		Name:      "Arckaringa Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -27.9466991425,
		Longitude: 134.7449951172,
		Timezone:  "Australia/Adelaide",
	},
	"YALA": {
		ICAO:      "YALA",
		IATA:      "MRP",
		Name:      "Marla Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 328,
		Latitude:  -27.3332996368,
		Longitude: 133.6269989014,
		Timezone:  "Australia/Adelaide",
	},
	"YALC": {
		ICAO:      "YALC",
		Name:      "Alcoota Station Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -22.8332996368,
		Longitude: 134.4499969482,
		Timezone:  "Australia/Darwin",
	},
	"YALG": {
		ICAO:      "YALG",
		Name:      "Adels Grove Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -18.7000007629,
		Longitude: 138.5330047607,
		Timezone:  "Australia/Brisbane",
	},
	"YALH": {
		ICAO:      "YALH",
		Name:      "Albilbah Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -24.7000007629,
		Longitude: 144.2830047607,
		Timezone:  "Australia/Brisbane",
	},
	"YALM": {
		ICAO:      "YALM",
		Name:      "Allambie Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -24.2667007446,
		Longitude: 134.3999938965,
		Timezone:  "Australia/Darwin",
	},
	"YALN": {
		ICAO:      "YALN",
		Name:      "Allandy Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -30.3833007813,
		Longitude: 142.6000061035,
		Timezone:  "Australia/Sydney",
	},
	"YALX": {
		ICAO:      "YALX",
		IATA:      "AXL",
		Name:      "Alexandria Homestead Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -19.0601997375,
		Longitude: 136.7100067139,
		Timezone:  "Australia/Darwin",
	},
	"YALY": {
		ICAO:      "YALY",
		Name:      "Alderley Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -22.4832992554,
		Longitude: 139.6329956055,
		Timezone:  "Australia/Brisbane",
	},
	"YAMB": {
		ICAO:      "YAMB",
		Name:      "RAAF Base Amberley",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 91,
		Latitude:  -27.6406002045,
		Longitude: 152.7120056152,
		Timezone:  "Australia/Brisbane",
	},
	"YAMC": {
		ICAO:      "YAMC",
		IATA:      "AXC",
		Name:      "Aramac Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 232,
		Latitude:  -22.9666996002,
		Longitude: 145.2420043945,
		Timezone:  "Australia/Brisbane",
	},
	"YAMJ": {
		ICAO:      "YAMJ",
		Name:      "Ampilatwatja Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -21.6550006866,
		Longitude: 135.2299957275,
		Timezone:  "Australia/Darwin",
	},
	"YAMK": {
		ICAO:      "YAMK",
		IATA:      "ADO",
		Name:      "Andamooka Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 76,
		Latitude:  -30.4382991791,
		Longitude: 137.1369934082,
		Timezone:  "Australia/Adelaide",
	},
	"YAML": {
		ICAO:      "YAML",
		Name:      "Armraynald Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -17.9666996002,
		Longitude: 139.766998291,
		Timezone:  "Australia/Brisbane",
	},
	"YAMM": {
		ICAO:      "YAMM",
		IATA:      "AMX",
		Name:      "Ammaroo Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -21.7383003235,
		Longitude: 135.2420043945,
		Timezone:  "Australia/Darwin",
	},
	"YAMT": {
		ICAO:      "YAMT",
		IATA:      "AMT",
		Name:      "Amata Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 695,
		Latitude:  -26.1082992554,
		Longitude: 131.2070007324,
		Timezone:  "Australia/Adelaide",
	},
	"YAND": {
		ICAO:      "YAND",
		Name:      "Answer Downs Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -21.6667003632,
		Longitude: 140.983001709,
		Timezone:  "Australia/Brisbane",
	},
	"YANG": {
		ICAO:      "YANG",
		IATA:      "WLP",
		Name:      "West Angelas Airport",
		Country:   "AU",
		Elevation: 2340,
		Latitude:  -23.1355555556,
		Longitude: 118.706975,
		Timezone:  "Australia/Perth",
	},
	"YANJ": {
		ICAO:      "YANJ",
		Name:      "Angatja Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -26.1000003815,
		Longitude: 130.3999938965,
		Timezone:  "Australia/Adelaide",
	},
	"YANK": {
		ICAO:      "YANK",
		Name:      "Anna Creek Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -28.8966999054,
		Longitude: 136.1699981689,
		Timezone:  "Australia/Adelaide",
	},
	"YANL": {
		ICAO:      "YANL",
		IATA:      "AYL",
		Name:      "Anthony Lagoon Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -18.018079031,
		Longitude: 135.535068512,
		Timezone:  "Australia/Darwin",
	},
	"YANN": {
		ICAO:      "YANN",
		Name:      "Anningie Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -21.8316993713,
		Longitude: 133.125,
		Timezone:  "Australia/Darwin",
	},
	"YANW": {
		ICAO:      "YANW",
		Name:      "Annitowa Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -21.2000007629,
		Longitude: 136.4479980469,
		Timezone:  "Australia/Darwin",
	},
	"YAOR": {
		ICAO:      "YAOR",
		Name:      "Ardmore Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -21.6499996185,
		Longitude: 139.1829986572,
		Timezone:  "Australia/Brisbane",
	},
	"YAPA": {
		ICAO:      "YAPA",
		Name:      "Anna Plains Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -19.2667007446,
		Longitude: 121.516998291,
		Timezone:  "Australia/Perth",
	},
	"YAPH": {
		ICAO:      "YAPH",
		IATA:      "ABH",
		Name:      "Alpha Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 1255,
		Latitude:  -23.6460990906,
		Longitude: 146.5839996338,
		Timezone:  "Australia/Brisbane",
	},
	"YAPO": {
		ICAO:      "YAPO",
		Name:      "Apollo Bay Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -38.7832984924,
		Longitude: 143.6329956055,
		Timezone:  "Australia/Melbourne",
	},
	"YARA": {
		ICAO:      "YARA",
		IATA:      "ARY",
		Name:      "Ararat Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 1008,
		Latitude:  -37.3093986511,
		Longitude: 142.9889984131,
		Timezone:  "Australia/Melbourne",
	},
	"YARD": {
		ICAO:      "YARD",
		Name:      "Argadargada Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -21.676700592,
		Longitude: 136.6779937744,
		Timezone:  "Australia/Darwin",
	},
	"YARG": {
		ICAO:      "YARG",
		IATA:      "GYL",
		Name:      "Argyle Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 522,
		Latitude:  -16.6368999481,
		Longitude: 128.4510040283,
		Timezone:  "Australia/Perth",
	},
	"YARI": {
		ICAO:      "YARI",
		Name:      "Arizona 1 Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -21.6667003632,
		Longitude: 141.5670013428,
		Timezone:  "Australia/Brisbane",
	},
	"YARK": {
		ICAO:      "YARK",
		Name:      "Arkaroola Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -30.4067001343,
		Longitude: 139.345993042,
		Timezone:  "Australia/Adelaide",
	},
	"YARL": {
		ICAO:      "YARL",
		Name:      "Ardlethan Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -34.3666992188,
		Longitude: 146.8829956055,
		Timezone:  "Australia/Sydney",
	},
	"YARM": {
		ICAO:      "YARM",
		IATA:      "ARM",
		Name:      "Armidale Airport",
		City:      "Armidale",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 3556,
		Latitude:  -30.5280990601,
		Longitude: 151.617004395,
		Timezone:  "Australia/Sydney",
	},
	"YARN": {
		ICAO:      "YARN",
		Name:      "Areyonga Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -24.0666999817,
		Longitude: 132.266998291,
		Timezone:  "Australia/Darwin",
	},
	"YARP": {
		ICAO:      "YARP",
		Name:      "Arapunya Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -22.3166999817,
		Longitude: 135.6999969482,
		Timezone:  "Australia/Darwin",
	},
	"YARS": {
		ICAO:      "YARS",
		Name:      "Ardrossan Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 92,
		Latitude:  -34.4416999817,
		Longitude: 137.8829956055,
		Timezone:  "Australia/Adelaide",
	},
	"YARY": {
		ICAO:      "YARY",
		IATA:      "AAB",
		Name:      "Arrabury Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 102,
		Latitude:  -26.7000007629,
		Longitude: 141.0500030518,
		Timezone:  "Australia/Brisbane",
	},
	"YASF": {
		ICAO:      "YASF",
		Name:      "Ashford Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -29.3166999817,
		Longitude: 151.0500030518,
		Timezone:  "Australia/Sydney",
	},
	"YASS": {
		ICAO:      "YASS",
		Name:      "Bakblok Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 1624,
		Latitude:  -34.8866996765,
		Longitude: 149.0149993896,
		Timezone:  "Australia/Sydney",
	},
	"YATL": {
		ICAO:      "YATL",
		Name:      "Atula Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -23.2833003998,
		Longitude: 136.3670043945,
		Timezone:  "Australia/Darwin",
	},
	"YATN": {
		ICAO:      "YATN",
		Name:      "Atherton Airport",
		City:      "Atherton",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 2450,
		Latitude:  -17.2616996765,
		Longitude: 145.51499939,
		Timezone:  "Australia/Brisbane",
	},
	"YATR": {
		ICAO:      "YATR",
		Name:      "Amphitheatre Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -37.1833000183,
		Longitude: 143.3999938965,
		Timezone:  "Australia/Melbourne",
	},
	"YATY": {
		ICAO:      "YATY",
		Name:      "Atley Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -28.2166996002,
		Longitude: 119.25,
		Timezone:  "Australia/Perth",
	},
	"YAUA": {
		ICAO:      "YAUA",
		Name:      "Augathella Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -25.7549991608,
		Longitude: 146.5870056152,
		Timezone:  "Australia/Brisbane",
	},
	"YAUG": {
		ICAO:      "YAUG",
		Name:      "Augusta Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -34.3666992188,
		Longitude: 115.1500015259,
		Timezone:  "Australia/Perth",
	},
	"YAUR": {
		ICAO:      "YAUR",
		IATA:      "AUU",
		Name:      "Aurukun Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 31,
		Latitude:  -13.3538999557,
		Longitude: 141.720993042,
		Timezone:  "Australia/Brisbane",
	},
	"YAUS": {
		ICAO:      "YAUS",
		IATA:      "AWP",
		Name:      "Austral Downs Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -20.5,
		Longitude: 137.75,
		Timezone:  "Australia/Darwin",
	},
	"YAUV": {
		ICAO:      "YAUV",
		IATA:      "AVG",
		Name:      "Auvergne Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -15.6999998093,
		Longitude: 130,
		Timezone:  "Australia/Darwin",
	},
	"YAVD": {
		ICAO:      "YAVD",
		Name:      "Avon Downs Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -20.0333003998,
		Longitude: 137.516998291,
		Timezone:  "Australia/Darwin",
	},
	"YAVM": {
		ICAO:      "YAVM",
		Name:      "Avonmore Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -36.5666999817,
		Longitude: 144.5500030518,
		Timezone:  "Australia/Melbourne",
	},
	"YAWT": {
		ICAO:      "YAWT",
		Name:      "Agnes Water Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -24.2000007629,
		Longitude: 151.8970031738,
		Timezone:  "Australia/Brisbane",
	},
	"YAYE": {
		ICAO:      "YAYE",
		IATA:      "AYQ",
		Name:      "Ayers Rock Connellan Airport",
		City:      "Ayers Rock",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 1626,
		Latitude:  -25.1861000061,
		Longitude: 130.9759979248,
		Timezone:  "Australia/Darwin",
	},
	"YAYR": {
		ICAO:      "YAYR",
		IATA:      "AYR",
		Name:      "Ayr Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 41,
		Latitude:  -19.584400177,
		Longitude: 147.328994751,
		Timezone:  "Australia/Brisbane",
	},
	"YBAB": {
		ICAO:      "YBAB",
		Name:      "Baralaba",
		City:      "Baralaba",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 340,
		Latitude:  -24.1866666667,
		Longitude: 149.845,
		Timezone:  "Australia/Brisbane",
	},
	"YBAD": {
		ICAO:      "YBAD",
		Name:      "Baradine Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -30.9549999237,
		Longitude: 149.0919952393,
		Timezone:  "Australia/Sydney",
	},
	"YBAF": {
		ICAO:      "YBAF",
		IATA:      "ACF",
		Name:      "Brisbane Archerfield Airport",
		City:      "Brisbane",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 63,
		Latitude:  -27.5702991486,
		Longitude: 153.007995605,
		Timezone:  "Australia/Brisbane",
	},
	"YBAH": {
		ICAO:      "YBAH",
		Name:      "Bauhinia Downs Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -16.1333007813,
		Longitude: 135.4669952393,
		Timezone:  "Australia/Darwin",
	},
	"YBAL": {
		ICAO:      "YBAL",
		Name:      "Balladonia Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -32.3499984741,
		Longitude: 123.6179962158,
		Timezone:  "Australia/Perth",
	},
	"YBAM": {
		ICAO:      "YBAM",
		IATA:      "ABM",
		Name:      "Bamaga Injinoo Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 34,
		Latitude:  -10.950799942,
		Longitude: 142.4589996338,
		Timezone:  "Australia/Brisbane",
	},
	"YBAN": {
		ICAO:      "YBAN",
		Name:      "Mount Barnet Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -16.7371997833,
		Longitude: 125.908996582,
		Timezone:  "Australia/Perth",
	},
	"YBAO": {
		ICAO:      "YBAO",
		Name:      "Braidwood Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -35.4500007629,
		Longitude: 149.8500061035,
		Timezone:  "Australia/Sydney",
	},
	"YBAR": {
		ICAO:      "YBAR",
		IATA:      "BCI",
		Name:      "Barcaldine Airport",
		City:      "Barcaldine",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 878,
		Latitude:  -23.5652999878,
		Longitude: 145.307006836,
		Timezone:  "Australia/Brisbane",
	},
	"YBAS": {
		ICAO:      "YBAS",
		IATA:      "ASP",
		Name:      "Alice Springs Airport",
		City:      "Alice Springs",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 1789,
		Latitude:  -23.8066997528,
		Longitude: 133.9019927979,
		Timezone:  "Australia/Darwin",
	},
	"YBAU": {
		ICAO:      "YBAU",
		IATA:      "BDD",
		Name:      "Badu Island Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 14,
		Latitude:  -10.1499996185,
		Longitude: 142.1734,
		Timezone:  "Australia/Brisbane",
	},
	"YBAW": {
		ICAO:      "YBAW",
		IATA:      "BKP",
		Name:      "Barkly Downs Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 810,
		Latitude:  -20.4958333333,
		Longitude: 138.474722222,
		Timezone:  "Australia/Brisbane",
	},
	"YBBA": {
		ICAO:      "YBBA",
		Name:      "Barraba Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -30.3833007813,
		Longitude: 150.6000061035,
		Timezone:  "Australia/Sydney",
	},
	"YBBC": {
		ICAO:      "YBBC",
		Name:      "Bamboo Creek Airport",
		City:      "Bamboo Creek Gold Mine",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -20.9445991516,
		Longitude: 120.1650009155,
		Timezone:  "Australia/Perth",
	},
	"YBBE": {
		ICAO:      "YBBE",
		IATA:      "BBE",
		Name:      "Big Bell Airport",
		City:      "Big Bell",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -27.3285999298,
		Longitude: 117.672996521,
		Timezone:  "Australia/Perth",
	},
	"YBBL": {
		ICAO:      "YBBL",
		Name:      "Billabalong Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -27.4167003632,
		Longitude: 115.8330001831,
		Timezone:  "Australia/Perth",
	},
	"YBBN": {
		ICAO:      "YBBN",
		IATA:      "BNE",
		Name:      "Brisbane International Airport",
		City:      "Brisbane",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 13,
		Latitude:  -27.3841991425,
		Longitude: 153.1170043945,
		Timezone:  "Australia/Brisbane",
	},
	"YBBO": {
		ICAO:      "YBBO",
		Name:      "Bon Bon Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -30.4067001343,
		Longitude: 135.4799957275,
		Timezone:  "Australia/Adelaide",
	},
	"YBBT": {
		ICAO:      "YBBT",
		Name:      "Boort Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 90,
		Latitude:  -36.1366996765,
		Longitude: 143.7270050049,
		Timezone:  "Australia/Melbourne",
	},
	"YBCB": {
		ICAO:      "YBCB",
		Name:      "Bencubbin Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -30.8332996368,
		Longitude: 117.8669967651,
		Timezone:  "Australia/Perth",
	},
	"YBCG": {
		ICAO:      "YBCG",
		IATA:      "OOL",
		Name:      "Gold Coast Airport",
		City:      "Gold Coast",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 21,
		Latitude:  -28.1644001007,
		Longitude: 153.505004883,
		Timezone:  "Australia/Brisbane",
	},
	"YBCH": {
		ICAO:      "YBCH",
		Name:      "Beechworth Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 2000,
		Latitude:  -36.3933333333,
		Longitude: 146.696388889,
		Timezone:  "Australia/Melbourne",
	},
	"YBCK": {
		ICAO:      "YBCK",
		IATA:      "BKQ",
		Name:      "Blackall Airport",
		City:      "Blackall",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 928,
		Latitude:  -24.4277992249,
		Longitude: 145.429000854,
		Timezone:  "Australia/Brisbane",
	},
	"YBCL": {
		ICAO:      "YBCL",
		Name:      "Boolcarrol Station Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -30.0499992371,
		Longitude: 149.4170074463,
		Timezone:  "Australia/Sydney",
	},
	"YBCM": {
		ICAO:      "YBCM",
		Name:      "Coominya",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 340,
		Latitude:  -27.3907447666,
		Longitude: 152.466545105,
		Timezone:  "Australia/Brisbane",
	},
	"YBCR": {
		ICAO:      "YBCR",
		Name:      "Batchelor Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -13.0666999817,
		Longitude: 131.0330047607,
		Timezone:  "Australia/Darwin",
	},
	"YBCS": {
		ICAO:      "YBCS",
		IATA:      "CNS",
		Name:      "Cairns International Airport",
		City:      "Cairns",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 10,
		Latitude:  -16.885799408,
		Longitude: 145.755004883,
		Timezone:  "Australia/Brisbane",
	},
	"YBCV": {
		ICAO:      "YBCV",
		IATA:      "CTL",
		Name:      "Charleville Airport",
		City:      "Charleville",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 1003,
		Latitude:  -26.4132995605,
		Longitude: 146.261993408,
		Timezone:  "Australia/Brisbane",
	},
	"YBDF": {
		ICAO:      "YBDF",
		IATA:      "BDW",
		Name:      "Bedford Downs Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 1750,
		Latitude:  -17.286699295,
		Longitude: 127.462997437,
		Timezone:  "Australia/Perth",
	},
	"YBDG": {
		ICAO:      "YBDG",
		IATA:      "BXG",
		Name:      "Bendigo Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 705,
		Latitude:  -36.7393989563,
		Longitude: 144.330001831,
		Timezone:  "Australia/Melbourne",
	},
	"YBDP": {
		ICAO:      "YBDP",
		Name:      "Bridport Airport",
		State:     "Tasmania",
		Country:   "AU",
		Latitude:  -41.0167007446,
		Longitude: 147.4170074463,
		Timezone:  "Australia/Hobart",
	},
	"YBDS": {
		ICAO:      "YBDS",
		Name:      "Birthday Siding Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -30.2833003998,
		Longitude: 134.516998291,
		Timezone:  "Australia/Adelaide",
	},
	"YBDU": {
		ICAO:      "YBDU",
		Name:      "Birrindudu Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -18.3833007813,
		Longitude: 129.483001709,
		Timezone:  "Australia/Darwin",
	},
	"YBDV": {
		ICAO:      "YBDV",
		IATA:      "BVI",
		Name:      "Birdsville Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 159,
		Latitude:  -25.8974990845,
		Longitude: 139.3480072021,
		Timezone:  "Australia/Brisbane",
	},
	"YBDX": {
		ICAO:      "YBDX",
		Name:      "Barradale Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -22.8633003235,
		Longitude: 114.9629974365,
		Timezone:  "Australia/Perth",
	},
	"YBEB": {
		ICAO:      "YBEB",
		IATA:      "BXF",
		Name:      "Pumululu National Park",
		City:      "Bellburn",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 810,
		Latitude:  -17.545,
		Longitude: 128.305,
		Timezone:  "Australia/Perth",
	},
	"YBEC": {
		ICAO:      "YBEC",
		Name:      "Beacon Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -30.4832992554,
		Longitude: 117.8830032349,
		Timezone:  "Australia/Perth",
	},
	"YBEE": {
		ICAO:      "YBEE",
		Name:      "Beverley Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 116,
		Latitude:  -30.1867008209,
		Longitude: 139.5579986572,
		Timezone:  "Australia/Adelaide",
	},
	"YBEF": {
		ICAO:      "YBEF",
		Name:      "Beaufort Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -37.4500007629,
		Longitude: 143.25,
		Timezone:  "Australia/Melbourne",
	},
	"YBEL": {
		ICAO:      "YBEL",
		Name:      "Bothwell Airport",
		State:     "Tasmania",
		Country:   "AU",
		Latitude:  -42.3666992188,
		Longitude: 147.0330047607,
		Timezone:  "Australia/Hobart",
	},
	"YBEO": {
		ICAO:      "YBEO",
		IATA:      "BTX",
		Name:      "Betoota Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 91,
		Latitude:  -25.6417007446,
		Longitude: 140.7830047607,
		Timezone:  "Australia/Brisbane",
	},
	"YBEU": {
		ICAO:      "YBEU",
		Name:      "Beulah 1 Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -35.9500007629,
		Longitude: 142.4170074463,
		Timezone:  "Australia/Melbourne",
	},
	"YBEV": {
		ICAO:      "YBEV",
		Name:      "Beverley Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -32.1333007813,
		Longitude: 116.9499969482,
		Timezone:  "Australia/Perth",
	},
	"YBEW": {
		ICAO:      "YBEW",
		Name:      "Beechworth Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -31.9167003632,
		Longitude: 144.6329956055,
		Timezone:  "Australia/Sydney",
	},
	"YBFR": {
		ICAO:      "YBFR",
		Name:      "Balfour Airport",
		State:     "Tasmania",
		Country:   "AU",
		Latitude:  -41.2590332628,
		Longitude: 144.886064529,
		Timezone:  "Australia/Hobart",
	},
	"YBFT": {
		ICAO:      "YBFT",
		Name:      "Beaufort",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 1300,
		Latitude:  -37.495,
		Longitude: 143.43,
		Timezone:  "Australia/Melbourne",
	},
	"YBGB": {
		ICAO:      "YBGB",
		Name:      "Beagle Bay Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -16.9832992554,
		Longitude: 122.6500015259,
		Timezone:  "Australia/Perth",
	},
	"YBGD": {
		ICAO:      "YBGD",
		IATA:      "OCM",
		Name:      "Boolgeeda Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 1871,
		Latitude:  -22.54,
		Longitude: 117.275,
		Timezone:  "Australia/Perth",
	},
	"YBGI": {
		ICAO:      "YBGI",
		Name:      "Balgair Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -31.0666999817,
		Longitude: 125.6669998169,
		Timezone:  "Australia/Perth",
	},
	"YBGO": {
		ICAO:      "YBGO",
		IATA:      "BQW",
		Name:      "Balgo Hill Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 439,
		Latitude:  -20.1483001709,
		Longitude: 127.9729995728,
		Timezone:  "Australia/Perth",
	},
	"YBGR": {
		ICAO:      "YBGR",
		Name:      "Bridgewater Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 91,
		Latitude:  -36.6166992188,
		Longitude: 143.9499969482,
		Timezone:  "Australia/Melbourne",
	},
	"YBGT": {
		ICAO:      "YBGT",
		Name:      "Budgerygar Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -25.3700008392,
		Longitude: 143.7729949951,
		Timezone:  "Australia/Brisbane",
	},
	"YBGY": {
		ICAO:      "YBGY",
		Name:      "Biniguy Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -29.5049991608,
		Longitude: 150.1920013428,
		Timezone:  "Australia/Sydney",
	},
	"YBHB": {
		ICAO:      "YBHB",
		Name:      "Bathurst Harbour Airport",
		State:     "Tasmania",
		Country:   "AU",
		Latitude:  -43.4207000732,
		Longitude: 146.1560058594,
		Timezone:  "Australia/Hobart",
	},
	"YBHI": {
		ICAO:      "YBHI",
		IATA:      "BHQ",
		Name:      "Broken Hill Airport",
		City:      "Broken Hill",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 958,
		Latitude:  -32.0013999939,
		Longitude: 141.472000122,
		Timezone:  "Australia/Broken_Hill",
	},
	"YBHK": {
		ICAO:      "YBHK",
		Name:      "Bushy Park Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -21.2667007446,
		Longitude: 139.7169952393,
		Timezone:  "Australia/Brisbane",
	},
	"YBHM": {
		ICAO:      "YBHM",
		IATA:      "HTI",
		Name:      "Hamilton Island Airport",
		City:      "Hamilton Island",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 15,
		Latitude:  -20.3581008911,
		Longitude: 148.95199585,
		Timezone:  "Australia/Lindeman",
	},
	"YBIA": {
		ICAO:      "YBIA",
		Name:      "Bingara Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -29.7999992371,
		Longitude: 150.5330047607,
		Timezone:  "Australia/Sydney",
	},
	"YBID": {
		ICAO:      "YBID",
		Name:      "Binda",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 2450,
		Latitude:  -34.3033333333,
		Longitude: 149.36,
		Timezone:  "Australia/Sydney",
	},
	"YBIE": {
		ICAO:      "YBIE",
		IATA:      "BEU",
		Name:      "Bedourie Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 300,
		Latitude:  -24.3460998535,
		Longitude: 139.4600067139,
		Timezone:  "Australia/Brisbane",
	},
	"YBIK": {
		ICAO:      "YBIK",
		Name:      "Bindook Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -34.1666984558,
		Longitude: 150.1000061035,
		Timezone:  "Australia/Sydney",
	},
	"YBIL": {
		ICAO:      "YBIL",
		IATA:      "BIW",
		Name:      "Billiluna Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 1000,
		Latitude:  -19.5666999817,
		Longitude: 127.666999817,
		Timezone:  "Australia/Perth",
	},
	"YBIN": {
		ICAO:      "YBIN",
		Name:      "Biggenden Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -25.5249996185,
		Longitude: 152.0500030518,
		Timezone:  "Australia/Brisbane",
	},
	"YBIR": {
		ICAO:      "YBIR",
		Name:      "Birchip Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 340,
		Latitude:  -35.9996986389,
		Longitude: 142.9170074463,
		Timezone:  "Australia/Melbourne",
	},
	"YBIU": {
		ICAO:      "YBIU",
		Name:      "Ballidu Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -30.5932998657,
		Longitude: 116.7799987793,
		Timezone:  "Australia/Perth",
	},
	"YBIZ": {
		ICAO:      "YBIZ",
		IATA:      "BZP",
		Name:      "Bizant Airport",
		City:      "Lakefield National Park",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 65,
		Latitude:  -14.7402777778,
		Longitude: 144.119444444,
		Timezone:  "Australia/Brisbane",
	},
	"YBJW": {
		ICAO:      "YBJW",
		Name:      "Banjawarn Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -27.6667003632,
		Longitude: 121.5999984741,
		Timezone:  "Australia/Perth",
	},
	"YBKE": {
		ICAO:      "YBKE",
		IATA:      "BRK",
		Name:      "Bourke Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 352,
		Latitude:  -30.0391998291,
		Longitude: 145.9519958496,
		Timezone:  "Australia/Sydney",
	},
	"YBKS": {
		ICAO:      "YBKS",
		Name:      "Barkly Wayside Inn Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -19.7091999054,
		Longitude: 135.8190002441,
		Timezone:  "Australia/Darwin",
	},
	"YBKT": {
		ICAO:      "YBKT",
		IATA:      "BUC",
		Name:      "Burketown Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 21,
		Latitude:  -17.7486000061,
		Longitude: 139.533996582,
		Timezone:  "Australia/Brisbane",
	},
	"YBLA": {
		ICAO:      "YBLA",
		IATA:      "BLN",
		Name:      "Benalla Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 569,
		Latitude:  -36.5518989563,
		Longitude: 146.0070037842,
		Timezone:  "Australia/Melbourne",
	},
	"YBLB": {
		ICAO:      "YBLB",
		Name:      "Billabong Road House Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -26.8166999817,
		Longitude: 114.6169967651,
		Timezone:  "Australia/Perth",
	},
	"YBLC": {
		ICAO:      "YBLC",
		IATA:      "LCN",
		Name:      "Balcanoona Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 144,
		Latitude:  -30.5349998474,
		Longitude: 139.3370056152,
		Timezone:  "Australia/Adelaide",
	},
	"YBLD": {
		ICAO:      "YBLD",
		Name:      "Brooklands Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -31.8250007629,
		Longitude: 116.7959976196,
		Timezone:  "Australia/Perth",
	},
	"YBLG": {
		ICAO:      "YBLG",
		Name:      "Bollards Lagoon Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -28.9832992554,
		Longitude: 140.8500061035,
		Timezone:  "Australia/Adelaide",
	},
	"YBLH": {
		ICAO:      "YBLH",
		Name:      "Bellalie Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -27.0333003998,
		Longitude: 142.9499969482,
		Timezone:  "Australia/Brisbane",
	},
	"YBLL": {
		ICAO:      "YBLL",
		IATA:      "BLS",
		Name:      "Bollon Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 182,
		Latitude:  -28.0583000183,
		Longitude: 147.483001709,
		Timezone:  "Australia/Brisbane",
	},
	"YBLM": {
		ICAO:      "YBLM",
		Name:      "Blinman Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 1991,
		Latitude:  -31.1186008453,
		Longitude: 138.7259979248,
		Timezone:  "Australia/Adelaide",
	},
	"YBLN": {
		ICAO:      "YBLN",
		IATA:      "BQB",
		Name:      "Busselton Regional Airport",
		City:      "Busselton",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 55,
		Latitude:  -33.6884231567,
		Longitude: 115.401596069,
		Timezone:  "Australia/Perth",
	},
	"YBLT": {
		ICAO:      "YBLT",
		Name:      "Ballarat Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 1433,
		Latitude:  -37.5116996765,
		Longitude: 143.7910003662,
		Timezone:  "Australia/Melbourne",
	},
	"YBLU": {
		ICAO:      "YBLU",
		Name:      "Bellevue Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -27.6149997711,
		Longitude: 120.6119995117,
		Timezone:  "Australia/Perth",
	},
	"YBMA": {
		ICAO:      "YBMA",
		IATA:      "ISA",
		Name:      "Mount Isa Airport",
		City:      "Mount Isa",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 1121,
		Latitude:  -20.6639003754,
		Longitude: 139.488998413,
		Timezone:  "Australia/Brisbane",
	},
	"YBMC": {
		ICAO:      "YBMC",
		IATA:      "MCY",
		Name:      "Sunshine Coast Airport",
		City:      "Maroochydore",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 15,
		Latitude:  -26.6033000946,
		Longitude: 153.091003418,
		Timezone:  "Australia/Brisbane",
	},
	"YBMD": {
		ICAO:      "YBMD",
		Name:      "Bloomfield River Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -15.8999996185,
		Longitude: 145.3329925537,
		Timezone:  "Australia/Brisbane",
	},
	"YBMI": {
		ICAO:      "YBMI",
		Name:      "Boomi Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -28.7332992554,
		Longitude: 149.5829925537,
		Timezone:  "Australia/Sydney",
	},
	"YBMK": {
		ICAO:      "YBMK",
		IATA:      "MKY",
		Name:      "Mackay Airport",
		City:      "Mackay",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 19,
		Latitude:  -21.1716995239,
		Longitude: 149.179992676,
		Timezone:  "Australia/Brisbane",
	},
	"YBMM": {
		ICAO:      "YBMM",
		Name:      "Bamawm Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -36.25,
		Longitude: 144.6329956055,
		Timezone:  "Australia/Melbourne",
	},
	"YBMO": {
		ICAO:      "YBMO",
		Name:      "Bombala Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 2415,
		Latitude:  -36.9049987793,
		Longitude: 149.1820068359,
		Timezone:  "Australia/Sydney",
	},
	"YBMR": {
		ICAO:      "YBMR",
		Name:      "Barmera Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -34.2566986084,
		Longitude: 140.4579925537,
		Timezone:  "Australia/Adelaide",
	},
	"YBMY": {
		ICAO:      "YBMY",
		Name:      "Bamyili Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 215,
		Latitude:  -14.5200004578,
		Longitude: 132.8829956055,
		Timezone:  "Australia/Darwin",
	},
	"YBNA": {
		ICAO:      "YBNA",
		IATA:      "BNK",
		Name:      "Ballina Byron Gateway Airport",
		City:      "Ballina",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 7,
		Latitude:  -28.8339004517,
		Longitude: 153.56199646,
		Timezone:  "Australia/Sydney",
	},
	"YBNC": {
		ICAO:      "YBNC",
		Name:      "Bannockburn Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -21.7999992371,
		Longitude: 145.0829925537,
		Timezone:  "Australia/Brisbane",
	},
	"YBNM": {
		ICAO:      "YBNM",
		Name:      "Benmara Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -17.9367008209,
		Longitude: 136.8999938965,
		Timezone:  "Australia/Darwin",
	},
	"YBNS": {
		ICAO:      "YBNS",
		IATA:      "BSJ",
		Name:      "Bairnsdale Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 165,
		Latitude:  -37.8875007629,
		Longitude: 147.5679931641,
		Timezone:  "Australia/Melbourne",
	},
	"YBNY": {
		ICAO:      "YBNY",
		Name:      "Brunchilly Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -18.8700008392,
		Longitude: 134.4620056152,
		Timezone:  "Australia/Darwin",
	},
	"YBOA": {
		ICAO:      "YBOA",
		Name:      "Boonah Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -28.0167007446,
		Longitude: 152.6820068359,
		Timezone:  "Australia/Brisbane",
	},
	"YBOC": {
		ICAO:      "YBOC",
		Name:      "Booleroo Centre Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 418,
		Latitude:  -32.9049987793,
		Longitude: 138.3600006104,
		Timezone:  "Australia/Adelaide",
	},
	"YBOD": {
		ICAO:      "YBOD",
		Name:      "Bodalla Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -26.2999992371,
		Longitude: 143.3829956055,
		Timezone:  "Australia/Brisbane",
	},
	"YBOG": {
		ICAO:      "YBOG",
		Name:      "Boggabri Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -30.7166996002,
		Longitude: 150.0500030518,
		Timezone:  "Australia/Sydney",
	},
	"YBOI": {
		ICAO:      "YBOI",
		IATA:      "GIC",
		Name:      "Boigu Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 23,
		Latitude:  -9.2327804565,
		Longitude: 142.218002319,
		Timezone:  "Australia/Brisbane",
	},
	"YBOK": {
		ICAO:      "YBOK",
		IATA:      "OKY",
		Name:      "Oakey Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 1335,
		Latitude:  -27.4113998413,
		Longitude: 151.7350006104,
		Timezone:  "Australia/Brisbane",
	},
	"YBOM": {
		ICAO:      "YBOM",
		Name:      "Bombala Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -36.9166984558,
		Longitude: 149.1670074463,
		Timezone:  "Australia/Sydney",
	},
	"YBOO": {
		ICAO:      "YBOO",
		Name:      "Catholic Agricultural College Bindoon Airstrip",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 935,
		Latitude:  -31.3402996063,
		Longitude: 116.18800354,
		Timezone:  "Australia/Perth",
	},
	"YBOP": {
		ICAO:      "YBOP",
		Name:      "Boyup Brook Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -33.9000015259,
		Longitude: 116.4670028687,
		Timezone:  "Australia/Perth",
	},
	"YBOR": {
		ICAO:      "YBOR",
		Name:      "Bordertown Airport",
		City:      "Bordertown",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 265,
		Latitude:  -36.2649993896,
		Longitude: 140.712005615,
		Timezone:  "Australia/Adelaide",
	},
	"YBOT": {
		ICAO:      "YBOT",
		Name:      "Boatman Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -27.25,
		Longitude: 146.9170074463,
		Timezone:  "Australia/Brisbane",
	},
	"YBOU": {
		ICAO:      "YBOU",
		IATA:      "BQL",
		Name:      "Boulia Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 542,
		Latitude:  -22.9132995605,
		Longitude: 139.8999938965,
		Timezone:  "Australia/Brisbane",
	},
	"YBOV": {
		ICAO:      "YBOV",
		Name:      "Border Village Airport",
		City:      "Border Village",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 308,
		Latitude:  -31.6394004822,
		Longitude: 129.0119934082,
		Timezone:  "Australia/Adelaide",
	},
	"YBOY": {
		ICAO:      "YBOY",
		Name:      "Booylgoo Springs Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -27.75,
		Longitude: 119.9000015259,
		Timezone:  "Australia/Perth",
	},
	"YBPI": {
		ICAO:      "YBPI",
		IATA:      "BMP",
		Name:      "Brampton Island Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 11,
		Latitude:  -20.8033008575,
		Longitude: 149.2700042725,
		Timezone:  "Australia/Brisbane",
	},
	"YBPN": {
		ICAO:      "YBPN",
		IATA:      "PPP",
		Name:      "Proserpine Whitsunday Coast Airport",
		City:      "Proserpine",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 82,
		Latitude:  -20.4950008392,
		Longitude: 148.552001953,
		Timezone:  "Australia/Brisbane",
	},
	"YBRA": {
		ICAO:      "YBRA",
		Name:      "Benambra Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 2200,
		Latitude:  -36.9668665594,
		Longitude: 147.699143887,
		Timezone:  "Australia/Melbourne",
	},
	"YBRC": {
		ICAO:      "YBRC",
		Name:      "Barrow Creek Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -21.5333003998,
		Longitude: 133.8829956055,
		Timezone:  "Australia/Darwin",
	},
	"YBRJ": {
		ICAO:      "YBRJ",
		Name:      "Burren Junction Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -30.1583003998,
		Longitude: 148.9750061035,
		Timezone:  "Australia/Sydney",
	},
	"YBRK": {
		ICAO:      "YBRK",
		IATA:      "ROK",
		Name:      "Rockhampton Airport",
		City:      "Rockhampton",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 34,
		Latitude:  -23.3819007874,
		Longitude: 150.475006104,
		Timezone:  "Australia/Brisbane",
	},
	"YBRL": {
		ICAO:      "YBRL",
		IATA:      "BOX",
		Name:      "Borroloola Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 55,
		Latitude:  -16.0753002167,
		Longitude: 136.3020019531,
		Timezone:  "Australia/Darwin",
	},
	"YBRM": {
		ICAO:      "YBRM",
		IATA:      "BME",
		Name:      "Broome International Airport",
		City:      "Broome",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 56,
		Latitude:  -17.9447002411,
		Longitude: 122.2320022583,
		Timezone:  "Australia/Perth",
	},
	"YBRN": {
		ICAO:      "YBRN",
		IATA:      "BZD",
		Name:      "Balranald Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 210,
		Latitude:  -34.6236000061,
		Longitude: 143.5780029297,
		Timezone:  "Australia/Sydney",
	},
	"YBRO": {
		ICAO:      "YBRO",
		Name:      "Bruce Rock Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -31.8833007813,
		Longitude: 118.1169967651,
		Timezone:  "Australia/Perth",
	},
	"YBRS": {
		ICAO:      "YBRS",
		Name:      "Barwon Heads Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 15,
		Latitude:  -38.258056,
		Longitude: 144.4275,
		Timezone:  "Australia/Melbourne",
	},
	"YBRU": {
		ICAO:      "YBRU",
		IATA:      "BTD",
		Name:      "Brunette Downs Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 700,
		Latitude:  -18.6399993896,
		Longitude: 135.93800354,
		Timezone:  "Australia/Darwin",
	},
	"YBRW": {
		ICAO:      "YBRW",
		IATA:      "BWQ",
		Name:      "Brewarrina Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 414,
		Latitude:  -29.9738998413,
		Longitude: 146.8170013428,
		Timezone:  "Australia/Sydney",
	},
	"YBRY": {
		ICAO:      "YBRY",
		IATA:      "BYP",
		Name:      "Barimunya Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 2082,
		Latitude:  -22.6739006042,
		Longitude: 119.1660003662,
		Timezone:  "Australia/Perth",
	},
	"YBSG": {
		ICAO:      "YBSG",
		Name:      "RAAF Scherger",
		City:      "Weipa",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 145,
		Latitude:  -12.6239004135,
		Longitude: 142.0870056152,
		Timezone:  "Australia/Brisbane",
	},
	"YBSP": {
		ICAO:      "YBSP",
		Name:      "Bond Springs Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -23.5149993896,
		Longitude: 133.8430023193,
		Timezone:  "Australia/Darwin",
	},
	"YBSR": {
		ICAO:      "YBSR",
		Name:      "Blackstone Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -25.986700058,
		Longitude: 128.2879943848,
		Timezone:  "Australia/Perth",
	},
	"YBSS": {
		ICAO:      "YBSS",
		Name:      "Bacchus Marsh Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 520,
		Latitude:  -37.7333333333,
		Longitude: 144.421666667,
		Timezone:  "Australia/Melbourne",
	},
	"YBTD": {
		ICAO:      "YBTD",
		Name:      "Brighton Downs Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -23.3500003815,
		Longitude: 143.5330047607,
		Timezone:  "Australia/Brisbane",
	},
	"YBTH": {
		ICAO:      "YBTH",
		IATA:      "BHS",
		Name:      "Bathurst Airport",
		City:      "Bathurst",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 2435,
		Latitude:  -33.4094009399,
		Longitude: 149.651992798,
		Timezone:  "Australia/Sydney",
	},
	"YBTI": {
		ICAO:      "YBTI",
		IATA:      "BRT",
		Name:      "Bathurst Island Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 67,
		Latitude:  -11.769200325,
		Longitude: 130.6199951172,
		Timezone:  "Australia/Darwin",
	},
	"YBTK": {
		ICAO:      "YBTK",
		Name:      "Bentinck Island Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 45,
		Latitude:  -17.0867023468,
		Longitude: 139.5657806396,
		Timezone:  "Australia/Brisbane",
	},
	"YBTL": {
		ICAO:      "YBTL",
		IATA:      "TSV",
		Name:      "Townsville Airport",
		City:      "Townsville",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 18,
		Latitude:  -19.2525005341,
		Longitude: 146.7649993896,
		Timezone:  "Australia/Brisbane",
	},
	"YBTN": {
		ICAO:      "YBTN",
		Name:      "Barton Siding Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -30.5249996185,
		Longitude: 132.6600036621,
		Timezone:  "Australia/Adelaide",
	},
	"YBTR": {
		ICAO:      "YBTR",
		IATA:      "BLT",
		Name:      "Blackwater Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 657,
		Latitude:  -23.603099823,
		Longitude: 148.8070068359,
		Timezone:  "Australia/Brisbane",
	},
	"YBTS": {
		ICAO:      "YBTS",
		Name:      "Battery Downs Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -19.4333000183,
		Longitude: 145.8500061035,
		Timezone:  "Australia/Brisbane",
	},
	"YBTT": {
		ICAO:      "YBTT",
		Name:      "Buttabone Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -31.3582992554,
		Longitude: 147.6329956055,
		Timezone:  "Australia/Sydney",
	},
	"YBTV": {
		ICAO:      "YBTV",
		Name:      "Batavia Downs Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -12.6667003632,
		Longitude: 142.6670074463,
		Timezone:  "Australia/Brisbane",
	},
	"YBUA": {
		ICAO:      "YBUA",
		Name:      "Bundarra Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -21.9666996002,
		Longitude: 148.5330047607,
		Timezone:  "Australia/Brisbane",
	},
	"YBUC": {
		ICAO:      "YBUC",
		Name:      "Butch Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -33.8333015442,
		Longitude: 115.0999984741,
		Timezone:  "Australia/Perth",
	},
	"YBUD": {
		ICAO:      "YBUD",
		IATA:      "BDB",
		Name:      "Bundaberg Airport",
		City:      "Bundaberg",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 107,
		Latitude:  -24.9039001465,
		Longitude: 152.319000244,
		Timezone:  "Australia/Brisbane",
	},
	"YBUG": {
		ICAO:      "YBUG",
		Name:      "Bulga Downs Station Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -28.5,
		Longitude: 119.733001709,
		Timezone:  "Australia/Perth",
	},
	"YBUI": {
		ICAO:      "YBUI",
		Name:      "Burnie Airport",
		State:     "Tasmania",
		Country:   "AU",
		Latitude:  -41.0491981506,
		Longitude: 145.8800048828,
		Timezone:  "Australia/Hobart",
	},
	"YBUL": {
		ICAO:      "YBUL",
		Name:      "Bulgunnia Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -30.1867008209,
		Longitude: 134.9029998779,
		Timezone:  "Australia/Adelaide",
	},
	"YBUN": {
		ICAO:      "YBUN",
		IATA:      "BUY",
		Name:      "Bunbury Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 53,
		Latitude:  -33.3782997131,
		Longitude: 115.6770019531,
		Timezone:  "Australia/Perth",
	},
	"YBUO": {
		ICAO:      "YBUO",
		Name:      "Bulloo Downs Station Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -24.0167007446,
		Longitude: 119.5670013428,
		Timezone:  "Australia/Perth",
	},
	"YBUP": {
		ICAO:      "YBUP",
		Name:      "Bunyip Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -38.0167007446,
		Longitude: 145.75,
		Timezone:  "Australia/Melbourne",
	},
	"YBUU": {
		ICAO:      "YBUU",
		Name:      "Bungle Bungle Airport",
		City:      "Bungle Bungle",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -17.54529953,
		Longitude: 128.3070068359,
		Timezone:  "Australia/Perth",
	},
	"YBUX": {
		ICAO:      "YBUX",
		Name:      "Bulleringa Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -17.6499996185,
		Longitude: 143.8000030518,
		Timezone:  "Australia/Brisbane",
	},
	"YBUY": {
		ICAO:      "YBUY",
		Name:      "Bunyan Airfield",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 2540,
		Latitude:  -36.1342008376,
		Longitude: 149.131979942,
		Timezone:  "Australia/Sydney",
	},
	"YBVA": {
		ICAO:      "YBVA",
		Name:      "Balaklava Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -34.0843009949,
		Longitude: 138.3339996338,
		Timezone:  "Australia/Adelaide",
	},
	"YBVL": {
		ICAO:      "YBVL",
		Name:      "Blackville Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -31.5832996368,
		Longitude: 150.1829986572,
		Timezone:  "Australia/Sydney",
	},
	"YBVY": {
		ICAO:      "YBVY",
		Name:      "Bullo River Valley Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -15.4666996002,
		Longitude: 129.766998291,
		Timezone:  "Australia/Darwin",
	},
	"YBWG": {
		ICAO:      "YBWG",
		Name:      "Bronzewing Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 1645,
		Latitude:  -27.3656005859,
		Longitude: 121.0360031128,
		Timezone:  "Australia/Perth",
	},
	"YBWI": {
		ICAO:      "YBWI",
		Name:      "Burdekin Falls Dam Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -20.6000003815,
		Longitude: 147.0829925537,
		Timezone:  "Australia/Brisbane",
	},
	"YBWM": {
		ICAO:      "YBWM",
		IATA:      "BIP",
		Name:      "Bulimba Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 470,
		Latitude:  -16.8808002472,
		Longitude: 143.479003906,
		Timezone:  "Australia/Brisbane",
	},
	"YBWN": {
		ICAO:      "YBWN",
		IATA:      "ZBO",
		Name:      "Bowen Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 8,
		Latitude:  -20.0182991028,
		Longitude: 148.2149963379,
		Timezone:  "Australia/Brisbane",
	},
	"YBWP": {
		ICAO:      "YBWP",
		IATA:      "WEI",
		Name:      "Weipa Airport",
		City:      "Weipa",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 63,
		Latitude:  -12.6786003113,
		Longitude: 141.925003052,
		Timezone:  "Australia/Brisbane",
	},
	"YBWR": {
		ICAO:      "YBWR",
		Name:      "Bolwarra Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -17.3999996185,
		Longitude: 144.1829986572,
		Timezone:  "Australia/Brisbane",
	},
	"YBWS": {
		ICAO:      "YBWS",
		Name:      "Browse Is Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -14.111700058,
		Longitude: 123.5469970703,
		Timezone:  "Australia/Perth",
	},
	"YBWT": {
		ICAO:      "YBWT",
		Name:      "Bowthorn Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -18.0832996368,
		Longitude: 138.2749938965,
		Timezone:  "Australia/Brisbane",
	},
	"YBWW": {
		ICAO:      "YBWW",
		IATA:      "WTB",
		Name:      "Toowoomba Wellcamp Airport",
		City:      "Wellcamp",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 1509,
		Latitude:  -27.558333,
		Longitude: 151.793333,
		Timezone:  "Australia/Brisbane",
	},
	"YBWX": {
		ICAO:      "YBWX",
		IATA:      "BWB",
		Name:      "Barrow Island Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 26,
		Latitude:  -20.8644008636,
		Longitude: 115.40599823,
		Timezone:  "Australia/Perth",
	},
	"YBXH": {
		ICAO:      "YBXH",
		Name:      "Branxholme Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -37.8672981262,
		Longitude: 141.7019958496,
		Timezone:  "Australia/Melbourne",
	},
	"YBYA": {
		ICAO:      "YBYA",
		Name:      "Bryah Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -25.5333003998,
		Longitude: 118.733001709,
		Timezone:  "Australia/Perth",
	},
	"YBYD": {
		ICAO:      "YBYD",
		Name:      "Bonney Downs Station Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -22.2667007446,
		Longitude: 119.8830032349,
		Timezone:  "Australia/Perth",
	},
	"YBYI": {
		ICAO:      "YBYI",
		Name:      "Bruny Island Airport",
		State:     "Tasmania",
		Country:   "AU",
		Latitude:  -43.2341995239,
		Longitude: 147.3800048828,
		Timezone:  "Australia/Hobart",
	},
	"YBYR": {
		ICAO:      "YBYR",
		Name:      "Byrock Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -30.6667003632,
		Longitude: 146.3999938965,
		Timezone:  "Australia/Sydney",
	},
	"YBYS": {
		ICAO:      "YBYS",
		IATA:      "BVZ",
		Name:      "Beverley Springs Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 385,
		Latitude:  -16.7332992554,
		Longitude: 125.4329986572,
		Timezone:  "Australia/Perth",
	},
	"YBYW": {
		ICAO:      "YBYW",
		Name:      "Bayswater Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -25.4500007629,
		Longitude: 145.5829925537,
		Timezone:  "Australia/Brisbane",
	},
	"YBZA": {
		ICAO:      "YBZA",
		Name:      "Breeza Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -31.3166999817,
		Longitude: 150.516998291,
		Timezone:  "Australia/Sydney",
	},
	"YCAB": {
		ICAO:      "YCAB",
		Name:      "Caboolture Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 12,
		Latitude:  -27.0832996368,
		Longitude: 152.983001709,
		Timezone:  "Australia/Brisbane",
	},
	"YCAC": {
		ICAO:      "YCAC",
		Name:      "Cattle Creek Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -17.6166992188,
		Longitude: 131.5500030518,
		Timezone:  "Australia/Darwin",
	},
	"YCAE": {
		ICAO:      "YCAE",
		Name:      "Campbell Town Airport",
		State:     "Tasmania",
		Country:   "AU",
		Elevation: 710,
		Latitude:  -41.9672241211,
		Longitude: 147.5305480957,
		Timezone:  "Australia/Hobart",
	},
	"YCAG": {
		ICAO:      "YCAG",
		IATA:      "CGV",
		Name:      "Caiguna Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 87,
		Latitude:  -32.2649993896,
		Longitude: 125.4929962158,
		Timezone:  "Australia/Eucla",
	},
	"YCAH": {
		ICAO:      "YCAH",
		IATA:      "CLH",
		Name:      "Coolah Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 504,
		Latitude:  -31.7733001709,
		Longitude: 149.6100006104,
		Timezone:  "Australia/Sydney",
	},
	"YCAI": {
		ICAO:      "YCAI",
		Name:      "Callion Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -30.1166992188,
		Longitude: 120.5830001831,
		Timezone:  "Australia/Perth",
	},
	"YCAJ": {
		ICAO:      "YCAJ",
		Name:      "Cadjebut Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -18.7000007629,
		Longitude: 125.9000015259,
		Timezone:  "Australia/Perth",
	},
	"YCAL": {
		ICAO:      "YCAL",
		Name:      "Castlemaine Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -37.1333007813,
		Longitude: 144.1670074463,
		Timezone:  "Australia/Melbourne",
	},
	"YCAM": {
		ICAO:      "YCAM",
		Name:      "Cannington Station Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -21.875,
		Longitude: 140.8999938965,
		Timezone:  "Australia/Brisbane",
	},
	"YCAN": {
		ICAO:      "YCAN",
		Name:      "Cannon Hill Community Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -12.3667001724,
		Longitude: 132.9499969482,
		Timezone:  "Australia/Darwin",
	},
	"YCAR": {
		ICAO:      "YCAR",
		IATA:      "CVQ",
		Name:      "Carnarvon Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 13,
		Latitude:  -24.8805999756,
		Longitude: 113.6719970703,
		Timezone:  "Australia/Perth",
	},
	"YCAS": {
		ICAO:      "YCAS",
		IATA:      "CSI",
		Name:      "Casino Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 86,
		Latitude:  -28.8827991486,
		Longitude: 153.0670013428,
		Timezone:  "Australia/Sydney",
	},
	"YCBA": {
		ICAO:      "YCBA",
		IATA:      "CAZ",
		Name:      "Cobar Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 724,
		Latitude:  -31.5382995605,
		Longitude: 145.7940063477,
		Timezone:  "Australia/Sydney",
	},
	"YCBB": {
		ICAO:      "YCBB",
		IATA:      "COJ",
		Name:      "Coonabarabran Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 2117,
		Latitude:  -31.3325004578,
		Longitude: 149.266998291,
		Timezone:  "Australia/Sydney",
	},
	"YCBE": {
		ICAO:      "YCBE",
		IATA:      "CBY",
		Name:      "Canobie Airport",
		City:      "Canobie",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -19.4794006348,
		Longitude: 140.927001953,
		Timezone:  "Australia/Brisbane",
	},
	"YCBG": {
		ICAO:      "YCBG",
		Name:      "Hobart Cambridge Airport",
		State:     "Tasmania",
		Country:   "AU",
		Elevation: 67,
		Latitude:  -42.8266983032,
		Longitude: 147.4750061035,
		Timezone:  "Australia/Hobart",
	},
	"YCBN": {
		ICAO:      "YCBN",
		IATA:      "CBI",
		Name:      "Cape Barren Island Airport",
		State:     "Tasmania",
		Country:   "AU",
		Latitude:  -40.3917007446,
		Longitude: 148.016998291,
		Timezone:  "Australia/Hobart",
	},
	"YCBO": {
		ICAO:      "YCBO",
		Name:      "Cape Borda Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -35.7566986084,
		Longitude: 136.5980072021,
		Timezone:  "Australia/Adelaide",
	},
	"YCBP": {
		ICAO:      "YCBP",
		IATA:      "CPD",
		Name:      "Coober Pedy Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 740,
		Latitude:  -29.0400009155,
		Longitude: 134.720993042,
		Timezone:  "Australia/Adelaide",
	},
	"YCBR": {
		ICAO:      "YCBR",
		IATA:      "CRB",
		Name:      "Collarenebri Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 152,
		Latitude:  -29.5216999054,
		Longitude: 148.5820007324,
		Timezone:  "Australia/Sydney",
	},
	"YCBY": {
		ICAO:      "YCBY",
		Name:      "Daintree Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -16.2193412781,
		Longitude: 145.4247283936,
		Timezone:  "Australia/Brisbane",
	},
	"YCCA": {
		ICAO:      "YCCA",
		IATA:      "CCL",
		Name:      "Chinchilla Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 1028,
		Latitude:  -26.7749996185,
		Longitude: 150.6170043945,
		Timezone:  "Australia/Brisbane",
	},
	"YCCF": {
		ICAO:      "YCCF",
		Name:      "Cape Crawford Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -16.6833000183,
		Longitude: 135.733001709,
		Timezone:  "Australia/Darwin",
	},
	"YCCK": {
		ICAO:      "YCCK",
		Name:      "Canteen Creek Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -20.650800705,
		Longitude: 135.5850067139,
		Timezone:  "Australia/Darwin",
	},
	"YCCT": {
		ICAO:      "YCCT",
		IATA:      "CNC",
		Name:      "Coconut Island Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 3,
		Latitude:  -10.0500001907,
		Longitude: 143.0700073242,
		Timezone:  "Australia/Brisbane",
	},
	"YCCY": {
		ICAO:      "YCCY",
		IATA:      "CNJ",
		Name:      "Cloncurry Airport",
		City:      "Cloncurry",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 616,
		Latitude:  -20.6686000824,
		Longitude: 140.503997803,
		Timezone:  "Australia/Brisbane",
	},
	"YCDB": {
		ICAO:      "YCDB",
		Name:      "Coondambo Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -31.0666999817,
		Longitude: 135.8829956055,
		Timezone:  "Australia/Adelaide",
	},
	"YCDE": {
		ICAO:      "YCDE",
		Name:      "Cobden Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -38.326666,
		Longitude: 143.056666,
		Timezone:  "Australia/Melbourne",
	},
	"YCDH": {
		ICAO:      "YCDH",
		Name:      "Cadney Homestead Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 287,
		Latitude:  -27.9083003998,
		Longitude: 134.0529937744,
		Timezone:  "Australia/Adelaide",
	},
	"YCDL": {
		ICAO:      "YCDL",
		Name:      "Cradle Mountain Airport",
		City:      "Middlesex",
		State:     "Tasmania",
		Country:   "AU",
		Latitude:  -41.581231359,
		Longitude: 145.93834877,
		Timezone:  "Australia/Hobart",
	},
	"YCDO": {
		ICAO:      "YCDO",
		IATA:      "CBX",
		Name:      "Condobolin Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 650,
		Latitude:  -33.0643997192,
		Longitude: 147.2089996338,
		Timezone:  "Australia/Sydney",
	},
	"YCDR": {
		ICAO:      "YCDR",
		IATA:      "CUD",
		Name:      "Caloundra Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 12,
		Latitude:  -26.7999992371,
		Longitude: 153.1000061035,
		Timezone:  "Australia/Brisbane",
	},
	"YCDS": {
		ICAO:      "YCDS",
		Name:      "Childers Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -25.2530800689,
		Longitude: 152.335395813,
		Timezone:  "Australia/Brisbane",
	},
	"YCDT": {
		ICAO:      "YCDT",
		Name:      "Carandotta Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -21.9666996002,
		Longitude: 138.6170043945,
		Timezone:  "Australia/Brisbane",
	},
	"YCDU": {
		ICAO:      "YCDU",
		IATA:      "CED",
		Name:      "Ceduna Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 77,
		Latitude:  -32.1305999756,
		Longitude: 133.7100067139,
		Timezone:  "Australia/Adelaide",
	},
	"YCDV": {
		ICAO:      "YCDV",
		Name:      "Caldervale Station Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -25.1082992554,
		Longitude: 146.8329925537,
		Timezone:  "Australia/Brisbane",
	},
	"YCEE": {
		ICAO:      "YCEE",
		IATA:      "CVC",
		Name:      "Cleve Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 589,
		Latitude:  -33.7097015381,
		Longitude: 136.5050048828,
		Timezone:  "Australia/Adelaide",
	},
	"YCEL": {
		ICAO:      "YCEL",
		Name:      "Capella Airport",
		City:      "Capella",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -23.1000003815,
		Longitude: 148.0330047607,
		Timezone:  "Australia/Brisbane",
	},
	"YCEM": {
		ICAO:      "YCEM",
		Name:      "Coldstream Airport",
		City:      "Coldstream",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 76,
		Latitude:  -37.7276687622,
		Longitude: 145.4083709717,
		Timezone:  "Australia/Melbourne",
	},
	"YCES": {
		ICAO:      "YCES",
		Name:      "Ceres Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -38.1483001709,
		Longitude: 144.2579956055,
		Timezone:  "Australia/Melbourne",
	},
	"YCFD": {
		ICAO:      "YCFD",
		IATA:      "CFI",
		Name:      "Camfield Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -17.0216999054,
		Longitude: 131.3269958496,
		Timezone:  "Australia/Darwin",
	},
	"YCFF": {
		ICAO:      "YCFF",
		Name:      "Dadswells Bridge Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -36.9682998657,
		Longitude: 142.5480041504,
		Timezone:  "Australia/Melbourne",
	},
	"YCFH": {
		ICAO:      "YCFH",
		Name:      "Clifton Hills Airport",
		City:      "Clifton Hills",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -27.0182991028,
		Longitude: 138.891998291,
		Timezone:  "Australia/Adelaide",
	},
	"YCFL": {
		ICAO:      "YCFL",
		Name:      "Cape Flattery Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -14.9666996002,
		Longitude: 145.3000030518,
		Timezone:  "Australia/Brisbane",
	},
	"YCFN": {
		ICAO:      "YCFN",
		Name:      "Clifton Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -27.9283008575,
		Longitude: 151.8470001221,
		Timezone:  "Australia/Brisbane",
	},
	"YCGH": {
		ICAO:      "YCGH",
		Name:      "Clonagh Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -20.1333007813,
		Longitude: 140.6829986572,
		Timezone:  "Australia/Brisbane",
	},
	"YCGI": {
		ICAO:      "YCGI",
		Name:      "Carnegie Station Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -25.7999992371,
		Longitude: 122.9670028687,
		Timezone:  "Australia/Perth",
	},
	"YCGO": {
		ICAO:      "YCGO",
		IATA:      "LLG",
		Name:      "Chillagoe Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 1110,
		Latitude:  -17.1427993774,
		Longitude: 144.529006958,
		Timezone:  "Australia/Brisbane",
	},
	"YCHB": {
		ICAO:      "YCHB",
		Name:      "Cherrabah Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -28.4167003632,
		Longitude: 152.1670074463,
		Timezone:  "Australia/Brisbane",
	},
	"YCHK": {
		ICAO:      "YCHK",
		IATA:      "CKW",
		Name:      "Graeme Rowley Aerodrome",
		City:      "Christmas Creek mine",
		State:     "Western Australia",
		Country:   "AU",
		Latitude:  -22.355556,
		Longitude: 119.652222,
		Timezone:  "Australia/Perth",
	},
	"YCHL": {
		ICAO:      "YCHL",
		Name:      "Charlton Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -36.25,
		Longitude: 143.3000030518,
		Timezone:  "Australia/Melbourne",
	},
	"YCHR": {
		ICAO:      "YCHR",
		Name:      "Childra Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -31.7000007629,
		Longitude: 134.8329925537,
		Timezone:  "Australia/Adelaide",
	},
	"YCHT": {
		ICAO:      "YCHT",
		IATA:      "CXT",
		Name:      "Charters Towers Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 955,
		Latitude:  -20.0431003571,
		Longitude: 146.2729949951,
		Timezone:  "Australia/Brisbane",
	},
	"YCIG": {
		ICAO:      "YCIG",
		Name:      "Corrigin Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -32.3333015442,
		Longitude: 117.8170013428,
		Timezone:  "Australia/Perth",
	},
	"YCIN": {
		ICAO:      "YCIN",
		IATA:      "DCN",
		Name:      "RAAF Base Curtin",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 300,
		Latitude:  -17.5813999176,
		Longitude: 123.82800293,
		Timezone:  "Australia/Perth",
	},
	"YCJU": {
		ICAO:      "YCJU",
		Name:      "Conjuboy Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -18.6833000183,
		Longitude: 144.75,
		Timezone:  "Australia/Brisbane",
	},
	"YCKA": {
		ICAO:      "YCKA",
		Name:      "Curdimurka Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -29.4766998291,
		Longitude: 137.0879974365,
		Timezone:  "Australia/Adelaide",
	},
	"YCKD": {
		ICAO:      "YCKD",
		Name:      "Clarke Island Airport",
		State:     "Tasmania",
		Country:   "AU",
		Latitude:  -40.5167007446,
		Longitude: 148.1329956055,
		Timezone:  "Australia/Hobart",
	},
	"YCKI": {
		ICAO:      "YCKI",
		IATA:      "CKI",
		Name:      "Croker Island Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 16,
		Latitude:  -11.1649999619,
		Longitude: 132.483001709,
		Timezone:  "Australia/Darwin",
	},
	"YCKN": {
		ICAO:      "YCKN",
		IATA:      "CTN",
		Name:      "Cooktown Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 26,
		Latitude:  -15.4447002411,
		Longitude: 145.1840057373,
		Timezone:  "Australia/Brisbane",
	},
	"YCKY": {
		ICAO:      "YCKY",
		Name:      "Cocklebiddy Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -32.0499992371,
		Longitude: 126.0920028687,
		Timezone:  "Australia/Eucla",
	},
	"YCLA": {
		ICAO:      "YCLA",
		Name:      "Clare Station Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 266,
		Latitude:  -33.4068984985,
		Longitude: 143.9400024414,
		Timezone:  "Australia/Sydney",
	},
	"YCLE": {
		ICAO:      "YCLE",
		Name:      "Callendale Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -37.2433013916,
		Longitude: 140.4450073242,
		Timezone:  "Australia/Adelaide",
	},
	"YCLG": {
		ICAO:      "YCLG",
		Name:      "Coolgardie Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -30.9500007629,
		Longitude: 121.1500015259,
		Timezone:  "Australia/Perth",
	},
	"YCLM": {
		ICAO:      "YCLM",
		Name:      "Collymongle Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -29.4533004761,
		Longitude: 148.7429962158,
		Timezone:  "Australia/Sydney",
	},
	"YCLQ": {
		ICAO:      "YCLQ",
		Name:      "Cape Leveque Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -16.3983001709,
		Longitude: 122.9319992065,
		Timezone:  "Australia/Perth",
	},
	"YCLT": {
		ICAO:      "YCLT",
		Name:      "Collector Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -34.8833007813,
		Longitude: 149.4170074463,
		Timezone:  "Australia/Sydney",
	},
	"YCLY": {
		ICAO:      "YCLY",
		Name:      "Coleambally Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -34.7999992371,
		Longitude: 145.8500061035,
		Timezone:  "Australia/Sydney",
	},
	"YCMH": {
		ICAO:      "YCMH",
		Name:      "Camden Haven Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 2,
		Latitude:  -31.6650009155,
		Longitude: 152.7420043945,
		Timezone:  "Australia/Sydney",
	},
	"YCMK": {
		ICAO:      "YCMK",
		Name:      "Camel Creek Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -18.8500003815,
		Longitude: 145.4669952393,
		Timezone:  "Australia/Brisbane",
	},
	"YCMM": {
		ICAO:      "YCMM",
		Name:      "Cummins Town Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -34.2583007813,
		Longitude: 135.7079925537,
		Timezone:  "Australia/Adelaide",
	},
	"YCMT": {
		ICAO:      "YCMT",
		IATA:      "CMQ",
		Name:      "Clermont Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 908,
		Latitude:  -22.7730998993,
		Longitude: 147.6210021973,
		Timezone:  "Australia/Brisbane",
	},
	"YCMU": {
		ICAO:      "YCMU",
		IATA:      "CMA",
		Name:      "Cunnamulla Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 630,
		Latitude:  -28.0300006866,
		Longitude: 145.6219940186,
		Timezone:  "Australia/Brisbane",
	},
	"YCMW": {
		ICAO:      "YCMW",
		IATA:      "CML",
		Name:      "Camooweal Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 241,
		Latitude:  -19.911699295,
		Longitude: 138.125,
		Timezone:  "Australia/Brisbane",
	},
	"YCNA": {
		ICAO:      "YCNA",
		Name:      "Corona Station Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 748,
		Latitude:  -31.2908000946,
		Longitude: 141.4519958496,
		Timezone:  "Australia/Sydney",
	},
	"YCNF": {
		ICAO:      "YCNF",
		IATA:      "NIF",
		Name:      "Camp Nifty Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 295,
		Latitude:  -21.6716995239,
		Longitude: 121.5869979858,
		Timezone:  "Australia/Perth",
	},
	"YCNG": {
		ICAO:      "YCNG",
		Name:      "Cooranga Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -36.8199996948,
		Longitude: 140.2729949951,
		Timezone:  "Australia/Adelaide",
	},
	"YCNH": {
		ICAO:      "YCNH",
		Name:      "Carnarmah Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -29.6833000183,
		Longitude: 115.8830032349,
		Timezone:  "Australia/Perth",
	},
	"YCNK": {
		ICAO:      "YCNK",
		IATA:      "CES",
		Name:      "Cessnock Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 211,
		Latitude:  -32.7874984741,
		Longitude: 151.3419952393,
		Timezone:  "Australia/Sydney",
	},
	"YCNM": {
		ICAO:      "YCNM",
		IATA:      "CNB",
		Name:      "Coonamble Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 604,
		Latitude:  -30.9832992554,
		Longitude: 148.3760070801,
		Timezone:  "Australia/Sydney",
	},
	"YCNN": {
		ICAO:      "YCNN",
		Name:      "Coonana Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -31.0167007446,
		Longitude: 123.1669998169,
		Timezone:  "Australia/Perth",
	},
	"YCNO": {
		ICAO:      "YCNO",
		Name:      "Conargo Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -35.3166999817,
		Longitude: 145.1170043945,
		Timezone:  "Australia/Sydney",
	},
	"YCNQ": {
		ICAO:      "YCNQ",
		Name:      "Coonawarra Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -37.2832984924,
		Longitude: 140.8000030518,
		Timezone:  "Australia/Adelaide",
	},
	"YCNR": {
		ICAO:      "YCNR",
		Name:      "Cann River Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -37.5167007446,
		Longitude: 149.1670074463,
		Timezone:  "Australia/Melbourne",
	},
	"YCNS": {
		ICAO:      "YCNS",
		Name:      "Coniston Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -22.1333007813,
		Longitude: 132.5330047607,
		Timezone:  "Australia/Darwin",
	},
	"YCNX": {
		ICAO:      "YCNX",
		Name:      "Cooranga Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -29.0666999817,
		Longitude: 150.766998291,
		Timezone:  "Australia/Sydney",
	},
	"YCNY": {
		ICAO:      "YCNY",
		Name:      "Century Mine Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 416,
		Latitude:  -18.7532997131,
		Longitude: 138.7070007324,
		Timezone:  "Australia/Brisbane",
	},
	"YCOD": {
		ICAO:      "YCOD",
		IATA:      "ODL",
		Name:      "Cordillo Downs Airport",
		City:      "Cordillo Downs",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 125,
		Latitude:  -26.745300293,
		Longitude: 140.6380004883,
		Timezone:  "Australia/Adelaide",
	},
	"YCOE": {
		ICAO:      "YCOE",
		IATA:      "CUQ",
		Name:      "Coen Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 532,
		Latitude:  -13.7608003616,
		Longitude: 143.1139984131,
		Timezone:  "Australia/Brisbane",
	},
	"YCOF": {
		ICAO:      "YCOF",
		Name:      "Coffin Bay Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -34.6333007813,
		Longitude: 135.516998291,
		Timezone:  "Australia/Adelaide",
	},
	"YCOG": {
		ICAO:      "YCOG",
		Name:      "Coongan Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -20.6833000183,
		Longitude: 119.6669998169,
		Timezone:  "Australia/Perth",
	},
	"YCOH": {
		ICAO:      "YCOH",
		Name:      "Cohuna Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -35.8166999817,
		Longitude: 144.1999969482,
		Timezone:  "Australia/Melbourne",
	},
	"YCOI": {
		ICAO:      "YCOI",
		IATA:      "CIE",
		Name:      "Collie Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -33.3666992188,
		Longitude: 116.2170028687,
		Timezone:  "Australia/Perth",
	},
	"YCOL": {
		ICAO:      "YCOL",
		Name:      "Coleraine Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -37.5999984741,
		Longitude: 141.6999969482,
		Timezone:  "Australia/Melbourne",
	},
	"YCOM": {
		ICAO:      "YCOM",
		IATA:      "OOM",
		Name:      "Cooma Snowy Mountains Airport",
		City:      "Cooma",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 3088,
		Latitude:  -36.3005981445,
		Longitude: 148.973999023,
		Timezone:  "Australia/Sydney",
	},
	"YCOO": {
		ICAO:      "YCOO",
		IATA:      "CDA",
		Name:      "Cooinda Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 13,
		Latitude:  -12.9033002853,
		Longitude: 132.5319976807,
		Timezone:  "Australia/Darwin",
	},
	"YCOR": {
		ICAO:      "YCOR",
		IATA:      "CWW",
		Name:      "Corowa Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 469,
		Latitude:  -35.9947013855,
		Longitude: 146.3569946289,
		Timezone:  "Australia/Melbourne",
	},
	"YCOS": {
		ICAO:      "YCOS",
		Name:      "Cosmo Newberry Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -27.9850006104,
		Longitude: 122.9029998779,
		Timezone:  "Australia/Perth",
	},
	"YCOY": {
		ICAO:      "YCOY",
		Name:      "Coral Bay Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 5,
		Latitude:  -23.1299991608,
		Longitude: 113.7770004272,
		Timezone:  "Australia/Perth",
	},
	"YCPD": {
		ICAO:      "YCPD",
		Name:      "Cape Don Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -11.2982997894,
		Longitude: 131.8269958496,
		Timezone:  "Australia/Darwin",
	},
	"YCPG": {
		ICAO:      "YCPG",
		Name:      "Coppins Gap Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -20.9500007629,
		Longitude: 120.0329971313,
		Timezone:  "Australia/Perth",
	},
	"YCPH": {
		ICAO:      "YCPH",
		Name:      "Copper Hills Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -27.9783000946,
		Longitude: 134.3170013428,
		Timezone:  "Australia/Adelaide",
	},
	"YCPN": {
		ICAO:      "YCPN",
		Name:      "Carpentaria Downs Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -18.7166996002,
		Longitude: 144.3170013428,
		Timezone:  "Australia/Brisbane",
	},
	"YCPP": {
		ICAO:      "YCPP",
		Name:      "Copper Triangle Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -33.9817008972,
		Longitude: 137.6620025635,
		Timezone:  "Australia/Adelaide",
	},
	"YCRA": {
		ICAO:      "YCRA",
		Name:      "Carinda Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 131,
		Latitude:  -30.4582996368,
		Longitude: 147.7079925537,
		Timezone:  "Australia/Sydney",
	},
	"YCRG": {
		ICAO:      "YCRG",
		IATA:      "CYG",
		Name:      "Corryong Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 963,
		Latitude:  -36.182800293,
		Longitude: 147.8880004883,
		Timezone:  "Australia/Melbourne",
	},
	"YCRK": {
		ICAO:      "YCRK",
		IATA:      "CXQ",
		Name:      "Christmas Creek Station Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -18.8833007813,
		Longitude: 125.9169998169,
		Timezone:  "Australia/Perth",
	},
	"YCRL": {
		ICAO:      "YCRL",
		Name:      "Crookwell Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -34.5,
		Longitude: 149.4499969482,
		Timezone:  "Australia/Sydney",
	},
	"YCRN": {
		ICAO:      "YCRN",
		Name:      "Cranbrook Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -34.2832984924,
		Longitude: 117.5500030518,
		Timezone:  "Australia/Perth",
	},
	"YCRT": {
		ICAO:      "YCRT",
		Name:      "Carrieton Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -32.4166984558,
		Longitude: 138.516998291,
		Timezone:  "Australia/Adelaide",
	},
	"YCRY": {
		ICAO:      "YCRY",
		IATA:      "CDQ",
		Name:      "Croydon Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 107,
		Latitude:  -18.2250003815,
		Longitude: 142.2579956055,
		Timezone:  "Australia/Brisbane",
	},
	"YCSI": {
		ICAO:      "YCSI",
		Name:      "Cassilis Rotherw Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 1545,
		Latitude:  -32.0022010803,
		Longitude: 149.9609985352,
		Timezone:  "Australia/Sydney",
	},
	"YCSL": {
		ICAO:      "YCSL",
		Name:      "Consuelo Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -24.6333007813,
		Longitude: 148.4669952393,
		Timezone:  "Australia/Brisbane",
	},
	"YCSP": {
		ICAO:      "YCSP",
		Name:      "Curtin Springs Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -25.3166999817,
		Longitude: 131.75,
		Timezone:  "Australia/Darwin",
	},
	"YCSV": {
		ICAO:      "YCSV",
		IATA:      "KCE",
		Name:      "Collinsville Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 180,
		Latitude:  -20.5967006683,
		Longitude: 147.8600006104,
		Timezone:  "Australia/Brisbane",
	},
	"YCTC": {
		ICAO:      "YCTC",
		Name:      "Cotten Creek Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -22.7999992371,
		Longitude: 122.5830001831,
		Timezone:  "Australia/Perth",
	},
	"YCTH": {
		ICAO:      "YCTH",
		Name:      "Chatsworth Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -21.9666996002,
		Longitude: 140.3000030518,
		Timezone:  "Australia/Brisbane",
	},
	"YCTI": {
		ICAO:      "YCTI",
		Name:      "Cockatoo Island Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -16.0832996368,
		Longitude: 123.6169967651,
		Timezone:  "Australia/Perth",
	},
	"YCTM": {
		ICAO:      "YCTM",
		IATA:      "CMD",
		Name:      "Cootamundra Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 1110,
		Latitude:  -34.6239013672,
		Longitude: 148.0279998779,
		Timezone:  "Australia/Sydney",
	},
	"YCTN": {
		ICAO:      "YCTN",
		Name:      "Casterton Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -37.5999984741,
		Longitude: 141.3999938965,
		Timezone:  "Australia/Melbourne",
	},
	"YCTS": {
		ICAO:      "YCTS",
		Name:      "Calton Hills Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -20.1333007813,
		Longitude: 139.4170074463,
		Timezone:  "Australia/Brisbane",
	},
	"YCTY": {
		ICAO:      "YCTY",
		Name:      "Cape Otway Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -38.8600006104,
		Longitude: 143.5149993896,
		Timezone:  "Australia/Melbourne",
	},
	"YCUA": {
		ICAO:      "YCUA",
		IATA:      "CUG",
		Name:      "Cudal Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 491,
		Latitude:  -33.278301239,
		Longitude: 148.7630004883,
		Timezone:  "Australia/Sydney",
	},
	"YCUD": {
		ICAO:      "YCUD",
		Name:      "Cuddapan Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -25.5900001526,
		Longitude: 141.5079956055,
		Timezone:  "Australia/Brisbane",
	},
	"YCUE": {
		ICAO:      "YCUE",
		IATA:      "CUY",
		Name:      "Cue Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 442,
		Latitude:  -27.4466991425,
		Longitude: 117.9179992676,
		Timezone:  "Australia/Perth",
	},
	"YCUN": {
		ICAO:      "YCUN",
		Name:      "Cunderdin Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 705,
		Latitude:  -31.6222000122,
		Longitude: 117.2170028687,
		Timezone:  "Australia/Perth",
	},
	"YCUR": {
		ICAO:      "YCUR",
		Name:      "Cabramurra Airport",
		City:      "Cabramurra Township",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -35.9268875122,
		Longitude: 148.3925476074,
		Timezone:  "Australia/Sydney",
	},
	"YCUS": {
		ICAO:      "YCUS",
		Name:      "Cummins Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 197,
		Latitude:  -34.2083015442,
		Longitude: 135.641998291,
		Timezone:  "Australia/Adelaide",
	},
	"YCUT": {
		ICAO:      "YCUT",
		Name:      "Cuthero Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -33.0332984924,
		Longitude: 142.016998291,
		Timezone:  "Australia/Sydney",
	},
	"YCVG": {
		ICAO:      "YCVG",
		Name:      "Calvin Grove Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -34.6967010498,
		Longitude: 138.5780029297,
		Timezone:  "Australia/Adelaide",
	},
	"YCVS": {
		ICAO:      "YCVS",
		Name:      "Cervantes Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -30.4832992554,
		Longitude: 115.0830001831,
		Timezone:  "Australia/Perth",
	},
	"YCWA": {
		ICAO:      "YCWA",
		IATA:      "CJF",
		Name:      "Coondewanna Wa Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 2327,
		Latitude:  -22.966667,
		Longitude: 118.802222,
		Timezone:  "Australia/Perth",
	},
	"YCWE": {
		ICAO:      "YCWE",
		Name:      "Cape Wessels Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -11,
		Longitude: 136.75,
		Timezone:  "Australia/Darwin",
	},
	"YCWH": {
		ICAO:      "YCWH",
		Name:      "Commonwealth Hill Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -29.9617004395,
		Longitude: 134.1430053711,
		Timezone:  "Australia/Adelaide",
	},
	"YCWI": {
		ICAO:      "YCWI",
		IATA:      "CWR",
		Name:      "Cowarie Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 52,
		Latitude:  -27.7117004395,
		Longitude: 138.3280029297,
		Timezone:  "Australia/Adelaide",
	},
	"YCWL": {
		ICAO:      "YCWL",
		IATA:      "CCW",
		Name:      "Cowell Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 39,
		Latitude:  -33.6666984558,
		Longitude: 136.891998291,
		Timezone:  "Australia/Adelaide",
	},
	"YCWN": {
		ICAO:      "YCWN",
		Name:      "Corowa Downs Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -26.4666996002,
		Longitude: 143.3500061035,
		Timezone:  "Australia/Brisbane",
	},
	"YCWO": {
		ICAO:      "YCWO",
		Name:      "Chatsworth Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -37.8333015442,
		Longitude: 142.7169952393,
		Timezone:  "Australia/Melbourne",
	},
	"YCWR": {
		ICAO:      "YCWR",
		IATA:      "CWT",
		Name:      "Cowra Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 966,
		Latitude:  -33.8446998596,
		Longitude: 148.6490020752,
		Timezone:  "Australia/Sydney",
	},
	"YCWS": {
		ICAO:      "YCWS",
		Name:      "Cowes Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -38.5083007813,
		Longitude: 145.2129974365,
		Timezone:  "Australia/Melbourne",
	},
	"YCWW": {
		ICAO:      "YCWW",
		Name:      "Canowindra Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 1073,
		Latitude:  -33.5428009033,
		Longitude: 148.6629943848,
		Timezone:  "Australia/Sydney",
	},
	"YCWY": {
		ICAO:      "YCWY",
		IATA:      "COY",
		Name:      "Coolawanyah Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -21.7833003998,
		Longitude: 117.8000030518,
		Timezone:  "Australia/Perth",
	},
	"YCXA": {
		ICAO:      "YCXA",
		Name:      "Cooloola Village Airpark",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 9,
		Latitude:  -25.9750003815,
		Longitude: 153,
		Timezone:  "Australia/Brisbane",
	},
	"YCYT": {
		ICAO:      "YCYT",
		Name:      "Crystal Brook Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -17.3833007813,
		Longitude: 144.4499969482,
		Timezone:  "Australia/Brisbane",
	},
	"YDAG": {
		ICAO:      "YDAG",
		Name:      "Dagworth Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -17.9416999817,
		Longitude: 143.5879974365,
		Timezone:  "Australia/Brisbane",
	},
	"YDAJ": {
		ICAO:      "YDAJ",
		IATA:      "DJR",
		Name:      "Dajarra Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 335,
		Latitude:  -21.7082996368,
		Longitude: 139.5330047607,
		Timezone:  "Australia/Brisbane",
	},
	"YDAL": {
		ICAO:      "YDAL",
		Name:      "Dallas Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -34.5666999817,
		Longitude: 146.1829986572,
		Timezone:  "Australia/Sydney",
	},
	"YDAR": {
		ICAO:      "YDAR",
		Name:      "Darlington Airport",
		State:     "Tasmania",
		Country:   "AU",
		Latitude:  -42.5727408735,
		Longitude: 148.069181442,
		Timezone:  "Australia/Hobart",
	},
	"YDAY": {
		ICAO:      "YDAY",
		IATA:      "DBY",
		Name:      "Dalby Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 1137,
		Latitude:  -27.1553001404,
		Longitude: 151.266998291,
		Timezone:  "Australia/Brisbane",
	},
	"YDBI": {
		ICAO:      "YDBI",
		IATA:      "DRN",
		Name:      "Dirranbandi Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 567,
		Latitude:  -28.5916996002,
		Longitude: 148.2169952393,
		Timezone:  "Australia/Brisbane",
	},
	"YDBR": {
		ICAO:      "YDBR",
		IATA:      "DNB",
		Name:      "Dunbar Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 15,
		Latitude:  -16.0499992371,
		Longitude: 142.3999938965,
		Timezone:  "Australia/Brisbane",
	},
	"YDBY": {
		ICAO:      "YDBY",
		IATA:      "DRB",
		Name:      "Derby Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 24,
		Latitude:  -17.3700008392,
		Longitude: 123.6610031128,
		Timezone:  "Australia/Perth",
	},
	"YDDF": {
		ICAO:      "YDDF",
		IATA:      "DFP",
		Name:      "Drumduff Airport",
		City:      "Drumduff",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -16.0529994965,
		Longitude: 143.011993408,
		Timezone:  "Australia/Brisbane",
	},
	"YDEG": {
		ICAO:      "YDEG",
		Name:      "Delegate Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 2546,
		Latitude:  -37.0614013672,
		Longitude: 148.9420013428,
		Timezone:  "Australia/Sydney",
	},
	"YDEK": {
		ICAO:      "YDEK",
		Name:      "Denmark Airport",
		City:      "Denmark",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 230,
		Latitude:  -34.9449996948,
		Longitude: 117.3970031738,
		Timezone:  "Australia/Perth",
	},
	"YDER": {
		ICAO:      "YDER",
		Name:      "Derrinallum Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 600,
		Latitude:  -37.9341666667,
		Longitude: 143.227777778,
		Timezone:  "Australia/Melbourne",
	},
	"YDEV": {
		ICAO:      "YDEV",
		Name:      "Devoncourt Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -21.2166996002,
		Longitude: 140.233001709,
		Timezone:  "Australia/Brisbane",
	},
	"YDGA": {
		ICAO:      "YDGA",
		IATA:      "DGD",
		Name:      "Dalgaranga Gold Mine Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 1459,
		Latitude:  -27.8302777778,
		Longitude: 117.316388889,
		Timezone:  "Australia/Perth",
	},
	"YDGI": {
		ICAO:      "YDGI",
		Name:      "Dullingari Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -28.1333007813,
		Longitude: 140.8820037842,
		Timezone:  "Australia/Adelaide",
	},
	"YDGN": {
		ICAO:      "YDGN",
		Name:      "Doongan Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -15.3999996185,
		Longitude: 126.3000030518,
		Timezone:  "Australia/Perth",
	},
	"YDGR": {
		ICAO:      "YDGR",
		Name:      "Dalgaranga Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -27.7914403561,
		Longitude: 116.993322372,
		Timezone:  "Australia/Perth",
	},
	"YDGT": {
		ICAO:      "YDGT",
		Name:      "Dalgety Downs Station Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -25.2833003998,
		Longitude: 116.1999969482,
		Timezone:  "Australia/Perth",
	},
	"YDGY": {
		ICAO:      "YDGY",
		Name:      "De Grey Homestead Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -20.1749992371,
		Longitude: 119.1689987183,
		Timezone:  "Australia/Perth",
	},
	"YDHL": {
		ICAO:      "YDHL",
		Name:      "Dhalinbuy Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -12.4132995605,
		Longitude: 136.5370025635,
		Timezone:  "Australia/Darwin",
	},
	"YDIM": {
		ICAO:      "YDIM",
		Name:      "Dimbulah Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -17.1333007813,
		Longitude: 145.0829925537,
		Timezone:  "Australia/Brisbane",
	},
	"YDIX": {
		ICAO:      "YDIX",
		IATA:      "DXD",
		Name:      "Dixie Airport",
		City:      "New Dixie",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 580,
		Latitude:  -15.1174944348,
		Longitude: 143.316049576,
		Timezone:  "Australia/Brisbane",
	},
	"YDKI": {
		ICAO:      "YDKI",
		IATA:      "DKI",
		Name:      "Dunk Island Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 6,
		Latitude:  -17.9416999817,
		Longitude: 146.1399993896,
		Timezone:  "Australia/Brisbane",
	},
	"YDLC": {
		ICAO:      "YDLC",
		Name:      "Dulacca Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -26.6333007813,
		Longitude: 149.7169952393,
		Timezone:  "Australia/Brisbane",
	},
	"YDLD": {
		ICAO:      "YDLD",
		Name:      "Delmore Downs Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -22.4585990906,
		Longitude: 134.8840026855,
		Timezone:  "Australia/Darwin",
	},
	"YDLH": {
		ICAO:      "YDLH",
		Name:      "Dalhousie Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -26.4349994659,
		Longitude: 135.516998291,
		Timezone:  "Australia/Adelaide",
	},
	"YDLK": {
		ICAO:      "YDLK",
		IATA:      "DLK",
		Name:      "Dulkaninna Airport",
		City:      "Dulkaninna",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -29.013299942,
		Longitude: 138.4810028076,
		Timezone:  "Australia/Adelaide",
	},
	"YDLL": {
		ICAO:      "YDLL",
		Name:      "Dunolly Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -36.8499984741,
		Longitude: 143.6999969482,
		Timezone:  "Australia/Melbourne",
	},
	"YDLO": {
		ICAO:      "YDLO",
		Name:      "Darlot Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -27.8833007813,
		Longitude: 121.266998291,
		Timezone:  "Australia/Perth",
	},
	"YDLQ": {
		ICAO:      "YDLQ",
		IATA:      "DNQ",
		Name:      "Deniliquin Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 316,
		Latitude:  -35.5593986511,
		Longitude: 144.945999146,
		Timezone:  "Australia/Sydney",
	},
	"YDLT": {
		ICAO:      "YDLT",
		IATA:      "DDN",
		Name:      "Delta Downs Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 15,
		Latitude:  -16.9916992188,
		Longitude: 141.3170013428,
		Timezone:  "Australia/Brisbane",
	},
	"YDLV": {
		ICAO:      "YDLV",
		IATA:      "DLV",
		Name:      "Delissaville Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 34,
		Latitude:  -12.5500001907,
		Longitude: 130.6849975586,
		Timezone:  "Australia/Darwin",
	},
	"YDLW": {
		ICAO:      "YDLW",
		IATA:      "DYW",
		Name:      "Daly Waters Airport",
		City:      "Daly Waters",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 700,
		Latitude:  -16.2646950664,
		Longitude: 133.383378983,
		Timezone:  "Australia/Darwin",
	},
	"YDME": {
		ICAO:      "YDME",
		Name:      "Mount Dimer Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -30.4016990662,
		Longitude: 119.8300018311,
		Timezone:  "Australia/Perth",
	},
	"YDMF": {
		ICAO:      "YDMF",
		Name:      "Diemal Find Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -29.6667003632,
		Longitude: 119.2829971313,
		Timezone:  "Australia/Perth",
	},
	"YDMG": {
		ICAO:      "YDMG",
		IATA:      "DMD",
		Name:      "Doomadgee Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 153,
		Latitude:  -17.9402999878,
		Longitude: 138.8220062256,
		Timezone:  "Australia/Brisbane",
	},
	"YDMN": {
		ICAO:      "YDMN",
		IATA:      "DVR",
		Name:      "Daly River Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -13.7498064041,
		Longitude: 130.6938781738,
		Timezone:  "Australia/Darwin",
	},
	"YDMR": {
		ICAO:      "YDMR",
		Name:      "Delamere Station Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -15.6199998856,
		Longitude: 131.6369934082,
		Timezone:  "Australia/Darwin",
	},
	"YDNB": {
		ICAO:      "YDNB",
		Name:      "Doongmabulla Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -22.0666999817,
		Longitude: 146.233001709,
		Timezone:  "Australia/Brisbane",
	},
	"YDNI": {
		ICAO:      "YDNI",
		IATA:      "NLF",
		Name:      "Darnley Island Airport",
		City:      "Darnley Island",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -9.5833301544,
		Longitude: 143.766998291,
		Timezone:  "Australia/Brisbane",
	},
	"YDNK": {
		ICAO:      "YDNK",
		Name:      "Darnick Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -32.8833007813,
		Longitude: 143.5829925537,
		Timezone:  "Australia/Sydney",
	},
	"YDNR": {
		ICAO:      "YDNR",
		Name:      "Dunmore Manila Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 1391,
		Latitude:  -30.6581001282,
		Longitude: 150.875,
		Timezone:  "Australia/Sydney",
	},
	"YDNV": {
		ICAO:      "YDNV",
		Name:      "Dynevor Downs Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -28.1000003815,
		Longitude: 144.3500061035,
		Timezone:  "Australia/Brisbane",
	},
	"YDOC": {
		ICAO:      "YDOC",
		Name:      "Dochra Airfield",
		City:      "Singleton",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 185,
		Latitude:  -32.6500015259,
		Longitude: 151.207992554,
		Timezone:  "Australia/Sydney",
	},
	"YDOD": {
		ICAO:      "YDOD",
		Name:      "Donald Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 409,
		Latitude:  -36.3602981567,
		Longitude: 143.0079956055,
		Timezone:  "Australia/Melbourne",
	},
	"YDON": {
		ICAO:      "YDON",
		Name:      "Dowerin Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -31.1833000183,
		Longitude: 117.0500030518,
		Timezone:  "Australia/Perth",
	},
	"YDOO": {
		ICAO:      "YDOO",
		Name:      "Donors Hill Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -18.7166996002,
		Longitude: 140.5500030518,
		Timezone:  "Australia/Brisbane",
	},
	"YDOP": {
		ICAO:      "YDOP",
		Name:      "Donnington Airpark",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 76,
		Latitude:  -19.611700058,
		Longitude: 146.8399963379,
		Timezone:  "Australia/Brisbane",
	},
	"YDOR": {
		ICAO:      "YDOR",
		IATA:      "DRD",
		Name:      "Dorunda Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 27,
		Latitude:  -16.5583000183,
		Longitude: 141.8079986572,
		Timezone:  "Australia/Brisbane",
	},
	"YDPD": {
		ICAO:      "YDPD",
		IATA:      "DVP",
		Name:      "Davenport Downs Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 95,
		Latitude:  -24.1499996185,
		Longitude: 141.108001709,
		Timezone:  "Australia/Brisbane",
	},
	"YDPO": {
		ICAO:      "YDPO",
		IATA:      "DPO",
		Name:      "Devonport Airport",
		City:      "Devonport",
		State:     "Tasmania",
		Country:   "AU",
		Elevation: 33,
		Latitude:  -41.1697006226,
		Longitude: 146.429992676,
		Timezone:  "Australia/Hobart",
	},
	"YDPR": {
		ICAO:      "YDPR",
		Name:      "Dneiper Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -22.6166992188,
		Longitude: 135.1999969482,
		Timezone:  "Australia/Darwin",
	},
	"YDPS": {
		ICAO:      "YDPS",
		Name:      "Depot Springs Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -27.8833007813,
		Longitude: 120.0830001831,
		Timezone:  "Australia/Perth",
	},
	"YDPT": {
		ICAO:      "YDPT",
		Name:      "Depot Outcamp Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -31.5722007751,
		Longitude: 125.18699646,
		Timezone:  "Australia/Perth",
	},
	"YDPW": {
		ICAO:      "YDPW",
		Name:      "Deep Well Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -24.2999992371,
		Longitude: 134.1329956055,
		Timezone:  "Australia/Darwin",
	},
	"YDRA": {
		ICAO:      "YDRA",
		IATA:      "DOX",
		Name:      "Dongara Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -29.2999992371,
		Longitude: 114.932998657,
		Timezone:  "Australia/Perth",
	},
	"YDRC": {
		ICAO:      "YDRC",
		Name:      "Dairy Creek Homestead Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -25.2833003998,
		Longitude: 115.8669967651,
		Timezone:  "Australia/Perth",
	},
	"YDRD": {
		ICAO:      "YDRD",
		IATA:      "DRY",
		Name:      "Drysdale River Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 360,
		Latitude:  -15.7135703992,
		Longitude: 126.38109684,
		Timezone:  "Australia/Perth",
	},
	"YDRH": {
		ICAO:      "YDRH",
		IATA:      "DHD",
		Name:      "Durham Downs Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 116,
		Latitude:  -27.0750007629,
		Longitude: 141.8999938965,
		Timezone:  "Australia/Brisbane",
	},
	"YDRI": {
		ICAO:      "YDRI",
		IATA:      "DRR",
		Name:      "Durrie Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 58,
		Latitude:  -25.6849994659,
		Longitude: 140.2279968262,
		Timezone:  "Australia/Brisbane",
	},
	"YDRL": {
		ICAO:      "YDRL",
		Name:      "De Rose Hill Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -26.4333000183,
		Longitude: 133.2579956055,
		Timezone:  "Australia/Adelaide",
	},
	"YDUK": {
		ICAO:      "YDUK",
		Name:      "Duketon Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -27.9582996368,
		Longitude: 122.3079986572,
		Timezone:  "Australia/Perth",
	},
	"YDUM": {
		ICAO:      "YDUM",
		Name:      "Dumbleyung Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -33.3333015442,
		Longitude: 117.75,
		Timezone:  "Australia/Perth",
	},
	"YDUN": {
		ICAO:      "YDUN",
		Name:      "Dunwich Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 76,
		Latitude:  -27.5167007446,
		Longitude: 153.4279937744,
		Timezone:  "Australia/Brisbane",
	},
	"YDVE": {
		ICAO:      "YDVE",
		Name:      "Dale River Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -32.2832984924,
		Longitude: 116.5500030518,
		Timezone:  "Australia/Perth",
	},
	"YDVR": {
		ICAO:      "YDVR",
		IATA:      "DKV",
		Name:      "Docker River Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 589,
		Latitude:  -24.8600006104,
		Longitude: 129.0700073242,
		Timezone:  "Australia/Darwin",
	},
	"YDWF": {
		ICAO:      "YDWF",
		Name:      "Delamere Range Facility Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 223,
		Latitude:  -15.7467002869,
		Longitude: 131.9199981689,
		Timezone:  "Australia/Darwin",
	},
	"YDWU": {
		ICAO:      "YDWU",
		Name:      "Dalwallinu Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -30.2000007629,
		Longitude: 116.6500015259,
		Timezone:  "Australia/Perth",
	},
	"YDYD": {
		ICAO:      "YDYD",
		Name:      "Derry Downs Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -22.0832996368,
		Longitude: 135.3329925537,
		Timezone:  "Australia/Darwin",
	},
	"YDYS": {
		ICAO:      "YDYS",
		IATA:      "DYA",
		Name:      "Dysart Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 670,
		Latitude:  -22.6222000122,
		Longitude: 148.3639984131,
		Timezone:  "Australia/Brisbane",
	},
	"YEAB": {
		ICAO:      "YEAB",
		Name:      "Euabalong Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -33.1166992188,
		Longitude: 146.4499969482,
		Timezone:  "Australia/Sydney",
	},
	"YEAT": {
		ICAO:      "YEAT",
		Name:      "Yea Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -37.215801239,
		Longitude: 146.4299926758,
		Timezone:  "Australia/Melbourne",
	},
	"YECH": {
		ICAO:      "YECH",
		IATA:      "ECH",
		Name:      "Echuca Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 323,
		Latitude:  -36.1571998596,
		Longitude: 144.7619934082,
		Timezone:  "Australia/Melbourne",
	},
	"YECL": {
		ICAO:      "YECL",
		IATA:      "EUC",
		Name:      "Eucla Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -31.7000007629,
		Longitude: 128.8829956055,
		Timezone:  "Australia/Eucla",
	},
	"YEDA": {
		ICAO:      "YEDA",
		IATA:      "ETD",
		Name:      "Etadunna Airport",
		City:      "Etadunna",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -28.7408008575,
		Longitude: 138.5890045166,
		Timezone:  "Australia/Adelaide",
	},
	"YEDE": {
		ICAO:      "YEDE",
		Name:      "Edenhope Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -37.0167007446,
		Longitude: 141.266998291,
		Timezone:  "Australia/Melbourne",
	},
	"YEDM": {
		ICAO:      "YEDM",
		Name:      "Edmund Station Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -23.75,
		Longitude: 116.0999984741,
		Timezone:  "Australia/Perth",
	},
	"YEEB": {
		ICAO:      "YEEB",
		IATA:      "ENB",
		Name:      "Eneabba Airport",
		City:      "Eneabba",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -29.8325004578,
		Longitude: 115.2460021973,
		Timezone:  "Australia/Perth",
	},
	"YEES": {
		ICAO:      "YEES",
		Name:      "Elderslie Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 112,
		Latitude:  -32.5997009277,
		Longitude: 151.3399963379,
		Timezone:  "Australia/Sydney",
	},
	"YEGA": {
		ICAO:      "YEGA",
		Name:      "Engonnia Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -29.3166999817,
		Longitude: 145.8329925537,
		Timezone:  "Australia/Sydney",
	},
	"YEIN": {
		ICAO:      "YEIN",
		IATA:      "EIH",
		Name:      "Einasleigh Airport",
		City:      "Einasleigh",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -18.5032997131,
		Longitude: 144.093994141,
		Timezone:  "Australia/Brisbane",
	},
	"YELD": {
		ICAO:      "YELD",
		IATA:      "ELC",
		Name:      "Elcho Island Airport",
		City:      "Elcho Island",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 101,
		Latitude:  -12.0193996429,
		Longitude: 135.570999146,
		Timezone:  "Australia/Darwin",
	},
	"YELE": {
		ICAO:      "YELE",
		Name:      "Belele Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -26.3666992188,
		Longitude: 118.016998291,
		Timezone:  "Australia/Perth",
	},
	"YELG": {
		ICAO:      "YELG",
		Name:      "Elengerah Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -31.8500003815,
		Longitude: 147.983001709,
		Timezone:  "Australia/Sydney",
	},
	"YELK": {
		ICAO:      "YELK",
		Name:      "Elkedra Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -21.1667003632,
		Longitude: 135.4329986572,
		Timezone:  "Australia/Darwin",
	},
	"YELL": {
		ICAO:      "YELL",
		Name:      "Elliott Airport",
		City:      "Elliott",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -17.5274295807,
		Longitude: 133.5298919678,
		Timezone:  "Australia/Darwin",
	},
	"YELM": {
		ICAO:      "YELM",
		Name:      "Elmore Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -36.4901793786,
		Longitude: 144.647197723,
		Timezone:  "Australia/Melbourne",
	},
	"YELN": {
		ICAO:      "YELN",
		Name:      "Elliston Airport",
		City:      "http://www.elliston.sa.gov.au",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 20,
		Latitude:  -33.6383018494,
		Longitude: 134.8999938965,
		Timezone:  "Australia/Adelaide",
	},
	"YELR": {
		ICAO:      "YELR",
		Name:      "Elderslie Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -32.6166992188,
		Longitude: 151.3329925537,
		Timezone:  "Australia/Sydney",
	},
	"YELS": {
		ICAO:      "YELS",
		Name:      "Earlston Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -36.5962867737,
		Longitude: 145.721862793,
		Timezone:  "Australia/Melbourne",
	},
	"YELW": {
		ICAO:      "YELW",
		Name:      "Eildon Weir Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -37.2083015442,
		Longitude: 145.8329925537,
		Timezone:  "Australia/Melbourne",
	},
	"YEMG": {
		ICAO:      "YEMG",
		Name:      "Eromanga Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 166,
		Latitude:  -26.7000007629,
		Longitude: 143.266998291,
		Timezone:  "Australia/Brisbane",
	},
	"YEMJ": {
		ICAO:      "YEMJ",
		Name:      "Emu Junction Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -28.6333007813,
		Longitude: 132.1829986572,
		Timezone:  "Australia/Adelaide",
	},
	"YEML": {
		ICAO:      "YEML",
		IATA:      "EMD",
		Name:      "Emerald Airport",
		City:      "Emerald",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 624,
		Latitude:  -23.5674991608,
		Longitude: 148.179000854,
		Timezone:  "Australia/Brisbane",
	},
	"YEMP": {
		ICAO:      "YEMP",
		Name:      "Emu Park Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -23.2567005157,
		Longitude: 150.81300354,
		Timezone:  "Australia/Brisbane",
	},
	"YEPL": {
		ICAO:      "YEPL",
		Name:      "Epsilon Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -28.2833003998,
		Longitude: 141.1999969482,
		Timezone:  "Australia/Brisbane",
	},
	"YEPR": {
		ICAO:      "YEPR",
		Name:      "Epenarra Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -20.4333000183,
		Longitude: 135.266998291,
		Timezone:  "Australia/Darwin",
	},
	"YEQO": {
		ICAO:      "YEQO",
		Name:      "El Questro Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -16.0083007813,
		Longitude: 127.9749984741,
		Timezone:  "Australia/Perth",
	},
	"YERA": {
		ICAO:      "YERA",
		Name:      "Errabiddy Homestead Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -25.5,
		Longitude: 117.1669998169,
		Timezone:  "Australia/Perth",
	},
	"YERH": {
		ICAO:      "YERH",
		Name:      "Earaheedy Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -25.6000003815,
		Longitude: 121.5830001831,
		Timezone:  "Australia/Perth",
	},
	"YERL": {
		ICAO:      "YERL",
		Name:      "Erldunda Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -25.2166996002,
		Longitude: 133.25,
		Timezone:  "Australia/Darwin",
	},
	"YERN": {
		ICAO:      "YERN",
		IATA:      "ERB",
		Name:      "Ernabella Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 707,
		Latitude:  -26.263299942,
		Longitude: 132.1820068359,
		Timezone:  "Australia/Adelaide",
	},
	"YERO": {
		ICAO:      "YERO",
		Name:      "Erong Station Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -25.5666999817,
		Longitude: 116.6669998169,
		Timezone:  "Australia/Perth",
	},
	"YERU": {
		ICAO:      "YERU",
		Name:      "Erudina Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -31.4344997406,
		Longitude: 139.4250030518,
		Timezone:  "Australia/Adelaide",
	},
	"YESC": {
		ICAO:      "YESC",
		Name:      "Escott Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 15,
		Latitude:  -17.7250003815,
		Longitude: 139.4170074463,
		Timezone:  "Australia/Brisbane",
	},
	"YESP": {
		ICAO:      "YESP",
		IATA:      "EPR",
		Name:      "Esperance Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 470,
		Latitude:  -33.6843986511,
		Longitude: 121.8229980469,
		Timezone:  "Australia/Perth",
	},
	"YEUA": {
		ICAO:      "YEUA",
		Name:      "Euroa Aerodrome",
		City:      "Euroa",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 555,
		Latitude:  -36.7447013855,
		Longitude: 145.513000488,
		Timezone:  "Australia/Melbourne",
	},
	"YEUD": {
		ICAO:      "YEUD",
		Name:      "Eudamulla Station Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -24.4500007629,
		Longitude: 115.5999984741,
		Timezone:  "Australia/Perth",
	},
	"YEUL": {
		ICAO:      "YEUL",
		Name:      "Eulalia Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -31.0333003998,
		Longitude: 148.1750030518,
		Timezone:  "Australia/Sydney",
	},
	"YEUO": {
		ICAO:      "YEUO",
		Name:      "Eulo Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 152,
		Latitude:  -28.1667003632,
		Longitude: 145.0420074463,
		Timezone:  "Australia/Brisbane",
	},
	"YEVA": {
		ICAO:      "YEVA",
		IATA:      "EVD",
		Name:      "Eva Downs Airport",
		City:      "Eva Downs",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -18.0009994507,
		Longitude: 134.863006592,
		Timezone:  "Australia/Darwin",
	},
	"YEVD": {
		ICAO:      "YEVD",
		IATA:      "EVH",
		Name:      "Evans Head Aerodrome",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 20,
		Latitude:  -29.0932998657,
		Longitude: 153.4199981689,
		Timezone:  "Australia/Sydney",
	},
	"YEVP": {
		ICAO:      "YEVP",
		Name:      "Everard Park Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -27.0167007446,
		Longitude: 132.7169952393,
		Timezone:  "Australia/Adelaide",
	},
	"YEXM": {
		ICAO:      "YEXM",
		IATA:      "EXM",
		Name:      "Exmouth Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -22.0333003998,
		Longitude: 114.0999984741,
		Timezone:  "Australia/Perth",
	},
	"YEYD": {
		ICAO:      "YEYD",
		Name:      "Evelyn Downs Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -28.20470047,
		Longitude: 134.4880065918,
		Timezone:  "Australia/Adelaide",
	},
	"YFBS": {
		ICAO:      "YFBS",
		IATA:      "FRB",
		Name:      "Forbes Airport",
		City:      "Forbes",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 760,
		Latitude:  -33.3636016846,
		Longitude: 147.934997559,
		Timezone:  "Australia/Sydney",
	},
	"YFCK": {
		ICAO:      "YFCK",
		Name:      "Fish Creek Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -38.7150001526,
		Longitude: 146.0529937744,
		Timezone:  "Australia/Melbourne",
	},
	"YFCU": {
		ICAO:      "YFCU",
		Name:      "Fortescue River Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -21.2999992371,
		Longitude: 116.1330032349,
		Timezone:  "Australia/Perth",
	},
	"YFDF": {
		ICAO:      "YFDF",
		IATA:      "KFE",
		Name:      "Fortescue - Dave Forrest Aerodrome",
		City:      "Cloudbreak Village",
		State:     "Central-Sulawesi",
		Country:   "AU",
		Elevation: 1555,
		Longitude: 119.429161111,
		Timezone:  "Asia/Makassar",
	},
	"YFDN": {
		ICAO:      "YFDN",
		Name:      "Federation Hsd Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 709,
		Latitude:  -35.0116996765,
		Longitude: 147.3739929199,
		Timezone:  "Australia/Sydney",
	},
	"YFFT": {
		ICAO:      "YFFT",
		Name:      "Farrell Flat Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -33.8333015442,
		Longitude: 138.766998291,
		Timezone:  "Australia/Adelaide",
	},
	"YFIL": {
		ICAO:      "YFIL",
		IATA:      "FLY",
		Name:      "Finley Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -35.6666984558,
		Longitude: 145.550003052,
		Timezone:  "Australia/Sydney",
	},
	"YFLI": {
		ICAO:      "YFLI",
		IATA:      "FLS",
		Name:      "Flinders Island Airport",
		State:     "Tasmania",
		Country:   "AU",
		Elevation: 10,
		Latitude:  -40.0917015076,
		Longitude: 147.9929962158,
		Timezone:  "Australia/Hobart",
	},
	"YFLO": {
		ICAO:      "YFLO",
		IATA:      "FVL",
		Name:      "Flora Valley Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -18.2833003998,
		Longitude: 128.417007446,
		Timezone:  "Australia/Perth",
	},
	"YFLS": {
		ICAO:      "YFLS",
		Name:      "Flinders Island Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -33.7000007629,
		Longitude: 134.516998291,
		Timezone:  "Australia/Adelaide",
	},
	"YFNE": {
		ICAO:      "YFNE",
		IATA:      "FIK",
		Name:      "Finke Airport",
		City:      "Finke",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -25.5946998596,
		Longitude: 134.582992554,
		Timezone:  "Australia/Darwin",
	},
	"YFOR": {
		ICAO:      "YFOR",
		Name:      "Fortnum Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -25.3332996368,
		Longitude: 118.3669967651,
		Timezone:  "Australia/Perth",
	},
	"YFRD": {
		ICAO:      "YFRD",
		Name:      "Frome Downs Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -31.2166996002,
		Longitude: 139.75,
		Timezone:  "Australia/Adelaide",
	},
	"YFRG": {
		ICAO:      "YFRG",
		Name:      "Fregon Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 528,
		Latitude:  -26.7749996185,
		Longitude: 132.016998291,
		Timezone:  "Australia/Adelaide",
	},
	"YFRI": {
		ICAO:      "YFRI",
		Name:      "Friendly Beaches Airport",
		State:     "Tasmania",
		Country:   "AU",
		Latitude:  -42,
		Longitude: 148.2590026855,
		Timezone:  "Australia/Hobart",
	},
	"YFRK": {
		ICAO:      "YFRK",
		Name:      "Frankland Airport",
		City:      "Frankland River Grazing co",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 800,
		Latitude:  -34.4230003357,
		Longitude: 117.0260009766,
		Timezone:  "Australia/Perth",
	},
	"YFRT": {
		ICAO:      "YFRT",
		IATA:      "FOS",
		Name:      "Forrest Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 511,
		Latitude:  -30.8381004333,
		Longitude: 128.1150054932,
		Timezone:  "Australia/Perth",
	},
	"YFSA": {
		ICAO:      "YFSA",
		Name:      "Forsayth Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -18.5848007202,
		Longitude: 143.5690002441,
		Timezone:  "Australia/Brisbane",
	},
	"YFSK": {
		ICAO:      "YFSK",
		Name:      "Fiskville Airport",
		City:      "Fiskville",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -37.6781143165,
		Longitude: 144.220790863,
		Timezone:  "Australia/Melbourne",
	},
	"YFST": {
		ICAO:      "YFST",
		IATA:      "FOT",
		Name:      "Forster (Wallis Is) Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 4,
		Latitude:  -32.2042007446,
		Longitude: 152.4790039063,
		Timezone:  "Australia/Sydney",
	},
	"YFTA": {
		ICAO:      "YFTA",
		Name:      "Forrestania Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -32.5800018311,
		Longitude: 119.7030029297,
		Timezone:  "Australia/Perth",
	},
	"YFTN": {
		ICAO:      "YFTN",
		Name:      "Mount Fitton Talc Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -29.9092006683,
		Longitude: 139.4739990234,
		Timezone:  "Australia/Adelaide",
	},
	"YFTZ": {
		ICAO:      "YFTZ",
		IATA:      "FIZ",
		Name:      "Fitzroy Crossing Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 368,
		Latitude:  -18.1819000244,
		Longitude: 125.5589981079,
		Timezone:  "Australia/Perth",
	},
	"YFVW": {
		ICAO:      "YFVW",
		Name:      "Fairview Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -32.7890014648,
		Longitude: 148.7200012207,
		Timezone:  "Australia/Sydney",
	},
	"YFWY": {
		ICAO:      "YFWY",
		Name:      "Faraway Bay Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -13.980799675,
		Longitude: 127.1809997559,
		Timezone:  "Australia/Perth",
	},
	"YGAD": {
		ICAO:      "YGAD",
		Name:      "Garden Island (Military) Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 9,
		Latitude:  -32.2416992188,
		Longitude: 115.6829986572,
		Timezone:  "Australia/Perth",
	},
	"YGAH": {
		ICAO:      "YGAH",
		Name:      "Greenbah Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -30.1258277893,
		Longitude: 149.4345703125,
		Timezone:  "Australia/Sydney",
	},
	"YGAM": {
		ICAO:      "YGAM",
		IATA:      "GBP",
		Name:      "Gamboola Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -16.5499992371,
		Longitude: 143.6670074463,
		Timezone:  "Australia/Brisbane",
	},
	"YGAN": {
		ICAO:      "YGAN",
		Name:      "Gan Gan Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -13.0666999817,
		Longitude: 135.9499969482,
		Timezone:  "Australia/Darwin",
	},
	"YGAR": {
		ICAO:      "YGAR",
		Name:      "Gnaraloo Station Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 10,
		Latitude:  -23.7915496826,
		Longitude: 113.5289382935,
		Timezone:  "Australia/Perth",
	},
	"YGAW": {
		ICAO:      "YGAW",
		Name:      "Gawler Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -34.6016998291,
		Longitude: 138.7169952393,
		Timezone:  "Australia/Adelaide",
	},
	"YGAY": {
		ICAO:      "YGAY",
		IATA:      "GAH",
		Name:      "Gayndah Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 369,
		Latitude:  -25.6144008636,
		Longitude: 151.6190032959,
		Timezone:  "Australia/Brisbane",
	},
	"YGBI": {
		ICAO:      "YGBI",
		IATA:      "GBL",
		Name:      "South Goulburn Is Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 19,
		Latitude:  -11.6499996185,
		Longitude: 133.3820037842,
		Timezone:  "Australia/Darwin",
	},
	"YGBO": {
		ICAO:      "YGBO",
		Name:      "Gabo Island Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -37.5666999817,
		Longitude: 149.8999938965,
		Timezone:  "Australia/Melbourne",
	},
	"YGBW": {
		ICAO:      "YGBW",
		Name:      "Gunbower Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -35.9667015076,
		Longitude: 144.3829956055,
		Timezone:  "Australia/Melbourne",
	},
	"YGCR": {
		ICAO:      "YGCR",
		Name:      "Gloucester Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -32.0499992371,
		Longitude: 151.983001709,
		Timezone:  "Australia/Sydney",
	},
	"YGDA": {
		ICAO:      "YGDA",
		Name:      "Goodooga Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 459,
		Latitude:  -29.073299408,
		Longitude: 147.3760070801,
		Timezone:  "Australia/Sydney",
	},
	"YGDH": {
		ICAO:      "YGDH",
		IATA:      "GUH",
		Name:      "Gunnedah Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 863,
		Latitude:  -30.9610996246,
		Longitude: 150.2510070801,
		Timezone:  "Australia/Sydney",
	},
	"YGDI": {
		ICAO:      "YGDI",
		IATA:      "GOO",
		Name:      "Goondiwindi Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 714,
		Latitude:  -28.5214004517,
		Longitude: 150.3200073242,
		Timezone:  "Australia/Brisbane",
	},
	"YGDN": {
		ICAO:      "YGDN",
		IATA:      "GDD",
		Name:      "Gordon Downs Airport",
		City:      "Gordon Downs",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 800,
		Latitude:  -18.6781005859,
		Longitude: 128.5919952393,
		Timezone:  "Australia/Perth",
	},
	"YGDR": {
		ICAO:      "YGDR",
		Name:      "The Garden Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -23.286699295,
		Longitude: 134.4400024414,
		Timezone:  "Australia/Darwin",
	},
	"YGDS": {
		ICAO:      "YGDS",
		IATA:      "GGD",
		Name:      "Gregory Downs Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 52,
		Latitude:  -18.625,
		Longitude: 139.233001709,
		Timezone:  "Australia/Brisbane",
	},
	"YGDV": {
		ICAO:      "YGDV",
		Name:      "Galdeville Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -20.25,
		Longitude: 142.4499969482,
		Timezone:  "Australia/Brisbane",
	},
	"YGDW": {
		ICAO:      "YGDW",
		IATA:      "GTS",
		Name:      "Granite Downs Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 337,
		Latitude:  -26.948299408,
		Longitude: 133.6069946289,
		Timezone:  "Australia/Adelaide",
	},
	"YGEL": {
		ICAO:      "YGEL",
		IATA:      "GET",
		Name:      "Geraldton Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 121,
		Latitude:  -28.7961006165,
		Longitude: 114.7070007324,
		Timezone:  "Australia/Perth",
	},
	"YGFN": {
		ICAO:      "YGFN",
		IATA:      "GFN",
		Name:      "Grafton Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 110,
		Latitude:  -29.7593994141,
		Longitude: 153.0299987793,
		Timezone:  "Australia/Sydney",
	},
	"YGGE": {
		ICAO:      "YGGE",
		Name:      "Golden Grove Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -28.75,
		Longitude: 116.9499969482,
		Timezone:  "Australia/Perth",
	},
	"YGGI": {
		ICAO:      "YGGI",
		Name:      "Goolgowi Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -34,
		Longitude: 145.7169952393,
		Timezone:  "Australia/Sydney",
	},
	"YGGL": {
		ICAO:      "YGGL",
		Name:      "Glen Garland Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -14.8667001724,
		Longitude: 143.2830047607,
		Timezone:  "Australia/Brisbane",
	},
	"YGGO": {
		ICAO:      "YGGO",
		Name:      "Goonoo Goonoo Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -31.3166999817,
		Longitude: 150.9170074463,
		Timezone:  "Australia/Sydney",
	},
	"YGGS": {
		ICAO:      "YGGS",
		Name:      "Gregory Springs Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -19.7166996002,
		Longitude: 144.3829956055,
		Timezone:  "Australia/Brisbane",
	},
	"YGHG": {
		ICAO:      "YGHG",
		Name:      "Glen Harding Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -18.2667007446,
		Longitude: 145.1329956055,
		Timezone:  "Australia/Brisbane",
	},
	"YGIA": {
		ICAO:      "YGIA",
		IATA:      "GBW",
		Name:      "Ginbata",
		City:      "Ginbata",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 509,
		Latitude:  -22.578666,
		Longitude: 120.039409,
		Timezone:  "Australia/Perth",
	},
	"YGIB": {
		ICAO:      "YGIB",
		IATA:      "GBV",
		Name:      "Gibb River Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 509,
		Latitude:  -16.4232997894,
		Longitude: 126.4449996948,
		Timezone:  "Australia/Perth",
	},
	"YGID": {
		ICAO:      "YGID",
		Name:      "Gidgee Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -27.2700004578,
		Longitude: 119.4049987793,
		Timezone:  "Australia/Perth",
	},
	"YGIF": {
		ICAO:      "YGIF",
		Name:      "Gifford Creek Station Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -24.0499992371,
		Longitude: 116.2170028687,
		Timezone:  "Australia/Perth",
	},
	"YGIG": {
		ICAO:      "YGIG",
		Name:      "RAAF Gingin",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 247,
		Latitude:  -31.4652996063,
		Longitude: 115.8629989624,
		Timezone:  "Australia/Perth",
	},
	"YGIL": {
		ICAO:      "YGIL",
		Name:      "Gilgandra Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 320,
		Latitude:  -31.7033004761,
		Longitude: 148.6380004883,
		Timezone:  "Australia/Sydney",
	},
	"YGIR": {
		ICAO:      "YGIR",
		Name:      "Giralia Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -22.6833000183,
		Longitude: 114.3669967651,
		Timezone:  "Australia/Perth",
	},
	"YGKL": {
		ICAO:      "YGKL",
		IATA:      "GKL",
		Name:      "Great Keppel Is Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 21,
		Latitude:  -23.1833000183,
		Longitude: 150.9420013428,
		Timezone:  "Australia/Brisbane",
	},
	"YGLA": {
		ICAO:      "YGLA",
		IATA:      "GLT",
		Name:      "Gladstone Airport",
		City:      "Gladstone",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 64,
		Latitude:  -23.8696994781,
		Longitude: 151.223007202,
		Timezone:  "Australia/Brisbane",
	},
	"YGLB": {
		ICAO:      "YGLB",
		IATA:      "GUL",
		Name:      "Goulburn Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 2141,
		Latitude:  -34.8102989197,
		Longitude: 149.7259979248,
		Timezone:  "Australia/Sydney",
	},
	"YGLD": {
		ICAO:      "YGLD",
		Name:      "Glendambo Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -30.9666996002,
		Longitude: 135.7749938965,
		Timezone:  "Australia/Adelaide",
	},
	"YGLE": {
		ICAO:      "YGLE",
		IATA:      "GLG",
		Name:      "Glengyle Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 91,
		Latitude:  -24.8083000183,
		Longitude: 139.6000061035,
		Timezone:  "Australia/Brisbane",
	},
	"YGLG": {
		ICAO:      "YGLG",
		IATA:      "GEX",
		Name:      "Geelong Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 43,
		Latitude:  -38.2249984741,
		Longitude: 144.3329925537,
		Timezone:  "Australia/Melbourne",
	},
	"YGLI": {
		ICAO:      "YGLI",
		IATA:      "GLI",
		Name:      "Glen Innes Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 3433,
		Latitude:  -29.6749992371,
		Longitude: 151.6889953613,
		Timezone:  "Australia/Sydney",
	},
	"YGLN": {
		ICAO:      "YGLN",
		Name:      "Glencoe Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -31.6166992188,
		Longitude: 148.1999969482,
		Timezone:  "Australia/Sydney",
	},
	"YGLO": {
		ICAO:      "YGLO",
		IATA:      "GLM",
		Name:      "Glenormiston Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 158,
		Latitude:  -22.888299942,
		Longitude: 138.8249969482,
		Timezone:  "Australia/Brisbane",
	},
	"YGLP": {
		ICAO:      "YGLP",
		Name:      "Gallipolli Airport",
		City:      "Nicholson",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 865,
		Latitude:  -19.14181,
		Longitude: 137.87412,
		Timezone:  "Australia/Darwin",
	},
	"YGLS": {
		ICAO:      "YGLS",
		Name:      "Giles Airport",
		City:      "Warakurna",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 579,
		Latitude:  -25.0436992645,
		Longitude: 128.296005249,
		Timezone:  "Australia/Perth",
	},
	"YGLY": {
		ICAO:      "YGLY",
		Name:      "Glenayle Homestead Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -25.2833003998,
		Longitude: 122.0500030518,
		Timezone:  "Australia/Perth",
	},
	"YGMD": {
		ICAO:      "YGMD",
		Name:      "Goomadeer Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -12.1000003815,
		Longitude: 133.6670074463,
		Timezone:  "Australia/Darwin",
	},
	"YGMP": {
		ICAO:      "YGMP",
		Name:      "Grampians Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -37.0499992371,
		Longitude: 142.2830047607,
		Timezone:  "Australia/Melbourne",
	},
	"YGNA": {
		ICAO:      "YGNA",
		Name:      "Granada Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -20.1000003815,
		Longitude: 140.3670043945,
		Timezone:  "Australia/Brisbane",
	},
	"YGNF": {
		ICAO:      "YGNF",
		Name:      "Grenfell Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -34,
		Longitude: 148.1329956055,
		Timezone:  "Australia/Sydney",
	},
	"YGNV": {
		ICAO:      "YGNV",
		IATA:      "GVP",
		Name:      "Greenvale Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -18.9832992554,
		Longitude: 145.1170043945,
		Timezone:  "Australia/Brisbane",
	},
	"YGNW": {
		ICAO:      "YGNW",
		Name:      "Gnowangerup Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -33.9832992554,
		Longitude: 118.0329971313,
		Timezone:  "Australia/Perth",
	},
	"YGOM": {
		ICAO:      "YGOM",
		Name:      "Goomalling Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -31.3666992188,
		Longitude: 116.8830032349,
		Timezone:  "Australia/Perth",
	},
	"YGOV": {
		ICAO:      "YGOV",
		Name:      "Gabo Is aAd sSte Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -37.5699996948,
		Longitude: 149.9179992676,
		Timezone:  "Australia/Melbourne",
	},
	"YGPI": {
		ICAO:      "YGPI",
		Name:      "Giles Point Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -23.2667007446,
		Longitude: 119.1669998169,
		Timezone:  "Australia/Perth",
	},
	"YGPR": {
		ICAO:      "YGPR",
		Name:      "Gunpowder Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -19.7000007629,
		Longitude: 139.3670043945,
		Timezone:  "Australia/Brisbane",
	},
	"YGPT": {
		ICAO:      "YGPT",
		IATA:      "GPN",
		Name:      "Garden Point Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 90,
		Latitude:  -11.4025001526,
		Longitude: 130.4219970703,
		Timezone:  "Australia/Darwin",
	},
	"YGRD": {
		ICAO:      "YGRD",
		Name:      "Grovedale Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -38.2167015076,
		Longitude: 144.3329925537,
		Timezone:  "Australia/Melbourne",
	},
	"YGRL": {
		ICAO:      "YGRL",
		Name:      "Great Lakes Vi Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 184,
		Latitude:  -37.8422012329,
		Longitude: 148,
		Timezone:  "Australia/Melbourne",
	},
	"YGRU": {
		ICAO:      "YGRU",
		Name:      "Glen Ruth Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -18.0832996368,
		Longitude: 145.3999938965,
		Timezone:  "Australia/Brisbane",
	},
	"YGSC": {
		ICAO:      "YGSC",
		IATA:      "GSC",
		Name:      "Gascoyne Junction Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -25.0499992371,
		Longitude: 115.1999969482,
		Timezone:  "Australia/Perth",
	},
	"YGTE": {
		ICAO:      "YGTE",
		IATA:      "GTE",
		Name:      "Groote Eylandt Airport",
		City:      "Groote Eylandt",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 53,
		Latitude:  -13.9750003815,
		Longitude: 136.460006714,
		Timezone:  "Australia/Darwin",
	},
	"YGTH": {
		ICAO:      "YGTH",
		IATA:      "GFF",
		Name:      "Griffith Airport",
		City:      "Griffith",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 439,
		Latitude:  -34.2508010864,
		Longitude: 146.067001343,
		Timezone:  "Australia/Sydney",
	},
	"YGTN": {
		ICAO:      "YGTN",
		IATA:      "GTT",
		Name:      "Georgetown Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 995,
		Latitude:  -18.3050003052,
		Longitude: 143.5299987793,
		Timezone:  "Australia/Brisbane",
	},
	"YGTO": {
		ICAO:      "YGTO",
		IATA:      "GEE",
		Name:      "Georgetown (Tas) Airport",
		State:     "Tasmania",
		Country:   "AU",
		Latitude:  -41.0800018311,
		Longitude: 146.8399963379,
		Timezone:  "Australia/Hobart",
	},
	"YGTP": {
		ICAO:      "YGTP",
		Name:      "Greenthorpe Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -33.9832992554,
		Longitude: 148.4170074463,
		Timezone:  "Australia/Sydney",
	},
	"YGUL": {
		ICAO:      "YGUL",
		Name:      "Gullewa Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -28.6333007813,
		Longitude: 116.2829971313,
		Timezone:  "Australia/Perth",
	},
	"YGUW": {
		ICAO:      "YGUW",
		Name:      "Gunnawarra Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -17.9500007629,
		Longitude: 145.1670074463,
		Timezone:  "Australia/Brisbane",
	},
	"YGVE": {
		ICAO:      "YGVE",
		Name:      "Glendevie Airport",
		State:     "Tasmania",
		Country:   "AU",
		Latitude:  -43.2299995422,
		Longitude: 147.0119934082,
		Timezone:  "Australia/Hobart",
	},
	"YGWA": {
		ICAO:      "YGWA",
		Name:      "Goolwa Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 32,
		Latitude:  -35.4817008972,
		Longitude: 138.7519989014,
		Timezone:  "Australia/Adelaide",
	},
	"YGWD": {
		ICAO:      "YGWD",
		Name:      "Galway Downs Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -25.1833000183,
		Longitude: 142.6670074463,
		Timezone:  "Australia/Brisbane",
	},
	"YGYM": {
		ICAO:      "YGYM",
		IATA:      "GYP",
		Name:      "Gympie Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 260,
		Latitude:  -26.2828006744,
		Longitude: 152.7019958496,
		Timezone:  "Australia/Brisbane",
	},
	"YHAA": {
		ICAO:      "YHAA",
		Name:      "Haasts Bluff Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 692,
		Latitude:  -23.4549999237,
		Longitude: 131.8529968262,
		Timezone:  "Australia/Darwin",
	},
	"YHAB": {
		ICAO:      "YHAB",
		Name:      "Hideaway Bay Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -20.1166992188,
		Longitude: 148.4329986572,
		Timezone:  "Australia/Brisbane",
	},
	"YHAE": {
		ICAO:      "YHAE",
		Name:      "Harden Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -34.5574989319,
		Longitude: 148.3880004883,
		Timezone:  "Australia/Sydney",
	},
	"YHAG": {
		ICAO:      "YHAG",
		Name:      "Haig Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -31,
		Longitude: 126.0830001831,
		Timezone:  "Australia/Perth",
	},
	"YHAW": {
		ICAO:      "YHAW",
		IATA:      "HWK",
		Name:      "Wilpena Pound Airport",
		City:      "Hawker",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 321,
		Latitude:  -31.8559074402,
		Longitude: 138.4680786133,
		Timezone:  "Australia/Adelaide",
	},
	"YHAY": {
		ICAO:      "YHAY",
		IATA:      "HXX",
		Name:      "Hay Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 305,
		Latitude:  -34.5313987732,
		Longitude: 144.8300018311,
		Timezone:  "Australia/Sydney",
	},
	"YHBA": {
		ICAO:      "YHBA",
		IATA:      "HVB",
		Name:      "Hervey Bay Airport",
		City:      "Hervey Bay",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 60,
		Latitude:  -25.3188991547,
		Longitude: 152.880004883,
		Timezone:  "Australia/Brisbane",
	},
	"YHBK": {
		ICAO:      "YHBK",
		Name:      "Holbrook Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 267,
		Latitude:  -35.6833000183,
		Longitude: 147.3170013428,
		Timezone:  "Australia/Sydney",
	},
	"YHBR": {
		ICAO:      "YHBR",
		IATA:      "HUB",
		Name:      "Humbert River Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -16.4896697998,
		Longitude: 130.630279541,
		Timezone:  "Australia/Darwin",
	},
	"YHBY": {
		ICAO:      "YHBY",
		IATA:      "HRY",
		Name:      "Henbury Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -24.5499992371,
		Longitude: 133.2169952393,
		Timezone:  "Australia/Darwin",
	},
	"YHCT": {
		ICAO:      "YHCT",
		Name:      "Heathcote Emergency Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 3537,
		Latitude:  -31.2402992249,
		Longitude: 151.4320068359,
		Timezone:  "Australia/Sydney",
	},
	"YHDD": {
		ICAO:      "YHDD",
		Name:      "Hodgson Downs Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -15.2216997147,
		Longitude: 134.0760040283,
		Timezone:  "Australia/Darwin",
	},
	"YHDY": {
		ICAO:      "YHDY",
		IATA:      "HIP",
		Name:      "Headingly Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -21.3332996368,
		Longitude: 138.2830047607,
		Timezone:  "Australia/Brisbane",
	},
	"YHEC": {
		ICAO:      "YHEC",
		Name:      "Heck Field Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -27.7675,
		Longitude: 153.339444,
		Timezone:  "Australia/Brisbane",
	},
	"YHEG": {
		ICAO:      "YHEG",
		Name:      "Hells Gate Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -17.4666996002,
		Longitude: 138.3670043945,
		Timezone:  "Australia/Brisbane",
	},
	"YHEW": {
		ICAO:      "YHEW",
		Name:      "Hedlow Airport",
		City:      "Mulara",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -23.2231006622,
		Longitude: 150.604995728,
		Timezone:  "Australia/Brisbane",
	},
	"YHGR": {
		ICAO:      "YHGR",
		Name:      "Hugh River Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -24.4167003632,
		Longitude: 133.6999969482,
		Timezone:  "Australia/Darwin",
	},
	"YHGS": {
		ICAO:      "YHGS",
		Name:      "Hughes Siding Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -30.7166996002,
		Longitude: 129.5149993896,
		Timezone:  "Australia/Adelaide",
	},
	"YHHY": {
		ICAO:      "YHHY",
		IATA:      "HIG",
		Name:      "Highbury Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -16.4333000183,
		Longitude: 143.1499938965,
		Timezone:  "Australia/Brisbane",
	},
	"YHID": {
		ICAO:      "YHID",
		IATA:      "HID",
		Name:      "Horn Island Airport",
		City:      "Horn Island",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 43,
		Latitude:  -10.586400032,
		Longitude: 142.289993286,
		Timezone:  "Australia/Brisbane",
	},
	"YHIL": {
		ICAO:      "YHIL",
		IATA:      "HLL",
		Name:      "Hillside Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -21.75,
		Longitude: 119.4169998169,
		Timezone:  "Australia/Perth",
	},
	"YHKT": {
		ICAO:      "YHKT",
		Name:      "Huckitta Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -22.9433002472,
		Longitude: 135.4470062256,
		Timezone:  "Australia/Darwin",
	},
	"YHLC": {
		ICAO:      "YHLC",
		IATA:      "HCQ",
		Name:      "Halls Creek Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 1346,
		Latitude:  -18.2339000702,
		Longitude: 127.6699981689,
		Timezone:  "Australia/Perth",
	},
	"YHLG": {
		ICAO:      "YHLG",
		Name:      "Hillgrove Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -19.6424999237,
		Longitude: 145.7969970703,
		Timezone:  "Australia/Brisbane",
	},
	"YHLM": {
		ICAO:      "YHLM",
		Name:      "Hillman Farm Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -33.2639007568,
		Longitude: 116.8150024414,
		Timezone:  "Australia/Perth",
	},
	"YHLS": {
		ICAO:      "YHLS",
		Name:      "Hillston Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 403,
		Latitude:  -33.4933013916,
		Longitude: 145.5229949951,
		Timezone:  "Australia/Sydney",
	},
	"YHMB": {
		ICAO:      "YHMB",
		IATA:      "HMG",
		Name:      "Hermannsburg Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 593,
		Latitude:  -23.9300003052,
		Longitude: 132.8049926758,
		Timezone:  "Australia/Darwin",
	},
	"YHML": {
		ICAO:      "YHML",
		IATA:      "HLT",
		Name:      "Hamilton Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 803,
		Latitude:  -37.6488990784,
		Longitude: 142.0650024414,
		Timezone:  "Australia/Melbourne",
	},
	"YHMT": {
		ICAO:      "YHMT",
		Name:      "Hamilton Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -26.7199993134,
		Longitude: 135.0729980469,
		Timezone:  "Australia/Adelaide",
	},
	"YHOA": {
		ICAO:      "YHOA",
		Name:      "Howard Island Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -12.095000267,
		Longitude: 135.3619995117,
		Timezone:  "Australia/Darwin",
	},
	"YHOO": {
		ICAO:      "YHOO",
		IATA:      "HOK",
		Name:      "Hooker Creek Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 320,
		Latitude:  -18.3367004395,
		Longitude: 130.6380004883,
		Timezone:  "Australia/Darwin",
	},
	"YHOT": {
		ICAO:      "YHOT",
		IATA:      "MHU",
		Name:      "Mount Hotham Airport",
		City:      "Mount Hotham",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 4260,
		Latitude:  -37.0475006104,
		Longitude: 147.333999634,
		Timezone:  "Australia/Melbourne",
	},
	"YHOV": {
		ICAO:      "YHOV",
		Name:      "Hodgson River Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -15.516699791,
		Longitude: 134.0670013428,
		Timezone:  "Australia/Darwin",
	},
	"YHOY": {
		ICAO:      "YHOY",
		Name:      "Hollins Bay Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -22.2667007446,
		Longitude: 150.0330047607,
		Timezone:  "Australia/Brisbane",
	},
	"YHPE": {
		ICAO:      "YHPE",
		Name:      "Hopetoun Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -33.9000015259,
		Longitude: 120.1579971313,
		Timezone:  "Australia/Perth",
	},
	"YHPN": {
		ICAO:      "YHPN",
		IATA:      "HTU",
		Name:      "Hopetoun Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 256,
		Latitude:  -35.7153015137,
		Longitude: 142.3600006104,
		Timezone:  "Australia/Melbourne",
	},
	"YHRD": {
		ICAO:      "YHRD",
		Name:      "Hungerford Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -28.9832992554,
		Longitude: 144.4499969482,
		Timezone:  "Australia/Brisbane",
	},
	"YHRG": {
		ICAO:      "YHRG",
		Name:      "Haddon Rig Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -31.4687995911,
		Longitude: 147.8950042725,
		Timezone:  "Australia/Sydney",
	},
	"YHSB": {
		ICAO:      "YHSB",
		Name:      "Horseshoe Bend Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -25.2166996002,
		Longitude: 134.266998291,
		Timezone:  "Australia/Darwin",
	},
	"YHSL": {
		ICAO:      "YHSL",
		Name:      "Horseshoe Lights Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -25.3500003815,
		Longitude: 118.6169967651,
		Timezone:  "Australia/Perth",
	},
	"YHSM": {
		ICAO:      "YHSM",
		IATA:      "HSM",
		Name:      "Horsham Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 445,
		Latitude:  -36.6697006226,
		Longitude: 142.1730041504,
		Timezone:  "Australia/Melbourne",
	},
	"YHTA": {
		ICAO:      "YHTA",
		Name:      "Hiltaba Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -32.1425018311,
		Longitude: 135.0989990234,
		Timezone:  "Australia/Adelaide",
	},
	"YHTL": {
		ICAO:      "YHTL",
		IATA:      "HAT",
		Name:      "Heathlands Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -11.75,
		Longitude: 142.5829925537,
		Timezone:  "Australia/Brisbane",
	},
	"YHTN": {
		ICAO:      "YHTN",
		Name:      "Herberton Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -17.4333000183,
		Longitude: 145.3999938965,
		Timezone:  "Australia/Brisbane",
	},
	"YHTR": {
		ICAO:      "YHTR",
		Name:      "Hunter Island Airport",
		State:     "Tasmania",
		Country:   "AU",
		Latitude:  -40.5167007446,
		Longitude: 144.733001709,
		Timezone:  "Australia/Hobart",
	},
	"YHTS": {
		ICAO:      "YHTS",
		Name:      "Harts Range Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -22.9850006104,
		Longitude: 134.9179992676,
		Timezone:  "Australia/Darwin",
	},
	"YHUG": {
		ICAO:      "YHUG",
		IATA:      "HGD",
		Name:      "Hughenden Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 1043,
		Latitude:  -20.8150005341,
		Longitude: 144.2250061035,
		Timezone:  "Australia/Brisbane",
	},
	"YHVH": {
		ICAO:      "YHVH",
		Name:      "Harvest Home Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -20.6833000183,
		Longitude: 146.6499938965,
		Timezone:  "Australia/Brisbane",
	},
	"YHYD": {
		ICAO:      "YHYD",
		Name:      "Hyden Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -32.4333000183,
		Longitude: 118.8669967651,
		Timezone:  "Australia/Perth",
	},
	"YIDK": {
		ICAO:      "YIDK",
		IATA:      "IDK",
		Name:      "Indulkana Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 401,
		Latitude:  -26.9666996002,
		Longitude: 133.3249969482,
		Timezone:  "Australia/Adelaide",
	},
	"YIDR": {
		ICAO:      "YIDR",
		Name:      "Idracowra Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -25.0632991791,
		Longitude: 133.733001709,
		Timezone:  "Australia/Darwin",
	},
	"YIFL": {
		ICAO:      "YIFL",
		IATA:      "IFL",
		Name:      "Innisfail Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 46,
		Latitude:  -17.5594005585,
		Longitude: 146.0119934082,
		Timezone:  "Australia/Brisbane",
	},
	"YIFY": {
		ICAO:      "YIFY",
		IATA:      "IFF",
		Name:      "Iffley Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 43,
		Latitude:  -18.8999996185,
		Longitude: 141.2169952393,
		Timezone:  "Australia/Brisbane",
	},
	"YIGM": {
		ICAO:      "YIGM",
		IATA:      "IGH",
		Name:      "Ingham Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 49,
		Latitude:  -18.6606006622,
		Longitude: 146.1519927979,
		Timezone:  "Australia/Brisbane",
	},
	"YIGR": {
		ICAO:      "YIGR",
		Name:      "Ingomar Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -29.6333007813,
		Longitude: 134.7929992676,
		Timezone:  "Australia/Adelaide",
	},
	"YIKL": {
		ICAO:      "YIKL",
		Name:      "Baikal Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -22.763299942,
		Longitude: 136.1629943848,
		Timezone:  "Australia/Darwin",
	},
	"YIKM": {
		ICAO:      "YIKM",
		IATA:      "IKP",
		Name:      "Inkerman Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 6,
		Latitude:  -16.2749996185,
		Longitude: 141.4420013428,
		Timezone:  "Australia/Brisbane",
	},
	"YILA": {
		ICAO:      "YILA",
		Name:      "Milawa Vineyard Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -36.4500007629,
		Longitude: 146.4329986572,
		Timezone:  "Australia/Melbourne",
	},
	"YILT": {
		ICAO:      "YILT",
		Name:      "Milton Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -35.3208007813,
		Longitude: 150.4389953613,
		Timezone:  "Australia/Sydney",
	},
	"YILW": {
		ICAO:      "YILW",
		Name:      "Inglewood Airport",
		City:      "Inglewood",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 840,
		Latitude:  -28.4167003632,
		Longitude: 151.0829925537,
		Timezone:  "Australia/Brisbane",
	},
	"YIMA": {
		ICAO:      "YIMA",
		Name:      "Imanpa Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -25.1333007813,
		Longitude: 132.5700073242,
		Timezone:  "Australia/Darwin",
	},
	"YIMB": {
		ICAO:      "YIMB",
		Name:      "Kimba Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 232,
		Latitude:  -33.0999984741,
		Longitude: 136.4600067139,
		Timezone:  "Australia/Adelaide",
	},
	"YIMP": {
		ICAO:      "YIMP",
		Name:      "Impadna Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -25.1499996185,
		Longitude: 133.5829925537,
		Timezone:  "Australia/Darwin",
	},
	"YIMT": {
		ICAO:      "YIMT",
		Name:      "Innamincka Township Airport",
		City:      "Innamincka Township",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -27.7416992188,
		Longitude: 140.7449951172,
		Timezone:  "Australia/Adelaide",
	},
	"YING": {
		ICAO:      "YING",
		Name:      "Ingelara Airport",
		City:      "Rewan",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -24.997027028,
		Longitude: 148.333282471,
		Timezone:  "Australia/Brisbane",
	},
	"YINJ": {
		ICAO:      "YINJ",
		IATA:      "INJ",
		Name:      "Injune Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 401,
		Latitude:  -25.8500003815,
		Longitude: 148.5330047607,
		Timezone:  "Australia/Brisbane",
	},
	"YINN": {
		ICAO:      "YINN",
		IATA:      "INM",
		Name:      "Innamincka Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 54,
		Latitude:  -27.7000007629,
		Longitude: 140.733001709,
		Timezone:  "Australia/Adelaide",
	},
	"YINO": {
		ICAO:      "YINO",
		Name:      "Inverell North Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -29.7667007446,
		Longitude: 151.1670074463,
		Timezone:  "Australia/Sydney",
	},
	"YINV": {
		ICAO:      "YINV",
		Name:      "Inverleigh Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -18.0167007446,
		Longitude: 140.5670013428,
		Timezone:  "Australia/Brisbane",
	},
	"YINW": {
		ICAO:      "YINW",
		IATA:      "IVW",
		Name:      "Inverway Airport",
		City:      "Inverway",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -17.8411006927,
		Longitude: 129.643005371,
		Timezone:  "Australia/Darwin",
	},
	"YISD": {
		ICAO:      "YISD",
		Name:      "Isis Downs Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -24.2166996002,
		Longitude: 144.6170043945,
		Timezone:  "Australia/Brisbane",
	},
	"YISF": {
		ICAO:      "YISF",
		IATA:      "ISI",
		Name:      "Isisford Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 694,
		Latitude:  -24.2583007813,
		Longitude: 144.4250030518,
		Timezone:  "Australia/Brisbane",
	},
	"YISV": {
		ICAO:      "YISV",
		Name:      "Innesvale Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -15.3832998276,
		Longitude: 131.266998291,
		Timezone:  "Australia/Darwin",
	},
	"YITT": {
		ICAO:      "YITT",
		Name:      "Mitta Mitta Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 820,
		Latitude:  -36.5138787416,
		Longitude: 147.359018326,
		Timezone:  "Australia/Melbourne",
	},
	"YIVL": {
		ICAO:      "YIVL",
		IATA:      "IVR",
		Name:      "Inverell Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 2667,
		Latitude:  -29.888299942,
		Longitude: 151.1439971924,
		Timezone:  "Australia/Sydney",
	},
	"YIVO": {
		ICAO:      "YIVO",
		Name:      "Ivanhoe Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 330,
		Latitude:  -32.8833007813,
		Longitude: 144.3099975586,
		Timezone:  "Australia/Sydney",
	},
	"YJAB": {
		ICAO:      "YJAB",
		IATA:      "JAB",
		Name:      "Jabiru Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 85,
		Latitude:  -12.6583003998,
		Longitude: 132.8930053711,
		Timezone:  "Australia/Darwin",
	},
	"YJAK": {
		ICAO:      "YJAK",
		Name:      "Jackson Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 106,
		Latitude:  -27.638299942,
		Longitude: 142.4080047607,
		Timezone:  "Australia/Brisbane",
	},
	"YJAM": {
		ICAO:      "YJAM",
		Name:      "Jameson Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -25.8500003815,
		Longitude: 127.6829986572,
		Timezone:  "Australia/Perth",
	},
	"YJBY": {
		ICAO:      "YJBY",
		Name:      "Jervis Bay Airport",
		City:      "Jervis Bay Territory",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 200,
		Latitude:  -35.1472015381,
		Longitude: 150.6970062256,
		Timezone:  "Australia/Sydney",
	},
	"YJCO": {
		ICAO:      "YJCO",
		Name:      "Jericho Airport",
		State:     "Tasmania",
		Country:   "AU",
		Latitude:  -42.3666992188,
		Longitude: 147.3170013428,
		Timezone:  "Australia/Hobart",
	},
	"YJCW": {
		ICAO:      "YJCW",
		Name:      "Jacobs Well Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -27.7616996765,
		Longitude: 153.3350067139,
		Timezone:  "Australia/Brisbane",
	},
	"YJDA": {
		ICAO:      "YJDA",
		IATA:      "JUN",
		Name:      "Jundah Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 145,
		Latitude:  -24.8416996002,
		Longitude: 143.0579986572,
		Timezone:  "Australia/Brisbane",
	},
	"YJEM": {
		ICAO:      "YJEM",
		Name:      "Jemalong Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -32.4099998474,
		Longitude: 147.6699981689,
		Timezone:  "Australia/Sydney",
	},
	"YJER": {
		ICAO:      "YJER",
		Name:      "Jerilderie Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 110,
		Latitude:  -35.3699989319,
		Longitude: 145.7250061035,
		Timezone:  "Australia/Sydney",
	},
	"YJEY": {
		ICAO:      "YJEY",
		Name:      "Jeedamya Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -29.3999996185,
		Longitude: 121.266998291,
		Timezone:  "Australia/Perth",
	},
	"YJGP": {
		ICAO:      "YJGP",
		Name:      "Jerramungup Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -33.9000015259,
		Longitude: 118.9000015259,
		Timezone:  "Australia/Perth",
	},
	"YJIG": {
		ICAO:      "YJIG",
		Name:      "Jiggalong Mission Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -23.3666992188,
		Longitude: 120.7829971313,
		Timezone:  "Australia/Perth",
	},
	"YJIN": {
		ICAO:      "YJIN",
		IATA:      "QJD",
		Name:      "Jindabyne Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 1036,
		Latitude:  -36.426700592,
		Longitude: 148.6020050049,
		Timezone:  "Australia/Sydney",
	},
	"YJLC": {
		ICAO:      "YJLC",
		IATA:      "JCK",
		Name:      "Julia Creek Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 404,
		Latitude:  -20.6683006287,
		Longitude: 141.7230072021,
		Timezone:  "Australia/Brisbane",
	},
	"YJNB": {
		ICAO:      "YJNB",
		IATA:      "JUR",
		Name:      "Jurien Bay Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 15,
		Latitude:  -30.2999992371,
		Longitude: 115.0329971313,
		Timezone:  "Australia/Perth",
	},
	"YJNK": {
		ICAO:      "YJNK",
		Name:      "Jinka Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -22.9500007629,
		Longitude: 135.733001709,
		Timezone:  "Australia/Darwin",
	},
	"YJNY": {
		ICAO:      "YJNY",
		Name:      "Jonroy Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -16.5,
		Longitude: 144.3999938965,
		Timezone:  "Australia/Brisbane",
	},
	"YJRO": {
		ICAO:      "YJRO",
		Name:      "Jericho Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -23.5832996368,
		Longitude: 146.1670074463,
		Timezone:  "Australia/Brisbane",
	},
	"YJST": {
		ICAO:      "YJST",
		Name:      "Hubert Wilkins Airstrip",
		City:      "Jamestown",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -33.1920845053,
		Longitude: 138.615531921,
		Timezone:  "Australia/Adelaide",
	},
	"YJUK": {
		ICAO:      "YJUK",
		Name:      "Tjukurla Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -24.3707103729,
		Longitude: 128.7393341064,
		Timezone:  "Australia/Perth",
	},
	"YJUN": {
		ICAO:      "YJUN",
		Name:      "Jundee Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 562,
		Latitude:  -26.4216995239,
		Longitude: 120.577003479,
		Timezone:  "Australia/Perth",
	},
	"YJVM": {
		ICAO:      "YJVM",
		Name:      "Jervois Mine Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -22.6333007813,
		Longitude: 136.266998291,
		Timezone:  "Australia/Darwin",
	},
	"YJVS": {
		ICAO:      "YJVS",
		Name:      "Jervois Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 331,
		Latitude:  -22.9132995605,
		Longitude: 136.1199951172,
		Timezone:  "Australia/Darwin",
	},
	"YJWB": {
		ICAO:      "YJWB",
		Name:      "Jowalbinna Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -15.733300209,
		Longitude: 144.266998291,
		Timezone:  "Australia/Brisbane",
	},
	"YKAE": {
		ICAO:      "YKAE",
		Name:      "Kalannie Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -30.3666992188,
		Longitude: 117.1330032349,
		Timezone:  "Australia/Perth",
	},
	"YKAJ": {
		ICAO:      "YKAJ",
		Name:      "Kajabbi Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -20.0333003998,
		Longitude: 140.0330047607,
		Timezone:  "Australia/Brisbane",
	},
	"YKAL": {
		ICAO:      "YKAL",
		IATA:      "UBU",
		Name:      "Kalumburu Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 29,
		Latitude:  -14.2882995605,
		Longitude: 126.6320037842,
		Timezone:  "Australia/Perth",
	},
	"YKAN": {
		ICAO:      "YKAN",
		Name:      "Kanandah Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -30.8950004578,
		Longitude: 124.8619995117,
		Timezone:  "Australia/Perth",
	},
	"YKAP": {
		ICAO:      "YKAP",
		Name:      "Kapunda Airport",
		City:      "Kapunda",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 1200,
		Latitude:  -34.25,
		Longitude: 138.9170074463,
		Timezone:  "Australia/Adelaide",
	},
	"YKAT": {
		ICAO:      "YKAT",
		Name:      "Katoomba Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 1000,
		Latitude:  -33.6683006287,
		Longitude: 150.3229980469,
		Timezone:  "Australia/Sydney",
	},
	"YKAY": {
		ICAO:      "YKAY",
		Name:      "Kayrunnera Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -30.6833000183,
		Longitude: 142.5330047607,
		Timezone:  "Australia/Sydney",
	},
	"YKBG": {
		ICAO:      "YKBG",
		Name:      "Koolyanobbing Range Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -30.8405990601,
		Longitude: 119.5289993286,
		Timezone:  "Australia/Perth",
	},
	"YKBL": {
		ICAO:      "YKBL",
		IATA:      "KDB",
		Name:      "Kambalda Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -31.1833000183,
		Longitude: 121.5999984741,
		Timezone:  "Australia/Perth",
	},
	"YKBN": {
		ICAO:      "YKBN",
		Name:      "Kooralbyn Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 94,
		Latitude:  -28.0900001526,
		Longitude: 152.8450012207,
		Timezone:  "Australia/Brisbane",
	},
	"YKBR": {
		ICAO:      "YKBR",
		IATA:      "KAX",
		Name:      "Kalbarri Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 157,
		Latitude:  -27.6900005341,
		Longitude: 114.2620010376,
		Timezone:  "Australia/Perth",
	},
	"YKBY": {
		ICAO:      "YKBY",
		IATA:      "KBY",
		Name:      "Streaky Bay Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 69,
		Latitude:  -32.8358001709,
		Longitude: 134.2929992676,
		Timezone:  "Australia/Adelaide",
	},
	"YKCA": {
		ICAO:      "YKCA",
		IATA:      "KBJ",
		Name:      "Kings Canyon Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -24.2600002289,
		Longitude: 131.4900054932,
		Timezone:  "Australia/Darwin",
	},
	"YKCK": {
		ICAO:      "YKCK",
		Name:      "Killiecrankie Airport",
		State:     "Tasmania",
		Country:   "AU",
		Latitude:  -39.8488006592,
		Longitude: 147.858001709,
		Timezone:  "Australia/Hobart",
	},
	"YKCS": {
		ICAO:      "YKCS",
		IATA:      "KCS",
		Name:      "Kings Creek Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 615,
		Latitude:  -24.4232997894,
		Longitude: 131.8350067139,
		Timezone:  "Australia/Darwin",
	},
	"YKDI": {
		ICAO:      "YKDI",
		Name:      "Kadina Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 42,
		Latitude:  -33.9766998291,
		Longitude: 137.6600036621,
		Timezone:  "Australia/Adelaide",
	},
	"YKDL": {
		ICAO:      "YKDL",
		Name:      "Kondoolka Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -32.0238990784,
		Longitude: 134.8609924316,
		Timezone:  "Australia/Adelaide",
	},
	"YKDM": {
		ICAO:      "YKDM",
		Name:      "Kidman Springs Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 88,
		Latitude:  -16.1166992188,
		Longitude: 130.9530029297,
		Timezone:  "Australia/Darwin",
	},
	"YKDN": {
		ICAO:      "YKDN",
		Name:      "Kondinin Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -32.4667015076,
		Longitude: 118.2829971313,
		Timezone:  "Australia/Perth",
	},
	"YKEB": {
		ICAO:      "YKEB",
		Name:      "Kellerberrin Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -31.6667003632,
		Longitude: 117.733001709,
		Timezone:  "Australia/Perth",
	},
	"YKEL": {
		ICAO:      "YKEL",
		Name:      "Kelvin Station Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 3383,
		Latitude:  -30.6550006866,
		Longitude: 151.6380004883,
		Timezone:  "Australia/Sydney",
	},
	"YKEN": {
		ICAO:      "YKEN",
		Name:      "Kenmore Park Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -26.3332996368,
		Longitude: 132.4669952393,
		Timezone:  "Australia/Adelaide",
	},
	"YKEP": {
		ICAO:      "YKEP",
		Name:      "Lake Keepit Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -30.8911034242,
		Longitude: 150.526149273,
		Timezone:  "Australia/Sydney",
	},
	"YKER": {
		ICAO:      "YKER",
		IATA:      "KRA",
		Name:      "Kerang Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 254,
		Latitude:  -35.7513999939,
		Longitude: 143.9389953613,
		Timezone:  "Australia/Melbourne",
	},
	"YKFC": {
		ICAO:      "YKFC",
		Name:      "Kingfisher Camp Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -17.8687992096,
		Longitude: 138.2949981689,
		Timezone:  "Australia/Brisbane",
	},
	"YKGA": {
		ICAO:      "YKGA",
		Name:      "Kingoonya Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -30.9016990662,
		Longitude: 135.3049926758,
		Timezone:  "Australia/Adelaide",
	},
	"YKHA": {
		ICAO:      "YKHA",
		Name:      "Khancoban Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -36.2332992554,
		Longitude: 148.1170043945,
		Timezone:  "Australia/Sydney",
	},
	"YKHG": {
		ICAO:      "YKHG",
		Name:      "Katherine Gorge Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -14.3886003494,
		Longitude: 132.3930053711,
		Timezone:  "Australia/Darwin",
	},
	"YKIA": {
		ICAO:      "YKIA",
		Name:      "Kiana Station Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -17.2416992188,
		Longitude: 136.1750030518,
		Timezone:  "Australia/Darwin",
	},
	"YKID": {
		ICAO:      "YKID",
		Name:      "Kidston Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 494,
		Latitude:  -18.8700008392,
		Longitude: 144.1730041504,
		Timezone:  "Australia/Brisbane",
	},
	"YKIG": {
		ICAO:      "YKIG",
		Name:      "Kingston Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 2,
		Latitude:  -36.823299408,
		Longitude: 139.875,
		Timezone:  "Australia/Adelaide",
	},
	"YKII": {
		ICAO:      "YKII",
		IATA:      "KNS",
		Name:      "King Island Airport",
		State:     "Tasmania",
		Country:   "AU",
		Elevation: 132,
		Latitude:  -39.8774986267,
		Longitude: 143.8780059814,
		Timezone:  "Australia/Currie",
	},
	"YKIL": {
		ICAO:      "YKIL",
		Name:      "Kildurk Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -16.4316997528,
		Longitude: 129.6150054932,
		Timezone:  "Australia/Darwin",
	},
	"YKIR": {
		ICAO:      "YKIR",
		IATA:      "KBB",
		Name:      "Kirkimbie Station Airport",
		City:      "Kirkimbie",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -17.7791996002,
		Longitude: 129.210006714,
		Timezone:  "Australia/Darwin",
	},
	"YKIU": {
		ICAO:      "YKIU",
		Name:      "Kaiuroo Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -23.1166992188,
		Longitude: 149.3670043945,
		Timezone:  "Australia/Brisbane",
	},
	"YKKG": {
		ICAO:      "YKKG",
		IATA:      "KFG",
		Name:      "Kalkgurung Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 646,
		Latitude:  -17.4319000244,
		Longitude: 130.8079986572,
		Timezone:  "Australia/Darwin",
	},
	"YKKH": {
		ICAO:      "YKKH",
		Name:      "Kokatha Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -31.2917003632,
		Longitude: 135.2420043945,
		Timezone:  "Australia/Adelaide",
	},
	"YKLA": {
		ICAO:      "YKLA",
		IATA:      "KOH",
		Name:      "Koolatah Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -15.9167003632,
		Longitude: 142.4499969482,
		Timezone:  "Australia/Brisbane",
	},
	"YKLB": {
		ICAO:      "YKLB",
		IATA:      "KKP",
		Name:      "Koolburra Airport",
		City:      "Koolburra",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 350,
		Latitude:  -15.3189001083,
		Longitude: 143.9550018311,
		Timezone:  "Australia/Brisbane",
	},
	"YKLE": {
		ICAO:      "YKLE",
		Name:      "Killarney Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 194,
		Latitude:  -16.25,
		Longitude: 131.7469940186,
		Timezone:  "Australia/Darwin",
	},
	"YKLG": {
		ICAO:      "YKLG",
		Name:      "Kalinga Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -15.1999998093,
		Longitude: 143.8500061035,
		Timezone:  "Australia/Brisbane",
	},
	"YKLL": {
		ICAO:      "YKLL",
		Name:      "Kallala Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -21.75,
		Longitude: 138.8829956055,
		Timezone:  "Australia/Brisbane",
	},
	"YKLN": {
		ICAO:      "YKLN",
		Name:      "Killarney Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -15.3999996185,
		Longitude: 143.5,
		Timezone:  "Australia/Brisbane",
	},
	"YKLR": {
		ICAO:      "YKLR",
		Name:      "Kalamurina Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -27.7166996002,
		Longitude: 138.2619934082,
		Timezone:  "Australia/Adelaide",
	},
	"YKMB": {
		ICAO:      "YKMB",
		IATA:      "KRB",
		Name:      "Karumba Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 5,
		Latitude:  -17.4566993713,
		Longitude: 140.8300018311,
		Timezone:  "Australia/Brisbane",
	},
	"YKML": {
		ICAO:      "YKML",
		IATA:      "KML",
		Name:      "Kamileroi Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 91,
		Latitude:  -19.375,
		Longitude: 140.0570068359,
		Timezone:  "Australia/Brisbane",
	},
	"YKMP": {
		ICAO:      "YKMP",
		IATA:      "KPS",
		Name:      "Kempsey Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 54,
		Latitude:  -31.0743999481,
		Longitude: 152.7700042725,
		Timezone:  "Australia/Sydney",
	},
	"YKNC": {
		ICAO:      "YKNC",
		Name:      "Kencherang Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -13.845000267,
		Longitude: 141.5919952393,
		Timezone:  "Australia/Brisbane",
	},
	"YKNG": {
		ICAO:      "YKNG",
		IATA:      "KNI",
		Name:      "Katanning Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -33.7167015076,
		Longitude: 117.6330032349,
		Timezone:  "Australia/Perth",
	},
	"YKNM": {
		ICAO:      "YKNM",
		Name:      "Koonmarra Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -26.2833003998,
		Longitude: 117.7829971313,
		Timezone:  "Australia/Perth",
	},
	"YKNP": {
		ICAO:      "YKNP",
		Name:      "Kununoppin Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -31.1333007813,
		Longitude: 117.9000015259,
		Timezone:  "Australia/Perth",
	},
	"YKNT": {
		ICAO:      "YKNT",
		Name:      "Kintore Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -23.2649993896,
		Longitude: 129.3869934082,
		Timezone:  "Australia/Darwin",
	},
	"YKNV": {
		ICAO:      "YKNV",
		Name:      "Kaniva Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -36.3833007813,
		Longitude: 141.25,
		Timezone:  "Australia/Melbourne",
	},
	"YKOG": {
		ICAO:      "YKOG",
		Name:      "Koongarra Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -36,
		Longitude: 142.2169952393,
		Timezone:  "Australia/Melbourne",
	},
	"YKOJ": {
		ICAO:      "YKOJ",
		Name:      "Kojonup Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -33.7667007446,
		Longitude: 117.1330032349,
		Timezone:  "Australia/Perth",
	},
	"YKOK": {
		ICAO:      "YKOK",
		Name:      "Kookynie Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -29.3500003815,
		Longitude: 121.483001709,
		Timezone:  "Australia/Perth",
	},
	"YKOL": {
		ICAO:      "YKOL",
		Name:      "Kolendo Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -32.4157981873,
		Longitude: 136.2980041504,
		Timezone:  "Australia/Adelaide",
	},
	"YKOW": {
		ICAO:      "YKOW",
		IATA:      "KWM",
		Name:      "Kowanyama Airport",
		City:      "Kowanyama",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 35,
		Latitude:  -15.4856004715,
		Longitude: 141.7510070801,
		Timezone:  "Australia/Brisbane",
	},
	"YKPR": {
		ICAO:      "YKPR",
		IATA:      "KPP",
		Name:      "Kalpowar Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -14.8999996185,
		Longitude: 144.1999969482,
		Timezone:  "Australia/Brisbane",
	},
	"YKRV": {
		ICAO:      "YKRV",
		Name:      "Kendall River Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -13.75,
		Longitude: 142.1829986572,
		Timezone:  "Australia/Brisbane",
	},
	"YKRY": {
		ICAO:      "YKRY",
		IATA:      "KGY",
		Name:      "Kingaroy Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 1492,
		Latitude:  -26.5807991028,
		Longitude: 151.841003418,
		Timezone:  "Australia/Brisbane",
	},
	"YKSC": {
		ICAO:      "YKSC",
		IATA:      "KGC",
		Name:      "Kingscote Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 24,
		Latitude:  -35.7139015198,
		Longitude: 137.5209960938,
		Timezone:  "Australia/Adelaide",
	},
	"YKTA": {
		ICAO:      "YKTA",
		Name:      "Kotta Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -36.1797981262,
		Longitude: 144.5249938965,
		Timezone:  "Australia/Melbourne",
	},
	"YKTH": {
		ICAO:      "YKTH",
		Name:      "Keith Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -36.1082992554,
		Longitude: 140.2420043945,
		Timezone:  "Australia/Adelaide",
	},
	"YKTN": {
		ICAO:      "YKTN",
		Name:      "Kyneton Airport",
		City:      "Kyneton",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 1650,
		Latitude:  -37.2262620936,
		Longitude: 144.44729805,
		Timezone:  "Australia/Melbourne",
	},
	"YKUB": {
		ICAO:      "YKUB",
		IATA:      "KUG",
		Name:      "Kubin Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 15,
		Latitude:  -10.2250003815,
		Longitude: 142.2180023193,
		Timezone:  "Australia/Brisbane",
	},
	"YKUW": {
		ICAO:      "YKUW",
		Name:      "Kurweeton Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -38.0499992371,
		Longitude: 143.1499938965,
		Timezone:  "Australia/Melbourne",
	},
	"YKUY": {
		ICAO:      "YKUY",
		Name:      "Kurray Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -28.2199993134,
		Longitude: 148.5780029297,
		Timezone:  "Australia/Brisbane",
	},
	"YKWG": {
		ICAO:      "YKWG",
		Name:      "Kangaroo Well Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -31.7849998474,
		Longitude: 135.6380004883,
		Timezone:  "Australia/Adelaide",
	},
	"YKYB": {
		ICAO:      "YKYB",
		Name:      "Kyabram Airport",
		City:      "Private Airfield in the Shire of Campaspe",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 340,
		Latitude:  -36.3324823653,
		Longitude: 144.972066879,
		Timezone:  "Australia/Melbourne",
	},
	"YKYN": {
		ICAO:      "YKYN",
		Name:      "Kynuna Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -21.6000003815,
		Longitude: 141.9170074463,
		Timezone:  "Australia/Brisbane",
	},
	"YLAG": {
		ICAO:      "YLAG",
		Name:      "Lagoon Bay Airport",
		State:     "Tasmania",
		Country:   "AU",
		Latitude:  -42.8849983215,
		Longitude: 147.9530029297,
		Timezone:  "Australia/Hobart",
	},
	"YLAH": {
		ICAO:      "YLAH",
		IATA:      "LWH",
		Name:      "Lawn Hill Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 122,
		Latitude:  -18.5683002472,
		Longitude: 138.6349945068,
		Timezone:  "Australia/Brisbane",
	},
	"YLAK": {
		ICAO:      "YLAK",
		Name:      "Lakeside Airpark",
		City:      "Lakeside",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -20.6816997528,
		Longitude: 148.6280059814,
		Timezone:  "Australia/Brisbane",
	},
	"YLAM": {
		ICAO:      "YLAM",
		Name:      "Lambina Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -26.9132995605,
		Longitude: 134.0579986572,
		Timezone:  "Australia/Adelaide",
	},
	"YLAN": {
		ICAO:      "YLAN",
		Name:      "Langawirra Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -31.4500007629,
		Longitude: 142.1329956055,
		Timezone:  "Australia/Sydney",
	},
	"YLAO": {
		ICAO:      "YLAO",
		Name:      "Lameroo Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -35.3666992188,
		Longitude: 140.5500030518,
		Timezone:  "Australia/Adelaide",
	},
	"YLAW": {
		ICAO:      "YLAW",
		Name:      "Lawlers Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -28.0832996368,
		Longitude: 120.5,
		Timezone:  "Australia/Perth",
	},
	"YLBD": {
		ICAO:      "YLBD",
		Name:      "Lombadina Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -16.5167007446,
		Longitude: 122.9169998169,
		Timezone:  "Australia/Perth",
	},
	"YLBG": {
		ICAO:      "YLBG",
		Name:      "Mount Liebig Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -23.2432994843,
		Longitude: 131.2599945068,
		Timezone:  "Australia/Darwin",
	},
	"YLBO": {
		ICAO:      "YLBO",
		Name:      "Lake Bolac Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -37.6833000183,
		Longitude: 142.8829956055,
		Timezone:  "Australia/Melbourne",
	},
	"YLCG": {
		ICAO:      "YLCG",
		Name:      "Lake Cargelligo Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 169,
		Latitude:  -33.278301239,
		Longitude: 146.3699951172,
		Timezone:  "Australia/Sydney",
	},
	"YLCS": {
		ICAO:      "YLCS",
		Name:      "Locksley Field",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 165,
		Latitude:  -36.8149986267,
		Longitude: 145.3480072021,
		Timezone:  "Australia/Melbourne",
	},
	"YLDB": {
		ICAO:      "YLDB",
		Name:      "Lady Barron (Flinders Is) Airport",
		State:     "Tasmania",
		Country:   "AU",
		Latitude:  -40.2000007629,
		Longitude: 148.25,
		Timezone:  "Australia/Hobart",
	},
	"YLDO": {
		ICAO:      "YLDO",
		Name:      "Landor Station Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -25.1000003815,
		Longitude: 116.9000015259,
		Timezone:  "Australia/Perth",
	},
	"YLEA": {
		ICAO:      "YLEA",
		Name:      "Leeman Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -29.9832992554,
		Longitude: 114.983001709,
		Timezone:  "Australia/Perth",
	},
	"YLEC": {
		ICAO:      "YLEC",
		IATA:      "LGH",
		Name:      "Leigh Creek Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 856,
		Latitude:  -30.5983009338,
		Longitude: 138.425994873,
		Timezone:  "Australia/Adelaide",
	},
	"YLED": {
		ICAO:      "YLED",
		Name:      "Lethbridge Park Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -37.9189987183,
		Longitude: 144.1009979248,
		Timezone:  "Australia/Melbourne",
	},
	"YLEE": {
		ICAO:      "YLEE",
		Name:      "Leeton Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -34.5,
		Longitude: 146.4329986572,
		Timezone:  "Australia/Sydney",
	},
	"YLEG": {
		ICAO:      "YLEG",
		Name:      "Leongatha Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 263,
		Latitude:  -38.4930992126,
		Longitude: 145.8600006104,
		Timezone:  "Australia/Melbourne",
	},
	"YLEO": {
		ICAO:      "YLEO",
		IATA:      "LNO",
		Name:      "Leonora Airport",
		City:      "Leonora",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 1217,
		Latitude:  -28.8780994415,
		Longitude: 121.3150024414,
		Timezone:  "Australia/Perth",
	},
	"YLET": {
		ICAO:      "YLET",
		Name:      "Lakes Entrance Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -37.8791999817,
		Longitude: 147.9869995117,
		Timezone:  "Australia/Melbourne",
	},
	"YLEV": {
		ICAO:      "YLEV",
		IATA:      "LEL",
		Name:      "Lake Evella Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 256,
		Latitude:  -12.4989004135,
		Longitude: 135.8059997559,
		Timezone:  "Australia/Darwin",
	},
	"YLFD": {
		ICAO:      "YLFD",
		IATA:      "LFP",
		Name:      "Lakefield Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -14.9333000183,
		Longitude: 144.1999969482,
		Timezone:  "Australia/Brisbane",
	},
	"YLGA": {
		ICAO:      "YLGA",
		Name:      "Mulga Downs Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -22.1072006226,
		Longitude: 118.4710006714,
		Timezone:  "Australia/Perth",
	},
	"YLGB": {
		ICAO:      "YLGB",
		Name:      "La Grange Bay Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -18.6833000183,
		Longitude: 121.8079986572,
		Timezone:  "Australia/Perth",
	},
	"YLGC": {
		ICAO:      "YLGC",
		Name:      "Lake Grace Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -33.1333007813,
		Longitude: 118.4000015259,
		Timezone:  "Australia/Perth",
	},
	"YLGD": {
		ICAO:      "YLGD",
		Name:      "Longdown Airport",
		State:     "Tasmania",
		Country:   "AU",
		Latitude:  -41.6953010559,
		Longitude: 147.1430053711,
		Timezone:  "Australia/Hobart",
	},
	"YLGL": {
		ICAO:      "YLGL",
		Name:      "Laglan Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -22.5,
		Longitude: 146.6670074463,
		Timezone:  "Australia/Brisbane",
	},
	"YLGN": {
		ICAO:      "YLGN",
		Name:      "Loongana Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -30.9500007629,
		Longitude: 127.0329971313,
		Timezone:  "Australia/Perth",
	},
	"YLGU": {
		ICAO:      "YLGU",
		Name:      "Legune Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 26,
		Latitude:  -15.2166996002,
		Longitude: 129.4479980469,
		Timezone:  "Australia/Darwin",
	},
	"YLHI": {
		ICAO:      "YLHI",
		IATA:      "LDH",
		Name:      "Lord Howe Island Airport",
		City:      "Lord Howe Island",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 5,
		Latitude:  -31.5382995605,
		Longitude: 159.07699585,
		Timezone:  "Australia/Lord_Howe",
	},
	"YLHL": {
		ICAO:      "YLHL",
		Name:      "Long Hill Airport",
		State:     "Tasmania",
		Country:   "AU",
		Latitude:  -41.3333015442,
		Longitude: 146.5,
		Timezone:  "Australia/Hobart",
	},
	"YLHR": {
		ICAO:      "YLHR",
		IATA:      "IRG",
		Name:      "Lockhart River Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 77,
		Latitude:  -12.7868995667,
		Longitude: 143.3049926758,
		Timezone:  "Australia/Brisbane",
	},
	"YLHS": {
		ICAO:      "YLHS",
		IATA:      "LTP",
		Name:      "Lyndhurst Airport",
		City:      "Lyndhurst",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -19.1958007812,
		Longitude: 144.371002197,
		Timezone:  "Australia/Brisbane",
	},
	"YLIL": {
		ICAO:      "YLIL",
		Name:      "Lilydale Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 76,
		Latitude:  -37.6916999817,
		Longitude: 145.3670043945,
		Timezone:  "Australia/Melbourne",
	},
	"YLIM": {
		ICAO:      "YLIM",
		Name:      "Limbunya Station Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -17.25,
		Longitude: 129.8829956055,
		Timezone:  "Australia/Darwin",
	},
	"YLIN": {
		ICAO:      "YLIN",
		IATA:      "LDC",
		Name:      "Lindeman Island Airport",
		City:      "Lindeman Island",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -20.4535999298,
		Longitude: 149.039993286,
		Timezone:  "Australia/Lindeman",
	},
	"YLIS": {
		ICAO:      "YLIS",
		IATA:      "LSY",
		Name:      "Lismore Airport",
		City:      "Lismore",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 35,
		Latitude:  -28.8302993774,
		Longitude: 153.259994507,
		Timezone:  "Australia/Sydney",
	},
	"YLKD": {
		ICAO:      "YLKD",
		Name:      "Lucky Downs Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -18.9167003632,
		Longitude: 145,
		Timezone:  "Australia/Brisbane",
	},
	"YLKE": {
		ICAO:      "YLKE",
		Name:      "Lakes Entrance Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -37.8526992798,
		Longitude: 147.9579925537,
		Timezone:  "Australia/Melbourne",
	},
	"YLKG": {
		ICAO:      "YLKG",
		Name:      "Lake King Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 1276,
		Latitude:  -33.0792007446,
		Longitude: 119.7659988403,
		Timezone:  "Australia/Perth",
	},
	"YLKN": {
		ICAO:      "YLKN",
		IATA:      "LNH",
		Name:      "Lake Nash Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -20.9666996002,
		Longitude: 137.9170074463,
		Timezone:  "Australia/Darwin",
	},
	"YLLA": {
		ICAO:      "YLLA",
		Name:      "Mobella Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -29.7982997894,
		Longitude: 133.3450012207,
		Timezone:  "Australia/Adelaide",
	},
	"YLLC": {
		ICAO:      "YLLC",
		Name:      "Lilla Creek Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -25.5666999817,
		Longitude: 134,
		Timezone:  "Australia/Darwin",
	},
	"YLLD": {
		ICAO:      "YLLD",
		Name:      "Langlo Downs Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -25.5333003998,
		Longitude: 145.766998291,
		Timezone:  "Australia/Brisbane",
	},
	"YLLE": {
		ICAO:      "YLLE",
		IATA:      "BBL",
		Name:      "Ballera Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 385,
		Latitude:  -27.4083003998,
		Longitude: 141.8079986572,
		Timezone:  "Australia/Brisbane",
	},
	"YLLR": {
		ICAO:      "YLLR",
		Name:      "Lake Leagur Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -35.9832992554,
		Longitude: 143.8000030518,
		Timezone:  "Australia/Melbourne",
	},
	"YLLV": {
		ICAO:      "YLLV",
		Name:      "Lily Vale Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -14.483300209,
		Longitude: 143.6670074463,
		Timezone:  "Australia/Brisbane",
	},
	"YLMB": {
		ICAO:      "YLMB",
		Name:      "Lambrook Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -31.1000003815,
		Longitude: 149.9329986572,
		Timezone:  "Australia/Sydney",
	},
	"YLMU": {
		ICAO:      "YLMU",
		Name:      "Mungo Lodge Airport",
		City:      "Mungo",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -33.7458244463,
		Longitude: 143.001308441,
		Timezone:  "Australia/Sydney",
	},
	"YLND": {
		ICAO:      "YLND",
		IATA:      "LKD",
		Name:      "Lakeland Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -15.8332996368,
		Longitude: 144.8500061035,
		Timezone:  "Australia/Brisbane",
	},
	"YLNR": {
		ICAO:      "YLNR",
		Name:      "Landor Races Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -24.9500007629,
		Longitude: 116.9670028687,
		Timezone:  "Australia/Perth",
	},
	"YLOC": {
		ICAO:      "YLOC",
		Name:      "Lochinvar Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -20.7583007813,
		Longitude: 121.016998291,
		Timezone:  "Australia/Perth",
	},
	"YLOD": {
		ICAO:      "YLOD",
		Name:      "Longwood Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -36.807800293,
		Longitude: 145.4570007324,
		Timezone:  "Australia/Melbourne",
	},
	"YLOH": {
		ICAO:      "YLOH",
		Name:      "Louth Airport",
		City:      "Louth",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -30.5417003632,
		Longitude: 145.1000061035,
		Timezone:  "Australia/Sydney",
	},
	"YLOK": {
		ICAO:      "YLOK",
		IATA:      "LOC",
		Name:      "Lock Airport",
		City:      "Lock",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 350,
		Latitude:  -33.5442008972,
		Longitude: 135.6929931641,
		Timezone:  "Australia/Adelaide",
	},
	"YLOR": {
		ICAO:      "YLOR",
		IATA:      "LOA",
		Name:      "Lorraine Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 61,
		Latitude:  -18.9932994843,
		Longitude: 139.9069976807,
		Timezone:  "Australia/Brisbane",
	},
	"YLOU": {
		ICAO:      "YLOU",
		Name:      "Louisa Downs Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -18.7166996002,
		Longitude: 126.7170028687,
		Timezone:  "Australia/Perth",
	},
	"YLOV": {
		ICAO:      "YLOV",
		IATA:      "LTV",
		Name:      "Lotus Vale Airport",
		City:      "Lotus Vale",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -17.0482997894,
		Longitude: 141.37600708,
		Timezone:  "Australia/Brisbane",
	},
	"YLOX": {
		ICAO:      "YLOX",
		Name:      "Loxton Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 38,
		Latitude:  -34.4749984741,
		Longitude: 140.6629943848,
		Timezone:  "Australia/Adelaide",
	},
	"YLOY": {
		ICAO:      "YLOY",
		Name:      "Longwarry Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -38.1082992554,
		Longitude: 145.7779998779,
		Timezone:  "Australia/Melbourne",
	},
	"YLPR": {
		ICAO:      "YLPR",
		Name:      "Lake Pedder Airport",
		State:     "Tasmania",
		Country:   "AU",
		Latitude:  -43.0499992371,
		Longitude: 146.3329925537,
		Timezone:  "Australia/Hobart",
	},
	"YLRA": {
		ICAO:      "YLRA",
		IATA:      "LUU",
		Name:      "Laura Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -15.5500001907,
		Longitude: 144.4499969482,
		Timezone:  "Australia/Brisbane",
	},
	"YLRD": {
		ICAO:      "YLRD",
		IATA:      "LHG",
		Name:      "Lightning Ridge Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 540,
		Latitude:  -29.4566993713,
		Longitude: 147.9839935303,
		Timezone:  "Australia/Sydney",
	},
	"YLRE": {
		ICAO:      "YLRE",
		IATA:      "LRE",
		Name:      "Longreach Airport",
		City:      "Longreach",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 627,
		Latitude:  -23.4342002869,
		Longitude: 144.279998779,
		Timezone:  "Australia/Brisbane",
	},
	"YLRG": {
		ICAO:      "YLRG",
		Name:      "Lorna Glen Homestead Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -26.2166996002,
		Longitude: 121.5500030518,
		Timezone:  "Australia/Perth",
	},
	"YLRN": {
		ICAO:      "YLRN",
		Name:      "Lorraine Station Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -22.5333003998,
		Longitude: 143.516998291,
		Timezone:  "Australia/Brisbane",
	},
	"YLRS": {
		ICAO:      "YLRS",
		IATA:      "LUT",
		Name:      "New Laura Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -15.1833000183,
		Longitude: 144.3670043945,
		Timezone:  "Australia/Brisbane",
	},
	"YLSM": {
		ICAO:      "YLSM",
		Name:      "Lismore Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -37.9333000183,
		Longitude: 143.3500061035,
		Timezone:  "Australia/Melbourne",
	},
	"YLSS": {
		ICAO:      "YLSS",
		Name:      "Lansdowne Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -25.0499992371,
		Longitude: 146.266998291,
		Timezone:  "Australia/Brisbane",
	},
	"YLST": {
		ICAO:      "YLST",
		IATA:      "LER",
		Name:      "Leinster Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 1631,
		Latitude:  -27.8432998657,
		Longitude: 120.7030029297,
		Timezone:  "Australia/Perth",
	},
	"YLSY": {
		ICAO:      "YLSY",
		Name:      "Mount Lindsay Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -27.0182991028,
		Longitude: 129.8849945068,
		Timezone:  "Australia/Adelaide",
	},
	"YLTN": {
		ICAO:      "YLTN",
		IATA:      "LVO",
		Name:      "Laverton Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 1530,
		Latitude:  -28.6135997772,
		Longitude: 122.4240036011,
		Timezone:  "Australia/Perth",
	},
	"YLTV": {
		ICAO:      "YLTV",
		IATA:      "TGN",
		Name:      "Latrobe Valley Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 180,
		Latitude:  -38.2071990967,
		Longitude: 146.470001221,
		Timezone:  "Australia/Melbourne",
	},
	"YLUC": {
		ICAO:      "YLUC",
		Name:      "Lucy Creek Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -22.4666996002,
		Longitude: 136.266998291,
		Timezone:  "Australia/Darwin",
	},
	"YLUI": {
		ICAO:      "YLUI",
		Name:      "Lucindale Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -36.971698761,
		Longitude: 140.3520050049,
		Timezone:  "Australia/Adelaide",
	},
	"YLUL": {
		ICAO:      "YLUL",
		Name:      "Mooleulooloo Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -31.6333007813,
		Longitude: 140.5330047607,
		Timezone:  "Australia/Adelaide",
	},
	"YLUW": {
		ICAO:      "YLUW",
		Name:      "Leeuwin Estate Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -34.0167007446,
		Longitude: 115.0500030518,
		Timezone:  "Australia/Perth",
	},
	"YLVB": {
		ICAO:      "YLVB",
		Name:      "Lovely Banks Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -38.0332984924,
		Longitude: 144.3329925537,
		Timezone:  "Australia/Melbourne",
	},
	"YLVD": {
		ICAO:      "YLVD",
		Name:      "Lake Everard Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -31.7332992554,
		Longitude: 135.0269927979,
		Timezone:  "Australia/Adelaide",
	},
	"YLVK": {
		ICAO:      "YLVK",
		Name:      "Laverak (Military) Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 9,
		Latitude:  -19.3216991425,
		Longitude: 146.8020019531,
		Timezone:  "Australia/Brisbane",
	},
	"YLVY": {
		ICAO:      "YLVY",
		Name:      "Lake Varley Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -32.7083015442,
		Longitude: 119.5130004883,
		Timezone:  "Australia/Perth",
	},
	"YLWY": {
		ICAO:      "YLWY",
		Name:      "Lake Way Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -26.9449996948,
		Longitude: 120.4700012207,
		Timezone:  "Australia/Perth",
	},
	"YLYD": {
		ICAO:      "YLYD",
		Name:      "Lyndley Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -26.8332996368,
		Longitude: 151.233001709,
		Timezone:  "Australia/Brisbane",
	},
	"YLYK": {
		ICAO:      "YLYK",
		Name:      "Lyndoch Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -34.6217002869,
		Longitude: 138.9140014648,
		Timezone:  "Australia/Adelaide",
	},
	"YLYN": {
		ICAO:      "YLYN",
		Name:      "Lyndon Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -23.6333007813,
		Longitude: 115.233001709,
		Timezone:  "Australia/Perth",
	},
	"YLZI": {
		ICAO:      "YLZI",
		IATA:      "LZR",
		Name:      "Lizard Island Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 70,
		Latitude:  -14.6667003632,
		Longitude: 145.4499969482,
		Timezone:  "Australia/Brisbane",
	},
	"YLZR": {
		ICAO:      "YLZR",
		Name:      "Lizard Island Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -14.6833000183,
		Longitude: 145.4499969482,
		Timezone:  "Australia/Brisbane",
	},
	"YMAA": {
		ICAO:      "YMAA",
		IATA:      "UBB",
		Name:      "Mabuiag Island Airport",
		City:      "Mabuiag Island",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -9.9499998093,
		Longitude: 142.1829986572,
		Timezone:  "Australia/Brisbane",
	},
	"YMAC": {
		ICAO:      "YMAC",
		Name:      "Macumba Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -27.2616996765,
		Longitude: 135.6380004883,
		Timezone:  "Australia/Adelaide",
	},
	"YMAD": {
		ICAO:      "YMAD",
		Name:      "Madura Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -31.8833007813,
		Longitude: 127.0329971313,
		Timezone:  "Australia/Eucla",
	},
	"YMAG": {
		ICAO:      "YMAG",
		Name:      "Manangatang Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -35.0499992371,
		Longitude: 142.8670043945,
		Timezone:  "Australia/Melbourne",
	},
	"YMAI": {
		ICAO:      "YMAI",
		Name:      "Manguri Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -28.9799995422,
		Longitude: 134.3719940186,
		Timezone:  "Australia/Adelaide",
	},
	"YMAK": {
		ICAO:      "YMAK",
		Name:      "Mabel Creek Station Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -28.9416999817,
		Longitude: 134.3329925537,
		Timezone:  "Australia/Adelaide",
	},
	"YMAU": {
		ICAO:      "YMAU",
		Name:      "Mount Augusta Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -24.2999992371,
		Longitude: 116.9169998169,
		Timezone:  "Australia/Perth",
	},
	"YMAV": {
		ICAO:      "YMAV",
		IATA:      "AVV",
		Name:      "Avalon Airport",
		City:      "Melbourne",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 35,
		Latitude:  -38.0393981934,
		Longitude: 144.468994141,
		Timezone:  "Australia/Melbourne",
	},
	"YMAY": {
		ICAO:      "YMAY",
		IATA:      "ABX",
		Name:      "Albury Airport",
		City:      "Albury",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 539,
		Latitude:  -36.0677986145,
		Longitude: 146.9579925537,
		Timezone:  "Australia/Melbourne",
	},
	"YMBA": {
		ICAO:      "YMBA",
		IATA:      "MRG",
		Name:      "Mareeba Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 1560,
		Latitude:  -17.0692005157,
		Longitude: 145.4190063477,
		Timezone:  "Australia/Brisbane",
	},
	"YMBD": {
		ICAO:      "YMBD",
		Name:      "Murray Bridge Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 55,
		Latitude:  -35.0666999817,
		Longitude: 139.2270050049,
		Timezone:  "Australia/Adelaide",
	},
	"YMBL": {
		ICAO:      "YMBL",
		IATA:      "MBB",
		Name:      "Marble Bar Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 194,
		Latitude:  -21.1632995605,
		Longitude: 119.8330001831,
		Timezone:  "Australia/Perth",
	},
	"YMBT": {
		ICAO:      "YMBT",
		Name:      "Mount Beauty Airport",
		City:      "Mount Beauty",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 1100,
		Latitude:  -36.7315756754,
		Longitude: 147.168903351,
		Timezone:  "Australia/Melbourne",
	},
	"YMBU": {
		ICAO:      "YMBU",
		Name:      "Maryborough Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 766,
		Latitude:  -37.0331001282,
		Longitude: 143.7089996338,
		Timezone:  "Australia/Melbourne",
	},
	"YMBX": {
		ICAO:      "YMBX",
		Name:      "Mundrabilla Airport",
		City:      "Mundrabilla",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 120,
		Latitude:  -31.8672008514,
		Longitude: 127.8539962769,
		Timezone:  "Australia/Eucla",
	},
	"YMBZ": {
		ICAO:      "YMBZ",
		Name:      "Mount Ebenezer Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -25.1667003632,
		Longitude: 132.6329956055,
		Timezone:  "Australia/Darwin",
	},
	"YMCE": {
		ICAO:      "YMCE",
		Name:      "Mount Clere Homestead Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -25.1000003815,
		Longitude: 117.5830001831,
		Timezone:  "Australia/Perth",
	},
	"YMCK": {
		ICAO:      "YMCK",
		Name:      "Mckinley Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -21.2833003998,
		Longitude: 141.2879943848,
		Timezone:  "Australia/Brisbane",
	},
	"YMCL": {
		ICAO:      "YMCL",
		Name:      "Mount Coolon Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 239,
		Latitude:  -21.3917007446,
		Longitude: 147.3249969482,
		Timezone:  "Australia/Brisbane",
	},
	"YMCO": {
		ICAO:      "YMCO",
		IATA:      "XMC",
		Name:      "Mallacoota Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 31,
		Latitude:  -37.5983009338,
		Longitude: 149.7200012207,
		Timezone:  "Australia/Melbourne",
	},
	"YMCR": {
		ICAO:      "YMCR",
		IATA:      "MFP",
		Name:      "Manners Creek Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -22.1000003815,
		Longitude: 137.983001709,
		Timezone:  "Australia/Darwin",
	},
	"YMCS": {
		ICAO:      "YMCS",
		Name:      "Macrossan Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -20.0167007446,
		Longitude: 146.483001709,
		Timezone:  "Australia/Brisbane",
	},
	"YMCT": {
		ICAO:      "YMCT",
		IATA:      "MLR",
		Name:      "Millicent Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 56,
		Latitude:  -37.5835990906,
		Longitude: 140.3659973145,
		Timezone:  "Australia/Adelaide",
	},
	"YMCU": {
		ICAO:      "YMCU",
		Name:      "Mount Mcclure Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -27.4699993134,
		Longitude: 120.9169998169,
		Timezone:  "Australia/Perth",
	},
	"YMDA": {
		ICAO:      "YMDA",
		Name:      "Mundubbera Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 122,
		Latitude:  -25.5916996002,
		Longitude: 151.3170013428,
		Timezone:  "Australia/Brisbane",
	},
	"YMDB": {
		ICAO:      "YMDB",
		Name:      "Mundabullangana Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -20.5167007446,
		Longitude: 118.0670013428,
		Timezone:  "Australia/Perth",
	},
	"YMDG": {
		ICAO:      "YMDG",
		IATA:      "DGE",
		Name:      "Mudgee Airport",
		City:      "Mudgee",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 1545,
		Latitude:  -32.5625,
		Longitude: 149.610992432,
		Timezone:  "Australia/Sydney",
	},
	"YMDI": {
		ICAO:      "YMDI",
		IATA:      "MQA",
		Name:      "Mandora Airport",
		City:      "Mandora",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -19.7383003235,
		Longitude: 120.837997437,
		Timezone:  "Australia/Perth",
	},
	"YMDK": {
		ICAO:      "YMDK",
		Name:      "Mount Riddock Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -23.0333003998,
		Longitude: 134.6829986572,
		Timezone:  "Australia/Darwin",
	},
	"YMDN": {
		ICAO:      "YMDN",
		Name:      "Merredin Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 396,
		Latitude:  -31.5049991608,
		Longitude: 118.3229980469,
		Timezone:  "Australia/Perth",
	},
	"YMDR": {
		ICAO:      "YMDR",
		Name:      "Minderoo Station Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -22,
		Longitude: 115.0500030518,
		Timezone:  "Australia/Perth",
	},
	"YMDS": {
		ICAO:      "YMDS",
		IATA:      "MNW",
		Name:      "Macdonald Downs Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -22.4666996002,
		Longitude: 135.233001709,
		Timezone:  "Australia/Darwin",
	},
	"YMDT": {
		ICAO:      "YMDT",
		Name:      "Mundrabilla Motel Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -31.8267002106,
		Longitude: 128.2299957275,
		Timezone:  "Australia/Eucla",
	},
	"YMDV": {
		ICAO:      "YMDV",
		Name:      "Mount Davies Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -26.1667003632,
		Longitude: 129.1329956055,
		Timezone:  "Australia/Adelaide",
	},
	"YMDW": {
		ICAO:      "YMDW",
		Name:      "Maitland Downs Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -16.2233009338,
		Longitude: 144.7030029297,
		Timezone:  "Australia/Brisbane",
	},
	"YMDY": {
		ICAO:      "YMDY",
		Name:      "Mount Bundey Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 150,
		Latitude:  -12.8900003433,
		Longitude: 131.9080047607,
		Timezone:  "Australia/Darwin",
	},
	"YMDZ": {
		ICAO:      "YMDZ",
		Name:      "Mardi Station Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -21.2000007629,
		Longitude: 115.983001709,
		Timezone:  "Australia/Perth",
	},
	"YMEB": {
		ICAO:      "YMEB",
		Name:      "Mount Eba Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -30.1760997772,
		Longitude: 135.671005249,
		Timezone:  "Australia/Adelaide",
	},
	"YMED": {
		ICAO:      "YMED",
		Name:      "Menindee Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -32.3666992188,
		Longitude: 142.4049987793,
		Timezone:  "Australia/Sydney",
	},
	"YMEI": {
		ICAO:      "YMEI",
		Name:      "Mereenie Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 735,
		Latitude:  -23.9766998291,
		Longitude: 131.56199646,
		Timezone:  "Australia/Darwin",
	},
	"YMEK": {
		ICAO:      "YMEK",
		IATA:      "MKR",
		Name:      "Meekatharra Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 1713,
		Latitude:  -26.611700058,
		Longitude: 118.547996521,
		Timezone:  "Australia/Perth",
	},
	"YMEL": {
		ICAO:      "YMEL",
		Name:      "Melton Airport",
		City:      "Melton",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 204,
		Latitude:  -37.6166992188,
		Longitude: 144.5670013428,
		Timezone:  "Australia/Melbourne",
	},
	"YMEN": {
		ICAO:      "YMEN",
		IATA:      "MEB",
		Name:      "Melbourne Essendon Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 282,
		Latitude:  -37.728099823,
		Longitude: 144.9019927979,
		Timezone:  "Australia/Melbourne",
	},
	"YMEO": {
		ICAO:      "YMEO",
		Name:      "Merton Airport",
		City:      "Merton",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 1101,
		Latitude:  -36.967704,
		Longitude: 145.707616,
		Timezone:  "Australia/Melbourne",
	},
	"YMEP": {
		ICAO:      "YMEP",
		Name:      "Merapah Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -13.7166996002,
		Longitude: 142.4170074463,
		Timezone:  "Australia/Brisbane",
	},
	"YMER": {
		ICAO:      "YMER",
		IATA:      "MIM",
		Name:      "Merimbula Airport",
		City:      "Merimbula",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 7,
		Latitude:  -36.9085998535,
		Longitude: 149.901000977,
		Timezone:  "Australia/Sydney",
	},
	"YMES": {
		ICAO:      "YMES",
		IATA:      "SXE",
		Name:      "RAAF Base East Sale",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 23,
		Latitude:  -38.0988998413,
		Longitude: 147.1490020752,
		Timezone:  "Australia/Melbourne",
	},
	"YMEU": {
		ICAO:      "YMEU",
		IATA:      "MLV",
		Name:      "Merluna Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -13.0500001907,
		Longitude: 142.483001709,
		Timezone:  "Australia/Brisbane",
	},
	"YMEW": {
		ICAO:      "YMEW",
		Name:      "Mingenew Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -29.2667007446,
		Longitude: 115.4499969482,
		Timezone:  "Australia/Perth",
	},
	"YMEY": {
		ICAO:      "YMEY",
		Name:      "Mullaley Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -31.1000003815,
		Longitude: 149.9170074463,
		Timezone:  "Australia/Sydney",
	},
	"YMFD": {
		ICAO:      "YMFD",
		Name:      "Mansfield Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 1050,
		Latitude:  -37.0666999817,
		Longitude: 146.117004395,
		Timezone:  "Australia/Melbourne",
	},
	"YMGB": {
		ICAO:      "YMGB",
		IATA:      "MGT",
		Name:      "Milingimbi Airport",
		City:      "Milingimbi Island",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 53,
		Latitude:  -12.0944004059,
		Longitude: 134.893997192,
		Timezone:  "Australia/Darwin",
	},
	"YMGD": {
		ICAO:      "YMGD",
		IATA:      "MNG",
		Name:      "Maningrida Airport",
		City:      "Maningrida",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 123,
		Latitude:  -12.0560998917,
		Longitude: 134.23399353,
		Timezone:  "Australia/Darwin",
	},
	"YMGG": {
		ICAO:      "YMGG",
		Name:      "Mulgathing Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -30.2292003632,
		Longitude: 133.9859924316,
		Timezone:  "Australia/Adelaide",
	},
	"YMGI": {
		ICAO:      "YMGI",
		Name:      "Mungindi Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 600,
		Latitude:  -28.9666996002,
		Longitude: 149.0579986572,
		Timezone:  "Australia/Sydney",
	},
	"YMGN": {
		ICAO:      "YMGN",
		IATA:      "GSN",
		Name:      "Mount Gunson Airport",
		City:      "Mount Gunson",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 285,
		Latitude:  -31.4596996307,
		Longitude: 137.1739959717,
		Timezone:  "Australia/Adelaide",
	},
	"YMGO": {
		ICAO:      "YMGO",
		Name:      "Murgoo Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -27.3666992188,
		Longitude: 116.4169998169,
		Timezone:  "Australia/Perth",
	},
	"YMGR": {
		ICAO:      "YMGR",
		IATA:      "MGV",
		Name:      "Margaret River (Station) Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 289,
		Latitude:  -18.6217002869,
		Longitude: 126.883003235,
		Timezone:  "Australia/Perth",
	},
	"YMGS": {
		ICAO:      "YMGS",
		Name:      "Mount Morgans Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -28.7872009277,
		Longitude: 122.033996582,
		Timezone:  "Australia/Perth",
	},
	"YMGT": {
		ICAO:      "YMGT",
		IATA:      "MQZ",
		Name:      "Margaret River Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 374,
		Latitude:  -33.9305992126,
		Longitude: 115.099998474,
		Timezone:  "Australia/Perth",
	},
	"YMGV": {
		ICAO:      "YMGV",
		IATA:      "MVU",
		Name:      "Musgrave Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -14.7833003998,
		Longitude: 143.5,
		Timezone:  "Australia/Brisbane",
	},
	"YMHB": {
		ICAO:      "YMHB",
		IATA:      "HBA",
		Name:      "Hobart International Airport",
		City:      "Hobart",
		State:     "Tasmania",
		Country:   "AU",
		Elevation: 13,
		Latitude:  -42.836101532,
		Longitude: 147.509994507,
		Timezone:  "Australia/Hobart",
	},
	"YMHL": {
		ICAO:      "YMHL",
		Name:      "Mount Holland Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -32.1183013916,
		Longitude: 119.7679977417,
		Timezone:  "Australia/Perth",
	},
	"YMHO": {
		ICAO:      "YMHO",
		IATA:      "MHO",
		Name:      "Mount House Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 289,
		Latitude:  -17.0550003052,
		Longitude: 125.7099990845,
		Timezone:  "Australia/Perth",
	},
	"YMHT": {
		ICAO:      "YMHT",
		Name:      "Mount Hart Station Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -16.8199996948,
		Longitude: 124.9079971313,
		Timezone:  "Australia/Perth",
	},
	"YMHU": {
		ICAO:      "YMHU",
		IATA:      "MCV",
		Name:      "McArthur River Mine Airport",
		City:      "McArthur River Mine",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 131,
		Latitude:  -16.4424991608,
		Longitude: 136.083999634,
		Timezone:  "Australia/Darwin",
	},
	"YMHW": {
		ICAO:      "YMHW",
		Name:      "Mount Howitt Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 105,
		Latitude:  -26.5116996765,
		Longitude: 142.2830047607,
		Timezone:  "Australia/Brisbane",
	},
	"YMIA": {
		ICAO:      "YMIA",
		IATA:      "MQL",
		Name:      "Mildura Airport",
		City:      "Mildura",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 167,
		Latitude:  -34.2291984558,
		Longitude: 142.085998535,
		Timezone:  "Australia/Melbourne",
	},
	"YMIB": {
		ICAO:      "YMIB",
		Name:      "Mintabie Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 366,
		Latitude:  -27.3283004761,
		Longitude: 133.3049926758,
		Timezone:  "Australia/Adelaide",
	},
	"YMIG": {
		ICAO:      "YMIG",
		Name:      "Mittagong Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -34.4667015076,
		Longitude: 150.5,
		Timezone:  "Australia/Sydney",
	},
	"YMIJ": {
		ICAO:      "YMIJ",
		Name:      "Minjilang Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 25,
		Latitude:  -11.1583003998,
		Longitude: 132.5469970703,
		Timezone:  "Australia/Darwin",
	},
	"YMIL": {
		ICAO:      "YMIL",
		Name:      "Milgun Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -25.0832996368,
		Longitude: 118.3330001831,
		Timezone:  "Australia/Perth",
	},
	"YMIN": {
		ICAO:      "YMIN",
		IATA:      "XML",
		Name:      "Minlaton Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -34.75,
		Longitude: 137.5330047607,
		Timezone:  "Australia/Adelaide",
	},
	"YMIP": {
		ICAO:      "YMIP",
		IATA:      "MIH",
		Name:      "Mitchell Plateau Airport",
		City:      "Mitchell Plateau",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -14.7913999557,
		Longitude: 125.8239974976,
		Timezone:  "Australia/Perth",
	},
	"YMIR": {
		ICAO:      "YMIR",
		IATA:      "MWY",
		Name:      "Miralwyn Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -30.1499996185,
		Longitude: 147.3350067139,
		Timezone:  "Australia/Sydney",
	},
	"YMIS": {
		ICAO:      "YMIS",
		Name:      "Millrose Homestead Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -26.3999996185,
		Longitude: 120.9499969482,
		Timezone:  "Australia/Perth",
	},
	"YMIT": {
		ICAO:      "YMIT",
		IATA:      "MTQ",
		Name:      "Mitchell Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 1144,
		Latitude:  -26.4832992554,
		Longitude: 147.93699646,
		Timezone:  "Australia/Brisbane",
	},
	"YMIV": {
		ICAO:      "YMIV",
		Name:      "Mount Ive Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -32.4441986084,
		Longitude: 136.0690002441,
		Timezone:  "Australia/Adelaide",
	},
	"YMIX": {
		ICAO:      "YMIX",
		Name:      "Middalya Homestead Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -23.8999996185,
		Longitude: 114.766998291,
		Timezone:  "Australia/Perth",
	},
	"YMJE": {
		ICAO:      "YMJE",
		Name:      "Mount James Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -24.6166992188,
		Longitude: 116.9329986572,
		Timezone:  "Australia/Perth",
	},
	"YMJM": {
		ICAO:      "YMJM",
		IATA:      "MJP",
		Name:      "Manjimup Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 940,
		Latitude:  -34.2653007507,
		Longitude: 116.1399993896,
		Timezone:  "Australia/Perth",
	},
	"YMKA": {
		ICAO:      "YMKA",
		Name:      "Mararanka Homestead Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -14.9316997528,
		Longitude: 133.125,
		Timezone:  "Australia/Darwin",
	},
	"YMKB": {
		ICAO:      "YMKB",
		Name:      "Mukinbudin Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -30.9333000183,
		Longitude: 118.25,
		Timezone:  "Australia/Perth",
	},
	"YMKT": {
		ICAO:      "YMKT",
		Name:      "Emkaytee (Unlic) Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -12.6167001724,
		Longitude: 131.0500030518,
		Timezone:  "Australia/Darwin",
	},
	"YMLA": {
		ICAO:      "YMLA",
		Name:      "Malina Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -20.8833007813,
		Longitude: 118.0329971313,
		Timezone:  "Australia/Perth",
	},
	"YMLC": {
		ICAO:      "YMLC",
		Name:      "Mole Creek Airport",
		State:     "Tasmania",
		Country:   "AU",
		Latitude:  -41.5332984924,
		Longitude: 146.3329925537,
		Timezone:  "Australia/Hobart",
	},
	"YMLD": {
		ICAO:      "YMLD",
		Name:      "Maitland Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -34.3927993774,
		Longitude: 137.7169952393,
		Timezone:  "Australia/Adelaide",
	},
	"YMLK": {
		ICAO:      "YMLK",
		Name:      "Minnamoolka Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -18.1833000183,
		Longitude: 145.1670074463,
		Timezone:  "Australia/Brisbane",
	},
	"YMLL": {
		ICAO:      "YMLL",
		Name:      "Millungera Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -19.8582992554,
		Longitude: 141.56199646,
		Timezone:  "Australia/Brisbane",
	},
	"YMLN": {
		ICAO:      "YMLN",
		Name:      "Monolon Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -30.2000007629,
		Longitude: 143.233001709,
		Timezone:  "Australia/Sydney",
	},
	"YMLR": {
		ICAO:      "YMLR",
		Name:      "Muloorina Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -29.2467002869,
		Longitude: 137.9100036621,
		Timezone:  "Australia/Adelaide",
	},
	"YMLS": {
		ICAO:      "YMLS",
		IATA:      "WLE",
		Name:      "Miles Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 304,
		Latitude:  -26.8083000183,
		Longitude: 150.1750030518,
		Timezone:  "Australia/Brisbane",
	},
	"YMLT": {
		ICAO:      "YMLT",
		IATA:      "LST",
		Name:      "Launceston Airport",
		City:      "Launceston",
		State:     "Tasmania",
		Country:   "AU",
		Elevation: 562,
		Latitude:  -41.54529953,
		Longitude: 147.214004517,
		Timezone:  "Australia/Hobart",
	},
	"YMLX": {
		ICAO:      "YMLX",
		Name:      "Milurie Homestead Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -27.5832996368,
		Longitude: 121.7829971313,
		Timezone:  "Australia/Perth",
	},
	"YMMB": {
		ICAO:      "YMMB",
		IATA:      "MBW",
		Name:      "Melbourne Moorabbin Airport",
		City:      "Melbourne",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 50,
		Latitude:  -37.9757995605,
		Longitude: 145.1020050049,
		Timezone:  "Australia/Melbourne",
	},
	"YMMI": {
		ICAO:      "YMMI",
		IATA:      "WUI",
		Name:      "Murrin Murrin Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 1535,
		Latitude:  -28.7052993774,
		Longitude: 121.8909988403,
		Timezone:  "Australia/Perth",
	},
	"YMML": {
		ICAO:      "YMML",
		IATA:      "MEL",
		Name:      "Melbourne International Airport",
		City:      "Melbourne",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 434,
		Latitude:  -37.6733016968,
		Longitude: 144.8430023193,
		Timezone:  "Australia/Melbourne",
	},
	"YMMN": {
		ICAO:      "YMMN",
		Name:      "Millmerran Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -27.8633003235,
		Longitude: 151.2749938965,
		Timezone:  "Australia/Brisbane",
	},
	"YMMT": {
		ICAO:      "YMMT",
		Name:      "Mount Margaret Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -26.9333000183,
		Longitude: 143.3170013428,
		Timezone:  "Australia/Brisbane",
	},
	"YMMU": {
		ICAO:      "YMMU",
		IATA:      "MMM",
		Name:      "Middlemount Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 547,
		Latitude:  -22.8024997711,
		Longitude: 148.7050018311,
		Timezone:  "Australia/Brisbane",
	},
	"YMNA": {
		ICAO:      "YMNA",
		Name:      "Mount Allan Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -22.2749996185,
		Longitude: 132.2169952393,
		Timezone:  "Australia/Darwin",
	},
	"YMNB": {
		ICAO:      "YMNB",
		Name:      "Mount Barry Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -28.2332992554,
		Longitude: 135,
		Timezone:  "Australia/Adelaide",
	},
	"YMND": {
		ICAO:      "YMND",
		IATA:      "MTL",
		Name:      "Maitland Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 85,
		Latitude:  -32.7033004761,
		Longitude: 151.488006592,
		Timezone:  "Australia/Sydney",
	},
	"YMNE": {
		ICAO:      "YMNE",
		IATA:      "WME",
		Name:      "Mount Keith Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 1792,
		Latitude:  -27.2863998413,
		Longitude: 120.5550003052,
		Timezone:  "Australia/Perth",
	},
	"YMNF": {
		ICAO:      "YMNF",
		Name:      "Manfred Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -26.4500007629,
		Longitude: 116.5500030518,
		Timezone:  "Australia/Perth",
	},
	"YMNG": {
		ICAO:      "YMNG",
		Name:      "Mangalore Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 467,
		Latitude:  -36.8883018494,
		Longitude: 145.1840057373,
		Timezone:  "Australia/Melbourne",
	},
	"YMNK": {
		ICAO:      "YMNK",
		IATA:      "ONR",
		Name:      "Monkira Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 107,
		Latitude:  -24.8166999817,
		Longitude: 140.5330047607,
		Timezone:  "Australia/Brisbane",
	},
	"YMNN": {
		ICAO:      "YMNN",
		Name:      "Mount Denison Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -22.1417007446,
		Longitude: 132.0700073242,
		Timezone:  "Australia/Darwin",
	},
	"YMNO": {
		ICAO:      "YMNO",
		Name:      "Maneroo Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -23.3666992188,
		Longitude: 143.8829956055,
		Timezone:  "Australia/Brisbane",
	},
	"YMNP": {
		ICAO:      "YMNP",
		Name:      "Murnpeowie Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -29.5916996002,
		Longitude: 139.0520019531,
		Timezone:  "Australia/Adelaide",
	},
	"YMNS": {
		ICAO:      "YMNS",
		IATA:      "MSF",
		Name:      "Mount Swan Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -22.6000003815,
		Longitude: 135.016998291,
		Timezone:  "Australia/Darwin",
	},
	"YMNT": {
		ICAO:      "YMNT",
		Name:      "Mornington Station Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -17.3936004639,
		Longitude: 126.2289962769,
		Timezone:  "Australia/Perth",
	},
	"YMNW": {
		ICAO:      "YMNW",
		Name:      "Mount Weld Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -28.7716999054,
		Longitude: 122.4400024414,
		Timezone:  "Australia/Perth",
	},
	"YMNX": {
		ICAO:      "YMNX",
		Name:      "Minura Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -28.8833007813,
		Longitude: 121.8000030518,
		Timezone:  "Australia/Perth",
	},
	"YMNY": {
		ICAO:      "YMNY",
		IATA:      "OXY",
		Name:      "Morney Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 104,
		Latitude:  -25.3582992554,
		Longitude: 141.4329986572,
		Timezone:  "Australia/Brisbane",
	},
	"YMOE": {
		ICAO:      "YMOE",
		Name:      "Moonaree Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -31.9661006927,
		Longitude: 135.875,
		Timezone:  "Australia/Adelaide",
	},
	"YMOG": {
		ICAO:      "YMOG",
		IATA:      "MMG",
		Name:      "Mount Magnet Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 1354,
		Latitude:  -28.1161003113,
		Longitude: 117.8420028687,
		Timezone:  "Australia/Perth",
	},
	"YMOM": {
		ICAO:      "YMOM",
		Name:      "Moulamein Airport",
		City:      "Moulamein",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 233,
		Latitude:  -35.0583856071,
		Longitude: 144.021320343,
		Timezone:  "Australia/Sydney",
	},
	"YMOO": {
		ICAO:      "YMOO",
		IATA:      "OOR",
		Name:      "Mooraberree Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 85,
		Latitude:  -25.25,
		Longitude: 140.983001709,
		Timezone:  "Australia/Brisbane",
	},
	"YMOR": {
		ICAO:      "YMOR",
		IATA:      "MRZ",
		Name:      "Moree Airport",
		City:      "Moree",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 701,
		Latitude:  -29.4988994598,
		Longitude: 149.845001221,
		Timezone:  "Australia/Sydney",
	},
	"YMOT": {
		ICAO:      "YMOT",
		IATA:      "MET",
		Name:      "Moreton Airport",
		City:      "Moreton",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -12.4441995621,
		Longitude: 142.638000488,
		Timezone:  "Australia/Brisbane",
	},
	"YMOU": {
		ICAO:      "YMOU",
		Name:      "Moura Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 147,
		Latitude:  -24.611700058,
		Longitude: 149.9949951172,
		Timezone:  "Australia/Brisbane",
	},
	"YMPA": {
		ICAO:      "YMPA",
		IATA:      "MIN",
		Name:      "Minnipa Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 155,
		Latitude:  -32.8432998657,
		Longitude: 135.1450042725,
		Timezone:  "Australia/Adelaide",
	},
	"YMPC": {
		ICAO:      "YMPC",
		Name:      "RAAF Williams Point Cook Base",
		City:      "Point Cook",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 14,
		Latitude:  -37.9322013855,
		Longitude: 144.7530059814,
		Timezone:  "Australia/Melbourne",
	},
	"YMPE": {
		ICAO:      "YMPE",
		Name:      "Mount Cooper Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -20.5167007446,
		Longitude: 146.766998291,
		Timezone:  "Australia/Brisbane",
	},
	"YMPH": {
		ICAO:      "YMPH",
		Name:      "Mount Elephant Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -37.9000015259,
		Longitude: 143.1670074463,
		Timezone:  "Australia/Melbourne",
	},
	"YMPK": {
		ICAO:      "YMPK",
		Name:      "Milton Park Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -23.3666992188,
		Longitude: 133,
		Timezone:  "Australia/Darwin",
	},
	"YMQA": {
		ICAO:      "YMQA",
		IATA:      "MQE",
		Name:      "Marqua Airport",
		City:      "Marqua",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -22.8057994843,
		Longitude: 137.2510070801,
		Timezone:  "Australia/Darwin",
	},
	"YMQD": {
		ICAO:      "YMQD",
		Name:      "Mount Mcquoid Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -33.1100006104,
		Longitude: 151.1380004883,
		Timezone:  "Australia/Sydney",
	},
	"YMRA": {
		ICAO:      "YMRA",
		Name:      "Maralinga Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -30.1632995605,
		Longitude: 131.625,
		Timezone:  "Australia/Adelaide",
	},
	"YMRB": {
		ICAO:      "YMRB",
		IATA:      "MOV",
		Name:      "Moranbah Airport",
		City:      "Moranbah",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 770,
		Latitude:  -22.057800293,
		Longitude: 148.07699585,
		Timezone:  "Australia/Brisbane",
	},
	"YMRE": {
		ICAO:      "YMRE",
		IATA:      "RRE",
		Name:      "Marree Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 50,
		Latitude:  -29.6632995605,
		Longitude: 138.0650024414,
		Timezone:  "Australia/Adelaide",
	},
	"YMRS": {
		ICAO:      "YMRS",
		Name:      "Melrose Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -27.9333000183,
		Longitude: 121.3000030518,
		Timezone:  "Australia/Perth",
	},
	"YMRT": {
		ICAO:      "YMRT",
		Name:      "Mount Garnet Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 657,
		Latitude:  -17.7000007629,
		Longitude: 145.1499938965,
		Timezone:  "Australia/Brisbane",
	},
	"YMRW": {
		ICAO:      "YMRW",
		IATA:      "MWB",
		Name:      "Morawa Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 270,
		Latitude:  -29.2017002106,
		Longitude: 116.0220031738,
		Timezone:  "Australia/Perth",
	},
	"YMRY": {
		ICAO:      "YMRY",
		IATA:      "MYA",
		Name:      "Moruya Airport",
		City:      "Moruya",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 14,
		Latitude:  -35.8978004456,
		Longitude: 150.143997192,
		Timezone:  "Australia/Sydney",
	},
	"YMSF": {
		ICAO:      "YMSF",
		IATA:      "MTD",
		Name:      "Mount Sanford Station Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 231,
		Latitude:  -16.9783000946,
		Longitude: 130.5549926758,
		Timezone:  "Australia/Darwin",
	},
	"YMSK": {
		ICAO:      "YMSK",
		Name:      "Mount Skinner Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -22.1833000183,
		Longitude: 134.1170043945,
		Timezone:  "Australia/Darwin",
	},
	"YMSO": {
		ICAO:      "YMSO",
		Name:      "Mount Sandon Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 1363,
		Latitude:  -31.3899993896,
		Longitude: 151.4100036621,
		Timezone:  "Australia/Sydney",
	},
	"YMSP": {
		ICAO:      "YMSP",
		Name:      "Mount Surprise Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -18.1263008118,
		Longitude: 144.2850036621,
		Timezone:  "Australia/Brisbane",
	},
	"YMSS": {
		ICAO:      "YMSS",
		Name:      "Murchison Shire Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -26.9167003632,
		Longitude: 115.983001709,
		Timezone:  "Australia/Perth",
	},
	"YMTA": {
		ICAO:      "YMTA",
		Name:      "Mittebah Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -18.7999992371,
		Longitude: 137,
		Timezone:  "Australia/Darwin",
	},
	"YMTB": {
		ICAO:      "YMTB",
		IATA:      "UTB",
		Name:      "Muttaburra Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 230,
		Latitude:  -22.5832996368,
		Longitude: 144.5330047607,
		Timezone:  "Australia/Brisbane",
	},
	"YMTC": {
		ICAO:      "YMTC",
		Name:      "Mount Clarence Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -28.8332996368,
		Longitude: 134.358001709,
		Timezone:  "Australia/Adelaide",
	},
	"YMTG": {
		ICAO:      "YMTG",
		IATA:      "MGB",
		Name:      "Mount Gambier Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 212,
		Latitude:  -37.7456016541,
		Longitude: 140.7850036621,
		Timezone:  "Australia/Adelaide",
	},
	"YMTI": {
		ICAO:      "YMTI",
		IATA:      "ONG",
		Name:      "Mornington Island Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 33,
		Latitude:  -16.6625003815,
		Longitude: 139.1779937744,
		Timezone:  "Australia/Brisbane",
	},
	"YMTO": {
		ICAO:      "YMTO",
		IATA:      "MNQ",
		Name:      "Monto Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 757,
		Latitude:  -24.885799408,
		Longitude: 151.1000061035,
		Timezone:  "Australia/Brisbane",
	},
	"YMTP": {
		ICAO:      "YMTP",
		Name:      "Mount Hope Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -34.1366996765,
		Longitude: 135.3379974365,
		Timezone:  "Australia/Adelaide",
	},
	"YMTW": {
		ICAO:      "YMTW",
		Name:      "Martins Well Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -31.4736003876,
		Longitude: 139.1109924316,
		Timezone:  "Australia/Adelaide",
	},
	"YMTX": {
		ICAO:      "YMTX",
		Name:      "Mount Dare Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -26.0617008209,
		Longitude: 135.2469940186,
		Timezone:  "Australia/Adelaide",
	},
	"YMTZ": {
		ICAO:      "YMTZ",
		Name:      "Mount Elizabeth Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -16.4333000183,
		Longitude: 126.0999984741,
		Timezone:  "Australia/Perth",
	},
	"YMUA": {
		ICAO:      "YMUA",
		Name:      "Monduran Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -24.875,
		Longitude: 151.9170074463,
		Timezone:  "Australia/Brisbane",
	},
	"YMUC": {
		ICAO:      "YMUC",
		IATA:      "MUQ",
		Name:      "Muccan Station Airport",
		City:      "Muccan Station",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 300,
		Latitude:  -20.6588993073,
		Longitude: 120.0670013428,
		Timezone:  "Australia/Perth",
	},
	"YMUE": {
		ICAO:      "YMUE",
		Name:      "Mount Borradale Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -12.0982999802,
		Longitude: 132.9069976807,
		Timezone:  "Australia/Darwin",
	},
	"YMUG": {
		ICAO:      "YMUG",
		IATA:      "MNE",
		Name:      "Mungeranie Airport",
		City:      "Mungeranie",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -28.0091991425,
		Longitude: 138.6569976807,
		Timezone:  "Australia/Adelaide",
	},
	"YMUI": {
		ICAO:      "YMUI",
		IATA:      "MYI",
		Name:      "Murray Island Airport",
		City:      "Murray Island",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -9.9166698456,
		Longitude: 144.0549926758,
		Timezone:  "Australia/Brisbane",
	},
	"YMUJ": {
		ICAO:      "YMUJ",
		Name:      "Munjina Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -22.3661003113,
		Longitude: 118.6809997559,
		Timezone:  "Australia/Perth",
	},
	"YMUK": {
		ICAO:      "YMUK",
		IATA:      "MVK",
		Name:      "Mulka Airport",
		City:      "Mulka",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -28.3477993011,
		Longitude: 138.6499938965,
		Timezone:  "Australia/Adelaide",
	},
	"YMUL": {
		ICAO:      "YMUL",
		Name:      "Murray Field",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 56,
		Latitude:  -32.5083007813,
		Longitude: 115.8420028687,
		Timezone:  "Australia/Perth",
	},
	"YMUP": {
		ICAO:      "YMUP",
		IATA:      "MUP",
		Name:      "Mulga Park Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -25.8999996185,
		Longitude: 131.6670074463,
		Timezone:  "Australia/Darwin",
	},
	"YMUR": {
		ICAO:      "YMUR",
		Name:      "Murwillumbah Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -28.3316993713,
		Longitude: 153.4129943848,
		Timezone:  "Australia/Sydney",
	},
	"YMUX": {
		ICAO:      "YMUX",
		Name:      "Mileura Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -26.3666992188,
		Longitude: 117.3330001831,
		Timezone:  "Australia/Perth",
	},
	"YMVG": {
		ICAO:      "YMVG",
		IATA:      "MKV",
		Name:      "Mount Cavenagh Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -25.9666996002,
		Longitude: 133.1999969482,
		Timezone:  "Australia/Darwin",
	},
	"YMVH": {
		ICAO:      "YMVH",
		Name:      "Marvel Loch Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -31.4666996002,
		Longitude: 119.5329971313,
		Timezone:  "Australia/Perth",
	},
	"YMVM": {
		ICAO:      "YMVM",
		Name:      "Mangrove Mountain Airport",
		City:      "Mangrove Mountain",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 920,
		Latitude:  -33.2851219177,
		Longitude: 151.2127380371,
		Timezone:  "Australia/Sydney",
	},
	"YMVN": {
		ICAO:      "YMVN",
		Name:      "Morven Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -26.3999996185,
		Longitude: 147.1000061035,
		Timezone:  "Australia/Brisbane",
	},
	"YMVR": {
		ICAO:      "YMVR",
		Name:      "Mount Vernon Station Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -24.2332992554,
		Longitude: 118.233001709,
		Timezone:  "Australia/Perth",
	},
	"YMVY": {
		ICAO:      "YMVY",
		Name:      "Mount Valley Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -14.0832996368,
		Longitude: 133.8170013428,
		Timezone:  "Australia/Darwin",
	},
	"YMWA": {
		ICAO:      "YMWA",
		IATA:      "MXU",
		Name:      "Mullewa Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 290,
		Latitude:  -28.4750003815,
		Longitude: 115.516998291,
		Timezone:  "Australia/Perth",
	},
	"YMWE": {
		ICAO:      "YMWE",
		Name:      "Mount Wedge Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 559,
		Latitude:  -22.7371997833,
		Longitude: 132.1510009766,
		Timezone:  "Australia/Darwin",
	},
	"YMWM": {
		ICAO:      "YMWM",
		Name:      "Mount William Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -37.2954958244,
		Longitude: 142.603397369,
		Timezone:  "Australia/Melbourne",
	},
	"YMWO": {
		ICAO:      "YMWO",
		Name:      "Mahanewo Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -31.7182998657,
		Longitude: 136.4329986572,
		Timezone:  "Australia/Adelaide",
	},
	"YMWT": {
		ICAO:      "YMWT",
		IATA:      "MWT",
		Name:      "Moolawatana Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -29.9167003632,
		Longitude: 139.75,
		Timezone:  "Australia/Adelaide",
	},
	"YMWX": {
		ICAO:      "YMWX",
		IATA:      "MXD",
		Name:      "Marion Downs Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -23.3666992188,
		Longitude: 139.6670074463,
		Timezone:  "Australia/Brisbane",
	},
	"YMYB": {
		ICAO:      "YMYB",
		IATA:      "MBH",
		Name:      "Maryborough Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 38,
		Latitude:  -25.513299942,
		Longitude: 152.7149963379,
		Timezone:  "Australia/Brisbane",
	},
	"YMYH": {
		ICAO:      "YMYH",
		Name:      "Mallapunyah Springs Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -16.9666996002,
		Longitude: 135.7830047607,
		Timezone:  "Australia/Darwin",
	},
	"YMYI": {
		ICAO:      "YMYI",
		Name:      "Marymia Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -25.0932998657,
		Longitude: 119.7549972534,
		Timezone:  "Australia/Perth",
	},
	"YMYR": {
		ICAO:      "YMYR",
		IATA:      "MYO",
		Name:      "Myroodan Station Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -18.1166992188,
		Longitude: 124.266998291,
		Timezone:  "Australia/Perth",
	},
	"YMYT": {
		ICAO:      "YMYT",
		IATA:      "RTY",
		Name:      "Merty Merty Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -28.5832996368,
		Longitude: 140.3170013428,
		Timezone:  "Australia/Adelaide",
	},
	"YMYV": {
		ICAO:      "YMYV",
		Name:      "Maryvale Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -24.6599998474,
		Longitude: 134.0319976807,
		Timezone:  "Australia/Darwin",
	},
	"YMYW": {
		ICAO:      "YMYW",
		Name:      "Murray Downs Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -21.0482997894,
		Longitude: 134.6829986572,
		Timezone:  "Australia/Darwin",
	},
	"YMYY": {
		ICAO:      "YMYY",
		Name:      "Mary Valley Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -15.0500001907,
		Longitude: 143.75,
		Timezone:  "Australia/Brisbane",
	},
	"YMZI": {
		ICAO:      "YMZI",
		Name:      "Menzies Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -29.6667003632,
		Longitude: 121.016998291,
		Timezone:  "Australia/Perth",
	},
	"YNAB": {
		ICAO:      "YNAB",
		Name:      "Nabarlek Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -12.3083000183,
		Longitude: 133.31300354,
		Timezone:  "Australia/Darwin",
	},
	"YNAN": {
		ICAO:      "YNAN",
		Name:      "Nanango Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -26.6900005341,
		Longitude: 151.983001709,
		Timezone:  "Australia/Brisbane",
	},
	"YNAP": {
		ICAO:      "YNAP",
		IATA:      "NMR",
		Name:      "Nappa Merrie Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 55,
		Latitude:  -27.5583000183,
		Longitude: 141.1329956055,
		Timezone:  "Australia/Brisbane",
	},
	"YNAR": {
		ICAO:      "YNAR",
		IATA:      "NRA",
		Name:      "Narrandera Airport",
		City:      "Narrandera",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 474,
		Latitude:  -34.7022018433,
		Longitude: 146.511993408,
		Timezone:  "Australia/Sydney",
	},
	"YNAU": {
		ICAO:      "YNAU",
		Name:      "Nannup Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -34.0332984924,
		Longitude: 115.733001709,
		Timezone:  "Australia/Perth",
	},
	"YNBR": {
		ICAO:      "YNBR",
		IATA:      "NAA",
		Name:      "Narrabri Airport",
		City:      "Narrabri",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 788,
		Latitude:  -30.3192005157,
		Longitude: 149.82699585,
		Timezone:  "Australia/Sydney",
	},
	"YNCD": {
		ICAO:      "YNCD",
		Name:      "Noccundra Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -27.7999992371,
		Longitude: 142.5829925537,
		Timezone:  "Australia/Brisbane",
	},
	"YNCS": {
		ICAO:      "YNCS",
		Name:      "New Crown Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -25.6667003632,
		Longitude: 134.8329925537,
		Timezone:  "Australia/Darwin",
	},
	"YNDG": {
		ICAO:      "YNDG",
		Name:      "Newdegate Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -33.1166992188,
		Longitude: 119,
		Timezone:  "Australia/Perth",
	},
	"YNDR": {
		ICAO:      "YNDR",
		Name:      "Nundroo Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -31.7800006866,
		Longitude: 132.2230072021,
		Timezone:  "Australia/Adelaide",
	},
	"YNDS": {
		ICAO:      "YNDS",
		Name:      "Natal Downs Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -21.0832996368,
		Longitude: 146.1499938965,
		Timezone:  "Australia/Brisbane",
	},
	"YNES": {
		ICAO:      "YNES",
		Name:      "Nelson Springs Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -17.2999992371,
		Longitude: 129.2830047607,
		Timezone:  "Australia/Darwin",
	},
	"YNGB": {
		ICAO:      "YNGB",
		Name:      "Nagambie Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -36.7383003235,
		Longitude: 145.125,
		Timezone:  "Australia/Melbourne",
	},
	"YNGU": {
		ICAO:      "YNGU",
		IATA:      "RPM",
		Name:      "Ngukurr Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 45,
		Latitude:  -14.7228002548,
		Longitude: 134.7469940186,
		Timezone:  "Australia/Darwin",
	},
	"YNHE": {
		ICAO:      "YNHE",
		Name:      "Newhaven Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -38.5332984924,
		Longitude: 145.3170013428,
		Timezone:  "Australia/Melbourne",
	},
	"YNHL": {
		ICAO:      "YNHL",
		Name:      "Nhill Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 454,
		Latitude:  -36.3097000122,
		Longitude: 141.6410064697,
		Timezone:  "Australia/Melbourne",
	},
	"YNHP": {
		ICAO:      "YNHP",
		Name:      "Northampton Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -28.3500003815,
		Longitude: 114.6829986572,
		Timezone:  "Australia/Perth",
	},
	"YNHS": {
		ICAO:      "YNHS",
		IATA:      "NBH",
		Name:      "Nambucca Heads Airport",
		City:      "Nambucca Heads",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -30.6499996185,
		Longitude: 153,
		Timezone:  "Australia/Sydney",
	},
	"YNHV": {
		ICAO:      "YNHV",
		Name:      "New Haven Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -22.7283000946,
		Longitude: 131.1450042725,
		Timezone:  "Australia/Darwin",
	},
	"YNIC": {
		ICAO:      "YNIC",
		IATA:      "NLS",
		Name:      "Nicholson Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -18.0499992371,
		Longitude: 128.8999938965,
		Timezone:  "Australia/Perth",
	},
	"YNIG": {
		ICAO:      "YNIG",
		Name:      "Nonning Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -32.521900177,
		Longitude: 136.4940032959,
		Timezone:  "Australia/Adelaide",
	},
	"YNIN": {
		ICAO:      "YNIN",
		Name:      "Meningie Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -35.7000007629,
		Longitude: 139.3329925537,
		Timezone:  "Australia/Adelaide",
	},
	"YNKA": {
		ICAO:      "YNKA",
		Name:      "Noonkanbah Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -18.5,
		Longitude: 124.8330001831,
		Timezone:  "Australia/Perth",
	},
	"YNMN": {
		ICAO:      "YNMN",
		Name:      "New Moon Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -19.2000007629,
		Longitude: 145.766998291,
		Timezone:  "Australia/Brisbane",
	},
	"YNNT": {
		ICAO:      "YNNT",
		Name:      "Nantarra Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -22.5333003998,
		Longitude: 115.5,
		Timezone:  "Australia/Perth",
	},
	"YNOC": {
		ICAO:      "YNOC",
		Name:      "Nockatunga Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -27.7166996002,
		Longitude: 142.7120056152,
		Timezone:  "Australia/Brisbane",
	},
	"YNOF": {
		ICAO:      "YNOF",
		Name:      "Norfolk Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -19.3332996368,
		Longitude: 138.3329925537,
		Timezone:  "Australia/Brisbane",
	},
	"YNON": {
		ICAO:      "YNON",
		Name:      "Noondoonia Homestead Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -32.2832984924,
		Longitude: 123.6669998169,
		Timezone:  "Australia/Perth",
	},
	"YNOO": {
		ICAO:      "YNOO",
		Name:      "Nooyeah Downs Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -28.0499992371,
		Longitude: 143.5829925537,
		Timezone:  "Australia/Brisbane",
	},
	"YNOR": {
		ICAO:      "YNOR",
		Name:      "Norwood Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -34.25,
		Longitude: 144.3329925537,
		Timezone:  "Australia/Sydney",
	},
	"YNPA": {
		ICAO:      "YNPA",
		Name:      "Nilpinna Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -28.4832992554,
		Longitude: 135.8999938965,
		Timezone:  "Australia/Adelaide",
	},
	"YNPB": {
		ICAO:      "YNPB",
		Name:      "Napperby Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -22.5333003998,
		Longitude: 133.7630004883,
		Timezone:  "Australia/Darwin",
	},
	"YNPD": {
		ICAO:      "YNPD",
		Name:      "Napier Downs Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -17.3166999817,
		Longitude: 124.8499984741,
		Timezone:  "Australia/Perth",
	},
	"YNPU": {
		ICAO:      "YNPU",
		Name:      "Nepabunna Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -30.6033000946,
		Longitude: 138.9479980469,
		Timezone:  "Australia/Adelaide",
	},
	"YNRB": {
		ICAO:      "YNRB",
		Name:      "Narembeen Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 305,
		Latitude:  -32.1166992188,
		Longitude: 118.4169998169,
		Timezone:  "Australia/Perth",
	},
	"YNRC": {
		ICAO:      "YNRC",
		IATA:      "NAC",
		Name:      "Naracoorte Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 169,
		Latitude:  -36.9852981567,
		Longitude: 140.7250061035,
		Timezone:  "Australia/Adelaide",
	},
	"YNRG": {
		ICAO:      "YNRG",
		IATA:      "NRG",
		Name:      "Narrogin Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 329,
		Latitude:  -32.9300003052,
		Longitude: 117.0800018311,
		Timezone:  "Australia/Perth",
	},
	"YNRL": {
		ICAO:      "YNRL",
		Name:      "Naryilco Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -28.5499992371,
		Longitude: 141.9170074463,
		Timezone:  "Australia/Brisbane",
	},
	"YNRM": {
		ICAO:      "YNRM",
		IATA:      "QRM",
		Name:      "Narromine Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 782,
		Latitude:  -32.2146987915,
		Longitude: 148.2250061035,
		Timezone:  "Australia/Sydney",
	},
	"YNRR": {
		ICAO:      "YNRR",
		Name:      "Nyrripi Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -22.6452999115,
		Longitude: 130.5650024414,
		Timezone:  "Australia/Darwin",
	},
	"YNRV": {
		ICAO:      "YNRV",
		IATA:      "RVT",
		Name:      "Ravensthorpe Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 197,
		Latitude:  -33.7971992493,
		Longitude: 120.208000183,
		Timezone:  "Australia/Perth",
	},
	"YNSH": {
		ICAO:      "YNSH",
		IATA:      "NSV",
		Name:      "Noosa Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 1,
		Latitude:  -26.4232997894,
		Longitude: 153.06300354,
		Timezone:  "Australia/Brisbane",
	},
	"YNSM": {
		ICAO:      "YNSM",
		IATA:      "NSM",
		Name:      "Norseman Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 263,
		Latitude:  -32.2099990845,
		Longitude: 121.7549972534,
		Timezone:  "Australia/Perth",
	},
	"YNTI": {
		ICAO:      "YNTI",
		Name:      "Abrolhos North Island",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 1,
		Latitude:  -28.2998166667,
		Longitude: 113.5958,
		Timezone:  "Australia/Perth",
	},
	"YNTM": {
		ICAO:      "YNTM",
		Name:      "Northam Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 152,
		Latitude:  -31.6299991608,
		Longitude: 116.6829986572,
		Timezone:  "Australia/Perth",
	},
	"YNTN": {
		ICAO:      "YNTN",
		IATA:      "NTN",
		Name:      "Normanton Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 73,
		Latitude:  -17.683599472,
		Longitude: 141.0700073242,
		Timezone:  "Australia/Brisbane",
	},
	"YNUB": {
		ICAO:      "YNUB",
		IATA:      "NUR",
		Name:      "Nullabor Motel Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 67,
		Latitude:  -31.4416999817,
		Longitude: 130.9019927979,
		Timezone:  "Australia/Adelaide",
	},
	"YNUD": {
		ICAO:      "YNUD",
		Name:      "Nammuldi Mine Airstrip",
		City:      "Nammuldi Mine",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 267,
		Latitude:  -22.39186,
		Longitude: 117.37623,
		Timezone:  "Australia/Perth",
	},
	"YNUE": {
		ICAO:      "YNUE",
		Name:      "Numery Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -23.9832992554,
		Longitude: 135.3999938965,
		Timezone:  "Australia/Darwin",
	},
	"YNUJ": {
		ICAO:      "YNUJ",
		Name:      "Nudjaburra Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -17.9666996002,
		Longitude: 137.9329986572,
		Timezone:  "Australia/Darwin",
	},
	"YNUL": {
		ICAO:      "YNUL",
		IATA:      "NLL",
		Name:      "Nullagine Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 381,
		Latitude:  -21.9132995605,
		Longitude: 120.1979980469,
		Timezone:  "Australia/Perth",
	},
	"YNUM": {
		ICAO:      "YNUM",
		IATA:      "NUB",
		Name:      "Numbulwar Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 31,
		Latitude:  -14.2716999054,
		Longitude: 135.7169952393,
		Timezone:  "Australia/Darwin",
	},
	"YNUT": {
		ICAO:      "YNUT",
		Name:      "Nutwood Downs Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -15.8166999817,
		Longitude: 134.1499938965,
		Timezone:  "Australia/Darwin",
	},
	"YNVE": {
		ICAO:      "YNVE",
		Name:      "Navarre Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -36.9166984558,
		Longitude: 143.1000061035,
		Timezone:  "Australia/Melbourne",
	},
	"YNVL": {
		ICAO:      "YNVL",
		Name:      "Normanville Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -35.7832984924,
		Longitude: 143.6999969482,
		Timezone:  "Australia/Melbourne",
	},
	"YNVR": {
		ICAO:      "YNVR",
		Name:      "Navarra Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -24.8500003815,
		Longitude: 143.6999969482,
		Timezone:  "Australia/Brisbane",
	},
	"YNWL": {
		ICAO:      "YNWL",
		Name:      "North Well Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -30.8460998535,
		Longitude: 135.3090057373,
		Timezone:  "Australia/Adelaide",
	},
	"YNWN": {
		ICAO:      "YNWN",
		IATA:      "ZNE",
		Name:      "Newman Airport",
		City:      "Newman",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 1724,
		Latitude:  -23.4178009033,
		Longitude: 119.803001404,
		Timezone:  "Australia/Perth",
	},
	"YNWT": {
		ICAO:      "YNWT",
		Name:      "Narwietooma Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -23.2332992554,
		Longitude: 132.6329956055,
		Timezone:  "Australia/Darwin",
	},
	"YNYG": {
		ICAO:      "YNYG",
		Name:      "Nyang Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -23.0333003998,
		Longitude: 115.0329971313,
		Timezone:  "Australia/Perth",
	},
	"YNYM": {
		ICAO:      "YNYM",
		Name:      "Nymagee Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -32.0499992371,
		Longitude: 146.3170013428,
		Timezone:  "Australia/Sydney",
	},
	"YNYN": {
		ICAO:      "YNYN",
		IATA:      "NYN",
		Name:      "Nyngan Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 569,
		Latitude:  -31.5510997772,
		Longitude: 147.2030029297,
		Timezone:  "Australia/Sydney",
	},
	"YNYP": {
		ICAO:      "YNYP",
		Name:      "Nypari Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -26.2000007629,
		Longitude: 130.233001709,
		Timezone:  "Australia/Adelaide",
	},
	"YOAD": {
		ICAO:      "YOAD",
		Name:      "Old Andado Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -25.3833007813,
		Longitude: 135.4170074463,
		Timezone:  "Australia/Darwin",
	},
	"YOAP": {
		ICAO:      "YOAP",
		Name:      "One Arm Point Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -16.4333000183,
		Longitude: 123.0500030518,
		Timezone:  "Australia/Perth",
	},
	"YOAS": {
		ICAO:      "YOAS",
		Name:      "The Oaks Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 909,
		Latitude:  -34.0839004517,
		Longitude: 150.5590057373,
		Timezone:  "Australia/Sydney",
	},
	"YOAY": {
		ICAO:      "YOAY",
		Name:      "Oaky Creek Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 177,
		Latitude:  -23.0599994659,
		Longitude: 148.4949951172,
		Timezone:  "Australia/Brisbane",
	},
	"YOBR": {
		ICAO:      "YOBR",
		Name:      "Old Bar Heritage Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 10,
		Latitude:  -31.9652996063,
		Longitude: 152.591003418,
		Timezone:  "Australia/Sydney",
	},
	"YODA": {
		ICAO:      "YODA",
		Name:      "Ooldea Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -30.4566993713,
		Longitude: 131.8229980469,
		Timezone:  "Australia/Adelaide",
	},
	"YODL": {
		ICAO:      "YODL",
		Name:      "Ourdel Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -25.3500003815,
		Longitude: 142.6329956055,
		Timezone:  "Australia/Brisbane",
	},
	"YOEN": {
		ICAO:      "YOEN",
		IATA:      "OPI",
		Name:      "Oenpelli Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 30,
		Latitude:  -12.3249998093,
		Longitude: 133.0059967041,
		Timezone:  "Australia/Darwin",
	},
	"YOIT": {
		ICAO:      "YOIT",
		Name:      "Orielton Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -22.0666999817,
		Longitude: 144.1499938965,
		Timezone:  "Australia/Brisbane",
	},
	"YOKE": {
		ICAO:      "YOKE",
		Name:      "Oakvale Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -22.2667007446,
		Longitude: 145.983001709,
		Timezone:  "Australia/Brisbane",
	},
	"YOKV": {
		ICAO:      "YOKV",
		Name:      "Oak Valley Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -29.5167007446,
		Longitude: 130.8670043945,
		Timezone:  "Australia/Adelaide",
	},
	"YOLA": {
		ICAO:      "YOLA",
		IATA:      "XCO",
		Name:      "Colac Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 137,
		Latitude:  -38.2867012024,
		Longitude: 143.6799926758,
		Timezone:  "Australia/Melbourne",
	},
	"YOLD": {
		ICAO:      "YOLD",
		IATA:      "OLP",
		Name:      "Olympic Dam Airport",
		City:      "Olympic Dam",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 343,
		Latitude:  -30.4850006104,
		Longitude: 136.876998901,
		Timezone:  "Australia/Adelaide",
	},
	"YOLW": {
		ICAO:      "YOLW",
		IATA:      "ONS",
		Name:      "Onslow Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 7,
		Latitude:  -21.6683006287,
		Longitude: 115.1129989624,
		Timezone:  "Australia/Perth",
	},
	"YOLY": {
		ICAO:      "YOLY",
		Name:      "Oxley Station Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 518,
		Latitude:  -31.013299942,
		Longitude: 147.7109985352,
		Timezone:  "Australia/Sydney",
	},
	"YOMI": {
		ICAO:      "YOMI",
		Name:      "Omicron Station Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -28.7667007446,
		Longitude: 141.1999969482,
		Timezone:  "Australia/Brisbane",
	},
	"YOMO": {
		ICAO:      "YOMO",
		Name:      "Omeo Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -37.096698761,
		Longitude: 147.5930023193,
		Timezone:  "Australia/Melbourne",
	},
	"YOOD": {
		ICAO:      "YOOD",
		IATA:      "ODD",
		Name:      "Oodnadatta Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 118,
		Latitude:  -27.5617008209,
		Longitude: 135.4470062256,
		Timezone:  "Australia/Adelaide",
	},
	"YOOK": {
		ICAO:      "YOOK",
		Name:      "Cook Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -30.6182994843,
		Longitude: 130.4060058594,
		Timezone:  "Australia/Adelaide",
	},
	"YOOM": {
		ICAO:      "YOOM",
		IATA:      "MOO",
		Name:      "Moomba Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 143,
		Latitude:  -28.0993995667,
		Longitude: 140.1970062256,
		Timezone:  "Australia/Adelaide",
	},
	"YOOO": {
		ICAO:      "YOOO",
		Name:      "Mooloola Homestead Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -16.3332996368,
		Longitude: 131.5,
		Timezone:  "Australia/Darwin",
	},
	"YORA": {
		ICAO:      "YORA",
		Name:      "Moonerah Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -31.6893997192,
		Longitude: 126.5859985352,
		Timezone:  "Australia/Eucla",
	},
	"YORB": {
		ICAO:      "YORB",
		IATA:      "RBS",
		Name:      "Orbost Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 28,
		Latitude:  -37.7900009155,
		Longitude: 148.6100006104,
		Timezone:  "Australia/Melbourne",
	},
	"YORG": {
		ICAO:      "YORG",
		IATA:      "OAG",
		Name:      "Orange Airport",
		City:      "Orange",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 3115,
		Latitude:  -33.3816986084,
		Longitude: 149.132995605,
		Timezone:  "Australia/Sydney",
	},
	"YORL": {
		ICAO:      "YORL",
		Name:      "Orleans Farm Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -33.7832984924,
		Longitude: 122.9499969482,
		Timezone:  "Australia/Perth",
	},
	"YORR": {
		ICAO:      "YORR",
		Name:      "Orroroo Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 381,
		Latitude:  -32.7733001709,
		Longitude: 138.6649932861,
		Timezone:  "Australia/Adelaide",
	},
	"YORT": {
		ICAO:      "YORT",
		Name:      "Ooratippra Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -21.9057998657,
		Longitude: 136.0690002441,
		Timezone:  "Australia/Darwin",
	},
	"YORV": {
		ICAO:      "YORV",
		IATA:      "ODR",
		Name:      "Ord River Airport",
		City:      "Ord River",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 390,
		Latitude:  -17.3407993317,
		Longitude: 128.9120025635,
		Timezone:  "Australia/Perth",
	},
	"YORW": {
		ICAO:      "YORW",
		Name:      "Orient Well Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -29.1991996765,
		Longitude: 121.43800354,
		Timezone:  "Australia/Perth",
	},
	"YOSB": {
		ICAO:      "YOSB",
		IATA:      "OSO",
		Name:      "Osborne Mine Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 285,
		Latitude:  -22.0816993713,
		Longitude: 140.554992676,
		Timezone:  "Australia/Brisbane",
	},
	"YOTN": {
		ICAO:      "YOTN",
		Name:      "Ootann Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -17.4333000183,
		Longitude: 144.6329956055,
		Timezone:  "Australia/Brisbane",
	},
	"YOUN": {
		ICAO:      "YOUN",
		Name:      "Youanmi Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -28.6149997711,
		Longitude: 118.8420028687,
		Timezone:  "Australia/Perth",
	},
	"YOUY": {
		ICAO:      "YOUY",
		IATA:      "OYN",
		Name:      "Ouyen Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -35.0890124268,
		Longitude: 142.354488373,
		Timezone:  "Australia/Melbourne",
	},
	"YPAC": {
		ICAO:      "YPAC",
		Name:      "Pacific Haven Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -25.236700058,
		Longitude: 152.5429992676,
		Timezone:  "Australia/Brisbane",
	},
	"YPAD": {
		ICAO:      "YPAD",
		IATA:      "ADL",
		Name:      "Adelaide International Airport",
		City:      "Adelaide",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 20,
		Latitude:  -34.9449996948,
		Longitude: 138.5310058594,
		Timezone:  "Australia/Adelaide",
	},
	"YPAG": {
		ICAO:      "YPAG",
		IATA:      "PUG",
		Name:      "Port Augusta Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 56,
		Latitude:  -32.5069007874,
		Longitude: 137.7169952393,
		Timezone:  "Australia/Adelaide",
	},
	"YPAM": {
		ICAO:      "YPAM",
		IATA:      "PMK",
		Name:      "Palm Island Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 28,
		Latitude:  -18.7553005219,
		Longitude: 146.5809936523,
		Timezone:  "Australia/Brisbane",
	},
	"YPAY": {
		ICAO:      "YPAY",
		Name:      "Papunya Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 620,
		Latitude:  -23.2467002869,
		Longitude: 131.9029998779,
		Timezone:  "Australia/Darwin",
	},
	"YPBH": {
		ICAO:      "YPBH",
		Name:      "Peterborough Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -38.5999984741,
		Longitude: 142.9170074463,
		Timezone:  "Australia/Melbourne",
	},
	"YPBO": {
		ICAO:      "YPBO",
		IATA:      "PBO",
		Name:      "Paraburdoo Airport",
		City:      "Paraburdoo",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 1406,
		Latitude:  -23.1711006165,
		Longitude: 117.745002747,
		Timezone:  "Australia/Perth",
	},
	"YPCC": {
		ICAO:      "YPCC",
		IATA:      "CCK",
		Name:      "Cocos (Keeling) Islands Airport",
		City:      "Cocos (Keeling) Islands",
		Country:   "CC",
		Elevation: 10,
		Latitude:  -12.1883001328,
		Longitude: 96.8339004517,
		Timezone:  "Indian/Cocos",
	},
	"YPCE": {
		ICAO:      "YPCE",
		Name:      "Pooncarie Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 50,
		Latitude:  -33.3666992188,
		Longitude: 142.5879974365,
		Timezone:  "Australia/Sydney",
	},
	"YPCH": {
		ICAO:      "YPCH",
		Name:      "Patchewollock Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -35.3666992188,
		Longitude: 142.1499938965,
		Timezone:  "Australia/Melbourne",
	},
	"YPCS": {
		ICAO:      "YPCS",
		Name:      "Pinnacles Homestead Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -28.2000007629,
		Longitude: 120.4329986572,
		Timezone:  "Australia/Perth",
	},
	"YPDI": {
		ICAO:      "YPDI",
		IATA:      "PDE",
		Name:      "Pandie Pandie Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -26.1166992188,
		Longitude: 139.3999938965,
		Timezone:  "Australia/Adelaide",
	},
	"YPDN": {
		ICAO:      "YPDN",
		IATA:      "DRW",
		Name:      "Darwin International Airport",
		City:      "Darwin",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 103,
		Latitude:  -12.4146995544,
		Longitude: 130.8769989014,
		Timezone:  "Australia/Darwin",
	},
	"YPDO": {
		ICAO:      "YPDO",
		IATA:      "PRD",
		Name:      "Pardoo Airport",
		City:      "Pardoo",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -20.1175003052,
		Longitude: 119.5899963379,
		Timezone:  "Australia/Perth",
	},
	"YPDY": {
		ICAO:      "YPDY",
		Name:      "Padthaway Station Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -36.6166992188,
		Longitude: 140.483001709,
		Timezone:  "Australia/Adelaide",
	},
	"YPEA": {
		ICAO:      "YPEA",
		Name:      "RAAF Base Pearce",
		City:      "Bullsbrook",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 149,
		Latitude:  -31.6678009033,
		Longitude: 116.0149993896,
		Timezone:  "Australia/Perth",
	},
	"YPEC": {
		ICAO:      "YPEC",
		IATA:      "BEO",
		Name:      "Aeropelican Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 2,
		Latitude:  -33.0666999817,
		Longitude: 151.6479949951,
		Timezone:  "Australia/Sydney",
	},
	"YPED": {
		ICAO:      "YPED",
		Name:      "RAAF Base Edinburgh",
		City:      "Adelaide",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 67,
		Latitude:  -34.7024993896,
		Longitude: 138.6210021973,
		Timezone:  "Australia/Adelaide",
	},
	"YPEF": {
		ICAO:      "YPEF",
		Name:      "Penfield Airfield",
		City:      "Sunbury",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 1150,
		Latitude:  -37.5133018494,
		Longitude: 144.697998047,
		Timezone:  "Australia/Melbourne",
	},
	"YPFT": {
		ICAO:      "YPFT",
		Name:      "Cooma/Polo Flat (Unlic) Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 823,
		Latitude:  -36.2299995422,
		Longitude: 149.1499938965,
		Timezone:  "Australia/Sydney",
	},
	"YPGH": {
		ICAO:      "YPGH",
		Name:      "Pigeon Hole Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -16.8166999817,
		Longitude: 131.2169952393,
		Timezone:  "Australia/Darwin",
	},
	"YPGV": {
		ICAO:      "YPGV",
		IATA:      "GOV",
		Name:      "Gove Airport",
		City:      "Nhulunbuy",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 192,
		Latitude:  -12.2693996429,
		Longitude: 136.817993164,
		Timezone:  "Australia/Darwin",
	},
	"YPID": {
		ICAO:      "YPID",
		Name:      "Phillip Island Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 13,
		Latitude:  -38.5233001709,
		Longitude: 145.3269958496,
		Timezone:  "Australia/Melbourne",
	},
	"YPIN": {
		ICAO:      "YPIN",
		Name:      "Pinnacle Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -15.6667003632,
		Longitude: 143.5500030518,
		Timezone:  "Australia/Brisbane",
	},
	"YPIR": {
		ICAO:      "YPIR",
		IATA:      "PPI",
		Name:      "Port Pirie Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 40,
		Latitude:  -33.238899231,
		Longitude: 137.9949951172,
		Timezone:  "Australia/Adelaide",
	},
	"YPIX": {
		ICAO:      "YPIX",
		Name:      "Pia Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -27.1916999817,
		Longitude: 116.2919998169,
		Timezone:  "Australia/Perth",
	},
	"YPIY": {
		ICAO:      "YPIY",
		Name:      "Pingelly Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -32.5400009155,
		Longitude: 117.0729980469,
		Timezone:  "Australia/Perth",
	},
	"YPJI": {
		ICAO:      "YPJI",
		Name:      "Perenjori Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -29.4333000183,
		Longitude: 116.2829971313,
		Timezone:  "Australia/Perth",
	},
	"YPJT": {
		ICAO:      "YPJT",
		IATA:      "JAD",
		Name:      "Perth Jandakot Airport",
		City:      "Perth",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 99,
		Latitude:  -32.0974998474,
		Longitude: 115.8809967041,
		Timezone:  "Australia/Perth",
	},
	"YPKA": {
		ICAO:      "YPKA",
		IATA:      "KTA",
		Name:      "Karratha Airport",
		City:      "Karratha",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 29,
		Latitude:  -20.7122001648,
		Longitude: 116.773002625,
		Timezone:  "Australia/Perth",
	},
	"YPKE": {
		ICAO:      "YPKE",
		Name:      "Peake Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -28.25,
		Longitude: 135.8999938965,
		Timezone:  "Australia/Adelaide",
	},
	"YPKG": {
		ICAO:      "YPKG",
		IATA:      "KGI",
		Name:      "Kalgoorlie Boulder Airport",
		City:      "Kalgoorlie",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 1203,
		Latitude:  -30.7894001007,
		Longitude: 121.461997986,
		Timezone:  "Australia/Perth",
	},
	"YPKH": {
		ICAO:      "YPKH",
		Name:      "Peak Hill Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -32.75,
		Longitude: 148.1999969482,
		Timezone:  "Australia/Sydney",
	},
	"YPKI": {
		ICAO:      "YPKI",
		Name:      "Parakylia Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -30.3999996185,
		Longitude: 136.391998291,
		Timezone:  "Australia/Adelaide",
	},
	"YPKL": {
		ICAO:      "YPKL",
		Name:      "Puckapunyal (Military) Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 168,
		Latitude:  -37,
		Longitude: 145.06300354,
		Timezone:  "Australia/Melbourne",
	},
	"YPKS": {
		ICAO:      "YPKS",
		IATA:      "PKE",
		Name:      "Parkes Airport",
		City:      "Parkes",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 1069,
		Latitude:  -33.131401062,
		Longitude: 148.238998413,
		Timezone:  "Australia/Sydney",
	},
	"YPKT": {
		ICAO:      "YPKT",
		IATA:      "PKT",
		Name:      "Port Keats Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 91,
		Latitude:  -14.25,
		Longitude: 129.529006958,
		Timezone:  "Australia/Darwin",
	},
	"YPKU": {
		ICAO:      "YPKU",
		IATA:      "KNX",
		Name:      "Kununurra Airport",
		City:      "Kununurra",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 145,
		Latitude:  -15.7781000137,
		Longitude: 128.707992554,
		Timezone:  "Australia/Perth",
	},
	"YPLC": {
		ICAO:      "YPLC",
		IATA:      "PLO",
		Name:      "Port Lincoln Airport",
		City:      "Port Lincoln",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 36,
		Latitude:  -34.6053009033,
		Longitude: 135.880004883,
		Timezone:  "Australia/Adelaide",
	},
	"YPLG": {
		ICAO:      "YPLG",
		Name:      "Pilliga Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -30.3666992188,
		Longitude: 148.8829956055,
		Timezone:  "Australia/Sydney",
	},
	"YPLI": {
		ICAO:      "YPLI",
		Name:      "Palmers Island/Yamba Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -29.4458007813,
		Longitude: 153.266998291,
		Timezone:  "Australia/Sydney",
	},
	"YPLL": {
		ICAO:      "YPLL",
		Name:      "Peak Hill Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -25.6000003815,
		Longitude: 118.6829986572,
		Timezone:  "Australia/Perth",
	},
	"YPLM": {
		ICAO:      "YPLM",
		IATA:      "LEA",
		Name:      "Learmonth Airport",
		City:      "Exmouth",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 19,
		Latitude:  -22.2355995178,
		Longitude: 114.088996887,
		Timezone:  "Australia/Perth",
	},
	"YPLU": {
		ICAO:      "YPLU",
		Name:      "Plutonic Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -25.3332996368,
		Longitude: 119.4250030518,
		Timezone:  "Australia/Perth",
	},
	"YPMB": {
		ICAO:      "YPMB",
		Name:      "Plumbago Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -32.0532989502,
		Longitude: 139.8719940186,
		Timezone:  "Australia/Adelaide",
	},
	"YPME": {
		ICAO:      "YPME",
		Name:      "Palmer Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -12.8332996368,
		Longitude: 130.9329986572,
		Timezone:  "Australia/Darwin",
	},
	"YPMP": {
		ICAO:      "YPMP",
		IATA:      "EDR",
		Name:      "Pormpuraaw Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 10,
		Latitude:  -14.8966999054,
		Longitude: 141.6089935303,
		Timezone:  "Australia/Brisbane",
	},
	"YPMQ": {
		ICAO:      "YPMQ",
		IATA:      "PQQ",
		Name:      "Port Macquarie Airport",
		City:      "Port Macquarie",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 12,
		Latitude:  -31.4358005524,
		Longitude: 152.863006592,
		Timezone:  "Australia/Sydney",
	},
	"YPNC": {
		ICAO:      "YPNC",
		Name:      "Pine Creek Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -13.8332996368,
		Longitude: 131.8329925537,
		Timezone:  "Australia/Darwin",
	},
	"YPNG": {
		ICAO:      "YPNG",
		Name:      "Penong Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -31.9167003632,
		Longitude: 133,
		Timezone:  "Australia/Adelaide",
	},
	"YPNI": {
		ICAO:      "YPNI",
		Name:      "Prenti Downs Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -26.5167007446,
		Longitude: 122.8000030518,
		Timezone:  "Australia/Perth",
	},
	"YPNN": {
		ICAO:      "YPNN",
		Name:      "Pinnaroo Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 76,
		Latitude:  -35.2532997131,
		Longitude: 140.9429931641,
		Timezone:  "Australia/Adelaide",
	},
	"YPNW": {
		ICAO:      "YPNW",
		Name:      "Pannawonica Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -21.6166992188,
		Longitude: 116.3170013428,
		Timezone:  "Australia/Perth",
	},
	"YPOD": {
		ICAO:      "YPOD",
		IATA:      "PTJ",
		Name:      "Portland Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 265,
		Latitude:  -38.3180999756,
		Longitude: 141.470993042,
		Timezone:  "Australia/Melbourne",
	},
	"YPOK": {
		ICAO:      "YPOK",
		IATA:      "MBF",
		Name:      "Porepunkah Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 935,
		Latitude:  -36.7177307062,
		Longitude: 146.890039444,
		Timezone:  "Australia/Melbourne",
	},
	"YPOP": {
		ICAO:      "YPOP",
		Name:      "Porphyry Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -29.7782993317,
		Longitude: 122.2679977417,
		Timezone:  "Australia/Perth",
	},
	"YPPD": {
		ICAO:      "YPPD",
		IATA:      "PHE",
		Name:      "Port Hedland International Airport",
		City:      "Port Hedland",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 33,
		Latitude:  -20.3777999878,
		Longitude: 118.625999451,
		Timezone:  "Australia/Perth",
	},
	"YPPF": {
		ICAO:      "YPPF",
		Name:      "Adelaide Parafield Airport",
		City:      "Adelaide",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 57,
		Latitude:  -34.7933006287,
		Longitude: 138.632995605,
		Timezone:  "Australia/Adelaide",
	},
	"YPPH": {
		ICAO:      "YPPH",
		IATA:      "PER",
		Name:      "Perth International Airport",
		City:      "Perth",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 67,
		Latitude:  -31.9402999878,
		Longitude: 115.9670028687,
		Timezone:  "Australia/Perth",
	},
	"YPRA": {
		ICAO:      "YPRA",
		Name:      "Prairie Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -36.3072013855,
		Longitude: 144.1390075684,
		Timezone:  "Australia/Melbourne",
	},
	"YPRE": {
		ICAO:      "YPRE",
		Name:      "Premer Betoota Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 1257,
		Latitude:  -31.4769001007,
		Longitude: 149.9029998779,
		Timezone:  "Australia/Sydney",
	},
	"YPSH": {
		ICAO:      "YPSH",
		IATA:      "PEA",
		Name:      "Penneshaw Airport",
		City:      "Ironstone",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -35.7558462874,
		Longitude: 137.962875366,
		Timezone:  "Australia/Adelaide",
	},
	"YPTB": {
		ICAO:      "YPTB",
		Name:      "Peterborough Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -33.0050010681,
		Longitude: 138.858001709,
		Timezone:  "Australia/Adelaide",
	},
	"YPTJ": {
		ICAO:      "YPTJ",
		Name:      "Patjarr Airport",
		City:      "Patjarr",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 1493,
		Latitude:  -24.6189002991,
		Longitude: 126.327003479,
		Timezone:  "Australia/Perth",
	},
	"YPTN": {
		ICAO:      "YPTN",
		IATA:      "KTR",
		Name:      "Tindal Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 443,
		Latitude:  -14.5211000443,
		Longitude: 132.3780059814,
		Timezone:  "Australia/Darwin",
	},
	"YPUA": {
		ICAO:      "YPUA",
		Name:      "Palumpa Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -14.3332996368,
		Longitude: 129.8670043945,
		Timezone:  "Australia/Darwin",
	},
	"YPUN": {
		ICAO:      "YPUN",
		Name:      "Punmu Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -22.0666999817,
		Longitude: 123.1669998169,
		Timezone:  "Australia/Perth",
	},
	"YPVD": {
		ICAO:      "YPVD",
		Name:      "Plevna Downs Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -26.6667003632,
		Longitude: 142.5829925537,
		Timezone:  "Australia/Brisbane",
	},
	"YPWH": {
		ICAO:      "YPWH",
		Name:      "Pittsworth Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -27.7208003998,
		Longitude: 151.6329956055,
		Timezone:  "Australia/Brisbane",
	},
	"YPWR": {
		ICAO:      "YPWR",
		IATA:      "UMR",
		Name:      "Woomera Airfield",
		City:      "Woomera",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 548,
		Latitude:  -31.1441993713,
		Longitude: 136.8170013428,
		Timezone:  "Australia/Adelaide",
	},
	"YPXM": {
		ICAO:      "YPXM",
		IATA:      "XCH",
		Name:      "Christmas Island Airport",
		City:      "Christmas Island",
		Country:   "CX",
		Elevation: 916,
		Latitude:  -10.4505996704,
		Longitude: 105.6900024414,
		Timezone:  "Indian/Christmas",
	},
	"YPYD": {
		ICAO:      "YPYD",
		Name:      "Pyramid Hill Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -36.0666999817,
		Longitude: 144.1329956055,
		Timezone:  "Australia/Melbourne",
	},
	"YPYF": {
		ICAO:      "YPYF",
		Name:      "Paynes Find Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -29.25,
		Longitude: 117.6669998169,
		Timezone:  "Australia/Perth",
	},
	"YQBE": {
		ICAO:      "YQBE",
		Name:      "Quambone Royona Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 489,
		Latitude:  -30.8791999817,
		Longitude: 147.8419952393,
		Timezone:  "Australia/Sydney",
	},
	"YQBK": {
		ICAO:      "YQBK",
		Name:      "Quambatook Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -35.8622016907,
		Longitude: 143.5099945068,
		Timezone:  "Australia/Melbourne",
	},
	"YQDG": {
		ICAO:      "YQDG",
		Name:      "Quairading Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -32,
		Longitude: 117.4000015259,
		Timezone:  "Australia/Perth",
	},
	"YQDI": {
		ICAO:      "YQDI",
		IATA:      "UIR",
		Name:      "Quirindi Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 1054,
		Latitude:  -31.4906005859,
		Longitude: 150.5140075684,
		Timezone:  "Australia/Sydney",
	},
	"YQLP": {
		ICAO:      "YQLP",
		IATA:      "ULP",
		Name:      "Quilpie Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 655,
		Latitude:  -26.6121997833,
		Longitude: 144.2530059814,
		Timezone:  "Australia/Brisbane",
	},
	"YQNS": {
		ICAO:      "YQNS",
		IATA:      "UEE",
		Name:      "Queenstown Airport",
		State:     "Tasmania",
		Country:   "AU",
		Elevation: 264,
		Latitude:  -42.0750007629,
		Longitude: 145.5319976807,
		Timezone:  "Australia/Hobart",
	},
	"YQON": {
		ICAO:      "YQON",
		Name:      "Quondong Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -33.1632995605,
		Longitude: 140.4519958496,
		Timezone:  "Australia/Adelaide",
	},
	"YQRN": {
		ICAO:      "YQRN",
		Name:      "Quorn Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 293,
		Latitude:  -32.3199996948,
		Longitude: 138.1020050049,
		Timezone:  "Australia/Adelaide",
	},
	"YQUA": {
		ICAO:      "YQUA",
		Name:      "Quarry2 Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 604,
		Latitude:  -30.2922000885,
		Longitude: 134.5269927979,
		Timezone:  "Australia/Adelaide",
	},
	"YQUE": {
		ICAO:      "YQUE",
		Name:      "Questa Park Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -30.3917007446,
		Longitude: 143.141998291,
		Timezone:  "Australia/Sydney",
	},
	"YRAG": {
		ICAO:      "YRAG",
		Name:      "Raglan Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -37.3773873391,
		Longitude: 143.305664062,
		Timezone:  "Australia/Melbourne",
	},
	"YRAM": {
		ICAO:      "YRAM",
		Name:      "Raymore Homestead Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -26.1499996185,
		Longitude: 143.016998291,
		Timezone:  "Australia/Brisbane",
	},
	"YRAT": {
		ICAO:      "YRAT",
		Name:      "Abrolhos Rat Island",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 16,
		Latitude:  -28.7197,
		Longitude: 113.7845,
		Timezone:  "Australia/Perth",
	},
	"YRAV": {
		ICAO:      "YRAV",
		Name:      "Ravensthorpe Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -33.528301239,
		Longitude: 119.9530029297,
		Timezone:  "Australia/Perth",
	},
	"YRAW": {
		ICAO:      "YRAW",
		Name:      "Rawlinna Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -31,
		Longitude: 125.3170013428,
		Timezone:  "Australia/Perth",
	},
	"YRBB": {
		ICAO:      "YRBB",
		Name:      "Rainbow Beach Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -25.8332996368,
		Longitude: 153.0670013428,
		Timezone:  "Australia/Brisbane",
	},
	"YRBE": {
		ICAO:      "YRBE",
		Name:      "Robe Airport",
		City:      "Robe",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 65,
		Latitude:  -37.1753636288,
		Longitude: 139.805231094,
		Timezone:  "Australia/Adelaide",
	},
	"YRBM": {
		ICAO:      "YRBM",
		Name:      "Redbank Mine Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -17.1833000183,
		Longitude: 137.766998291,
		Timezone:  "Australia/Darwin",
	},
	"YRBR": {
		ICAO:      "YRBR",
		Name:      "Robinson River Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -16.7182998657,
		Longitude: 136.9450073242,
		Timezone:  "Australia/Darwin",
	},
	"YRBT": {
		ICAO:      "YRBT",
		Name:      "Rabbit Flat Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -20.1833000183,
		Longitude: 130.016998291,
		Timezone:  "Australia/Darwin",
	},
	"YRBW": {
		ICAO:      "YRBW",
		Name:      "Rainbow Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -35.9166984558,
		Longitude: 141.983001709,
		Timezone:  "Australia/Melbourne",
	},
	"YRDA": {
		ICAO:      "YRDA",
		Name:      "Yardea Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -32.4099998474,
		Longitude: 135.4850006104,
		Timezone:  "Australia/Adelaide",
	},
	"YRDM": {
		ICAO:      "YRDM",
		Name:      "Redmont Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -21.9666996002,
		Longitude: 119.016998291,
		Timezone:  "Australia/Perth",
	},
	"YRDY": {
		ICAO:      "YRDY",
		Name:      "Reedys Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -27.1317005157,
		Longitude: 118.2799987793,
		Timezone:  "Australia/Perth",
	},
	"YRED": {
		ICAO:      "YRED",
		Name:      "Redcliffe Airport",
		City:      "Redcliffe",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 2,
		Latitude:  -27.2066993713,
		Longitude: 153.0679931641,
		Timezone:  "Australia/Brisbane",
	},
	"YREN": {
		ICAO:      "YREN",
		IATA:      "RMK",
		Name:      "Renmark Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 115,
		Latitude:  -34.1963996887,
		Longitude: 140.6739959717,
		Timezone:  "Australia/Adelaide",
	},
	"YRHL": {
		ICAO:      "YRHL",
		Name:      "Red Hill Station Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -21.9666996002,
		Longitude: 116.0670013428,
		Timezone:  "Australia/Perth",
	},
	"YRID": {
		ICAO:      "YRID",
		Name:      "Riddell Airport",
		City:      "Riddell",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -37.4799995422,
		Longitude: 144.7180023193,
		Timezone:  "Australia/Melbourne",
	},
	"YRKE": {
		ICAO:      "YRKE",
		Name:      "Kuruc-A-Ruc South Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -37.9000015259,
		Longitude: 143.7169952393,
		Timezone:  "Australia/Melbourne",
	},
	"YRKS": {
		ICAO:      "YRKS",
		Name:      "Rocklands Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -19.8666992188,
		Longitude: 138.1000061035,
		Timezone:  "Australia/Brisbane",
	},
	"YRLE": {
		ICAO:      "YRLE",
		Name:      "Rocklea Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -22.8833007813,
		Longitude: 117.4499969482,
		Timezone:  "Australia/Perth",
	},
	"YRLL": {
		ICAO:      "YRLL",
		Name:      "Rolleston Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -24.4617004395,
		Longitude: 148.6320037842,
		Timezone:  "Australia/Brisbane",
	},
	"YRMD": {
		ICAO:      "YRMD",
		IATA:      "RCM",
		Name:      "Richmond Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 676,
		Latitude:  -20.7019004822,
		Longitude: 143.1150054932,
		Timezone:  "Australia/Brisbane",
	},
	"YRNG": {
		ICAO:      "YRNG",
		IATA:      "RAM",
		Name:      "Ramingining Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 206,
		Latitude:  -12.3563995361,
		Longitude: 134.8979949951,
		Timezone:  "Australia/Darwin",
	},
	"YRNW": {
		ICAO:      "YRNW",
		Name:      "Ringwood Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -23.8283004761,
		Longitude: 134.9620056152,
		Timezone:  "Australia/Darwin",
	},
	"YROB": {
		ICAO:      "YROB",
		IATA:      "ROH",
		Name:      "Robinhood Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 460,
		Latitude:  -18.8449993134,
		Longitude: 143.7100067139,
		Timezone:  "Australia/Brisbane",
	},
	"YROE": {
		ICAO:      "YROE",
		IATA:      "RBU",
		Name:      "Roebourne Airport",
		City:      "Roebourne",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -20.7616996765,
		Longitude: 117.156997681,
		Timezone:  "Australia/Perth",
	},
	"YROI": {
		ICAO:      "YROI",
		IATA:      "RBC",
		Name:      "Robinvale Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 87,
		Latitude:  -34.6500015259,
		Longitude: 142.7830047607,
		Timezone:  "Australia/Melbourne",
	},
	"YROK": {
		ICAO:      "YROK",
		Name:      "Rocky River Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -35.9300003052,
		Longitude: 136.7200012207,
		Timezone:  "Australia/Adelaide",
	},
	"YROM": {
		ICAO:      "YROM",
		IATA:      "RMA",
		Name:      "Roma Airport",
		City:      "Roma",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 1032,
		Latitude:  -26.5450000763,
		Longitude: 148.774993896,
		Timezone:  "Australia/Brisbane",
	},
	"YROT": {
		ICAO:      "YROT",
		Name:      "Rothsay Mine Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -29.2917003632,
		Longitude: 116.8669967651,
		Timezone:  "Australia/Perth",
	},
	"YROV": {
		ICAO:      "YROV",
		Name:      "Ross River Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -23.599199295,
		Longitude: 134.516998291,
		Timezone:  "Australia/Darwin",
	},
	"YRRB": {
		ICAO:      "YRRB",
		IATA:      "RPB",
		Name:      "Roper Bar Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -14.7383003235,
		Longitude: 134.5180053711,
		Timezone:  "Australia/Darwin",
	},
	"YRRG": {
		ICAO:      "YRRG",
		Name:      "Rhodes Ridge Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -23.1082992554,
		Longitude: 119.3330001831,
		Timezone:  "Australia/Perth",
	},
	"YRRR": {
		ICAO:      "YRRR",
		Name:      "Raymangirr Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -12.3332996368,
		Longitude: 135.6499938965,
		Timezone:  "Australia/Darwin",
	},
	"YRSB": {
		ICAO:      "YRSB",
		IATA:      "RSB",
		Name:      "Roseberth Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 55,
		Latitude:  -25.8332996368,
		Longitude: 139.6499938965,
		Timezone:  "Australia/Brisbane",
	},
	"YRSH": {
		ICAO:      "YRSH",
		Name:      "Rosehill Heliport Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 4,
		Latitude:  -33.8300966368,
		Longitude: 151.024138927,
		Timezone:  "Australia/Sydney",
	},
	"YRSK": {
		ICAO:      "YRSK",
		Name:      "Ringer Soak Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -18.7886009216,
		Longitude: 128.6239929199,
		Timezone:  "Australia/Perth",
	},
	"YRSV": {
		ICAO:      "YRSV",
		Name:      "Rosevale Resort Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -27.1333007813,
		Longitude: 145.8999938965,
		Timezone:  "Australia/Brisbane",
	},
	"YRSY": {
		ICAO:      "YRSY",
		Name:      "Romsey (Riddell/Penfield) Airport",
		City:      "Romsey",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 351,
		Latitude:  -37.391667,
		Longitude: 144.738333,
		Timezone:  "Australia/Melbourne",
	},
	"YRTI": {
		ICAO:      "YRTI",
		IATA:      "RTS",
		Name:      "Rottnest Island Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 12,
		Latitude:  -32.0066986084,
		Longitude: 115.5400009155,
		Timezone:  "Australia/Perth",
	},
	"YRTP": {
		ICAO:      "YRTP",
		IATA:      "RTP",
		Name:      "Rutland Plains Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 15,
		Latitude:  -15.6433000565,
		Longitude: 141.8430023193,
		Timezone:  "Australia/Brisbane",
	},
	"YRUD": {
		ICAO:      "YRUD",
		Name:      "Rudal River Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -22.5499992371,
		Longitude: 122.1500015259,
		Timezone:  "Australia/Perth",
	},
	"YRUG": {
		ICAO:      "YRUG",
		Name:      "Rugby Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -34.3917007446,
		Longitude: 148.9750061035,
		Timezone:  "Australia/Sydney",
	},
	"YRVE": {
		ICAO:      "YRVE",
		Name:      "Riveren Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -17.8999996185,
		Longitude: 130.2169952393,
		Timezone:  "Australia/Darwin",
	},
	"YRWA": {
		ICAO:      "YRWA",
		Name:      "Rawlinna Station Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -31.0251998901,
		Longitude: 125.1910018921,
		Timezone:  "Australia/Perth",
	},
	"YRWH": {
		ICAO:      "YRWH",
		Name:      "Ravensworth Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -34.6333007813,
		Longitude: 144.2830047607,
		Timezone:  "Australia/Sydney",
	},
	"YRXB": {
		ICAO:      "YRXB",
		Name:      "Roxby Downs Station Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -30.7166996002,
		Longitude: 136.7369995117,
		Timezone:  "Australia/Adelaide",
	},
	"YRYH": {
		ICAO:      "YRYH",
		IATA:      "RHL",
		Name:      "Roy Hill Station Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -22.625815891,
		Longitude: 119.959030151,
		Timezone:  "Australia/Perth",
	},
	"YRYK": {
		ICAO:      "YRYK",
		Name:      "Rawnsley Park Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -31.6535282135,
		Longitude: 138.6095275879,
		Timezone:  "Australia/Adelaide",
	},
	"YSAH": {
		ICAO:      "YSAH",
		Name:      "Sandhill Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -22.7999992371,
		Longitude: 119.6169967651,
		Timezone:  "Australia/Perth",
	},
	"YSAI": {
		ICAO:      "YSAI",
		Name:      "Saipem Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -23.7000007629,
		Longitude: 131.8670043945,
		Timezone:  "Australia/Darwin",
	},
	"YSAN": {
		ICAO:      "YSAN",
		IATA:      "NDS",
		Name:      "Sandstone Airport",
		City:      "Sandstone",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -27.9799995422,
		Longitude: 119.2969970703,
		Timezone:  "Australia/Perth",
	},
	"YSAR": {
		ICAO:      "YSAR",
		Name:      "Mount Sarah Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -27.0431003571,
		Longitude: 135.2420043945,
		Timezone:  "Australia/Adelaide",
	},
	"YSBG": {
		ICAO:      "YSBG",
		Name:      "Strathbogie Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -36.8549995422,
		Longitude: 145.7380065918,
		Timezone:  "Australia/Melbourne",
	},
	"YSBK": {
		ICAO:      "YSBK",
		IATA:      "BWU",
		Name:      "Sydney Bankstown Airport",
		City:      "Sydney",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 29,
		Latitude:  -33.9244003296,
		Longitude: 150.9880065918,
		Timezone:  "Australia/Sydney",
	},
	"YSBO": {
		ICAO:      "YSBO",
		Name:      "Stanbroke Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -21.5666999817,
		Longitude: 139.7169952393,
		Timezone:  "Australia/Brisbane",
	},
	"YSCA": {
		ICAO:      "YSCA",
		Name:      "Scotia Sanctuary Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 177,
		Latitude:  -33.2071990967,
		Longitude: 141.17300415,
		Timezone:  "Australia/Sydney",
	},
	"YSCB": {
		ICAO:      "YSCB",
		IATA:      "CBR",
		Name:      "Canberra International Airport",
		City:      "Canberra",
		State:     "ACT",
		Country:   "AU",
		Elevation: 1886,
		Latitude:  -35.3069000244,
		Longitude: 149.1950073242,
		Timezone:  "Australia/Sydney",
	},
	"YSCH": {
		ICAO:      "YSCH",
		IATA:      "CFS",
		Name:      "Coffs Harbour Airport",
		City:      "Coffs Harbour",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 18,
		Latitude:  -30.3206005096,
		Longitude: 153.115997314,
		Timezone:  "Australia/Sydney",
	},
	"YSCN": {
		ICAO:      "YSCN",
		IATA:      "CDU",
		Name:      "Camden Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 230,
		Latitude:  -34.0402984619,
		Longitude: 150.68699646,
		Timezone:  "Australia/Sydney",
	},
	"YSCO": {
		ICAO:      "YSCO",
		IATA:      "NSO",
		Name:      "Scone Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 745,
		Latitude:  -32.0372009277,
		Longitude: 150.8320007324,
		Timezone:  "Australia/Sydney",
	},
	"YSCR": {
		ICAO:      "YSCR",
		IATA:      "SQC",
		Name:      "Southern Cross Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 354,
		Latitude:  -31.2399997711,
		Longitude: 119.3600006104,
		Timezone:  "Australia/Perth",
	},
	"YSDL": {
		ICAO:      "YSDL",
		Name:      "Sudley Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -12.75,
		Longitude: 142.3670043945,
		Timezone:  "Australia/Brisbane",
	},
	"YSDN": {
		ICAO:      "YSDN",
		Name:      "Soudan Station Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -20.0499992371,
		Longitude: 137.016998291,
		Timezone:  "Australia/Darwin",
	},
	"YSDU": {
		ICAO:      "YSDU",
		IATA:      "DBO",
		Name:      "Dubbo City Regional Airport",
		City:      "Dubbo",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 935,
		Latitude:  -32.2167015076,
		Longitude: 148.574996948,
		Timezone:  "Australia/Sydney",
	},
	"YSDW": {
		ICAO:      "YSDW",
		Name:      "Strathdownie Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -37.7317008972,
		Longitude: 141.125,
		Timezone:  "Australia/Melbourne",
	},
	"YSEN": {
		ICAO:      "YSEN",
		Name:      "Serpentine Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 9,
		Latitude:  -32.3949661255,
		Longitude: 115.8709869385,
		Timezone:  "Australia/Perth",
	},
	"YSFI": {
		ICAO:      "YSFI",
		Name:      "Sandfire Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -19.7667007446,
		Longitude: 121.1669998169,
		Timezone:  "Australia/Perth",
	},
	"YSFL": {
		ICAO:      "YSFL",
		Name:      "Stonefield Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -34.3418769603,
		Longitude: 139.308185577,
		Timezone:  "Australia/Adelaide",
	},
	"YSFY": {
		ICAO:      "YSFY",
		Name:      "Sandfly Airport",
		State:     "Tasmania",
		Country:   "AU",
		Latitude:  -42.9916992188,
		Longitude: 147.2079925537,
		Timezone:  "Australia/Hobart",
	},
	"YSGD": {
		ICAO:      "YSGD",
		Name:      "Strathgordon Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -14.7950000763,
		Longitude: 142.4329986572,
		Timezone:  "Australia/Brisbane",
	},
	"YSGE": {
		ICAO:      "YSGE",
		IATA:      "SGO",
		Name:      "St George Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 656,
		Latitude:  -28.0496997833,
		Longitude: 148.5950012207,
		Timezone:  "Australia/Brisbane",
	},
	"YSGR": {
		ICAO:      "YSGR",
		Name:      "South Grafton Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 6,
		Latitude:  -29.7082996368,
		Longitude: 152.9279937744,
		Timezone:  "Australia/Sydney",
	},
	"YSGT": {
		ICAO:      "YSGT",
		IATA:      "SIX",
		Name:      "Singleton Airport",
		City:      "Singleton",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 46,
		Latitude:  -32.600832,
		Longitude: 151.193056,
		Timezone:  "Australia/Sydney",
	},
	"YSGW": {
		ICAO:      "YSGW",
		IATA:      "ZGL",
		Name:      "South Galway Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 116,
		Latitude:  -25.6833000183,
		Longitude: 142.108001709,
		Timezone:  "Australia/Brisbane",
	},
	"YSHG": {
		ICAO:      "YSHG",
		IATA:      "SGP",
		Name:      "Shay Gap Airport",
		City:      "Shay Gap",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 620,
		Latitude:  -20.4246997833,
		Longitude: 120.1409988403,
		Timezone:  "Australia/Perth",
	},
	"YSHK": {
		ICAO:      "YSHK",
		IATA:      "MJK",
		Name:      "Shark Bay Airport",
		City:      "Monkey Mia",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 111,
		Latitude:  -25.8938999176,
		Longitude: 113.577003479,
		Timezone:  "Australia/Perth",
	},
	"YSHN": {
		ICAO:      "YSHN",
		Name:      "Shannon River Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -34.75,
		Longitude: 116.483001709,
		Timezone:  "Australia/Perth",
	},
	"YSHR": {
		ICAO:      "YSHR",
		IATA:      "JHQ",
		Name:      "Shute Harbour Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 12,
		Latitude:  -20.2782993317,
		Longitude: 148.7570037842,
		Timezone:  "Australia/Brisbane",
	},
	"YSHT": {
		ICAO:      "YSHT",
		IATA:      "SHT",
		Name:      "Shepparton Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 374,
		Latitude:  -36.4289016724,
		Longitude: 145.3930053711,
		Timezone:  "Australia/Melbourne",
	},
	"YSHW": {
		ICAO:      "YSHW",
		Name:      "Holsworthy (Military) Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 76,
		Latitude:  -33.9949989319,
		Longitude: 150.9519958496,
		Timezone:  "Australia/Sydney",
	},
	"YSIA": {
		ICAO:      "YSIA",
		Name:      "Siam Airport",
		City:      "Siam",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -32.5574989319,
		Longitude: 136.7089996338,
		Timezone:  "Australia/Adelaide",
	},
	"YSII": {
		ICAO:      "YSII",
		IATA:      "SBR",
		Name:      "Saibai Island Airport",
		City:      "Saibai Island",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 15,
		Latitude:  -9.3783302307,
		Longitude: 142.625,
		Timezone:  "Australia/Brisbane",
	},
	"YSLE": {
		ICAO:      "YSLE",
		Name:      "St Leonards Airfield",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 47,
		Latitude:  -38.1701011658,
		Longitude: 144.6889953613,
		Timezone:  "Australia/Melbourne",
	},
	"YSLG": {
		ICAO:      "YSLG",
		Name:      "Silent Grove Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -17.0666999817,
		Longitude: 125.25,
		Timezone:  "Australia/Perth",
	},
	"YSLK": {
		ICAO:      "YSLK",
		Name:      "Sea Lake Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 56,
		Latitude:  -35.5266990662,
		Longitude: 142.8880004883,
		Timezone:  "Australia/Melbourne",
	},
	"YSLN": {
		ICAO:      "YSLN",
		Name:      "Strathleven Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -15.8983001709,
		Longitude: 143.3829956055,
		Timezone:  "Australia/Brisbane",
	},
	"YSLT": {
		ICAO:      "YSLT",
		Name:      "Sale Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -38.1166992188,
		Longitude: 147.0749969482,
		Timezone:  "Australia/Melbourne",
	},
	"YSLV": {
		ICAO:      "YSLV",
		Name:      "Sylvania Homestead Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -23.5583000183,
		Longitude: 120.0469970703,
		Timezone:  "Australia/Perth",
	},
	"YSMI": {
		ICAO:      "YSMI",
		IATA:      "SIO",
		Name:      "Smithton Airport",
		State:     "Tasmania",
		Country:   "AU",
		Elevation: 31,
		Latitude:  -40.8349990845,
		Longitude: 145.0839996338,
		Timezone:  "Australia/Hobart",
	},
	"YSMP": {
		ICAO:      "YSMP",
		IATA:      "SHU",
		Name:      "Smith Point Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -11.1499996185,
		Longitude: 132.149993896,
		Timezone:  "Australia/Darwin",
	},
	"YSMR": {
		ICAO:      "YSMR",
		IATA:      "STH",
		Name:      "Strathmore Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -17.8500003815,
		Longitude: 142.5670013428,
		Timezone:  "Australia/Brisbane",
	},
	"YSMY": {
		ICAO:      "YSMY",
		Name:      "Strathmay Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -14.8832998276,
		Longitude: 142.8020019531,
		Timezone:  "Australia/Brisbane",
	},
	"YSNB": {
		ICAO:      "YSNB",
		IATA:      "SNB",
		Name:      "Snake Bay Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 173,
		Latitude:  -11.4228000641,
		Longitude: 130.654006958,
		Timezone:  "Australia/Darwin",
	},
	"YSNF": {
		ICAO:      "YSNF",
		IATA:      "NLK",
		Name:      "Norfolk Island International Airport",
		City:      "Burnt Pine",
		Country:   "NF",
		Elevation: 371,
		Latitude:  -29.0415992737,
		Longitude: 167.9389953613,
		Timezone:  "Pacific/Norfolk",
	},
	"YSNT": {
		ICAO:      "YSNT",
		Name:      "Snowtown Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -33.7999992371,
		Longitude: 138.3670043945,
		Timezone:  "Australia/Adelaide",
	},
	"YSNW": {
		ICAO:      "YSNW",
		IATA:      "NOA",
		Name:      "Nowra Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 400,
		Latitude:  -34.9488983154,
		Longitude: 150.5370025635,
		Timezone:  "Australia/Sydney",
	},
	"YSOL": {
		ICAO:      "YSOL",
		IATA:      "SLJ",
		Name:      "Solomon Airport",
		City:      "Karijini National Park",
		State:     "Westrn-Australia",
		Country:   "AU",
		Elevation: 2021,
		Latitude:  -22.255278,
		Longitude: 117.761944,
		Timezone:  "Australia/Perth",
	},
	"YSOW": {
		ICAO:      "YSOW",
		Name:      "Southwell Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -14.516699791,
		Longitude: 142.0330047607,
		Timezone:  "Australia/Brisbane",
	},
	"YSPE": {
		ICAO:      "YSPE",
		IATA:      "SNH",
		Name:      "Stanthorpe Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 2938,
		Latitude:  -28.620300293,
		Longitude: 151.9909973145,
		Timezone:  "Australia/Brisbane",
	},
	"YSPF": {
		ICAO:      "YSPF",
		Name:      "Springfield Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -24.2999992371,
		Longitude: 144.8829956055,
		Timezone:  "Australia/Brisbane",
	},
	"YSPI": {
		ICAO:      "YSPI",
		Name:      "Springsure Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -24.1282997131,
		Longitude: 148.0749969482,
		Timezone:  "Australia/Brisbane",
	},
	"YSPK": {
		ICAO:      "YSPK",
		IATA:      "SCG",
		Name:      "Spring Creek Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -18.6333007813,
		Longitude: 144.5670013428,
		Timezone:  "Australia/Brisbane",
	},
	"YSPT": {
		ICAO:      "YSPT",
		IATA:      "SHQ",
		Name:      "Southport Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 2,
		Latitude:  -27.9150009155,
		Longitude: 153.3730010986,
		Timezone:  "Australia/Brisbane",
	},
	"YSPV": {
		ICAO:      "YSPV",
		IATA:      "KSV",
		Name:      "Springvale Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -23.5499992371,
		Longitude: 140.6999969482,
		Timezone:  "Australia/Brisbane",
	},
	"YSRD": {
		ICAO:      "YSRD",
		Name:      "Sunrise Dam Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -29.1016998291,
		Longitude: 122.4499969482,
		Timezone:  "Australia/Perth",
	},
	"YSRI": {
		ICAO:      "YSRI",
		IATA:      "XRH",
		Name:      "RAAF Base Richmond",
		City:      "Richmond",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 67,
		Latitude:  -33.6006011963,
		Longitude: 150.7810058594,
		Timezone:  "Australia/Sydney",
	},
	"YSRN": {
		ICAO:      "YSRN",
		IATA:      "SRN",
		Name:      "Strahan Airport",
		State:     "Tasmania",
		Country:   "AU",
		Elevation: 20,
		Latitude:  -42.1549987793,
		Longitude: 145.2920074463,
		Timezone:  "Australia/Hobart",
	},
	"YSRT": {
		ICAO:      "YSRT",
		Name:      "Surat Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -27.1499996185,
		Longitude: 149.0829925537,
		Timezone:  "Australia/Brisbane",
	},
	"YSSY": {
		ICAO:      "YSSY",
		IATA:      "SYD",
		Name:      "Sydney Kingsford Smith International Airport",
		City:      "Sydney",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 21,
		Latitude:  -33.9460983276,
		Longitude: 151.1770019531,
		Timezone:  "Australia/Sydney",
	},
	"YSTA": {
		ICAO:      "YSTA",
		Name:      "Saint Arnaud Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 639,
		Latitude:  -36.6366996765,
		Longitude: 143.1860046387,
		Timezone:  "Australia/Melbourne",
	},
	"YSTB": {
		ICAO:      "YSTB",
		Name:      "Strathburn Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -14.4816999435,
		Longitude: 142.8170013428,
		Timezone:  "Australia/Brisbane",
	},
	"YSTC": {
		ICAO:      "YSTC",
		Name:      "Stuart Creek Airport",
		City:      "Stuart Creek",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 122,
		Latitude:  -29.7166996002,
		Longitude: 137.06300354,
		Timezone:  "Australia/Adelaide",
	},
	"YSTH": {
		ICAO:      "YSTH",
		IATA:      "HLS",
		Name:      "St Helens Airport",
		State:     "Tasmania",
		Country:   "AU",
		Elevation: 48,
		Latitude:  -41.3367004395,
		Longitude: 148.2819976807,
		Timezone:  "Australia/Hobart",
	},
	"YSTO": {
		ICAO:      "YSTO",
		Name:      "Stonehenge Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 166,
		Latitude:  -24.3582992554,
		Longitude: 143.3000030518,
		Timezone:  "Australia/Brisbane",
	},
	"YSTR": {
		ICAO:      "YSTR",
		Name:      "Strathearn Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -31.7667007446,
		Longitude: 140.3329925537,
		Timezone:  "Australia/Adelaide",
	},
	"YSTT": {
		ICAO:      "YSTT",
		Name:      "Santa Teresa Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -24.1182994843,
		Longitude: 134.3899993896,
		Timezone:  "Australia/Darwin",
	},
	"YSTW": {
		ICAO:      "YSTW",
		IATA:      "TMW",
		Name:      "Tamworth Airport",
		City:      "Tamworth",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 1334,
		Latitude:  -31.0839004517,
		Longitude: 150.847000122,
		Timezone:  "Australia/Sydney",
	},
	"YSUJ": {
		ICAO:      "YSUJ",
		Name:      "Supplejack Downs Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -19.2667007446,
		Longitude: 129.9499969482,
		Timezone:  "Australia/Darwin",
	},
	"YSVH": {
		ICAO:      "YSVH",
		Name:      "Silver Hills Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -20.6166992188,
		Longitude: 143.0829925537,
		Timezone:  "Australia/Brisbane",
	},
	"YSVN": {
		ICAO:      "YSVN",
		Name:      "Strathaven Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -14.8999996185,
		Longitude: 142.9669952393,
		Timezone:  "Australia/Brisbane",
	},
	"YSWA": {
		ICAO:      "YSWA",
		Name:      "Swansea Airport",
		State:     "Tasmania",
		Country:   "AU",
		Latitude:  -42.1016998291,
		Longitude: 148.0679931641,
		Timezone:  "Australia/Hobart",
	},
	"YSWE": {
		ICAO:      "YSWE",
		Name:      "Sweers Island Resort Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -17.1217002869,
		Longitude: 139.5980072021,
		Timezone:  "Australia/Brisbane",
	},
	"YSWG": {
		ICAO:      "YSWG",
		IATA:      "WGA",
		Name:      "Wagga Wagga City Airport",
		City:      "Wagga Wagga",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 724,
		Latitude:  -35.1652984619,
		Longitude: 147.466003418,
		Timezone:  "Australia/Sydney",
	},
	"YSWH": {
		ICAO:      "YSWH",
		IATA:      "SWH",
		Name:      "Swan Hill Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 234,
		Latitude:  -35.3758010864,
		Longitude: 143.5330047607,
		Timezone:  "Australia/Melbourne",
	},
	"YSWL": {
		ICAO:      "YSWL",
		IATA:      "SWC",
		Name:      "Stawell Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 807,
		Latitude:  -37.0717010498,
		Longitude: 142.7409973145,
		Timezone:  "Australia/Melbourne",
	},
	"YSYN": {
		ICAO:      "YSYN",
		Name:      "Strathalbyn Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -35.3116989136,
		Longitude: 138.9900054932,
		Timezone:  "Australia/Adelaide",
	},
	"YTAA": {
		ICAO:      "YTAA",
		IATA:      "XTR",
		Name:      "Tara Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 360,
		Latitude:  -27.1567001343,
		Longitude: 150.4770050049,
		Timezone:  "Australia/Brisbane",
	},
	"YTAB": {
		ICAO:      "YTAB",
		IATA:      "TBL",
		Name:      "Tableland Homestead Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -17.2833003998,
		Longitude: 126.900001526,
		Timezone:  "Australia/Perth",
	},
	"YTAD": {
		ICAO:      "YTAD",
		Name:      "Tandou Lake Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -32.6333007813,
		Longitude: 142.0829925537,
		Timezone:  "Australia/Sydney",
	},
	"YTAM": {
		ICAO:      "YTAM",
		IATA:      "XTO",
		Name:      "Taroom Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 240,
		Latitude:  -25.801700592,
		Longitude: 149.8999938965,
		Timezone:  "Australia/Brisbane",
	},
	"YTAN": {
		ICAO:      "YTAN",
		Name:      "Tanami Downs Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -20.5736541748,
		Longitude: 129.7401275635,
		Timezone:  "Australia/Darwin",
	},
	"YTAR": {
		ICAO:      "YTAR",
		IATA:      "TAQ",
		Name:      "Tarcoola Airport",
		City:      "Tarcoola",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -30.7033004761,
		Longitude: 134.583999634,
		Timezone:  "Australia/Adelaide",
	},
	"YTBB": {
		ICAO:      "YTBB",
		Name:      "Tumby Bay Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 11,
		Latitude:  -34.3616981506,
		Longitude: 136.0950012207,
		Timezone:  "Australia/Adelaide",
	},
	"YTBF": {
		ICAO:      "YTBF",
		Name:      "Pattaya Airpark",
		State:     "Chon-Buri",
		Country:   "TH",
		Elevation: 100,
		Latitude:  12.8325,
		Longitude: 100.949444444,
		Timezone:  "Asia/Bangkok",
	},
	"YTBG": {
		ICAO:      "YTBG",
		Name:      "Talbingo Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -35.5833015442,
		Longitude: 148.2830047607,
		Timezone:  "Australia/Sydney",
	},
	"YTBR": {
		ICAO:      "YTBR",
		IATA:      "TBK",
		Name:      "Timber Creek Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 16,
		Latitude:  -15.6199998856,
		Longitude: 130.4450073242,
		Timezone:  "Australia/Darwin",
	},
	"YTCF": {
		ICAO:      "YTCF",
		Name:      "Tracies Field",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -22.3166999817,
		Longitude: 122.0670013428,
		Timezone:  "Australia/Perth",
	},
	"YTCK": {
		ICAO:      "YTCK",
		Name:      "Torrens Creek Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -20.7833003998,
		Longitude: 145,
		Timezone:  "Australia/Brisbane",
	},
	"YTCY": {
		ICAO:      "YTCY",
		Name:      "Tarcoonyinna Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -26.75,
		Longitude: 133.3500061035,
		Timezone:  "Australia/Adelaide",
	},
	"YTDL": {
		ICAO:      "YTDL",
		Name:      "Tidal River Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -39.034198761,
		Longitude: 146.3269958496,
		Timezone:  "Australia/Melbourne",
	},
	"YTDM": {
		ICAO:      "YTDM",
		Name:      "Todmorden Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -27.1242008209,
		Longitude: 134.7700042725,
		Timezone:  "Australia/Adelaide",
	},
	"YTDN": {
		ICAO:      "YTDN",
		Name:      "Tooradin Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 3,
		Latitude:  -38.2167015076,
		Longitude: 145.4270019531,
		Timezone:  "Australia/Melbourne",
	},
	"YTDR": {
		ICAO:      "YTDR",
		IATA:      "TDR",
		Name:      "Theodore Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 171,
		Latitude:  -24.9932994843,
		Longitude: 150.0930023193,
		Timezone:  "Australia/Brisbane",
	},
	"YTEC": {
		ICAO:      "YTEC",
		Name:      "Tenneco Station Three Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -30.3367004395,
		Longitude: 139.6649932861,
		Timezone:  "Australia/Adelaide",
	},
	"YTEE": {
		ICAO:      "YTEE",
		IATA:      "TQP",
		Name:      "Trepell Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 891,
		Latitude:  -21.8349990845,
		Longitude: 140.8880004883,
		Timezone:  "Australia/Brisbane",
	},
	"YTEF": {
		ICAO:      "YTEF",
		IATA:      "TEF",
		Name:      "Telfer Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 970,
		Latitude:  -21.7150001526,
		Longitude: 122.2289962769,
		Timezone:  "Australia/Perth",
	},
	"YTEM": {
		ICAO:      "YTEM",
		IATA:      "TEM",
		Name:      "Temora Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 921,
		Latitude:  -34.4213981628,
		Longitude: 147.5119934082,
		Timezone:  "Australia/Sydney",
	},
	"YTEN": {
		ICAO:      "YTEN",
		Name:      "Tenneco Station Five Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -31.9750003815,
		Longitude: 139.2070007324,
		Timezone:  "Australia/Adelaide",
	},
	"YTFA": {
		ICAO:      "YTFA",
		Name:      "Truro Flat Airpark",
		City:      "Truro Flat",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -34.3983001709,
		Longitude: 139.3849945068,
		Timezone:  "Australia/Adelaide",
	},
	"YTFD": {
		ICAO:      "YTFD",
		Name:      "Tenterfield Airport",
		City:      "Tenterfield",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -28.9915500425,
		Longitude: 151.930103302,
		Timezone:  "Australia/Sydney",
	},
	"YTGA": {
		ICAO:      "YTGA",
		IATA:      "TAN",
		Name:      "Tangalooma Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -27.1299991608,
		Longitude: 153.363006592,
		Timezone:  "Australia/Brisbane",
	},
	"YTGG": {
		ICAO:      "YTGG",
		Name:      "Taggerty Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -37.3504981995,
		Longitude: 145.6999969482,
		Timezone:  "Australia/Melbourne",
	},
	"YTGI": {
		ICAO:      "YTGI",
		Name:      "Trangie Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -31.9666996002,
		Longitude: 148,
		Timezone:  "Australia/Sydney",
	},
	"YTGM": {
		ICAO:      "YTGM",
		IATA:      "XTG",
		Name:      "Thargomindah Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 433,
		Latitude:  -27.9864006042,
		Longitude: 143.8110046387,
		Timezone:  "Australia/Brisbane",
	},
	"YTGT": {
		ICAO:      "YTGT",
		IATA:      "GTS",
		Name:      "The Granites Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -20.5482997894,
		Longitude: 130.3470001221,
		Timezone:  "Australia/Darwin",
	},
	"YTGV": {
		ICAO:      "YTGV",
		Name:      "The Grove Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -34.686000824,
		Longitude: 148.1040039063,
		Timezone:  "Australia/Sydney",
	},
	"YTHD": {
		ICAO:      "YTHD",
		IATA:      "TDN",
		Name:      "Theda Station Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -14.7881002426,
		Longitude: 126.4960021973,
		Timezone:  "Australia/Perth",
	},
	"YTHI": {
		ICAO:      "YTHI",
		Name:      "Thistle Island Airport",
		City:      "Thistle Island",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 120,
		Latitude:  -35.0247001648,
		Longitude: 136.1799926758,
		Timezone:  "Australia/Adelaide",
	},
	"YTHR": {
		ICAO:      "YTHR",
		Name:      "Three Rivers Homestead Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -25.1333007813,
		Longitude: 119.1169967651,
		Timezone:  "Australia/Perth",
	},
	"YTHS": {
		ICAO:      "YTHS",
		Name:      "Three Springs Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 900,
		Latitude:  -29.5279109299,
		Longitude: 115.858039856,
		Timezone:  "Australia/Perth",
	},
	"YTHV": {
		ICAO:      "YTHV",
		Name:      "Thevenard Island Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -21.4666996002,
		Longitude: 115,
		Timezone:  "Australia/Perth",
	},
	"YTHY": {
		ICAO:      "YTHY",
		IATA:      "TYG",
		Name:      "Thylungra Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 165,
		Latitude:  -26.0832996368,
		Longitude: 143.4669952393,
		Timezone:  "Australia/Brisbane",
	},
	"YTIB": {
		ICAO:      "YTIB",
		IATA:      "TYB",
		Name:      "Tibooburra Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 584,
		Latitude:  -29.4510993958,
		Longitude: 142.0579986572,
		Timezone:  "Australia/Sydney",
	},
	"YTII": {
		ICAO:      "YTII",
		Name:      "Trinidad Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -25.6000003815,
		Longitude: 143.9170074463,
		Timezone:  "Australia/Brisbane",
	},
	"YTIN": {
		ICAO:      "YTIN",
		Name:      "Tintinara Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -35.8666992188,
		Longitude: 140.0670013428,
		Timezone:  "Australia/Adelaide",
	},
	"YTIT": {
		ICAO:      "YTIT",
		Name:      "Ti Tree Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -22.1317005157,
		Longitude: 133.4199981689,
		Timezone:  "Australia/Darwin",
	},
	"YTJI": {
		ICAO:      "YTJI",
		Name:      "Tjirrkarli Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -26.0349998474,
		Longitude: 125.4720001221,
		Timezone:  "Australia/Perth",
	},
	"YTJU": {
		ICAO:      "YTJU",
		Name:      "Tjuntjuntjarra Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -29.3582992554,
		Longitude: 127.1279983521,
		Timezone:  "Australia/Perth",
	},
	"YTKS": {
		ICAO:      "YTKS",
		Name:      "Toorak Research Station Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 128,
		Latitude:  -21.0417003632,
		Longitude: 141.7870025635,
		Timezone:  "Australia/Brisbane",
	},
	"YTKY": {
		ICAO:      "YTKY",
		IATA:      "TKY",
		Name:      "Turkey Creek Airport",
		City:      "Turkey Creek",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -17.0408000946,
		Longitude: 128.205993652,
		Timezone:  "Australia/Perth",
	},
	"YTLG": {
		ICAO:      "YTLG",
		Name:      "Taralga Grathawa Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 3031,
		Latitude:  -34.2891998291,
		Longitude: 149.7819976807,
		Timezone:  "Australia/Sydney",
	},
	"YTLL": {
		ICAO:      "YTLL",
		Name:      "Tullamore Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -32.6166992188,
		Longitude: 147.6000061035,
		Timezone:  "Australia/Sydney",
	},
	"YTLP": {
		ICAO:      "YTLP",
		Name:      "Tilpa Airport",
		City:      "Tilpa",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -30.9333000183,
		Longitude: 144.4170074463,
		Timezone:  "Australia/Sydney",
	},
	"YTLT": {
		ICAO:      "YTLT",
		Name:      "Tarlton Downs Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -22.6333007813,
		Longitude: 136.8000030518,
		Timezone:  "Australia/Darwin",
	},
	"YTMB": {
		ICAO:      "YTMB",
		Name:      "Tambo Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 457,
		Latitude:  -24.8500003815,
		Longitude: 146.2830047607,
		Timezone:  "Australia/Brisbane",
	},
	"YTML": {
		ICAO:      "YTML",
		Name:      "Tamala Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -26.6667003632,
		Longitude: 113.733001709,
		Timezone:  "Australia/Perth",
	},
	"YTMN": {
		ICAO:      "YTMN",
		Name:      "Tanami Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 427,
		Latitude:  -19.9167003632,
		Longitude: 129.7250061035,
		Timezone:  "Australia/Darwin",
	},
	"YTMO": {
		ICAO:      "YTMO",
		IATA:      "PHQ",
		Name:      "The Monument Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 949,
		Latitude:  -21.8111000061,
		Longitude: 139.9239959717,
		Timezone:  "Australia/Brisbane",
	},
	"YTMP": {
		ICAO:      "YTMP",
		IATA:      "TPR",
		Name:      "Tom Price Airport",
		City:      "Tom Price",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 2300,
		Latitude:  -22.7460002899,
		Longitude: 117.869003296,
		Timezone:  "Australia/Perth",
	},
	"YTMS": {
		ICAO:      "YTMS",
		Name:      "Tambar Springs Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -31.3150005341,
		Longitude: 149.8470001221,
		Timezone:  "Australia/Sydney",
	},
	"YTMU": {
		ICAO:      "YTMU",
		IATA:      "TUM",
		Name:      "Tumut Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 878,
		Latitude:  -35.2627983093,
		Longitude: 148.2409973145,
		Timezone:  "Australia/Sydney",
	},
	"YTMY": {
		ICAO:      "YTMY",
		IATA:      "TYP",
		Name:      "Tobermorey Airport",
		City:      "Tobermorey",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -22.2558002472,
		Longitude: 137.9530029297,
		Timezone:  "Australia/Darwin",
	},
	"YTNB": {
		ICAO:      "YTNB",
		Name:      "Tanbar Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 344,
		Latitude:  -25.8477993011,
		Longitude: 141.9279937744,
		Timezone:  "Australia/Brisbane",
	},
	"YTNC": {
		ICAO:      "YTNC",
		Name:      "Tuncurry Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -32.1500015259,
		Longitude: 152.483001709,
		Timezone:  "Australia/Sydney",
	},
	"YTNE": {
		ICAO:      "YTNE",
		Name:      "Tenneco Station One Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -28.7332992554,
		Longitude: 140.0500030518,
		Timezone:  "Australia/Adelaide",
	},
	"YTNG": {
		ICAO:      "YTNG",
		IATA:      "THG",
		Name:      "Thangool Airport",
		City:      "Biloela",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 644,
		Latitude:  -24.4939002991,
		Longitude: 150.5760040283,
		Timezone:  "Australia/Brisbane",
	},
	"YTNK": {
		ICAO:      "YTNK",
		IATA:      "TCA",
		Name:      "Tennant Creek Airport",
		City:      "Tennant Creek",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 1236,
		Latitude:  -19.6343994141,
		Longitude: 134.1829986572,
		Timezone:  "Australia/Darwin",
	},
	"YTNN": {
		ICAO:      "YTNN",
		Name:      "Tenneco Station Four Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -31.1833000183,
		Longitude: 139.3829956055,
		Timezone:  "Australia/Adelaide",
	},
	"YTNR": {
		ICAO:      "YTNR",
		Name:      "Tanumbirini Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -16.4500007629,
		Longitude: 134.6499938965,
		Timezone:  "Australia/Darwin",
	},
	"YTNS": {
		ICAO:      "YTNS",
		Name:      "Toliness Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -25.3214511871,
		Longitude: 146.0688171387,
		Timezone:  "Australia/Brisbane",
	},
	"YTOA": {
		ICAO:      "YTOA",
		Name:      "Toolachie Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -28.4083003998,
		Longitude: 140.8099975586,
		Timezone:  "Australia/Adelaide",
	},
	"YTOB": {
		ICAO:      "YTOB",
		Name:      "Toomba Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -19.9333000183,
		Longitude: 145.5829925537,
		Timezone:  "Australia/Brisbane",
	},
	"YTOC": {
		ICAO:      "YTOC",
		IATA:      "TCW",
		Name:      "Tocumwal Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 372,
		Latitude:  -35.8116989136,
		Longitude: 145.608001709,
		Timezone:  "Australia/Sydney",
	},
	"YTOG": {
		ICAO:      "YTOG",
		Name:      "Togo Station Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -30.0821990967,
		Longitude: 149.5319976807,
		Timezone:  "Australia/Sydney",
	},
	"YTOK": {
		ICAO:      "YTOK",
		Name:      "Torres Park Homestead Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -25.1166992188,
		Longitude: 147.1670074463,
		Timezone:  "Australia/Brisbane",
	},
	"YTOS": {
		ICAO:      "YTOS",
		Name:      "Tenneco Station Two Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -29.5499992371,
		Longitude: 139.9499969482,
		Timezone:  "Australia/Adelaide",
	},
	"YTOT": {
		ICAO:      "YTOT",
		Name:      "Tottenham Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 238,
		Latitude:  -32.248298645,
		Longitude: 147.3679962158,
		Timezone:  "Australia/Sydney",
	},
	"YTPE": {
		ICAO:      "YTPE",
		Name:      "Tempe Downs Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 1696,
		Latitude:  -24.3824996948,
		Longitude: 132.4230041504,
		Timezone:  "Australia/Darwin",
	},
	"YTQY": {
		ICAO:      "YTQY",
		Name:      "Torquay Airport",
		City:      "Torquay",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 9,
		Latitude:  -38.2999992371,
		Longitude: 144.3650054932,
		Timezone:  "Australia/Melbourne",
	},
	"YTRB": {
		ICAO:      "YTRB",
		Name:      "Torrumbarry Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -36.0666999817,
		Longitude: 144.5330047607,
		Timezone:  "Australia/Melbourne",
	},
	"YTRC": {
		ICAO:      "YTRC",
		Name:      "Tarcombe Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -24.0832996368,
		Longitude: 143.3999938965,
		Timezone:  "Australia/Brisbane",
	},
	"YTRE": {
		ICAO:      "YTRE",
		IATA:      "TRO",
		Name:      "Taree Airport",
		City:      "Taree",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 38,
		Latitude:  -31.8885993958,
		Longitude: 152.514007568,
		Timezone:  "Australia/Sydney",
	},
	"YTRK": {
		ICAO:      "YTRK",
		Name:      "Turee Creek Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -23.6166992188,
		Longitude: 118.6169967651,
		Timezone:  "Australia/Perth",
	},
	"YTST": {
		ICAO:      "YTST",
		IATA:      "TTX",
		Name:      "Truscott Mungalalu Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 181,
		Latitude:  -14.0896997452,
		Longitude: 126.3809967041,
		Timezone:  "Australia/Perth",
	},
	"YTTE": {
		ICAO:      "YTTE",
		Name:      "Ti Tree Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -13.9849996567,
		Longitude: 141.6569976807,
		Timezone:  "Australia/Brisbane",
	},
	"YTTI": {
		ICAO:      "YTTI",
		Name:      "Troughton Is Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 8,
		Latitude:  -13.7517004013,
		Longitude: 126.1480026245,
		Timezone:  "Australia/Perth",
	},
	"YTUA": {
		ICAO:      "YTUA",
		Name:      "Triabunna Airport",
		State:     "Tasmania",
		Country:   "AU",
		Latitude:  -42.5136985779,
		Longitude: 147.8979949951,
		Timezone:  "Australia/Hobart",
	},
	"YTUC": {
		ICAO:      "YTUC",
		Name:      "Tuckabiana Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -27.4750003815,
		Longitude: 118.125,
		Timezone:  "Australia/Perth",
	},
	"YTUG": {
		ICAO:      "YTUG",
		Name:      "Trugananni Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -24.4666996002,
		Longitude: 149.266998291,
		Timezone:  "Australia/Brisbane",
	},
	"YTUN": {
		ICAO:      "YTUN",
		Name:      "Tunbridge Airport",
		State:     "Tasmania",
		Country:   "AU",
		Latitude:  -42.0999984741,
		Longitude: 147.3670043945,
		Timezone:  "Australia/Hobart",
	},
	"YTUY": {
		ICAO:      "YTUY",
		Name:      "Tully Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -17.9333000183,
		Longitude: 145.9170074463,
		Timezone:  "Australia/Brisbane",
	},
	"YTVA": {
		ICAO:      "YTVA",
		Name:      "Talavera Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -19.2166996002,
		Longitude: 143.6670074463,
		Timezone:  "Australia/Brisbane",
	},
	"YTWB": {
		ICAO:      "YTWB",
		IATA:      "TWB",
		Name:      "Toowoomba Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 2086,
		Latitude:  -27.5428009033,
		Longitude: 151.9160003662,
		Timezone:  "Australia/Brisbane",
	},
	"YTWE": {
		ICAO:      "YTWE",
		Name:      "Trelawney Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -22.8500003815,
		Longitude: 146.7169952393,
		Timezone:  "Australia/Brisbane",
	},
	"YTWN": {
		ICAO:      "YTWN",
		Name:      "Tooraweenah Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 421,
		Latitude:  -31.4416999817,
		Longitude: 148.8999938965,
		Timezone:  "Australia/Sydney",
	},
	"YTYA": {
		ICAO:      "YTYA",
		Name:      "Tyabb Airport",
		City:      "Melbourne",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 88,
		Latitude:  -38.2667007446,
		Longitude: 145.175003052,
		Timezone:  "Australia/Melbourne",
	},
	"YTYH": {
		ICAO:      "YTYH",
		Name:      "Tyagarah Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -28.5946960394,
		Longitude: 153.551101685,
		Timezone:  "Australia/Sydney",
	},
	"YTYN": {
		ICAO:      "YTYN",
		Name:      "Tieyon Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -26.2166996002,
		Longitude: 133.9170074463,
		Timezone:  "Australia/Adelaide",
	},
	"YUCH": {
		ICAO:      "YUCH",
		Name:      "Ucharonidge Airport",
		City:      "Ucharonidge Station",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -17.6724014282,
		Longitude: 134.2409057617,
		Timezone:  "Australia/Darwin",
	},
	"YUDA": {
		ICAO:      "YUDA",
		IATA:      "UDA",
		Name:      "Undara Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -18.2000007629,
		Longitude: 144.6000061035,
		Timezone:  "Australia/Brisbane",
	},
	"YUDG": {
		ICAO:      "YUDG",
		Name:      "Urandangi Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -21.5900001526,
		Longitude: 138.358001709,
		Timezone:  "Australia/Brisbane",
	},
	"YUDL": {
		ICAO:      "YUDL",
		Name:      "Undilla Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -19.625,
		Longitude: 138.6329956055,
		Timezone:  "Australia/Brisbane",
	},
	"YUMB": {
		ICAO:      "YUMB",
		Name:      "Umbeara Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -25.7467002869,
		Longitude: 133.7019958496,
		Timezone:  "Australia/Darwin",
	},
	"YUMU": {
		ICAO:      "YUMU",
		Name:      "Umuwa Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -26.486700058,
		Longitude: 132.0399932861,
		Timezone:  "Australia/Adelaide",
	},
	"YUNY": {
		ICAO:      "YUNY",
		IATA:      "CZY",
		Name:      "Cluny Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 90,
		Latitude:  -24.5167007446,
		Longitude: 139.6170043945,
		Timezone:  "Australia/Brisbane",
	},
	"YUPG": {
		ICAO:      "YUPG",
		Name:      "Urapunga Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -14.7166996002,
		Longitude: 134.5670013428,
		Timezone:  "Australia/Darwin",
	},
	"YUPH": {
		ICAO:      "YUPH",
		Name:      "Upper Horton Wyl Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 1253,
		Latitude:  -30.1047000885,
		Longitude: 150.404006958,
		Timezone:  "Australia/Sydney",
	},
	"YUSL": {
		ICAO:      "YUSL",
		IATA:      "USL",
		Name:      "Useless Loop Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -26.1667003632,
		Longitude: 113.4000015259,
		Timezone:  "Australia/Perth",
	},
	"YUTP": {
		ICAO:      "YUTP",
		Name:      "Utopia Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -22.0550003052,
		Longitude: 134.7899932861,
		Timezone:  "Australia/Darwin",
	},
	"YUTS": {
		ICAO:      "YUTS",
		Name:      "Utopia Station Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -22.2332992554,
		Longitude: 134.5800018311,
		Timezone:  "Australia/Darwin",
	},
	"YVAF": {
		ICAO:      "YVAF",
		Name:      "Valley Field Airport",
		State:     "Tasmania",
		Country:   "AU",
		Latitude:  -41.8100013733,
		Longitude: 147.2920074463,
		Timezone:  "Australia/Hobart",
	},
	"YVIV": {
		ICAO:      "YVIV",
		Name:      "Mount Vivian Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -30.5832996368,
		Longitude: 135.7169952393,
		Timezone:  "Australia/Adelaide",
	},
	"YVLG": {
		ICAO:      "YVLG",
		Name:      "Valley of Lagoons Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -18.6667003632,
		Longitude: 145.1000061035,
		Timezone:  "Australia/Brisbane",
	},
	"YVNS": {
		ICAO:      "YVNS",
		Name:      "Vaughan Springs Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -22.3332996368,
		Longitude: 130.8670043945,
		Timezone:  "Australia/Darwin",
	},
	"YVRD": {
		ICAO:      "YVRD",
		IATA:      "VCD",
		Name:      "Victoria River Downs Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 89,
		Latitude:  -16.4021244049,
		Longitude: 131.0049743652,
		Timezone:  "Australia/Darwin",
	},
	"YVRS": {
		ICAO:      "YVRS",
		IATA:      "VNR",
		Name:      "Vanrook Station Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 43,
		Latitude:  -16.963300705,
		Longitude: 141.9499969482,
		Timezone:  "Australia/Brisbane",
	},
	"YVSH": {
		ICAO:      "YVSH",
		Name:      "Vashon Head Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -11.1499996185,
		Longitude: 131.983001709,
		Timezone:  "Australia/Darwin",
	},
	"YVVA": {
		ICAO:      "YVVA",
		Name:      "Victoria Valley Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -37.5,
		Longitude: 142.2700042725,
		Timezone:  "Australia/Melbourne",
	},
	"YVVL": {
		ICAO:      "YVVL",
		Name:      "Violet Vale Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -14.733300209,
		Longitude: 143.5829925537,
		Timezone:  "Australia/Brisbane",
	},
	"YWAB": {
		ICAO:      "YWAB",
		Name:      "Waldburg Homestead Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -24.75,
		Longitude: 117.3669967651,
		Timezone:  "Australia/Perth",
	},
	"YWAC": {
		ICAO:      "YWAC",
		Name:      "Wauchope Airport",
		City:      "Wauchope",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -20.6499996185,
		Longitude: 134.2169952393,
		Timezone:  "Australia/Darwin",
	},
	"YWAG": {
		ICAO:      "YWAG",
		Name:      "Wanaaring Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -29.7166996002,
		Longitude: 144.1670074463,
		Timezone:  "Australia/Sydney",
	},
	"YWAL": {
		ICAO:      "YWAL",
		IATA:      "WLA",
		Name:      "Wallal Airport",
		City:      "Wallal",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -19.7735996246,
		Longitude: 120.649002075,
		Timezone:  "Australia/Perth",
	},
	"YWAT": {
		ICAO:      "YWAT",
		Name:      "Wattle Hills Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -12.6332998276,
		Longitude: 143.0500030518,
		Timezone:  "Australia/Brisbane",
	},
	"YWAV": {
		ICAO:      "YWAV",
		IATA:      "WAV",
		Name:      "Wave Hill Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 201,
		Latitude:  -17.3932991028,
		Longitude: 131.1179962158,
		Timezone:  "Australia/Darwin",
	},
	"YWAX": {
		ICAO:      "YWAX",
		Name:      "Wanarn Airport",
		City:      "Wanarn",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -25.2975521088,
		Longitude: 127.5564193726,
		Timezone:  "Australia/Perth",
	},
	"YWBH": {
		ICAO:      "YWBH",
		Name:      "Wallabadah Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -31.5333003998,
		Longitude: 150.8670043945,
		Timezone:  "Australia/Sydney",
	},
	"YWBI": {
		ICAO:      "YWBI",
		Name:      "Warrabri Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 383,
		Latitude:  -21,
		Longitude: 134.3970031738,
		Timezone:  "Australia/Darwin",
	},
	"YWBL": {
		ICAO:      "YWBL",
		IATA:      "WMB",
		Name:      "Warrnambool Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 242,
		Latitude:  -38.29529953,
		Longitude: 142.4470062256,
		Timezone:  "Australia/Melbourne",
	},
	"YWBN": {
		ICAO:      "YWBN",
		Name:      "Wedderburn Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 259,
		Latitude:  -34.1866989136,
		Longitude: 150.8049926758,
		Timezone:  "Australia/Sydney",
	},
	"YWBR": {
		ICAO:      "YWBR",
		Name:      "Warburton Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 1500,
		Latitude:  -26.1282997131,
		Longitude: 126.5830001831,
		Timezone:  "Australia/Perth",
	},
	"YWBS": {
		ICAO:      "YWBS",
		IATA:      "SYU",
		Name:      "Warraber Island Airport",
		City:      "Sue Islet",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 3,
		Latitude:  -10.2082996368,
		Longitude: 142.8249969482,
		Timezone:  "Australia/Brisbane",
	},
	"YWCA": {
		ICAO:      "YWCA",
		IATA:      "WIO",
		Name:      "Wilcannia Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 250,
		Latitude:  -31.5263996124,
		Longitude: 143.375,
		Timezone:  "Australia/Sydney",
	},
	"YWCH": {
		ICAO:      "YWCH",
		IATA:      "WLC",
		Name:      "Walcha Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -31,
		Longitude: 151.5670013428,
		Timezone:  "Australia/Sydney",
	},
	"YWCK": {
		ICAO:      "YWCK",
		IATA:      "WAZ",
		Name:      "Warwick Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 1500,
		Latitude:  -28.1494007111,
		Longitude: 151.9429931641,
		Timezone:  "Australia/Brisbane",
	},
	"YWCM": {
		ICAO:      "YWCM",
		Name:      "Wilson's Camp Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -20.1014003754,
		Longitude: 129.1260070801,
		Timezone:  "Australia/Darwin",
	},
	"YWDA": {
		ICAO:      "YWDA",
		IATA:      "WND",
		Name:      "Windarra Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -28.4750003815,
		Longitude: 122.241996765,
		Timezone:  "Australia/Perth",
	},
	"YWDC": {
		ICAO:      "YWDC",
		Name:      "Wodgina Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -21.1417007446,
		Longitude: 118.6920013428,
		Timezone:  "Australia/Perth",
	},
	"YWDH": {
		ICAO:      "YWDH",
		IATA:      "WNR",
		Name:      "Windorah Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 452,
		Latitude:  -25.4130992889,
		Longitude: 142.6670074463,
		Timezone:  "Australia/Brisbane",
	},
	"YWDJ": {
		ICAO:      "YWDJ",
		Name:      "Windjana Grove Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -17.421387587,
		Longitude: 124.925107956,
		Timezone:  "Australia/Perth",
	},
	"YWDL": {
		ICAO:      "YWDL",
		IATA:      "WON",
		Name:      "Wondoola Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 58,
		Latitude:  -18.5750007629,
		Longitude: 140.891998291,
		Timezone:  "Australia/Brisbane",
	},
	"YWDR": {
		ICAO:      "YWDR",
		Name:      "Collector2 Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 2274,
		Latitude:  -34.9388999939,
		Longitude: 149.4100036621,
		Timezone:  "Australia/Sydney",
	},
	"YWDS": {
		ICAO:      "YWDS",
		Name:      "Woodside Au Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 354,
		Latitude:  -35.1333007813,
		Longitude: 145.6670074463,
		Timezone:  "Australia/Sydney",
	},
	"YWDT": {
		ICAO:      "YWDT",
		Name:      "Wyandotte Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -18.75,
		Longitude: 144.8329925537,
		Timezone:  "Australia/Brisbane",
	},
	"YWDV": {
		ICAO:      "YWDV",
		Name:      "Wandovale Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -19.6667003632,
		Longitude: 144.8999938965,
		Timezone:  "Australia/Brisbane",
	},
	"YWEC": {
		ICAO:      "YWEC",
		Name:      "Wellclose Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -25.8500003815,
		Longitude: 145.1000061035,
		Timezone:  "Australia/Brisbane",
	},
	"YWED": {
		ICAO:      "YWED",
		Name:      "Wedderburn Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -36.4333000183,
		Longitude: 143.483001709,
		Timezone:  "Australia/Melbourne",
	},
	"YWEE": {
		ICAO:      "YWEE",
		Name:      "Wooleen Homestead Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -27.0816993713,
		Longitude: 116.1500015259,
		Timezone:  "Australia/Perth",
	},
	"YWEL": {
		ICAO:      "YWEL",
		Name:      "Wellington Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 426,
		Latitude:  -32.4667015076,
		Longitude: 148.983001709,
		Timezone:  "Australia/Sydney",
	},
	"YWEO": {
		ICAO:      "YWEO",
		Name:      "Wertaloona Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -30.6499996185,
		Longitude: 139.3500061035,
		Timezone:  "Australia/Adelaide",
	},
	"YWER": {
		ICAO:      "YWER",
		Name:      "Wernadinga Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -18.1333007813,
		Longitude: 139.9669952393,
		Timezone:  "Australia/Brisbane",
	},
	"YWEX": {
		ICAO:      "YWEX",
		Name:      "Well 33 Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -22.3666992188,
		Longitude: 124.766998291,
		Timezone:  "Australia/Perth",
	},
	"YWFD": {
		ICAO:      "YWFD",
		Name:      "Western Field Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -19.2833003998,
		Longitude: 125.4169998169,
		Timezone:  "Australia/Perth",
	},
	"YWGN": {
		ICAO:      "YWGN",
		Name:      "Wagin Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -33.3333015442,
		Longitude: 117.3669967651,
		Timezone:  "Australia/Perth",
	},
	"YWGT": {
		ICAO:      "YWGT",
		IATA:      "WGT",
		Name:      "Wangaratta Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 504,
		Latitude:  -36.4157981873,
		Longitude: 146.3070068359,
		Timezone:  "Australia/Melbourne",
	},
	"YWGW": {
		ICAO:      "YWGW",
		Name:      "Wongawol Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -26.1333007813,
		Longitude: 121.9670028687,
		Timezone:  "Australia/Perth",
	},
	"YWHA": {
		ICAO:      "YWHA",
		IATA:      "WYA",
		Name:      "Whyalla Airport",
		City:      "Whyalla",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 41,
		Latitude:  -33.0588989258,
		Longitude: 137.5140075684,
		Timezone:  "Australia/Adelaide",
	},
	"YWHC": {
		ICAO:      "YWHC",
		Name:      "White Cliffs Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 162,
		Latitude:  -30.8533000946,
		Longitude: 143.0720062256,
		Timezone:  "Australia/Sydney",
	},
	"YWHI": {
		ICAO:      "YWHI",
		Name:      "Whitsunday Island Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -20.2667007446,
		Longitude: 148.766998291,
		Timezone:  "Australia/Brisbane",
	},
	"YWHL": {
		ICAO:      "YWHL",
		Name:      "Walhallow Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -17.7667007446,
		Longitude: 135.6499938965,
		Timezone:  "Australia/Darwin",
	},
	"YWIB": {
		ICAO:      "YWIB",
		Name:      "Mount Willoughby Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -27.9832992554,
		Longitude: 134.1499938965,
		Timezone:  "Australia/Adelaide",
	},
	"YWIE": {
		ICAO:      "YWIE",
		Name:      "Wirralie Gold Mine Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -21.1166992188,
		Longitude: 147.266998291,
		Timezone:  "Australia/Brisbane",
	},
	"YWIL": {
		ICAO:      "YWIL",
		Name:      "Wilandra Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -31.2833003998,
		Longitude: 142.6670074463,
		Timezone:  "Australia/Sydney",
	},
	"YWIO": {
		ICAO:      "YWIO",
		Name:      "Wilton Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -34.2332992554,
		Longitude: 150.6670074463,
		Timezone:  "Australia/Sydney",
	},
	"YWIS": {
		ICAO:      "YWIS",
		Name:      "Williamson Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 89,
		Latitude:  -22.4733009338,
		Longitude: 150.1779937744,
		Timezone:  "Australia/Brisbane",
	},
	"YWIT": {
		ICAO:      "YWIT",
		IATA:      "WIT",
		Name:      "Wittenoom Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 444,
		Latitude:  -22.2182998657,
		Longitude: 118.3479995728,
		Timezone:  "Australia/Perth",
	},
	"YWJS": {
		ICAO:      "YWJS",
		Name:      "Wee Jasper Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -35.2616996765,
		Longitude: 148.6670074463,
		Timezone:  "Australia/Sydney",
	},
	"YWKB": {
		ICAO:      "YWKB",
		IATA:      "WKB",
		Name:      "Warracknabeal Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 397,
		Latitude:  -36.3210983276,
		Longitude: 142.4190063477,
		Timezone:  "Australia/Melbourne",
	},
	"YWKI": {
		ICAO:      "YWKI",
		Name:      "Waikerie Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 138,
		Latitude:  -34.1838989258,
		Longitude: 140.0310058594,
		Timezone:  "Australia/Adelaide",
	},
	"YWKM": {
		ICAO:      "YWKM",
		Name:      "Wyalkatchem Airport",
		City:      "Wyalkatchem",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -31.2026341022,
		Longitude: 117.378873825,
		Timezone:  "Australia/Perth",
	},
	"YWKS": {
		ICAO:      "YWKS",
		Name:      "Wilkins Runway",
		Country:   "AQ",
		Elevation: 2529,
		Latitude:  -66.690833,
		Longitude: 111.523611,
		Timezone:  "Antarctica/Casey",
	},
	"YWKW": {
		ICAO:      "YWKW",
		Name:      "Warkworth Airport",
		City:      "Warkworth",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -32.5491283809,
		Longitude: 151.024246216,
		Timezone:  "Australia/Sydney",
	},
	"YWLA": {
		ICAO:      "YWLA",
		Name:      "Willowra Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -21.2766990662,
		Longitude: 132.6230010986,
		Timezone:  "Australia/Darwin",
	},
	"YWLB": {
		ICAO:      "YWLB",
		Name:      "Welbourn Hill Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -27.3581008911,
		Longitude: 134.0930023193,
		Timezone:  "Australia/Adelaide",
	},
	"YWLG": {
		ICAO:      "YWLG",
		IATA:      "WGE",
		Name:      "Walgett Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 439,
		Latitude:  -30.0328006744,
		Longitude: 148.1260070801,
		Timezone:  "Australia/Sydney",
	},
	"YWLH": {
		ICAO:      "YWLH",
		Name:      "Wallara Ranch Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -24.6499996185,
		Longitude: 132.3170013428,
		Timezone:  "Australia/Darwin",
	},
	"YWLM": {
		ICAO:      "YWLM",
		IATA:      "NTL",
		Name:      "Newcastle Airport",
		City:      "Williamtown",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 31,
		Latitude:  -32.7949981689,
		Longitude: 151.8339996338,
		Timezone:  "Australia/Sydney",
	},
	"YWLN": {
		ICAO:      "YWLN",
		Name:      "Wooltana Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -30.4216995239,
		Longitude: 139.4360046387,
		Timezone:  "Australia/Adelaide",
	},
	"YWLP": {
		ICAO:      "YWLP",
		Name:      "Wilsons Promontory Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -39.1310997009,
		Longitude: 146.4250030518,
		Timezone:  "Australia/Melbourne",
	},
	"YWLU": {
		ICAO:      "YWLU",
		IATA:      "WUN",
		Name:      "Wiluna Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 1649,
		Latitude:  -26.6291999817,
		Longitude: 120.2210006714,
		Timezone:  "Australia/Perth",
	},
	"YWLX": {
		ICAO:      "YWLX",
		Name:      "Wallacia Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -33.8666992188,
		Longitude: 150.6499938965,
		Timezone:  "Australia/Sydney",
	},
	"YWMA": {
		ICAO:      "YWMA",
		Name:      "Wonnaminta Stat Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 610,
		Latitude:  -30.6511001587,
		Longitude: 142.3419952393,
		Timezone:  "Australia/Sydney",
	},
	"YWMC": {
		ICAO:      "YWMC",
		Name:      "William Creek Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 91,
		Latitude:  -28.9067001343,
		Longitude: 136.3419952393,
		Timezone:  "Australia/Adelaide",
	},
	"YWMG": {
		ICAO:      "YWMG",
		Name:      "Weilmoringle Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -29.2332992554,
		Longitude: 146.9329986572,
		Timezone:  "Australia/Sydney",
	},
	"YWML": {
		ICAO:      "YWML",
		Name:      "Westmoreland Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -17.3332996368,
		Longitude: 138.233001709,
		Timezone:  "Australia/Brisbane",
	},
	"YWMM": {
		ICAO:      "YWMM",
		Name:      "Wollomombi Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -30.5333003998,
		Longitude: 152.0829925537,
		Timezone:  "Australia/Sydney",
	},
	"YWMP": {
		ICAO:      "YWMP",
		IATA:      "WPK",
		Name:      "Wrotham Park Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 152,
		Latitude:  -16.6583003998,
		Longitude: 144.0019989014,
		Timezone:  "Australia/Brisbane",
	},
	"YWMY": {
		ICAO:      "YWMY",
		Name:      "Williambury Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -23.8666992188,
		Longitude: 115.1500015259,
		Timezone:  "Australia/Perth",
	},
	"YWNA": {
		ICAO:      "YWNA",
		Name:      "Wilgena Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -30.7667007446,
		Longitude: 134.7299957275,
		Timezone:  "Australia/Adelaide",
	},
	"YWND": {
		ICAO:      "YWND",
		IATA:      "WDI",
		Name:      "Wondai Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 320,
		Latitude:  -26.2833003998,
		Longitude: 151.858001709,
		Timezone:  "Australia/Brisbane",
	},
	"YWNI": {
		ICAO:      "YWNI",
		Name:      "Wathanin Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -13.6999998093,
		Longitude: 141.5500030518,
		Timezone:  "Australia/Brisbane",
	},
	"YWNL": {
		ICAO:      "YWNL",
		Name:      "Wingellina Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -26.0666999817,
		Longitude: 128.9499969482,
		Timezone:  "Australia/Perth",
	},
	"YWNO": {
		ICAO:      "YWNO",
		Name:      "Wonganoo Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -27.125,
		Longitude: 121.3330001831,
		Timezone:  "Australia/Perth",
	},
	"YWNS": {
		ICAO:      "YWNS",
		Name:      "Wandsworth Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -25.0499992371,
		Longitude: 143.6670074463,
		Timezone:  "Australia/Brisbane",
	},
	"YWOH": {
		ICAO:      "YWOH",
		Name:      "Wongan Hills Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -30.8666992188,
		Longitude: 116.733001709,
		Timezone:  "Australia/Perth",
	},
	"YWOL": {
		ICAO:      "YWOL",
		IATA:      "WOL",
		Name:      "Wollongong Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 31,
		Latitude:  -34.5611000061,
		Longitude: 150.7890014648,
		Timezone:  "Australia/Sydney",
	},
	"YWOM": {
		ICAO:      "YWOM",
		Name:      "Woolomin Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -31.3166999817,
		Longitude: 151.1329956055,
		Timezone:  "Australia/Sydney",
	},
	"YWON": {
		ICAO:      "YWON",
		Name:      "Wonthaggi Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -38.471698761,
		Longitude: 145.6230010986,
		Timezone:  "Australia/Melbourne",
	},
	"YWOR": {
		ICAO:      "YWOR",
		IATA:      "WLL",
		Name:      "Wollogorang Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -17.2199993134,
		Longitude: 137.9230041504,
		Timezone:  "Australia/Darwin",
	},
	"YWOV": {
		ICAO:      "YWOV",
		Name:      "Woodvale Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -36.6333007813,
		Longitude: 144.1829986572,
		Timezone:  "Australia/Melbourne",
	},
	"YWOX": {
		ICAO:      "YWOX",
		Name:      "Woorlba Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -32.4000015259,
		Longitude: 124.0329971313,
		Timezone:  "Australia/Perth",
	},
	"YWPA": {
		ICAO:      "YWPA",
		Name:      "Wirrealpa Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -31.1333007813,
		Longitude: 138.9669952393,
		Timezone:  "Australia/Adelaide",
	},
	"YWPE": {
		ICAO:      "YWPE",
		Name:      "Walpole Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -34.9500007629,
		Longitude: 116.6999969482,
		Timezone:  "Australia/Perth",
	},
	"YWPL": {
		ICAO:      "YWPL",
		Name:      "Winning Pool North Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -23.1000003815,
		Longitude: 114.5329971313,
		Timezone:  "Australia/Perth",
	},
	"YWRA": {
		ICAO:      "YWRA",
		Name:      "Wooroona Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -20.4817008972,
		Longitude: 138.1679992676,
		Timezone:  "Australia/Brisbane",
	},
	"YWRC": {
		ICAO:      "YWRC",
		Name:      "Wave Rock Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -32.426700592,
		Longitude: 118.9079971313,
		Timezone:  "Australia/Perth",
	},
	"YWRE": {
		ICAO:      "YWRE",
		Name:      "Wirralie Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -22.2700004578,
		Longitude: 146.0500030518,
		Timezone:  "Australia/Brisbane",
	},
	"YWRL": {
		ICAO:      "YWRL",
		Name:      "Warialda Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 347,
		Latitude:  -29.5333003998,
		Longitude: 150.5330047607,
		Timezone:  "Australia/Sydney",
	},
	"YWRN": {
		ICAO:      "YWRN",
		IATA:      "QRR",
		Name:      "Warren Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 669,
		Latitude:  -31.7332992554,
		Longitude: 147.8029937744,
		Timezone:  "Australia/Sydney",
	},
	"YWRR": {
		ICAO:      "YWRR",
		Name:      "Warroora Homestead Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -23.4832992554,
		Longitude: 113.8000030518,
		Timezone:  "Australia/Perth",
	},
	"YWRT": {
		ICAO:      "YWRT",
		Name:      "Waratah Airport",
		State:     "Tasmania",
		Country:   "AU",
		Latitude:  -41.4500007629,
		Longitude: 145.5500030518,
		Timezone:  "Australia/Hobart",
	},
	"YWRV": {
		ICAO:      "YWRV",
		Name:      "Walker River Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Latitude:  -13.5932998657,
		Longitude: 135.7550048828,
		Timezone:  "Australia/Darwin",
	},
	"YWSD": {
		ICAO:      "YWSD",
		Name:      "Westward Downs Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -30.7250003815,
		Longitude: 141.3999938965,
		Timezone:  "Australia/Sydney",
	},
	"YWSG": {
		ICAO:      "YWSG",
		Name:      "Watts Bridge Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 61,
		Latitude:  -27.0983009338,
		Longitude: 152.4600067139,
		Timezone:  "Australia/Brisbane",
	},
	"YWSI": {
		ICAO:      "YWSI",
		Name:      "Wirrida Siding Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -29.5666999817,
		Longitude: 134.516998291,
		Timezone:  "Australia/Adelaide",
	},
	"YWSL": {
		ICAO:      "YWSL",
		IATA:      "SXE",
		Name:      "West Sale Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 93,
		Latitude:  -38.0917015076,
		Longitude: 146.964996338,
		Timezone:  "Australia/Melbourne",
	},
	"YWSX": {
		ICAO:      "YWSX",
		Name:      "Westonia Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -31.3332996368,
		Longitude: 118.6669998169,
		Timezone:  "Australia/Perth",
	},
	"YWTL": {
		ICAO:      "YWTL",
		IATA:      "WLO",
		Name:      "Waterloo Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 132,
		Latitude:  -16.6299991608,
		Longitude: 129.3200073242,
		Timezone:  "Australia/Darwin",
	},
	"YWTN": {
		ICAO:      "YWTN",
		IATA:      "WIN",
		Name:      "Winton Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 638,
		Latitude:  -22.3635997772,
		Longitude: 143.0859985352,
		Timezone:  "Australia/Brisbane",
	},
	"YWTO": {
		ICAO:      "YWTO",
		Name:      "Wentworth Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 37,
		Latitude:  -34.0882987976,
		Longitude: 141.891998291,
		Timezone:  "Australia/Sydney",
	},
	"YWTV": {
		ICAO:      "YWTV",
		Name:      "Watson River Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -13.2166996002,
		Longitude: 142.233001709,
		Timezone:  "Australia/Brisbane",
	},
	"YWUD": {
		ICAO:      "YWUD",
		IATA:      "WUD",
		Name:      "Wudinna Airport",
		State:     "South-Australia",
		Country:   "AU",
		Elevation: 310,
		Latitude:  -33.0433006287,
		Longitude: 135.4470062256,
		Timezone:  "Australia/Adelaide",
	},
	"YWVA": {
		ICAO:      "YWVA",
		Name:      "Warnervale Airport",
		City:      "Warnervale",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 25,
		Latitude:  -33.240278,
		Longitude: 151.429722,
		Timezone:  "Australia/Sydney",
	},
	"YWVL": {
		ICAO:      "YWVL",
		Name:      "Woodville Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 3707,
		Latitude:  -30.4160995483,
		Longitude: 151.7550048828,
		Timezone:  "Australia/Sydney",
	},
	"YWWA": {
		ICAO:      "YWWA",
		IATA:      "WEW",
		Name:      "Wee Waa Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 190,
		Latitude:  -30.2583007813,
		Longitude: 149.4080047607,
		Timezone:  "Australia/Sydney",
	},
	"YWWG": {
		ICAO:      "YWWG",
		IATA:      "WRW",
		Name:      "Warrawagine Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -20.8464596515,
		Longitude: 120.705327988,
		Timezone:  "Australia/Perth",
	},
	"YWWH": {
		ICAO:      "YWWH",
		Name:      "Wentworth Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -22.1000003815,
		Longitude: 147.6999969482,
		Timezone:  "Australia/Brisbane",
	},
	"YWWI": {
		ICAO:      "YWWI",
		IATA:      "WWI",
		Name:      "Woodie Woodie Airport",
		City:      "Woodie Woodie",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -21.6627998352,
		Longitude: 121.2340011597,
		Timezone:  "Australia/Perth",
	},
	"YWWL": {
		ICAO:      "YWWL",
		IATA:      "WWY",
		Name:      "West Wyalong Airport",
		City:      "West Wyalong",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 859,
		Latitude:  -33.9371986389,
		Longitude: 147.190994263,
		Timezone:  "Australia/Sydney",
	},
	"YWYA": {
		ICAO:      "YWYA",
		Name:      "Wyandra Airport",
		State:     "Queensland",
		Country:   "AU",
		Elevation: 244,
		Latitude:  -27.2667007446,
		Longitude: 145.9900054932,
		Timezone:  "Australia/Brisbane",
	},
	"YWYB": {
		ICAO:      "YWYB",
		Name:      "Wynbring Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -30.5617008209,
		Longitude: 133.5330047607,
		Timezone:  "Australia/Adelaide",
	},
	"YWYF": {
		ICAO:      "YWYF",
		Name:      "Wycheproof Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 107,
		Latitude:  -36.0583000183,
		Longitude: 143.2429962158,
		Timezone:  "Australia/Melbourne",
	},
	"YWYM": {
		ICAO:      "YWYM",
		IATA:      "WYN",
		Name:      "Wyndham Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 14,
		Latitude:  -15.5114002228,
		Longitude: 128.1529998779,
		Timezone:  "Australia/Perth",
	},
	"YWYY": {
		ICAO:      "YWYY",
		IATA:      "BWT",
		Name:      "Wynyard Airport",
		City:      "Burnie",
		State:     "Tasmania",
		Country:   "AU",
		Elevation: 62,
		Latitude:  -40.9989013672,
		Longitude: 145.7310028076,
		Timezone:  "Australia/Hobart",
	},
	"YXGS": {
		ICAO:      "YXGS",
		Name:      "Gosford Airport",
		City:      "Gosford",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 853,
		Latitude:  -33.3680992126,
		Longitude: 151.300003052,
		Timezone:  "Australia/Sydney",
	},
	"YXTA": {
		ICAO:      "YXTA",
		Name:      "Tangalooma Resort Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -27.2000007629,
		Longitude: 153.3829956055,
		Timezone:  "Australia/Brisbane",
	},
	"YYAA": {
		ICAO:      "YYAA",
		Name:      "Yandama Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -29.6833000183,
		Longitude: 141.4250030518,
		Timezone:  "Australia/Sydney",
	},
	"YYAC": {
		ICAO:      "YYAC",
		Name:      "Yacamunda Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -21.3833007813,
		Longitude: 147.1000061035,
		Timezone:  "Australia/Brisbane",
	},
	"YYAG": {
		ICAO:      "YYAG",
		Name:      "Yagga Yagga Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -20.9666996002,
		Longitude: 128.0829925537,
		Timezone:  "Australia/Perth",
	},
	"YYAK": {
		ICAO:      "YYAK",
		Name:      "Yalkulka Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -16.7332992554,
		Longitude: 145.3329925537,
		Timezone:  "Australia/Brisbane",
	},
	"YYAL": {
		ICAO:      "YYAL",
		IATA:      "YLG",
		Name:      "Yalgoo Airport",
		City:      "Yalgoo",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -28.3553009033,
		Longitude: 116.683998108,
		Timezone:  "Australia/Perth",
	},
	"YYAS": {
		ICAO:      "YYAS",
		Name:      "Yass Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -34.8300018311,
		Longitude: 149.0420074463,
		Timezone:  "Australia/Sydney",
	},
	"YYBE": {
		ICAO:      "YYBE",
		Name:      "Yarrabee Mine Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -23.2667007446,
		Longitude: 149.016998291,
		Timezone:  "Australia/Brisbane",
	},
	"YYDE": {
		ICAO:      "YYDE",
		Name:      "Yandee Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -21.3332996368,
		Longitude: 118.8669967651,
		Timezone:  "Australia/Perth",
	},
	"YYDM": {
		ICAO:      "YYDM",
		Name:      "Yundamindera Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -29.1166992188,
		Longitude: 122.0329971313,
		Timezone:  "Australia/Perth",
	},
	"YYEA": {
		ICAO:      "YYEA",
		Name:      "Yeaburn Airport",
		State:     "Victoria",
		Country:   "AU",
		Latitude:  -37.1727981567,
		Longitude: 145.3220062256,
		Timezone:  "Australia/Melbourne",
	},
	"YYER": {
		ICAO:      "YYER",
		Name:      "Yerilla Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -29.4675998688,
		Longitude: 121.8300018311,
		Timezone:  "Australia/Perth",
	},
	"YYGG": {
		ICAO:      "YYGG",
		Name:      "Yagga Yagga Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -20.8906002045,
		Longitude: 127.9489974976,
		Timezone:  "Australia/Perth",
	},
	"YYKI": {
		ICAO:      "YYKI",
		IATA:      "OKR",
		Name:      "Yorke Island Airport",
		City:      "Yorke Island",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -9.7570295334,
		Longitude: 143.4109954834,
		Timezone:  "Australia/Brisbane",
	},
	"YYLD": {
		ICAO:      "YYLD",
		Name:      "Yalda Downs Homestead Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Latitude:  -30.2614002228,
		Longitude: 142.9770050049,
		Timezone:  "Australia/Sydney",
	},
	"YYLG": {
		ICAO:      "YYLG",
		Name:      "Yallalong Homestead Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -27.4263000488,
		Longitude: 115.5179977417,
		Timezone:  "Australia/Perth",
	},
	"YYLR": {
		ICAO:      "YYLR",
		IATA:      "KYF",
		Name:      "Yeelirrie Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -27.2770600378,
		Longitude: 120.095672607,
		Timezone:  "Australia/Perth",
	},
	"YYMI": {
		ICAO:      "YYMI",
		IATA:      "XMY",
		Name:      "Yam Island Airport",
		City:      "Yam Island",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -9.9011096954,
		Longitude: 142.7760009766,
		Timezone:  "Australia/Brisbane",
	},
	"YYND": {
		ICAO:      "YYND",
		IATA:      "YUE",
		Name:      "Yuendumu Airport",
		State:     "Northern-Territory",
		Country:   "AU",
		Elevation: 2205,
		Latitude:  -22.2541999817,
		Longitude: 131.7819976807,
		Timezone:  "Australia/Darwin",
	},
	"YYNG": {
		ICAO:      "YYNG",
		IATA:      "NGA",
		Name:      "Young Airport",
		State:     "New-South-Wales",
		Country:   "AU",
		Elevation: 1267,
		Latitude:  -34.2555999756,
		Longitude: 148.2480010986,
		Timezone:  "Australia/Sydney",
	},
	"YYNR": {
		ICAO:      "YYNR",
		Name:      "Yanrey Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Elevation: 80,
		Latitude:  -22.5167007446,
		Longitude: 114.8000030518,
		Timezone:  "Australia/Perth",
	},
	"YYOO": {
		ICAO:      "YYOO",
		Name:      "Yalymboo Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -31.8033008575,
		Longitude: 136.8600006104,
		Timezone:  "Australia/Adelaide",
	},
	"YYOR": {
		ICAO:      "YYOR",
		IATA:      "ORR",
		Name:      "Yorketown Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -35,
		Longitude: 137.6170043945,
		Timezone:  "Australia/Adelaide",
	},
	"YYRK": {
		ICAO:      "YYRK",
		Name:      "York Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -31.8551006317,
		Longitude: 116.7990036011,
		Timezone:  "Australia/Perth",
	},
	"YYRM": {
		ICAO:      "YYRM",
		Name:      "Yarram Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 15,
		Latitude:  -38.5666999817,
		Longitude: 146.7550048828,
		Timezone:  "Australia/Melbourne",
	},
	"YYRN": {
		ICAO:      "YYRN",
		Name:      "Yamarna Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -28.1539993286,
		Longitude: 123.672996521,
		Timezone:  "Australia/Perth",
	},
	"YYRW": {
		ICAO:      "YYRW",
		Name:      "Yarlarweelor Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -25.4582996368,
		Longitude: 117.9749984741,
		Timezone:  "Australia/Perth",
	},
	"YYTA": {
		ICAO:      "YYTA",
		IATA:      "KYI",
		Name:      "Yalata Mission Airport",
		City:      "Yalata Mission",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -31.4706001282,
		Longitude: 131.8249969482,
		Timezone:  "Australia/Adelaide",
	},
	"YYUM": {
		ICAO:      "YYUM",
		Name:      "Yuinmery Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -28.5629997253,
		Longitude: 119.0189971924,
		Timezone:  "Australia/Perth",
	},
	"YYUN": {
		ICAO:      "YYUN",
		Name:      "Yunta Airport",
		State:     "South-Australia",
		Country:   "AU",
		Latitude:  -32.5833015442,
		Longitude: 139.5500030518,
		Timezone:  "Australia/Adelaide",
	},
	"YYWA": {
		ICAO:      "YYWA",
		Name:      "Yowah Airport",
		State:     "Queensland",
		Country:   "AU",
		Latitude:  -27.9333000183,
		Longitude: 144.6170043945,
		Timezone:  "Australia/Brisbane",
	},
	"YYWG": {
		ICAO:      "YYWG",
		Name:      "Yarrawonga Airport",
		State:     "Victoria",
		Country:   "AU",
		Elevation: 423,
		Latitude:  -36.0289001465,
		Longitude: 146.029006958,
		Timezone:  "Australia/Melbourne",
	},
	"YZAN": {
		ICAO:      "YZAN",
		Name:      "Zanthus Airport",
		State:     "Western-Australia",
		Country:   "AU",
		Latitude:  -31.0333003998,
		Longitude: 123.5670013428,
		Timezone:  "Australia/Perth",
	},
	"YZHN": {
		ICAO:      "YZHN",
		Name:      "Zeehan Airport",
		State:     "Tasmania",
		Country:   "AU",
		Latitude:  -41.8833007813,
		Longitude: 145.3500061035,
		Timezone:  "Australia/Hobart",
	},
	"Z19M": {
		ICAO:      "Z19M",
		Name:      "Al Abdaliyah Highway Strip",
		City:      "Al Abdaliyah",
		Country:   "KW",
		Elevation: 330,
		Latitude:  29.04529953,
		Longitude: 47.7907981873,
		Timezone:  "Asia/Kuwait",
	},
	"Z23Z": {
		ICAO:      "Z23Z",
		Name:      "Paraguari Airport",
		City:      "Paraguari",
		State:     "Paraguari",
		Country:   "PY",
		Elevation: 500,
		Latitude:  -25.6000003815,
		Longitude: -57.1500015259,
		Timezone:  "America/Asuncion",
	},
	"ZBAA": {
		ICAO:      "ZBAA",
		IATA:      "PEK",
		Name:      "Beijing Capital International Airport",
		City:      "Beijing",
		State:     "Beijing",
		Country:   "CN",
		Elevation: 116,
		Latitude:  40.0801010132,
		Longitude: 116.5849990845,
		Timezone:  "Asia/Shanghai",
	},
	"ZBAD": {
		ICAO:      "ZBAD",
		IATA:      "PKX",
		Name:      "Beijing Daxing International Airport",
		City:      "Beijing",
		State:     "Beijing",
		Country:   "CN",
		Elevation: 98,
		Latitude:  39.509167,
		Longitude: 116.410556,
		Timezone:  "Asia/Shanghai",
	},
	"ZBBB": {
		ICAO:      "ZBBB",
		Name:      "Beijing Xijiao Airport",
		City:      "Beijing",
		State:     "Beijing",
		Country:   "CN",
		Latitude:  39.9608001709,
		Longitude: 116.2570037842,
		Timezone:  "Asia/Shanghai",
	},
	"ZBCF": {
		ICAO:      "ZBCF",
		IATA:      "CIF",
		Name:      "Chifeng Airport",
		City:      "Chifeng",
		State:     "Inner-Mongolia",
		Country:   "CN",
		Latitude:  42.2350006104,
		Longitude: 118.9079971313,
		Timezone:  "Asia/Shanghai",
	},
	"ZBCZ": {
		ICAO:      "ZBCZ",
		IATA:      "CIH",
		Name:      "Changzhi Airport",
		City:      "Changzhi",
		State:     "Shanxi",
		Country:   "CN",
		Latitude:  36.2475013733,
		Longitude: 113.1259994507,
		Timezone:  "Asia/Shanghai",
	},
	"ZBDS": {
		ICAO:      "ZBDS",
		IATA:      "DSN",
		Name:      "Ordos Ejin Horo Airport",
		City:      "Ordos",
		State:     "Inner-Mongolia",
		Country:   "CN",
		Elevation: 4557,
		Latitude:  39.49,
		Longitude: 109.861388889,
		Timezone:  "Asia/Shanghai",
	},
	"ZBDT": {
		ICAO:      "ZBDT",
		IATA:      "DAT",
		Name:      "Datong Airport",
		City:      "Datong",
		State:     "Shanxi",
		Country:   "CN",
		Elevation: 3442,
		Latitude:  40.0602989197,
		Longitude: 113.4820022583,
		Timezone:  "Asia/Shanghai",
	},
	"ZBER": {
		ICAO:      "ZBER",
		IATA:      "ERL",
		Name:      "Erenhot Saiwusu International Airport",
		City:      "Erenhot",
		State:     "Inner-Mongolia",
		Country:   "CN",
		Elevation: 3301,
		Latitude:  43.4225,
		Longitude: 112.096666667,
		Timezone:  "Asia/Shanghai",
	},
	"ZBGS": {
		ICAO:      "ZBGS",
		IATA:      "AEB",
		Name:      "Tian Yang Air Base",
		City:      "Baise",
		State:     "Guangxi",
		Country:   "CN",
		Latitude:  23.7206001282,
		Longitude: 106.959999084,
		Timezone:  "Asia/Shanghai",
	},
	"ZBHD": {
		ICAO:      "ZBHD",
		IATA:      "HDG",
		Name:      "Handan Airport",
		City:      "Handan",
		State:     "Hebei",
		Country:   "CN",
		Elevation: 229,
		Latitude:  36.5258333333,
		Longitude: 114.425555556,
		Timezone:  "Asia/Shanghai",
	},
	"ZBHH": {
		ICAO:      "ZBHH",
		IATA:      "HET",
		Name:      "Baita International Airport",
		City:      "Hohhot",
		State:     "Inner-Mongolia",
		Country:   "CN",
		Elevation: 3556,
		Latitude:  40.851398468,
		Longitude: 111.823997498,
		Timezone:  "Asia/Shanghai",
	},
	"ZBLA": {
		ICAO:      "ZBLA",
		IATA:      "HLD",
		Name:      "Dongshan Airport",
		City:      "Hailar",
		State:     "Inner-Mongolia",
		Country:   "CN",
		Elevation: 2169,
		Latitude:  49.2050018311,
		Longitude: 119.824996948,
		Timezone:  "Asia/Shanghai",
	},
	"ZBNY": {
		ICAO:      "ZBNY",
		IATA:      "NAY",
		Name:      "Beijing Nanyuan Airport",
		City:      "Beijing",
		State:     "Beijing",
		Country:   "CN",
		Latitude:  39.7827987671,
		Longitude: 116.3880004883,
		Timezone:  "Asia/Shanghai",
	},
	"ZBOW": {
		ICAO:      "ZBOW",
		IATA:      "BAV",
		Name:      "Baotou Airport",
		City:      "Baotou",
		State:     "Inner-Mongolia",
		Country:   "CN",
		Elevation: 3321,
		Latitude:  40.5600013733,
		Longitude: 109.9970016479,
		Timezone:  "Asia/Shanghai",
	},
	"ZBSH": {
		ICAO:      "ZBSH",
		IATA:      "SHP",
		Name:      "Shanhaiguan Airport",
		City:      "Qinhuangdao",
		State:     "Hebei",
		Country:   "CN",
		Elevation: 30,
		Latitude:  39.9681015015,
		Longitude: 119.731002808,
		Timezone:  "Asia/Shanghai",
	},
	"ZBSJ": {
		ICAO:      "ZBSJ",
		IATA:      "SJW",
		Name:      "Shijiazhuang Daguocun International Airport",
		City:      "Shijiazhuang",
		State:     "Hebei",
		Country:   "CN",
		Elevation: 233,
		Latitude:  38.2807006836,
		Longitude: 114.6969985962,
		Timezone:  "Asia/Shanghai",
	},
	"ZBTJ": {
		ICAO:      "ZBTJ",
		IATA:      "TSN",
		Name:      "Tianjin Binhai International Airport",
		City:      "Tianjin",
		State:     "Tianjin",
		Country:   "CN",
		Elevation: 10,
		Latitude:  39.1244010925,
		Longitude: 117.346000671,
		Timezone:  "Asia/Shanghai",
	},
	"ZBTL": {
		ICAO:      "ZBTL",
		IATA:      "TGO",
		Name:      "Tongliao Airport",
		City:      "Tongliao",
		State:     "Inner-Mongolia",
		Country:   "CN",
		Latitude:  43.5567016602,
		Longitude: 122.1999969482,
		Timezone:  "Asia/Shanghai",
	},
	"ZBUL": {
		ICAO:      "ZBUL",
		IATA:      "HLH",
		Name:      "Ulanhot Airport",
		City:      "Ulanhot",
		State:     "Inner-Mongolia",
		Country:   "CN",
		Latitude:  46.195333,
		Longitude: 122.008333,
		Timezone:  "Asia/Shanghai",
	},
	"ZBXH": {
		ICAO:      "ZBXH",
		IATA:      "XIL",
		Name:      "Xilinhot Airport",
		City:      "Xilinhot",
		State:     "Inner-Mongolia",
		Country:   "CN",
		Latitude:  43.915599823,
		Longitude: 115.9639968872,
		Timezone:  "Asia/Shanghai",
	},
	"ZBYC": {
		ICAO:      "ZBYC",
		IATA:      "YCU",
		Name:      "Yuncheng Guangong Airport",
		City:      "Yuncheng",
		State:     "Shanxi",
		Country:   "CN",
		Elevation: 1242,
		Latitude:  35.116391,
		Longitude: 111.031388889,
		Timezone:  "Asia/Shanghai",
	},
	"ZBYN": {
		ICAO:      "ZBYN",
		IATA:      "TYN",
		Name:      "Taiyuan Wusu Airport",
		City:      "Taiyuan",
		State:     "Shanxi",
		Country:   "CN",
		Elevation: 2575,
		Latitude:  37.7468986511,
		Longitude: 112.6279983521,
		Timezone:  "Asia/Shanghai",
	},
	"ZGBH": {
		ICAO:      "ZGBH",
		IATA:      "BHY",
		Name:      "Beihai Airport",
		City:      "Beihai",
		State:     "Guangxi",
		Country:   "CN",
		Latitude:  21.5394001007,
		Longitude: 109.2939987183,
		Timezone:  "Asia/Shanghai",
	},
	"ZGCD": {
		ICAO:      "ZGCD",
		IATA:      "CGD",
		Name:      "Changde Airport",
		City:      "Changde",
		State:     "Hunan",
		Country:   "CN",
		Elevation: 128,
		Latitude:  28.9188995361,
		Longitude: 111.63999939,
		Timezone:  "Asia/Shanghai",
	},
	"ZGCJ": {
		ICAO:      "ZGCJ",
		IATA:      "HJJ",
		Name:      "Zhijiang Airport",
		City:      "Huaihua",
		State:     "Hunan",
		Country:   "CN",
		Elevation: 882,
		Latitude:  27.4411111111,
		Longitude: 109.7,
		Timezone:  "Asia/Shanghai",
	},
	"ZGDY": {
		ICAO:      "ZGDY",
		IATA:      "DYG",
		Name:      "Dayong Airport",
		City:      "Dayong",
		State:     "Hunan",
		Country:   "CN",
		Elevation: 692,
		Latitude:  29.1028003693,
		Longitude: 110.4430007935,
		Timezone:  "Asia/Shanghai",
	},
	"ZGFS": {
		ICAO:      "ZGFS",
		IATA:      "FUO",
		Name:      "Foshan Shadi Airport",
		City:      "Foshan",
		State:     "Guangdong",
		Country:   "CN",
		Elevation: 6,
		Latitude:  23.0832996368,
		Longitude: 113.069999695,
		Timezone:  "Asia/Shanghai",
	},
	"ZGGG": {
		ICAO:      "ZGGG",
		IATA:      "CAN",
		Name:      "Guangzhou Baiyun International Airport",
		City:      "Guangzhou",
		State:     "Guangdong",
		Country:   "CN",
		Elevation: 50,
		Latitude:  23.3924007416,
		Longitude: 113.2990036011,
		Timezone:  "Asia/Shanghai",
	},
	"ZGHA": {
		ICAO:      "ZGHA",
		IATA:      "CSX",
		Name:      "Changsha Huanghua Airport",
		City:      "Changsha",
		State:     "Hunan",
		Country:   "CN",
		Elevation: 217,
		Latitude:  28.1891994476,
		Longitude: 113.2200012207,
		Timezone:  "Asia/Shanghai",
	},
	"ZGHY": {
		ICAO:      "ZGHY",
		IATA:      "HNY",
		Name:      "Hengyang Airport",
		City:      "Hengyang",
		State:     "Hunan",
		Country:   "CN",
		Latitude:  26.9053001404,
		Longitude: 112.6279983521,
		Timezone:  "Asia/Shanghai",
	},
	"ZGHZ": {
		ICAO:      "ZGHZ",
		IATA:      "HUZ",
		Name:      "Huizhou Airport",
		City:      "Huizhou",
		State:     "Guangdong",
		Country:   "CN",
		Elevation: 50,
		Latitude:  23.0499992371,
		Longitude: 114.599998474,
		Timezone:  "Asia/Shanghai",
	},
	"ZGKL": {
		ICAO:      "ZGKL",
		IATA:      "KWL",
		Name:      "Guilin Liangjiang International Airport",
		City:      "Guilin City",
		State:     "Guangxi",
		Country:   "CN",
		Elevation: 570,
		Latitude:  25.2180995941,
		Longitude: 110.0390014648,
		Timezone:  "Asia/Shanghai",
	},
	"ZGLD": {
		ICAO:      "ZGLD",
		Name:      "Luoding Sulong Airport.",
		City:      "Louding City",
		State:     "Guangdong",
		Country:   "CN",
		Elevation: 190,
		Latitude:  22.711169,
		Longitude: 111.60134,
		Timezone:  "Asia/Shanghai",
	},
	"ZGLG": {
		ICAO:      "ZGLG",
		IATA:      "LLF",
		Name:      "Lingling Airport",
		City:      "Yongzhou",
		State:     "Hunan",
		Country:   "CN",
		Elevation: 340,
		Latitude:  26.338661,
		Longitude: 111.610043,
		Timezone:  "Asia/Shanghai",
	},
	"ZGMX": {
		ICAO:      "ZGMX",
		IATA:      "MXZ",
		Name:      "Meixian Airport",
		City:      "Meixian",
		State:     "Guangdong",
		Country:   "CN",
		Latitude:  24.3500003815,
		Longitude: 116.1330032349,
		Timezone:  "Asia/Shanghai",
	},
	"ZGNN": {
		ICAO:      "ZGNN",
		IATA:      "NNG",
		Name:      "Nanning Wuxu Airport",
		City:      "Nanning",
		State:     "Guangxi",
		Country:   "CN",
		Elevation: 421,
		Latitude:  22.6082992554,
		Longitude: 108.1719970703,
		Timezone:  "Asia/Shanghai",
	},
	"ZGOW": {
		ICAO:      "ZGOW",
		IATA:      "SWA",
		Name:      "Shantou Waisha Airport",
		City:      "Shantou",
		State:     "Guangdong",
		Country:   "CN",
		Latitude:  23.4269008636,
		Longitude: 116.7620010376,
		Timezone:  "Asia/Shanghai",
	},
	"ZGSD": {
		ICAO:      "ZGSD",
		IATA:      "ZUH",
		Name:      "Zhuhai Airport",
		City:      "Zhuhai",
		State:     "Guangdong",
		Country:   "CN",
		Latitude:  22.0063991547,
		Longitude: 113.3759994507,
		Timezone:  "Asia/Shanghai",
	},
	"ZGSZ": {
		ICAO:      "ZGSZ",
		IATA:      "SZX",
		Name:      "Shenzhen Bao'an International Airport",
		City:      "Shenzhen",
		State:     "Guangdong",
		Country:   "CN",
		Elevation: 13,
		Latitude:  22.6392993927,
		Longitude: 113.8109970093,
		Timezone:  "Asia/Shanghai",
	},
	"ZGWZ": {
		ICAO:      "ZGWZ",
		IATA:      "WUZ",
		Name:      "Changzhoudao Airport",
		City:      "Wuzhou",
		State:     "Guangxi",
		Country:   "CN",
		Elevation: 89,
		Latitude:  23.4566993713,
		Longitude: 111.2480010986,
		Timezone:  "Asia/Shanghai",
	},
	"ZGXN": {
		ICAO:      "ZGXN",
		IATA:      "XIN",
		Name:      "Xingning Airport",
		City:      "Xingning",
		State:     "Guangdong",
		Country:   "CN",
		Latitude:  24.1492004395,
		Longitude: 115.7580032349,
		Timezone:  "Asia/Shanghai",
	},
	"ZGZH": {
		ICAO:      "ZGZH",
		IATA:      "LZH",
		Name:      "Bailian Airport",
		City:      "Liuzhou",
		State:     "Guangxi",
		Country:   "CN",
		Elevation: 295,
		Latitude:  24.2075004578,
		Longitude: 109.3909988403,
		Timezone:  "Asia/Shanghai",
	},
	"ZGZJ": {
		ICAO:      "ZGZJ",
		IATA:      "ZHA",
		Name:      "Zhanjiang Airport",
		City:      "Zhanjiang",
		State:     "Guangdong",
		Country:   "CN",
		Latitude:  21.2143993378,
		Longitude: 110.358001709,
		Timezone:  "Asia/Shanghai",
	},
	"ZHAY": {
		ICAO:      "ZHAY",
		IATA:      "AYN",
		Name:      "Anyang Airport",
		City:      "Anyang",
		State:     "Henan",
		Country:   "CN",
		Latitude:  36.1338996887,
		Longitude: 114.34400177,
		Timezone:  "Asia/Shanghai",
	},
	"ZHCC": {
		ICAO:      "ZHCC",
		IATA:      "CGO",
		Name:      "Xinzheng Airport",
		City:      "Zhengzhou",
		State:     "Henan",
		Country:   "CN",
		Elevation: 495,
		Latitude:  34.5196990967,
		Longitude: 113.841003418,
		Timezone:  "Asia/Shanghai",
	},
	"ZHES": {
		ICAO:      "ZHES",
		IATA:      "ENH",
		Name:      "Enshi Airport",
		City:      "Enshi",
		State:     "Hubei",
		Country:   "CN",
		Elevation: 1605,
		Latitude:  30.3202991486,
		Longitude: 109.48500061,
		Timezone:  "Asia/Shanghai",
	},
	"ZHGH": {
		ICAO:      "ZHGH",
		IATA:      "LHK",
		Name:      "Guangzhou MR Air Base",
		City:      "Guanghua",
		State:     "Hubei",
		Country:   "CN",
		Latitude:  32.3894004822,
		Longitude: 111.6949996948,
		Timezone:  "Asia/Shanghai",
	},
	"ZHHH": {
		ICAO:      "ZHHH",
		IATA:      "WUH",
		Name:      "Wuhan Tianhe International Airport",
		City:      "Wuhan",
		State:     "Hubei",
		Country:   "CN",
		Elevation: 113,
		Latitude:  30.7838001251,
		Longitude: 114.2080001831,
		Timezone:  "Asia/Shanghai",
	},
	"ZHLY": {
		ICAO:      "ZHLY",
		IATA:      "LYA",
		Name:      "Luoyang Airport",
		City:      "Luoyang",
		State:     "Henan",
		Country:   "CN",
		Elevation: 840,
		Latitude:  34.7411003113,
		Longitude: 112.388000488,
		Timezone:  "Asia/Shanghai",
	},
	"ZHNY": {
		ICAO:      "ZHNY",
		IATA:      "NNY",
		Name:      "Nanyang Airport",
		City:      "Nanyang",
		State:     "Henan",
		Country:   "CN",
		Latitude:  32.9808006287,
		Longitude: 112.6149978638,
		Timezone:  "Asia/Shanghai",
	},
	"ZHSS": {
		ICAO:      "ZHSS",
		IATA:      "SHS",
		Name:      "Shashi Airport",
		City:      "Shashi",
		State:     "Hubei",
		Country:   "CN",
		Latitude:  30.3243999481,
		Longitude: 112.28099823,
		Timezone:  "Asia/Shanghai",
	},
	"ZHXF": {
		ICAO:      "ZHXF",
		IATA:      "XFN",
		Name:      "Xiangfan Airport",
		City:      "Xiangfan",
		State:     "Hubei",
		Country:   "CN",
		Latitude:  32.1506004333,
		Longitude: 112.2910003662,
		Timezone:  "Asia/Shanghai",
	},
	"ZHYC": {
		ICAO:      "ZHYC",
		IATA:      "YIH",
		Name:      "Yichang Airport",
		City:      "Yichang",
		State:     "Hubei",
		Country:   "CN",
		Latitude:  30.556549722,
		Longitude: 111.479988333,
		Timezone:  "Asia/Shanghai",
	},
	"ZJHK": {
		ICAO:      "ZJHK",
		IATA:      "HAK",
		Name:      "Haikou Meilan International Airport",
		City:      "Haikou",
		State:     "Hainan",
		Country:   "CN",
		Elevation: 75,
		Latitude:  19.9349002838,
		Longitude: 110.4589996338,
		Timezone:  "Asia/Shanghai",
	},
	"ZJSY": {
		ICAO:      "ZJSY",
		IATA:      "SYX",
		Name:      "Sanya Phoenix International Airport",
		City:      "Sanya",
		State:     "Hainan",
		Country:   "CN",
		Elevation: 92,
		Latitude:  18.3029003143,
		Longitude: 109.4120025635,
		Timezone:  "Asia/Shanghai",
	},
	"ZJYX": {
		ICAO:      "ZJYX",
		Name:      "Woody Island Airport",
		City:      "Woody Island",
		Country:   "CN",
		Latitude:  16.8327999115,
		Longitude: 112.34400177,
		Timezone:  "Asia/Shanghai",
	},
	"ZKHM": {
		ICAO:      "ZKHM",
		IATA:      "RGO",
		Name:      "Orang Airport",
		State:     "Hamgyong-bukto",
		Country:   "KP",
		Elevation: 12,
		Latitude:  41.4281005859,
		Longitude: 129.647003174,
		Timezone:  "Asia/Pyongyang",
	},
	"ZKPY": {
		ICAO:      "ZKPY",
		IATA:      "FNJ",
		Name:      "Sunan International Airport",
		City:      "Pyongyang",
		State:     "South-Pyongan",
		Country:   "KP",
		Elevation: 117,
		Latitude:  39.2240982056,
		Longitude: 125.6699981689,
		Timezone:  "Asia/Pyongyang",
	},
	"ZKSD": {
		ICAO:      "ZKSD",
		IATA:      "DSO",
		Name:      "Sondok Airport",
		State:     "Hamgyong-namdo",
		Country:   "KP",
		Elevation: 12,
		Latitude:  39.7452011108,
		Longitude: 127.473999023,
		Timezone:  "Asia/Pyongyang",
	},
	"ZKSE": {
		ICAO:      "ZKSE",
		IATA:      "YJS",
		Name:      "Samjiyon Airport",
		City:      "Samjiyon",
		State:     "Yanggang-do",
		Country:   "KP",
		Elevation: 4547,
		Latitude:  41.9058990479,
		Longitude: 128.410003662,
		Timezone:  "Asia/Pyongyang",
	},
	"ZLAK": {
		ICAO:      "ZLAK",
		IATA:      "AKA",
		Name:      "Ankang Airport",
		City:      "Ankang",
		State:     "Shaanxi",
		Country:   "CN",
		Latitude:  32.7080993652,
		Longitude: 108.9309997559,
		Timezone:  "Asia/Shanghai",
	},
	"ZLDH": {
		ICAO:      "ZLDH",
		IATA:      "DNH",
		Name:      "Dunhuang Airport",
		City:      "Dunhuang",
		State:     "Gansu",
		Country:   "CN",
		Latitude:  40.1610984802,
		Longitude: 94.8091964722,
		Timezone:  "Asia/Shanghai",
	},
	"ZLGM": {
		ICAO:      "ZLGM",
		IATA:      "GOQ",
		Name:      "Golmud Airport",
		City:      "Golmud",
		State:     "Qinghai",
		Country:   "CN",
		Latitude:  36.4006004333,
		Longitude: 94.7861022949,
		Timezone:  "Asia/Shanghai",
	},
	"ZLGY": {
		ICAO:      "ZLGY",
		IATA:      "GYU",
		Name:      "Guyuan Liupanshan Airport",
		City:      "Guyuan",
		State:     "Ningsia-Hui-Autonomous-Region",
		Country:   "CN",
		Elevation: 5696,
		Latitude:  36.0788888889,
		Longitude: 106.216944444,
		Timezone:  "Asia/Shanghai",
	},
	"ZLHZ": {
		ICAO:      "ZLHZ",
		IATA:      "HZG",
		Name:      "Hanzhong Airport",
		City:      "Hanzhong",
		State:     "Shaanxi",
		Country:   "CN",
		Latitude:  33.0635986328,
		Longitude: 107.0080032349,
		Timezone:  "Asia/Shanghai",
	},
	"ZLIC": {
		ICAO:      "ZLIC",
		IATA:      "INC",
		Name:      "Yinchuan Airport",
		City:      "Yinchuan",
		State:     "Ningsia-Hui-Autonomous-Region",
		Country:   "CN",
		Latitude:  38.4818992615,
		Longitude: 106.0090026855,
		Timezone:  "Asia/Shanghai",
	},
	"ZLJC": {
		ICAO:      "ZLJC",
		IATA:      "JIC",
		Name:      "Jinchuan Airport",
		City:      "Jinchang",
		State:     "Gansu",
		Country:   "CN",
		Elevation: 4740,
		Latitude:  38.5422222222,
		Longitude: 102.348333333,
		Timezone:  "Asia/Shanghai",
	},
	"ZLJN": {
		ICAO:      "ZLJN",
		IATA:      "JNG",
		Name:      "Jining Qufu Airport",
		City:      "Jining",
		State:     "Shandong",
		Country:   "CN",
		Elevation: 134,
		Latitude:  35.2927777778,
		Longitude: 116.346666667,
		Timezone:  "Asia/Shanghai",
	},
	"ZLJQ": {
		ICAO:      "ZLJQ",
		IATA:      "JGN",
		Name:      "Jiayuguan Airport",
		City:      "Jiayuguan",
		State:     "Gansu",
		Country:   "CN",
		Latitude:  39.8568992615,
		Longitude: 98.3414001465,
		Timezone:  "Asia/Shanghai",
	},
	"ZLLL": {
		ICAO:      "ZLLL",
		IATA:      "LHW",
		Name:      "Lanzhou Zhongchuan Airport",
		City:      "Lanzhou",
		State:     "Gansu",
		Country:   "CN",
		Elevation: 6388,
		Latitude:  36.5152015686,
		Longitude: 103.620002747,
		Timezone:  "Asia/Shanghai",
	},
	"ZLQY": {
		ICAO:      "ZLQY",
		IATA:      "IQN",
		Name:      "Qingyang Airport",
		City:      "Qingyang",
		State:     "Gansu",
		Country:   "CN",
		Latitude:  35.7997016907,
		Longitude: 107.6029968262,
		Timezone:  "Asia/Shanghai",
	},
	"ZLSN": {
		ICAO:      "ZLSN",
		IATA:      "SIA",
		Name:      "Xiguan Airport",
		City:      "Xi'an",
		State:     "Shaanxi",
		Country:   "CN",
		Latitude:  34.376701355,
		Longitude: 109.1200027466,
		Timezone:  "Asia/Shanghai",
	},
	"ZLTS": {
		ICAO:      "ZLTS",
		IATA:      "THQ",
		Name:      "Tianshui Maijishan Airport",
		City:      "Tianshui",
		State:     "Gansu",
		Country:   "CN",
		Elevation: 3590,
		Latitude:  34.5593986511,
		Longitude: 105.86000061,
		Timezone:  "Asia/Shanghai",
	},
	"ZLXN": {
		ICAO:      "ZLXN",
		IATA:      "XNN",
		Name:      "Xining Caojiabu Airport",
		City:      "Xining",
		State:     "Qinghai",
		Country:   "CN",
		Latitude:  36.5275001526,
		Longitude: 102.0429992676,
		Timezone:  "Asia/Shanghai",
	},
	"ZLXY": {
		ICAO:      "ZLXY",
		IATA:      "XIY",
		Name:      "Xi'an Xianyang International Airport",
		City:      "Xianyang",
		State:     "Shaanxi",
		Country:   "CN",
		Elevation: 1572,
		Latitude:  34.447101593,
		Longitude: 108.7519989014,
		Timezone:  "Asia/Shanghai",
	},
	"ZLYA": {
		ICAO:      "ZLYA",
		IATA:      "ENY",
		Name:      "Yan'an Airport",
		City:      "Yan'an",
		State:     "Shaanxi",
		Country:   "CN",
		Latitude:  36.6369018555,
		Longitude: 109.5540008545,
		Timezone:  "Asia/Shanghai",
	},
	"ZLYL": {
		ICAO:      "ZLYL",
		IATA:      "UYN",
		Name:      "Yulin Airport",
		City:      "Yulin",
		State:     "Shaanxi",
		Country:   "CN",
		Latitude:  38.2691993713,
		Longitude: 109.7310028076,
		Timezone:  "Asia/Shanghai",
	},
	"ZLZW": {
		ICAO:      "ZLZW",
		IATA:      "ZHY",
		Name:      "Zhongwei Shapotou Airport",
		City:      "Zhongwei",
		State:     "Ningxia Hui Autonomous Region",
		Country:   "CN",
		Elevation: 8202,
		Latitude:  37.572778,
		Longitude: 105.154444,
		Timezone:  "Asia/Shanghai",
	},
	"ZMAH": {
		ICAO:      "ZMAH",
		IATA:      "AVK",
		Name:      "Arvaikheer Airport",
		City:      "Arvaikheer",
		State:     "Oevorhangay",
		Country:   "MN",
		Elevation: 5932,
		Latitude:  46.2503013611,
		Longitude: 102.8020019531,
		Timezone:  "Asia/Ulaanbaatar",
	},
	"ZMAT": {
		ICAO:      "ZMAT",
		IATA:      "LTI",
		Name:      "Altai Airport",
		City:      "Altai",
		State:     "Govi-Altay",
		Country:   "MN",
		Elevation: 7260,
		Latitude:  46.3763999939,
		Longitude: 96.2210998535,
		Timezone:  "Asia/Hovd",
	},
	"ZMBD": {
		ICAO:      "ZMBD",
		Name:      "Binder Airport",
		City:      "Binder",
		State:     "Hentiy",
		Country:   "MN",
		Elevation: 3422,
		Latitude:  48.6067008972,
		Longitude: 110.608001709,
		Timezone:  "Asia/Ulaanbaatar",
	},
	"ZMBH": {
		ICAO:      "ZMBH",
		IATA:      "BYN",
		Name:      "Bayankhongor Airport",
		City:      "Bayankhongor",
		State:     "Bayanhongor",
		Country:   "MN",
		Elevation: 6085,
		Latitude:  46.1632995605,
		Longitude: 100.7040023804,
		Timezone:  "Asia/Ulaanbaatar",
	},
	"ZMBN": {
		ICAO:      "ZMBN",
		IATA:      "UGA",
		Name:      "Bulgan Airport",
		City:      "Bulgan",
		State:     "Bulgan",
		Country:   "MN",
		Elevation: 4311,
		Latitude:  48.8549995422,
		Longitude: 103.4759979248,
		Timezone:  "Asia/Ulaanbaatar",
	},
	"ZMBR": {
		ICAO:      "ZMBR",
		IATA:      "UGT",
		Name:      "Bulagtai Resort Airport",
		City:      "Umnugobitour",
		State:     "Oemnogovi",
		Country:   "MN",
		Elevation: 4659,
		Latitude:  43.749304182,
		Longitude: 104.114985466,
		Timezone:  "Asia/Ulaanbaatar",
	},
	"ZMBS": {
		ICAO:      "ZMBS",
		IATA:      "HBU",
		Name:      "Bulgan Sum Airport",
		State:     "Hovd",
		Country:   "MN",
		Elevation: 3921,
		Latitude:  46.1006011963,
		Longitude: 91.584197998,
		Timezone:  "Asia/Hovd",
	},
	"ZMBU": {
		ICAO:      "ZMBU",
		IATA:      "UUN",
		Name:      "Baruun Urt Airport",
		State:     "Suehbaatar",
		Country:   "MN",
		Elevation: 3205,
		Latitude:  46.6603012085,
		Longitude: 113.2850036621,
		Timezone:  "Asia/Choibalsan",
	},
	"ZMCD": {
		ICAO:      "ZMCD",
		IATA:      "COQ",
		Name:      "Choibalsan Airport",
		State:     "East-Aimak",
		Country:   "MN",
		Elevation: 2457,
		Latitude:  48.1357002258,
		Longitude: 114.6460037231,
		Timezone:  "Asia/Choibalsan",
	},
	"ZMCK": {
		ICAO:      "ZMCK",
		IATA:      "UBN",
		Name:      "Chinggis Khaan International Airport",
		City:      "Ulaanbaatar",
		State:     "Ulaanbaatar",
		Country:   "MN",
		Elevation: 4482,
		Latitude:  47.651581,
		Longitude: 106.821772,
		Timezone:  "Asia/Ulaanbaatar",
	},
	"ZMDZ": {
		ICAO:      "ZMDZ",
		IATA:      "DLZ",
		Name:      "Dalanzadgad Airport",
		City:      "Dalanzadgad",
		State:     "Oemnogovi",
		Country:   "MN",
		Elevation: 4787,
		Latitude:  43.5917015076,
		Longitude: 104.4300003052,
		Timezone:  "Asia/Ulaanbaatar",
	},
	"ZMHH": {
		ICAO:      "ZMHH",
		IATA:      "KHR",
		Name:      "Kharkhorin Airport",
		State:     "Oevorhangay",
		Country:   "MN",
		Elevation: 4759,
		Latitude:  47.246667,
		Longitude: 102.826111,
		Timezone:  "Asia/Ulaanbaatar",
	},
	"ZMHU": {
		ICAO:      "ZMHU",
		IATA:      "HJT",
		Name:      "Khujirt Airport",
		State:     "Oevorhangay",
		Country:   "MN",
		Elevation: 5522,
		Latitude:  46.9258003235,
		Longitude: 102.7730026245,
		Timezone:  "Asia/Ulaanbaatar",
	},
	"ZMKD": {
		ICAO:      "ZMKD",
		IATA:      "HVD",
		Name:      "Khovd Airport",
		City:      "Khovd",
		State:     "Hovd",
		Country:   "MN",
		Elevation: 4898,
		Latitude:  47.9541015625,
		Longitude: 91.6281967163,
		Timezone:  "Asia/Hovd",
	},
	"ZMMN": {
		ICAO:      "ZMMN",
		IATA:      "MXV",
		Name:      "Moron Airport",
		City:      "Moron",
		State:     "Hovsgol",
		Country:   "MN",
		Elevation: 4272,
		Latitude:  49.6632995605,
		Longitude: 100.0989990234,
		Timezone:  "Asia/Ulaanbaatar",
	},
	"ZMOT": {
		ICAO:      "ZMOT",
		Name:      "Oyu Tolgoi",
		State:     "Oemnogovi",
		Country:   "MN",
		Latitude:  43.0157638761,
		Longitude: 106.839991808,
		Timezone:  "Asia/Ulaanbaatar",
	},
	"ZMSH": {
		ICAO:      "ZMSH",
		Name:      "Sainshand Airport",
		State:     "East-Gobi-Aymag",
		Country:   "MN",
		Elevation: 3022,
		Latitude:  44.9831008911,
		Longitude: 110.1669998169,
		Timezone:  "Asia/Ulaanbaatar",
	},
	"ZMUB": {
		ICAO:      "ZMUB",
		IATA:      "ULN",
		Name:      "Chinggis Khaan International Airport",
		City:      "Ulan Bator",
		State:     "Ulaanbaatar",
		Country:   "MN",
		Elevation: 4364,
		Latitude:  47.8431015015,
		Longitude: 106.766998291,
		Timezone:  "Asia/Ulaanbaatar",
	},
	"ZMUG": {
		ICAO:      "ZMUG",
		IATA:      "ULO",
		Name:      "Ulaangom Airport",
		State:     "Uvs",
		Country:   "MN",
		Latitude:  49.973333,
		Longitude: 92.079722,
		Timezone:  "Asia/Hovd",
	},
	"ZMUL": {
		ICAO:      "ZMUL",
		IATA:      "ULG",
		Name:      "Ulgii Mongolei Airport",
		State:     "Bayan-Oelgiy",
		Country:   "MN",
		Elevation: 5732,
		Latitude:  48.9933013916,
		Longitude: 89.9225006104,
		Timezone:  "Asia/Hovd",
	},
	"ZPBS": {
		ICAO:      "ZPBS",
		IATA:      "BSD",
		Name:      "Baoshan Yunduan Airport",
		State:     "Yunnan",
		Country:   "CN",
		Latitude:  25.0533008575,
		Longitude: 99.168296814,
		Timezone:  "Asia/Shanghai",
	},
	"ZPDL": {
		ICAO:      "ZPDL",
		IATA:      "DLU",
		Name:      "Dali Airport",
		City:      "Xiaguan",
		State:     "Yunnan",
		Country:   "CN",
		Latitude:  25.6494007111,
		Longitude: 100.3190002441,
		Timezone:  "Asia/Shanghai",
	},
	"ZPDQ": {
		ICAO:      "ZPDQ",
		IATA:      "DIG",
		Name:      "Diqing Airport",
		City:      "Shangri-La",
		State:     "Yunnan",
		Country:   "CN",
		Latitude:  27.7936000824,
		Longitude: 99.6772003174,
		Timezone:  "Asia/Shanghai",
	},
	"ZPJH": {
		ICAO:      "ZPJH",
		IATA:      "JHG",
		Name:      "Xishuangbanna Gasa Airport",
		City:      "Jinghong",
		State:     "Yunnan",
		Country:   "CN",
		Elevation: 1815,
		Latitude:  21.9738998413,
		Longitude: 100.7600021362,
		Timezone:  "Asia/Shanghai",
	},
	"ZPLC": {
		ICAO:      "ZPLC",
		IATA:      "LNJ",
		Name:      "Lincang Airport",
		City:      "Lincang",
		State:     "Yunnan",
		Country:   "CN",
		Elevation: 6127,
		Latitude:  23.738333,
		Longitude: 100.025,
		Timezone:  "Asia/Shanghai",
	},
	"ZPLJ": {
		ICAO:      "ZPLJ",
		IATA:      "LJG",
		Name:      "Lijiang Airport",
		City:      "Lijiang",
		State:     "Yunnan",
		Country:   "CN",
		Latitude:  26.6800003052,
		Longitude: 100.246002197,
		Timezone:  "Asia/Shanghai",
	},
	"ZPLX": {
		ICAO:      "ZPLX",
		IATA:      "LUM",
		Name:      "Mangshi Airport",
		City:      "Luxi",
		State:     "Yunnan",
		Country:   "CN",
		Elevation: 2890,
		Latitude:  24.4011001587,
		Longitude: 98.5317001343,
		Timezone:  "Asia/Shanghai",
	},
	"ZPPP": {
		ICAO:      "ZPPP",
		IATA:      "KMG",
		Name:      "Kunming Wujiaba International Airport",
		City:      "Kunming",
		State:     "Yunnan",
		Country:   "CN",
		Elevation: 6217,
		Latitude:  24.9923992157,
		Longitude: 102.7440032959,
		Timezone:  "Asia/Shanghai",
	},
	"ZPSM": {
		ICAO:      "ZPSM",
		IATA:      "SYM",
		Name:      "Simao Airport",
		City:      "Simao",
		State:     "Yunnan",
		Country:   "CN",
		Latitude:  22.7933006287,
		Longitude: 100.9589996338,
		Timezone:  "Asia/Shanghai",
	},
	"ZPZT": {
		ICAO:      "ZPZT",
		IATA:      "ZAT",
		Name:      "Zhaotong Airport",
		City:      "Zhaotong",
		State:     "Yunnan",
		Country:   "CN",
		Latitude:  27.3255996704,
		Longitude: 103.7549972534,
		Timezone:  "Asia/Shanghai",
	},
	"ZSAM": {
		ICAO:      "ZSAM",
		IATA:      "XMN",
		Name:      "Xiamen Gaoqi International Airport",
		City:      "Xiamen",
		State:     "Fujian",
		Country:   "CN",
		Elevation: 59,
		Latitude:  24.5440006256,
		Longitude: 118.1279983521,
		Timezone:  "Asia/Shanghai",
	},
	"ZSAQ": {
		ICAO:      "ZSAQ",
		IATA:      "AQG",
		Name:      "Anqing Airport",
		City:      "Anqing",
		State:     "Anhui",
		Country:   "CN",
		Latitude:  30.5821990967,
		Longitude: 117.0500030518,
		Timezone:  "Asia/Shanghai",
	},
	"ZSBB": {
		ICAO:      "ZSBB",
		IATA:      "BFU",
		Name:      "Bengbu Airport",
		City:      "Bengbu",
		State:     "Anhui",
		Country:   "CN",
		Elevation: 100,
		Latitude:  32.8477333333,
		Longitude: 117.320244444,
		Timezone:  "Asia/Shanghai",
	},
	"ZSCG": {
		ICAO:      "ZSCG",
		IATA:      "CZX",
		Name:      "Changzhou Airport",
		City:      "Changzhou",
		State:     "Jiangsu",
		Country:   "CN",
		Latitude:  31.9197006226,
		Longitude: 119.7789993286,
		Timezone:  "Asia/Shanghai",
	},
	"ZSCN": {
		ICAO:      "ZSCN",
		IATA:      "KHN",
		Name:      "Nanchang Changbei International Airport",
		City:      "Nanchang",
		State:     "Jiangxi",
		Country:   "CN",
		Elevation: 143,
		Latitude:  28.8649997711,
		Longitude: 115.9000015259,
		Timezone:  "Asia/Shanghai",
	},
	"ZSDY": {
		ICAO:      "ZSDY",
		IATA:      "DOY",
		Name:      "Dongying Shengli Airport",
		City:      "Dongying",
		State:     "Shandong",
		Country:   "CN",
		Elevation: 15,
		Latitude:  37.5085983276,
		Longitude: 118.788002014,
		Timezone:  "Asia/Shanghai",
	},
	"ZSFY": {
		ICAO:      "ZSFY",
		IATA:      "FUG",
		Name:      "Fuyang Xiguan Airport",
		City:      "Fuyang",
		State:     "Anhui",
		Country:   "CN",
		Elevation: 104,
		Latitude:  32.882157,
		Longitude: 115.734364,
		Timezone:  "Asia/Shanghai",
	},
	"ZSFZ": {
		ICAO:      "ZSFZ",
		IATA:      "FOC",
		Name:      "Fuzhou Changle International Airport",
		City:      "Fuzhou",
		State:     "Fujian",
		Country:   "CN",
		Elevation: 46,
		Latitude:  25.9351005554,
		Longitude: 119.6630020142,
		Timezone:  "Asia/Shanghai",
	},
	"ZSGS": {
		ICAO:      "ZSGS",
		IATA:      "JGS",
		Name:      "Jinggangshan Airport",
		City:      "Ji'an",
		State:     "Jiangxi",
		Country:   "CN",
		Elevation: 281,
		Latitude:  26.8568992615,
		Longitude: 114.736999512,
		Timezone:  "Asia/Shanghai",
	},
	"ZSGZ": {
		ICAO:      "ZSGZ",
		IATA:      "KOW",
		Name:      "Ganzhou Airport",
		City:      "Ganzhou",
		State:     "Jiangxi",
		Country:   "CN",
		Elevation: 387,
		Latitude:  25.825799942,
		Longitude: 114.912002563,
		Timezone:  "Asia/Shanghai",
	},
	"ZSHC": {
		ICAO:      "ZSHC",
		IATA:      "HGH",
		Name:      "Hangzhou Xiaoshan International Airport",
		City:      "Hangzhou",
		State:     "Zhejiang",
		Country:   "CN",
		Elevation: 23,
		Latitude:  30.2294998169,
		Longitude: 120.4339981079,
		Timezone:  "Asia/Shanghai",
	},
	"ZSJD": {
		ICAO:      "ZSJD",
		IATA:      "JDZ",
		Name:      "Jingdezhen Airport",
		City:      "Jingdezhen",
		State:     "Jiangxi",
		Country:   "CN",
		Elevation: 112,
		Latitude:  29.3386001587,
		Longitude: 117.176002502,
		Timezone:  "Asia/Shanghai",
	},
	"ZSJJ": {
		ICAO:      "ZSJJ",
		IATA:      "JIU",
		Name:      "Jiujiang Lushan Airport",
		City:      "Jiujiang",
		State:     "Jiangxi",
		Country:   "CN",
		Latitude:  29.476944,
		Longitude: 115.801111,
		Timezone:  "Asia/Shanghai",
	},
	"ZSJN": {
		ICAO:      "ZSJN",
		IATA:      "TNA",
		Name:      "Yaoqiang Airport",
		City:      "Jinan",
		State:     "Shandong",
		Country:   "CN",
		Elevation: 76,
		Latitude:  36.8572006226,
		Longitude: 117.216003418,
		Timezone:  "Asia/Shanghai",
	},
	"ZSJU": {
		ICAO:      "ZSJU",
		IATA:      "JUZ",
		Name:      "Quzhou Airport",
		City:      "Quzhou",
		State:     "Zhejiang",
		Country:   "CN",
		Latitude:  28.9657993317,
		Longitude: 118.8990020752,
		Timezone:  "Asia/Shanghai",
	},
	"ZSLD": {
		ICAO:      "ZSLD",
		IATA:      "LCX",
		Name:      "Longyan Guanzhishan Airport",
		City:      "Longyan",
		State:     "Fujian",
		Country:   "CN",
		Elevation: 1225,
		Latitude:  25.6746997833,
		Longitude: 116.747001648,
		Timezone:  "Asia/Shanghai",
	},
	"ZSLG": {
		ICAO:      "ZSLG",
		IATA:      "LYG",
		Name:      "Lianyungang Airport",
		City:      "Lianyungang",
		State:     "Jiangsu",
		Country:   "CN",
		Latitude:  34.571667,
		Longitude: 118.873611,
		Timezone:  "Asia/Shanghai",
	},
	"ZSLQ": {
		ICAO:      "ZSLQ",
		IATA:      "HYN",
		Name:      "Huangyan Luqiao Airport",
		City:      "Huangyan",
		State:     "Zhejiang",
		Country:   "CN",
		Latitude:  28.5622005463,
		Longitude: 121.4290008545,
		Timezone:  "Asia/Shanghai",
	},
	"ZSLY": {
		ICAO:      "ZSLY",
		IATA:      "LYI",
		Name:      "Shubuling Airport",
		City:      "Linyi",
		State:     "Shandong",
		Country:   "CN",
		Latitude:  35.0461006165,
		Longitude: 118.4120025635,
		Timezone:  "Asia/Shanghai",
	},
	"ZSNB": {
		ICAO:      "ZSNB",
		IATA:      "NGB",
		Name:      "Ningbo Lishe International Airport",
		City:      "Ningbo",
		State:     "Zhejiang",
		Country:   "CN",
		Elevation: 13,
		Latitude:  29.8267002106,
		Longitude: 121.4619979858,
		Timezone:  "Asia/Shanghai",
	},
	"ZSNJ": {
		ICAO:      "ZSNJ",
		IATA:      "NKG",
		Name:      "Nanjing Lukou Airport",
		City:      "Nanjing",
		State:     "Jiangsu",
		Country:   "CN",
		Elevation: 49,
		Latitude:  31.7420005798,
		Longitude: 118.8619995117,
		Timezone:  "Asia/Shanghai",
	},
	"ZSNT": {
		ICAO:      "ZSNT",
		IATA:      "NTG",
		Name:      "Nantong Airport",
		City:      "Nantong",
		State:     "Jiangsu",
		Country:   "CN",
		Elevation: 16,
		Latitude:  32.0708007812,
		Longitude: 120.975997925,
		Timezone:  "Asia/Shanghai",
	},
	"ZSOF": {
		ICAO:      "ZSOF",
		IATA:      "HFE",
		Name:      "Hefei Xinqiao International Airport",
		City:      "Hefei",
		State:     "Anhui",
		Country:   "CN",
		Elevation: 207,
		Latitude:  31.98898,
		Longitude: 116.9638,
		Timezone:  "Asia/Shanghai",
	},
	"ZSPD": {
		ICAO:      "ZSPD",
		IATA:      "PVG",
		Name:      "Shanghai Pudong International Airport",
		City:      "Shanghai",
		State:     "Shanghai",
		Country:   "CN",
		Elevation: 13,
		Latitude:  31.1434001923,
		Longitude: 121.8050003052,
		Timezone:  "Asia/Shanghai",
	},
	"ZSQD": {
		ICAO:      "ZSQD",
		IATA:      "TAO",
		Name:      "Liuting Airport",
		City:      "Qingdao",
		State:     "Shandong",
		Country:   "CN",
		Elevation: 33,
		Latitude:  36.2661018372,
		Longitude: 120.374000549,
		Timezone:  "Asia/Shanghai",
	},
	"ZSQZ": {
		ICAO:      "ZSQZ",
		IATA:      "JJN",
		Name:      "Quanzhou Airport",
		City:      "Quanzhou",
		State:     "Fujian",
		Country:   "CN",
		Latitude:  24.7964000702,
		Longitude: 118.5899963379,
		Timezone:  "Asia/Shanghai",
	},
	"ZSRG": {
		ICAO:      "ZSRG",
		IATA:      "RUG",
		Name:      "Rugao Air Base",
		City:      "Rugao",
		State:     "Jiangsu",
		Country:   "CN",
		Latitude:  32.2578849792,
		Longitude: 120.5016555786,
		Timezone:  "Asia/Shanghai",
	},
	"ZSSH": {
		ICAO:      "ZSSH",
		IATA:      "HIA",
		Name:      "Lianshui Airport",
		City:      "Huai'an",
		State:     "Jiangsu",
		Country:   "CN",
		Elevation: 28,
		Latitude:  33.7908333333,
		Longitude: 119.125,
		Timezone:  "Asia/Shanghai",
	},
	"ZSSL": {
		ICAO:      "ZSSL",
		Name:      "Shanghai Longhua Airport",
		City:      "Shanghai",
		State:     "Shanghai",
		Country:   "CN",
		Latitude:  31.1669006348,
		Longitude: 121.45400238,
		Timezone:  "Asia/Shanghai",
	},
	"ZSSM": {
		ICAO:      "ZSSM",
		IATA:      "SQJ",
		Name:      "Sanming Shaxian Airport",
		City:      "Sanming",
		State:     "Fujian",
		Country:   "CN",
		Elevation: 889,
		Latitude:  26.428056,
		Longitude: 117.845,
		Timezone:  "Asia/Shanghai",
	},
	"ZSSS": {
		ICAO:      "ZSSS",
		IATA:      "SHA",
		Name:      "Shanghai Hongqiao International Airport",
		City:      "Shanghai",
		State:     "Shanghai",
		Country:   "CN",
		Elevation: 10,
		Latitude:  31.1979007721,
		Longitude: 121.3359985352,
		Timezone:  "Asia/Shanghai",
	},
	"ZSSZ": {
		ICAO:      "ZSSZ",
		IATA:      "SZV",
		Name:      "Guangfu Airport",
		City:      "Suzhou",
		State:     "Jiangsu",
		Country:   "CN",
		Latitude:  31.2630996704,
		Longitude: 120.4010009766,
		Timezone:  "Asia/Shanghai",
	},
	"ZSTX": {
		ICAO:      "ZSTX",
		IATA:      "TXN",
		Name:      "Tunxi International Airport",
		City:      "Huangshan",
		State:     "Anhui",
		Country:   "CN",
		Latitude:  29.7332992554,
		Longitude: 118.2559967041,
		Timezone:  "Asia/Shanghai",
	},
	"ZSWF": {
		ICAO:      "ZSWF",
		IATA:      "WEF",
		Name:      "Weifang Airport",
		City:      "Weifang",
		State:     "Shandong",
		Country:   "CN",
		Latitude:  36.6467018127,
		Longitude: 119.1190032959,
		Timezone:  "Asia/Shanghai",
	},
	"ZSWH": {
		ICAO:      "ZSWH",
		IATA:      "WEH",
		Name:      "Weihai Airport",
		City:      "Weihai",
		State:     "Shandong",
		Country:   "CN",
		Elevation: 145,
		Latitude:  37.1870994568,
		Longitude: 122.2289962769,
		Timezone:  "Asia/Shanghai",
	},
	"ZSWU": {
		ICAO:      "ZSWU",
		IATA:      "WHU",
		Name:      "Wuhu Air Base",
		City:      "Wuhu",
		State:     "Anhui",
		Country:   "CN",
		Latitude:  31.3906002045,
		Longitude: 118.408996582,
		Timezone:  "Asia/Shanghai",
	},
	"ZSWX": {
		ICAO:      "ZSWX",
		IATA:      "WUX",
		Name:      "Sunan Shuofang International Airport",
		City:      "Wuxi",
		State:     "Jiangsu",
		Country:   "CN",
		Elevation: 24,
		Latitude:  31.4944000244,
		Longitude: 120.429000854,
		Timezone:  "Asia/Shanghai",
	},
	"ZSWY": {
		ICAO:      "ZSWY",
		IATA:      "WUS",
		Name:      "Nanping Wuyishan Airport",
		City:      "Wuyishan",
		State:     "Fujian",
		Country:   "CN",
		Elevation: 614,
		Latitude:  27.7019004822,
		Longitude: 118.0009994507,
		Timezone:  "Asia/Shanghai",
	},
	"ZSWZ": {
		ICAO:      "ZSWZ",
		IATA:      "WNZ",
		Name:      "Wenzhou Yongqiang Airport",
		City:      "Wenzhou",
		State:     "Zhejiang",
		Country:   "CN",
		Latitude:  27.9122009277,
		Longitude: 120.8519973755,
		Timezone:  "Asia/Shanghai",
	},
	"ZSXZ": {
		ICAO:      "ZSXZ",
		IATA:      "XUZ",
		Name:      "Xuzhou Guanyin Airport",
		City:      "Xuzhou",
		State:     "Jiangsu",
		Country:   "CN",
		Latitude:  34.059056,
		Longitude: 117.555278,
		Timezone:  "Asia/Shanghai",
	},
	"ZSYN": {
		ICAO:      "ZSYN",
		IATA:      "YNZ",
		Name:      "Yancheng Airport",
		City:      "Yancheng",
		State:     "Jiangsu",
		Country:   "CN",
		Latitude:  33.425833,
		Longitude: 120.203056,
		Timezone:  "Asia/Shanghai",
	},
	"ZSYT": {
		ICAO:      "ZSYT",
		IATA:      "YNT",
		Name:      "Yantai Laishan Airport",
		City:      "Yantai",
		State:     "Shandong",
		Country:   "CN",
		Elevation: 59,
		Latitude:  37.4016990662,
		Longitude: 121.3720016479,
		Timezone:  "Asia/Shanghai",
	},
	"ZSYW": {
		ICAO:      "ZSYW",
		IATA:      "YIW",
		Name:      "Yiwu Airport",
		City:      "Yiwu",
		State:     "Zhejiang",
		Country:   "CN",
		Elevation: 262,
		Latitude:  29.3446998596,
		Longitude: 120.031997681,
		Timezone:  "Asia/Shanghai",
	},
	"ZSZS": {
		ICAO:      "ZSZS",
		IATA:      "HSN",
		Name:      "Zhoushan Airport",
		City:      "Zhoushan",
		State:     "Zhejiang",
		Country:   "CN",
		Elevation: 3,
		Latitude:  29.9342002869,
		Longitude: 122.361999512,
		Timezone:  "Asia/Shanghai",
	},
	"ZUAL": {
		ICAO:      "ZUAL",
		IATA:      "NGQ",
		Name:      "Ngari Gunsa Airport",
		City:      "Shiquanhe",
		State:     "Tibet",
		Country:   "CN",
		Elevation: 14022,
		Latitude:  32.1,
		Longitude: 80.0530555556,
		Timezone:  "Asia/Shanghai",
	},
	"ZUAS": {
		ICAO:      "ZUAS",
		IATA:      "AVA",
		Name:      "Anshun Huangguoshu Airport",
		City:      "Anshun",
		State:     "Beijing",
		Country:   "CN",
		Elevation: 4812,
		Latitude:  26.2605555556,
		Longitude: 105.873333333,
		Timezone:  "Asia/Shanghai",
	},
	"ZUBD": {
		ICAO:      "ZUBD",
		IATA:      "BPX",
		Name:      "Qamdo Bangda Airport",
		City:      "Bangda",
		State:     "Tibet",
		Country:   "CN",
		Elevation: 14219,
		Latitude:  30.5536003113,
		Longitude: 97.1082992554,
		Timezone:  "Asia/Shanghai",
	},
	"ZUCK": {
		ICAO:      "ZUCK",
		IATA:      "CKG",
		Name:      "Chongqing Jiangbei International Airport",
		City:      "Chongqing",
		State:     "Chongqing",
		Country:   "CN",
		Elevation: 1365,
		Latitude:  29.7192001343,
		Longitude: 106.641998291,
		Timezone:  "Asia/Shanghai",
	},
	"ZUDX": {
		ICAO:      "ZUDX",
		IATA:      "DAX",
		Name:      "Dachuan Airport",
		City:      "Dazhou",
		State:     "Sichuan",
		Country:   "CN",
		Latitude:  31.1302,
		Longitude: 107.4295,
		Timezone:  "Asia/Shanghai",
	},
	"ZUGU": {
		ICAO:      "ZUGU",
		IATA:      "GYS",
		Name:      "Guangyuan Airport",
		City:      "Guangyuan",
		State:     "Sichuan",
		Country:   "CN",
		Latitude:  32.3911018372,
		Longitude: 105.702003479,
		Timezone:  "Asia/Shanghai",
	},
	"ZUGY": {
		ICAO:      "ZUGY",
		IATA:      "KWE",
		Name:      "Longdongbao Airport",
		City:      "Guiyang",
		State:     "Guizhou",
		Country:   "CN",
		Elevation: 3736,
		Latitude:  26.5384998322,
		Longitude: 106.8010025024,
		Timezone:  "Asia/Shanghai",
	},
	"ZUJZ": {
		ICAO:      "ZUJZ",
		IATA:      "JZH",
		Name:      "Jiuzhai Huanglong Airport",
		City:      "Jiuzhaigou",
		State:     "Sichuan",
		Country:   "CN",
		Elevation: 11327,
		Latitude:  32.8533333333,
		Longitude: 103.682222222,
		Timezone:  "Asia/Shanghai",
	},
	"ZULP": {
		ICAO:      "ZULP",
		IATA:      "LIA",
		Name:      "Liangping Airport",
		City:      "Liangping",
		State:     "Sichuan",
		Country:   "CN",
		Latitude:  30.6793994904,
		Longitude: 107.7860031128,
		Timezone:  "Asia/Shanghai",
	},
	"ZULS": {
		ICAO:      "ZULS",
		IATA:      "LXA",
		Name:      "Lhasa Gonggar Airport",
		City:      "Lhasa",
		State:     "Tibet",
		Country:   "CN",
		Elevation: 13136,
		Latitude:  29.2978000641,
		Longitude: 90.9119033813,
		Timezone:  "Asia/Shanghai",
	},
	"ZULZ": {
		ICAO:      "ZULZ",
		IATA:      "LZO",
		Name:      "Luzhou Airport",
		City:      "Luzhou",
		State:     "Sichuan",
		Country:   "CN",
		Latitude:  28.8521995544,
		Longitude: 105.3929977417,
		Timezone:  "Asia/Shanghai",
	},
	"ZUMY": {
		ICAO:      "ZUMY",
		IATA:      "MIG",
		Name:      "Mianyang Airport",
		City:      "Mianyang",
		State:     "Sichuan",
		Country:   "CN",
		Latitude:  31.4281005859,
		Longitude: 104.7409973145,
		Timezone:  "Asia/Shanghai",
	},
	"ZUNC": {
		ICAO:      "ZUNC",
		IATA:      "NAO",
		Name:      "Nanchong Airport",
		City:      "Nanchong",
		State:     "Sichuan",
		Country:   "CN",
		Latitude:  30.79545,
		Longitude: 106.1626,
		Timezone:  "Asia/Shanghai",
	},
	"ZUNZ": {
		ICAO:      "ZUNZ",
		IATA:      "LZY",
		Name:      "Nyingchi Airport",
		City:      "Nyingchi",
		State:     "Tibet",
		Country:   "CN",
		Elevation: 9675,
		Latitude:  29.3033008575,
		Longitude: 94.3352966309,
		Timezone:  "Asia/Shanghai",
	},
	"ZUQJ": {
		ICAO:      "ZUQJ",
		IATA:      "JIQ",
		Name:      "Qianjiang Wulingshan Airport",
		City:      "Chongqing",
		State:     "Chongqing",
		Country:   "CN",
		Elevation: 1365,
		Latitude:  29.514559,
		Longitude: 108.83371499999998,
		Timezone:  "Asia/Shanghai",
	},
	"ZUTC": {
		ICAO:      "ZUTC",
		IATA:      "TCZ",
		Name:      "Tengchong Tuofeng Airport",
		City:      "Tengchong",
		State:     "Yunnan",
		Country:   "CN",
		Elevation: 6250,
		Latitude:  24.9380555556,
		Longitude: 98.4858333333,
		Timezone:  "Asia/Shanghai",
	},
	"ZUTF": {
		ICAO:      "ZUTF",
		IATA:      "TFU",
		Name:      "Chengdu Tianfu International Airport",
		City:      "Chengdu",
		State:     "Sichuan",
		Country:   "CN",
		Elevation: 1444,
		Latitude:  30.319,
		Longitude: 104.445,
		Timezone:  "Asia/Shanghai",
	},
	"ZUTR": {
		ICAO:      "ZUTR",
		IATA:      "TEN",
		Name:      "Tongren Fenghuang Airport",
		State:     "Hunan",
		Country:   "CN",
		Latitude:  27.883333,
		Longitude: 109.308889,
		Timezone:  "Asia/Shanghai",
	},
	"ZUUU": {
		ICAO:      "ZUUU",
		IATA:      "CTU",
		Name:      "Chengdu Shuangliu International Airport",
		City:      "Chengdu",
		State:     "Sichuan",
		Country:   "CN",
		Elevation: 1625,
		Latitude:  30.5785007477,
		Longitude: 103.9469985962,
		Timezone:  "Asia/Shanghai",
	},
	"ZUWX": {
		ICAO:      "ZUWX",
		IATA:      "WXN",
		Name:      "Wanxian Airport",
		City:      "Wanxian",
		State:     "Chongqing",
		Country:   "CN",
		Latitude:  30.8017,
		Longitude: 108.433,
		Timezone:  "Asia/Shanghai",
	},
	"ZUXC": {
		ICAO:      "ZUXC",
		IATA:      "XIC",
		Name:      "Xichang Qingshan Airport",
		City:      "Xichang",
		State:     "Sichuan",
		Country:   "CN",
		Elevation: 5112,
		Latitude:  27.9890995026,
		Longitude: 102.1839981079,
		Timezone:  "Asia/Shanghai",
	},
	"ZUYB": {
		ICAO:      "ZUYB",
		IATA:      "YBP",
		Name:      "Yibin Caiba Airport",
		City:      "Yibin",
		State:     "Sichuan",
		Country:   "CN",
		Elevation: 924,
		Latitude:  28.8005555556,
		Longitude: 104.545,
		Timezone:  "Asia/Shanghai",
	},
	"ZUYI": {
		ICAO:      "ZUYI",
		IATA:      "ACX",
		Name:      "Xingyi Airport",
		City:      "Xingyi",
		State:     "Guizhou",
		Country:   "CN",
		Elevation: 4150,
		Latitude:  25.0863888889,
		Longitude: 104.959444444,
		Timezone:  "Asia/Shanghai",
	},
	"ZUZY": {
		ICAO:      "ZUZY",
		IATA:      "ZYI",
		Name:      "Zunyi Xinzhou Airport",
		City:      "Zunyi",
		State:     "Guizhou",
		Country:   "CN",
		Elevation: 2920,
		Latitude:  27.5895,
		Longitude: 107.0007,
		Timezone:  "Asia/Shanghai",
	},
	"ZWAK": {
		ICAO:      "ZWAK",
		IATA:      "AKU",
		Name:      "Aksu Airport",
		City:      "Aksu",
		State:     "Xinjiang",
		Country:   "CN",
		Latitude:  41.2625007629,
		Longitude: 80.2917022705,
		Timezone:  "Asia/Shanghai",
	},
	"ZWAT": {
		ICAO:      "ZWAT",
		IATA:      "AAT",
		Name:      "Altay Air Base",
		City:      "Altay",
		State:     "Xinjiang",
		Country:   "CN",
		Elevation: 2460,
		Latitude:  47.7498855591,
		Longitude: 88.0858078003,
		Timezone:  "Asia/Shanghai",
	},
	"ZWBL": {
		ICAO:      "ZWBL",
		IATA:      "BPL",
		Name:      "Alashankou Bole (Bortala) airport",
		City:      "Bole",
		State:     "Xinjiang",
		Country:   "CN",
		Elevation: 1253,
		Latitude:  44.895,
		Longitude: 82.3,
		Timezone:  "Asia/Shanghai",
	},
	"ZWCM": {
		ICAO:      "ZWCM",
		IATA:      "IQM",
		Name:      "Qiemo Airport",
		City:      "Qiemo",
		State:     "Xinjiang",
		Country:   "CN",
		Elevation: 4108,
		Latitude:  38.1493988037,
		Longitude: 85.5327987671,
		Timezone:  "Asia/Shanghai",
	},
	"ZWHM": {
		ICAO:      "ZWHM",
		IATA:      "HMI",
		Name:      "Hami Airport",
		City:      "Hami",
		State:     "Xinjiang",
		Country:   "CN",
		Elevation: 2703,
		Latitude:  42.8414001465,
		Longitude: 93.6691970825,
		Timezone:  "Asia/Shanghai",
	},
	"ZWKC": {
		ICAO:      "ZWKC",
		IATA:      "KCA",
		Name:      "Kuqa Airport",
		City:      "Kuqa",
		State:     "Xinjiang",
		Country:   "CN",
		Elevation: 3524,
		Latitude:  41.7181015015,
		Longitude: 82.9869003296,
		Timezone:  "Asia/Shanghai",
	},
	"ZWKL": {
		ICAO:      "ZWKL",
		IATA:      "KRL",
		Name:      "Korla Airport",
		City:      "Korla",
		State:     "Xinjiang",
		Country:   "CN",
		Latitude:  41.6977996826,
		Longitude: 86.1288986206,
		Timezone:  "Asia/Shanghai",
	},
	"ZWKM": {
		ICAO:      "ZWKM",
		IATA:      "KRY",
		Name:      "Karamay Airport",
		City:      "Karamay",
		State:     "Xinjiang",
		Country:   "CN",
		Latitude:  45.46655,
		Longitude: 84.9527,
		Timezone:  "Asia/Shanghai",
	},
	"ZWSH": {
		ICAO:      "ZWSH",
		IATA:      "KHG",
		Name:      "Kashgar Airport",
		City:      "Kashgar",
		State:     "Xinjiang",
		Country:   "CN",
		Elevation: 4529,
		Latitude:  39.5429000854,
		Longitude: 76.0199966431,
		Timezone:  "Asia/Shanghai",
	},
	"ZWSS": {
		ICAO:      "ZWSS",
		IATA:      "SXJ",
		Name:      "Shanshan Airport",
		City:      "Shanshan",
		State:     "Xinjiang",
		Country:   "CN",
		Latitude:  42.9117012024,
		Longitude: 90.2474975586,
		Timezone:  "Asia/Shanghai",
	},
	"ZWTC": {
		ICAO:      "ZWTC",
		IATA:      "TCG",
		Name:      "Tacheng Airport",
		City:      "Tacheng",
		State:     "Xinjiang",
		Country:   "CN",
		Latitude:  46.6725006104,
		Longitude: 83.3407974243,
		Timezone:  "Asia/Shanghai",
	},
	"ZWTN": {
		ICAO:      "ZWTN",
		IATA:      "HTN",
		Name:      "Hotan Airport",
		City:      "Hotan",
		State:     "Xinjiang",
		Country:   "CN",
		Elevation: 4672,
		Latitude:  37.0385017395,
		Longitude: 79.8648986816,
		Timezone:  "Asia/Shanghai",
	},
	"ZWWW": {
		ICAO:      "ZWWW",
		IATA:      "URC",
		Name:      "Urumqi Diwopu International Airport",
		City:      "Urumqi",
		State:     "Xinjiang",
		Country:   "CN",
		Elevation: 2125,
		Latitude:  43.9071006775,
		Longitude: 87.4741973877,
		Timezone:  "Asia/Shanghai",
	},
	"ZWYN": {
		ICAO:      "ZWYN",
		IATA:      "YIN",
		Name:      "Yining Airport",
		City:      "Yining",
		State:     "Xinjiang",
		Country:   "CN",
		Latitude:  43.9557991028,
		Longitude: 81.3302993774,
		Timezone:  "Asia/Shanghai",
	},
	"ZYAS": {
		ICAO:      "ZYAS",
		IATA:      "AOG",
		Name:      "Anshan Air Base",
		City:      "Anshan",
		State:     "Liaoning",
		Country:   "CN",
		Latitude:  41.1053009033,
		Longitude: 122.8539962769,
		Timezone:  "Asia/Shanghai",
	},
	"ZYCC": {
		ICAO:      "ZYCC",
		IATA:      "CGQ",
		Name:      "Longjia Airport",
		City:      "Changchun",
		State:     "Jilin",
		Country:   "CN",
		Elevation: 706,
		Latitude:  43.9962005615,
		Longitude: 125.684997559,
		Timezone:  "Asia/Shanghai",
	},
	"ZYCH": {
		ICAO:      "ZYCH",
		IATA:      "CNI",
		Name:      "Changhai Airport",
		City:      "Changhai",
		State:     "Liaoning",
		Country:   "CN",
		Elevation: 80,
		Latitude:  39.2666666667,
		Longitude: 122.666944444,
		Timezone:  "Asia/Shanghai",
	},
	"ZYCY": {
		ICAO:      "ZYCY",
		IATA:      "CHG",
		Name:      "Chaoyang Airport",
		City:      "Chaoyang",
		State:     "Liaoning",
		Country:   "CN",
		Latitude:  41.5381011963,
		Longitude: 120.4349975586,
		Timezone:  "Asia/Shanghai",
	},
	"ZYDD": {
		ICAO:      "ZYDD",
		IATA:      "DDG",
		Name:      "Dandong Airport",
		City:      "Dandong",
		State:     "Liaoning",
		Country:   "CN",
		Latitude:  40.0247001648,
		Longitude: 124.286003113,
		Timezone:  "Asia/Shanghai",
	},
	"ZYDQ": {
		ICAO:      "ZYDQ",
		IATA:      "DQA",
		Name:      "Saertu Airport",
		City:      "Daqing Shi",
		State:     "Heilongjiang",
		Country:   "CN",
		Elevation: 496,
		Latitude:  46.7463888889,
		Longitude: 125.140555556,
		Timezone:  "Asia/Shanghai",
	},
	"ZYHB": {
		ICAO:      "ZYHB",
		IATA:      "HRB",
		Name:      "Taiping Airport",
		City:      "Harbin",
		State:     "Heilongjiang",
		Country:   "CN",
		Elevation: 457,
		Latitude:  45.6234016418,
		Longitude: 126.25,
		Timezone:  "Asia/Shanghai",
	},
	"ZYHE": {
		ICAO:      "ZYHE",
		IATA:      "HEK",
		Name:      "Heihe Airport",
		City:      "Heihe",
		State:     "Heilongjiang",
		Country:   "CN",
		Elevation: 8530,
		Latitude:  50.1716209371,
		Longitude: 127.308883667,
		Timezone:  "Asia/Shanghai",
	},
	"ZYJL": {
		ICAO:      "ZYJL",
		IATA:      "JIL",
		Name:      "Jilin Airport",
		City:      "Jilin",
		State:     "Jilin",
		Country:   "CN",
		Latitude:  44.0022010803,
		Longitude: 126.3960037231,
		Timezone:  "Asia/Shanghai",
	},
	"ZYJM": {
		ICAO:      "ZYJM",
		IATA:      "JMU",
		Name:      "Jiamusi Airport",
		City:      "Jiamusi",
		State:     "Heilongjiang",
		Country:   "CN",
		Elevation: 262,
		Latitude:  46.8433990479,
		Longitude: 130.464996338,
		Timezone:  "Asia/Shanghai",
	},
	"ZYJZ": {
		ICAO:      "ZYJZ",
		IATA:      "JNZ",
		Name:      "Jinzhou Airport",
		City:      "Jinzhou",
		State:     "Liaoning",
		Country:   "CN",
		Latitude:  41.101398468,
		Longitude: 121.06199646,
		Timezone:  "Asia/Shanghai",
	},
	"ZYLD": {
		ICAO:      "ZYLD",
		IATA:      "LDS",
		Name:      "Lindu Airport",
		City:      "Yichun",
		State:     "Heilongjiang",
		Country:   "CN",
		Elevation: 791,
		Latitude:  47.7520555556,
		Longitude: 129.019125,
		Timezone:  "Asia/Shanghai",
	},
	"ZYLS": {
		ICAO:      "ZYLS",
		IATA:      "YUS",
		Name:      "Yushu Batang Airport",
		City:      "Yushu",
		State:     "Qinghai",
		Country:   "CN",
		Elevation: 12816,
		Latitude:  32.8363888889,
		Longitude: 97.0363888889,
		Timezone:  "Asia/Shanghai",
	},
	"ZYMD": {
		ICAO:      "ZYMD",
		IATA:      "MDG",
		Name:      "Mudanjiang Hailang International Airport",
		City:      "Mudanjiang",
		State:     "Heilongjiang",
		Country:   "CN",
		Elevation: 883,
		Latitude:  44.5241012573,
		Longitude: 129.569000244,
		Timezone:  "Asia/Shanghai",
	},
	"ZYMH": {
		ICAO:      "ZYMH",
		IATA:      "OHE",
		Name:      "Gu-Lian Airport",
		City:      "Mohe",
		State:     "Heilongjiang",
		Country:   "CN",
		Elevation: 1836,
		Latitude:  52.9127777778,
		Longitude: 122.43,
		Timezone:  "Asia/Shanghai",
	},
	"ZYQQ": {
		ICAO:      "ZYQQ",
		IATA:      "NDG",
		Name:      "Qiqihar Sanjiazi Airport",
		City:      "Qiqihar",
		State:     "Heilongjiang",
		Country:   "CN",
		Elevation: 477,
		Latitude:  47.2396011353,
		Longitude: 123.9179992676,
		Timezone:  "Asia/Shanghai",
	},
	"ZYTH": {
		ICAO:      "ZYTH",
		Name:      "Tahe Airport",
		City:      "Tahe",
		State:     "Heilongjiang",
		Country:   "CN",
		Elevation: 1240,
		Latitude:  52.2244444444,
		Longitude: 124.720222222,
		Timezone:  "Asia/Shanghai",
	},
	"ZYTL": {
		ICAO:      "ZYTL",
		IATA:      "DLC",
		Name:      "Zhoushuizi Airport",
		City:      "Dalian",
		State:     "Liaoning",
		Country:   "CN",
		Elevation: 107,
		Latitude:  38.9656982422,
		Longitude: 121.5390014648,
		Timezone:  "Asia/Shanghai",
	},
	"ZYTN": {
		ICAO:      "ZYTN",
		IATA:      "TNH",
		Name:      "Tonghua Sanyuanpu Airport",
		City:      "Tonghua",
		State:     "Jilin",
		Country:   "CN",
		Elevation: 1200,
		Latitude:  42.2538888889,
		Longitude: 125.703333333,
		Timezone:  "Asia/Shanghai",
	},
	"ZYTX": {
		ICAO:      "ZYTX",
		IATA:      "SHE",
		Name:      "Taoxian Airport",
		City:      "Shenyang",
		State:     "Liaoning",
		Country:   "CN",
		Elevation: 198,
		Latitude:  41.6398010254,
		Longitude: 123.483001709,
		Timezone:  "Asia/Shanghai",
	},
	"ZYXC": {
		ICAO:      "ZYXC",
		IATA:      "XEN",
		Name:      "Xingcheng Air Base",
		State:     "Liaoning",
		Country:   "CN",
		Elevation: 30,
		Latitude:  40.5802993774,
		Longitude: 120.697998047,
		Timezone:  "Asia/Shanghai",
	},
	"ZYYJ": {
		ICAO:      "ZYYJ",
		IATA:      "YNJ",
		Name:      "Yanji Chaoyangchuan Airport",
		City:      "Yanji",
		State:     "Jilin",
		Country:   "CN",
		Elevation: 624,
		Latitude:  42.8828010559,
		Longitude: 129.451004028,
		Timezone:  "Asia/Shanghai",
	},
	"ZYYY": {
		ICAO:      "ZYYY",
		Name:      "Shenyang Dongta Airport",
		City:      "Shenyang",
		State:     "Liaoning",
		Country:   "CN",
		Latitude:  41.7844009399,
		Longitude: 123.4960021973,
		Timezone:  "Asia/Shanghai",
	},
}
